#include "td_api.h"

#include "td/tl/tl_jni_object.h"

#include "td/utils/common.h"
#include "td/utils/format.h"
#include "td/utils/logging.h"
#include "td/utils/SliceBuilder.h"
#include "td/utils/tl_parsers.h"
#include "td/utils/tl_storers.h"
#include "td/utils/TlStorerToString.h"

namespace td {
namespace td_api {

std::string to_string(const BaseObject &value) {
  TlStorerToString storer;
  value.store(storer, "");
  return storer.move_as_string();
}

const char *&get_package_name_ref() {
  static const char *package_name = "Package name must be initialized first";
  return package_name;
}

const char *get_git_commit_hash() {
  return "GITDIR-NOTFOUND";
}

jclass Object::Class;

object_ptr<Object> Object::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case accentColor::ID:
      return accentColor::fetch(env, p);
    case acceptedGiftTypes::ID:
      return acceptedGiftTypes::fetch(env, p);
    case accountInfo::ID:
      return accountInfo::fetch(env, p);
    case accountTtl::ID:
      return accountTtl::fetch(env, p);
    case addedReaction::ID:
      return addedReaction::fetch(env, p);
    case addedReactions::ID:
      return addedReactions::fetch(env, p);
    case address::ID:
      return address::fetch(env, p);
    case affiliateInfo::ID:
      return affiliateInfo::fetch(env, p);
    case affiliateProgramInfo::ID:
      return affiliateProgramInfo::fetch(env, p);
    case affiliateProgramParameters::ID:
      return affiliateProgramParameters::fetch(env, p);
    case affiliateProgramSortOrderProfitability::ID:
      return affiliateProgramSortOrderProfitability::fetch(env, p);
    case affiliateProgramSortOrderCreationDate::ID:
      return affiliateProgramSortOrderCreationDate::fetch(env, p);
    case affiliateProgramSortOrderRevenue::ID:
      return affiliateProgramSortOrderRevenue::fetch(env, p);
    case affiliateTypeCurrentUser::ID:
      return affiliateTypeCurrentUser::fetch(env, p);
    case affiliateTypeBot::ID:
      return affiliateTypeBot::fetch(env, p);
    case affiliateTypeChannel::ID:
      return affiliateTypeChannel::fetch(env, p);
    case alternativeVideo::ID:
      return alternativeVideo::fetch(env, p);
    case animatedChatPhoto::ID:
      return animatedChatPhoto::fetch(env, p);
    case animatedEmoji::ID:
      return animatedEmoji::fetch(env, p);
    case animation::ID:
      return animation::fetch(env, p);
    case animations::ID:
      return animations::fetch(env, p);
    case archiveChatListSettings::ID:
      return archiveChatListSettings::fetch(env, p);
    case attachmentMenuBot::ID:
      return attachmentMenuBot::fetch(env, p);
    case attachmentMenuBotColor::ID:
      return attachmentMenuBotColor::fetch(env, p);
    case audio::ID:
      return audio::fetch(env, p);
    case authenticationCodeInfo::ID:
      return authenticationCodeInfo::fetch(env, p);
    case authenticationCodeTypeTelegramMessage::ID:
      return authenticationCodeTypeTelegramMessage::fetch(env, p);
    case authenticationCodeTypeSms::ID:
      return authenticationCodeTypeSms::fetch(env, p);
    case authenticationCodeTypeSmsWord::ID:
      return authenticationCodeTypeSmsWord::fetch(env, p);
    case authenticationCodeTypeSmsPhrase::ID:
      return authenticationCodeTypeSmsPhrase::fetch(env, p);
    case authenticationCodeTypeCall::ID:
      return authenticationCodeTypeCall::fetch(env, p);
    case authenticationCodeTypeFlashCall::ID:
      return authenticationCodeTypeFlashCall::fetch(env, p);
    case authenticationCodeTypeMissedCall::ID:
      return authenticationCodeTypeMissedCall::fetch(env, p);
    case authenticationCodeTypeFragment::ID:
      return authenticationCodeTypeFragment::fetch(env, p);
    case authenticationCodeTypeFirebaseAndroid::ID:
      return authenticationCodeTypeFirebaseAndroid::fetch(env, p);
    case authenticationCodeTypeFirebaseIos::ID:
      return authenticationCodeTypeFirebaseIos::fetch(env, p);
    case authorizationStateWaitTdlibParameters::ID:
      return authorizationStateWaitTdlibParameters::fetch(env, p);
    case authorizationStateWaitPhoneNumber::ID:
      return authorizationStateWaitPhoneNumber::fetch(env, p);
    case authorizationStateWaitPremiumPurchase::ID:
      return authorizationStateWaitPremiumPurchase::fetch(env, p);
    case authorizationStateWaitEmailAddress::ID:
      return authorizationStateWaitEmailAddress::fetch(env, p);
    case authorizationStateWaitEmailCode::ID:
      return authorizationStateWaitEmailCode::fetch(env, p);
    case authorizationStateWaitCode::ID:
      return authorizationStateWaitCode::fetch(env, p);
    case authorizationStateWaitOtherDeviceConfirmation::ID:
      return authorizationStateWaitOtherDeviceConfirmation::fetch(env, p);
    case authorizationStateWaitRegistration::ID:
      return authorizationStateWaitRegistration::fetch(env, p);
    case authorizationStateWaitPassword::ID:
      return authorizationStateWaitPassword::fetch(env, p);
    case authorizationStateReady::ID:
      return authorizationStateReady::fetch(env, p);
    case authorizationStateLoggingOut::ID:
      return authorizationStateLoggingOut::fetch(env, p);
    case authorizationStateClosing::ID:
      return authorizationStateClosing::fetch(env, p);
    case authorizationStateClosed::ID:
      return authorizationStateClosed::fetch(env, p);
    case autoDownloadSettings::ID:
      return autoDownloadSettings::fetch(env, p);
    case autoDownloadSettingsPresets::ID:
      return autoDownloadSettingsPresets::fetch(env, p);
    case autosaveSettings::ID:
      return autosaveSettings::fetch(env, p);
    case autosaveSettingsException::ID:
      return autosaveSettingsException::fetch(env, p);
    case autosaveSettingsScopePrivateChats::ID:
      return autosaveSettingsScopePrivateChats::fetch(env, p);
    case autosaveSettingsScopeGroupChats::ID:
      return autosaveSettingsScopeGroupChats::fetch(env, p);
    case autosaveSettingsScopeChannelChats::ID:
      return autosaveSettingsScopeChannelChats::fetch(env, p);
    case autosaveSettingsScopeChat::ID:
      return autosaveSettingsScopeChat::fetch(env, p);
    case availableGift::ID:
      return availableGift::fetch(env, p);
    case availableGifts::ID:
      return availableGifts::fetch(env, p);
    case availableReaction::ID:
      return availableReaction::fetch(env, p);
    case availableReactions::ID:
      return availableReactions::fetch(env, p);
    case background::ID:
      return background::fetch(env, p);
    case backgroundFillSolid::ID:
      return backgroundFillSolid::fetch(env, p);
    case backgroundFillGradient::ID:
      return backgroundFillGradient::fetch(env, p);
    case backgroundFillFreeformGradient::ID:
      return backgroundFillFreeformGradient::fetch(env, p);
    case backgroundTypeWallpaper::ID:
      return backgroundTypeWallpaper::fetch(env, p);
    case backgroundTypePattern::ID:
      return backgroundTypePattern::fetch(env, p);
    case backgroundTypeFill::ID:
      return backgroundTypeFill::fetch(env, p);
    case backgroundTypeChatTheme::ID:
      return backgroundTypeChatTheme::fetch(env, p);
    case backgrounds::ID:
      return backgrounds::fetch(env, p);
    case bankCardActionOpenUrl::ID:
      return bankCardActionOpenUrl::fetch(env, p);
    case bankCardInfo::ID:
      return bankCardInfo::fetch(env, p);
    case basicGroup::ID:
      return basicGroup::fetch(env, p);
    case basicGroupFullInfo::ID:
      return basicGroupFullInfo::fetch(env, p);
    case birthdate::ID:
      return birthdate::fetch(env, p);
    case blockListMain::ID:
      return blockListMain::fetch(env, p);
    case blockListStories::ID:
      return blockListStories::fetch(env, p);
    case botCommand::ID:
      return botCommand::fetch(env, p);
    case botCommandScopeDefault::ID:
      return botCommandScopeDefault::fetch(env, p);
    case botCommandScopeAllPrivateChats::ID:
      return botCommandScopeAllPrivateChats::fetch(env, p);
    case botCommandScopeAllGroupChats::ID:
      return botCommandScopeAllGroupChats::fetch(env, p);
    case botCommandScopeAllChatAdministrators::ID:
      return botCommandScopeAllChatAdministrators::fetch(env, p);
    case botCommandScopeChat::ID:
      return botCommandScopeChat::fetch(env, p);
    case botCommandScopeChatAdministrators::ID:
      return botCommandScopeChatAdministrators::fetch(env, p);
    case botCommandScopeChatMember::ID:
      return botCommandScopeChatMember::fetch(env, p);
    case botCommands::ID:
      return botCommands::fetch(env, p);
    case botInfo::ID:
      return botInfo::fetch(env, p);
    case botMediaPreview::ID:
      return botMediaPreview::fetch(env, p);
    case botMediaPreviewInfo::ID:
      return botMediaPreviewInfo::fetch(env, p);
    case botMediaPreviews::ID:
      return botMediaPreviews::fetch(env, p);
    case botMenuButton::ID:
      return botMenuButton::fetch(env, p);
    case botVerification::ID:
      return botVerification::fetch(env, p);
    case botVerificationParameters::ID:
      return botVerificationParameters::fetch(env, p);
    case botWriteAccessAllowReasonConnectedWebsite::ID:
      return botWriteAccessAllowReasonConnectedWebsite::fetch(env, p);
    case botWriteAccessAllowReasonAddedToAttachmentMenu::ID:
      return botWriteAccessAllowReasonAddedToAttachmentMenu::fetch(env, p);
    case botWriteAccessAllowReasonLaunchedWebApp::ID:
      return botWriteAccessAllowReasonLaunchedWebApp::fetch(env, p);
    case botWriteAccessAllowReasonAcceptedRequest::ID:
      return botWriteAccessAllowReasonAcceptedRequest::fetch(env, p);
    case businessAwayMessageScheduleAlways::ID:
      return businessAwayMessageScheduleAlways::fetch(env, p);
    case businessAwayMessageScheduleOutsideOfOpeningHours::ID:
      return businessAwayMessageScheduleOutsideOfOpeningHours::fetch(env, p);
    case businessAwayMessageScheduleCustom::ID:
      return businessAwayMessageScheduleCustom::fetch(env, p);
    case businessAwayMessageSettings::ID:
      return businessAwayMessageSettings::fetch(env, p);
    case businessBotManageBar::ID:
      return businessBotManageBar::fetch(env, p);
    case businessBotRights::ID:
      return businessBotRights::fetch(env, p);
    case businessChatLink::ID:
      return businessChatLink::fetch(env, p);
    case businessChatLinkInfo::ID:
      return businessChatLinkInfo::fetch(env, p);
    case businessChatLinks::ID:
      return businessChatLinks::fetch(env, p);
    case businessConnectedBot::ID:
      return businessConnectedBot::fetch(env, p);
    case businessConnection::ID:
      return businessConnection::fetch(env, p);
    case businessFeatureLocation::ID:
      return businessFeatureLocation::fetch(env, p);
    case businessFeatureOpeningHours::ID:
      return businessFeatureOpeningHours::fetch(env, p);
    case businessFeatureQuickReplies::ID:
      return businessFeatureQuickReplies::fetch(env, p);
    case businessFeatureGreetingMessage::ID:
      return businessFeatureGreetingMessage::fetch(env, p);
    case businessFeatureAwayMessage::ID:
      return businessFeatureAwayMessage::fetch(env, p);
    case businessFeatureAccountLinks::ID:
      return businessFeatureAccountLinks::fetch(env, p);
    case businessFeatureStartPage::ID:
      return businessFeatureStartPage::fetch(env, p);
    case businessFeatureBots::ID:
      return businessFeatureBots::fetch(env, p);
    case businessFeatureEmojiStatus::ID:
      return businessFeatureEmojiStatus::fetch(env, p);
    case businessFeatureChatFolderTags::ID:
      return businessFeatureChatFolderTags::fetch(env, p);
    case businessFeatureUpgradedStories::ID:
      return businessFeatureUpgradedStories::fetch(env, p);
    case businessFeaturePromotionAnimation::ID:
      return businessFeaturePromotionAnimation::fetch(env, p);
    case businessFeatures::ID:
      return businessFeatures::fetch(env, p);
    case businessGreetingMessageSettings::ID:
      return businessGreetingMessageSettings::fetch(env, p);
    case businessInfo::ID:
      return businessInfo::fetch(env, p);
    case businessLocation::ID:
      return businessLocation::fetch(env, p);
    case businessMessage::ID:
      return businessMessage::fetch(env, p);
    case businessMessages::ID:
      return businessMessages::fetch(env, p);
    case businessOpeningHours::ID:
      return businessOpeningHours::fetch(env, p);
    case businessOpeningHoursInterval::ID:
      return businessOpeningHoursInterval::fetch(env, p);
    case businessRecipients::ID:
      return businessRecipients::fetch(env, p);
    case businessStartPage::ID:
      return businessStartPage::fetch(env, p);
    case call::ID:
      return call::fetch(env, p);
    case callDiscardReasonEmpty::ID:
      return callDiscardReasonEmpty::fetch(env, p);
    case callDiscardReasonMissed::ID:
      return callDiscardReasonMissed::fetch(env, p);
    case callDiscardReasonDeclined::ID:
      return callDiscardReasonDeclined::fetch(env, p);
    case callDiscardReasonDisconnected::ID:
      return callDiscardReasonDisconnected::fetch(env, p);
    case callDiscardReasonHungUp::ID:
      return callDiscardReasonHungUp::fetch(env, p);
    case callDiscardReasonUpgradeToGroupCall::ID:
      return callDiscardReasonUpgradeToGroupCall::fetch(env, p);
    case callId::ID:
      return callId::fetch(env, p);
    case callProblemEcho::ID:
      return callProblemEcho::fetch(env, p);
    case callProblemNoise::ID:
      return callProblemNoise::fetch(env, p);
    case callProblemInterruptions::ID:
      return callProblemInterruptions::fetch(env, p);
    case callProblemDistortedSpeech::ID:
      return callProblemDistortedSpeech::fetch(env, p);
    case callProblemSilentLocal::ID:
      return callProblemSilentLocal::fetch(env, p);
    case callProblemSilentRemote::ID:
      return callProblemSilentRemote::fetch(env, p);
    case callProblemDropped::ID:
      return callProblemDropped::fetch(env, p);
    case callProblemDistortedVideo::ID:
      return callProblemDistortedVideo::fetch(env, p);
    case callProblemPixelatedVideo::ID:
      return callProblemPixelatedVideo::fetch(env, p);
    case callProtocol::ID:
      return callProtocol::fetch(env, p);
    case callServer::ID:
      return callServer::fetch(env, p);
    case callServerTypeTelegramReflector::ID:
      return callServerTypeTelegramReflector::fetch(env, p);
    case callServerTypeWebrtc::ID:
      return callServerTypeWebrtc::fetch(env, p);
    case callStatePending::ID:
      return callStatePending::fetch(env, p);
    case callStateExchangingKeys::ID:
      return callStateExchangingKeys::fetch(env, p);
    case callStateReady::ID:
      return callStateReady::fetch(env, p);
    case callStateHangingUp::ID:
      return callStateHangingUp::fetch(env, p);
    case callStateDiscarded::ID:
      return callStateDiscarded::fetch(env, p);
    case callStateError::ID:
      return callStateError::fetch(env, p);
    case callbackQueryAnswer::ID:
      return callbackQueryAnswer::fetch(env, p);
    case callbackQueryPayloadData::ID:
      return callbackQueryPayloadData::fetch(env, p);
    case callbackQueryPayloadDataWithPassword::ID:
      return callbackQueryPayloadDataWithPassword::fetch(env, p);
    case callbackQueryPayloadGame::ID:
      return callbackQueryPayloadGame::fetch(env, p);
    case canPostStoryResultOk::ID:
      return canPostStoryResultOk::fetch(env, p);
    case canPostStoryResultPremiumNeeded::ID:
      return canPostStoryResultPremiumNeeded::fetch(env, p);
    case canPostStoryResultBoostNeeded::ID:
      return canPostStoryResultBoostNeeded::fetch(env, p);
    case canPostStoryResultActiveStoryLimitExceeded::ID:
      return canPostStoryResultActiveStoryLimitExceeded::fetch(env, p);
    case canPostStoryResultWeeklyLimitExceeded::ID:
      return canPostStoryResultWeeklyLimitExceeded::fetch(env, p);
    case canPostStoryResultMonthlyLimitExceeded::ID:
      return canPostStoryResultMonthlyLimitExceeded::fetch(env, p);
    case canSendMessageToUserResultOk::ID:
      return canSendMessageToUserResultOk::fetch(env, p);
    case canSendMessageToUserResultUserHasPaidMessages::ID:
      return canSendMessageToUserResultUserHasPaidMessages::fetch(env, p);
    case canSendMessageToUserResultUserIsDeleted::ID:
      return canSendMessageToUserResultUserIsDeleted::fetch(env, p);
    case canSendMessageToUserResultUserRestrictsNewChats::ID:
      return canSendMessageToUserResultUserRestrictsNewChats::fetch(env, p);
    case canTransferOwnershipResultOk::ID:
      return canTransferOwnershipResultOk::fetch(env, p);
    case canTransferOwnershipResultPasswordNeeded::ID:
      return canTransferOwnershipResultPasswordNeeded::fetch(env, p);
    case canTransferOwnershipResultPasswordTooFresh::ID:
      return canTransferOwnershipResultPasswordTooFresh::fetch(env, p);
    case canTransferOwnershipResultSessionTooFresh::ID:
      return canTransferOwnershipResultSessionTooFresh::fetch(env, p);
    case chat::ID:
      return chat::fetch(env, p);
    case chatActionTyping::ID:
      return chatActionTyping::fetch(env, p);
    case chatActionRecordingVideo::ID:
      return chatActionRecordingVideo::fetch(env, p);
    case chatActionUploadingVideo::ID:
      return chatActionUploadingVideo::fetch(env, p);
    case chatActionRecordingVoiceNote::ID:
      return chatActionRecordingVoiceNote::fetch(env, p);
    case chatActionUploadingVoiceNote::ID:
      return chatActionUploadingVoiceNote::fetch(env, p);
    case chatActionUploadingPhoto::ID:
      return chatActionUploadingPhoto::fetch(env, p);
    case chatActionUploadingDocument::ID:
      return chatActionUploadingDocument::fetch(env, p);
    case chatActionChoosingSticker::ID:
      return chatActionChoosingSticker::fetch(env, p);
    case chatActionChoosingLocation::ID:
      return chatActionChoosingLocation::fetch(env, p);
    case chatActionChoosingContact::ID:
      return chatActionChoosingContact::fetch(env, p);
    case chatActionStartPlayingGame::ID:
      return chatActionStartPlayingGame::fetch(env, p);
    case chatActionRecordingVideoNote::ID:
      return chatActionRecordingVideoNote::fetch(env, p);
    case chatActionUploadingVideoNote::ID:
      return chatActionUploadingVideoNote::fetch(env, p);
    case chatActionWatchingAnimations::ID:
      return chatActionWatchingAnimations::fetch(env, p);
    case chatActionCancel::ID:
      return chatActionCancel::fetch(env, p);
    case chatActionBarReportSpam::ID:
      return chatActionBarReportSpam::fetch(env, p);
    case chatActionBarInviteMembers::ID:
      return chatActionBarInviteMembers::fetch(env, p);
    case chatActionBarReportAddBlock::ID:
      return chatActionBarReportAddBlock::fetch(env, p);
    case chatActionBarAddContact::ID:
      return chatActionBarAddContact::fetch(env, p);
    case chatActionBarSharePhoneNumber::ID:
      return chatActionBarSharePhoneNumber::fetch(env, p);
    case chatActionBarJoinRequest::ID:
      return chatActionBarJoinRequest::fetch(env, p);
    case chatActiveStories::ID:
      return chatActiveStories::fetch(env, p);
    case chatAdministrator::ID:
      return chatAdministrator::fetch(env, p);
    case chatAdministratorRights::ID:
      return chatAdministratorRights::fetch(env, p);
    case chatAdministrators::ID:
      return chatAdministrators::fetch(env, p);
    case chatAvailableReactionsAll::ID:
      return chatAvailableReactionsAll::fetch(env, p);
    case chatAvailableReactionsSome::ID:
      return chatAvailableReactionsSome::fetch(env, p);
    case chatBackground::ID:
      return chatBackground::fetch(env, p);
    case chatBoost::ID:
      return chatBoost::fetch(env, p);
    case chatBoostFeatures::ID:
      return chatBoostFeatures::fetch(env, p);
    case chatBoostLevelFeatures::ID:
      return chatBoostLevelFeatures::fetch(env, p);
    case chatBoostLink::ID:
      return chatBoostLink::fetch(env, p);
    case chatBoostLinkInfo::ID:
      return chatBoostLinkInfo::fetch(env, p);
    case chatBoostSlot::ID:
      return chatBoostSlot::fetch(env, p);
    case chatBoostSlots::ID:
      return chatBoostSlots::fetch(env, p);
    case chatBoostSourceGiftCode::ID:
      return chatBoostSourceGiftCode::fetch(env, p);
    case chatBoostSourceGiveaway::ID:
      return chatBoostSourceGiveaway::fetch(env, p);
    case chatBoostSourcePremium::ID:
      return chatBoostSourcePremium::fetch(env, p);
    case chatBoostStatus::ID:
      return chatBoostStatus::fetch(env, p);
    case chatEvent::ID:
      return chatEvent::fetch(env, p);
    case chatEventMessageEdited::ID:
      return chatEventMessageEdited::fetch(env, p);
    case chatEventMessageDeleted::ID:
      return chatEventMessageDeleted::fetch(env, p);
    case chatEventMessagePinned::ID:
      return chatEventMessagePinned::fetch(env, p);
    case chatEventMessageUnpinned::ID:
      return chatEventMessageUnpinned::fetch(env, p);
    case chatEventPollStopped::ID:
      return chatEventPollStopped::fetch(env, p);
    case chatEventMemberJoined::ID:
      return chatEventMemberJoined::fetch(env, p);
    case chatEventMemberJoinedByInviteLink::ID:
      return chatEventMemberJoinedByInviteLink::fetch(env, p);
    case chatEventMemberJoinedByRequest::ID:
      return chatEventMemberJoinedByRequest::fetch(env, p);
    case chatEventMemberInvited::ID:
      return chatEventMemberInvited::fetch(env, p);
    case chatEventMemberLeft::ID:
      return chatEventMemberLeft::fetch(env, p);
    case chatEventMemberPromoted::ID:
      return chatEventMemberPromoted::fetch(env, p);
    case chatEventMemberRestricted::ID:
      return chatEventMemberRestricted::fetch(env, p);
    case chatEventMemberSubscriptionExtended::ID:
      return chatEventMemberSubscriptionExtended::fetch(env, p);
    case chatEventAvailableReactionsChanged::ID:
      return chatEventAvailableReactionsChanged::fetch(env, p);
    case chatEventBackgroundChanged::ID:
      return chatEventBackgroundChanged::fetch(env, p);
    case chatEventDescriptionChanged::ID:
      return chatEventDescriptionChanged::fetch(env, p);
    case chatEventEmojiStatusChanged::ID:
      return chatEventEmojiStatusChanged::fetch(env, p);
    case chatEventLinkedChatChanged::ID:
      return chatEventLinkedChatChanged::fetch(env, p);
    case chatEventLocationChanged::ID:
      return chatEventLocationChanged::fetch(env, p);
    case chatEventMessageAutoDeleteTimeChanged::ID:
      return chatEventMessageAutoDeleteTimeChanged::fetch(env, p);
    case chatEventPermissionsChanged::ID:
      return chatEventPermissionsChanged::fetch(env, p);
    case chatEventPhotoChanged::ID:
      return chatEventPhotoChanged::fetch(env, p);
    case chatEventSlowModeDelayChanged::ID:
      return chatEventSlowModeDelayChanged::fetch(env, p);
    case chatEventStickerSetChanged::ID:
      return chatEventStickerSetChanged::fetch(env, p);
    case chatEventCustomEmojiStickerSetChanged::ID:
      return chatEventCustomEmojiStickerSetChanged::fetch(env, p);
    case chatEventTitleChanged::ID:
      return chatEventTitleChanged::fetch(env, p);
    case chatEventUsernameChanged::ID:
      return chatEventUsernameChanged::fetch(env, p);
    case chatEventActiveUsernamesChanged::ID:
      return chatEventActiveUsernamesChanged::fetch(env, p);
    case chatEventAccentColorChanged::ID:
      return chatEventAccentColorChanged::fetch(env, p);
    case chatEventProfileAccentColorChanged::ID:
      return chatEventProfileAccentColorChanged::fetch(env, p);
    case chatEventHasProtectedContentToggled::ID:
      return chatEventHasProtectedContentToggled::fetch(env, p);
    case chatEventInvitesToggled::ID:
      return chatEventInvitesToggled::fetch(env, p);
    case chatEventIsAllHistoryAvailableToggled::ID:
      return chatEventIsAllHistoryAvailableToggled::fetch(env, p);
    case chatEventHasAggressiveAntiSpamEnabledToggled::ID:
      return chatEventHasAggressiveAntiSpamEnabledToggled::fetch(env, p);
    case chatEventSignMessagesToggled::ID:
      return chatEventSignMessagesToggled::fetch(env, p);
    case chatEventShowMessageSenderToggled::ID:
      return chatEventShowMessageSenderToggled::fetch(env, p);
    case chatEventAutomaticTranslationToggled::ID:
      return chatEventAutomaticTranslationToggled::fetch(env, p);
    case chatEventInviteLinkEdited::ID:
      return chatEventInviteLinkEdited::fetch(env, p);
    case chatEventInviteLinkRevoked::ID:
      return chatEventInviteLinkRevoked::fetch(env, p);
    case chatEventInviteLinkDeleted::ID:
      return chatEventInviteLinkDeleted::fetch(env, p);
    case chatEventVideoChatCreated::ID:
      return chatEventVideoChatCreated::fetch(env, p);
    case chatEventVideoChatEnded::ID:
      return chatEventVideoChatEnded::fetch(env, p);
    case chatEventVideoChatMuteNewParticipantsToggled::ID:
      return chatEventVideoChatMuteNewParticipantsToggled::fetch(env, p);
    case chatEventVideoChatParticipantIsMutedToggled::ID:
      return chatEventVideoChatParticipantIsMutedToggled::fetch(env, p);
    case chatEventVideoChatParticipantVolumeLevelChanged::ID:
      return chatEventVideoChatParticipantVolumeLevelChanged::fetch(env, p);
    case chatEventIsForumToggled::ID:
      return chatEventIsForumToggled::fetch(env, p);
    case chatEventForumTopicCreated::ID:
      return chatEventForumTopicCreated::fetch(env, p);
    case chatEventForumTopicEdited::ID:
      return chatEventForumTopicEdited::fetch(env, p);
    case chatEventForumTopicToggleIsClosed::ID:
      return chatEventForumTopicToggleIsClosed::fetch(env, p);
    case chatEventForumTopicToggleIsHidden::ID:
      return chatEventForumTopicToggleIsHidden::fetch(env, p);
    case chatEventForumTopicDeleted::ID:
      return chatEventForumTopicDeleted::fetch(env, p);
    case chatEventForumTopicPinned::ID:
      return chatEventForumTopicPinned::fetch(env, p);
    case chatEventLogFilters::ID:
      return chatEventLogFilters::fetch(env, p);
    case chatEvents::ID:
      return chatEvents::fetch(env, p);
    case chatFolder::ID:
      return chatFolder::fetch(env, p);
    case chatFolderIcon::ID:
      return chatFolderIcon::fetch(env, p);
    case chatFolderInfo::ID:
      return chatFolderInfo::fetch(env, p);
    case chatFolderInviteLink::ID:
      return chatFolderInviteLink::fetch(env, p);
    case chatFolderInviteLinkInfo::ID:
      return chatFolderInviteLinkInfo::fetch(env, p);
    case chatFolderInviteLinks::ID:
      return chatFolderInviteLinks::fetch(env, p);
    case chatFolderName::ID:
      return chatFolderName::fetch(env, p);
    case chatInviteLink::ID:
      return chatInviteLink::fetch(env, p);
    case chatInviteLinkCount::ID:
      return chatInviteLinkCount::fetch(env, p);
    case chatInviteLinkCounts::ID:
      return chatInviteLinkCounts::fetch(env, p);
    case chatInviteLinkInfo::ID:
      return chatInviteLinkInfo::fetch(env, p);
    case chatInviteLinkMember::ID:
      return chatInviteLinkMember::fetch(env, p);
    case chatInviteLinkMembers::ID:
      return chatInviteLinkMembers::fetch(env, p);
    case chatInviteLinkSubscriptionInfo::ID:
      return chatInviteLinkSubscriptionInfo::fetch(env, p);
    case chatInviteLinks::ID:
      return chatInviteLinks::fetch(env, p);
    case chatJoinRequest::ID:
      return chatJoinRequest::fetch(env, p);
    case chatJoinRequests::ID:
      return chatJoinRequests::fetch(env, p);
    case chatJoinRequestsInfo::ID:
      return chatJoinRequestsInfo::fetch(env, p);
    case chatListMain::ID:
      return chatListMain::fetch(env, p);
    case chatListArchive::ID:
      return chatListArchive::fetch(env, p);
    case chatListFolder::ID:
      return chatListFolder::fetch(env, p);
    case chatLists::ID:
      return chatLists::fetch(env, p);
    case chatLocation::ID:
      return chatLocation::fetch(env, p);
    case chatMember::ID:
      return chatMember::fetch(env, p);
    case chatMemberStatusCreator::ID:
      return chatMemberStatusCreator::fetch(env, p);
    case chatMemberStatusAdministrator::ID:
      return chatMemberStatusAdministrator::fetch(env, p);
    case chatMemberStatusMember::ID:
      return chatMemberStatusMember::fetch(env, p);
    case chatMemberStatusRestricted::ID:
      return chatMemberStatusRestricted::fetch(env, p);
    case chatMemberStatusLeft::ID:
      return chatMemberStatusLeft::fetch(env, p);
    case chatMemberStatusBanned::ID:
      return chatMemberStatusBanned::fetch(env, p);
    case chatMembers::ID:
      return chatMembers::fetch(env, p);
    case chatMembersFilterContacts::ID:
      return chatMembersFilterContacts::fetch(env, p);
    case chatMembersFilterAdministrators::ID:
      return chatMembersFilterAdministrators::fetch(env, p);
    case chatMembersFilterMembers::ID:
      return chatMembersFilterMembers::fetch(env, p);
    case chatMembersFilterMention::ID:
      return chatMembersFilterMention::fetch(env, p);
    case chatMembersFilterRestricted::ID:
      return chatMembersFilterRestricted::fetch(env, p);
    case chatMembersFilterBanned::ID:
      return chatMembersFilterBanned::fetch(env, p);
    case chatMembersFilterBots::ID:
      return chatMembersFilterBots::fetch(env, p);
    case chatMessageSender::ID:
      return chatMessageSender::fetch(env, p);
    case chatMessageSenders::ID:
      return chatMessageSenders::fetch(env, p);
    case chatNotificationSettings::ID:
      return chatNotificationSettings::fetch(env, p);
    case chatPermissions::ID:
      return chatPermissions::fetch(env, p);
    case chatPhoto::ID:
      return chatPhoto::fetch(env, p);
    case chatPhotoInfo::ID:
      return chatPhotoInfo::fetch(env, p);
    case chatPhotoSticker::ID:
      return chatPhotoSticker::fetch(env, p);
    case chatPhotoStickerTypeRegularOrMask::ID:
      return chatPhotoStickerTypeRegularOrMask::fetch(env, p);
    case chatPhotoStickerTypeCustomEmoji::ID:
      return chatPhotoStickerTypeCustomEmoji::fetch(env, p);
    case chatPhotos::ID:
      return chatPhotos::fetch(env, p);
    case chatPosition::ID:
      return chatPosition::fetch(env, p);
    case chatRevenueAmount::ID:
      return chatRevenueAmount::fetch(env, p);
    case chatRevenueStatistics::ID:
      return chatRevenueStatistics::fetch(env, p);
    case chatRevenueTransaction::ID:
      return chatRevenueTransaction::fetch(env, p);
    case chatRevenueTransactionTypeEarnings::ID:
      return chatRevenueTransactionTypeEarnings::fetch(env, p);
    case chatRevenueTransactionTypeWithdrawal::ID:
      return chatRevenueTransactionTypeWithdrawal::fetch(env, p);
    case chatRevenueTransactionTypeRefund::ID:
      return chatRevenueTransactionTypeRefund::fetch(env, p);
    case chatRevenueTransactions::ID:
      return chatRevenueTransactions::fetch(env, p);
    case chatSourceMtprotoProxy::ID:
      return chatSourceMtprotoProxy::fetch(env, p);
    case chatSourcePublicServiceAnnouncement::ID:
      return chatSourcePublicServiceAnnouncement::fetch(env, p);
    case chatStatisticsSupergroup::ID:
      return chatStatisticsSupergroup::fetch(env, p);
    case chatStatisticsChannel::ID:
      return chatStatisticsChannel::fetch(env, p);
    case chatStatisticsAdministratorActionsInfo::ID:
      return chatStatisticsAdministratorActionsInfo::fetch(env, p);
    case chatStatisticsInteractionInfo::ID:
      return chatStatisticsInteractionInfo::fetch(env, p);
    case chatStatisticsInviterInfo::ID:
      return chatStatisticsInviterInfo::fetch(env, p);
    case chatStatisticsMessageSenderInfo::ID:
      return chatStatisticsMessageSenderInfo::fetch(env, p);
    case chatStatisticsObjectTypeMessage::ID:
      return chatStatisticsObjectTypeMessage::fetch(env, p);
    case chatStatisticsObjectTypeStory::ID:
      return chatStatisticsObjectTypeStory::fetch(env, p);
    case chatTheme::ID:
      return chatTheme::fetch(env, p);
    case chatTypePrivate::ID:
      return chatTypePrivate::fetch(env, p);
    case chatTypeBasicGroup::ID:
      return chatTypeBasicGroup::fetch(env, p);
    case chatTypeSupergroup::ID:
      return chatTypeSupergroup::fetch(env, p);
    case chatTypeSecret::ID:
      return chatTypeSecret::fetch(env, p);
    case chats::ID:
      return chats::fetch(env, p);
    case checkChatUsernameResultOk::ID:
      return checkChatUsernameResultOk::fetch(env, p);
    case checkChatUsernameResultUsernameInvalid::ID:
      return checkChatUsernameResultUsernameInvalid::fetch(env, p);
    case checkChatUsernameResultUsernameOccupied::ID:
      return checkChatUsernameResultUsernameOccupied::fetch(env, p);
    case checkChatUsernameResultUsernamePurchasable::ID:
      return checkChatUsernameResultUsernamePurchasable::fetch(env, p);
    case checkChatUsernameResultPublicChatsTooMany::ID:
      return checkChatUsernameResultPublicChatsTooMany::fetch(env, p);
    case checkChatUsernameResultPublicGroupsUnavailable::ID:
      return checkChatUsernameResultPublicGroupsUnavailable::fetch(env, p);
    case checkStickerSetNameResultOk::ID:
      return checkStickerSetNameResultOk::fetch(env, p);
    case checkStickerSetNameResultNameInvalid::ID:
      return checkStickerSetNameResultNameInvalid::fetch(env, p);
    case checkStickerSetNameResultNameOccupied::ID:
      return checkStickerSetNameResultNameOccupied::fetch(env, p);
    case closeBirthdayUser::ID:
      return closeBirthdayUser::fetch(env, p);
    case closedVectorPath::ID:
      return closedVectorPath::fetch(env, p);
    case collectibleItemInfo::ID:
      return collectibleItemInfo::fetch(env, p);
    case collectibleItemTypeUsername::ID:
      return collectibleItemTypeUsername::fetch(env, p);
    case collectibleItemTypePhoneNumber::ID:
      return collectibleItemTypePhoneNumber::fetch(env, p);
    case connectedAffiliateProgram::ID:
      return connectedAffiliateProgram::fetch(env, p);
    case connectedAffiliatePrograms::ID:
      return connectedAffiliatePrograms::fetch(env, p);
    case connectedWebsite::ID:
      return connectedWebsite::fetch(env, p);
    case connectedWebsites::ID:
      return connectedWebsites::fetch(env, p);
    case connectionStateWaitingForNetwork::ID:
      return connectionStateWaitingForNetwork::fetch(env, p);
    case connectionStateConnectingToProxy::ID:
      return connectionStateConnectingToProxy::fetch(env, p);
    case connectionStateConnecting::ID:
      return connectionStateConnecting::fetch(env, p);
    case connectionStateUpdating::ID:
      return connectionStateUpdating::fetch(env, p);
    case connectionStateReady::ID:
      return connectionStateReady::fetch(env, p);
    case contact::ID:
      return contact::fetch(env, p);
    case count::ID:
      return count::fetch(env, p);
    case countries::ID:
      return countries::fetch(env, p);
    case countryInfo::ID:
      return countryInfo::fetch(env, p);
    case createdBasicGroupChat::ID:
      return createdBasicGroupChat::fetch(env, p);
    case currentWeather::ID:
      return currentWeather::fetch(env, p);
    case customRequestResult::ID:
      return customRequestResult::fetch(env, p);
    case data::ID:
      return data::fetch(env, p);
    case databaseStatistics::ID:
      return databaseStatistics::fetch(env, p);
    case date::ID:
      return date::fetch(env, p);
    case dateRange::ID:
      return dateRange::fetch(env, p);
    case datedFile::ID:
      return datedFile::fetch(env, p);
    case deepLinkInfo::ID:
      return deepLinkInfo::fetch(env, p);
    case deviceTokenFirebaseCloudMessaging::ID:
      return deviceTokenFirebaseCloudMessaging::fetch(env, p);
    case deviceTokenApplePush::ID:
      return deviceTokenApplePush::fetch(env, p);
    case deviceTokenApplePushVoIP::ID:
      return deviceTokenApplePushVoIP::fetch(env, p);
    case deviceTokenWindowsPush::ID:
      return deviceTokenWindowsPush::fetch(env, p);
    case deviceTokenMicrosoftPush::ID:
      return deviceTokenMicrosoftPush::fetch(env, p);
    case deviceTokenMicrosoftPushVoIP::ID:
      return deviceTokenMicrosoftPushVoIP::fetch(env, p);
    case deviceTokenWebPush::ID:
      return deviceTokenWebPush::fetch(env, p);
    case deviceTokenSimplePush::ID:
      return deviceTokenSimplePush::fetch(env, p);
    case deviceTokenUbuntuPush::ID:
      return deviceTokenUbuntuPush::fetch(env, p);
    case deviceTokenBlackBerryPush::ID:
      return deviceTokenBlackBerryPush::fetch(env, p);
    case deviceTokenTizenPush::ID:
      return deviceTokenTizenPush::fetch(env, p);
    case deviceTokenHuaweiPush::ID:
      return deviceTokenHuaweiPush::fetch(env, p);
    case diceStickersRegular::ID:
      return diceStickersRegular::fetch(env, p);
    case diceStickersSlotMachine::ID:
      return diceStickersSlotMachine::fetch(env, p);
    case document::ID:
      return document::fetch(env, p);
    case downloadedFileCounts::ID:
      return downloadedFileCounts::fetch(env, p);
    case draftMessage::ID:
      return draftMessage::fetch(env, p);
    case emailAddressAuthenticationCode::ID:
      return emailAddressAuthenticationCode::fetch(env, p);
    case emailAddressAuthenticationAppleId::ID:
      return emailAddressAuthenticationAppleId::fetch(env, p);
    case emailAddressAuthenticationGoogleId::ID:
      return emailAddressAuthenticationGoogleId::fetch(env, p);
    case emailAddressAuthenticationCodeInfo::ID:
      return emailAddressAuthenticationCodeInfo::fetch(env, p);
    case emailAddressResetStateAvailable::ID:
      return emailAddressResetStateAvailable::fetch(env, p);
    case emailAddressResetStatePending::ID:
      return emailAddressResetStatePending::fetch(env, p);
    case emojiCategories::ID:
      return emojiCategories::fetch(env, p);
    case emojiCategory::ID:
      return emojiCategory::fetch(env, p);
    case emojiCategorySourceSearch::ID:
      return emojiCategorySourceSearch::fetch(env, p);
    case emojiCategorySourcePremium::ID:
      return emojiCategorySourcePremium::fetch(env, p);
    case emojiCategoryTypeDefault::ID:
      return emojiCategoryTypeDefault::fetch(env, p);
    case emojiCategoryTypeRegularStickers::ID:
      return emojiCategoryTypeRegularStickers::fetch(env, p);
    case emojiCategoryTypeEmojiStatus::ID:
      return emojiCategoryTypeEmojiStatus::fetch(env, p);
    case emojiCategoryTypeChatPhoto::ID:
      return emojiCategoryTypeChatPhoto::fetch(env, p);
    case emojiKeyword::ID:
      return emojiKeyword::fetch(env, p);
    case emojiKeywords::ID:
      return emojiKeywords::fetch(env, p);
    case emojiReaction::ID:
      return emojiReaction::fetch(env, p);
    case emojiStatus::ID:
      return emojiStatus::fetch(env, p);
    case emojiStatusCustomEmojis::ID:
      return emojiStatusCustomEmojis::fetch(env, p);
    case emojiStatusTypeCustomEmoji::ID:
      return emojiStatusTypeCustomEmoji::fetch(env, p);
    case emojiStatusTypeUpgradedGift::ID:
      return emojiStatusTypeUpgradedGift::fetch(env, p);
    case emojiStatuses::ID:
      return emojiStatuses::fetch(env, p);
    case emojis::ID:
      return emojis::fetch(env, p);
    case encryptedCredentials::ID:
      return encryptedCredentials::fetch(env, p);
    case encryptedPassportElement::ID:
      return encryptedPassportElement::fetch(env, p);
    case error::ID:
      return error::fetch(env, p);
    case factCheck::ID:
      return factCheck::fetch(env, p);
    case failedToAddMember::ID:
      return failedToAddMember::fetch(env, p);
    case failedToAddMembers::ID:
      return failedToAddMembers::fetch(env, p);
    case file::ID:
      return file::fetch(env, p);
    case fileDownload::ID:
      return fileDownload::fetch(env, p);
    case fileDownloadedPrefixSize::ID:
      return fileDownloadedPrefixSize::fetch(env, p);
    case fileTypeNone::ID:
      return fileTypeNone::fetch(env, p);
    case fileTypeAnimation::ID:
      return fileTypeAnimation::fetch(env, p);
    case fileTypeAudio::ID:
      return fileTypeAudio::fetch(env, p);
    case fileTypeDocument::ID:
      return fileTypeDocument::fetch(env, p);
    case fileTypeNotificationSound::ID:
      return fileTypeNotificationSound::fetch(env, p);
    case fileTypePhoto::ID:
      return fileTypePhoto::fetch(env, p);
    case fileTypePhotoStory::ID:
      return fileTypePhotoStory::fetch(env, p);
    case fileTypeProfilePhoto::ID:
      return fileTypeProfilePhoto::fetch(env, p);
    case fileTypeSecret::ID:
      return fileTypeSecret::fetch(env, p);
    case fileTypeSecretThumbnail::ID:
      return fileTypeSecretThumbnail::fetch(env, p);
    case fileTypeSecure::ID:
      return fileTypeSecure::fetch(env, p);
    case fileTypeSelfDestructingPhoto::ID:
      return fileTypeSelfDestructingPhoto::fetch(env, p);
    case fileTypeSelfDestructingVideo::ID:
      return fileTypeSelfDestructingVideo::fetch(env, p);
    case fileTypeSelfDestructingVideoNote::ID:
      return fileTypeSelfDestructingVideoNote::fetch(env, p);
    case fileTypeSelfDestructingVoiceNote::ID:
      return fileTypeSelfDestructingVoiceNote::fetch(env, p);
    case fileTypeSticker::ID:
      return fileTypeSticker::fetch(env, p);
    case fileTypeThumbnail::ID:
      return fileTypeThumbnail::fetch(env, p);
    case fileTypeUnknown::ID:
      return fileTypeUnknown::fetch(env, p);
    case fileTypeVideo::ID:
      return fileTypeVideo::fetch(env, p);
    case fileTypeVideoNote::ID:
      return fileTypeVideoNote::fetch(env, p);
    case fileTypeVideoStory::ID:
      return fileTypeVideoStory::fetch(env, p);
    case fileTypeVoiceNote::ID:
      return fileTypeVoiceNote::fetch(env, p);
    case fileTypeWallpaper::ID:
      return fileTypeWallpaper::fetch(env, p);
    case firebaseAuthenticationSettingsAndroid::ID:
      return firebaseAuthenticationSettingsAndroid::fetch(env, p);
    case firebaseAuthenticationSettingsIos::ID:
      return firebaseAuthenticationSettingsIos::fetch(env, p);
    case firebaseDeviceVerificationParametersSafetyNet::ID:
      return firebaseDeviceVerificationParametersSafetyNet::fetch(env, p);
    case firebaseDeviceVerificationParametersPlayIntegrity::ID:
      return firebaseDeviceVerificationParametersPlayIntegrity::fetch(env, p);
    case formattedText::ID:
      return formattedText::fetch(env, p);
    case forumTopic::ID:
      return forumTopic::fetch(env, p);
    case forumTopicIcon::ID:
      return forumTopicIcon::fetch(env, p);
    case forumTopicInfo::ID:
      return forumTopicInfo::fetch(env, p);
    case forumTopics::ID:
      return forumTopics::fetch(env, p);
    case forwardSource::ID:
      return forwardSource::fetch(env, p);
    case foundAffiliateProgram::ID:
      return foundAffiliateProgram::fetch(env, p);
    case foundAffiliatePrograms::ID:
      return foundAffiliatePrograms::fetch(env, p);
    case foundChatBoosts::ID:
      return foundChatBoosts::fetch(env, p);
    case foundChatMessages::ID:
      return foundChatMessages::fetch(env, p);
    case foundFileDownloads::ID:
      return foundFileDownloads::fetch(env, p);
    case foundMessages::ID:
      return foundMessages::fetch(env, p);
    case foundPosition::ID:
      return foundPosition::fetch(env, p);
    case foundPositions::ID:
      return foundPositions::fetch(env, p);
    case foundStories::ID:
      return foundStories::fetch(env, p);
    case foundUsers::ID:
      return foundUsers::fetch(env, p);
    case foundWebApp::ID:
      return foundWebApp::fetch(env, p);
    case game::ID:
      return game::fetch(env, p);
    case gameHighScore::ID:
      return gameHighScore::fetch(env, p);
    case gameHighScores::ID:
      return gameHighScores::fetch(env, p);
    case gift::ID:
      return gift::fetch(env, p);
    case giftForResale::ID:
      return giftForResale::fetch(env, p);
    case giftForResaleOrderPrice::ID:
      return giftForResaleOrderPrice::fetch(env, p);
    case giftForResaleOrderPriceChangeDate::ID:
      return giftForResaleOrderPriceChangeDate::fetch(env, p);
    case giftForResaleOrderNumber::ID:
      return giftForResaleOrderNumber::fetch(env, p);
    case giftSettings::ID:
      return giftSettings::fetch(env, p);
    case giftUpgradePreview::ID:
      return giftUpgradePreview::fetch(env, p);
    case giftsForResale::ID:
      return giftsForResale::fetch(env, p);
    case giveawayInfoOngoing::ID:
      return giveawayInfoOngoing::fetch(env, p);
    case giveawayInfoCompleted::ID:
      return giveawayInfoCompleted::fetch(env, p);
    case giveawayParameters::ID:
      return giveawayParameters::fetch(env, p);
    case giveawayParticipantStatusEligible::ID:
      return giveawayParticipantStatusEligible::fetch(env, p);
    case giveawayParticipantStatusParticipating::ID:
      return giveawayParticipantStatusParticipating::fetch(env, p);
    case giveawayParticipantStatusAlreadyWasMember::ID:
      return giveawayParticipantStatusAlreadyWasMember::fetch(env, p);
    case giveawayParticipantStatusAdministrator::ID:
      return giveawayParticipantStatusAdministrator::fetch(env, p);
    case giveawayParticipantStatusDisallowedCountry::ID:
      return giveawayParticipantStatusDisallowedCountry::fetch(env, p);
    case giveawayPrizePremium::ID:
      return giveawayPrizePremium::fetch(env, p);
    case giveawayPrizeStars::ID:
      return giveawayPrizeStars::fetch(env, p);
    case groupCall::ID:
      return groupCall::fetch(env, p);
    case groupCallDataChannelMain::ID:
      return groupCallDataChannelMain::fetch(env, p);
    case groupCallDataChannelScreenSharing::ID:
      return groupCallDataChannelScreenSharing::fetch(env, p);
    case groupCallId::ID:
      return groupCallId::fetch(env, p);
    case groupCallInfo::ID:
      return groupCallInfo::fetch(env, p);
    case groupCallJoinParameters::ID:
      return groupCallJoinParameters::fetch(env, p);
    case groupCallParticipant::ID:
      return groupCallParticipant::fetch(env, p);
    case groupCallParticipantVideoInfo::ID:
      return groupCallParticipantVideoInfo::fetch(env, p);
    case groupCallParticipants::ID:
      return groupCallParticipants::fetch(env, p);
    case groupCallRecentSpeaker::ID:
      return groupCallRecentSpeaker::fetch(env, p);
    case groupCallVideoQualityThumbnail::ID:
      return groupCallVideoQualityThumbnail::fetch(env, p);
    case groupCallVideoQualityMedium::ID:
      return groupCallVideoQualityMedium::fetch(env, p);
    case groupCallVideoQualityFull::ID:
      return groupCallVideoQualityFull::fetch(env, p);
    case groupCallVideoSourceGroup::ID:
      return groupCallVideoSourceGroup::fetch(env, p);
    case hashtags::ID:
      return hashtags::fetch(env, p);
    case httpUrl::ID:
      return httpUrl::fetch(env, p);
    case identityDocument::ID:
      return identityDocument::fetch(env, p);
    case importedContacts::ID:
      return importedContacts::fetch(env, p);
    case inlineKeyboardButton::ID:
      return inlineKeyboardButton::fetch(env, p);
    case inlineKeyboardButtonTypeUrl::ID:
      return inlineKeyboardButtonTypeUrl::fetch(env, p);
    case inlineKeyboardButtonTypeLoginUrl::ID:
      return inlineKeyboardButtonTypeLoginUrl::fetch(env, p);
    case inlineKeyboardButtonTypeWebApp::ID:
      return inlineKeyboardButtonTypeWebApp::fetch(env, p);
    case inlineKeyboardButtonTypeCallback::ID:
      return inlineKeyboardButtonTypeCallback::fetch(env, p);
    case inlineKeyboardButtonTypeCallbackWithPassword::ID:
      return inlineKeyboardButtonTypeCallbackWithPassword::fetch(env, p);
    case inlineKeyboardButtonTypeCallbackGame::ID:
      return inlineKeyboardButtonTypeCallbackGame::fetch(env, p);
    case inlineKeyboardButtonTypeSwitchInline::ID:
      return inlineKeyboardButtonTypeSwitchInline::fetch(env, p);
    case inlineKeyboardButtonTypeBuy::ID:
      return inlineKeyboardButtonTypeBuy::fetch(env, p);
    case inlineKeyboardButtonTypeUser::ID:
      return inlineKeyboardButtonTypeUser::fetch(env, p);
    case inlineKeyboardButtonTypeCopyText::ID:
      return inlineKeyboardButtonTypeCopyText::fetch(env, p);
    case inlineQueryResultArticle::ID:
      return inlineQueryResultArticle::fetch(env, p);
    case inlineQueryResultContact::ID:
      return inlineQueryResultContact::fetch(env, p);
    case inlineQueryResultLocation::ID:
      return inlineQueryResultLocation::fetch(env, p);
    case inlineQueryResultVenue::ID:
      return inlineQueryResultVenue::fetch(env, p);
    case inlineQueryResultGame::ID:
      return inlineQueryResultGame::fetch(env, p);
    case inlineQueryResultAnimation::ID:
      return inlineQueryResultAnimation::fetch(env, p);
    case inlineQueryResultAudio::ID:
      return inlineQueryResultAudio::fetch(env, p);
    case inlineQueryResultDocument::ID:
      return inlineQueryResultDocument::fetch(env, p);
    case inlineQueryResultPhoto::ID:
      return inlineQueryResultPhoto::fetch(env, p);
    case inlineQueryResultSticker::ID:
      return inlineQueryResultSticker::fetch(env, p);
    case inlineQueryResultVideo::ID:
      return inlineQueryResultVideo::fetch(env, p);
    case inlineQueryResultVoiceNote::ID:
      return inlineQueryResultVoiceNote::fetch(env, p);
    case inlineQueryResults::ID:
      return inlineQueryResults::fetch(env, p);
    case inlineQueryResultsButton::ID:
      return inlineQueryResultsButton::fetch(env, p);
    case inlineQueryResultsButtonTypeStartBot::ID:
      return inlineQueryResultsButtonTypeStartBot::fetch(env, p);
    case inlineQueryResultsButtonTypeWebApp::ID:
      return inlineQueryResultsButtonTypeWebApp::fetch(env, p);
    case inputBackgroundLocal::ID:
      return inputBackgroundLocal::fetch(env, p);
    case inputBackgroundRemote::ID:
      return inputBackgroundRemote::fetch(env, p);
    case inputBackgroundPrevious::ID:
      return inputBackgroundPrevious::fetch(env, p);
    case inputBusinessChatLink::ID:
      return inputBusinessChatLink::fetch(env, p);
    case inputBusinessStartPage::ID:
      return inputBusinessStartPage::fetch(env, p);
    case inputChatPhotoPrevious::ID:
      return inputChatPhotoPrevious::fetch(env, p);
    case inputChatPhotoStatic::ID:
      return inputChatPhotoStatic::fetch(env, p);
    case inputChatPhotoAnimation::ID:
      return inputChatPhotoAnimation::fetch(env, p);
    case inputChatPhotoSticker::ID:
      return inputChatPhotoSticker::fetch(env, p);
    case inputCredentialsSaved::ID:
      return inputCredentialsSaved::fetch(env, p);
    case inputCredentialsNew::ID:
      return inputCredentialsNew::fetch(env, p);
    case inputCredentialsApplePay::ID:
      return inputCredentialsApplePay::fetch(env, p);
    case inputCredentialsGooglePay::ID:
      return inputCredentialsGooglePay::fetch(env, p);
    case inputFileId::ID:
      return inputFileId::fetch(env, p);
    case inputFileRemote::ID:
      return inputFileRemote::fetch(env, p);
    case inputFileLocal::ID:
      return inputFileLocal::fetch(env, p);
    case inputFileGenerated::ID:
      return inputFileGenerated::fetch(env, p);
    case inputGroupCallLink::ID:
      return inputGroupCallLink::fetch(env, p);
    case inputGroupCallMessage::ID:
      return inputGroupCallMessage::fetch(env, p);
    case inputIdentityDocument::ID:
      return inputIdentityDocument::fetch(env, p);
    case inputInlineQueryResultAnimation::ID:
      return inputInlineQueryResultAnimation::fetch(env, p);
    case inputInlineQueryResultArticle::ID:
      return inputInlineQueryResultArticle::fetch(env, p);
    case inputInlineQueryResultAudio::ID:
      return inputInlineQueryResultAudio::fetch(env, p);
    case inputInlineQueryResultContact::ID:
      return inputInlineQueryResultContact::fetch(env, p);
    case inputInlineQueryResultDocument::ID:
      return inputInlineQueryResultDocument::fetch(env, p);
    case inputInlineQueryResultGame::ID:
      return inputInlineQueryResultGame::fetch(env, p);
    case inputInlineQueryResultLocation::ID:
      return inputInlineQueryResultLocation::fetch(env, p);
    case inputInlineQueryResultPhoto::ID:
      return inputInlineQueryResultPhoto::fetch(env, p);
    case inputInlineQueryResultSticker::ID:
      return inputInlineQueryResultSticker::fetch(env, p);
    case inputInlineQueryResultVenue::ID:
      return inputInlineQueryResultVenue::fetch(env, p);
    case inputInlineQueryResultVideo::ID:
      return inputInlineQueryResultVideo::fetch(env, p);
    case inputInlineQueryResultVoiceNote::ID:
      return inputInlineQueryResultVoiceNote::fetch(env, p);
    case inputInvoiceMessage::ID:
      return inputInvoiceMessage::fetch(env, p);
    case inputInvoiceName::ID:
      return inputInvoiceName::fetch(env, p);
    case inputInvoiceTelegram::ID:
      return inputInvoiceTelegram::fetch(env, p);
    case inputMessageText::ID:
      return inputMessageText::fetch(env, p);
    case inputMessageAnimation::ID:
      return inputMessageAnimation::fetch(env, p);
    case inputMessageAudio::ID:
      return inputMessageAudio::fetch(env, p);
    case inputMessageDocument::ID:
      return inputMessageDocument::fetch(env, p);
    case inputMessagePaidMedia::ID:
      return inputMessagePaidMedia::fetch(env, p);
    case inputMessagePhoto::ID:
      return inputMessagePhoto::fetch(env, p);
    case inputMessageSticker::ID:
      return inputMessageSticker::fetch(env, p);
    case inputMessageVideo::ID:
      return inputMessageVideo::fetch(env, p);
    case inputMessageVideoNote::ID:
      return inputMessageVideoNote::fetch(env, p);
    case inputMessageVoiceNote::ID:
      return inputMessageVoiceNote::fetch(env, p);
    case inputMessageLocation::ID:
      return inputMessageLocation::fetch(env, p);
    case inputMessageVenue::ID:
      return inputMessageVenue::fetch(env, p);
    case inputMessageContact::ID:
      return inputMessageContact::fetch(env, p);
    case inputMessageDice::ID:
      return inputMessageDice::fetch(env, p);
    case inputMessageGame::ID:
      return inputMessageGame::fetch(env, p);
    case inputMessageInvoice::ID:
      return inputMessageInvoice::fetch(env, p);
    case inputMessagePoll::ID:
      return inputMessagePoll::fetch(env, p);
    case inputMessageStory::ID:
      return inputMessageStory::fetch(env, p);
    case inputMessageForwarded::ID:
      return inputMessageForwarded::fetch(env, p);
    case inputMessageReplyToMessage::ID:
      return inputMessageReplyToMessage::fetch(env, p);
    case inputMessageReplyToExternalMessage::ID:
      return inputMessageReplyToExternalMessage::fetch(env, p);
    case inputMessageReplyToStory::ID:
      return inputMessageReplyToStory::fetch(env, p);
    case inputPaidMedia::ID:
      return inputPaidMedia::fetch(env, p);
    case inputPaidMediaTypePhoto::ID:
      return inputPaidMediaTypePhoto::fetch(env, p);
    case inputPaidMediaTypeVideo::ID:
      return inputPaidMediaTypeVideo::fetch(env, p);
    case inputPassportElementPersonalDetails::ID:
      return inputPassportElementPersonalDetails::fetch(env, p);
    case inputPassportElementPassport::ID:
      return inputPassportElementPassport::fetch(env, p);
    case inputPassportElementDriverLicense::ID:
      return inputPassportElementDriverLicense::fetch(env, p);
    case inputPassportElementIdentityCard::ID:
      return inputPassportElementIdentityCard::fetch(env, p);
    case inputPassportElementInternalPassport::ID:
      return inputPassportElementInternalPassport::fetch(env, p);
    case inputPassportElementAddress::ID:
      return inputPassportElementAddress::fetch(env, p);
    case inputPassportElementUtilityBill::ID:
      return inputPassportElementUtilityBill::fetch(env, p);
    case inputPassportElementBankStatement::ID:
      return inputPassportElementBankStatement::fetch(env, p);
    case inputPassportElementRentalAgreement::ID:
      return inputPassportElementRentalAgreement::fetch(env, p);
    case inputPassportElementPassportRegistration::ID:
      return inputPassportElementPassportRegistration::fetch(env, p);
    case inputPassportElementTemporaryRegistration::ID:
      return inputPassportElementTemporaryRegistration::fetch(env, p);
    case inputPassportElementPhoneNumber::ID:
      return inputPassportElementPhoneNumber::fetch(env, p);
    case inputPassportElementEmailAddress::ID:
      return inputPassportElementEmailAddress::fetch(env, p);
    case inputPassportElementError::ID:
      return inputPassportElementError::fetch(env, p);
    case inputPassportElementErrorSourceUnspecified::ID:
      return inputPassportElementErrorSourceUnspecified::fetch(env, p);
    case inputPassportElementErrorSourceDataField::ID:
      return inputPassportElementErrorSourceDataField::fetch(env, p);
    case inputPassportElementErrorSourceFrontSide::ID:
      return inputPassportElementErrorSourceFrontSide::fetch(env, p);
    case inputPassportElementErrorSourceReverseSide::ID:
      return inputPassportElementErrorSourceReverseSide::fetch(env, p);
    case inputPassportElementErrorSourceSelfie::ID:
      return inputPassportElementErrorSourceSelfie::fetch(env, p);
    case inputPassportElementErrorSourceTranslationFile::ID:
      return inputPassportElementErrorSourceTranslationFile::fetch(env, p);
    case inputPassportElementErrorSourceTranslationFiles::ID:
      return inputPassportElementErrorSourceTranslationFiles::fetch(env, p);
    case inputPassportElementErrorSourceFile::ID:
      return inputPassportElementErrorSourceFile::fetch(env, p);
    case inputPassportElementErrorSourceFiles::ID:
      return inputPassportElementErrorSourceFiles::fetch(env, p);
    case inputPersonalDocument::ID:
      return inputPersonalDocument::fetch(env, p);
    case inputSticker::ID:
      return inputSticker::fetch(env, p);
    case inputStoryArea::ID:
      return inputStoryArea::fetch(env, p);
    case inputStoryAreaTypeLocation::ID:
      return inputStoryAreaTypeLocation::fetch(env, p);
    case inputStoryAreaTypeFoundVenue::ID:
      return inputStoryAreaTypeFoundVenue::fetch(env, p);
    case inputStoryAreaTypePreviousVenue::ID:
      return inputStoryAreaTypePreviousVenue::fetch(env, p);
    case inputStoryAreaTypeSuggestedReaction::ID:
      return inputStoryAreaTypeSuggestedReaction::fetch(env, p);
    case inputStoryAreaTypeMessage::ID:
      return inputStoryAreaTypeMessage::fetch(env, p);
    case inputStoryAreaTypeLink::ID:
      return inputStoryAreaTypeLink::fetch(env, p);
    case inputStoryAreaTypeWeather::ID:
      return inputStoryAreaTypeWeather::fetch(env, p);
    case inputStoryAreaTypeUpgradedGift::ID:
      return inputStoryAreaTypeUpgradedGift::fetch(env, p);
    case inputStoryAreas::ID:
      return inputStoryAreas::fetch(env, p);
    case inputStoryContentPhoto::ID:
      return inputStoryContentPhoto::fetch(env, p);
    case inputStoryContentVideo::ID:
      return inputStoryContentVideo::fetch(env, p);
    case inputTextQuote::ID:
      return inputTextQuote::fetch(env, p);
    case inputThumbnail::ID:
      return inputThumbnail::fetch(env, p);
    case internalLinkTypeActiveSessions::ID:
      return internalLinkTypeActiveSessions::fetch(env, p);
    case internalLinkTypeAttachmentMenuBot::ID:
      return internalLinkTypeAttachmentMenuBot::fetch(env, p);
    case internalLinkTypeAuthenticationCode::ID:
      return internalLinkTypeAuthenticationCode::fetch(env, p);
    case internalLinkTypeBackground::ID:
      return internalLinkTypeBackground::fetch(env, p);
    case internalLinkTypeBotAddToChannel::ID:
      return internalLinkTypeBotAddToChannel::fetch(env, p);
    case internalLinkTypeBotStart::ID:
      return internalLinkTypeBotStart::fetch(env, p);
    case internalLinkTypeBotStartInGroup::ID:
      return internalLinkTypeBotStartInGroup::fetch(env, p);
    case internalLinkTypeBusinessChat::ID:
      return internalLinkTypeBusinessChat::fetch(env, p);
    case internalLinkTypeBuyStars::ID:
      return internalLinkTypeBuyStars::fetch(env, p);
    case internalLinkTypeChangePhoneNumber::ID:
      return internalLinkTypeChangePhoneNumber::fetch(env, p);
    case internalLinkTypeChatAffiliateProgram::ID:
      return internalLinkTypeChatAffiliateProgram::fetch(env, p);
    case internalLinkTypeChatBoost::ID:
      return internalLinkTypeChatBoost::fetch(env, p);
    case internalLinkTypeChatFolderInvite::ID:
      return internalLinkTypeChatFolderInvite::fetch(env, p);
    case internalLinkTypeChatFolderSettings::ID:
      return internalLinkTypeChatFolderSettings::fetch(env, p);
    case internalLinkTypeChatInvite::ID:
      return internalLinkTypeChatInvite::fetch(env, p);
    case internalLinkTypeDefaultMessageAutoDeleteTimerSettings::ID:
      return internalLinkTypeDefaultMessageAutoDeleteTimerSettings::fetch(env, p);
    case internalLinkTypeEditProfileSettings::ID:
      return internalLinkTypeEditProfileSettings::fetch(env, p);
    case internalLinkTypeGame::ID:
      return internalLinkTypeGame::fetch(env, p);
    case internalLinkTypeGroupCall::ID:
      return internalLinkTypeGroupCall::fetch(env, p);
    case internalLinkTypeInstantView::ID:
      return internalLinkTypeInstantView::fetch(env, p);
    case internalLinkTypeInvoice::ID:
      return internalLinkTypeInvoice::fetch(env, p);
    case internalLinkTypeLanguagePack::ID:
      return internalLinkTypeLanguagePack::fetch(env, p);
    case internalLinkTypeLanguageSettings::ID:
      return internalLinkTypeLanguageSettings::fetch(env, p);
    case internalLinkTypeMainWebApp::ID:
      return internalLinkTypeMainWebApp::fetch(env, p);
    case internalLinkTypeMessage::ID:
      return internalLinkTypeMessage::fetch(env, p);
    case internalLinkTypeMessageDraft::ID:
      return internalLinkTypeMessageDraft::fetch(env, p);
    case internalLinkTypeMyStars::ID:
      return internalLinkTypeMyStars::fetch(env, p);
    case internalLinkTypePassportDataRequest::ID:
      return internalLinkTypePassportDataRequest::fetch(env, p);
    case internalLinkTypePhoneNumberConfirmation::ID:
      return internalLinkTypePhoneNumberConfirmation::fetch(env, p);
    case internalLinkTypePremiumFeatures::ID:
      return internalLinkTypePremiumFeatures::fetch(env, p);
    case internalLinkTypePremiumGift::ID:
      return internalLinkTypePremiumGift::fetch(env, p);
    case internalLinkTypePremiumGiftCode::ID:
      return internalLinkTypePremiumGiftCode::fetch(env, p);
    case internalLinkTypePrivacyAndSecuritySettings::ID:
      return internalLinkTypePrivacyAndSecuritySettings::fetch(env, p);
    case internalLinkTypeProxy::ID:
      return internalLinkTypeProxy::fetch(env, p);
    case internalLinkTypePublicChat::ID:
      return internalLinkTypePublicChat::fetch(env, p);
    case internalLinkTypeQrCodeAuthentication::ID:
      return internalLinkTypeQrCodeAuthentication::fetch(env, p);
    case internalLinkTypeRestorePurchases::ID:
      return internalLinkTypeRestorePurchases::fetch(env, p);
    case internalLinkTypeSettings::ID:
      return internalLinkTypeSettings::fetch(env, p);
    case internalLinkTypeStickerSet::ID:
      return internalLinkTypeStickerSet::fetch(env, p);
    case internalLinkTypeStory::ID:
      return internalLinkTypeStory::fetch(env, p);
    case internalLinkTypeTheme::ID:
      return internalLinkTypeTheme::fetch(env, p);
    case internalLinkTypeThemeSettings::ID:
      return internalLinkTypeThemeSettings::fetch(env, p);
    case internalLinkTypeUnknownDeepLink::ID:
      return internalLinkTypeUnknownDeepLink::fetch(env, p);
    case internalLinkTypeUnsupportedProxy::ID:
      return internalLinkTypeUnsupportedProxy::fetch(env, p);
    case internalLinkTypeUpgradedGift::ID:
      return internalLinkTypeUpgradedGift::fetch(env, p);
    case internalLinkTypeUserPhoneNumber::ID:
      return internalLinkTypeUserPhoneNumber::fetch(env, p);
    case internalLinkTypeUserToken::ID:
      return internalLinkTypeUserToken::fetch(env, p);
    case internalLinkTypeVideoChat::ID:
      return internalLinkTypeVideoChat::fetch(env, p);
    case internalLinkTypeWebApp::ID:
      return internalLinkTypeWebApp::fetch(env, p);
    case inviteGroupCallParticipantResultUserPrivacyRestricted::ID:
      return inviteGroupCallParticipantResultUserPrivacyRestricted::fetch(env, p);
    case inviteGroupCallParticipantResultUserAlreadyParticipant::ID:
      return inviteGroupCallParticipantResultUserAlreadyParticipant::fetch(env, p);
    case inviteGroupCallParticipantResultUserWasBanned::ID:
      return inviteGroupCallParticipantResultUserWasBanned::fetch(env, p);
    case inviteGroupCallParticipantResultSuccess::ID:
      return inviteGroupCallParticipantResultSuccess::fetch(env, p);
    case inviteLinkChatTypeBasicGroup::ID:
      return inviteLinkChatTypeBasicGroup::fetch(env, p);
    case inviteLinkChatTypeSupergroup::ID:
      return inviteLinkChatTypeSupergroup::fetch(env, p);
    case inviteLinkChatTypeChannel::ID:
      return inviteLinkChatTypeChannel::fetch(env, p);
    case invoice::ID:
      return invoice::fetch(env, p);
    case jsonObjectMember::ID:
      return jsonObjectMember::fetch(env, p);
    case jsonValueNull::ID:
      return jsonValueNull::fetch(env, p);
    case jsonValueBoolean::ID:
      return jsonValueBoolean::fetch(env, p);
    case jsonValueNumber::ID:
      return jsonValueNumber::fetch(env, p);
    case jsonValueString::ID:
      return jsonValueString::fetch(env, p);
    case jsonValueArray::ID:
      return jsonValueArray::fetch(env, p);
    case jsonValueObject::ID:
      return jsonValueObject::fetch(env, p);
    case keyboardButton::ID:
      return keyboardButton::fetch(env, p);
    case keyboardButtonTypeText::ID:
      return keyboardButtonTypeText::fetch(env, p);
    case keyboardButtonTypeRequestPhoneNumber::ID:
      return keyboardButtonTypeRequestPhoneNumber::fetch(env, p);
    case keyboardButtonTypeRequestLocation::ID:
      return keyboardButtonTypeRequestLocation::fetch(env, p);
    case keyboardButtonTypeRequestPoll::ID:
      return keyboardButtonTypeRequestPoll::fetch(env, p);
    case keyboardButtonTypeRequestUsers::ID:
      return keyboardButtonTypeRequestUsers::fetch(env, p);
    case keyboardButtonTypeRequestChat::ID:
      return keyboardButtonTypeRequestChat::fetch(env, p);
    case keyboardButtonTypeWebApp::ID:
      return keyboardButtonTypeWebApp::fetch(env, p);
    case labeledPricePart::ID:
      return labeledPricePart::fetch(env, p);
    case languagePackInfo::ID:
      return languagePackInfo::fetch(env, p);
    case languagePackString::ID:
      return languagePackString::fetch(env, p);
    case languagePackStringValueOrdinary::ID:
      return languagePackStringValueOrdinary::fetch(env, p);
    case languagePackStringValuePluralized::ID:
      return languagePackStringValuePluralized::fetch(env, p);
    case languagePackStringValueDeleted::ID:
      return languagePackStringValueDeleted::fetch(env, p);
    case languagePackStrings::ID:
      return languagePackStrings::fetch(env, p);
    case linkPreview::ID:
      return linkPreview::fetch(env, p);
    case linkPreviewAlbumMediaPhoto::ID:
      return linkPreviewAlbumMediaPhoto::fetch(env, p);
    case linkPreviewAlbumMediaVideo::ID:
      return linkPreviewAlbumMediaVideo::fetch(env, p);
    case linkPreviewOptions::ID:
      return linkPreviewOptions::fetch(env, p);
    case linkPreviewTypeAlbum::ID:
      return linkPreviewTypeAlbum::fetch(env, p);
    case linkPreviewTypeAnimation::ID:
      return linkPreviewTypeAnimation::fetch(env, p);
    case linkPreviewTypeApp::ID:
      return linkPreviewTypeApp::fetch(env, p);
    case linkPreviewTypeArticle::ID:
      return linkPreviewTypeArticle::fetch(env, p);
    case linkPreviewTypeAudio::ID:
      return linkPreviewTypeAudio::fetch(env, p);
    case linkPreviewTypeBackground::ID:
      return linkPreviewTypeBackground::fetch(env, p);
    case linkPreviewTypeChannelBoost::ID:
      return linkPreviewTypeChannelBoost::fetch(env, p);
    case linkPreviewTypeChat::ID:
      return linkPreviewTypeChat::fetch(env, p);
    case linkPreviewTypeDocument::ID:
      return linkPreviewTypeDocument::fetch(env, p);
    case linkPreviewTypeEmbeddedAnimationPlayer::ID:
      return linkPreviewTypeEmbeddedAnimationPlayer::fetch(env, p);
    case linkPreviewTypeEmbeddedAudioPlayer::ID:
      return linkPreviewTypeEmbeddedAudioPlayer::fetch(env, p);
    case linkPreviewTypeEmbeddedVideoPlayer::ID:
      return linkPreviewTypeEmbeddedVideoPlayer::fetch(env, p);
    case linkPreviewTypeExternalAudio::ID:
      return linkPreviewTypeExternalAudio::fetch(env, p);
    case linkPreviewTypeExternalVideo::ID:
      return linkPreviewTypeExternalVideo::fetch(env, p);
    case linkPreviewTypeGroupCall::ID:
      return linkPreviewTypeGroupCall::fetch(env, p);
    case linkPreviewTypeInvoice::ID:
      return linkPreviewTypeInvoice::fetch(env, p);
    case linkPreviewTypeMessage::ID:
      return linkPreviewTypeMessage::fetch(env, p);
    case linkPreviewTypePhoto::ID:
      return linkPreviewTypePhoto::fetch(env, p);
    case linkPreviewTypePremiumGiftCode::ID:
      return linkPreviewTypePremiumGiftCode::fetch(env, p);
    case linkPreviewTypeShareableChatFolder::ID:
      return linkPreviewTypeShareableChatFolder::fetch(env, p);
    case linkPreviewTypeSticker::ID:
      return linkPreviewTypeSticker::fetch(env, p);
    case linkPreviewTypeStickerSet::ID:
      return linkPreviewTypeStickerSet::fetch(env, p);
    case linkPreviewTypeStory::ID:
      return linkPreviewTypeStory::fetch(env, p);
    case linkPreviewTypeSupergroupBoost::ID:
      return linkPreviewTypeSupergroupBoost::fetch(env, p);
    case linkPreviewTypeTheme::ID:
      return linkPreviewTypeTheme::fetch(env, p);
    case linkPreviewTypeUnsupported::ID:
      return linkPreviewTypeUnsupported::fetch(env, p);
    case linkPreviewTypeUpgradedGift::ID:
      return linkPreviewTypeUpgradedGift::fetch(env, p);
    case linkPreviewTypeUser::ID:
      return linkPreviewTypeUser::fetch(env, p);
    case linkPreviewTypeVideo::ID:
      return linkPreviewTypeVideo::fetch(env, p);
    case linkPreviewTypeVideoChat::ID:
      return linkPreviewTypeVideoChat::fetch(env, p);
    case linkPreviewTypeVideoNote::ID:
      return linkPreviewTypeVideoNote::fetch(env, p);
    case linkPreviewTypeVoiceNote::ID:
      return linkPreviewTypeVoiceNote::fetch(env, p);
    case linkPreviewTypeWebApp::ID:
      return linkPreviewTypeWebApp::fetch(env, p);
    case localFile::ID:
      return localFile::fetch(env, p);
    case localizationTargetInfo::ID:
      return localizationTargetInfo::fetch(env, p);
    case location::ID:
      return location::fetch(env, p);
    case locationAddress::ID:
      return locationAddress::fetch(env, p);
    case logStreamDefault::ID:
      return logStreamDefault::fetch(env, p);
    case logStreamFile::ID:
      return logStreamFile::fetch(env, p);
    case logStreamEmpty::ID:
      return logStreamEmpty::fetch(env, p);
    case logTags::ID:
      return logTags::fetch(env, p);
    case logVerbosityLevel::ID:
      return logVerbosityLevel::fetch(env, p);
    case loginUrlInfoOpen::ID:
      return loginUrlInfoOpen::fetch(env, p);
    case loginUrlInfoRequestConfirmation::ID:
      return loginUrlInfoRequestConfirmation::fetch(env, p);
    case mainWebApp::ID:
      return mainWebApp::fetch(env, p);
    case maskPointForehead::ID:
      return maskPointForehead::fetch(env, p);
    case maskPointEyes::ID:
      return maskPointEyes::fetch(env, p);
    case maskPointMouth::ID:
      return maskPointMouth::fetch(env, p);
    case maskPointChin::ID:
      return maskPointChin::fetch(env, p);
    case maskPosition::ID:
      return maskPosition::fetch(env, p);
    case message::ID:
      return message::fetch(env, p);
    case messageAutoDeleteTime::ID:
      return messageAutoDeleteTime::fetch(env, p);
    case messageCalendar::ID:
      return messageCalendar::fetch(env, p);
    case messageCalendarDay::ID:
      return messageCalendarDay::fetch(env, p);
    case messageText::ID:
      return messageText::fetch(env, p);
    case messageAnimation::ID:
      return messageAnimation::fetch(env, p);
    case messageAudio::ID:
      return messageAudio::fetch(env, p);
    case messageDocument::ID:
      return messageDocument::fetch(env, p);
    case messagePaidMedia::ID:
      return messagePaidMedia::fetch(env, p);
    case messagePhoto::ID:
      return messagePhoto::fetch(env, p);
    case messageSticker::ID:
      return messageSticker::fetch(env, p);
    case messageVideo::ID:
      return messageVideo::fetch(env, p);
    case messageVideoNote::ID:
      return messageVideoNote::fetch(env, p);
    case messageVoiceNote::ID:
      return messageVoiceNote::fetch(env, p);
    case messageExpiredPhoto::ID:
      return messageExpiredPhoto::fetch(env, p);
    case messageExpiredVideo::ID:
      return messageExpiredVideo::fetch(env, p);
    case messageExpiredVideoNote::ID:
      return messageExpiredVideoNote::fetch(env, p);
    case messageExpiredVoiceNote::ID:
      return messageExpiredVoiceNote::fetch(env, p);
    case messageLocation::ID:
      return messageLocation::fetch(env, p);
    case messageVenue::ID:
      return messageVenue::fetch(env, p);
    case messageContact::ID:
      return messageContact::fetch(env, p);
    case messageAnimatedEmoji::ID:
      return messageAnimatedEmoji::fetch(env, p);
    case messageDice::ID:
      return messageDice::fetch(env, p);
    case messageGame::ID:
      return messageGame::fetch(env, p);
    case messagePoll::ID:
      return messagePoll::fetch(env, p);
    case messageStory::ID:
      return messageStory::fetch(env, p);
    case messageInvoice::ID:
      return messageInvoice::fetch(env, p);
    case messageCall::ID:
      return messageCall::fetch(env, p);
    case messageGroupCall::ID:
      return messageGroupCall::fetch(env, p);
    case messageVideoChatScheduled::ID:
      return messageVideoChatScheduled::fetch(env, p);
    case messageVideoChatStarted::ID:
      return messageVideoChatStarted::fetch(env, p);
    case messageVideoChatEnded::ID:
      return messageVideoChatEnded::fetch(env, p);
    case messageInviteVideoChatParticipants::ID:
      return messageInviteVideoChatParticipants::fetch(env, p);
    case messageBasicGroupChatCreate::ID:
      return messageBasicGroupChatCreate::fetch(env, p);
    case messageSupergroupChatCreate::ID:
      return messageSupergroupChatCreate::fetch(env, p);
    case messageChatChangeTitle::ID:
      return messageChatChangeTitle::fetch(env, p);
    case messageChatChangePhoto::ID:
      return messageChatChangePhoto::fetch(env, p);
    case messageChatDeletePhoto::ID:
      return messageChatDeletePhoto::fetch(env, p);
    case messageChatAddMembers::ID:
      return messageChatAddMembers::fetch(env, p);
    case messageChatJoinByLink::ID:
      return messageChatJoinByLink::fetch(env, p);
    case messageChatJoinByRequest::ID:
      return messageChatJoinByRequest::fetch(env, p);
    case messageChatDeleteMember::ID:
      return messageChatDeleteMember::fetch(env, p);
    case messageChatUpgradeTo::ID:
      return messageChatUpgradeTo::fetch(env, p);
    case messageChatUpgradeFrom::ID:
      return messageChatUpgradeFrom::fetch(env, p);
    case messagePinMessage::ID:
      return messagePinMessage::fetch(env, p);
    case messageScreenshotTaken::ID:
      return messageScreenshotTaken::fetch(env, p);
    case messageChatSetBackground::ID:
      return messageChatSetBackground::fetch(env, p);
    case messageChatSetTheme::ID:
      return messageChatSetTheme::fetch(env, p);
    case messageChatSetMessageAutoDeleteTime::ID:
      return messageChatSetMessageAutoDeleteTime::fetch(env, p);
    case messageChatBoost::ID:
      return messageChatBoost::fetch(env, p);
    case messageForumTopicCreated::ID:
      return messageForumTopicCreated::fetch(env, p);
    case messageForumTopicEdited::ID:
      return messageForumTopicEdited::fetch(env, p);
    case messageForumTopicIsClosedToggled::ID:
      return messageForumTopicIsClosedToggled::fetch(env, p);
    case messageForumTopicIsHiddenToggled::ID:
      return messageForumTopicIsHiddenToggled::fetch(env, p);
    case messageSuggestProfilePhoto::ID:
      return messageSuggestProfilePhoto::fetch(env, p);
    case messageCustomServiceAction::ID:
      return messageCustomServiceAction::fetch(env, p);
    case messageGameScore::ID:
      return messageGameScore::fetch(env, p);
    case messagePaymentSuccessful::ID:
      return messagePaymentSuccessful::fetch(env, p);
    case messagePaymentSuccessfulBot::ID:
      return messagePaymentSuccessfulBot::fetch(env, p);
    case messagePaymentRefunded::ID:
      return messagePaymentRefunded::fetch(env, p);
    case messageGiftedPremium::ID:
      return messageGiftedPremium::fetch(env, p);
    case messagePremiumGiftCode::ID:
      return messagePremiumGiftCode::fetch(env, p);
    case messageGiveawayCreated::ID:
      return messageGiveawayCreated::fetch(env, p);
    case messageGiveaway::ID:
      return messageGiveaway::fetch(env, p);
    case messageGiveawayCompleted::ID:
      return messageGiveawayCompleted::fetch(env, p);
    case messageGiveawayWinners::ID:
      return messageGiveawayWinners::fetch(env, p);
    case messageGiftedStars::ID:
      return messageGiftedStars::fetch(env, p);
    case messageGiveawayPrizeStars::ID:
      return messageGiveawayPrizeStars::fetch(env, p);
    case messageGift::ID:
      return messageGift::fetch(env, p);
    case messageUpgradedGift::ID:
      return messageUpgradedGift::fetch(env, p);
    case messageRefundedUpgradedGift::ID:
      return messageRefundedUpgradedGift::fetch(env, p);
    case messagePaidMessagesRefunded::ID:
      return messagePaidMessagesRefunded::fetch(env, p);
    case messagePaidMessagePriceChanged::ID:
      return messagePaidMessagePriceChanged::fetch(env, p);
    case messageContactRegistered::ID:
      return messageContactRegistered::fetch(env, p);
    case messageUsersShared::ID:
      return messageUsersShared::fetch(env, p);
    case messageChatShared::ID:
      return messageChatShared::fetch(env, p);
    case messageBotWriteAccessAllowed::ID:
      return messageBotWriteAccessAllowed::fetch(env, p);
    case messageWebAppDataSent::ID:
      return messageWebAppDataSent::fetch(env, p);
    case messageWebAppDataReceived::ID:
      return messageWebAppDataReceived::fetch(env, p);
    case messagePassportDataSent::ID:
      return messagePassportDataSent::fetch(env, p);
    case messagePassportDataReceived::ID:
      return messagePassportDataReceived::fetch(env, p);
    case messageProximityAlertTriggered::ID:
      return messageProximityAlertTriggered::fetch(env, p);
    case messageUnsupported::ID:
      return messageUnsupported::fetch(env, p);
    case messageCopyOptions::ID:
      return messageCopyOptions::fetch(env, p);
    case messageEffect::ID:
      return messageEffect::fetch(env, p);
    case messageEffectTypeEmojiReaction::ID:
      return messageEffectTypeEmojiReaction::fetch(env, p);
    case messageEffectTypePremiumSticker::ID:
      return messageEffectTypePremiumSticker::fetch(env, p);
    case messageFileTypePrivate::ID:
      return messageFileTypePrivate::fetch(env, p);
    case messageFileTypeGroup::ID:
      return messageFileTypeGroup::fetch(env, p);
    case messageFileTypeUnknown::ID:
      return messageFileTypeUnknown::fetch(env, p);
    case messageForwardInfo::ID:
      return messageForwardInfo::fetch(env, p);
    case messageImportInfo::ID:
      return messageImportInfo::fetch(env, p);
    case messageInteractionInfo::ID:
      return messageInteractionInfo::fetch(env, p);
    case messageLink::ID:
      return messageLink::fetch(env, p);
    case messageLinkInfo::ID:
      return messageLinkInfo::fetch(env, p);
    case messageOriginUser::ID:
      return messageOriginUser::fetch(env, p);
    case messageOriginHiddenUser::ID:
      return messageOriginHiddenUser::fetch(env, p);
    case messageOriginChat::ID:
      return messageOriginChat::fetch(env, p);
    case messageOriginChannel::ID:
      return messageOriginChannel::fetch(env, p);
    case messagePosition::ID:
      return messagePosition::fetch(env, p);
    case messagePositions::ID:
      return messagePositions::fetch(env, p);
    case messageProperties::ID:
      return messageProperties::fetch(env, p);
    case messageReaction::ID:
      return messageReaction::fetch(env, p);
    case messageReactions::ID:
      return messageReactions::fetch(env, p);
    case messageReadDateRead::ID:
      return messageReadDateRead::fetch(env, p);
    case messageReadDateUnread::ID:
      return messageReadDateUnread::fetch(env, p);
    case messageReadDateTooOld::ID:
      return messageReadDateTooOld::fetch(env, p);
    case messageReadDateUserPrivacyRestricted::ID:
      return messageReadDateUserPrivacyRestricted::fetch(env, p);
    case messageReadDateMyPrivacyRestricted::ID:
      return messageReadDateMyPrivacyRestricted::fetch(env, p);
    case messageReplyInfo::ID:
      return messageReplyInfo::fetch(env, p);
    case messageReplyToMessage::ID:
      return messageReplyToMessage::fetch(env, p);
    case messageReplyToStory::ID:
      return messageReplyToStory::fetch(env, p);
    case messageSchedulingStateSendAtDate::ID:
      return messageSchedulingStateSendAtDate::fetch(env, p);
    case messageSchedulingStateSendWhenOnline::ID:
      return messageSchedulingStateSendWhenOnline::fetch(env, p);
    case messageSchedulingStateSendWhenVideoProcessed::ID:
      return messageSchedulingStateSendWhenVideoProcessed::fetch(env, p);
    case messageSelfDestructTypeTimer::ID:
      return messageSelfDestructTypeTimer::fetch(env, p);
    case messageSelfDestructTypeImmediately::ID:
      return messageSelfDestructTypeImmediately::fetch(env, p);
    case messageSendOptions::ID:
      return messageSendOptions::fetch(env, p);
    case messageSenderUser::ID:
      return messageSenderUser::fetch(env, p);
    case messageSenderChat::ID:
      return messageSenderChat::fetch(env, p);
    case messageSenders::ID:
      return messageSenders::fetch(env, p);
    case messageSendingStatePending::ID:
      return messageSendingStatePending::fetch(env, p);
    case messageSendingStateFailed::ID:
      return messageSendingStateFailed::fetch(env, p);
    case messageSourceChatHistory::ID:
      return messageSourceChatHistory::fetch(env, p);
    case messageSourceMessageThreadHistory::ID:
      return messageSourceMessageThreadHistory::fetch(env, p);
    case messageSourceForumTopicHistory::ID:
      return messageSourceForumTopicHistory::fetch(env, p);
    case messageSourceHistoryPreview::ID:
      return messageSourceHistoryPreview::fetch(env, p);
    case messageSourceChatList::ID:
      return messageSourceChatList::fetch(env, p);
    case messageSourceSearch::ID:
      return messageSourceSearch::fetch(env, p);
    case messageSourceChatEventLog::ID:
      return messageSourceChatEventLog::fetch(env, p);
    case messageSourceNotification::ID:
      return messageSourceNotification::fetch(env, p);
    case messageSourceScreenshot::ID:
      return messageSourceScreenshot::fetch(env, p);
    case messageSourceOther::ID:
      return messageSourceOther::fetch(env, p);
    case messageSponsor::ID:
      return messageSponsor::fetch(env, p);
    case messageStatistics::ID:
      return messageStatistics::fetch(env, p);
    case messageThreadInfo::ID:
      return messageThreadInfo::fetch(env, p);
    case messageViewer::ID:
      return messageViewer::fetch(env, p);
    case messageViewers::ID:
      return messageViewers::fetch(env, p);
    case messages::ID:
      return messages::fetch(env, p);
    case minithumbnail::ID:
      return minithumbnail::fetch(env, p);
    case networkStatistics::ID:
      return networkStatistics::fetch(env, p);
    case networkStatisticsEntryFile::ID:
      return networkStatisticsEntryFile::fetch(env, p);
    case networkStatisticsEntryCall::ID:
      return networkStatisticsEntryCall::fetch(env, p);
    case networkTypeNone::ID:
      return networkTypeNone::fetch(env, p);
    case networkTypeMobile::ID:
      return networkTypeMobile::fetch(env, p);
    case networkTypeMobileRoaming::ID:
      return networkTypeMobileRoaming::fetch(env, p);
    case networkTypeWiFi::ID:
      return networkTypeWiFi::fetch(env, p);
    case networkTypeOther::ID:
      return networkTypeOther::fetch(env, p);
    case newChatPrivacySettings::ID:
      return newChatPrivacySettings::fetch(env, p);
    case notification::ID:
      return notification::fetch(env, p);
    case notificationGroup::ID:
      return notificationGroup::fetch(env, p);
    case notificationGroupTypeMessages::ID:
      return notificationGroupTypeMessages::fetch(env, p);
    case notificationGroupTypeMentions::ID:
      return notificationGroupTypeMentions::fetch(env, p);
    case notificationGroupTypeSecretChat::ID:
      return notificationGroupTypeSecretChat::fetch(env, p);
    case notificationGroupTypeCalls::ID:
      return notificationGroupTypeCalls::fetch(env, p);
    case notificationSettingsScopePrivateChats::ID:
      return notificationSettingsScopePrivateChats::fetch(env, p);
    case notificationSettingsScopeGroupChats::ID:
      return notificationSettingsScopeGroupChats::fetch(env, p);
    case notificationSettingsScopeChannelChats::ID:
      return notificationSettingsScopeChannelChats::fetch(env, p);
    case notificationSound::ID:
      return notificationSound::fetch(env, p);
    case notificationSounds::ID:
      return notificationSounds::fetch(env, p);
    case notificationTypeNewMessage::ID:
      return notificationTypeNewMessage::fetch(env, p);
    case notificationTypeNewSecretChat::ID:
      return notificationTypeNewSecretChat::fetch(env, p);
    case notificationTypeNewCall::ID:
      return notificationTypeNewCall::fetch(env, p);
    case notificationTypeNewPushMessage::ID:
      return notificationTypeNewPushMessage::fetch(env, p);
    case ok::ID:
      return ok::fetch(env, p);
    case optionValueBoolean::ID:
      return optionValueBoolean::fetch(env, p);
    case optionValueEmpty::ID:
      return optionValueEmpty::fetch(env, p);
    case optionValueInteger::ID:
      return optionValueInteger::fetch(env, p);
    case optionValueString::ID:
      return optionValueString::fetch(env, p);
    case orderInfo::ID:
      return orderInfo::fetch(env, p);
    case outline::ID:
      return outline::fetch(env, p);
    case pageBlockTitle::ID:
      return pageBlockTitle::fetch(env, p);
    case pageBlockSubtitle::ID:
      return pageBlockSubtitle::fetch(env, p);
    case pageBlockAuthorDate::ID:
      return pageBlockAuthorDate::fetch(env, p);
    case pageBlockHeader::ID:
      return pageBlockHeader::fetch(env, p);
    case pageBlockSubheader::ID:
      return pageBlockSubheader::fetch(env, p);
    case pageBlockKicker::ID:
      return pageBlockKicker::fetch(env, p);
    case pageBlockParagraph::ID:
      return pageBlockParagraph::fetch(env, p);
    case pageBlockPreformatted::ID:
      return pageBlockPreformatted::fetch(env, p);
    case pageBlockFooter::ID:
      return pageBlockFooter::fetch(env, p);
    case pageBlockDivider::ID:
      return pageBlockDivider::fetch(env, p);
    case pageBlockAnchor::ID:
      return pageBlockAnchor::fetch(env, p);
    case pageBlockList::ID:
      return pageBlockList::fetch(env, p);
    case pageBlockBlockQuote::ID:
      return pageBlockBlockQuote::fetch(env, p);
    case pageBlockPullQuote::ID:
      return pageBlockPullQuote::fetch(env, p);
    case pageBlockAnimation::ID:
      return pageBlockAnimation::fetch(env, p);
    case pageBlockAudio::ID:
      return pageBlockAudio::fetch(env, p);
    case pageBlockPhoto::ID:
      return pageBlockPhoto::fetch(env, p);
    case pageBlockVideo::ID:
      return pageBlockVideo::fetch(env, p);
    case pageBlockVoiceNote::ID:
      return pageBlockVoiceNote::fetch(env, p);
    case pageBlockCover::ID:
      return pageBlockCover::fetch(env, p);
    case pageBlockEmbedded::ID:
      return pageBlockEmbedded::fetch(env, p);
    case pageBlockEmbeddedPost::ID:
      return pageBlockEmbeddedPost::fetch(env, p);
    case pageBlockCollage::ID:
      return pageBlockCollage::fetch(env, p);
    case pageBlockSlideshow::ID:
      return pageBlockSlideshow::fetch(env, p);
    case pageBlockChatLink::ID:
      return pageBlockChatLink::fetch(env, p);
    case pageBlockTable::ID:
      return pageBlockTable::fetch(env, p);
    case pageBlockDetails::ID:
      return pageBlockDetails::fetch(env, p);
    case pageBlockRelatedArticles::ID:
      return pageBlockRelatedArticles::fetch(env, p);
    case pageBlockMap::ID:
      return pageBlockMap::fetch(env, p);
    case pageBlockCaption::ID:
      return pageBlockCaption::fetch(env, p);
    case pageBlockHorizontalAlignmentLeft::ID:
      return pageBlockHorizontalAlignmentLeft::fetch(env, p);
    case pageBlockHorizontalAlignmentCenter::ID:
      return pageBlockHorizontalAlignmentCenter::fetch(env, p);
    case pageBlockHorizontalAlignmentRight::ID:
      return pageBlockHorizontalAlignmentRight::fetch(env, p);
    case pageBlockListItem::ID:
      return pageBlockListItem::fetch(env, p);
    case pageBlockRelatedArticle::ID:
      return pageBlockRelatedArticle::fetch(env, p);
    case pageBlockTableCell::ID:
      return pageBlockTableCell::fetch(env, p);
    case pageBlockVerticalAlignmentTop::ID:
      return pageBlockVerticalAlignmentTop::fetch(env, p);
    case pageBlockVerticalAlignmentMiddle::ID:
      return pageBlockVerticalAlignmentMiddle::fetch(env, p);
    case pageBlockVerticalAlignmentBottom::ID:
      return pageBlockVerticalAlignmentBottom::fetch(env, p);
    case paidMediaPreview::ID:
      return paidMediaPreview::fetch(env, p);
    case paidMediaPhoto::ID:
      return paidMediaPhoto::fetch(env, p);
    case paidMediaVideo::ID:
      return paidMediaVideo::fetch(env, p);
    case paidMediaUnsupported::ID:
      return paidMediaUnsupported::fetch(env, p);
    case paidReactionTypeRegular::ID:
      return paidReactionTypeRegular::fetch(env, p);
    case paidReactionTypeAnonymous::ID:
      return paidReactionTypeAnonymous::fetch(env, p);
    case paidReactionTypeChat::ID:
      return paidReactionTypeChat::fetch(env, p);
    case paidReactor::ID:
      return paidReactor::fetch(env, p);
    case passportAuthorizationForm::ID:
      return passportAuthorizationForm::fetch(env, p);
    case passportElementPersonalDetails::ID:
      return passportElementPersonalDetails::fetch(env, p);
    case passportElementPassport::ID:
      return passportElementPassport::fetch(env, p);
    case passportElementDriverLicense::ID:
      return passportElementDriverLicense::fetch(env, p);
    case passportElementIdentityCard::ID:
      return passportElementIdentityCard::fetch(env, p);
    case passportElementInternalPassport::ID:
      return passportElementInternalPassport::fetch(env, p);
    case passportElementAddress::ID:
      return passportElementAddress::fetch(env, p);
    case passportElementUtilityBill::ID:
      return passportElementUtilityBill::fetch(env, p);
    case passportElementBankStatement::ID:
      return passportElementBankStatement::fetch(env, p);
    case passportElementRentalAgreement::ID:
      return passportElementRentalAgreement::fetch(env, p);
    case passportElementPassportRegistration::ID:
      return passportElementPassportRegistration::fetch(env, p);
    case passportElementTemporaryRegistration::ID:
      return passportElementTemporaryRegistration::fetch(env, p);
    case passportElementPhoneNumber::ID:
      return passportElementPhoneNumber::fetch(env, p);
    case passportElementEmailAddress::ID:
      return passportElementEmailAddress::fetch(env, p);
    case passportElementError::ID:
      return passportElementError::fetch(env, p);
    case passportElementErrorSourceUnspecified::ID:
      return passportElementErrorSourceUnspecified::fetch(env, p);
    case passportElementErrorSourceDataField::ID:
      return passportElementErrorSourceDataField::fetch(env, p);
    case passportElementErrorSourceFrontSide::ID:
      return passportElementErrorSourceFrontSide::fetch(env, p);
    case passportElementErrorSourceReverseSide::ID:
      return passportElementErrorSourceReverseSide::fetch(env, p);
    case passportElementErrorSourceSelfie::ID:
      return passportElementErrorSourceSelfie::fetch(env, p);
    case passportElementErrorSourceTranslationFile::ID:
      return passportElementErrorSourceTranslationFile::fetch(env, p);
    case passportElementErrorSourceTranslationFiles::ID:
      return passportElementErrorSourceTranslationFiles::fetch(env, p);
    case passportElementErrorSourceFile::ID:
      return passportElementErrorSourceFile::fetch(env, p);
    case passportElementErrorSourceFiles::ID:
      return passportElementErrorSourceFiles::fetch(env, p);
    case passportElementTypePersonalDetails::ID:
      return passportElementTypePersonalDetails::fetch(env, p);
    case passportElementTypePassport::ID:
      return passportElementTypePassport::fetch(env, p);
    case passportElementTypeDriverLicense::ID:
      return passportElementTypeDriverLicense::fetch(env, p);
    case passportElementTypeIdentityCard::ID:
      return passportElementTypeIdentityCard::fetch(env, p);
    case passportElementTypeInternalPassport::ID:
      return passportElementTypeInternalPassport::fetch(env, p);
    case passportElementTypeAddress::ID:
      return passportElementTypeAddress::fetch(env, p);
    case passportElementTypeUtilityBill::ID:
      return passportElementTypeUtilityBill::fetch(env, p);
    case passportElementTypeBankStatement::ID:
      return passportElementTypeBankStatement::fetch(env, p);
    case passportElementTypeRentalAgreement::ID:
      return passportElementTypeRentalAgreement::fetch(env, p);
    case passportElementTypePassportRegistration::ID:
      return passportElementTypePassportRegistration::fetch(env, p);
    case passportElementTypeTemporaryRegistration::ID:
      return passportElementTypeTemporaryRegistration::fetch(env, p);
    case passportElementTypePhoneNumber::ID:
      return passportElementTypePhoneNumber::fetch(env, p);
    case passportElementTypeEmailAddress::ID:
      return passportElementTypeEmailAddress::fetch(env, p);
    case passportElements::ID:
      return passportElements::fetch(env, p);
    case passportElementsWithErrors::ID:
      return passportElementsWithErrors::fetch(env, p);
    case passportRequiredElement::ID:
      return passportRequiredElement::fetch(env, p);
    case passportSuitableElement::ID:
      return passportSuitableElement::fetch(env, p);
    case passwordState::ID:
      return passwordState::fetch(env, p);
    case paymentForm::ID:
      return paymentForm::fetch(env, p);
    case paymentFormTypeRegular::ID:
      return paymentFormTypeRegular::fetch(env, p);
    case paymentFormTypeStars::ID:
      return paymentFormTypeStars::fetch(env, p);
    case paymentFormTypeStarSubscription::ID:
      return paymentFormTypeStarSubscription::fetch(env, p);
    case paymentOption::ID:
      return paymentOption::fetch(env, p);
    case paymentProviderSmartGlocal::ID:
      return paymentProviderSmartGlocal::fetch(env, p);
    case paymentProviderStripe::ID:
      return paymentProviderStripe::fetch(env, p);
    case paymentProviderOther::ID:
      return paymentProviderOther::fetch(env, p);
    case paymentReceipt::ID:
      return paymentReceipt::fetch(env, p);
    case paymentReceiptTypeRegular::ID:
      return paymentReceiptTypeRegular::fetch(env, p);
    case paymentReceiptTypeStars::ID:
      return paymentReceiptTypeStars::fetch(env, p);
    case paymentResult::ID:
      return paymentResult::fetch(env, p);
    case personalDetails::ID:
      return personalDetails::fetch(env, p);
    case personalDocument::ID:
      return personalDocument::fetch(env, p);
    case phoneNumberAuthenticationSettings::ID:
      return phoneNumberAuthenticationSettings::fetch(env, p);
    case phoneNumberCodeTypeChange::ID:
      return phoneNumberCodeTypeChange::fetch(env, p);
    case phoneNumberCodeTypeVerify::ID:
      return phoneNumberCodeTypeVerify::fetch(env, p);
    case phoneNumberCodeTypeConfirmOwnership::ID:
      return phoneNumberCodeTypeConfirmOwnership::fetch(env, p);
    case phoneNumberInfo::ID:
      return phoneNumberInfo::fetch(env, p);
    case photo::ID:
      return photo::fetch(env, p);
    case photoSize::ID:
      return photoSize::fetch(env, p);
    case point::ID:
      return point::fetch(env, p);
    case poll::ID:
      return poll::fetch(env, p);
    case pollOption::ID:
      return pollOption::fetch(env, p);
    case pollTypeRegular::ID:
      return pollTypeRegular::fetch(env, p);
    case pollTypeQuiz::ID:
      return pollTypeQuiz::fetch(env, p);
    case premiumFeatureIncreasedLimits::ID:
      return premiumFeatureIncreasedLimits::fetch(env, p);
    case premiumFeatureIncreasedUploadFileSize::ID:
      return premiumFeatureIncreasedUploadFileSize::fetch(env, p);
    case premiumFeatureImprovedDownloadSpeed::ID:
      return premiumFeatureImprovedDownloadSpeed::fetch(env, p);
    case premiumFeatureVoiceRecognition::ID:
      return premiumFeatureVoiceRecognition::fetch(env, p);
    case premiumFeatureDisabledAds::ID:
      return premiumFeatureDisabledAds::fetch(env, p);
    case premiumFeatureUniqueReactions::ID:
      return premiumFeatureUniqueReactions::fetch(env, p);
    case premiumFeatureUniqueStickers::ID:
      return premiumFeatureUniqueStickers::fetch(env, p);
    case premiumFeatureCustomEmoji::ID:
      return premiumFeatureCustomEmoji::fetch(env, p);
    case premiumFeatureAdvancedChatManagement::ID:
      return premiumFeatureAdvancedChatManagement::fetch(env, p);
    case premiumFeatureProfileBadge::ID:
      return premiumFeatureProfileBadge::fetch(env, p);
    case premiumFeatureEmojiStatus::ID:
      return premiumFeatureEmojiStatus::fetch(env, p);
    case premiumFeatureAnimatedProfilePhoto::ID:
      return premiumFeatureAnimatedProfilePhoto::fetch(env, p);
    case premiumFeatureForumTopicIcon::ID:
      return premiumFeatureForumTopicIcon::fetch(env, p);
    case premiumFeatureAppIcons::ID:
      return premiumFeatureAppIcons::fetch(env, p);
    case premiumFeatureRealTimeChatTranslation::ID:
      return premiumFeatureRealTimeChatTranslation::fetch(env, p);
    case premiumFeatureUpgradedStories::ID:
      return premiumFeatureUpgradedStories::fetch(env, p);
    case premiumFeatureChatBoost::ID:
      return premiumFeatureChatBoost::fetch(env, p);
    case premiumFeatureAccentColor::ID:
      return premiumFeatureAccentColor::fetch(env, p);
    case premiumFeatureBackgroundForBoth::ID:
      return premiumFeatureBackgroundForBoth::fetch(env, p);
    case premiumFeatureSavedMessagesTags::ID:
      return premiumFeatureSavedMessagesTags::fetch(env, p);
    case premiumFeatureMessagePrivacy::ID:
      return premiumFeatureMessagePrivacy::fetch(env, p);
    case premiumFeatureLastSeenTimes::ID:
      return premiumFeatureLastSeenTimes::fetch(env, p);
    case premiumFeatureBusiness::ID:
      return premiumFeatureBusiness::fetch(env, p);
    case premiumFeatureMessageEffects::ID:
      return premiumFeatureMessageEffects::fetch(env, p);
    case premiumFeaturePromotionAnimation::ID:
      return premiumFeaturePromotionAnimation::fetch(env, p);
    case premiumFeatures::ID:
      return premiumFeatures::fetch(env, p);
    case premiumGiftCodeInfo::ID:
      return premiumGiftCodeInfo::fetch(env, p);
    case premiumGiftPaymentOption::ID:
      return premiumGiftPaymentOption::fetch(env, p);
    case premiumGiftPaymentOptions::ID:
      return premiumGiftPaymentOptions::fetch(env, p);
    case premiumGiveawayPaymentOption::ID:
      return premiumGiveawayPaymentOption::fetch(env, p);
    case premiumGiveawayPaymentOptions::ID:
      return premiumGiveawayPaymentOptions::fetch(env, p);
    case premiumLimit::ID:
      return premiumLimit::fetch(env, p);
    case premiumLimitTypeSupergroupCount::ID:
      return premiumLimitTypeSupergroupCount::fetch(env, p);
    case premiumLimitTypePinnedChatCount::ID:
      return premiumLimitTypePinnedChatCount::fetch(env, p);
    case premiumLimitTypeCreatedPublicChatCount::ID:
      return premiumLimitTypeCreatedPublicChatCount::fetch(env, p);
    case premiumLimitTypeSavedAnimationCount::ID:
      return premiumLimitTypeSavedAnimationCount::fetch(env, p);
    case premiumLimitTypeFavoriteStickerCount::ID:
      return premiumLimitTypeFavoriteStickerCount::fetch(env, p);
    case premiumLimitTypeChatFolderCount::ID:
      return premiumLimitTypeChatFolderCount::fetch(env, p);
    case premiumLimitTypeChatFolderChosenChatCount::ID:
      return premiumLimitTypeChatFolderChosenChatCount::fetch(env, p);
    case premiumLimitTypePinnedArchivedChatCount::ID:
      return premiumLimitTypePinnedArchivedChatCount::fetch(env, p);
    case premiumLimitTypePinnedSavedMessagesTopicCount::ID:
      return premiumLimitTypePinnedSavedMessagesTopicCount::fetch(env, p);
    case premiumLimitTypeCaptionLength::ID:
      return premiumLimitTypeCaptionLength::fetch(env, p);
    case premiumLimitTypeBioLength::ID:
      return premiumLimitTypeBioLength::fetch(env, p);
    case premiumLimitTypeChatFolderInviteLinkCount::ID:
      return premiumLimitTypeChatFolderInviteLinkCount::fetch(env, p);
    case premiumLimitTypeShareableChatFolderCount::ID:
      return premiumLimitTypeShareableChatFolderCount::fetch(env, p);
    case premiumLimitTypeActiveStoryCount::ID:
      return premiumLimitTypeActiveStoryCount::fetch(env, p);
    case premiumLimitTypeWeeklyPostedStoryCount::ID:
      return premiumLimitTypeWeeklyPostedStoryCount::fetch(env, p);
    case premiumLimitTypeMonthlyPostedStoryCount::ID:
      return premiumLimitTypeMonthlyPostedStoryCount::fetch(env, p);
    case premiumLimitTypeStoryCaptionLength::ID:
      return premiumLimitTypeStoryCaptionLength::fetch(env, p);
    case premiumLimitTypeStorySuggestedReactionAreaCount::ID:
      return premiumLimitTypeStorySuggestedReactionAreaCount::fetch(env, p);
    case premiumLimitTypeSimilarChatCount::ID:
      return premiumLimitTypeSimilarChatCount::fetch(env, p);
    case premiumPaymentOption::ID:
      return premiumPaymentOption::fetch(env, p);
    case premiumSourceLimitExceeded::ID:
      return premiumSourceLimitExceeded::fetch(env, p);
    case premiumSourceFeature::ID:
      return premiumSourceFeature::fetch(env, p);
    case premiumSourceBusinessFeature::ID:
      return premiumSourceBusinessFeature::fetch(env, p);
    case premiumSourceStoryFeature::ID:
      return premiumSourceStoryFeature::fetch(env, p);
    case premiumSourceLink::ID:
      return premiumSourceLink::fetch(env, p);
    case premiumSourceSettings::ID:
      return premiumSourceSettings::fetch(env, p);
    case premiumState::ID:
      return premiumState::fetch(env, p);
    case premiumStatePaymentOption::ID:
      return premiumStatePaymentOption::fetch(env, p);
    case premiumStoryFeaturePriorityOrder::ID:
      return premiumStoryFeaturePriorityOrder::fetch(env, p);
    case premiumStoryFeatureStealthMode::ID:
      return premiumStoryFeatureStealthMode::fetch(env, p);
    case premiumStoryFeaturePermanentViewsHistory::ID:
      return premiumStoryFeaturePermanentViewsHistory::fetch(env, p);
    case premiumStoryFeatureCustomExpirationDuration::ID:
      return premiumStoryFeatureCustomExpirationDuration::fetch(env, p);
    case premiumStoryFeatureSaveStories::ID:
      return premiumStoryFeatureSaveStories::fetch(env, p);
    case premiumStoryFeatureLinksAndFormatting::ID:
      return premiumStoryFeatureLinksAndFormatting::fetch(env, p);
    case premiumStoryFeatureVideoQuality::ID:
      return premiumStoryFeatureVideoQuality::fetch(env, p);
    case prepaidGiveaway::ID:
      return prepaidGiveaway::fetch(env, p);
    case preparedInlineMessage::ID:
      return preparedInlineMessage::fetch(env, p);
    case preparedInlineMessageId::ID:
      return preparedInlineMessageId::fetch(env, p);
    case productInfo::ID:
      return productInfo::fetch(env, p);
    case profileAccentColor::ID:
      return profileAccentColor::fetch(env, p);
    case profileAccentColors::ID:
      return profileAccentColors::fetch(env, p);
    case profilePhoto::ID:
      return profilePhoto::fetch(env, p);
    case proxies::ID:
      return proxies::fetch(env, p);
    case proxy::ID:
      return proxy::fetch(env, p);
    case proxyTypeSocks5::ID:
      return proxyTypeSocks5::fetch(env, p);
    case proxyTypeHttp::ID:
      return proxyTypeHttp::fetch(env, p);
    case proxyTypeMtproto::ID:
      return proxyTypeMtproto::fetch(env, p);
    case publicChatTypeHasUsername::ID:
      return publicChatTypeHasUsername::fetch(env, p);
    case publicChatTypeIsLocationBased::ID:
      return publicChatTypeIsLocationBased::fetch(env, p);
    case publicForwardMessage::ID:
      return publicForwardMessage::fetch(env, p);
    case publicForwardStory::ID:
      return publicForwardStory::fetch(env, p);
    case publicForwards::ID:
      return publicForwards::fetch(env, p);
    case pushMessageContentHidden::ID:
      return pushMessageContentHidden::fetch(env, p);
    case pushMessageContentAnimation::ID:
      return pushMessageContentAnimation::fetch(env, p);
    case pushMessageContentAudio::ID:
      return pushMessageContentAudio::fetch(env, p);
    case pushMessageContentContact::ID:
      return pushMessageContentContact::fetch(env, p);
    case pushMessageContentContactRegistered::ID:
      return pushMessageContentContactRegistered::fetch(env, p);
    case pushMessageContentDocument::ID:
      return pushMessageContentDocument::fetch(env, p);
    case pushMessageContentGame::ID:
      return pushMessageContentGame::fetch(env, p);
    case pushMessageContentGameScore::ID:
      return pushMessageContentGameScore::fetch(env, p);
    case pushMessageContentInvoice::ID:
      return pushMessageContentInvoice::fetch(env, p);
    case pushMessageContentLocation::ID:
      return pushMessageContentLocation::fetch(env, p);
    case pushMessageContentPaidMedia::ID:
      return pushMessageContentPaidMedia::fetch(env, p);
    case pushMessageContentPhoto::ID:
      return pushMessageContentPhoto::fetch(env, p);
    case pushMessageContentPoll::ID:
      return pushMessageContentPoll::fetch(env, p);
    case pushMessageContentPremiumGiftCode::ID:
      return pushMessageContentPremiumGiftCode::fetch(env, p);
    case pushMessageContentGiveaway::ID:
      return pushMessageContentGiveaway::fetch(env, p);
    case pushMessageContentGift::ID:
      return pushMessageContentGift::fetch(env, p);
    case pushMessageContentUpgradedGift::ID:
      return pushMessageContentUpgradedGift::fetch(env, p);
    case pushMessageContentScreenshotTaken::ID:
      return pushMessageContentScreenshotTaken::fetch(env, p);
    case pushMessageContentSticker::ID:
      return pushMessageContentSticker::fetch(env, p);
    case pushMessageContentStory::ID:
      return pushMessageContentStory::fetch(env, p);
    case pushMessageContentText::ID:
      return pushMessageContentText::fetch(env, p);
    case pushMessageContentVideo::ID:
      return pushMessageContentVideo::fetch(env, p);
    case pushMessageContentVideoNote::ID:
      return pushMessageContentVideoNote::fetch(env, p);
    case pushMessageContentVoiceNote::ID:
      return pushMessageContentVoiceNote::fetch(env, p);
    case pushMessageContentBasicGroupChatCreate::ID:
      return pushMessageContentBasicGroupChatCreate::fetch(env, p);
    case pushMessageContentVideoChatStarted::ID:
      return pushMessageContentVideoChatStarted::fetch(env, p);
    case pushMessageContentVideoChatEnded::ID:
      return pushMessageContentVideoChatEnded::fetch(env, p);
    case pushMessageContentInviteVideoChatParticipants::ID:
      return pushMessageContentInviteVideoChatParticipants::fetch(env, p);
    case pushMessageContentChatAddMembers::ID:
      return pushMessageContentChatAddMembers::fetch(env, p);
    case pushMessageContentChatChangePhoto::ID:
      return pushMessageContentChatChangePhoto::fetch(env, p);
    case pushMessageContentChatChangeTitle::ID:
      return pushMessageContentChatChangeTitle::fetch(env, p);
    case pushMessageContentChatSetBackground::ID:
      return pushMessageContentChatSetBackground::fetch(env, p);
    case pushMessageContentChatSetTheme::ID:
      return pushMessageContentChatSetTheme::fetch(env, p);
    case pushMessageContentChatDeleteMember::ID:
      return pushMessageContentChatDeleteMember::fetch(env, p);
    case pushMessageContentChatJoinByLink::ID:
      return pushMessageContentChatJoinByLink::fetch(env, p);
    case pushMessageContentChatJoinByRequest::ID:
      return pushMessageContentChatJoinByRequest::fetch(env, p);
    case pushMessageContentRecurringPayment::ID:
      return pushMessageContentRecurringPayment::fetch(env, p);
    case pushMessageContentSuggestProfilePhoto::ID:
      return pushMessageContentSuggestProfilePhoto::fetch(env, p);
    case pushMessageContentProximityAlertTriggered::ID:
      return pushMessageContentProximityAlertTriggered::fetch(env, p);
    case pushMessageContentMessageForwards::ID:
      return pushMessageContentMessageForwards::fetch(env, p);
    case pushMessageContentMediaAlbum::ID:
      return pushMessageContentMediaAlbum::fetch(env, p);
    case pushReceiverId::ID:
      return pushReceiverId::fetch(env, p);
    case quickReplyMessage::ID:
      return quickReplyMessage::fetch(env, p);
    case quickReplyMessages::ID:
      return quickReplyMessages::fetch(env, p);
    case quickReplyShortcut::ID:
      return quickReplyShortcut::fetch(env, p);
    case reactionNotificationSettings::ID:
      return reactionNotificationSettings::fetch(env, p);
    case reactionNotificationSourceNone::ID:
      return reactionNotificationSourceNone::fetch(env, p);
    case reactionNotificationSourceContacts::ID:
      return reactionNotificationSourceContacts::fetch(env, p);
    case reactionNotificationSourceAll::ID:
      return reactionNotificationSourceAll::fetch(env, p);
    case reactionTypeEmoji::ID:
      return reactionTypeEmoji::fetch(env, p);
    case reactionTypeCustomEmoji::ID:
      return reactionTypeCustomEmoji::fetch(env, p);
    case reactionTypePaid::ID:
      return reactionTypePaid::fetch(env, p);
    case reactionUnavailabilityReasonAnonymousAdministrator::ID:
      return reactionUnavailabilityReasonAnonymousAdministrator::fetch(env, p);
    case reactionUnavailabilityReasonGuest::ID:
      return reactionUnavailabilityReasonGuest::fetch(env, p);
    case readDatePrivacySettings::ID:
      return readDatePrivacySettings::fetch(env, p);
    case receivedGift::ID:
      return receivedGift::fetch(env, p);
    case receivedGifts::ID:
      return receivedGifts::fetch(env, p);
    case recommendedChatFolder::ID:
      return recommendedChatFolder::fetch(env, p);
    case recommendedChatFolders::ID:
      return recommendedChatFolders::fetch(env, p);
    case recoveryEmailAddress::ID:
      return recoveryEmailAddress::fetch(env, p);
    case remoteFile::ID:
      return remoteFile::fetch(env, p);
    case replyMarkupRemoveKeyboard::ID:
      return replyMarkupRemoveKeyboard::fetch(env, p);
    case replyMarkupForceReply::ID:
      return replyMarkupForceReply::fetch(env, p);
    case replyMarkupShowKeyboard::ID:
      return replyMarkupShowKeyboard::fetch(env, p);
    case replyMarkupInlineKeyboard::ID:
      return replyMarkupInlineKeyboard::fetch(env, p);
    case reportChatResultOk::ID:
      return reportChatResultOk::fetch(env, p);
    case reportChatResultOptionRequired::ID:
      return reportChatResultOptionRequired::fetch(env, p);
    case reportChatResultTextRequired::ID:
      return reportChatResultTextRequired::fetch(env, p);
    case reportChatResultMessagesRequired::ID:
      return reportChatResultMessagesRequired::fetch(env, p);
    case reportOption::ID:
      return reportOption::fetch(env, p);
    case reportReasonSpam::ID:
      return reportReasonSpam::fetch(env, p);
    case reportReasonViolence::ID:
      return reportReasonViolence::fetch(env, p);
    case reportReasonPornography::ID:
      return reportReasonPornography::fetch(env, p);
    case reportReasonChildAbuse::ID:
      return reportReasonChildAbuse::fetch(env, p);
    case reportReasonCopyright::ID:
      return reportReasonCopyright::fetch(env, p);
    case reportReasonUnrelatedLocation::ID:
      return reportReasonUnrelatedLocation::fetch(env, p);
    case reportReasonFake::ID:
      return reportReasonFake::fetch(env, p);
    case reportReasonIllegalDrugs::ID:
      return reportReasonIllegalDrugs::fetch(env, p);
    case reportReasonPersonalDetails::ID:
      return reportReasonPersonalDetails::fetch(env, p);
    case reportReasonCustom::ID:
      return reportReasonCustom::fetch(env, p);
    case reportSponsoredResultOk::ID:
      return reportSponsoredResultOk::fetch(env, p);
    case reportSponsoredResultFailed::ID:
      return reportSponsoredResultFailed::fetch(env, p);
    case reportSponsoredResultOptionRequired::ID:
      return reportSponsoredResultOptionRequired::fetch(env, p);
    case reportSponsoredResultAdsHidden::ID:
      return reportSponsoredResultAdsHidden::fetch(env, p);
    case reportSponsoredResultPremiumRequired::ID:
      return reportSponsoredResultPremiumRequired::fetch(env, p);
    case reportStoryResultOk::ID:
      return reportStoryResultOk::fetch(env, p);
    case reportStoryResultOptionRequired::ID:
      return reportStoryResultOptionRequired::fetch(env, p);
    case reportStoryResultTextRequired::ID:
      return reportStoryResultTextRequired::fetch(env, p);
    case resendCodeReasonUserRequest::ID:
      return resendCodeReasonUserRequest::fetch(env, p);
    case resendCodeReasonVerificationFailed::ID:
      return resendCodeReasonVerificationFailed::fetch(env, p);
    case resetPasswordResultOk::ID:
      return resetPasswordResultOk::fetch(env, p);
    case resetPasswordResultPending::ID:
      return resetPasswordResultPending::fetch(env, p);
    case resetPasswordResultDeclined::ID:
      return resetPasswordResultDeclined::fetch(env, p);
    case revenueWithdrawalStatePending::ID:
      return revenueWithdrawalStatePending::fetch(env, p);
    case revenueWithdrawalStateSucceeded::ID:
      return revenueWithdrawalStateSucceeded::fetch(env, p);
    case revenueWithdrawalStateFailed::ID:
      return revenueWithdrawalStateFailed::fetch(env, p);
    case richTextPlain::ID:
      return richTextPlain::fetch(env, p);
    case richTextBold::ID:
      return richTextBold::fetch(env, p);
    case richTextItalic::ID:
      return richTextItalic::fetch(env, p);
    case richTextUnderline::ID:
      return richTextUnderline::fetch(env, p);
    case richTextStrikethrough::ID:
      return richTextStrikethrough::fetch(env, p);
    case richTextFixed::ID:
      return richTextFixed::fetch(env, p);
    case richTextUrl::ID:
      return richTextUrl::fetch(env, p);
    case richTextEmailAddress::ID:
      return richTextEmailAddress::fetch(env, p);
    case richTextSubscript::ID:
      return richTextSubscript::fetch(env, p);
    case richTextSuperscript::ID:
      return richTextSuperscript::fetch(env, p);
    case richTextMarked::ID:
      return richTextMarked::fetch(env, p);
    case richTextPhoneNumber::ID:
      return richTextPhoneNumber::fetch(env, p);
    case richTextIcon::ID:
      return richTextIcon::fetch(env, p);
    case richTextReference::ID:
      return richTextReference::fetch(env, p);
    case richTextAnchor::ID:
      return richTextAnchor::fetch(env, p);
    case richTextAnchorLink::ID:
      return richTextAnchorLink::fetch(env, p);
    case richTexts::ID:
      return richTexts::fetch(env, p);
    case rtmpUrl::ID:
      return rtmpUrl::fetch(env, p);
    case savedCredentials::ID:
      return savedCredentials::fetch(env, p);
    case savedMessagesTag::ID:
      return savedMessagesTag::fetch(env, p);
    case savedMessagesTags::ID:
      return savedMessagesTags::fetch(env, p);
    case savedMessagesTopic::ID:
      return savedMessagesTopic::fetch(env, p);
    case savedMessagesTopicTypeMyNotes::ID:
      return savedMessagesTopicTypeMyNotes::fetch(env, p);
    case savedMessagesTopicTypeAuthorHidden::ID:
      return savedMessagesTopicTypeAuthorHidden::fetch(env, p);
    case savedMessagesTopicTypeSavedFromChat::ID:
      return savedMessagesTopicTypeSavedFromChat::fetch(env, p);
    case scopeAutosaveSettings::ID:
      return scopeAutosaveSettings::fetch(env, p);
    case scopeNotificationSettings::ID:
      return scopeNotificationSettings::fetch(env, p);
    case searchMessagesChatTypeFilterPrivate::ID:
      return searchMessagesChatTypeFilterPrivate::fetch(env, p);
    case searchMessagesChatTypeFilterGroup::ID:
      return searchMessagesChatTypeFilterGroup::fetch(env, p);
    case searchMessagesChatTypeFilterChannel::ID:
      return searchMessagesChatTypeFilterChannel::fetch(env, p);
    case searchMessagesFilterEmpty::ID:
      return searchMessagesFilterEmpty::fetch(env, p);
    case searchMessagesFilterAnimation::ID:
      return searchMessagesFilterAnimation::fetch(env, p);
    case searchMessagesFilterAudio::ID:
      return searchMessagesFilterAudio::fetch(env, p);
    case searchMessagesFilterDocument::ID:
      return searchMessagesFilterDocument::fetch(env, p);
    case searchMessagesFilterPhoto::ID:
      return searchMessagesFilterPhoto::fetch(env, p);
    case searchMessagesFilterVideo::ID:
      return searchMessagesFilterVideo::fetch(env, p);
    case searchMessagesFilterVoiceNote::ID:
      return searchMessagesFilterVoiceNote::fetch(env, p);
    case searchMessagesFilterPhotoAndVideo::ID:
      return searchMessagesFilterPhotoAndVideo::fetch(env, p);
    case searchMessagesFilterUrl::ID:
      return searchMessagesFilterUrl::fetch(env, p);
    case searchMessagesFilterChatPhoto::ID:
      return searchMessagesFilterChatPhoto::fetch(env, p);
    case searchMessagesFilterVideoNote::ID:
      return searchMessagesFilterVideoNote::fetch(env, p);
    case searchMessagesFilterVoiceAndVideoNote::ID:
      return searchMessagesFilterVoiceAndVideoNote::fetch(env, p);
    case searchMessagesFilterMention::ID:
      return searchMessagesFilterMention::fetch(env, p);
    case searchMessagesFilterUnreadMention::ID:
      return searchMessagesFilterUnreadMention::fetch(env, p);
    case searchMessagesFilterUnreadReaction::ID:
      return searchMessagesFilterUnreadReaction::fetch(env, p);
    case searchMessagesFilterFailedToSend::ID:
      return searchMessagesFilterFailedToSend::fetch(env, p);
    case searchMessagesFilterPinned::ID:
      return searchMessagesFilterPinned::fetch(env, p);
    case seconds::ID:
      return seconds::fetch(env, p);
    case secretChat::ID:
      return secretChat::fetch(env, p);
    case secretChatStatePending::ID:
      return secretChatStatePending::fetch(env, p);
    case secretChatStateReady::ID:
      return secretChatStateReady::fetch(env, p);
    case secretChatStateClosed::ID:
      return secretChatStateClosed::fetch(env, p);
    case sentGiftRegular::ID:
      return sentGiftRegular::fetch(env, p);
    case sentGiftUpgraded::ID:
      return sentGiftUpgraded::fetch(env, p);
    case sentWebAppMessage::ID:
      return sentWebAppMessage::fetch(env, p);
    case session::ID:
      return session::fetch(env, p);
    case sessionTypeAndroid::ID:
      return sessionTypeAndroid::fetch(env, p);
    case sessionTypeApple::ID:
      return sessionTypeApple::fetch(env, p);
    case sessionTypeBrave::ID:
      return sessionTypeBrave::fetch(env, p);
    case sessionTypeChrome::ID:
      return sessionTypeChrome::fetch(env, p);
    case sessionTypeEdge::ID:
      return sessionTypeEdge::fetch(env, p);
    case sessionTypeFirefox::ID:
      return sessionTypeFirefox::fetch(env, p);
    case sessionTypeIpad::ID:
      return sessionTypeIpad::fetch(env, p);
    case sessionTypeIphone::ID:
      return sessionTypeIphone::fetch(env, p);
    case sessionTypeLinux::ID:
      return sessionTypeLinux::fetch(env, p);
    case sessionTypeMac::ID:
      return sessionTypeMac::fetch(env, p);
    case sessionTypeOpera::ID:
      return sessionTypeOpera::fetch(env, p);
    case sessionTypeSafari::ID:
      return sessionTypeSafari::fetch(env, p);
    case sessionTypeUbuntu::ID:
      return sessionTypeUbuntu::fetch(env, p);
    case sessionTypeUnknown::ID:
      return sessionTypeUnknown::fetch(env, p);
    case sessionTypeVivaldi::ID:
      return sessionTypeVivaldi::fetch(env, p);
    case sessionTypeWindows::ID:
      return sessionTypeWindows::fetch(env, p);
    case sessionTypeXbox::ID:
      return sessionTypeXbox::fetch(env, p);
    case sessions::ID:
      return sessions::fetch(env, p);
    case sharedChat::ID:
      return sharedChat::fetch(env, p);
    case sharedUser::ID:
      return sharedUser::fetch(env, p);
    case shippingOption::ID:
      return shippingOption::fetch(env, p);
    case speechRecognitionResultPending::ID:
      return speechRecognitionResultPending::fetch(env, p);
    case speechRecognitionResultText::ID:
      return speechRecognitionResultText::fetch(env, p);
    case speechRecognitionResultError::ID:
      return speechRecognitionResultError::fetch(env, p);
    case sponsoredChat::ID:
      return sponsoredChat::fetch(env, p);
    case sponsoredChats::ID:
      return sponsoredChats::fetch(env, p);
    case sponsoredMessage::ID:
      return sponsoredMessage::fetch(env, p);
    case sponsoredMessages::ID:
      return sponsoredMessages::fetch(env, p);
    case starAmount::ID:
      return starAmount::fetch(env, p);
    case starCount::ID:
      return starCount::fetch(env, p);
    case starGiveawayPaymentOption::ID:
      return starGiveawayPaymentOption::fetch(env, p);
    case starGiveawayPaymentOptions::ID:
      return starGiveawayPaymentOptions::fetch(env, p);
    case starGiveawayWinnerOption::ID:
      return starGiveawayWinnerOption::fetch(env, p);
    case starPaymentOption::ID:
      return starPaymentOption::fetch(env, p);
    case starPaymentOptions::ID:
      return starPaymentOptions::fetch(env, p);
    case starRevenueStatistics::ID:
      return starRevenueStatistics::fetch(env, p);
    case starRevenueStatus::ID:
      return starRevenueStatus::fetch(env, p);
    case starSubscription::ID:
      return starSubscription::fetch(env, p);
    case starSubscriptionPricing::ID:
      return starSubscriptionPricing::fetch(env, p);
    case starSubscriptionTypeChannel::ID:
      return starSubscriptionTypeChannel::fetch(env, p);
    case starSubscriptionTypeBot::ID:
      return starSubscriptionTypeBot::fetch(env, p);
    case starSubscriptions::ID:
      return starSubscriptions::fetch(env, p);
    case starTransaction::ID:
      return starTransaction::fetch(env, p);
    case starTransactionDirectionIncoming::ID:
      return starTransactionDirectionIncoming::fetch(env, p);
    case starTransactionDirectionOutgoing::ID:
      return starTransactionDirectionOutgoing::fetch(env, p);
    case starTransactionTypePremiumBotDeposit::ID:
      return starTransactionTypePremiumBotDeposit::fetch(env, p);
    case starTransactionTypeAppStoreDeposit::ID:
      return starTransactionTypeAppStoreDeposit::fetch(env, p);
    case starTransactionTypeGooglePlayDeposit::ID:
      return starTransactionTypeGooglePlayDeposit::fetch(env, p);
    case starTransactionTypeFragmentDeposit::ID:
      return starTransactionTypeFragmentDeposit::fetch(env, p);
    case starTransactionTypeUserDeposit::ID:
      return starTransactionTypeUserDeposit::fetch(env, p);
    case starTransactionTypeGiveawayDeposit::ID:
      return starTransactionTypeGiveawayDeposit::fetch(env, p);
    case starTransactionTypeFragmentWithdrawal::ID:
      return starTransactionTypeFragmentWithdrawal::fetch(env, p);
    case starTransactionTypeTelegramAdsWithdrawal::ID:
      return starTransactionTypeTelegramAdsWithdrawal::fetch(env, p);
    case starTransactionTypeTelegramApiUsage::ID:
      return starTransactionTypeTelegramApiUsage::fetch(env, p);
    case starTransactionTypeBotPaidMediaPurchase::ID:
      return starTransactionTypeBotPaidMediaPurchase::fetch(env, p);
    case starTransactionTypeBotPaidMediaSale::ID:
      return starTransactionTypeBotPaidMediaSale::fetch(env, p);
    case starTransactionTypeChannelPaidMediaPurchase::ID:
      return starTransactionTypeChannelPaidMediaPurchase::fetch(env, p);
    case starTransactionTypeChannelPaidMediaSale::ID:
      return starTransactionTypeChannelPaidMediaSale::fetch(env, p);
    case starTransactionTypeBotInvoicePurchase::ID:
      return starTransactionTypeBotInvoicePurchase::fetch(env, p);
    case starTransactionTypeBotInvoiceSale::ID:
      return starTransactionTypeBotInvoiceSale::fetch(env, p);
    case starTransactionTypeBotSubscriptionPurchase::ID:
      return starTransactionTypeBotSubscriptionPurchase::fetch(env, p);
    case starTransactionTypeBotSubscriptionSale::ID:
      return starTransactionTypeBotSubscriptionSale::fetch(env, p);
    case starTransactionTypeChannelSubscriptionPurchase::ID:
      return starTransactionTypeChannelSubscriptionPurchase::fetch(env, p);
    case starTransactionTypeChannelSubscriptionSale::ID:
      return starTransactionTypeChannelSubscriptionSale::fetch(env, p);
    case starTransactionTypeGiftPurchase::ID:
      return starTransactionTypeGiftPurchase::fetch(env, p);
    case starTransactionTypeGiftTransfer::ID:
      return starTransactionTypeGiftTransfer::fetch(env, p);
    case starTransactionTypeGiftSale::ID:
      return starTransactionTypeGiftSale::fetch(env, p);
    case starTransactionTypeGiftUpgrade::ID:
      return starTransactionTypeGiftUpgrade::fetch(env, p);
    case starTransactionTypeUpgradedGiftPurchase::ID:
      return starTransactionTypeUpgradedGiftPurchase::fetch(env, p);
    case starTransactionTypeUpgradedGiftSale::ID:
      return starTransactionTypeUpgradedGiftSale::fetch(env, p);
    case starTransactionTypeChannelPaidReactionSend::ID:
      return starTransactionTypeChannelPaidReactionSend::fetch(env, p);
    case starTransactionTypeChannelPaidReactionReceive::ID:
      return starTransactionTypeChannelPaidReactionReceive::fetch(env, p);
    case starTransactionTypeAffiliateProgramCommission::ID:
      return starTransactionTypeAffiliateProgramCommission::fetch(env, p);
    case starTransactionTypePaidMessageSend::ID:
      return starTransactionTypePaidMessageSend::fetch(env, p);
    case starTransactionTypePaidMessageReceive::ID:
      return starTransactionTypePaidMessageReceive::fetch(env, p);
    case starTransactionTypePremiumPurchase::ID:
      return starTransactionTypePremiumPurchase::fetch(env, p);
    case starTransactionTypeBusinessBotTransferSend::ID:
      return starTransactionTypeBusinessBotTransferSend::fetch(env, p);
    case starTransactionTypeBusinessBotTransferReceive::ID:
      return starTransactionTypeBusinessBotTransferReceive::fetch(env, p);
    case starTransactionTypeUnsupported::ID:
      return starTransactionTypeUnsupported::fetch(env, p);
    case starTransactions::ID:
      return starTransactions::fetch(env, p);
    case statisticalGraphData::ID:
      return statisticalGraphData::fetch(env, p);
    case statisticalGraphAsync::ID:
      return statisticalGraphAsync::fetch(env, p);
    case statisticalGraphError::ID:
      return statisticalGraphError::fetch(env, p);
    case statisticalValue::ID:
      return statisticalValue::fetch(env, p);
    case sticker::ID:
      return sticker::fetch(env, p);
    case stickerFormatWebp::ID:
      return stickerFormatWebp::fetch(env, p);
    case stickerFormatTgs::ID:
      return stickerFormatTgs::fetch(env, p);
    case stickerFormatWebm::ID:
      return stickerFormatWebm::fetch(env, p);
    case stickerFullTypeRegular::ID:
      return stickerFullTypeRegular::fetch(env, p);
    case stickerFullTypeMask::ID:
      return stickerFullTypeMask::fetch(env, p);
    case stickerFullTypeCustomEmoji::ID:
      return stickerFullTypeCustomEmoji::fetch(env, p);
    case stickerSet::ID:
      return stickerSet::fetch(env, p);
    case stickerSetInfo::ID:
      return stickerSetInfo::fetch(env, p);
    case stickerSets::ID:
      return stickerSets::fetch(env, p);
    case stickerTypeRegular::ID:
      return stickerTypeRegular::fetch(env, p);
    case stickerTypeMask::ID:
      return stickerTypeMask::fetch(env, p);
    case stickerTypeCustomEmoji::ID:
      return stickerTypeCustomEmoji::fetch(env, p);
    case stickers::ID:
      return stickers::fetch(env, p);
    case storageStatistics::ID:
      return storageStatistics::fetch(env, p);
    case storageStatisticsByChat::ID:
      return storageStatisticsByChat::fetch(env, p);
    case storageStatisticsByFileType::ID:
      return storageStatisticsByFileType::fetch(env, p);
    case storageStatisticsFast::ID:
      return storageStatisticsFast::fetch(env, p);
    case storePaymentPurposePremiumSubscription::ID:
      return storePaymentPurposePremiumSubscription::fetch(env, p);
    case storePaymentPurposePremiumGift::ID:
      return storePaymentPurposePremiumGift::fetch(env, p);
    case storePaymentPurposePremiumGiftCodes::ID:
      return storePaymentPurposePremiumGiftCodes::fetch(env, p);
    case storePaymentPurposePremiumGiveaway::ID:
      return storePaymentPurposePremiumGiveaway::fetch(env, p);
    case storePaymentPurposeStarGiveaway::ID:
      return storePaymentPurposeStarGiveaway::fetch(env, p);
    case storePaymentPurposeStars::ID:
      return storePaymentPurposeStars::fetch(env, p);
    case storePaymentPurposeGiftedStars::ID:
      return storePaymentPurposeGiftedStars::fetch(env, p);
    case storeTransactionAppStore::ID:
      return storeTransactionAppStore::fetch(env, p);
    case storeTransactionGooglePlay::ID:
      return storeTransactionGooglePlay::fetch(env, p);
    case stories::ID:
      return stories::fetch(env, p);
    case story::ID:
      return story::fetch(env, p);
    case storyArea::ID:
      return storyArea::fetch(env, p);
    case storyAreaPosition::ID:
      return storyAreaPosition::fetch(env, p);
    case storyAreaTypeLocation::ID:
      return storyAreaTypeLocation::fetch(env, p);
    case storyAreaTypeVenue::ID:
      return storyAreaTypeVenue::fetch(env, p);
    case storyAreaTypeSuggestedReaction::ID:
      return storyAreaTypeSuggestedReaction::fetch(env, p);
    case storyAreaTypeMessage::ID:
      return storyAreaTypeMessage::fetch(env, p);
    case storyAreaTypeLink::ID:
      return storyAreaTypeLink::fetch(env, p);
    case storyAreaTypeWeather::ID:
      return storyAreaTypeWeather::fetch(env, p);
    case storyAreaTypeUpgradedGift::ID:
      return storyAreaTypeUpgradedGift::fetch(env, p);
    case storyContentPhoto::ID:
      return storyContentPhoto::fetch(env, p);
    case storyContentVideo::ID:
      return storyContentVideo::fetch(env, p);
    case storyContentUnsupported::ID:
      return storyContentUnsupported::fetch(env, p);
    case storyFullId::ID:
      return storyFullId::fetch(env, p);
    case storyInfo::ID:
      return storyInfo::fetch(env, p);
    case storyInteraction::ID:
      return storyInteraction::fetch(env, p);
    case storyInteractionInfo::ID:
      return storyInteractionInfo::fetch(env, p);
    case storyInteractionTypeView::ID:
      return storyInteractionTypeView::fetch(env, p);
    case storyInteractionTypeForward::ID:
      return storyInteractionTypeForward::fetch(env, p);
    case storyInteractionTypeRepost::ID:
      return storyInteractionTypeRepost::fetch(env, p);
    case storyInteractions::ID:
      return storyInteractions::fetch(env, p);
    case storyListMain::ID:
      return storyListMain::fetch(env, p);
    case storyListArchive::ID:
      return storyListArchive::fetch(env, p);
    case storyOriginPublicStory::ID:
      return storyOriginPublicStory::fetch(env, p);
    case storyOriginHiddenUser::ID:
      return storyOriginHiddenUser::fetch(env, p);
    case storyPrivacySettingsEveryone::ID:
      return storyPrivacySettingsEveryone::fetch(env, p);
    case storyPrivacySettingsContacts::ID:
      return storyPrivacySettingsContacts::fetch(env, p);
    case storyPrivacySettingsCloseFriends::ID:
      return storyPrivacySettingsCloseFriends::fetch(env, p);
    case storyPrivacySettingsSelectedUsers::ID:
      return storyPrivacySettingsSelectedUsers::fetch(env, p);
    case storyRepostInfo::ID:
      return storyRepostInfo::fetch(env, p);
    case storyStatistics::ID:
      return storyStatistics::fetch(env, p);
    case storyVideo::ID:
      return storyVideo::fetch(env, p);
    case suggestedActionEnableArchiveAndMuteNewChats::ID:
      return suggestedActionEnableArchiveAndMuteNewChats::fetch(env, p);
    case suggestedActionCheckPassword::ID:
      return suggestedActionCheckPassword::fetch(env, p);
    case suggestedActionCheckPhoneNumber::ID:
      return suggestedActionCheckPhoneNumber::fetch(env, p);
    case suggestedActionViewChecksHint::ID:
      return suggestedActionViewChecksHint::fetch(env, p);
    case suggestedActionConvertToBroadcastGroup::ID:
      return suggestedActionConvertToBroadcastGroup::fetch(env, p);
    case suggestedActionSetPassword::ID:
      return suggestedActionSetPassword::fetch(env, p);
    case suggestedActionUpgradePremium::ID:
      return suggestedActionUpgradePremium::fetch(env, p);
    case suggestedActionRestorePremium::ID:
      return suggestedActionRestorePremium::fetch(env, p);
    case suggestedActionSubscribeToAnnualPremium::ID:
      return suggestedActionSubscribeToAnnualPremium::fetch(env, p);
    case suggestedActionGiftPremiumForChristmas::ID:
      return suggestedActionGiftPremiumForChristmas::fetch(env, p);
    case suggestedActionSetBirthdate::ID:
      return suggestedActionSetBirthdate::fetch(env, p);
    case suggestedActionSetProfilePhoto::ID:
      return suggestedActionSetProfilePhoto::fetch(env, p);
    case suggestedActionExtendPremium::ID:
      return suggestedActionExtendPremium::fetch(env, p);
    case suggestedActionExtendStarSubscriptions::ID:
      return suggestedActionExtendStarSubscriptions::fetch(env, p);
    case suggestedActionCustom::ID:
      return suggestedActionCustom::fetch(env, p);
    case supergroup::ID:
      return supergroup::fetch(env, p);
    case supergroupFullInfo::ID:
      return supergroupFullInfo::fetch(env, p);
    case supergroupMembersFilterRecent::ID:
      return supergroupMembersFilterRecent::fetch(env, p);
    case supergroupMembersFilterContacts::ID:
      return supergroupMembersFilterContacts::fetch(env, p);
    case supergroupMembersFilterAdministrators::ID:
      return supergroupMembersFilterAdministrators::fetch(env, p);
    case supergroupMembersFilterSearch::ID:
      return supergroupMembersFilterSearch::fetch(env, p);
    case supergroupMembersFilterRestricted::ID:
      return supergroupMembersFilterRestricted::fetch(env, p);
    case supergroupMembersFilterBanned::ID:
      return supergroupMembersFilterBanned::fetch(env, p);
    case supergroupMembersFilterMention::ID:
      return supergroupMembersFilterMention::fetch(env, p);
    case supergroupMembersFilterBots::ID:
      return supergroupMembersFilterBots::fetch(env, p);
    case tMeUrl::ID:
      return tMeUrl::fetch(env, p);
    case tMeUrlTypeUser::ID:
      return tMeUrlTypeUser::fetch(env, p);
    case tMeUrlTypeSupergroup::ID:
      return tMeUrlTypeSupergroup::fetch(env, p);
    case tMeUrlTypeChatInvite::ID:
      return tMeUrlTypeChatInvite::fetch(env, p);
    case tMeUrlTypeStickerSet::ID:
      return tMeUrlTypeStickerSet::fetch(env, p);
    case tMeUrls::ID:
      return tMeUrls::fetch(env, p);
    case targetChatCurrent::ID:
      return targetChatCurrent::fetch(env, p);
    case targetChatChosen::ID:
      return targetChatChosen::fetch(env, p);
    case targetChatInternalLink::ID:
      return targetChatInternalLink::fetch(env, p);
    case targetChatTypes::ID:
      return targetChatTypes::fetch(env, p);
    case telegramPaymentPurposePremiumGift::ID:
      return telegramPaymentPurposePremiumGift::fetch(env, p);
    case telegramPaymentPurposePremiumGiftCodes::ID:
      return telegramPaymentPurposePremiumGiftCodes::fetch(env, p);
    case telegramPaymentPurposePremiumGiveaway::ID:
      return telegramPaymentPurposePremiumGiveaway::fetch(env, p);
    case telegramPaymentPurposeStars::ID:
      return telegramPaymentPurposeStars::fetch(env, p);
    case telegramPaymentPurposeGiftedStars::ID:
      return telegramPaymentPurposeGiftedStars::fetch(env, p);
    case telegramPaymentPurposeStarGiveaway::ID:
      return telegramPaymentPurposeStarGiveaway::fetch(env, p);
    case telegramPaymentPurposeJoinChat::ID:
      return telegramPaymentPurposeJoinChat::fetch(env, p);
    case temporaryPasswordState::ID:
      return temporaryPasswordState::fetch(env, p);
    case termsOfService::ID:
      return termsOfService::fetch(env, p);
    case testBytes::ID:
      return testBytes::fetch(env, p);
    case testInt::ID:
      return testInt::fetch(env, p);
    case testString::ID:
      return testString::fetch(env, p);
    case testVectorInt::ID:
      return testVectorInt::fetch(env, p);
    case testVectorIntObject::ID:
      return testVectorIntObject::fetch(env, p);
    case testVectorString::ID:
      return testVectorString::fetch(env, p);
    case testVectorStringObject::ID:
      return testVectorStringObject::fetch(env, p);
    case text::ID:
      return text::fetch(env, p);
    case textEntities::ID:
      return textEntities::fetch(env, p);
    case textEntity::ID:
      return textEntity::fetch(env, p);
    case textEntityTypeMention::ID:
      return textEntityTypeMention::fetch(env, p);
    case textEntityTypeHashtag::ID:
      return textEntityTypeHashtag::fetch(env, p);
    case textEntityTypeCashtag::ID:
      return textEntityTypeCashtag::fetch(env, p);
    case textEntityTypeBotCommand::ID:
      return textEntityTypeBotCommand::fetch(env, p);
    case textEntityTypeUrl::ID:
      return textEntityTypeUrl::fetch(env, p);
    case textEntityTypeEmailAddress::ID:
      return textEntityTypeEmailAddress::fetch(env, p);
    case textEntityTypePhoneNumber::ID:
      return textEntityTypePhoneNumber::fetch(env, p);
    case textEntityTypeBankCardNumber::ID:
      return textEntityTypeBankCardNumber::fetch(env, p);
    case textEntityTypeBold::ID:
      return textEntityTypeBold::fetch(env, p);
    case textEntityTypeItalic::ID:
      return textEntityTypeItalic::fetch(env, p);
    case textEntityTypeUnderline::ID:
      return textEntityTypeUnderline::fetch(env, p);
    case textEntityTypeStrikethrough::ID:
      return textEntityTypeStrikethrough::fetch(env, p);
    case textEntityTypeSpoiler::ID:
      return textEntityTypeSpoiler::fetch(env, p);
    case textEntityTypeCode::ID:
      return textEntityTypeCode::fetch(env, p);
    case textEntityTypePre::ID:
      return textEntityTypePre::fetch(env, p);
    case textEntityTypePreCode::ID:
      return textEntityTypePreCode::fetch(env, p);
    case textEntityTypeBlockQuote::ID:
      return textEntityTypeBlockQuote::fetch(env, p);
    case textEntityTypeExpandableBlockQuote::ID:
      return textEntityTypeExpandableBlockQuote::fetch(env, p);
    case textEntityTypeTextUrl::ID:
      return textEntityTypeTextUrl::fetch(env, p);
    case textEntityTypeMentionName::ID:
      return textEntityTypeMentionName::fetch(env, p);
    case textEntityTypeCustomEmoji::ID:
      return textEntityTypeCustomEmoji::fetch(env, p);
    case textEntityTypeMediaTimestamp::ID:
      return textEntityTypeMediaTimestamp::fetch(env, p);
    case textParseModeMarkdown::ID:
      return textParseModeMarkdown::fetch(env, p);
    case textParseModeHTML::ID:
      return textParseModeHTML::fetch(env, p);
    case textQuote::ID:
      return textQuote::fetch(env, p);
    case themeParameters::ID:
      return themeParameters::fetch(env, p);
    case themeSettings::ID:
      return themeSettings::fetch(env, p);
    case thumbnail::ID:
      return thumbnail::fetch(env, p);
    case thumbnailFormatJpeg::ID:
      return thumbnailFormatJpeg::fetch(env, p);
    case thumbnailFormatGif::ID:
      return thumbnailFormatGif::fetch(env, p);
    case thumbnailFormatMpeg4::ID:
      return thumbnailFormatMpeg4::fetch(env, p);
    case thumbnailFormatPng::ID:
      return thumbnailFormatPng::fetch(env, p);
    case thumbnailFormatTgs::ID:
      return thumbnailFormatTgs::fetch(env, p);
    case thumbnailFormatWebm::ID:
      return thumbnailFormatWebm::fetch(env, p);
    case thumbnailFormatWebp::ID:
      return thumbnailFormatWebp::fetch(env, p);
    case timeZone::ID:
      return timeZone::fetch(env, p);
    case timeZones::ID:
      return timeZones::fetch(env, p);
    case topChatCategoryUsers::ID:
      return topChatCategoryUsers::fetch(env, p);
    case topChatCategoryBots::ID:
      return topChatCategoryBots::fetch(env, p);
    case topChatCategoryGroups::ID:
      return topChatCategoryGroups::fetch(env, p);
    case topChatCategoryChannels::ID:
      return topChatCategoryChannels::fetch(env, p);
    case topChatCategoryInlineBots::ID:
      return topChatCategoryInlineBots::fetch(env, p);
    case topChatCategoryWebAppBots::ID:
      return topChatCategoryWebAppBots::fetch(env, p);
    case topChatCategoryCalls::ID:
      return topChatCategoryCalls::fetch(env, p);
    case topChatCategoryForwardChats::ID:
      return topChatCategoryForwardChats::fetch(env, p);
    case trendingStickerSets::ID:
      return trendingStickerSets::fetch(env, p);
    case unconfirmedSession::ID:
      return unconfirmedSession::fetch(env, p);
    case unreadReaction::ID:
      return unreadReaction::fetch(env, p);
    case updateAuthorizationState::ID:
      return updateAuthorizationState::fetch(env, p);
    case updateNewMessage::ID:
      return updateNewMessage::fetch(env, p);
    case updateMessageSendAcknowledged::ID:
      return updateMessageSendAcknowledged::fetch(env, p);
    case updateMessageSendSucceeded::ID:
      return updateMessageSendSucceeded::fetch(env, p);
    case updateMessageSendFailed::ID:
      return updateMessageSendFailed::fetch(env, p);
    case updateMessageContent::ID:
      return updateMessageContent::fetch(env, p);
    case updateMessageEdited::ID:
      return updateMessageEdited::fetch(env, p);
    case updateMessageIsPinned::ID:
      return updateMessageIsPinned::fetch(env, p);
    case updateMessageInteractionInfo::ID:
      return updateMessageInteractionInfo::fetch(env, p);
    case updateMessageContentOpened::ID:
      return updateMessageContentOpened::fetch(env, p);
    case updateMessageMentionRead::ID:
      return updateMessageMentionRead::fetch(env, p);
    case updateMessageUnreadReactions::ID:
      return updateMessageUnreadReactions::fetch(env, p);
    case updateMessageFactCheck::ID:
      return updateMessageFactCheck::fetch(env, p);
    case updateMessageLiveLocationViewed::ID:
      return updateMessageLiveLocationViewed::fetch(env, p);
    case updateVideoPublished::ID:
      return updateVideoPublished::fetch(env, p);
    case updateNewChat::ID:
      return updateNewChat::fetch(env, p);
    case updateChatTitle::ID:
      return updateChatTitle::fetch(env, p);
    case updateChatPhoto::ID:
      return updateChatPhoto::fetch(env, p);
    case updateChatAccentColors::ID:
      return updateChatAccentColors::fetch(env, p);
    case updateChatPermissions::ID:
      return updateChatPermissions::fetch(env, p);
    case updateChatLastMessage::ID:
      return updateChatLastMessage::fetch(env, p);
    case updateChatPosition::ID:
      return updateChatPosition::fetch(env, p);
    case updateChatAddedToList::ID:
      return updateChatAddedToList::fetch(env, p);
    case updateChatRemovedFromList::ID:
      return updateChatRemovedFromList::fetch(env, p);
    case updateChatReadInbox::ID:
      return updateChatReadInbox::fetch(env, p);
    case updateChatReadOutbox::ID:
      return updateChatReadOutbox::fetch(env, p);
    case updateChatActionBar::ID:
      return updateChatActionBar::fetch(env, p);
    case updateChatBusinessBotManageBar::ID:
      return updateChatBusinessBotManageBar::fetch(env, p);
    case updateChatAvailableReactions::ID:
      return updateChatAvailableReactions::fetch(env, p);
    case updateChatDraftMessage::ID:
      return updateChatDraftMessage::fetch(env, p);
    case updateChatEmojiStatus::ID:
      return updateChatEmojiStatus::fetch(env, p);
    case updateChatMessageSender::ID:
      return updateChatMessageSender::fetch(env, p);
    case updateChatMessageAutoDeleteTime::ID:
      return updateChatMessageAutoDeleteTime::fetch(env, p);
    case updateChatNotificationSettings::ID:
      return updateChatNotificationSettings::fetch(env, p);
    case updateChatPendingJoinRequests::ID:
      return updateChatPendingJoinRequests::fetch(env, p);
    case updateChatReplyMarkup::ID:
      return updateChatReplyMarkup::fetch(env, p);
    case updateChatBackground::ID:
      return updateChatBackground::fetch(env, p);
    case updateChatTheme::ID:
      return updateChatTheme::fetch(env, p);
    case updateChatUnreadMentionCount::ID:
      return updateChatUnreadMentionCount::fetch(env, p);
    case updateChatUnreadReactionCount::ID:
      return updateChatUnreadReactionCount::fetch(env, p);
    case updateChatVideoChat::ID:
      return updateChatVideoChat::fetch(env, p);
    case updateChatDefaultDisableNotification::ID:
      return updateChatDefaultDisableNotification::fetch(env, p);
    case updateChatHasProtectedContent::ID:
      return updateChatHasProtectedContent::fetch(env, p);
    case updateChatIsTranslatable::ID:
      return updateChatIsTranslatable::fetch(env, p);
    case updateChatIsMarkedAsUnread::ID:
      return updateChatIsMarkedAsUnread::fetch(env, p);
    case updateChatViewAsTopics::ID:
      return updateChatViewAsTopics::fetch(env, p);
    case updateChatBlockList::ID:
      return updateChatBlockList::fetch(env, p);
    case updateChatHasScheduledMessages::ID:
      return updateChatHasScheduledMessages::fetch(env, p);
    case updateChatFolders::ID:
      return updateChatFolders::fetch(env, p);
    case updateChatOnlineMemberCount::ID:
      return updateChatOnlineMemberCount::fetch(env, p);
    case updateSavedMessagesTopic::ID:
      return updateSavedMessagesTopic::fetch(env, p);
    case updateSavedMessagesTopicCount::ID:
      return updateSavedMessagesTopicCount::fetch(env, p);
    case updateQuickReplyShortcut::ID:
      return updateQuickReplyShortcut::fetch(env, p);
    case updateQuickReplyShortcutDeleted::ID:
      return updateQuickReplyShortcutDeleted::fetch(env, p);
    case updateQuickReplyShortcuts::ID:
      return updateQuickReplyShortcuts::fetch(env, p);
    case updateQuickReplyShortcutMessages::ID:
      return updateQuickReplyShortcutMessages::fetch(env, p);
    case updateForumTopicInfo::ID:
      return updateForumTopicInfo::fetch(env, p);
    case updateForumTopic::ID:
      return updateForumTopic::fetch(env, p);
    case updateScopeNotificationSettings::ID:
      return updateScopeNotificationSettings::fetch(env, p);
    case updateReactionNotificationSettings::ID:
      return updateReactionNotificationSettings::fetch(env, p);
    case updateNotification::ID:
      return updateNotification::fetch(env, p);
    case updateNotificationGroup::ID:
      return updateNotificationGroup::fetch(env, p);
    case updateActiveNotifications::ID:
      return updateActiveNotifications::fetch(env, p);
    case updateHavePendingNotifications::ID:
      return updateHavePendingNotifications::fetch(env, p);
    case updateDeleteMessages::ID:
      return updateDeleteMessages::fetch(env, p);
    case updateChatAction::ID:
      return updateChatAction::fetch(env, p);
    case updateUserStatus::ID:
      return updateUserStatus::fetch(env, p);
    case updateUser::ID:
      return updateUser::fetch(env, p);
    case updateBasicGroup::ID:
      return updateBasicGroup::fetch(env, p);
    case updateSupergroup::ID:
      return updateSupergroup::fetch(env, p);
    case updateSecretChat::ID:
      return updateSecretChat::fetch(env, p);
    case updateUserFullInfo::ID:
      return updateUserFullInfo::fetch(env, p);
    case updateBasicGroupFullInfo::ID:
      return updateBasicGroupFullInfo::fetch(env, p);
    case updateSupergroupFullInfo::ID:
      return updateSupergroupFullInfo::fetch(env, p);
    case updateServiceNotification::ID:
      return updateServiceNotification::fetch(env, p);
    case updateFile::ID:
      return updateFile::fetch(env, p);
    case updateFileGenerationStart::ID:
      return updateFileGenerationStart::fetch(env, p);
    case updateFileGenerationStop::ID:
      return updateFileGenerationStop::fetch(env, p);
    case updateFileDownloads::ID:
      return updateFileDownloads::fetch(env, p);
    case updateFileAddedToDownloads::ID:
      return updateFileAddedToDownloads::fetch(env, p);
    case updateFileDownload::ID:
      return updateFileDownload::fetch(env, p);
    case updateFileRemovedFromDownloads::ID:
      return updateFileRemovedFromDownloads::fetch(env, p);
    case updateApplicationVerificationRequired::ID:
      return updateApplicationVerificationRequired::fetch(env, p);
    case updateApplicationRecaptchaVerificationRequired::ID:
      return updateApplicationRecaptchaVerificationRequired::fetch(env, p);
    case updateCall::ID:
      return updateCall::fetch(env, p);
    case updateGroupCall::ID:
      return updateGroupCall::fetch(env, p);
    case updateGroupCallParticipant::ID:
      return updateGroupCallParticipant::fetch(env, p);
    case updateGroupCallParticipants::ID:
      return updateGroupCallParticipants::fetch(env, p);
    case updateGroupCallVerificationState::ID:
      return updateGroupCallVerificationState::fetch(env, p);
    case updateNewCallSignalingData::ID:
      return updateNewCallSignalingData::fetch(env, p);
    case updateUserPrivacySettingRules::ID:
      return updateUserPrivacySettingRules::fetch(env, p);
    case updateUnreadMessageCount::ID:
      return updateUnreadMessageCount::fetch(env, p);
    case updateUnreadChatCount::ID:
      return updateUnreadChatCount::fetch(env, p);
    case updateStory::ID:
      return updateStory::fetch(env, p);
    case updateStoryDeleted::ID:
      return updateStoryDeleted::fetch(env, p);
    case updateStoryPostSucceeded::ID:
      return updateStoryPostSucceeded::fetch(env, p);
    case updateStoryPostFailed::ID:
      return updateStoryPostFailed::fetch(env, p);
    case updateChatActiveStories::ID:
      return updateChatActiveStories::fetch(env, p);
    case updateStoryListChatCount::ID:
      return updateStoryListChatCount::fetch(env, p);
    case updateStoryStealthMode::ID:
      return updateStoryStealthMode::fetch(env, p);
    case updateOption::ID:
      return updateOption::fetch(env, p);
    case updateStickerSet::ID:
      return updateStickerSet::fetch(env, p);
    case updateInstalledStickerSets::ID:
      return updateInstalledStickerSets::fetch(env, p);
    case updateTrendingStickerSets::ID:
      return updateTrendingStickerSets::fetch(env, p);
    case updateRecentStickers::ID:
      return updateRecentStickers::fetch(env, p);
    case updateFavoriteStickers::ID:
      return updateFavoriteStickers::fetch(env, p);
    case updateSavedAnimations::ID:
      return updateSavedAnimations::fetch(env, p);
    case updateSavedNotificationSounds::ID:
      return updateSavedNotificationSounds::fetch(env, p);
    case updateDefaultBackground::ID:
      return updateDefaultBackground::fetch(env, p);
    case updateChatThemes::ID:
      return updateChatThemes::fetch(env, p);
    case updateAccentColors::ID:
      return updateAccentColors::fetch(env, p);
    case updateProfileAccentColors::ID:
      return updateProfileAccentColors::fetch(env, p);
    case updateLanguagePackStrings::ID:
      return updateLanguagePackStrings::fetch(env, p);
    case updateConnectionState::ID:
      return updateConnectionState::fetch(env, p);
    case updateFreezeState::ID:
      return updateFreezeState::fetch(env, p);
    case updateTermsOfService::ID:
      return updateTermsOfService::fetch(env, p);
    case updateUnconfirmedSession::ID:
      return updateUnconfirmedSession::fetch(env, p);
    case updateAttachmentMenuBots::ID:
      return updateAttachmentMenuBots::fetch(env, p);
    case updateWebAppMessageSent::ID:
      return updateWebAppMessageSent::fetch(env, p);
    case updateActiveEmojiReactions::ID:
      return updateActiveEmojiReactions::fetch(env, p);
    case updateAvailableMessageEffects::ID:
      return updateAvailableMessageEffects::fetch(env, p);
    case updateDefaultReactionType::ID:
      return updateDefaultReactionType::fetch(env, p);
    case updateDefaultPaidReactionType::ID:
      return updateDefaultPaidReactionType::fetch(env, p);
    case updateSavedMessagesTags::ID:
      return updateSavedMessagesTags::fetch(env, p);
    case updateActiveLiveLocationMessages::ID:
      return updateActiveLiveLocationMessages::fetch(env, p);
    case updateOwnedStarCount::ID:
      return updateOwnedStarCount::fetch(env, p);
    case updateChatRevenueAmount::ID:
      return updateChatRevenueAmount::fetch(env, p);
    case updateStarRevenueStatus::ID:
      return updateStarRevenueStatus::fetch(env, p);
    case updateSpeechRecognitionTrial::ID:
      return updateSpeechRecognitionTrial::fetch(env, p);
    case updateDiceEmojis::ID:
      return updateDiceEmojis::fetch(env, p);
    case updateAnimatedEmojiMessageClicked::ID:
      return updateAnimatedEmojiMessageClicked::fetch(env, p);
    case updateAnimationSearchParameters::ID:
      return updateAnimationSearchParameters::fetch(env, p);
    case updateSuggestedActions::ID:
      return updateSuggestedActions::fetch(env, p);
    case updateSpeedLimitNotification::ID:
      return updateSpeedLimitNotification::fetch(env, p);
    case updateContactCloseBirthdays::ID:
      return updateContactCloseBirthdays::fetch(env, p);
    case updateAutosaveSettings::ID:
      return updateAutosaveSettings::fetch(env, p);
    case updateBusinessConnection::ID:
      return updateBusinessConnection::fetch(env, p);
    case updateNewBusinessMessage::ID:
      return updateNewBusinessMessage::fetch(env, p);
    case updateBusinessMessageEdited::ID:
      return updateBusinessMessageEdited::fetch(env, p);
    case updateBusinessMessagesDeleted::ID:
      return updateBusinessMessagesDeleted::fetch(env, p);
    case updateNewInlineQuery::ID:
      return updateNewInlineQuery::fetch(env, p);
    case updateNewChosenInlineResult::ID:
      return updateNewChosenInlineResult::fetch(env, p);
    case updateNewCallbackQuery::ID:
      return updateNewCallbackQuery::fetch(env, p);
    case updateNewInlineCallbackQuery::ID:
      return updateNewInlineCallbackQuery::fetch(env, p);
    case updateNewBusinessCallbackQuery::ID:
      return updateNewBusinessCallbackQuery::fetch(env, p);
    case updateNewShippingQuery::ID:
      return updateNewShippingQuery::fetch(env, p);
    case updateNewPreCheckoutQuery::ID:
      return updateNewPreCheckoutQuery::fetch(env, p);
    case updateNewCustomEvent::ID:
      return updateNewCustomEvent::fetch(env, p);
    case updateNewCustomQuery::ID:
      return updateNewCustomQuery::fetch(env, p);
    case updatePoll::ID:
      return updatePoll::fetch(env, p);
    case updatePollAnswer::ID:
      return updatePollAnswer::fetch(env, p);
    case updateChatMember::ID:
      return updateChatMember::fetch(env, p);
    case updateNewChatJoinRequest::ID:
      return updateNewChatJoinRequest::fetch(env, p);
    case updateChatBoost::ID:
      return updateChatBoost::fetch(env, p);
    case updateMessageReaction::ID:
      return updateMessageReaction::fetch(env, p);
    case updateMessageReactions::ID:
      return updateMessageReactions::fetch(env, p);
    case updatePaidMediaPurchased::ID:
      return updatePaidMediaPurchased::fetch(env, p);
    case updates::ID:
      return updates::fetch(env, p);
    case upgradeGiftResult::ID:
      return upgradeGiftResult::fetch(env, p);
    case upgradedGift::ID:
      return upgradedGift::fetch(env, p);
    case upgradedGiftAttributeIdModel::ID:
      return upgradedGiftAttributeIdModel::fetch(env, p);
    case upgradedGiftAttributeIdSymbol::ID:
      return upgradedGiftAttributeIdSymbol::fetch(env, p);
    case upgradedGiftAttributeIdBackdrop::ID:
      return upgradedGiftAttributeIdBackdrop::fetch(env, p);
    case upgradedGiftBackdrop::ID:
      return upgradedGiftBackdrop::fetch(env, p);
    case upgradedGiftBackdropColors::ID:
      return upgradedGiftBackdropColors::fetch(env, p);
    case upgradedGiftBackdropCount::ID:
      return upgradedGiftBackdropCount::fetch(env, p);
    case upgradedGiftModel::ID:
      return upgradedGiftModel::fetch(env, p);
    case upgradedGiftModelCount::ID:
      return upgradedGiftModelCount::fetch(env, p);
    case upgradedGiftOriginalDetails::ID:
      return upgradedGiftOriginalDetails::fetch(env, p);
    case upgradedGiftSymbol::ID:
      return upgradedGiftSymbol::fetch(env, p);
    case upgradedGiftSymbolCount::ID:
      return upgradedGiftSymbolCount::fetch(env, p);
    case user::ID:
      return user::fetch(env, p);
    case userFullInfo::ID:
      return userFullInfo::fetch(env, p);
    case userLink::ID:
      return userLink::fetch(env, p);
    case userPrivacySettingShowStatus::ID:
      return userPrivacySettingShowStatus::fetch(env, p);
    case userPrivacySettingShowProfilePhoto::ID:
      return userPrivacySettingShowProfilePhoto::fetch(env, p);
    case userPrivacySettingShowLinkInForwardedMessages::ID:
      return userPrivacySettingShowLinkInForwardedMessages::fetch(env, p);
    case userPrivacySettingShowPhoneNumber::ID:
      return userPrivacySettingShowPhoneNumber::fetch(env, p);
    case userPrivacySettingShowBio::ID:
      return userPrivacySettingShowBio::fetch(env, p);
    case userPrivacySettingShowBirthdate::ID:
      return userPrivacySettingShowBirthdate::fetch(env, p);
    case userPrivacySettingAllowChatInvites::ID:
      return userPrivacySettingAllowChatInvites::fetch(env, p);
    case userPrivacySettingAllowCalls::ID:
      return userPrivacySettingAllowCalls::fetch(env, p);
    case userPrivacySettingAllowPeerToPeerCalls::ID:
      return userPrivacySettingAllowPeerToPeerCalls::fetch(env, p);
    case userPrivacySettingAllowFindingByPhoneNumber::ID:
      return userPrivacySettingAllowFindingByPhoneNumber::fetch(env, p);
    case userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::ID:
      return userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::fetch(env, p);
    case userPrivacySettingAutosaveGifts::ID:
      return userPrivacySettingAutosaveGifts::fetch(env, p);
    case userPrivacySettingAllowUnpaidMessages::ID:
      return userPrivacySettingAllowUnpaidMessages::fetch(env, p);
    case userPrivacySettingRuleAllowAll::ID:
      return userPrivacySettingRuleAllowAll::fetch(env, p);
    case userPrivacySettingRuleAllowContacts::ID:
      return userPrivacySettingRuleAllowContacts::fetch(env, p);
    case userPrivacySettingRuleAllowBots::ID:
      return userPrivacySettingRuleAllowBots::fetch(env, p);
    case userPrivacySettingRuleAllowPremiumUsers::ID:
      return userPrivacySettingRuleAllowPremiumUsers::fetch(env, p);
    case userPrivacySettingRuleAllowUsers::ID:
      return userPrivacySettingRuleAllowUsers::fetch(env, p);
    case userPrivacySettingRuleAllowChatMembers::ID:
      return userPrivacySettingRuleAllowChatMembers::fetch(env, p);
    case userPrivacySettingRuleRestrictAll::ID:
      return userPrivacySettingRuleRestrictAll::fetch(env, p);
    case userPrivacySettingRuleRestrictContacts::ID:
      return userPrivacySettingRuleRestrictContacts::fetch(env, p);
    case userPrivacySettingRuleRestrictBots::ID:
      return userPrivacySettingRuleRestrictBots::fetch(env, p);
    case userPrivacySettingRuleRestrictUsers::ID:
      return userPrivacySettingRuleRestrictUsers::fetch(env, p);
    case userPrivacySettingRuleRestrictChatMembers::ID:
      return userPrivacySettingRuleRestrictChatMembers::fetch(env, p);
    case userPrivacySettingRules::ID:
      return userPrivacySettingRules::fetch(env, p);
    case userStatusEmpty::ID:
      return userStatusEmpty::fetch(env, p);
    case userStatusOnline::ID:
      return userStatusOnline::fetch(env, p);
    case userStatusOffline::ID:
      return userStatusOffline::fetch(env, p);
    case userStatusRecently::ID:
      return userStatusRecently::fetch(env, p);
    case userStatusLastWeek::ID:
      return userStatusLastWeek::fetch(env, p);
    case userStatusLastMonth::ID:
      return userStatusLastMonth::fetch(env, p);
    case userSupportInfo::ID:
      return userSupportInfo::fetch(env, p);
    case userTypeRegular::ID:
      return userTypeRegular::fetch(env, p);
    case userTypeDeleted::ID:
      return userTypeDeleted::fetch(env, p);
    case userTypeBot::ID:
      return userTypeBot::fetch(env, p);
    case userTypeUnknown::ID:
      return userTypeUnknown::fetch(env, p);
    case usernames::ID:
      return usernames::fetch(env, p);
    case users::ID:
      return users::fetch(env, p);
    case validatedOrderInfo::ID:
      return validatedOrderInfo::fetch(env, p);
    case vectorPathCommandLine::ID:
      return vectorPathCommandLine::fetch(env, p);
    case vectorPathCommandCubicBezierCurve::ID:
      return vectorPathCommandCubicBezierCurve::fetch(env, p);
    case venue::ID:
      return venue::fetch(env, p);
    case verificationStatus::ID:
      return verificationStatus::fetch(env, p);
    case video::ID:
      return video::fetch(env, p);
    case videoChat::ID:
      return videoChat::fetch(env, p);
    case videoChatStream::ID:
      return videoChatStream::fetch(env, p);
    case videoChatStreams::ID:
      return videoChatStreams::fetch(env, p);
    case videoNote::ID:
      return videoNote::fetch(env, p);
    case voiceNote::ID:
      return voiceNote::fetch(env, p);
    case webApp::ID:
      return webApp::fetch(env, p);
    case webAppInfo::ID:
      return webAppInfo::fetch(env, p);
    case webAppOpenModeCompact::ID:
      return webAppOpenModeCompact::fetch(env, p);
    case webAppOpenModeFullSize::ID:
      return webAppOpenModeFullSize::fetch(env, p);
    case webAppOpenModeFullScreen::ID:
      return webAppOpenModeFullScreen::fetch(env, p);
    case webAppOpenParameters::ID:
      return webAppOpenParameters::fetch(env, p);
    case webPageInstantView::ID:
      return webPageInstantView::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void Object::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Object").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass Function::Class;

object_ptr<Function> Function::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case acceptCall::ID:
      return acceptCall::fetch(env, p);
    case acceptTermsOfService::ID:
      return acceptTermsOfService::fetch(env, p);
    case activateStoryStealthMode::ID:
      return activateStoryStealthMode::fetch(env, p);
    case addBotMediaPreview::ID:
      return addBotMediaPreview::fetch(env, p);
    case addChatFolderByInviteLink::ID:
      return addChatFolderByInviteLink::fetch(env, p);
    case addChatMember::ID:
      return addChatMember::fetch(env, p);
    case addChatMembers::ID:
      return addChatMembers::fetch(env, p);
    case addChatToList::ID:
      return addChatToList::fetch(env, p);
    case addContact::ID:
      return addContact::fetch(env, p);
    case addCustomServerLanguagePack::ID:
      return addCustomServerLanguagePack::fetch(env, p);
    case addFavoriteSticker::ID:
      return addFavoriteSticker::fetch(env, p);
    case addFileToDownloads::ID:
      return addFileToDownloads::fetch(env, p);
    case addLocalMessage::ID:
      return addLocalMessage::fetch(env, p);
    case addLogMessage::ID:
      return addLogMessage::fetch(env, p);
    case addMessageReaction::ID:
      return addMessageReaction::fetch(env, p);
    case addNetworkStatistics::ID:
      return addNetworkStatistics::fetch(env, p);
    case addPendingPaidMessageReaction::ID:
      return addPendingPaidMessageReaction::fetch(env, p);
    case addProxy::ID:
      return addProxy::fetch(env, p);
    case addQuickReplyShortcutInlineQueryResultMessage::ID:
      return addQuickReplyShortcutInlineQueryResultMessage::fetch(env, p);
    case addQuickReplyShortcutMessage::ID:
      return addQuickReplyShortcutMessage::fetch(env, p);
    case addQuickReplyShortcutMessageAlbum::ID:
      return addQuickReplyShortcutMessageAlbum::fetch(env, p);
    case addRecentSticker::ID:
      return addRecentSticker::fetch(env, p);
    case addRecentlyFoundChat::ID:
      return addRecentlyFoundChat::fetch(env, p);
    case addSavedAnimation::ID:
      return addSavedAnimation::fetch(env, p);
    case addSavedNotificationSound::ID:
      return addSavedNotificationSound::fetch(env, p);
    case addStickerToSet::ID:
      return addStickerToSet::fetch(env, p);
    case allowBotToSendMessages::ID:
      return allowBotToSendMessages::fetch(env, p);
    case allowUnpaidMessagesFromUser::ID:
      return allowUnpaidMessagesFromUser::fetch(env, p);
    case answerCallbackQuery::ID:
      return answerCallbackQuery::fetch(env, p);
    case answerCustomQuery::ID:
      return answerCustomQuery::fetch(env, p);
    case answerInlineQuery::ID:
      return answerInlineQuery::fetch(env, p);
    case answerPreCheckoutQuery::ID:
      return answerPreCheckoutQuery::fetch(env, p);
    case answerShippingQuery::ID:
      return answerShippingQuery::fetch(env, p);
    case answerWebAppQuery::ID:
      return answerWebAppQuery::fetch(env, p);
    case applyPremiumGiftCode::ID:
      return applyPremiumGiftCode::fetch(env, p);
    case assignStoreTransaction::ID:
      return assignStoreTransaction::fetch(env, p);
    case banChatMember::ID:
      return banChatMember::fetch(env, p);
    case banGroupCallParticipants::ID:
      return banGroupCallParticipants::fetch(env, p);
    case blockMessageSenderFromReplies::ID:
      return blockMessageSenderFromReplies::fetch(env, p);
    case boostChat::ID:
      return boostChat::fetch(env, p);
    case canBotSendMessages::ID:
      return canBotSendMessages::fetch(env, p);
    case canPostStory::ID:
      return canPostStory::fetch(env, p);
    case canPurchaseFromStore::ID:
      return canPurchaseFromStore::fetch(env, p);
    case canSendMessageToUser::ID:
      return canSendMessageToUser::fetch(env, p);
    case canTransferOwnership::ID:
      return canTransferOwnership::fetch(env, p);
    case cancelDownloadFile::ID:
      return cancelDownloadFile::fetch(env, p);
    case cancelPasswordReset::ID:
      return cancelPasswordReset::fetch(env, p);
    case cancelPreliminaryUploadFile::ID:
      return cancelPreliminaryUploadFile::fetch(env, p);
    case cancelRecoveryEmailAddressVerification::ID:
      return cancelRecoveryEmailAddressVerification::fetch(env, p);
    case changeImportedContacts::ID:
      return changeImportedContacts::fetch(env, p);
    case changeStickerSet::ID:
      return changeStickerSet::fetch(env, p);
    case checkAuthenticationBotToken::ID:
      return checkAuthenticationBotToken::fetch(env, p);
    case checkAuthenticationCode::ID:
      return checkAuthenticationCode::fetch(env, p);
    case checkAuthenticationEmailCode::ID:
      return checkAuthenticationEmailCode::fetch(env, p);
    case checkAuthenticationPassword::ID:
      return checkAuthenticationPassword::fetch(env, p);
    case checkAuthenticationPasswordRecoveryCode::ID:
      return checkAuthenticationPasswordRecoveryCode::fetch(env, p);
    case checkAuthenticationPremiumPurchase::ID:
      return checkAuthenticationPremiumPurchase::fetch(env, p);
    case checkChatFolderInviteLink::ID:
      return checkChatFolderInviteLink::fetch(env, p);
    case checkChatInviteLink::ID:
      return checkChatInviteLink::fetch(env, p);
    case checkChatUsername::ID:
      return checkChatUsername::fetch(env, p);
    case checkCreatedPublicChatsLimit::ID:
      return checkCreatedPublicChatsLimit::fetch(env, p);
    case checkEmailAddressVerificationCode::ID:
      return checkEmailAddressVerificationCode::fetch(env, p);
    case checkLoginEmailAddressCode::ID:
      return checkLoginEmailAddressCode::fetch(env, p);
    case checkPasswordRecoveryCode::ID:
      return checkPasswordRecoveryCode::fetch(env, p);
    case checkPhoneNumberCode::ID:
      return checkPhoneNumberCode::fetch(env, p);
    case checkPremiumGiftCode::ID:
      return checkPremiumGiftCode::fetch(env, p);
    case checkQuickReplyShortcutName::ID:
      return checkQuickReplyShortcutName::fetch(env, p);
    case checkRecoveryEmailAddressCode::ID:
      return checkRecoveryEmailAddressCode::fetch(env, p);
    case checkStickerSetName::ID:
      return checkStickerSetName::fetch(env, p);
    case checkWebAppFileDownload::ID:
      return checkWebAppFileDownload::fetch(env, p);
    case cleanFileName::ID:
      return cleanFileName::fetch(env, p);
    case clearAllDraftMessages::ID:
      return clearAllDraftMessages::fetch(env, p);
    case clearAutosaveSettingsExceptions::ID:
      return clearAutosaveSettingsExceptions::fetch(env, p);
    case clearImportedContacts::ID:
      return clearImportedContacts::fetch(env, p);
    case clearRecentEmojiStatuses::ID:
      return clearRecentEmojiStatuses::fetch(env, p);
    case clearRecentReactions::ID:
      return clearRecentReactions::fetch(env, p);
    case clearRecentStickers::ID:
      return clearRecentStickers::fetch(env, p);
    case clearRecentlyFoundChats::ID:
      return clearRecentlyFoundChats::fetch(env, p);
    case clearSearchedForTags::ID:
      return clearSearchedForTags::fetch(env, p);
    case clickAnimatedEmojiMessage::ID:
      return clickAnimatedEmojiMessage::fetch(env, p);
    case clickChatSponsoredMessage::ID:
      return clickChatSponsoredMessage::fetch(env, p);
    case clickPremiumSubscriptionButton::ID:
      return clickPremiumSubscriptionButton::fetch(env, p);
    case close::ID:
      return close::fetch(env, p);
    case closeChat::ID:
      return closeChat::fetch(env, p);
    case closeSecretChat::ID:
      return closeSecretChat::fetch(env, p);
    case closeStory::ID:
      return closeStory::fetch(env, p);
    case closeWebApp::ID:
      return closeWebApp::fetch(env, p);
    case commitPendingPaidMessageReactions::ID:
      return commitPendingPaidMessageReactions::fetch(env, p);
    case confirmQrCodeAuthentication::ID:
      return confirmQrCodeAuthentication::fetch(env, p);
    case confirmSession::ID:
      return confirmSession::fetch(env, p);
    case connectAffiliateProgram::ID:
      return connectAffiliateProgram::fetch(env, p);
    case createBasicGroupChat::ID:
      return createBasicGroupChat::fetch(env, p);
    case createBusinessChatLink::ID:
      return createBusinessChatLink::fetch(env, p);
    case createCall::ID:
      return createCall::fetch(env, p);
    case createChatFolder::ID:
      return createChatFolder::fetch(env, p);
    case createChatFolderInviteLink::ID:
      return createChatFolderInviteLink::fetch(env, p);
    case createChatInviteLink::ID:
      return createChatInviteLink::fetch(env, p);
    case createChatSubscriptionInviteLink::ID:
      return createChatSubscriptionInviteLink::fetch(env, p);
    case createForumTopic::ID:
      return createForumTopic::fetch(env, p);
    case createGroupCall::ID:
      return createGroupCall::fetch(env, p);
    case createInvoiceLink::ID:
      return createInvoiceLink::fetch(env, p);
    case createNewBasicGroupChat::ID:
      return createNewBasicGroupChat::fetch(env, p);
    case createNewSecretChat::ID:
      return createNewSecretChat::fetch(env, p);
    case createNewStickerSet::ID:
      return createNewStickerSet::fetch(env, p);
    case createNewSupergroupChat::ID:
      return createNewSupergroupChat::fetch(env, p);
    case createPrivateChat::ID:
      return createPrivateChat::fetch(env, p);
    case createSecretChat::ID:
      return createSecretChat::fetch(env, p);
    case createSupergroupChat::ID:
      return createSupergroupChat::fetch(env, p);
    case createTemporaryPassword::ID:
      return createTemporaryPassword::fetch(env, p);
    case createVideoChat::ID:
      return createVideoChat::fetch(env, p);
    case declineGroupCallInvitation::ID:
      return declineGroupCallInvitation::fetch(env, p);
    case decryptGroupCallData::ID:
      return decryptGroupCallData::fetch(env, p);
    case deleteAccount::ID:
      return deleteAccount::fetch(env, p);
    case deleteAllCallMessages::ID:
      return deleteAllCallMessages::fetch(env, p);
    case deleteAllRevokedChatInviteLinks::ID:
      return deleteAllRevokedChatInviteLinks::fetch(env, p);
    case deleteBotMediaPreviews::ID:
      return deleteBotMediaPreviews::fetch(env, p);
    case deleteBusinessChatLink::ID:
      return deleteBusinessChatLink::fetch(env, p);
    case deleteBusinessConnectedBot::ID:
      return deleteBusinessConnectedBot::fetch(env, p);
    case deleteBusinessMessages::ID:
      return deleteBusinessMessages::fetch(env, p);
    case deleteBusinessStory::ID:
      return deleteBusinessStory::fetch(env, p);
    case deleteChat::ID:
      return deleteChat::fetch(env, p);
    case deleteChatBackground::ID:
      return deleteChatBackground::fetch(env, p);
    case deleteChatFolder::ID:
      return deleteChatFolder::fetch(env, p);
    case deleteChatFolderInviteLink::ID:
      return deleteChatFolderInviteLink::fetch(env, p);
    case deleteChatHistory::ID:
      return deleteChatHistory::fetch(env, p);
    case deleteChatMessagesByDate::ID:
      return deleteChatMessagesByDate::fetch(env, p);
    case deleteChatMessagesBySender::ID:
      return deleteChatMessagesBySender::fetch(env, p);
    case deleteChatReplyMarkup::ID:
      return deleteChatReplyMarkup::fetch(env, p);
    case deleteCommands::ID:
      return deleteCommands::fetch(env, p);
    case deleteDefaultBackground::ID:
      return deleteDefaultBackground::fetch(env, p);
    case deleteFile::ID:
      return deleteFile::fetch(env, p);
    case deleteForumTopic::ID:
      return deleteForumTopic::fetch(env, p);
    case deleteLanguagePack::ID:
      return deleteLanguagePack::fetch(env, p);
    case deleteMessages::ID:
      return deleteMessages::fetch(env, p);
    case deletePassportElement::ID:
      return deletePassportElement::fetch(env, p);
    case deleteProfilePhoto::ID:
      return deleteProfilePhoto::fetch(env, p);
    case deleteQuickReplyShortcut::ID:
      return deleteQuickReplyShortcut::fetch(env, p);
    case deleteQuickReplyShortcutMessages::ID:
      return deleteQuickReplyShortcutMessages::fetch(env, p);
    case deleteRevokedChatInviteLink::ID:
      return deleteRevokedChatInviteLink::fetch(env, p);
    case deleteSavedCredentials::ID:
      return deleteSavedCredentials::fetch(env, p);
    case deleteSavedMessagesTopicHistory::ID:
      return deleteSavedMessagesTopicHistory::fetch(env, p);
    case deleteSavedMessagesTopicMessagesByDate::ID:
      return deleteSavedMessagesTopicMessagesByDate::fetch(env, p);
    case deleteSavedOrderInfo::ID:
      return deleteSavedOrderInfo::fetch(env, p);
    case deleteStickerSet::ID:
      return deleteStickerSet::fetch(env, p);
    case deleteStory::ID:
      return deleteStory::fetch(env, p);
    case destroy::ID:
      return destroy::fetch(env, p);
    case disableAllSupergroupUsernames::ID:
      return disableAllSupergroupUsernames::fetch(env, p);
    case disableProxy::ID:
      return disableProxy::fetch(env, p);
    case discardCall::ID:
      return discardCall::fetch(env, p);
    case disconnectAffiliateProgram::ID:
      return disconnectAffiliateProgram::fetch(env, p);
    case disconnectAllWebsites::ID:
      return disconnectAllWebsites::fetch(env, p);
    case disconnectWebsite::ID:
      return disconnectWebsite::fetch(env, p);
    case downloadFile::ID:
      return downloadFile::fetch(env, p);
    case editBotMediaPreview::ID:
      return editBotMediaPreview::fetch(env, p);
    case editBusinessChatLink::ID:
      return editBusinessChatLink::fetch(env, p);
    case editBusinessMessageCaption::ID:
      return editBusinessMessageCaption::fetch(env, p);
    case editBusinessMessageLiveLocation::ID:
      return editBusinessMessageLiveLocation::fetch(env, p);
    case editBusinessMessageMedia::ID:
      return editBusinessMessageMedia::fetch(env, p);
    case editBusinessMessageReplyMarkup::ID:
      return editBusinessMessageReplyMarkup::fetch(env, p);
    case editBusinessMessageText::ID:
      return editBusinessMessageText::fetch(env, p);
    case editBusinessStory::ID:
      return editBusinessStory::fetch(env, p);
    case editChatFolder::ID:
      return editChatFolder::fetch(env, p);
    case editChatFolderInviteLink::ID:
      return editChatFolderInviteLink::fetch(env, p);
    case editChatInviteLink::ID:
      return editChatInviteLink::fetch(env, p);
    case editChatSubscriptionInviteLink::ID:
      return editChatSubscriptionInviteLink::fetch(env, p);
    case editCustomLanguagePackInfo::ID:
      return editCustomLanguagePackInfo::fetch(env, p);
    case editForumTopic::ID:
      return editForumTopic::fetch(env, p);
    case editInlineMessageCaption::ID:
      return editInlineMessageCaption::fetch(env, p);
    case editInlineMessageLiveLocation::ID:
      return editInlineMessageLiveLocation::fetch(env, p);
    case editInlineMessageMedia::ID:
      return editInlineMessageMedia::fetch(env, p);
    case editInlineMessageReplyMarkup::ID:
      return editInlineMessageReplyMarkup::fetch(env, p);
    case editInlineMessageText::ID:
      return editInlineMessageText::fetch(env, p);
    case editMessageCaption::ID:
      return editMessageCaption::fetch(env, p);
    case editMessageLiveLocation::ID:
      return editMessageLiveLocation::fetch(env, p);
    case editMessageMedia::ID:
      return editMessageMedia::fetch(env, p);
    case editMessageReplyMarkup::ID:
      return editMessageReplyMarkup::fetch(env, p);
    case editMessageSchedulingState::ID:
      return editMessageSchedulingState::fetch(env, p);
    case editMessageText::ID:
      return editMessageText::fetch(env, p);
    case editProxy::ID:
      return editProxy::fetch(env, p);
    case editQuickReplyMessage::ID:
      return editQuickReplyMessage::fetch(env, p);
    case editStarSubscription::ID:
      return editStarSubscription::fetch(env, p);
    case editStory::ID:
      return editStory::fetch(env, p);
    case editStoryCover::ID:
      return editStoryCover::fetch(env, p);
    case editUserStarSubscription::ID:
      return editUserStarSubscription::fetch(env, p);
    case enableProxy::ID:
      return enableProxy::fetch(env, p);
    case encryptGroupCallData::ID:
      return encryptGroupCallData::fetch(env, p);
    case endGroupCall::ID:
      return endGroupCall::fetch(env, p);
    case endGroupCallRecording::ID:
      return endGroupCallRecording::fetch(env, p);
    case endGroupCallScreenSharing::ID:
      return endGroupCallScreenSharing::fetch(env, p);
    case finishFileGeneration::ID:
      return finishFileGeneration::fetch(env, p);
    case forwardMessages::ID:
      return forwardMessages::fetch(env, p);
    case getAccountTtl::ID:
      return getAccountTtl::fetch(env, p);
    case getActiveSessions::ID:
      return getActiveSessions::fetch(env, p);
    case getAllPassportElements::ID:
      return getAllPassportElements::fetch(env, p);
    case getAllStickerEmojis::ID:
      return getAllStickerEmojis::fetch(env, p);
    case getAnimatedEmoji::ID:
      return getAnimatedEmoji::fetch(env, p);
    case getApplicationConfig::ID:
      return getApplicationConfig::fetch(env, p);
    case getApplicationDownloadLink::ID:
      return getApplicationDownloadLink::fetch(env, p);
    case getArchiveChatListSettings::ID:
      return getArchiveChatListSettings::fetch(env, p);
    case getArchivedStickerSets::ID:
      return getArchivedStickerSets::fetch(env, p);
    case getAttachedStickerSets::ID:
      return getAttachedStickerSets::fetch(env, p);
    case getAttachmentMenuBot::ID:
      return getAttachmentMenuBot::fetch(env, p);
    case getAuthorizationState::ID:
      return getAuthorizationState::fetch(env, p);
    case getAutoDownloadSettingsPresets::ID:
      return getAutoDownloadSettingsPresets::fetch(env, p);
    case getAutosaveSettings::ID:
      return getAutosaveSettings::fetch(env, p);
    case getAvailableChatBoostSlots::ID:
      return getAvailableChatBoostSlots::fetch(env, p);
    case getAvailableGifts::ID:
      return getAvailableGifts::fetch(env, p);
    case getBackgroundUrl::ID:
      return getBackgroundUrl::fetch(env, p);
    case getBankCardInfo::ID:
      return getBankCardInfo::fetch(env, p);
    case getBasicGroup::ID:
      return getBasicGroup::fetch(env, p);
    case getBasicGroupFullInfo::ID:
      return getBasicGroupFullInfo::fetch(env, p);
    case getBlockedMessageSenders::ID:
      return getBlockedMessageSenders::fetch(env, p);
    case getBotInfoDescription::ID:
      return getBotInfoDescription::fetch(env, p);
    case getBotInfoShortDescription::ID:
      return getBotInfoShortDescription::fetch(env, p);
    case getBotMediaPreviewInfo::ID:
      return getBotMediaPreviewInfo::fetch(env, p);
    case getBotMediaPreviews::ID:
      return getBotMediaPreviews::fetch(env, p);
    case getBotName::ID:
      return getBotName::fetch(env, p);
    case getBotSimilarBotCount::ID:
      return getBotSimilarBotCount::fetch(env, p);
    case getBotSimilarBots::ID:
      return getBotSimilarBots::fetch(env, p);
    case getBusinessAccountStarAmount::ID:
      return getBusinessAccountStarAmount::fetch(env, p);
    case getBusinessChatLinkInfo::ID:
      return getBusinessChatLinkInfo::fetch(env, p);
    case getBusinessChatLinks::ID:
      return getBusinessChatLinks::fetch(env, p);
    case getBusinessConnectedBot::ID:
      return getBusinessConnectedBot::fetch(env, p);
    case getBusinessConnection::ID:
      return getBusinessConnection::fetch(env, p);
    case getBusinessFeatures::ID:
      return getBusinessFeatures::fetch(env, p);
    case getCallbackQueryAnswer::ID:
      return getCallbackQueryAnswer::fetch(env, p);
    case getCallbackQueryMessage::ID:
      return getCallbackQueryMessage::fetch(env, p);
    case getChat::ID:
      return getChat::fetch(env, p);
    case getChatActiveStories::ID:
      return getChatActiveStories::fetch(env, p);
    case getChatAdministrators::ID:
      return getChatAdministrators::fetch(env, p);
    case getChatArchivedStories::ID:
      return getChatArchivedStories::fetch(env, p);
    case getChatAvailableMessageSenders::ID:
      return getChatAvailableMessageSenders::fetch(env, p);
    case getChatAvailablePaidMessageReactionSenders::ID:
      return getChatAvailablePaidMessageReactionSenders::fetch(env, p);
    case getChatBoostFeatures::ID:
      return getChatBoostFeatures::fetch(env, p);
    case getChatBoostLevelFeatures::ID:
      return getChatBoostLevelFeatures::fetch(env, p);
    case getChatBoostLink::ID:
      return getChatBoostLink::fetch(env, p);
    case getChatBoostLinkInfo::ID:
      return getChatBoostLinkInfo::fetch(env, p);
    case getChatBoostStatus::ID:
      return getChatBoostStatus::fetch(env, p);
    case getChatBoosts::ID:
      return getChatBoosts::fetch(env, p);
    case getChatEventLog::ID:
      return getChatEventLog::fetch(env, p);
    case getChatFolder::ID:
      return getChatFolder::fetch(env, p);
    case getChatFolderChatCount::ID:
      return getChatFolderChatCount::fetch(env, p);
    case getChatFolderChatsToLeave::ID:
      return getChatFolderChatsToLeave::fetch(env, p);
    case getChatFolderDefaultIconName::ID:
      return getChatFolderDefaultIconName::fetch(env, p);
    case getChatFolderInviteLinks::ID:
      return getChatFolderInviteLinks::fetch(env, p);
    case getChatFolderNewChats::ID:
      return getChatFolderNewChats::fetch(env, p);
    case getChatHistory::ID:
      return getChatHistory::fetch(env, p);
    case getChatInviteLink::ID:
      return getChatInviteLink::fetch(env, p);
    case getChatInviteLinkCounts::ID:
      return getChatInviteLinkCounts::fetch(env, p);
    case getChatInviteLinkMembers::ID:
      return getChatInviteLinkMembers::fetch(env, p);
    case getChatInviteLinks::ID:
      return getChatInviteLinks::fetch(env, p);
    case getChatJoinRequests::ID:
      return getChatJoinRequests::fetch(env, p);
    case getChatListsToAddChat::ID:
      return getChatListsToAddChat::fetch(env, p);
    case getChatMember::ID:
      return getChatMember::fetch(env, p);
    case getChatMessageByDate::ID:
      return getChatMessageByDate::fetch(env, p);
    case getChatMessageCalendar::ID:
      return getChatMessageCalendar::fetch(env, p);
    case getChatMessageCount::ID:
      return getChatMessageCount::fetch(env, p);
    case getChatMessagePosition::ID:
      return getChatMessagePosition::fetch(env, p);
    case getChatNotificationSettingsExceptions::ID:
      return getChatNotificationSettingsExceptions::fetch(env, p);
    case getChatPinnedMessage::ID:
      return getChatPinnedMessage::fetch(env, p);
    case getChatPostedToChatPageStories::ID:
      return getChatPostedToChatPageStories::fetch(env, p);
    case getChatRevenueStatistics::ID:
      return getChatRevenueStatistics::fetch(env, p);
    case getChatRevenueTransactions::ID:
      return getChatRevenueTransactions::fetch(env, p);
    case getChatRevenueWithdrawalUrl::ID:
      return getChatRevenueWithdrawalUrl::fetch(env, p);
    case getChatScheduledMessages::ID:
      return getChatScheduledMessages::fetch(env, p);
    case getChatSimilarChatCount::ID:
      return getChatSimilarChatCount::fetch(env, p);
    case getChatSimilarChats::ID:
      return getChatSimilarChats::fetch(env, p);
    case getChatSparseMessagePositions::ID:
      return getChatSparseMessagePositions::fetch(env, p);
    case getChatSponsoredMessages::ID:
      return getChatSponsoredMessages::fetch(env, p);
    case getChatStatistics::ID:
      return getChatStatistics::fetch(env, p);
    case getChatStoryInteractions::ID:
      return getChatStoryInteractions::fetch(env, p);
    case getChats::ID:
      return getChats::fetch(env, p);
    case getChatsForChatFolderInviteLink::ID:
      return getChatsForChatFolderInviteLink::fetch(env, p);
    case getChatsToPostStories::ID:
      return getChatsToPostStories::fetch(env, p);
    case getCloseFriends::ID:
      return getCloseFriends::fetch(env, p);
    case getCollectibleItemInfo::ID:
      return getCollectibleItemInfo::fetch(env, p);
    case getCommands::ID:
      return getCommands::fetch(env, p);
    case getConnectedAffiliateProgram::ID:
      return getConnectedAffiliateProgram::fetch(env, p);
    case getConnectedAffiliatePrograms::ID:
      return getConnectedAffiliatePrograms::fetch(env, p);
    case getConnectedWebsites::ID:
      return getConnectedWebsites::fetch(env, p);
    case getContacts::ID:
      return getContacts::fetch(env, p);
    case getCountries::ID:
      return getCountries::fetch(env, p);
    case getCountryCode::ID:
      return getCountryCode::fetch(env, p);
    case getCountryFlagEmoji::ID:
      return getCountryFlagEmoji::fetch(env, p);
    case getCreatedPublicChats::ID:
      return getCreatedPublicChats::fetch(env, p);
    case getCurrentState::ID:
      return getCurrentState::fetch(env, p);
    case getCurrentWeather::ID:
      return getCurrentWeather::fetch(env, p);
    case getCustomEmojiReactionAnimations::ID:
      return getCustomEmojiReactionAnimations::fetch(env, p);
    case getCustomEmojiStickers::ID:
      return getCustomEmojiStickers::fetch(env, p);
    case getDatabaseStatistics::ID:
      return getDatabaseStatistics::fetch(env, p);
    case getDeepLinkInfo::ID:
      return getDeepLinkInfo::fetch(env, p);
    case getDefaultBackgroundCustomEmojiStickers::ID:
      return getDefaultBackgroundCustomEmojiStickers::fetch(env, p);
    case getDefaultChatEmojiStatuses::ID:
      return getDefaultChatEmojiStatuses::fetch(env, p);
    case getDefaultChatPhotoCustomEmojiStickers::ID:
      return getDefaultChatPhotoCustomEmojiStickers::fetch(env, p);
    case getDefaultEmojiStatuses::ID:
      return getDefaultEmojiStatuses::fetch(env, p);
    case getDefaultMessageAutoDeleteTime::ID:
      return getDefaultMessageAutoDeleteTime::fetch(env, p);
    case getDefaultProfilePhotoCustomEmojiStickers::ID:
      return getDefaultProfilePhotoCustomEmojiStickers::fetch(env, p);
    case getDisallowedChatEmojiStatuses::ID:
      return getDisallowedChatEmojiStatuses::fetch(env, p);
    case getEmojiCategories::ID:
      return getEmojiCategories::fetch(env, p);
    case getEmojiReaction::ID:
      return getEmojiReaction::fetch(env, p);
    case getEmojiSuggestionsUrl::ID:
      return getEmojiSuggestionsUrl::fetch(env, p);
    case getExternalLink::ID:
      return getExternalLink::fetch(env, p);
    case getExternalLinkInfo::ID:
      return getExternalLinkInfo::fetch(env, p);
    case getFavoriteStickers::ID:
      return getFavoriteStickers::fetch(env, p);
    case getFile::ID:
      return getFile::fetch(env, p);
    case getFileDownloadedPrefixSize::ID:
      return getFileDownloadedPrefixSize::fetch(env, p);
    case getFileExtension::ID:
      return getFileExtension::fetch(env, p);
    case getFileMimeType::ID:
      return getFileMimeType::fetch(env, p);
    case getForumTopic::ID:
      return getForumTopic::fetch(env, p);
    case getForumTopicDefaultIcons::ID:
      return getForumTopicDefaultIcons::fetch(env, p);
    case getForumTopicLink::ID:
      return getForumTopicLink::fetch(env, p);
    case getForumTopics::ID:
      return getForumTopics::fetch(env, p);
    case getGameHighScores::ID:
      return getGameHighScores::fetch(env, p);
    case getGiftUpgradePreview::ID:
      return getGiftUpgradePreview::fetch(env, p);
    case getGiveawayInfo::ID:
      return getGiveawayInfo::fetch(env, p);
    case getGreetingStickers::ID:
      return getGreetingStickers::fetch(env, p);
    case getGrossingWebAppBots::ID:
      return getGrossingWebAppBots::fetch(env, p);
    case getGroupCall::ID:
      return getGroupCall::fetch(env, p);
    case getGroupCallParticipants::ID:
      return getGroupCallParticipants::fetch(env, p);
    case getGroupsInCommon::ID:
      return getGroupsInCommon::fetch(env, p);
    case getImportedContactCount::ID:
      return getImportedContactCount::fetch(env, p);
    case getInactiveSupergroupChats::ID:
      return getInactiveSupergroupChats::fetch(env, p);
    case getInlineGameHighScores::ID:
      return getInlineGameHighScores::fetch(env, p);
    case getInlineQueryResults::ID:
      return getInlineQueryResults::fetch(env, p);
    case getInstalledBackgrounds::ID:
      return getInstalledBackgrounds::fetch(env, p);
    case getInstalledStickerSets::ID:
      return getInstalledStickerSets::fetch(env, p);
    case getInternalLink::ID:
      return getInternalLink::fetch(env, p);
    case getInternalLinkType::ID:
      return getInternalLinkType::fetch(env, p);
    case getJsonString::ID:
      return getJsonString::fetch(env, p);
    case getJsonValue::ID:
      return getJsonValue::fetch(env, p);
    case getKeywordEmojis::ID:
      return getKeywordEmojis::fetch(env, p);
    case getLanguagePackInfo::ID:
      return getLanguagePackInfo::fetch(env, p);
    case getLanguagePackString::ID:
      return getLanguagePackString::fetch(env, p);
    case getLanguagePackStrings::ID:
      return getLanguagePackStrings::fetch(env, p);
    case getLinkPreview::ID:
      return getLinkPreview::fetch(env, p);
    case getLocalizationTargetInfo::ID:
      return getLocalizationTargetInfo::fetch(env, p);
    case getLogStream::ID:
      return getLogStream::fetch(env, p);
    case getLogTagVerbosityLevel::ID:
      return getLogTagVerbosityLevel::fetch(env, p);
    case getLogTags::ID:
      return getLogTags::fetch(env, p);
    case getLogVerbosityLevel::ID:
      return getLogVerbosityLevel::fetch(env, p);
    case getLoginUrl::ID:
      return getLoginUrl::fetch(env, p);
    case getLoginUrlInfo::ID:
      return getLoginUrlInfo::fetch(env, p);
    case getMainWebApp::ID:
      return getMainWebApp::fetch(env, p);
    case getMapThumbnailFile::ID:
      return getMapThumbnailFile::fetch(env, p);
    case getMarkdownText::ID:
      return getMarkdownText::fetch(env, p);
    case getMe::ID:
      return getMe::fetch(env, p);
    case getMenuButton::ID:
      return getMenuButton::fetch(env, p);
    case getMessage::ID:
      return getMessage::fetch(env, p);
    case getMessageAddedReactions::ID:
      return getMessageAddedReactions::fetch(env, p);
    case getMessageAvailableReactions::ID:
      return getMessageAvailableReactions::fetch(env, p);
    case getMessageEffect::ID:
      return getMessageEffect::fetch(env, p);
    case getMessageEmbeddingCode::ID:
      return getMessageEmbeddingCode::fetch(env, p);
    case getMessageFileType::ID:
      return getMessageFileType::fetch(env, p);
    case getMessageImportConfirmationText::ID:
      return getMessageImportConfirmationText::fetch(env, p);
    case getMessageLink::ID:
      return getMessageLink::fetch(env, p);
    case getMessageLinkInfo::ID:
      return getMessageLinkInfo::fetch(env, p);
    case getMessageLocally::ID:
      return getMessageLocally::fetch(env, p);
    case getMessageProperties::ID:
      return getMessageProperties::fetch(env, p);
    case getMessagePublicForwards::ID:
      return getMessagePublicForwards::fetch(env, p);
    case getMessageReadDate::ID:
      return getMessageReadDate::fetch(env, p);
    case getMessageStatistics::ID:
      return getMessageStatistics::fetch(env, p);
    case getMessageThread::ID:
      return getMessageThread::fetch(env, p);
    case getMessageThreadHistory::ID:
      return getMessageThreadHistory::fetch(env, p);
    case getMessageViewers::ID:
      return getMessageViewers::fetch(env, p);
    case getMessages::ID:
      return getMessages::fetch(env, p);
    case getNetworkStatistics::ID:
      return getNetworkStatistics::fetch(env, p);
    case getNewChatPrivacySettings::ID:
      return getNewChatPrivacySettings::fetch(env, p);
    case getOption::ID:
      return getOption::fetch(env, p);
    case getOwnedBots::ID:
      return getOwnedBots::fetch(env, p);
    case getOwnedStickerSets::ID:
      return getOwnedStickerSets::fetch(env, p);
    case getPaidMessageRevenue::ID:
      return getPaidMessageRevenue::fetch(env, p);
    case getPassportAuthorizationForm::ID:
      return getPassportAuthorizationForm::fetch(env, p);
    case getPassportAuthorizationFormAvailableElements::ID:
      return getPassportAuthorizationFormAvailableElements::fetch(env, p);
    case getPassportElement::ID:
      return getPassportElement::fetch(env, p);
    case getPasswordState::ID:
      return getPasswordState::fetch(env, p);
    case getPaymentForm::ID:
      return getPaymentForm::fetch(env, p);
    case getPaymentReceipt::ID:
      return getPaymentReceipt::fetch(env, p);
    case getPhoneNumberInfo::ID:
      return getPhoneNumberInfo::fetch(env, p);
    case getPhoneNumberInfoSync::ID:
      return getPhoneNumberInfoSync::fetch(env, p);
    case getPollVoters::ID:
      return getPollVoters::fetch(env, p);
    case getPreferredCountryLanguage::ID:
      return getPreferredCountryLanguage::fetch(env, p);
    case getPremiumFeatures::ID:
      return getPremiumFeatures::fetch(env, p);
    case getPremiumGiftPaymentOptions::ID:
      return getPremiumGiftPaymentOptions::fetch(env, p);
    case getPremiumGiveawayPaymentOptions::ID:
      return getPremiumGiveawayPaymentOptions::fetch(env, p);
    case getPremiumInfoSticker::ID:
      return getPremiumInfoSticker::fetch(env, p);
    case getPremiumLimit::ID:
      return getPremiumLimit::fetch(env, p);
    case getPremiumState::ID:
      return getPremiumState::fetch(env, p);
    case getPremiumStickerExamples::ID:
      return getPremiumStickerExamples::fetch(env, p);
    case getPremiumStickers::ID:
      return getPremiumStickers::fetch(env, p);
    case getPreparedInlineMessage::ID:
      return getPreparedInlineMessage::fetch(env, p);
    case getProxies::ID:
      return getProxies::fetch(env, p);
    case getProxyLink::ID:
      return getProxyLink::fetch(env, p);
    case getPushReceiverId::ID:
      return getPushReceiverId::fetch(env, p);
    case getReadDatePrivacySettings::ID:
      return getReadDatePrivacySettings::fetch(env, p);
    case getReceivedGift::ID:
      return getReceivedGift::fetch(env, p);
    case getReceivedGifts::ID:
      return getReceivedGifts::fetch(env, p);
    case getRecentEmojiStatuses::ID:
      return getRecentEmojiStatuses::fetch(env, p);
    case getRecentInlineBots::ID:
      return getRecentInlineBots::fetch(env, p);
    case getRecentStickers::ID:
      return getRecentStickers::fetch(env, p);
    case getRecentlyOpenedChats::ID:
      return getRecentlyOpenedChats::fetch(env, p);
    case getRecentlyVisitedTMeUrls::ID:
      return getRecentlyVisitedTMeUrls::fetch(env, p);
    case getRecommendedChatFolders::ID:
      return getRecommendedChatFolders::fetch(env, p);
    case getRecommendedChats::ID:
      return getRecommendedChats::fetch(env, p);
    case getRecoveryEmailAddress::ID:
      return getRecoveryEmailAddress::fetch(env, p);
    case getRemoteFile::ID:
      return getRemoteFile::fetch(env, p);
    case getRepliedMessage::ID:
      return getRepliedMessage::fetch(env, p);
    case getSavedAnimations::ID:
      return getSavedAnimations::fetch(env, p);
    case getSavedMessagesTags::ID:
      return getSavedMessagesTags::fetch(env, p);
    case getSavedMessagesTopicHistory::ID:
      return getSavedMessagesTopicHistory::fetch(env, p);
    case getSavedMessagesTopicMessageByDate::ID:
      return getSavedMessagesTopicMessageByDate::fetch(env, p);
    case getSavedNotificationSound::ID:
      return getSavedNotificationSound::fetch(env, p);
    case getSavedNotificationSounds::ID:
      return getSavedNotificationSounds::fetch(env, p);
    case getSavedOrderInfo::ID:
      return getSavedOrderInfo::fetch(env, p);
    case getScopeNotificationSettings::ID:
      return getScopeNotificationSettings::fetch(env, p);
    case getSearchSponsoredChats::ID:
      return getSearchSponsoredChats::fetch(env, p);
    case getSearchedForTags::ID:
      return getSearchedForTags::fetch(env, p);
    case getSecretChat::ID:
      return getSecretChat::fetch(env, p);
    case getStarAdAccountUrl::ID:
      return getStarAdAccountUrl::fetch(env, p);
    case getStarGiftPaymentOptions::ID:
      return getStarGiftPaymentOptions::fetch(env, p);
    case getStarGiveawayPaymentOptions::ID:
      return getStarGiveawayPaymentOptions::fetch(env, p);
    case getStarPaymentOptions::ID:
      return getStarPaymentOptions::fetch(env, p);
    case getStarRevenueStatistics::ID:
      return getStarRevenueStatistics::fetch(env, p);
    case getStarSubscriptions::ID:
      return getStarSubscriptions::fetch(env, p);
    case getStarTransactions::ID:
      return getStarTransactions::fetch(env, p);
    case getStarWithdrawalUrl::ID:
      return getStarWithdrawalUrl::fetch(env, p);
    case getStatisticalGraph::ID:
      return getStatisticalGraph::fetch(env, p);
    case getStickerEmojis::ID:
      return getStickerEmojis::fetch(env, p);
    case getStickerOutline::ID:
      return getStickerOutline::fetch(env, p);
    case getStickerSet::ID:
      return getStickerSet::fetch(env, p);
    case getStickerSetName::ID:
      return getStickerSetName::fetch(env, p);
    case getStickers::ID:
      return getStickers::fetch(env, p);
    case getStorageStatistics::ID:
      return getStorageStatistics::fetch(env, p);
    case getStorageStatisticsFast::ID:
      return getStorageStatisticsFast::fetch(env, p);
    case getStory::ID:
      return getStory::fetch(env, p);
    case getStoryAvailableReactions::ID:
      return getStoryAvailableReactions::fetch(env, p);
    case getStoryInteractions::ID:
      return getStoryInteractions::fetch(env, p);
    case getStoryNotificationSettingsExceptions::ID:
      return getStoryNotificationSettingsExceptions::fetch(env, p);
    case getStoryPublicForwards::ID:
      return getStoryPublicForwards::fetch(env, p);
    case getStoryStatistics::ID:
      return getStoryStatistics::fetch(env, p);
    case getSuggestedFileName::ID:
      return getSuggestedFileName::fetch(env, p);
    case getSuggestedStickerSetName::ID:
      return getSuggestedStickerSetName::fetch(env, p);
    case getSuitableDiscussionChats::ID:
      return getSuitableDiscussionChats::fetch(env, p);
    case getSuitablePersonalChats::ID:
      return getSuitablePersonalChats::fetch(env, p);
    case getSupergroup::ID:
      return getSupergroup::fetch(env, p);
    case getSupergroupFullInfo::ID:
      return getSupergroupFullInfo::fetch(env, p);
    case getSupergroupMembers::ID:
      return getSupergroupMembers::fetch(env, p);
    case getSupportName::ID:
      return getSupportName::fetch(env, p);
    case getSupportUser::ID:
      return getSupportUser::fetch(env, p);
    case getTemporaryPasswordState::ID:
      return getTemporaryPasswordState::fetch(env, p);
    case getTextEntities::ID:
      return getTextEntities::fetch(env, p);
    case getThemeParametersJsonString::ID:
      return getThemeParametersJsonString::fetch(env, p);
    case getThemedChatEmojiStatuses::ID:
      return getThemedChatEmojiStatuses::fetch(env, p);
    case getThemedEmojiStatuses::ID:
      return getThemedEmojiStatuses::fetch(env, p);
    case getTimeZones::ID:
      return getTimeZones::fetch(env, p);
    case getTopChats::ID:
      return getTopChats::fetch(env, p);
    case getTrendingStickerSets::ID:
      return getTrendingStickerSets::fetch(env, p);
    case getUpgradedGift::ID:
      return getUpgradedGift::fetch(env, p);
    case getUpgradedGiftEmojiStatuses::ID:
      return getUpgradedGiftEmojiStatuses::fetch(env, p);
    case getUpgradedGiftWithdrawalUrl::ID:
      return getUpgradedGiftWithdrawalUrl::fetch(env, p);
    case getUser::ID:
      return getUser::fetch(env, p);
    case getUserChatBoosts::ID:
      return getUserChatBoosts::fetch(env, p);
    case getUserFullInfo::ID:
      return getUserFullInfo::fetch(env, p);
    case getUserLink::ID:
      return getUserLink::fetch(env, p);
    case getUserPrivacySettingRules::ID:
      return getUserPrivacySettingRules::fetch(env, p);
    case getUserProfilePhotos::ID:
      return getUserProfilePhotos::fetch(env, p);
    case getUserSupportInfo::ID:
      return getUserSupportInfo::fetch(env, p);
    case getVideoChatAvailableParticipants::ID:
      return getVideoChatAvailableParticipants::fetch(env, p);
    case getVideoChatInviteLink::ID:
      return getVideoChatInviteLink::fetch(env, p);
    case getVideoChatRtmpUrl::ID:
      return getVideoChatRtmpUrl::fetch(env, p);
    case getVideoChatStreamSegment::ID:
      return getVideoChatStreamSegment::fetch(env, p);
    case getVideoChatStreams::ID:
      return getVideoChatStreams::fetch(env, p);
    case getWebAppLinkUrl::ID:
      return getWebAppLinkUrl::fetch(env, p);
    case getWebAppPlaceholder::ID:
      return getWebAppPlaceholder::fetch(env, p);
    case getWebAppUrl::ID:
      return getWebAppUrl::fetch(env, p);
    case getWebPageInstantView::ID:
      return getWebPageInstantView::fetch(env, p);
    case giftPremiumWithStars::ID:
      return giftPremiumWithStars::fetch(env, p);
    case hideContactCloseBirthdays::ID:
      return hideContactCloseBirthdays::fetch(env, p);
    case hideSuggestedAction::ID:
      return hideSuggestedAction::fetch(env, p);
    case importContacts::ID:
      return importContacts::fetch(env, p);
    case importMessages::ID:
      return importMessages::fetch(env, p);
    case inviteGroupCallParticipant::ID:
      return inviteGroupCallParticipant::fetch(env, p);
    case inviteVideoChatParticipants::ID:
      return inviteVideoChatParticipants::fetch(env, p);
    case joinChat::ID:
      return joinChat::fetch(env, p);
    case joinChatByInviteLink::ID:
      return joinChatByInviteLink::fetch(env, p);
    case joinGroupCall::ID:
      return joinGroupCall::fetch(env, p);
    case joinVideoChat::ID:
      return joinVideoChat::fetch(env, p);
    case launchPrepaidGiveaway::ID:
      return launchPrepaidGiveaway::fetch(env, p);
    case leaveChat::ID:
      return leaveChat::fetch(env, p);
    case leaveGroupCall::ID:
      return leaveGroupCall::fetch(env, p);
    case loadActiveStories::ID:
      return loadActiveStories::fetch(env, p);
    case loadChats::ID:
      return loadChats::fetch(env, p);
    case loadGroupCallParticipants::ID:
      return loadGroupCallParticipants::fetch(env, p);
    case loadQuickReplyShortcutMessages::ID:
      return loadQuickReplyShortcutMessages::fetch(env, p);
    case loadQuickReplyShortcuts::ID:
      return loadQuickReplyShortcuts::fetch(env, p);
    case loadSavedMessagesTopics::ID:
      return loadSavedMessagesTopics::fetch(env, p);
    case logOut::ID:
      return logOut::fetch(env, p);
    case openBotSimilarBot::ID:
      return openBotSimilarBot::fetch(env, p);
    case openChat::ID:
      return openChat::fetch(env, p);
    case openChatSimilarChat::ID:
      return openChatSimilarChat::fetch(env, p);
    case openMessageContent::ID:
      return openMessageContent::fetch(env, p);
    case openSponsoredChat::ID:
      return openSponsoredChat::fetch(env, p);
    case openStory::ID:
      return openStory::fetch(env, p);
    case openWebApp::ID:
      return openWebApp::fetch(env, p);
    case optimizeStorage::ID:
      return optimizeStorage::fetch(env, p);
    case parseMarkdown::ID:
      return parseMarkdown::fetch(env, p);
    case parseTextEntities::ID:
      return parseTextEntities::fetch(env, p);
    case pinChatMessage::ID:
      return pinChatMessage::fetch(env, p);
    case pingProxy::ID:
      return pingProxy::fetch(env, p);
    case postStory::ID:
      return postStory::fetch(env, p);
    case preliminaryUploadFile::ID:
      return preliminaryUploadFile::fetch(env, p);
    case processChatFolderNewChats::ID:
      return processChatFolderNewChats::fetch(env, p);
    case processChatJoinRequest::ID:
      return processChatJoinRequest::fetch(env, p);
    case processChatJoinRequests::ID:
      return processChatJoinRequests::fetch(env, p);
    case processPushNotification::ID:
      return processPushNotification::fetch(env, p);
    case rateSpeechRecognition::ID:
      return rateSpeechRecognition::fetch(env, p);
    case readAllChatMentions::ID:
      return readAllChatMentions::fetch(env, p);
    case readAllChatReactions::ID:
      return readAllChatReactions::fetch(env, p);
    case readAllMessageThreadMentions::ID:
      return readAllMessageThreadMentions::fetch(env, p);
    case readAllMessageThreadReactions::ID:
      return readAllMessageThreadReactions::fetch(env, p);
    case readBusinessMessage::ID:
      return readBusinessMessage::fetch(env, p);
    case readChatList::ID:
      return readChatList::fetch(env, p);
    case readFilePart::ID:
      return readFilePart::fetch(env, p);
    case readdQuickReplyShortcutMessages::ID:
      return readdQuickReplyShortcutMessages::fetch(env, p);
    case recognizeSpeech::ID:
      return recognizeSpeech::fetch(env, p);
    case recoverAuthenticationPassword::ID:
      return recoverAuthenticationPassword::fetch(env, p);
    case recoverPassword::ID:
      return recoverPassword::fetch(env, p);
    case refundStarPayment::ID:
      return refundStarPayment::fetch(env, p);
    case registerDevice::ID:
      return registerDevice::fetch(env, p);
    case registerUser::ID:
      return registerUser::fetch(env, p);
    case removeAllFilesFromDownloads::ID:
      return removeAllFilesFromDownloads::fetch(env, p);
    case removeBusinessConnectedBotFromChat::ID:
      return removeBusinessConnectedBotFromChat::fetch(env, p);
    case removeChatActionBar::ID:
      return removeChatActionBar::fetch(env, p);
    case removeContacts::ID:
      return removeContacts::fetch(env, p);
    case removeFavoriteSticker::ID:
      return removeFavoriteSticker::fetch(env, p);
    case removeFileFromDownloads::ID:
      return removeFileFromDownloads::fetch(env, p);
    case removeInstalledBackground::ID:
      return removeInstalledBackground::fetch(env, p);
    case removeMessageReaction::ID:
      return removeMessageReaction::fetch(env, p);
    case removeMessageSenderBotVerification::ID:
      return removeMessageSenderBotVerification::fetch(env, p);
    case removeNotification::ID:
      return removeNotification::fetch(env, p);
    case removeNotificationGroup::ID:
      return removeNotificationGroup::fetch(env, p);
    case removePendingPaidMessageReactions::ID:
      return removePendingPaidMessageReactions::fetch(env, p);
    case removeProxy::ID:
      return removeProxy::fetch(env, p);
    case removeRecentHashtag::ID:
      return removeRecentHashtag::fetch(env, p);
    case removeRecentSticker::ID:
      return removeRecentSticker::fetch(env, p);
    case removeRecentlyFoundChat::ID:
      return removeRecentlyFoundChat::fetch(env, p);
    case removeSavedAnimation::ID:
      return removeSavedAnimation::fetch(env, p);
    case removeSavedNotificationSound::ID:
      return removeSavedNotificationSound::fetch(env, p);
    case removeSearchedForTag::ID:
      return removeSearchedForTag::fetch(env, p);
    case removeStickerFromSet::ID:
      return removeStickerFromSet::fetch(env, p);
    case removeTopChat::ID:
      return removeTopChat::fetch(env, p);
    case reorderActiveUsernames::ID:
      return reorderActiveUsernames::fetch(env, p);
    case reorderBotActiveUsernames::ID:
      return reorderBotActiveUsernames::fetch(env, p);
    case reorderBotMediaPreviews::ID:
      return reorderBotMediaPreviews::fetch(env, p);
    case reorderChatFolders::ID:
      return reorderChatFolders::fetch(env, p);
    case reorderInstalledStickerSets::ID:
      return reorderInstalledStickerSets::fetch(env, p);
    case reorderQuickReplyShortcuts::ID:
      return reorderQuickReplyShortcuts::fetch(env, p);
    case reorderSupergroupActiveUsernames::ID:
      return reorderSupergroupActiveUsernames::fetch(env, p);
    case replacePrimaryChatInviteLink::ID:
      return replacePrimaryChatInviteLink::fetch(env, p);
    case replaceStickerInSet::ID:
      return replaceStickerInSet::fetch(env, p);
    case replaceVideoChatRtmpUrl::ID:
      return replaceVideoChatRtmpUrl::fetch(env, p);
    case reportAuthenticationCodeMissing::ID:
      return reportAuthenticationCodeMissing::fetch(env, p);
    case reportChat::ID:
      return reportChat::fetch(env, p);
    case reportChatPhoto::ID:
      return reportChatPhoto::fetch(env, p);
    case reportChatSponsoredMessage::ID:
      return reportChatSponsoredMessage::fetch(env, p);
    case reportMessageReactions::ID:
      return reportMessageReactions::fetch(env, p);
    case reportPhoneNumberCodeMissing::ID:
      return reportPhoneNumberCodeMissing::fetch(env, p);
    case reportSponsoredChat::ID:
      return reportSponsoredChat::fetch(env, p);
    case reportStory::ID:
      return reportStory::fetch(env, p);
    case reportSupergroupAntiSpamFalsePositive::ID:
      return reportSupergroupAntiSpamFalsePositive::fetch(env, p);
    case reportSupergroupSpam::ID:
      return reportSupergroupSpam::fetch(env, p);
    case requestAuthenticationPasswordRecovery::ID:
      return requestAuthenticationPasswordRecovery::fetch(env, p);
    case requestPasswordRecovery::ID:
      return requestPasswordRecovery::fetch(env, p);
    case requestQrCodeAuthentication::ID:
      return requestQrCodeAuthentication::fetch(env, p);
    case resendAuthenticationCode::ID:
      return resendAuthenticationCode::fetch(env, p);
    case resendEmailAddressVerificationCode::ID:
      return resendEmailAddressVerificationCode::fetch(env, p);
    case resendLoginEmailAddressCode::ID:
      return resendLoginEmailAddressCode::fetch(env, p);
    case resendMessages::ID:
      return resendMessages::fetch(env, p);
    case resendPhoneNumberCode::ID:
      return resendPhoneNumberCode::fetch(env, p);
    case resendRecoveryEmailAddressCode::ID:
      return resendRecoveryEmailAddressCode::fetch(env, p);
    case resetAllNotificationSettings::ID:
      return resetAllNotificationSettings::fetch(env, p);
    case resetAuthenticationEmailAddress::ID:
      return resetAuthenticationEmailAddress::fetch(env, p);
    case resetInstalledBackgrounds::ID:
      return resetInstalledBackgrounds::fetch(env, p);
    case resetNetworkStatistics::ID:
      return resetNetworkStatistics::fetch(env, p);
    case resetPassword::ID:
      return resetPassword::fetch(env, p);
    case reuseStarSubscription::ID:
      return reuseStarSubscription::fetch(env, p);
    case revokeChatInviteLink::ID:
      return revokeChatInviteLink::fetch(env, p);
    case revokeGroupCallInviteLink::ID:
      return revokeGroupCallInviteLink::fetch(env, p);
    case saveApplicationLogEvent::ID:
      return saveApplicationLogEvent::fetch(env, p);
    case savePreparedInlineMessage::ID:
      return savePreparedInlineMessage::fetch(env, p);
    case searchAffiliatePrograms::ID:
      return searchAffiliatePrograms::fetch(env, p);
    case searchBackground::ID:
      return searchBackground::fetch(env, p);
    case searchCallMessages::ID:
      return searchCallMessages::fetch(env, p);
    case searchChatAffiliateProgram::ID:
      return searchChatAffiliateProgram::fetch(env, p);
    case searchChatMembers::ID:
      return searchChatMembers::fetch(env, p);
    case searchChatMessages::ID:
      return searchChatMessages::fetch(env, p);
    case searchChatRecentLocationMessages::ID:
      return searchChatRecentLocationMessages::fetch(env, p);
    case searchChats::ID:
      return searchChats::fetch(env, p);
    case searchChatsOnServer::ID:
      return searchChatsOnServer::fetch(env, p);
    case searchContacts::ID:
      return searchContacts::fetch(env, p);
    case searchEmojis::ID:
      return searchEmojis::fetch(env, p);
    case searchFileDownloads::ID:
      return searchFileDownloads::fetch(env, p);
    case searchGiftsForResale::ID:
      return searchGiftsForResale::fetch(env, p);
    case searchHashtags::ID:
      return searchHashtags::fetch(env, p);
    case searchInstalledStickerSets::ID:
      return searchInstalledStickerSets::fetch(env, p);
    case searchMessages::ID:
      return searchMessages::fetch(env, p);
    case searchOutgoingDocumentMessages::ID:
      return searchOutgoingDocumentMessages::fetch(env, p);
    case searchPublicChat::ID:
      return searchPublicChat::fetch(env, p);
    case searchPublicChats::ID:
      return searchPublicChats::fetch(env, p);
    case searchPublicMessagesByTag::ID:
      return searchPublicMessagesByTag::fetch(env, p);
    case searchPublicStoriesByLocation::ID:
      return searchPublicStoriesByLocation::fetch(env, p);
    case searchPublicStoriesByTag::ID:
      return searchPublicStoriesByTag::fetch(env, p);
    case searchPublicStoriesByVenue::ID:
      return searchPublicStoriesByVenue::fetch(env, p);
    case searchQuote::ID:
      return searchQuote::fetch(env, p);
    case searchRecentlyFoundChats::ID:
      return searchRecentlyFoundChats::fetch(env, p);
    case searchSavedMessages::ID:
      return searchSavedMessages::fetch(env, p);
    case searchSecretMessages::ID:
      return searchSecretMessages::fetch(env, p);
    case searchStickerSet::ID:
      return searchStickerSet::fetch(env, p);
    case searchStickerSets::ID:
      return searchStickerSets::fetch(env, p);
    case searchStickers::ID:
      return searchStickers::fetch(env, p);
    case searchStringsByPrefix::ID:
      return searchStringsByPrefix::fetch(env, p);
    case searchUserByPhoneNumber::ID:
      return searchUserByPhoneNumber::fetch(env, p);
    case searchUserByToken::ID:
      return searchUserByToken::fetch(env, p);
    case searchWebApp::ID:
      return searchWebApp::fetch(env, p);
    case sellGift::ID:
      return sellGift::fetch(env, p);
    case sendAuthenticationFirebaseSms::ID:
      return sendAuthenticationFirebaseSms::fetch(env, p);
    case sendBotStartMessage::ID:
      return sendBotStartMessage::fetch(env, p);
    case sendBusinessMessage::ID:
      return sendBusinessMessage::fetch(env, p);
    case sendBusinessMessageAlbum::ID:
      return sendBusinessMessageAlbum::fetch(env, p);
    case sendCallDebugInformation::ID:
      return sendCallDebugInformation::fetch(env, p);
    case sendCallLog::ID:
      return sendCallLog::fetch(env, p);
    case sendCallRating::ID:
      return sendCallRating::fetch(env, p);
    case sendCallSignalingData::ID:
      return sendCallSignalingData::fetch(env, p);
    case sendChatAction::ID:
      return sendChatAction::fetch(env, p);
    case sendCustomRequest::ID:
      return sendCustomRequest::fetch(env, p);
    case sendEmailAddressVerificationCode::ID:
      return sendEmailAddressVerificationCode::fetch(env, p);
    case sendGift::ID:
      return sendGift::fetch(env, p);
    case sendInlineQueryResultMessage::ID:
      return sendInlineQueryResultMessage::fetch(env, p);
    case sendMessage::ID:
      return sendMessage::fetch(env, p);
    case sendMessageAlbum::ID:
      return sendMessageAlbum::fetch(env, p);
    case sendPassportAuthorizationForm::ID:
      return sendPassportAuthorizationForm::fetch(env, p);
    case sendPaymentForm::ID:
      return sendPaymentForm::fetch(env, p);
    case sendPhoneNumberCode::ID:
      return sendPhoneNumberCode::fetch(env, p);
    case sendPhoneNumberFirebaseSms::ID:
      return sendPhoneNumberFirebaseSms::fetch(env, p);
    case sendQuickReplyShortcutMessages::ID:
      return sendQuickReplyShortcutMessages::fetch(env, p);
    case sendResoldGift::ID:
      return sendResoldGift::fetch(env, p);
    case sendWebAppCustomRequest::ID:
      return sendWebAppCustomRequest::fetch(env, p);
    case sendWebAppData::ID:
      return sendWebAppData::fetch(env, p);
    case setAccentColor::ID:
      return setAccentColor::fetch(env, p);
    case setAccountTtl::ID:
      return setAccountTtl::fetch(env, p);
    case setAlarm::ID:
      return setAlarm::fetch(env, p);
    case setApplicationVerificationToken::ID:
      return setApplicationVerificationToken::fetch(env, p);
    case setArchiveChatListSettings::ID:
      return setArchiveChatListSettings::fetch(env, p);
    case setAuthenticationEmailAddress::ID:
      return setAuthenticationEmailAddress::fetch(env, p);
    case setAuthenticationPhoneNumber::ID:
      return setAuthenticationPhoneNumber::fetch(env, p);
    case setAuthenticationPremiumPurchaseTransaction::ID:
      return setAuthenticationPremiumPurchaseTransaction::fetch(env, p);
    case setAutoDownloadSettings::ID:
      return setAutoDownloadSettings::fetch(env, p);
    case setAutosaveSettings::ID:
      return setAutosaveSettings::fetch(env, p);
    case setBio::ID:
      return setBio::fetch(env, p);
    case setBirthdate::ID:
      return setBirthdate::fetch(env, p);
    case setBotInfoDescription::ID:
      return setBotInfoDescription::fetch(env, p);
    case setBotInfoShortDescription::ID:
      return setBotInfoShortDescription::fetch(env, p);
    case setBotName::ID:
      return setBotName::fetch(env, p);
    case setBotProfilePhoto::ID:
      return setBotProfilePhoto::fetch(env, p);
    case setBotUpdatesStatus::ID:
      return setBotUpdatesStatus::fetch(env, p);
    case setBusinessAccountBio::ID:
      return setBusinessAccountBio::fetch(env, p);
    case setBusinessAccountGiftSettings::ID:
      return setBusinessAccountGiftSettings::fetch(env, p);
    case setBusinessAccountName::ID:
      return setBusinessAccountName::fetch(env, p);
    case setBusinessAccountProfilePhoto::ID:
      return setBusinessAccountProfilePhoto::fetch(env, p);
    case setBusinessAccountUsername::ID:
      return setBusinessAccountUsername::fetch(env, p);
    case setBusinessAwayMessageSettings::ID:
      return setBusinessAwayMessageSettings::fetch(env, p);
    case setBusinessConnectedBot::ID:
      return setBusinessConnectedBot::fetch(env, p);
    case setBusinessGreetingMessageSettings::ID:
      return setBusinessGreetingMessageSettings::fetch(env, p);
    case setBusinessLocation::ID:
      return setBusinessLocation::fetch(env, p);
    case setBusinessMessageIsPinned::ID:
      return setBusinessMessageIsPinned::fetch(env, p);
    case setBusinessOpeningHours::ID:
      return setBusinessOpeningHours::fetch(env, p);
    case setBusinessStartPage::ID:
      return setBusinessStartPage::fetch(env, p);
    case setChatAccentColor::ID:
      return setChatAccentColor::fetch(env, p);
    case setChatActiveStoriesList::ID:
      return setChatActiveStoriesList::fetch(env, p);
    case setChatAffiliateProgram::ID:
      return setChatAffiliateProgram::fetch(env, p);
    case setChatAvailableReactions::ID:
      return setChatAvailableReactions::fetch(env, p);
    case setChatBackground::ID:
      return setChatBackground::fetch(env, p);
    case setChatClientData::ID:
      return setChatClientData::fetch(env, p);
    case setChatDescription::ID:
      return setChatDescription::fetch(env, p);
    case setChatDiscussionGroup::ID:
      return setChatDiscussionGroup::fetch(env, p);
    case setChatDraftMessage::ID:
      return setChatDraftMessage::fetch(env, p);
    case setChatEmojiStatus::ID:
      return setChatEmojiStatus::fetch(env, p);
    case setChatLocation::ID:
      return setChatLocation::fetch(env, p);
    case setChatMemberStatus::ID:
      return setChatMemberStatus::fetch(env, p);
    case setChatMessageAutoDeleteTime::ID:
      return setChatMessageAutoDeleteTime::fetch(env, p);
    case setChatMessageSender::ID:
      return setChatMessageSender::fetch(env, p);
    case setChatNotificationSettings::ID:
      return setChatNotificationSettings::fetch(env, p);
    case setChatPaidMessageStarCount::ID:
      return setChatPaidMessageStarCount::fetch(env, p);
    case setChatPermissions::ID:
      return setChatPermissions::fetch(env, p);
    case setChatPhoto::ID:
      return setChatPhoto::fetch(env, p);
    case setChatPinnedStories::ID:
      return setChatPinnedStories::fetch(env, p);
    case setChatProfileAccentColor::ID:
      return setChatProfileAccentColor::fetch(env, p);
    case setChatSlowModeDelay::ID:
      return setChatSlowModeDelay::fetch(env, p);
    case setChatTheme::ID:
      return setChatTheme::fetch(env, p);
    case setChatTitle::ID:
      return setChatTitle::fetch(env, p);
    case setCloseFriends::ID:
      return setCloseFriends::fetch(env, p);
    case setCommands::ID:
      return setCommands::fetch(env, p);
    case setCustomEmojiStickerSetThumbnail::ID:
      return setCustomEmojiStickerSetThumbnail::fetch(env, p);
    case setCustomLanguagePack::ID:
      return setCustomLanguagePack::fetch(env, p);
    case setCustomLanguagePackString::ID:
      return setCustomLanguagePackString::fetch(env, p);
    case setDatabaseEncryptionKey::ID:
      return setDatabaseEncryptionKey::fetch(env, p);
    case setDefaultBackground::ID:
      return setDefaultBackground::fetch(env, p);
    case setDefaultChannelAdministratorRights::ID:
      return setDefaultChannelAdministratorRights::fetch(env, p);
    case setDefaultGroupAdministratorRights::ID:
      return setDefaultGroupAdministratorRights::fetch(env, p);
    case setDefaultMessageAutoDeleteTime::ID:
      return setDefaultMessageAutoDeleteTime::fetch(env, p);
    case setDefaultReactionType::ID:
      return setDefaultReactionType::fetch(env, p);
    case setEmojiStatus::ID:
      return setEmojiStatus::fetch(env, p);
    case setFileGenerationProgress::ID:
      return setFileGenerationProgress::fetch(env, p);
    case setForumTopicNotificationSettings::ID:
      return setForumTopicNotificationSettings::fetch(env, p);
    case setGameScore::ID:
      return setGameScore::fetch(env, p);
    case setGiftResalePrice::ID:
      return setGiftResalePrice::fetch(env, p);
    case setGiftSettings::ID:
      return setGiftSettings::fetch(env, p);
    case setGroupCallParticipantIsSpeaking::ID:
      return setGroupCallParticipantIsSpeaking::fetch(env, p);
    case setGroupCallParticipantVolumeLevel::ID:
      return setGroupCallParticipantVolumeLevel::fetch(env, p);
    case setInactiveSessionTtl::ID:
      return setInactiveSessionTtl::fetch(env, p);
    case setInlineGameScore::ID:
      return setInlineGameScore::fetch(env, p);
    case setLogStream::ID:
      return setLogStream::fetch(env, p);
    case setLogTagVerbosityLevel::ID:
      return setLogTagVerbosityLevel::fetch(env, p);
    case setLogVerbosityLevel::ID:
      return setLogVerbosityLevel::fetch(env, p);
    case setLoginEmailAddress::ID:
      return setLoginEmailAddress::fetch(env, p);
    case setMenuButton::ID:
      return setMenuButton::fetch(env, p);
    case setMessageFactCheck::ID:
      return setMessageFactCheck::fetch(env, p);
    case setMessageReactions::ID:
      return setMessageReactions::fetch(env, p);
    case setMessageSenderBlockList::ID:
      return setMessageSenderBlockList::fetch(env, p);
    case setMessageSenderBotVerification::ID:
      return setMessageSenderBotVerification::fetch(env, p);
    case setName::ID:
      return setName::fetch(env, p);
    case setNetworkType::ID:
      return setNetworkType::fetch(env, p);
    case setNewChatPrivacySettings::ID:
      return setNewChatPrivacySettings::fetch(env, p);
    case setOption::ID:
      return setOption::fetch(env, p);
    case setPaidMessageReactionType::ID:
      return setPaidMessageReactionType::fetch(env, p);
    case setPassportElement::ID:
      return setPassportElement::fetch(env, p);
    case setPassportElementErrors::ID:
      return setPassportElementErrors::fetch(env, p);
    case setPassword::ID:
      return setPassword::fetch(env, p);
    case setPersonalChat::ID:
      return setPersonalChat::fetch(env, p);
    case setPinnedChats::ID:
      return setPinnedChats::fetch(env, p);
    case setPinnedForumTopics::ID:
      return setPinnedForumTopics::fetch(env, p);
    case setPinnedGifts::ID:
      return setPinnedGifts::fetch(env, p);
    case setPinnedSavedMessagesTopics::ID:
      return setPinnedSavedMessagesTopics::fetch(env, p);
    case setPollAnswer::ID:
      return setPollAnswer::fetch(env, p);
    case setProfileAccentColor::ID:
      return setProfileAccentColor::fetch(env, p);
    case setProfilePhoto::ID:
      return setProfilePhoto::fetch(env, p);
    case setQuickReplyShortcutName::ID:
      return setQuickReplyShortcutName::fetch(env, p);
    case setReactionNotificationSettings::ID:
      return setReactionNotificationSettings::fetch(env, p);
    case setReadDatePrivacySettings::ID:
      return setReadDatePrivacySettings::fetch(env, p);
    case setRecoveryEmailAddress::ID:
      return setRecoveryEmailAddress::fetch(env, p);
    case setSavedMessagesTagLabel::ID:
      return setSavedMessagesTagLabel::fetch(env, p);
    case setScopeNotificationSettings::ID:
      return setScopeNotificationSettings::fetch(env, p);
    case setStickerEmojis::ID:
      return setStickerEmojis::fetch(env, p);
    case setStickerKeywords::ID:
      return setStickerKeywords::fetch(env, p);
    case setStickerMaskPosition::ID:
      return setStickerMaskPosition::fetch(env, p);
    case setStickerPositionInSet::ID:
      return setStickerPositionInSet::fetch(env, p);
    case setStickerSetThumbnail::ID:
      return setStickerSetThumbnail::fetch(env, p);
    case setStickerSetTitle::ID:
      return setStickerSetTitle::fetch(env, p);
    case setStoryPrivacySettings::ID:
      return setStoryPrivacySettings::fetch(env, p);
    case setStoryReaction::ID:
      return setStoryReaction::fetch(env, p);
    case setSupergroupCustomEmojiStickerSet::ID:
      return setSupergroupCustomEmojiStickerSet::fetch(env, p);
    case setSupergroupStickerSet::ID:
      return setSupergroupStickerSet::fetch(env, p);
    case setSupergroupUnrestrictBoostCount::ID:
      return setSupergroupUnrestrictBoostCount::fetch(env, p);
    case setSupergroupUsername::ID:
      return setSupergroupUsername::fetch(env, p);
    case setTdlibParameters::ID:
      return setTdlibParameters::fetch(env, p);
    case setUserEmojiStatus::ID:
      return setUserEmojiStatus::fetch(env, p);
    case setUserPersonalProfilePhoto::ID:
      return setUserPersonalProfilePhoto::fetch(env, p);
    case setUserPrivacySettingRules::ID:
      return setUserPrivacySettingRules::fetch(env, p);
    case setUserSupportInfo::ID:
      return setUserSupportInfo::fetch(env, p);
    case setUsername::ID:
      return setUsername::fetch(env, p);
    case setVideoChatDefaultParticipant::ID:
      return setVideoChatDefaultParticipant::fetch(env, p);
    case setVideoChatTitle::ID:
      return setVideoChatTitle::fetch(env, p);
    case shareChatWithBot::ID:
      return shareChatWithBot::fetch(env, p);
    case sharePhoneNumber::ID:
      return sharePhoneNumber::fetch(env, p);
    case shareUsersWithBot::ID:
      return shareUsersWithBot::fetch(env, p);
    case startGroupCallRecording::ID:
      return startGroupCallRecording::fetch(env, p);
    case startGroupCallScreenSharing::ID:
      return startGroupCallScreenSharing::fetch(env, p);
    case startScheduledVideoChat::ID:
      return startScheduledVideoChat::fetch(env, p);
    case stopBusinessPoll::ID:
      return stopBusinessPoll::fetch(env, p);
    case stopPoll::ID:
      return stopPoll::fetch(env, p);
    case suggestUserProfilePhoto::ID:
      return suggestUserProfilePhoto::fetch(env, p);
    case synchronizeLanguagePack::ID:
      return synchronizeLanguagePack::fetch(env, p);
    case terminateAllOtherSessions::ID:
      return terminateAllOtherSessions::fetch(env, p);
    case terminateSession::ID:
      return terminateSession::fetch(env, p);
    case testCallBytes::ID:
      return testCallBytes::fetch(env, p);
    case testCallEmpty::ID:
      return testCallEmpty::fetch(env, p);
    case testCallString::ID:
      return testCallString::fetch(env, p);
    case testCallVectorInt::ID:
      return testCallVectorInt::fetch(env, p);
    case testCallVectorIntObject::ID:
      return testCallVectorIntObject::fetch(env, p);
    case testCallVectorString::ID:
      return testCallVectorString::fetch(env, p);
    case testCallVectorStringObject::ID:
      return testCallVectorStringObject::fetch(env, p);
    case testGetDifference::ID:
      return testGetDifference::fetch(env, p);
    case testNetwork::ID:
      return testNetwork::fetch(env, p);
    case testProxy::ID:
      return testProxy::fetch(env, p);
    case testReturnError::ID:
      return testReturnError::fetch(env, p);
    case testSquareInt::ID:
      return testSquareInt::fetch(env, p);
    case testUseUpdate::ID:
      return testUseUpdate::fetch(env, p);
    case toggleAllDownloadsArePaused::ID:
      return toggleAllDownloadsArePaused::fetch(env, p);
    case toggleBotCanManageEmojiStatus::ID:
      return toggleBotCanManageEmojiStatus::fetch(env, p);
    case toggleBotIsAddedToAttachmentMenu::ID:
      return toggleBotIsAddedToAttachmentMenu::fetch(env, p);
    case toggleBotUsernameIsActive::ID:
      return toggleBotUsernameIsActive::fetch(env, p);
    case toggleBusinessConnectedBotChatIsPaused::ID:
      return toggleBusinessConnectedBotChatIsPaused::fetch(env, p);
    case toggleChatDefaultDisableNotification::ID:
      return toggleChatDefaultDisableNotification::fetch(env, p);
    case toggleChatFolderTags::ID:
      return toggleChatFolderTags::fetch(env, p);
    case toggleChatGiftNotifications::ID:
      return toggleChatGiftNotifications::fetch(env, p);
    case toggleChatHasProtectedContent::ID:
      return toggleChatHasProtectedContent::fetch(env, p);
    case toggleChatIsMarkedAsUnread::ID:
      return toggleChatIsMarkedAsUnread::fetch(env, p);
    case toggleChatIsPinned::ID:
      return toggleChatIsPinned::fetch(env, p);
    case toggleChatIsTranslatable::ID:
      return toggleChatIsTranslatable::fetch(env, p);
    case toggleChatViewAsTopics::ID:
      return toggleChatViewAsTopics::fetch(env, p);
    case toggleDownloadIsPaused::ID:
      return toggleDownloadIsPaused::fetch(env, p);
    case toggleForumTopicIsClosed::ID:
      return toggleForumTopicIsClosed::fetch(env, p);
    case toggleForumTopicIsPinned::ID:
      return toggleForumTopicIsPinned::fetch(env, p);
    case toggleGeneralForumTopicIsHidden::ID:
      return toggleGeneralForumTopicIsHidden::fetch(env, p);
    case toggleGiftIsSaved::ID:
      return toggleGiftIsSaved::fetch(env, p);
    case toggleGroupCallIsMyVideoEnabled::ID:
      return toggleGroupCallIsMyVideoEnabled::fetch(env, p);
    case toggleGroupCallIsMyVideoPaused::ID:
      return toggleGroupCallIsMyVideoPaused::fetch(env, p);
    case toggleGroupCallParticipantIsHandRaised::ID:
      return toggleGroupCallParticipantIsHandRaised::fetch(env, p);
    case toggleGroupCallParticipantIsMuted::ID:
      return toggleGroupCallParticipantIsMuted::fetch(env, p);
    case toggleGroupCallScreenSharingIsPaused::ID:
      return toggleGroupCallScreenSharingIsPaused::fetch(env, p);
    case toggleHasSponsoredMessagesEnabled::ID:
      return toggleHasSponsoredMessagesEnabled::fetch(env, p);
    case toggleSavedMessagesTopicIsPinned::ID:
      return toggleSavedMessagesTopicIsPinned::fetch(env, p);
    case toggleSessionCanAcceptCalls::ID:
      return toggleSessionCanAcceptCalls::fetch(env, p);
    case toggleSessionCanAcceptSecretChats::ID:
      return toggleSessionCanAcceptSecretChats::fetch(env, p);
    case toggleStoryIsPostedToChatPage::ID:
      return toggleStoryIsPostedToChatPage::fetch(env, p);
    case toggleSupergroupCanHaveSponsoredMessages::ID:
      return toggleSupergroupCanHaveSponsoredMessages::fetch(env, p);
    case toggleSupergroupHasAggressiveAntiSpamEnabled::ID:
      return toggleSupergroupHasAggressiveAntiSpamEnabled::fetch(env, p);
    case toggleSupergroupHasAutomaticTranslation::ID:
      return toggleSupergroupHasAutomaticTranslation::fetch(env, p);
    case toggleSupergroupHasHiddenMembers::ID:
      return toggleSupergroupHasHiddenMembers::fetch(env, p);
    case toggleSupergroupIsAllHistoryAvailable::ID:
      return toggleSupergroupIsAllHistoryAvailable::fetch(env, p);
    case toggleSupergroupIsBroadcastGroup::ID:
      return toggleSupergroupIsBroadcastGroup::fetch(env, p);
    case toggleSupergroupIsForum::ID:
      return toggleSupergroupIsForum::fetch(env, p);
    case toggleSupergroupJoinByRequest::ID:
      return toggleSupergroupJoinByRequest::fetch(env, p);
    case toggleSupergroupJoinToSendMessages::ID:
      return toggleSupergroupJoinToSendMessages::fetch(env, p);
    case toggleSupergroupSignMessages::ID:
      return toggleSupergroupSignMessages::fetch(env, p);
    case toggleSupergroupUsernameIsActive::ID:
      return toggleSupergroupUsernameIsActive::fetch(env, p);
    case toggleUsernameIsActive::ID:
      return toggleUsernameIsActive::fetch(env, p);
    case toggleVideoChatEnabledStartNotification::ID:
      return toggleVideoChatEnabledStartNotification::fetch(env, p);
    case toggleVideoChatMuteNewParticipants::ID:
      return toggleVideoChatMuteNewParticipants::fetch(env, p);
    case transferBusinessAccountStars::ID:
      return transferBusinessAccountStars::fetch(env, p);
    case transferChatOwnership::ID:
      return transferChatOwnership::fetch(env, p);
    case transferGift::ID:
      return transferGift::fetch(env, p);
    case translateMessageText::ID:
      return translateMessageText::fetch(env, p);
    case translateText::ID:
      return translateText::fetch(env, p);
    case unpinAllChatMessages::ID:
      return unpinAllChatMessages::fetch(env, p);
    case unpinAllMessageThreadMessages::ID:
      return unpinAllMessageThreadMessages::fetch(env, p);
    case unpinChatMessage::ID:
      return unpinChatMessage::fetch(env, p);
    case upgradeBasicGroupChatToSupergroupChat::ID:
      return upgradeBasicGroupChatToSupergroupChat::fetch(env, p);
    case upgradeGift::ID:
      return upgradeGift::fetch(env, p);
    case uploadStickerFile::ID:
      return uploadStickerFile::fetch(env, p);
    case validateOrderInfo::ID:
      return validateOrderInfo::fetch(env, p);
    case viewMessages::ID:
      return viewMessages::fetch(env, p);
    case viewPremiumFeature::ID:
      return viewPremiumFeature::fetch(env, p);
    case viewSponsoredChat::ID:
      return viewSponsoredChat::fetch(env, p);
    case viewTrendingStickerSets::ID:
      return viewTrendingStickerSets::fetch(env, p);
    case writeGeneratedFilePart::ID:
      return writeGeneratedFilePart::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void Function::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Function").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass accentColor::Class;
jfieldID accentColor::id_fieldID;
jfieldID accentColor::built_in_accent_color_id_fieldID;
jfieldID accentColor::light_theme_colors_fieldID;
jfieldID accentColor::dark_theme_colors_fieldID;
jfieldID accentColor::min_channel_chat_boost_level_fieldID;

accentColor::accentColor()
  : id_()
  , built_in_accent_color_id_()
  , light_theme_colors_()
  , dark_theme_colors_()
  , min_channel_chat_boost_level_()
{}

accentColor::accentColor(int32 id_, int32 built_in_accent_color_id_, array<int32> &&light_theme_colors_, array<int32> &&dark_theme_colors_, int32 min_channel_chat_boost_level_)
  : id_(id_)
  , built_in_accent_color_id_(built_in_accent_color_id_)
  , light_theme_colors_(std::move(light_theme_colors_))
  , dark_theme_colors_(std::move(dark_theme_colors_))
  , min_channel_chat_boost_level_(min_channel_chat_boost_level_)
{}

const std::int32_t accentColor::ID;

object_ptr<accentColor> accentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<accentColor> res = make_object<accentColor>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->built_in_accent_color_id_ = env->GetIntField(p, res->built_in_accent_color_id_fieldID);
  res->light_theme_colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->light_theme_colors_fieldID));
  res->dark_theme_colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->dark_theme_colors_fieldID));
  res->min_channel_chat_boost_level_ = env->GetIntField(p, res->min_channel_chat_boost_level_fieldID);
  return res;
}

void accentColor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetIntField(s, built_in_accent_color_id_fieldID, built_in_accent_color_id_);
  { auto arr_tmp_ = jni::store_vector(env, light_theme_colors_); if (arr_tmp_) { env->SetObjectField(s, light_theme_colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, dark_theme_colors_); if (arr_tmp_) { env->SetObjectField(s, dark_theme_colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, min_channel_chat_boost_level_fieldID, min_channel_chat_boost_level_);
}

void accentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AccentColor");
    s.store_field("id", id_);
    s.store_field("builtInAccentColorId", built_in_accent_color_id_);
    { s.store_vector_begin("lightThemeColors", light_theme_colors_.size()); for (const auto &_value : light_theme_colors_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("darkThemeColors", dark_theme_colors_.size()); for (const auto &_value : dark_theme_colors_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("minChannelChatBoostLevel", min_channel_chat_boost_level_);
    s.store_class_end();
  }
}

void accentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AccentColor").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    built_in_accent_color_id_fieldID = jni::get_field_id(env, Class, "builtInAccentColorId", "I");
    light_theme_colors_fieldID = jni::get_field_id(env, Class, "lightThemeColors", "[I");
    dark_theme_colors_fieldID = jni::get_field_id(env, Class, "darkThemeColors", "[I");
    min_channel_chat_boost_level_fieldID = jni::get_field_id(env, Class, "minChannelChatBoostLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass acceptedGiftTypes::Class;
jfieldID acceptedGiftTypes::unlimited_gifts_fieldID;
jfieldID acceptedGiftTypes::limited_gifts_fieldID;
jfieldID acceptedGiftTypes::upgraded_gifts_fieldID;
jfieldID acceptedGiftTypes::premium_subscription_fieldID;

acceptedGiftTypes::acceptedGiftTypes()
  : unlimited_gifts_()
  , limited_gifts_()
  , upgraded_gifts_()
  , premium_subscription_()
{}

acceptedGiftTypes::acceptedGiftTypes(bool unlimited_gifts_, bool limited_gifts_, bool upgraded_gifts_, bool premium_subscription_)
  : unlimited_gifts_(unlimited_gifts_)
  , limited_gifts_(limited_gifts_)
  , upgraded_gifts_(upgraded_gifts_)
  , premium_subscription_(premium_subscription_)
{}

const std::int32_t acceptedGiftTypes::ID;

object_ptr<acceptedGiftTypes> acceptedGiftTypes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<acceptedGiftTypes> res = make_object<acceptedGiftTypes>();
  res->unlimited_gifts_ = (env->GetBooleanField(p, res->unlimited_gifts_fieldID) != 0);
  res->limited_gifts_ = (env->GetBooleanField(p, res->limited_gifts_fieldID) != 0);
  res->upgraded_gifts_ = (env->GetBooleanField(p, res->upgraded_gifts_fieldID) != 0);
  res->premium_subscription_ = (env->GetBooleanField(p, res->premium_subscription_fieldID) != 0);
  return res;
}

void acceptedGiftTypes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, unlimited_gifts_fieldID, unlimited_gifts_);
  env->SetBooleanField(s, limited_gifts_fieldID, limited_gifts_);
  env->SetBooleanField(s, upgraded_gifts_fieldID, upgraded_gifts_);
  env->SetBooleanField(s, premium_subscription_fieldID, premium_subscription_);
}

void acceptedGiftTypes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AcceptedGiftTypes");
    s.store_field("unlimitedGifts", unlimited_gifts_);
    s.store_field("limitedGifts", limited_gifts_);
    s.store_field("upgradedGifts", upgraded_gifts_);
    s.store_field("premiumSubscription", premium_subscription_);
    s.store_class_end();
  }
}

void acceptedGiftTypes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AcceptedGiftTypes").c_str());
    unlimited_gifts_fieldID = jni::get_field_id(env, Class, "unlimitedGifts", "Z");
    limited_gifts_fieldID = jni::get_field_id(env, Class, "limitedGifts", "Z");
    upgraded_gifts_fieldID = jni::get_field_id(env, Class, "upgradedGifts", "Z");
    premium_subscription_fieldID = jni::get_field_id(env, Class, "premiumSubscription", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass accountInfo::Class;
jfieldID accountInfo::registration_month_fieldID;
jfieldID accountInfo::registration_year_fieldID;
jfieldID accountInfo::phone_number_country_code_fieldID;
jfieldID accountInfo::last_name_change_date_fieldID;
jfieldID accountInfo::last_photo_change_date_fieldID;

accountInfo::accountInfo()
  : registration_month_()
  , registration_year_()
  , phone_number_country_code_()
  , last_name_change_date_()
  , last_photo_change_date_()
{}

accountInfo::accountInfo(int32 registration_month_, int32 registration_year_, string const &phone_number_country_code_, int32 last_name_change_date_, int32 last_photo_change_date_)
  : registration_month_(registration_month_)
  , registration_year_(registration_year_)
  , phone_number_country_code_(phone_number_country_code_)
  , last_name_change_date_(last_name_change_date_)
  , last_photo_change_date_(last_photo_change_date_)
{}

const std::int32_t accountInfo::ID;

object_ptr<accountInfo> accountInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<accountInfo> res = make_object<accountInfo>();
  res->registration_month_ = env->GetIntField(p, res->registration_month_fieldID);
  res->registration_year_ = env->GetIntField(p, res->registration_year_fieldID);
  res->phone_number_country_code_ = jni::fetch_string(env, p, res->phone_number_country_code_fieldID);
  res->last_name_change_date_ = env->GetIntField(p, res->last_name_change_date_fieldID);
  res->last_photo_change_date_ = env->GetIntField(p, res->last_photo_change_date_fieldID);
  return res;
}

void accountInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, registration_month_fieldID, registration_month_);
  env->SetIntField(s, registration_year_fieldID, registration_year_);
  { jstring nextString = jni::to_jstring(env, phone_number_country_code_); if (nextString) { env->SetObjectField(s, phone_number_country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, last_name_change_date_fieldID, last_name_change_date_);
  env->SetIntField(s, last_photo_change_date_fieldID, last_photo_change_date_);
}

void accountInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AccountInfo");
    s.store_field("registrationMonth", registration_month_);
    s.store_field("registrationYear", registration_year_);
    s.store_field("phoneNumberCountryCode", phone_number_country_code_);
    s.store_field("lastNameChangeDate", last_name_change_date_);
    s.store_field("lastPhotoChangeDate", last_photo_change_date_);
    s.store_class_end();
  }
}

void accountInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AccountInfo").c_str());
    registration_month_fieldID = jni::get_field_id(env, Class, "registrationMonth", "I");
    registration_year_fieldID = jni::get_field_id(env, Class, "registrationYear", "I");
    phone_number_country_code_fieldID = jni::get_field_id(env, Class, "phoneNumberCountryCode", "Ljava/lang/String;");
    last_name_change_date_fieldID = jni::get_field_id(env, Class, "lastNameChangeDate", "I");
    last_photo_change_date_fieldID = jni::get_field_id(env, Class, "lastPhotoChangeDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass accountTtl::Class;
jfieldID accountTtl::days_fieldID;

accountTtl::accountTtl()
  : days_()
{}

accountTtl::accountTtl(int32 days_)
  : days_(days_)
{}

const std::int32_t accountTtl::ID;

object_ptr<accountTtl> accountTtl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<accountTtl> res = make_object<accountTtl>();
  res->days_ = env->GetIntField(p, res->days_fieldID);
  return res;
}

void accountTtl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, days_fieldID, days_);
}

void accountTtl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AccountTtl");
    s.store_field("days", days_);
    s.store_class_end();
  }
}

void accountTtl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AccountTtl").c_str());
    days_fieldID = jni::get_field_id(env, Class, "days", "I");
    return true;
  }();
  (void)is_inited;
}

jclass addedReaction::Class;
jfieldID addedReaction::type_fieldID;
jfieldID addedReaction::sender_id_fieldID;
jfieldID addedReaction::is_outgoing_fieldID;
jfieldID addedReaction::date_fieldID;

addedReaction::addedReaction()
  : type_()
  , sender_id_()
  , is_outgoing_()
  , date_()
{}

addedReaction::addedReaction(object_ptr<ReactionType> &&type_, object_ptr<MessageSender> &&sender_id_, bool is_outgoing_, int32 date_)
  : type_(std::move(type_))
  , sender_id_(std::move(sender_id_))
  , is_outgoing_(is_outgoing_)
  , date_(date_)
{}

const std::int32_t addedReaction::ID;

object_ptr<addedReaction> addedReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addedReaction> res = make_object<addedReaction>();
  res->type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void addedReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
  env->SetIntField(s, date_fieldID, date_);
}

void addedReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddedReaction");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("isOutgoing", is_outgoing_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void addedReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddedReaction").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass addedReactions::Class;
jfieldID addedReactions::total_count_fieldID;
jfieldID addedReactions::reactions_fieldID;
jfieldID addedReactions::next_offset_fieldID;

addedReactions::addedReactions()
  : total_count_()
  , reactions_()
  , next_offset_()
{}

addedReactions::addedReactions(int32 total_count_, array<object_ptr<addedReaction>> &&reactions_, string const &next_offset_)
  : total_count_(total_count_)
  , reactions_(std::move(reactions_))
  , next_offset_(next_offset_)
{}

const std::int32_t addedReactions::ID;

object_ptr<addedReactions> addedReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addedReactions> res = make_object<addedReactions>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->reactions_ = jni::FetchVector<addedReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->reactions_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void addedReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, reactions_); if (arr_tmp_) { env->SetObjectField(s, reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void addedReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddedReactions");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("reactions", reactions_.size()); for (const auto &_value : reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void addedReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddedReactions").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    reactions_fieldID = jni::get_field_id(env, Class, "reactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AddedReaction;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass address::Class;
jfieldID address::country_code_fieldID;
jfieldID address::state_fieldID;
jfieldID address::city_fieldID;
jfieldID address::street_line1_fieldID;
jfieldID address::street_line2_fieldID;
jfieldID address::postal_code_fieldID;

address::address()
  : country_code_()
  , state_()
  , city_()
  , street_line1_()
  , street_line2_()
  , postal_code_()
{}

address::address(string const &country_code_, string const &state_, string const &city_, string const &street_line1_, string const &street_line2_, string const &postal_code_)
  : country_code_(country_code_)
  , state_(state_)
  , city_(city_)
  , street_line1_(street_line1_)
  , street_line2_(street_line2_)
  , postal_code_(postal_code_)
{}

const std::int32_t address::ID;

object_ptr<address> address::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<address> res = make_object<address>();
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  res->state_ = jni::fetch_string(env, p, res->state_fieldID);
  res->city_ = jni::fetch_string(env, p, res->city_fieldID);
  res->street_line1_ = jni::fetch_string(env, p, res->street_line1_fieldID);
  res->street_line2_ = jni::fetch_string(env, p, res->street_line2_fieldID);
  res->postal_code_ = jni::fetch_string(env, p, res->postal_code_fieldID);
  return res;
}

void address::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, country_code_); if (nextString) { env->SetObjectField(s, country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, state_); if (nextString) { env->SetObjectField(s, state_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, city_); if (nextString) { env->SetObjectField(s, city_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, street_line1_); if (nextString) { env->SetObjectField(s, street_line1_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, street_line2_); if (nextString) { env->SetObjectField(s, street_line2_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, postal_code_); if (nextString) { env->SetObjectField(s, postal_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void address::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Address");
    s.store_field("countryCode", country_code_);
    s.store_field("state", state_);
    s.store_field("city", city_);
    s.store_field("streetLine1", street_line1_);
    s.store_field("streetLine2", street_line2_);
    s.store_field("postalCode", postal_code_);
    s.store_class_end();
  }
}

void address::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Address").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    state_fieldID = jni::get_field_id(env, Class, "state", "Ljava/lang/String;");
    city_fieldID = jni::get_field_id(env, Class, "city", "Ljava/lang/String;");
    street_line1_fieldID = jni::get_field_id(env, Class, "streetLine1", "Ljava/lang/String;");
    street_line2_fieldID = jni::get_field_id(env, Class, "streetLine2", "Ljava/lang/String;");
    postal_code_fieldID = jni::get_field_id(env, Class, "postalCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass affiliateInfo::Class;
jfieldID affiliateInfo::commission_per_mille_fieldID;
jfieldID affiliateInfo::affiliate_chat_id_fieldID;
jfieldID affiliateInfo::star_amount_fieldID;

affiliateInfo::affiliateInfo()
  : commission_per_mille_()
  , affiliate_chat_id_()
  , star_amount_()
{}

affiliateInfo::affiliateInfo(int32 commission_per_mille_, int53 affiliate_chat_id_, object_ptr<starAmount> &&star_amount_)
  : commission_per_mille_(commission_per_mille_)
  , affiliate_chat_id_(affiliate_chat_id_)
  , star_amount_(std::move(star_amount_))
{}

const std::int32_t affiliateInfo::ID;

object_ptr<affiliateInfo> affiliateInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<affiliateInfo> res = make_object<affiliateInfo>();
  res->commission_per_mille_ = env->GetIntField(p, res->commission_per_mille_fieldID);
  res->affiliate_chat_id_ = env->GetLongField(p, res->affiliate_chat_id_fieldID);
  res->star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->star_amount_fieldID));
  return res;
}

void affiliateInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, commission_per_mille_fieldID, commission_per_mille_);
  env->SetLongField(s, affiliate_chat_id_fieldID, affiliate_chat_id_);
  if (star_amount_ != nullptr) { jobject next; star_amount_->store(env, next); if (next) { env->SetObjectField(s, star_amount_fieldID, next); env->DeleteLocalRef(next); } }
}

void affiliateInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateInfo");
    s.store_field("commissionPerMille", commission_per_mille_);
    s.store_field("affiliateChatId", affiliate_chat_id_);
    s.store_object_field("starAmount", static_cast<const BaseObject *>(star_amount_.get()));
    s.store_class_end();
  }
}

void affiliateInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateInfo").c_str());
    commission_per_mille_fieldID = jni::get_field_id(env, Class, "commissionPerMille", "I");
    affiliate_chat_id_fieldID = jni::get_field_id(env, Class, "affiliateChatId", "J");
    star_amount_fieldID = jni::get_field_id(env, Class, "starAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateProgramInfo::Class;
jfieldID affiliateProgramInfo::parameters_fieldID;
jfieldID affiliateProgramInfo::end_date_fieldID;
jfieldID affiliateProgramInfo::daily_revenue_per_user_amount_fieldID;

affiliateProgramInfo::affiliateProgramInfo()
  : parameters_()
  , end_date_()
  , daily_revenue_per_user_amount_()
{}

affiliateProgramInfo::affiliateProgramInfo(object_ptr<affiliateProgramParameters> &&parameters_, int32 end_date_, object_ptr<starAmount> &&daily_revenue_per_user_amount_)
  : parameters_(std::move(parameters_))
  , end_date_(end_date_)
  , daily_revenue_per_user_amount_(std::move(daily_revenue_per_user_amount_))
{}

const std::int32_t affiliateProgramInfo::ID;

object_ptr<affiliateProgramInfo> affiliateProgramInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<affiliateProgramInfo> res = make_object<affiliateProgramInfo>();
  res->parameters_ = jni::fetch_tl_object<affiliateProgramParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->end_date_ = env->GetIntField(p, res->end_date_fieldID);
  res->daily_revenue_per_user_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->daily_revenue_per_user_amount_fieldID));
  return res;
}

void affiliateProgramInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (parameters_ != nullptr) { jobject next; parameters_->store(env, next); if (next) { env->SetObjectField(s, parameters_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, end_date_fieldID, end_date_);
  if (daily_revenue_per_user_amount_ != nullptr) { jobject next; daily_revenue_per_user_amount_->store(env, next); if (next) { env->SetObjectField(s, daily_revenue_per_user_amount_fieldID, next); env->DeleteLocalRef(next); } }
}

void affiliateProgramInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateProgramInfo");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("endDate", end_date_);
    s.store_object_field("dailyRevenuePerUserAmount", static_cast<const BaseObject *>(daily_revenue_per_user_amount_.get()));
    s.store_class_end();
  }
}

void affiliateProgramInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramInfo").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramParameters;").c_str());
    end_date_fieldID = jni::get_field_id(env, Class, "endDate", "I");
    daily_revenue_per_user_amount_fieldID = jni::get_field_id(env, Class, "dailyRevenuePerUserAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateProgramParameters::Class;
jfieldID affiliateProgramParameters::commission_per_mille_fieldID;
jfieldID affiliateProgramParameters::month_count_fieldID;

affiliateProgramParameters::affiliateProgramParameters()
  : commission_per_mille_()
  , month_count_()
{}

affiliateProgramParameters::affiliateProgramParameters(int32 commission_per_mille_, int32 month_count_)
  : commission_per_mille_(commission_per_mille_)
  , month_count_(month_count_)
{}

const std::int32_t affiliateProgramParameters::ID;

object_ptr<affiliateProgramParameters> affiliateProgramParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<affiliateProgramParameters> res = make_object<affiliateProgramParameters>();
  res->commission_per_mille_ = env->GetIntField(p, res->commission_per_mille_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  return res;
}

void affiliateProgramParameters::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, commission_per_mille_fieldID, commission_per_mille_);
  env->SetIntField(s, month_count_fieldID, month_count_);
}

void affiliateProgramParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateProgramParameters");
    s.store_field("commissionPerMille", commission_per_mille_);
    s.store_field("monthCount", month_count_);
    s.store_class_end();
  }
}

void affiliateProgramParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramParameters").c_str());
    commission_per_mille_fieldID = jni::get_field_id(env, Class, "commissionPerMille", "I");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass AffiliateProgramSortOrder::Class;

object_ptr<AffiliateProgramSortOrder> AffiliateProgramSortOrder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case affiliateProgramSortOrderProfitability::ID:
      return affiliateProgramSortOrderProfitability::fetch(env, p);
    case affiliateProgramSortOrderCreationDate::ID:
      return affiliateProgramSortOrderCreationDate::fetch(env, p);
    case affiliateProgramSortOrderRevenue::ID:
      return affiliateProgramSortOrderRevenue::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void AffiliateProgramSortOrder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramSortOrder").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateProgramSortOrderProfitability::Class;

affiliateProgramSortOrderProfitability::affiliateProgramSortOrderProfitability() {
}

const std::int32_t affiliateProgramSortOrderProfitability::ID;

object_ptr<AffiliateProgramSortOrder> affiliateProgramSortOrderProfitability::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<affiliateProgramSortOrderProfitability>();
}

void affiliateProgramSortOrderProfitability::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateProgramSortOrderProfitability");
    s.store_class_end();
  }
}

void affiliateProgramSortOrderProfitability::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramSortOrderProfitability").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateProgramSortOrderCreationDate::Class;

affiliateProgramSortOrderCreationDate::affiliateProgramSortOrderCreationDate() {
}

const std::int32_t affiliateProgramSortOrderCreationDate::ID;

object_ptr<AffiliateProgramSortOrder> affiliateProgramSortOrderCreationDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<affiliateProgramSortOrderCreationDate>();
}

void affiliateProgramSortOrderCreationDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateProgramSortOrderCreationDate");
    s.store_class_end();
  }
}

void affiliateProgramSortOrderCreationDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramSortOrderCreationDate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateProgramSortOrderRevenue::Class;

affiliateProgramSortOrderRevenue::affiliateProgramSortOrderRevenue() {
}

const std::int32_t affiliateProgramSortOrderRevenue::ID;

object_ptr<AffiliateProgramSortOrder> affiliateProgramSortOrderRevenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<affiliateProgramSortOrderRevenue>();
}

void affiliateProgramSortOrderRevenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateProgramSortOrderRevenue");
    s.store_class_end();
  }
}

void affiliateProgramSortOrderRevenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateProgramSortOrderRevenue").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass AffiliateType::Class;

object_ptr<AffiliateType> AffiliateType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case affiliateTypeCurrentUser::ID:
      return affiliateTypeCurrentUser::fetch(env, p);
    case affiliateTypeBot::ID:
      return affiliateTypeBot::fetch(env, p);
    case affiliateTypeChannel::ID:
      return affiliateTypeChannel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void AffiliateType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateTypeCurrentUser::Class;

affiliateTypeCurrentUser::affiliateTypeCurrentUser() {
}

const std::int32_t affiliateTypeCurrentUser::ID;

object_ptr<AffiliateType> affiliateTypeCurrentUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<affiliateTypeCurrentUser>();
}

void affiliateTypeCurrentUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateTypeCurrentUser");
    s.store_class_end();
  }
}

void affiliateTypeCurrentUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateTypeCurrentUser").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass affiliateTypeBot::Class;
jfieldID affiliateTypeBot::user_id_fieldID;

affiliateTypeBot::affiliateTypeBot()
  : user_id_()
{}

affiliateTypeBot::affiliateTypeBot(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t affiliateTypeBot::ID;

object_ptr<AffiliateType> affiliateTypeBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<affiliateTypeBot> res = make_object<affiliateTypeBot>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void affiliateTypeBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateTypeBot");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void affiliateTypeBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateTypeBot").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass affiliateTypeChannel::Class;
jfieldID affiliateTypeChannel::chat_id_fieldID;

affiliateTypeChannel::affiliateTypeChannel()
  : chat_id_()
{}

affiliateTypeChannel::affiliateTypeChannel(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t affiliateTypeChannel::ID;

object_ptr<AffiliateType> affiliateTypeChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<affiliateTypeChannel> res = make_object<affiliateTypeChannel>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void affiliateTypeChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AffiliateTypeChannel");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void affiliateTypeChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AffiliateTypeChannel").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass alternativeVideo::Class;
jfieldID alternativeVideo::id_fieldID;
jfieldID alternativeVideo::width_fieldID;
jfieldID alternativeVideo::height_fieldID;
jfieldID alternativeVideo::codec_fieldID;
jfieldID alternativeVideo::hls_file_fieldID;
jfieldID alternativeVideo::video_fieldID;

alternativeVideo::alternativeVideo()
  : id_()
  , width_()
  , height_()
  , codec_()
  , hls_file_()
  , video_()
{}

alternativeVideo::alternativeVideo(int64 id_, int32 width_, int32 height_, string const &codec_, object_ptr<file> &&hls_file_, object_ptr<file> &&video_)
  : id_(id_)
  , width_(width_)
  , height_(height_)
  , codec_(codec_)
  , hls_file_(std::move(hls_file_))
  , video_(std::move(video_))
{}

const std::int32_t alternativeVideo::ID;

object_ptr<alternativeVideo> alternativeVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<alternativeVideo> res = make_object<alternativeVideo>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->codec_ = jni::fetch_string(env, p, res->codec_fieldID);
  res->hls_file_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->hls_file_fieldID));
  res->video_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->video_fieldID));
  return res;
}

void alternativeVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jstring nextString = jni::to_jstring(env, codec_); if (nextString) { env->SetObjectField(s, codec_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (hls_file_ != nullptr) { jobject next; hls_file_->store(env, next); if (next) { env->SetObjectField(s, hls_file_fieldID, next); env->DeleteLocalRef(next); } }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
}

void alternativeVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AlternativeVideo");
    s.store_field("id", id_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("codec", codec_);
    s.store_object_field("hlsFile", static_cast<const BaseObject *>(hls_file_.get()));
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_class_end();
  }
}

void alternativeVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AlternativeVideo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    codec_fieldID = jni::get_field_id(env, Class, "codec", "Ljava/lang/String;");
    hls_file_fieldID = jni::get_field_id(env, Class, "hlsFile", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass animatedChatPhoto::Class;
jfieldID animatedChatPhoto::length_fieldID;
jfieldID animatedChatPhoto::file_fieldID;
jfieldID animatedChatPhoto::main_frame_timestamp_fieldID;

animatedChatPhoto::animatedChatPhoto()
  : length_()
  , file_()
  , main_frame_timestamp_()
{}

animatedChatPhoto::animatedChatPhoto(int32 length_, object_ptr<file> &&file_, double main_frame_timestamp_)
  : length_(length_)
  , file_(std::move(file_))
  , main_frame_timestamp_(main_frame_timestamp_)
{}

const std::int32_t animatedChatPhoto::ID;

object_ptr<animatedChatPhoto> animatedChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<animatedChatPhoto> res = make_object<animatedChatPhoto>();
  res->length_ = env->GetIntField(p, res->length_fieldID);
  res->file_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->file_fieldID));
  res->main_frame_timestamp_ = env->GetDoubleField(p, res->main_frame_timestamp_fieldID);
  return res;
}

void animatedChatPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, length_fieldID, length_);
  if (file_ != nullptr) { jobject next; file_->store(env, next); if (next) { env->SetObjectField(s, file_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, main_frame_timestamp_fieldID, main_frame_timestamp_);
}

void animatedChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnimatedChatPhoto");
    s.store_field("length", length_);
    s.store_object_field("file", static_cast<const BaseObject *>(file_.get()));
    s.store_field("mainFrameTimestamp", main_frame_timestamp_);
    s.store_class_end();
  }
}

void animatedChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnimatedChatPhoto").c_str());
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    file_fieldID = jni::get_field_id(env, Class, "file", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    main_frame_timestamp_fieldID = jni::get_field_id(env, Class, "mainFrameTimestamp", "D");
    return true;
  }();
  (void)is_inited;
}

jclass animatedEmoji::Class;
jfieldID animatedEmoji::sticker_fieldID;
jfieldID animatedEmoji::sticker_width_fieldID;
jfieldID animatedEmoji::sticker_height_fieldID;
jfieldID animatedEmoji::fitzpatrick_type_fieldID;
jfieldID animatedEmoji::sound_fieldID;

animatedEmoji::animatedEmoji()
  : sticker_()
  , sticker_width_()
  , sticker_height_()
  , fitzpatrick_type_()
  , sound_()
{}

animatedEmoji::animatedEmoji(object_ptr<sticker> &&sticker_, int32 sticker_width_, int32 sticker_height_, int32 fitzpatrick_type_, object_ptr<file> &&sound_)
  : sticker_(std::move(sticker_))
  , sticker_width_(sticker_width_)
  , sticker_height_(sticker_height_)
  , fitzpatrick_type_(fitzpatrick_type_)
  , sound_(std::move(sound_))
{}

const std::int32_t animatedEmoji::ID;

object_ptr<animatedEmoji> animatedEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<animatedEmoji> res = make_object<animatedEmoji>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->sticker_width_ = env->GetIntField(p, res->sticker_width_fieldID);
  res->sticker_height_ = env->GetIntField(p, res->sticker_height_fieldID);
  res->fitzpatrick_type_ = env->GetIntField(p, res->fitzpatrick_type_fieldID);
  res->sound_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->sound_fieldID));
  return res;
}

void animatedEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, sticker_width_fieldID, sticker_width_);
  env->SetIntField(s, sticker_height_fieldID, sticker_height_);
  env->SetIntField(s, fitzpatrick_type_fieldID, fitzpatrick_type_);
  if (sound_ != nullptr) { jobject next; sound_->store(env, next); if (next) { env->SetObjectField(s, sound_fieldID, next); env->DeleteLocalRef(next); } }
}

void animatedEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnimatedEmoji");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("stickerWidth", sticker_width_);
    s.store_field("stickerHeight", sticker_height_);
    s.store_field("fitzpatrickType", fitzpatrick_type_);
    s.store_object_field("sound", static_cast<const BaseObject *>(sound_.get()));
    s.store_class_end();
  }
}

void animatedEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnimatedEmoji").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    sticker_width_fieldID = jni::get_field_id(env, Class, "stickerWidth", "I");
    sticker_height_fieldID = jni::get_field_id(env, Class, "stickerHeight", "I");
    fitzpatrick_type_fieldID = jni::get_field_id(env, Class, "fitzpatrickType", "I");
    sound_fieldID = jni::get_field_id(env, Class, "sound", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass animation::Class;
jfieldID animation::duration_fieldID;
jfieldID animation::width_fieldID;
jfieldID animation::height_fieldID;
jfieldID animation::file_name_fieldID;
jfieldID animation::mime_type_fieldID;
jfieldID animation::has_stickers_fieldID;
jfieldID animation::minithumbnail_fieldID;
jfieldID animation::thumbnail_fieldID;
jfieldID animation::animation_fieldID;

animation::animation()
  : duration_()
  , width_()
  , height_()
  , file_name_()
  , mime_type_()
  , has_stickers_()
  , minithumbnail_()
  , thumbnail_()
  , animation_()
{}

animation::animation(int32 duration_, int32 width_, int32 height_, string const &file_name_, string const &mime_type_, bool has_stickers_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&animation_)
  : duration_(duration_)
  , width_(width_)
  , height_(height_)
  , file_name_(file_name_)
  , mime_type_(mime_type_)
  , has_stickers_(has_stickers_)
  , minithumbnail_(std::move(minithumbnail_))
  , thumbnail_(std::move(thumbnail_))
  , animation_(std::move(animation_))
{}

const std::int32_t animation::ID;

object_ptr<animation> animation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<animation> res = make_object<animation>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->has_stickers_ = (env->GetBooleanField(p, res->has_stickers_fieldID) != 0);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->animation_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void animation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jstring nextString = jni::to_jstring(env, file_name_); if (nextString) { env->SetObjectField(s, file_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, has_stickers_fieldID, has_stickers_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void animation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Animation");
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("fileName", file_name_);
    s.store_field("mimeType", mime_type_);
    s.store_field("hasStickers", has_stickers_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void animation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Animation").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    has_stickers_fieldID = jni::get_field_id(env, Class, "hasStickers", "Z");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass animations::Class;
jfieldID animations::animations_fieldID;

animations::animations()
  : animations_()
{}

animations::animations(array<object_ptr<animation>> &&animations_)
  : animations_(std::move(animations_))
{}

const std::int32_t animations::ID;

object_ptr<animations> animations::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<animations> res = make_object<animations>();
  res->animations_ = jni::FetchVector<animation>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->animations_fieldID));
  return res;
}

void animations::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, animations_); if (arr_tmp_) { env->SetObjectField(s, animations_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void animations::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Animations");
    { s.store_vector_begin("animations", animations_.size()); for (const auto &_value : animations_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void animations::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Animations").c_str());
    animations_fieldID = jni::get_field_id(env, Class, "animations", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass archiveChatListSettings::Class;
jfieldID archiveChatListSettings::archive_and_mute_new_chats_from_unknown_users_fieldID;
jfieldID archiveChatListSettings::keep_unmuted_chats_archived_fieldID;
jfieldID archiveChatListSettings::keep_chats_from_folders_archived_fieldID;

archiveChatListSettings::archiveChatListSettings()
  : archive_and_mute_new_chats_from_unknown_users_()
  , keep_unmuted_chats_archived_()
  , keep_chats_from_folders_archived_()
{}

archiveChatListSettings::archiveChatListSettings(bool archive_and_mute_new_chats_from_unknown_users_, bool keep_unmuted_chats_archived_, bool keep_chats_from_folders_archived_)
  : archive_and_mute_new_chats_from_unknown_users_(archive_and_mute_new_chats_from_unknown_users_)
  , keep_unmuted_chats_archived_(keep_unmuted_chats_archived_)
  , keep_chats_from_folders_archived_(keep_chats_from_folders_archived_)
{}

const std::int32_t archiveChatListSettings::ID;

object_ptr<archiveChatListSettings> archiveChatListSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<archiveChatListSettings> res = make_object<archiveChatListSettings>();
  res->archive_and_mute_new_chats_from_unknown_users_ = (env->GetBooleanField(p, res->archive_and_mute_new_chats_from_unknown_users_fieldID) != 0);
  res->keep_unmuted_chats_archived_ = (env->GetBooleanField(p, res->keep_unmuted_chats_archived_fieldID) != 0);
  res->keep_chats_from_folders_archived_ = (env->GetBooleanField(p, res->keep_chats_from_folders_archived_fieldID) != 0);
  return res;
}

void archiveChatListSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, archive_and_mute_new_chats_from_unknown_users_fieldID, archive_and_mute_new_chats_from_unknown_users_);
  env->SetBooleanField(s, keep_unmuted_chats_archived_fieldID, keep_unmuted_chats_archived_);
  env->SetBooleanField(s, keep_chats_from_folders_archived_fieldID, keep_chats_from_folders_archived_);
}

void archiveChatListSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ArchiveChatListSettings");
    s.store_field("archiveAndMuteNewChatsFromUnknownUsers", archive_and_mute_new_chats_from_unknown_users_);
    s.store_field("keepUnmutedChatsArchived", keep_unmuted_chats_archived_);
    s.store_field("keepChatsFromFoldersArchived", keep_chats_from_folders_archived_);
    s.store_class_end();
  }
}

void archiveChatListSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ArchiveChatListSettings").c_str());
    archive_and_mute_new_chats_from_unknown_users_fieldID = jni::get_field_id(env, Class, "archiveAndMuteNewChatsFromUnknownUsers", "Z");
    keep_unmuted_chats_archived_fieldID = jni::get_field_id(env, Class, "keepUnmutedChatsArchived", "Z");
    keep_chats_from_folders_archived_fieldID = jni::get_field_id(env, Class, "keepChatsFromFoldersArchived", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass attachmentMenuBot::Class;
jfieldID attachmentMenuBot::bot_user_id_fieldID;
jfieldID attachmentMenuBot::supports_self_chat_fieldID;
jfieldID attachmentMenuBot::supports_user_chats_fieldID;
jfieldID attachmentMenuBot::supports_bot_chats_fieldID;
jfieldID attachmentMenuBot::supports_group_chats_fieldID;
jfieldID attachmentMenuBot::supports_channel_chats_fieldID;
jfieldID attachmentMenuBot::request_write_access_fieldID;
jfieldID attachmentMenuBot::is_added_fieldID;
jfieldID attachmentMenuBot::show_in_attachment_menu_fieldID;
jfieldID attachmentMenuBot::show_in_side_menu_fieldID;
jfieldID attachmentMenuBot::show_disclaimer_in_side_menu_fieldID;
jfieldID attachmentMenuBot::name_fieldID;
jfieldID attachmentMenuBot::name_color_fieldID;
jfieldID attachmentMenuBot::default_icon_fieldID;
jfieldID attachmentMenuBot::ios_static_icon_fieldID;
jfieldID attachmentMenuBot::ios_animated_icon_fieldID;
jfieldID attachmentMenuBot::ios_side_menu_icon_fieldID;
jfieldID attachmentMenuBot::android_icon_fieldID;
jfieldID attachmentMenuBot::android_side_menu_icon_fieldID;
jfieldID attachmentMenuBot::macos_icon_fieldID;
jfieldID attachmentMenuBot::macos_side_menu_icon_fieldID;
jfieldID attachmentMenuBot::icon_color_fieldID;
jfieldID attachmentMenuBot::web_app_placeholder_fieldID;

attachmentMenuBot::attachmentMenuBot()
  : bot_user_id_()
  , supports_self_chat_()
  , supports_user_chats_()
  , supports_bot_chats_()
  , supports_group_chats_()
  , supports_channel_chats_()
  , request_write_access_()
  , is_added_()
  , show_in_attachment_menu_()
  , show_in_side_menu_()
  , show_disclaimer_in_side_menu_()
  , name_()
  , name_color_()
  , default_icon_()
  , ios_static_icon_()
  , ios_animated_icon_()
  , ios_side_menu_icon_()
  , android_icon_()
  , android_side_menu_icon_()
  , macos_icon_()
  , macos_side_menu_icon_()
  , icon_color_()
  , web_app_placeholder_()
{}

attachmentMenuBot::attachmentMenuBot(int53 bot_user_id_, bool supports_self_chat_, bool supports_user_chats_, bool supports_bot_chats_, bool supports_group_chats_, bool supports_channel_chats_, bool request_write_access_, bool is_added_, bool show_in_attachment_menu_, bool show_in_side_menu_, bool show_disclaimer_in_side_menu_, string const &name_, object_ptr<attachmentMenuBotColor> &&name_color_, object_ptr<file> &&default_icon_, object_ptr<file> &&ios_static_icon_, object_ptr<file> &&ios_animated_icon_, object_ptr<file> &&ios_side_menu_icon_, object_ptr<file> &&android_icon_, object_ptr<file> &&android_side_menu_icon_, object_ptr<file> &&macos_icon_, object_ptr<file> &&macos_side_menu_icon_, object_ptr<attachmentMenuBotColor> &&icon_color_, object_ptr<file> &&web_app_placeholder_)
  : bot_user_id_(bot_user_id_)
  , supports_self_chat_(supports_self_chat_)
  , supports_user_chats_(supports_user_chats_)
  , supports_bot_chats_(supports_bot_chats_)
  , supports_group_chats_(supports_group_chats_)
  , supports_channel_chats_(supports_channel_chats_)
  , request_write_access_(request_write_access_)
  , is_added_(is_added_)
  , show_in_attachment_menu_(show_in_attachment_menu_)
  , show_in_side_menu_(show_in_side_menu_)
  , show_disclaimer_in_side_menu_(show_disclaimer_in_side_menu_)
  , name_(name_)
  , name_color_(std::move(name_color_))
  , default_icon_(std::move(default_icon_))
  , ios_static_icon_(std::move(ios_static_icon_))
  , ios_animated_icon_(std::move(ios_animated_icon_))
  , ios_side_menu_icon_(std::move(ios_side_menu_icon_))
  , android_icon_(std::move(android_icon_))
  , android_side_menu_icon_(std::move(android_side_menu_icon_))
  , macos_icon_(std::move(macos_icon_))
  , macos_side_menu_icon_(std::move(macos_side_menu_icon_))
  , icon_color_(std::move(icon_color_))
  , web_app_placeholder_(std::move(web_app_placeholder_))
{}

const std::int32_t attachmentMenuBot::ID;

object_ptr<attachmentMenuBot> attachmentMenuBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<attachmentMenuBot> res = make_object<attachmentMenuBot>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->supports_self_chat_ = (env->GetBooleanField(p, res->supports_self_chat_fieldID) != 0);
  res->supports_user_chats_ = (env->GetBooleanField(p, res->supports_user_chats_fieldID) != 0);
  res->supports_bot_chats_ = (env->GetBooleanField(p, res->supports_bot_chats_fieldID) != 0);
  res->supports_group_chats_ = (env->GetBooleanField(p, res->supports_group_chats_fieldID) != 0);
  res->supports_channel_chats_ = (env->GetBooleanField(p, res->supports_channel_chats_fieldID) != 0);
  res->request_write_access_ = (env->GetBooleanField(p, res->request_write_access_fieldID) != 0);
  res->is_added_ = (env->GetBooleanField(p, res->is_added_fieldID) != 0);
  res->show_in_attachment_menu_ = (env->GetBooleanField(p, res->show_in_attachment_menu_fieldID) != 0);
  res->show_in_side_menu_ = (env->GetBooleanField(p, res->show_in_side_menu_fieldID) != 0);
  res->show_disclaimer_in_side_menu_ = (env->GetBooleanField(p, res->show_disclaimer_in_side_menu_fieldID) != 0);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->name_color_ = jni::fetch_tl_object<attachmentMenuBotColor>(env, jni::fetch_object(env, p, res->name_color_fieldID));
  res->default_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->default_icon_fieldID));
  res->ios_static_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->ios_static_icon_fieldID));
  res->ios_animated_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->ios_animated_icon_fieldID));
  res->ios_side_menu_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->ios_side_menu_icon_fieldID));
  res->android_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->android_icon_fieldID));
  res->android_side_menu_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->android_side_menu_icon_fieldID));
  res->macos_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->macos_icon_fieldID));
  res->macos_side_menu_icon_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->macos_side_menu_icon_fieldID));
  res->icon_color_ = jni::fetch_tl_object<attachmentMenuBotColor>(env, jni::fetch_object(env, p, res->icon_color_fieldID));
  res->web_app_placeholder_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->web_app_placeholder_fieldID));
  return res;
}

void attachmentMenuBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  env->SetBooleanField(s, supports_self_chat_fieldID, supports_self_chat_);
  env->SetBooleanField(s, supports_user_chats_fieldID, supports_user_chats_);
  env->SetBooleanField(s, supports_bot_chats_fieldID, supports_bot_chats_);
  env->SetBooleanField(s, supports_group_chats_fieldID, supports_group_chats_);
  env->SetBooleanField(s, supports_channel_chats_fieldID, supports_channel_chats_);
  env->SetBooleanField(s, request_write_access_fieldID, request_write_access_);
  env->SetBooleanField(s, is_added_fieldID, is_added_);
  env->SetBooleanField(s, show_in_attachment_menu_fieldID, show_in_attachment_menu_);
  env->SetBooleanField(s, show_in_side_menu_fieldID, show_in_side_menu_);
  env->SetBooleanField(s, show_disclaimer_in_side_menu_fieldID, show_disclaimer_in_side_menu_);
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (name_color_ != nullptr) { jobject next; name_color_->store(env, next); if (next) { env->SetObjectField(s, name_color_fieldID, next); env->DeleteLocalRef(next); } }
  if (default_icon_ != nullptr) { jobject next; default_icon_->store(env, next); if (next) { env->SetObjectField(s, default_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (ios_static_icon_ != nullptr) { jobject next; ios_static_icon_->store(env, next); if (next) { env->SetObjectField(s, ios_static_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (ios_animated_icon_ != nullptr) { jobject next; ios_animated_icon_->store(env, next); if (next) { env->SetObjectField(s, ios_animated_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (ios_side_menu_icon_ != nullptr) { jobject next; ios_side_menu_icon_->store(env, next); if (next) { env->SetObjectField(s, ios_side_menu_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (android_icon_ != nullptr) { jobject next; android_icon_->store(env, next); if (next) { env->SetObjectField(s, android_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (android_side_menu_icon_ != nullptr) { jobject next; android_side_menu_icon_->store(env, next); if (next) { env->SetObjectField(s, android_side_menu_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (macos_icon_ != nullptr) { jobject next; macos_icon_->store(env, next); if (next) { env->SetObjectField(s, macos_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (macos_side_menu_icon_ != nullptr) { jobject next; macos_side_menu_icon_->store(env, next); if (next) { env->SetObjectField(s, macos_side_menu_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (icon_color_ != nullptr) { jobject next; icon_color_->store(env, next); if (next) { env->SetObjectField(s, icon_color_fieldID, next); env->DeleteLocalRef(next); } }
  if (web_app_placeholder_ != nullptr) { jobject next; web_app_placeholder_->store(env, next); if (next) { env->SetObjectField(s, web_app_placeholder_fieldID, next); env->DeleteLocalRef(next); } }
}

void attachmentMenuBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AttachmentMenuBot");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("supportsSelfChat", supports_self_chat_);
    s.store_field("supportsUserChats", supports_user_chats_);
    s.store_field("supportsBotChats", supports_bot_chats_);
    s.store_field("supportsGroupChats", supports_group_chats_);
    s.store_field("supportsChannelChats", supports_channel_chats_);
    s.store_field("requestWriteAccess", request_write_access_);
    s.store_field("isAdded", is_added_);
    s.store_field("showInAttachmentMenu", show_in_attachment_menu_);
    s.store_field("showInSideMenu", show_in_side_menu_);
    s.store_field("showDisclaimerInSideMenu", show_disclaimer_in_side_menu_);
    s.store_field("name", name_);
    s.store_object_field("nameColor", static_cast<const BaseObject *>(name_color_.get()));
    s.store_object_field("defaultIcon", static_cast<const BaseObject *>(default_icon_.get()));
    s.store_object_field("iosStaticIcon", static_cast<const BaseObject *>(ios_static_icon_.get()));
    s.store_object_field("iosAnimatedIcon", static_cast<const BaseObject *>(ios_animated_icon_.get()));
    s.store_object_field("iosSideMenuIcon", static_cast<const BaseObject *>(ios_side_menu_icon_.get()));
    s.store_object_field("androidIcon", static_cast<const BaseObject *>(android_icon_.get()));
    s.store_object_field("androidSideMenuIcon", static_cast<const BaseObject *>(android_side_menu_icon_.get()));
    s.store_object_field("macosIcon", static_cast<const BaseObject *>(macos_icon_.get()));
    s.store_object_field("macosSideMenuIcon", static_cast<const BaseObject *>(macos_side_menu_icon_.get()));
    s.store_object_field("iconColor", static_cast<const BaseObject *>(icon_color_.get()));
    s.store_object_field("webAppPlaceholder", static_cast<const BaseObject *>(web_app_placeholder_.get()));
    s.store_class_end();
  }
}

void attachmentMenuBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AttachmentMenuBot").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    supports_self_chat_fieldID = jni::get_field_id(env, Class, "supportsSelfChat", "Z");
    supports_user_chats_fieldID = jni::get_field_id(env, Class, "supportsUserChats", "Z");
    supports_bot_chats_fieldID = jni::get_field_id(env, Class, "supportsBotChats", "Z");
    supports_group_chats_fieldID = jni::get_field_id(env, Class, "supportsGroupChats", "Z");
    supports_channel_chats_fieldID = jni::get_field_id(env, Class, "supportsChannelChats", "Z");
    request_write_access_fieldID = jni::get_field_id(env, Class, "requestWriteAccess", "Z");
    is_added_fieldID = jni::get_field_id(env, Class, "isAdded", "Z");
    show_in_attachment_menu_fieldID = jni::get_field_id(env, Class, "showInAttachmentMenu", "Z");
    show_in_side_menu_fieldID = jni::get_field_id(env, Class, "showInSideMenu", "Z");
    show_disclaimer_in_side_menu_fieldID = jni::get_field_id(env, Class, "showDisclaimerInSideMenu", "Z");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    name_color_fieldID = jni::get_field_id(env, Class, "nameColor", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AttachmentMenuBotColor;").c_str());
    default_icon_fieldID = jni::get_field_id(env, Class, "defaultIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    ios_static_icon_fieldID = jni::get_field_id(env, Class, "iosStaticIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    ios_animated_icon_fieldID = jni::get_field_id(env, Class, "iosAnimatedIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    ios_side_menu_icon_fieldID = jni::get_field_id(env, Class, "iosSideMenuIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    android_icon_fieldID = jni::get_field_id(env, Class, "androidIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    android_side_menu_icon_fieldID = jni::get_field_id(env, Class, "androidSideMenuIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    macos_icon_fieldID = jni::get_field_id(env, Class, "macosIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    macos_side_menu_icon_fieldID = jni::get_field_id(env, Class, "macosSideMenuIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    icon_color_fieldID = jni::get_field_id(env, Class, "iconColor", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AttachmentMenuBotColor;").c_str());
    web_app_placeholder_fieldID = jni::get_field_id(env, Class, "webAppPlaceholder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass attachmentMenuBotColor::Class;
jfieldID attachmentMenuBotColor::light_color_fieldID;
jfieldID attachmentMenuBotColor::dark_color_fieldID;

attachmentMenuBotColor::attachmentMenuBotColor()
  : light_color_()
  , dark_color_()
{}

attachmentMenuBotColor::attachmentMenuBotColor(int32 light_color_, int32 dark_color_)
  : light_color_(light_color_)
  , dark_color_(dark_color_)
{}

const std::int32_t attachmentMenuBotColor::ID;

object_ptr<attachmentMenuBotColor> attachmentMenuBotColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<attachmentMenuBotColor> res = make_object<attachmentMenuBotColor>();
  res->light_color_ = env->GetIntField(p, res->light_color_fieldID);
  res->dark_color_ = env->GetIntField(p, res->dark_color_fieldID);
  return res;
}

void attachmentMenuBotColor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, light_color_fieldID, light_color_);
  env->SetIntField(s, dark_color_fieldID, dark_color_);
}

void attachmentMenuBotColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AttachmentMenuBotColor");
    s.store_field("lightColor", light_color_);
    s.store_field("darkColor", dark_color_);
    s.store_class_end();
  }
}

void attachmentMenuBotColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AttachmentMenuBotColor").c_str());
    light_color_fieldID = jni::get_field_id(env, Class, "lightColor", "I");
    dark_color_fieldID = jni::get_field_id(env, Class, "darkColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass audio::Class;
jfieldID audio::duration_fieldID;
jfieldID audio::title_fieldID;
jfieldID audio::performer_fieldID;
jfieldID audio::file_name_fieldID;
jfieldID audio::mime_type_fieldID;
jfieldID audio::album_cover_minithumbnail_fieldID;
jfieldID audio::album_cover_thumbnail_fieldID;
jfieldID audio::external_album_covers_fieldID;
jfieldID audio::audio_fieldID;

audio::audio()
  : duration_()
  , title_()
  , performer_()
  , file_name_()
  , mime_type_()
  , album_cover_minithumbnail_()
  , album_cover_thumbnail_()
  , external_album_covers_()
  , audio_()
{}

audio::audio(int32 duration_, string const &title_, string const &performer_, string const &file_name_, string const &mime_type_, object_ptr<minithumbnail> &&album_cover_minithumbnail_, object_ptr<thumbnail> &&album_cover_thumbnail_, array<object_ptr<thumbnail>> &&external_album_covers_, object_ptr<file> &&audio_)
  : duration_(duration_)
  , title_(title_)
  , performer_(performer_)
  , file_name_(file_name_)
  , mime_type_(mime_type_)
  , album_cover_minithumbnail_(std::move(album_cover_minithumbnail_))
  , album_cover_thumbnail_(std::move(album_cover_thumbnail_))
  , external_album_covers_(std::move(external_album_covers_))
  , audio_(std::move(audio_))
{}

const std::int32_t audio::ID;

object_ptr<audio> audio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<audio> res = make_object<audio>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->performer_ = jni::fetch_string(env, p, res->performer_fieldID);
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->album_cover_minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->album_cover_minithumbnail_fieldID));
  res->album_cover_thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->album_cover_thumbnail_fieldID));
  res->external_album_covers_ = jni::FetchVector<thumbnail>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->external_album_covers_fieldID));
  res->audio_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->audio_fieldID));
  return res;
}

void audio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, performer_); if (nextString) { env->SetObjectField(s, performer_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, file_name_); if (nextString) { env->SetObjectField(s, file_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (album_cover_minithumbnail_ != nullptr) { jobject next; album_cover_minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, album_cover_minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (album_cover_thumbnail_ != nullptr) { jobject next; album_cover_thumbnail_->store(env, next); if (next) { env->SetObjectField(s, album_cover_thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, external_album_covers_); if (arr_tmp_) { env->SetObjectField(s, external_album_covers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
}

void audio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Audio");
    s.store_field("duration", duration_);
    s.store_field("title", title_);
    s.store_field("performer", performer_);
    s.store_field("fileName", file_name_);
    s.store_field("mimeType", mime_type_);
    s.store_object_field("albumCoverMinithumbnail", static_cast<const BaseObject *>(album_cover_minithumbnail_.get()));
    s.store_object_field("albumCoverThumbnail", static_cast<const BaseObject *>(album_cover_thumbnail_.get()));
    { s.store_vector_begin("externalAlbumCovers", external_album_covers_.size()); for (const auto &_value : external_album_covers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_class_end();
  }
}

void audio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Audio").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    performer_fieldID = jni::get_field_id(env, Class, "performer", "Ljava/lang/String;");
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    album_cover_minithumbnail_fieldID = jni::get_field_id(env, Class, "albumCoverMinithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    album_cover_thumbnail_fieldID = jni::get_field_id(env, Class, "albumCoverThumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    external_album_covers_fieldID = jni::get_field_id(env, Class, "externalAlbumCovers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeInfo::Class;
jfieldID authenticationCodeInfo::phone_number_fieldID;
jfieldID authenticationCodeInfo::type_fieldID;
jfieldID authenticationCodeInfo::next_type_fieldID;
jfieldID authenticationCodeInfo::timeout_fieldID;

authenticationCodeInfo::authenticationCodeInfo()
  : phone_number_()
  , type_()
  , next_type_()
  , timeout_()
{}

authenticationCodeInfo::authenticationCodeInfo(string const &phone_number_, object_ptr<AuthenticationCodeType> &&type_, object_ptr<AuthenticationCodeType> &&next_type_, int32 timeout_)
  : phone_number_(phone_number_)
  , type_(std::move(type_))
  , next_type_(std::move(next_type_))
  , timeout_(timeout_)
{}

const std::int32_t authenticationCodeInfo::ID;

object_ptr<authenticationCodeInfo> authenticationCodeInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeInfo> res = make_object<authenticationCodeInfo>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->type_ = jni::fetch_tl_object<AuthenticationCodeType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->next_type_ = jni::fetch_tl_object<AuthenticationCodeType>(env, jni::fetch_object(env, p, res->next_type_fieldID));
  res->timeout_ = env->GetIntField(p, res->timeout_fieldID);
  return res;
}

void authenticationCodeInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (next_type_ != nullptr) { jobject next; next_type_->store(env, next); if (next) { env->SetObjectField(s, next_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, timeout_fieldID, timeout_);
}

void authenticationCodeInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeInfo");
    s.store_field("phoneNumber", phone_number_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("nextType", static_cast<const BaseObject *>(next_type_.get()));
    s.store_field("timeout", timeout_);
    s.store_class_end();
  }
}

void authenticationCodeInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeInfo").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AuthenticationCodeType;").c_str());
    next_type_fieldID = jni::get_field_id(env, Class, "nextType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AuthenticationCodeType;").c_str());
    timeout_fieldID = jni::get_field_id(env, Class, "timeout", "I");
    return true;
  }();
  (void)is_inited;
}

jclass AuthenticationCodeType::Class;

object_ptr<AuthenticationCodeType> AuthenticationCodeType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case authenticationCodeTypeTelegramMessage::ID:
      return authenticationCodeTypeTelegramMessage::fetch(env, p);
    case authenticationCodeTypeSms::ID:
      return authenticationCodeTypeSms::fetch(env, p);
    case authenticationCodeTypeSmsWord::ID:
      return authenticationCodeTypeSmsWord::fetch(env, p);
    case authenticationCodeTypeSmsPhrase::ID:
      return authenticationCodeTypeSmsPhrase::fetch(env, p);
    case authenticationCodeTypeCall::ID:
      return authenticationCodeTypeCall::fetch(env, p);
    case authenticationCodeTypeFlashCall::ID:
      return authenticationCodeTypeFlashCall::fetch(env, p);
    case authenticationCodeTypeMissedCall::ID:
      return authenticationCodeTypeMissedCall::fetch(env, p);
    case authenticationCodeTypeFragment::ID:
      return authenticationCodeTypeFragment::fetch(env, p);
    case authenticationCodeTypeFirebaseAndroid::ID:
      return authenticationCodeTypeFirebaseAndroid::fetch(env, p);
    case authenticationCodeTypeFirebaseIos::ID:
      return authenticationCodeTypeFirebaseIos::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void AuthenticationCodeType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeTelegramMessage::Class;
jfieldID authenticationCodeTypeTelegramMessage::length_fieldID;

authenticationCodeTypeTelegramMessage::authenticationCodeTypeTelegramMessage()
  : length_()
{}

authenticationCodeTypeTelegramMessage::authenticationCodeTypeTelegramMessage(int32 length_)
  : length_(length_)
{}

const std::int32_t authenticationCodeTypeTelegramMessage::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeTelegramMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeTelegramMessage> res = make_object<authenticationCodeTypeTelegramMessage>();
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeTelegramMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeTelegramMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeTelegramMessage");
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeTelegramMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeTelegramMessage").c_str());
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeSms::Class;
jfieldID authenticationCodeTypeSms::length_fieldID;

authenticationCodeTypeSms::authenticationCodeTypeSms()
  : length_()
{}

authenticationCodeTypeSms::authenticationCodeTypeSms(int32 length_)
  : length_(length_)
{}

const std::int32_t authenticationCodeTypeSms::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeSms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeSms> res = make_object<authenticationCodeTypeSms>();
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeSms::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeSms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeSms");
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeSms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeSms").c_str());
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeSmsWord::Class;
jfieldID authenticationCodeTypeSmsWord::first_letter_fieldID;

authenticationCodeTypeSmsWord::authenticationCodeTypeSmsWord()
  : first_letter_()
{}

authenticationCodeTypeSmsWord::authenticationCodeTypeSmsWord(string const &first_letter_)
  : first_letter_(first_letter_)
{}

const std::int32_t authenticationCodeTypeSmsWord::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeSmsWord::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeSmsWord> res = make_object<authenticationCodeTypeSmsWord>();
  res->first_letter_ = jni::fetch_string(env, p, res->first_letter_fieldID);
  return std::move(res);
}

void authenticationCodeTypeSmsWord::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, first_letter_); if (nextString) { env->SetObjectField(s, first_letter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authenticationCodeTypeSmsWord::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeSmsWord");
    s.store_field("firstLetter", first_letter_);
    s.store_class_end();
  }
}

void authenticationCodeTypeSmsWord::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeSmsWord").c_str());
    first_letter_fieldID = jni::get_field_id(env, Class, "firstLetter", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeSmsPhrase::Class;
jfieldID authenticationCodeTypeSmsPhrase::first_word_fieldID;

authenticationCodeTypeSmsPhrase::authenticationCodeTypeSmsPhrase()
  : first_word_()
{}

authenticationCodeTypeSmsPhrase::authenticationCodeTypeSmsPhrase(string const &first_word_)
  : first_word_(first_word_)
{}

const std::int32_t authenticationCodeTypeSmsPhrase::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeSmsPhrase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeSmsPhrase> res = make_object<authenticationCodeTypeSmsPhrase>();
  res->first_word_ = jni::fetch_string(env, p, res->first_word_fieldID);
  return std::move(res);
}

void authenticationCodeTypeSmsPhrase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, first_word_); if (nextString) { env->SetObjectField(s, first_word_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authenticationCodeTypeSmsPhrase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeSmsPhrase");
    s.store_field("firstWord", first_word_);
    s.store_class_end();
  }
}

void authenticationCodeTypeSmsPhrase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeSmsPhrase").c_str());
    first_word_fieldID = jni::get_field_id(env, Class, "firstWord", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeCall::Class;
jfieldID authenticationCodeTypeCall::length_fieldID;

authenticationCodeTypeCall::authenticationCodeTypeCall()
  : length_()
{}

authenticationCodeTypeCall::authenticationCodeTypeCall(int32 length_)
  : length_(length_)
{}

const std::int32_t authenticationCodeTypeCall::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeCall> res = make_object<authenticationCodeTypeCall>();
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeCall");
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeCall").c_str());
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeFlashCall::Class;
jfieldID authenticationCodeTypeFlashCall::pattern_fieldID;

authenticationCodeTypeFlashCall::authenticationCodeTypeFlashCall()
  : pattern_()
{}

authenticationCodeTypeFlashCall::authenticationCodeTypeFlashCall(string const &pattern_)
  : pattern_(pattern_)
{}

const std::int32_t authenticationCodeTypeFlashCall::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeFlashCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeFlashCall> res = make_object<authenticationCodeTypeFlashCall>();
  res->pattern_ = jni::fetch_string(env, p, res->pattern_fieldID);
  return std::move(res);
}

void authenticationCodeTypeFlashCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, pattern_); if (nextString) { env->SetObjectField(s, pattern_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authenticationCodeTypeFlashCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeFlashCall");
    s.store_field("pattern", pattern_);
    s.store_class_end();
  }
}

void authenticationCodeTypeFlashCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeFlashCall").c_str());
    pattern_fieldID = jni::get_field_id(env, Class, "pattern", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeMissedCall::Class;
jfieldID authenticationCodeTypeMissedCall::phone_number_prefix_fieldID;
jfieldID authenticationCodeTypeMissedCall::length_fieldID;

authenticationCodeTypeMissedCall::authenticationCodeTypeMissedCall()
  : phone_number_prefix_()
  , length_()
{}

authenticationCodeTypeMissedCall::authenticationCodeTypeMissedCall(string const &phone_number_prefix_, int32 length_)
  : phone_number_prefix_(phone_number_prefix_)
  , length_(length_)
{}

const std::int32_t authenticationCodeTypeMissedCall::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeMissedCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeMissedCall> res = make_object<authenticationCodeTypeMissedCall>();
  res->phone_number_prefix_ = jni::fetch_string(env, p, res->phone_number_prefix_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeMissedCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, phone_number_prefix_); if (nextString) { env->SetObjectField(s, phone_number_prefix_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeMissedCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeMissedCall");
    s.store_field("phoneNumberPrefix", phone_number_prefix_);
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeMissedCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeMissedCall").c_str());
    phone_number_prefix_fieldID = jni::get_field_id(env, Class, "phoneNumberPrefix", "Ljava/lang/String;");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeFragment::Class;
jfieldID authenticationCodeTypeFragment::url_fieldID;
jfieldID authenticationCodeTypeFragment::length_fieldID;

authenticationCodeTypeFragment::authenticationCodeTypeFragment()
  : url_()
  , length_()
{}

authenticationCodeTypeFragment::authenticationCodeTypeFragment(string const &url_, int32 length_)
  : url_(url_)
  , length_(length_)
{}

const std::int32_t authenticationCodeTypeFragment::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeFragment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeFragment> res = make_object<authenticationCodeTypeFragment>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeFragment::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeFragment::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeFragment");
    s.store_field("url", url_);
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeFragment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeFragment").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeFirebaseAndroid::Class;
jfieldID authenticationCodeTypeFirebaseAndroid::device_verification_parameters_fieldID;
jfieldID authenticationCodeTypeFirebaseAndroid::length_fieldID;

authenticationCodeTypeFirebaseAndroid::authenticationCodeTypeFirebaseAndroid()
  : device_verification_parameters_()
  , length_()
{}

authenticationCodeTypeFirebaseAndroid::authenticationCodeTypeFirebaseAndroid(object_ptr<FirebaseDeviceVerificationParameters> &&device_verification_parameters_, int32 length_)
  : device_verification_parameters_(std::move(device_verification_parameters_))
  , length_(length_)
{}

const std::int32_t authenticationCodeTypeFirebaseAndroid::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeFirebaseAndroid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeFirebaseAndroid> res = make_object<authenticationCodeTypeFirebaseAndroid>();
  res->device_verification_parameters_ = jni::fetch_tl_object<FirebaseDeviceVerificationParameters>(env, jni::fetch_object(env, p, res->device_verification_parameters_fieldID));
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeFirebaseAndroid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (device_verification_parameters_ != nullptr) { jobject next; device_verification_parameters_->store(env, next); if (next) { env->SetObjectField(s, device_verification_parameters_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeFirebaseAndroid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeFirebaseAndroid");
    s.store_object_field("deviceVerificationParameters", static_cast<const BaseObject *>(device_verification_parameters_.get()));
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeFirebaseAndroid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeFirebaseAndroid").c_str());
    device_verification_parameters_fieldID = jni::get_field_id(env, Class, "deviceVerificationParameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FirebaseDeviceVerificationParameters;").c_str());
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass authenticationCodeTypeFirebaseIos::Class;
jfieldID authenticationCodeTypeFirebaseIos::receipt_fieldID;
jfieldID authenticationCodeTypeFirebaseIos::push_timeout_fieldID;
jfieldID authenticationCodeTypeFirebaseIos::length_fieldID;

authenticationCodeTypeFirebaseIos::authenticationCodeTypeFirebaseIos()
  : receipt_()
  , push_timeout_()
  , length_()
{}

authenticationCodeTypeFirebaseIos::authenticationCodeTypeFirebaseIos(string const &receipt_, int32 push_timeout_, int32 length_)
  : receipt_(receipt_)
  , push_timeout_(push_timeout_)
  , length_(length_)
{}

const std::int32_t authenticationCodeTypeFirebaseIos::ID;

object_ptr<AuthenticationCodeType> authenticationCodeTypeFirebaseIos::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authenticationCodeTypeFirebaseIos> res = make_object<authenticationCodeTypeFirebaseIos>();
  res->receipt_ = jni::fetch_string(env, p, res->receipt_fieldID);
  res->push_timeout_ = env->GetIntField(p, res->push_timeout_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return std::move(res);
}

void authenticationCodeTypeFirebaseIos::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, receipt_); if (nextString) { env->SetObjectField(s, receipt_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, push_timeout_fieldID, push_timeout_);
  env->SetIntField(s, length_fieldID, length_);
}

void authenticationCodeTypeFirebaseIos::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthenticationCodeTypeFirebaseIos");
    s.store_field("receipt", receipt_);
    s.store_field("pushTimeout", push_timeout_);
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void authenticationCodeTypeFirebaseIos::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthenticationCodeTypeFirebaseIos").c_str());
    receipt_fieldID = jni::get_field_id(env, Class, "receipt", "Ljava/lang/String;");
    push_timeout_fieldID = jni::get_field_id(env, Class, "pushTimeout", "I");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass AuthorizationState::Class;

object_ptr<AuthorizationState> AuthorizationState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case authorizationStateWaitTdlibParameters::ID:
      return authorizationStateWaitTdlibParameters::fetch(env, p);
    case authorizationStateWaitPhoneNumber::ID:
      return authorizationStateWaitPhoneNumber::fetch(env, p);
    case authorizationStateWaitPremiumPurchase::ID:
      return authorizationStateWaitPremiumPurchase::fetch(env, p);
    case authorizationStateWaitEmailAddress::ID:
      return authorizationStateWaitEmailAddress::fetch(env, p);
    case authorizationStateWaitEmailCode::ID:
      return authorizationStateWaitEmailCode::fetch(env, p);
    case authorizationStateWaitCode::ID:
      return authorizationStateWaitCode::fetch(env, p);
    case authorizationStateWaitOtherDeviceConfirmation::ID:
      return authorizationStateWaitOtherDeviceConfirmation::fetch(env, p);
    case authorizationStateWaitRegistration::ID:
      return authorizationStateWaitRegistration::fetch(env, p);
    case authorizationStateWaitPassword::ID:
      return authorizationStateWaitPassword::fetch(env, p);
    case authorizationStateReady::ID:
      return authorizationStateReady::fetch(env, p);
    case authorizationStateLoggingOut::ID:
      return authorizationStateLoggingOut::fetch(env, p);
    case authorizationStateClosing::ID:
      return authorizationStateClosing::fetch(env, p);
    case authorizationStateClosed::ID:
      return authorizationStateClosed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void AuthorizationState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitTdlibParameters::Class;

authorizationStateWaitTdlibParameters::authorizationStateWaitTdlibParameters() {
}

const std::int32_t authorizationStateWaitTdlibParameters::ID;

object_ptr<AuthorizationState> authorizationStateWaitTdlibParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateWaitTdlibParameters>();
}

void authorizationStateWaitTdlibParameters::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateWaitTdlibParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitTdlibParameters");
    s.store_class_end();
  }
}

void authorizationStateWaitTdlibParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitTdlibParameters").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitPhoneNumber::Class;

authorizationStateWaitPhoneNumber::authorizationStateWaitPhoneNumber() {
}

const std::int32_t authorizationStateWaitPhoneNumber::ID;

object_ptr<AuthorizationState> authorizationStateWaitPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateWaitPhoneNumber>();
}

void authorizationStateWaitPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateWaitPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitPhoneNumber");
    s.store_class_end();
  }
}

void authorizationStateWaitPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitPhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitPremiumPurchase::Class;
jfieldID authorizationStateWaitPremiumPurchase::store_product_id_fieldID;

authorizationStateWaitPremiumPurchase::authorizationStateWaitPremiumPurchase()
  : store_product_id_()
{}

authorizationStateWaitPremiumPurchase::authorizationStateWaitPremiumPurchase(string const &store_product_id_)
  : store_product_id_(store_product_id_)
{}

const std::int32_t authorizationStateWaitPremiumPurchase::ID;

object_ptr<AuthorizationState> authorizationStateWaitPremiumPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitPremiumPurchase> res = make_object<authorizationStateWaitPremiumPurchase>();
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  return std::move(res);
}

void authorizationStateWaitPremiumPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authorizationStateWaitPremiumPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitPremiumPurchase");
    s.store_field("storeProductId", store_product_id_);
    s.store_class_end();
  }
}

void authorizationStateWaitPremiumPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitPremiumPurchase").c_str());
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitEmailAddress::Class;
jfieldID authorizationStateWaitEmailAddress::allow_apple_id_fieldID;
jfieldID authorizationStateWaitEmailAddress::allow_google_id_fieldID;

authorizationStateWaitEmailAddress::authorizationStateWaitEmailAddress()
  : allow_apple_id_()
  , allow_google_id_()
{}

authorizationStateWaitEmailAddress::authorizationStateWaitEmailAddress(bool allow_apple_id_, bool allow_google_id_)
  : allow_apple_id_(allow_apple_id_)
  , allow_google_id_(allow_google_id_)
{}

const std::int32_t authorizationStateWaitEmailAddress::ID;

object_ptr<AuthorizationState> authorizationStateWaitEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitEmailAddress> res = make_object<authorizationStateWaitEmailAddress>();
  res->allow_apple_id_ = (env->GetBooleanField(p, res->allow_apple_id_fieldID) != 0);
  res->allow_google_id_ = (env->GetBooleanField(p, res->allow_google_id_fieldID) != 0);
  return std::move(res);
}

void authorizationStateWaitEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, allow_apple_id_fieldID, allow_apple_id_);
  env->SetBooleanField(s, allow_google_id_fieldID, allow_google_id_);
}

void authorizationStateWaitEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitEmailAddress");
    s.store_field("allowAppleId", allow_apple_id_);
    s.store_field("allowGoogleId", allow_google_id_);
    s.store_class_end();
  }
}

void authorizationStateWaitEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitEmailAddress").c_str());
    allow_apple_id_fieldID = jni::get_field_id(env, Class, "allowAppleId", "Z");
    allow_google_id_fieldID = jni::get_field_id(env, Class, "allowGoogleId", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitEmailCode::Class;
jfieldID authorizationStateWaitEmailCode::allow_apple_id_fieldID;
jfieldID authorizationStateWaitEmailCode::allow_google_id_fieldID;
jfieldID authorizationStateWaitEmailCode::code_info_fieldID;
jfieldID authorizationStateWaitEmailCode::email_address_reset_state_fieldID;

authorizationStateWaitEmailCode::authorizationStateWaitEmailCode()
  : allow_apple_id_()
  , allow_google_id_()
  , code_info_()
  , email_address_reset_state_()
{}

authorizationStateWaitEmailCode::authorizationStateWaitEmailCode(bool allow_apple_id_, bool allow_google_id_, object_ptr<emailAddressAuthenticationCodeInfo> &&code_info_, object_ptr<EmailAddressResetState> &&email_address_reset_state_)
  : allow_apple_id_(allow_apple_id_)
  , allow_google_id_(allow_google_id_)
  , code_info_(std::move(code_info_))
  , email_address_reset_state_(std::move(email_address_reset_state_))
{}

const std::int32_t authorizationStateWaitEmailCode::ID;

object_ptr<AuthorizationState> authorizationStateWaitEmailCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitEmailCode> res = make_object<authorizationStateWaitEmailCode>();
  res->allow_apple_id_ = (env->GetBooleanField(p, res->allow_apple_id_fieldID) != 0);
  res->allow_google_id_ = (env->GetBooleanField(p, res->allow_google_id_fieldID) != 0);
  res->code_info_ = jni::fetch_tl_object<emailAddressAuthenticationCodeInfo>(env, jni::fetch_object(env, p, res->code_info_fieldID));
  res->email_address_reset_state_ = jni::fetch_tl_object<EmailAddressResetState>(env, jni::fetch_object(env, p, res->email_address_reset_state_fieldID));
  return std::move(res);
}

void authorizationStateWaitEmailCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, allow_apple_id_fieldID, allow_apple_id_);
  env->SetBooleanField(s, allow_google_id_fieldID, allow_google_id_);
  if (code_info_ != nullptr) { jobject next; code_info_->store(env, next); if (next) { env->SetObjectField(s, code_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (email_address_reset_state_ != nullptr) { jobject next; email_address_reset_state_->store(env, next); if (next) { env->SetObjectField(s, email_address_reset_state_fieldID, next); env->DeleteLocalRef(next); } }
}

void authorizationStateWaitEmailCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitEmailCode");
    s.store_field("allowAppleId", allow_apple_id_);
    s.store_field("allowGoogleId", allow_google_id_);
    s.store_object_field("codeInfo", static_cast<const BaseObject *>(code_info_.get()));
    s.store_object_field("emailAddressResetState", static_cast<const BaseObject *>(email_address_reset_state_.get()));
    s.store_class_end();
  }
}

void authorizationStateWaitEmailCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitEmailCode").c_str());
    allow_apple_id_fieldID = jni::get_field_id(env, Class, "allowAppleId", "Z");
    allow_google_id_fieldID = jni::get_field_id(env, Class, "allowGoogleId", "Z");
    code_info_fieldID = jni::get_field_id(env, Class, "codeInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationCodeInfo;").c_str());
    email_address_reset_state_fieldID = jni::get_field_id(env, Class, "emailAddressResetState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmailAddressResetState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitCode::Class;
jfieldID authorizationStateWaitCode::code_info_fieldID;

authorizationStateWaitCode::authorizationStateWaitCode()
  : code_info_()
{}

authorizationStateWaitCode::authorizationStateWaitCode(object_ptr<authenticationCodeInfo> &&code_info_)
  : code_info_(std::move(code_info_))
{}

const std::int32_t authorizationStateWaitCode::ID;

object_ptr<AuthorizationState> authorizationStateWaitCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitCode> res = make_object<authorizationStateWaitCode>();
  res->code_info_ = jni::fetch_tl_object<authenticationCodeInfo>(env, jni::fetch_object(env, p, res->code_info_fieldID));
  return std::move(res);
}

void authorizationStateWaitCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (code_info_ != nullptr) { jobject next; code_info_->store(env, next); if (next) { env->SetObjectField(s, code_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void authorizationStateWaitCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitCode");
    s.store_object_field("codeInfo", static_cast<const BaseObject *>(code_info_.get()));
    s.store_class_end();
  }
}

void authorizationStateWaitCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitCode").c_str());
    code_info_fieldID = jni::get_field_id(env, Class, "codeInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AuthenticationCodeInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitOtherDeviceConfirmation::Class;
jfieldID authorizationStateWaitOtherDeviceConfirmation::link_fieldID;

authorizationStateWaitOtherDeviceConfirmation::authorizationStateWaitOtherDeviceConfirmation()
  : link_()
{}

authorizationStateWaitOtherDeviceConfirmation::authorizationStateWaitOtherDeviceConfirmation(string const &link_)
  : link_(link_)
{}

const std::int32_t authorizationStateWaitOtherDeviceConfirmation::ID;

object_ptr<AuthorizationState> authorizationStateWaitOtherDeviceConfirmation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitOtherDeviceConfirmation> res = make_object<authorizationStateWaitOtherDeviceConfirmation>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return std::move(res);
}

void authorizationStateWaitOtherDeviceConfirmation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_); if (nextString) { env->SetObjectField(s, link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authorizationStateWaitOtherDeviceConfirmation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitOtherDeviceConfirmation");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

void authorizationStateWaitOtherDeviceConfirmation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitOtherDeviceConfirmation").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitRegistration::Class;
jfieldID authorizationStateWaitRegistration::terms_of_service_fieldID;

authorizationStateWaitRegistration::authorizationStateWaitRegistration()
  : terms_of_service_()
{}

authorizationStateWaitRegistration::authorizationStateWaitRegistration(object_ptr<termsOfService> &&terms_of_service_)
  : terms_of_service_(std::move(terms_of_service_))
{}

const std::int32_t authorizationStateWaitRegistration::ID;

object_ptr<AuthorizationState> authorizationStateWaitRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitRegistration> res = make_object<authorizationStateWaitRegistration>();
  res->terms_of_service_ = jni::fetch_tl_object<termsOfService>(env, jni::fetch_object(env, p, res->terms_of_service_fieldID));
  return std::move(res);
}

void authorizationStateWaitRegistration::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (terms_of_service_ != nullptr) { jobject next; terms_of_service_->store(env, next); if (next) { env->SetObjectField(s, terms_of_service_fieldID, next); env->DeleteLocalRef(next); } }
}

void authorizationStateWaitRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitRegistration");
    s.store_object_field("termsOfService", static_cast<const BaseObject *>(terms_of_service_.get()));
    s.store_class_end();
  }
}

void authorizationStateWaitRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitRegistration").c_str());
    terms_of_service_fieldID = jni::get_field_id(env, Class, "termsOfService", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TermsOfService;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateWaitPassword::Class;
jfieldID authorizationStateWaitPassword::password_hint_fieldID;
jfieldID authorizationStateWaitPassword::has_recovery_email_address_fieldID;
jfieldID authorizationStateWaitPassword::has_passport_data_fieldID;
jfieldID authorizationStateWaitPassword::recovery_email_address_pattern_fieldID;

authorizationStateWaitPassword::authorizationStateWaitPassword()
  : password_hint_()
  , has_recovery_email_address_()
  , has_passport_data_()
  , recovery_email_address_pattern_()
{}

authorizationStateWaitPassword::authorizationStateWaitPassword(string const &password_hint_, bool has_recovery_email_address_, bool has_passport_data_, string const &recovery_email_address_pattern_)
  : password_hint_(password_hint_)
  , has_recovery_email_address_(has_recovery_email_address_)
  , has_passport_data_(has_passport_data_)
  , recovery_email_address_pattern_(recovery_email_address_pattern_)
{}

const std::int32_t authorizationStateWaitPassword::ID;

object_ptr<AuthorizationState> authorizationStateWaitPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<authorizationStateWaitPassword> res = make_object<authorizationStateWaitPassword>();
  res->password_hint_ = jni::fetch_string(env, p, res->password_hint_fieldID);
  res->has_recovery_email_address_ = (env->GetBooleanField(p, res->has_recovery_email_address_fieldID) != 0);
  res->has_passport_data_ = (env->GetBooleanField(p, res->has_passport_data_fieldID) != 0);
  res->recovery_email_address_pattern_ = jni::fetch_string(env, p, res->recovery_email_address_pattern_fieldID);
  return std::move(res);
}

void authorizationStateWaitPassword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, password_hint_); if (nextString) { env->SetObjectField(s, password_hint_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, has_recovery_email_address_fieldID, has_recovery_email_address_);
  env->SetBooleanField(s, has_passport_data_fieldID, has_passport_data_);
  { jstring nextString = jni::to_jstring(env, recovery_email_address_pattern_); if (nextString) { env->SetObjectField(s, recovery_email_address_pattern_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void authorizationStateWaitPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateWaitPassword");
    s.store_field("passwordHint", password_hint_);
    s.store_field("hasRecoveryEmailAddress", has_recovery_email_address_);
    s.store_field("hasPassportData", has_passport_data_);
    s.store_field("recoveryEmailAddressPattern", recovery_email_address_pattern_);
    s.store_class_end();
  }
}

void authorizationStateWaitPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateWaitPassword").c_str());
    password_hint_fieldID = jni::get_field_id(env, Class, "passwordHint", "Ljava/lang/String;");
    has_recovery_email_address_fieldID = jni::get_field_id(env, Class, "hasRecoveryEmailAddress", "Z");
    has_passport_data_fieldID = jni::get_field_id(env, Class, "hasPassportData", "Z");
    recovery_email_address_pattern_fieldID = jni::get_field_id(env, Class, "recoveryEmailAddressPattern", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateReady::Class;

authorizationStateReady::authorizationStateReady() {
}

const std::int32_t authorizationStateReady::ID;

object_ptr<AuthorizationState> authorizationStateReady::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateReady>();
}

void authorizationStateReady::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateReady::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateReady");
    s.store_class_end();
  }
}

void authorizationStateReady::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateReady").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateLoggingOut::Class;

authorizationStateLoggingOut::authorizationStateLoggingOut() {
}

const std::int32_t authorizationStateLoggingOut::ID;

object_ptr<AuthorizationState> authorizationStateLoggingOut::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateLoggingOut>();
}

void authorizationStateLoggingOut::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateLoggingOut::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateLoggingOut");
    s.store_class_end();
  }
}

void authorizationStateLoggingOut::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateLoggingOut").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateClosing::Class;

authorizationStateClosing::authorizationStateClosing() {
}

const std::int32_t authorizationStateClosing::ID;

object_ptr<AuthorizationState> authorizationStateClosing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateClosing>();
}

void authorizationStateClosing::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateClosing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateClosing");
    s.store_class_end();
  }
}

void authorizationStateClosing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateClosing").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass authorizationStateClosed::Class;

authorizationStateClosed::authorizationStateClosed() {
}

const std::int32_t authorizationStateClosed::ID;

object_ptr<AuthorizationState> authorizationStateClosed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<authorizationStateClosed>();
}

void authorizationStateClosed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void authorizationStateClosed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AuthorizationStateClosed");
    s.store_class_end();
  }
}

void authorizationStateClosed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AuthorizationStateClosed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autoDownloadSettings::Class;
jfieldID autoDownloadSettings::is_auto_download_enabled_fieldID;
jfieldID autoDownloadSettings::max_photo_file_size_fieldID;
jfieldID autoDownloadSettings::max_video_file_size_fieldID;
jfieldID autoDownloadSettings::max_other_file_size_fieldID;
jfieldID autoDownloadSettings::video_upload_bitrate_fieldID;
jfieldID autoDownloadSettings::preload_large_videos_fieldID;
jfieldID autoDownloadSettings::preload_next_audio_fieldID;
jfieldID autoDownloadSettings::preload_stories_fieldID;
jfieldID autoDownloadSettings::use_less_data_for_calls_fieldID;

autoDownloadSettings::autoDownloadSettings()
  : is_auto_download_enabled_()
  , max_photo_file_size_()
  , max_video_file_size_()
  , max_other_file_size_()
  , video_upload_bitrate_()
  , preload_large_videos_()
  , preload_next_audio_()
  , preload_stories_()
  , use_less_data_for_calls_()
{}

autoDownloadSettings::autoDownloadSettings(bool is_auto_download_enabled_, int32 max_photo_file_size_, int53 max_video_file_size_, int53 max_other_file_size_, int32 video_upload_bitrate_, bool preload_large_videos_, bool preload_next_audio_, bool preload_stories_, bool use_less_data_for_calls_)
  : is_auto_download_enabled_(is_auto_download_enabled_)
  , max_photo_file_size_(max_photo_file_size_)
  , max_video_file_size_(max_video_file_size_)
  , max_other_file_size_(max_other_file_size_)
  , video_upload_bitrate_(video_upload_bitrate_)
  , preload_large_videos_(preload_large_videos_)
  , preload_next_audio_(preload_next_audio_)
  , preload_stories_(preload_stories_)
  , use_less_data_for_calls_(use_less_data_for_calls_)
{}

const std::int32_t autoDownloadSettings::ID;

object_ptr<autoDownloadSettings> autoDownloadSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<autoDownloadSettings> res = make_object<autoDownloadSettings>();
  res->is_auto_download_enabled_ = (env->GetBooleanField(p, res->is_auto_download_enabled_fieldID) != 0);
  res->max_photo_file_size_ = env->GetIntField(p, res->max_photo_file_size_fieldID);
  res->max_video_file_size_ = env->GetLongField(p, res->max_video_file_size_fieldID);
  res->max_other_file_size_ = env->GetLongField(p, res->max_other_file_size_fieldID);
  res->video_upload_bitrate_ = env->GetIntField(p, res->video_upload_bitrate_fieldID);
  res->preload_large_videos_ = (env->GetBooleanField(p, res->preload_large_videos_fieldID) != 0);
  res->preload_next_audio_ = (env->GetBooleanField(p, res->preload_next_audio_fieldID) != 0);
  res->preload_stories_ = (env->GetBooleanField(p, res->preload_stories_fieldID) != 0);
  res->use_less_data_for_calls_ = (env->GetBooleanField(p, res->use_less_data_for_calls_fieldID) != 0);
  return res;
}

void autoDownloadSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_auto_download_enabled_fieldID, is_auto_download_enabled_);
  env->SetIntField(s, max_photo_file_size_fieldID, max_photo_file_size_);
  env->SetLongField(s, max_video_file_size_fieldID, max_video_file_size_);
  env->SetLongField(s, max_other_file_size_fieldID, max_other_file_size_);
  env->SetIntField(s, video_upload_bitrate_fieldID, video_upload_bitrate_);
  env->SetBooleanField(s, preload_large_videos_fieldID, preload_large_videos_);
  env->SetBooleanField(s, preload_next_audio_fieldID, preload_next_audio_);
  env->SetBooleanField(s, preload_stories_fieldID, preload_stories_);
  env->SetBooleanField(s, use_less_data_for_calls_fieldID, use_less_data_for_calls_);
}

void autoDownloadSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutoDownloadSettings");
    s.store_field("isAutoDownloadEnabled", is_auto_download_enabled_);
    s.store_field("maxPhotoFileSize", max_photo_file_size_);
    s.store_field("maxVideoFileSize", max_video_file_size_);
    s.store_field("maxOtherFileSize", max_other_file_size_);
    s.store_field("videoUploadBitrate", video_upload_bitrate_);
    s.store_field("preloadLargeVideos", preload_large_videos_);
    s.store_field("preloadNextAudio", preload_next_audio_);
    s.store_field("preloadStories", preload_stories_);
    s.store_field("useLessDataForCalls", use_less_data_for_calls_);
    s.store_class_end();
  }
}

void autoDownloadSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutoDownloadSettings").c_str());
    is_auto_download_enabled_fieldID = jni::get_field_id(env, Class, "isAutoDownloadEnabled", "Z");
    max_photo_file_size_fieldID = jni::get_field_id(env, Class, "maxPhotoFileSize", "I");
    max_video_file_size_fieldID = jni::get_field_id(env, Class, "maxVideoFileSize", "J");
    max_other_file_size_fieldID = jni::get_field_id(env, Class, "maxOtherFileSize", "J");
    video_upload_bitrate_fieldID = jni::get_field_id(env, Class, "videoUploadBitrate", "I");
    preload_large_videos_fieldID = jni::get_field_id(env, Class, "preloadLargeVideos", "Z");
    preload_next_audio_fieldID = jni::get_field_id(env, Class, "preloadNextAudio", "Z");
    preload_stories_fieldID = jni::get_field_id(env, Class, "preloadStories", "Z");
    use_less_data_for_calls_fieldID = jni::get_field_id(env, Class, "useLessDataForCalls", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass autoDownloadSettingsPresets::Class;
jfieldID autoDownloadSettingsPresets::low_fieldID;
jfieldID autoDownloadSettingsPresets::medium_fieldID;
jfieldID autoDownloadSettingsPresets::high_fieldID;

autoDownloadSettingsPresets::autoDownloadSettingsPresets()
  : low_()
  , medium_()
  , high_()
{}

autoDownloadSettingsPresets::autoDownloadSettingsPresets(object_ptr<autoDownloadSettings> &&low_, object_ptr<autoDownloadSettings> &&medium_, object_ptr<autoDownloadSettings> &&high_)
  : low_(std::move(low_))
  , medium_(std::move(medium_))
  , high_(std::move(high_))
{}

const std::int32_t autoDownloadSettingsPresets::ID;

object_ptr<autoDownloadSettingsPresets> autoDownloadSettingsPresets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<autoDownloadSettingsPresets> res = make_object<autoDownloadSettingsPresets>();
  res->low_ = jni::fetch_tl_object<autoDownloadSettings>(env, jni::fetch_object(env, p, res->low_fieldID));
  res->medium_ = jni::fetch_tl_object<autoDownloadSettings>(env, jni::fetch_object(env, p, res->medium_fieldID));
  res->high_ = jni::fetch_tl_object<autoDownloadSettings>(env, jni::fetch_object(env, p, res->high_fieldID));
  return res;
}

void autoDownloadSettingsPresets::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (low_ != nullptr) { jobject next; low_->store(env, next); if (next) { env->SetObjectField(s, low_fieldID, next); env->DeleteLocalRef(next); } }
  if (medium_ != nullptr) { jobject next; medium_->store(env, next); if (next) { env->SetObjectField(s, medium_fieldID, next); env->DeleteLocalRef(next); } }
  if (high_ != nullptr) { jobject next; high_->store(env, next); if (next) { env->SetObjectField(s, high_fieldID, next); env->DeleteLocalRef(next); } }
}

void autoDownloadSettingsPresets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutoDownloadSettingsPresets");
    s.store_object_field("low", static_cast<const BaseObject *>(low_.get()));
    s.store_object_field("medium", static_cast<const BaseObject *>(medium_.get()));
    s.store_object_field("high", static_cast<const BaseObject *>(high_.get()));
    s.store_class_end();
  }
}

void autoDownloadSettingsPresets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutoDownloadSettingsPresets").c_str());
    low_fieldID = jni::get_field_id(env, Class, "low", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutoDownloadSettings;").c_str());
    medium_fieldID = jni::get_field_id(env, Class, "medium", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutoDownloadSettings;").c_str());
    high_fieldID = jni::get_field_id(env, Class, "high", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutoDownloadSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettings::Class;
jfieldID autosaveSettings::private_chat_settings_fieldID;
jfieldID autosaveSettings::group_settings_fieldID;
jfieldID autosaveSettings::channel_settings_fieldID;
jfieldID autosaveSettings::exceptions_fieldID;

autosaveSettings::autosaveSettings()
  : private_chat_settings_()
  , group_settings_()
  , channel_settings_()
  , exceptions_()
{}

autosaveSettings::autosaveSettings(object_ptr<scopeAutosaveSettings> &&private_chat_settings_, object_ptr<scopeAutosaveSettings> &&group_settings_, object_ptr<scopeAutosaveSettings> &&channel_settings_, array<object_ptr<autosaveSettingsException>> &&exceptions_)
  : private_chat_settings_(std::move(private_chat_settings_))
  , group_settings_(std::move(group_settings_))
  , channel_settings_(std::move(channel_settings_))
  , exceptions_(std::move(exceptions_))
{}

const std::int32_t autosaveSettings::ID;

object_ptr<autosaveSettings> autosaveSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<autosaveSettings> res = make_object<autosaveSettings>();
  res->private_chat_settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->private_chat_settings_fieldID));
  res->group_settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->group_settings_fieldID));
  res->channel_settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->channel_settings_fieldID));
  res->exceptions_ = jni::FetchVector<autosaveSettingsException>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->exceptions_fieldID));
  return res;
}

void autosaveSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (private_chat_settings_ != nullptr) { jobject next; private_chat_settings_->store(env, next); if (next) { env->SetObjectField(s, private_chat_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (group_settings_ != nullptr) { jobject next; group_settings_->store(env, next); if (next) { env->SetObjectField(s, group_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (channel_settings_ != nullptr) { jobject next; channel_settings_->store(env, next); if (next) { env->SetObjectField(s, channel_settings_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, exceptions_); if (arr_tmp_) { env->SetObjectField(s, exceptions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void autosaveSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettings");
    s.store_object_field("privateChatSettings", static_cast<const BaseObject *>(private_chat_settings_.get()));
    s.store_object_field("groupSettings", static_cast<const BaseObject *>(group_settings_.get()));
    s.store_object_field("channelSettings", static_cast<const BaseObject *>(channel_settings_.get()));
    { s.store_vector_begin("exceptions", exceptions_.size()); for (const auto &_value : exceptions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void autosaveSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettings").c_str());
    private_chat_settings_fieldID = jni::get_field_id(env, Class, "privateChatSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    group_settings_fieldID = jni::get_field_id(env, Class, "groupSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    channel_settings_fieldID = jni::get_field_id(env, Class, "channelSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    exceptions_fieldID = jni::get_field_id(env, Class, "exceptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AutosaveSettingsException;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettingsException::Class;
jfieldID autosaveSettingsException::chat_id_fieldID;
jfieldID autosaveSettingsException::settings_fieldID;

autosaveSettingsException::autosaveSettingsException()
  : chat_id_()
  , settings_()
{}

autosaveSettingsException::autosaveSettingsException(int53 chat_id_, object_ptr<scopeAutosaveSettings> &&settings_)
  : chat_id_(chat_id_)
  , settings_(std::move(settings_))
{}

const std::int32_t autosaveSettingsException::ID;

object_ptr<autosaveSettingsException> autosaveSettingsException::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<autosaveSettingsException> res = make_object<autosaveSettingsException>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void autosaveSettingsException::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (settings_ != nullptr) { jobject next; settings_->store(env, next); if (next) { env->SetObjectField(s, settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void autosaveSettingsException::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettingsException");
    s.store_field("chatId", chat_id_);
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

void autosaveSettingsException::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsException").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass AutosaveSettingsScope::Class;

object_ptr<AutosaveSettingsScope> AutosaveSettingsScope::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case autosaveSettingsScopePrivateChats::ID:
      return autosaveSettingsScopePrivateChats::fetch(env, p);
    case autosaveSettingsScopeGroupChats::ID:
      return autosaveSettingsScopeGroupChats::fetch(env, p);
    case autosaveSettingsScopeChannelChats::ID:
      return autosaveSettingsScopeChannelChats::fetch(env, p);
    case autosaveSettingsScopeChat::ID:
      return autosaveSettingsScopeChat::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void AutosaveSettingsScope::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsScope").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettingsScopePrivateChats::Class;

autosaveSettingsScopePrivateChats::autosaveSettingsScopePrivateChats() {
}

const std::int32_t autosaveSettingsScopePrivateChats::ID;

object_ptr<AutosaveSettingsScope> autosaveSettingsScopePrivateChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<autosaveSettingsScopePrivateChats>();
}

void autosaveSettingsScopePrivateChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void autosaveSettingsScopePrivateChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettingsScopePrivateChats");
    s.store_class_end();
  }
}

void autosaveSettingsScopePrivateChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsScopePrivateChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettingsScopeGroupChats::Class;

autosaveSettingsScopeGroupChats::autosaveSettingsScopeGroupChats() {
}

const std::int32_t autosaveSettingsScopeGroupChats::ID;

object_ptr<AutosaveSettingsScope> autosaveSettingsScopeGroupChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<autosaveSettingsScopeGroupChats>();
}

void autosaveSettingsScopeGroupChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void autosaveSettingsScopeGroupChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettingsScopeGroupChats");
    s.store_class_end();
  }
}

void autosaveSettingsScopeGroupChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsScopeGroupChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettingsScopeChannelChats::Class;

autosaveSettingsScopeChannelChats::autosaveSettingsScopeChannelChats() {
}

const std::int32_t autosaveSettingsScopeChannelChats::ID;

object_ptr<AutosaveSettingsScope> autosaveSettingsScopeChannelChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<autosaveSettingsScopeChannelChats>();
}

void autosaveSettingsScopeChannelChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void autosaveSettingsScopeChannelChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettingsScopeChannelChats");
    s.store_class_end();
  }
}

void autosaveSettingsScopeChannelChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsScopeChannelChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass autosaveSettingsScopeChat::Class;
jfieldID autosaveSettingsScopeChat::chat_id_fieldID;

autosaveSettingsScopeChat::autosaveSettingsScopeChat()
  : chat_id_()
{}

autosaveSettingsScopeChat::autosaveSettingsScopeChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t autosaveSettingsScopeChat::ID;

object_ptr<AutosaveSettingsScope> autosaveSettingsScopeChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<autosaveSettingsScopeChat> res = make_object<autosaveSettingsScopeChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void autosaveSettingsScopeChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void autosaveSettingsScopeChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AutosaveSettingsScopeChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void autosaveSettingsScopeChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AutosaveSettingsScopeChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass availableGift::Class;
jfieldID availableGift::gift_fieldID;
jfieldID availableGift::resale_count_fieldID;
jfieldID availableGift::min_resale_star_count_fieldID;
jfieldID availableGift::title_fieldID;

availableGift::availableGift()
  : gift_()
  , resale_count_()
  , min_resale_star_count_()
  , title_()
{}

availableGift::availableGift(object_ptr<gift> &&gift_, int32 resale_count_, int53 min_resale_star_count_, string const &title_)
  : gift_(std::move(gift_))
  , resale_count_(resale_count_)
  , min_resale_star_count_(min_resale_star_count_)
  , title_(title_)
{}

const std::int32_t availableGift::ID;

object_ptr<availableGift> availableGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<availableGift> res = make_object<availableGift>();
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->resale_count_ = env->GetIntField(p, res->resale_count_fieldID);
  res->min_resale_star_count_ = env->GetLongField(p, res->min_resale_star_count_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void availableGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, resale_count_fieldID, resale_count_);
  env->SetLongField(s, min_resale_star_count_fieldID, min_resale_star_count_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void availableGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AvailableGift");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_field("resaleCount", resale_count_);
    s.store_field("minResaleStarCount", min_resale_star_count_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void availableGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AvailableGift").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    resale_count_fieldID = jni::get_field_id(env, Class, "resaleCount", "I");
    min_resale_star_count_fieldID = jni::get_field_id(env, Class, "minResaleStarCount", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass availableGifts::Class;
jfieldID availableGifts::gifts_fieldID;

availableGifts::availableGifts()
  : gifts_()
{}

availableGifts::availableGifts(array<object_ptr<availableGift>> &&gifts_)
  : gifts_(std::move(gifts_))
{}

const std::int32_t availableGifts::ID;

object_ptr<availableGifts> availableGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<availableGifts> res = make_object<availableGifts>();
  res->gifts_ = jni::FetchVector<availableGift>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->gifts_fieldID));
  return res;
}

void availableGifts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, gifts_); if (arr_tmp_) { env->SetObjectField(s, gifts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void availableGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AvailableGifts");
    { s.store_vector_begin("gifts", gifts_.size()); for (const auto &_value : gifts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void availableGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AvailableGifts").c_str());
    gifts_fieldID = jni::get_field_id(env, Class, "gifts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AvailableGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass availableReaction::Class;
jfieldID availableReaction::type_fieldID;
jfieldID availableReaction::needs_premium_fieldID;

availableReaction::availableReaction()
  : type_()
  , needs_premium_()
{}

availableReaction::availableReaction(object_ptr<ReactionType> &&type_, bool needs_premium_)
  : type_(std::move(type_))
  , needs_premium_(needs_premium_)
{}

const std::int32_t availableReaction::ID;

object_ptr<availableReaction> availableReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<availableReaction> res = make_object<availableReaction>();
  res->type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->needs_premium_ = (env->GetBooleanField(p, res->needs_premium_fieldID) != 0);
  return res;
}

void availableReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, needs_premium_fieldID, needs_premium_);
}

void availableReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AvailableReaction");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("needsPremium", needs_premium_);
    s.store_class_end();
  }
}

void availableReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AvailableReaction").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    needs_premium_fieldID = jni::get_field_id(env, Class, "needsPremium", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass availableReactions::Class;
jfieldID availableReactions::top_reactions_fieldID;
jfieldID availableReactions::recent_reactions_fieldID;
jfieldID availableReactions::popular_reactions_fieldID;
jfieldID availableReactions::allow_custom_emoji_fieldID;
jfieldID availableReactions::are_tags_fieldID;
jfieldID availableReactions::unavailability_reason_fieldID;

availableReactions::availableReactions()
  : top_reactions_()
  , recent_reactions_()
  , popular_reactions_()
  , allow_custom_emoji_()
  , are_tags_()
  , unavailability_reason_()
{}

availableReactions::availableReactions(array<object_ptr<availableReaction>> &&top_reactions_, array<object_ptr<availableReaction>> &&recent_reactions_, array<object_ptr<availableReaction>> &&popular_reactions_, bool allow_custom_emoji_, bool are_tags_, object_ptr<ReactionUnavailabilityReason> &&unavailability_reason_)
  : top_reactions_(std::move(top_reactions_))
  , recent_reactions_(std::move(recent_reactions_))
  , popular_reactions_(std::move(popular_reactions_))
  , allow_custom_emoji_(allow_custom_emoji_)
  , are_tags_(are_tags_)
  , unavailability_reason_(std::move(unavailability_reason_))
{}

const std::int32_t availableReactions::ID;

object_ptr<availableReactions> availableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<availableReactions> res = make_object<availableReactions>();
  res->top_reactions_ = jni::FetchVector<availableReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->top_reactions_fieldID));
  res->recent_reactions_ = jni::FetchVector<availableReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_reactions_fieldID));
  res->popular_reactions_ = jni::FetchVector<availableReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->popular_reactions_fieldID));
  res->allow_custom_emoji_ = (env->GetBooleanField(p, res->allow_custom_emoji_fieldID) != 0);
  res->are_tags_ = (env->GetBooleanField(p, res->are_tags_fieldID) != 0);
  res->unavailability_reason_ = jni::fetch_tl_object<ReactionUnavailabilityReason>(env, jni::fetch_object(env, p, res->unavailability_reason_fieldID));
  return res;
}

void availableReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, top_reactions_); if (arr_tmp_) { env->SetObjectField(s, top_reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, recent_reactions_); if (arr_tmp_) { env->SetObjectField(s, recent_reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, popular_reactions_); if (arr_tmp_) { env->SetObjectField(s, popular_reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, allow_custom_emoji_fieldID, allow_custom_emoji_);
  env->SetBooleanField(s, are_tags_fieldID, are_tags_);
  if (unavailability_reason_ != nullptr) { jobject next; unavailability_reason_->store(env, next); if (next) { env->SetObjectField(s, unavailability_reason_fieldID, next); env->DeleteLocalRef(next); } }
}

void availableReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AvailableReactions");
    { s.store_vector_begin("topReactions", top_reactions_.size()); for (const auto &_value : top_reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("recentReactions", recent_reactions_.size()); for (const auto &_value : recent_reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("popularReactions", popular_reactions_.size()); for (const auto &_value : popular_reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("allowCustomEmoji", allow_custom_emoji_);
    s.store_field("areTags", are_tags_);
    s.store_object_field("unavailabilityReason", static_cast<const BaseObject *>(unavailability_reason_.get()));
    s.store_class_end();
  }
}

void availableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AvailableReactions").c_str());
    top_reactions_fieldID = jni::get_field_id(env, Class, "topReactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AvailableReaction;").c_str());
    recent_reactions_fieldID = jni::get_field_id(env, Class, "recentReactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AvailableReaction;").c_str());
    popular_reactions_fieldID = jni::get_field_id(env, Class, "popularReactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AvailableReaction;").c_str());
    allow_custom_emoji_fieldID = jni::get_field_id(env, Class, "allowCustomEmoji", "Z");
    are_tags_fieldID = jni::get_field_id(env, Class, "areTags", "Z");
    unavailability_reason_fieldID = jni::get_field_id(env, Class, "unavailabilityReason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionUnavailabilityReason;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass background::Class;
jfieldID background::id_fieldID;
jfieldID background::is_default_fieldID;
jfieldID background::is_dark_fieldID;
jfieldID background::name_fieldID;
jfieldID background::document_fieldID;
jfieldID background::type_fieldID;

background::background()
  : id_()
  , is_default_()
  , is_dark_()
  , name_()
  , document_()
  , type_()
{}

background::background(int64 id_, bool is_default_, bool is_dark_, string const &name_, object_ptr<document> &&document_, object_ptr<BackgroundType> &&type_)
  : id_(id_)
  , is_default_(is_default_)
  , is_dark_(is_dark_)
  , name_(name_)
  , document_(std::move(document_))
  , type_(std::move(type_))
{}

const std::int32_t background::ID;

object_ptr<background> background::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<background> res = make_object<background>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->is_default_ = (env->GetBooleanField(p, res->is_default_fieldID) != 0);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->type_ = jni::fetch_tl_object<BackgroundType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void background::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetBooleanField(s, is_default_fieldID, is_default_);
  env->SetBooleanField(s, is_dark_fieldID, is_dark_);
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void background::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Background");
    s.store_field("id", id_);
    s.store_field("isDefault", is_default_);
    s.store_field("isDark", is_dark_);
    s.store_field("name", name_);
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void background::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Background").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    is_default_fieldID = jni::get_field_id(env, Class, "isDefault", "Z");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass BackgroundFill::Class;

object_ptr<BackgroundFill> BackgroundFill::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case backgroundFillSolid::ID:
      return backgroundFillSolid::fetch(env, p);
    case backgroundFillGradient::ID:
      return backgroundFillGradient::fetch(env, p);
    case backgroundFillFreeformGradient::ID:
      return backgroundFillFreeformGradient::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BackgroundFill::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundFill").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass backgroundFillSolid::Class;
jfieldID backgroundFillSolid::color_fieldID;

backgroundFillSolid::backgroundFillSolid()
  : color_()
{}

backgroundFillSolid::backgroundFillSolid(int32 color_)
  : color_(color_)
{}

const std::int32_t backgroundFillSolid::ID;

object_ptr<BackgroundFill> backgroundFillSolid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundFillSolid> res = make_object<backgroundFillSolid>();
  res->color_ = env->GetIntField(p, res->color_fieldID);
  return std::move(res);
}

void backgroundFillSolid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, color_fieldID, color_);
}

void backgroundFillSolid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundFillSolid");
    s.store_field("color", color_);
    s.store_class_end();
  }
}

void backgroundFillSolid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundFillSolid").c_str());
    color_fieldID = jni::get_field_id(env, Class, "color", "I");
    return true;
  }();
  (void)is_inited;
}

jclass backgroundFillGradient::Class;
jfieldID backgroundFillGradient::top_color_fieldID;
jfieldID backgroundFillGradient::bottom_color_fieldID;
jfieldID backgroundFillGradient::rotation_angle_fieldID;

backgroundFillGradient::backgroundFillGradient()
  : top_color_()
  , bottom_color_()
  , rotation_angle_()
{}

backgroundFillGradient::backgroundFillGradient(int32 top_color_, int32 bottom_color_, int32 rotation_angle_)
  : top_color_(top_color_)
  , bottom_color_(bottom_color_)
  , rotation_angle_(rotation_angle_)
{}

const std::int32_t backgroundFillGradient::ID;

object_ptr<BackgroundFill> backgroundFillGradient::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundFillGradient> res = make_object<backgroundFillGradient>();
  res->top_color_ = env->GetIntField(p, res->top_color_fieldID);
  res->bottom_color_ = env->GetIntField(p, res->bottom_color_fieldID);
  res->rotation_angle_ = env->GetIntField(p, res->rotation_angle_fieldID);
  return std::move(res);
}

void backgroundFillGradient::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, top_color_fieldID, top_color_);
  env->SetIntField(s, bottom_color_fieldID, bottom_color_);
  env->SetIntField(s, rotation_angle_fieldID, rotation_angle_);
}

void backgroundFillGradient::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundFillGradient");
    s.store_field("topColor", top_color_);
    s.store_field("bottomColor", bottom_color_);
    s.store_field("rotationAngle", rotation_angle_);
    s.store_class_end();
  }
}

void backgroundFillGradient::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundFillGradient").c_str());
    top_color_fieldID = jni::get_field_id(env, Class, "topColor", "I");
    bottom_color_fieldID = jni::get_field_id(env, Class, "bottomColor", "I");
    rotation_angle_fieldID = jni::get_field_id(env, Class, "rotationAngle", "I");
    return true;
  }();
  (void)is_inited;
}

jclass backgroundFillFreeformGradient::Class;
jfieldID backgroundFillFreeformGradient::colors_fieldID;

backgroundFillFreeformGradient::backgroundFillFreeformGradient()
  : colors_()
{}

backgroundFillFreeformGradient::backgroundFillFreeformGradient(array<int32> &&colors_)
  : colors_(std::move(colors_))
{}

const std::int32_t backgroundFillFreeformGradient::ID;

object_ptr<BackgroundFill> backgroundFillFreeformGradient::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundFillFreeformGradient> res = make_object<backgroundFillFreeformGradient>();
  res->colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->colors_fieldID));
  return std::move(res);
}

void backgroundFillFreeformGradient::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, colors_); if (arr_tmp_) { env->SetObjectField(s, colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void backgroundFillFreeformGradient::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundFillFreeformGradient");
    { s.store_vector_begin("colors", colors_.size()); for (const auto &_value : colors_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void backgroundFillFreeformGradient::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundFillFreeformGradient").c_str());
    colors_fieldID = jni::get_field_id(env, Class, "colors", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass BackgroundType::Class;

object_ptr<BackgroundType> BackgroundType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case backgroundTypeWallpaper::ID:
      return backgroundTypeWallpaper::fetch(env, p);
    case backgroundTypePattern::ID:
      return backgroundTypePattern::fetch(env, p);
    case backgroundTypeFill::ID:
      return backgroundTypeFill::fetch(env, p);
    case backgroundTypeChatTheme::ID:
      return backgroundTypeChatTheme::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BackgroundType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass backgroundTypeWallpaper::Class;
jfieldID backgroundTypeWallpaper::is_blurred_fieldID;
jfieldID backgroundTypeWallpaper::is_moving_fieldID;

backgroundTypeWallpaper::backgroundTypeWallpaper()
  : is_blurred_()
  , is_moving_()
{}

backgroundTypeWallpaper::backgroundTypeWallpaper(bool is_blurred_, bool is_moving_)
  : is_blurred_(is_blurred_)
  , is_moving_(is_moving_)
{}

const std::int32_t backgroundTypeWallpaper::ID;

object_ptr<BackgroundType> backgroundTypeWallpaper::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundTypeWallpaper> res = make_object<backgroundTypeWallpaper>();
  res->is_blurred_ = (env->GetBooleanField(p, res->is_blurred_fieldID) != 0);
  res->is_moving_ = (env->GetBooleanField(p, res->is_moving_fieldID) != 0);
  return std::move(res);
}

void backgroundTypeWallpaper::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_blurred_fieldID, is_blurred_);
  env->SetBooleanField(s, is_moving_fieldID, is_moving_);
}

void backgroundTypeWallpaper::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundTypeWallpaper");
    s.store_field("isBlurred", is_blurred_);
    s.store_field("isMoving", is_moving_);
    s.store_class_end();
  }
}

void backgroundTypeWallpaper::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundTypeWallpaper").c_str());
    is_blurred_fieldID = jni::get_field_id(env, Class, "isBlurred", "Z");
    is_moving_fieldID = jni::get_field_id(env, Class, "isMoving", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass backgroundTypePattern::Class;
jfieldID backgroundTypePattern::fill_fieldID;
jfieldID backgroundTypePattern::intensity_fieldID;
jfieldID backgroundTypePattern::is_inverted_fieldID;
jfieldID backgroundTypePattern::is_moving_fieldID;

backgroundTypePattern::backgroundTypePattern()
  : fill_()
  , intensity_()
  , is_inverted_()
  , is_moving_()
{}

backgroundTypePattern::backgroundTypePattern(object_ptr<BackgroundFill> &&fill_, int32 intensity_, bool is_inverted_, bool is_moving_)
  : fill_(std::move(fill_))
  , intensity_(intensity_)
  , is_inverted_(is_inverted_)
  , is_moving_(is_moving_)
{}

const std::int32_t backgroundTypePattern::ID;

object_ptr<BackgroundType> backgroundTypePattern::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundTypePattern> res = make_object<backgroundTypePattern>();
  res->fill_ = jni::fetch_tl_object<BackgroundFill>(env, jni::fetch_object(env, p, res->fill_fieldID));
  res->intensity_ = env->GetIntField(p, res->intensity_fieldID);
  res->is_inverted_ = (env->GetBooleanField(p, res->is_inverted_fieldID) != 0);
  res->is_moving_ = (env->GetBooleanField(p, res->is_moving_fieldID) != 0);
  return std::move(res);
}

void backgroundTypePattern::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (fill_ != nullptr) { jobject next; fill_->store(env, next); if (next) { env->SetObjectField(s, fill_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, intensity_fieldID, intensity_);
  env->SetBooleanField(s, is_inverted_fieldID, is_inverted_);
  env->SetBooleanField(s, is_moving_fieldID, is_moving_);
}

void backgroundTypePattern::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundTypePattern");
    s.store_object_field("fill", static_cast<const BaseObject *>(fill_.get()));
    s.store_field("intensity", intensity_);
    s.store_field("isInverted", is_inverted_);
    s.store_field("isMoving", is_moving_);
    s.store_class_end();
  }
}

void backgroundTypePattern::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundTypePattern").c_str());
    fill_fieldID = jni::get_field_id(env, Class, "fill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundFill;").c_str());
    intensity_fieldID = jni::get_field_id(env, Class, "intensity", "I");
    is_inverted_fieldID = jni::get_field_id(env, Class, "isInverted", "Z");
    is_moving_fieldID = jni::get_field_id(env, Class, "isMoving", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass backgroundTypeFill::Class;
jfieldID backgroundTypeFill::fill_fieldID;

backgroundTypeFill::backgroundTypeFill()
  : fill_()
{}

backgroundTypeFill::backgroundTypeFill(object_ptr<BackgroundFill> &&fill_)
  : fill_(std::move(fill_))
{}

const std::int32_t backgroundTypeFill::ID;

object_ptr<BackgroundType> backgroundTypeFill::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundTypeFill> res = make_object<backgroundTypeFill>();
  res->fill_ = jni::fetch_tl_object<BackgroundFill>(env, jni::fetch_object(env, p, res->fill_fieldID));
  return std::move(res);
}

void backgroundTypeFill::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (fill_ != nullptr) { jobject next; fill_->store(env, next); if (next) { env->SetObjectField(s, fill_fieldID, next); env->DeleteLocalRef(next); } }
}

void backgroundTypeFill::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundTypeFill");
    s.store_object_field("fill", static_cast<const BaseObject *>(fill_.get()));
    s.store_class_end();
  }
}

void backgroundTypeFill::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundTypeFill").c_str());
    fill_fieldID = jni::get_field_id(env, Class, "fill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundFill;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass backgroundTypeChatTheme::Class;
jfieldID backgroundTypeChatTheme::theme_name_fieldID;

backgroundTypeChatTheme::backgroundTypeChatTheme()
  : theme_name_()
{}

backgroundTypeChatTheme::backgroundTypeChatTheme(string const &theme_name_)
  : theme_name_(theme_name_)
{}

const std::int32_t backgroundTypeChatTheme::ID;

object_ptr<BackgroundType> backgroundTypeChatTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgroundTypeChatTheme> res = make_object<backgroundTypeChatTheme>();
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return std::move(res);
}

void backgroundTypeChatTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void backgroundTypeChatTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BackgroundTypeChatTheme");
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

void backgroundTypeChatTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BackgroundTypeChatTheme").c_str());
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass backgrounds::Class;
jfieldID backgrounds::backgrounds_fieldID;

backgrounds::backgrounds()
  : backgrounds_()
{}

backgrounds::backgrounds(array<object_ptr<background>> &&backgrounds_)
  : backgrounds_(std::move(backgrounds_))
{}

const std::int32_t backgrounds::ID;

object_ptr<backgrounds> backgrounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<backgrounds> res = make_object<backgrounds>();
  res->backgrounds_ = jni::FetchVector<background>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->backgrounds_fieldID));
  return res;
}

void backgrounds::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, backgrounds_); if (arr_tmp_) { env->SetObjectField(s, backgrounds_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void backgrounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Backgrounds");
    { s.store_vector_begin("backgrounds", backgrounds_.size()); for (const auto &_value : backgrounds_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void backgrounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Backgrounds").c_str());
    backgrounds_fieldID = jni::get_field_id(env, Class, "backgrounds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Background;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass bankCardActionOpenUrl::Class;
jfieldID bankCardActionOpenUrl::text_fieldID;
jfieldID bankCardActionOpenUrl::url_fieldID;

bankCardActionOpenUrl::bankCardActionOpenUrl()
  : text_()
  , url_()
{}

bankCardActionOpenUrl::bankCardActionOpenUrl(string const &text_, string const &url_)
  : text_(text_)
  , url_(url_)
{}

const std::int32_t bankCardActionOpenUrl::ID;

object_ptr<bankCardActionOpenUrl> bankCardActionOpenUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<bankCardActionOpenUrl> res = make_object<bankCardActionOpenUrl>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void bankCardActionOpenUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void bankCardActionOpenUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BankCardActionOpenUrl");
    s.store_field("text", text_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void bankCardActionOpenUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BankCardActionOpenUrl").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass bankCardInfo::Class;
jfieldID bankCardInfo::title_fieldID;
jfieldID bankCardInfo::actions_fieldID;

bankCardInfo::bankCardInfo()
  : title_()
  , actions_()
{}

bankCardInfo::bankCardInfo(string const &title_, array<object_ptr<bankCardActionOpenUrl>> &&actions_)
  : title_(title_)
  , actions_(std::move(actions_))
{}

const std::int32_t bankCardInfo::ID;

object_ptr<bankCardInfo> bankCardInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<bankCardInfo> res = make_object<bankCardInfo>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->actions_ = jni::FetchVector<bankCardActionOpenUrl>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->actions_fieldID));
  return res;
}

void bankCardInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, actions_); if (arr_tmp_) { env->SetObjectField(s, actions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void bankCardInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BankCardInfo");
    s.store_field("title", title_);
    { s.store_vector_begin("actions", actions_.size()); for (const auto &_value : actions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void bankCardInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BankCardInfo").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    actions_fieldID = jni::get_field_id(env, Class, "actions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BankCardActionOpenUrl;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass basicGroup::Class;
jfieldID basicGroup::id_fieldID;
jfieldID basicGroup::member_count_fieldID;
jfieldID basicGroup::status_fieldID;
jfieldID basicGroup::is_active_fieldID;
jfieldID basicGroup::upgraded_to_supergroup_id_fieldID;

basicGroup::basicGroup()
  : id_()
  , member_count_()
  , status_()
  , is_active_()
  , upgraded_to_supergroup_id_()
{}

basicGroup::basicGroup(int53 id_, int32 member_count_, object_ptr<ChatMemberStatus> &&status_, bool is_active_, int53 upgraded_to_supergroup_id_)
  : id_(id_)
  , member_count_(member_count_)
  , status_(std::move(status_))
  , is_active_(is_active_)
  , upgraded_to_supergroup_id_(upgraded_to_supergroup_id_)
{}

const std::int32_t basicGroup::ID;

object_ptr<basicGroup> basicGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<basicGroup> res = make_object<basicGroup>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->member_count_ = env->GetIntField(p, res->member_count_fieldID);
  res->status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  res->upgraded_to_supergroup_id_ = env->GetLongField(p, res->upgraded_to_supergroup_id_fieldID);
  return res;
}

void basicGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, member_count_fieldID, member_count_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_active_fieldID, is_active_);
  env->SetLongField(s, upgraded_to_supergroup_id_fieldID, upgraded_to_supergroup_id_);
}

void basicGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BasicGroup");
    s.store_field("id", id_);
    s.store_field("memberCount", member_count_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_field("isActive", is_active_);
    s.store_field("upgradedToSupergroupId", upgraded_to_supergroup_id_);
    s.store_class_end();
  }
}

void basicGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BasicGroup").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", "I");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    upgraded_to_supergroup_id_fieldID = jni::get_field_id(env, Class, "upgradedToSupergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass basicGroupFullInfo::Class;
jfieldID basicGroupFullInfo::photo_fieldID;
jfieldID basicGroupFullInfo::description_fieldID;
jfieldID basicGroupFullInfo::creator_user_id_fieldID;
jfieldID basicGroupFullInfo::members_fieldID;
jfieldID basicGroupFullInfo::can_hide_members_fieldID;
jfieldID basicGroupFullInfo::can_toggle_aggressive_anti_spam_fieldID;
jfieldID basicGroupFullInfo::invite_link_fieldID;
jfieldID basicGroupFullInfo::bot_commands_fieldID;

basicGroupFullInfo::basicGroupFullInfo()
  : photo_()
  , description_()
  , creator_user_id_()
  , members_()
  , can_hide_members_()
  , can_toggle_aggressive_anti_spam_()
  , invite_link_()
  , bot_commands_()
{}

basicGroupFullInfo::basicGroupFullInfo(object_ptr<chatPhoto> &&photo_, string const &description_, int53 creator_user_id_, array<object_ptr<chatMember>> &&members_, bool can_hide_members_, bool can_toggle_aggressive_anti_spam_, object_ptr<chatInviteLink> &&invite_link_, array<object_ptr<botCommands>> &&bot_commands_)
  : photo_(std::move(photo_))
  , description_(description_)
  , creator_user_id_(creator_user_id_)
  , members_(std::move(members_))
  , can_hide_members_(can_hide_members_)
  , can_toggle_aggressive_anti_spam_(can_toggle_aggressive_anti_spam_)
  , invite_link_(std::move(invite_link_))
  , bot_commands_(std::move(bot_commands_))
{}

const std::int32_t basicGroupFullInfo::ID;

object_ptr<basicGroupFullInfo> basicGroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<basicGroupFullInfo> res = make_object<basicGroupFullInfo>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->creator_user_id_ = env->GetLongField(p, res->creator_user_id_fieldID);
  res->members_ = jni::FetchVector<chatMember>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->members_fieldID));
  res->can_hide_members_ = (env->GetBooleanField(p, res->can_hide_members_fieldID) != 0);
  res->can_toggle_aggressive_anti_spam_ = (env->GetBooleanField(p, res->can_toggle_aggressive_anti_spam_fieldID) != 0);
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  res->bot_commands_ = jni::FetchVector<botCommands>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->bot_commands_fieldID));
  return res;
}

void basicGroupFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, creator_user_id_fieldID, creator_user_id_);
  { auto arr_tmp_ = jni::store_vector(env, members_); if (arr_tmp_) { env->SetObjectField(s, members_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, can_hide_members_fieldID, can_hide_members_);
  env->SetBooleanField(s, can_toggle_aggressive_anti_spam_fieldID, can_toggle_aggressive_anti_spam_);
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, bot_commands_); if (arr_tmp_) { env->SetObjectField(s, bot_commands_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void basicGroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BasicGroupFullInfo");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("description", description_);
    s.store_field("creatorUserId", creator_user_id_);
    { s.store_vector_begin("members", members_.size()); for (const auto &_value : members_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("canHideMembers", can_hide_members_);
    s.store_field("canToggleAggressiveAntiSpam", can_toggle_aggressive_anti_spam_);
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    { s.store_vector_begin("botCommands", bot_commands_.size()); for (const auto &_value : bot_commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void basicGroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BasicGroupFullInfo").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    creator_user_id_fieldID = jni::get_field_id(env, Class, "creatorUserId", "J");
    members_fieldID = jni::get_field_id(env, Class, "members", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatMember;").c_str());
    can_hide_members_fieldID = jni::get_field_id(env, Class, "canHideMembers", "Z");
    can_toggle_aggressive_anti_spam_fieldID = jni::get_field_id(env, Class, "canToggleAggressiveAntiSpam", "Z");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    bot_commands_fieldID = jni::get_field_id(env, Class, "botCommands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotCommands;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass birthdate::Class;
jfieldID birthdate::day_fieldID;
jfieldID birthdate::month_fieldID;
jfieldID birthdate::year_fieldID;

birthdate::birthdate()
  : day_()
  , month_()
  , year_()
{}

birthdate::birthdate(int32 day_, int32 month_, int32 year_)
  : day_(day_)
  , month_(month_)
  , year_(year_)
{}

const std::int32_t birthdate::ID;

object_ptr<birthdate> birthdate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<birthdate> res = make_object<birthdate>();
  res->day_ = env->GetIntField(p, res->day_fieldID);
  res->month_ = env->GetIntField(p, res->month_fieldID);
  res->year_ = env->GetIntField(p, res->year_fieldID);
  return res;
}

void birthdate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, day_fieldID, day_);
  env->SetIntField(s, month_fieldID, month_);
  env->SetIntField(s, year_fieldID, year_);
}

void birthdate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Birthdate");
    s.store_field("day", day_);
    s.store_field("month", month_);
    s.store_field("year", year_);
    s.store_class_end();
  }
}

void birthdate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Birthdate").c_str());
    day_fieldID = jni::get_field_id(env, Class, "day", "I");
    month_fieldID = jni::get_field_id(env, Class, "month", "I");
    year_fieldID = jni::get_field_id(env, Class, "year", "I");
    return true;
  }();
  (void)is_inited;
}

jclass BlockList::Class;

object_ptr<BlockList> BlockList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case blockListMain::ID:
      return blockListMain::fetch(env, p);
    case blockListStories::ID:
      return blockListStories::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BlockList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BlockList").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass blockListMain::Class;

blockListMain::blockListMain() {
}

const std::int32_t blockListMain::ID;

object_ptr<BlockList> blockListMain::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<blockListMain>();
}

void blockListMain::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void blockListMain::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BlockListMain");
    s.store_class_end();
  }
}

void blockListMain::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BlockListMain").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass blockListStories::Class;

blockListStories::blockListStories() {
}

const std::int32_t blockListStories::ID;

object_ptr<BlockList> blockListStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<blockListStories>();
}

void blockListStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void blockListStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BlockListStories");
    s.store_class_end();
  }
}

void blockListStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BlockListStories").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommand::Class;
jfieldID botCommand::command_fieldID;
jfieldID botCommand::description_fieldID;

botCommand::botCommand()
  : command_()
  , description_()
{}

botCommand::botCommand(string const &command_, string const &description_)
  : command_(command_)
  , description_(description_)
{}

const std::int32_t botCommand::ID;

object_ptr<botCommand> botCommand::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botCommand> res = make_object<botCommand>();
  res->command_ = jni::fetch_string(env, p, res->command_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return res;
}

void botCommand::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, command_); if (nextString) { env->SetObjectField(s, command_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void botCommand::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommand");
    s.store_field("command", command_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

void botCommand::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommand").c_str());
    command_fieldID = jni::get_field_id(env, Class, "command", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass BotCommandScope::Class;

object_ptr<BotCommandScope> BotCommandScope::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case botCommandScopeDefault::ID:
      return botCommandScopeDefault::fetch(env, p);
    case botCommandScopeAllPrivateChats::ID:
      return botCommandScopeAllPrivateChats::fetch(env, p);
    case botCommandScopeAllGroupChats::ID:
      return botCommandScopeAllGroupChats::fetch(env, p);
    case botCommandScopeAllChatAdministrators::ID:
      return botCommandScopeAllChatAdministrators::fetch(env, p);
    case botCommandScopeChat::ID:
      return botCommandScopeChat::fetch(env, p);
    case botCommandScopeChatAdministrators::ID:
      return botCommandScopeChatAdministrators::fetch(env, p);
    case botCommandScopeChatMember::ID:
      return botCommandScopeChatMember::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BotCommandScope::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScope").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeDefault::Class;

botCommandScopeDefault::botCommandScopeDefault() {
}

const std::int32_t botCommandScopeDefault::ID;

object_ptr<BotCommandScope> botCommandScopeDefault::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botCommandScopeDefault>();
}

void botCommandScopeDefault::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeDefault");
    s.store_class_end();
  }
}

void botCommandScopeDefault::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeDefault").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeAllPrivateChats::Class;

botCommandScopeAllPrivateChats::botCommandScopeAllPrivateChats() {
}

const std::int32_t botCommandScopeAllPrivateChats::ID;

object_ptr<BotCommandScope> botCommandScopeAllPrivateChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botCommandScopeAllPrivateChats>();
}

void botCommandScopeAllPrivateChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeAllPrivateChats");
    s.store_class_end();
  }
}

void botCommandScopeAllPrivateChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeAllPrivateChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeAllGroupChats::Class;

botCommandScopeAllGroupChats::botCommandScopeAllGroupChats() {
}

const std::int32_t botCommandScopeAllGroupChats::ID;

object_ptr<BotCommandScope> botCommandScopeAllGroupChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botCommandScopeAllGroupChats>();
}

void botCommandScopeAllGroupChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeAllGroupChats");
    s.store_class_end();
  }
}

void botCommandScopeAllGroupChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeAllGroupChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeAllChatAdministrators::Class;

botCommandScopeAllChatAdministrators::botCommandScopeAllChatAdministrators() {
}

const std::int32_t botCommandScopeAllChatAdministrators::ID;

object_ptr<BotCommandScope> botCommandScopeAllChatAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botCommandScopeAllChatAdministrators>();
}

void botCommandScopeAllChatAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeAllChatAdministrators");
    s.store_class_end();
  }
}

void botCommandScopeAllChatAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeAllChatAdministrators").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeChat::Class;
jfieldID botCommandScopeChat::chat_id_fieldID;

botCommandScopeChat::botCommandScopeChat()
  : chat_id_()
{}

botCommandScopeChat::botCommandScopeChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t botCommandScopeChat::ID;

object_ptr<BotCommandScope> botCommandScopeChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botCommandScopeChat> res = make_object<botCommandScopeChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void botCommandScopeChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void botCommandScopeChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeChatAdministrators::Class;
jfieldID botCommandScopeChatAdministrators::chat_id_fieldID;

botCommandScopeChatAdministrators::botCommandScopeChatAdministrators()
  : chat_id_()
{}

botCommandScopeChatAdministrators::botCommandScopeChatAdministrators(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t botCommandScopeChatAdministrators::ID;

object_ptr<BotCommandScope> botCommandScopeChatAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botCommandScopeChatAdministrators> res = make_object<botCommandScopeChatAdministrators>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void botCommandScopeChatAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeChatAdministrators");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void botCommandScopeChatAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeChatAdministrators").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass botCommandScopeChatMember::Class;
jfieldID botCommandScopeChatMember::chat_id_fieldID;
jfieldID botCommandScopeChatMember::user_id_fieldID;

botCommandScopeChatMember::botCommandScopeChatMember()
  : chat_id_()
  , user_id_()
{}

botCommandScopeChatMember::botCommandScopeChatMember(int53 chat_id_, int53 user_id_)
  : chat_id_(chat_id_)
  , user_id_(user_id_)
{}

const std::int32_t botCommandScopeChatMember::ID;

object_ptr<BotCommandScope> botCommandScopeChatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botCommandScopeChatMember> res = make_object<botCommandScopeChatMember>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void botCommandScopeChatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommandScopeChatMember");
    s.store_field("chatId", chat_id_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void botCommandScopeChatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommandScopeChatMember").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass botCommands::Class;
jfieldID botCommands::bot_user_id_fieldID;
jfieldID botCommands::commands_fieldID;

botCommands::botCommands()
  : bot_user_id_()
  , commands_()
{}

botCommands::botCommands(int53 bot_user_id_, array<object_ptr<botCommand>> &&commands_)
  : bot_user_id_(bot_user_id_)
  , commands_(std::move(commands_))
{}

const std::int32_t botCommands::ID;

object_ptr<botCommands> botCommands::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botCommands> res = make_object<botCommands>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->commands_ = jni::FetchVector<botCommand>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->commands_fieldID));
  return res;
}

void botCommands::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  { auto arr_tmp_ = jni::store_vector(env, commands_); if (arr_tmp_) { env->SetObjectField(s, commands_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void botCommands::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotCommands");
    s.store_field("botUserId", bot_user_id_);
    { s.store_vector_begin("commands", commands_.size()); for (const auto &_value : commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void botCommands::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotCommands").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    commands_fieldID = jni::get_field_id(env, Class, "commands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotCommand;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botInfo::Class;
jfieldID botInfo::short_description_fieldID;
jfieldID botInfo::description_fieldID;
jfieldID botInfo::photo_fieldID;
jfieldID botInfo::animation_fieldID;
jfieldID botInfo::menu_button_fieldID;
jfieldID botInfo::commands_fieldID;
jfieldID botInfo::privacy_policy_url_fieldID;
jfieldID botInfo::default_group_administrator_rights_fieldID;
jfieldID botInfo::default_channel_administrator_rights_fieldID;
jfieldID botInfo::affiliate_program_fieldID;
jfieldID botInfo::web_app_background_light_color_fieldID;
jfieldID botInfo::web_app_background_dark_color_fieldID;
jfieldID botInfo::web_app_header_light_color_fieldID;
jfieldID botInfo::web_app_header_dark_color_fieldID;
jfieldID botInfo::verification_parameters_fieldID;
jfieldID botInfo::can_get_revenue_statistics_fieldID;
jfieldID botInfo::can_manage_emoji_status_fieldID;
jfieldID botInfo::has_media_previews_fieldID;
jfieldID botInfo::edit_commands_link_fieldID;
jfieldID botInfo::edit_description_link_fieldID;
jfieldID botInfo::edit_description_media_link_fieldID;
jfieldID botInfo::edit_settings_link_fieldID;

botInfo::botInfo()
  : short_description_()
  , description_()
  , photo_()
  , animation_()
  , menu_button_()
  , commands_()
  , privacy_policy_url_()
  , default_group_administrator_rights_()
  , default_channel_administrator_rights_()
  , affiliate_program_()
  , web_app_background_light_color_()
  , web_app_background_dark_color_()
  , web_app_header_light_color_()
  , web_app_header_dark_color_()
  , verification_parameters_()
  , can_get_revenue_statistics_()
  , can_manage_emoji_status_()
  , has_media_previews_()
  , edit_commands_link_()
  , edit_description_link_()
  , edit_description_media_link_()
  , edit_settings_link_()
{}

botInfo::botInfo(string const &short_description_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_, object_ptr<botMenuButton> &&menu_button_, array<object_ptr<botCommand>> &&commands_, string const &privacy_policy_url_, object_ptr<chatAdministratorRights> &&default_group_administrator_rights_, object_ptr<chatAdministratorRights> &&default_channel_administrator_rights_, object_ptr<affiliateProgramInfo> &&affiliate_program_, int32 web_app_background_light_color_, int32 web_app_background_dark_color_, int32 web_app_header_light_color_, int32 web_app_header_dark_color_, object_ptr<botVerificationParameters> &&verification_parameters_, bool can_get_revenue_statistics_, bool can_manage_emoji_status_, bool has_media_previews_, object_ptr<InternalLinkType> &&edit_commands_link_, object_ptr<InternalLinkType> &&edit_description_link_, object_ptr<InternalLinkType> &&edit_description_media_link_, object_ptr<InternalLinkType> &&edit_settings_link_)
  : short_description_(short_description_)
  , description_(description_)
  , photo_(std::move(photo_))
  , animation_(std::move(animation_))
  , menu_button_(std::move(menu_button_))
  , commands_(std::move(commands_))
  , privacy_policy_url_(privacy_policy_url_)
  , default_group_administrator_rights_(std::move(default_group_administrator_rights_))
  , default_channel_administrator_rights_(std::move(default_channel_administrator_rights_))
  , affiliate_program_(std::move(affiliate_program_))
  , web_app_background_light_color_(web_app_background_light_color_)
  , web_app_background_dark_color_(web_app_background_dark_color_)
  , web_app_header_light_color_(web_app_header_light_color_)
  , web_app_header_dark_color_(web_app_header_dark_color_)
  , verification_parameters_(std::move(verification_parameters_))
  , can_get_revenue_statistics_(can_get_revenue_statistics_)
  , can_manage_emoji_status_(can_manage_emoji_status_)
  , has_media_previews_(has_media_previews_)
  , edit_commands_link_(std::move(edit_commands_link_))
  , edit_description_link_(std::move(edit_description_link_))
  , edit_description_media_link_(std::move(edit_description_media_link_))
  , edit_settings_link_(std::move(edit_settings_link_))
{}

const std::int32_t botInfo::ID;

object_ptr<botInfo> botInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botInfo> res = make_object<botInfo>();
  res->short_description_ = jni::fetch_string(env, p, res->short_description_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->menu_button_ = jni::fetch_tl_object<botMenuButton>(env, jni::fetch_object(env, p, res->menu_button_fieldID));
  res->commands_ = jni::FetchVector<botCommand>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->commands_fieldID));
  res->privacy_policy_url_ = jni::fetch_string(env, p, res->privacy_policy_url_fieldID);
  res->default_group_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->default_group_administrator_rights_fieldID));
  res->default_channel_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->default_channel_administrator_rights_fieldID));
  res->affiliate_program_ = jni::fetch_tl_object<affiliateProgramInfo>(env, jni::fetch_object(env, p, res->affiliate_program_fieldID));
  res->web_app_background_light_color_ = env->GetIntField(p, res->web_app_background_light_color_fieldID);
  res->web_app_background_dark_color_ = env->GetIntField(p, res->web_app_background_dark_color_fieldID);
  res->web_app_header_light_color_ = env->GetIntField(p, res->web_app_header_light_color_fieldID);
  res->web_app_header_dark_color_ = env->GetIntField(p, res->web_app_header_dark_color_fieldID);
  res->verification_parameters_ = jni::fetch_tl_object<botVerificationParameters>(env, jni::fetch_object(env, p, res->verification_parameters_fieldID));
  res->can_get_revenue_statistics_ = (env->GetBooleanField(p, res->can_get_revenue_statistics_fieldID) != 0);
  res->can_manage_emoji_status_ = (env->GetBooleanField(p, res->can_manage_emoji_status_fieldID) != 0);
  res->has_media_previews_ = (env->GetBooleanField(p, res->has_media_previews_fieldID) != 0);
  res->edit_commands_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->edit_commands_link_fieldID));
  res->edit_description_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->edit_description_link_fieldID));
  res->edit_description_media_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->edit_description_media_link_fieldID));
  res->edit_settings_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->edit_settings_link_fieldID));
  return res;
}

void botInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, short_description_); if (nextString) { env->SetObjectField(s, short_description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (menu_button_ != nullptr) { jobject next; menu_button_->store(env, next); if (next) { env->SetObjectField(s, menu_button_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, commands_); if (arr_tmp_) { env->SetObjectField(s, commands_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, privacy_policy_url_); if (nextString) { env->SetObjectField(s, privacy_policy_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (default_group_administrator_rights_ != nullptr) { jobject next; default_group_administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, default_group_administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
  if (default_channel_administrator_rights_ != nullptr) { jobject next; default_channel_administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, default_channel_administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
  if (affiliate_program_ != nullptr) { jobject next; affiliate_program_->store(env, next); if (next) { env->SetObjectField(s, affiliate_program_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, web_app_background_light_color_fieldID, web_app_background_light_color_);
  env->SetIntField(s, web_app_background_dark_color_fieldID, web_app_background_dark_color_);
  env->SetIntField(s, web_app_header_light_color_fieldID, web_app_header_light_color_);
  env->SetIntField(s, web_app_header_dark_color_fieldID, web_app_header_dark_color_);
  if (verification_parameters_ != nullptr) { jobject next; verification_parameters_->store(env, next); if (next) { env->SetObjectField(s, verification_parameters_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_get_revenue_statistics_fieldID, can_get_revenue_statistics_);
  env->SetBooleanField(s, can_manage_emoji_status_fieldID, can_manage_emoji_status_);
  env->SetBooleanField(s, has_media_previews_fieldID, has_media_previews_);
  if (edit_commands_link_ != nullptr) { jobject next; edit_commands_link_->store(env, next); if (next) { env->SetObjectField(s, edit_commands_link_fieldID, next); env->DeleteLocalRef(next); } }
  if (edit_description_link_ != nullptr) { jobject next; edit_description_link_->store(env, next); if (next) { env->SetObjectField(s, edit_description_link_fieldID, next); env->DeleteLocalRef(next); } }
  if (edit_description_media_link_ != nullptr) { jobject next; edit_description_media_link_->store(env, next); if (next) { env->SetObjectField(s, edit_description_media_link_fieldID, next); env->DeleteLocalRef(next); } }
  if (edit_settings_link_ != nullptr) { jobject next; edit_settings_link_->store(env, next); if (next) { env->SetObjectField(s, edit_settings_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void botInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotInfo");
    s.store_field("shortDescription", short_description_);
    s.store_field("description", description_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_object_field("menuButton", static_cast<const BaseObject *>(menu_button_.get()));
    { s.store_vector_begin("commands", commands_.size()); for (const auto &_value : commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("privacyPolicyUrl", privacy_policy_url_);
    s.store_object_field("defaultGroupAdministratorRights", static_cast<const BaseObject *>(default_group_administrator_rights_.get()));
    s.store_object_field("defaultChannelAdministratorRights", static_cast<const BaseObject *>(default_channel_administrator_rights_.get()));
    s.store_object_field("affiliateProgram", static_cast<const BaseObject *>(affiliate_program_.get()));
    s.store_field("webAppBackgroundLightColor", web_app_background_light_color_);
    s.store_field("webAppBackgroundDarkColor", web_app_background_dark_color_);
    s.store_field("webAppHeaderLightColor", web_app_header_light_color_);
    s.store_field("webAppHeaderDarkColor", web_app_header_dark_color_);
    s.store_object_field("verificationParameters", static_cast<const BaseObject *>(verification_parameters_.get()));
    s.store_field("canGetRevenueStatistics", can_get_revenue_statistics_);
    s.store_field("canManageEmojiStatus", can_manage_emoji_status_);
    s.store_field("hasMediaPreviews", has_media_previews_);
    s.store_object_field("editCommandsLink", static_cast<const BaseObject *>(edit_commands_link_.get()));
    s.store_object_field("editDescriptionLink", static_cast<const BaseObject *>(edit_description_link_.get()));
    s.store_object_field("editDescriptionMediaLink", static_cast<const BaseObject *>(edit_description_media_link_.get()));
    s.store_object_field("editSettingsLink", static_cast<const BaseObject *>(edit_settings_link_.get()));
    s.store_class_end();
  }
}

void botInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotInfo").c_str());
    short_description_fieldID = jni::get_field_id(env, Class, "shortDescription", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    menu_button_fieldID = jni::get_field_id(env, Class, "menuButton", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotMenuButton;").c_str());
    commands_fieldID = jni::get_field_id(env, Class, "commands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotCommand;").c_str());
    privacy_policy_url_fieldID = jni::get_field_id(env, Class, "privacyPolicyUrl", "Ljava/lang/String;");
    default_group_administrator_rights_fieldID = jni::get_field_id(env, Class, "defaultGroupAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    default_channel_administrator_rights_fieldID = jni::get_field_id(env, Class, "defaultChannelAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    affiliate_program_fieldID = jni::get_field_id(env, Class, "affiliateProgram", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramInfo;").c_str());
    web_app_background_light_color_fieldID = jni::get_field_id(env, Class, "webAppBackgroundLightColor", "I");
    web_app_background_dark_color_fieldID = jni::get_field_id(env, Class, "webAppBackgroundDarkColor", "I");
    web_app_header_light_color_fieldID = jni::get_field_id(env, Class, "webAppHeaderLightColor", "I");
    web_app_header_dark_color_fieldID = jni::get_field_id(env, Class, "webAppHeaderDarkColor", "I");
    verification_parameters_fieldID = jni::get_field_id(env, Class, "verificationParameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotVerificationParameters;").c_str());
    can_get_revenue_statistics_fieldID = jni::get_field_id(env, Class, "canGetRevenueStatistics", "Z");
    can_manage_emoji_status_fieldID = jni::get_field_id(env, Class, "canManageEmojiStatus", "Z");
    has_media_previews_fieldID = jni::get_field_id(env, Class, "hasMediaPreviews", "Z");
    edit_commands_link_fieldID = jni::get_field_id(env, Class, "editCommandsLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    edit_description_link_fieldID = jni::get_field_id(env, Class, "editDescriptionLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    edit_description_media_link_fieldID = jni::get_field_id(env, Class, "editDescriptionMediaLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    edit_settings_link_fieldID = jni::get_field_id(env, Class, "editSettingsLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botMediaPreview::Class;
jfieldID botMediaPreview::date_fieldID;
jfieldID botMediaPreview::content_fieldID;

botMediaPreview::botMediaPreview()
  : date_()
  , content_()
{}

botMediaPreview::botMediaPreview(int32 date_, object_ptr<StoryContent> &&content_)
  : date_(date_)
  , content_(std::move(content_))
{}

const std::int32_t botMediaPreview::ID;

object_ptr<botMediaPreview> botMediaPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botMediaPreview> res = make_object<botMediaPreview>();
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->content_ = jni::fetch_tl_object<StoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return res;
}

void botMediaPreview::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, date_fieldID, date_);
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
}

void botMediaPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotMediaPreview");
    s.store_field("date", date_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

void botMediaPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotMediaPreview").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botMediaPreviewInfo::Class;
jfieldID botMediaPreviewInfo::previews_fieldID;
jfieldID botMediaPreviewInfo::language_codes_fieldID;

botMediaPreviewInfo::botMediaPreviewInfo()
  : previews_()
  , language_codes_()
{}

botMediaPreviewInfo::botMediaPreviewInfo(array<object_ptr<botMediaPreview>> &&previews_, array<string> &&language_codes_)
  : previews_(std::move(previews_))
  , language_codes_(std::move(language_codes_))
{}

const std::int32_t botMediaPreviewInfo::ID;

object_ptr<botMediaPreviewInfo> botMediaPreviewInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botMediaPreviewInfo> res = make_object<botMediaPreviewInfo>();
  res->previews_ = jni::FetchVector<botMediaPreview>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->previews_fieldID));
  res->language_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->language_codes_fieldID));
  return res;
}

void botMediaPreviewInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, previews_); if (arr_tmp_) { env->SetObjectField(s, previews_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, language_codes_); if (arr_tmp_) { env->SetObjectField(s, language_codes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void botMediaPreviewInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotMediaPreviewInfo");
    { s.store_vector_begin("previews", previews_.size()); for (const auto &_value : previews_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("languageCodes", language_codes_.size()); for (const auto &_value : language_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void botMediaPreviewInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotMediaPreviewInfo").c_str());
    previews_fieldID = jni::get_field_id(env, Class, "previews", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotMediaPreview;").c_str());
    language_codes_fieldID = jni::get_field_id(env, Class, "languageCodes", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass botMediaPreviews::Class;
jfieldID botMediaPreviews::previews_fieldID;

botMediaPreviews::botMediaPreviews()
  : previews_()
{}

botMediaPreviews::botMediaPreviews(array<object_ptr<botMediaPreview>> &&previews_)
  : previews_(std::move(previews_))
{}

const std::int32_t botMediaPreviews::ID;

object_ptr<botMediaPreviews> botMediaPreviews::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botMediaPreviews> res = make_object<botMediaPreviews>();
  res->previews_ = jni::FetchVector<botMediaPreview>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->previews_fieldID));
  return res;
}

void botMediaPreviews::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, previews_); if (arr_tmp_) { env->SetObjectField(s, previews_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void botMediaPreviews::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotMediaPreviews");
    { s.store_vector_begin("previews", previews_.size()); for (const auto &_value : previews_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void botMediaPreviews::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotMediaPreviews").c_str());
    previews_fieldID = jni::get_field_id(env, Class, "previews", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotMediaPreview;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botMenuButton::Class;
jfieldID botMenuButton::text_fieldID;
jfieldID botMenuButton::url_fieldID;

botMenuButton::botMenuButton()
  : text_()
  , url_()
{}

botMenuButton::botMenuButton(string const &text_, string const &url_)
  : text_(text_)
  , url_(url_)
{}

const std::int32_t botMenuButton::ID;

object_ptr<botMenuButton> botMenuButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botMenuButton> res = make_object<botMenuButton>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void botMenuButton::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void botMenuButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotMenuButton");
    s.store_field("text", text_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void botMenuButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotMenuButton").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass botVerification::Class;
jfieldID botVerification::bot_user_id_fieldID;
jfieldID botVerification::icon_custom_emoji_id_fieldID;
jfieldID botVerification::custom_description_fieldID;

botVerification::botVerification()
  : bot_user_id_()
  , icon_custom_emoji_id_()
  , custom_description_()
{}

botVerification::botVerification(int53 bot_user_id_, int64 icon_custom_emoji_id_, object_ptr<formattedText> &&custom_description_)
  : bot_user_id_(bot_user_id_)
  , icon_custom_emoji_id_(icon_custom_emoji_id_)
  , custom_description_(std::move(custom_description_))
{}

const std::int32_t botVerification::ID;

object_ptr<botVerification> botVerification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botVerification> res = make_object<botVerification>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->icon_custom_emoji_id_ = env->GetLongField(p, res->icon_custom_emoji_id_fieldID);
  res->custom_description_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->custom_description_fieldID));
  return res;
}

void botVerification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  env->SetLongField(s, icon_custom_emoji_id_fieldID, icon_custom_emoji_id_);
  if (custom_description_ != nullptr) { jobject next; custom_description_->store(env, next); if (next) { env->SetObjectField(s, custom_description_fieldID, next); env->DeleteLocalRef(next); } }
}

void botVerification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotVerification");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("iconCustomEmojiId", icon_custom_emoji_id_);
    s.store_object_field("customDescription", static_cast<const BaseObject *>(custom_description_.get()));
    s.store_class_end();
  }
}

void botVerification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotVerification").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "iconCustomEmojiId", "J");
    custom_description_fieldID = jni::get_field_id(env, Class, "customDescription", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botVerificationParameters::Class;
jfieldID botVerificationParameters::icon_custom_emoji_id_fieldID;
jfieldID botVerificationParameters::organization_name_fieldID;
jfieldID botVerificationParameters::default_custom_description_fieldID;
jfieldID botVerificationParameters::can_set_custom_description_fieldID;

botVerificationParameters::botVerificationParameters()
  : icon_custom_emoji_id_()
  , organization_name_()
  , default_custom_description_()
  , can_set_custom_description_()
{}

botVerificationParameters::botVerificationParameters(int64 icon_custom_emoji_id_, string const &organization_name_, object_ptr<formattedText> &&default_custom_description_, bool can_set_custom_description_)
  : icon_custom_emoji_id_(icon_custom_emoji_id_)
  , organization_name_(organization_name_)
  , default_custom_description_(std::move(default_custom_description_))
  , can_set_custom_description_(can_set_custom_description_)
{}

const std::int32_t botVerificationParameters::ID;

object_ptr<botVerificationParameters> botVerificationParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botVerificationParameters> res = make_object<botVerificationParameters>();
  res->icon_custom_emoji_id_ = env->GetLongField(p, res->icon_custom_emoji_id_fieldID);
  res->organization_name_ = jni::fetch_string(env, p, res->organization_name_fieldID);
  res->default_custom_description_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->default_custom_description_fieldID));
  res->can_set_custom_description_ = (env->GetBooleanField(p, res->can_set_custom_description_fieldID) != 0);
  return res;
}

void botVerificationParameters::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, icon_custom_emoji_id_fieldID, icon_custom_emoji_id_);
  { jstring nextString = jni::to_jstring(env, organization_name_); if (nextString) { env->SetObjectField(s, organization_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (default_custom_description_ != nullptr) { jobject next; default_custom_description_->store(env, next); if (next) { env->SetObjectField(s, default_custom_description_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_set_custom_description_fieldID, can_set_custom_description_);
}

void botVerificationParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotVerificationParameters");
    s.store_field("iconCustomEmojiId", icon_custom_emoji_id_);
    s.store_field("organizationName", organization_name_);
    s.store_object_field("defaultCustomDescription", static_cast<const BaseObject *>(default_custom_description_.get()));
    s.store_field("canSetCustomDescription", can_set_custom_description_);
    s.store_class_end();
  }
}

void botVerificationParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotVerificationParameters").c_str());
    icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "iconCustomEmojiId", "J");
    organization_name_fieldID = jni::get_field_id(env, Class, "organizationName", "Ljava/lang/String;");
    default_custom_description_fieldID = jni::get_field_id(env, Class, "defaultCustomDescription", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    can_set_custom_description_fieldID = jni::get_field_id(env, Class, "canSetCustomDescription", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass BotWriteAccessAllowReason::Class;

object_ptr<BotWriteAccessAllowReason> BotWriteAccessAllowReason::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case botWriteAccessAllowReasonConnectedWebsite::ID:
      return botWriteAccessAllowReasonConnectedWebsite::fetch(env, p);
    case botWriteAccessAllowReasonAddedToAttachmentMenu::ID:
      return botWriteAccessAllowReasonAddedToAttachmentMenu::fetch(env, p);
    case botWriteAccessAllowReasonLaunchedWebApp::ID:
      return botWriteAccessAllowReasonLaunchedWebApp::fetch(env, p);
    case botWriteAccessAllowReasonAcceptedRequest::ID:
      return botWriteAccessAllowReasonAcceptedRequest::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BotWriteAccessAllowReason::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReason").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botWriteAccessAllowReasonConnectedWebsite::Class;
jfieldID botWriteAccessAllowReasonConnectedWebsite::domain_name_fieldID;

botWriteAccessAllowReasonConnectedWebsite::botWriteAccessAllowReasonConnectedWebsite()
  : domain_name_()
{}

botWriteAccessAllowReasonConnectedWebsite::botWriteAccessAllowReasonConnectedWebsite(string const &domain_name_)
  : domain_name_(domain_name_)
{}

const std::int32_t botWriteAccessAllowReasonConnectedWebsite::ID;

object_ptr<BotWriteAccessAllowReason> botWriteAccessAllowReasonConnectedWebsite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botWriteAccessAllowReasonConnectedWebsite> res = make_object<botWriteAccessAllowReasonConnectedWebsite>();
  res->domain_name_ = jni::fetch_string(env, p, res->domain_name_fieldID);
  return std::move(res);
}

void botWriteAccessAllowReasonConnectedWebsite::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, domain_name_); if (nextString) { env->SetObjectField(s, domain_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void botWriteAccessAllowReasonConnectedWebsite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotWriteAccessAllowReasonConnectedWebsite");
    s.store_field("domainName", domain_name_);
    s.store_class_end();
  }
}

void botWriteAccessAllowReasonConnectedWebsite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReasonConnectedWebsite").c_str());
    domain_name_fieldID = jni::get_field_id(env, Class, "domainName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass botWriteAccessAllowReasonAddedToAttachmentMenu::Class;

botWriteAccessAllowReasonAddedToAttachmentMenu::botWriteAccessAllowReasonAddedToAttachmentMenu() {
}

const std::int32_t botWriteAccessAllowReasonAddedToAttachmentMenu::ID;

object_ptr<BotWriteAccessAllowReason> botWriteAccessAllowReasonAddedToAttachmentMenu::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botWriteAccessAllowReasonAddedToAttachmentMenu>();
}

void botWriteAccessAllowReasonAddedToAttachmentMenu::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void botWriteAccessAllowReasonAddedToAttachmentMenu::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotWriteAccessAllowReasonAddedToAttachmentMenu");
    s.store_class_end();
  }
}

void botWriteAccessAllowReasonAddedToAttachmentMenu::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReasonAddedToAttachmentMenu").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botWriteAccessAllowReasonLaunchedWebApp::Class;
jfieldID botWriteAccessAllowReasonLaunchedWebApp::web_app_fieldID;

botWriteAccessAllowReasonLaunchedWebApp::botWriteAccessAllowReasonLaunchedWebApp()
  : web_app_()
{}

botWriteAccessAllowReasonLaunchedWebApp::botWriteAccessAllowReasonLaunchedWebApp(object_ptr<webApp> &&web_app_)
  : web_app_(std::move(web_app_))
{}

const std::int32_t botWriteAccessAllowReasonLaunchedWebApp::ID;

object_ptr<BotWriteAccessAllowReason> botWriteAccessAllowReasonLaunchedWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<botWriteAccessAllowReasonLaunchedWebApp> res = make_object<botWriteAccessAllowReasonLaunchedWebApp>();
  res->web_app_ = jni::fetch_tl_object<webApp>(env, jni::fetch_object(env, p, res->web_app_fieldID));
  return std::move(res);
}

void botWriteAccessAllowReasonLaunchedWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (web_app_ != nullptr) { jobject next; web_app_->store(env, next); if (next) { env->SetObjectField(s, web_app_fieldID, next); env->DeleteLocalRef(next); } }
}

void botWriteAccessAllowReasonLaunchedWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotWriteAccessAllowReasonLaunchedWebApp");
    s.store_object_field("webApp", static_cast<const BaseObject *>(web_app_.get()));
    s.store_class_end();
  }
}

void botWriteAccessAllowReasonLaunchedWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReasonLaunchedWebApp").c_str());
    web_app_fieldID = jni::get_field_id(env, Class, "webApp", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebApp;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass botWriteAccessAllowReasonAcceptedRequest::Class;

botWriteAccessAllowReasonAcceptedRequest::botWriteAccessAllowReasonAcceptedRequest() {
}

const std::int32_t botWriteAccessAllowReasonAcceptedRequest::ID;

object_ptr<BotWriteAccessAllowReason> botWriteAccessAllowReasonAcceptedRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<botWriteAccessAllowReasonAcceptedRequest>();
}

void botWriteAccessAllowReasonAcceptedRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void botWriteAccessAllowReasonAcceptedRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BotWriteAccessAllowReasonAcceptedRequest");
    s.store_class_end();
  }
}

void botWriteAccessAllowReasonAcceptedRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReasonAcceptedRequest").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass BusinessAwayMessageSchedule::Class;

object_ptr<BusinessAwayMessageSchedule> BusinessAwayMessageSchedule::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case businessAwayMessageScheduleAlways::ID:
      return businessAwayMessageScheduleAlways::fetch(env, p);
    case businessAwayMessageScheduleOutsideOfOpeningHours::ID:
      return businessAwayMessageScheduleOutsideOfOpeningHours::fetch(env, p);
    case businessAwayMessageScheduleCustom::ID:
      return businessAwayMessageScheduleCustom::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BusinessAwayMessageSchedule::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessAwayMessageSchedule").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessAwayMessageScheduleAlways::Class;

businessAwayMessageScheduleAlways::businessAwayMessageScheduleAlways() {
}

const std::int32_t businessAwayMessageScheduleAlways::ID;

object_ptr<BusinessAwayMessageSchedule> businessAwayMessageScheduleAlways::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessAwayMessageScheduleAlways>();
}

void businessAwayMessageScheduleAlways::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessAwayMessageScheduleAlways::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessAwayMessageScheduleAlways");
    s.store_class_end();
  }
}

void businessAwayMessageScheduleAlways::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessAwayMessageScheduleAlways").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessAwayMessageScheduleOutsideOfOpeningHours::Class;

businessAwayMessageScheduleOutsideOfOpeningHours::businessAwayMessageScheduleOutsideOfOpeningHours() {
}

const std::int32_t businessAwayMessageScheduleOutsideOfOpeningHours::ID;

object_ptr<BusinessAwayMessageSchedule> businessAwayMessageScheduleOutsideOfOpeningHours::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessAwayMessageScheduleOutsideOfOpeningHours>();
}

void businessAwayMessageScheduleOutsideOfOpeningHours::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessAwayMessageScheduleOutsideOfOpeningHours::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessAwayMessageScheduleOutsideOfOpeningHours");
    s.store_class_end();
  }
}

void businessAwayMessageScheduleOutsideOfOpeningHours::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessAwayMessageScheduleOutsideOfOpeningHours").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessAwayMessageScheduleCustom::Class;
jfieldID businessAwayMessageScheduleCustom::start_date_fieldID;
jfieldID businessAwayMessageScheduleCustom::end_date_fieldID;

businessAwayMessageScheduleCustom::businessAwayMessageScheduleCustom()
  : start_date_()
  , end_date_()
{}

businessAwayMessageScheduleCustom::businessAwayMessageScheduleCustom(int32 start_date_, int32 end_date_)
  : start_date_(start_date_)
  , end_date_(end_date_)
{}

const std::int32_t businessAwayMessageScheduleCustom::ID;

object_ptr<BusinessAwayMessageSchedule> businessAwayMessageScheduleCustom::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessAwayMessageScheduleCustom> res = make_object<businessAwayMessageScheduleCustom>();
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->end_date_ = env->GetIntField(p, res->end_date_fieldID);
  return std::move(res);
}

void businessAwayMessageScheduleCustom::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, start_date_fieldID, start_date_);
  env->SetIntField(s, end_date_fieldID, end_date_);
}

void businessAwayMessageScheduleCustom::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessAwayMessageScheduleCustom");
    s.store_field("startDate", start_date_);
    s.store_field("endDate", end_date_);
    s.store_class_end();
  }
}

void businessAwayMessageScheduleCustom::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessAwayMessageScheduleCustom").c_str());
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    end_date_fieldID = jni::get_field_id(env, Class, "endDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass businessAwayMessageSettings::Class;
jfieldID businessAwayMessageSettings::shortcut_id_fieldID;
jfieldID businessAwayMessageSettings::recipients_fieldID;
jfieldID businessAwayMessageSettings::schedule_fieldID;
jfieldID businessAwayMessageSettings::offline_only_fieldID;

businessAwayMessageSettings::businessAwayMessageSettings()
  : shortcut_id_()
  , recipients_()
  , schedule_()
  , offline_only_()
{}

businessAwayMessageSettings::businessAwayMessageSettings(int32 shortcut_id_, object_ptr<businessRecipients> &&recipients_, object_ptr<BusinessAwayMessageSchedule> &&schedule_, bool offline_only_)
  : shortcut_id_(shortcut_id_)
  , recipients_(std::move(recipients_))
  , schedule_(std::move(schedule_))
  , offline_only_(offline_only_)
{}

const std::int32_t businessAwayMessageSettings::ID;

object_ptr<businessAwayMessageSettings> businessAwayMessageSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessAwayMessageSettings> res = make_object<businessAwayMessageSettings>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->recipients_ = jni::fetch_tl_object<businessRecipients>(env, jni::fetch_object(env, p, res->recipients_fieldID));
  res->schedule_ = jni::fetch_tl_object<BusinessAwayMessageSchedule>(env, jni::fetch_object(env, p, res->schedule_fieldID));
  res->offline_only_ = (env->GetBooleanField(p, res->offline_only_fieldID) != 0);
  return res;
}

void businessAwayMessageSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, shortcut_id_fieldID, shortcut_id_);
  if (recipients_ != nullptr) { jobject next; recipients_->store(env, next); if (next) { env->SetObjectField(s, recipients_fieldID, next); env->DeleteLocalRef(next); } }
  if (schedule_ != nullptr) { jobject next; schedule_->store(env, next); if (next) { env->SetObjectField(s, schedule_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, offline_only_fieldID, offline_only_);
}

void businessAwayMessageSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessAwayMessageSettings");
    s.store_field("shortcutId", shortcut_id_);
    s.store_object_field("recipients", static_cast<const BaseObject *>(recipients_.get()));
    s.store_object_field("schedule", static_cast<const BaseObject *>(schedule_.get()));
    s.store_field("offlineOnly", offline_only_);
    s.store_class_end();
  }
}

void businessAwayMessageSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessAwayMessageSettings").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    recipients_fieldID = jni::get_field_id(env, Class, "recipients", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessRecipients;").c_str());
    schedule_fieldID = jni::get_field_id(env, Class, "schedule", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessAwayMessageSchedule;").c_str());
    offline_only_fieldID = jni::get_field_id(env, Class, "offlineOnly", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass businessBotManageBar::Class;
jfieldID businessBotManageBar::bot_user_id_fieldID;
jfieldID businessBotManageBar::manage_url_fieldID;
jfieldID businessBotManageBar::is_bot_paused_fieldID;
jfieldID businessBotManageBar::can_bot_reply_fieldID;

businessBotManageBar::businessBotManageBar()
  : bot_user_id_()
  , manage_url_()
  , is_bot_paused_()
  , can_bot_reply_()
{}

businessBotManageBar::businessBotManageBar(int53 bot_user_id_, string const &manage_url_, bool is_bot_paused_, bool can_bot_reply_)
  : bot_user_id_(bot_user_id_)
  , manage_url_(manage_url_)
  , is_bot_paused_(is_bot_paused_)
  , can_bot_reply_(can_bot_reply_)
{}

const std::int32_t businessBotManageBar::ID;

object_ptr<businessBotManageBar> businessBotManageBar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessBotManageBar> res = make_object<businessBotManageBar>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->manage_url_ = jni::fetch_string(env, p, res->manage_url_fieldID);
  res->is_bot_paused_ = (env->GetBooleanField(p, res->is_bot_paused_fieldID) != 0);
  res->can_bot_reply_ = (env->GetBooleanField(p, res->can_bot_reply_fieldID) != 0);
  return res;
}

void businessBotManageBar::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  { jstring nextString = jni::to_jstring(env, manage_url_); if (nextString) { env->SetObjectField(s, manage_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_bot_paused_fieldID, is_bot_paused_);
  env->SetBooleanField(s, can_bot_reply_fieldID, can_bot_reply_);
}

void businessBotManageBar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessBotManageBar");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("manageUrl", manage_url_);
    s.store_field("isBotPaused", is_bot_paused_);
    s.store_field("canBotReply", can_bot_reply_);
    s.store_class_end();
  }
}

void businessBotManageBar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessBotManageBar").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    manage_url_fieldID = jni::get_field_id(env, Class, "manageUrl", "Ljava/lang/String;");
    is_bot_paused_fieldID = jni::get_field_id(env, Class, "isBotPaused", "Z");
    can_bot_reply_fieldID = jni::get_field_id(env, Class, "canBotReply", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass businessBotRights::Class;
jfieldID businessBotRights::can_reply_fieldID;
jfieldID businessBotRights::can_read_messages_fieldID;
jfieldID businessBotRights::can_delete_sent_messages_fieldID;
jfieldID businessBotRights::can_delete_all_messages_fieldID;
jfieldID businessBotRights::can_edit_name_fieldID;
jfieldID businessBotRights::can_edit_bio_fieldID;
jfieldID businessBotRights::can_edit_profile_photo_fieldID;
jfieldID businessBotRights::can_edit_username_fieldID;
jfieldID businessBotRights::can_view_gifts_and_stars_fieldID;
jfieldID businessBotRights::can_sell_gifts_fieldID;
jfieldID businessBotRights::can_change_gift_settings_fieldID;
jfieldID businessBotRights::can_transfer_and_upgrade_gifts_fieldID;
jfieldID businessBotRights::can_transfer_stars_fieldID;
jfieldID businessBotRights::can_manage_stories_fieldID;

businessBotRights::businessBotRights()
  : can_reply_()
  , can_read_messages_()
  , can_delete_sent_messages_()
  , can_delete_all_messages_()
  , can_edit_name_()
  , can_edit_bio_()
  , can_edit_profile_photo_()
  , can_edit_username_()
  , can_view_gifts_and_stars_()
  , can_sell_gifts_()
  , can_change_gift_settings_()
  , can_transfer_and_upgrade_gifts_()
  , can_transfer_stars_()
  , can_manage_stories_()
{}

businessBotRights::businessBotRights(bool can_reply_, bool can_read_messages_, bool can_delete_sent_messages_, bool can_delete_all_messages_, bool can_edit_name_, bool can_edit_bio_, bool can_edit_profile_photo_, bool can_edit_username_, bool can_view_gifts_and_stars_, bool can_sell_gifts_, bool can_change_gift_settings_, bool can_transfer_and_upgrade_gifts_, bool can_transfer_stars_, bool can_manage_stories_)
  : can_reply_(can_reply_)
  , can_read_messages_(can_read_messages_)
  , can_delete_sent_messages_(can_delete_sent_messages_)
  , can_delete_all_messages_(can_delete_all_messages_)
  , can_edit_name_(can_edit_name_)
  , can_edit_bio_(can_edit_bio_)
  , can_edit_profile_photo_(can_edit_profile_photo_)
  , can_edit_username_(can_edit_username_)
  , can_view_gifts_and_stars_(can_view_gifts_and_stars_)
  , can_sell_gifts_(can_sell_gifts_)
  , can_change_gift_settings_(can_change_gift_settings_)
  , can_transfer_and_upgrade_gifts_(can_transfer_and_upgrade_gifts_)
  , can_transfer_stars_(can_transfer_stars_)
  , can_manage_stories_(can_manage_stories_)
{}

const std::int32_t businessBotRights::ID;

object_ptr<businessBotRights> businessBotRights::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessBotRights> res = make_object<businessBotRights>();
  res->can_reply_ = (env->GetBooleanField(p, res->can_reply_fieldID) != 0);
  res->can_read_messages_ = (env->GetBooleanField(p, res->can_read_messages_fieldID) != 0);
  res->can_delete_sent_messages_ = (env->GetBooleanField(p, res->can_delete_sent_messages_fieldID) != 0);
  res->can_delete_all_messages_ = (env->GetBooleanField(p, res->can_delete_all_messages_fieldID) != 0);
  res->can_edit_name_ = (env->GetBooleanField(p, res->can_edit_name_fieldID) != 0);
  res->can_edit_bio_ = (env->GetBooleanField(p, res->can_edit_bio_fieldID) != 0);
  res->can_edit_profile_photo_ = (env->GetBooleanField(p, res->can_edit_profile_photo_fieldID) != 0);
  res->can_edit_username_ = (env->GetBooleanField(p, res->can_edit_username_fieldID) != 0);
  res->can_view_gifts_and_stars_ = (env->GetBooleanField(p, res->can_view_gifts_and_stars_fieldID) != 0);
  res->can_sell_gifts_ = (env->GetBooleanField(p, res->can_sell_gifts_fieldID) != 0);
  res->can_change_gift_settings_ = (env->GetBooleanField(p, res->can_change_gift_settings_fieldID) != 0);
  res->can_transfer_and_upgrade_gifts_ = (env->GetBooleanField(p, res->can_transfer_and_upgrade_gifts_fieldID) != 0);
  res->can_transfer_stars_ = (env->GetBooleanField(p, res->can_transfer_stars_fieldID) != 0);
  res->can_manage_stories_ = (env->GetBooleanField(p, res->can_manage_stories_fieldID) != 0);
  return res;
}

void businessBotRights::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_reply_fieldID, can_reply_);
  env->SetBooleanField(s, can_read_messages_fieldID, can_read_messages_);
  env->SetBooleanField(s, can_delete_sent_messages_fieldID, can_delete_sent_messages_);
  env->SetBooleanField(s, can_delete_all_messages_fieldID, can_delete_all_messages_);
  env->SetBooleanField(s, can_edit_name_fieldID, can_edit_name_);
  env->SetBooleanField(s, can_edit_bio_fieldID, can_edit_bio_);
  env->SetBooleanField(s, can_edit_profile_photo_fieldID, can_edit_profile_photo_);
  env->SetBooleanField(s, can_edit_username_fieldID, can_edit_username_);
  env->SetBooleanField(s, can_view_gifts_and_stars_fieldID, can_view_gifts_and_stars_);
  env->SetBooleanField(s, can_sell_gifts_fieldID, can_sell_gifts_);
  env->SetBooleanField(s, can_change_gift_settings_fieldID, can_change_gift_settings_);
  env->SetBooleanField(s, can_transfer_and_upgrade_gifts_fieldID, can_transfer_and_upgrade_gifts_);
  env->SetBooleanField(s, can_transfer_stars_fieldID, can_transfer_stars_);
  env->SetBooleanField(s, can_manage_stories_fieldID, can_manage_stories_);
}

void businessBotRights::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessBotRights");
    s.store_field("canReply", can_reply_);
    s.store_field("canReadMessages", can_read_messages_);
    s.store_field("canDeleteSentMessages", can_delete_sent_messages_);
    s.store_field("canDeleteAllMessages", can_delete_all_messages_);
    s.store_field("canEditName", can_edit_name_);
    s.store_field("canEditBio", can_edit_bio_);
    s.store_field("canEditProfilePhoto", can_edit_profile_photo_);
    s.store_field("canEditUsername", can_edit_username_);
    s.store_field("canViewGiftsAndStars", can_view_gifts_and_stars_);
    s.store_field("canSellGifts", can_sell_gifts_);
    s.store_field("canChangeGiftSettings", can_change_gift_settings_);
    s.store_field("canTransferAndUpgradeGifts", can_transfer_and_upgrade_gifts_);
    s.store_field("canTransferStars", can_transfer_stars_);
    s.store_field("canManageStories", can_manage_stories_);
    s.store_class_end();
  }
}

void businessBotRights::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessBotRights").c_str());
    can_reply_fieldID = jni::get_field_id(env, Class, "canReply", "Z");
    can_read_messages_fieldID = jni::get_field_id(env, Class, "canReadMessages", "Z");
    can_delete_sent_messages_fieldID = jni::get_field_id(env, Class, "canDeleteSentMessages", "Z");
    can_delete_all_messages_fieldID = jni::get_field_id(env, Class, "canDeleteAllMessages", "Z");
    can_edit_name_fieldID = jni::get_field_id(env, Class, "canEditName", "Z");
    can_edit_bio_fieldID = jni::get_field_id(env, Class, "canEditBio", "Z");
    can_edit_profile_photo_fieldID = jni::get_field_id(env, Class, "canEditProfilePhoto", "Z");
    can_edit_username_fieldID = jni::get_field_id(env, Class, "canEditUsername", "Z");
    can_view_gifts_and_stars_fieldID = jni::get_field_id(env, Class, "canViewGiftsAndStars", "Z");
    can_sell_gifts_fieldID = jni::get_field_id(env, Class, "canSellGifts", "Z");
    can_change_gift_settings_fieldID = jni::get_field_id(env, Class, "canChangeGiftSettings", "Z");
    can_transfer_and_upgrade_gifts_fieldID = jni::get_field_id(env, Class, "canTransferAndUpgradeGifts", "Z");
    can_transfer_stars_fieldID = jni::get_field_id(env, Class, "canTransferStars", "Z");
    can_manage_stories_fieldID = jni::get_field_id(env, Class, "canManageStories", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass businessChatLink::Class;
jfieldID businessChatLink::link_fieldID;
jfieldID businessChatLink::text_fieldID;
jfieldID businessChatLink::title_fieldID;
jfieldID businessChatLink::view_count_fieldID;

businessChatLink::businessChatLink()
  : link_()
  , text_()
  , title_()
  , view_count_()
{}

businessChatLink::businessChatLink(string const &link_, object_ptr<formattedText> &&text_, string const &title_, int32 view_count_)
  : link_(link_)
  , text_(std::move(text_))
  , title_(title_)
  , view_count_(view_count_)
{}

const std::int32_t businessChatLink::ID;

object_ptr<businessChatLink> businessChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessChatLink> res = make_object<businessChatLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->view_count_ = env->GetIntField(p, res->view_count_fieldID);
  return res;
}

void businessChatLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_); if (nextString) { env->SetObjectField(s, link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, view_count_fieldID, view_count_);
}

void businessChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessChatLink");
    s.store_field("link", link_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("title", title_);
    s.store_field("viewCount", view_count_);
    s.store_class_end();
  }
}

void businessChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessChatLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    view_count_fieldID = jni::get_field_id(env, Class, "viewCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass businessChatLinkInfo::Class;
jfieldID businessChatLinkInfo::chat_id_fieldID;
jfieldID businessChatLinkInfo::text_fieldID;

businessChatLinkInfo::businessChatLinkInfo()
  : chat_id_()
  , text_()
{}

businessChatLinkInfo::businessChatLinkInfo(int53 chat_id_, object_ptr<formattedText> &&text_)
  : chat_id_(chat_id_)
  , text_(std::move(text_))
{}

const std::int32_t businessChatLinkInfo::ID;

object_ptr<businessChatLinkInfo> businessChatLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessChatLinkInfo> res = make_object<businessChatLinkInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return res;
}

void businessChatLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessChatLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessChatLinkInfo");
    s.store_field("chatId", chat_id_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void businessChatLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessChatLinkInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessChatLinks::Class;
jfieldID businessChatLinks::links_fieldID;

businessChatLinks::businessChatLinks()
  : links_()
{}

businessChatLinks::businessChatLinks(array<object_ptr<businessChatLink>> &&links_)
  : links_(std::move(links_))
{}

const std::int32_t businessChatLinks::ID;

object_ptr<businessChatLinks> businessChatLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessChatLinks> res = make_object<businessChatLinks>();
  res->links_ = jni::FetchVector<businessChatLink>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->links_fieldID));
  return res;
}

void businessChatLinks::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, links_); if (arr_tmp_) { env->SetObjectField(s, links_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void businessChatLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessChatLinks");
    { s.store_vector_begin("links", links_.size()); for (const auto &_value : links_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void businessChatLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessChatLinks").c_str());
    links_fieldID = jni::get_field_id(env, Class, "links", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BusinessChatLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessConnectedBot::Class;
jfieldID businessConnectedBot::bot_user_id_fieldID;
jfieldID businessConnectedBot::recipients_fieldID;
jfieldID businessConnectedBot::rights_fieldID;

businessConnectedBot::businessConnectedBot()
  : bot_user_id_()
  , recipients_()
  , rights_()
{}

businessConnectedBot::businessConnectedBot(int53 bot_user_id_, object_ptr<businessRecipients> &&recipients_, object_ptr<businessBotRights> &&rights_)
  : bot_user_id_(bot_user_id_)
  , recipients_(std::move(recipients_))
  , rights_(std::move(rights_))
{}

const std::int32_t businessConnectedBot::ID;

object_ptr<businessConnectedBot> businessConnectedBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessConnectedBot> res = make_object<businessConnectedBot>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->recipients_ = jni::fetch_tl_object<businessRecipients>(env, jni::fetch_object(env, p, res->recipients_fieldID));
  res->rights_ = jni::fetch_tl_object<businessBotRights>(env, jni::fetch_object(env, p, res->rights_fieldID));
  return res;
}

void businessConnectedBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  if (recipients_ != nullptr) { jobject next; recipients_->store(env, next); if (next) { env->SetObjectField(s, recipients_fieldID, next); env->DeleteLocalRef(next); } }
  if (rights_ != nullptr) { jobject next; rights_->store(env, next); if (next) { env->SetObjectField(s, rights_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessConnectedBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessConnectedBot");
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("recipients", static_cast<const BaseObject *>(recipients_.get()));
    s.store_object_field("rights", static_cast<const BaseObject *>(rights_.get()));
    s.store_class_end();
  }
}

void businessConnectedBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessConnectedBot").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    recipients_fieldID = jni::get_field_id(env, Class, "recipients", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessRecipients;").c_str());
    rights_fieldID = jni::get_field_id(env, Class, "rights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessBotRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessConnection::Class;
jfieldID businessConnection::id_fieldID;
jfieldID businessConnection::user_id_fieldID;
jfieldID businessConnection::user_chat_id_fieldID;
jfieldID businessConnection::date_fieldID;
jfieldID businessConnection::rights_fieldID;
jfieldID businessConnection::is_enabled_fieldID;

businessConnection::businessConnection()
  : id_()
  , user_id_()
  , user_chat_id_()
  , date_()
  , rights_()
  , is_enabled_()
{}

businessConnection::businessConnection(string const &id_, int53 user_id_, int53 user_chat_id_, int32 date_, object_ptr<businessBotRights> &&rights_, bool is_enabled_)
  : id_(id_)
  , user_id_(user_id_)
  , user_chat_id_(user_chat_id_)
  , date_(date_)
  , rights_(std::move(rights_))
  , is_enabled_(is_enabled_)
{}

const std::int32_t businessConnection::ID;

object_ptr<businessConnection> businessConnection::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessConnection> res = make_object<businessConnection>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->user_chat_id_ = env->GetLongField(p, res->user_chat_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->rights_ = jni::fetch_tl_object<businessBotRights>(env, jni::fetch_object(env, p, res->rights_fieldID));
  res->is_enabled_ = (env->GetBooleanField(p, res->is_enabled_fieldID) != 0);
  return res;
}

void businessConnection::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetLongField(s, user_chat_id_fieldID, user_chat_id_);
  env->SetIntField(s, date_fieldID, date_);
  if (rights_ != nullptr) { jobject next; rights_->store(env, next); if (next) { env->SetObjectField(s, rights_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_enabled_fieldID, is_enabled_);
}

void businessConnection::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessConnection");
    s.store_field("id", id_);
    s.store_field("userId", user_id_);
    s.store_field("userChatId", user_chat_id_);
    s.store_field("date", date_);
    s.store_object_field("rights", static_cast<const BaseObject *>(rights_.get()));
    s.store_field("isEnabled", is_enabled_);
    s.store_class_end();
  }
}

void businessConnection::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessConnection").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    user_chat_id_fieldID = jni::get_field_id(env, Class, "userChatId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    rights_fieldID = jni::get_field_id(env, Class, "rights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessBotRights;").c_str());
    is_enabled_fieldID = jni::get_field_id(env, Class, "isEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass BusinessFeature::Class;

object_ptr<BusinessFeature> BusinessFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case businessFeatureLocation::ID:
      return businessFeatureLocation::fetch(env, p);
    case businessFeatureOpeningHours::ID:
      return businessFeatureOpeningHours::fetch(env, p);
    case businessFeatureQuickReplies::ID:
      return businessFeatureQuickReplies::fetch(env, p);
    case businessFeatureGreetingMessage::ID:
      return businessFeatureGreetingMessage::fetch(env, p);
    case businessFeatureAwayMessage::ID:
      return businessFeatureAwayMessage::fetch(env, p);
    case businessFeatureAccountLinks::ID:
      return businessFeatureAccountLinks::fetch(env, p);
    case businessFeatureStartPage::ID:
      return businessFeatureStartPage::fetch(env, p);
    case businessFeatureBots::ID:
      return businessFeatureBots::fetch(env, p);
    case businessFeatureEmojiStatus::ID:
      return businessFeatureEmojiStatus::fetch(env, p);
    case businessFeatureChatFolderTags::ID:
      return businessFeatureChatFolderTags::fetch(env, p);
    case businessFeatureUpgradedStories::ID:
      return businessFeatureUpgradedStories::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void BusinessFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeature").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureLocation::Class;

businessFeatureLocation::businessFeatureLocation() {
}

const std::int32_t businessFeatureLocation::ID;

object_ptr<BusinessFeature> businessFeatureLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureLocation>();
}

void businessFeatureLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureLocation");
    s.store_class_end();
  }
}

void businessFeatureLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureLocation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureOpeningHours::Class;

businessFeatureOpeningHours::businessFeatureOpeningHours() {
}

const std::int32_t businessFeatureOpeningHours::ID;

object_ptr<BusinessFeature> businessFeatureOpeningHours::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureOpeningHours>();
}

void businessFeatureOpeningHours::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureOpeningHours::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureOpeningHours");
    s.store_class_end();
  }
}

void businessFeatureOpeningHours::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureOpeningHours").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureQuickReplies::Class;

businessFeatureQuickReplies::businessFeatureQuickReplies() {
}

const std::int32_t businessFeatureQuickReplies::ID;

object_ptr<BusinessFeature> businessFeatureQuickReplies::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureQuickReplies>();
}

void businessFeatureQuickReplies::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureQuickReplies::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureQuickReplies");
    s.store_class_end();
  }
}

void businessFeatureQuickReplies::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureQuickReplies").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureGreetingMessage::Class;

businessFeatureGreetingMessage::businessFeatureGreetingMessage() {
}

const std::int32_t businessFeatureGreetingMessage::ID;

object_ptr<BusinessFeature> businessFeatureGreetingMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureGreetingMessage>();
}

void businessFeatureGreetingMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureGreetingMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureGreetingMessage");
    s.store_class_end();
  }
}

void businessFeatureGreetingMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureGreetingMessage").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureAwayMessage::Class;

businessFeatureAwayMessage::businessFeatureAwayMessage() {
}

const std::int32_t businessFeatureAwayMessage::ID;

object_ptr<BusinessFeature> businessFeatureAwayMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureAwayMessage>();
}

void businessFeatureAwayMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureAwayMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureAwayMessage");
    s.store_class_end();
  }
}

void businessFeatureAwayMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureAwayMessage").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureAccountLinks::Class;

businessFeatureAccountLinks::businessFeatureAccountLinks() {
}

const std::int32_t businessFeatureAccountLinks::ID;

object_ptr<BusinessFeature> businessFeatureAccountLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureAccountLinks>();
}

void businessFeatureAccountLinks::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureAccountLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureAccountLinks");
    s.store_class_end();
  }
}

void businessFeatureAccountLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureAccountLinks").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureStartPage::Class;

businessFeatureStartPage::businessFeatureStartPage() {
}

const std::int32_t businessFeatureStartPage::ID;

object_ptr<BusinessFeature> businessFeatureStartPage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureStartPage>();
}

void businessFeatureStartPage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureStartPage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureStartPage");
    s.store_class_end();
  }
}

void businessFeatureStartPage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureStartPage").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureBots::Class;

businessFeatureBots::businessFeatureBots() {
}

const std::int32_t businessFeatureBots::ID;

object_ptr<BusinessFeature> businessFeatureBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureBots>();
}

void businessFeatureBots::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureBots");
    s.store_class_end();
  }
}

void businessFeatureBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureEmojiStatus::Class;

businessFeatureEmojiStatus::businessFeatureEmojiStatus() {
}

const std::int32_t businessFeatureEmojiStatus::ID;

object_ptr<BusinessFeature> businessFeatureEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureEmojiStatus>();
}

void businessFeatureEmojiStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureEmojiStatus");
    s.store_class_end();
  }
}

void businessFeatureEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureEmojiStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureChatFolderTags::Class;

businessFeatureChatFolderTags::businessFeatureChatFolderTags() {
}

const std::int32_t businessFeatureChatFolderTags::ID;

object_ptr<BusinessFeature> businessFeatureChatFolderTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureChatFolderTags>();
}

void businessFeatureChatFolderTags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureChatFolderTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureChatFolderTags");
    s.store_class_end();
  }
}

void businessFeatureChatFolderTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureChatFolderTags").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatureUpgradedStories::Class;

businessFeatureUpgradedStories::businessFeatureUpgradedStories() {
}

const std::int32_t businessFeatureUpgradedStories::ID;

object_ptr<BusinessFeature> businessFeatureUpgradedStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<businessFeatureUpgradedStories>();
}

void businessFeatureUpgradedStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void businessFeatureUpgradedStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatureUpgradedStories");
    s.store_class_end();
  }
}

void businessFeatureUpgradedStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatureUpgradedStories").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeaturePromotionAnimation::Class;
jfieldID businessFeaturePromotionAnimation::feature_fieldID;
jfieldID businessFeaturePromotionAnimation::animation_fieldID;

businessFeaturePromotionAnimation::businessFeaturePromotionAnimation()
  : feature_()
  , animation_()
{}

businessFeaturePromotionAnimation::businessFeaturePromotionAnimation(object_ptr<BusinessFeature> &&feature_, object_ptr<animation> &&animation_)
  : feature_(std::move(feature_))
  , animation_(std::move(animation_))
{}

const std::int32_t businessFeaturePromotionAnimation::ID;

object_ptr<businessFeaturePromotionAnimation> businessFeaturePromotionAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessFeaturePromotionAnimation> res = make_object<businessFeaturePromotionAnimation>();
  res->feature_ = jni::fetch_tl_object<BusinessFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void businessFeaturePromotionAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (feature_ != nullptr) { jobject next; feature_->store(env, next); if (next) { env->SetObjectField(s, feature_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessFeaturePromotionAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeaturePromotionAnimation");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void businessFeaturePromotionAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeaturePromotionAnimation").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessFeature;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessFeatures::Class;
jfieldID businessFeatures::features_fieldID;

businessFeatures::businessFeatures()
  : features_()
{}

businessFeatures::businessFeatures(array<object_ptr<BusinessFeature>> &&features_)
  : features_(std::move(features_))
{}

const std::int32_t businessFeatures::ID;

object_ptr<businessFeatures> businessFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessFeatures> res = make_object<businessFeatures>();
  res->features_ = jni::FetchVector<BusinessFeature>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->features_fieldID));
  return res;
}

void businessFeatures::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, features_); if (arr_tmp_) { env->SetObjectField(s, features_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void businessFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessFeatures");
    { s.store_vector_begin("features", features_.size()); for (const auto &_value : features_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void businessFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessFeatures").c_str());
    features_fieldID = jni::get_field_id(env, Class, "features", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BusinessFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessGreetingMessageSettings::Class;
jfieldID businessGreetingMessageSettings::shortcut_id_fieldID;
jfieldID businessGreetingMessageSettings::recipients_fieldID;
jfieldID businessGreetingMessageSettings::inactivity_days_fieldID;

businessGreetingMessageSettings::businessGreetingMessageSettings()
  : shortcut_id_()
  , recipients_()
  , inactivity_days_()
{}

businessGreetingMessageSettings::businessGreetingMessageSettings(int32 shortcut_id_, object_ptr<businessRecipients> &&recipients_, int32 inactivity_days_)
  : shortcut_id_(shortcut_id_)
  , recipients_(std::move(recipients_))
  , inactivity_days_(inactivity_days_)
{}

const std::int32_t businessGreetingMessageSettings::ID;

object_ptr<businessGreetingMessageSettings> businessGreetingMessageSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessGreetingMessageSettings> res = make_object<businessGreetingMessageSettings>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->recipients_ = jni::fetch_tl_object<businessRecipients>(env, jni::fetch_object(env, p, res->recipients_fieldID));
  res->inactivity_days_ = env->GetIntField(p, res->inactivity_days_fieldID);
  return res;
}

void businessGreetingMessageSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, shortcut_id_fieldID, shortcut_id_);
  if (recipients_ != nullptr) { jobject next; recipients_->store(env, next); if (next) { env->SetObjectField(s, recipients_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, inactivity_days_fieldID, inactivity_days_);
}

void businessGreetingMessageSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessGreetingMessageSettings");
    s.store_field("shortcutId", shortcut_id_);
    s.store_object_field("recipients", static_cast<const BaseObject *>(recipients_.get()));
    s.store_field("inactivityDays", inactivity_days_);
    s.store_class_end();
  }
}

void businessGreetingMessageSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessGreetingMessageSettings").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    recipients_fieldID = jni::get_field_id(env, Class, "recipients", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessRecipients;").c_str());
    inactivity_days_fieldID = jni::get_field_id(env, Class, "inactivityDays", "I");
    return true;
  }();
  (void)is_inited;
}

jclass businessInfo::Class;
jfieldID businessInfo::location_fieldID;
jfieldID businessInfo::opening_hours_fieldID;
jfieldID businessInfo::local_opening_hours_fieldID;
jfieldID businessInfo::next_open_in_fieldID;
jfieldID businessInfo::next_close_in_fieldID;
jfieldID businessInfo::greeting_message_settings_fieldID;
jfieldID businessInfo::away_message_settings_fieldID;
jfieldID businessInfo::start_page_fieldID;

businessInfo::businessInfo()
  : location_()
  , opening_hours_()
  , local_opening_hours_()
  , next_open_in_()
  , next_close_in_()
  , greeting_message_settings_()
  , away_message_settings_()
  , start_page_()
{}

businessInfo::businessInfo(object_ptr<businessLocation> &&location_, object_ptr<businessOpeningHours> &&opening_hours_, object_ptr<businessOpeningHours> &&local_opening_hours_, int32 next_open_in_, int32 next_close_in_, object_ptr<businessGreetingMessageSettings> &&greeting_message_settings_, object_ptr<businessAwayMessageSettings> &&away_message_settings_, object_ptr<businessStartPage> &&start_page_)
  : location_(std::move(location_))
  , opening_hours_(std::move(opening_hours_))
  , local_opening_hours_(std::move(local_opening_hours_))
  , next_open_in_(next_open_in_)
  , next_close_in_(next_close_in_)
  , greeting_message_settings_(std::move(greeting_message_settings_))
  , away_message_settings_(std::move(away_message_settings_))
  , start_page_(std::move(start_page_))
{}

const std::int32_t businessInfo::ID;

object_ptr<businessInfo> businessInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessInfo> res = make_object<businessInfo>();
  res->location_ = jni::fetch_tl_object<businessLocation>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->opening_hours_ = jni::fetch_tl_object<businessOpeningHours>(env, jni::fetch_object(env, p, res->opening_hours_fieldID));
  res->local_opening_hours_ = jni::fetch_tl_object<businessOpeningHours>(env, jni::fetch_object(env, p, res->local_opening_hours_fieldID));
  res->next_open_in_ = env->GetIntField(p, res->next_open_in_fieldID);
  res->next_close_in_ = env->GetIntField(p, res->next_close_in_fieldID);
  res->greeting_message_settings_ = jni::fetch_tl_object<businessGreetingMessageSettings>(env, jni::fetch_object(env, p, res->greeting_message_settings_fieldID));
  res->away_message_settings_ = jni::fetch_tl_object<businessAwayMessageSettings>(env, jni::fetch_object(env, p, res->away_message_settings_fieldID));
  res->start_page_ = jni::fetch_tl_object<businessStartPage>(env, jni::fetch_object(env, p, res->start_page_fieldID));
  return res;
}

void businessInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  if (opening_hours_ != nullptr) { jobject next; opening_hours_->store(env, next); if (next) { env->SetObjectField(s, opening_hours_fieldID, next); env->DeleteLocalRef(next); } }
  if (local_opening_hours_ != nullptr) { jobject next; local_opening_hours_->store(env, next); if (next) { env->SetObjectField(s, local_opening_hours_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, next_open_in_fieldID, next_open_in_);
  env->SetIntField(s, next_close_in_fieldID, next_close_in_);
  if (greeting_message_settings_ != nullptr) { jobject next; greeting_message_settings_->store(env, next); if (next) { env->SetObjectField(s, greeting_message_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (away_message_settings_ != nullptr) { jobject next; away_message_settings_->store(env, next); if (next) { env->SetObjectField(s, away_message_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (start_page_ != nullptr) { jobject next; start_page_->store(env, next); if (next) { env->SetObjectField(s, start_page_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessInfo");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_object_field("openingHours", static_cast<const BaseObject *>(opening_hours_.get()));
    s.store_object_field("localOpeningHours", static_cast<const BaseObject *>(local_opening_hours_.get()));
    s.store_field("nextOpenIn", next_open_in_);
    s.store_field("nextCloseIn", next_close_in_);
    s.store_object_field("greetingMessageSettings", static_cast<const BaseObject *>(greeting_message_settings_.get()));
    s.store_object_field("awayMessageSettings", static_cast<const BaseObject *>(away_message_settings_.get()));
    s.store_object_field("startPage", static_cast<const BaseObject *>(start_page_.get()));
    s.store_class_end();
  }
}

void businessInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessInfo").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessLocation;").c_str());
    opening_hours_fieldID = jni::get_field_id(env, Class, "openingHours", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessOpeningHours;").c_str());
    local_opening_hours_fieldID = jni::get_field_id(env, Class, "localOpeningHours", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessOpeningHours;").c_str());
    next_open_in_fieldID = jni::get_field_id(env, Class, "nextOpenIn", "I");
    next_close_in_fieldID = jni::get_field_id(env, Class, "nextCloseIn", "I");
    greeting_message_settings_fieldID = jni::get_field_id(env, Class, "greetingMessageSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessGreetingMessageSettings;").c_str());
    away_message_settings_fieldID = jni::get_field_id(env, Class, "awayMessageSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessAwayMessageSettings;").c_str());
    start_page_fieldID = jni::get_field_id(env, Class, "startPage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessStartPage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessLocation::Class;
jfieldID businessLocation::location_fieldID;
jfieldID businessLocation::address_fieldID;

businessLocation::businessLocation()
  : location_()
  , address_()
{}

businessLocation::businessLocation(object_ptr<location> &&location_, string const &address_)
  : location_(std::move(location_))
  , address_(address_)
{}

const std::int32_t businessLocation::ID;

object_ptr<businessLocation> businessLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessLocation> res = make_object<businessLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->address_ = jni::fetch_string(env, p, res->address_fieldID);
  return res;
}

void businessLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, address_); if (nextString) { env->SetObjectField(s, address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void businessLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("address", address_);
    s.store_class_end();
  }
}

void businessLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass businessMessage::Class;
jfieldID businessMessage::message_fieldID;
jfieldID businessMessage::reply_to_message_fieldID;

businessMessage::businessMessage()
  : message_()
  , reply_to_message_()
{}

businessMessage::businessMessage(object_ptr<message> &&message_, object_ptr<message> &&reply_to_message_)
  : message_(std::move(message_))
  , reply_to_message_(std::move(reply_to_message_))
{}

const std::int32_t businessMessage::ID;

object_ptr<businessMessage> businessMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessMessage> res = make_object<businessMessage>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->reply_to_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->reply_to_message_fieldID));
  return res;
}

void businessMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  if (reply_to_message_ != nullptr) { jobject next; reply_to_message_->store(env, next); if (next) { env->SetObjectField(s, reply_to_message_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessMessage");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_object_field("replyToMessage", static_cast<const BaseObject *>(reply_to_message_.get()));
    s.store_class_end();
  }
}

void businessMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessMessage").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    reply_to_message_fieldID = jni::get_field_id(env, Class, "replyToMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessMessages::Class;
jfieldID businessMessages::messages_fieldID;

businessMessages::businessMessages()
  : messages_()
{}

businessMessages::businessMessages(array<object_ptr<businessMessage>> &&messages_)
  : messages_(std::move(messages_))
{}

const std::int32_t businessMessages::ID;

object_ptr<businessMessages> businessMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessMessages> res = make_object<businessMessages>();
  res->messages_ = jni::FetchVector<businessMessage>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  return res;
}

void businessMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void businessMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessMessages");
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void businessMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessMessages").c_str());
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BusinessMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessOpeningHours::Class;
jfieldID businessOpeningHours::time_zone_id_fieldID;
jfieldID businessOpeningHours::opening_hours_fieldID;

businessOpeningHours::businessOpeningHours()
  : time_zone_id_()
  , opening_hours_()
{}

businessOpeningHours::businessOpeningHours(string const &time_zone_id_, array<object_ptr<businessOpeningHoursInterval>> &&opening_hours_)
  : time_zone_id_(time_zone_id_)
  , opening_hours_(std::move(opening_hours_))
{}

const std::int32_t businessOpeningHours::ID;

object_ptr<businessOpeningHours> businessOpeningHours::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessOpeningHours> res = make_object<businessOpeningHours>();
  res->time_zone_id_ = jni::fetch_string(env, p, res->time_zone_id_fieldID);
  res->opening_hours_ = jni::FetchVector<businessOpeningHoursInterval>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->opening_hours_fieldID));
  return res;
}

void businessOpeningHours::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, time_zone_id_); if (nextString) { env->SetObjectField(s, time_zone_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, opening_hours_); if (arr_tmp_) { env->SetObjectField(s, opening_hours_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void businessOpeningHours::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessOpeningHours");
    s.store_field("timeZoneId", time_zone_id_);
    { s.store_vector_begin("openingHours", opening_hours_.size()); for (const auto &_value : opening_hours_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void businessOpeningHours::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessOpeningHours").c_str());
    time_zone_id_fieldID = jni::get_field_id(env, Class, "timeZoneId", "Ljava/lang/String;");
    opening_hours_fieldID = jni::get_field_id(env, Class, "openingHours", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BusinessOpeningHoursInterval;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass businessOpeningHoursInterval::Class;
jfieldID businessOpeningHoursInterval::start_minute_fieldID;
jfieldID businessOpeningHoursInterval::end_minute_fieldID;

businessOpeningHoursInterval::businessOpeningHoursInterval()
  : start_minute_()
  , end_minute_()
{}

businessOpeningHoursInterval::businessOpeningHoursInterval(int32 start_minute_, int32 end_minute_)
  : start_minute_(start_minute_)
  , end_minute_(end_minute_)
{}

const std::int32_t businessOpeningHoursInterval::ID;

object_ptr<businessOpeningHoursInterval> businessOpeningHoursInterval::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessOpeningHoursInterval> res = make_object<businessOpeningHoursInterval>();
  res->start_minute_ = env->GetIntField(p, res->start_minute_fieldID);
  res->end_minute_ = env->GetIntField(p, res->end_minute_fieldID);
  return res;
}

void businessOpeningHoursInterval::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, start_minute_fieldID, start_minute_);
  env->SetIntField(s, end_minute_fieldID, end_minute_);
}

void businessOpeningHoursInterval::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessOpeningHoursInterval");
    s.store_field("startMinute", start_minute_);
    s.store_field("endMinute", end_minute_);
    s.store_class_end();
  }
}

void businessOpeningHoursInterval::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessOpeningHoursInterval").c_str());
    start_minute_fieldID = jni::get_field_id(env, Class, "startMinute", "I");
    end_minute_fieldID = jni::get_field_id(env, Class, "endMinute", "I");
    return true;
  }();
  (void)is_inited;
}

jclass businessRecipients::Class;
jfieldID businessRecipients::chat_ids_fieldID;
jfieldID businessRecipients::excluded_chat_ids_fieldID;
jfieldID businessRecipients::select_existing_chats_fieldID;
jfieldID businessRecipients::select_new_chats_fieldID;
jfieldID businessRecipients::select_contacts_fieldID;
jfieldID businessRecipients::select_non_contacts_fieldID;
jfieldID businessRecipients::exclude_selected_fieldID;

businessRecipients::businessRecipients()
  : chat_ids_()
  , excluded_chat_ids_()
  , select_existing_chats_()
  , select_new_chats_()
  , select_contacts_()
  , select_non_contacts_()
  , exclude_selected_()
{}

businessRecipients::businessRecipients(array<int53> &&chat_ids_, array<int53> &&excluded_chat_ids_, bool select_existing_chats_, bool select_new_chats_, bool select_contacts_, bool select_non_contacts_, bool exclude_selected_)
  : chat_ids_(std::move(chat_ids_))
  , excluded_chat_ids_(std::move(excluded_chat_ids_))
  , select_existing_chats_(select_existing_chats_)
  , select_new_chats_(select_new_chats_)
  , select_contacts_(select_contacts_)
  , select_non_contacts_(select_non_contacts_)
  , exclude_selected_(exclude_selected_)
{}

const std::int32_t businessRecipients::ID;

object_ptr<businessRecipients> businessRecipients::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessRecipients> res = make_object<businessRecipients>();
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  res->excluded_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->excluded_chat_ids_fieldID));
  res->select_existing_chats_ = (env->GetBooleanField(p, res->select_existing_chats_fieldID) != 0);
  res->select_new_chats_ = (env->GetBooleanField(p, res->select_new_chats_fieldID) != 0);
  res->select_contacts_ = (env->GetBooleanField(p, res->select_contacts_fieldID) != 0);
  res->select_non_contacts_ = (env->GetBooleanField(p, res->select_non_contacts_fieldID) != 0);
  res->exclude_selected_ = (env->GetBooleanField(p, res->exclude_selected_fieldID) != 0);
  return res;
}

void businessRecipients::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_ids_); if (arr_tmp_) { env->SetObjectField(s, chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, excluded_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, excluded_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, select_existing_chats_fieldID, select_existing_chats_);
  env->SetBooleanField(s, select_new_chats_fieldID, select_new_chats_);
  env->SetBooleanField(s, select_contacts_fieldID, select_contacts_);
  env->SetBooleanField(s, select_non_contacts_fieldID, select_non_contacts_);
  env->SetBooleanField(s, exclude_selected_fieldID, exclude_selected_);
}

void businessRecipients::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessRecipients");
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("excludedChatIds", excluded_chat_ids_.size()); for (const auto &_value : excluded_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("selectExistingChats", select_existing_chats_);
    s.store_field("selectNewChats", select_new_chats_);
    s.store_field("selectContacts", select_contacts_);
    s.store_field("selectNonContacts", select_non_contacts_);
    s.store_field("excludeSelected", exclude_selected_);
    s.store_class_end();
  }
}

void businessRecipients::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessRecipients").c_str());
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    excluded_chat_ids_fieldID = jni::get_field_id(env, Class, "excludedChatIds", "[J");
    select_existing_chats_fieldID = jni::get_field_id(env, Class, "selectExistingChats", "Z");
    select_new_chats_fieldID = jni::get_field_id(env, Class, "selectNewChats", "Z");
    select_contacts_fieldID = jni::get_field_id(env, Class, "selectContacts", "Z");
    select_non_contacts_fieldID = jni::get_field_id(env, Class, "selectNonContacts", "Z");
    exclude_selected_fieldID = jni::get_field_id(env, Class, "excludeSelected", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass businessStartPage::Class;
jfieldID businessStartPage::title_fieldID;
jfieldID businessStartPage::message_fieldID;
jfieldID businessStartPage::sticker_fieldID;

businessStartPage::businessStartPage()
  : title_()
  , message_()
  , sticker_()
{}

businessStartPage::businessStartPage(string const &title_, string const &message_, object_ptr<sticker> &&sticker_)
  : title_(title_)
  , message_(message_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t businessStartPage::ID;

object_ptr<businessStartPage> businessStartPage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<businessStartPage> res = make_object<businessStartPage>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->message_ = jni::fetch_string(env, p, res->message_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void businessStartPage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, message_); if (nextString) { env->SetObjectField(s, message_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void businessStartPage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BusinessStartPage");
    s.store_field("title", title_);
    s.store_field("message", message_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void businessStartPage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BusinessStartPage").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    message_fieldID = jni::get_field_id(env, Class, "message", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass call::Class;
jfieldID call::id_fieldID;
jfieldID call::user_id_fieldID;
jfieldID call::is_outgoing_fieldID;
jfieldID call::is_video_fieldID;
jfieldID call::state_fieldID;

call::call()
  : id_()
  , user_id_()
  , is_outgoing_()
  , is_video_()
  , state_()
{}

call::call(int32 id_, int53 user_id_, bool is_outgoing_, bool is_video_, object_ptr<CallState> &&state_)
  : id_(id_)
  , user_id_(user_id_)
  , is_outgoing_(is_outgoing_)
  , is_video_(is_video_)
  , state_(std::move(state_))
{}

const std::int32_t call::ID;

object_ptr<call> call::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<call> res = make_object<call>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  res->state_ = jni::fetch_tl_object<CallState>(env, jni::fetch_object(env, p, res->state_fieldID));
  return res;
}

void call::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
  env->SetBooleanField(s, is_video_fieldID, is_video_);
  if (state_ != nullptr) { jobject next; state_->store(env, next); if (next) { env->SetObjectField(s, state_fieldID, next); env->DeleteLocalRef(next); } }
}

void call::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Call");
    s.store_field("id", id_);
    s.store_field("userId", user_id_);
    s.store_field("isOutgoing", is_outgoing_);
    s.store_field("isVideo", is_video_);
    s.store_object_field("state", static_cast<const BaseObject *>(state_.get()));
    s.store_class_end();
  }
}

void call::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Call").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    state_fieldID = jni::get_field_id(env, Class, "state", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass CallDiscardReason::Class;

object_ptr<CallDiscardReason> CallDiscardReason::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case callDiscardReasonEmpty::ID:
      return callDiscardReasonEmpty::fetch(env, p);
    case callDiscardReasonMissed::ID:
      return callDiscardReasonMissed::fetch(env, p);
    case callDiscardReasonDeclined::ID:
      return callDiscardReasonDeclined::fetch(env, p);
    case callDiscardReasonDisconnected::ID:
      return callDiscardReasonDisconnected::fetch(env, p);
    case callDiscardReasonHungUp::ID:
      return callDiscardReasonHungUp::fetch(env, p);
    case callDiscardReasonUpgradeToGroupCall::ID:
      return callDiscardReasonUpgradeToGroupCall::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CallDiscardReason::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReason").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonEmpty::Class;

callDiscardReasonEmpty::callDiscardReasonEmpty() {
}

const std::int32_t callDiscardReasonEmpty::ID;

object_ptr<CallDiscardReason> callDiscardReasonEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callDiscardReasonEmpty>();
}

void callDiscardReasonEmpty::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callDiscardReasonEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonEmpty");
    s.store_class_end();
  }
}

void callDiscardReasonEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonMissed::Class;

callDiscardReasonMissed::callDiscardReasonMissed() {
}

const std::int32_t callDiscardReasonMissed::ID;

object_ptr<CallDiscardReason> callDiscardReasonMissed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callDiscardReasonMissed>();
}

void callDiscardReasonMissed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callDiscardReasonMissed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonMissed");
    s.store_class_end();
  }
}

void callDiscardReasonMissed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonMissed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonDeclined::Class;

callDiscardReasonDeclined::callDiscardReasonDeclined() {
}

const std::int32_t callDiscardReasonDeclined::ID;

object_ptr<CallDiscardReason> callDiscardReasonDeclined::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callDiscardReasonDeclined>();
}

void callDiscardReasonDeclined::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callDiscardReasonDeclined::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonDeclined");
    s.store_class_end();
  }
}

void callDiscardReasonDeclined::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonDeclined").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonDisconnected::Class;

callDiscardReasonDisconnected::callDiscardReasonDisconnected() {
}

const std::int32_t callDiscardReasonDisconnected::ID;

object_ptr<CallDiscardReason> callDiscardReasonDisconnected::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callDiscardReasonDisconnected>();
}

void callDiscardReasonDisconnected::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callDiscardReasonDisconnected::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonDisconnected");
    s.store_class_end();
  }
}

void callDiscardReasonDisconnected::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonDisconnected").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonHungUp::Class;

callDiscardReasonHungUp::callDiscardReasonHungUp() {
}

const std::int32_t callDiscardReasonHungUp::ID;

object_ptr<CallDiscardReason> callDiscardReasonHungUp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callDiscardReasonHungUp>();
}

void callDiscardReasonHungUp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callDiscardReasonHungUp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonHungUp");
    s.store_class_end();
  }
}

void callDiscardReasonHungUp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonHungUp").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callDiscardReasonUpgradeToGroupCall::Class;
jfieldID callDiscardReasonUpgradeToGroupCall::invite_link_fieldID;

callDiscardReasonUpgradeToGroupCall::callDiscardReasonUpgradeToGroupCall()
  : invite_link_()
{}

callDiscardReasonUpgradeToGroupCall::callDiscardReasonUpgradeToGroupCall(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t callDiscardReasonUpgradeToGroupCall::ID;

object_ptr<CallDiscardReason> callDiscardReasonUpgradeToGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callDiscardReasonUpgradeToGroupCall> res = make_object<callDiscardReasonUpgradeToGroupCall>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void callDiscardReasonUpgradeToGroupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void callDiscardReasonUpgradeToGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallDiscardReasonUpgradeToGroupCall");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void callDiscardReasonUpgradeToGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallDiscardReasonUpgradeToGroupCall").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass callId::Class;
jfieldID callId::id_fieldID;

callId::callId()
  : id_()
{}

callId::callId(int32 id_)
  : id_(id_)
{}

const std::int32_t callId::ID;

object_ptr<callId> callId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callId> res = make_object<callId>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  return res;
}

void callId::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
}

void callId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallId");
    s.store_field("id", id_);
    s.store_class_end();
  }
}

void callId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallId").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    return true;
  }();
  (void)is_inited;
}

jclass CallProblem::Class;

object_ptr<CallProblem> CallProblem::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case callProblemEcho::ID:
      return callProblemEcho::fetch(env, p);
    case callProblemNoise::ID:
      return callProblemNoise::fetch(env, p);
    case callProblemInterruptions::ID:
      return callProblemInterruptions::fetch(env, p);
    case callProblemDistortedSpeech::ID:
      return callProblemDistortedSpeech::fetch(env, p);
    case callProblemSilentLocal::ID:
      return callProblemSilentLocal::fetch(env, p);
    case callProblemSilentRemote::ID:
      return callProblemSilentRemote::fetch(env, p);
    case callProblemDropped::ID:
      return callProblemDropped::fetch(env, p);
    case callProblemDistortedVideo::ID:
      return callProblemDistortedVideo::fetch(env, p);
    case callProblemPixelatedVideo::ID:
      return callProblemPixelatedVideo::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CallProblem::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblem").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemEcho::Class;

callProblemEcho::callProblemEcho() {
}

const std::int32_t callProblemEcho::ID;

object_ptr<CallProblem> callProblemEcho::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemEcho>();
}

void callProblemEcho::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemEcho");
    s.store_class_end();
  }
}

void callProblemEcho::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemEcho").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemNoise::Class;

callProblemNoise::callProblemNoise() {
}

const std::int32_t callProblemNoise::ID;

object_ptr<CallProblem> callProblemNoise::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemNoise>();
}

void callProblemNoise::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemNoise");
    s.store_class_end();
  }
}

void callProblemNoise::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemNoise").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemInterruptions::Class;

callProblemInterruptions::callProblemInterruptions() {
}

const std::int32_t callProblemInterruptions::ID;

object_ptr<CallProblem> callProblemInterruptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemInterruptions>();
}

void callProblemInterruptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemInterruptions");
    s.store_class_end();
  }
}

void callProblemInterruptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemInterruptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemDistortedSpeech::Class;

callProblemDistortedSpeech::callProblemDistortedSpeech() {
}

const std::int32_t callProblemDistortedSpeech::ID;

object_ptr<CallProblem> callProblemDistortedSpeech::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemDistortedSpeech>();
}

void callProblemDistortedSpeech::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemDistortedSpeech");
    s.store_class_end();
  }
}

void callProblemDistortedSpeech::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemDistortedSpeech").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemSilentLocal::Class;

callProblemSilentLocal::callProblemSilentLocal() {
}

const std::int32_t callProblemSilentLocal::ID;

object_ptr<CallProblem> callProblemSilentLocal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemSilentLocal>();
}

void callProblemSilentLocal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemSilentLocal");
    s.store_class_end();
  }
}

void callProblemSilentLocal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemSilentLocal").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemSilentRemote::Class;

callProblemSilentRemote::callProblemSilentRemote() {
}

const std::int32_t callProblemSilentRemote::ID;

object_ptr<CallProblem> callProblemSilentRemote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemSilentRemote>();
}

void callProblemSilentRemote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemSilentRemote");
    s.store_class_end();
  }
}

void callProblemSilentRemote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemSilentRemote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemDropped::Class;

callProblemDropped::callProblemDropped() {
}

const std::int32_t callProblemDropped::ID;

object_ptr<CallProblem> callProblemDropped::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemDropped>();
}

void callProblemDropped::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemDropped");
    s.store_class_end();
  }
}

void callProblemDropped::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemDropped").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemDistortedVideo::Class;

callProblemDistortedVideo::callProblemDistortedVideo() {
}

const std::int32_t callProblemDistortedVideo::ID;

object_ptr<CallProblem> callProblemDistortedVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemDistortedVideo>();
}

void callProblemDistortedVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemDistortedVideo");
    s.store_class_end();
  }
}

void callProblemDistortedVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemDistortedVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProblemPixelatedVideo::Class;

callProblemPixelatedVideo::callProblemPixelatedVideo() {
}

const std::int32_t callProblemPixelatedVideo::ID;

object_ptr<CallProblem> callProblemPixelatedVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callProblemPixelatedVideo>();
}

void callProblemPixelatedVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProblemPixelatedVideo");
    s.store_class_end();
  }
}

void callProblemPixelatedVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProblemPixelatedVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callProtocol::Class;
jfieldID callProtocol::udp_p2p_fieldID;
jfieldID callProtocol::udp_reflector_fieldID;
jfieldID callProtocol::min_layer_fieldID;
jfieldID callProtocol::max_layer_fieldID;
jfieldID callProtocol::library_versions_fieldID;

callProtocol::callProtocol()
  : udp_p2p_()
  , udp_reflector_()
  , min_layer_()
  , max_layer_()
  , library_versions_()
{}

callProtocol::callProtocol(bool udp_p2p_, bool udp_reflector_, int32 min_layer_, int32 max_layer_, array<string> &&library_versions_)
  : udp_p2p_(udp_p2p_)
  , udp_reflector_(udp_reflector_)
  , min_layer_(min_layer_)
  , max_layer_(max_layer_)
  , library_versions_(std::move(library_versions_))
{}

const std::int32_t callProtocol::ID;

object_ptr<callProtocol> callProtocol::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callProtocol> res = make_object<callProtocol>();
  res->udp_p2p_ = (env->GetBooleanField(p, res->udp_p2p_fieldID) != 0);
  res->udp_reflector_ = (env->GetBooleanField(p, res->udp_reflector_fieldID) != 0);
  res->min_layer_ = env->GetIntField(p, res->min_layer_fieldID);
  res->max_layer_ = env->GetIntField(p, res->max_layer_fieldID);
  res->library_versions_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->library_versions_fieldID));
  return res;
}

void callProtocol::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, udp_p2p_fieldID, udp_p2p_);
  env->SetBooleanField(s, udp_reflector_fieldID, udp_reflector_);
  env->SetIntField(s, min_layer_fieldID, min_layer_);
  env->SetIntField(s, max_layer_fieldID, max_layer_);
  { auto arr_tmp_ = jni::store_vector(env, library_versions_); if (arr_tmp_) { env->SetObjectField(s, library_versions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void callProtocol::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallProtocol");
    s.store_field("udpP2p", udp_p2p_);
    s.store_field("udpReflector", udp_reflector_);
    s.store_field("minLayer", min_layer_);
    s.store_field("maxLayer", max_layer_);
    { s.store_vector_begin("libraryVersions", library_versions_.size()); for (const auto &_value : library_versions_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void callProtocol::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallProtocol").c_str());
    udp_p2p_fieldID = jni::get_field_id(env, Class, "udpP2p", "Z");
    udp_reflector_fieldID = jni::get_field_id(env, Class, "udpReflector", "Z");
    min_layer_fieldID = jni::get_field_id(env, Class, "minLayer", "I");
    max_layer_fieldID = jni::get_field_id(env, Class, "maxLayer", "I");
    library_versions_fieldID = jni::get_field_id(env, Class, "libraryVersions", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass callServer::Class;
jfieldID callServer::id_fieldID;
jfieldID callServer::ip_address_fieldID;
jfieldID callServer::ipv6_address_fieldID;
jfieldID callServer::port_fieldID;
jfieldID callServer::type_fieldID;

callServer::callServer()
  : id_()
  , ip_address_()
  , ipv6_address_()
  , port_()
  , type_()
{}

callServer::callServer(int64 id_, string const &ip_address_, string const &ipv6_address_, int32 port_, object_ptr<CallServerType> &&type_)
  : id_(id_)
  , ip_address_(ip_address_)
  , ipv6_address_(ipv6_address_)
  , port_(port_)
  , type_(std::move(type_))
{}

const std::int32_t callServer::ID;

object_ptr<callServer> callServer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callServer> res = make_object<callServer>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->ip_address_ = jni::fetch_string(env, p, res->ip_address_fieldID);
  res->ipv6_address_ = jni::fetch_string(env, p, res->ipv6_address_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->type_ = jni::fetch_tl_object<CallServerType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void callServer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, ip_address_); if (nextString) { env->SetObjectField(s, ip_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, ipv6_address_); if (nextString) { env->SetObjectField(s, ipv6_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, port_fieldID, port_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void callServer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallServer");
    s.store_field("id", id_);
    s.store_field("ipAddress", ip_address_);
    s.store_field("ipv6Address", ipv6_address_);
    s.store_field("port", port_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void callServer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallServer").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    ip_address_fieldID = jni::get_field_id(env, Class, "ipAddress", "Ljava/lang/String;");
    ipv6_address_fieldID = jni::get_field_id(env, Class, "ipv6Address", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallServerType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass CallServerType::Class;

object_ptr<CallServerType> CallServerType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case callServerTypeTelegramReflector::ID:
      return callServerTypeTelegramReflector::fetch(env, p);
    case callServerTypeWebrtc::ID:
      return callServerTypeWebrtc::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CallServerType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallServerType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callServerTypeTelegramReflector::Class;
jfieldID callServerTypeTelegramReflector::peer_tag_fieldID;
jfieldID callServerTypeTelegramReflector::is_tcp_fieldID;

callServerTypeTelegramReflector::callServerTypeTelegramReflector()
  : peer_tag_()
  , is_tcp_()
{}

callServerTypeTelegramReflector::callServerTypeTelegramReflector(bytes const &peer_tag_, bool is_tcp_)
  : peer_tag_(std::move(peer_tag_))
  , is_tcp_(is_tcp_)
{}

const std::int32_t callServerTypeTelegramReflector::ID;

object_ptr<CallServerType> callServerTypeTelegramReflector::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callServerTypeTelegramReflector> res = make_object<callServerTypeTelegramReflector>();
  res->peer_tag_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->peer_tag_fieldID));
  res->is_tcp_ = (env->GetBooleanField(p, res->is_tcp_fieldID) != 0);
  return std::move(res);
}

void callServerTypeTelegramReflector::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, peer_tag_); if (nextBytes) { env->SetObjectField(s, peer_tag_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  env->SetBooleanField(s, is_tcp_fieldID, is_tcp_);
}

void callServerTypeTelegramReflector::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallServerTypeTelegramReflector");
    s.store_bytes_field("peerTag", peer_tag_);
    s.store_field("isTcp", is_tcp_);
    s.store_class_end();
  }
}

void callServerTypeTelegramReflector::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallServerTypeTelegramReflector").c_str());
    peer_tag_fieldID = jni::get_field_id(env, Class, "peerTag", "[B");
    is_tcp_fieldID = jni::get_field_id(env, Class, "isTcp", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass callServerTypeWebrtc::Class;
jfieldID callServerTypeWebrtc::username_fieldID;
jfieldID callServerTypeWebrtc::password_fieldID;
jfieldID callServerTypeWebrtc::supports_turn_fieldID;
jfieldID callServerTypeWebrtc::supports_stun_fieldID;

callServerTypeWebrtc::callServerTypeWebrtc()
  : username_()
  , password_()
  , supports_turn_()
  , supports_stun_()
{}

callServerTypeWebrtc::callServerTypeWebrtc(string const &username_, string const &password_, bool supports_turn_, bool supports_stun_)
  : username_(username_)
  , password_(password_)
  , supports_turn_(supports_turn_)
  , supports_stun_(supports_stun_)
{}

const std::int32_t callServerTypeWebrtc::ID;

object_ptr<CallServerType> callServerTypeWebrtc::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callServerTypeWebrtc> res = make_object<callServerTypeWebrtc>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  res->supports_turn_ = (env->GetBooleanField(p, res->supports_turn_fieldID) != 0);
  res->supports_stun_ = (env->GetBooleanField(p, res->supports_stun_fieldID) != 0);
  return std::move(res);
}

void callServerTypeWebrtc::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, password_); if (nextString) { env->SetObjectField(s, password_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, supports_turn_fieldID, supports_turn_);
  env->SetBooleanField(s, supports_stun_fieldID, supports_stun_);
}

void callServerTypeWebrtc::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallServerTypeWebrtc");
    s.store_field("username", username_);
    s.store_field("password", password_);
    s.store_field("supportsTurn", supports_turn_);
    s.store_field("supportsStun", supports_stun_);
    s.store_class_end();
  }
}

void callServerTypeWebrtc::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallServerTypeWebrtc").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    supports_turn_fieldID = jni::get_field_id(env, Class, "supportsTurn", "Z");
    supports_stun_fieldID = jni::get_field_id(env, Class, "supportsStun", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass CallState::Class;

object_ptr<CallState> CallState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case callStatePending::ID:
      return callStatePending::fetch(env, p);
    case callStateExchangingKeys::ID:
      return callStateExchangingKeys::fetch(env, p);
    case callStateReady::ID:
      return callStateReady::fetch(env, p);
    case callStateHangingUp::ID:
      return callStateHangingUp::fetch(env, p);
    case callStateDiscarded::ID:
      return callStateDiscarded::fetch(env, p);
    case callStateError::ID:
      return callStateError::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CallState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callStatePending::Class;
jfieldID callStatePending::is_created_fieldID;
jfieldID callStatePending::is_received_fieldID;

callStatePending::callStatePending()
  : is_created_()
  , is_received_()
{}

callStatePending::callStatePending(bool is_created_, bool is_received_)
  : is_created_(is_created_)
  , is_received_(is_received_)
{}

const std::int32_t callStatePending::ID;

object_ptr<CallState> callStatePending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callStatePending> res = make_object<callStatePending>();
  res->is_created_ = (env->GetBooleanField(p, res->is_created_fieldID) != 0);
  res->is_received_ = (env->GetBooleanField(p, res->is_received_fieldID) != 0);
  return std::move(res);
}

void callStatePending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_created_fieldID, is_created_);
  env->SetBooleanField(s, is_received_fieldID, is_received_);
}

void callStatePending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStatePending");
    s.store_field("isCreated", is_created_);
    s.store_field("isReceived", is_received_);
    s.store_class_end();
  }
}

void callStatePending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStatePending").c_str());
    is_created_fieldID = jni::get_field_id(env, Class, "isCreated", "Z");
    is_received_fieldID = jni::get_field_id(env, Class, "isReceived", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass callStateExchangingKeys::Class;

callStateExchangingKeys::callStateExchangingKeys() {
}

const std::int32_t callStateExchangingKeys::ID;

object_ptr<CallState> callStateExchangingKeys::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callStateExchangingKeys>();
}

void callStateExchangingKeys::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callStateExchangingKeys::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStateExchangingKeys");
    s.store_class_end();
  }
}

void callStateExchangingKeys::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStateExchangingKeys").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callStateReady::Class;
jfieldID callStateReady::protocol_fieldID;
jfieldID callStateReady::servers_fieldID;
jfieldID callStateReady::config_fieldID;
jfieldID callStateReady::encryption_key_fieldID;
jfieldID callStateReady::emojis_fieldID;
jfieldID callStateReady::allow_p2p_fieldID;
jfieldID callStateReady::is_group_call_supported_fieldID;
jfieldID callStateReady::custom_parameters_fieldID;

callStateReady::callStateReady()
  : protocol_()
  , servers_()
  , config_()
  , encryption_key_()
  , emojis_()
  , allow_p2p_()
  , is_group_call_supported_()
  , custom_parameters_()
{}

callStateReady::callStateReady(object_ptr<callProtocol> &&protocol_, array<object_ptr<callServer>> &&servers_, string const &config_, bytes const &encryption_key_, array<string> &&emojis_, bool allow_p2p_, bool is_group_call_supported_, string const &custom_parameters_)
  : protocol_(std::move(protocol_))
  , servers_(std::move(servers_))
  , config_(config_)
  , encryption_key_(std::move(encryption_key_))
  , emojis_(std::move(emojis_))
  , allow_p2p_(allow_p2p_)
  , is_group_call_supported_(is_group_call_supported_)
  , custom_parameters_(custom_parameters_)
{}

const std::int32_t callStateReady::ID;

object_ptr<CallState> callStateReady::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callStateReady> res = make_object<callStateReady>();
  res->protocol_ = jni::fetch_tl_object<callProtocol>(env, jni::fetch_object(env, p, res->protocol_fieldID));
  res->servers_ = jni::FetchVector<callServer>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->servers_fieldID));
  res->config_ = jni::fetch_string(env, p, res->config_fieldID);
  res->encryption_key_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->encryption_key_fieldID));
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  res->allow_p2p_ = (env->GetBooleanField(p, res->allow_p2p_fieldID) != 0);
  res->is_group_call_supported_ = (env->GetBooleanField(p, res->is_group_call_supported_fieldID) != 0);
  res->custom_parameters_ = jni::fetch_string(env, p, res->custom_parameters_fieldID);
  return std::move(res);
}

void callStateReady::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (protocol_ != nullptr) { jobject next; protocol_->store(env, next); if (next) { env->SetObjectField(s, protocol_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, servers_); if (arr_tmp_) { env->SetObjectField(s, servers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, config_); if (nextString) { env->SetObjectField(s, config_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jbyteArray nextBytes = jni::to_bytes(env, encryption_key_); if (nextBytes) { env->SetObjectField(s, encryption_key_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, allow_p2p_fieldID, allow_p2p_);
  env->SetBooleanField(s, is_group_call_supported_fieldID, is_group_call_supported_);
  { jstring nextString = jni::to_jstring(env, custom_parameters_); if (nextString) { env->SetObjectField(s, custom_parameters_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void callStateReady::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStateReady");
    s.store_object_field("protocol", static_cast<const BaseObject *>(protocol_.get()));
    { s.store_vector_begin("servers", servers_.size()); for (const auto &_value : servers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("config", config_);
    s.store_bytes_field("encryptionKey", encryption_key_);
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("allowP2p", allow_p2p_);
    s.store_field("isGroupCallSupported", is_group_call_supported_);
    s.store_field("customParameters", custom_parameters_);
    s.store_class_end();
  }
}

void callStateReady::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStateReady").c_str());
    protocol_fieldID = jni::get_field_id(env, Class, "protocol", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallProtocol;").c_str());
    servers_fieldID = jni::get_field_id(env, Class, "servers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$CallServer;").c_str());
    config_fieldID = jni::get_field_id(env, Class, "config", "Ljava/lang/String;");
    encryption_key_fieldID = jni::get_field_id(env, Class, "encryptionKey", "[B");
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    allow_p2p_fieldID = jni::get_field_id(env, Class, "allowP2p", "Z");
    is_group_call_supported_fieldID = jni::get_field_id(env, Class, "isGroupCallSupported", "Z");
    custom_parameters_fieldID = jni::get_field_id(env, Class, "customParameters", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass callStateHangingUp::Class;

callStateHangingUp::callStateHangingUp() {
}

const std::int32_t callStateHangingUp::ID;

object_ptr<CallState> callStateHangingUp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<callStateHangingUp>();
}

void callStateHangingUp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void callStateHangingUp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStateHangingUp");
    s.store_class_end();
  }
}

void callStateHangingUp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStateHangingUp").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callStateDiscarded::Class;
jfieldID callStateDiscarded::reason_fieldID;
jfieldID callStateDiscarded::need_rating_fieldID;
jfieldID callStateDiscarded::need_debug_information_fieldID;
jfieldID callStateDiscarded::need_log_fieldID;

callStateDiscarded::callStateDiscarded()
  : reason_()
  , need_rating_()
  , need_debug_information_()
  , need_log_()
{}

callStateDiscarded::callStateDiscarded(object_ptr<CallDiscardReason> &&reason_, bool need_rating_, bool need_debug_information_, bool need_log_)
  : reason_(std::move(reason_))
  , need_rating_(need_rating_)
  , need_debug_information_(need_debug_information_)
  , need_log_(need_log_)
{}

const std::int32_t callStateDiscarded::ID;

object_ptr<CallState> callStateDiscarded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callStateDiscarded> res = make_object<callStateDiscarded>();
  res->reason_ = jni::fetch_tl_object<CallDiscardReason>(env, jni::fetch_object(env, p, res->reason_fieldID));
  res->need_rating_ = (env->GetBooleanField(p, res->need_rating_fieldID) != 0);
  res->need_debug_information_ = (env->GetBooleanField(p, res->need_debug_information_fieldID) != 0);
  res->need_log_ = (env->GetBooleanField(p, res->need_log_fieldID) != 0);
  return std::move(res);
}

void callStateDiscarded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (reason_ != nullptr) { jobject next; reason_->store(env, next); if (next) { env->SetObjectField(s, reason_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, need_rating_fieldID, need_rating_);
  env->SetBooleanField(s, need_debug_information_fieldID, need_debug_information_);
  env->SetBooleanField(s, need_log_fieldID, need_log_);
}

void callStateDiscarded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStateDiscarded");
    s.store_object_field("reason", static_cast<const BaseObject *>(reason_.get()));
    s.store_field("needRating", need_rating_);
    s.store_field("needDebugInformation", need_debug_information_);
    s.store_field("needLog", need_log_);
    s.store_class_end();
  }
}

void callStateDiscarded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStateDiscarded").c_str());
    reason_fieldID = jni::get_field_id(env, Class, "reason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallDiscardReason;").c_str());
    need_rating_fieldID = jni::get_field_id(env, Class, "needRating", "Z");
    need_debug_information_fieldID = jni::get_field_id(env, Class, "needDebugInformation", "Z");
    need_log_fieldID = jni::get_field_id(env, Class, "needLog", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass callStateError::Class;
jfieldID callStateError::error_fieldID;

callStateError::callStateError()
  : error_()
{}

callStateError::callStateError(object_ptr<error> &&error_)
  : error_(std::move(error_))
{}

const std::int32_t callStateError::ID;

object_ptr<CallState> callStateError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callStateError> res = make_object<callStateError>();
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  return std::move(res);
}

void callStateError::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (error_ != nullptr) { jobject next; error_->store(env, next); if (next) { env->SetObjectField(s, error_fieldID, next); env->DeleteLocalRef(next); } }
}

void callStateError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallStateError");
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_class_end();
  }
}

void callStateError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallStateError").c_str());
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callbackQueryAnswer::Class;
jfieldID callbackQueryAnswer::text_fieldID;
jfieldID callbackQueryAnswer::show_alert_fieldID;
jfieldID callbackQueryAnswer::url_fieldID;

callbackQueryAnswer::callbackQueryAnswer()
  : text_()
  , show_alert_()
  , url_()
{}

callbackQueryAnswer::callbackQueryAnswer(string const &text_, bool show_alert_, string const &url_)
  : text_(text_)
  , show_alert_(show_alert_)
  , url_(url_)
{}

const std::int32_t callbackQueryAnswer::ID;

object_ptr<callbackQueryAnswer> callbackQueryAnswer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callbackQueryAnswer> res = make_object<callbackQueryAnswer>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->show_alert_ = (env->GetBooleanField(p, res->show_alert_fieldID) != 0);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void callbackQueryAnswer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, show_alert_fieldID, show_alert_);
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void callbackQueryAnswer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallbackQueryAnswer");
    s.store_field("text", text_);
    s.store_field("showAlert", show_alert_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void callbackQueryAnswer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallbackQueryAnswer").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    show_alert_fieldID = jni::get_field_id(env, Class, "showAlert", "Z");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass CallbackQueryPayload::Class;

object_ptr<CallbackQueryPayload> CallbackQueryPayload::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case callbackQueryPayloadData::ID:
      return callbackQueryPayloadData::fetch(env, p);
    case callbackQueryPayloadDataWithPassword::ID:
      return callbackQueryPayloadDataWithPassword::fetch(env, p);
    case callbackQueryPayloadGame::ID:
      return callbackQueryPayloadGame::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CallbackQueryPayload::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallbackQueryPayload").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass callbackQueryPayloadData::Class;
jfieldID callbackQueryPayloadData::data_fieldID;

callbackQueryPayloadData::callbackQueryPayloadData()
  : data_()
{}

callbackQueryPayloadData::callbackQueryPayloadData(bytes const &data_)
  : data_(std::move(data_))
{}

const std::int32_t callbackQueryPayloadData::ID;

object_ptr<CallbackQueryPayload> callbackQueryPayloadData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callbackQueryPayloadData> res = make_object<callbackQueryPayloadData>();
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return std::move(res);
}

void callbackQueryPayloadData::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void callbackQueryPayloadData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallbackQueryPayloadData");
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void callbackQueryPayloadData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallbackQueryPayloadData").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass callbackQueryPayloadDataWithPassword::Class;
jfieldID callbackQueryPayloadDataWithPassword::password_fieldID;
jfieldID callbackQueryPayloadDataWithPassword::data_fieldID;

callbackQueryPayloadDataWithPassword::callbackQueryPayloadDataWithPassword()
  : password_()
  , data_()
{}

callbackQueryPayloadDataWithPassword::callbackQueryPayloadDataWithPassword(string const &password_, bytes const &data_)
  : password_(password_)
  , data_(std::move(data_))
{}

const std::int32_t callbackQueryPayloadDataWithPassword::ID;

object_ptr<CallbackQueryPayload> callbackQueryPayloadDataWithPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callbackQueryPayloadDataWithPassword> res = make_object<callbackQueryPayloadDataWithPassword>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return std::move(res);
}

void callbackQueryPayloadDataWithPassword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, password_); if (nextString) { env->SetObjectField(s, password_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void callbackQueryPayloadDataWithPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallbackQueryPayloadDataWithPassword");
    s.store_field("password", password_);
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void callbackQueryPayloadDataWithPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallbackQueryPayloadDataWithPassword").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass callbackQueryPayloadGame::Class;
jfieldID callbackQueryPayloadGame::game_short_name_fieldID;

callbackQueryPayloadGame::callbackQueryPayloadGame()
  : game_short_name_()
{}

callbackQueryPayloadGame::callbackQueryPayloadGame(string const &game_short_name_)
  : game_short_name_(game_short_name_)
{}

const std::int32_t callbackQueryPayloadGame::ID;

object_ptr<CallbackQueryPayload> callbackQueryPayloadGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<callbackQueryPayloadGame> res = make_object<callbackQueryPayloadGame>();
  res->game_short_name_ = jni::fetch_string(env, p, res->game_short_name_fieldID);
  return std::move(res);
}

void callbackQueryPayloadGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, game_short_name_); if (nextString) { env->SetObjectField(s, game_short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void callbackQueryPayloadGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CallbackQueryPayloadGame");
    s.store_field("gameShortName", game_short_name_);
    s.store_class_end();
  }
}

void callbackQueryPayloadGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CallbackQueryPayloadGame").c_str());
    game_short_name_fieldID = jni::get_field_id(env, Class, "gameShortName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass CanPostStoryResult::Class;

object_ptr<CanPostStoryResult> CanPostStoryResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case canPostStoryResultOk::ID:
      return canPostStoryResultOk::fetch(env, p);
    case canPostStoryResultPremiumNeeded::ID:
      return canPostStoryResultPremiumNeeded::fetch(env, p);
    case canPostStoryResultBoostNeeded::ID:
      return canPostStoryResultBoostNeeded::fetch(env, p);
    case canPostStoryResultActiveStoryLimitExceeded::ID:
      return canPostStoryResultActiveStoryLimitExceeded::fetch(env, p);
    case canPostStoryResultWeeklyLimitExceeded::ID:
      return canPostStoryResultWeeklyLimitExceeded::fetch(env, p);
    case canPostStoryResultMonthlyLimitExceeded::ID:
      return canPostStoryResultMonthlyLimitExceeded::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CanPostStoryResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultOk::Class;
jfieldID canPostStoryResultOk::story_count_fieldID;

canPostStoryResultOk::canPostStoryResultOk()
  : story_count_()
{}

canPostStoryResultOk::canPostStoryResultOk(int32 story_count_)
  : story_count_(story_count_)
{}

const std::int32_t canPostStoryResultOk::ID;

object_ptr<CanPostStoryResult> canPostStoryResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canPostStoryResultOk> res = make_object<canPostStoryResultOk>();
  res->story_count_ = env->GetIntField(p, res->story_count_fieldID);
  return std::move(res);
}

void canPostStoryResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, story_count_fieldID, story_count_);
}

void canPostStoryResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultOk");
    s.store_field("storyCount", story_count_);
    s.store_class_end();
  }
}

void canPostStoryResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultOk").c_str());
    story_count_fieldID = jni::get_field_id(env, Class, "storyCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultPremiumNeeded::Class;

canPostStoryResultPremiumNeeded::canPostStoryResultPremiumNeeded() {
}

const std::int32_t canPostStoryResultPremiumNeeded::ID;

object_ptr<CanPostStoryResult> canPostStoryResultPremiumNeeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canPostStoryResultPremiumNeeded>();
}

void canPostStoryResultPremiumNeeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canPostStoryResultPremiumNeeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultPremiumNeeded");
    s.store_class_end();
  }
}

void canPostStoryResultPremiumNeeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultPremiumNeeded").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultBoostNeeded::Class;

canPostStoryResultBoostNeeded::canPostStoryResultBoostNeeded() {
}

const std::int32_t canPostStoryResultBoostNeeded::ID;

object_ptr<CanPostStoryResult> canPostStoryResultBoostNeeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canPostStoryResultBoostNeeded>();
}

void canPostStoryResultBoostNeeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canPostStoryResultBoostNeeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultBoostNeeded");
    s.store_class_end();
  }
}

void canPostStoryResultBoostNeeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultBoostNeeded").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultActiveStoryLimitExceeded::Class;

canPostStoryResultActiveStoryLimitExceeded::canPostStoryResultActiveStoryLimitExceeded() {
}

const std::int32_t canPostStoryResultActiveStoryLimitExceeded::ID;

object_ptr<CanPostStoryResult> canPostStoryResultActiveStoryLimitExceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canPostStoryResultActiveStoryLimitExceeded>();
}

void canPostStoryResultActiveStoryLimitExceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canPostStoryResultActiveStoryLimitExceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultActiveStoryLimitExceeded");
    s.store_class_end();
  }
}

void canPostStoryResultActiveStoryLimitExceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultActiveStoryLimitExceeded").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultWeeklyLimitExceeded::Class;
jfieldID canPostStoryResultWeeklyLimitExceeded::retry_after_fieldID;

canPostStoryResultWeeklyLimitExceeded::canPostStoryResultWeeklyLimitExceeded()
  : retry_after_()
{}

canPostStoryResultWeeklyLimitExceeded::canPostStoryResultWeeklyLimitExceeded(int32 retry_after_)
  : retry_after_(retry_after_)
{}

const std::int32_t canPostStoryResultWeeklyLimitExceeded::ID;

object_ptr<CanPostStoryResult> canPostStoryResultWeeklyLimitExceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canPostStoryResultWeeklyLimitExceeded> res = make_object<canPostStoryResultWeeklyLimitExceeded>();
  res->retry_after_ = env->GetIntField(p, res->retry_after_fieldID);
  return std::move(res);
}

void canPostStoryResultWeeklyLimitExceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, retry_after_fieldID, retry_after_);
}

void canPostStoryResultWeeklyLimitExceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultWeeklyLimitExceeded");
    s.store_field("retryAfter", retry_after_);
    s.store_class_end();
  }
}

void canPostStoryResultWeeklyLimitExceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultWeeklyLimitExceeded").c_str());
    retry_after_fieldID = jni::get_field_id(env, Class, "retryAfter", "I");
    return true;
  }();
  (void)is_inited;
}

jclass canPostStoryResultMonthlyLimitExceeded::Class;
jfieldID canPostStoryResultMonthlyLimitExceeded::retry_after_fieldID;

canPostStoryResultMonthlyLimitExceeded::canPostStoryResultMonthlyLimitExceeded()
  : retry_after_()
{}

canPostStoryResultMonthlyLimitExceeded::canPostStoryResultMonthlyLimitExceeded(int32 retry_after_)
  : retry_after_(retry_after_)
{}

const std::int32_t canPostStoryResultMonthlyLimitExceeded::ID;

object_ptr<CanPostStoryResult> canPostStoryResultMonthlyLimitExceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canPostStoryResultMonthlyLimitExceeded> res = make_object<canPostStoryResultMonthlyLimitExceeded>();
  res->retry_after_ = env->GetIntField(p, res->retry_after_fieldID);
  return std::move(res);
}

void canPostStoryResultMonthlyLimitExceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, retry_after_fieldID, retry_after_);
}

void canPostStoryResultMonthlyLimitExceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStoryResultMonthlyLimitExceeded");
    s.store_field("retryAfter", retry_after_);
    s.store_class_end();
  }
}

void canPostStoryResultMonthlyLimitExceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStoryResultMonthlyLimitExceeded").c_str());
    retry_after_fieldID = jni::get_field_id(env, Class, "retryAfter", "I");
    return true;
  }();
  (void)is_inited;
}

jclass CanSendMessageToUserResult::Class;

object_ptr<CanSendMessageToUserResult> CanSendMessageToUserResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case canSendMessageToUserResultOk::ID:
      return canSendMessageToUserResultOk::fetch(env, p);
    case canSendMessageToUserResultUserHasPaidMessages::ID:
      return canSendMessageToUserResultUserHasPaidMessages::fetch(env, p);
    case canSendMessageToUserResultUserIsDeleted::ID:
      return canSendMessageToUserResultUserIsDeleted::fetch(env, p);
    case canSendMessageToUserResultUserRestrictsNewChats::ID:
      return canSendMessageToUserResultUserRestrictsNewChats::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CanSendMessageToUserResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUserResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canSendMessageToUserResultOk::Class;

canSendMessageToUserResultOk::canSendMessageToUserResultOk() {
}

const std::int32_t canSendMessageToUserResultOk::ID;

object_ptr<CanSendMessageToUserResult> canSendMessageToUserResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canSendMessageToUserResultOk>();
}

void canSendMessageToUserResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canSendMessageToUserResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanSendMessageToUserResultOk");
    s.store_class_end();
  }
}

void canSendMessageToUserResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUserResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canSendMessageToUserResultUserHasPaidMessages::Class;
jfieldID canSendMessageToUserResultUserHasPaidMessages::outgoing_paid_message_star_count_fieldID;

canSendMessageToUserResultUserHasPaidMessages::canSendMessageToUserResultUserHasPaidMessages()
  : outgoing_paid_message_star_count_()
{}

canSendMessageToUserResultUserHasPaidMessages::canSendMessageToUserResultUserHasPaidMessages(int53 outgoing_paid_message_star_count_)
  : outgoing_paid_message_star_count_(outgoing_paid_message_star_count_)
{}

const std::int32_t canSendMessageToUserResultUserHasPaidMessages::ID;

object_ptr<CanSendMessageToUserResult> canSendMessageToUserResultUserHasPaidMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canSendMessageToUserResultUserHasPaidMessages> res = make_object<canSendMessageToUserResultUserHasPaidMessages>();
  res->outgoing_paid_message_star_count_ = env->GetLongField(p, res->outgoing_paid_message_star_count_fieldID);
  return std::move(res);
}

void canSendMessageToUserResultUserHasPaidMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, outgoing_paid_message_star_count_fieldID, outgoing_paid_message_star_count_);
}

void canSendMessageToUserResultUserHasPaidMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanSendMessageToUserResultUserHasPaidMessages");
    s.store_field("outgoingPaidMessageStarCount", outgoing_paid_message_star_count_);
    s.store_class_end();
  }
}

void canSendMessageToUserResultUserHasPaidMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUserResultUserHasPaidMessages").c_str());
    outgoing_paid_message_star_count_fieldID = jni::get_field_id(env, Class, "outgoingPaidMessageStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass canSendMessageToUserResultUserIsDeleted::Class;

canSendMessageToUserResultUserIsDeleted::canSendMessageToUserResultUserIsDeleted() {
}

const std::int32_t canSendMessageToUserResultUserIsDeleted::ID;

object_ptr<CanSendMessageToUserResult> canSendMessageToUserResultUserIsDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canSendMessageToUserResultUserIsDeleted>();
}

void canSendMessageToUserResultUserIsDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canSendMessageToUserResultUserIsDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanSendMessageToUserResultUserIsDeleted");
    s.store_class_end();
  }
}

void canSendMessageToUserResultUserIsDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUserResultUserIsDeleted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canSendMessageToUserResultUserRestrictsNewChats::Class;

canSendMessageToUserResultUserRestrictsNewChats::canSendMessageToUserResultUserRestrictsNewChats() {
}

const std::int32_t canSendMessageToUserResultUserRestrictsNewChats::ID;

object_ptr<CanSendMessageToUserResult> canSendMessageToUserResultUserRestrictsNewChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canSendMessageToUserResultUserRestrictsNewChats>();
}

void canSendMessageToUserResultUserRestrictsNewChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canSendMessageToUserResultUserRestrictsNewChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanSendMessageToUserResultUserRestrictsNewChats");
    s.store_class_end();
  }
}

void canSendMessageToUserResultUserRestrictsNewChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUserResultUserRestrictsNewChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass CanTransferOwnershipResult::Class;

object_ptr<CanTransferOwnershipResult> CanTransferOwnershipResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case canTransferOwnershipResultOk::ID:
      return canTransferOwnershipResultOk::fetch(env, p);
    case canTransferOwnershipResultPasswordNeeded::ID:
      return canTransferOwnershipResultPasswordNeeded::fetch(env, p);
    case canTransferOwnershipResultPasswordTooFresh::ID:
      return canTransferOwnershipResultPasswordTooFresh::fetch(env, p);
    case canTransferOwnershipResultSessionTooFresh::ID:
      return canTransferOwnershipResultSessionTooFresh::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CanTransferOwnershipResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnershipResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canTransferOwnershipResultOk::Class;

canTransferOwnershipResultOk::canTransferOwnershipResultOk() {
}

const std::int32_t canTransferOwnershipResultOk::ID;

object_ptr<CanTransferOwnershipResult> canTransferOwnershipResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canTransferOwnershipResultOk>();
}

void canTransferOwnershipResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canTransferOwnershipResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanTransferOwnershipResultOk");
    s.store_class_end();
  }
}

void canTransferOwnershipResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnershipResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canTransferOwnershipResultPasswordNeeded::Class;

canTransferOwnershipResultPasswordNeeded::canTransferOwnershipResultPasswordNeeded() {
}

const std::int32_t canTransferOwnershipResultPasswordNeeded::ID;

object_ptr<CanTransferOwnershipResult> canTransferOwnershipResultPasswordNeeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canTransferOwnershipResultPasswordNeeded>();
}

void canTransferOwnershipResultPasswordNeeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void canTransferOwnershipResultPasswordNeeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanTransferOwnershipResultPasswordNeeded");
    s.store_class_end();
  }
}

void canTransferOwnershipResultPasswordNeeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnershipResultPasswordNeeded").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canTransferOwnershipResultPasswordTooFresh::Class;
jfieldID canTransferOwnershipResultPasswordTooFresh::retry_after_fieldID;

canTransferOwnershipResultPasswordTooFresh::canTransferOwnershipResultPasswordTooFresh()
  : retry_after_()
{}

canTransferOwnershipResultPasswordTooFresh::canTransferOwnershipResultPasswordTooFresh(int32 retry_after_)
  : retry_after_(retry_after_)
{}

const std::int32_t canTransferOwnershipResultPasswordTooFresh::ID;

object_ptr<CanTransferOwnershipResult> canTransferOwnershipResultPasswordTooFresh::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canTransferOwnershipResultPasswordTooFresh> res = make_object<canTransferOwnershipResultPasswordTooFresh>();
  res->retry_after_ = env->GetIntField(p, res->retry_after_fieldID);
  return std::move(res);
}

void canTransferOwnershipResultPasswordTooFresh::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, retry_after_fieldID, retry_after_);
}

void canTransferOwnershipResultPasswordTooFresh::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanTransferOwnershipResultPasswordTooFresh");
    s.store_field("retryAfter", retry_after_);
    s.store_class_end();
  }
}

void canTransferOwnershipResultPasswordTooFresh::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnershipResultPasswordTooFresh").c_str());
    retry_after_fieldID = jni::get_field_id(env, Class, "retryAfter", "I");
    return true;
  }();
  (void)is_inited;
}

jclass canTransferOwnershipResultSessionTooFresh::Class;
jfieldID canTransferOwnershipResultSessionTooFresh::retry_after_fieldID;

canTransferOwnershipResultSessionTooFresh::canTransferOwnershipResultSessionTooFresh()
  : retry_after_()
{}

canTransferOwnershipResultSessionTooFresh::canTransferOwnershipResultSessionTooFresh(int32 retry_after_)
  : retry_after_(retry_after_)
{}

const std::int32_t canTransferOwnershipResultSessionTooFresh::ID;

object_ptr<CanTransferOwnershipResult> canTransferOwnershipResultSessionTooFresh::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canTransferOwnershipResultSessionTooFresh> res = make_object<canTransferOwnershipResultSessionTooFresh>();
  res->retry_after_ = env->GetIntField(p, res->retry_after_fieldID);
  return std::move(res);
}

void canTransferOwnershipResultSessionTooFresh::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, retry_after_fieldID, retry_after_);
}

void canTransferOwnershipResultSessionTooFresh::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanTransferOwnershipResultSessionTooFresh");
    s.store_field("retryAfter", retry_after_);
    s.store_class_end();
  }
}

void canTransferOwnershipResultSessionTooFresh::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnershipResultSessionTooFresh").c_str());
    retry_after_fieldID = jni::get_field_id(env, Class, "retryAfter", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chat::Class;
jfieldID chat::id_fieldID;
jfieldID chat::type_fieldID;
jfieldID chat::title_fieldID;
jfieldID chat::photo_fieldID;
jfieldID chat::accent_color_id_fieldID;
jfieldID chat::background_custom_emoji_id_fieldID;
jfieldID chat::profile_accent_color_id_fieldID;
jfieldID chat::profile_background_custom_emoji_id_fieldID;
jfieldID chat::permissions_fieldID;
jfieldID chat::last_message_fieldID;
jfieldID chat::positions_fieldID;
jfieldID chat::chat_lists_fieldID;
jfieldID chat::message_sender_id_fieldID;
jfieldID chat::block_list_fieldID;
jfieldID chat::has_protected_content_fieldID;
jfieldID chat::is_translatable_fieldID;
jfieldID chat::is_marked_as_unread_fieldID;
jfieldID chat::view_as_topics_fieldID;
jfieldID chat::has_scheduled_messages_fieldID;
jfieldID chat::can_be_deleted_only_for_self_fieldID;
jfieldID chat::can_be_deleted_for_all_users_fieldID;
jfieldID chat::can_be_reported_fieldID;
jfieldID chat::default_disable_notification_fieldID;
jfieldID chat::unread_count_fieldID;
jfieldID chat::last_read_inbox_message_id_fieldID;
jfieldID chat::last_read_outbox_message_id_fieldID;
jfieldID chat::unread_mention_count_fieldID;
jfieldID chat::unread_reaction_count_fieldID;
jfieldID chat::notification_settings_fieldID;
jfieldID chat::available_reactions_fieldID;
jfieldID chat::message_auto_delete_time_fieldID;
jfieldID chat::emoji_status_fieldID;
jfieldID chat::background_fieldID;
jfieldID chat::theme_name_fieldID;
jfieldID chat::action_bar_fieldID;
jfieldID chat::business_bot_manage_bar_fieldID;
jfieldID chat::video_chat_fieldID;
jfieldID chat::pending_join_requests_fieldID;
jfieldID chat::reply_markup_message_id_fieldID;
jfieldID chat::draft_message_fieldID;
jfieldID chat::client_data_fieldID;

chat::chat()
  : id_()
  , type_()
  , title_()
  , photo_()
  , accent_color_id_()
  , background_custom_emoji_id_()
  , profile_accent_color_id_()
  , profile_background_custom_emoji_id_()
  , permissions_()
  , last_message_()
  , positions_()
  , chat_lists_()
  , message_sender_id_()
  , block_list_()
  , has_protected_content_()
  , is_translatable_()
  , is_marked_as_unread_()
  , view_as_topics_()
  , has_scheduled_messages_()
  , can_be_deleted_only_for_self_()
  , can_be_deleted_for_all_users_()
  , can_be_reported_()
  , default_disable_notification_()
  , unread_count_()
  , last_read_inbox_message_id_()
  , last_read_outbox_message_id_()
  , unread_mention_count_()
  , unread_reaction_count_()
  , notification_settings_()
  , available_reactions_()
  , message_auto_delete_time_()
  , emoji_status_()
  , background_()
  , theme_name_()
  , action_bar_()
  , business_bot_manage_bar_()
  , video_chat_()
  , pending_join_requests_()
  , reply_markup_message_id_()
  , draft_message_()
  , client_data_()
{}

chat::chat(int53 id_, object_ptr<ChatType> &&type_, string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_, object_ptr<chatPermissions> &&permissions_, object_ptr<message> &&last_message_, array<object_ptr<chatPosition>> &&positions_, array<object_ptr<ChatList>> &&chat_lists_, object_ptr<MessageSender> &&message_sender_id_, object_ptr<BlockList> &&block_list_, bool has_protected_content_, bool is_translatable_, bool is_marked_as_unread_, bool view_as_topics_, bool has_scheduled_messages_, bool can_be_deleted_only_for_self_, bool can_be_deleted_for_all_users_, bool can_be_reported_, bool default_disable_notification_, int32 unread_count_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int32 unread_mention_count_, int32 unread_reaction_count_, object_ptr<chatNotificationSettings> &&notification_settings_, object_ptr<ChatAvailableReactions> &&available_reactions_, int32 message_auto_delete_time_, object_ptr<emojiStatus> &&emoji_status_, object_ptr<chatBackground> &&background_, string const &theme_name_, object_ptr<ChatActionBar> &&action_bar_, object_ptr<businessBotManageBar> &&business_bot_manage_bar_, object_ptr<videoChat> &&video_chat_, object_ptr<chatJoinRequestsInfo> &&pending_join_requests_, int53 reply_markup_message_id_, object_ptr<draftMessage> &&draft_message_, string const &client_data_)
  : id_(id_)
  , type_(std::move(type_))
  , title_(title_)
  , photo_(std::move(photo_))
  , accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
  , profile_accent_color_id_(profile_accent_color_id_)
  , profile_background_custom_emoji_id_(profile_background_custom_emoji_id_)
  , permissions_(std::move(permissions_))
  , last_message_(std::move(last_message_))
  , positions_(std::move(positions_))
  , chat_lists_(std::move(chat_lists_))
  , message_sender_id_(std::move(message_sender_id_))
  , block_list_(std::move(block_list_))
  , has_protected_content_(has_protected_content_)
  , is_translatable_(is_translatable_)
  , is_marked_as_unread_(is_marked_as_unread_)
  , view_as_topics_(view_as_topics_)
  , has_scheduled_messages_(has_scheduled_messages_)
  , can_be_deleted_only_for_self_(can_be_deleted_only_for_self_)
  , can_be_deleted_for_all_users_(can_be_deleted_for_all_users_)
  , can_be_reported_(can_be_reported_)
  , default_disable_notification_(default_disable_notification_)
  , unread_count_(unread_count_)
  , last_read_inbox_message_id_(last_read_inbox_message_id_)
  , last_read_outbox_message_id_(last_read_outbox_message_id_)
  , unread_mention_count_(unread_mention_count_)
  , unread_reaction_count_(unread_reaction_count_)
  , notification_settings_(std::move(notification_settings_))
  , available_reactions_(std::move(available_reactions_))
  , message_auto_delete_time_(message_auto_delete_time_)
  , emoji_status_(std::move(emoji_status_))
  , background_(std::move(background_))
  , theme_name_(theme_name_)
  , action_bar_(std::move(action_bar_))
  , business_bot_manage_bar_(std::move(business_bot_manage_bar_))
  , video_chat_(std::move(video_chat_))
  , pending_join_requests_(std::move(pending_join_requests_))
  , reply_markup_message_id_(reply_markup_message_id_)
  , draft_message_(std::move(draft_message_))
  , client_data_(client_data_)
{}

const std::int32_t chat::ID;

object_ptr<chat> chat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chat> res = make_object<chat>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->type_ = jni::fetch_tl_object<ChatType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->photo_ = jni::fetch_tl_object<chatPhotoInfo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  res->profile_accent_color_id_ = env->GetIntField(p, res->profile_accent_color_id_fieldID);
  res->profile_background_custom_emoji_id_ = env->GetLongField(p, res->profile_background_custom_emoji_id_fieldID);
  res->permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->permissions_fieldID));
  res->last_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->last_message_fieldID));
  res->positions_ = jni::FetchVector<chatPosition>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->positions_fieldID));
  res->chat_lists_ = jni::FetchVector<ChatList>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chat_lists_fieldID));
  res->message_sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->message_sender_id_fieldID));
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  res->has_protected_content_ = (env->GetBooleanField(p, res->has_protected_content_fieldID) != 0);
  res->is_translatable_ = (env->GetBooleanField(p, res->is_translatable_fieldID) != 0);
  res->is_marked_as_unread_ = (env->GetBooleanField(p, res->is_marked_as_unread_fieldID) != 0);
  res->view_as_topics_ = (env->GetBooleanField(p, res->view_as_topics_fieldID) != 0);
  res->has_scheduled_messages_ = (env->GetBooleanField(p, res->has_scheduled_messages_fieldID) != 0);
  res->can_be_deleted_only_for_self_ = (env->GetBooleanField(p, res->can_be_deleted_only_for_self_fieldID) != 0);
  res->can_be_deleted_for_all_users_ = (env->GetBooleanField(p, res->can_be_deleted_for_all_users_fieldID) != 0);
  res->can_be_reported_ = (env->GetBooleanField(p, res->can_be_reported_fieldID) != 0);
  res->default_disable_notification_ = (env->GetBooleanField(p, res->default_disable_notification_fieldID) != 0);
  res->unread_count_ = env->GetIntField(p, res->unread_count_fieldID);
  res->last_read_inbox_message_id_ = env->GetLongField(p, res->last_read_inbox_message_id_fieldID);
  res->last_read_outbox_message_id_ = env->GetLongField(p, res->last_read_outbox_message_id_fieldID);
  res->unread_mention_count_ = env->GetIntField(p, res->unread_mention_count_fieldID);
  res->unread_reaction_count_ = env->GetIntField(p, res->unread_reaction_count_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  res->available_reactions_ = jni::fetch_tl_object<ChatAvailableReactions>(env, jni::fetch_object(env, p, res->available_reactions_fieldID));
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  res->background_ = jni::fetch_tl_object<chatBackground>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  res->action_bar_ = jni::fetch_tl_object<ChatActionBar>(env, jni::fetch_object(env, p, res->action_bar_fieldID));
  res->business_bot_manage_bar_ = jni::fetch_tl_object<businessBotManageBar>(env, jni::fetch_object(env, p, res->business_bot_manage_bar_fieldID));
  res->video_chat_ = jni::fetch_tl_object<videoChat>(env, jni::fetch_object(env, p, res->video_chat_fieldID));
  res->pending_join_requests_ = jni::fetch_tl_object<chatJoinRequestsInfo>(env, jni::fetch_object(env, p, res->pending_join_requests_fieldID));
  res->reply_markup_message_id_ = env->GetLongField(p, res->reply_markup_message_id_fieldID);
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  res->client_data_ = jni::fetch_string(env, p, res->client_data_fieldID);
  return res;
}

void chat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  env->SetLongField(s, background_custom_emoji_id_fieldID, background_custom_emoji_id_);
  env->SetIntField(s, profile_accent_color_id_fieldID, profile_accent_color_id_);
  env->SetLongField(s, profile_background_custom_emoji_id_fieldID, profile_background_custom_emoji_id_);
  if (permissions_ != nullptr) { jobject next; permissions_->store(env, next); if (next) { env->SetObjectField(s, permissions_fieldID, next); env->DeleteLocalRef(next); } }
  if (last_message_ != nullptr) { jobject next; last_message_->store(env, next); if (next) { env->SetObjectField(s, last_message_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, positions_); if (arr_tmp_) { env->SetObjectField(s, positions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, chat_lists_); if (arr_tmp_) { env->SetObjectField(s, chat_lists_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (message_sender_id_ != nullptr) { jobject next; message_sender_id_->store(env, next); if (next) { env->SetObjectField(s, message_sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (block_list_ != nullptr) { jobject next; block_list_->store(env, next); if (next) { env->SetObjectField(s, block_list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_protected_content_fieldID, has_protected_content_);
  env->SetBooleanField(s, is_translatable_fieldID, is_translatable_);
  env->SetBooleanField(s, is_marked_as_unread_fieldID, is_marked_as_unread_);
  env->SetBooleanField(s, view_as_topics_fieldID, view_as_topics_);
  env->SetBooleanField(s, has_scheduled_messages_fieldID, has_scheduled_messages_);
  env->SetBooleanField(s, can_be_deleted_only_for_self_fieldID, can_be_deleted_only_for_self_);
  env->SetBooleanField(s, can_be_deleted_for_all_users_fieldID, can_be_deleted_for_all_users_);
  env->SetBooleanField(s, can_be_reported_fieldID, can_be_reported_);
  env->SetBooleanField(s, default_disable_notification_fieldID, default_disable_notification_);
  env->SetIntField(s, unread_count_fieldID, unread_count_);
  env->SetLongField(s, last_read_inbox_message_id_fieldID, last_read_inbox_message_id_);
  env->SetLongField(s, last_read_outbox_message_id_fieldID, last_read_outbox_message_id_);
  env->SetIntField(s, unread_mention_count_fieldID, unread_mention_count_);
  env->SetIntField(s, unread_reaction_count_fieldID, unread_reaction_count_);
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (available_reactions_ != nullptr) { jobject next; available_reactions_->store(env, next); if (next) { env->SetObjectField(s, available_reactions_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, message_auto_delete_time_fieldID, message_auto_delete_time_);
  if (emoji_status_ != nullptr) { jobject next; emoji_status_->store(env, next); if (next) { env->SetObjectField(s, emoji_status_fieldID, next); env->DeleteLocalRef(next); } }
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (action_bar_ != nullptr) { jobject next; action_bar_->store(env, next); if (next) { env->SetObjectField(s, action_bar_fieldID, next); env->DeleteLocalRef(next); } }
  if (business_bot_manage_bar_ != nullptr) { jobject next; business_bot_manage_bar_->store(env, next); if (next) { env->SetObjectField(s, business_bot_manage_bar_fieldID, next); env->DeleteLocalRef(next); } }
  if (video_chat_ != nullptr) { jobject next; video_chat_->store(env, next); if (next) { env->SetObjectField(s, video_chat_fieldID, next); env->DeleteLocalRef(next); } }
  if (pending_join_requests_ != nullptr) { jobject next; pending_join_requests_->store(env, next); if (next) { env->SetObjectField(s, pending_join_requests_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, reply_markup_message_id_fieldID, reply_markup_message_id_);
  if (draft_message_ != nullptr) { jobject next; draft_message_->store(env, next); if (next) { env->SetObjectField(s, draft_message_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, client_data_); if (nextString) { env->SetObjectField(s, client_data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Chat");
    s.store_field("id", id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("title", title_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_field("profileAccentColorId", profile_accent_color_id_);
    s.store_field("profileBackgroundCustomEmojiId", profile_background_custom_emoji_id_);
    s.store_object_field("permissions", static_cast<const BaseObject *>(permissions_.get()));
    s.store_object_field("lastMessage", static_cast<const BaseObject *>(last_message_.get()));
    { s.store_vector_begin("positions", positions_.size()); for (const auto &_value : positions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("chatLists", chat_lists_.size()); for (const auto &_value : chat_lists_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("messageSenderId", static_cast<const BaseObject *>(message_sender_id_.get()));
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_field("hasProtectedContent", has_protected_content_);
    s.store_field("isTranslatable", is_translatable_);
    s.store_field("isMarkedAsUnread", is_marked_as_unread_);
    s.store_field("viewAsTopics", view_as_topics_);
    s.store_field("hasScheduledMessages", has_scheduled_messages_);
    s.store_field("canBeDeletedOnlyForSelf", can_be_deleted_only_for_self_);
    s.store_field("canBeDeletedForAllUsers", can_be_deleted_for_all_users_);
    s.store_field("canBeReported", can_be_reported_);
    s.store_field("defaultDisableNotification", default_disable_notification_);
    s.store_field("unreadCount", unread_count_);
    s.store_field("lastReadInboxMessageId", last_read_inbox_message_id_);
    s.store_field("lastReadOutboxMessageId", last_read_outbox_message_id_);
    s.store_field("unreadMentionCount", unread_mention_count_);
    s.store_field("unreadReactionCount", unread_reaction_count_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_object_field("availableReactions", static_cast<const BaseObject *>(available_reactions_.get()));
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_field("themeName", theme_name_);
    s.store_object_field("actionBar", static_cast<const BaseObject *>(action_bar_.get()));
    s.store_object_field("businessBotManageBar", static_cast<const BaseObject *>(business_bot_manage_bar_.get()));
    s.store_object_field("videoChat", static_cast<const BaseObject *>(video_chat_.get()));
    s.store_object_field("pendingJoinRequests", static_cast<const BaseObject *>(pending_join_requests_.get()));
    s.store_field("replyMarkupMessageId", reply_markup_message_id_);
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    s.store_field("clientData", client_data_);
    s.store_class_end();
  }
}

void chat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Chat").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatType;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoInfo;").c_str());
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "profileAccentColorId", "I");
    profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "profileBackgroundCustomEmojiId", "J");
    permissions_fieldID = jni::get_field_id(env, Class, "permissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    last_message_fieldID = jni::get_field_id(env, Class, "lastMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    positions_fieldID = jni::get_field_id(env, Class, "positions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatPosition;").c_str());
    chat_lists_fieldID = jni::get_field_id(env, Class, "chatLists", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    message_sender_id_fieldID = jni::get_field_id(env, Class, "messageSenderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    has_protected_content_fieldID = jni::get_field_id(env, Class, "hasProtectedContent", "Z");
    is_translatable_fieldID = jni::get_field_id(env, Class, "isTranslatable", "Z");
    is_marked_as_unread_fieldID = jni::get_field_id(env, Class, "isMarkedAsUnread", "Z");
    view_as_topics_fieldID = jni::get_field_id(env, Class, "viewAsTopics", "Z");
    has_scheduled_messages_fieldID = jni::get_field_id(env, Class, "hasScheduledMessages", "Z");
    can_be_deleted_only_for_self_fieldID = jni::get_field_id(env, Class, "canBeDeletedOnlyForSelf", "Z");
    can_be_deleted_for_all_users_fieldID = jni::get_field_id(env, Class, "canBeDeletedForAllUsers", "Z");
    can_be_reported_fieldID = jni::get_field_id(env, Class, "canBeReported", "Z");
    default_disable_notification_fieldID = jni::get_field_id(env, Class, "defaultDisableNotification", "Z");
    unread_count_fieldID = jni::get_field_id(env, Class, "unreadCount", "I");
    last_read_inbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadInboxMessageId", "J");
    last_read_outbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadOutboxMessageId", "J");
    unread_mention_count_fieldID = jni::get_field_id(env, Class, "unreadMentionCount", "I");
    unread_reaction_count_fieldID = jni::get_field_id(env, Class, "unreadReactionCount", "I");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    available_reactions_fieldID = jni::get_field_id(env, Class, "availableReactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAvailableReactions;").c_str());
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBackground;").c_str());
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    action_bar_fieldID = jni::get_field_id(env, Class, "actionBar", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatActionBar;").c_str());
    business_bot_manage_bar_fieldID = jni::get_field_id(env, Class, "businessBotManageBar", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessBotManageBar;").c_str());
    video_chat_fieldID = jni::get_field_id(env, Class, "videoChat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VideoChat;").c_str());
    pending_join_requests_fieldID = jni::get_field_id(env, Class, "pendingJoinRequests", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatJoinRequestsInfo;").c_str());
    reply_markup_message_id_fieldID = jni::get_field_id(env, Class, "replyMarkupMessageId", "J");
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    client_data_fieldID = jni::get_field_id(env, Class, "clientData", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass ChatAction::Class;

object_ptr<ChatAction> ChatAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatActionTyping::ID:
      return chatActionTyping::fetch(env, p);
    case chatActionRecordingVideo::ID:
      return chatActionRecordingVideo::fetch(env, p);
    case chatActionUploadingVideo::ID:
      return chatActionUploadingVideo::fetch(env, p);
    case chatActionRecordingVoiceNote::ID:
      return chatActionRecordingVoiceNote::fetch(env, p);
    case chatActionUploadingVoiceNote::ID:
      return chatActionUploadingVoiceNote::fetch(env, p);
    case chatActionUploadingPhoto::ID:
      return chatActionUploadingPhoto::fetch(env, p);
    case chatActionUploadingDocument::ID:
      return chatActionUploadingDocument::fetch(env, p);
    case chatActionChoosingSticker::ID:
      return chatActionChoosingSticker::fetch(env, p);
    case chatActionChoosingLocation::ID:
      return chatActionChoosingLocation::fetch(env, p);
    case chatActionChoosingContact::ID:
      return chatActionChoosingContact::fetch(env, p);
    case chatActionStartPlayingGame::ID:
      return chatActionStartPlayingGame::fetch(env, p);
    case chatActionRecordingVideoNote::ID:
      return chatActionRecordingVideoNote::fetch(env, p);
    case chatActionUploadingVideoNote::ID:
      return chatActionUploadingVideoNote::fetch(env, p);
    case chatActionWatchingAnimations::ID:
      return chatActionWatchingAnimations::fetch(env, p);
    case chatActionCancel::ID:
      return chatActionCancel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAction").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionTyping::Class;

chatActionTyping::chatActionTyping() {
}

const std::int32_t chatActionTyping::ID;

object_ptr<ChatAction> chatActionTyping::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionTyping>();
}

void chatActionTyping::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionTyping::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionTyping");
    s.store_class_end();
  }
}

void chatActionTyping::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionTyping").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionRecordingVideo::Class;

chatActionRecordingVideo::chatActionRecordingVideo() {
}

const std::int32_t chatActionRecordingVideo::ID;

object_ptr<ChatAction> chatActionRecordingVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionRecordingVideo>();
}

void chatActionRecordingVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionRecordingVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionRecordingVideo");
    s.store_class_end();
  }
}

void chatActionRecordingVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionRecordingVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionUploadingVideo::Class;
jfieldID chatActionUploadingVideo::progress_fieldID;

chatActionUploadingVideo::chatActionUploadingVideo()
  : progress_()
{}

chatActionUploadingVideo::chatActionUploadingVideo(int32 progress_)
  : progress_(progress_)
{}

const std::int32_t chatActionUploadingVideo::ID;

object_ptr<ChatAction> chatActionUploadingVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionUploadingVideo> res = make_object<chatActionUploadingVideo>();
  res->progress_ = env->GetIntField(p, res->progress_fieldID);
  return std::move(res);
}

void chatActionUploadingVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, progress_fieldID, progress_);
}

void chatActionUploadingVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionUploadingVideo");
    s.store_field("progress", progress_);
    s.store_class_end();
  }
}

void chatActionUploadingVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionUploadingVideo").c_str());
    progress_fieldID = jni::get_field_id(env, Class, "progress", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionRecordingVoiceNote::Class;

chatActionRecordingVoiceNote::chatActionRecordingVoiceNote() {
}

const std::int32_t chatActionRecordingVoiceNote::ID;

object_ptr<ChatAction> chatActionRecordingVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionRecordingVoiceNote>();
}

void chatActionRecordingVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionRecordingVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionRecordingVoiceNote");
    s.store_class_end();
  }
}

void chatActionRecordingVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionRecordingVoiceNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionUploadingVoiceNote::Class;
jfieldID chatActionUploadingVoiceNote::progress_fieldID;

chatActionUploadingVoiceNote::chatActionUploadingVoiceNote()
  : progress_()
{}

chatActionUploadingVoiceNote::chatActionUploadingVoiceNote(int32 progress_)
  : progress_(progress_)
{}

const std::int32_t chatActionUploadingVoiceNote::ID;

object_ptr<ChatAction> chatActionUploadingVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionUploadingVoiceNote> res = make_object<chatActionUploadingVoiceNote>();
  res->progress_ = env->GetIntField(p, res->progress_fieldID);
  return std::move(res);
}

void chatActionUploadingVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, progress_fieldID, progress_);
}

void chatActionUploadingVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionUploadingVoiceNote");
    s.store_field("progress", progress_);
    s.store_class_end();
  }
}

void chatActionUploadingVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionUploadingVoiceNote").c_str());
    progress_fieldID = jni::get_field_id(env, Class, "progress", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionUploadingPhoto::Class;
jfieldID chatActionUploadingPhoto::progress_fieldID;

chatActionUploadingPhoto::chatActionUploadingPhoto()
  : progress_()
{}

chatActionUploadingPhoto::chatActionUploadingPhoto(int32 progress_)
  : progress_(progress_)
{}

const std::int32_t chatActionUploadingPhoto::ID;

object_ptr<ChatAction> chatActionUploadingPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionUploadingPhoto> res = make_object<chatActionUploadingPhoto>();
  res->progress_ = env->GetIntField(p, res->progress_fieldID);
  return std::move(res);
}

void chatActionUploadingPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, progress_fieldID, progress_);
}

void chatActionUploadingPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionUploadingPhoto");
    s.store_field("progress", progress_);
    s.store_class_end();
  }
}

void chatActionUploadingPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionUploadingPhoto").c_str());
    progress_fieldID = jni::get_field_id(env, Class, "progress", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionUploadingDocument::Class;
jfieldID chatActionUploadingDocument::progress_fieldID;

chatActionUploadingDocument::chatActionUploadingDocument()
  : progress_()
{}

chatActionUploadingDocument::chatActionUploadingDocument(int32 progress_)
  : progress_(progress_)
{}

const std::int32_t chatActionUploadingDocument::ID;

object_ptr<ChatAction> chatActionUploadingDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionUploadingDocument> res = make_object<chatActionUploadingDocument>();
  res->progress_ = env->GetIntField(p, res->progress_fieldID);
  return std::move(res);
}

void chatActionUploadingDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, progress_fieldID, progress_);
}

void chatActionUploadingDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionUploadingDocument");
    s.store_field("progress", progress_);
    s.store_class_end();
  }
}

void chatActionUploadingDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionUploadingDocument").c_str());
    progress_fieldID = jni::get_field_id(env, Class, "progress", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionChoosingSticker::Class;

chatActionChoosingSticker::chatActionChoosingSticker() {
}

const std::int32_t chatActionChoosingSticker::ID;

object_ptr<ChatAction> chatActionChoosingSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionChoosingSticker>();
}

void chatActionChoosingSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionChoosingSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionChoosingSticker");
    s.store_class_end();
  }
}

void chatActionChoosingSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionChoosingSticker").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionChoosingLocation::Class;

chatActionChoosingLocation::chatActionChoosingLocation() {
}

const std::int32_t chatActionChoosingLocation::ID;

object_ptr<ChatAction> chatActionChoosingLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionChoosingLocation>();
}

void chatActionChoosingLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionChoosingLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionChoosingLocation");
    s.store_class_end();
  }
}

void chatActionChoosingLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionChoosingLocation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionChoosingContact::Class;

chatActionChoosingContact::chatActionChoosingContact() {
}

const std::int32_t chatActionChoosingContact::ID;

object_ptr<ChatAction> chatActionChoosingContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionChoosingContact>();
}

void chatActionChoosingContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionChoosingContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionChoosingContact");
    s.store_class_end();
  }
}

void chatActionChoosingContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionChoosingContact").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionStartPlayingGame::Class;

chatActionStartPlayingGame::chatActionStartPlayingGame() {
}

const std::int32_t chatActionStartPlayingGame::ID;

object_ptr<ChatAction> chatActionStartPlayingGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionStartPlayingGame>();
}

void chatActionStartPlayingGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionStartPlayingGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionStartPlayingGame");
    s.store_class_end();
  }
}

void chatActionStartPlayingGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionStartPlayingGame").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionRecordingVideoNote::Class;

chatActionRecordingVideoNote::chatActionRecordingVideoNote() {
}

const std::int32_t chatActionRecordingVideoNote::ID;

object_ptr<ChatAction> chatActionRecordingVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionRecordingVideoNote>();
}

void chatActionRecordingVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionRecordingVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionRecordingVideoNote");
    s.store_class_end();
  }
}

void chatActionRecordingVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionRecordingVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionUploadingVideoNote::Class;
jfieldID chatActionUploadingVideoNote::progress_fieldID;

chatActionUploadingVideoNote::chatActionUploadingVideoNote()
  : progress_()
{}

chatActionUploadingVideoNote::chatActionUploadingVideoNote(int32 progress_)
  : progress_(progress_)
{}

const std::int32_t chatActionUploadingVideoNote::ID;

object_ptr<ChatAction> chatActionUploadingVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionUploadingVideoNote> res = make_object<chatActionUploadingVideoNote>();
  res->progress_ = env->GetIntField(p, res->progress_fieldID);
  return std::move(res);
}

void chatActionUploadingVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, progress_fieldID, progress_);
}

void chatActionUploadingVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionUploadingVideoNote");
    s.store_field("progress", progress_);
    s.store_class_end();
  }
}

void chatActionUploadingVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionUploadingVideoNote").c_str());
    progress_fieldID = jni::get_field_id(env, Class, "progress", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionWatchingAnimations::Class;
jfieldID chatActionWatchingAnimations::emoji_fieldID;

chatActionWatchingAnimations::chatActionWatchingAnimations()
  : emoji_()
{}

chatActionWatchingAnimations::chatActionWatchingAnimations(string const &emoji_)
  : emoji_(emoji_)
{}

const std::int32_t chatActionWatchingAnimations::ID;

object_ptr<ChatAction> chatActionWatchingAnimations::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionWatchingAnimations> res = make_object<chatActionWatchingAnimations>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return std::move(res);
}

void chatActionWatchingAnimations::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatActionWatchingAnimations::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionWatchingAnimations");
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

void chatActionWatchingAnimations::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionWatchingAnimations").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionCancel::Class;

chatActionCancel::chatActionCancel() {
}

const std::int32_t chatActionCancel::ID;

object_ptr<ChatAction> chatActionCancel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionCancel>();
}

void chatActionCancel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionCancel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionCancel");
    s.store_class_end();
  }
}

void chatActionCancel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionCancel").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatActionBar::Class;

object_ptr<ChatActionBar> ChatActionBar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatActionBarReportSpam::ID:
      return chatActionBarReportSpam::fetch(env, p);
    case chatActionBarInviteMembers::ID:
      return chatActionBarInviteMembers::fetch(env, p);
    case chatActionBarReportAddBlock::ID:
      return chatActionBarReportAddBlock::fetch(env, p);
    case chatActionBarAddContact::ID:
      return chatActionBarAddContact::fetch(env, p);
    case chatActionBarSharePhoneNumber::ID:
      return chatActionBarSharePhoneNumber::fetch(env, p);
    case chatActionBarJoinRequest::ID:
      return chatActionBarJoinRequest::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatActionBar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBar").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarReportSpam::Class;
jfieldID chatActionBarReportSpam::can_unarchive_fieldID;

chatActionBarReportSpam::chatActionBarReportSpam()
  : can_unarchive_()
{}

chatActionBarReportSpam::chatActionBarReportSpam(bool can_unarchive_)
  : can_unarchive_(can_unarchive_)
{}

const std::int32_t chatActionBarReportSpam::ID;

object_ptr<ChatActionBar> chatActionBarReportSpam::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionBarReportSpam> res = make_object<chatActionBarReportSpam>();
  res->can_unarchive_ = (env->GetBooleanField(p, res->can_unarchive_fieldID) != 0);
  return std::move(res);
}

void chatActionBarReportSpam::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_unarchive_fieldID, can_unarchive_);
}

void chatActionBarReportSpam::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarReportSpam");
    s.store_field("canUnarchive", can_unarchive_);
    s.store_class_end();
  }
}

void chatActionBarReportSpam::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarReportSpam").c_str());
    can_unarchive_fieldID = jni::get_field_id(env, Class, "canUnarchive", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarInviteMembers::Class;

chatActionBarInviteMembers::chatActionBarInviteMembers() {
}

const std::int32_t chatActionBarInviteMembers::ID;

object_ptr<ChatActionBar> chatActionBarInviteMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionBarInviteMembers>();
}

void chatActionBarInviteMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionBarInviteMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarInviteMembers");
    s.store_class_end();
  }
}

void chatActionBarInviteMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarInviteMembers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarReportAddBlock::Class;
jfieldID chatActionBarReportAddBlock::can_unarchive_fieldID;
jfieldID chatActionBarReportAddBlock::account_info_fieldID;

chatActionBarReportAddBlock::chatActionBarReportAddBlock()
  : can_unarchive_()
  , account_info_()
{}

chatActionBarReportAddBlock::chatActionBarReportAddBlock(bool can_unarchive_, object_ptr<accountInfo> &&account_info_)
  : can_unarchive_(can_unarchive_)
  , account_info_(std::move(account_info_))
{}

const std::int32_t chatActionBarReportAddBlock::ID;

object_ptr<ChatActionBar> chatActionBarReportAddBlock::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionBarReportAddBlock> res = make_object<chatActionBarReportAddBlock>();
  res->can_unarchive_ = (env->GetBooleanField(p, res->can_unarchive_fieldID) != 0);
  res->account_info_ = jni::fetch_tl_object<accountInfo>(env, jni::fetch_object(env, p, res->account_info_fieldID));
  return std::move(res);
}

void chatActionBarReportAddBlock::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_unarchive_fieldID, can_unarchive_);
  if (account_info_ != nullptr) { jobject next; account_info_->store(env, next); if (next) { env->SetObjectField(s, account_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatActionBarReportAddBlock::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarReportAddBlock");
    s.store_field("canUnarchive", can_unarchive_);
    s.store_object_field("accountInfo", static_cast<const BaseObject *>(account_info_.get()));
    s.store_class_end();
  }
}

void chatActionBarReportAddBlock::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarReportAddBlock").c_str());
    can_unarchive_fieldID = jni::get_field_id(env, Class, "canUnarchive", "Z");
    account_info_fieldID = jni::get_field_id(env, Class, "accountInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AccountInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarAddContact::Class;

chatActionBarAddContact::chatActionBarAddContact() {
}

const std::int32_t chatActionBarAddContact::ID;

object_ptr<ChatActionBar> chatActionBarAddContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionBarAddContact>();
}

void chatActionBarAddContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionBarAddContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarAddContact");
    s.store_class_end();
  }
}

void chatActionBarAddContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarAddContact").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarSharePhoneNumber::Class;

chatActionBarSharePhoneNumber::chatActionBarSharePhoneNumber() {
}

const std::int32_t chatActionBarSharePhoneNumber::ID;

object_ptr<ChatActionBar> chatActionBarSharePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatActionBarSharePhoneNumber>();
}

void chatActionBarSharePhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatActionBarSharePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarSharePhoneNumber");
    s.store_class_end();
  }
}

void chatActionBarSharePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarSharePhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatActionBarJoinRequest::Class;
jfieldID chatActionBarJoinRequest::title_fieldID;
jfieldID chatActionBarJoinRequest::is_channel_fieldID;
jfieldID chatActionBarJoinRequest::request_date_fieldID;

chatActionBarJoinRequest::chatActionBarJoinRequest()
  : title_()
  , is_channel_()
  , request_date_()
{}

chatActionBarJoinRequest::chatActionBarJoinRequest(string const &title_, bool is_channel_, int32 request_date_)
  : title_(title_)
  , is_channel_(is_channel_)
  , request_date_(request_date_)
{}

const std::int32_t chatActionBarJoinRequest::ID;

object_ptr<ChatActionBar> chatActionBarJoinRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActionBarJoinRequest> res = make_object<chatActionBarJoinRequest>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  res->request_date_ = env->GetIntField(p, res->request_date_fieldID);
  return std::move(res);
}

void chatActionBarJoinRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_channel_fieldID, is_channel_);
  env->SetIntField(s, request_date_fieldID, request_date_);
}

void chatActionBarJoinRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActionBarJoinRequest");
    s.store_field("title", title_);
    s.store_field("isChannel", is_channel_);
    s.store_field("requestDate", request_date_);
    s.store_class_end();
  }
}

void chatActionBarJoinRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActionBarJoinRequest").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    request_date_fieldID = jni::get_field_id(env, Class, "requestDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatActiveStories::Class;
jfieldID chatActiveStories::chat_id_fieldID;
jfieldID chatActiveStories::list_fieldID;
jfieldID chatActiveStories::order_fieldID;
jfieldID chatActiveStories::max_read_story_id_fieldID;
jfieldID chatActiveStories::stories_fieldID;

chatActiveStories::chatActiveStories()
  : chat_id_()
  , list_()
  , order_()
  , max_read_story_id_()
  , stories_()
{}

chatActiveStories::chatActiveStories(int53 chat_id_, object_ptr<StoryList> &&list_, int53 order_, int32 max_read_story_id_, array<object_ptr<storyInfo>> &&stories_)
  : chat_id_(chat_id_)
  , list_(std::move(list_))
  , order_(order_)
  , max_read_story_id_(max_read_story_id_)
  , stories_(std::move(stories_))
{}

const std::int32_t chatActiveStories::ID;

object_ptr<chatActiveStories> chatActiveStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatActiveStories> res = make_object<chatActiveStories>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->list_ = jni::fetch_tl_object<StoryList>(env, jni::fetch_object(env, p, res->list_fieldID));
  res->order_ = env->GetLongField(p, res->order_fieldID);
  res->max_read_story_id_ = env->GetIntField(p, res->max_read_story_id_fieldID);
  res->stories_ = jni::FetchVector<storyInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stories_fieldID));
  return res;
}

void chatActiveStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (list_ != nullptr) { jobject next; list_->store(env, next); if (next) { env->SetObjectField(s, list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, order_fieldID, order_);
  env->SetIntField(s, max_read_story_id_fieldID, max_read_story_id_);
  { auto arr_tmp_ = jni::store_vector(env, stories_); if (arr_tmp_) { env->SetObjectField(s, stories_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatActiveStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatActiveStories");
    s.store_field("chatId", chat_id_);
    s.store_object_field("list", static_cast<const BaseObject *>(list_.get()));
    s.store_field("order", order_);
    s.store_field("maxReadStoryId", max_read_story_id_);
    { s.store_vector_begin("stories", stories_.size()); for (const auto &_value : stories_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatActiveStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatActiveStories").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    list_fieldID = jni::get_field_id(env, Class, "list", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryList;").c_str());
    order_fieldID = jni::get_field_id(env, Class, "order", "J");
    max_read_story_id_fieldID = jni::get_field_id(env, Class, "maxReadStoryId", "I");
    stories_fieldID = jni::get_field_id(env, Class, "stories", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StoryInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatAdministrator::Class;
jfieldID chatAdministrator::user_id_fieldID;
jfieldID chatAdministrator::custom_title_fieldID;
jfieldID chatAdministrator::is_owner_fieldID;

chatAdministrator::chatAdministrator()
  : user_id_()
  , custom_title_()
  , is_owner_()
{}

chatAdministrator::chatAdministrator(int53 user_id_, string const &custom_title_, bool is_owner_)
  : user_id_(user_id_)
  , custom_title_(custom_title_)
  , is_owner_(is_owner_)
{}

const std::int32_t chatAdministrator::ID;

object_ptr<chatAdministrator> chatAdministrator::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatAdministrator> res = make_object<chatAdministrator>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->custom_title_ = jni::fetch_string(env, p, res->custom_title_fieldID);
  res->is_owner_ = (env->GetBooleanField(p, res->is_owner_fieldID) != 0);
  return res;
}

void chatAdministrator::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { jstring nextString = jni::to_jstring(env, custom_title_); if (nextString) { env->SetObjectField(s, custom_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_owner_fieldID, is_owner_);
}

void chatAdministrator::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatAdministrator");
    s.store_field("userId", user_id_);
    s.store_field("customTitle", custom_title_);
    s.store_field("isOwner", is_owner_);
    s.store_class_end();
  }
}

void chatAdministrator::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAdministrator").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    custom_title_fieldID = jni::get_field_id(env, Class, "customTitle", "Ljava/lang/String;");
    is_owner_fieldID = jni::get_field_id(env, Class, "isOwner", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatAdministratorRights::Class;
jfieldID chatAdministratorRights::can_manage_chat_fieldID;
jfieldID chatAdministratorRights::can_change_info_fieldID;
jfieldID chatAdministratorRights::can_post_messages_fieldID;
jfieldID chatAdministratorRights::can_edit_messages_fieldID;
jfieldID chatAdministratorRights::can_delete_messages_fieldID;
jfieldID chatAdministratorRights::can_invite_users_fieldID;
jfieldID chatAdministratorRights::can_restrict_members_fieldID;
jfieldID chatAdministratorRights::can_pin_messages_fieldID;
jfieldID chatAdministratorRights::can_manage_topics_fieldID;
jfieldID chatAdministratorRights::can_promote_members_fieldID;
jfieldID chatAdministratorRights::can_manage_video_chats_fieldID;
jfieldID chatAdministratorRights::can_post_stories_fieldID;
jfieldID chatAdministratorRights::can_edit_stories_fieldID;
jfieldID chatAdministratorRights::can_delete_stories_fieldID;
jfieldID chatAdministratorRights::is_anonymous_fieldID;

chatAdministratorRights::chatAdministratorRights()
  : can_manage_chat_()
  , can_change_info_()
  , can_post_messages_()
  , can_edit_messages_()
  , can_delete_messages_()
  , can_invite_users_()
  , can_restrict_members_()
  , can_pin_messages_()
  , can_manage_topics_()
  , can_promote_members_()
  , can_manage_video_chats_()
  , can_post_stories_()
  , can_edit_stories_()
  , can_delete_stories_()
  , is_anonymous_()
{}

chatAdministratorRights::chatAdministratorRights(bool can_manage_chat_, bool can_change_info_, bool can_post_messages_, bool can_edit_messages_, bool can_delete_messages_, bool can_invite_users_, bool can_restrict_members_, bool can_pin_messages_, bool can_manage_topics_, bool can_promote_members_, bool can_manage_video_chats_, bool can_post_stories_, bool can_edit_stories_, bool can_delete_stories_, bool is_anonymous_)
  : can_manage_chat_(can_manage_chat_)
  , can_change_info_(can_change_info_)
  , can_post_messages_(can_post_messages_)
  , can_edit_messages_(can_edit_messages_)
  , can_delete_messages_(can_delete_messages_)
  , can_invite_users_(can_invite_users_)
  , can_restrict_members_(can_restrict_members_)
  , can_pin_messages_(can_pin_messages_)
  , can_manage_topics_(can_manage_topics_)
  , can_promote_members_(can_promote_members_)
  , can_manage_video_chats_(can_manage_video_chats_)
  , can_post_stories_(can_post_stories_)
  , can_edit_stories_(can_edit_stories_)
  , can_delete_stories_(can_delete_stories_)
  , is_anonymous_(is_anonymous_)
{}

const std::int32_t chatAdministratorRights::ID;

object_ptr<chatAdministratorRights> chatAdministratorRights::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatAdministratorRights> res = make_object<chatAdministratorRights>();
  res->can_manage_chat_ = (env->GetBooleanField(p, res->can_manage_chat_fieldID) != 0);
  res->can_change_info_ = (env->GetBooleanField(p, res->can_change_info_fieldID) != 0);
  res->can_post_messages_ = (env->GetBooleanField(p, res->can_post_messages_fieldID) != 0);
  res->can_edit_messages_ = (env->GetBooleanField(p, res->can_edit_messages_fieldID) != 0);
  res->can_delete_messages_ = (env->GetBooleanField(p, res->can_delete_messages_fieldID) != 0);
  res->can_invite_users_ = (env->GetBooleanField(p, res->can_invite_users_fieldID) != 0);
  res->can_restrict_members_ = (env->GetBooleanField(p, res->can_restrict_members_fieldID) != 0);
  res->can_pin_messages_ = (env->GetBooleanField(p, res->can_pin_messages_fieldID) != 0);
  res->can_manage_topics_ = (env->GetBooleanField(p, res->can_manage_topics_fieldID) != 0);
  res->can_promote_members_ = (env->GetBooleanField(p, res->can_promote_members_fieldID) != 0);
  res->can_manage_video_chats_ = (env->GetBooleanField(p, res->can_manage_video_chats_fieldID) != 0);
  res->can_post_stories_ = (env->GetBooleanField(p, res->can_post_stories_fieldID) != 0);
  res->can_edit_stories_ = (env->GetBooleanField(p, res->can_edit_stories_fieldID) != 0);
  res->can_delete_stories_ = (env->GetBooleanField(p, res->can_delete_stories_fieldID) != 0);
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  return res;
}

void chatAdministratorRights::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_manage_chat_fieldID, can_manage_chat_);
  env->SetBooleanField(s, can_change_info_fieldID, can_change_info_);
  env->SetBooleanField(s, can_post_messages_fieldID, can_post_messages_);
  env->SetBooleanField(s, can_edit_messages_fieldID, can_edit_messages_);
  env->SetBooleanField(s, can_delete_messages_fieldID, can_delete_messages_);
  env->SetBooleanField(s, can_invite_users_fieldID, can_invite_users_);
  env->SetBooleanField(s, can_restrict_members_fieldID, can_restrict_members_);
  env->SetBooleanField(s, can_pin_messages_fieldID, can_pin_messages_);
  env->SetBooleanField(s, can_manage_topics_fieldID, can_manage_topics_);
  env->SetBooleanField(s, can_promote_members_fieldID, can_promote_members_);
  env->SetBooleanField(s, can_manage_video_chats_fieldID, can_manage_video_chats_);
  env->SetBooleanField(s, can_post_stories_fieldID, can_post_stories_);
  env->SetBooleanField(s, can_edit_stories_fieldID, can_edit_stories_);
  env->SetBooleanField(s, can_delete_stories_fieldID, can_delete_stories_);
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
}

void chatAdministratorRights::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatAdministratorRights");
    s.store_field("canManageChat", can_manage_chat_);
    s.store_field("canChangeInfo", can_change_info_);
    s.store_field("canPostMessages", can_post_messages_);
    s.store_field("canEditMessages", can_edit_messages_);
    s.store_field("canDeleteMessages", can_delete_messages_);
    s.store_field("canInviteUsers", can_invite_users_);
    s.store_field("canRestrictMembers", can_restrict_members_);
    s.store_field("canPinMessages", can_pin_messages_);
    s.store_field("canManageTopics", can_manage_topics_);
    s.store_field("canPromoteMembers", can_promote_members_);
    s.store_field("canManageVideoChats", can_manage_video_chats_);
    s.store_field("canPostStories", can_post_stories_);
    s.store_field("canEditStories", can_edit_stories_);
    s.store_field("canDeleteStories", can_delete_stories_);
    s.store_field("isAnonymous", is_anonymous_);
    s.store_class_end();
  }
}

void chatAdministratorRights::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAdministratorRights").c_str());
    can_manage_chat_fieldID = jni::get_field_id(env, Class, "canManageChat", "Z");
    can_change_info_fieldID = jni::get_field_id(env, Class, "canChangeInfo", "Z");
    can_post_messages_fieldID = jni::get_field_id(env, Class, "canPostMessages", "Z");
    can_edit_messages_fieldID = jni::get_field_id(env, Class, "canEditMessages", "Z");
    can_delete_messages_fieldID = jni::get_field_id(env, Class, "canDeleteMessages", "Z");
    can_invite_users_fieldID = jni::get_field_id(env, Class, "canInviteUsers", "Z");
    can_restrict_members_fieldID = jni::get_field_id(env, Class, "canRestrictMembers", "Z");
    can_pin_messages_fieldID = jni::get_field_id(env, Class, "canPinMessages", "Z");
    can_manage_topics_fieldID = jni::get_field_id(env, Class, "canManageTopics", "Z");
    can_promote_members_fieldID = jni::get_field_id(env, Class, "canPromoteMembers", "Z");
    can_manage_video_chats_fieldID = jni::get_field_id(env, Class, "canManageVideoChats", "Z");
    can_post_stories_fieldID = jni::get_field_id(env, Class, "canPostStories", "Z");
    can_edit_stories_fieldID = jni::get_field_id(env, Class, "canEditStories", "Z");
    can_delete_stories_fieldID = jni::get_field_id(env, Class, "canDeleteStories", "Z");
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatAdministrators::Class;
jfieldID chatAdministrators::administrators_fieldID;

chatAdministrators::chatAdministrators()
  : administrators_()
{}

chatAdministrators::chatAdministrators(array<object_ptr<chatAdministrator>> &&administrators_)
  : administrators_(std::move(administrators_))
{}

const std::int32_t chatAdministrators::ID;

object_ptr<chatAdministrators> chatAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatAdministrators> res = make_object<chatAdministrators>();
  res->administrators_ = jni::FetchVector<chatAdministrator>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->administrators_fieldID));
  return res;
}

void chatAdministrators::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, administrators_); if (arr_tmp_) { env->SetObjectField(s, administrators_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatAdministrators");
    { s.store_vector_begin("administrators", administrators_.size()); for (const auto &_value : administrators_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAdministrators").c_str());
    administrators_fieldID = jni::get_field_id(env, Class, "administrators", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatAdministrator;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatAvailableReactions::Class;

object_ptr<ChatAvailableReactions> ChatAvailableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatAvailableReactionsAll::ID:
      return chatAvailableReactionsAll::fetch(env, p);
    case chatAvailableReactionsSome::ID:
      return chatAvailableReactionsSome::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatAvailableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAvailableReactions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatAvailableReactionsAll::Class;
jfieldID chatAvailableReactionsAll::max_reaction_count_fieldID;

chatAvailableReactionsAll::chatAvailableReactionsAll()
  : max_reaction_count_()
{}

chatAvailableReactionsAll::chatAvailableReactionsAll(int32 max_reaction_count_)
  : max_reaction_count_(max_reaction_count_)
{}

const std::int32_t chatAvailableReactionsAll::ID;

object_ptr<ChatAvailableReactions> chatAvailableReactionsAll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatAvailableReactionsAll> res = make_object<chatAvailableReactionsAll>();
  res->max_reaction_count_ = env->GetIntField(p, res->max_reaction_count_fieldID);
  return std::move(res);
}

void chatAvailableReactionsAll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, max_reaction_count_fieldID, max_reaction_count_);
}

void chatAvailableReactionsAll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatAvailableReactionsAll");
    s.store_field("maxReactionCount", max_reaction_count_);
    s.store_class_end();
  }
}

void chatAvailableReactionsAll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAvailableReactionsAll").c_str());
    max_reaction_count_fieldID = jni::get_field_id(env, Class, "maxReactionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatAvailableReactionsSome::Class;
jfieldID chatAvailableReactionsSome::reactions_fieldID;
jfieldID chatAvailableReactionsSome::max_reaction_count_fieldID;

chatAvailableReactionsSome::chatAvailableReactionsSome()
  : reactions_()
  , max_reaction_count_()
{}

chatAvailableReactionsSome::chatAvailableReactionsSome(array<object_ptr<ReactionType>> &&reactions_, int32 max_reaction_count_)
  : reactions_(std::move(reactions_))
  , max_reaction_count_(max_reaction_count_)
{}

const std::int32_t chatAvailableReactionsSome::ID;

object_ptr<ChatAvailableReactions> chatAvailableReactionsSome::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatAvailableReactionsSome> res = make_object<chatAvailableReactionsSome>();
  res->reactions_ = jni::FetchVector<ReactionType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->reactions_fieldID));
  res->max_reaction_count_ = env->GetIntField(p, res->max_reaction_count_fieldID);
  return std::move(res);
}

void chatAvailableReactionsSome::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, reactions_); if (arr_tmp_) { env->SetObjectField(s, reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, max_reaction_count_fieldID, max_reaction_count_);
}

void chatAvailableReactionsSome::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatAvailableReactionsSome");
    { s.store_vector_begin("reactions", reactions_.size()); for (const auto &_value : reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("maxReactionCount", max_reaction_count_);
    s.store_class_end();
  }
}

void chatAvailableReactionsSome::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatAvailableReactionsSome").c_str());
    reactions_fieldID = jni::get_field_id(env, Class, "reactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    max_reaction_count_fieldID = jni::get_field_id(env, Class, "maxReactionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatBackground::Class;
jfieldID chatBackground::background_fieldID;
jfieldID chatBackground::dark_theme_dimming_fieldID;

chatBackground::chatBackground()
  : background_()
  , dark_theme_dimming_()
{}

chatBackground::chatBackground(object_ptr<background> &&background_, int32 dark_theme_dimming_)
  : background_(std::move(background_))
  , dark_theme_dimming_(dark_theme_dimming_)
{}

const std::int32_t chatBackground::ID;

object_ptr<chatBackground> chatBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBackground> res = make_object<chatBackground>();
  res->background_ = jni::fetch_tl_object<background>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->dark_theme_dimming_ = env->GetIntField(p, res->dark_theme_dimming_fieldID);
  return res;
}

void chatBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, dark_theme_dimming_fieldID, dark_theme_dimming_);
}

void chatBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBackground");
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_field("darkThemeDimming", dark_theme_dimming_);
    s.store_class_end();
  }
}

void chatBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBackground").c_str());
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Background;").c_str());
    dark_theme_dimming_fieldID = jni::get_field_id(env, Class, "darkThemeDimming", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoost::Class;
jfieldID chatBoost::id_fieldID;
jfieldID chatBoost::count_fieldID;
jfieldID chatBoost::source_fieldID;
jfieldID chatBoost::start_date_fieldID;
jfieldID chatBoost::expiration_date_fieldID;

chatBoost::chatBoost()
  : id_()
  , count_()
  , source_()
  , start_date_()
  , expiration_date_()
{}

chatBoost::chatBoost(string const &id_, int32 count_, object_ptr<ChatBoostSource> &&source_, int32 start_date_, int32 expiration_date_)
  : id_(id_)
  , count_(count_)
  , source_(std::move(source_))
  , start_date_(start_date_)
  , expiration_date_(expiration_date_)
{}

const std::int32_t chatBoost::ID;

object_ptr<chatBoost> chatBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoost> res = make_object<chatBoost>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  res->source_ = jni::fetch_tl_object<ChatBoostSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  return res;
}

void chatBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, count_fieldID, count_);
  if (source_ != nullptr) { jobject next; source_->store(env, next); if (next) { env->SetObjectField(s, source_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_date_fieldID, start_date_);
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
}

void chatBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoost");
    s.store_field("id", id_);
    s.store_field("count", count_);
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_field("startDate", start_date_);
    s.store_field("expirationDate", expiration_date_);
    s.store_class_end();
  }
}

void chatBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoost").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBoostSource;").c_str());
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostFeatures::Class;
jfieldID chatBoostFeatures::features_fieldID;
jfieldID chatBoostFeatures::min_profile_background_custom_emoji_boost_level_fieldID;
jfieldID chatBoostFeatures::min_background_custom_emoji_boost_level_fieldID;
jfieldID chatBoostFeatures::min_emoji_status_boost_level_fieldID;
jfieldID chatBoostFeatures::min_chat_theme_background_boost_level_fieldID;
jfieldID chatBoostFeatures::min_custom_background_boost_level_fieldID;
jfieldID chatBoostFeatures::min_custom_emoji_sticker_set_boost_level_fieldID;
jfieldID chatBoostFeatures::min_automatic_translation_boost_level_fieldID;
jfieldID chatBoostFeatures::min_speech_recognition_boost_level_fieldID;
jfieldID chatBoostFeatures::min_sponsored_message_disable_boost_level_fieldID;

chatBoostFeatures::chatBoostFeatures()
  : features_()
  , min_profile_background_custom_emoji_boost_level_()
  , min_background_custom_emoji_boost_level_()
  , min_emoji_status_boost_level_()
  , min_chat_theme_background_boost_level_()
  , min_custom_background_boost_level_()
  , min_custom_emoji_sticker_set_boost_level_()
  , min_automatic_translation_boost_level_()
  , min_speech_recognition_boost_level_()
  , min_sponsored_message_disable_boost_level_()
{}

chatBoostFeatures::chatBoostFeatures(array<object_ptr<chatBoostLevelFeatures>> &&features_, int32 min_profile_background_custom_emoji_boost_level_, int32 min_background_custom_emoji_boost_level_, int32 min_emoji_status_boost_level_, int32 min_chat_theme_background_boost_level_, int32 min_custom_background_boost_level_, int32 min_custom_emoji_sticker_set_boost_level_, int32 min_automatic_translation_boost_level_, int32 min_speech_recognition_boost_level_, int32 min_sponsored_message_disable_boost_level_)
  : features_(std::move(features_))
  , min_profile_background_custom_emoji_boost_level_(min_profile_background_custom_emoji_boost_level_)
  , min_background_custom_emoji_boost_level_(min_background_custom_emoji_boost_level_)
  , min_emoji_status_boost_level_(min_emoji_status_boost_level_)
  , min_chat_theme_background_boost_level_(min_chat_theme_background_boost_level_)
  , min_custom_background_boost_level_(min_custom_background_boost_level_)
  , min_custom_emoji_sticker_set_boost_level_(min_custom_emoji_sticker_set_boost_level_)
  , min_automatic_translation_boost_level_(min_automatic_translation_boost_level_)
  , min_speech_recognition_boost_level_(min_speech_recognition_boost_level_)
  , min_sponsored_message_disable_boost_level_(min_sponsored_message_disable_boost_level_)
{}

const std::int32_t chatBoostFeatures::ID;

object_ptr<chatBoostFeatures> chatBoostFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostFeatures> res = make_object<chatBoostFeatures>();
  res->features_ = jni::FetchVector<chatBoostLevelFeatures>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->features_fieldID));
  res->min_profile_background_custom_emoji_boost_level_ = env->GetIntField(p, res->min_profile_background_custom_emoji_boost_level_fieldID);
  res->min_background_custom_emoji_boost_level_ = env->GetIntField(p, res->min_background_custom_emoji_boost_level_fieldID);
  res->min_emoji_status_boost_level_ = env->GetIntField(p, res->min_emoji_status_boost_level_fieldID);
  res->min_chat_theme_background_boost_level_ = env->GetIntField(p, res->min_chat_theme_background_boost_level_fieldID);
  res->min_custom_background_boost_level_ = env->GetIntField(p, res->min_custom_background_boost_level_fieldID);
  res->min_custom_emoji_sticker_set_boost_level_ = env->GetIntField(p, res->min_custom_emoji_sticker_set_boost_level_fieldID);
  res->min_automatic_translation_boost_level_ = env->GetIntField(p, res->min_automatic_translation_boost_level_fieldID);
  res->min_speech_recognition_boost_level_ = env->GetIntField(p, res->min_speech_recognition_boost_level_fieldID);
  res->min_sponsored_message_disable_boost_level_ = env->GetIntField(p, res->min_sponsored_message_disable_boost_level_fieldID);
  return res;
}

void chatBoostFeatures::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, features_); if (arr_tmp_) { env->SetObjectField(s, features_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, min_profile_background_custom_emoji_boost_level_fieldID, min_profile_background_custom_emoji_boost_level_);
  env->SetIntField(s, min_background_custom_emoji_boost_level_fieldID, min_background_custom_emoji_boost_level_);
  env->SetIntField(s, min_emoji_status_boost_level_fieldID, min_emoji_status_boost_level_);
  env->SetIntField(s, min_chat_theme_background_boost_level_fieldID, min_chat_theme_background_boost_level_);
  env->SetIntField(s, min_custom_background_boost_level_fieldID, min_custom_background_boost_level_);
  env->SetIntField(s, min_custom_emoji_sticker_set_boost_level_fieldID, min_custom_emoji_sticker_set_boost_level_);
  env->SetIntField(s, min_automatic_translation_boost_level_fieldID, min_automatic_translation_boost_level_);
  env->SetIntField(s, min_speech_recognition_boost_level_fieldID, min_speech_recognition_boost_level_);
  env->SetIntField(s, min_sponsored_message_disable_boost_level_fieldID, min_sponsored_message_disable_boost_level_);
}

void chatBoostFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostFeatures");
    { s.store_vector_begin("features", features_.size()); for (const auto &_value : features_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("minProfileBackgroundCustomEmojiBoostLevel", min_profile_background_custom_emoji_boost_level_);
    s.store_field("minBackgroundCustomEmojiBoostLevel", min_background_custom_emoji_boost_level_);
    s.store_field("minEmojiStatusBoostLevel", min_emoji_status_boost_level_);
    s.store_field("minChatThemeBackgroundBoostLevel", min_chat_theme_background_boost_level_);
    s.store_field("minCustomBackgroundBoostLevel", min_custom_background_boost_level_);
    s.store_field("minCustomEmojiStickerSetBoostLevel", min_custom_emoji_sticker_set_boost_level_);
    s.store_field("minAutomaticTranslationBoostLevel", min_automatic_translation_boost_level_);
    s.store_field("minSpeechRecognitionBoostLevel", min_speech_recognition_boost_level_);
    s.store_field("minSponsoredMessageDisableBoostLevel", min_sponsored_message_disable_boost_level_);
    s.store_class_end();
  }
}

void chatBoostFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostFeatures").c_str());
    features_fieldID = jni::get_field_id(env, Class, "features", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatBoostLevelFeatures;").c_str());
    min_profile_background_custom_emoji_boost_level_fieldID = jni::get_field_id(env, Class, "minProfileBackgroundCustomEmojiBoostLevel", "I");
    min_background_custom_emoji_boost_level_fieldID = jni::get_field_id(env, Class, "minBackgroundCustomEmojiBoostLevel", "I");
    min_emoji_status_boost_level_fieldID = jni::get_field_id(env, Class, "minEmojiStatusBoostLevel", "I");
    min_chat_theme_background_boost_level_fieldID = jni::get_field_id(env, Class, "minChatThemeBackgroundBoostLevel", "I");
    min_custom_background_boost_level_fieldID = jni::get_field_id(env, Class, "minCustomBackgroundBoostLevel", "I");
    min_custom_emoji_sticker_set_boost_level_fieldID = jni::get_field_id(env, Class, "minCustomEmojiStickerSetBoostLevel", "I");
    min_automatic_translation_boost_level_fieldID = jni::get_field_id(env, Class, "minAutomaticTranslationBoostLevel", "I");
    min_speech_recognition_boost_level_fieldID = jni::get_field_id(env, Class, "minSpeechRecognitionBoostLevel", "I");
    min_sponsored_message_disable_boost_level_fieldID = jni::get_field_id(env, Class, "minSponsoredMessageDisableBoostLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostLevelFeatures::Class;
jfieldID chatBoostLevelFeatures::level_fieldID;
jfieldID chatBoostLevelFeatures::story_per_day_count_fieldID;
jfieldID chatBoostLevelFeatures::custom_emoji_reaction_count_fieldID;
jfieldID chatBoostLevelFeatures::title_color_count_fieldID;
jfieldID chatBoostLevelFeatures::profile_accent_color_count_fieldID;
jfieldID chatBoostLevelFeatures::can_set_profile_background_custom_emoji_fieldID;
jfieldID chatBoostLevelFeatures::accent_color_count_fieldID;
jfieldID chatBoostLevelFeatures::can_set_background_custom_emoji_fieldID;
jfieldID chatBoostLevelFeatures::can_set_emoji_status_fieldID;
jfieldID chatBoostLevelFeatures::chat_theme_background_count_fieldID;
jfieldID chatBoostLevelFeatures::can_set_custom_background_fieldID;
jfieldID chatBoostLevelFeatures::can_set_custom_emoji_sticker_set_fieldID;
jfieldID chatBoostLevelFeatures::can_enable_automatic_translation_fieldID;
jfieldID chatBoostLevelFeatures::can_recognize_speech_fieldID;
jfieldID chatBoostLevelFeatures::can_disable_sponsored_messages_fieldID;

chatBoostLevelFeatures::chatBoostLevelFeatures()
  : level_()
  , story_per_day_count_()
  , custom_emoji_reaction_count_()
  , title_color_count_()
  , profile_accent_color_count_()
  , can_set_profile_background_custom_emoji_()
  , accent_color_count_()
  , can_set_background_custom_emoji_()
  , can_set_emoji_status_()
  , chat_theme_background_count_()
  , can_set_custom_background_()
  , can_set_custom_emoji_sticker_set_()
  , can_enable_automatic_translation_()
  , can_recognize_speech_()
  , can_disable_sponsored_messages_()
{}

chatBoostLevelFeatures::chatBoostLevelFeatures(int32 level_, int32 story_per_day_count_, int32 custom_emoji_reaction_count_, int32 title_color_count_, int32 profile_accent_color_count_, bool can_set_profile_background_custom_emoji_, int32 accent_color_count_, bool can_set_background_custom_emoji_, bool can_set_emoji_status_, int32 chat_theme_background_count_, bool can_set_custom_background_, bool can_set_custom_emoji_sticker_set_, bool can_enable_automatic_translation_, bool can_recognize_speech_, bool can_disable_sponsored_messages_)
  : level_(level_)
  , story_per_day_count_(story_per_day_count_)
  , custom_emoji_reaction_count_(custom_emoji_reaction_count_)
  , title_color_count_(title_color_count_)
  , profile_accent_color_count_(profile_accent_color_count_)
  , can_set_profile_background_custom_emoji_(can_set_profile_background_custom_emoji_)
  , accent_color_count_(accent_color_count_)
  , can_set_background_custom_emoji_(can_set_background_custom_emoji_)
  , can_set_emoji_status_(can_set_emoji_status_)
  , chat_theme_background_count_(chat_theme_background_count_)
  , can_set_custom_background_(can_set_custom_background_)
  , can_set_custom_emoji_sticker_set_(can_set_custom_emoji_sticker_set_)
  , can_enable_automatic_translation_(can_enable_automatic_translation_)
  , can_recognize_speech_(can_recognize_speech_)
  , can_disable_sponsored_messages_(can_disable_sponsored_messages_)
{}

const std::int32_t chatBoostLevelFeatures::ID;

object_ptr<chatBoostLevelFeatures> chatBoostLevelFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostLevelFeatures> res = make_object<chatBoostLevelFeatures>();
  res->level_ = env->GetIntField(p, res->level_fieldID);
  res->story_per_day_count_ = env->GetIntField(p, res->story_per_day_count_fieldID);
  res->custom_emoji_reaction_count_ = env->GetIntField(p, res->custom_emoji_reaction_count_fieldID);
  res->title_color_count_ = env->GetIntField(p, res->title_color_count_fieldID);
  res->profile_accent_color_count_ = env->GetIntField(p, res->profile_accent_color_count_fieldID);
  res->can_set_profile_background_custom_emoji_ = (env->GetBooleanField(p, res->can_set_profile_background_custom_emoji_fieldID) != 0);
  res->accent_color_count_ = env->GetIntField(p, res->accent_color_count_fieldID);
  res->can_set_background_custom_emoji_ = (env->GetBooleanField(p, res->can_set_background_custom_emoji_fieldID) != 0);
  res->can_set_emoji_status_ = (env->GetBooleanField(p, res->can_set_emoji_status_fieldID) != 0);
  res->chat_theme_background_count_ = env->GetIntField(p, res->chat_theme_background_count_fieldID);
  res->can_set_custom_background_ = (env->GetBooleanField(p, res->can_set_custom_background_fieldID) != 0);
  res->can_set_custom_emoji_sticker_set_ = (env->GetBooleanField(p, res->can_set_custom_emoji_sticker_set_fieldID) != 0);
  res->can_enable_automatic_translation_ = (env->GetBooleanField(p, res->can_enable_automatic_translation_fieldID) != 0);
  res->can_recognize_speech_ = (env->GetBooleanField(p, res->can_recognize_speech_fieldID) != 0);
  res->can_disable_sponsored_messages_ = (env->GetBooleanField(p, res->can_disable_sponsored_messages_fieldID) != 0);
  return res;
}

void chatBoostLevelFeatures::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, level_fieldID, level_);
  env->SetIntField(s, story_per_day_count_fieldID, story_per_day_count_);
  env->SetIntField(s, custom_emoji_reaction_count_fieldID, custom_emoji_reaction_count_);
  env->SetIntField(s, title_color_count_fieldID, title_color_count_);
  env->SetIntField(s, profile_accent_color_count_fieldID, profile_accent_color_count_);
  env->SetBooleanField(s, can_set_profile_background_custom_emoji_fieldID, can_set_profile_background_custom_emoji_);
  env->SetIntField(s, accent_color_count_fieldID, accent_color_count_);
  env->SetBooleanField(s, can_set_background_custom_emoji_fieldID, can_set_background_custom_emoji_);
  env->SetBooleanField(s, can_set_emoji_status_fieldID, can_set_emoji_status_);
  env->SetIntField(s, chat_theme_background_count_fieldID, chat_theme_background_count_);
  env->SetBooleanField(s, can_set_custom_background_fieldID, can_set_custom_background_);
  env->SetBooleanField(s, can_set_custom_emoji_sticker_set_fieldID, can_set_custom_emoji_sticker_set_);
  env->SetBooleanField(s, can_enable_automatic_translation_fieldID, can_enable_automatic_translation_);
  env->SetBooleanField(s, can_recognize_speech_fieldID, can_recognize_speech_);
  env->SetBooleanField(s, can_disable_sponsored_messages_fieldID, can_disable_sponsored_messages_);
}

void chatBoostLevelFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostLevelFeatures");
    s.store_field("level", level_);
    s.store_field("storyPerDayCount", story_per_day_count_);
    s.store_field("customEmojiReactionCount", custom_emoji_reaction_count_);
    s.store_field("titleColorCount", title_color_count_);
    s.store_field("profileAccentColorCount", profile_accent_color_count_);
    s.store_field("canSetProfileBackgroundCustomEmoji", can_set_profile_background_custom_emoji_);
    s.store_field("accentColorCount", accent_color_count_);
    s.store_field("canSetBackgroundCustomEmoji", can_set_background_custom_emoji_);
    s.store_field("canSetEmojiStatus", can_set_emoji_status_);
    s.store_field("chatThemeBackgroundCount", chat_theme_background_count_);
    s.store_field("canSetCustomBackground", can_set_custom_background_);
    s.store_field("canSetCustomEmojiStickerSet", can_set_custom_emoji_sticker_set_);
    s.store_field("canEnableAutomaticTranslation", can_enable_automatic_translation_);
    s.store_field("canRecognizeSpeech", can_recognize_speech_);
    s.store_field("canDisableSponsoredMessages", can_disable_sponsored_messages_);
    s.store_class_end();
  }
}

void chatBoostLevelFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostLevelFeatures").c_str());
    level_fieldID = jni::get_field_id(env, Class, "level", "I");
    story_per_day_count_fieldID = jni::get_field_id(env, Class, "storyPerDayCount", "I");
    custom_emoji_reaction_count_fieldID = jni::get_field_id(env, Class, "customEmojiReactionCount", "I");
    title_color_count_fieldID = jni::get_field_id(env, Class, "titleColorCount", "I");
    profile_accent_color_count_fieldID = jni::get_field_id(env, Class, "profileAccentColorCount", "I");
    can_set_profile_background_custom_emoji_fieldID = jni::get_field_id(env, Class, "canSetProfileBackgroundCustomEmoji", "Z");
    accent_color_count_fieldID = jni::get_field_id(env, Class, "accentColorCount", "I");
    can_set_background_custom_emoji_fieldID = jni::get_field_id(env, Class, "canSetBackgroundCustomEmoji", "Z");
    can_set_emoji_status_fieldID = jni::get_field_id(env, Class, "canSetEmojiStatus", "Z");
    chat_theme_background_count_fieldID = jni::get_field_id(env, Class, "chatThemeBackgroundCount", "I");
    can_set_custom_background_fieldID = jni::get_field_id(env, Class, "canSetCustomBackground", "Z");
    can_set_custom_emoji_sticker_set_fieldID = jni::get_field_id(env, Class, "canSetCustomEmojiStickerSet", "Z");
    can_enable_automatic_translation_fieldID = jni::get_field_id(env, Class, "canEnableAutomaticTranslation", "Z");
    can_recognize_speech_fieldID = jni::get_field_id(env, Class, "canRecognizeSpeech", "Z");
    can_disable_sponsored_messages_fieldID = jni::get_field_id(env, Class, "canDisableSponsoredMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostLink::Class;
jfieldID chatBoostLink::link_fieldID;
jfieldID chatBoostLink::is_public_fieldID;

chatBoostLink::chatBoostLink()
  : link_()
  , is_public_()
{}

chatBoostLink::chatBoostLink(string const &link_, bool is_public_)
  : link_(link_)
  , is_public_(is_public_)
{}

const std::int32_t chatBoostLink::ID;

object_ptr<chatBoostLink> chatBoostLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostLink> res = make_object<chatBoostLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  return res;
}

void chatBoostLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_); if (nextString) { env->SetObjectField(s, link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_public_fieldID, is_public_);
}

void chatBoostLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostLink");
    s.store_field("link", link_);
    s.store_field("isPublic", is_public_);
    s.store_class_end();
  }
}

void chatBoostLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostLinkInfo::Class;
jfieldID chatBoostLinkInfo::is_public_fieldID;
jfieldID chatBoostLinkInfo::chat_id_fieldID;

chatBoostLinkInfo::chatBoostLinkInfo()
  : is_public_()
  , chat_id_()
{}

chatBoostLinkInfo::chatBoostLinkInfo(bool is_public_, int53 chat_id_)
  : is_public_(is_public_)
  , chat_id_(chat_id_)
{}

const std::int32_t chatBoostLinkInfo::ID;

object_ptr<chatBoostLinkInfo> chatBoostLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostLinkInfo> res = make_object<chatBoostLinkInfo>();
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void chatBoostLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_public_fieldID, is_public_);
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void chatBoostLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostLinkInfo");
    s.store_field("isPublic", is_public_);
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void chatBoostLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostLinkInfo").c_str());
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostSlot::Class;
jfieldID chatBoostSlot::slot_id_fieldID;
jfieldID chatBoostSlot::currently_boosted_chat_id_fieldID;
jfieldID chatBoostSlot::start_date_fieldID;
jfieldID chatBoostSlot::expiration_date_fieldID;
jfieldID chatBoostSlot::cooldown_until_date_fieldID;

chatBoostSlot::chatBoostSlot()
  : slot_id_()
  , currently_boosted_chat_id_()
  , start_date_()
  , expiration_date_()
  , cooldown_until_date_()
{}

chatBoostSlot::chatBoostSlot(int32 slot_id_, int53 currently_boosted_chat_id_, int32 start_date_, int32 expiration_date_, int32 cooldown_until_date_)
  : slot_id_(slot_id_)
  , currently_boosted_chat_id_(currently_boosted_chat_id_)
  , start_date_(start_date_)
  , expiration_date_(expiration_date_)
  , cooldown_until_date_(cooldown_until_date_)
{}

const std::int32_t chatBoostSlot::ID;

object_ptr<chatBoostSlot> chatBoostSlot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostSlot> res = make_object<chatBoostSlot>();
  res->slot_id_ = env->GetIntField(p, res->slot_id_fieldID);
  res->currently_boosted_chat_id_ = env->GetLongField(p, res->currently_boosted_chat_id_fieldID);
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  res->cooldown_until_date_ = env->GetIntField(p, res->cooldown_until_date_fieldID);
  return res;
}

void chatBoostSlot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, slot_id_fieldID, slot_id_);
  env->SetLongField(s, currently_boosted_chat_id_fieldID, currently_boosted_chat_id_);
  env->SetIntField(s, start_date_fieldID, start_date_);
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
  env->SetIntField(s, cooldown_until_date_fieldID, cooldown_until_date_);
}

void chatBoostSlot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostSlot");
    s.store_field("slotId", slot_id_);
    s.store_field("currentlyBoostedChatId", currently_boosted_chat_id_);
    s.store_field("startDate", start_date_);
    s.store_field("expirationDate", expiration_date_);
    s.store_field("cooldownUntilDate", cooldown_until_date_);
    s.store_class_end();
  }
}

void chatBoostSlot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSlot").c_str());
    slot_id_fieldID = jni::get_field_id(env, Class, "slotId", "I");
    currently_boosted_chat_id_fieldID = jni::get_field_id(env, Class, "currentlyBoostedChatId", "J");
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    cooldown_until_date_fieldID = jni::get_field_id(env, Class, "cooldownUntilDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostSlots::Class;
jfieldID chatBoostSlots::slots_fieldID;

chatBoostSlots::chatBoostSlots()
  : slots_()
{}

chatBoostSlots::chatBoostSlots(array<object_ptr<chatBoostSlot>> &&slots_)
  : slots_(std::move(slots_))
{}

const std::int32_t chatBoostSlots::ID;

object_ptr<chatBoostSlots> chatBoostSlots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostSlots> res = make_object<chatBoostSlots>();
  res->slots_ = jni::FetchVector<chatBoostSlot>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->slots_fieldID));
  return res;
}

void chatBoostSlots::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, slots_); if (arr_tmp_) { env->SetObjectField(s, slots_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatBoostSlots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostSlots");
    { s.store_vector_begin("slots", slots_.size()); for (const auto &_value : slots_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatBoostSlots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSlots").c_str());
    slots_fieldID = jni::get_field_id(env, Class, "slots", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatBoostSlot;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatBoostSource::Class;

object_ptr<ChatBoostSource> ChatBoostSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatBoostSourceGiftCode::ID:
      return chatBoostSourceGiftCode::fetch(env, p);
    case chatBoostSourceGiveaway::ID:
      return chatBoostSourceGiveaway::fetch(env, p);
    case chatBoostSourcePremium::ID:
      return chatBoostSourcePremium::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatBoostSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostSourceGiftCode::Class;
jfieldID chatBoostSourceGiftCode::user_id_fieldID;
jfieldID chatBoostSourceGiftCode::gift_code_fieldID;

chatBoostSourceGiftCode::chatBoostSourceGiftCode()
  : user_id_()
  , gift_code_()
{}

chatBoostSourceGiftCode::chatBoostSourceGiftCode(int53 user_id_, string const &gift_code_)
  : user_id_(user_id_)
  , gift_code_(gift_code_)
{}

const std::int32_t chatBoostSourceGiftCode::ID;

object_ptr<ChatBoostSource> chatBoostSourceGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostSourceGiftCode> res = make_object<chatBoostSourceGiftCode>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_code_ = jni::fetch_string(env, p, res->gift_code_fieldID);
  return std::move(res);
}

void chatBoostSourceGiftCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { jstring nextString = jni::to_jstring(env, gift_code_); if (nextString) { env->SetObjectField(s, gift_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatBoostSourceGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostSourceGiftCode");
    s.store_field("userId", user_id_);
    s.store_field("giftCode", gift_code_);
    s.store_class_end();
  }
}

void chatBoostSourceGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSourceGiftCode").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_code_fieldID = jni::get_field_id(env, Class, "giftCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostSourceGiveaway::Class;
jfieldID chatBoostSourceGiveaway::user_id_fieldID;
jfieldID chatBoostSourceGiveaway::gift_code_fieldID;
jfieldID chatBoostSourceGiveaway::star_count_fieldID;
jfieldID chatBoostSourceGiveaway::giveaway_message_id_fieldID;
jfieldID chatBoostSourceGiveaway::is_unclaimed_fieldID;

chatBoostSourceGiveaway::chatBoostSourceGiveaway()
  : user_id_()
  , gift_code_()
  , star_count_()
  , giveaway_message_id_()
  , is_unclaimed_()
{}

chatBoostSourceGiveaway::chatBoostSourceGiveaway(int53 user_id_, string const &gift_code_, int53 star_count_, int53 giveaway_message_id_, bool is_unclaimed_)
  : user_id_(user_id_)
  , gift_code_(gift_code_)
  , star_count_(star_count_)
  , giveaway_message_id_(giveaway_message_id_)
  , is_unclaimed_(is_unclaimed_)
{}

const std::int32_t chatBoostSourceGiveaway::ID;

object_ptr<ChatBoostSource> chatBoostSourceGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostSourceGiveaway> res = make_object<chatBoostSourceGiveaway>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_code_ = jni::fetch_string(env, p, res->gift_code_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  res->is_unclaimed_ = (env->GetBooleanField(p, res->is_unclaimed_fieldID) != 0);
  return std::move(res);
}

void chatBoostSourceGiveaway::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { jstring nextString = jni::to_jstring(env, gift_code_); if (nextString) { env->SetObjectField(s, gift_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, star_count_fieldID, star_count_);
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
  env->SetBooleanField(s, is_unclaimed_fieldID, is_unclaimed_);
}

void chatBoostSourceGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostSourceGiveaway");
    s.store_field("userId", user_id_);
    s.store_field("giftCode", gift_code_);
    s.store_field("starCount", star_count_);
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_field("isUnclaimed", is_unclaimed_);
    s.store_class_end();
  }
}

void chatBoostSourceGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSourceGiveaway").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_code_fieldID = jni::get_field_id(env, Class, "giftCode", "Ljava/lang/String;");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    is_unclaimed_fieldID = jni::get_field_id(env, Class, "isUnclaimed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostSourcePremium::Class;
jfieldID chatBoostSourcePremium::user_id_fieldID;

chatBoostSourcePremium::chatBoostSourcePremium()
  : user_id_()
{}

chatBoostSourcePremium::chatBoostSourcePremium(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t chatBoostSourcePremium::ID;

object_ptr<ChatBoostSource> chatBoostSourcePremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostSourcePremium> res = make_object<chatBoostSourcePremium>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void chatBoostSourcePremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void chatBoostSourcePremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostSourcePremium");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void chatBoostSourcePremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostSourcePremium").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatBoostStatus::Class;
jfieldID chatBoostStatus::boost_url_fieldID;
jfieldID chatBoostStatus::applied_slot_ids_fieldID;
jfieldID chatBoostStatus::level_fieldID;
jfieldID chatBoostStatus::gift_code_boost_count_fieldID;
jfieldID chatBoostStatus::boost_count_fieldID;
jfieldID chatBoostStatus::current_level_boost_count_fieldID;
jfieldID chatBoostStatus::next_level_boost_count_fieldID;
jfieldID chatBoostStatus::premium_member_count_fieldID;
jfieldID chatBoostStatus::premium_member_percentage_fieldID;
jfieldID chatBoostStatus::prepaid_giveaways_fieldID;

chatBoostStatus::chatBoostStatus()
  : boost_url_()
  , applied_slot_ids_()
  , level_()
  , gift_code_boost_count_()
  , boost_count_()
  , current_level_boost_count_()
  , next_level_boost_count_()
  , premium_member_count_()
  , premium_member_percentage_()
  , prepaid_giveaways_()
{}

chatBoostStatus::chatBoostStatus(string const &boost_url_, array<int32> &&applied_slot_ids_, int32 level_, int32 gift_code_boost_count_, int32 boost_count_, int32 current_level_boost_count_, int32 next_level_boost_count_, int32 premium_member_count_, double premium_member_percentage_, array<object_ptr<prepaidGiveaway>> &&prepaid_giveaways_)
  : boost_url_(boost_url_)
  , applied_slot_ids_(std::move(applied_slot_ids_))
  , level_(level_)
  , gift_code_boost_count_(gift_code_boost_count_)
  , boost_count_(boost_count_)
  , current_level_boost_count_(current_level_boost_count_)
  , next_level_boost_count_(next_level_boost_count_)
  , premium_member_count_(premium_member_count_)
  , premium_member_percentage_(premium_member_percentage_)
  , prepaid_giveaways_(std::move(prepaid_giveaways_))
{}

const std::int32_t chatBoostStatus::ID;

object_ptr<chatBoostStatus> chatBoostStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatBoostStatus> res = make_object<chatBoostStatus>();
  res->boost_url_ = jni::fetch_string(env, p, res->boost_url_fieldID);
  res->applied_slot_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->applied_slot_ids_fieldID));
  res->level_ = env->GetIntField(p, res->level_fieldID);
  res->gift_code_boost_count_ = env->GetIntField(p, res->gift_code_boost_count_fieldID);
  res->boost_count_ = env->GetIntField(p, res->boost_count_fieldID);
  res->current_level_boost_count_ = env->GetIntField(p, res->current_level_boost_count_fieldID);
  res->next_level_boost_count_ = env->GetIntField(p, res->next_level_boost_count_fieldID);
  res->premium_member_count_ = env->GetIntField(p, res->premium_member_count_fieldID);
  res->premium_member_percentage_ = env->GetDoubleField(p, res->premium_member_percentage_fieldID);
  res->prepaid_giveaways_ = jni::FetchVector<prepaidGiveaway>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->prepaid_giveaways_fieldID));
  return res;
}

void chatBoostStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, boost_url_); if (nextString) { env->SetObjectField(s, boost_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, applied_slot_ids_); if (arr_tmp_) { env->SetObjectField(s, applied_slot_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, level_fieldID, level_);
  env->SetIntField(s, gift_code_boost_count_fieldID, gift_code_boost_count_);
  env->SetIntField(s, boost_count_fieldID, boost_count_);
  env->SetIntField(s, current_level_boost_count_fieldID, current_level_boost_count_);
  env->SetIntField(s, next_level_boost_count_fieldID, next_level_boost_count_);
  env->SetIntField(s, premium_member_count_fieldID, premium_member_count_);
  env->SetDoubleField(s, premium_member_percentage_fieldID, premium_member_percentage_);
  { auto arr_tmp_ = jni::store_vector(env, prepaid_giveaways_); if (arr_tmp_) { env->SetObjectField(s, prepaid_giveaways_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatBoostStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatBoostStatus");
    s.store_field("boostUrl", boost_url_);
    { s.store_vector_begin("appliedSlotIds", applied_slot_ids_.size()); for (const auto &_value : applied_slot_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("level", level_);
    s.store_field("giftCodeBoostCount", gift_code_boost_count_);
    s.store_field("boostCount", boost_count_);
    s.store_field("currentLevelBoostCount", current_level_boost_count_);
    s.store_field("nextLevelBoostCount", next_level_boost_count_);
    s.store_field("premiumMemberCount", premium_member_count_);
    s.store_field("premiumMemberPercentage", premium_member_percentage_);
    { s.store_vector_begin("prepaidGiveaways", prepaid_giveaways_.size()); for (const auto &_value : prepaid_giveaways_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatBoostStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatBoostStatus").c_str());
    boost_url_fieldID = jni::get_field_id(env, Class, "boostUrl", "Ljava/lang/String;");
    applied_slot_ids_fieldID = jni::get_field_id(env, Class, "appliedSlotIds", "[I");
    level_fieldID = jni::get_field_id(env, Class, "level", "I");
    gift_code_boost_count_fieldID = jni::get_field_id(env, Class, "giftCodeBoostCount", "I");
    boost_count_fieldID = jni::get_field_id(env, Class, "boostCount", "I");
    current_level_boost_count_fieldID = jni::get_field_id(env, Class, "currentLevelBoostCount", "I");
    next_level_boost_count_fieldID = jni::get_field_id(env, Class, "nextLevelBoostCount", "I");
    premium_member_count_fieldID = jni::get_field_id(env, Class, "premiumMemberCount", "I");
    premium_member_percentage_fieldID = jni::get_field_id(env, Class, "premiumMemberPercentage", "D");
    prepaid_giveaways_fieldID = jni::get_field_id(env, Class, "prepaidGiveaways", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PrepaidGiveaway;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEvent::Class;
jfieldID chatEvent::id_fieldID;
jfieldID chatEvent::date_fieldID;
jfieldID chatEvent::member_id_fieldID;
jfieldID chatEvent::action_fieldID;

chatEvent::chatEvent()
  : id_()
  , date_()
  , member_id_()
  , action_()
{}

chatEvent::chatEvent(int64 id_, int32 date_, object_ptr<MessageSender> &&member_id_, object_ptr<ChatEventAction> &&action_)
  : id_(id_)
  , date_(date_)
  , member_id_(std::move(member_id_))
  , action_(std::move(action_))
{}

const std::int32_t chatEvent::ID;

object_ptr<chatEvent> chatEvent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEvent> res = make_object<chatEvent>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  res->action_ = jni::fetch_tl_object<ChatEventAction>(env, jni::fetch_object(env, p, res->action_fieldID));
  return res;
}

void chatEvent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, date_fieldID, date_);
  if (member_id_ != nullptr) { jobject next; member_id_->store(env, next); if (next) { env->SetObjectField(s, member_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (action_ != nullptr) { jobject next; action_->store(env, next); if (next) { env->SetObjectField(s, action_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEvent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEvent");
    s.store_field("id", id_);
    s.store_field("date", date_);
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_object_field("action", static_cast<const BaseObject *>(action_.get()));
    s.store_class_end();
  }
}

void chatEvent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEvent").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    action_fieldID = jni::get_field_id(env, Class, "action", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatEventAction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatEventAction::Class;

object_ptr<ChatEventAction> ChatEventAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatEventMessageEdited::ID:
      return chatEventMessageEdited::fetch(env, p);
    case chatEventMessageDeleted::ID:
      return chatEventMessageDeleted::fetch(env, p);
    case chatEventMessagePinned::ID:
      return chatEventMessagePinned::fetch(env, p);
    case chatEventMessageUnpinned::ID:
      return chatEventMessageUnpinned::fetch(env, p);
    case chatEventPollStopped::ID:
      return chatEventPollStopped::fetch(env, p);
    case chatEventMemberJoined::ID:
      return chatEventMemberJoined::fetch(env, p);
    case chatEventMemberJoinedByInviteLink::ID:
      return chatEventMemberJoinedByInviteLink::fetch(env, p);
    case chatEventMemberJoinedByRequest::ID:
      return chatEventMemberJoinedByRequest::fetch(env, p);
    case chatEventMemberInvited::ID:
      return chatEventMemberInvited::fetch(env, p);
    case chatEventMemberLeft::ID:
      return chatEventMemberLeft::fetch(env, p);
    case chatEventMemberPromoted::ID:
      return chatEventMemberPromoted::fetch(env, p);
    case chatEventMemberRestricted::ID:
      return chatEventMemberRestricted::fetch(env, p);
    case chatEventMemberSubscriptionExtended::ID:
      return chatEventMemberSubscriptionExtended::fetch(env, p);
    case chatEventAvailableReactionsChanged::ID:
      return chatEventAvailableReactionsChanged::fetch(env, p);
    case chatEventBackgroundChanged::ID:
      return chatEventBackgroundChanged::fetch(env, p);
    case chatEventDescriptionChanged::ID:
      return chatEventDescriptionChanged::fetch(env, p);
    case chatEventEmojiStatusChanged::ID:
      return chatEventEmojiStatusChanged::fetch(env, p);
    case chatEventLinkedChatChanged::ID:
      return chatEventLinkedChatChanged::fetch(env, p);
    case chatEventLocationChanged::ID:
      return chatEventLocationChanged::fetch(env, p);
    case chatEventMessageAutoDeleteTimeChanged::ID:
      return chatEventMessageAutoDeleteTimeChanged::fetch(env, p);
    case chatEventPermissionsChanged::ID:
      return chatEventPermissionsChanged::fetch(env, p);
    case chatEventPhotoChanged::ID:
      return chatEventPhotoChanged::fetch(env, p);
    case chatEventSlowModeDelayChanged::ID:
      return chatEventSlowModeDelayChanged::fetch(env, p);
    case chatEventStickerSetChanged::ID:
      return chatEventStickerSetChanged::fetch(env, p);
    case chatEventCustomEmojiStickerSetChanged::ID:
      return chatEventCustomEmojiStickerSetChanged::fetch(env, p);
    case chatEventTitleChanged::ID:
      return chatEventTitleChanged::fetch(env, p);
    case chatEventUsernameChanged::ID:
      return chatEventUsernameChanged::fetch(env, p);
    case chatEventActiveUsernamesChanged::ID:
      return chatEventActiveUsernamesChanged::fetch(env, p);
    case chatEventAccentColorChanged::ID:
      return chatEventAccentColorChanged::fetch(env, p);
    case chatEventProfileAccentColorChanged::ID:
      return chatEventProfileAccentColorChanged::fetch(env, p);
    case chatEventHasProtectedContentToggled::ID:
      return chatEventHasProtectedContentToggled::fetch(env, p);
    case chatEventInvitesToggled::ID:
      return chatEventInvitesToggled::fetch(env, p);
    case chatEventIsAllHistoryAvailableToggled::ID:
      return chatEventIsAllHistoryAvailableToggled::fetch(env, p);
    case chatEventHasAggressiveAntiSpamEnabledToggled::ID:
      return chatEventHasAggressiveAntiSpamEnabledToggled::fetch(env, p);
    case chatEventSignMessagesToggled::ID:
      return chatEventSignMessagesToggled::fetch(env, p);
    case chatEventShowMessageSenderToggled::ID:
      return chatEventShowMessageSenderToggled::fetch(env, p);
    case chatEventAutomaticTranslationToggled::ID:
      return chatEventAutomaticTranslationToggled::fetch(env, p);
    case chatEventInviteLinkEdited::ID:
      return chatEventInviteLinkEdited::fetch(env, p);
    case chatEventInviteLinkRevoked::ID:
      return chatEventInviteLinkRevoked::fetch(env, p);
    case chatEventInviteLinkDeleted::ID:
      return chatEventInviteLinkDeleted::fetch(env, p);
    case chatEventVideoChatCreated::ID:
      return chatEventVideoChatCreated::fetch(env, p);
    case chatEventVideoChatEnded::ID:
      return chatEventVideoChatEnded::fetch(env, p);
    case chatEventVideoChatMuteNewParticipantsToggled::ID:
      return chatEventVideoChatMuteNewParticipantsToggled::fetch(env, p);
    case chatEventVideoChatParticipantIsMutedToggled::ID:
      return chatEventVideoChatParticipantIsMutedToggled::fetch(env, p);
    case chatEventVideoChatParticipantVolumeLevelChanged::ID:
      return chatEventVideoChatParticipantVolumeLevelChanged::fetch(env, p);
    case chatEventIsForumToggled::ID:
      return chatEventIsForumToggled::fetch(env, p);
    case chatEventForumTopicCreated::ID:
      return chatEventForumTopicCreated::fetch(env, p);
    case chatEventForumTopicEdited::ID:
      return chatEventForumTopicEdited::fetch(env, p);
    case chatEventForumTopicToggleIsClosed::ID:
      return chatEventForumTopicToggleIsClosed::fetch(env, p);
    case chatEventForumTopicToggleIsHidden::ID:
      return chatEventForumTopicToggleIsHidden::fetch(env, p);
    case chatEventForumTopicDeleted::ID:
      return chatEventForumTopicDeleted::fetch(env, p);
    case chatEventForumTopicPinned::ID:
      return chatEventForumTopicPinned::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatEventAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventAction").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMessageEdited::Class;
jfieldID chatEventMessageEdited::old_message_fieldID;
jfieldID chatEventMessageEdited::new_message_fieldID;

chatEventMessageEdited::chatEventMessageEdited()
  : old_message_()
  , new_message_()
{}

chatEventMessageEdited::chatEventMessageEdited(object_ptr<message> &&old_message_, object_ptr<message> &&new_message_)
  : old_message_(std::move(old_message_))
  , new_message_(std::move(new_message_))
{}

const std::int32_t chatEventMessageEdited::ID;

object_ptr<ChatEventAction> chatEventMessageEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMessageEdited> res = make_object<chatEventMessageEdited>();
  res->old_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->old_message_fieldID));
  res->new_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->new_message_fieldID));
  return std::move(res);
}

void chatEventMessageEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_message_ != nullptr) { jobject next; old_message_->store(env, next); if (next) { env->SetObjectField(s, old_message_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_message_ != nullptr) { jobject next; new_message_->store(env, next); if (next) { env->SetObjectField(s, new_message_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMessageEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMessageEdited");
    s.store_object_field("oldMessage", static_cast<const BaseObject *>(old_message_.get()));
    s.store_object_field("newMessage", static_cast<const BaseObject *>(new_message_.get()));
    s.store_class_end();
  }
}

void chatEventMessageEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMessageEdited").c_str());
    old_message_fieldID = jni::get_field_id(env, Class, "oldMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    new_message_fieldID = jni::get_field_id(env, Class, "newMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMessageDeleted::Class;
jfieldID chatEventMessageDeleted::message_fieldID;
jfieldID chatEventMessageDeleted::can_report_anti_spam_false_positive_fieldID;

chatEventMessageDeleted::chatEventMessageDeleted()
  : message_()
  , can_report_anti_spam_false_positive_()
{}

chatEventMessageDeleted::chatEventMessageDeleted(object_ptr<message> &&message_, bool can_report_anti_spam_false_positive_)
  : message_(std::move(message_))
  , can_report_anti_spam_false_positive_(can_report_anti_spam_false_positive_)
{}

const std::int32_t chatEventMessageDeleted::ID;

object_ptr<ChatEventAction> chatEventMessageDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMessageDeleted> res = make_object<chatEventMessageDeleted>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->can_report_anti_spam_false_positive_ = (env->GetBooleanField(p, res->can_report_anti_spam_false_positive_fieldID) != 0);
  return std::move(res);
}

void chatEventMessageDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_report_anti_spam_false_positive_fieldID, can_report_anti_spam_false_positive_);
}

void chatEventMessageDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMessageDeleted");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("canReportAntiSpamFalsePositive", can_report_anti_spam_false_positive_);
    s.store_class_end();
  }
}

void chatEventMessageDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMessageDeleted").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    can_report_anti_spam_false_positive_fieldID = jni::get_field_id(env, Class, "canReportAntiSpamFalsePositive", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMessagePinned::Class;
jfieldID chatEventMessagePinned::message_fieldID;

chatEventMessagePinned::chatEventMessagePinned()
  : message_()
{}

chatEventMessagePinned::chatEventMessagePinned(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t chatEventMessagePinned::ID;

object_ptr<ChatEventAction> chatEventMessagePinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMessagePinned> res = make_object<chatEventMessagePinned>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void chatEventMessagePinned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMessagePinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMessagePinned");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void chatEventMessagePinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMessagePinned").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMessageUnpinned::Class;
jfieldID chatEventMessageUnpinned::message_fieldID;

chatEventMessageUnpinned::chatEventMessageUnpinned()
  : message_()
{}

chatEventMessageUnpinned::chatEventMessageUnpinned(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t chatEventMessageUnpinned::ID;

object_ptr<ChatEventAction> chatEventMessageUnpinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMessageUnpinned> res = make_object<chatEventMessageUnpinned>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void chatEventMessageUnpinned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMessageUnpinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMessageUnpinned");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void chatEventMessageUnpinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMessageUnpinned").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventPollStopped::Class;
jfieldID chatEventPollStopped::message_fieldID;

chatEventPollStopped::chatEventPollStopped()
  : message_()
{}

chatEventPollStopped::chatEventPollStopped(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t chatEventPollStopped::ID;

object_ptr<ChatEventAction> chatEventPollStopped::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventPollStopped> res = make_object<chatEventPollStopped>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void chatEventPollStopped::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventPollStopped::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventPollStopped");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void chatEventPollStopped::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventPollStopped").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberJoined::Class;

chatEventMemberJoined::chatEventMemberJoined() {
}

const std::int32_t chatEventMemberJoined::ID;

object_ptr<ChatEventAction> chatEventMemberJoined::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatEventMemberJoined>();
}

void chatEventMemberJoined::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatEventMemberJoined::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberJoined");
    s.store_class_end();
  }
}

void chatEventMemberJoined::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberJoined").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberJoinedByInviteLink::Class;
jfieldID chatEventMemberJoinedByInviteLink::invite_link_fieldID;
jfieldID chatEventMemberJoinedByInviteLink::via_chat_folder_invite_link_fieldID;

chatEventMemberJoinedByInviteLink::chatEventMemberJoinedByInviteLink()
  : invite_link_()
  , via_chat_folder_invite_link_()
{}

chatEventMemberJoinedByInviteLink::chatEventMemberJoinedByInviteLink(object_ptr<chatInviteLink> &&invite_link_, bool via_chat_folder_invite_link_)
  : invite_link_(std::move(invite_link_))
  , via_chat_folder_invite_link_(via_chat_folder_invite_link_)
{}

const std::int32_t chatEventMemberJoinedByInviteLink::ID;

object_ptr<ChatEventAction> chatEventMemberJoinedByInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberJoinedByInviteLink> res = make_object<chatEventMemberJoinedByInviteLink>();
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  res->via_chat_folder_invite_link_ = (env->GetBooleanField(p, res->via_chat_folder_invite_link_fieldID) != 0);
  return std::move(res);
}

void chatEventMemberJoinedByInviteLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, via_chat_folder_invite_link_fieldID, via_chat_folder_invite_link_);
}

void chatEventMemberJoinedByInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberJoinedByInviteLink");
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_field("viaChatFolderInviteLink", via_chat_folder_invite_link_);
    s.store_class_end();
  }
}

void chatEventMemberJoinedByInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberJoinedByInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    via_chat_folder_invite_link_fieldID = jni::get_field_id(env, Class, "viaChatFolderInviteLink", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberJoinedByRequest::Class;
jfieldID chatEventMemberJoinedByRequest::approver_user_id_fieldID;
jfieldID chatEventMemberJoinedByRequest::invite_link_fieldID;

chatEventMemberJoinedByRequest::chatEventMemberJoinedByRequest()
  : approver_user_id_()
  , invite_link_()
{}

chatEventMemberJoinedByRequest::chatEventMemberJoinedByRequest(int53 approver_user_id_, object_ptr<chatInviteLink> &&invite_link_)
  : approver_user_id_(approver_user_id_)
  , invite_link_(std::move(invite_link_))
{}

const std::int32_t chatEventMemberJoinedByRequest::ID;

object_ptr<ChatEventAction> chatEventMemberJoinedByRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberJoinedByRequest> res = make_object<chatEventMemberJoinedByRequest>();
  res->approver_user_id_ = env->GetLongField(p, res->approver_user_id_fieldID);
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  return std::move(res);
}

void chatEventMemberJoinedByRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, approver_user_id_fieldID, approver_user_id_);
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMemberJoinedByRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberJoinedByRequest");
    s.store_field("approverUserId", approver_user_id_);
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_class_end();
  }
}

void chatEventMemberJoinedByRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberJoinedByRequest").c_str());
    approver_user_id_fieldID = jni::get_field_id(env, Class, "approverUserId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberInvited::Class;
jfieldID chatEventMemberInvited::user_id_fieldID;
jfieldID chatEventMemberInvited::status_fieldID;

chatEventMemberInvited::chatEventMemberInvited()
  : user_id_()
  , status_()
{}

chatEventMemberInvited::chatEventMemberInvited(int53 user_id_, object_ptr<ChatMemberStatus> &&status_)
  : user_id_(user_id_)
  , status_(std::move(status_))
{}

const std::int32_t chatEventMemberInvited::ID;

object_ptr<ChatEventAction> chatEventMemberInvited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberInvited> res = make_object<chatEventMemberInvited>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  return std::move(res);
}

void chatEventMemberInvited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMemberInvited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberInvited");
    s.store_field("userId", user_id_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_class_end();
  }
}

void chatEventMemberInvited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberInvited").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberLeft::Class;

chatEventMemberLeft::chatEventMemberLeft() {
}

const std::int32_t chatEventMemberLeft::ID;

object_ptr<ChatEventAction> chatEventMemberLeft::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatEventMemberLeft>();
}

void chatEventMemberLeft::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatEventMemberLeft::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberLeft");
    s.store_class_end();
  }
}

void chatEventMemberLeft::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberLeft").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberPromoted::Class;
jfieldID chatEventMemberPromoted::user_id_fieldID;
jfieldID chatEventMemberPromoted::old_status_fieldID;
jfieldID chatEventMemberPromoted::new_status_fieldID;

chatEventMemberPromoted::chatEventMemberPromoted()
  : user_id_()
  , old_status_()
  , new_status_()
{}

chatEventMemberPromoted::chatEventMemberPromoted(int53 user_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_)
  : user_id_(user_id_)
  , old_status_(std::move(old_status_))
  , new_status_(std::move(new_status_))
{}

const std::int32_t chatEventMemberPromoted::ID;

object_ptr<ChatEventAction> chatEventMemberPromoted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberPromoted> res = make_object<chatEventMemberPromoted>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->old_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->old_status_fieldID));
  res->new_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->new_status_fieldID));
  return std::move(res);
}

void chatEventMemberPromoted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (old_status_ != nullptr) { jobject next; old_status_->store(env, next); if (next) { env->SetObjectField(s, old_status_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_status_ != nullptr) { jobject next; new_status_->store(env, next); if (next) { env->SetObjectField(s, new_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMemberPromoted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberPromoted");
    s.store_field("userId", user_id_);
    s.store_object_field("oldStatus", static_cast<const BaseObject *>(old_status_.get()));
    s.store_object_field("newStatus", static_cast<const BaseObject *>(new_status_.get()));
    s.store_class_end();
  }
}

void chatEventMemberPromoted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberPromoted").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    old_status_fieldID = jni::get_field_id(env, Class, "oldStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    new_status_fieldID = jni::get_field_id(env, Class, "newStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberRestricted::Class;
jfieldID chatEventMemberRestricted::member_id_fieldID;
jfieldID chatEventMemberRestricted::old_status_fieldID;
jfieldID chatEventMemberRestricted::new_status_fieldID;

chatEventMemberRestricted::chatEventMemberRestricted()
  : member_id_()
  , old_status_()
  , new_status_()
{}

chatEventMemberRestricted::chatEventMemberRestricted(object_ptr<MessageSender> &&member_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_)
  : member_id_(std::move(member_id_))
  , old_status_(std::move(old_status_))
  , new_status_(std::move(new_status_))
{}

const std::int32_t chatEventMemberRestricted::ID;

object_ptr<ChatEventAction> chatEventMemberRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberRestricted> res = make_object<chatEventMemberRestricted>();
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  res->old_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->old_status_fieldID));
  res->new_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->new_status_fieldID));
  return std::move(res);
}

void chatEventMemberRestricted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (member_id_ != nullptr) { jobject next; member_id_->store(env, next); if (next) { env->SetObjectField(s, member_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (old_status_ != nullptr) { jobject next; old_status_->store(env, next); if (next) { env->SetObjectField(s, old_status_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_status_ != nullptr) { jobject next; new_status_->store(env, next); if (next) { env->SetObjectField(s, new_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMemberRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberRestricted");
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_object_field("oldStatus", static_cast<const BaseObject *>(old_status_.get()));
    s.store_object_field("newStatus", static_cast<const BaseObject *>(new_status_.get()));
    s.store_class_end();
  }
}

void chatEventMemberRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberRestricted").c_str());
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    old_status_fieldID = jni::get_field_id(env, Class, "oldStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    new_status_fieldID = jni::get_field_id(env, Class, "newStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMemberSubscriptionExtended::Class;
jfieldID chatEventMemberSubscriptionExtended::user_id_fieldID;
jfieldID chatEventMemberSubscriptionExtended::old_status_fieldID;
jfieldID chatEventMemberSubscriptionExtended::new_status_fieldID;

chatEventMemberSubscriptionExtended::chatEventMemberSubscriptionExtended()
  : user_id_()
  , old_status_()
  , new_status_()
{}

chatEventMemberSubscriptionExtended::chatEventMemberSubscriptionExtended(int53 user_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_)
  : user_id_(user_id_)
  , old_status_(std::move(old_status_))
  , new_status_(std::move(new_status_))
{}

const std::int32_t chatEventMemberSubscriptionExtended::ID;

object_ptr<ChatEventAction> chatEventMemberSubscriptionExtended::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMemberSubscriptionExtended> res = make_object<chatEventMemberSubscriptionExtended>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->old_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->old_status_fieldID));
  res->new_status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->new_status_fieldID));
  return std::move(res);
}

void chatEventMemberSubscriptionExtended::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (old_status_ != nullptr) { jobject next; old_status_->store(env, next); if (next) { env->SetObjectField(s, old_status_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_status_ != nullptr) { jobject next; new_status_->store(env, next); if (next) { env->SetObjectField(s, new_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventMemberSubscriptionExtended::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMemberSubscriptionExtended");
    s.store_field("userId", user_id_);
    s.store_object_field("oldStatus", static_cast<const BaseObject *>(old_status_.get()));
    s.store_object_field("newStatus", static_cast<const BaseObject *>(new_status_.get()));
    s.store_class_end();
  }
}

void chatEventMemberSubscriptionExtended::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMemberSubscriptionExtended").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    old_status_fieldID = jni::get_field_id(env, Class, "oldStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    new_status_fieldID = jni::get_field_id(env, Class, "newStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventAvailableReactionsChanged::Class;
jfieldID chatEventAvailableReactionsChanged::old_available_reactions_fieldID;
jfieldID chatEventAvailableReactionsChanged::new_available_reactions_fieldID;

chatEventAvailableReactionsChanged::chatEventAvailableReactionsChanged()
  : old_available_reactions_()
  , new_available_reactions_()
{}

chatEventAvailableReactionsChanged::chatEventAvailableReactionsChanged(object_ptr<ChatAvailableReactions> &&old_available_reactions_, object_ptr<ChatAvailableReactions> &&new_available_reactions_)
  : old_available_reactions_(std::move(old_available_reactions_))
  , new_available_reactions_(std::move(new_available_reactions_))
{}

const std::int32_t chatEventAvailableReactionsChanged::ID;

object_ptr<ChatEventAction> chatEventAvailableReactionsChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventAvailableReactionsChanged> res = make_object<chatEventAvailableReactionsChanged>();
  res->old_available_reactions_ = jni::fetch_tl_object<ChatAvailableReactions>(env, jni::fetch_object(env, p, res->old_available_reactions_fieldID));
  res->new_available_reactions_ = jni::fetch_tl_object<ChatAvailableReactions>(env, jni::fetch_object(env, p, res->new_available_reactions_fieldID));
  return std::move(res);
}

void chatEventAvailableReactionsChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_available_reactions_ != nullptr) { jobject next; old_available_reactions_->store(env, next); if (next) { env->SetObjectField(s, old_available_reactions_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_available_reactions_ != nullptr) { jobject next; new_available_reactions_->store(env, next); if (next) { env->SetObjectField(s, new_available_reactions_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventAvailableReactionsChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventAvailableReactionsChanged");
    s.store_object_field("oldAvailableReactions", static_cast<const BaseObject *>(old_available_reactions_.get()));
    s.store_object_field("newAvailableReactions", static_cast<const BaseObject *>(new_available_reactions_.get()));
    s.store_class_end();
  }
}

void chatEventAvailableReactionsChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventAvailableReactionsChanged").c_str());
    old_available_reactions_fieldID = jni::get_field_id(env, Class, "oldAvailableReactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAvailableReactions;").c_str());
    new_available_reactions_fieldID = jni::get_field_id(env, Class, "newAvailableReactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAvailableReactions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventBackgroundChanged::Class;
jfieldID chatEventBackgroundChanged::old_background_fieldID;
jfieldID chatEventBackgroundChanged::new_background_fieldID;

chatEventBackgroundChanged::chatEventBackgroundChanged()
  : old_background_()
  , new_background_()
{}

chatEventBackgroundChanged::chatEventBackgroundChanged(object_ptr<chatBackground> &&old_background_, object_ptr<chatBackground> &&new_background_)
  : old_background_(std::move(old_background_))
  , new_background_(std::move(new_background_))
{}

const std::int32_t chatEventBackgroundChanged::ID;

object_ptr<ChatEventAction> chatEventBackgroundChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventBackgroundChanged> res = make_object<chatEventBackgroundChanged>();
  res->old_background_ = jni::fetch_tl_object<chatBackground>(env, jni::fetch_object(env, p, res->old_background_fieldID));
  res->new_background_ = jni::fetch_tl_object<chatBackground>(env, jni::fetch_object(env, p, res->new_background_fieldID));
  return std::move(res);
}

void chatEventBackgroundChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_background_ != nullptr) { jobject next; old_background_->store(env, next); if (next) { env->SetObjectField(s, old_background_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_background_ != nullptr) { jobject next; new_background_->store(env, next); if (next) { env->SetObjectField(s, new_background_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventBackgroundChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventBackgroundChanged");
    s.store_object_field("oldBackground", static_cast<const BaseObject *>(old_background_.get()));
    s.store_object_field("newBackground", static_cast<const BaseObject *>(new_background_.get()));
    s.store_class_end();
  }
}

void chatEventBackgroundChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventBackgroundChanged").c_str());
    old_background_fieldID = jni::get_field_id(env, Class, "oldBackground", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBackground;").c_str());
    new_background_fieldID = jni::get_field_id(env, Class, "newBackground", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBackground;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventDescriptionChanged::Class;
jfieldID chatEventDescriptionChanged::old_description_fieldID;
jfieldID chatEventDescriptionChanged::new_description_fieldID;

chatEventDescriptionChanged::chatEventDescriptionChanged()
  : old_description_()
  , new_description_()
{}

chatEventDescriptionChanged::chatEventDescriptionChanged(string const &old_description_, string const &new_description_)
  : old_description_(old_description_)
  , new_description_(new_description_)
{}

const std::int32_t chatEventDescriptionChanged::ID;

object_ptr<ChatEventAction> chatEventDescriptionChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventDescriptionChanged> res = make_object<chatEventDescriptionChanged>();
  res->old_description_ = jni::fetch_string(env, p, res->old_description_fieldID);
  res->new_description_ = jni::fetch_string(env, p, res->new_description_fieldID);
  return std::move(res);
}

void chatEventDescriptionChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, old_description_); if (nextString) { env->SetObjectField(s, old_description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, new_description_); if (nextString) { env->SetObjectField(s, new_description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatEventDescriptionChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventDescriptionChanged");
    s.store_field("oldDescription", old_description_);
    s.store_field("newDescription", new_description_);
    s.store_class_end();
  }
}

void chatEventDescriptionChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventDescriptionChanged").c_str());
    old_description_fieldID = jni::get_field_id(env, Class, "oldDescription", "Ljava/lang/String;");
    new_description_fieldID = jni::get_field_id(env, Class, "newDescription", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventEmojiStatusChanged::Class;
jfieldID chatEventEmojiStatusChanged::old_emoji_status_fieldID;
jfieldID chatEventEmojiStatusChanged::new_emoji_status_fieldID;

chatEventEmojiStatusChanged::chatEventEmojiStatusChanged()
  : old_emoji_status_()
  , new_emoji_status_()
{}

chatEventEmojiStatusChanged::chatEventEmojiStatusChanged(object_ptr<emojiStatus> &&old_emoji_status_, object_ptr<emojiStatus> &&new_emoji_status_)
  : old_emoji_status_(std::move(old_emoji_status_))
  , new_emoji_status_(std::move(new_emoji_status_))
{}

const std::int32_t chatEventEmojiStatusChanged::ID;

object_ptr<ChatEventAction> chatEventEmojiStatusChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventEmojiStatusChanged> res = make_object<chatEventEmojiStatusChanged>();
  res->old_emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->old_emoji_status_fieldID));
  res->new_emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->new_emoji_status_fieldID));
  return std::move(res);
}

void chatEventEmojiStatusChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_emoji_status_ != nullptr) { jobject next; old_emoji_status_->store(env, next); if (next) { env->SetObjectField(s, old_emoji_status_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_emoji_status_ != nullptr) { jobject next; new_emoji_status_->store(env, next); if (next) { env->SetObjectField(s, new_emoji_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventEmojiStatusChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventEmojiStatusChanged");
    s.store_object_field("oldEmojiStatus", static_cast<const BaseObject *>(old_emoji_status_.get()));
    s.store_object_field("newEmojiStatus", static_cast<const BaseObject *>(new_emoji_status_.get()));
    s.store_class_end();
  }
}

void chatEventEmojiStatusChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventEmojiStatusChanged").c_str());
    old_emoji_status_fieldID = jni::get_field_id(env, Class, "oldEmojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    new_emoji_status_fieldID = jni::get_field_id(env, Class, "newEmojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventLinkedChatChanged::Class;
jfieldID chatEventLinkedChatChanged::old_linked_chat_id_fieldID;
jfieldID chatEventLinkedChatChanged::new_linked_chat_id_fieldID;

chatEventLinkedChatChanged::chatEventLinkedChatChanged()
  : old_linked_chat_id_()
  , new_linked_chat_id_()
{}

chatEventLinkedChatChanged::chatEventLinkedChatChanged(int53 old_linked_chat_id_, int53 new_linked_chat_id_)
  : old_linked_chat_id_(old_linked_chat_id_)
  , new_linked_chat_id_(new_linked_chat_id_)
{}

const std::int32_t chatEventLinkedChatChanged::ID;

object_ptr<ChatEventAction> chatEventLinkedChatChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventLinkedChatChanged> res = make_object<chatEventLinkedChatChanged>();
  res->old_linked_chat_id_ = env->GetLongField(p, res->old_linked_chat_id_fieldID);
  res->new_linked_chat_id_ = env->GetLongField(p, res->new_linked_chat_id_fieldID);
  return std::move(res);
}

void chatEventLinkedChatChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, old_linked_chat_id_fieldID, old_linked_chat_id_);
  env->SetLongField(s, new_linked_chat_id_fieldID, new_linked_chat_id_);
}

void chatEventLinkedChatChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventLinkedChatChanged");
    s.store_field("oldLinkedChatId", old_linked_chat_id_);
    s.store_field("newLinkedChatId", new_linked_chat_id_);
    s.store_class_end();
  }
}

void chatEventLinkedChatChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventLinkedChatChanged").c_str());
    old_linked_chat_id_fieldID = jni::get_field_id(env, Class, "oldLinkedChatId", "J");
    new_linked_chat_id_fieldID = jni::get_field_id(env, Class, "newLinkedChatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventLocationChanged::Class;
jfieldID chatEventLocationChanged::old_location_fieldID;
jfieldID chatEventLocationChanged::new_location_fieldID;

chatEventLocationChanged::chatEventLocationChanged()
  : old_location_()
  , new_location_()
{}

chatEventLocationChanged::chatEventLocationChanged(object_ptr<chatLocation> &&old_location_, object_ptr<chatLocation> &&new_location_)
  : old_location_(std::move(old_location_))
  , new_location_(std::move(new_location_))
{}

const std::int32_t chatEventLocationChanged::ID;

object_ptr<ChatEventAction> chatEventLocationChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventLocationChanged> res = make_object<chatEventLocationChanged>();
  res->old_location_ = jni::fetch_tl_object<chatLocation>(env, jni::fetch_object(env, p, res->old_location_fieldID));
  res->new_location_ = jni::fetch_tl_object<chatLocation>(env, jni::fetch_object(env, p, res->new_location_fieldID));
  return std::move(res);
}

void chatEventLocationChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_location_ != nullptr) { jobject next; old_location_->store(env, next); if (next) { env->SetObjectField(s, old_location_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_location_ != nullptr) { jobject next; new_location_->store(env, next); if (next) { env->SetObjectField(s, new_location_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventLocationChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventLocationChanged");
    s.store_object_field("oldLocation", static_cast<const BaseObject *>(old_location_.get()));
    s.store_object_field("newLocation", static_cast<const BaseObject *>(new_location_.get()));
    s.store_class_end();
  }
}

void chatEventLocationChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventLocationChanged").c_str());
    old_location_fieldID = jni::get_field_id(env, Class, "oldLocation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatLocation;").c_str());
    new_location_fieldID = jni::get_field_id(env, Class, "newLocation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatLocation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventMessageAutoDeleteTimeChanged::Class;
jfieldID chatEventMessageAutoDeleteTimeChanged::old_message_auto_delete_time_fieldID;
jfieldID chatEventMessageAutoDeleteTimeChanged::new_message_auto_delete_time_fieldID;

chatEventMessageAutoDeleteTimeChanged::chatEventMessageAutoDeleteTimeChanged()
  : old_message_auto_delete_time_()
  , new_message_auto_delete_time_()
{}

chatEventMessageAutoDeleteTimeChanged::chatEventMessageAutoDeleteTimeChanged(int32 old_message_auto_delete_time_, int32 new_message_auto_delete_time_)
  : old_message_auto_delete_time_(old_message_auto_delete_time_)
  , new_message_auto_delete_time_(new_message_auto_delete_time_)
{}

const std::int32_t chatEventMessageAutoDeleteTimeChanged::ID;

object_ptr<ChatEventAction> chatEventMessageAutoDeleteTimeChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventMessageAutoDeleteTimeChanged> res = make_object<chatEventMessageAutoDeleteTimeChanged>();
  res->old_message_auto_delete_time_ = env->GetIntField(p, res->old_message_auto_delete_time_fieldID);
  res->new_message_auto_delete_time_ = env->GetIntField(p, res->new_message_auto_delete_time_fieldID);
  return std::move(res);
}

void chatEventMessageAutoDeleteTimeChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, old_message_auto_delete_time_fieldID, old_message_auto_delete_time_);
  env->SetIntField(s, new_message_auto_delete_time_fieldID, new_message_auto_delete_time_);
}

void chatEventMessageAutoDeleteTimeChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventMessageAutoDeleteTimeChanged");
    s.store_field("oldMessageAutoDeleteTime", old_message_auto_delete_time_);
    s.store_field("newMessageAutoDeleteTime", new_message_auto_delete_time_);
    s.store_class_end();
  }
}

void chatEventMessageAutoDeleteTimeChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventMessageAutoDeleteTimeChanged").c_str());
    old_message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "oldMessageAutoDeleteTime", "I");
    new_message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "newMessageAutoDeleteTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventPermissionsChanged::Class;
jfieldID chatEventPermissionsChanged::old_permissions_fieldID;
jfieldID chatEventPermissionsChanged::new_permissions_fieldID;

chatEventPermissionsChanged::chatEventPermissionsChanged()
  : old_permissions_()
  , new_permissions_()
{}

chatEventPermissionsChanged::chatEventPermissionsChanged(object_ptr<chatPermissions> &&old_permissions_, object_ptr<chatPermissions> &&new_permissions_)
  : old_permissions_(std::move(old_permissions_))
  , new_permissions_(std::move(new_permissions_))
{}

const std::int32_t chatEventPermissionsChanged::ID;

object_ptr<ChatEventAction> chatEventPermissionsChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventPermissionsChanged> res = make_object<chatEventPermissionsChanged>();
  res->old_permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->old_permissions_fieldID));
  res->new_permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->new_permissions_fieldID));
  return std::move(res);
}

void chatEventPermissionsChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_permissions_ != nullptr) { jobject next; old_permissions_->store(env, next); if (next) { env->SetObjectField(s, old_permissions_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_permissions_ != nullptr) { jobject next; new_permissions_->store(env, next); if (next) { env->SetObjectField(s, new_permissions_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventPermissionsChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventPermissionsChanged");
    s.store_object_field("oldPermissions", static_cast<const BaseObject *>(old_permissions_.get()));
    s.store_object_field("newPermissions", static_cast<const BaseObject *>(new_permissions_.get()));
    s.store_class_end();
  }
}

void chatEventPermissionsChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventPermissionsChanged").c_str());
    old_permissions_fieldID = jni::get_field_id(env, Class, "oldPermissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    new_permissions_fieldID = jni::get_field_id(env, Class, "newPermissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventPhotoChanged::Class;
jfieldID chatEventPhotoChanged::old_photo_fieldID;
jfieldID chatEventPhotoChanged::new_photo_fieldID;

chatEventPhotoChanged::chatEventPhotoChanged()
  : old_photo_()
  , new_photo_()
{}

chatEventPhotoChanged::chatEventPhotoChanged(object_ptr<chatPhoto> &&old_photo_, object_ptr<chatPhoto> &&new_photo_)
  : old_photo_(std::move(old_photo_))
  , new_photo_(std::move(new_photo_))
{}

const std::int32_t chatEventPhotoChanged::ID;

object_ptr<ChatEventAction> chatEventPhotoChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventPhotoChanged> res = make_object<chatEventPhotoChanged>();
  res->old_photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->old_photo_fieldID));
  res->new_photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->new_photo_fieldID));
  return std::move(res);
}

void chatEventPhotoChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_photo_ != nullptr) { jobject next; old_photo_->store(env, next); if (next) { env->SetObjectField(s, old_photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_photo_ != nullptr) { jobject next; new_photo_->store(env, next); if (next) { env->SetObjectField(s, new_photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventPhotoChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventPhotoChanged");
    s.store_object_field("oldPhoto", static_cast<const BaseObject *>(old_photo_.get()));
    s.store_object_field("newPhoto", static_cast<const BaseObject *>(new_photo_.get()));
    s.store_class_end();
  }
}

void chatEventPhotoChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventPhotoChanged").c_str());
    old_photo_fieldID = jni::get_field_id(env, Class, "oldPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    new_photo_fieldID = jni::get_field_id(env, Class, "newPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventSlowModeDelayChanged::Class;
jfieldID chatEventSlowModeDelayChanged::old_slow_mode_delay_fieldID;
jfieldID chatEventSlowModeDelayChanged::new_slow_mode_delay_fieldID;

chatEventSlowModeDelayChanged::chatEventSlowModeDelayChanged()
  : old_slow_mode_delay_()
  , new_slow_mode_delay_()
{}

chatEventSlowModeDelayChanged::chatEventSlowModeDelayChanged(int32 old_slow_mode_delay_, int32 new_slow_mode_delay_)
  : old_slow_mode_delay_(old_slow_mode_delay_)
  , new_slow_mode_delay_(new_slow_mode_delay_)
{}

const std::int32_t chatEventSlowModeDelayChanged::ID;

object_ptr<ChatEventAction> chatEventSlowModeDelayChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventSlowModeDelayChanged> res = make_object<chatEventSlowModeDelayChanged>();
  res->old_slow_mode_delay_ = env->GetIntField(p, res->old_slow_mode_delay_fieldID);
  res->new_slow_mode_delay_ = env->GetIntField(p, res->new_slow_mode_delay_fieldID);
  return std::move(res);
}

void chatEventSlowModeDelayChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, old_slow_mode_delay_fieldID, old_slow_mode_delay_);
  env->SetIntField(s, new_slow_mode_delay_fieldID, new_slow_mode_delay_);
}

void chatEventSlowModeDelayChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventSlowModeDelayChanged");
    s.store_field("oldSlowModeDelay", old_slow_mode_delay_);
    s.store_field("newSlowModeDelay", new_slow_mode_delay_);
    s.store_class_end();
  }
}

void chatEventSlowModeDelayChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventSlowModeDelayChanged").c_str());
    old_slow_mode_delay_fieldID = jni::get_field_id(env, Class, "oldSlowModeDelay", "I");
    new_slow_mode_delay_fieldID = jni::get_field_id(env, Class, "newSlowModeDelay", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventStickerSetChanged::Class;
jfieldID chatEventStickerSetChanged::old_sticker_set_id_fieldID;
jfieldID chatEventStickerSetChanged::new_sticker_set_id_fieldID;

chatEventStickerSetChanged::chatEventStickerSetChanged()
  : old_sticker_set_id_()
  , new_sticker_set_id_()
{}

chatEventStickerSetChanged::chatEventStickerSetChanged(int64 old_sticker_set_id_, int64 new_sticker_set_id_)
  : old_sticker_set_id_(old_sticker_set_id_)
  , new_sticker_set_id_(new_sticker_set_id_)
{}

const std::int32_t chatEventStickerSetChanged::ID;

object_ptr<ChatEventAction> chatEventStickerSetChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventStickerSetChanged> res = make_object<chatEventStickerSetChanged>();
  res->old_sticker_set_id_ = env->GetLongField(p, res->old_sticker_set_id_fieldID);
  res->new_sticker_set_id_ = env->GetLongField(p, res->new_sticker_set_id_fieldID);
  return std::move(res);
}

void chatEventStickerSetChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, old_sticker_set_id_fieldID, old_sticker_set_id_);
  env->SetLongField(s, new_sticker_set_id_fieldID, new_sticker_set_id_);
}

void chatEventStickerSetChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventStickerSetChanged");
    s.store_field("oldStickerSetId", old_sticker_set_id_);
    s.store_field("newStickerSetId", new_sticker_set_id_);
    s.store_class_end();
  }
}

void chatEventStickerSetChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventStickerSetChanged").c_str());
    old_sticker_set_id_fieldID = jni::get_field_id(env, Class, "oldStickerSetId", "J");
    new_sticker_set_id_fieldID = jni::get_field_id(env, Class, "newStickerSetId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventCustomEmojiStickerSetChanged::Class;
jfieldID chatEventCustomEmojiStickerSetChanged::old_sticker_set_id_fieldID;
jfieldID chatEventCustomEmojiStickerSetChanged::new_sticker_set_id_fieldID;

chatEventCustomEmojiStickerSetChanged::chatEventCustomEmojiStickerSetChanged()
  : old_sticker_set_id_()
  , new_sticker_set_id_()
{}

chatEventCustomEmojiStickerSetChanged::chatEventCustomEmojiStickerSetChanged(int64 old_sticker_set_id_, int64 new_sticker_set_id_)
  : old_sticker_set_id_(old_sticker_set_id_)
  , new_sticker_set_id_(new_sticker_set_id_)
{}

const std::int32_t chatEventCustomEmojiStickerSetChanged::ID;

object_ptr<ChatEventAction> chatEventCustomEmojiStickerSetChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventCustomEmojiStickerSetChanged> res = make_object<chatEventCustomEmojiStickerSetChanged>();
  res->old_sticker_set_id_ = env->GetLongField(p, res->old_sticker_set_id_fieldID);
  res->new_sticker_set_id_ = env->GetLongField(p, res->new_sticker_set_id_fieldID);
  return std::move(res);
}

void chatEventCustomEmojiStickerSetChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, old_sticker_set_id_fieldID, old_sticker_set_id_);
  env->SetLongField(s, new_sticker_set_id_fieldID, new_sticker_set_id_);
}

void chatEventCustomEmojiStickerSetChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventCustomEmojiStickerSetChanged");
    s.store_field("oldStickerSetId", old_sticker_set_id_);
    s.store_field("newStickerSetId", new_sticker_set_id_);
    s.store_class_end();
  }
}

void chatEventCustomEmojiStickerSetChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventCustomEmojiStickerSetChanged").c_str());
    old_sticker_set_id_fieldID = jni::get_field_id(env, Class, "oldStickerSetId", "J");
    new_sticker_set_id_fieldID = jni::get_field_id(env, Class, "newStickerSetId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventTitleChanged::Class;
jfieldID chatEventTitleChanged::old_title_fieldID;
jfieldID chatEventTitleChanged::new_title_fieldID;

chatEventTitleChanged::chatEventTitleChanged()
  : old_title_()
  , new_title_()
{}

chatEventTitleChanged::chatEventTitleChanged(string const &old_title_, string const &new_title_)
  : old_title_(old_title_)
  , new_title_(new_title_)
{}

const std::int32_t chatEventTitleChanged::ID;

object_ptr<ChatEventAction> chatEventTitleChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventTitleChanged> res = make_object<chatEventTitleChanged>();
  res->old_title_ = jni::fetch_string(env, p, res->old_title_fieldID);
  res->new_title_ = jni::fetch_string(env, p, res->new_title_fieldID);
  return std::move(res);
}

void chatEventTitleChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, old_title_); if (nextString) { env->SetObjectField(s, old_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, new_title_); if (nextString) { env->SetObjectField(s, new_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatEventTitleChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventTitleChanged");
    s.store_field("oldTitle", old_title_);
    s.store_field("newTitle", new_title_);
    s.store_class_end();
  }
}

void chatEventTitleChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventTitleChanged").c_str());
    old_title_fieldID = jni::get_field_id(env, Class, "oldTitle", "Ljava/lang/String;");
    new_title_fieldID = jni::get_field_id(env, Class, "newTitle", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventUsernameChanged::Class;
jfieldID chatEventUsernameChanged::old_username_fieldID;
jfieldID chatEventUsernameChanged::new_username_fieldID;

chatEventUsernameChanged::chatEventUsernameChanged()
  : old_username_()
  , new_username_()
{}

chatEventUsernameChanged::chatEventUsernameChanged(string const &old_username_, string const &new_username_)
  : old_username_(old_username_)
  , new_username_(new_username_)
{}

const std::int32_t chatEventUsernameChanged::ID;

object_ptr<ChatEventAction> chatEventUsernameChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventUsernameChanged> res = make_object<chatEventUsernameChanged>();
  res->old_username_ = jni::fetch_string(env, p, res->old_username_fieldID);
  res->new_username_ = jni::fetch_string(env, p, res->new_username_fieldID);
  return std::move(res);
}

void chatEventUsernameChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, old_username_); if (nextString) { env->SetObjectField(s, old_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, new_username_); if (nextString) { env->SetObjectField(s, new_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatEventUsernameChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventUsernameChanged");
    s.store_field("oldUsername", old_username_);
    s.store_field("newUsername", new_username_);
    s.store_class_end();
  }
}

void chatEventUsernameChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventUsernameChanged").c_str());
    old_username_fieldID = jni::get_field_id(env, Class, "oldUsername", "Ljava/lang/String;");
    new_username_fieldID = jni::get_field_id(env, Class, "newUsername", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventActiveUsernamesChanged::Class;
jfieldID chatEventActiveUsernamesChanged::old_usernames_fieldID;
jfieldID chatEventActiveUsernamesChanged::new_usernames_fieldID;

chatEventActiveUsernamesChanged::chatEventActiveUsernamesChanged()
  : old_usernames_()
  , new_usernames_()
{}

chatEventActiveUsernamesChanged::chatEventActiveUsernamesChanged(array<string> &&old_usernames_, array<string> &&new_usernames_)
  : old_usernames_(std::move(old_usernames_))
  , new_usernames_(std::move(new_usernames_))
{}

const std::int32_t chatEventActiveUsernamesChanged::ID;

object_ptr<ChatEventAction> chatEventActiveUsernamesChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventActiveUsernamesChanged> res = make_object<chatEventActiveUsernamesChanged>();
  res->old_usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->old_usernames_fieldID));
  res->new_usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->new_usernames_fieldID));
  return std::move(res);
}

void chatEventActiveUsernamesChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, old_usernames_); if (arr_tmp_) { env->SetObjectField(s, old_usernames_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, new_usernames_); if (arr_tmp_) { env->SetObjectField(s, new_usernames_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatEventActiveUsernamesChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventActiveUsernamesChanged");
    { s.store_vector_begin("oldUsernames", old_usernames_.size()); for (const auto &_value : old_usernames_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("newUsernames", new_usernames_.size()); for (const auto &_value : new_usernames_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatEventActiveUsernamesChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventActiveUsernamesChanged").c_str());
    old_usernames_fieldID = jni::get_field_id(env, Class, "oldUsernames", "[Ljava/lang/String;");
    new_usernames_fieldID = jni::get_field_id(env, Class, "newUsernames", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventAccentColorChanged::Class;
jfieldID chatEventAccentColorChanged::old_accent_color_id_fieldID;
jfieldID chatEventAccentColorChanged::old_background_custom_emoji_id_fieldID;
jfieldID chatEventAccentColorChanged::new_accent_color_id_fieldID;
jfieldID chatEventAccentColorChanged::new_background_custom_emoji_id_fieldID;

chatEventAccentColorChanged::chatEventAccentColorChanged()
  : old_accent_color_id_()
  , old_background_custom_emoji_id_()
  , new_accent_color_id_()
  , new_background_custom_emoji_id_()
{}

chatEventAccentColorChanged::chatEventAccentColorChanged(int32 old_accent_color_id_, int64 old_background_custom_emoji_id_, int32 new_accent_color_id_, int64 new_background_custom_emoji_id_)
  : old_accent_color_id_(old_accent_color_id_)
  , old_background_custom_emoji_id_(old_background_custom_emoji_id_)
  , new_accent_color_id_(new_accent_color_id_)
  , new_background_custom_emoji_id_(new_background_custom_emoji_id_)
{}

const std::int32_t chatEventAccentColorChanged::ID;

object_ptr<ChatEventAction> chatEventAccentColorChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventAccentColorChanged> res = make_object<chatEventAccentColorChanged>();
  res->old_accent_color_id_ = env->GetIntField(p, res->old_accent_color_id_fieldID);
  res->old_background_custom_emoji_id_ = env->GetLongField(p, res->old_background_custom_emoji_id_fieldID);
  res->new_accent_color_id_ = env->GetIntField(p, res->new_accent_color_id_fieldID);
  res->new_background_custom_emoji_id_ = env->GetLongField(p, res->new_background_custom_emoji_id_fieldID);
  return std::move(res);
}

void chatEventAccentColorChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, old_accent_color_id_fieldID, old_accent_color_id_);
  env->SetLongField(s, old_background_custom_emoji_id_fieldID, old_background_custom_emoji_id_);
  env->SetIntField(s, new_accent_color_id_fieldID, new_accent_color_id_);
  env->SetLongField(s, new_background_custom_emoji_id_fieldID, new_background_custom_emoji_id_);
}

void chatEventAccentColorChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventAccentColorChanged");
    s.store_field("oldAccentColorId", old_accent_color_id_);
    s.store_field("oldBackgroundCustomEmojiId", old_background_custom_emoji_id_);
    s.store_field("newAccentColorId", new_accent_color_id_);
    s.store_field("newBackgroundCustomEmojiId", new_background_custom_emoji_id_);
    s.store_class_end();
  }
}

void chatEventAccentColorChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventAccentColorChanged").c_str());
    old_accent_color_id_fieldID = jni::get_field_id(env, Class, "oldAccentColorId", "I");
    old_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "oldBackgroundCustomEmojiId", "J");
    new_accent_color_id_fieldID = jni::get_field_id(env, Class, "newAccentColorId", "I");
    new_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "newBackgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventProfileAccentColorChanged::Class;
jfieldID chatEventProfileAccentColorChanged::old_profile_accent_color_id_fieldID;
jfieldID chatEventProfileAccentColorChanged::old_profile_background_custom_emoji_id_fieldID;
jfieldID chatEventProfileAccentColorChanged::new_profile_accent_color_id_fieldID;
jfieldID chatEventProfileAccentColorChanged::new_profile_background_custom_emoji_id_fieldID;

chatEventProfileAccentColorChanged::chatEventProfileAccentColorChanged()
  : old_profile_accent_color_id_()
  , old_profile_background_custom_emoji_id_()
  , new_profile_accent_color_id_()
  , new_profile_background_custom_emoji_id_()
{}

chatEventProfileAccentColorChanged::chatEventProfileAccentColorChanged(int32 old_profile_accent_color_id_, int64 old_profile_background_custom_emoji_id_, int32 new_profile_accent_color_id_, int64 new_profile_background_custom_emoji_id_)
  : old_profile_accent_color_id_(old_profile_accent_color_id_)
  , old_profile_background_custom_emoji_id_(old_profile_background_custom_emoji_id_)
  , new_profile_accent_color_id_(new_profile_accent_color_id_)
  , new_profile_background_custom_emoji_id_(new_profile_background_custom_emoji_id_)
{}

const std::int32_t chatEventProfileAccentColorChanged::ID;

object_ptr<ChatEventAction> chatEventProfileAccentColorChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventProfileAccentColorChanged> res = make_object<chatEventProfileAccentColorChanged>();
  res->old_profile_accent_color_id_ = env->GetIntField(p, res->old_profile_accent_color_id_fieldID);
  res->old_profile_background_custom_emoji_id_ = env->GetLongField(p, res->old_profile_background_custom_emoji_id_fieldID);
  res->new_profile_accent_color_id_ = env->GetIntField(p, res->new_profile_accent_color_id_fieldID);
  res->new_profile_background_custom_emoji_id_ = env->GetLongField(p, res->new_profile_background_custom_emoji_id_fieldID);
  return std::move(res);
}

void chatEventProfileAccentColorChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, old_profile_accent_color_id_fieldID, old_profile_accent_color_id_);
  env->SetLongField(s, old_profile_background_custom_emoji_id_fieldID, old_profile_background_custom_emoji_id_);
  env->SetIntField(s, new_profile_accent_color_id_fieldID, new_profile_accent_color_id_);
  env->SetLongField(s, new_profile_background_custom_emoji_id_fieldID, new_profile_background_custom_emoji_id_);
}

void chatEventProfileAccentColorChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventProfileAccentColorChanged");
    s.store_field("oldProfileAccentColorId", old_profile_accent_color_id_);
    s.store_field("oldProfileBackgroundCustomEmojiId", old_profile_background_custom_emoji_id_);
    s.store_field("newProfileAccentColorId", new_profile_accent_color_id_);
    s.store_field("newProfileBackgroundCustomEmojiId", new_profile_background_custom_emoji_id_);
    s.store_class_end();
  }
}

void chatEventProfileAccentColorChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventProfileAccentColorChanged").c_str());
    old_profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "oldProfileAccentColorId", "I");
    old_profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "oldProfileBackgroundCustomEmojiId", "J");
    new_profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "newProfileAccentColorId", "I");
    new_profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "newProfileBackgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventHasProtectedContentToggled::Class;
jfieldID chatEventHasProtectedContentToggled::has_protected_content_fieldID;

chatEventHasProtectedContentToggled::chatEventHasProtectedContentToggled()
  : has_protected_content_()
{}

chatEventHasProtectedContentToggled::chatEventHasProtectedContentToggled(bool has_protected_content_)
  : has_protected_content_(has_protected_content_)
{}

const std::int32_t chatEventHasProtectedContentToggled::ID;

object_ptr<ChatEventAction> chatEventHasProtectedContentToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventHasProtectedContentToggled> res = make_object<chatEventHasProtectedContentToggled>();
  res->has_protected_content_ = (env->GetBooleanField(p, res->has_protected_content_fieldID) != 0);
  return std::move(res);
}

void chatEventHasProtectedContentToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_protected_content_fieldID, has_protected_content_);
}

void chatEventHasProtectedContentToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventHasProtectedContentToggled");
    s.store_field("hasProtectedContent", has_protected_content_);
    s.store_class_end();
  }
}

void chatEventHasProtectedContentToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventHasProtectedContentToggled").c_str());
    has_protected_content_fieldID = jni::get_field_id(env, Class, "hasProtectedContent", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventInvitesToggled::Class;
jfieldID chatEventInvitesToggled::can_invite_users_fieldID;

chatEventInvitesToggled::chatEventInvitesToggled()
  : can_invite_users_()
{}

chatEventInvitesToggled::chatEventInvitesToggled(bool can_invite_users_)
  : can_invite_users_(can_invite_users_)
{}

const std::int32_t chatEventInvitesToggled::ID;

object_ptr<ChatEventAction> chatEventInvitesToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventInvitesToggled> res = make_object<chatEventInvitesToggled>();
  res->can_invite_users_ = (env->GetBooleanField(p, res->can_invite_users_fieldID) != 0);
  return std::move(res);
}

void chatEventInvitesToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_invite_users_fieldID, can_invite_users_);
}

void chatEventInvitesToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventInvitesToggled");
    s.store_field("canInviteUsers", can_invite_users_);
    s.store_class_end();
  }
}

void chatEventInvitesToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventInvitesToggled").c_str());
    can_invite_users_fieldID = jni::get_field_id(env, Class, "canInviteUsers", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventIsAllHistoryAvailableToggled::Class;
jfieldID chatEventIsAllHistoryAvailableToggled::is_all_history_available_fieldID;

chatEventIsAllHistoryAvailableToggled::chatEventIsAllHistoryAvailableToggled()
  : is_all_history_available_()
{}

chatEventIsAllHistoryAvailableToggled::chatEventIsAllHistoryAvailableToggled(bool is_all_history_available_)
  : is_all_history_available_(is_all_history_available_)
{}

const std::int32_t chatEventIsAllHistoryAvailableToggled::ID;

object_ptr<ChatEventAction> chatEventIsAllHistoryAvailableToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventIsAllHistoryAvailableToggled> res = make_object<chatEventIsAllHistoryAvailableToggled>();
  res->is_all_history_available_ = (env->GetBooleanField(p, res->is_all_history_available_fieldID) != 0);
  return std::move(res);
}

void chatEventIsAllHistoryAvailableToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_all_history_available_fieldID, is_all_history_available_);
}

void chatEventIsAllHistoryAvailableToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventIsAllHistoryAvailableToggled");
    s.store_field("isAllHistoryAvailable", is_all_history_available_);
    s.store_class_end();
  }
}

void chatEventIsAllHistoryAvailableToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventIsAllHistoryAvailableToggled").c_str());
    is_all_history_available_fieldID = jni::get_field_id(env, Class, "isAllHistoryAvailable", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventHasAggressiveAntiSpamEnabledToggled::Class;
jfieldID chatEventHasAggressiveAntiSpamEnabledToggled::has_aggressive_anti_spam_enabled_fieldID;

chatEventHasAggressiveAntiSpamEnabledToggled::chatEventHasAggressiveAntiSpamEnabledToggled()
  : has_aggressive_anti_spam_enabled_()
{}

chatEventHasAggressiveAntiSpamEnabledToggled::chatEventHasAggressiveAntiSpamEnabledToggled(bool has_aggressive_anti_spam_enabled_)
  : has_aggressive_anti_spam_enabled_(has_aggressive_anti_spam_enabled_)
{}

const std::int32_t chatEventHasAggressiveAntiSpamEnabledToggled::ID;

object_ptr<ChatEventAction> chatEventHasAggressiveAntiSpamEnabledToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventHasAggressiveAntiSpamEnabledToggled> res = make_object<chatEventHasAggressiveAntiSpamEnabledToggled>();
  res->has_aggressive_anti_spam_enabled_ = (env->GetBooleanField(p, res->has_aggressive_anti_spam_enabled_fieldID) != 0);
  return std::move(res);
}

void chatEventHasAggressiveAntiSpamEnabledToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_aggressive_anti_spam_enabled_fieldID, has_aggressive_anti_spam_enabled_);
}

void chatEventHasAggressiveAntiSpamEnabledToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventHasAggressiveAntiSpamEnabledToggled");
    s.store_field("hasAggressiveAntiSpamEnabled", has_aggressive_anti_spam_enabled_);
    s.store_class_end();
  }
}

void chatEventHasAggressiveAntiSpamEnabledToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventHasAggressiveAntiSpamEnabledToggled").c_str());
    has_aggressive_anti_spam_enabled_fieldID = jni::get_field_id(env, Class, "hasAggressiveAntiSpamEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventSignMessagesToggled::Class;
jfieldID chatEventSignMessagesToggled::sign_messages_fieldID;

chatEventSignMessagesToggled::chatEventSignMessagesToggled()
  : sign_messages_()
{}

chatEventSignMessagesToggled::chatEventSignMessagesToggled(bool sign_messages_)
  : sign_messages_(sign_messages_)
{}

const std::int32_t chatEventSignMessagesToggled::ID;

object_ptr<ChatEventAction> chatEventSignMessagesToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventSignMessagesToggled> res = make_object<chatEventSignMessagesToggled>();
  res->sign_messages_ = (env->GetBooleanField(p, res->sign_messages_fieldID) != 0);
  return std::move(res);
}

void chatEventSignMessagesToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, sign_messages_fieldID, sign_messages_);
}

void chatEventSignMessagesToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventSignMessagesToggled");
    s.store_field("signMessages", sign_messages_);
    s.store_class_end();
  }
}

void chatEventSignMessagesToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventSignMessagesToggled").c_str());
    sign_messages_fieldID = jni::get_field_id(env, Class, "signMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventShowMessageSenderToggled::Class;
jfieldID chatEventShowMessageSenderToggled::show_message_sender_fieldID;

chatEventShowMessageSenderToggled::chatEventShowMessageSenderToggled()
  : show_message_sender_()
{}

chatEventShowMessageSenderToggled::chatEventShowMessageSenderToggled(bool show_message_sender_)
  : show_message_sender_(show_message_sender_)
{}

const std::int32_t chatEventShowMessageSenderToggled::ID;

object_ptr<ChatEventAction> chatEventShowMessageSenderToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventShowMessageSenderToggled> res = make_object<chatEventShowMessageSenderToggled>();
  res->show_message_sender_ = (env->GetBooleanField(p, res->show_message_sender_fieldID) != 0);
  return std::move(res);
}

void chatEventShowMessageSenderToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, show_message_sender_fieldID, show_message_sender_);
}

void chatEventShowMessageSenderToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventShowMessageSenderToggled");
    s.store_field("showMessageSender", show_message_sender_);
    s.store_class_end();
  }
}

void chatEventShowMessageSenderToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventShowMessageSenderToggled").c_str());
    show_message_sender_fieldID = jni::get_field_id(env, Class, "showMessageSender", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventAutomaticTranslationToggled::Class;
jfieldID chatEventAutomaticTranslationToggled::has_automatic_translation_fieldID;

chatEventAutomaticTranslationToggled::chatEventAutomaticTranslationToggled()
  : has_automatic_translation_()
{}

chatEventAutomaticTranslationToggled::chatEventAutomaticTranslationToggled(bool has_automatic_translation_)
  : has_automatic_translation_(has_automatic_translation_)
{}

const std::int32_t chatEventAutomaticTranslationToggled::ID;

object_ptr<ChatEventAction> chatEventAutomaticTranslationToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventAutomaticTranslationToggled> res = make_object<chatEventAutomaticTranslationToggled>();
  res->has_automatic_translation_ = (env->GetBooleanField(p, res->has_automatic_translation_fieldID) != 0);
  return std::move(res);
}

void chatEventAutomaticTranslationToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_automatic_translation_fieldID, has_automatic_translation_);
}

void chatEventAutomaticTranslationToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventAutomaticTranslationToggled");
    s.store_field("hasAutomaticTranslation", has_automatic_translation_);
    s.store_class_end();
  }
}

void chatEventAutomaticTranslationToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventAutomaticTranslationToggled").c_str());
    has_automatic_translation_fieldID = jni::get_field_id(env, Class, "hasAutomaticTranslation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventInviteLinkEdited::Class;
jfieldID chatEventInviteLinkEdited::old_invite_link_fieldID;
jfieldID chatEventInviteLinkEdited::new_invite_link_fieldID;

chatEventInviteLinkEdited::chatEventInviteLinkEdited()
  : old_invite_link_()
  , new_invite_link_()
{}

chatEventInviteLinkEdited::chatEventInviteLinkEdited(object_ptr<chatInviteLink> &&old_invite_link_, object_ptr<chatInviteLink> &&new_invite_link_)
  : old_invite_link_(std::move(old_invite_link_))
  , new_invite_link_(std::move(new_invite_link_))
{}

const std::int32_t chatEventInviteLinkEdited::ID;

object_ptr<ChatEventAction> chatEventInviteLinkEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventInviteLinkEdited> res = make_object<chatEventInviteLinkEdited>();
  res->old_invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->old_invite_link_fieldID));
  res->new_invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->new_invite_link_fieldID));
  return std::move(res);
}

void chatEventInviteLinkEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_invite_link_ != nullptr) { jobject next; old_invite_link_->store(env, next); if (next) { env->SetObjectField(s, old_invite_link_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_invite_link_ != nullptr) { jobject next; new_invite_link_->store(env, next); if (next) { env->SetObjectField(s, new_invite_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventInviteLinkEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventInviteLinkEdited");
    s.store_object_field("oldInviteLink", static_cast<const BaseObject *>(old_invite_link_.get()));
    s.store_object_field("newInviteLink", static_cast<const BaseObject *>(new_invite_link_.get()));
    s.store_class_end();
  }
}

void chatEventInviteLinkEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventInviteLinkEdited").c_str());
    old_invite_link_fieldID = jni::get_field_id(env, Class, "oldInviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    new_invite_link_fieldID = jni::get_field_id(env, Class, "newInviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventInviteLinkRevoked::Class;
jfieldID chatEventInviteLinkRevoked::invite_link_fieldID;

chatEventInviteLinkRevoked::chatEventInviteLinkRevoked()
  : invite_link_()
{}

chatEventInviteLinkRevoked::chatEventInviteLinkRevoked(object_ptr<chatInviteLink> &&invite_link_)
  : invite_link_(std::move(invite_link_))
{}

const std::int32_t chatEventInviteLinkRevoked::ID;

object_ptr<ChatEventAction> chatEventInviteLinkRevoked::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventInviteLinkRevoked> res = make_object<chatEventInviteLinkRevoked>();
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  return std::move(res);
}

void chatEventInviteLinkRevoked::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventInviteLinkRevoked::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventInviteLinkRevoked");
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_class_end();
  }
}

void chatEventInviteLinkRevoked::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventInviteLinkRevoked").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventInviteLinkDeleted::Class;
jfieldID chatEventInviteLinkDeleted::invite_link_fieldID;

chatEventInviteLinkDeleted::chatEventInviteLinkDeleted()
  : invite_link_()
{}

chatEventInviteLinkDeleted::chatEventInviteLinkDeleted(object_ptr<chatInviteLink> &&invite_link_)
  : invite_link_(std::move(invite_link_))
{}

const std::int32_t chatEventInviteLinkDeleted::ID;

object_ptr<ChatEventAction> chatEventInviteLinkDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventInviteLinkDeleted> res = make_object<chatEventInviteLinkDeleted>();
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  return std::move(res);
}

void chatEventInviteLinkDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventInviteLinkDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventInviteLinkDeleted");
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_class_end();
  }
}

void chatEventInviteLinkDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventInviteLinkDeleted").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventVideoChatCreated::Class;
jfieldID chatEventVideoChatCreated::group_call_id_fieldID;

chatEventVideoChatCreated::chatEventVideoChatCreated()
  : group_call_id_()
{}

chatEventVideoChatCreated::chatEventVideoChatCreated(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t chatEventVideoChatCreated::ID;

object_ptr<ChatEventAction> chatEventVideoChatCreated::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventVideoChatCreated> res = make_object<chatEventVideoChatCreated>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return std::move(res);
}

void chatEventVideoChatCreated::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
}

void chatEventVideoChatCreated::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventVideoChatCreated");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

void chatEventVideoChatCreated::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventVideoChatCreated").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventVideoChatEnded::Class;
jfieldID chatEventVideoChatEnded::group_call_id_fieldID;

chatEventVideoChatEnded::chatEventVideoChatEnded()
  : group_call_id_()
{}

chatEventVideoChatEnded::chatEventVideoChatEnded(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t chatEventVideoChatEnded::ID;

object_ptr<ChatEventAction> chatEventVideoChatEnded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventVideoChatEnded> res = make_object<chatEventVideoChatEnded>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return std::move(res);
}

void chatEventVideoChatEnded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
}

void chatEventVideoChatEnded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventVideoChatEnded");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

void chatEventVideoChatEnded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventVideoChatEnded").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventVideoChatMuteNewParticipantsToggled::Class;
jfieldID chatEventVideoChatMuteNewParticipantsToggled::mute_new_participants_fieldID;

chatEventVideoChatMuteNewParticipantsToggled::chatEventVideoChatMuteNewParticipantsToggled()
  : mute_new_participants_()
{}

chatEventVideoChatMuteNewParticipantsToggled::chatEventVideoChatMuteNewParticipantsToggled(bool mute_new_participants_)
  : mute_new_participants_(mute_new_participants_)
{}

const std::int32_t chatEventVideoChatMuteNewParticipantsToggled::ID;

object_ptr<ChatEventAction> chatEventVideoChatMuteNewParticipantsToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventVideoChatMuteNewParticipantsToggled> res = make_object<chatEventVideoChatMuteNewParticipantsToggled>();
  res->mute_new_participants_ = (env->GetBooleanField(p, res->mute_new_participants_fieldID) != 0);
  return std::move(res);
}

void chatEventVideoChatMuteNewParticipantsToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, mute_new_participants_fieldID, mute_new_participants_);
}

void chatEventVideoChatMuteNewParticipantsToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventVideoChatMuteNewParticipantsToggled");
    s.store_field("muteNewParticipants", mute_new_participants_);
    s.store_class_end();
  }
}

void chatEventVideoChatMuteNewParticipantsToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventVideoChatMuteNewParticipantsToggled").c_str());
    mute_new_participants_fieldID = jni::get_field_id(env, Class, "muteNewParticipants", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventVideoChatParticipantIsMutedToggled::Class;
jfieldID chatEventVideoChatParticipantIsMutedToggled::participant_id_fieldID;
jfieldID chatEventVideoChatParticipantIsMutedToggled::is_muted_fieldID;

chatEventVideoChatParticipantIsMutedToggled::chatEventVideoChatParticipantIsMutedToggled()
  : participant_id_()
  , is_muted_()
{}

chatEventVideoChatParticipantIsMutedToggled::chatEventVideoChatParticipantIsMutedToggled(object_ptr<MessageSender> &&participant_id_, bool is_muted_)
  : participant_id_(std::move(participant_id_))
  , is_muted_(is_muted_)
{}

const std::int32_t chatEventVideoChatParticipantIsMutedToggled::ID;

object_ptr<ChatEventAction> chatEventVideoChatParticipantIsMutedToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventVideoChatParticipantIsMutedToggled> res = make_object<chatEventVideoChatParticipantIsMutedToggled>();
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->is_muted_ = (env->GetBooleanField(p, res->is_muted_fieldID) != 0);
  return std::move(res);
}

void chatEventVideoChatParticipantIsMutedToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (participant_id_ != nullptr) { jobject next; participant_id_->store(env, next); if (next) { env->SetObjectField(s, participant_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_muted_fieldID, is_muted_);
}

void chatEventVideoChatParticipantIsMutedToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventVideoChatParticipantIsMutedToggled");
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("isMuted", is_muted_);
    s.store_class_end();
  }
}

void chatEventVideoChatParticipantIsMutedToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventVideoChatParticipantIsMutedToggled").c_str());
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_muted_fieldID = jni::get_field_id(env, Class, "isMuted", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventVideoChatParticipantVolumeLevelChanged::Class;
jfieldID chatEventVideoChatParticipantVolumeLevelChanged::participant_id_fieldID;
jfieldID chatEventVideoChatParticipantVolumeLevelChanged::volume_level_fieldID;

chatEventVideoChatParticipantVolumeLevelChanged::chatEventVideoChatParticipantVolumeLevelChanged()
  : participant_id_()
  , volume_level_()
{}

chatEventVideoChatParticipantVolumeLevelChanged::chatEventVideoChatParticipantVolumeLevelChanged(object_ptr<MessageSender> &&participant_id_, int32 volume_level_)
  : participant_id_(std::move(participant_id_))
  , volume_level_(volume_level_)
{}

const std::int32_t chatEventVideoChatParticipantVolumeLevelChanged::ID;

object_ptr<ChatEventAction> chatEventVideoChatParticipantVolumeLevelChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventVideoChatParticipantVolumeLevelChanged> res = make_object<chatEventVideoChatParticipantVolumeLevelChanged>();
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->volume_level_ = env->GetIntField(p, res->volume_level_fieldID);
  return std::move(res);
}

void chatEventVideoChatParticipantVolumeLevelChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (participant_id_ != nullptr) { jobject next; participant_id_->store(env, next); if (next) { env->SetObjectField(s, participant_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, volume_level_fieldID, volume_level_);
}

void chatEventVideoChatParticipantVolumeLevelChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventVideoChatParticipantVolumeLevelChanged");
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("volumeLevel", volume_level_);
    s.store_class_end();
  }
}

void chatEventVideoChatParticipantVolumeLevelChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventVideoChatParticipantVolumeLevelChanged").c_str());
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    volume_level_fieldID = jni::get_field_id(env, Class, "volumeLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventIsForumToggled::Class;
jfieldID chatEventIsForumToggled::is_forum_fieldID;

chatEventIsForumToggled::chatEventIsForumToggled()
  : is_forum_()
{}

chatEventIsForumToggled::chatEventIsForumToggled(bool is_forum_)
  : is_forum_(is_forum_)
{}

const std::int32_t chatEventIsForumToggled::ID;

object_ptr<ChatEventAction> chatEventIsForumToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventIsForumToggled> res = make_object<chatEventIsForumToggled>();
  res->is_forum_ = (env->GetBooleanField(p, res->is_forum_fieldID) != 0);
  return std::move(res);
}

void chatEventIsForumToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_forum_fieldID, is_forum_);
}

void chatEventIsForumToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventIsForumToggled");
    s.store_field("isForum", is_forum_);
    s.store_class_end();
  }
}

void chatEventIsForumToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventIsForumToggled").c_str());
    is_forum_fieldID = jni::get_field_id(env, Class, "isForum", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicCreated::Class;
jfieldID chatEventForumTopicCreated::topic_info_fieldID;

chatEventForumTopicCreated::chatEventForumTopicCreated()
  : topic_info_()
{}

chatEventForumTopicCreated::chatEventForumTopicCreated(object_ptr<forumTopicInfo> &&topic_info_)
  : topic_info_(std::move(topic_info_))
{}

const std::int32_t chatEventForumTopicCreated::ID;

object_ptr<ChatEventAction> chatEventForumTopicCreated::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicCreated> res = make_object<chatEventForumTopicCreated>();
  res->topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicCreated::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (topic_info_ != nullptr) { jobject next; topic_info_->store(env, next); if (next) { env->SetObjectField(s, topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicCreated::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicCreated");
    s.store_object_field("topicInfo", static_cast<const BaseObject *>(topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicCreated::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicCreated").c_str());
    topic_info_fieldID = jni::get_field_id(env, Class, "topicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicEdited::Class;
jfieldID chatEventForumTopicEdited::old_topic_info_fieldID;
jfieldID chatEventForumTopicEdited::new_topic_info_fieldID;

chatEventForumTopicEdited::chatEventForumTopicEdited()
  : old_topic_info_()
  , new_topic_info_()
{}

chatEventForumTopicEdited::chatEventForumTopicEdited(object_ptr<forumTopicInfo> &&old_topic_info_, object_ptr<forumTopicInfo> &&new_topic_info_)
  : old_topic_info_(std::move(old_topic_info_))
  , new_topic_info_(std::move(new_topic_info_))
{}

const std::int32_t chatEventForumTopicEdited::ID;

object_ptr<ChatEventAction> chatEventForumTopicEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicEdited> res = make_object<chatEventForumTopicEdited>();
  res->old_topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->old_topic_info_fieldID));
  res->new_topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->new_topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_topic_info_ != nullptr) { jobject next; old_topic_info_->store(env, next); if (next) { env->SetObjectField(s, old_topic_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_topic_info_ != nullptr) { jobject next; new_topic_info_->store(env, next); if (next) { env->SetObjectField(s, new_topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicEdited");
    s.store_object_field("oldTopicInfo", static_cast<const BaseObject *>(old_topic_info_.get()));
    s.store_object_field("newTopicInfo", static_cast<const BaseObject *>(new_topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicEdited").c_str());
    old_topic_info_fieldID = jni::get_field_id(env, Class, "oldTopicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    new_topic_info_fieldID = jni::get_field_id(env, Class, "newTopicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicToggleIsClosed::Class;
jfieldID chatEventForumTopicToggleIsClosed::topic_info_fieldID;

chatEventForumTopicToggleIsClosed::chatEventForumTopicToggleIsClosed()
  : topic_info_()
{}

chatEventForumTopicToggleIsClosed::chatEventForumTopicToggleIsClosed(object_ptr<forumTopicInfo> &&topic_info_)
  : topic_info_(std::move(topic_info_))
{}

const std::int32_t chatEventForumTopicToggleIsClosed::ID;

object_ptr<ChatEventAction> chatEventForumTopicToggleIsClosed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicToggleIsClosed> res = make_object<chatEventForumTopicToggleIsClosed>();
  res->topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicToggleIsClosed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (topic_info_ != nullptr) { jobject next; topic_info_->store(env, next); if (next) { env->SetObjectField(s, topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicToggleIsClosed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicToggleIsClosed");
    s.store_object_field("topicInfo", static_cast<const BaseObject *>(topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicToggleIsClosed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicToggleIsClosed").c_str());
    topic_info_fieldID = jni::get_field_id(env, Class, "topicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicToggleIsHidden::Class;
jfieldID chatEventForumTopicToggleIsHidden::topic_info_fieldID;

chatEventForumTopicToggleIsHidden::chatEventForumTopicToggleIsHidden()
  : topic_info_()
{}

chatEventForumTopicToggleIsHidden::chatEventForumTopicToggleIsHidden(object_ptr<forumTopicInfo> &&topic_info_)
  : topic_info_(std::move(topic_info_))
{}

const std::int32_t chatEventForumTopicToggleIsHidden::ID;

object_ptr<ChatEventAction> chatEventForumTopicToggleIsHidden::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicToggleIsHidden> res = make_object<chatEventForumTopicToggleIsHidden>();
  res->topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicToggleIsHidden::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (topic_info_ != nullptr) { jobject next; topic_info_->store(env, next); if (next) { env->SetObjectField(s, topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicToggleIsHidden::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicToggleIsHidden");
    s.store_object_field("topicInfo", static_cast<const BaseObject *>(topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicToggleIsHidden::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicToggleIsHidden").c_str());
    topic_info_fieldID = jni::get_field_id(env, Class, "topicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicDeleted::Class;
jfieldID chatEventForumTopicDeleted::topic_info_fieldID;

chatEventForumTopicDeleted::chatEventForumTopicDeleted()
  : topic_info_()
{}

chatEventForumTopicDeleted::chatEventForumTopicDeleted(object_ptr<forumTopicInfo> &&topic_info_)
  : topic_info_(std::move(topic_info_))
{}

const std::int32_t chatEventForumTopicDeleted::ID;

object_ptr<ChatEventAction> chatEventForumTopicDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicDeleted> res = make_object<chatEventForumTopicDeleted>();
  res->topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (topic_info_ != nullptr) { jobject next; topic_info_->store(env, next); if (next) { env->SetObjectField(s, topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicDeleted");
    s.store_object_field("topicInfo", static_cast<const BaseObject *>(topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicDeleted").c_str());
    topic_info_fieldID = jni::get_field_id(env, Class, "topicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventForumTopicPinned::Class;
jfieldID chatEventForumTopicPinned::old_topic_info_fieldID;
jfieldID chatEventForumTopicPinned::new_topic_info_fieldID;

chatEventForumTopicPinned::chatEventForumTopicPinned()
  : old_topic_info_()
  , new_topic_info_()
{}

chatEventForumTopicPinned::chatEventForumTopicPinned(object_ptr<forumTopicInfo> &&old_topic_info_, object_ptr<forumTopicInfo> &&new_topic_info_)
  : old_topic_info_(std::move(old_topic_info_))
  , new_topic_info_(std::move(new_topic_info_))
{}

const std::int32_t chatEventForumTopicPinned::ID;

object_ptr<ChatEventAction> chatEventForumTopicPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventForumTopicPinned> res = make_object<chatEventForumTopicPinned>();
  res->old_topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->old_topic_info_fieldID));
  res->new_topic_info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->new_topic_info_fieldID));
  return std::move(res);
}

void chatEventForumTopicPinned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (old_topic_info_ != nullptr) { jobject next; old_topic_info_->store(env, next); if (next) { env->SetObjectField(s, old_topic_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_topic_info_ != nullptr) { jobject next; new_topic_info_->store(env, next); if (next) { env->SetObjectField(s, new_topic_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatEventForumTopicPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventForumTopicPinned");
    s.store_object_field("oldTopicInfo", static_cast<const BaseObject *>(old_topic_info_.get()));
    s.store_object_field("newTopicInfo", static_cast<const BaseObject *>(new_topic_info_.get()));
    s.store_class_end();
  }
}

void chatEventForumTopicPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventForumTopicPinned").c_str());
    old_topic_info_fieldID = jni::get_field_id(env, Class, "oldTopicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    new_topic_info_fieldID = jni::get_field_id(env, Class, "newTopicInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatEventLogFilters::Class;
jfieldID chatEventLogFilters::message_edits_fieldID;
jfieldID chatEventLogFilters::message_deletions_fieldID;
jfieldID chatEventLogFilters::message_pins_fieldID;
jfieldID chatEventLogFilters::member_joins_fieldID;
jfieldID chatEventLogFilters::member_leaves_fieldID;
jfieldID chatEventLogFilters::member_invites_fieldID;
jfieldID chatEventLogFilters::member_promotions_fieldID;
jfieldID chatEventLogFilters::member_restrictions_fieldID;
jfieldID chatEventLogFilters::info_changes_fieldID;
jfieldID chatEventLogFilters::setting_changes_fieldID;
jfieldID chatEventLogFilters::invite_link_changes_fieldID;
jfieldID chatEventLogFilters::video_chat_changes_fieldID;
jfieldID chatEventLogFilters::forum_changes_fieldID;
jfieldID chatEventLogFilters::subscription_extensions_fieldID;

chatEventLogFilters::chatEventLogFilters()
  : message_edits_()
  , message_deletions_()
  , message_pins_()
  , member_joins_()
  , member_leaves_()
  , member_invites_()
  , member_promotions_()
  , member_restrictions_()
  , info_changes_()
  , setting_changes_()
  , invite_link_changes_()
  , video_chat_changes_()
  , forum_changes_()
  , subscription_extensions_()
{}

chatEventLogFilters::chatEventLogFilters(bool message_edits_, bool message_deletions_, bool message_pins_, bool member_joins_, bool member_leaves_, bool member_invites_, bool member_promotions_, bool member_restrictions_, bool info_changes_, bool setting_changes_, bool invite_link_changes_, bool video_chat_changes_, bool forum_changes_, bool subscription_extensions_)
  : message_edits_(message_edits_)
  , message_deletions_(message_deletions_)
  , message_pins_(message_pins_)
  , member_joins_(member_joins_)
  , member_leaves_(member_leaves_)
  , member_invites_(member_invites_)
  , member_promotions_(member_promotions_)
  , member_restrictions_(member_restrictions_)
  , info_changes_(info_changes_)
  , setting_changes_(setting_changes_)
  , invite_link_changes_(invite_link_changes_)
  , video_chat_changes_(video_chat_changes_)
  , forum_changes_(forum_changes_)
  , subscription_extensions_(subscription_extensions_)
{}

const std::int32_t chatEventLogFilters::ID;

object_ptr<chatEventLogFilters> chatEventLogFilters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEventLogFilters> res = make_object<chatEventLogFilters>();
  res->message_edits_ = (env->GetBooleanField(p, res->message_edits_fieldID) != 0);
  res->message_deletions_ = (env->GetBooleanField(p, res->message_deletions_fieldID) != 0);
  res->message_pins_ = (env->GetBooleanField(p, res->message_pins_fieldID) != 0);
  res->member_joins_ = (env->GetBooleanField(p, res->member_joins_fieldID) != 0);
  res->member_leaves_ = (env->GetBooleanField(p, res->member_leaves_fieldID) != 0);
  res->member_invites_ = (env->GetBooleanField(p, res->member_invites_fieldID) != 0);
  res->member_promotions_ = (env->GetBooleanField(p, res->member_promotions_fieldID) != 0);
  res->member_restrictions_ = (env->GetBooleanField(p, res->member_restrictions_fieldID) != 0);
  res->info_changes_ = (env->GetBooleanField(p, res->info_changes_fieldID) != 0);
  res->setting_changes_ = (env->GetBooleanField(p, res->setting_changes_fieldID) != 0);
  res->invite_link_changes_ = (env->GetBooleanField(p, res->invite_link_changes_fieldID) != 0);
  res->video_chat_changes_ = (env->GetBooleanField(p, res->video_chat_changes_fieldID) != 0);
  res->forum_changes_ = (env->GetBooleanField(p, res->forum_changes_fieldID) != 0);
  res->subscription_extensions_ = (env->GetBooleanField(p, res->subscription_extensions_fieldID) != 0);
  return res;
}

void chatEventLogFilters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEventLogFilters");
    s.store_field("messageEdits", message_edits_);
    s.store_field("messageDeletions", message_deletions_);
    s.store_field("messagePins", message_pins_);
    s.store_field("memberJoins", member_joins_);
    s.store_field("memberLeaves", member_leaves_);
    s.store_field("memberInvites", member_invites_);
    s.store_field("memberPromotions", member_promotions_);
    s.store_field("memberRestrictions", member_restrictions_);
    s.store_field("infoChanges", info_changes_);
    s.store_field("settingChanges", setting_changes_);
    s.store_field("inviteLinkChanges", invite_link_changes_);
    s.store_field("videoChatChanges", video_chat_changes_);
    s.store_field("forumChanges", forum_changes_);
    s.store_field("subscriptionExtensions", subscription_extensions_);
    s.store_class_end();
  }
}

void chatEventLogFilters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEventLogFilters").c_str());
    message_edits_fieldID = jni::get_field_id(env, Class, "messageEdits", "Z");
    message_deletions_fieldID = jni::get_field_id(env, Class, "messageDeletions", "Z");
    message_pins_fieldID = jni::get_field_id(env, Class, "messagePins", "Z");
    member_joins_fieldID = jni::get_field_id(env, Class, "memberJoins", "Z");
    member_leaves_fieldID = jni::get_field_id(env, Class, "memberLeaves", "Z");
    member_invites_fieldID = jni::get_field_id(env, Class, "memberInvites", "Z");
    member_promotions_fieldID = jni::get_field_id(env, Class, "memberPromotions", "Z");
    member_restrictions_fieldID = jni::get_field_id(env, Class, "memberRestrictions", "Z");
    info_changes_fieldID = jni::get_field_id(env, Class, "infoChanges", "Z");
    setting_changes_fieldID = jni::get_field_id(env, Class, "settingChanges", "Z");
    invite_link_changes_fieldID = jni::get_field_id(env, Class, "inviteLinkChanges", "Z");
    video_chat_changes_fieldID = jni::get_field_id(env, Class, "videoChatChanges", "Z");
    forum_changes_fieldID = jni::get_field_id(env, Class, "forumChanges", "Z");
    subscription_extensions_fieldID = jni::get_field_id(env, Class, "subscriptionExtensions", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatEvents::Class;
jfieldID chatEvents::events_fieldID;

chatEvents::chatEvents()
  : events_()
{}

chatEvents::chatEvents(array<object_ptr<chatEvent>> &&events_)
  : events_(std::move(events_))
{}

const std::int32_t chatEvents::ID;

object_ptr<chatEvents> chatEvents::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatEvents> res = make_object<chatEvents>();
  res->events_ = jni::FetchVector<chatEvent>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->events_fieldID));
  return res;
}

void chatEvents::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, events_); if (arr_tmp_) { env->SetObjectField(s, events_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatEvents::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatEvents");
    { s.store_vector_begin("events", events_.size()); for (const auto &_value : events_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatEvents::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatEvents").c_str());
    events_fieldID = jni::get_field_id(env, Class, "events", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatEvent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatFolder::Class;
jfieldID chatFolder::name_fieldID;
jfieldID chatFolder::icon_fieldID;
jfieldID chatFolder::color_id_fieldID;
jfieldID chatFolder::is_shareable_fieldID;
jfieldID chatFolder::pinned_chat_ids_fieldID;
jfieldID chatFolder::included_chat_ids_fieldID;
jfieldID chatFolder::excluded_chat_ids_fieldID;
jfieldID chatFolder::exclude_muted_fieldID;
jfieldID chatFolder::exclude_read_fieldID;
jfieldID chatFolder::exclude_archived_fieldID;
jfieldID chatFolder::include_contacts_fieldID;
jfieldID chatFolder::include_non_contacts_fieldID;
jfieldID chatFolder::include_bots_fieldID;
jfieldID chatFolder::include_groups_fieldID;
jfieldID chatFolder::include_channels_fieldID;

chatFolder::chatFolder()
  : name_()
  , icon_()
  , color_id_()
  , is_shareable_()
  , pinned_chat_ids_()
  , included_chat_ids_()
  , excluded_chat_ids_()
  , exclude_muted_()
  , exclude_read_()
  , exclude_archived_()
  , include_contacts_()
  , include_non_contacts_()
  , include_bots_()
  , include_groups_()
  , include_channels_()
{}

chatFolder::chatFolder(object_ptr<chatFolderName> &&name_, object_ptr<chatFolderIcon> &&icon_, int32 color_id_, bool is_shareable_, array<int53> &&pinned_chat_ids_, array<int53> &&included_chat_ids_, array<int53> &&excluded_chat_ids_, bool exclude_muted_, bool exclude_read_, bool exclude_archived_, bool include_contacts_, bool include_non_contacts_, bool include_bots_, bool include_groups_, bool include_channels_)
  : name_(std::move(name_))
  , icon_(std::move(icon_))
  , color_id_(color_id_)
  , is_shareable_(is_shareable_)
  , pinned_chat_ids_(std::move(pinned_chat_ids_))
  , included_chat_ids_(std::move(included_chat_ids_))
  , excluded_chat_ids_(std::move(excluded_chat_ids_))
  , exclude_muted_(exclude_muted_)
  , exclude_read_(exclude_read_)
  , exclude_archived_(exclude_archived_)
  , include_contacts_(include_contacts_)
  , include_non_contacts_(include_non_contacts_)
  , include_bots_(include_bots_)
  , include_groups_(include_groups_)
  , include_channels_(include_channels_)
{}

const std::int32_t chatFolder::ID;

object_ptr<chatFolder> chatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolder> res = make_object<chatFolder>();
  res->name_ = jni::fetch_tl_object<chatFolderName>(env, jni::fetch_object(env, p, res->name_fieldID));
  res->icon_ = jni::fetch_tl_object<chatFolderIcon>(env, jni::fetch_object(env, p, res->icon_fieldID));
  res->color_id_ = env->GetIntField(p, res->color_id_fieldID);
  res->is_shareable_ = (env->GetBooleanField(p, res->is_shareable_fieldID) != 0);
  res->pinned_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->pinned_chat_ids_fieldID));
  res->included_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->included_chat_ids_fieldID));
  res->excluded_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->excluded_chat_ids_fieldID));
  res->exclude_muted_ = (env->GetBooleanField(p, res->exclude_muted_fieldID) != 0);
  res->exclude_read_ = (env->GetBooleanField(p, res->exclude_read_fieldID) != 0);
  res->exclude_archived_ = (env->GetBooleanField(p, res->exclude_archived_fieldID) != 0);
  res->include_contacts_ = (env->GetBooleanField(p, res->include_contacts_fieldID) != 0);
  res->include_non_contacts_ = (env->GetBooleanField(p, res->include_non_contacts_fieldID) != 0);
  res->include_bots_ = (env->GetBooleanField(p, res->include_bots_fieldID) != 0);
  res->include_groups_ = (env->GetBooleanField(p, res->include_groups_fieldID) != 0);
  res->include_channels_ = (env->GetBooleanField(p, res->include_channels_fieldID) != 0);
  return res;
}

void chatFolder::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (name_ != nullptr) { jobject next; name_->store(env, next); if (next) { env->SetObjectField(s, name_fieldID, next); env->DeleteLocalRef(next); } }
  if (icon_ != nullptr) { jobject next; icon_->store(env, next); if (next) { env->SetObjectField(s, icon_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, color_id_fieldID, color_id_);
  env->SetBooleanField(s, is_shareable_fieldID, is_shareable_);
  { auto arr_tmp_ = jni::store_vector(env, pinned_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, pinned_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, included_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, included_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, excluded_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, excluded_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, exclude_muted_fieldID, exclude_muted_);
  env->SetBooleanField(s, exclude_read_fieldID, exclude_read_);
  env->SetBooleanField(s, exclude_archived_fieldID, exclude_archived_);
  env->SetBooleanField(s, include_contacts_fieldID, include_contacts_);
  env->SetBooleanField(s, include_non_contacts_fieldID, include_non_contacts_);
  env->SetBooleanField(s, include_bots_fieldID, include_bots_);
  env->SetBooleanField(s, include_groups_fieldID, include_groups_);
  env->SetBooleanField(s, include_channels_fieldID, include_channels_);
}

void chatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolder");
    s.store_object_field("name", static_cast<const BaseObject *>(name_.get()));
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_field("colorId", color_id_);
    s.store_field("isShareable", is_shareable_);
    { s.store_vector_begin("pinnedChatIds", pinned_chat_ids_.size()); for (const auto &_value : pinned_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("includedChatIds", included_chat_ids_.size()); for (const auto &_value : included_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("excludedChatIds", excluded_chat_ids_.size()); for (const auto &_value : excluded_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("excludeMuted", exclude_muted_);
    s.store_field("excludeRead", exclude_read_);
    s.store_field("excludeArchived", exclude_archived_);
    s.store_field("includeContacts", include_contacts_);
    s.store_field("includeNonContacts", include_non_contacts_);
    s.store_field("includeBots", include_bots_);
    s.store_field("includeGroups", include_groups_);
    s.store_field("includeChannels", include_channels_);
    s.store_class_end();
  }
}

void chatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolder").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolderName;").c_str());
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolderIcon;").c_str());
    color_id_fieldID = jni::get_field_id(env, Class, "colorId", "I");
    is_shareable_fieldID = jni::get_field_id(env, Class, "isShareable", "Z");
    pinned_chat_ids_fieldID = jni::get_field_id(env, Class, "pinnedChatIds", "[J");
    included_chat_ids_fieldID = jni::get_field_id(env, Class, "includedChatIds", "[J");
    excluded_chat_ids_fieldID = jni::get_field_id(env, Class, "excludedChatIds", "[J");
    exclude_muted_fieldID = jni::get_field_id(env, Class, "excludeMuted", "Z");
    exclude_read_fieldID = jni::get_field_id(env, Class, "excludeRead", "Z");
    exclude_archived_fieldID = jni::get_field_id(env, Class, "excludeArchived", "Z");
    include_contacts_fieldID = jni::get_field_id(env, Class, "includeContacts", "Z");
    include_non_contacts_fieldID = jni::get_field_id(env, Class, "includeNonContacts", "Z");
    include_bots_fieldID = jni::get_field_id(env, Class, "includeBots", "Z");
    include_groups_fieldID = jni::get_field_id(env, Class, "includeGroups", "Z");
    include_channels_fieldID = jni::get_field_id(env, Class, "includeChannels", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderIcon::Class;
jfieldID chatFolderIcon::name_fieldID;

chatFolderIcon::chatFolderIcon()
  : name_()
{}

chatFolderIcon::chatFolderIcon(string const &name_)
  : name_(name_)
{}

const std::int32_t chatFolderIcon::ID;

object_ptr<chatFolderIcon> chatFolderIcon::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderIcon> res = make_object<chatFolderIcon>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void chatFolderIcon::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatFolderIcon::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderIcon");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void chatFolderIcon::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderIcon").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderInfo::Class;
jfieldID chatFolderInfo::id_fieldID;
jfieldID chatFolderInfo::name_fieldID;
jfieldID chatFolderInfo::icon_fieldID;
jfieldID chatFolderInfo::color_id_fieldID;
jfieldID chatFolderInfo::is_shareable_fieldID;
jfieldID chatFolderInfo::has_my_invite_links_fieldID;

chatFolderInfo::chatFolderInfo()
  : id_()
  , name_()
  , icon_()
  , color_id_()
  , is_shareable_()
  , has_my_invite_links_()
{}

chatFolderInfo::chatFolderInfo(int32 id_, object_ptr<chatFolderName> &&name_, object_ptr<chatFolderIcon> &&icon_, int32 color_id_, bool is_shareable_, bool has_my_invite_links_)
  : id_(id_)
  , name_(std::move(name_))
  , icon_(std::move(icon_))
  , color_id_(color_id_)
  , is_shareable_(is_shareable_)
  , has_my_invite_links_(has_my_invite_links_)
{}

const std::int32_t chatFolderInfo::ID;

object_ptr<chatFolderInfo> chatFolderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderInfo> res = make_object<chatFolderInfo>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->name_ = jni::fetch_tl_object<chatFolderName>(env, jni::fetch_object(env, p, res->name_fieldID));
  res->icon_ = jni::fetch_tl_object<chatFolderIcon>(env, jni::fetch_object(env, p, res->icon_fieldID));
  res->color_id_ = env->GetIntField(p, res->color_id_fieldID);
  res->is_shareable_ = (env->GetBooleanField(p, res->is_shareable_fieldID) != 0);
  res->has_my_invite_links_ = (env->GetBooleanField(p, res->has_my_invite_links_fieldID) != 0);
  return res;
}

void chatFolderInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  if (name_ != nullptr) { jobject next; name_->store(env, next); if (next) { env->SetObjectField(s, name_fieldID, next); env->DeleteLocalRef(next); } }
  if (icon_ != nullptr) { jobject next; icon_->store(env, next); if (next) { env->SetObjectField(s, icon_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, color_id_fieldID, color_id_);
  env->SetBooleanField(s, is_shareable_fieldID, is_shareable_);
  env->SetBooleanField(s, has_my_invite_links_fieldID, has_my_invite_links_);
}

void chatFolderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderInfo");
    s.store_field("id", id_);
    s.store_object_field("name", static_cast<const BaseObject *>(name_.get()));
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_field("colorId", color_id_);
    s.store_field("isShareable", is_shareable_);
    s.store_field("hasMyInviteLinks", has_my_invite_links_);
    s.store_class_end();
  }
}

void chatFolderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderInfo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    name_fieldID = jni::get_field_id(env, Class, "name", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolderName;").c_str());
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolderIcon;").c_str());
    color_id_fieldID = jni::get_field_id(env, Class, "colorId", "I");
    is_shareable_fieldID = jni::get_field_id(env, Class, "isShareable", "Z");
    has_my_invite_links_fieldID = jni::get_field_id(env, Class, "hasMyInviteLinks", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderInviteLink::Class;
jfieldID chatFolderInviteLink::invite_link_fieldID;
jfieldID chatFolderInviteLink::name_fieldID;
jfieldID chatFolderInviteLink::chat_ids_fieldID;

chatFolderInviteLink::chatFolderInviteLink()
  : invite_link_()
  , name_()
  , chat_ids_()
{}

chatFolderInviteLink::chatFolderInviteLink(string const &invite_link_, string const &name_, array<int53> &&chat_ids_)
  : invite_link_(invite_link_)
  , name_(name_)
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t chatFolderInviteLink::ID;

object_ptr<chatFolderInviteLink> chatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderInviteLink> res = make_object<chatFolderInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void chatFolderInviteLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, chat_ids_); if (arr_tmp_) { env->SetObjectField(s, chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderInviteLink");
    s.store_field("inviteLink", invite_link_);
    s.store_field("name", name_);
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderInviteLinkInfo::Class;
jfieldID chatFolderInviteLinkInfo::chat_folder_info_fieldID;
jfieldID chatFolderInviteLinkInfo::missing_chat_ids_fieldID;
jfieldID chatFolderInviteLinkInfo::added_chat_ids_fieldID;

chatFolderInviteLinkInfo::chatFolderInviteLinkInfo()
  : chat_folder_info_()
  , missing_chat_ids_()
  , added_chat_ids_()
{}

chatFolderInviteLinkInfo::chatFolderInviteLinkInfo(object_ptr<chatFolderInfo> &&chat_folder_info_, array<int53> &&missing_chat_ids_, array<int53> &&added_chat_ids_)
  : chat_folder_info_(std::move(chat_folder_info_))
  , missing_chat_ids_(std::move(missing_chat_ids_))
  , added_chat_ids_(std::move(added_chat_ids_))
{}

const std::int32_t chatFolderInviteLinkInfo::ID;

object_ptr<chatFolderInviteLinkInfo> chatFolderInviteLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderInviteLinkInfo> res = make_object<chatFolderInviteLinkInfo>();
  res->chat_folder_info_ = jni::fetch_tl_object<chatFolderInfo>(env, jni::fetch_object(env, p, res->chat_folder_info_fieldID));
  res->missing_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->missing_chat_ids_fieldID));
  res->added_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->added_chat_ids_fieldID));
  return res;
}

void chatFolderInviteLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chat_folder_info_ != nullptr) { jobject next; chat_folder_info_->store(env, next); if (next) { env->SetObjectField(s, chat_folder_info_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, missing_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, missing_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, added_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, added_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatFolderInviteLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderInviteLinkInfo");
    s.store_object_field("chatFolderInfo", static_cast<const BaseObject *>(chat_folder_info_.get()));
    { s.store_vector_begin("missingChatIds", missing_chat_ids_.size()); for (const auto &_value : missing_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("addedChatIds", added_chat_ids_.size()); for (const auto &_value : added_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatFolderInviteLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderInviteLinkInfo").c_str());
    chat_folder_info_fieldID = jni::get_field_id(env, Class, "chatFolderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolderInfo;").c_str());
    missing_chat_ids_fieldID = jni::get_field_id(env, Class, "missingChatIds", "[J");
    added_chat_ids_fieldID = jni::get_field_id(env, Class, "addedChatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderInviteLinks::Class;
jfieldID chatFolderInviteLinks::invite_links_fieldID;

chatFolderInviteLinks::chatFolderInviteLinks()
  : invite_links_()
{}

chatFolderInviteLinks::chatFolderInviteLinks(array<object_ptr<chatFolderInviteLink>> &&invite_links_)
  : invite_links_(std::move(invite_links_))
{}

const std::int32_t chatFolderInviteLinks::ID;

object_ptr<chatFolderInviteLinks> chatFolderInviteLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderInviteLinks> res = make_object<chatFolderInviteLinks>();
  res->invite_links_ = jni::FetchVector<chatFolderInviteLink>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->invite_links_fieldID));
  return res;
}

void chatFolderInviteLinks::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, invite_links_); if (arr_tmp_) { env->SetObjectField(s, invite_links_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatFolderInviteLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderInviteLinks");
    { s.store_vector_begin("inviteLinks", invite_links_.size()); for (const auto &_value : invite_links_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatFolderInviteLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderInviteLinks").c_str());
    invite_links_fieldID = jni::get_field_id(env, Class, "inviteLinks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatFolderInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatFolderName::Class;
jfieldID chatFolderName::text_fieldID;
jfieldID chatFolderName::animate_custom_emoji_fieldID;

chatFolderName::chatFolderName()
  : text_()
  , animate_custom_emoji_()
{}

chatFolderName::chatFolderName(object_ptr<formattedText> &&text_, bool animate_custom_emoji_)
  : text_(std::move(text_))
  , animate_custom_emoji_(animate_custom_emoji_)
{}

const std::int32_t chatFolderName::ID;

object_ptr<chatFolderName> chatFolderName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatFolderName> res = make_object<chatFolderName>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->animate_custom_emoji_ = (env->GetBooleanField(p, res->animate_custom_emoji_fieldID) != 0);
  return res;
}

void chatFolderName::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, animate_custom_emoji_fieldID, animate_custom_emoji_);
}

void chatFolderName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatFolderName");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("animateCustomEmoji", animate_custom_emoji_);
    s.store_class_end();
  }
}

void chatFolderName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatFolderName").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    animate_custom_emoji_fieldID = jni::get_field_id(env, Class, "animateCustomEmoji", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLink::Class;
jfieldID chatInviteLink::invite_link_fieldID;
jfieldID chatInviteLink::name_fieldID;
jfieldID chatInviteLink::creator_user_id_fieldID;
jfieldID chatInviteLink::date_fieldID;
jfieldID chatInviteLink::edit_date_fieldID;
jfieldID chatInviteLink::expiration_date_fieldID;
jfieldID chatInviteLink::subscription_pricing_fieldID;
jfieldID chatInviteLink::member_limit_fieldID;
jfieldID chatInviteLink::member_count_fieldID;
jfieldID chatInviteLink::expired_member_count_fieldID;
jfieldID chatInviteLink::pending_join_request_count_fieldID;
jfieldID chatInviteLink::creates_join_request_fieldID;
jfieldID chatInviteLink::is_primary_fieldID;
jfieldID chatInviteLink::is_revoked_fieldID;

chatInviteLink::chatInviteLink()
  : invite_link_()
  , name_()
  , creator_user_id_()
  , date_()
  , edit_date_()
  , expiration_date_()
  , subscription_pricing_()
  , member_limit_()
  , member_count_()
  , expired_member_count_()
  , pending_join_request_count_()
  , creates_join_request_()
  , is_primary_()
  , is_revoked_()
{}

chatInviteLink::chatInviteLink(string const &invite_link_, string const &name_, int53 creator_user_id_, int32 date_, int32 edit_date_, int32 expiration_date_, object_ptr<starSubscriptionPricing> &&subscription_pricing_, int32 member_limit_, int32 member_count_, int32 expired_member_count_, int32 pending_join_request_count_, bool creates_join_request_, bool is_primary_, bool is_revoked_)
  : invite_link_(invite_link_)
  , name_(name_)
  , creator_user_id_(creator_user_id_)
  , date_(date_)
  , edit_date_(edit_date_)
  , expiration_date_(expiration_date_)
  , subscription_pricing_(std::move(subscription_pricing_))
  , member_limit_(member_limit_)
  , member_count_(member_count_)
  , expired_member_count_(expired_member_count_)
  , pending_join_request_count_(pending_join_request_count_)
  , creates_join_request_(creates_join_request_)
  , is_primary_(is_primary_)
  , is_revoked_(is_revoked_)
{}

const std::int32_t chatInviteLink::ID;

object_ptr<chatInviteLink> chatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLink> res = make_object<chatInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->creator_user_id_ = env->GetLongField(p, res->creator_user_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->edit_date_ = env->GetIntField(p, res->edit_date_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  res->subscription_pricing_ = jni::fetch_tl_object<starSubscriptionPricing>(env, jni::fetch_object(env, p, res->subscription_pricing_fieldID));
  res->member_limit_ = env->GetIntField(p, res->member_limit_fieldID);
  res->member_count_ = env->GetIntField(p, res->member_count_fieldID);
  res->expired_member_count_ = env->GetIntField(p, res->expired_member_count_fieldID);
  res->pending_join_request_count_ = env->GetIntField(p, res->pending_join_request_count_fieldID);
  res->creates_join_request_ = (env->GetBooleanField(p, res->creates_join_request_fieldID) != 0);
  res->is_primary_ = (env->GetBooleanField(p, res->is_primary_fieldID) != 0);
  res->is_revoked_ = (env->GetBooleanField(p, res->is_revoked_fieldID) != 0);
  return res;
}

void chatInviteLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, creator_user_id_fieldID, creator_user_id_);
  env->SetIntField(s, date_fieldID, date_);
  env->SetIntField(s, edit_date_fieldID, edit_date_);
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
  if (subscription_pricing_ != nullptr) { jobject next; subscription_pricing_->store(env, next); if (next) { env->SetObjectField(s, subscription_pricing_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, member_limit_fieldID, member_limit_);
  env->SetIntField(s, member_count_fieldID, member_count_);
  env->SetIntField(s, expired_member_count_fieldID, expired_member_count_);
  env->SetIntField(s, pending_join_request_count_fieldID, pending_join_request_count_);
  env->SetBooleanField(s, creates_join_request_fieldID, creates_join_request_);
  env->SetBooleanField(s, is_primary_fieldID, is_primary_);
  env->SetBooleanField(s, is_revoked_fieldID, is_revoked_);
}

void chatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLink");
    s.store_field("inviteLink", invite_link_);
    s.store_field("name", name_);
    s.store_field("creatorUserId", creator_user_id_);
    s.store_field("date", date_);
    s.store_field("editDate", edit_date_);
    s.store_field("expirationDate", expiration_date_);
    s.store_object_field("subscriptionPricing", static_cast<const BaseObject *>(subscription_pricing_.get()));
    s.store_field("memberLimit", member_limit_);
    s.store_field("memberCount", member_count_);
    s.store_field("expiredMemberCount", expired_member_count_);
    s.store_field("pendingJoinRequestCount", pending_join_request_count_);
    s.store_field("createsJoinRequest", creates_join_request_);
    s.store_field("isPrimary", is_primary_);
    s.store_field("isRevoked", is_revoked_);
    s.store_class_end();
  }
}

void chatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    creator_user_id_fieldID = jni::get_field_id(env, Class, "creatorUserId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    edit_date_fieldID = jni::get_field_id(env, Class, "editDate", "I");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    subscription_pricing_fieldID = jni::get_field_id(env, Class, "subscriptionPricing", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionPricing;").c_str());
    member_limit_fieldID = jni::get_field_id(env, Class, "memberLimit", "I");
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", "I");
    expired_member_count_fieldID = jni::get_field_id(env, Class, "expiredMemberCount", "I");
    pending_join_request_count_fieldID = jni::get_field_id(env, Class, "pendingJoinRequestCount", "I");
    creates_join_request_fieldID = jni::get_field_id(env, Class, "createsJoinRequest", "Z");
    is_primary_fieldID = jni::get_field_id(env, Class, "isPrimary", "Z");
    is_revoked_fieldID = jni::get_field_id(env, Class, "isRevoked", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkCount::Class;
jfieldID chatInviteLinkCount::user_id_fieldID;
jfieldID chatInviteLinkCount::invite_link_count_fieldID;
jfieldID chatInviteLinkCount::revoked_invite_link_count_fieldID;

chatInviteLinkCount::chatInviteLinkCount()
  : user_id_()
  , invite_link_count_()
  , revoked_invite_link_count_()
{}

chatInviteLinkCount::chatInviteLinkCount(int53 user_id_, int32 invite_link_count_, int32 revoked_invite_link_count_)
  : user_id_(user_id_)
  , invite_link_count_(invite_link_count_)
  , revoked_invite_link_count_(revoked_invite_link_count_)
{}

const std::int32_t chatInviteLinkCount::ID;

object_ptr<chatInviteLinkCount> chatInviteLinkCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkCount> res = make_object<chatInviteLinkCount>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->invite_link_count_ = env->GetIntField(p, res->invite_link_count_fieldID);
  res->revoked_invite_link_count_ = env->GetIntField(p, res->revoked_invite_link_count_fieldID);
  return res;
}

void chatInviteLinkCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, invite_link_count_fieldID, invite_link_count_);
  env->SetIntField(s, revoked_invite_link_count_fieldID, revoked_invite_link_count_);
}

void chatInviteLinkCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkCount");
    s.store_field("userId", user_id_);
    s.store_field("inviteLinkCount", invite_link_count_);
    s.store_field("revokedInviteLinkCount", revoked_invite_link_count_);
    s.store_class_end();
  }
}

void chatInviteLinkCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkCount").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    invite_link_count_fieldID = jni::get_field_id(env, Class, "inviteLinkCount", "I");
    revoked_invite_link_count_fieldID = jni::get_field_id(env, Class, "revokedInviteLinkCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkCounts::Class;
jfieldID chatInviteLinkCounts::invite_link_counts_fieldID;

chatInviteLinkCounts::chatInviteLinkCounts()
  : invite_link_counts_()
{}

chatInviteLinkCounts::chatInviteLinkCounts(array<object_ptr<chatInviteLinkCount>> &&invite_link_counts_)
  : invite_link_counts_(std::move(invite_link_counts_))
{}

const std::int32_t chatInviteLinkCounts::ID;

object_ptr<chatInviteLinkCounts> chatInviteLinkCounts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkCounts> res = make_object<chatInviteLinkCounts>();
  res->invite_link_counts_ = jni::FetchVector<chatInviteLinkCount>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->invite_link_counts_fieldID));
  return res;
}

void chatInviteLinkCounts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, invite_link_counts_); if (arr_tmp_) { env->SetObjectField(s, invite_link_counts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatInviteLinkCounts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkCounts");
    { s.store_vector_begin("inviteLinkCounts", invite_link_counts_.size()); for (const auto &_value : invite_link_counts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatInviteLinkCounts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkCounts").c_str());
    invite_link_counts_fieldID = jni::get_field_id(env, Class, "inviteLinkCounts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatInviteLinkCount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkInfo::Class;
jfieldID chatInviteLinkInfo::chat_id_fieldID;
jfieldID chatInviteLinkInfo::accessible_for_fieldID;
jfieldID chatInviteLinkInfo::type_fieldID;
jfieldID chatInviteLinkInfo::title_fieldID;
jfieldID chatInviteLinkInfo::photo_fieldID;
jfieldID chatInviteLinkInfo::accent_color_id_fieldID;
jfieldID chatInviteLinkInfo::description_fieldID;
jfieldID chatInviteLinkInfo::member_count_fieldID;
jfieldID chatInviteLinkInfo::member_user_ids_fieldID;
jfieldID chatInviteLinkInfo::subscription_info_fieldID;
jfieldID chatInviteLinkInfo::creates_join_request_fieldID;
jfieldID chatInviteLinkInfo::is_public_fieldID;
jfieldID chatInviteLinkInfo::verification_status_fieldID;

chatInviteLinkInfo::chatInviteLinkInfo()
  : chat_id_()
  , accessible_for_()
  , type_()
  , title_()
  , photo_()
  , accent_color_id_()
  , description_()
  , member_count_()
  , member_user_ids_()
  , subscription_info_()
  , creates_join_request_()
  , is_public_()
  , verification_status_()
{}

chatInviteLinkInfo::chatInviteLinkInfo(int53 chat_id_, int32 accessible_for_, object_ptr<InviteLinkChatType> &&type_, string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, string const &description_, int32 member_count_, array<int53> &&member_user_ids_, object_ptr<chatInviteLinkSubscriptionInfo> &&subscription_info_, bool creates_join_request_, bool is_public_, object_ptr<verificationStatus> &&verification_status_)
  : chat_id_(chat_id_)
  , accessible_for_(accessible_for_)
  , type_(std::move(type_))
  , title_(title_)
  , photo_(std::move(photo_))
  , accent_color_id_(accent_color_id_)
  , description_(description_)
  , member_count_(member_count_)
  , member_user_ids_(std::move(member_user_ids_))
  , subscription_info_(std::move(subscription_info_))
  , creates_join_request_(creates_join_request_)
  , is_public_(is_public_)
  , verification_status_(std::move(verification_status_))
{}

const std::int32_t chatInviteLinkInfo::ID;

object_ptr<chatInviteLinkInfo> chatInviteLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkInfo> res = make_object<chatInviteLinkInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->accessible_for_ = env->GetIntField(p, res->accessible_for_fieldID);
  res->type_ = jni::fetch_tl_object<InviteLinkChatType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->photo_ = jni::fetch_tl_object<chatPhotoInfo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->member_count_ = env->GetIntField(p, res->member_count_fieldID);
  res->member_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->member_user_ids_fieldID));
  res->subscription_info_ = jni::fetch_tl_object<chatInviteLinkSubscriptionInfo>(env, jni::fetch_object(env, p, res->subscription_info_fieldID));
  res->creates_join_request_ = (env->GetBooleanField(p, res->creates_join_request_fieldID) != 0);
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  res->verification_status_ = jni::fetch_tl_object<verificationStatus>(env, jni::fetch_object(env, p, res->verification_status_fieldID));
  return res;
}

void chatInviteLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, accessible_for_fieldID, accessible_for_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, member_count_fieldID, member_count_);
  { auto arr_tmp_ = jni::store_vector(env, member_user_ids_); if (arr_tmp_) { env->SetObjectField(s, member_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (subscription_info_ != nullptr) { jobject next; subscription_info_->store(env, next); if (next) { env->SetObjectField(s, subscription_info_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, creates_join_request_fieldID, creates_join_request_);
  env->SetBooleanField(s, is_public_fieldID, is_public_);
  if (verification_status_ != nullptr) { jobject next; verification_status_->store(env, next); if (next) { env->SetObjectField(s, verification_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatInviteLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("accessibleFor", accessible_for_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("title", title_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("description", description_);
    s.store_field("memberCount", member_count_);
    { s.store_vector_begin("memberUserIds", member_user_ids_.size()); for (const auto &_value : member_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_object_field("subscriptionInfo", static_cast<const BaseObject *>(subscription_info_.get()));
    s.store_field("createsJoinRequest", creates_join_request_);
    s.store_field("isPublic", is_public_);
    s.store_object_field("verificationStatus", static_cast<const BaseObject *>(verification_status_.get()));
    s.store_class_end();
  }
}

void chatInviteLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    accessible_for_fieldID = jni::get_field_id(env, Class, "accessibleFor", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InviteLinkChatType;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoInfo;").c_str());
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", "I");
    member_user_ids_fieldID = jni::get_field_id(env, Class, "memberUserIds", "[J");
    subscription_info_fieldID = jni::get_field_id(env, Class, "subscriptionInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLinkSubscriptionInfo;").c_str());
    creates_join_request_fieldID = jni::get_field_id(env, Class, "createsJoinRequest", "Z");
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    verification_status_fieldID = jni::get_field_id(env, Class, "verificationStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VerificationStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkMember::Class;
jfieldID chatInviteLinkMember::user_id_fieldID;
jfieldID chatInviteLinkMember::joined_chat_date_fieldID;
jfieldID chatInviteLinkMember::via_chat_folder_invite_link_fieldID;
jfieldID chatInviteLinkMember::approver_user_id_fieldID;

chatInviteLinkMember::chatInviteLinkMember()
  : user_id_()
  , joined_chat_date_()
  , via_chat_folder_invite_link_()
  , approver_user_id_()
{}

chatInviteLinkMember::chatInviteLinkMember(int53 user_id_, int32 joined_chat_date_, bool via_chat_folder_invite_link_, int53 approver_user_id_)
  : user_id_(user_id_)
  , joined_chat_date_(joined_chat_date_)
  , via_chat_folder_invite_link_(via_chat_folder_invite_link_)
  , approver_user_id_(approver_user_id_)
{}

const std::int32_t chatInviteLinkMember::ID;

object_ptr<chatInviteLinkMember> chatInviteLinkMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkMember> res = make_object<chatInviteLinkMember>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->joined_chat_date_ = env->GetIntField(p, res->joined_chat_date_fieldID);
  res->via_chat_folder_invite_link_ = (env->GetBooleanField(p, res->via_chat_folder_invite_link_fieldID) != 0);
  res->approver_user_id_ = env->GetLongField(p, res->approver_user_id_fieldID);
  return res;
}

void chatInviteLinkMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, joined_chat_date_fieldID, joined_chat_date_);
  env->SetBooleanField(s, via_chat_folder_invite_link_fieldID, via_chat_folder_invite_link_);
  env->SetLongField(s, approver_user_id_fieldID, approver_user_id_);
}

void chatInviteLinkMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkMember");
    s.store_field("userId", user_id_);
    s.store_field("joinedChatDate", joined_chat_date_);
    s.store_field("viaChatFolderInviteLink", via_chat_folder_invite_link_);
    s.store_field("approverUserId", approver_user_id_);
    s.store_class_end();
  }
}

void chatInviteLinkMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkMember").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    joined_chat_date_fieldID = jni::get_field_id(env, Class, "joinedChatDate", "I");
    via_chat_folder_invite_link_fieldID = jni::get_field_id(env, Class, "viaChatFolderInviteLink", "Z");
    approver_user_id_fieldID = jni::get_field_id(env, Class, "approverUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkMembers::Class;
jfieldID chatInviteLinkMembers::total_count_fieldID;
jfieldID chatInviteLinkMembers::members_fieldID;

chatInviteLinkMembers::chatInviteLinkMembers()
  : total_count_()
  , members_()
{}

chatInviteLinkMembers::chatInviteLinkMembers(int32 total_count_, array<object_ptr<chatInviteLinkMember>> &&members_)
  : total_count_(total_count_)
  , members_(std::move(members_))
{}

const std::int32_t chatInviteLinkMembers::ID;

object_ptr<chatInviteLinkMembers> chatInviteLinkMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkMembers> res = make_object<chatInviteLinkMembers>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->members_ = jni::FetchVector<chatInviteLinkMember>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->members_fieldID));
  return res;
}

void chatInviteLinkMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, members_); if (arr_tmp_) { env->SetObjectField(s, members_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatInviteLinkMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkMembers");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("members", members_.size()); for (const auto &_value : members_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatInviteLinkMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkMembers").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    members_fieldID = jni::get_field_id(env, Class, "members", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatInviteLinkMember;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinkSubscriptionInfo::Class;
jfieldID chatInviteLinkSubscriptionInfo::pricing_fieldID;
jfieldID chatInviteLinkSubscriptionInfo::can_reuse_fieldID;
jfieldID chatInviteLinkSubscriptionInfo::form_id_fieldID;

chatInviteLinkSubscriptionInfo::chatInviteLinkSubscriptionInfo()
  : pricing_()
  , can_reuse_()
  , form_id_()
{}

chatInviteLinkSubscriptionInfo::chatInviteLinkSubscriptionInfo(object_ptr<starSubscriptionPricing> &&pricing_, bool can_reuse_, int64 form_id_)
  : pricing_(std::move(pricing_))
  , can_reuse_(can_reuse_)
  , form_id_(form_id_)
{}

const std::int32_t chatInviteLinkSubscriptionInfo::ID;

object_ptr<chatInviteLinkSubscriptionInfo> chatInviteLinkSubscriptionInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinkSubscriptionInfo> res = make_object<chatInviteLinkSubscriptionInfo>();
  res->pricing_ = jni::fetch_tl_object<starSubscriptionPricing>(env, jni::fetch_object(env, p, res->pricing_fieldID));
  res->can_reuse_ = (env->GetBooleanField(p, res->can_reuse_fieldID) != 0);
  res->form_id_ = env->GetLongField(p, res->form_id_fieldID);
  return res;
}

void chatInviteLinkSubscriptionInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (pricing_ != nullptr) { jobject next; pricing_->store(env, next); if (next) { env->SetObjectField(s, pricing_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_reuse_fieldID, can_reuse_);
  env->SetLongField(s, form_id_fieldID, form_id_);
}

void chatInviteLinkSubscriptionInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinkSubscriptionInfo");
    s.store_object_field("pricing", static_cast<const BaseObject *>(pricing_.get()));
    s.store_field("canReuse", can_reuse_);
    s.store_field("formId", form_id_);
    s.store_class_end();
  }
}

void chatInviteLinkSubscriptionInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinkSubscriptionInfo").c_str());
    pricing_fieldID = jni::get_field_id(env, Class, "pricing", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionPricing;").c_str());
    can_reuse_fieldID = jni::get_field_id(env, Class, "canReuse", "Z");
    form_id_fieldID = jni::get_field_id(env, Class, "formId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatInviteLinks::Class;
jfieldID chatInviteLinks::total_count_fieldID;
jfieldID chatInviteLinks::invite_links_fieldID;

chatInviteLinks::chatInviteLinks()
  : total_count_()
  , invite_links_()
{}

chatInviteLinks::chatInviteLinks(int32 total_count_, array<object_ptr<chatInviteLink>> &&invite_links_)
  : total_count_(total_count_)
  , invite_links_(std::move(invite_links_))
{}

const std::int32_t chatInviteLinks::ID;

object_ptr<chatInviteLinks> chatInviteLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatInviteLinks> res = make_object<chatInviteLinks>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->invite_links_ = jni::FetchVector<chatInviteLink>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->invite_links_fieldID));
  return res;
}

void chatInviteLinks::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, invite_links_); if (arr_tmp_) { env->SetObjectField(s, invite_links_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatInviteLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatInviteLinks");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("inviteLinks", invite_links_.size()); for (const auto &_value : invite_links_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatInviteLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatInviteLinks").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    invite_links_fieldID = jni::get_field_id(env, Class, "inviteLinks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatJoinRequest::Class;
jfieldID chatJoinRequest::user_id_fieldID;
jfieldID chatJoinRequest::date_fieldID;
jfieldID chatJoinRequest::bio_fieldID;

chatJoinRequest::chatJoinRequest()
  : user_id_()
  , date_()
  , bio_()
{}

chatJoinRequest::chatJoinRequest(int53 user_id_, int32 date_, string const &bio_)
  : user_id_(user_id_)
  , date_(date_)
  , bio_(bio_)
{}

const std::int32_t chatJoinRequest::ID;

object_ptr<chatJoinRequest> chatJoinRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatJoinRequest> res = make_object<chatJoinRequest>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->bio_ = jni::fetch_string(env, p, res->bio_fieldID);
  return res;
}

void chatJoinRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, date_fieldID, date_);
  { jstring nextString = jni::to_jstring(env, bio_); if (nextString) { env->SetObjectField(s, bio_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatJoinRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatJoinRequest");
    s.store_field("userId", user_id_);
    s.store_field("date", date_);
    s.store_field("bio", bio_);
    s.store_class_end();
  }
}

void chatJoinRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatJoinRequest").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    bio_fieldID = jni::get_field_id(env, Class, "bio", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatJoinRequests::Class;
jfieldID chatJoinRequests::total_count_fieldID;
jfieldID chatJoinRequests::requests_fieldID;

chatJoinRequests::chatJoinRequests()
  : total_count_()
  , requests_()
{}

chatJoinRequests::chatJoinRequests(int32 total_count_, array<object_ptr<chatJoinRequest>> &&requests_)
  : total_count_(total_count_)
  , requests_(std::move(requests_))
{}

const std::int32_t chatJoinRequests::ID;

object_ptr<chatJoinRequests> chatJoinRequests::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatJoinRequests> res = make_object<chatJoinRequests>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->requests_ = jni::FetchVector<chatJoinRequest>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->requests_fieldID));
  return res;
}

void chatJoinRequests::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, requests_); if (arr_tmp_) { env->SetObjectField(s, requests_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatJoinRequests::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatJoinRequests");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("requests", requests_.size()); for (const auto &_value : requests_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatJoinRequests::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatJoinRequests").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    requests_fieldID = jni::get_field_id(env, Class, "requests", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatJoinRequest;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatJoinRequestsInfo::Class;
jfieldID chatJoinRequestsInfo::total_count_fieldID;
jfieldID chatJoinRequestsInfo::user_ids_fieldID;

chatJoinRequestsInfo::chatJoinRequestsInfo()
  : total_count_()
  , user_ids_()
{}

chatJoinRequestsInfo::chatJoinRequestsInfo(int32 total_count_, array<int53> &&user_ids_)
  : total_count_(total_count_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t chatJoinRequestsInfo::ID;

object_ptr<chatJoinRequestsInfo> chatJoinRequestsInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatJoinRequestsInfo> res = make_object<chatJoinRequestsInfo>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void chatJoinRequestsInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatJoinRequestsInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatJoinRequestsInfo");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatJoinRequestsInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatJoinRequestsInfo").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass ChatList::Class;

object_ptr<ChatList> ChatList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatListMain::ID:
      return chatListMain::fetch(env, p);
    case chatListArchive::ID:
      return chatListArchive::fetch(env, p);
    case chatListFolder::ID:
      return chatListFolder::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatList").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatListMain::Class;

chatListMain::chatListMain() {
}

const std::int32_t chatListMain::ID;

object_ptr<ChatList> chatListMain::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatListMain>();
}

void chatListMain::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatListMain::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatListMain");
    s.store_class_end();
  }
}

void chatListMain::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatListMain").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatListArchive::Class;

chatListArchive::chatListArchive() {
}

const std::int32_t chatListArchive::ID;

object_ptr<ChatList> chatListArchive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatListArchive>();
}

void chatListArchive::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatListArchive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatListArchive");
    s.store_class_end();
  }
}

void chatListArchive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatListArchive").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatListFolder::Class;
jfieldID chatListFolder::chat_folder_id_fieldID;

chatListFolder::chatListFolder()
  : chat_folder_id_()
{}

chatListFolder::chatListFolder(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t chatListFolder::ID;

object_ptr<ChatList> chatListFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatListFolder> res = make_object<chatListFolder>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return std::move(res);
}

void chatListFolder::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, chat_folder_id_fieldID, chat_folder_id_);
}

void chatListFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatListFolder");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

void chatListFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatListFolder").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatLists::Class;
jfieldID chatLists::chat_lists_fieldID;

chatLists::chatLists()
  : chat_lists_()
{}

chatLists::chatLists(array<object_ptr<ChatList>> &&chat_lists_)
  : chat_lists_(std::move(chat_lists_))
{}

const std::int32_t chatLists::ID;

object_ptr<chatLists> chatLists::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatLists> res = make_object<chatLists>();
  res->chat_lists_ = jni::FetchVector<ChatList>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chat_lists_fieldID));
  return res;
}

void chatLists::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_lists_); if (arr_tmp_) { env->SetObjectField(s, chat_lists_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatLists::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatLists");
    { s.store_vector_begin("chatLists", chat_lists_.size()); for (const auto &_value : chat_lists_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatLists::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatLists").c_str());
    chat_lists_fieldID = jni::get_field_id(env, Class, "chatLists", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatLocation::Class;
jfieldID chatLocation::location_fieldID;
jfieldID chatLocation::address_fieldID;

chatLocation::chatLocation()
  : location_()
  , address_()
{}

chatLocation::chatLocation(object_ptr<location> &&location_, string const &address_)
  : location_(std::move(location_))
  , address_(address_)
{}

const std::int32_t chatLocation::ID;

object_ptr<chatLocation> chatLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatLocation> res = make_object<chatLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->address_ = jni::fetch_string(env, p, res->address_fieldID);
  return res;
}

void chatLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, address_); if (nextString) { env->SetObjectField(s, address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("address", address_);
    s.store_class_end();
  }
}

void chatLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatMember::Class;
jfieldID chatMember::member_id_fieldID;
jfieldID chatMember::inviter_user_id_fieldID;
jfieldID chatMember::joined_chat_date_fieldID;
jfieldID chatMember::status_fieldID;

chatMember::chatMember()
  : member_id_()
  , inviter_user_id_()
  , joined_chat_date_()
  , status_()
{}

chatMember::chatMember(object_ptr<MessageSender> &&member_id_, int53 inviter_user_id_, int32 joined_chat_date_, object_ptr<ChatMemberStatus> &&status_)
  : member_id_(std::move(member_id_))
  , inviter_user_id_(inviter_user_id_)
  , joined_chat_date_(joined_chat_date_)
  , status_(std::move(status_))
{}

const std::int32_t chatMember::ID;

object_ptr<chatMember> chatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMember> res = make_object<chatMember>();
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  res->inviter_user_id_ = env->GetLongField(p, res->inviter_user_id_fieldID);
  res->joined_chat_date_ = env->GetIntField(p, res->joined_chat_date_fieldID);
  res->status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  return res;
}

void chatMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (member_id_ != nullptr) { jobject next; member_id_->store(env, next); if (next) { env->SetObjectField(s, member_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, inviter_user_id_fieldID, inviter_user_id_);
  env->SetIntField(s, joined_chat_date_fieldID, joined_chat_date_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMember");
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_field("inviterUserId", inviter_user_id_);
    s.store_field("joinedChatDate", joined_chat_date_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_class_end();
  }
}

void chatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMember").c_str());
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    inviter_user_id_fieldID = jni::get_field_id(env, Class, "inviterUserId", "J");
    joined_chat_date_fieldID = jni::get_field_id(env, Class, "joinedChatDate", "I");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatMemberStatus::Class;

object_ptr<ChatMemberStatus> ChatMemberStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatMemberStatusCreator::ID:
      return chatMemberStatusCreator::fetch(env, p);
    case chatMemberStatusAdministrator::ID:
      return chatMemberStatusAdministrator::fetch(env, p);
    case chatMemberStatusMember::ID:
      return chatMemberStatusMember::fetch(env, p);
    case chatMemberStatusRestricted::ID:
      return chatMemberStatusRestricted::fetch(env, p);
    case chatMemberStatusLeft::ID:
      return chatMemberStatusLeft::fetch(env, p);
    case chatMemberStatusBanned::ID:
      return chatMemberStatusBanned::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatMemberStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusCreator::Class;
jfieldID chatMemberStatusCreator::custom_title_fieldID;
jfieldID chatMemberStatusCreator::is_anonymous_fieldID;
jfieldID chatMemberStatusCreator::is_member_fieldID;

chatMemberStatusCreator::chatMemberStatusCreator()
  : custom_title_()
  , is_anonymous_()
  , is_member_()
{}

chatMemberStatusCreator::chatMemberStatusCreator(string const &custom_title_, bool is_anonymous_, bool is_member_)
  : custom_title_(custom_title_)
  , is_anonymous_(is_anonymous_)
  , is_member_(is_member_)
{}

const std::int32_t chatMemberStatusCreator::ID;

object_ptr<ChatMemberStatus> chatMemberStatusCreator::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMemberStatusCreator> res = make_object<chatMemberStatusCreator>();
  res->custom_title_ = jni::fetch_string(env, p, res->custom_title_fieldID);
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  res->is_member_ = (env->GetBooleanField(p, res->is_member_fieldID) != 0);
  return std::move(res);
}

void chatMemberStatusCreator::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, custom_title_); if (nextString) { env->SetObjectField(s, custom_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
  env->SetBooleanField(s, is_member_fieldID, is_member_);
}

void chatMemberStatusCreator::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusCreator");
    s.store_field("customTitle", custom_title_);
    s.store_field("isAnonymous", is_anonymous_);
    s.store_field("isMember", is_member_);
    s.store_class_end();
  }
}

void chatMemberStatusCreator::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusCreator").c_str());
    custom_title_fieldID = jni::get_field_id(env, Class, "customTitle", "Ljava/lang/String;");
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    is_member_fieldID = jni::get_field_id(env, Class, "isMember", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusAdministrator::Class;
jfieldID chatMemberStatusAdministrator::custom_title_fieldID;
jfieldID chatMemberStatusAdministrator::can_be_edited_fieldID;
jfieldID chatMemberStatusAdministrator::rights_fieldID;

chatMemberStatusAdministrator::chatMemberStatusAdministrator()
  : custom_title_()
  , can_be_edited_()
  , rights_()
{}

chatMemberStatusAdministrator::chatMemberStatusAdministrator(string const &custom_title_, bool can_be_edited_, object_ptr<chatAdministratorRights> &&rights_)
  : custom_title_(custom_title_)
  , can_be_edited_(can_be_edited_)
  , rights_(std::move(rights_))
{}

const std::int32_t chatMemberStatusAdministrator::ID;

object_ptr<ChatMemberStatus> chatMemberStatusAdministrator::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMemberStatusAdministrator> res = make_object<chatMemberStatusAdministrator>();
  res->custom_title_ = jni::fetch_string(env, p, res->custom_title_fieldID);
  res->can_be_edited_ = (env->GetBooleanField(p, res->can_be_edited_fieldID) != 0);
  res->rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->rights_fieldID));
  return std::move(res);
}

void chatMemberStatusAdministrator::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, custom_title_); if (nextString) { env->SetObjectField(s, custom_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, can_be_edited_fieldID, can_be_edited_);
  if (rights_ != nullptr) { jobject next; rights_->store(env, next); if (next) { env->SetObjectField(s, rights_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatMemberStatusAdministrator::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusAdministrator");
    s.store_field("customTitle", custom_title_);
    s.store_field("canBeEdited", can_be_edited_);
    s.store_object_field("rights", static_cast<const BaseObject *>(rights_.get()));
    s.store_class_end();
  }
}

void chatMemberStatusAdministrator::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusAdministrator").c_str());
    custom_title_fieldID = jni::get_field_id(env, Class, "customTitle", "Ljava/lang/String;");
    can_be_edited_fieldID = jni::get_field_id(env, Class, "canBeEdited", "Z");
    rights_fieldID = jni::get_field_id(env, Class, "rights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusMember::Class;
jfieldID chatMemberStatusMember::member_until_date_fieldID;

chatMemberStatusMember::chatMemberStatusMember()
  : member_until_date_()
{}

chatMemberStatusMember::chatMemberStatusMember(int32 member_until_date_)
  : member_until_date_(member_until_date_)
{}

const std::int32_t chatMemberStatusMember::ID;

object_ptr<ChatMemberStatus> chatMemberStatusMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMemberStatusMember> res = make_object<chatMemberStatusMember>();
  res->member_until_date_ = env->GetIntField(p, res->member_until_date_fieldID);
  return std::move(res);
}

void chatMemberStatusMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, member_until_date_fieldID, member_until_date_);
}

void chatMemberStatusMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusMember");
    s.store_field("memberUntilDate", member_until_date_);
    s.store_class_end();
  }
}

void chatMemberStatusMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusMember").c_str());
    member_until_date_fieldID = jni::get_field_id(env, Class, "memberUntilDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusRestricted::Class;
jfieldID chatMemberStatusRestricted::is_member_fieldID;
jfieldID chatMemberStatusRestricted::restricted_until_date_fieldID;
jfieldID chatMemberStatusRestricted::permissions_fieldID;

chatMemberStatusRestricted::chatMemberStatusRestricted()
  : is_member_()
  , restricted_until_date_()
  , permissions_()
{}

chatMemberStatusRestricted::chatMemberStatusRestricted(bool is_member_, int32 restricted_until_date_, object_ptr<chatPermissions> &&permissions_)
  : is_member_(is_member_)
  , restricted_until_date_(restricted_until_date_)
  , permissions_(std::move(permissions_))
{}

const std::int32_t chatMemberStatusRestricted::ID;

object_ptr<ChatMemberStatus> chatMemberStatusRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMemberStatusRestricted> res = make_object<chatMemberStatusRestricted>();
  res->is_member_ = (env->GetBooleanField(p, res->is_member_fieldID) != 0);
  res->restricted_until_date_ = env->GetIntField(p, res->restricted_until_date_fieldID);
  res->permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->permissions_fieldID));
  return std::move(res);
}

void chatMemberStatusRestricted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_member_fieldID, is_member_);
  env->SetIntField(s, restricted_until_date_fieldID, restricted_until_date_);
  if (permissions_ != nullptr) { jobject next; permissions_->store(env, next); if (next) { env->SetObjectField(s, permissions_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatMemberStatusRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusRestricted");
    s.store_field("isMember", is_member_);
    s.store_field("restrictedUntilDate", restricted_until_date_);
    s.store_object_field("permissions", static_cast<const BaseObject *>(permissions_.get()));
    s.store_class_end();
  }
}

void chatMemberStatusRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusRestricted").c_str());
    is_member_fieldID = jni::get_field_id(env, Class, "isMember", "Z");
    restricted_until_date_fieldID = jni::get_field_id(env, Class, "restrictedUntilDate", "I");
    permissions_fieldID = jni::get_field_id(env, Class, "permissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusLeft::Class;

chatMemberStatusLeft::chatMemberStatusLeft() {
}

const std::int32_t chatMemberStatusLeft::ID;

object_ptr<ChatMemberStatus> chatMemberStatusLeft::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMemberStatusLeft>();
}

void chatMemberStatusLeft::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatMemberStatusLeft::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusLeft");
    s.store_class_end();
  }
}

void chatMemberStatusLeft::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusLeft").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMemberStatusBanned::Class;
jfieldID chatMemberStatusBanned::banned_until_date_fieldID;

chatMemberStatusBanned::chatMemberStatusBanned()
  : banned_until_date_()
{}

chatMemberStatusBanned::chatMemberStatusBanned(int32 banned_until_date_)
  : banned_until_date_(banned_until_date_)
{}

const std::int32_t chatMemberStatusBanned::ID;

object_ptr<ChatMemberStatus> chatMemberStatusBanned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMemberStatusBanned> res = make_object<chatMemberStatusBanned>();
  res->banned_until_date_ = env->GetIntField(p, res->banned_until_date_fieldID);
  return std::move(res);
}

void chatMemberStatusBanned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, banned_until_date_fieldID, banned_until_date_);
}

void chatMemberStatusBanned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMemberStatusBanned");
    s.store_field("bannedUntilDate", banned_until_date_);
    s.store_class_end();
  }
}

void chatMemberStatusBanned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMemberStatusBanned").c_str());
    banned_until_date_fieldID = jni::get_field_id(env, Class, "bannedUntilDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatMembers::Class;
jfieldID chatMembers::total_count_fieldID;
jfieldID chatMembers::members_fieldID;

chatMembers::chatMembers()
  : total_count_()
  , members_()
{}

chatMembers::chatMembers(int32 total_count_, array<object_ptr<chatMember>> &&members_)
  : total_count_(total_count_)
  , members_(std::move(members_))
{}

const std::int32_t chatMembers::ID;

object_ptr<chatMembers> chatMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMembers> res = make_object<chatMembers>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->members_ = jni::FetchVector<chatMember>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->members_fieldID));
  return res;
}

void chatMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, members_); if (arr_tmp_) { env->SetObjectField(s, members_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembers");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("members", members_.size()); for (const auto &_value : members_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembers").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    members_fieldID = jni::get_field_id(env, Class, "members", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatMember;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatMembersFilter::Class;

object_ptr<ChatMembersFilter> ChatMembersFilter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatMembersFilterContacts::ID:
      return chatMembersFilterContacts::fetch(env, p);
    case chatMembersFilterAdministrators::ID:
      return chatMembersFilterAdministrators::fetch(env, p);
    case chatMembersFilterMembers::ID:
      return chatMembersFilterMembers::fetch(env, p);
    case chatMembersFilterMention::ID:
      return chatMembersFilterMention::fetch(env, p);
    case chatMembersFilterRestricted::ID:
      return chatMembersFilterRestricted::fetch(env, p);
    case chatMembersFilterBanned::ID:
      return chatMembersFilterBanned::fetch(env, p);
    case chatMembersFilterBots::ID:
      return chatMembersFilterBots::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatMembersFilter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilter").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterContacts::Class;

chatMembersFilterContacts::chatMembersFilterContacts() {
}

const std::int32_t chatMembersFilterContacts::ID;

object_ptr<ChatMembersFilter> chatMembersFilterContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterContacts>();
}

void chatMembersFilterContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterContacts");
    s.store_class_end();
  }
}

void chatMembersFilterContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterAdministrators::Class;

chatMembersFilterAdministrators::chatMembersFilterAdministrators() {
}

const std::int32_t chatMembersFilterAdministrators::ID;

object_ptr<ChatMembersFilter> chatMembersFilterAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterAdministrators>();
}

void chatMembersFilterAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterAdministrators");
    s.store_class_end();
  }
}

void chatMembersFilterAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterAdministrators").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterMembers::Class;

chatMembersFilterMembers::chatMembersFilterMembers() {
}

const std::int32_t chatMembersFilterMembers::ID;

object_ptr<ChatMembersFilter> chatMembersFilterMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterMembers>();
}

void chatMembersFilterMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterMembers");
    s.store_class_end();
  }
}

void chatMembersFilterMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterMembers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterMention::Class;
jfieldID chatMembersFilterMention::message_thread_id_fieldID;

chatMembersFilterMention::chatMembersFilterMention()
  : message_thread_id_()
{}

chatMembersFilterMention::chatMembersFilterMention(int53 message_thread_id_)
  : message_thread_id_(message_thread_id_)
{}

const std::int32_t chatMembersFilterMention::ID;

object_ptr<ChatMembersFilter> chatMembersFilterMention::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMembersFilterMention> res = make_object<chatMembersFilterMention>();
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return std::move(res);
}

void chatMembersFilterMention::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterMention");
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

void chatMembersFilterMention::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterMention").c_str());
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterRestricted::Class;

chatMembersFilterRestricted::chatMembersFilterRestricted() {
}

const std::int32_t chatMembersFilterRestricted::ID;

object_ptr<ChatMembersFilter> chatMembersFilterRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterRestricted>();
}

void chatMembersFilterRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterRestricted");
    s.store_class_end();
  }
}

void chatMembersFilterRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterRestricted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterBanned::Class;

chatMembersFilterBanned::chatMembersFilterBanned() {
}

const std::int32_t chatMembersFilterBanned::ID;

object_ptr<ChatMembersFilter> chatMembersFilterBanned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterBanned>();
}

void chatMembersFilterBanned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterBanned");
    s.store_class_end();
  }
}

void chatMembersFilterBanned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterBanned").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMembersFilterBots::Class;

chatMembersFilterBots::chatMembersFilterBots() {
}

const std::int32_t chatMembersFilterBots::ID;

object_ptr<ChatMembersFilter> chatMembersFilterBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatMembersFilterBots>();
}

void chatMembersFilterBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMembersFilterBots");
    s.store_class_end();
  }
}

void chatMembersFilterBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMembersFilterBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatMessageSender::Class;
jfieldID chatMessageSender::sender_fieldID;
jfieldID chatMessageSender::needs_premium_fieldID;

chatMessageSender::chatMessageSender()
  : sender_()
  , needs_premium_()
{}

chatMessageSender::chatMessageSender(object_ptr<MessageSender> &&sender_, bool needs_premium_)
  : sender_(std::move(sender_))
  , needs_premium_(needs_premium_)
{}

const std::int32_t chatMessageSender::ID;

object_ptr<chatMessageSender> chatMessageSender::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMessageSender> res = make_object<chatMessageSender>();
  res->sender_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_fieldID));
  res->needs_premium_ = (env->GetBooleanField(p, res->needs_premium_fieldID) != 0);
  return res;
}

void chatMessageSender::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sender_ != nullptr) { jobject next; sender_->store(env, next); if (next) { env->SetObjectField(s, sender_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, needs_premium_fieldID, needs_premium_);
}

void chatMessageSender::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMessageSender");
    s.store_object_field("sender", static_cast<const BaseObject *>(sender_.get()));
    s.store_field("needsPremium", needs_premium_);
    s.store_class_end();
  }
}

void chatMessageSender::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMessageSender").c_str());
    sender_fieldID = jni::get_field_id(env, Class, "sender", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    needs_premium_fieldID = jni::get_field_id(env, Class, "needsPremium", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatMessageSenders::Class;
jfieldID chatMessageSenders::senders_fieldID;

chatMessageSenders::chatMessageSenders()
  : senders_()
{}

chatMessageSenders::chatMessageSenders(array<object_ptr<chatMessageSender>> &&senders_)
  : senders_(std::move(senders_))
{}

const std::int32_t chatMessageSenders::ID;

object_ptr<chatMessageSenders> chatMessageSenders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatMessageSenders> res = make_object<chatMessageSenders>();
  res->senders_ = jni::FetchVector<chatMessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->senders_fieldID));
  return res;
}

void chatMessageSenders::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, senders_); if (arr_tmp_) { env->SetObjectField(s, senders_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatMessageSenders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatMessageSenders");
    { s.store_vector_begin("senders", senders_.size()); for (const auto &_value : senders_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatMessageSenders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatMessageSenders").c_str());
    senders_fieldID = jni::get_field_id(env, Class, "senders", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatMessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatNotificationSettings::Class;
jfieldID chatNotificationSettings::use_default_mute_for_fieldID;
jfieldID chatNotificationSettings::mute_for_fieldID;
jfieldID chatNotificationSettings::use_default_sound_fieldID;
jfieldID chatNotificationSettings::sound_id_fieldID;
jfieldID chatNotificationSettings::use_default_show_preview_fieldID;
jfieldID chatNotificationSettings::show_preview_fieldID;
jfieldID chatNotificationSettings::use_default_mute_stories_fieldID;
jfieldID chatNotificationSettings::mute_stories_fieldID;
jfieldID chatNotificationSettings::use_default_story_sound_fieldID;
jfieldID chatNotificationSettings::story_sound_id_fieldID;
jfieldID chatNotificationSettings::use_default_show_story_poster_fieldID;
jfieldID chatNotificationSettings::show_story_poster_fieldID;
jfieldID chatNotificationSettings::use_default_disable_pinned_message_notifications_fieldID;
jfieldID chatNotificationSettings::disable_pinned_message_notifications_fieldID;
jfieldID chatNotificationSettings::use_default_disable_mention_notifications_fieldID;
jfieldID chatNotificationSettings::disable_mention_notifications_fieldID;

chatNotificationSettings::chatNotificationSettings()
  : use_default_mute_for_()
  , mute_for_()
  , use_default_sound_()
  , sound_id_()
  , use_default_show_preview_()
  , show_preview_()
  , use_default_mute_stories_()
  , mute_stories_()
  , use_default_story_sound_()
  , story_sound_id_()
  , use_default_show_story_poster_()
  , show_story_poster_()
  , use_default_disable_pinned_message_notifications_()
  , disable_pinned_message_notifications_()
  , use_default_disable_mention_notifications_()
  , disable_mention_notifications_()
{}

chatNotificationSettings::chatNotificationSettings(bool use_default_mute_for_, int32 mute_for_, bool use_default_sound_, int64 sound_id_, bool use_default_show_preview_, bool show_preview_, bool use_default_mute_stories_, bool mute_stories_, bool use_default_story_sound_, int64 story_sound_id_, bool use_default_show_story_poster_, bool show_story_poster_, bool use_default_disable_pinned_message_notifications_, bool disable_pinned_message_notifications_, bool use_default_disable_mention_notifications_, bool disable_mention_notifications_)
  : use_default_mute_for_(use_default_mute_for_)
  , mute_for_(mute_for_)
  , use_default_sound_(use_default_sound_)
  , sound_id_(sound_id_)
  , use_default_show_preview_(use_default_show_preview_)
  , show_preview_(show_preview_)
  , use_default_mute_stories_(use_default_mute_stories_)
  , mute_stories_(mute_stories_)
  , use_default_story_sound_(use_default_story_sound_)
  , story_sound_id_(story_sound_id_)
  , use_default_show_story_poster_(use_default_show_story_poster_)
  , show_story_poster_(show_story_poster_)
  , use_default_disable_pinned_message_notifications_(use_default_disable_pinned_message_notifications_)
  , disable_pinned_message_notifications_(disable_pinned_message_notifications_)
  , use_default_disable_mention_notifications_(use_default_disable_mention_notifications_)
  , disable_mention_notifications_(disable_mention_notifications_)
{}

const std::int32_t chatNotificationSettings::ID;

object_ptr<chatNotificationSettings> chatNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatNotificationSettings> res = make_object<chatNotificationSettings>();
  res->use_default_mute_for_ = (env->GetBooleanField(p, res->use_default_mute_for_fieldID) != 0);
  res->mute_for_ = env->GetIntField(p, res->mute_for_fieldID);
  res->use_default_sound_ = (env->GetBooleanField(p, res->use_default_sound_fieldID) != 0);
  res->sound_id_ = env->GetLongField(p, res->sound_id_fieldID);
  res->use_default_show_preview_ = (env->GetBooleanField(p, res->use_default_show_preview_fieldID) != 0);
  res->show_preview_ = (env->GetBooleanField(p, res->show_preview_fieldID) != 0);
  res->use_default_mute_stories_ = (env->GetBooleanField(p, res->use_default_mute_stories_fieldID) != 0);
  res->mute_stories_ = (env->GetBooleanField(p, res->mute_stories_fieldID) != 0);
  res->use_default_story_sound_ = (env->GetBooleanField(p, res->use_default_story_sound_fieldID) != 0);
  res->story_sound_id_ = env->GetLongField(p, res->story_sound_id_fieldID);
  res->use_default_show_story_poster_ = (env->GetBooleanField(p, res->use_default_show_story_poster_fieldID) != 0);
  res->show_story_poster_ = (env->GetBooleanField(p, res->show_story_poster_fieldID) != 0);
  res->use_default_disable_pinned_message_notifications_ = (env->GetBooleanField(p, res->use_default_disable_pinned_message_notifications_fieldID) != 0);
  res->disable_pinned_message_notifications_ = (env->GetBooleanField(p, res->disable_pinned_message_notifications_fieldID) != 0);
  res->use_default_disable_mention_notifications_ = (env->GetBooleanField(p, res->use_default_disable_mention_notifications_fieldID) != 0);
  res->disable_mention_notifications_ = (env->GetBooleanField(p, res->disable_mention_notifications_fieldID) != 0);
  return res;
}

void chatNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, use_default_mute_for_fieldID, use_default_mute_for_);
  env->SetIntField(s, mute_for_fieldID, mute_for_);
  env->SetBooleanField(s, use_default_sound_fieldID, use_default_sound_);
  env->SetLongField(s, sound_id_fieldID, sound_id_);
  env->SetBooleanField(s, use_default_show_preview_fieldID, use_default_show_preview_);
  env->SetBooleanField(s, show_preview_fieldID, show_preview_);
  env->SetBooleanField(s, use_default_mute_stories_fieldID, use_default_mute_stories_);
  env->SetBooleanField(s, mute_stories_fieldID, mute_stories_);
  env->SetBooleanField(s, use_default_story_sound_fieldID, use_default_story_sound_);
  env->SetLongField(s, story_sound_id_fieldID, story_sound_id_);
  env->SetBooleanField(s, use_default_show_story_poster_fieldID, use_default_show_story_poster_);
  env->SetBooleanField(s, show_story_poster_fieldID, show_story_poster_);
  env->SetBooleanField(s, use_default_disable_pinned_message_notifications_fieldID, use_default_disable_pinned_message_notifications_);
  env->SetBooleanField(s, disable_pinned_message_notifications_fieldID, disable_pinned_message_notifications_);
  env->SetBooleanField(s, use_default_disable_mention_notifications_fieldID, use_default_disable_mention_notifications_);
  env->SetBooleanField(s, disable_mention_notifications_fieldID, disable_mention_notifications_);
}

void chatNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatNotificationSettings");
    s.store_field("useDefaultMuteFor", use_default_mute_for_);
    s.store_field("muteFor", mute_for_);
    s.store_field("useDefaultSound", use_default_sound_);
    s.store_field("soundId", sound_id_);
    s.store_field("useDefaultShowPreview", use_default_show_preview_);
    s.store_field("showPreview", show_preview_);
    s.store_field("useDefaultMuteStories", use_default_mute_stories_);
    s.store_field("muteStories", mute_stories_);
    s.store_field("useDefaultStorySound", use_default_story_sound_);
    s.store_field("storySoundId", story_sound_id_);
    s.store_field("useDefaultShowStoryPoster", use_default_show_story_poster_);
    s.store_field("showStoryPoster", show_story_poster_);
    s.store_field("useDefaultDisablePinnedMessageNotifications", use_default_disable_pinned_message_notifications_);
    s.store_field("disablePinnedMessageNotifications", disable_pinned_message_notifications_);
    s.store_field("useDefaultDisableMentionNotifications", use_default_disable_mention_notifications_);
    s.store_field("disableMentionNotifications", disable_mention_notifications_);
    s.store_class_end();
  }
}

void chatNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatNotificationSettings").c_str());
    use_default_mute_for_fieldID = jni::get_field_id(env, Class, "useDefaultMuteFor", "Z");
    mute_for_fieldID = jni::get_field_id(env, Class, "muteFor", "I");
    use_default_sound_fieldID = jni::get_field_id(env, Class, "useDefaultSound", "Z");
    sound_id_fieldID = jni::get_field_id(env, Class, "soundId", "J");
    use_default_show_preview_fieldID = jni::get_field_id(env, Class, "useDefaultShowPreview", "Z");
    show_preview_fieldID = jni::get_field_id(env, Class, "showPreview", "Z");
    use_default_mute_stories_fieldID = jni::get_field_id(env, Class, "useDefaultMuteStories", "Z");
    mute_stories_fieldID = jni::get_field_id(env, Class, "muteStories", "Z");
    use_default_story_sound_fieldID = jni::get_field_id(env, Class, "useDefaultStorySound", "Z");
    story_sound_id_fieldID = jni::get_field_id(env, Class, "storySoundId", "J");
    use_default_show_story_poster_fieldID = jni::get_field_id(env, Class, "useDefaultShowStoryPoster", "Z");
    show_story_poster_fieldID = jni::get_field_id(env, Class, "showStoryPoster", "Z");
    use_default_disable_pinned_message_notifications_fieldID = jni::get_field_id(env, Class, "useDefaultDisablePinnedMessageNotifications", "Z");
    disable_pinned_message_notifications_fieldID = jni::get_field_id(env, Class, "disablePinnedMessageNotifications", "Z");
    use_default_disable_mention_notifications_fieldID = jni::get_field_id(env, Class, "useDefaultDisableMentionNotifications", "Z");
    disable_mention_notifications_fieldID = jni::get_field_id(env, Class, "disableMentionNotifications", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatPermissions::Class;
jfieldID chatPermissions::can_send_basic_messages_fieldID;
jfieldID chatPermissions::can_send_audios_fieldID;
jfieldID chatPermissions::can_send_documents_fieldID;
jfieldID chatPermissions::can_send_photos_fieldID;
jfieldID chatPermissions::can_send_videos_fieldID;
jfieldID chatPermissions::can_send_video_notes_fieldID;
jfieldID chatPermissions::can_send_voice_notes_fieldID;
jfieldID chatPermissions::can_send_polls_fieldID;
jfieldID chatPermissions::can_send_other_messages_fieldID;
jfieldID chatPermissions::can_add_link_previews_fieldID;
jfieldID chatPermissions::can_change_info_fieldID;
jfieldID chatPermissions::can_invite_users_fieldID;
jfieldID chatPermissions::can_pin_messages_fieldID;
jfieldID chatPermissions::can_create_topics_fieldID;

chatPermissions::chatPermissions()
  : can_send_basic_messages_()
  , can_send_audios_()
  , can_send_documents_()
  , can_send_photos_()
  , can_send_videos_()
  , can_send_video_notes_()
  , can_send_voice_notes_()
  , can_send_polls_()
  , can_send_other_messages_()
  , can_add_link_previews_()
  , can_change_info_()
  , can_invite_users_()
  , can_pin_messages_()
  , can_create_topics_()
{}

chatPermissions::chatPermissions(bool can_send_basic_messages_, bool can_send_audios_, bool can_send_documents_, bool can_send_photos_, bool can_send_videos_, bool can_send_video_notes_, bool can_send_voice_notes_, bool can_send_polls_, bool can_send_other_messages_, bool can_add_link_previews_, bool can_change_info_, bool can_invite_users_, bool can_pin_messages_, bool can_create_topics_)
  : can_send_basic_messages_(can_send_basic_messages_)
  , can_send_audios_(can_send_audios_)
  , can_send_documents_(can_send_documents_)
  , can_send_photos_(can_send_photos_)
  , can_send_videos_(can_send_videos_)
  , can_send_video_notes_(can_send_video_notes_)
  , can_send_voice_notes_(can_send_voice_notes_)
  , can_send_polls_(can_send_polls_)
  , can_send_other_messages_(can_send_other_messages_)
  , can_add_link_previews_(can_add_link_previews_)
  , can_change_info_(can_change_info_)
  , can_invite_users_(can_invite_users_)
  , can_pin_messages_(can_pin_messages_)
  , can_create_topics_(can_create_topics_)
{}

const std::int32_t chatPermissions::ID;

object_ptr<chatPermissions> chatPermissions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPermissions> res = make_object<chatPermissions>();
  res->can_send_basic_messages_ = (env->GetBooleanField(p, res->can_send_basic_messages_fieldID) != 0);
  res->can_send_audios_ = (env->GetBooleanField(p, res->can_send_audios_fieldID) != 0);
  res->can_send_documents_ = (env->GetBooleanField(p, res->can_send_documents_fieldID) != 0);
  res->can_send_photos_ = (env->GetBooleanField(p, res->can_send_photos_fieldID) != 0);
  res->can_send_videos_ = (env->GetBooleanField(p, res->can_send_videos_fieldID) != 0);
  res->can_send_video_notes_ = (env->GetBooleanField(p, res->can_send_video_notes_fieldID) != 0);
  res->can_send_voice_notes_ = (env->GetBooleanField(p, res->can_send_voice_notes_fieldID) != 0);
  res->can_send_polls_ = (env->GetBooleanField(p, res->can_send_polls_fieldID) != 0);
  res->can_send_other_messages_ = (env->GetBooleanField(p, res->can_send_other_messages_fieldID) != 0);
  res->can_add_link_previews_ = (env->GetBooleanField(p, res->can_add_link_previews_fieldID) != 0);
  res->can_change_info_ = (env->GetBooleanField(p, res->can_change_info_fieldID) != 0);
  res->can_invite_users_ = (env->GetBooleanField(p, res->can_invite_users_fieldID) != 0);
  res->can_pin_messages_ = (env->GetBooleanField(p, res->can_pin_messages_fieldID) != 0);
  res->can_create_topics_ = (env->GetBooleanField(p, res->can_create_topics_fieldID) != 0);
  return res;
}

void chatPermissions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_send_basic_messages_fieldID, can_send_basic_messages_);
  env->SetBooleanField(s, can_send_audios_fieldID, can_send_audios_);
  env->SetBooleanField(s, can_send_documents_fieldID, can_send_documents_);
  env->SetBooleanField(s, can_send_photos_fieldID, can_send_photos_);
  env->SetBooleanField(s, can_send_videos_fieldID, can_send_videos_);
  env->SetBooleanField(s, can_send_video_notes_fieldID, can_send_video_notes_);
  env->SetBooleanField(s, can_send_voice_notes_fieldID, can_send_voice_notes_);
  env->SetBooleanField(s, can_send_polls_fieldID, can_send_polls_);
  env->SetBooleanField(s, can_send_other_messages_fieldID, can_send_other_messages_);
  env->SetBooleanField(s, can_add_link_previews_fieldID, can_add_link_previews_);
  env->SetBooleanField(s, can_change_info_fieldID, can_change_info_);
  env->SetBooleanField(s, can_invite_users_fieldID, can_invite_users_);
  env->SetBooleanField(s, can_pin_messages_fieldID, can_pin_messages_);
  env->SetBooleanField(s, can_create_topics_fieldID, can_create_topics_);
}

void chatPermissions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPermissions");
    s.store_field("canSendBasicMessages", can_send_basic_messages_);
    s.store_field("canSendAudios", can_send_audios_);
    s.store_field("canSendDocuments", can_send_documents_);
    s.store_field("canSendPhotos", can_send_photos_);
    s.store_field("canSendVideos", can_send_videos_);
    s.store_field("canSendVideoNotes", can_send_video_notes_);
    s.store_field("canSendVoiceNotes", can_send_voice_notes_);
    s.store_field("canSendPolls", can_send_polls_);
    s.store_field("canSendOtherMessages", can_send_other_messages_);
    s.store_field("canAddLinkPreviews", can_add_link_previews_);
    s.store_field("canChangeInfo", can_change_info_);
    s.store_field("canInviteUsers", can_invite_users_);
    s.store_field("canPinMessages", can_pin_messages_);
    s.store_field("canCreateTopics", can_create_topics_);
    s.store_class_end();
  }
}

void chatPermissions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPermissions").c_str());
    can_send_basic_messages_fieldID = jni::get_field_id(env, Class, "canSendBasicMessages", "Z");
    can_send_audios_fieldID = jni::get_field_id(env, Class, "canSendAudios", "Z");
    can_send_documents_fieldID = jni::get_field_id(env, Class, "canSendDocuments", "Z");
    can_send_photos_fieldID = jni::get_field_id(env, Class, "canSendPhotos", "Z");
    can_send_videos_fieldID = jni::get_field_id(env, Class, "canSendVideos", "Z");
    can_send_video_notes_fieldID = jni::get_field_id(env, Class, "canSendVideoNotes", "Z");
    can_send_voice_notes_fieldID = jni::get_field_id(env, Class, "canSendVoiceNotes", "Z");
    can_send_polls_fieldID = jni::get_field_id(env, Class, "canSendPolls", "Z");
    can_send_other_messages_fieldID = jni::get_field_id(env, Class, "canSendOtherMessages", "Z");
    can_add_link_previews_fieldID = jni::get_field_id(env, Class, "canAddLinkPreviews", "Z");
    can_change_info_fieldID = jni::get_field_id(env, Class, "canChangeInfo", "Z");
    can_invite_users_fieldID = jni::get_field_id(env, Class, "canInviteUsers", "Z");
    can_pin_messages_fieldID = jni::get_field_id(env, Class, "canPinMessages", "Z");
    can_create_topics_fieldID = jni::get_field_id(env, Class, "canCreateTopics", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatPhoto::Class;
jfieldID chatPhoto::id_fieldID;
jfieldID chatPhoto::added_date_fieldID;
jfieldID chatPhoto::minithumbnail_fieldID;
jfieldID chatPhoto::sizes_fieldID;
jfieldID chatPhoto::animation_fieldID;
jfieldID chatPhoto::small_animation_fieldID;
jfieldID chatPhoto::sticker_fieldID;

chatPhoto::chatPhoto()
  : id_()
  , added_date_()
  , minithumbnail_()
  , sizes_()
  , animation_()
  , small_animation_()
  , sticker_()
{}

chatPhoto::chatPhoto(int64 id_, int32 added_date_, object_ptr<minithumbnail> &&minithumbnail_, array<object_ptr<photoSize>> &&sizes_, object_ptr<animatedChatPhoto> &&animation_, object_ptr<animatedChatPhoto> &&small_animation_, object_ptr<chatPhotoSticker> &&sticker_)
  : id_(id_)
  , added_date_(added_date_)
  , minithumbnail_(std::move(minithumbnail_))
  , sizes_(std::move(sizes_))
  , animation_(std::move(animation_))
  , small_animation_(std::move(small_animation_))
  , sticker_(std::move(sticker_))
{}

const std::int32_t chatPhoto::ID;

object_ptr<chatPhoto> chatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhoto> res = make_object<chatPhoto>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->added_date_ = env->GetIntField(p, res->added_date_fieldID);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->sizes_ = jni::FetchVector<photoSize>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->sizes_fieldID));
  res->animation_ = jni::fetch_tl_object<animatedChatPhoto>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->small_animation_ = jni::fetch_tl_object<animatedChatPhoto>(env, jni::fetch_object(env, p, res->small_animation_fieldID));
  res->sticker_ = jni::fetch_tl_object<chatPhotoSticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void chatPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, added_date_fieldID, added_date_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, sizes_); if (arr_tmp_) { env->SetObjectField(s, sizes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (small_animation_ != nullptr) { jobject next; small_animation_->store(env, next); if (next) { env->SetObjectField(s, small_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhoto");
    s.store_field("id", id_);
    s.store_field("addedDate", added_date_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    { s.store_vector_begin("sizes", sizes_.size()); for (const auto &_value : sizes_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_object_field("smallAnimation", static_cast<const BaseObject *>(small_animation_.get()));
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void chatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhoto").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    added_date_fieldID = jni::get_field_id(env, Class, "addedDate", "I");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    sizes_fieldID = jni::get_field_id(env, Class, "sizes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PhotoSize;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AnimatedChatPhoto;").c_str());
    small_animation_fieldID = jni::get_field_id(env, Class, "smallAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AnimatedChatPhoto;").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoSticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatPhotoInfo::Class;
jfieldID chatPhotoInfo::small_fieldID;
jfieldID chatPhotoInfo::big_fieldID;
jfieldID chatPhotoInfo::minithumbnail_fieldID;
jfieldID chatPhotoInfo::has_animation_fieldID;
jfieldID chatPhotoInfo::is_personal_fieldID;

chatPhotoInfo::chatPhotoInfo()
  : small_()
  , big_()
  , minithumbnail_()
  , has_animation_()
  , is_personal_()
{}

chatPhotoInfo::chatPhotoInfo(object_ptr<file> &&small_, object_ptr<file> &&big_, object_ptr<minithumbnail> &&minithumbnail_, bool has_animation_, bool is_personal_)
  : small_(std::move(small_))
  , big_(std::move(big_))
  , minithumbnail_(std::move(minithumbnail_))
  , has_animation_(has_animation_)
  , is_personal_(is_personal_)
{}

const std::int32_t chatPhotoInfo::ID;

object_ptr<chatPhotoInfo> chatPhotoInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhotoInfo> res = make_object<chatPhotoInfo>();
  res->small_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->small_fieldID));
  res->big_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->big_fieldID));
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->has_animation_ = (env->GetBooleanField(p, res->has_animation_fieldID) != 0);
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  return res;
}

void chatPhotoInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (small_ != nullptr) { jobject next; small_->store(env, next); if (next) { env->SetObjectField(s, small_fieldID, next); env->DeleteLocalRef(next); } }
  if (big_ != nullptr) { jobject next; big_->store(env, next); if (next) { env->SetObjectField(s, big_fieldID, next); env->DeleteLocalRef(next); } }
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_animation_fieldID, has_animation_);
  env->SetBooleanField(s, is_personal_fieldID, is_personal_);
}

void chatPhotoInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhotoInfo");
    s.store_object_field("small", static_cast<const BaseObject *>(small_.get()));
    s.store_object_field("big", static_cast<const BaseObject *>(big_.get()));
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_field("hasAnimation", has_animation_);
    s.store_field("isPersonal", is_personal_);
    s.store_class_end();
  }
}

void chatPhotoInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotoInfo").c_str());
    small_fieldID = jni::get_field_id(env, Class, "small", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    big_fieldID = jni::get_field_id(env, Class, "big", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    has_animation_fieldID = jni::get_field_id(env, Class, "hasAnimation", "Z");
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatPhotoSticker::Class;
jfieldID chatPhotoSticker::type_fieldID;
jfieldID chatPhotoSticker::background_fill_fieldID;

chatPhotoSticker::chatPhotoSticker()
  : type_()
  , background_fill_()
{}

chatPhotoSticker::chatPhotoSticker(object_ptr<ChatPhotoStickerType> &&type_, object_ptr<BackgroundFill> &&background_fill_)
  : type_(std::move(type_))
  , background_fill_(std::move(background_fill_))
{}

const std::int32_t chatPhotoSticker::ID;

object_ptr<chatPhotoSticker> chatPhotoSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhotoSticker> res = make_object<chatPhotoSticker>();
  res->type_ = jni::fetch_tl_object<ChatPhotoStickerType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->background_fill_ = jni::fetch_tl_object<BackgroundFill>(env, jni::fetch_object(env, p, res->background_fill_fieldID));
  return res;
}

void chatPhotoSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (background_fill_ != nullptr) { jobject next; background_fill_->store(env, next); if (next) { env->SetObjectField(s, background_fill_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatPhotoSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhotoSticker");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("backgroundFill", static_cast<const BaseObject *>(background_fill_.get()));
    s.store_class_end();
  }
}

void chatPhotoSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotoSticker").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoStickerType;").c_str());
    background_fill_fieldID = jni::get_field_id(env, Class, "backgroundFill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundFill;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatPhotoStickerType::Class;

object_ptr<ChatPhotoStickerType> ChatPhotoStickerType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatPhotoStickerTypeRegularOrMask::ID:
      return chatPhotoStickerTypeRegularOrMask::fetch(env, p);
    case chatPhotoStickerTypeCustomEmoji::ID:
      return chatPhotoStickerTypeCustomEmoji::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatPhotoStickerType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotoStickerType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatPhotoStickerTypeRegularOrMask::Class;
jfieldID chatPhotoStickerTypeRegularOrMask::sticker_set_id_fieldID;
jfieldID chatPhotoStickerTypeRegularOrMask::sticker_id_fieldID;

chatPhotoStickerTypeRegularOrMask::chatPhotoStickerTypeRegularOrMask()
  : sticker_set_id_()
  , sticker_id_()
{}

chatPhotoStickerTypeRegularOrMask::chatPhotoStickerTypeRegularOrMask(int64 sticker_set_id_, int64 sticker_id_)
  : sticker_set_id_(sticker_set_id_)
  , sticker_id_(sticker_id_)
{}

const std::int32_t chatPhotoStickerTypeRegularOrMask::ID;

object_ptr<ChatPhotoStickerType> chatPhotoStickerTypeRegularOrMask::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhotoStickerTypeRegularOrMask> res = make_object<chatPhotoStickerTypeRegularOrMask>();
  res->sticker_set_id_ = env->GetLongField(p, res->sticker_set_id_fieldID);
  res->sticker_id_ = env->GetLongField(p, res->sticker_id_fieldID);
  return std::move(res);
}

void chatPhotoStickerTypeRegularOrMask::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, sticker_set_id_fieldID, sticker_set_id_);
  env->SetLongField(s, sticker_id_fieldID, sticker_id_);
}

void chatPhotoStickerTypeRegularOrMask::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhotoStickerTypeRegularOrMask");
    s.store_field("stickerSetId", sticker_set_id_);
    s.store_field("stickerId", sticker_id_);
    s.store_class_end();
  }
}

void chatPhotoStickerTypeRegularOrMask::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotoStickerTypeRegularOrMask").c_str());
    sticker_set_id_fieldID = jni::get_field_id(env, Class, "stickerSetId", "J");
    sticker_id_fieldID = jni::get_field_id(env, Class, "stickerId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatPhotoStickerTypeCustomEmoji::Class;
jfieldID chatPhotoStickerTypeCustomEmoji::custom_emoji_id_fieldID;

chatPhotoStickerTypeCustomEmoji::chatPhotoStickerTypeCustomEmoji()
  : custom_emoji_id_()
{}

chatPhotoStickerTypeCustomEmoji::chatPhotoStickerTypeCustomEmoji(int64 custom_emoji_id_)
  : custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t chatPhotoStickerTypeCustomEmoji::ID;

object_ptr<ChatPhotoStickerType> chatPhotoStickerTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhotoStickerTypeCustomEmoji> res = make_object<chatPhotoStickerTypeCustomEmoji>();
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return std::move(res);
}

void chatPhotoStickerTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
}

void chatPhotoStickerTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhotoStickerTypeCustomEmoji");
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

void chatPhotoStickerTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotoStickerTypeCustomEmoji").c_str());
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatPhotos::Class;
jfieldID chatPhotos::total_count_fieldID;
jfieldID chatPhotos::photos_fieldID;

chatPhotos::chatPhotos()
  : total_count_()
  , photos_()
{}

chatPhotos::chatPhotos(int32 total_count_, array<object_ptr<chatPhoto>> &&photos_)
  : total_count_(total_count_)
  , photos_(std::move(photos_))
{}

const std::int32_t chatPhotos::ID;

object_ptr<chatPhotos> chatPhotos::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPhotos> res = make_object<chatPhotos>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->photos_ = jni::FetchVector<chatPhoto>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->photos_fieldID));
  return res;
}

void chatPhotos::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, photos_); if (arr_tmp_) { env->SetObjectField(s, photos_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatPhotos::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPhotos");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("photos", photos_.size()); for (const auto &_value : photos_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatPhotos::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPhotos").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    photos_fieldID = jni::get_field_id(env, Class, "photos", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatPosition::Class;
jfieldID chatPosition::list_fieldID;
jfieldID chatPosition::order_fieldID;
jfieldID chatPosition::is_pinned_fieldID;
jfieldID chatPosition::source_fieldID;

chatPosition::chatPosition()
  : list_()
  , order_()
  , is_pinned_()
  , source_()
{}

chatPosition::chatPosition(object_ptr<ChatList> &&list_, int64 order_, bool is_pinned_, object_ptr<ChatSource> &&source_)
  : list_(std::move(list_))
  , order_(order_)
  , is_pinned_(is_pinned_)
  , source_(std::move(source_))
{}

const std::int32_t chatPosition::ID;

object_ptr<chatPosition> chatPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatPosition> res = make_object<chatPosition>();
  res->list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->list_fieldID));
  res->order_ = env->GetLongField(p, res->order_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->source_ = jni::fetch_tl_object<ChatSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  return res;
}

void chatPosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (list_ != nullptr) { jobject next; list_->store(env, next); if (next) { env->SetObjectField(s, list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, order_fieldID, order_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  if (source_ != nullptr) { jobject next; source_->store(env, next); if (next) { env->SetObjectField(s, source_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatPosition");
    s.store_object_field("list", static_cast<const BaseObject *>(list_.get()));
    s.store_field("order", order_);
    s.store_field("isPinned", is_pinned_);
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_class_end();
  }
}

void chatPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatPosition").c_str());
    list_fieldID = jni::get_field_id(env, Class, "list", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    order_fieldID = jni::get_field_id(env, Class, "order", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatSource;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueAmount::Class;
jfieldID chatRevenueAmount::cryptocurrency_fieldID;
jfieldID chatRevenueAmount::total_amount_fieldID;
jfieldID chatRevenueAmount::balance_amount_fieldID;
jfieldID chatRevenueAmount::available_amount_fieldID;
jfieldID chatRevenueAmount::withdrawal_enabled_fieldID;

chatRevenueAmount::chatRevenueAmount()
  : cryptocurrency_()
  , total_amount_()
  , balance_amount_()
  , available_amount_()
  , withdrawal_enabled_()
{}

chatRevenueAmount::chatRevenueAmount(string const &cryptocurrency_, int64 total_amount_, int64 balance_amount_, int64 available_amount_, bool withdrawal_enabled_)
  : cryptocurrency_(cryptocurrency_)
  , total_amount_(total_amount_)
  , balance_amount_(balance_amount_)
  , available_amount_(available_amount_)
  , withdrawal_enabled_(withdrawal_enabled_)
{}

const std::int32_t chatRevenueAmount::ID;

object_ptr<chatRevenueAmount> chatRevenueAmount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueAmount> res = make_object<chatRevenueAmount>();
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->balance_amount_ = env->GetLongField(p, res->balance_amount_fieldID);
  res->available_amount_ = env->GetLongField(p, res->available_amount_fieldID);
  res->withdrawal_enabled_ = (env->GetBooleanField(p, res->withdrawal_enabled_fieldID) != 0);
  return res;
}

void chatRevenueAmount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  env->SetLongField(s, balance_amount_fieldID, balance_amount_);
  env->SetLongField(s, available_amount_fieldID, available_amount_);
  env->SetBooleanField(s, withdrawal_enabled_fieldID, withdrawal_enabled_);
}

void chatRevenueAmount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueAmount");
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("totalAmount", total_amount_);
    s.store_field("balanceAmount", balance_amount_);
    s.store_field("availableAmount", available_amount_);
    s.store_field("withdrawalEnabled", withdrawal_enabled_);
    s.store_class_end();
  }
}

void chatRevenueAmount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueAmount").c_str());
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    balance_amount_fieldID = jni::get_field_id(env, Class, "balanceAmount", "J");
    available_amount_fieldID = jni::get_field_id(env, Class, "availableAmount", "J");
    withdrawal_enabled_fieldID = jni::get_field_id(env, Class, "withdrawalEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueStatistics::Class;
jfieldID chatRevenueStatistics::revenue_by_hour_graph_fieldID;
jfieldID chatRevenueStatistics::revenue_graph_fieldID;
jfieldID chatRevenueStatistics::revenue_amount_fieldID;
jfieldID chatRevenueStatistics::usd_rate_fieldID;

chatRevenueStatistics::chatRevenueStatistics()
  : revenue_by_hour_graph_()
  , revenue_graph_()
  , revenue_amount_()
  , usd_rate_()
{}

chatRevenueStatistics::chatRevenueStatistics(object_ptr<StatisticalGraph> &&revenue_by_hour_graph_, object_ptr<StatisticalGraph> &&revenue_graph_, object_ptr<chatRevenueAmount> &&revenue_amount_, double usd_rate_)
  : revenue_by_hour_graph_(std::move(revenue_by_hour_graph_))
  , revenue_graph_(std::move(revenue_graph_))
  , revenue_amount_(std::move(revenue_amount_))
  , usd_rate_(usd_rate_)
{}

const std::int32_t chatRevenueStatistics::ID;

object_ptr<chatRevenueStatistics> chatRevenueStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueStatistics> res = make_object<chatRevenueStatistics>();
  res->revenue_by_hour_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->revenue_by_hour_graph_fieldID));
  res->revenue_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->revenue_graph_fieldID));
  res->revenue_amount_ = jni::fetch_tl_object<chatRevenueAmount>(env, jni::fetch_object(env, p, res->revenue_amount_fieldID));
  res->usd_rate_ = env->GetDoubleField(p, res->usd_rate_fieldID);
  return res;
}

void chatRevenueStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (revenue_by_hour_graph_ != nullptr) { jobject next; revenue_by_hour_graph_->store(env, next); if (next) { env->SetObjectField(s, revenue_by_hour_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (revenue_graph_ != nullptr) { jobject next; revenue_graph_->store(env, next); if (next) { env->SetObjectField(s, revenue_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (revenue_amount_ != nullptr) { jobject next; revenue_amount_->store(env, next); if (next) { env->SetObjectField(s, revenue_amount_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, usd_rate_fieldID, usd_rate_);
}

void chatRevenueStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueStatistics");
    s.store_object_field("revenueByHourGraph", static_cast<const BaseObject *>(revenue_by_hour_graph_.get()));
    s.store_object_field("revenueGraph", static_cast<const BaseObject *>(revenue_graph_.get()));
    s.store_object_field("revenueAmount", static_cast<const BaseObject *>(revenue_amount_.get()));
    s.store_field("usdRate", usd_rate_);
    s.store_class_end();
  }
}

void chatRevenueStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueStatistics").c_str());
    revenue_by_hour_graph_fieldID = jni::get_field_id(env, Class, "revenueByHourGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    revenue_graph_fieldID = jni::get_field_id(env, Class, "revenueGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    revenue_amount_fieldID = jni::get_field_id(env, Class, "revenueAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatRevenueAmount;").c_str());
    usd_rate_fieldID = jni::get_field_id(env, Class, "usdRate", "D");
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueTransaction::Class;
jfieldID chatRevenueTransaction::cryptocurrency_fieldID;
jfieldID chatRevenueTransaction::cryptocurrency_amount_fieldID;
jfieldID chatRevenueTransaction::type_fieldID;

chatRevenueTransaction::chatRevenueTransaction()
  : cryptocurrency_()
  , cryptocurrency_amount_()
  , type_()
{}

chatRevenueTransaction::chatRevenueTransaction(string const &cryptocurrency_, int64 cryptocurrency_amount_, object_ptr<ChatRevenueTransactionType> &&type_)
  : cryptocurrency_(cryptocurrency_)
  , cryptocurrency_amount_(cryptocurrency_amount_)
  , type_(std::move(type_))
{}

const std::int32_t chatRevenueTransaction::ID;

object_ptr<chatRevenueTransaction> chatRevenueTransaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueTransaction> res = make_object<chatRevenueTransaction>();
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->cryptocurrency_amount_ = env->GetLongField(p, res->cryptocurrency_amount_fieldID);
  res->type_ = jni::fetch_tl_object<ChatRevenueTransactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void chatRevenueTransaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cryptocurrency_amount_fieldID, cryptocurrency_amount_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatRevenueTransaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueTransaction");
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("cryptocurrencyAmount", cryptocurrency_amount_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void chatRevenueTransaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransaction").c_str());
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    cryptocurrency_amount_fieldID = jni::get_field_id(env, Class, "cryptocurrencyAmount", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatRevenueTransactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatRevenueTransactionType::Class;

object_ptr<ChatRevenueTransactionType> ChatRevenueTransactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatRevenueTransactionTypeEarnings::ID:
      return chatRevenueTransactionTypeEarnings::fetch(env, p);
    case chatRevenueTransactionTypeWithdrawal::ID:
      return chatRevenueTransactionTypeWithdrawal::fetch(env, p);
    case chatRevenueTransactionTypeRefund::ID:
      return chatRevenueTransactionTypeRefund::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatRevenueTransactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransactionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueTransactionTypeEarnings::Class;
jfieldID chatRevenueTransactionTypeEarnings::start_date_fieldID;
jfieldID chatRevenueTransactionTypeEarnings::end_date_fieldID;

chatRevenueTransactionTypeEarnings::chatRevenueTransactionTypeEarnings()
  : start_date_()
  , end_date_()
{}

chatRevenueTransactionTypeEarnings::chatRevenueTransactionTypeEarnings(int32 start_date_, int32 end_date_)
  : start_date_(start_date_)
  , end_date_(end_date_)
{}

const std::int32_t chatRevenueTransactionTypeEarnings::ID;

object_ptr<ChatRevenueTransactionType> chatRevenueTransactionTypeEarnings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueTransactionTypeEarnings> res = make_object<chatRevenueTransactionTypeEarnings>();
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->end_date_ = env->GetIntField(p, res->end_date_fieldID);
  return std::move(res);
}

void chatRevenueTransactionTypeEarnings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, start_date_fieldID, start_date_);
  env->SetIntField(s, end_date_fieldID, end_date_);
}

void chatRevenueTransactionTypeEarnings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueTransactionTypeEarnings");
    s.store_field("startDate", start_date_);
    s.store_field("endDate", end_date_);
    s.store_class_end();
  }
}

void chatRevenueTransactionTypeEarnings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransactionTypeEarnings").c_str());
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    end_date_fieldID = jni::get_field_id(env, Class, "endDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueTransactionTypeWithdrawal::Class;
jfieldID chatRevenueTransactionTypeWithdrawal::withdrawal_date_fieldID;
jfieldID chatRevenueTransactionTypeWithdrawal::provider_fieldID;
jfieldID chatRevenueTransactionTypeWithdrawal::state_fieldID;

chatRevenueTransactionTypeWithdrawal::chatRevenueTransactionTypeWithdrawal()
  : withdrawal_date_()
  , provider_()
  , state_()
{}

chatRevenueTransactionTypeWithdrawal::chatRevenueTransactionTypeWithdrawal(int32 withdrawal_date_, string const &provider_, object_ptr<RevenueWithdrawalState> &&state_)
  : withdrawal_date_(withdrawal_date_)
  , provider_(provider_)
  , state_(std::move(state_))
{}

const std::int32_t chatRevenueTransactionTypeWithdrawal::ID;

object_ptr<ChatRevenueTransactionType> chatRevenueTransactionTypeWithdrawal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueTransactionTypeWithdrawal> res = make_object<chatRevenueTransactionTypeWithdrawal>();
  res->withdrawal_date_ = env->GetIntField(p, res->withdrawal_date_fieldID);
  res->provider_ = jni::fetch_string(env, p, res->provider_fieldID);
  res->state_ = jni::fetch_tl_object<RevenueWithdrawalState>(env, jni::fetch_object(env, p, res->state_fieldID));
  return std::move(res);
}

void chatRevenueTransactionTypeWithdrawal::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, withdrawal_date_fieldID, withdrawal_date_);
  { jstring nextString = jni::to_jstring(env, provider_); if (nextString) { env->SetObjectField(s, provider_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (state_ != nullptr) { jobject next; state_->store(env, next); if (next) { env->SetObjectField(s, state_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatRevenueTransactionTypeWithdrawal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueTransactionTypeWithdrawal");
    s.store_field("withdrawalDate", withdrawal_date_);
    s.store_field("provider", provider_);
    s.store_object_field("state", static_cast<const BaseObject *>(state_.get()));
    s.store_class_end();
  }
}

void chatRevenueTransactionTypeWithdrawal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransactionTypeWithdrawal").c_str());
    withdrawal_date_fieldID = jni::get_field_id(env, Class, "withdrawalDate", "I");
    provider_fieldID = jni::get_field_id(env, Class, "provider", "Ljava/lang/String;");
    state_fieldID = jni::get_field_id(env, Class, "state", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RevenueWithdrawalState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueTransactionTypeRefund::Class;
jfieldID chatRevenueTransactionTypeRefund::refund_date_fieldID;
jfieldID chatRevenueTransactionTypeRefund::provider_fieldID;

chatRevenueTransactionTypeRefund::chatRevenueTransactionTypeRefund()
  : refund_date_()
  , provider_()
{}

chatRevenueTransactionTypeRefund::chatRevenueTransactionTypeRefund(int32 refund_date_, string const &provider_)
  : refund_date_(refund_date_)
  , provider_(provider_)
{}

const std::int32_t chatRevenueTransactionTypeRefund::ID;

object_ptr<ChatRevenueTransactionType> chatRevenueTransactionTypeRefund::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueTransactionTypeRefund> res = make_object<chatRevenueTransactionTypeRefund>();
  res->refund_date_ = env->GetIntField(p, res->refund_date_fieldID);
  res->provider_ = jni::fetch_string(env, p, res->provider_fieldID);
  return std::move(res);
}

void chatRevenueTransactionTypeRefund::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, refund_date_fieldID, refund_date_);
  { jstring nextString = jni::to_jstring(env, provider_); if (nextString) { env->SetObjectField(s, provider_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatRevenueTransactionTypeRefund::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueTransactionTypeRefund");
    s.store_field("refundDate", refund_date_);
    s.store_field("provider", provider_);
    s.store_class_end();
  }
}

void chatRevenueTransactionTypeRefund::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransactionTypeRefund").c_str());
    refund_date_fieldID = jni::get_field_id(env, Class, "refundDate", "I");
    provider_fieldID = jni::get_field_id(env, Class, "provider", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass chatRevenueTransactions::Class;
jfieldID chatRevenueTransactions::total_count_fieldID;
jfieldID chatRevenueTransactions::transactions_fieldID;

chatRevenueTransactions::chatRevenueTransactions()
  : total_count_()
  , transactions_()
{}

chatRevenueTransactions::chatRevenueTransactions(int32 total_count_, array<object_ptr<chatRevenueTransaction>> &&transactions_)
  : total_count_(total_count_)
  , transactions_(std::move(transactions_))
{}

const std::int32_t chatRevenueTransactions::ID;

object_ptr<chatRevenueTransactions> chatRevenueTransactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatRevenueTransactions> res = make_object<chatRevenueTransactions>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->transactions_ = jni::FetchVector<chatRevenueTransaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->transactions_fieldID));
  return res;
}

void chatRevenueTransactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, transactions_); if (arr_tmp_) { env->SetObjectField(s, transactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatRevenueTransactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatRevenueTransactions");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("transactions", transactions_.size()); for (const auto &_value : transactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatRevenueTransactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatRevenueTransactions").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    transactions_fieldID = jni::get_field_id(env, Class, "transactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatRevenueTransaction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatSource::Class;

object_ptr<ChatSource> ChatSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatSourceMtprotoProxy::ID:
      return chatSourceMtprotoProxy::fetch(env, p);
    case chatSourcePublicServiceAnnouncement::ID:
      return chatSourcePublicServiceAnnouncement::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatSourceMtprotoProxy::Class;

chatSourceMtprotoProxy::chatSourceMtprotoProxy() {
}

const std::int32_t chatSourceMtprotoProxy::ID;

object_ptr<ChatSource> chatSourceMtprotoProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<chatSourceMtprotoProxy>();
}

void chatSourceMtprotoProxy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void chatSourceMtprotoProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatSourceMtprotoProxy");
    s.store_class_end();
  }
}

void chatSourceMtprotoProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatSourceMtprotoProxy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatSourcePublicServiceAnnouncement::Class;
jfieldID chatSourcePublicServiceAnnouncement::type_fieldID;
jfieldID chatSourcePublicServiceAnnouncement::text_fieldID;

chatSourcePublicServiceAnnouncement::chatSourcePublicServiceAnnouncement()
  : type_()
  , text_()
{}

chatSourcePublicServiceAnnouncement::chatSourcePublicServiceAnnouncement(string const &type_, string const &text_)
  : type_(type_)
  , text_(text_)
{}

const std::int32_t chatSourcePublicServiceAnnouncement::ID;

object_ptr<ChatSource> chatSourcePublicServiceAnnouncement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatSourcePublicServiceAnnouncement> res = make_object<chatSourcePublicServiceAnnouncement>();
  res->type_ = jni::fetch_string(env, p, res->type_fieldID);
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return std::move(res);
}

void chatSourcePublicServiceAnnouncement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, type_); if (nextString) { env->SetObjectField(s, type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void chatSourcePublicServiceAnnouncement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatSourcePublicServiceAnnouncement");
    s.store_field("type", type_);
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void chatSourcePublicServiceAnnouncement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatSourcePublicServiceAnnouncement").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", "Ljava/lang/String;");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass ChatStatistics::Class;

object_ptr<ChatStatistics> ChatStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatStatisticsSupergroup::ID:
      return chatStatisticsSupergroup::fetch(env, p);
    case chatStatisticsChannel::ID:
      return chatStatisticsChannel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatistics").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsSupergroup::Class;
jfieldID chatStatisticsSupergroup::period_fieldID;
jfieldID chatStatisticsSupergroup::member_count_fieldID;
jfieldID chatStatisticsSupergroup::message_count_fieldID;
jfieldID chatStatisticsSupergroup::viewer_count_fieldID;
jfieldID chatStatisticsSupergroup::sender_count_fieldID;
jfieldID chatStatisticsSupergroup::member_count_graph_fieldID;
jfieldID chatStatisticsSupergroup::join_graph_fieldID;
jfieldID chatStatisticsSupergroup::join_by_source_graph_fieldID;
jfieldID chatStatisticsSupergroup::language_graph_fieldID;
jfieldID chatStatisticsSupergroup::message_content_graph_fieldID;
jfieldID chatStatisticsSupergroup::action_graph_fieldID;
jfieldID chatStatisticsSupergroup::day_graph_fieldID;
jfieldID chatStatisticsSupergroup::week_graph_fieldID;
jfieldID chatStatisticsSupergroup::top_senders_fieldID;
jfieldID chatStatisticsSupergroup::top_administrators_fieldID;
jfieldID chatStatisticsSupergroup::top_inviters_fieldID;

chatStatisticsSupergroup::chatStatisticsSupergroup()
  : period_()
  , member_count_()
  , message_count_()
  , viewer_count_()
  , sender_count_()
  , member_count_graph_()
  , join_graph_()
  , join_by_source_graph_()
  , language_graph_()
  , message_content_graph_()
  , action_graph_()
  , day_graph_()
  , week_graph_()
  , top_senders_()
  , top_administrators_()
  , top_inviters_()
{}

chatStatisticsSupergroup::chatStatisticsSupergroup(object_ptr<dateRange> &&period_, object_ptr<statisticalValue> &&member_count_, object_ptr<statisticalValue> &&message_count_, object_ptr<statisticalValue> &&viewer_count_, object_ptr<statisticalValue> &&sender_count_, object_ptr<StatisticalGraph> &&member_count_graph_, object_ptr<StatisticalGraph> &&join_graph_, object_ptr<StatisticalGraph> &&join_by_source_graph_, object_ptr<StatisticalGraph> &&language_graph_, object_ptr<StatisticalGraph> &&message_content_graph_, object_ptr<StatisticalGraph> &&action_graph_, object_ptr<StatisticalGraph> &&day_graph_, object_ptr<StatisticalGraph> &&week_graph_, array<object_ptr<chatStatisticsMessageSenderInfo>> &&top_senders_, array<object_ptr<chatStatisticsAdministratorActionsInfo>> &&top_administrators_, array<object_ptr<chatStatisticsInviterInfo>> &&top_inviters_)
  : period_(std::move(period_))
  , member_count_(std::move(member_count_))
  , message_count_(std::move(message_count_))
  , viewer_count_(std::move(viewer_count_))
  , sender_count_(std::move(sender_count_))
  , member_count_graph_(std::move(member_count_graph_))
  , join_graph_(std::move(join_graph_))
  , join_by_source_graph_(std::move(join_by_source_graph_))
  , language_graph_(std::move(language_graph_))
  , message_content_graph_(std::move(message_content_graph_))
  , action_graph_(std::move(action_graph_))
  , day_graph_(std::move(day_graph_))
  , week_graph_(std::move(week_graph_))
  , top_senders_(std::move(top_senders_))
  , top_administrators_(std::move(top_administrators_))
  , top_inviters_(std::move(top_inviters_))
{}

const std::int32_t chatStatisticsSupergroup::ID;

object_ptr<ChatStatistics> chatStatisticsSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsSupergroup> res = make_object<chatStatisticsSupergroup>();
  res->period_ = jni::fetch_tl_object<dateRange>(env, jni::fetch_object(env, p, res->period_fieldID));
  res->member_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->member_count_fieldID));
  res->message_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->message_count_fieldID));
  res->viewer_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->viewer_count_fieldID));
  res->sender_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->sender_count_fieldID));
  res->member_count_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->member_count_graph_fieldID));
  res->join_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->join_graph_fieldID));
  res->join_by_source_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->join_by_source_graph_fieldID));
  res->language_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->language_graph_fieldID));
  res->message_content_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->message_content_graph_fieldID));
  res->action_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->action_graph_fieldID));
  res->day_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->day_graph_fieldID));
  res->week_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->week_graph_fieldID));
  res->top_senders_ = jni::FetchVector<chatStatisticsMessageSenderInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->top_senders_fieldID));
  res->top_administrators_ = jni::FetchVector<chatStatisticsAdministratorActionsInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->top_administrators_fieldID));
  res->top_inviters_ = jni::FetchVector<chatStatisticsInviterInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->top_inviters_fieldID));
  return std::move(res);
}

void chatStatisticsSupergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (period_ != nullptr) { jobject next; period_->store(env, next); if (next) { env->SetObjectField(s, period_fieldID, next); env->DeleteLocalRef(next); } }
  if (member_count_ != nullptr) { jobject next; member_count_->store(env, next); if (next) { env->SetObjectField(s, member_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (message_count_ != nullptr) { jobject next; message_count_->store(env, next); if (next) { env->SetObjectField(s, message_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (viewer_count_ != nullptr) { jobject next; viewer_count_->store(env, next); if (next) { env->SetObjectField(s, viewer_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_count_ != nullptr) { jobject next; sender_count_->store(env, next); if (next) { env->SetObjectField(s, sender_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (member_count_graph_ != nullptr) { jobject next; member_count_graph_->store(env, next); if (next) { env->SetObjectField(s, member_count_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (join_graph_ != nullptr) { jobject next; join_graph_->store(env, next); if (next) { env->SetObjectField(s, join_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (join_by_source_graph_ != nullptr) { jobject next; join_by_source_graph_->store(env, next); if (next) { env->SetObjectField(s, join_by_source_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (language_graph_ != nullptr) { jobject next; language_graph_->store(env, next); if (next) { env->SetObjectField(s, language_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (message_content_graph_ != nullptr) { jobject next; message_content_graph_->store(env, next); if (next) { env->SetObjectField(s, message_content_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (action_graph_ != nullptr) { jobject next; action_graph_->store(env, next); if (next) { env->SetObjectField(s, action_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (day_graph_ != nullptr) { jobject next; day_graph_->store(env, next); if (next) { env->SetObjectField(s, day_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (week_graph_ != nullptr) { jobject next; week_graph_->store(env, next); if (next) { env->SetObjectField(s, week_graph_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, top_senders_); if (arr_tmp_) { env->SetObjectField(s, top_senders_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, top_administrators_); if (arr_tmp_) { env->SetObjectField(s, top_administrators_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, top_inviters_); if (arr_tmp_) { env->SetObjectField(s, top_inviters_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatStatisticsSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsSupergroup");
    s.store_object_field("period", static_cast<const BaseObject *>(period_.get()));
    s.store_object_field("memberCount", static_cast<const BaseObject *>(member_count_.get()));
    s.store_object_field("messageCount", static_cast<const BaseObject *>(message_count_.get()));
    s.store_object_field("viewerCount", static_cast<const BaseObject *>(viewer_count_.get()));
    s.store_object_field("senderCount", static_cast<const BaseObject *>(sender_count_.get()));
    s.store_object_field("memberCountGraph", static_cast<const BaseObject *>(member_count_graph_.get()));
    s.store_object_field("joinGraph", static_cast<const BaseObject *>(join_graph_.get()));
    s.store_object_field("joinBySourceGraph", static_cast<const BaseObject *>(join_by_source_graph_.get()));
    s.store_object_field("languageGraph", static_cast<const BaseObject *>(language_graph_.get()));
    s.store_object_field("messageContentGraph", static_cast<const BaseObject *>(message_content_graph_.get()));
    s.store_object_field("actionGraph", static_cast<const BaseObject *>(action_graph_.get()));
    s.store_object_field("dayGraph", static_cast<const BaseObject *>(day_graph_.get()));
    s.store_object_field("weekGraph", static_cast<const BaseObject *>(week_graph_.get()));
    { s.store_vector_begin("topSenders", top_senders_.size()); for (const auto &_value : top_senders_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("topAdministrators", top_administrators_.size()); for (const auto &_value : top_administrators_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("topInviters", top_inviters_.size()); for (const auto &_value : top_inviters_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatStatisticsSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsSupergroup").c_str());
    period_fieldID = jni::get_field_id(env, Class, "period", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DateRange;").c_str());
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    message_count_fieldID = jni::get_field_id(env, Class, "messageCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    viewer_count_fieldID = jni::get_field_id(env, Class, "viewerCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    sender_count_fieldID = jni::get_field_id(env, Class, "senderCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    member_count_graph_fieldID = jni::get_field_id(env, Class, "memberCountGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    join_graph_fieldID = jni::get_field_id(env, Class, "joinGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    join_by_source_graph_fieldID = jni::get_field_id(env, Class, "joinBySourceGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    language_graph_fieldID = jni::get_field_id(env, Class, "languageGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    message_content_graph_fieldID = jni::get_field_id(env, Class, "messageContentGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    action_graph_fieldID = jni::get_field_id(env, Class, "actionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    day_graph_fieldID = jni::get_field_id(env, Class, "dayGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    week_graph_fieldID = jni::get_field_id(env, Class, "weekGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    top_senders_fieldID = jni::get_field_id(env, Class, "topSenders", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatStatisticsMessageSenderInfo;").c_str());
    top_administrators_fieldID = jni::get_field_id(env, Class, "topAdministrators", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatStatisticsAdministratorActionsInfo;").c_str());
    top_inviters_fieldID = jni::get_field_id(env, Class, "topInviters", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatStatisticsInviterInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsChannel::Class;
jfieldID chatStatisticsChannel::period_fieldID;
jfieldID chatStatisticsChannel::member_count_fieldID;
jfieldID chatStatisticsChannel::mean_message_view_count_fieldID;
jfieldID chatStatisticsChannel::mean_message_share_count_fieldID;
jfieldID chatStatisticsChannel::mean_message_reaction_count_fieldID;
jfieldID chatStatisticsChannel::mean_story_view_count_fieldID;
jfieldID chatStatisticsChannel::mean_story_share_count_fieldID;
jfieldID chatStatisticsChannel::mean_story_reaction_count_fieldID;
jfieldID chatStatisticsChannel::enabled_notifications_percentage_fieldID;
jfieldID chatStatisticsChannel::member_count_graph_fieldID;
jfieldID chatStatisticsChannel::join_graph_fieldID;
jfieldID chatStatisticsChannel::mute_graph_fieldID;
jfieldID chatStatisticsChannel::view_count_by_hour_graph_fieldID;
jfieldID chatStatisticsChannel::view_count_by_source_graph_fieldID;
jfieldID chatStatisticsChannel::join_by_source_graph_fieldID;
jfieldID chatStatisticsChannel::language_graph_fieldID;
jfieldID chatStatisticsChannel::message_interaction_graph_fieldID;
jfieldID chatStatisticsChannel::message_reaction_graph_fieldID;
jfieldID chatStatisticsChannel::story_interaction_graph_fieldID;
jfieldID chatStatisticsChannel::story_reaction_graph_fieldID;
jfieldID chatStatisticsChannel::instant_view_interaction_graph_fieldID;
jfieldID chatStatisticsChannel::recent_interactions_fieldID;

chatStatisticsChannel::chatStatisticsChannel()
  : period_()
  , member_count_()
  , mean_message_view_count_()
  , mean_message_share_count_()
  , mean_message_reaction_count_()
  , mean_story_view_count_()
  , mean_story_share_count_()
  , mean_story_reaction_count_()
  , enabled_notifications_percentage_()
  , member_count_graph_()
  , join_graph_()
  , mute_graph_()
  , view_count_by_hour_graph_()
  , view_count_by_source_graph_()
  , join_by_source_graph_()
  , language_graph_()
  , message_interaction_graph_()
  , message_reaction_graph_()
  , story_interaction_graph_()
  , story_reaction_graph_()
  , instant_view_interaction_graph_()
  , recent_interactions_()
{}

chatStatisticsChannel::chatStatisticsChannel(object_ptr<dateRange> &&period_, object_ptr<statisticalValue> &&member_count_, object_ptr<statisticalValue> &&mean_message_view_count_, object_ptr<statisticalValue> &&mean_message_share_count_, object_ptr<statisticalValue> &&mean_message_reaction_count_, object_ptr<statisticalValue> &&mean_story_view_count_, object_ptr<statisticalValue> &&mean_story_share_count_, object_ptr<statisticalValue> &&mean_story_reaction_count_, double enabled_notifications_percentage_, object_ptr<StatisticalGraph> &&member_count_graph_, object_ptr<StatisticalGraph> &&join_graph_, object_ptr<StatisticalGraph> &&mute_graph_, object_ptr<StatisticalGraph> &&view_count_by_hour_graph_, object_ptr<StatisticalGraph> &&view_count_by_source_graph_, object_ptr<StatisticalGraph> &&join_by_source_graph_, object_ptr<StatisticalGraph> &&language_graph_, object_ptr<StatisticalGraph> &&message_interaction_graph_, object_ptr<StatisticalGraph> &&message_reaction_graph_, object_ptr<StatisticalGraph> &&story_interaction_graph_, object_ptr<StatisticalGraph> &&story_reaction_graph_, object_ptr<StatisticalGraph> &&instant_view_interaction_graph_, array<object_ptr<chatStatisticsInteractionInfo>> &&recent_interactions_)
  : period_(std::move(period_))
  , member_count_(std::move(member_count_))
  , mean_message_view_count_(std::move(mean_message_view_count_))
  , mean_message_share_count_(std::move(mean_message_share_count_))
  , mean_message_reaction_count_(std::move(mean_message_reaction_count_))
  , mean_story_view_count_(std::move(mean_story_view_count_))
  , mean_story_share_count_(std::move(mean_story_share_count_))
  , mean_story_reaction_count_(std::move(mean_story_reaction_count_))
  , enabled_notifications_percentage_(enabled_notifications_percentage_)
  , member_count_graph_(std::move(member_count_graph_))
  , join_graph_(std::move(join_graph_))
  , mute_graph_(std::move(mute_graph_))
  , view_count_by_hour_graph_(std::move(view_count_by_hour_graph_))
  , view_count_by_source_graph_(std::move(view_count_by_source_graph_))
  , join_by_source_graph_(std::move(join_by_source_graph_))
  , language_graph_(std::move(language_graph_))
  , message_interaction_graph_(std::move(message_interaction_graph_))
  , message_reaction_graph_(std::move(message_reaction_graph_))
  , story_interaction_graph_(std::move(story_interaction_graph_))
  , story_reaction_graph_(std::move(story_reaction_graph_))
  , instant_view_interaction_graph_(std::move(instant_view_interaction_graph_))
  , recent_interactions_(std::move(recent_interactions_))
{}

const std::int32_t chatStatisticsChannel::ID;

object_ptr<ChatStatistics> chatStatisticsChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsChannel> res = make_object<chatStatisticsChannel>();
  res->period_ = jni::fetch_tl_object<dateRange>(env, jni::fetch_object(env, p, res->period_fieldID));
  res->member_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->member_count_fieldID));
  res->mean_message_view_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_message_view_count_fieldID));
  res->mean_message_share_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_message_share_count_fieldID));
  res->mean_message_reaction_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_message_reaction_count_fieldID));
  res->mean_story_view_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_story_view_count_fieldID));
  res->mean_story_share_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_story_share_count_fieldID));
  res->mean_story_reaction_count_ = jni::fetch_tl_object<statisticalValue>(env, jni::fetch_object(env, p, res->mean_story_reaction_count_fieldID));
  res->enabled_notifications_percentage_ = env->GetDoubleField(p, res->enabled_notifications_percentage_fieldID);
  res->member_count_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->member_count_graph_fieldID));
  res->join_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->join_graph_fieldID));
  res->mute_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->mute_graph_fieldID));
  res->view_count_by_hour_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->view_count_by_hour_graph_fieldID));
  res->view_count_by_source_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->view_count_by_source_graph_fieldID));
  res->join_by_source_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->join_by_source_graph_fieldID));
  res->language_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->language_graph_fieldID));
  res->message_interaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->message_interaction_graph_fieldID));
  res->message_reaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->message_reaction_graph_fieldID));
  res->story_interaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->story_interaction_graph_fieldID));
  res->story_reaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->story_reaction_graph_fieldID));
  res->instant_view_interaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->instant_view_interaction_graph_fieldID));
  res->recent_interactions_ = jni::FetchVector<chatStatisticsInteractionInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_interactions_fieldID));
  return std::move(res);
}

void chatStatisticsChannel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (period_ != nullptr) { jobject next; period_->store(env, next); if (next) { env->SetObjectField(s, period_fieldID, next); env->DeleteLocalRef(next); } }
  if (member_count_ != nullptr) { jobject next; member_count_->store(env, next); if (next) { env->SetObjectField(s, member_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_message_view_count_ != nullptr) { jobject next; mean_message_view_count_->store(env, next); if (next) { env->SetObjectField(s, mean_message_view_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_message_share_count_ != nullptr) { jobject next; mean_message_share_count_->store(env, next); if (next) { env->SetObjectField(s, mean_message_share_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_message_reaction_count_ != nullptr) { jobject next; mean_message_reaction_count_->store(env, next); if (next) { env->SetObjectField(s, mean_message_reaction_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_story_view_count_ != nullptr) { jobject next; mean_story_view_count_->store(env, next); if (next) { env->SetObjectField(s, mean_story_view_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_story_share_count_ != nullptr) { jobject next; mean_story_share_count_->store(env, next); if (next) { env->SetObjectField(s, mean_story_share_count_fieldID, next); env->DeleteLocalRef(next); } }
  if (mean_story_reaction_count_ != nullptr) { jobject next; mean_story_reaction_count_->store(env, next); if (next) { env->SetObjectField(s, mean_story_reaction_count_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, enabled_notifications_percentage_fieldID, enabled_notifications_percentage_);
  if (member_count_graph_ != nullptr) { jobject next; member_count_graph_->store(env, next); if (next) { env->SetObjectField(s, member_count_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (join_graph_ != nullptr) { jobject next; join_graph_->store(env, next); if (next) { env->SetObjectField(s, join_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (mute_graph_ != nullptr) { jobject next; mute_graph_->store(env, next); if (next) { env->SetObjectField(s, mute_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (view_count_by_hour_graph_ != nullptr) { jobject next; view_count_by_hour_graph_->store(env, next); if (next) { env->SetObjectField(s, view_count_by_hour_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (view_count_by_source_graph_ != nullptr) { jobject next; view_count_by_source_graph_->store(env, next); if (next) { env->SetObjectField(s, view_count_by_source_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (join_by_source_graph_ != nullptr) { jobject next; join_by_source_graph_->store(env, next); if (next) { env->SetObjectField(s, join_by_source_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (language_graph_ != nullptr) { jobject next; language_graph_->store(env, next); if (next) { env->SetObjectField(s, language_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (message_interaction_graph_ != nullptr) { jobject next; message_interaction_graph_->store(env, next); if (next) { env->SetObjectField(s, message_interaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (message_reaction_graph_ != nullptr) { jobject next; message_reaction_graph_->store(env, next); if (next) { env->SetObjectField(s, message_reaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (story_interaction_graph_ != nullptr) { jobject next; story_interaction_graph_->store(env, next); if (next) { env->SetObjectField(s, story_interaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (story_reaction_graph_ != nullptr) { jobject next; story_reaction_graph_->store(env, next); if (next) { env->SetObjectField(s, story_reaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (instant_view_interaction_graph_ != nullptr) { jobject next; instant_view_interaction_graph_->store(env, next); if (next) { env->SetObjectField(s, instant_view_interaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, recent_interactions_); if (arr_tmp_) { env->SetObjectField(s, recent_interactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chatStatisticsChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsChannel");
    s.store_object_field("period", static_cast<const BaseObject *>(period_.get()));
    s.store_object_field("memberCount", static_cast<const BaseObject *>(member_count_.get()));
    s.store_object_field("meanMessageViewCount", static_cast<const BaseObject *>(mean_message_view_count_.get()));
    s.store_object_field("meanMessageShareCount", static_cast<const BaseObject *>(mean_message_share_count_.get()));
    s.store_object_field("meanMessageReactionCount", static_cast<const BaseObject *>(mean_message_reaction_count_.get()));
    s.store_object_field("meanStoryViewCount", static_cast<const BaseObject *>(mean_story_view_count_.get()));
    s.store_object_field("meanStoryShareCount", static_cast<const BaseObject *>(mean_story_share_count_.get()));
    s.store_object_field("meanStoryReactionCount", static_cast<const BaseObject *>(mean_story_reaction_count_.get()));
    s.store_field("enabledNotificationsPercentage", enabled_notifications_percentage_);
    s.store_object_field("memberCountGraph", static_cast<const BaseObject *>(member_count_graph_.get()));
    s.store_object_field("joinGraph", static_cast<const BaseObject *>(join_graph_.get()));
    s.store_object_field("muteGraph", static_cast<const BaseObject *>(mute_graph_.get()));
    s.store_object_field("viewCountByHourGraph", static_cast<const BaseObject *>(view_count_by_hour_graph_.get()));
    s.store_object_field("viewCountBySourceGraph", static_cast<const BaseObject *>(view_count_by_source_graph_.get()));
    s.store_object_field("joinBySourceGraph", static_cast<const BaseObject *>(join_by_source_graph_.get()));
    s.store_object_field("languageGraph", static_cast<const BaseObject *>(language_graph_.get()));
    s.store_object_field("messageInteractionGraph", static_cast<const BaseObject *>(message_interaction_graph_.get()));
    s.store_object_field("messageReactionGraph", static_cast<const BaseObject *>(message_reaction_graph_.get()));
    s.store_object_field("storyInteractionGraph", static_cast<const BaseObject *>(story_interaction_graph_.get()));
    s.store_object_field("storyReactionGraph", static_cast<const BaseObject *>(story_reaction_graph_.get()));
    s.store_object_field("instantViewInteractionGraph", static_cast<const BaseObject *>(instant_view_interaction_graph_.get()));
    { s.store_vector_begin("recentInteractions", recent_interactions_.size()); for (const auto &_value : recent_interactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chatStatisticsChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsChannel").c_str());
    period_fieldID = jni::get_field_id(env, Class, "period", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DateRange;").c_str());
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_message_view_count_fieldID = jni::get_field_id(env, Class, "meanMessageViewCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_message_share_count_fieldID = jni::get_field_id(env, Class, "meanMessageShareCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_message_reaction_count_fieldID = jni::get_field_id(env, Class, "meanMessageReactionCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_story_view_count_fieldID = jni::get_field_id(env, Class, "meanStoryViewCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_story_share_count_fieldID = jni::get_field_id(env, Class, "meanStoryShareCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    mean_story_reaction_count_fieldID = jni::get_field_id(env, Class, "meanStoryReactionCount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalValue;").c_str());
    enabled_notifications_percentage_fieldID = jni::get_field_id(env, Class, "enabledNotificationsPercentage", "D");
    member_count_graph_fieldID = jni::get_field_id(env, Class, "memberCountGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    join_graph_fieldID = jni::get_field_id(env, Class, "joinGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    mute_graph_fieldID = jni::get_field_id(env, Class, "muteGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    view_count_by_hour_graph_fieldID = jni::get_field_id(env, Class, "viewCountByHourGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    view_count_by_source_graph_fieldID = jni::get_field_id(env, Class, "viewCountBySourceGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    join_by_source_graph_fieldID = jni::get_field_id(env, Class, "joinBySourceGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    language_graph_fieldID = jni::get_field_id(env, Class, "languageGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    message_interaction_graph_fieldID = jni::get_field_id(env, Class, "messageInteractionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    message_reaction_graph_fieldID = jni::get_field_id(env, Class, "messageReactionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    story_interaction_graph_fieldID = jni::get_field_id(env, Class, "storyInteractionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    story_reaction_graph_fieldID = jni::get_field_id(env, Class, "storyReactionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    instant_view_interaction_graph_fieldID = jni::get_field_id(env, Class, "instantViewInteractionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    recent_interactions_fieldID = jni::get_field_id(env, Class, "recentInteractions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatStatisticsInteractionInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsAdministratorActionsInfo::Class;
jfieldID chatStatisticsAdministratorActionsInfo::user_id_fieldID;
jfieldID chatStatisticsAdministratorActionsInfo::deleted_message_count_fieldID;
jfieldID chatStatisticsAdministratorActionsInfo::banned_user_count_fieldID;
jfieldID chatStatisticsAdministratorActionsInfo::restricted_user_count_fieldID;

chatStatisticsAdministratorActionsInfo::chatStatisticsAdministratorActionsInfo()
  : user_id_()
  , deleted_message_count_()
  , banned_user_count_()
  , restricted_user_count_()
{}

chatStatisticsAdministratorActionsInfo::chatStatisticsAdministratorActionsInfo(int53 user_id_, int32 deleted_message_count_, int32 banned_user_count_, int32 restricted_user_count_)
  : user_id_(user_id_)
  , deleted_message_count_(deleted_message_count_)
  , banned_user_count_(banned_user_count_)
  , restricted_user_count_(restricted_user_count_)
{}

const std::int32_t chatStatisticsAdministratorActionsInfo::ID;

object_ptr<chatStatisticsAdministratorActionsInfo> chatStatisticsAdministratorActionsInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsAdministratorActionsInfo> res = make_object<chatStatisticsAdministratorActionsInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->deleted_message_count_ = env->GetIntField(p, res->deleted_message_count_fieldID);
  res->banned_user_count_ = env->GetIntField(p, res->banned_user_count_fieldID);
  res->restricted_user_count_ = env->GetIntField(p, res->restricted_user_count_fieldID);
  return res;
}

void chatStatisticsAdministratorActionsInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, deleted_message_count_fieldID, deleted_message_count_);
  env->SetIntField(s, banned_user_count_fieldID, banned_user_count_);
  env->SetIntField(s, restricted_user_count_fieldID, restricted_user_count_);
}

void chatStatisticsAdministratorActionsInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsAdministratorActionsInfo");
    s.store_field("userId", user_id_);
    s.store_field("deletedMessageCount", deleted_message_count_);
    s.store_field("bannedUserCount", banned_user_count_);
    s.store_field("restrictedUserCount", restricted_user_count_);
    s.store_class_end();
  }
}

void chatStatisticsAdministratorActionsInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsAdministratorActionsInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    deleted_message_count_fieldID = jni::get_field_id(env, Class, "deletedMessageCount", "I");
    banned_user_count_fieldID = jni::get_field_id(env, Class, "bannedUserCount", "I");
    restricted_user_count_fieldID = jni::get_field_id(env, Class, "restrictedUserCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsInteractionInfo::Class;
jfieldID chatStatisticsInteractionInfo::object_type_fieldID;
jfieldID chatStatisticsInteractionInfo::view_count_fieldID;
jfieldID chatStatisticsInteractionInfo::forward_count_fieldID;
jfieldID chatStatisticsInteractionInfo::reaction_count_fieldID;

chatStatisticsInteractionInfo::chatStatisticsInteractionInfo()
  : object_type_()
  , view_count_()
  , forward_count_()
  , reaction_count_()
{}

chatStatisticsInteractionInfo::chatStatisticsInteractionInfo(object_ptr<ChatStatisticsObjectType> &&object_type_, int32 view_count_, int32 forward_count_, int32 reaction_count_)
  : object_type_(std::move(object_type_))
  , view_count_(view_count_)
  , forward_count_(forward_count_)
  , reaction_count_(reaction_count_)
{}

const std::int32_t chatStatisticsInteractionInfo::ID;

object_ptr<chatStatisticsInteractionInfo> chatStatisticsInteractionInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsInteractionInfo> res = make_object<chatStatisticsInteractionInfo>();
  res->object_type_ = jni::fetch_tl_object<ChatStatisticsObjectType>(env, jni::fetch_object(env, p, res->object_type_fieldID));
  res->view_count_ = env->GetIntField(p, res->view_count_fieldID);
  res->forward_count_ = env->GetIntField(p, res->forward_count_fieldID);
  res->reaction_count_ = env->GetIntField(p, res->reaction_count_fieldID);
  return res;
}

void chatStatisticsInteractionInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (object_type_ != nullptr) { jobject next; object_type_->store(env, next); if (next) { env->SetObjectField(s, object_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, view_count_fieldID, view_count_);
  env->SetIntField(s, forward_count_fieldID, forward_count_);
  env->SetIntField(s, reaction_count_fieldID, reaction_count_);
}

void chatStatisticsInteractionInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsInteractionInfo");
    s.store_object_field("objectType", static_cast<const BaseObject *>(object_type_.get()));
    s.store_field("viewCount", view_count_);
    s.store_field("forwardCount", forward_count_);
    s.store_field("reactionCount", reaction_count_);
    s.store_class_end();
  }
}

void chatStatisticsInteractionInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsInteractionInfo").c_str());
    object_type_fieldID = jni::get_field_id(env, Class, "objectType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatStatisticsObjectType;").c_str());
    view_count_fieldID = jni::get_field_id(env, Class, "viewCount", "I");
    forward_count_fieldID = jni::get_field_id(env, Class, "forwardCount", "I");
    reaction_count_fieldID = jni::get_field_id(env, Class, "reactionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsInviterInfo::Class;
jfieldID chatStatisticsInviterInfo::user_id_fieldID;
jfieldID chatStatisticsInviterInfo::added_member_count_fieldID;

chatStatisticsInviterInfo::chatStatisticsInviterInfo()
  : user_id_()
  , added_member_count_()
{}

chatStatisticsInviterInfo::chatStatisticsInviterInfo(int53 user_id_, int32 added_member_count_)
  : user_id_(user_id_)
  , added_member_count_(added_member_count_)
{}

const std::int32_t chatStatisticsInviterInfo::ID;

object_ptr<chatStatisticsInviterInfo> chatStatisticsInviterInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsInviterInfo> res = make_object<chatStatisticsInviterInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->added_member_count_ = env->GetIntField(p, res->added_member_count_fieldID);
  return res;
}

void chatStatisticsInviterInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, added_member_count_fieldID, added_member_count_);
}

void chatStatisticsInviterInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsInviterInfo");
    s.store_field("userId", user_id_);
    s.store_field("addedMemberCount", added_member_count_);
    s.store_class_end();
  }
}

void chatStatisticsInviterInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsInviterInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    added_member_count_fieldID = jni::get_field_id(env, Class, "addedMemberCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsMessageSenderInfo::Class;
jfieldID chatStatisticsMessageSenderInfo::user_id_fieldID;
jfieldID chatStatisticsMessageSenderInfo::sent_message_count_fieldID;
jfieldID chatStatisticsMessageSenderInfo::average_character_count_fieldID;

chatStatisticsMessageSenderInfo::chatStatisticsMessageSenderInfo()
  : user_id_()
  , sent_message_count_()
  , average_character_count_()
{}

chatStatisticsMessageSenderInfo::chatStatisticsMessageSenderInfo(int53 user_id_, int32 sent_message_count_, int32 average_character_count_)
  : user_id_(user_id_)
  , sent_message_count_(sent_message_count_)
  , average_character_count_(average_character_count_)
{}

const std::int32_t chatStatisticsMessageSenderInfo::ID;

object_ptr<chatStatisticsMessageSenderInfo> chatStatisticsMessageSenderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsMessageSenderInfo> res = make_object<chatStatisticsMessageSenderInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->sent_message_count_ = env->GetIntField(p, res->sent_message_count_fieldID);
  res->average_character_count_ = env->GetIntField(p, res->average_character_count_fieldID);
  return res;
}

void chatStatisticsMessageSenderInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, sent_message_count_fieldID, sent_message_count_);
  env->SetIntField(s, average_character_count_fieldID, average_character_count_);
}

void chatStatisticsMessageSenderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsMessageSenderInfo");
    s.store_field("userId", user_id_);
    s.store_field("sentMessageCount", sent_message_count_);
    s.store_field("averageCharacterCount", average_character_count_);
    s.store_class_end();
  }
}

void chatStatisticsMessageSenderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsMessageSenderInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    sent_message_count_fieldID = jni::get_field_id(env, Class, "sentMessageCount", "I");
    average_character_count_fieldID = jni::get_field_id(env, Class, "averageCharacterCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass ChatStatisticsObjectType::Class;

object_ptr<ChatStatisticsObjectType> ChatStatisticsObjectType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatStatisticsObjectTypeMessage::ID:
      return chatStatisticsObjectTypeMessage::fetch(env, p);
    case chatStatisticsObjectTypeStory::ID:
      return chatStatisticsObjectTypeStory::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatStatisticsObjectType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsObjectType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsObjectTypeMessage::Class;
jfieldID chatStatisticsObjectTypeMessage::message_id_fieldID;

chatStatisticsObjectTypeMessage::chatStatisticsObjectTypeMessage()
  : message_id_()
{}

chatStatisticsObjectTypeMessage::chatStatisticsObjectTypeMessage(int53 message_id_)
  : message_id_(message_id_)
{}

const std::int32_t chatStatisticsObjectTypeMessage::ID;

object_ptr<ChatStatisticsObjectType> chatStatisticsObjectTypeMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsObjectTypeMessage> res = make_object<chatStatisticsObjectTypeMessage>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void chatStatisticsObjectTypeMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void chatStatisticsObjectTypeMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsObjectTypeMessage");
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void chatStatisticsObjectTypeMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsObjectTypeMessage").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatStatisticsObjectTypeStory::Class;
jfieldID chatStatisticsObjectTypeStory::story_id_fieldID;

chatStatisticsObjectTypeStory::chatStatisticsObjectTypeStory()
  : story_id_()
{}

chatStatisticsObjectTypeStory::chatStatisticsObjectTypeStory(int32 story_id_)
  : story_id_(story_id_)
{}

const std::int32_t chatStatisticsObjectTypeStory::ID;

object_ptr<ChatStatisticsObjectType> chatStatisticsObjectTypeStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatStatisticsObjectTypeStory> res = make_object<chatStatisticsObjectTypeStory>();
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void chatStatisticsObjectTypeStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void chatStatisticsObjectTypeStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatStatisticsObjectTypeStory");
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void chatStatisticsObjectTypeStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatStatisticsObjectTypeStory").c_str());
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass chatTheme::Class;
jfieldID chatTheme::name_fieldID;
jfieldID chatTheme::light_settings_fieldID;
jfieldID chatTheme::dark_settings_fieldID;

chatTheme::chatTheme()
  : name_()
  , light_settings_()
  , dark_settings_()
{}

chatTheme::chatTheme(string const &name_, object_ptr<themeSettings> &&light_settings_, object_ptr<themeSettings> &&dark_settings_)
  : name_(name_)
  , light_settings_(std::move(light_settings_))
  , dark_settings_(std::move(dark_settings_))
{}

const std::int32_t chatTheme::ID;

object_ptr<chatTheme> chatTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatTheme> res = make_object<chatTheme>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->light_settings_ = jni::fetch_tl_object<themeSettings>(env, jni::fetch_object(env, p, res->light_settings_fieldID));
  res->dark_settings_ = jni::fetch_tl_object<themeSettings>(env, jni::fetch_object(env, p, res->dark_settings_fieldID));
  return res;
}

void chatTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (light_settings_ != nullptr) { jobject next; light_settings_->store(env, next); if (next) { env->SetObjectField(s, light_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (dark_settings_ != nullptr) { jobject next; dark_settings_->store(env, next); if (next) { env->SetObjectField(s, dark_settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void chatTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatTheme");
    s.store_field("name", name_);
    s.store_object_field("lightSettings", static_cast<const BaseObject *>(light_settings_.get()));
    s.store_object_field("darkSettings", static_cast<const BaseObject *>(dark_settings_.get()));
    s.store_class_end();
  }
}

void chatTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatTheme").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    light_settings_fieldID = jni::get_field_id(env, Class, "lightSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeSettings;").c_str());
    dark_settings_fieldID = jni::get_field_id(env, Class, "darkSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ChatType::Class;

object_ptr<ChatType> ChatType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case chatTypePrivate::ID:
      return chatTypePrivate::fetch(env, p);
    case chatTypeBasicGroup::ID:
      return chatTypeBasicGroup::fetch(env, p);
    case chatTypeSupergroup::ID:
      return chatTypeSupergroup::fetch(env, p);
    case chatTypeSecret::ID:
      return chatTypeSecret::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ChatType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass chatTypePrivate::Class;
jfieldID chatTypePrivate::user_id_fieldID;

chatTypePrivate::chatTypePrivate()
  : user_id_()
{}

chatTypePrivate::chatTypePrivate(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t chatTypePrivate::ID;

object_ptr<ChatType> chatTypePrivate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatTypePrivate> res = make_object<chatTypePrivate>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void chatTypePrivate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void chatTypePrivate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatTypePrivate");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void chatTypePrivate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatTypePrivate").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatTypeBasicGroup::Class;
jfieldID chatTypeBasicGroup::basic_group_id_fieldID;

chatTypeBasicGroup::chatTypeBasicGroup()
  : basic_group_id_()
{}

chatTypeBasicGroup::chatTypeBasicGroup(int53 basic_group_id_)
  : basic_group_id_(basic_group_id_)
{}

const std::int32_t chatTypeBasicGroup::ID;

object_ptr<ChatType> chatTypeBasicGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatTypeBasicGroup> res = make_object<chatTypeBasicGroup>();
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  return std::move(res);
}

void chatTypeBasicGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, basic_group_id_fieldID, basic_group_id_);
}

void chatTypeBasicGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatTypeBasicGroup");
    s.store_field("basicGroupId", basic_group_id_);
    s.store_class_end();
  }
}

void chatTypeBasicGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatTypeBasicGroup").c_str());
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chatTypeSupergroup::Class;
jfieldID chatTypeSupergroup::supergroup_id_fieldID;
jfieldID chatTypeSupergroup::is_channel_fieldID;

chatTypeSupergroup::chatTypeSupergroup()
  : supergroup_id_()
  , is_channel_()
{}

chatTypeSupergroup::chatTypeSupergroup(int53 supergroup_id_, bool is_channel_)
  : supergroup_id_(supergroup_id_)
  , is_channel_(is_channel_)
{}

const std::int32_t chatTypeSupergroup::ID;

object_ptr<ChatType> chatTypeSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatTypeSupergroup> res = make_object<chatTypeSupergroup>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  return std::move(res);
}

void chatTypeSupergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, supergroup_id_fieldID, supergroup_id_);
  env->SetBooleanField(s, is_channel_fieldID, is_channel_);
}

void chatTypeSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatTypeSupergroup");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("isChannel", is_channel_);
    s.store_class_end();
  }
}

void chatTypeSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatTypeSupergroup").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass chatTypeSecret::Class;
jfieldID chatTypeSecret::secret_chat_id_fieldID;
jfieldID chatTypeSecret::user_id_fieldID;

chatTypeSecret::chatTypeSecret()
  : secret_chat_id_()
  , user_id_()
{}

chatTypeSecret::chatTypeSecret(int32 secret_chat_id_, int53 user_id_)
  : secret_chat_id_(secret_chat_id_)
  , user_id_(user_id_)
{}

const std::int32_t chatTypeSecret::ID;

object_ptr<ChatType> chatTypeSecret::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chatTypeSecret> res = make_object<chatTypeSecret>();
  res->secret_chat_id_ = env->GetIntField(p, res->secret_chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void chatTypeSecret::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, secret_chat_id_fieldID, secret_chat_id_);
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void chatTypeSecret::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChatTypeSecret");
    s.store_field("secretChatId", secret_chat_id_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void chatTypeSecret::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChatTypeSecret").c_str());
    secret_chat_id_fieldID = jni::get_field_id(env, Class, "secretChatId", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass chats::Class;
jfieldID chats::total_count_fieldID;
jfieldID chats::chat_ids_fieldID;

chats::chats()
  : total_count_()
  , chat_ids_()
{}

chats::chats(int32 total_count_, array<int53> &&chat_ids_)
  : total_count_(total_count_)
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t chats::ID;

object_ptr<chats> chats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<chats> res = make_object<chats>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void chats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, chat_ids_); if (arr_tmp_) { env->SetObjectField(s, chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void chats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Chats");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void chats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Chats").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass CheckChatUsernameResult::Class;

object_ptr<CheckChatUsernameResult> CheckChatUsernameResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case checkChatUsernameResultOk::ID:
      return checkChatUsernameResultOk::fetch(env, p);
    case checkChatUsernameResultUsernameInvalid::ID:
      return checkChatUsernameResultUsernameInvalid::fetch(env, p);
    case checkChatUsernameResultUsernameOccupied::ID:
      return checkChatUsernameResultUsernameOccupied::fetch(env, p);
    case checkChatUsernameResultUsernamePurchasable::ID:
      return checkChatUsernameResultUsernamePurchasable::fetch(env, p);
    case checkChatUsernameResultPublicChatsTooMany::ID:
      return checkChatUsernameResultPublicChatsTooMany::fetch(env, p);
    case checkChatUsernameResultPublicGroupsUnavailable::ID:
      return checkChatUsernameResultPublicGroupsUnavailable::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CheckChatUsernameResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultOk::Class;

checkChatUsernameResultOk::checkChatUsernameResultOk() {
}

const std::int32_t checkChatUsernameResultOk::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultOk>();
}

void checkChatUsernameResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultOk");
    s.store_class_end();
  }
}

void checkChatUsernameResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultUsernameInvalid::Class;

checkChatUsernameResultUsernameInvalid::checkChatUsernameResultUsernameInvalid() {
}

const std::int32_t checkChatUsernameResultUsernameInvalid::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultUsernameInvalid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultUsernameInvalid>();
}

void checkChatUsernameResultUsernameInvalid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultUsernameInvalid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultUsernameInvalid");
    s.store_class_end();
  }
}

void checkChatUsernameResultUsernameInvalid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultUsernameInvalid").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultUsernameOccupied::Class;

checkChatUsernameResultUsernameOccupied::checkChatUsernameResultUsernameOccupied() {
}

const std::int32_t checkChatUsernameResultUsernameOccupied::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultUsernameOccupied::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultUsernameOccupied>();
}

void checkChatUsernameResultUsernameOccupied::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultUsernameOccupied::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultUsernameOccupied");
    s.store_class_end();
  }
}

void checkChatUsernameResultUsernameOccupied::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultUsernameOccupied").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultUsernamePurchasable::Class;

checkChatUsernameResultUsernamePurchasable::checkChatUsernameResultUsernamePurchasable() {
}

const std::int32_t checkChatUsernameResultUsernamePurchasable::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultUsernamePurchasable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultUsernamePurchasable>();
}

void checkChatUsernameResultUsernamePurchasable::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultUsernamePurchasable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultUsernamePurchasable");
    s.store_class_end();
  }
}

void checkChatUsernameResultUsernamePurchasable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultUsernamePurchasable").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultPublicChatsTooMany::Class;

checkChatUsernameResultPublicChatsTooMany::checkChatUsernameResultPublicChatsTooMany() {
}

const std::int32_t checkChatUsernameResultPublicChatsTooMany::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultPublicChatsTooMany::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultPublicChatsTooMany>();
}

void checkChatUsernameResultPublicChatsTooMany::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultPublicChatsTooMany::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultPublicChatsTooMany");
    s.store_class_end();
  }
}

void checkChatUsernameResultPublicChatsTooMany::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultPublicChatsTooMany").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsernameResultPublicGroupsUnavailable::Class;

checkChatUsernameResultPublicGroupsUnavailable::checkChatUsernameResultPublicGroupsUnavailable() {
}

const std::int32_t checkChatUsernameResultPublicGroupsUnavailable::ID;

object_ptr<CheckChatUsernameResult> checkChatUsernameResultPublicGroupsUnavailable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkChatUsernameResultPublicGroupsUnavailable>();
}

void checkChatUsernameResultPublicGroupsUnavailable::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkChatUsernameResultPublicGroupsUnavailable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsernameResultPublicGroupsUnavailable");
    s.store_class_end();
  }
}

void checkChatUsernameResultPublicGroupsUnavailable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsernameResultPublicGroupsUnavailable").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass CheckStickerSetNameResult::Class;

object_ptr<CheckStickerSetNameResult> CheckStickerSetNameResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case checkStickerSetNameResultOk::ID:
      return checkStickerSetNameResultOk::fetch(env, p);
    case checkStickerSetNameResultNameInvalid::ID:
      return checkStickerSetNameResultNameInvalid::fetch(env, p);
    case checkStickerSetNameResultNameOccupied::ID:
      return checkStickerSetNameResultNameOccupied::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CheckStickerSetNameResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckStickerSetNameResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkStickerSetNameResultOk::Class;

checkStickerSetNameResultOk::checkStickerSetNameResultOk() {
}

const std::int32_t checkStickerSetNameResultOk::ID;

object_ptr<CheckStickerSetNameResult> checkStickerSetNameResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkStickerSetNameResultOk>();
}

void checkStickerSetNameResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkStickerSetNameResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckStickerSetNameResultOk");
    s.store_class_end();
  }
}

void checkStickerSetNameResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckStickerSetNameResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkStickerSetNameResultNameInvalid::Class;

checkStickerSetNameResultNameInvalid::checkStickerSetNameResultNameInvalid() {
}

const std::int32_t checkStickerSetNameResultNameInvalid::ID;

object_ptr<CheckStickerSetNameResult> checkStickerSetNameResultNameInvalid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkStickerSetNameResultNameInvalid>();
}

void checkStickerSetNameResultNameInvalid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkStickerSetNameResultNameInvalid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckStickerSetNameResultNameInvalid");
    s.store_class_end();
  }
}

void checkStickerSetNameResultNameInvalid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckStickerSetNameResultNameInvalid").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkStickerSetNameResultNameOccupied::Class;

checkStickerSetNameResultNameOccupied::checkStickerSetNameResultNameOccupied() {
}

const std::int32_t checkStickerSetNameResultNameOccupied::ID;

object_ptr<CheckStickerSetNameResult> checkStickerSetNameResultNameOccupied::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<checkStickerSetNameResultNameOccupied>();
}

void checkStickerSetNameResultNameOccupied::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void checkStickerSetNameResultNameOccupied::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckStickerSetNameResultNameOccupied");
    s.store_class_end();
  }
}

void checkStickerSetNameResultNameOccupied::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckStickerSetNameResultNameOccupied").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass closeBirthdayUser::Class;
jfieldID closeBirthdayUser::user_id_fieldID;
jfieldID closeBirthdayUser::birthdate_fieldID;

closeBirthdayUser::closeBirthdayUser()
  : user_id_()
  , birthdate_()
{}

closeBirthdayUser::closeBirthdayUser(int53 user_id_, object_ptr<birthdate> &&birthdate_)
  : user_id_(user_id_)
  , birthdate_(std::move(birthdate_))
{}

const std::int32_t closeBirthdayUser::ID;

object_ptr<closeBirthdayUser> closeBirthdayUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closeBirthdayUser> res = make_object<closeBirthdayUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->birthdate_ = jni::fetch_tl_object<birthdate>(env, jni::fetch_object(env, p, res->birthdate_fieldID));
  return res;
}

void closeBirthdayUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (birthdate_ != nullptr) { jobject next; birthdate_->store(env, next); if (next) { env->SetObjectField(s, birthdate_fieldID, next); env->DeleteLocalRef(next); } }
}

void closeBirthdayUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CloseBirthdayUser");
    s.store_field("userId", user_id_);
    s.store_object_field("birthdate", static_cast<const BaseObject *>(birthdate_.get()));
    s.store_class_end();
  }
}

void closeBirthdayUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CloseBirthdayUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    birthdate_fieldID = jni::get_field_id(env, Class, "birthdate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Birthdate;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass closedVectorPath::Class;
jfieldID closedVectorPath::commands_fieldID;

closedVectorPath::closedVectorPath()
  : commands_()
{}

closedVectorPath::closedVectorPath(array<object_ptr<VectorPathCommand>> &&commands_)
  : commands_(std::move(commands_))
{}

const std::int32_t closedVectorPath::ID;

object_ptr<closedVectorPath> closedVectorPath::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closedVectorPath> res = make_object<closedVectorPath>();
  res->commands_ = jni::FetchVector<VectorPathCommand>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->commands_fieldID));
  return res;
}

void closedVectorPath::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, commands_); if (arr_tmp_) { env->SetObjectField(s, commands_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void closedVectorPath::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClosedVectorPath");
    { s.store_vector_begin("commands", commands_.size()); for (const auto &_value : commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void closedVectorPath::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClosedVectorPath").c_str());
    commands_fieldID = jni::get_field_id(env, Class, "commands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$VectorPathCommand;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass collectibleItemInfo::Class;
jfieldID collectibleItemInfo::purchase_date_fieldID;
jfieldID collectibleItemInfo::currency_fieldID;
jfieldID collectibleItemInfo::amount_fieldID;
jfieldID collectibleItemInfo::cryptocurrency_fieldID;
jfieldID collectibleItemInfo::cryptocurrency_amount_fieldID;
jfieldID collectibleItemInfo::url_fieldID;

collectibleItemInfo::collectibleItemInfo()
  : purchase_date_()
  , currency_()
  , amount_()
  , cryptocurrency_()
  , cryptocurrency_amount_()
  , url_()
{}

collectibleItemInfo::collectibleItemInfo(int32 purchase_date_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, string const &url_)
  : purchase_date_(purchase_date_)
  , currency_(currency_)
  , amount_(amount_)
  , cryptocurrency_(cryptocurrency_)
  , cryptocurrency_amount_(cryptocurrency_amount_)
  , url_(url_)
{}

const std::int32_t collectibleItemInfo::ID;

object_ptr<collectibleItemInfo> collectibleItemInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<collectibleItemInfo> res = make_object<collectibleItemInfo>();
  res->purchase_date_ = env->GetIntField(p, res->purchase_date_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->cryptocurrency_amount_ = env->GetLongField(p, res->cryptocurrency_amount_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void collectibleItemInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, purchase_date_fieldID, purchase_date_);
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cryptocurrency_amount_fieldID, cryptocurrency_amount_);
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void collectibleItemInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CollectibleItemInfo");
    s.store_field("purchaseDate", purchase_date_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("cryptocurrencyAmount", cryptocurrency_amount_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void collectibleItemInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CollectibleItemInfo").c_str());
    purchase_date_fieldID = jni::get_field_id(env, Class, "purchaseDate", "I");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    cryptocurrency_amount_fieldID = jni::get_field_id(env, Class, "cryptocurrencyAmount", "J");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass CollectibleItemType::Class;

object_ptr<CollectibleItemType> CollectibleItemType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case collectibleItemTypeUsername::ID:
      return collectibleItemTypeUsername::fetch(env, p);
    case collectibleItemTypePhoneNumber::ID:
      return collectibleItemTypePhoneNumber::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void CollectibleItemType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CollectibleItemType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass collectibleItemTypeUsername::Class;
jfieldID collectibleItemTypeUsername::username_fieldID;

collectibleItemTypeUsername::collectibleItemTypeUsername()
  : username_()
{}

collectibleItemTypeUsername::collectibleItemTypeUsername(string const &username_)
  : username_(username_)
{}

const std::int32_t collectibleItemTypeUsername::ID;

object_ptr<CollectibleItemType> collectibleItemTypeUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<collectibleItemTypeUsername> res = make_object<collectibleItemTypeUsername>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return std::move(res);
}

void collectibleItemTypeUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CollectibleItemTypeUsername");
    s.store_field("username", username_);
    s.store_class_end();
  }
}

void collectibleItemTypeUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CollectibleItemTypeUsername").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass collectibleItemTypePhoneNumber::Class;
jfieldID collectibleItemTypePhoneNumber::phone_number_fieldID;

collectibleItemTypePhoneNumber::collectibleItemTypePhoneNumber()
  : phone_number_()
{}

collectibleItemTypePhoneNumber::collectibleItemTypePhoneNumber(string const &phone_number_)
  : phone_number_(phone_number_)
{}

const std::int32_t collectibleItemTypePhoneNumber::ID;

object_ptr<CollectibleItemType> collectibleItemTypePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<collectibleItemTypePhoneNumber> res = make_object<collectibleItemTypePhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  return std::move(res);
}

void collectibleItemTypePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CollectibleItemTypePhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_class_end();
  }
}

void collectibleItemTypePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CollectibleItemTypePhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass connectedAffiliateProgram::Class;
jfieldID connectedAffiliateProgram::url_fieldID;
jfieldID connectedAffiliateProgram::bot_user_id_fieldID;
jfieldID connectedAffiliateProgram::parameters_fieldID;
jfieldID connectedAffiliateProgram::connection_date_fieldID;
jfieldID connectedAffiliateProgram::is_disconnected_fieldID;
jfieldID connectedAffiliateProgram::user_count_fieldID;
jfieldID connectedAffiliateProgram::revenue_star_count_fieldID;

connectedAffiliateProgram::connectedAffiliateProgram()
  : url_()
  , bot_user_id_()
  , parameters_()
  , connection_date_()
  , is_disconnected_()
  , user_count_()
  , revenue_star_count_()
{}

connectedAffiliateProgram::connectedAffiliateProgram(string const &url_, int53 bot_user_id_, object_ptr<affiliateProgramParameters> &&parameters_, int32 connection_date_, bool is_disconnected_, int64 user_count_, int64 revenue_star_count_)
  : url_(url_)
  , bot_user_id_(bot_user_id_)
  , parameters_(std::move(parameters_))
  , connection_date_(connection_date_)
  , is_disconnected_(is_disconnected_)
  , user_count_(user_count_)
  , revenue_star_count_(revenue_star_count_)
{}

const std::int32_t connectedAffiliateProgram::ID;

object_ptr<connectedAffiliateProgram> connectedAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<connectedAffiliateProgram> res = make_object<connectedAffiliateProgram>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->parameters_ = jni::fetch_tl_object<affiliateProgramParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->connection_date_ = env->GetIntField(p, res->connection_date_fieldID);
  res->is_disconnected_ = (env->GetBooleanField(p, res->is_disconnected_fieldID) != 0);
  res->user_count_ = env->GetLongField(p, res->user_count_fieldID);
  res->revenue_star_count_ = env->GetLongField(p, res->revenue_star_count_fieldID);
  return res;
}

void connectedAffiliateProgram::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  if (parameters_ != nullptr) { jobject next; parameters_->store(env, next); if (next) { env->SetObjectField(s, parameters_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, connection_date_fieldID, connection_date_);
  env->SetBooleanField(s, is_disconnected_fieldID, is_disconnected_);
  env->SetLongField(s, user_count_fieldID, user_count_);
  env->SetLongField(s, revenue_star_count_fieldID, revenue_star_count_);
}

void connectedAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectedAffiliateProgram");
    s.store_field("url", url_);
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("connectionDate", connection_date_);
    s.store_field("isDisconnected", is_disconnected_);
    s.store_field("userCount", user_count_);
    s.store_field("revenueStarCount", revenue_star_count_);
    s.store_class_end();
  }
}

void connectedAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectedAffiliateProgram").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramParameters;").c_str());
    connection_date_fieldID = jni::get_field_id(env, Class, "connectionDate", "I");
    is_disconnected_fieldID = jni::get_field_id(env, Class, "isDisconnected", "Z");
    user_count_fieldID = jni::get_field_id(env, Class, "userCount", "J");
    revenue_star_count_fieldID = jni::get_field_id(env, Class, "revenueStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass connectedAffiliatePrograms::Class;
jfieldID connectedAffiliatePrograms::total_count_fieldID;
jfieldID connectedAffiliatePrograms::programs_fieldID;
jfieldID connectedAffiliatePrograms::next_offset_fieldID;

connectedAffiliatePrograms::connectedAffiliatePrograms()
  : total_count_()
  , programs_()
  , next_offset_()
{}

connectedAffiliatePrograms::connectedAffiliatePrograms(int32 total_count_, array<object_ptr<connectedAffiliateProgram>> &&programs_, string const &next_offset_)
  : total_count_(total_count_)
  , programs_(std::move(programs_))
  , next_offset_(next_offset_)
{}

const std::int32_t connectedAffiliatePrograms::ID;

object_ptr<connectedAffiliatePrograms> connectedAffiliatePrograms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<connectedAffiliatePrograms> res = make_object<connectedAffiliatePrograms>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->programs_ = jni::FetchVector<connectedAffiliateProgram>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->programs_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void connectedAffiliatePrograms::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, programs_); if (arr_tmp_) { env->SetObjectField(s, programs_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void connectedAffiliatePrograms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectedAffiliatePrograms");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("programs", programs_.size()); for (const auto &_value : programs_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void connectedAffiliatePrograms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectedAffiliatePrograms").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    programs_fieldID = jni::get_field_id(env, Class, "programs", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ConnectedAffiliateProgram;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass connectedWebsite::Class;
jfieldID connectedWebsite::id_fieldID;
jfieldID connectedWebsite::domain_name_fieldID;
jfieldID connectedWebsite::bot_user_id_fieldID;
jfieldID connectedWebsite::browser_fieldID;
jfieldID connectedWebsite::platform_fieldID;
jfieldID connectedWebsite::log_in_date_fieldID;
jfieldID connectedWebsite::last_active_date_fieldID;
jfieldID connectedWebsite::ip_address_fieldID;
jfieldID connectedWebsite::location_fieldID;

connectedWebsite::connectedWebsite()
  : id_()
  , domain_name_()
  , bot_user_id_()
  , browser_()
  , platform_()
  , log_in_date_()
  , last_active_date_()
  , ip_address_()
  , location_()
{}

connectedWebsite::connectedWebsite(int64 id_, string const &domain_name_, int53 bot_user_id_, string const &browser_, string const &platform_, int32 log_in_date_, int32 last_active_date_, string const &ip_address_, string const &location_)
  : id_(id_)
  , domain_name_(domain_name_)
  , bot_user_id_(bot_user_id_)
  , browser_(browser_)
  , platform_(platform_)
  , log_in_date_(log_in_date_)
  , last_active_date_(last_active_date_)
  , ip_address_(ip_address_)
  , location_(location_)
{}

const std::int32_t connectedWebsite::ID;

object_ptr<connectedWebsite> connectedWebsite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<connectedWebsite> res = make_object<connectedWebsite>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->domain_name_ = jni::fetch_string(env, p, res->domain_name_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->browser_ = jni::fetch_string(env, p, res->browser_fieldID);
  res->platform_ = jni::fetch_string(env, p, res->platform_fieldID);
  res->log_in_date_ = env->GetIntField(p, res->log_in_date_fieldID);
  res->last_active_date_ = env->GetIntField(p, res->last_active_date_fieldID);
  res->ip_address_ = jni::fetch_string(env, p, res->ip_address_fieldID);
  res->location_ = jni::fetch_string(env, p, res->location_fieldID);
  return res;
}

void connectedWebsite::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, domain_name_); if (nextString) { env->SetObjectField(s, domain_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  { jstring nextString = jni::to_jstring(env, browser_); if (nextString) { env->SetObjectField(s, browser_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, platform_); if (nextString) { env->SetObjectField(s, platform_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, log_in_date_fieldID, log_in_date_);
  env->SetIntField(s, last_active_date_fieldID, last_active_date_);
  { jstring nextString = jni::to_jstring(env, ip_address_); if (nextString) { env->SetObjectField(s, ip_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, location_); if (nextString) { env->SetObjectField(s, location_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void connectedWebsite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectedWebsite");
    s.store_field("id", id_);
    s.store_field("domainName", domain_name_);
    s.store_field("botUserId", bot_user_id_);
    s.store_field("browser", browser_);
    s.store_field("platform", platform_);
    s.store_field("logInDate", log_in_date_);
    s.store_field("lastActiveDate", last_active_date_);
    s.store_field("ipAddress", ip_address_);
    s.store_field("location", location_);
    s.store_class_end();
  }
}

void connectedWebsite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectedWebsite").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    domain_name_fieldID = jni::get_field_id(env, Class, "domainName", "Ljava/lang/String;");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    browser_fieldID = jni::get_field_id(env, Class, "browser", "Ljava/lang/String;");
    platform_fieldID = jni::get_field_id(env, Class, "platform", "Ljava/lang/String;");
    log_in_date_fieldID = jni::get_field_id(env, Class, "logInDate", "I");
    last_active_date_fieldID = jni::get_field_id(env, Class, "lastActiveDate", "I");
    ip_address_fieldID = jni::get_field_id(env, Class, "ipAddress", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass connectedWebsites::Class;
jfieldID connectedWebsites::websites_fieldID;

connectedWebsites::connectedWebsites()
  : websites_()
{}

connectedWebsites::connectedWebsites(array<object_ptr<connectedWebsite>> &&websites_)
  : websites_(std::move(websites_))
{}

const std::int32_t connectedWebsites::ID;

object_ptr<connectedWebsites> connectedWebsites::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<connectedWebsites> res = make_object<connectedWebsites>();
  res->websites_ = jni::FetchVector<connectedWebsite>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->websites_fieldID));
  return res;
}

void connectedWebsites::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, websites_); if (arr_tmp_) { env->SetObjectField(s, websites_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void connectedWebsites::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectedWebsites");
    { s.store_vector_begin("websites", websites_.size()); for (const auto &_value : websites_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void connectedWebsites::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectedWebsites").c_str());
    websites_fieldID = jni::get_field_id(env, Class, "websites", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ConnectedWebsite;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ConnectionState::Class;

object_ptr<ConnectionState> ConnectionState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case connectionStateWaitingForNetwork::ID:
      return connectionStateWaitingForNetwork::fetch(env, p);
    case connectionStateConnectingToProxy::ID:
      return connectionStateConnectingToProxy::fetch(env, p);
    case connectionStateConnecting::ID:
      return connectionStateConnecting::fetch(env, p);
    case connectionStateUpdating::ID:
      return connectionStateUpdating::fetch(env, p);
    case connectionStateReady::ID:
      return connectionStateReady::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ConnectionState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass connectionStateWaitingForNetwork::Class;

connectionStateWaitingForNetwork::connectionStateWaitingForNetwork() {
}

const std::int32_t connectionStateWaitingForNetwork::ID;

object_ptr<ConnectionState> connectionStateWaitingForNetwork::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<connectionStateWaitingForNetwork>();
}

void connectionStateWaitingForNetwork::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void connectionStateWaitingForNetwork::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectionStateWaitingForNetwork");
    s.store_class_end();
  }
}

void connectionStateWaitingForNetwork::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionStateWaitingForNetwork").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass connectionStateConnectingToProxy::Class;

connectionStateConnectingToProxy::connectionStateConnectingToProxy() {
}

const std::int32_t connectionStateConnectingToProxy::ID;

object_ptr<ConnectionState> connectionStateConnectingToProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<connectionStateConnectingToProxy>();
}

void connectionStateConnectingToProxy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void connectionStateConnectingToProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectionStateConnectingToProxy");
    s.store_class_end();
  }
}

void connectionStateConnectingToProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionStateConnectingToProxy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass connectionStateConnecting::Class;

connectionStateConnecting::connectionStateConnecting() {
}

const std::int32_t connectionStateConnecting::ID;

object_ptr<ConnectionState> connectionStateConnecting::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<connectionStateConnecting>();
}

void connectionStateConnecting::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void connectionStateConnecting::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectionStateConnecting");
    s.store_class_end();
  }
}

void connectionStateConnecting::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionStateConnecting").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass connectionStateUpdating::Class;

connectionStateUpdating::connectionStateUpdating() {
}

const std::int32_t connectionStateUpdating::ID;

object_ptr<ConnectionState> connectionStateUpdating::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<connectionStateUpdating>();
}

void connectionStateUpdating::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void connectionStateUpdating::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectionStateUpdating");
    s.store_class_end();
  }
}

void connectionStateUpdating::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionStateUpdating").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass connectionStateReady::Class;

connectionStateReady::connectionStateReady() {
}

const std::int32_t connectionStateReady::ID;

object_ptr<ConnectionState> connectionStateReady::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<connectionStateReady>();
}

void connectionStateReady::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void connectionStateReady::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectionStateReady");
    s.store_class_end();
  }
}

void connectionStateReady::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectionStateReady").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass contact::Class;
jfieldID contact::phone_number_fieldID;
jfieldID contact::first_name_fieldID;
jfieldID contact::last_name_fieldID;
jfieldID contact::vcard_fieldID;
jfieldID contact::user_id_fieldID;

contact::contact()
  : phone_number_()
  , first_name_()
  , last_name_()
  , vcard_()
  , user_id_()
{}

contact::contact(string const &phone_number_, string const &first_name_, string const &last_name_, string const &vcard_, int53 user_id_)
  : phone_number_(phone_number_)
  , first_name_(first_name_)
  , last_name_(last_name_)
  , vcard_(vcard_)
  , user_id_(user_id_)
{}

const std::int32_t contact::ID;

object_ptr<contact> contact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<contact> res = make_object<contact>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  res->vcard_ = jni::fetch_string(env, p, res->vcard_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void contact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, first_name_); if (nextString) { env->SetObjectField(s, first_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, last_name_); if (nextString) { env->SetObjectField(s, last_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, vcard_); if (nextString) { env->SetObjectField(s, vcard_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void contact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Contact");
    s.store_field("phoneNumber", phone_number_);
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_field("vcard", vcard_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void contact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Contact").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    vcard_fieldID = jni::get_field_id(env, Class, "vcard", "Ljava/lang/String;");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass count::Class;
jfieldID count::count_fieldID;

count::count()
  : count_()
{}

count::count(int32 count_)
  : count_(count_)
{}

const std::int32_t count::ID;

object_ptr<count> count::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<count> res = make_object<count>();
  res->count_ = env->GetIntField(p, res->count_fieldID);
  return res;
}

void count::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, count_fieldID, count_);
}

void count::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Count");
    s.store_field("count", count_);
    s.store_class_end();
  }
}

void count::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Count").c_str());
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    return true;
  }();
  (void)is_inited;
}

jclass countries::Class;
jfieldID countries::countries_fieldID;

countries::countries()
  : countries_()
{}

countries::countries(array<object_ptr<countryInfo>> &&countries_)
  : countries_(std::move(countries_))
{}

const std::int32_t countries::ID;

object_ptr<countries> countries::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<countries> res = make_object<countries>();
  res->countries_ = jni::FetchVector<countryInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->countries_fieldID));
  return res;
}

void countries::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, countries_); if (arr_tmp_) { env->SetObjectField(s, countries_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void countries::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Countries");
    { s.store_vector_begin("countries", countries_.size()); for (const auto &_value : countries_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void countries::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Countries").c_str());
    countries_fieldID = jni::get_field_id(env, Class, "countries", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$CountryInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass countryInfo::Class;
jfieldID countryInfo::country_code_fieldID;
jfieldID countryInfo::name_fieldID;
jfieldID countryInfo::english_name_fieldID;
jfieldID countryInfo::is_hidden_fieldID;
jfieldID countryInfo::calling_codes_fieldID;

countryInfo::countryInfo()
  : country_code_()
  , name_()
  , english_name_()
  , is_hidden_()
  , calling_codes_()
{}

countryInfo::countryInfo(string const &country_code_, string const &name_, string const &english_name_, bool is_hidden_, array<string> &&calling_codes_)
  : country_code_(country_code_)
  , name_(name_)
  , english_name_(english_name_)
  , is_hidden_(is_hidden_)
  , calling_codes_(std::move(calling_codes_))
{}

const std::int32_t countryInfo::ID;

object_ptr<countryInfo> countryInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<countryInfo> res = make_object<countryInfo>();
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->english_name_ = jni::fetch_string(env, p, res->english_name_fieldID);
  res->is_hidden_ = (env->GetBooleanField(p, res->is_hidden_fieldID) != 0);
  res->calling_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->calling_codes_fieldID));
  return res;
}

void countryInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, country_code_); if (nextString) { env->SetObjectField(s, country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, english_name_); if (nextString) { env->SetObjectField(s, english_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_hidden_fieldID, is_hidden_);
  { auto arr_tmp_ = jni::store_vector(env, calling_codes_); if (arr_tmp_) { env->SetObjectField(s, calling_codes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void countryInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CountryInfo");
    s.store_field("countryCode", country_code_);
    s.store_field("name", name_);
    s.store_field("englishName", english_name_);
    s.store_field("isHidden", is_hidden_);
    { s.store_vector_begin("callingCodes", calling_codes_.size()); for (const auto &_value : calling_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void countryInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CountryInfo").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    english_name_fieldID = jni::get_field_id(env, Class, "englishName", "Ljava/lang/String;");
    is_hidden_fieldID = jni::get_field_id(env, Class, "isHidden", "Z");
    calling_codes_fieldID = jni::get_field_id(env, Class, "callingCodes", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass createdBasicGroupChat::Class;
jfieldID createdBasicGroupChat::chat_id_fieldID;
jfieldID createdBasicGroupChat::failed_to_add_members_fieldID;

createdBasicGroupChat::createdBasicGroupChat()
  : chat_id_()
  , failed_to_add_members_()
{}

createdBasicGroupChat::createdBasicGroupChat(int53 chat_id_, object_ptr<failedToAddMembers> &&failed_to_add_members_)
  : chat_id_(chat_id_)
  , failed_to_add_members_(std::move(failed_to_add_members_))
{}

const std::int32_t createdBasicGroupChat::ID;

object_ptr<createdBasicGroupChat> createdBasicGroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createdBasicGroupChat> res = make_object<createdBasicGroupChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->failed_to_add_members_ = jni::fetch_tl_object<failedToAddMembers>(env, jni::fetch_object(env, p, res->failed_to_add_members_fieldID));
  return res;
}

void createdBasicGroupChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (failed_to_add_members_ != nullptr) { jobject next; failed_to_add_members_->store(env, next); if (next) { env->SetObjectField(s, failed_to_add_members_fieldID, next); env->DeleteLocalRef(next); } }
}

void createdBasicGroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreatedBasicGroupChat");
    s.store_field("chatId", chat_id_);
    s.store_object_field("failedToAddMembers", static_cast<const BaseObject *>(failed_to_add_members_.get()));
    s.store_class_end();
  }
}

void createdBasicGroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreatedBasicGroupChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    failed_to_add_members_fieldID = jni::get_field_id(env, Class, "failedToAddMembers", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FailedToAddMembers;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass currentWeather::Class;
jfieldID currentWeather::temperature_fieldID;
jfieldID currentWeather::emoji_fieldID;

currentWeather::currentWeather()
  : temperature_()
  , emoji_()
{}

currentWeather::currentWeather(double temperature_, string const &emoji_)
  : temperature_(temperature_)
  , emoji_(emoji_)
{}

const std::int32_t currentWeather::ID;

object_ptr<currentWeather> currentWeather::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<currentWeather> res = make_object<currentWeather>();
  res->temperature_ = env->GetDoubleField(p, res->temperature_fieldID);
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return res;
}

void currentWeather::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, temperature_fieldID, temperature_);
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void currentWeather::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CurrentWeather");
    s.store_field("temperature", temperature_);
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

void currentWeather::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CurrentWeather").c_str());
    temperature_fieldID = jni::get_field_id(env, Class, "temperature", "D");
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass customRequestResult::Class;
jfieldID customRequestResult::result_fieldID;

customRequestResult::customRequestResult()
  : result_()
{}

customRequestResult::customRequestResult(string const &result_)
  : result_(result_)
{}

const std::int32_t customRequestResult::ID;

object_ptr<customRequestResult> customRequestResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<customRequestResult> res = make_object<customRequestResult>();
  res->result_ = jni::fetch_string(env, p, res->result_fieldID);
  return res;
}

void customRequestResult::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, result_); if (nextString) { env->SetObjectField(s, result_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void customRequestResult::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CustomRequestResult");
    s.store_field("result", result_);
    s.store_class_end();
  }
}

void customRequestResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CustomRequestResult").c_str());
    result_fieldID = jni::get_field_id(env, Class, "result", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass data::Class;
jfieldID data::data_fieldID;

data::data()
  : data_()
{}

data::data(bytes const &data_)
  : data_(std::move(data_))
{}

const std::int32_t data::ID;

object_ptr<data> data::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<data> res = make_object<data>();
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void data::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void data::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Data");
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void data::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Data").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass databaseStatistics::Class;
jfieldID databaseStatistics::statistics_fieldID;

databaseStatistics::databaseStatistics()
  : statistics_()
{}

databaseStatistics::databaseStatistics(string const &statistics_)
  : statistics_(statistics_)
{}

const std::int32_t databaseStatistics::ID;

object_ptr<databaseStatistics> databaseStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<databaseStatistics> res = make_object<databaseStatistics>();
  res->statistics_ = jni::fetch_string(env, p, res->statistics_fieldID);
  return res;
}

void databaseStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, statistics_); if (nextString) { env->SetObjectField(s, statistics_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void databaseStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DatabaseStatistics");
    s.store_field("statistics", statistics_);
    s.store_class_end();
  }
}

void databaseStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DatabaseStatistics").c_str());
    statistics_fieldID = jni::get_field_id(env, Class, "statistics", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass date::Class;
jfieldID date::day_fieldID;
jfieldID date::month_fieldID;
jfieldID date::year_fieldID;

date::date()
  : day_()
  , month_()
  , year_()
{}

date::date(int32 day_, int32 month_, int32 year_)
  : day_(day_)
  , month_(month_)
  , year_(year_)
{}

const std::int32_t date::ID;

object_ptr<date> date::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<date> res = make_object<date>();
  res->day_ = env->GetIntField(p, res->day_fieldID);
  res->month_ = env->GetIntField(p, res->month_fieldID);
  res->year_ = env->GetIntField(p, res->year_fieldID);
  return res;
}

void date::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, day_fieldID, day_);
  env->SetIntField(s, month_fieldID, month_);
  env->SetIntField(s, year_fieldID, year_);
}

void date::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Date");
    s.store_field("day", day_);
    s.store_field("month", month_);
    s.store_field("year", year_);
    s.store_class_end();
  }
}

void date::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Date").c_str());
    day_fieldID = jni::get_field_id(env, Class, "day", "I");
    month_fieldID = jni::get_field_id(env, Class, "month", "I");
    year_fieldID = jni::get_field_id(env, Class, "year", "I");
    return true;
  }();
  (void)is_inited;
}

jclass dateRange::Class;
jfieldID dateRange::start_date_fieldID;
jfieldID dateRange::end_date_fieldID;

dateRange::dateRange()
  : start_date_()
  , end_date_()
{}

dateRange::dateRange(int32 start_date_, int32 end_date_)
  : start_date_(start_date_)
  , end_date_(end_date_)
{}

const std::int32_t dateRange::ID;

object_ptr<dateRange> dateRange::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<dateRange> res = make_object<dateRange>();
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->end_date_ = env->GetIntField(p, res->end_date_fieldID);
  return res;
}

void dateRange::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, start_date_fieldID, start_date_);
  env->SetIntField(s, end_date_fieldID, end_date_);
}

void dateRange::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DateRange");
    s.store_field("startDate", start_date_);
    s.store_field("endDate", end_date_);
    s.store_class_end();
  }
}

void dateRange::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DateRange").c_str());
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    end_date_fieldID = jni::get_field_id(env, Class, "endDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass datedFile::Class;
jfieldID datedFile::file_fieldID;
jfieldID datedFile::date_fieldID;

datedFile::datedFile()
  : file_()
  , date_()
{}

datedFile::datedFile(object_ptr<file> &&file_, int32 date_)
  : file_(std::move(file_))
  , date_(date_)
{}

const std::int32_t datedFile::ID;

object_ptr<datedFile> datedFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<datedFile> res = make_object<datedFile>();
  res->file_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->file_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void datedFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (file_ != nullptr) { jobject next; file_->store(env, next); if (next) { env->SetObjectField(s, file_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
}

void datedFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DatedFile");
    s.store_object_field("file", static_cast<const BaseObject *>(file_.get()));
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void datedFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DatedFile").c_str());
    file_fieldID = jni::get_field_id(env, Class, "file", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass deepLinkInfo::Class;
jfieldID deepLinkInfo::text_fieldID;
jfieldID deepLinkInfo::need_update_application_fieldID;

deepLinkInfo::deepLinkInfo()
  : text_()
  , need_update_application_()
{}

deepLinkInfo::deepLinkInfo(object_ptr<formattedText> &&text_, bool need_update_application_)
  : text_(std::move(text_))
  , need_update_application_(need_update_application_)
{}

const std::int32_t deepLinkInfo::ID;

object_ptr<deepLinkInfo> deepLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deepLinkInfo> res = make_object<deepLinkInfo>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->need_update_application_ = (env->GetBooleanField(p, res->need_update_application_fieldID) != 0);
  return res;
}

void deepLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, need_update_application_fieldID, need_update_application_);
}

void deepLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeepLinkInfo");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("needUpdateApplication", need_update_application_);
    s.store_class_end();
  }
}

void deepLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeepLinkInfo").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    need_update_application_fieldID = jni::get_field_id(env, Class, "needUpdateApplication", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass DeviceToken::Class;

object_ptr<DeviceToken> DeviceToken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case deviceTokenFirebaseCloudMessaging::ID:
      return deviceTokenFirebaseCloudMessaging::fetch(env, p);
    case deviceTokenApplePush::ID:
      return deviceTokenApplePush::fetch(env, p);
    case deviceTokenApplePushVoIP::ID:
      return deviceTokenApplePushVoIP::fetch(env, p);
    case deviceTokenWindowsPush::ID:
      return deviceTokenWindowsPush::fetch(env, p);
    case deviceTokenMicrosoftPush::ID:
      return deviceTokenMicrosoftPush::fetch(env, p);
    case deviceTokenMicrosoftPushVoIP::ID:
      return deviceTokenMicrosoftPushVoIP::fetch(env, p);
    case deviceTokenWebPush::ID:
      return deviceTokenWebPush::fetch(env, p);
    case deviceTokenSimplePush::ID:
      return deviceTokenSimplePush::fetch(env, p);
    case deviceTokenUbuntuPush::ID:
      return deviceTokenUbuntuPush::fetch(env, p);
    case deviceTokenBlackBerryPush::ID:
      return deviceTokenBlackBerryPush::fetch(env, p);
    case deviceTokenTizenPush::ID:
      return deviceTokenTizenPush::fetch(env, p);
    case deviceTokenHuaweiPush::ID:
      return deviceTokenHuaweiPush::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void DeviceToken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceToken").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenFirebaseCloudMessaging::Class;
jfieldID deviceTokenFirebaseCloudMessaging::token_fieldID;
jfieldID deviceTokenFirebaseCloudMessaging::encrypt_fieldID;

deviceTokenFirebaseCloudMessaging::deviceTokenFirebaseCloudMessaging()
  : token_()
  , encrypt_()
{}

deviceTokenFirebaseCloudMessaging::deviceTokenFirebaseCloudMessaging(string const &token_, bool encrypt_)
  : token_(token_)
  , encrypt_(encrypt_)
{}

const std::int32_t deviceTokenFirebaseCloudMessaging::ID;

object_ptr<DeviceToken> deviceTokenFirebaseCloudMessaging::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenFirebaseCloudMessaging> res = make_object<deviceTokenFirebaseCloudMessaging>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  res->encrypt_ = (env->GetBooleanField(p, res->encrypt_fieldID) != 0);
  return std::move(res);
}

void deviceTokenFirebaseCloudMessaging::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenFirebaseCloudMessaging");
    s.store_field("token", token_);
    s.store_field("encrypt", encrypt_);
    s.store_class_end();
  }
}

void deviceTokenFirebaseCloudMessaging::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenFirebaseCloudMessaging").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    encrypt_fieldID = jni::get_field_id(env, Class, "encrypt", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenApplePush::Class;
jfieldID deviceTokenApplePush::device_token_fieldID;
jfieldID deviceTokenApplePush::is_app_sandbox_fieldID;

deviceTokenApplePush::deviceTokenApplePush()
  : device_token_()
  , is_app_sandbox_()
{}

deviceTokenApplePush::deviceTokenApplePush(string const &device_token_, bool is_app_sandbox_)
  : device_token_(device_token_)
  , is_app_sandbox_(is_app_sandbox_)
{}

const std::int32_t deviceTokenApplePush::ID;

object_ptr<DeviceToken> deviceTokenApplePush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenApplePush> res = make_object<deviceTokenApplePush>();
  res->device_token_ = jni::fetch_string(env, p, res->device_token_fieldID);
  res->is_app_sandbox_ = (env->GetBooleanField(p, res->is_app_sandbox_fieldID) != 0);
  return std::move(res);
}

void deviceTokenApplePush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenApplePush");
    s.store_field("deviceToken", device_token_);
    s.store_field("isAppSandbox", is_app_sandbox_);
    s.store_class_end();
  }
}

void deviceTokenApplePush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenApplePush").c_str());
    device_token_fieldID = jni::get_field_id(env, Class, "deviceToken", "Ljava/lang/String;");
    is_app_sandbox_fieldID = jni::get_field_id(env, Class, "isAppSandbox", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenApplePushVoIP::Class;
jfieldID deviceTokenApplePushVoIP::device_token_fieldID;
jfieldID deviceTokenApplePushVoIP::is_app_sandbox_fieldID;
jfieldID deviceTokenApplePushVoIP::encrypt_fieldID;

deviceTokenApplePushVoIP::deviceTokenApplePushVoIP()
  : device_token_()
  , is_app_sandbox_()
  , encrypt_()
{}

deviceTokenApplePushVoIP::deviceTokenApplePushVoIP(string const &device_token_, bool is_app_sandbox_, bool encrypt_)
  : device_token_(device_token_)
  , is_app_sandbox_(is_app_sandbox_)
  , encrypt_(encrypt_)
{}

const std::int32_t deviceTokenApplePushVoIP::ID;

object_ptr<DeviceToken> deviceTokenApplePushVoIP::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenApplePushVoIP> res = make_object<deviceTokenApplePushVoIP>();
  res->device_token_ = jni::fetch_string(env, p, res->device_token_fieldID);
  res->is_app_sandbox_ = (env->GetBooleanField(p, res->is_app_sandbox_fieldID) != 0);
  res->encrypt_ = (env->GetBooleanField(p, res->encrypt_fieldID) != 0);
  return std::move(res);
}

void deviceTokenApplePushVoIP::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenApplePushVoIP");
    s.store_field("deviceToken", device_token_);
    s.store_field("isAppSandbox", is_app_sandbox_);
    s.store_field("encrypt", encrypt_);
    s.store_class_end();
  }
}

void deviceTokenApplePushVoIP::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenApplePushVoIP").c_str());
    device_token_fieldID = jni::get_field_id(env, Class, "deviceToken", "Ljava/lang/String;");
    is_app_sandbox_fieldID = jni::get_field_id(env, Class, "isAppSandbox", "Z");
    encrypt_fieldID = jni::get_field_id(env, Class, "encrypt", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenWindowsPush::Class;
jfieldID deviceTokenWindowsPush::access_token_fieldID;

deviceTokenWindowsPush::deviceTokenWindowsPush()
  : access_token_()
{}

deviceTokenWindowsPush::deviceTokenWindowsPush(string const &access_token_)
  : access_token_(access_token_)
{}

const std::int32_t deviceTokenWindowsPush::ID;

object_ptr<DeviceToken> deviceTokenWindowsPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenWindowsPush> res = make_object<deviceTokenWindowsPush>();
  res->access_token_ = jni::fetch_string(env, p, res->access_token_fieldID);
  return std::move(res);
}

void deviceTokenWindowsPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenWindowsPush");
    s.store_field("accessToken", access_token_);
    s.store_class_end();
  }
}

void deviceTokenWindowsPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenWindowsPush").c_str());
    access_token_fieldID = jni::get_field_id(env, Class, "accessToken", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenMicrosoftPush::Class;
jfieldID deviceTokenMicrosoftPush::channel_uri_fieldID;

deviceTokenMicrosoftPush::deviceTokenMicrosoftPush()
  : channel_uri_()
{}

deviceTokenMicrosoftPush::deviceTokenMicrosoftPush(string const &channel_uri_)
  : channel_uri_(channel_uri_)
{}

const std::int32_t deviceTokenMicrosoftPush::ID;

object_ptr<DeviceToken> deviceTokenMicrosoftPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenMicrosoftPush> res = make_object<deviceTokenMicrosoftPush>();
  res->channel_uri_ = jni::fetch_string(env, p, res->channel_uri_fieldID);
  return std::move(res);
}

void deviceTokenMicrosoftPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenMicrosoftPush");
    s.store_field("channelUri", channel_uri_);
    s.store_class_end();
  }
}

void deviceTokenMicrosoftPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenMicrosoftPush").c_str());
    channel_uri_fieldID = jni::get_field_id(env, Class, "channelUri", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenMicrosoftPushVoIP::Class;
jfieldID deviceTokenMicrosoftPushVoIP::channel_uri_fieldID;

deviceTokenMicrosoftPushVoIP::deviceTokenMicrosoftPushVoIP()
  : channel_uri_()
{}

deviceTokenMicrosoftPushVoIP::deviceTokenMicrosoftPushVoIP(string const &channel_uri_)
  : channel_uri_(channel_uri_)
{}

const std::int32_t deviceTokenMicrosoftPushVoIP::ID;

object_ptr<DeviceToken> deviceTokenMicrosoftPushVoIP::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenMicrosoftPushVoIP> res = make_object<deviceTokenMicrosoftPushVoIP>();
  res->channel_uri_ = jni::fetch_string(env, p, res->channel_uri_fieldID);
  return std::move(res);
}

void deviceTokenMicrosoftPushVoIP::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenMicrosoftPushVoIP");
    s.store_field("channelUri", channel_uri_);
    s.store_class_end();
  }
}

void deviceTokenMicrosoftPushVoIP::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenMicrosoftPushVoIP").c_str());
    channel_uri_fieldID = jni::get_field_id(env, Class, "channelUri", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenWebPush::Class;
jfieldID deviceTokenWebPush::endpoint_fieldID;
jfieldID deviceTokenWebPush::p256dh_base64url_fieldID;
jfieldID deviceTokenWebPush::auth_base64url_fieldID;

deviceTokenWebPush::deviceTokenWebPush()
  : endpoint_()
  , p256dh_base64url_()
  , auth_base64url_()
{}

deviceTokenWebPush::deviceTokenWebPush(string const &endpoint_, string const &p256dh_base64url_, string const &auth_base64url_)
  : endpoint_(endpoint_)
  , p256dh_base64url_(p256dh_base64url_)
  , auth_base64url_(auth_base64url_)
{}

const std::int32_t deviceTokenWebPush::ID;

object_ptr<DeviceToken> deviceTokenWebPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenWebPush> res = make_object<deviceTokenWebPush>();
  res->endpoint_ = jni::fetch_string(env, p, res->endpoint_fieldID);
  res->p256dh_base64url_ = jni::fetch_string(env, p, res->p256dh_base64url_fieldID);
  res->auth_base64url_ = jni::fetch_string(env, p, res->auth_base64url_fieldID);
  return std::move(res);
}

void deviceTokenWebPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenWebPush");
    s.store_field("endpoint", endpoint_);
    s.store_field("p256dhBase64url", p256dh_base64url_);
    s.store_field("authBase64url", auth_base64url_);
    s.store_class_end();
  }
}

void deviceTokenWebPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenWebPush").c_str());
    endpoint_fieldID = jni::get_field_id(env, Class, "endpoint", "Ljava/lang/String;");
    p256dh_base64url_fieldID = jni::get_field_id(env, Class, "p256dhBase64url", "Ljava/lang/String;");
    auth_base64url_fieldID = jni::get_field_id(env, Class, "authBase64url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenSimplePush::Class;
jfieldID deviceTokenSimplePush::endpoint_fieldID;

deviceTokenSimplePush::deviceTokenSimplePush()
  : endpoint_()
{}

deviceTokenSimplePush::deviceTokenSimplePush(string const &endpoint_)
  : endpoint_(endpoint_)
{}

const std::int32_t deviceTokenSimplePush::ID;

object_ptr<DeviceToken> deviceTokenSimplePush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenSimplePush> res = make_object<deviceTokenSimplePush>();
  res->endpoint_ = jni::fetch_string(env, p, res->endpoint_fieldID);
  return std::move(res);
}

void deviceTokenSimplePush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenSimplePush");
    s.store_field("endpoint", endpoint_);
    s.store_class_end();
  }
}

void deviceTokenSimplePush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenSimplePush").c_str());
    endpoint_fieldID = jni::get_field_id(env, Class, "endpoint", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenUbuntuPush::Class;
jfieldID deviceTokenUbuntuPush::token_fieldID;

deviceTokenUbuntuPush::deviceTokenUbuntuPush()
  : token_()
{}

deviceTokenUbuntuPush::deviceTokenUbuntuPush(string const &token_)
  : token_(token_)
{}

const std::int32_t deviceTokenUbuntuPush::ID;

object_ptr<DeviceToken> deviceTokenUbuntuPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenUbuntuPush> res = make_object<deviceTokenUbuntuPush>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void deviceTokenUbuntuPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenUbuntuPush");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void deviceTokenUbuntuPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenUbuntuPush").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenBlackBerryPush::Class;
jfieldID deviceTokenBlackBerryPush::token_fieldID;

deviceTokenBlackBerryPush::deviceTokenBlackBerryPush()
  : token_()
{}

deviceTokenBlackBerryPush::deviceTokenBlackBerryPush(string const &token_)
  : token_(token_)
{}

const std::int32_t deviceTokenBlackBerryPush::ID;

object_ptr<DeviceToken> deviceTokenBlackBerryPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenBlackBerryPush> res = make_object<deviceTokenBlackBerryPush>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void deviceTokenBlackBerryPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenBlackBerryPush");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void deviceTokenBlackBerryPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenBlackBerryPush").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenTizenPush::Class;
jfieldID deviceTokenTizenPush::reg_id_fieldID;

deviceTokenTizenPush::deviceTokenTizenPush()
  : reg_id_()
{}

deviceTokenTizenPush::deviceTokenTizenPush(string const &reg_id_)
  : reg_id_(reg_id_)
{}

const std::int32_t deviceTokenTizenPush::ID;

object_ptr<DeviceToken> deviceTokenTizenPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenTizenPush> res = make_object<deviceTokenTizenPush>();
  res->reg_id_ = jni::fetch_string(env, p, res->reg_id_fieldID);
  return std::move(res);
}

void deviceTokenTizenPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenTizenPush");
    s.store_field("regId", reg_id_);
    s.store_class_end();
  }
}

void deviceTokenTizenPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenTizenPush").c_str());
    reg_id_fieldID = jni::get_field_id(env, Class, "regId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deviceTokenHuaweiPush::Class;
jfieldID deviceTokenHuaweiPush::token_fieldID;
jfieldID deviceTokenHuaweiPush::encrypt_fieldID;

deviceTokenHuaweiPush::deviceTokenHuaweiPush()
  : token_()
  , encrypt_()
{}

deviceTokenHuaweiPush::deviceTokenHuaweiPush(string const &token_, bool encrypt_)
  : token_(token_)
  , encrypt_(encrypt_)
{}

const std::int32_t deviceTokenHuaweiPush::ID;

object_ptr<DeviceToken> deviceTokenHuaweiPush::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deviceTokenHuaweiPush> res = make_object<deviceTokenHuaweiPush>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  res->encrypt_ = (env->GetBooleanField(p, res->encrypt_fieldID) != 0);
  return std::move(res);
}

void deviceTokenHuaweiPush::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeviceTokenHuaweiPush");
    s.store_field("token", token_);
    s.store_field("encrypt", encrypt_);
    s.store_class_end();
  }
}

void deviceTokenHuaweiPush::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeviceTokenHuaweiPush").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    encrypt_fieldID = jni::get_field_id(env, Class, "encrypt", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass DiceStickers::Class;

object_ptr<DiceStickers> DiceStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case diceStickersRegular::ID:
      return diceStickersRegular::fetch(env, p);
    case diceStickersSlotMachine::ID:
      return diceStickersSlotMachine::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void DiceStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DiceStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass diceStickersRegular::Class;
jfieldID diceStickersRegular::sticker_fieldID;

diceStickersRegular::diceStickersRegular()
  : sticker_()
{}

diceStickersRegular::diceStickersRegular(object_ptr<sticker> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t diceStickersRegular::ID;

object_ptr<DiceStickers> diceStickersRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<diceStickersRegular> res = make_object<diceStickersRegular>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void diceStickersRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void diceStickersRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DiceStickersRegular");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void diceStickersRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DiceStickersRegular").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass diceStickersSlotMachine::Class;
jfieldID diceStickersSlotMachine::background_fieldID;
jfieldID diceStickersSlotMachine::lever_fieldID;
jfieldID diceStickersSlotMachine::left_reel_fieldID;
jfieldID diceStickersSlotMachine::center_reel_fieldID;
jfieldID diceStickersSlotMachine::right_reel_fieldID;

diceStickersSlotMachine::diceStickersSlotMachine()
  : background_()
  , lever_()
  , left_reel_()
  , center_reel_()
  , right_reel_()
{}

diceStickersSlotMachine::diceStickersSlotMachine(object_ptr<sticker> &&background_, object_ptr<sticker> &&lever_, object_ptr<sticker> &&left_reel_, object_ptr<sticker> &&center_reel_, object_ptr<sticker> &&right_reel_)
  : background_(std::move(background_))
  , lever_(std::move(lever_))
  , left_reel_(std::move(left_reel_))
  , center_reel_(std::move(center_reel_))
  , right_reel_(std::move(right_reel_))
{}

const std::int32_t diceStickersSlotMachine::ID;

object_ptr<DiceStickers> diceStickersSlotMachine::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<diceStickersSlotMachine> res = make_object<diceStickersSlotMachine>();
  res->background_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->lever_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->lever_fieldID));
  res->left_reel_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->left_reel_fieldID));
  res->center_reel_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->center_reel_fieldID));
  res->right_reel_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->right_reel_fieldID));
  return std::move(res);
}

void diceStickersSlotMachine::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
  if (lever_ != nullptr) { jobject next; lever_->store(env, next); if (next) { env->SetObjectField(s, lever_fieldID, next); env->DeleteLocalRef(next); } }
  if (left_reel_ != nullptr) { jobject next; left_reel_->store(env, next); if (next) { env->SetObjectField(s, left_reel_fieldID, next); env->DeleteLocalRef(next); } }
  if (center_reel_ != nullptr) { jobject next; center_reel_->store(env, next); if (next) { env->SetObjectField(s, center_reel_fieldID, next); env->DeleteLocalRef(next); } }
  if (right_reel_ != nullptr) { jobject next; right_reel_->store(env, next); if (next) { env->SetObjectField(s, right_reel_fieldID, next); env->DeleteLocalRef(next); } }
}

void diceStickersSlotMachine::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DiceStickersSlotMachine");
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_object_field("lever", static_cast<const BaseObject *>(lever_.get()));
    s.store_object_field("leftReel", static_cast<const BaseObject *>(left_reel_.get()));
    s.store_object_field("centerReel", static_cast<const BaseObject *>(center_reel_.get()));
    s.store_object_field("rightReel", static_cast<const BaseObject *>(right_reel_.get()));
    s.store_class_end();
  }
}

void diceStickersSlotMachine::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DiceStickersSlotMachine").c_str());
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    lever_fieldID = jni::get_field_id(env, Class, "lever", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    left_reel_fieldID = jni::get_field_id(env, Class, "leftReel", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    center_reel_fieldID = jni::get_field_id(env, Class, "centerReel", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    right_reel_fieldID = jni::get_field_id(env, Class, "rightReel", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass document::Class;
jfieldID document::file_name_fieldID;
jfieldID document::mime_type_fieldID;
jfieldID document::minithumbnail_fieldID;
jfieldID document::thumbnail_fieldID;
jfieldID document::document_fieldID;

document::document()
  : file_name_()
  , mime_type_()
  , minithumbnail_()
  , thumbnail_()
  , document_()
{}

document::document(string const &file_name_, string const &mime_type_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&document_)
  : file_name_(file_name_)
  , mime_type_(mime_type_)
  , minithumbnail_(std::move(minithumbnail_))
  , thumbnail_(std::move(thumbnail_))
  , document_(std::move(document_))
{}

const std::int32_t document::ID;

object_ptr<document> document::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<document> res = make_object<document>();
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->document_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->document_fieldID));
  return res;
}

void document::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, file_name_); if (nextString) { env->SetObjectField(s, file_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
}

void document::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Document");
    s.store_field("fileName", file_name_);
    s.store_field("mimeType", mime_type_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_class_end();
  }
}

void document::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Document").c_str());
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass downloadedFileCounts::Class;
jfieldID downloadedFileCounts::active_count_fieldID;
jfieldID downloadedFileCounts::paused_count_fieldID;
jfieldID downloadedFileCounts::completed_count_fieldID;

downloadedFileCounts::downloadedFileCounts()
  : active_count_()
  , paused_count_()
  , completed_count_()
{}

downloadedFileCounts::downloadedFileCounts(int32 active_count_, int32 paused_count_, int32 completed_count_)
  : active_count_(active_count_)
  , paused_count_(paused_count_)
  , completed_count_(completed_count_)
{}

const std::int32_t downloadedFileCounts::ID;

object_ptr<downloadedFileCounts> downloadedFileCounts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<downloadedFileCounts> res = make_object<downloadedFileCounts>();
  res->active_count_ = env->GetIntField(p, res->active_count_fieldID);
  res->paused_count_ = env->GetIntField(p, res->paused_count_fieldID);
  res->completed_count_ = env->GetIntField(p, res->completed_count_fieldID);
  return res;
}

void downloadedFileCounts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, active_count_fieldID, active_count_);
  env->SetIntField(s, paused_count_fieldID, paused_count_);
  env->SetIntField(s, completed_count_fieldID, completed_count_);
}

void downloadedFileCounts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DownloadedFileCounts");
    s.store_field("activeCount", active_count_);
    s.store_field("pausedCount", paused_count_);
    s.store_field("completedCount", completed_count_);
    s.store_class_end();
  }
}

void downloadedFileCounts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DownloadedFileCounts").c_str());
    active_count_fieldID = jni::get_field_id(env, Class, "activeCount", "I");
    paused_count_fieldID = jni::get_field_id(env, Class, "pausedCount", "I");
    completed_count_fieldID = jni::get_field_id(env, Class, "completedCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass draftMessage::Class;
jfieldID draftMessage::reply_to_fieldID;
jfieldID draftMessage::date_fieldID;
jfieldID draftMessage::input_message_text_fieldID;
jfieldID draftMessage::effect_id_fieldID;

draftMessage::draftMessage()
  : reply_to_()
  , date_()
  , input_message_text_()
  , effect_id_()
{}

draftMessage::draftMessage(object_ptr<InputMessageReplyTo> &&reply_to_, int32 date_, object_ptr<InputMessageContent> &&input_message_text_, int64 effect_id_)
  : reply_to_(std::move(reply_to_))
  , date_(date_)
  , input_message_text_(std::move(input_message_text_))
  , effect_id_(effect_id_)
{}

const std::int32_t draftMessage::ID;

object_ptr<draftMessage> draftMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<draftMessage> res = make_object<draftMessage>();
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->input_message_text_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_text_fieldID));
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  return res;
}

void draftMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (reply_to_ != nullptr) { jobject next; reply_to_->store(env, next); if (next) { env->SetObjectField(s, reply_to_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  if (input_message_text_ != nullptr) { jobject next; input_message_text_->store(env, next); if (next) { env->SetObjectField(s, input_message_text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, effect_id_fieldID, effect_id_);
}

void draftMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DraftMessage");
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_field("date", date_);
    s.store_object_field("inputMessageText", static_cast<const BaseObject *>(input_message_text_.get()));
    s.store_field("effectId", effect_id_);
    s.store_class_end();
  }
}

void draftMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DraftMessage").c_str());
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    input_message_text_fieldID = jni::get_field_id(env, Class, "inputMessageText", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass EmailAddressAuthentication::Class;

object_ptr<EmailAddressAuthentication> EmailAddressAuthentication::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case emailAddressAuthenticationCode::ID:
      return emailAddressAuthenticationCode::fetch(env, p);
    case emailAddressAuthenticationAppleId::ID:
      return emailAddressAuthenticationAppleId::fetch(env, p);
    case emailAddressAuthenticationGoogleId::ID:
      return emailAddressAuthenticationGoogleId::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void EmailAddressAuthentication::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressAuthentication").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressAuthenticationCode::Class;
jfieldID emailAddressAuthenticationCode::code_fieldID;

emailAddressAuthenticationCode::emailAddressAuthenticationCode()
  : code_()
{}

emailAddressAuthenticationCode::emailAddressAuthenticationCode(string const &code_)
  : code_(code_)
{}

const std::int32_t emailAddressAuthenticationCode::ID;

object_ptr<EmailAddressAuthentication> emailAddressAuthenticationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressAuthenticationCode> res = make_object<emailAddressAuthenticationCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return std::move(res);
}

void emailAddressAuthenticationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressAuthenticationCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

void emailAddressAuthenticationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressAuthenticationAppleId::Class;
jfieldID emailAddressAuthenticationAppleId::token_fieldID;

emailAddressAuthenticationAppleId::emailAddressAuthenticationAppleId()
  : token_()
{}

emailAddressAuthenticationAppleId::emailAddressAuthenticationAppleId(string const &token_)
  : token_(token_)
{}

const std::int32_t emailAddressAuthenticationAppleId::ID;

object_ptr<EmailAddressAuthentication> emailAddressAuthenticationAppleId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressAuthenticationAppleId> res = make_object<emailAddressAuthenticationAppleId>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void emailAddressAuthenticationAppleId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressAuthenticationAppleId");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void emailAddressAuthenticationAppleId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationAppleId").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressAuthenticationGoogleId::Class;
jfieldID emailAddressAuthenticationGoogleId::token_fieldID;

emailAddressAuthenticationGoogleId::emailAddressAuthenticationGoogleId()
  : token_()
{}

emailAddressAuthenticationGoogleId::emailAddressAuthenticationGoogleId(string const &token_)
  : token_(token_)
{}

const std::int32_t emailAddressAuthenticationGoogleId::ID;

object_ptr<EmailAddressAuthentication> emailAddressAuthenticationGoogleId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressAuthenticationGoogleId> res = make_object<emailAddressAuthenticationGoogleId>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void emailAddressAuthenticationGoogleId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressAuthenticationGoogleId");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void emailAddressAuthenticationGoogleId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationGoogleId").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressAuthenticationCodeInfo::Class;
jfieldID emailAddressAuthenticationCodeInfo::email_address_pattern_fieldID;
jfieldID emailAddressAuthenticationCodeInfo::length_fieldID;

emailAddressAuthenticationCodeInfo::emailAddressAuthenticationCodeInfo()
  : email_address_pattern_()
  , length_()
{}

emailAddressAuthenticationCodeInfo::emailAddressAuthenticationCodeInfo(string const &email_address_pattern_, int32 length_)
  : email_address_pattern_(email_address_pattern_)
  , length_(length_)
{}

const std::int32_t emailAddressAuthenticationCodeInfo::ID;

object_ptr<emailAddressAuthenticationCodeInfo> emailAddressAuthenticationCodeInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressAuthenticationCodeInfo> res = make_object<emailAddressAuthenticationCodeInfo>();
  res->email_address_pattern_ = jni::fetch_string(env, p, res->email_address_pattern_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  return res;
}

void emailAddressAuthenticationCodeInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, email_address_pattern_); if (nextString) { env->SetObjectField(s, email_address_pattern_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, length_fieldID, length_);
}

void emailAddressAuthenticationCodeInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressAuthenticationCodeInfo");
    s.store_field("emailAddressPattern", email_address_pattern_);
    s.store_field("length", length_);
    s.store_class_end();
  }
}

void emailAddressAuthenticationCodeInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationCodeInfo").c_str());
    email_address_pattern_fieldID = jni::get_field_id(env, Class, "emailAddressPattern", "Ljava/lang/String;");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    return true;
  }();
  (void)is_inited;
}

jclass EmailAddressResetState::Class;

object_ptr<EmailAddressResetState> EmailAddressResetState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case emailAddressResetStateAvailable::ID:
      return emailAddressResetStateAvailable::fetch(env, p);
    case emailAddressResetStatePending::ID:
      return emailAddressResetStatePending::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void EmailAddressResetState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressResetState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressResetStateAvailable::Class;
jfieldID emailAddressResetStateAvailable::wait_period_fieldID;

emailAddressResetStateAvailable::emailAddressResetStateAvailable()
  : wait_period_()
{}

emailAddressResetStateAvailable::emailAddressResetStateAvailable(int32 wait_period_)
  : wait_period_(wait_period_)
{}

const std::int32_t emailAddressResetStateAvailable::ID;

object_ptr<EmailAddressResetState> emailAddressResetStateAvailable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressResetStateAvailable> res = make_object<emailAddressResetStateAvailable>();
  res->wait_period_ = env->GetIntField(p, res->wait_period_fieldID);
  return std::move(res);
}

void emailAddressResetStateAvailable::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, wait_period_fieldID, wait_period_);
}

void emailAddressResetStateAvailable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressResetStateAvailable");
    s.store_field("waitPeriod", wait_period_);
    s.store_class_end();
  }
}

void emailAddressResetStateAvailable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressResetStateAvailable").c_str());
    wait_period_fieldID = jni::get_field_id(env, Class, "waitPeriod", "I");
    return true;
  }();
  (void)is_inited;
}

jclass emailAddressResetStatePending::Class;
jfieldID emailAddressResetStatePending::reset_in_fieldID;

emailAddressResetStatePending::emailAddressResetStatePending()
  : reset_in_()
{}

emailAddressResetStatePending::emailAddressResetStatePending(int32 reset_in_)
  : reset_in_(reset_in_)
{}

const std::int32_t emailAddressResetStatePending::ID;

object_ptr<EmailAddressResetState> emailAddressResetStatePending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emailAddressResetStatePending> res = make_object<emailAddressResetStatePending>();
  res->reset_in_ = env->GetIntField(p, res->reset_in_fieldID);
  return std::move(res);
}

void emailAddressResetStatePending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, reset_in_fieldID, reset_in_);
}

void emailAddressResetStatePending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmailAddressResetStatePending");
    s.store_field("resetIn", reset_in_);
    s.store_class_end();
  }
}

void emailAddressResetStatePending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmailAddressResetStatePending").c_str());
    reset_in_fieldID = jni::get_field_id(env, Class, "resetIn", "I");
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategories::Class;
jfieldID emojiCategories::categories_fieldID;

emojiCategories::emojiCategories()
  : categories_()
{}

emojiCategories::emojiCategories(array<object_ptr<emojiCategory>> &&categories_)
  : categories_(std::move(categories_))
{}

const std::int32_t emojiCategories::ID;

object_ptr<emojiCategories> emojiCategories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiCategories> res = make_object<emojiCategories>();
  res->categories_ = jni::FetchVector<emojiCategory>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->categories_fieldID));
  return res;
}

void emojiCategories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, categories_); if (arr_tmp_) { env->SetObjectField(s, categories_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojiCategories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategories");
    { s.store_vector_begin("categories", categories_.size()); for (const auto &_value : categories_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojiCategories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategories").c_str());
    categories_fieldID = jni::get_field_id(env, Class, "categories", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$EmojiCategory;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategory::Class;
jfieldID emojiCategory::name_fieldID;
jfieldID emojiCategory::icon_fieldID;
jfieldID emojiCategory::source_fieldID;
jfieldID emojiCategory::is_greeting_fieldID;

emojiCategory::emojiCategory()
  : name_()
  , icon_()
  , source_()
  , is_greeting_()
{}

emojiCategory::emojiCategory(string const &name_, object_ptr<sticker> &&icon_, object_ptr<EmojiCategorySource> &&source_, bool is_greeting_)
  : name_(name_)
  , icon_(std::move(icon_))
  , source_(std::move(source_))
  , is_greeting_(is_greeting_)
{}

const std::int32_t emojiCategory::ID;

object_ptr<emojiCategory> emojiCategory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiCategory> res = make_object<emojiCategory>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->icon_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->icon_fieldID));
  res->source_ = jni::fetch_tl_object<EmojiCategorySource>(env, jni::fetch_object(env, p, res->source_fieldID));
  res->is_greeting_ = (env->GetBooleanField(p, res->is_greeting_fieldID) != 0);
  return res;
}

void emojiCategory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (icon_ != nullptr) { jobject next; icon_->store(env, next); if (next) { env->SetObjectField(s, icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (source_ != nullptr) { jobject next; source_->store(env, next); if (next) { env->SetObjectField(s, source_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_greeting_fieldID, is_greeting_);
}

void emojiCategory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategory");
    s.store_field("name", name_);
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_field("isGreeting", is_greeting_);
    s.store_class_end();
  }
}

void emojiCategory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategory").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiCategorySource;").c_str());
    is_greeting_fieldID = jni::get_field_id(env, Class, "isGreeting", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass EmojiCategorySource::Class;

object_ptr<EmojiCategorySource> EmojiCategorySource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case emojiCategorySourceSearch::ID:
      return emojiCategorySourceSearch::fetch(env, p);
    case emojiCategorySourcePremium::ID:
      return emojiCategorySourcePremium::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void EmojiCategorySource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategorySource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategorySourceSearch::Class;
jfieldID emojiCategorySourceSearch::emojis_fieldID;

emojiCategorySourceSearch::emojiCategorySourceSearch()
  : emojis_()
{}

emojiCategorySourceSearch::emojiCategorySourceSearch(array<string> &&emojis_)
  : emojis_(std::move(emojis_))
{}

const std::int32_t emojiCategorySourceSearch::ID;

object_ptr<EmojiCategorySource> emojiCategorySourceSearch::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiCategorySourceSearch> res = make_object<emojiCategorySourceSearch>();
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return std::move(res);
}

void emojiCategorySourceSearch::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojiCategorySourceSearch::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategorySourceSearch");
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojiCategorySourceSearch::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategorySourceSearch").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategorySourcePremium::Class;

emojiCategorySourcePremium::emojiCategorySourcePremium() {
}

const std::int32_t emojiCategorySourcePremium::ID;

object_ptr<EmojiCategorySource> emojiCategorySourcePremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<emojiCategorySourcePremium>();
}

void emojiCategorySourcePremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void emojiCategorySourcePremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategorySourcePremium");
    s.store_class_end();
  }
}

void emojiCategorySourcePremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategorySourcePremium").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass EmojiCategoryType::Class;

object_ptr<EmojiCategoryType> EmojiCategoryType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case emojiCategoryTypeDefault::ID:
      return emojiCategoryTypeDefault::fetch(env, p);
    case emojiCategoryTypeRegularStickers::ID:
      return emojiCategoryTypeRegularStickers::fetch(env, p);
    case emojiCategoryTypeEmojiStatus::ID:
      return emojiCategoryTypeEmojiStatus::fetch(env, p);
    case emojiCategoryTypeChatPhoto::ID:
      return emojiCategoryTypeChatPhoto::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void EmojiCategoryType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategoryType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategoryTypeDefault::Class;

emojiCategoryTypeDefault::emojiCategoryTypeDefault() {
}

const std::int32_t emojiCategoryTypeDefault::ID;

object_ptr<EmojiCategoryType> emojiCategoryTypeDefault::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<emojiCategoryTypeDefault>();
}

void emojiCategoryTypeDefault::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategoryTypeDefault");
    s.store_class_end();
  }
}

void emojiCategoryTypeDefault::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategoryTypeDefault").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategoryTypeRegularStickers::Class;

emojiCategoryTypeRegularStickers::emojiCategoryTypeRegularStickers() {
}

const std::int32_t emojiCategoryTypeRegularStickers::ID;

object_ptr<EmojiCategoryType> emojiCategoryTypeRegularStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<emojiCategoryTypeRegularStickers>();
}

void emojiCategoryTypeRegularStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategoryTypeRegularStickers");
    s.store_class_end();
  }
}

void emojiCategoryTypeRegularStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategoryTypeRegularStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategoryTypeEmojiStatus::Class;

emojiCategoryTypeEmojiStatus::emojiCategoryTypeEmojiStatus() {
}

const std::int32_t emojiCategoryTypeEmojiStatus::ID;

object_ptr<EmojiCategoryType> emojiCategoryTypeEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<emojiCategoryTypeEmojiStatus>();
}

void emojiCategoryTypeEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategoryTypeEmojiStatus");
    s.store_class_end();
  }
}

void emojiCategoryTypeEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategoryTypeEmojiStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiCategoryTypeChatPhoto::Class;

emojiCategoryTypeChatPhoto::emojiCategoryTypeChatPhoto() {
}

const std::int32_t emojiCategoryTypeChatPhoto::ID;

object_ptr<EmojiCategoryType> emojiCategoryTypeChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<emojiCategoryTypeChatPhoto>();
}

void emojiCategoryTypeChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiCategoryTypeChatPhoto");
    s.store_class_end();
  }
}

void emojiCategoryTypeChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiCategoryTypeChatPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiKeyword::Class;
jfieldID emojiKeyword::emoji_fieldID;
jfieldID emojiKeyword::keyword_fieldID;

emojiKeyword::emojiKeyword()
  : emoji_()
  , keyword_()
{}

emojiKeyword::emojiKeyword(string const &emoji_, string const &keyword_)
  : emoji_(emoji_)
  , keyword_(keyword_)
{}

const std::int32_t emojiKeyword::ID;

object_ptr<emojiKeyword> emojiKeyword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiKeyword> res = make_object<emojiKeyword>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->keyword_ = jni::fetch_string(env, p, res->keyword_fieldID);
  return res;
}

void emojiKeyword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, keyword_); if (nextString) { env->SetObjectField(s, keyword_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void emojiKeyword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiKeyword");
    s.store_field("emoji", emoji_);
    s.store_field("keyword", keyword_);
    s.store_class_end();
  }
}

void emojiKeyword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiKeyword").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    keyword_fieldID = jni::get_field_id(env, Class, "keyword", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass emojiKeywords::Class;
jfieldID emojiKeywords::emoji_keywords_fieldID;

emojiKeywords::emojiKeywords()
  : emoji_keywords_()
{}

emojiKeywords::emojiKeywords(array<object_ptr<emojiKeyword>> &&emoji_keywords_)
  : emoji_keywords_(std::move(emoji_keywords_))
{}

const std::int32_t emojiKeywords::ID;

object_ptr<emojiKeywords> emojiKeywords::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiKeywords> res = make_object<emojiKeywords>();
  res->emoji_keywords_ = jni::FetchVector<emojiKeyword>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emoji_keywords_fieldID));
  return res;
}

void emojiKeywords::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emoji_keywords_); if (arr_tmp_) { env->SetObjectField(s, emoji_keywords_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojiKeywords::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiKeywords");
    { s.store_vector_begin("emojiKeywords", emoji_keywords_.size()); for (const auto &_value : emoji_keywords_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojiKeywords::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiKeywords").c_str());
    emoji_keywords_fieldID = jni::get_field_id(env, Class, "emojiKeywords", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$EmojiKeyword;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiReaction::Class;
jfieldID emojiReaction::emoji_fieldID;
jfieldID emojiReaction::title_fieldID;
jfieldID emojiReaction::is_active_fieldID;
jfieldID emojiReaction::static_icon_fieldID;
jfieldID emojiReaction::appear_animation_fieldID;
jfieldID emojiReaction::select_animation_fieldID;
jfieldID emojiReaction::activate_animation_fieldID;
jfieldID emojiReaction::effect_animation_fieldID;
jfieldID emojiReaction::around_animation_fieldID;
jfieldID emojiReaction::center_animation_fieldID;

emojiReaction::emojiReaction()
  : emoji_()
  , title_()
  , is_active_()
  , static_icon_()
  , appear_animation_()
  , select_animation_()
  , activate_animation_()
  , effect_animation_()
  , around_animation_()
  , center_animation_()
{}

emojiReaction::emojiReaction(string const &emoji_, string const &title_, bool is_active_, object_ptr<sticker> &&static_icon_, object_ptr<sticker> &&appear_animation_, object_ptr<sticker> &&select_animation_, object_ptr<sticker> &&activate_animation_, object_ptr<sticker> &&effect_animation_, object_ptr<sticker> &&around_animation_, object_ptr<sticker> &&center_animation_)
  : emoji_(emoji_)
  , title_(title_)
  , is_active_(is_active_)
  , static_icon_(std::move(static_icon_))
  , appear_animation_(std::move(appear_animation_))
  , select_animation_(std::move(select_animation_))
  , activate_animation_(std::move(activate_animation_))
  , effect_animation_(std::move(effect_animation_))
  , around_animation_(std::move(around_animation_))
  , center_animation_(std::move(center_animation_))
{}

const std::int32_t emojiReaction::ID;

object_ptr<emojiReaction> emojiReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiReaction> res = make_object<emojiReaction>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  res->static_icon_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->static_icon_fieldID));
  res->appear_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->appear_animation_fieldID));
  res->select_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->select_animation_fieldID));
  res->activate_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->activate_animation_fieldID));
  res->effect_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->effect_animation_fieldID));
  res->around_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->around_animation_fieldID));
  res->center_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->center_animation_fieldID));
  return res;
}

void emojiReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_active_fieldID, is_active_);
  if (static_icon_ != nullptr) { jobject next; static_icon_->store(env, next); if (next) { env->SetObjectField(s, static_icon_fieldID, next); env->DeleteLocalRef(next); } }
  if (appear_animation_ != nullptr) { jobject next; appear_animation_->store(env, next); if (next) { env->SetObjectField(s, appear_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (select_animation_ != nullptr) { jobject next; select_animation_->store(env, next); if (next) { env->SetObjectField(s, select_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (activate_animation_ != nullptr) { jobject next; activate_animation_->store(env, next); if (next) { env->SetObjectField(s, activate_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (effect_animation_ != nullptr) { jobject next; effect_animation_->store(env, next); if (next) { env->SetObjectField(s, effect_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (around_animation_ != nullptr) { jobject next; around_animation_->store(env, next); if (next) { env->SetObjectField(s, around_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (center_animation_ != nullptr) { jobject next; center_animation_->store(env, next); if (next) { env->SetObjectField(s, center_animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void emojiReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiReaction");
    s.store_field("emoji", emoji_);
    s.store_field("title", title_);
    s.store_field("isActive", is_active_);
    s.store_object_field("staticIcon", static_cast<const BaseObject *>(static_icon_.get()));
    s.store_object_field("appearAnimation", static_cast<const BaseObject *>(appear_animation_.get()));
    s.store_object_field("selectAnimation", static_cast<const BaseObject *>(select_animation_.get()));
    s.store_object_field("activateAnimation", static_cast<const BaseObject *>(activate_animation_.get()));
    s.store_object_field("effectAnimation", static_cast<const BaseObject *>(effect_animation_.get()));
    s.store_object_field("aroundAnimation", static_cast<const BaseObject *>(around_animation_.get()));
    s.store_object_field("centerAnimation", static_cast<const BaseObject *>(center_animation_.get()));
    s.store_class_end();
  }
}

void emojiReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiReaction").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    static_icon_fieldID = jni::get_field_id(env, Class, "staticIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    appear_animation_fieldID = jni::get_field_id(env, Class, "appearAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    select_animation_fieldID = jni::get_field_id(env, Class, "selectAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    activate_animation_fieldID = jni::get_field_id(env, Class, "activateAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    effect_animation_fieldID = jni::get_field_id(env, Class, "effectAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    around_animation_fieldID = jni::get_field_id(env, Class, "aroundAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    center_animation_fieldID = jni::get_field_id(env, Class, "centerAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiStatus::Class;
jfieldID emojiStatus::type_fieldID;
jfieldID emojiStatus::expiration_date_fieldID;

emojiStatus::emojiStatus()
  : type_()
  , expiration_date_()
{}

emojiStatus::emojiStatus(object_ptr<EmojiStatusType> &&type_, int32 expiration_date_)
  : type_(std::move(type_))
  , expiration_date_(expiration_date_)
{}

const std::int32_t emojiStatus::ID;

object_ptr<emojiStatus> emojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiStatus> res = make_object<emojiStatus>();
  res->type_ = jni::fetch_tl_object<EmojiStatusType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  return res;
}

void emojiStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
}

void emojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiStatus");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("expirationDate", expiration_date_);
    s.store_class_end();
  }
}

void emojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatus").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatusType;").c_str());
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass emojiStatusCustomEmojis::Class;
jfieldID emojiStatusCustomEmojis::custom_emoji_ids_fieldID;

emojiStatusCustomEmojis::emojiStatusCustomEmojis()
  : custom_emoji_ids_()
{}

emojiStatusCustomEmojis::emojiStatusCustomEmojis(array<int64> &&custom_emoji_ids_)
  : custom_emoji_ids_(std::move(custom_emoji_ids_))
{}

const std::int32_t emojiStatusCustomEmojis::ID;

object_ptr<emojiStatusCustomEmojis> emojiStatusCustomEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiStatusCustomEmojis> res = make_object<emojiStatusCustomEmojis>();
  res->custom_emoji_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->custom_emoji_ids_fieldID));
  return res;
}

void emojiStatusCustomEmojis::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, custom_emoji_ids_); if (arr_tmp_) { env->SetObjectField(s, custom_emoji_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojiStatusCustomEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiStatusCustomEmojis");
    { s.store_vector_begin("customEmojiIds", custom_emoji_ids_.size()); for (const auto &_value : custom_emoji_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojiStatusCustomEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatusCustomEmojis").c_str());
    custom_emoji_ids_fieldID = jni::get_field_id(env, Class, "customEmojiIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass EmojiStatusType::Class;

object_ptr<EmojiStatusType> EmojiStatusType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case emojiStatusTypeCustomEmoji::ID:
      return emojiStatusTypeCustomEmoji::fetch(env, p);
    case emojiStatusTypeUpgradedGift::ID:
      return emojiStatusTypeUpgradedGift::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void EmojiStatusType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatusType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiStatusTypeCustomEmoji::Class;
jfieldID emojiStatusTypeCustomEmoji::custom_emoji_id_fieldID;

emojiStatusTypeCustomEmoji::emojiStatusTypeCustomEmoji()
  : custom_emoji_id_()
{}

emojiStatusTypeCustomEmoji::emojiStatusTypeCustomEmoji(int64 custom_emoji_id_)
  : custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t emojiStatusTypeCustomEmoji::ID;

object_ptr<EmojiStatusType> emojiStatusTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiStatusTypeCustomEmoji> res = make_object<emojiStatusTypeCustomEmoji>();
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return std::move(res);
}

void emojiStatusTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
}

void emojiStatusTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiStatusTypeCustomEmoji");
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

void emojiStatusTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatusTypeCustomEmoji").c_str());
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass emojiStatusTypeUpgradedGift::Class;
jfieldID emojiStatusTypeUpgradedGift::upgraded_gift_id_fieldID;
jfieldID emojiStatusTypeUpgradedGift::gift_title_fieldID;
jfieldID emojiStatusTypeUpgradedGift::gift_name_fieldID;
jfieldID emojiStatusTypeUpgradedGift::model_custom_emoji_id_fieldID;
jfieldID emojiStatusTypeUpgradedGift::symbol_custom_emoji_id_fieldID;
jfieldID emojiStatusTypeUpgradedGift::backdrop_colors_fieldID;

emojiStatusTypeUpgradedGift::emojiStatusTypeUpgradedGift()
  : upgraded_gift_id_()
  , gift_title_()
  , gift_name_()
  , model_custom_emoji_id_()
  , symbol_custom_emoji_id_()
  , backdrop_colors_()
{}

emojiStatusTypeUpgradedGift::emojiStatusTypeUpgradedGift(int64 upgraded_gift_id_, string const &gift_title_, string const &gift_name_, int64 model_custom_emoji_id_, int64 symbol_custom_emoji_id_, object_ptr<upgradedGiftBackdropColors> &&backdrop_colors_)
  : upgraded_gift_id_(upgraded_gift_id_)
  , gift_title_(gift_title_)
  , gift_name_(gift_name_)
  , model_custom_emoji_id_(model_custom_emoji_id_)
  , symbol_custom_emoji_id_(symbol_custom_emoji_id_)
  , backdrop_colors_(std::move(backdrop_colors_))
{}

const std::int32_t emojiStatusTypeUpgradedGift::ID;

object_ptr<EmojiStatusType> emojiStatusTypeUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiStatusTypeUpgradedGift> res = make_object<emojiStatusTypeUpgradedGift>();
  res->upgraded_gift_id_ = env->GetLongField(p, res->upgraded_gift_id_fieldID);
  res->gift_title_ = jni::fetch_string(env, p, res->gift_title_fieldID);
  res->gift_name_ = jni::fetch_string(env, p, res->gift_name_fieldID);
  res->model_custom_emoji_id_ = env->GetLongField(p, res->model_custom_emoji_id_fieldID);
  res->symbol_custom_emoji_id_ = env->GetLongField(p, res->symbol_custom_emoji_id_fieldID);
  res->backdrop_colors_ = jni::fetch_tl_object<upgradedGiftBackdropColors>(env, jni::fetch_object(env, p, res->backdrop_colors_fieldID));
  return std::move(res);
}

void emojiStatusTypeUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, upgraded_gift_id_fieldID, upgraded_gift_id_);
  { jstring nextString = jni::to_jstring(env, gift_title_); if (nextString) { env->SetObjectField(s, gift_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, gift_name_); if (nextString) { env->SetObjectField(s, gift_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, model_custom_emoji_id_fieldID, model_custom_emoji_id_);
  env->SetLongField(s, symbol_custom_emoji_id_fieldID, symbol_custom_emoji_id_);
  if (backdrop_colors_ != nullptr) { jobject next; backdrop_colors_->store(env, next); if (next) { env->SetObjectField(s, backdrop_colors_fieldID, next); env->DeleteLocalRef(next); } }
}

void emojiStatusTypeUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiStatusTypeUpgradedGift");
    s.store_field("upgradedGiftId", upgraded_gift_id_);
    s.store_field("giftTitle", gift_title_);
    s.store_field("giftName", gift_name_);
    s.store_field("modelCustomEmojiId", model_custom_emoji_id_);
    s.store_field("symbolCustomEmojiId", symbol_custom_emoji_id_);
    s.store_object_field("backdropColors", static_cast<const BaseObject *>(backdrop_colors_.get()));
    s.store_class_end();
  }
}

void emojiStatusTypeUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatusTypeUpgradedGift").c_str());
    upgraded_gift_id_fieldID = jni::get_field_id(env, Class, "upgradedGiftId", "J");
    gift_title_fieldID = jni::get_field_id(env, Class, "giftTitle", "Ljava/lang/String;");
    gift_name_fieldID = jni::get_field_id(env, Class, "giftName", "Ljava/lang/String;");
    model_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "modelCustomEmojiId", "J");
    symbol_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "symbolCustomEmojiId", "J");
    backdrop_colors_fieldID = jni::get_field_id(env, Class, "backdropColors", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdropColors;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojiStatuses::Class;
jfieldID emojiStatuses::emoji_statuses_fieldID;

emojiStatuses::emojiStatuses()
  : emoji_statuses_()
{}

emojiStatuses::emojiStatuses(array<object_ptr<emojiStatus>> &&emoji_statuses_)
  : emoji_statuses_(std::move(emoji_statuses_))
{}

const std::int32_t emojiStatuses::ID;

object_ptr<emojiStatuses> emojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojiStatuses> res = make_object<emojiStatuses>();
  res->emoji_statuses_ = jni::FetchVector<emojiStatus>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emoji_statuses_fieldID));
  return res;
}

void emojiStatuses::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emoji_statuses_); if (arr_tmp_) { env->SetObjectField(s, emoji_statuses_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EmojiStatuses");
    { s.store_vector_begin("emojiStatuses", emoji_statuses_.size()); for (const auto &_value : emoji_statuses_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EmojiStatuses").c_str());
    emoji_statuses_fieldID = jni::get_field_id(env, Class, "emojiStatuses", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass emojis::Class;
jfieldID emojis::emojis_fieldID;

emojis::emojis()
  : emojis_()
{}

emojis::emojis(array<string> &&emojis_)
  : emojis_(std::move(emojis_))
{}

const std::int32_t emojis::ID;

object_ptr<emojis> emojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<emojis> res = make_object<emojis>();
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return res;
}

void emojis::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void emojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Emojis");
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void emojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Emojis").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass encryptedCredentials::Class;
jfieldID encryptedCredentials::data_fieldID;
jfieldID encryptedCredentials::hash_fieldID;
jfieldID encryptedCredentials::secret_fieldID;

encryptedCredentials::encryptedCredentials()
  : data_()
  , hash_()
  , secret_()
{}

encryptedCredentials::encryptedCredentials(bytes const &data_, bytes const &hash_, bytes const &secret_)
  : data_(std::move(data_))
  , hash_(std::move(hash_))
  , secret_(std::move(secret_))
{}

const std::int32_t encryptedCredentials::ID;

object_ptr<encryptedCredentials> encryptedCredentials::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<encryptedCredentials> res = make_object<encryptedCredentials>();
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  res->hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->hash_fieldID));
  res->secret_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->secret_fieldID));
  return res;
}

void encryptedCredentials::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jbyteArray nextBytes = jni::to_bytes(env, hash_); if (nextBytes) { env->SetObjectField(s, hash_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jbyteArray nextBytes = jni::to_bytes(env, secret_); if (nextBytes) { env->SetObjectField(s, secret_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void encryptedCredentials::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EncryptedCredentials");
    s.store_bytes_field("data", data_);
    s.store_bytes_field("hash", hash_);
    s.store_bytes_field("secret", secret_);
    s.store_class_end();
  }
}

void encryptedCredentials::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EncryptedCredentials").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    hash_fieldID = jni::get_field_id(env, Class, "hash", "[B");
    secret_fieldID = jni::get_field_id(env, Class, "secret", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass encryptedPassportElement::Class;
jfieldID encryptedPassportElement::type_fieldID;
jfieldID encryptedPassportElement::data_fieldID;
jfieldID encryptedPassportElement::front_side_fieldID;
jfieldID encryptedPassportElement::reverse_side_fieldID;
jfieldID encryptedPassportElement::selfie_fieldID;
jfieldID encryptedPassportElement::translation_fieldID;
jfieldID encryptedPassportElement::files_fieldID;
jfieldID encryptedPassportElement::value_fieldID;
jfieldID encryptedPassportElement::hash_fieldID;

encryptedPassportElement::encryptedPassportElement()
  : type_()
  , data_()
  , front_side_()
  , reverse_side_()
  , selfie_()
  , translation_()
  , files_()
  , value_()
  , hash_()
{}

encryptedPassportElement::encryptedPassportElement(object_ptr<PassportElementType> &&type_, bytes const &data_, object_ptr<datedFile> &&front_side_, object_ptr<datedFile> &&reverse_side_, object_ptr<datedFile> &&selfie_, array<object_ptr<datedFile>> &&translation_, array<object_ptr<datedFile>> &&files_, string const &value_, string const &hash_)
  : type_(std::move(type_))
  , data_(std::move(data_))
  , front_side_(std::move(front_side_))
  , reverse_side_(std::move(reverse_side_))
  , selfie_(std::move(selfie_))
  , translation_(std::move(translation_))
  , files_(std::move(files_))
  , value_(value_)
  , hash_(hash_)
{}

const std::int32_t encryptedPassportElement::ID;

object_ptr<encryptedPassportElement> encryptedPassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<encryptedPassportElement> res = make_object<encryptedPassportElement>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  res->front_side_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->front_side_fieldID));
  res->reverse_side_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->reverse_side_fieldID));
  res->selfie_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->selfie_fieldID));
  res->translation_ = jni::FetchVector<datedFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->translation_fieldID));
  res->files_ = jni::FetchVector<datedFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->files_fieldID));
  res->value_ = jni::fetch_string(env, p, res->value_fieldID);
  res->hash_ = jni::fetch_string(env, p, res->hash_fieldID);
  return res;
}

void encryptedPassportElement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  if (front_side_ != nullptr) { jobject next; front_side_->store(env, next); if (next) { env->SetObjectField(s, front_side_fieldID, next); env->DeleteLocalRef(next); } }
  if (reverse_side_ != nullptr) { jobject next; reverse_side_->store(env, next); if (next) { env->SetObjectField(s, reverse_side_fieldID, next); env->DeleteLocalRef(next); } }
  if (selfie_ != nullptr) { jobject next; selfie_->store(env, next); if (next) { env->SetObjectField(s, selfie_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, translation_); if (arr_tmp_) { env->SetObjectField(s, translation_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, files_); if (arr_tmp_) { env->SetObjectField(s, files_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, value_); if (nextString) { env->SetObjectField(s, value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, hash_); if (nextString) { env->SetObjectField(s, hash_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void encryptedPassportElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EncryptedPassportElement");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_bytes_field("data", data_);
    s.store_object_field("frontSide", static_cast<const BaseObject *>(front_side_.get()));
    s.store_object_field("reverseSide", static_cast<const BaseObject *>(reverse_side_.get()));
    s.store_object_field("selfie", static_cast<const BaseObject *>(selfie_.get()));
    { s.store_vector_begin("translation", translation_.size()); for (const auto &_value : translation_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("files", files_.size()); for (const auto &_value : files_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("value", value_);
    s.store_field("hash", hash_);
    s.store_class_end();
  }
}

void encryptedPassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EncryptedPassportElement").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    front_side_fieldID = jni::get_field_id(env, Class, "frontSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    reverse_side_fieldID = jni::get_field_id(env, Class, "reverseSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    selfie_fieldID = jni::get_field_id(env, Class, "selfie", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    translation_fieldID = jni::get_field_id(env, Class, "translation", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    files_fieldID = jni::get_field_id(env, Class, "files", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Ljava/lang/String;");
    hash_fieldID = jni::get_field_id(env, Class, "hash", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass error::Class;
jfieldID error::code_fieldID;
jfieldID error::message_fieldID;

error::error()
  : code_()
  , message_()
{}

error::error(int32 code_, string const &message_)
  : code_(code_)
  , message_(message_)
{}

const std::int32_t error::ID;

object_ptr<error> error::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<error> res = make_object<error>();
  res->code_ = env->GetIntField(p, res->code_fieldID);
  res->message_ = jni::fetch_string(env, p, res->message_fieldID);
  return res;
}

void error::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, code_fieldID, code_);
  { jstring nextString = jni::to_jstring(env, message_); if (nextString) { env->SetObjectField(s, message_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void error::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Error");
    s.store_field("code", code_);
    s.store_field("message", message_);
    s.store_class_end();
  }
}

void error::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Error").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "I");
    message_fieldID = jni::get_field_id(env, Class, "message", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass factCheck::Class;
jfieldID factCheck::text_fieldID;
jfieldID factCheck::country_code_fieldID;

factCheck::factCheck()
  : text_()
  , country_code_()
{}

factCheck::factCheck(object_ptr<formattedText> &&text_, string const &country_code_)
  : text_(std::move(text_))
  , country_code_(country_code_)
{}

const std::int32_t factCheck::ID;

object_ptr<factCheck> factCheck::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<factCheck> res = make_object<factCheck>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  return res;
}

void factCheck::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, country_code_); if (nextString) { env->SetObjectField(s, country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void factCheck::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FactCheck");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("countryCode", country_code_);
    s.store_class_end();
  }
}

void factCheck::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FactCheck").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass failedToAddMember::Class;
jfieldID failedToAddMember::user_id_fieldID;
jfieldID failedToAddMember::premium_would_allow_invite_fieldID;
jfieldID failedToAddMember::premium_required_to_send_messages_fieldID;

failedToAddMember::failedToAddMember()
  : user_id_()
  , premium_would_allow_invite_()
  , premium_required_to_send_messages_()
{}

failedToAddMember::failedToAddMember(int53 user_id_, bool premium_would_allow_invite_, bool premium_required_to_send_messages_)
  : user_id_(user_id_)
  , premium_would_allow_invite_(premium_would_allow_invite_)
  , premium_required_to_send_messages_(premium_required_to_send_messages_)
{}

const std::int32_t failedToAddMember::ID;

object_ptr<failedToAddMember> failedToAddMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<failedToAddMember> res = make_object<failedToAddMember>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->premium_would_allow_invite_ = (env->GetBooleanField(p, res->premium_would_allow_invite_fieldID) != 0);
  res->premium_required_to_send_messages_ = (env->GetBooleanField(p, res->premium_required_to_send_messages_fieldID) != 0);
  return res;
}

void failedToAddMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetBooleanField(s, premium_would_allow_invite_fieldID, premium_would_allow_invite_);
  env->SetBooleanField(s, premium_required_to_send_messages_fieldID, premium_required_to_send_messages_);
}

void failedToAddMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FailedToAddMember");
    s.store_field("userId", user_id_);
    s.store_field("premiumWouldAllowInvite", premium_would_allow_invite_);
    s.store_field("premiumRequiredToSendMessages", premium_required_to_send_messages_);
    s.store_class_end();
  }
}

void failedToAddMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FailedToAddMember").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    premium_would_allow_invite_fieldID = jni::get_field_id(env, Class, "premiumWouldAllowInvite", "Z");
    premium_required_to_send_messages_fieldID = jni::get_field_id(env, Class, "premiumRequiredToSendMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass failedToAddMembers::Class;
jfieldID failedToAddMembers::failed_to_add_members_fieldID;

failedToAddMembers::failedToAddMembers()
  : failed_to_add_members_()
{}

failedToAddMembers::failedToAddMembers(array<object_ptr<failedToAddMember>> &&failed_to_add_members_)
  : failed_to_add_members_(std::move(failed_to_add_members_))
{}

const std::int32_t failedToAddMembers::ID;

object_ptr<failedToAddMembers> failedToAddMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<failedToAddMembers> res = make_object<failedToAddMembers>();
  res->failed_to_add_members_ = jni::FetchVector<failedToAddMember>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->failed_to_add_members_fieldID));
  return res;
}

void failedToAddMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, failed_to_add_members_); if (arr_tmp_) { env->SetObjectField(s, failed_to_add_members_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void failedToAddMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FailedToAddMembers");
    { s.store_vector_begin("failedToAddMembers", failed_to_add_members_.size()); for (const auto &_value : failed_to_add_members_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void failedToAddMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FailedToAddMembers").c_str());
    failed_to_add_members_fieldID = jni::get_field_id(env, Class, "failedToAddMembers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$FailedToAddMember;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass file::Class;
jfieldID file::id_fieldID;
jfieldID file::size_fieldID;
jfieldID file::expected_size_fieldID;
jfieldID file::local_fieldID;
jfieldID file::remote_fieldID;

file::file()
  : id_()
  , size_()
  , expected_size_()
  , local_()
  , remote_()
{}

file::file(int32 id_, int53 size_, int53 expected_size_, object_ptr<localFile> &&local_, object_ptr<remoteFile> &&remote_)
  : id_(id_)
  , size_(size_)
  , expected_size_(expected_size_)
  , local_(std::move(local_))
  , remote_(std::move(remote_))
{}

const std::int32_t file::ID;

object_ptr<file> file::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<file> res = make_object<file>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->size_ = env->GetLongField(p, res->size_fieldID);
  res->expected_size_ = env->GetLongField(p, res->expected_size_fieldID);
  res->local_ = jni::fetch_tl_object<localFile>(env, jni::fetch_object(env, p, res->local_fieldID));
  res->remote_ = jni::fetch_tl_object<remoteFile>(env, jni::fetch_object(env, p, res->remote_fieldID));
  return res;
}

void file::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetLongField(s, size_fieldID, size_);
  env->SetLongField(s, expected_size_fieldID, expected_size_);
  if (local_ != nullptr) { jobject next; local_->store(env, next); if (next) { env->SetObjectField(s, local_fieldID, next); env->DeleteLocalRef(next); } }
  if (remote_ != nullptr) { jobject next; remote_->store(env, next); if (next) { env->SetObjectField(s, remote_fieldID, next); env->DeleteLocalRef(next); } }
}

void file::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "File");
    s.store_field("id", id_);
    s.store_field("size", size_);
    s.store_field("expectedSize", expected_size_);
    s.store_object_field("local", static_cast<const BaseObject *>(local_.get()));
    s.store_object_field("remote", static_cast<const BaseObject *>(remote_.get()));
    s.store_class_end();
  }
}

void file::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$File").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    expected_size_fieldID = jni::get_field_id(env, Class, "expectedSize", "J");
    local_fieldID = jni::get_field_id(env, Class, "local", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LocalFile;").c_str());
    remote_fieldID = jni::get_field_id(env, Class, "remote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RemoteFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileDownload::Class;
jfieldID fileDownload::file_id_fieldID;
jfieldID fileDownload::message_fieldID;
jfieldID fileDownload::add_date_fieldID;
jfieldID fileDownload::complete_date_fieldID;
jfieldID fileDownload::is_paused_fieldID;

fileDownload::fileDownload()
  : file_id_()
  , message_()
  , add_date_()
  , complete_date_()
  , is_paused_()
{}

fileDownload::fileDownload(int32 file_id_, object_ptr<message> &&message_, int32 add_date_, int32 complete_date_, bool is_paused_)
  : file_id_(file_id_)
  , message_(std::move(message_))
  , add_date_(add_date_)
  , complete_date_(complete_date_)
  , is_paused_(is_paused_)
{}

const std::int32_t fileDownload::ID;

object_ptr<fileDownload> fileDownload::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<fileDownload> res = make_object<fileDownload>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->add_date_ = env->GetIntField(p, res->add_date_fieldID);
  res->complete_date_ = env->GetIntField(p, res->complete_date_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  return res;
}

void fileDownload::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, file_id_fieldID, file_id_);
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, add_date_fieldID, add_date_);
  env->SetIntField(s, complete_date_fieldID, complete_date_);
  env->SetBooleanField(s, is_paused_fieldID, is_paused_);
}

void fileDownload::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileDownload");
    s.store_field("fileId", file_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("addDate", add_date_);
    s.store_field("completeDate", complete_date_);
    s.store_field("isPaused", is_paused_);
    s.store_class_end();
  }
}

void fileDownload::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileDownload").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    add_date_fieldID = jni::get_field_id(env, Class, "addDate", "I");
    complete_date_fieldID = jni::get_field_id(env, Class, "completeDate", "I");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass fileDownloadedPrefixSize::Class;
jfieldID fileDownloadedPrefixSize::size_fieldID;

fileDownloadedPrefixSize::fileDownloadedPrefixSize()
  : size_()
{}

fileDownloadedPrefixSize::fileDownloadedPrefixSize(int53 size_)
  : size_(size_)
{}

const std::int32_t fileDownloadedPrefixSize::ID;

object_ptr<fileDownloadedPrefixSize> fileDownloadedPrefixSize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<fileDownloadedPrefixSize> res = make_object<fileDownloadedPrefixSize>();
  res->size_ = env->GetLongField(p, res->size_fieldID);
  return res;
}

void fileDownloadedPrefixSize::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, size_fieldID, size_);
}

void fileDownloadedPrefixSize::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileDownloadedPrefixSize");
    s.store_field("size", size_);
    s.store_class_end();
  }
}

void fileDownloadedPrefixSize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileDownloadedPrefixSize").c_str());
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    return true;
  }();
  (void)is_inited;
}

jclass FileType::Class;

object_ptr<FileType> FileType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case fileTypeNone::ID:
      return fileTypeNone::fetch(env, p);
    case fileTypeAnimation::ID:
      return fileTypeAnimation::fetch(env, p);
    case fileTypeAudio::ID:
      return fileTypeAudio::fetch(env, p);
    case fileTypeDocument::ID:
      return fileTypeDocument::fetch(env, p);
    case fileTypeNotificationSound::ID:
      return fileTypeNotificationSound::fetch(env, p);
    case fileTypePhoto::ID:
      return fileTypePhoto::fetch(env, p);
    case fileTypePhotoStory::ID:
      return fileTypePhotoStory::fetch(env, p);
    case fileTypeProfilePhoto::ID:
      return fileTypeProfilePhoto::fetch(env, p);
    case fileTypeSecret::ID:
      return fileTypeSecret::fetch(env, p);
    case fileTypeSecretThumbnail::ID:
      return fileTypeSecretThumbnail::fetch(env, p);
    case fileTypeSecure::ID:
      return fileTypeSecure::fetch(env, p);
    case fileTypeSelfDestructingPhoto::ID:
      return fileTypeSelfDestructingPhoto::fetch(env, p);
    case fileTypeSelfDestructingVideo::ID:
      return fileTypeSelfDestructingVideo::fetch(env, p);
    case fileTypeSelfDestructingVideoNote::ID:
      return fileTypeSelfDestructingVideoNote::fetch(env, p);
    case fileTypeSelfDestructingVoiceNote::ID:
      return fileTypeSelfDestructingVoiceNote::fetch(env, p);
    case fileTypeSticker::ID:
      return fileTypeSticker::fetch(env, p);
    case fileTypeThumbnail::ID:
      return fileTypeThumbnail::fetch(env, p);
    case fileTypeUnknown::ID:
      return fileTypeUnknown::fetch(env, p);
    case fileTypeVideo::ID:
      return fileTypeVideo::fetch(env, p);
    case fileTypeVideoNote::ID:
      return fileTypeVideoNote::fetch(env, p);
    case fileTypeVideoStory::ID:
      return fileTypeVideoStory::fetch(env, p);
    case fileTypeVoiceNote::ID:
      return fileTypeVoiceNote::fetch(env, p);
    case fileTypeWallpaper::ID:
      return fileTypeWallpaper::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void FileType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeNone::Class;

fileTypeNone::fileTypeNone() {
}

const std::int32_t fileTypeNone::ID;

object_ptr<FileType> fileTypeNone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeNone>();
}

void fileTypeNone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeNone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeNone");
    s.store_class_end();
  }
}

void fileTypeNone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeNone").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeAnimation::Class;

fileTypeAnimation::fileTypeAnimation() {
}

const std::int32_t fileTypeAnimation::ID;

object_ptr<FileType> fileTypeAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeAnimation>();
}

void fileTypeAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeAnimation");
    s.store_class_end();
  }
}

void fileTypeAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeAnimation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeAudio::Class;

fileTypeAudio::fileTypeAudio() {
}

const std::int32_t fileTypeAudio::ID;

object_ptr<FileType> fileTypeAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeAudio>();
}

void fileTypeAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeAudio");
    s.store_class_end();
  }
}

void fileTypeAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeAudio").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeDocument::Class;

fileTypeDocument::fileTypeDocument() {
}

const std::int32_t fileTypeDocument::ID;

object_ptr<FileType> fileTypeDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeDocument>();
}

void fileTypeDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeDocument");
    s.store_class_end();
  }
}

void fileTypeDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeDocument").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeNotificationSound::Class;

fileTypeNotificationSound::fileTypeNotificationSound() {
}

const std::int32_t fileTypeNotificationSound::ID;

object_ptr<FileType> fileTypeNotificationSound::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeNotificationSound>();
}

void fileTypeNotificationSound::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeNotificationSound::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeNotificationSound");
    s.store_class_end();
  }
}

void fileTypeNotificationSound::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeNotificationSound").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypePhoto::Class;

fileTypePhoto::fileTypePhoto() {
}

const std::int32_t fileTypePhoto::ID;

object_ptr<FileType> fileTypePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypePhoto>();
}

void fileTypePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypePhoto");
    s.store_class_end();
  }
}

void fileTypePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypePhotoStory::Class;

fileTypePhotoStory::fileTypePhotoStory() {
}

const std::int32_t fileTypePhotoStory::ID;

object_ptr<FileType> fileTypePhotoStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypePhotoStory>();
}

void fileTypePhotoStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypePhotoStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypePhotoStory");
    s.store_class_end();
  }
}

void fileTypePhotoStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypePhotoStory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeProfilePhoto::Class;

fileTypeProfilePhoto::fileTypeProfilePhoto() {
}

const std::int32_t fileTypeProfilePhoto::ID;

object_ptr<FileType> fileTypeProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeProfilePhoto>();
}

void fileTypeProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeProfilePhoto");
    s.store_class_end();
  }
}

void fileTypeProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeProfilePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSecret::Class;

fileTypeSecret::fileTypeSecret() {
}

const std::int32_t fileTypeSecret::ID;

object_ptr<FileType> fileTypeSecret::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSecret>();
}

void fileTypeSecret::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSecret::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSecret");
    s.store_class_end();
  }
}

void fileTypeSecret::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSecret").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSecretThumbnail::Class;

fileTypeSecretThumbnail::fileTypeSecretThumbnail() {
}

const std::int32_t fileTypeSecretThumbnail::ID;

object_ptr<FileType> fileTypeSecretThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSecretThumbnail>();
}

void fileTypeSecretThumbnail::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSecretThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSecretThumbnail");
    s.store_class_end();
  }
}

void fileTypeSecretThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSecretThumbnail").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSecure::Class;

fileTypeSecure::fileTypeSecure() {
}

const std::int32_t fileTypeSecure::ID;

object_ptr<FileType> fileTypeSecure::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSecure>();
}

void fileTypeSecure::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSecure::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSecure");
    s.store_class_end();
  }
}

void fileTypeSecure::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSecure").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSelfDestructingPhoto::Class;

fileTypeSelfDestructingPhoto::fileTypeSelfDestructingPhoto() {
}

const std::int32_t fileTypeSelfDestructingPhoto::ID;

object_ptr<FileType> fileTypeSelfDestructingPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSelfDestructingPhoto>();
}

void fileTypeSelfDestructingPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSelfDestructingPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSelfDestructingPhoto");
    s.store_class_end();
  }
}

void fileTypeSelfDestructingPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSelfDestructingPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSelfDestructingVideo::Class;

fileTypeSelfDestructingVideo::fileTypeSelfDestructingVideo() {
}

const std::int32_t fileTypeSelfDestructingVideo::ID;

object_ptr<FileType> fileTypeSelfDestructingVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSelfDestructingVideo>();
}

void fileTypeSelfDestructingVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSelfDestructingVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSelfDestructingVideo");
    s.store_class_end();
  }
}

void fileTypeSelfDestructingVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSelfDestructingVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSelfDestructingVideoNote::Class;

fileTypeSelfDestructingVideoNote::fileTypeSelfDestructingVideoNote() {
}

const std::int32_t fileTypeSelfDestructingVideoNote::ID;

object_ptr<FileType> fileTypeSelfDestructingVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSelfDestructingVideoNote>();
}

void fileTypeSelfDestructingVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSelfDestructingVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSelfDestructingVideoNote");
    s.store_class_end();
  }
}

void fileTypeSelfDestructingVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSelfDestructingVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSelfDestructingVoiceNote::Class;

fileTypeSelfDestructingVoiceNote::fileTypeSelfDestructingVoiceNote() {
}

const std::int32_t fileTypeSelfDestructingVoiceNote::ID;

object_ptr<FileType> fileTypeSelfDestructingVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSelfDestructingVoiceNote>();
}

void fileTypeSelfDestructingVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSelfDestructingVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSelfDestructingVoiceNote");
    s.store_class_end();
  }
}

void fileTypeSelfDestructingVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSelfDestructingVoiceNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeSticker::Class;

fileTypeSticker::fileTypeSticker() {
}

const std::int32_t fileTypeSticker::ID;

object_ptr<FileType> fileTypeSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeSticker>();
}

void fileTypeSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeSticker");
    s.store_class_end();
  }
}

void fileTypeSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeSticker").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeThumbnail::Class;

fileTypeThumbnail::fileTypeThumbnail() {
}

const std::int32_t fileTypeThumbnail::ID;

object_ptr<FileType> fileTypeThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeThumbnail>();
}

void fileTypeThumbnail::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeThumbnail");
    s.store_class_end();
  }
}

void fileTypeThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeThumbnail").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeUnknown::Class;

fileTypeUnknown::fileTypeUnknown() {
}

const std::int32_t fileTypeUnknown::ID;

object_ptr<FileType> fileTypeUnknown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeUnknown>();
}

void fileTypeUnknown::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeUnknown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeUnknown");
    s.store_class_end();
  }
}

void fileTypeUnknown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeUnknown").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeVideo::Class;

fileTypeVideo::fileTypeVideo() {
}

const std::int32_t fileTypeVideo::ID;

object_ptr<FileType> fileTypeVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeVideo>();
}

void fileTypeVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeVideo");
    s.store_class_end();
  }
}

void fileTypeVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeVideoNote::Class;

fileTypeVideoNote::fileTypeVideoNote() {
}

const std::int32_t fileTypeVideoNote::ID;

object_ptr<FileType> fileTypeVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeVideoNote>();
}

void fileTypeVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeVideoNote");
    s.store_class_end();
  }
}

void fileTypeVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeVideoStory::Class;

fileTypeVideoStory::fileTypeVideoStory() {
}

const std::int32_t fileTypeVideoStory::ID;

object_ptr<FileType> fileTypeVideoStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeVideoStory>();
}

void fileTypeVideoStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeVideoStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeVideoStory");
    s.store_class_end();
  }
}

void fileTypeVideoStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeVideoStory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeVoiceNote::Class;

fileTypeVoiceNote::fileTypeVoiceNote() {
}

const std::int32_t fileTypeVoiceNote::ID;

object_ptr<FileType> fileTypeVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeVoiceNote>();
}

void fileTypeVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeVoiceNote");
    s.store_class_end();
  }
}

void fileTypeVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeVoiceNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass fileTypeWallpaper::Class;

fileTypeWallpaper::fileTypeWallpaper() {
}

const std::int32_t fileTypeWallpaper::ID;

object_ptr<FileType> fileTypeWallpaper::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<fileTypeWallpaper>();
}

void fileTypeWallpaper::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void fileTypeWallpaper::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FileTypeWallpaper");
    s.store_class_end();
  }
}

void fileTypeWallpaper::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FileTypeWallpaper").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass FirebaseAuthenticationSettings::Class;

object_ptr<FirebaseAuthenticationSettings> FirebaseAuthenticationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case firebaseAuthenticationSettingsAndroid::ID:
      return firebaseAuthenticationSettingsAndroid::fetch(env, p);
    case firebaseAuthenticationSettingsIos::ID:
      return firebaseAuthenticationSettingsIos::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void FirebaseAuthenticationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseAuthenticationSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass firebaseAuthenticationSettingsAndroid::Class;

firebaseAuthenticationSettingsAndroid::firebaseAuthenticationSettingsAndroid() {
}

const std::int32_t firebaseAuthenticationSettingsAndroid::ID;

object_ptr<FirebaseAuthenticationSettings> firebaseAuthenticationSettingsAndroid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<firebaseAuthenticationSettingsAndroid>();
}

void firebaseAuthenticationSettingsAndroid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FirebaseAuthenticationSettingsAndroid");
    s.store_class_end();
  }
}

void firebaseAuthenticationSettingsAndroid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseAuthenticationSettingsAndroid").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass firebaseAuthenticationSettingsIos::Class;
jfieldID firebaseAuthenticationSettingsIos::device_token_fieldID;
jfieldID firebaseAuthenticationSettingsIos::is_app_sandbox_fieldID;

firebaseAuthenticationSettingsIos::firebaseAuthenticationSettingsIos()
  : device_token_()
  , is_app_sandbox_()
{}

firebaseAuthenticationSettingsIos::firebaseAuthenticationSettingsIos(string const &device_token_, bool is_app_sandbox_)
  : device_token_(device_token_)
  , is_app_sandbox_(is_app_sandbox_)
{}

const std::int32_t firebaseAuthenticationSettingsIos::ID;

object_ptr<FirebaseAuthenticationSettings> firebaseAuthenticationSettingsIos::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<firebaseAuthenticationSettingsIos> res = make_object<firebaseAuthenticationSettingsIos>();
  res->device_token_ = jni::fetch_string(env, p, res->device_token_fieldID);
  res->is_app_sandbox_ = (env->GetBooleanField(p, res->is_app_sandbox_fieldID) != 0);
  return std::move(res);
}

void firebaseAuthenticationSettingsIos::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FirebaseAuthenticationSettingsIos");
    s.store_field("deviceToken", device_token_);
    s.store_field("isAppSandbox", is_app_sandbox_);
    s.store_class_end();
  }
}

void firebaseAuthenticationSettingsIos::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseAuthenticationSettingsIos").c_str());
    device_token_fieldID = jni::get_field_id(env, Class, "deviceToken", "Ljava/lang/String;");
    is_app_sandbox_fieldID = jni::get_field_id(env, Class, "isAppSandbox", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass FirebaseDeviceVerificationParameters::Class;

object_ptr<FirebaseDeviceVerificationParameters> FirebaseDeviceVerificationParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case firebaseDeviceVerificationParametersSafetyNet::ID:
      return firebaseDeviceVerificationParametersSafetyNet::fetch(env, p);
    case firebaseDeviceVerificationParametersPlayIntegrity::ID:
      return firebaseDeviceVerificationParametersPlayIntegrity::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void FirebaseDeviceVerificationParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseDeviceVerificationParameters").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass firebaseDeviceVerificationParametersSafetyNet::Class;
jfieldID firebaseDeviceVerificationParametersSafetyNet::nonce_fieldID;

firebaseDeviceVerificationParametersSafetyNet::firebaseDeviceVerificationParametersSafetyNet()
  : nonce_()
{}

firebaseDeviceVerificationParametersSafetyNet::firebaseDeviceVerificationParametersSafetyNet(bytes const &nonce_)
  : nonce_(std::move(nonce_))
{}

const std::int32_t firebaseDeviceVerificationParametersSafetyNet::ID;

object_ptr<FirebaseDeviceVerificationParameters> firebaseDeviceVerificationParametersSafetyNet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<firebaseDeviceVerificationParametersSafetyNet> res = make_object<firebaseDeviceVerificationParametersSafetyNet>();
  res->nonce_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->nonce_fieldID));
  return std::move(res);
}

void firebaseDeviceVerificationParametersSafetyNet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, nonce_); if (nextBytes) { env->SetObjectField(s, nonce_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void firebaseDeviceVerificationParametersSafetyNet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FirebaseDeviceVerificationParametersSafetyNet");
    s.store_bytes_field("nonce", nonce_);
    s.store_class_end();
  }
}

void firebaseDeviceVerificationParametersSafetyNet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseDeviceVerificationParametersSafetyNet").c_str());
    nonce_fieldID = jni::get_field_id(env, Class, "nonce", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass firebaseDeviceVerificationParametersPlayIntegrity::Class;
jfieldID firebaseDeviceVerificationParametersPlayIntegrity::nonce_fieldID;
jfieldID firebaseDeviceVerificationParametersPlayIntegrity::cloud_project_number_fieldID;

firebaseDeviceVerificationParametersPlayIntegrity::firebaseDeviceVerificationParametersPlayIntegrity()
  : nonce_()
  , cloud_project_number_()
{}

firebaseDeviceVerificationParametersPlayIntegrity::firebaseDeviceVerificationParametersPlayIntegrity(string const &nonce_, int64 cloud_project_number_)
  : nonce_(nonce_)
  , cloud_project_number_(cloud_project_number_)
{}

const std::int32_t firebaseDeviceVerificationParametersPlayIntegrity::ID;

object_ptr<FirebaseDeviceVerificationParameters> firebaseDeviceVerificationParametersPlayIntegrity::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<firebaseDeviceVerificationParametersPlayIntegrity> res = make_object<firebaseDeviceVerificationParametersPlayIntegrity>();
  res->nonce_ = jni::fetch_string(env, p, res->nonce_fieldID);
  res->cloud_project_number_ = env->GetLongField(p, res->cloud_project_number_fieldID);
  return std::move(res);
}

void firebaseDeviceVerificationParametersPlayIntegrity::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, nonce_); if (nextString) { env->SetObjectField(s, nonce_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cloud_project_number_fieldID, cloud_project_number_);
}

void firebaseDeviceVerificationParametersPlayIntegrity::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FirebaseDeviceVerificationParametersPlayIntegrity");
    s.store_field("nonce", nonce_);
    s.store_field("cloudProjectNumber", cloud_project_number_);
    s.store_class_end();
  }
}

void firebaseDeviceVerificationParametersPlayIntegrity::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FirebaseDeviceVerificationParametersPlayIntegrity").c_str());
    nonce_fieldID = jni::get_field_id(env, Class, "nonce", "Ljava/lang/String;");
    cloud_project_number_fieldID = jni::get_field_id(env, Class, "cloudProjectNumber", "J");
    return true;
  }();
  (void)is_inited;
}

jclass formattedText::Class;
jfieldID formattedText::text_fieldID;
jfieldID formattedText::entities_fieldID;

formattedText::formattedText()
  : text_()
  , entities_()
{}

formattedText::formattedText(string const &text_, array<object_ptr<textEntity>> &&entities_)
  : text_(text_)
  , entities_(std::move(entities_))
{}

const std::int32_t formattedText::ID;

object_ptr<formattedText> formattedText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<formattedText> res = make_object<formattedText>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->entities_ = jni::FetchVector<textEntity>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->entities_fieldID));
  return res;
}

void formattedText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, entities_); if (arr_tmp_) { env->SetObjectField(s, entities_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void formattedText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FormattedText");
    s.store_field("text", text_);
    { s.store_vector_begin("entities", entities_.size()); for (const auto &_value : entities_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void formattedText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FormattedText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    entities_fieldID = jni::get_field_id(env, Class, "entities", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TextEntity;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass forumTopic::Class;
jfieldID forumTopic::info_fieldID;
jfieldID forumTopic::last_message_fieldID;
jfieldID forumTopic::order_fieldID;
jfieldID forumTopic::is_pinned_fieldID;
jfieldID forumTopic::unread_count_fieldID;
jfieldID forumTopic::last_read_inbox_message_id_fieldID;
jfieldID forumTopic::last_read_outbox_message_id_fieldID;
jfieldID forumTopic::unread_mention_count_fieldID;
jfieldID forumTopic::unread_reaction_count_fieldID;
jfieldID forumTopic::notification_settings_fieldID;
jfieldID forumTopic::draft_message_fieldID;

forumTopic::forumTopic()
  : info_()
  , last_message_()
  , order_()
  , is_pinned_()
  , unread_count_()
  , last_read_inbox_message_id_()
  , last_read_outbox_message_id_()
  , unread_mention_count_()
  , unread_reaction_count_()
  , notification_settings_()
  , draft_message_()
{}

forumTopic::forumTopic(object_ptr<forumTopicInfo> &&info_, object_ptr<message> &&last_message_, int64 order_, bool is_pinned_, int32 unread_count_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int32 unread_mention_count_, int32 unread_reaction_count_, object_ptr<chatNotificationSettings> &&notification_settings_, object_ptr<draftMessage> &&draft_message_)
  : info_(std::move(info_))
  , last_message_(std::move(last_message_))
  , order_(order_)
  , is_pinned_(is_pinned_)
  , unread_count_(unread_count_)
  , last_read_inbox_message_id_(last_read_inbox_message_id_)
  , last_read_outbox_message_id_(last_read_outbox_message_id_)
  , unread_mention_count_(unread_mention_count_)
  , unread_reaction_count_(unread_reaction_count_)
  , notification_settings_(std::move(notification_settings_))
  , draft_message_(std::move(draft_message_))
{}

const std::int32_t forumTopic::ID;

object_ptr<forumTopic> forumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forumTopic> res = make_object<forumTopic>();
  res->info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  res->last_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->last_message_fieldID));
  res->order_ = env->GetLongField(p, res->order_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->unread_count_ = env->GetIntField(p, res->unread_count_fieldID);
  res->last_read_inbox_message_id_ = env->GetLongField(p, res->last_read_inbox_message_id_fieldID);
  res->last_read_outbox_message_id_ = env->GetLongField(p, res->last_read_outbox_message_id_fieldID);
  res->unread_mention_count_ = env->GetIntField(p, res->unread_mention_count_fieldID);
  res->unread_reaction_count_ = env->GetIntField(p, res->unread_reaction_count_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  return res;
}

void forumTopic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (info_ != nullptr) { jobject next; info_->store(env, next); if (next) { env->SetObjectField(s, info_fieldID, next); env->DeleteLocalRef(next); } }
  if (last_message_ != nullptr) { jobject next; last_message_->store(env, next); if (next) { env->SetObjectField(s, last_message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, order_fieldID, order_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  env->SetIntField(s, unread_count_fieldID, unread_count_);
  env->SetLongField(s, last_read_inbox_message_id_fieldID, last_read_inbox_message_id_);
  env->SetLongField(s, last_read_outbox_message_id_fieldID, last_read_outbox_message_id_);
  env->SetIntField(s, unread_mention_count_fieldID, unread_mention_count_);
  env->SetIntField(s, unread_reaction_count_fieldID, unread_reaction_count_);
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (draft_message_ != nullptr) { jobject next; draft_message_->store(env, next); if (next) { env->SetObjectField(s, draft_message_fieldID, next); env->DeleteLocalRef(next); } }
}

void forumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForumTopic");
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    s.store_object_field("lastMessage", static_cast<const BaseObject *>(last_message_.get()));
    s.store_field("order", order_);
    s.store_field("isPinned", is_pinned_);
    s.store_field("unreadCount", unread_count_);
    s.store_field("lastReadInboxMessageId", last_read_inbox_message_id_);
    s.store_field("lastReadOutboxMessageId", last_read_outbox_message_id_);
    s.store_field("unreadMentionCount", unread_mention_count_);
    s.store_field("unreadReactionCount", unread_reaction_count_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    s.store_class_end();
  }
}

void forumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForumTopic").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    last_message_fieldID = jni::get_field_id(env, Class, "lastMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    order_fieldID = jni::get_field_id(env, Class, "order", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    unread_count_fieldID = jni::get_field_id(env, Class, "unreadCount", "I");
    last_read_inbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadInboxMessageId", "J");
    last_read_outbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadOutboxMessageId", "J");
    unread_mention_count_fieldID = jni::get_field_id(env, Class, "unreadMentionCount", "I");
    unread_reaction_count_fieldID = jni::get_field_id(env, Class, "unreadReactionCount", "I");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass forumTopicIcon::Class;
jfieldID forumTopicIcon::color_fieldID;
jfieldID forumTopicIcon::custom_emoji_id_fieldID;

forumTopicIcon::forumTopicIcon()
  : color_()
  , custom_emoji_id_()
{}

forumTopicIcon::forumTopicIcon(int32 color_, int64 custom_emoji_id_)
  : color_(color_)
  , custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t forumTopicIcon::ID;

object_ptr<forumTopicIcon> forumTopicIcon::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forumTopicIcon> res = make_object<forumTopicIcon>();
  res->color_ = env->GetIntField(p, res->color_fieldID);
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return res;
}

void forumTopicIcon::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, color_fieldID, color_);
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
}

void forumTopicIcon::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForumTopicIcon");
    s.store_field("color", color_);
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

void forumTopicIcon::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForumTopicIcon").c_str());
    color_fieldID = jni::get_field_id(env, Class, "color", "I");
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass forumTopicInfo::Class;
jfieldID forumTopicInfo::chat_id_fieldID;
jfieldID forumTopicInfo::message_thread_id_fieldID;
jfieldID forumTopicInfo::name_fieldID;
jfieldID forumTopicInfo::icon_fieldID;
jfieldID forumTopicInfo::creation_date_fieldID;
jfieldID forumTopicInfo::creator_id_fieldID;
jfieldID forumTopicInfo::is_general_fieldID;
jfieldID forumTopicInfo::is_outgoing_fieldID;
jfieldID forumTopicInfo::is_closed_fieldID;
jfieldID forumTopicInfo::is_hidden_fieldID;

forumTopicInfo::forumTopicInfo()
  : chat_id_()
  , message_thread_id_()
  , name_()
  , icon_()
  , creation_date_()
  , creator_id_()
  , is_general_()
  , is_outgoing_()
  , is_closed_()
  , is_hidden_()
{}

forumTopicInfo::forumTopicInfo(int53 chat_id_, int53 message_thread_id_, string const &name_, object_ptr<forumTopicIcon> &&icon_, int32 creation_date_, object_ptr<MessageSender> &&creator_id_, bool is_general_, bool is_outgoing_, bool is_closed_, bool is_hidden_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , name_(name_)
  , icon_(std::move(icon_))
  , creation_date_(creation_date_)
  , creator_id_(std::move(creator_id_))
  , is_general_(is_general_)
  , is_outgoing_(is_outgoing_)
  , is_closed_(is_closed_)
  , is_hidden_(is_hidden_)
{}

const std::int32_t forumTopicInfo::ID;

object_ptr<forumTopicInfo> forumTopicInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forumTopicInfo> res = make_object<forumTopicInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->icon_ = jni::fetch_tl_object<forumTopicIcon>(env, jni::fetch_object(env, p, res->icon_fieldID));
  res->creation_date_ = env->GetIntField(p, res->creation_date_fieldID);
  res->creator_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->creator_id_fieldID));
  res->is_general_ = (env->GetBooleanField(p, res->is_general_fieldID) != 0);
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  res->is_closed_ = (env->GetBooleanField(p, res->is_closed_fieldID) != 0);
  res->is_hidden_ = (env->GetBooleanField(p, res->is_hidden_fieldID) != 0);
  return res;
}

void forumTopicInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (icon_ != nullptr) { jobject next; icon_->store(env, next); if (next) { env->SetObjectField(s, icon_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, creation_date_fieldID, creation_date_);
  if (creator_id_ != nullptr) { jobject next; creator_id_->store(env, next); if (next) { env->SetObjectField(s, creator_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_general_fieldID, is_general_);
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
  env->SetBooleanField(s, is_closed_fieldID, is_closed_);
  env->SetBooleanField(s, is_hidden_fieldID, is_hidden_);
}

void forumTopicInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForumTopicInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("name", name_);
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_field("creationDate", creation_date_);
    s.store_object_field("creatorId", static_cast<const BaseObject *>(creator_id_.get()));
    s.store_field("isGeneral", is_general_);
    s.store_field("isOutgoing", is_outgoing_);
    s.store_field("isClosed", is_closed_);
    s.store_field("isHidden", is_hidden_);
    s.store_class_end();
  }
}

void forumTopicInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForumTopicInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicIcon;").c_str());
    creation_date_fieldID = jni::get_field_id(env, Class, "creationDate", "I");
    creator_id_fieldID = jni::get_field_id(env, Class, "creatorId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_general_fieldID = jni::get_field_id(env, Class, "isGeneral", "Z");
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    is_closed_fieldID = jni::get_field_id(env, Class, "isClosed", "Z");
    is_hidden_fieldID = jni::get_field_id(env, Class, "isHidden", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass forumTopics::Class;
jfieldID forumTopics::total_count_fieldID;
jfieldID forumTopics::topics_fieldID;
jfieldID forumTopics::next_offset_date_fieldID;
jfieldID forumTopics::next_offset_message_id_fieldID;
jfieldID forumTopics::next_offset_message_thread_id_fieldID;

forumTopics::forumTopics()
  : total_count_()
  , topics_()
  , next_offset_date_()
  , next_offset_message_id_()
  , next_offset_message_thread_id_()
{}

forumTopics::forumTopics(int32 total_count_, array<object_ptr<forumTopic>> &&topics_, int32 next_offset_date_, int53 next_offset_message_id_, int53 next_offset_message_thread_id_)
  : total_count_(total_count_)
  , topics_(std::move(topics_))
  , next_offset_date_(next_offset_date_)
  , next_offset_message_id_(next_offset_message_id_)
  , next_offset_message_thread_id_(next_offset_message_thread_id_)
{}

const std::int32_t forumTopics::ID;

object_ptr<forumTopics> forumTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forumTopics> res = make_object<forumTopics>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->topics_ = jni::FetchVector<forumTopic>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->topics_fieldID));
  res->next_offset_date_ = env->GetIntField(p, res->next_offset_date_fieldID);
  res->next_offset_message_id_ = env->GetLongField(p, res->next_offset_message_id_fieldID);
  res->next_offset_message_thread_id_ = env->GetLongField(p, res->next_offset_message_thread_id_fieldID);
  return res;
}

void forumTopics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, topics_); if (arr_tmp_) { env->SetObjectField(s, topics_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, next_offset_date_fieldID, next_offset_date_);
  env->SetLongField(s, next_offset_message_id_fieldID, next_offset_message_id_);
  env->SetLongField(s, next_offset_message_thread_id_fieldID, next_offset_message_thread_id_);
}

void forumTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForumTopics");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("topics", topics_.size()); for (const auto &_value : topics_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffsetDate", next_offset_date_);
    s.store_field("nextOffsetMessageId", next_offset_message_id_);
    s.store_field("nextOffsetMessageThreadId", next_offset_message_thread_id_);
    s.store_class_end();
  }
}

void forumTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForumTopics").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    topics_fieldID = jni::get_field_id(env, Class, "topics", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ForumTopic;").c_str());
    next_offset_date_fieldID = jni::get_field_id(env, Class, "nextOffsetDate", "I");
    next_offset_message_id_fieldID = jni::get_field_id(env, Class, "nextOffsetMessageId", "J");
    next_offset_message_thread_id_fieldID = jni::get_field_id(env, Class, "nextOffsetMessageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass forwardSource::Class;
jfieldID forwardSource::chat_id_fieldID;
jfieldID forwardSource::message_id_fieldID;
jfieldID forwardSource::sender_id_fieldID;
jfieldID forwardSource::sender_name_fieldID;
jfieldID forwardSource::date_fieldID;
jfieldID forwardSource::is_outgoing_fieldID;

forwardSource::forwardSource()
  : chat_id_()
  , message_id_()
  , sender_id_()
  , sender_name_()
  , date_()
  , is_outgoing_()
{}

forwardSource::forwardSource(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&sender_id_, string const &sender_name_, int32 date_, bool is_outgoing_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , sender_id_(std::move(sender_id_))
  , sender_name_(sender_name_)
  , date_(date_)
  , is_outgoing_(is_outgoing_)
{}

const std::int32_t forwardSource::ID;

object_ptr<forwardSource> forwardSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forwardSource> res = make_object<forwardSource>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->sender_name_ = jni::fetch_string(env, p, res->sender_name_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  return res;
}

void forwardSource::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, sender_name_); if (nextString) { env->SetObjectField(s, sender_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, date_fieldID, date_);
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
}

void forwardSource::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForwardSource");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("senderName", sender_name_);
    s.store_field("date", date_);
    s.store_field("isOutgoing", is_outgoing_);
    s.store_class_end();
  }
}

void forwardSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForwardSource").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    sender_name_fieldID = jni::get_field_id(env, Class, "senderName", "Ljava/lang/String;");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass foundAffiliateProgram::Class;
jfieldID foundAffiliateProgram::bot_user_id_fieldID;
jfieldID foundAffiliateProgram::info_fieldID;

foundAffiliateProgram::foundAffiliateProgram()
  : bot_user_id_()
  , info_()
{}

foundAffiliateProgram::foundAffiliateProgram(int53 bot_user_id_, object_ptr<affiliateProgramInfo> &&info_)
  : bot_user_id_(bot_user_id_)
  , info_(std::move(info_))
{}

const std::int32_t foundAffiliateProgram::ID;

object_ptr<foundAffiliateProgram> foundAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundAffiliateProgram> res = make_object<foundAffiliateProgram>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->info_ = jni::fetch_tl_object<affiliateProgramInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  return res;
}

void foundAffiliateProgram::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  if (info_ != nullptr) { jobject next; info_->store(env, next); if (next) { env->SetObjectField(s, info_fieldID, next); env->DeleteLocalRef(next); } }
}

void foundAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundAffiliateProgram");
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    s.store_class_end();
  }
}

void foundAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundAffiliateProgram").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass foundAffiliatePrograms::Class;
jfieldID foundAffiliatePrograms::total_count_fieldID;
jfieldID foundAffiliatePrograms::programs_fieldID;
jfieldID foundAffiliatePrograms::next_offset_fieldID;

foundAffiliatePrograms::foundAffiliatePrograms()
  : total_count_()
  , programs_()
  , next_offset_()
{}

foundAffiliatePrograms::foundAffiliatePrograms(int32 total_count_, array<object_ptr<foundAffiliateProgram>> &&programs_, string const &next_offset_)
  : total_count_(total_count_)
  , programs_(std::move(programs_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundAffiliatePrograms::ID;

object_ptr<foundAffiliatePrograms> foundAffiliatePrograms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundAffiliatePrograms> res = make_object<foundAffiliatePrograms>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->programs_ = jni::FetchVector<foundAffiliateProgram>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->programs_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundAffiliatePrograms::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, programs_); if (arr_tmp_) { env->SetObjectField(s, programs_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundAffiliatePrograms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundAffiliatePrograms");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("programs", programs_.size()); for (const auto &_value : programs_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundAffiliatePrograms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundAffiliatePrograms").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    programs_fieldID = jni::get_field_id(env, Class, "programs", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$FoundAffiliateProgram;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundChatBoosts::Class;
jfieldID foundChatBoosts::total_count_fieldID;
jfieldID foundChatBoosts::boosts_fieldID;
jfieldID foundChatBoosts::next_offset_fieldID;

foundChatBoosts::foundChatBoosts()
  : total_count_()
  , boosts_()
  , next_offset_()
{}

foundChatBoosts::foundChatBoosts(int32 total_count_, array<object_ptr<chatBoost>> &&boosts_, string const &next_offset_)
  : total_count_(total_count_)
  , boosts_(std::move(boosts_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundChatBoosts::ID;

object_ptr<foundChatBoosts> foundChatBoosts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundChatBoosts> res = make_object<foundChatBoosts>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->boosts_ = jni::FetchVector<chatBoost>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->boosts_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundChatBoosts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, boosts_); if (arr_tmp_) { env->SetObjectField(s, boosts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundChatBoosts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundChatBoosts");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("boosts", boosts_.size()); for (const auto &_value : boosts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundChatBoosts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundChatBoosts").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    boosts_fieldID = jni::get_field_id(env, Class, "boosts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatBoost;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundChatMessages::Class;
jfieldID foundChatMessages::total_count_fieldID;
jfieldID foundChatMessages::messages_fieldID;
jfieldID foundChatMessages::next_from_message_id_fieldID;

foundChatMessages::foundChatMessages()
  : total_count_()
  , messages_()
  , next_from_message_id_()
{}

foundChatMessages::foundChatMessages(int32 total_count_, array<object_ptr<message>> &&messages_, int53 next_from_message_id_)
  : total_count_(total_count_)
  , messages_(std::move(messages_))
  , next_from_message_id_(next_from_message_id_)
{}

const std::int32_t foundChatMessages::ID;

object_ptr<foundChatMessages> foundChatMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundChatMessages> res = make_object<foundChatMessages>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->messages_ = jni::FetchVector<message>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  res->next_from_message_id_ = env->GetLongField(p, res->next_from_message_id_fieldID);
  return res;
}

void foundChatMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetLongField(s, next_from_message_id_fieldID, next_from_message_id_);
}

void foundChatMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundChatMessages");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextFromMessageId", next_from_message_id_);
    s.store_class_end();
  }
}

void foundChatMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundChatMessages").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    next_from_message_id_fieldID = jni::get_field_id(env, Class, "nextFromMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass foundFileDownloads::Class;
jfieldID foundFileDownloads::total_counts_fieldID;
jfieldID foundFileDownloads::files_fieldID;
jfieldID foundFileDownloads::next_offset_fieldID;

foundFileDownloads::foundFileDownloads()
  : total_counts_()
  , files_()
  , next_offset_()
{}

foundFileDownloads::foundFileDownloads(object_ptr<downloadedFileCounts> &&total_counts_, array<object_ptr<fileDownload>> &&files_, string const &next_offset_)
  : total_counts_(std::move(total_counts_))
  , files_(std::move(files_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundFileDownloads::ID;

object_ptr<foundFileDownloads> foundFileDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundFileDownloads> res = make_object<foundFileDownloads>();
  res->total_counts_ = jni::fetch_tl_object<downloadedFileCounts>(env, jni::fetch_object(env, p, res->total_counts_fieldID));
  res->files_ = jni::FetchVector<fileDownload>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->files_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundFileDownloads::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (total_counts_ != nullptr) { jobject next; total_counts_->store(env, next); if (next) { env->SetObjectField(s, total_counts_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, files_); if (arr_tmp_) { env->SetObjectField(s, files_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundFileDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundFileDownloads");
    s.store_object_field("totalCounts", static_cast<const BaseObject *>(total_counts_.get()));
    { s.store_vector_begin("files", files_.size()); for (const auto &_value : files_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundFileDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundFileDownloads").c_str());
    total_counts_fieldID = jni::get_field_id(env, Class, "totalCounts", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DownloadedFileCounts;").c_str());
    files_fieldID = jni::get_field_id(env, Class, "files", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$FileDownload;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundMessages::Class;
jfieldID foundMessages::total_count_fieldID;
jfieldID foundMessages::messages_fieldID;
jfieldID foundMessages::next_offset_fieldID;

foundMessages::foundMessages()
  : total_count_()
  , messages_()
  , next_offset_()
{}

foundMessages::foundMessages(int32 total_count_, array<object_ptr<message>> &&messages_, string const &next_offset_)
  : total_count_(total_count_)
  , messages_(std::move(messages_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundMessages::ID;

object_ptr<foundMessages> foundMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundMessages> res = make_object<foundMessages>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->messages_ = jni::FetchVector<message>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundMessages");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundMessages").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundPosition::Class;
jfieldID foundPosition::position_fieldID;

foundPosition::foundPosition()
  : position_()
{}

foundPosition::foundPosition(int32 position_)
  : position_(position_)
{}

const std::int32_t foundPosition::ID;

object_ptr<foundPosition> foundPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundPosition> res = make_object<foundPosition>();
  res->position_ = env->GetIntField(p, res->position_fieldID);
  return res;
}

void foundPosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, position_fieldID, position_);
}

void foundPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundPosition");
    s.store_field("position", position_);
    s.store_class_end();
  }
}

void foundPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundPosition").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    return true;
  }();
  (void)is_inited;
}

jclass foundPositions::Class;
jfieldID foundPositions::total_count_fieldID;
jfieldID foundPositions::positions_fieldID;

foundPositions::foundPositions()
  : total_count_()
  , positions_()
{}

foundPositions::foundPositions(int32 total_count_, array<int32> &&positions_)
  : total_count_(total_count_)
  , positions_(std::move(positions_))
{}

const std::int32_t foundPositions::ID;

object_ptr<foundPositions> foundPositions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundPositions> res = make_object<foundPositions>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->positions_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->positions_fieldID));
  return res;
}

void foundPositions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, positions_); if (arr_tmp_) { env->SetObjectField(s, positions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void foundPositions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundPositions");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("positions", positions_.size()); for (const auto &_value : positions_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void foundPositions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundPositions").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    positions_fieldID = jni::get_field_id(env, Class, "positions", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass foundStories::Class;
jfieldID foundStories::total_count_fieldID;
jfieldID foundStories::stories_fieldID;
jfieldID foundStories::next_offset_fieldID;

foundStories::foundStories()
  : total_count_()
  , stories_()
  , next_offset_()
{}

foundStories::foundStories(int32 total_count_, array<object_ptr<story>> &&stories_, string const &next_offset_)
  : total_count_(total_count_)
  , stories_(std::move(stories_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundStories::ID;

object_ptr<foundStories> foundStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundStories> res = make_object<foundStories>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->stories_ = jni::FetchVector<story>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stories_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, stories_); if (arr_tmp_) { env->SetObjectField(s, stories_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundStories");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("stories", stories_.size()); for (const auto &_value : stories_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundStories").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    stories_fieldID = jni::get_field_id(env, Class, "stories", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundUsers::Class;
jfieldID foundUsers::user_ids_fieldID;
jfieldID foundUsers::next_offset_fieldID;

foundUsers::foundUsers()
  : user_ids_()
  , next_offset_()
{}

foundUsers::foundUsers(array<int53> &&user_ids_, string const &next_offset_)
  : user_ids_(std::move(user_ids_))
  , next_offset_(next_offset_)
{}

const std::int32_t foundUsers::ID;

object_ptr<foundUsers> foundUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundUsers> res = make_object<foundUsers>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void foundUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void foundUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundUsers");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void foundUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundUsers").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass foundWebApp::Class;
jfieldID foundWebApp::web_app_fieldID;
jfieldID foundWebApp::request_write_access_fieldID;
jfieldID foundWebApp::skip_confirmation_fieldID;

foundWebApp::foundWebApp()
  : web_app_()
  , request_write_access_()
  , skip_confirmation_()
{}

foundWebApp::foundWebApp(object_ptr<webApp> &&web_app_, bool request_write_access_, bool skip_confirmation_)
  : web_app_(std::move(web_app_))
  , request_write_access_(request_write_access_)
  , skip_confirmation_(skip_confirmation_)
{}

const std::int32_t foundWebApp::ID;

object_ptr<foundWebApp> foundWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<foundWebApp> res = make_object<foundWebApp>();
  res->web_app_ = jni::fetch_tl_object<webApp>(env, jni::fetch_object(env, p, res->web_app_fieldID));
  res->request_write_access_ = (env->GetBooleanField(p, res->request_write_access_fieldID) != 0);
  res->skip_confirmation_ = (env->GetBooleanField(p, res->skip_confirmation_fieldID) != 0);
  return res;
}

void foundWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (web_app_ != nullptr) { jobject next; web_app_->store(env, next); if (next) { env->SetObjectField(s, web_app_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, request_write_access_fieldID, request_write_access_);
  env->SetBooleanField(s, skip_confirmation_fieldID, skip_confirmation_);
}

void foundWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FoundWebApp");
    s.store_object_field("webApp", static_cast<const BaseObject *>(web_app_.get()));
    s.store_field("requestWriteAccess", request_write_access_);
    s.store_field("skipConfirmation", skip_confirmation_);
    s.store_class_end();
  }
}

void foundWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FoundWebApp").c_str());
    web_app_fieldID = jni::get_field_id(env, Class, "webApp", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebApp;").c_str());
    request_write_access_fieldID = jni::get_field_id(env, Class, "requestWriteAccess", "Z");
    skip_confirmation_fieldID = jni::get_field_id(env, Class, "skipConfirmation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass game::Class;
jfieldID game::id_fieldID;
jfieldID game::short_name_fieldID;
jfieldID game::title_fieldID;
jfieldID game::text_fieldID;
jfieldID game::description_fieldID;
jfieldID game::photo_fieldID;
jfieldID game::animation_fieldID;

game::game()
  : id_()
  , short_name_()
  , title_()
  , text_()
  , description_()
  , photo_()
  , animation_()
{}

game::game(int64 id_, string const &short_name_, string const &title_, object_ptr<formattedText> &&text_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_)
  : id_(id_)
  , short_name_(short_name_)
  , title_(title_)
  , text_(std::move(text_))
  , description_(description_)
  , photo_(std::move(photo_))
  , animation_(std::move(animation_))
{}

const std::int32_t game::ID;

object_ptr<game> game::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<game> res = make_object<game>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->short_name_ = jni::fetch_string(env, p, res->short_name_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void game::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, short_name_); if (nextString) { env->SetObjectField(s, short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void game::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Game");
    s.store_field("id", id_);
    s.store_field("shortName", short_name_);
    s.store_field("title", title_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("description", description_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void game::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Game").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    short_name_fieldID = jni::get_field_id(env, Class, "shortName", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass gameHighScore::Class;
jfieldID gameHighScore::position_fieldID;
jfieldID gameHighScore::user_id_fieldID;
jfieldID gameHighScore::score_fieldID;

gameHighScore::gameHighScore()
  : position_()
  , user_id_()
  , score_()
{}

gameHighScore::gameHighScore(int32 position_, int53 user_id_, int32 score_)
  : position_(position_)
  , user_id_(user_id_)
  , score_(score_)
{}

const std::int32_t gameHighScore::ID;

object_ptr<gameHighScore> gameHighScore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<gameHighScore> res = make_object<gameHighScore>();
  res->position_ = env->GetIntField(p, res->position_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->score_ = env->GetIntField(p, res->score_fieldID);
  return res;
}

void gameHighScore::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, position_fieldID, position_);
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, score_fieldID, score_);
}

void gameHighScore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GameHighScore");
    s.store_field("position", position_);
    s.store_field("userId", user_id_);
    s.store_field("score", score_);
    s.store_class_end();
  }
}

void gameHighScore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GameHighScore").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    score_fieldID = jni::get_field_id(env, Class, "score", "I");
    return true;
  }();
  (void)is_inited;
}

jclass gameHighScores::Class;
jfieldID gameHighScores::scores_fieldID;

gameHighScores::gameHighScores()
  : scores_()
{}

gameHighScores::gameHighScores(array<object_ptr<gameHighScore>> &&scores_)
  : scores_(std::move(scores_))
{}

const std::int32_t gameHighScores::ID;

object_ptr<gameHighScores> gameHighScores::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<gameHighScores> res = make_object<gameHighScores>();
  res->scores_ = jni::FetchVector<gameHighScore>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->scores_fieldID));
  return res;
}

void gameHighScores::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, scores_); if (arr_tmp_) { env->SetObjectField(s, scores_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void gameHighScores::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GameHighScores");
    { s.store_vector_begin("scores", scores_.size()); for (const auto &_value : scores_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void gameHighScores::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GameHighScores").c_str());
    scores_fieldID = jni::get_field_id(env, Class, "scores", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$GameHighScore;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass gift::Class;
jfieldID gift::id_fieldID;
jfieldID gift::sticker_fieldID;
jfieldID gift::star_count_fieldID;
jfieldID gift::default_sell_star_count_fieldID;
jfieldID gift::upgrade_star_count_fieldID;
jfieldID gift::is_for_birthday_fieldID;
jfieldID gift::remaining_count_fieldID;
jfieldID gift::total_count_fieldID;
jfieldID gift::first_send_date_fieldID;
jfieldID gift::last_send_date_fieldID;

gift::gift()
  : id_()
  , sticker_()
  , star_count_()
  , default_sell_star_count_()
  , upgrade_star_count_()
  , is_for_birthday_()
  , remaining_count_()
  , total_count_()
  , first_send_date_()
  , last_send_date_()
{}

gift::gift(int64 id_, object_ptr<sticker> &&sticker_, int53 star_count_, int53 default_sell_star_count_, int53 upgrade_star_count_, bool is_for_birthday_, int32 remaining_count_, int32 total_count_, int32 first_send_date_, int32 last_send_date_)
  : id_(id_)
  , sticker_(std::move(sticker_))
  , star_count_(star_count_)
  , default_sell_star_count_(default_sell_star_count_)
  , upgrade_star_count_(upgrade_star_count_)
  , is_for_birthday_(is_for_birthday_)
  , remaining_count_(remaining_count_)
  , total_count_(total_count_)
  , first_send_date_(first_send_date_)
  , last_send_date_(last_send_date_)
{}

const std::int32_t gift::ID;

object_ptr<gift> gift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<gift> res = make_object<gift>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->default_sell_star_count_ = env->GetLongField(p, res->default_sell_star_count_fieldID);
  res->upgrade_star_count_ = env->GetLongField(p, res->upgrade_star_count_fieldID);
  res->is_for_birthday_ = (env->GetBooleanField(p, res->is_for_birthday_fieldID) != 0);
  res->remaining_count_ = env->GetIntField(p, res->remaining_count_fieldID);
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->first_send_date_ = env->GetIntField(p, res->first_send_date_fieldID);
  res->last_send_date_ = env->GetIntField(p, res->last_send_date_fieldID);
  return res;
}

void gift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, star_count_fieldID, star_count_);
  env->SetLongField(s, default_sell_star_count_fieldID, default_sell_star_count_);
  env->SetLongField(s, upgrade_star_count_fieldID, upgrade_star_count_);
  env->SetBooleanField(s, is_for_birthday_fieldID, is_for_birthday_);
  env->SetIntField(s, remaining_count_fieldID, remaining_count_);
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetIntField(s, first_send_date_fieldID, first_send_date_);
  env->SetIntField(s, last_send_date_fieldID, last_send_date_);
}

void gift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Gift");
    s.store_field("id", id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("starCount", star_count_);
    s.store_field("defaultSellStarCount", default_sell_star_count_);
    s.store_field("upgradeStarCount", upgrade_star_count_);
    s.store_field("isForBirthday", is_for_birthday_);
    s.store_field("remainingCount", remaining_count_);
    s.store_field("totalCount", total_count_);
    s.store_field("firstSendDate", first_send_date_);
    s.store_field("lastSendDate", last_send_date_);
    s.store_class_end();
  }
}

void gift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Gift").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    default_sell_star_count_fieldID = jni::get_field_id(env, Class, "defaultSellStarCount", "J");
    upgrade_star_count_fieldID = jni::get_field_id(env, Class, "upgradeStarCount", "J");
    is_for_birthday_fieldID = jni::get_field_id(env, Class, "isForBirthday", "Z");
    remaining_count_fieldID = jni::get_field_id(env, Class, "remainingCount", "I");
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    first_send_date_fieldID = jni::get_field_id(env, Class, "firstSendDate", "I");
    last_send_date_fieldID = jni::get_field_id(env, Class, "lastSendDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass giftForResale::Class;
jfieldID giftForResale::gift_fieldID;
jfieldID giftForResale::received_gift_id_fieldID;

giftForResale::giftForResale()
  : gift_()
  , received_gift_id_()
{}

giftForResale::giftForResale(object_ptr<upgradedGift> &&gift_, string const &received_gift_id_)
  : gift_(std::move(gift_))
  , received_gift_id_(received_gift_id_)
{}

const std::int32_t giftForResale::ID;

object_ptr<giftForResale> giftForResale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giftForResale> res = make_object<giftForResale>();
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  return res;
}

void giftForResale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, received_gift_id_); if (nextString) { env->SetObjectField(s, received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void giftForResale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftForResale");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_class_end();
  }
}

void giftForResale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftForResale").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass GiftForResaleOrder::Class;

object_ptr<GiftForResaleOrder> GiftForResaleOrder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case giftForResaleOrderPrice::ID:
      return giftForResaleOrderPrice::fetch(env, p);
    case giftForResaleOrderPriceChangeDate::ID:
      return giftForResaleOrderPriceChangeDate::fetch(env, p);
    case giftForResaleOrderNumber::ID:
      return giftForResaleOrderNumber::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GiftForResaleOrder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftForResaleOrder").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftForResaleOrderPrice::Class;

giftForResaleOrderPrice::giftForResaleOrderPrice() {
}

const std::int32_t giftForResaleOrderPrice::ID;

object_ptr<GiftForResaleOrder> giftForResaleOrderPrice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<giftForResaleOrderPrice>();
}

void giftForResaleOrderPrice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftForResaleOrderPrice");
    s.store_class_end();
  }
}

void giftForResaleOrderPrice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftForResaleOrderPrice").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftForResaleOrderPriceChangeDate::Class;

giftForResaleOrderPriceChangeDate::giftForResaleOrderPriceChangeDate() {
}

const std::int32_t giftForResaleOrderPriceChangeDate::ID;

object_ptr<GiftForResaleOrder> giftForResaleOrderPriceChangeDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<giftForResaleOrderPriceChangeDate>();
}

void giftForResaleOrderPriceChangeDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftForResaleOrderPriceChangeDate");
    s.store_class_end();
  }
}

void giftForResaleOrderPriceChangeDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftForResaleOrderPriceChangeDate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftForResaleOrderNumber::Class;

giftForResaleOrderNumber::giftForResaleOrderNumber() {
}

const std::int32_t giftForResaleOrderNumber::ID;

object_ptr<GiftForResaleOrder> giftForResaleOrderNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<giftForResaleOrderNumber>();
}

void giftForResaleOrderNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftForResaleOrderNumber");
    s.store_class_end();
  }
}

void giftForResaleOrderNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftForResaleOrderNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftSettings::Class;
jfieldID giftSettings::show_gift_button_fieldID;
jfieldID giftSettings::accepted_gift_types_fieldID;

giftSettings::giftSettings()
  : show_gift_button_()
  , accepted_gift_types_()
{}

giftSettings::giftSettings(bool show_gift_button_, object_ptr<acceptedGiftTypes> &&accepted_gift_types_)
  : show_gift_button_(show_gift_button_)
  , accepted_gift_types_(std::move(accepted_gift_types_))
{}

const std::int32_t giftSettings::ID;

object_ptr<giftSettings> giftSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giftSettings> res = make_object<giftSettings>();
  res->show_gift_button_ = (env->GetBooleanField(p, res->show_gift_button_fieldID) != 0);
  res->accepted_gift_types_ = jni::fetch_tl_object<acceptedGiftTypes>(env, jni::fetch_object(env, p, res->accepted_gift_types_fieldID));
  return res;
}

void giftSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, show_gift_button_fieldID, show_gift_button_);
  if (accepted_gift_types_ != nullptr) { jobject next; accepted_gift_types_->store(env, next); if (next) { env->SetObjectField(s, accepted_gift_types_fieldID, next); env->DeleteLocalRef(next); } }
}

void giftSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftSettings");
    s.store_field("showGiftButton", show_gift_button_);
    s.store_object_field("acceptedGiftTypes", static_cast<const BaseObject *>(accepted_gift_types_.get()));
    s.store_class_end();
  }
}

void giftSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftSettings").c_str());
    show_gift_button_fieldID = jni::get_field_id(env, Class, "showGiftButton", "Z");
    accepted_gift_types_fieldID = jni::get_field_id(env, Class, "acceptedGiftTypes", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AcceptedGiftTypes;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftUpgradePreview::Class;
jfieldID giftUpgradePreview::models_fieldID;
jfieldID giftUpgradePreview::symbols_fieldID;
jfieldID giftUpgradePreview::backdrops_fieldID;

giftUpgradePreview::giftUpgradePreview()
  : models_()
  , symbols_()
  , backdrops_()
{}

giftUpgradePreview::giftUpgradePreview(array<object_ptr<upgradedGiftModel>> &&models_, array<object_ptr<upgradedGiftSymbol>> &&symbols_, array<object_ptr<upgradedGiftBackdrop>> &&backdrops_)
  : models_(std::move(models_))
  , symbols_(std::move(symbols_))
  , backdrops_(std::move(backdrops_))
{}

const std::int32_t giftUpgradePreview::ID;

object_ptr<giftUpgradePreview> giftUpgradePreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giftUpgradePreview> res = make_object<giftUpgradePreview>();
  res->models_ = jni::FetchVector<upgradedGiftModel>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->models_fieldID));
  res->symbols_ = jni::FetchVector<upgradedGiftSymbol>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->symbols_fieldID));
  res->backdrops_ = jni::FetchVector<upgradedGiftBackdrop>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->backdrops_fieldID));
  return res;
}

void giftUpgradePreview::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, models_); if (arr_tmp_) { env->SetObjectField(s, models_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, symbols_); if (arr_tmp_) { env->SetObjectField(s, symbols_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, backdrops_); if (arr_tmp_) { env->SetObjectField(s, backdrops_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void giftUpgradePreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftUpgradePreview");
    { s.store_vector_begin("models", models_.size()); for (const auto &_value : models_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("symbols", symbols_.size()); for (const auto &_value : symbols_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("backdrops", backdrops_.size()); for (const auto &_value : backdrops_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void giftUpgradePreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftUpgradePreview").c_str());
    models_fieldID = jni::get_field_id(env, Class, "models", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftModel;").c_str());
    symbols_fieldID = jni::get_field_id(env, Class, "symbols", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftSymbol;").c_str());
    backdrops_fieldID = jni::get_field_id(env, Class, "backdrops", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdrop;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giftsForResale::Class;
jfieldID giftsForResale::total_count_fieldID;
jfieldID giftsForResale::gifts_fieldID;
jfieldID giftsForResale::models_fieldID;
jfieldID giftsForResale::symbols_fieldID;
jfieldID giftsForResale::backdrops_fieldID;
jfieldID giftsForResale::next_offset_fieldID;

giftsForResale::giftsForResale()
  : total_count_()
  , gifts_()
  , models_()
  , symbols_()
  , backdrops_()
  , next_offset_()
{}

giftsForResale::giftsForResale(int32 total_count_, array<object_ptr<giftForResale>> &&gifts_, array<object_ptr<upgradedGiftModelCount>> &&models_, array<object_ptr<upgradedGiftSymbolCount>> &&symbols_, array<object_ptr<upgradedGiftBackdropCount>> &&backdrops_, string const &next_offset_)
  : total_count_(total_count_)
  , gifts_(std::move(gifts_))
  , models_(std::move(models_))
  , symbols_(std::move(symbols_))
  , backdrops_(std::move(backdrops_))
  , next_offset_(next_offset_)
{}

const std::int32_t giftsForResale::ID;

object_ptr<giftsForResale> giftsForResale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giftsForResale> res = make_object<giftsForResale>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->gifts_ = jni::FetchVector<giftForResale>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->gifts_fieldID));
  res->models_ = jni::FetchVector<upgradedGiftModelCount>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->models_fieldID));
  res->symbols_ = jni::FetchVector<upgradedGiftSymbolCount>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->symbols_fieldID));
  res->backdrops_ = jni::FetchVector<upgradedGiftBackdropCount>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->backdrops_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void giftsForResale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, gifts_); if (arr_tmp_) { env->SetObjectField(s, gifts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, models_); if (arr_tmp_) { env->SetObjectField(s, models_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, symbols_); if (arr_tmp_) { env->SetObjectField(s, symbols_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, backdrops_); if (arr_tmp_) { env->SetObjectField(s, backdrops_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void giftsForResale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftsForResale");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("gifts", gifts_.size()); for (const auto &_value : gifts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("models", models_.size()); for (const auto &_value : models_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("symbols", symbols_.size()); for (const auto &_value : symbols_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("backdrops", backdrops_.size()); for (const auto &_value : backdrops_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void giftsForResale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftsForResale").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    gifts_fieldID = jni::get_field_id(env, Class, "gifts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$GiftForResale;").c_str());
    models_fieldID = jni::get_field_id(env, Class, "models", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftModelCount;").c_str());
    symbols_fieldID = jni::get_field_id(env, Class, "symbols", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftSymbolCount;").c_str());
    backdrops_fieldID = jni::get_field_id(env, Class, "backdrops", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdropCount;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass GiveawayInfo::Class;

object_ptr<GiveawayInfo> GiveawayInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case giveawayInfoOngoing::ID:
      return giveawayInfoOngoing::fetch(env, p);
    case giveawayInfoCompleted::ID:
      return giveawayInfoCompleted::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GiveawayInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayInfo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giveawayInfoOngoing::Class;
jfieldID giveawayInfoOngoing::creation_date_fieldID;
jfieldID giveawayInfoOngoing::status_fieldID;
jfieldID giveawayInfoOngoing::is_ended_fieldID;

giveawayInfoOngoing::giveawayInfoOngoing()
  : creation_date_()
  , status_()
  , is_ended_()
{}

giveawayInfoOngoing::giveawayInfoOngoing(int32 creation_date_, object_ptr<GiveawayParticipantStatus> &&status_, bool is_ended_)
  : creation_date_(creation_date_)
  , status_(std::move(status_))
  , is_ended_(is_ended_)
{}

const std::int32_t giveawayInfoOngoing::ID;

object_ptr<GiveawayInfo> giveawayInfoOngoing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayInfoOngoing> res = make_object<giveawayInfoOngoing>();
  res->creation_date_ = env->GetIntField(p, res->creation_date_fieldID);
  res->status_ = jni::fetch_tl_object<GiveawayParticipantStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  res->is_ended_ = (env->GetBooleanField(p, res->is_ended_fieldID) != 0);
  return std::move(res);
}

void giveawayInfoOngoing::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, creation_date_fieldID, creation_date_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_ended_fieldID, is_ended_);
}

void giveawayInfoOngoing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayInfoOngoing");
    s.store_field("creationDate", creation_date_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_field("isEnded", is_ended_);
    s.store_class_end();
  }
}

void giveawayInfoOngoing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayInfoOngoing").c_str());
    creation_date_fieldID = jni::get_field_id(env, Class, "creationDate", "I");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParticipantStatus;").c_str());
    is_ended_fieldID = jni::get_field_id(env, Class, "isEnded", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass giveawayInfoCompleted::Class;
jfieldID giveawayInfoCompleted::creation_date_fieldID;
jfieldID giveawayInfoCompleted::actual_winners_selection_date_fieldID;
jfieldID giveawayInfoCompleted::was_refunded_fieldID;
jfieldID giveawayInfoCompleted::is_winner_fieldID;
jfieldID giveawayInfoCompleted::winner_count_fieldID;
jfieldID giveawayInfoCompleted::activation_count_fieldID;
jfieldID giveawayInfoCompleted::gift_code_fieldID;
jfieldID giveawayInfoCompleted::won_star_count_fieldID;

giveawayInfoCompleted::giveawayInfoCompleted()
  : creation_date_()
  , actual_winners_selection_date_()
  , was_refunded_()
  , is_winner_()
  , winner_count_()
  , activation_count_()
  , gift_code_()
  , won_star_count_()
{}

giveawayInfoCompleted::giveawayInfoCompleted(int32 creation_date_, int32 actual_winners_selection_date_, bool was_refunded_, bool is_winner_, int32 winner_count_, int32 activation_count_, string const &gift_code_, int53 won_star_count_)
  : creation_date_(creation_date_)
  , actual_winners_selection_date_(actual_winners_selection_date_)
  , was_refunded_(was_refunded_)
  , is_winner_(is_winner_)
  , winner_count_(winner_count_)
  , activation_count_(activation_count_)
  , gift_code_(gift_code_)
  , won_star_count_(won_star_count_)
{}

const std::int32_t giveawayInfoCompleted::ID;

object_ptr<GiveawayInfo> giveawayInfoCompleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayInfoCompleted> res = make_object<giveawayInfoCompleted>();
  res->creation_date_ = env->GetIntField(p, res->creation_date_fieldID);
  res->actual_winners_selection_date_ = env->GetIntField(p, res->actual_winners_selection_date_fieldID);
  res->was_refunded_ = (env->GetBooleanField(p, res->was_refunded_fieldID) != 0);
  res->is_winner_ = (env->GetBooleanField(p, res->is_winner_fieldID) != 0);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->activation_count_ = env->GetIntField(p, res->activation_count_fieldID);
  res->gift_code_ = jni::fetch_string(env, p, res->gift_code_fieldID);
  res->won_star_count_ = env->GetLongField(p, res->won_star_count_fieldID);
  return std::move(res);
}

void giveawayInfoCompleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, creation_date_fieldID, creation_date_);
  env->SetIntField(s, actual_winners_selection_date_fieldID, actual_winners_selection_date_);
  env->SetBooleanField(s, was_refunded_fieldID, was_refunded_);
  env->SetBooleanField(s, is_winner_fieldID, is_winner_);
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  env->SetIntField(s, activation_count_fieldID, activation_count_);
  { jstring nextString = jni::to_jstring(env, gift_code_); if (nextString) { env->SetObjectField(s, gift_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, won_star_count_fieldID, won_star_count_);
}

void giveawayInfoCompleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayInfoCompleted");
    s.store_field("creationDate", creation_date_);
    s.store_field("actualWinnersSelectionDate", actual_winners_selection_date_);
    s.store_field("wasRefunded", was_refunded_);
    s.store_field("isWinner", is_winner_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("activationCount", activation_count_);
    s.store_field("giftCode", gift_code_);
    s.store_field("wonStarCount", won_star_count_);
    s.store_class_end();
  }
}

void giveawayInfoCompleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayInfoCompleted").c_str());
    creation_date_fieldID = jni::get_field_id(env, Class, "creationDate", "I");
    actual_winners_selection_date_fieldID = jni::get_field_id(env, Class, "actualWinnersSelectionDate", "I");
    was_refunded_fieldID = jni::get_field_id(env, Class, "wasRefunded", "Z");
    is_winner_fieldID = jni::get_field_id(env, Class, "isWinner", "Z");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    activation_count_fieldID = jni::get_field_id(env, Class, "activationCount", "I");
    gift_code_fieldID = jni::get_field_id(env, Class, "giftCode", "Ljava/lang/String;");
    won_star_count_fieldID = jni::get_field_id(env, Class, "wonStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParameters::Class;
jfieldID giveawayParameters::boosted_chat_id_fieldID;
jfieldID giveawayParameters::additional_chat_ids_fieldID;
jfieldID giveawayParameters::winners_selection_date_fieldID;
jfieldID giveawayParameters::only_new_members_fieldID;
jfieldID giveawayParameters::has_public_winners_fieldID;
jfieldID giveawayParameters::country_codes_fieldID;
jfieldID giveawayParameters::prize_description_fieldID;

giveawayParameters::giveawayParameters()
  : boosted_chat_id_()
  , additional_chat_ids_()
  , winners_selection_date_()
  , only_new_members_()
  , has_public_winners_()
  , country_codes_()
  , prize_description_()
{}

giveawayParameters::giveawayParameters(int53 boosted_chat_id_, array<int53> &&additional_chat_ids_, int32 winners_selection_date_, bool only_new_members_, bool has_public_winners_, array<string> &&country_codes_, string const &prize_description_)
  : boosted_chat_id_(boosted_chat_id_)
  , additional_chat_ids_(std::move(additional_chat_ids_))
  , winners_selection_date_(winners_selection_date_)
  , only_new_members_(only_new_members_)
  , has_public_winners_(has_public_winners_)
  , country_codes_(std::move(country_codes_))
  , prize_description_(prize_description_)
{}

const std::int32_t giveawayParameters::ID;

object_ptr<giveawayParameters> giveawayParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayParameters> res = make_object<giveawayParameters>();
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  res->additional_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->additional_chat_ids_fieldID));
  res->winners_selection_date_ = env->GetIntField(p, res->winners_selection_date_fieldID);
  res->only_new_members_ = (env->GetBooleanField(p, res->only_new_members_fieldID) != 0);
  res->has_public_winners_ = (env->GetBooleanField(p, res->has_public_winners_fieldID) != 0);
  res->country_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->country_codes_fieldID));
  res->prize_description_ = jni::fetch_string(env, p, res->prize_description_fieldID);
  return res;
}

void giveawayParameters::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, boosted_chat_id_fieldID, boosted_chat_id_);
  { auto arr_tmp_ = jni::store_vector(env, additional_chat_ids_); if (arr_tmp_) { env->SetObjectField(s, additional_chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, winners_selection_date_fieldID, winners_selection_date_);
  env->SetBooleanField(s, only_new_members_fieldID, only_new_members_);
  env->SetBooleanField(s, has_public_winners_fieldID, has_public_winners_);
  { auto arr_tmp_ = jni::store_vector(env, country_codes_); if (arr_tmp_) { env->SetObjectField(s, country_codes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, prize_description_); if (nextString) { env->SetObjectField(s, prize_description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void giveawayParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParameters");
    s.store_field("boostedChatId", boosted_chat_id_);
    { s.store_vector_begin("additionalChatIds", additional_chat_ids_.size()); for (const auto &_value : additional_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("winnersSelectionDate", winners_selection_date_);
    s.store_field("onlyNewMembers", only_new_members_);
    s.store_field("hasPublicWinners", has_public_winners_);
    { s.store_vector_begin("countryCodes", country_codes_.size()); for (const auto &_value : country_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("prizeDescription", prize_description_);
    s.store_class_end();
  }
}

void giveawayParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParameters").c_str());
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    additional_chat_ids_fieldID = jni::get_field_id(env, Class, "additionalChatIds", "[J");
    winners_selection_date_fieldID = jni::get_field_id(env, Class, "winnersSelectionDate", "I");
    only_new_members_fieldID = jni::get_field_id(env, Class, "onlyNewMembers", "Z");
    has_public_winners_fieldID = jni::get_field_id(env, Class, "hasPublicWinners", "Z");
    country_codes_fieldID = jni::get_field_id(env, Class, "countryCodes", "[Ljava/lang/String;");
    prize_description_fieldID = jni::get_field_id(env, Class, "prizeDescription", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass GiveawayParticipantStatus::Class;

object_ptr<GiveawayParticipantStatus> GiveawayParticipantStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case giveawayParticipantStatusEligible::ID:
      return giveawayParticipantStatusEligible::fetch(env, p);
    case giveawayParticipantStatusParticipating::ID:
      return giveawayParticipantStatusParticipating::fetch(env, p);
    case giveawayParticipantStatusAlreadyWasMember::ID:
      return giveawayParticipantStatusAlreadyWasMember::fetch(env, p);
    case giveawayParticipantStatusAdministrator::ID:
      return giveawayParticipantStatusAdministrator::fetch(env, p);
    case giveawayParticipantStatusDisallowedCountry::ID:
      return giveawayParticipantStatusDisallowedCountry::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GiveawayParticipantStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParticipantStatusEligible::Class;

giveawayParticipantStatusEligible::giveawayParticipantStatusEligible() {
}

const std::int32_t giveawayParticipantStatusEligible::ID;

object_ptr<GiveawayParticipantStatus> giveawayParticipantStatusEligible::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<giveawayParticipantStatusEligible>();
}

void giveawayParticipantStatusEligible::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void giveawayParticipantStatusEligible::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParticipantStatusEligible");
    s.store_class_end();
  }
}

void giveawayParticipantStatusEligible::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatusEligible").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParticipantStatusParticipating::Class;

giveawayParticipantStatusParticipating::giveawayParticipantStatusParticipating() {
}

const std::int32_t giveawayParticipantStatusParticipating::ID;

object_ptr<GiveawayParticipantStatus> giveawayParticipantStatusParticipating::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<giveawayParticipantStatusParticipating>();
}

void giveawayParticipantStatusParticipating::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void giveawayParticipantStatusParticipating::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParticipantStatusParticipating");
    s.store_class_end();
  }
}

void giveawayParticipantStatusParticipating::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatusParticipating").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParticipantStatusAlreadyWasMember::Class;
jfieldID giveawayParticipantStatusAlreadyWasMember::joined_chat_date_fieldID;

giveawayParticipantStatusAlreadyWasMember::giveawayParticipantStatusAlreadyWasMember()
  : joined_chat_date_()
{}

giveawayParticipantStatusAlreadyWasMember::giveawayParticipantStatusAlreadyWasMember(int32 joined_chat_date_)
  : joined_chat_date_(joined_chat_date_)
{}

const std::int32_t giveawayParticipantStatusAlreadyWasMember::ID;

object_ptr<GiveawayParticipantStatus> giveawayParticipantStatusAlreadyWasMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayParticipantStatusAlreadyWasMember> res = make_object<giveawayParticipantStatusAlreadyWasMember>();
  res->joined_chat_date_ = env->GetIntField(p, res->joined_chat_date_fieldID);
  return std::move(res);
}

void giveawayParticipantStatusAlreadyWasMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, joined_chat_date_fieldID, joined_chat_date_);
}

void giveawayParticipantStatusAlreadyWasMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParticipantStatusAlreadyWasMember");
    s.store_field("joinedChatDate", joined_chat_date_);
    s.store_class_end();
  }
}

void giveawayParticipantStatusAlreadyWasMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatusAlreadyWasMember").c_str());
    joined_chat_date_fieldID = jni::get_field_id(env, Class, "joinedChatDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParticipantStatusAdministrator::Class;
jfieldID giveawayParticipantStatusAdministrator::chat_id_fieldID;

giveawayParticipantStatusAdministrator::giveawayParticipantStatusAdministrator()
  : chat_id_()
{}

giveawayParticipantStatusAdministrator::giveawayParticipantStatusAdministrator(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t giveawayParticipantStatusAdministrator::ID;

object_ptr<GiveawayParticipantStatus> giveawayParticipantStatusAdministrator::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayParticipantStatusAdministrator> res = make_object<giveawayParticipantStatusAdministrator>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void giveawayParticipantStatusAdministrator::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void giveawayParticipantStatusAdministrator::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParticipantStatusAdministrator");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void giveawayParticipantStatusAdministrator::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatusAdministrator").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass giveawayParticipantStatusDisallowedCountry::Class;
jfieldID giveawayParticipantStatusDisallowedCountry::user_country_code_fieldID;

giveawayParticipantStatusDisallowedCountry::giveawayParticipantStatusDisallowedCountry()
  : user_country_code_()
{}

giveawayParticipantStatusDisallowedCountry::giveawayParticipantStatusDisallowedCountry(string const &user_country_code_)
  : user_country_code_(user_country_code_)
{}

const std::int32_t giveawayParticipantStatusDisallowedCountry::ID;

object_ptr<GiveawayParticipantStatus> giveawayParticipantStatusDisallowedCountry::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayParticipantStatusDisallowedCountry> res = make_object<giveawayParticipantStatusDisallowedCountry>();
  res->user_country_code_ = jni::fetch_string(env, p, res->user_country_code_fieldID);
  return std::move(res);
}

void giveawayParticipantStatusDisallowedCountry::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, user_country_code_); if (nextString) { env->SetObjectField(s, user_country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void giveawayParticipantStatusDisallowedCountry::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayParticipantStatusDisallowedCountry");
    s.store_field("userCountryCode", user_country_code_);
    s.store_class_end();
  }
}

void giveawayParticipantStatusDisallowedCountry::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayParticipantStatusDisallowedCountry").c_str());
    user_country_code_fieldID = jni::get_field_id(env, Class, "userCountryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass GiveawayPrize::Class;

object_ptr<GiveawayPrize> GiveawayPrize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case giveawayPrizePremium::ID:
      return giveawayPrizePremium::fetch(env, p);
    case giveawayPrizeStars::ID:
      return giveawayPrizeStars::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GiveawayPrize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayPrize").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass giveawayPrizePremium::Class;
jfieldID giveawayPrizePremium::month_count_fieldID;

giveawayPrizePremium::giveawayPrizePremium()
  : month_count_()
{}

giveawayPrizePremium::giveawayPrizePremium(int32 month_count_)
  : month_count_(month_count_)
{}

const std::int32_t giveawayPrizePremium::ID;

object_ptr<GiveawayPrize> giveawayPrizePremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayPrizePremium> res = make_object<giveawayPrizePremium>();
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  return std::move(res);
}

void giveawayPrizePremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, month_count_fieldID, month_count_);
}

void giveawayPrizePremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayPrizePremium");
    s.store_field("monthCount", month_count_);
    s.store_class_end();
  }
}

void giveawayPrizePremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayPrizePremium").c_str());
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass giveawayPrizeStars::Class;
jfieldID giveawayPrizeStars::star_count_fieldID;

giveawayPrizeStars::giveawayPrizeStars()
  : star_count_()
{}

giveawayPrizeStars::giveawayPrizeStars(int53 star_count_)
  : star_count_(star_count_)
{}

const std::int32_t giveawayPrizeStars::ID;

object_ptr<GiveawayPrize> giveawayPrizeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giveawayPrizeStars> res = make_object<giveawayPrizeStars>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void giveawayPrizeStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void giveawayPrizeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiveawayPrizeStars");
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void giveawayPrizeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiveawayPrizeStars").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass groupCall::Class;
jfieldID groupCall::id_fieldID;
jfieldID groupCall::title_fieldID;
jfieldID groupCall::invite_link_fieldID;
jfieldID groupCall::scheduled_start_date_fieldID;
jfieldID groupCall::enabled_start_notification_fieldID;
jfieldID groupCall::is_active_fieldID;
jfieldID groupCall::is_video_chat_fieldID;
jfieldID groupCall::is_rtmp_stream_fieldID;
jfieldID groupCall::is_joined_fieldID;
jfieldID groupCall::need_rejoin_fieldID;
jfieldID groupCall::is_owned_fieldID;
jfieldID groupCall::can_be_managed_fieldID;
jfieldID groupCall::participant_count_fieldID;
jfieldID groupCall::has_hidden_listeners_fieldID;
jfieldID groupCall::loaded_all_participants_fieldID;
jfieldID groupCall::recent_speakers_fieldID;
jfieldID groupCall::is_my_video_enabled_fieldID;
jfieldID groupCall::is_my_video_paused_fieldID;
jfieldID groupCall::can_enable_video_fieldID;
jfieldID groupCall::mute_new_participants_fieldID;
jfieldID groupCall::can_toggle_mute_new_participants_fieldID;
jfieldID groupCall::record_duration_fieldID;
jfieldID groupCall::is_video_recorded_fieldID;
jfieldID groupCall::duration_fieldID;

groupCall::groupCall()
  : id_()
  , title_()
  , invite_link_()
  , scheduled_start_date_()
  , enabled_start_notification_()
  , is_active_()
  , is_video_chat_()
  , is_rtmp_stream_()
  , is_joined_()
  , need_rejoin_()
  , is_owned_()
  , can_be_managed_()
  , participant_count_()
  , has_hidden_listeners_()
  , loaded_all_participants_()
  , recent_speakers_()
  , is_my_video_enabled_()
  , is_my_video_paused_()
  , can_enable_video_()
  , mute_new_participants_()
  , can_toggle_mute_new_participants_()
  , record_duration_()
  , is_video_recorded_()
  , duration_()
{}

groupCall::groupCall(int32 id_, string const &title_, string const &invite_link_, int32 scheduled_start_date_, bool enabled_start_notification_, bool is_active_, bool is_video_chat_, bool is_rtmp_stream_, bool is_joined_, bool need_rejoin_, bool is_owned_, bool can_be_managed_, int32 participant_count_, bool has_hidden_listeners_, bool loaded_all_participants_, array<object_ptr<groupCallRecentSpeaker>> &&recent_speakers_, bool is_my_video_enabled_, bool is_my_video_paused_, bool can_enable_video_, bool mute_new_participants_, bool can_toggle_mute_new_participants_, int32 record_duration_, bool is_video_recorded_, int32 duration_)
  : id_(id_)
  , title_(title_)
  , invite_link_(invite_link_)
  , scheduled_start_date_(scheduled_start_date_)
  , enabled_start_notification_(enabled_start_notification_)
  , is_active_(is_active_)
  , is_video_chat_(is_video_chat_)
  , is_rtmp_stream_(is_rtmp_stream_)
  , is_joined_(is_joined_)
  , need_rejoin_(need_rejoin_)
  , is_owned_(is_owned_)
  , can_be_managed_(can_be_managed_)
  , participant_count_(participant_count_)
  , has_hidden_listeners_(has_hidden_listeners_)
  , loaded_all_participants_(loaded_all_participants_)
  , recent_speakers_(std::move(recent_speakers_))
  , is_my_video_enabled_(is_my_video_enabled_)
  , is_my_video_paused_(is_my_video_paused_)
  , can_enable_video_(can_enable_video_)
  , mute_new_participants_(mute_new_participants_)
  , can_toggle_mute_new_participants_(can_toggle_mute_new_participants_)
  , record_duration_(record_duration_)
  , is_video_recorded_(is_video_recorded_)
  , duration_(duration_)
{}

const std::int32_t groupCall::ID;

object_ptr<groupCall> groupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCall> res = make_object<groupCall>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->scheduled_start_date_ = env->GetIntField(p, res->scheduled_start_date_fieldID);
  res->enabled_start_notification_ = (env->GetBooleanField(p, res->enabled_start_notification_fieldID) != 0);
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  res->is_video_chat_ = (env->GetBooleanField(p, res->is_video_chat_fieldID) != 0);
  res->is_rtmp_stream_ = (env->GetBooleanField(p, res->is_rtmp_stream_fieldID) != 0);
  res->is_joined_ = (env->GetBooleanField(p, res->is_joined_fieldID) != 0);
  res->need_rejoin_ = (env->GetBooleanField(p, res->need_rejoin_fieldID) != 0);
  res->is_owned_ = (env->GetBooleanField(p, res->is_owned_fieldID) != 0);
  res->can_be_managed_ = (env->GetBooleanField(p, res->can_be_managed_fieldID) != 0);
  res->participant_count_ = env->GetIntField(p, res->participant_count_fieldID);
  res->has_hidden_listeners_ = (env->GetBooleanField(p, res->has_hidden_listeners_fieldID) != 0);
  res->loaded_all_participants_ = (env->GetBooleanField(p, res->loaded_all_participants_fieldID) != 0);
  res->recent_speakers_ = jni::FetchVector<groupCallRecentSpeaker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_speakers_fieldID));
  res->is_my_video_enabled_ = (env->GetBooleanField(p, res->is_my_video_enabled_fieldID) != 0);
  res->is_my_video_paused_ = (env->GetBooleanField(p, res->is_my_video_paused_fieldID) != 0);
  res->can_enable_video_ = (env->GetBooleanField(p, res->can_enable_video_fieldID) != 0);
  res->mute_new_participants_ = (env->GetBooleanField(p, res->mute_new_participants_fieldID) != 0);
  res->can_toggle_mute_new_participants_ = (env->GetBooleanField(p, res->can_toggle_mute_new_participants_fieldID) != 0);
  res->record_duration_ = env->GetIntField(p, res->record_duration_fieldID);
  res->is_video_recorded_ = (env->GetBooleanField(p, res->is_video_recorded_fieldID) != 0);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  return res;
}

void groupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, scheduled_start_date_fieldID, scheduled_start_date_);
  env->SetBooleanField(s, enabled_start_notification_fieldID, enabled_start_notification_);
  env->SetBooleanField(s, is_active_fieldID, is_active_);
  env->SetBooleanField(s, is_video_chat_fieldID, is_video_chat_);
  env->SetBooleanField(s, is_rtmp_stream_fieldID, is_rtmp_stream_);
  env->SetBooleanField(s, is_joined_fieldID, is_joined_);
  env->SetBooleanField(s, need_rejoin_fieldID, need_rejoin_);
  env->SetBooleanField(s, is_owned_fieldID, is_owned_);
  env->SetBooleanField(s, can_be_managed_fieldID, can_be_managed_);
  env->SetIntField(s, participant_count_fieldID, participant_count_);
  env->SetBooleanField(s, has_hidden_listeners_fieldID, has_hidden_listeners_);
  env->SetBooleanField(s, loaded_all_participants_fieldID, loaded_all_participants_);
  { auto arr_tmp_ = jni::store_vector(env, recent_speakers_); if (arr_tmp_) { env->SetObjectField(s, recent_speakers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_my_video_enabled_fieldID, is_my_video_enabled_);
  env->SetBooleanField(s, is_my_video_paused_fieldID, is_my_video_paused_);
  env->SetBooleanField(s, can_enable_video_fieldID, can_enable_video_);
  env->SetBooleanField(s, mute_new_participants_fieldID, mute_new_participants_);
  env->SetBooleanField(s, can_toggle_mute_new_participants_fieldID, can_toggle_mute_new_participants_);
  env->SetIntField(s, record_duration_fieldID, record_duration_);
  env->SetBooleanField(s, is_video_recorded_fieldID, is_video_recorded_);
  env->SetIntField(s, duration_fieldID, duration_);
}

void groupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCall");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("scheduledStartDate", scheduled_start_date_);
    s.store_field("enabledStartNotification", enabled_start_notification_);
    s.store_field("isActive", is_active_);
    s.store_field("isVideoChat", is_video_chat_);
    s.store_field("isRtmpStream", is_rtmp_stream_);
    s.store_field("isJoined", is_joined_);
    s.store_field("needRejoin", need_rejoin_);
    s.store_field("isOwned", is_owned_);
    s.store_field("canBeManaged", can_be_managed_);
    s.store_field("participantCount", participant_count_);
    s.store_field("hasHiddenListeners", has_hidden_listeners_);
    s.store_field("loadedAllParticipants", loaded_all_participants_);
    { s.store_vector_begin("recentSpeakers", recent_speakers_.size()); for (const auto &_value : recent_speakers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isMyVideoEnabled", is_my_video_enabled_);
    s.store_field("isMyVideoPaused", is_my_video_paused_);
    s.store_field("canEnableVideo", can_enable_video_);
    s.store_field("muteNewParticipants", mute_new_participants_);
    s.store_field("canToggleMuteNewParticipants", can_toggle_mute_new_participants_);
    s.store_field("recordDuration", record_duration_);
    s.store_field("isVideoRecorded", is_video_recorded_);
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void groupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCall").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    scheduled_start_date_fieldID = jni::get_field_id(env, Class, "scheduledStartDate", "I");
    enabled_start_notification_fieldID = jni::get_field_id(env, Class, "enabledStartNotification", "Z");
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    is_video_chat_fieldID = jni::get_field_id(env, Class, "isVideoChat", "Z");
    is_rtmp_stream_fieldID = jni::get_field_id(env, Class, "isRtmpStream", "Z");
    is_joined_fieldID = jni::get_field_id(env, Class, "isJoined", "Z");
    need_rejoin_fieldID = jni::get_field_id(env, Class, "needRejoin", "Z");
    is_owned_fieldID = jni::get_field_id(env, Class, "isOwned", "Z");
    can_be_managed_fieldID = jni::get_field_id(env, Class, "canBeManaged", "Z");
    participant_count_fieldID = jni::get_field_id(env, Class, "participantCount", "I");
    has_hidden_listeners_fieldID = jni::get_field_id(env, Class, "hasHiddenListeners", "Z");
    loaded_all_participants_fieldID = jni::get_field_id(env, Class, "loadedAllParticipants", "Z");
    recent_speakers_fieldID = jni::get_field_id(env, Class, "recentSpeakers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$GroupCallRecentSpeaker;").c_str());
    is_my_video_enabled_fieldID = jni::get_field_id(env, Class, "isMyVideoEnabled", "Z");
    is_my_video_paused_fieldID = jni::get_field_id(env, Class, "isMyVideoPaused", "Z");
    can_enable_video_fieldID = jni::get_field_id(env, Class, "canEnableVideo", "Z");
    mute_new_participants_fieldID = jni::get_field_id(env, Class, "muteNewParticipants", "Z");
    can_toggle_mute_new_participants_fieldID = jni::get_field_id(env, Class, "canToggleMuteNewParticipants", "Z");
    record_duration_fieldID = jni::get_field_id(env, Class, "recordDuration", "I");
    is_video_recorded_fieldID = jni::get_field_id(env, Class, "isVideoRecorded", "Z");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    return true;
  }();
  (void)is_inited;
}

jclass GroupCallDataChannel::Class;

object_ptr<GroupCallDataChannel> GroupCallDataChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case groupCallDataChannelMain::ID:
      return groupCallDataChannelMain::fetch(env, p);
    case groupCallDataChannelScreenSharing::ID:
      return groupCallDataChannelScreenSharing::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GroupCallDataChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallDataChannel").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallDataChannelMain::Class;

groupCallDataChannelMain::groupCallDataChannelMain() {
}

const std::int32_t groupCallDataChannelMain::ID;

object_ptr<GroupCallDataChannel> groupCallDataChannelMain::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<groupCallDataChannelMain>();
}

void groupCallDataChannelMain::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallDataChannelMain");
    s.store_class_end();
  }
}

void groupCallDataChannelMain::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallDataChannelMain").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallDataChannelScreenSharing::Class;

groupCallDataChannelScreenSharing::groupCallDataChannelScreenSharing() {
}

const std::int32_t groupCallDataChannelScreenSharing::ID;

object_ptr<GroupCallDataChannel> groupCallDataChannelScreenSharing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<groupCallDataChannelScreenSharing>();
}

void groupCallDataChannelScreenSharing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallDataChannelScreenSharing");
    s.store_class_end();
  }
}

void groupCallDataChannelScreenSharing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallDataChannelScreenSharing").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallId::Class;
jfieldID groupCallId::id_fieldID;

groupCallId::groupCallId()
  : id_()
{}

groupCallId::groupCallId(int32 id_)
  : id_(id_)
{}

const std::int32_t groupCallId::ID;

object_ptr<groupCallId> groupCallId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallId> res = make_object<groupCallId>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  return res;
}

void groupCallId::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
}

void groupCallId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallId");
    s.store_field("id", id_);
    s.store_class_end();
  }
}

void groupCallId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallId").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    return true;
  }();
  (void)is_inited;
}

jclass groupCallInfo::Class;
jfieldID groupCallInfo::group_call_id_fieldID;
jfieldID groupCallInfo::join_payload_fieldID;

groupCallInfo::groupCallInfo()
  : group_call_id_()
  , join_payload_()
{}

groupCallInfo::groupCallInfo(int32 group_call_id_, string const &join_payload_)
  : group_call_id_(group_call_id_)
  , join_payload_(join_payload_)
{}

const std::int32_t groupCallInfo::ID;

object_ptr<groupCallInfo> groupCallInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallInfo> res = make_object<groupCallInfo>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->join_payload_ = jni::fetch_string(env, p, res->join_payload_fieldID);
  return res;
}

void groupCallInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  { jstring nextString = jni::to_jstring(env, join_payload_); if (nextString) { env->SetObjectField(s, join_payload_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void groupCallInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallInfo");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("joinPayload", join_payload_);
    s.store_class_end();
  }
}

void groupCallInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallInfo").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    join_payload_fieldID = jni::get_field_id(env, Class, "joinPayload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass groupCallJoinParameters::Class;
jfieldID groupCallJoinParameters::audio_source_id_fieldID;
jfieldID groupCallJoinParameters::payload_fieldID;
jfieldID groupCallJoinParameters::is_muted_fieldID;
jfieldID groupCallJoinParameters::is_my_video_enabled_fieldID;

groupCallJoinParameters::groupCallJoinParameters()
  : audio_source_id_()
  , payload_()
  , is_muted_()
  , is_my_video_enabled_()
{}

groupCallJoinParameters::groupCallJoinParameters(int32 audio_source_id_, string const &payload_, bool is_muted_, bool is_my_video_enabled_)
  : audio_source_id_(audio_source_id_)
  , payload_(payload_)
  , is_muted_(is_muted_)
  , is_my_video_enabled_(is_my_video_enabled_)
{}

const std::int32_t groupCallJoinParameters::ID;

object_ptr<groupCallJoinParameters> groupCallJoinParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallJoinParameters> res = make_object<groupCallJoinParameters>();
  res->audio_source_id_ = env->GetIntField(p, res->audio_source_id_fieldID);
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  res->is_muted_ = (env->GetBooleanField(p, res->is_muted_fieldID) != 0);
  res->is_my_video_enabled_ = (env->GetBooleanField(p, res->is_my_video_enabled_fieldID) != 0);
  return res;
}

void groupCallJoinParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallJoinParameters");
    s.store_field("audioSourceId", audio_source_id_);
    s.store_field("payload", payload_);
    s.store_field("isMuted", is_muted_);
    s.store_field("isMyVideoEnabled", is_my_video_enabled_);
    s.store_class_end();
  }
}

void groupCallJoinParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallJoinParameters").c_str());
    audio_source_id_fieldID = jni::get_field_id(env, Class, "audioSourceId", "I");
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    is_muted_fieldID = jni::get_field_id(env, Class, "isMuted", "Z");
    is_my_video_enabled_fieldID = jni::get_field_id(env, Class, "isMyVideoEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass groupCallParticipant::Class;
jfieldID groupCallParticipant::participant_id_fieldID;
jfieldID groupCallParticipant::audio_source_id_fieldID;
jfieldID groupCallParticipant::screen_sharing_audio_source_id_fieldID;
jfieldID groupCallParticipant::video_info_fieldID;
jfieldID groupCallParticipant::screen_sharing_video_info_fieldID;
jfieldID groupCallParticipant::bio_fieldID;
jfieldID groupCallParticipant::is_current_user_fieldID;
jfieldID groupCallParticipant::is_speaking_fieldID;
jfieldID groupCallParticipant::is_hand_raised_fieldID;
jfieldID groupCallParticipant::can_be_muted_for_all_users_fieldID;
jfieldID groupCallParticipant::can_be_unmuted_for_all_users_fieldID;
jfieldID groupCallParticipant::can_be_muted_for_current_user_fieldID;
jfieldID groupCallParticipant::can_be_unmuted_for_current_user_fieldID;
jfieldID groupCallParticipant::is_muted_for_all_users_fieldID;
jfieldID groupCallParticipant::is_muted_for_current_user_fieldID;
jfieldID groupCallParticipant::can_unmute_self_fieldID;
jfieldID groupCallParticipant::volume_level_fieldID;
jfieldID groupCallParticipant::order_fieldID;

groupCallParticipant::groupCallParticipant()
  : participant_id_()
  , audio_source_id_()
  , screen_sharing_audio_source_id_()
  , video_info_()
  , screen_sharing_video_info_()
  , bio_()
  , is_current_user_()
  , is_speaking_()
  , is_hand_raised_()
  , can_be_muted_for_all_users_()
  , can_be_unmuted_for_all_users_()
  , can_be_muted_for_current_user_()
  , can_be_unmuted_for_current_user_()
  , is_muted_for_all_users_()
  , is_muted_for_current_user_()
  , can_unmute_self_()
  , volume_level_()
  , order_()
{}

groupCallParticipant::groupCallParticipant(object_ptr<MessageSender> &&participant_id_, int32 audio_source_id_, int32 screen_sharing_audio_source_id_, object_ptr<groupCallParticipantVideoInfo> &&video_info_, object_ptr<groupCallParticipantVideoInfo> &&screen_sharing_video_info_, string const &bio_, bool is_current_user_, bool is_speaking_, bool is_hand_raised_, bool can_be_muted_for_all_users_, bool can_be_unmuted_for_all_users_, bool can_be_muted_for_current_user_, bool can_be_unmuted_for_current_user_, bool is_muted_for_all_users_, bool is_muted_for_current_user_, bool can_unmute_self_, int32 volume_level_, string const &order_)
  : participant_id_(std::move(participant_id_))
  , audio_source_id_(audio_source_id_)
  , screen_sharing_audio_source_id_(screen_sharing_audio_source_id_)
  , video_info_(std::move(video_info_))
  , screen_sharing_video_info_(std::move(screen_sharing_video_info_))
  , bio_(bio_)
  , is_current_user_(is_current_user_)
  , is_speaking_(is_speaking_)
  , is_hand_raised_(is_hand_raised_)
  , can_be_muted_for_all_users_(can_be_muted_for_all_users_)
  , can_be_unmuted_for_all_users_(can_be_unmuted_for_all_users_)
  , can_be_muted_for_current_user_(can_be_muted_for_current_user_)
  , can_be_unmuted_for_current_user_(can_be_unmuted_for_current_user_)
  , is_muted_for_all_users_(is_muted_for_all_users_)
  , is_muted_for_current_user_(is_muted_for_current_user_)
  , can_unmute_self_(can_unmute_self_)
  , volume_level_(volume_level_)
  , order_(order_)
{}

const std::int32_t groupCallParticipant::ID;

object_ptr<groupCallParticipant> groupCallParticipant::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallParticipant> res = make_object<groupCallParticipant>();
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->audio_source_id_ = env->GetIntField(p, res->audio_source_id_fieldID);
  res->screen_sharing_audio_source_id_ = env->GetIntField(p, res->screen_sharing_audio_source_id_fieldID);
  res->video_info_ = jni::fetch_tl_object<groupCallParticipantVideoInfo>(env, jni::fetch_object(env, p, res->video_info_fieldID));
  res->screen_sharing_video_info_ = jni::fetch_tl_object<groupCallParticipantVideoInfo>(env, jni::fetch_object(env, p, res->screen_sharing_video_info_fieldID));
  res->bio_ = jni::fetch_string(env, p, res->bio_fieldID);
  res->is_current_user_ = (env->GetBooleanField(p, res->is_current_user_fieldID) != 0);
  res->is_speaking_ = (env->GetBooleanField(p, res->is_speaking_fieldID) != 0);
  res->is_hand_raised_ = (env->GetBooleanField(p, res->is_hand_raised_fieldID) != 0);
  res->can_be_muted_for_all_users_ = (env->GetBooleanField(p, res->can_be_muted_for_all_users_fieldID) != 0);
  res->can_be_unmuted_for_all_users_ = (env->GetBooleanField(p, res->can_be_unmuted_for_all_users_fieldID) != 0);
  res->can_be_muted_for_current_user_ = (env->GetBooleanField(p, res->can_be_muted_for_current_user_fieldID) != 0);
  res->can_be_unmuted_for_current_user_ = (env->GetBooleanField(p, res->can_be_unmuted_for_current_user_fieldID) != 0);
  res->is_muted_for_all_users_ = (env->GetBooleanField(p, res->is_muted_for_all_users_fieldID) != 0);
  res->is_muted_for_current_user_ = (env->GetBooleanField(p, res->is_muted_for_current_user_fieldID) != 0);
  res->can_unmute_self_ = (env->GetBooleanField(p, res->can_unmute_self_fieldID) != 0);
  res->volume_level_ = env->GetIntField(p, res->volume_level_fieldID);
  res->order_ = jni::fetch_string(env, p, res->order_fieldID);
  return res;
}

void groupCallParticipant::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (participant_id_ != nullptr) { jobject next; participant_id_->store(env, next); if (next) { env->SetObjectField(s, participant_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, audio_source_id_fieldID, audio_source_id_);
  env->SetIntField(s, screen_sharing_audio_source_id_fieldID, screen_sharing_audio_source_id_);
  if (video_info_ != nullptr) { jobject next; video_info_->store(env, next); if (next) { env->SetObjectField(s, video_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (screen_sharing_video_info_ != nullptr) { jobject next; screen_sharing_video_info_->store(env, next); if (next) { env->SetObjectField(s, screen_sharing_video_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, bio_); if (nextString) { env->SetObjectField(s, bio_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_current_user_fieldID, is_current_user_);
  env->SetBooleanField(s, is_speaking_fieldID, is_speaking_);
  env->SetBooleanField(s, is_hand_raised_fieldID, is_hand_raised_);
  env->SetBooleanField(s, can_be_muted_for_all_users_fieldID, can_be_muted_for_all_users_);
  env->SetBooleanField(s, can_be_unmuted_for_all_users_fieldID, can_be_unmuted_for_all_users_);
  env->SetBooleanField(s, can_be_muted_for_current_user_fieldID, can_be_muted_for_current_user_);
  env->SetBooleanField(s, can_be_unmuted_for_current_user_fieldID, can_be_unmuted_for_current_user_);
  env->SetBooleanField(s, is_muted_for_all_users_fieldID, is_muted_for_all_users_);
  env->SetBooleanField(s, is_muted_for_current_user_fieldID, is_muted_for_current_user_);
  env->SetBooleanField(s, can_unmute_self_fieldID, can_unmute_self_);
  env->SetIntField(s, volume_level_fieldID, volume_level_);
  { jstring nextString = jni::to_jstring(env, order_); if (nextString) { env->SetObjectField(s, order_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void groupCallParticipant::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallParticipant");
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("audioSourceId", audio_source_id_);
    s.store_field("screenSharingAudioSourceId", screen_sharing_audio_source_id_);
    s.store_object_field("videoInfo", static_cast<const BaseObject *>(video_info_.get()));
    s.store_object_field("screenSharingVideoInfo", static_cast<const BaseObject *>(screen_sharing_video_info_.get()));
    s.store_field("bio", bio_);
    s.store_field("isCurrentUser", is_current_user_);
    s.store_field("isSpeaking", is_speaking_);
    s.store_field("isHandRaised", is_hand_raised_);
    s.store_field("canBeMutedForAllUsers", can_be_muted_for_all_users_);
    s.store_field("canBeUnmutedForAllUsers", can_be_unmuted_for_all_users_);
    s.store_field("canBeMutedForCurrentUser", can_be_muted_for_current_user_);
    s.store_field("canBeUnmutedForCurrentUser", can_be_unmuted_for_current_user_);
    s.store_field("isMutedForAllUsers", is_muted_for_all_users_);
    s.store_field("isMutedForCurrentUser", is_muted_for_current_user_);
    s.store_field("canUnmuteSelf", can_unmute_self_);
    s.store_field("volumeLevel", volume_level_);
    s.store_field("order", order_);
    s.store_class_end();
  }
}

void groupCallParticipant::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallParticipant").c_str());
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    audio_source_id_fieldID = jni::get_field_id(env, Class, "audioSourceId", "I");
    screen_sharing_audio_source_id_fieldID = jni::get_field_id(env, Class, "screenSharingAudioSourceId", "I");
    video_info_fieldID = jni::get_field_id(env, Class, "videoInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallParticipantVideoInfo;").c_str());
    screen_sharing_video_info_fieldID = jni::get_field_id(env, Class, "screenSharingVideoInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallParticipantVideoInfo;").c_str());
    bio_fieldID = jni::get_field_id(env, Class, "bio", "Ljava/lang/String;");
    is_current_user_fieldID = jni::get_field_id(env, Class, "isCurrentUser", "Z");
    is_speaking_fieldID = jni::get_field_id(env, Class, "isSpeaking", "Z");
    is_hand_raised_fieldID = jni::get_field_id(env, Class, "isHandRaised", "Z");
    can_be_muted_for_all_users_fieldID = jni::get_field_id(env, Class, "canBeMutedForAllUsers", "Z");
    can_be_unmuted_for_all_users_fieldID = jni::get_field_id(env, Class, "canBeUnmutedForAllUsers", "Z");
    can_be_muted_for_current_user_fieldID = jni::get_field_id(env, Class, "canBeMutedForCurrentUser", "Z");
    can_be_unmuted_for_current_user_fieldID = jni::get_field_id(env, Class, "canBeUnmutedForCurrentUser", "Z");
    is_muted_for_all_users_fieldID = jni::get_field_id(env, Class, "isMutedForAllUsers", "Z");
    is_muted_for_current_user_fieldID = jni::get_field_id(env, Class, "isMutedForCurrentUser", "Z");
    can_unmute_self_fieldID = jni::get_field_id(env, Class, "canUnmuteSelf", "Z");
    volume_level_fieldID = jni::get_field_id(env, Class, "volumeLevel", "I");
    order_fieldID = jni::get_field_id(env, Class, "order", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass groupCallParticipantVideoInfo::Class;
jfieldID groupCallParticipantVideoInfo::source_groups_fieldID;
jfieldID groupCallParticipantVideoInfo::endpoint_id_fieldID;
jfieldID groupCallParticipantVideoInfo::is_paused_fieldID;

groupCallParticipantVideoInfo::groupCallParticipantVideoInfo()
  : source_groups_()
  , endpoint_id_()
  , is_paused_()
{}

groupCallParticipantVideoInfo::groupCallParticipantVideoInfo(array<object_ptr<groupCallVideoSourceGroup>> &&source_groups_, string const &endpoint_id_, bool is_paused_)
  : source_groups_(std::move(source_groups_))
  , endpoint_id_(endpoint_id_)
  , is_paused_(is_paused_)
{}

const std::int32_t groupCallParticipantVideoInfo::ID;

object_ptr<groupCallParticipantVideoInfo> groupCallParticipantVideoInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallParticipantVideoInfo> res = make_object<groupCallParticipantVideoInfo>();
  res->source_groups_ = jni::FetchVector<groupCallVideoSourceGroup>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->source_groups_fieldID));
  res->endpoint_id_ = jni::fetch_string(env, p, res->endpoint_id_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  return res;
}

void groupCallParticipantVideoInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, source_groups_); if (arr_tmp_) { env->SetObjectField(s, source_groups_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, endpoint_id_); if (nextString) { env->SetObjectField(s, endpoint_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_paused_fieldID, is_paused_);
}

void groupCallParticipantVideoInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallParticipantVideoInfo");
    { s.store_vector_begin("sourceGroups", source_groups_.size()); for (const auto &_value : source_groups_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("endpointId", endpoint_id_);
    s.store_field("isPaused", is_paused_);
    s.store_class_end();
  }
}

void groupCallParticipantVideoInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallParticipantVideoInfo").c_str());
    source_groups_fieldID = jni::get_field_id(env, Class, "sourceGroups", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$GroupCallVideoSourceGroup;").c_str());
    endpoint_id_fieldID = jni::get_field_id(env, Class, "endpointId", "Ljava/lang/String;");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass groupCallParticipants::Class;
jfieldID groupCallParticipants::total_count_fieldID;
jfieldID groupCallParticipants::participant_ids_fieldID;

groupCallParticipants::groupCallParticipants()
  : total_count_()
  , participant_ids_()
{}

groupCallParticipants::groupCallParticipants(int32 total_count_, array<object_ptr<MessageSender>> &&participant_ids_)
  : total_count_(total_count_)
  , participant_ids_(std::move(participant_ids_))
{}

const std::int32_t groupCallParticipants::ID;

object_ptr<groupCallParticipants> groupCallParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallParticipants> res = make_object<groupCallParticipants>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->participant_ids_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->participant_ids_fieldID));
  return res;
}

void groupCallParticipants::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, participant_ids_); if (arr_tmp_) { env->SetObjectField(s, participant_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void groupCallParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallParticipants");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("participantIds", participant_ids_.size()); for (const auto &_value : participant_ids_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void groupCallParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallParticipants").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    participant_ids_fieldID = jni::get_field_id(env, Class, "participantIds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallRecentSpeaker::Class;
jfieldID groupCallRecentSpeaker::participant_id_fieldID;
jfieldID groupCallRecentSpeaker::is_speaking_fieldID;

groupCallRecentSpeaker::groupCallRecentSpeaker()
  : participant_id_()
  , is_speaking_()
{}

groupCallRecentSpeaker::groupCallRecentSpeaker(object_ptr<MessageSender> &&participant_id_, bool is_speaking_)
  : participant_id_(std::move(participant_id_))
  , is_speaking_(is_speaking_)
{}

const std::int32_t groupCallRecentSpeaker::ID;

object_ptr<groupCallRecentSpeaker> groupCallRecentSpeaker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallRecentSpeaker> res = make_object<groupCallRecentSpeaker>();
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->is_speaking_ = (env->GetBooleanField(p, res->is_speaking_fieldID) != 0);
  return res;
}

void groupCallRecentSpeaker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (participant_id_ != nullptr) { jobject next; participant_id_->store(env, next); if (next) { env->SetObjectField(s, participant_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_speaking_fieldID, is_speaking_);
}

void groupCallRecentSpeaker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallRecentSpeaker");
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("isSpeaking", is_speaking_);
    s.store_class_end();
  }
}

void groupCallRecentSpeaker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallRecentSpeaker").c_str());
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_speaking_fieldID = jni::get_field_id(env, Class, "isSpeaking", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass GroupCallVideoQuality::Class;

object_ptr<GroupCallVideoQuality> GroupCallVideoQuality::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case groupCallVideoQualityThumbnail::ID:
      return groupCallVideoQualityThumbnail::fetch(env, p);
    case groupCallVideoQualityMedium::ID:
      return groupCallVideoQualityMedium::fetch(env, p);
    case groupCallVideoQualityFull::ID:
      return groupCallVideoQualityFull::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void GroupCallVideoQuality::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallVideoQuality").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallVideoQualityThumbnail::Class;

groupCallVideoQualityThumbnail::groupCallVideoQualityThumbnail() {
}

const std::int32_t groupCallVideoQualityThumbnail::ID;

object_ptr<GroupCallVideoQuality> groupCallVideoQualityThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<groupCallVideoQualityThumbnail>();
}

void groupCallVideoQualityThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallVideoQualityThumbnail");
    s.store_class_end();
  }
}

void groupCallVideoQualityThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallVideoQualityThumbnail").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallVideoQualityMedium::Class;

groupCallVideoQualityMedium::groupCallVideoQualityMedium() {
}

const std::int32_t groupCallVideoQualityMedium::ID;

object_ptr<GroupCallVideoQuality> groupCallVideoQualityMedium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<groupCallVideoQualityMedium>();
}

void groupCallVideoQualityMedium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallVideoQualityMedium");
    s.store_class_end();
  }
}

void groupCallVideoQualityMedium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallVideoQualityMedium").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallVideoQualityFull::Class;

groupCallVideoQualityFull::groupCallVideoQualityFull() {
}

const std::int32_t groupCallVideoQualityFull::ID;

object_ptr<GroupCallVideoQuality> groupCallVideoQualityFull::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<groupCallVideoQualityFull>();
}

void groupCallVideoQualityFull::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallVideoQualityFull");
    s.store_class_end();
  }
}

void groupCallVideoQualityFull::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallVideoQualityFull").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass groupCallVideoSourceGroup::Class;
jfieldID groupCallVideoSourceGroup::semantics_fieldID;
jfieldID groupCallVideoSourceGroup::source_ids_fieldID;

groupCallVideoSourceGroup::groupCallVideoSourceGroup()
  : semantics_()
  , source_ids_()
{}

groupCallVideoSourceGroup::groupCallVideoSourceGroup(string const &semantics_, array<int32> &&source_ids_)
  : semantics_(semantics_)
  , source_ids_(std::move(source_ids_))
{}

const std::int32_t groupCallVideoSourceGroup::ID;

object_ptr<groupCallVideoSourceGroup> groupCallVideoSourceGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<groupCallVideoSourceGroup> res = make_object<groupCallVideoSourceGroup>();
  res->semantics_ = jni::fetch_string(env, p, res->semantics_fieldID);
  res->source_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->source_ids_fieldID));
  return res;
}

void groupCallVideoSourceGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, semantics_); if (nextString) { env->SetObjectField(s, semantics_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, source_ids_); if (arr_tmp_) { env->SetObjectField(s, source_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void groupCallVideoSourceGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GroupCallVideoSourceGroup");
    s.store_field("semantics", semantics_);
    { s.store_vector_begin("sourceIds", source_ids_.size()); for (const auto &_value : source_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void groupCallVideoSourceGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GroupCallVideoSourceGroup").c_str());
    semantics_fieldID = jni::get_field_id(env, Class, "semantics", "Ljava/lang/String;");
    source_ids_fieldID = jni::get_field_id(env, Class, "sourceIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass hashtags::Class;
jfieldID hashtags::hashtags_fieldID;

hashtags::hashtags()
  : hashtags_()
{}

hashtags::hashtags(array<string> &&hashtags_)
  : hashtags_(std::move(hashtags_))
{}

const std::int32_t hashtags::ID;

object_ptr<hashtags> hashtags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<hashtags> res = make_object<hashtags>();
  res->hashtags_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->hashtags_fieldID));
  return res;
}

void hashtags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, hashtags_); if (arr_tmp_) { env->SetObjectField(s, hashtags_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void hashtags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Hashtags");
    { s.store_vector_begin("hashtags", hashtags_.size()); for (const auto &_value : hashtags_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void hashtags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Hashtags").c_str());
    hashtags_fieldID = jni::get_field_id(env, Class, "hashtags", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass httpUrl::Class;
jfieldID httpUrl::url_fieldID;

httpUrl::httpUrl()
  : url_()
{}

httpUrl::httpUrl(string const &url_)
  : url_(url_)
{}

const std::int32_t httpUrl::ID;

object_ptr<httpUrl> httpUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<httpUrl> res = make_object<httpUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void httpUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void httpUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "HttpUrl");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void httpUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$HttpUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass identityDocument::Class;
jfieldID identityDocument::number_fieldID;
jfieldID identityDocument::expiration_date_fieldID;
jfieldID identityDocument::front_side_fieldID;
jfieldID identityDocument::reverse_side_fieldID;
jfieldID identityDocument::selfie_fieldID;
jfieldID identityDocument::translation_fieldID;

identityDocument::identityDocument()
  : number_()
  , expiration_date_()
  , front_side_()
  , reverse_side_()
  , selfie_()
  , translation_()
{}

identityDocument::identityDocument(string const &number_, object_ptr<date> &&expiration_date_, object_ptr<datedFile> &&front_side_, object_ptr<datedFile> &&reverse_side_, object_ptr<datedFile> &&selfie_, array<object_ptr<datedFile>> &&translation_)
  : number_(number_)
  , expiration_date_(std::move(expiration_date_))
  , front_side_(std::move(front_side_))
  , reverse_side_(std::move(reverse_side_))
  , selfie_(std::move(selfie_))
  , translation_(std::move(translation_))
{}

const std::int32_t identityDocument::ID;

object_ptr<identityDocument> identityDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<identityDocument> res = make_object<identityDocument>();
  res->number_ = jni::fetch_string(env, p, res->number_fieldID);
  res->expiration_date_ = jni::fetch_tl_object<date>(env, jni::fetch_object(env, p, res->expiration_date_fieldID));
  res->front_side_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->front_side_fieldID));
  res->reverse_side_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->reverse_side_fieldID));
  res->selfie_ = jni::fetch_tl_object<datedFile>(env, jni::fetch_object(env, p, res->selfie_fieldID));
  res->translation_ = jni::FetchVector<datedFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->translation_fieldID));
  return res;
}

void identityDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, number_); if (nextString) { env->SetObjectField(s, number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (expiration_date_ != nullptr) { jobject next; expiration_date_->store(env, next); if (next) { env->SetObjectField(s, expiration_date_fieldID, next); env->DeleteLocalRef(next); } }
  if (front_side_ != nullptr) { jobject next; front_side_->store(env, next); if (next) { env->SetObjectField(s, front_side_fieldID, next); env->DeleteLocalRef(next); } }
  if (reverse_side_ != nullptr) { jobject next; reverse_side_->store(env, next); if (next) { env->SetObjectField(s, reverse_side_fieldID, next); env->DeleteLocalRef(next); } }
  if (selfie_ != nullptr) { jobject next; selfie_->store(env, next); if (next) { env->SetObjectField(s, selfie_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, translation_); if (arr_tmp_) { env->SetObjectField(s, translation_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void identityDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "IdentityDocument");
    s.store_field("number", number_);
    s.store_object_field("expirationDate", static_cast<const BaseObject *>(expiration_date_.get()));
    s.store_object_field("frontSide", static_cast<const BaseObject *>(front_side_.get()));
    s.store_object_field("reverseSide", static_cast<const BaseObject *>(reverse_side_.get()));
    s.store_object_field("selfie", static_cast<const BaseObject *>(selfie_.get()));
    { s.store_vector_begin("translation", translation_.size()); for (const auto &_value : translation_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void identityDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$IdentityDocument").c_str());
    number_fieldID = jni::get_field_id(env, Class, "number", "Ljava/lang/String;");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Date;").c_str());
    front_side_fieldID = jni::get_field_id(env, Class, "frontSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    reverse_side_fieldID = jni::get_field_id(env, Class, "reverseSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    selfie_fieldID = jni::get_field_id(env, Class, "selfie", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    translation_fieldID = jni::get_field_id(env, Class, "translation", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass importedContacts::Class;
jfieldID importedContacts::user_ids_fieldID;
jfieldID importedContacts::importer_count_fieldID;

importedContacts::importedContacts()
  : user_ids_()
  , importer_count_()
{}

importedContacts::importedContacts(array<int53> &&user_ids_, array<int32> &&importer_count_)
  : user_ids_(std::move(user_ids_))
  , importer_count_(std::move(importer_count_))
{}

const std::int32_t importedContacts::ID;

object_ptr<importedContacts> importedContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<importedContacts> res = make_object<importedContacts>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  res->importer_count_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->importer_count_fieldID));
  return res;
}

void importedContacts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, importer_count_); if (arr_tmp_) { env->SetObjectField(s, importer_count_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void importedContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ImportedContacts");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("importerCount", importer_count_.size()); for (const auto &_value : importer_count_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void importedContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ImportedContacts").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    importer_count_fieldID = jni::get_field_id(env, Class, "importerCount", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButton::Class;
jfieldID inlineKeyboardButton::text_fieldID;
jfieldID inlineKeyboardButton::type_fieldID;

inlineKeyboardButton::inlineKeyboardButton()
  : text_()
  , type_()
{}

inlineKeyboardButton::inlineKeyboardButton(string const &text_, object_ptr<InlineKeyboardButtonType> &&type_)
  : text_(text_)
  , type_(std::move(type_))
{}

const std::int32_t inlineKeyboardButton::ID;

object_ptr<inlineKeyboardButton> inlineKeyboardButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButton> res = make_object<inlineKeyboardButton>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->type_ = jni::fetch_tl_object<InlineKeyboardButtonType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void inlineKeyboardButton::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineKeyboardButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButton");
    s.store_field("text", text_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void inlineKeyboardButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButton").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InlineKeyboardButtonType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InlineKeyboardButtonType::Class;

object_ptr<InlineKeyboardButtonType> InlineKeyboardButtonType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inlineKeyboardButtonTypeUrl::ID:
      return inlineKeyboardButtonTypeUrl::fetch(env, p);
    case inlineKeyboardButtonTypeLoginUrl::ID:
      return inlineKeyboardButtonTypeLoginUrl::fetch(env, p);
    case inlineKeyboardButtonTypeWebApp::ID:
      return inlineKeyboardButtonTypeWebApp::fetch(env, p);
    case inlineKeyboardButtonTypeCallback::ID:
      return inlineKeyboardButtonTypeCallback::fetch(env, p);
    case inlineKeyboardButtonTypeCallbackWithPassword::ID:
      return inlineKeyboardButtonTypeCallbackWithPassword::fetch(env, p);
    case inlineKeyboardButtonTypeCallbackGame::ID:
      return inlineKeyboardButtonTypeCallbackGame::fetch(env, p);
    case inlineKeyboardButtonTypeSwitchInline::ID:
      return inlineKeyboardButtonTypeSwitchInline::fetch(env, p);
    case inlineKeyboardButtonTypeBuy::ID:
      return inlineKeyboardButtonTypeBuy::fetch(env, p);
    case inlineKeyboardButtonTypeUser::ID:
      return inlineKeyboardButtonTypeUser::fetch(env, p);
    case inlineKeyboardButtonTypeCopyText::ID:
      return inlineKeyboardButtonTypeCopyText::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InlineKeyboardButtonType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeUrl::Class;
jfieldID inlineKeyboardButtonTypeUrl::url_fieldID;

inlineKeyboardButtonTypeUrl::inlineKeyboardButtonTypeUrl()
  : url_()
{}

inlineKeyboardButtonTypeUrl::inlineKeyboardButtonTypeUrl(string const &url_)
  : url_(url_)
{}

const std::int32_t inlineKeyboardButtonTypeUrl::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeUrl> res = make_object<inlineKeyboardButtonTypeUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void inlineKeyboardButtonTypeUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineKeyboardButtonTypeUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeUrl");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeLoginUrl::Class;
jfieldID inlineKeyboardButtonTypeLoginUrl::url_fieldID;
jfieldID inlineKeyboardButtonTypeLoginUrl::id_fieldID;
jfieldID inlineKeyboardButtonTypeLoginUrl::forward_text_fieldID;

inlineKeyboardButtonTypeLoginUrl::inlineKeyboardButtonTypeLoginUrl()
  : url_()
  , id_()
  , forward_text_()
{}

inlineKeyboardButtonTypeLoginUrl::inlineKeyboardButtonTypeLoginUrl(string const &url_, int53 id_, string const &forward_text_)
  : url_(url_)
  , id_(id_)
  , forward_text_(forward_text_)
{}

const std::int32_t inlineKeyboardButtonTypeLoginUrl::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeLoginUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeLoginUrl> res = make_object<inlineKeyboardButtonTypeLoginUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->forward_text_ = jni::fetch_string(env, p, res->forward_text_fieldID);
  return std::move(res);
}

void inlineKeyboardButtonTypeLoginUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, forward_text_); if (nextString) { env->SetObjectField(s, forward_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineKeyboardButtonTypeLoginUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeLoginUrl");
    s.store_field("url", url_);
    s.store_field("id", id_);
    s.store_field("forwardText", forward_text_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeLoginUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeLoginUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    forward_text_fieldID = jni::get_field_id(env, Class, "forwardText", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeWebApp::Class;
jfieldID inlineKeyboardButtonTypeWebApp::url_fieldID;

inlineKeyboardButtonTypeWebApp::inlineKeyboardButtonTypeWebApp()
  : url_()
{}

inlineKeyboardButtonTypeWebApp::inlineKeyboardButtonTypeWebApp(string const &url_)
  : url_(url_)
{}

const std::int32_t inlineKeyboardButtonTypeWebApp::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeWebApp> res = make_object<inlineKeyboardButtonTypeWebApp>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void inlineKeyboardButtonTypeWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineKeyboardButtonTypeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeWebApp");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeWebApp").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeCallback::Class;
jfieldID inlineKeyboardButtonTypeCallback::data_fieldID;

inlineKeyboardButtonTypeCallback::inlineKeyboardButtonTypeCallback()
  : data_()
{}

inlineKeyboardButtonTypeCallback::inlineKeyboardButtonTypeCallback(bytes const &data_)
  : data_(std::move(data_))
{}

const std::int32_t inlineKeyboardButtonTypeCallback::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeCallback::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeCallback> res = make_object<inlineKeyboardButtonTypeCallback>();
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return std::move(res);
}

void inlineKeyboardButtonTypeCallback::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void inlineKeyboardButtonTypeCallback::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeCallback");
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeCallback::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeCallback").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeCallbackWithPassword::Class;
jfieldID inlineKeyboardButtonTypeCallbackWithPassword::data_fieldID;

inlineKeyboardButtonTypeCallbackWithPassword::inlineKeyboardButtonTypeCallbackWithPassword()
  : data_()
{}

inlineKeyboardButtonTypeCallbackWithPassword::inlineKeyboardButtonTypeCallbackWithPassword(bytes const &data_)
  : data_(std::move(data_))
{}

const std::int32_t inlineKeyboardButtonTypeCallbackWithPassword::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeCallbackWithPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeCallbackWithPassword> res = make_object<inlineKeyboardButtonTypeCallbackWithPassword>();
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return std::move(res);
}

void inlineKeyboardButtonTypeCallbackWithPassword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void inlineKeyboardButtonTypeCallbackWithPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeCallbackWithPassword");
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeCallbackWithPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeCallbackWithPassword").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeCallbackGame::Class;

inlineKeyboardButtonTypeCallbackGame::inlineKeyboardButtonTypeCallbackGame() {
}

const std::int32_t inlineKeyboardButtonTypeCallbackGame::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeCallbackGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inlineKeyboardButtonTypeCallbackGame>();
}

void inlineKeyboardButtonTypeCallbackGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inlineKeyboardButtonTypeCallbackGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeCallbackGame");
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeCallbackGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeCallbackGame").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeSwitchInline::Class;
jfieldID inlineKeyboardButtonTypeSwitchInline::query_fieldID;
jfieldID inlineKeyboardButtonTypeSwitchInline::target_chat_fieldID;

inlineKeyboardButtonTypeSwitchInline::inlineKeyboardButtonTypeSwitchInline()
  : query_()
  , target_chat_()
{}

inlineKeyboardButtonTypeSwitchInline::inlineKeyboardButtonTypeSwitchInline(string const &query_, object_ptr<TargetChat> &&target_chat_)
  : query_(query_)
  , target_chat_(std::move(target_chat_))
{}

const std::int32_t inlineKeyboardButtonTypeSwitchInline::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeSwitchInline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeSwitchInline> res = make_object<inlineKeyboardButtonTypeSwitchInline>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->target_chat_ = jni::fetch_tl_object<TargetChat>(env, jni::fetch_object(env, p, res->target_chat_fieldID));
  return std::move(res);
}

void inlineKeyboardButtonTypeSwitchInline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, query_); if (nextString) { env->SetObjectField(s, query_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (target_chat_ != nullptr) { jobject next; target_chat_->store(env, next); if (next) { env->SetObjectField(s, target_chat_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineKeyboardButtonTypeSwitchInline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeSwitchInline");
    s.store_field("query", query_);
    s.store_object_field("targetChat", static_cast<const BaseObject *>(target_chat_.get()));
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeSwitchInline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeSwitchInline").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    target_chat_fieldID = jni::get_field_id(env, Class, "targetChat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TargetChat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeBuy::Class;

inlineKeyboardButtonTypeBuy::inlineKeyboardButtonTypeBuy() {
}

const std::int32_t inlineKeyboardButtonTypeBuy::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeBuy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inlineKeyboardButtonTypeBuy>();
}

void inlineKeyboardButtonTypeBuy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inlineKeyboardButtonTypeBuy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeBuy");
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeBuy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeBuy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeUser::Class;
jfieldID inlineKeyboardButtonTypeUser::user_id_fieldID;

inlineKeyboardButtonTypeUser::inlineKeyboardButtonTypeUser()
  : user_id_()
{}

inlineKeyboardButtonTypeUser::inlineKeyboardButtonTypeUser(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t inlineKeyboardButtonTypeUser::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeUser> res = make_object<inlineKeyboardButtonTypeUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void inlineKeyboardButtonTypeUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void inlineKeyboardButtonTypeUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeUser");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inlineKeyboardButtonTypeCopyText::Class;
jfieldID inlineKeyboardButtonTypeCopyText::text_fieldID;

inlineKeyboardButtonTypeCopyText::inlineKeyboardButtonTypeCopyText()
  : text_()
{}

inlineKeyboardButtonTypeCopyText::inlineKeyboardButtonTypeCopyText(string const &text_)
  : text_(text_)
{}

const std::int32_t inlineKeyboardButtonTypeCopyText::ID;

object_ptr<InlineKeyboardButtonType> inlineKeyboardButtonTypeCopyText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineKeyboardButtonTypeCopyText> res = make_object<inlineKeyboardButtonTypeCopyText>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return std::move(res);
}

void inlineKeyboardButtonTypeCopyText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineKeyboardButtonTypeCopyText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineKeyboardButtonTypeCopyText");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void inlineKeyboardButtonTypeCopyText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineKeyboardButtonTypeCopyText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass InlineQueryResult::Class;

object_ptr<InlineQueryResult> InlineQueryResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inlineQueryResultArticle::ID:
      return inlineQueryResultArticle::fetch(env, p);
    case inlineQueryResultContact::ID:
      return inlineQueryResultContact::fetch(env, p);
    case inlineQueryResultLocation::ID:
      return inlineQueryResultLocation::fetch(env, p);
    case inlineQueryResultVenue::ID:
      return inlineQueryResultVenue::fetch(env, p);
    case inlineQueryResultGame::ID:
      return inlineQueryResultGame::fetch(env, p);
    case inlineQueryResultAnimation::ID:
      return inlineQueryResultAnimation::fetch(env, p);
    case inlineQueryResultAudio::ID:
      return inlineQueryResultAudio::fetch(env, p);
    case inlineQueryResultDocument::ID:
      return inlineQueryResultDocument::fetch(env, p);
    case inlineQueryResultPhoto::ID:
      return inlineQueryResultPhoto::fetch(env, p);
    case inlineQueryResultSticker::ID:
      return inlineQueryResultSticker::fetch(env, p);
    case inlineQueryResultVideo::ID:
      return inlineQueryResultVideo::fetch(env, p);
    case inlineQueryResultVoiceNote::ID:
      return inlineQueryResultVoiceNote::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InlineQueryResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultArticle::Class;
jfieldID inlineQueryResultArticle::id_fieldID;
jfieldID inlineQueryResultArticle::url_fieldID;
jfieldID inlineQueryResultArticle::title_fieldID;
jfieldID inlineQueryResultArticle::description_fieldID;
jfieldID inlineQueryResultArticle::thumbnail_fieldID;

inlineQueryResultArticle::inlineQueryResultArticle()
  : id_()
  , url_()
  , title_()
  , description_()
  , thumbnail_()
{}

inlineQueryResultArticle::inlineQueryResultArticle(string const &id_, string const &url_, string const &title_, string const &description_, object_ptr<thumbnail> &&thumbnail_)
  : id_(id_)
  , url_(url_)
  , title_(title_)
  , description_(description_)
  , thumbnail_(std::move(thumbnail_))
{}

const std::int32_t inlineQueryResultArticle::ID;

object_ptr<InlineQueryResult> inlineQueryResultArticle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultArticle> res = make_object<inlineQueryResultArticle>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  return std::move(res);
}

void inlineQueryResultArticle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultArticle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultArticle");
    s.store_field("id", id_);
    s.store_field("url", url_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultArticle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultArticle").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultContact::Class;
jfieldID inlineQueryResultContact::id_fieldID;
jfieldID inlineQueryResultContact::contact_fieldID;
jfieldID inlineQueryResultContact::thumbnail_fieldID;

inlineQueryResultContact::inlineQueryResultContact()
  : id_()
  , contact_()
  , thumbnail_()
{}

inlineQueryResultContact::inlineQueryResultContact(string const &id_, object_ptr<contact> &&contact_, object_ptr<thumbnail> &&thumbnail_)
  : id_(id_)
  , contact_(std::move(contact_))
  , thumbnail_(std::move(thumbnail_))
{}

const std::int32_t inlineQueryResultContact::ID;

object_ptr<InlineQueryResult> inlineQueryResultContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultContact> res = make_object<inlineQueryResultContact>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->contact_ = jni::fetch_tl_object<contact>(env, jni::fetch_object(env, p, res->contact_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  return std::move(res);
}

void inlineQueryResultContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (contact_ != nullptr) { jobject next; contact_->store(env, next); if (next) { env->SetObjectField(s, contact_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultContact");
    s.store_field("id", id_);
    s.store_object_field("contact", static_cast<const BaseObject *>(contact_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultContact").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    contact_fieldID = jni::get_field_id(env, Class, "contact", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultLocation::Class;
jfieldID inlineQueryResultLocation::id_fieldID;
jfieldID inlineQueryResultLocation::location_fieldID;
jfieldID inlineQueryResultLocation::title_fieldID;
jfieldID inlineQueryResultLocation::thumbnail_fieldID;

inlineQueryResultLocation::inlineQueryResultLocation()
  : id_()
  , location_()
  , title_()
  , thumbnail_()
{}

inlineQueryResultLocation::inlineQueryResultLocation(string const &id_, object_ptr<location> &&location_, string const &title_, object_ptr<thumbnail> &&thumbnail_)
  : id_(id_)
  , location_(std::move(location_))
  , title_(title_)
  , thumbnail_(std::move(thumbnail_))
{}

const std::int32_t inlineQueryResultLocation::ID;

object_ptr<InlineQueryResult> inlineQueryResultLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultLocation> res = make_object<inlineQueryResultLocation>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  return std::move(res);
}

void inlineQueryResultLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultLocation");
    s.store_field("id", id_);
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("title", title_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultLocation").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultVenue::Class;
jfieldID inlineQueryResultVenue::id_fieldID;
jfieldID inlineQueryResultVenue::venue_fieldID;
jfieldID inlineQueryResultVenue::thumbnail_fieldID;

inlineQueryResultVenue::inlineQueryResultVenue()
  : id_()
  , venue_()
  , thumbnail_()
{}

inlineQueryResultVenue::inlineQueryResultVenue(string const &id_, object_ptr<venue> &&venue_, object_ptr<thumbnail> &&thumbnail_)
  : id_(id_)
  , venue_(std::move(venue_))
  , thumbnail_(std::move(thumbnail_))
{}

const std::int32_t inlineQueryResultVenue::ID;

object_ptr<InlineQueryResult> inlineQueryResultVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultVenue> res = make_object<inlineQueryResultVenue>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->venue_ = jni::fetch_tl_object<venue>(env, jni::fetch_object(env, p, res->venue_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  return std::move(res);
}

void inlineQueryResultVenue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (venue_ != nullptr) { jobject next; venue_->store(env, next); if (next) { env->SetObjectField(s, venue_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultVenue");
    s.store_field("id", id_);
    s.store_object_field("venue", static_cast<const BaseObject *>(venue_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultVenue").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    venue_fieldID = jni::get_field_id(env, Class, "venue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Venue;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultGame::Class;
jfieldID inlineQueryResultGame::id_fieldID;
jfieldID inlineQueryResultGame::game_fieldID;

inlineQueryResultGame::inlineQueryResultGame()
  : id_()
  , game_()
{}

inlineQueryResultGame::inlineQueryResultGame(string const &id_, object_ptr<game> &&game_)
  : id_(id_)
  , game_(std::move(game_))
{}

const std::int32_t inlineQueryResultGame::ID;

object_ptr<InlineQueryResult> inlineQueryResultGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultGame> res = make_object<inlineQueryResultGame>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->game_ = jni::fetch_tl_object<game>(env, jni::fetch_object(env, p, res->game_fieldID));
  return std::move(res);
}

void inlineQueryResultGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (game_ != nullptr) { jobject next; game_->store(env, next); if (next) { env->SetObjectField(s, game_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultGame");
    s.store_field("id", id_);
    s.store_object_field("game", static_cast<const BaseObject *>(game_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultGame").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    game_fieldID = jni::get_field_id(env, Class, "game", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Game;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultAnimation::Class;
jfieldID inlineQueryResultAnimation::id_fieldID;
jfieldID inlineQueryResultAnimation::animation_fieldID;
jfieldID inlineQueryResultAnimation::title_fieldID;

inlineQueryResultAnimation::inlineQueryResultAnimation()
  : id_()
  , animation_()
  , title_()
{}

inlineQueryResultAnimation::inlineQueryResultAnimation(string const &id_, object_ptr<animation> &&animation_, string const &title_)
  : id_(id_)
  , animation_(std::move(animation_))
  , title_(title_)
{}

const std::int32_t inlineQueryResultAnimation::ID;

object_ptr<InlineQueryResult> inlineQueryResultAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultAnimation> res = make_object<inlineQueryResultAnimation>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void inlineQueryResultAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultAnimation");
    s.store_field("id", id_);
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void inlineQueryResultAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultAnimation").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultAudio::Class;
jfieldID inlineQueryResultAudio::id_fieldID;
jfieldID inlineQueryResultAudio::audio_fieldID;

inlineQueryResultAudio::inlineQueryResultAudio()
  : id_()
  , audio_()
{}

inlineQueryResultAudio::inlineQueryResultAudio(string const &id_, object_ptr<audio> &&audio_)
  : id_(id_)
  , audio_(std::move(audio_))
{}

const std::int32_t inlineQueryResultAudio::ID;

object_ptr<InlineQueryResult> inlineQueryResultAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultAudio> res = make_object<inlineQueryResultAudio>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->audio_ = jni::fetch_tl_object<audio>(env, jni::fetch_object(env, p, res->audio_fieldID));
  return std::move(res);
}

void inlineQueryResultAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultAudio");
    s.store_field("id", id_);
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultAudio").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Audio;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultDocument::Class;
jfieldID inlineQueryResultDocument::id_fieldID;
jfieldID inlineQueryResultDocument::document_fieldID;
jfieldID inlineQueryResultDocument::title_fieldID;
jfieldID inlineQueryResultDocument::description_fieldID;

inlineQueryResultDocument::inlineQueryResultDocument()
  : id_()
  , document_()
  , title_()
  , description_()
{}

inlineQueryResultDocument::inlineQueryResultDocument(string const &id_, object_ptr<document> &&document_, string const &title_, string const &description_)
  : id_(id_)
  , document_(std::move(document_))
  , title_(title_)
  , description_(description_)
{}

const std::int32_t inlineQueryResultDocument::ID;

object_ptr<InlineQueryResult> inlineQueryResultDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultDocument> res = make_object<inlineQueryResultDocument>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return std::move(res);
}

void inlineQueryResultDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultDocument");
    s.store_field("id", id_);
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

void inlineQueryResultDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultDocument").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultPhoto::Class;
jfieldID inlineQueryResultPhoto::id_fieldID;
jfieldID inlineQueryResultPhoto::photo_fieldID;
jfieldID inlineQueryResultPhoto::title_fieldID;
jfieldID inlineQueryResultPhoto::description_fieldID;

inlineQueryResultPhoto::inlineQueryResultPhoto()
  : id_()
  , photo_()
  , title_()
  , description_()
{}

inlineQueryResultPhoto::inlineQueryResultPhoto(string const &id_, object_ptr<photo> &&photo_, string const &title_, string const &description_)
  : id_(id_)
  , photo_(std::move(photo_))
  , title_(title_)
  , description_(description_)
{}

const std::int32_t inlineQueryResultPhoto::ID;

object_ptr<InlineQueryResult> inlineQueryResultPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultPhoto> res = make_object<inlineQueryResultPhoto>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return std::move(res);
}

void inlineQueryResultPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultPhoto");
    s.store_field("id", id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

void inlineQueryResultPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultPhoto").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultSticker::Class;
jfieldID inlineQueryResultSticker::id_fieldID;
jfieldID inlineQueryResultSticker::sticker_fieldID;

inlineQueryResultSticker::inlineQueryResultSticker()
  : id_()
  , sticker_()
{}

inlineQueryResultSticker::inlineQueryResultSticker(string const &id_, object_ptr<sticker> &&sticker_)
  : id_(id_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t inlineQueryResultSticker::ID;

object_ptr<InlineQueryResult> inlineQueryResultSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultSticker> res = make_object<inlineQueryResultSticker>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void inlineQueryResultSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultSticker");
    s.store_field("id", id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultSticker").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultVideo::Class;
jfieldID inlineQueryResultVideo::id_fieldID;
jfieldID inlineQueryResultVideo::video_fieldID;
jfieldID inlineQueryResultVideo::title_fieldID;
jfieldID inlineQueryResultVideo::description_fieldID;

inlineQueryResultVideo::inlineQueryResultVideo()
  : id_()
  , video_()
  , title_()
  , description_()
{}

inlineQueryResultVideo::inlineQueryResultVideo(string const &id_, object_ptr<video> &&video_, string const &title_, string const &description_)
  : id_(id_)
  , video_(std::move(video_))
  , title_(title_)
  , description_(description_)
{}

const std::int32_t inlineQueryResultVideo::ID;

object_ptr<InlineQueryResult> inlineQueryResultVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultVideo> res = make_object<inlineQueryResultVideo>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return std::move(res);
}

void inlineQueryResultVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultVideo");
    s.store_field("id", id_);
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

void inlineQueryResultVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultVideo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultVoiceNote::Class;
jfieldID inlineQueryResultVoiceNote::id_fieldID;
jfieldID inlineQueryResultVoiceNote::voice_note_fieldID;
jfieldID inlineQueryResultVoiceNote::title_fieldID;

inlineQueryResultVoiceNote::inlineQueryResultVoiceNote()
  : id_()
  , voice_note_()
  , title_()
{}

inlineQueryResultVoiceNote::inlineQueryResultVoiceNote(string const &id_, object_ptr<voiceNote> &&voice_note_, string const &title_)
  : id_(id_)
  , voice_note_(std::move(voice_note_))
  , title_(title_)
{}

const std::int32_t inlineQueryResultVoiceNote::ID;

object_ptr<InlineQueryResult> inlineQueryResultVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultVoiceNote> res = make_object<inlineQueryResultVoiceNote>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->voice_note_ = jni::fetch_tl_object<voiceNote>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void inlineQueryResultVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultVoiceNote");
    s.store_field("id", id_);
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void inlineQueryResultVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultVoiceNote").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VoiceNote;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResults::Class;
jfieldID inlineQueryResults::inline_query_id_fieldID;
jfieldID inlineQueryResults::button_fieldID;
jfieldID inlineQueryResults::results_fieldID;
jfieldID inlineQueryResults::next_offset_fieldID;

inlineQueryResults::inlineQueryResults()
  : inline_query_id_()
  , button_()
  , results_()
  , next_offset_()
{}

inlineQueryResults::inlineQueryResults(int64 inline_query_id_, object_ptr<inlineQueryResultsButton> &&button_, array<object_ptr<InlineQueryResult>> &&results_, string const &next_offset_)
  : inline_query_id_(inline_query_id_)
  , button_(std::move(button_))
  , results_(std::move(results_))
  , next_offset_(next_offset_)
{}

const std::int32_t inlineQueryResults::ID;

object_ptr<inlineQueryResults> inlineQueryResults::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResults> res = make_object<inlineQueryResults>();
  res->inline_query_id_ = env->GetLongField(p, res->inline_query_id_fieldID);
  res->button_ = jni::fetch_tl_object<inlineQueryResultsButton>(env, jni::fetch_object(env, p, res->button_fieldID));
  res->results_ = jni::FetchVector<InlineQueryResult>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->results_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void inlineQueryResults::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, inline_query_id_fieldID, inline_query_id_);
  if (button_ != nullptr) { jobject next; button_->store(env, next); if (next) { env->SetObjectField(s, button_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, results_); if (arr_tmp_) { env->SetObjectField(s, results_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResults::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResults");
    s.store_field("inlineQueryId", inline_query_id_);
    s.store_object_field("button", static_cast<const BaseObject *>(button_.get()));
    { s.store_vector_begin("results", results_.size()); for (const auto &_value : results_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void inlineQueryResults::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResults").c_str());
    inline_query_id_fieldID = jni::get_field_id(env, Class, "inlineQueryId", "J");
    button_fieldID = jni::get_field_id(env, Class, "button", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InlineQueryResultsButton;").c_str());
    results_fieldID = jni::get_field_id(env, Class, "results", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InlineQueryResult;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultsButton::Class;
jfieldID inlineQueryResultsButton::text_fieldID;
jfieldID inlineQueryResultsButton::type_fieldID;

inlineQueryResultsButton::inlineQueryResultsButton()
  : text_()
  , type_()
{}

inlineQueryResultsButton::inlineQueryResultsButton(string const &text_, object_ptr<InlineQueryResultsButtonType> &&type_)
  : text_(text_)
  , type_(std::move(type_))
{}

const std::int32_t inlineQueryResultsButton::ID;

object_ptr<inlineQueryResultsButton> inlineQueryResultsButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultsButton> res = make_object<inlineQueryResultsButton>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->type_ = jni::fetch_tl_object<InlineQueryResultsButtonType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void inlineQueryResultsButton::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void inlineQueryResultsButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultsButton");
    s.store_field("text", text_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void inlineQueryResultsButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultsButton").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InlineQueryResultsButtonType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InlineQueryResultsButtonType::Class;

object_ptr<InlineQueryResultsButtonType> InlineQueryResultsButtonType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inlineQueryResultsButtonTypeStartBot::ID:
      return inlineQueryResultsButtonTypeStartBot::fetch(env, p);
    case inlineQueryResultsButtonTypeWebApp::ID:
      return inlineQueryResultsButtonTypeWebApp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InlineQueryResultsButtonType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultsButtonType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultsButtonTypeStartBot::Class;
jfieldID inlineQueryResultsButtonTypeStartBot::parameter_fieldID;

inlineQueryResultsButtonTypeStartBot::inlineQueryResultsButtonTypeStartBot()
  : parameter_()
{}

inlineQueryResultsButtonTypeStartBot::inlineQueryResultsButtonTypeStartBot(string const &parameter_)
  : parameter_(parameter_)
{}

const std::int32_t inlineQueryResultsButtonTypeStartBot::ID;

object_ptr<InlineQueryResultsButtonType> inlineQueryResultsButtonTypeStartBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultsButtonTypeStartBot> res = make_object<inlineQueryResultsButtonTypeStartBot>();
  res->parameter_ = jni::fetch_string(env, p, res->parameter_fieldID);
  return std::move(res);
}

void inlineQueryResultsButtonTypeStartBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, parameter_); if (nextString) { env->SetObjectField(s, parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultsButtonTypeStartBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultsButtonTypeStartBot");
    s.store_field("parameter", parameter_);
    s.store_class_end();
  }
}

void inlineQueryResultsButtonTypeStartBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultsButtonTypeStartBot").c_str());
    parameter_fieldID = jni::get_field_id(env, Class, "parameter", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inlineQueryResultsButtonTypeWebApp::Class;
jfieldID inlineQueryResultsButtonTypeWebApp::url_fieldID;

inlineQueryResultsButtonTypeWebApp::inlineQueryResultsButtonTypeWebApp()
  : url_()
{}

inlineQueryResultsButtonTypeWebApp::inlineQueryResultsButtonTypeWebApp(string const &url_)
  : url_(url_)
{}

const std::int32_t inlineQueryResultsButtonTypeWebApp::ID;

object_ptr<InlineQueryResultsButtonType> inlineQueryResultsButtonTypeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inlineQueryResultsButtonTypeWebApp> res = make_object<inlineQueryResultsButtonTypeWebApp>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void inlineQueryResultsButtonTypeWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inlineQueryResultsButtonTypeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InlineQueryResultsButtonTypeWebApp");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void inlineQueryResultsButtonTypeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InlineQueryResultsButtonTypeWebApp").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass InputBackground::Class;

object_ptr<InputBackground> InputBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputBackgroundLocal::ID:
      return inputBackgroundLocal::fetch(env, p);
    case inputBackgroundRemote::ID:
      return inputBackgroundRemote::fetch(env, p);
    case inputBackgroundPrevious::ID:
      return inputBackgroundPrevious::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBackground").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputBackgroundLocal::Class;
jfieldID inputBackgroundLocal::background_fieldID;

inputBackgroundLocal::inputBackgroundLocal()
  : background_()
{}

inputBackgroundLocal::inputBackgroundLocal(object_ptr<InputFile> &&background_)
  : background_(std::move(background_))
{}

const std::int32_t inputBackgroundLocal::ID;

object_ptr<InputBackground> inputBackgroundLocal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputBackgroundLocal> res = make_object<inputBackgroundLocal>();
  res->background_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->background_fieldID));
  return std::move(res);
}

void inputBackgroundLocal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputBackgroundLocal");
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_class_end();
  }
}

void inputBackgroundLocal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBackgroundLocal").c_str());
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputBackgroundRemote::Class;
jfieldID inputBackgroundRemote::background_id_fieldID;

inputBackgroundRemote::inputBackgroundRemote()
  : background_id_()
{}

inputBackgroundRemote::inputBackgroundRemote(int64 background_id_)
  : background_id_(background_id_)
{}

const std::int32_t inputBackgroundRemote::ID;

object_ptr<InputBackground> inputBackgroundRemote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputBackgroundRemote> res = make_object<inputBackgroundRemote>();
  res->background_id_ = env->GetLongField(p, res->background_id_fieldID);
  return std::move(res);
}

void inputBackgroundRemote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputBackgroundRemote");
    s.store_field("backgroundId", background_id_);
    s.store_class_end();
  }
}

void inputBackgroundRemote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBackgroundRemote").c_str());
    background_id_fieldID = jni::get_field_id(env, Class, "backgroundId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputBackgroundPrevious::Class;
jfieldID inputBackgroundPrevious::message_id_fieldID;

inputBackgroundPrevious::inputBackgroundPrevious()
  : message_id_()
{}

inputBackgroundPrevious::inputBackgroundPrevious(int53 message_id_)
  : message_id_(message_id_)
{}

const std::int32_t inputBackgroundPrevious::ID;

object_ptr<InputBackground> inputBackgroundPrevious::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputBackgroundPrevious> res = make_object<inputBackgroundPrevious>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void inputBackgroundPrevious::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputBackgroundPrevious");
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void inputBackgroundPrevious::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBackgroundPrevious").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputBusinessChatLink::Class;
jfieldID inputBusinessChatLink::text_fieldID;
jfieldID inputBusinessChatLink::title_fieldID;

inputBusinessChatLink::inputBusinessChatLink()
  : text_()
  , title_()
{}

inputBusinessChatLink::inputBusinessChatLink(object_ptr<formattedText> &&text_, string const &title_)
  : text_(std::move(text_))
  , title_(title_)
{}

const std::int32_t inputBusinessChatLink::ID;

object_ptr<inputBusinessChatLink> inputBusinessChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputBusinessChatLink> res = make_object<inputBusinessChatLink>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void inputBusinessChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputBusinessChatLink");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void inputBusinessChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBusinessChatLink").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputBusinessStartPage::Class;
jfieldID inputBusinessStartPage::title_fieldID;
jfieldID inputBusinessStartPage::message_fieldID;
jfieldID inputBusinessStartPage::sticker_fieldID;

inputBusinessStartPage::inputBusinessStartPage()
  : title_()
  , message_()
  , sticker_()
{}

inputBusinessStartPage::inputBusinessStartPage(string const &title_, string const &message_, object_ptr<InputFile> &&sticker_)
  : title_(title_)
  , message_(message_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t inputBusinessStartPage::ID;

object_ptr<inputBusinessStartPage> inputBusinessStartPage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputBusinessStartPage> res = make_object<inputBusinessStartPage>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->message_ = jni::fetch_string(env, p, res->message_fieldID);
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void inputBusinessStartPage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputBusinessStartPage");
    s.store_field("title", title_);
    s.store_field("message", message_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void inputBusinessStartPage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputBusinessStartPage").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    message_fieldID = jni::get_field_id(env, Class, "message", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputChatPhoto::Class;

object_ptr<InputChatPhoto> InputChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputChatPhotoPrevious::ID:
      return inputChatPhotoPrevious::fetch(env, p);
    case inputChatPhotoStatic::ID:
      return inputChatPhotoStatic::fetch(env, p);
    case inputChatPhotoAnimation::ID:
      return inputChatPhotoAnimation::fetch(env, p);
    case inputChatPhotoSticker::ID:
      return inputChatPhotoSticker::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputChatPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputChatPhotoPrevious::Class;
jfieldID inputChatPhotoPrevious::chat_photo_id_fieldID;

inputChatPhotoPrevious::inputChatPhotoPrevious()
  : chat_photo_id_()
{}

inputChatPhotoPrevious::inputChatPhotoPrevious(int64 chat_photo_id_)
  : chat_photo_id_(chat_photo_id_)
{}

const std::int32_t inputChatPhotoPrevious::ID;

object_ptr<InputChatPhoto> inputChatPhotoPrevious::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputChatPhotoPrevious> res = make_object<inputChatPhotoPrevious>();
  res->chat_photo_id_ = env->GetLongField(p, res->chat_photo_id_fieldID);
  return std::move(res);
}

void inputChatPhotoPrevious::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputChatPhotoPrevious");
    s.store_field("chatPhotoId", chat_photo_id_);
    s.store_class_end();
  }
}

void inputChatPhotoPrevious::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputChatPhotoPrevious").c_str());
    chat_photo_id_fieldID = jni::get_field_id(env, Class, "chatPhotoId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputChatPhotoStatic::Class;
jfieldID inputChatPhotoStatic::photo_fieldID;

inputChatPhotoStatic::inputChatPhotoStatic()
  : photo_()
{}

inputChatPhotoStatic::inputChatPhotoStatic(object_ptr<InputFile> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t inputChatPhotoStatic::ID;

object_ptr<InputChatPhoto> inputChatPhotoStatic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputChatPhotoStatic> res = make_object<inputChatPhotoStatic>();
  res->photo_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void inputChatPhotoStatic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputChatPhotoStatic");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void inputChatPhotoStatic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputChatPhotoStatic").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputChatPhotoAnimation::Class;
jfieldID inputChatPhotoAnimation::animation_fieldID;
jfieldID inputChatPhotoAnimation::main_frame_timestamp_fieldID;

inputChatPhotoAnimation::inputChatPhotoAnimation()
  : animation_()
  , main_frame_timestamp_()
{}

inputChatPhotoAnimation::inputChatPhotoAnimation(object_ptr<InputFile> &&animation_, double main_frame_timestamp_)
  : animation_(std::move(animation_))
  , main_frame_timestamp_(main_frame_timestamp_)
{}

const std::int32_t inputChatPhotoAnimation::ID;

object_ptr<InputChatPhoto> inputChatPhotoAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputChatPhotoAnimation> res = make_object<inputChatPhotoAnimation>();
  res->animation_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->main_frame_timestamp_ = env->GetDoubleField(p, res->main_frame_timestamp_fieldID);
  return std::move(res);
}

void inputChatPhotoAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputChatPhotoAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_field("mainFrameTimestamp", main_frame_timestamp_);
    s.store_class_end();
  }
}

void inputChatPhotoAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputChatPhotoAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    main_frame_timestamp_fieldID = jni::get_field_id(env, Class, "mainFrameTimestamp", "D");
    return true;
  }();
  (void)is_inited;
}

jclass inputChatPhotoSticker::Class;
jfieldID inputChatPhotoSticker::sticker_fieldID;

inputChatPhotoSticker::inputChatPhotoSticker()
  : sticker_()
{}

inputChatPhotoSticker::inputChatPhotoSticker(object_ptr<chatPhotoSticker> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t inputChatPhotoSticker::ID;

object_ptr<InputChatPhoto> inputChatPhotoSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputChatPhotoSticker> res = make_object<inputChatPhotoSticker>();
  res->sticker_ = jni::fetch_tl_object<chatPhotoSticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void inputChatPhotoSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputChatPhotoSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void inputChatPhotoSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputChatPhotoSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoSticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputCredentials::Class;

object_ptr<InputCredentials> InputCredentials::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputCredentialsSaved::ID:
      return inputCredentialsSaved::fetch(env, p);
    case inputCredentialsNew::ID:
      return inputCredentialsNew::fetch(env, p);
    case inputCredentialsApplePay::ID:
      return inputCredentialsApplePay::fetch(env, p);
    case inputCredentialsGooglePay::ID:
      return inputCredentialsGooglePay::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputCredentials::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputCredentials").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputCredentialsSaved::Class;
jfieldID inputCredentialsSaved::saved_credentials_id_fieldID;

inputCredentialsSaved::inputCredentialsSaved()
  : saved_credentials_id_()
{}

inputCredentialsSaved::inputCredentialsSaved(string const &saved_credentials_id_)
  : saved_credentials_id_(saved_credentials_id_)
{}

const std::int32_t inputCredentialsSaved::ID;

object_ptr<InputCredentials> inputCredentialsSaved::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputCredentialsSaved> res = make_object<inputCredentialsSaved>();
  res->saved_credentials_id_ = jni::fetch_string(env, p, res->saved_credentials_id_fieldID);
  return std::move(res);
}

void inputCredentialsSaved::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputCredentialsSaved");
    s.store_field("savedCredentialsId", saved_credentials_id_);
    s.store_class_end();
  }
}

void inputCredentialsSaved::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputCredentialsSaved").c_str());
    saved_credentials_id_fieldID = jni::get_field_id(env, Class, "savedCredentialsId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputCredentialsNew::Class;
jfieldID inputCredentialsNew::data_fieldID;
jfieldID inputCredentialsNew::allow_save_fieldID;

inputCredentialsNew::inputCredentialsNew()
  : data_()
  , allow_save_()
{}

inputCredentialsNew::inputCredentialsNew(string const &data_, bool allow_save_)
  : data_(data_)
  , allow_save_(allow_save_)
{}

const std::int32_t inputCredentialsNew::ID;

object_ptr<InputCredentials> inputCredentialsNew::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputCredentialsNew> res = make_object<inputCredentialsNew>();
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  res->allow_save_ = (env->GetBooleanField(p, res->allow_save_fieldID) != 0);
  return std::move(res);
}

void inputCredentialsNew::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputCredentialsNew");
    s.store_field("data", data_);
    s.store_field("allowSave", allow_save_);
    s.store_class_end();
  }
}

void inputCredentialsNew::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputCredentialsNew").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    allow_save_fieldID = jni::get_field_id(env, Class, "allowSave", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputCredentialsApplePay::Class;
jfieldID inputCredentialsApplePay::data_fieldID;

inputCredentialsApplePay::inputCredentialsApplePay()
  : data_()
{}

inputCredentialsApplePay::inputCredentialsApplePay(string const &data_)
  : data_(data_)
{}

const std::int32_t inputCredentialsApplePay::ID;

object_ptr<InputCredentials> inputCredentialsApplePay::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputCredentialsApplePay> res = make_object<inputCredentialsApplePay>();
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  return std::move(res);
}

void inputCredentialsApplePay::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputCredentialsApplePay");
    s.store_field("data", data_);
    s.store_class_end();
  }
}

void inputCredentialsApplePay::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputCredentialsApplePay").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputCredentialsGooglePay::Class;
jfieldID inputCredentialsGooglePay::data_fieldID;

inputCredentialsGooglePay::inputCredentialsGooglePay()
  : data_()
{}

inputCredentialsGooglePay::inputCredentialsGooglePay(string const &data_)
  : data_(data_)
{}

const std::int32_t inputCredentialsGooglePay::ID;

object_ptr<InputCredentials> inputCredentialsGooglePay::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputCredentialsGooglePay> res = make_object<inputCredentialsGooglePay>();
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  return std::move(res);
}

void inputCredentialsGooglePay::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputCredentialsGooglePay");
    s.store_field("data", data_);
    s.store_class_end();
  }
}

void inputCredentialsGooglePay::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputCredentialsGooglePay").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass InputFile::Class;

object_ptr<InputFile> InputFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputFileId::ID:
      return inputFileId::fetch(env, p);
    case inputFileRemote::ID:
      return inputFileRemote::fetch(env, p);
    case inputFileLocal::ID:
      return inputFileLocal::fetch(env, p);
    case inputFileGenerated::ID:
      return inputFileGenerated::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputFile").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputFileId::Class;
jfieldID inputFileId::id_fieldID;

inputFileId::inputFileId()
  : id_()
{}

inputFileId::inputFileId(int32 id_)
  : id_(id_)
{}

const std::int32_t inputFileId::ID;

object_ptr<InputFile> inputFileId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputFileId> res = make_object<inputFileId>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  return std::move(res);
}

void inputFileId::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
}

void inputFileId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputFileId");
    s.store_field("id", id_);
    s.store_class_end();
  }
}

void inputFileId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputFileId").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputFileRemote::Class;
jfieldID inputFileRemote::id_fieldID;

inputFileRemote::inputFileRemote()
  : id_()
{}

inputFileRemote::inputFileRemote(string const &id_)
  : id_(id_)
{}

const std::int32_t inputFileRemote::ID;

object_ptr<InputFile> inputFileRemote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputFileRemote> res = make_object<inputFileRemote>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  return std::move(res);
}

void inputFileRemote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inputFileRemote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputFileRemote");
    s.store_field("id", id_);
    s.store_class_end();
  }
}

void inputFileRemote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputFileRemote").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputFileLocal::Class;
jfieldID inputFileLocal::path_fieldID;

inputFileLocal::inputFileLocal()
  : path_()
{}

inputFileLocal::inputFileLocal(string const &path_)
  : path_(path_)
{}

const std::int32_t inputFileLocal::ID;

object_ptr<InputFile> inputFileLocal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputFileLocal> res = make_object<inputFileLocal>();
  res->path_ = jni::fetch_string(env, p, res->path_fieldID);
  return std::move(res);
}

void inputFileLocal::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, path_); if (nextString) { env->SetObjectField(s, path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inputFileLocal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputFileLocal");
    s.store_field("path", path_);
    s.store_class_end();
  }
}

void inputFileLocal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputFileLocal").c_str());
    path_fieldID = jni::get_field_id(env, Class, "path", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputFileGenerated::Class;
jfieldID inputFileGenerated::original_path_fieldID;
jfieldID inputFileGenerated::conversion_fieldID;
jfieldID inputFileGenerated::expected_size_fieldID;

inputFileGenerated::inputFileGenerated()
  : original_path_()
  , conversion_()
  , expected_size_()
{}

inputFileGenerated::inputFileGenerated(string const &original_path_, string const &conversion_, int53 expected_size_)
  : original_path_(original_path_)
  , conversion_(conversion_)
  , expected_size_(expected_size_)
{}

const std::int32_t inputFileGenerated::ID;

object_ptr<InputFile> inputFileGenerated::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputFileGenerated> res = make_object<inputFileGenerated>();
  res->original_path_ = jni::fetch_string(env, p, res->original_path_fieldID);
  res->conversion_ = jni::fetch_string(env, p, res->conversion_fieldID);
  res->expected_size_ = env->GetLongField(p, res->expected_size_fieldID);
  return std::move(res);
}

void inputFileGenerated::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, original_path_); if (nextString) { env->SetObjectField(s, original_path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, conversion_); if (nextString) { env->SetObjectField(s, conversion_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, expected_size_fieldID, expected_size_);
}

void inputFileGenerated::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputFileGenerated");
    s.store_field("originalPath", original_path_);
    s.store_field("conversion", conversion_);
    s.store_field("expectedSize", expected_size_);
    s.store_class_end();
  }
}

void inputFileGenerated::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputFileGenerated").c_str());
    original_path_fieldID = jni::get_field_id(env, Class, "originalPath", "Ljava/lang/String;");
    conversion_fieldID = jni::get_field_id(env, Class, "conversion", "Ljava/lang/String;");
    expected_size_fieldID = jni::get_field_id(env, Class, "expectedSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass InputGroupCall::Class;

object_ptr<InputGroupCall> InputGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputGroupCallLink::ID:
      return inputGroupCallLink::fetch(env, p);
    case inputGroupCallMessage::ID:
      return inputGroupCallMessage::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputGroupCall").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputGroupCallLink::Class;
jfieldID inputGroupCallLink::link_fieldID;

inputGroupCallLink::inputGroupCallLink()
  : link_()
{}

inputGroupCallLink::inputGroupCallLink(string const &link_)
  : link_(link_)
{}

const std::int32_t inputGroupCallLink::ID;

object_ptr<InputGroupCall> inputGroupCallLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputGroupCallLink> res = make_object<inputGroupCallLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return std::move(res);
}

void inputGroupCallLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputGroupCallLink");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

void inputGroupCallLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputGroupCallLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputGroupCallMessage::Class;
jfieldID inputGroupCallMessage::chat_id_fieldID;
jfieldID inputGroupCallMessage::message_id_fieldID;

inputGroupCallMessage::inputGroupCallMessage()
  : chat_id_()
  , message_id_()
{}

inputGroupCallMessage::inputGroupCallMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t inputGroupCallMessage::ID;

object_ptr<InputGroupCall> inputGroupCallMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputGroupCallMessage> res = make_object<inputGroupCallMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void inputGroupCallMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputGroupCallMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void inputGroupCallMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputGroupCallMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputIdentityDocument::Class;
jfieldID inputIdentityDocument::number_fieldID;
jfieldID inputIdentityDocument::expiration_date_fieldID;
jfieldID inputIdentityDocument::front_side_fieldID;
jfieldID inputIdentityDocument::reverse_side_fieldID;
jfieldID inputIdentityDocument::selfie_fieldID;
jfieldID inputIdentityDocument::translation_fieldID;

inputIdentityDocument::inputIdentityDocument()
  : number_()
  , expiration_date_()
  , front_side_()
  , reverse_side_()
  , selfie_()
  , translation_()
{}

inputIdentityDocument::inputIdentityDocument(string const &number_, object_ptr<date> &&expiration_date_, object_ptr<InputFile> &&front_side_, object_ptr<InputFile> &&reverse_side_, object_ptr<InputFile> &&selfie_, array<object_ptr<InputFile>> &&translation_)
  : number_(number_)
  , expiration_date_(std::move(expiration_date_))
  , front_side_(std::move(front_side_))
  , reverse_side_(std::move(reverse_side_))
  , selfie_(std::move(selfie_))
  , translation_(std::move(translation_))
{}

const std::int32_t inputIdentityDocument::ID;

object_ptr<inputIdentityDocument> inputIdentityDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputIdentityDocument> res = make_object<inputIdentityDocument>();
  res->number_ = jni::fetch_string(env, p, res->number_fieldID);
  res->expiration_date_ = jni::fetch_tl_object<date>(env, jni::fetch_object(env, p, res->expiration_date_fieldID));
  res->front_side_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->front_side_fieldID));
  res->reverse_side_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->reverse_side_fieldID));
  res->selfie_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->selfie_fieldID));
  res->translation_ = jni::FetchVector<InputFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->translation_fieldID));
  return res;
}

void inputIdentityDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputIdentityDocument");
    s.store_field("number", number_);
    s.store_object_field("expirationDate", static_cast<const BaseObject *>(expiration_date_.get()));
    s.store_object_field("frontSide", static_cast<const BaseObject *>(front_side_.get()));
    s.store_object_field("reverseSide", static_cast<const BaseObject *>(reverse_side_.get()));
    s.store_object_field("selfie", static_cast<const BaseObject *>(selfie_.get()));
    { s.store_vector_begin("translation", translation_.size()); for (const auto &_value : translation_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputIdentityDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputIdentityDocument").c_str());
    number_fieldID = jni::get_field_id(env, Class, "number", "Ljava/lang/String;");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Date;").c_str());
    front_side_fieldID = jni::get_field_id(env, Class, "frontSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    reverse_side_fieldID = jni::get_field_id(env, Class, "reverseSide", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    selfie_fieldID = jni::get_field_id(env, Class, "selfie", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    translation_fieldID = jni::get_field_id(env, Class, "translation", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputInlineQueryResult::Class;

object_ptr<InputInlineQueryResult> InputInlineQueryResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputInlineQueryResultAnimation::ID:
      return inputInlineQueryResultAnimation::fetch(env, p);
    case inputInlineQueryResultArticle::ID:
      return inputInlineQueryResultArticle::fetch(env, p);
    case inputInlineQueryResultAudio::ID:
      return inputInlineQueryResultAudio::fetch(env, p);
    case inputInlineQueryResultContact::ID:
      return inputInlineQueryResultContact::fetch(env, p);
    case inputInlineQueryResultDocument::ID:
      return inputInlineQueryResultDocument::fetch(env, p);
    case inputInlineQueryResultGame::ID:
      return inputInlineQueryResultGame::fetch(env, p);
    case inputInlineQueryResultLocation::ID:
      return inputInlineQueryResultLocation::fetch(env, p);
    case inputInlineQueryResultPhoto::ID:
      return inputInlineQueryResultPhoto::fetch(env, p);
    case inputInlineQueryResultSticker::ID:
      return inputInlineQueryResultSticker::fetch(env, p);
    case inputInlineQueryResultVenue::ID:
      return inputInlineQueryResultVenue::fetch(env, p);
    case inputInlineQueryResultVideo::ID:
      return inputInlineQueryResultVideo::fetch(env, p);
    case inputInlineQueryResultVoiceNote::ID:
      return inputInlineQueryResultVoiceNote::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputInlineQueryResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultAnimation::Class;
jfieldID inputInlineQueryResultAnimation::id_fieldID;
jfieldID inputInlineQueryResultAnimation::title_fieldID;
jfieldID inputInlineQueryResultAnimation::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultAnimation::thumbnail_mime_type_fieldID;
jfieldID inputInlineQueryResultAnimation::video_url_fieldID;
jfieldID inputInlineQueryResultAnimation::video_mime_type_fieldID;
jfieldID inputInlineQueryResultAnimation::video_duration_fieldID;
jfieldID inputInlineQueryResultAnimation::video_width_fieldID;
jfieldID inputInlineQueryResultAnimation::video_height_fieldID;
jfieldID inputInlineQueryResultAnimation::reply_markup_fieldID;
jfieldID inputInlineQueryResultAnimation::input_message_content_fieldID;

inputInlineQueryResultAnimation::inputInlineQueryResultAnimation()
  : id_()
  , title_()
  , thumbnail_url_()
  , thumbnail_mime_type_()
  , video_url_()
  , video_mime_type_()
  , video_duration_()
  , video_width_()
  , video_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultAnimation::inputInlineQueryResultAnimation(string const &id_, string const &title_, string const &thumbnail_url_, string const &thumbnail_mime_type_, string const &video_url_, string const &video_mime_type_, int32 video_duration_, int32 video_width_, int32 video_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_mime_type_(thumbnail_mime_type_)
  , video_url_(video_url_)
  , video_mime_type_(video_mime_type_)
  , video_duration_(video_duration_)
  , video_width_(video_width_)
  , video_height_(video_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultAnimation::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultAnimation> res = make_object<inputInlineQueryResultAnimation>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_mime_type_ = jni::fetch_string(env, p, res->thumbnail_mime_type_fieldID);
  res->video_url_ = jni::fetch_string(env, p, res->video_url_fieldID);
  res->video_mime_type_ = jni::fetch_string(env, p, res->video_mime_type_fieldID);
  res->video_duration_ = env->GetIntField(p, res->video_duration_fieldID);
  res->video_width_ = env->GetIntField(p, res->video_width_fieldID);
  res->video_height_ = env->GetIntField(p, res->video_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultAnimation");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailMimeType", thumbnail_mime_type_);
    s.store_field("videoUrl", video_url_);
    s.store_field("videoMimeType", video_mime_type_);
    s.store_field("videoDuration", video_duration_);
    s.store_field("videoWidth", video_width_);
    s.store_field("videoHeight", video_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultAnimation").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_mime_type_fieldID = jni::get_field_id(env, Class, "thumbnailMimeType", "Ljava/lang/String;");
    video_url_fieldID = jni::get_field_id(env, Class, "videoUrl", "Ljava/lang/String;");
    video_mime_type_fieldID = jni::get_field_id(env, Class, "videoMimeType", "Ljava/lang/String;");
    video_duration_fieldID = jni::get_field_id(env, Class, "videoDuration", "I");
    video_width_fieldID = jni::get_field_id(env, Class, "videoWidth", "I");
    video_height_fieldID = jni::get_field_id(env, Class, "videoHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultArticle::Class;
jfieldID inputInlineQueryResultArticle::id_fieldID;
jfieldID inputInlineQueryResultArticle::url_fieldID;
jfieldID inputInlineQueryResultArticle::title_fieldID;
jfieldID inputInlineQueryResultArticle::description_fieldID;
jfieldID inputInlineQueryResultArticle::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultArticle::thumbnail_width_fieldID;
jfieldID inputInlineQueryResultArticle::thumbnail_height_fieldID;
jfieldID inputInlineQueryResultArticle::reply_markup_fieldID;
jfieldID inputInlineQueryResultArticle::input_message_content_fieldID;

inputInlineQueryResultArticle::inputInlineQueryResultArticle()
  : id_()
  , url_()
  , title_()
  , description_()
  , thumbnail_url_()
  , thumbnail_width_()
  , thumbnail_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultArticle::inputInlineQueryResultArticle(string const &id_, string const &url_, string const &title_, string const &description_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , url_(url_)
  , title_(title_)
  , description_(description_)
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_width_(thumbnail_width_)
  , thumbnail_height_(thumbnail_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultArticle::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultArticle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultArticle> res = make_object<inputInlineQueryResultArticle>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_width_ = env->GetIntField(p, res->thumbnail_width_fieldID);
  res->thumbnail_height_ = env->GetIntField(p, res->thumbnail_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultArticle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultArticle");
    s.store_field("id", id_);
    s.store_field("url", url_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailWidth", thumbnail_width_);
    s.store_field("thumbnailHeight", thumbnail_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultArticle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultArticle").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_width_fieldID = jni::get_field_id(env, Class, "thumbnailWidth", "I");
    thumbnail_height_fieldID = jni::get_field_id(env, Class, "thumbnailHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultAudio::Class;
jfieldID inputInlineQueryResultAudio::id_fieldID;
jfieldID inputInlineQueryResultAudio::title_fieldID;
jfieldID inputInlineQueryResultAudio::performer_fieldID;
jfieldID inputInlineQueryResultAudio::audio_url_fieldID;
jfieldID inputInlineQueryResultAudio::audio_duration_fieldID;
jfieldID inputInlineQueryResultAudio::reply_markup_fieldID;
jfieldID inputInlineQueryResultAudio::input_message_content_fieldID;

inputInlineQueryResultAudio::inputInlineQueryResultAudio()
  : id_()
  , title_()
  , performer_()
  , audio_url_()
  , audio_duration_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultAudio::inputInlineQueryResultAudio(string const &id_, string const &title_, string const &performer_, string const &audio_url_, int32 audio_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , performer_(performer_)
  , audio_url_(audio_url_)
  , audio_duration_(audio_duration_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultAudio::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultAudio> res = make_object<inputInlineQueryResultAudio>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->performer_ = jni::fetch_string(env, p, res->performer_fieldID);
  res->audio_url_ = jni::fetch_string(env, p, res->audio_url_fieldID);
  res->audio_duration_ = env->GetIntField(p, res->audio_duration_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultAudio");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("performer", performer_);
    s.store_field("audioUrl", audio_url_);
    s.store_field("audioDuration", audio_duration_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultAudio").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    performer_fieldID = jni::get_field_id(env, Class, "performer", "Ljava/lang/String;");
    audio_url_fieldID = jni::get_field_id(env, Class, "audioUrl", "Ljava/lang/String;");
    audio_duration_fieldID = jni::get_field_id(env, Class, "audioDuration", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultContact::Class;
jfieldID inputInlineQueryResultContact::id_fieldID;
jfieldID inputInlineQueryResultContact::contact_fieldID;
jfieldID inputInlineQueryResultContact::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultContact::thumbnail_width_fieldID;
jfieldID inputInlineQueryResultContact::thumbnail_height_fieldID;
jfieldID inputInlineQueryResultContact::reply_markup_fieldID;
jfieldID inputInlineQueryResultContact::input_message_content_fieldID;

inputInlineQueryResultContact::inputInlineQueryResultContact()
  : id_()
  , contact_()
  , thumbnail_url_()
  , thumbnail_width_()
  , thumbnail_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultContact::inputInlineQueryResultContact(string const &id_, object_ptr<contact> &&contact_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , contact_(std::move(contact_))
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_width_(thumbnail_width_)
  , thumbnail_height_(thumbnail_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultContact::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultContact> res = make_object<inputInlineQueryResultContact>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->contact_ = jni::fetch_tl_object<contact>(env, jni::fetch_object(env, p, res->contact_fieldID));
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_width_ = env->GetIntField(p, res->thumbnail_width_fieldID);
  res->thumbnail_height_ = env->GetIntField(p, res->thumbnail_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultContact");
    s.store_field("id", id_);
    s.store_object_field("contact", static_cast<const BaseObject *>(contact_.get()));
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailWidth", thumbnail_width_);
    s.store_field("thumbnailHeight", thumbnail_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultContact").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    contact_fieldID = jni::get_field_id(env, Class, "contact", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_width_fieldID = jni::get_field_id(env, Class, "thumbnailWidth", "I");
    thumbnail_height_fieldID = jni::get_field_id(env, Class, "thumbnailHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultDocument::Class;
jfieldID inputInlineQueryResultDocument::id_fieldID;
jfieldID inputInlineQueryResultDocument::title_fieldID;
jfieldID inputInlineQueryResultDocument::description_fieldID;
jfieldID inputInlineQueryResultDocument::document_url_fieldID;
jfieldID inputInlineQueryResultDocument::mime_type_fieldID;
jfieldID inputInlineQueryResultDocument::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultDocument::thumbnail_width_fieldID;
jfieldID inputInlineQueryResultDocument::thumbnail_height_fieldID;
jfieldID inputInlineQueryResultDocument::reply_markup_fieldID;
jfieldID inputInlineQueryResultDocument::input_message_content_fieldID;

inputInlineQueryResultDocument::inputInlineQueryResultDocument()
  : id_()
  , title_()
  , description_()
  , document_url_()
  , mime_type_()
  , thumbnail_url_()
  , thumbnail_width_()
  , thumbnail_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultDocument::inputInlineQueryResultDocument(string const &id_, string const &title_, string const &description_, string const &document_url_, string const &mime_type_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , description_(description_)
  , document_url_(document_url_)
  , mime_type_(mime_type_)
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_width_(thumbnail_width_)
  , thumbnail_height_(thumbnail_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultDocument::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultDocument> res = make_object<inputInlineQueryResultDocument>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->document_url_ = jni::fetch_string(env, p, res->document_url_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_width_ = env->GetIntField(p, res->thumbnail_width_fieldID);
  res->thumbnail_height_ = env->GetIntField(p, res->thumbnail_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultDocument");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_field("documentUrl", document_url_);
    s.store_field("mimeType", mime_type_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailWidth", thumbnail_width_);
    s.store_field("thumbnailHeight", thumbnail_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultDocument").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    document_url_fieldID = jni::get_field_id(env, Class, "documentUrl", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_width_fieldID = jni::get_field_id(env, Class, "thumbnailWidth", "I");
    thumbnail_height_fieldID = jni::get_field_id(env, Class, "thumbnailHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultGame::Class;
jfieldID inputInlineQueryResultGame::id_fieldID;
jfieldID inputInlineQueryResultGame::game_short_name_fieldID;
jfieldID inputInlineQueryResultGame::reply_markup_fieldID;

inputInlineQueryResultGame::inputInlineQueryResultGame()
  : id_()
  , game_short_name_()
  , reply_markup_()
{}

inputInlineQueryResultGame::inputInlineQueryResultGame(string const &id_, string const &game_short_name_, object_ptr<ReplyMarkup> &&reply_markup_)
  : id_(id_)
  , game_short_name_(game_short_name_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t inputInlineQueryResultGame::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultGame> res = make_object<inputInlineQueryResultGame>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->game_short_name_ = jni::fetch_string(env, p, res->game_short_name_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return std::move(res);
}

void inputInlineQueryResultGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultGame");
    s.store_field("id", id_);
    s.store_field("gameShortName", game_short_name_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultGame").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    game_short_name_fieldID = jni::get_field_id(env, Class, "gameShortName", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultLocation::Class;
jfieldID inputInlineQueryResultLocation::id_fieldID;
jfieldID inputInlineQueryResultLocation::location_fieldID;
jfieldID inputInlineQueryResultLocation::live_period_fieldID;
jfieldID inputInlineQueryResultLocation::title_fieldID;
jfieldID inputInlineQueryResultLocation::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultLocation::thumbnail_width_fieldID;
jfieldID inputInlineQueryResultLocation::thumbnail_height_fieldID;
jfieldID inputInlineQueryResultLocation::reply_markup_fieldID;
jfieldID inputInlineQueryResultLocation::input_message_content_fieldID;

inputInlineQueryResultLocation::inputInlineQueryResultLocation()
  : id_()
  , location_()
  , live_period_()
  , title_()
  , thumbnail_url_()
  , thumbnail_width_()
  , thumbnail_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultLocation::inputInlineQueryResultLocation(string const &id_, object_ptr<location> &&location_, int32 live_period_, string const &title_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , location_(std::move(location_))
  , live_period_(live_period_)
  , title_(title_)
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_width_(thumbnail_width_)
  , thumbnail_height_(thumbnail_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultLocation::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultLocation> res = make_object<inputInlineQueryResultLocation>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_width_ = env->GetIntField(p, res->thumbnail_width_fieldID);
  res->thumbnail_height_ = env->GetIntField(p, res->thumbnail_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultLocation");
    s.store_field("id", id_);
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("title", title_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailWidth", thumbnail_width_);
    s.store_field("thumbnailHeight", thumbnail_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultLocation").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_width_fieldID = jni::get_field_id(env, Class, "thumbnailWidth", "I");
    thumbnail_height_fieldID = jni::get_field_id(env, Class, "thumbnailHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultPhoto::Class;
jfieldID inputInlineQueryResultPhoto::id_fieldID;
jfieldID inputInlineQueryResultPhoto::title_fieldID;
jfieldID inputInlineQueryResultPhoto::description_fieldID;
jfieldID inputInlineQueryResultPhoto::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultPhoto::photo_url_fieldID;
jfieldID inputInlineQueryResultPhoto::photo_width_fieldID;
jfieldID inputInlineQueryResultPhoto::photo_height_fieldID;
jfieldID inputInlineQueryResultPhoto::reply_markup_fieldID;
jfieldID inputInlineQueryResultPhoto::input_message_content_fieldID;

inputInlineQueryResultPhoto::inputInlineQueryResultPhoto()
  : id_()
  , title_()
  , description_()
  , thumbnail_url_()
  , photo_url_()
  , photo_width_()
  , photo_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultPhoto::inputInlineQueryResultPhoto(string const &id_, string const &title_, string const &description_, string const &thumbnail_url_, string const &photo_url_, int32 photo_width_, int32 photo_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , description_(description_)
  , thumbnail_url_(thumbnail_url_)
  , photo_url_(photo_url_)
  , photo_width_(photo_width_)
  , photo_height_(photo_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultPhoto::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultPhoto> res = make_object<inputInlineQueryResultPhoto>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->photo_url_ = jni::fetch_string(env, p, res->photo_url_fieldID);
  res->photo_width_ = env->GetIntField(p, res->photo_width_fieldID);
  res->photo_height_ = env->GetIntField(p, res->photo_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultPhoto");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("photoUrl", photo_url_);
    s.store_field("photoWidth", photo_width_);
    s.store_field("photoHeight", photo_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultPhoto").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    photo_url_fieldID = jni::get_field_id(env, Class, "photoUrl", "Ljava/lang/String;");
    photo_width_fieldID = jni::get_field_id(env, Class, "photoWidth", "I");
    photo_height_fieldID = jni::get_field_id(env, Class, "photoHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultSticker::Class;
jfieldID inputInlineQueryResultSticker::id_fieldID;
jfieldID inputInlineQueryResultSticker::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultSticker::sticker_url_fieldID;
jfieldID inputInlineQueryResultSticker::sticker_width_fieldID;
jfieldID inputInlineQueryResultSticker::sticker_height_fieldID;
jfieldID inputInlineQueryResultSticker::reply_markup_fieldID;
jfieldID inputInlineQueryResultSticker::input_message_content_fieldID;

inputInlineQueryResultSticker::inputInlineQueryResultSticker()
  : id_()
  , thumbnail_url_()
  , sticker_url_()
  , sticker_width_()
  , sticker_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultSticker::inputInlineQueryResultSticker(string const &id_, string const &thumbnail_url_, string const &sticker_url_, int32 sticker_width_, int32 sticker_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , thumbnail_url_(thumbnail_url_)
  , sticker_url_(sticker_url_)
  , sticker_width_(sticker_width_)
  , sticker_height_(sticker_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultSticker::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultSticker> res = make_object<inputInlineQueryResultSticker>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->sticker_url_ = jni::fetch_string(env, p, res->sticker_url_fieldID);
  res->sticker_width_ = env->GetIntField(p, res->sticker_width_fieldID);
  res->sticker_height_ = env->GetIntField(p, res->sticker_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultSticker");
    s.store_field("id", id_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("stickerUrl", sticker_url_);
    s.store_field("stickerWidth", sticker_width_);
    s.store_field("stickerHeight", sticker_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultSticker").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    sticker_url_fieldID = jni::get_field_id(env, Class, "stickerUrl", "Ljava/lang/String;");
    sticker_width_fieldID = jni::get_field_id(env, Class, "stickerWidth", "I");
    sticker_height_fieldID = jni::get_field_id(env, Class, "stickerHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultVenue::Class;
jfieldID inputInlineQueryResultVenue::id_fieldID;
jfieldID inputInlineQueryResultVenue::venue_fieldID;
jfieldID inputInlineQueryResultVenue::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultVenue::thumbnail_width_fieldID;
jfieldID inputInlineQueryResultVenue::thumbnail_height_fieldID;
jfieldID inputInlineQueryResultVenue::reply_markup_fieldID;
jfieldID inputInlineQueryResultVenue::input_message_content_fieldID;

inputInlineQueryResultVenue::inputInlineQueryResultVenue()
  : id_()
  , venue_()
  , thumbnail_url_()
  , thumbnail_width_()
  , thumbnail_height_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultVenue::inputInlineQueryResultVenue(string const &id_, object_ptr<venue> &&venue_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , venue_(std::move(venue_))
  , thumbnail_url_(thumbnail_url_)
  , thumbnail_width_(thumbnail_width_)
  , thumbnail_height_(thumbnail_height_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultVenue::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultVenue> res = make_object<inputInlineQueryResultVenue>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->venue_ = jni::fetch_tl_object<venue>(env, jni::fetch_object(env, p, res->venue_fieldID));
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->thumbnail_width_ = env->GetIntField(p, res->thumbnail_width_fieldID);
  res->thumbnail_height_ = env->GetIntField(p, res->thumbnail_height_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultVenue");
    s.store_field("id", id_);
    s.store_object_field("venue", static_cast<const BaseObject *>(venue_.get()));
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("thumbnailWidth", thumbnail_width_);
    s.store_field("thumbnailHeight", thumbnail_height_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultVenue").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    venue_fieldID = jni::get_field_id(env, Class, "venue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Venue;").c_str());
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    thumbnail_width_fieldID = jni::get_field_id(env, Class, "thumbnailWidth", "I");
    thumbnail_height_fieldID = jni::get_field_id(env, Class, "thumbnailHeight", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultVideo::Class;
jfieldID inputInlineQueryResultVideo::id_fieldID;
jfieldID inputInlineQueryResultVideo::title_fieldID;
jfieldID inputInlineQueryResultVideo::description_fieldID;
jfieldID inputInlineQueryResultVideo::thumbnail_url_fieldID;
jfieldID inputInlineQueryResultVideo::video_url_fieldID;
jfieldID inputInlineQueryResultVideo::mime_type_fieldID;
jfieldID inputInlineQueryResultVideo::video_width_fieldID;
jfieldID inputInlineQueryResultVideo::video_height_fieldID;
jfieldID inputInlineQueryResultVideo::video_duration_fieldID;
jfieldID inputInlineQueryResultVideo::reply_markup_fieldID;
jfieldID inputInlineQueryResultVideo::input_message_content_fieldID;

inputInlineQueryResultVideo::inputInlineQueryResultVideo()
  : id_()
  , title_()
  , description_()
  , thumbnail_url_()
  , video_url_()
  , mime_type_()
  , video_width_()
  , video_height_()
  , video_duration_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultVideo::inputInlineQueryResultVideo(string const &id_, string const &title_, string const &description_, string const &thumbnail_url_, string const &video_url_, string const &mime_type_, int32 video_width_, int32 video_height_, int32 video_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , description_(description_)
  , thumbnail_url_(thumbnail_url_)
  , video_url_(video_url_)
  , mime_type_(mime_type_)
  , video_width_(video_width_)
  , video_height_(video_height_)
  , video_duration_(video_duration_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultVideo::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultVideo> res = make_object<inputInlineQueryResultVideo>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->thumbnail_url_ = jni::fetch_string(env, p, res->thumbnail_url_fieldID);
  res->video_url_ = jni::fetch_string(env, p, res->video_url_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->video_width_ = env->GetIntField(p, res->video_width_fieldID);
  res->video_height_ = env->GetIntField(p, res->video_height_fieldID);
  res->video_duration_ = env->GetIntField(p, res->video_duration_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultVideo");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_field("thumbnailUrl", thumbnail_url_);
    s.store_field("videoUrl", video_url_);
    s.store_field("mimeType", mime_type_);
    s.store_field("videoWidth", video_width_);
    s.store_field("videoHeight", video_height_);
    s.store_field("videoDuration", video_duration_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultVideo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    thumbnail_url_fieldID = jni::get_field_id(env, Class, "thumbnailUrl", "Ljava/lang/String;");
    video_url_fieldID = jni::get_field_id(env, Class, "videoUrl", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    video_width_fieldID = jni::get_field_id(env, Class, "videoWidth", "I");
    video_height_fieldID = jni::get_field_id(env, Class, "videoHeight", "I");
    video_duration_fieldID = jni::get_field_id(env, Class, "videoDuration", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInlineQueryResultVoiceNote::Class;
jfieldID inputInlineQueryResultVoiceNote::id_fieldID;
jfieldID inputInlineQueryResultVoiceNote::title_fieldID;
jfieldID inputInlineQueryResultVoiceNote::voice_note_url_fieldID;
jfieldID inputInlineQueryResultVoiceNote::voice_note_duration_fieldID;
jfieldID inputInlineQueryResultVoiceNote::reply_markup_fieldID;
jfieldID inputInlineQueryResultVoiceNote::input_message_content_fieldID;

inputInlineQueryResultVoiceNote::inputInlineQueryResultVoiceNote()
  : id_()
  , title_()
  , voice_note_url_()
  , voice_note_duration_()
  , reply_markup_()
  , input_message_content_()
{}

inputInlineQueryResultVoiceNote::inputInlineQueryResultVoiceNote(string const &id_, string const &title_, string const &voice_note_url_, int32 voice_note_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : id_(id_)
  , title_(title_)
  , voice_note_url_(voice_note_url_)
  , voice_note_duration_(voice_note_duration_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t inputInlineQueryResultVoiceNote::ID;

object_ptr<InputInlineQueryResult> inputInlineQueryResultVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInlineQueryResultVoiceNote> res = make_object<inputInlineQueryResultVoiceNote>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->voice_note_url_ = jni::fetch_string(env, p, res->voice_note_url_fieldID);
  res->voice_note_duration_ = env->GetIntField(p, res->voice_note_duration_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return std::move(res);
}

void inputInlineQueryResultVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInlineQueryResultVoiceNote");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("voiceNoteUrl", voice_note_url_);
    s.store_field("voiceNoteDuration", voice_note_duration_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

void inputInlineQueryResultVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInlineQueryResultVoiceNote").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    voice_note_url_fieldID = jni::get_field_id(env, Class, "voiceNoteUrl", "Ljava/lang/String;");
    voice_note_duration_fieldID = jni::get_field_id(env, Class, "voiceNoteDuration", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputInvoice::Class;

object_ptr<InputInvoice> InputInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputInvoiceMessage::ID:
      return inputInvoiceMessage::fetch(env, p);
    case inputInvoiceName::ID:
      return inputInvoiceName::fetch(env, p);
    case inputInvoiceTelegram::ID:
      return inputInvoiceTelegram::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInvoice").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputInvoiceMessage::Class;
jfieldID inputInvoiceMessage::chat_id_fieldID;
jfieldID inputInvoiceMessage::message_id_fieldID;

inputInvoiceMessage::inputInvoiceMessage()
  : chat_id_()
  , message_id_()
{}

inputInvoiceMessage::inputInvoiceMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t inputInvoiceMessage::ID;

object_ptr<InputInvoice> inputInvoiceMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInvoiceMessage> res = make_object<inputInvoiceMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void inputInvoiceMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInvoiceMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void inputInvoiceMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInvoiceMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputInvoiceName::Class;
jfieldID inputInvoiceName::name_fieldID;

inputInvoiceName::inputInvoiceName()
  : name_()
{}

inputInvoiceName::inputInvoiceName(string const &name_)
  : name_(name_)
{}

const std::int32_t inputInvoiceName::ID;

object_ptr<InputInvoice> inputInvoiceName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInvoiceName> res = make_object<inputInvoiceName>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return std::move(res);
}

void inputInvoiceName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInvoiceName");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void inputInvoiceName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInvoiceName").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputInvoiceTelegram::Class;
jfieldID inputInvoiceTelegram::purpose_fieldID;

inputInvoiceTelegram::inputInvoiceTelegram()
  : purpose_()
{}

inputInvoiceTelegram::inputInvoiceTelegram(object_ptr<TelegramPaymentPurpose> &&purpose_)
  : purpose_(std::move(purpose_))
{}

const std::int32_t inputInvoiceTelegram::ID;

object_ptr<InputInvoice> inputInvoiceTelegram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputInvoiceTelegram> res = make_object<inputInvoiceTelegram>();
  res->purpose_ = jni::fetch_tl_object<TelegramPaymentPurpose>(env, jni::fetch_object(env, p, res->purpose_fieldID));
  return std::move(res);
}

void inputInvoiceTelegram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputInvoiceTelegram");
    s.store_object_field("purpose", static_cast<const BaseObject *>(purpose_.get()));
    s.store_class_end();
  }
}

void inputInvoiceTelegram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputInvoiceTelegram").c_str());
    purpose_fieldID = jni::get_field_id(env, Class, "purpose", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TelegramPaymentPurpose;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputMessageContent::Class;

object_ptr<InputMessageContent> InputMessageContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputMessageText::ID:
      return inputMessageText::fetch(env, p);
    case inputMessageAnimation::ID:
      return inputMessageAnimation::fetch(env, p);
    case inputMessageAudio::ID:
      return inputMessageAudio::fetch(env, p);
    case inputMessageDocument::ID:
      return inputMessageDocument::fetch(env, p);
    case inputMessagePaidMedia::ID:
      return inputMessagePaidMedia::fetch(env, p);
    case inputMessagePhoto::ID:
      return inputMessagePhoto::fetch(env, p);
    case inputMessageSticker::ID:
      return inputMessageSticker::fetch(env, p);
    case inputMessageVideo::ID:
      return inputMessageVideo::fetch(env, p);
    case inputMessageVideoNote::ID:
      return inputMessageVideoNote::fetch(env, p);
    case inputMessageVoiceNote::ID:
      return inputMessageVoiceNote::fetch(env, p);
    case inputMessageLocation::ID:
      return inputMessageLocation::fetch(env, p);
    case inputMessageVenue::ID:
      return inputMessageVenue::fetch(env, p);
    case inputMessageContact::ID:
      return inputMessageContact::fetch(env, p);
    case inputMessageDice::ID:
      return inputMessageDice::fetch(env, p);
    case inputMessageGame::ID:
      return inputMessageGame::fetch(env, p);
    case inputMessageInvoice::ID:
      return inputMessageInvoice::fetch(env, p);
    case inputMessagePoll::ID:
      return inputMessagePoll::fetch(env, p);
    case inputMessageStory::ID:
      return inputMessageStory::fetch(env, p);
    case inputMessageForwarded::ID:
      return inputMessageForwarded::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputMessageContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageContent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageText::Class;
jfieldID inputMessageText::text_fieldID;
jfieldID inputMessageText::link_preview_options_fieldID;
jfieldID inputMessageText::clear_draft_fieldID;

inputMessageText::inputMessageText()
  : text_()
  , link_preview_options_()
  , clear_draft_()
{}

inputMessageText::inputMessageText(object_ptr<formattedText> &&text_, object_ptr<linkPreviewOptions> &&link_preview_options_, bool clear_draft_)
  : text_(std::move(text_))
  , link_preview_options_(std::move(link_preview_options_))
  , clear_draft_(clear_draft_)
{}

const std::int32_t inputMessageText::ID;

object_ptr<InputMessageContent> inputMessageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageText> res = make_object<inputMessageText>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->link_preview_options_ = jni::fetch_tl_object<linkPreviewOptions>(env, jni::fetch_object(env, p, res->link_preview_options_fieldID));
  res->clear_draft_ = (env->GetBooleanField(p, res->clear_draft_fieldID) != 0);
  return std::move(res);
}

void inputMessageText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  if (link_preview_options_ != nullptr) { jobject next; link_preview_options_->store(env, next); if (next) { env->SetObjectField(s, link_preview_options_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, clear_draft_fieldID, clear_draft_);
}

void inputMessageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageText");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("linkPreviewOptions", static_cast<const BaseObject *>(link_preview_options_.get()));
    s.store_field("clearDraft", clear_draft_);
    s.store_class_end();
  }
}

void inputMessageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    link_preview_options_fieldID = jni::get_field_id(env, Class, "linkPreviewOptions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LinkPreviewOptions;").c_str());
    clear_draft_fieldID = jni::get_field_id(env, Class, "clearDraft", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageAnimation::Class;
jfieldID inputMessageAnimation::animation_fieldID;
jfieldID inputMessageAnimation::thumbnail_fieldID;
jfieldID inputMessageAnimation::added_sticker_file_ids_fieldID;
jfieldID inputMessageAnimation::duration_fieldID;
jfieldID inputMessageAnimation::width_fieldID;
jfieldID inputMessageAnimation::height_fieldID;
jfieldID inputMessageAnimation::caption_fieldID;
jfieldID inputMessageAnimation::show_caption_above_media_fieldID;
jfieldID inputMessageAnimation::has_spoiler_fieldID;

inputMessageAnimation::inputMessageAnimation()
  : animation_()
  , thumbnail_()
  , added_sticker_file_ids_()
  , duration_()
  , width_()
  , height_()
  , caption_()
  , show_caption_above_media_()
  , has_spoiler_()
{}

inputMessageAnimation::inputMessageAnimation(object_ptr<InputFile> &&animation_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 duration_, int32 width_, int32 height_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_)
  : animation_(std::move(animation_))
  , thumbnail_(std::move(thumbnail_))
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
  , duration_(duration_)
  , width_(width_)
  , height_(height_)
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , has_spoiler_(has_spoiler_)
{}

const std::int32_t inputMessageAnimation::ID;

object_ptr<InputMessageContent> inputMessageAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageAnimation> res = make_object<inputMessageAnimation>();
  res->animation_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  return std::move(res);
}

void inputMessageAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, added_sticker_file_ids_); if (arr_tmp_) { env->SetObjectField(s, added_sticker_file_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
}

void inputMessageAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_class_end();
  }
}

void inputMessageAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageAudio::Class;
jfieldID inputMessageAudio::audio_fieldID;
jfieldID inputMessageAudio::album_cover_thumbnail_fieldID;
jfieldID inputMessageAudio::duration_fieldID;
jfieldID inputMessageAudio::title_fieldID;
jfieldID inputMessageAudio::performer_fieldID;
jfieldID inputMessageAudio::caption_fieldID;

inputMessageAudio::inputMessageAudio()
  : audio_()
  , album_cover_thumbnail_()
  , duration_()
  , title_()
  , performer_()
  , caption_()
{}

inputMessageAudio::inputMessageAudio(object_ptr<InputFile> &&audio_, object_ptr<inputThumbnail> &&album_cover_thumbnail_, int32 duration_, string const &title_, string const &performer_, object_ptr<formattedText> &&caption_)
  : audio_(std::move(audio_))
  , album_cover_thumbnail_(std::move(album_cover_thumbnail_))
  , duration_(duration_)
  , title_(title_)
  , performer_(performer_)
  , caption_(std::move(caption_))
{}

const std::int32_t inputMessageAudio::ID;

object_ptr<InputMessageContent> inputMessageAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageAudio> res = make_object<inputMessageAudio>();
  res->audio_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->audio_fieldID));
  res->album_cover_thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->album_cover_thumbnail_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->performer_ = jni::fetch_string(env, p, res->performer_fieldID);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void inputMessageAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
  if (album_cover_thumbnail_ != nullptr) { jobject next; album_cover_thumbnail_->store(env, next); if (next) { env->SetObjectField(s, album_cover_thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, performer_); if (nextString) { env->SetObjectField(s, performer_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageAudio");
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_object_field("albumCoverThumbnail", static_cast<const BaseObject *>(album_cover_thumbnail_.get()));
    s.store_field("duration", duration_);
    s.store_field("title", title_);
    s.store_field("performer", performer_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void inputMessageAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageAudio").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    album_cover_thumbnail_fieldID = jni::get_field_id(env, Class, "albumCoverThumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    performer_fieldID = jni::get_field_id(env, Class, "performer", "Ljava/lang/String;");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageDocument::Class;
jfieldID inputMessageDocument::document_fieldID;
jfieldID inputMessageDocument::thumbnail_fieldID;
jfieldID inputMessageDocument::disable_content_type_detection_fieldID;
jfieldID inputMessageDocument::caption_fieldID;

inputMessageDocument::inputMessageDocument()
  : document_()
  , thumbnail_()
  , disable_content_type_detection_()
  , caption_()
{}

inputMessageDocument::inputMessageDocument(object_ptr<InputFile> &&document_, object_ptr<inputThumbnail> &&thumbnail_, bool disable_content_type_detection_, object_ptr<formattedText> &&caption_)
  : document_(std::move(document_))
  , thumbnail_(std::move(thumbnail_))
  , disable_content_type_detection_(disable_content_type_detection_)
  , caption_(std::move(caption_))
{}

const std::int32_t inputMessageDocument::ID;

object_ptr<InputMessageContent> inputMessageDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageDocument> res = make_object<inputMessageDocument>();
  res->document_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->disable_content_type_detection_ = (env->GetBooleanField(p, res->disable_content_type_detection_fieldID) != 0);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void inputMessageDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, disable_content_type_detection_fieldID, disable_content_type_detection_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageDocument");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("disableContentTypeDetection", disable_content_type_detection_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void inputMessageDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageDocument").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    disable_content_type_detection_fieldID = jni::get_field_id(env, Class, "disableContentTypeDetection", "Z");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessagePaidMedia::Class;
jfieldID inputMessagePaidMedia::star_count_fieldID;
jfieldID inputMessagePaidMedia::paid_media_fieldID;
jfieldID inputMessagePaidMedia::caption_fieldID;
jfieldID inputMessagePaidMedia::show_caption_above_media_fieldID;
jfieldID inputMessagePaidMedia::payload_fieldID;

inputMessagePaidMedia::inputMessagePaidMedia()
  : star_count_()
  , paid_media_()
  , caption_()
  , show_caption_above_media_()
  , payload_()
{}

inputMessagePaidMedia::inputMessagePaidMedia(int53 star_count_, array<object_ptr<inputPaidMedia>> &&paid_media_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, string const &payload_)
  : star_count_(star_count_)
  , paid_media_(std::move(paid_media_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , payload_(payload_)
{}

const std::int32_t inputMessagePaidMedia::ID;

object_ptr<InputMessageContent> inputMessagePaidMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessagePaidMedia> res = make_object<inputMessagePaidMedia>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->paid_media_ = jni::FetchVector<inputPaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->paid_media_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  return std::move(res);
}

void inputMessagePaidMedia::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  { auto arr_tmp_ = jni::store_vector(env, paid_media_); if (arr_tmp_) { env->SetObjectField(s, paid_media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  { jstring nextString = jni::to_jstring(env, payload_); if (nextString) { env->SetObjectField(s, payload_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inputMessagePaidMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessagePaidMedia");
    s.store_field("starCount", star_count_);
    { s.store_vector_begin("paidMedia", paid_media_.size()); for (const auto &_value : paid_media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_field("payload", payload_);
    s.store_class_end();
  }
}

void inputMessagePaidMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessagePaidMedia").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    paid_media_fieldID = jni::get_field_id(env, Class, "paidMedia", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputPaidMedia;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessagePhoto::Class;
jfieldID inputMessagePhoto::photo_fieldID;
jfieldID inputMessagePhoto::thumbnail_fieldID;
jfieldID inputMessagePhoto::added_sticker_file_ids_fieldID;
jfieldID inputMessagePhoto::width_fieldID;
jfieldID inputMessagePhoto::height_fieldID;
jfieldID inputMessagePhoto::caption_fieldID;
jfieldID inputMessagePhoto::show_caption_above_media_fieldID;
jfieldID inputMessagePhoto::self_destruct_type_fieldID;
jfieldID inputMessagePhoto::has_spoiler_fieldID;

inputMessagePhoto::inputMessagePhoto()
  : photo_()
  , thumbnail_()
  , added_sticker_file_ids_()
  , width_()
  , height_()
  , caption_()
  , show_caption_above_media_()
  , self_destruct_type_()
  , has_spoiler_()
{}

inputMessagePhoto::inputMessagePhoto(object_ptr<InputFile> &&photo_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 width_, int32 height_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, bool has_spoiler_)
  : photo_(std::move(photo_))
  , thumbnail_(std::move(thumbnail_))
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
  , width_(width_)
  , height_(height_)
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , self_destruct_type_(std::move(self_destruct_type_))
  , has_spoiler_(has_spoiler_)
{}

const std::int32_t inputMessagePhoto::ID;

object_ptr<InputMessageContent> inputMessagePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessagePhoto> res = make_object<inputMessagePhoto>();
  res->photo_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->self_destruct_type_ = jni::fetch_tl_object<MessageSelfDestructType>(env, jni::fetch_object(env, p, res->self_destruct_type_fieldID));
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  return std::move(res);
}

void inputMessagePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, added_sticker_file_ids_); if (arr_tmp_) { env->SetObjectField(s, added_sticker_file_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  if (self_destruct_type_ != nullptr) { jobject next; self_destruct_type_->store(env, next); if (next) { env->SetObjectField(s, self_destruct_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
}

void inputMessagePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessagePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_object_field("selfDestructType", static_cast<const BaseObject *>(self_destruct_type_.get()));
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_class_end();
  }
}

void inputMessagePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessagePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    self_destruct_type_fieldID = jni::get_field_id(env, Class, "selfDestructType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSelfDestructType;").c_str());
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageSticker::Class;
jfieldID inputMessageSticker::sticker_fieldID;
jfieldID inputMessageSticker::thumbnail_fieldID;
jfieldID inputMessageSticker::width_fieldID;
jfieldID inputMessageSticker::height_fieldID;
jfieldID inputMessageSticker::emoji_fieldID;

inputMessageSticker::inputMessageSticker()
  : sticker_()
  , thumbnail_()
  , width_()
  , height_()
  , emoji_()
{}

inputMessageSticker::inputMessageSticker(object_ptr<InputFile> &&sticker_, object_ptr<inputThumbnail> &&thumbnail_, int32 width_, int32 height_, string const &emoji_)
  : sticker_(std::move(sticker_))
  , thumbnail_(std::move(thumbnail_))
  , width_(width_)
  , height_(height_)
  , emoji_(emoji_)
{}

const std::int32_t inputMessageSticker::ID;

object_ptr<InputMessageContent> inputMessageSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageSticker> res = make_object<inputMessageSticker>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return std::move(res);
}

void inputMessageSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inputMessageSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

void inputMessageSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageVideo::Class;
jfieldID inputMessageVideo::video_fieldID;
jfieldID inputMessageVideo::thumbnail_fieldID;
jfieldID inputMessageVideo::cover_fieldID;
jfieldID inputMessageVideo::start_timestamp_fieldID;
jfieldID inputMessageVideo::added_sticker_file_ids_fieldID;
jfieldID inputMessageVideo::duration_fieldID;
jfieldID inputMessageVideo::width_fieldID;
jfieldID inputMessageVideo::height_fieldID;
jfieldID inputMessageVideo::supports_streaming_fieldID;
jfieldID inputMessageVideo::caption_fieldID;
jfieldID inputMessageVideo::show_caption_above_media_fieldID;
jfieldID inputMessageVideo::self_destruct_type_fieldID;
jfieldID inputMessageVideo::has_spoiler_fieldID;

inputMessageVideo::inputMessageVideo()
  : video_()
  , thumbnail_()
  , cover_()
  , start_timestamp_()
  , added_sticker_file_ids_()
  , duration_()
  , width_()
  , height_()
  , supports_streaming_()
  , caption_()
  , show_caption_above_media_()
  , self_destruct_type_()
  , has_spoiler_()
{}

inputMessageVideo::inputMessageVideo(object_ptr<InputFile> &&video_, object_ptr<inputThumbnail> &&thumbnail_, object_ptr<InputFile> &&cover_, int32 start_timestamp_, array<int32> &&added_sticker_file_ids_, int32 duration_, int32 width_, int32 height_, bool supports_streaming_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, bool has_spoiler_)
  : video_(std::move(video_))
  , thumbnail_(std::move(thumbnail_))
  , cover_(std::move(cover_))
  , start_timestamp_(start_timestamp_)
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
  , duration_(duration_)
  , width_(width_)
  , height_(height_)
  , supports_streaming_(supports_streaming_)
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , self_destruct_type_(std::move(self_destruct_type_))
  , has_spoiler_(has_spoiler_)
{}

const std::int32_t inputMessageVideo::ID;

object_ptr<InputMessageContent> inputMessageVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageVideo> res = make_object<inputMessageVideo>();
  res->video_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->cover_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->cover_fieldID));
  res->start_timestamp_ = env->GetIntField(p, res->start_timestamp_fieldID);
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->supports_streaming_ = (env->GetBooleanField(p, res->supports_streaming_fieldID) != 0);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->self_destruct_type_ = jni::fetch_tl_object<MessageSelfDestructType>(env, jni::fetch_object(env, p, res->self_destruct_type_fieldID));
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  return std::move(res);
}

void inputMessageVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_timestamp_fieldID, start_timestamp_);
  { auto arr_tmp_ = jni::store_vector(env, added_sticker_file_ids_); if (arr_tmp_) { env->SetObjectField(s, added_sticker_file_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  env->SetBooleanField(s, supports_streaming_fieldID, supports_streaming_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  if (self_destruct_type_ != nullptr) { jobject next; self_destruct_type_->store(env, next); if (next) { env->SetObjectField(s, self_destruct_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
}

void inputMessageVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_field("startTimestamp", start_timestamp_);
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("supportsStreaming", supports_streaming_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_object_field("selfDestructType", static_cast<const BaseObject *>(self_destruct_type_.get()));
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_class_end();
  }
}

void inputMessageVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    start_timestamp_fieldID = jni::get_field_id(env, Class, "startTimestamp", "I");
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    supports_streaming_fieldID = jni::get_field_id(env, Class, "supportsStreaming", "Z");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    self_destruct_type_fieldID = jni::get_field_id(env, Class, "selfDestructType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSelfDestructType;").c_str());
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageVideoNote::Class;
jfieldID inputMessageVideoNote::video_note_fieldID;
jfieldID inputMessageVideoNote::thumbnail_fieldID;
jfieldID inputMessageVideoNote::duration_fieldID;
jfieldID inputMessageVideoNote::length_fieldID;
jfieldID inputMessageVideoNote::self_destruct_type_fieldID;

inputMessageVideoNote::inputMessageVideoNote()
  : video_note_()
  , thumbnail_()
  , duration_()
  , length_()
  , self_destruct_type_()
{}

inputMessageVideoNote::inputMessageVideoNote(object_ptr<InputFile> &&video_note_, object_ptr<inputThumbnail> &&thumbnail_, int32 duration_, int32 length_, object_ptr<MessageSelfDestructType> &&self_destruct_type_)
  : video_note_(std::move(video_note_))
  , thumbnail_(std::move(thumbnail_))
  , duration_(duration_)
  , length_(length_)
  , self_destruct_type_(std::move(self_destruct_type_))
{}

const std::int32_t inputMessageVideoNote::ID;

object_ptr<InputMessageContent> inputMessageVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageVideoNote> res = make_object<inputMessageVideoNote>();
  res->video_note_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->video_note_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  res->self_destruct_type_ = jni::fetch_tl_object<MessageSelfDestructType>(env, jni::fetch_object(env, p, res->self_destruct_type_fieldID));
  return std::move(res);
}

void inputMessageVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_note_ != nullptr) { jobject next; video_note_->store(env, next); if (next) { env->SetObjectField(s, video_note_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, length_fieldID, length_);
  if (self_destruct_type_ != nullptr) { jobject next; self_destruct_type_->store(env, next); if (next) { env->SetObjectField(s, self_destruct_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageVideoNote");
    s.store_object_field("videoNote", static_cast<const BaseObject *>(video_note_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("duration", duration_);
    s.store_field("length", length_);
    s.store_object_field("selfDestructType", static_cast<const BaseObject *>(self_destruct_type_.get()));
    s.store_class_end();
  }
}

void inputMessageVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageVideoNote").c_str());
    video_note_fieldID = jni::get_field_id(env, Class, "videoNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    self_destruct_type_fieldID = jni::get_field_id(env, Class, "selfDestructType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSelfDestructType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageVoiceNote::Class;
jfieldID inputMessageVoiceNote::voice_note_fieldID;
jfieldID inputMessageVoiceNote::duration_fieldID;
jfieldID inputMessageVoiceNote::waveform_fieldID;
jfieldID inputMessageVoiceNote::caption_fieldID;
jfieldID inputMessageVoiceNote::self_destruct_type_fieldID;

inputMessageVoiceNote::inputMessageVoiceNote()
  : voice_note_()
  , duration_()
  , waveform_()
  , caption_()
  , self_destruct_type_()
{}

inputMessageVoiceNote::inputMessageVoiceNote(object_ptr<InputFile> &&voice_note_, int32 duration_, bytes const &waveform_, object_ptr<formattedText> &&caption_, object_ptr<MessageSelfDestructType> &&self_destruct_type_)
  : voice_note_(std::move(voice_note_))
  , duration_(duration_)
  , waveform_(std::move(waveform_))
  , caption_(std::move(caption_))
  , self_destruct_type_(std::move(self_destruct_type_))
{}

const std::int32_t inputMessageVoiceNote::ID;

object_ptr<InputMessageContent> inputMessageVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageVoiceNote> res = make_object<inputMessageVoiceNote>();
  res->voice_note_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->waveform_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->waveform_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->self_destruct_type_ = jni::fetch_tl_object<MessageSelfDestructType>(env, jni::fetch_object(env, p, res->self_destruct_type_fieldID));
  return std::move(res);
}

void inputMessageVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  { jbyteArray nextBytes = jni::to_bytes(env, waveform_); if (nextBytes) { env->SetObjectField(s, waveform_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  if (self_destruct_type_ != nullptr) { jobject next; self_destruct_type_->store(env, next); if (next) { env->SetObjectField(s, self_destruct_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageVoiceNote");
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_field("duration", duration_);
    s.store_bytes_field("waveform", waveform_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_object_field("selfDestructType", static_cast<const BaseObject *>(self_destruct_type_.get()));
    s.store_class_end();
  }
}

void inputMessageVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageVoiceNote").c_str());
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    waveform_fieldID = jni::get_field_id(env, Class, "waveform", "[B");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    self_destruct_type_fieldID = jni::get_field_id(env, Class, "selfDestructType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSelfDestructType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageLocation::Class;
jfieldID inputMessageLocation::location_fieldID;
jfieldID inputMessageLocation::live_period_fieldID;
jfieldID inputMessageLocation::heading_fieldID;
jfieldID inputMessageLocation::proximity_alert_radius_fieldID;

inputMessageLocation::inputMessageLocation()
  : location_()
  , live_period_()
  , heading_()
  , proximity_alert_radius_()
{}

inputMessageLocation::inputMessageLocation(object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_)
  : location_(std::move(location_))
  , live_period_(live_period_)
  , heading_(heading_)
  , proximity_alert_radius_(proximity_alert_radius_)
{}

const std::int32_t inputMessageLocation::ID;

object_ptr<InputMessageContent> inputMessageLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageLocation> res = make_object<inputMessageLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->heading_ = env->GetIntField(p, res->heading_fieldID);
  res->proximity_alert_radius_ = env->GetIntField(p, res->proximity_alert_radius_fieldID);
  return std::move(res);
}

void inputMessageLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, live_period_fieldID, live_period_);
  env->SetIntField(s, heading_fieldID, heading_);
  env->SetIntField(s, proximity_alert_radius_fieldID, proximity_alert_radius_);
}

void inputMessageLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("heading", heading_);
    s.store_field("proximityAlertRadius", proximity_alert_radius_);
    s.store_class_end();
  }
}

void inputMessageLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    heading_fieldID = jni::get_field_id(env, Class, "heading", "I");
    proximity_alert_radius_fieldID = jni::get_field_id(env, Class, "proximityAlertRadius", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageVenue::Class;
jfieldID inputMessageVenue::venue_fieldID;

inputMessageVenue::inputMessageVenue()
  : venue_()
{}

inputMessageVenue::inputMessageVenue(object_ptr<venue> &&venue_)
  : venue_(std::move(venue_))
{}

const std::int32_t inputMessageVenue::ID;

object_ptr<InputMessageContent> inputMessageVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageVenue> res = make_object<inputMessageVenue>();
  res->venue_ = jni::fetch_tl_object<venue>(env, jni::fetch_object(env, p, res->venue_fieldID));
  return std::move(res);
}

void inputMessageVenue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (venue_ != nullptr) { jobject next; venue_->store(env, next); if (next) { env->SetObjectField(s, venue_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageVenue");
    s.store_object_field("venue", static_cast<const BaseObject *>(venue_.get()));
    s.store_class_end();
  }
}

void inputMessageVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageVenue").c_str());
    venue_fieldID = jni::get_field_id(env, Class, "venue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Venue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageContact::Class;
jfieldID inputMessageContact::contact_fieldID;

inputMessageContact::inputMessageContact()
  : contact_()
{}

inputMessageContact::inputMessageContact(object_ptr<contact> &&contact_)
  : contact_(std::move(contact_))
{}

const std::int32_t inputMessageContact::ID;

object_ptr<InputMessageContent> inputMessageContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageContact> res = make_object<inputMessageContact>();
  res->contact_ = jni::fetch_tl_object<contact>(env, jni::fetch_object(env, p, res->contact_fieldID));
  return std::move(res);
}

void inputMessageContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (contact_ != nullptr) { jobject next; contact_->store(env, next); if (next) { env->SetObjectField(s, contact_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageContact");
    s.store_object_field("contact", static_cast<const BaseObject *>(contact_.get()));
    s.store_class_end();
  }
}

void inputMessageContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageContact").c_str());
    contact_fieldID = jni::get_field_id(env, Class, "contact", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageDice::Class;
jfieldID inputMessageDice::emoji_fieldID;
jfieldID inputMessageDice::clear_draft_fieldID;

inputMessageDice::inputMessageDice()
  : emoji_()
  , clear_draft_()
{}

inputMessageDice::inputMessageDice(string const &emoji_, bool clear_draft_)
  : emoji_(emoji_)
  , clear_draft_(clear_draft_)
{}

const std::int32_t inputMessageDice::ID;

object_ptr<InputMessageContent> inputMessageDice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageDice> res = make_object<inputMessageDice>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->clear_draft_ = (env->GetBooleanField(p, res->clear_draft_fieldID) != 0);
  return std::move(res);
}

void inputMessageDice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, clear_draft_fieldID, clear_draft_);
}

void inputMessageDice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageDice");
    s.store_field("emoji", emoji_);
    s.store_field("clearDraft", clear_draft_);
    s.store_class_end();
  }
}

void inputMessageDice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageDice").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    clear_draft_fieldID = jni::get_field_id(env, Class, "clearDraft", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageGame::Class;
jfieldID inputMessageGame::bot_user_id_fieldID;
jfieldID inputMessageGame::game_short_name_fieldID;

inputMessageGame::inputMessageGame()
  : bot_user_id_()
  , game_short_name_()
{}

inputMessageGame::inputMessageGame(int53 bot_user_id_, string const &game_short_name_)
  : bot_user_id_(bot_user_id_)
  , game_short_name_(game_short_name_)
{}

const std::int32_t inputMessageGame::ID;

object_ptr<InputMessageContent> inputMessageGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageGame> res = make_object<inputMessageGame>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->game_short_name_ = jni::fetch_string(env, p, res->game_short_name_fieldID);
  return std::move(res);
}

void inputMessageGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  { jstring nextString = jni::to_jstring(env, game_short_name_); if (nextString) { env->SetObjectField(s, game_short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void inputMessageGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageGame");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("gameShortName", game_short_name_);
    s.store_class_end();
  }
}

void inputMessageGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageGame").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    game_short_name_fieldID = jni::get_field_id(env, Class, "gameShortName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageInvoice::Class;
jfieldID inputMessageInvoice::invoice_fieldID;
jfieldID inputMessageInvoice::title_fieldID;
jfieldID inputMessageInvoice::description_fieldID;
jfieldID inputMessageInvoice::photo_url_fieldID;
jfieldID inputMessageInvoice::photo_size_fieldID;
jfieldID inputMessageInvoice::photo_width_fieldID;
jfieldID inputMessageInvoice::photo_height_fieldID;
jfieldID inputMessageInvoice::payload_fieldID;
jfieldID inputMessageInvoice::provider_token_fieldID;
jfieldID inputMessageInvoice::provider_data_fieldID;
jfieldID inputMessageInvoice::start_parameter_fieldID;
jfieldID inputMessageInvoice::paid_media_fieldID;
jfieldID inputMessageInvoice::paid_media_caption_fieldID;

inputMessageInvoice::inputMessageInvoice()
  : invoice_()
  , title_()
  , description_()
  , photo_url_()
  , photo_size_()
  , photo_width_()
  , photo_height_()
  , payload_()
  , provider_token_()
  , provider_data_()
  , start_parameter_()
  , paid_media_()
  , paid_media_caption_()
{}

inputMessageInvoice::inputMessageInvoice(object_ptr<invoice> &&invoice_, string const &title_, string const &description_, string const &photo_url_, int32 photo_size_, int32 photo_width_, int32 photo_height_, bytes const &payload_, string const &provider_token_, string const &provider_data_, string const &start_parameter_, object_ptr<inputPaidMedia> &&paid_media_, object_ptr<formattedText> &&paid_media_caption_)
  : invoice_(std::move(invoice_))
  , title_(title_)
  , description_(description_)
  , photo_url_(photo_url_)
  , photo_size_(photo_size_)
  , photo_width_(photo_width_)
  , photo_height_(photo_height_)
  , payload_(std::move(payload_))
  , provider_token_(provider_token_)
  , provider_data_(provider_data_)
  , start_parameter_(start_parameter_)
  , paid_media_(std::move(paid_media_))
  , paid_media_caption_(std::move(paid_media_caption_))
{}

const std::int32_t inputMessageInvoice::ID;

object_ptr<InputMessageContent> inputMessageInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageInvoice> res = make_object<inputMessageInvoice>();
  res->invoice_ = jni::fetch_tl_object<invoice>(env, jni::fetch_object(env, p, res->invoice_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->photo_url_ = jni::fetch_string(env, p, res->photo_url_fieldID);
  res->photo_size_ = env->GetIntField(p, res->photo_size_fieldID);
  res->photo_width_ = env->GetIntField(p, res->photo_width_fieldID);
  res->photo_height_ = env->GetIntField(p, res->photo_height_fieldID);
  res->payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->payload_fieldID));
  res->provider_token_ = jni::fetch_string(env, p, res->provider_token_fieldID);
  res->provider_data_ = jni::fetch_string(env, p, res->provider_data_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->paid_media_ = jni::fetch_tl_object<inputPaidMedia>(env, jni::fetch_object(env, p, res->paid_media_fieldID));
  res->paid_media_caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->paid_media_caption_fieldID));
  return std::move(res);
}

void inputMessageInvoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (invoice_ != nullptr) { jobject next; invoice_->store(env, next); if (next) { env->SetObjectField(s, invoice_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, photo_url_); if (nextString) { env->SetObjectField(s, photo_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, photo_size_fieldID, photo_size_);
  env->SetIntField(s, photo_width_fieldID, photo_width_);
  env->SetIntField(s, photo_height_fieldID, photo_height_);
  { jbyteArray nextBytes = jni::to_bytes(env, payload_); if (nextBytes) { env->SetObjectField(s, payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, provider_token_); if (nextString) { env->SetObjectField(s, provider_token_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, provider_data_); if (nextString) { env->SetObjectField(s, provider_data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (paid_media_ != nullptr) { jobject next; paid_media_->store(env, next); if (next) { env->SetObjectField(s, paid_media_fieldID, next); env->DeleteLocalRef(next); } }
  if (paid_media_caption_ != nullptr) { jobject next; paid_media_caption_->store(env, next); if (next) { env->SetObjectField(s, paid_media_caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageInvoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageInvoice");
    s.store_object_field("invoice", static_cast<const BaseObject *>(invoice_.get()));
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_field("photoUrl", photo_url_);
    s.store_field("photoSize", photo_size_);
    s.store_field("photoWidth", photo_width_);
    s.store_field("photoHeight", photo_height_);
    s.store_bytes_field("payload", payload_);
    s.store_field("providerToken", provider_token_);
    s.store_field("providerData", provider_data_);
    s.store_field("startParameter", start_parameter_);
    s.store_object_field("paidMedia", static_cast<const BaseObject *>(paid_media_.get()));
    s.store_object_field("paidMediaCaption", static_cast<const BaseObject *>(paid_media_caption_.get()));
    s.store_class_end();
  }
}

void inputMessageInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageInvoice").c_str());
    invoice_fieldID = jni::get_field_id(env, Class, "invoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Invoice;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    photo_url_fieldID = jni::get_field_id(env, Class, "photoUrl", "Ljava/lang/String;");
    photo_size_fieldID = jni::get_field_id(env, Class, "photoSize", "I");
    photo_width_fieldID = jni::get_field_id(env, Class, "photoWidth", "I");
    photo_height_fieldID = jni::get_field_id(env, Class, "photoHeight", "I");
    payload_fieldID = jni::get_field_id(env, Class, "payload", "[B");
    provider_token_fieldID = jni::get_field_id(env, Class, "providerToken", "Ljava/lang/String;");
    provider_data_fieldID = jni::get_field_id(env, Class, "providerData", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    paid_media_fieldID = jni::get_field_id(env, Class, "paidMedia", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPaidMedia;").c_str());
    paid_media_caption_fieldID = jni::get_field_id(env, Class, "paidMediaCaption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessagePoll::Class;
jfieldID inputMessagePoll::question_fieldID;
jfieldID inputMessagePoll::options_fieldID;
jfieldID inputMessagePoll::is_anonymous_fieldID;
jfieldID inputMessagePoll::type_fieldID;
jfieldID inputMessagePoll::open_period_fieldID;
jfieldID inputMessagePoll::close_date_fieldID;
jfieldID inputMessagePoll::is_closed_fieldID;

inputMessagePoll::inputMessagePoll()
  : question_()
  , options_()
  , is_anonymous_()
  , type_()
  , open_period_()
  , close_date_()
  , is_closed_()
{}

inputMessagePoll::inputMessagePoll(object_ptr<formattedText> &&question_, array<object_ptr<formattedText>> &&options_, bool is_anonymous_, object_ptr<PollType> &&type_, int32 open_period_, int32 close_date_, bool is_closed_)
  : question_(std::move(question_))
  , options_(std::move(options_))
  , is_anonymous_(is_anonymous_)
  , type_(std::move(type_))
  , open_period_(open_period_)
  , close_date_(close_date_)
  , is_closed_(is_closed_)
{}

const std::int32_t inputMessagePoll::ID;

object_ptr<InputMessageContent> inputMessagePoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessagePoll> res = make_object<inputMessagePoll>();
  res->question_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->question_fieldID));
  res->options_ = jni::FetchVector<formattedText>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<PollType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->open_period_ = env->GetIntField(p, res->open_period_fieldID);
  res->close_date_ = env->GetIntField(p, res->close_date_fieldID);
  res->is_closed_ = (env->GetBooleanField(p, res->is_closed_fieldID) != 0);
  return std::move(res);
}

void inputMessagePoll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (question_ != nullptr) { jobject next; question_->store(env, next); if (next) { env->SetObjectField(s, question_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, open_period_fieldID, open_period_);
  env->SetIntField(s, close_date_fieldID, close_date_);
  env->SetBooleanField(s, is_closed_fieldID, is_closed_);
}

void inputMessagePoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessagePoll");
    s.store_object_field("question", static_cast<const BaseObject *>(question_.get()));
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isAnonymous", is_anonymous_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("openPeriod", open_period_);
    s.store_field("closeDate", close_date_);
    s.store_field("isClosed", is_closed_);
    s.store_class_end();
  }
}

void inputMessagePoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessagePoll").c_str());
    question_fieldID = jni::get_field_id(env, Class, "question", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PollType;").c_str());
    open_period_fieldID = jni::get_field_id(env, Class, "openPeriod", "I");
    close_date_fieldID = jni::get_field_id(env, Class, "closeDate", "I");
    is_closed_fieldID = jni::get_field_id(env, Class, "isClosed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageStory::Class;
jfieldID inputMessageStory::story_poster_chat_id_fieldID;
jfieldID inputMessageStory::story_id_fieldID;

inputMessageStory::inputMessageStory()
  : story_poster_chat_id_()
  , story_id_()
{}

inputMessageStory::inputMessageStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t inputMessageStory::ID;

object_ptr<InputMessageContent> inputMessageStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageStory> res = make_object<inputMessageStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void inputMessageStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void inputMessageStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void inputMessageStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageForwarded::Class;
jfieldID inputMessageForwarded::from_chat_id_fieldID;
jfieldID inputMessageForwarded::message_id_fieldID;
jfieldID inputMessageForwarded::in_game_share_fieldID;
jfieldID inputMessageForwarded::replace_video_start_timestamp_fieldID;
jfieldID inputMessageForwarded::new_video_start_timestamp_fieldID;
jfieldID inputMessageForwarded::copy_options_fieldID;

inputMessageForwarded::inputMessageForwarded()
  : from_chat_id_()
  , message_id_()
  , in_game_share_()
  , replace_video_start_timestamp_()
  , new_video_start_timestamp_()
  , copy_options_()
{}

inputMessageForwarded::inputMessageForwarded(int53 from_chat_id_, int53 message_id_, bool in_game_share_, bool replace_video_start_timestamp_, int32 new_video_start_timestamp_, object_ptr<messageCopyOptions> &&copy_options_)
  : from_chat_id_(from_chat_id_)
  , message_id_(message_id_)
  , in_game_share_(in_game_share_)
  , replace_video_start_timestamp_(replace_video_start_timestamp_)
  , new_video_start_timestamp_(new_video_start_timestamp_)
  , copy_options_(std::move(copy_options_))
{}

const std::int32_t inputMessageForwarded::ID;

object_ptr<InputMessageContent> inputMessageForwarded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageForwarded> res = make_object<inputMessageForwarded>();
  res->from_chat_id_ = env->GetLongField(p, res->from_chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->in_game_share_ = (env->GetBooleanField(p, res->in_game_share_fieldID) != 0);
  res->replace_video_start_timestamp_ = (env->GetBooleanField(p, res->replace_video_start_timestamp_fieldID) != 0);
  res->new_video_start_timestamp_ = env->GetIntField(p, res->new_video_start_timestamp_fieldID);
  res->copy_options_ = jni::fetch_tl_object<messageCopyOptions>(env, jni::fetch_object(env, p, res->copy_options_fieldID));
  return std::move(res);
}

void inputMessageForwarded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, from_chat_id_fieldID, from_chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetBooleanField(s, in_game_share_fieldID, in_game_share_);
  env->SetBooleanField(s, replace_video_start_timestamp_fieldID, replace_video_start_timestamp_);
  env->SetIntField(s, new_video_start_timestamp_fieldID, new_video_start_timestamp_);
  if (copy_options_ != nullptr) { jobject next; copy_options_->store(env, next); if (next) { env->SetObjectField(s, copy_options_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageForwarded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageForwarded");
    s.store_field("fromChatId", from_chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("inGameShare", in_game_share_);
    s.store_field("replaceVideoStartTimestamp", replace_video_start_timestamp_);
    s.store_field("newVideoStartTimestamp", new_video_start_timestamp_);
    s.store_object_field("copyOptions", static_cast<const BaseObject *>(copy_options_.get()));
    s.store_class_end();
  }
}

void inputMessageForwarded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageForwarded").c_str());
    from_chat_id_fieldID = jni::get_field_id(env, Class, "fromChatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    in_game_share_fieldID = jni::get_field_id(env, Class, "inGameShare", "Z");
    replace_video_start_timestamp_fieldID = jni::get_field_id(env, Class, "replaceVideoStartTimestamp", "Z");
    new_video_start_timestamp_fieldID = jni::get_field_id(env, Class, "newVideoStartTimestamp", "I");
    copy_options_fieldID = jni::get_field_id(env, Class, "copyOptions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageCopyOptions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputMessageReplyTo::Class;

object_ptr<InputMessageReplyTo> InputMessageReplyTo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputMessageReplyToMessage::ID:
      return inputMessageReplyToMessage::fetch(env, p);
    case inputMessageReplyToExternalMessage::ID:
      return inputMessageReplyToExternalMessage::fetch(env, p);
    case inputMessageReplyToStory::ID:
      return inputMessageReplyToStory::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputMessageReplyTo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageReplyTo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageReplyToMessage::Class;
jfieldID inputMessageReplyToMessage::message_id_fieldID;
jfieldID inputMessageReplyToMessage::quote_fieldID;

inputMessageReplyToMessage::inputMessageReplyToMessage()
  : message_id_()
  , quote_()
{}

inputMessageReplyToMessage::inputMessageReplyToMessage(int53 message_id_, object_ptr<inputTextQuote> &&quote_)
  : message_id_(message_id_)
  , quote_(std::move(quote_))
{}

const std::int32_t inputMessageReplyToMessage::ID;

object_ptr<InputMessageReplyTo> inputMessageReplyToMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageReplyToMessage> res = make_object<inputMessageReplyToMessage>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->quote_ = jni::fetch_tl_object<inputTextQuote>(env, jni::fetch_object(env, p, res->quote_fieldID));
  return std::move(res);
}

void inputMessageReplyToMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (quote_ != nullptr) { jobject next; quote_->store(env, next); if (next) { env->SetObjectField(s, quote_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageReplyToMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageReplyToMessage");
    s.store_field("messageId", message_id_);
    s.store_object_field("quote", static_cast<const BaseObject *>(quote_.get()));
    s.store_class_end();
  }
}

void inputMessageReplyToMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageReplyToMessage").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    quote_fieldID = jni::get_field_id(env, Class, "quote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputTextQuote;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageReplyToExternalMessage::Class;
jfieldID inputMessageReplyToExternalMessage::chat_id_fieldID;
jfieldID inputMessageReplyToExternalMessage::message_id_fieldID;
jfieldID inputMessageReplyToExternalMessage::quote_fieldID;

inputMessageReplyToExternalMessage::inputMessageReplyToExternalMessage()
  : chat_id_()
  , message_id_()
  , quote_()
{}

inputMessageReplyToExternalMessage::inputMessageReplyToExternalMessage(int53 chat_id_, int53 message_id_, object_ptr<inputTextQuote> &&quote_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , quote_(std::move(quote_))
{}

const std::int32_t inputMessageReplyToExternalMessage::ID;

object_ptr<InputMessageReplyTo> inputMessageReplyToExternalMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageReplyToExternalMessage> res = make_object<inputMessageReplyToExternalMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->quote_ = jni::fetch_tl_object<inputTextQuote>(env, jni::fetch_object(env, p, res->quote_fieldID));
  return std::move(res);
}

void inputMessageReplyToExternalMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (quote_ != nullptr) { jobject next; quote_->store(env, next); if (next) { env->SetObjectField(s, quote_fieldID, next); env->DeleteLocalRef(next); } }
}

void inputMessageReplyToExternalMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageReplyToExternalMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("quote", static_cast<const BaseObject *>(quote_.get()));
    s.store_class_end();
  }
}

void inputMessageReplyToExternalMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageReplyToExternalMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    quote_fieldID = jni::get_field_id(env, Class, "quote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputTextQuote;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputMessageReplyToStory::Class;
jfieldID inputMessageReplyToStory::story_poster_chat_id_fieldID;
jfieldID inputMessageReplyToStory::story_id_fieldID;

inputMessageReplyToStory::inputMessageReplyToStory()
  : story_poster_chat_id_()
  , story_id_()
{}

inputMessageReplyToStory::inputMessageReplyToStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t inputMessageReplyToStory::ID;

object_ptr<InputMessageReplyTo> inputMessageReplyToStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputMessageReplyToStory> res = make_object<inputMessageReplyToStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void inputMessageReplyToStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void inputMessageReplyToStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputMessageReplyToStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void inputMessageReplyToStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputMessageReplyToStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputPaidMedia::Class;
jfieldID inputPaidMedia::type_fieldID;
jfieldID inputPaidMedia::media_fieldID;
jfieldID inputPaidMedia::thumbnail_fieldID;
jfieldID inputPaidMedia::added_sticker_file_ids_fieldID;
jfieldID inputPaidMedia::width_fieldID;
jfieldID inputPaidMedia::height_fieldID;

inputPaidMedia::inputPaidMedia()
  : type_()
  , media_()
  , thumbnail_()
  , added_sticker_file_ids_()
  , width_()
  , height_()
{}

inputPaidMedia::inputPaidMedia(object_ptr<InputPaidMediaType> &&type_, object_ptr<InputFile> &&media_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 width_, int32 height_)
  : type_(std::move(type_))
  , media_(std::move(media_))
  , thumbnail_(std::move(thumbnail_))
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
  , width_(width_)
  , height_(height_)
{}

const std::int32_t inputPaidMedia::ID;

object_ptr<inputPaidMedia> inputPaidMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPaidMedia> res = make_object<inputPaidMedia>();
  res->type_ = jni::fetch_tl_object<InputPaidMediaType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->media_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->media_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<inputThumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return res;
}

void inputPaidMedia::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (media_ != nullptr) { jobject next; media_->store(env, next); if (next) { env->SetObjectField(s, media_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, added_sticker_file_ids_); if (arr_tmp_) { env->SetObjectField(s, added_sticker_file_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void inputPaidMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPaidMedia");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("media", static_cast<const BaseObject *>(media_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void inputPaidMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPaidMedia").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPaidMediaType;").c_str());
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputThumbnail;").c_str());
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass InputPaidMediaType::Class;

object_ptr<InputPaidMediaType> InputPaidMediaType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputPaidMediaTypePhoto::ID:
      return inputPaidMediaTypePhoto::fetch(env, p);
    case inputPaidMediaTypeVideo::ID:
      return inputPaidMediaTypeVideo::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputPaidMediaType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPaidMediaType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPaidMediaTypePhoto::Class;

inputPaidMediaTypePhoto::inputPaidMediaTypePhoto() {
}

const std::int32_t inputPaidMediaTypePhoto::ID;

object_ptr<InputPaidMediaType> inputPaidMediaTypePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inputPaidMediaTypePhoto>();
}

void inputPaidMediaTypePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inputPaidMediaTypePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPaidMediaTypePhoto");
    s.store_class_end();
  }
}

void inputPaidMediaTypePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPaidMediaTypePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPaidMediaTypeVideo::Class;
jfieldID inputPaidMediaTypeVideo::cover_fieldID;
jfieldID inputPaidMediaTypeVideo::start_timestamp_fieldID;
jfieldID inputPaidMediaTypeVideo::duration_fieldID;
jfieldID inputPaidMediaTypeVideo::supports_streaming_fieldID;

inputPaidMediaTypeVideo::inputPaidMediaTypeVideo()
  : cover_()
  , start_timestamp_()
  , duration_()
  , supports_streaming_()
{}

inputPaidMediaTypeVideo::inputPaidMediaTypeVideo(object_ptr<InputFile> &&cover_, int32 start_timestamp_, int32 duration_, bool supports_streaming_)
  : cover_(std::move(cover_))
  , start_timestamp_(start_timestamp_)
  , duration_(duration_)
  , supports_streaming_(supports_streaming_)
{}

const std::int32_t inputPaidMediaTypeVideo::ID;

object_ptr<InputPaidMediaType> inputPaidMediaTypeVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPaidMediaTypeVideo> res = make_object<inputPaidMediaTypeVideo>();
  res->cover_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->cover_fieldID));
  res->start_timestamp_ = env->GetIntField(p, res->start_timestamp_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->supports_streaming_ = (env->GetBooleanField(p, res->supports_streaming_fieldID) != 0);
  return std::move(res);
}

void inputPaidMediaTypeVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_timestamp_fieldID, start_timestamp_);
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetBooleanField(s, supports_streaming_fieldID, supports_streaming_);
}

void inputPaidMediaTypeVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPaidMediaTypeVideo");
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_field("startTimestamp", start_timestamp_);
    s.store_field("duration", duration_);
    s.store_field("supportsStreaming", supports_streaming_);
    s.store_class_end();
  }
}

void inputPaidMediaTypeVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPaidMediaTypeVideo").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    start_timestamp_fieldID = jni::get_field_id(env, Class, "startTimestamp", "I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    supports_streaming_fieldID = jni::get_field_id(env, Class, "supportsStreaming", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass InputPassportElement::Class;

object_ptr<InputPassportElement> InputPassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputPassportElementPersonalDetails::ID:
      return inputPassportElementPersonalDetails::fetch(env, p);
    case inputPassportElementPassport::ID:
      return inputPassportElementPassport::fetch(env, p);
    case inputPassportElementDriverLicense::ID:
      return inputPassportElementDriverLicense::fetch(env, p);
    case inputPassportElementIdentityCard::ID:
      return inputPassportElementIdentityCard::fetch(env, p);
    case inputPassportElementInternalPassport::ID:
      return inputPassportElementInternalPassport::fetch(env, p);
    case inputPassportElementAddress::ID:
      return inputPassportElementAddress::fetch(env, p);
    case inputPassportElementUtilityBill::ID:
      return inputPassportElementUtilityBill::fetch(env, p);
    case inputPassportElementBankStatement::ID:
      return inputPassportElementBankStatement::fetch(env, p);
    case inputPassportElementRentalAgreement::ID:
      return inputPassportElementRentalAgreement::fetch(env, p);
    case inputPassportElementPassportRegistration::ID:
      return inputPassportElementPassportRegistration::fetch(env, p);
    case inputPassportElementTemporaryRegistration::ID:
      return inputPassportElementTemporaryRegistration::fetch(env, p);
    case inputPassportElementPhoneNumber::ID:
      return inputPassportElementPhoneNumber::fetch(env, p);
    case inputPassportElementEmailAddress::ID:
      return inputPassportElementEmailAddress::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputPassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElement").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementPersonalDetails::Class;
jfieldID inputPassportElementPersonalDetails::personal_details_fieldID;

inputPassportElementPersonalDetails::inputPassportElementPersonalDetails()
  : personal_details_()
{}

inputPassportElementPersonalDetails::inputPassportElementPersonalDetails(object_ptr<personalDetails> &&personal_details_)
  : personal_details_(std::move(personal_details_))
{}

const std::int32_t inputPassportElementPersonalDetails::ID;

object_ptr<InputPassportElement> inputPassportElementPersonalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementPersonalDetails> res = make_object<inputPassportElementPersonalDetails>();
  res->personal_details_ = jni::fetch_tl_object<personalDetails>(env, jni::fetch_object(env, p, res->personal_details_fieldID));
  return std::move(res);
}

void inputPassportElementPersonalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementPersonalDetails");
    s.store_object_field("personalDetails", static_cast<const BaseObject *>(personal_details_.get()));
    s.store_class_end();
  }
}

void inputPassportElementPersonalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementPersonalDetails").c_str());
    personal_details_fieldID = jni::get_field_id(env, Class, "personalDetails", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDetails;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementPassport::Class;
jfieldID inputPassportElementPassport::passport_fieldID;

inputPassportElementPassport::inputPassportElementPassport()
  : passport_()
{}

inputPassportElementPassport::inputPassportElementPassport(object_ptr<inputIdentityDocument> &&passport_)
  : passport_(std::move(passport_))
{}

const std::int32_t inputPassportElementPassport::ID;

object_ptr<InputPassportElement> inputPassportElementPassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementPassport> res = make_object<inputPassportElementPassport>();
  res->passport_ = jni::fetch_tl_object<inputIdentityDocument>(env, jni::fetch_object(env, p, res->passport_fieldID));
  return std::move(res);
}

void inputPassportElementPassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementPassport");
    s.store_object_field("passport", static_cast<const BaseObject *>(passport_.get()));
    s.store_class_end();
  }
}

void inputPassportElementPassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementPassport").c_str());
    passport_fieldID = jni::get_field_id(env, Class, "passport", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputIdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementDriverLicense::Class;
jfieldID inputPassportElementDriverLicense::driver_license_fieldID;

inputPassportElementDriverLicense::inputPassportElementDriverLicense()
  : driver_license_()
{}

inputPassportElementDriverLicense::inputPassportElementDriverLicense(object_ptr<inputIdentityDocument> &&driver_license_)
  : driver_license_(std::move(driver_license_))
{}

const std::int32_t inputPassportElementDriverLicense::ID;

object_ptr<InputPassportElement> inputPassportElementDriverLicense::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementDriverLicense> res = make_object<inputPassportElementDriverLicense>();
  res->driver_license_ = jni::fetch_tl_object<inputIdentityDocument>(env, jni::fetch_object(env, p, res->driver_license_fieldID));
  return std::move(res);
}

void inputPassportElementDriverLicense::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementDriverLicense");
    s.store_object_field("driverLicense", static_cast<const BaseObject *>(driver_license_.get()));
    s.store_class_end();
  }
}

void inputPassportElementDriverLicense::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementDriverLicense").c_str());
    driver_license_fieldID = jni::get_field_id(env, Class, "driverLicense", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputIdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementIdentityCard::Class;
jfieldID inputPassportElementIdentityCard::identity_card_fieldID;

inputPassportElementIdentityCard::inputPassportElementIdentityCard()
  : identity_card_()
{}

inputPassportElementIdentityCard::inputPassportElementIdentityCard(object_ptr<inputIdentityDocument> &&identity_card_)
  : identity_card_(std::move(identity_card_))
{}

const std::int32_t inputPassportElementIdentityCard::ID;

object_ptr<InputPassportElement> inputPassportElementIdentityCard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementIdentityCard> res = make_object<inputPassportElementIdentityCard>();
  res->identity_card_ = jni::fetch_tl_object<inputIdentityDocument>(env, jni::fetch_object(env, p, res->identity_card_fieldID));
  return std::move(res);
}

void inputPassportElementIdentityCard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementIdentityCard");
    s.store_object_field("identityCard", static_cast<const BaseObject *>(identity_card_.get()));
    s.store_class_end();
  }
}

void inputPassportElementIdentityCard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementIdentityCard").c_str());
    identity_card_fieldID = jni::get_field_id(env, Class, "identityCard", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputIdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementInternalPassport::Class;
jfieldID inputPassportElementInternalPassport::internal_passport_fieldID;

inputPassportElementInternalPassport::inputPassportElementInternalPassport()
  : internal_passport_()
{}

inputPassportElementInternalPassport::inputPassportElementInternalPassport(object_ptr<inputIdentityDocument> &&internal_passport_)
  : internal_passport_(std::move(internal_passport_))
{}

const std::int32_t inputPassportElementInternalPassport::ID;

object_ptr<InputPassportElement> inputPassportElementInternalPassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementInternalPassport> res = make_object<inputPassportElementInternalPassport>();
  res->internal_passport_ = jni::fetch_tl_object<inputIdentityDocument>(env, jni::fetch_object(env, p, res->internal_passport_fieldID));
  return std::move(res);
}

void inputPassportElementInternalPassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementInternalPassport");
    s.store_object_field("internalPassport", static_cast<const BaseObject *>(internal_passport_.get()));
    s.store_class_end();
  }
}

void inputPassportElementInternalPassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementInternalPassport").c_str());
    internal_passport_fieldID = jni::get_field_id(env, Class, "internalPassport", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputIdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementAddress::Class;
jfieldID inputPassportElementAddress::address_fieldID;

inputPassportElementAddress::inputPassportElementAddress()
  : address_()
{}

inputPassportElementAddress::inputPassportElementAddress(object_ptr<address> &&address_)
  : address_(std::move(address_))
{}

const std::int32_t inputPassportElementAddress::ID;

object_ptr<InputPassportElement> inputPassportElementAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementAddress> res = make_object<inputPassportElementAddress>();
  res->address_ = jni::fetch_tl_object<address>(env, jni::fetch_object(env, p, res->address_fieldID));
  return std::move(res);
}

void inputPassportElementAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementAddress");
    s.store_object_field("address", static_cast<const BaseObject *>(address_.get()));
    s.store_class_end();
  }
}

void inputPassportElementAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementAddress").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Address;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementUtilityBill::Class;
jfieldID inputPassportElementUtilityBill::utility_bill_fieldID;

inputPassportElementUtilityBill::inputPassportElementUtilityBill()
  : utility_bill_()
{}

inputPassportElementUtilityBill::inputPassportElementUtilityBill(object_ptr<inputPersonalDocument> &&utility_bill_)
  : utility_bill_(std::move(utility_bill_))
{}

const std::int32_t inputPassportElementUtilityBill::ID;

object_ptr<InputPassportElement> inputPassportElementUtilityBill::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementUtilityBill> res = make_object<inputPassportElementUtilityBill>();
  res->utility_bill_ = jni::fetch_tl_object<inputPersonalDocument>(env, jni::fetch_object(env, p, res->utility_bill_fieldID));
  return std::move(res);
}

void inputPassportElementUtilityBill::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementUtilityBill");
    s.store_object_field("utilityBill", static_cast<const BaseObject *>(utility_bill_.get()));
    s.store_class_end();
  }
}

void inputPassportElementUtilityBill::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementUtilityBill").c_str());
    utility_bill_fieldID = jni::get_field_id(env, Class, "utilityBill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementBankStatement::Class;
jfieldID inputPassportElementBankStatement::bank_statement_fieldID;

inputPassportElementBankStatement::inputPassportElementBankStatement()
  : bank_statement_()
{}

inputPassportElementBankStatement::inputPassportElementBankStatement(object_ptr<inputPersonalDocument> &&bank_statement_)
  : bank_statement_(std::move(bank_statement_))
{}

const std::int32_t inputPassportElementBankStatement::ID;

object_ptr<InputPassportElement> inputPassportElementBankStatement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementBankStatement> res = make_object<inputPassportElementBankStatement>();
  res->bank_statement_ = jni::fetch_tl_object<inputPersonalDocument>(env, jni::fetch_object(env, p, res->bank_statement_fieldID));
  return std::move(res);
}

void inputPassportElementBankStatement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementBankStatement");
    s.store_object_field("bankStatement", static_cast<const BaseObject *>(bank_statement_.get()));
    s.store_class_end();
  }
}

void inputPassportElementBankStatement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementBankStatement").c_str());
    bank_statement_fieldID = jni::get_field_id(env, Class, "bankStatement", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementRentalAgreement::Class;
jfieldID inputPassportElementRentalAgreement::rental_agreement_fieldID;

inputPassportElementRentalAgreement::inputPassportElementRentalAgreement()
  : rental_agreement_()
{}

inputPassportElementRentalAgreement::inputPassportElementRentalAgreement(object_ptr<inputPersonalDocument> &&rental_agreement_)
  : rental_agreement_(std::move(rental_agreement_))
{}

const std::int32_t inputPassportElementRentalAgreement::ID;

object_ptr<InputPassportElement> inputPassportElementRentalAgreement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementRentalAgreement> res = make_object<inputPassportElementRentalAgreement>();
  res->rental_agreement_ = jni::fetch_tl_object<inputPersonalDocument>(env, jni::fetch_object(env, p, res->rental_agreement_fieldID));
  return std::move(res);
}

void inputPassportElementRentalAgreement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementRentalAgreement");
    s.store_object_field("rentalAgreement", static_cast<const BaseObject *>(rental_agreement_.get()));
    s.store_class_end();
  }
}

void inputPassportElementRentalAgreement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementRentalAgreement").c_str());
    rental_agreement_fieldID = jni::get_field_id(env, Class, "rentalAgreement", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementPassportRegistration::Class;
jfieldID inputPassportElementPassportRegistration::passport_registration_fieldID;

inputPassportElementPassportRegistration::inputPassportElementPassportRegistration()
  : passport_registration_()
{}

inputPassportElementPassportRegistration::inputPassportElementPassportRegistration(object_ptr<inputPersonalDocument> &&passport_registration_)
  : passport_registration_(std::move(passport_registration_))
{}

const std::int32_t inputPassportElementPassportRegistration::ID;

object_ptr<InputPassportElement> inputPassportElementPassportRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementPassportRegistration> res = make_object<inputPassportElementPassportRegistration>();
  res->passport_registration_ = jni::fetch_tl_object<inputPersonalDocument>(env, jni::fetch_object(env, p, res->passport_registration_fieldID));
  return std::move(res);
}

void inputPassportElementPassportRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementPassportRegistration");
    s.store_object_field("passportRegistration", static_cast<const BaseObject *>(passport_registration_.get()));
    s.store_class_end();
  }
}

void inputPassportElementPassportRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementPassportRegistration").c_str());
    passport_registration_fieldID = jni::get_field_id(env, Class, "passportRegistration", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementTemporaryRegistration::Class;
jfieldID inputPassportElementTemporaryRegistration::temporary_registration_fieldID;

inputPassportElementTemporaryRegistration::inputPassportElementTemporaryRegistration()
  : temporary_registration_()
{}

inputPassportElementTemporaryRegistration::inputPassportElementTemporaryRegistration(object_ptr<inputPersonalDocument> &&temporary_registration_)
  : temporary_registration_(std::move(temporary_registration_))
{}

const std::int32_t inputPassportElementTemporaryRegistration::ID;

object_ptr<InputPassportElement> inputPassportElementTemporaryRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementTemporaryRegistration> res = make_object<inputPassportElementTemporaryRegistration>();
  res->temporary_registration_ = jni::fetch_tl_object<inputPersonalDocument>(env, jni::fetch_object(env, p, res->temporary_registration_fieldID));
  return std::move(res);
}

void inputPassportElementTemporaryRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementTemporaryRegistration");
    s.store_object_field("temporaryRegistration", static_cast<const BaseObject *>(temporary_registration_.get()));
    s.store_class_end();
  }
}

void inputPassportElementTemporaryRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementTemporaryRegistration").c_str());
    temporary_registration_fieldID = jni::get_field_id(env, Class, "temporaryRegistration", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementPhoneNumber::Class;
jfieldID inputPassportElementPhoneNumber::phone_number_fieldID;

inputPassportElementPhoneNumber::inputPassportElementPhoneNumber()
  : phone_number_()
{}

inputPassportElementPhoneNumber::inputPassportElementPhoneNumber(string const &phone_number_)
  : phone_number_(phone_number_)
{}

const std::int32_t inputPassportElementPhoneNumber::ID;

object_ptr<InputPassportElement> inputPassportElementPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementPhoneNumber> res = make_object<inputPassportElementPhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  return std::move(res);
}

void inputPassportElementPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementPhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_class_end();
  }
}

void inputPassportElementPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementPhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementEmailAddress::Class;
jfieldID inputPassportElementEmailAddress::email_address_fieldID;

inputPassportElementEmailAddress::inputPassportElementEmailAddress()
  : email_address_()
{}

inputPassportElementEmailAddress::inputPassportElementEmailAddress(string const &email_address_)
  : email_address_(email_address_)
{}

const std::int32_t inputPassportElementEmailAddress::ID;

object_ptr<InputPassportElement> inputPassportElementEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementEmailAddress> res = make_object<inputPassportElementEmailAddress>();
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  return std::move(res);
}

void inputPassportElementEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementEmailAddress");
    s.store_field("emailAddress", email_address_);
    s.store_class_end();
  }
}

void inputPassportElementEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementEmailAddress").c_str());
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementError::Class;
jfieldID inputPassportElementError::type_fieldID;
jfieldID inputPassportElementError::message_fieldID;
jfieldID inputPassportElementError::source_fieldID;

inputPassportElementError::inputPassportElementError()
  : type_()
  , message_()
  , source_()
{}

inputPassportElementError::inputPassportElementError(object_ptr<PassportElementType> &&type_, string const &message_, object_ptr<InputPassportElementErrorSource> &&source_)
  : type_(std::move(type_))
  , message_(message_)
  , source_(std::move(source_))
{}

const std::int32_t inputPassportElementError::ID;

object_ptr<inputPassportElementError> inputPassportElementError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementError> res = make_object<inputPassportElementError>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->message_ = jni::fetch_string(env, p, res->message_fieldID);
  res->source_ = jni::fetch_tl_object<InputPassportElementErrorSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  return res;
}

void inputPassportElementError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementError");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("message", message_);
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_class_end();
  }
}

void inputPassportElementError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementError").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", "Ljava/lang/String;");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPassportElementErrorSource;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputPassportElementErrorSource::Class;

object_ptr<InputPassportElementErrorSource> InputPassportElementErrorSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputPassportElementErrorSourceUnspecified::ID:
      return inputPassportElementErrorSourceUnspecified::fetch(env, p);
    case inputPassportElementErrorSourceDataField::ID:
      return inputPassportElementErrorSourceDataField::fetch(env, p);
    case inputPassportElementErrorSourceFrontSide::ID:
      return inputPassportElementErrorSourceFrontSide::fetch(env, p);
    case inputPassportElementErrorSourceReverseSide::ID:
      return inputPassportElementErrorSourceReverseSide::fetch(env, p);
    case inputPassportElementErrorSourceSelfie::ID:
      return inputPassportElementErrorSourceSelfie::fetch(env, p);
    case inputPassportElementErrorSourceTranslationFile::ID:
      return inputPassportElementErrorSourceTranslationFile::fetch(env, p);
    case inputPassportElementErrorSourceTranslationFiles::ID:
      return inputPassportElementErrorSourceTranslationFiles::fetch(env, p);
    case inputPassportElementErrorSourceFile::ID:
      return inputPassportElementErrorSourceFile::fetch(env, p);
    case inputPassportElementErrorSourceFiles::ID:
      return inputPassportElementErrorSourceFiles::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputPassportElementErrorSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceUnspecified::Class;
jfieldID inputPassportElementErrorSourceUnspecified::element_hash_fieldID;

inputPassportElementErrorSourceUnspecified::inputPassportElementErrorSourceUnspecified()
  : element_hash_()
{}

inputPassportElementErrorSourceUnspecified::inputPassportElementErrorSourceUnspecified(bytes const &element_hash_)
  : element_hash_(std::move(element_hash_))
{}

const std::int32_t inputPassportElementErrorSourceUnspecified::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceUnspecified::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceUnspecified> res = make_object<inputPassportElementErrorSourceUnspecified>();
  res->element_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->element_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceUnspecified::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceUnspecified");
    s.store_bytes_field("elementHash", element_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceUnspecified::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceUnspecified").c_str());
    element_hash_fieldID = jni::get_field_id(env, Class, "elementHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceDataField::Class;
jfieldID inputPassportElementErrorSourceDataField::field_name_fieldID;
jfieldID inputPassportElementErrorSourceDataField::data_hash_fieldID;

inputPassportElementErrorSourceDataField::inputPassportElementErrorSourceDataField()
  : field_name_()
  , data_hash_()
{}

inputPassportElementErrorSourceDataField::inputPassportElementErrorSourceDataField(string const &field_name_, bytes const &data_hash_)
  : field_name_(field_name_)
  , data_hash_(std::move(data_hash_))
{}

const std::int32_t inputPassportElementErrorSourceDataField::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceDataField::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceDataField> res = make_object<inputPassportElementErrorSourceDataField>();
  res->field_name_ = jni::fetch_string(env, p, res->field_name_fieldID);
  res->data_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceDataField::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceDataField");
    s.store_field("fieldName", field_name_);
    s.store_bytes_field("dataHash", data_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceDataField::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceDataField").c_str());
    field_name_fieldID = jni::get_field_id(env, Class, "fieldName", "Ljava/lang/String;");
    data_hash_fieldID = jni::get_field_id(env, Class, "dataHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceFrontSide::Class;
jfieldID inputPassportElementErrorSourceFrontSide::file_hash_fieldID;

inputPassportElementErrorSourceFrontSide::inputPassportElementErrorSourceFrontSide()
  : file_hash_()
{}

inputPassportElementErrorSourceFrontSide::inputPassportElementErrorSourceFrontSide(bytes const &file_hash_)
  : file_hash_(std::move(file_hash_))
{}

const std::int32_t inputPassportElementErrorSourceFrontSide::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceFrontSide::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceFrontSide> res = make_object<inputPassportElementErrorSourceFrontSide>();
  res->file_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->file_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceFrontSide::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceFrontSide");
    s.store_bytes_field("fileHash", file_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceFrontSide::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceFrontSide").c_str());
    file_hash_fieldID = jni::get_field_id(env, Class, "fileHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceReverseSide::Class;
jfieldID inputPassportElementErrorSourceReverseSide::file_hash_fieldID;

inputPassportElementErrorSourceReverseSide::inputPassportElementErrorSourceReverseSide()
  : file_hash_()
{}

inputPassportElementErrorSourceReverseSide::inputPassportElementErrorSourceReverseSide(bytes const &file_hash_)
  : file_hash_(std::move(file_hash_))
{}

const std::int32_t inputPassportElementErrorSourceReverseSide::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceReverseSide::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceReverseSide> res = make_object<inputPassportElementErrorSourceReverseSide>();
  res->file_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->file_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceReverseSide::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceReverseSide");
    s.store_bytes_field("fileHash", file_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceReverseSide::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceReverseSide").c_str());
    file_hash_fieldID = jni::get_field_id(env, Class, "fileHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceSelfie::Class;
jfieldID inputPassportElementErrorSourceSelfie::file_hash_fieldID;

inputPassportElementErrorSourceSelfie::inputPassportElementErrorSourceSelfie()
  : file_hash_()
{}

inputPassportElementErrorSourceSelfie::inputPassportElementErrorSourceSelfie(bytes const &file_hash_)
  : file_hash_(std::move(file_hash_))
{}

const std::int32_t inputPassportElementErrorSourceSelfie::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceSelfie::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceSelfie> res = make_object<inputPassportElementErrorSourceSelfie>();
  res->file_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->file_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceSelfie::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceSelfie");
    s.store_bytes_field("fileHash", file_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceSelfie::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceSelfie").c_str());
    file_hash_fieldID = jni::get_field_id(env, Class, "fileHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceTranslationFile::Class;
jfieldID inputPassportElementErrorSourceTranslationFile::file_hash_fieldID;

inputPassportElementErrorSourceTranslationFile::inputPassportElementErrorSourceTranslationFile()
  : file_hash_()
{}

inputPassportElementErrorSourceTranslationFile::inputPassportElementErrorSourceTranslationFile(bytes const &file_hash_)
  : file_hash_(std::move(file_hash_))
{}

const std::int32_t inputPassportElementErrorSourceTranslationFile::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceTranslationFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceTranslationFile> res = make_object<inputPassportElementErrorSourceTranslationFile>();
  res->file_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->file_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceTranslationFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceTranslationFile");
    s.store_bytes_field("fileHash", file_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceTranslationFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceTranslationFile").c_str());
    file_hash_fieldID = jni::get_field_id(env, Class, "fileHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceTranslationFiles::Class;
jfieldID inputPassportElementErrorSourceTranslationFiles::file_hashes_fieldID;

inputPassportElementErrorSourceTranslationFiles::inputPassportElementErrorSourceTranslationFiles()
  : file_hashes_()
{}

inputPassportElementErrorSourceTranslationFiles::inputPassportElementErrorSourceTranslationFiles(array<bytes> &&file_hashes_)
  : file_hashes_(std::move(file_hashes_))
{}

const std::int32_t inputPassportElementErrorSourceTranslationFiles::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceTranslationFiles::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceTranslationFiles> res = make_object<inputPassportElementErrorSourceTranslationFiles>();
  res->file_hashes_ = jni::FetchVector<jbyteArray>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->file_hashes_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceTranslationFiles::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceTranslationFiles");
    { s.store_vector_begin("fileHashes", file_hashes_.size()); for (const auto &_value : file_hashes_) { s.store_bytes_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceTranslationFiles::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceTranslationFiles").c_str());
    file_hashes_fieldID = jni::get_field_id(env, Class, "fileHashes", "[[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceFile::Class;
jfieldID inputPassportElementErrorSourceFile::file_hash_fieldID;

inputPassportElementErrorSourceFile::inputPassportElementErrorSourceFile()
  : file_hash_()
{}

inputPassportElementErrorSourceFile::inputPassportElementErrorSourceFile(bytes const &file_hash_)
  : file_hash_(std::move(file_hash_))
{}

const std::int32_t inputPassportElementErrorSourceFile::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceFile> res = make_object<inputPassportElementErrorSourceFile>();
  res->file_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->file_hash_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceFile");
    s.store_bytes_field("fileHash", file_hash_);
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceFile").c_str());
    file_hash_fieldID = jni::get_field_id(env, Class, "fileHash", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPassportElementErrorSourceFiles::Class;
jfieldID inputPassportElementErrorSourceFiles::file_hashes_fieldID;

inputPassportElementErrorSourceFiles::inputPassportElementErrorSourceFiles()
  : file_hashes_()
{}

inputPassportElementErrorSourceFiles::inputPassportElementErrorSourceFiles(array<bytes> &&file_hashes_)
  : file_hashes_(std::move(file_hashes_))
{}

const std::int32_t inputPassportElementErrorSourceFiles::ID;

object_ptr<InputPassportElementErrorSource> inputPassportElementErrorSourceFiles::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPassportElementErrorSourceFiles> res = make_object<inputPassportElementErrorSourceFiles>();
  res->file_hashes_ = jni::FetchVector<jbyteArray>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->file_hashes_fieldID));
  return std::move(res);
}

void inputPassportElementErrorSourceFiles::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPassportElementErrorSourceFiles");
    { s.store_vector_begin("fileHashes", file_hashes_.size()); for (const auto &_value : file_hashes_) { s.store_bytes_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputPassportElementErrorSourceFiles::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPassportElementErrorSourceFiles").c_str());
    file_hashes_fieldID = jni::get_field_id(env, Class, "fileHashes", "[[B");
    return true;
  }();
  (void)is_inited;
}

jclass inputPersonalDocument::Class;
jfieldID inputPersonalDocument::files_fieldID;
jfieldID inputPersonalDocument::translation_fieldID;

inputPersonalDocument::inputPersonalDocument()
  : files_()
  , translation_()
{}

inputPersonalDocument::inputPersonalDocument(array<object_ptr<InputFile>> &&files_, array<object_ptr<InputFile>> &&translation_)
  : files_(std::move(files_))
  , translation_(std::move(translation_))
{}

const std::int32_t inputPersonalDocument::ID;

object_ptr<inputPersonalDocument> inputPersonalDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputPersonalDocument> res = make_object<inputPersonalDocument>();
  res->files_ = jni::FetchVector<InputFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->files_fieldID));
  res->translation_ = jni::FetchVector<InputFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->translation_fieldID));
  return res;
}

void inputPersonalDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputPersonalDocument");
    { s.store_vector_begin("files", files_.size()); for (const auto &_value : files_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("translation", translation_.size()); for (const auto &_value : translation_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputPersonalDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputPersonalDocument").c_str());
    files_fieldID = jni::get_field_id(env, Class, "files", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    translation_fieldID = jni::get_field_id(env, Class, "translation", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputSticker::Class;
jfieldID inputSticker::sticker_fieldID;
jfieldID inputSticker::format_fieldID;
jfieldID inputSticker::emojis_fieldID;
jfieldID inputSticker::mask_position_fieldID;
jfieldID inputSticker::keywords_fieldID;

inputSticker::inputSticker()
  : sticker_()
  , format_()
  , emojis_()
  , mask_position_()
  , keywords_()
{}

inputSticker::inputSticker(object_ptr<InputFile> &&sticker_, object_ptr<StickerFormat> &&format_, string const &emojis_, object_ptr<maskPosition> &&mask_position_, array<string> &&keywords_)
  : sticker_(std::move(sticker_))
  , format_(std::move(format_))
  , emojis_(emojis_)
  , mask_position_(std::move(mask_position_))
  , keywords_(std::move(keywords_))
{}

const std::int32_t inputSticker::ID;

object_ptr<inputSticker> inputSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputSticker> res = make_object<inputSticker>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->format_ = jni::fetch_tl_object<StickerFormat>(env, jni::fetch_object(env, p, res->format_fieldID));
  res->emojis_ = jni::fetch_string(env, p, res->emojis_fieldID);
  res->mask_position_ = jni::fetch_tl_object<maskPosition>(env, jni::fetch_object(env, p, res->mask_position_fieldID));
  res->keywords_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->keywords_fieldID));
  return res;
}

void inputSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_object_field("format", static_cast<const BaseObject *>(format_.get()));
    s.store_field("emojis", emojis_);
    s.store_object_field("maskPosition", static_cast<const BaseObject *>(mask_position_.get()));
    { s.store_vector_begin("keywords", keywords_.size()); for (const auto &_value : keywords_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    format_fieldID = jni::get_field_id(env, Class, "format", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerFormat;").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "Ljava/lang/String;");
    mask_position_fieldID = jni::get_field_id(env, Class, "maskPosition", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MaskPosition;").c_str());
    keywords_fieldID = jni::get_field_id(env, Class, "keywords", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryArea::Class;
jfieldID inputStoryArea::position_fieldID;
jfieldID inputStoryArea::type_fieldID;

inputStoryArea::inputStoryArea()
  : position_()
  , type_()
{}

inputStoryArea::inputStoryArea(object_ptr<storyAreaPosition> &&position_, object_ptr<InputStoryAreaType> &&type_)
  : position_(std::move(position_))
  , type_(std::move(type_))
{}

const std::int32_t inputStoryArea::ID;

object_ptr<inputStoryArea> inputStoryArea::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryArea> res = make_object<inputStoryArea>();
  res->position_ = jni::fetch_tl_object<storyAreaPosition>(env, jni::fetch_object(env, p, res->position_fieldID));
  res->type_ = jni::fetch_tl_object<InputStoryAreaType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void inputStoryArea::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryArea");
    s.store_object_field("position", static_cast<const BaseObject *>(position_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void inputStoryArea::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryArea").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryAreaPosition;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryAreaType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputStoryAreaType::Class;

object_ptr<InputStoryAreaType> InputStoryAreaType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputStoryAreaTypeLocation::ID:
      return inputStoryAreaTypeLocation::fetch(env, p);
    case inputStoryAreaTypeFoundVenue::ID:
      return inputStoryAreaTypeFoundVenue::fetch(env, p);
    case inputStoryAreaTypePreviousVenue::ID:
      return inputStoryAreaTypePreviousVenue::fetch(env, p);
    case inputStoryAreaTypeSuggestedReaction::ID:
      return inputStoryAreaTypeSuggestedReaction::fetch(env, p);
    case inputStoryAreaTypeMessage::ID:
      return inputStoryAreaTypeMessage::fetch(env, p);
    case inputStoryAreaTypeLink::ID:
      return inputStoryAreaTypeLink::fetch(env, p);
    case inputStoryAreaTypeWeather::ID:
      return inputStoryAreaTypeWeather::fetch(env, p);
    case inputStoryAreaTypeUpgradedGift::ID:
      return inputStoryAreaTypeUpgradedGift::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputStoryAreaType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeLocation::Class;
jfieldID inputStoryAreaTypeLocation::location_fieldID;
jfieldID inputStoryAreaTypeLocation::address_fieldID;

inputStoryAreaTypeLocation::inputStoryAreaTypeLocation()
  : location_()
  , address_()
{}

inputStoryAreaTypeLocation::inputStoryAreaTypeLocation(object_ptr<location> &&location_, object_ptr<locationAddress> &&address_)
  : location_(std::move(location_))
  , address_(std::move(address_))
{}

const std::int32_t inputStoryAreaTypeLocation::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeLocation> res = make_object<inputStoryAreaTypeLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->address_ = jni::fetch_tl_object<locationAddress>(env, jni::fetch_object(env, p, res->address_fieldID));
  return std::move(res);
}

void inputStoryAreaTypeLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_object_field("address", static_cast<const BaseObject *>(address_.get()));
    s.store_class_end();
  }
}

void inputStoryAreaTypeLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LocationAddress;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeFoundVenue::Class;
jfieldID inputStoryAreaTypeFoundVenue::query_id_fieldID;
jfieldID inputStoryAreaTypeFoundVenue::result_id_fieldID;

inputStoryAreaTypeFoundVenue::inputStoryAreaTypeFoundVenue()
  : query_id_()
  , result_id_()
{}

inputStoryAreaTypeFoundVenue::inputStoryAreaTypeFoundVenue(int64 query_id_, string const &result_id_)
  : query_id_(query_id_)
  , result_id_(result_id_)
{}

const std::int32_t inputStoryAreaTypeFoundVenue::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeFoundVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeFoundVenue> res = make_object<inputStoryAreaTypeFoundVenue>();
  res->query_id_ = env->GetLongField(p, res->query_id_fieldID);
  res->result_id_ = jni::fetch_string(env, p, res->result_id_fieldID);
  return std::move(res);
}

void inputStoryAreaTypeFoundVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeFoundVenue");
    s.store_field("queryId", query_id_);
    s.store_field("resultId", result_id_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeFoundVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeFoundVenue").c_str());
    query_id_fieldID = jni::get_field_id(env, Class, "queryId", "J");
    result_id_fieldID = jni::get_field_id(env, Class, "resultId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypePreviousVenue::Class;
jfieldID inputStoryAreaTypePreviousVenue::venue_provider_fieldID;
jfieldID inputStoryAreaTypePreviousVenue::venue_id_fieldID;

inputStoryAreaTypePreviousVenue::inputStoryAreaTypePreviousVenue()
  : venue_provider_()
  , venue_id_()
{}

inputStoryAreaTypePreviousVenue::inputStoryAreaTypePreviousVenue(string const &venue_provider_, string const &venue_id_)
  : venue_provider_(venue_provider_)
  , venue_id_(venue_id_)
{}

const std::int32_t inputStoryAreaTypePreviousVenue::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypePreviousVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypePreviousVenue> res = make_object<inputStoryAreaTypePreviousVenue>();
  res->venue_provider_ = jni::fetch_string(env, p, res->venue_provider_fieldID);
  res->venue_id_ = jni::fetch_string(env, p, res->venue_id_fieldID);
  return std::move(res);
}

void inputStoryAreaTypePreviousVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypePreviousVenue");
    s.store_field("venueProvider", venue_provider_);
    s.store_field("venueId", venue_id_);
    s.store_class_end();
  }
}

void inputStoryAreaTypePreviousVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypePreviousVenue").c_str());
    venue_provider_fieldID = jni::get_field_id(env, Class, "venueProvider", "Ljava/lang/String;");
    venue_id_fieldID = jni::get_field_id(env, Class, "venueId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeSuggestedReaction::Class;
jfieldID inputStoryAreaTypeSuggestedReaction::reaction_type_fieldID;
jfieldID inputStoryAreaTypeSuggestedReaction::is_dark_fieldID;
jfieldID inputStoryAreaTypeSuggestedReaction::is_flipped_fieldID;

inputStoryAreaTypeSuggestedReaction::inputStoryAreaTypeSuggestedReaction()
  : reaction_type_()
  , is_dark_()
  , is_flipped_()
{}

inputStoryAreaTypeSuggestedReaction::inputStoryAreaTypeSuggestedReaction(object_ptr<ReactionType> &&reaction_type_, bool is_dark_, bool is_flipped_)
  : reaction_type_(std::move(reaction_type_))
  , is_dark_(is_dark_)
  , is_flipped_(is_flipped_)
{}

const std::int32_t inputStoryAreaTypeSuggestedReaction::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeSuggestedReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeSuggestedReaction> res = make_object<inputStoryAreaTypeSuggestedReaction>();
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  res->is_flipped_ = (env->GetBooleanField(p, res->is_flipped_fieldID) != 0);
  return std::move(res);
}

void inputStoryAreaTypeSuggestedReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeSuggestedReaction");
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("isDark", is_dark_);
    s.store_field("isFlipped", is_flipped_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeSuggestedReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeSuggestedReaction").c_str());
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    is_flipped_fieldID = jni::get_field_id(env, Class, "isFlipped", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeMessage::Class;
jfieldID inputStoryAreaTypeMessage::chat_id_fieldID;
jfieldID inputStoryAreaTypeMessage::message_id_fieldID;

inputStoryAreaTypeMessage::inputStoryAreaTypeMessage()
  : chat_id_()
  , message_id_()
{}

inputStoryAreaTypeMessage::inputStoryAreaTypeMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t inputStoryAreaTypeMessage::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeMessage> res = make_object<inputStoryAreaTypeMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void inputStoryAreaTypeMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeLink::Class;
jfieldID inputStoryAreaTypeLink::url_fieldID;

inputStoryAreaTypeLink::inputStoryAreaTypeLink()
  : url_()
{}

inputStoryAreaTypeLink::inputStoryAreaTypeLink(string const &url_)
  : url_(url_)
{}

const std::int32_t inputStoryAreaTypeLink::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeLink> res = make_object<inputStoryAreaTypeLink>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void inputStoryAreaTypeLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeLink");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeLink").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeWeather::Class;
jfieldID inputStoryAreaTypeWeather::temperature_fieldID;
jfieldID inputStoryAreaTypeWeather::emoji_fieldID;
jfieldID inputStoryAreaTypeWeather::background_color_fieldID;

inputStoryAreaTypeWeather::inputStoryAreaTypeWeather()
  : temperature_()
  , emoji_()
  , background_color_()
{}

inputStoryAreaTypeWeather::inputStoryAreaTypeWeather(double temperature_, string const &emoji_, int32 background_color_)
  : temperature_(temperature_)
  , emoji_(emoji_)
  , background_color_(background_color_)
{}

const std::int32_t inputStoryAreaTypeWeather::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeWeather::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeWeather> res = make_object<inputStoryAreaTypeWeather>();
  res->temperature_ = env->GetDoubleField(p, res->temperature_fieldID);
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->background_color_ = env->GetIntField(p, res->background_color_fieldID);
  return std::move(res);
}

void inputStoryAreaTypeWeather::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeWeather");
    s.store_field("temperature", temperature_);
    s.store_field("emoji", emoji_);
    s.store_field("backgroundColor", background_color_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeWeather::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeWeather").c_str());
    temperature_fieldID = jni::get_field_id(env, Class, "temperature", "D");
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    background_color_fieldID = jni::get_field_id(env, Class, "backgroundColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreaTypeUpgradedGift::Class;
jfieldID inputStoryAreaTypeUpgradedGift::gift_name_fieldID;

inputStoryAreaTypeUpgradedGift::inputStoryAreaTypeUpgradedGift()
  : gift_name_()
{}

inputStoryAreaTypeUpgradedGift::inputStoryAreaTypeUpgradedGift(string const &gift_name_)
  : gift_name_(gift_name_)
{}

const std::int32_t inputStoryAreaTypeUpgradedGift::ID;

object_ptr<InputStoryAreaType> inputStoryAreaTypeUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreaTypeUpgradedGift> res = make_object<inputStoryAreaTypeUpgradedGift>();
  res->gift_name_ = jni::fetch_string(env, p, res->gift_name_fieldID);
  return std::move(res);
}

void inputStoryAreaTypeUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreaTypeUpgradedGift");
    s.store_field("giftName", gift_name_);
    s.store_class_end();
  }
}

void inputStoryAreaTypeUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreaTypeUpgradedGift").c_str());
    gift_name_fieldID = jni::get_field_id(env, Class, "giftName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryAreas::Class;
jfieldID inputStoryAreas::areas_fieldID;

inputStoryAreas::inputStoryAreas()
  : areas_()
{}

inputStoryAreas::inputStoryAreas(array<object_ptr<inputStoryArea>> &&areas_)
  : areas_(std::move(areas_))
{}

const std::int32_t inputStoryAreas::ID;

object_ptr<inputStoryAreas> inputStoryAreas::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryAreas> res = make_object<inputStoryAreas>();
  res->areas_ = jni::FetchVector<inputStoryArea>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->areas_fieldID));
  return res;
}

void inputStoryAreas::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryAreas");
    { s.store_vector_begin("areas", areas_.size()); for (const auto &_value : areas_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputStoryAreas::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryAreas").c_str());
    areas_fieldID = jni::get_field_id(env, Class, "areas", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputStoryArea;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InputStoryContent::Class;

object_ptr<InputStoryContent> InputStoryContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inputStoryContentPhoto::ID:
      return inputStoryContentPhoto::fetch(env, p);
    case inputStoryContentVideo::ID:
      return inputStoryContentVideo::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InputStoryContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryContent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryContentPhoto::Class;
jfieldID inputStoryContentPhoto::photo_fieldID;
jfieldID inputStoryContentPhoto::added_sticker_file_ids_fieldID;

inputStoryContentPhoto::inputStoryContentPhoto()
  : photo_()
  , added_sticker_file_ids_()
{}

inputStoryContentPhoto::inputStoryContentPhoto(object_ptr<InputFile> &&photo_, array<int32> &&added_sticker_file_ids_)
  : photo_(std::move(photo_))
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
{}

const std::int32_t inputStoryContentPhoto::ID;

object_ptr<InputStoryContent> inputStoryContentPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryContentPhoto> res = make_object<inputStoryContentPhoto>();
  res->photo_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  return std::move(res);
}

void inputStoryContentPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryContentPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void inputStoryContentPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryContentPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass inputStoryContentVideo::Class;
jfieldID inputStoryContentVideo::video_fieldID;
jfieldID inputStoryContentVideo::added_sticker_file_ids_fieldID;
jfieldID inputStoryContentVideo::duration_fieldID;
jfieldID inputStoryContentVideo::cover_frame_timestamp_fieldID;
jfieldID inputStoryContentVideo::is_animation_fieldID;

inputStoryContentVideo::inputStoryContentVideo()
  : video_()
  , added_sticker_file_ids_()
  , duration_()
  , cover_frame_timestamp_()
  , is_animation_()
{}

inputStoryContentVideo::inputStoryContentVideo(object_ptr<InputFile> &&video_, array<int32> &&added_sticker_file_ids_, double duration_, double cover_frame_timestamp_, bool is_animation_)
  : video_(std::move(video_))
  , added_sticker_file_ids_(std::move(added_sticker_file_ids_))
  , duration_(duration_)
  , cover_frame_timestamp_(cover_frame_timestamp_)
  , is_animation_(is_animation_)
{}

const std::int32_t inputStoryContentVideo::ID;

object_ptr<InputStoryContent> inputStoryContentVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputStoryContentVideo> res = make_object<inputStoryContentVideo>();
  res->video_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->added_sticker_file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->added_sticker_file_ids_fieldID));
  res->duration_ = env->GetDoubleField(p, res->duration_fieldID);
  res->cover_frame_timestamp_ = env->GetDoubleField(p, res->cover_frame_timestamp_fieldID);
  res->is_animation_ = (env->GetBooleanField(p, res->is_animation_fieldID) != 0);
  return std::move(res);
}

void inputStoryContentVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputStoryContentVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    { s.store_vector_begin("addedStickerFileIds", added_sticker_file_ids_.size()); for (const auto &_value : added_sticker_file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("duration", duration_);
    s.store_field("coverFrameTimestamp", cover_frame_timestamp_);
    s.store_field("isAnimation", is_animation_);
    s.store_class_end();
  }
}

void inputStoryContentVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputStoryContentVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    added_sticker_file_ids_fieldID = jni::get_field_id(env, Class, "addedStickerFileIds", "[I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "D");
    cover_frame_timestamp_fieldID = jni::get_field_id(env, Class, "coverFrameTimestamp", "D");
    is_animation_fieldID = jni::get_field_id(env, Class, "isAnimation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inputTextQuote::Class;
jfieldID inputTextQuote::text_fieldID;
jfieldID inputTextQuote::position_fieldID;

inputTextQuote::inputTextQuote()
  : text_()
  , position_()
{}

inputTextQuote::inputTextQuote(object_ptr<formattedText> &&text_, int32 position_)
  : text_(std::move(text_))
  , position_(position_)
{}

const std::int32_t inputTextQuote::ID;

object_ptr<inputTextQuote> inputTextQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputTextQuote> res = make_object<inputTextQuote>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->position_ = env->GetIntField(p, res->position_fieldID);
  return res;
}

void inputTextQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, position_fieldID, position_);
}

void inputTextQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputTextQuote");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("position", position_);
    s.store_class_end();
  }
}

void inputTextQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputTextQuote").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    return true;
  }();
  (void)is_inited;
}

jclass inputThumbnail::Class;
jfieldID inputThumbnail::thumbnail_fieldID;
jfieldID inputThumbnail::width_fieldID;
jfieldID inputThumbnail::height_fieldID;

inputThumbnail::inputThumbnail()
  : thumbnail_()
  , width_()
  , height_()
{}

inputThumbnail::inputThumbnail(object_ptr<InputFile> &&thumbnail_, int32 width_, int32 height_)
  : thumbnail_(std::move(thumbnail_))
  , width_(width_)
  , height_(height_)
{}

const std::int32_t inputThumbnail::ID;

object_ptr<inputThumbnail> inputThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inputThumbnail> res = make_object<inputThumbnail>();
  res->thumbnail_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return res;
}

void inputThumbnail::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void inputThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InputThumbnail");
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void inputThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InputThumbnail").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass InternalLinkType::Class;

object_ptr<InternalLinkType> InternalLinkType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case internalLinkTypeActiveSessions::ID:
      return internalLinkTypeActiveSessions::fetch(env, p);
    case internalLinkTypeAttachmentMenuBot::ID:
      return internalLinkTypeAttachmentMenuBot::fetch(env, p);
    case internalLinkTypeAuthenticationCode::ID:
      return internalLinkTypeAuthenticationCode::fetch(env, p);
    case internalLinkTypeBackground::ID:
      return internalLinkTypeBackground::fetch(env, p);
    case internalLinkTypeBotAddToChannel::ID:
      return internalLinkTypeBotAddToChannel::fetch(env, p);
    case internalLinkTypeBotStart::ID:
      return internalLinkTypeBotStart::fetch(env, p);
    case internalLinkTypeBotStartInGroup::ID:
      return internalLinkTypeBotStartInGroup::fetch(env, p);
    case internalLinkTypeBusinessChat::ID:
      return internalLinkTypeBusinessChat::fetch(env, p);
    case internalLinkTypeBuyStars::ID:
      return internalLinkTypeBuyStars::fetch(env, p);
    case internalLinkTypeChangePhoneNumber::ID:
      return internalLinkTypeChangePhoneNumber::fetch(env, p);
    case internalLinkTypeChatAffiliateProgram::ID:
      return internalLinkTypeChatAffiliateProgram::fetch(env, p);
    case internalLinkTypeChatBoost::ID:
      return internalLinkTypeChatBoost::fetch(env, p);
    case internalLinkTypeChatFolderInvite::ID:
      return internalLinkTypeChatFolderInvite::fetch(env, p);
    case internalLinkTypeChatFolderSettings::ID:
      return internalLinkTypeChatFolderSettings::fetch(env, p);
    case internalLinkTypeChatInvite::ID:
      return internalLinkTypeChatInvite::fetch(env, p);
    case internalLinkTypeDefaultMessageAutoDeleteTimerSettings::ID:
      return internalLinkTypeDefaultMessageAutoDeleteTimerSettings::fetch(env, p);
    case internalLinkTypeEditProfileSettings::ID:
      return internalLinkTypeEditProfileSettings::fetch(env, p);
    case internalLinkTypeGame::ID:
      return internalLinkTypeGame::fetch(env, p);
    case internalLinkTypeGroupCall::ID:
      return internalLinkTypeGroupCall::fetch(env, p);
    case internalLinkTypeInstantView::ID:
      return internalLinkTypeInstantView::fetch(env, p);
    case internalLinkTypeInvoice::ID:
      return internalLinkTypeInvoice::fetch(env, p);
    case internalLinkTypeLanguagePack::ID:
      return internalLinkTypeLanguagePack::fetch(env, p);
    case internalLinkTypeLanguageSettings::ID:
      return internalLinkTypeLanguageSettings::fetch(env, p);
    case internalLinkTypeMainWebApp::ID:
      return internalLinkTypeMainWebApp::fetch(env, p);
    case internalLinkTypeMessage::ID:
      return internalLinkTypeMessage::fetch(env, p);
    case internalLinkTypeMessageDraft::ID:
      return internalLinkTypeMessageDraft::fetch(env, p);
    case internalLinkTypeMyStars::ID:
      return internalLinkTypeMyStars::fetch(env, p);
    case internalLinkTypePassportDataRequest::ID:
      return internalLinkTypePassportDataRequest::fetch(env, p);
    case internalLinkTypePhoneNumberConfirmation::ID:
      return internalLinkTypePhoneNumberConfirmation::fetch(env, p);
    case internalLinkTypePremiumFeatures::ID:
      return internalLinkTypePremiumFeatures::fetch(env, p);
    case internalLinkTypePremiumGift::ID:
      return internalLinkTypePremiumGift::fetch(env, p);
    case internalLinkTypePremiumGiftCode::ID:
      return internalLinkTypePremiumGiftCode::fetch(env, p);
    case internalLinkTypePrivacyAndSecuritySettings::ID:
      return internalLinkTypePrivacyAndSecuritySettings::fetch(env, p);
    case internalLinkTypeProxy::ID:
      return internalLinkTypeProxy::fetch(env, p);
    case internalLinkTypePublicChat::ID:
      return internalLinkTypePublicChat::fetch(env, p);
    case internalLinkTypeQrCodeAuthentication::ID:
      return internalLinkTypeQrCodeAuthentication::fetch(env, p);
    case internalLinkTypeRestorePurchases::ID:
      return internalLinkTypeRestorePurchases::fetch(env, p);
    case internalLinkTypeSettings::ID:
      return internalLinkTypeSettings::fetch(env, p);
    case internalLinkTypeStickerSet::ID:
      return internalLinkTypeStickerSet::fetch(env, p);
    case internalLinkTypeStory::ID:
      return internalLinkTypeStory::fetch(env, p);
    case internalLinkTypeTheme::ID:
      return internalLinkTypeTheme::fetch(env, p);
    case internalLinkTypeThemeSettings::ID:
      return internalLinkTypeThemeSettings::fetch(env, p);
    case internalLinkTypeUnknownDeepLink::ID:
      return internalLinkTypeUnknownDeepLink::fetch(env, p);
    case internalLinkTypeUnsupportedProxy::ID:
      return internalLinkTypeUnsupportedProxy::fetch(env, p);
    case internalLinkTypeUpgradedGift::ID:
      return internalLinkTypeUpgradedGift::fetch(env, p);
    case internalLinkTypeUserPhoneNumber::ID:
      return internalLinkTypeUserPhoneNumber::fetch(env, p);
    case internalLinkTypeUserToken::ID:
      return internalLinkTypeUserToken::fetch(env, p);
    case internalLinkTypeVideoChat::ID:
      return internalLinkTypeVideoChat::fetch(env, p);
    case internalLinkTypeWebApp::ID:
      return internalLinkTypeWebApp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InternalLinkType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeActiveSessions::Class;

internalLinkTypeActiveSessions::internalLinkTypeActiveSessions() {
}

const std::int32_t internalLinkTypeActiveSessions::ID;

object_ptr<InternalLinkType> internalLinkTypeActiveSessions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeActiveSessions>();
}

void internalLinkTypeActiveSessions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeActiveSessions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeActiveSessions");
    s.store_class_end();
  }
}

void internalLinkTypeActiveSessions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeActiveSessions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeAttachmentMenuBot::Class;
jfieldID internalLinkTypeAttachmentMenuBot::target_chat_fieldID;
jfieldID internalLinkTypeAttachmentMenuBot::bot_username_fieldID;
jfieldID internalLinkTypeAttachmentMenuBot::url_fieldID;

internalLinkTypeAttachmentMenuBot::internalLinkTypeAttachmentMenuBot()
  : target_chat_()
  , bot_username_()
  , url_()
{}

internalLinkTypeAttachmentMenuBot::internalLinkTypeAttachmentMenuBot(object_ptr<TargetChat> &&target_chat_, string const &bot_username_, string const &url_)
  : target_chat_(std::move(target_chat_))
  , bot_username_(bot_username_)
  , url_(url_)
{}

const std::int32_t internalLinkTypeAttachmentMenuBot::ID;

object_ptr<InternalLinkType> internalLinkTypeAttachmentMenuBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeAttachmentMenuBot> res = make_object<internalLinkTypeAttachmentMenuBot>();
  res->target_chat_ = jni::fetch_tl_object<TargetChat>(env, jni::fetch_object(env, p, res->target_chat_fieldID));
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void internalLinkTypeAttachmentMenuBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (target_chat_ != nullptr) { jobject next; target_chat_->store(env, next); if (next) { env->SetObjectField(s, target_chat_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeAttachmentMenuBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeAttachmentMenuBot");
    s.store_object_field("targetChat", static_cast<const BaseObject *>(target_chat_.get()));
    s.store_field("botUsername", bot_username_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void internalLinkTypeAttachmentMenuBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeAttachmentMenuBot").c_str());
    target_chat_fieldID = jni::get_field_id(env, Class, "targetChat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TargetChat;").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeAuthenticationCode::Class;
jfieldID internalLinkTypeAuthenticationCode::code_fieldID;

internalLinkTypeAuthenticationCode::internalLinkTypeAuthenticationCode()
  : code_()
{}

internalLinkTypeAuthenticationCode::internalLinkTypeAuthenticationCode(string const &code_)
  : code_(code_)
{}

const std::int32_t internalLinkTypeAuthenticationCode::ID;

object_ptr<InternalLinkType> internalLinkTypeAuthenticationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeAuthenticationCode> res = make_object<internalLinkTypeAuthenticationCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return std::move(res);
}

void internalLinkTypeAuthenticationCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, code_); if (nextString) { env->SetObjectField(s, code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeAuthenticationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeAuthenticationCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

void internalLinkTypeAuthenticationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeAuthenticationCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBackground::Class;
jfieldID internalLinkTypeBackground::background_name_fieldID;

internalLinkTypeBackground::internalLinkTypeBackground()
  : background_name_()
{}

internalLinkTypeBackground::internalLinkTypeBackground(string const &background_name_)
  : background_name_(background_name_)
{}

const std::int32_t internalLinkTypeBackground::ID;

object_ptr<InternalLinkType> internalLinkTypeBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBackground> res = make_object<internalLinkTypeBackground>();
  res->background_name_ = jni::fetch_string(env, p, res->background_name_fieldID);
  return std::move(res);
}

void internalLinkTypeBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, background_name_); if (nextString) { env->SetObjectField(s, background_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBackground");
    s.store_field("backgroundName", background_name_);
    s.store_class_end();
  }
}

void internalLinkTypeBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBackground").c_str());
    background_name_fieldID = jni::get_field_id(env, Class, "backgroundName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBotAddToChannel::Class;
jfieldID internalLinkTypeBotAddToChannel::bot_username_fieldID;
jfieldID internalLinkTypeBotAddToChannel::administrator_rights_fieldID;

internalLinkTypeBotAddToChannel::internalLinkTypeBotAddToChannel()
  : bot_username_()
  , administrator_rights_()
{}

internalLinkTypeBotAddToChannel::internalLinkTypeBotAddToChannel(string const &bot_username_, object_ptr<chatAdministratorRights> &&administrator_rights_)
  : bot_username_(bot_username_)
  , administrator_rights_(std::move(administrator_rights_))
{}

const std::int32_t internalLinkTypeBotAddToChannel::ID;

object_ptr<InternalLinkType> internalLinkTypeBotAddToChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBotAddToChannel> res = make_object<internalLinkTypeBotAddToChannel>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->administrator_rights_fieldID));
  return std::move(res);
}

void internalLinkTypeBotAddToChannel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (administrator_rights_ != nullptr) { jobject next; administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
}

void internalLinkTypeBotAddToChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBotAddToChannel");
    s.store_field("botUsername", bot_username_);
    s.store_object_field("administratorRights", static_cast<const BaseObject *>(administrator_rights_.get()));
    s.store_class_end();
  }
}

void internalLinkTypeBotAddToChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBotAddToChannel").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    administrator_rights_fieldID = jni::get_field_id(env, Class, "administratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBotStart::Class;
jfieldID internalLinkTypeBotStart::bot_username_fieldID;
jfieldID internalLinkTypeBotStart::start_parameter_fieldID;
jfieldID internalLinkTypeBotStart::autostart_fieldID;

internalLinkTypeBotStart::internalLinkTypeBotStart()
  : bot_username_()
  , start_parameter_()
  , autostart_()
{}

internalLinkTypeBotStart::internalLinkTypeBotStart(string const &bot_username_, string const &start_parameter_, bool autostart_)
  : bot_username_(bot_username_)
  , start_parameter_(start_parameter_)
  , autostart_(autostart_)
{}

const std::int32_t internalLinkTypeBotStart::ID;

object_ptr<InternalLinkType> internalLinkTypeBotStart::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBotStart> res = make_object<internalLinkTypeBotStart>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->autostart_ = (env->GetBooleanField(p, res->autostart_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypeBotStart::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, autostart_fieldID, autostart_);
}

void internalLinkTypeBotStart::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBotStart");
    s.store_field("botUsername", bot_username_);
    s.store_field("startParameter", start_parameter_);
    s.store_field("autostart", autostart_);
    s.store_class_end();
  }
}

void internalLinkTypeBotStart::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBotStart").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    autostart_fieldID = jni::get_field_id(env, Class, "autostart", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBotStartInGroup::Class;
jfieldID internalLinkTypeBotStartInGroup::bot_username_fieldID;
jfieldID internalLinkTypeBotStartInGroup::start_parameter_fieldID;
jfieldID internalLinkTypeBotStartInGroup::administrator_rights_fieldID;

internalLinkTypeBotStartInGroup::internalLinkTypeBotStartInGroup()
  : bot_username_()
  , start_parameter_()
  , administrator_rights_()
{}

internalLinkTypeBotStartInGroup::internalLinkTypeBotStartInGroup(string const &bot_username_, string const &start_parameter_, object_ptr<chatAdministratorRights> &&administrator_rights_)
  : bot_username_(bot_username_)
  , start_parameter_(start_parameter_)
  , administrator_rights_(std::move(administrator_rights_))
{}

const std::int32_t internalLinkTypeBotStartInGroup::ID;

object_ptr<InternalLinkType> internalLinkTypeBotStartInGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBotStartInGroup> res = make_object<internalLinkTypeBotStartInGroup>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->administrator_rights_fieldID));
  return std::move(res);
}

void internalLinkTypeBotStartInGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (administrator_rights_ != nullptr) { jobject next; administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
}

void internalLinkTypeBotStartInGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBotStartInGroup");
    s.store_field("botUsername", bot_username_);
    s.store_field("startParameter", start_parameter_);
    s.store_object_field("administratorRights", static_cast<const BaseObject *>(administrator_rights_.get()));
    s.store_class_end();
  }
}

void internalLinkTypeBotStartInGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBotStartInGroup").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    administrator_rights_fieldID = jni::get_field_id(env, Class, "administratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBusinessChat::Class;
jfieldID internalLinkTypeBusinessChat::link_name_fieldID;

internalLinkTypeBusinessChat::internalLinkTypeBusinessChat()
  : link_name_()
{}

internalLinkTypeBusinessChat::internalLinkTypeBusinessChat(string const &link_name_)
  : link_name_(link_name_)
{}

const std::int32_t internalLinkTypeBusinessChat::ID;

object_ptr<InternalLinkType> internalLinkTypeBusinessChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBusinessChat> res = make_object<internalLinkTypeBusinessChat>();
  res->link_name_ = jni::fetch_string(env, p, res->link_name_fieldID);
  return std::move(res);
}

void internalLinkTypeBusinessChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_name_); if (nextString) { env->SetObjectField(s, link_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeBusinessChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBusinessChat");
    s.store_field("linkName", link_name_);
    s.store_class_end();
  }
}

void internalLinkTypeBusinessChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBusinessChat").c_str());
    link_name_fieldID = jni::get_field_id(env, Class, "linkName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeBuyStars::Class;
jfieldID internalLinkTypeBuyStars::star_count_fieldID;
jfieldID internalLinkTypeBuyStars::purpose_fieldID;

internalLinkTypeBuyStars::internalLinkTypeBuyStars()
  : star_count_()
  , purpose_()
{}

internalLinkTypeBuyStars::internalLinkTypeBuyStars(int53 star_count_, string const &purpose_)
  : star_count_(star_count_)
  , purpose_(purpose_)
{}

const std::int32_t internalLinkTypeBuyStars::ID;

object_ptr<InternalLinkType> internalLinkTypeBuyStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeBuyStars> res = make_object<internalLinkTypeBuyStars>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->purpose_ = jni::fetch_string(env, p, res->purpose_fieldID);
  return std::move(res);
}

void internalLinkTypeBuyStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, purpose_); if (nextString) { env->SetObjectField(s, purpose_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeBuyStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeBuyStars");
    s.store_field("starCount", star_count_);
    s.store_field("purpose", purpose_);
    s.store_class_end();
  }
}

void internalLinkTypeBuyStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeBuyStars").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    purpose_fieldID = jni::get_field_id(env, Class, "purpose", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChangePhoneNumber::Class;

internalLinkTypeChangePhoneNumber::internalLinkTypeChangePhoneNumber() {
}

const std::int32_t internalLinkTypeChangePhoneNumber::ID;

object_ptr<InternalLinkType> internalLinkTypeChangePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeChangePhoneNumber>();
}

void internalLinkTypeChangePhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeChangePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChangePhoneNumber");
    s.store_class_end();
  }
}

void internalLinkTypeChangePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChangePhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChatAffiliateProgram::Class;
jfieldID internalLinkTypeChatAffiliateProgram::username_fieldID;
jfieldID internalLinkTypeChatAffiliateProgram::referrer_fieldID;

internalLinkTypeChatAffiliateProgram::internalLinkTypeChatAffiliateProgram()
  : username_()
  , referrer_()
{}

internalLinkTypeChatAffiliateProgram::internalLinkTypeChatAffiliateProgram(string const &username_, string const &referrer_)
  : username_(username_)
  , referrer_(referrer_)
{}

const std::int32_t internalLinkTypeChatAffiliateProgram::ID;

object_ptr<InternalLinkType> internalLinkTypeChatAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeChatAffiliateProgram> res = make_object<internalLinkTypeChatAffiliateProgram>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return std::move(res);
}

void internalLinkTypeChatAffiliateProgram::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, referrer_); if (nextString) { env->SetObjectField(s, referrer_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeChatAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChatAffiliateProgram");
    s.store_field("username", username_);
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

void internalLinkTypeChatAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChatAffiliateProgram").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChatBoost::Class;
jfieldID internalLinkTypeChatBoost::url_fieldID;

internalLinkTypeChatBoost::internalLinkTypeChatBoost()
  : url_()
{}

internalLinkTypeChatBoost::internalLinkTypeChatBoost(string const &url_)
  : url_(url_)
{}

const std::int32_t internalLinkTypeChatBoost::ID;

object_ptr<InternalLinkType> internalLinkTypeChatBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeChatBoost> res = make_object<internalLinkTypeChatBoost>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void internalLinkTypeChatBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeChatBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChatBoost");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void internalLinkTypeChatBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChatBoost").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChatFolderInvite::Class;
jfieldID internalLinkTypeChatFolderInvite::invite_link_fieldID;

internalLinkTypeChatFolderInvite::internalLinkTypeChatFolderInvite()
  : invite_link_()
{}

internalLinkTypeChatFolderInvite::internalLinkTypeChatFolderInvite(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t internalLinkTypeChatFolderInvite::ID;

object_ptr<InternalLinkType> internalLinkTypeChatFolderInvite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeChatFolderInvite> res = make_object<internalLinkTypeChatFolderInvite>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void internalLinkTypeChatFolderInvite::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeChatFolderInvite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChatFolderInvite");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void internalLinkTypeChatFolderInvite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChatFolderInvite").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChatFolderSettings::Class;

internalLinkTypeChatFolderSettings::internalLinkTypeChatFolderSettings() {
}

const std::int32_t internalLinkTypeChatFolderSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeChatFolderSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeChatFolderSettings>();
}

void internalLinkTypeChatFolderSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeChatFolderSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChatFolderSettings");
    s.store_class_end();
  }
}

void internalLinkTypeChatFolderSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChatFolderSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeChatInvite::Class;
jfieldID internalLinkTypeChatInvite::invite_link_fieldID;

internalLinkTypeChatInvite::internalLinkTypeChatInvite()
  : invite_link_()
{}

internalLinkTypeChatInvite::internalLinkTypeChatInvite(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t internalLinkTypeChatInvite::ID;

object_ptr<InternalLinkType> internalLinkTypeChatInvite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeChatInvite> res = make_object<internalLinkTypeChatInvite>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void internalLinkTypeChatInvite::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeChatInvite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeChatInvite");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void internalLinkTypeChatInvite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeChatInvite").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeDefaultMessageAutoDeleteTimerSettings::Class;

internalLinkTypeDefaultMessageAutoDeleteTimerSettings::internalLinkTypeDefaultMessageAutoDeleteTimerSettings() {
}

const std::int32_t internalLinkTypeDefaultMessageAutoDeleteTimerSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeDefaultMessageAutoDeleteTimerSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeDefaultMessageAutoDeleteTimerSettings>();
}

void internalLinkTypeDefaultMessageAutoDeleteTimerSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeDefaultMessageAutoDeleteTimerSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeDefaultMessageAutoDeleteTimerSettings");
    s.store_class_end();
  }
}

void internalLinkTypeDefaultMessageAutoDeleteTimerSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeDefaultMessageAutoDeleteTimerSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeEditProfileSettings::Class;

internalLinkTypeEditProfileSettings::internalLinkTypeEditProfileSettings() {
}

const std::int32_t internalLinkTypeEditProfileSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeEditProfileSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeEditProfileSettings>();
}

void internalLinkTypeEditProfileSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeEditProfileSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeEditProfileSettings");
    s.store_class_end();
  }
}

void internalLinkTypeEditProfileSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeEditProfileSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeGame::Class;
jfieldID internalLinkTypeGame::bot_username_fieldID;
jfieldID internalLinkTypeGame::game_short_name_fieldID;

internalLinkTypeGame::internalLinkTypeGame()
  : bot_username_()
  , game_short_name_()
{}

internalLinkTypeGame::internalLinkTypeGame(string const &bot_username_, string const &game_short_name_)
  : bot_username_(bot_username_)
  , game_short_name_(game_short_name_)
{}

const std::int32_t internalLinkTypeGame::ID;

object_ptr<InternalLinkType> internalLinkTypeGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeGame> res = make_object<internalLinkTypeGame>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->game_short_name_ = jni::fetch_string(env, p, res->game_short_name_fieldID);
  return std::move(res);
}

void internalLinkTypeGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, game_short_name_); if (nextString) { env->SetObjectField(s, game_short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeGame");
    s.store_field("botUsername", bot_username_);
    s.store_field("gameShortName", game_short_name_);
    s.store_class_end();
  }
}

void internalLinkTypeGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeGame").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    game_short_name_fieldID = jni::get_field_id(env, Class, "gameShortName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeGroupCall::Class;
jfieldID internalLinkTypeGroupCall::invite_link_fieldID;

internalLinkTypeGroupCall::internalLinkTypeGroupCall()
  : invite_link_()
{}

internalLinkTypeGroupCall::internalLinkTypeGroupCall(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t internalLinkTypeGroupCall::ID;

object_ptr<InternalLinkType> internalLinkTypeGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeGroupCall> res = make_object<internalLinkTypeGroupCall>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void internalLinkTypeGroupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeGroupCall");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void internalLinkTypeGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeGroupCall").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeInstantView::Class;
jfieldID internalLinkTypeInstantView::url_fieldID;
jfieldID internalLinkTypeInstantView::fallback_url_fieldID;

internalLinkTypeInstantView::internalLinkTypeInstantView()
  : url_()
  , fallback_url_()
{}

internalLinkTypeInstantView::internalLinkTypeInstantView(string const &url_, string const &fallback_url_)
  : url_(url_)
  , fallback_url_(fallback_url_)
{}

const std::int32_t internalLinkTypeInstantView::ID;

object_ptr<InternalLinkType> internalLinkTypeInstantView::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeInstantView> res = make_object<internalLinkTypeInstantView>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->fallback_url_ = jni::fetch_string(env, p, res->fallback_url_fieldID);
  return std::move(res);
}

void internalLinkTypeInstantView::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, fallback_url_); if (nextString) { env->SetObjectField(s, fallback_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeInstantView::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeInstantView");
    s.store_field("url", url_);
    s.store_field("fallbackUrl", fallback_url_);
    s.store_class_end();
  }
}

void internalLinkTypeInstantView::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeInstantView").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    fallback_url_fieldID = jni::get_field_id(env, Class, "fallbackUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeInvoice::Class;
jfieldID internalLinkTypeInvoice::invoice_name_fieldID;

internalLinkTypeInvoice::internalLinkTypeInvoice()
  : invoice_name_()
{}

internalLinkTypeInvoice::internalLinkTypeInvoice(string const &invoice_name_)
  : invoice_name_(invoice_name_)
{}

const std::int32_t internalLinkTypeInvoice::ID;

object_ptr<InternalLinkType> internalLinkTypeInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeInvoice> res = make_object<internalLinkTypeInvoice>();
  res->invoice_name_ = jni::fetch_string(env, p, res->invoice_name_fieldID);
  return std::move(res);
}

void internalLinkTypeInvoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, invoice_name_); if (nextString) { env->SetObjectField(s, invoice_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeInvoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeInvoice");
    s.store_field("invoiceName", invoice_name_);
    s.store_class_end();
  }
}

void internalLinkTypeInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeInvoice").c_str());
    invoice_name_fieldID = jni::get_field_id(env, Class, "invoiceName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeLanguagePack::Class;
jfieldID internalLinkTypeLanguagePack::language_pack_id_fieldID;

internalLinkTypeLanguagePack::internalLinkTypeLanguagePack()
  : language_pack_id_()
{}

internalLinkTypeLanguagePack::internalLinkTypeLanguagePack(string const &language_pack_id_)
  : language_pack_id_(language_pack_id_)
{}

const std::int32_t internalLinkTypeLanguagePack::ID;

object_ptr<InternalLinkType> internalLinkTypeLanguagePack::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeLanguagePack> res = make_object<internalLinkTypeLanguagePack>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  return std::move(res);
}

void internalLinkTypeLanguagePack::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, language_pack_id_); if (nextString) { env->SetObjectField(s, language_pack_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeLanguagePack::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeLanguagePack");
    s.store_field("languagePackId", language_pack_id_);
    s.store_class_end();
  }
}

void internalLinkTypeLanguagePack::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeLanguagePack").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeLanguageSettings::Class;

internalLinkTypeLanguageSettings::internalLinkTypeLanguageSettings() {
}

const std::int32_t internalLinkTypeLanguageSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeLanguageSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeLanguageSettings>();
}

void internalLinkTypeLanguageSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeLanguageSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeLanguageSettings");
    s.store_class_end();
  }
}

void internalLinkTypeLanguageSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeLanguageSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeMainWebApp::Class;
jfieldID internalLinkTypeMainWebApp::bot_username_fieldID;
jfieldID internalLinkTypeMainWebApp::start_parameter_fieldID;
jfieldID internalLinkTypeMainWebApp::mode_fieldID;

internalLinkTypeMainWebApp::internalLinkTypeMainWebApp()
  : bot_username_()
  , start_parameter_()
  , mode_()
{}

internalLinkTypeMainWebApp::internalLinkTypeMainWebApp(string const &bot_username_, string const &start_parameter_, object_ptr<WebAppOpenMode> &&mode_)
  : bot_username_(bot_username_)
  , start_parameter_(start_parameter_)
  , mode_(std::move(mode_))
{}

const std::int32_t internalLinkTypeMainWebApp::ID;

object_ptr<InternalLinkType> internalLinkTypeMainWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeMainWebApp> res = make_object<internalLinkTypeMainWebApp>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->mode_ = jni::fetch_tl_object<WebAppOpenMode>(env, jni::fetch_object(env, p, res->mode_fieldID));
  return std::move(res);
}

void internalLinkTypeMainWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (mode_ != nullptr) { jobject next; mode_->store(env, next); if (next) { env->SetObjectField(s, mode_fieldID, next); env->DeleteLocalRef(next); } }
}

void internalLinkTypeMainWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeMainWebApp");
    s.store_field("botUsername", bot_username_);
    s.store_field("startParameter", start_parameter_);
    s.store_object_field("mode", static_cast<const BaseObject *>(mode_.get()));
    s.store_class_end();
  }
}

void internalLinkTypeMainWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeMainWebApp").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    mode_fieldID = jni::get_field_id(env, Class, "mode", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenMode;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeMessage::Class;
jfieldID internalLinkTypeMessage::url_fieldID;

internalLinkTypeMessage::internalLinkTypeMessage()
  : url_()
{}

internalLinkTypeMessage::internalLinkTypeMessage(string const &url_)
  : url_(url_)
{}

const std::int32_t internalLinkTypeMessage::ID;

object_ptr<InternalLinkType> internalLinkTypeMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeMessage> res = make_object<internalLinkTypeMessage>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void internalLinkTypeMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeMessage");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void internalLinkTypeMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeMessage").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeMessageDraft::Class;
jfieldID internalLinkTypeMessageDraft::text_fieldID;
jfieldID internalLinkTypeMessageDraft::contains_link_fieldID;

internalLinkTypeMessageDraft::internalLinkTypeMessageDraft()
  : text_()
  , contains_link_()
{}

internalLinkTypeMessageDraft::internalLinkTypeMessageDraft(object_ptr<formattedText> &&text_, bool contains_link_)
  : text_(std::move(text_))
  , contains_link_(contains_link_)
{}

const std::int32_t internalLinkTypeMessageDraft::ID;

object_ptr<InternalLinkType> internalLinkTypeMessageDraft::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeMessageDraft> res = make_object<internalLinkTypeMessageDraft>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->contains_link_ = (env->GetBooleanField(p, res->contains_link_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypeMessageDraft::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, contains_link_fieldID, contains_link_);
}

void internalLinkTypeMessageDraft::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeMessageDraft");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("containsLink", contains_link_);
    s.store_class_end();
  }
}

void internalLinkTypeMessageDraft::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeMessageDraft").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    contains_link_fieldID = jni::get_field_id(env, Class, "containsLink", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeMyStars::Class;

internalLinkTypeMyStars::internalLinkTypeMyStars() {
}

const std::int32_t internalLinkTypeMyStars::ID;

object_ptr<InternalLinkType> internalLinkTypeMyStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeMyStars>();
}

void internalLinkTypeMyStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeMyStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeMyStars");
    s.store_class_end();
  }
}

void internalLinkTypeMyStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeMyStars").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePassportDataRequest::Class;
jfieldID internalLinkTypePassportDataRequest::bot_user_id_fieldID;
jfieldID internalLinkTypePassportDataRequest::scope_fieldID;
jfieldID internalLinkTypePassportDataRequest::public_key_fieldID;
jfieldID internalLinkTypePassportDataRequest::nonce_fieldID;
jfieldID internalLinkTypePassportDataRequest::callback_url_fieldID;

internalLinkTypePassportDataRequest::internalLinkTypePassportDataRequest()
  : bot_user_id_()
  , scope_()
  , public_key_()
  , nonce_()
  , callback_url_()
{}

internalLinkTypePassportDataRequest::internalLinkTypePassportDataRequest(int53 bot_user_id_, string const &scope_, string const &public_key_, string const &nonce_, string const &callback_url_)
  : bot_user_id_(bot_user_id_)
  , scope_(scope_)
  , public_key_(public_key_)
  , nonce_(nonce_)
  , callback_url_(callback_url_)
{}

const std::int32_t internalLinkTypePassportDataRequest::ID;

object_ptr<InternalLinkType> internalLinkTypePassportDataRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePassportDataRequest> res = make_object<internalLinkTypePassportDataRequest>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->scope_ = jni::fetch_string(env, p, res->scope_fieldID);
  res->public_key_ = jni::fetch_string(env, p, res->public_key_fieldID);
  res->nonce_ = jni::fetch_string(env, p, res->nonce_fieldID);
  res->callback_url_ = jni::fetch_string(env, p, res->callback_url_fieldID);
  return std::move(res);
}

void internalLinkTypePassportDataRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  { jstring nextString = jni::to_jstring(env, scope_); if (nextString) { env->SetObjectField(s, scope_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, public_key_); if (nextString) { env->SetObjectField(s, public_key_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, nonce_); if (nextString) { env->SetObjectField(s, nonce_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, callback_url_); if (nextString) { env->SetObjectField(s, callback_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypePassportDataRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePassportDataRequest");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("scope", scope_);
    s.store_field("publicKey", public_key_);
    s.store_field("nonce", nonce_);
    s.store_field("callbackUrl", callback_url_);
    s.store_class_end();
  }
}

void internalLinkTypePassportDataRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePassportDataRequest").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    scope_fieldID = jni::get_field_id(env, Class, "scope", "Ljava/lang/String;");
    public_key_fieldID = jni::get_field_id(env, Class, "publicKey", "Ljava/lang/String;");
    nonce_fieldID = jni::get_field_id(env, Class, "nonce", "Ljava/lang/String;");
    callback_url_fieldID = jni::get_field_id(env, Class, "callbackUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePhoneNumberConfirmation::Class;
jfieldID internalLinkTypePhoneNumberConfirmation::hash_fieldID;
jfieldID internalLinkTypePhoneNumberConfirmation::phone_number_fieldID;

internalLinkTypePhoneNumberConfirmation::internalLinkTypePhoneNumberConfirmation()
  : hash_()
  , phone_number_()
{}

internalLinkTypePhoneNumberConfirmation::internalLinkTypePhoneNumberConfirmation(string const &hash_, string const &phone_number_)
  : hash_(hash_)
  , phone_number_(phone_number_)
{}

const std::int32_t internalLinkTypePhoneNumberConfirmation::ID;

object_ptr<InternalLinkType> internalLinkTypePhoneNumberConfirmation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePhoneNumberConfirmation> res = make_object<internalLinkTypePhoneNumberConfirmation>();
  res->hash_ = jni::fetch_string(env, p, res->hash_fieldID);
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  return std::move(res);
}

void internalLinkTypePhoneNumberConfirmation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, hash_); if (nextString) { env->SetObjectField(s, hash_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypePhoneNumberConfirmation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePhoneNumberConfirmation");
    s.store_field("hash", hash_);
    s.store_field("phoneNumber", phone_number_);
    s.store_class_end();
  }
}

void internalLinkTypePhoneNumberConfirmation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePhoneNumberConfirmation").c_str());
    hash_fieldID = jni::get_field_id(env, Class, "hash", "Ljava/lang/String;");
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePremiumFeatures::Class;
jfieldID internalLinkTypePremiumFeatures::referrer_fieldID;

internalLinkTypePremiumFeatures::internalLinkTypePremiumFeatures()
  : referrer_()
{}

internalLinkTypePremiumFeatures::internalLinkTypePremiumFeatures(string const &referrer_)
  : referrer_(referrer_)
{}

const std::int32_t internalLinkTypePremiumFeatures::ID;

object_ptr<InternalLinkType> internalLinkTypePremiumFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePremiumFeatures> res = make_object<internalLinkTypePremiumFeatures>();
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return std::move(res);
}

void internalLinkTypePremiumFeatures::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, referrer_); if (nextString) { env->SetObjectField(s, referrer_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypePremiumFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePremiumFeatures");
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

void internalLinkTypePremiumFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePremiumFeatures").c_str());
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePremiumGift::Class;
jfieldID internalLinkTypePremiumGift::referrer_fieldID;

internalLinkTypePremiumGift::internalLinkTypePremiumGift()
  : referrer_()
{}

internalLinkTypePremiumGift::internalLinkTypePremiumGift(string const &referrer_)
  : referrer_(referrer_)
{}

const std::int32_t internalLinkTypePremiumGift::ID;

object_ptr<InternalLinkType> internalLinkTypePremiumGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePremiumGift> res = make_object<internalLinkTypePremiumGift>();
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return std::move(res);
}

void internalLinkTypePremiumGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, referrer_); if (nextString) { env->SetObjectField(s, referrer_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypePremiumGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePremiumGift");
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

void internalLinkTypePremiumGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePremiumGift").c_str());
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePremiumGiftCode::Class;
jfieldID internalLinkTypePremiumGiftCode::code_fieldID;

internalLinkTypePremiumGiftCode::internalLinkTypePremiumGiftCode()
  : code_()
{}

internalLinkTypePremiumGiftCode::internalLinkTypePremiumGiftCode(string const &code_)
  : code_(code_)
{}

const std::int32_t internalLinkTypePremiumGiftCode::ID;

object_ptr<InternalLinkType> internalLinkTypePremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePremiumGiftCode> res = make_object<internalLinkTypePremiumGiftCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return std::move(res);
}

void internalLinkTypePremiumGiftCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, code_); if (nextString) { env->SetObjectField(s, code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypePremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePremiumGiftCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

void internalLinkTypePremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePremiumGiftCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePrivacyAndSecuritySettings::Class;

internalLinkTypePrivacyAndSecuritySettings::internalLinkTypePrivacyAndSecuritySettings() {
}

const std::int32_t internalLinkTypePrivacyAndSecuritySettings::ID;

object_ptr<InternalLinkType> internalLinkTypePrivacyAndSecuritySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypePrivacyAndSecuritySettings>();
}

void internalLinkTypePrivacyAndSecuritySettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypePrivacyAndSecuritySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePrivacyAndSecuritySettings");
    s.store_class_end();
  }
}

void internalLinkTypePrivacyAndSecuritySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePrivacyAndSecuritySettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeProxy::Class;
jfieldID internalLinkTypeProxy::server_fieldID;
jfieldID internalLinkTypeProxy::port_fieldID;
jfieldID internalLinkTypeProxy::type_fieldID;

internalLinkTypeProxy::internalLinkTypeProxy()
  : server_()
  , port_()
  , type_()
{}

internalLinkTypeProxy::internalLinkTypeProxy(string const &server_, int32 port_, object_ptr<ProxyType> &&type_)
  : server_(server_)
  , port_(port_)
  , type_(std::move(type_))
{}

const std::int32_t internalLinkTypeProxy::ID;

object_ptr<InternalLinkType> internalLinkTypeProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeProxy> res = make_object<internalLinkTypeProxy>();
  res->server_ = jni::fetch_string(env, p, res->server_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->type_ = jni::fetch_tl_object<ProxyType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return std::move(res);
}

void internalLinkTypeProxy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, server_); if (nextString) { env->SetObjectField(s, server_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, port_fieldID, port_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void internalLinkTypeProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeProxy");
    s.store_field("server", server_);
    s.store_field("port", port_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void internalLinkTypeProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeProxy").c_str());
    server_fieldID = jni::get_field_id(env, Class, "server", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProxyType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypePublicChat::Class;
jfieldID internalLinkTypePublicChat::chat_username_fieldID;
jfieldID internalLinkTypePublicChat::draft_text_fieldID;
jfieldID internalLinkTypePublicChat::open_profile_fieldID;

internalLinkTypePublicChat::internalLinkTypePublicChat()
  : chat_username_()
  , draft_text_()
  , open_profile_()
{}

internalLinkTypePublicChat::internalLinkTypePublicChat(string const &chat_username_, string const &draft_text_, bool open_profile_)
  : chat_username_(chat_username_)
  , draft_text_(draft_text_)
  , open_profile_(open_profile_)
{}

const std::int32_t internalLinkTypePublicChat::ID;

object_ptr<InternalLinkType> internalLinkTypePublicChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypePublicChat> res = make_object<internalLinkTypePublicChat>();
  res->chat_username_ = jni::fetch_string(env, p, res->chat_username_fieldID);
  res->draft_text_ = jni::fetch_string(env, p, res->draft_text_fieldID);
  res->open_profile_ = (env->GetBooleanField(p, res->open_profile_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypePublicChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, chat_username_); if (nextString) { env->SetObjectField(s, chat_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, draft_text_); if (nextString) { env->SetObjectField(s, draft_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, open_profile_fieldID, open_profile_);
}

void internalLinkTypePublicChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypePublicChat");
    s.store_field("chatUsername", chat_username_);
    s.store_field("draftText", draft_text_);
    s.store_field("openProfile", open_profile_);
    s.store_class_end();
  }
}

void internalLinkTypePublicChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypePublicChat").c_str());
    chat_username_fieldID = jni::get_field_id(env, Class, "chatUsername", "Ljava/lang/String;");
    draft_text_fieldID = jni::get_field_id(env, Class, "draftText", "Ljava/lang/String;");
    open_profile_fieldID = jni::get_field_id(env, Class, "openProfile", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeQrCodeAuthentication::Class;

internalLinkTypeQrCodeAuthentication::internalLinkTypeQrCodeAuthentication() {
}

const std::int32_t internalLinkTypeQrCodeAuthentication::ID;

object_ptr<InternalLinkType> internalLinkTypeQrCodeAuthentication::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeQrCodeAuthentication>();
}

void internalLinkTypeQrCodeAuthentication::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeQrCodeAuthentication::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeQrCodeAuthentication");
    s.store_class_end();
  }
}

void internalLinkTypeQrCodeAuthentication::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeQrCodeAuthentication").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeRestorePurchases::Class;

internalLinkTypeRestorePurchases::internalLinkTypeRestorePurchases() {
}

const std::int32_t internalLinkTypeRestorePurchases::ID;

object_ptr<InternalLinkType> internalLinkTypeRestorePurchases::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeRestorePurchases>();
}

void internalLinkTypeRestorePurchases::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeRestorePurchases::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeRestorePurchases");
    s.store_class_end();
  }
}

void internalLinkTypeRestorePurchases::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeRestorePurchases").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeSettings::Class;

internalLinkTypeSettings::internalLinkTypeSettings() {
}

const std::int32_t internalLinkTypeSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeSettings>();
}

void internalLinkTypeSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeSettings");
    s.store_class_end();
  }
}

void internalLinkTypeSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeStickerSet::Class;
jfieldID internalLinkTypeStickerSet::sticker_set_name_fieldID;
jfieldID internalLinkTypeStickerSet::expect_custom_emoji_fieldID;

internalLinkTypeStickerSet::internalLinkTypeStickerSet()
  : sticker_set_name_()
  , expect_custom_emoji_()
{}

internalLinkTypeStickerSet::internalLinkTypeStickerSet(string const &sticker_set_name_, bool expect_custom_emoji_)
  : sticker_set_name_(sticker_set_name_)
  , expect_custom_emoji_(expect_custom_emoji_)
{}

const std::int32_t internalLinkTypeStickerSet::ID;

object_ptr<InternalLinkType> internalLinkTypeStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeStickerSet> res = make_object<internalLinkTypeStickerSet>();
  res->sticker_set_name_ = jni::fetch_string(env, p, res->sticker_set_name_fieldID);
  res->expect_custom_emoji_ = (env->GetBooleanField(p, res->expect_custom_emoji_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypeStickerSet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, sticker_set_name_); if (nextString) { env->SetObjectField(s, sticker_set_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, expect_custom_emoji_fieldID, expect_custom_emoji_);
}

void internalLinkTypeStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeStickerSet");
    s.store_field("stickerSetName", sticker_set_name_);
    s.store_field("expectCustomEmoji", expect_custom_emoji_);
    s.store_class_end();
  }
}

void internalLinkTypeStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeStickerSet").c_str());
    sticker_set_name_fieldID = jni::get_field_id(env, Class, "stickerSetName", "Ljava/lang/String;");
    expect_custom_emoji_fieldID = jni::get_field_id(env, Class, "expectCustomEmoji", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeStory::Class;
jfieldID internalLinkTypeStory::story_poster_username_fieldID;
jfieldID internalLinkTypeStory::story_id_fieldID;

internalLinkTypeStory::internalLinkTypeStory()
  : story_poster_username_()
  , story_id_()
{}

internalLinkTypeStory::internalLinkTypeStory(string const &story_poster_username_, int32 story_id_)
  : story_poster_username_(story_poster_username_)
  , story_id_(story_id_)
{}

const std::int32_t internalLinkTypeStory::ID;

object_ptr<InternalLinkType> internalLinkTypeStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeStory> res = make_object<internalLinkTypeStory>();
  res->story_poster_username_ = jni::fetch_string(env, p, res->story_poster_username_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void internalLinkTypeStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, story_poster_username_); if (nextString) { env->SetObjectField(s, story_poster_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void internalLinkTypeStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeStory");
    s.store_field("storyPosterUsername", story_poster_username_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void internalLinkTypeStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeStory").c_str());
    story_poster_username_fieldID = jni::get_field_id(env, Class, "storyPosterUsername", "Ljava/lang/String;");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeTheme::Class;
jfieldID internalLinkTypeTheme::theme_name_fieldID;

internalLinkTypeTheme::internalLinkTypeTheme()
  : theme_name_()
{}

internalLinkTypeTheme::internalLinkTypeTheme(string const &theme_name_)
  : theme_name_(theme_name_)
{}

const std::int32_t internalLinkTypeTheme::ID;

object_ptr<InternalLinkType> internalLinkTypeTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeTheme> res = make_object<internalLinkTypeTheme>();
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return std::move(res);
}

void internalLinkTypeTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeTheme");
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

void internalLinkTypeTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeTheme").c_str());
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeThemeSettings::Class;

internalLinkTypeThemeSettings::internalLinkTypeThemeSettings() {
}

const std::int32_t internalLinkTypeThemeSettings::ID;

object_ptr<InternalLinkType> internalLinkTypeThemeSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeThemeSettings>();
}

void internalLinkTypeThemeSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeThemeSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeThemeSettings");
    s.store_class_end();
  }
}

void internalLinkTypeThemeSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeThemeSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeUnknownDeepLink::Class;
jfieldID internalLinkTypeUnknownDeepLink::link_fieldID;

internalLinkTypeUnknownDeepLink::internalLinkTypeUnknownDeepLink()
  : link_()
{}

internalLinkTypeUnknownDeepLink::internalLinkTypeUnknownDeepLink(string const &link_)
  : link_(link_)
{}

const std::int32_t internalLinkTypeUnknownDeepLink::ID;

object_ptr<InternalLinkType> internalLinkTypeUnknownDeepLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeUnknownDeepLink> res = make_object<internalLinkTypeUnknownDeepLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return std::move(res);
}

void internalLinkTypeUnknownDeepLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_); if (nextString) { env->SetObjectField(s, link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeUnknownDeepLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeUnknownDeepLink");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

void internalLinkTypeUnknownDeepLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeUnknownDeepLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeUnsupportedProxy::Class;

internalLinkTypeUnsupportedProxy::internalLinkTypeUnsupportedProxy() {
}

const std::int32_t internalLinkTypeUnsupportedProxy::ID;

object_ptr<InternalLinkType> internalLinkTypeUnsupportedProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<internalLinkTypeUnsupportedProxy>();
}

void internalLinkTypeUnsupportedProxy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void internalLinkTypeUnsupportedProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeUnsupportedProxy");
    s.store_class_end();
  }
}

void internalLinkTypeUnsupportedProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeUnsupportedProxy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeUpgradedGift::Class;
jfieldID internalLinkTypeUpgradedGift::name_fieldID;

internalLinkTypeUpgradedGift::internalLinkTypeUpgradedGift()
  : name_()
{}

internalLinkTypeUpgradedGift::internalLinkTypeUpgradedGift(string const &name_)
  : name_(name_)
{}

const std::int32_t internalLinkTypeUpgradedGift::ID;

object_ptr<InternalLinkType> internalLinkTypeUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeUpgradedGift> res = make_object<internalLinkTypeUpgradedGift>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return std::move(res);
}

void internalLinkTypeUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeUpgradedGift");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void internalLinkTypeUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeUpgradedGift").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeUserPhoneNumber::Class;
jfieldID internalLinkTypeUserPhoneNumber::phone_number_fieldID;
jfieldID internalLinkTypeUserPhoneNumber::draft_text_fieldID;
jfieldID internalLinkTypeUserPhoneNumber::open_profile_fieldID;

internalLinkTypeUserPhoneNumber::internalLinkTypeUserPhoneNumber()
  : phone_number_()
  , draft_text_()
  , open_profile_()
{}

internalLinkTypeUserPhoneNumber::internalLinkTypeUserPhoneNumber(string const &phone_number_, string const &draft_text_, bool open_profile_)
  : phone_number_(phone_number_)
  , draft_text_(draft_text_)
  , open_profile_(open_profile_)
{}

const std::int32_t internalLinkTypeUserPhoneNumber::ID;

object_ptr<InternalLinkType> internalLinkTypeUserPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeUserPhoneNumber> res = make_object<internalLinkTypeUserPhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->draft_text_ = jni::fetch_string(env, p, res->draft_text_fieldID);
  res->open_profile_ = (env->GetBooleanField(p, res->open_profile_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypeUserPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, draft_text_); if (nextString) { env->SetObjectField(s, draft_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, open_profile_fieldID, open_profile_);
}

void internalLinkTypeUserPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeUserPhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_field("draftText", draft_text_);
    s.store_field("openProfile", open_profile_);
    s.store_class_end();
  }
}

void internalLinkTypeUserPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeUserPhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    draft_text_fieldID = jni::get_field_id(env, Class, "draftText", "Ljava/lang/String;");
    open_profile_fieldID = jni::get_field_id(env, Class, "openProfile", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeUserToken::Class;
jfieldID internalLinkTypeUserToken::token_fieldID;

internalLinkTypeUserToken::internalLinkTypeUserToken()
  : token_()
{}

internalLinkTypeUserToken::internalLinkTypeUserToken(string const &token_)
  : token_(token_)
{}

const std::int32_t internalLinkTypeUserToken::ID;

object_ptr<InternalLinkType> internalLinkTypeUserToken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeUserToken> res = make_object<internalLinkTypeUserToken>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void internalLinkTypeUserToken::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, token_); if (nextString) { env->SetObjectField(s, token_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void internalLinkTypeUserToken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeUserToken");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void internalLinkTypeUserToken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeUserToken").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeVideoChat::Class;
jfieldID internalLinkTypeVideoChat::chat_username_fieldID;
jfieldID internalLinkTypeVideoChat::invite_hash_fieldID;
jfieldID internalLinkTypeVideoChat::is_live_stream_fieldID;

internalLinkTypeVideoChat::internalLinkTypeVideoChat()
  : chat_username_()
  , invite_hash_()
  , is_live_stream_()
{}

internalLinkTypeVideoChat::internalLinkTypeVideoChat(string const &chat_username_, string const &invite_hash_, bool is_live_stream_)
  : chat_username_(chat_username_)
  , invite_hash_(invite_hash_)
  , is_live_stream_(is_live_stream_)
{}

const std::int32_t internalLinkTypeVideoChat::ID;

object_ptr<InternalLinkType> internalLinkTypeVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeVideoChat> res = make_object<internalLinkTypeVideoChat>();
  res->chat_username_ = jni::fetch_string(env, p, res->chat_username_fieldID);
  res->invite_hash_ = jni::fetch_string(env, p, res->invite_hash_fieldID);
  res->is_live_stream_ = (env->GetBooleanField(p, res->is_live_stream_fieldID) != 0);
  return std::move(res);
}

void internalLinkTypeVideoChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, chat_username_); if (nextString) { env->SetObjectField(s, chat_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, invite_hash_); if (nextString) { env->SetObjectField(s, invite_hash_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_live_stream_fieldID, is_live_stream_);
}

void internalLinkTypeVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeVideoChat");
    s.store_field("chatUsername", chat_username_);
    s.store_field("inviteHash", invite_hash_);
    s.store_field("isLiveStream", is_live_stream_);
    s.store_class_end();
  }
}

void internalLinkTypeVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeVideoChat").c_str());
    chat_username_fieldID = jni::get_field_id(env, Class, "chatUsername", "Ljava/lang/String;");
    invite_hash_fieldID = jni::get_field_id(env, Class, "inviteHash", "Ljava/lang/String;");
    is_live_stream_fieldID = jni::get_field_id(env, Class, "isLiveStream", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass internalLinkTypeWebApp::Class;
jfieldID internalLinkTypeWebApp::bot_username_fieldID;
jfieldID internalLinkTypeWebApp::web_app_short_name_fieldID;
jfieldID internalLinkTypeWebApp::start_parameter_fieldID;
jfieldID internalLinkTypeWebApp::mode_fieldID;

internalLinkTypeWebApp::internalLinkTypeWebApp()
  : bot_username_()
  , web_app_short_name_()
  , start_parameter_()
  , mode_()
{}

internalLinkTypeWebApp::internalLinkTypeWebApp(string const &bot_username_, string const &web_app_short_name_, string const &start_parameter_, object_ptr<WebAppOpenMode> &&mode_)
  : bot_username_(bot_username_)
  , web_app_short_name_(web_app_short_name_)
  , start_parameter_(start_parameter_)
  , mode_(std::move(mode_))
{}

const std::int32_t internalLinkTypeWebApp::ID;

object_ptr<InternalLinkType> internalLinkTypeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<internalLinkTypeWebApp> res = make_object<internalLinkTypeWebApp>();
  res->bot_username_ = jni::fetch_string(env, p, res->bot_username_fieldID);
  res->web_app_short_name_ = jni::fetch_string(env, p, res->web_app_short_name_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->mode_ = jni::fetch_tl_object<WebAppOpenMode>(env, jni::fetch_object(env, p, res->mode_fieldID));
  return std::move(res);
}

void internalLinkTypeWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, bot_username_); if (nextString) { env->SetObjectField(s, bot_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, web_app_short_name_); if (nextString) { env->SetObjectField(s, web_app_short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (mode_ != nullptr) { jobject next; mode_->store(env, next); if (next) { env->SetObjectField(s, mode_fieldID, next); env->DeleteLocalRef(next); } }
}

void internalLinkTypeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InternalLinkTypeWebApp");
    s.store_field("botUsername", bot_username_);
    s.store_field("webAppShortName", web_app_short_name_);
    s.store_field("startParameter", start_parameter_);
    s.store_object_field("mode", static_cast<const BaseObject *>(mode_.get()));
    s.store_class_end();
  }
}

void internalLinkTypeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InternalLinkTypeWebApp").c_str());
    bot_username_fieldID = jni::get_field_id(env, Class, "botUsername", "Ljava/lang/String;");
    web_app_short_name_fieldID = jni::get_field_id(env, Class, "webAppShortName", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    mode_fieldID = jni::get_field_id(env, Class, "mode", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenMode;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass InviteGroupCallParticipantResult::Class;

object_ptr<InviteGroupCallParticipantResult> InviteGroupCallParticipantResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inviteGroupCallParticipantResultUserPrivacyRestricted::ID:
      return inviteGroupCallParticipantResultUserPrivacyRestricted::fetch(env, p);
    case inviteGroupCallParticipantResultUserAlreadyParticipant::ID:
      return inviteGroupCallParticipantResultUserAlreadyParticipant::fetch(env, p);
    case inviteGroupCallParticipantResultUserWasBanned::ID:
      return inviteGroupCallParticipantResultUserWasBanned::fetch(env, p);
    case inviteGroupCallParticipantResultSuccess::ID:
      return inviteGroupCallParticipantResultSuccess::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InviteGroupCallParticipantResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipantResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteGroupCallParticipantResultUserPrivacyRestricted::Class;

inviteGroupCallParticipantResultUserPrivacyRestricted::inviteGroupCallParticipantResultUserPrivacyRestricted() {
}

const std::int32_t inviteGroupCallParticipantResultUserPrivacyRestricted::ID;

object_ptr<InviteGroupCallParticipantResult> inviteGroupCallParticipantResultUserPrivacyRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteGroupCallParticipantResultUserPrivacyRestricted>();
}

void inviteGroupCallParticipantResultUserPrivacyRestricted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteGroupCallParticipantResultUserPrivacyRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteGroupCallParticipantResultUserPrivacyRestricted");
    s.store_class_end();
  }
}

void inviteGroupCallParticipantResultUserPrivacyRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipantResultUserPrivacyRestricted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteGroupCallParticipantResultUserAlreadyParticipant::Class;

inviteGroupCallParticipantResultUserAlreadyParticipant::inviteGroupCallParticipantResultUserAlreadyParticipant() {
}

const std::int32_t inviteGroupCallParticipantResultUserAlreadyParticipant::ID;

object_ptr<InviteGroupCallParticipantResult> inviteGroupCallParticipantResultUserAlreadyParticipant::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteGroupCallParticipantResultUserAlreadyParticipant>();
}

void inviteGroupCallParticipantResultUserAlreadyParticipant::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteGroupCallParticipantResultUserAlreadyParticipant::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteGroupCallParticipantResultUserAlreadyParticipant");
    s.store_class_end();
  }
}

void inviteGroupCallParticipantResultUserAlreadyParticipant::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipantResultUserAlreadyParticipant").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteGroupCallParticipantResultUserWasBanned::Class;

inviteGroupCallParticipantResultUserWasBanned::inviteGroupCallParticipantResultUserWasBanned() {
}

const std::int32_t inviteGroupCallParticipantResultUserWasBanned::ID;

object_ptr<InviteGroupCallParticipantResult> inviteGroupCallParticipantResultUserWasBanned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteGroupCallParticipantResultUserWasBanned>();
}

void inviteGroupCallParticipantResultUserWasBanned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteGroupCallParticipantResultUserWasBanned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteGroupCallParticipantResultUserWasBanned");
    s.store_class_end();
  }
}

void inviteGroupCallParticipantResultUserWasBanned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipantResultUserWasBanned").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteGroupCallParticipantResultSuccess::Class;
jfieldID inviteGroupCallParticipantResultSuccess::chat_id_fieldID;
jfieldID inviteGroupCallParticipantResultSuccess::message_id_fieldID;

inviteGroupCallParticipantResultSuccess::inviteGroupCallParticipantResultSuccess()
  : chat_id_()
  , message_id_()
{}

inviteGroupCallParticipantResultSuccess::inviteGroupCallParticipantResultSuccess(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t inviteGroupCallParticipantResultSuccess::ID;

object_ptr<InviteGroupCallParticipantResult> inviteGroupCallParticipantResultSuccess::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inviteGroupCallParticipantResultSuccess> res = make_object<inviteGroupCallParticipantResultSuccess>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void inviteGroupCallParticipantResultSuccess::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void inviteGroupCallParticipantResultSuccess::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteGroupCallParticipantResultSuccess");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void inviteGroupCallParticipantResultSuccess::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipantResultSuccess").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass InviteLinkChatType::Class;

object_ptr<InviteLinkChatType> InviteLinkChatType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case inviteLinkChatTypeBasicGroup::ID:
      return inviteLinkChatTypeBasicGroup::fetch(env, p);
    case inviteLinkChatTypeSupergroup::ID:
      return inviteLinkChatTypeSupergroup::fetch(env, p);
    case inviteLinkChatTypeChannel::ID:
      return inviteLinkChatTypeChannel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void InviteLinkChatType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteLinkChatType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteLinkChatTypeBasicGroup::Class;

inviteLinkChatTypeBasicGroup::inviteLinkChatTypeBasicGroup() {
}

const std::int32_t inviteLinkChatTypeBasicGroup::ID;

object_ptr<InviteLinkChatType> inviteLinkChatTypeBasicGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteLinkChatTypeBasicGroup>();
}

void inviteLinkChatTypeBasicGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteLinkChatTypeBasicGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteLinkChatTypeBasicGroup");
    s.store_class_end();
  }
}

void inviteLinkChatTypeBasicGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteLinkChatTypeBasicGroup").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteLinkChatTypeSupergroup::Class;

inviteLinkChatTypeSupergroup::inviteLinkChatTypeSupergroup() {
}

const std::int32_t inviteLinkChatTypeSupergroup::ID;

object_ptr<InviteLinkChatType> inviteLinkChatTypeSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteLinkChatTypeSupergroup>();
}

void inviteLinkChatTypeSupergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteLinkChatTypeSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteLinkChatTypeSupergroup");
    s.store_class_end();
  }
}

void inviteLinkChatTypeSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteLinkChatTypeSupergroup").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteLinkChatTypeChannel::Class;

inviteLinkChatTypeChannel::inviteLinkChatTypeChannel() {
}

const std::int32_t inviteLinkChatTypeChannel::ID;

object_ptr<InviteLinkChatType> inviteLinkChatTypeChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<inviteLinkChatTypeChannel>();
}

void inviteLinkChatTypeChannel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void inviteLinkChatTypeChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteLinkChatTypeChannel");
    s.store_class_end();
  }
}

void inviteLinkChatTypeChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteLinkChatTypeChannel").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass invoice::Class;
jfieldID invoice::currency_fieldID;
jfieldID invoice::price_parts_fieldID;
jfieldID invoice::subscription_period_fieldID;
jfieldID invoice::max_tip_amount_fieldID;
jfieldID invoice::suggested_tip_amounts_fieldID;
jfieldID invoice::recurring_payment_terms_of_service_url_fieldID;
jfieldID invoice::terms_of_service_url_fieldID;
jfieldID invoice::is_test_fieldID;
jfieldID invoice::need_name_fieldID;
jfieldID invoice::need_phone_number_fieldID;
jfieldID invoice::need_email_address_fieldID;
jfieldID invoice::need_shipping_address_fieldID;
jfieldID invoice::send_phone_number_to_provider_fieldID;
jfieldID invoice::send_email_address_to_provider_fieldID;
jfieldID invoice::is_flexible_fieldID;

invoice::invoice()
  : currency_()
  , price_parts_()
  , subscription_period_()
  , max_tip_amount_()
  , suggested_tip_amounts_()
  , recurring_payment_terms_of_service_url_()
  , terms_of_service_url_()
  , is_test_()
  , need_name_()
  , need_phone_number_()
  , need_email_address_()
  , need_shipping_address_()
  , send_phone_number_to_provider_()
  , send_email_address_to_provider_()
  , is_flexible_()
{}

invoice::invoice(string const &currency_, array<object_ptr<labeledPricePart>> &&price_parts_, int32 subscription_period_, int53 max_tip_amount_, array<int53> &&suggested_tip_amounts_, string const &recurring_payment_terms_of_service_url_, string const &terms_of_service_url_, bool is_test_, bool need_name_, bool need_phone_number_, bool need_email_address_, bool need_shipping_address_, bool send_phone_number_to_provider_, bool send_email_address_to_provider_, bool is_flexible_)
  : currency_(currency_)
  , price_parts_(std::move(price_parts_))
  , subscription_period_(subscription_period_)
  , max_tip_amount_(max_tip_amount_)
  , suggested_tip_amounts_(std::move(suggested_tip_amounts_))
  , recurring_payment_terms_of_service_url_(recurring_payment_terms_of_service_url_)
  , terms_of_service_url_(terms_of_service_url_)
  , is_test_(is_test_)
  , need_name_(need_name_)
  , need_phone_number_(need_phone_number_)
  , need_email_address_(need_email_address_)
  , need_shipping_address_(need_shipping_address_)
  , send_phone_number_to_provider_(send_phone_number_to_provider_)
  , send_email_address_to_provider_(send_email_address_to_provider_)
  , is_flexible_(is_flexible_)
{}

const std::int32_t invoice::ID;

object_ptr<invoice> invoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<invoice> res = make_object<invoice>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->price_parts_ = jni::FetchVector<labeledPricePart>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->price_parts_fieldID));
  res->subscription_period_ = env->GetIntField(p, res->subscription_period_fieldID);
  res->max_tip_amount_ = env->GetLongField(p, res->max_tip_amount_fieldID);
  res->suggested_tip_amounts_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->suggested_tip_amounts_fieldID));
  res->recurring_payment_terms_of_service_url_ = jni::fetch_string(env, p, res->recurring_payment_terms_of_service_url_fieldID);
  res->terms_of_service_url_ = jni::fetch_string(env, p, res->terms_of_service_url_fieldID);
  res->is_test_ = (env->GetBooleanField(p, res->is_test_fieldID) != 0);
  res->need_name_ = (env->GetBooleanField(p, res->need_name_fieldID) != 0);
  res->need_phone_number_ = (env->GetBooleanField(p, res->need_phone_number_fieldID) != 0);
  res->need_email_address_ = (env->GetBooleanField(p, res->need_email_address_fieldID) != 0);
  res->need_shipping_address_ = (env->GetBooleanField(p, res->need_shipping_address_fieldID) != 0);
  res->send_phone_number_to_provider_ = (env->GetBooleanField(p, res->send_phone_number_to_provider_fieldID) != 0);
  res->send_email_address_to_provider_ = (env->GetBooleanField(p, res->send_email_address_to_provider_fieldID) != 0);
  res->is_flexible_ = (env->GetBooleanField(p, res->is_flexible_fieldID) != 0);
  return res;
}

void invoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, price_parts_); if (arr_tmp_) { env->SetObjectField(s, price_parts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, subscription_period_fieldID, subscription_period_);
  env->SetLongField(s, max_tip_amount_fieldID, max_tip_amount_);
  { auto arr_tmp_ = jni::store_vector(env, suggested_tip_amounts_); if (arr_tmp_) { env->SetObjectField(s, suggested_tip_amounts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, recurring_payment_terms_of_service_url_); if (nextString) { env->SetObjectField(s, recurring_payment_terms_of_service_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, terms_of_service_url_); if (nextString) { env->SetObjectField(s, terms_of_service_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_test_fieldID, is_test_);
  env->SetBooleanField(s, need_name_fieldID, need_name_);
  env->SetBooleanField(s, need_phone_number_fieldID, need_phone_number_);
  env->SetBooleanField(s, need_email_address_fieldID, need_email_address_);
  env->SetBooleanField(s, need_shipping_address_fieldID, need_shipping_address_);
  env->SetBooleanField(s, send_phone_number_to_provider_fieldID, send_phone_number_to_provider_);
  env->SetBooleanField(s, send_email_address_to_provider_fieldID, send_email_address_to_provider_);
  env->SetBooleanField(s, is_flexible_fieldID, is_flexible_);
}

void invoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Invoice");
    s.store_field("currency", currency_);
    { s.store_vector_begin("priceParts", price_parts_.size()); for (const auto &_value : price_parts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("subscriptionPeriod", subscription_period_);
    s.store_field("maxTipAmount", max_tip_amount_);
    { s.store_vector_begin("suggestedTipAmounts", suggested_tip_amounts_.size()); for (const auto &_value : suggested_tip_amounts_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("recurringPaymentTermsOfServiceUrl", recurring_payment_terms_of_service_url_);
    s.store_field("termsOfServiceUrl", terms_of_service_url_);
    s.store_field("isTest", is_test_);
    s.store_field("needName", need_name_);
    s.store_field("needPhoneNumber", need_phone_number_);
    s.store_field("needEmailAddress", need_email_address_);
    s.store_field("needShippingAddress", need_shipping_address_);
    s.store_field("sendPhoneNumberToProvider", send_phone_number_to_provider_);
    s.store_field("sendEmailAddressToProvider", send_email_address_to_provider_);
    s.store_field("isFlexible", is_flexible_);
    s.store_class_end();
  }
}

void invoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Invoice").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    price_parts_fieldID = jni::get_field_id(env, Class, "priceParts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LabeledPricePart;").c_str());
    subscription_period_fieldID = jni::get_field_id(env, Class, "subscriptionPeriod", "I");
    max_tip_amount_fieldID = jni::get_field_id(env, Class, "maxTipAmount", "J");
    suggested_tip_amounts_fieldID = jni::get_field_id(env, Class, "suggestedTipAmounts", "[J");
    recurring_payment_terms_of_service_url_fieldID = jni::get_field_id(env, Class, "recurringPaymentTermsOfServiceUrl", "Ljava/lang/String;");
    terms_of_service_url_fieldID = jni::get_field_id(env, Class, "termsOfServiceUrl", "Ljava/lang/String;");
    is_test_fieldID = jni::get_field_id(env, Class, "isTest", "Z");
    need_name_fieldID = jni::get_field_id(env, Class, "needName", "Z");
    need_phone_number_fieldID = jni::get_field_id(env, Class, "needPhoneNumber", "Z");
    need_email_address_fieldID = jni::get_field_id(env, Class, "needEmailAddress", "Z");
    need_shipping_address_fieldID = jni::get_field_id(env, Class, "needShippingAddress", "Z");
    send_phone_number_to_provider_fieldID = jni::get_field_id(env, Class, "sendPhoneNumberToProvider", "Z");
    send_email_address_to_provider_fieldID = jni::get_field_id(env, Class, "sendEmailAddressToProvider", "Z");
    is_flexible_fieldID = jni::get_field_id(env, Class, "isFlexible", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass jsonObjectMember::Class;
jfieldID jsonObjectMember::key_fieldID;
jfieldID jsonObjectMember::value_fieldID;

jsonObjectMember::jsonObjectMember()
  : key_()
  , value_()
{}

jsonObjectMember::jsonObjectMember(string const &key_, object_ptr<JsonValue> &&value_)
  : key_(key_)
  , value_(std::move(value_))
{}

const std::int32_t jsonObjectMember::ID;

object_ptr<jsonObjectMember> jsonObjectMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonObjectMember> res = make_object<jsonObjectMember>();
  res->key_ = jni::fetch_string(env, p, res->key_fieldID);
  res->value_ = jni::fetch_tl_object<JsonValue>(env, jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void jsonObjectMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, key_); if (nextString) { env->SetObjectField(s, key_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (value_ != nullptr) { jobject next; value_->store(env, next); if (next) { env->SetObjectField(s, value_fieldID, next); env->DeleteLocalRef(next); } }
}

void jsonObjectMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonObjectMember");
    s.store_field("key", key_);
    s.store_object_field("value", static_cast<const BaseObject *>(value_.get()));
    s.store_class_end();
  }
}

void jsonObjectMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonObjectMember").c_str());
    key_fieldID = jni::get_field_id(env, Class, "key", "Ljava/lang/String;");
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$JsonValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass JsonValue::Class;

object_ptr<JsonValue> JsonValue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case jsonValueNull::ID:
      return jsonValueNull::fetch(env, p);
    case jsonValueBoolean::ID:
      return jsonValueBoolean::fetch(env, p);
    case jsonValueNumber::ID:
      return jsonValueNumber::fetch(env, p);
    case jsonValueString::ID:
      return jsonValueString::fetch(env, p);
    case jsonValueArray::ID:
      return jsonValueArray::fetch(env, p);
    case jsonValueObject::ID:
      return jsonValueObject::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void JsonValue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValue").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueNull::Class;

jsonValueNull::jsonValueNull() {
}

const std::int32_t jsonValueNull::ID;

object_ptr<JsonValue> jsonValueNull::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<jsonValueNull>();
}

void jsonValueNull::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void jsonValueNull::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueNull");
    s.store_class_end();
  }
}

void jsonValueNull::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueNull").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueBoolean::Class;
jfieldID jsonValueBoolean::value_fieldID;

jsonValueBoolean::jsonValueBoolean()
  : value_()
{}

jsonValueBoolean::jsonValueBoolean(bool value_)
  : value_(value_)
{}

const std::int32_t jsonValueBoolean::ID;

object_ptr<JsonValue> jsonValueBoolean::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonValueBoolean> res = make_object<jsonValueBoolean>();
  res->value_ = (env->GetBooleanField(p, res->value_fieldID) != 0);
  return std::move(res);
}

void jsonValueBoolean::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, value_fieldID, value_);
}

void jsonValueBoolean::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueBoolean");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void jsonValueBoolean::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueBoolean").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueNumber::Class;
jfieldID jsonValueNumber::value_fieldID;

jsonValueNumber::jsonValueNumber()
  : value_()
{}

jsonValueNumber::jsonValueNumber(double value_)
  : value_(value_)
{}

const std::int32_t jsonValueNumber::ID;

object_ptr<JsonValue> jsonValueNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonValueNumber> res = make_object<jsonValueNumber>();
  res->value_ = env->GetDoubleField(p, res->value_fieldID);
  return std::move(res);
}

void jsonValueNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, value_fieldID, value_);
}

void jsonValueNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueNumber");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void jsonValueNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueNumber").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "D");
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueString::Class;
jfieldID jsonValueString::value_fieldID;

jsonValueString::jsonValueString()
  : value_()
{}

jsonValueString::jsonValueString(string const &value_)
  : value_(value_)
{}

const std::int32_t jsonValueString::ID;

object_ptr<JsonValue> jsonValueString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonValueString> res = make_object<jsonValueString>();
  res->value_ = jni::fetch_string(env, p, res->value_fieldID);
  return std::move(res);
}

void jsonValueString::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, value_); if (nextString) { env->SetObjectField(s, value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void jsonValueString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueString");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void jsonValueString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueString").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueArray::Class;
jfieldID jsonValueArray::values_fieldID;

jsonValueArray::jsonValueArray()
  : values_()
{}

jsonValueArray::jsonValueArray(array<object_ptr<JsonValue>> &&values_)
  : values_(std::move(values_))
{}

const std::int32_t jsonValueArray::ID;

object_ptr<JsonValue> jsonValueArray::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonValueArray> res = make_object<jsonValueArray>();
  res->values_ = jni::FetchVector<JsonValue>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->values_fieldID));
  return std::move(res);
}

void jsonValueArray::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, values_); if (arr_tmp_) { env->SetObjectField(s, values_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void jsonValueArray::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueArray");
    { s.store_vector_begin("values", values_.size()); for (const auto &_value : values_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void jsonValueArray::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueArray").c_str());
    values_fieldID = jni::get_field_id(env, Class, "values", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$JsonValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass jsonValueObject::Class;
jfieldID jsonValueObject::members_fieldID;

jsonValueObject::jsonValueObject()
  : members_()
{}

jsonValueObject::jsonValueObject(array<object_ptr<jsonObjectMember>> &&members_)
  : members_(std::move(members_))
{}

const std::int32_t jsonValueObject::ID;

object_ptr<JsonValue> jsonValueObject::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<jsonValueObject> res = make_object<jsonValueObject>();
  res->members_ = jni::FetchVector<jsonObjectMember>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->members_fieldID));
  return std::move(res);
}

void jsonValueObject::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, members_); if (arr_tmp_) { env->SetObjectField(s, members_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void jsonValueObject::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JsonValueObject");
    { s.store_vector_begin("members", members_.size()); for (const auto &_value : members_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void jsonValueObject::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JsonValueObject").c_str());
    members_fieldID = jni::get_field_id(env, Class, "members", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$JsonObjectMember;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButton::Class;
jfieldID keyboardButton::text_fieldID;
jfieldID keyboardButton::type_fieldID;

keyboardButton::keyboardButton()
  : text_()
  , type_()
{}

keyboardButton::keyboardButton(string const &text_, object_ptr<KeyboardButtonType> &&type_)
  : text_(text_)
  , type_(std::move(type_))
{}

const std::int32_t keyboardButton::ID;

object_ptr<keyboardButton> keyboardButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<keyboardButton> res = make_object<keyboardButton>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->type_ = jni::fetch_tl_object<KeyboardButtonType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void keyboardButton::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void keyboardButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButton");
    s.store_field("text", text_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void keyboardButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButton").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$KeyboardButtonType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass KeyboardButtonType::Class;

object_ptr<KeyboardButtonType> KeyboardButtonType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case keyboardButtonTypeText::ID:
      return keyboardButtonTypeText::fetch(env, p);
    case keyboardButtonTypeRequestPhoneNumber::ID:
      return keyboardButtonTypeRequestPhoneNumber::fetch(env, p);
    case keyboardButtonTypeRequestLocation::ID:
      return keyboardButtonTypeRequestLocation::fetch(env, p);
    case keyboardButtonTypeRequestPoll::ID:
      return keyboardButtonTypeRequestPoll::fetch(env, p);
    case keyboardButtonTypeRequestUsers::ID:
      return keyboardButtonTypeRequestUsers::fetch(env, p);
    case keyboardButtonTypeRequestChat::ID:
      return keyboardButtonTypeRequestChat::fetch(env, p);
    case keyboardButtonTypeWebApp::ID:
      return keyboardButtonTypeWebApp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void KeyboardButtonType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeText::Class;

keyboardButtonTypeText::keyboardButtonTypeText() {
}

const std::int32_t keyboardButtonTypeText::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<keyboardButtonTypeText>();
}

void keyboardButtonTypeText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void keyboardButtonTypeText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeText");
    s.store_class_end();
  }
}

void keyboardButtonTypeText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeText").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeRequestPhoneNumber::Class;

keyboardButtonTypeRequestPhoneNumber::keyboardButtonTypeRequestPhoneNumber() {
}

const std::int32_t keyboardButtonTypeRequestPhoneNumber::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeRequestPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<keyboardButtonTypeRequestPhoneNumber>();
}

void keyboardButtonTypeRequestPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void keyboardButtonTypeRequestPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeRequestPhoneNumber");
    s.store_class_end();
  }
}

void keyboardButtonTypeRequestPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeRequestPhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeRequestLocation::Class;

keyboardButtonTypeRequestLocation::keyboardButtonTypeRequestLocation() {
}

const std::int32_t keyboardButtonTypeRequestLocation::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeRequestLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<keyboardButtonTypeRequestLocation>();
}

void keyboardButtonTypeRequestLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void keyboardButtonTypeRequestLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeRequestLocation");
    s.store_class_end();
  }
}

void keyboardButtonTypeRequestLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeRequestLocation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeRequestPoll::Class;
jfieldID keyboardButtonTypeRequestPoll::force_regular_fieldID;
jfieldID keyboardButtonTypeRequestPoll::force_quiz_fieldID;

keyboardButtonTypeRequestPoll::keyboardButtonTypeRequestPoll()
  : force_regular_()
  , force_quiz_()
{}

keyboardButtonTypeRequestPoll::keyboardButtonTypeRequestPoll(bool force_regular_, bool force_quiz_)
  : force_regular_(force_regular_)
  , force_quiz_(force_quiz_)
{}

const std::int32_t keyboardButtonTypeRequestPoll::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeRequestPoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<keyboardButtonTypeRequestPoll> res = make_object<keyboardButtonTypeRequestPoll>();
  res->force_regular_ = (env->GetBooleanField(p, res->force_regular_fieldID) != 0);
  res->force_quiz_ = (env->GetBooleanField(p, res->force_quiz_fieldID) != 0);
  return std::move(res);
}

void keyboardButtonTypeRequestPoll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, force_regular_fieldID, force_regular_);
  env->SetBooleanField(s, force_quiz_fieldID, force_quiz_);
}

void keyboardButtonTypeRequestPoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeRequestPoll");
    s.store_field("forceRegular", force_regular_);
    s.store_field("forceQuiz", force_quiz_);
    s.store_class_end();
  }
}

void keyboardButtonTypeRequestPoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeRequestPoll").c_str());
    force_regular_fieldID = jni::get_field_id(env, Class, "forceRegular", "Z");
    force_quiz_fieldID = jni::get_field_id(env, Class, "forceQuiz", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeRequestUsers::Class;
jfieldID keyboardButtonTypeRequestUsers::id_fieldID;
jfieldID keyboardButtonTypeRequestUsers::restrict_user_is_bot_fieldID;
jfieldID keyboardButtonTypeRequestUsers::user_is_bot_fieldID;
jfieldID keyboardButtonTypeRequestUsers::restrict_user_is_premium_fieldID;
jfieldID keyboardButtonTypeRequestUsers::user_is_premium_fieldID;
jfieldID keyboardButtonTypeRequestUsers::max_quantity_fieldID;
jfieldID keyboardButtonTypeRequestUsers::request_name_fieldID;
jfieldID keyboardButtonTypeRequestUsers::request_username_fieldID;
jfieldID keyboardButtonTypeRequestUsers::request_photo_fieldID;

keyboardButtonTypeRequestUsers::keyboardButtonTypeRequestUsers()
  : id_()
  , restrict_user_is_bot_()
  , user_is_bot_()
  , restrict_user_is_premium_()
  , user_is_premium_()
  , max_quantity_()
  , request_name_()
  , request_username_()
  , request_photo_()
{}

keyboardButtonTypeRequestUsers::keyboardButtonTypeRequestUsers(int32 id_, bool restrict_user_is_bot_, bool user_is_bot_, bool restrict_user_is_premium_, bool user_is_premium_, int32 max_quantity_, bool request_name_, bool request_username_, bool request_photo_)
  : id_(id_)
  , restrict_user_is_bot_(restrict_user_is_bot_)
  , user_is_bot_(user_is_bot_)
  , restrict_user_is_premium_(restrict_user_is_premium_)
  , user_is_premium_(user_is_premium_)
  , max_quantity_(max_quantity_)
  , request_name_(request_name_)
  , request_username_(request_username_)
  , request_photo_(request_photo_)
{}

const std::int32_t keyboardButtonTypeRequestUsers::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeRequestUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<keyboardButtonTypeRequestUsers> res = make_object<keyboardButtonTypeRequestUsers>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->restrict_user_is_bot_ = (env->GetBooleanField(p, res->restrict_user_is_bot_fieldID) != 0);
  res->user_is_bot_ = (env->GetBooleanField(p, res->user_is_bot_fieldID) != 0);
  res->restrict_user_is_premium_ = (env->GetBooleanField(p, res->restrict_user_is_premium_fieldID) != 0);
  res->user_is_premium_ = (env->GetBooleanField(p, res->user_is_premium_fieldID) != 0);
  res->max_quantity_ = env->GetIntField(p, res->max_quantity_fieldID);
  res->request_name_ = (env->GetBooleanField(p, res->request_name_fieldID) != 0);
  res->request_username_ = (env->GetBooleanField(p, res->request_username_fieldID) != 0);
  res->request_photo_ = (env->GetBooleanField(p, res->request_photo_fieldID) != 0);
  return std::move(res);
}

void keyboardButtonTypeRequestUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetBooleanField(s, restrict_user_is_bot_fieldID, restrict_user_is_bot_);
  env->SetBooleanField(s, user_is_bot_fieldID, user_is_bot_);
  env->SetBooleanField(s, restrict_user_is_premium_fieldID, restrict_user_is_premium_);
  env->SetBooleanField(s, user_is_premium_fieldID, user_is_premium_);
  env->SetIntField(s, max_quantity_fieldID, max_quantity_);
  env->SetBooleanField(s, request_name_fieldID, request_name_);
  env->SetBooleanField(s, request_username_fieldID, request_username_);
  env->SetBooleanField(s, request_photo_fieldID, request_photo_);
}

void keyboardButtonTypeRequestUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeRequestUsers");
    s.store_field("id", id_);
    s.store_field("restrictUserIsBot", restrict_user_is_bot_);
    s.store_field("userIsBot", user_is_bot_);
    s.store_field("restrictUserIsPremium", restrict_user_is_premium_);
    s.store_field("userIsPremium", user_is_premium_);
    s.store_field("maxQuantity", max_quantity_);
    s.store_field("requestName", request_name_);
    s.store_field("requestUsername", request_username_);
    s.store_field("requestPhoto", request_photo_);
    s.store_class_end();
  }
}

void keyboardButtonTypeRequestUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeRequestUsers").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    restrict_user_is_bot_fieldID = jni::get_field_id(env, Class, "restrictUserIsBot", "Z");
    user_is_bot_fieldID = jni::get_field_id(env, Class, "userIsBot", "Z");
    restrict_user_is_premium_fieldID = jni::get_field_id(env, Class, "restrictUserIsPremium", "Z");
    user_is_premium_fieldID = jni::get_field_id(env, Class, "userIsPremium", "Z");
    max_quantity_fieldID = jni::get_field_id(env, Class, "maxQuantity", "I");
    request_name_fieldID = jni::get_field_id(env, Class, "requestName", "Z");
    request_username_fieldID = jni::get_field_id(env, Class, "requestUsername", "Z");
    request_photo_fieldID = jni::get_field_id(env, Class, "requestPhoto", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeRequestChat::Class;
jfieldID keyboardButtonTypeRequestChat::id_fieldID;
jfieldID keyboardButtonTypeRequestChat::chat_is_channel_fieldID;
jfieldID keyboardButtonTypeRequestChat::restrict_chat_is_forum_fieldID;
jfieldID keyboardButtonTypeRequestChat::chat_is_forum_fieldID;
jfieldID keyboardButtonTypeRequestChat::restrict_chat_has_username_fieldID;
jfieldID keyboardButtonTypeRequestChat::chat_has_username_fieldID;
jfieldID keyboardButtonTypeRequestChat::chat_is_created_fieldID;
jfieldID keyboardButtonTypeRequestChat::user_administrator_rights_fieldID;
jfieldID keyboardButtonTypeRequestChat::bot_administrator_rights_fieldID;
jfieldID keyboardButtonTypeRequestChat::bot_is_member_fieldID;
jfieldID keyboardButtonTypeRequestChat::request_title_fieldID;
jfieldID keyboardButtonTypeRequestChat::request_username_fieldID;
jfieldID keyboardButtonTypeRequestChat::request_photo_fieldID;

keyboardButtonTypeRequestChat::keyboardButtonTypeRequestChat()
  : id_()
  , chat_is_channel_()
  , restrict_chat_is_forum_()
  , chat_is_forum_()
  , restrict_chat_has_username_()
  , chat_has_username_()
  , chat_is_created_()
  , user_administrator_rights_()
  , bot_administrator_rights_()
  , bot_is_member_()
  , request_title_()
  , request_username_()
  , request_photo_()
{}

keyboardButtonTypeRequestChat::keyboardButtonTypeRequestChat(int32 id_, bool chat_is_channel_, bool restrict_chat_is_forum_, bool chat_is_forum_, bool restrict_chat_has_username_, bool chat_has_username_, bool chat_is_created_, object_ptr<chatAdministratorRights> &&user_administrator_rights_, object_ptr<chatAdministratorRights> &&bot_administrator_rights_, bool bot_is_member_, bool request_title_, bool request_username_, bool request_photo_)
  : id_(id_)
  , chat_is_channel_(chat_is_channel_)
  , restrict_chat_is_forum_(restrict_chat_is_forum_)
  , chat_is_forum_(chat_is_forum_)
  , restrict_chat_has_username_(restrict_chat_has_username_)
  , chat_has_username_(chat_has_username_)
  , chat_is_created_(chat_is_created_)
  , user_administrator_rights_(std::move(user_administrator_rights_))
  , bot_administrator_rights_(std::move(bot_administrator_rights_))
  , bot_is_member_(bot_is_member_)
  , request_title_(request_title_)
  , request_username_(request_username_)
  , request_photo_(request_photo_)
{}

const std::int32_t keyboardButtonTypeRequestChat::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeRequestChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<keyboardButtonTypeRequestChat> res = make_object<keyboardButtonTypeRequestChat>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->chat_is_channel_ = (env->GetBooleanField(p, res->chat_is_channel_fieldID) != 0);
  res->restrict_chat_is_forum_ = (env->GetBooleanField(p, res->restrict_chat_is_forum_fieldID) != 0);
  res->chat_is_forum_ = (env->GetBooleanField(p, res->chat_is_forum_fieldID) != 0);
  res->restrict_chat_has_username_ = (env->GetBooleanField(p, res->restrict_chat_has_username_fieldID) != 0);
  res->chat_has_username_ = (env->GetBooleanField(p, res->chat_has_username_fieldID) != 0);
  res->chat_is_created_ = (env->GetBooleanField(p, res->chat_is_created_fieldID) != 0);
  res->user_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->user_administrator_rights_fieldID));
  res->bot_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->bot_administrator_rights_fieldID));
  res->bot_is_member_ = (env->GetBooleanField(p, res->bot_is_member_fieldID) != 0);
  res->request_title_ = (env->GetBooleanField(p, res->request_title_fieldID) != 0);
  res->request_username_ = (env->GetBooleanField(p, res->request_username_fieldID) != 0);
  res->request_photo_ = (env->GetBooleanField(p, res->request_photo_fieldID) != 0);
  return std::move(res);
}

void keyboardButtonTypeRequestChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetBooleanField(s, chat_is_channel_fieldID, chat_is_channel_);
  env->SetBooleanField(s, restrict_chat_is_forum_fieldID, restrict_chat_is_forum_);
  env->SetBooleanField(s, chat_is_forum_fieldID, chat_is_forum_);
  env->SetBooleanField(s, restrict_chat_has_username_fieldID, restrict_chat_has_username_);
  env->SetBooleanField(s, chat_has_username_fieldID, chat_has_username_);
  env->SetBooleanField(s, chat_is_created_fieldID, chat_is_created_);
  if (user_administrator_rights_ != nullptr) { jobject next; user_administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, user_administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
  if (bot_administrator_rights_ != nullptr) { jobject next; bot_administrator_rights_->store(env, next); if (next) { env->SetObjectField(s, bot_administrator_rights_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, bot_is_member_fieldID, bot_is_member_);
  env->SetBooleanField(s, request_title_fieldID, request_title_);
  env->SetBooleanField(s, request_username_fieldID, request_username_);
  env->SetBooleanField(s, request_photo_fieldID, request_photo_);
}

void keyboardButtonTypeRequestChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeRequestChat");
    s.store_field("id", id_);
    s.store_field("chatIsChannel", chat_is_channel_);
    s.store_field("restrictChatIsForum", restrict_chat_is_forum_);
    s.store_field("chatIsForum", chat_is_forum_);
    s.store_field("restrictChatHasUsername", restrict_chat_has_username_);
    s.store_field("chatHasUsername", chat_has_username_);
    s.store_field("chatIsCreated", chat_is_created_);
    s.store_object_field("userAdministratorRights", static_cast<const BaseObject *>(user_administrator_rights_.get()));
    s.store_object_field("botAdministratorRights", static_cast<const BaseObject *>(bot_administrator_rights_.get()));
    s.store_field("botIsMember", bot_is_member_);
    s.store_field("requestTitle", request_title_);
    s.store_field("requestUsername", request_username_);
    s.store_field("requestPhoto", request_photo_);
    s.store_class_end();
  }
}

void keyboardButtonTypeRequestChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeRequestChat").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    chat_is_channel_fieldID = jni::get_field_id(env, Class, "chatIsChannel", "Z");
    restrict_chat_is_forum_fieldID = jni::get_field_id(env, Class, "restrictChatIsForum", "Z");
    chat_is_forum_fieldID = jni::get_field_id(env, Class, "chatIsForum", "Z");
    restrict_chat_has_username_fieldID = jni::get_field_id(env, Class, "restrictChatHasUsername", "Z");
    chat_has_username_fieldID = jni::get_field_id(env, Class, "chatHasUsername", "Z");
    chat_is_created_fieldID = jni::get_field_id(env, Class, "chatIsCreated", "Z");
    user_administrator_rights_fieldID = jni::get_field_id(env, Class, "userAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    bot_administrator_rights_fieldID = jni::get_field_id(env, Class, "botAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    bot_is_member_fieldID = jni::get_field_id(env, Class, "botIsMember", "Z");
    request_title_fieldID = jni::get_field_id(env, Class, "requestTitle", "Z");
    request_username_fieldID = jni::get_field_id(env, Class, "requestUsername", "Z");
    request_photo_fieldID = jni::get_field_id(env, Class, "requestPhoto", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass keyboardButtonTypeWebApp::Class;
jfieldID keyboardButtonTypeWebApp::url_fieldID;

keyboardButtonTypeWebApp::keyboardButtonTypeWebApp()
  : url_()
{}

keyboardButtonTypeWebApp::keyboardButtonTypeWebApp(string const &url_)
  : url_(url_)
{}

const std::int32_t keyboardButtonTypeWebApp::ID;

object_ptr<KeyboardButtonType> keyboardButtonTypeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<keyboardButtonTypeWebApp> res = make_object<keyboardButtonTypeWebApp>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void keyboardButtonTypeWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void keyboardButtonTypeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "KeyboardButtonTypeWebApp");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void keyboardButtonTypeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$KeyboardButtonTypeWebApp").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass labeledPricePart::Class;
jfieldID labeledPricePart::label_fieldID;
jfieldID labeledPricePart::amount_fieldID;

labeledPricePart::labeledPricePart()
  : label_()
  , amount_()
{}

labeledPricePart::labeledPricePart(string const &label_, int53 amount_)
  : label_(label_)
  , amount_(amount_)
{}

const std::int32_t labeledPricePart::ID;

object_ptr<labeledPricePart> labeledPricePart::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<labeledPricePart> res = make_object<labeledPricePart>();
  res->label_ = jni::fetch_string(env, p, res->label_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  return res;
}

void labeledPricePart::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, label_); if (nextString) { env->SetObjectField(s, label_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
}

void labeledPricePart::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LabeledPricePart");
    s.store_field("label", label_);
    s.store_field("amount", amount_);
    s.store_class_end();
  }
}

void labeledPricePart::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LabeledPricePart").c_str());
    label_fieldID = jni::get_field_id(env, Class, "label", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass languagePackInfo::Class;
jfieldID languagePackInfo::id_fieldID;
jfieldID languagePackInfo::base_language_pack_id_fieldID;
jfieldID languagePackInfo::name_fieldID;
jfieldID languagePackInfo::native_name_fieldID;
jfieldID languagePackInfo::plural_code_fieldID;
jfieldID languagePackInfo::is_official_fieldID;
jfieldID languagePackInfo::is_rtl_fieldID;
jfieldID languagePackInfo::is_beta_fieldID;
jfieldID languagePackInfo::is_installed_fieldID;
jfieldID languagePackInfo::total_string_count_fieldID;
jfieldID languagePackInfo::translated_string_count_fieldID;
jfieldID languagePackInfo::local_string_count_fieldID;
jfieldID languagePackInfo::translation_url_fieldID;

languagePackInfo::languagePackInfo()
  : id_()
  , base_language_pack_id_()
  , name_()
  , native_name_()
  , plural_code_()
  , is_official_()
  , is_rtl_()
  , is_beta_()
  , is_installed_()
  , total_string_count_()
  , translated_string_count_()
  , local_string_count_()
  , translation_url_()
{}

languagePackInfo::languagePackInfo(string const &id_, string const &base_language_pack_id_, string const &name_, string const &native_name_, string const &plural_code_, bool is_official_, bool is_rtl_, bool is_beta_, bool is_installed_, int32 total_string_count_, int32 translated_string_count_, int32 local_string_count_, string const &translation_url_)
  : id_(id_)
  , base_language_pack_id_(base_language_pack_id_)
  , name_(name_)
  , native_name_(native_name_)
  , plural_code_(plural_code_)
  , is_official_(is_official_)
  , is_rtl_(is_rtl_)
  , is_beta_(is_beta_)
  , is_installed_(is_installed_)
  , total_string_count_(total_string_count_)
  , translated_string_count_(translated_string_count_)
  , local_string_count_(local_string_count_)
  , translation_url_(translation_url_)
{}

const std::int32_t languagePackInfo::ID;

object_ptr<languagePackInfo> languagePackInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<languagePackInfo> res = make_object<languagePackInfo>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->base_language_pack_id_ = jni::fetch_string(env, p, res->base_language_pack_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->native_name_ = jni::fetch_string(env, p, res->native_name_fieldID);
  res->plural_code_ = jni::fetch_string(env, p, res->plural_code_fieldID);
  res->is_official_ = (env->GetBooleanField(p, res->is_official_fieldID) != 0);
  res->is_rtl_ = (env->GetBooleanField(p, res->is_rtl_fieldID) != 0);
  res->is_beta_ = (env->GetBooleanField(p, res->is_beta_fieldID) != 0);
  res->is_installed_ = (env->GetBooleanField(p, res->is_installed_fieldID) != 0);
  res->total_string_count_ = env->GetIntField(p, res->total_string_count_fieldID);
  res->translated_string_count_ = env->GetIntField(p, res->translated_string_count_fieldID);
  res->local_string_count_ = env->GetIntField(p, res->local_string_count_fieldID);
  res->translation_url_ = jni::fetch_string(env, p, res->translation_url_fieldID);
  return res;
}

void languagePackInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, base_language_pack_id_); if (nextString) { env->SetObjectField(s, base_language_pack_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, native_name_); if (nextString) { env->SetObjectField(s, native_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, plural_code_); if (nextString) { env->SetObjectField(s, plural_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_official_fieldID, is_official_);
  env->SetBooleanField(s, is_rtl_fieldID, is_rtl_);
  env->SetBooleanField(s, is_beta_fieldID, is_beta_);
  env->SetBooleanField(s, is_installed_fieldID, is_installed_);
  env->SetIntField(s, total_string_count_fieldID, total_string_count_);
  env->SetIntField(s, translated_string_count_fieldID, translated_string_count_);
  env->SetIntField(s, local_string_count_fieldID, local_string_count_);
  { jstring nextString = jni::to_jstring(env, translation_url_); if (nextString) { env->SetObjectField(s, translation_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void languagePackInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackInfo");
    s.store_field("id", id_);
    s.store_field("baseLanguagePackId", base_language_pack_id_);
    s.store_field("name", name_);
    s.store_field("nativeName", native_name_);
    s.store_field("pluralCode", plural_code_);
    s.store_field("isOfficial", is_official_);
    s.store_field("isRtl", is_rtl_);
    s.store_field("isBeta", is_beta_);
    s.store_field("isInstalled", is_installed_);
    s.store_field("totalStringCount", total_string_count_);
    s.store_field("translatedStringCount", translated_string_count_);
    s.store_field("localStringCount", local_string_count_);
    s.store_field("translationUrl", translation_url_);
    s.store_class_end();
  }
}

void languagePackInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackInfo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    base_language_pack_id_fieldID = jni::get_field_id(env, Class, "baseLanguagePackId", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    native_name_fieldID = jni::get_field_id(env, Class, "nativeName", "Ljava/lang/String;");
    plural_code_fieldID = jni::get_field_id(env, Class, "pluralCode", "Ljava/lang/String;");
    is_official_fieldID = jni::get_field_id(env, Class, "isOfficial", "Z");
    is_rtl_fieldID = jni::get_field_id(env, Class, "isRtl", "Z");
    is_beta_fieldID = jni::get_field_id(env, Class, "isBeta", "Z");
    is_installed_fieldID = jni::get_field_id(env, Class, "isInstalled", "Z");
    total_string_count_fieldID = jni::get_field_id(env, Class, "totalStringCount", "I");
    translated_string_count_fieldID = jni::get_field_id(env, Class, "translatedStringCount", "I");
    local_string_count_fieldID = jni::get_field_id(env, Class, "localStringCount", "I");
    translation_url_fieldID = jni::get_field_id(env, Class, "translationUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass languagePackString::Class;
jfieldID languagePackString::key_fieldID;
jfieldID languagePackString::value_fieldID;

languagePackString::languagePackString()
  : key_()
  , value_()
{}

languagePackString::languagePackString(string const &key_, object_ptr<LanguagePackStringValue> &&value_)
  : key_(key_)
  , value_(std::move(value_))
{}

const std::int32_t languagePackString::ID;

object_ptr<languagePackString> languagePackString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<languagePackString> res = make_object<languagePackString>();
  res->key_ = jni::fetch_string(env, p, res->key_fieldID);
  res->value_ = jni::fetch_tl_object<LanguagePackStringValue>(env, jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void languagePackString::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, key_); if (nextString) { env->SetObjectField(s, key_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (value_ != nullptr) { jobject next; value_->store(env, next); if (next) { env->SetObjectField(s, value_fieldID, next); env->DeleteLocalRef(next); } }
}

void languagePackString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackString");
    s.store_field("key", key_);
    s.store_object_field("value", static_cast<const BaseObject *>(value_.get()));
    s.store_class_end();
  }
}

void languagePackString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackString").c_str());
    key_fieldID = jni::get_field_id(env, Class, "key", "Ljava/lang/String;");
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LanguagePackStringValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass LanguagePackStringValue::Class;

object_ptr<LanguagePackStringValue> LanguagePackStringValue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case languagePackStringValueOrdinary::ID:
      return languagePackStringValueOrdinary::fetch(env, p);
    case languagePackStringValuePluralized::ID:
      return languagePackStringValuePluralized::fetch(env, p);
    case languagePackStringValueDeleted::ID:
      return languagePackStringValueDeleted::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void LanguagePackStringValue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackStringValue").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass languagePackStringValueOrdinary::Class;
jfieldID languagePackStringValueOrdinary::value_fieldID;

languagePackStringValueOrdinary::languagePackStringValueOrdinary()
  : value_()
{}

languagePackStringValueOrdinary::languagePackStringValueOrdinary(string const &value_)
  : value_(value_)
{}

const std::int32_t languagePackStringValueOrdinary::ID;

object_ptr<LanguagePackStringValue> languagePackStringValueOrdinary::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<languagePackStringValueOrdinary> res = make_object<languagePackStringValueOrdinary>();
  res->value_ = jni::fetch_string(env, p, res->value_fieldID);
  return std::move(res);
}

void languagePackStringValueOrdinary::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, value_); if (nextString) { env->SetObjectField(s, value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void languagePackStringValueOrdinary::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackStringValueOrdinary");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void languagePackStringValueOrdinary::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackStringValueOrdinary").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass languagePackStringValuePluralized::Class;
jfieldID languagePackStringValuePluralized::zero_value_fieldID;
jfieldID languagePackStringValuePluralized::one_value_fieldID;
jfieldID languagePackStringValuePluralized::two_value_fieldID;
jfieldID languagePackStringValuePluralized::few_value_fieldID;
jfieldID languagePackStringValuePluralized::many_value_fieldID;
jfieldID languagePackStringValuePluralized::other_value_fieldID;

languagePackStringValuePluralized::languagePackStringValuePluralized()
  : zero_value_()
  , one_value_()
  , two_value_()
  , few_value_()
  , many_value_()
  , other_value_()
{}

languagePackStringValuePluralized::languagePackStringValuePluralized(string const &zero_value_, string const &one_value_, string const &two_value_, string const &few_value_, string const &many_value_, string const &other_value_)
  : zero_value_(zero_value_)
  , one_value_(one_value_)
  , two_value_(two_value_)
  , few_value_(few_value_)
  , many_value_(many_value_)
  , other_value_(other_value_)
{}

const std::int32_t languagePackStringValuePluralized::ID;

object_ptr<LanguagePackStringValue> languagePackStringValuePluralized::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<languagePackStringValuePluralized> res = make_object<languagePackStringValuePluralized>();
  res->zero_value_ = jni::fetch_string(env, p, res->zero_value_fieldID);
  res->one_value_ = jni::fetch_string(env, p, res->one_value_fieldID);
  res->two_value_ = jni::fetch_string(env, p, res->two_value_fieldID);
  res->few_value_ = jni::fetch_string(env, p, res->few_value_fieldID);
  res->many_value_ = jni::fetch_string(env, p, res->many_value_fieldID);
  res->other_value_ = jni::fetch_string(env, p, res->other_value_fieldID);
  return std::move(res);
}

void languagePackStringValuePluralized::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, zero_value_); if (nextString) { env->SetObjectField(s, zero_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, one_value_); if (nextString) { env->SetObjectField(s, one_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, two_value_); if (nextString) { env->SetObjectField(s, two_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, few_value_); if (nextString) { env->SetObjectField(s, few_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, many_value_); if (nextString) { env->SetObjectField(s, many_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, other_value_); if (nextString) { env->SetObjectField(s, other_value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void languagePackStringValuePluralized::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackStringValuePluralized");
    s.store_field("zeroValue", zero_value_);
    s.store_field("oneValue", one_value_);
    s.store_field("twoValue", two_value_);
    s.store_field("fewValue", few_value_);
    s.store_field("manyValue", many_value_);
    s.store_field("otherValue", other_value_);
    s.store_class_end();
  }
}

void languagePackStringValuePluralized::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackStringValuePluralized").c_str());
    zero_value_fieldID = jni::get_field_id(env, Class, "zeroValue", "Ljava/lang/String;");
    one_value_fieldID = jni::get_field_id(env, Class, "oneValue", "Ljava/lang/String;");
    two_value_fieldID = jni::get_field_id(env, Class, "twoValue", "Ljava/lang/String;");
    few_value_fieldID = jni::get_field_id(env, Class, "fewValue", "Ljava/lang/String;");
    many_value_fieldID = jni::get_field_id(env, Class, "manyValue", "Ljava/lang/String;");
    other_value_fieldID = jni::get_field_id(env, Class, "otherValue", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass languagePackStringValueDeleted::Class;

languagePackStringValueDeleted::languagePackStringValueDeleted() {
}

const std::int32_t languagePackStringValueDeleted::ID;

object_ptr<LanguagePackStringValue> languagePackStringValueDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<languagePackStringValueDeleted>();
}

void languagePackStringValueDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void languagePackStringValueDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackStringValueDeleted");
    s.store_class_end();
  }
}

void languagePackStringValueDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackStringValueDeleted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass languagePackStrings::Class;
jfieldID languagePackStrings::strings_fieldID;

languagePackStrings::languagePackStrings()
  : strings_()
{}

languagePackStrings::languagePackStrings(array<object_ptr<languagePackString>> &&strings_)
  : strings_(std::move(strings_))
{}

const std::int32_t languagePackStrings::ID;

object_ptr<languagePackStrings> languagePackStrings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<languagePackStrings> res = make_object<languagePackStrings>();
  res->strings_ = jni::FetchVector<languagePackString>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->strings_fieldID));
  return res;
}

void languagePackStrings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, strings_); if (arr_tmp_) { env->SetObjectField(s, strings_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void languagePackStrings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LanguagePackStrings");
    { s.store_vector_begin("strings", strings_.size()); for (const auto &_value : strings_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void languagePackStrings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LanguagePackStrings").c_str());
    strings_fieldID = jni::get_field_id(env, Class, "strings", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LanguagePackString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreview::Class;
jfieldID linkPreview::url_fieldID;
jfieldID linkPreview::display_url_fieldID;
jfieldID linkPreview::site_name_fieldID;
jfieldID linkPreview::title_fieldID;
jfieldID linkPreview::description_fieldID;
jfieldID linkPreview::author_fieldID;
jfieldID linkPreview::type_fieldID;
jfieldID linkPreview::has_large_media_fieldID;
jfieldID linkPreview::show_large_media_fieldID;
jfieldID linkPreview::show_media_above_description_fieldID;
jfieldID linkPreview::skip_confirmation_fieldID;
jfieldID linkPreview::show_above_text_fieldID;
jfieldID linkPreview::instant_view_version_fieldID;

linkPreview::linkPreview()
  : url_()
  , display_url_()
  , site_name_()
  , title_()
  , description_()
  , author_()
  , type_()
  , has_large_media_()
  , show_large_media_()
  , show_media_above_description_()
  , skip_confirmation_()
  , show_above_text_()
  , instant_view_version_()
{}

linkPreview::linkPreview(string const &url_, string const &display_url_, string const &site_name_, string const &title_, object_ptr<formattedText> &&description_, string const &author_, object_ptr<LinkPreviewType> &&type_, bool has_large_media_, bool show_large_media_, bool show_media_above_description_, bool skip_confirmation_, bool show_above_text_, int32 instant_view_version_)
  : url_(url_)
  , display_url_(display_url_)
  , site_name_(site_name_)
  , title_(title_)
  , description_(std::move(description_))
  , author_(author_)
  , type_(std::move(type_))
  , has_large_media_(has_large_media_)
  , show_large_media_(show_large_media_)
  , show_media_above_description_(show_media_above_description_)
  , skip_confirmation_(skip_confirmation_)
  , show_above_text_(show_above_text_)
  , instant_view_version_(instant_view_version_)
{}

const std::int32_t linkPreview::ID;

object_ptr<linkPreview> linkPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreview> res = make_object<linkPreview>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->display_url_ = jni::fetch_string(env, p, res->display_url_fieldID);
  res->site_name_ = jni::fetch_string(env, p, res->site_name_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->description_fieldID));
  res->author_ = jni::fetch_string(env, p, res->author_fieldID);
  res->type_ = jni::fetch_tl_object<LinkPreviewType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->has_large_media_ = (env->GetBooleanField(p, res->has_large_media_fieldID) != 0);
  res->show_large_media_ = (env->GetBooleanField(p, res->show_large_media_fieldID) != 0);
  res->show_media_above_description_ = (env->GetBooleanField(p, res->show_media_above_description_fieldID) != 0);
  res->skip_confirmation_ = (env->GetBooleanField(p, res->skip_confirmation_fieldID) != 0);
  res->show_above_text_ = (env->GetBooleanField(p, res->show_above_text_fieldID) != 0);
  res->instant_view_version_ = env->GetIntField(p, res->instant_view_version_fieldID);
  return res;
}

void linkPreview::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, display_url_); if (nextString) { env->SetObjectField(s, display_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, site_name_); if (nextString) { env->SetObjectField(s, site_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (description_ != nullptr) { jobject next; description_->store(env, next); if (next) { env->SetObjectField(s, description_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, author_); if (nextString) { env->SetObjectField(s, author_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_large_media_fieldID, has_large_media_);
  env->SetBooleanField(s, show_large_media_fieldID, show_large_media_);
  env->SetBooleanField(s, show_media_above_description_fieldID, show_media_above_description_);
  env->SetBooleanField(s, skip_confirmation_fieldID, skip_confirmation_);
  env->SetBooleanField(s, show_above_text_fieldID, show_above_text_);
  env->SetIntField(s, instant_view_version_fieldID, instant_view_version_);
}

void linkPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreview");
    s.store_field("url", url_);
    s.store_field("displayUrl", display_url_);
    s.store_field("siteName", site_name_);
    s.store_field("title", title_);
    s.store_object_field("description", static_cast<const BaseObject *>(description_.get()));
    s.store_field("author", author_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("hasLargeMedia", has_large_media_);
    s.store_field("showLargeMedia", show_large_media_);
    s.store_field("showMediaAboveDescription", show_media_above_description_);
    s.store_field("skipConfirmation", skip_confirmation_);
    s.store_field("showAboveText", show_above_text_);
    s.store_field("instantViewVersion", instant_view_version_);
    s.store_class_end();
  }
}

void linkPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreview").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    display_url_fieldID = jni::get_field_id(env, Class, "displayUrl", "Ljava/lang/String;");
    site_name_fieldID = jni::get_field_id(env, Class, "siteName", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    author_fieldID = jni::get_field_id(env, Class, "author", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LinkPreviewType;").c_str());
    has_large_media_fieldID = jni::get_field_id(env, Class, "hasLargeMedia", "Z");
    show_large_media_fieldID = jni::get_field_id(env, Class, "showLargeMedia", "Z");
    show_media_above_description_fieldID = jni::get_field_id(env, Class, "showMediaAboveDescription", "Z");
    skip_confirmation_fieldID = jni::get_field_id(env, Class, "skipConfirmation", "Z");
    show_above_text_fieldID = jni::get_field_id(env, Class, "showAboveText", "Z");
    instant_view_version_fieldID = jni::get_field_id(env, Class, "instantViewVersion", "I");
    return true;
  }();
  (void)is_inited;
}

jclass LinkPreviewAlbumMedia::Class;

object_ptr<LinkPreviewAlbumMedia> LinkPreviewAlbumMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case linkPreviewAlbumMediaPhoto::ID:
      return linkPreviewAlbumMediaPhoto::fetch(env, p);
    case linkPreviewAlbumMediaVideo::ID:
      return linkPreviewAlbumMediaVideo::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void LinkPreviewAlbumMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewAlbumMedia").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewAlbumMediaPhoto::Class;
jfieldID linkPreviewAlbumMediaPhoto::photo_fieldID;

linkPreviewAlbumMediaPhoto::linkPreviewAlbumMediaPhoto()
  : photo_()
{}

linkPreviewAlbumMediaPhoto::linkPreviewAlbumMediaPhoto(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewAlbumMediaPhoto::ID;

object_ptr<LinkPreviewAlbumMedia> linkPreviewAlbumMediaPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewAlbumMediaPhoto> res = make_object<linkPreviewAlbumMediaPhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewAlbumMediaPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewAlbumMediaPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewAlbumMediaPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewAlbumMediaPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewAlbumMediaPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewAlbumMediaVideo::Class;
jfieldID linkPreviewAlbumMediaVideo::video_fieldID;

linkPreviewAlbumMediaVideo::linkPreviewAlbumMediaVideo()
  : video_()
{}

linkPreviewAlbumMediaVideo::linkPreviewAlbumMediaVideo(object_ptr<video> &&video_)
  : video_(std::move(video_))
{}

const std::int32_t linkPreviewAlbumMediaVideo::ID;

object_ptr<LinkPreviewAlbumMedia> linkPreviewAlbumMediaVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewAlbumMediaVideo> res = make_object<linkPreviewAlbumMediaVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  return std::move(res);
}

void linkPreviewAlbumMediaVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewAlbumMediaVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewAlbumMediaVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_class_end();
  }
}

void linkPreviewAlbumMediaVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewAlbumMediaVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewOptions::Class;
jfieldID linkPreviewOptions::is_disabled_fieldID;
jfieldID linkPreviewOptions::url_fieldID;
jfieldID linkPreviewOptions::force_small_media_fieldID;
jfieldID linkPreviewOptions::force_large_media_fieldID;
jfieldID linkPreviewOptions::show_above_text_fieldID;

linkPreviewOptions::linkPreviewOptions()
  : is_disabled_()
  , url_()
  , force_small_media_()
  , force_large_media_()
  , show_above_text_()
{}

linkPreviewOptions::linkPreviewOptions(bool is_disabled_, string const &url_, bool force_small_media_, bool force_large_media_, bool show_above_text_)
  : is_disabled_(is_disabled_)
  , url_(url_)
  , force_small_media_(force_small_media_)
  , force_large_media_(force_large_media_)
  , show_above_text_(show_above_text_)
{}

const std::int32_t linkPreviewOptions::ID;

object_ptr<linkPreviewOptions> linkPreviewOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewOptions> res = make_object<linkPreviewOptions>();
  res->is_disabled_ = (env->GetBooleanField(p, res->is_disabled_fieldID) != 0);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->force_small_media_ = (env->GetBooleanField(p, res->force_small_media_fieldID) != 0);
  res->force_large_media_ = (env->GetBooleanField(p, res->force_large_media_fieldID) != 0);
  res->show_above_text_ = (env->GetBooleanField(p, res->show_above_text_fieldID) != 0);
  return res;
}

void linkPreviewOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_disabled_fieldID, is_disabled_);
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, force_small_media_fieldID, force_small_media_);
  env->SetBooleanField(s, force_large_media_fieldID, force_large_media_);
  env->SetBooleanField(s, show_above_text_fieldID, show_above_text_);
}

void linkPreviewOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewOptions");
    s.store_field("isDisabled", is_disabled_);
    s.store_field("url", url_);
    s.store_field("forceSmallMedia", force_small_media_);
    s.store_field("forceLargeMedia", force_large_media_);
    s.store_field("showAboveText", show_above_text_);
    s.store_class_end();
  }
}

void linkPreviewOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewOptions").c_str());
    is_disabled_fieldID = jni::get_field_id(env, Class, "isDisabled", "Z");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    force_small_media_fieldID = jni::get_field_id(env, Class, "forceSmallMedia", "Z");
    force_large_media_fieldID = jni::get_field_id(env, Class, "forceLargeMedia", "Z");
    show_above_text_fieldID = jni::get_field_id(env, Class, "showAboveText", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass LinkPreviewType::Class;

object_ptr<LinkPreviewType> LinkPreviewType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case linkPreviewTypeAlbum::ID:
      return linkPreviewTypeAlbum::fetch(env, p);
    case linkPreviewTypeAnimation::ID:
      return linkPreviewTypeAnimation::fetch(env, p);
    case linkPreviewTypeApp::ID:
      return linkPreviewTypeApp::fetch(env, p);
    case linkPreviewTypeArticle::ID:
      return linkPreviewTypeArticle::fetch(env, p);
    case linkPreviewTypeAudio::ID:
      return linkPreviewTypeAudio::fetch(env, p);
    case linkPreviewTypeBackground::ID:
      return linkPreviewTypeBackground::fetch(env, p);
    case linkPreviewTypeChannelBoost::ID:
      return linkPreviewTypeChannelBoost::fetch(env, p);
    case linkPreviewTypeChat::ID:
      return linkPreviewTypeChat::fetch(env, p);
    case linkPreviewTypeDocument::ID:
      return linkPreviewTypeDocument::fetch(env, p);
    case linkPreviewTypeEmbeddedAnimationPlayer::ID:
      return linkPreviewTypeEmbeddedAnimationPlayer::fetch(env, p);
    case linkPreviewTypeEmbeddedAudioPlayer::ID:
      return linkPreviewTypeEmbeddedAudioPlayer::fetch(env, p);
    case linkPreviewTypeEmbeddedVideoPlayer::ID:
      return linkPreviewTypeEmbeddedVideoPlayer::fetch(env, p);
    case linkPreviewTypeExternalAudio::ID:
      return linkPreviewTypeExternalAudio::fetch(env, p);
    case linkPreviewTypeExternalVideo::ID:
      return linkPreviewTypeExternalVideo::fetch(env, p);
    case linkPreviewTypeGroupCall::ID:
      return linkPreviewTypeGroupCall::fetch(env, p);
    case linkPreviewTypeInvoice::ID:
      return linkPreviewTypeInvoice::fetch(env, p);
    case linkPreviewTypeMessage::ID:
      return linkPreviewTypeMessage::fetch(env, p);
    case linkPreviewTypePhoto::ID:
      return linkPreviewTypePhoto::fetch(env, p);
    case linkPreviewTypePremiumGiftCode::ID:
      return linkPreviewTypePremiumGiftCode::fetch(env, p);
    case linkPreviewTypeShareableChatFolder::ID:
      return linkPreviewTypeShareableChatFolder::fetch(env, p);
    case linkPreviewTypeSticker::ID:
      return linkPreviewTypeSticker::fetch(env, p);
    case linkPreviewTypeStickerSet::ID:
      return linkPreviewTypeStickerSet::fetch(env, p);
    case linkPreviewTypeStory::ID:
      return linkPreviewTypeStory::fetch(env, p);
    case linkPreviewTypeSupergroupBoost::ID:
      return linkPreviewTypeSupergroupBoost::fetch(env, p);
    case linkPreviewTypeTheme::ID:
      return linkPreviewTypeTheme::fetch(env, p);
    case linkPreviewTypeUnsupported::ID:
      return linkPreviewTypeUnsupported::fetch(env, p);
    case linkPreviewTypeUpgradedGift::ID:
      return linkPreviewTypeUpgradedGift::fetch(env, p);
    case linkPreviewTypeUser::ID:
      return linkPreviewTypeUser::fetch(env, p);
    case linkPreviewTypeVideo::ID:
      return linkPreviewTypeVideo::fetch(env, p);
    case linkPreviewTypeVideoChat::ID:
      return linkPreviewTypeVideoChat::fetch(env, p);
    case linkPreviewTypeVideoNote::ID:
      return linkPreviewTypeVideoNote::fetch(env, p);
    case linkPreviewTypeVoiceNote::ID:
      return linkPreviewTypeVoiceNote::fetch(env, p);
    case linkPreviewTypeWebApp::ID:
      return linkPreviewTypeWebApp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void LinkPreviewType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeAlbum::Class;
jfieldID linkPreviewTypeAlbum::media_fieldID;
jfieldID linkPreviewTypeAlbum::caption_fieldID;

linkPreviewTypeAlbum::linkPreviewTypeAlbum()
  : media_()
  , caption_()
{}

linkPreviewTypeAlbum::linkPreviewTypeAlbum(array<object_ptr<LinkPreviewAlbumMedia>> &&media_, string const &caption_)
  : media_(std::move(media_))
  , caption_(caption_)
{}

const std::int32_t linkPreviewTypeAlbum::ID;

object_ptr<LinkPreviewType> linkPreviewTypeAlbum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeAlbum> res = make_object<linkPreviewTypeAlbum>();
  res->media_ = jni::FetchVector<LinkPreviewAlbumMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  res->caption_ = jni::fetch_string(env, p, res->caption_fieldID);
  return std::move(res);
}

void linkPreviewTypeAlbum::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, caption_); if (nextString) { env->SetObjectField(s, caption_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void linkPreviewTypeAlbum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeAlbum");
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("caption", caption_);
    s.store_class_end();
  }
}

void linkPreviewTypeAlbum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeAlbum").c_str());
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LinkPreviewAlbumMedia;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeAnimation::Class;
jfieldID linkPreviewTypeAnimation::animation_fieldID;

linkPreviewTypeAnimation::linkPreviewTypeAnimation()
  : animation_()
{}

linkPreviewTypeAnimation::linkPreviewTypeAnimation(object_ptr<animation> &&animation_)
  : animation_(std::move(animation_))
{}

const std::int32_t linkPreviewTypeAnimation::ID;

object_ptr<LinkPreviewType> linkPreviewTypeAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeAnimation> res = make_object<linkPreviewTypeAnimation>();
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return std::move(res);
}

void linkPreviewTypeAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeApp::Class;
jfieldID linkPreviewTypeApp::photo_fieldID;

linkPreviewTypeApp::linkPreviewTypeApp()
  : photo_()
{}

linkPreviewTypeApp::linkPreviewTypeApp(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypeApp::ID;

object_ptr<LinkPreviewType> linkPreviewTypeApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeApp> res = make_object<linkPreviewTypeApp>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypeApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeApp");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeApp").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeArticle::Class;
jfieldID linkPreviewTypeArticle::photo_fieldID;

linkPreviewTypeArticle::linkPreviewTypeArticle()
  : photo_()
{}

linkPreviewTypeArticle::linkPreviewTypeArticle(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypeArticle::ID;

object_ptr<LinkPreviewType> linkPreviewTypeArticle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeArticle> res = make_object<linkPreviewTypeArticle>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypeArticle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeArticle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeArticle");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeArticle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeArticle").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeAudio::Class;
jfieldID linkPreviewTypeAudio::audio_fieldID;

linkPreviewTypeAudio::linkPreviewTypeAudio()
  : audio_()
{}

linkPreviewTypeAudio::linkPreviewTypeAudio(object_ptr<audio> &&audio_)
  : audio_(std::move(audio_))
{}

const std::int32_t linkPreviewTypeAudio::ID;

object_ptr<LinkPreviewType> linkPreviewTypeAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeAudio> res = make_object<linkPreviewTypeAudio>();
  res->audio_ = jni::fetch_tl_object<audio>(env, jni::fetch_object(env, p, res->audio_fieldID));
  return std::move(res);
}

void linkPreviewTypeAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeAudio");
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeAudio").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Audio;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeBackground::Class;
jfieldID linkPreviewTypeBackground::document_fieldID;
jfieldID linkPreviewTypeBackground::background_type_fieldID;

linkPreviewTypeBackground::linkPreviewTypeBackground()
  : document_()
  , background_type_()
{}

linkPreviewTypeBackground::linkPreviewTypeBackground(object_ptr<document> &&document_, object_ptr<BackgroundType> &&background_type_)
  : document_(std::move(document_))
  , background_type_(std::move(background_type_))
{}

const std::int32_t linkPreviewTypeBackground::ID;

object_ptr<LinkPreviewType> linkPreviewTypeBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeBackground> res = make_object<linkPreviewTypeBackground>();
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->background_type_ = jni::fetch_tl_object<BackgroundType>(env, jni::fetch_object(env, p, res->background_type_fieldID));
  return std::move(res);
}

void linkPreviewTypeBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  if (background_type_ != nullptr) { jobject next; background_type_->store(env, next); if (next) { env->SetObjectField(s, background_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeBackground");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_object_field("backgroundType", static_cast<const BaseObject *>(background_type_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeBackground").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    background_type_fieldID = jni::get_field_id(env, Class, "backgroundType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeChannelBoost::Class;
jfieldID linkPreviewTypeChannelBoost::photo_fieldID;

linkPreviewTypeChannelBoost::linkPreviewTypeChannelBoost()
  : photo_()
{}

linkPreviewTypeChannelBoost::linkPreviewTypeChannelBoost(object_ptr<chatPhoto> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypeChannelBoost::ID;

object_ptr<LinkPreviewType> linkPreviewTypeChannelBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeChannelBoost> res = make_object<linkPreviewTypeChannelBoost>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypeChannelBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeChannelBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeChannelBoost");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeChannelBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeChannelBoost").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeChat::Class;
jfieldID linkPreviewTypeChat::type_fieldID;
jfieldID linkPreviewTypeChat::photo_fieldID;
jfieldID linkPreviewTypeChat::creates_join_request_fieldID;

linkPreviewTypeChat::linkPreviewTypeChat()
  : type_()
  , photo_()
  , creates_join_request_()
{}

linkPreviewTypeChat::linkPreviewTypeChat(object_ptr<InviteLinkChatType> &&type_, object_ptr<chatPhoto> &&photo_, bool creates_join_request_)
  : type_(std::move(type_))
  , photo_(std::move(photo_))
  , creates_join_request_(creates_join_request_)
{}

const std::int32_t linkPreviewTypeChat::ID;

object_ptr<LinkPreviewType> linkPreviewTypeChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeChat> res = make_object<linkPreviewTypeChat>();
  res->type_ = jni::fetch_tl_object<InviteLinkChatType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->creates_join_request_ = (env->GetBooleanField(p, res->creates_join_request_fieldID) != 0);
  return std::move(res);
}

void linkPreviewTypeChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, creates_join_request_fieldID, creates_join_request_);
}

void linkPreviewTypeChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeChat");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("createsJoinRequest", creates_join_request_);
    s.store_class_end();
  }
}

void linkPreviewTypeChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeChat").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InviteLinkChatType;").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    creates_join_request_fieldID = jni::get_field_id(env, Class, "createsJoinRequest", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeDocument::Class;
jfieldID linkPreviewTypeDocument::document_fieldID;

linkPreviewTypeDocument::linkPreviewTypeDocument()
  : document_()
{}

linkPreviewTypeDocument::linkPreviewTypeDocument(object_ptr<document> &&document_)
  : document_(std::move(document_))
{}

const std::int32_t linkPreviewTypeDocument::ID;

object_ptr<LinkPreviewType> linkPreviewTypeDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeDocument> res = make_object<linkPreviewTypeDocument>();
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  return std::move(res);
}

void linkPreviewTypeDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeDocument");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeDocument").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeEmbeddedAnimationPlayer::Class;
jfieldID linkPreviewTypeEmbeddedAnimationPlayer::url_fieldID;
jfieldID linkPreviewTypeEmbeddedAnimationPlayer::thumbnail_fieldID;
jfieldID linkPreviewTypeEmbeddedAnimationPlayer::duration_fieldID;
jfieldID linkPreviewTypeEmbeddedAnimationPlayer::width_fieldID;
jfieldID linkPreviewTypeEmbeddedAnimationPlayer::height_fieldID;

linkPreviewTypeEmbeddedAnimationPlayer::linkPreviewTypeEmbeddedAnimationPlayer()
  : url_()
  , thumbnail_()
  , duration_()
  , width_()
  , height_()
{}

linkPreviewTypeEmbeddedAnimationPlayer::linkPreviewTypeEmbeddedAnimationPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_)
  : url_(url_)
  , thumbnail_(std::move(thumbnail_))
  , duration_(duration_)
  , width_(width_)
  , height_(height_)
{}

const std::int32_t linkPreviewTypeEmbeddedAnimationPlayer::ID;

object_ptr<LinkPreviewType> linkPreviewTypeEmbeddedAnimationPlayer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeEmbeddedAnimationPlayer> res = make_object<linkPreviewTypeEmbeddedAnimationPlayer>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return std::move(res);
}

void linkPreviewTypeEmbeddedAnimationPlayer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void linkPreviewTypeEmbeddedAnimationPlayer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeEmbeddedAnimationPlayer");
    s.store_field("url", url_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void linkPreviewTypeEmbeddedAnimationPlayer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeEmbeddedAnimationPlayer").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeEmbeddedAudioPlayer::Class;
jfieldID linkPreviewTypeEmbeddedAudioPlayer::url_fieldID;
jfieldID linkPreviewTypeEmbeddedAudioPlayer::thumbnail_fieldID;
jfieldID linkPreviewTypeEmbeddedAudioPlayer::duration_fieldID;
jfieldID linkPreviewTypeEmbeddedAudioPlayer::width_fieldID;
jfieldID linkPreviewTypeEmbeddedAudioPlayer::height_fieldID;

linkPreviewTypeEmbeddedAudioPlayer::linkPreviewTypeEmbeddedAudioPlayer()
  : url_()
  , thumbnail_()
  , duration_()
  , width_()
  , height_()
{}

linkPreviewTypeEmbeddedAudioPlayer::linkPreviewTypeEmbeddedAudioPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_)
  : url_(url_)
  , thumbnail_(std::move(thumbnail_))
  , duration_(duration_)
  , width_(width_)
  , height_(height_)
{}

const std::int32_t linkPreviewTypeEmbeddedAudioPlayer::ID;

object_ptr<LinkPreviewType> linkPreviewTypeEmbeddedAudioPlayer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeEmbeddedAudioPlayer> res = make_object<linkPreviewTypeEmbeddedAudioPlayer>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return std::move(res);
}

void linkPreviewTypeEmbeddedAudioPlayer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void linkPreviewTypeEmbeddedAudioPlayer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeEmbeddedAudioPlayer");
    s.store_field("url", url_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void linkPreviewTypeEmbeddedAudioPlayer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeEmbeddedAudioPlayer").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeEmbeddedVideoPlayer::Class;
jfieldID linkPreviewTypeEmbeddedVideoPlayer::url_fieldID;
jfieldID linkPreviewTypeEmbeddedVideoPlayer::thumbnail_fieldID;
jfieldID linkPreviewTypeEmbeddedVideoPlayer::duration_fieldID;
jfieldID linkPreviewTypeEmbeddedVideoPlayer::width_fieldID;
jfieldID linkPreviewTypeEmbeddedVideoPlayer::height_fieldID;

linkPreviewTypeEmbeddedVideoPlayer::linkPreviewTypeEmbeddedVideoPlayer()
  : url_()
  , thumbnail_()
  , duration_()
  , width_()
  , height_()
{}

linkPreviewTypeEmbeddedVideoPlayer::linkPreviewTypeEmbeddedVideoPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_)
  : url_(url_)
  , thumbnail_(std::move(thumbnail_))
  , duration_(duration_)
  , width_(width_)
  , height_(height_)
{}

const std::int32_t linkPreviewTypeEmbeddedVideoPlayer::ID;

object_ptr<LinkPreviewType> linkPreviewTypeEmbeddedVideoPlayer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeEmbeddedVideoPlayer> res = make_object<linkPreviewTypeEmbeddedVideoPlayer>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return std::move(res);
}

void linkPreviewTypeEmbeddedVideoPlayer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void linkPreviewTypeEmbeddedVideoPlayer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeEmbeddedVideoPlayer");
    s.store_field("url", url_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void linkPreviewTypeEmbeddedVideoPlayer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeEmbeddedVideoPlayer").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeExternalAudio::Class;
jfieldID linkPreviewTypeExternalAudio::url_fieldID;
jfieldID linkPreviewTypeExternalAudio::mime_type_fieldID;
jfieldID linkPreviewTypeExternalAudio::duration_fieldID;

linkPreviewTypeExternalAudio::linkPreviewTypeExternalAudio()
  : url_()
  , mime_type_()
  , duration_()
{}

linkPreviewTypeExternalAudio::linkPreviewTypeExternalAudio(string const &url_, string const &mime_type_, int32 duration_)
  : url_(url_)
  , mime_type_(mime_type_)
  , duration_(duration_)
{}

const std::int32_t linkPreviewTypeExternalAudio::ID;

object_ptr<LinkPreviewType> linkPreviewTypeExternalAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeExternalAudio> res = make_object<linkPreviewTypeExternalAudio>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  return std::move(res);
}

void linkPreviewTypeExternalAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, duration_fieldID, duration_);
}

void linkPreviewTypeExternalAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeExternalAudio");
    s.store_field("url", url_);
    s.store_field("mimeType", mime_type_);
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void linkPreviewTypeExternalAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeExternalAudio").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeExternalVideo::Class;
jfieldID linkPreviewTypeExternalVideo::url_fieldID;
jfieldID linkPreviewTypeExternalVideo::mime_type_fieldID;
jfieldID linkPreviewTypeExternalVideo::width_fieldID;
jfieldID linkPreviewTypeExternalVideo::height_fieldID;
jfieldID linkPreviewTypeExternalVideo::duration_fieldID;

linkPreviewTypeExternalVideo::linkPreviewTypeExternalVideo()
  : url_()
  , mime_type_()
  , width_()
  , height_()
  , duration_()
{}

linkPreviewTypeExternalVideo::linkPreviewTypeExternalVideo(string const &url_, string const &mime_type_, int32 width_, int32 height_, int32 duration_)
  : url_(url_)
  , mime_type_(mime_type_)
  , width_(width_)
  , height_(height_)
  , duration_(duration_)
{}

const std::int32_t linkPreviewTypeExternalVideo::ID;

object_ptr<LinkPreviewType> linkPreviewTypeExternalVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeExternalVideo> res = make_object<linkPreviewTypeExternalVideo>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  return std::move(res);
}

void linkPreviewTypeExternalVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  env->SetIntField(s, duration_fieldID, duration_);
}

void linkPreviewTypeExternalVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeExternalVideo");
    s.store_field("url", url_);
    s.store_field("mimeType", mime_type_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void linkPreviewTypeExternalVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeExternalVideo").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeGroupCall::Class;

linkPreviewTypeGroupCall::linkPreviewTypeGroupCall() {
}

const std::int32_t linkPreviewTypeGroupCall::ID;

object_ptr<LinkPreviewType> linkPreviewTypeGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypeGroupCall>();
}

void linkPreviewTypeGroupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypeGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeGroupCall");
    s.store_class_end();
  }
}

void linkPreviewTypeGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeGroupCall").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeInvoice::Class;

linkPreviewTypeInvoice::linkPreviewTypeInvoice() {
}

const std::int32_t linkPreviewTypeInvoice::ID;

object_ptr<LinkPreviewType> linkPreviewTypeInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypeInvoice>();
}

void linkPreviewTypeInvoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypeInvoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeInvoice");
    s.store_class_end();
  }
}

void linkPreviewTypeInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeInvoice").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeMessage::Class;

linkPreviewTypeMessage::linkPreviewTypeMessage() {
}

const std::int32_t linkPreviewTypeMessage::ID;

object_ptr<LinkPreviewType> linkPreviewTypeMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypeMessage>();
}

void linkPreviewTypeMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypeMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeMessage");
    s.store_class_end();
  }
}

void linkPreviewTypeMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeMessage").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypePhoto::Class;
jfieldID linkPreviewTypePhoto::photo_fieldID;

linkPreviewTypePhoto::linkPreviewTypePhoto()
  : photo_()
{}

linkPreviewTypePhoto::linkPreviewTypePhoto(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypePhoto::ID;

object_ptr<LinkPreviewType> linkPreviewTypePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypePhoto> res = make_object<linkPreviewTypePhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypePremiumGiftCode::Class;

linkPreviewTypePremiumGiftCode::linkPreviewTypePremiumGiftCode() {
}

const std::int32_t linkPreviewTypePremiumGiftCode::ID;

object_ptr<LinkPreviewType> linkPreviewTypePremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypePremiumGiftCode>();
}

void linkPreviewTypePremiumGiftCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypePremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypePremiumGiftCode");
    s.store_class_end();
  }
}

void linkPreviewTypePremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypePremiumGiftCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeShareableChatFolder::Class;

linkPreviewTypeShareableChatFolder::linkPreviewTypeShareableChatFolder() {
}

const std::int32_t linkPreviewTypeShareableChatFolder::ID;

object_ptr<LinkPreviewType> linkPreviewTypeShareableChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypeShareableChatFolder>();
}

void linkPreviewTypeShareableChatFolder::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypeShareableChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeShareableChatFolder");
    s.store_class_end();
  }
}

void linkPreviewTypeShareableChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeShareableChatFolder").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeSticker::Class;
jfieldID linkPreviewTypeSticker::sticker_fieldID;

linkPreviewTypeSticker::linkPreviewTypeSticker()
  : sticker_()
{}

linkPreviewTypeSticker::linkPreviewTypeSticker(object_ptr<sticker> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t linkPreviewTypeSticker::ID;

object_ptr<LinkPreviewType> linkPreviewTypeSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeSticker> res = make_object<linkPreviewTypeSticker>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void linkPreviewTypeSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeStickerSet::Class;
jfieldID linkPreviewTypeStickerSet::stickers_fieldID;

linkPreviewTypeStickerSet::linkPreviewTypeStickerSet()
  : stickers_()
{}

linkPreviewTypeStickerSet::linkPreviewTypeStickerSet(array<object_ptr<sticker>> &&stickers_)
  : stickers_(std::move(stickers_))
{}

const std::int32_t linkPreviewTypeStickerSet::ID;

object_ptr<LinkPreviewType> linkPreviewTypeStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeStickerSet> res = make_object<linkPreviewTypeStickerSet>();
  res->stickers_ = jni::FetchVector<sticker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stickers_fieldID));
  return std::move(res);
}

void linkPreviewTypeStickerSet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, stickers_); if (arr_tmp_) { env->SetObjectField(s, stickers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void linkPreviewTypeStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeStickerSet");
    { s.store_vector_begin("stickers", stickers_.size()); for (const auto &_value : stickers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void linkPreviewTypeStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeStickerSet").c_str());
    stickers_fieldID = jni::get_field_id(env, Class, "stickers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeStory::Class;
jfieldID linkPreviewTypeStory::story_poster_chat_id_fieldID;
jfieldID linkPreviewTypeStory::story_id_fieldID;

linkPreviewTypeStory::linkPreviewTypeStory()
  : story_poster_chat_id_()
  , story_id_()
{}

linkPreviewTypeStory::linkPreviewTypeStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t linkPreviewTypeStory::ID;

object_ptr<LinkPreviewType> linkPreviewTypeStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeStory> res = make_object<linkPreviewTypeStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void linkPreviewTypeStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void linkPreviewTypeStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void linkPreviewTypeStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeSupergroupBoost::Class;
jfieldID linkPreviewTypeSupergroupBoost::photo_fieldID;

linkPreviewTypeSupergroupBoost::linkPreviewTypeSupergroupBoost()
  : photo_()
{}

linkPreviewTypeSupergroupBoost::linkPreviewTypeSupergroupBoost(object_ptr<chatPhoto> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypeSupergroupBoost::ID;

object_ptr<LinkPreviewType> linkPreviewTypeSupergroupBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeSupergroupBoost> res = make_object<linkPreviewTypeSupergroupBoost>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypeSupergroupBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeSupergroupBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeSupergroupBoost");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeSupergroupBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeSupergroupBoost").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeTheme::Class;
jfieldID linkPreviewTypeTheme::documents_fieldID;
jfieldID linkPreviewTypeTheme::settings_fieldID;

linkPreviewTypeTheme::linkPreviewTypeTheme()
  : documents_()
  , settings_()
{}

linkPreviewTypeTheme::linkPreviewTypeTheme(array<object_ptr<document>> &&documents_, object_ptr<themeSettings> &&settings_)
  : documents_(std::move(documents_))
  , settings_(std::move(settings_))
{}

const std::int32_t linkPreviewTypeTheme::ID;

object_ptr<LinkPreviewType> linkPreviewTypeTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeTheme> res = make_object<linkPreviewTypeTheme>();
  res->documents_ = jni::FetchVector<document>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->documents_fieldID));
  res->settings_ = jni::fetch_tl_object<themeSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return std::move(res);
}

void linkPreviewTypeTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, documents_); if (arr_tmp_) { env->SetObjectField(s, documents_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (settings_ != nullptr) { jobject next; settings_->store(env, next); if (next) { env->SetObjectField(s, settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeTheme");
    { s.store_vector_begin("documents", documents_.size()); for (const auto &_value : documents_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeTheme").c_str());
    documents_fieldID = jni::get_field_id(env, Class, "documents", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeUnsupported::Class;

linkPreviewTypeUnsupported::linkPreviewTypeUnsupported() {
}

const std::int32_t linkPreviewTypeUnsupported::ID;

object_ptr<LinkPreviewType> linkPreviewTypeUnsupported::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<linkPreviewTypeUnsupported>();
}

void linkPreviewTypeUnsupported::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void linkPreviewTypeUnsupported::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeUnsupported");
    s.store_class_end();
  }
}

void linkPreviewTypeUnsupported::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeUnsupported").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeUpgradedGift::Class;
jfieldID linkPreviewTypeUpgradedGift::gift_fieldID;

linkPreviewTypeUpgradedGift::linkPreviewTypeUpgradedGift()
  : gift_()
{}

linkPreviewTypeUpgradedGift::linkPreviewTypeUpgradedGift(object_ptr<upgradedGift> &&gift_)
  : gift_(std::move(gift_))
{}

const std::int32_t linkPreviewTypeUpgradedGift::ID;

object_ptr<LinkPreviewType> linkPreviewTypeUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeUpgradedGift> res = make_object<linkPreviewTypeUpgradedGift>();
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void linkPreviewTypeUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeUpgradedGift");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeUpgradedGift").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeUser::Class;
jfieldID linkPreviewTypeUser::photo_fieldID;
jfieldID linkPreviewTypeUser::is_bot_fieldID;

linkPreviewTypeUser::linkPreviewTypeUser()
  : photo_()
  , is_bot_()
{}

linkPreviewTypeUser::linkPreviewTypeUser(object_ptr<chatPhoto> &&photo_, bool is_bot_)
  : photo_(std::move(photo_))
  , is_bot_(is_bot_)
{}

const std::int32_t linkPreviewTypeUser::ID;

object_ptr<LinkPreviewType> linkPreviewTypeUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeUser> res = make_object<linkPreviewTypeUser>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->is_bot_ = (env->GetBooleanField(p, res->is_bot_fieldID) != 0);
  return std::move(res);
}

void linkPreviewTypeUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_bot_fieldID, is_bot_);
}

void linkPreviewTypeUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeUser");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("isBot", is_bot_);
    s.store_class_end();
  }
}

void linkPreviewTypeUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeUser").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    is_bot_fieldID = jni::get_field_id(env, Class, "isBot", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeVideo::Class;
jfieldID linkPreviewTypeVideo::video_fieldID;
jfieldID linkPreviewTypeVideo::cover_fieldID;
jfieldID linkPreviewTypeVideo::start_timestamp_fieldID;

linkPreviewTypeVideo::linkPreviewTypeVideo()
  : video_()
  , cover_()
  , start_timestamp_()
{}

linkPreviewTypeVideo::linkPreviewTypeVideo(object_ptr<video> &&video_, object_ptr<photo> &&cover_, int32 start_timestamp_)
  : video_(std::move(video_))
  , cover_(std::move(cover_))
  , start_timestamp_(start_timestamp_)
{}

const std::int32_t linkPreviewTypeVideo::ID;

object_ptr<LinkPreviewType> linkPreviewTypeVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeVideo> res = make_object<linkPreviewTypeVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->cover_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->cover_fieldID));
  res->start_timestamp_ = env->GetIntField(p, res->start_timestamp_fieldID);
  return std::move(res);
}

void linkPreviewTypeVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_timestamp_fieldID, start_timestamp_);
}

void linkPreviewTypeVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_field("startTimestamp", start_timestamp_);
    s.store_class_end();
  }
}

void linkPreviewTypeVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    start_timestamp_fieldID = jni::get_field_id(env, Class, "startTimestamp", "I");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeVideoChat::Class;
jfieldID linkPreviewTypeVideoChat::photo_fieldID;
jfieldID linkPreviewTypeVideoChat::is_live_stream_fieldID;

linkPreviewTypeVideoChat::linkPreviewTypeVideoChat()
  : photo_()
  , is_live_stream_()
{}

linkPreviewTypeVideoChat::linkPreviewTypeVideoChat(object_ptr<chatPhoto> &&photo_, bool is_live_stream_)
  : photo_(std::move(photo_))
  , is_live_stream_(is_live_stream_)
{}

const std::int32_t linkPreviewTypeVideoChat::ID;

object_ptr<LinkPreviewType> linkPreviewTypeVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeVideoChat> res = make_object<linkPreviewTypeVideoChat>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->is_live_stream_ = (env->GetBooleanField(p, res->is_live_stream_fieldID) != 0);
  return std::move(res);
}

void linkPreviewTypeVideoChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_live_stream_fieldID, is_live_stream_);
}

void linkPreviewTypeVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeVideoChat");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("isLiveStream", is_live_stream_);
    s.store_class_end();
  }
}

void linkPreviewTypeVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeVideoChat").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    is_live_stream_fieldID = jni::get_field_id(env, Class, "isLiveStream", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeVideoNote::Class;
jfieldID linkPreviewTypeVideoNote::video_note_fieldID;

linkPreviewTypeVideoNote::linkPreviewTypeVideoNote()
  : video_note_()
{}

linkPreviewTypeVideoNote::linkPreviewTypeVideoNote(object_ptr<videoNote> &&video_note_)
  : video_note_(std::move(video_note_))
{}

const std::int32_t linkPreviewTypeVideoNote::ID;

object_ptr<LinkPreviewType> linkPreviewTypeVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeVideoNote> res = make_object<linkPreviewTypeVideoNote>();
  res->video_note_ = jni::fetch_tl_object<videoNote>(env, jni::fetch_object(env, p, res->video_note_fieldID));
  return std::move(res);
}

void linkPreviewTypeVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_note_ != nullptr) { jobject next; video_note_->store(env, next); if (next) { env->SetObjectField(s, video_note_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeVideoNote");
    s.store_object_field("videoNote", static_cast<const BaseObject *>(video_note_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeVideoNote").c_str());
    video_note_fieldID = jni::get_field_id(env, Class, "videoNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VideoNote;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeVoiceNote::Class;
jfieldID linkPreviewTypeVoiceNote::voice_note_fieldID;

linkPreviewTypeVoiceNote::linkPreviewTypeVoiceNote()
  : voice_note_()
{}

linkPreviewTypeVoiceNote::linkPreviewTypeVoiceNote(object_ptr<voiceNote> &&voice_note_)
  : voice_note_(std::move(voice_note_))
{}

const std::int32_t linkPreviewTypeVoiceNote::ID;

object_ptr<LinkPreviewType> linkPreviewTypeVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeVoiceNote> res = make_object<linkPreviewTypeVoiceNote>();
  res->voice_note_ = jni::fetch_tl_object<voiceNote>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  return std::move(res);
}

void linkPreviewTypeVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeVoiceNote");
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeVoiceNote").c_str());
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VoiceNote;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass linkPreviewTypeWebApp::Class;
jfieldID linkPreviewTypeWebApp::photo_fieldID;

linkPreviewTypeWebApp::linkPreviewTypeWebApp()
  : photo_()
{}

linkPreviewTypeWebApp::linkPreviewTypeWebApp(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t linkPreviewTypeWebApp::ID;

object_ptr<LinkPreviewType> linkPreviewTypeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<linkPreviewTypeWebApp> res = make_object<linkPreviewTypeWebApp>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void linkPreviewTypeWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void linkPreviewTypeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LinkPreviewTypeWebApp");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void linkPreviewTypeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LinkPreviewTypeWebApp").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass localFile::Class;
jfieldID localFile::path_fieldID;
jfieldID localFile::can_be_downloaded_fieldID;
jfieldID localFile::can_be_deleted_fieldID;
jfieldID localFile::is_downloading_active_fieldID;
jfieldID localFile::is_downloading_completed_fieldID;
jfieldID localFile::download_offset_fieldID;
jfieldID localFile::downloaded_prefix_size_fieldID;
jfieldID localFile::downloaded_size_fieldID;

localFile::localFile()
  : path_()
  , can_be_downloaded_()
  , can_be_deleted_()
  , is_downloading_active_()
  , is_downloading_completed_()
  , download_offset_()
  , downloaded_prefix_size_()
  , downloaded_size_()
{}

localFile::localFile(string const &path_, bool can_be_downloaded_, bool can_be_deleted_, bool is_downloading_active_, bool is_downloading_completed_, int53 download_offset_, int53 downloaded_prefix_size_, int53 downloaded_size_)
  : path_(path_)
  , can_be_downloaded_(can_be_downloaded_)
  , can_be_deleted_(can_be_deleted_)
  , is_downloading_active_(is_downloading_active_)
  , is_downloading_completed_(is_downloading_completed_)
  , download_offset_(download_offset_)
  , downloaded_prefix_size_(downloaded_prefix_size_)
  , downloaded_size_(downloaded_size_)
{}

const std::int32_t localFile::ID;

object_ptr<localFile> localFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<localFile> res = make_object<localFile>();
  res->path_ = jni::fetch_string(env, p, res->path_fieldID);
  res->can_be_downloaded_ = (env->GetBooleanField(p, res->can_be_downloaded_fieldID) != 0);
  res->can_be_deleted_ = (env->GetBooleanField(p, res->can_be_deleted_fieldID) != 0);
  res->is_downloading_active_ = (env->GetBooleanField(p, res->is_downloading_active_fieldID) != 0);
  res->is_downloading_completed_ = (env->GetBooleanField(p, res->is_downloading_completed_fieldID) != 0);
  res->download_offset_ = env->GetLongField(p, res->download_offset_fieldID);
  res->downloaded_prefix_size_ = env->GetLongField(p, res->downloaded_prefix_size_fieldID);
  res->downloaded_size_ = env->GetLongField(p, res->downloaded_size_fieldID);
  return res;
}

void localFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, path_); if (nextString) { env->SetObjectField(s, path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, can_be_downloaded_fieldID, can_be_downloaded_);
  env->SetBooleanField(s, can_be_deleted_fieldID, can_be_deleted_);
  env->SetBooleanField(s, is_downloading_active_fieldID, is_downloading_active_);
  env->SetBooleanField(s, is_downloading_completed_fieldID, is_downloading_completed_);
  env->SetLongField(s, download_offset_fieldID, download_offset_);
  env->SetLongField(s, downloaded_prefix_size_fieldID, downloaded_prefix_size_);
  env->SetLongField(s, downloaded_size_fieldID, downloaded_size_);
}

void localFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LocalFile");
    s.store_field("path", path_);
    s.store_field("canBeDownloaded", can_be_downloaded_);
    s.store_field("canBeDeleted", can_be_deleted_);
    s.store_field("isDownloadingActive", is_downloading_active_);
    s.store_field("isDownloadingCompleted", is_downloading_completed_);
    s.store_field("downloadOffset", download_offset_);
    s.store_field("downloadedPrefixSize", downloaded_prefix_size_);
    s.store_field("downloadedSize", downloaded_size_);
    s.store_class_end();
  }
}

void localFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LocalFile").c_str());
    path_fieldID = jni::get_field_id(env, Class, "path", "Ljava/lang/String;");
    can_be_downloaded_fieldID = jni::get_field_id(env, Class, "canBeDownloaded", "Z");
    can_be_deleted_fieldID = jni::get_field_id(env, Class, "canBeDeleted", "Z");
    is_downloading_active_fieldID = jni::get_field_id(env, Class, "isDownloadingActive", "Z");
    is_downloading_completed_fieldID = jni::get_field_id(env, Class, "isDownloadingCompleted", "Z");
    download_offset_fieldID = jni::get_field_id(env, Class, "downloadOffset", "J");
    downloaded_prefix_size_fieldID = jni::get_field_id(env, Class, "downloadedPrefixSize", "J");
    downloaded_size_fieldID = jni::get_field_id(env, Class, "downloadedSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass localizationTargetInfo::Class;
jfieldID localizationTargetInfo::language_packs_fieldID;

localizationTargetInfo::localizationTargetInfo()
  : language_packs_()
{}

localizationTargetInfo::localizationTargetInfo(array<object_ptr<languagePackInfo>> &&language_packs_)
  : language_packs_(std::move(language_packs_))
{}

const std::int32_t localizationTargetInfo::ID;

object_ptr<localizationTargetInfo> localizationTargetInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<localizationTargetInfo> res = make_object<localizationTargetInfo>();
  res->language_packs_ = jni::FetchVector<languagePackInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->language_packs_fieldID));
  return res;
}

void localizationTargetInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, language_packs_); if (arr_tmp_) { env->SetObjectField(s, language_packs_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void localizationTargetInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LocalizationTargetInfo");
    { s.store_vector_begin("languagePacks", language_packs_.size()); for (const auto &_value : language_packs_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void localizationTargetInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LocalizationTargetInfo").c_str());
    language_packs_fieldID = jni::get_field_id(env, Class, "languagePacks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LanguagePackInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass location::Class;
jfieldID location::latitude_fieldID;
jfieldID location::longitude_fieldID;
jfieldID location::horizontal_accuracy_fieldID;

location::location()
  : latitude_()
  , longitude_()
  , horizontal_accuracy_()
{}

location::location(double latitude_, double longitude_, double horizontal_accuracy_)
  : latitude_(latitude_)
  , longitude_(longitude_)
  , horizontal_accuracy_(horizontal_accuracy_)
{}

const std::int32_t location::ID;

object_ptr<location> location::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<location> res = make_object<location>();
  res->latitude_ = env->GetDoubleField(p, res->latitude_fieldID);
  res->longitude_ = env->GetDoubleField(p, res->longitude_fieldID);
  res->horizontal_accuracy_ = env->GetDoubleField(p, res->horizontal_accuracy_fieldID);
  return res;
}

void location::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, latitude_fieldID, latitude_);
  env->SetDoubleField(s, longitude_fieldID, longitude_);
  env->SetDoubleField(s, horizontal_accuracy_fieldID, horizontal_accuracy_);
}

void location::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Location");
    s.store_field("latitude", latitude_);
    s.store_field("longitude", longitude_);
    s.store_field("horizontalAccuracy", horizontal_accuracy_);
    s.store_class_end();
  }
}

void location::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Location").c_str());
    latitude_fieldID = jni::get_field_id(env, Class, "latitude", "D");
    longitude_fieldID = jni::get_field_id(env, Class, "longitude", "D");
    horizontal_accuracy_fieldID = jni::get_field_id(env, Class, "horizontalAccuracy", "D");
    return true;
  }();
  (void)is_inited;
}

jclass locationAddress::Class;
jfieldID locationAddress::country_code_fieldID;
jfieldID locationAddress::state_fieldID;
jfieldID locationAddress::city_fieldID;
jfieldID locationAddress::street_fieldID;

locationAddress::locationAddress()
  : country_code_()
  , state_()
  , city_()
  , street_()
{}

locationAddress::locationAddress(string const &country_code_, string const &state_, string const &city_, string const &street_)
  : country_code_(country_code_)
  , state_(state_)
  , city_(city_)
  , street_(street_)
{}

const std::int32_t locationAddress::ID;

object_ptr<locationAddress> locationAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<locationAddress> res = make_object<locationAddress>();
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  res->state_ = jni::fetch_string(env, p, res->state_fieldID);
  res->city_ = jni::fetch_string(env, p, res->city_fieldID);
  res->street_ = jni::fetch_string(env, p, res->street_fieldID);
  return res;
}

void locationAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, country_code_); if (nextString) { env->SetObjectField(s, country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, state_); if (nextString) { env->SetObjectField(s, state_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, city_); if (nextString) { env->SetObjectField(s, city_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, street_); if (nextString) { env->SetObjectField(s, street_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void locationAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LocationAddress");
    s.store_field("countryCode", country_code_);
    s.store_field("state", state_);
    s.store_field("city", city_);
    s.store_field("street", street_);
    s.store_class_end();
  }
}

void locationAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LocationAddress").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    state_fieldID = jni::get_field_id(env, Class, "state", "Ljava/lang/String;");
    city_fieldID = jni::get_field_id(env, Class, "city", "Ljava/lang/String;");
    street_fieldID = jni::get_field_id(env, Class, "street", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass LogStream::Class;

object_ptr<LogStream> LogStream::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case logStreamDefault::ID:
      return logStreamDefault::fetch(env, p);
    case logStreamFile::ID:
      return logStreamFile::fetch(env, p);
    case logStreamEmpty::ID:
      return logStreamEmpty::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void LogStream::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogStream").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass logStreamDefault::Class;

logStreamDefault::logStreamDefault() {
}

const std::int32_t logStreamDefault::ID;

object_ptr<LogStream> logStreamDefault::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<logStreamDefault>();
}

void logStreamDefault::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void logStreamDefault::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogStreamDefault");
    s.store_class_end();
  }
}

void logStreamDefault::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogStreamDefault").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass logStreamFile::Class;
jfieldID logStreamFile::path_fieldID;
jfieldID logStreamFile::max_file_size_fieldID;
jfieldID logStreamFile::redirect_stderr_fieldID;

logStreamFile::logStreamFile()
  : path_()
  , max_file_size_()
  , redirect_stderr_()
{}

logStreamFile::logStreamFile(string const &path_, int53 max_file_size_, bool redirect_stderr_)
  : path_(path_)
  , max_file_size_(max_file_size_)
  , redirect_stderr_(redirect_stderr_)
{}

const std::int32_t logStreamFile::ID;

object_ptr<LogStream> logStreamFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<logStreamFile> res = make_object<logStreamFile>();
  res->path_ = jni::fetch_string(env, p, res->path_fieldID);
  res->max_file_size_ = env->GetLongField(p, res->max_file_size_fieldID);
  res->redirect_stderr_ = (env->GetBooleanField(p, res->redirect_stderr_fieldID) != 0);
  return std::move(res);
}

void logStreamFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, path_); if (nextString) { env->SetObjectField(s, path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, max_file_size_fieldID, max_file_size_);
  env->SetBooleanField(s, redirect_stderr_fieldID, redirect_stderr_);
}

void logStreamFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogStreamFile");
    s.store_field("path", path_);
    s.store_field("maxFileSize", max_file_size_);
    s.store_field("redirectStderr", redirect_stderr_);
    s.store_class_end();
  }
}

void logStreamFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogStreamFile").c_str());
    path_fieldID = jni::get_field_id(env, Class, "path", "Ljava/lang/String;");
    max_file_size_fieldID = jni::get_field_id(env, Class, "maxFileSize", "J");
    redirect_stderr_fieldID = jni::get_field_id(env, Class, "redirectStderr", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass logStreamEmpty::Class;

logStreamEmpty::logStreamEmpty() {
}

const std::int32_t logStreamEmpty::ID;

object_ptr<LogStream> logStreamEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<logStreamEmpty>();
}

void logStreamEmpty::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void logStreamEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogStreamEmpty");
    s.store_class_end();
  }
}

void logStreamEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogStreamEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass logTags::Class;
jfieldID logTags::tags_fieldID;

logTags::logTags()
  : tags_()
{}

logTags::logTags(array<string> &&tags_)
  : tags_(std::move(tags_))
{}

const std::int32_t logTags::ID;

object_ptr<logTags> logTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<logTags> res = make_object<logTags>();
  res->tags_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->tags_fieldID));
  return res;
}

void logTags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, tags_); if (arr_tmp_) { env->SetObjectField(s, tags_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void logTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogTags");
    { s.store_vector_begin("tags", tags_.size()); for (const auto &_value : tags_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void logTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogTags").c_str());
    tags_fieldID = jni::get_field_id(env, Class, "tags", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass logVerbosityLevel::Class;
jfieldID logVerbosityLevel::verbosity_level_fieldID;

logVerbosityLevel::logVerbosityLevel()
  : verbosity_level_()
{}

logVerbosityLevel::logVerbosityLevel(int32 verbosity_level_)
  : verbosity_level_(verbosity_level_)
{}

const std::int32_t logVerbosityLevel::ID;

object_ptr<logVerbosityLevel> logVerbosityLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<logVerbosityLevel> res = make_object<logVerbosityLevel>();
  res->verbosity_level_ = env->GetIntField(p, res->verbosity_level_fieldID);
  return res;
}

void logVerbosityLevel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, verbosity_level_fieldID, verbosity_level_);
}

void logVerbosityLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogVerbosityLevel");
    s.store_field("verbosityLevel", verbosity_level_);
    s.store_class_end();
  }
}

void logVerbosityLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogVerbosityLevel").c_str());
    verbosity_level_fieldID = jni::get_field_id(env, Class, "verbosityLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass LoginUrlInfo::Class;

object_ptr<LoginUrlInfo> LoginUrlInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case loginUrlInfoOpen::ID:
      return loginUrlInfoOpen::fetch(env, p);
    case loginUrlInfoRequestConfirmation::ID:
      return loginUrlInfoRequestConfirmation::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void LoginUrlInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoginUrlInfo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass loginUrlInfoOpen::Class;
jfieldID loginUrlInfoOpen::url_fieldID;
jfieldID loginUrlInfoOpen::skip_confirmation_fieldID;

loginUrlInfoOpen::loginUrlInfoOpen()
  : url_()
  , skip_confirmation_()
{}

loginUrlInfoOpen::loginUrlInfoOpen(string const &url_, bool skip_confirmation_)
  : url_(url_)
  , skip_confirmation_(skip_confirmation_)
{}

const std::int32_t loginUrlInfoOpen::ID;

object_ptr<LoginUrlInfo> loginUrlInfoOpen::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loginUrlInfoOpen> res = make_object<loginUrlInfoOpen>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->skip_confirmation_ = (env->GetBooleanField(p, res->skip_confirmation_fieldID) != 0);
  return std::move(res);
}

void loginUrlInfoOpen::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, skip_confirmation_fieldID, skip_confirmation_);
}

void loginUrlInfoOpen::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoginUrlInfoOpen");
    s.store_field("url", url_);
    s.store_field("skipConfirmation", skip_confirmation_);
    s.store_class_end();
  }
}

void loginUrlInfoOpen::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoginUrlInfoOpen").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    skip_confirmation_fieldID = jni::get_field_id(env, Class, "skipConfirmation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass loginUrlInfoRequestConfirmation::Class;
jfieldID loginUrlInfoRequestConfirmation::url_fieldID;
jfieldID loginUrlInfoRequestConfirmation::domain_fieldID;
jfieldID loginUrlInfoRequestConfirmation::bot_user_id_fieldID;
jfieldID loginUrlInfoRequestConfirmation::request_write_access_fieldID;

loginUrlInfoRequestConfirmation::loginUrlInfoRequestConfirmation()
  : url_()
  , domain_()
  , bot_user_id_()
  , request_write_access_()
{}

loginUrlInfoRequestConfirmation::loginUrlInfoRequestConfirmation(string const &url_, string const &domain_, int53 bot_user_id_, bool request_write_access_)
  : url_(url_)
  , domain_(domain_)
  , bot_user_id_(bot_user_id_)
  , request_write_access_(request_write_access_)
{}

const std::int32_t loginUrlInfoRequestConfirmation::ID;

object_ptr<LoginUrlInfo> loginUrlInfoRequestConfirmation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loginUrlInfoRequestConfirmation> res = make_object<loginUrlInfoRequestConfirmation>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->domain_ = jni::fetch_string(env, p, res->domain_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->request_write_access_ = (env->GetBooleanField(p, res->request_write_access_fieldID) != 0);
  return std::move(res);
}

void loginUrlInfoRequestConfirmation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, domain_); if (nextString) { env->SetObjectField(s, domain_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, bot_user_id_fieldID, bot_user_id_);
  env->SetBooleanField(s, request_write_access_fieldID, request_write_access_);
}

void loginUrlInfoRequestConfirmation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoginUrlInfoRequestConfirmation");
    s.store_field("url", url_);
    s.store_field("domain", domain_);
    s.store_field("botUserId", bot_user_id_);
    s.store_field("requestWriteAccess", request_write_access_);
    s.store_class_end();
  }
}

void loginUrlInfoRequestConfirmation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoginUrlInfoRequestConfirmation").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    domain_fieldID = jni::get_field_id(env, Class, "domain", "Ljava/lang/String;");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    request_write_access_fieldID = jni::get_field_id(env, Class, "requestWriteAccess", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass mainWebApp::Class;
jfieldID mainWebApp::url_fieldID;
jfieldID mainWebApp::mode_fieldID;

mainWebApp::mainWebApp()
  : url_()
  , mode_()
{}

mainWebApp::mainWebApp(string const &url_, object_ptr<WebAppOpenMode> &&mode_)
  : url_(url_)
  , mode_(std::move(mode_))
{}

const std::int32_t mainWebApp::ID;

object_ptr<mainWebApp> mainWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<mainWebApp> res = make_object<mainWebApp>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->mode_ = jni::fetch_tl_object<WebAppOpenMode>(env, jni::fetch_object(env, p, res->mode_fieldID));
  return res;
}

void mainWebApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (mode_ != nullptr) { jobject next; mode_->store(env, next); if (next) { env->SetObjectField(s, mode_fieldID, next); env->DeleteLocalRef(next); } }
}

void mainWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MainWebApp");
    s.store_field("url", url_);
    s.store_object_field("mode", static_cast<const BaseObject *>(mode_.get()));
    s.store_class_end();
  }
}

void mainWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MainWebApp").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    mode_fieldID = jni::get_field_id(env, Class, "mode", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenMode;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass MaskPoint::Class;

object_ptr<MaskPoint> MaskPoint::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case maskPointForehead::ID:
      return maskPointForehead::fetch(env, p);
    case maskPointEyes::ID:
      return maskPointEyes::fetch(env, p);
    case maskPointMouth::ID:
      return maskPointMouth::fetch(env, p);
    case maskPointChin::ID:
      return maskPointChin::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MaskPoint::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPoint").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass maskPointForehead::Class;

maskPointForehead::maskPointForehead() {
}

const std::int32_t maskPointForehead::ID;

object_ptr<MaskPoint> maskPointForehead::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<maskPointForehead>();
}

void maskPointForehead::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void maskPointForehead::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MaskPointForehead");
    s.store_class_end();
  }
}

void maskPointForehead::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPointForehead").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass maskPointEyes::Class;

maskPointEyes::maskPointEyes() {
}

const std::int32_t maskPointEyes::ID;

object_ptr<MaskPoint> maskPointEyes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<maskPointEyes>();
}

void maskPointEyes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void maskPointEyes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MaskPointEyes");
    s.store_class_end();
  }
}

void maskPointEyes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPointEyes").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass maskPointMouth::Class;

maskPointMouth::maskPointMouth() {
}

const std::int32_t maskPointMouth::ID;

object_ptr<MaskPoint> maskPointMouth::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<maskPointMouth>();
}

void maskPointMouth::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void maskPointMouth::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MaskPointMouth");
    s.store_class_end();
  }
}

void maskPointMouth::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPointMouth").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass maskPointChin::Class;

maskPointChin::maskPointChin() {
}

const std::int32_t maskPointChin::ID;

object_ptr<MaskPoint> maskPointChin::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<maskPointChin>();
}

void maskPointChin::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void maskPointChin::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MaskPointChin");
    s.store_class_end();
  }
}

void maskPointChin::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPointChin").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass maskPosition::Class;
jfieldID maskPosition::point_fieldID;
jfieldID maskPosition::x_shift_fieldID;
jfieldID maskPosition::y_shift_fieldID;
jfieldID maskPosition::scale_fieldID;

maskPosition::maskPosition()
  : point_()
  , x_shift_()
  , y_shift_()
  , scale_()
{}

maskPosition::maskPosition(object_ptr<MaskPoint> &&point_, double x_shift_, double y_shift_, double scale_)
  : point_(std::move(point_))
  , x_shift_(x_shift_)
  , y_shift_(y_shift_)
  , scale_(scale_)
{}

const std::int32_t maskPosition::ID;

object_ptr<maskPosition> maskPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<maskPosition> res = make_object<maskPosition>();
  res->point_ = jni::fetch_tl_object<MaskPoint>(env, jni::fetch_object(env, p, res->point_fieldID));
  res->x_shift_ = env->GetDoubleField(p, res->x_shift_fieldID);
  res->y_shift_ = env->GetDoubleField(p, res->y_shift_fieldID);
  res->scale_ = env->GetDoubleField(p, res->scale_fieldID);
  return res;
}

void maskPosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (point_ != nullptr) { jobject next; point_->store(env, next); if (next) { env->SetObjectField(s, point_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, x_shift_fieldID, x_shift_);
  env->SetDoubleField(s, y_shift_fieldID, y_shift_);
  env->SetDoubleField(s, scale_fieldID, scale_);
}

void maskPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MaskPosition");
    s.store_object_field("point", static_cast<const BaseObject *>(point_.get()));
    s.store_field("xShift", x_shift_);
    s.store_field("yShift", y_shift_);
    s.store_field("scale", scale_);
    s.store_class_end();
  }
}

void maskPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MaskPosition").c_str());
    point_fieldID = jni::get_field_id(env, Class, "point", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MaskPoint;").c_str());
    x_shift_fieldID = jni::get_field_id(env, Class, "xShift", "D");
    y_shift_fieldID = jni::get_field_id(env, Class, "yShift", "D");
    scale_fieldID = jni::get_field_id(env, Class, "scale", "D");
    return true;
  }();
  (void)is_inited;
}

jclass message::Class;
jfieldID message::id_fieldID;
jfieldID message::sender_id_fieldID;
jfieldID message::chat_id_fieldID;
jfieldID message::sending_state_fieldID;
jfieldID message::scheduling_state_fieldID;
jfieldID message::is_outgoing_fieldID;
jfieldID message::is_pinned_fieldID;
jfieldID message::is_from_offline_fieldID;
jfieldID message::can_be_saved_fieldID;
jfieldID message::has_timestamped_media_fieldID;
jfieldID message::is_channel_post_fieldID;
jfieldID message::is_topic_message_fieldID;
jfieldID message::contains_unread_mention_fieldID;
jfieldID message::date_fieldID;
jfieldID message::edit_date_fieldID;
jfieldID message::forward_info_fieldID;
jfieldID message::import_info_fieldID;
jfieldID message::interaction_info_fieldID;
jfieldID message::unread_reactions_fieldID;
jfieldID message::fact_check_fieldID;
jfieldID message::reply_to_fieldID;
jfieldID message::message_thread_id_fieldID;
jfieldID message::saved_messages_topic_id_fieldID;
jfieldID message::self_destruct_type_fieldID;
jfieldID message::self_destruct_in_fieldID;
jfieldID message::auto_delete_in_fieldID;
jfieldID message::via_bot_user_id_fieldID;
jfieldID message::sender_business_bot_user_id_fieldID;
jfieldID message::sender_boost_count_fieldID;
jfieldID message::paid_message_star_count_fieldID;
jfieldID message::author_signature_fieldID;
jfieldID message::media_album_id_fieldID;
jfieldID message::effect_id_fieldID;
jfieldID message::has_sensitive_content_fieldID;
jfieldID message::restriction_reason_fieldID;
jfieldID message::content_fieldID;
jfieldID message::reply_markup_fieldID;

message::message()
  : id_()
  , sender_id_()
  , chat_id_()
  , sending_state_()
  , scheduling_state_()
  , is_outgoing_()
  , is_pinned_()
  , is_from_offline_()
  , can_be_saved_()
  , has_timestamped_media_()
  , is_channel_post_()
  , is_topic_message_()
  , contains_unread_mention_()
  , date_()
  , edit_date_()
  , forward_info_()
  , import_info_()
  , interaction_info_()
  , unread_reactions_()
  , fact_check_()
  , reply_to_()
  , message_thread_id_()
  , saved_messages_topic_id_()
  , self_destruct_type_()
  , self_destruct_in_()
  , auto_delete_in_()
  , via_bot_user_id_()
  , sender_business_bot_user_id_()
  , sender_boost_count_()
  , paid_message_star_count_()
  , author_signature_()
  , media_album_id_()
  , effect_id_()
  , has_sensitive_content_()
  , restriction_reason_()
  , content_()
  , reply_markup_()
{}

message::message(int53 id_, object_ptr<MessageSender> &&sender_id_, int53 chat_id_, object_ptr<MessageSendingState> &&sending_state_, object_ptr<MessageSchedulingState> &&scheduling_state_, bool is_outgoing_, bool is_pinned_, bool is_from_offline_, bool can_be_saved_, bool has_timestamped_media_, bool is_channel_post_, bool is_topic_message_, bool contains_unread_mention_, int32 date_, int32 edit_date_, object_ptr<messageForwardInfo> &&forward_info_, object_ptr<messageImportInfo> &&import_info_, object_ptr<messageInteractionInfo> &&interaction_info_, array<object_ptr<unreadReaction>> &&unread_reactions_, object_ptr<factCheck> &&fact_check_, object_ptr<MessageReplyTo> &&reply_to_, int53 message_thread_id_, int53 saved_messages_topic_id_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, double self_destruct_in_, double auto_delete_in_, int53 via_bot_user_id_, int53 sender_business_bot_user_id_, int32 sender_boost_count_, int53 paid_message_star_count_, string const &author_signature_, int64 media_album_id_, int64 effect_id_, bool has_sensitive_content_, string const &restriction_reason_, object_ptr<MessageContent> &&content_, object_ptr<ReplyMarkup> &&reply_markup_)
  : id_(id_)
  , sender_id_(std::move(sender_id_))
  , chat_id_(chat_id_)
  , sending_state_(std::move(sending_state_))
  , scheduling_state_(std::move(scheduling_state_))
  , is_outgoing_(is_outgoing_)
  , is_pinned_(is_pinned_)
  , is_from_offline_(is_from_offline_)
  , can_be_saved_(can_be_saved_)
  , has_timestamped_media_(has_timestamped_media_)
  , is_channel_post_(is_channel_post_)
  , is_topic_message_(is_topic_message_)
  , contains_unread_mention_(contains_unread_mention_)
  , date_(date_)
  , edit_date_(edit_date_)
  , forward_info_(std::move(forward_info_))
  , import_info_(std::move(import_info_))
  , interaction_info_(std::move(interaction_info_))
  , unread_reactions_(std::move(unread_reactions_))
  , fact_check_(std::move(fact_check_))
  , reply_to_(std::move(reply_to_))
  , message_thread_id_(message_thread_id_)
  , saved_messages_topic_id_(saved_messages_topic_id_)
  , self_destruct_type_(std::move(self_destruct_type_))
  , self_destruct_in_(self_destruct_in_)
  , auto_delete_in_(auto_delete_in_)
  , via_bot_user_id_(via_bot_user_id_)
  , sender_business_bot_user_id_(sender_business_bot_user_id_)
  , sender_boost_count_(sender_boost_count_)
  , paid_message_star_count_(paid_message_star_count_)
  , author_signature_(author_signature_)
  , media_album_id_(media_album_id_)
  , effect_id_(effect_id_)
  , has_sensitive_content_(has_sensitive_content_)
  , restriction_reason_(restriction_reason_)
  , content_(std::move(content_))
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t message::ID;

object_ptr<message> message::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<message> res = make_object<message>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->sending_state_ = jni::fetch_tl_object<MessageSendingState>(env, jni::fetch_object(env, p, res->sending_state_fieldID));
  res->scheduling_state_ = jni::fetch_tl_object<MessageSchedulingState>(env, jni::fetch_object(env, p, res->scheduling_state_fieldID));
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->is_from_offline_ = (env->GetBooleanField(p, res->is_from_offline_fieldID) != 0);
  res->can_be_saved_ = (env->GetBooleanField(p, res->can_be_saved_fieldID) != 0);
  res->has_timestamped_media_ = (env->GetBooleanField(p, res->has_timestamped_media_fieldID) != 0);
  res->is_channel_post_ = (env->GetBooleanField(p, res->is_channel_post_fieldID) != 0);
  res->is_topic_message_ = (env->GetBooleanField(p, res->is_topic_message_fieldID) != 0);
  res->contains_unread_mention_ = (env->GetBooleanField(p, res->contains_unread_mention_fieldID) != 0);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->edit_date_ = env->GetIntField(p, res->edit_date_fieldID);
  res->forward_info_ = jni::fetch_tl_object<messageForwardInfo>(env, jni::fetch_object(env, p, res->forward_info_fieldID));
  res->import_info_ = jni::fetch_tl_object<messageImportInfo>(env, jni::fetch_object(env, p, res->import_info_fieldID));
  res->interaction_info_ = jni::fetch_tl_object<messageInteractionInfo>(env, jni::fetch_object(env, p, res->interaction_info_fieldID));
  res->unread_reactions_ = jni::FetchVector<unreadReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->unread_reactions_fieldID));
  res->fact_check_ = jni::fetch_tl_object<factCheck>(env, jni::fetch_object(env, p, res->fact_check_fieldID));
  res->reply_to_ = jni::fetch_tl_object<MessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->self_destruct_type_ = jni::fetch_tl_object<MessageSelfDestructType>(env, jni::fetch_object(env, p, res->self_destruct_type_fieldID));
  res->self_destruct_in_ = env->GetDoubleField(p, res->self_destruct_in_fieldID);
  res->auto_delete_in_ = env->GetDoubleField(p, res->auto_delete_in_fieldID);
  res->via_bot_user_id_ = env->GetLongField(p, res->via_bot_user_id_fieldID);
  res->sender_business_bot_user_id_ = env->GetLongField(p, res->sender_business_bot_user_id_fieldID);
  res->sender_boost_count_ = env->GetIntField(p, res->sender_boost_count_fieldID);
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  res->author_signature_ = jni::fetch_string(env, p, res->author_signature_fieldID);
  res->media_album_id_ = env->GetLongField(p, res->media_album_id_fieldID);
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  res->has_sensitive_content_ = (env->GetBooleanField(p, res->has_sensitive_content_fieldID) != 0);
  res->restriction_reason_ = jni::fetch_string(env, p, res->restriction_reason_fieldID);
  res->content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void message::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (sending_state_ != nullptr) { jobject next; sending_state_->store(env, next); if (next) { env->SetObjectField(s, sending_state_fieldID, next); env->DeleteLocalRef(next); } }
  if (scheduling_state_ != nullptr) { jobject next; scheduling_state_->store(env, next); if (next) { env->SetObjectField(s, scheduling_state_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  env->SetBooleanField(s, is_from_offline_fieldID, is_from_offline_);
  env->SetBooleanField(s, can_be_saved_fieldID, can_be_saved_);
  env->SetBooleanField(s, has_timestamped_media_fieldID, has_timestamped_media_);
  env->SetBooleanField(s, is_channel_post_fieldID, is_channel_post_);
  env->SetBooleanField(s, is_topic_message_fieldID, is_topic_message_);
  env->SetBooleanField(s, contains_unread_mention_fieldID, contains_unread_mention_);
  env->SetIntField(s, date_fieldID, date_);
  env->SetIntField(s, edit_date_fieldID, edit_date_);
  if (forward_info_ != nullptr) { jobject next; forward_info_->store(env, next); if (next) { env->SetObjectField(s, forward_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (import_info_ != nullptr) { jobject next; import_info_->store(env, next); if (next) { env->SetObjectField(s, import_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (interaction_info_ != nullptr) { jobject next; interaction_info_->store(env, next); if (next) { env->SetObjectField(s, interaction_info_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, unread_reactions_); if (arr_tmp_) { env->SetObjectField(s, unread_reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (fact_check_ != nullptr) { jobject next; fact_check_->store(env, next); if (next) { env->SetObjectField(s, fact_check_fieldID, next); env->DeleteLocalRef(next); } }
  if (reply_to_ != nullptr) { jobject next; reply_to_->store(env, next); if (next) { env->SetObjectField(s, reply_to_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  env->SetLongField(s, saved_messages_topic_id_fieldID, saved_messages_topic_id_);
  if (self_destruct_type_ != nullptr) { jobject next; self_destruct_type_->store(env, next); if (next) { env->SetObjectField(s, self_destruct_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, self_destruct_in_fieldID, self_destruct_in_);
  env->SetDoubleField(s, auto_delete_in_fieldID, auto_delete_in_);
  env->SetLongField(s, via_bot_user_id_fieldID, via_bot_user_id_);
  env->SetLongField(s, sender_business_bot_user_id_fieldID, sender_business_bot_user_id_);
  env->SetIntField(s, sender_boost_count_fieldID, sender_boost_count_);
  env->SetLongField(s, paid_message_star_count_fieldID, paid_message_star_count_);
  { jstring nextString = jni::to_jstring(env, author_signature_); if (nextString) { env->SetObjectField(s, author_signature_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, media_album_id_fieldID, media_album_id_);
  env->SetLongField(s, effect_id_fieldID, effect_id_);
  env->SetBooleanField(s, has_sensitive_content_fieldID, has_sensitive_content_);
  { jstring nextString = jni::to_jstring(env, restriction_reason_); if (nextString) { env->SetObjectField(s, restriction_reason_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
  if (reply_markup_ != nullptr) { jobject next; reply_markup_->store(env, next); if (next) { env->SetObjectField(s, reply_markup_fieldID, next); env->DeleteLocalRef(next); } }
}

void message::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Message");
    s.store_field("id", id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("chatId", chat_id_);
    s.store_object_field("sendingState", static_cast<const BaseObject *>(sending_state_.get()));
    s.store_object_field("schedulingState", static_cast<const BaseObject *>(scheduling_state_.get()));
    s.store_field("isOutgoing", is_outgoing_);
    s.store_field("isPinned", is_pinned_);
    s.store_field("isFromOffline", is_from_offline_);
    s.store_field("canBeSaved", can_be_saved_);
    s.store_field("hasTimestampedMedia", has_timestamped_media_);
    s.store_field("isChannelPost", is_channel_post_);
    s.store_field("isTopicMessage", is_topic_message_);
    s.store_field("containsUnreadMention", contains_unread_mention_);
    s.store_field("date", date_);
    s.store_field("editDate", edit_date_);
    s.store_object_field("forwardInfo", static_cast<const BaseObject *>(forward_info_.get()));
    s.store_object_field("importInfo", static_cast<const BaseObject *>(import_info_.get()));
    s.store_object_field("interactionInfo", static_cast<const BaseObject *>(interaction_info_.get()));
    { s.store_vector_begin("unreadReactions", unread_reactions_.size()); for (const auto &_value : unread_reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("factCheck", static_cast<const BaseObject *>(fact_check_.get()));
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_object_field("selfDestructType", static_cast<const BaseObject *>(self_destruct_type_.get()));
    s.store_field("selfDestructIn", self_destruct_in_);
    s.store_field("autoDeleteIn", auto_delete_in_);
    s.store_field("viaBotUserId", via_bot_user_id_);
    s.store_field("senderBusinessBotUserId", sender_business_bot_user_id_);
    s.store_field("senderBoostCount", sender_boost_count_);
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_field("authorSignature", author_signature_);
    s.store_field("mediaAlbumId", media_album_id_);
    s.store_field("effectId", effect_id_);
    s.store_field("hasSensitiveContent", has_sensitive_content_);
    s.store_field("restrictionReason", restriction_reason_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

void message::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Message").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    sending_state_fieldID = jni::get_field_id(env, Class, "sendingState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendingState;").c_str());
    scheduling_state_fieldID = jni::get_field_id(env, Class, "schedulingState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSchedulingState;").c_str());
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    is_from_offline_fieldID = jni::get_field_id(env, Class, "isFromOffline", "Z");
    can_be_saved_fieldID = jni::get_field_id(env, Class, "canBeSaved", "Z");
    has_timestamped_media_fieldID = jni::get_field_id(env, Class, "hasTimestampedMedia", "Z");
    is_channel_post_fieldID = jni::get_field_id(env, Class, "isChannelPost", "Z");
    is_topic_message_fieldID = jni::get_field_id(env, Class, "isTopicMessage", "Z");
    contains_unread_mention_fieldID = jni::get_field_id(env, Class, "containsUnreadMention", "Z");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    edit_date_fieldID = jni::get_field_id(env, Class, "editDate", "I");
    forward_info_fieldID = jni::get_field_id(env, Class, "forwardInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageForwardInfo;").c_str());
    import_info_fieldID = jni::get_field_id(env, Class, "importInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageImportInfo;").c_str());
    interaction_info_fieldID = jni::get_field_id(env, Class, "interactionInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageInteractionInfo;").c_str());
    unread_reactions_fieldID = jni::get_field_id(env, Class, "unreadReactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UnreadReaction;").c_str());
    fact_check_fieldID = jni::get_field_id(env, Class, "factCheck", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FactCheck;").c_str());
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageReplyTo;").c_str());
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    self_destruct_type_fieldID = jni::get_field_id(env, Class, "selfDestructType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSelfDestructType;").c_str());
    self_destruct_in_fieldID = jni::get_field_id(env, Class, "selfDestructIn", "D");
    auto_delete_in_fieldID = jni::get_field_id(env, Class, "autoDeleteIn", "D");
    via_bot_user_id_fieldID = jni::get_field_id(env, Class, "viaBotUserId", "J");
    sender_business_bot_user_id_fieldID = jni::get_field_id(env, Class, "senderBusinessBotUserId", "J");
    sender_boost_count_fieldID = jni::get_field_id(env, Class, "senderBoostCount", "I");
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    author_signature_fieldID = jni::get_field_id(env, Class, "authorSignature", "Ljava/lang/String;");
    media_album_id_fieldID = jni::get_field_id(env, Class, "mediaAlbumId", "J");
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    has_sensitive_content_fieldID = jni::get_field_id(env, Class, "hasSensitiveContent", "Z");
    restriction_reason_fieldID = jni::get_field_id(env, Class, "restrictionReason", "Ljava/lang/String;");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageAutoDeleteTime::Class;
jfieldID messageAutoDeleteTime::time_fieldID;

messageAutoDeleteTime::messageAutoDeleteTime()
  : time_()
{}

messageAutoDeleteTime::messageAutoDeleteTime(int32 time_)
  : time_(time_)
{}

const std::int32_t messageAutoDeleteTime::ID;

object_ptr<messageAutoDeleteTime> messageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageAutoDeleteTime> res = make_object<messageAutoDeleteTime>();
  res->time_ = env->GetIntField(p, res->time_fieldID);
  return res;
}

void messageAutoDeleteTime::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, time_fieldID, time_);
}

void messageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageAutoDeleteTime");
    s.store_field("time", time_);
    s.store_class_end();
  }
}

void messageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageAutoDeleteTime").c_str());
    time_fieldID = jni::get_field_id(env, Class, "time", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageCalendar::Class;
jfieldID messageCalendar::total_count_fieldID;
jfieldID messageCalendar::days_fieldID;

messageCalendar::messageCalendar()
  : total_count_()
  , days_()
{}

messageCalendar::messageCalendar(int32 total_count_, array<object_ptr<messageCalendarDay>> &&days_)
  : total_count_(total_count_)
  , days_(std::move(days_))
{}

const std::int32_t messageCalendar::ID;

object_ptr<messageCalendar> messageCalendar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageCalendar> res = make_object<messageCalendar>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->days_ = jni::FetchVector<messageCalendarDay>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->days_fieldID));
  return res;
}

void messageCalendar::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, days_); if (arr_tmp_) { env->SetObjectField(s, days_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageCalendar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageCalendar");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("days", days_.size()); for (const auto &_value : days_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageCalendar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageCalendar").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    days_fieldID = jni::get_field_id(env, Class, "days", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageCalendarDay;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageCalendarDay::Class;
jfieldID messageCalendarDay::total_count_fieldID;
jfieldID messageCalendarDay::message_fieldID;

messageCalendarDay::messageCalendarDay()
  : total_count_()
  , message_()
{}

messageCalendarDay::messageCalendarDay(int32 total_count_, object_ptr<message> &&message_)
  : total_count_(total_count_)
  , message_(std::move(message_))
{}

const std::int32_t messageCalendarDay::ID;

object_ptr<messageCalendarDay> messageCalendarDay::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageCalendarDay> res = make_object<messageCalendarDay>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return res;
}

void messageCalendarDay::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageCalendarDay::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageCalendarDay");
    s.store_field("totalCount", total_count_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void messageCalendarDay::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageCalendarDay").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass MessageContent::Class;

object_ptr<MessageContent> MessageContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageText::ID:
      return messageText::fetch(env, p);
    case messageAnimation::ID:
      return messageAnimation::fetch(env, p);
    case messageAudio::ID:
      return messageAudio::fetch(env, p);
    case messageDocument::ID:
      return messageDocument::fetch(env, p);
    case messagePaidMedia::ID:
      return messagePaidMedia::fetch(env, p);
    case messagePhoto::ID:
      return messagePhoto::fetch(env, p);
    case messageSticker::ID:
      return messageSticker::fetch(env, p);
    case messageVideo::ID:
      return messageVideo::fetch(env, p);
    case messageVideoNote::ID:
      return messageVideoNote::fetch(env, p);
    case messageVoiceNote::ID:
      return messageVoiceNote::fetch(env, p);
    case messageExpiredPhoto::ID:
      return messageExpiredPhoto::fetch(env, p);
    case messageExpiredVideo::ID:
      return messageExpiredVideo::fetch(env, p);
    case messageExpiredVideoNote::ID:
      return messageExpiredVideoNote::fetch(env, p);
    case messageExpiredVoiceNote::ID:
      return messageExpiredVoiceNote::fetch(env, p);
    case messageLocation::ID:
      return messageLocation::fetch(env, p);
    case messageVenue::ID:
      return messageVenue::fetch(env, p);
    case messageContact::ID:
      return messageContact::fetch(env, p);
    case messageAnimatedEmoji::ID:
      return messageAnimatedEmoji::fetch(env, p);
    case messageDice::ID:
      return messageDice::fetch(env, p);
    case messageGame::ID:
      return messageGame::fetch(env, p);
    case messagePoll::ID:
      return messagePoll::fetch(env, p);
    case messageStory::ID:
      return messageStory::fetch(env, p);
    case messageInvoice::ID:
      return messageInvoice::fetch(env, p);
    case messageCall::ID:
      return messageCall::fetch(env, p);
    case messageGroupCall::ID:
      return messageGroupCall::fetch(env, p);
    case messageVideoChatScheduled::ID:
      return messageVideoChatScheduled::fetch(env, p);
    case messageVideoChatStarted::ID:
      return messageVideoChatStarted::fetch(env, p);
    case messageVideoChatEnded::ID:
      return messageVideoChatEnded::fetch(env, p);
    case messageInviteVideoChatParticipants::ID:
      return messageInviteVideoChatParticipants::fetch(env, p);
    case messageBasicGroupChatCreate::ID:
      return messageBasicGroupChatCreate::fetch(env, p);
    case messageSupergroupChatCreate::ID:
      return messageSupergroupChatCreate::fetch(env, p);
    case messageChatChangeTitle::ID:
      return messageChatChangeTitle::fetch(env, p);
    case messageChatChangePhoto::ID:
      return messageChatChangePhoto::fetch(env, p);
    case messageChatDeletePhoto::ID:
      return messageChatDeletePhoto::fetch(env, p);
    case messageChatAddMembers::ID:
      return messageChatAddMembers::fetch(env, p);
    case messageChatJoinByLink::ID:
      return messageChatJoinByLink::fetch(env, p);
    case messageChatJoinByRequest::ID:
      return messageChatJoinByRequest::fetch(env, p);
    case messageChatDeleteMember::ID:
      return messageChatDeleteMember::fetch(env, p);
    case messageChatUpgradeTo::ID:
      return messageChatUpgradeTo::fetch(env, p);
    case messageChatUpgradeFrom::ID:
      return messageChatUpgradeFrom::fetch(env, p);
    case messagePinMessage::ID:
      return messagePinMessage::fetch(env, p);
    case messageScreenshotTaken::ID:
      return messageScreenshotTaken::fetch(env, p);
    case messageChatSetBackground::ID:
      return messageChatSetBackground::fetch(env, p);
    case messageChatSetTheme::ID:
      return messageChatSetTheme::fetch(env, p);
    case messageChatSetMessageAutoDeleteTime::ID:
      return messageChatSetMessageAutoDeleteTime::fetch(env, p);
    case messageChatBoost::ID:
      return messageChatBoost::fetch(env, p);
    case messageForumTopicCreated::ID:
      return messageForumTopicCreated::fetch(env, p);
    case messageForumTopicEdited::ID:
      return messageForumTopicEdited::fetch(env, p);
    case messageForumTopicIsClosedToggled::ID:
      return messageForumTopicIsClosedToggled::fetch(env, p);
    case messageForumTopicIsHiddenToggled::ID:
      return messageForumTopicIsHiddenToggled::fetch(env, p);
    case messageSuggestProfilePhoto::ID:
      return messageSuggestProfilePhoto::fetch(env, p);
    case messageCustomServiceAction::ID:
      return messageCustomServiceAction::fetch(env, p);
    case messageGameScore::ID:
      return messageGameScore::fetch(env, p);
    case messagePaymentSuccessful::ID:
      return messagePaymentSuccessful::fetch(env, p);
    case messagePaymentSuccessfulBot::ID:
      return messagePaymentSuccessfulBot::fetch(env, p);
    case messagePaymentRefunded::ID:
      return messagePaymentRefunded::fetch(env, p);
    case messageGiftedPremium::ID:
      return messageGiftedPremium::fetch(env, p);
    case messagePremiumGiftCode::ID:
      return messagePremiumGiftCode::fetch(env, p);
    case messageGiveawayCreated::ID:
      return messageGiveawayCreated::fetch(env, p);
    case messageGiveaway::ID:
      return messageGiveaway::fetch(env, p);
    case messageGiveawayCompleted::ID:
      return messageGiveawayCompleted::fetch(env, p);
    case messageGiveawayWinners::ID:
      return messageGiveawayWinners::fetch(env, p);
    case messageGiftedStars::ID:
      return messageGiftedStars::fetch(env, p);
    case messageGiveawayPrizeStars::ID:
      return messageGiveawayPrizeStars::fetch(env, p);
    case messageGift::ID:
      return messageGift::fetch(env, p);
    case messageUpgradedGift::ID:
      return messageUpgradedGift::fetch(env, p);
    case messageRefundedUpgradedGift::ID:
      return messageRefundedUpgradedGift::fetch(env, p);
    case messagePaidMessagesRefunded::ID:
      return messagePaidMessagesRefunded::fetch(env, p);
    case messagePaidMessagePriceChanged::ID:
      return messagePaidMessagePriceChanged::fetch(env, p);
    case messageContactRegistered::ID:
      return messageContactRegistered::fetch(env, p);
    case messageUsersShared::ID:
      return messageUsersShared::fetch(env, p);
    case messageChatShared::ID:
      return messageChatShared::fetch(env, p);
    case messageBotWriteAccessAllowed::ID:
      return messageBotWriteAccessAllowed::fetch(env, p);
    case messageWebAppDataSent::ID:
      return messageWebAppDataSent::fetch(env, p);
    case messageWebAppDataReceived::ID:
      return messageWebAppDataReceived::fetch(env, p);
    case messagePassportDataSent::ID:
      return messagePassportDataSent::fetch(env, p);
    case messagePassportDataReceived::ID:
      return messagePassportDataReceived::fetch(env, p);
    case messageProximityAlertTriggered::ID:
      return messageProximityAlertTriggered::fetch(env, p);
    case messageUnsupported::ID:
      return messageUnsupported::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageContent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageText::Class;
jfieldID messageText::text_fieldID;
jfieldID messageText::link_preview_fieldID;
jfieldID messageText::link_preview_options_fieldID;

messageText::messageText()
  : text_()
  , link_preview_()
  , link_preview_options_()
{}

messageText::messageText(object_ptr<formattedText> &&text_, object_ptr<linkPreview> &&link_preview_, object_ptr<linkPreviewOptions> &&link_preview_options_)
  : text_(std::move(text_))
  , link_preview_(std::move(link_preview_))
  , link_preview_options_(std::move(link_preview_options_))
{}

const std::int32_t messageText::ID;

object_ptr<MessageContent> messageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageText> res = make_object<messageText>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->link_preview_ = jni::fetch_tl_object<linkPreview>(env, jni::fetch_object(env, p, res->link_preview_fieldID));
  res->link_preview_options_ = jni::fetch_tl_object<linkPreviewOptions>(env, jni::fetch_object(env, p, res->link_preview_options_fieldID));
  return std::move(res);
}

void messageText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  if (link_preview_ != nullptr) { jobject next; link_preview_->store(env, next); if (next) { env->SetObjectField(s, link_preview_fieldID, next); env->DeleteLocalRef(next); } }
  if (link_preview_options_ != nullptr) { jobject next; link_preview_options_->store(env, next); if (next) { env->SetObjectField(s, link_preview_options_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageText");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("linkPreview", static_cast<const BaseObject *>(link_preview_.get()));
    s.store_object_field("linkPreviewOptions", static_cast<const BaseObject *>(link_preview_options_.get()));
    s.store_class_end();
  }
}

void messageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    link_preview_fieldID = jni::get_field_id(env, Class, "linkPreview", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LinkPreview;").c_str());
    link_preview_options_fieldID = jni::get_field_id(env, Class, "linkPreviewOptions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LinkPreviewOptions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageAnimation::Class;
jfieldID messageAnimation::animation_fieldID;
jfieldID messageAnimation::caption_fieldID;
jfieldID messageAnimation::show_caption_above_media_fieldID;
jfieldID messageAnimation::has_spoiler_fieldID;
jfieldID messageAnimation::is_secret_fieldID;

messageAnimation::messageAnimation()
  : animation_()
  , caption_()
  , show_caption_above_media_()
  , has_spoiler_()
  , is_secret_()
{}

messageAnimation::messageAnimation(object_ptr<animation> &&animation_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_)
  : animation_(std::move(animation_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , has_spoiler_(has_spoiler_)
  , is_secret_(is_secret_)
{}

const std::int32_t messageAnimation::ID;

object_ptr<MessageContent> messageAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageAnimation> res = make_object<messageAnimation>();
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  return std::move(res);
}

void messageAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
}

void messageAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_field("isSecret", is_secret_);
    s.store_class_end();
  }
}

void messageAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageAudio::Class;
jfieldID messageAudio::audio_fieldID;
jfieldID messageAudio::caption_fieldID;

messageAudio::messageAudio()
  : audio_()
  , caption_()
{}

messageAudio::messageAudio(object_ptr<audio> &&audio_, object_ptr<formattedText> &&caption_)
  : audio_(std::move(audio_))
  , caption_(std::move(caption_))
{}

const std::int32_t messageAudio::ID;

object_ptr<MessageContent> messageAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageAudio> res = make_object<messageAudio>();
  res->audio_ = jni::fetch_tl_object<audio>(env, jni::fetch_object(env, p, res->audio_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void messageAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageAudio");
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void messageAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageAudio").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Audio;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageDocument::Class;
jfieldID messageDocument::document_fieldID;
jfieldID messageDocument::caption_fieldID;

messageDocument::messageDocument()
  : document_()
  , caption_()
{}

messageDocument::messageDocument(object_ptr<document> &&document_, object_ptr<formattedText> &&caption_)
  : document_(std::move(document_))
  , caption_(std::move(caption_))
{}

const std::int32_t messageDocument::ID;

object_ptr<MessageContent> messageDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageDocument> res = make_object<messageDocument>();
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void messageDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageDocument");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void messageDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageDocument").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messagePaidMedia::Class;
jfieldID messagePaidMedia::star_count_fieldID;
jfieldID messagePaidMedia::media_fieldID;
jfieldID messagePaidMedia::caption_fieldID;
jfieldID messagePaidMedia::show_caption_above_media_fieldID;

messagePaidMedia::messagePaidMedia()
  : star_count_()
  , media_()
  , caption_()
  , show_caption_above_media_()
{}

messagePaidMedia::messagePaidMedia(int53 star_count_, array<object_ptr<PaidMedia>> &&media_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_)
  : star_count_(star_count_)
  , media_(std::move(media_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
{}

const std::int32_t messagePaidMedia::ID;

object_ptr<MessageContent> messagePaidMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaidMedia> res = make_object<messagePaidMedia>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->media_ = jni::FetchVector<PaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  return std::move(res);
}

void messagePaidMedia::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
}

void messagePaidMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaidMedia");
    s.store_field("starCount", star_count_);
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_class_end();
  }
}

void messagePaidMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaidMedia").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messagePhoto::Class;
jfieldID messagePhoto::photo_fieldID;
jfieldID messagePhoto::caption_fieldID;
jfieldID messagePhoto::show_caption_above_media_fieldID;
jfieldID messagePhoto::has_spoiler_fieldID;
jfieldID messagePhoto::is_secret_fieldID;

messagePhoto::messagePhoto()
  : photo_()
  , caption_()
  , show_caption_above_media_()
  , has_spoiler_()
  , is_secret_()
{}

messagePhoto::messagePhoto(object_ptr<photo> &&photo_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_)
  : photo_(std::move(photo_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , has_spoiler_(has_spoiler_)
  , is_secret_(is_secret_)
{}

const std::int32_t messagePhoto::ID;

object_ptr<MessageContent> messagePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePhoto> res = make_object<messagePhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  return std::move(res);
}

void messagePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
}

void messagePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_field("isSecret", is_secret_);
    s.store_class_end();
  }
}

void messagePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageSticker::Class;
jfieldID messageSticker::sticker_fieldID;
jfieldID messageSticker::is_premium_fieldID;

messageSticker::messageSticker()
  : sticker_()
  , is_premium_()
{}

messageSticker::messageSticker(object_ptr<sticker> &&sticker_, bool is_premium_)
  : sticker_(std::move(sticker_))
  , is_premium_(is_premium_)
{}

const std::int32_t messageSticker::ID;

object_ptr<MessageContent> messageSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSticker> res = make_object<messageSticker>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->is_premium_ = (env->GetBooleanField(p, res->is_premium_fieldID) != 0);
  return std::move(res);
}

void messageSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_premium_fieldID, is_premium_);
}

void messageSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("isPremium", is_premium_);
    s.store_class_end();
  }
}

void messageSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    is_premium_fieldID = jni::get_field_id(env, Class, "isPremium", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageVideo::Class;
jfieldID messageVideo::video_fieldID;
jfieldID messageVideo::alternative_videos_fieldID;
jfieldID messageVideo::cover_fieldID;
jfieldID messageVideo::start_timestamp_fieldID;
jfieldID messageVideo::caption_fieldID;
jfieldID messageVideo::show_caption_above_media_fieldID;
jfieldID messageVideo::has_spoiler_fieldID;
jfieldID messageVideo::is_secret_fieldID;

messageVideo::messageVideo()
  : video_()
  , alternative_videos_()
  , cover_()
  , start_timestamp_()
  , caption_()
  , show_caption_above_media_()
  , has_spoiler_()
  , is_secret_()
{}

messageVideo::messageVideo(object_ptr<video> &&video_, array<object_ptr<alternativeVideo>> &&alternative_videos_, object_ptr<photo> &&cover_, int32 start_timestamp_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_)
  : video_(std::move(video_))
  , alternative_videos_(std::move(alternative_videos_))
  , cover_(std::move(cover_))
  , start_timestamp_(start_timestamp_)
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
  , has_spoiler_(has_spoiler_)
  , is_secret_(is_secret_)
{}

const std::int32_t messageVideo::ID;

object_ptr<MessageContent> messageVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVideo> res = make_object<messageVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->alternative_videos_ = jni::FetchVector<alternativeVideo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->alternative_videos_fieldID));
  res->cover_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->cover_fieldID));
  res->start_timestamp_ = env->GetIntField(p, res->start_timestamp_fieldID);
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  res->has_spoiler_ = (env->GetBooleanField(p, res->has_spoiler_fieldID) != 0);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  return std::move(res);
}

void messageVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, alternative_videos_); if (arr_tmp_) { env->SetObjectField(s, alternative_videos_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_timestamp_fieldID, start_timestamp_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_caption_above_media_fieldID, show_caption_above_media_);
  env->SetBooleanField(s, has_spoiler_fieldID, has_spoiler_);
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
}

void messageVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    { s.store_vector_begin("alternativeVideos", alternative_videos_.size()); for (const auto &_value : alternative_videos_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_field("startTimestamp", start_timestamp_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_field("hasSpoiler", has_spoiler_);
    s.store_field("isSecret", is_secret_);
    s.store_class_end();
  }
}

void messageVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    alternative_videos_fieldID = jni::get_field_id(env, Class, "alternativeVideos", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AlternativeVideo;").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    start_timestamp_fieldID = jni::get_field_id(env, Class, "startTimestamp", "I");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    has_spoiler_fieldID = jni::get_field_id(env, Class, "hasSpoiler", "Z");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageVideoNote::Class;
jfieldID messageVideoNote::video_note_fieldID;
jfieldID messageVideoNote::is_viewed_fieldID;
jfieldID messageVideoNote::is_secret_fieldID;

messageVideoNote::messageVideoNote()
  : video_note_()
  , is_viewed_()
  , is_secret_()
{}

messageVideoNote::messageVideoNote(object_ptr<videoNote> &&video_note_, bool is_viewed_, bool is_secret_)
  : video_note_(std::move(video_note_))
  , is_viewed_(is_viewed_)
  , is_secret_(is_secret_)
{}

const std::int32_t messageVideoNote::ID;

object_ptr<MessageContent> messageVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVideoNote> res = make_object<messageVideoNote>();
  res->video_note_ = jni::fetch_tl_object<videoNote>(env, jni::fetch_object(env, p, res->video_note_fieldID));
  res->is_viewed_ = (env->GetBooleanField(p, res->is_viewed_fieldID) != 0);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  return std::move(res);
}

void messageVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_note_ != nullptr) { jobject next; video_note_->store(env, next); if (next) { env->SetObjectField(s, video_note_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_viewed_fieldID, is_viewed_);
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
}

void messageVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVideoNote");
    s.store_object_field("videoNote", static_cast<const BaseObject *>(video_note_.get()));
    s.store_field("isViewed", is_viewed_);
    s.store_field("isSecret", is_secret_);
    s.store_class_end();
  }
}

void messageVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVideoNote").c_str());
    video_note_fieldID = jni::get_field_id(env, Class, "videoNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VideoNote;").c_str());
    is_viewed_fieldID = jni::get_field_id(env, Class, "isViewed", "Z");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageVoiceNote::Class;
jfieldID messageVoiceNote::voice_note_fieldID;
jfieldID messageVoiceNote::caption_fieldID;
jfieldID messageVoiceNote::is_listened_fieldID;

messageVoiceNote::messageVoiceNote()
  : voice_note_()
  , caption_()
  , is_listened_()
{}

messageVoiceNote::messageVoiceNote(object_ptr<voiceNote> &&voice_note_, object_ptr<formattedText> &&caption_, bool is_listened_)
  : voice_note_(std::move(voice_note_))
  , caption_(std::move(caption_))
  , is_listened_(is_listened_)
{}

const std::int32_t messageVoiceNote::ID;

object_ptr<MessageContent> messageVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVoiceNote> res = make_object<messageVoiceNote>();
  res->voice_note_ = jni::fetch_tl_object<voiceNote>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->is_listened_ = (env->GetBooleanField(p, res->is_listened_fieldID) != 0);
  return std::move(res);
}

void messageVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_listened_fieldID, is_listened_);
}

void messageVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVoiceNote");
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("isListened", is_listened_);
    s.store_class_end();
  }
}

void messageVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVoiceNote").c_str());
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VoiceNote;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    is_listened_fieldID = jni::get_field_id(env, Class, "isListened", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageExpiredPhoto::Class;

messageExpiredPhoto::messageExpiredPhoto() {
}

const std::int32_t messageExpiredPhoto::ID;

object_ptr<MessageContent> messageExpiredPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageExpiredPhoto>();
}

void messageExpiredPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageExpiredPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageExpiredPhoto");
    s.store_class_end();
  }
}

void messageExpiredPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageExpiredPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageExpiredVideo::Class;

messageExpiredVideo::messageExpiredVideo() {
}

const std::int32_t messageExpiredVideo::ID;

object_ptr<MessageContent> messageExpiredVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageExpiredVideo>();
}

void messageExpiredVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageExpiredVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageExpiredVideo");
    s.store_class_end();
  }
}

void messageExpiredVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageExpiredVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageExpiredVideoNote::Class;

messageExpiredVideoNote::messageExpiredVideoNote() {
}

const std::int32_t messageExpiredVideoNote::ID;

object_ptr<MessageContent> messageExpiredVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageExpiredVideoNote>();
}

void messageExpiredVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageExpiredVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageExpiredVideoNote");
    s.store_class_end();
  }
}

void messageExpiredVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageExpiredVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageExpiredVoiceNote::Class;

messageExpiredVoiceNote::messageExpiredVoiceNote() {
}

const std::int32_t messageExpiredVoiceNote::ID;

object_ptr<MessageContent> messageExpiredVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageExpiredVoiceNote>();
}

void messageExpiredVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageExpiredVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageExpiredVoiceNote");
    s.store_class_end();
  }
}

void messageExpiredVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageExpiredVoiceNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageLocation::Class;
jfieldID messageLocation::location_fieldID;
jfieldID messageLocation::live_period_fieldID;
jfieldID messageLocation::expires_in_fieldID;
jfieldID messageLocation::heading_fieldID;
jfieldID messageLocation::proximity_alert_radius_fieldID;

messageLocation::messageLocation()
  : location_()
  , live_period_()
  , expires_in_()
  , heading_()
  , proximity_alert_radius_()
{}

messageLocation::messageLocation(object_ptr<location> &&location_, int32 live_period_, int32 expires_in_, int32 heading_, int32 proximity_alert_radius_)
  : location_(std::move(location_))
  , live_period_(live_period_)
  , expires_in_(expires_in_)
  , heading_(heading_)
  , proximity_alert_radius_(proximity_alert_radius_)
{}

const std::int32_t messageLocation::ID;

object_ptr<MessageContent> messageLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageLocation> res = make_object<messageLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->expires_in_ = env->GetIntField(p, res->expires_in_fieldID);
  res->heading_ = env->GetIntField(p, res->heading_fieldID);
  res->proximity_alert_radius_ = env->GetIntField(p, res->proximity_alert_radius_fieldID);
  return std::move(res);
}

void messageLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, live_period_fieldID, live_period_);
  env->SetIntField(s, expires_in_fieldID, expires_in_);
  env->SetIntField(s, heading_fieldID, heading_);
  env->SetIntField(s, proximity_alert_radius_fieldID, proximity_alert_radius_);
}

void messageLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("expiresIn", expires_in_);
    s.store_field("heading", heading_);
    s.store_field("proximityAlertRadius", proximity_alert_radius_);
    s.store_class_end();
  }
}

void messageLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    expires_in_fieldID = jni::get_field_id(env, Class, "expiresIn", "I");
    heading_fieldID = jni::get_field_id(env, Class, "heading", "I");
    proximity_alert_radius_fieldID = jni::get_field_id(env, Class, "proximityAlertRadius", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageVenue::Class;
jfieldID messageVenue::venue_fieldID;

messageVenue::messageVenue()
  : venue_()
{}

messageVenue::messageVenue(object_ptr<venue> &&venue_)
  : venue_(std::move(venue_))
{}

const std::int32_t messageVenue::ID;

object_ptr<MessageContent> messageVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVenue> res = make_object<messageVenue>();
  res->venue_ = jni::fetch_tl_object<venue>(env, jni::fetch_object(env, p, res->venue_fieldID));
  return std::move(res);
}

void messageVenue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (venue_ != nullptr) { jobject next; venue_->store(env, next); if (next) { env->SetObjectField(s, venue_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVenue");
    s.store_object_field("venue", static_cast<const BaseObject *>(venue_.get()));
    s.store_class_end();
  }
}

void messageVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVenue").c_str());
    venue_fieldID = jni::get_field_id(env, Class, "venue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Venue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageContact::Class;
jfieldID messageContact::contact_fieldID;

messageContact::messageContact()
  : contact_()
{}

messageContact::messageContact(object_ptr<contact> &&contact_)
  : contact_(std::move(contact_))
{}

const std::int32_t messageContact::ID;

object_ptr<MessageContent> messageContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageContact> res = make_object<messageContact>();
  res->contact_ = jni::fetch_tl_object<contact>(env, jni::fetch_object(env, p, res->contact_fieldID));
  return std::move(res);
}

void messageContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (contact_ != nullptr) { jobject next; contact_->store(env, next); if (next) { env->SetObjectField(s, contact_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageContact");
    s.store_object_field("contact", static_cast<const BaseObject *>(contact_.get()));
    s.store_class_end();
  }
}

void messageContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageContact").c_str());
    contact_fieldID = jni::get_field_id(env, Class, "contact", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageAnimatedEmoji::Class;
jfieldID messageAnimatedEmoji::animated_emoji_fieldID;
jfieldID messageAnimatedEmoji::emoji_fieldID;

messageAnimatedEmoji::messageAnimatedEmoji()
  : animated_emoji_()
  , emoji_()
{}

messageAnimatedEmoji::messageAnimatedEmoji(object_ptr<animatedEmoji> &&animated_emoji_, string const &emoji_)
  : animated_emoji_(std::move(animated_emoji_))
  , emoji_(emoji_)
{}

const std::int32_t messageAnimatedEmoji::ID;

object_ptr<MessageContent> messageAnimatedEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageAnimatedEmoji> res = make_object<messageAnimatedEmoji>();
  res->animated_emoji_ = jni::fetch_tl_object<animatedEmoji>(env, jni::fetch_object(env, p, res->animated_emoji_fieldID));
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return std::move(res);
}

void messageAnimatedEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animated_emoji_ != nullptr) { jobject next; animated_emoji_->store(env, next); if (next) { env->SetObjectField(s, animated_emoji_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageAnimatedEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageAnimatedEmoji");
    s.store_object_field("animatedEmoji", static_cast<const BaseObject *>(animated_emoji_.get()));
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

void messageAnimatedEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageAnimatedEmoji").c_str());
    animated_emoji_fieldID = jni::get_field_id(env, Class, "animatedEmoji", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AnimatedEmoji;").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageDice::Class;
jfieldID messageDice::initial_state_fieldID;
jfieldID messageDice::final_state_fieldID;
jfieldID messageDice::emoji_fieldID;
jfieldID messageDice::value_fieldID;
jfieldID messageDice::success_animation_frame_number_fieldID;

messageDice::messageDice()
  : initial_state_()
  , final_state_()
  , emoji_()
  , value_()
  , success_animation_frame_number_()
{}

messageDice::messageDice(object_ptr<DiceStickers> &&initial_state_, object_ptr<DiceStickers> &&final_state_, string const &emoji_, int32 value_, int32 success_animation_frame_number_)
  : initial_state_(std::move(initial_state_))
  , final_state_(std::move(final_state_))
  , emoji_(emoji_)
  , value_(value_)
  , success_animation_frame_number_(success_animation_frame_number_)
{}

const std::int32_t messageDice::ID;

object_ptr<MessageContent> messageDice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageDice> res = make_object<messageDice>();
  res->initial_state_ = jni::fetch_tl_object<DiceStickers>(env, jni::fetch_object(env, p, res->initial_state_fieldID));
  res->final_state_ = jni::fetch_tl_object<DiceStickers>(env, jni::fetch_object(env, p, res->final_state_fieldID));
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->value_ = env->GetIntField(p, res->value_fieldID);
  res->success_animation_frame_number_ = env->GetIntField(p, res->success_animation_frame_number_fieldID);
  return std::move(res);
}

void messageDice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (initial_state_ != nullptr) { jobject next; initial_state_->store(env, next); if (next) { env->SetObjectField(s, initial_state_fieldID, next); env->DeleteLocalRef(next); } }
  if (final_state_ != nullptr) { jobject next; final_state_->store(env, next); if (next) { env->SetObjectField(s, final_state_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, value_fieldID, value_);
  env->SetIntField(s, success_animation_frame_number_fieldID, success_animation_frame_number_);
}

void messageDice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageDice");
    s.store_object_field("initialState", static_cast<const BaseObject *>(initial_state_.get()));
    s.store_object_field("finalState", static_cast<const BaseObject *>(final_state_.get()));
    s.store_field("emoji", emoji_);
    s.store_field("value", value_);
    s.store_field("successAnimationFrameNumber", success_animation_frame_number_);
    s.store_class_end();
  }
}

void messageDice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageDice").c_str());
    initial_state_fieldID = jni::get_field_id(env, Class, "initialState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DiceStickers;").c_str());
    final_state_fieldID = jni::get_field_id(env, Class, "finalState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DiceStickers;").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    value_fieldID = jni::get_field_id(env, Class, "value", "I");
    success_animation_frame_number_fieldID = jni::get_field_id(env, Class, "successAnimationFrameNumber", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageGame::Class;
jfieldID messageGame::game_fieldID;

messageGame::messageGame()
  : game_()
{}

messageGame::messageGame(object_ptr<game> &&game_)
  : game_(std::move(game_))
{}

const std::int32_t messageGame::ID;

object_ptr<MessageContent> messageGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGame> res = make_object<messageGame>();
  res->game_ = jni::fetch_tl_object<game>(env, jni::fetch_object(env, p, res->game_fieldID));
  return std::move(res);
}

void messageGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (game_ != nullptr) { jobject next; game_->store(env, next); if (next) { env->SetObjectField(s, game_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGame");
    s.store_object_field("game", static_cast<const BaseObject *>(game_.get()));
    s.store_class_end();
  }
}

void messageGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGame").c_str());
    game_fieldID = jni::get_field_id(env, Class, "game", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Game;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messagePoll::Class;
jfieldID messagePoll::poll_fieldID;

messagePoll::messagePoll()
  : poll_()
{}

messagePoll::messagePoll(object_ptr<poll> &&poll_)
  : poll_(std::move(poll_))
{}

const std::int32_t messagePoll::ID;

object_ptr<MessageContent> messagePoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePoll> res = make_object<messagePoll>();
  res->poll_ = jni::fetch_tl_object<poll>(env, jni::fetch_object(env, p, res->poll_fieldID));
  return std::move(res);
}

void messagePoll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (poll_ != nullptr) { jobject next; poll_->store(env, next); if (next) { env->SetObjectField(s, poll_fieldID, next); env->DeleteLocalRef(next); } }
}

void messagePoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePoll");
    s.store_object_field("poll", static_cast<const BaseObject *>(poll_.get()));
    s.store_class_end();
  }
}

void messagePoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePoll").c_str());
    poll_fieldID = jni::get_field_id(env, Class, "poll", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Poll;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageStory::Class;
jfieldID messageStory::story_poster_chat_id_fieldID;
jfieldID messageStory::story_id_fieldID;
jfieldID messageStory::via_mention_fieldID;

messageStory::messageStory()
  : story_poster_chat_id_()
  , story_id_()
  , via_mention_()
{}

messageStory::messageStory(int53 story_poster_chat_id_, int32 story_id_, bool via_mention_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , via_mention_(via_mention_)
{}

const std::int32_t messageStory::ID;

object_ptr<MessageContent> messageStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageStory> res = make_object<messageStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->via_mention_ = (env->GetBooleanField(p, res->via_mention_fieldID) != 0);
  return std::move(res);
}

void messageStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
  env->SetBooleanField(s, via_mention_fieldID, via_mention_);
}

void messageStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("viaMention", via_mention_);
    s.store_class_end();
  }
}

void messageStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    via_mention_fieldID = jni::get_field_id(env, Class, "viaMention", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageInvoice::Class;
jfieldID messageInvoice::product_info_fieldID;
jfieldID messageInvoice::currency_fieldID;
jfieldID messageInvoice::total_amount_fieldID;
jfieldID messageInvoice::start_parameter_fieldID;
jfieldID messageInvoice::is_test_fieldID;
jfieldID messageInvoice::need_shipping_address_fieldID;
jfieldID messageInvoice::receipt_message_id_fieldID;
jfieldID messageInvoice::paid_media_fieldID;
jfieldID messageInvoice::paid_media_caption_fieldID;

messageInvoice::messageInvoice()
  : product_info_()
  , currency_()
  , total_amount_()
  , start_parameter_()
  , is_test_()
  , need_shipping_address_()
  , receipt_message_id_()
  , paid_media_()
  , paid_media_caption_()
{}

messageInvoice::messageInvoice(object_ptr<productInfo> &&product_info_, string const &currency_, int53 total_amount_, string const &start_parameter_, bool is_test_, bool need_shipping_address_, int53 receipt_message_id_, object_ptr<PaidMedia> &&paid_media_, object_ptr<formattedText> &&paid_media_caption_)
  : product_info_(std::move(product_info_))
  , currency_(currency_)
  , total_amount_(total_amount_)
  , start_parameter_(start_parameter_)
  , is_test_(is_test_)
  , need_shipping_address_(need_shipping_address_)
  , receipt_message_id_(receipt_message_id_)
  , paid_media_(std::move(paid_media_))
  , paid_media_caption_(std::move(paid_media_caption_))
{}

const std::int32_t messageInvoice::ID;

object_ptr<MessageContent> messageInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageInvoice> res = make_object<messageInvoice>();
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->is_test_ = (env->GetBooleanField(p, res->is_test_fieldID) != 0);
  res->need_shipping_address_ = (env->GetBooleanField(p, res->need_shipping_address_fieldID) != 0);
  res->receipt_message_id_ = env->GetLongField(p, res->receipt_message_id_fieldID);
  res->paid_media_ = jni::fetch_tl_object<PaidMedia>(env, jni::fetch_object(env, p, res->paid_media_fieldID));
  res->paid_media_caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->paid_media_caption_fieldID));
  return std::move(res);
}

void messageInvoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  { jstring nextString = jni::to_jstring(env, start_parameter_); if (nextString) { env->SetObjectField(s, start_parameter_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_test_fieldID, is_test_);
  env->SetBooleanField(s, need_shipping_address_fieldID, need_shipping_address_);
  env->SetLongField(s, receipt_message_id_fieldID, receipt_message_id_);
  if (paid_media_ != nullptr) { jobject next; paid_media_->store(env, next); if (next) { env->SetObjectField(s, paid_media_fieldID, next); env->DeleteLocalRef(next); } }
  if (paid_media_caption_ != nullptr) { jobject next; paid_media_caption_->store(env, next); if (next) { env->SetObjectField(s, paid_media_caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageInvoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageInvoice");
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_field("currency", currency_);
    s.store_field("totalAmount", total_amount_);
    s.store_field("startParameter", start_parameter_);
    s.store_field("isTest", is_test_);
    s.store_field("needShippingAddress", need_shipping_address_);
    s.store_field("receiptMessageId", receipt_message_id_);
    s.store_object_field("paidMedia", static_cast<const BaseObject *>(paid_media_.get()));
    s.store_object_field("paidMediaCaption", static_cast<const BaseObject *>(paid_media_caption_.get()));
    s.store_class_end();
  }
}

void messageInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageInvoice").c_str());
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    is_test_fieldID = jni::get_field_id(env, Class, "isTest", "Z");
    need_shipping_address_fieldID = jni::get_field_id(env, Class, "needShippingAddress", "Z");
    receipt_message_id_fieldID = jni::get_field_id(env, Class, "receiptMessageId", "J");
    paid_media_fieldID = jni::get_field_id(env, Class, "paidMedia", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    paid_media_caption_fieldID = jni::get_field_id(env, Class, "paidMediaCaption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageCall::Class;
jfieldID messageCall::is_video_fieldID;
jfieldID messageCall::discard_reason_fieldID;
jfieldID messageCall::duration_fieldID;

messageCall::messageCall()
  : is_video_()
  , discard_reason_()
  , duration_()
{}

messageCall::messageCall(bool is_video_, object_ptr<CallDiscardReason> &&discard_reason_, int32 duration_)
  : is_video_(is_video_)
  , discard_reason_(std::move(discard_reason_))
  , duration_(duration_)
{}

const std::int32_t messageCall::ID;

object_ptr<MessageContent> messageCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageCall> res = make_object<messageCall>();
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  res->discard_reason_ = jni::fetch_tl_object<CallDiscardReason>(env, jni::fetch_object(env, p, res->discard_reason_fieldID));
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  return std::move(res);
}

void messageCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_video_fieldID, is_video_);
  if (discard_reason_ != nullptr) { jobject next; discard_reason_->store(env, next); if (next) { env->SetObjectField(s, discard_reason_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, duration_fieldID, duration_);
}

void messageCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageCall");
    s.store_field("isVideo", is_video_);
    s.store_object_field("discardReason", static_cast<const BaseObject *>(discard_reason_.get()));
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void messageCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageCall").c_str());
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    discard_reason_fieldID = jni::get_field_id(env, Class, "discardReason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallDiscardReason;").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageGroupCall::Class;
jfieldID messageGroupCall::is_active_fieldID;
jfieldID messageGroupCall::was_missed_fieldID;
jfieldID messageGroupCall::is_video_fieldID;
jfieldID messageGroupCall::duration_fieldID;
jfieldID messageGroupCall::other_participant_ids_fieldID;

messageGroupCall::messageGroupCall()
  : is_active_()
  , was_missed_()
  , is_video_()
  , duration_()
  , other_participant_ids_()
{}

messageGroupCall::messageGroupCall(bool is_active_, bool was_missed_, bool is_video_, int32 duration_, array<object_ptr<MessageSender>> &&other_participant_ids_)
  : is_active_(is_active_)
  , was_missed_(was_missed_)
  , is_video_(is_video_)
  , duration_(duration_)
  , other_participant_ids_(std::move(other_participant_ids_))
{}

const std::int32_t messageGroupCall::ID;

object_ptr<MessageContent> messageGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGroupCall> res = make_object<messageGroupCall>();
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  res->was_missed_ = (env->GetBooleanField(p, res->was_missed_fieldID) != 0);
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->other_participant_ids_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->other_participant_ids_fieldID));
  return std::move(res);
}

void messageGroupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_active_fieldID, is_active_);
  env->SetBooleanField(s, was_missed_fieldID, was_missed_);
  env->SetBooleanField(s, is_video_fieldID, is_video_);
  env->SetIntField(s, duration_fieldID, duration_);
  { auto arr_tmp_ = jni::store_vector(env, other_participant_ids_); if (arr_tmp_) { env->SetObjectField(s, other_participant_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGroupCall");
    s.store_field("isActive", is_active_);
    s.store_field("wasMissed", was_missed_);
    s.store_field("isVideo", is_video_);
    s.store_field("duration", duration_);
    { s.store_vector_begin("otherParticipantIds", other_participant_ids_.size()); for (const auto &_value : other_participant_ids_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGroupCall").c_str());
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    was_missed_fieldID = jni::get_field_id(env, Class, "wasMissed", "Z");
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    other_participant_ids_fieldID = jni::get_field_id(env, Class, "otherParticipantIds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageVideoChatScheduled::Class;
jfieldID messageVideoChatScheduled::group_call_id_fieldID;
jfieldID messageVideoChatScheduled::start_date_fieldID;

messageVideoChatScheduled::messageVideoChatScheduled()
  : group_call_id_()
  , start_date_()
{}

messageVideoChatScheduled::messageVideoChatScheduled(int32 group_call_id_, int32 start_date_)
  : group_call_id_(group_call_id_)
  , start_date_(start_date_)
{}

const std::int32_t messageVideoChatScheduled::ID;

object_ptr<MessageContent> messageVideoChatScheduled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVideoChatScheduled> res = make_object<messageVideoChatScheduled>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  return std::move(res);
}

void messageVideoChatScheduled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  env->SetIntField(s, start_date_fieldID, start_date_);
}

void messageVideoChatScheduled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVideoChatScheduled");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("startDate", start_date_);
    s.store_class_end();
  }
}

void messageVideoChatScheduled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVideoChatScheduled").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageVideoChatStarted::Class;
jfieldID messageVideoChatStarted::group_call_id_fieldID;

messageVideoChatStarted::messageVideoChatStarted()
  : group_call_id_()
{}

messageVideoChatStarted::messageVideoChatStarted(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t messageVideoChatStarted::ID;

object_ptr<MessageContent> messageVideoChatStarted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVideoChatStarted> res = make_object<messageVideoChatStarted>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return std::move(res);
}

void messageVideoChatStarted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
}

void messageVideoChatStarted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVideoChatStarted");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

void messageVideoChatStarted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVideoChatStarted").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageVideoChatEnded::Class;
jfieldID messageVideoChatEnded::duration_fieldID;

messageVideoChatEnded::messageVideoChatEnded()
  : duration_()
{}

messageVideoChatEnded::messageVideoChatEnded(int32 duration_)
  : duration_(duration_)
{}

const std::int32_t messageVideoChatEnded::ID;

object_ptr<MessageContent> messageVideoChatEnded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageVideoChatEnded> res = make_object<messageVideoChatEnded>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  return std::move(res);
}

void messageVideoChatEnded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
}

void messageVideoChatEnded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageVideoChatEnded");
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void messageVideoChatEnded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageVideoChatEnded").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageInviteVideoChatParticipants::Class;
jfieldID messageInviteVideoChatParticipants::group_call_id_fieldID;
jfieldID messageInviteVideoChatParticipants::user_ids_fieldID;

messageInviteVideoChatParticipants::messageInviteVideoChatParticipants()
  : group_call_id_()
  , user_ids_()
{}

messageInviteVideoChatParticipants::messageInviteVideoChatParticipants(int32 group_call_id_, array<int53> &&user_ids_)
  : group_call_id_(group_call_id_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t messageInviteVideoChatParticipants::ID;

object_ptr<MessageContent> messageInviteVideoChatParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageInviteVideoChatParticipants> res = make_object<messageInviteVideoChatParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return std::move(res);
}

void messageInviteVideoChatParticipants::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageInviteVideoChatParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageInviteVideoChatParticipants");
    s.store_field("groupCallId", group_call_id_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageInviteVideoChatParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageInviteVideoChatParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass messageBasicGroupChatCreate::Class;
jfieldID messageBasicGroupChatCreate::title_fieldID;
jfieldID messageBasicGroupChatCreate::member_user_ids_fieldID;

messageBasicGroupChatCreate::messageBasicGroupChatCreate()
  : title_()
  , member_user_ids_()
{}

messageBasicGroupChatCreate::messageBasicGroupChatCreate(string const &title_, array<int53> &&member_user_ids_)
  : title_(title_)
  , member_user_ids_(std::move(member_user_ids_))
{}

const std::int32_t messageBasicGroupChatCreate::ID;

object_ptr<MessageContent> messageBasicGroupChatCreate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageBasicGroupChatCreate> res = make_object<messageBasicGroupChatCreate>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->member_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->member_user_ids_fieldID));
  return std::move(res);
}

void messageBasicGroupChatCreate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, member_user_ids_); if (arr_tmp_) { env->SetObjectField(s, member_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageBasicGroupChatCreate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageBasicGroupChatCreate");
    s.store_field("title", title_);
    { s.store_vector_begin("memberUserIds", member_user_ids_.size()); for (const auto &_value : member_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageBasicGroupChatCreate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageBasicGroupChatCreate").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    member_user_ids_fieldID = jni::get_field_id(env, Class, "memberUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass messageSupergroupChatCreate::Class;
jfieldID messageSupergroupChatCreate::title_fieldID;

messageSupergroupChatCreate::messageSupergroupChatCreate()
  : title_()
{}

messageSupergroupChatCreate::messageSupergroupChatCreate(string const &title_)
  : title_(title_)
{}

const std::int32_t messageSupergroupChatCreate::ID;

object_ptr<MessageContent> messageSupergroupChatCreate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSupergroupChatCreate> res = make_object<messageSupergroupChatCreate>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void messageSupergroupChatCreate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageSupergroupChatCreate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSupergroupChatCreate");
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void messageSupergroupChatCreate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSupergroupChatCreate").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatChangeTitle::Class;
jfieldID messageChatChangeTitle::title_fieldID;

messageChatChangeTitle::messageChatChangeTitle()
  : title_()
{}

messageChatChangeTitle::messageChatChangeTitle(string const &title_)
  : title_(title_)
{}

const std::int32_t messageChatChangeTitle::ID;

object_ptr<MessageContent> messageChatChangeTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatChangeTitle> res = make_object<messageChatChangeTitle>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void messageChatChangeTitle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageChatChangeTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatChangeTitle");
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void messageChatChangeTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatChangeTitle").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatChangePhoto::Class;
jfieldID messageChatChangePhoto::photo_fieldID;

messageChatChangePhoto::messageChatChangePhoto()
  : photo_()
{}

messageChatChangePhoto::messageChatChangePhoto(object_ptr<chatPhoto> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t messageChatChangePhoto::ID;

object_ptr<MessageContent> messageChatChangePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatChangePhoto> res = make_object<messageChatChangePhoto>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void messageChatChangePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageChatChangePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatChangePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void messageChatChangePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatChangePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageChatDeletePhoto::Class;

messageChatDeletePhoto::messageChatDeletePhoto() {
}

const std::int32_t messageChatDeletePhoto::ID;

object_ptr<MessageContent> messageChatDeletePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageChatDeletePhoto>();
}

void messageChatDeletePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageChatDeletePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatDeletePhoto");
    s.store_class_end();
  }
}

void messageChatDeletePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatDeletePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageChatAddMembers::Class;
jfieldID messageChatAddMembers::member_user_ids_fieldID;

messageChatAddMembers::messageChatAddMembers()
  : member_user_ids_()
{}

messageChatAddMembers::messageChatAddMembers(array<int53> &&member_user_ids_)
  : member_user_ids_(std::move(member_user_ids_))
{}

const std::int32_t messageChatAddMembers::ID;

object_ptr<MessageContent> messageChatAddMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatAddMembers> res = make_object<messageChatAddMembers>();
  res->member_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->member_user_ids_fieldID));
  return std::move(res);
}

void messageChatAddMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, member_user_ids_); if (arr_tmp_) { env->SetObjectField(s, member_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageChatAddMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatAddMembers");
    { s.store_vector_begin("memberUserIds", member_user_ids_.size()); for (const auto &_value : member_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageChatAddMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatAddMembers").c_str());
    member_user_ids_fieldID = jni::get_field_id(env, Class, "memberUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatJoinByLink::Class;

messageChatJoinByLink::messageChatJoinByLink() {
}

const std::int32_t messageChatJoinByLink::ID;

object_ptr<MessageContent> messageChatJoinByLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageChatJoinByLink>();
}

void messageChatJoinByLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageChatJoinByLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatJoinByLink");
    s.store_class_end();
  }
}

void messageChatJoinByLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatJoinByLink").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageChatJoinByRequest::Class;

messageChatJoinByRequest::messageChatJoinByRequest() {
}

const std::int32_t messageChatJoinByRequest::ID;

object_ptr<MessageContent> messageChatJoinByRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageChatJoinByRequest>();
}

void messageChatJoinByRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageChatJoinByRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatJoinByRequest");
    s.store_class_end();
  }
}

void messageChatJoinByRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatJoinByRequest").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageChatDeleteMember::Class;
jfieldID messageChatDeleteMember::user_id_fieldID;

messageChatDeleteMember::messageChatDeleteMember()
  : user_id_()
{}

messageChatDeleteMember::messageChatDeleteMember(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t messageChatDeleteMember::ID;

object_ptr<MessageContent> messageChatDeleteMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatDeleteMember> res = make_object<messageChatDeleteMember>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void messageChatDeleteMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void messageChatDeleteMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatDeleteMember");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void messageChatDeleteMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatDeleteMember").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatUpgradeTo::Class;
jfieldID messageChatUpgradeTo::supergroup_id_fieldID;

messageChatUpgradeTo::messageChatUpgradeTo()
  : supergroup_id_()
{}

messageChatUpgradeTo::messageChatUpgradeTo(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t messageChatUpgradeTo::ID;

object_ptr<MessageContent> messageChatUpgradeTo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatUpgradeTo> res = make_object<messageChatUpgradeTo>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return std::move(res);
}

void messageChatUpgradeTo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, supergroup_id_fieldID, supergroup_id_);
}

void messageChatUpgradeTo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatUpgradeTo");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

void messageChatUpgradeTo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatUpgradeTo").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatUpgradeFrom::Class;
jfieldID messageChatUpgradeFrom::title_fieldID;
jfieldID messageChatUpgradeFrom::basic_group_id_fieldID;

messageChatUpgradeFrom::messageChatUpgradeFrom()
  : title_()
  , basic_group_id_()
{}

messageChatUpgradeFrom::messageChatUpgradeFrom(string const &title_, int53 basic_group_id_)
  : title_(title_)
  , basic_group_id_(basic_group_id_)
{}

const std::int32_t messageChatUpgradeFrom::ID;

object_ptr<MessageContent> messageChatUpgradeFrom::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatUpgradeFrom> res = make_object<messageChatUpgradeFrom>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  return std::move(res);
}

void messageChatUpgradeFrom::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, basic_group_id_fieldID, basic_group_id_);
}

void messageChatUpgradeFrom::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatUpgradeFrom");
    s.store_field("title", title_);
    s.store_field("basicGroupId", basic_group_id_);
    s.store_class_end();
  }
}

void messageChatUpgradeFrom::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatUpgradeFrom").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messagePinMessage::Class;
jfieldID messagePinMessage::message_id_fieldID;

messagePinMessage::messagePinMessage()
  : message_id_()
{}

messagePinMessage::messagePinMessage(int53 message_id_)
  : message_id_(message_id_)
{}

const std::int32_t messagePinMessage::ID;

object_ptr<MessageContent> messagePinMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePinMessage> res = make_object<messagePinMessage>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void messagePinMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void messagePinMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePinMessage");
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void messagePinMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePinMessage").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageScreenshotTaken::Class;

messageScreenshotTaken::messageScreenshotTaken() {
}

const std::int32_t messageScreenshotTaken::ID;

object_ptr<MessageContent> messageScreenshotTaken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageScreenshotTaken>();
}

void messageScreenshotTaken::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageScreenshotTaken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageScreenshotTaken");
    s.store_class_end();
  }
}

void messageScreenshotTaken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageScreenshotTaken").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageChatSetBackground::Class;
jfieldID messageChatSetBackground::old_background_message_id_fieldID;
jfieldID messageChatSetBackground::background_fieldID;
jfieldID messageChatSetBackground::only_for_self_fieldID;

messageChatSetBackground::messageChatSetBackground()
  : old_background_message_id_()
  , background_()
  , only_for_self_()
{}

messageChatSetBackground::messageChatSetBackground(int53 old_background_message_id_, object_ptr<chatBackground> &&background_, bool only_for_self_)
  : old_background_message_id_(old_background_message_id_)
  , background_(std::move(background_))
  , only_for_self_(only_for_self_)
{}

const std::int32_t messageChatSetBackground::ID;

object_ptr<MessageContent> messageChatSetBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatSetBackground> res = make_object<messageChatSetBackground>();
  res->old_background_message_id_ = env->GetLongField(p, res->old_background_message_id_fieldID);
  res->background_ = jni::fetch_tl_object<chatBackground>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->only_for_self_ = (env->GetBooleanField(p, res->only_for_self_fieldID) != 0);
  return std::move(res);
}

void messageChatSetBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, old_background_message_id_fieldID, old_background_message_id_);
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, only_for_self_fieldID, only_for_self_);
}

void messageChatSetBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatSetBackground");
    s.store_field("oldBackgroundMessageId", old_background_message_id_);
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_field("onlyForSelf", only_for_self_);
    s.store_class_end();
  }
}

void messageChatSetBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatSetBackground").c_str());
    old_background_message_id_fieldID = jni::get_field_id(env, Class, "oldBackgroundMessageId", "J");
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBackground;").c_str());
    only_for_self_fieldID = jni::get_field_id(env, Class, "onlyForSelf", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatSetTheme::Class;
jfieldID messageChatSetTheme::theme_name_fieldID;

messageChatSetTheme::messageChatSetTheme()
  : theme_name_()
{}

messageChatSetTheme::messageChatSetTheme(string const &theme_name_)
  : theme_name_(theme_name_)
{}

const std::int32_t messageChatSetTheme::ID;

object_ptr<MessageContent> messageChatSetTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatSetTheme> res = make_object<messageChatSetTheme>();
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return std::move(res);
}

void messageChatSetTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageChatSetTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatSetTheme");
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

void messageChatSetTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatSetTheme").c_str());
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatSetMessageAutoDeleteTime::Class;
jfieldID messageChatSetMessageAutoDeleteTime::message_auto_delete_time_fieldID;
jfieldID messageChatSetMessageAutoDeleteTime::from_user_id_fieldID;

messageChatSetMessageAutoDeleteTime::messageChatSetMessageAutoDeleteTime()
  : message_auto_delete_time_()
  , from_user_id_()
{}

messageChatSetMessageAutoDeleteTime::messageChatSetMessageAutoDeleteTime(int32 message_auto_delete_time_, int53 from_user_id_)
  : message_auto_delete_time_(message_auto_delete_time_)
  , from_user_id_(from_user_id_)
{}

const std::int32_t messageChatSetMessageAutoDeleteTime::ID;

object_ptr<MessageContent> messageChatSetMessageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatSetMessageAutoDeleteTime> res = make_object<messageChatSetMessageAutoDeleteTime>();
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  res->from_user_id_ = env->GetLongField(p, res->from_user_id_fieldID);
  return std::move(res);
}

void messageChatSetMessageAutoDeleteTime::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, message_auto_delete_time_fieldID, message_auto_delete_time_);
  env->SetLongField(s, from_user_id_fieldID, from_user_id_);
}

void messageChatSetMessageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatSetMessageAutoDeleteTime");
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_field("fromUserId", from_user_id_);
    s.store_class_end();
  }
}

void messageChatSetMessageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatSetMessageAutoDeleteTime").c_str());
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    from_user_id_fieldID = jni::get_field_id(env, Class, "fromUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatBoost::Class;
jfieldID messageChatBoost::boost_count_fieldID;

messageChatBoost::messageChatBoost()
  : boost_count_()
{}

messageChatBoost::messageChatBoost(int32 boost_count_)
  : boost_count_(boost_count_)
{}

const std::int32_t messageChatBoost::ID;

object_ptr<MessageContent> messageChatBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatBoost> res = make_object<messageChatBoost>();
  res->boost_count_ = env->GetIntField(p, res->boost_count_fieldID);
  return std::move(res);
}

void messageChatBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, boost_count_fieldID, boost_count_);
}

void messageChatBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatBoost");
    s.store_field("boostCount", boost_count_);
    s.store_class_end();
  }
}

void messageChatBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatBoost").c_str());
    boost_count_fieldID = jni::get_field_id(env, Class, "boostCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageForumTopicCreated::Class;
jfieldID messageForumTopicCreated::name_fieldID;
jfieldID messageForumTopicCreated::icon_fieldID;

messageForumTopicCreated::messageForumTopicCreated()
  : name_()
  , icon_()
{}

messageForumTopicCreated::messageForumTopicCreated(string const &name_, object_ptr<forumTopicIcon> &&icon_)
  : name_(name_)
  , icon_(std::move(icon_))
{}

const std::int32_t messageForumTopicCreated::ID;

object_ptr<MessageContent> messageForumTopicCreated::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageForumTopicCreated> res = make_object<messageForumTopicCreated>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->icon_ = jni::fetch_tl_object<forumTopicIcon>(env, jni::fetch_object(env, p, res->icon_fieldID));
  return std::move(res);
}

void messageForumTopicCreated::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (icon_ != nullptr) { jobject next; icon_->store(env, next); if (next) { env->SetObjectField(s, icon_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageForumTopicCreated::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageForumTopicCreated");
    s.store_field("name", name_);
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_class_end();
  }
}

void messageForumTopicCreated::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageForumTopicCreated").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicIcon;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageForumTopicEdited::Class;
jfieldID messageForumTopicEdited::name_fieldID;
jfieldID messageForumTopicEdited::edit_icon_custom_emoji_id_fieldID;
jfieldID messageForumTopicEdited::icon_custom_emoji_id_fieldID;

messageForumTopicEdited::messageForumTopicEdited()
  : name_()
  , edit_icon_custom_emoji_id_()
  , icon_custom_emoji_id_()
{}

messageForumTopicEdited::messageForumTopicEdited(string const &name_, bool edit_icon_custom_emoji_id_, int64 icon_custom_emoji_id_)
  : name_(name_)
  , edit_icon_custom_emoji_id_(edit_icon_custom_emoji_id_)
  , icon_custom_emoji_id_(icon_custom_emoji_id_)
{}

const std::int32_t messageForumTopicEdited::ID;

object_ptr<MessageContent> messageForumTopicEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageForumTopicEdited> res = make_object<messageForumTopicEdited>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->edit_icon_custom_emoji_id_ = (env->GetBooleanField(p, res->edit_icon_custom_emoji_id_fieldID) != 0);
  res->icon_custom_emoji_id_ = env->GetLongField(p, res->icon_custom_emoji_id_fieldID);
  return std::move(res);
}

void messageForumTopicEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, edit_icon_custom_emoji_id_fieldID, edit_icon_custom_emoji_id_);
  env->SetLongField(s, icon_custom_emoji_id_fieldID, icon_custom_emoji_id_);
}

void messageForumTopicEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageForumTopicEdited");
    s.store_field("name", name_);
    s.store_field("editIconCustomEmojiId", edit_icon_custom_emoji_id_);
    s.store_field("iconCustomEmojiId", icon_custom_emoji_id_);
    s.store_class_end();
  }
}

void messageForumTopicEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageForumTopicEdited").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    edit_icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "editIconCustomEmojiId", "Z");
    icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "iconCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageForumTopicIsClosedToggled::Class;
jfieldID messageForumTopicIsClosedToggled::is_closed_fieldID;

messageForumTopicIsClosedToggled::messageForumTopicIsClosedToggled()
  : is_closed_()
{}

messageForumTopicIsClosedToggled::messageForumTopicIsClosedToggled(bool is_closed_)
  : is_closed_(is_closed_)
{}

const std::int32_t messageForumTopicIsClosedToggled::ID;

object_ptr<MessageContent> messageForumTopicIsClosedToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageForumTopicIsClosedToggled> res = make_object<messageForumTopicIsClosedToggled>();
  res->is_closed_ = (env->GetBooleanField(p, res->is_closed_fieldID) != 0);
  return std::move(res);
}

void messageForumTopicIsClosedToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_closed_fieldID, is_closed_);
}

void messageForumTopicIsClosedToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageForumTopicIsClosedToggled");
    s.store_field("isClosed", is_closed_);
    s.store_class_end();
  }
}

void messageForumTopicIsClosedToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageForumTopicIsClosedToggled").c_str());
    is_closed_fieldID = jni::get_field_id(env, Class, "isClosed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageForumTopicIsHiddenToggled::Class;
jfieldID messageForumTopicIsHiddenToggled::is_hidden_fieldID;

messageForumTopicIsHiddenToggled::messageForumTopicIsHiddenToggled()
  : is_hidden_()
{}

messageForumTopicIsHiddenToggled::messageForumTopicIsHiddenToggled(bool is_hidden_)
  : is_hidden_(is_hidden_)
{}

const std::int32_t messageForumTopicIsHiddenToggled::ID;

object_ptr<MessageContent> messageForumTopicIsHiddenToggled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageForumTopicIsHiddenToggled> res = make_object<messageForumTopicIsHiddenToggled>();
  res->is_hidden_ = (env->GetBooleanField(p, res->is_hidden_fieldID) != 0);
  return std::move(res);
}

void messageForumTopicIsHiddenToggled::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_hidden_fieldID, is_hidden_);
}

void messageForumTopicIsHiddenToggled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageForumTopicIsHiddenToggled");
    s.store_field("isHidden", is_hidden_);
    s.store_class_end();
  }
}

void messageForumTopicIsHiddenToggled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageForumTopicIsHiddenToggled").c_str());
    is_hidden_fieldID = jni::get_field_id(env, Class, "isHidden", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageSuggestProfilePhoto::Class;
jfieldID messageSuggestProfilePhoto::photo_fieldID;

messageSuggestProfilePhoto::messageSuggestProfilePhoto()
  : photo_()
{}

messageSuggestProfilePhoto::messageSuggestProfilePhoto(object_ptr<chatPhoto> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t messageSuggestProfilePhoto::ID;

object_ptr<MessageContent> messageSuggestProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSuggestProfilePhoto> res = make_object<messageSuggestProfilePhoto>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void messageSuggestProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageSuggestProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSuggestProfilePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void messageSuggestProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSuggestProfilePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageCustomServiceAction::Class;
jfieldID messageCustomServiceAction::text_fieldID;

messageCustomServiceAction::messageCustomServiceAction()
  : text_()
{}

messageCustomServiceAction::messageCustomServiceAction(string const &text_)
  : text_(text_)
{}

const std::int32_t messageCustomServiceAction::ID;

object_ptr<MessageContent> messageCustomServiceAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageCustomServiceAction> res = make_object<messageCustomServiceAction>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return std::move(res);
}

void messageCustomServiceAction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageCustomServiceAction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageCustomServiceAction");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void messageCustomServiceAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageCustomServiceAction").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageGameScore::Class;
jfieldID messageGameScore::game_message_id_fieldID;
jfieldID messageGameScore::game_id_fieldID;
jfieldID messageGameScore::score_fieldID;

messageGameScore::messageGameScore()
  : game_message_id_()
  , game_id_()
  , score_()
{}

messageGameScore::messageGameScore(int53 game_message_id_, int64 game_id_, int32 score_)
  : game_message_id_(game_message_id_)
  , game_id_(game_id_)
  , score_(score_)
{}

const std::int32_t messageGameScore::ID;

object_ptr<MessageContent> messageGameScore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGameScore> res = make_object<messageGameScore>();
  res->game_message_id_ = env->GetLongField(p, res->game_message_id_fieldID);
  res->game_id_ = env->GetLongField(p, res->game_id_fieldID);
  res->score_ = env->GetIntField(p, res->score_fieldID);
  return std::move(res);
}

void messageGameScore::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, game_message_id_fieldID, game_message_id_);
  env->SetLongField(s, game_id_fieldID, game_id_);
  env->SetIntField(s, score_fieldID, score_);
}

void messageGameScore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGameScore");
    s.store_field("gameMessageId", game_message_id_);
    s.store_field("gameId", game_id_);
    s.store_field("score", score_);
    s.store_class_end();
  }
}

void messageGameScore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGameScore").c_str());
    game_message_id_fieldID = jni::get_field_id(env, Class, "gameMessageId", "J");
    game_id_fieldID = jni::get_field_id(env, Class, "gameId", "J");
    score_fieldID = jni::get_field_id(env, Class, "score", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messagePaymentSuccessful::Class;
jfieldID messagePaymentSuccessful::invoice_chat_id_fieldID;
jfieldID messagePaymentSuccessful::invoice_message_id_fieldID;
jfieldID messagePaymentSuccessful::currency_fieldID;
jfieldID messagePaymentSuccessful::total_amount_fieldID;
jfieldID messagePaymentSuccessful::subscription_until_date_fieldID;
jfieldID messagePaymentSuccessful::is_recurring_fieldID;
jfieldID messagePaymentSuccessful::is_first_recurring_fieldID;
jfieldID messagePaymentSuccessful::invoice_name_fieldID;

messagePaymentSuccessful::messagePaymentSuccessful()
  : invoice_chat_id_()
  , invoice_message_id_()
  , currency_()
  , total_amount_()
  , subscription_until_date_()
  , is_recurring_()
  , is_first_recurring_()
  , invoice_name_()
{}

messagePaymentSuccessful::messagePaymentSuccessful(int53 invoice_chat_id_, int53 invoice_message_id_, string const &currency_, int53 total_amount_, int32 subscription_until_date_, bool is_recurring_, bool is_first_recurring_, string const &invoice_name_)
  : invoice_chat_id_(invoice_chat_id_)
  , invoice_message_id_(invoice_message_id_)
  , currency_(currency_)
  , total_amount_(total_amount_)
  , subscription_until_date_(subscription_until_date_)
  , is_recurring_(is_recurring_)
  , is_first_recurring_(is_first_recurring_)
  , invoice_name_(invoice_name_)
{}

const std::int32_t messagePaymentSuccessful::ID;

object_ptr<MessageContent> messagePaymentSuccessful::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaymentSuccessful> res = make_object<messagePaymentSuccessful>();
  res->invoice_chat_id_ = env->GetLongField(p, res->invoice_chat_id_fieldID);
  res->invoice_message_id_ = env->GetLongField(p, res->invoice_message_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->subscription_until_date_ = env->GetIntField(p, res->subscription_until_date_fieldID);
  res->is_recurring_ = (env->GetBooleanField(p, res->is_recurring_fieldID) != 0);
  res->is_first_recurring_ = (env->GetBooleanField(p, res->is_first_recurring_fieldID) != 0);
  res->invoice_name_ = jni::fetch_string(env, p, res->invoice_name_fieldID);
  return std::move(res);
}

void messagePaymentSuccessful::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, invoice_chat_id_fieldID, invoice_chat_id_);
  env->SetLongField(s, invoice_message_id_fieldID, invoice_message_id_);
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  env->SetIntField(s, subscription_until_date_fieldID, subscription_until_date_);
  env->SetBooleanField(s, is_recurring_fieldID, is_recurring_);
  env->SetBooleanField(s, is_first_recurring_fieldID, is_first_recurring_);
  { jstring nextString = jni::to_jstring(env, invoice_name_); if (nextString) { env->SetObjectField(s, invoice_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messagePaymentSuccessful::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaymentSuccessful");
    s.store_field("invoiceChatId", invoice_chat_id_);
    s.store_field("invoiceMessageId", invoice_message_id_);
    s.store_field("currency", currency_);
    s.store_field("totalAmount", total_amount_);
    s.store_field("subscriptionUntilDate", subscription_until_date_);
    s.store_field("isRecurring", is_recurring_);
    s.store_field("isFirstRecurring", is_first_recurring_);
    s.store_field("invoiceName", invoice_name_);
    s.store_class_end();
  }
}

void messagePaymentSuccessful::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaymentSuccessful").c_str());
    invoice_chat_id_fieldID = jni::get_field_id(env, Class, "invoiceChatId", "J");
    invoice_message_id_fieldID = jni::get_field_id(env, Class, "invoiceMessageId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    subscription_until_date_fieldID = jni::get_field_id(env, Class, "subscriptionUntilDate", "I");
    is_recurring_fieldID = jni::get_field_id(env, Class, "isRecurring", "Z");
    is_first_recurring_fieldID = jni::get_field_id(env, Class, "isFirstRecurring", "Z");
    invoice_name_fieldID = jni::get_field_id(env, Class, "invoiceName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messagePaymentSuccessfulBot::Class;
jfieldID messagePaymentSuccessfulBot::currency_fieldID;
jfieldID messagePaymentSuccessfulBot::total_amount_fieldID;
jfieldID messagePaymentSuccessfulBot::subscription_until_date_fieldID;
jfieldID messagePaymentSuccessfulBot::is_recurring_fieldID;
jfieldID messagePaymentSuccessfulBot::is_first_recurring_fieldID;
jfieldID messagePaymentSuccessfulBot::invoice_payload_fieldID;
jfieldID messagePaymentSuccessfulBot::shipping_option_id_fieldID;
jfieldID messagePaymentSuccessfulBot::order_info_fieldID;
jfieldID messagePaymentSuccessfulBot::telegram_payment_charge_id_fieldID;
jfieldID messagePaymentSuccessfulBot::provider_payment_charge_id_fieldID;

messagePaymentSuccessfulBot::messagePaymentSuccessfulBot()
  : currency_()
  , total_amount_()
  , subscription_until_date_()
  , is_recurring_()
  , is_first_recurring_()
  , invoice_payload_()
  , shipping_option_id_()
  , order_info_()
  , telegram_payment_charge_id_()
  , provider_payment_charge_id_()
{}

messagePaymentSuccessfulBot::messagePaymentSuccessfulBot(string const &currency_, int53 total_amount_, int32 subscription_until_date_, bool is_recurring_, bool is_first_recurring_, bytes const &invoice_payload_, string const &shipping_option_id_, object_ptr<orderInfo> &&order_info_, string const &telegram_payment_charge_id_, string const &provider_payment_charge_id_)
  : currency_(currency_)
  , total_amount_(total_amount_)
  , subscription_until_date_(subscription_until_date_)
  , is_recurring_(is_recurring_)
  , is_first_recurring_(is_first_recurring_)
  , invoice_payload_(std::move(invoice_payload_))
  , shipping_option_id_(shipping_option_id_)
  , order_info_(std::move(order_info_))
  , telegram_payment_charge_id_(telegram_payment_charge_id_)
  , provider_payment_charge_id_(provider_payment_charge_id_)
{}

const std::int32_t messagePaymentSuccessfulBot::ID;

object_ptr<MessageContent> messagePaymentSuccessfulBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaymentSuccessfulBot> res = make_object<messagePaymentSuccessfulBot>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->subscription_until_date_ = env->GetIntField(p, res->subscription_until_date_fieldID);
  res->is_recurring_ = (env->GetBooleanField(p, res->is_recurring_fieldID) != 0);
  res->is_first_recurring_ = (env->GetBooleanField(p, res->is_first_recurring_fieldID) != 0);
  res->invoice_payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->invoice_payload_fieldID));
  res->shipping_option_id_ = jni::fetch_string(env, p, res->shipping_option_id_fieldID);
  res->order_info_ = jni::fetch_tl_object<orderInfo>(env, jni::fetch_object(env, p, res->order_info_fieldID));
  res->telegram_payment_charge_id_ = jni::fetch_string(env, p, res->telegram_payment_charge_id_fieldID);
  res->provider_payment_charge_id_ = jni::fetch_string(env, p, res->provider_payment_charge_id_fieldID);
  return std::move(res);
}

void messagePaymentSuccessfulBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  env->SetIntField(s, subscription_until_date_fieldID, subscription_until_date_);
  env->SetBooleanField(s, is_recurring_fieldID, is_recurring_);
  env->SetBooleanField(s, is_first_recurring_fieldID, is_first_recurring_);
  { jbyteArray nextBytes = jni::to_bytes(env, invoice_payload_); if (nextBytes) { env->SetObjectField(s, invoice_payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, shipping_option_id_); if (nextString) { env->SetObjectField(s, shipping_option_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (order_info_ != nullptr) { jobject next; order_info_->store(env, next); if (next) { env->SetObjectField(s, order_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, telegram_payment_charge_id_); if (nextString) { env->SetObjectField(s, telegram_payment_charge_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, provider_payment_charge_id_); if (nextString) { env->SetObjectField(s, provider_payment_charge_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messagePaymentSuccessfulBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaymentSuccessfulBot");
    s.store_field("currency", currency_);
    s.store_field("totalAmount", total_amount_);
    s.store_field("subscriptionUntilDate", subscription_until_date_);
    s.store_field("isRecurring", is_recurring_);
    s.store_field("isFirstRecurring", is_first_recurring_);
    s.store_bytes_field("invoicePayload", invoice_payload_);
    s.store_field("shippingOptionId", shipping_option_id_);
    s.store_object_field("orderInfo", static_cast<const BaseObject *>(order_info_.get()));
    s.store_field("telegramPaymentChargeId", telegram_payment_charge_id_);
    s.store_field("providerPaymentChargeId", provider_payment_charge_id_);
    s.store_class_end();
  }
}

void messagePaymentSuccessfulBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaymentSuccessfulBot").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    subscription_until_date_fieldID = jni::get_field_id(env, Class, "subscriptionUntilDate", "I");
    is_recurring_fieldID = jni::get_field_id(env, Class, "isRecurring", "Z");
    is_first_recurring_fieldID = jni::get_field_id(env, Class, "isFirstRecurring", "Z");
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "[B");
    shipping_option_id_fieldID = jni::get_field_id(env, Class, "shippingOptionId", "Ljava/lang/String;");
    order_info_fieldID = jni::get_field_id(env, Class, "orderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OrderInfo;").c_str());
    telegram_payment_charge_id_fieldID = jni::get_field_id(env, Class, "telegramPaymentChargeId", "Ljava/lang/String;");
    provider_payment_charge_id_fieldID = jni::get_field_id(env, Class, "providerPaymentChargeId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messagePaymentRefunded::Class;
jfieldID messagePaymentRefunded::owner_id_fieldID;
jfieldID messagePaymentRefunded::currency_fieldID;
jfieldID messagePaymentRefunded::total_amount_fieldID;
jfieldID messagePaymentRefunded::invoice_payload_fieldID;
jfieldID messagePaymentRefunded::telegram_payment_charge_id_fieldID;
jfieldID messagePaymentRefunded::provider_payment_charge_id_fieldID;

messagePaymentRefunded::messagePaymentRefunded()
  : owner_id_()
  , currency_()
  , total_amount_()
  , invoice_payload_()
  , telegram_payment_charge_id_()
  , provider_payment_charge_id_()
{}

messagePaymentRefunded::messagePaymentRefunded(object_ptr<MessageSender> &&owner_id_, string const &currency_, int53 total_amount_, bytes const &invoice_payload_, string const &telegram_payment_charge_id_, string const &provider_payment_charge_id_)
  : owner_id_(std::move(owner_id_))
  , currency_(currency_)
  , total_amount_(total_amount_)
  , invoice_payload_(std::move(invoice_payload_))
  , telegram_payment_charge_id_(telegram_payment_charge_id_)
  , provider_payment_charge_id_(provider_payment_charge_id_)
{}

const std::int32_t messagePaymentRefunded::ID;

object_ptr<MessageContent> messagePaymentRefunded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaymentRefunded> res = make_object<messagePaymentRefunded>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->invoice_payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->invoice_payload_fieldID));
  res->telegram_payment_charge_id_ = jni::fetch_string(env, p, res->telegram_payment_charge_id_fieldID);
  res->provider_payment_charge_id_ = jni::fetch_string(env, p, res->provider_payment_charge_id_fieldID);
  return std::move(res);
}

void messagePaymentRefunded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (owner_id_ != nullptr) { jobject next; owner_id_->store(env, next); if (next) { env->SetObjectField(s, owner_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  { jbyteArray nextBytes = jni::to_bytes(env, invoice_payload_); if (nextBytes) { env->SetObjectField(s, invoice_payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, telegram_payment_charge_id_); if (nextString) { env->SetObjectField(s, telegram_payment_charge_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, provider_payment_charge_id_); if (nextString) { env->SetObjectField(s, provider_payment_charge_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messagePaymentRefunded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaymentRefunded");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("currency", currency_);
    s.store_field("totalAmount", total_amount_);
    s.store_bytes_field("invoicePayload", invoice_payload_);
    s.store_field("telegramPaymentChargeId", telegram_payment_charge_id_);
    s.store_field("providerPaymentChargeId", provider_payment_charge_id_);
    s.store_class_end();
  }
}

void messagePaymentRefunded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaymentRefunded").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "[B");
    telegram_payment_charge_id_fieldID = jni::get_field_id(env, Class, "telegramPaymentChargeId", "Ljava/lang/String;");
    provider_payment_charge_id_fieldID = jni::get_field_id(env, Class, "providerPaymentChargeId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageGiftedPremium::Class;
jfieldID messageGiftedPremium::gifter_user_id_fieldID;
jfieldID messageGiftedPremium::receiver_user_id_fieldID;
jfieldID messageGiftedPremium::text_fieldID;
jfieldID messageGiftedPremium::currency_fieldID;
jfieldID messageGiftedPremium::amount_fieldID;
jfieldID messageGiftedPremium::cryptocurrency_fieldID;
jfieldID messageGiftedPremium::cryptocurrency_amount_fieldID;
jfieldID messageGiftedPremium::month_count_fieldID;
jfieldID messageGiftedPremium::sticker_fieldID;

messageGiftedPremium::messageGiftedPremium()
  : gifter_user_id_()
  , receiver_user_id_()
  , text_()
  , currency_()
  , amount_()
  , cryptocurrency_()
  , cryptocurrency_amount_()
  , month_count_()
  , sticker_()
{}

messageGiftedPremium::messageGiftedPremium(int53 gifter_user_id_, int53 receiver_user_id_, object_ptr<formattedText> &&text_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int32 month_count_, object_ptr<sticker> &&sticker_)
  : gifter_user_id_(gifter_user_id_)
  , receiver_user_id_(receiver_user_id_)
  , text_(std::move(text_))
  , currency_(currency_)
  , amount_(amount_)
  , cryptocurrency_(cryptocurrency_)
  , cryptocurrency_amount_(cryptocurrency_amount_)
  , month_count_(month_count_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t messageGiftedPremium::ID;

object_ptr<MessageContent> messageGiftedPremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiftedPremium> res = make_object<messageGiftedPremium>();
  res->gifter_user_id_ = env->GetLongField(p, res->gifter_user_id_fieldID);
  res->receiver_user_id_ = env->GetLongField(p, res->receiver_user_id_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->cryptocurrency_amount_ = env->GetLongField(p, res->cryptocurrency_amount_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void messageGiftedPremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, gifter_user_id_fieldID, gifter_user_id_);
  env->SetLongField(s, receiver_user_id_fieldID, receiver_user_id_);
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cryptocurrency_amount_fieldID, cryptocurrency_amount_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageGiftedPremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiftedPremium");
    s.store_field("gifterUserId", gifter_user_id_);
    s.store_field("receiverUserId", receiver_user_id_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("cryptocurrencyAmount", cryptocurrency_amount_);
    s.store_field("monthCount", month_count_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void messageGiftedPremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiftedPremium").c_str());
    gifter_user_id_fieldID = jni::get_field_id(env, Class, "gifterUserId", "J");
    receiver_user_id_fieldID = jni::get_field_id(env, Class, "receiverUserId", "J");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    cryptocurrency_amount_fieldID = jni::get_field_id(env, Class, "cryptocurrencyAmount", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messagePremiumGiftCode::Class;
jfieldID messagePremiumGiftCode::creator_id_fieldID;
jfieldID messagePremiumGiftCode::text_fieldID;
jfieldID messagePremiumGiftCode::is_from_giveaway_fieldID;
jfieldID messagePremiumGiftCode::is_unclaimed_fieldID;
jfieldID messagePremiumGiftCode::currency_fieldID;
jfieldID messagePremiumGiftCode::amount_fieldID;
jfieldID messagePremiumGiftCode::cryptocurrency_fieldID;
jfieldID messagePremiumGiftCode::cryptocurrency_amount_fieldID;
jfieldID messagePremiumGiftCode::month_count_fieldID;
jfieldID messagePremiumGiftCode::sticker_fieldID;
jfieldID messagePremiumGiftCode::code_fieldID;

messagePremiumGiftCode::messagePremiumGiftCode()
  : creator_id_()
  , text_()
  , is_from_giveaway_()
  , is_unclaimed_()
  , currency_()
  , amount_()
  , cryptocurrency_()
  , cryptocurrency_amount_()
  , month_count_()
  , sticker_()
  , code_()
{}

messagePremiumGiftCode::messagePremiumGiftCode(object_ptr<MessageSender> &&creator_id_, object_ptr<formattedText> &&text_, bool is_from_giveaway_, bool is_unclaimed_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int32 month_count_, object_ptr<sticker> &&sticker_, string const &code_)
  : creator_id_(std::move(creator_id_))
  , text_(std::move(text_))
  , is_from_giveaway_(is_from_giveaway_)
  , is_unclaimed_(is_unclaimed_)
  , currency_(currency_)
  , amount_(amount_)
  , cryptocurrency_(cryptocurrency_)
  , cryptocurrency_amount_(cryptocurrency_amount_)
  , month_count_(month_count_)
  , sticker_(std::move(sticker_))
  , code_(code_)
{}

const std::int32_t messagePremiumGiftCode::ID;

object_ptr<MessageContent> messagePremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePremiumGiftCode> res = make_object<messagePremiumGiftCode>();
  res->creator_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->creator_id_fieldID));
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->is_from_giveaway_ = (env->GetBooleanField(p, res->is_from_giveaway_fieldID) != 0);
  res->is_unclaimed_ = (env->GetBooleanField(p, res->is_unclaimed_fieldID) != 0);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->cryptocurrency_amount_ = env->GetLongField(p, res->cryptocurrency_amount_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return std::move(res);
}

void messagePremiumGiftCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (creator_id_ != nullptr) { jobject next; creator_id_->store(env, next); if (next) { env->SetObjectField(s, creator_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_from_giveaway_fieldID, is_from_giveaway_);
  env->SetBooleanField(s, is_unclaimed_fieldID, is_unclaimed_);
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cryptocurrency_amount_fieldID, cryptocurrency_amount_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, code_); if (nextString) { env->SetObjectField(s, code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messagePremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePremiumGiftCode");
    s.store_object_field("creatorId", static_cast<const BaseObject *>(creator_id_.get()));
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("isFromGiveaway", is_from_giveaway_);
    s.store_field("isUnclaimed", is_unclaimed_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("cryptocurrencyAmount", cryptocurrency_amount_);
    s.store_field("monthCount", month_count_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("code", code_);
    s.store_class_end();
  }
}

void messagePremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePremiumGiftCode").c_str());
    creator_id_fieldID = jni::get_field_id(env, Class, "creatorId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    is_from_giveaway_fieldID = jni::get_field_id(env, Class, "isFromGiveaway", "Z");
    is_unclaimed_fieldID = jni::get_field_id(env, Class, "isUnclaimed", "Z");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    cryptocurrency_amount_fieldID = jni::get_field_id(env, Class, "cryptocurrencyAmount", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageGiveawayCreated::Class;
jfieldID messageGiveawayCreated::star_count_fieldID;

messageGiveawayCreated::messageGiveawayCreated()
  : star_count_()
{}

messageGiveawayCreated::messageGiveawayCreated(int53 star_count_)
  : star_count_(star_count_)
{}

const std::int32_t messageGiveawayCreated::ID;

object_ptr<MessageContent> messageGiveawayCreated::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiveawayCreated> res = make_object<messageGiveawayCreated>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void messageGiveawayCreated::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void messageGiveawayCreated::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiveawayCreated");
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void messageGiveawayCreated::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiveawayCreated").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageGiveaway::Class;
jfieldID messageGiveaway::parameters_fieldID;
jfieldID messageGiveaway::winner_count_fieldID;
jfieldID messageGiveaway::prize_fieldID;
jfieldID messageGiveaway::sticker_fieldID;

messageGiveaway::messageGiveaway()
  : parameters_()
  , winner_count_()
  , prize_()
  , sticker_()
{}

messageGiveaway::messageGiveaway(object_ptr<giveawayParameters> &&parameters_, int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, object_ptr<sticker> &&sticker_)
  : parameters_(std::move(parameters_))
  , winner_count_(winner_count_)
  , prize_(std::move(prize_))
  , sticker_(std::move(sticker_))
{}

const std::int32_t messageGiveaway::ID;

object_ptr<MessageContent> messageGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiveaway> res = make_object<messageGiveaway>();
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->prize_ = jni::fetch_tl_object<GiveawayPrize>(env, jni::fetch_object(env, p, res->prize_fieldID));
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void messageGiveaway::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (parameters_ != nullptr) { jobject next; parameters_->store(env, next); if (next) { env->SetObjectField(s, parameters_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  if (prize_ != nullptr) { jobject next; prize_->store(env, next); if (next) { env->SetObjectField(s, prize_fieldID, next); env->DeleteLocalRef(next); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiveaway");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("winnerCount", winner_count_);
    s.store_object_field("prize", static_cast<const BaseObject *>(prize_.get()));
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void messageGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiveaway").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    prize_fieldID = jni::get_field_id(env, Class, "prize", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayPrize;").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageGiveawayCompleted::Class;
jfieldID messageGiveawayCompleted::giveaway_message_id_fieldID;
jfieldID messageGiveawayCompleted::winner_count_fieldID;
jfieldID messageGiveawayCompleted::is_star_giveaway_fieldID;
jfieldID messageGiveawayCompleted::unclaimed_prize_count_fieldID;

messageGiveawayCompleted::messageGiveawayCompleted()
  : giveaway_message_id_()
  , winner_count_()
  , is_star_giveaway_()
  , unclaimed_prize_count_()
{}

messageGiveawayCompleted::messageGiveawayCompleted(int53 giveaway_message_id_, int32 winner_count_, bool is_star_giveaway_, int32 unclaimed_prize_count_)
  : giveaway_message_id_(giveaway_message_id_)
  , winner_count_(winner_count_)
  , is_star_giveaway_(is_star_giveaway_)
  , unclaimed_prize_count_(unclaimed_prize_count_)
{}

const std::int32_t messageGiveawayCompleted::ID;

object_ptr<MessageContent> messageGiveawayCompleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiveawayCompleted> res = make_object<messageGiveawayCompleted>();
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->is_star_giveaway_ = (env->GetBooleanField(p, res->is_star_giveaway_fieldID) != 0);
  res->unclaimed_prize_count_ = env->GetIntField(p, res->unclaimed_prize_count_fieldID);
  return std::move(res);
}

void messageGiveawayCompleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  env->SetBooleanField(s, is_star_giveaway_fieldID, is_star_giveaway_);
  env->SetIntField(s, unclaimed_prize_count_fieldID, unclaimed_prize_count_);
}

void messageGiveawayCompleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiveawayCompleted");
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("isStarGiveaway", is_star_giveaway_);
    s.store_field("unclaimedPrizeCount", unclaimed_prize_count_);
    s.store_class_end();
  }
}

void messageGiveawayCompleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiveawayCompleted").c_str());
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    is_star_giveaway_fieldID = jni::get_field_id(env, Class, "isStarGiveaway", "Z");
    unclaimed_prize_count_fieldID = jni::get_field_id(env, Class, "unclaimedPrizeCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageGiveawayWinners::Class;
jfieldID messageGiveawayWinners::boosted_chat_id_fieldID;
jfieldID messageGiveawayWinners::giveaway_message_id_fieldID;
jfieldID messageGiveawayWinners::additional_chat_count_fieldID;
jfieldID messageGiveawayWinners::actual_winners_selection_date_fieldID;
jfieldID messageGiveawayWinners::only_new_members_fieldID;
jfieldID messageGiveawayWinners::was_refunded_fieldID;
jfieldID messageGiveawayWinners::prize_fieldID;
jfieldID messageGiveawayWinners::prize_description_fieldID;
jfieldID messageGiveawayWinners::winner_count_fieldID;
jfieldID messageGiveawayWinners::winner_user_ids_fieldID;
jfieldID messageGiveawayWinners::unclaimed_prize_count_fieldID;

messageGiveawayWinners::messageGiveawayWinners()
  : boosted_chat_id_()
  , giveaway_message_id_()
  , additional_chat_count_()
  , actual_winners_selection_date_()
  , only_new_members_()
  , was_refunded_()
  , prize_()
  , prize_description_()
  , winner_count_()
  , winner_user_ids_()
  , unclaimed_prize_count_()
{}

messageGiveawayWinners::messageGiveawayWinners(int53 boosted_chat_id_, int53 giveaway_message_id_, int32 additional_chat_count_, int32 actual_winners_selection_date_, bool only_new_members_, bool was_refunded_, object_ptr<GiveawayPrize> &&prize_, string const &prize_description_, int32 winner_count_, array<int53> &&winner_user_ids_, int32 unclaimed_prize_count_)
  : boosted_chat_id_(boosted_chat_id_)
  , giveaway_message_id_(giveaway_message_id_)
  , additional_chat_count_(additional_chat_count_)
  , actual_winners_selection_date_(actual_winners_selection_date_)
  , only_new_members_(only_new_members_)
  , was_refunded_(was_refunded_)
  , prize_(std::move(prize_))
  , prize_description_(prize_description_)
  , winner_count_(winner_count_)
  , winner_user_ids_(std::move(winner_user_ids_))
  , unclaimed_prize_count_(unclaimed_prize_count_)
{}

const std::int32_t messageGiveawayWinners::ID;

object_ptr<MessageContent> messageGiveawayWinners::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiveawayWinners> res = make_object<messageGiveawayWinners>();
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  res->additional_chat_count_ = env->GetIntField(p, res->additional_chat_count_fieldID);
  res->actual_winners_selection_date_ = env->GetIntField(p, res->actual_winners_selection_date_fieldID);
  res->only_new_members_ = (env->GetBooleanField(p, res->only_new_members_fieldID) != 0);
  res->was_refunded_ = (env->GetBooleanField(p, res->was_refunded_fieldID) != 0);
  res->prize_ = jni::fetch_tl_object<GiveawayPrize>(env, jni::fetch_object(env, p, res->prize_fieldID));
  res->prize_description_ = jni::fetch_string(env, p, res->prize_description_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->winner_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->winner_user_ids_fieldID));
  res->unclaimed_prize_count_ = env->GetIntField(p, res->unclaimed_prize_count_fieldID);
  return std::move(res);
}

void messageGiveawayWinners::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, boosted_chat_id_fieldID, boosted_chat_id_);
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
  env->SetIntField(s, additional_chat_count_fieldID, additional_chat_count_);
  env->SetIntField(s, actual_winners_selection_date_fieldID, actual_winners_selection_date_);
  env->SetBooleanField(s, only_new_members_fieldID, only_new_members_);
  env->SetBooleanField(s, was_refunded_fieldID, was_refunded_);
  if (prize_ != nullptr) { jobject next; prize_->store(env, next); if (next) { env->SetObjectField(s, prize_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, prize_description_); if (nextString) { env->SetObjectField(s, prize_description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  { auto arr_tmp_ = jni::store_vector(env, winner_user_ids_); if (arr_tmp_) { env->SetObjectField(s, winner_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, unclaimed_prize_count_fieldID, unclaimed_prize_count_);
}

void messageGiveawayWinners::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiveawayWinners");
    s.store_field("boostedChatId", boosted_chat_id_);
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_field("additionalChatCount", additional_chat_count_);
    s.store_field("actualWinnersSelectionDate", actual_winners_selection_date_);
    s.store_field("onlyNewMembers", only_new_members_);
    s.store_field("wasRefunded", was_refunded_);
    s.store_object_field("prize", static_cast<const BaseObject *>(prize_.get()));
    s.store_field("prizeDescription", prize_description_);
    s.store_field("winnerCount", winner_count_);
    { s.store_vector_begin("winnerUserIds", winner_user_ids_.size()); for (const auto &_value : winner_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("unclaimedPrizeCount", unclaimed_prize_count_);
    s.store_class_end();
  }
}

void messageGiveawayWinners::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiveawayWinners").c_str());
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    additional_chat_count_fieldID = jni::get_field_id(env, Class, "additionalChatCount", "I");
    actual_winners_selection_date_fieldID = jni::get_field_id(env, Class, "actualWinnersSelectionDate", "I");
    only_new_members_fieldID = jni::get_field_id(env, Class, "onlyNewMembers", "Z");
    was_refunded_fieldID = jni::get_field_id(env, Class, "wasRefunded", "Z");
    prize_fieldID = jni::get_field_id(env, Class, "prize", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayPrize;").c_str());
    prize_description_fieldID = jni::get_field_id(env, Class, "prizeDescription", "Ljava/lang/String;");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    winner_user_ids_fieldID = jni::get_field_id(env, Class, "winnerUserIds", "[J");
    unclaimed_prize_count_fieldID = jni::get_field_id(env, Class, "unclaimedPrizeCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageGiftedStars::Class;
jfieldID messageGiftedStars::gifter_user_id_fieldID;
jfieldID messageGiftedStars::receiver_user_id_fieldID;
jfieldID messageGiftedStars::currency_fieldID;
jfieldID messageGiftedStars::amount_fieldID;
jfieldID messageGiftedStars::cryptocurrency_fieldID;
jfieldID messageGiftedStars::cryptocurrency_amount_fieldID;
jfieldID messageGiftedStars::star_count_fieldID;
jfieldID messageGiftedStars::transaction_id_fieldID;
jfieldID messageGiftedStars::sticker_fieldID;

messageGiftedStars::messageGiftedStars()
  : gifter_user_id_()
  , receiver_user_id_()
  , currency_()
  , amount_()
  , cryptocurrency_()
  , cryptocurrency_amount_()
  , star_count_()
  , transaction_id_()
  , sticker_()
{}

messageGiftedStars::messageGiftedStars(int53 gifter_user_id_, int53 receiver_user_id_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int53 star_count_, string const &transaction_id_, object_ptr<sticker> &&sticker_)
  : gifter_user_id_(gifter_user_id_)
  , receiver_user_id_(receiver_user_id_)
  , currency_(currency_)
  , amount_(amount_)
  , cryptocurrency_(cryptocurrency_)
  , cryptocurrency_amount_(cryptocurrency_amount_)
  , star_count_(star_count_)
  , transaction_id_(transaction_id_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t messageGiftedStars::ID;

object_ptr<MessageContent> messageGiftedStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiftedStars> res = make_object<messageGiftedStars>();
  res->gifter_user_id_ = env->GetLongField(p, res->gifter_user_id_fieldID);
  res->receiver_user_id_ = env->GetLongField(p, res->receiver_user_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->cryptocurrency_ = jni::fetch_string(env, p, res->cryptocurrency_fieldID);
  res->cryptocurrency_amount_ = env->GetLongField(p, res->cryptocurrency_amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->transaction_id_ = jni::fetch_string(env, p, res->transaction_id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void messageGiftedStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, gifter_user_id_fieldID, gifter_user_id_);
  env->SetLongField(s, receiver_user_id_fieldID, receiver_user_id_);
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  { jstring nextString = jni::to_jstring(env, cryptocurrency_); if (nextString) { env->SetObjectField(s, cryptocurrency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cryptocurrency_amount_fieldID, cryptocurrency_amount_);
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, transaction_id_); if (nextString) { env->SetObjectField(s, transaction_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageGiftedStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiftedStars");
    s.store_field("gifterUserId", gifter_user_id_);
    s.store_field("receiverUserId", receiver_user_id_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("cryptocurrency", cryptocurrency_);
    s.store_field("cryptocurrencyAmount", cryptocurrency_amount_);
    s.store_field("starCount", star_count_);
    s.store_field("transactionId", transaction_id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void messageGiftedStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiftedStars").c_str());
    gifter_user_id_fieldID = jni::get_field_id(env, Class, "gifterUserId", "J");
    receiver_user_id_fieldID = jni::get_field_id(env, Class, "receiverUserId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    cryptocurrency_fieldID = jni::get_field_id(env, Class, "cryptocurrency", "Ljava/lang/String;");
    cryptocurrency_amount_fieldID = jni::get_field_id(env, Class, "cryptocurrencyAmount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    transaction_id_fieldID = jni::get_field_id(env, Class, "transactionId", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageGiveawayPrizeStars::Class;
jfieldID messageGiveawayPrizeStars::star_count_fieldID;
jfieldID messageGiveawayPrizeStars::transaction_id_fieldID;
jfieldID messageGiveawayPrizeStars::boosted_chat_id_fieldID;
jfieldID messageGiveawayPrizeStars::giveaway_message_id_fieldID;
jfieldID messageGiveawayPrizeStars::is_unclaimed_fieldID;
jfieldID messageGiveawayPrizeStars::sticker_fieldID;

messageGiveawayPrizeStars::messageGiveawayPrizeStars()
  : star_count_()
  , transaction_id_()
  , boosted_chat_id_()
  , giveaway_message_id_()
  , is_unclaimed_()
  , sticker_()
{}

messageGiveawayPrizeStars::messageGiveawayPrizeStars(int53 star_count_, string const &transaction_id_, int53 boosted_chat_id_, int53 giveaway_message_id_, bool is_unclaimed_, object_ptr<sticker> &&sticker_)
  : star_count_(star_count_)
  , transaction_id_(transaction_id_)
  , boosted_chat_id_(boosted_chat_id_)
  , giveaway_message_id_(giveaway_message_id_)
  , is_unclaimed_(is_unclaimed_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t messageGiveawayPrizeStars::ID;

object_ptr<MessageContent> messageGiveawayPrizeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGiveawayPrizeStars> res = make_object<messageGiveawayPrizeStars>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->transaction_id_ = jni::fetch_string(env, p, res->transaction_id_fieldID);
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  res->is_unclaimed_ = (env->GetBooleanField(p, res->is_unclaimed_fieldID) != 0);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void messageGiveawayPrizeStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, transaction_id_); if (nextString) { env->SetObjectField(s, transaction_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, boosted_chat_id_fieldID, boosted_chat_id_);
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
  env->SetBooleanField(s, is_unclaimed_fieldID, is_unclaimed_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageGiveawayPrizeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGiveawayPrizeStars");
    s.store_field("starCount", star_count_);
    s.store_field("transactionId", transaction_id_);
    s.store_field("boostedChatId", boosted_chat_id_);
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_field("isUnclaimed", is_unclaimed_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void messageGiveawayPrizeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGiveawayPrizeStars").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    transaction_id_fieldID = jni::get_field_id(env, Class, "transactionId", "Ljava/lang/String;");
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    is_unclaimed_fieldID = jni::get_field_id(env, Class, "isUnclaimed", "Z");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageGift::Class;
jfieldID messageGift::gift_fieldID;
jfieldID messageGift::sender_id_fieldID;
jfieldID messageGift::received_gift_id_fieldID;
jfieldID messageGift::text_fieldID;
jfieldID messageGift::sell_star_count_fieldID;
jfieldID messageGift::prepaid_upgrade_star_count_fieldID;
jfieldID messageGift::is_private_fieldID;
jfieldID messageGift::is_saved_fieldID;
jfieldID messageGift::can_be_upgraded_fieldID;
jfieldID messageGift::was_converted_fieldID;
jfieldID messageGift::was_upgraded_fieldID;
jfieldID messageGift::was_refunded_fieldID;
jfieldID messageGift::upgraded_received_gift_id_fieldID;

messageGift::messageGift()
  : gift_()
  , sender_id_()
  , received_gift_id_()
  , text_()
  , sell_star_count_()
  , prepaid_upgrade_star_count_()
  , is_private_()
  , is_saved_()
  , can_be_upgraded_()
  , was_converted_()
  , was_upgraded_()
  , was_refunded_()
  , upgraded_received_gift_id_()
{}

messageGift::messageGift(object_ptr<gift> &&gift_, object_ptr<MessageSender> &&sender_id_, string const &received_gift_id_, object_ptr<formattedText> &&text_, int53 sell_star_count_, int53 prepaid_upgrade_star_count_, bool is_private_, bool is_saved_, bool can_be_upgraded_, bool was_converted_, bool was_upgraded_, bool was_refunded_, string const &upgraded_received_gift_id_)
  : gift_(std::move(gift_))
  , sender_id_(std::move(sender_id_))
  , received_gift_id_(received_gift_id_)
  , text_(std::move(text_))
  , sell_star_count_(sell_star_count_)
  , prepaid_upgrade_star_count_(prepaid_upgrade_star_count_)
  , is_private_(is_private_)
  , is_saved_(is_saved_)
  , can_be_upgraded_(can_be_upgraded_)
  , was_converted_(was_converted_)
  , was_upgraded_(was_upgraded_)
  , was_refunded_(was_refunded_)
  , upgraded_received_gift_id_(upgraded_received_gift_id_)
{}

const std::int32_t messageGift::ID;

object_ptr<MessageContent> messageGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageGift> res = make_object<messageGift>();
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->sell_star_count_ = env->GetLongField(p, res->sell_star_count_fieldID);
  res->prepaid_upgrade_star_count_ = env->GetLongField(p, res->prepaid_upgrade_star_count_fieldID);
  res->is_private_ = (env->GetBooleanField(p, res->is_private_fieldID) != 0);
  res->is_saved_ = (env->GetBooleanField(p, res->is_saved_fieldID) != 0);
  res->can_be_upgraded_ = (env->GetBooleanField(p, res->can_be_upgraded_fieldID) != 0);
  res->was_converted_ = (env->GetBooleanField(p, res->was_converted_fieldID) != 0);
  res->was_upgraded_ = (env->GetBooleanField(p, res->was_upgraded_fieldID) != 0);
  res->was_refunded_ = (env->GetBooleanField(p, res->was_refunded_fieldID) != 0);
  res->upgraded_received_gift_id_ = jni::fetch_string(env, p, res->upgraded_received_gift_id_fieldID);
  return std::move(res);
}

void messageGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, received_gift_id_); if (nextString) { env->SetObjectField(s, received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, sell_star_count_fieldID, sell_star_count_);
  env->SetLongField(s, prepaid_upgrade_star_count_fieldID, prepaid_upgrade_star_count_);
  env->SetBooleanField(s, is_private_fieldID, is_private_);
  env->SetBooleanField(s, is_saved_fieldID, is_saved_);
  env->SetBooleanField(s, can_be_upgraded_fieldID, can_be_upgraded_);
  env->SetBooleanField(s, was_converted_fieldID, was_converted_);
  env->SetBooleanField(s, was_upgraded_fieldID, was_upgraded_);
  env->SetBooleanField(s, was_refunded_fieldID, was_refunded_);
  { jstring nextString = jni::to_jstring(env, upgraded_received_gift_id_); if (nextString) { env->SetObjectField(s, upgraded_received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageGift");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("sellStarCount", sell_star_count_);
    s.store_field("prepaidUpgradeStarCount", prepaid_upgrade_star_count_);
    s.store_field("isPrivate", is_private_);
    s.store_field("isSaved", is_saved_);
    s.store_field("canBeUpgraded", can_be_upgraded_);
    s.store_field("wasConverted", was_converted_);
    s.store_field("wasUpgraded", was_upgraded_);
    s.store_field("wasRefunded", was_refunded_);
    s.store_field("upgradedReceivedGiftId", upgraded_received_gift_id_);
    s.store_class_end();
  }
}

void messageGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageGift").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    sell_star_count_fieldID = jni::get_field_id(env, Class, "sellStarCount", "J");
    prepaid_upgrade_star_count_fieldID = jni::get_field_id(env, Class, "prepaidUpgradeStarCount", "J");
    is_private_fieldID = jni::get_field_id(env, Class, "isPrivate", "Z");
    is_saved_fieldID = jni::get_field_id(env, Class, "isSaved", "Z");
    can_be_upgraded_fieldID = jni::get_field_id(env, Class, "canBeUpgraded", "Z");
    was_converted_fieldID = jni::get_field_id(env, Class, "wasConverted", "Z");
    was_upgraded_fieldID = jni::get_field_id(env, Class, "wasUpgraded", "Z");
    was_refunded_fieldID = jni::get_field_id(env, Class, "wasRefunded", "Z");
    upgraded_received_gift_id_fieldID = jni::get_field_id(env, Class, "upgradedReceivedGiftId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageUpgradedGift::Class;
jfieldID messageUpgradedGift::gift_fieldID;
jfieldID messageUpgradedGift::sender_id_fieldID;
jfieldID messageUpgradedGift::received_gift_id_fieldID;
jfieldID messageUpgradedGift::is_upgrade_fieldID;
jfieldID messageUpgradedGift::is_saved_fieldID;
jfieldID messageUpgradedGift::can_be_transferred_fieldID;
jfieldID messageUpgradedGift::was_transferred_fieldID;
jfieldID messageUpgradedGift::last_resale_star_count_fieldID;
jfieldID messageUpgradedGift::transfer_star_count_fieldID;
jfieldID messageUpgradedGift::next_transfer_date_fieldID;
jfieldID messageUpgradedGift::next_resale_date_fieldID;
jfieldID messageUpgradedGift::export_date_fieldID;

messageUpgradedGift::messageUpgradedGift()
  : gift_()
  , sender_id_()
  , received_gift_id_()
  , is_upgrade_()
  , is_saved_()
  , can_be_transferred_()
  , was_transferred_()
  , last_resale_star_count_()
  , transfer_star_count_()
  , next_transfer_date_()
  , next_resale_date_()
  , export_date_()
{}

messageUpgradedGift::messageUpgradedGift(object_ptr<upgradedGift> &&gift_, object_ptr<MessageSender> &&sender_id_, string const &received_gift_id_, bool is_upgrade_, bool is_saved_, bool can_be_transferred_, bool was_transferred_, int53 last_resale_star_count_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_)
  : gift_(std::move(gift_))
  , sender_id_(std::move(sender_id_))
  , received_gift_id_(received_gift_id_)
  , is_upgrade_(is_upgrade_)
  , is_saved_(is_saved_)
  , can_be_transferred_(can_be_transferred_)
  , was_transferred_(was_transferred_)
  , last_resale_star_count_(last_resale_star_count_)
  , transfer_star_count_(transfer_star_count_)
  , next_transfer_date_(next_transfer_date_)
  , next_resale_date_(next_resale_date_)
  , export_date_(export_date_)
{}

const std::int32_t messageUpgradedGift::ID;

object_ptr<MessageContent> messageUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageUpgradedGift> res = make_object<messageUpgradedGift>();
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->is_upgrade_ = (env->GetBooleanField(p, res->is_upgrade_fieldID) != 0);
  res->is_saved_ = (env->GetBooleanField(p, res->is_saved_fieldID) != 0);
  res->can_be_transferred_ = (env->GetBooleanField(p, res->can_be_transferred_fieldID) != 0);
  res->was_transferred_ = (env->GetBooleanField(p, res->was_transferred_fieldID) != 0);
  res->last_resale_star_count_ = env->GetLongField(p, res->last_resale_star_count_fieldID);
  res->transfer_star_count_ = env->GetLongField(p, res->transfer_star_count_fieldID);
  res->next_transfer_date_ = env->GetIntField(p, res->next_transfer_date_fieldID);
  res->next_resale_date_ = env->GetIntField(p, res->next_resale_date_fieldID);
  res->export_date_ = env->GetIntField(p, res->export_date_fieldID);
  return std::move(res);
}

void messageUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, received_gift_id_); if (nextString) { env->SetObjectField(s, received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_upgrade_fieldID, is_upgrade_);
  env->SetBooleanField(s, is_saved_fieldID, is_saved_);
  env->SetBooleanField(s, can_be_transferred_fieldID, can_be_transferred_);
  env->SetBooleanField(s, was_transferred_fieldID, was_transferred_);
  env->SetLongField(s, last_resale_star_count_fieldID, last_resale_star_count_);
  env->SetLongField(s, transfer_star_count_fieldID, transfer_star_count_);
  env->SetIntField(s, next_transfer_date_fieldID, next_transfer_date_);
  env->SetIntField(s, next_resale_date_fieldID, next_resale_date_);
  env->SetIntField(s, export_date_fieldID, export_date_);
}

void messageUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageUpgradedGift");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("isUpgrade", is_upgrade_);
    s.store_field("isSaved", is_saved_);
    s.store_field("canBeTransferred", can_be_transferred_);
    s.store_field("wasTransferred", was_transferred_);
    s.store_field("lastResaleStarCount", last_resale_star_count_);
    s.store_field("transferStarCount", transfer_star_count_);
    s.store_field("nextTransferDate", next_transfer_date_);
    s.store_field("nextResaleDate", next_resale_date_);
    s.store_field("exportDate", export_date_);
    s.store_class_end();
  }
}

void messageUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageUpgradedGift").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    is_upgrade_fieldID = jni::get_field_id(env, Class, "isUpgrade", "Z");
    is_saved_fieldID = jni::get_field_id(env, Class, "isSaved", "Z");
    can_be_transferred_fieldID = jni::get_field_id(env, Class, "canBeTransferred", "Z");
    was_transferred_fieldID = jni::get_field_id(env, Class, "wasTransferred", "Z");
    last_resale_star_count_fieldID = jni::get_field_id(env, Class, "lastResaleStarCount", "J");
    transfer_star_count_fieldID = jni::get_field_id(env, Class, "transferStarCount", "J");
    next_transfer_date_fieldID = jni::get_field_id(env, Class, "nextTransferDate", "I");
    next_resale_date_fieldID = jni::get_field_id(env, Class, "nextResaleDate", "I");
    export_date_fieldID = jni::get_field_id(env, Class, "exportDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageRefundedUpgradedGift::Class;
jfieldID messageRefundedUpgradedGift::gift_fieldID;
jfieldID messageRefundedUpgradedGift::sender_id_fieldID;
jfieldID messageRefundedUpgradedGift::is_upgrade_fieldID;

messageRefundedUpgradedGift::messageRefundedUpgradedGift()
  : gift_()
  , sender_id_()
  , is_upgrade_()
{}

messageRefundedUpgradedGift::messageRefundedUpgradedGift(object_ptr<gift> &&gift_, object_ptr<MessageSender> &&sender_id_, bool is_upgrade_)
  : gift_(std::move(gift_))
  , sender_id_(std::move(sender_id_))
  , is_upgrade_(is_upgrade_)
{}

const std::int32_t messageRefundedUpgradedGift::ID;

object_ptr<MessageContent> messageRefundedUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageRefundedUpgradedGift> res = make_object<messageRefundedUpgradedGift>();
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->is_upgrade_ = (env->GetBooleanField(p, res->is_upgrade_fieldID) != 0);
  return std::move(res);
}

void messageRefundedUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_upgrade_fieldID, is_upgrade_);
}

void messageRefundedUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageRefundedUpgradedGift");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("isUpgrade", is_upgrade_);
    s.store_class_end();
  }
}

void messageRefundedUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageRefundedUpgradedGift").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_upgrade_fieldID = jni::get_field_id(env, Class, "isUpgrade", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messagePaidMessagesRefunded::Class;
jfieldID messagePaidMessagesRefunded::message_count_fieldID;
jfieldID messagePaidMessagesRefunded::star_count_fieldID;

messagePaidMessagesRefunded::messagePaidMessagesRefunded()
  : message_count_()
  , star_count_()
{}

messagePaidMessagesRefunded::messagePaidMessagesRefunded(int32 message_count_, int53 star_count_)
  : message_count_(message_count_)
  , star_count_(star_count_)
{}

const std::int32_t messagePaidMessagesRefunded::ID;

object_ptr<MessageContent> messagePaidMessagesRefunded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaidMessagesRefunded> res = make_object<messagePaidMessagesRefunded>();
  res->message_count_ = env->GetIntField(p, res->message_count_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void messagePaidMessagesRefunded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, message_count_fieldID, message_count_);
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void messagePaidMessagesRefunded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaidMessagesRefunded");
    s.store_field("messageCount", message_count_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void messagePaidMessagesRefunded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaidMessagesRefunded").c_str());
    message_count_fieldID = jni::get_field_id(env, Class, "messageCount", "I");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messagePaidMessagePriceChanged::Class;
jfieldID messagePaidMessagePriceChanged::paid_message_star_count_fieldID;

messagePaidMessagePriceChanged::messagePaidMessagePriceChanged()
  : paid_message_star_count_()
{}

messagePaidMessagePriceChanged::messagePaidMessagePriceChanged(int53 paid_message_star_count_)
  : paid_message_star_count_(paid_message_star_count_)
{}

const std::int32_t messagePaidMessagePriceChanged::ID;

object_ptr<MessageContent> messagePaidMessagePriceChanged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePaidMessagePriceChanged> res = make_object<messagePaidMessagePriceChanged>();
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  return std::move(res);
}

void messagePaidMessagePriceChanged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, paid_message_star_count_fieldID, paid_message_star_count_);
}

void messagePaidMessagePriceChanged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePaidMessagePriceChanged");
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_class_end();
  }
}

void messagePaidMessagePriceChanged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePaidMessagePriceChanged").c_str());
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageContactRegistered::Class;

messageContactRegistered::messageContactRegistered() {
}

const std::int32_t messageContactRegistered::ID;

object_ptr<MessageContent> messageContactRegistered::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageContactRegistered>();
}

void messageContactRegistered::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageContactRegistered::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageContactRegistered");
    s.store_class_end();
  }
}

void messageContactRegistered::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageContactRegistered").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageUsersShared::Class;
jfieldID messageUsersShared::users_fieldID;
jfieldID messageUsersShared::button_id_fieldID;

messageUsersShared::messageUsersShared()
  : users_()
  , button_id_()
{}

messageUsersShared::messageUsersShared(array<object_ptr<sharedUser>> &&users_, int32 button_id_)
  : users_(std::move(users_))
  , button_id_(button_id_)
{}

const std::int32_t messageUsersShared::ID;

object_ptr<MessageContent> messageUsersShared::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageUsersShared> res = make_object<messageUsersShared>();
  res->users_ = jni::FetchVector<sharedUser>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->users_fieldID));
  res->button_id_ = env->GetIntField(p, res->button_id_fieldID);
  return std::move(res);
}

void messageUsersShared::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, users_); if (arr_tmp_) { env->SetObjectField(s, users_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, button_id_fieldID, button_id_);
}

void messageUsersShared::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageUsersShared");
    { s.store_vector_begin("users", users_.size()); for (const auto &_value : users_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("buttonId", button_id_);
    s.store_class_end();
  }
}

void messageUsersShared::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageUsersShared").c_str());
    users_fieldID = jni::get_field_id(env, Class, "users", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SharedUser;").c_str());
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageChatShared::Class;
jfieldID messageChatShared::chat_fieldID;
jfieldID messageChatShared::button_id_fieldID;

messageChatShared::messageChatShared()
  : chat_()
  , button_id_()
{}

messageChatShared::messageChatShared(object_ptr<sharedChat> &&chat_, int32 button_id_)
  : chat_(std::move(chat_))
  , button_id_(button_id_)
{}

const std::int32_t messageChatShared::ID;

object_ptr<MessageContent> messageChatShared::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageChatShared> res = make_object<messageChatShared>();
  res->chat_ = jni::fetch_tl_object<sharedChat>(env, jni::fetch_object(env, p, res->chat_fieldID));
  res->button_id_ = env->GetIntField(p, res->button_id_fieldID);
  return std::move(res);
}

void messageChatShared::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chat_ != nullptr) { jobject next; chat_->store(env, next); if (next) { env->SetObjectField(s, chat_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, button_id_fieldID, button_id_);
}

void messageChatShared::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageChatShared");
    s.store_object_field("chat", static_cast<const BaseObject *>(chat_.get()));
    s.store_field("buttonId", button_id_);
    s.store_class_end();
  }
}

void messageChatShared::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageChatShared").c_str());
    chat_fieldID = jni::get_field_id(env, Class, "chat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SharedChat;").c_str());
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageBotWriteAccessAllowed::Class;
jfieldID messageBotWriteAccessAllowed::reason_fieldID;

messageBotWriteAccessAllowed::messageBotWriteAccessAllowed()
  : reason_()
{}

messageBotWriteAccessAllowed::messageBotWriteAccessAllowed(object_ptr<BotWriteAccessAllowReason> &&reason_)
  : reason_(std::move(reason_))
{}

const std::int32_t messageBotWriteAccessAllowed::ID;

object_ptr<MessageContent> messageBotWriteAccessAllowed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageBotWriteAccessAllowed> res = make_object<messageBotWriteAccessAllowed>();
  res->reason_ = jni::fetch_tl_object<BotWriteAccessAllowReason>(env, jni::fetch_object(env, p, res->reason_fieldID));
  return std::move(res);
}

void messageBotWriteAccessAllowed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (reason_ != nullptr) { jobject next; reason_->store(env, next); if (next) { env->SetObjectField(s, reason_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageBotWriteAccessAllowed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageBotWriteAccessAllowed");
    s.store_object_field("reason", static_cast<const BaseObject *>(reason_.get()));
    s.store_class_end();
  }
}

void messageBotWriteAccessAllowed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageBotWriteAccessAllowed").c_str());
    reason_fieldID = jni::get_field_id(env, Class, "reason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotWriteAccessAllowReason;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageWebAppDataSent::Class;
jfieldID messageWebAppDataSent::button_text_fieldID;

messageWebAppDataSent::messageWebAppDataSent()
  : button_text_()
{}

messageWebAppDataSent::messageWebAppDataSent(string const &button_text_)
  : button_text_(button_text_)
{}

const std::int32_t messageWebAppDataSent::ID;

object_ptr<MessageContent> messageWebAppDataSent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageWebAppDataSent> res = make_object<messageWebAppDataSent>();
  res->button_text_ = jni::fetch_string(env, p, res->button_text_fieldID);
  return std::move(res);
}

void messageWebAppDataSent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, button_text_); if (nextString) { env->SetObjectField(s, button_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageWebAppDataSent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageWebAppDataSent");
    s.store_field("buttonText", button_text_);
    s.store_class_end();
  }
}

void messageWebAppDataSent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageWebAppDataSent").c_str());
    button_text_fieldID = jni::get_field_id(env, Class, "buttonText", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageWebAppDataReceived::Class;
jfieldID messageWebAppDataReceived::button_text_fieldID;
jfieldID messageWebAppDataReceived::data_fieldID;

messageWebAppDataReceived::messageWebAppDataReceived()
  : button_text_()
  , data_()
{}

messageWebAppDataReceived::messageWebAppDataReceived(string const &button_text_, string const &data_)
  : button_text_(button_text_)
  , data_(data_)
{}

const std::int32_t messageWebAppDataReceived::ID;

object_ptr<MessageContent> messageWebAppDataReceived::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageWebAppDataReceived> res = make_object<messageWebAppDataReceived>();
  res->button_text_ = jni::fetch_string(env, p, res->button_text_fieldID);
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  return std::move(res);
}

void messageWebAppDataReceived::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, button_text_); if (nextString) { env->SetObjectField(s, button_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, data_); if (nextString) { env->SetObjectField(s, data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageWebAppDataReceived::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageWebAppDataReceived");
    s.store_field("buttonText", button_text_);
    s.store_field("data", data_);
    s.store_class_end();
  }
}

void messageWebAppDataReceived::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageWebAppDataReceived").c_str());
    button_text_fieldID = jni::get_field_id(env, Class, "buttonText", "Ljava/lang/String;");
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messagePassportDataSent::Class;
jfieldID messagePassportDataSent::types_fieldID;

messagePassportDataSent::messagePassportDataSent()
  : types_()
{}

messagePassportDataSent::messagePassportDataSent(array<object_ptr<PassportElementType>> &&types_)
  : types_(std::move(types_))
{}

const std::int32_t messagePassportDataSent::ID;

object_ptr<MessageContent> messagePassportDataSent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePassportDataSent> res = make_object<messagePassportDataSent>();
  res->types_ = jni::FetchVector<PassportElementType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->types_fieldID));
  return std::move(res);
}

void messagePassportDataSent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, types_); if (arr_tmp_) { env->SetObjectField(s, types_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messagePassportDataSent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePassportDataSent");
    { s.store_vector_begin("types", types_.size()); for (const auto &_value : types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messagePassportDataSent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePassportDataSent").c_str());
    types_fieldID = jni::get_field_id(env, Class, "types", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messagePassportDataReceived::Class;
jfieldID messagePassportDataReceived::elements_fieldID;
jfieldID messagePassportDataReceived::credentials_fieldID;

messagePassportDataReceived::messagePassportDataReceived()
  : elements_()
  , credentials_()
{}

messagePassportDataReceived::messagePassportDataReceived(array<object_ptr<encryptedPassportElement>> &&elements_, object_ptr<encryptedCredentials> &&credentials_)
  : elements_(std::move(elements_))
  , credentials_(std::move(credentials_))
{}

const std::int32_t messagePassportDataReceived::ID;

object_ptr<MessageContent> messagePassportDataReceived::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePassportDataReceived> res = make_object<messagePassportDataReceived>();
  res->elements_ = jni::FetchVector<encryptedPassportElement>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->elements_fieldID));
  res->credentials_ = jni::fetch_tl_object<encryptedCredentials>(env, jni::fetch_object(env, p, res->credentials_fieldID));
  return std::move(res);
}

void messagePassportDataReceived::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, elements_); if (arr_tmp_) { env->SetObjectField(s, elements_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (credentials_ != nullptr) { jobject next; credentials_->store(env, next); if (next) { env->SetObjectField(s, credentials_fieldID, next); env->DeleteLocalRef(next); } }
}

void messagePassportDataReceived::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePassportDataReceived");
    { s.store_vector_begin("elements", elements_.size()); for (const auto &_value : elements_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("credentials", static_cast<const BaseObject *>(credentials_.get()));
    s.store_class_end();
  }
}

void messagePassportDataReceived::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePassportDataReceived").c_str());
    elements_fieldID = jni::get_field_id(env, Class, "elements", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$EncryptedPassportElement;").c_str());
    credentials_fieldID = jni::get_field_id(env, Class, "credentials", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EncryptedCredentials;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageProximityAlertTriggered::Class;
jfieldID messageProximityAlertTriggered::traveler_id_fieldID;
jfieldID messageProximityAlertTriggered::watcher_id_fieldID;
jfieldID messageProximityAlertTriggered::distance_fieldID;

messageProximityAlertTriggered::messageProximityAlertTriggered()
  : traveler_id_()
  , watcher_id_()
  , distance_()
{}

messageProximityAlertTriggered::messageProximityAlertTriggered(object_ptr<MessageSender> &&traveler_id_, object_ptr<MessageSender> &&watcher_id_, int32 distance_)
  : traveler_id_(std::move(traveler_id_))
  , watcher_id_(std::move(watcher_id_))
  , distance_(distance_)
{}

const std::int32_t messageProximityAlertTriggered::ID;

object_ptr<MessageContent> messageProximityAlertTriggered::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageProximityAlertTriggered> res = make_object<messageProximityAlertTriggered>();
  res->traveler_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->traveler_id_fieldID));
  res->watcher_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->watcher_id_fieldID));
  res->distance_ = env->GetIntField(p, res->distance_fieldID);
  return std::move(res);
}

void messageProximityAlertTriggered::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (traveler_id_ != nullptr) { jobject next; traveler_id_->store(env, next); if (next) { env->SetObjectField(s, traveler_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (watcher_id_ != nullptr) { jobject next; watcher_id_->store(env, next); if (next) { env->SetObjectField(s, watcher_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, distance_fieldID, distance_);
}

void messageProximityAlertTriggered::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageProximityAlertTriggered");
    s.store_object_field("travelerId", static_cast<const BaseObject *>(traveler_id_.get()));
    s.store_object_field("watcherId", static_cast<const BaseObject *>(watcher_id_.get()));
    s.store_field("distance", distance_);
    s.store_class_end();
  }
}

void messageProximityAlertTriggered::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageProximityAlertTriggered").c_str());
    traveler_id_fieldID = jni::get_field_id(env, Class, "travelerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    watcher_id_fieldID = jni::get_field_id(env, Class, "watcherId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    distance_fieldID = jni::get_field_id(env, Class, "distance", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageUnsupported::Class;

messageUnsupported::messageUnsupported() {
}

const std::int32_t messageUnsupported::ID;

object_ptr<MessageContent> messageUnsupported::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageUnsupported>();
}

void messageUnsupported::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageUnsupported::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageUnsupported");
    s.store_class_end();
  }
}

void messageUnsupported::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageUnsupported").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageCopyOptions::Class;
jfieldID messageCopyOptions::send_copy_fieldID;
jfieldID messageCopyOptions::replace_caption_fieldID;
jfieldID messageCopyOptions::new_caption_fieldID;
jfieldID messageCopyOptions::new_show_caption_above_media_fieldID;

messageCopyOptions::messageCopyOptions()
  : send_copy_()
  , replace_caption_()
  , new_caption_()
  , new_show_caption_above_media_()
{}

messageCopyOptions::messageCopyOptions(bool send_copy_, bool replace_caption_, object_ptr<formattedText> &&new_caption_, bool new_show_caption_above_media_)
  : send_copy_(send_copy_)
  , replace_caption_(replace_caption_)
  , new_caption_(std::move(new_caption_))
  , new_show_caption_above_media_(new_show_caption_above_media_)
{}

const std::int32_t messageCopyOptions::ID;

object_ptr<messageCopyOptions> messageCopyOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageCopyOptions> res = make_object<messageCopyOptions>();
  res->send_copy_ = (env->GetBooleanField(p, res->send_copy_fieldID) != 0);
  res->replace_caption_ = (env->GetBooleanField(p, res->replace_caption_fieldID) != 0);
  res->new_caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->new_caption_fieldID));
  res->new_show_caption_above_media_ = (env->GetBooleanField(p, res->new_show_caption_above_media_fieldID) != 0);
  return res;
}

void messageCopyOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, send_copy_fieldID, send_copy_);
  env->SetBooleanField(s, replace_caption_fieldID, replace_caption_);
  if (new_caption_ != nullptr) { jobject next; new_caption_->store(env, next); if (next) { env->SetObjectField(s, new_caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, new_show_caption_above_media_fieldID, new_show_caption_above_media_);
}

void messageCopyOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageCopyOptions");
    s.store_field("sendCopy", send_copy_);
    s.store_field("replaceCaption", replace_caption_);
    s.store_object_field("newCaption", static_cast<const BaseObject *>(new_caption_.get()));
    s.store_field("newShowCaptionAboveMedia", new_show_caption_above_media_);
    s.store_class_end();
  }
}

void messageCopyOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageCopyOptions").c_str());
    send_copy_fieldID = jni::get_field_id(env, Class, "sendCopy", "Z");
    replace_caption_fieldID = jni::get_field_id(env, Class, "replaceCaption", "Z");
    new_caption_fieldID = jni::get_field_id(env, Class, "newCaption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    new_show_caption_above_media_fieldID = jni::get_field_id(env, Class, "newShowCaptionAboveMedia", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageEffect::Class;
jfieldID messageEffect::id_fieldID;
jfieldID messageEffect::static_icon_fieldID;
jfieldID messageEffect::emoji_fieldID;
jfieldID messageEffect::is_premium_fieldID;
jfieldID messageEffect::type_fieldID;

messageEffect::messageEffect()
  : id_()
  , static_icon_()
  , emoji_()
  , is_premium_()
  , type_()
{}

messageEffect::messageEffect(int64 id_, object_ptr<sticker> &&static_icon_, string const &emoji_, bool is_premium_, object_ptr<MessageEffectType> &&type_)
  : id_(id_)
  , static_icon_(std::move(static_icon_))
  , emoji_(emoji_)
  , is_premium_(is_premium_)
  , type_(std::move(type_))
{}

const std::int32_t messageEffect::ID;

object_ptr<messageEffect> messageEffect::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageEffect> res = make_object<messageEffect>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->static_icon_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->static_icon_fieldID));
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->is_premium_ = (env->GetBooleanField(p, res->is_premium_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<MessageEffectType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void messageEffect::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (static_icon_ != nullptr) { jobject next; static_icon_->store(env, next); if (next) { env->SetObjectField(s, static_icon_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_premium_fieldID, is_premium_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageEffect::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageEffect");
    s.store_field("id", id_);
    s.store_object_field("staticIcon", static_cast<const BaseObject *>(static_icon_.get()));
    s.store_field("emoji", emoji_);
    s.store_field("isPremium", is_premium_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void messageEffect::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageEffect").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    static_icon_fieldID = jni::get_field_id(env, Class, "staticIcon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    is_premium_fieldID = jni::get_field_id(env, Class, "isPremium", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageEffectType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass MessageEffectType::Class;

object_ptr<MessageEffectType> MessageEffectType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageEffectTypeEmojiReaction::ID:
      return messageEffectTypeEmojiReaction::fetch(env, p);
    case messageEffectTypePremiumSticker::ID:
      return messageEffectTypePremiumSticker::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageEffectType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageEffectType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageEffectTypeEmojiReaction::Class;
jfieldID messageEffectTypeEmojiReaction::select_animation_fieldID;
jfieldID messageEffectTypeEmojiReaction::effect_animation_fieldID;

messageEffectTypeEmojiReaction::messageEffectTypeEmojiReaction()
  : select_animation_()
  , effect_animation_()
{}

messageEffectTypeEmojiReaction::messageEffectTypeEmojiReaction(object_ptr<sticker> &&select_animation_, object_ptr<sticker> &&effect_animation_)
  : select_animation_(std::move(select_animation_))
  , effect_animation_(std::move(effect_animation_))
{}

const std::int32_t messageEffectTypeEmojiReaction::ID;

object_ptr<MessageEffectType> messageEffectTypeEmojiReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageEffectTypeEmojiReaction> res = make_object<messageEffectTypeEmojiReaction>();
  res->select_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->select_animation_fieldID));
  res->effect_animation_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->effect_animation_fieldID));
  return std::move(res);
}

void messageEffectTypeEmojiReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (select_animation_ != nullptr) { jobject next; select_animation_->store(env, next); if (next) { env->SetObjectField(s, select_animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (effect_animation_ != nullptr) { jobject next; effect_animation_->store(env, next); if (next) { env->SetObjectField(s, effect_animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageEffectTypeEmojiReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageEffectTypeEmojiReaction");
    s.store_object_field("selectAnimation", static_cast<const BaseObject *>(select_animation_.get()));
    s.store_object_field("effectAnimation", static_cast<const BaseObject *>(effect_animation_.get()));
    s.store_class_end();
  }
}

void messageEffectTypeEmojiReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageEffectTypeEmojiReaction").c_str());
    select_animation_fieldID = jni::get_field_id(env, Class, "selectAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    effect_animation_fieldID = jni::get_field_id(env, Class, "effectAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageEffectTypePremiumSticker::Class;
jfieldID messageEffectTypePremiumSticker::sticker_fieldID;

messageEffectTypePremiumSticker::messageEffectTypePremiumSticker()
  : sticker_()
{}

messageEffectTypePremiumSticker::messageEffectTypePremiumSticker(object_ptr<sticker> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t messageEffectTypePremiumSticker::ID;

object_ptr<MessageEffectType> messageEffectTypePremiumSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageEffectTypePremiumSticker> res = make_object<messageEffectTypePremiumSticker>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void messageEffectTypePremiumSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageEffectTypePremiumSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageEffectTypePremiumSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void messageEffectTypePremiumSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageEffectTypePremiumSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass MessageFileType::Class;

object_ptr<MessageFileType> MessageFileType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageFileTypePrivate::ID:
      return messageFileTypePrivate::fetch(env, p);
    case messageFileTypeGroup::ID:
      return messageFileTypeGroup::fetch(env, p);
    case messageFileTypeUnknown::ID:
      return messageFileTypeUnknown::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageFileType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageFileType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageFileTypePrivate::Class;
jfieldID messageFileTypePrivate::name_fieldID;

messageFileTypePrivate::messageFileTypePrivate()
  : name_()
{}

messageFileTypePrivate::messageFileTypePrivate(string const &name_)
  : name_(name_)
{}

const std::int32_t messageFileTypePrivate::ID;

object_ptr<MessageFileType> messageFileTypePrivate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageFileTypePrivate> res = make_object<messageFileTypePrivate>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return std::move(res);
}

void messageFileTypePrivate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageFileTypePrivate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageFileTypePrivate");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void messageFileTypePrivate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageFileTypePrivate").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageFileTypeGroup::Class;
jfieldID messageFileTypeGroup::title_fieldID;

messageFileTypeGroup::messageFileTypeGroup()
  : title_()
{}

messageFileTypeGroup::messageFileTypeGroup(string const &title_)
  : title_(title_)
{}

const std::int32_t messageFileTypeGroup::ID;

object_ptr<MessageFileType> messageFileTypeGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageFileTypeGroup> res = make_object<messageFileTypeGroup>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void messageFileTypeGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageFileTypeGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageFileTypeGroup");
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void messageFileTypeGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageFileTypeGroup").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageFileTypeUnknown::Class;

messageFileTypeUnknown::messageFileTypeUnknown() {
}

const std::int32_t messageFileTypeUnknown::ID;

object_ptr<MessageFileType> messageFileTypeUnknown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageFileTypeUnknown>();
}

void messageFileTypeUnknown::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageFileTypeUnknown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageFileTypeUnknown");
    s.store_class_end();
  }
}

void messageFileTypeUnknown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageFileTypeUnknown").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageForwardInfo::Class;
jfieldID messageForwardInfo::origin_fieldID;
jfieldID messageForwardInfo::date_fieldID;
jfieldID messageForwardInfo::source_fieldID;
jfieldID messageForwardInfo::public_service_announcement_type_fieldID;

messageForwardInfo::messageForwardInfo()
  : origin_()
  , date_()
  , source_()
  , public_service_announcement_type_()
{}

messageForwardInfo::messageForwardInfo(object_ptr<MessageOrigin> &&origin_, int32 date_, object_ptr<forwardSource> &&source_, string const &public_service_announcement_type_)
  : origin_(std::move(origin_))
  , date_(date_)
  , source_(std::move(source_))
  , public_service_announcement_type_(public_service_announcement_type_)
{}

const std::int32_t messageForwardInfo::ID;

object_ptr<messageForwardInfo> messageForwardInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageForwardInfo> res = make_object<messageForwardInfo>();
  res->origin_ = jni::fetch_tl_object<MessageOrigin>(env, jni::fetch_object(env, p, res->origin_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->source_ = jni::fetch_tl_object<forwardSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  res->public_service_announcement_type_ = jni::fetch_string(env, p, res->public_service_announcement_type_fieldID);
  return res;
}

void messageForwardInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (origin_ != nullptr) { jobject next; origin_->store(env, next); if (next) { env->SetObjectField(s, origin_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  if (source_ != nullptr) { jobject next; source_->store(env, next); if (next) { env->SetObjectField(s, source_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, public_service_announcement_type_); if (nextString) { env->SetObjectField(s, public_service_announcement_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageForwardInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageForwardInfo");
    s.store_object_field("origin", static_cast<const BaseObject *>(origin_.get()));
    s.store_field("date", date_);
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_field("publicServiceAnnouncementType", public_service_announcement_type_);
    s.store_class_end();
  }
}

void messageForwardInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageForwardInfo").c_str());
    origin_fieldID = jni::get_field_id(env, Class, "origin", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageOrigin;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForwardSource;").c_str());
    public_service_announcement_type_fieldID = jni::get_field_id(env, Class, "publicServiceAnnouncementType", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageImportInfo::Class;
jfieldID messageImportInfo::sender_name_fieldID;
jfieldID messageImportInfo::date_fieldID;

messageImportInfo::messageImportInfo()
  : sender_name_()
  , date_()
{}

messageImportInfo::messageImportInfo(string const &sender_name_, int32 date_)
  : sender_name_(sender_name_)
  , date_(date_)
{}

const std::int32_t messageImportInfo::ID;

object_ptr<messageImportInfo> messageImportInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageImportInfo> res = make_object<messageImportInfo>();
  res->sender_name_ = jni::fetch_string(env, p, res->sender_name_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void messageImportInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, sender_name_); if (nextString) { env->SetObjectField(s, sender_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, date_fieldID, date_);
}

void messageImportInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageImportInfo");
    s.store_field("senderName", sender_name_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void messageImportInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageImportInfo").c_str());
    sender_name_fieldID = jni::get_field_id(env, Class, "senderName", "Ljava/lang/String;");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageInteractionInfo::Class;
jfieldID messageInteractionInfo::view_count_fieldID;
jfieldID messageInteractionInfo::forward_count_fieldID;
jfieldID messageInteractionInfo::reply_info_fieldID;
jfieldID messageInteractionInfo::reactions_fieldID;

messageInteractionInfo::messageInteractionInfo()
  : view_count_()
  , forward_count_()
  , reply_info_()
  , reactions_()
{}

messageInteractionInfo::messageInteractionInfo(int32 view_count_, int32 forward_count_, object_ptr<messageReplyInfo> &&reply_info_, object_ptr<messageReactions> &&reactions_)
  : view_count_(view_count_)
  , forward_count_(forward_count_)
  , reply_info_(std::move(reply_info_))
  , reactions_(std::move(reactions_))
{}

const std::int32_t messageInteractionInfo::ID;

object_ptr<messageInteractionInfo> messageInteractionInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageInteractionInfo> res = make_object<messageInteractionInfo>();
  res->view_count_ = env->GetIntField(p, res->view_count_fieldID);
  res->forward_count_ = env->GetIntField(p, res->forward_count_fieldID);
  res->reply_info_ = jni::fetch_tl_object<messageReplyInfo>(env, jni::fetch_object(env, p, res->reply_info_fieldID));
  res->reactions_ = jni::fetch_tl_object<messageReactions>(env, jni::fetch_object(env, p, res->reactions_fieldID));
  return res;
}

void messageInteractionInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, view_count_fieldID, view_count_);
  env->SetIntField(s, forward_count_fieldID, forward_count_);
  if (reply_info_ != nullptr) { jobject next; reply_info_->store(env, next); if (next) { env->SetObjectField(s, reply_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (reactions_ != nullptr) { jobject next; reactions_->store(env, next); if (next) { env->SetObjectField(s, reactions_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageInteractionInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageInteractionInfo");
    s.store_field("viewCount", view_count_);
    s.store_field("forwardCount", forward_count_);
    s.store_object_field("replyInfo", static_cast<const BaseObject *>(reply_info_.get()));
    s.store_object_field("reactions", static_cast<const BaseObject *>(reactions_.get()));
    s.store_class_end();
  }
}

void messageInteractionInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageInteractionInfo").c_str());
    view_count_fieldID = jni::get_field_id(env, Class, "viewCount", "I");
    forward_count_fieldID = jni::get_field_id(env, Class, "forwardCount", "I");
    reply_info_fieldID = jni::get_field_id(env, Class, "replyInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageReplyInfo;").c_str());
    reactions_fieldID = jni::get_field_id(env, Class, "reactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageReactions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageLink::Class;
jfieldID messageLink::link_fieldID;
jfieldID messageLink::is_public_fieldID;

messageLink::messageLink()
  : link_()
  , is_public_()
{}

messageLink::messageLink(string const &link_, bool is_public_)
  : link_(link_)
  , is_public_(is_public_)
{}

const std::int32_t messageLink::ID;

object_ptr<messageLink> messageLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageLink> res = make_object<messageLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  return res;
}

void messageLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, link_); if (nextString) { env->SetObjectField(s, link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_public_fieldID, is_public_);
}

void messageLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageLink");
    s.store_field("link", link_);
    s.store_field("isPublic", is_public_);
    s.store_class_end();
  }
}

void messageLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageLinkInfo::Class;
jfieldID messageLinkInfo::is_public_fieldID;
jfieldID messageLinkInfo::chat_id_fieldID;
jfieldID messageLinkInfo::message_thread_id_fieldID;
jfieldID messageLinkInfo::message_fieldID;
jfieldID messageLinkInfo::media_timestamp_fieldID;
jfieldID messageLinkInfo::for_album_fieldID;

messageLinkInfo::messageLinkInfo()
  : is_public_()
  , chat_id_()
  , message_thread_id_()
  , message_()
  , media_timestamp_()
  , for_album_()
{}

messageLinkInfo::messageLinkInfo(bool is_public_, int53 chat_id_, int53 message_thread_id_, object_ptr<message> &&message_, int32 media_timestamp_, bool for_album_)
  : is_public_(is_public_)
  , chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , message_(std::move(message_))
  , media_timestamp_(media_timestamp_)
  , for_album_(for_album_)
{}

const std::int32_t messageLinkInfo::ID;

object_ptr<messageLinkInfo> messageLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageLinkInfo> res = make_object<messageLinkInfo>();
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->media_timestamp_ = env->GetIntField(p, res->media_timestamp_fieldID);
  res->for_album_ = (env->GetBooleanField(p, res->for_album_fieldID) != 0);
  return res;
}

void messageLinkInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_public_fieldID, is_public_);
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, media_timestamp_fieldID, media_timestamp_);
  env->SetBooleanField(s, for_album_fieldID, for_album_);
}

void messageLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageLinkInfo");
    s.store_field("isPublic", is_public_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("mediaTimestamp", media_timestamp_);
    s.store_field("forAlbum", for_album_);
    s.store_class_end();
  }
}

void messageLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageLinkInfo").c_str());
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    media_timestamp_fieldID = jni::get_field_id(env, Class, "mediaTimestamp", "I");
    for_album_fieldID = jni::get_field_id(env, Class, "forAlbum", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass MessageOrigin::Class;

object_ptr<MessageOrigin> MessageOrigin::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageOriginUser::ID:
      return messageOriginUser::fetch(env, p);
    case messageOriginHiddenUser::ID:
      return messageOriginHiddenUser::fetch(env, p);
    case messageOriginChat::ID:
      return messageOriginChat::fetch(env, p);
    case messageOriginChannel::ID:
      return messageOriginChannel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageOrigin::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageOrigin").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageOriginUser::Class;
jfieldID messageOriginUser::sender_user_id_fieldID;

messageOriginUser::messageOriginUser()
  : sender_user_id_()
{}

messageOriginUser::messageOriginUser(int53 sender_user_id_)
  : sender_user_id_(sender_user_id_)
{}

const std::int32_t messageOriginUser::ID;

object_ptr<MessageOrigin> messageOriginUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageOriginUser> res = make_object<messageOriginUser>();
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  return std::move(res);
}

void messageOriginUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
}

void messageOriginUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageOriginUser");
    s.store_field("senderUserId", sender_user_id_);
    s.store_class_end();
  }
}

void messageOriginUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageOriginUser").c_str());
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageOriginHiddenUser::Class;
jfieldID messageOriginHiddenUser::sender_name_fieldID;

messageOriginHiddenUser::messageOriginHiddenUser()
  : sender_name_()
{}

messageOriginHiddenUser::messageOriginHiddenUser(string const &sender_name_)
  : sender_name_(sender_name_)
{}

const std::int32_t messageOriginHiddenUser::ID;

object_ptr<MessageOrigin> messageOriginHiddenUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageOriginHiddenUser> res = make_object<messageOriginHiddenUser>();
  res->sender_name_ = jni::fetch_string(env, p, res->sender_name_fieldID);
  return std::move(res);
}

void messageOriginHiddenUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, sender_name_); if (nextString) { env->SetObjectField(s, sender_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageOriginHiddenUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageOriginHiddenUser");
    s.store_field("senderName", sender_name_);
    s.store_class_end();
  }
}

void messageOriginHiddenUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageOriginHiddenUser").c_str());
    sender_name_fieldID = jni::get_field_id(env, Class, "senderName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageOriginChat::Class;
jfieldID messageOriginChat::sender_chat_id_fieldID;
jfieldID messageOriginChat::author_signature_fieldID;

messageOriginChat::messageOriginChat()
  : sender_chat_id_()
  , author_signature_()
{}

messageOriginChat::messageOriginChat(int53 sender_chat_id_, string const &author_signature_)
  : sender_chat_id_(sender_chat_id_)
  , author_signature_(author_signature_)
{}

const std::int32_t messageOriginChat::ID;

object_ptr<MessageOrigin> messageOriginChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageOriginChat> res = make_object<messageOriginChat>();
  res->sender_chat_id_ = env->GetLongField(p, res->sender_chat_id_fieldID);
  res->author_signature_ = jni::fetch_string(env, p, res->author_signature_fieldID);
  return std::move(res);
}

void messageOriginChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, sender_chat_id_fieldID, sender_chat_id_);
  { jstring nextString = jni::to_jstring(env, author_signature_); if (nextString) { env->SetObjectField(s, author_signature_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageOriginChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageOriginChat");
    s.store_field("senderChatId", sender_chat_id_);
    s.store_field("authorSignature", author_signature_);
    s.store_class_end();
  }
}

void messageOriginChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageOriginChat").c_str());
    sender_chat_id_fieldID = jni::get_field_id(env, Class, "senderChatId", "J");
    author_signature_fieldID = jni::get_field_id(env, Class, "authorSignature", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageOriginChannel::Class;
jfieldID messageOriginChannel::chat_id_fieldID;
jfieldID messageOriginChannel::message_id_fieldID;
jfieldID messageOriginChannel::author_signature_fieldID;

messageOriginChannel::messageOriginChannel()
  : chat_id_()
  , message_id_()
  , author_signature_()
{}

messageOriginChannel::messageOriginChannel(int53 chat_id_, int53 message_id_, string const &author_signature_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , author_signature_(author_signature_)
{}

const std::int32_t messageOriginChannel::ID;

object_ptr<MessageOrigin> messageOriginChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageOriginChannel> res = make_object<messageOriginChannel>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->author_signature_ = jni::fetch_string(env, p, res->author_signature_fieldID);
  return std::move(res);
}

void messageOriginChannel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  { jstring nextString = jni::to_jstring(env, author_signature_); if (nextString) { env->SetObjectField(s, author_signature_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageOriginChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageOriginChannel");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("authorSignature", author_signature_);
    s.store_class_end();
  }
}

void messageOriginChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageOriginChannel").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    author_signature_fieldID = jni::get_field_id(env, Class, "authorSignature", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messagePosition::Class;
jfieldID messagePosition::position_fieldID;
jfieldID messagePosition::message_id_fieldID;
jfieldID messagePosition::date_fieldID;

messagePosition::messagePosition()
  : position_()
  , message_id_()
  , date_()
{}

messagePosition::messagePosition(int32 position_, int53 message_id_, int32 date_)
  : position_(position_)
  , message_id_(message_id_)
  , date_(date_)
{}

const std::int32_t messagePosition::ID;

object_ptr<messagePosition> messagePosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePosition> res = make_object<messagePosition>();
  res->position_ = env->GetIntField(p, res->position_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void messagePosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, position_fieldID, position_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetIntField(s, date_fieldID, date_);
}

void messagePosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePosition");
    s.store_field("position", position_);
    s.store_field("messageId", message_id_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void messagePosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePosition").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messagePositions::Class;
jfieldID messagePositions::total_count_fieldID;
jfieldID messagePositions::positions_fieldID;

messagePositions::messagePositions()
  : total_count_()
  , positions_()
{}

messagePositions::messagePositions(int32 total_count_, array<object_ptr<messagePosition>> &&positions_)
  : total_count_(total_count_)
  , positions_(std::move(positions_))
{}

const std::int32_t messagePositions::ID;

object_ptr<messagePositions> messagePositions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messagePositions> res = make_object<messagePositions>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->positions_ = jni::FetchVector<messagePosition>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->positions_fieldID));
  return res;
}

void messagePositions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, positions_); if (arr_tmp_) { env->SetObjectField(s, positions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messagePositions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessagePositions");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("positions", positions_.size()); for (const auto &_value : positions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messagePositions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessagePositions").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    positions_fieldID = jni::get_field_id(env, Class, "positions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessagePosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageProperties::Class;
jfieldID messageProperties::can_be_copied_to_secret_chat_fieldID;
jfieldID messageProperties::can_be_deleted_only_for_self_fieldID;
jfieldID messageProperties::can_be_deleted_for_all_users_fieldID;
jfieldID messageProperties::can_be_edited_fieldID;
jfieldID messageProperties::can_be_forwarded_fieldID;
jfieldID messageProperties::can_be_paid_fieldID;
jfieldID messageProperties::can_be_pinned_fieldID;
jfieldID messageProperties::can_be_replied_fieldID;
jfieldID messageProperties::can_be_replied_in_another_chat_fieldID;
jfieldID messageProperties::can_be_saved_fieldID;
jfieldID messageProperties::can_be_shared_in_story_fieldID;
jfieldID messageProperties::can_edit_media_fieldID;
jfieldID messageProperties::can_edit_scheduling_state_fieldID;
jfieldID messageProperties::can_get_embedding_code_fieldID;
jfieldID messageProperties::can_get_link_fieldID;
jfieldID messageProperties::can_get_media_timestamp_links_fieldID;
jfieldID messageProperties::can_get_message_thread_fieldID;
jfieldID messageProperties::can_get_read_date_fieldID;
jfieldID messageProperties::can_get_statistics_fieldID;
jfieldID messageProperties::can_get_viewers_fieldID;
jfieldID messageProperties::can_recognize_speech_fieldID;
jfieldID messageProperties::can_report_chat_fieldID;
jfieldID messageProperties::can_report_reactions_fieldID;
jfieldID messageProperties::can_report_supergroup_spam_fieldID;
jfieldID messageProperties::can_set_fact_check_fieldID;
jfieldID messageProperties::need_show_statistics_fieldID;

messageProperties::messageProperties()
  : can_be_copied_to_secret_chat_()
  , can_be_deleted_only_for_self_()
  , can_be_deleted_for_all_users_()
  , can_be_edited_()
  , can_be_forwarded_()
  , can_be_paid_()
  , can_be_pinned_()
  , can_be_replied_()
  , can_be_replied_in_another_chat_()
  , can_be_saved_()
  , can_be_shared_in_story_()
  , can_edit_media_()
  , can_edit_scheduling_state_()
  , can_get_embedding_code_()
  , can_get_link_()
  , can_get_media_timestamp_links_()
  , can_get_message_thread_()
  , can_get_read_date_()
  , can_get_statistics_()
  , can_get_viewers_()
  , can_recognize_speech_()
  , can_report_chat_()
  , can_report_reactions_()
  , can_report_supergroup_spam_()
  , can_set_fact_check_()
  , need_show_statistics_()
{}

messageProperties::messageProperties(bool can_be_copied_to_secret_chat_, bool can_be_deleted_only_for_self_, bool can_be_deleted_for_all_users_, bool can_be_edited_, bool can_be_forwarded_, bool can_be_paid_, bool can_be_pinned_, bool can_be_replied_, bool can_be_replied_in_another_chat_, bool can_be_saved_, bool can_be_shared_in_story_, bool can_edit_media_, bool can_edit_scheduling_state_, bool can_get_embedding_code_, bool can_get_link_, bool can_get_media_timestamp_links_, bool can_get_message_thread_, bool can_get_read_date_, bool can_get_statistics_, bool can_get_viewers_, bool can_recognize_speech_, bool can_report_chat_, bool can_report_reactions_, bool can_report_supergroup_spam_, bool can_set_fact_check_, bool need_show_statistics_)
  : can_be_copied_to_secret_chat_(can_be_copied_to_secret_chat_)
  , can_be_deleted_only_for_self_(can_be_deleted_only_for_self_)
  , can_be_deleted_for_all_users_(can_be_deleted_for_all_users_)
  , can_be_edited_(can_be_edited_)
  , can_be_forwarded_(can_be_forwarded_)
  , can_be_paid_(can_be_paid_)
  , can_be_pinned_(can_be_pinned_)
  , can_be_replied_(can_be_replied_)
  , can_be_replied_in_another_chat_(can_be_replied_in_another_chat_)
  , can_be_saved_(can_be_saved_)
  , can_be_shared_in_story_(can_be_shared_in_story_)
  , can_edit_media_(can_edit_media_)
  , can_edit_scheduling_state_(can_edit_scheduling_state_)
  , can_get_embedding_code_(can_get_embedding_code_)
  , can_get_link_(can_get_link_)
  , can_get_media_timestamp_links_(can_get_media_timestamp_links_)
  , can_get_message_thread_(can_get_message_thread_)
  , can_get_read_date_(can_get_read_date_)
  , can_get_statistics_(can_get_statistics_)
  , can_get_viewers_(can_get_viewers_)
  , can_recognize_speech_(can_recognize_speech_)
  , can_report_chat_(can_report_chat_)
  , can_report_reactions_(can_report_reactions_)
  , can_report_supergroup_spam_(can_report_supergroup_spam_)
  , can_set_fact_check_(can_set_fact_check_)
  , need_show_statistics_(need_show_statistics_)
{}

const std::int32_t messageProperties::ID;

object_ptr<messageProperties> messageProperties::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageProperties> res = make_object<messageProperties>();
  res->can_be_copied_to_secret_chat_ = (env->GetBooleanField(p, res->can_be_copied_to_secret_chat_fieldID) != 0);
  res->can_be_deleted_only_for_self_ = (env->GetBooleanField(p, res->can_be_deleted_only_for_self_fieldID) != 0);
  res->can_be_deleted_for_all_users_ = (env->GetBooleanField(p, res->can_be_deleted_for_all_users_fieldID) != 0);
  res->can_be_edited_ = (env->GetBooleanField(p, res->can_be_edited_fieldID) != 0);
  res->can_be_forwarded_ = (env->GetBooleanField(p, res->can_be_forwarded_fieldID) != 0);
  res->can_be_paid_ = (env->GetBooleanField(p, res->can_be_paid_fieldID) != 0);
  res->can_be_pinned_ = (env->GetBooleanField(p, res->can_be_pinned_fieldID) != 0);
  res->can_be_replied_ = (env->GetBooleanField(p, res->can_be_replied_fieldID) != 0);
  res->can_be_replied_in_another_chat_ = (env->GetBooleanField(p, res->can_be_replied_in_another_chat_fieldID) != 0);
  res->can_be_saved_ = (env->GetBooleanField(p, res->can_be_saved_fieldID) != 0);
  res->can_be_shared_in_story_ = (env->GetBooleanField(p, res->can_be_shared_in_story_fieldID) != 0);
  res->can_edit_media_ = (env->GetBooleanField(p, res->can_edit_media_fieldID) != 0);
  res->can_edit_scheduling_state_ = (env->GetBooleanField(p, res->can_edit_scheduling_state_fieldID) != 0);
  res->can_get_embedding_code_ = (env->GetBooleanField(p, res->can_get_embedding_code_fieldID) != 0);
  res->can_get_link_ = (env->GetBooleanField(p, res->can_get_link_fieldID) != 0);
  res->can_get_media_timestamp_links_ = (env->GetBooleanField(p, res->can_get_media_timestamp_links_fieldID) != 0);
  res->can_get_message_thread_ = (env->GetBooleanField(p, res->can_get_message_thread_fieldID) != 0);
  res->can_get_read_date_ = (env->GetBooleanField(p, res->can_get_read_date_fieldID) != 0);
  res->can_get_statistics_ = (env->GetBooleanField(p, res->can_get_statistics_fieldID) != 0);
  res->can_get_viewers_ = (env->GetBooleanField(p, res->can_get_viewers_fieldID) != 0);
  res->can_recognize_speech_ = (env->GetBooleanField(p, res->can_recognize_speech_fieldID) != 0);
  res->can_report_chat_ = (env->GetBooleanField(p, res->can_report_chat_fieldID) != 0);
  res->can_report_reactions_ = (env->GetBooleanField(p, res->can_report_reactions_fieldID) != 0);
  res->can_report_supergroup_spam_ = (env->GetBooleanField(p, res->can_report_supergroup_spam_fieldID) != 0);
  res->can_set_fact_check_ = (env->GetBooleanField(p, res->can_set_fact_check_fieldID) != 0);
  res->need_show_statistics_ = (env->GetBooleanField(p, res->need_show_statistics_fieldID) != 0);
  return res;
}

void messageProperties::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_be_copied_to_secret_chat_fieldID, can_be_copied_to_secret_chat_);
  env->SetBooleanField(s, can_be_deleted_only_for_self_fieldID, can_be_deleted_only_for_self_);
  env->SetBooleanField(s, can_be_deleted_for_all_users_fieldID, can_be_deleted_for_all_users_);
  env->SetBooleanField(s, can_be_edited_fieldID, can_be_edited_);
  env->SetBooleanField(s, can_be_forwarded_fieldID, can_be_forwarded_);
  env->SetBooleanField(s, can_be_paid_fieldID, can_be_paid_);
  env->SetBooleanField(s, can_be_pinned_fieldID, can_be_pinned_);
  env->SetBooleanField(s, can_be_replied_fieldID, can_be_replied_);
  env->SetBooleanField(s, can_be_replied_in_another_chat_fieldID, can_be_replied_in_another_chat_);
  env->SetBooleanField(s, can_be_saved_fieldID, can_be_saved_);
  env->SetBooleanField(s, can_be_shared_in_story_fieldID, can_be_shared_in_story_);
  env->SetBooleanField(s, can_edit_media_fieldID, can_edit_media_);
  env->SetBooleanField(s, can_edit_scheduling_state_fieldID, can_edit_scheduling_state_);
  env->SetBooleanField(s, can_get_embedding_code_fieldID, can_get_embedding_code_);
  env->SetBooleanField(s, can_get_link_fieldID, can_get_link_);
  env->SetBooleanField(s, can_get_media_timestamp_links_fieldID, can_get_media_timestamp_links_);
  env->SetBooleanField(s, can_get_message_thread_fieldID, can_get_message_thread_);
  env->SetBooleanField(s, can_get_read_date_fieldID, can_get_read_date_);
  env->SetBooleanField(s, can_get_statistics_fieldID, can_get_statistics_);
  env->SetBooleanField(s, can_get_viewers_fieldID, can_get_viewers_);
  env->SetBooleanField(s, can_recognize_speech_fieldID, can_recognize_speech_);
  env->SetBooleanField(s, can_report_chat_fieldID, can_report_chat_);
  env->SetBooleanField(s, can_report_reactions_fieldID, can_report_reactions_);
  env->SetBooleanField(s, can_report_supergroup_spam_fieldID, can_report_supergroup_spam_);
  env->SetBooleanField(s, can_set_fact_check_fieldID, can_set_fact_check_);
  env->SetBooleanField(s, need_show_statistics_fieldID, need_show_statistics_);
}

void messageProperties::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageProperties");
    s.store_field("canBeCopiedToSecretChat", can_be_copied_to_secret_chat_);
    s.store_field("canBeDeletedOnlyForSelf", can_be_deleted_only_for_self_);
    s.store_field("canBeDeletedForAllUsers", can_be_deleted_for_all_users_);
    s.store_field("canBeEdited", can_be_edited_);
    s.store_field("canBeForwarded", can_be_forwarded_);
    s.store_field("canBePaid", can_be_paid_);
    s.store_field("canBePinned", can_be_pinned_);
    s.store_field("canBeReplied", can_be_replied_);
    s.store_field("canBeRepliedInAnotherChat", can_be_replied_in_another_chat_);
    s.store_field("canBeSaved", can_be_saved_);
    s.store_field("canBeSharedInStory", can_be_shared_in_story_);
    s.store_field("canEditMedia", can_edit_media_);
    s.store_field("canEditSchedulingState", can_edit_scheduling_state_);
    s.store_field("canGetEmbeddingCode", can_get_embedding_code_);
    s.store_field("canGetLink", can_get_link_);
    s.store_field("canGetMediaTimestampLinks", can_get_media_timestamp_links_);
    s.store_field("canGetMessageThread", can_get_message_thread_);
    s.store_field("canGetReadDate", can_get_read_date_);
    s.store_field("canGetStatistics", can_get_statistics_);
    s.store_field("canGetViewers", can_get_viewers_);
    s.store_field("canRecognizeSpeech", can_recognize_speech_);
    s.store_field("canReportChat", can_report_chat_);
    s.store_field("canReportReactions", can_report_reactions_);
    s.store_field("canReportSupergroupSpam", can_report_supergroup_spam_);
    s.store_field("canSetFactCheck", can_set_fact_check_);
    s.store_field("needShowStatistics", need_show_statistics_);
    s.store_class_end();
  }
}

void messageProperties::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageProperties").c_str());
    can_be_copied_to_secret_chat_fieldID = jni::get_field_id(env, Class, "canBeCopiedToSecretChat", "Z");
    can_be_deleted_only_for_self_fieldID = jni::get_field_id(env, Class, "canBeDeletedOnlyForSelf", "Z");
    can_be_deleted_for_all_users_fieldID = jni::get_field_id(env, Class, "canBeDeletedForAllUsers", "Z");
    can_be_edited_fieldID = jni::get_field_id(env, Class, "canBeEdited", "Z");
    can_be_forwarded_fieldID = jni::get_field_id(env, Class, "canBeForwarded", "Z");
    can_be_paid_fieldID = jni::get_field_id(env, Class, "canBePaid", "Z");
    can_be_pinned_fieldID = jni::get_field_id(env, Class, "canBePinned", "Z");
    can_be_replied_fieldID = jni::get_field_id(env, Class, "canBeReplied", "Z");
    can_be_replied_in_another_chat_fieldID = jni::get_field_id(env, Class, "canBeRepliedInAnotherChat", "Z");
    can_be_saved_fieldID = jni::get_field_id(env, Class, "canBeSaved", "Z");
    can_be_shared_in_story_fieldID = jni::get_field_id(env, Class, "canBeSharedInStory", "Z");
    can_edit_media_fieldID = jni::get_field_id(env, Class, "canEditMedia", "Z");
    can_edit_scheduling_state_fieldID = jni::get_field_id(env, Class, "canEditSchedulingState", "Z");
    can_get_embedding_code_fieldID = jni::get_field_id(env, Class, "canGetEmbeddingCode", "Z");
    can_get_link_fieldID = jni::get_field_id(env, Class, "canGetLink", "Z");
    can_get_media_timestamp_links_fieldID = jni::get_field_id(env, Class, "canGetMediaTimestampLinks", "Z");
    can_get_message_thread_fieldID = jni::get_field_id(env, Class, "canGetMessageThread", "Z");
    can_get_read_date_fieldID = jni::get_field_id(env, Class, "canGetReadDate", "Z");
    can_get_statistics_fieldID = jni::get_field_id(env, Class, "canGetStatistics", "Z");
    can_get_viewers_fieldID = jni::get_field_id(env, Class, "canGetViewers", "Z");
    can_recognize_speech_fieldID = jni::get_field_id(env, Class, "canRecognizeSpeech", "Z");
    can_report_chat_fieldID = jni::get_field_id(env, Class, "canReportChat", "Z");
    can_report_reactions_fieldID = jni::get_field_id(env, Class, "canReportReactions", "Z");
    can_report_supergroup_spam_fieldID = jni::get_field_id(env, Class, "canReportSupergroupSpam", "Z");
    can_set_fact_check_fieldID = jni::get_field_id(env, Class, "canSetFactCheck", "Z");
    need_show_statistics_fieldID = jni::get_field_id(env, Class, "needShowStatistics", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass messageReaction::Class;
jfieldID messageReaction::type_fieldID;
jfieldID messageReaction::total_count_fieldID;
jfieldID messageReaction::is_chosen_fieldID;
jfieldID messageReaction::used_sender_id_fieldID;
jfieldID messageReaction::recent_sender_ids_fieldID;

messageReaction::messageReaction()
  : type_()
  , total_count_()
  , is_chosen_()
  , used_sender_id_()
  , recent_sender_ids_()
{}

messageReaction::messageReaction(object_ptr<ReactionType> &&type_, int32 total_count_, bool is_chosen_, object_ptr<MessageSender> &&used_sender_id_, array<object_ptr<MessageSender>> &&recent_sender_ids_)
  : type_(std::move(type_))
  , total_count_(total_count_)
  , is_chosen_(is_chosen_)
  , used_sender_id_(std::move(used_sender_id_))
  , recent_sender_ids_(std::move(recent_sender_ids_))
{}

const std::int32_t messageReaction::ID;

object_ptr<messageReaction> messageReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReaction> res = make_object<messageReaction>();
  res->type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->is_chosen_ = (env->GetBooleanField(p, res->is_chosen_fieldID) != 0);
  res->used_sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->used_sender_id_fieldID));
  res->recent_sender_ids_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_sender_ids_fieldID));
  return res;
}

void messageReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetBooleanField(s, is_chosen_fieldID, is_chosen_);
  if (used_sender_id_ != nullptr) { jobject next; used_sender_id_->store(env, next); if (next) { env->SetObjectField(s, used_sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, recent_sender_ids_); if (arr_tmp_) { env->SetObjectField(s, recent_sender_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReaction");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("totalCount", total_count_);
    s.store_field("isChosen", is_chosen_);
    s.store_object_field("usedSenderId", static_cast<const BaseObject *>(used_sender_id_.get()));
    { s.store_vector_begin("recentSenderIds", recent_sender_ids_.size()); for (const auto &_value : recent_sender_ids_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReaction").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    is_chosen_fieldID = jni::get_field_id(env, Class, "isChosen", "Z");
    used_sender_id_fieldID = jni::get_field_id(env, Class, "usedSenderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    recent_sender_ids_fieldID = jni::get_field_id(env, Class, "recentSenderIds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReactions::Class;
jfieldID messageReactions::reactions_fieldID;
jfieldID messageReactions::are_tags_fieldID;
jfieldID messageReactions::paid_reactors_fieldID;
jfieldID messageReactions::can_get_added_reactions_fieldID;

messageReactions::messageReactions()
  : reactions_()
  , are_tags_()
  , paid_reactors_()
  , can_get_added_reactions_()
{}

messageReactions::messageReactions(array<object_ptr<messageReaction>> &&reactions_, bool are_tags_, array<object_ptr<paidReactor>> &&paid_reactors_, bool can_get_added_reactions_)
  : reactions_(std::move(reactions_))
  , are_tags_(are_tags_)
  , paid_reactors_(std::move(paid_reactors_))
  , can_get_added_reactions_(can_get_added_reactions_)
{}

const std::int32_t messageReactions::ID;

object_ptr<messageReactions> messageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReactions> res = make_object<messageReactions>();
  res->reactions_ = jni::FetchVector<messageReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->reactions_fieldID));
  res->are_tags_ = (env->GetBooleanField(p, res->are_tags_fieldID) != 0);
  res->paid_reactors_ = jni::FetchVector<paidReactor>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->paid_reactors_fieldID));
  res->can_get_added_reactions_ = (env->GetBooleanField(p, res->can_get_added_reactions_fieldID) != 0);
  return res;
}

void messageReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, reactions_); if (arr_tmp_) { env->SetObjectField(s, reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, are_tags_fieldID, are_tags_);
  { auto arr_tmp_ = jni::store_vector(env, paid_reactors_); if (arr_tmp_) { env->SetObjectField(s, paid_reactors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, can_get_added_reactions_fieldID, can_get_added_reactions_);
}

void messageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReactions");
    { s.store_vector_begin("reactions", reactions_.size()); for (const auto &_value : reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("areTags", are_tags_);
    { s.store_vector_begin("paidReactors", paid_reactors_.size()); for (const auto &_value : paid_reactors_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("canGetAddedReactions", can_get_added_reactions_);
    s.store_class_end();
  }
}

void messageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReactions").c_str());
    reactions_fieldID = jni::get_field_id(env, Class, "reactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageReaction;").c_str());
    are_tags_fieldID = jni::get_field_id(env, Class, "areTags", "Z");
    paid_reactors_fieldID = jni::get_field_id(env, Class, "paidReactors", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidReactor;").c_str());
    can_get_added_reactions_fieldID = jni::get_field_id(env, Class, "canGetAddedReactions", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass MessageReadDate::Class;

object_ptr<MessageReadDate> MessageReadDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageReadDateRead::ID:
      return messageReadDateRead::fetch(env, p);
    case messageReadDateUnread::ID:
      return messageReadDateUnread::fetch(env, p);
    case messageReadDateTooOld::ID:
      return messageReadDateTooOld::fetch(env, p);
    case messageReadDateUserPrivacyRestricted::ID:
      return messageReadDateUserPrivacyRestricted::fetch(env, p);
    case messageReadDateMyPrivacyRestricted::ID:
      return messageReadDateMyPrivacyRestricted::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageReadDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReadDateRead::Class;
jfieldID messageReadDateRead::read_date_fieldID;

messageReadDateRead::messageReadDateRead()
  : read_date_()
{}

messageReadDateRead::messageReadDateRead(int32 read_date_)
  : read_date_(read_date_)
{}

const std::int32_t messageReadDateRead::ID;

object_ptr<MessageReadDate> messageReadDateRead::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReadDateRead> res = make_object<messageReadDateRead>();
  res->read_date_ = env->GetIntField(p, res->read_date_fieldID);
  return std::move(res);
}

void messageReadDateRead::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, read_date_fieldID, read_date_);
}

void messageReadDateRead::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReadDateRead");
    s.store_field("readDate", read_date_);
    s.store_class_end();
  }
}

void messageReadDateRead::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDateRead").c_str());
    read_date_fieldID = jni::get_field_id(env, Class, "readDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageReadDateUnread::Class;

messageReadDateUnread::messageReadDateUnread() {
}

const std::int32_t messageReadDateUnread::ID;

object_ptr<MessageReadDate> messageReadDateUnread::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageReadDateUnread>();
}

void messageReadDateUnread::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageReadDateUnread::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReadDateUnread");
    s.store_class_end();
  }
}

void messageReadDateUnread::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDateUnread").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReadDateTooOld::Class;

messageReadDateTooOld::messageReadDateTooOld() {
}

const std::int32_t messageReadDateTooOld::ID;

object_ptr<MessageReadDate> messageReadDateTooOld::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageReadDateTooOld>();
}

void messageReadDateTooOld::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageReadDateTooOld::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReadDateTooOld");
    s.store_class_end();
  }
}

void messageReadDateTooOld::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDateTooOld").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReadDateUserPrivacyRestricted::Class;

messageReadDateUserPrivacyRestricted::messageReadDateUserPrivacyRestricted() {
}

const std::int32_t messageReadDateUserPrivacyRestricted::ID;

object_ptr<MessageReadDate> messageReadDateUserPrivacyRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageReadDateUserPrivacyRestricted>();
}

void messageReadDateUserPrivacyRestricted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageReadDateUserPrivacyRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReadDateUserPrivacyRestricted");
    s.store_class_end();
  }
}

void messageReadDateUserPrivacyRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDateUserPrivacyRestricted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReadDateMyPrivacyRestricted::Class;

messageReadDateMyPrivacyRestricted::messageReadDateMyPrivacyRestricted() {
}

const std::int32_t messageReadDateMyPrivacyRestricted::ID;

object_ptr<MessageReadDate> messageReadDateMyPrivacyRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageReadDateMyPrivacyRestricted>();
}

void messageReadDateMyPrivacyRestricted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageReadDateMyPrivacyRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReadDateMyPrivacyRestricted");
    s.store_class_end();
  }
}

void messageReadDateMyPrivacyRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReadDateMyPrivacyRestricted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReplyInfo::Class;
jfieldID messageReplyInfo::reply_count_fieldID;
jfieldID messageReplyInfo::recent_replier_ids_fieldID;
jfieldID messageReplyInfo::last_read_inbox_message_id_fieldID;
jfieldID messageReplyInfo::last_read_outbox_message_id_fieldID;
jfieldID messageReplyInfo::last_message_id_fieldID;

messageReplyInfo::messageReplyInfo()
  : reply_count_()
  , recent_replier_ids_()
  , last_read_inbox_message_id_()
  , last_read_outbox_message_id_()
  , last_message_id_()
{}

messageReplyInfo::messageReplyInfo(int32 reply_count_, array<object_ptr<MessageSender>> &&recent_replier_ids_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int53 last_message_id_)
  : reply_count_(reply_count_)
  , recent_replier_ids_(std::move(recent_replier_ids_))
  , last_read_inbox_message_id_(last_read_inbox_message_id_)
  , last_read_outbox_message_id_(last_read_outbox_message_id_)
  , last_message_id_(last_message_id_)
{}

const std::int32_t messageReplyInfo::ID;

object_ptr<messageReplyInfo> messageReplyInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReplyInfo> res = make_object<messageReplyInfo>();
  res->reply_count_ = env->GetIntField(p, res->reply_count_fieldID);
  res->recent_replier_ids_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_replier_ids_fieldID));
  res->last_read_inbox_message_id_ = env->GetLongField(p, res->last_read_inbox_message_id_fieldID);
  res->last_read_outbox_message_id_ = env->GetLongField(p, res->last_read_outbox_message_id_fieldID);
  res->last_message_id_ = env->GetLongField(p, res->last_message_id_fieldID);
  return res;
}

void messageReplyInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, reply_count_fieldID, reply_count_);
  { auto arr_tmp_ = jni::store_vector(env, recent_replier_ids_); if (arr_tmp_) { env->SetObjectField(s, recent_replier_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetLongField(s, last_read_inbox_message_id_fieldID, last_read_inbox_message_id_);
  env->SetLongField(s, last_read_outbox_message_id_fieldID, last_read_outbox_message_id_);
  env->SetLongField(s, last_message_id_fieldID, last_message_id_);
}

void messageReplyInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReplyInfo");
    s.store_field("replyCount", reply_count_);
    { s.store_vector_begin("recentReplierIds", recent_replier_ids_.size()); for (const auto &_value : recent_replier_ids_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("lastReadInboxMessageId", last_read_inbox_message_id_);
    s.store_field("lastReadOutboxMessageId", last_read_outbox_message_id_);
    s.store_field("lastMessageId", last_message_id_);
    s.store_class_end();
  }
}

void messageReplyInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReplyInfo").c_str());
    reply_count_fieldID = jni::get_field_id(env, Class, "replyCount", "I");
    recent_replier_ids_fieldID = jni::get_field_id(env, Class, "recentReplierIds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    last_read_inbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadInboxMessageId", "J");
    last_read_outbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadOutboxMessageId", "J");
    last_message_id_fieldID = jni::get_field_id(env, Class, "lastMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass MessageReplyTo::Class;

object_ptr<MessageReplyTo> MessageReplyTo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageReplyToMessage::ID:
      return messageReplyToMessage::fetch(env, p);
    case messageReplyToStory::ID:
      return messageReplyToStory::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageReplyTo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReplyTo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReplyToMessage::Class;
jfieldID messageReplyToMessage::chat_id_fieldID;
jfieldID messageReplyToMessage::message_id_fieldID;
jfieldID messageReplyToMessage::quote_fieldID;
jfieldID messageReplyToMessage::origin_fieldID;
jfieldID messageReplyToMessage::origin_send_date_fieldID;
jfieldID messageReplyToMessage::content_fieldID;

messageReplyToMessage::messageReplyToMessage()
  : chat_id_()
  , message_id_()
  , quote_()
  , origin_()
  , origin_send_date_()
  , content_()
{}

messageReplyToMessage::messageReplyToMessage(int53 chat_id_, int53 message_id_, object_ptr<textQuote> &&quote_, object_ptr<MessageOrigin> &&origin_, int32 origin_send_date_, object_ptr<MessageContent> &&content_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , quote_(std::move(quote_))
  , origin_(std::move(origin_))
  , origin_send_date_(origin_send_date_)
  , content_(std::move(content_))
{}

const std::int32_t messageReplyToMessage::ID;

object_ptr<MessageReplyTo> messageReplyToMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReplyToMessage> res = make_object<messageReplyToMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->quote_ = jni::fetch_tl_object<textQuote>(env, jni::fetch_object(env, p, res->quote_fieldID));
  res->origin_ = jni::fetch_tl_object<MessageOrigin>(env, jni::fetch_object(env, p, res->origin_fieldID));
  res->origin_send_date_ = env->GetIntField(p, res->origin_send_date_fieldID);
  res->content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return std::move(res);
}

void messageReplyToMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (quote_ != nullptr) { jobject next; quote_->store(env, next); if (next) { env->SetObjectField(s, quote_fieldID, next); env->DeleteLocalRef(next); } }
  if (origin_ != nullptr) { jobject next; origin_->store(env, next); if (next) { env->SetObjectField(s, origin_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, origin_send_date_fieldID, origin_send_date_);
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageReplyToMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReplyToMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("quote", static_cast<const BaseObject *>(quote_.get()));
    s.store_object_field("origin", static_cast<const BaseObject *>(origin_.get()));
    s.store_field("originSendDate", origin_send_date_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

void messageReplyToMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReplyToMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    quote_fieldID = jni::get_field_id(env, Class, "quote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TextQuote;").c_str());
    origin_fieldID = jni::get_field_id(env, Class, "origin", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageOrigin;").c_str());
    origin_send_date_fieldID = jni::get_field_id(env, Class, "originSendDate", "I");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageReplyToStory::Class;
jfieldID messageReplyToStory::story_poster_chat_id_fieldID;
jfieldID messageReplyToStory::story_id_fieldID;

messageReplyToStory::messageReplyToStory()
  : story_poster_chat_id_()
  , story_id_()
{}

messageReplyToStory::messageReplyToStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t messageReplyToStory::ID;

object_ptr<MessageReplyTo> messageReplyToStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageReplyToStory> res = make_object<messageReplyToStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void messageReplyToStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void messageReplyToStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageReplyToStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void messageReplyToStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageReplyToStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass MessageSchedulingState::Class;

object_ptr<MessageSchedulingState> MessageSchedulingState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageSchedulingStateSendAtDate::ID:
      return messageSchedulingStateSendAtDate::fetch(env, p);
    case messageSchedulingStateSendWhenOnline::ID:
      return messageSchedulingStateSendWhenOnline::fetch(env, p);
    case messageSchedulingStateSendWhenVideoProcessed::ID:
      return messageSchedulingStateSendWhenVideoProcessed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageSchedulingState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSchedulingState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSchedulingStateSendAtDate::Class;
jfieldID messageSchedulingStateSendAtDate::send_date_fieldID;

messageSchedulingStateSendAtDate::messageSchedulingStateSendAtDate()
  : send_date_()
{}

messageSchedulingStateSendAtDate::messageSchedulingStateSendAtDate(int32 send_date_)
  : send_date_(send_date_)
{}

const std::int32_t messageSchedulingStateSendAtDate::ID;

object_ptr<MessageSchedulingState> messageSchedulingStateSendAtDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSchedulingStateSendAtDate> res = make_object<messageSchedulingStateSendAtDate>();
  res->send_date_ = env->GetIntField(p, res->send_date_fieldID);
  return std::move(res);
}

void messageSchedulingStateSendAtDate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, send_date_fieldID, send_date_);
}

void messageSchedulingStateSendAtDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSchedulingStateSendAtDate");
    s.store_field("sendDate", send_date_);
    s.store_class_end();
  }
}

void messageSchedulingStateSendAtDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSchedulingStateSendAtDate").c_str());
    send_date_fieldID = jni::get_field_id(env, Class, "sendDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageSchedulingStateSendWhenOnline::Class;

messageSchedulingStateSendWhenOnline::messageSchedulingStateSendWhenOnline() {
}

const std::int32_t messageSchedulingStateSendWhenOnline::ID;

object_ptr<MessageSchedulingState> messageSchedulingStateSendWhenOnline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSchedulingStateSendWhenOnline>();
}

void messageSchedulingStateSendWhenOnline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageSchedulingStateSendWhenOnline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSchedulingStateSendWhenOnline");
    s.store_class_end();
  }
}

void messageSchedulingStateSendWhenOnline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSchedulingStateSendWhenOnline").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSchedulingStateSendWhenVideoProcessed::Class;
jfieldID messageSchedulingStateSendWhenVideoProcessed::send_date_fieldID;

messageSchedulingStateSendWhenVideoProcessed::messageSchedulingStateSendWhenVideoProcessed()
  : send_date_()
{}

messageSchedulingStateSendWhenVideoProcessed::messageSchedulingStateSendWhenVideoProcessed(int32 send_date_)
  : send_date_(send_date_)
{}

const std::int32_t messageSchedulingStateSendWhenVideoProcessed::ID;

object_ptr<MessageSchedulingState> messageSchedulingStateSendWhenVideoProcessed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSchedulingStateSendWhenVideoProcessed> res = make_object<messageSchedulingStateSendWhenVideoProcessed>();
  res->send_date_ = env->GetIntField(p, res->send_date_fieldID);
  return std::move(res);
}

void messageSchedulingStateSendWhenVideoProcessed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, send_date_fieldID, send_date_);
}

void messageSchedulingStateSendWhenVideoProcessed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSchedulingStateSendWhenVideoProcessed");
    s.store_field("sendDate", send_date_);
    s.store_class_end();
  }
}

void messageSchedulingStateSendWhenVideoProcessed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSchedulingStateSendWhenVideoProcessed").c_str());
    send_date_fieldID = jni::get_field_id(env, Class, "sendDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass MessageSelfDestructType::Class;

object_ptr<MessageSelfDestructType> MessageSelfDestructType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageSelfDestructTypeTimer::ID:
      return messageSelfDestructTypeTimer::fetch(env, p);
    case messageSelfDestructTypeImmediately::ID:
      return messageSelfDestructTypeImmediately::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageSelfDestructType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSelfDestructType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSelfDestructTypeTimer::Class;
jfieldID messageSelfDestructTypeTimer::self_destruct_time_fieldID;

messageSelfDestructTypeTimer::messageSelfDestructTypeTimer()
  : self_destruct_time_()
{}

messageSelfDestructTypeTimer::messageSelfDestructTypeTimer(int32 self_destruct_time_)
  : self_destruct_time_(self_destruct_time_)
{}

const std::int32_t messageSelfDestructTypeTimer::ID;

object_ptr<MessageSelfDestructType> messageSelfDestructTypeTimer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSelfDestructTypeTimer> res = make_object<messageSelfDestructTypeTimer>();
  res->self_destruct_time_ = env->GetIntField(p, res->self_destruct_time_fieldID);
  return std::move(res);
}

void messageSelfDestructTypeTimer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, self_destruct_time_fieldID, self_destruct_time_);
}

void messageSelfDestructTypeTimer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSelfDestructTypeTimer");
    s.store_field("selfDestructTime", self_destruct_time_);
    s.store_class_end();
  }
}

void messageSelfDestructTypeTimer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSelfDestructTypeTimer").c_str());
    self_destruct_time_fieldID = jni::get_field_id(env, Class, "selfDestructTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageSelfDestructTypeImmediately::Class;

messageSelfDestructTypeImmediately::messageSelfDestructTypeImmediately() {
}

const std::int32_t messageSelfDestructTypeImmediately::ID;

object_ptr<MessageSelfDestructType> messageSelfDestructTypeImmediately::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSelfDestructTypeImmediately>();
}

void messageSelfDestructTypeImmediately::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void messageSelfDestructTypeImmediately::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSelfDestructTypeImmediately");
    s.store_class_end();
  }
}

void messageSelfDestructTypeImmediately::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSelfDestructTypeImmediately").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSendOptions::Class;
jfieldID messageSendOptions::disable_notification_fieldID;
jfieldID messageSendOptions::from_background_fieldID;
jfieldID messageSendOptions::protect_content_fieldID;
jfieldID messageSendOptions::allow_paid_broadcast_fieldID;
jfieldID messageSendOptions::paid_message_star_count_fieldID;
jfieldID messageSendOptions::update_order_of_installed_sticker_sets_fieldID;
jfieldID messageSendOptions::scheduling_state_fieldID;
jfieldID messageSendOptions::effect_id_fieldID;
jfieldID messageSendOptions::sending_id_fieldID;
jfieldID messageSendOptions::only_preview_fieldID;

messageSendOptions::messageSendOptions()
  : disable_notification_()
  , from_background_()
  , protect_content_()
  , allow_paid_broadcast_()
  , paid_message_star_count_()
  , update_order_of_installed_sticker_sets_()
  , scheduling_state_()
  , effect_id_()
  , sending_id_()
  , only_preview_()
{}

messageSendOptions::messageSendOptions(bool disable_notification_, bool from_background_, bool protect_content_, bool allow_paid_broadcast_, int53 paid_message_star_count_, bool update_order_of_installed_sticker_sets_, object_ptr<MessageSchedulingState> &&scheduling_state_, int64 effect_id_, int32 sending_id_, bool only_preview_)
  : disable_notification_(disable_notification_)
  , from_background_(from_background_)
  , protect_content_(protect_content_)
  , allow_paid_broadcast_(allow_paid_broadcast_)
  , paid_message_star_count_(paid_message_star_count_)
  , update_order_of_installed_sticker_sets_(update_order_of_installed_sticker_sets_)
  , scheduling_state_(std::move(scheduling_state_))
  , effect_id_(effect_id_)
  , sending_id_(sending_id_)
  , only_preview_(only_preview_)
{}

const std::int32_t messageSendOptions::ID;

object_ptr<messageSendOptions> messageSendOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSendOptions> res = make_object<messageSendOptions>();
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  res->from_background_ = (env->GetBooleanField(p, res->from_background_fieldID) != 0);
  res->protect_content_ = (env->GetBooleanField(p, res->protect_content_fieldID) != 0);
  res->allow_paid_broadcast_ = (env->GetBooleanField(p, res->allow_paid_broadcast_fieldID) != 0);
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  res->update_order_of_installed_sticker_sets_ = (env->GetBooleanField(p, res->update_order_of_installed_sticker_sets_fieldID) != 0);
  res->scheduling_state_ = jni::fetch_tl_object<MessageSchedulingState>(env, jni::fetch_object(env, p, res->scheduling_state_fieldID));
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  res->sending_id_ = env->GetIntField(p, res->sending_id_fieldID);
  res->only_preview_ = (env->GetBooleanField(p, res->only_preview_fieldID) != 0);
  return res;
}

void messageSendOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSendOptions");
    s.store_field("disableNotification", disable_notification_);
    s.store_field("fromBackground", from_background_);
    s.store_field("protectContent", protect_content_);
    s.store_field("allowPaidBroadcast", allow_paid_broadcast_);
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_field("updateOrderOfInstalledStickerSets", update_order_of_installed_sticker_sets_);
    s.store_object_field("schedulingState", static_cast<const BaseObject *>(scheduling_state_.get()));
    s.store_field("effectId", effect_id_);
    s.store_field("sendingId", sending_id_);
    s.store_field("onlyPreview", only_preview_);
    s.store_class_end();
  }
}

void messageSendOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSendOptions").c_str());
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    from_background_fieldID = jni::get_field_id(env, Class, "fromBackground", "Z");
    protect_content_fieldID = jni::get_field_id(env, Class, "protectContent", "Z");
    allow_paid_broadcast_fieldID = jni::get_field_id(env, Class, "allowPaidBroadcast", "Z");
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    update_order_of_installed_sticker_sets_fieldID = jni::get_field_id(env, Class, "updateOrderOfInstalledStickerSets", "Z");
    scheduling_state_fieldID = jni::get_field_id(env, Class, "schedulingState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSchedulingState;").c_str());
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    sending_id_fieldID = jni::get_field_id(env, Class, "sendingId", "I");
    only_preview_fieldID = jni::get_field_id(env, Class, "onlyPreview", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass MessageSender::Class;

object_ptr<MessageSender> MessageSender::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageSenderUser::ID:
      return messageSenderUser::fetch(env, p);
    case messageSenderChat::ID:
      return messageSenderChat::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageSender::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSender").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSenderUser::Class;
jfieldID messageSenderUser::user_id_fieldID;

messageSenderUser::messageSenderUser()
  : user_id_()
{}

messageSenderUser::messageSenderUser(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t messageSenderUser::ID;

object_ptr<MessageSender> messageSenderUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSenderUser> res = make_object<messageSenderUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void messageSenderUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void messageSenderUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSenderUser");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void messageSenderUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSenderUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageSenderChat::Class;
jfieldID messageSenderChat::chat_id_fieldID;

messageSenderChat::messageSenderChat()
  : chat_id_()
{}

messageSenderChat::messageSenderChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t messageSenderChat::ID;

object_ptr<MessageSender> messageSenderChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSenderChat> res = make_object<messageSenderChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void messageSenderChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void messageSenderChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSenderChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void messageSenderChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSenderChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass messageSenders::Class;
jfieldID messageSenders::total_count_fieldID;
jfieldID messageSenders::senders_fieldID;

messageSenders::messageSenders()
  : total_count_()
  , senders_()
{}

messageSenders::messageSenders(int32 total_count_, array<object_ptr<MessageSender>> &&senders_)
  : total_count_(total_count_)
  , senders_(std::move(senders_))
{}

const std::int32_t messageSenders::ID;

object_ptr<messageSenders> messageSenders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSenders> res = make_object<messageSenders>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->senders_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->senders_fieldID));
  return res;
}

void messageSenders::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, senders_); if (arr_tmp_) { env->SetObjectField(s, senders_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageSenders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSenders");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("senders", senders_.size()); for (const auto &_value : senders_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageSenders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSenders").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    senders_fieldID = jni::get_field_id(env, Class, "senders", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass MessageSendingState::Class;

object_ptr<MessageSendingState> MessageSendingState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageSendingStatePending::ID:
      return messageSendingStatePending::fetch(env, p);
    case messageSendingStateFailed::ID:
      return messageSendingStateFailed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageSendingState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSendingState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSendingStatePending::Class;
jfieldID messageSendingStatePending::sending_id_fieldID;

messageSendingStatePending::messageSendingStatePending()
  : sending_id_()
{}

messageSendingStatePending::messageSendingStatePending(int32 sending_id_)
  : sending_id_(sending_id_)
{}

const std::int32_t messageSendingStatePending::ID;

object_ptr<MessageSendingState> messageSendingStatePending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSendingStatePending> res = make_object<messageSendingStatePending>();
  res->sending_id_ = env->GetIntField(p, res->sending_id_fieldID);
  return std::move(res);
}

void messageSendingStatePending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, sending_id_fieldID, sending_id_);
}

void messageSendingStatePending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSendingStatePending");
    s.store_field("sendingId", sending_id_);
    s.store_class_end();
  }
}

void messageSendingStatePending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSendingStatePending").c_str());
    sending_id_fieldID = jni::get_field_id(env, Class, "sendingId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageSendingStateFailed::Class;
jfieldID messageSendingStateFailed::error_fieldID;
jfieldID messageSendingStateFailed::can_retry_fieldID;
jfieldID messageSendingStateFailed::need_another_sender_fieldID;
jfieldID messageSendingStateFailed::need_another_reply_quote_fieldID;
jfieldID messageSendingStateFailed::need_drop_reply_fieldID;
jfieldID messageSendingStateFailed::required_paid_message_star_count_fieldID;
jfieldID messageSendingStateFailed::retry_after_fieldID;

messageSendingStateFailed::messageSendingStateFailed()
  : error_()
  , can_retry_()
  , need_another_sender_()
  , need_another_reply_quote_()
  , need_drop_reply_()
  , required_paid_message_star_count_()
  , retry_after_()
{}

messageSendingStateFailed::messageSendingStateFailed(object_ptr<error> &&error_, bool can_retry_, bool need_another_sender_, bool need_another_reply_quote_, bool need_drop_reply_, int53 required_paid_message_star_count_, double retry_after_)
  : error_(std::move(error_))
  , can_retry_(can_retry_)
  , need_another_sender_(need_another_sender_)
  , need_another_reply_quote_(need_another_reply_quote_)
  , need_drop_reply_(need_drop_reply_)
  , required_paid_message_star_count_(required_paid_message_star_count_)
  , retry_after_(retry_after_)
{}

const std::int32_t messageSendingStateFailed::ID;

object_ptr<MessageSendingState> messageSendingStateFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSendingStateFailed> res = make_object<messageSendingStateFailed>();
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  res->can_retry_ = (env->GetBooleanField(p, res->can_retry_fieldID) != 0);
  res->need_another_sender_ = (env->GetBooleanField(p, res->need_another_sender_fieldID) != 0);
  res->need_another_reply_quote_ = (env->GetBooleanField(p, res->need_another_reply_quote_fieldID) != 0);
  res->need_drop_reply_ = (env->GetBooleanField(p, res->need_drop_reply_fieldID) != 0);
  res->required_paid_message_star_count_ = env->GetLongField(p, res->required_paid_message_star_count_fieldID);
  res->retry_after_ = env->GetDoubleField(p, res->retry_after_fieldID);
  return std::move(res);
}

void messageSendingStateFailed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (error_ != nullptr) { jobject next; error_->store(env, next); if (next) { env->SetObjectField(s, error_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_retry_fieldID, can_retry_);
  env->SetBooleanField(s, need_another_sender_fieldID, need_another_sender_);
  env->SetBooleanField(s, need_another_reply_quote_fieldID, need_another_reply_quote_);
  env->SetBooleanField(s, need_drop_reply_fieldID, need_drop_reply_);
  env->SetLongField(s, required_paid_message_star_count_fieldID, required_paid_message_star_count_);
  env->SetDoubleField(s, retry_after_fieldID, retry_after_);
}

void messageSendingStateFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSendingStateFailed");
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_field("canRetry", can_retry_);
    s.store_field("needAnotherSender", need_another_sender_);
    s.store_field("needAnotherReplyQuote", need_another_reply_quote_);
    s.store_field("needDropReply", need_drop_reply_);
    s.store_field("requiredPaidMessageStarCount", required_paid_message_star_count_);
    s.store_field("retryAfter", retry_after_);
    s.store_class_end();
  }
}

void messageSendingStateFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSendingStateFailed").c_str());
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    can_retry_fieldID = jni::get_field_id(env, Class, "canRetry", "Z");
    need_another_sender_fieldID = jni::get_field_id(env, Class, "needAnotherSender", "Z");
    need_another_reply_quote_fieldID = jni::get_field_id(env, Class, "needAnotherReplyQuote", "Z");
    need_drop_reply_fieldID = jni::get_field_id(env, Class, "needDropReply", "Z");
    required_paid_message_star_count_fieldID = jni::get_field_id(env, Class, "requiredPaidMessageStarCount", "J");
    retry_after_fieldID = jni::get_field_id(env, Class, "retryAfter", "D");
    return true;
  }();
  (void)is_inited;
}

jclass MessageSource::Class;

object_ptr<MessageSource> MessageSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case messageSourceChatHistory::ID:
      return messageSourceChatHistory::fetch(env, p);
    case messageSourceMessageThreadHistory::ID:
      return messageSourceMessageThreadHistory::fetch(env, p);
    case messageSourceForumTopicHistory::ID:
      return messageSourceForumTopicHistory::fetch(env, p);
    case messageSourceHistoryPreview::ID:
      return messageSourceHistoryPreview::fetch(env, p);
    case messageSourceChatList::ID:
      return messageSourceChatList::fetch(env, p);
    case messageSourceSearch::ID:
      return messageSourceSearch::fetch(env, p);
    case messageSourceChatEventLog::ID:
      return messageSourceChatEventLog::fetch(env, p);
    case messageSourceNotification::ID:
      return messageSourceNotification::fetch(env, p);
    case messageSourceScreenshot::ID:
      return messageSourceScreenshot::fetch(env, p);
    case messageSourceOther::ID:
      return messageSourceOther::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void MessageSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceChatHistory::Class;

messageSourceChatHistory::messageSourceChatHistory() {
}

const std::int32_t messageSourceChatHistory::ID;

object_ptr<MessageSource> messageSourceChatHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceChatHistory>();
}

void messageSourceChatHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceChatHistory");
    s.store_class_end();
  }
}

void messageSourceChatHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceChatHistory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceMessageThreadHistory::Class;

messageSourceMessageThreadHistory::messageSourceMessageThreadHistory() {
}

const std::int32_t messageSourceMessageThreadHistory::ID;

object_ptr<MessageSource> messageSourceMessageThreadHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceMessageThreadHistory>();
}

void messageSourceMessageThreadHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceMessageThreadHistory");
    s.store_class_end();
  }
}

void messageSourceMessageThreadHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceMessageThreadHistory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceForumTopicHistory::Class;

messageSourceForumTopicHistory::messageSourceForumTopicHistory() {
}

const std::int32_t messageSourceForumTopicHistory::ID;

object_ptr<MessageSource> messageSourceForumTopicHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceForumTopicHistory>();
}

void messageSourceForumTopicHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceForumTopicHistory");
    s.store_class_end();
  }
}

void messageSourceForumTopicHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceForumTopicHistory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceHistoryPreview::Class;

messageSourceHistoryPreview::messageSourceHistoryPreview() {
}

const std::int32_t messageSourceHistoryPreview::ID;

object_ptr<MessageSource> messageSourceHistoryPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceHistoryPreview>();
}

void messageSourceHistoryPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceHistoryPreview");
    s.store_class_end();
  }
}

void messageSourceHistoryPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceHistoryPreview").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceChatList::Class;

messageSourceChatList::messageSourceChatList() {
}

const std::int32_t messageSourceChatList::ID;

object_ptr<MessageSource> messageSourceChatList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceChatList>();
}

void messageSourceChatList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceChatList");
    s.store_class_end();
  }
}

void messageSourceChatList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceChatList").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceSearch::Class;

messageSourceSearch::messageSourceSearch() {
}

const std::int32_t messageSourceSearch::ID;

object_ptr<MessageSource> messageSourceSearch::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceSearch>();
}

void messageSourceSearch::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceSearch");
    s.store_class_end();
  }
}

void messageSourceSearch::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceSearch").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceChatEventLog::Class;

messageSourceChatEventLog::messageSourceChatEventLog() {
}

const std::int32_t messageSourceChatEventLog::ID;

object_ptr<MessageSource> messageSourceChatEventLog::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceChatEventLog>();
}

void messageSourceChatEventLog::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceChatEventLog");
    s.store_class_end();
  }
}

void messageSourceChatEventLog::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceChatEventLog").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceNotification::Class;

messageSourceNotification::messageSourceNotification() {
}

const std::int32_t messageSourceNotification::ID;

object_ptr<MessageSource> messageSourceNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceNotification>();
}

void messageSourceNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceNotification");
    s.store_class_end();
  }
}

void messageSourceNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceNotification").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceScreenshot::Class;

messageSourceScreenshot::messageSourceScreenshot() {
}

const std::int32_t messageSourceScreenshot::ID;

object_ptr<MessageSource> messageSourceScreenshot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceScreenshot>();
}

void messageSourceScreenshot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceScreenshot");
    s.store_class_end();
  }
}

void messageSourceScreenshot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceScreenshot").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSourceOther::Class;

messageSourceOther::messageSourceOther() {
}

const std::int32_t messageSourceOther::ID;

object_ptr<MessageSource> messageSourceOther::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<messageSourceOther>();
}

void messageSourceOther::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSourceOther");
    s.store_class_end();
  }
}

void messageSourceOther::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSourceOther").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageSponsor::Class;
jfieldID messageSponsor::url_fieldID;
jfieldID messageSponsor::photo_fieldID;
jfieldID messageSponsor::info_fieldID;

messageSponsor::messageSponsor()
  : url_()
  , photo_()
  , info_()
{}

messageSponsor::messageSponsor(string const &url_, object_ptr<photo> &&photo_, string const &info_)
  : url_(url_)
  , photo_(std::move(photo_))
  , info_(info_)
{}

const std::int32_t messageSponsor::ID;

object_ptr<messageSponsor> messageSponsor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageSponsor> res = make_object<messageSponsor>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->info_ = jni::fetch_string(env, p, res->info_fieldID);
  return res;
}

void messageSponsor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, info_); if (nextString) { env->SetObjectField(s, info_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void messageSponsor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageSponsor");
    s.store_field("url", url_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("info", info_);
    s.store_class_end();
  }
}

void messageSponsor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageSponsor").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass messageStatistics::Class;
jfieldID messageStatistics::message_interaction_graph_fieldID;
jfieldID messageStatistics::message_reaction_graph_fieldID;

messageStatistics::messageStatistics()
  : message_interaction_graph_()
  , message_reaction_graph_()
{}

messageStatistics::messageStatistics(object_ptr<StatisticalGraph> &&message_interaction_graph_, object_ptr<StatisticalGraph> &&message_reaction_graph_)
  : message_interaction_graph_(std::move(message_interaction_graph_))
  , message_reaction_graph_(std::move(message_reaction_graph_))
{}

const std::int32_t messageStatistics::ID;

object_ptr<messageStatistics> messageStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageStatistics> res = make_object<messageStatistics>();
  res->message_interaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->message_interaction_graph_fieldID));
  res->message_reaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->message_reaction_graph_fieldID));
  return res;
}

void messageStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_interaction_graph_ != nullptr) { jobject next; message_interaction_graph_->store(env, next); if (next) { env->SetObjectField(s, message_interaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (message_reaction_graph_ != nullptr) { jobject next; message_reaction_graph_->store(env, next); if (next) { env->SetObjectField(s, message_reaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageStatistics");
    s.store_object_field("messageInteractionGraph", static_cast<const BaseObject *>(message_interaction_graph_.get()));
    s.store_object_field("messageReactionGraph", static_cast<const BaseObject *>(message_reaction_graph_.get()));
    s.store_class_end();
  }
}

void messageStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageStatistics").c_str());
    message_interaction_graph_fieldID = jni::get_field_id(env, Class, "messageInteractionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    message_reaction_graph_fieldID = jni::get_field_id(env, Class, "messageReactionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageThreadInfo::Class;
jfieldID messageThreadInfo::chat_id_fieldID;
jfieldID messageThreadInfo::message_thread_id_fieldID;
jfieldID messageThreadInfo::reply_info_fieldID;
jfieldID messageThreadInfo::unread_message_count_fieldID;
jfieldID messageThreadInfo::messages_fieldID;
jfieldID messageThreadInfo::draft_message_fieldID;

messageThreadInfo::messageThreadInfo()
  : chat_id_()
  , message_thread_id_()
  , reply_info_()
  , unread_message_count_()
  , messages_()
  , draft_message_()
{}

messageThreadInfo::messageThreadInfo(int53 chat_id_, int53 message_thread_id_, object_ptr<messageReplyInfo> &&reply_info_, int32 unread_message_count_, array<object_ptr<message>> &&messages_, object_ptr<draftMessage> &&draft_message_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , reply_info_(std::move(reply_info_))
  , unread_message_count_(unread_message_count_)
  , messages_(std::move(messages_))
  , draft_message_(std::move(draft_message_))
{}

const std::int32_t messageThreadInfo::ID;

object_ptr<messageThreadInfo> messageThreadInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageThreadInfo> res = make_object<messageThreadInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->reply_info_ = jni::fetch_tl_object<messageReplyInfo>(env, jni::fetch_object(env, p, res->reply_info_fieldID));
  res->unread_message_count_ = env->GetIntField(p, res->unread_message_count_fieldID);
  res->messages_ = jni::FetchVector<message>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  return res;
}

void messageThreadInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  if (reply_info_ != nullptr) { jobject next; reply_info_->store(env, next); if (next) { env->SetObjectField(s, reply_info_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, unread_message_count_fieldID, unread_message_count_);
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (draft_message_ != nullptr) { jobject next; draft_message_->store(env, next); if (next) { env->SetObjectField(s, draft_message_fieldID, next); env->DeleteLocalRef(next); } }
}

void messageThreadInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageThreadInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("replyInfo", static_cast<const BaseObject *>(reply_info_.get()));
    s.store_field("unreadMessageCount", unread_message_count_);
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    s.store_class_end();
  }
}

void messageThreadInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageThreadInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    reply_info_fieldID = jni::get_field_id(env, Class, "replyInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageReplyInfo;").c_str());
    unread_message_count_fieldID = jni::get_field_id(env, Class, "unreadMessageCount", "I");
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messageViewer::Class;
jfieldID messageViewer::user_id_fieldID;
jfieldID messageViewer::view_date_fieldID;

messageViewer::messageViewer()
  : user_id_()
  , view_date_()
{}

messageViewer::messageViewer(int53 user_id_, int32 view_date_)
  : user_id_(user_id_)
  , view_date_(view_date_)
{}

const std::int32_t messageViewer::ID;

object_ptr<messageViewer> messageViewer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageViewer> res = make_object<messageViewer>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->view_date_ = env->GetIntField(p, res->view_date_fieldID);
  return res;
}

void messageViewer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, view_date_fieldID, view_date_);
}

void messageViewer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageViewer");
    s.store_field("userId", user_id_);
    s.store_field("viewDate", view_date_);
    s.store_class_end();
  }
}

void messageViewer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageViewer").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    view_date_fieldID = jni::get_field_id(env, Class, "viewDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass messageViewers::Class;
jfieldID messageViewers::viewers_fieldID;

messageViewers::messageViewers()
  : viewers_()
{}

messageViewers::messageViewers(array<object_ptr<messageViewer>> &&viewers_)
  : viewers_(std::move(viewers_))
{}

const std::int32_t messageViewers::ID;

object_ptr<messageViewers> messageViewers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messageViewers> res = make_object<messageViewers>();
  res->viewers_ = jni::FetchVector<messageViewer>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->viewers_fieldID));
  return res;
}

void messageViewers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, viewers_); if (arr_tmp_) { env->SetObjectField(s, viewers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messageViewers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "MessageViewers");
    { s.store_vector_begin("viewers", viewers_.size()); for (const auto &_value : viewers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messageViewers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$MessageViewers").c_str());
    viewers_fieldID = jni::get_field_id(env, Class, "viewers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageViewer;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass messages::Class;
jfieldID messages::total_count_fieldID;
jfieldID messages::messages_fieldID;

messages::messages()
  : total_count_()
  , messages_()
{}

messages::messages(int32 total_count_, array<object_ptr<message>> &&messages_)
  : total_count_(total_count_)
  , messages_(std::move(messages_))
{}

const std::int32_t messages::ID;

object_ptr<messages> messages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<messages> res = make_object<messages>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->messages_ = jni::FetchVector<message>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  return res;
}

void messages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void messages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Messages");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void messages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Messages").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass minithumbnail::Class;
jfieldID minithumbnail::width_fieldID;
jfieldID minithumbnail::height_fieldID;
jfieldID minithumbnail::data_fieldID;

minithumbnail::minithumbnail()
  : width_()
  , height_()
  , data_()
{}

minithumbnail::minithumbnail(int32 width_, int32 height_, bytes const &data_)
  : width_(width_)
  , height_(height_)
  , data_(std::move(data_))
{}

const std::int32_t minithumbnail::ID;

object_ptr<minithumbnail> minithumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<minithumbnail> res = make_object<minithumbnail>();
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void minithumbnail::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void minithumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Minithumbnail");
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void minithumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Minithumbnail").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass networkStatistics::Class;
jfieldID networkStatistics::since_date_fieldID;
jfieldID networkStatistics::entries_fieldID;

networkStatistics::networkStatistics()
  : since_date_()
  , entries_()
{}

networkStatistics::networkStatistics(int32 since_date_, array<object_ptr<NetworkStatisticsEntry>> &&entries_)
  : since_date_(since_date_)
  , entries_(std::move(entries_))
{}

const std::int32_t networkStatistics::ID;

object_ptr<networkStatistics> networkStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<networkStatistics> res = make_object<networkStatistics>();
  res->since_date_ = env->GetIntField(p, res->since_date_fieldID);
  res->entries_ = jni::FetchVector<NetworkStatisticsEntry>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->entries_fieldID));
  return res;
}

void networkStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, since_date_fieldID, since_date_);
  { auto arr_tmp_ = jni::store_vector(env, entries_); if (arr_tmp_) { env->SetObjectField(s, entries_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void networkStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkStatistics");
    s.store_field("sinceDate", since_date_);
    { s.store_vector_begin("entries", entries_.size()); for (const auto &_value : entries_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void networkStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkStatistics").c_str());
    since_date_fieldID = jni::get_field_id(env, Class, "sinceDate", "I");
    entries_fieldID = jni::get_field_id(env, Class, "entries", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$NetworkStatisticsEntry;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass NetworkStatisticsEntry::Class;

object_ptr<NetworkStatisticsEntry> NetworkStatisticsEntry::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case networkStatisticsEntryFile::ID:
      return networkStatisticsEntryFile::fetch(env, p);
    case networkStatisticsEntryCall::ID:
      return networkStatisticsEntryCall::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void NetworkStatisticsEntry::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkStatisticsEntry").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkStatisticsEntryFile::Class;
jfieldID networkStatisticsEntryFile::file_type_fieldID;
jfieldID networkStatisticsEntryFile::network_type_fieldID;
jfieldID networkStatisticsEntryFile::sent_bytes_fieldID;
jfieldID networkStatisticsEntryFile::received_bytes_fieldID;

networkStatisticsEntryFile::networkStatisticsEntryFile()
  : file_type_()
  , network_type_()
  , sent_bytes_()
  , received_bytes_()
{}

networkStatisticsEntryFile::networkStatisticsEntryFile(object_ptr<FileType> &&file_type_, object_ptr<NetworkType> &&network_type_, int53 sent_bytes_, int53 received_bytes_)
  : file_type_(std::move(file_type_))
  , network_type_(std::move(network_type_))
  , sent_bytes_(sent_bytes_)
  , received_bytes_(received_bytes_)
{}

const std::int32_t networkStatisticsEntryFile::ID;

object_ptr<NetworkStatisticsEntry> networkStatisticsEntryFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<networkStatisticsEntryFile> res = make_object<networkStatisticsEntryFile>();
  res->file_type_ = jni::fetch_tl_object<FileType>(env, jni::fetch_object(env, p, res->file_type_fieldID));
  res->network_type_ = jni::fetch_tl_object<NetworkType>(env, jni::fetch_object(env, p, res->network_type_fieldID));
  res->sent_bytes_ = env->GetLongField(p, res->sent_bytes_fieldID);
  res->received_bytes_ = env->GetLongField(p, res->received_bytes_fieldID);
  return std::move(res);
}

void networkStatisticsEntryFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (file_type_ != nullptr) { jobject next; file_type_->store(env, next); if (next) { env->SetObjectField(s, file_type_fieldID, next); env->DeleteLocalRef(next); } }
  if (network_type_ != nullptr) { jobject next; network_type_->store(env, next); if (next) { env->SetObjectField(s, network_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, sent_bytes_fieldID, sent_bytes_);
  env->SetLongField(s, received_bytes_fieldID, received_bytes_);
}

void networkStatisticsEntryFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkStatisticsEntryFile");
    s.store_object_field("fileType", static_cast<const BaseObject *>(file_type_.get()));
    s.store_object_field("networkType", static_cast<const BaseObject *>(network_type_.get()));
    s.store_field("sentBytes", sent_bytes_);
    s.store_field("receivedBytes", received_bytes_);
    s.store_class_end();
  }
}

void networkStatisticsEntryFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkStatisticsEntryFile").c_str());
    file_type_fieldID = jni::get_field_id(env, Class, "fileType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FileType;").c_str());
    network_type_fieldID = jni::get_field_id(env, Class, "networkType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NetworkType;").c_str());
    sent_bytes_fieldID = jni::get_field_id(env, Class, "sentBytes", "J");
    received_bytes_fieldID = jni::get_field_id(env, Class, "receivedBytes", "J");
    return true;
  }();
  (void)is_inited;
}

jclass networkStatisticsEntryCall::Class;
jfieldID networkStatisticsEntryCall::network_type_fieldID;
jfieldID networkStatisticsEntryCall::sent_bytes_fieldID;
jfieldID networkStatisticsEntryCall::received_bytes_fieldID;
jfieldID networkStatisticsEntryCall::duration_fieldID;

networkStatisticsEntryCall::networkStatisticsEntryCall()
  : network_type_()
  , sent_bytes_()
  , received_bytes_()
  , duration_()
{}

networkStatisticsEntryCall::networkStatisticsEntryCall(object_ptr<NetworkType> &&network_type_, int53 sent_bytes_, int53 received_bytes_, double duration_)
  : network_type_(std::move(network_type_))
  , sent_bytes_(sent_bytes_)
  , received_bytes_(received_bytes_)
  , duration_(duration_)
{}

const std::int32_t networkStatisticsEntryCall::ID;

object_ptr<NetworkStatisticsEntry> networkStatisticsEntryCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<networkStatisticsEntryCall> res = make_object<networkStatisticsEntryCall>();
  res->network_type_ = jni::fetch_tl_object<NetworkType>(env, jni::fetch_object(env, p, res->network_type_fieldID));
  res->sent_bytes_ = env->GetLongField(p, res->sent_bytes_fieldID);
  res->received_bytes_ = env->GetLongField(p, res->received_bytes_fieldID);
  res->duration_ = env->GetDoubleField(p, res->duration_fieldID);
  return std::move(res);
}

void networkStatisticsEntryCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (network_type_ != nullptr) { jobject next; network_type_->store(env, next); if (next) { env->SetObjectField(s, network_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, sent_bytes_fieldID, sent_bytes_);
  env->SetLongField(s, received_bytes_fieldID, received_bytes_);
  env->SetDoubleField(s, duration_fieldID, duration_);
}

void networkStatisticsEntryCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkStatisticsEntryCall");
    s.store_object_field("networkType", static_cast<const BaseObject *>(network_type_.get()));
    s.store_field("sentBytes", sent_bytes_);
    s.store_field("receivedBytes", received_bytes_);
    s.store_field("duration", duration_);
    s.store_class_end();
  }
}

void networkStatisticsEntryCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkStatisticsEntryCall").c_str());
    network_type_fieldID = jni::get_field_id(env, Class, "networkType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NetworkType;").c_str());
    sent_bytes_fieldID = jni::get_field_id(env, Class, "sentBytes", "J");
    received_bytes_fieldID = jni::get_field_id(env, Class, "receivedBytes", "J");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "D");
    return true;
  }();
  (void)is_inited;
}

jclass NetworkType::Class;

object_ptr<NetworkType> NetworkType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case networkTypeNone::ID:
      return networkTypeNone::fetch(env, p);
    case networkTypeMobile::ID:
      return networkTypeMobile::fetch(env, p);
    case networkTypeMobileRoaming::ID:
      return networkTypeMobileRoaming::fetch(env, p);
    case networkTypeWiFi::ID:
      return networkTypeWiFi::fetch(env, p);
    case networkTypeOther::ID:
      return networkTypeOther::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void NetworkType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkTypeNone::Class;

networkTypeNone::networkTypeNone() {
}

const std::int32_t networkTypeNone::ID;

object_ptr<NetworkType> networkTypeNone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<networkTypeNone>();
}

void networkTypeNone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void networkTypeNone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkTypeNone");
    s.store_class_end();
  }
}

void networkTypeNone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkTypeNone").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkTypeMobile::Class;

networkTypeMobile::networkTypeMobile() {
}

const std::int32_t networkTypeMobile::ID;

object_ptr<NetworkType> networkTypeMobile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<networkTypeMobile>();
}

void networkTypeMobile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void networkTypeMobile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkTypeMobile");
    s.store_class_end();
  }
}

void networkTypeMobile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkTypeMobile").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkTypeMobileRoaming::Class;

networkTypeMobileRoaming::networkTypeMobileRoaming() {
}

const std::int32_t networkTypeMobileRoaming::ID;

object_ptr<NetworkType> networkTypeMobileRoaming::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<networkTypeMobileRoaming>();
}

void networkTypeMobileRoaming::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void networkTypeMobileRoaming::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkTypeMobileRoaming");
    s.store_class_end();
  }
}

void networkTypeMobileRoaming::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkTypeMobileRoaming").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkTypeWiFi::Class;

networkTypeWiFi::networkTypeWiFi() {
}

const std::int32_t networkTypeWiFi::ID;

object_ptr<NetworkType> networkTypeWiFi::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<networkTypeWiFi>();
}

void networkTypeWiFi::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void networkTypeWiFi::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkTypeWiFi");
    s.store_class_end();
  }
}

void networkTypeWiFi::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkTypeWiFi").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass networkTypeOther::Class;

networkTypeOther::networkTypeOther() {
}

const std::int32_t networkTypeOther::ID;

object_ptr<NetworkType> networkTypeOther::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<networkTypeOther>();
}

void networkTypeOther::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void networkTypeOther::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NetworkTypeOther");
    s.store_class_end();
  }
}

void networkTypeOther::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NetworkTypeOther").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass newChatPrivacySettings::Class;
jfieldID newChatPrivacySettings::allow_new_chats_from_unknown_users_fieldID;
jfieldID newChatPrivacySettings::incoming_paid_message_star_count_fieldID;

newChatPrivacySettings::newChatPrivacySettings()
  : allow_new_chats_from_unknown_users_()
  , incoming_paid_message_star_count_()
{}

newChatPrivacySettings::newChatPrivacySettings(bool allow_new_chats_from_unknown_users_, int53 incoming_paid_message_star_count_)
  : allow_new_chats_from_unknown_users_(allow_new_chats_from_unknown_users_)
  , incoming_paid_message_star_count_(incoming_paid_message_star_count_)
{}

const std::int32_t newChatPrivacySettings::ID;

object_ptr<newChatPrivacySettings> newChatPrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<newChatPrivacySettings> res = make_object<newChatPrivacySettings>();
  res->allow_new_chats_from_unknown_users_ = (env->GetBooleanField(p, res->allow_new_chats_from_unknown_users_fieldID) != 0);
  res->incoming_paid_message_star_count_ = env->GetLongField(p, res->incoming_paid_message_star_count_fieldID);
  return res;
}

void newChatPrivacySettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, allow_new_chats_from_unknown_users_fieldID, allow_new_chats_from_unknown_users_);
  env->SetLongField(s, incoming_paid_message_star_count_fieldID, incoming_paid_message_star_count_);
}

void newChatPrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NewChatPrivacySettings");
    s.store_field("allowNewChatsFromUnknownUsers", allow_new_chats_from_unknown_users_);
    s.store_field("incomingPaidMessageStarCount", incoming_paid_message_star_count_);
    s.store_class_end();
  }
}

void newChatPrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NewChatPrivacySettings").c_str());
    allow_new_chats_from_unknown_users_fieldID = jni::get_field_id(env, Class, "allowNewChatsFromUnknownUsers", "Z");
    incoming_paid_message_star_count_fieldID = jni::get_field_id(env, Class, "incomingPaidMessageStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass notification::Class;
jfieldID notification::id_fieldID;
jfieldID notification::date_fieldID;
jfieldID notification::is_silent_fieldID;
jfieldID notification::type_fieldID;

notification::notification()
  : id_()
  , date_()
  , is_silent_()
  , type_()
{}

notification::notification(int32 id_, int32 date_, bool is_silent_, object_ptr<NotificationType> &&type_)
  : id_(id_)
  , date_(date_)
  , is_silent_(is_silent_)
  , type_(std::move(type_))
{}

const std::int32_t notification::ID;

object_ptr<notification> notification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notification> res = make_object<notification>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->is_silent_ = (env->GetBooleanField(p, res->is_silent_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<NotificationType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void notification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetIntField(s, date_fieldID, date_);
  env->SetBooleanField(s, is_silent_fieldID, is_silent_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void notification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Notification");
    s.store_field("id", id_);
    s.store_field("date", date_);
    s.store_field("isSilent", is_silent_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void notification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Notification").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    is_silent_fieldID = jni::get_field_id(env, Class, "isSilent", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationGroup::Class;
jfieldID notificationGroup::id_fieldID;
jfieldID notificationGroup::type_fieldID;
jfieldID notificationGroup::chat_id_fieldID;
jfieldID notificationGroup::total_count_fieldID;
jfieldID notificationGroup::notifications_fieldID;

notificationGroup::notificationGroup()
  : id_()
  , type_()
  , chat_id_()
  , total_count_()
  , notifications_()
{}

notificationGroup::notificationGroup(int32 id_, object_ptr<NotificationGroupType> &&type_, int53 chat_id_, int32 total_count_, array<object_ptr<notification>> &&notifications_)
  : id_(id_)
  , type_(std::move(type_))
  , chat_id_(chat_id_)
  , total_count_(total_count_)
  , notifications_(std::move(notifications_))
{}

const std::int32_t notificationGroup::ID;

object_ptr<notificationGroup> notificationGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationGroup> res = make_object<notificationGroup>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->type_ = jni::fetch_tl_object<NotificationGroupType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->notifications_ = jni::FetchVector<notification>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->notifications_fieldID));
  return res;
}

void notificationGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, notifications_); if (arr_tmp_) { env->SetObjectField(s, notifications_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void notificationGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationGroup");
    s.store_field("id", id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("chatId", chat_id_);
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("notifications", notifications_.size()); for (const auto &_value : notifications_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void notificationGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroup").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationGroupType;").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    notifications_fieldID = jni::get_field_id(env, Class, "notifications", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Notification;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass NotificationGroupType::Class;

object_ptr<NotificationGroupType> NotificationGroupType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case notificationGroupTypeMessages::ID:
      return notificationGroupTypeMessages::fetch(env, p);
    case notificationGroupTypeMentions::ID:
      return notificationGroupTypeMentions::fetch(env, p);
    case notificationGroupTypeSecretChat::ID:
      return notificationGroupTypeSecretChat::fetch(env, p);
    case notificationGroupTypeCalls::ID:
      return notificationGroupTypeCalls::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void NotificationGroupType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroupType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationGroupTypeMessages::Class;

notificationGroupTypeMessages::notificationGroupTypeMessages() {
}

const std::int32_t notificationGroupTypeMessages::ID;

object_ptr<NotificationGroupType> notificationGroupTypeMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationGroupTypeMessages>();
}

void notificationGroupTypeMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationGroupTypeMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationGroupTypeMessages");
    s.store_class_end();
  }
}

void notificationGroupTypeMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroupTypeMessages").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationGroupTypeMentions::Class;

notificationGroupTypeMentions::notificationGroupTypeMentions() {
}

const std::int32_t notificationGroupTypeMentions::ID;

object_ptr<NotificationGroupType> notificationGroupTypeMentions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationGroupTypeMentions>();
}

void notificationGroupTypeMentions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationGroupTypeMentions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationGroupTypeMentions");
    s.store_class_end();
  }
}

void notificationGroupTypeMentions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroupTypeMentions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationGroupTypeSecretChat::Class;

notificationGroupTypeSecretChat::notificationGroupTypeSecretChat() {
}

const std::int32_t notificationGroupTypeSecretChat::ID;

object_ptr<NotificationGroupType> notificationGroupTypeSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationGroupTypeSecretChat>();
}

void notificationGroupTypeSecretChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationGroupTypeSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationGroupTypeSecretChat");
    s.store_class_end();
  }
}

void notificationGroupTypeSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroupTypeSecretChat").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationGroupTypeCalls::Class;

notificationGroupTypeCalls::notificationGroupTypeCalls() {
}

const std::int32_t notificationGroupTypeCalls::ID;

object_ptr<NotificationGroupType> notificationGroupTypeCalls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationGroupTypeCalls>();
}

void notificationGroupTypeCalls::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationGroupTypeCalls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationGroupTypeCalls");
    s.store_class_end();
  }
}

void notificationGroupTypeCalls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationGroupTypeCalls").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass NotificationSettingsScope::Class;

object_ptr<NotificationSettingsScope> NotificationSettingsScope::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case notificationSettingsScopePrivateChats::ID:
      return notificationSettingsScopePrivateChats::fetch(env, p);
    case notificationSettingsScopeGroupChats::ID:
      return notificationSettingsScopeGroupChats::fetch(env, p);
    case notificationSettingsScopeChannelChats::ID:
      return notificationSettingsScopeChannelChats::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void NotificationSettingsScope::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSettingsScope").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationSettingsScopePrivateChats::Class;

notificationSettingsScopePrivateChats::notificationSettingsScopePrivateChats() {
}

const std::int32_t notificationSettingsScopePrivateChats::ID;

object_ptr<NotificationSettingsScope> notificationSettingsScopePrivateChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationSettingsScopePrivateChats>();
}

void notificationSettingsScopePrivateChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationSettingsScopePrivateChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationSettingsScopePrivateChats");
    s.store_class_end();
  }
}

void notificationSettingsScopePrivateChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSettingsScopePrivateChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationSettingsScopeGroupChats::Class;

notificationSettingsScopeGroupChats::notificationSettingsScopeGroupChats() {
}

const std::int32_t notificationSettingsScopeGroupChats::ID;

object_ptr<NotificationSettingsScope> notificationSettingsScopeGroupChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationSettingsScopeGroupChats>();
}

void notificationSettingsScopeGroupChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationSettingsScopeGroupChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationSettingsScopeGroupChats");
    s.store_class_end();
  }
}

void notificationSettingsScopeGroupChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSettingsScopeGroupChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationSettingsScopeChannelChats::Class;

notificationSettingsScopeChannelChats::notificationSettingsScopeChannelChats() {
}

const std::int32_t notificationSettingsScopeChannelChats::ID;

object_ptr<NotificationSettingsScope> notificationSettingsScopeChannelChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationSettingsScopeChannelChats>();
}

void notificationSettingsScopeChannelChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationSettingsScopeChannelChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationSettingsScopeChannelChats");
    s.store_class_end();
  }
}

void notificationSettingsScopeChannelChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSettingsScopeChannelChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationSound::Class;
jfieldID notificationSound::id_fieldID;
jfieldID notificationSound::duration_fieldID;
jfieldID notificationSound::date_fieldID;
jfieldID notificationSound::title_fieldID;
jfieldID notificationSound::data_fieldID;
jfieldID notificationSound::sound_fieldID;

notificationSound::notificationSound()
  : id_()
  , duration_()
  , date_()
  , title_()
  , data_()
  , sound_()
{}

notificationSound::notificationSound(int64 id_, int32 duration_, int32 date_, string const &title_, string const &data_, object_ptr<file> &&sound_)
  : id_(id_)
  , duration_(duration_)
  , date_(date_)
  , title_(title_)
  , data_(data_)
  , sound_(std::move(sound_))
{}

const std::int32_t notificationSound::ID;

object_ptr<notificationSound> notificationSound::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationSound> res = make_object<notificationSound>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  res->sound_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->sound_fieldID));
  return res;
}

void notificationSound::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, date_fieldID, date_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, data_); if (nextString) { env->SetObjectField(s, data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sound_ != nullptr) { jobject next; sound_->store(env, next); if (next) { env->SetObjectField(s, sound_fieldID, next); env->DeleteLocalRef(next); } }
}

void notificationSound::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationSound");
    s.store_field("id", id_);
    s.store_field("duration", duration_);
    s.store_field("date", date_);
    s.store_field("title", title_);
    s.store_field("data", data_);
    s.store_object_field("sound", static_cast<const BaseObject *>(sound_.get()));
    s.store_class_end();
  }
}

void notificationSound::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSound").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    sound_fieldID = jni::get_field_id(env, Class, "sound", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationSounds::Class;
jfieldID notificationSounds::notification_sounds_fieldID;

notificationSounds::notificationSounds()
  : notification_sounds_()
{}

notificationSounds::notificationSounds(array<object_ptr<notificationSound>> &&notification_sounds_)
  : notification_sounds_(std::move(notification_sounds_))
{}

const std::int32_t notificationSounds::ID;

object_ptr<notificationSounds> notificationSounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationSounds> res = make_object<notificationSounds>();
  res->notification_sounds_ = jni::FetchVector<notificationSound>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->notification_sounds_fieldID));
  return res;
}

void notificationSounds::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, notification_sounds_); if (arr_tmp_) { env->SetObjectField(s, notification_sounds_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void notificationSounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationSounds");
    { s.store_vector_begin("notificationSounds", notification_sounds_.size()); for (const auto &_value : notification_sounds_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void notificationSounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationSounds").c_str());
    notification_sounds_fieldID = jni::get_field_id(env, Class, "notificationSounds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$NotificationSound;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass NotificationType::Class;

object_ptr<NotificationType> NotificationType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case notificationTypeNewMessage::ID:
      return notificationTypeNewMessage::fetch(env, p);
    case notificationTypeNewSecretChat::ID:
      return notificationTypeNewSecretChat::fetch(env, p);
    case notificationTypeNewCall::ID:
      return notificationTypeNewCall::fetch(env, p);
    case notificationTypeNewPushMessage::ID:
      return notificationTypeNewPushMessage::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void NotificationType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationTypeNewMessage::Class;
jfieldID notificationTypeNewMessage::message_fieldID;
jfieldID notificationTypeNewMessage::show_preview_fieldID;

notificationTypeNewMessage::notificationTypeNewMessage()
  : message_()
  , show_preview_()
{}

notificationTypeNewMessage::notificationTypeNewMessage(object_ptr<message> &&message_, bool show_preview_)
  : message_(std::move(message_))
  , show_preview_(show_preview_)
{}

const std::int32_t notificationTypeNewMessage::ID;

object_ptr<NotificationType> notificationTypeNewMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationTypeNewMessage> res = make_object<notificationTypeNewMessage>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->show_preview_ = (env->GetBooleanField(p, res->show_preview_fieldID) != 0);
  return std::move(res);
}

void notificationTypeNewMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, show_preview_fieldID, show_preview_);
}

void notificationTypeNewMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationTypeNewMessage");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("showPreview", show_preview_);
    s.store_class_end();
  }
}

void notificationTypeNewMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationTypeNewMessage").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    show_preview_fieldID = jni::get_field_id(env, Class, "showPreview", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass notificationTypeNewSecretChat::Class;

notificationTypeNewSecretChat::notificationTypeNewSecretChat() {
}

const std::int32_t notificationTypeNewSecretChat::ID;

object_ptr<NotificationType> notificationTypeNewSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<notificationTypeNewSecretChat>();
}

void notificationTypeNewSecretChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void notificationTypeNewSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationTypeNewSecretChat");
    s.store_class_end();
  }
}

void notificationTypeNewSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationTypeNewSecretChat").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass notificationTypeNewCall::Class;
jfieldID notificationTypeNewCall::call_id_fieldID;

notificationTypeNewCall::notificationTypeNewCall()
  : call_id_()
{}

notificationTypeNewCall::notificationTypeNewCall(int32 call_id_)
  : call_id_(call_id_)
{}

const std::int32_t notificationTypeNewCall::ID;

object_ptr<NotificationType> notificationTypeNewCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationTypeNewCall> res = make_object<notificationTypeNewCall>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  return std::move(res);
}

void notificationTypeNewCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, call_id_fieldID, call_id_);
}

void notificationTypeNewCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationTypeNewCall");
    s.store_field("callId", call_id_);
    s.store_class_end();
  }
}

void notificationTypeNewCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationTypeNewCall").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass notificationTypeNewPushMessage::Class;
jfieldID notificationTypeNewPushMessage::message_id_fieldID;
jfieldID notificationTypeNewPushMessage::sender_id_fieldID;
jfieldID notificationTypeNewPushMessage::sender_name_fieldID;
jfieldID notificationTypeNewPushMessage::is_outgoing_fieldID;
jfieldID notificationTypeNewPushMessage::content_fieldID;

notificationTypeNewPushMessage::notificationTypeNewPushMessage()
  : message_id_()
  , sender_id_()
  , sender_name_()
  , is_outgoing_()
  , content_()
{}

notificationTypeNewPushMessage::notificationTypeNewPushMessage(int53 message_id_, object_ptr<MessageSender> &&sender_id_, string const &sender_name_, bool is_outgoing_, object_ptr<PushMessageContent> &&content_)
  : message_id_(message_id_)
  , sender_id_(std::move(sender_id_))
  , sender_name_(sender_name_)
  , is_outgoing_(is_outgoing_)
  , content_(std::move(content_))
{}

const std::int32_t notificationTypeNewPushMessage::ID;

object_ptr<NotificationType> notificationTypeNewPushMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<notificationTypeNewPushMessage> res = make_object<notificationTypeNewPushMessage>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->sender_name_ = jni::fetch_string(env, p, res->sender_name_fieldID);
  res->is_outgoing_ = (env->GetBooleanField(p, res->is_outgoing_fieldID) != 0);
  res->content_ = jni::fetch_tl_object<PushMessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return std::move(res);
}

void notificationTypeNewPushMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, sender_name_); if (nextString) { env->SetObjectField(s, sender_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_outgoing_fieldID, is_outgoing_);
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
}

void notificationTypeNewPushMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "NotificationTypeNewPushMessage");
    s.store_field("messageId", message_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("senderName", sender_name_);
    s.store_field("isOutgoing", is_outgoing_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

void notificationTypeNewPushMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$NotificationTypeNewPushMessage").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    sender_name_fieldID = jni::get_field_id(env, Class, "senderName", "Ljava/lang/String;");
    is_outgoing_fieldID = jni::get_field_id(env, Class, "isOutgoing", "Z");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PushMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ok::Class;

ok::ok() {
}

const std::int32_t ok::ID;

object_ptr<ok> ok::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<ok>();
}

void ok::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void ok::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Ok");
    s.store_class_end();
  }
}

void ok::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Ok").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass OptionValue::Class;

object_ptr<OptionValue> OptionValue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case optionValueBoolean::ID:
      return optionValueBoolean::fetch(env, p);
    case optionValueEmpty::ID:
      return optionValueEmpty::fetch(env, p);
    case optionValueInteger::ID:
      return optionValueInteger::fetch(env, p);
    case optionValueString::ID:
      return optionValueString::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void OptionValue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptionValue").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass optionValueBoolean::Class;
jfieldID optionValueBoolean::value_fieldID;

optionValueBoolean::optionValueBoolean()
  : value_()
{}

optionValueBoolean::optionValueBoolean(bool value_)
  : value_(value_)
{}

const std::int32_t optionValueBoolean::ID;

object_ptr<OptionValue> optionValueBoolean::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<optionValueBoolean> res = make_object<optionValueBoolean>();
  res->value_ = (env->GetBooleanField(p, res->value_fieldID) != 0);
  return std::move(res);
}

void optionValueBoolean::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, value_fieldID, value_);
}

void optionValueBoolean::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OptionValueBoolean");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void optionValueBoolean::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptionValueBoolean").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass optionValueEmpty::Class;

optionValueEmpty::optionValueEmpty() {
}

const std::int32_t optionValueEmpty::ID;

object_ptr<OptionValue> optionValueEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<optionValueEmpty>();
}

void optionValueEmpty::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void optionValueEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OptionValueEmpty");
    s.store_class_end();
  }
}

void optionValueEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptionValueEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass optionValueInteger::Class;
jfieldID optionValueInteger::value_fieldID;

optionValueInteger::optionValueInteger()
  : value_()
{}

optionValueInteger::optionValueInteger(int64 value_)
  : value_(value_)
{}

const std::int32_t optionValueInteger::ID;

object_ptr<OptionValue> optionValueInteger::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<optionValueInteger> res = make_object<optionValueInteger>();
  res->value_ = env->GetLongField(p, res->value_fieldID);
  return std::move(res);
}

void optionValueInteger::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, value_fieldID, value_);
}

void optionValueInteger::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OptionValueInteger");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void optionValueInteger::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptionValueInteger").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "J");
    return true;
  }();
  (void)is_inited;
}

jclass optionValueString::Class;
jfieldID optionValueString::value_fieldID;

optionValueString::optionValueString()
  : value_()
{}

optionValueString::optionValueString(string const &value_)
  : value_(value_)
{}

const std::int32_t optionValueString::ID;

object_ptr<OptionValue> optionValueString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<optionValueString> res = make_object<optionValueString>();
  res->value_ = jni::fetch_string(env, p, res->value_fieldID);
  return std::move(res);
}

void optionValueString::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, value_); if (nextString) { env->SetObjectField(s, value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void optionValueString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OptionValueString");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void optionValueString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptionValueString").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass orderInfo::Class;
jfieldID orderInfo::name_fieldID;
jfieldID orderInfo::phone_number_fieldID;
jfieldID orderInfo::email_address_fieldID;
jfieldID orderInfo::shipping_address_fieldID;

orderInfo::orderInfo()
  : name_()
  , phone_number_()
  , email_address_()
  , shipping_address_()
{}

orderInfo::orderInfo(string const &name_, string const &phone_number_, string const &email_address_, object_ptr<address> &&shipping_address_)
  : name_(name_)
  , phone_number_(phone_number_)
  , email_address_(email_address_)
  , shipping_address_(std::move(shipping_address_))
{}

const std::int32_t orderInfo::ID;

object_ptr<orderInfo> orderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<orderInfo> res = make_object<orderInfo>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  res->shipping_address_ = jni::fetch_tl_object<address>(env, jni::fetch_object(env, p, res->shipping_address_fieldID));
  return res;
}

void orderInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, email_address_); if (nextString) { env->SetObjectField(s, email_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (shipping_address_ != nullptr) { jobject next; shipping_address_->store(env, next); if (next) { env->SetObjectField(s, shipping_address_fieldID, next); env->DeleteLocalRef(next); } }
}

void orderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OrderInfo");
    s.store_field("name", name_);
    s.store_field("phoneNumber", phone_number_);
    s.store_field("emailAddress", email_address_);
    s.store_object_field("shippingAddress", static_cast<const BaseObject *>(shipping_address_.get()));
    s.store_class_end();
  }
}

void orderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OrderInfo").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    shipping_address_fieldID = jni::get_field_id(env, Class, "shippingAddress", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Address;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass outline::Class;
jfieldID outline::paths_fieldID;

outline::outline()
  : paths_()
{}

outline::outline(array<object_ptr<closedVectorPath>> &&paths_)
  : paths_(std::move(paths_))
{}

const std::int32_t outline::ID;

object_ptr<outline> outline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<outline> res = make_object<outline>();
  res->paths_ = jni::FetchVector<closedVectorPath>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->paths_fieldID));
  return res;
}

void outline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, paths_); if (arr_tmp_) { env->SetObjectField(s, paths_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void outline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Outline");
    { s.store_vector_begin("paths", paths_.size()); for (const auto &_value : paths_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void outline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Outline").c_str());
    paths_fieldID = jni::get_field_id(env, Class, "paths", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ClosedVectorPath;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PageBlock::Class;

object_ptr<PageBlock> PageBlock::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case pageBlockTitle::ID:
      return pageBlockTitle::fetch(env, p);
    case pageBlockSubtitle::ID:
      return pageBlockSubtitle::fetch(env, p);
    case pageBlockAuthorDate::ID:
      return pageBlockAuthorDate::fetch(env, p);
    case pageBlockHeader::ID:
      return pageBlockHeader::fetch(env, p);
    case pageBlockSubheader::ID:
      return pageBlockSubheader::fetch(env, p);
    case pageBlockKicker::ID:
      return pageBlockKicker::fetch(env, p);
    case pageBlockParagraph::ID:
      return pageBlockParagraph::fetch(env, p);
    case pageBlockPreformatted::ID:
      return pageBlockPreformatted::fetch(env, p);
    case pageBlockFooter::ID:
      return pageBlockFooter::fetch(env, p);
    case pageBlockDivider::ID:
      return pageBlockDivider::fetch(env, p);
    case pageBlockAnchor::ID:
      return pageBlockAnchor::fetch(env, p);
    case pageBlockList::ID:
      return pageBlockList::fetch(env, p);
    case pageBlockBlockQuote::ID:
      return pageBlockBlockQuote::fetch(env, p);
    case pageBlockPullQuote::ID:
      return pageBlockPullQuote::fetch(env, p);
    case pageBlockAnimation::ID:
      return pageBlockAnimation::fetch(env, p);
    case pageBlockAudio::ID:
      return pageBlockAudio::fetch(env, p);
    case pageBlockPhoto::ID:
      return pageBlockPhoto::fetch(env, p);
    case pageBlockVideo::ID:
      return pageBlockVideo::fetch(env, p);
    case pageBlockVoiceNote::ID:
      return pageBlockVoiceNote::fetch(env, p);
    case pageBlockCover::ID:
      return pageBlockCover::fetch(env, p);
    case pageBlockEmbedded::ID:
      return pageBlockEmbedded::fetch(env, p);
    case pageBlockEmbeddedPost::ID:
      return pageBlockEmbeddedPost::fetch(env, p);
    case pageBlockCollage::ID:
      return pageBlockCollage::fetch(env, p);
    case pageBlockSlideshow::ID:
      return pageBlockSlideshow::fetch(env, p);
    case pageBlockChatLink::ID:
      return pageBlockChatLink::fetch(env, p);
    case pageBlockTable::ID:
      return pageBlockTable::fetch(env, p);
    case pageBlockDetails::ID:
      return pageBlockDetails::fetch(env, p);
    case pageBlockRelatedArticles::ID:
      return pageBlockRelatedArticles::fetch(env, p);
    case pageBlockMap::ID:
      return pageBlockMap::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PageBlock::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlock").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockTitle::Class;
jfieldID pageBlockTitle::title_fieldID;

pageBlockTitle::pageBlockTitle()
  : title_()
{}

pageBlockTitle::pageBlockTitle(object_ptr<RichText> &&title_)
  : title_(std::move(title_))
{}

const std::int32_t pageBlockTitle::ID;

object_ptr<PageBlock> pageBlockTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockTitle> res = make_object<pageBlockTitle>();
  res->title_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->title_fieldID));
  return std::move(res);
}

void pageBlockTitle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (title_ != nullptr) { jobject next; title_->store(env, next); if (next) { env->SetObjectField(s, title_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockTitle");
    s.store_object_field("title", static_cast<const BaseObject *>(title_.get()));
    s.store_class_end();
  }
}

void pageBlockTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockTitle").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockSubtitle::Class;
jfieldID pageBlockSubtitle::subtitle_fieldID;

pageBlockSubtitle::pageBlockSubtitle()
  : subtitle_()
{}

pageBlockSubtitle::pageBlockSubtitle(object_ptr<RichText> &&subtitle_)
  : subtitle_(std::move(subtitle_))
{}

const std::int32_t pageBlockSubtitle::ID;

object_ptr<PageBlock> pageBlockSubtitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockSubtitle> res = make_object<pageBlockSubtitle>();
  res->subtitle_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->subtitle_fieldID));
  return std::move(res);
}

void pageBlockSubtitle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (subtitle_ != nullptr) { jobject next; subtitle_->store(env, next); if (next) { env->SetObjectField(s, subtitle_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockSubtitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockSubtitle");
    s.store_object_field("subtitle", static_cast<const BaseObject *>(subtitle_.get()));
    s.store_class_end();
  }
}

void pageBlockSubtitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockSubtitle").c_str());
    subtitle_fieldID = jni::get_field_id(env, Class, "subtitle", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockAuthorDate::Class;
jfieldID pageBlockAuthorDate::author_fieldID;
jfieldID pageBlockAuthorDate::publish_date_fieldID;

pageBlockAuthorDate::pageBlockAuthorDate()
  : author_()
  , publish_date_()
{}

pageBlockAuthorDate::pageBlockAuthorDate(object_ptr<RichText> &&author_, int32 publish_date_)
  : author_(std::move(author_))
  , publish_date_(publish_date_)
{}

const std::int32_t pageBlockAuthorDate::ID;

object_ptr<PageBlock> pageBlockAuthorDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockAuthorDate> res = make_object<pageBlockAuthorDate>();
  res->author_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->author_fieldID));
  res->publish_date_ = env->GetIntField(p, res->publish_date_fieldID);
  return std::move(res);
}

void pageBlockAuthorDate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (author_ != nullptr) { jobject next; author_->store(env, next); if (next) { env->SetObjectField(s, author_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, publish_date_fieldID, publish_date_);
}

void pageBlockAuthorDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockAuthorDate");
    s.store_object_field("author", static_cast<const BaseObject *>(author_.get()));
    s.store_field("publishDate", publish_date_);
    s.store_class_end();
  }
}

void pageBlockAuthorDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockAuthorDate").c_str());
    author_fieldID = jni::get_field_id(env, Class, "author", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    publish_date_fieldID = jni::get_field_id(env, Class, "publishDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockHeader::Class;
jfieldID pageBlockHeader::header_fieldID;

pageBlockHeader::pageBlockHeader()
  : header_()
{}

pageBlockHeader::pageBlockHeader(object_ptr<RichText> &&header_)
  : header_(std::move(header_))
{}

const std::int32_t pageBlockHeader::ID;

object_ptr<PageBlock> pageBlockHeader::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockHeader> res = make_object<pageBlockHeader>();
  res->header_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->header_fieldID));
  return std::move(res);
}

void pageBlockHeader::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (header_ != nullptr) { jobject next; header_->store(env, next); if (next) { env->SetObjectField(s, header_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockHeader::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockHeader");
    s.store_object_field("header", static_cast<const BaseObject *>(header_.get()));
    s.store_class_end();
  }
}

void pageBlockHeader::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockHeader").c_str());
    header_fieldID = jni::get_field_id(env, Class, "header", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockSubheader::Class;
jfieldID pageBlockSubheader::subheader_fieldID;

pageBlockSubheader::pageBlockSubheader()
  : subheader_()
{}

pageBlockSubheader::pageBlockSubheader(object_ptr<RichText> &&subheader_)
  : subheader_(std::move(subheader_))
{}

const std::int32_t pageBlockSubheader::ID;

object_ptr<PageBlock> pageBlockSubheader::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockSubheader> res = make_object<pageBlockSubheader>();
  res->subheader_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->subheader_fieldID));
  return std::move(res);
}

void pageBlockSubheader::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (subheader_ != nullptr) { jobject next; subheader_->store(env, next); if (next) { env->SetObjectField(s, subheader_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockSubheader::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockSubheader");
    s.store_object_field("subheader", static_cast<const BaseObject *>(subheader_.get()));
    s.store_class_end();
  }
}

void pageBlockSubheader::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockSubheader").c_str());
    subheader_fieldID = jni::get_field_id(env, Class, "subheader", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockKicker::Class;
jfieldID pageBlockKicker::kicker_fieldID;

pageBlockKicker::pageBlockKicker()
  : kicker_()
{}

pageBlockKicker::pageBlockKicker(object_ptr<RichText> &&kicker_)
  : kicker_(std::move(kicker_))
{}

const std::int32_t pageBlockKicker::ID;

object_ptr<PageBlock> pageBlockKicker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockKicker> res = make_object<pageBlockKicker>();
  res->kicker_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->kicker_fieldID));
  return std::move(res);
}

void pageBlockKicker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (kicker_ != nullptr) { jobject next; kicker_->store(env, next); if (next) { env->SetObjectField(s, kicker_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockKicker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockKicker");
    s.store_object_field("kicker", static_cast<const BaseObject *>(kicker_.get()));
    s.store_class_end();
  }
}

void pageBlockKicker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockKicker").c_str());
    kicker_fieldID = jni::get_field_id(env, Class, "kicker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockParagraph::Class;
jfieldID pageBlockParagraph::text_fieldID;

pageBlockParagraph::pageBlockParagraph()
  : text_()
{}

pageBlockParagraph::pageBlockParagraph(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t pageBlockParagraph::ID;

object_ptr<PageBlock> pageBlockParagraph::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockParagraph> res = make_object<pageBlockParagraph>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void pageBlockParagraph::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockParagraph::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockParagraph");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void pageBlockParagraph::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockParagraph").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockPreformatted::Class;
jfieldID pageBlockPreformatted::text_fieldID;
jfieldID pageBlockPreformatted::language_fieldID;

pageBlockPreformatted::pageBlockPreformatted()
  : text_()
  , language_()
{}

pageBlockPreformatted::pageBlockPreformatted(object_ptr<RichText> &&text_, string const &language_)
  : text_(std::move(text_))
  , language_(language_)
{}

const std::int32_t pageBlockPreformatted::ID;

object_ptr<PageBlock> pageBlockPreformatted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockPreformatted> res = make_object<pageBlockPreformatted>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->language_ = jni::fetch_string(env, p, res->language_fieldID);
  return std::move(res);
}

void pageBlockPreformatted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, language_); if (nextString) { env->SetObjectField(s, language_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pageBlockPreformatted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockPreformatted");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("language", language_);
    s.store_class_end();
  }
}

void pageBlockPreformatted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockPreformatted").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    language_fieldID = jni::get_field_id(env, Class, "language", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockFooter::Class;
jfieldID pageBlockFooter::footer_fieldID;

pageBlockFooter::pageBlockFooter()
  : footer_()
{}

pageBlockFooter::pageBlockFooter(object_ptr<RichText> &&footer_)
  : footer_(std::move(footer_))
{}

const std::int32_t pageBlockFooter::ID;

object_ptr<PageBlock> pageBlockFooter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockFooter> res = make_object<pageBlockFooter>();
  res->footer_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->footer_fieldID));
  return std::move(res);
}

void pageBlockFooter::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (footer_ != nullptr) { jobject next; footer_->store(env, next); if (next) { env->SetObjectField(s, footer_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockFooter::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockFooter");
    s.store_object_field("footer", static_cast<const BaseObject *>(footer_.get()));
    s.store_class_end();
  }
}

void pageBlockFooter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockFooter").c_str());
    footer_fieldID = jni::get_field_id(env, Class, "footer", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockDivider::Class;

pageBlockDivider::pageBlockDivider() {
}

const std::int32_t pageBlockDivider::ID;

object_ptr<PageBlock> pageBlockDivider::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockDivider>();
}

void pageBlockDivider::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockDivider::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockDivider");
    s.store_class_end();
  }
}

void pageBlockDivider::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockDivider").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockAnchor::Class;
jfieldID pageBlockAnchor::name_fieldID;

pageBlockAnchor::pageBlockAnchor()
  : name_()
{}

pageBlockAnchor::pageBlockAnchor(string const &name_)
  : name_(name_)
{}

const std::int32_t pageBlockAnchor::ID;

object_ptr<PageBlock> pageBlockAnchor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockAnchor> res = make_object<pageBlockAnchor>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return std::move(res);
}

void pageBlockAnchor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pageBlockAnchor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockAnchor");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void pageBlockAnchor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockAnchor").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockList::Class;
jfieldID pageBlockList::items_fieldID;

pageBlockList::pageBlockList()
  : items_()
{}

pageBlockList::pageBlockList(array<object_ptr<pageBlockListItem>> &&items_)
  : items_(std::move(items_))
{}

const std::int32_t pageBlockList::ID;

object_ptr<PageBlock> pageBlockList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockList> res = make_object<pageBlockList>();
  res->items_ = jni::FetchVector<pageBlockListItem>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->items_fieldID));
  return std::move(res);
}

void pageBlockList::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, items_); if (arr_tmp_) { env->SetObjectField(s, items_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void pageBlockList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockList");
    { s.store_vector_begin("items", items_.size()); for (const auto &_value : items_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void pageBlockList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockList").c_str());
    items_fieldID = jni::get_field_id(env, Class, "items", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlockListItem;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockBlockQuote::Class;
jfieldID pageBlockBlockQuote::text_fieldID;
jfieldID pageBlockBlockQuote::credit_fieldID;

pageBlockBlockQuote::pageBlockBlockQuote()
  : text_()
  , credit_()
{}

pageBlockBlockQuote::pageBlockBlockQuote(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_)
  : text_(std::move(text_))
  , credit_(std::move(credit_))
{}

const std::int32_t pageBlockBlockQuote::ID;

object_ptr<PageBlock> pageBlockBlockQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockBlockQuote> res = make_object<pageBlockBlockQuote>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->credit_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->credit_fieldID));
  return std::move(res);
}

void pageBlockBlockQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  if (credit_ != nullptr) { jobject next; credit_->store(env, next); if (next) { env->SetObjectField(s, credit_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockBlockQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockBlockQuote");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("credit", static_cast<const BaseObject *>(credit_.get()));
    s.store_class_end();
  }
}

void pageBlockBlockQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockBlockQuote").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    credit_fieldID = jni::get_field_id(env, Class, "credit", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockPullQuote::Class;
jfieldID pageBlockPullQuote::text_fieldID;
jfieldID pageBlockPullQuote::credit_fieldID;

pageBlockPullQuote::pageBlockPullQuote()
  : text_()
  , credit_()
{}

pageBlockPullQuote::pageBlockPullQuote(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_)
  : text_(std::move(text_))
  , credit_(std::move(credit_))
{}

const std::int32_t pageBlockPullQuote::ID;

object_ptr<PageBlock> pageBlockPullQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockPullQuote> res = make_object<pageBlockPullQuote>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->credit_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->credit_fieldID));
  return std::move(res);
}

void pageBlockPullQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  if (credit_ != nullptr) { jobject next; credit_->store(env, next); if (next) { env->SetObjectField(s, credit_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockPullQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockPullQuote");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("credit", static_cast<const BaseObject *>(credit_.get()));
    s.store_class_end();
  }
}

void pageBlockPullQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockPullQuote").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    credit_fieldID = jni::get_field_id(env, Class, "credit", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockAnimation::Class;
jfieldID pageBlockAnimation::animation_fieldID;
jfieldID pageBlockAnimation::caption_fieldID;
jfieldID pageBlockAnimation::need_autoplay_fieldID;

pageBlockAnimation::pageBlockAnimation()
  : animation_()
  , caption_()
  , need_autoplay_()
{}

pageBlockAnimation::pageBlockAnimation(object_ptr<animation> &&animation_, object_ptr<pageBlockCaption> &&caption_, bool need_autoplay_)
  : animation_(std::move(animation_))
  , caption_(std::move(caption_))
  , need_autoplay_(need_autoplay_)
{}

const std::int32_t pageBlockAnimation::ID;

object_ptr<PageBlock> pageBlockAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockAnimation> res = make_object<pageBlockAnimation>();
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->need_autoplay_ = (env->GetBooleanField(p, res->need_autoplay_fieldID) != 0);
  return std::move(res);
}

void pageBlockAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, need_autoplay_fieldID, need_autoplay_);
}

void pageBlockAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("needAutoplay", need_autoplay_);
    s.store_class_end();
  }
}

void pageBlockAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    need_autoplay_fieldID = jni::get_field_id(env, Class, "needAutoplay", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockAudio::Class;
jfieldID pageBlockAudio::audio_fieldID;
jfieldID pageBlockAudio::caption_fieldID;

pageBlockAudio::pageBlockAudio()
  : audio_()
  , caption_()
{}

pageBlockAudio::pageBlockAudio(object_ptr<audio> &&audio_, object_ptr<pageBlockCaption> &&caption_)
  : audio_(std::move(audio_))
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockAudio::ID;

object_ptr<PageBlock> pageBlockAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockAudio> res = make_object<pageBlockAudio>();
  res->audio_ = jni::fetch_tl_object<audio>(env, jni::fetch_object(env, p, res->audio_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockAudio");
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockAudio").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Audio;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockPhoto::Class;
jfieldID pageBlockPhoto::photo_fieldID;
jfieldID pageBlockPhoto::caption_fieldID;
jfieldID pageBlockPhoto::url_fieldID;

pageBlockPhoto::pageBlockPhoto()
  : photo_()
  , caption_()
  , url_()
{}

pageBlockPhoto::pageBlockPhoto(object_ptr<photo> &&photo_, object_ptr<pageBlockCaption> &&caption_, string const &url_)
  : photo_(std::move(photo_))
  , caption_(std::move(caption_))
  , url_(url_)
{}

const std::int32_t pageBlockPhoto::ID;

object_ptr<PageBlock> pageBlockPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockPhoto> res = make_object<pageBlockPhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void pageBlockPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pageBlockPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void pageBlockPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockVideo::Class;
jfieldID pageBlockVideo::video_fieldID;
jfieldID pageBlockVideo::caption_fieldID;
jfieldID pageBlockVideo::need_autoplay_fieldID;
jfieldID pageBlockVideo::is_looped_fieldID;

pageBlockVideo::pageBlockVideo()
  : video_()
  , caption_()
  , need_autoplay_()
  , is_looped_()
{}

pageBlockVideo::pageBlockVideo(object_ptr<video> &&video_, object_ptr<pageBlockCaption> &&caption_, bool need_autoplay_, bool is_looped_)
  : video_(std::move(video_))
  , caption_(std::move(caption_))
  , need_autoplay_(need_autoplay_)
  , is_looped_(is_looped_)
{}

const std::int32_t pageBlockVideo::ID;

object_ptr<PageBlock> pageBlockVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockVideo> res = make_object<pageBlockVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->need_autoplay_ = (env->GetBooleanField(p, res->need_autoplay_fieldID) != 0);
  res->is_looped_ = (env->GetBooleanField(p, res->is_looped_fieldID) != 0);
  return std::move(res);
}

void pageBlockVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, need_autoplay_fieldID, need_autoplay_);
  env->SetBooleanField(s, is_looped_fieldID, is_looped_);
}

void pageBlockVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("needAutoplay", need_autoplay_);
    s.store_field("isLooped", is_looped_);
    s.store_class_end();
  }
}

void pageBlockVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    need_autoplay_fieldID = jni::get_field_id(env, Class, "needAutoplay", "Z");
    is_looped_fieldID = jni::get_field_id(env, Class, "isLooped", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockVoiceNote::Class;
jfieldID pageBlockVoiceNote::voice_note_fieldID;
jfieldID pageBlockVoiceNote::caption_fieldID;

pageBlockVoiceNote::pageBlockVoiceNote()
  : voice_note_()
  , caption_()
{}

pageBlockVoiceNote::pageBlockVoiceNote(object_ptr<voiceNote> &&voice_note_, object_ptr<pageBlockCaption> &&caption_)
  : voice_note_(std::move(voice_note_))
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockVoiceNote::ID;

object_ptr<PageBlock> pageBlockVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockVoiceNote> res = make_object<pageBlockVoiceNote>();
  res->voice_note_ = jni::fetch_tl_object<voiceNote>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockVoiceNote");
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVoiceNote").c_str());
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VoiceNote;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockCover::Class;
jfieldID pageBlockCover::cover_fieldID;

pageBlockCover::pageBlockCover()
  : cover_()
{}

pageBlockCover::pageBlockCover(object_ptr<PageBlock> &&cover_)
  : cover_(std::move(cover_))
{}

const std::int32_t pageBlockCover::ID;

object_ptr<PageBlock> pageBlockCover::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockCover> res = make_object<pageBlockCover>();
  res->cover_ = jni::fetch_tl_object<PageBlock>(env, jni::fetch_object(env, p, res->cover_fieldID));
  return std::move(res);
}

void pageBlockCover::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockCover::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockCover");
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_class_end();
  }
}

void pageBlockCover::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockCover").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockEmbedded::Class;
jfieldID pageBlockEmbedded::url_fieldID;
jfieldID pageBlockEmbedded::html_fieldID;
jfieldID pageBlockEmbedded::poster_photo_fieldID;
jfieldID pageBlockEmbedded::width_fieldID;
jfieldID pageBlockEmbedded::height_fieldID;
jfieldID pageBlockEmbedded::caption_fieldID;
jfieldID pageBlockEmbedded::is_full_width_fieldID;
jfieldID pageBlockEmbedded::allow_scrolling_fieldID;

pageBlockEmbedded::pageBlockEmbedded()
  : url_()
  , html_()
  , poster_photo_()
  , width_()
  , height_()
  , caption_()
  , is_full_width_()
  , allow_scrolling_()
{}

pageBlockEmbedded::pageBlockEmbedded(string const &url_, string const &html_, object_ptr<photo> &&poster_photo_, int32 width_, int32 height_, object_ptr<pageBlockCaption> &&caption_, bool is_full_width_, bool allow_scrolling_)
  : url_(url_)
  , html_(html_)
  , poster_photo_(std::move(poster_photo_))
  , width_(width_)
  , height_(height_)
  , caption_(std::move(caption_))
  , is_full_width_(is_full_width_)
  , allow_scrolling_(allow_scrolling_)
{}

const std::int32_t pageBlockEmbedded::ID;

object_ptr<PageBlock> pageBlockEmbedded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockEmbedded> res = make_object<pageBlockEmbedded>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->html_ = jni::fetch_string(env, p, res->html_fieldID);
  res->poster_photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->poster_photo_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->is_full_width_ = (env->GetBooleanField(p, res->is_full_width_fieldID) != 0);
  res->allow_scrolling_ = (env->GetBooleanField(p, res->allow_scrolling_fieldID) != 0);
  return std::move(res);
}

void pageBlockEmbedded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, html_); if (nextString) { env->SetObjectField(s, html_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (poster_photo_ != nullptr) { jobject next; poster_photo_->store(env, next); if (next) { env->SetObjectField(s, poster_photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_full_width_fieldID, is_full_width_);
  env->SetBooleanField(s, allow_scrolling_fieldID, allow_scrolling_);
}

void pageBlockEmbedded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockEmbedded");
    s.store_field("url", url_);
    s.store_field("html", html_);
    s.store_object_field("posterPhoto", static_cast<const BaseObject *>(poster_photo_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("isFullWidth", is_full_width_);
    s.store_field("allowScrolling", allow_scrolling_);
    s.store_class_end();
  }
}

void pageBlockEmbedded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockEmbedded").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    html_fieldID = jni::get_field_id(env, Class, "html", "Ljava/lang/String;");
    poster_photo_fieldID = jni::get_field_id(env, Class, "posterPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    is_full_width_fieldID = jni::get_field_id(env, Class, "isFullWidth", "Z");
    allow_scrolling_fieldID = jni::get_field_id(env, Class, "allowScrolling", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockEmbeddedPost::Class;
jfieldID pageBlockEmbeddedPost::url_fieldID;
jfieldID pageBlockEmbeddedPost::author_fieldID;
jfieldID pageBlockEmbeddedPost::author_photo_fieldID;
jfieldID pageBlockEmbeddedPost::date_fieldID;
jfieldID pageBlockEmbeddedPost::page_blocks_fieldID;
jfieldID pageBlockEmbeddedPost::caption_fieldID;

pageBlockEmbeddedPost::pageBlockEmbeddedPost()
  : url_()
  , author_()
  , author_photo_()
  , date_()
  , page_blocks_()
  , caption_()
{}

pageBlockEmbeddedPost::pageBlockEmbeddedPost(string const &url_, string const &author_, object_ptr<photo> &&author_photo_, int32 date_, array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_)
  : url_(url_)
  , author_(author_)
  , author_photo_(std::move(author_photo_))
  , date_(date_)
  , page_blocks_(std::move(page_blocks_))
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockEmbeddedPost::ID;

object_ptr<PageBlock> pageBlockEmbeddedPost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockEmbeddedPost> res = make_object<pageBlockEmbeddedPost>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->author_ = jni::fetch_string(env, p, res->author_fieldID);
  res->author_photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->author_photo_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockEmbeddedPost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, author_); if (nextString) { env->SetObjectField(s, author_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (author_photo_ != nullptr) { jobject next; author_photo_->store(env, next); if (next) { env->SetObjectField(s, author_photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockEmbeddedPost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockEmbeddedPost");
    s.store_field("url", url_);
    s.store_field("author", author_);
    s.store_object_field("authorPhoto", static_cast<const BaseObject *>(author_photo_.get()));
    s.store_field("date", date_);
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockEmbeddedPost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockEmbeddedPost").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    author_fieldID = jni::get_field_id(env, Class, "author", "Ljava/lang/String;");
    author_photo_fieldID = jni::get_field_id(env, Class, "authorPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockCollage::Class;
jfieldID pageBlockCollage::page_blocks_fieldID;
jfieldID pageBlockCollage::caption_fieldID;

pageBlockCollage::pageBlockCollage()
  : page_blocks_()
  , caption_()
{}

pageBlockCollage::pageBlockCollage(array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_)
  : page_blocks_(std::move(page_blocks_))
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockCollage::ID;

object_ptr<PageBlock> pageBlockCollage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockCollage> res = make_object<pageBlockCollage>();
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockCollage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockCollage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockCollage");
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockCollage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockCollage").c_str());
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockSlideshow::Class;
jfieldID pageBlockSlideshow::page_blocks_fieldID;
jfieldID pageBlockSlideshow::caption_fieldID;

pageBlockSlideshow::pageBlockSlideshow()
  : page_blocks_()
  , caption_()
{}

pageBlockSlideshow::pageBlockSlideshow(array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_)
  : page_blocks_(std::move(page_blocks_))
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockSlideshow::ID;

object_ptr<PageBlock> pageBlockSlideshow::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockSlideshow> res = make_object<pageBlockSlideshow>();
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockSlideshow::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockSlideshow::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockSlideshow");
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockSlideshow::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockSlideshow").c_str());
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockChatLink::Class;
jfieldID pageBlockChatLink::title_fieldID;
jfieldID pageBlockChatLink::photo_fieldID;
jfieldID pageBlockChatLink::accent_color_id_fieldID;
jfieldID pageBlockChatLink::username_fieldID;

pageBlockChatLink::pageBlockChatLink()
  : title_()
  , photo_()
  , accent_color_id_()
  , username_()
{}

pageBlockChatLink::pageBlockChatLink(string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, string const &username_)
  : title_(title_)
  , photo_(std::move(photo_))
  , accent_color_id_(accent_color_id_)
  , username_(username_)
{}

const std::int32_t pageBlockChatLink::ID;

object_ptr<PageBlock> pageBlockChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockChatLink> res = make_object<pageBlockChatLink>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->photo_ = jni::fetch_tl_object<chatPhotoInfo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return std::move(res);
}

void pageBlockChatLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pageBlockChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockChatLink");
    s.store_field("title", title_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("username", username_);
    s.store_class_end();
  }
}

void pageBlockChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockChatLink").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoInfo;").c_str());
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockTable::Class;
jfieldID pageBlockTable::caption_fieldID;
jfieldID pageBlockTable::cells_fieldID;
jfieldID pageBlockTable::is_bordered_fieldID;
jfieldID pageBlockTable::is_striped_fieldID;

pageBlockTable::pageBlockTable()
  : caption_()
  , cells_()
  , is_bordered_()
  , is_striped_()
{}

pageBlockTable::pageBlockTable(object_ptr<RichText> &&caption_, array<array<object_ptr<pageBlockTableCell>>> &&cells_, bool is_bordered_, bool is_striped_)
  : caption_(std::move(caption_))
  , cells_(std::move(cells_))
  , is_bordered_(is_bordered_)
  , is_striped_(is_striped_)
{}

const std::int32_t pageBlockTable::ID;

object_ptr<PageBlock> pageBlockTable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockTable> res = make_object<pageBlockTable>();
  res->caption_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->cells_ = jni::FetchVector<array<pageBlockTableCell>>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->cells_fieldID));
  res->is_bordered_ = (env->GetBooleanField(p, res->is_bordered_fieldID) != 0);
  res->is_striped_ = (env->GetBooleanField(p, res->is_striped_fieldID) != 0);
  return std::move(res);
}

void pageBlockTable::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, cells_); if (arr_tmp_) { env->SetObjectField(s, cells_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_bordered_fieldID, is_bordered_);
  env->SetBooleanField(s, is_striped_fieldID, is_striped_);
}

void pageBlockTable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockTable");
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    { s.store_vector_begin("cells", cells_.size()); for (const auto &_value : cells_) { { s.store_vector_begin("", _value.size()); for (const auto &_value2 : _value) { s.store_object_field("", static_cast<const BaseObject *>(_value2.get())); } s.store_class_end(); } } s.store_class_end(); }
    s.store_field("isBordered", is_bordered_);
    s.store_field("isStriped", is_striped_);
    s.store_class_end();
  }
}

void pageBlockTable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockTable").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    cells_fieldID = jni::get_field_id(env, Class, "cells", (PSLICE() << "[[L" << get_package_name_ref() << "/TdApi$PageBlockTableCell;").c_str());
    is_bordered_fieldID = jni::get_field_id(env, Class, "isBordered", "Z");
    is_striped_fieldID = jni::get_field_id(env, Class, "isStriped", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockDetails::Class;
jfieldID pageBlockDetails::header_fieldID;
jfieldID pageBlockDetails::page_blocks_fieldID;
jfieldID pageBlockDetails::is_open_fieldID;

pageBlockDetails::pageBlockDetails()
  : header_()
  , page_blocks_()
  , is_open_()
{}

pageBlockDetails::pageBlockDetails(object_ptr<RichText> &&header_, array<object_ptr<PageBlock>> &&page_blocks_, bool is_open_)
  : header_(std::move(header_))
  , page_blocks_(std::move(page_blocks_))
  , is_open_(is_open_)
{}

const std::int32_t pageBlockDetails::ID;

object_ptr<PageBlock> pageBlockDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockDetails> res = make_object<pageBlockDetails>();
  res->header_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->header_fieldID));
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  res->is_open_ = (env->GetBooleanField(p, res->is_open_fieldID) != 0);
  return std::move(res);
}

void pageBlockDetails::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (header_ != nullptr) { jobject next; header_->store(env, next); if (next) { env->SetObjectField(s, header_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_open_fieldID, is_open_);
}

void pageBlockDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockDetails");
    s.store_object_field("header", static_cast<const BaseObject *>(header_.get()));
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isOpen", is_open_);
    s.store_class_end();
  }
}

void pageBlockDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockDetails").c_str());
    header_fieldID = jni::get_field_id(env, Class, "header", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    is_open_fieldID = jni::get_field_id(env, Class, "isOpen", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockRelatedArticles::Class;
jfieldID pageBlockRelatedArticles::header_fieldID;
jfieldID pageBlockRelatedArticles::articles_fieldID;

pageBlockRelatedArticles::pageBlockRelatedArticles()
  : header_()
  , articles_()
{}

pageBlockRelatedArticles::pageBlockRelatedArticles(object_ptr<RichText> &&header_, array<object_ptr<pageBlockRelatedArticle>> &&articles_)
  : header_(std::move(header_))
  , articles_(std::move(articles_))
{}

const std::int32_t pageBlockRelatedArticles::ID;

object_ptr<PageBlock> pageBlockRelatedArticles::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockRelatedArticles> res = make_object<pageBlockRelatedArticles>();
  res->header_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->header_fieldID));
  res->articles_ = jni::FetchVector<pageBlockRelatedArticle>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->articles_fieldID));
  return std::move(res);
}

void pageBlockRelatedArticles::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (header_ != nullptr) { jobject next; header_->store(env, next); if (next) { env->SetObjectField(s, header_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, articles_); if (arr_tmp_) { env->SetObjectField(s, articles_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void pageBlockRelatedArticles::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockRelatedArticles");
    s.store_object_field("header", static_cast<const BaseObject *>(header_.get()));
    { s.store_vector_begin("articles", articles_.size()); for (const auto &_value : articles_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void pageBlockRelatedArticles::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockRelatedArticles").c_str());
    header_fieldID = jni::get_field_id(env, Class, "header", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    articles_fieldID = jni::get_field_id(env, Class, "articles", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlockRelatedArticle;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockMap::Class;
jfieldID pageBlockMap::location_fieldID;
jfieldID pageBlockMap::zoom_fieldID;
jfieldID pageBlockMap::width_fieldID;
jfieldID pageBlockMap::height_fieldID;
jfieldID pageBlockMap::caption_fieldID;

pageBlockMap::pageBlockMap()
  : location_()
  , zoom_()
  , width_()
  , height_()
  , caption_()
{}

pageBlockMap::pageBlockMap(object_ptr<location> &&location_, int32 zoom_, int32 width_, int32 height_, object_ptr<pageBlockCaption> &&caption_)
  : location_(std::move(location_))
  , zoom_(zoom_)
  , width_(width_)
  , height_(height_)
  , caption_(std::move(caption_))
{}

const std::int32_t pageBlockMap::ID;

object_ptr<PageBlock> pageBlockMap::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockMap> res = make_object<pageBlockMap>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->zoom_ = env->GetIntField(p, res->zoom_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->caption_ = jni::fetch_tl_object<pageBlockCaption>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return std::move(res);
}

void pageBlockMap::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, zoom_fieldID, zoom_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockMap::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockMap");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("zoom", zoom_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void pageBlockMap::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockMap").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    zoom_fieldID = jni::get_field_id(env, Class, "zoom", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockCaption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockCaption::Class;
jfieldID pageBlockCaption::text_fieldID;
jfieldID pageBlockCaption::credit_fieldID;

pageBlockCaption::pageBlockCaption()
  : text_()
  , credit_()
{}

pageBlockCaption::pageBlockCaption(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_)
  : text_(std::move(text_))
  , credit_(std::move(credit_))
{}

const std::int32_t pageBlockCaption::ID;

object_ptr<pageBlockCaption> pageBlockCaption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockCaption> res = make_object<pageBlockCaption>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->credit_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->credit_fieldID));
  return res;
}

void pageBlockCaption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  if (credit_ != nullptr) { jobject next; credit_->store(env, next); if (next) { env->SetObjectField(s, credit_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockCaption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockCaption");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("credit", static_cast<const BaseObject *>(credit_.get()));
    s.store_class_end();
  }
}

void pageBlockCaption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockCaption").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    credit_fieldID = jni::get_field_id(env, Class, "credit", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PageBlockHorizontalAlignment::Class;

object_ptr<PageBlockHorizontalAlignment> PageBlockHorizontalAlignment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case pageBlockHorizontalAlignmentLeft::ID:
      return pageBlockHorizontalAlignmentLeft::fetch(env, p);
    case pageBlockHorizontalAlignmentCenter::ID:
      return pageBlockHorizontalAlignmentCenter::fetch(env, p);
    case pageBlockHorizontalAlignmentRight::ID:
      return pageBlockHorizontalAlignmentRight::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PageBlockHorizontalAlignment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockHorizontalAlignment").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockHorizontalAlignmentLeft::Class;

pageBlockHorizontalAlignmentLeft::pageBlockHorizontalAlignmentLeft() {
}

const std::int32_t pageBlockHorizontalAlignmentLeft::ID;

object_ptr<PageBlockHorizontalAlignment> pageBlockHorizontalAlignmentLeft::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockHorizontalAlignmentLeft>();
}

void pageBlockHorizontalAlignmentLeft::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockHorizontalAlignmentLeft::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockHorizontalAlignmentLeft");
    s.store_class_end();
  }
}

void pageBlockHorizontalAlignmentLeft::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockHorizontalAlignmentLeft").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockHorizontalAlignmentCenter::Class;

pageBlockHorizontalAlignmentCenter::pageBlockHorizontalAlignmentCenter() {
}

const std::int32_t pageBlockHorizontalAlignmentCenter::ID;

object_ptr<PageBlockHorizontalAlignment> pageBlockHorizontalAlignmentCenter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockHorizontalAlignmentCenter>();
}

void pageBlockHorizontalAlignmentCenter::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockHorizontalAlignmentCenter::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockHorizontalAlignmentCenter");
    s.store_class_end();
  }
}

void pageBlockHorizontalAlignmentCenter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockHorizontalAlignmentCenter").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockHorizontalAlignmentRight::Class;

pageBlockHorizontalAlignmentRight::pageBlockHorizontalAlignmentRight() {
}

const std::int32_t pageBlockHorizontalAlignmentRight::ID;

object_ptr<PageBlockHorizontalAlignment> pageBlockHorizontalAlignmentRight::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockHorizontalAlignmentRight>();
}

void pageBlockHorizontalAlignmentRight::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockHorizontalAlignmentRight::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockHorizontalAlignmentRight");
    s.store_class_end();
  }
}

void pageBlockHorizontalAlignmentRight::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockHorizontalAlignmentRight").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockListItem::Class;
jfieldID pageBlockListItem::label_fieldID;
jfieldID pageBlockListItem::page_blocks_fieldID;

pageBlockListItem::pageBlockListItem()
  : label_()
  , page_blocks_()
{}

pageBlockListItem::pageBlockListItem(string const &label_, array<object_ptr<PageBlock>> &&page_blocks_)
  : label_(label_)
  , page_blocks_(std::move(page_blocks_))
{}

const std::int32_t pageBlockListItem::ID;

object_ptr<pageBlockListItem> pageBlockListItem::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockListItem> res = make_object<pageBlockListItem>();
  res->label_ = jni::fetch_string(env, p, res->label_fieldID);
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  return res;
}

void pageBlockListItem::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, label_); if (nextString) { env->SetObjectField(s, label_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void pageBlockListItem::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockListItem");
    s.store_field("label", label_);
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void pageBlockListItem::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockListItem").c_str());
    label_fieldID = jni::get_field_id(env, Class, "label", "Ljava/lang/String;");
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockRelatedArticle::Class;
jfieldID pageBlockRelatedArticle::url_fieldID;
jfieldID pageBlockRelatedArticle::title_fieldID;
jfieldID pageBlockRelatedArticle::description_fieldID;
jfieldID pageBlockRelatedArticle::photo_fieldID;
jfieldID pageBlockRelatedArticle::author_fieldID;
jfieldID pageBlockRelatedArticle::publish_date_fieldID;

pageBlockRelatedArticle::pageBlockRelatedArticle()
  : url_()
  , title_()
  , description_()
  , photo_()
  , author_()
  , publish_date_()
{}

pageBlockRelatedArticle::pageBlockRelatedArticle(string const &url_, string const &title_, string const &description_, object_ptr<photo> &&photo_, string const &author_, int32 publish_date_)
  : url_(url_)
  , title_(title_)
  , description_(description_)
  , photo_(std::move(photo_))
  , author_(author_)
  , publish_date_(publish_date_)
{}

const std::int32_t pageBlockRelatedArticle::ID;

object_ptr<pageBlockRelatedArticle> pageBlockRelatedArticle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockRelatedArticle> res = make_object<pageBlockRelatedArticle>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->author_ = jni::fetch_string(env, p, res->author_fieldID);
  res->publish_date_ = env->GetIntField(p, res->publish_date_fieldID);
  return res;
}

void pageBlockRelatedArticle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, author_); if (nextString) { env->SetObjectField(s, author_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, publish_date_fieldID, publish_date_);
}

void pageBlockRelatedArticle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockRelatedArticle");
    s.store_field("url", url_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("author", author_);
    s.store_field("publishDate", publish_date_);
    s.store_class_end();
  }
}

void pageBlockRelatedArticle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockRelatedArticle").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    author_fieldID = jni::get_field_id(env, Class, "author", "Ljava/lang/String;");
    publish_date_fieldID = jni::get_field_id(env, Class, "publishDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockTableCell::Class;
jfieldID pageBlockTableCell::text_fieldID;
jfieldID pageBlockTableCell::is_header_fieldID;
jfieldID pageBlockTableCell::colspan_fieldID;
jfieldID pageBlockTableCell::rowspan_fieldID;
jfieldID pageBlockTableCell::align_fieldID;
jfieldID pageBlockTableCell::valign_fieldID;

pageBlockTableCell::pageBlockTableCell()
  : text_()
  , is_header_()
  , colspan_()
  , rowspan_()
  , align_()
  , valign_()
{}

pageBlockTableCell::pageBlockTableCell(object_ptr<RichText> &&text_, bool is_header_, int32 colspan_, int32 rowspan_, object_ptr<PageBlockHorizontalAlignment> &&align_, object_ptr<PageBlockVerticalAlignment> &&valign_)
  : text_(std::move(text_))
  , is_header_(is_header_)
  , colspan_(colspan_)
  , rowspan_(rowspan_)
  , align_(std::move(align_))
  , valign_(std::move(valign_))
{}

const std::int32_t pageBlockTableCell::ID;

object_ptr<pageBlockTableCell> pageBlockTableCell::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pageBlockTableCell> res = make_object<pageBlockTableCell>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->is_header_ = (env->GetBooleanField(p, res->is_header_fieldID) != 0);
  res->colspan_ = env->GetIntField(p, res->colspan_fieldID);
  res->rowspan_ = env->GetIntField(p, res->rowspan_fieldID);
  res->align_ = jni::fetch_tl_object<PageBlockHorizontalAlignment>(env, jni::fetch_object(env, p, res->align_fieldID));
  res->valign_ = jni::fetch_tl_object<PageBlockVerticalAlignment>(env, jni::fetch_object(env, p, res->valign_fieldID));
  return res;
}

void pageBlockTableCell::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_header_fieldID, is_header_);
  env->SetIntField(s, colspan_fieldID, colspan_);
  env->SetIntField(s, rowspan_fieldID, rowspan_);
  if (align_ != nullptr) { jobject next; align_->store(env, next); if (next) { env->SetObjectField(s, align_fieldID, next); env->DeleteLocalRef(next); } }
  if (valign_ != nullptr) { jobject next; valign_->store(env, next); if (next) { env->SetObjectField(s, valign_fieldID, next); env->DeleteLocalRef(next); } }
}

void pageBlockTableCell::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockTableCell");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("isHeader", is_header_);
    s.store_field("colspan", colspan_);
    s.store_field("rowspan", rowspan_);
    s.store_object_field("align", static_cast<const BaseObject *>(align_.get()));
    s.store_object_field("valign", static_cast<const BaseObject *>(valign_.get()));
    s.store_class_end();
  }
}

void pageBlockTableCell::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockTableCell").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    is_header_fieldID = jni::get_field_id(env, Class, "isHeader", "Z");
    colspan_fieldID = jni::get_field_id(env, Class, "colspan", "I");
    rowspan_fieldID = jni::get_field_id(env, Class, "rowspan", "I");
    align_fieldID = jni::get_field_id(env, Class, "align", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockHorizontalAlignment;").c_str());
    valign_fieldID = jni::get_field_id(env, Class, "valign", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PageBlockVerticalAlignment;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PageBlockVerticalAlignment::Class;

object_ptr<PageBlockVerticalAlignment> PageBlockVerticalAlignment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case pageBlockVerticalAlignmentTop::ID:
      return pageBlockVerticalAlignmentTop::fetch(env, p);
    case pageBlockVerticalAlignmentMiddle::ID:
      return pageBlockVerticalAlignmentMiddle::fetch(env, p);
    case pageBlockVerticalAlignmentBottom::ID:
      return pageBlockVerticalAlignmentBottom::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PageBlockVerticalAlignment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVerticalAlignment").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockVerticalAlignmentTop::Class;

pageBlockVerticalAlignmentTop::pageBlockVerticalAlignmentTop() {
}

const std::int32_t pageBlockVerticalAlignmentTop::ID;

object_ptr<PageBlockVerticalAlignment> pageBlockVerticalAlignmentTop::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockVerticalAlignmentTop>();
}

void pageBlockVerticalAlignmentTop::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockVerticalAlignmentTop::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockVerticalAlignmentTop");
    s.store_class_end();
  }
}

void pageBlockVerticalAlignmentTop::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVerticalAlignmentTop").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockVerticalAlignmentMiddle::Class;

pageBlockVerticalAlignmentMiddle::pageBlockVerticalAlignmentMiddle() {
}

const std::int32_t pageBlockVerticalAlignmentMiddle::ID;

object_ptr<PageBlockVerticalAlignment> pageBlockVerticalAlignmentMiddle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockVerticalAlignmentMiddle>();
}

void pageBlockVerticalAlignmentMiddle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockVerticalAlignmentMiddle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockVerticalAlignmentMiddle");
    s.store_class_end();
  }
}

void pageBlockVerticalAlignmentMiddle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVerticalAlignmentMiddle").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pageBlockVerticalAlignmentBottom::Class;

pageBlockVerticalAlignmentBottom::pageBlockVerticalAlignmentBottom() {
}

const std::int32_t pageBlockVerticalAlignmentBottom::ID;

object_ptr<PageBlockVerticalAlignment> pageBlockVerticalAlignmentBottom::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pageBlockVerticalAlignmentBottom>();
}

void pageBlockVerticalAlignmentBottom::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pageBlockVerticalAlignmentBottom::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PageBlockVerticalAlignmentBottom");
    s.store_class_end();
  }
}

void pageBlockVerticalAlignmentBottom::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PageBlockVerticalAlignmentBottom").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PaidMedia::Class;

object_ptr<PaidMedia> PaidMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case paidMediaPreview::ID:
      return paidMediaPreview::fetch(env, p);
    case paidMediaPhoto::ID:
      return paidMediaPhoto::fetch(env, p);
    case paidMediaVideo::ID:
      return paidMediaVideo::fetch(env, p);
    case paidMediaUnsupported::ID:
      return paidMediaUnsupported::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PaidMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidMedia").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidMediaPreview::Class;
jfieldID paidMediaPreview::width_fieldID;
jfieldID paidMediaPreview::height_fieldID;
jfieldID paidMediaPreview::duration_fieldID;
jfieldID paidMediaPreview::minithumbnail_fieldID;

paidMediaPreview::paidMediaPreview()
  : width_()
  , height_()
  , duration_()
  , minithumbnail_()
{}

paidMediaPreview::paidMediaPreview(int32 width_, int32 height_, int32 duration_, object_ptr<minithumbnail> &&minithumbnail_)
  : width_(width_)
  , height_(height_)
  , duration_(duration_)
  , minithumbnail_(std::move(minithumbnail_))
{}

const std::int32_t paidMediaPreview::ID;

object_ptr<PaidMedia> paidMediaPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paidMediaPreview> res = make_object<paidMediaPreview>();
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  return std::move(res);
}

void paidMediaPreview::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  env->SetIntField(s, duration_fieldID, duration_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
}

void paidMediaPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidMediaPreview");
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("duration", duration_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_class_end();
  }
}

void paidMediaPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidMediaPreview").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidMediaPhoto::Class;
jfieldID paidMediaPhoto::photo_fieldID;

paidMediaPhoto::paidMediaPhoto()
  : photo_()
{}

paidMediaPhoto::paidMediaPhoto(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t paidMediaPhoto::ID;

object_ptr<PaidMedia> paidMediaPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paidMediaPhoto> res = make_object<paidMediaPhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void paidMediaPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void paidMediaPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidMediaPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void paidMediaPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidMediaPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidMediaVideo::Class;
jfieldID paidMediaVideo::video_fieldID;
jfieldID paidMediaVideo::cover_fieldID;
jfieldID paidMediaVideo::start_timestamp_fieldID;

paidMediaVideo::paidMediaVideo()
  : video_()
  , cover_()
  , start_timestamp_()
{}

paidMediaVideo::paidMediaVideo(object_ptr<video> &&video_, object_ptr<photo> &&cover_, int32 start_timestamp_)
  : video_(std::move(video_))
  , cover_(std::move(cover_))
  , start_timestamp_(start_timestamp_)
{}

const std::int32_t paidMediaVideo::ID;

object_ptr<PaidMedia> paidMediaVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paidMediaVideo> res = make_object<paidMediaVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->cover_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->cover_fieldID));
  res->start_timestamp_ = env->GetIntField(p, res->start_timestamp_fieldID);
  return std::move(res);
}

void paidMediaVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  if (cover_ != nullptr) { jobject next; cover_->store(env, next); if (next) { env->SetObjectField(s, cover_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, start_timestamp_fieldID, start_timestamp_);
}

void paidMediaVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidMediaVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_object_field("cover", static_cast<const BaseObject *>(cover_.get()));
    s.store_field("startTimestamp", start_timestamp_);
    s.store_class_end();
  }
}

void paidMediaVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidMediaVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    cover_fieldID = jni::get_field_id(env, Class, "cover", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    start_timestamp_fieldID = jni::get_field_id(env, Class, "startTimestamp", "I");
    return true;
  }();
  (void)is_inited;
}

jclass paidMediaUnsupported::Class;

paidMediaUnsupported::paidMediaUnsupported() {
}

const std::int32_t paidMediaUnsupported::ID;

object_ptr<PaidMedia> paidMediaUnsupported::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<paidMediaUnsupported>();
}

void paidMediaUnsupported::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void paidMediaUnsupported::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidMediaUnsupported");
    s.store_class_end();
  }
}

void paidMediaUnsupported::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidMediaUnsupported").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PaidReactionType::Class;

object_ptr<PaidReactionType> PaidReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case paidReactionTypeRegular::ID:
      return paidReactionTypeRegular::fetch(env, p);
    case paidReactionTypeAnonymous::ID:
      return paidReactionTypeAnonymous::fetch(env, p);
    case paidReactionTypeChat::ID:
      return paidReactionTypeChat::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PaidReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidReactionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidReactionTypeRegular::Class;

paidReactionTypeRegular::paidReactionTypeRegular() {
}

const std::int32_t paidReactionTypeRegular::ID;

object_ptr<PaidReactionType> paidReactionTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<paidReactionTypeRegular>();
}

void paidReactionTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void paidReactionTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidReactionTypeRegular");
    s.store_class_end();
  }
}

void paidReactionTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidReactionTypeRegular").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidReactionTypeAnonymous::Class;

paidReactionTypeAnonymous::paidReactionTypeAnonymous() {
}

const std::int32_t paidReactionTypeAnonymous::ID;

object_ptr<PaidReactionType> paidReactionTypeAnonymous::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<paidReactionTypeAnonymous>();
}

void paidReactionTypeAnonymous::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void paidReactionTypeAnonymous::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidReactionTypeAnonymous");
    s.store_class_end();
  }
}

void paidReactionTypeAnonymous::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidReactionTypeAnonymous").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paidReactionTypeChat::Class;
jfieldID paidReactionTypeChat::chat_id_fieldID;

paidReactionTypeChat::paidReactionTypeChat()
  : chat_id_()
{}

paidReactionTypeChat::paidReactionTypeChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t paidReactionTypeChat::ID;

object_ptr<PaidReactionType> paidReactionTypeChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paidReactionTypeChat> res = make_object<paidReactionTypeChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void paidReactionTypeChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void paidReactionTypeChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidReactionTypeChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void paidReactionTypeChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidReactionTypeChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass paidReactor::Class;
jfieldID paidReactor::sender_id_fieldID;
jfieldID paidReactor::star_count_fieldID;
jfieldID paidReactor::is_top_fieldID;
jfieldID paidReactor::is_me_fieldID;
jfieldID paidReactor::is_anonymous_fieldID;

paidReactor::paidReactor()
  : sender_id_()
  , star_count_()
  , is_top_()
  , is_me_()
  , is_anonymous_()
{}

paidReactor::paidReactor(object_ptr<MessageSender> &&sender_id_, int32 star_count_, bool is_top_, bool is_me_, bool is_anonymous_)
  : sender_id_(std::move(sender_id_))
  , star_count_(star_count_)
  , is_top_(is_top_)
  , is_me_(is_me_)
  , is_anonymous_(is_anonymous_)
{}

const std::int32_t paidReactor::ID;

object_ptr<paidReactor> paidReactor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paidReactor> res = make_object<paidReactor>();
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->star_count_ = env->GetIntField(p, res->star_count_fieldID);
  res->is_top_ = (env->GetBooleanField(p, res->is_top_fieldID) != 0);
  res->is_me_ = (env->GetBooleanField(p, res->is_me_fieldID) != 0);
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  return res;
}

void paidReactor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, star_count_fieldID, star_count_);
  env->SetBooleanField(s, is_top_fieldID, is_top_);
  env->SetBooleanField(s, is_me_fieldID, is_me_);
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
}

void paidReactor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaidReactor");
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("starCount", star_count_);
    s.store_field("isTop", is_top_);
    s.store_field("isMe", is_me_);
    s.store_field("isAnonymous", is_anonymous_);
    s.store_class_end();
  }
}

void paidReactor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaidReactor").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "I");
    is_top_fieldID = jni::get_field_id(env, Class, "isTop", "Z");
    is_me_fieldID = jni::get_field_id(env, Class, "isMe", "Z");
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass passportAuthorizationForm::Class;
jfieldID passportAuthorizationForm::id_fieldID;
jfieldID passportAuthorizationForm::required_elements_fieldID;
jfieldID passportAuthorizationForm::privacy_policy_url_fieldID;

passportAuthorizationForm::passportAuthorizationForm()
  : id_()
  , required_elements_()
  , privacy_policy_url_()
{}

passportAuthorizationForm::passportAuthorizationForm(int32 id_, array<object_ptr<passportRequiredElement>> &&required_elements_, string const &privacy_policy_url_)
  : id_(id_)
  , required_elements_(std::move(required_elements_))
  , privacy_policy_url_(privacy_policy_url_)
{}

const std::int32_t passportAuthorizationForm::ID;

object_ptr<passportAuthorizationForm> passportAuthorizationForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportAuthorizationForm> res = make_object<passportAuthorizationForm>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->required_elements_ = jni::FetchVector<passportRequiredElement>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->required_elements_fieldID));
  res->privacy_policy_url_ = jni::fetch_string(env, p, res->privacy_policy_url_fieldID);
  return res;
}

void passportAuthorizationForm::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  { auto arr_tmp_ = jni::store_vector(env, required_elements_); if (arr_tmp_) { env->SetObjectField(s, required_elements_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, privacy_policy_url_); if (nextString) { env->SetObjectField(s, privacy_policy_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void passportAuthorizationForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportAuthorizationForm");
    s.store_field("id", id_);
    { s.store_vector_begin("requiredElements", required_elements_.size()); for (const auto &_value : required_elements_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("privacyPolicyUrl", privacy_policy_url_);
    s.store_class_end();
  }
}

void passportAuthorizationForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportAuthorizationForm").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    required_elements_fieldID = jni::get_field_id(env, Class, "requiredElements", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportRequiredElement;").c_str());
    privacy_policy_url_fieldID = jni::get_field_id(env, Class, "privacyPolicyUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PassportElement::Class;

object_ptr<PassportElement> PassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case passportElementPersonalDetails::ID:
      return passportElementPersonalDetails::fetch(env, p);
    case passportElementPassport::ID:
      return passportElementPassport::fetch(env, p);
    case passportElementDriverLicense::ID:
      return passportElementDriverLicense::fetch(env, p);
    case passportElementIdentityCard::ID:
      return passportElementIdentityCard::fetch(env, p);
    case passportElementInternalPassport::ID:
      return passportElementInternalPassport::fetch(env, p);
    case passportElementAddress::ID:
      return passportElementAddress::fetch(env, p);
    case passportElementUtilityBill::ID:
      return passportElementUtilityBill::fetch(env, p);
    case passportElementBankStatement::ID:
      return passportElementBankStatement::fetch(env, p);
    case passportElementRentalAgreement::ID:
      return passportElementRentalAgreement::fetch(env, p);
    case passportElementPassportRegistration::ID:
      return passportElementPassportRegistration::fetch(env, p);
    case passportElementTemporaryRegistration::ID:
      return passportElementTemporaryRegistration::fetch(env, p);
    case passportElementPhoneNumber::ID:
      return passportElementPhoneNumber::fetch(env, p);
    case passportElementEmailAddress::ID:
      return passportElementEmailAddress::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElement").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementPersonalDetails::Class;
jfieldID passportElementPersonalDetails::personal_details_fieldID;

passportElementPersonalDetails::passportElementPersonalDetails()
  : personal_details_()
{}

passportElementPersonalDetails::passportElementPersonalDetails(object_ptr<personalDetails> &&personal_details_)
  : personal_details_(std::move(personal_details_))
{}

const std::int32_t passportElementPersonalDetails::ID;

object_ptr<PassportElement> passportElementPersonalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementPersonalDetails> res = make_object<passportElementPersonalDetails>();
  res->personal_details_ = jni::fetch_tl_object<personalDetails>(env, jni::fetch_object(env, p, res->personal_details_fieldID));
  return std::move(res);
}

void passportElementPersonalDetails::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (personal_details_ != nullptr) { jobject next; personal_details_->store(env, next); if (next) { env->SetObjectField(s, personal_details_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementPersonalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementPersonalDetails");
    s.store_object_field("personalDetails", static_cast<const BaseObject *>(personal_details_.get()));
    s.store_class_end();
  }
}

void passportElementPersonalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementPersonalDetails").c_str());
    personal_details_fieldID = jni::get_field_id(env, Class, "personalDetails", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDetails;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementPassport::Class;
jfieldID passportElementPassport::passport_fieldID;

passportElementPassport::passportElementPassport()
  : passport_()
{}

passportElementPassport::passportElementPassport(object_ptr<identityDocument> &&passport_)
  : passport_(std::move(passport_))
{}

const std::int32_t passportElementPassport::ID;

object_ptr<PassportElement> passportElementPassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementPassport> res = make_object<passportElementPassport>();
  res->passport_ = jni::fetch_tl_object<identityDocument>(env, jni::fetch_object(env, p, res->passport_fieldID));
  return std::move(res);
}

void passportElementPassport::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (passport_ != nullptr) { jobject next; passport_->store(env, next); if (next) { env->SetObjectField(s, passport_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementPassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementPassport");
    s.store_object_field("passport", static_cast<const BaseObject *>(passport_.get()));
    s.store_class_end();
  }
}

void passportElementPassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementPassport").c_str());
    passport_fieldID = jni::get_field_id(env, Class, "passport", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$IdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementDriverLicense::Class;
jfieldID passportElementDriverLicense::driver_license_fieldID;

passportElementDriverLicense::passportElementDriverLicense()
  : driver_license_()
{}

passportElementDriverLicense::passportElementDriverLicense(object_ptr<identityDocument> &&driver_license_)
  : driver_license_(std::move(driver_license_))
{}

const std::int32_t passportElementDriverLicense::ID;

object_ptr<PassportElement> passportElementDriverLicense::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementDriverLicense> res = make_object<passportElementDriverLicense>();
  res->driver_license_ = jni::fetch_tl_object<identityDocument>(env, jni::fetch_object(env, p, res->driver_license_fieldID));
  return std::move(res);
}

void passportElementDriverLicense::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (driver_license_ != nullptr) { jobject next; driver_license_->store(env, next); if (next) { env->SetObjectField(s, driver_license_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementDriverLicense::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementDriverLicense");
    s.store_object_field("driverLicense", static_cast<const BaseObject *>(driver_license_.get()));
    s.store_class_end();
  }
}

void passportElementDriverLicense::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementDriverLicense").c_str());
    driver_license_fieldID = jni::get_field_id(env, Class, "driverLicense", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$IdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementIdentityCard::Class;
jfieldID passportElementIdentityCard::identity_card_fieldID;

passportElementIdentityCard::passportElementIdentityCard()
  : identity_card_()
{}

passportElementIdentityCard::passportElementIdentityCard(object_ptr<identityDocument> &&identity_card_)
  : identity_card_(std::move(identity_card_))
{}

const std::int32_t passportElementIdentityCard::ID;

object_ptr<PassportElement> passportElementIdentityCard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementIdentityCard> res = make_object<passportElementIdentityCard>();
  res->identity_card_ = jni::fetch_tl_object<identityDocument>(env, jni::fetch_object(env, p, res->identity_card_fieldID));
  return std::move(res);
}

void passportElementIdentityCard::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (identity_card_ != nullptr) { jobject next; identity_card_->store(env, next); if (next) { env->SetObjectField(s, identity_card_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementIdentityCard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementIdentityCard");
    s.store_object_field("identityCard", static_cast<const BaseObject *>(identity_card_.get()));
    s.store_class_end();
  }
}

void passportElementIdentityCard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementIdentityCard").c_str());
    identity_card_fieldID = jni::get_field_id(env, Class, "identityCard", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$IdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementInternalPassport::Class;
jfieldID passportElementInternalPassport::internal_passport_fieldID;

passportElementInternalPassport::passportElementInternalPassport()
  : internal_passport_()
{}

passportElementInternalPassport::passportElementInternalPassport(object_ptr<identityDocument> &&internal_passport_)
  : internal_passport_(std::move(internal_passport_))
{}

const std::int32_t passportElementInternalPassport::ID;

object_ptr<PassportElement> passportElementInternalPassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementInternalPassport> res = make_object<passportElementInternalPassport>();
  res->internal_passport_ = jni::fetch_tl_object<identityDocument>(env, jni::fetch_object(env, p, res->internal_passport_fieldID));
  return std::move(res);
}

void passportElementInternalPassport::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (internal_passport_ != nullptr) { jobject next; internal_passport_->store(env, next); if (next) { env->SetObjectField(s, internal_passport_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementInternalPassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementInternalPassport");
    s.store_object_field("internalPassport", static_cast<const BaseObject *>(internal_passport_.get()));
    s.store_class_end();
  }
}

void passportElementInternalPassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementInternalPassport").c_str());
    internal_passport_fieldID = jni::get_field_id(env, Class, "internalPassport", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$IdentityDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementAddress::Class;
jfieldID passportElementAddress::address_fieldID;

passportElementAddress::passportElementAddress()
  : address_()
{}

passportElementAddress::passportElementAddress(object_ptr<address> &&address_)
  : address_(std::move(address_))
{}

const std::int32_t passportElementAddress::ID;

object_ptr<PassportElement> passportElementAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementAddress> res = make_object<passportElementAddress>();
  res->address_ = jni::fetch_tl_object<address>(env, jni::fetch_object(env, p, res->address_fieldID));
  return std::move(res);
}

void passportElementAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (address_ != nullptr) { jobject next; address_->store(env, next); if (next) { env->SetObjectField(s, address_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementAddress");
    s.store_object_field("address", static_cast<const BaseObject *>(address_.get()));
    s.store_class_end();
  }
}

void passportElementAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementAddress").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Address;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementUtilityBill::Class;
jfieldID passportElementUtilityBill::utility_bill_fieldID;

passportElementUtilityBill::passportElementUtilityBill()
  : utility_bill_()
{}

passportElementUtilityBill::passportElementUtilityBill(object_ptr<personalDocument> &&utility_bill_)
  : utility_bill_(std::move(utility_bill_))
{}

const std::int32_t passportElementUtilityBill::ID;

object_ptr<PassportElement> passportElementUtilityBill::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementUtilityBill> res = make_object<passportElementUtilityBill>();
  res->utility_bill_ = jni::fetch_tl_object<personalDocument>(env, jni::fetch_object(env, p, res->utility_bill_fieldID));
  return std::move(res);
}

void passportElementUtilityBill::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (utility_bill_ != nullptr) { jobject next; utility_bill_->store(env, next); if (next) { env->SetObjectField(s, utility_bill_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementUtilityBill::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementUtilityBill");
    s.store_object_field("utilityBill", static_cast<const BaseObject *>(utility_bill_.get()));
    s.store_class_end();
  }
}

void passportElementUtilityBill::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementUtilityBill").c_str());
    utility_bill_fieldID = jni::get_field_id(env, Class, "utilityBill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementBankStatement::Class;
jfieldID passportElementBankStatement::bank_statement_fieldID;

passportElementBankStatement::passportElementBankStatement()
  : bank_statement_()
{}

passportElementBankStatement::passportElementBankStatement(object_ptr<personalDocument> &&bank_statement_)
  : bank_statement_(std::move(bank_statement_))
{}

const std::int32_t passportElementBankStatement::ID;

object_ptr<PassportElement> passportElementBankStatement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementBankStatement> res = make_object<passportElementBankStatement>();
  res->bank_statement_ = jni::fetch_tl_object<personalDocument>(env, jni::fetch_object(env, p, res->bank_statement_fieldID));
  return std::move(res);
}

void passportElementBankStatement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (bank_statement_ != nullptr) { jobject next; bank_statement_->store(env, next); if (next) { env->SetObjectField(s, bank_statement_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementBankStatement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementBankStatement");
    s.store_object_field("bankStatement", static_cast<const BaseObject *>(bank_statement_.get()));
    s.store_class_end();
  }
}

void passportElementBankStatement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementBankStatement").c_str());
    bank_statement_fieldID = jni::get_field_id(env, Class, "bankStatement", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementRentalAgreement::Class;
jfieldID passportElementRentalAgreement::rental_agreement_fieldID;

passportElementRentalAgreement::passportElementRentalAgreement()
  : rental_agreement_()
{}

passportElementRentalAgreement::passportElementRentalAgreement(object_ptr<personalDocument> &&rental_agreement_)
  : rental_agreement_(std::move(rental_agreement_))
{}

const std::int32_t passportElementRentalAgreement::ID;

object_ptr<PassportElement> passportElementRentalAgreement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementRentalAgreement> res = make_object<passportElementRentalAgreement>();
  res->rental_agreement_ = jni::fetch_tl_object<personalDocument>(env, jni::fetch_object(env, p, res->rental_agreement_fieldID));
  return std::move(res);
}

void passportElementRentalAgreement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (rental_agreement_ != nullptr) { jobject next; rental_agreement_->store(env, next); if (next) { env->SetObjectField(s, rental_agreement_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementRentalAgreement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementRentalAgreement");
    s.store_object_field("rentalAgreement", static_cast<const BaseObject *>(rental_agreement_.get()));
    s.store_class_end();
  }
}

void passportElementRentalAgreement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementRentalAgreement").c_str());
    rental_agreement_fieldID = jni::get_field_id(env, Class, "rentalAgreement", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementPassportRegistration::Class;
jfieldID passportElementPassportRegistration::passport_registration_fieldID;

passportElementPassportRegistration::passportElementPassportRegistration()
  : passport_registration_()
{}

passportElementPassportRegistration::passportElementPassportRegistration(object_ptr<personalDocument> &&passport_registration_)
  : passport_registration_(std::move(passport_registration_))
{}

const std::int32_t passportElementPassportRegistration::ID;

object_ptr<PassportElement> passportElementPassportRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementPassportRegistration> res = make_object<passportElementPassportRegistration>();
  res->passport_registration_ = jni::fetch_tl_object<personalDocument>(env, jni::fetch_object(env, p, res->passport_registration_fieldID));
  return std::move(res);
}

void passportElementPassportRegistration::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (passport_registration_ != nullptr) { jobject next; passport_registration_->store(env, next); if (next) { env->SetObjectField(s, passport_registration_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementPassportRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementPassportRegistration");
    s.store_object_field("passportRegistration", static_cast<const BaseObject *>(passport_registration_.get()));
    s.store_class_end();
  }
}

void passportElementPassportRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementPassportRegistration").c_str());
    passport_registration_fieldID = jni::get_field_id(env, Class, "passportRegistration", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTemporaryRegistration::Class;
jfieldID passportElementTemporaryRegistration::temporary_registration_fieldID;

passportElementTemporaryRegistration::passportElementTemporaryRegistration()
  : temporary_registration_()
{}

passportElementTemporaryRegistration::passportElementTemporaryRegistration(object_ptr<personalDocument> &&temporary_registration_)
  : temporary_registration_(std::move(temporary_registration_))
{}

const std::int32_t passportElementTemporaryRegistration::ID;

object_ptr<PassportElement> passportElementTemporaryRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementTemporaryRegistration> res = make_object<passportElementTemporaryRegistration>();
  res->temporary_registration_ = jni::fetch_tl_object<personalDocument>(env, jni::fetch_object(env, p, res->temporary_registration_fieldID));
  return std::move(res);
}

void passportElementTemporaryRegistration::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (temporary_registration_ != nullptr) { jobject next; temporary_registration_->store(env, next); if (next) { env->SetObjectField(s, temporary_registration_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementTemporaryRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTemporaryRegistration");
    s.store_object_field("temporaryRegistration", static_cast<const BaseObject *>(temporary_registration_.get()));
    s.store_class_end();
  }
}

void passportElementTemporaryRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTemporaryRegistration").c_str());
    temporary_registration_fieldID = jni::get_field_id(env, Class, "temporaryRegistration", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PersonalDocument;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementPhoneNumber::Class;
jfieldID passportElementPhoneNumber::phone_number_fieldID;

passportElementPhoneNumber::passportElementPhoneNumber()
  : phone_number_()
{}

passportElementPhoneNumber::passportElementPhoneNumber(string const &phone_number_)
  : phone_number_(phone_number_)
{}

const std::int32_t passportElementPhoneNumber::ID;

object_ptr<PassportElement> passportElementPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementPhoneNumber> res = make_object<passportElementPhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  return std::move(res);
}

void passportElementPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void passportElementPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementPhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_class_end();
  }
}

void passportElementPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementPhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass passportElementEmailAddress::Class;
jfieldID passportElementEmailAddress::email_address_fieldID;

passportElementEmailAddress::passportElementEmailAddress()
  : email_address_()
{}

passportElementEmailAddress::passportElementEmailAddress(string const &email_address_)
  : email_address_(email_address_)
{}

const std::int32_t passportElementEmailAddress::ID;

object_ptr<PassportElement> passportElementEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementEmailAddress> res = make_object<passportElementEmailAddress>();
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  return std::move(res);
}

void passportElementEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, email_address_); if (nextString) { env->SetObjectField(s, email_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void passportElementEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementEmailAddress");
    s.store_field("emailAddress", email_address_);
    s.store_class_end();
  }
}

void passportElementEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementEmailAddress").c_str());
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass passportElementError::Class;
jfieldID passportElementError::type_fieldID;
jfieldID passportElementError::message_fieldID;
jfieldID passportElementError::source_fieldID;

passportElementError::passportElementError()
  : type_()
  , message_()
  , source_()
{}

passportElementError::passportElementError(object_ptr<PassportElementType> &&type_, string const &message_, object_ptr<PassportElementErrorSource> &&source_)
  : type_(std::move(type_))
  , message_(message_)
  , source_(std::move(source_))
{}

const std::int32_t passportElementError::ID;

object_ptr<passportElementError> passportElementError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementError> res = make_object<passportElementError>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->message_ = jni::fetch_string(env, p, res->message_fieldID);
  res->source_ = jni::fetch_tl_object<PassportElementErrorSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  return res;
}

void passportElementError::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, message_); if (nextString) { env->SetObjectField(s, message_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (source_ != nullptr) { jobject next; source_->store(env, next); if (next) { env->SetObjectField(s, source_fieldID, next); env->DeleteLocalRef(next); } }
}

void passportElementError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementError");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("message", message_);
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_class_end();
  }
}

void passportElementError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementError").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", "Ljava/lang/String;");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementErrorSource;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PassportElementErrorSource::Class;

object_ptr<PassportElementErrorSource> PassportElementErrorSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case passportElementErrorSourceUnspecified::ID:
      return passportElementErrorSourceUnspecified::fetch(env, p);
    case passportElementErrorSourceDataField::ID:
      return passportElementErrorSourceDataField::fetch(env, p);
    case passportElementErrorSourceFrontSide::ID:
      return passportElementErrorSourceFrontSide::fetch(env, p);
    case passportElementErrorSourceReverseSide::ID:
      return passportElementErrorSourceReverseSide::fetch(env, p);
    case passportElementErrorSourceSelfie::ID:
      return passportElementErrorSourceSelfie::fetch(env, p);
    case passportElementErrorSourceTranslationFile::ID:
      return passportElementErrorSourceTranslationFile::fetch(env, p);
    case passportElementErrorSourceTranslationFiles::ID:
      return passportElementErrorSourceTranslationFiles::fetch(env, p);
    case passportElementErrorSourceFile::ID:
      return passportElementErrorSourceFile::fetch(env, p);
    case passportElementErrorSourceFiles::ID:
      return passportElementErrorSourceFiles::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PassportElementErrorSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceUnspecified::Class;

passportElementErrorSourceUnspecified::passportElementErrorSourceUnspecified() {
}

const std::int32_t passportElementErrorSourceUnspecified::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceUnspecified::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceUnspecified>();
}

void passportElementErrorSourceUnspecified::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceUnspecified::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceUnspecified");
    s.store_class_end();
  }
}

void passportElementErrorSourceUnspecified::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceUnspecified").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceDataField::Class;
jfieldID passportElementErrorSourceDataField::field_name_fieldID;

passportElementErrorSourceDataField::passportElementErrorSourceDataField()
  : field_name_()
{}

passportElementErrorSourceDataField::passportElementErrorSourceDataField(string const &field_name_)
  : field_name_(field_name_)
{}

const std::int32_t passportElementErrorSourceDataField::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceDataField::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementErrorSourceDataField> res = make_object<passportElementErrorSourceDataField>();
  res->field_name_ = jni::fetch_string(env, p, res->field_name_fieldID);
  return std::move(res);
}

void passportElementErrorSourceDataField::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, field_name_); if (nextString) { env->SetObjectField(s, field_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void passportElementErrorSourceDataField::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceDataField");
    s.store_field("fieldName", field_name_);
    s.store_class_end();
  }
}

void passportElementErrorSourceDataField::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceDataField").c_str());
    field_name_fieldID = jni::get_field_id(env, Class, "fieldName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceFrontSide::Class;

passportElementErrorSourceFrontSide::passportElementErrorSourceFrontSide() {
}

const std::int32_t passportElementErrorSourceFrontSide::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceFrontSide::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceFrontSide>();
}

void passportElementErrorSourceFrontSide::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceFrontSide::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceFrontSide");
    s.store_class_end();
  }
}

void passportElementErrorSourceFrontSide::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceFrontSide").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceReverseSide::Class;

passportElementErrorSourceReverseSide::passportElementErrorSourceReverseSide() {
}

const std::int32_t passportElementErrorSourceReverseSide::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceReverseSide::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceReverseSide>();
}

void passportElementErrorSourceReverseSide::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceReverseSide::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceReverseSide");
    s.store_class_end();
  }
}

void passportElementErrorSourceReverseSide::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceReverseSide").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceSelfie::Class;

passportElementErrorSourceSelfie::passportElementErrorSourceSelfie() {
}

const std::int32_t passportElementErrorSourceSelfie::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceSelfie::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceSelfie>();
}

void passportElementErrorSourceSelfie::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceSelfie::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceSelfie");
    s.store_class_end();
  }
}

void passportElementErrorSourceSelfie::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceSelfie").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceTranslationFile::Class;
jfieldID passportElementErrorSourceTranslationFile::file_index_fieldID;

passportElementErrorSourceTranslationFile::passportElementErrorSourceTranslationFile()
  : file_index_()
{}

passportElementErrorSourceTranslationFile::passportElementErrorSourceTranslationFile(int32 file_index_)
  : file_index_(file_index_)
{}

const std::int32_t passportElementErrorSourceTranslationFile::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceTranslationFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementErrorSourceTranslationFile> res = make_object<passportElementErrorSourceTranslationFile>();
  res->file_index_ = env->GetIntField(p, res->file_index_fieldID);
  return std::move(res);
}

void passportElementErrorSourceTranslationFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, file_index_fieldID, file_index_);
}

void passportElementErrorSourceTranslationFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceTranslationFile");
    s.store_field("fileIndex", file_index_);
    s.store_class_end();
  }
}

void passportElementErrorSourceTranslationFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceTranslationFile").c_str());
    file_index_fieldID = jni::get_field_id(env, Class, "fileIndex", "I");
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceTranslationFiles::Class;

passportElementErrorSourceTranslationFiles::passportElementErrorSourceTranslationFiles() {
}

const std::int32_t passportElementErrorSourceTranslationFiles::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceTranslationFiles::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceTranslationFiles>();
}

void passportElementErrorSourceTranslationFiles::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceTranslationFiles::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceTranslationFiles");
    s.store_class_end();
  }
}

void passportElementErrorSourceTranslationFiles::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceTranslationFiles").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceFile::Class;
jfieldID passportElementErrorSourceFile::file_index_fieldID;

passportElementErrorSourceFile::passportElementErrorSourceFile()
  : file_index_()
{}

passportElementErrorSourceFile::passportElementErrorSourceFile(int32 file_index_)
  : file_index_(file_index_)
{}

const std::int32_t passportElementErrorSourceFile::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementErrorSourceFile> res = make_object<passportElementErrorSourceFile>();
  res->file_index_ = env->GetIntField(p, res->file_index_fieldID);
  return std::move(res);
}

void passportElementErrorSourceFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, file_index_fieldID, file_index_);
}

void passportElementErrorSourceFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceFile");
    s.store_field("fileIndex", file_index_);
    s.store_class_end();
  }
}

void passportElementErrorSourceFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceFile").c_str());
    file_index_fieldID = jni::get_field_id(env, Class, "fileIndex", "I");
    return true;
  }();
  (void)is_inited;
}

jclass passportElementErrorSourceFiles::Class;

passportElementErrorSourceFiles::passportElementErrorSourceFiles() {
}

const std::int32_t passportElementErrorSourceFiles::ID;

object_ptr<PassportElementErrorSource> passportElementErrorSourceFiles::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementErrorSourceFiles>();
}

void passportElementErrorSourceFiles::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementErrorSourceFiles::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementErrorSourceFiles");
    s.store_class_end();
  }
}

void passportElementErrorSourceFiles::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementErrorSourceFiles").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PassportElementType::Class;

object_ptr<PassportElementType> PassportElementType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case passportElementTypePersonalDetails::ID:
      return passportElementTypePersonalDetails::fetch(env, p);
    case passportElementTypePassport::ID:
      return passportElementTypePassport::fetch(env, p);
    case passportElementTypeDriverLicense::ID:
      return passportElementTypeDriverLicense::fetch(env, p);
    case passportElementTypeIdentityCard::ID:
      return passportElementTypeIdentityCard::fetch(env, p);
    case passportElementTypeInternalPassport::ID:
      return passportElementTypeInternalPassport::fetch(env, p);
    case passportElementTypeAddress::ID:
      return passportElementTypeAddress::fetch(env, p);
    case passportElementTypeUtilityBill::ID:
      return passportElementTypeUtilityBill::fetch(env, p);
    case passportElementTypeBankStatement::ID:
      return passportElementTypeBankStatement::fetch(env, p);
    case passportElementTypeRentalAgreement::ID:
      return passportElementTypeRentalAgreement::fetch(env, p);
    case passportElementTypePassportRegistration::ID:
      return passportElementTypePassportRegistration::fetch(env, p);
    case passportElementTypeTemporaryRegistration::ID:
      return passportElementTypeTemporaryRegistration::fetch(env, p);
    case passportElementTypePhoneNumber::ID:
      return passportElementTypePhoneNumber::fetch(env, p);
    case passportElementTypeEmailAddress::ID:
      return passportElementTypeEmailAddress::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PassportElementType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypePersonalDetails::Class;

passportElementTypePersonalDetails::passportElementTypePersonalDetails() {
}

const std::int32_t passportElementTypePersonalDetails::ID;

object_ptr<PassportElementType> passportElementTypePersonalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypePersonalDetails>();
}

void passportElementTypePersonalDetails::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypePersonalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypePersonalDetails");
    s.store_class_end();
  }
}

void passportElementTypePersonalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypePersonalDetails").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypePassport::Class;

passportElementTypePassport::passportElementTypePassport() {
}

const std::int32_t passportElementTypePassport::ID;

object_ptr<PassportElementType> passportElementTypePassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypePassport>();
}

void passportElementTypePassport::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypePassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypePassport");
    s.store_class_end();
  }
}

void passportElementTypePassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypePassport").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeDriverLicense::Class;

passportElementTypeDriverLicense::passportElementTypeDriverLicense() {
}

const std::int32_t passportElementTypeDriverLicense::ID;

object_ptr<PassportElementType> passportElementTypeDriverLicense::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeDriverLicense>();
}

void passportElementTypeDriverLicense::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeDriverLicense::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeDriverLicense");
    s.store_class_end();
  }
}

void passportElementTypeDriverLicense::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeDriverLicense").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeIdentityCard::Class;

passportElementTypeIdentityCard::passportElementTypeIdentityCard() {
}

const std::int32_t passportElementTypeIdentityCard::ID;

object_ptr<PassportElementType> passportElementTypeIdentityCard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeIdentityCard>();
}

void passportElementTypeIdentityCard::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeIdentityCard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeIdentityCard");
    s.store_class_end();
  }
}

void passportElementTypeIdentityCard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeIdentityCard").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeInternalPassport::Class;

passportElementTypeInternalPassport::passportElementTypeInternalPassport() {
}

const std::int32_t passportElementTypeInternalPassport::ID;

object_ptr<PassportElementType> passportElementTypeInternalPassport::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeInternalPassport>();
}

void passportElementTypeInternalPassport::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeInternalPassport::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeInternalPassport");
    s.store_class_end();
  }
}

void passportElementTypeInternalPassport::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeInternalPassport").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeAddress::Class;

passportElementTypeAddress::passportElementTypeAddress() {
}

const std::int32_t passportElementTypeAddress::ID;

object_ptr<PassportElementType> passportElementTypeAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeAddress>();
}

void passportElementTypeAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeAddress");
    s.store_class_end();
  }
}

void passportElementTypeAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeAddress").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeUtilityBill::Class;

passportElementTypeUtilityBill::passportElementTypeUtilityBill() {
}

const std::int32_t passportElementTypeUtilityBill::ID;

object_ptr<PassportElementType> passportElementTypeUtilityBill::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeUtilityBill>();
}

void passportElementTypeUtilityBill::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeUtilityBill::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeUtilityBill");
    s.store_class_end();
  }
}

void passportElementTypeUtilityBill::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeUtilityBill").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeBankStatement::Class;

passportElementTypeBankStatement::passportElementTypeBankStatement() {
}

const std::int32_t passportElementTypeBankStatement::ID;

object_ptr<PassportElementType> passportElementTypeBankStatement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeBankStatement>();
}

void passportElementTypeBankStatement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeBankStatement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeBankStatement");
    s.store_class_end();
  }
}

void passportElementTypeBankStatement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeBankStatement").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeRentalAgreement::Class;

passportElementTypeRentalAgreement::passportElementTypeRentalAgreement() {
}

const std::int32_t passportElementTypeRentalAgreement::ID;

object_ptr<PassportElementType> passportElementTypeRentalAgreement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeRentalAgreement>();
}

void passportElementTypeRentalAgreement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeRentalAgreement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeRentalAgreement");
    s.store_class_end();
  }
}

void passportElementTypeRentalAgreement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeRentalAgreement").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypePassportRegistration::Class;

passportElementTypePassportRegistration::passportElementTypePassportRegistration() {
}

const std::int32_t passportElementTypePassportRegistration::ID;

object_ptr<PassportElementType> passportElementTypePassportRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypePassportRegistration>();
}

void passportElementTypePassportRegistration::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypePassportRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypePassportRegistration");
    s.store_class_end();
  }
}

void passportElementTypePassportRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypePassportRegistration").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeTemporaryRegistration::Class;

passportElementTypeTemporaryRegistration::passportElementTypeTemporaryRegistration() {
}

const std::int32_t passportElementTypeTemporaryRegistration::ID;

object_ptr<PassportElementType> passportElementTypeTemporaryRegistration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeTemporaryRegistration>();
}

void passportElementTypeTemporaryRegistration::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeTemporaryRegistration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeTemporaryRegistration");
    s.store_class_end();
  }
}

void passportElementTypeTemporaryRegistration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeTemporaryRegistration").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypePhoneNumber::Class;

passportElementTypePhoneNumber::passportElementTypePhoneNumber() {
}

const std::int32_t passportElementTypePhoneNumber::ID;

object_ptr<PassportElementType> passportElementTypePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypePhoneNumber>();
}

void passportElementTypePhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypePhoneNumber");
    s.store_class_end();
  }
}

void passportElementTypePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypePhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementTypeEmailAddress::Class;

passportElementTypeEmailAddress::passportElementTypeEmailAddress() {
}

const std::int32_t passportElementTypeEmailAddress::ID;

object_ptr<PassportElementType> passportElementTypeEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<passportElementTypeEmailAddress>();
}

void passportElementTypeEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void passportElementTypeEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementTypeEmailAddress");
    s.store_class_end();
  }
}

void passportElementTypeEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementTypeEmailAddress").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElements::Class;
jfieldID passportElements::elements_fieldID;

passportElements::passportElements()
  : elements_()
{}

passportElements::passportElements(array<object_ptr<PassportElement>> &&elements_)
  : elements_(std::move(elements_))
{}

const std::int32_t passportElements::ID;

object_ptr<passportElements> passportElements::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElements> res = make_object<passportElements>();
  res->elements_ = jni::FetchVector<PassportElement>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->elements_fieldID));
  return res;
}

void passportElements::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, elements_); if (arr_tmp_) { env->SetObjectField(s, elements_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void passportElements::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElements");
    { s.store_vector_begin("elements", elements_.size()); for (const auto &_value : elements_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void passportElements::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElements").c_str());
    elements_fieldID = jni::get_field_id(env, Class, "elements", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportElement;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportElementsWithErrors::Class;
jfieldID passportElementsWithErrors::elements_fieldID;
jfieldID passportElementsWithErrors::errors_fieldID;

passportElementsWithErrors::passportElementsWithErrors()
  : elements_()
  , errors_()
{}

passportElementsWithErrors::passportElementsWithErrors(array<object_ptr<PassportElement>> &&elements_, array<object_ptr<passportElementError>> &&errors_)
  : elements_(std::move(elements_))
  , errors_(std::move(errors_))
{}

const std::int32_t passportElementsWithErrors::ID;

object_ptr<passportElementsWithErrors> passportElementsWithErrors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportElementsWithErrors> res = make_object<passportElementsWithErrors>();
  res->elements_ = jni::FetchVector<PassportElement>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->elements_fieldID));
  res->errors_ = jni::FetchVector<passportElementError>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->errors_fieldID));
  return res;
}

void passportElementsWithErrors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, elements_); if (arr_tmp_) { env->SetObjectField(s, elements_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, errors_); if (arr_tmp_) { env->SetObjectField(s, errors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void passportElementsWithErrors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportElementsWithErrors");
    { s.store_vector_begin("elements", elements_.size()); for (const auto &_value : elements_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("errors", errors_.size()); for (const auto &_value : errors_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void passportElementsWithErrors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportElementsWithErrors").c_str());
    elements_fieldID = jni::get_field_id(env, Class, "elements", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportElement;").c_str());
    errors_fieldID = jni::get_field_id(env, Class, "errors", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportElementError;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportRequiredElement::Class;
jfieldID passportRequiredElement::suitable_elements_fieldID;

passportRequiredElement::passportRequiredElement()
  : suitable_elements_()
{}

passportRequiredElement::passportRequiredElement(array<object_ptr<passportSuitableElement>> &&suitable_elements_)
  : suitable_elements_(std::move(suitable_elements_))
{}

const std::int32_t passportRequiredElement::ID;

object_ptr<passportRequiredElement> passportRequiredElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportRequiredElement> res = make_object<passportRequiredElement>();
  res->suitable_elements_ = jni::FetchVector<passportSuitableElement>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->suitable_elements_fieldID));
  return res;
}

void passportRequiredElement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, suitable_elements_); if (arr_tmp_) { env->SetObjectField(s, suitable_elements_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void passportRequiredElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportRequiredElement");
    { s.store_vector_begin("suitableElements", suitable_elements_.size()); for (const auto &_value : suitable_elements_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void passportRequiredElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportRequiredElement").c_str());
    suitable_elements_fieldID = jni::get_field_id(env, Class, "suitableElements", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportSuitableElement;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass passportSuitableElement::Class;
jfieldID passportSuitableElement::type_fieldID;
jfieldID passportSuitableElement::is_selfie_required_fieldID;
jfieldID passportSuitableElement::is_translation_required_fieldID;
jfieldID passportSuitableElement::is_native_name_required_fieldID;

passportSuitableElement::passportSuitableElement()
  : type_()
  , is_selfie_required_()
  , is_translation_required_()
  , is_native_name_required_()
{}

passportSuitableElement::passportSuitableElement(object_ptr<PassportElementType> &&type_, bool is_selfie_required_, bool is_translation_required_, bool is_native_name_required_)
  : type_(std::move(type_))
  , is_selfie_required_(is_selfie_required_)
  , is_translation_required_(is_translation_required_)
  , is_native_name_required_(is_native_name_required_)
{}

const std::int32_t passportSuitableElement::ID;

object_ptr<passportSuitableElement> passportSuitableElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passportSuitableElement> res = make_object<passportSuitableElement>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->is_selfie_required_ = (env->GetBooleanField(p, res->is_selfie_required_fieldID) != 0);
  res->is_translation_required_ = (env->GetBooleanField(p, res->is_translation_required_fieldID) != 0);
  res->is_native_name_required_ = (env->GetBooleanField(p, res->is_native_name_required_fieldID) != 0);
  return res;
}

void passportSuitableElement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_selfie_required_fieldID, is_selfie_required_);
  env->SetBooleanField(s, is_translation_required_fieldID, is_translation_required_);
  env->SetBooleanField(s, is_native_name_required_fieldID, is_native_name_required_);
}

void passportSuitableElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PassportSuitableElement");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("isSelfieRequired", is_selfie_required_);
    s.store_field("isTranslationRequired", is_translation_required_);
    s.store_field("isNativeNameRequired", is_native_name_required_);
    s.store_class_end();
  }
}

void passportSuitableElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PassportSuitableElement").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    is_selfie_required_fieldID = jni::get_field_id(env, Class, "isSelfieRequired", "Z");
    is_translation_required_fieldID = jni::get_field_id(env, Class, "isTranslationRequired", "Z");
    is_native_name_required_fieldID = jni::get_field_id(env, Class, "isNativeNameRequired", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass passwordState::Class;
jfieldID passwordState::has_password_fieldID;
jfieldID passwordState::password_hint_fieldID;
jfieldID passwordState::has_recovery_email_address_fieldID;
jfieldID passwordState::has_passport_data_fieldID;
jfieldID passwordState::recovery_email_address_code_info_fieldID;
jfieldID passwordState::login_email_address_pattern_fieldID;
jfieldID passwordState::pending_reset_date_fieldID;

passwordState::passwordState()
  : has_password_()
  , password_hint_()
  , has_recovery_email_address_()
  , has_passport_data_()
  , recovery_email_address_code_info_()
  , login_email_address_pattern_()
  , pending_reset_date_()
{}

passwordState::passwordState(bool has_password_, string const &password_hint_, bool has_recovery_email_address_, bool has_passport_data_, object_ptr<emailAddressAuthenticationCodeInfo> &&recovery_email_address_code_info_, string const &login_email_address_pattern_, int32 pending_reset_date_)
  : has_password_(has_password_)
  , password_hint_(password_hint_)
  , has_recovery_email_address_(has_recovery_email_address_)
  , has_passport_data_(has_passport_data_)
  , recovery_email_address_code_info_(std::move(recovery_email_address_code_info_))
  , login_email_address_pattern_(login_email_address_pattern_)
  , pending_reset_date_(pending_reset_date_)
{}

const std::int32_t passwordState::ID;

object_ptr<passwordState> passwordState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<passwordState> res = make_object<passwordState>();
  res->has_password_ = (env->GetBooleanField(p, res->has_password_fieldID) != 0);
  res->password_hint_ = jni::fetch_string(env, p, res->password_hint_fieldID);
  res->has_recovery_email_address_ = (env->GetBooleanField(p, res->has_recovery_email_address_fieldID) != 0);
  res->has_passport_data_ = (env->GetBooleanField(p, res->has_passport_data_fieldID) != 0);
  res->recovery_email_address_code_info_ = jni::fetch_tl_object<emailAddressAuthenticationCodeInfo>(env, jni::fetch_object(env, p, res->recovery_email_address_code_info_fieldID));
  res->login_email_address_pattern_ = jni::fetch_string(env, p, res->login_email_address_pattern_fieldID);
  res->pending_reset_date_ = env->GetIntField(p, res->pending_reset_date_fieldID);
  return res;
}

void passwordState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_password_fieldID, has_password_);
  { jstring nextString = jni::to_jstring(env, password_hint_); if (nextString) { env->SetObjectField(s, password_hint_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, has_recovery_email_address_fieldID, has_recovery_email_address_);
  env->SetBooleanField(s, has_passport_data_fieldID, has_passport_data_);
  if (recovery_email_address_code_info_ != nullptr) { jobject next; recovery_email_address_code_info_->store(env, next); if (next) { env->SetObjectField(s, recovery_email_address_code_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, login_email_address_pattern_); if (nextString) { env->SetObjectField(s, login_email_address_pattern_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, pending_reset_date_fieldID, pending_reset_date_);
}

void passwordState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PasswordState");
    s.store_field("hasPassword", has_password_);
    s.store_field("passwordHint", password_hint_);
    s.store_field("hasRecoveryEmailAddress", has_recovery_email_address_);
    s.store_field("hasPassportData", has_passport_data_);
    s.store_object_field("recoveryEmailAddressCodeInfo", static_cast<const BaseObject *>(recovery_email_address_code_info_.get()));
    s.store_field("loginEmailAddressPattern", login_email_address_pattern_);
    s.store_field("pendingResetDate", pending_reset_date_);
    s.store_class_end();
  }
}

void passwordState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PasswordState").c_str());
    has_password_fieldID = jni::get_field_id(env, Class, "hasPassword", "Z");
    password_hint_fieldID = jni::get_field_id(env, Class, "passwordHint", "Ljava/lang/String;");
    has_recovery_email_address_fieldID = jni::get_field_id(env, Class, "hasRecoveryEmailAddress", "Z");
    has_passport_data_fieldID = jni::get_field_id(env, Class, "hasPassportData", "Z");
    recovery_email_address_code_info_fieldID = jni::get_field_id(env, Class, "recoveryEmailAddressCodeInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmailAddressAuthenticationCodeInfo;").c_str());
    login_email_address_pattern_fieldID = jni::get_field_id(env, Class, "loginEmailAddressPattern", "Ljava/lang/String;");
    pending_reset_date_fieldID = jni::get_field_id(env, Class, "pendingResetDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass paymentForm::Class;
jfieldID paymentForm::id_fieldID;
jfieldID paymentForm::type_fieldID;
jfieldID paymentForm::seller_bot_user_id_fieldID;
jfieldID paymentForm::product_info_fieldID;

paymentForm::paymentForm()
  : id_()
  , type_()
  , seller_bot_user_id_()
  , product_info_()
{}

paymentForm::paymentForm(int64 id_, object_ptr<PaymentFormType> &&type_, int53 seller_bot_user_id_, object_ptr<productInfo> &&product_info_)
  : id_(id_)
  , type_(std::move(type_))
  , seller_bot_user_id_(seller_bot_user_id_)
  , product_info_(std::move(product_info_))
{}

const std::int32_t paymentForm::ID;

object_ptr<paymentForm> paymentForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentForm> res = make_object<paymentForm>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->type_ = jni::fetch_tl_object<PaymentFormType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->seller_bot_user_id_ = env->GetLongField(p, res->seller_bot_user_id_fieldID);
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  return res;
}

void paymentForm::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, seller_bot_user_id_fieldID, seller_bot_user_id_);
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void paymentForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentForm");
    s.store_field("id", id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("sellerBotUserId", seller_bot_user_id_);
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_class_end();
  }
}

void paymentForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentForm").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaymentFormType;").c_str());
    seller_bot_user_id_fieldID = jni::get_field_id(env, Class, "sellerBotUserId", "J");
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PaymentFormType::Class;

object_ptr<PaymentFormType> PaymentFormType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case paymentFormTypeRegular::ID:
      return paymentFormTypeRegular::fetch(env, p);
    case paymentFormTypeStars::ID:
      return paymentFormTypeStars::fetch(env, p);
    case paymentFormTypeStarSubscription::ID:
      return paymentFormTypeStarSubscription::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PaymentFormType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentFormType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paymentFormTypeRegular::Class;
jfieldID paymentFormTypeRegular::invoice_fieldID;
jfieldID paymentFormTypeRegular::payment_provider_user_id_fieldID;
jfieldID paymentFormTypeRegular::payment_provider_fieldID;
jfieldID paymentFormTypeRegular::additional_payment_options_fieldID;
jfieldID paymentFormTypeRegular::saved_order_info_fieldID;
jfieldID paymentFormTypeRegular::saved_credentials_fieldID;
jfieldID paymentFormTypeRegular::can_save_credentials_fieldID;
jfieldID paymentFormTypeRegular::need_password_fieldID;

paymentFormTypeRegular::paymentFormTypeRegular()
  : invoice_()
  , payment_provider_user_id_()
  , payment_provider_()
  , additional_payment_options_()
  , saved_order_info_()
  , saved_credentials_()
  , can_save_credentials_()
  , need_password_()
{}

paymentFormTypeRegular::paymentFormTypeRegular(object_ptr<invoice> &&invoice_, int53 payment_provider_user_id_, object_ptr<PaymentProvider> &&payment_provider_, array<object_ptr<paymentOption>> &&additional_payment_options_, object_ptr<orderInfo> &&saved_order_info_, array<object_ptr<savedCredentials>> &&saved_credentials_, bool can_save_credentials_, bool need_password_)
  : invoice_(std::move(invoice_))
  , payment_provider_user_id_(payment_provider_user_id_)
  , payment_provider_(std::move(payment_provider_))
  , additional_payment_options_(std::move(additional_payment_options_))
  , saved_order_info_(std::move(saved_order_info_))
  , saved_credentials_(std::move(saved_credentials_))
  , can_save_credentials_(can_save_credentials_)
  , need_password_(need_password_)
{}

const std::int32_t paymentFormTypeRegular::ID;

object_ptr<PaymentFormType> paymentFormTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentFormTypeRegular> res = make_object<paymentFormTypeRegular>();
  res->invoice_ = jni::fetch_tl_object<invoice>(env, jni::fetch_object(env, p, res->invoice_fieldID));
  res->payment_provider_user_id_ = env->GetLongField(p, res->payment_provider_user_id_fieldID);
  res->payment_provider_ = jni::fetch_tl_object<PaymentProvider>(env, jni::fetch_object(env, p, res->payment_provider_fieldID));
  res->additional_payment_options_ = jni::FetchVector<paymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->additional_payment_options_fieldID));
  res->saved_order_info_ = jni::fetch_tl_object<orderInfo>(env, jni::fetch_object(env, p, res->saved_order_info_fieldID));
  res->saved_credentials_ = jni::FetchVector<savedCredentials>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->saved_credentials_fieldID));
  res->can_save_credentials_ = (env->GetBooleanField(p, res->can_save_credentials_fieldID) != 0);
  res->need_password_ = (env->GetBooleanField(p, res->need_password_fieldID) != 0);
  return std::move(res);
}

void paymentFormTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (invoice_ != nullptr) { jobject next; invoice_->store(env, next); if (next) { env->SetObjectField(s, invoice_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, payment_provider_user_id_fieldID, payment_provider_user_id_);
  if (payment_provider_ != nullptr) { jobject next; payment_provider_->store(env, next); if (next) { env->SetObjectField(s, payment_provider_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, additional_payment_options_); if (arr_tmp_) { env->SetObjectField(s, additional_payment_options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (saved_order_info_ != nullptr) { jobject next; saved_order_info_->store(env, next); if (next) { env->SetObjectField(s, saved_order_info_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, saved_credentials_); if (arr_tmp_) { env->SetObjectField(s, saved_credentials_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, can_save_credentials_fieldID, can_save_credentials_);
  env->SetBooleanField(s, need_password_fieldID, need_password_);
}

void paymentFormTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentFormTypeRegular");
    s.store_object_field("invoice", static_cast<const BaseObject *>(invoice_.get()));
    s.store_field("paymentProviderUserId", payment_provider_user_id_);
    s.store_object_field("paymentProvider", static_cast<const BaseObject *>(payment_provider_.get()));
    { s.store_vector_begin("additionalPaymentOptions", additional_payment_options_.size()); for (const auto &_value : additional_payment_options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("savedOrderInfo", static_cast<const BaseObject *>(saved_order_info_.get()));
    { s.store_vector_begin("savedCredentials", saved_credentials_.size()); for (const auto &_value : saved_credentials_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("canSaveCredentials", can_save_credentials_);
    s.store_field("needPassword", need_password_);
    s.store_class_end();
  }
}

void paymentFormTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentFormTypeRegular").c_str());
    invoice_fieldID = jni::get_field_id(env, Class, "invoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Invoice;").c_str());
    payment_provider_user_id_fieldID = jni::get_field_id(env, Class, "paymentProviderUserId", "J");
    payment_provider_fieldID = jni::get_field_id(env, Class, "paymentProvider", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaymentProvider;").c_str());
    additional_payment_options_fieldID = jni::get_field_id(env, Class, "additionalPaymentOptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaymentOption;").c_str());
    saved_order_info_fieldID = jni::get_field_id(env, Class, "savedOrderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OrderInfo;").c_str());
    saved_credentials_fieldID = jni::get_field_id(env, Class, "savedCredentials", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SavedCredentials;").c_str());
    can_save_credentials_fieldID = jni::get_field_id(env, Class, "canSaveCredentials", "Z");
    need_password_fieldID = jni::get_field_id(env, Class, "needPassword", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass paymentFormTypeStars::Class;
jfieldID paymentFormTypeStars::star_count_fieldID;

paymentFormTypeStars::paymentFormTypeStars()
  : star_count_()
{}

paymentFormTypeStars::paymentFormTypeStars(int53 star_count_)
  : star_count_(star_count_)
{}

const std::int32_t paymentFormTypeStars::ID;

object_ptr<PaymentFormType> paymentFormTypeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentFormTypeStars> res = make_object<paymentFormTypeStars>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void paymentFormTypeStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void paymentFormTypeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentFormTypeStars");
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void paymentFormTypeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentFormTypeStars").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass paymentFormTypeStarSubscription::Class;
jfieldID paymentFormTypeStarSubscription::pricing_fieldID;

paymentFormTypeStarSubscription::paymentFormTypeStarSubscription()
  : pricing_()
{}

paymentFormTypeStarSubscription::paymentFormTypeStarSubscription(object_ptr<starSubscriptionPricing> &&pricing_)
  : pricing_(std::move(pricing_))
{}

const std::int32_t paymentFormTypeStarSubscription::ID;

object_ptr<PaymentFormType> paymentFormTypeStarSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentFormTypeStarSubscription> res = make_object<paymentFormTypeStarSubscription>();
  res->pricing_ = jni::fetch_tl_object<starSubscriptionPricing>(env, jni::fetch_object(env, p, res->pricing_fieldID));
  return std::move(res);
}

void paymentFormTypeStarSubscription::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (pricing_ != nullptr) { jobject next; pricing_->store(env, next); if (next) { env->SetObjectField(s, pricing_fieldID, next); env->DeleteLocalRef(next); } }
}

void paymentFormTypeStarSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentFormTypeStarSubscription");
    s.store_object_field("pricing", static_cast<const BaseObject *>(pricing_.get()));
    s.store_class_end();
  }
}

void paymentFormTypeStarSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentFormTypeStarSubscription").c_str());
    pricing_fieldID = jni::get_field_id(env, Class, "pricing", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionPricing;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paymentOption::Class;
jfieldID paymentOption::title_fieldID;
jfieldID paymentOption::url_fieldID;

paymentOption::paymentOption()
  : title_()
  , url_()
{}

paymentOption::paymentOption(string const &title_, string const &url_)
  : title_(title_)
  , url_(url_)
{}

const std::int32_t paymentOption::ID;

object_ptr<paymentOption> paymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentOption> res = make_object<paymentOption>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void paymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void paymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentOption");
    s.store_field("title", title_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void paymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentOption").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PaymentProvider::Class;

object_ptr<PaymentProvider> PaymentProvider::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case paymentProviderSmartGlocal::ID:
      return paymentProviderSmartGlocal::fetch(env, p);
    case paymentProviderStripe::ID:
      return paymentProviderStripe::fetch(env, p);
    case paymentProviderOther::ID:
      return paymentProviderOther::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PaymentProvider::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentProvider").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paymentProviderSmartGlocal::Class;
jfieldID paymentProviderSmartGlocal::public_token_fieldID;
jfieldID paymentProviderSmartGlocal::tokenize_url_fieldID;

paymentProviderSmartGlocal::paymentProviderSmartGlocal()
  : public_token_()
  , tokenize_url_()
{}

paymentProviderSmartGlocal::paymentProviderSmartGlocal(string const &public_token_, string const &tokenize_url_)
  : public_token_(public_token_)
  , tokenize_url_(tokenize_url_)
{}

const std::int32_t paymentProviderSmartGlocal::ID;

object_ptr<PaymentProvider> paymentProviderSmartGlocal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentProviderSmartGlocal> res = make_object<paymentProviderSmartGlocal>();
  res->public_token_ = jni::fetch_string(env, p, res->public_token_fieldID);
  res->tokenize_url_ = jni::fetch_string(env, p, res->tokenize_url_fieldID);
  return std::move(res);
}

void paymentProviderSmartGlocal::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, public_token_); if (nextString) { env->SetObjectField(s, public_token_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, tokenize_url_); if (nextString) { env->SetObjectField(s, tokenize_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void paymentProviderSmartGlocal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentProviderSmartGlocal");
    s.store_field("publicToken", public_token_);
    s.store_field("tokenizeUrl", tokenize_url_);
    s.store_class_end();
  }
}

void paymentProviderSmartGlocal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentProviderSmartGlocal").c_str());
    public_token_fieldID = jni::get_field_id(env, Class, "publicToken", "Ljava/lang/String;");
    tokenize_url_fieldID = jni::get_field_id(env, Class, "tokenizeUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass paymentProviderStripe::Class;
jfieldID paymentProviderStripe::publishable_key_fieldID;
jfieldID paymentProviderStripe::need_country_fieldID;
jfieldID paymentProviderStripe::need_postal_code_fieldID;
jfieldID paymentProviderStripe::need_cardholder_name_fieldID;

paymentProviderStripe::paymentProviderStripe()
  : publishable_key_()
  , need_country_()
  , need_postal_code_()
  , need_cardholder_name_()
{}

paymentProviderStripe::paymentProviderStripe(string const &publishable_key_, bool need_country_, bool need_postal_code_, bool need_cardholder_name_)
  : publishable_key_(publishable_key_)
  , need_country_(need_country_)
  , need_postal_code_(need_postal_code_)
  , need_cardholder_name_(need_cardholder_name_)
{}

const std::int32_t paymentProviderStripe::ID;

object_ptr<PaymentProvider> paymentProviderStripe::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentProviderStripe> res = make_object<paymentProviderStripe>();
  res->publishable_key_ = jni::fetch_string(env, p, res->publishable_key_fieldID);
  res->need_country_ = (env->GetBooleanField(p, res->need_country_fieldID) != 0);
  res->need_postal_code_ = (env->GetBooleanField(p, res->need_postal_code_fieldID) != 0);
  res->need_cardholder_name_ = (env->GetBooleanField(p, res->need_cardholder_name_fieldID) != 0);
  return std::move(res);
}

void paymentProviderStripe::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, publishable_key_); if (nextString) { env->SetObjectField(s, publishable_key_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, need_country_fieldID, need_country_);
  env->SetBooleanField(s, need_postal_code_fieldID, need_postal_code_);
  env->SetBooleanField(s, need_cardholder_name_fieldID, need_cardholder_name_);
}

void paymentProviderStripe::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentProviderStripe");
    s.store_field("publishableKey", publishable_key_);
    s.store_field("needCountry", need_country_);
    s.store_field("needPostalCode", need_postal_code_);
    s.store_field("needCardholderName", need_cardholder_name_);
    s.store_class_end();
  }
}

void paymentProviderStripe::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentProviderStripe").c_str());
    publishable_key_fieldID = jni::get_field_id(env, Class, "publishableKey", "Ljava/lang/String;");
    need_country_fieldID = jni::get_field_id(env, Class, "needCountry", "Z");
    need_postal_code_fieldID = jni::get_field_id(env, Class, "needPostalCode", "Z");
    need_cardholder_name_fieldID = jni::get_field_id(env, Class, "needCardholderName", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass paymentProviderOther::Class;
jfieldID paymentProviderOther::url_fieldID;

paymentProviderOther::paymentProviderOther()
  : url_()
{}

paymentProviderOther::paymentProviderOther(string const &url_)
  : url_(url_)
{}

const std::int32_t paymentProviderOther::ID;

object_ptr<PaymentProvider> paymentProviderOther::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentProviderOther> res = make_object<paymentProviderOther>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void paymentProviderOther::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void paymentProviderOther::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentProviderOther");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void paymentProviderOther::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentProviderOther").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass paymentReceipt::Class;
jfieldID paymentReceipt::product_info_fieldID;
jfieldID paymentReceipt::date_fieldID;
jfieldID paymentReceipt::seller_bot_user_id_fieldID;
jfieldID paymentReceipt::type_fieldID;

paymentReceipt::paymentReceipt()
  : product_info_()
  , date_()
  , seller_bot_user_id_()
  , type_()
{}

paymentReceipt::paymentReceipt(object_ptr<productInfo> &&product_info_, int32 date_, int53 seller_bot_user_id_, object_ptr<PaymentReceiptType> &&type_)
  : product_info_(std::move(product_info_))
  , date_(date_)
  , seller_bot_user_id_(seller_bot_user_id_)
  , type_(std::move(type_))
{}

const std::int32_t paymentReceipt::ID;

object_ptr<paymentReceipt> paymentReceipt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentReceipt> res = make_object<paymentReceipt>();
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->seller_bot_user_id_ = env->GetLongField(p, res->seller_bot_user_id_fieldID);
  res->type_ = jni::fetch_tl_object<PaymentReceiptType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void paymentReceipt::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  env->SetLongField(s, seller_bot_user_id_fieldID, seller_bot_user_id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void paymentReceipt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentReceipt");
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_field("date", date_);
    s.store_field("sellerBotUserId", seller_bot_user_id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void paymentReceipt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentReceipt").c_str());
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    seller_bot_user_id_fieldID = jni::get_field_id(env, Class, "sellerBotUserId", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaymentReceiptType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PaymentReceiptType::Class;

object_ptr<PaymentReceiptType> PaymentReceiptType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case paymentReceiptTypeRegular::ID:
      return paymentReceiptTypeRegular::fetch(env, p);
    case paymentReceiptTypeStars::ID:
      return paymentReceiptTypeStars::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PaymentReceiptType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentReceiptType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass paymentReceiptTypeRegular::Class;
jfieldID paymentReceiptTypeRegular::payment_provider_user_id_fieldID;
jfieldID paymentReceiptTypeRegular::invoice_fieldID;
jfieldID paymentReceiptTypeRegular::order_info_fieldID;
jfieldID paymentReceiptTypeRegular::shipping_option_fieldID;
jfieldID paymentReceiptTypeRegular::credentials_title_fieldID;
jfieldID paymentReceiptTypeRegular::tip_amount_fieldID;

paymentReceiptTypeRegular::paymentReceiptTypeRegular()
  : payment_provider_user_id_()
  , invoice_()
  , order_info_()
  , shipping_option_()
  , credentials_title_()
  , tip_amount_()
{}

paymentReceiptTypeRegular::paymentReceiptTypeRegular(int53 payment_provider_user_id_, object_ptr<invoice> &&invoice_, object_ptr<orderInfo> &&order_info_, object_ptr<shippingOption> &&shipping_option_, string const &credentials_title_, int53 tip_amount_)
  : payment_provider_user_id_(payment_provider_user_id_)
  , invoice_(std::move(invoice_))
  , order_info_(std::move(order_info_))
  , shipping_option_(std::move(shipping_option_))
  , credentials_title_(credentials_title_)
  , tip_amount_(tip_amount_)
{}

const std::int32_t paymentReceiptTypeRegular::ID;

object_ptr<PaymentReceiptType> paymentReceiptTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentReceiptTypeRegular> res = make_object<paymentReceiptTypeRegular>();
  res->payment_provider_user_id_ = env->GetLongField(p, res->payment_provider_user_id_fieldID);
  res->invoice_ = jni::fetch_tl_object<invoice>(env, jni::fetch_object(env, p, res->invoice_fieldID));
  res->order_info_ = jni::fetch_tl_object<orderInfo>(env, jni::fetch_object(env, p, res->order_info_fieldID));
  res->shipping_option_ = jni::fetch_tl_object<shippingOption>(env, jni::fetch_object(env, p, res->shipping_option_fieldID));
  res->credentials_title_ = jni::fetch_string(env, p, res->credentials_title_fieldID);
  res->tip_amount_ = env->GetLongField(p, res->tip_amount_fieldID);
  return std::move(res);
}

void paymentReceiptTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, payment_provider_user_id_fieldID, payment_provider_user_id_);
  if (invoice_ != nullptr) { jobject next; invoice_->store(env, next); if (next) { env->SetObjectField(s, invoice_fieldID, next); env->DeleteLocalRef(next); } }
  if (order_info_ != nullptr) { jobject next; order_info_->store(env, next); if (next) { env->SetObjectField(s, order_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (shipping_option_ != nullptr) { jobject next; shipping_option_->store(env, next); if (next) { env->SetObjectField(s, shipping_option_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, credentials_title_); if (nextString) { env->SetObjectField(s, credentials_title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, tip_amount_fieldID, tip_amount_);
}

void paymentReceiptTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentReceiptTypeRegular");
    s.store_field("paymentProviderUserId", payment_provider_user_id_);
    s.store_object_field("invoice", static_cast<const BaseObject *>(invoice_.get()));
    s.store_object_field("orderInfo", static_cast<const BaseObject *>(order_info_.get()));
    s.store_object_field("shippingOption", static_cast<const BaseObject *>(shipping_option_.get()));
    s.store_field("credentialsTitle", credentials_title_);
    s.store_field("tipAmount", tip_amount_);
    s.store_class_end();
  }
}

void paymentReceiptTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentReceiptTypeRegular").c_str());
    payment_provider_user_id_fieldID = jni::get_field_id(env, Class, "paymentProviderUserId", "J");
    invoice_fieldID = jni::get_field_id(env, Class, "invoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Invoice;").c_str());
    order_info_fieldID = jni::get_field_id(env, Class, "orderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OrderInfo;").c_str());
    shipping_option_fieldID = jni::get_field_id(env, Class, "shippingOption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ShippingOption;").c_str());
    credentials_title_fieldID = jni::get_field_id(env, Class, "credentialsTitle", "Ljava/lang/String;");
    tip_amount_fieldID = jni::get_field_id(env, Class, "tipAmount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass paymentReceiptTypeStars::Class;
jfieldID paymentReceiptTypeStars::star_count_fieldID;
jfieldID paymentReceiptTypeStars::transaction_id_fieldID;

paymentReceiptTypeStars::paymentReceiptTypeStars()
  : star_count_()
  , transaction_id_()
{}

paymentReceiptTypeStars::paymentReceiptTypeStars(int53 star_count_, string const &transaction_id_)
  : star_count_(star_count_)
  , transaction_id_(transaction_id_)
{}

const std::int32_t paymentReceiptTypeStars::ID;

object_ptr<PaymentReceiptType> paymentReceiptTypeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentReceiptTypeStars> res = make_object<paymentReceiptTypeStars>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->transaction_id_ = jni::fetch_string(env, p, res->transaction_id_fieldID);
  return std::move(res);
}

void paymentReceiptTypeStars::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, transaction_id_); if (nextString) { env->SetObjectField(s, transaction_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void paymentReceiptTypeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentReceiptTypeStars");
    s.store_field("starCount", star_count_);
    s.store_field("transactionId", transaction_id_);
    s.store_class_end();
  }
}

void paymentReceiptTypeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentReceiptTypeStars").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    transaction_id_fieldID = jni::get_field_id(env, Class, "transactionId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass paymentResult::Class;
jfieldID paymentResult::success_fieldID;
jfieldID paymentResult::verification_url_fieldID;

paymentResult::paymentResult()
  : success_()
  , verification_url_()
{}

paymentResult::paymentResult(bool success_, string const &verification_url_)
  : success_(success_)
  , verification_url_(verification_url_)
{}

const std::int32_t paymentResult::ID;

object_ptr<paymentResult> paymentResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<paymentResult> res = make_object<paymentResult>();
  res->success_ = (env->GetBooleanField(p, res->success_fieldID) != 0);
  res->verification_url_ = jni::fetch_string(env, p, res->verification_url_fieldID);
  return res;
}

void paymentResult::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, success_fieldID, success_);
  { jstring nextString = jni::to_jstring(env, verification_url_); if (nextString) { env->SetObjectField(s, verification_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void paymentResult::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PaymentResult");
    s.store_field("success", success_);
    s.store_field("verificationUrl", verification_url_);
    s.store_class_end();
  }
}

void paymentResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PaymentResult").c_str());
    success_fieldID = jni::get_field_id(env, Class, "success", "Z");
    verification_url_fieldID = jni::get_field_id(env, Class, "verificationUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass personalDetails::Class;
jfieldID personalDetails::first_name_fieldID;
jfieldID personalDetails::middle_name_fieldID;
jfieldID personalDetails::last_name_fieldID;
jfieldID personalDetails::native_first_name_fieldID;
jfieldID personalDetails::native_middle_name_fieldID;
jfieldID personalDetails::native_last_name_fieldID;
jfieldID personalDetails::birthdate_fieldID;
jfieldID personalDetails::gender_fieldID;
jfieldID personalDetails::country_code_fieldID;
jfieldID personalDetails::residence_country_code_fieldID;

personalDetails::personalDetails()
  : first_name_()
  , middle_name_()
  , last_name_()
  , native_first_name_()
  , native_middle_name_()
  , native_last_name_()
  , birthdate_()
  , gender_()
  , country_code_()
  , residence_country_code_()
{}

personalDetails::personalDetails(string const &first_name_, string const &middle_name_, string const &last_name_, string const &native_first_name_, string const &native_middle_name_, string const &native_last_name_, object_ptr<date> &&birthdate_, string const &gender_, string const &country_code_, string const &residence_country_code_)
  : first_name_(first_name_)
  , middle_name_(middle_name_)
  , last_name_(last_name_)
  , native_first_name_(native_first_name_)
  , native_middle_name_(native_middle_name_)
  , native_last_name_(native_last_name_)
  , birthdate_(std::move(birthdate_))
  , gender_(gender_)
  , country_code_(country_code_)
  , residence_country_code_(residence_country_code_)
{}

const std::int32_t personalDetails::ID;

object_ptr<personalDetails> personalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<personalDetails> res = make_object<personalDetails>();
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->middle_name_ = jni::fetch_string(env, p, res->middle_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  res->native_first_name_ = jni::fetch_string(env, p, res->native_first_name_fieldID);
  res->native_middle_name_ = jni::fetch_string(env, p, res->native_middle_name_fieldID);
  res->native_last_name_ = jni::fetch_string(env, p, res->native_last_name_fieldID);
  res->birthdate_ = jni::fetch_tl_object<date>(env, jni::fetch_object(env, p, res->birthdate_fieldID));
  res->gender_ = jni::fetch_string(env, p, res->gender_fieldID);
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  res->residence_country_code_ = jni::fetch_string(env, p, res->residence_country_code_fieldID);
  return res;
}

void personalDetails::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, first_name_); if (nextString) { env->SetObjectField(s, first_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, middle_name_); if (nextString) { env->SetObjectField(s, middle_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, last_name_); if (nextString) { env->SetObjectField(s, last_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, native_first_name_); if (nextString) { env->SetObjectField(s, native_first_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, native_middle_name_); if (nextString) { env->SetObjectField(s, native_middle_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, native_last_name_); if (nextString) { env->SetObjectField(s, native_last_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (birthdate_ != nullptr) { jobject next; birthdate_->store(env, next); if (next) { env->SetObjectField(s, birthdate_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, gender_); if (nextString) { env->SetObjectField(s, gender_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, country_code_); if (nextString) { env->SetObjectField(s, country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, residence_country_code_); if (nextString) { env->SetObjectField(s, residence_country_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void personalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PersonalDetails");
    s.store_field("firstName", first_name_);
    s.store_field("middleName", middle_name_);
    s.store_field("lastName", last_name_);
    s.store_field("nativeFirstName", native_first_name_);
    s.store_field("nativeMiddleName", native_middle_name_);
    s.store_field("nativeLastName", native_last_name_);
    s.store_object_field("birthdate", static_cast<const BaseObject *>(birthdate_.get()));
    s.store_field("gender", gender_);
    s.store_field("countryCode", country_code_);
    s.store_field("residenceCountryCode", residence_country_code_);
    s.store_class_end();
  }
}

void personalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PersonalDetails").c_str());
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    middle_name_fieldID = jni::get_field_id(env, Class, "middleName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    native_first_name_fieldID = jni::get_field_id(env, Class, "nativeFirstName", "Ljava/lang/String;");
    native_middle_name_fieldID = jni::get_field_id(env, Class, "nativeMiddleName", "Ljava/lang/String;");
    native_last_name_fieldID = jni::get_field_id(env, Class, "nativeLastName", "Ljava/lang/String;");
    birthdate_fieldID = jni::get_field_id(env, Class, "birthdate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Date;").c_str());
    gender_fieldID = jni::get_field_id(env, Class, "gender", "Ljava/lang/String;");
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    residence_country_code_fieldID = jni::get_field_id(env, Class, "residenceCountryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass personalDocument::Class;
jfieldID personalDocument::files_fieldID;
jfieldID personalDocument::translation_fieldID;

personalDocument::personalDocument()
  : files_()
  , translation_()
{}

personalDocument::personalDocument(array<object_ptr<datedFile>> &&files_, array<object_ptr<datedFile>> &&translation_)
  : files_(std::move(files_))
  , translation_(std::move(translation_))
{}

const std::int32_t personalDocument::ID;

object_ptr<personalDocument> personalDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<personalDocument> res = make_object<personalDocument>();
  res->files_ = jni::FetchVector<datedFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->files_fieldID));
  res->translation_ = jni::FetchVector<datedFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->translation_fieldID));
  return res;
}

void personalDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, files_); if (arr_tmp_) { env->SetObjectField(s, files_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, translation_); if (arr_tmp_) { env->SetObjectField(s, translation_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void personalDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PersonalDocument");
    { s.store_vector_begin("files", files_.size()); for (const auto &_value : files_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("translation", translation_.size()); for (const auto &_value : translation_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void personalDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PersonalDocument").c_str());
    files_fieldID = jni::get_field_id(env, Class, "files", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    translation_fieldID = jni::get_field_id(env, Class, "translation", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$DatedFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass phoneNumberAuthenticationSettings::Class;
jfieldID phoneNumberAuthenticationSettings::allow_flash_call_fieldID;
jfieldID phoneNumberAuthenticationSettings::allow_missed_call_fieldID;
jfieldID phoneNumberAuthenticationSettings::is_current_phone_number_fieldID;
jfieldID phoneNumberAuthenticationSettings::has_unknown_phone_number_fieldID;
jfieldID phoneNumberAuthenticationSettings::allow_sms_retriever_api_fieldID;
jfieldID phoneNumberAuthenticationSettings::firebase_authentication_settings_fieldID;
jfieldID phoneNumberAuthenticationSettings::authentication_tokens_fieldID;

phoneNumberAuthenticationSettings::phoneNumberAuthenticationSettings()
  : allow_flash_call_()
  , allow_missed_call_()
  , is_current_phone_number_()
  , has_unknown_phone_number_()
  , allow_sms_retriever_api_()
  , firebase_authentication_settings_()
  , authentication_tokens_()
{}

phoneNumberAuthenticationSettings::phoneNumberAuthenticationSettings(bool allow_flash_call_, bool allow_missed_call_, bool is_current_phone_number_, bool has_unknown_phone_number_, bool allow_sms_retriever_api_, object_ptr<FirebaseAuthenticationSettings> &&firebase_authentication_settings_, array<string> &&authentication_tokens_)
  : allow_flash_call_(allow_flash_call_)
  , allow_missed_call_(allow_missed_call_)
  , is_current_phone_number_(is_current_phone_number_)
  , has_unknown_phone_number_(has_unknown_phone_number_)
  , allow_sms_retriever_api_(allow_sms_retriever_api_)
  , firebase_authentication_settings_(std::move(firebase_authentication_settings_))
  , authentication_tokens_(std::move(authentication_tokens_))
{}

const std::int32_t phoneNumberAuthenticationSettings::ID;

object_ptr<phoneNumberAuthenticationSettings> phoneNumberAuthenticationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<phoneNumberAuthenticationSettings> res = make_object<phoneNumberAuthenticationSettings>();
  res->allow_flash_call_ = (env->GetBooleanField(p, res->allow_flash_call_fieldID) != 0);
  res->allow_missed_call_ = (env->GetBooleanField(p, res->allow_missed_call_fieldID) != 0);
  res->is_current_phone_number_ = (env->GetBooleanField(p, res->is_current_phone_number_fieldID) != 0);
  res->has_unknown_phone_number_ = (env->GetBooleanField(p, res->has_unknown_phone_number_fieldID) != 0);
  res->allow_sms_retriever_api_ = (env->GetBooleanField(p, res->allow_sms_retriever_api_fieldID) != 0);
  res->firebase_authentication_settings_ = jni::fetch_tl_object<FirebaseAuthenticationSettings>(env, jni::fetch_object(env, p, res->firebase_authentication_settings_fieldID));
  res->authentication_tokens_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->authentication_tokens_fieldID));
  return res;
}

void phoneNumberAuthenticationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhoneNumberAuthenticationSettings");
    s.store_field("allowFlashCall", allow_flash_call_);
    s.store_field("allowMissedCall", allow_missed_call_);
    s.store_field("isCurrentPhoneNumber", is_current_phone_number_);
    s.store_field("hasUnknownPhoneNumber", has_unknown_phone_number_);
    s.store_field("allowSmsRetrieverApi", allow_sms_retriever_api_);
    s.store_object_field("firebaseAuthenticationSettings", static_cast<const BaseObject *>(firebase_authentication_settings_.get()));
    { s.store_vector_begin("authenticationTokens", authentication_tokens_.size()); for (const auto &_value : authentication_tokens_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void phoneNumberAuthenticationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberAuthenticationSettings").c_str());
    allow_flash_call_fieldID = jni::get_field_id(env, Class, "allowFlashCall", "Z");
    allow_missed_call_fieldID = jni::get_field_id(env, Class, "allowMissedCall", "Z");
    is_current_phone_number_fieldID = jni::get_field_id(env, Class, "isCurrentPhoneNumber", "Z");
    has_unknown_phone_number_fieldID = jni::get_field_id(env, Class, "hasUnknownPhoneNumber", "Z");
    allow_sms_retriever_api_fieldID = jni::get_field_id(env, Class, "allowSmsRetrieverApi", "Z");
    firebase_authentication_settings_fieldID = jni::get_field_id(env, Class, "firebaseAuthenticationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FirebaseAuthenticationSettings;").c_str());
    authentication_tokens_fieldID = jni::get_field_id(env, Class, "authenticationTokens", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PhoneNumberCodeType::Class;

object_ptr<PhoneNumberCodeType> PhoneNumberCodeType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case phoneNumberCodeTypeChange::ID:
      return phoneNumberCodeTypeChange::fetch(env, p);
    case phoneNumberCodeTypeVerify::ID:
      return phoneNumberCodeTypeVerify::fetch(env, p);
    case phoneNumberCodeTypeConfirmOwnership::ID:
      return phoneNumberCodeTypeConfirmOwnership::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PhoneNumberCodeType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberCodeType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass phoneNumberCodeTypeChange::Class;

phoneNumberCodeTypeChange::phoneNumberCodeTypeChange() {
}

const std::int32_t phoneNumberCodeTypeChange::ID;

object_ptr<PhoneNumberCodeType> phoneNumberCodeTypeChange::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<phoneNumberCodeTypeChange>();
}

void phoneNumberCodeTypeChange::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhoneNumberCodeTypeChange");
    s.store_class_end();
  }
}

void phoneNumberCodeTypeChange::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberCodeTypeChange").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass phoneNumberCodeTypeVerify::Class;

phoneNumberCodeTypeVerify::phoneNumberCodeTypeVerify() {
}

const std::int32_t phoneNumberCodeTypeVerify::ID;

object_ptr<PhoneNumberCodeType> phoneNumberCodeTypeVerify::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<phoneNumberCodeTypeVerify>();
}

void phoneNumberCodeTypeVerify::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhoneNumberCodeTypeVerify");
    s.store_class_end();
  }
}

void phoneNumberCodeTypeVerify::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberCodeTypeVerify").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass phoneNumberCodeTypeConfirmOwnership::Class;
jfieldID phoneNumberCodeTypeConfirmOwnership::hash_fieldID;

phoneNumberCodeTypeConfirmOwnership::phoneNumberCodeTypeConfirmOwnership()
  : hash_()
{}

phoneNumberCodeTypeConfirmOwnership::phoneNumberCodeTypeConfirmOwnership(string const &hash_)
  : hash_(hash_)
{}

const std::int32_t phoneNumberCodeTypeConfirmOwnership::ID;

object_ptr<PhoneNumberCodeType> phoneNumberCodeTypeConfirmOwnership::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<phoneNumberCodeTypeConfirmOwnership> res = make_object<phoneNumberCodeTypeConfirmOwnership>();
  res->hash_ = jni::fetch_string(env, p, res->hash_fieldID);
  return std::move(res);
}

void phoneNumberCodeTypeConfirmOwnership::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhoneNumberCodeTypeConfirmOwnership");
    s.store_field("hash", hash_);
    s.store_class_end();
  }
}

void phoneNumberCodeTypeConfirmOwnership::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberCodeTypeConfirmOwnership").c_str());
    hash_fieldID = jni::get_field_id(env, Class, "hash", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass phoneNumberInfo::Class;
jfieldID phoneNumberInfo::country_fieldID;
jfieldID phoneNumberInfo::country_calling_code_fieldID;
jfieldID phoneNumberInfo::formatted_phone_number_fieldID;
jfieldID phoneNumberInfo::is_anonymous_fieldID;

phoneNumberInfo::phoneNumberInfo()
  : country_()
  , country_calling_code_()
  , formatted_phone_number_()
  , is_anonymous_()
{}

phoneNumberInfo::phoneNumberInfo(object_ptr<countryInfo> &&country_, string const &country_calling_code_, string const &formatted_phone_number_, bool is_anonymous_)
  : country_(std::move(country_))
  , country_calling_code_(country_calling_code_)
  , formatted_phone_number_(formatted_phone_number_)
  , is_anonymous_(is_anonymous_)
{}

const std::int32_t phoneNumberInfo::ID;

object_ptr<phoneNumberInfo> phoneNumberInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<phoneNumberInfo> res = make_object<phoneNumberInfo>();
  res->country_ = jni::fetch_tl_object<countryInfo>(env, jni::fetch_object(env, p, res->country_fieldID));
  res->country_calling_code_ = jni::fetch_string(env, p, res->country_calling_code_fieldID);
  res->formatted_phone_number_ = jni::fetch_string(env, p, res->formatted_phone_number_fieldID);
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  return res;
}

void phoneNumberInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (country_ != nullptr) { jobject next; country_->store(env, next); if (next) { env->SetObjectField(s, country_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, country_calling_code_); if (nextString) { env->SetObjectField(s, country_calling_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, formatted_phone_number_); if (nextString) { env->SetObjectField(s, formatted_phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
}

void phoneNumberInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhoneNumberInfo");
    s.store_object_field("country", static_cast<const BaseObject *>(country_.get()));
    s.store_field("countryCallingCode", country_calling_code_);
    s.store_field("formattedPhoneNumber", formatted_phone_number_);
    s.store_field("isAnonymous", is_anonymous_);
    s.store_class_end();
  }
}

void phoneNumberInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhoneNumberInfo").c_str());
    country_fieldID = jni::get_field_id(env, Class, "country", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CountryInfo;").c_str());
    country_calling_code_fieldID = jni::get_field_id(env, Class, "countryCallingCode", "Ljava/lang/String;");
    formatted_phone_number_fieldID = jni::get_field_id(env, Class, "formattedPhoneNumber", "Ljava/lang/String;");
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass photo::Class;
jfieldID photo::has_stickers_fieldID;
jfieldID photo::minithumbnail_fieldID;
jfieldID photo::sizes_fieldID;

photo::photo()
  : has_stickers_()
  , minithumbnail_()
  , sizes_()
{}

photo::photo(bool has_stickers_, object_ptr<minithumbnail> &&minithumbnail_, array<object_ptr<photoSize>> &&sizes_)
  : has_stickers_(has_stickers_)
  , minithumbnail_(std::move(minithumbnail_))
  , sizes_(std::move(sizes_))
{}

const std::int32_t photo::ID;

object_ptr<photo> photo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<photo> res = make_object<photo>();
  res->has_stickers_ = (env->GetBooleanField(p, res->has_stickers_fieldID) != 0);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->sizes_ = jni::FetchVector<photoSize>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->sizes_fieldID));
  return res;
}

void photo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_stickers_fieldID, has_stickers_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, sizes_); if (arr_tmp_) { env->SetObjectField(s, sizes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void photo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Photo");
    s.store_field("hasStickers", has_stickers_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    { s.store_vector_begin("sizes", sizes_.size()); for (const auto &_value : sizes_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void photo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Photo").c_str());
    has_stickers_fieldID = jni::get_field_id(env, Class, "hasStickers", "Z");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    sizes_fieldID = jni::get_field_id(env, Class, "sizes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PhotoSize;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass photoSize::Class;
jfieldID photoSize::type_fieldID;
jfieldID photoSize::photo_fieldID;
jfieldID photoSize::width_fieldID;
jfieldID photoSize::height_fieldID;
jfieldID photoSize::progressive_sizes_fieldID;

photoSize::photoSize()
  : type_()
  , photo_()
  , width_()
  , height_()
  , progressive_sizes_()
{}

photoSize::photoSize(string const &type_, object_ptr<file> &&photo_, int32 width_, int32 height_, array<int32> &&progressive_sizes_)
  : type_(type_)
  , photo_(std::move(photo_))
  , width_(width_)
  , height_(height_)
  , progressive_sizes_(std::move(progressive_sizes_))
{}

const std::int32_t photoSize::ID;

object_ptr<photoSize> photoSize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<photoSize> res = make_object<photoSize>();
  res->type_ = jni::fetch_string(env, p, res->type_fieldID);
  res->photo_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->progressive_sizes_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->progressive_sizes_fieldID));
  return res;
}

void photoSize::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, type_); if (nextString) { env->SetObjectField(s, type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { auto arr_tmp_ = jni::store_vector(env, progressive_sizes_); if (arr_tmp_) { env->SetObjectField(s, progressive_sizes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void photoSize::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PhotoSize");
    s.store_field("type", type_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    { s.store_vector_begin("progressiveSizes", progressive_sizes_.size()); for (const auto &_value : progressive_sizes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void photoSize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PhotoSize").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    progressive_sizes_fieldID = jni::get_field_id(env, Class, "progressiveSizes", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass point::Class;
jfieldID point::x_fieldID;
jfieldID point::y_fieldID;

point::point()
  : x_()
  , y_()
{}

point::point(double x_, double y_)
  : x_(x_)
  , y_(y_)
{}

const std::int32_t point::ID;

object_ptr<point> point::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<point> res = make_object<point>();
  res->x_ = env->GetDoubleField(p, res->x_fieldID);
  res->y_ = env->GetDoubleField(p, res->y_fieldID);
  return res;
}

void point::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, x_fieldID, x_);
  env->SetDoubleField(s, y_fieldID, y_);
}

void point::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Point");
    s.store_field("x", x_);
    s.store_field("y", y_);
    s.store_class_end();
  }
}

void point::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Point").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "D");
    y_fieldID = jni::get_field_id(env, Class, "y", "D");
    return true;
  }();
  (void)is_inited;
}

jclass poll::Class;
jfieldID poll::id_fieldID;
jfieldID poll::question_fieldID;
jfieldID poll::options_fieldID;
jfieldID poll::total_voter_count_fieldID;
jfieldID poll::recent_voter_ids_fieldID;
jfieldID poll::is_anonymous_fieldID;
jfieldID poll::type_fieldID;
jfieldID poll::open_period_fieldID;
jfieldID poll::close_date_fieldID;
jfieldID poll::is_closed_fieldID;

poll::poll()
  : id_()
  , question_()
  , options_()
  , total_voter_count_()
  , recent_voter_ids_()
  , is_anonymous_()
  , type_()
  , open_period_()
  , close_date_()
  , is_closed_()
{}

poll::poll(int64 id_, object_ptr<formattedText> &&question_, array<object_ptr<pollOption>> &&options_, int32 total_voter_count_, array<object_ptr<MessageSender>> &&recent_voter_ids_, bool is_anonymous_, object_ptr<PollType> &&type_, int32 open_period_, int32 close_date_, bool is_closed_)
  : id_(id_)
  , question_(std::move(question_))
  , options_(std::move(options_))
  , total_voter_count_(total_voter_count_)
  , recent_voter_ids_(std::move(recent_voter_ids_))
  , is_anonymous_(is_anonymous_)
  , type_(std::move(type_))
  , open_period_(open_period_)
  , close_date_(close_date_)
  , is_closed_(is_closed_)
{}

const std::int32_t poll::ID;

object_ptr<poll> poll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<poll> res = make_object<poll>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->question_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->question_fieldID));
  res->options_ = jni::FetchVector<pollOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  res->total_voter_count_ = env->GetIntField(p, res->total_voter_count_fieldID);
  res->recent_voter_ids_ = jni::FetchVector<MessageSender>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->recent_voter_ids_fieldID));
  res->is_anonymous_ = (env->GetBooleanField(p, res->is_anonymous_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<PollType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->open_period_ = env->GetIntField(p, res->open_period_fieldID);
  res->close_date_ = env->GetIntField(p, res->close_date_fieldID);
  res->is_closed_ = (env->GetBooleanField(p, res->is_closed_fieldID) != 0);
  return res;
}

void poll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (question_ != nullptr) { jobject next; question_->store(env, next); if (next) { env->SetObjectField(s, question_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, total_voter_count_fieldID, total_voter_count_);
  { auto arr_tmp_ = jni::store_vector(env, recent_voter_ids_); if (arr_tmp_) { env->SetObjectField(s, recent_voter_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_anonymous_fieldID, is_anonymous_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, open_period_fieldID, open_period_);
  env->SetIntField(s, close_date_fieldID, close_date_);
  env->SetBooleanField(s, is_closed_fieldID, is_closed_);
}

void poll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Poll");
    s.store_field("id", id_);
    s.store_object_field("question", static_cast<const BaseObject *>(question_.get()));
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("totalVoterCount", total_voter_count_);
    { s.store_vector_begin("recentVoterIds", recent_voter_ids_.size()); for (const auto &_value : recent_voter_ids_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isAnonymous", is_anonymous_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("openPeriod", open_period_);
    s.store_field("closeDate", close_date_);
    s.store_field("isClosed", is_closed_);
    s.store_class_end();
  }
}

void poll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Poll").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    question_fieldID = jni::get_field_id(env, Class, "question", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PollOption;").c_str());
    total_voter_count_fieldID = jni::get_field_id(env, Class, "totalVoterCount", "I");
    recent_voter_ids_fieldID = jni::get_field_id(env, Class, "recentVoterIds", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_anonymous_fieldID = jni::get_field_id(env, Class, "isAnonymous", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PollType;").c_str());
    open_period_fieldID = jni::get_field_id(env, Class, "openPeriod", "I");
    close_date_fieldID = jni::get_field_id(env, Class, "closeDate", "I");
    is_closed_fieldID = jni::get_field_id(env, Class, "isClosed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pollOption::Class;
jfieldID pollOption::text_fieldID;
jfieldID pollOption::voter_count_fieldID;
jfieldID pollOption::vote_percentage_fieldID;
jfieldID pollOption::is_chosen_fieldID;
jfieldID pollOption::is_being_chosen_fieldID;

pollOption::pollOption()
  : text_()
  , voter_count_()
  , vote_percentage_()
  , is_chosen_()
  , is_being_chosen_()
{}

pollOption::pollOption(object_ptr<formattedText> &&text_, int32 voter_count_, int32 vote_percentage_, bool is_chosen_, bool is_being_chosen_)
  : text_(std::move(text_))
  , voter_count_(voter_count_)
  , vote_percentage_(vote_percentage_)
  , is_chosen_(is_chosen_)
  , is_being_chosen_(is_being_chosen_)
{}

const std::int32_t pollOption::ID;

object_ptr<pollOption> pollOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pollOption> res = make_object<pollOption>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->voter_count_ = env->GetIntField(p, res->voter_count_fieldID);
  res->vote_percentage_ = env->GetIntField(p, res->vote_percentage_fieldID);
  res->is_chosen_ = (env->GetBooleanField(p, res->is_chosen_fieldID) != 0);
  res->is_being_chosen_ = (env->GetBooleanField(p, res->is_being_chosen_fieldID) != 0);
  return res;
}

void pollOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, voter_count_fieldID, voter_count_);
  env->SetIntField(s, vote_percentage_fieldID, vote_percentage_);
  env->SetBooleanField(s, is_chosen_fieldID, is_chosen_);
  env->SetBooleanField(s, is_being_chosen_fieldID, is_being_chosen_);
}

void pollOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PollOption");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("voterCount", voter_count_);
    s.store_field("votePercentage", vote_percentage_);
    s.store_field("isChosen", is_chosen_);
    s.store_field("isBeingChosen", is_being_chosen_);
    s.store_class_end();
  }
}

void pollOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PollOption").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    voter_count_fieldID = jni::get_field_id(env, Class, "voterCount", "I");
    vote_percentage_fieldID = jni::get_field_id(env, Class, "votePercentage", "I");
    is_chosen_fieldID = jni::get_field_id(env, Class, "isChosen", "Z");
    is_being_chosen_fieldID = jni::get_field_id(env, Class, "isBeingChosen", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass PollType::Class;

object_ptr<PollType> PollType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case pollTypeRegular::ID:
      return pollTypeRegular::fetch(env, p);
    case pollTypeQuiz::ID:
      return pollTypeQuiz::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PollType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PollType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pollTypeRegular::Class;
jfieldID pollTypeRegular::allow_multiple_answers_fieldID;

pollTypeRegular::pollTypeRegular()
  : allow_multiple_answers_()
{}

pollTypeRegular::pollTypeRegular(bool allow_multiple_answers_)
  : allow_multiple_answers_(allow_multiple_answers_)
{}

const std::int32_t pollTypeRegular::ID;

object_ptr<PollType> pollTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pollTypeRegular> res = make_object<pollTypeRegular>();
  res->allow_multiple_answers_ = (env->GetBooleanField(p, res->allow_multiple_answers_fieldID) != 0);
  return std::move(res);
}

void pollTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, allow_multiple_answers_fieldID, allow_multiple_answers_);
}

void pollTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PollTypeRegular");
    s.store_field("allowMultipleAnswers", allow_multiple_answers_);
    s.store_class_end();
  }
}

void pollTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PollTypeRegular").c_str());
    allow_multiple_answers_fieldID = jni::get_field_id(env, Class, "allowMultipleAnswers", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pollTypeQuiz::Class;
jfieldID pollTypeQuiz::correct_option_id_fieldID;
jfieldID pollTypeQuiz::explanation_fieldID;

pollTypeQuiz::pollTypeQuiz()
  : correct_option_id_()
  , explanation_()
{}

pollTypeQuiz::pollTypeQuiz(int32 correct_option_id_, object_ptr<formattedText> &&explanation_)
  : correct_option_id_(correct_option_id_)
  , explanation_(std::move(explanation_))
{}

const std::int32_t pollTypeQuiz::ID;

object_ptr<PollType> pollTypeQuiz::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pollTypeQuiz> res = make_object<pollTypeQuiz>();
  res->correct_option_id_ = env->GetIntField(p, res->correct_option_id_fieldID);
  res->explanation_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->explanation_fieldID));
  return std::move(res);
}

void pollTypeQuiz::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, correct_option_id_fieldID, correct_option_id_);
  if (explanation_ != nullptr) { jobject next; explanation_->store(env, next); if (next) { env->SetObjectField(s, explanation_fieldID, next); env->DeleteLocalRef(next); } }
}

void pollTypeQuiz::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PollTypeQuiz");
    s.store_field("correctOptionId", correct_option_id_);
    s.store_object_field("explanation", static_cast<const BaseObject *>(explanation_.get()));
    s.store_class_end();
  }
}

void pollTypeQuiz::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PollTypeQuiz").c_str());
    correct_option_id_fieldID = jni::get_field_id(env, Class, "correctOptionId", "I");
    explanation_fieldID = jni::get_field_id(env, Class, "explanation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PremiumFeature::Class;

object_ptr<PremiumFeature> PremiumFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case premiumFeatureIncreasedLimits::ID:
      return premiumFeatureIncreasedLimits::fetch(env, p);
    case premiumFeatureIncreasedUploadFileSize::ID:
      return premiumFeatureIncreasedUploadFileSize::fetch(env, p);
    case premiumFeatureImprovedDownloadSpeed::ID:
      return premiumFeatureImprovedDownloadSpeed::fetch(env, p);
    case premiumFeatureVoiceRecognition::ID:
      return premiumFeatureVoiceRecognition::fetch(env, p);
    case premiumFeatureDisabledAds::ID:
      return premiumFeatureDisabledAds::fetch(env, p);
    case premiumFeatureUniqueReactions::ID:
      return premiumFeatureUniqueReactions::fetch(env, p);
    case premiumFeatureUniqueStickers::ID:
      return premiumFeatureUniqueStickers::fetch(env, p);
    case premiumFeatureCustomEmoji::ID:
      return premiumFeatureCustomEmoji::fetch(env, p);
    case premiumFeatureAdvancedChatManagement::ID:
      return premiumFeatureAdvancedChatManagement::fetch(env, p);
    case premiumFeatureProfileBadge::ID:
      return premiumFeatureProfileBadge::fetch(env, p);
    case premiumFeatureEmojiStatus::ID:
      return premiumFeatureEmojiStatus::fetch(env, p);
    case premiumFeatureAnimatedProfilePhoto::ID:
      return premiumFeatureAnimatedProfilePhoto::fetch(env, p);
    case premiumFeatureForumTopicIcon::ID:
      return premiumFeatureForumTopicIcon::fetch(env, p);
    case premiumFeatureAppIcons::ID:
      return premiumFeatureAppIcons::fetch(env, p);
    case premiumFeatureRealTimeChatTranslation::ID:
      return premiumFeatureRealTimeChatTranslation::fetch(env, p);
    case premiumFeatureUpgradedStories::ID:
      return premiumFeatureUpgradedStories::fetch(env, p);
    case premiumFeatureChatBoost::ID:
      return premiumFeatureChatBoost::fetch(env, p);
    case premiumFeatureAccentColor::ID:
      return premiumFeatureAccentColor::fetch(env, p);
    case premiumFeatureBackgroundForBoth::ID:
      return premiumFeatureBackgroundForBoth::fetch(env, p);
    case premiumFeatureSavedMessagesTags::ID:
      return premiumFeatureSavedMessagesTags::fetch(env, p);
    case premiumFeatureMessagePrivacy::ID:
      return premiumFeatureMessagePrivacy::fetch(env, p);
    case premiumFeatureLastSeenTimes::ID:
      return premiumFeatureLastSeenTimes::fetch(env, p);
    case premiumFeatureBusiness::ID:
      return premiumFeatureBusiness::fetch(env, p);
    case premiumFeatureMessageEffects::ID:
      return premiumFeatureMessageEffects::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PremiumFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeature").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureIncreasedLimits::Class;

premiumFeatureIncreasedLimits::premiumFeatureIncreasedLimits() {
}

const std::int32_t premiumFeatureIncreasedLimits::ID;

object_ptr<PremiumFeature> premiumFeatureIncreasedLimits::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureIncreasedLimits>();
}

void premiumFeatureIncreasedLimits::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureIncreasedLimits::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureIncreasedLimits");
    s.store_class_end();
  }
}

void premiumFeatureIncreasedLimits::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureIncreasedLimits").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureIncreasedUploadFileSize::Class;

premiumFeatureIncreasedUploadFileSize::premiumFeatureIncreasedUploadFileSize() {
}

const std::int32_t premiumFeatureIncreasedUploadFileSize::ID;

object_ptr<PremiumFeature> premiumFeatureIncreasedUploadFileSize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureIncreasedUploadFileSize>();
}

void premiumFeatureIncreasedUploadFileSize::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureIncreasedUploadFileSize::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureIncreasedUploadFileSize");
    s.store_class_end();
  }
}

void premiumFeatureIncreasedUploadFileSize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureIncreasedUploadFileSize").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureImprovedDownloadSpeed::Class;

premiumFeatureImprovedDownloadSpeed::premiumFeatureImprovedDownloadSpeed() {
}

const std::int32_t premiumFeatureImprovedDownloadSpeed::ID;

object_ptr<PremiumFeature> premiumFeatureImprovedDownloadSpeed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureImprovedDownloadSpeed>();
}

void premiumFeatureImprovedDownloadSpeed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureImprovedDownloadSpeed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureImprovedDownloadSpeed");
    s.store_class_end();
  }
}

void premiumFeatureImprovedDownloadSpeed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureImprovedDownloadSpeed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureVoiceRecognition::Class;

premiumFeatureVoiceRecognition::premiumFeatureVoiceRecognition() {
}

const std::int32_t premiumFeatureVoiceRecognition::ID;

object_ptr<PremiumFeature> premiumFeatureVoiceRecognition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureVoiceRecognition>();
}

void premiumFeatureVoiceRecognition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureVoiceRecognition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureVoiceRecognition");
    s.store_class_end();
  }
}

void premiumFeatureVoiceRecognition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureVoiceRecognition").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureDisabledAds::Class;

premiumFeatureDisabledAds::premiumFeatureDisabledAds() {
}

const std::int32_t premiumFeatureDisabledAds::ID;

object_ptr<PremiumFeature> premiumFeatureDisabledAds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureDisabledAds>();
}

void premiumFeatureDisabledAds::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureDisabledAds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureDisabledAds");
    s.store_class_end();
  }
}

void premiumFeatureDisabledAds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureDisabledAds").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureUniqueReactions::Class;

premiumFeatureUniqueReactions::premiumFeatureUniqueReactions() {
}

const std::int32_t premiumFeatureUniqueReactions::ID;

object_ptr<PremiumFeature> premiumFeatureUniqueReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureUniqueReactions>();
}

void premiumFeatureUniqueReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureUniqueReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureUniqueReactions");
    s.store_class_end();
  }
}

void premiumFeatureUniqueReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureUniqueReactions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureUniqueStickers::Class;

premiumFeatureUniqueStickers::premiumFeatureUniqueStickers() {
}

const std::int32_t premiumFeatureUniqueStickers::ID;

object_ptr<PremiumFeature> premiumFeatureUniqueStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureUniqueStickers>();
}

void premiumFeatureUniqueStickers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureUniqueStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureUniqueStickers");
    s.store_class_end();
  }
}

void premiumFeatureUniqueStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureUniqueStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureCustomEmoji::Class;

premiumFeatureCustomEmoji::premiumFeatureCustomEmoji() {
}

const std::int32_t premiumFeatureCustomEmoji::ID;

object_ptr<PremiumFeature> premiumFeatureCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureCustomEmoji>();
}

void premiumFeatureCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureCustomEmoji");
    s.store_class_end();
  }
}

void premiumFeatureCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureCustomEmoji").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureAdvancedChatManagement::Class;

premiumFeatureAdvancedChatManagement::premiumFeatureAdvancedChatManagement() {
}

const std::int32_t premiumFeatureAdvancedChatManagement::ID;

object_ptr<PremiumFeature> premiumFeatureAdvancedChatManagement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureAdvancedChatManagement>();
}

void premiumFeatureAdvancedChatManagement::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureAdvancedChatManagement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureAdvancedChatManagement");
    s.store_class_end();
  }
}

void premiumFeatureAdvancedChatManagement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureAdvancedChatManagement").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureProfileBadge::Class;

premiumFeatureProfileBadge::premiumFeatureProfileBadge() {
}

const std::int32_t premiumFeatureProfileBadge::ID;

object_ptr<PremiumFeature> premiumFeatureProfileBadge::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureProfileBadge>();
}

void premiumFeatureProfileBadge::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureProfileBadge::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureProfileBadge");
    s.store_class_end();
  }
}

void premiumFeatureProfileBadge::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureProfileBadge").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureEmojiStatus::Class;

premiumFeatureEmojiStatus::premiumFeatureEmojiStatus() {
}

const std::int32_t premiumFeatureEmojiStatus::ID;

object_ptr<PremiumFeature> premiumFeatureEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureEmojiStatus>();
}

void premiumFeatureEmojiStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureEmojiStatus");
    s.store_class_end();
  }
}

void premiumFeatureEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureEmojiStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureAnimatedProfilePhoto::Class;

premiumFeatureAnimatedProfilePhoto::premiumFeatureAnimatedProfilePhoto() {
}

const std::int32_t premiumFeatureAnimatedProfilePhoto::ID;

object_ptr<PremiumFeature> premiumFeatureAnimatedProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureAnimatedProfilePhoto>();
}

void premiumFeatureAnimatedProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureAnimatedProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureAnimatedProfilePhoto");
    s.store_class_end();
  }
}

void premiumFeatureAnimatedProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureAnimatedProfilePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureForumTopicIcon::Class;

premiumFeatureForumTopicIcon::premiumFeatureForumTopicIcon() {
}

const std::int32_t premiumFeatureForumTopicIcon::ID;

object_ptr<PremiumFeature> premiumFeatureForumTopicIcon::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureForumTopicIcon>();
}

void premiumFeatureForumTopicIcon::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureForumTopicIcon::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureForumTopicIcon");
    s.store_class_end();
  }
}

void premiumFeatureForumTopicIcon::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureForumTopicIcon").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureAppIcons::Class;

premiumFeatureAppIcons::premiumFeatureAppIcons() {
}

const std::int32_t premiumFeatureAppIcons::ID;

object_ptr<PremiumFeature> premiumFeatureAppIcons::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureAppIcons>();
}

void premiumFeatureAppIcons::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureAppIcons::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureAppIcons");
    s.store_class_end();
  }
}

void premiumFeatureAppIcons::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureAppIcons").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureRealTimeChatTranslation::Class;

premiumFeatureRealTimeChatTranslation::premiumFeatureRealTimeChatTranslation() {
}

const std::int32_t premiumFeatureRealTimeChatTranslation::ID;

object_ptr<PremiumFeature> premiumFeatureRealTimeChatTranslation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureRealTimeChatTranslation>();
}

void premiumFeatureRealTimeChatTranslation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureRealTimeChatTranslation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureRealTimeChatTranslation");
    s.store_class_end();
  }
}

void premiumFeatureRealTimeChatTranslation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureRealTimeChatTranslation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureUpgradedStories::Class;

premiumFeatureUpgradedStories::premiumFeatureUpgradedStories() {
}

const std::int32_t premiumFeatureUpgradedStories::ID;

object_ptr<PremiumFeature> premiumFeatureUpgradedStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureUpgradedStories>();
}

void premiumFeatureUpgradedStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureUpgradedStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureUpgradedStories");
    s.store_class_end();
  }
}

void premiumFeatureUpgradedStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureUpgradedStories").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureChatBoost::Class;

premiumFeatureChatBoost::premiumFeatureChatBoost() {
}

const std::int32_t premiumFeatureChatBoost::ID;

object_ptr<PremiumFeature> premiumFeatureChatBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureChatBoost>();
}

void premiumFeatureChatBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureChatBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureChatBoost");
    s.store_class_end();
  }
}

void premiumFeatureChatBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureChatBoost").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureAccentColor::Class;

premiumFeatureAccentColor::premiumFeatureAccentColor() {
}

const std::int32_t premiumFeatureAccentColor::ID;

object_ptr<PremiumFeature> premiumFeatureAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureAccentColor>();
}

void premiumFeatureAccentColor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureAccentColor");
    s.store_class_end();
  }
}

void premiumFeatureAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureAccentColor").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureBackgroundForBoth::Class;

premiumFeatureBackgroundForBoth::premiumFeatureBackgroundForBoth() {
}

const std::int32_t premiumFeatureBackgroundForBoth::ID;

object_ptr<PremiumFeature> premiumFeatureBackgroundForBoth::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureBackgroundForBoth>();
}

void premiumFeatureBackgroundForBoth::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureBackgroundForBoth::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureBackgroundForBoth");
    s.store_class_end();
  }
}

void premiumFeatureBackgroundForBoth::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureBackgroundForBoth").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureSavedMessagesTags::Class;

premiumFeatureSavedMessagesTags::premiumFeatureSavedMessagesTags() {
}

const std::int32_t premiumFeatureSavedMessagesTags::ID;

object_ptr<PremiumFeature> premiumFeatureSavedMessagesTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureSavedMessagesTags>();
}

void premiumFeatureSavedMessagesTags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureSavedMessagesTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureSavedMessagesTags");
    s.store_class_end();
  }
}

void premiumFeatureSavedMessagesTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureSavedMessagesTags").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureMessagePrivacy::Class;

premiumFeatureMessagePrivacy::premiumFeatureMessagePrivacy() {
}

const std::int32_t premiumFeatureMessagePrivacy::ID;

object_ptr<PremiumFeature> premiumFeatureMessagePrivacy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureMessagePrivacy>();
}

void premiumFeatureMessagePrivacy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureMessagePrivacy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureMessagePrivacy");
    s.store_class_end();
  }
}

void premiumFeatureMessagePrivacy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureMessagePrivacy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureLastSeenTimes::Class;

premiumFeatureLastSeenTimes::premiumFeatureLastSeenTimes() {
}

const std::int32_t premiumFeatureLastSeenTimes::ID;

object_ptr<PremiumFeature> premiumFeatureLastSeenTimes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureLastSeenTimes>();
}

void premiumFeatureLastSeenTimes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureLastSeenTimes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureLastSeenTimes");
    s.store_class_end();
  }
}

void premiumFeatureLastSeenTimes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureLastSeenTimes").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureBusiness::Class;

premiumFeatureBusiness::premiumFeatureBusiness() {
}

const std::int32_t premiumFeatureBusiness::ID;

object_ptr<PremiumFeature> premiumFeatureBusiness::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureBusiness>();
}

void premiumFeatureBusiness::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureBusiness::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureBusiness");
    s.store_class_end();
  }
}

void premiumFeatureBusiness::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureBusiness").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatureMessageEffects::Class;

premiumFeatureMessageEffects::premiumFeatureMessageEffects() {
}

const std::int32_t premiumFeatureMessageEffects::ID;

object_ptr<PremiumFeature> premiumFeatureMessageEffects::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumFeatureMessageEffects>();
}

void premiumFeatureMessageEffects::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumFeatureMessageEffects::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatureMessageEffects");
    s.store_class_end();
  }
}

void premiumFeatureMessageEffects::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatureMessageEffects").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeaturePromotionAnimation::Class;
jfieldID premiumFeaturePromotionAnimation::feature_fieldID;
jfieldID premiumFeaturePromotionAnimation::animation_fieldID;

premiumFeaturePromotionAnimation::premiumFeaturePromotionAnimation()
  : feature_()
  , animation_()
{}

premiumFeaturePromotionAnimation::premiumFeaturePromotionAnimation(object_ptr<PremiumFeature> &&feature_, object_ptr<animation> &&animation_)
  : feature_(std::move(feature_))
  , animation_(std::move(animation_))
{}

const std::int32_t premiumFeaturePromotionAnimation::ID;

object_ptr<premiumFeaturePromotionAnimation> premiumFeaturePromotionAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumFeaturePromotionAnimation> res = make_object<premiumFeaturePromotionAnimation>();
  res->feature_ = jni::fetch_tl_object<PremiumFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void premiumFeaturePromotionAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (feature_ != nullptr) { jobject next; feature_->store(env, next); if (next) { env->SetObjectField(s, feature_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void premiumFeaturePromotionAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeaturePromotionAnimation");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void premiumFeaturePromotionAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeaturePromotionAnimation").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumFeature;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumFeatures::Class;
jfieldID premiumFeatures::features_fieldID;
jfieldID premiumFeatures::limits_fieldID;
jfieldID premiumFeatures::payment_link_fieldID;

premiumFeatures::premiumFeatures()
  : features_()
  , limits_()
  , payment_link_()
{}

premiumFeatures::premiumFeatures(array<object_ptr<PremiumFeature>> &&features_, array<object_ptr<premiumLimit>> &&limits_, object_ptr<InternalLinkType> &&payment_link_)
  : features_(std::move(features_))
  , limits_(std::move(limits_))
  , payment_link_(std::move(payment_link_))
{}

const std::int32_t premiumFeatures::ID;

object_ptr<premiumFeatures> premiumFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumFeatures> res = make_object<premiumFeatures>();
  res->features_ = jni::FetchVector<PremiumFeature>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->features_fieldID));
  res->limits_ = jni::FetchVector<premiumLimit>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->limits_fieldID));
  res->payment_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->payment_link_fieldID));
  return res;
}

void premiumFeatures::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, features_); if (arr_tmp_) { env->SetObjectField(s, features_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, limits_); if (arr_tmp_) { env->SetObjectField(s, limits_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (payment_link_ != nullptr) { jobject next; payment_link_->store(env, next); if (next) { env->SetObjectField(s, payment_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void premiumFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumFeatures");
    { s.store_vector_begin("features", features_.size()); for (const auto &_value : features_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("limits", limits_.size()); for (const auto &_value : limits_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("paymentLink", static_cast<const BaseObject *>(payment_link_.get()));
    s.store_class_end();
  }
}

void premiumFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumFeatures").c_str());
    features_fieldID = jni::get_field_id(env, Class, "features", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumFeature;").c_str());
    limits_fieldID = jni::get_field_id(env, Class, "limits", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumLimit;").c_str());
    payment_link_fieldID = jni::get_field_id(env, Class, "paymentLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumGiftCodeInfo::Class;
jfieldID premiumGiftCodeInfo::creator_id_fieldID;
jfieldID premiumGiftCodeInfo::creation_date_fieldID;
jfieldID premiumGiftCodeInfo::is_from_giveaway_fieldID;
jfieldID premiumGiftCodeInfo::giveaway_message_id_fieldID;
jfieldID premiumGiftCodeInfo::month_count_fieldID;
jfieldID premiumGiftCodeInfo::user_id_fieldID;
jfieldID premiumGiftCodeInfo::use_date_fieldID;

premiumGiftCodeInfo::premiumGiftCodeInfo()
  : creator_id_()
  , creation_date_()
  , is_from_giveaway_()
  , giveaway_message_id_()
  , month_count_()
  , user_id_()
  , use_date_()
{}

premiumGiftCodeInfo::premiumGiftCodeInfo(object_ptr<MessageSender> &&creator_id_, int32 creation_date_, bool is_from_giveaway_, int53 giveaway_message_id_, int32 month_count_, int53 user_id_, int32 use_date_)
  : creator_id_(std::move(creator_id_))
  , creation_date_(creation_date_)
  , is_from_giveaway_(is_from_giveaway_)
  , giveaway_message_id_(giveaway_message_id_)
  , month_count_(month_count_)
  , user_id_(user_id_)
  , use_date_(use_date_)
{}

const std::int32_t premiumGiftCodeInfo::ID;

object_ptr<premiumGiftCodeInfo> premiumGiftCodeInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumGiftCodeInfo> res = make_object<premiumGiftCodeInfo>();
  res->creator_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->creator_id_fieldID));
  res->creation_date_ = env->GetIntField(p, res->creation_date_fieldID);
  res->is_from_giveaway_ = (env->GetBooleanField(p, res->is_from_giveaway_fieldID) != 0);
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->use_date_ = env->GetIntField(p, res->use_date_fieldID);
  return res;
}

void premiumGiftCodeInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (creator_id_ != nullptr) { jobject next; creator_id_->store(env, next); if (next) { env->SetObjectField(s, creator_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, creation_date_fieldID, creation_date_);
  env->SetBooleanField(s, is_from_giveaway_fieldID, is_from_giveaway_);
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, use_date_fieldID, use_date_);
}

void premiumGiftCodeInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumGiftCodeInfo");
    s.store_object_field("creatorId", static_cast<const BaseObject *>(creator_id_.get()));
    s.store_field("creationDate", creation_date_);
    s.store_field("isFromGiveaway", is_from_giveaway_);
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_field("monthCount", month_count_);
    s.store_field("userId", user_id_);
    s.store_field("useDate", use_date_);
    s.store_class_end();
  }
}

void premiumGiftCodeInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumGiftCodeInfo").c_str());
    creator_id_fieldID = jni::get_field_id(env, Class, "creatorId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    creation_date_fieldID = jni::get_field_id(env, Class, "creationDate", "I");
    is_from_giveaway_fieldID = jni::get_field_id(env, Class, "isFromGiveaway", "Z");
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    use_date_fieldID = jni::get_field_id(env, Class, "useDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass premiumGiftPaymentOption::Class;
jfieldID premiumGiftPaymentOption::currency_fieldID;
jfieldID premiumGiftPaymentOption::amount_fieldID;
jfieldID premiumGiftPaymentOption::star_count_fieldID;
jfieldID premiumGiftPaymentOption::discount_percentage_fieldID;
jfieldID premiumGiftPaymentOption::month_count_fieldID;
jfieldID premiumGiftPaymentOption::store_product_id_fieldID;
jfieldID premiumGiftPaymentOption::sticker_fieldID;

premiumGiftPaymentOption::premiumGiftPaymentOption()
  : currency_()
  , amount_()
  , star_count_()
  , discount_percentage_()
  , month_count_()
  , store_product_id_()
  , sticker_()
{}

premiumGiftPaymentOption::premiumGiftPaymentOption(string const &currency_, int53 amount_, int53 star_count_, int32 discount_percentage_, int32 month_count_, string const &store_product_id_, object_ptr<sticker> &&sticker_)
  : currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
  , discount_percentage_(discount_percentage_)
  , month_count_(month_count_)
  , store_product_id_(store_product_id_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t premiumGiftPaymentOption::ID;

object_ptr<premiumGiftPaymentOption> premiumGiftPaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumGiftPaymentOption> res = make_object<premiumGiftPaymentOption>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->discount_percentage_ = env->GetIntField(p, res->discount_percentage_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void premiumGiftPaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  env->SetLongField(s, star_count_fieldID, star_count_);
  env->SetIntField(s, discount_percentage_fieldID, discount_percentage_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void premiumGiftPaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumGiftPaymentOption");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_field("discountPercentage", discount_percentage_);
    s.store_field("monthCount", month_count_);
    s.store_field("storeProductId", store_product_id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void premiumGiftPaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumGiftPaymentOption").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    discount_percentage_fieldID = jni::get_field_id(env, Class, "discountPercentage", "I");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumGiftPaymentOptions::Class;
jfieldID premiumGiftPaymentOptions::options_fieldID;

premiumGiftPaymentOptions::premiumGiftPaymentOptions()
  : options_()
{}

premiumGiftPaymentOptions::premiumGiftPaymentOptions(array<object_ptr<premiumGiftPaymentOption>> &&options_)
  : options_(std::move(options_))
{}

const std::int32_t premiumGiftPaymentOptions::ID;

object_ptr<premiumGiftPaymentOptions> premiumGiftPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumGiftPaymentOptions> res = make_object<premiumGiftPaymentOptions>();
  res->options_ = jni::FetchVector<premiumGiftPaymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return res;
}

void premiumGiftPaymentOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void premiumGiftPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumGiftPaymentOptions");
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void premiumGiftPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumGiftPaymentOptions").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumGiftPaymentOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumGiveawayPaymentOption::Class;
jfieldID premiumGiveawayPaymentOption::currency_fieldID;
jfieldID premiumGiveawayPaymentOption::amount_fieldID;
jfieldID premiumGiveawayPaymentOption::winner_count_fieldID;
jfieldID premiumGiveawayPaymentOption::month_count_fieldID;
jfieldID premiumGiveawayPaymentOption::store_product_id_fieldID;
jfieldID premiumGiveawayPaymentOption::store_product_quantity_fieldID;

premiumGiveawayPaymentOption::premiumGiveawayPaymentOption()
  : currency_()
  , amount_()
  , winner_count_()
  , month_count_()
  , store_product_id_()
  , store_product_quantity_()
{}

premiumGiveawayPaymentOption::premiumGiveawayPaymentOption(string const &currency_, int53 amount_, int32 winner_count_, int32 month_count_, string const &store_product_id_, int32 store_product_quantity_)
  : currency_(currency_)
  , amount_(amount_)
  , winner_count_(winner_count_)
  , month_count_(month_count_)
  , store_product_id_(store_product_id_)
  , store_product_quantity_(store_product_quantity_)
{}

const std::int32_t premiumGiveawayPaymentOption::ID;

object_ptr<premiumGiveawayPaymentOption> premiumGiveawayPaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumGiveawayPaymentOption> res = make_object<premiumGiveawayPaymentOption>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->store_product_quantity_ = env->GetIntField(p, res->store_product_quantity_fieldID);
  return res;
}

void premiumGiveawayPaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, store_product_quantity_fieldID, store_product_quantity_);
}

void premiumGiveawayPaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumGiveawayPaymentOption");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("monthCount", month_count_);
    s.store_field("storeProductId", store_product_id_);
    s.store_field("storeProductQuantity", store_product_quantity_);
    s.store_class_end();
  }
}

void premiumGiveawayPaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumGiveawayPaymentOption").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    store_product_quantity_fieldID = jni::get_field_id(env, Class, "storeProductQuantity", "I");
    return true;
  }();
  (void)is_inited;
}

jclass premiumGiveawayPaymentOptions::Class;
jfieldID premiumGiveawayPaymentOptions::options_fieldID;

premiumGiveawayPaymentOptions::premiumGiveawayPaymentOptions()
  : options_()
{}

premiumGiveawayPaymentOptions::premiumGiveawayPaymentOptions(array<object_ptr<premiumGiveawayPaymentOption>> &&options_)
  : options_(std::move(options_))
{}

const std::int32_t premiumGiveawayPaymentOptions::ID;

object_ptr<premiumGiveawayPaymentOptions> premiumGiveawayPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumGiveawayPaymentOptions> res = make_object<premiumGiveawayPaymentOptions>();
  res->options_ = jni::FetchVector<premiumGiveawayPaymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return res;
}

void premiumGiveawayPaymentOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void premiumGiveawayPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumGiveawayPaymentOptions");
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void premiumGiveawayPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumGiveawayPaymentOptions").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumGiveawayPaymentOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimit::Class;
jfieldID premiumLimit::type_fieldID;
jfieldID premiumLimit::default_value_fieldID;
jfieldID premiumLimit::premium_value_fieldID;

premiumLimit::premiumLimit()
  : type_()
  , default_value_()
  , premium_value_()
{}

premiumLimit::premiumLimit(object_ptr<PremiumLimitType> &&type_, int32 default_value_, int32 premium_value_)
  : type_(std::move(type_))
  , default_value_(default_value_)
  , premium_value_(premium_value_)
{}

const std::int32_t premiumLimit::ID;

object_ptr<premiumLimit> premiumLimit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumLimit> res = make_object<premiumLimit>();
  res->type_ = jni::fetch_tl_object<PremiumLimitType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->default_value_ = env->GetIntField(p, res->default_value_fieldID);
  res->premium_value_ = env->GetIntField(p, res->premium_value_fieldID);
  return res;
}

void premiumLimit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, default_value_fieldID, default_value_);
  env->SetIntField(s, premium_value_fieldID, premium_value_);
}

void premiumLimit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimit");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("defaultValue", default_value_);
    s.store_field("premiumValue", premium_value_);
    s.store_class_end();
  }
}

void premiumLimit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimit").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumLimitType;").c_str());
    default_value_fieldID = jni::get_field_id(env, Class, "defaultValue", "I");
    premium_value_fieldID = jni::get_field_id(env, Class, "premiumValue", "I");
    return true;
  }();
  (void)is_inited;
}

jclass PremiumLimitType::Class;

object_ptr<PremiumLimitType> PremiumLimitType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case premiumLimitTypeSupergroupCount::ID:
      return premiumLimitTypeSupergroupCount::fetch(env, p);
    case premiumLimitTypePinnedChatCount::ID:
      return premiumLimitTypePinnedChatCount::fetch(env, p);
    case premiumLimitTypeCreatedPublicChatCount::ID:
      return premiumLimitTypeCreatedPublicChatCount::fetch(env, p);
    case premiumLimitTypeSavedAnimationCount::ID:
      return premiumLimitTypeSavedAnimationCount::fetch(env, p);
    case premiumLimitTypeFavoriteStickerCount::ID:
      return premiumLimitTypeFavoriteStickerCount::fetch(env, p);
    case premiumLimitTypeChatFolderCount::ID:
      return premiumLimitTypeChatFolderCount::fetch(env, p);
    case premiumLimitTypeChatFolderChosenChatCount::ID:
      return premiumLimitTypeChatFolderChosenChatCount::fetch(env, p);
    case premiumLimitTypePinnedArchivedChatCount::ID:
      return premiumLimitTypePinnedArchivedChatCount::fetch(env, p);
    case premiumLimitTypePinnedSavedMessagesTopicCount::ID:
      return premiumLimitTypePinnedSavedMessagesTopicCount::fetch(env, p);
    case premiumLimitTypeCaptionLength::ID:
      return premiumLimitTypeCaptionLength::fetch(env, p);
    case premiumLimitTypeBioLength::ID:
      return premiumLimitTypeBioLength::fetch(env, p);
    case premiumLimitTypeChatFolderInviteLinkCount::ID:
      return premiumLimitTypeChatFolderInviteLinkCount::fetch(env, p);
    case premiumLimitTypeShareableChatFolderCount::ID:
      return premiumLimitTypeShareableChatFolderCount::fetch(env, p);
    case premiumLimitTypeActiveStoryCount::ID:
      return premiumLimitTypeActiveStoryCount::fetch(env, p);
    case premiumLimitTypeWeeklyPostedStoryCount::ID:
      return premiumLimitTypeWeeklyPostedStoryCount::fetch(env, p);
    case premiumLimitTypeMonthlyPostedStoryCount::ID:
      return premiumLimitTypeMonthlyPostedStoryCount::fetch(env, p);
    case premiumLimitTypeStoryCaptionLength::ID:
      return premiumLimitTypeStoryCaptionLength::fetch(env, p);
    case premiumLimitTypeStorySuggestedReactionAreaCount::ID:
      return premiumLimitTypeStorySuggestedReactionAreaCount::fetch(env, p);
    case premiumLimitTypeSimilarChatCount::ID:
      return premiumLimitTypeSimilarChatCount::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PremiumLimitType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeSupergroupCount::Class;

premiumLimitTypeSupergroupCount::premiumLimitTypeSupergroupCount() {
}

const std::int32_t premiumLimitTypeSupergroupCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeSupergroupCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeSupergroupCount>();
}

void premiumLimitTypeSupergroupCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeSupergroupCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeSupergroupCount");
    s.store_class_end();
  }
}

void premiumLimitTypeSupergroupCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeSupergroupCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypePinnedChatCount::Class;

premiumLimitTypePinnedChatCount::premiumLimitTypePinnedChatCount() {
}

const std::int32_t premiumLimitTypePinnedChatCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypePinnedChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypePinnedChatCount>();
}

void premiumLimitTypePinnedChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypePinnedChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypePinnedChatCount");
    s.store_class_end();
  }
}

void premiumLimitTypePinnedChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypePinnedChatCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeCreatedPublicChatCount::Class;

premiumLimitTypeCreatedPublicChatCount::premiumLimitTypeCreatedPublicChatCount() {
}

const std::int32_t premiumLimitTypeCreatedPublicChatCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeCreatedPublicChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeCreatedPublicChatCount>();
}

void premiumLimitTypeCreatedPublicChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeCreatedPublicChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeCreatedPublicChatCount");
    s.store_class_end();
  }
}

void premiumLimitTypeCreatedPublicChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeCreatedPublicChatCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeSavedAnimationCount::Class;

premiumLimitTypeSavedAnimationCount::premiumLimitTypeSavedAnimationCount() {
}

const std::int32_t premiumLimitTypeSavedAnimationCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeSavedAnimationCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeSavedAnimationCount>();
}

void premiumLimitTypeSavedAnimationCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeSavedAnimationCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeSavedAnimationCount");
    s.store_class_end();
  }
}

void premiumLimitTypeSavedAnimationCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeSavedAnimationCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeFavoriteStickerCount::Class;

premiumLimitTypeFavoriteStickerCount::premiumLimitTypeFavoriteStickerCount() {
}

const std::int32_t premiumLimitTypeFavoriteStickerCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeFavoriteStickerCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeFavoriteStickerCount>();
}

void premiumLimitTypeFavoriteStickerCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeFavoriteStickerCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeFavoriteStickerCount");
    s.store_class_end();
  }
}

void premiumLimitTypeFavoriteStickerCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeFavoriteStickerCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeChatFolderCount::Class;

premiumLimitTypeChatFolderCount::premiumLimitTypeChatFolderCount() {
}

const std::int32_t premiumLimitTypeChatFolderCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeChatFolderCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeChatFolderCount>();
}

void premiumLimitTypeChatFolderCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeChatFolderCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeChatFolderCount");
    s.store_class_end();
  }
}

void premiumLimitTypeChatFolderCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeChatFolderCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeChatFolderChosenChatCount::Class;

premiumLimitTypeChatFolderChosenChatCount::premiumLimitTypeChatFolderChosenChatCount() {
}

const std::int32_t premiumLimitTypeChatFolderChosenChatCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeChatFolderChosenChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeChatFolderChosenChatCount>();
}

void premiumLimitTypeChatFolderChosenChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeChatFolderChosenChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeChatFolderChosenChatCount");
    s.store_class_end();
  }
}

void premiumLimitTypeChatFolderChosenChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeChatFolderChosenChatCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypePinnedArchivedChatCount::Class;

premiumLimitTypePinnedArchivedChatCount::premiumLimitTypePinnedArchivedChatCount() {
}

const std::int32_t premiumLimitTypePinnedArchivedChatCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypePinnedArchivedChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypePinnedArchivedChatCount>();
}

void premiumLimitTypePinnedArchivedChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypePinnedArchivedChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypePinnedArchivedChatCount");
    s.store_class_end();
  }
}

void premiumLimitTypePinnedArchivedChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypePinnedArchivedChatCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypePinnedSavedMessagesTopicCount::Class;

premiumLimitTypePinnedSavedMessagesTopicCount::premiumLimitTypePinnedSavedMessagesTopicCount() {
}

const std::int32_t premiumLimitTypePinnedSavedMessagesTopicCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypePinnedSavedMessagesTopicCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypePinnedSavedMessagesTopicCount>();
}

void premiumLimitTypePinnedSavedMessagesTopicCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypePinnedSavedMessagesTopicCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypePinnedSavedMessagesTopicCount");
    s.store_class_end();
  }
}

void premiumLimitTypePinnedSavedMessagesTopicCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypePinnedSavedMessagesTopicCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeCaptionLength::Class;

premiumLimitTypeCaptionLength::premiumLimitTypeCaptionLength() {
}

const std::int32_t premiumLimitTypeCaptionLength::ID;

object_ptr<PremiumLimitType> premiumLimitTypeCaptionLength::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeCaptionLength>();
}

void premiumLimitTypeCaptionLength::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeCaptionLength::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeCaptionLength");
    s.store_class_end();
  }
}

void premiumLimitTypeCaptionLength::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeCaptionLength").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeBioLength::Class;

premiumLimitTypeBioLength::premiumLimitTypeBioLength() {
}

const std::int32_t premiumLimitTypeBioLength::ID;

object_ptr<PremiumLimitType> premiumLimitTypeBioLength::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeBioLength>();
}

void premiumLimitTypeBioLength::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeBioLength::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeBioLength");
    s.store_class_end();
  }
}

void premiumLimitTypeBioLength::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeBioLength").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeChatFolderInviteLinkCount::Class;

premiumLimitTypeChatFolderInviteLinkCount::premiumLimitTypeChatFolderInviteLinkCount() {
}

const std::int32_t premiumLimitTypeChatFolderInviteLinkCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeChatFolderInviteLinkCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeChatFolderInviteLinkCount>();
}

void premiumLimitTypeChatFolderInviteLinkCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeChatFolderInviteLinkCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeChatFolderInviteLinkCount");
    s.store_class_end();
  }
}

void premiumLimitTypeChatFolderInviteLinkCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeChatFolderInviteLinkCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeShareableChatFolderCount::Class;

premiumLimitTypeShareableChatFolderCount::premiumLimitTypeShareableChatFolderCount() {
}

const std::int32_t premiumLimitTypeShareableChatFolderCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeShareableChatFolderCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeShareableChatFolderCount>();
}

void premiumLimitTypeShareableChatFolderCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeShareableChatFolderCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeShareableChatFolderCount");
    s.store_class_end();
  }
}

void premiumLimitTypeShareableChatFolderCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeShareableChatFolderCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeActiveStoryCount::Class;

premiumLimitTypeActiveStoryCount::premiumLimitTypeActiveStoryCount() {
}

const std::int32_t premiumLimitTypeActiveStoryCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeActiveStoryCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeActiveStoryCount>();
}

void premiumLimitTypeActiveStoryCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeActiveStoryCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeActiveStoryCount");
    s.store_class_end();
  }
}

void premiumLimitTypeActiveStoryCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeActiveStoryCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeWeeklyPostedStoryCount::Class;

premiumLimitTypeWeeklyPostedStoryCount::premiumLimitTypeWeeklyPostedStoryCount() {
}

const std::int32_t premiumLimitTypeWeeklyPostedStoryCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeWeeklyPostedStoryCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeWeeklyPostedStoryCount>();
}

void premiumLimitTypeWeeklyPostedStoryCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeWeeklyPostedStoryCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeWeeklyPostedStoryCount");
    s.store_class_end();
  }
}

void premiumLimitTypeWeeklyPostedStoryCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeWeeklyPostedStoryCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeMonthlyPostedStoryCount::Class;

premiumLimitTypeMonthlyPostedStoryCount::premiumLimitTypeMonthlyPostedStoryCount() {
}

const std::int32_t premiumLimitTypeMonthlyPostedStoryCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeMonthlyPostedStoryCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeMonthlyPostedStoryCount>();
}

void premiumLimitTypeMonthlyPostedStoryCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeMonthlyPostedStoryCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeMonthlyPostedStoryCount");
    s.store_class_end();
  }
}

void premiumLimitTypeMonthlyPostedStoryCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeMonthlyPostedStoryCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeStoryCaptionLength::Class;

premiumLimitTypeStoryCaptionLength::premiumLimitTypeStoryCaptionLength() {
}

const std::int32_t premiumLimitTypeStoryCaptionLength::ID;

object_ptr<PremiumLimitType> premiumLimitTypeStoryCaptionLength::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeStoryCaptionLength>();
}

void premiumLimitTypeStoryCaptionLength::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeStoryCaptionLength::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeStoryCaptionLength");
    s.store_class_end();
  }
}

void premiumLimitTypeStoryCaptionLength::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeStoryCaptionLength").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeStorySuggestedReactionAreaCount::Class;

premiumLimitTypeStorySuggestedReactionAreaCount::premiumLimitTypeStorySuggestedReactionAreaCount() {
}

const std::int32_t premiumLimitTypeStorySuggestedReactionAreaCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeStorySuggestedReactionAreaCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeStorySuggestedReactionAreaCount>();
}

void premiumLimitTypeStorySuggestedReactionAreaCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeStorySuggestedReactionAreaCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeStorySuggestedReactionAreaCount");
    s.store_class_end();
  }
}

void premiumLimitTypeStorySuggestedReactionAreaCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeStorySuggestedReactionAreaCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumLimitTypeSimilarChatCount::Class;

premiumLimitTypeSimilarChatCount::premiumLimitTypeSimilarChatCount() {
}

const std::int32_t premiumLimitTypeSimilarChatCount::ID;

object_ptr<PremiumLimitType> premiumLimitTypeSimilarChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumLimitTypeSimilarChatCount>();
}

void premiumLimitTypeSimilarChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void premiumLimitTypeSimilarChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumLimitTypeSimilarChatCount");
    s.store_class_end();
  }
}

void premiumLimitTypeSimilarChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumLimitTypeSimilarChatCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumPaymentOption::Class;
jfieldID premiumPaymentOption::currency_fieldID;
jfieldID premiumPaymentOption::amount_fieldID;
jfieldID premiumPaymentOption::discount_percentage_fieldID;
jfieldID premiumPaymentOption::month_count_fieldID;
jfieldID premiumPaymentOption::store_product_id_fieldID;
jfieldID premiumPaymentOption::payment_link_fieldID;

premiumPaymentOption::premiumPaymentOption()
  : currency_()
  , amount_()
  , discount_percentage_()
  , month_count_()
  , store_product_id_()
  , payment_link_()
{}

premiumPaymentOption::premiumPaymentOption(string const &currency_, int53 amount_, int32 discount_percentage_, int32 month_count_, string const &store_product_id_, object_ptr<InternalLinkType> &&payment_link_)
  : currency_(currency_)
  , amount_(amount_)
  , discount_percentage_(discount_percentage_)
  , month_count_(month_count_)
  , store_product_id_(store_product_id_)
  , payment_link_(std::move(payment_link_))
{}

const std::int32_t premiumPaymentOption::ID;

object_ptr<premiumPaymentOption> premiumPaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumPaymentOption> res = make_object<premiumPaymentOption>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->discount_percentage_ = env->GetIntField(p, res->discount_percentage_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->payment_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->payment_link_fieldID));
  return res;
}

void premiumPaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  env->SetIntField(s, discount_percentage_fieldID, discount_percentage_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (payment_link_ != nullptr) { jobject next; payment_link_->store(env, next); if (next) { env->SetObjectField(s, payment_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void premiumPaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumPaymentOption");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("discountPercentage", discount_percentage_);
    s.store_field("monthCount", month_count_);
    s.store_field("storeProductId", store_product_id_);
    s.store_object_field("paymentLink", static_cast<const BaseObject *>(payment_link_.get()));
    s.store_class_end();
  }
}

void premiumPaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumPaymentOption").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    discount_percentage_fieldID = jni::get_field_id(env, Class, "discountPercentage", "I");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    payment_link_fieldID = jni::get_field_id(env, Class, "paymentLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PremiumSource::Class;

object_ptr<PremiumSource> PremiumSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case premiumSourceLimitExceeded::ID:
      return premiumSourceLimitExceeded::fetch(env, p);
    case premiumSourceFeature::ID:
      return premiumSourceFeature::fetch(env, p);
    case premiumSourceBusinessFeature::ID:
      return premiumSourceBusinessFeature::fetch(env, p);
    case premiumSourceStoryFeature::ID:
      return premiumSourceStoryFeature::fetch(env, p);
    case premiumSourceLink::ID:
      return premiumSourceLink::fetch(env, p);
    case premiumSourceSettings::ID:
      return premiumSourceSettings::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PremiumSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceLimitExceeded::Class;
jfieldID premiumSourceLimitExceeded::limit_type_fieldID;

premiumSourceLimitExceeded::premiumSourceLimitExceeded()
  : limit_type_()
{}

premiumSourceLimitExceeded::premiumSourceLimitExceeded(object_ptr<PremiumLimitType> &&limit_type_)
  : limit_type_(std::move(limit_type_))
{}

const std::int32_t premiumSourceLimitExceeded::ID;

object_ptr<PremiumSource> premiumSourceLimitExceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumSourceLimitExceeded> res = make_object<premiumSourceLimitExceeded>();
  res->limit_type_ = jni::fetch_tl_object<PremiumLimitType>(env, jni::fetch_object(env, p, res->limit_type_fieldID));
  return std::move(res);
}

void premiumSourceLimitExceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceLimitExceeded");
    s.store_object_field("limitType", static_cast<const BaseObject *>(limit_type_.get()));
    s.store_class_end();
  }
}

void premiumSourceLimitExceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceLimitExceeded").c_str());
    limit_type_fieldID = jni::get_field_id(env, Class, "limitType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumLimitType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceFeature::Class;
jfieldID premiumSourceFeature::feature_fieldID;

premiumSourceFeature::premiumSourceFeature()
  : feature_()
{}

premiumSourceFeature::premiumSourceFeature(object_ptr<PremiumFeature> &&feature_)
  : feature_(std::move(feature_))
{}

const std::int32_t premiumSourceFeature::ID;

object_ptr<PremiumSource> premiumSourceFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumSourceFeature> res = make_object<premiumSourceFeature>();
  res->feature_ = jni::fetch_tl_object<PremiumFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  return std::move(res);
}

void premiumSourceFeature::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceFeature");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_class_end();
  }
}

void premiumSourceFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceFeature").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceBusinessFeature::Class;
jfieldID premiumSourceBusinessFeature::feature_fieldID;

premiumSourceBusinessFeature::premiumSourceBusinessFeature()
  : feature_()
{}

premiumSourceBusinessFeature::premiumSourceBusinessFeature(object_ptr<BusinessFeature> &&feature_)
  : feature_(std::move(feature_))
{}

const std::int32_t premiumSourceBusinessFeature::ID;

object_ptr<PremiumSource> premiumSourceBusinessFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumSourceBusinessFeature> res = make_object<premiumSourceBusinessFeature>();
  res->feature_ = jni::fetch_tl_object<BusinessFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  return std::move(res);
}

void premiumSourceBusinessFeature::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceBusinessFeature");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_class_end();
  }
}

void premiumSourceBusinessFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceBusinessFeature").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceStoryFeature::Class;
jfieldID premiumSourceStoryFeature::feature_fieldID;

premiumSourceStoryFeature::premiumSourceStoryFeature()
  : feature_()
{}

premiumSourceStoryFeature::premiumSourceStoryFeature(object_ptr<PremiumStoryFeature> &&feature_)
  : feature_(std::move(feature_))
{}

const std::int32_t premiumSourceStoryFeature::ID;

object_ptr<PremiumSource> premiumSourceStoryFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumSourceStoryFeature> res = make_object<premiumSourceStoryFeature>();
  res->feature_ = jni::fetch_tl_object<PremiumStoryFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  return std::move(res);
}

void premiumSourceStoryFeature::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceStoryFeature");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_class_end();
  }
}

void premiumSourceStoryFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceStoryFeature").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumStoryFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceLink::Class;
jfieldID premiumSourceLink::referrer_fieldID;

premiumSourceLink::premiumSourceLink()
  : referrer_()
{}

premiumSourceLink::premiumSourceLink(string const &referrer_)
  : referrer_(referrer_)
{}

const std::int32_t premiumSourceLink::ID;

object_ptr<PremiumSource> premiumSourceLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumSourceLink> res = make_object<premiumSourceLink>();
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return std::move(res);
}

void premiumSourceLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceLink");
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

void premiumSourceLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceLink").c_str());
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass premiumSourceSettings::Class;

premiumSourceSettings::premiumSourceSettings() {
}

const std::int32_t premiumSourceSettings::ID;

object_ptr<PremiumSource> premiumSourceSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumSourceSettings>();
}

void premiumSourceSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumSourceSettings");
    s.store_class_end();
  }
}

void premiumSourceSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumSourceSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumState::Class;
jfieldID premiumState::state_fieldID;
jfieldID premiumState::payment_options_fieldID;
jfieldID premiumState::animations_fieldID;
jfieldID premiumState::business_animations_fieldID;

premiumState::premiumState()
  : state_()
  , payment_options_()
  , animations_()
  , business_animations_()
{}

premiumState::premiumState(object_ptr<formattedText> &&state_, array<object_ptr<premiumStatePaymentOption>> &&payment_options_, array<object_ptr<premiumFeaturePromotionAnimation>> &&animations_, array<object_ptr<businessFeaturePromotionAnimation>> &&business_animations_)
  : state_(std::move(state_))
  , payment_options_(std::move(payment_options_))
  , animations_(std::move(animations_))
  , business_animations_(std::move(business_animations_))
{}

const std::int32_t premiumState::ID;

object_ptr<premiumState> premiumState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumState> res = make_object<premiumState>();
  res->state_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->state_fieldID));
  res->payment_options_ = jni::FetchVector<premiumStatePaymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->payment_options_fieldID));
  res->animations_ = jni::FetchVector<premiumFeaturePromotionAnimation>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->animations_fieldID));
  res->business_animations_ = jni::FetchVector<businessFeaturePromotionAnimation>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->business_animations_fieldID));
  return res;
}

void premiumState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (state_ != nullptr) { jobject next; state_->store(env, next); if (next) { env->SetObjectField(s, state_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, payment_options_); if (arr_tmp_) { env->SetObjectField(s, payment_options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, animations_); if (arr_tmp_) { env->SetObjectField(s, animations_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, business_animations_); if (arr_tmp_) { env->SetObjectField(s, business_animations_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void premiumState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumState");
    s.store_object_field("state", static_cast<const BaseObject *>(state_.get()));
    { s.store_vector_begin("paymentOptions", payment_options_.size()); for (const auto &_value : payment_options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("animations", animations_.size()); for (const auto &_value : animations_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("businessAnimations", business_animations_.size()); for (const auto &_value : business_animations_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void premiumState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumState").c_str());
    state_fieldID = jni::get_field_id(env, Class, "state", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    payment_options_fieldID = jni::get_field_id(env, Class, "paymentOptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumStatePaymentOption;").c_str());
    animations_fieldID = jni::get_field_id(env, Class, "animations", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PremiumFeaturePromotionAnimation;").c_str());
    business_animations_fieldID = jni::get_field_id(env, Class, "businessAnimations", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BusinessFeaturePromotionAnimation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStatePaymentOption::Class;
jfieldID premiumStatePaymentOption::payment_option_fieldID;
jfieldID premiumStatePaymentOption::is_current_fieldID;
jfieldID premiumStatePaymentOption::is_upgrade_fieldID;
jfieldID premiumStatePaymentOption::last_transaction_id_fieldID;

premiumStatePaymentOption::premiumStatePaymentOption()
  : payment_option_()
  , is_current_()
  , is_upgrade_()
  , last_transaction_id_()
{}

premiumStatePaymentOption::premiumStatePaymentOption(object_ptr<premiumPaymentOption> &&payment_option_, bool is_current_, bool is_upgrade_, string const &last_transaction_id_)
  : payment_option_(std::move(payment_option_))
  , is_current_(is_current_)
  , is_upgrade_(is_upgrade_)
  , last_transaction_id_(last_transaction_id_)
{}

const std::int32_t premiumStatePaymentOption::ID;

object_ptr<premiumStatePaymentOption> premiumStatePaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<premiumStatePaymentOption> res = make_object<premiumStatePaymentOption>();
  res->payment_option_ = jni::fetch_tl_object<premiumPaymentOption>(env, jni::fetch_object(env, p, res->payment_option_fieldID));
  res->is_current_ = (env->GetBooleanField(p, res->is_current_fieldID) != 0);
  res->is_upgrade_ = (env->GetBooleanField(p, res->is_upgrade_fieldID) != 0);
  res->last_transaction_id_ = jni::fetch_string(env, p, res->last_transaction_id_fieldID);
  return res;
}

void premiumStatePaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (payment_option_ != nullptr) { jobject next; payment_option_->store(env, next); if (next) { env->SetObjectField(s, payment_option_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_current_fieldID, is_current_);
  env->SetBooleanField(s, is_upgrade_fieldID, is_upgrade_);
  { jstring nextString = jni::to_jstring(env, last_transaction_id_); if (nextString) { env->SetObjectField(s, last_transaction_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void premiumStatePaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStatePaymentOption");
    s.store_object_field("paymentOption", static_cast<const BaseObject *>(payment_option_.get()));
    s.store_field("isCurrent", is_current_);
    s.store_field("isUpgrade", is_upgrade_);
    s.store_field("lastTransactionId", last_transaction_id_);
    s.store_class_end();
  }
}

void premiumStatePaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStatePaymentOption").c_str());
    payment_option_fieldID = jni::get_field_id(env, Class, "paymentOption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumPaymentOption;").c_str());
    is_current_fieldID = jni::get_field_id(env, Class, "isCurrent", "Z");
    is_upgrade_fieldID = jni::get_field_id(env, Class, "isUpgrade", "Z");
    last_transaction_id_fieldID = jni::get_field_id(env, Class, "lastTransactionId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PremiumStoryFeature::Class;

object_ptr<PremiumStoryFeature> PremiumStoryFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case premiumStoryFeaturePriorityOrder::ID:
      return premiumStoryFeaturePriorityOrder::fetch(env, p);
    case premiumStoryFeatureStealthMode::ID:
      return premiumStoryFeatureStealthMode::fetch(env, p);
    case premiumStoryFeaturePermanentViewsHistory::ID:
      return premiumStoryFeaturePermanentViewsHistory::fetch(env, p);
    case premiumStoryFeatureCustomExpirationDuration::ID:
      return premiumStoryFeatureCustomExpirationDuration::fetch(env, p);
    case premiumStoryFeatureSaveStories::ID:
      return premiumStoryFeatureSaveStories::fetch(env, p);
    case premiumStoryFeatureLinksAndFormatting::ID:
      return premiumStoryFeatureLinksAndFormatting::fetch(env, p);
    case premiumStoryFeatureVideoQuality::ID:
      return premiumStoryFeatureVideoQuality::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PremiumStoryFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeature").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeaturePriorityOrder::Class;

premiumStoryFeaturePriorityOrder::premiumStoryFeaturePriorityOrder() {
}

const std::int32_t premiumStoryFeaturePriorityOrder::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeaturePriorityOrder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeaturePriorityOrder>();
}

void premiumStoryFeaturePriorityOrder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeaturePriorityOrder");
    s.store_class_end();
  }
}

void premiumStoryFeaturePriorityOrder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeaturePriorityOrder").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeatureStealthMode::Class;

premiumStoryFeatureStealthMode::premiumStoryFeatureStealthMode() {
}

const std::int32_t premiumStoryFeatureStealthMode::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeatureStealthMode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeatureStealthMode>();
}

void premiumStoryFeatureStealthMode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeatureStealthMode");
    s.store_class_end();
  }
}

void premiumStoryFeatureStealthMode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeatureStealthMode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeaturePermanentViewsHistory::Class;

premiumStoryFeaturePermanentViewsHistory::premiumStoryFeaturePermanentViewsHistory() {
}

const std::int32_t premiumStoryFeaturePermanentViewsHistory::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeaturePermanentViewsHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeaturePermanentViewsHistory>();
}

void premiumStoryFeaturePermanentViewsHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeaturePermanentViewsHistory");
    s.store_class_end();
  }
}

void premiumStoryFeaturePermanentViewsHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeaturePermanentViewsHistory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeatureCustomExpirationDuration::Class;

premiumStoryFeatureCustomExpirationDuration::premiumStoryFeatureCustomExpirationDuration() {
}

const std::int32_t premiumStoryFeatureCustomExpirationDuration::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeatureCustomExpirationDuration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeatureCustomExpirationDuration>();
}

void premiumStoryFeatureCustomExpirationDuration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeatureCustomExpirationDuration");
    s.store_class_end();
  }
}

void premiumStoryFeatureCustomExpirationDuration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeatureCustomExpirationDuration").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeatureSaveStories::Class;

premiumStoryFeatureSaveStories::premiumStoryFeatureSaveStories() {
}

const std::int32_t premiumStoryFeatureSaveStories::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeatureSaveStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeatureSaveStories>();
}

void premiumStoryFeatureSaveStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeatureSaveStories");
    s.store_class_end();
  }
}

void premiumStoryFeatureSaveStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeatureSaveStories").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeatureLinksAndFormatting::Class;

premiumStoryFeatureLinksAndFormatting::premiumStoryFeatureLinksAndFormatting() {
}

const std::int32_t premiumStoryFeatureLinksAndFormatting::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeatureLinksAndFormatting::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeatureLinksAndFormatting>();
}

void premiumStoryFeatureLinksAndFormatting::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeatureLinksAndFormatting");
    s.store_class_end();
  }
}

void premiumStoryFeatureLinksAndFormatting::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeatureLinksAndFormatting").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass premiumStoryFeatureVideoQuality::Class;

premiumStoryFeatureVideoQuality::premiumStoryFeatureVideoQuality() {
}

const std::int32_t premiumStoryFeatureVideoQuality::ID;

object_ptr<PremiumStoryFeature> premiumStoryFeatureVideoQuality::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<premiumStoryFeatureVideoQuality>();
}

void premiumStoryFeatureVideoQuality::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PremiumStoryFeatureVideoQuality");
    s.store_class_end();
  }
}

void premiumStoryFeatureVideoQuality::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PremiumStoryFeatureVideoQuality").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass prepaidGiveaway::Class;
jfieldID prepaidGiveaway::id_fieldID;
jfieldID prepaidGiveaway::winner_count_fieldID;
jfieldID prepaidGiveaway::prize_fieldID;
jfieldID prepaidGiveaway::boost_count_fieldID;
jfieldID prepaidGiveaway::payment_date_fieldID;

prepaidGiveaway::prepaidGiveaway()
  : id_()
  , winner_count_()
  , prize_()
  , boost_count_()
  , payment_date_()
{}

prepaidGiveaway::prepaidGiveaway(int64 id_, int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, int32 boost_count_, int32 payment_date_)
  : id_(id_)
  , winner_count_(winner_count_)
  , prize_(std::move(prize_))
  , boost_count_(boost_count_)
  , payment_date_(payment_date_)
{}

const std::int32_t prepaidGiveaway::ID;

object_ptr<prepaidGiveaway> prepaidGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<prepaidGiveaway> res = make_object<prepaidGiveaway>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->prize_ = jni::fetch_tl_object<GiveawayPrize>(env, jni::fetch_object(env, p, res->prize_fieldID));
  res->boost_count_ = env->GetIntField(p, res->boost_count_fieldID);
  res->payment_date_ = env->GetIntField(p, res->payment_date_fieldID);
  return res;
}

void prepaidGiveaway::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  if (prize_ != nullptr) { jobject next; prize_->store(env, next); if (next) { env->SetObjectField(s, prize_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, boost_count_fieldID, boost_count_);
  env->SetIntField(s, payment_date_fieldID, payment_date_);
}

void prepaidGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PrepaidGiveaway");
    s.store_field("id", id_);
    s.store_field("winnerCount", winner_count_);
    s.store_object_field("prize", static_cast<const BaseObject *>(prize_.get()));
    s.store_field("boostCount", boost_count_);
    s.store_field("paymentDate", payment_date_);
    s.store_class_end();
  }
}

void prepaidGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PrepaidGiveaway").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    prize_fieldID = jni::get_field_id(env, Class, "prize", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayPrize;").c_str());
    boost_count_fieldID = jni::get_field_id(env, Class, "boostCount", "I");
    payment_date_fieldID = jni::get_field_id(env, Class, "paymentDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass preparedInlineMessage::Class;
jfieldID preparedInlineMessage::inline_query_id_fieldID;
jfieldID preparedInlineMessage::result_fieldID;
jfieldID preparedInlineMessage::chat_types_fieldID;

preparedInlineMessage::preparedInlineMessage()
  : inline_query_id_()
  , result_()
  , chat_types_()
{}

preparedInlineMessage::preparedInlineMessage(int64 inline_query_id_, object_ptr<InlineQueryResult> &&result_, object_ptr<targetChatTypes> &&chat_types_)
  : inline_query_id_(inline_query_id_)
  , result_(std::move(result_))
  , chat_types_(std::move(chat_types_))
{}

const std::int32_t preparedInlineMessage::ID;

object_ptr<preparedInlineMessage> preparedInlineMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<preparedInlineMessage> res = make_object<preparedInlineMessage>();
  res->inline_query_id_ = env->GetLongField(p, res->inline_query_id_fieldID);
  res->result_ = jni::fetch_tl_object<InlineQueryResult>(env, jni::fetch_object(env, p, res->result_fieldID));
  res->chat_types_ = jni::fetch_tl_object<targetChatTypes>(env, jni::fetch_object(env, p, res->chat_types_fieldID));
  return res;
}

void preparedInlineMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, inline_query_id_fieldID, inline_query_id_);
  if (result_ != nullptr) { jobject next; result_->store(env, next); if (next) { env->SetObjectField(s, result_fieldID, next); env->DeleteLocalRef(next); } }
  if (chat_types_ != nullptr) { jobject next; chat_types_->store(env, next); if (next) { env->SetObjectField(s, chat_types_fieldID, next); env->DeleteLocalRef(next); } }
}

void preparedInlineMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PreparedInlineMessage");
    s.store_field("inlineQueryId", inline_query_id_);
    s.store_object_field("result", static_cast<const BaseObject *>(result_.get()));
    s.store_object_field("chatTypes", static_cast<const BaseObject *>(chat_types_.get()));
    s.store_class_end();
  }
}

void preparedInlineMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PreparedInlineMessage").c_str());
    inline_query_id_fieldID = jni::get_field_id(env, Class, "inlineQueryId", "J");
    result_fieldID = jni::get_field_id(env, Class, "result", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InlineQueryResult;").c_str());
    chat_types_fieldID = jni::get_field_id(env, Class, "chatTypes", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TargetChatTypes;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass preparedInlineMessageId::Class;
jfieldID preparedInlineMessageId::id_fieldID;
jfieldID preparedInlineMessageId::expiration_date_fieldID;

preparedInlineMessageId::preparedInlineMessageId()
  : id_()
  , expiration_date_()
{}

preparedInlineMessageId::preparedInlineMessageId(string const &id_, int32 expiration_date_)
  : id_(id_)
  , expiration_date_(expiration_date_)
{}

const std::int32_t preparedInlineMessageId::ID;

object_ptr<preparedInlineMessageId> preparedInlineMessageId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<preparedInlineMessageId> res = make_object<preparedInlineMessageId>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  return res;
}

void preparedInlineMessageId::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
}

void preparedInlineMessageId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PreparedInlineMessageId");
    s.store_field("id", id_);
    s.store_field("expirationDate", expiration_date_);
    s.store_class_end();
  }
}

void preparedInlineMessageId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PreparedInlineMessageId").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass productInfo::Class;
jfieldID productInfo::title_fieldID;
jfieldID productInfo::description_fieldID;
jfieldID productInfo::photo_fieldID;

productInfo::productInfo()
  : title_()
  , description_()
  , photo_()
{}

productInfo::productInfo(string const &title_, object_ptr<formattedText> &&description_, object_ptr<photo> &&photo_)
  : title_(title_)
  , description_(std::move(description_))
  , photo_(std::move(photo_))
{}

const std::int32_t productInfo::ID;

object_ptr<productInfo> productInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<productInfo> res = make_object<productInfo>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->description_fieldID));
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void productInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (description_ != nullptr) { jobject next; description_->store(env, next); if (next) { env->SetObjectField(s, description_fieldID, next); env->DeleteLocalRef(next); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void productInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProductInfo");
    s.store_field("title", title_);
    s.store_object_field("description", static_cast<const BaseObject *>(description_.get()));
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void productInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProductInfo").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass profileAccentColor::Class;
jfieldID profileAccentColor::id_fieldID;
jfieldID profileAccentColor::light_theme_colors_fieldID;
jfieldID profileAccentColor::dark_theme_colors_fieldID;
jfieldID profileAccentColor::min_supergroup_chat_boost_level_fieldID;
jfieldID profileAccentColor::min_channel_chat_boost_level_fieldID;

profileAccentColor::profileAccentColor()
  : id_()
  , light_theme_colors_()
  , dark_theme_colors_()
  , min_supergroup_chat_boost_level_()
  , min_channel_chat_boost_level_()
{}

profileAccentColor::profileAccentColor(int32 id_, object_ptr<profileAccentColors> &&light_theme_colors_, object_ptr<profileAccentColors> &&dark_theme_colors_, int32 min_supergroup_chat_boost_level_, int32 min_channel_chat_boost_level_)
  : id_(id_)
  , light_theme_colors_(std::move(light_theme_colors_))
  , dark_theme_colors_(std::move(dark_theme_colors_))
  , min_supergroup_chat_boost_level_(min_supergroup_chat_boost_level_)
  , min_channel_chat_boost_level_(min_channel_chat_boost_level_)
{}

const std::int32_t profileAccentColor::ID;

object_ptr<profileAccentColor> profileAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<profileAccentColor> res = make_object<profileAccentColor>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->light_theme_colors_ = jni::fetch_tl_object<profileAccentColors>(env, jni::fetch_object(env, p, res->light_theme_colors_fieldID));
  res->dark_theme_colors_ = jni::fetch_tl_object<profileAccentColors>(env, jni::fetch_object(env, p, res->dark_theme_colors_fieldID));
  res->min_supergroup_chat_boost_level_ = env->GetIntField(p, res->min_supergroup_chat_boost_level_fieldID);
  res->min_channel_chat_boost_level_ = env->GetIntField(p, res->min_channel_chat_boost_level_fieldID);
  return res;
}

void profileAccentColor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  if (light_theme_colors_ != nullptr) { jobject next; light_theme_colors_->store(env, next); if (next) { env->SetObjectField(s, light_theme_colors_fieldID, next); env->DeleteLocalRef(next); } }
  if (dark_theme_colors_ != nullptr) { jobject next; dark_theme_colors_->store(env, next); if (next) { env->SetObjectField(s, dark_theme_colors_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, min_supergroup_chat_boost_level_fieldID, min_supergroup_chat_boost_level_);
  env->SetIntField(s, min_channel_chat_boost_level_fieldID, min_channel_chat_boost_level_);
}

void profileAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProfileAccentColor");
    s.store_field("id", id_);
    s.store_object_field("lightThemeColors", static_cast<const BaseObject *>(light_theme_colors_.get()));
    s.store_object_field("darkThemeColors", static_cast<const BaseObject *>(dark_theme_colors_.get()));
    s.store_field("minSupergroupChatBoostLevel", min_supergroup_chat_boost_level_);
    s.store_field("minChannelChatBoostLevel", min_channel_chat_boost_level_);
    s.store_class_end();
  }
}

void profileAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProfileAccentColor").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    light_theme_colors_fieldID = jni::get_field_id(env, Class, "lightThemeColors", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProfileAccentColors;").c_str());
    dark_theme_colors_fieldID = jni::get_field_id(env, Class, "darkThemeColors", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProfileAccentColors;").c_str());
    min_supergroup_chat_boost_level_fieldID = jni::get_field_id(env, Class, "minSupergroupChatBoostLevel", "I");
    min_channel_chat_boost_level_fieldID = jni::get_field_id(env, Class, "minChannelChatBoostLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass profileAccentColors::Class;
jfieldID profileAccentColors::palette_colors_fieldID;
jfieldID profileAccentColors::background_colors_fieldID;
jfieldID profileAccentColors::story_colors_fieldID;

profileAccentColors::profileAccentColors()
  : palette_colors_()
  , background_colors_()
  , story_colors_()
{}

profileAccentColors::profileAccentColors(array<int32> &&palette_colors_, array<int32> &&background_colors_, array<int32> &&story_colors_)
  : palette_colors_(std::move(palette_colors_))
  , background_colors_(std::move(background_colors_))
  , story_colors_(std::move(story_colors_))
{}

const std::int32_t profileAccentColors::ID;

object_ptr<profileAccentColors> profileAccentColors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<profileAccentColors> res = make_object<profileAccentColors>();
  res->palette_colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->palette_colors_fieldID));
  res->background_colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->background_colors_fieldID));
  res->story_colors_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->story_colors_fieldID));
  return res;
}

void profileAccentColors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, palette_colors_); if (arr_tmp_) { env->SetObjectField(s, palette_colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, background_colors_); if (arr_tmp_) { env->SetObjectField(s, background_colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, story_colors_); if (arr_tmp_) { env->SetObjectField(s, story_colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void profileAccentColors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProfileAccentColors");
    { s.store_vector_begin("paletteColors", palette_colors_.size()); for (const auto &_value : palette_colors_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("backgroundColors", background_colors_.size()); for (const auto &_value : background_colors_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("storyColors", story_colors_.size()); for (const auto &_value : story_colors_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void profileAccentColors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProfileAccentColors").c_str());
    palette_colors_fieldID = jni::get_field_id(env, Class, "paletteColors", "[I");
    background_colors_fieldID = jni::get_field_id(env, Class, "backgroundColors", "[I");
    story_colors_fieldID = jni::get_field_id(env, Class, "storyColors", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass profilePhoto::Class;
jfieldID profilePhoto::id_fieldID;
jfieldID profilePhoto::small_fieldID;
jfieldID profilePhoto::big_fieldID;
jfieldID profilePhoto::minithumbnail_fieldID;
jfieldID profilePhoto::has_animation_fieldID;
jfieldID profilePhoto::is_personal_fieldID;

profilePhoto::profilePhoto()
  : id_()
  , small_()
  , big_()
  , minithumbnail_()
  , has_animation_()
  , is_personal_()
{}

profilePhoto::profilePhoto(int64 id_, object_ptr<file> &&small_, object_ptr<file> &&big_, object_ptr<minithumbnail> &&minithumbnail_, bool has_animation_, bool is_personal_)
  : id_(id_)
  , small_(std::move(small_))
  , big_(std::move(big_))
  , minithumbnail_(std::move(minithumbnail_))
  , has_animation_(has_animation_)
  , is_personal_(is_personal_)
{}

const std::int32_t profilePhoto::ID;

object_ptr<profilePhoto> profilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<profilePhoto> res = make_object<profilePhoto>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->small_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->small_fieldID));
  res->big_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->big_fieldID));
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->has_animation_ = (env->GetBooleanField(p, res->has_animation_fieldID) != 0);
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  return res;
}

void profilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (small_ != nullptr) { jobject next; small_->store(env, next); if (next) { env->SetObjectField(s, small_fieldID, next); env->DeleteLocalRef(next); } }
  if (big_ != nullptr) { jobject next; big_->store(env, next); if (next) { env->SetObjectField(s, big_fieldID, next); env->DeleteLocalRef(next); } }
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_animation_fieldID, has_animation_);
  env->SetBooleanField(s, is_personal_fieldID, is_personal_);
}

void profilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProfilePhoto");
    s.store_field("id", id_);
    s.store_object_field("small", static_cast<const BaseObject *>(small_.get()));
    s.store_object_field("big", static_cast<const BaseObject *>(big_.get()));
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_field("hasAnimation", has_animation_);
    s.store_field("isPersonal", is_personal_);
    s.store_class_end();
  }
}

void profilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProfilePhoto").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    small_fieldID = jni::get_field_id(env, Class, "small", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    big_fieldID = jni::get_field_id(env, Class, "big", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    has_animation_fieldID = jni::get_field_id(env, Class, "hasAnimation", "Z");
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass proxies::Class;
jfieldID proxies::proxies_fieldID;

proxies::proxies()
  : proxies_()
{}

proxies::proxies(array<object_ptr<proxy>> &&proxies_)
  : proxies_(std::move(proxies_))
{}

const std::int32_t proxies::ID;

object_ptr<proxies> proxies::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<proxies> res = make_object<proxies>();
  res->proxies_ = jni::FetchVector<proxy>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->proxies_fieldID));
  return res;
}

void proxies::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, proxies_); if (arr_tmp_) { env->SetObjectField(s, proxies_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void proxies::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Proxies");
    { s.store_vector_begin("proxies", proxies_.size()); for (const auto &_value : proxies_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void proxies::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Proxies").c_str());
    proxies_fieldID = jni::get_field_id(env, Class, "proxies", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Proxy;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass proxy::Class;
jfieldID proxy::id_fieldID;
jfieldID proxy::server_fieldID;
jfieldID proxy::port_fieldID;
jfieldID proxy::last_used_date_fieldID;
jfieldID proxy::is_enabled_fieldID;
jfieldID proxy::type_fieldID;

proxy::proxy()
  : id_()
  , server_()
  , port_()
  , last_used_date_()
  , is_enabled_()
  , type_()
{}

proxy::proxy(int32 id_, string const &server_, int32 port_, int32 last_used_date_, bool is_enabled_, object_ptr<ProxyType> &&type_)
  : id_(id_)
  , server_(server_)
  , port_(port_)
  , last_used_date_(last_used_date_)
  , is_enabled_(is_enabled_)
  , type_(std::move(type_))
{}

const std::int32_t proxy::ID;

object_ptr<proxy> proxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<proxy> res = make_object<proxy>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->server_ = jni::fetch_string(env, p, res->server_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->last_used_date_ = env->GetIntField(p, res->last_used_date_fieldID);
  res->is_enabled_ = (env->GetBooleanField(p, res->is_enabled_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<ProxyType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void proxy::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, server_); if (nextString) { env->SetObjectField(s, server_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, port_fieldID, port_);
  env->SetIntField(s, last_used_date_fieldID, last_used_date_);
  env->SetBooleanField(s, is_enabled_fieldID, is_enabled_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void proxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Proxy");
    s.store_field("id", id_);
    s.store_field("server", server_);
    s.store_field("port", port_);
    s.store_field("lastUsedDate", last_used_date_);
    s.store_field("isEnabled", is_enabled_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void proxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Proxy").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    server_fieldID = jni::get_field_id(env, Class, "server", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    last_used_date_fieldID = jni::get_field_id(env, Class, "lastUsedDate", "I");
    is_enabled_fieldID = jni::get_field_id(env, Class, "isEnabled", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProxyType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ProxyType::Class;

object_ptr<ProxyType> ProxyType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case proxyTypeSocks5::ID:
      return proxyTypeSocks5::fetch(env, p);
    case proxyTypeHttp::ID:
      return proxyTypeHttp::fetch(env, p);
    case proxyTypeMtproto::ID:
      return proxyTypeMtproto::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ProxyType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProxyType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass proxyTypeSocks5::Class;
jfieldID proxyTypeSocks5::username_fieldID;
jfieldID proxyTypeSocks5::password_fieldID;

proxyTypeSocks5::proxyTypeSocks5()
  : username_()
  , password_()
{}

proxyTypeSocks5::proxyTypeSocks5(string const &username_, string const &password_)
  : username_(username_)
  , password_(password_)
{}

const std::int32_t proxyTypeSocks5::ID;

object_ptr<ProxyType> proxyTypeSocks5::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<proxyTypeSocks5> res = make_object<proxyTypeSocks5>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return std::move(res);
}

void proxyTypeSocks5::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, password_); if (nextString) { env->SetObjectField(s, password_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void proxyTypeSocks5::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProxyTypeSocks5");
    s.store_field("username", username_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

void proxyTypeSocks5::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProxyTypeSocks5").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass proxyTypeHttp::Class;
jfieldID proxyTypeHttp::username_fieldID;
jfieldID proxyTypeHttp::password_fieldID;
jfieldID proxyTypeHttp::http_only_fieldID;

proxyTypeHttp::proxyTypeHttp()
  : username_()
  , password_()
  , http_only_()
{}

proxyTypeHttp::proxyTypeHttp(string const &username_, string const &password_, bool http_only_)
  : username_(username_)
  , password_(password_)
  , http_only_(http_only_)
{}

const std::int32_t proxyTypeHttp::ID;

object_ptr<ProxyType> proxyTypeHttp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<proxyTypeHttp> res = make_object<proxyTypeHttp>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  res->http_only_ = (env->GetBooleanField(p, res->http_only_fieldID) != 0);
  return std::move(res);
}

void proxyTypeHttp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, password_); if (nextString) { env->SetObjectField(s, password_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, http_only_fieldID, http_only_);
}

void proxyTypeHttp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProxyTypeHttp");
    s.store_field("username", username_);
    s.store_field("password", password_);
    s.store_field("httpOnly", http_only_);
    s.store_class_end();
  }
}

void proxyTypeHttp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProxyTypeHttp").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    http_only_fieldID = jni::get_field_id(env, Class, "httpOnly", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass proxyTypeMtproto::Class;
jfieldID proxyTypeMtproto::secret_fieldID;

proxyTypeMtproto::proxyTypeMtproto()
  : secret_()
{}

proxyTypeMtproto::proxyTypeMtproto(string const &secret_)
  : secret_(secret_)
{}

const std::int32_t proxyTypeMtproto::ID;

object_ptr<ProxyType> proxyTypeMtproto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<proxyTypeMtproto> res = make_object<proxyTypeMtproto>();
  res->secret_ = jni::fetch_string(env, p, res->secret_fieldID);
  return std::move(res);
}

void proxyTypeMtproto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, secret_); if (nextString) { env->SetObjectField(s, secret_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void proxyTypeMtproto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProxyTypeMtproto");
    s.store_field("secret", secret_);
    s.store_class_end();
  }
}

void proxyTypeMtproto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProxyTypeMtproto").c_str());
    secret_fieldID = jni::get_field_id(env, Class, "secret", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PublicChatType::Class;

object_ptr<PublicChatType> PublicChatType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case publicChatTypeHasUsername::ID:
      return publicChatTypeHasUsername::fetch(env, p);
    case publicChatTypeIsLocationBased::ID:
      return publicChatTypeIsLocationBased::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PublicChatType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicChatType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass publicChatTypeHasUsername::Class;

publicChatTypeHasUsername::publicChatTypeHasUsername() {
}

const std::int32_t publicChatTypeHasUsername::ID;

object_ptr<PublicChatType> publicChatTypeHasUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<publicChatTypeHasUsername>();
}

void publicChatTypeHasUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PublicChatTypeHasUsername");
    s.store_class_end();
  }
}

void publicChatTypeHasUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicChatTypeHasUsername").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass publicChatTypeIsLocationBased::Class;

publicChatTypeIsLocationBased::publicChatTypeIsLocationBased() {
}

const std::int32_t publicChatTypeIsLocationBased::ID;

object_ptr<PublicChatType> publicChatTypeIsLocationBased::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<publicChatTypeIsLocationBased>();
}

void publicChatTypeIsLocationBased::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PublicChatTypeIsLocationBased");
    s.store_class_end();
  }
}

void publicChatTypeIsLocationBased::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicChatTypeIsLocationBased").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass PublicForward::Class;

object_ptr<PublicForward> PublicForward::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case publicForwardMessage::ID:
      return publicForwardMessage::fetch(env, p);
    case publicForwardStory::ID:
      return publicForwardStory::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PublicForward::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicForward").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass publicForwardMessage::Class;
jfieldID publicForwardMessage::message_fieldID;

publicForwardMessage::publicForwardMessage()
  : message_()
{}

publicForwardMessage::publicForwardMessage(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t publicForwardMessage::ID;

object_ptr<PublicForward> publicForwardMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<publicForwardMessage> res = make_object<publicForwardMessage>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void publicForwardMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void publicForwardMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PublicForwardMessage");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void publicForwardMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicForwardMessage").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass publicForwardStory::Class;
jfieldID publicForwardStory::story_fieldID;

publicForwardStory::publicForwardStory()
  : story_()
{}

publicForwardStory::publicForwardStory(object_ptr<story> &&story_)
  : story_(std::move(story_))
{}

const std::int32_t publicForwardStory::ID;

object_ptr<PublicForward> publicForwardStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<publicForwardStory> res = make_object<publicForwardStory>();
  res->story_ = jni::fetch_tl_object<story>(env, jni::fetch_object(env, p, res->story_fieldID));
  return std::move(res);
}

void publicForwardStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_ != nullptr) { jobject next; story_->store(env, next); if (next) { env->SetObjectField(s, story_fieldID, next); env->DeleteLocalRef(next); } }
}

void publicForwardStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PublicForwardStory");
    s.store_object_field("story", static_cast<const BaseObject *>(story_.get()));
    s.store_class_end();
  }
}

void publicForwardStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicForwardStory").c_str());
    story_fieldID = jni::get_field_id(env, Class, "story", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass publicForwards::Class;
jfieldID publicForwards::total_count_fieldID;
jfieldID publicForwards::forwards_fieldID;
jfieldID publicForwards::next_offset_fieldID;

publicForwards::publicForwards()
  : total_count_()
  , forwards_()
  , next_offset_()
{}

publicForwards::publicForwards(int32 total_count_, array<object_ptr<PublicForward>> &&forwards_, string const &next_offset_)
  : total_count_(total_count_)
  , forwards_(std::move(forwards_))
  , next_offset_(next_offset_)
{}

const std::int32_t publicForwards::ID;

object_ptr<publicForwards> publicForwards::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<publicForwards> res = make_object<publicForwards>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->forwards_ = jni::FetchVector<PublicForward>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->forwards_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void publicForwards::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, forwards_); if (arr_tmp_) { env->SetObjectField(s, forwards_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void publicForwards::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PublicForwards");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("forwards", forwards_.size()); for (const auto &_value : forwards_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void publicForwards::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PublicForwards").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    forwards_fieldID = jni::get_field_id(env, Class, "forwards", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PublicForward;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass PushMessageContent::Class;

object_ptr<PushMessageContent> PushMessageContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case pushMessageContentHidden::ID:
      return pushMessageContentHidden::fetch(env, p);
    case pushMessageContentAnimation::ID:
      return pushMessageContentAnimation::fetch(env, p);
    case pushMessageContentAudio::ID:
      return pushMessageContentAudio::fetch(env, p);
    case pushMessageContentContact::ID:
      return pushMessageContentContact::fetch(env, p);
    case pushMessageContentContactRegistered::ID:
      return pushMessageContentContactRegistered::fetch(env, p);
    case pushMessageContentDocument::ID:
      return pushMessageContentDocument::fetch(env, p);
    case pushMessageContentGame::ID:
      return pushMessageContentGame::fetch(env, p);
    case pushMessageContentGameScore::ID:
      return pushMessageContentGameScore::fetch(env, p);
    case pushMessageContentInvoice::ID:
      return pushMessageContentInvoice::fetch(env, p);
    case pushMessageContentLocation::ID:
      return pushMessageContentLocation::fetch(env, p);
    case pushMessageContentPaidMedia::ID:
      return pushMessageContentPaidMedia::fetch(env, p);
    case pushMessageContentPhoto::ID:
      return pushMessageContentPhoto::fetch(env, p);
    case pushMessageContentPoll::ID:
      return pushMessageContentPoll::fetch(env, p);
    case pushMessageContentPremiumGiftCode::ID:
      return pushMessageContentPremiumGiftCode::fetch(env, p);
    case pushMessageContentGiveaway::ID:
      return pushMessageContentGiveaway::fetch(env, p);
    case pushMessageContentGift::ID:
      return pushMessageContentGift::fetch(env, p);
    case pushMessageContentUpgradedGift::ID:
      return pushMessageContentUpgradedGift::fetch(env, p);
    case pushMessageContentScreenshotTaken::ID:
      return pushMessageContentScreenshotTaken::fetch(env, p);
    case pushMessageContentSticker::ID:
      return pushMessageContentSticker::fetch(env, p);
    case pushMessageContentStory::ID:
      return pushMessageContentStory::fetch(env, p);
    case pushMessageContentText::ID:
      return pushMessageContentText::fetch(env, p);
    case pushMessageContentVideo::ID:
      return pushMessageContentVideo::fetch(env, p);
    case pushMessageContentVideoNote::ID:
      return pushMessageContentVideoNote::fetch(env, p);
    case pushMessageContentVoiceNote::ID:
      return pushMessageContentVoiceNote::fetch(env, p);
    case pushMessageContentBasicGroupChatCreate::ID:
      return pushMessageContentBasicGroupChatCreate::fetch(env, p);
    case pushMessageContentVideoChatStarted::ID:
      return pushMessageContentVideoChatStarted::fetch(env, p);
    case pushMessageContentVideoChatEnded::ID:
      return pushMessageContentVideoChatEnded::fetch(env, p);
    case pushMessageContentInviteVideoChatParticipants::ID:
      return pushMessageContentInviteVideoChatParticipants::fetch(env, p);
    case pushMessageContentChatAddMembers::ID:
      return pushMessageContentChatAddMembers::fetch(env, p);
    case pushMessageContentChatChangePhoto::ID:
      return pushMessageContentChatChangePhoto::fetch(env, p);
    case pushMessageContentChatChangeTitle::ID:
      return pushMessageContentChatChangeTitle::fetch(env, p);
    case pushMessageContentChatSetBackground::ID:
      return pushMessageContentChatSetBackground::fetch(env, p);
    case pushMessageContentChatSetTheme::ID:
      return pushMessageContentChatSetTheme::fetch(env, p);
    case pushMessageContentChatDeleteMember::ID:
      return pushMessageContentChatDeleteMember::fetch(env, p);
    case pushMessageContentChatJoinByLink::ID:
      return pushMessageContentChatJoinByLink::fetch(env, p);
    case pushMessageContentChatJoinByRequest::ID:
      return pushMessageContentChatJoinByRequest::fetch(env, p);
    case pushMessageContentRecurringPayment::ID:
      return pushMessageContentRecurringPayment::fetch(env, p);
    case pushMessageContentSuggestProfilePhoto::ID:
      return pushMessageContentSuggestProfilePhoto::fetch(env, p);
    case pushMessageContentProximityAlertTriggered::ID:
      return pushMessageContentProximityAlertTriggered::fetch(env, p);
    case pushMessageContentMessageForwards::ID:
      return pushMessageContentMessageForwards::fetch(env, p);
    case pushMessageContentMediaAlbum::ID:
      return pushMessageContentMediaAlbum::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void PushMessageContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentHidden::Class;
jfieldID pushMessageContentHidden::is_pinned_fieldID;

pushMessageContentHidden::pushMessageContentHidden()
  : is_pinned_()
{}

pushMessageContentHidden::pushMessageContentHidden(bool is_pinned_)
  : is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentHidden::ID;

object_ptr<PushMessageContent> pushMessageContentHidden::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentHidden> res = make_object<pushMessageContentHidden>();
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentHidden::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentHidden::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentHidden");
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentHidden::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentHidden").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentAnimation::Class;
jfieldID pushMessageContentAnimation::animation_fieldID;
jfieldID pushMessageContentAnimation::caption_fieldID;
jfieldID pushMessageContentAnimation::is_pinned_fieldID;

pushMessageContentAnimation::pushMessageContentAnimation()
  : animation_()
  , caption_()
  , is_pinned_()
{}

pushMessageContentAnimation::pushMessageContentAnimation(object_ptr<animation> &&animation_, string const &caption_, bool is_pinned_)
  : animation_(std::move(animation_))
  , caption_(caption_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentAnimation::ID;

object_ptr<PushMessageContent> pushMessageContentAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentAnimation> res = make_object<pushMessageContentAnimation>();
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  res->caption_ = jni::fetch_string(env, p, res->caption_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentAnimation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, caption_); if (nextString) { env->SetObjectField(s, caption_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_field("caption", caption_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentAudio::Class;
jfieldID pushMessageContentAudio::audio_fieldID;
jfieldID pushMessageContentAudio::is_pinned_fieldID;

pushMessageContentAudio::pushMessageContentAudio()
  : audio_()
  , is_pinned_()
{}

pushMessageContentAudio::pushMessageContentAudio(object_ptr<audio> &&audio_, bool is_pinned_)
  : audio_(std::move(audio_))
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentAudio::ID;

object_ptr<PushMessageContent> pushMessageContentAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentAudio> res = make_object<pushMessageContentAudio>();
  res->audio_ = jni::fetch_tl_object<audio>(env, jni::fetch_object(env, p, res->audio_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentAudio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (audio_ != nullptr) { jobject next; audio_->store(env, next); if (next) { env->SetObjectField(s, audio_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentAudio");
    s.store_object_field("audio", static_cast<const BaseObject *>(audio_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentAudio").c_str());
    audio_fieldID = jni::get_field_id(env, Class, "audio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Audio;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentContact::Class;
jfieldID pushMessageContentContact::name_fieldID;
jfieldID pushMessageContentContact::is_pinned_fieldID;

pushMessageContentContact::pushMessageContentContact()
  : name_()
  , is_pinned_()
{}

pushMessageContentContact::pushMessageContentContact(string const &name_, bool is_pinned_)
  : name_(name_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentContact::ID;

object_ptr<PushMessageContent> pushMessageContentContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentContact> res = make_object<pushMessageContentContact>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentContact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentContact");
    s.store_field("name", name_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentContact").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentContactRegistered::Class;

pushMessageContentContactRegistered::pushMessageContentContactRegistered() {
}

const std::int32_t pushMessageContentContactRegistered::ID;

object_ptr<PushMessageContent> pushMessageContentContactRegistered::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentContactRegistered>();
}

void pushMessageContentContactRegistered::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentContactRegistered::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentContactRegistered");
    s.store_class_end();
  }
}

void pushMessageContentContactRegistered::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentContactRegistered").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentDocument::Class;
jfieldID pushMessageContentDocument::document_fieldID;
jfieldID pushMessageContentDocument::is_pinned_fieldID;

pushMessageContentDocument::pushMessageContentDocument()
  : document_()
  , is_pinned_()
{}

pushMessageContentDocument::pushMessageContentDocument(object_ptr<document> &&document_, bool is_pinned_)
  : document_(std::move(document_))
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentDocument::ID;

object_ptr<PushMessageContent> pushMessageContentDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentDocument> res = make_object<pushMessageContentDocument>();
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentDocument::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentDocument");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentDocument").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentGame::Class;
jfieldID pushMessageContentGame::title_fieldID;
jfieldID pushMessageContentGame::is_pinned_fieldID;

pushMessageContentGame::pushMessageContentGame()
  : title_()
  , is_pinned_()
{}

pushMessageContentGame::pushMessageContentGame(string const &title_, bool is_pinned_)
  : title_(title_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentGame::ID;

object_ptr<PushMessageContent> pushMessageContentGame::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentGame> res = make_object<pushMessageContentGame>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentGame::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentGame::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentGame");
    s.store_field("title", title_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentGame::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentGame").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentGameScore::Class;
jfieldID pushMessageContentGameScore::title_fieldID;
jfieldID pushMessageContentGameScore::score_fieldID;
jfieldID pushMessageContentGameScore::is_pinned_fieldID;

pushMessageContentGameScore::pushMessageContentGameScore()
  : title_()
  , score_()
  , is_pinned_()
{}

pushMessageContentGameScore::pushMessageContentGameScore(string const &title_, int32 score_, bool is_pinned_)
  : title_(title_)
  , score_(score_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentGameScore::ID;

object_ptr<PushMessageContent> pushMessageContentGameScore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentGameScore> res = make_object<pushMessageContentGameScore>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->score_ = env->GetIntField(p, res->score_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentGameScore::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, score_fieldID, score_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentGameScore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentGameScore");
    s.store_field("title", title_);
    s.store_field("score", score_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentGameScore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentGameScore").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    score_fieldID = jni::get_field_id(env, Class, "score", "I");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentInvoice::Class;
jfieldID pushMessageContentInvoice::price_fieldID;
jfieldID pushMessageContentInvoice::is_pinned_fieldID;

pushMessageContentInvoice::pushMessageContentInvoice()
  : price_()
  , is_pinned_()
{}

pushMessageContentInvoice::pushMessageContentInvoice(string const &price_, bool is_pinned_)
  : price_(price_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentInvoice::ID;

object_ptr<PushMessageContent> pushMessageContentInvoice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentInvoice> res = make_object<pushMessageContentInvoice>();
  res->price_ = jni::fetch_string(env, p, res->price_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentInvoice::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, price_); if (nextString) { env->SetObjectField(s, price_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentInvoice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentInvoice");
    s.store_field("price", price_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentInvoice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentInvoice").c_str());
    price_fieldID = jni::get_field_id(env, Class, "price", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentLocation::Class;
jfieldID pushMessageContentLocation::is_live_fieldID;
jfieldID pushMessageContentLocation::is_pinned_fieldID;

pushMessageContentLocation::pushMessageContentLocation()
  : is_live_()
  , is_pinned_()
{}

pushMessageContentLocation::pushMessageContentLocation(bool is_live_, bool is_pinned_)
  : is_live_(is_live_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentLocation::ID;

object_ptr<PushMessageContent> pushMessageContentLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentLocation> res = make_object<pushMessageContentLocation>();
  res->is_live_ = (env->GetBooleanField(p, res->is_live_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_live_fieldID, is_live_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentLocation");
    s.store_field("isLive", is_live_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentLocation").c_str());
    is_live_fieldID = jni::get_field_id(env, Class, "isLive", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentPaidMedia::Class;
jfieldID pushMessageContentPaidMedia::star_count_fieldID;
jfieldID pushMessageContentPaidMedia::is_pinned_fieldID;

pushMessageContentPaidMedia::pushMessageContentPaidMedia()
  : star_count_()
  , is_pinned_()
{}

pushMessageContentPaidMedia::pushMessageContentPaidMedia(int53 star_count_, bool is_pinned_)
  : star_count_(star_count_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentPaidMedia::ID;

object_ptr<PushMessageContent> pushMessageContentPaidMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentPaidMedia> res = make_object<pushMessageContentPaidMedia>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentPaidMedia::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentPaidMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentPaidMedia");
    s.store_field("starCount", star_count_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentPaidMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentPaidMedia").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentPhoto::Class;
jfieldID pushMessageContentPhoto::photo_fieldID;
jfieldID pushMessageContentPhoto::caption_fieldID;
jfieldID pushMessageContentPhoto::is_secret_fieldID;
jfieldID pushMessageContentPhoto::is_pinned_fieldID;

pushMessageContentPhoto::pushMessageContentPhoto()
  : photo_()
  , caption_()
  , is_secret_()
  , is_pinned_()
{}

pushMessageContentPhoto::pushMessageContentPhoto(object_ptr<photo> &&photo_, string const &caption_, bool is_secret_, bool is_pinned_)
  : photo_(std::move(photo_))
  , caption_(caption_)
  , is_secret_(is_secret_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentPhoto::ID;

object_ptr<PushMessageContent> pushMessageContentPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentPhoto> res = make_object<pushMessageContentPhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->caption_ = jni::fetch_string(env, p, res->caption_fieldID);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, caption_); if (nextString) { env->SetObjectField(s, caption_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("caption", caption_);
    s.store_field("isSecret", is_secret_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", "Ljava/lang/String;");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentPoll::Class;
jfieldID pushMessageContentPoll::question_fieldID;
jfieldID pushMessageContentPoll::is_regular_fieldID;
jfieldID pushMessageContentPoll::is_pinned_fieldID;

pushMessageContentPoll::pushMessageContentPoll()
  : question_()
  , is_regular_()
  , is_pinned_()
{}

pushMessageContentPoll::pushMessageContentPoll(string const &question_, bool is_regular_, bool is_pinned_)
  : question_(question_)
  , is_regular_(is_regular_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentPoll::ID;

object_ptr<PushMessageContent> pushMessageContentPoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentPoll> res = make_object<pushMessageContentPoll>();
  res->question_ = jni::fetch_string(env, p, res->question_fieldID);
  res->is_regular_ = (env->GetBooleanField(p, res->is_regular_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentPoll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, question_); if (nextString) { env->SetObjectField(s, question_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_regular_fieldID, is_regular_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentPoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentPoll");
    s.store_field("question", question_);
    s.store_field("isRegular", is_regular_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentPoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentPoll").c_str());
    question_fieldID = jni::get_field_id(env, Class, "question", "Ljava/lang/String;");
    is_regular_fieldID = jni::get_field_id(env, Class, "isRegular", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentPremiumGiftCode::Class;
jfieldID pushMessageContentPremiumGiftCode::month_count_fieldID;

pushMessageContentPremiumGiftCode::pushMessageContentPremiumGiftCode()
  : month_count_()
{}

pushMessageContentPremiumGiftCode::pushMessageContentPremiumGiftCode(int32 month_count_)
  : month_count_(month_count_)
{}

const std::int32_t pushMessageContentPremiumGiftCode::ID;

object_ptr<PushMessageContent> pushMessageContentPremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentPremiumGiftCode> res = make_object<pushMessageContentPremiumGiftCode>();
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  return std::move(res);
}

void pushMessageContentPremiumGiftCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, month_count_fieldID, month_count_);
}

void pushMessageContentPremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentPremiumGiftCode");
    s.store_field("monthCount", month_count_);
    s.store_class_end();
  }
}

void pushMessageContentPremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentPremiumGiftCode").c_str());
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentGiveaway::Class;
jfieldID pushMessageContentGiveaway::winner_count_fieldID;
jfieldID pushMessageContentGiveaway::prize_fieldID;
jfieldID pushMessageContentGiveaway::is_pinned_fieldID;

pushMessageContentGiveaway::pushMessageContentGiveaway()
  : winner_count_()
  , prize_()
  , is_pinned_()
{}

pushMessageContentGiveaway::pushMessageContentGiveaway(int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, bool is_pinned_)
  : winner_count_(winner_count_)
  , prize_(std::move(prize_))
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentGiveaway::ID;

object_ptr<PushMessageContent> pushMessageContentGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentGiveaway> res = make_object<pushMessageContentGiveaway>();
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->prize_ = jni::fetch_tl_object<GiveawayPrize>(env, jni::fetch_object(env, p, res->prize_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentGiveaway::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  if (prize_ != nullptr) { jobject next; prize_->store(env, next); if (next) { env->SetObjectField(s, prize_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentGiveaway");
    s.store_field("winnerCount", winner_count_);
    s.store_object_field("prize", static_cast<const BaseObject *>(prize_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentGiveaway").c_str());
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    prize_fieldID = jni::get_field_id(env, Class, "prize", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayPrize;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentGift::Class;
jfieldID pushMessageContentGift::star_count_fieldID;

pushMessageContentGift::pushMessageContentGift()
  : star_count_()
{}

pushMessageContentGift::pushMessageContentGift(int53 star_count_)
  : star_count_(star_count_)
{}

const std::int32_t pushMessageContentGift::ID;

object_ptr<PushMessageContent> pushMessageContentGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentGift> res = make_object<pushMessageContentGift>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void pushMessageContentGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void pushMessageContentGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentGift");
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void pushMessageContentGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentGift").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentUpgradedGift::Class;
jfieldID pushMessageContentUpgradedGift::is_upgrade_fieldID;

pushMessageContentUpgradedGift::pushMessageContentUpgradedGift()
  : is_upgrade_()
{}

pushMessageContentUpgradedGift::pushMessageContentUpgradedGift(bool is_upgrade_)
  : is_upgrade_(is_upgrade_)
{}

const std::int32_t pushMessageContentUpgradedGift::ID;

object_ptr<PushMessageContent> pushMessageContentUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentUpgradedGift> res = make_object<pushMessageContentUpgradedGift>();
  res->is_upgrade_ = (env->GetBooleanField(p, res->is_upgrade_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_upgrade_fieldID, is_upgrade_);
}

void pushMessageContentUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentUpgradedGift");
    s.store_field("isUpgrade", is_upgrade_);
    s.store_class_end();
  }
}

void pushMessageContentUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentUpgradedGift").c_str());
    is_upgrade_fieldID = jni::get_field_id(env, Class, "isUpgrade", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentScreenshotTaken::Class;

pushMessageContentScreenshotTaken::pushMessageContentScreenshotTaken() {
}

const std::int32_t pushMessageContentScreenshotTaken::ID;

object_ptr<PushMessageContent> pushMessageContentScreenshotTaken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentScreenshotTaken>();
}

void pushMessageContentScreenshotTaken::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentScreenshotTaken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentScreenshotTaken");
    s.store_class_end();
  }
}

void pushMessageContentScreenshotTaken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentScreenshotTaken").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentSticker::Class;
jfieldID pushMessageContentSticker::sticker_fieldID;
jfieldID pushMessageContentSticker::emoji_fieldID;
jfieldID pushMessageContentSticker::is_pinned_fieldID;

pushMessageContentSticker::pushMessageContentSticker()
  : sticker_()
  , emoji_()
  , is_pinned_()
{}

pushMessageContentSticker::pushMessageContentSticker(object_ptr<sticker> &&sticker_, string const &emoji_, bool is_pinned_)
  : sticker_(std::move(sticker_))
  , emoji_(emoji_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentSticker::ID;

object_ptr<PushMessageContent> pushMessageContentSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentSticker> res = make_object<pushMessageContentSticker>();
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentSticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("emoji", emoji_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentStory::Class;
jfieldID pushMessageContentStory::is_mention_fieldID;
jfieldID pushMessageContentStory::is_pinned_fieldID;

pushMessageContentStory::pushMessageContentStory()
  : is_mention_()
  , is_pinned_()
{}

pushMessageContentStory::pushMessageContentStory(bool is_mention_, bool is_pinned_)
  : is_mention_(is_mention_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentStory::ID;

object_ptr<PushMessageContent> pushMessageContentStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentStory> res = make_object<pushMessageContentStory>();
  res->is_mention_ = (env->GetBooleanField(p, res->is_mention_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_mention_fieldID, is_mention_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentStory");
    s.store_field("isMention", is_mention_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentStory").c_str());
    is_mention_fieldID = jni::get_field_id(env, Class, "isMention", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentText::Class;
jfieldID pushMessageContentText::text_fieldID;
jfieldID pushMessageContentText::is_pinned_fieldID;

pushMessageContentText::pushMessageContentText()
  : text_()
  , is_pinned_()
{}

pushMessageContentText::pushMessageContentText(string const &text_, bool is_pinned_)
  : text_(text_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentText::ID;

object_ptr<PushMessageContent> pushMessageContentText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentText> res = make_object<pushMessageContentText>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentText");
    s.store_field("text", text_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentVideo::Class;
jfieldID pushMessageContentVideo::video_fieldID;
jfieldID pushMessageContentVideo::caption_fieldID;
jfieldID pushMessageContentVideo::is_secret_fieldID;
jfieldID pushMessageContentVideo::is_pinned_fieldID;

pushMessageContentVideo::pushMessageContentVideo()
  : video_()
  , caption_()
  , is_secret_()
  , is_pinned_()
{}

pushMessageContentVideo::pushMessageContentVideo(object_ptr<video> &&video_, string const &caption_, bool is_secret_, bool is_pinned_)
  : video_(std::move(video_))
  , caption_(caption_)
  , is_secret_(is_secret_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentVideo::ID;

object_ptr<PushMessageContent> pushMessageContentVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentVideo> res = make_object<pushMessageContentVideo>();
  res->video_ = jni::fetch_tl_object<video>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->caption_ = jni::fetch_string(env, p, res->caption_fieldID);
  res->is_secret_ = (env->GetBooleanField(p, res->is_secret_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, caption_); if (nextString) { env->SetObjectField(s, caption_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_secret_fieldID, is_secret_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_field("caption", caption_);
    s.store_field("isSecret", is_secret_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Video;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", "Ljava/lang/String;");
    is_secret_fieldID = jni::get_field_id(env, Class, "isSecret", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentVideoNote::Class;
jfieldID pushMessageContentVideoNote::video_note_fieldID;
jfieldID pushMessageContentVideoNote::is_pinned_fieldID;

pushMessageContentVideoNote::pushMessageContentVideoNote()
  : video_note_()
  , is_pinned_()
{}

pushMessageContentVideoNote::pushMessageContentVideoNote(object_ptr<videoNote> &&video_note_, bool is_pinned_)
  : video_note_(std::move(video_note_))
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentVideoNote::ID;

object_ptr<PushMessageContent> pushMessageContentVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentVideoNote> res = make_object<pushMessageContentVideoNote>();
  res->video_note_ = jni::fetch_tl_object<videoNote>(env, jni::fetch_object(env, p, res->video_note_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentVideoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_note_ != nullptr) { jobject next; video_note_->store(env, next); if (next) { env->SetObjectField(s, video_note_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentVideoNote");
    s.store_object_field("videoNote", static_cast<const BaseObject *>(video_note_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentVideoNote").c_str());
    video_note_fieldID = jni::get_field_id(env, Class, "videoNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VideoNote;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentVoiceNote::Class;
jfieldID pushMessageContentVoiceNote::voice_note_fieldID;
jfieldID pushMessageContentVoiceNote::is_pinned_fieldID;

pushMessageContentVoiceNote::pushMessageContentVoiceNote()
  : voice_note_()
  , is_pinned_()
{}

pushMessageContentVoiceNote::pushMessageContentVoiceNote(object_ptr<voiceNote> &&voice_note_, bool is_pinned_)
  : voice_note_(std::move(voice_note_))
  , is_pinned_(is_pinned_)
{}

const std::int32_t pushMessageContentVoiceNote::ID;

object_ptr<PushMessageContent> pushMessageContentVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentVoiceNote> res = make_object<pushMessageContentVoiceNote>();
  res->voice_note_ = jni::fetch_tl_object<voiceNote>(env, jni::fetch_object(env, p, res->voice_note_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentVoiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (voice_note_ != nullptr) { jobject next; voice_note_->store(env, next); if (next) { env->SetObjectField(s, voice_note_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void pushMessageContentVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentVoiceNote");
    s.store_object_field("voiceNote", static_cast<const BaseObject *>(voice_note_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void pushMessageContentVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentVoiceNote").c_str());
    voice_note_fieldID = jni::get_field_id(env, Class, "voiceNote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VoiceNote;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentBasicGroupChatCreate::Class;

pushMessageContentBasicGroupChatCreate::pushMessageContentBasicGroupChatCreate() {
}

const std::int32_t pushMessageContentBasicGroupChatCreate::ID;

object_ptr<PushMessageContent> pushMessageContentBasicGroupChatCreate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentBasicGroupChatCreate>();
}

void pushMessageContentBasicGroupChatCreate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentBasicGroupChatCreate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentBasicGroupChatCreate");
    s.store_class_end();
  }
}

void pushMessageContentBasicGroupChatCreate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentBasicGroupChatCreate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentVideoChatStarted::Class;

pushMessageContentVideoChatStarted::pushMessageContentVideoChatStarted() {
}

const std::int32_t pushMessageContentVideoChatStarted::ID;

object_ptr<PushMessageContent> pushMessageContentVideoChatStarted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentVideoChatStarted>();
}

void pushMessageContentVideoChatStarted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentVideoChatStarted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentVideoChatStarted");
    s.store_class_end();
  }
}

void pushMessageContentVideoChatStarted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentVideoChatStarted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentVideoChatEnded::Class;

pushMessageContentVideoChatEnded::pushMessageContentVideoChatEnded() {
}

const std::int32_t pushMessageContentVideoChatEnded::ID;

object_ptr<PushMessageContent> pushMessageContentVideoChatEnded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentVideoChatEnded>();
}

void pushMessageContentVideoChatEnded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentVideoChatEnded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentVideoChatEnded");
    s.store_class_end();
  }
}

void pushMessageContentVideoChatEnded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentVideoChatEnded").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentInviteVideoChatParticipants::Class;
jfieldID pushMessageContentInviteVideoChatParticipants::is_current_user_fieldID;

pushMessageContentInviteVideoChatParticipants::pushMessageContentInviteVideoChatParticipants()
  : is_current_user_()
{}

pushMessageContentInviteVideoChatParticipants::pushMessageContentInviteVideoChatParticipants(bool is_current_user_)
  : is_current_user_(is_current_user_)
{}

const std::int32_t pushMessageContentInviteVideoChatParticipants::ID;

object_ptr<PushMessageContent> pushMessageContentInviteVideoChatParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentInviteVideoChatParticipants> res = make_object<pushMessageContentInviteVideoChatParticipants>();
  res->is_current_user_ = (env->GetBooleanField(p, res->is_current_user_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentInviteVideoChatParticipants::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_current_user_fieldID, is_current_user_);
}

void pushMessageContentInviteVideoChatParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentInviteVideoChatParticipants");
    s.store_field("isCurrentUser", is_current_user_);
    s.store_class_end();
  }
}

void pushMessageContentInviteVideoChatParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentInviteVideoChatParticipants").c_str());
    is_current_user_fieldID = jni::get_field_id(env, Class, "isCurrentUser", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatAddMembers::Class;
jfieldID pushMessageContentChatAddMembers::member_name_fieldID;
jfieldID pushMessageContentChatAddMembers::is_current_user_fieldID;
jfieldID pushMessageContentChatAddMembers::is_returned_fieldID;

pushMessageContentChatAddMembers::pushMessageContentChatAddMembers()
  : member_name_()
  , is_current_user_()
  , is_returned_()
{}

pushMessageContentChatAddMembers::pushMessageContentChatAddMembers(string const &member_name_, bool is_current_user_, bool is_returned_)
  : member_name_(member_name_)
  , is_current_user_(is_current_user_)
  , is_returned_(is_returned_)
{}

const std::int32_t pushMessageContentChatAddMembers::ID;

object_ptr<PushMessageContent> pushMessageContentChatAddMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentChatAddMembers> res = make_object<pushMessageContentChatAddMembers>();
  res->member_name_ = jni::fetch_string(env, p, res->member_name_fieldID);
  res->is_current_user_ = (env->GetBooleanField(p, res->is_current_user_fieldID) != 0);
  res->is_returned_ = (env->GetBooleanField(p, res->is_returned_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentChatAddMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, member_name_); if (nextString) { env->SetObjectField(s, member_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_current_user_fieldID, is_current_user_);
  env->SetBooleanField(s, is_returned_fieldID, is_returned_);
}

void pushMessageContentChatAddMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatAddMembers");
    s.store_field("memberName", member_name_);
    s.store_field("isCurrentUser", is_current_user_);
    s.store_field("isReturned", is_returned_);
    s.store_class_end();
  }
}

void pushMessageContentChatAddMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatAddMembers").c_str());
    member_name_fieldID = jni::get_field_id(env, Class, "memberName", "Ljava/lang/String;");
    is_current_user_fieldID = jni::get_field_id(env, Class, "isCurrentUser", "Z");
    is_returned_fieldID = jni::get_field_id(env, Class, "isReturned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatChangePhoto::Class;

pushMessageContentChatChangePhoto::pushMessageContentChatChangePhoto() {
}

const std::int32_t pushMessageContentChatChangePhoto::ID;

object_ptr<PushMessageContent> pushMessageContentChatChangePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentChatChangePhoto>();
}

void pushMessageContentChatChangePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentChatChangePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatChangePhoto");
    s.store_class_end();
  }
}

void pushMessageContentChatChangePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatChangePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatChangeTitle::Class;
jfieldID pushMessageContentChatChangeTitle::title_fieldID;

pushMessageContentChatChangeTitle::pushMessageContentChatChangeTitle()
  : title_()
{}

pushMessageContentChatChangeTitle::pushMessageContentChatChangeTitle(string const &title_)
  : title_(title_)
{}

const std::int32_t pushMessageContentChatChangeTitle::ID;

object_ptr<PushMessageContent> pushMessageContentChatChangeTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentChatChangeTitle> res = make_object<pushMessageContentChatChangeTitle>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void pushMessageContentChatChangeTitle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pushMessageContentChatChangeTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatChangeTitle");
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void pushMessageContentChatChangeTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatChangeTitle").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatSetBackground::Class;
jfieldID pushMessageContentChatSetBackground::is_same_fieldID;

pushMessageContentChatSetBackground::pushMessageContentChatSetBackground()
  : is_same_()
{}

pushMessageContentChatSetBackground::pushMessageContentChatSetBackground(bool is_same_)
  : is_same_(is_same_)
{}

const std::int32_t pushMessageContentChatSetBackground::ID;

object_ptr<PushMessageContent> pushMessageContentChatSetBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentChatSetBackground> res = make_object<pushMessageContentChatSetBackground>();
  res->is_same_ = (env->GetBooleanField(p, res->is_same_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentChatSetBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_same_fieldID, is_same_);
}

void pushMessageContentChatSetBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatSetBackground");
    s.store_field("isSame", is_same_);
    s.store_class_end();
  }
}

void pushMessageContentChatSetBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatSetBackground").c_str());
    is_same_fieldID = jni::get_field_id(env, Class, "isSame", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatSetTheme::Class;
jfieldID pushMessageContentChatSetTheme::theme_name_fieldID;

pushMessageContentChatSetTheme::pushMessageContentChatSetTheme()
  : theme_name_()
{}

pushMessageContentChatSetTheme::pushMessageContentChatSetTheme(string const &theme_name_)
  : theme_name_(theme_name_)
{}

const std::int32_t pushMessageContentChatSetTheme::ID;

object_ptr<PushMessageContent> pushMessageContentChatSetTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentChatSetTheme> res = make_object<pushMessageContentChatSetTheme>();
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return std::move(res);
}

void pushMessageContentChatSetTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pushMessageContentChatSetTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatSetTheme");
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

void pushMessageContentChatSetTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatSetTheme").c_str());
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatDeleteMember::Class;
jfieldID pushMessageContentChatDeleteMember::member_name_fieldID;
jfieldID pushMessageContentChatDeleteMember::is_current_user_fieldID;
jfieldID pushMessageContentChatDeleteMember::is_left_fieldID;

pushMessageContentChatDeleteMember::pushMessageContentChatDeleteMember()
  : member_name_()
  , is_current_user_()
  , is_left_()
{}

pushMessageContentChatDeleteMember::pushMessageContentChatDeleteMember(string const &member_name_, bool is_current_user_, bool is_left_)
  : member_name_(member_name_)
  , is_current_user_(is_current_user_)
  , is_left_(is_left_)
{}

const std::int32_t pushMessageContentChatDeleteMember::ID;

object_ptr<PushMessageContent> pushMessageContentChatDeleteMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentChatDeleteMember> res = make_object<pushMessageContentChatDeleteMember>();
  res->member_name_ = jni::fetch_string(env, p, res->member_name_fieldID);
  res->is_current_user_ = (env->GetBooleanField(p, res->is_current_user_fieldID) != 0);
  res->is_left_ = (env->GetBooleanField(p, res->is_left_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentChatDeleteMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, member_name_); if (nextString) { env->SetObjectField(s, member_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_current_user_fieldID, is_current_user_);
  env->SetBooleanField(s, is_left_fieldID, is_left_);
}

void pushMessageContentChatDeleteMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatDeleteMember");
    s.store_field("memberName", member_name_);
    s.store_field("isCurrentUser", is_current_user_);
    s.store_field("isLeft", is_left_);
    s.store_class_end();
  }
}

void pushMessageContentChatDeleteMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatDeleteMember").c_str());
    member_name_fieldID = jni::get_field_id(env, Class, "memberName", "Ljava/lang/String;");
    is_current_user_fieldID = jni::get_field_id(env, Class, "isCurrentUser", "Z");
    is_left_fieldID = jni::get_field_id(env, Class, "isLeft", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatJoinByLink::Class;

pushMessageContentChatJoinByLink::pushMessageContentChatJoinByLink() {
}

const std::int32_t pushMessageContentChatJoinByLink::ID;

object_ptr<PushMessageContent> pushMessageContentChatJoinByLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentChatJoinByLink>();
}

void pushMessageContentChatJoinByLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentChatJoinByLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatJoinByLink");
    s.store_class_end();
  }
}

void pushMessageContentChatJoinByLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatJoinByLink").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentChatJoinByRequest::Class;

pushMessageContentChatJoinByRequest::pushMessageContentChatJoinByRequest() {
}

const std::int32_t pushMessageContentChatJoinByRequest::ID;

object_ptr<PushMessageContent> pushMessageContentChatJoinByRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentChatJoinByRequest>();
}

void pushMessageContentChatJoinByRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentChatJoinByRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentChatJoinByRequest");
    s.store_class_end();
  }
}

void pushMessageContentChatJoinByRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentChatJoinByRequest").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentRecurringPayment::Class;
jfieldID pushMessageContentRecurringPayment::amount_fieldID;

pushMessageContentRecurringPayment::pushMessageContentRecurringPayment()
  : amount_()
{}

pushMessageContentRecurringPayment::pushMessageContentRecurringPayment(string const &amount_)
  : amount_(amount_)
{}

const std::int32_t pushMessageContentRecurringPayment::ID;

object_ptr<PushMessageContent> pushMessageContentRecurringPayment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentRecurringPayment> res = make_object<pushMessageContentRecurringPayment>();
  res->amount_ = jni::fetch_string(env, p, res->amount_fieldID);
  return std::move(res);
}

void pushMessageContentRecurringPayment::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, amount_); if (nextString) { env->SetObjectField(s, amount_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void pushMessageContentRecurringPayment::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentRecurringPayment");
    s.store_field("amount", amount_);
    s.store_class_end();
  }
}

void pushMessageContentRecurringPayment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentRecurringPayment").c_str());
    amount_fieldID = jni::get_field_id(env, Class, "amount", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentSuggestProfilePhoto::Class;

pushMessageContentSuggestProfilePhoto::pushMessageContentSuggestProfilePhoto() {
}

const std::int32_t pushMessageContentSuggestProfilePhoto::ID;

object_ptr<PushMessageContent> pushMessageContentSuggestProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<pushMessageContentSuggestProfilePhoto>();
}

void pushMessageContentSuggestProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void pushMessageContentSuggestProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentSuggestProfilePhoto");
    s.store_class_end();
  }
}

void pushMessageContentSuggestProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentSuggestProfilePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentProximityAlertTriggered::Class;
jfieldID pushMessageContentProximityAlertTriggered::distance_fieldID;

pushMessageContentProximityAlertTriggered::pushMessageContentProximityAlertTriggered()
  : distance_()
{}

pushMessageContentProximityAlertTriggered::pushMessageContentProximityAlertTriggered(int32 distance_)
  : distance_(distance_)
{}

const std::int32_t pushMessageContentProximityAlertTriggered::ID;

object_ptr<PushMessageContent> pushMessageContentProximityAlertTriggered::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentProximityAlertTriggered> res = make_object<pushMessageContentProximityAlertTriggered>();
  res->distance_ = env->GetIntField(p, res->distance_fieldID);
  return std::move(res);
}

void pushMessageContentProximityAlertTriggered::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, distance_fieldID, distance_);
}

void pushMessageContentProximityAlertTriggered::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentProximityAlertTriggered");
    s.store_field("distance", distance_);
    s.store_class_end();
  }
}

void pushMessageContentProximityAlertTriggered::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentProximityAlertTriggered").c_str());
    distance_fieldID = jni::get_field_id(env, Class, "distance", "I");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentMessageForwards::Class;
jfieldID pushMessageContentMessageForwards::total_count_fieldID;

pushMessageContentMessageForwards::pushMessageContentMessageForwards()
  : total_count_()
{}

pushMessageContentMessageForwards::pushMessageContentMessageForwards(int32 total_count_)
  : total_count_(total_count_)
{}

const std::int32_t pushMessageContentMessageForwards::ID;

object_ptr<PushMessageContent> pushMessageContentMessageForwards::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentMessageForwards> res = make_object<pushMessageContentMessageForwards>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  return std::move(res);
}

void pushMessageContentMessageForwards::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
}

void pushMessageContentMessageForwards::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentMessageForwards");
    s.store_field("totalCount", total_count_);
    s.store_class_end();
  }
}

void pushMessageContentMessageForwards::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentMessageForwards").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass pushMessageContentMediaAlbum::Class;
jfieldID pushMessageContentMediaAlbum::total_count_fieldID;
jfieldID pushMessageContentMediaAlbum::has_photos_fieldID;
jfieldID pushMessageContentMediaAlbum::has_videos_fieldID;
jfieldID pushMessageContentMediaAlbum::has_audios_fieldID;
jfieldID pushMessageContentMediaAlbum::has_documents_fieldID;

pushMessageContentMediaAlbum::pushMessageContentMediaAlbum()
  : total_count_()
  , has_photos_()
  , has_videos_()
  , has_audios_()
  , has_documents_()
{}

pushMessageContentMediaAlbum::pushMessageContentMediaAlbum(int32 total_count_, bool has_photos_, bool has_videos_, bool has_audios_, bool has_documents_)
  : total_count_(total_count_)
  , has_photos_(has_photos_)
  , has_videos_(has_videos_)
  , has_audios_(has_audios_)
  , has_documents_(has_documents_)
{}

const std::int32_t pushMessageContentMediaAlbum::ID;

object_ptr<PushMessageContent> pushMessageContentMediaAlbum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushMessageContentMediaAlbum> res = make_object<pushMessageContentMediaAlbum>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->has_photos_ = (env->GetBooleanField(p, res->has_photos_fieldID) != 0);
  res->has_videos_ = (env->GetBooleanField(p, res->has_videos_fieldID) != 0);
  res->has_audios_ = (env->GetBooleanField(p, res->has_audios_fieldID) != 0);
  res->has_documents_ = (env->GetBooleanField(p, res->has_documents_fieldID) != 0);
  return std::move(res);
}

void pushMessageContentMediaAlbum::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetBooleanField(s, has_photos_fieldID, has_photos_);
  env->SetBooleanField(s, has_videos_fieldID, has_videos_);
  env->SetBooleanField(s, has_audios_fieldID, has_audios_);
  env->SetBooleanField(s, has_documents_fieldID, has_documents_);
}

void pushMessageContentMediaAlbum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushMessageContentMediaAlbum");
    s.store_field("totalCount", total_count_);
    s.store_field("hasPhotos", has_photos_);
    s.store_field("hasVideos", has_videos_);
    s.store_field("hasAudios", has_audios_);
    s.store_field("hasDocuments", has_documents_);
    s.store_class_end();
  }
}

void pushMessageContentMediaAlbum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushMessageContentMediaAlbum").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    has_photos_fieldID = jni::get_field_id(env, Class, "hasPhotos", "Z");
    has_videos_fieldID = jni::get_field_id(env, Class, "hasVideos", "Z");
    has_audios_fieldID = jni::get_field_id(env, Class, "hasAudios", "Z");
    has_documents_fieldID = jni::get_field_id(env, Class, "hasDocuments", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pushReceiverId::Class;
jfieldID pushReceiverId::id_fieldID;

pushReceiverId::pushReceiverId()
  : id_()
{}

pushReceiverId::pushReceiverId(int64 id_)
  : id_(id_)
{}

const std::int32_t pushReceiverId::ID;

object_ptr<pushReceiverId> pushReceiverId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pushReceiverId> res = make_object<pushReceiverId>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  return res;
}

void pushReceiverId::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
}

void pushReceiverId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PushReceiverId");
    s.store_field("id", id_);
    s.store_class_end();
  }
}

void pushReceiverId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PushReceiverId").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    return true;
  }();
  (void)is_inited;
}

jclass quickReplyMessage::Class;
jfieldID quickReplyMessage::id_fieldID;
jfieldID quickReplyMessage::sending_state_fieldID;
jfieldID quickReplyMessage::can_be_edited_fieldID;
jfieldID quickReplyMessage::reply_to_message_id_fieldID;
jfieldID quickReplyMessage::via_bot_user_id_fieldID;
jfieldID quickReplyMessage::media_album_id_fieldID;
jfieldID quickReplyMessage::content_fieldID;
jfieldID quickReplyMessage::reply_markup_fieldID;

quickReplyMessage::quickReplyMessage()
  : id_()
  , sending_state_()
  , can_be_edited_()
  , reply_to_message_id_()
  , via_bot_user_id_()
  , media_album_id_()
  , content_()
  , reply_markup_()
{}

quickReplyMessage::quickReplyMessage(int53 id_, object_ptr<MessageSendingState> &&sending_state_, bool can_be_edited_, int53 reply_to_message_id_, int53 via_bot_user_id_, int64 media_album_id_, object_ptr<MessageContent> &&content_, object_ptr<ReplyMarkup> &&reply_markup_)
  : id_(id_)
  , sending_state_(std::move(sending_state_))
  , can_be_edited_(can_be_edited_)
  , reply_to_message_id_(reply_to_message_id_)
  , via_bot_user_id_(via_bot_user_id_)
  , media_album_id_(media_album_id_)
  , content_(std::move(content_))
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t quickReplyMessage::ID;

object_ptr<quickReplyMessage> quickReplyMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<quickReplyMessage> res = make_object<quickReplyMessage>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sending_state_ = jni::fetch_tl_object<MessageSendingState>(env, jni::fetch_object(env, p, res->sending_state_fieldID));
  res->can_be_edited_ = (env->GetBooleanField(p, res->can_be_edited_fieldID) != 0);
  res->reply_to_message_id_ = env->GetLongField(p, res->reply_to_message_id_fieldID);
  res->via_bot_user_id_ = env->GetLongField(p, res->via_bot_user_id_fieldID);
  res->media_album_id_ = env->GetLongField(p, res->media_album_id_fieldID);
  res->content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void quickReplyMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (sending_state_ != nullptr) { jobject next; sending_state_->store(env, next); if (next) { env->SetObjectField(s, sending_state_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_be_edited_fieldID, can_be_edited_);
  env->SetLongField(s, reply_to_message_id_fieldID, reply_to_message_id_);
  env->SetLongField(s, via_bot_user_id_fieldID, via_bot_user_id_);
  env->SetLongField(s, media_album_id_fieldID, media_album_id_);
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
  if (reply_markup_ != nullptr) { jobject next; reply_markup_->store(env, next); if (next) { env->SetObjectField(s, reply_markup_fieldID, next); env->DeleteLocalRef(next); } }
}

void quickReplyMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "QuickReplyMessage");
    s.store_field("id", id_);
    s.store_object_field("sendingState", static_cast<const BaseObject *>(sending_state_.get()));
    s.store_field("canBeEdited", can_be_edited_);
    s.store_field("replyToMessageId", reply_to_message_id_);
    s.store_field("viaBotUserId", via_bot_user_id_);
    s.store_field("mediaAlbumId", media_album_id_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

void quickReplyMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$QuickReplyMessage").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sending_state_fieldID = jni::get_field_id(env, Class, "sendingState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendingState;").c_str());
    can_be_edited_fieldID = jni::get_field_id(env, Class, "canBeEdited", "Z");
    reply_to_message_id_fieldID = jni::get_field_id(env, Class, "replyToMessageId", "J");
    via_bot_user_id_fieldID = jni::get_field_id(env, Class, "viaBotUserId", "J");
    media_album_id_fieldID = jni::get_field_id(env, Class, "mediaAlbumId", "J");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass quickReplyMessages::Class;
jfieldID quickReplyMessages::messages_fieldID;

quickReplyMessages::quickReplyMessages()
  : messages_()
{}

quickReplyMessages::quickReplyMessages(array<object_ptr<quickReplyMessage>> &&messages_)
  : messages_(std::move(messages_))
{}

const std::int32_t quickReplyMessages::ID;

object_ptr<quickReplyMessages> quickReplyMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<quickReplyMessages> res = make_object<quickReplyMessages>();
  res->messages_ = jni::FetchVector<quickReplyMessage>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  return res;
}

void quickReplyMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void quickReplyMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "QuickReplyMessages");
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void quickReplyMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$QuickReplyMessages").c_str());
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$QuickReplyMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass quickReplyShortcut::Class;
jfieldID quickReplyShortcut::id_fieldID;
jfieldID quickReplyShortcut::name_fieldID;
jfieldID quickReplyShortcut::first_message_fieldID;
jfieldID quickReplyShortcut::message_count_fieldID;

quickReplyShortcut::quickReplyShortcut()
  : id_()
  , name_()
  , first_message_()
  , message_count_()
{}

quickReplyShortcut::quickReplyShortcut(int32 id_, string const &name_, object_ptr<quickReplyMessage> &&first_message_, int32 message_count_)
  : id_(id_)
  , name_(name_)
  , first_message_(std::move(first_message_))
  , message_count_(message_count_)
{}

const std::int32_t quickReplyShortcut::ID;

object_ptr<quickReplyShortcut> quickReplyShortcut::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<quickReplyShortcut> res = make_object<quickReplyShortcut>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->first_message_ = jni::fetch_tl_object<quickReplyMessage>(env, jni::fetch_object(env, p, res->first_message_fieldID));
  res->message_count_ = env->GetIntField(p, res->message_count_fieldID);
  return res;
}

void quickReplyShortcut::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (first_message_ != nullptr) { jobject next; first_message_->store(env, next); if (next) { env->SetObjectField(s, first_message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, message_count_fieldID, message_count_);
}

void quickReplyShortcut::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "QuickReplyShortcut");
    s.store_field("id", id_);
    s.store_field("name", name_);
    s.store_object_field("firstMessage", static_cast<const BaseObject *>(first_message_.get()));
    s.store_field("messageCount", message_count_);
    s.store_class_end();
  }
}

void quickReplyShortcut::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$QuickReplyShortcut").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    first_message_fieldID = jni::get_field_id(env, Class, "firstMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$QuickReplyMessage;").c_str());
    message_count_fieldID = jni::get_field_id(env, Class, "messageCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass reactionNotificationSettings::Class;
jfieldID reactionNotificationSettings::message_reaction_source_fieldID;
jfieldID reactionNotificationSettings::story_reaction_source_fieldID;
jfieldID reactionNotificationSettings::sound_id_fieldID;
jfieldID reactionNotificationSettings::show_preview_fieldID;

reactionNotificationSettings::reactionNotificationSettings()
  : message_reaction_source_()
  , story_reaction_source_()
  , sound_id_()
  , show_preview_()
{}

reactionNotificationSettings::reactionNotificationSettings(object_ptr<ReactionNotificationSource> &&message_reaction_source_, object_ptr<ReactionNotificationSource> &&story_reaction_source_, int64 sound_id_, bool show_preview_)
  : message_reaction_source_(std::move(message_reaction_source_))
  , story_reaction_source_(std::move(story_reaction_source_))
  , sound_id_(sound_id_)
  , show_preview_(show_preview_)
{}

const std::int32_t reactionNotificationSettings::ID;

object_ptr<reactionNotificationSettings> reactionNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reactionNotificationSettings> res = make_object<reactionNotificationSettings>();
  res->message_reaction_source_ = jni::fetch_tl_object<ReactionNotificationSource>(env, jni::fetch_object(env, p, res->message_reaction_source_fieldID));
  res->story_reaction_source_ = jni::fetch_tl_object<ReactionNotificationSource>(env, jni::fetch_object(env, p, res->story_reaction_source_fieldID));
  res->sound_id_ = env->GetLongField(p, res->sound_id_fieldID);
  res->show_preview_ = (env->GetBooleanField(p, res->show_preview_fieldID) != 0);
  return res;
}

void reactionNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_reaction_source_ != nullptr) { jobject next; message_reaction_source_->store(env, next); if (next) { env->SetObjectField(s, message_reaction_source_fieldID, next); env->DeleteLocalRef(next); } }
  if (story_reaction_source_ != nullptr) { jobject next; story_reaction_source_->store(env, next); if (next) { env->SetObjectField(s, story_reaction_source_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, sound_id_fieldID, sound_id_);
  env->SetBooleanField(s, show_preview_fieldID, show_preview_);
}

void reactionNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionNotificationSettings");
    s.store_object_field("messageReactionSource", static_cast<const BaseObject *>(message_reaction_source_.get()));
    s.store_object_field("storyReactionSource", static_cast<const BaseObject *>(story_reaction_source_.get()));
    s.store_field("soundId", sound_id_);
    s.store_field("showPreview", show_preview_);
    s.store_class_end();
  }
}

void reactionNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionNotificationSettings").c_str());
    message_reaction_source_fieldID = jni::get_field_id(env, Class, "messageReactionSource", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionNotificationSource;").c_str());
    story_reaction_source_fieldID = jni::get_field_id(env, Class, "storyReactionSource", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionNotificationSource;").c_str());
    sound_id_fieldID = jni::get_field_id(env, Class, "soundId", "J");
    show_preview_fieldID = jni::get_field_id(env, Class, "showPreview", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass ReactionNotificationSource::Class;

object_ptr<ReactionNotificationSource> ReactionNotificationSource::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reactionNotificationSourceNone::ID:
      return reactionNotificationSourceNone::fetch(env, p);
    case reactionNotificationSourceContacts::ID:
      return reactionNotificationSourceContacts::fetch(env, p);
    case reactionNotificationSourceAll::ID:
      return reactionNotificationSourceAll::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReactionNotificationSource::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionNotificationSource").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionNotificationSourceNone::Class;

reactionNotificationSourceNone::reactionNotificationSourceNone() {
}

const std::int32_t reactionNotificationSourceNone::ID;

object_ptr<ReactionNotificationSource> reactionNotificationSourceNone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionNotificationSourceNone>();
}

void reactionNotificationSourceNone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionNotificationSourceNone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionNotificationSourceNone");
    s.store_class_end();
  }
}

void reactionNotificationSourceNone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionNotificationSourceNone").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionNotificationSourceContacts::Class;

reactionNotificationSourceContacts::reactionNotificationSourceContacts() {
}

const std::int32_t reactionNotificationSourceContacts::ID;

object_ptr<ReactionNotificationSource> reactionNotificationSourceContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionNotificationSourceContacts>();
}

void reactionNotificationSourceContacts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionNotificationSourceContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionNotificationSourceContacts");
    s.store_class_end();
  }
}

void reactionNotificationSourceContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionNotificationSourceContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionNotificationSourceAll::Class;

reactionNotificationSourceAll::reactionNotificationSourceAll() {
}

const std::int32_t reactionNotificationSourceAll::ID;

object_ptr<ReactionNotificationSource> reactionNotificationSourceAll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionNotificationSourceAll>();
}

void reactionNotificationSourceAll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionNotificationSourceAll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionNotificationSourceAll");
    s.store_class_end();
  }
}

void reactionNotificationSourceAll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionNotificationSourceAll").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ReactionType::Class;

object_ptr<ReactionType> ReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reactionTypeEmoji::ID:
      return reactionTypeEmoji::fetch(env, p);
    case reactionTypeCustomEmoji::ID:
      return reactionTypeCustomEmoji::fetch(env, p);
    case reactionTypePaid::ID:
      return reactionTypePaid::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionTypeEmoji::Class;
jfieldID reactionTypeEmoji::emoji_fieldID;

reactionTypeEmoji::reactionTypeEmoji()
  : emoji_()
{}

reactionTypeEmoji::reactionTypeEmoji(string const &emoji_)
  : emoji_(emoji_)
{}

const std::int32_t reactionTypeEmoji::ID;

object_ptr<ReactionType> reactionTypeEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reactionTypeEmoji> res = make_object<reactionTypeEmoji>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return std::move(res);
}

void reactionTypeEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void reactionTypeEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionTypeEmoji");
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

void reactionTypeEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionTypeEmoji").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reactionTypeCustomEmoji::Class;
jfieldID reactionTypeCustomEmoji::custom_emoji_id_fieldID;

reactionTypeCustomEmoji::reactionTypeCustomEmoji()
  : custom_emoji_id_()
{}

reactionTypeCustomEmoji::reactionTypeCustomEmoji(int64 custom_emoji_id_)
  : custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t reactionTypeCustomEmoji::ID;

object_ptr<ReactionType> reactionTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reactionTypeCustomEmoji> res = make_object<reactionTypeCustomEmoji>();
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return std::move(res);
}

void reactionTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
}

void reactionTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionTypeCustomEmoji");
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

void reactionTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionTypeCustomEmoji").c_str());
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass reactionTypePaid::Class;

reactionTypePaid::reactionTypePaid() {
}

const std::int32_t reactionTypePaid::ID;

object_ptr<ReactionType> reactionTypePaid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionTypePaid>();
}

void reactionTypePaid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionTypePaid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionTypePaid");
    s.store_class_end();
  }
}

void reactionTypePaid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionTypePaid").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ReactionUnavailabilityReason::Class;

object_ptr<ReactionUnavailabilityReason> ReactionUnavailabilityReason::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reactionUnavailabilityReasonAnonymousAdministrator::ID:
      return reactionUnavailabilityReasonAnonymousAdministrator::fetch(env, p);
    case reactionUnavailabilityReasonGuest::ID:
      return reactionUnavailabilityReasonGuest::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReactionUnavailabilityReason::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionUnavailabilityReason").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionUnavailabilityReasonAnonymousAdministrator::Class;

reactionUnavailabilityReasonAnonymousAdministrator::reactionUnavailabilityReasonAnonymousAdministrator() {
}

const std::int32_t reactionUnavailabilityReasonAnonymousAdministrator::ID;

object_ptr<ReactionUnavailabilityReason> reactionUnavailabilityReasonAnonymousAdministrator::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionUnavailabilityReasonAnonymousAdministrator>();
}

void reactionUnavailabilityReasonAnonymousAdministrator::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionUnavailabilityReasonAnonymousAdministrator::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionUnavailabilityReasonAnonymousAdministrator");
    s.store_class_end();
  }
}

void reactionUnavailabilityReasonAnonymousAdministrator::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionUnavailabilityReasonAnonymousAdministrator").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reactionUnavailabilityReasonGuest::Class;

reactionUnavailabilityReasonGuest::reactionUnavailabilityReasonGuest() {
}

const std::int32_t reactionUnavailabilityReasonGuest::ID;

object_ptr<ReactionUnavailabilityReason> reactionUnavailabilityReasonGuest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reactionUnavailabilityReasonGuest>();
}

void reactionUnavailabilityReasonGuest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reactionUnavailabilityReasonGuest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReactionUnavailabilityReasonGuest");
    s.store_class_end();
  }
}

void reactionUnavailabilityReasonGuest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReactionUnavailabilityReasonGuest").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass readDatePrivacySettings::Class;
jfieldID readDatePrivacySettings::show_read_date_fieldID;

readDatePrivacySettings::readDatePrivacySettings()
  : show_read_date_()
{}

readDatePrivacySettings::readDatePrivacySettings(bool show_read_date_)
  : show_read_date_(show_read_date_)
{}

const std::int32_t readDatePrivacySettings::ID;

object_ptr<readDatePrivacySettings> readDatePrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readDatePrivacySettings> res = make_object<readDatePrivacySettings>();
  res->show_read_date_ = (env->GetBooleanField(p, res->show_read_date_fieldID) != 0);
  return res;
}

void readDatePrivacySettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, show_read_date_fieldID, show_read_date_);
}

void readDatePrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadDatePrivacySettings");
    s.store_field("showReadDate", show_read_date_);
    s.store_class_end();
  }
}

void readDatePrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadDatePrivacySettings").c_str());
    show_read_date_fieldID = jni::get_field_id(env, Class, "showReadDate", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass receivedGift::Class;
jfieldID receivedGift::received_gift_id_fieldID;
jfieldID receivedGift::sender_id_fieldID;
jfieldID receivedGift::text_fieldID;
jfieldID receivedGift::is_private_fieldID;
jfieldID receivedGift::is_saved_fieldID;
jfieldID receivedGift::is_pinned_fieldID;
jfieldID receivedGift::can_be_upgraded_fieldID;
jfieldID receivedGift::can_be_transferred_fieldID;
jfieldID receivedGift::was_refunded_fieldID;
jfieldID receivedGift::date_fieldID;
jfieldID receivedGift::gift_fieldID;
jfieldID receivedGift::sell_star_count_fieldID;
jfieldID receivedGift::prepaid_upgrade_star_count_fieldID;
jfieldID receivedGift::transfer_star_count_fieldID;
jfieldID receivedGift::next_transfer_date_fieldID;
jfieldID receivedGift::next_resale_date_fieldID;
jfieldID receivedGift::export_date_fieldID;

receivedGift::receivedGift()
  : received_gift_id_()
  , sender_id_()
  , text_()
  , is_private_()
  , is_saved_()
  , is_pinned_()
  , can_be_upgraded_()
  , can_be_transferred_()
  , was_refunded_()
  , date_()
  , gift_()
  , sell_star_count_()
  , prepaid_upgrade_star_count_()
  , transfer_star_count_()
  , next_transfer_date_()
  , next_resale_date_()
  , export_date_()
{}

receivedGift::receivedGift(string const &received_gift_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<formattedText> &&text_, bool is_private_, bool is_saved_, bool is_pinned_, bool can_be_upgraded_, bool can_be_transferred_, bool was_refunded_, int32 date_, object_ptr<SentGift> &&gift_, int53 sell_star_count_, int53 prepaid_upgrade_star_count_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_)
  : received_gift_id_(received_gift_id_)
  , sender_id_(std::move(sender_id_))
  , text_(std::move(text_))
  , is_private_(is_private_)
  , is_saved_(is_saved_)
  , is_pinned_(is_pinned_)
  , can_be_upgraded_(can_be_upgraded_)
  , can_be_transferred_(can_be_transferred_)
  , was_refunded_(was_refunded_)
  , date_(date_)
  , gift_(std::move(gift_))
  , sell_star_count_(sell_star_count_)
  , prepaid_upgrade_star_count_(prepaid_upgrade_star_count_)
  , transfer_star_count_(transfer_star_count_)
  , next_transfer_date_(next_transfer_date_)
  , next_resale_date_(next_resale_date_)
  , export_date_(export_date_)
{}

const std::int32_t receivedGift::ID;

object_ptr<receivedGift> receivedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<receivedGift> res = make_object<receivedGift>();
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->is_private_ = (env->GetBooleanField(p, res->is_private_fieldID) != 0);
  res->is_saved_ = (env->GetBooleanField(p, res->is_saved_fieldID) != 0);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->can_be_upgraded_ = (env->GetBooleanField(p, res->can_be_upgraded_fieldID) != 0);
  res->can_be_transferred_ = (env->GetBooleanField(p, res->can_be_transferred_fieldID) != 0);
  res->was_refunded_ = (env->GetBooleanField(p, res->was_refunded_fieldID) != 0);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->gift_ = jni::fetch_tl_object<SentGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->sell_star_count_ = env->GetLongField(p, res->sell_star_count_fieldID);
  res->prepaid_upgrade_star_count_ = env->GetLongField(p, res->prepaid_upgrade_star_count_fieldID);
  res->transfer_star_count_ = env->GetLongField(p, res->transfer_star_count_fieldID);
  res->next_transfer_date_ = env->GetIntField(p, res->next_transfer_date_fieldID);
  res->next_resale_date_ = env->GetIntField(p, res->next_resale_date_fieldID);
  res->export_date_ = env->GetIntField(p, res->export_date_fieldID);
  return res;
}

void receivedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, received_gift_id_); if (nextString) { env->SetObjectField(s, received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_private_fieldID, is_private_);
  env->SetBooleanField(s, is_saved_fieldID, is_saved_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  env->SetBooleanField(s, can_be_upgraded_fieldID, can_be_upgraded_);
  env->SetBooleanField(s, can_be_transferred_fieldID, can_be_transferred_);
  env->SetBooleanField(s, was_refunded_fieldID, was_refunded_);
  env->SetIntField(s, date_fieldID, date_);
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, sell_star_count_fieldID, sell_star_count_);
  env->SetLongField(s, prepaid_upgrade_star_count_fieldID, prepaid_upgrade_star_count_);
  env->SetLongField(s, transfer_star_count_fieldID, transfer_star_count_);
  env->SetIntField(s, next_transfer_date_fieldID, next_transfer_date_);
  env->SetIntField(s, next_resale_date_fieldID, next_resale_date_);
  env->SetIntField(s, export_date_fieldID, export_date_);
}

void receivedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReceivedGift");
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("isPrivate", is_private_);
    s.store_field("isSaved", is_saved_);
    s.store_field("isPinned", is_pinned_);
    s.store_field("canBeUpgraded", can_be_upgraded_);
    s.store_field("canBeTransferred", can_be_transferred_);
    s.store_field("wasRefunded", was_refunded_);
    s.store_field("date", date_);
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_field("sellStarCount", sell_star_count_);
    s.store_field("prepaidUpgradeStarCount", prepaid_upgrade_star_count_);
    s.store_field("transferStarCount", transfer_star_count_);
    s.store_field("nextTransferDate", next_transfer_date_);
    s.store_field("nextResaleDate", next_resale_date_);
    s.store_field("exportDate", export_date_);
    s.store_class_end();
  }
}

void receivedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReceivedGift").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    is_private_fieldID = jni::get_field_id(env, Class, "isPrivate", "Z");
    is_saved_fieldID = jni::get_field_id(env, Class, "isSaved", "Z");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    can_be_upgraded_fieldID = jni::get_field_id(env, Class, "canBeUpgraded", "Z");
    can_be_transferred_fieldID = jni::get_field_id(env, Class, "canBeTransferred", "Z");
    was_refunded_fieldID = jni::get_field_id(env, Class, "wasRefunded", "Z");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SentGift;").c_str());
    sell_star_count_fieldID = jni::get_field_id(env, Class, "sellStarCount", "J");
    prepaid_upgrade_star_count_fieldID = jni::get_field_id(env, Class, "prepaidUpgradeStarCount", "J");
    transfer_star_count_fieldID = jni::get_field_id(env, Class, "transferStarCount", "J");
    next_transfer_date_fieldID = jni::get_field_id(env, Class, "nextTransferDate", "I");
    next_resale_date_fieldID = jni::get_field_id(env, Class, "nextResaleDate", "I");
    export_date_fieldID = jni::get_field_id(env, Class, "exportDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass receivedGifts::Class;
jfieldID receivedGifts::total_count_fieldID;
jfieldID receivedGifts::gifts_fieldID;
jfieldID receivedGifts::are_notifications_enabled_fieldID;
jfieldID receivedGifts::next_offset_fieldID;

receivedGifts::receivedGifts()
  : total_count_()
  , gifts_()
  , are_notifications_enabled_()
  , next_offset_()
{}

receivedGifts::receivedGifts(int32 total_count_, array<object_ptr<receivedGift>> &&gifts_, bool are_notifications_enabled_, string const &next_offset_)
  : total_count_(total_count_)
  , gifts_(std::move(gifts_))
  , are_notifications_enabled_(are_notifications_enabled_)
  , next_offset_(next_offset_)
{}

const std::int32_t receivedGifts::ID;

object_ptr<receivedGifts> receivedGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<receivedGifts> res = make_object<receivedGifts>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->gifts_ = jni::FetchVector<receivedGift>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->gifts_fieldID));
  res->are_notifications_enabled_ = (env->GetBooleanField(p, res->are_notifications_enabled_fieldID) != 0);
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void receivedGifts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, gifts_); if (arr_tmp_) { env->SetObjectField(s, gifts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, are_notifications_enabled_fieldID, are_notifications_enabled_);
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void receivedGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReceivedGifts");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("gifts", gifts_.size()); for (const auto &_value : gifts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("areNotificationsEnabled", are_notifications_enabled_);
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void receivedGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReceivedGifts").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    gifts_fieldID = jni::get_field_id(env, Class, "gifts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReceivedGift;").c_str());
    are_notifications_enabled_fieldID = jni::get_field_id(env, Class, "areNotificationsEnabled", "Z");
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass recommendedChatFolder::Class;
jfieldID recommendedChatFolder::folder_fieldID;
jfieldID recommendedChatFolder::description_fieldID;

recommendedChatFolder::recommendedChatFolder()
  : folder_()
  , description_()
{}

recommendedChatFolder::recommendedChatFolder(object_ptr<chatFolder> &&folder_, string const &description_)
  : folder_(std::move(folder_))
  , description_(description_)
{}

const std::int32_t recommendedChatFolder::ID;

object_ptr<recommendedChatFolder> recommendedChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recommendedChatFolder> res = make_object<recommendedChatFolder>();
  res->folder_ = jni::fetch_tl_object<chatFolder>(env, jni::fetch_object(env, p, res->folder_fieldID));
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return res;
}

void recommendedChatFolder::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (folder_ != nullptr) { jobject next; folder_->store(env, next); if (next) { env->SetObjectField(s, folder_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void recommendedChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecommendedChatFolder");
    s.store_object_field("folder", static_cast<const BaseObject *>(folder_.get()));
    s.store_field("description", description_);
    s.store_class_end();
  }
}

void recommendedChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecommendedChatFolder").c_str());
    folder_fieldID = jni::get_field_id(env, Class, "folder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolder;").c_str());
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass recommendedChatFolders::Class;
jfieldID recommendedChatFolders::chat_folders_fieldID;

recommendedChatFolders::recommendedChatFolders()
  : chat_folders_()
{}

recommendedChatFolders::recommendedChatFolders(array<object_ptr<recommendedChatFolder>> &&chat_folders_)
  : chat_folders_(std::move(chat_folders_))
{}

const std::int32_t recommendedChatFolders::ID;

object_ptr<recommendedChatFolders> recommendedChatFolders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recommendedChatFolders> res = make_object<recommendedChatFolders>();
  res->chat_folders_ = jni::FetchVector<recommendedChatFolder>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chat_folders_fieldID));
  return res;
}

void recommendedChatFolders::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_folders_); if (arr_tmp_) { env->SetObjectField(s, chat_folders_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void recommendedChatFolders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecommendedChatFolders");
    { s.store_vector_begin("chatFolders", chat_folders_.size()); for (const auto &_value : chat_folders_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void recommendedChatFolders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecommendedChatFolders").c_str());
    chat_folders_fieldID = jni::get_field_id(env, Class, "chatFolders", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$RecommendedChatFolder;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass recoveryEmailAddress::Class;
jfieldID recoveryEmailAddress::recovery_email_address_fieldID;

recoveryEmailAddress::recoveryEmailAddress()
  : recovery_email_address_()
{}

recoveryEmailAddress::recoveryEmailAddress(string const &recovery_email_address_)
  : recovery_email_address_(recovery_email_address_)
{}

const std::int32_t recoveryEmailAddress::ID;

object_ptr<recoveryEmailAddress> recoveryEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recoveryEmailAddress> res = make_object<recoveryEmailAddress>();
  res->recovery_email_address_ = jni::fetch_string(env, p, res->recovery_email_address_fieldID);
  return res;
}

void recoveryEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, recovery_email_address_); if (nextString) { env->SetObjectField(s, recovery_email_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void recoveryEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecoveryEmailAddress");
    s.store_field("recoveryEmailAddress", recovery_email_address_);
    s.store_class_end();
  }
}

void recoveryEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecoveryEmailAddress").c_str());
    recovery_email_address_fieldID = jni::get_field_id(env, Class, "recoveryEmailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass remoteFile::Class;
jfieldID remoteFile::id_fieldID;
jfieldID remoteFile::unique_id_fieldID;
jfieldID remoteFile::is_uploading_active_fieldID;
jfieldID remoteFile::is_uploading_completed_fieldID;
jfieldID remoteFile::uploaded_size_fieldID;

remoteFile::remoteFile()
  : id_()
  , unique_id_()
  , is_uploading_active_()
  , is_uploading_completed_()
  , uploaded_size_()
{}

remoteFile::remoteFile(string const &id_, string const &unique_id_, bool is_uploading_active_, bool is_uploading_completed_, int53 uploaded_size_)
  : id_(id_)
  , unique_id_(unique_id_)
  , is_uploading_active_(is_uploading_active_)
  , is_uploading_completed_(is_uploading_completed_)
  , uploaded_size_(uploaded_size_)
{}

const std::int32_t remoteFile::ID;

object_ptr<remoteFile> remoteFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<remoteFile> res = make_object<remoteFile>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->unique_id_ = jni::fetch_string(env, p, res->unique_id_fieldID);
  res->is_uploading_active_ = (env->GetBooleanField(p, res->is_uploading_active_fieldID) != 0);
  res->is_uploading_completed_ = (env->GetBooleanField(p, res->is_uploading_completed_fieldID) != 0);
  res->uploaded_size_ = env->GetLongField(p, res->uploaded_size_fieldID);
  return res;
}

void remoteFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, unique_id_); if (nextString) { env->SetObjectField(s, unique_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_uploading_active_fieldID, is_uploading_active_);
  env->SetBooleanField(s, is_uploading_completed_fieldID, is_uploading_completed_);
  env->SetLongField(s, uploaded_size_fieldID, uploaded_size_);
}

void remoteFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoteFile");
    s.store_field("id", id_);
    s.store_field("uniqueId", unique_id_);
    s.store_field("isUploadingActive", is_uploading_active_);
    s.store_field("isUploadingCompleted", is_uploading_completed_);
    s.store_field("uploadedSize", uploaded_size_);
    s.store_class_end();
  }
}

void remoteFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoteFile").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    unique_id_fieldID = jni::get_field_id(env, Class, "uniqueId", "Ljava/lang/String;");
    is_uploading_active_fieldID = jni::get_field_id(env, Class, "isUploadingActive", "Z");
    is_uploading_completed_fieldID = jni::get_field_id(env, Class, "isUploadingCompleted", "Z");
    uploaded_size_fieldID = jni::get_field_id(env, Class, "uploadedSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass ReplyMarkup::Class;

object_ptr<ReplyMarkup> ReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case replyMarkupRemoveKeyboard::ID:
      return replyMarkupRemoveKeyboard::fetch(env, p);
    case replyMarkupForceReply::ID:
      return replyMarkupForceReply::fetch(env, p);
    case replyMarkupShowKeyboard::ID:
      return replyMarkupShowKeyboard::fetch(env, p);
    case replyMarkupInlineKeyboard::ID:
      return replyMarkupInlineKeyboard::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplyMarkup").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass replyMarkupRemoveKeyboard::Class;
jfieldID replyMarkupRemoveKeyboard::is_personal_fieldID;

replyMarkupRemoveKeyboard::replyMarkupRemoveKeyboard()
  : is_personal_()
{}

replyMarkupRemoveKeyboard::replyMarkupRemoveKeyboard(bool is_personal_)
  : is_personal_(is_personal_)
{}

const std::int32_t replyMarkupRemoveKeyboard::ID;

object_ptr<ReplyMarkup> replyMarkupRemoveKeyboard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replyMarkupRemoveKeyboard> res = make_object<replyMarkupRemoveKeyboard>();
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  return std::move(res);
}

void replyMarkupRemoveKeyboard::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_personal_fieldID, is_personal_);
}

void replyMarkupRemoveKeyboard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplyMarkupRemoveKeyboard");
    s.store_field("isPersonal", is_personal_);
    s.store_class_end();
  }
}

void replyMarkupRemoveKeyboard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplyMarkupRemoveKeyboard").c_str());
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass replyMarkupForceReply::Class;
jfieldID replyMarkupForceReply::is_personal_fieldID;
jfieldID replyMarkupForceReply::input_field_placeholder_fieldID;

replyMarkupForceReply::replyMarkupForceReply()
  : is_personal_()
  , input_field_placeholder_()
{}

replyMarkupForceReply::replyMarkupForceReply(bool is_personal_, string const &input_field_placeholder_)
  : is_personal_(is_personal_)
  , input_field_placeholder_(input_field_placeholder_)
{}

const std::int32_t replyMarkupForceReply::ID;

object_ptr<ReplyMarkup> replyMarkupForceReply::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replyMarkupForceReply> res = make_object<replyMarkupForceReply>();
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  res->input_field_placeholder_ = jni::fetch_string(env, p, res->input_field_placeholder_fieldID);
  return std::move(res);
}

void replyMarkupForceReply::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_personal_fieldID, is_personal_);
  { jstring nextString = jni::to_jstring(env, input_field_placeholder_); if (nextString) { env->SetObjectField(s, input_field_placeholder_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void replyMarkupForceReply::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplyMarkupForceReply");
    s.store_field("isPersonal", is_personal_);
    s.store_field("inputFieldPlaceholder", input_field_placeholder_);
    s.store_class_end();
  }
}

void replyMarkupForceReply::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplyMarkupForceReply").c_str());
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    input_field_placeholder_fieldID = jni::get_field_id(env, Class, "inputFieldPlaceholder", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass replyMarkupShowKeyboard::Class;
jfieldID replyMarkupShowKeyboard::rows_fieldID;
jfieldID replyMarkupShowKeyboard::is_persistent_fieldID;
jfieldID replyMarkupShowKeyboard::resize_keyboard_fieldID;
jfieldID replyMarkupShowKeyboard::one_time_fieldID;
jfieldID replyMarkupShowKeyboard::is_personal_fieldID;
jfieldID replyMarkupShowKeyboard::input_field_placeholder_fieldID;

replyMarkupShowKeyboard::replyMarkupShowKeyboard()
  : rows_()
  , is_persistent_()
  , resize_keyboard_()
  , one_time_()
  , is_personal_()
  , input_field_placeholder_()
{}

replyMarkupShowKeyboard::replyMarkupShowKeyboard(array<array<object_ptr<keyboardButton>>> &&rows_, bool is_persistent_, bool resize_keyboard_, bool one_time_, bool is_personal_, string const &input_field_placeholder_)
  : rows_(std::move(rows_))
  , is_persistent_(is_persistent_)
  , resize_keyboard_(resize_keyboard_)
  , one_time_(one_time_)
  , is_personal_(is_personal_)
  , input_field_placeholder_(input_field_placeholder_)
{}

const std::int32_t replyMarkupShowKeyboard::ID;

object_ptr<ReplyMarkup> replyMarkupShowKeyboard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replyMarkupShowKeyboard> res = make_object<replyMarkupShowKeyboard>();
  res->rows_ = jni::FetchVector<array<keyboardButton>>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->rows_fieldID));
  res->is_persistent_ = (env->GetBooleanField(p, res->is_persistent_fieldID) != 0);
  res->resize_keyboard_ = (env->GetBooleanField(p, res->resize_keyboard_fieldID) != 0);
  res->one_time_ = (env->GetBooleanField(p, res->one_time_fieldID) != 0);
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  res->input_field_placeholder_ = jni::fetch_string(env, p, res->input_field_placeholder_fieldID);
  return std::move(res);
}

void replyMarkupShowKeyboard::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, rows_); if (arr_tmp_) { env->SetObjectField(s, rows_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_persistent_fieldID, is_persistent_);
  env->SetBooleanField(s, resize_keyboard_fieldID, resize_keyboard_);
  env->SetBooleanField(s, one_time_fieldID, one_time_);
  env->SetBooleanField(s, is_personal_fieldID, is_personal_);
  { jstring nextString = jni::to_jstring(env, input_field_placeholder_); if (nextString) { env->SetObjectField(s, input_field_placeholder_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void replyMarkupShowKeyboard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplyMarkupShowKeyboard");
    { s.store_vector_begin("rows", rows_.size()); for (const auto &_value : rows_) { { s.store_vector_begin("", _value.size()); for (const auto &_value2 : _value) { s.store_object_field("", static_cast<const BaseObject *>(_value2.get())); } s.store_class_end(); } } s.store_class_end(); }
    s.store_field("isPersistent", is_persistent_);
    s.store_field("resizeKeyboard", resize_keyboard_);
    s.store_field("oneTime", one_time_);
    s.store_field("isPersonal", is_personal_);
    s.store_field("inputFieldPlaceholder", input_field_placeholder_);
    s.store_class_end();
  }
}

void replyMarkupShowKeyboard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplyMarkupShowKeyboard").c_str());
    rows_fieldID = jni::get_field_id(env, Class, "rows", (PSLICE() << "[[L" << get_package_name_ref() << "/TdApi$KeyboardButton;").c_str());
    is_persistent_fieldID = jni::get_field_id(env, Class, "isPersistent", "Z");
    resize_keyboard_fieldID = jni::get_field_id(env, Class, "resizeKeyboard", "Z");
    one_time_fieldID = jni::get_field_id(env, Class, "oneTime", "Z");
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    input_field_placeholder_fieldID = jni::get_field_id(env, Class, "inputFieldPlaceholder", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass replyMarkupInlineKeyboard::Class;
jfieldID replyMarkupInlineKeyboard::rows_fieldID;

replyMarkupInlineKeyboard::replyMarkupInlineKeyboard()
  : rows_()
{}

replyMarkupInlineKeyboard::replyMarkupInlineKeyboard(array<array<object_ptr<inlineKeyboardButton>>> &&rows_)
  : rows_(std::move(rows_))
{}

const std::int32_t replyMarkupInlineKeyboard::ID;

object_ptr<ReplyMarkup> replyMarkupInlineKeyboard::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replyMarkupInlineKeyboard> res = make_object<replyMarkupInlineKeyboard>();
  res->rows_ = jni::FetchVector<array<inlineKeyboardButton>>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->rows_fieldID));
  return std::move(res);
}

void replyMarkupInlineKeyboard::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, rows_); if (arr_tmp_) { env->SetObjectField(s, rows_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void replyMarkupInlineKeyboard::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplyMarkupInlineKeyboard");
    { s.store_vector_begin("rows", rows_.size()); for (const auto &_value : rows_) { { s.store_vector_begin("", _value.size()); for (const auto &_value2 : _value) { s.store_object_field("", static_cast<const BaseObject *>(_value2.get())); } s.store_class_end(); } } s.store_class_end(); }
    s.store_class_end();
  }
}

void replyMarkupInlineKeyboard::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplyMarkupInlineKeyboard").c_str());
    rows_fieldID = jni::get_field_id(env, Class, "rows", (PSLICE() << "[[L" << get_package_name_ref() << "/TdApi$InlineKeyboardButton;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ReportChatResult::Class;

object_ptr<ReportChatResult> ReportChatResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reportChatResultOk::ID:
      return reportChatResultOk::fetch(env, p);
    case reportChatResultOptionRequired::ID:
      return reportChatResultOptionRequired::fetch(env, p);
    case reportChatResultTextRequired::ID:
      return reportChatResultTextRequired::fetch(env, p);
    case reportChatResultMessagesRequired::ID:
      return reportChatResultMessagesRequired::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReportChatResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportChatResultOk::Class;

reportChatResultOk::reportChatResultOk() {
}

const std::int32_t reportChatResultOk::ID;

object_ptr<ReportChatResult> reportChatResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportChatResultOk>();
}

void reportChatResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportChatResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatResultOk");
    s.store_class_end();
  }
}

void reportChatResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportChatResultOptionRequired::Class;
jfieldID reportChatResultOptionRequired::title_fieldID;
jfieldID reportChatResultOptionRequired::options_fieldID;

reportChatResultOptionRequired::reportChatResultOptionRequired()
  : title_()
  , options_()
{}

reportChatResultOptionRequired::reportChatResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_)
  : title_(title_)
  , options_(std::move(options_))
{}

const std::int32_t reportChatResultOptionRequired::ID;

object_ptr<ReportChatResult> reportChatResultOptionRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportChatResultOptionRequired> res = make_object<reportChatResultOptionRequired>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->options_ = jni::FetchVector<reportOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return std::move(res);
}

void reportChatResultOptionRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void reportChatResultOptionRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatResultOptionRequired");
    s.store_field("title", title_);
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void reportChatResultOptionRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatResultOptionRequired").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReportOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportChatResultTextRequired::Class;
jfieldID reportChatResultTextRequired::option_id_fieldID;
jfieldID reportChatResultTextRequired::is_optional_fieldID;

reportChatResultTextRequired::reportChatResultTextRequired()
  : option_id_()
  , is_optional_()
{}

reportChatResultTextRequired::reportChatResultTextRequired(bytes const &option_id_, bool is_optional_)
  : option_id_(std::move(option_id_))
  , is_optional_(is_optional_)
{}

const std::int32_t reportChatResultTextRequired::ID;

object_ptr<ReportChatResult> reportChatResultTextRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportChatResultTextRequired> res = make_object<reportChatResultTextRequired>();
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  res->is_optional_ = (env->GetBooleanField(p, res->is_optional_fieldID) != 0);
  return std::move(res);
}

void reportChatResultTextRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, option_id_); if (nextBytes) { env->SetObjectField(s, option_id_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  env->SetBooleanField(s, is_optional_fieldID, is_optional_);
}

void reportChatResultTextRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatResultTextRequired");
    s.store_bytes_field("optionId", option_id_);
    s.store_field("isOptional", is_optional_);
    s.store_class_end();
  }
}

void reportChatResultTextRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatResultTextRequired").c_str());
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    is_optional_fieldID = jni::get_field_id(env, Class, "isOptional", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass reportChatResultMessagesRequired::Class;

reportChatResultMessagesRequired::reportChatResultMessagesRequired() {
}

const std::int32_t reportChatResultMessagesRequired::ID;

object_ptr<ReportChatResult> reportChatResultMessagesRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportChatResultMessagesRequired>();
}

void reportChatResultMessagesRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportChatResultMessagesRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatResultMessagesRequired");
    s.store_class_end();
  }
}

void reportChatResultMessagesRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatResultMessagesRequired").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportOption::Class;
jfieldID reportOption::id_fieldID;
jfieldID reportOption::text_fieldID;

reportOption::reportOption()
  : id_()
  , text_()
{}

reportOption::reportOption(bytes const &id_, string const &text_)
  : id_(std::move(id_))
  , text_(text_)
{}

const std::int32_t reportOption::ID;

object_ptr<reportOption> reportOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportOption> res = make_object<reportOption>();
  res->id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->id_fieldID));
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void reportOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, id_); if (nextBytes) { env->SetObjectField(s, id_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void reportOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportOption");
    s.store_bytes_field("id", id_);
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void reportOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportOption").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "[B");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass ReportReason::Class;

object_ptr<ReportReason> ReportReason::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reportReasonSpam::ID:
      return reportReasonSpam::fetch(env, p);
    case reportReasonViolence::ID:
      return reportReasonViolence::fetch(env, p);
    case reportReasonPornography::ID:
      return reportReasonPornography::fetch(env, p);
    case reportReasonChildAbuse::ID:
      return reportReasonChildAbuse::fetch(env, p);
    case reportReasonCopyright::ID:
      return reportReasonCopyright::fetch(env, p);
    case reportReasonUnrelatedLocation::ID:
      return reportReasonUnrelatedLocation::fetch(env, p);
    case reportReasonFake::ID:
      return reportReasonFake::fetch(env, p);
    case reportReasonIllegalDrugs::ID:
      return reportReasonIllegalDrugs::fetch(env, p);
    case reportReasonPersonalDetails::ID:
      return reportReasonPersonalDetails::fetch(env, p);
    case reportReasonCustom::ID:
      return reportReasonCustom::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReportReason::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReason").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonSpam::Class;

reportReasonSpam::reportReasonSpam() {
}

const std::int32_t reportReasonSpam::ID;

object_ptr<ReportReason> reportReasonSpam::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonSpam>();
}

void reportReasonSpam::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonSpam");
    s.store_class_end();
  }
}

void reportReasonSpam::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonSpam").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonViolence::Class;

reportReasonViolence::reportReasonViolence() {
}

const std::int32_t reportReasonViolence::ID;

object_ptr<ReportReason> reportReasonViolence::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonViolence>();
}

void reportReasonViolence::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonViolence");
    s.store_class_end();
  }
}

void reportReasonViolence::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonViolence").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonPornography::Class;

reportReasonPornography::reportReasonPornography() {
}

const std::int32_t reportReasonPornography::ID;

object_ptr<ReportReason> reportReasonPornography::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonPornography>();
}

void reportReasonPornography::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonPornography");
    s.store_class_end();
  }
}

void reportReasonPornography::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonPornography").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonChildAbuse::Class;

reportReasonChildAbuse::reportReasonChildAbuse() {
}

const std::int32_t reportReasonChildAbuse::ID;

object_ptr<ReportReason> reportReasonChildAbuse::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonChildAbuse>();
}

void reportReasonChildAbuse::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonChildAbuse");
    s.store_class_end();
  }
}

void reportReasonChildAbuse::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonChildAbuse").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonCopyright::Class;

reportReasonCopyright::reportReasonCopyright() {
}

const std::int32_t reportReasonCopyright::ID;

object_ptr<ReportReason> reportReasonCopyright::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonCopyright>();
}

void reportReasonCopyright::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonCopyright");
    s.store_class_end();
  }
}

void reportReasonCopyright::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonCopyright").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonUnrelatedLocation::Class;

reportReasonUnrelatedLocation::reportReasonUnrelatedLocation() {
}

const std::int32_t reportReasonUnrelatedLocation::ID;

object_ptr<ReportReason> reportReasonUnrelatedLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonUnrelatedLocation>();
}

void reportReasonUnrelatedLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonUnrelatedLocation");
    s.store_class_end();
  }
}

void reportReasonUnrelatedLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonUnrelatedLocation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonFake::Class;

reportReasonFake::reportReasonFake() {
}

const std::int32_t reportReasonFake::ID;

object_ptr<ReportReason> reportReasonFake::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonFake>();
}

void reportReasonFake::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonFake");
    s.store_class_end();
  }
}

void reportReasonFake::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonFake").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonIllegalDrugs::Class;

reportReasonIllegalDrugs::reportReasonIllegalDrugs() {
}

const std::int32_t reportReasonIllegalDrugs::ID;

object_ptr<ReportReason> reportReasonIllegalDrugs::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonIllegalDrugs>();
}

void reportReasonIllegalDrugs::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonIllegalDrugs");
    s.store_class_end();
  }
}

void reportReasonIllegalDrugs::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonIllegalDrugs").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonPersonalDetails::Class;

reportReasonPersonalDetails::reportReasonPersonalDetails() {
}

const std::int32_t reportReasonPersonalDetails::ID;

object_ptr<ReportReason> reportReasonPersonalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonPersonalDetails>();
}

void reportReasonPersonalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonPersonalDetails");
    s.store_class_end();
  }
}

void reportReasonPersonalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonPersonalDetails").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportReasonCustom::Class;

reportReasonCustom::reportReasonCustom() {
}

const std::int32_t reportReasonCustom::ID;

object_ptr<ReportReason> reportReasonCustom::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportReasonCustom>();
}

void reportReasonCustom::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportReasonCustom");
    s.store_class_end();
  }
}

void reportReasonCustom::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportReasonCustom").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ReportSponsoredResult::Class;

object_ptr<ReportSponsoredResult> ReportSponsoredResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reportSponsoredResultOk::ID:
      return reportSponsoredResultOk::fetch(env, p);
    case reportSponsoredResultFailed::ID:
      return reportSponsoredResultFailed::fetch(env, p);
    case reportSponsoredResultOptionRequired::ID:
      return reportSponsoredResultOptionRequired::fetch(env, p);
    case reportSponsoredResultAdsHidden::ID:
      return reportSponsoredResultAdsHidden::fetch(env, p);
    case reportSponsoredResultPremiumRequired::ID:
      return reportSponsoredResultPremiumRequired::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReportSponsoredResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredResultOk::Class;

reportSponsoredResultOk::reportSponsoredResultOk() {
}

const std::int32_t reportSponsoredResultOk::ID;

object_ptr<ReportSponsoredResult> reportSponsoredResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportSponsoredResultOk>();
}

void reportSponsoredResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportSponsoredResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredResultOk");
    s.store_class_end();
  }
}

void reportSponsoredResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredResultFailed::Class;

reportSponsoredResultFailed::reportSponsoredResultFailed() {
}

const std::int32_t reportSponsoredResultFailed::ID;

object_ptr<ReportSponsoredResult> reportSponsoredResultFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportSponsoredResultFailed>();
}

void reportSponsoredResultFailed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportSponsoredResultFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredResultFailed");
    s.store_class_end();
  }
}

void reportSponsoredResultFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResultFailed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredResultOptionRequired::Class;
jfieldID reportSponsoredResultOptionRequired::title_fieldID;
jfieldID reportSponsoredResultOptionRequired::options_fieldID;

reportSponsoredResultOptionRequired::reportSponsoredResultOptionRequired()
  : title_()
  , options_()
{}

reportSponsoredResultOptionRequired::reportSponsoredResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_)
  : title_(title_)
  , options_(std::move(options_))
{}

const std::int32_t reportSponsoredResultOptionRequired::ID;

object_ptr<ReportSponsoredResult> reportSponsoredResultOptionRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportSponsoredResultOptionRequired> res = make_object<reportSponsoredResultOptionRequired>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->options_ = jni::FetchVector<reportOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return std::move(res);
}

void reportSponsoredResultOptionRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void reportSponsoredResultOptionRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredResultOptionRequired");
    s.store_field("title", title_);
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void reportSponsoredResultOptionRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResultOptionRequired").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReportOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredResultAdsHidden::Class;

reportSponsoredResultAdsHidden::reportSponsoredResultAdsHidden() {
}

const std::int32_t reportSponsoredResultAdsHidden::ID;

object_ptr<ReportSponsoredResult> reportSponsoredResultAdsHidden::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportSponsoredResultAdsHidden>();
}

void reportSponsoredResultAdsHidden::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportSponsoredResultAdsHidden::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredResultAdsHidden");
    s.store_class_end();
  }
}

void reportSponsoredResultAdsHidden::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResultAdsHidden").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredResultPremiumRequired::Class;

reportSponsoredResultPremiumRequired::reportSponsoredResultPremiumRequired() {
}

const std::int32_t reportSponsoredResultPremiumRequired::ID;

object_ptr<ReportSponsoredResult> reportSponsoredResultPremiumRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportSponsoredResultPremiumRequired>();
}

void reportSponsoredResultPremiumRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportSponsoredResultPremiumRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredResultPremiumRequired");
    s.store_class_end();
  }
}

void reportSponsoredResultPremiumRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredResultPremiumRequired").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ReportStoryResult::Class;

object_ptr<ReportStoryResult> ReportStoryResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case reportStoryResultOk::ID:
      return reportStoryResultOk::fetch(env, p);
    case reportStoryResultOptionRequired::ID:
      return reportStoryResultOptionRequired::fetch(env, p);
    case reportStoryResultTextRequired::ID:
      return reportStoryResultTextRequired::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ReportStoryResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportStoryResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportStoryResultOk::Class;

reportStoryResultOk::reportStoryResultOk() {
}

const std::int32_t reportStoryResultOk::ID;

object_ptr<ReportStoryResult> reportStoryResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<reportStoryResultOk>();
}

void reportStoryResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void reportStoryResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportStoryResultOk");
    s.store_class_end();
  }
}

void reportStoryResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportStoryResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportStoryResultOptionRequired::Class;
jfieldID reportStoryResultOptionRequired::title_fieldID;
jfieldID reportStoryResultOptionRequired::options_fieldID;

reportStoryResultOptionRequired::reportStoryResultOptionRequired()
  : title_()
  , options_()
{}

reportStoryResultOptionRequired::reportStoryResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_)
  : title_(title_)
  , options_(std::move(options_))
{}

const std::int32_t reportStoryResultOptionRequired::ID;

object_ptr<ReportStoryResult> reportStoryResultOptionRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportStoryResultOptionRequired> res = make_object<reportStoryResultOptionRequired>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->options_ = jni::FetchVector<reportOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return std::move(res);
}

void reportStoryResultOptionRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void reportStoryResultOptionRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportStoryResultOptionRequired");
    s.store_field("title", title_);
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void reportStoryResultOptionRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportStoryResultOptionRequired").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReportOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportStoryResultTextRequired::Class;
jfieldID reportStoryResultTextRequired::option_id_fieldID;
jfieldID reportStoryResultTextRequired::is_optional_fieldID;

reportStoryResultTextRequired::reportStoryResultTextRequired()
  : option_id_()
  , is_optional_()
{}

reportStoryResultTextRequired::reportStoryResultTextRequired(bytes const &option_id_, bool is_optional_)
  : option_id_(std::move(option_id_))
  , is_optional_(is_optional_)
{}

const std::int32_t reportStoryResultTextRequired::ID;

object_ptr<ReportStoryResult> reportStoryResultTextRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportStoryResultTextRequired> res = make_object<reportStoryResultTextRequired>();
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  res->is_optional_ = (env->GetBooleanField(p, res->is_optional_fieldID) != 0);
  return std::move(res);
}

void reportStoryResultTextRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, option_id_); if (nextBytes) { env->SetObjectField(s, option_id_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  env->SetBooleanField(s, is_optional_fieldID, is_optional_);
}

void reportStoryResultTextRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportStoryResultTextRequired");
    s.store_bytes_field("optionId", option_id_);
    s.store_field("isOptional", is_optional_);
    s.store_class_end();
  }
}

void reportStoryResultTextRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportStoryResultTextRequired").c_str());
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    is_optional_fieldID = jni::get_field_id(env, Class, "isOptional", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass ResendCodeReason::Class;

object_ptr<ResendCodeReason> ResendCodeReason::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case resendCodeReasonUserRequest::ID:
      return resendCodeReasonUserRequest::fetch(env, p);
    case resendCodeReasonVerificationFailed::ID:
      return resendCodeReasonVerificationFailed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ResendCodeReason::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendCodeReason").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendCodeReasonUserRequest::Class;

resendCodeReasonUserRequest::resendCodeReasonUserRequest() {
}

const std::int32_t resendCodeReasonUserRequest::ID;

object_ptr<ResendCodeReason> resendCodeReasonUserRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resendCodeReasonUserRequest>();
}

void resendCodeReasonUserRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendCodeReasonUserRequest");
    s.store_class_end();
  }
}

void resendCodeReasonUserRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendCodeReasonUserRequest").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendCodeReasonVerificationFailed::Class;
jfieldID resendCodeReasonVerificationFailed::error_message_fieldID;

resendCodeReasonVerificationFailed::resendCodeReasonVerificationFailed()
  : error_message_()
{}

resendCodeReasonVerificationFailed::resendCodeReasonVerificationFailed(string const &error_message_)
  : error_message_(error_message_)
{}

const std::int32_t resendCodeReasonVerificationFailed::ID;

object_ptr<ResendCodeReason> resendCodeReasonVerificationFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resendCodeReasonVerificationFailed> res = make_object<resendCodeReasonVerificationFailed>();
  res->error_message_ = jni::fetch_string(env, p, res->error_message_fieldID);
  return std::move(res);
}

void resendCodeReasonVerificationFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendCodeReasonVerificationFailed");
    s.store_field("errorMessage", error_message_);
    s.store_class_end();
  }
}

void resendCodeReasonVerificationFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendCodeReasonVerificationFailed").c_str());
    error_message_fieldID = jni::get_field_id(env, Class, "errorMessage", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass ResetPasswordResult::Class;

object_ptr<ResetPasswordResult> ResetPasswordResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case resetPasswordResultOk::ID:
      return resetPasswordResultOk::fetch(env, p);
    case resetPasswordResultPending::ID:
      return resetPasswordResultPending::fetch(env, p);
    case resetPasswordResultDeclined::ID:
      return resetPasswordResultDeclined::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ResetPasswordResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetPasswordResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetPasswordResultOk::Class;

resetPasswordResultOk::resetPasswordResultOk() {
}

const std::int32_t resetPasswordResultOk::ID;

object_ptr<ResetPasswordResult> resetPasswordResultOk::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetPasswordResultOk>();
}

void resetPasswordResultOk::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void resetPasswordResultOk::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetPasswordResultOk");
    s.store_class_end();
  }
}

void resetPasswordResultOk::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetPasswordResultOk").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetPasswordResultPending::Class;
jfieldID resetPasswordResultPending::pending_reset_date_fieldID;

resetPasswordResultPending::resetPasswordResultPending()
  : pending_reset_date_()
{}

resetPasswordResultPending::resetPasswordResultPending(int32 pending_reset_date_)
  : pending_reset_date_(pending_reset_date_)
{}

const std::int32_t resetPasswordResultPending::ID;

object_ptr<ResetPasswordResult> resetPasswordResultPending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resetPasswordResultPending> res = make_object<resetPasswordResultPending>();
  res->pending_reset_date_ = env->GetIntField(p, res->pending_reset_date_fieldID);
  return std::move(res);
}

void resetPasswordResultPending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, pending_reset_date_fieldID, pending_reset_date_);
}

void resetPasswordResultPending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetPasswordResultPending");
    s.store_field("pendingResetDate", pending_reset_date_);
    s.store_class_end();
  }
}

void resetPasswordResultPending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetPasswordResultPending").c_str());
    pending_reset_date_fieldID = jni::get_field_id(env, Class, "pendingResetDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass resetPasswordResultDeclined::Class;
jfieldID resetPasswordResultDeclined::retry_date_fieldID;

resetPasswordResultDeclined::resetPasswordResultDeclined()
  : retry_date_()
{}

resetPasswordResultDeclined::resetPasswordResultDeclined(int32 retry_date_)
  : retry_date_(retry_date_)
{}

const std::int32_t resetPasswordResultDeclined::ID;

object_ptr<ResetPasswordResult> resetPasswordResultDeclined::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resetPasswordResultDeclined> res = make_object<resetPasswordResultDeclined>();
  res->retry_date_ = env->GetIntField(p, res->retry_date_fieldID);
  return std::move(res);
}

void resetPasswordResultDeclined::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, retry_date_fieldID, retry_date_);
}

void resetPasswordResultDeclined::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetPasswordResultDeclined");
    s.store_field("retryDate", retry_date_);
    s.store_class_end();
  }
}

void resetPasswordResultDeclined::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetPasswordResultDeclined").c_str());
    retry_date_fieldID = jni::get_field_id(env, Class, "retryDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass RevenueWithdrawalState::Class;

object_ptr<RevenueWithdrawalState> RevenueWithdrawalState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case revenueWithdrawalStatePending::ID:
      return revenueWithdrawalStatePending::fetch(env, p);
    case revenueWithdrawalStateSucceeded::ID:
      return revenueWithdrawalStateSucceeded::fetch(env, p);
    case revenueWithdrawalStateFailed::ID:
      return revenueWithdrawalStateFailed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void RevenueWithdrawalState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevenueWithdrawalState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass revenueWithdrawalStatePending::Class;

revenueWithdrawalStatePending::revenueWithdrawalStatePending() {
}

const std::int32_t revenueWithdrawalStatePending::ID;

object_ptr<RevenueWithdrawalState> revenueWithdrawalStatePending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<revenueWithdrawalStatePending>();
}

void revenueWithdrawalStatePending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void revenueWithdrawalStatePending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RevenueWithdrawalStatePending");
    s.store_class_end();
  }
}

void revenueWithdrawalStatePending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevenueWithdrawalStatePending").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass revenueWithdrawalStateSucceeded::Class;
jfieldID revenueWithdrawalStateSucceeded::date_fieldID;
jfieldID revenueWithdrawalStateSucceeded::url_fieldID;

revenueWithdrawalStateSucceeded::revenueWithdrawalStateSucceeded()
  : date_()
  , url_()
{}

revenueWithdrawalStateSucceeded::revenueWithdrawalStateSucceeded(int32 date_, string const &url_)
  : date_(date_)
  , url_(url_)
{}

const std::int32_t revenueWithdrawalStateSucceeded::ID;

object_ptr<RevenueWithdrawalState> revenueWithdrawalStateSucceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<revenueWithdrawalStateSucceeded> res = make_object<revenueWithdrawalStateSucceeded>();
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void revenueWithdrawalStateSucceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, date_fieldID, date_);
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void revenueWithdrawalStateSucceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RevenueWithdrawalStateSucceeded");
    s.store_field("date", date_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void revenueWithdrawalStateSucceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevenueWithdrawalStateSucceeded").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass revenueWithdrawalStateFailed::Class;

revenueWithdrawalStateFailed::revenueWithdrawalStateFailed() {
}

const std::int32_t revenueWithdrawalStateFailed::ID;

object_ptr<RevenueWithdrawalState> revenueWithdrawalStateFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<revenueWithdrawalStateFailed>();
}

void revenueWithdrawalStateFailed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void revenueWithdrawalStateFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RevenueWithdrawalStateFailed");
    s.store_class_end();
  }
}

void revenueWithdrawalStateFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevenueWithdrawalStateFailed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass RichText::Class;

object_ptr<RichText> RichText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case richTextPlain::ID:
      return richTextPlain::fetch(env, p);
    case richTextBold::ID:
      return richTextBold::fetch(env, p);
    case richTextItalic::ID:
      return richTextItalic::fetch(env, p);
    case richTextUnderline::ID:
      return richTextUnderline::fetch(env, p);
    case richTextStrikethrough::ID:
      return richTextStrikethrough::fetch(env, p);
    case richTextFixed::ID:
      return richTextFixed::fetch(env, p);
    case richTextUrl::ID:
      return richTextUrl::fetch(env, p);
    case richTextEmailAddress::ID:
      return richTextEmailAddress::fetch(env, p);
    case richTextSubscript::ID:
      return richTextSubscript::fetch(env, p);
    case richTextSuperscript::ID:
      return richTextSuperscript::fetch(env, p);
    case richTextMarked::ID:
      return richTextMarked::fetch(env, p);
    case richTextPhoneNumber::ID:
      return richTextPhoneNumber::fetch(env, p);
    case richTextIcon::ID:
      return richTextIcon::fetch(env, p);
    case richTextReference::ID:
      return richTextReference::fetch(env, p);
    case richTextAnchor::ID:
      return richTextAnchor::fetch(env, p);
    case richTextAnchorLink::ID:
      return richTextAnchorLink::fetch(env, p);
    case richTexts::ID:
      return richTexts::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void RichText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichText").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextPlain::Class;
jfieldID richTextPlain::text_fieldID;

richTextPlain::richTextPlain()
  : text_()
{}

richTextPlain::richTextPlain(string const &text_)
  : text_(text_)
{}

const std::int32_t richTextPlain::ID;

object_ptr<RichText> richTextPlain::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextPlain> res = make_object<richTextPlain>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return std::move(res);
}

void richTextPlain::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextPlain::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextPlain");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void richTextPlain::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextPlain").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTextBold::Class;
jfieldID richTextBold::text_fieldID;

richTextBold::richTextBold()
  : text_()
{}

richTextBold::richTextBold(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextBold::ID;

object_ptr<RichText> richTextBold::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextBold> res = make_object<richTextBold>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextBold::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextBold::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextBold");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextBold::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextBold").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextItalic::Class;
jfieldID richTextItalic::text_fieldID;

richTextItalic::richTextItalic()
  : text_()
{}

richTextItalic::richTextItalic(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextItalic::ID;

object_ptr<RichText> richTextItalic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextItalic> res = make_object<richTextItalic>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextItalic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextItalic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextItalic");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextItalic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextItalic").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextUnderline::Class;
jfieldID richTextUnderline::text_fieldID;

richTextUnderline::richTextUnderline()
  : text_()
{}

richTextUnderline::richTextUnderline(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextUnderline::ID;

object_ptr<RichText> richTextUnderline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextUnderline> res = make_object<richTextUnderline>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextUnderline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextUnderline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextUnderline");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextUnderline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextUnderline").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextStrikethrough::Class;
jfieldID richTextStrikethrough::text_fieldID;

richTextStrikethrough::richTextStrikethrough()
  : text_()
{}

richTextStrikethrough::richTextStrikethrough(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextStrikethrough::ID;

object_ptr<RichText> richTextStrikethrough::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextStrikethrough> res = make_object<richTextStrikethrough>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextStrikethrough::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextStrikethrough::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextStrikethrough");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextStrikethrough::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextStrikethrough").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextFixed::Class;
jfieldID richTextFixed::text_fieldID;

richTextFixed::richTextFixed()
  : text_()
{}

richTextFixed::richTextFixed(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextFixed::ID;

object_ptr<RichText> richTextFixed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextFixed> res = make_object<richTextFixed>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextFixed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextFixed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextFixed");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextFixed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextFixed").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextUrl::Class;
jfieldID richTextUrl::text_fieldID;
jfieldID richTextUrl::url_fieldID;
jfieldID richTextUrl::is_cached_fieldID;

richTextUrl::richTextUrl()
  : text_()
  , url_()
  , is_cached_()
{}

richTextUrl::richTextUrl(object_ptr<RichText> &&text_, string const &url_, bool is_cached_)
  : text_(std::move(text_))
  , url_(url_)
  , is_cached_(is_cached_)
{}

const std::int32_t richTextUrl::ID;

object_ptr<RichText> richTextUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextUrl> res = make_object<richTextUrl>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->is_cached_ = (env->GetBooleanField(p, res->is_cached_fieldID) != 0);
  return std::move(res);
}

void richTextUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_cached_fieldID, is_cached_);
}

void richTextUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextUrl");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("url", url_);
    s.store_field("isCached", is_cached_);
    s.store_class_end();
  }
}

void richTextUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextUrl").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    is_cached_fieldID = jni::get_field_id(env, Class, "isCached", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass richTextEmailAddress::Class;
jfieldID richTextEmailAddress::text_fieldID;
jfieldID richTextEmailAddress::email_address_fieldID;

richTextEmailAddress::richTextEmailAddress()
  : text_()
  , email_address_()
{}

richTextEmailAddress::richTextEmailAddress(object_ptr<RichText> &&text_, string const &email_address_)
  : text_(std::move(text_))
  , email_address_(email_address_)
{}

const std::int32_t richTextEmailAddress::ID;

object_ptr<RichText> richTextEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextEmailAddress> res = make_object<richTextEmailAddress>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  return std::move(res);
}

void richTextEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, email_address_); if (nextString) { env->SetObjectField(s, email_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextEmailAddress");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("emailAddress", email_address_);
    s.store_class_end();
  }
}

void richTextEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextEmailAddress").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTextSubscript::Class;
jfieldID richTextSubscript::text_fieldID;

richTextSubscript::richTextSubscript()
  : text_()
{}

richTextSubscript::richTextSubscript(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextSubscript::ID;

object_ptr<RichText> richTextSubscript::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextSubscript> res = make_object<richTextSubscript>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextSubscript::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextSubscript::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextSubscript");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextSubscript::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextSubscript").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextSuperscript::Class;
jfieldID richTextSuperscript::text_fieldID;

richTextSuperscript::richTextSuperscript()
  : text_()
{}

richTextSuperscript::richTextSuperscript(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextSuperscript::ID;

object_ptr<RichText> richTextSuperscript::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextSuperscript> res = make_object<richTextSuperscript>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextSuperscript::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextSuperscript::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextSuperscript");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextSuperscript::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextSuperscript").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextMarked::Class;
jfieldID richTextMarked::text_fieldID;

richTextMarked::richTextMarked()
  : text_()
{}

richTextMarked::richTextMarked(object_ptr<RichText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t richTextMarked::ID;

object_ptr<RichText> richTextMarked::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextMarked> res = make_object<richTextMarked>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void richTextMarked::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
}

void richTextMarked::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextMarked");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void richTextMarked::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextMarked").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass richTextPhoneNumber::Class;
jfieldID richTextPhoneNumber::text_fieldID;
jfieldID richTextPhoneNumber::phone_number_fieldID;

richTextPhoneNumber::richTextPhoneNumber()
  : text_()
  , phone_number_()
{}

richTextPhoneNumber::richTextPhoneNumber(object_ptr<RichText> &&text_, string const &phone_number_)
  : text_(std::move(text_))
  , phone_number_(phone_number_)
{}

const std::int32_t richTextPhoneNumber::ID;

object_ptr<RichText> richTextPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextPhoneNumber> res = make_object<richTextPhoneNumber>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  return std::move(res);
}

void richTextPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextPhoneNumber");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("phoneNumber", phone_number_);
    s.store_class_end();
  }
}

void richTextPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextPhoneNumber").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTextIcon::Class;
jfieldID richTextIcon::document_fieldID;
jfieldID richTextIcon::width_fieldID;
jfieldID richTextIcon::height_fieldID;

richTextIcon::richTextIcon()
  : document_()
  , width_()
  , height_()
{}

richTextIcon::richTextIcon(object_ptr<document> &&document_, int32 width_, int32 height_)
  : document_(std::move(document_))
  , width_(width_)
  , height_(height_)
{}

const std::int32_t richTextIcon::ID;

object_ptr<RichText> richTextIcon::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextIcon> res = make_object<richTextIcon>();
  res->document_ = jni::fetch_tl_object<document>(env, jni::fetch_object(env, p, res->document_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  return std::move(res);
}

void richTextIcon::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (document_ != nullptr) { jobject next; document_->store(env, next); if (next) { env->SetObjectField(s, document_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
}

void richTextIcon::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextIcon");
    s.store_object_field("document", static_cast<const BaseObject *>(document_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_class_end();
  }
}

void richTextIcon::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextIcon").c_str());
    document_fieldID = jni::get_field_id(env, Class, "document", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Document;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    return true;
  }();
  (void)is_inited;
}

jclass richTextReference::Class;
jfieldID richTextReference::text_fieldID;
jfieldID richTextReference::anchor_name_fieldID;
jfieldID richTextReference::url_fieldID;

richTextReference::richTextReference()
  : text_()
  , anchor_name_()
  , url_()
{}

richTextReference::richTextReference(object_ptr<RichText> &&text_, string const &anchor_name_, string const &url_)
  : text_(std::move(text_))
  , anchor_name_(anchor_name_)
  , url_(url_)
{}

const std::int32_t richTextReference::ID;

object_ptr<RichText> richTextReference::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextReference> res = make_object<richTextReference>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->anchor_name_ = jni::fetch_string(env, p, res->anchor_name_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void richTextReference::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, anchor_name_); if (nextString) { env->SetObjectField(s, anchor_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextReference::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextReference");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("anchorName", anchor_name_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void richTextReference::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextReference").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    anchor_name_fieldID = jni::get_field_id(env, Class, "anchorName", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTextAnchor::Class;
jfieldID richTextAnchor::name_fieldID;

richTextAnchor::richTextAnchor()
  : name_()
{}

richTextAnchor::richTextAnchor(string const &name_)
  : name_(name_)
{}

const std::int32_t richTextAnchor::ID;

object_ptr<RichText> richTextAnchor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextAnchor> res = make_object<richTextAnchor>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return std::move(res);
}

void richTextAnchor::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextAnchor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextAnchor");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

void richTextAnchor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextAnchor").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTextAnchorLink::Class;
jfieldID richTextAnchorLink::text_fieldID;
jfieldID richTextAnchorLink::anchor_name_fieldID;
jfieldID richTextAnchorLink::url_fieldID;

richTextAnchorLink::richTextAnchorLink()
  : text_()
  , anchor_name_()
  , url_()
{}

richTextAnchorLink::richTextAnchorLink(object_ptr<RichText> &&text_, string const &anchor_name_, string const &url_)
  : text_(std::move(text_))
  , anchor_name_(anchor_name_)
  , url_(url_)
{}

const std::int32_t richTextAnchorLink::ID;

object_ptr<RichText> richTextAnchorLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTextAnchorLink> res = make_object<richTextAnchorLink>();
  res->text_ = jni::fetch_tl_object<RichText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->anchor_name_ = jni::fetch_string(env, p, res->anchor_name_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void richTextAnchorLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, anchor_name_); if (nextString) { env->SetObjectField(s, anchor_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void richTextAnchorLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTextAnchorLink");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("anchorName", anchor_name_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void richTextAnchorLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTextAnchorLink").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    anchor_name_fieldID = jni::get_field_id(env, Class, "anchorName", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass richTexts::Class;
jfieldID richTexts::texts_fieldID;

richTexts::richTexts()
  : texts_()
{}

richTexts::richTexts(array<object_ptr<RichText>> &&texts_)
  : texts_(std::move(texts_))
{}

const std::int32_t richTexts::ID;

object_ptr<RichText> richTexts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<richTexts> res = make_object<richTexts>();
  res->texts_ = jni::FetchVector<RichText>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->texts_fieldID));
  return std::move(res);
}

void richTexts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, texts_); if (arr_tmp_) { env->SetObjectField(s, texts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void richTexts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RichTexts");
    { s.store_vector_begin("texts", texts_.size()); for (const auto &_value : texts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void richTexts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RichTexts").c_str());
    texts_fieldID = jni::get_field_id(env, Class, "texts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$RichText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass rtmpUrl::Class;
jfieldID rtmpUrl::url_fieldID;
jfieldID rtmpUrl::stream_key_fieldID;

rtmpUrl::rtmpUrl()
  : url_()
  , stream_key_()
{}

rtmpUrl::rtmpUrl(string const &url_, string const &stream_key_)
  : url_(url_)
  , stream_key_(stream_key_)
{}

const std::int32_t rtmpUrl::ID;

object_ptr<rtmpUrl> rtmpUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<rtmpUrl> res = make_object<rtmpUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->stream_key_ = jni::fetch_string(env, p, res->stream_key_fieldID);
  return res;
}

void rtmpUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, stream_key_); if (nextString) { env->SetObjectField(s, stream_key_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void rtmpUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RtmpUrl");
    s.store_field("url", url_);
    s.store_field("streamKey", stream_key_);
    s.store_class_end();
  }
}

void rtmpUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RtmpUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    stream_key_fieldID = jni::get_field_id(env, Class, "streamKey", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass savedCredentials::Class;
jfieldID savedCredentials::id_fieldID;
jfieldID savedCredentials::title_fieldID;

savedCredentials::savedCredentials()
  : id_()
  , title_()
{}

savedCredentials::savedCredentials(string const &id_, string const &title_)
  : id_(id_)
  , title_(title_)
{}

const std::int32_t savedCredentials::ID;

object_ptr<savedCredentials> savedCredentials::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savedCredentials> res = make_object<savedCredentials>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void savedCredentials::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void savedCredentials::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedCredentials");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void savedCredentials::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedCredentials").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTag::Class;
jfieldID savedMessagesTag::tag_fieldID;
jfieldID savedMessagesTag::label_fieldID;
jfieldID savedMessagesTag::count_fieldID;

savedMessagesTag::savedMessagesTag()
  : tag_()
  , label_()
  , count_()
{}

savedMessagesTag::savedMessagesTag(object_ptr<ReactionType> &&tag_, string const &label_, int32 count_)
  : tag_(std::move(tag_))
  , label_(label_)
  , count_(count_)
{}

const std::int32_t savedMessagesTag::ID;

object_ptr<savedMessagesTag> savedMessagesTag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savedMessagesTag> res = make_object<savedMessagesTag>();
  res->tag_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->tag_fieldID));
  res->label_ = jni::fetch_string(env, p, res->label_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  return res;
}

void savedMessagesTag::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (tag_ != nullptr) { jobject next; tag_->store(env, next); if (next) { env->SetObjectField(s, tag_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, label_); if (nextString) { env->SetObjectField(s, label_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, count_fieldID, count_);
}

void savedMessagesTag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTag");
    s.store_object_field("tag", static_cast<const BaseObject *>(tag_.get()));
    s.store_field("label", label_);
    s.store_field("count", count_);
    s.store_class_end();
  }
}

void savedMessagesTag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTag").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    label_fieldID = jni::get_field_id(env, Class, "label", "Ljava/lang/String;");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTags::Class;
jfieldID savedMessagesTags::tags_fieldID;

savedMessagesTags::savedMessagesTags()
  : tags_()
{}

savedMessagesTags::savedMessagesTags(array<object_ptr<savedMessagesTag>> &&tags_)
  : tags_(std::move(tags_))
{}

const std::int32_t savedMessagesTags::ID;

object_ptr<savedMessagesTags> savedMessagesTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savedMessagesTags> res = make_object<savedMessagesTags>();
  res->tags_ = jni::FetchVector<savedMessagesTag>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->tags_fieldID));
  return res;
}

void savedMessagesTags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, tags_); if (arr_tmp_) { env->SetObjectField(s, tags_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void savedMessagesTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTags");
    { s.store_vector_begin("tags", tags_.size()); for (const auto &_value : tags_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void savedMessagesTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTags").c_str());
    tags_fieldID = jni::get_field_id(env, Class, "tags", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SavedMessagesTag;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTopic::Class;
jfieldID savedMessagesTopic::id_fieldID;
jfieldID savedMessagesTopic::type_fieldID;
jfieldID savedMessagesTopic::is_pinned_fieldID;
jfieldID savedMessagesTopic::order_fieldID;
jfieldID savedMessagesTopic::last_message_fieldID;
jfieldID savedMessagesTopic::draft_message_fieldID;

savedMessagesTopic::savedMessagesTopic()
  : id_()
  , type_()
  , is_pinned_()
  , order_()
  , last_message_()
  , draft_message_()
{}

savedMessagesTopic::savedMessagesTopic(int53 id_, object_ptr<SavedMessagesTopicType> &&type_, bool is_pinned_, int64 order_, object_ptr<message> &&last_message_, object_ptr<draftMessage> &&draft_message_)
  : id_(id_)
  , type_(std::move(type_))
  , is_pinned_(is_pinned_)
  , order_(order_)
  , last_message_(std::move(last_message_))
  , draft_message_(std::move(draft_message_))
{}

const std::int32_t savedMessagesTopic::ID;

object_ptr<savedMessagesTopic> savedMessagesTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savedMessagesTopic> res = make_object<savedMessagesTopic>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->type_ = jni::fetch_tl_object<SavedMessagesTopicType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->order_ = env->GetLongField(p, res->order_fieldID);
  res->last_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->last_message_fieldID));
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  return res;
}

void savedMessagesTopic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  env->SetLongField(s, order_fieldID, order_);
  if (last_message_ != nullptr) { jobject next; last_message_->store(env, next); if (next) { env->SetObjectField(s, last_message_fieldID, next); env->DeleteLocalRef(next); } }
  if (draft_message_ != nullptr) { jobject next; draft_message_->store(env, next); if (next) { env->SetObjectField(s, draft_message_fieldID, next); env->DeleteLocalRef(next); } }
}

void savedMessagesTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTopic");
    s.store_field("id", id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("isPinned", is_pinned_);
    s.store_field("order", order_);
    s.store_object_field("lastMessage", static_cast<const BaseObject *>(last_message_.get()));
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    s.store_class_end();
  }
}

void savedMessagesTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTopic").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SavedMessagesTopicType;").c_str());
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    order_fieldID = jni::get_field_id(env, Class, "order", "J");
    last_message_fieldID = jni::get_field_id(env, Class, "lastMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass SavedMessagesTopicType::Class;

object_ptr<SavedMessagesTopicType> SavedMessagesTopicType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case savedMessagesTopicTypeMyNotes::ID:
      return savedMessagesTopicTypeMyNotes::fetch(env, p);
    case savedMessagesTopicTypeAuthorHidden::ID:
      return savedMessagesTopicTypeAuthorHidden::fetch(env, p);
    case savedMessagesTopicTypeSavedFromChat::ID:
      return savedMessagesTopicTypeSavedFromChat::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SavedMessagesTopicType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTopicType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTopicTypeMyNotes::Class;

savedMessagesTopicTypeMyNotes::savedMessagesTopicTypeMyNotes() {
}

const std::int32_t savedMessagesTopicTypeMyNotes::ID;

object_ptr<SavedMessagesTopicType> savedMessagesTopicTypeMyNotes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<savedMessagesTopicTypeMyNotes>();
}

void savedMessagesTopicTypeMyNotes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void savedMessagesTopicTypeMyNotes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTopicTypeMyNotes");
    s.store_class_end();
  }
}

void savedMessagesTopicTypeMyNotes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTopicTypeMyNotes").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTopicTypeAuthorHidden::Class;

savedMessagesTopicTypeAuthorHidden::savedMessagesTopicTypeAuthorHidden() {
}

const std::int32_t savedMessagesTopicTypeAuthorHidden::ID;

object_ptr<SavedMessagesTopicType> savedMessagesTopicTypeAuthorHidden::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<savedMessagesTopicTypeAuthorHidden>();
}

void savedMessagesTopicTypeAuthorHidden::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void savedMessagesTopicTypeAuthorHidden::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTopicTypeAuthorHidden");
    s.store_class_end();
  }
}

void savedMessagesTopicTypeAuthorHidden::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTopicTypeAuthorHidden").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass savedMessagesTopicTypeSavedFromChat::Class;
jfieldID savedMessagesTopicTypeSavedFromChat::chat_id_fieldID;

savedMessagesTopicTypeSavedFromChat::savedMessagesTopicTypeSavedFromChat()
  : chat_id_()
{}

savedMessagesTopicTypeSavedFromChat::savedMessagesTopicTypeSavedFromChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t savedMessagesTopicTypeSavedFromChat::ID;

object_ptr<SavedMessagesTopicType> savedMessagesTopicTypeSavedFromChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savedMessagesTopicTypeSavedFromChat> res = make_object<savedMessagesTopicTypeSavedFromChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return std::move(res);
}

void savedMessagesTopicTypeSavedFromChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
}

void savedMessagesTopicTypeSavedFromChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavedMessagesTopicTypeSavedFromChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

void savedMessagesTopicTypeSavedFromChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavedMessagesTopicTypeSavedFromChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass scopeAutosaveSettings::Class;
jfieldID scopeAutosaveSettings::autosave_photos_fieldID;
jfieldID scopeAutosaveSettings::autosave_videos_fieldID;
jfieldID scopeAutosaveSettings::max_video_file_size_fieldID;

scopeAutosaveSettings::scopeAutosaveSettings()
  : autosave_photos_()
  , autosave_videos_()
  , max_video_file_size_()
{}

scopeAutosaveSettings::scopeAutosaveSettings(bool autosave_photos_, bool autosave_videos_, int53 max_video_file_size_)
  : autosave_photos_(autosave_photos_)
  , autosave_videos_(autosave_videos_)
  , max_video_file_size_(max_video_file_size_)
{}

const std::int32_t scopeAutosaveSettings::ID;

object_ptr<scopeAutosaveSettings> scopeAutosaveSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<scopeAutosaveSettings> res = make_object<scopeAutosaveSettings>();
  res->autosave_photos_ = (env->GetBooleanField(p, res->autosave_photos_fieldID) != 0);
  res->autosave_videos_ = (env->GetBooleanField(p, res->autosave_videos_fieldID) != 0);
  res->max_video_file_size_ = env->GetLongField(p, res->max_video_file_size_fieldID);
  return res;
}

void scopeAutosaveSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, autosave_photos_fieldID, autosave_photos_);
  env->SetBooleanField(s, autosave_videos_fieldID, autosave_videos_);
  env->SetLongField(s, max_video_file_size_fieldID, max_video_file_size_);
}

void scopeAutosaveSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ScopeAutosaveSettings");
    s.store_field("autosavePhotos", autosave_photos_);
    s.store_field("autosaveVideos", autosave_videos_);
    s.store_field("maxVideoFileSize", max_video_file_size_);
    s.store_class_end();
  }
}

void scopeAutosaveSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings").c_str());
    autosave_photos_fieldID = jni::get_field_id(env, Class, "autosavePhotos", "Z");
    autosave_videos_fieldID = jni::get_field_id(env, Class, "autosaveVideos", "Z");
    max_video_file_size_fieldID = jni::get_field_id(env, Class, "maxVideoFileSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass scopeNotificationSettings::Class;
jfieldID scopeNotificationSettings::mute_for_fieldID;
jfieldID scopeNotificationSettings::sound_id_fieldID;
jfieldID scopeNotificationSettings::show_preview_fieldID;
jfieldID scopeNotificationSettings::use_default_mute_stories_fieldID;
jfieldID scopeNotificationSettings::mute_stories_fieldID;
jfieldID scopeNotificationSettings::story_sound_id_fieldID;
jfieldID scopeNotificationSettings::show_story_poster_fieldID;
jfieldID scopeNotificationSettings::disable_pinned_message_notifications_fieldID;
jfieldID scopeNotificationSettings::disable_mention_notifications_fieldID;

scopeNotificationSettings::scopeNotificationSettings()
  : mute_for_()
  , sound_id_()
  , show_preview_()
  , use_default_mute_stories_()
  , mute_stories_()
  , story_sound_id_()
  , show_story_poster_()
  , disable_pinned_message_notifications_()
  , disable_mention_notifications_()
{}

scopeNotificationSettings::scopeNotificationSettings(int32 mute_for_, int64 sound_id_, bool show_preview_, bool use_default_mute_stories_, bool mute_stories_, int64 story_sound_id_, bool show_story_poster_, bool disable_pinned_message_notifications_, bool disable_mention_notifications_)
  : mute_for_(mute_for_)
  , sound_id_(sound_id_)
  , show_preview_(show_preview_)
  , use_default_mute_stories_(use_default_mute_stories_)
  , mute_stories_(mute_stories_)
  , story_sound_id_(story_sound_id_)
  , show_story_poster_(show_story_poster_)
  , disable_pinned_message_notifications_(disable_pinned_message_notifications_)
  , disable_mention_notifications_(disable_mention_notifications_)
{}

const std::int32_t scopeNotificationSettings::ID;

object_ptr<scopeNotificationSettings> scopeNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<scopeNotificationSettings> res = make_object<scopeNotificationSettings>();
  res->mute_for_ = env->GetIntField(p, res->mute_for_fieldID);
  res->sound_id_ = env->GetLongField(p, res->sound_id_fieldID);
  res->show_preview_ = (env->GetBooleanField(p, res->show_preview_fieldID) != 0);
  res->use_default_mute_stories_ = (env->GetBooleanField(p, res->use_default_mute_stories_fieldID) != 0);
  res->mute_stories_ = (env->GetBooleanField(p, res->mute_stories_fieldID) != 0);
  res->story_sound_id_ = env->GetLongField(p, res->story_sound_id_fieldID);
  res->show_story_poster_ = (env->GetBooleanField(p, res->show_story_poster_fieldID) != 0);
  res->disable_pinned_message_notifications_ = (env->GetBooleanField(p, res->disable_pinned_message_notifications_fieldID) != 0);
  res->disable_mention_notifications_ = (env->GetBooleanField(p, res->disable_mention_notifications_fieldID) != 0);
  return res;
}

void scopeNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, mute_for_fieldID, mute_for_);
  env->SetLongField(s, sound_id_fieldID, sound_id_);
  env->SetBooleanField(s, show_preview_fieldID, show_preview_);
  env->SetBooleanField(s, use_default_mute_stories_fieldID, use_default_mute_stories_);
  env->SetBooleanField(s, mute_stories_fieldID, mute_stories_);
  env->SetLongField(s, story_sound_id_fieldID, story_sound_id_);
  env->SetBooleanField(s, show_story_poster_fieldID, show_story_poster_);
  env->SetBooleanField(s, disable_pinned_message_notifications_fieldID, disable_pinned_message_notifications_);
  env->SetBooleanField(s, disable_mention_notifications_fieldID, disable_mention_notifications_);
}

void scopeNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ScopeNotificationSettings");
    s.store_field("muteFor", mute_for_);
    s.store_field("soundId", sound_id_);
    s.store_field("showPreview", show_preview_);
    s.store_field("useDefaultMuteStories", use_default_mute_stories_);
    s.store_field("muteStories", mute_stories_);
    s.store_field("storySoundId", story_sound_id_);
    s.store_field("showStoryPoster", show_story_poster_);
    s.store_field("disablePinnedMessageNotifications", disable_pinned_message_notifications_);
    s.store_field("disableMentionNotifications", disable_mention_notifications_);
    s.store_class_end();
  }
}

void scopeNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ScopeNotificationSettings").c_str());
    mute_for_fieldID = jni::get_field_id(env, Class, "muteFor", "I");
    sound_id_fieldID = jni::get_field_id(env, Class, "soundId", "J");
    show_preview_fieldID = jni::get_field_id(env, Class, "showPreview", "Z");
    use_default_mute_stories_fieldID = jni::get_field_id(env, Class, "useDefaultMuteStories", "Z");
    mute_stories_fieldID = jni::get_field_id(env, Class, "muteStories", "Z");
    story_sound_id_fieldID = jni::get_field_id(env, Class, "storySoundId", "J");
    show_story_poster_fieldID = jni::get_field_id(env, Class, "showStoryPoster", "Z");
    disable_pinned_message_notifications_fieldID = jni::get_field_id(env, Class, "disablePinnedMessageNotifications", "Z");
    disable_mention_notifications_fieldID = jni::get_field_id(env, Class, "disableMentionNotifications", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass SearchMessagesChatTypeFilter::Class;

object_ptr<SearchMessagesChatTypeFilter> SearchMessagesChatTypeFilter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case searchMessagesChatTypeFilterPrivate::ID:
      return searchMessagesChatTypeFilterPrivate::fetch(env, p);
    case searchMessagesChatTypeFilterGroup::ID:
      return searchMessagesChatTypeFilterGroup::fetch(env, p);
    case searchMessagesChatTypeFilterChannel::ID:
      return searchMessagesChatTypeFilterChannel::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SearchMessagesChatTypeFilter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesChatTypeFilter").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesChatTypeFilterPrivate::Class;

searchMessagesChatTypeFilterPrivate::searchMessagesChatTypeFilterPrivate() {
}

const std::int32_t searchMessagesChatTypeFilterPrivate::ID;

object_ptr<SearchMessagesChatTypeFilter> searchMessagesChatTypeFilterPrivate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesChatTypeFilterPrivate>();
}

void searchMessagesChatTypeFilterPrivate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesChatTypeFilterPrivate");
    s.store_class_end();
  }
}

void searchMessagesChatTypeFilterPrivate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesChatTypeFilterPrivate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesChatTypeFilterGroup::Class;

searchMessagesChatTypeFilterGroup::searchMessagesChatTypeFilterGroup() {
}

const std::int32_t searchMessagesChatTypeFilterGroup::ID;

object_ptr<SearchMessagesChatTypeFilter> searchMessagesChatTypeFilterGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesChatTypeFilterGroup>();
}

void searchMessagesChatTypeFilterGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesChatTypeFilterGroup");
    s.store_class_end();
  }
}

void searchMessagesChatTypeFilterGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesChatTypeFilterGroup").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesChatTypeFilterChannel::Class;

searchMessagesChatTypeFilterChannel::searchMessagesChatTypeFilterChannel() {
}

const std::int32_t searchMessagesChatTypeFilterChannel::ID;

object_ptr<SearchMessagesChatTypeFilter> searchMessagesChatTypeFilterChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesChatTypeFilterChannel>();
}

void searchMessagesChatTypeFilterChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesChatTypeFilterChannel");
    s.store_class_end();
  }
}

void searchMessagesChatTypeFilterChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesChatTypeFilterChannel").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass SearchMessagesFilter::Class;

object_ptr<SearchMessagesFilter> SearchMessagesFilter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case searchMessagesFilterEmpty::ID:
      return searchMessagesFilterEmpty::fetch(env, p);
    case searchMessagesFilterAnimation::ID:
      return searchMessagesFilterAnimation::fetch(env, p);
    case searchMessagesFilterAudio::ID:
      return searchMessagesFilterAudio::fetch(env, p);
    case searchMessagesFilterDocument::ID:
      return searchMessagesFilterDocument::fetch(env, p);
    case searchMessagesFilterPhoto::ID:
      return searchMessagesFilterPhoto::fetch(env, p);
    case searchMessagesFilterVideo::ID:
      return searchMessagesFilterVideo::fetch(env, p);
    case searchMessagesFilterVoiceNote::ID:
      return searchMessagesFilterVoiceNote::fetch(env, p);
    case searchMessagesFilterPhotoAndVideo::ID:
      return searchMessagesFilterPhotoAndVideo::fetch(env, p);
    case searchMessagesFilterUrl::ID:
      return searchMessagesFilterUrl::fetch(env, p);
    case searchMessagesFilterChatPhoto::ID:
      return searchMessagesFilterChatPhoto::fetch(env, p);
    case searchMessagesFilterVideoNote::ID:
      return searchMessagesFilterVideoNote::fetch(env, p);
    case searchMessagesFilterVoiceAndVideoNote::ID:
      return searchMessagesFilterVoiceAndVideoNote::fetch(env, p);
    case searchMessagesFilterMention::ID:
      return searchMessagesFilterMention::fetch(env, p);
    case searchMessagesFilterUnreadMention::ID:
      return searchMessagesFilterUnreadMention::fetch(env, p);
    case searchMessagesFilterUnreadReaction::ID:
      return searchMessagesFilterUnreadReaction::fetch(env, p);
    case searchMessagesFilterFailedToSend::ID:
      return searchMessagesFilterFailedToSend::fetch(env, p);
    case searchMessagesFilterPinned::ID:
      return searchMessagesFilterPinned::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SearchMessagesFilter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilter").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterEmpty::Class;

searchMessagesFilterEmpty::searchMessagesFilterEmpty() {
}

const std::int32_t searchMessagesFilterEmpty::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterEmpty>();
}

void searchMessagesFilterEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterEmpty");
    s.store_class_end();
  }
}

void searchMessagesFilterEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterAnimation::Class;

searchMessagesFilterAnimation::searchMessagesFilterAnimation() {
}

const std::int32_t searchMessagesFilterAnimation::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterAnimation>();
}

void searchMessagesFilterAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterAnimation");
    s.store_class_end();
  }
}

void searchMessagesFilterAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterAnimation").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterAudio::Class;

searchMessagesFilterAudio::searchMessagesFilterAudio() {
}

const std::int32_t searchMessagesFilterAudio::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterAudio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterAudio>();
}

void searchMessagesFilterAudio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterAudio");
    s.store_class_end();
  }
}

void searchMessagesFilterAudio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterAudio").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterDocument::Class;

searchMessagesFilterDocument::searchMessagesFilterDocument() {
}

const std::int32_t searchMessagesFilterDocument::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterDocument::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterDocument>();
}

void searchMessagesFilterDocument::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterDocument");
    s.store_class_end();
  }
}

void searchMessagesFilterDocument::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterDocument").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterPhoto::Class;

searchMessagesFilterPhoto::searchMessagesFilterPhoto() {
}

const std::int32_t searchMessagesFilterPhoto::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterPhoto>();
}

void searchMessagesFilterPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterPhoto");
    s.store_class_end();
  }
}

void searchMessagesFilterPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterVideo::Class;

searchMessagesFilterVideo::searchMessagesFilterVideo() {
}

const std::int32_t searchMessagesFilterVideo::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterVideo>();
}

void searchMessagesFilterVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterVideo");
    s.store_class_end();
  }
}

void searchMessagesFilterVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterVoiceNote::Class;

searchMessagesFilterVoiceNote::searchMessagesFilterVoiceNote() {
}

const std::int32_t searchMessagesFilterVoiceNote::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterVoiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterVoiceNote>();
}

void searchMessagesFilterVoiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterVoiceNote");
    s.store_class_end();
  }
}

void searchMessagesFilterVoiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterVoiceNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterPhotoAndVideo::Class;

searchMessagesFilterPhotoAndVideo::searchMessagesFilterPhotoAndVideo() {
}

const std::int32_t searchMessagesFilterPhotoAndVideo::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterPhotoAndVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterPhotoAndVideo>();
}

void searchMessagesFilterPhotoAndVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterPhotoAndVideo");
    s.store_class_end();
  }
}

void searchMessagesFilterPhotoAndVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterPhotoAndVideo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterUrl::Class;

searchMessagesFilterUrl::searchMessagesFilterUrl() {
}

const std::int32_t searchMessagesFilterUrl::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterUrl>();
}

void searchMessagesFilterUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterUrl");
    s.store_class_end();
  }
}

void searchMessagesFilterUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterUrl").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterChatPhoto::Class;

searchMessagesFilterChatPhoto::searchMessagesFilterChatPhoto() {
}

const std::int32_t searchMessagesFilterChatPhoto::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterChatPhoto>();
}

void searchMessagesFilterChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterChatPhoto");
    s.store_class_end();
  }
}

void searchMessagesFilterChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterChatPhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterVideoNote::Class;

searchMessagesFilterVideoNote::searchMessagesFilterVideoNote() {
}

const std::int32_t searchMessagesFilterVideoNote::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterVideoNote>();
}

void searchMessagesFilterVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterVideoNote");
    s.store_class_end();
  }
}

void searchMessagesFilterVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterVoiceAndVideoNote::Class;

searchMessagesFilterVoiceAndVideoNote::searchMessagesFilterVoiceAndVideoNote() {
}

const std::int32_t searchMessagesFilterVoiceAndVideoNote::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterVoiceAndVideoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterVoiceAndVideoNote>();
}

void searchMessagesFilterVoiceAndVideoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterVoiceAndVideoNote");
    s.store_class_end();
  }
}

void searchMessagesFilterVoiceAndVideoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterVoiceAndVideoNote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterMention::Class;

searchMessagesFilterMention::searchMessagesFilterMention() {
}

const std::int32_t searchMessagesFilterMention::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterMention::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterMention>();
}

void searchMessagesFilterMention::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterMention");
    s.store_class_end();
  }
}

void searchMessagesFilterMention::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterMention").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterUnreadMention::Class;

searchMessagesFilterUnreadMention::searchMessagesFilterUnreadMention() {
}

const std::int32_t searchMessagesFilterUnreadMention::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterUnreadMention::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterUnreadMention>();
}

void searchMessagesFilterUnreadMention::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterUnreadMention");
    s.store_class_end();
  }
}

void searchMessagesFilterUnreadMention::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterUnreadMention").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterUnreadReaction::Class;

searchMessagesFilterUnreadReaction::searchMessagesFilterUnreadReaction() {
}

const std::int32_t searchMessagesFilterUnreadReaction::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterUnreadReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterUnreadReaction>();
}

void searchMessagesFilterUnreadReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterUnreadReaction");
    s.store_class_end();
  }
}

void searchMessagesFilterUnreadReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterUnreadReaction").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterFailedToSend::Class;

searchMessagesFilterFailedToSend::searchMessagesFilterFailedToSend() {
}

const std::int32_t searchMessagesFilterFailedToSend::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterFailedToSend::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterFailedToSend>();
}

void searchMessagesFilterFailedToSend::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterFailedToSend");
    s.store_class_end();
  }
}

void searchMessagesFilterFailedToSend::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterFailedToSend").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchMessagesFilterPinned::Class;

searchMessagesFilterPinned::searchMessagesFilterPinned() {
}

const std::int32_t searchMessagesFilterPinned::ID;

object_ptr<SearchMessagesFilter> searchMessagesFilterPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<searchMessagesFilterPinned>();
}

void searchMessagesFilterPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessagesFilterPinned");
    s.store_class_end();
  }
}

void searchMessagesFilterPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessagesFilterPinned").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass seconds::Class;
jfieldID seconds::seconds_fieldID;

seconds::seconds()
  : seconds_()
{}

seconds::seconds(double seconds_)
  : seconds_(seconds_)
{}

const std::int32_t seconds::ID;

object_ptr<seconds> seconds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<seconds> res = make_object<seconds>();
  res->seconds_ = env->GetDoubleField(p, res->seconds_fieldID);
  return res;
}

void seconds::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, seconds_fieldID, seconds_);
}

void seconds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Seconds");
    s.store_field("seconds", seconds_);
    s.store_class_end();
  }
}

void seconds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Seconds").c_str());
    seconds_fieldID = jni::get_field_id(env, Class, "seconds", "D");
    return true;
  }();
  (void)is_inited;
}

jclass secretChat::Class;
jfieldID secretChat::id_fieldID;
jfieldID secretChat::user_id_fieldID;
jfieldID secretChat::state_fieldID;
jfieldID secretChat::is_outbound_fieldID;
jfieldID secretChat::key_hash_fieldID;
jfieldID secretChat::layer_fieldID;

secretChat::secretChat()
  : id_()
  , user_id_()
  , state_()
  , is_outbound_()
  , key_hash_()
  , layer_()
{}

secretChat::secretChat(int32 id_, int53 user_id_, object_ptr<SecretChatState> &&state_, bool is_outbound_, bytes const &key_hash_, int32 layer_)
  : id_(id_)
  , user_id_(user_id_)
  , state_(std::move(state_))
  , is_outbound_(is_outbound_)
  , key_hash_(std::move(key_hash_))
  , layer_(layer_)
{}

const std::int32_t secretChat::ID;

object_ptr<secretChat> secretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<secretChat> res = make_object<secretChat>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->state_ = jni::fetch_tl_object<SecretChatState>(env, jni::fetch_object(env, p, res->state_fieldID));
  res->is_outbound_ = (env->GetBooleanField(p, res->is_outbound_fieldID) != 0);
  res->key_hash_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->key_hash_fieldID));
  res->layer_ = env->GetIntField(p, res->layer_fieldID);
  return res;
}

void secretChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (state_ != nullptr) { jobject next; state_->store(env, next); if (next) { env->SetObjectField(s, state_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_outbound_fieldID, is_outbound_);
  { jbyteArray nextBytes = jni::to_bytes(env, key_hash_); if (nextBytes) { env->SetObjectField(s, key_hash_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  env->SetIntField(s, layer_fieldID, layer_);
}

void secretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SecretChat");
    s.store_field("id", id_);
    s.store_field("userId", user_id_);
    s.store_object_field("state", static_cast<const BaseObject *>(state_.get()));
    s.store_field("isOutbound", is_outbound_);
    s.store_bytes_field("keyHash", key_hash_);
    s.store_field("layer", layer_);
    s.store_class_end();
  }
}

void secretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SecretChat").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    state_fieldID = jni::get_field_id(env, Class, "state", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SecretChatState;").c_str());
    is_outbound_fieldID = jni::get_field_id(env, Class, "isOutbound", "Z");
    key_hash_fieldID = jni::get_field_id(env, Class, "keyHash", "[B");
    layer_fieldID = jni::get_field_id(env, Class, "layer", "I");
    return true;
  }();
  (void)is_inited;
}

jclass SecretChatState::Class;

object_ptr<SecretChatState> SecretChatState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case secretChatStatePending::ID:
      return secretChatStatePending::fetch(env, p);
    case secretChatStateReady::ID:
      return secretChatStateReady::fetch(env, p);
    case secretChatStateClosed::ID:
      return secretChatStateClosed::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SecretChatState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SecretChatState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass secretChatStatePending::Class;

secretChatStatePending::secretChatStatePending() {
}

const std::int32_t secretChatStatePending::ID;

object_ptr<SecretChatState> secretChatStatePending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<secretChatStatePending>();
}

void secretChatStatePending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void secretChatStatePending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SecretChatStatePending");
    s.store_class_end();
  }
}

void secretChatStatePending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SecretChatStatePending").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass secretChatStateReady::Class;

secretChatStateReady::secretChatStateReady() {
}

const std::int32_t secretChatStateReady::ID;

object_ptr<SecretChatState> secretChatStateReady::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<secretChatStateReady>();
}

void secretChatStateReady::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void secretChatStateReady::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SecretChatStateReady");
    s.store_class_end();
  }
}

void secretChatStateReady::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SecretChatStateReady").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass secretChatStateClosed::Class;

secretChatStateClosed::secretChatStateClosed() {
}

const std::int32_t secretChatStateClosed::ID;

object_ptr<SecretChatState> secretChatStateClosed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<secretChatStateClosed>();
}

void secretChatStateClosed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void secretChatStateClosed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SecretChatStateClosed");
    s.store_class_end();
  }
}

void secretChatStateClosed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SecretChatStateClosed").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass SentGift::Class;

object_ptr<SentGift> SentGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case sentGiftRegular::ID:
      return sentGiftRegular::fetch(env, p);
    case sentGiftUpgraded::ID:
      return sentGiftUpgraded::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SentGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SentGift").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sentGiftRegular::Class;
jfieldID sentGiftRegular::gift_fieldID;

sentGiftRegular::sentGiftRegular()
  : gift_()
{}

sentGiftRegular::sentGiftRegular(object_ptr<gift> &&gift_)
  : gift_(std::move(gift_))
{}

const std::int32_t sentGiftRegular::ID;

object_ptr<SentGift> sentGiftRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sentGiftRegular> res = make_object<sentGiftRegular>();
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void sentGiftRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void sentGiftRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SentGiftRegular");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void sentGiftRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SentGiftRegular").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sentGiftUpgraded::Class;
jfieldID sentGiftUpgraded::gift_fieldID;

sentGiftUpgraded::sentGiftUpgraded()
  : gift_()
{}

sentGiftUpgraded::sentGiftUpgraded(object_ptr<upgradedGift> &&gift_)
  : gift_(std::move(gift_))
{}

const std::int32_t sentGiftUpgraded::ID;

object_ptr<SentGift> sentGiftUpgraded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sentGiftUpgraded> res = make_object<sentGiftUpgraded>();
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void sentGiftUpgraded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void sentGiftUpgraded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SentGiftUpgraded");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void sentGiftUpgraded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SentGiftUpgraded").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sentWebAppMessage::Class;
jfieldID sentWebAppMessage::inline_message_id_fieldID;

sentWebAppMessage::sentWebAppMessage()
  : inline_message_id_()
{}

sentWebAppMessage::sentWebAppMessage(string const &inline_message_id_)
  : inline_message_id_(inline_message_id_)
{}

const std::int32_t sentWebAppMessage::ID;

object_ptr<sentWebAppMessage> sentWebAppMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sentWebAppMessage> res = make_object<sentWebAppMessage>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  return res;
}

void sentWebAppMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, inline_message_id_); if (nextString) { env->SetObjectField(s, inline_message_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void sentWebAppMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SentWebAppMessage");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_class_end();
  }
}

void sentWebAppMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SentWebAppMessage").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass session::Class;
jfieldID session::id_fieldID;
jfieldID session::is_current_fieldID;
jfieldID session::is_password_pending_fieldID;
jfieldID session::is_unconfirmed_fieldID;
jfieldID session::can_accept_secret_chats_fieldID;
jfieldID session::can_accept_calls_fieldID;
jfieldID session::type_fieldID;
jfieldID session::api_id_fieldID;
jfieldID session::application_name_fieldID;
jfieldID session::application_version_fieldID;
jfieldID session::is_official_application_fieldID;
jfieldID session::device_model_fieldID;
jfieldID session::platform_fieldID;
jfieldID session::system_version_fieldID;
jfieldID session::log_in_date_fieldID;
jfieldID session::last_active_date_fieldID;
jfieldID session::ip_address_fieldID;
jfieldID session::location_fieldID;

session::session()
  : id_()
  , is_current_()
  , is_password_pending_()
  , is_unconfirmed_()
  , can_accept_secret_chats_()
  , can_accept_calls_()
  , type_()
  , api_id_()
  , application_name_()
  , application_version_()
  , is_official_application_()
  , device_model_()
  , platform_()
  , system_version_()
  , log_in_date_()
  , last_active_date_()
  , ip_address_()
  , location_()
{}

session::session(int64 id_, bool is_current_, bool is_password_pending_, bool is_unconfirmed_, bool can_accept_secret_chats_, bool can_accept_calls_, object_ptr<SessionType> &&type_, int32 api_id_, string const &application_name_, string const &application_version_, bool is_official_application_, string const &device_model_, string const &platform_, string const &system_version_, int32 log_in_date_, int32 last_active_date_, string const &ip_address_, string const &location_)
  : id_(id_)
  , is_current_(is_current_)
  , is_password_pending_(is_password_pending_)
  , is_unconfirmed_(is_unconfirmed_)
  , can_accept_secret_chats_(can_accept_secret_chats_)
  , can_accept_calls_(can_accept_calls_)
  , type_(std::move(type_))
  , api_id_(api_id_)
  , application_name_(application_name_)
  , application_version_(application_version_)
  , is_official_application_(is_official_application_)
  , device_model_(device_model_)
  , platform_(platform_)
  , system_version_(system_version_)
  , log_in_date_(log_in_date_)
  , last_active_date_(last_active_date_)
  , ip_address_(ip_address_)
  , location_(location_)
{}

const std::int32_t session::ID;

object_ptr<session> session::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<session> res = make_object<session>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->is_current_ = (env->GetBooleanField(p, res->is_current_fieldID) != 0);
  res->is_password_pending_ = (env->GetBooleanField(p, res->is_password_pending_fieldID) != 0);
  res->is_unconfirmed_ = (env->GetBooleanField(p, res->is_unconfirmed_fieldID) != 0);
  res->can_accept_secret_chats_ = (env->GetBooleanField(p, res->can_accept_secret_chats_fieldID) != 0);
  res->can_accept_calls_ = (env->GetBooleanField(p, res->can_accept_calls_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<SessionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->api_id_ = env->GetIntField(p, res->api_id_fieldID);
  res->application_name_ = jni::fetch_string(env, p, res->application_name_fieldID);
  res->application_version_ = jni::fetch_string(env, p, res->application_version_fieldID);
  res->is_official_application_ = (env->GetBooleanField(p, res->is_official_application_fieldID) != 0);
  res->device_model_ = jni::fetch_string(env, p, res->device_model_fieldID);
  res->platform_ = jni::fetch_string(env, p, res->platform_fieldID);
  res->system_version_ = jni::fetch_string(env, p, res->system_version_fieldID);
  res->log_in_date_ = env->GetIntField(p, res->log_in_date_fieldID);
  res->last_active_date_ = env->GetIntField(p, res->last_active_date_fieldID);
  res->ip_address_ = jni::fetch_string(env, p, res->ip_address_fieldID);
  res->location_ = jni::fetch_string(env, p, res->location_fieldID);
  return res;
}

void session::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetBooleanField(s, is_current_fieldID, is_current_);
  env->SetBooleanField(s, is_password_pending_fieldID, is_password_pending_);
  env->SetBooleanField(s, is_unconfirmed_fieldID, is_unconfirmed_);
  env->SetBooleanField(s, can_accept_secret_chats_fieldID, can_accept_secret_chats_);
  env->SetBooleanField(s, can_accept_calls_fieldID, can_accept_calls_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, api_id_fieldID, api_id_);
  { jstring nextString = jni::to_jstring(env, application_name_); if (nextString) { env->SetObjectField(s, application_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, application_version_); if (nextString) { env->SetObjectField(s, application_version_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_official_application_fieldID, is_official_application_);
  { jstring nextString = jni::to_jstring(env, device_model_); if (nextString) { env->SetObjectField(s, device_model_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, platform_); if (nextString) { env->SetObjectField(s, platform_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, system_version_); if (nextString) { env->SetObjectField(s, system_version_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, log_in_date_fieldID, log_in_date_);
  env->SetIntField(s, last_active_date_fieldID, last_active_date_);
  { jstring nextString = jni::to_jstring(env, ip_address_); if (nextString) { env->SetObjectField(s, ip_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, location_); if (nextString) { env->SetObjectField(s, location_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void session::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Session");
    s.store_field("id", id_);
    s.store_field("isCurrent", is_current_);
    s.store_field("isPasswordPending", is_password_pending_);
    s.store_field("isUnconfirmed", is_unconfirmed_);
    s.store_field("canAcceptSecretChats", can_accept_secret_chats_);
    s.store_field("canAcceptCalls", can_accept_calls_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("apiId", api_id_);
    s.store_field("applicationName", application_name_);
    s.store_field("applicationVersion", application_version_);
    s.store_field("isOfficialApplication", is_official_application_);
    s.store_field("deviceModel", device_model_);
    s.store_field("platform", platform_);
    s.store_field("systemVersion", system_version_);
    s.store_field("logInDate", log_in_date_);
    s.store_field("lastActiveDate", last_active_date_);
    s.store_field("ipAddress", ip_address_);
    s.store_field("location", location_);
    s.store_class_end();
  }
}

void session::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Session").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    is_current_fieldID = jni::get_field_id(env, Class, "isCurrent", "Z");
    is_password_pending_fieldID = jni::get_field_id(env, Class, "isPasswordPending", "Z");
    is_unconfirmed_fieldID = jni::get_field_id(env, Class, "isUnconfirmed", "Z");
    can_accept_secret_chats_fieldID = jni::get_field_id(env, Class, "canAcceptSecretChats", "Z");
    can_accept_calls_fieldID = jni::get_field_id(env, Class, "canAcceptCalls", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SessionType;").c_str());
    api_id_fieldID = jni::get_field_id(env, Class, "apiId", "I");
    application_name_fieldID = jni::get_field_id(env, Class, "applicationName", "Ljava/lang/String;");
    application_version_fieldID = jni::get_field_id(env, Class, "applicationVersion", "Ljava/lang/String;");
    is_official_application_fieldID = jni::get_field_id(env, Class, "isOfficialApplication", "Z");
    device_model_fieldID = jni::get_field_id(env, Class, "deviceModel", "Ljava/lang/String;");
    platform_fieldID = jni::get_field_id(env, Class, "platform", "Ljava/lang/String;");
    system_version_fieldID = jni::get_field_id(env, Class, "systemVersion", "Ljava/lang/String;");
    log_in_date_fieldID = jni::get_field_id(env, Class, "logInDate", "I");
    last_active_date_fieldID = jni::get_field_id(env, Class, "lastActiveDate", "I");
    ip_address_fieldID = jni::get_field_id(env, Class, "ipAddress", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass SessionType::Class;

object_ptr<SessionType> SessionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case sessionTypeAndroid::ID:
      return sessionTypeAndroid::fetch(env, p);
    case sessionTypeApple::ID:
      return sessionTypeApple::fetch(env, p);
    case sessionTypeBrave::ID:
      return sessionTypeBrave::fetch(env, p);
    case sessionTypeChrome::ID:
      return sessionTypeChrome::fetch(env, p);
    case sessionTypeEdge::ID:
      return sessionTypeEdge::fetch(env, p);
    case sessionTypeFirefox::ID:
      return sessionTypeFirefox::fetch(env, p);
    case sessionTypeIpad::ID:
      return sessionTypeIpad::fetch(env, p);
    case sessionTypeIphone::ID:
      return sessionTypeIphone::fetch(env, p);
    case sessionTypeLinux::ID:
      return sessionTypeLinux::fetch(env, p);
    case sessionTypeMac::ID:
      return sessionTypeMac::fetch(env, p);
    case sessionTypeOpera::ID:
      return sessionTypeOpera::fetch(env, p);
    case sessionTypeSafari::ID:
      return sessionTypeSafari::fetch(env, p);
    case sessionTypeUbuntu::ID:
      return sessionTypeUbuntu::fetch(env, p);
    case sessionTypeUnknown::ID:
      return sessionTypeUnknown::fetch(env, p);
    case sessionTypeVivaldi::ID:
      return sessionTypeVivaldi::fetch(env, p);
    case sessionTypeWindows::ID:
      return sessionTypeWindows::fetch(env, p);
    case sessionTypeXbox::ID:
      return sessionTypeXbox::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SessionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeAndroid::Class;

sessionTypeAndroid::sessionTypeAndroid() {
}

const std::int32_t sessionTypeAndroid::ID;

object_ptr<SessionType> sessionTypeAndroid::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeAndroid>();
}

void sessionTypeAndroid::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeAndroid::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeAndroid");
    s.store_class_end();
  }
}

void sessionTypeAndroid::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeAndroid").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeApple::Class;

sessionTypeApple::sessionTypeApple() {
}

const std::int32_t sessionTypeApple::ID;

object_ptr<SessionType> sessionTypeApple::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeApple>();
}

void sessionTypeApple::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeApple::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeApple");
    s.store_class_end();
  }
}

void sessionTypeApple::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeApple").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeBrave::Class;

sessionTypeBrave::sessionTypeBrave() {
}

const std::int32_t sessionTypeBrave::ID;

object_ptr<SessionType> sessionTypeBrave::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeBrave>();
}

void sessionTypeBrave::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeBrave::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeBrave");
    s.store_class_end();
  }
}

void sessionTypeBrave::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeBrave").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeChrome::Class;

sessionTypeChrome::sessionTypeChrome() {
}

const std::int32_t sessionTypeChrome::ID;

object_ptr<SessionType> sessionTypeChrome::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeChrome>();
}

void sessionTypeChrome::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeChrome::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeChrome");
    s.store_class_end();
  }
}

void sessionTypeChrome::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeChrome").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeEdge::Class;

sessionTypeEdge::sessionTypeEdge() {
}

const std::int32_t sessionTypeEdge::ID;

object_ptr<SessionType> sessionTypeEdge::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeEdge>();
}

void sessionTypeEdge::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeEdge::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeEdge");
    s.store_class_end();
  }
}

void sessionTypeEdge::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeEdge").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeFirefox::Class;

sessionTypeFirefox::sessionTypeFirefox() {
}

const std::int32_t sessionTypeFirefox::ID;

object_ptr<SessionType> sessionTypeFirefox::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeFirefox>();
}

void sessionTypeFirefox::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeFirefox::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeFirefox");
    s.store_class_end();
  }
}

void sessionTypeFirefox::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeFirefox").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeIpad::Class;

sessionTypeIpad::sessionTypeIpad() {
}

const std::int32_t sessionTypeIpad::ID;

object_ptr<SessionType> sessionTypeIpad::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeIpad>();
}

void sessionTypeIpad::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeIpad::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeIpad");
    s.store_class_end();
  }
}

void sessionTypeIpad::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeIpad").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeIphone::Class;

sessionTypeIphone::sessionTypeIphone() {
}

const std::int32_t sessionTypeIphone::ID;

object_ptr<SessionType> sessionTypeIphone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeIphone>();
}

void sessionTypeIphone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeIphone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeIphone");
    s.store_class_end();
  }
}

void sessionTypeIphone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeIphone").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeLinux::Class;

sessionTypeLinux::sessionTypeLinux() {
}

const std::int32_t sessionTypeLinux::ID;

object_ptr<SessionType> sessionTypeLinux::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeLinux>();
}

void sessionTypeLinux::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeLinux::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeLinux");
    s.store_class_end();
  }
}

void sessionTypeLinux::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeLinux").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeMac::Class;

sessionTypeMac::sessionTypeMac() {
}

const std::int32_t sessionTypeMac::ID;

object_ptr<SessionType> sessionTypeMac::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeMac>();
}

void sessionTypeMac::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeMac::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeMac");
    s.store_class_end();
  }
}

void sessionTypeMac::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeMac").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeOpera::Class;

sessionTypeOpera::sessionTypeOpera() {
}

const std::int32_t sessionTypeOpera::ID;

object_ptr<SessionType> sessionTypeOpera::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeOpera>();
}

void sessionTypeOpera::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeOpera::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeOpera");
    s.store_class_end();
  }
}

void sessionTypeOpera::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeOpera").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeSafari::Class;

sessionTypeSafari::sessionTypeSafari() {
}

const std::int32_t sessionTypeSafari::ID;

object_ptr<SessionType> sessionTypeSafari::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeSafari>();
}

void sessionTypeSafari::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeSafari::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeSafari");
    s.store_class_end();
  }
}

void sessionTypeSafari::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeSafari").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeUbuntu::Class;

sessionTypeUbuntu::sessionTypeUbuntu() {
}

const std::int32_t sessionTypeUbuntu::ID;

object_ptr<SessionType> sessionTypeUbuntu::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeUbuntu>();
}

void sessionTypeUbuntu::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeUbuntu::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeUbuntu");
    s.store_class_end();
  }
}

void sessionTypeUbuntu::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeUbuntu").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeUnknown::Class;

sessionTypeUnknown::sessionTypeUnknown() {
}

const std::int32_t sessionTypeUnknown::ID;

object_ptr<SessionType> sessionTypeUnknown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeUnknown>();
}

void sessionTypeUnknown::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeUnknown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeUnknown");
    s.store_class_end();
  }
}

void sessionTypeUnknown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeUnknown").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeVivaldi::Class;

sessionTypeVivaldi::sessionTypeVivaldi() {
}

const std::int32_t sessionTypeVivaldi::ID;

object_ptr<SessionType> sessionTypeVivaldi::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeVivaldi>();
}

void sessionTypeVivaldi::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeVivaldi::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeVivaldi");
    s.store_class_end();
  }
}

void sessionTypeVivaldi::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeVivaldi").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeWindows::Class;

sessionTypeWindows::sessionTypeWindows() {
}

const std::int32_t sessionTypeWindows::ID;

object_ptr<SessionType> sessionTypeWindows::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeWindows>();
}

void sessionTypeWindows::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeWindows::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeWindows");
    s.store_class_end();
  }
}

void sessionTypeWindows::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeWindows").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessionTypeXbox::Class;

sessionTypeXbox::sessionTypeXbox() {
}

const std::int32_t sessionTypeXbox::ID;

object_ptr<SessionType> sessionTypeXbox::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<sessionTypeXbox>();
}

void sessionTypeXbox::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void sessionTypeXbox::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SessionTypeXbox");
    s.store_class_end();
  }
}

void sessionTypeXbox::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SessionTypeXbox").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sessions::Class;
jfieldID sessions::sessions_fieldID;
jfieldID sessions::inactive_session_ttl_days_fieldID;

sessions::sessions()
  : sessions_()
  , inactive_session_ttl_days_()
{}

sessions::sessions(array<object_ptr<session>> &&sessions_, int32 inactive_session_ttl_days_)
  : sessions_(std::move(sessions_))
  , inactive_session_ttl_days_(inactive_session_ttl_days_)
{}

const std::int32_t sessions::ID;

object_ptr<sessions> sessions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sessions> res = make_object<sessions>();
  res->sessions_ = jni::FetchVector<session>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->sessions_fieldID));
  res->inactive_session_ttl_days_ = env->GetIntField(p, res->inactive_session_ttl_days_fieldID);
  return res;
}

void sessions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, sessions_); if (arr_tmp_) { env->SetObjectField(s, sessions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, inactive_session_ttl_days_fieldID, inactive_session_ttl_days_);
}

void sessions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Sessions");
    { s.store_vector_begin("sessions", sessions_.size()); for (const auto &_value : sessions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("inactiveSessionTtlDays", inactive_session_ttl_days_);
    s.store_class_end();
  }
}

void sessions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Sessions").c_str());
    sessions_fieldID = jni::get_field_id(env, Class, "sessions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Session;").c_str());
    inactive_session_ttl_days_fieldID = jni::get_field_id(env, Class, "inactiveSessionTtlDays", "I");
    return true;
  }();
  (void)is_inited;
}

jclass sharedChat::Class;
jfieldID sharedChat::chat_id_fieldID;
jfieldID sharedChat::title_fieldID;
jfieldID sharedChat::username_fieldID;
jfieldID sharedChat::photo_fieldID;

sharedChat::sharedChat()
  : chat_id_()
  , title_()
  , username_()
  , photo_()
{}

sharedChat::sharedChat(int53 chat_id_, string const &title_, string const &username_, object_ptr<photo> &&photo_)
  : chat_id_(chat_id_)
  , title_(title_)
  , username_(username_)
  , photo_(std::move(photo_))
{}

const std::int32_t sharedChat::ID;

object_ptr<sharedChat> sharedChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sharedChat> res = make_object<sharedChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void sharedChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void sharedChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SharedChat");
    s.store_field("chatId", chat_id_);
    s.store_field("title", title_);
    s.store_field("username", username_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void sharedChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SharedChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sharedUser::Class;
jfieldID sharedUser::user_id_fieldID;
jfieldID sharedUser::first_name_fieldID;
jfieldID sharedUser::last_name_fieldID;
jfieldID sharedUser::username_fieldID;
jfieldID sharedUser::photo_fieldID;

sharedUser::sharedUser()
  : user_id_()
  , first_name_()
  , last_name_()
  , username_()
  , photo_()
{}

sharedUser::sharedUser(int53 user_id_, string const &first_name_, string const &last_name_, string const &username_, object_ptr<photo> &&photo_)
  : user_id_(user_id_)
  , first_name_(first_name_)
  , last_name_(last_name_)
  , username_(username_)
  , photo_(std::move(photo_))
{}

const std::int32_t sharedUser::ID;

object_ptr<sharedUser> sharedUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sharedUser> res = make_object<sharedUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void sharedUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { jstring nextString = jni::to_jstring(env, first_name_); if (nextString) { env->SetObjectField(s, first_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, last_name_); if (nextString) { env->SetObjectField(s, last_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, username_); if (nextString) { env->SetObjectField(s, username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void sharedUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SharedUser");
    s.store_field("userId", user_id_);
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_field("username", username_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void sharedUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SharedUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass shippingOption::Class;
jfieldID shippingOption::id_fieldID;
jfieldID shippingOption::title_fieldID;
jfieldID shippingOption::price_parts_fieldID;

shippingOption::shippingOption()
  : id_()
  , title_()
  , price_parts_()
{}

shippingOption::shippingOption(string const &id_, string const &title_, array<object_ptr<labeledPricePart>> &&price_parts_)
  : id_(id_)
  , title_(title_)
  , price_parts_(std::move(price_parts_))
{}

const std::int32_t shippingOption::ID;

object_ptr<shippingOption> shippingOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<shippingOption> res = make_object<shippingOption>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->price_parts_ = jni::FetchVector<labeledPricePart>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->price_parts_fieldID));
  return res;
}

void shippingOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, price_parts_); if (arr_tmp_) { env->SetObjectField(s, price_parts_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void shippingOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ShippingOption");
    s.store_field("id", id_);
    s.store_field("title", title_);
    { s.store_vector_begin("priceParts", price_parts_.size()); for (const auto &_value : price_parts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void shippingOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ShippingOption").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    price_parts_fieldID = jni::get_field_id(env, Class, "priceParts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LabeledPricePart;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass SpeechRecognitionResult::Class;

object_ptr<SpeechRecognitionResult> SpeechRecognitionResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case speechRecognitionResultPending::ID:
      return speechRecognitionResultPending::fetch(env, p);
    case speechRecognitionResultText::ID:
      return speechRecognitionResultText::fetch(env, p);
    case speechRecognitionResultError::ID:
      return speechRecognitionResultError::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SpeechRecognitionResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SpeechRecognitionResult").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass speechRecognitionResultPending::Class;
jfieldID speechRecognitionResultPending::partial_text_fieldID;

speechRecognitionResultPending::speechRecognitionResultPending()
  : partial_text_()
{}

speechRecognitionResultPending::speechRecognitionResultPending(string const &partial_text_)
  : partial_text_(partial_text_)
{}

const std::int32_t speechRecognitionResultPending::ID;

object_ptr<SpeechRecognitionResult> speechRecognitionResultPending::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<speechRecognitionResultPending> res = make_object<speechRecognitionResultPending>();
  res->partial_text_ = jni::fetch_string(env, p, res->partial_text_fieldID);
  return std::move(res);
}

void speechRecognitionResultPending::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, partial_text_); if (nextString) { env->SetObjectField(s, partial_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void speechRecognitionResultPending::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SpeechRecognitionResultPending");
    s.store_field("partialText", partial_text_);
    s.store_class_end();
  }
}

void speechRecognitionResultPending::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SpeechRecognitionResultPending").c_str());
    partial_text_fieldID = jni::get_field_id(env, Class, "partialText", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass speechRecognitionResultText::Class;
jfieldID speechRecognitionResultText::text_fieldID;

speechRecognitionResultText::speechRecognitionResultText()
  : text_()
{}

speechRecognitionResultText::speechRecognitionResultText(string const &text_)
  : text_(text_)
{}

const std::int32_t speechRecognitionResultText::ID;

object_ptr<SpeechRecognitionResult> speechRecognitionResultText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<speechRecognitionResultText> res = make_object<speechRecognitionResultText>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return std::move(res);
}

void speechRecognitionResultText::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void speechRecognitionResultText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SpeechRecognitionResultText");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void speechRecognitionResultText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SpeechRecognitionResultText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass speechRecognitionResultError::Class;
jfieldID speechRecognitionResultError::error_fieldID;

speechRecognitionResultError::speechRecognitionResultError()
  : error_()
{}

speechRecognitionResultError::speechRecognitionResultError(object_ptr<error> &&error_)
  : error_(std::move(error_))
{}

const std::int32_t speechRecognitionResultError::ID;

object_ptr<SpeechRecognitionResult> speechRecognitionResultError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<speechRecognitionResultError> res = make_object<speechRecognitionResultError>();
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  return std::move(res);
}

void speechRecognitionResultError::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (error_ != nullptr) { jobject next; error_->store(env, next); if (next) { env->SetObjectField(s, error_fieldID, next); env->DeleteLocalRef(next); } }
}

void speechRecognitionResultError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SpeechRecognitionResultError");
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_class_end();
  }
}

void speechRecognitionResultError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SpeechRecognitionResultError").c_str());
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sponsoredChat::Class;
jfieldID sponsoredChat::unique_id_fieldID;
jfieldID sponsoredChat::chat_id_fieldID;
jfieldID sponsoredChat::sponsor_info_fieldID;
jfieldID sponsoredChat::additional_info_fieldID;

sponsoredChat::sponsoredChat()
  : unique_id_()
  , chat_id_()
  , sponsor_info_()
  , additional_info_()
{}

sponsoredChat::sponsoredChat(int53 unique_id_, int53 chat_id_, string const &sponsor_info_, string const &additional_info_)
  : unique_id_(unique_id_)
  , chat_id_(chat_id_)
  , sponsor_info_(sponsor_info_)
  , additional_info_(additional_info_)
{}

const std::int32_t sponsoredChat::ID;

object_ptr<sponsoredChat> sponsoredChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sponsoredChat> res = make_object<sponsoredChat>();
  res->unique_id_ = env->GetLongField(p, res->unique_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->sponsor_info_ = jni::fetch_string(env, p, res->sponsor_info_fieldID);
  res->additional_info_ = jni::fetch_string(env, p, res->additional_info_fieldID);
  return res;
}

void sponsoredChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, unique_id_fieldID, unique_id_);
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { jstring nextString = jni::to_jstring(env, sponsor_info_); if (nextString) { env->SetObjectField(s, sponsor_info_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, additional_info_); if (nextString) { env->SetObjectField(s, additional_info_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void sponsoredChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SponsoredChat");
    s.store_field("uniqueId", unique_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("sponsorInfo", sponsor_info_);
    s.store_field("additionalInfo", additional_info_);
    s.store_class_end();
  }
}

void sponsoredChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SponsoredChat").c_str());
    unique_id_fieldID = jni::get_field_id(env, Class, "uniqueId", "J");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    sponsor_info_fieldID = jni::get_field_id(env, Class, "sponsorInfo", "Ljava/lang/String;");
    additional_info_fieldID = jni::get_field_id(env, Class, "additionalInfo", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sponsoredChats::Class;
jfieldID sponsoredChats::chats_fieldID;

sponsoredChats::sponsoredChats()
  : chats_()
{}

sponsoredChats::sponsoredChats(array<object_ptr<sponsoredChat>> &&chats_)
  : chats_(std::move(chats_))
{}

const std::int32_t sponsoredChats::ID;

object_ptr<sponsoredChats> sponsoredChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sponsoredChats> res = make_object<sponsoredChats>();
  res->chats_ = jni::FetchVector<sponsoredChat>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chats_fieldID));
  return res;
}

void sponsoredChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chats_); if (arr_tmp_) { env->SetObjectField(s, chats_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void sponsoredChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SponsoredChats");
    { s.store_vector_begin("chats", chats_.size()); for (const auto &_value : chats_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void sponsoredChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SponsoredChats").c_str());
    chats_fieldID = jni::get_field_id(env, Class, "chats", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SponsoredChat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sponsoredMessage::Class;
jfieldID sponsoredMessage::message_id_fieldID;
jfieldID sponsoredMessage::is_recommended_fieldID;
jfieldID sponsoredMessage::can_be_reported_fieldID;
jfieldID sponsoredMessage::content_fieldID;
jfieldID sponsoredMessage::sponsor_fieldID;
jfieldID sponsoredMessage::title_fieldID;
jfieldID sponsoredMessage::button_text_fieldID;
jfieldID sponsoredMessage::accent_color_id_fieldID;
jfieldID sponsoredMessage::background_custom_emoji_id_fieldID;
jfieldID sponsoredMessage::additional_info_fieldID;

sponsoredMessage::sponsoredMessage()
  : message_id_()
  , is_recommended_()
  , can_be_reported_()
  , content_()
  , sponsor_()
  , title_()
  , button_text_()
  , accent_color_id_()
  , background_custom_emoji_id_()
  , additional_info_()
{}

sponsoredMessage::sponsoredMessage(int53 message_id_, bool is_recommended_, bool can_be_reported_, object_ptr<MessageContent> &&content_, object_ptr<messageSponsor> &&sponsor_, string const &title_, string const &button_text_, int32 accent_color_id_, int64 background_custom_emoji_id_, string const &additional_info_)
  : message_id_(message_id_)
  , is_recommended_(is_recommended_)
  , can_be_reported_(can_be_reported_)
  , content_(std::move(content_))
  , sponsor_(std::move(sponsor_))
  , title_(title_)
  , button_text_(button_text_)
  , accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
  , additional_info_(additional_info_)
{}

const std::int32_t sponsoredMessage::ID;

object_ptr<sponsoredMessage> sponsoredMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sponsoredMessage> res = make_object<sponsoredMessage>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_recommended_ = (env->GetBooleanField(p, res->is_recommended_fieldID) != 0);
  res->can_be_reported_ = (env->GetBooleanField(p, res->can_be_reported_fieldID) != 0);
  res->content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->sponsor_ = jni::fetch_tl_object<messageSponsor>(env, jni::fetch_object(env, p, res->sponsor_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->button_text_ = jni::fetch_string(env, p, res->button_text_fieldID);
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  res->additional_info_ = jni::fetch_string(env, p, res->additional_info_fieldID);
  return res;
}

void sponsoredMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetBooleanField(s, is_recommended_fieldID, is_recommended_);
  env->SetBooleanField(s, can_be_reported_fieldID, can_be_reported_);
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
  if (sponsor_ != nullptr) { jobject next; sponsor_->store(env, next); if (next) { env->SetObjectField(s, sponsor_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, button_text_); if (nextString) { env->SetObjectField(s, button_text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  env->SetLongField(s, background_custom_emoji_id_fieldID, background_custom_emoji_id_);
  { jstring nextString = jni::to_jstring(env, additional_info_); if (nextString) { env->SetObjectField(s, additional_info_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void sponsoredMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SponsoredMessage");
    s.store_field("messageId", message_id_);
    s.store_field("isRecommended", is_recommended_);
    s.store_field("canBeReported", can_be_reported_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("sponsor", static_cast<const BaseObject *>(sponsor_.get()));
    s.store_field("title", title_);
    s.store_field("buttonText", button_text_);
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_field("additionalInfo", additional_info_);
    s.store_class_end();
  }
}

void sponsoredMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SponsoredMessage").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_recommended_fieldID = jni::get_field_id(env, Class, "isRecommended", "Z");
    can_be_reported_fieldID = jni::get_field_id(env, Class, "canBeReported", "Z");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    sponsor_fieldID = jni::get_field_id(env, Class, "sponsor", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSponsor;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    button_text_fieldID = jni::get_field_id(env, Class, "buttonText", "Ljava/lang/String;");
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    additional_info_fieldID = jni::get_field_id(env, Class, "additionalInfo", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sponsoredMessages::Class;
jfieldID sponsoredMessages::messages_fieldID;
jfieldID sponsoredMessages::messages_between_fieldID;

sponsoredMessages::sponsoredMessages()
  : messages_()
  , messages_between_()
{}

sponsoredMessages::sponsoredMessages(array<object_ptr<sponsoredMessage>> &&messages_, int32 messages_between_)
  : messages_(std::move(messages_))
  , messages_between_(messages_between_)
{}

const std::int32_t sponsoredMessages::ID;

object_ptr<sponsoredMessages> sponsoredMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sponsoredMessages> res = make_object<sponsoredMessages>();
  res->messages_ = jni::FetchVector<sponsoredMessage>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  res->messages_between_ = env->GetIntField(p, res->messages_between_fieldID);
  return res;
}

void sponsoredMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, messages_between_fieldID, messages_between_);
}

void sponsoredMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SponsoredMessages");
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("messagesBetween", messages_between_);
    s.store_class_end();
  }
}

void sponsoredMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SponsoredMessages").c_str());
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SponsoredMessage;").c_str());
    messages_between_fieldID = jni::get_field_id(env, Class, "messagesBetween", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starAmount::Class;
jfieldID starAmount::star_count_fieldID;
jfieldID starAmount::nanostar_count_fieldID;

starAmount::starAmount()
  : star_count_()
  , nanostar_count_()
{}

starAmount::starAmount(int53 star_count_, int32 nanostar_count_)
  : star_count_(star_count_)
  , nanostar_count_(nanostar_count_)
{}

const std::int32_t starAmount::ID;

object_ptr<starAmount> starAmount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starAmount> res = make_object<starAmount>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->nanostar_count_ = env->GetIntField(p, res->nanostar_count_fieldID);
  return res;
}

void starAmount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
  env->SetIntField(s, nanostar_count_fieldID, nanostar_count_);
}

void starAmount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarAmount");
    s.store_field("starCount", star_count_);
    s.store_field("nanostarCount", nanostar_count_);
    s.store_class_end();
  }
}

void starAmount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarAmount").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    nanostar_count_fieldID = jni::get_field_id(env, Class, "nanostarCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starCount::Class;
jfieldID starCount::star_count_fieldID;

starCount::starCount()
  : star_count_()
{}

starCount::starCount(int53 star_count_)
  : star_count_(star_count_)
{}

const std::int32_t starCount::ID;

object_ptr<starCount> starCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starCount> res = make_object<starCount>();
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void starCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void starCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarCount");
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void starCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarCount").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starGiveawayPaymentOption::Class;
jfieldID starGiveawayPaymentOption::currency_fieldID;
jfieldID starGiveawayPaymentOption::amount_fieldID;
jfieldID starGiveawayPaymentOption::star_count_fieldID;
jfieldID starGiveawayPaymentOption::store_product_id_fieldID;
jfieldID starGiveawayPaymentOption::yearly_boost_count_fieldID;
jfieldID starGiveawayPaymentOption::winner_options_fieldID;
jfieldID starGiveawayPaymentOption::is_default_fieldID;
jfieldID starGiveawayPaymentOption::is_additional_fieldID;

starGiveawayPaymentOption::starGiveawayPaymentOption()
  : currency_()
  , amount_()
  , star_count_()
  , store_product_id_()
  , yearly_boost_count_()
  , winner_options_()
  , is_default_()
  , is_additional_()
{}

starGiveawayPaymentOption::starGiveawayPaymentOption(string const &currency_, int53 amount_, int53 star_count_, string const &store_product_id_, int32 yearly_boost_count_, array<object_ptr<starGiveawayWinnerOption>> &&winner_options_, bool is_default_, bool is_additional_)
  : currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
  , store_product_id_(store_product_id_)
  , yearly_boost_count_(yearly_boost_count_)
  , winner_options_(std::move(winner_options_))
  , is_default_(is_default_)
  , is_additional_(is_additional_)
{}

const std::int32_t starGiveawayPaymentOption::ID;

object_ptr<starGiveawayPaymentOption> starGiveawayPaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starGiveawayPaymentOption> res = make_object<starGiveawayPaymentOption>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->yearly_boost_count_ = env->GetIntField(p, res->yearly_boost_count_fieldID);
  res->winner_options_ = jni::FetchVector<starGiveawayWinnerOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->winner_options_fieldID));
  res->is_default_ = (env->GetBooleanField(p, res->is_default_fieldID) != 0);
  res->is_additional_ = (env->GetBooleanField(p, res->is_additional_fieldID) != 0);
  return res;
}

void starGiveawayPaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, yearly_boost_count_fieldID, yearly_boost_count_);
  { auto arr_tmp_ = jni::store_vector(env, winner_options_); if (arr_tmp_) { env->SetObjectField(s, winner_options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_default_fieldID, is_default_);
  env->SetBooleanField(s, is_additional_fieldID, is_additional_);
}

void starGiveawayPaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarGiveawayPaymentOption");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_field("storeProductId", store_product_id_);
    s.store_field("yearlyBoostCount", yearly_boost_count_);
    { s.store_vector_begin("winnerOptions", winner_options_.size()); for (const auto &_value : winner_options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isDefault", is_default_);
    s.store_field("isAdditional", is_additional_);
    s.store_class_end();
  }
}

void starGiveawayPaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarGiveawayPaymentOption").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    yearly_boost_count_fieldID = jni::get_field_id(env, Class, "yearlyBoostCount", "I");
    winner_options_fieldID = jni::get_field_id(env, Class, "winnerOptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StarGiveawayWinnerOption;").c_str());
    is_default_fieldID = jni::get_field_id(env, Class, "isDefault", "Z");
    is_additional_fieldID = jni::get_field_id(env, Class, "isAdditional", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass starGiveawayPaymentOptions::Class;
jfieldID starGiveawayPaymentOptions::options_fieldID;

starGiveawayPaymentOptions::starGiveawayPaymentOptions()
  : options_()
{}

starGiveawayPaymentOptions::starGiveawayPaymentOptions(array<object_ptr<starGiveawayPaymentOption>> &&options_)
  : options_(std::move(options_))
{}

const std::int32_t starGiveawayPaymentOptions::ID;

object_ptr<starGiveawayPaymentOptions> starGiveawayPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starGiveawayPaymentOptions> res = make_object<starGiveawayPaymentOptions>();
  res->options_ = jni::FetchVector<starGiveawayPaymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return res;
}

void starGiveawayPaymentOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void starGiveawayPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarGiveawayPaymentOptions");
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void starGiveawayPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarGiveawayPaymentOptions").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StarGiveawayPaymentOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starGiveawayWinnerOption::Class;
jfieldID starGiveawayWinnerOption::winner_count_fieldID;
jfieldID starGiveawayWinnerOption::won_star_count_fieldID;
jfieldID starGiveawayWinnerOption::is_default_fieldID;

starGiveawayWinnerOption::starGiveawayWinnerOption()
  : winner_count_()
  , won_star_count_()
  , is_default_()
{}

starGiveawayWinnerOption::starGiveawayWinnerOption(int32 winner_count_, int53 won_star_count_, bool is_default_)
  : winner_count_(winner_count_)
  , won_star_count_(won_star_count_)
  , is_default_(is_default_)
{}

const std::int32_t starGiveawayWinnerOption::ID;

object_ptr<starGiveawayWinnerOption> starGiveawayWinnerOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starGiveawayWinnerOption> res = make_object<starGiveawayWinnerOption>();
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->won_star_count_ = env->GetLongField(p, res->won_star_count_fieldID);
  res->is_default_ = (env->GetBooleanField(p, res->is_default_fieldID) != 0);
  return res;
}

void starGiveawayWinnerOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, winner_count_fieldID, winner_count_);
  env->SetLongField(s, won_star_count_fieldID, won_star_count_);
  env->SetBooleanField(s, is_default_fieldID, is_default_);
}

void starGiveawayWinnerOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarGiveawayWinnerOption");
    s.store_field("winnerCount", winner_count_);
    s.store_field("wonStarCount", won_star_count_);
    s.store_field("isDefault", is_default_);
    s.store_class_end();
  }
}

void starGiveawayWinnerOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarGiveawayWinnerOption").c_str());
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    won_star_count_fieldID = jni::get_field_id(env, Class, "wonStarCount", "J");
    is_default_fieldID = jni::get_field_id(env, Class, "isDefault", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass starPaymentOption::Class;
jfieldID starPaymentOption::currency_fieldID;
jfieldID starPaymentOption::amount_fieldID;
jfieldID starPaymentOption::star_count_fieldID;
jfieldID starPaymentOption::store_product_id_fieldID;
jfieldID starPaymentOption::is_additional_fieldID;

starPaymentOption::starPaymentOption()
  : currency_()
  , amount_()
  , star_count_()
  , store_product_id_()
  , is_additional_()
{}

starPaymentOption::starPaymentOption(string const &currency_, int53 amount_, int53 star_count_, string const &store_product_id_, bool is_additional_)
  : currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
  , store_product_id_(store_product_id_)
  , is_additional_(is_additional_)
{}

const std::int32_t starPaymentOption::ID;

object_ptr<starPaymentOption> starPaymentOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starPaymentOption> res = make_object<starPaymentOption>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->is_additional_ = (env->GetBooleanField(p, res->is_additional_fieldID) != 0);
  return res;
}

void starPaymentOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, amount_fieldID, amount_);
  env->SetLongField(s, star_count_fieldID, star_count_);
  { jstring nextString = jni::to_jstring(env, store_product_id_); if (nextString) { env->SetObjectField(s, store_product_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_additional_fieldID, is_additional_);
}

void starPaymentOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarPaymentOption");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_field("storeProductId", store_product_id_);
    s.store_field("isAdditional", is_additional_);
    s.store_class_end();
  }
}

void starPaymentOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarPaymentOption").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    is_additional_fieldID = jni::get_field_id(env, Class, "isAdditional", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass starPaymentOptions::Class;
jfieldID starPaymentOptions::options_fieldID;

starPaymentOptions::starPaymentOptions()
  : options_()
{}

starPaymentOptions::starPaymentOptions(array<object_ptr<starPaymentOption>> &&options_)
  : options_(std::move(options_))
{}

const std::int32_t starPaymentOptions::ID;

object_ptr<starPaymentOptions> starPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starPaymentOptions> res = make_object<starPaymentOptions>();
  res->options_ = jni::FetchVector<starPaymentOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->options_fieldID));
  return res;
}

void starPaymentOptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, options_); if (arr_tmp_) { env->SetObjectField(s, options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void starPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarPaymentOptions");
    { s.store_vector_begin("options", options_.size()); for (const auto &_value : options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void starPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarPaymentOptions").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StarPaymentOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starRevenueStatistics::Class;
jfieldID starRevenueStatistics::revenue_by_day_graph_fieldID;
jfieldID starRevenueStatistics::status_fieldID;
jfieldID starRevenueStatistics::usd_rate_fieldID;

starRevenueStatistics::starRevenueStatistics()
  : revenue_by_day_graph_()
  , status_()
  , usd_rate_()
{}

starRevenueStatistics::starRevenueStatistics(object_ptr<StatisticalGraph> &&revenue_by_day_graph_, object_ptr<starRevenueStatus> &&status_, double usd_rate_)
  : revenue_by_day_graph_(std::move(revenue_by_day_graph_))
  , status_(std::move(status_))
  , usd_rate_(usd_rate_)
{}

const std::int32_t starRevenueStatistics::ID;

object_ptr<starRevenueStatistics> starRevenueStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starRevenueStatistics> res = make_object<starRevenueStatistics>();
  res->revenue_by_day_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->revenue_by_day_graph_fieldID));
  res->status_ = jni::fetch_tl_object<starRevenueStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  res->usd_rate_ = env->GetDoubleField(p, res->usd_rate_fieldID);
  return res;
}

void starRevenueStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (revenue_by_day_graph_ != nullptr) { jobject next; revenue_by_day_graph_->store(env, next); if (next) { env->SetObjectField(s, revenue_by_day_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetDoubleField(s, usd_rate_fieldID, usd_rate_);
}

void starRevenueStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarRevenueStatistics");
    s.store_object_field("revenueByDayGraph", static_cast<const BaseObject *>(revenue_by_day_graph_.get()));
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_field("usdRate", usd_rate_);
    s.store_class_end();
  }
}

void starRevenueStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarRevenueStatistics").c_str());
    revenue_by_day_graph_fieldID = jni::get_field_id(env, Class, "revenueByDayGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarRevenueStatus;").c_str());
    usd_rate_fieldID = jni::get_field_id(env, Class, "usdRate", "D");
    return true;
  }();
  (void)is_inited;
}

jclass starRevenueStatus::Class;
jfieldID starRevenueStatus::total_amount_fieldID;
jfieldID starRevenueStatus::current_amount_fieldID;
jfieldID starRevenueStatus::available_amount_fieldID;
jfieldID starRevenueStatus::withdrawal_enabled_fieldID;
jfieldID starRevenueStatus::next_withdrawal_in_fieldID;

starRevenueStatus::starRevenueStatus()
  : total_amount_()
  , current_amount_()
  , available_amount_()
  , withdrawal_enabled_()
  , next_withdrawal_in_()
{}

starRevenueStatus::starRevenueStatus(object_ptr<starAmount> &&total_amount_, object_ptr<starAmount> &&current_amount_, object_ptr<starAmount> &&available_amount_, bool withdrawal_enabled_, int32 next_withdrawal_in_)
  : total_amount_(std::move(total_amount_))
  , current_amount_(std::move(current_amount_))
  , available_amount_(std::move(available_amount_))
  , withdrawal_enabled_(withdrawal_enabled_)
  , next_withdrawal_in_(next_withdrawal_in_)
{}

const std::int32_t starRevenueStatus::ID;

object_ptr<starRevenueStatus> starRevenueStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starRevenueStatus> res = make_object<starRevenueStatus>();
  res->total_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->total_amount_fieldID));
  res->current_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->current_amount_fieldID));
  res->available_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->available_amount_fieldID));
  res->withdrawal_enabled_ = (env->GetBooleanField(p, res->withdrawal_enabled_fieldID) != 0);
  res->next_withdrawal_in_ = env->GetIntField(p, res->next_withdrawal_in_fieldID);
  return res;
}

void starRevenueStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (total_amount_ != nullptr) { jobject next; total_amount_->store(env, next); if (next) { env->SetObjectField(s, total_amount_fieldID, next); env->DeleteLocalRef(next); } }
  if (current_amount_ != nullptr) { jobject next; current_amount_->store(env, next); if (next) { env->SetObjectField(s, current_amount_fieldID, next); env->DeleteLocalRef(next); } }
  if (available_amount_ != nullptr) { jobject next; available_amount_->store(env, next); if (next) { env->SetObjectField(s, available_amount_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, withdrawal_enabled_fieldID, withdrawal_enabled_);
  env->SetIntField(s, next_withdrawal_in_fieldID, next_withdrawal_in_);
}

void starRevenueStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarRevenueStatus");
    s.store_object_field("totalAmount", static_cast<const BaseObject *>(total_amount_.get()));
    s.store_object_field("currentAmount", static_cast<const BaseObject *>(current_amount_.get()));
    s.store_object_field("availableAmount", static_cast<const BaseObject *>(available_amount_.get()));
    s.store_field("withdrawalEnabled", withdrawal_enabled_);
    s.store_field("nextWithdrawalIn", next_withdrawal_in_);
    s.store_class_end();
  }
}

void starRevenueStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarRevenueStatus").c_str());
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    current_amount_fieldID = jni::get_field_id(env, Class, "currentAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    available_amount_fieldID = jni::get_field_id(env, Class, "availableAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    withdrawal_enabled_fieldID = jni::get_field_id(env, Class, "withdrawalEnabled", "Z");
    next_withdrawal_in_fieldID = jni::get_field_id(env, Class, "nextWithdrawalIn", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starSubscription::Class;
jfieldID starSubscription::id_fieldID;
jfieldID starSubscription::chat_id_fieldID;
jfieldID starSubscription::expiration_date_fieldID;
jfieldID starSubscription::is_canceled_fieldID;
jfieldID starSubscription::is_expiring_fieldID;
jfieldID starSubscription::pricing_fieldID;
jfieldID starSubscription::type_fieldID;

starSubscription::starSubscription()
  : id_()
  , chat_id_()
  , expiration_date_()
  , is_canceled_()
  , is_expiring_()
  , pricing_()
  , type_()
{}

starSubscription::starSubscription(string const &id_, int53 chat_id_, int32 expiration_date_, bool is_canceled_, bool is_expiring_, object_ptr<starSubscriptionPricing> &&pricing_, object_ptr<StarSubscriptionType> &&type_)
  : id_(id_)
  , chat_id_(chat_id_)
  , expiration_date_(expiration_date_)
  , is_canceled_(is_canceled_)
  , is_expiring_(is_expiring_)
  , pricing_(std::move(pricing_))
  , type_(std::move(type_))
{}

const std::int32_t starSubscription::ID;

object_ptr<starSubscription> starSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starSubscription> res = make_object<starSubscription>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  res->is_canceled_ = (env->GetBooleanField(p, res->is_canceled_fieldID) != 0);
  res->is_expiring_ = (env->GetBooleanField(p, res->is_expiring_fieldID) != 0);
  res->pricing_ = jni::fetch_tl_object<starSubscriptionPricing>(env, jni::fetch_object(env, p, res->pricing_fieldID));
  res->type_ = jni::fetch_tl_object<StarSubscriptionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void starSubscription::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, expiration_date_fieldID, expiration_date_);
  env->SetBooleanField(s, is_canceled_fieldID, is_canceled_);
  env->SetBooleanField(s, is_expiring_fieldID, is_expiring_);
  if (pricing_ != nullptr) { jobject next; pricing_->store(env, next); if (next) { env->SetObjectField(s, pricing_fieldID, next); env->DeleteLocalRef(next); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void starSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarSubscription");
    s.store_field("id", id_);
    s.store_field("chatId", chat_id_);
    s.store_field("expirationDate", expiration_date_);
    s.store_field("isCanceled", is_canceled_);
    s.store_field("isExpiring", is_expiring_);
    s.store_object_field("pricing", static_cast<const BaseObject *>(pricing_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void starSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscription").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    is_canceled_fieldID = jni::get_field_id(env, Class, "isCanceled", "Z");
    is_expiring_fieldID = jni::get_field_id(env, Class, "isExpiring", "Z");
    pricing_fieldID = jni::get_field_id(env, Class, "pricing", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionPricing;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starSubscriptionPricing::Class;
jfieldID starSubscriptionPricing::period_fieldID;
jfieldID starSubscriptionPricing::star_count_fieldID;

starSubscriptionPricing::starSubscriptionPricing()
  : period_()
  , star_count_()
{}

starSubscriptionPricing::starSubscriptionPricing(int32 period_, int53 star_count_)
  : period_(period_)
  , star_count_(star_count_)
{}

const std::int32_t starSubscriptionPricing::ID;

object_ptr<starSubscriptionPricing> starSubscriptionPricing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starSubscriptionPricing> res = make_object<starSubscriptionPricing>();
  res->period_ = env->GetIntField(p, res->period_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void starSubscriptionPricing::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, period_fieldID, period_);
  env->SetLongField(s, star_count_fieldID, star_count_);
}

void starSubscriptionPricing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarSubscriptionPricing");
    s.store_field("period", period_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void starSubscriptionPricing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscriptionPricing").c_str());
    period_fieldID = jni::get_field_id(env, Class, "period", "I");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass StarSubscriptionType::Class;

object_ptr<StarSubscriptionType> StarSubscriptionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case starSubscriptionTypeChannel::ID:
      return starSubscriptionTypeChannel::fetch(env, p);
    case starSubscriptionTypeBot::ID:
      return starSubscriptionTypeBot::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StarSubscriptionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscriptionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starSubscriptionTypeChannel::Class;
jfieldID starSubscriptionTypeChannel::can_reuse_fieldID;
jfieldID starSubscriptionTypeChannel::invite_link_fieldID;

starSubscriptionTypeChannel::starSubscriptionTypeChannel()
  : can_reuse_()
  , invite_link_()
{}

starSubscriptionTypeChannel::starSubscriptionTypeChannel(bool can_reuse_, string const &invite_link_)
  : can_reuse_(can_reuse_)
  , invite_link_(invite_link_)
{}

const std::int32_t starSubscriptionTypeChannel::ID;

object_ptr<StarSubscriptionType> starSubscriptionTypeChannel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starSubscriptionTypeChannel> res = make_object<starSubscriptionTypeChannel>();
  res->can_reuse_ = (env->GetBooleanField(p, res->can_reuse_fieldID) != 0);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void starSubscriptionTypeChannel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_reuse_fieldID, can_reuse_);
  { jstring nextString = jni::to_jstring(env, invite_link_); if (nextString) { env->SetObjectField(s, invite_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void starSubscriptionTypeChannel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarSubscriptionTypeChannel");
    s.store_field("canReuse", can_reuse_);
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void starSubscriptionTypeChannel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscriptionTypeChannel").c_str());
    can_reuse_fieldID = jni::get_field_id(env, Class, "canReuse", "Z");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass starSubscriptionTypeBot::Class;
jfieldID starSubscriptionTypeBot::is_canceled_by_bot_fieldID;
jfieldID starSubscriptionTypeBot::title_fieldID;
jfieldID starSubscriptionTypeBot::photo_fieldID;
jfieldID starSubscriptionTypeBot::invoice_link_fieldID;

starSubscriptionTypeBot::starSubscriptionTypeBot()
  : is_canceled_by_bot_()
  , title_()
  , photo_()
  , invoice_link_()
{}

starSubscriptionTypeBot::starSubscriptionTypeBot(bool is_canceled_by_bot_, string const &title_, object_ptr<photo> &&photo_, string const &invoice_link_)
  : is_canceled_by_bot_(is_canceled_by_bot_)
  , title_(title_)
  , photo_(std::move(photo_))
  , invoice_link_(invoice_link_)
{}

const std::int32_t starSubscriptionTypeBot::ID;

object_ptr<StarSubscriptionType> starSubscriptionTypeBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starSubscriptionTypeBot> res = make_object<starSubscriptionTypeBot>();
  res->is_canceled_by_bot_ = (env->GetBooleanField(p, res->is_canceled_by_bot_fieldID) != 0);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->invoice_link_ = jni::fetch_string(env, p, res->invoice_link_fieldID);
  return std::move(res);
}

void starSubscriptionTypeBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_canceled_by_bot_fieldID, is_canceled_by_bot_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, invoice_link_); if (nextString) { env->SetObjectField(s, invoice_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void starSubscriptionTypeBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarSubscriptionTypeBot");
    s.store_field("isCanceledByBot", is_canceled_by_bot_);
    s.store_field("title", title_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("invoiceLink", invoice_link_);
    s.store_class_end();
  }
}

void starSubscriptionTypeBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscriptionTypeBot").c_str());
    is_canceled_by_bot_fieldID = jni::get_field_id(env, Class, "isCanceledByBot", "Z");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    invoice_link_fieldID = jni::get_field_id(env, Class, "invoiceLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass starSubscriptions::Class;
jfieldID starSubscriptions::star_amount_fieldID;
jfieldID starSubscriptions::subscriptions_fieldID;
jfieldID starSubscriptions::required_star_count_fieldID;
jfieldID starSubscriptions::next_offset_fieldID;

starSubscriptions::starSubscriptions()
  : star_amount_()
  , subscriptions_()
  , required_star_count_()
  , next_offset_()
{}

starSubscriptions::starSubscriptions(object_ptr<starAmount> &&star_amount_, array<object_ptr<starSubscription>> &&subscriptions_, int53 required_star_count_, string const &next_offset_)
  : star_amount_(std::move(star_amount_))
  , subscriptions_(std::move(subscriptions_))
  , required_star_count_(required_star_count_)
  , next_offset_(next_offset_)
{}

const std::int32_t starSubscriptions::ID;

object_ptr<starSubscriptions> starSubscriptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starSubscriptions> res = make_object<starSubscriptions>();
  res->star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->star_amount_fieldID));
  res->subscriptions_ = jni::FetchVector<starSubscription>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->subscriptions_fieldID));
  res->required_star_count_ = env->GetLongField(p, res->required_star_count_fieldID);
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void starSubscriptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (star_amount_ != nullptr) { jobject next; star_amount_->store(env, next); if (next) { env->SetObjectField(s, star_amount_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, subscriptions_); if (arr_tmp_) { env->SetObjectField(s, subscriptions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetLongField(s, required_star_count_fieldID, required_star_count_);
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void starSubscriptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarSubscriptions");
    s.store_object_field("starAmount", static_cast<const BaseObject *>(star_amount_.get()));
    { s.store_vector_begin("subscriptions", subscriptions_.size()); for (const auto &_value : subscriptions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("requiredStarCount", required_star_count_);
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void starSubscriptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarSubscriptions").c_str());
    star_amount_fieldID = jni::get_field_id(env, Class, "starAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    subscriptions_fieldID = jni::get_field_id(env, Class, "subscriptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StarSubscription;").c_str());
    required_star_count_fieldID = jni::get_field_id(env, Class, "requiredStarCount", "J");
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass starTransaction::Class;
jfieldID starTransaction::id_fieldID;
jfieldID starTransaction::star_amount_fieldID;
jfieldID starTransaction::is_refund_fieldID;
jfieldID starTransaction::date_fieldID;
jfieldID starTransaction::type_fieldID;

starTransaction::starTransaction()
  : id_()
  , star_amount_()
  , is_refund_()
  , date_()
  , type_()
{}

starTransaction::starTransaction(string const &id_, object_ptr<starAmount> &&star_amount_, bool is_refund_, int32 date_, object_ptr<StarTransactionType> &&type_)
  : id_(id_)
  , star_amount_(std::move(star_amount_))
  , is_refund_(is_refund_)
  , date_(date_)
  , type_(std::move(type_))
{}

const std::int32_t starTransaction::ID;

object_ptr<starTransaction> starTransaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransaction> res = make_object<starTransaction>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->star_amount_fieldID));
  res->is_refund_ = (env->GetBooleanField(p, res->is_refund_fieldID) != 0);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->type_ = jni::fetch_tl_object<StarTransactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void starTransaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (star_amount_ != nullptr) { jobject next; star_amount_->store(env, next); if (next) { env->SetObjectField(s, star_amount_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_refund_fieldID, is_refund_);
  env->SetIntField(s, date_fieldID, date_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransaction");
    s.store_field("id", id_);
    s.store_object_field("starAmount", static_cast<const BaseObject *>(star_amount_.get()));
    s.store_field("isRefund", is_refund_);
    s.store_field("date", date_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void starTransaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransaction").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    star_amount_fieldID = jni::get_field_id(env, Class, "starAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    is_refund_fieldID = jni::get_field_id(env, Class, "isRefund", "Z");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarTransactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StarTransactionDirection::Class;

object_ptr<StarTransactionDirection> StarTransactionDirection::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case starTransactionDirectionIncoming::ID:
      return starTransactionDirectionIncoming::fetch(env, p);
    case starTransactionDirectionOutgoing::ID:
      return starTransactionDirectionOutgoing::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StarTransactionDirection::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionDirection").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionDirectionIncoming::Class;

starTransactionDirectionIncoming::starTransactionDirectionIncoming() {
}

const std::int32_t starTransactionDirectionIncoming::ID;

object_ptr<StarTransactionDirection> starTransactionDirectionIncoming::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionDirectionIncoming>();
}

void starTransactionDirectionIncoming::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionDirectionIncoming");
    s.store_class_end();
  }
}

void starTransactionDirectionIncoming::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionDirectionIncoming").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionDirectionOutgoing::Class;

starTransactionDirectionOutgoing::starTransactionDirectionOutgoing() {
}

const std::int32_t starTransactionDirectionOutgoing::ID;

object_ptr<StarTransactionDirection> starTransactionDirectionOutgoing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionDirectionOutgoing>();
}

void starTransactionDirectionOutgoing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionDirectionOutgoing");
    s.store_class_end();
  }
}

void starTransactionDirectionOutgoing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionDirectionOutgoing").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StarTransactionType::Class;

object_ptr<StarTransactionType> StarTransactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case starTransactionTypePremiumBotDeposit::ID:
      return starTransactionTypePremiumBotDeposit::fetch(env, p);
    case starTransactionTypeAppStoreDeposit::ID:
      return starTransactionTypeAppStoreDeposit::fetch(env, p);
    case starTransactionTypeGooglePlayDeposit::ID:
      return starTransactionTypeGooglePlayDeposit::fetch(env, p);
    case starTransactionTypeFragmentDeposit::ID:
      return starTransactionTypeFragmentDeposit::fetch(env, p);
    case starTransactionTypeUserDeposit::ID:
      return starTransactionTypeUserDeposit::fetch(env, p);
    case starTransactionTypeGiveawayDeposit::ID:
      return starTransactionTypeGiveawayDeposit::fetch(env, p);
    case starTransactionTypeFragmentWithdrawal::ID:
      return starTransactionTypeFragmentWithdrawal::fetch(env, p);
    case starTransactionTypeTelegramAdsWithdrawal::ID:
      return starTransactionTypeTelegramAdsWithdrawal::fetch(env, p);
    case starTransactionTypeTelegramApiUsage::ID:
      return starTransactionTypeTelegramApiUsage::fetch(env, p);
    case starTransactionTypeBotPaidMediaPurchase::ID:
      return starTransactionTypeBotPaidMediaPurchase::fetch(env, p);
    case starTransactionTypeBotPaidMediaSale::ID:
      return starTransactionTypeBotPaidMediaSale::fetch(env, p);
    case starTransactionTypeChannelPaidMediaPurchase::ID:
      return starTransactionTypeChannelPaidMediaPurchase::fetch(env, p);
    case starTransactionTypeChannelPaidMediaSale::ID:
      return starTransactionTypeChannelPaidMediaSale::fetch(env, p);
    case starTransactionTypeBotInvoicePurchase::ID:
      return starTransactionTypeBotInvoicePurchase::fetch(env, p);
    case starTransactionTypeBotInvoiceSale::ID:
      return starTransactionTypeBotInvoiceSale::fetch(env, p);
    case starTransactionTypeBotSubscriptionPurchase::ID:
      return starTransactionTypeBotSubscriptionPurchase::fetch(env, p);
    case starTransactionTypeBotSubscriptionSale::ID:
      return starTransactionTypeBotSubscriptionSale::fetch(env, p);
    case starTransactionTypeChannelSubscriptionPurchase::ID:
      return starTransactionTypeChannelSubscriptionPurchase::fetch(env, p);
    case starTransactionTypeChannelSubscriptionSale::ID:
      return starTransactionTypeChannelSubscriptionSale::fetch(env, p);
    case starTransactionTypeGiftPurchase::ID:
      return starTransactionTypeGiftPurchase::fetch(env, p);
    case starTransactionTypeGiftTransfer::ID:
      return starTransactionTypeGiftTransfer::fetch(env, p);
    case starTransactionTypeGiftSale::ID:
      return starTransactionTypeGiftSale::fetch(env, p);
    case starTransactionTypeGiftUpgrade::ID:
      return starTransactionTypeGiftUpgrade::fetch(env, p);
    case starTransactionTypeUpgradedGiftPurchase::ID:
      return starTransactionTypeUpgradedGiftPurchase::fetch(env, p);
    case starTransactionTypeUpgradedGiftSale::ID:
      return starTransactionTypeUpgradedGiftSale::fetch(env, p);
    case starTransactionTypeChannelPaidReactionSend::ID:
      return starTransactionTypeChannelPaidReactionSend::fetch(env, p);
    case starTransactionTypeChannelPaidReactionReceive::ID:
      return starTransactionTypeChannelPaidReactionReceive::fetch(env, p);
    case starTransactionTypeAffiliateProgramCommission::ID:
      return starTransactionTypeAffiliateProgramCommission::fetch(env, p);
    case starTransactionTypePaidMessageSend::ID:
      return starTransactionTypePaidMessageSend::fetch(env, p);
    case starTransactionTypePaidMessageReceive::ID:
      return starTransactionTypePaidMessageReceive::fetch(env, p);
    case starTransactionTypePremiumPurchase::ID:
      return starTransactionTypePremiumPurchase::fetch(env, p);
    case starTransactionTypeBusinessBotTransferSend::ID:
      return starTransactionTypeBusinessBotTransferSend::fetch(env, p);
    case starTransactionTypeBusinessBotTransferReceive::ID:
      return starTransactionTypeBusinessBotTransferReceive::fetch(env, p);
    case starTransactionTypeUnsupported::ID:
      return starTransactionTypeUnsupported::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StarTransactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypePremiumBotDeposit::Class;

starTransactionTypePremiumBotDeposit::starTransactionTypePremiumBotDeposit() {
}

const std::int32_t starTransactionTypePremiumBotDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypePremiumBotDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypePremiumBotDeposit>();
}

void starTransactionTypePremiumBotDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypePremiumBotDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypePremiumBotDeposit");
    s.store_class_end();
  }
}

void starTransactionTypePremiumBotDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypePremiumBotDeposit").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeAppStoreDeposit::Class;

starTransactionTypeAppStoreDeposit::starTransactionTypeAppStoreDeposit() {
}

const std::int32_t starTransactionTypeAppStoreDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypeAppStoreDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypeAppStoreDeposit>();
}

void starTransactionTypeAppStoreDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypeAppStoreDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeAppStoreDeposit");
    s.store_class_end();
  }
}

void starTransactionTypeAppStoreDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeAppStoreDeposit").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGooglePlayDeposit::Class;

starTransactionTypeGooglePlayDeposit::starTransactionTypeGooglePlayDeposit() {
}

const std::int32_t starTransactionTypeGooglePlayDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypeGooglePlayDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypeGooglePlayDeposit>();
}

void starTransactionTypeGooglePlayDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypeGooglePlayDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGooglePlayDeposit");
    s.store_class_end();
  }
}

void starTransactionTypeGooglePlayDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGooglePlayDeposit").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeFragmentDeposit::Class;

starTransactionTypeFragmentDeposit::starTransactionTypeFragmentDeposit() {
}

const std::int32_t starTransactionTypeFragmentDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypeFragmentDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypeFragmentDeposit>();
}

void starTransactionTypeFragmentDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypeFragmentDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeFragmentDeposit");
    s.store_class_end();
  }
}

void starTransactionTypeFragmentDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeFragmentDeposit").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeUserDeposit::Class;
jfieldID starTransactionTypeUserDeposit::user_id_fieldID;
jfieldID starTransactionTypeUserDeposit::sticker_fieldID;

starTransactionTypeUserDeposit::starTransactionTypeUserDeposit()
  : user_id_()
  , sticker_()
{}

starTransactionTypeUserDeposit::starTransactionTypeUserDeposit(int53 user_id_, object_ptr<sticker> &&sticker_)
  : user_id_(user_id_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t starTransactionTypeUserDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypeUserDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeUserDeposit> res = make_object<starTransactionTypeUserDeposit>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void starTransactionTypeUserDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeUserDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeUserDeposit");
    s.store_field("userId", user_id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeUserDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeUserDeposit").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGiveawayDeposit::Class;
jfieldID starTransactionTypeGiveawayDeposit::chat_id_fieldID;
jfieldID starTransactionTypeGiveawayDeposit::giveaway_message_id_fieldID;

starTransactionTypeGiveawayDeposit::starTransactionTypeGiveawayDeposit()
  : chat_id_()
  , giveaway_message_id_()
{}

starTransactionTypeGiveawayDeposit::starTransactionTypeGiveawayDeposit(int53 chat_id_, int53 giveaway_message_id_)
  : chat_id_(chat_id_)
  , giveaway_message_id_(giveaway_message_id_)
{}

const std::int32_t starTransactionTypeGiveawayDeposit::ID;

object_ptr<StarTransactionType> starTransactionTypeGiveawayDeposit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeGiveawayDeposit> res = make_object<starTransactionTypeGiveawayDeposit>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->giveaway_message_id_ = env->GetLongField(p, res->giveaway_message_id_fieldID);
  return std::move(res);
}

void starTransactionTypeGiveawayDeposit::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, giveaway_message_id_fieldID, giveaway_message_id_);
}

void starTransactionTypeGiveawayDeposit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGiveawayDeposit");
    s.store_field("chatId", chat_id_);
    s.store_field("giveawayMessageId", giveaway_message_id_);
    s.store_class_end();
  }
}

void starTransactionTypeGiveawayDeposit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGiveawayDeposit").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    giveaway_message_id_fieldID = jni::get_field_id(env, Class, "giveawayMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeFragmentWithdrawal::Class;
jfieldID starTransactionTypeFragmentWithdrawal::withdrawal_state_fieldID;

starTransactionTypeFragmentWithdrawal::starTransactionTypeFragmentWithdrawal()
  : withdrawal_state_()
{}

starTransactionTypeFragmentWithdrawal::starTransactionTypeFragmentWithdrawal(object_ptr<RevenueWithdrawalState> &&withdrawal_state_)
  : withdrawal_state_(std::move(withdrawal_state_))
{}

const std::int32_t starTransactionTypeFragmentWithdrawal::ID;

object_ptr<StarTransactionType> starTransactionTypeFragmentWithdrawal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeFragmentWithdrawal> res = make_object<starTransactionTypeFragmentWithdrawal>();
  res->withdrawal_state_ = jni::fetch_tl_object<RevenueWithdrawalState>(env, jni::fetch_object(env, p, res->withdrawal_state_fieldID));
  return std::move(res);
}

void starTransactionTypeFragmentWithdrawal::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (withdrawal_state_ != nullptr) { jobject next; withdrawal_state_->store(env, next); if (next) { env->SetObjectField(s, withdrawal_state_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeFragmentWithdrawal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeFragmentWithdrawal");
    s.store_object_field("withdrawalState", static_cast<const BaseObject *>(withdrawal_state_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeFragmentWithdrawal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeFragmentWithdrawal").c_str());
    withdrawal_state_fieldID = jni::get_field_id(env, Class, "withdrawalState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$RevenueWithdrawalState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeTelegramAdsWithdrawal::Class;

starTransactionTypeTelegramAdsWithdrawal::starTransactionTypeTelegramAdsWithdrawal() {
}

const std::int32_t starTransactionTypeTelegramAdsWithdrawal::ID;

object_ptr<StarTransactionType> starTransactionTypeTelegramAdsWithdrawal::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypeTelegramAdsWithdrawal>();
}

void starTransactionTypeTelegramAdsWithdrawal::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypeTelegramAdsWithdrawal::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeTelegramAdsWithdrawal");
    s.store_class_end();
  }
}

void starTransactionTypeTelegramAdsWithdrawal::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeTelegramAdsWithdrawal").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeTelegramApiUsage::Class;
jfieldID starTransactionTypeTelegramApiUsage::request_count_fieldID;

starTransactionTypeTelegramApiUsage::starTransactionTypeTelegramApiUsage()
  : request_count_()
{}

starTransactionTypeTelegramApiUsage::starTransactionTypeTelegramApiUsage(int32 request_count_)
  : request_count_(request_count_)
{}

const std::int32_t starTransactionTypeTelegramApiUsage::ID;

object_ptr<StarTransactionType> starTransactionTypeTelegramApiUsage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeTelegramApiUsage> res = make_object<starTransactionTypeTelegramApiUsage>();
  res->request_count_ = env->GetIntField(p, res->request_count_fieldID);
  return std::move(res);
}

void starTransactionTypeTelegramApiUsage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, request_count_fieldID, request_count_);
}

void starTransactionTypeTelegramApiUsage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeTelegramApiUsage");
    s.store_field("requestCount", request_count_);
    s.store_class_end();
  }
}

void starTransactionTypeTelegramApiUsage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeTelegramApiUsage").c_str());
    request_count_fieldID = jni::get_field_id(env, Class, "requestCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotPaidMediaPurchase::Class;
jfieldID starTransactionTypeBotPaidMediaPurchase::user_id_fieldID;
jfieldID starTransactionTypeBotPaidMediaPurchase::media_fieldID;

starTransactionTypeBotPaidMediaPurchase::starTransactionTypeBotPaidMediaPurchase()
  : user_id_()
  , media_()
{}

starTransactionTypeBotPaidMediaPurchase::starTransactionTypeBotPaidMediaPurchase(int53 user_id_, array<object_ptr<PaidMedia>> &&media_)
  : user_id_(user_id_)
  , media_(std::move(media_))
{}

const std::int32_t starTransactionTypeBotPaidMediaPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeBotPaidMediaPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotPaidMediaPurchase> res = make_object<starTransactionTypeBotPaidMediaPurchase>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->media_ = jni::FetchVector<PaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  return std::move(res);
}

void starTransactionTypeBotPaidMediaPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void starTransactionTypeBotPaidMediaPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotPaidMediaPurchase");
    s.store_field("userId", user_id_);
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void starTransactionTypeBotPaidMediaPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotPaidMediaPurchase").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotPaidMediaSale::Class;
jfieldID starTransactionTypeBotPaidMediaSale::user_id_fieldID;
jfieldID starTransactionTypeBotPaidMediaSale::media_fieldID;
jfieldID starTransactionTypeBotPaidMediaSale::payload_fieldID;
jfieldID starTransactionTypeBotPaidMediaSale::affiliate_fieldID;

starTransactionTypeBotPaidMediaSale::starTransactionTypeBotPaidMediaSale()
  : user_id_()
  , media_()
  , payload_()
  , affiliate_()
{}

starTransactionTypeBotPaidMediaSale::starTransactionTypeBotPaidMediaSale(int53 user_id_, array<object_ptr<PaidMedia>> &&media_, string const &payload_, object_ptr<affiliateInfo> &&affiliate_)
  : user_id_(user_id_)
  , media_(std::move(media_))
  , payload_(payload_)
  , affiliate_(std::move(affiliate_))
{}

const std::int32_t starTransactionTypeBotPaidMediaSale::ID;

object_ptr<StarTransactionType> starTransactionTypeBotPaidMediaSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotPaidMediaSale> res = make_object<starTransactionTypeBotPaidMediaSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->media_ = jni::FetchVector<PaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  res->affiliate_ = jni::fetch_tl_object<affiliateInfo>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  return std::move(res);
}

void starTransactionTypeBotPaidMediaSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, payload_); if (nextString) { env->SetObjectField(s, payload_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (affiliate_ != nullptr) { jobject next; affiliate_->store(env, next); if (next) { env->SetObjectField(s, affiliate_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeBotPaidMediaSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotPaidMediaSale");
    s.store_field("userId", user_id_);
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("payload", payload_);
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeBotPaidMediaSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotPaidMediaSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelPaidMediaPurchase::Class;
jfieldID starTransactionTypeChannelPaidMediaPurchase::chat_id_fieldID;
jfieldID starTransactionTypeChannelPaidMediaPurchase::message_id_fieldID;
jfieldID starTransactionTypeChannelPaidMediaPurchase::media_fieldID;

starTransactionTypeChannelPaidMediaPurchase::starTransactionTypeChannelPaidMediaPurchase()
  : chat_id_()
  , message_id_()
  , media_()
{}

starTransactionTypeChannelPaidMediaPurchase::starTransactionTypeChannelPaidMediaPurchase(int53 chat_id_, int53 message_id_, array<object_ptr<PaidMedia>> &&media_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , media_(std::move(media_))
{}

const std::int32_t starTransactionTypeChannelPaidMediaPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelPaidMediaPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelPaidMediaPurchase> res = make_object<starTransactionTypeChannelPaidMediaPurchase>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->media_ = jni::FetchVector<PaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  return std::move(res);
}

void starTransactionTypeChannelPaidMediaPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void starTransactionTypeChannelPaidMediaPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelPaidMediaPurchase");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void starTransactionTypeChannelPaidMediaPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelPaidMediaPurchase").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelPaidMediaSale::Class;
jfieldID starTransactionTypeChannelPaidMediaSale::user_id_fieldID;
jfieldID starTransactionTypeChannelPaidMediaSale::message_id_fieldID;
jfieldID starTransactionTypeChannelPaidMediaSale::media_fieldID;

starTransactionTypeChannelPaidMediaSale::starTransactionTypeChannelPaidMediaSale()
  : user_id_()
  , message_id_()
  , media_()
{}

starTransactionTypeChannelPaidMediaSale::starTransactionTypeChannelPaidMediaSale(int53 user_id_, int53 message_id_, array<object_ptr<PaidMedia>> &&media_)
  : user_id_(user_id_)
  , message_id_(message_id_)
  , media_(std::move(media_))
{}

const std::int32_t starTransactionTypeChannelPaidMediaSale::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelPaidMediaSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelPaidMediaSale> res = make_object<starTransactionTypeChannelPaidMediaSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->media_ = jni::FetchVector<PaidMedia>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->media_fieldID));
  return std::move(res);
}

void starTransactionTypeChannelPaidMediaSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  { auto arr_tmp_ = jni::store_vector(env, media_); if (arr_tmp_) { env->SetObjectField(s, media_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void starTransactionTypeChannelPaidMediaSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelPaidMediaSale");
    s.store_field("userId", user_id_);
    s.store_field("messageId", message_id_);
    { s.store_vector_begin("media", media_.size()); for (const auto &_value : media_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void starTransactionTypeChannelPaidMediaSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelPaidMediaSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    media_fieldID = jni::get_field_id(env, Class, "media", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PaidMedia;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotInvoicePurchase::Class;
jfieldID starTransactionTypeBotInvoicePurchase::user_id_fieldID;
jfieldID starTransactionTypeBotInvoicePurchase::product_info_fieldID;

starTransactionTypeBotInvoicePurchase::starTransactionTypeBotInvoicePurchase()
  : user_id_()
  , product_info_()
{}

starTransactionTypeBotInvoicePurchase::starTransactionTypeBotInvoicePurchase(int53 user_id_, object_ptr<productInfo> &&product_info_)
  : user_id_(user_id_)
  , product_info_(std::move(product_info_))
{}

const std::int32_t starTransactionTypeBotInvoicePurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeBotInvoicePurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotInvoicePurchase> res = make_object<starTransactionTypeBotInvoicePurchase>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  return std::move(res);
}

void starTransactionTypeBotInvoicePurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeBotInvoicePurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotInvoicePurchase");
    s.store_field("userId", user_id_);
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeBotInvoicePurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotInvoicePurchase").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotInvoiceSale::Class;
jfieldID starTransactionTypeBotInvoiceSale::user_id_fieldID;
jfieldID starTransactionTypeBotInvoiceSale::product_info_fieldID;
jfieldID starTransactionTypeBotInvoiceSale::invoice_payload_fieldID;
jfieldID starTransactionTypeBotInvoiceSale::affiliate_fieldID;

starTransactionTypeBotInvoiceSale::starTransactionTypeBotInvoiceSale()
  : user_id_()
  , product_info_()
  , invoice_payload_()
  , affiliate_()
{}

starTransactionTypeBotInvoiceSale::starTransactionTypeBotInvoiceSale(int53 user_id_, object_ptr<productInfo> &&product_info_, bytes const &invoice_payload_, object_ptr<affiliateInfo> &&affiliate_)
  : user_id_(user_id_)
  , product_info_(std::move(product_info_))
  , invoice_payload_(std::move(invoice_payload_))
  , affiliate_(std::move(affiliate_))
{}

const std::int32_t starTransactionTypeBotInvoiceSale::ID;

object_ptr<StarTransactionType> starTransactionTypeBotInvoiceSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotInvoiceSale> res = make_object<starTransactionTypeBotInvoiceSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  res->invoice_payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->invoice_payload_fieldID));
  res->affiliate_ = jni::fetch_tl_object<affiliateInfo>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  return std::move(res);
}

void starTransactionTypeBotInvoiceSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jbyteArray nextBytes = jni::to_bytes(env, invoice_payload_); if (nextBytes) { env->SetObjectField(s, invoice_payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  if (affiliate_ != nullptr) { jobject next; affiliate_->store(env, next); if (next) { env->SetObjectField(s, affiliate_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeBotInvoiceSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotInvoiceSale");
    s.store_field("userId", user_id_);
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_bytes_field("invoicePayload", invoice_payload_);
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeBotInvoiceSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotInvoiceSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "[B");
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotSubscriptionPurchase::Class;
jfieldID starTransactionTypeBotSubscriptionPurchase::user_id_fieldID;
jfieldID starTransactionTypeBotSubscriptionPurchase::subscription_period_fieldID;
jfieldID starTransactionTypeBotSubscriptionPurchase::product_info_fieldID;

starTransactionTypeBotSubscriptionPurchase::starTransactionTypeBotSubscriptionPurchase()
  : user_id_()
  , subscription_period_()
  , product_info_()
{}

starTransactionTypeBotSubscriptionPurchase::starTransactionTypeBotSubscriptionPurchase(int53 user_id_, int32 subscription_period_, object_ptr<productInfo> &&product_info_)
  : user_id_(user_id_)
  , subscription_period_(subscription_period_)
  , product_info_(std::move(product_info_))
{}

const std::int32_t starTransactionTypeBotSubscriptionPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeBotSubscriptionPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotSubscriptionPurchase> res = make_object<starTransactionTypeBotSubscriptionPurchase>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->subscription_period_ = env->GetIntField(p, res->subscription_period_fieldID);
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  return std::move(res);
}

void starTransactionTypeBotSubscriptionPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, subscription_period_fieldID, subscription_period_);
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeBotSubscriptionPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotSubscriptionPurchase");
    s.store_field("userId", user_id_);
    s.store_field("subscriptionPeriod", subscription_period_);
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeBotSubscriptionPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotSubscriptionPurchase").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    subscription_period_fieldID = jni::get_field_id(env, Class, "subscriptionPeriod", "I");
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBotSubscriptionSale::Class;
jfieldID starTransactionTypeBotSubscriptionSale::user_id_fieldID;
jfieldID starTransactionTypeBotSubscriptionSale::subscription_period_fieldID;
jfieldID starTransactionTypeBotSubscriptionSale::product_info_fieldID;
jfieldID starTransactionTypeBotSubscriptionSale::invoice_payload_fieldID;
jfieldID starTransactionTypeBotSubscriptionSale::affiliate_fieldID;

starTransactionTypeBotSubscriptionSale::starTransactionTypeBotSubscriptionSale()
  : user_id_()
  , subscription_period_()
  , product_info_()
  , invoice_payload_()
  , affiliate_()
{}

starTransactionTypeBotSubscriptionSale::starTransactionTypeBotSubscriptionSale(int53 user_id_, int32 subscription_period_, object_ptr<productInfo> &&product_info_, bytes const &invoice_payload_, object_ptr<affiliateInfo> &&affiliate_)
  : user_id_(user_id_)
  , subscription_period_(subscription_period_)
  , product_info_(std::move(product_info_))
  , invoice_payload_(std::move(invoice_payload_))
  , affiliate_(std::move(affiliate_))
{}

const std::int32_t starTransactionTypeBotSubscriptionSale::ID;

object_ptr<StarTransactionType> starTransactionTypeBotSubscriptionSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBotSubscriptionSale> res = make_object<starTransactionTypeBotSubscriptionSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->subscription_period_ = env->GetIntField(p, res->subscription_period_fieldID);
  res->product_info_ = jni::fetch_tl_object<productInfo>(env, jni::fetch_object(env, p, res->product_info_fieldID));
  res->invoice_payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->invoice_payload_fieldID));
  res->affiliate_ = jni::fetch_tl_object<affiliateInfo>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  return std::move(res);
}

void starTransactionTypeBotSubscriptionSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, subscription_period_fieldID, subscription_period_);
  if (product_info_ != nullptr) { jobject next; product_info_->store(env, next); if (next) { env->SetObjectField(s, product_info_fieldID, next); env->DeleteLocalRef(next); } }
  { jbyteArray nextBytes = jni::to_bytes(env, invoice_payload_); if (nextBytes) { env->SetObjectField(s, invoice_payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  if (affiliate_ != nullptr) { jobject next; affiliate_->store(env, next); if (next) { env->SetObjectField(s, affiliate_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeBotSubscriptionSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBotSubscriptionSale");
    s.store_field("userId", user_id_);
    s.store_field("subscriptionPeriod", subscription_period_);
    s.store_object_field("productInfo", static_cast<const BaseObject *>(product_info_.get()));
    s.store_bytes_field("invoicePayload", invoice_payload_);
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeBotSubscriptionSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBotSubscriptionSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    subscription_period_fieldID = jni::get_field_id(env, Class, "subscriptionPeriod", "I");
    product_info_fieldID = jni::get_field_id(env, Class, "productInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProductInfo;").c_str());
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "[B");
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelSubscriptionPurchase::Class;
jfieldID starTransactionTypeChannelSubscriptionPurchase::chat_id_fieldID;
jfieldID starTransactionTypeChannelSubscriptionPurchase::subscription_period_fieldID;

starTransactionTypeChannelSubscriptionPurchase::starTransactionTypeChannelSubscriptionPurchase()
  : chat_id_()
  , subscription_period_()
{}

starTransactionTypeChannelSubscriptionPurchase::starTransactionTypeChannelSubscriptionPurchase(int53 chat_id_, int32 subscription_period_)
  : chat_id_(chat_id_)
  , subscription_period_(subscription_period_)
{}

const std::int32_t starTransactionTypeChannelSubscriptionPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelSubscriptionPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelSubscriptionPurchase> res = make_object<starTransactionTypeChannelSubscriptionPurchase>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->subscription_period_ = env->GetIntField(p, res->subscription_period_fieldID);
  return std::move(res);
}

void starTransactionTypeChannelSubscriptionPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, subscription_period_fieldID, subscription_period_);
}

void starTransactionTypeChannelSubscriptionPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelSubscriptionPurchase");
    s.store_field("chatId", chat_id_);
    s.store_field("subscriptionPeriod", subscription_period_);
    s.store_class_end();
  }
}

void starTransactionTypeChannelSubscriptionPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelSubscriptionPurchase").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    subscription_period_fieldID = jni::get_field_id(env, Class, "subscriptionPeriod", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelSubscriptionSale::Class;
jfieldID starTransactionTypeChannelSubscriptionSale::user_id_fieldID;
jfieldID starTransactionTypeChannelSubscriptionSale::subscription_period_fieldID;

starTransactionTypeChannelSubscriptionSale::starTransactionTypeChannelSubscriptionSale()
  : user_id_()
  , subscription_period_()
{}

starTransactionTypeChannelSubscriptionSale::starTransactionTypeChannelSubscriptionSale(int53 user_id_, int32 subscription_period_)
  : user_id_(user_id_)
  , subscription_period_(subscription_period_)
{}

const std::int32_t starTransactionTypeChannelSubscriptionSale::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelSubscriptionSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelSubscriptionSale> res = make_object<starTransactionTypeChannelSubscriptionSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->subscription_period_ = env->GetIntField(p, res->subscription_period_fieldID);
  return std::move(res);
}

void starTransactionTypeChannelSubscriptionSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, subscription_period_fieldID, subscription_period_);
}

void starTransactionTypeChannelSubscriptionSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelSubscriptionSale");
    s.store_field("userId", user_id_);
    s.store_field("subscriptionPeriod", subscription_period_);
    s.store_class_end();
  }
}

void starTransactionTypeChannelSubscriptionSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelSubscriptionSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    subscription_period_fieldID = jni::get_field_id(env, Class, "subscriptionPeriod", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGiftPurchase::Class;
jfieldID starTransactionTypeGiftPurchase::owner_id_fieldID;
jfieldID starTransactionTypeGiftPurchase::gift_fieldID;

starTransactionTypeGiftPurchase::starTransactionTypeGiftPurchase()
  : owner_id_()
  , gift_()
{}

starTransactionTypeGiftPurchase::starTransactionTypeGiftPurchase(object_ptr<MessageSender> &&owner_id_, object_ptr<gift> &&gift_)
  : owner_id_(std::move(owner_id_))
  , gift_(std::move(gift_))
{}

const std::int32_t starTransactionTypeGiftPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeGiftPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeGiftPurchase> res = make_object<starTransactionTypeGiftPurchase>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void starTransactionTypeGiftPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (owner_id_ != nullptr) { jobject next; owner_id_->store(env, next); if (next) { env->SetObjectField(s, owner_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeGiftPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGiftPurchase");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeGiftPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGiftPurchase").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGiftTransfer::Class;
jfieldID starTransactionTypeGiftTransfer::owner_id_fieldID;
jfieldID starTransactionTypeGiftTransfer::gift_fieldID;

starTransactionTypeGiftTransfer::starTransactionTypeGiftTransfer()
  : owner_id_()
  , gift_()
{}

starTransactionTypeGiftTransfer::starTransactionTypeGiftTransfer(object_ptr<MessageSender> &&owner_id_, object_ptr<upgradedGift> &&gift_)
  : owner_id_(std::move(owner_id_))
  , gift_(std::move(gift_))
{}

const std::int32_t starTransactionTypeGiftTransfer::ID;

object_ptr<StarTransactionType> starTransactionTypeGiftTransfer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeGiftTransfer> res = make_object<starTransactionTypeGiftTransfer>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void starTransactionTypeGiftTransfer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (owner_id_ != nullptr) { jobject next; owner_id_->store(env, next); if (next) { env->SetObjectField(s, owner_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeGiftTransfer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGiftTransfer");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeGiftTransfer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGiftTransfer").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGiftSale::Class;
jfieldID starTransactionTypeGiftSale::user_id_fieldID;
jfieldID starTransactionTypeGiftSale::gift_fieldID;

starTransactionTypeGiftSale::starTransactionTypeGiftSale()
  : user_id_()
  , gift_()
{}

starTransactionTypeGiftSale::starTransactionTypeGiftSale(int53 user_id_, object_ptr<gift> &&gift_)
  : user_id_(user_id_)
  , gift_(std::move(gift_))
{}

const std::int32_t starTransactionTypeGiftSale::ID;

object_ptr<StarTransactionType> starTransactionTypeGiftSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeGiftSale> res = make_object<starTransactionTypeGiftSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_ = jni::fetch_tl_object<gift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void starTransactionTypeGiftSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeGiftSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGiftSale");
    s.store_field("userId", user_id_);
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeGiftSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGiftSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Gift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeGiftUpgrade::Class;
jfieldID starTransactionTypeGiftUpgrade::user_id_fieldID;
jfieldID starTransactionTypeGiftUpgrade::gift_fieldID;

starTransactionTypeGiftUpgrade::starTransactionTypeGiftUpgrade()
  : user_id_()
  , gift_()
{}

starTransactionTypeGiftUpgrade::starTransactionTypeGiftUpgrade(int53 user_id_, object_ptr<upgradedGift> &&gift_)
  : user_id_(user_id_)
  , gift_(std::move(gift_))
{}

const std::int32_t starTransactionTypeGiftUpgrade::ID;

object_ptr<StarTransactionType> starTransactionTypeGiftUpgrade::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeGiftUpgrade> res = make_object<starTransactionTypeGiftUpgrade>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void starTransactionTypeGiftUpgrade::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeGiftUpgrade::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeGiftUpgrade");
    s.store_field("userId", user_id_);
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeGiftUpgrade::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeGiftUpgrade").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeUpgradedGiftPurchase::Class;
jfieldID starTransactionTypeUpgradedGiftPurchase::user_id_fieldID;
jfieldID starTransactionTypeUpgradedGiftPurchase::gift_fieldID;

starTransactionTypeUpgradedGiftPurchase::starTransactionTypeUpgradedGiftPurchase()
  : user_id_()
  , gift_()
{}

starTransactionTypeUpgradedGiftPurchase::starTransactionTypeUpgradedGiftPurchase(int53 user_id_, object_ptr<upgradedGift> &&gift_)
  : user_id_(user_id_)
  , gift_(std::move(gift_))
{}

const std::int32_t starTransactionTypeUpgradedGiftPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypeUpgradedGiftPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeUpgradedGiftPurchase> res = make_object<starTransactionTypeUpgradedGiftPurchase>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  return std::move(res);
}

void starTransactionTypeUpgradedGiftPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeUpgradedGiftPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeUpgradedGiftPurchase");
    s.store_field("userId", user_id_);
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeUpgradedGiftPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeUpgradedGiftPurchase").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeUpgradedGiftSale::Class;
jfieldID starTransactionTypeUpgradedGiftSale::user_id_fieldID;
jfieldID starTransactionTypeUpgradedGiftSale::gift_fieldID;
jfieldID starTransactionTypeUpgradedGiftSale::affiliate_fieldID;

starTransactionTypeUpgradedGiftSale::starTransactionTypeUpgradedGiftSale()
  : user_id_()
  , gift_()
  , affiliate_()
{}

starTransactionTypeUpgradedGiftSale::starTransactionTypeUpgradedGiftSale(int53 user_id_, object_ptr<upgradedGift> &&gift_, object_ptr<affiliateInfo> &&affiliate_)
  : user_id_(user_id_)
  , gift_(std::move(gift_))
  , affiliate_(std::move(affiliate_))
{}

const std::int32_t starTransactionTypeUpgradedGiftSale::ID;

object_ptr<StarTransactionType> starTransactionTypeUpgradedGiftSale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeUpgradedGiftSale> res = make_object<starTransactionTypeUpgradedGiftSale>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->affiliate_ = jni::fetch_tl_object<affiliateInfo>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  return std::move(res);
}

void starTransactionTypeUpgradedGiftSale::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  if (affiliate_ != nullptr) { jobject next; affiliate_->store(env, next); if (next) { env->SetObjectField(s, affiliate_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypeUpgradedGiftSale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeUpgradedGiftSale");
    s.store_field("userId", user_id_);
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_class_end();
  }
}

void starTransactionTypeUpgradedGiftSale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeUpgradedGiftSale").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelPaidReactionSend::Class;
jfieldID starTransactionTypeChannelPaidReactionSend::chat_id_fieldID;
jfieldID starTransactionTypeChannelPaidReactionSend::message_id_fieldID;

starTransactionTypeChannelPaidReactionSend::starTransactionTypeChannelPaidReactionSend()
  : chat_id_()
  , message_id_()
{}

starTransactionTypeChannelPaidReactionSend::starTransactionTypeChannelPaidReactionSend(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t starTransactionTypeChannelPaidReactionSend::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelPaidReactionSend::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelPaidReactionSend> res = make_object<starTransactionTypeChannelPaidReactionSend>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void starTransactionTypeChannelPaidReactionSend::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void starTransactionTypeChannelPaidReactionSend::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelPaidReactionSend");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void starTransactionTypeChannelPaidReactionSend::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelPaidReactionSend").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeChannelPaidReactionReceive::Class;
jfieldID starTransactionTypeChannelPaidReactionReceive::user_id_fieldID;
jfieldID starTransactionTypeChannelPaidReactionReceive::message_id_fieldID;

starTransactionTypeChannelPaidReactionReceive::starTransactionTypeChannelPaidReactionReceive()
  : user_id_()
  , message_id_()
{}

starTransactionTypeChannelPaidReactionReceive::starTransactionTypeChannelPaidReactionReceive(int53 user_id_, int53 message_id_)
  : user_id_(user_id_)
  , message_id_(message_id_)
{}

const std::int32_t starTransactionTypeChannelPaidReactionReceive::ID;

object_ptr<StarTransactionType> starTransactionTypeChannelPaidReactionReceive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeChannelPaidReactionReceive> res = make_object<starTransactionTypeChannelPaidReactionReceive>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void starTransactionTypeChannelPaidReactionReceive::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void starTransactionTypeChannelPaidReactionReceive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeChannelPaidReactionReceive");
    s.store_field("userId", user_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void starTransactionTypeChannelPaidReactionReceive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeChannelPaidReactionReceive").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeAffiliateProgramCommission::Class;
jfieldID starTransactionTypeAffiliateProgramCommission::chat_id_fieldID;
jfieldID starTransactionTypeAffiliateProgramCommission::commission_per_mille_fieldID;

starTransactionTypeAffiliateProgramCommission::starTransactionTypeAffiliateProgramCommission()
  : chat_id_()
  , commission_per_mille_()
{}

starTransactionTypeAffiliateProgramCommission::starTransactionTypeAffiliateProgramCommission(int53 chat_id_, int32 commission_per_mille_)
  : chat_id_(chat_id_)
  , commission_per_mille_(commission_per_mille_)
{}

const std::int32_t starTransactionTypeAffiliateProgramCommission::ID;

object_ptr<StarTransactionType> starTransactionTypeAffiliateProgramCommission::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeAffiliateProgramCommission> res = make_object<starTransactionTypeAffiliateProgramCommission>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->commission_per_mille_ = env->GetIntField(p, res->commission_per_mille_fieldID);
  return std::move(res);
}

void starTransactionTypeAffiliateProgramCommission::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, commission_per_mille_fieldID, commission_per_mille_);
}

void starTransactionTypeAffiliateProgramCommission::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeAffiliateProgramCommission");
    s.store_field("chatId", chat_id_);
    s.store_field("commissionPerMille", commission_per_mille_);
    s.store_class_end();
  }
}

void starTransactionTypeAffiliateProgramCommission::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeAffiliateProgramCommission").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    commission_per_mille_fieldID = jni::get_field_id(env, Class, "commissionPerMille", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypePaidMessageSend::Class;
jfieldID starTransactionTypePaidMessageSend::chat_id_fieldID;
jfieldID starTransactionTypePaidMessageSend::message_count_fieldID;

starTransactionTypePaidMessageSend::starTransactionTypePaidMessageSend()
  : chat_id_()
  , message_count_()
{}

starTransactionTypePaidMessageSend::starTransactionTypePaidMessageSend(int53 chat_id_, int32 message_count_)
  : chat_id_(chat_id_)
  , message_count_(message_count_)
{}

const std::int32_t starTransactionTypePaidMessageSend::ID;

object_ptr<StarTransactionType> starTransactionTypePaidMessageSend::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypePaidMessageSend> res = make_object<starTransactionTypePaidMessageSend>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_count_ = env->GetIntField(p, res->message_count_fieldID);
  return std::move(res);
}

void starTransactionTypePaidMessageSend::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, message_count_fieldID, message_count_);
}

void starTransactionTypePaidMessageSend::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypePaidMessageSend");
    s.store_field("chatId", chat_id_);
    s.store_field("messageCount", message_count_);
    s.store_class_end();
  }
}

void starTransactionTypePaidMessageSend::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypePaidMessageSend").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_count_fieldID = jni::get_field_id(env, Class, "messageCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypePaidMessageReceive::Class;
jfieldID starTransactionTypePaidMessageReceive::sender_id_fieldID;
jfieldID starTransactionTypePaidMessageReceive::message_count_fieldID;
jfieldID starTransactionTypePaidMessageReceive::commission_per_mille_fieldID;
jfieldID starTransactionTypePaidMessageReceive::commission_star_amount_fieldID;

starTransactionTypePaidMessageReceive::starTransactionTypePaidMessageReceive()
  : sender_id_()
  , message_count_()
  , commission_per_mille_()
  , commission_star_amount_()
{}

starTransactionTypePaidMessageReceive::starTransactionTypePaidMessageReceive(object_ptr<MessageSender> &&sender_id_, int32 message_count_, int32 commission_per_mille_, object_ptr<starAmount> &&commission_star_amount_)
  : sender_id_(std::move(sender_id_))
  , message_count_(message_count_)
  , commission_per_mille_(commission_per_mille_)
  , commission_star_amount_(std::move(commission_star_amount_))
{}

const std::int32_t starTransactionTypePaidMessageReceive::ID;

object_ptr<StarTransactionType> starTransactionTypePaidMessageReceive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypePaidMessageReceive> res = make_object<starTransactionTypePaidMessageReceive>();
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->message_count_ = env->GetIntField(p, res->message_count_fieldID);
  res->commission_per_mille_ = env->GetIntField(p, res->commission_per_mille_fieldID);
  res->commission_star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->commission_star_amount_fieldID));
  return std::move(res);
}

void starTransactionTypePaidMessageReceive::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, message_count_fieldID, message_count_);
  env->SetIntField(s, commission_per_mille_fieldID, commission_per_mille_);
  if (commission_star_amount_ != nullptr) { jobject next; commission_star_amount_->store(env, next); if (next) { env->SetObjectField(s, commission_star_amount_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypePaidMessageReceive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypePaidMessageReceive");
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("messageCount", message_count_);
    s.store_field("commissionPerMille", commission_per_mille_);
    s.store_object_field("commissionStarAmount", static_cast<const BaseObject *>(commission_star_amount_.get()));
    s.store_class_end();
  }
}

void starTransactionTypePaidMessageReceive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypePaidMessageReceive").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    message_count_fieldID = jni::get_field_id(env, Class, "messageCount", "I");
    commission_per_mille_fieldID = jni::get_field_id(env, Class, "commissionPerMille", "I");
    commission_star_amount_fieldID = jni::get_field_id(env, Class, "commissionStarAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypePremiumPurchase::Class;
jfieldID starTransactionTypePremiumPurchase::user_id_fieldID;
jfieldID starTransactionTypePremiumPurchase::month_count_fieldID;
jfieldID starTransactionTypePremiumPurchase::sticker_fieldID;

starTransactionTypePremiumPurchase::starTransactionTypePremiumPurchase()
  : user_id_()
  , month_count_()
  , sticker_()
{}

starTransactionTypePremiumPurchase::starTransactionTypePremiumPurchase(int53 user_id_, int32 month_count_, object_ptr<sticker> &&sticker_)
  : user_id_(user_id_)
  , month_count_(month_count_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t starTransactionTypePremiumPurchase::ID;

object_ptr<StarTransactionType> starTransactionTypePremiumPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypePremiumPurchase> res = make_object<starTransactionTypePremiumPurchase>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void starTransactionTypePremiumPurchase::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  env->SetIntField(s, month_count_fieldID, month_count_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void starTransactionTypePremiumPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypePremiumPurchase");
    s.store_field("userId", user_id_);
    s.store_field("monthCount", month_count_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void starTransactionTypePremiumPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypePremiumPurchase").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBusinessBotTransferSend::Class;
jfieldID starTransactionTypeBusinessBotTransferSend::user_id_fieldID;

starTransactionTypeBusinessBotTransferSend::starTransactionTypeBusinessBotTransferSend()
  : user_id_()
{}

starTransactionTypeBusinessBotTransferSend::starTransactionTypeBusinessBotTransferSend(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t starTransactionTypeBusinessBotTransferSend::ID;

object_ptr<StarTransactionType> starTransactionTypeBusinessBotTransferSend::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBusinessBotTransferSend> res = make_object<starTransactionTypeBusinessBotTransferSend>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void starTransactionTypeBusinessBotTransferSend::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void starTransactionTypeBusinessBotTransferSend::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBusinessBotTransferSend");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void starTransactionTypeBusinessBotTransferSend::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBusinessBotTransferSend").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeBusinessBotTransferReceive::Class;
jfieldID starTransactionTypeBusinessBotTransferReceive::user_id_fieldID;

starTransactionTypeBusinessBotTransferReceive::starTransactionTypeBusinessBotTransferReceive()
  : user_id_()
{}

starTransactionTypeBusinessBotTransferReceive::starTransactionTypeBusinessBotTransferReceive(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t starTransactionTypeBusinessBotTransferReceive::ID;

object_ptr<StarTransactionType> starTransactionTypeBusinessBotTransferReceive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactionTypeBusinessBotTransferReceive> res = make_object<starTransactionTypeBusinessBotTransferReceive>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void starTransactionTypeBusinessBotTransferReceive::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void starTransactionTypeBusinessBotTransferReceive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeBusinessBotTransferReceive");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void starTransactionTypeBusinessBotTransferReceive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeBusinessBotTransferReceive").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass starTransactionTypeUnsupported::Class;

starTransactionTypeUnsupported::starTransactionTypeUnsupported() {
}

const std::int32_t starTransactionTypeUnsupported::ID;

object_ptr<StarTransactionType> starTransactionTypeUnsupported::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<starTransactionTypeUnsupported>();
}

void starTransactionTypeUnsupported::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void starTransactionTypeUnsupported::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactionTypeUnsupported");
    s.store_class_end();
  }
}

void starTransactionTypeUnsupported::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactionTypeUnsupported").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass starTransactions::Class;
jfieldID starTransactions::star_amount_fieldID;
jfieldID starTransactions::transactions_fieldID;
jfieldID starTransactions::next_offset_fieldID;

starTransactions::starTransactions()
  : star_amount_()
  , transactions_()
  , next_offset_()
{}

starTransactions::starTransactions(object_ptr<starAmount> &&star_amount_, array<object_ptr<starTransaction>> &&transactions_, string const &next_offset_)
  : star_amount_(std::move(star_amount_))
  , transactions_(std::move(transactions_))
  , next_offset_(next_offset_)
{}

const std::int32_t starTransactions::ID;

object_ptr<starTransactions> starTransactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<starTransactions> res = make_object<starTransactions>();
  res->star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->star_amount_fieldID));
  res->transactions_ = jni::FetchVector<starTransaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->transactions_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void starTransactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (star_amount_ != nullptr) { jobject next; star_amount_->store(env, next); if (next) { env->SetObjectField(s, star_amount_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, transactions_); if (arr_tmp_) { env->SetObjectField(s, transactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void starTransactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StarTransactions");
    s.store_object_field("starAmount", static_cast<const BaseObject *>(star_amount_.get()));
    { s.store_vector_begin("transactions", transactions_.size()); for (const auto &_value : transactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void starTransactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StarTransactions").c_str());
    star_amount_fieldID = jni::get_field_id(env, Class, "starAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    transactions_fieldID = jni::get_field_id(env, Class, "transactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StarTransaction;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass StatisticalGraph::Class;

object_ptr<StatisticalGraph> StatisticalGraph::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case statisticalGraphData::ID:
      return statisticalGraphData::fetch(env, p);
    case statisticalGraphAsync::ID:
      return statisticalGraphAsync::fetch(env, p);
    case statisticalGraphError::ID:
      return statisticalGraphError::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StatisticalGraph::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StatisticalGraph").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass statisticalGraphData::Class;
jfieldID statisticalGraphData::json_data_fieldID;
jfieldID statisticalGraphData::zoom_token_fieldID;

statisticalGraphData::statisticalGraphData()
  : json_data_()
  , zoom_token_()
{}

statisticalGraphData::statisticalGraphData(string const &json_data_, string const &zoom_token_)
  : json_data_(json_data_)
  , zoom_token_(zoom_token_)
{}

const std::int32_t statisticalGraphData::ID;

object_ptr<StatisticalGraph> statisticalGraphData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<statisticalGraphData> res = make_object<statisticalGraphData>();
  res->json_data_ = jni::fetch_string(env, p, res->json_data_fieldID);
  res->zoom_token_ = jni::fetch_string(env, p, res->zoom_token_fieldID);
  return std::move(res);
}

void statisticalGraphData::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, json_data_); if (nextString) { env->SetObjectField(s, json_data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, zoom_token_); if (nextString) { env->SetObjectField(s, zoom_token_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void statisticalGraphData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StatisticalGraphData");
    s.store_field("jsonData", json_data_);
    s.store_field("zoomToken", zoom_token_);
    s.store_class_end();
  }
}

void statisticalGraphData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StatisticalGraphData").c_str());
    json_data_fieldID = jni::get_field_id(env, Class, "jsonData", "Ljava/lang/String;");
    zoom_token_fieldID = jni::get_field_id(env, Class, "zoomToken", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass statisticalGraphAsync::Class;
jfieldID statisticalGraphAsync::token_fieldID;

statisticalGraphAsync::statisticalGraphAsync()
  : token_()
{}

statisticalGraphAsync::statisticalGraphAsync(string const &token_)
  : token_(token_)
{}

const std::int32_t statisticalGraphAsync::ID;

object_ptr<StatisticalGraph> statisticalGraphAsync::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<statisticalGraphAsync> res = make_object<statisticalGraphAsync>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return std::move(res);
}

void statisticalGraphAsync::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, token_); if (nextString) { env->SetObjectField(s, token_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void statisticalGraphAsync::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StatisticalGraphAsync");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

void statisticalGraphAsync::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StatisticalGraphAsync").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass statisticalGraphError::Class;
jfieldID statisticalGraphError::error_message_fieldID;

statisticalGraphError::statisticalGraphError()
  : error_message_()
{}

statisticalGraphError::statisticalGraphError(string const &error_message_)
  : error_message_(error_message_)
{}

const std::int32_t statisticalGraphError::ID;

object_ptr<StatisticalGraph> statisticalGraphError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<statisticalGraphError> res = make_object<statisticalGraphError>();
  res->error_message_ = jni::fetch_string(env, p, res->error_message_fieldID);
  return std::move(res);
}

void statisticalGraphError::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, error_message_); if (nextString) { env->SetObjectField(s, error_message_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void statisticalGraphError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StatisticalGraphError");
    s.store_field("errorMessage", error_message_);
    s.store_class_end();
  }
}

void statisticalGraphError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StatisticalGraphError").c_str());
    error_message_fieldID = jni::get_field_id(env, Class, "errorMessage", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass statisticalValue::Class;
jfieldID statisticalValue::value_fieldID;
jfieldID statisticalValue::previous_value_fieldID;
jfieldID statisticalValue::growth_rate_percentage_fieldID;

statisticalValue::statisticalValue()
  : value_()
  , previous_value_()
  , growth_rate_percentage_()
{}

statisticalValue::statisticalValue(double value_, double previous_value_, double growth_rate_percentage_)
  : value_(value_)
  , previous_value_(previous_value_)
  , growth_rate_percentage_(growth_rate_percentage_)
{}

const std::int32_t statisticalValue::ID;

object_ptr<statisticalValue> statisticalValue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<statisticalValue> res = make_object<statisticalValue>();
  res->value_ = env->GetDoubleField(p, res->value_fieldID);
  res->previous_value_ = env->GetDoubleField(p, res->previous_value_fieldID);
  res->growth_rate_percentage_ = env->GetDoubleField(p, res->growth_rate_percentage_fieldID);
  return res;
}

void statisticalValue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, value_fieldID, value_);
  env->SetDoubleField(s, previous_value_fieldID, previous_value_);
  env->SetDoubleField(s, growth_rate_percentage_fieldID, growth_rate_percentage_);
}

void statisticalValue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StatisticalValue");
    s.store_field("value", value_);
    s.store_field("previousValue", previous_value_);
    s.store_field("growthRatePercentage", growth_rate_percentage_);
    s.store_class_end();
  }
}

void statisticalValue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StatisticalValue").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "D");
    previous_value_fieldID = jni::get_field_id(env, Class, "previousValue", "D");
    growth_rate_percentage_fieldID = jni::get_field_id(env, Class, "growthRatePercentage", "D");
    return true;
  }();
  (void)is_inited;
}

jclass sticker::Class;
jfieldID sticker::id_fieldID;
jfieldID sticker::set_id_fieldID;
jfieldID sticker::width_fieldID;
jfieldID sticker::height_fieldID;
jfieldID sticker::emoji_fieldID;
jfieldID sticker::format_fieldID;
jfieldID sticker::full_type_fieldID;
jfieldID sticker::thumbnail_fieldID;
jfieldID sticker::sticker_fieldID;

sticker::sticker()
  : id_()
  , set_id_()
  , width_()
  , height_()
  , emoji_()
  , format_()
  , full_type_()
  , thumbnail_()
  , sticker_()
{}

sticker::sticker(int64 id_, int64 set_id_, int32 width_, int32 height_, string const &emoji_, object_ptr<StickerFormat> &&format_, object_ptr<StickerFullType> &&full_type_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&sticker_)
  : id_(id_)
  , set_id_(set_id_)
  , width_(width_)
  , height_(height_)
  , emoji_(emoji_)
  , format_(std::move(format_))
  , full_type_(std::move(full_type_))
  , thumbnail_(std::move(thumbnail_))
  , sticker_(std::move(sticker_))
{}

const std::int32_t sticker::ID;

object_ptr<sticker> sticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sticker> res = make_object<sticker>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->set_id_ = env->GetLongField(p, res->set_id_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->format_ = jni::fetch_tl_object<StickerFormat>(env, jni::fetch_object(env, p, res->format_fieldID));
  res->full_type_ = jni::fetch_tl_object<StickerFullType>(env, jni::fetch_object(env, p, res->full_type_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->sticker_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void sticker::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, set_id_fieldID, set_id_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (format_ != nullptr) { jobject next; format_->store(env, next); if (next) { env->SetObjectField(s, format_fieldID, next); env->DeleteLocalRef(next); } }
  if (full_type_ != nullptr) { jobject next; full_type_->store(env, next); if (next) { env->SetObjectField(s, full_type_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void sticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Sticker");
    s.store_field("id", id_);
    s.store_field("setId", set_id_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("emoji", emoji_);
    s.store_object_field("format", static_cast<const BaseObject *>(format_.get()));
    s.store_object_field("fullType", static_cast<const BaseObject *>(full_type_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void sticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Sticker").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    set_id_fieldID = jni::get_field_id(env, Class, "setId", "J");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    format_fieldID = jni::get_field_id(env, Class, "format", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerFormat;").c_str());
    full_type_fieldID = jni::get_field_id(env, Class, "fullType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerFullType;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StickerFormat::Class;

object_ptr<StickerFormat> StickerFormat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case stickerFormatWebp::ID:
      return stickerFormatWebp::fetch(env, p);
    case stickerFormatTgs::ID:
      return stickerFormatTgs::fetch(env, p);
    case stickerFormatWebm::ID:
      return stickerFormatWebm::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StickerFormat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFormat").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFormatWebp::Class;

stickerFormatWebp::stickerFormatWebp() {
}

const std::int32_t stickerFormatWebp::ID;

object_ptr<StickerFormat> stickerFormatWebp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerFormatWebp>();
}

void stickerFormatWebp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerFormatWebp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFormatWebp");
    s.store_class_end();
  }
}

void stickerFormatWebp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFormatWebp").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFormatTgs::Class;

stickerFormatTgs::stickerFormatTgs() {
}

const std::int32_t stickerFormatTgs::ID;

object_ptr<StickerFormat> stickerFormatTgs::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerFormatTgs>();
}

void stickerFormatTgs::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerFormatTgs::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFormatTgs");
    s.store_class_end();
  }
}

void stickerFormatTgs::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFormatTgs").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFormatWebm::Class;

stickerFormatWebm::stickerFormatWebm() {
}

const std::int32_t stickerFormatWebm::ID;

object_ptr<StickerFormat> stickerFormatWebm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerFormatWebm>();
}

void stickerFormatWebm::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerFormatWebm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFormatWebm");
    s.store_class_end();
  }
}

void stickerFormatWebm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFormatWebm").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StickerFullType::Class;

object_ptr<StickerFullType> StickerFullType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case stickerFullTypeRegular::ID:
      return stickerFullTypeRegular::fetch(env, p);
    case stickerFullTypeMask::ID:
      return stickerFullTypeMask::fetch(env, p);
    case stickerFullTypeCustomEmoji::ID:
      return stickerFullTypeCustomEmoji::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StickerFullType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFullType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFullTypeRegular::Class;
jfieldID stickerFullTypeRegular::premium_animation_fieldID;

stickerFullTypeRegular::stickerFullTypeRegular()
  : premium_animation_()
{}

stickerFullTypeRegular::stickerFullTypeRegular(object_ptr<file> &&premium_animation_)
  : premium_animation_(std::move(premium_animation_))
{}

const std::int32_t stickerFullTypeRegular::ID;

object_ptr<StickerFullType> stickerFullTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerFullTypeRegular> res = make_object<stickerFullTypeRegular>();
  res->premium_animation_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->premium_animation_fieldID));
  return std::move(res);
}

void stickerFullTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (premium_animation_ != nullptr) { jobject next; premium_animation_->store(env, next); if (next) { env->SetObjectField(s, premium_animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void stickerFullTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFullTypeRegular");
    s.store_object_field("premiumAnimation", static_cast<const BaseObject *>(premium_animation_.get()));
    s.store_class_end();
  }
}

void stickerFullTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFullTypeRegular").c_str());
    premium_animation_fieldID = jni::get_field_id(env, Class, "premiumAnimation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFullTypeMask::Class;
jfieldID stickerFullTypeMask::mask_position_fieldID;

stickerFullTypeMask::stickerFullTypeMask()
  : mask_position_()
{}

stickerFullTypeMask::stickerFullTypeMask(object_ptr<maskPosition> &&mask_position_)
  : mask_position_(std::move(mask_position_))
{}

const std::int32_t stickerFullTypeMask::ID;

object_ptr<StickerFullType> stickerFullTypeMask::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerFullTypeMask> res = make_object<stickerFullTypeMask>();
  res->mask_position_ = jni::fetch_tl_object<maskPosition>(env, jni::fetch_object(env, p, res->mask_position_fieldID));
  return std::move(res);
}

void stickerFullTypeMask::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (mask_position_ != nullptr) { jobject next; mask_position_->store(env, next); if (next) { env->SetObjectField(s, mask_position_fieldID, next); env->DeleteLocalRef(next); } }
}

void stickerFullTypeMask::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFullTypeMask");
    s.store_object_field("maskPosition", static_cast<const BaseObject *>(mask_position_.get()));
    s.store_class_end();
  }
}

void stickerFullTypeMask::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFullTypeMask").c_str());
    mask_position_fieldID = jni::get_field_id(env, Class, "maskPosition", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MaskPosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerFullTypeCustomEmoji::Class;
jfieldID stickerFullTypeCustomEmoji::custom_emoji_id_fieldID;
jfieldID stickerFullTypeCustomEmoji::needs_repainting_fieldID;

stickerFullTypeCustomEmoji::stickerFullTypeCustomEmoji()
  : custom_emoji_id_()
  , needs_repainting_()
{}

stickerFullTypeCustomEmoji::stickerFullTypeCustomEmoji(int64 custom_emoji_id_, bool needs_repainting_)
  : custom_emoji_id_(custom_emoji_id_)
  , needs_repainting_(needs_repainting_)
{}

const std::int32_t stickerFullTypeCustomEmoji::ID;

object_ptr<StickerFullType> stickerFullTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerFullTypeCustomEmoji> res = make_object<stickerFullTypeCustomEmoji>();
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  res->needs_repainting_ = (env->GetBooleanField(p, res->needs_repainting_fieldID) != 0);
  return std::move(res);
}

void stickerFullTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
  env->SetBooleanField(s, needs_repainting_fieldID, needs_repainting_);
}

void stickerFullTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerFullTypeCustomEmoji");
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_field("needsRepainting", needs_repainting_);
    s.store_class_end();
  }
}

void stickerFullTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerFullTypeCustomEmoji").c_str());
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    needs_repainting_fieldID = jni::get_field_id(env, Class, "needsRepainting", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass stickerSet::Class;
jfieldID stickerSet::id_fieldID;
jfieldID stickerSet::title_fieldID;
jfieldID stickerSet::name_fieldID;
jfieldID stickerSet::thumbnail_fieldID;
jfieldID stickerSet::thumbnail_outline_fieldID;
jfieldID stickerSet::is_owned_fieldID;
jfieldID stickerSet::is_installed_fieldID;
jfieldID stickerSet::is_archived_fieldID;
jfieldID stickerSet::is_official_fieldID;
jfieldID stickerSet::sticker_type_fieldID;
jfieldID stickerSet::needs_repainting_fieldID;
jfieldID stickerSet::is_allowed_as_chat_emoji_status_fieldID;
jfieldID stickerSet::is_viewed_fieldID;
jfieldID stickerSet::stickers_fieldID;
jfieldID stickerSet::emojis_fieldID;

stickerSet::stickerSet()
  : id_()
  , title_()
  , name_()
  , thumbnail_()
  , thumbnail_outline_()
  , is_owned_()
  , is_installed_()
  , is_archived_()
  , is_official_()
  , sticker_type_()
  , needs_repainting_()
  , is_allowed_as_chat_emoji_status_()
  , is_viewed_()
  , stickers_()
  , emojis_()
{}

stickerSet::stickerSet(int64 id_, string const &title_, string const &name_, object_ptr<thumbnail> &&thumbnail_, object_ptr<outline> &&thumbnail_outline_, bool is_owned_, bool is_installed_, bool is_archived_, bool is_official_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, bool is_allowed_as_chat_emoji_status_, bool is_viewed_, array<object_ptr<sticker>> &&stickers_, array<object_ptr<emojis>> &&emojis_)
  : id_(id_)
  , title_(title_)
  , name_(name_)
  , thumbnail_(std::move(thumbnail_))
  , thumbnail_outline_(std::move(thumbnail_outline_))
  , is_owned_(is_owned_)
  , is_installed_(is_installed_)
  , is_archived_(is_archived_)
  , is_official_(is_official_)
  , sticker_type_(std::move(sticker_type_))
  , needs_repainting_(needs_repainting_)
  , is_allowed_as_chat_emoji_status_(is_allowed_as_chat_emoji_status_)
  , is_viewed_(is_viewed_)
  , stickers_(std::move(stickers_))
  , emojis_(std::move(emojis_))
{}

const std::int32_t stickerSet::ID;

object_ptr<stickerSet> stickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerSet> res = make_object<stickerSet>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->thumbnail_outline_ = jni::fetch_tl_object<outline>(env, jni::fetch_object(env, p, res->thumbnail_outline_fieldID));
  res->is_owned_ = (env->GetBooleanField(p, res->is_owned_fieldID) != 0);
  res->is_installed_ = (env->GetBooleanField(p, res->is_installed_fieldID) != 0);
  res->is_archived_ = (env->GetBooleanField(p, res->is_archived_fieldID) != 0);
  res->is_official_ = (env->GetBooleanField(p, res->is_official_fieldID) != 0);
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->needs_repainting_ = (env->GetBooleanField(p, res->needs_repainting_fieldID) != 0);
  res->is_allowed_as_chat_emoji_status_ = (env->GetBooleanField(p, res->is_allowed_as_chat_emoji_status_fieldID) != 0);
  res->is_viewed_ = (env->GetBooleanField(p, res->is_viewed_fieldID) != 0);
  res->stickers_ = jni::FetchVector<sticker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stickers_fieldID));
  res->emojis_ = jni::FetchVector<emojis>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return res;
}

void stickerSet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_outline_ != nullptr) { jobject next; thumbnail_outline_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_outline_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_owned_fieldID, is_owned_);
  env->SetBooleanField(s, is_installed_fieldID, is_installed_);
  env->SetBooleanField(s, is_archived_fieldID, is_archived_);
  env->SetBooleanField(s, is_official_fieldID, is_official_);
  if (sticker_type_ != nullptr) { jobject next; sticker_type_->store(env, next); if (next) { env->SetObjectField(s, sticker_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, needs_repainting_fieldID, needs_repainting_);
  env->SetBooleanField(s, is_allowed_as_chat_emoji_status_fieldID, is_allowed_as_chat_emoji_status_);
  env->SetBooleanField(s, is_viewed_fieldID, is_viewed_);
  { auto arr_tmp_ = jni::store_vector(env, stickers_); if (arr_tmp_) { env->SetObjectField(s, stickers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void stickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerSet");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("name", name_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("thumbnailOutline", static_cast<const BaseObject *>(thumbnail_outline_.get()));
    s.store_field("isOwned", is_owned_);
    s.store_field("isInstalled", is_installed_);
    s.store_field("isArchived", is_archived_);
    s.store_field("isOfficial", is_official_);
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("needsRepainting", needs_repainting_);
    s.store_field("isAllowedAsChatEmojiStatus", is_allowed_as_chat_emoji_status_);
    s.store_field("isViewed", is_viewed_);
    { s.store_vector_begin("stickers", stickers_.size()); for (const auto &_value : stickers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void stickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerSet").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    thumbnail_outline_fieldID = jni::get_field_id(env, Class, "thumbnailOutline", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Outline;").c_str());
    is_owned_fieldID = jni::get_field_id(env, Class, "isOwned", "Z");
    is_installed_fieldID = jni::get_field_id(env, Class, "isInstalled", "Z");
    is_archived_fieldID = jni::get_field_id(env, Class, "isArchived", "Z");
    is_official_fieldID = jni::get_field_id(env, Class, "isOfficial", "Z");
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    needs_repainting_fieldID = jni::get_field_id(env, Class, "needsRepainting", "Z");
    is_allowed_as_chat_emoji_status_fieldID = jni::get_field_id(env, Class, "isAllowedAsChatEmojiStatus", "Z");
    is_viewed_fieldID = jni::get_field_id(env, Class, "isViewed", "Z");
    stickers_fieldID = jni::get_field_id(env, Class, "stickers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Emojis;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerSetInfo::Class;
jfieldID stickerSetInfo::id_fieldID;
jfieldID stickerSetInfo::title_fieldID;
jfieldID stickerSetInfo::name_fieldID;
jfieldID stickerSetInfo::thumbnail_fieldID;
jfieldID stickerSetInfo::thumbnail_outline_fieldID;
jfieldID stickerSetInfo::is_owned_fieldID;
jfieldID stickerSetInfo::is_installed_fieldID;
jfieldID stickerSetInfo::is_archived_fieldID;
jfieldID stickerSetInfo::is_official_fieldID;
jfieldID stickerSetInfo::sticker_type_fieldID;
jfieldID stickerSetInfo::needs_repainting_fieldID;
jfieldID stickerSetInfo::is_allowed_as_chat_emoji_status_fieldID;
jfieldID stickerSetInfo::is_viewed_fieldID;
jfieldID stickerSetInfo::size_fieldID;
jfieldID stickerSetInfo::covers_fieldID;

stickerSetInfo::stickerSetInfo()
  : id_()
  , title_()
  , name_()
  , thumbnail_()
  , thumbnail_outline_()
  , is_owned_()
  , is_installed_()
  , is_archived_()
  , is_official_()
  , sticker_type_()
  , needs_repainting_()
  , is_allowed_as_chat_emoji_status_()
  , is_viewed_()
  , size_()
  , covers_()
{}

stickerSetInfo::stickerSetInfo(int64 id_, string const &title_, string const &name_, object_ptr<thumbnail> &&thumbnail_, object_ptr<outline> &&thumbnail_outline_, bool is_owned_, bool is_installed_, bool is_archived_, bool is_official_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, bool is_allowed_as_chat_emoji_status_, bool is_viewed_, int32 size_, array<object_ptr<sticker>> &&covers_)
  : id_(id_)
  , title_(title_)
  , name_(name_)
  , thumbnail_(std::move(thumbnail_))
  , thumbnail_outline_(std::move(thumbnail_outline_))
  , is_owned_(is_owned_)
  , is_installed_(is_installed_)
  , is_archived_(is_archived_)
  , is_official_(is_official_)
  , sticker_type_(std::move(sticker_type_))
  , needs_repainting_(needs_repainting_)
  , is_allowed_as_chat_emoji_status_(is_allowed_as_chat_emoji_status_)
  , is_viewed_(is_viewed_)
  , size_(size_)
  , covers_(std::move(covers_))
{}

const std::int32_t stickerSetInfo::ID;

object_ptr<stickerSetInfo> stickerSetInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerSetInfo> res = make_object<stickerSetInfo>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->thumbnail_outline_ = jni::fetch_tl_object<outline>(env, jni::fetch_object(env, p, res->thumbnail_outline_fieldID));
  res->is_owned_ = (env->GetBooleanField(p, res->is_owned_fieldID) != 0);
  res->is_installed_ = (env->GetBooleanField(p, res->is_installed_fieldID) != 0);
  res->is_archived_ = (env->GetBooleanField(p, res->is_archived_fieldID) != 0);
  res->is_official_ = (env->GetBooleanField(p, res->is_official_fieldID) != 0);
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->needs_repainting_ = (env->GetBooleanField(p, res->needs_repainting_fieldID) != 0);
  res->is_allowed_as_chat_emoji_status_ = (env->GetBooleanField(p, res->is_allowed_as_chat_emoji_status_fieldID) != 0);
  res->is_viewed_ = (env->GetBooleanField(p, res->is_viewed_fieldID) != 0);
  res->size_ = env->GetIntField(p, res->size_fieldID);
  res->covers_ = jni::FetchVector<sticker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->covers_fieldID));
  return res;
}

void stickerSetInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_outline_ != nullptr) { jobject next; thumbnail_outline_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_outline_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_owned_fieldID, is_owned_);
  env->SetBooleanField(s, is_installed_fieldID, is_installed_);
  env->SetBooleanField(s, is_archived_fieldID, is_archived_);
  env->SetBooleanField(s, is_official_fieldID, is_official_);
  if (sticker_type_ != nullptr) { jobject next; sticker_type_->store(env, next); if (next) { env->SetObjectField(s, sticker_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, needs_repainting_fieldID, needs_repainting_);
  env->SetBooleanField(s, is_allowed_as_chat_emoji_status_fieldID, is_allowed_as_chat_emoji_status_);
  env->SetBooleanField(s, is_viewed_fieldID, is_viewed_);
  env->SetIntField(s, size_fieldID, size_);
  { auto arr_tmp_ = jni::store_vector(env, covers_); if (arr_tmp_) { env->SetObjectField(s, covers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void stickerSetInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerSetInfo");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("name", name_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("thumbnailOutline", static_cast<const BaseObject *>(thumbnail_outline_.get()));
    s.store_field("isOwned", is_owned_);
    s.store_field("isInstalled", is_installed_);
    s.store_field("isArchived", is_archived_);
    s.store_field("isOfficial", is_official_);
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("needsRepainting", needs_repainting_);
    s.store_field("isAllowedAsChatEmojiStatus", is_allowed_as_chat_emoji_status_);
    s.store_field("isViewed", is_viewed_);
    s.store_field("size", size_);
    { s.store_vector_begin("covers", covers_.size()); for (const auto &_value : covers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void stickerSetInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerSetInfo").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    thumbnail_outline_fieldID = jni::get_field_id(env, Class, "thumbnailOutline", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Outline;").c_str());
    is_owned_fieldID = jni::get_field_id(env, Class, "isOwned", "Z");
    is_installed_fieldID = jni::get_field_id(env, Class, "isInstalled", "Z");
    is_archived_fieldID = jni::get_field_id(env, Class, "isArchived", "Z");
    is_official_fieldID = jni::get_field_id(env, Class, "isOfficial", "Z");
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    needs_repainting_fieldID = jni::get_field_id(env, Class, "needsRepainting", "Z");
    is_allowed_as_chat_emoji_status_fieldID = jni::get_field_id(env, Class, "isAllowedAsChatEmojiStatus", "Z");
    is_viewed_fieldID = jni::get_field_id(env, Class, "isViewed", "Z");
    size_fieldID = jni::get_field_id(env, Class, "size", "I");
    covers_fieldID = jni::get_field_id(env, Class, "covers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerSets::Class;
jfieldID stickerSets::total_count_fieldID;
jfieldID stickerSets::sets_fieldID;

stickerSets::stickerSets()
  : total_count_()
  , sets_()
{}

stickerSets::stickerSets(int32 total_count_, array<object_ptr<stickerSetInfo>> &&sets_)
  : total_count_(total_count_)
  , sets_(std::move(sets_))
{}

const std::int32_t stickerSets::ID;

object_ptr<stickerSets> stickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickerSets> res = make_object<stickerSets>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->sets_ = jni::FetchVector<stickerSetInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->sets_fieldID));
  return res;
}

void stickerSets::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, sets_); if (arr_tmp_) { env->SetObjectField(s, sets_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void stickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerSets");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("sets", sets_.size()); for (const auto &_value : sets_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void stickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerSets").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    sets_fieldID = jni::get_field_id(env, Class, "sets", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StickerSetInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StickerType::Class;

object_ptr<StickerType> StickerType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case stickerTypeRegular::ID:
      return stickerTypeRegular::fetch(env, p);
    case stickerTypeMask::ID:
      return stickerTypeMask::fetch(env, p);
    case stickerTypeCustomEmoji::ID:
      return stickerTypeCustomEmoji::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StickerType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerTypeRegular::Class;

stickerTypeRegular::stickerTypeRegular() {
}

const std::int32_t stickerTypeRegular::ID;

object_ptr<StickerType> stickerTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerTypeRegular>();
}

void stickerTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerTypeRegular");
    s.store_class_end();
  }
}

void stickerTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerTypeRegular").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerTypeMask::Class;

stickerTypeMask::stickerTypeMask() {
}

const std::int32_t stickerTypeMask::ID;

object_ptr<StickerType> stickerTypeMask::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerTypeMask>();
}

void stickerTypeMask::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerTypeMask::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerTypeMask");
    s.store_class_end();
  }
}

void stickerTypeMask::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerTypeMask").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickerTypeCustomEmoji::Class;

stickerTypeCustomEmoji::stickerTypeCustomEmoji() {
}

const std::int32_t stickerTypeCustomEmoji::ID;

object_ptr<StickerType> stickerTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<stickerTypeCustomEmoji>();
}

void stickerTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void stickerTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StickerTypeCustomEmoji");
    s.store_class_end();
  }
}

void stickerTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StickerTypeCustomEmoji").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stickers::Class;
jfieldID stickers::stickers_fieldID;

stickers::stickers()
  : stickers_()
{}

stickers::stickers(array<object_ptr<sticker>> &&stickers_)
  : stickers_(std::move(stickers_))
{}

const std::int32_t stickers::ID;

object_ptr<stickers> stickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stickers> res = make_object<stickers>();
  res->stickers_ = jni::FetchVector<sticker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stickers_fieldID));
  return res;
}

void stickers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, stickers_); if (arr_tmp_) { env->SetObjectField(s, stickers_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void stickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Stickers");
    { s.store_vector_begin("stickers", stickers_.size()); for (const auto &_value : stickers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void stickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Stickers").c_str());
    stickers_fieldID = jni::get_field_id(env, Class, "stickers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storageStatistics::Class;
jfieldID storageStatistics::size_fieldID;
jfieldID storageStatistics::count_fieldID;
jfieldID storageStatistics::by_chat_fieldID;

storageStatistics::storageStatistics()
  : size_()
  , count_()
  , by_chat_()
{}

storageStatistics::storageStatistics(int53 size_, int32 count_, array<object_ptr<storageStatisticsByChat>> &&by_chat_)
  : size_(size_)
  , count_(count_)
  , by_chat_(std::move(by_chat_))
{}

const std::int32_t storageStatistics::ID;

object_ptr<storageStatistics> storageStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storageStatistics> res = make_object<storageStatistics>();
  res->size_ = env->GetLongField(p, res->size_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  res->by_chat_ = jni::FetchVector<storageStatisticsByChat>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->by_chat_fieldID));
  return res;
}

void storageStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, size_fieldID, size_);
  env->SetIntField(s, count_fieldID, count_);
  { auto arr_tmp_ = jni::store_vector(env, by_chat_); if (arr_tmp_) { env->SetObjectField(s, by_chat_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storageStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorageStatistics");
    s.store_field("size", size_);
    s.store_field("count", count_);
    { s.store_vector_begin("byChat", by_chat_.size()); for (const auto &_value : by_chat_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storageStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorageStatistics").c_str());
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    by_chat_fieldID = jni::get_field_id(env, Class, "byChat", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StorageStatisticsByChat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storageStatisticsByChat::Class;
jfieldID storageStatisticsByChat::chat_id_fieldID;
jfieldID storageStatisticsByChat::size_fieldID;
jfieldID storageStatisticsByChat::count_fieldID;
jfieldID storageStatisticsByChat::by_file_type_fieldID;

storageStatisticsByChat::storageStatisticsByChat()
  : chat_id_()
  , size_()
  , count_()
  , by_file_type_()
{}

storageStatisticsByChat::storageStatisticsByChat(int53 chat_id_, int53 size_, int32 count_, array<object_ptr<storageStatisticsByFileType>> &&by_file_type_)
  : chat_id_(chat_id_)
  , size_(size_)
  , count_(count_)
  , by_file_type_(std::move(by_file_type_))
{}

const std::int32_t storageStatisticsByChat::ID;

object_ptr<storageStatisticsByChat> storageStatisticsByChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storageStatisticsByChat> res = make_object<storageStatisticsByChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->size_ = env->GetLongField(p, res->size_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  res->by_file_type_ = jni::FetchVector<storageStatisticsByFileType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->by_file_type_fieldID));
  return res;
}

void storageStatisticsByChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, size_fieldID, size_);
  env->SetIntField(s, count_fieldID, count_);
  { auto arr_tmp_ = jni::store_vector(env, by_file_type_); if (arr_tmp_) { env->SetObjectField(s, by_file_type_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storageStatisticsByChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorageStatisticsByChat");
    s.store_field("chatId", chat_id_);
    s.store_field("size", size_);
    s.store_field("count", count_);
    { s.store_vector_begin("byFileType", by_file_type_.size()); for (const auto &_value : by_file_type_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storageStatisticsByChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorageStatisticsByChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    by_file_type_fieldID = jni::get_field_id(env, Class, "byFileType", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StorageStatisticsByFileType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storageStatisticsByFileType::Class;
jfieldID storageStatisticsByFileType::file_type_fieldID;
jfieldID storageStatisticsByFileType::size_fieldID;
jfieldID storageStatisticsByFileType::count_fieldID;

storageStatisticsByFileType::storageStatisticsByFileType()
  : file_type_()
  , size_()
  , count_()
{}

storageStatisticsByFileType::storageStatisticsByFileType(object_ptr<FileType> &&file_type_, int53 size_, int32 count_)
  : file_type_(std::move(file_type_))
  , size_(size_)
  , count_(count_)
{}

const std::int32_t storageStatisticsByFileType::ID;

object_ptr<storageStatisticsByFileType> storageStatisticsByFileType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storageStatisticsByFileType> res = make_object<storageStatisticsByFileType>();
  res->file_type_ = jni::fetch_tl_object<FileType>(env, jni::fetch_object(env, p, res->file_type_fieldID));
  res->size_ = env->GetLongField(p, res->size_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  return res;
}

void storageStatisticsByFileType::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (file_type_ != nullptr) { jobject next; file_type_->store(env, next); if (next) { env->SetObjectField(s, file_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, size_fieldID, size_);
  env->SetIntField(s, count_fieldID, count_);
}

void storageStatisticsByFileType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorageStatisticsByFileType");
    s.store_object_field("fileType", static_cast<const BaseObject *>(file_type_.get()));
    s.store_field("size", size_);
    s.store_field("count", count_);
    s.store_class_end();
  }
}

void storageStatisticsByFileType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorageStatisticsByFileType").c_str());
    file_type_fieldID = jni::get_field_id(env, Class, "fileType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FileType;").c_str());
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    return true;
  }();
  (void)is_inited;
}

jclass storageStatisticsFast::Class;
jfieldID storageStatisticsFast::files_size_fieldID;
jfieldID storageStatisticsFast::file_count_fieldID;
jfieldID storageStatisticsFast::database_size_fieldID;
jfieldID storageStatisticsFast::language_pack_database_size_fieldID;
jfieldID storageStatisticsFast::log_size_fieldID;

storageStatisticsFast::storageStatisticsFast()
  : files_size_()
  , file_count_()
  , database_size_()
  , language_pack_database_size_()
  , log_size_()
{}

storageStatisticsFast::storageStatisticsFast(int53 files_size_, int32 file_count_, int53 database_size_, int53 language_pack_database_size_, int53 log_size_)
  : files_size_(files_size_)
  , file_count_(file_count_)
  , database_size_(database_size_)
  , language_pack_database_size_(language_pack_database_size_)
  , log_size_(log_size_)
{}

const std::int32_t storageStatisticsFast::ID;

object_ptr<storageStatisticsFast> storageStatisticsFast::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storageStatisticsFast> res = make_object<storageStatisticsFast>();
  res->files_size_ = env->GetLongField(p, res->files_size_fieldID);
  res->file_count_ = env->GetIntField(p, res->file_count_fieldID);
  res->database_size_ = env->GetLongField(p, res->database_size_fieldID);
  res->language_pack_database_size_ = env->GetLongField(p, res->language_pack_database_size_fieldID);
  res->log_size_ = env->GetLongField(p, res->log_size_fieldID);
  return res;
}

void storageStatisticsFast::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, files_size_fieldID, files_size_);
  env->SetIntField(s, file_count_fieldID, file_count_);
  env->SetLongField(s, database_size_fieldID, database_size_);
  env->SetLongField(s, language_pack_database_size_fieldID, language_pack_database_size_);
  env->SetLongField(s, log_size_fieldID, log_size_);
}

void storageStatisticsFast::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorageStatisticsFast");
    s.store_field("filesSize", files_size_);
    s.store_field("fileCount", file_count_);
    s.store_field("databaseSize", database_size_);
    s.store_field("languagePackDatabaseSize", language_pack_database_size_);
    s.store_field("logSize", log_size_);
    s.store_class_end();
  }
}

void storageStatisticsFast::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorageStatisticsFast").c_str());
    files_size_fieldID = jni::get_field_id(env, Class, "filesSize", "J");
    file_count_fieldID = jni::get_field_id(env, Class, "fileCount", "I");
    database_size_fieldID = jni::get_field_id(env, Class, "databaseSize", "J");
    language_pack_database_size_fieldID = jni::get_field_id(env, Class, "languagePackDatabaseSize", "J");
    log_size_fieldID = jni::get_field_id(env, Class, "logSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass StorePaymentPurpose::Class;

object_ptr<StorePaymentPurpose> StorePaymentPurpose::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storePaymentPurposePremiumSubscription::ID:
      return storePaymentPurposePremiumSubscription::fetch(env, p);
    case storePaymentPurposePremiumGift::ID:
      return storePaymentPurposePremiumGift::fetch(env, p);
    case storePaymentPurposePremiumGiftCodes::ID:
      return storePaymentPurposePremiumGiftCodes::fetch(env, p);
    case storePaymentPurposePremiumGiveaway::ID:
      return storePaymentPurposePremiumGiveaway::fetch(env, p);
    case storePaymentPurposeStarGiveaway::ID:
      return storePaymentPurposeStarGiveaway::fetch(env, p);
    case storePaymentPurposeStars::ID:
      return storePaymentPurposeStars::fetch(env, p);
    case storePaymentPurposeGiftedStars::ID:
      return storePaymentPurposeGiftedStars::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StorePaymentPurpose::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurpose").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposePremiumSubscription::Class;
jfieldID storePaymentPurposePremiumSubscription::is_restore_fieldID;
jfieldID storePaymentPurposePremiumSubscription::is_upgrade_fieldID;

storePaymentPurposePremiumSubscription::storePaymentPurposePremiumSubscription()
  : is_restore_()
  , is_upgrade_()
{}

storePaymentPurposePremiumSubscription::storePaymentPurposePremiumSubscription(bool is_restore_, bool is_upgrade_)
  : is_restore_(is_restore_)
  , is_upgrade_(is_upgrade_)
{}

const std::int32_t storePaymentPurposePremiumSubscription::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposePremiumSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposePremiumSubscription> res = make_object<storePaymentPurposePremiumSubscription>();
  res->is_restore_ = (env->GetBooleanField(p, res->is_restore_fieldID) != 0);
  res->is_upgrade_ = (env->GetBooleanField(p, res->is_upgrade_fieldID) != 0);
  return std::move(res);
}

void storePaymentPurposePremiumSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposePremiumSubscription");
    s.store_field("isRestore", is_restore_);
    s.store_field("isUpgrade", is_upgrade_);
    s.store_class_end();
  }
}

void storePaymentPurposePremiumSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposePremiumSubscription").c_str());
    is_restore_fieldID = jni::get_field_id(env, Class, "isRestore", "Z");
    is_upgrade_fieldID = jni::get_field_id(env, Class, "isUpgrade", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposePremiumGift::Class;
jfieldID storePaymentPurposePremiumGift::currency_fieldID;
jfieldID storePaymentPurposePremiumGift::amount_fieldID;
jfieldID storePaymentPurposePremiumGift::user_id_fieldID;
jfieldID storePaymentPurposePremiumGift::text_fieldID;

storePaymentPurposePremiumGift::storePaymentPurposePremiumGift()
  : currency_()
  , amount_()
  , user_id_()
  , text_()
{}

storePaymentPurposePremiumGift::storePaymentPurposePremiumGift(string const &currency_, int53 amount_, int53 user_id_, object_ptr<formattedText> &&text_)
  : currency_(currency_)
  , amount_(amount_)
  , user_id_(user_id_)
  , text_(std::move(text_))
{}

const std::int32_t storePaymentPurposePremiumGift::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposePremiumGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposePremiumGift> res = make_object<storePaymentPurposePremiumGift>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void storePaymentPurposePremiumGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposePremiumGift");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("userId", user_id_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void storePaymentPurposePremiumGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposePremiumGift").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposePremiumGiftCodes::Class;
jfieldID storePaymentPurposePremiumGiftCodes::boosted_chat_id_fieldID;
jfieldID storePaymentPurposePremiumGiftCodes::currency_fieldID;
jfieldID storePaymentPurposePremiumGiftCodes::amount_fieldID;
jfieldID storePaymentPurposePremiumGiftCodes::user_ids_fieldID;
jfieldID storePaymentPurposePremiumGiftCodes::text_fieldID;

storePaymentPurposePremiumGiftCodes::storePaymentPurposePremiumGiftCodes()
  : boosted_chat_id_()
  , currency_()
  , amount_()
  , user_ids_()
  , text_()
{}

storePaymentPurposePremiumGiftCodes::storePaymentPurposePremiumGiftCodes(int53 boosted_chat_id_, string const &currency_, int53 amount_, array<int53> &&user_ids_, object_ptr<formattedText> &&text_)
  : boosted_chat_id_(boosted_chat_id_)
  , currency_(currency_)
  , amount_(amount_)
  , user_ids_(std::move(user_ids_))
  , text_(std::move(text_))
{}

const std::int32_t storePaymentPurposePremiumGiftCodes::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposePremiumGiftCodes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposePremiumGiftCodes> res = make_object<storePaymentPurposePremiumGiftCodes>();
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void storePaymentPurposePremiumGiftCodes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposePremiumGiftCodes");
    s.store_field("boostedChatId", boosted_chat_id_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void storePaymentPurposePremiumGiftCodes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposePremiumGiftCodes").c_str());
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposePremiumGiveaway::Class;
jfieldID storePaymentPurposePremiumGiveaway::parameters_fieldID;
jfieldID storePaymentPurposePremiumGiveaway::currency_fieldID;
jfieldID storePaymentPurposePremiumGiveaway::amount_fieldID;

storePaymentPurposePremiumGiveaway::storePaymentPurposePremiumGiveaway()
  : parameters_()
  , currency_()
  , amount_()
{}

storePaymentPurposePremiumGiveaway::storePaymentPurposePremiumGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_)
  : parameters_(std::move(parameters_))
  , currency_(currency_)
  , amount_(amount_)
{}

const std::int32_t storePaymentPurposePremiumGiveaway::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposePremiumGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposePremiumGiveaway> res = make_object<storePaymentPurposePremiumGiveaway>();
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  return std::move(res);
}

void storePaymentPurposePremiumGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposePremiumGiveaway");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_class_end();
  }
}

void storePaymentPurposePremiumGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposePremiumGiveaway").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposeStarGiveaway::Class;
jfieldID storePaymentPurposeStarGiveaway::parameters_fieldID;
jfieldID storePaymentPurposeStarGiveaway::currency_fieldID;
jfieldID storePaymentPurposeStarGiveaway::amount_fieldID;
jfieldID storePaymentPurposeStarGiveaway::winner_count_fieldID;
jfieldID storePaymentPurposeStarGiveaway::star_count_fieldID;

storePaymentPurposeStarGiveaway::storePaymentPurposeStarGiveaway()
  : parameters_()
  , currency_()
  , amount_()
  , winner_count_()
  , star_count_()
{}

storePaymentPurposeStarGiveaway::storePaymentPurposeStarGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int53 star_count_)
  : parameters_(std::move(parameters_))
  , currency_(currency_)
  , amount_(amount_)
  , winner_count_(winner_count_)
  , star_count_(star_count_)
{}

const std::int32_t storePaymentPurposeStarGiveaway::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposeStarGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposeStarGiveaway> res = make_object<storePaymentPurposeStarGiveaway>();
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void storePaymentPurposeStarGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposeStarGiveaway");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void storePaymentPurposeStarGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposeStarGiveaway").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposeStars::Class;
jfieldID storePaymentPurposeStars::currency_fieldID;
jfieldID storePaymentPurposeStars::amount_fieldID;
jfieldID storePaymentPurposeStars::star_count_fieldID;

storePaymentPurposeStars::storePaymentPurposeStars()
  : currency_()
  , amount_()
  , star_count_()
{}

storePaymentPurposeStars::storePaymentPurposeStars(string const &currency_, int53 amount_, int53 star_count_)
  : currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
{}

const std::int32_t storePaymentPurposeStars::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposeStars> res = make_object<storePaymentPurposeStars>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void storePaymentPurposeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposeStars");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void storePaymentPurposeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposeStars").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass storePaymentPurposeGiftedStars::Class;
jfieldID storePaymentPurposeGiftedStars::user_id_fieldID;
jfieldID storePaymentPurposeGiftedStars::currency_fieldID;
jfieldID storePaymentPurposeGiftedStars::amount_fieldID;
jfieldID storePaymentPurposeGiftedStars::star_count_fieldID;

storePaymentPurposeGiftedStars::storePaymentPurposeGiftedStars()
  : user_id_()
  , currency_()
  , amount_()
  , star_count_()
{}

storePaymentPurposeGiftedStars::storePaymentPurposeGiftedStars(int53 user_id_, string const &currency_, int53 amount_, int53 star_count_)
  : user_id_(user_id_)
  , currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
{}

const std::int32_t storePaymentPurposeGiftedStars::ID;

object_ptr<StorePaymentPurpose> storePaymentPurposeGiftedStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storePaymentPurposeGiftedStars> res = make_object<storePaymentPurposeGiftedStars>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void storePaymentPurposeGiftedStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StorePaymentPurposeGiftedStars");
    s.store_field("userId", user_id_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void storePaymentPurposeGiftedStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StorePaymentPurposeGiftedStars").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass StoreTransaction::Class;

object_ptr<StoreTransaction> StoreTransaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storeTransactionAppStore::ID:
      return storeTransactionAppStore::fetch(env, p);
    case storeTransactionGooglePlay::ID:
      return storeTransactionGooglePlay::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoreTransaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoreTransaction").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storeTransactionAppStore::Class;
jfieldID storeTransactionAppStore::receipt_fieldID;

storeTransactionAppStore::storeTransactionAppStore()
  : receipt_()
{}

storeTransactionAppStore::storeTransactionAppStore(bytes const &receipt_)
  : receipt_(std::move(receipt_))
{}

const std::int32_t storeTransactionAppStore::ID;

object_ptr<StoreTransaction> storeTransactionAppStore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storeTransactionAppStore> res = make_object<storeTransactionAppStore>();
  res->receipt_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->receipt_fieldID));
  return std::move(res);
}

void storeTransactionAppStore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoreTransactionAppStore");
    s.store_bytes_field("receipt", receipt_);
    s.store_class_end();
  }
}

void storeTransactionAppStore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoreTransactionAppStore").c_str());
    receipt_fieldID = jni::get_field_id(env, Class, "receipt", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass storeTransactionGooglePlay::Class;
jfieldID storeTransactionGooglePlay::package_name_fieldID;
jfieldID storeTransactionGooglePlay::store_product_id_fieldID;
jfieldID storeTransactionGooglePlay::purchase_token_fieldID;

storeTransactionGooglePlay::storeTransactionGooglePlay()
  : package_name_()
  , store_product_id_()
  , purchase_token_()
{}

storeTransactionGooglePlay::storeTransactionGooglePlay(string const &package_name_, string const &store_product_id_, string const &purchase_token_)
  : package_name_(package_name_)
  , store_product_id_(store_product_id_)
  , purchase_token_(purchase_token_)
{}

const std::int32_t storeTransactionGooglePlay::ID;

object_ptr<StoreTransaction> storeTransactionGooglePlay::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storeTransactionGooglePlay> res = make_object<storeTransactionGooglePlay>();
  res->package_name_ = jni::fetch_string(env, p, res->package_name_fieldID);
  res->store_product_id_ = jni::fetch_string(env, p, res->store_product_id_fieldID);
  res->purchase_token_ = jni::fetch_string(env, p, res->purchase_token_fieldID);
  return std::move(res);
}

void storeTransactionGooglePlay::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoreTransactionGooglePlay");
    s.store_field("packageName", package_name_);
    s.store_field("storeProductId", store_product_id_);
    s.store_field("purchaseToken", purchase_token_);
    s.store_class_end();
  }
}

void storeTransactionGooglePlay::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoreTransactionGooglePlay").c_str());
    package_name_fieldID = jni::get_field_id(env, Class, "packageName", "Ljava/lang/String;");
    store_product_id_fieldID = jni::get_field_id(env, Class, "storeProductId", "Ljava/lang/String;");
    purchase_token_fieldID = jni::get_field_id(env, Class, "purchaseToken", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass stories::Class;
jfieldID stories::total_count_fieldID;
jfieldID stories::stories_fieldID;
jfieldID stories::pinned_story_ids_fieldID;

stories::stories()
  : total_count_()
  , stories_()
  , pinned_story_ids_()
{}

stories::stories(int32 total_count_, array<object_ptr<story>> &&stories_, array<int32> &&pinned_story_ids_)
  : total_count_(total_count_)
  , stories_(std::move(stories_))
  , pinned_story_ids_(std::move(pinned_story_ids_))
{}

const std::int32_t stories::ID;

object_ptr<stories> stories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stories> res = make_object<stories>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->stories_ = jni::FetchVector<story>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stories_fieldID));
  res->pinned_story_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->pinned_story_ids_fieldID));
  return res;
}

void stories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, stories_); if (arr_tmp_) { env->SetObjectField(s, stories_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, pinned_story_ids_); if (arr_tmp_) { env->SetObjectField(s, pinned_story_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void stories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Stories");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("stories", stories_.size()); for (const auto &_value : stories_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("pinnedStoryIds", pinned_story_ids_.size()); for (const auto &_value : pinned_story_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void stories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Stories").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    stories_fieldID = jni::get_field_id(env, Class, "stories", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    pinned_story_ids_fieldID = jni::get_field_id(env, Class, "pinnedStoryIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass story::Class;
jfieldID story::id_fieldID;
jfieldID story::poster_chat_id_fieldID;
jfieldID story::poster_id_fieldID;
jfieldID story::date_fieldID;
jfieldID story::is_being_posted_fieldID;
jfieldID story::is_being_edited_fieldID;
jfieldID story::is_edited_fieldID;
jfieldID story::is_posted_to_chat_page_fieldID;
jfieldID story::is_visible_only_for_self_fieldID;
jfieldID story::can_be_deleted_fieldID;
jfieldID story::can_be_edited_fieldID;
jfieldID story::can_be_forwarded_fieldID;
jfieldID story::can_be_replied_fieldID;
jfieldID story::can_toggle_is_posted_to_chat_page_fieldID;
jfieldID story::can_get_statistics_fieldID;
jfieldID story::can_get_interactions_fieldID;
jfieldID story::has_expired_viewers_fieldID;
jfieldID story::repost_info_fieldID;
jfieldID story::interaction_info_fieldID;
jfieldID story::chosen_reaction_type_fieldID;
jfieldID story::privacy_settings_fieldID;
jfieldID story::content_fieldID;
jfieldID story::areas_fieldID;
jfieldID story::caption_fieldID;

story::story()
  : id_()
  , poster_chat_id_()
  , poster_id_()
  , date_()
  , is_being_posted_()
  , is_being_edited_()
  , is_edited_()
  , is_posted_to_chat_page_()
  , is_visible_only_for_self_()
  , can_be_deleted_()
  , can_be_edited_()
  , can_be_forwarded_()
  , can_be_replied_()
  , can_toggle_is_posted_to_chat_page_()
  , can_get_statistics_()
  , can_get_interactions_()
  , has_expired_viewers_()
  , repost_info_()
  , interaction_info_()
  , chosen_reaction_type_()
  , privacy_settings_()
  , content_()
  , areas_()
  , caption_()
{}

story::story(int32 id_, int53 poster_chat_id_, object_ptr<MessageSender> &&poster_id_, int32 date_, bool is_being_posted_, bool is_being_edited_, bool is_edited_, bool is_posted_to_chat_page_, bool is_visible_only_for_self_, bool can_be_deleted_, bool can_be_edited_, bool can_be_forwarded_, bool can_be_replied_, bool can_toggle_is_posted_to_chat_page_, bool can_get_statistics_, bool can_get_interactions_, bool has_expired_viewers_, object_ptr<storyRepostInfo> &&repost_info_, object_ptr<storyInteractionInfo> &&interaction_info_, object_ptr<ReactionType> &&chosen_reaction_type_, object_ptr<StoryPrivacySettings> &&privacy_settings_, object_ptr<StoryContent> &&content_, array<object_ptr<storyArea>> &&areas_, object_ptr<formattedText> &&caption_)
  : id_(id_)
  , poster_chat_id_(poster_chat_id_)
  , poster_id_(std::move(poster_id_))
  , date_(date_)
  , is_being_posted_(is_being_posted_)
  , is_being_edited_(is_being_edited_)
  , is_edited_(is_edited_)
  , is_posted_to_chat_page_(is_posted_to_chat_page_)
  , is_visible_only_for_self_(is_visible_only_for_self_)
  , can_be_deleted_(can_be_deleted_)
  , can_be_edited_(can_be_edited_)
  , can_be_forwarded_(can_be_forwarded_)
  , can_be_replied_(can_be_replied_)
  , can_toggle_is_posted_to_chat_page_(can_toggle_is_posted_to_chat_page_)
  , can_get_statistics_(can_get_statistics_)
  , can_get_interactions_(can_get_interactions_)
  , has_expired_viewers_(has_expired_viewers_)
  , repost_info_(std::move(repost_info_))
  , interaction_info_(std::move(interaction_info_))
  , chosen_reaction_type_(std::move(chosen_reaction_type_))
  , privacy_settings_(std::move(privacy_settings_))
  , content_(std::move(content_))
  , areas_(std::move(areas_))
  , caption_(std::move(caption_))
{}

const std::int32_t story::ID;

object_ptr<story> story::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<story> res = make_object<story>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->poster_chat_id_ = env->GetLongField(p, res->poster_chat_id_fieldID);
  res->poster_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->poster_id_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->is_being_posted_ = (env->GetBooleanField(p, res->is_being_posted_fieldID) != 0);
  res->is_being_edited_ = (env->GetBooleanField(p, res->is_being_edited_fieldID) != 0);
  res->is_edited_ = (env->GetBooleanField(p, res->is_edited_fieldID) != 0);
  res->is_posted_to_chat_page_ = (env->GetBooleanField(p, res->is_posted_to_chat_page_fieldID) != 0);
  res->is_visible_only_for_self_ = (env->GetBooleanField(p, res->is_visible_only_for_self_fieldID) != 0);
  res->can_be_deleted_ = (env->GetBooleanField(p, res->can_be_deleted_fieldID) != 0);
  res->can_be_edited_ = (env->GetBooleanField(p, res->can_be_edited_fieldID) != 0);
  res->can_be_forwarded_ = (env->GetBooleanField(p, res->can_be_forwarded_fieldID) != 0);
  res->can_be_replied_ = (env->GetBooleanField(p, res->can_be_replied_fieldID) != 0);
  res->can_toggle_is_posted_to_chat_page_ = (env->GetBooleanField(p, res->can_toggle_is_posted_to_chat_page_fieldID) != 0);
  res->can_get_statistics_ = (env->GetBooleanField(p, res->can_get_statistics_fieldID) != 0);
  res->can_get_interactions_ = (env->GetBooleanField(p, res->can_get_interactions_fieldID) != 0);
  res->has_expired_viewers_ = (env->GetBooleanField(p, res->has_expired_viewers_fieldID) != 0);
  res->repost_info_ = jni::fetch_tl_object<storyRepostInfo>(env, jni::fetch_object(env, p, res->repost_info_fieldID));
  res->interaction_info_ = jni::fetch_tl_object<storyInteractionInfo>(env, jni::fetch_object(env, p, res->interaction_info_fieldID));
  res->chosen_reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->chosen_reaction_type_fieldID));
  res->privacy_settings_ = jni::fetch_tl_object<StoryPrivacySettings>(env, jni::fetch_object(env, p, res->privacy_settings_fieldID));
  res->content_ = jni::fetch_tl_object<StoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->areas_ = jni::FetchVector<storyArea>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->areas_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return res;
}

void story::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  env->SetLongField(s, poster_chat_id_fieldID, poster_chat_id_);
  if (poster_id_ != nullptr) { jobject next; poster_id_->store(env, next); if (next) { env->SetObjectField(s, poster_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  env->SetBooleanField(s, is_being_posted_fieldID, is_being_posted_);
  env->SetBooleanField(s, is_being_edited_fieldID, is_being_edited_);
  env->SetBooleanField(s, is_edited_fieldID, is_edited_);
  env->SetBooleanField(s, is_posted_to_chat_page_fieldID, is_posted_to_chat_page_);
  env->SetBooleanField(s, is_visible_only_for_self_fieldID, is_visible_only_for_self_);
  env->SetBooleanField(s, can_be_deleted_fieldID, can_be_deleted_);
  env->SetBooleanField(s, can_be_edited_fieldID, can_be_edited_);
  env->SetBooleanField(s, can_be_forwarded_fieldID, can_be_forwarded_);
  env->SetBooleanField(s, can_be_replied_fieldID, can_be_replied_);
  env->SetBooleanField(s, can_toggle_is_posted_to_chat_page_fieldID, can_toggle_is_posted_to_chat_page_);
  env->SetBooleanField(s, can_get_statistics_fieldID, can_get_statistics_);
  env->SetBooleanField(s, can_get_interactions_fieldID, can_get_interactions_);
  env->SetBooleanField(s, has_expired_viewers_fieldID, has_expired_viewers_);
  if (repost_info_ != nullptr) { jobject next; repost_info_->store(env, next); if (next) { env->SetObjectField(s, repost_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (interaction_info_ != nullptr) { jobject next; interaction_info_->store(env, next); if (next) { env->SetObjectField(s, interaction_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (chosen_reaction_type_ != nullptr) { jobject next; chosen_reaction_type_->store(env, next); if (next) { env->SetObjectField(s, chosen_reaction_type_fieldID, next); env->DeleteLocalRef(next); } }
  if (privacy_settings_ != nullptr) { jobject next; privacy_settings_->store(env, next); if (next) { env->SetObjectField(s, privacy_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, areas_); if (arr_tmp_) { env->SetObjectField(s, areas_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (caption_ != nullptr) { jobject next; caption_->store(env, next); if (next) { env->SetObjectField(s, caption_fieldID, next); env->DeleteLocalRef(next); } }
}

void story::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Story");
    s.store_field("id", id_);
    s.store_field("posterChatId", poster_chat_id_);
    s.store_object_field("posterId", static_cast<const BaseObject *>(poster_id_.get()));
    s.store_field("date", date_);
    s.store_field("isBeingPosted", is_being_posted_);
    s.store_field("isBeingEdited", is_being_edited_);
    s.store_field("isEdited", is_edited_);
    s.store_field("isPostedToChatPage", is_posted_to_chat_page_);
    s.store_field("isVisibleOnlyForSelf", is_visible_only_for_self_);
    s.store_field("canBeDeleted", can_be_deleted_);
    s.store_field("canBeEdited", can_be_edited_);
    s.store_field("canBeForwarded", can_be_forwarded_);
    s.store_field("canBeReplied", can_be_replied_);
    s.store_field("canToggleIsPostedToChatPage", can_toggle_is_posted_to_chat_page_);
    s.store_field("canGetStatistics", can_get_statistics_);
    s.store_field("canGetInteractions", can_get_interactions_);
    s.store_field("hasExpiredViewers", has_expired_viewers_);
    s.store_object_field("repostInfo", static_cast<const BaseObject *>(repost_info_.get()));
    s.store_object_field("interactionInfo", static_cast<const BaseObject *>(interaction_info_.get()));
    s.store_object_field("chosenReactionType", static_cast<const BaseObject *>(chosen_reaction_type_.get()));
    s.store_object_field("privacySettings", static_cast<const BaseObject *>(privacy_settings_.get()));
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    { s.store_vector_begin("areas", areas_.size()); for (const auto &_value : areas_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

void story::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Story").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    poster_chat_id_fieldID = jni::get_field_id(env, Class, "posterChatId", "J");
    poster_id_fieldID = jni::get_field_id(env, Class, "posterId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    is_being_posted_fieldID = jni::get_field_id(env, Class, "isBeingPosted", "Z");
    is_being_edited_fieldID = jni::get_field_id(env, Class, "isBeingEdited", "Z");
    is_edited_fieldID = jni::get_field_id(env, Class, "isEdited", "Z");
    is_posted_to_chat_page_fieldID = jni::get_field_id(env, Class, "isPostedToChatPage", "Z");
    is_visible_only_for_self_fieldID = jni::get_field_id(env, Class, "isVisibleOnlyForSelf", "Z");
    can_be_deleted_fieldID = jni::get_field_id(env, Class, "canBeDeleted", "Z");
    can_be_edited_fieldID = jni::get_field_id(env, Class, "canBeEdited", "Z");
    can_be_forwarded_fieldID = jni::get_field_id(env, Class, "canBeForwarded", "Z");
    can_be_replied_fieldID = jni::get_field_id(env, Class, "canBeReplied", "Z");
    can_toggle_is_posted_to_chat_page_fieldID = jni::get_field_id(env, Class, "canToggleIsPostedToChatPage", "Z");
    can_get_statistics_fieldID = jni::get_field_id(env, Class, "canGetStatistics", "Z");
    can_get_interactions_fieldID = jni::get_field_id(env, Class, "canGetInteractions", "Z");
    has_expired_viewers_fieldID = jni::get_field_id(env, Class, "hasExpiredViewers", "Z");
    repost_info_fieldID = jni::get_field_id(env, Class, "repostInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryRepostInfo;").c_str());
    interaction_info_fieldID = jni::get_field_id(env, Class, "interactionInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryInteractionInfo;").c_str());
    chosen_reaction_type_fieldID = jni::get_field_id(env, Class, "chosenReactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    privacy_settings_fieldID = jni::get_field_id(env, Class, "privacySettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryPrivacySettings;").c_str());
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryContent;").c_str());
    areas_fieldID = jni::get_field_id(env, Class, "areas", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StoryArea;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyArea::Class;
jfieldID storyArea::position_fieldID;
jfieldID storyArea::type_fieldID;

storyArea::storyArea()
  : position_()
  , type_()
{}

storyArea::storyArea(object_ptr<storyAreaPosition> &&position_, object_ptr<StoryAreaType> &&type_)
  : position_(std::move(position_))
  , type_(std::move(type_))
{}

const std::int32_t storyArea::ID;

object_ptr<storyArea> storyArea::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyArea> res = make_object<storyArea>();
  res->position_ = jni::fetch_tl_object<storyAreaPosition>(env, jni::fetch_object(env, p, res->position_fieldID));
  res->type_ = jni::fetch_tl_object<StoryAreaType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void storyArea::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (position_ != nullptr) { jobject next; position_->store(env, next); if (next) { env->SetObjectField(s, position_fieldID, next); env->DeleteLocalRef(next); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyArea::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryArea");
    s.store_object_field("position", static_cast<const BaseObject *>(position_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void storyArea::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryArea").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryAreaPosition;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryAreaType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaPosition::Class;
jfieldID storyAreaPosition::x_percentage_fieldID;
jfieldID storyAreaPosition::y_percentage_fieldID;
jfieldID storyAreaPosition::width_percentage_fieldID;
jfieldID storyAreaPosition::height_percentage_fieldID;
jfieldID storyAreaPosition::rotation_angle_fieldID;
jfieldID storyAreaPosition::corner_radius_percentage_fieldID;

storyAreaPosition::storyAreaPosition()
  : x_percentage_()
  , y_percentage_()
  , width_percentage_()
  , height_percentage_()
  , rotation_angle_()
  , corner_radius_percentage_()
{}

storyAreaPosition::storyAreaPosition(double x_percentage_, double y_percentage_, double width_percentage_, double height_percentage_, double rotation_angle_, double corner_radius_percentage_)
  : x_percentage_(x_percentage_)
  , y_percentage_(y_percentage_)
  , width_percentage_(width_percentage_)
  , height_percentage_(height_percentage_)
  , rotation_angle_(rotation_angle_)
  , corner_radius_percentage_(corner_radius_percentage_)
{}

const std::int32_t storyAreaPosition::ID;

object_ptr<storyAreaPosition> storyAreaPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaPosition> res = make_object<storyAreaPosition>();
  res->x_percentage_ = env->GetDoubleField(p, res->x_percentage_fieldID);
  res->y_percentage_ = env->GetDoubleField(p, res->y_percentage_fieldID);
  res->width_percentage_ = env->GetDoubleField(p, res->width_percentage_fieldID);
  res->height_percentage_ = env->GetDoubleField(p, res->height_percentage_fieldID);
  res->rotation_angle_ = env->GetDoubleField(p, res->rotation_angle_fieldID);
  res->corner_radius_percentage_ = env->GetDoubleField(p, res->corner_radius_percentage_fieldID);
  return res;
}

void storyAreaPosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, x_percentage_fieldID, x_percentage_);
  env->SetDoubleField(s, y_percentage_fieldID, y_percentage_);
  env->SetDoubleField(s, width_percentage_fieldID, width_percentage_);
  env->SetDoubleField(s, height_percentage_fieldID, height_percentage_);
  env->SetDoubleField(s, rotation_angle_fieldID, rotation_angle_);
  env->SetDoubleField(s, corner_radius_percentage_fieldID, corner_radius_percentage_);
}

void storyAreaPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaPosition");
    s.store_field("xPercentage", x_percentage_);
    s.store_field("yPercentage", y_percentage_);
    s.store_field("widthPercentage", width_percentage_);
    s.store_field("heightPercentage", height_percentage_);
    s.store_field("rotationAngle", rotation_angle_);
    s.store_field("cornerRadiusPercentage", corner_radius_percentage_);
    s.store_class_end();
  }
}

void storyAreaPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaPosition").c_str());
    x_percentage_fieldID = jni::get_field_id(env, Class, "xPercentage", "D");
    y_percentage_fieldID = jni::get_field_id(env, Class, "yPercentage", "D");
    width_percentage_fieldID = jni::get_field_id(env, Class, "widthPercentage", "D");
    height_percentage_fieldID = jni::get_field_id(env, Class, "heightPercentage", "D");
    rotation_angle_fieldID = jni::get_field_id(env, Class, "rotationAngle", "D");
    corner_radius_percentage_fieldID = jni::get_field_id(env, Class, "cornerRadiusPercentage", "D");
    return true;
  }();
  (void)is_inited;
}

jclass StoryAreaType::Class;

object_ptr<StoryAreaType> StoryAreaType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyAreaTypeLocation::ID:
      return storyAreaTypeLocation::fetch(env, p);
    case storyAreaTypeVenue::ID:
      return storyAreaTypeVenue::fetch(env, p);
    case storyAreaTypeSuggestedReaction::ID:
      return storyAreaTypeSuggestedReaction::fetch(env, p);
    case storyAreaTypeMessage::ID:
      return storyAreaTypeMessage::fetch(env, p);
    case storyAreaTypeLink::ID:
      return storyAreaTypeLink::fetch(env, p);
    case storyAreaTypeWeather::ID:
      return storyAreaTypeWeather::fetch(env, p);
    case storyAreaTypeUpgradedGift::ID:
      return storyAreaTypeUpgradedGift::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryAreaType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeLocation::Class;
jfieldID storyAreaTypeLocation::location_fieldID;
jfieldID storyAreaTypeLocation::address_fieldID;

storyAreaTypeLocation::storyAreaTypeLocation()
  : location_()
  , address_()
{}

storyAreaTypeLocation::storyAreaTypeLocation(object_ptr<location> &&location_, object_ptr<locationAddress> &&address_)
  : location_(std::move(location_))
  , address_(std::move(address_))
{}

const std::int32_t storyAreaTypeLocation::ID;

object_ptr<StoryAreaType> storyAreaTypeLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeLocation> res = make_object<storyAreaTypeLocation>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->address_ = jni::fetch_tl_object<locationAddress>(env, jni::fetch_object(env, p, res->address_fieldID));
  return std::move(res);
}

void storyAreaTypeLocation::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  if (address_ != nullptr) { jobject next; address_->store(env, next); if (next) { env->SetObjectField(s, address_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyAreaTypeLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_object_field("address", static_cast<const BaseObject *>(address_.get()));
    s.store_class_end();
  }
}

void storyAreaTypeLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LocationAddress;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeVenue::Class;
jfieldID storyAreaTypeVenue::venue_fieldID;

storyAreaTypeVenue::storyAreaTypeVenue()
  : venue_()
{}

storyAreaTypeVenue::storyAreaTypeVenue(object_ptr<venue> &&venue_)
  : venue_(std::move(venue_))
{}

const std::int32_t storyAreaTypeVenue::ID;

object_ptr<StoryAreaType> storyAreaTypeVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeVenue> res = make_object<storyAreaTypeVenue>();
  res->venue_ = jni::fetch_tl_object<venue>(env, jni::fetch_object(env, p, res->venue_fieldID));
  return std::move(res);
}

void storyAreaTypeVenue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (venue_ != nullptr) { jobject next; venue_->store(env, next); if (next) { env->SetObjectField(s, venue_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyAreaTypeVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeVenue");
    s.store_object_field("venue", static_cast<const BaseObject *>(venue_.get()));
    s.store_class_end();
  }
}

void storyAreaTypeVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeVenue").c_str());
    venue_fieldID = jni::get_field_id(env, Class, "venue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Venue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeSuggestedReaction::Class;
jfieldID storyAreaTypeSuggestedReaction::reaction_type_fieldID;
jfieldID storyAreaTypeSuggestedReaction::total_count_fieldID;
jfieldID storyAreaTypeSuggestedReaction::is_dark_fieldID;
jfieldID storyAreaTypeSuggestedReaction::is_flipped_fieldID;

storyAreaTypeSuggestedReaction::storyAreaTypeSuggestedReaction()
  : reaction_type_()
  , total_count_()
  , is_dark_()
  , is_flipped_()
{}

storyAreaTypeSuggestedReaction::storyAreaTypeSuggestedReaction(object_ptr<ReactionType> &&reaction_type_, int32 total_count_, bool is_dark_, bool is_flipped_)
  : reaction_type_(std::move(reaction_type_))
  , total_count_(total_count_)
  , is_dark_(is_dark_)
  , is_flipped_(is_flipped_)
{}

const std::int32_t storyAreaTypeSuggestedReaction::ID;

object_ptr<StoryAreaType> storyAreaTypeSuggestedReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeSuggestedReaction> res = make_object<storyAreaTypeSuggestedReaction>();
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  res->is_flipped_ = (env->GetBooleanField(p, res->is_flipped_fieldID) != 0);
  return std::move(res);
}

void storyAreaTypeSuggestedReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (reaction_type_ != nullptr) { jobject next; reaction_type_->store(env, next); if (next) { env->SetObjectField(s, reaction_type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetBooleanField(s, is_dark_fieldID, is_dark_);
  env->SetBooleanField(s, is_flipped_fieldID, is_flipped_);
}

void storyAreaTypeSuggestedReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeSuggestedReaction");
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("totalCount", total_count_);
    s.store_field("isDark", is_dark_);
    s.store_field("isFlipped", is_flipped_);
    s.store_class_end();
  }
}

void storyAreaTypeSuggestedReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeSuggestedReaction").c_str());
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    is_flipped_fieldID = jni::get_field_id(env, Class, "isFlipped", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeMessage::Class;
jfieldID storyAreaTypeMessage::chat_id_fieldID;
jfieldID storyAreaTypeMessage::message_id_fieldID;

storyAreaTypeMessage::storyAreaTypeMessage()
  : chat_id_()
  , message_id_()
{}

storyAreaTypeMessage::storyAreaTypeMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t storyAreaTypeMessage::ID;

object_ptr<StoryAreaType> storyAreaTypeMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeMessage> res = make_object<storyAreaTypeMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void storyAreaTypeMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void storyAreaTypeMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void storyAreaTypeMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeLink::Class;
jfieldID storyAreaTypeLink::url_fieldID;

storyAreaTypeLink::storyAreaTypeLink()
  : url_()
{}

storyAreaTypeLink::storyAreaTypeLink(string const &url_)
  : url_(url_)
{}

const std::int32_t storyAreaTypeLink::ID;

object_ptr<StoryAreaType> storyAreaTypeLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeLink> res = make_object<storyAreaTypeLink>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void storyAreaTypeLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void storyAreaTypeLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeLink");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void storyAreaTypeLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeLink").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeWeather::Class;
jfieldID storyAreaTypeWeather::temperature_fieldID;
jfieldID storyAreaTypeWeather::emoji_fieldID;
jfieldID storyAreaTypeWeather::background_color_fieldID;

storyAreaTypeWeather::storyAreaTypeWeather()
  : temperature_()
  , emoji_()
  , background_color_()
{}

storyAreaTypeWeather::storyAreaTypeWeather(double temperature_, string const &emoji_, int32 background_color_)
  : temperature_(temperature_)
  , emoji_(emoji_)
  , background_color_(background_color_)
{}

const std::int32_t storyAreaTypeWeather::ID;

object_ptr<StoryAreaType> storyAreaTypeWeather::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeWeather> res = make_object<storyAreaTypeWeather>();
  res->temperature_ = env->GetDoubleField(p, res->temperature_fieldID);
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  res->background_color_ = env->GetIntField(p, res->background_color_fieldID);
  return std::move(res);
}

void storyAreaTypeWeather::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, temperature_fieldID, temperature_);
  { jstring nextString = jni::to_jstring(env, emoji_); if (nextString) { env->SetObjectField(s, emoji_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, background_color_fieldID, background_color_);
}

void storyAreaTypeWeather::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeWeather");
    s.store_field("temperature", temperature_);
    s.store_field("emoji", emoji_);
    s.store_field("backgroundColor", background_color_);
    s.store_class_end();
  }
}

void storyAreaTypeWeather::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeWeather").c_str());
    temperature_fieldID = jni::get_field_id(env, Class, "temperature", "D");
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    background_color_fieldID = jni::get_field_id(env, Class, "backgroundColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass storyAreaTypeUpgradedGift::Class;
jfieldID storyAreaTypeUpgradedGift::gift_name_fieldID;

storyAreaTypeUpgradedGift::storyAreaTypeUpgradedGift()
  : gift_name_()
{}

storyAreaTypeUpgradedGift::storyAreaTypeUpgradedGift(string const &gift_name_)
  : gift_name_(gift_name_)
{}

const std::int32_t storyAreaTypeUpgradedGift::ID;

object_ptr<StoryAreaType> storyAreaTypeUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyAreaTypeUpgradedGift> res = make_object<storyAreaTypeUpgradedGift>();
  res->gift_name_ = jni::fetch_string(env, p, res->gift_name_fieldID);
  return std::move(res);
}

void storyAreaTypeUpgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, gift_name_); if (nextString) { env->SetObjectField(s, gift_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void storyAreaTypeUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryAreaTypeUpgradedGift");
    s.store_field("giftName", gift_name_);
    s.store_class_end();
  }
}

void storyAreaTypeUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryAreaTypeUpgradedGift").c_str());
    gift_name_fieldID = jni::get_field_id(env, Class, "giftName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass StoryContent::Class;

object_ptr<StoryContent> StoryContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyContentPhoto::ID:
      return storyContentPhoto::fetch(env, p);
    case storyContentVideo::ID:
      return storyContentVideo::fetch(env, p);
    case storyContentUnsupported::ID:
      return storyContentUnsupported::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryContent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyContentPhoto::Class;
jfieldID storyContentPhoto::photo_fieldID;

storyContentPhoto::storyContentPhoto()
  : photo_()
{}

storyContentPhoto::storyContentPhoto(object_ptr<photo> &&photo_)
  : photo_(std::move(photo_))
{}

const std::int32_t storyContentPhoto::ID;

object_ptr<StoryContent> storyContentPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyContentPhoto> res = make_object<storyContentPhoto>();
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void storyContentPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyContentPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryContentPhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void storyContentPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryContentPhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyContentVideo::Class;
jfieldID storyContentVideo::video_fieldID;
jfieldID storyContentVideo::alternative_video_fieldID;

storyContentVideo::storyContentVideo()
  : video_()
  , alternative_video_()
{}

storyContentVideo::storyContentVideo(object_ptr<storyVideo> &&video_, object_ptr<storyVideo> &&alternative_video_)
  : video_(std::move(video_))
  , alternative_video_(std::move(alternative_video_))
{}

const std::int32_t storyContentVideo::ID;

object_ptr<StoryContent> storyContentVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyContentVideo> res = make_object<storyContentVideo>();
  res->video_ = jni::fetch_tl_object<storyVideo>(env, jni::fetch_object(env, p, res->video_fieldID));
  res->alternative_video_ = jni::fetch_tl_object<storyVideo>(env, jni::fetch_object(env, p, res->alternative_video_fieldID));
  return std::move(res);
}

void storyContentVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
  if (alternative_video_ != nullptr) { jobject next; alternative_video_->store(env, next); if (next) { env->SetObjectField(s, alternative_video_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyContentVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryContentVideo");
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_object_field("alternativeVideo", static_cast<const BaseObject *>(alternative_video_.get()));
    s.store_class_end();
  }
}

void storyContentVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryContentVideo").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryVideo;").c_str());
    alternative_video_fieldID = jni::get_field_id(env, Class, "alternativeVideo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryVideo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyContentUnsupported::Class;

storyContentUnsupported::storyContentUnsupported() {
}

const std::int32_t storyContentUnsupported::ID;

object_ptr<StoryContent> storyContentUnsupported::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<storyContentUnsupported>();
}

void storyContentUnsupported::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void storyContentUnsupported::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryContentUnsupported");
    s.store_class_end();
  }
}

void storyContentUnsupported::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryContentUnsupported").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyFullId::Class;
jfieldID storyFullId::poster_chat_id_fieldID;
jfieldID storyFullId::story_id_fieldID;

storyFullId::storyFullId()
  : poster_chat_id_()
  , story_id_()
{}

storyFullId::storyFullId(int53 poster_chat_id_, int32 story_id_)
  : poster_chat_id_(poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t storyFullId::ID;

object_ptr<storyFullId> storyFullId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyFullId> res = make_object<storyFullId>();
  res->poster_chat_id_ = env->GetLongField(p, res->poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return res;
}

void storyFullId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryFullId");
    s.store_field("posterChatId", poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void storyFullId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryFullId").c_str());
    poster_chat_id_fieldID = jni::get_field_id(env, Class, "posterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass storyInfo::Class;
jfieldID storyInfo::story_id_fieldID;
jfieldID storyInfo::date_fieldID;
jfieldID storyInfo::is_for_close_friends_fieldID;

storyInfo::storyInfo()
  : story_id_()
  , date_()
  , is_for_close_friends_()
{}

storyInfo::storyInfo(int32 story_id_, int32 date_, bool is_for_close_friends_)
  : story_id_(story_id_)
  , date_(date_)
  , is_for_close_friends_(is_for_close_friends_)
{}

const std::int32_t storyInfo::ID;

object_ptr<storyInfo> storyInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInfo> res = make_object<storyInfo>();
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->is_for_close_friends_ = (env->GetBooleanField(p, res->is_for_close_friends_fieldID) != 0);
  return res;
}

void storyInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, story_id_fieldID, story_id_);
  env->SetIntField(s, date_fieldID, date_);
  env->SetBooleanField(s, is_for_close_friends_fieldID, is_for_close_friends_);
}

void storyInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInfo");
    s.store_field("storyId", story_id_);
    s.store_field("date", date_);
    s.store_field("isForCloseFriends", is_for_close_friends_);
    s.store_class_end();
  }
}

void storyInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInfo").c_str());
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    is_for_close_friends_fieldID = jni::get_field_id(env, Class, "isForCloseFriends", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass storyInteraction::Class;
jfieldID storyInteraction::actor_id_fieldID;
jfieldID storyInteraction::interaction_date_fieldID;
jfieldID storyInteraction::block_list_fieldID;
jfieldID storyInteraction::type_fieldID;

storyInteraction::storyInteraction()
  : actor_id_()
  , interaction_date_()
  , block_list_()
  , type_()
{}

storyInteraction::storyInteraction(object_ptr<MessageSender> &&actor_id_, int32 interaction_date_, object_ptr<BlockList> &&block_list_, object_ptr<StoryInteractionType> &&type_)
  : actor_id_(std::move(actor_id_))
  , interaction_date_(interaction_date_)
  , block_list_(std::move(block_list_))
  , type_(std::move(type_))
{}

const std::int32_t storyInteraction::ID;

object_ptr<storyInteraction> storyInteraction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteraction> res = make_object<storyInteraction>();
  res->actor_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->actor_id_fieldID));
  res->interaction_date_ = env->GetIntField(p, res->interaction_date_fieldID);
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  res->type_ = jni::fetch_tl_object<StoryInteractionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void storyInteraction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (actor_id_ != nullptr) { jobject next; actor_id_->store(env, next); if (next) { env->SetObjectField(s, actor_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, interaction_date_fieldID, interaction_date_);
  if (block_list_ != nullptr) { jobject next; block_list_->store(env, next); if (next) { env->SetObjectField(s, block_list_fieldID, next); env->DeleteLocalRef(next); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyInteraction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteraction");
    s.store_object_field("actorId", static_cast<const BaseObject *>(actor_id_.get()));
    s.store_field("interactionDate", interaction_date_);
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void storyInteraction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteraction").c_str());
    actor_id_fieldID = jni::get_field_id(env, Class, "actorId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    interaction_date_fieldID = jni::get_field_id(env, Class, "interactionDate", "I");
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryInteractionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyInteractionInfo::Class;
jfieldID storyInteractionInfo::view_count_fieldID;
jfieldID storyInteractionInfo::forward_count_fieldID;
jfieldID storyInteractionInfo::reaction_count_fieldID;
jfieldID storyInteractionInfo::recent_viewer_user_ids_fieldID;

storyInteractionInfo::storyInteractionInfo()
  : view_count_()
  , forward_count_()
  , reaction_count_()
  , recent_viewer_user_ids_()
{}

storyInteractionInfo::storyInteractionInfo(int32 view_count_, int32 forward_count_, int32 reaction_count_, array<int53> &&recent_viewer_user_ids_)
  : view_count_(view_count_)
  , forward_count_(forward_count_)
  , reaction_count_(reaction_count_)
  , recent_viewer_user_ids_(std::move(recent_viewer_user_ids_))
{}

const std::int32_t storyInteractionInfo::ID;

object_ptr<storyInteractionInfo> storyInteractionInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteractionInfo> res = make_object<storyInteractionInfo>();
  res->view_count_ = env->GetIntField(p, res->view_count_fieldID);
  res->forward_count_ = env->GetIntField(p, res->forward_count_fieldID);
  res->reaction_count_ = env->GetIntField(p, res->reaction_count_fieldID);
  res->recent_viewer_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->recent_viewer_user_ids_fieldID));
  return res;
}

void storyInteractionInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, view_count_fieldID, view_count_);
  env->SetIntField(s, forward_count_fieldID, forward_count_);
  env->SetIntField(s, reaction_count_fieldID, reaction_count_);
  { auto arr_tmp_ = jni::store_vector(env, recent_viewer_user_ids_); if (arr_tmp_) { env->SetObjectField(s, recent_viewer_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storyInteractionInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteractionInfo");
    s.store_field("viewCount", view_count_);
    s.store_field("forwardCount", forward_count_);
    s.store_field("reactionCount", reaction_count_);
    { s.store_vector_begin("recentViewerUserIds", recent_viewer_user_ids_.size()); for (const auto &_value : recent_viewer_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storyInteractionInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractionInfo").c_str());
    view_count_fieldID = jni::get_field_id(env, Class, "viewCount", "I");
    forward_count_fieldID = jni::get_field_id(env, Class, "forwardCount", "I");
    reaction_count_fieldID = jni::get_field_id(env, Class, "reactionCount", "I");
    recent_viewer_user_ids_fieldID = jni::get_field_id(env, Class, "recentViewerUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass StoryInteractionType::Class;

object_ptr<StoryInteractionType> StoryInteractionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyInteractionTypeView::ID:
      return storyInteractionTypeView::fetch(env, p);
    case storyInteractionTypeForward::ID:
      return storyInteractionTypeForward::fetch(env, p);
    case storyInteractionTypeRepost::ID:
      return storyInteractionTypeRepost::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryInteractionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractionType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyInteractionTypeView::Class;
jfieldID storyInteractionTypeView::chosen_reaction_type_fieldID;

storyInteractionTypeView::storyInteractionTypeView()
  : chosen_reaction_type_()
{}

storyInteractionTypeView::storyInteractionTypeView(object_ptr<ReactionType> &&chosen_reaction_type_)
  : chosen_reaction_type_(std::move(chosen_reaction_type_))
{}

const std::int32_t storyInteractionTypeView::ID;

object_ptr<StoryInteractionType> storyInteractionTypeView::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteractionTypeView> res = make_object<storyInteractionTypeView>();
  res->chosen_reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->chosen_reaction_type_fieldID));
  return std::move(res);
}

void storyInteractionTypeView::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chosen_reaction_type_ != nullptr) { jobject next; chosen_reaction_type_->store(env, next); if (next) { env->SetObjectField(s, chosen_reaction_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyInteractionTypeView::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteractionTypeView");
    s.store_object_field("chosenReactionType", static_cast<const BaseObject *>(chosen_reaction_type_.get()));
    s.store_class_end();
  }
}

void storyInteractionTypeView::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractionTypeView").c_str());
    chosen_reaction_type_fieldID = jni::get_field_id(env, Class, "chosenReactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyInteractionTypeForward::Class;
jfieldID storyInteractionTypeForward::message_fieldID;

storyInteractionTypeForward::storyInteractionTypeForward()
  : message_()
{}

storyInteractionTypeForward::storyInteractionTypeForward(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t storyInteractionTypeForward::ID;

object_ptr<StoryInteractionType> storyInteractionTypeForward::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteractionTypeForward> res = make_object<storyInteractionTypeForward>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void storyInteractionTypeForward::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyInteractionTypeForward::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteractionTypeForward");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void storyInteractionTypeForward::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractionTypeForward").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyInteractionTypeRepost::Class;
jfieldID storyInteractionTypeRepost::story_fieldID;

storyInteractionTypeRepost::storyInteractionTypeRepost()
  : story_()
{}

storyInteractionTypeRepost::storyInteractionTypeRepost(object_ptr<story> &&story_)
  : story_(std::move(story_))
{}

const std::int32_t storyInteractionTypeRepost::ID;

object_ptr<StoryInteractionType> storyInteractionTypeRepost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteractionTypeRepost> res = make_object<storyInteractionTypeRepost>();
  res->story_ = jni::fetch_tl_object<story>(env, jni::fetch_object(env, p, res->story_fieldID));
  return std::move(res);
}

void storyInteractionTypeRepost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_ != nullptr) { jobject next; story_->store(env, next); if (next) { env->SetObjectField(s, story_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyInteractionTypeRepost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteractionTypeRepost");
    s.store_object_field("story", static_cast<const BaseObject *>(story_.get()));
    s.store_class_end();
  }
}

void storyInteractionTypeRepost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractionTypeRepost").c_str());
    story_fieldID = jni::get_field_id(env, Class, "story", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyInteractions::Class;
jfieldID storyInteractions::total_count_fieldID;
jfieldID storyInteractions::total_forward_count_fieldID;
jfieldID storyInteractions::total_reaction_count_fieldID;
jfieldID storyInteractions::interactions_fieldID;
jfieldID storyInteractions::next_offset_fieldID;

storyInteractions::storyInteractions()
  : total_count_()
  , total_forward_count_()
  , total_reaction_count_()
  , interactions_()
  , next_offset_()
{}

storyInteractions::storyInteractions(int32 total_count_, int32 total_forward_count_, int32 total_reaction_count_, array<object_ptr<storyInteraction>> &&interactions_, string const &next_offset_)
  : total_count_(total_count_)
  , total_forward_count_(total_forward_count_)
  , total_reaction_count_(total_reaction_count_)
  , interactions_(std::move(interactions_))
  , next_offset_(next_offset_)
{}

const std::int32_t storyInteractions::ID;

object_ptr<storyInteractions> storyInteractions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyInteractions> res = make_object<storyInteractions>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->total_forward_count_ = env->GetIntField(p, res->total_forward_count_fieldID);
  res->total_reaction_count_ = env->GetIntField(p, res->total_reaction_count_fieldID);
  res->interactions_ = jni::FetchVector<storyInteraction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->interactions_fieldID));
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void storyInteractions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetIntField(s, total_forward_count_fieldID, total_forward_count_);
  env->SetIntField(s, total_reaction_count_fieldID, total_reaction_count_);
  { auto arr_tmp_ = jni::store_vector(env, interactions_); if (arr_tmp_) { env->SetObjectField(s, interactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, next_offset_); if (nextString) { env->SetObjectField(s, next_offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void storyInteractions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryInteractions");
    s.store_field("totalCount", total_count_);
    s.store_field("totalForwardCount", total_forward_count_);
    s.store_field("totalReactionCount", total_reaction_count_);
    { s.store_vector_begin("interactions", interactions_.size()); for (const auto &_value : interactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

void storyInteractions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryInteractions").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    total_forward_count_fieldID = jni::get_field_id(env, Class, "totalForwardCount", "I");
    total_reaction_count_fieldID = jni::get_field_id(env, Class, "totalReactionCount", "I");
    interactions_fieldID = jni::get_field_id(env, Class, "interactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StoryInteraction;").c_str());
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass StoryList::Class;

object_ptr<StoryList> StoryList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyListMain::ID:
      return storyListMain::fetch(env, p);
    case storyListArchive::ID:
      return storyListArchive::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryList").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyListMain::Class;

storyListMain::storyListMain() {
}

const std::int32_t storyListMain::ID;

object_ptr<StoryList> storyListMain::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<storyListMain>();
}

void storyListMain::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void storyListMain::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryListMain");
    s.store_class_end();
  }
}

void storyListMain::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryListMain").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyListArchive::Class;

storyListArchive::storyListArchive() {
}

const std::int32_t storyListArchive::ID;

object_ptr<StoryList> storyListArchive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<storyListArchive>();
}

void storyListArchive::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void storyListArchive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryListArchive");
    s.store_class_end();
  }
}

void storyListArchive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryListArchive").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass StoryOrigin::Class;

object_ptr<StoryOrigin> StoryOrigin::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyOriginPublicStory::ID:
      return storyOriginPublicStory::fetch(env, p);
    case storyOriginHiddenUser::ID:
      return storyOriginHiddenUser::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryOrigin::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryOrigin").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyOriginPublicStory::Class;
jfieldID storyOriginPublicStory::chat_id_fieldID;
jfieldID storyOriginPublicStory::story_id_fieldID;

storyOriginPublicStory::storyOriginPublicStory()
  : chat_id_()
  , story_id_()
{}

storyOriginPublicStory::storyOriginPublicStory(int53 chat_id_, int32 story_id_)
  : chat_id_(chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t storyOriginPublicStory::ID;

object_ptr<StoryOrigin> storyOriginPublicStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyOriginPublicStory> res = make_object<storyOriginPublicStory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void storyOriginPublicStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void storyOriginPublicStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryOriginPublicStory");
    s.store_field("chatId", chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void storyOriginPublicStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryOriginPublicStory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass storyOriginHiddenUser::Class;
jfieldID storyOriginHiddenUser::poster_name_fieldID;

storyOriginHiddenUser::storyOriginHiddenUser()
  : poster_name_()
{}

storyOriginHiddenUser::storyOriginHiddenUser(string const &poster_name_)
  : poster_name_(poster_name_)
{}

const std::int32_t storyOriginHiddenUser::ID;

object_ptr<StoryOrigin> storyOriginHiddenUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyOriginHiddenUser> res = make_object<storyOriginHiddenUser>();
  res->poster_name_ = jni::fetch_string(env, p, res->poster_name_fieldID);
  return std::move(res);
}

void storyOriginHiddenUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, poster_name_); if (nextString) { env->SetObjectField(s, poster_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void storyOriginHiddenUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryOriginHiddenUser");
    s.store_field("posterName", poster_name_);
    s.store_class_end();
  }
}

void storyOriginHiddenUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryOriginHiddenUser").c_str());
    poster_name_fieldID = jni::get_field_id(env, Class, "posterName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass StoryPrivacySettings::Class;

object_ptr<StoryPrivacySettings> StoryPrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case storyPrivacySettingsEveryone::ID:
      return storyPrivacySettingsEveryone::fetch(env, p);
    case storyPrivacySettingsContacts::ID:
      return storyPrivacySettingsContacts::fetch(env, p);
    case storyPrivacySettingsCloseFriends::ID:
      return storyPrivacySettingsCloseFriends::fetch(env, p);
    case storyPrivacySettingsSelectedUsers::ID:
      return storyPrivacySettingsSelectedUsers::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void StoryPrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryPrivacySettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyPrivacySettingsEveryone::Class;
jfieldID storyPrivacySettingsEveryone::except_user_ids_fieldID;

storyPrivacySettingsEveryone::storyPrivacySettingsEveryone()
  : except_user_ids_()
{}

storyPrivacySettingsEveryone::storyPrivacySettingsEveryone(array<int53> &&except_user_ids_)
  : except_user_ids_(std::move(except_user_ids_))
{}

const std::int32_t storyPrivacySettingsEveryone::ID;

object_ptr<StoryPrivacySettings> storyPrivacySettingsEveryone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyPrivacySettingsEveryone> res = make_object<storyPrivacySettingsEveryone>();
  res->except_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->except_user_ids_fieldID));
  return std::move(res);
}

void storyPrivacySettingsEveryone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, except_user_ids_); if (arr_tmp_) { env->SetObjectField(s, except_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storyPrivacySettingsEveryone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryPrivacySettingsEveryone");
    { s.store_vector_begin("exceptUserIds", except_user_ids_.size()); for (const auto &_value : except_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storyPrivacySettingsEveryone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryPrivacySettingsEveryone").c_str());
    except_user_ids_fieldID = jni::get_field_id(env, Class, "exceptUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass storyPrivacySettingsContacts::Class;
jfieldID storyPrivacySettingsContacts::except_user_ids_fieldID;

storyPrivacySettingsContacts::storyPrivacySettingsContacts()
  : except_user_ids_()
{}

storyPrivacySettingsContacts::storyPrivacySettingsContacts(array<int53> &&except_user_ids_)
  : except_user_ids_(std::move(except_user_ids_))
{}

const std::int32_t storyPrivacySettingsContacts::ID;

object_ptr<StoryPrivacySettings> storyPrivacySettingsContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyPrivacySettingsContacts> res = make_object<storyPrivacySettingsContacts>();
  res->except_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->except_user_ids_fieldID));
  return std::move(res);
}

void storyPrivacySettingsContacts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, except_user_ids_); if (arr_tmp_) { env->SetObjectField(s, except_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storyPrivacySettingsContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryPrivacySettingsContacts");
    { s.store_vector_begin("exceptUserIds", except_user_ids_.size()); for (const auto &_value : except_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storyPrivacySettingsContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryPrivacySettingsContacts").c_str());
    except_user_ids_fieldID = jni::get_field_id(env, Class, "exceptUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass storyPrivacySettingsCloseFriends::Class;

storyPrivacySettingsCloseFriends::storyPrivacySettingsCloseFriends() {
}

const std::int32_t storyPrivacySettingsCloseFriends::ID;

object_ptr<StoryPrivacySettings> storyPrivacySettingsCloseFriends::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<storyPrivacySettingsCloseFriends>();
}

void storyPrivacySettingsCloseFriends::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void storyPrivacySettingsCloseFriends::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryPrivacySettingsCloseFriends");
    s.store_class_end();
  }
}

void storyPrivacySettingsCloseFriends::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryPrivacySettingsCloseFriends").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyPrivacySettingsSelectedUsers::Class;
jfieldID storyPrivacySettingsSelectedUsers::user_ids_fieldID;

storyPrivacySettingsSelectedUsers::storyPrivacySettingsSelectedUsers()
  : user_ids_()
{}

storyPrivacySettingsSelectedUsers::storyPrivacySettingsSelectedUsers(array<int53> &&user_ids_)
  : user_ids_(std::move(user_ids_))
{}

const std::int32_t storyPrivacySettingsSelectedUsers::ID;

object_ptr<StoryPrivacySettings> storyPrivacySettingsSelectedUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyPrivacySettingsSelectedUsers> res = make_object<storyPrivacySettingsSelectedUsers>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return std::move(res);
}

void storyPrivacySettingsSelectedUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void storyPrivacySettingsSelectedUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryPrivacySettingsSelectedUsers");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void storyPrivacySettingsSelectedUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryPrivacySettingsSelectedUsers").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass storyRepostInfo::Class;
jfieldID storyRepostInfo::origin_fieldID;
jfieldID storyRepostInfo::is_content_modified_fieldID;

storyRepostInfo::storyRepostInfo()
  : origin_()
  , is_content_modified_()
{}

storyRepostInfo::storyRepostInfo(object_ptr<StoryOrigin> &&origin_, bool is_content_modified_)
  : origin_(std::move(origin_))
  , is_content_modified_(is_content_modified_)
{}

const std::int32_t storyRepostInfo::ID;

object_ptr<storyRepostInfo> storyRepostInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyRepostInfo> res = make_object<storyRepostInfo>();
  res->origin_ = jni::fetch_tl_object<StoryOrigin>(env, jni::fetch_object(env, p, res->origin_fieldID));
  res->is_content_modified_ = (env->GetBooleanField(p, res->is_content_modified_fieldID) != 0);
  return res;
}

void storyRepostInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (origin_ != nullptr) { jobject next; origin_->store(env, next); if (next) { env->SetObjectField(s, origin_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_content_modified_fieldID, is_content_modified_);
}

void storyRepostInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryRepostInfo");
    s.store_object_field("origin", static_cast<const BaseObject *>(origin_.get()));
    s.store_field("isContentModified", is_content_modified_);
    s.store_class_end();
  }
}

void storyRepostInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryRepostInfo").c_str());
    origin_fieldID = jni::get_field_id(env, Class, "origin", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryOrigin;").c_str());
    is_content_modified_fieldID = jni::get_field_id(env, Class, "isContentModified", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass storyStatistics::Class;
jfieldID storyStatistics::story_interaction_graph_fieldID;
jfieldID storyStatistics::story_reaction_graph_fieldID;

storyStatistics::storyStatistics()
  : story_interaction_graph_()
  , story_reaction_graph_()
{}

storyStatistics::storyStatistics(object_ptr<StatisticalGraph> &&story_interaction_graph_, object_ptr<StatisticalGraph> &&story_reaction_graph_)
  : story_interaction_graph_(std::move(story_interaction_graph_))
  , story_reaction_graph_(std::move(story_reaction_graph_))
{}

const std::int32_t storyStatistics::ID;

object_ptr<storyStatistics> storyStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyStatistics> res = make_object<storyStatistics>();
  res->story_interaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->story_interaction_graph_fieldID));
  res->story_reaction_graph_ = jni::fetch_tl_object<StatisticalGraph>(env, jni::fetch_object(env, p, res->story_reaction_graph_fieldID));
  return res;
}

void storyStatistics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_interaction_graph_ != nullptr) { jobject next; story_interaction_graph_->store(env, next); if (next) { env->SetObjectField(s, story_interaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
  if (story_reaction_graph_ != nullptr) { jobject next; story_reaction_graph_->store(env, next); if (next) { env->SetObjectField(s, story_reaction_graph_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryStatistics");
    s.store_object_field("storyInteractionGraph", static_cast<const BaseObject *>(story_interaction_graph_.get()));
    s.store_object_field("storyReactionGraph", static_cast<const BaseObject *>(story_reaction_graph_.get()));
    s.store_class_end();
  }
}

void storyStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryStatistics").c_str());
    story_interaction_graph_fieldID = jni::get_field_id(env, Class, "storyInteractionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    story_reaction_graph_fieldID = jni::get_field_id(env, Class, "storyReactionGraph", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StatisticalGraph;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass storyVideo::Class;
jfieldID storyVideo::duration_fieldID;
jfieldID storyVideo::width_fieldID;
jfieldID storyVideo::height_fieldID;
jfieldID storyVideo::has_stickers_fieldID;
jfieldID storyVideo::is_animation_fieldID;
jfieldID storyVideo::minithumbnail_fieldID;
jfieldID storyVideo::thumbnail_fieldID;
jfieldID storyVideo::preload_prefix_size_fieldID;
jfieldID storyVideo::cover_frame_timestamp_fieldID;
jfieldID storyVideo::video_fieldID;

storyVideo::storyVideo()
  : duration_()
  , width_()
  , height_()
  , has_stickers_()
  , is_animation_()
  , minithumbnail_()
  , thumbnail_()
  , preload_prefix_size_()
  , cover_frame_timestamp_()
  , video_()
{}

storyVideo::storyVideo(double duration_, int32 width_, int32 height_, bool has_stickers_, bool is_animation_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, int32 preload_prefix_size_, double cover_frame_timestamp_, object_ptr<file> &&video_)
  : duration_(duration_)
  , width_(width_)
  , height_(height_)
  , has_stickers_(has_stickers_)
  , is_animation_(is_animation_)
  , minithumbnail_(std::move(minithumbnail_))
  , thumbnail_(std::move(thumbnail_))
  , preload_prefix_size_(preload_prefix_size_)
  , cover_frame_timestamp_(cover_frame_timestamp_)
  , video_(std::move(video_))
{}

const std::int32_t storyVideo::ID;

object_ptr<storyVideo> storyVideo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<storyVideo> res = make_object<storyVideo>();
  res->duration_ = env->GetDoubleField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->has_stickers_ = (env->GetBooleanField(p, res->has_stickers_fieldID) != 0);
  res->is_animation_ = (env->GetBooleanField(p, res->is_animation_fieldID) != 0);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->preload_prefix_size_ = env->GetIntField(p, res->preload_prefix_size_fieldID);
  res->cover_frame_timestamp_ = env->GetDoubleField(p, res->cover_frame_timestamp_fieldID);
  res->video_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->video_fieldID));
  return res;
}

void storyVideo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetDoubleField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  env->SetBooleanField(s, has_stickers_fieldID, has_stickers_);
  env->SetBooleanField(s, is_animation_fieldID, is_animation_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, preload_prefix_size_fieldID, preload_prefix_size_);
  env->SetDoubleField(s, cover_frame_timestamp_fieldID, cover_frame_timestamp_);
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
}

void storyVideo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StoryVideo");
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("hasStickers", has_stickers_);
    s.store_field("isAnimation", is_animation_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_field("preloadPrefixSize", preload_prefix_size_);
    s.store_field("coverFrameTimestamp", cover_frame_timestamp_);
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_class_end();
  }
}

void storyVideo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StoryVideo").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "D");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    has_stickers_fieldID = jni::get_field_id(env, Class, "hasStickers", "Z");
    is_animation_fieldID = jni::get_field_id(env, Class, "isAnimation", "Z");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    preload_prefix_size_fieldID = jni::get_field_id(env, Class, "preloadPrefixSize", "I");
    cover_frame_timestamp_fieldID = jni::get_field_id(env, Class, "coverFrameTimestamp", "D");
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass SuggestedAction::Class;

object_ptr<SuggestedAction> SuggestedAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case suggestedActionEnableArchiveAndMuteNewChats::ID:
      return suggestedActionEnableArchiveAndMuteNewChats::fetch(env, p);
    case suggestedActionCheckPassword::ID:
      return suggestedActionCheckPassword::fetch(env, p);
    case suggestedActionCheckPhoneNumber::ID:
      return suggestedActionCheckPhoneNumber::fetch(env, p);
    case suggestedActionViewChecksHint::ID:
      return suggestedActionViewChecksHint::fetch(env, p);
    case suggestedActionConvertToBroadcastGroup::ID:
      return suggestedActionConvertToBroadcastGroup::fetch(env, p);
    case suggestedActionSetPassword::ID:
      return suggestedActionSetPassword::fetch(env, p);
    case suggestedActionUpgradePremium::ID:
      return suggestedActionUpgradePremium::fetch(env, p);
    case suggestedActionRestorePremium::ID:
      return suggestedActionRestorePremium::fetch(env, p);
    case suggestedActionSubscribeToAnnualPremium::ID:
      return suggestedActionSubscribeToAnnualPremium::fetch(env, p);
    case suggestedActionGiftPremiumForChristmas::ID:
      return suggestedActionGiftPremiumForChristmas::fetch(env, p);
    case suggestedActionSetBirthdate::ID:
      return suggestedActionSetBirthdate::fetch(env, p);
    case suggestedActionSetProfilePhoto::ID:
      return suggestedActionSetProfilePhoto::fetch(env, p);
    case suggestedActionExtendPremium::ID:
      return suggestedActionExtendPremium::fetch(env, p);
    case suggestedActionExtendStarSubscriptions::ID:
      return suggestedActionExtendStarSubscriptions::fetch(env, p);
    case suggestedActionCustom::ID:
      return suggestedActionCustom::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SuggestedAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedAction").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionEnableArchiveAndMuteNewChats::Class;

suggestedActionEnableArchiveAndMuteNewChats::suggestedActionEnableArchiveAndMuteNewChats() {
}

const std::int32_t suggestedActionEnableArchiveAndMuteNewChats::ID;

object_ptr<SuggestedAction> suggestedActionEnableArchiveAndMuteNewChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionEnableArchiveAndMuteNewChats>();
}

void suggestedActionEnableArchiveAndMuteNewChats::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionEnableArchiveAndMuteNewChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionEnableArchiveAndMuteNewChats");
    s.store_class_end();
  }
}

void suggestedActionEnableArchiveAndMuteNewChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionEnableArchiveAndMuteNewChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionCheckPassword::Class;

suggestedActionCheckPassword::suggestedActionCheckPassword() {
}

const std::int32_t suggestedActionCheckPassword::ID;

object_ptr<SuggestedAction> suggestedActionCheckPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionCheckPassword>();
}

void suggestedActionCheckPassword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionCheckPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionCheckPassword");
    s.store_class_end();
  }
}

void suggestedActionCheckPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionCheckPassword").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionCheckPhoneNumber::Class;

suggestedActionCheckPhoneNumber::suggestedActionCheckPhoneNumber() {
}

const std::int32_t suggestedActionCheckPhoneNumber::ID;

object_ptr<SuggestedAction> suggestedActionCheckPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionCheckPhoneNumber>();
}

void suggestedActionCheckPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionCheckPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionCheckPhoneNumber");
    s.store_class_end();
  }
}

void suggestedActionCheckPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionCheckPhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionViewChecksHint::Class;

suggestedActionViewChecksHint::suggestedActionViewChecksHint() {
}

const std::int32_t suggestedActionViewChecksHint::ID;

object_ptr<SuggestedAction> suggestedActionViewChecksHint::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionViewChecksHint>();
}

void suggestedActionViewChecksHint::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionViewChecksHint::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionViewChecksHint");
    s.store_class_end();
  }
}

void suggestedActionViewChecksHint::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionViewChecksHint").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionConvertToBroadcastGroup::Class;
jfieldID suggestedActionConvertToBroadcastGroup::supergroup_id_fieldID;

suggestedActionConvertToBroadcastGroup::suggestedActionConvertToBroadcastGroup()
  : supergroup_id_()
{}

suggestedActionConvertToBroadcastGroup::suggestedActionConvertToBroadcastGroup(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t suggestedActionConvertToBroadcastGroup::ID;

object_ptr<SuggestedAction> suggestedActionConvertToBroadcastGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<suggestedActionConvertToBroadcastGroup> res = make_object<suggestedActionConvertToBroadcastGroup>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return std::move(res);
}

void suggestedActionConvertToBroadcastGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, supergroup_id_fieldID, supergroup_id_);
}

void suggestedActionConvertToBroadcastGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionConvertToBroadcastGroup");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

void suggestedActionConvertToBroadcastGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionConvertToBroadcastGroup").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionSetPassword::Class;
jfieldID suggestedActionSetPassword::authorization_delay_fieldID;

suggestedActionSetPassword::suggestedActionSetPassword()
  : authorization_delay_()
{}

suggestedActionSetPassword::suggestedActionSetPassword(int32 authorization_delay_)
  : authorization_delay_(authorization_delay_)
{}

const std::int32_t suggestedActionSetPassword::ID;

object_ptr<SuggestedAction> suggestedActionSetPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<suggestedActionSetPassword> res = make_object<suggestedActionSetPassword>();
  res->authorization_delay_ = env->GetIntField(p, res->authorization_delay_fieldID);
  return std::move(res);
}

void suggestedActionSetPassword::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, authorization_delay_fieldID, authorization_delay_);
}

void suggestedActionSetPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionSetPassword");
    s.store_field("authorizationDelay", authorization_delay_);
    s.store_class_end();
  }
}

void suggestedActionSetPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionSetPassword").c_str());
    authorization_delay_fieldID = jni::get_field_id(env, Class, "authorizationDelay", "I");
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionUpgradePremium::Class;

suggestedActionUpgradePremium::suggestedActionUpgradePremium() {
}

const std::int32_t suggestedActionUpgradePremium::ID;

object_ptr<SuggestedAction> suggestedActionUpgradePremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionUpgradePremium>();
}

void suggestedActionUpgradePremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionUpgradePremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionUpgradePremium");
    s.store_class_end();
  }
}

void suggestedActionUpgradePremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionUpgradePremium").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionRestorePremium::Class;

suggestedActionRestorePremium::suggestedActionRestorePremium() {
}

const std::int32_t suggestedActionRestorePremium::ID;

object_ptr<SuggestedAction> suggestedActionRestorePremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionRestorePremium>();
}

void suggestedActionRestorePremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionRestorePremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionRestorePremium");
    s.store_class_end();
  }
}

void suggestedActionRestorePremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionRestorePremium").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionSubscribeToAnnualPremium::Class;

suggestedActionSubscribeToAnnualPremium::suggestedActionSubscribeToAnnualPremium() {
}

const std::int32_t suggestedActionSubscribeToAnnualPremium::ID;

object_ptr<SuggestedAction> suggestedActionSubscribeToAnnualPremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionSubscribeToAnnualPremium>();
}

void suggestedActionSubscribeToAnnualPremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionSubscribeToAnnualPremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionSubscribeToAnnualPremium");
    s.store_class_end();
  }
}

void suggestedActionSubscribeToAnnualPremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionSubscribeToAnnualPremium").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionGiftPremiumForChristmas::Class;

suggestedActionGiftPremiumForChristmas::suggestedActionGiftPremiumForChristmas() {
}

const std::int32_t suggestedActionGiftPremiumForChristmas::ID;

object_ptr<SuggestedAction> suggestedActionGiftPremiumForChristmas::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionGiftPremiumForChristmas>();
}

void suggestedActionGiftPremiumForChristmas::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionGiftPremiumForChristmas::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionGiftPremiumForChristmas");
    s.store_class_end();
  }
}

void suggestedActionGiftPremiumForChristmas::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionGiftPremiumForChristmas").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionSetBirthdate::Class;

suggestedActionSetBirthdate::suggestedActionSetBirthdate() {
}

const std::int32_t suggestedActionSetBirthdate::ID;

object_ptr<SuggestedAction> suggestedActionSetBirthdate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionSetBirthdate>();
}

void suggestedActionSetBirthdate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionSetBirthdate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionSetBirthdate");
    s.store_class_end();
  }
}

void suggestedActionSetBirthdate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionSetBirthdate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionSetProfilePhoto::Class;

suggestedActionSetProfilePhoto::suggestedActionSetProfilePhoto() {
}

const std::int32_t suggestedActionSetProfilePhoto::ID;

object_ptr<SuggestedAction> suggestedActionSetProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionSetProfilePhoto>();
}

void suggestedActionSetProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionSetProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionSetProfilePhoto");
    s.store_class_end();
  }
}

void suggestedActionSetProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionSetProfilePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionExtendPremium::Class;
jfieldID suggestedActionExtendPremium::manage_premium_subscription_url_fieldID;

suggestedActionExtendPremium::suggestedActionExtendPremium()
  : manage_premium_subscription_url_()
{}

suggestedActionExtendPremium::suggestedActionExtendPremium(string const &manage_premium_subscription_url_)
  : manage_premium_subscription_url_(manage_premium_subscription_url_)
{}

const std::int32_t suggestedActionExtendPremium::ID;

object_ptr<SuggestedAction> suggestedActionExtendPremium::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<suggestedActionExtendPremium> res = make_object<suggestedActionExtendPremium>();
  res->manage_premium_subscription_url_ = jni::fetch_string(env, p, res->manage_premium_subscription_url_fieldID);
  return std::move(res);
}

void suggestedActionExtendPremium::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, manage_premium_subscription_url_); if (nextString) { env->SetObjectField(s, manage_premium_subscription_url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void suggestedActionExtendPremium::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionExtendPremium");
    s.store_field("managePremiumSubscriptionUrl", manage_premium_subscription_url_);
    s.store_class_end();
  }
}

void suggestedActionExtendPremium::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionExtendPremium").c_str());
    manage_premium_subscription_url_fieldID = jni::get_field_id(env, Class, "managePremiumSubscriptionUrl", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionExtendStarSubscriptions::Class;

suggestedActionExtendStarSubscriptions::suggestedActionExtendStarSubscriptions() {
}

const std::int32_t suggestedActionExtendStarSubscriptions::ID;

object_ptr<SuggestedAction> suggestedActionExtendStarSubscriptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<suggestedActionExtendStarSubscriptions>();
}

void suggestedActionExtendStarSubscriptions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void suggestedActionExtendStarSubscriptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionExtendStarSubscriptions");
    s.store_class_end();
  }
}

void suggestedActionExtendStarSubscriptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionExtendStarSubscriptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestedActionCustom::Class;
jfieldID suggestedActionCustom::name_fieldID;
jfieldID suggestedActionCustom::title_fieldID;
jfieldID suggestedActionCustom::description_fieldID;
jfieldID suggestedActionCustom::url_fieldID;

suggestedActionCustom::suggestedActionCustom()
  : name_()
  , title_()
  , description_()
  , url_()
{}

suggestedActionCustom::suggestedActionCustom(string const &name_, object_ptr<formattedText> &&title_, object_ptr<formattedText> &&description_, string const &url_)
  : name_(name_)
  , title_(std::move(title_))
  , description_(std::move(description_))
  , url_(url_)
{}

const std::int32_t suggestedActionCustom::ID;

object_ptr<SuggestedAction> suggestedActionCustom::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<suggestedActionCustom> res = make_object<suggestedActionCustom>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->title_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->title_fieldID));
  res->description_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->description_fieldID));
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void suggestedActionCustom::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (title_ != nullptr) { jobject next; title_->store(env, next); if (next) { env->SetObjectField(s, title_fieldID, next); env->DeleteLocalRef(next); } }
  if (description_ != nullptr) { jobject next; description_->store(env, next); if (next) { env->SetObjectField(s, description_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void suggestedActionCustom::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestedActionCustom");
    s.store_field("name", name_);
    s.store_object_field("title", static_cast<const BaseObject *>(title_.get()));
    s.store_object_field("description", static_cast<const BaseObject *>(description_.get()));
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void suggestedActionCustom::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestedActionCustom").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    description_fieldID = jni::get_field_id(env, Class, "description", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass supergroup::Class;
jfieldID supergroup::id_fieldID;
jfieldID supergroup::usernames_fieldID;
jfieldID supergroup::date_fieldID;
jfieldID supergroup::status_fieldID;
jfieldID supergroup::member_count_fieldID;
jfieldID supergroup::boost_level_fieldID;
jfieldID supergroup::has_automatic_translation_fieldID;
jfieldID supergroup::has_linked_chat_fieldID;
jfieldID supergroup::has_location_fieldID;
jfieldID supergroup::sign_messages_fieldID;
jfieldID supergroup::show_message_sender_fieldID;
jfieldID supergroup::join_to_send_messages_fieldID;
jfieldID supergroup::join_by_request_fieldID;
jfieldID supergroup::is_slow_mode_enabled_fieldID;
jfieldID supergroup::is_channel_fieldID;
jfieldID supergroup::is_broadcast_group_fieldID;
jfieldID supergroup::is_forum_fieldID;
jfieldID supergroup::verification_status_fieldID;
jfieldID supergroup::has_sensitive_content_fieldID;
jfieldID supergroup::restriction_reason_fieldID;
jfieldID supergroup::paid_message_star_count_fieldID;
jfieldID supergroup::has_active_stories_fieldID;
jfieldID supergroup::has_unread_active_stories_fieldID;

supergroup::supergroup()
  : id_()
  , usernames_()
  , date_()
  , status_()
  , member_count_()
  , boost_level_()
  , has_automatic_translation_()
  , has_linked_chat_()
  , has_location_()
  , sign_messages_()
  , show_message_sender_()
  , join_to_send_messages_()
  , join_by_request_()
  , is_slow_mode_enabled_()
  , is_channel_()
  , is_broadcast_group_()
  , is_forum_()
  , verification_status_()
  , has_sensitive_content_()
  , restriction_reason_()
  , paid_message_star_count_()
  , has_active_stories_()
  , has_unread_active_stories_()
{}

supergroup::supergroup(int53 id_, object_ptr<usernames> &&usernames_, int32 date_, object_ptr<ChatMemberStatus> &&status_, int32 member_count_, int32 boost_level_, bool has_automatic_translation_, bool has_linked_chat_, bool has_location_, bool sign_messages_, bool show_message_sender_, bool join_to_send_messages_, bool join_by_request_, bool is_slow_mode_enabled_, bool is_channel_, bool is_broadcast_group_, bool is_forum_, object_ptr<verificationStatus> &&verification_status_, bool has_sensitive_content_, string const &restriction_reason_, int53 paid_message_star_count_, bool has_active_stories_, bool has_unread_active_stories_)
  : id_(id_)
  , usernames_(std::move(usernames_))
  , date_(date_)
  , status_(std::move(status_))
  , member_count_(member_count_)
  , boost_level_(boost_level_)
  , has_automatic_translation_(has_automatic_translation_)
  , has_linked_chat_(has_linked_chat_)
  , has_location_(has_location_)
  , sign_messages_(sign_messages_)
  , show_message_sender_(show_message_sender_)
  , join_to_send_messages_(join_to_send_messages_)
  , join_by_request_(join_by_request_)
  , is_slow_mode_enabled_(is_slow_mode_enabled_)
  , is_channel_(is_channel_)
  , is_broadcast_group_(is_broadcast_group_)
  , is_forum_(is_forum_)
  , verification_status_(std::move(verification_status_))
  , has_sensitive_content_(has_sensitive_content_)
  , restriction_reason_(restriction_reason_)
  , paid_message_star_count_(paid_message_star_count_)
  , has_active_stories_(has_active_stories_)
  , has_unread_active_stories_(has_unread_active_stories_)
{}

const std::int32_t supergroup::ID;

object_ptr<supergroup> supergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroup> res = make_object<supergroup>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->usernames_ = jni::fetch_tl_object<usernames>(env, jni::fetch_object(env, p, res->usernames_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  res->member_count_ = env->GetIntField(p, res->member_count_fieldID);
  res->boost_level_ = env->GetIntField(p, res->boost_level_fieldID);
  res->has_automatic_translation_ = (env->GetBooleanField(p, res->has_automatic_translation_fieldID) != 0);
  res->has_linked_chat_ = (env->GetBooleanField(p, res->has_linked_chat_fieldID) != 0);
  res->has_location_ = (env->GetBooleanField(p, res->has_location_fieldID) != 0);
  res->sign_messages_ = (env->GetBooleanField(p, res->sign_messages_fieldID) != 0);
  res->show_message_sender_ = (env->GetBooleanField(p, res->show_message_sender_fieldID) != 0);
  res->join_to_send_messages_ = (env->GetBooleanField(p, res->join_to_send_messages_fieldID) != 0);
  res->join_by_request_ = (env->GetBooleanField(p, res->join_by_request_fieldID) != 0);
  res->is_slow_mode_enabled_ = (env->GetBooleanField(p, res->is_slow_mode_enabled_fieldID) != 0);
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  res->is_broadcast_group_ = (env->GetBooleanField(p, res->is_broadcast_group_fieldID) != 0);
  res->is_forum_ = (env->GetBooleanField(p, res->is_forum_fieldID) != 0);
  res->verification_status_ = jni::fetch_tl_object<verificationStatus>(env, jni::fetch_object(env, p, res->verification_status_fieldID));
  res->has_sensitive_content_ = (env->GetBooleanField(p, res->has_sensitive_content_fieldID) != 0);
  res->restriction_reason_ = jni::fetch_string(env, p, res->restriction_reason_fieldID);
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  res->has_active_stories_ = (env->GetBooleanField(p, res->has_active_stories_fieldID) != 0);
  res->has_unread_active_stories_ = (env->GetBooleanField(p, res->has_unread_active_stories_fieldID) != 0);
  return res;
}

void supergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  if (usernames_ != nullptr) { jobject next; usernames_->store(env, next); if (next) { env->SetObjectField(s, usernames_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, member_count_fieldID, member_count_);
  env->SetIntField(s, boost_level_fieldID, boost_level_);
  env->SetBooleanField(s, has_automatic_translation_fieldID, has_automatic_translation_);
  env->SetBooleanField(s, has_linked_chat_fieldID, has_linked_chat_);
  env->SetBooleanField(s, has_location_fieldID, has_location_);
  env->SetBooleanField(s, sign_messages_fieldID, sign_messages_);
  env->SetBooleanField(s, show_message_sender_fieldID, show_message_sender_);
  env->SetBooleanField(s, join_to_send_messages_fieldID, join_to_send_messages_);
  env->SetBooleanField(s, join_by_request_fieldID, join_by_request_);
  env->SetBooleanField(s, is_slow_mode_enabled_fieldID, is_slow_mode_enabled_);
  env->SetBooleanField(s, is_channel_fieldID, is_channel_);
  env->SetBooleanField(s, is_broadcast_group_fieldID, is_broadcast_group_);
  env->SetBooleanField(s, is_forum_fieldID, is_forum_);
  if (verification_status_ != nullptr) { jobject next; verification_status_->store(env, next); if (next) { env->SetObjectField(s, verification_status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, has_sensitive_content_fieldID, has_sensitive_content_);
  { jstring nextString = jni::to_jstring(env, restriction_reason_); if (nextString) { env->SetObjectField(s, restriction_reason_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, paid_message_star_count_fieldID, paid_message_star_count_);
  env->SetBooleanField(s, has_active_stories_fieldID, has_active_stories_);
  env->SetBooleanField(s, has_unread_active_stories_fieldID, has_unread_active_stories_);
}

void supergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Supergroup");
    s.store_field("id", id_);
    s.store_object_field("usernames", static_cast<const BaseObject *>(usernames_.get()));
    s.store_field("date", date_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_field("memberCount", member_count_);
    s.store_field("boostLevel", boost_level_);
    s.store_field("hasAutomaticTranslation", has_automatic_translation_);
    s.store_field("hasLinkedChat", has_linked_chat_);
    s.store_field("hasLocation", has_location_);
    s.store_field("signMessages", sign_messages_);
    s.store_field("showMessageSender", show_message_sender_);
    s.store_field("joinToSendMessages", join_to_send_messages_);
    s.store_field("joinByRequest", join_by_request_);
    s.store_field("isSlowModeEnabled", is_slow_mode_enabled_);
    s.store_field("isChannel", is_channel_);
    s.store_field("isBroadcastGroup", is_broadcast_group_);
    s.store_field("isForum", is_forum_);
    s.store_object_field("verificationStatus", static_cast<const BaseObject *>(verification_status_.get()));
    s.store_field("hasSensitiveContent", has_sensitive_content_);
    s.store_field("restrictionReason", restriction_reason_);
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_field("hasActiveStories", has_active_stories_);
    s.store_field("hasUnreadActiveStories", has_unread_active_stories_);
    s.store_class_end();
  }
}

void supergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Supergroup").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    usernames_fieldID = jni::get_field_id(env, Class, "usernames", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Usernames;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", "I");
    boost_level_fieldID = jni::get_field_id(env, Class, "boostLevel", "I");
    has_automatic_translation_fieldID = jni::get_field_id(env, Class, "hasAutomaticTranslation", "Z");
    has_linked_chat_fieldID = jni::get_field_id(env, Class, "hasLinkedChat", "Z");
    has_location_fieldID = jni::get_field_id(env, Class, "hasLocation", "Z");
    sign_messages_fieldID = jni::get_field_id(env, Class, "signMessages", "Z");
    show_message_sender_fieldID = jni::get_field_id(env, Class, "showMessageSender", "Z");
    join_to_send_messages_fieldID = jni::get_field_id(env, Class, "joinToSendMessages", "Z");
    join_by_request_fieldID = jni::get_field_id(env, Class, "joinByRequest", "Z");
    is_slow_mode_enabled_fieldID = jni::get_field_id(env, Class, "isSlowModeEnabled", "Z");
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    is_broadcast_group_fieldID = jni::get_field_id(env, Class, "isBroadcastGroup", "Z");
    is_forum_fieldID = jni::get_field_id(env, Class, "isForum", "Z");
    verification_status_fieldID = jni::get_field_id(env, Class, "verificationStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VerificationStatus;").c_str());
    has_sensitive_content_fieldID = jni::get_field_id(env, Class, "hasSensitiveContent", "Z");
    restriction_reason_fieldID = jni::get_field_id(env, Class, "restrictionReason", "Ljava/lang/String;");
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    has_active_stories_fieldID = jni::get_field_id(env, Class, "hasActiveStories", "Z");
    has_unread_active_stories_fieldID = jni::get_field_id(env, Class, "hasUnreadActiveStories", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupFullInfo::Class;
jfieldID supergroupFullInfo::photo_fieldID;
jfieldID supergroupFullInfo::description_fieldID;
jfieldID supergroupFullInfo::member_count_fieldID;
jfieldID supergroupFullInfo::administrator_count_fieldID;
jfieldID supergroupFullInfo::restricted_count_fieldID;
jfieldID supergroupFullInfo::banned_count_fieldID;
jfieldID supergroupFullInfo::linked_chat_id_fieldID;
jfieldID supergroupFullInfo::slow_mode_delay_fieldID;
jfieldID supergroupFullInfo::slow_mode_delay_expires_in_fieldID;
jfieldID supergroupFullInfo::can_enable_paid_messages_fieldID;
jfieldID supergroupFullInfo::can_enable_paid_reaction_fieldID;
jfieldID supergroupFullInfo::can_get_members_fieldID;
jfieldID supergroupFullInfo::has_hidden_members_fieldID;
jfieldID supergroupFullInfo::can_hide_members_fieldID;
jfieldID supergroupFullInfo::can_set_sticker_set_fieldID;
jfieldID supergroupFullInfo::can_set_location_fieldID;
jfieldID supergroupFullInfo::can_get_statistics_fieldID;
jfieldID supergroupFullInfo::can_get_revenue_statistics_fieldID;
jfieldID supergroupFullInfo::can_get_star_revenue_statistics_fieldID;
jfieldID supergroupFullInfo::can_send_gift_fieldID;
jfieldID supergroupFullInfo::can_toggle_aggressive_anti_spam_fieldID;
jfieldID supergroupFullInfo::is_all_history_available_fieldID;
jfieldID supergroupFullInfo::can_have_sponsored_messages_fieldID;
jfieldID supergroupFullInfo::has_aggressive_anti_spam_enabled_fieldID;
jfieldID supergroupFullInfo::has_paid_media_allowed_fieldID;
jfieldID supergroupFullInfo::has_pinned_stories_fieldID;
jfieldID supergroupFullInfo::gift_count_fieldID;
jfieldID supergroupFullInfo::my_boost_count_fieldID;
jfieldID supergroupFullInfo::unrestrict_boost_count_fieldID;
jfieldID supergroupFullInfo::sticker_set_id_fieldID;
jfieldID supergroupFullInfo::custom_emoji_sticker_set_id_fieldID;
jfieldID supergroupFullInfo::location_fieldID;
jfieldID supergroupFullInfo::invite_link_fieldID;
jfieldID supergroupFullInfo::bot_commands_fieldID;
jfieldID supergroupFullInfo::bot_verification_fieldID;
jfieldID supergroupFullInfo::upgraded_from_basic_group_id_fieldID;
jfieldID supergroupFullInfo::upgraded_from_max_message_id_fieldID;

supergroupFullInfo::supergroupFullInfo()
  : photo_()
  , description_()
  , member_count_()
  , administrator_count_()
  , restricted_count_()
  , banned_count_()
  , linked_chat_id_()
  , slow_mode_delay_()
  , slow_mode_delay_expires_in_()
  , can_enable_paid_messages_()
  , can_enable_paid_reaction_()
  , can_get_members_()
  , has_hidden_members_()
  , can_hide_members_()
  , can_set_sticker_set_()
  , can_set_location_()
  , can_get_statistics_()
  , can_get_revenue_statistics_()
  , can_get_star_revenue_statistics_()
  , can_send_gift_()
  , can_toggle_aggressive_anti_spam_()
  , is_all_history_available_()
  , can_have_sponsored_messages_()
  , has_aggressive_anti_spam_enabled_()
  , has_paid_media_allowed_()
  , has_pinned_stories_()
  , gift_count_()
  , my_boost_count_()
  , unrestrict_boost_count_()
  , sticker_set_id_()
  , custom_emoji_sticker_set_id_()
  , location_()
  , invite_link_()
  , bot_commands_()
  , bot_verification_()
  , upgraded_from_basic_group_id_()
  , upgraded_from_max_message_id_()
{}

supergroupFullInfo::supergroupFullInfo(object_ptr<chatPhoto> &&photo_, string const &description_, int32 member_count_, int32 administrator_count_, int32 restricted_count_, int32 banned_count_, int53 linked_chat_id_, int32 slow_mode_delay_, double slow_mode_delay_expires_in_, bool can_enable_paid_messages_, bool can_enable_paid_reaction_, bool can_get_members_, bool has_hidden_members_, bool can_hide_members_, bool can_set_sticker_set_, bool can_set_location_, bool can_get_statistics_, bool can_get_revenue_statistics_, bool can_get_star_revenue_statistics_, bool can_send_gift_, bool can_toggle_aggressive_anti_spam_, bool is_all_history_available_, bool can_have_sponsored_messages_, bool has_aggressive_anti_spam_enabled_, bool has_paid_media_allowed_, bool has_pinned_stories_, int32 gift_count_, int32 my_boost_count_, int32 unrestrict_boost_count_, int64 sticker_set_id_, int64 custom_emoji_sticker_set_id_, object_ptr<chatLocation> &&location_, object_ptr<chatInviteLink> &&invite_link_, array<object_ptr<botCommands>> &&bot_commands_, object_ptr<botVerification> &&bot_verification_, int53 upgraded_from_basic_group_id_, int53 upgraded_from_max_message_id_)
  : photo_(std::move(photo_))
  , description_(description_)
  , member_count_(member_count_)
  , administrator_count_(administrator_count_)
  , restricted_count_(restricted_count_)
  , banned_count_(banned_count_)
  , linked_chat_id_(linked_chat_id_)
  , slow_mode_delay_(slow_mode_delay_)
  , slow_mode_delay_expires_in_(slow_mode_delay_expires_in_)
  , can_enable_paid_messages_(can_enable_paid_messages_)
  , can_enable_paid_reaction_(can_enable_paid_reaction_)
  , can_get_members_(can_get_members_)
  , has_hidden_members_(has_hidden_members_)
  , can_hide_members_(can_hide_members_)
  , can_set_sticker_set_(can_set_sticker_set_)
  , can_set_location_(can_set_location_)
  , can_get_statistics_(can_get_statistics_)
  , can_get_revenue_statistics_(can_get_revenue_statistics_)
  , can_get_star_revenue_statistics_(can_get_star_revenue_statistics_)
  , can_send_gift_(can_send_gift_)
  , can_toggle_aggressive_anti_spam_(can_toggle_aggressive_anti_spam_)
  , is_all_history_available_(is_all_history_available_)
  , can_have_sponsored_messages_(can_have_sponsored_messages_)
  , has_aggressive_anti_spam_enabled_(has_aggressive_anti_spam_enabled_)
  , has_paid_media_allowed_(has_paid_media_allowed_)
  , has_pinned_stories_(has_pinned_stories_)
  , gift_count_(gift_count_)
  , my_boost_count_(my_boost_count_)
  , unrestrict_boost_count_(unrestrict_boost_count_)
  , sticker_set_id_(sticker_set_id_)
  , custom_emoji_sticker_set_id_(custom_emoji_sticker_set_id_)
  , location_(std::move(location_))
  , invite_link_(std::move(invite_link_))
  , bot_commands_(std::move(bot_commands_))
  , bot_verification_(std::move(bot_verification_))
  , upgraded_from_basic_group_id_(upgraded_from_basic_group_id_)
  , upgraded_from_max_message_id_(upgraded_from_max_message_id_)
{}

const std::int32_t supergroupFullInfo::ID;

object_ptr<supergroupFullInfo> supergroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupFullInfo> res = make_object<supergroupFullInfo>();
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->member_count_ = env->GetIntField(p, res->member_count_fieldID);
  res->administrator_count_ = env->GetIntField(p, res->administrator_count_fieldID);
  res->restricted_count_ = env->GetIntField(p, res->restricted_count_fieldID);
  res->banned_count_ = env->GetIntField(p, res->banned_count_fieldID);
  res->linked_chat_id_ = env->GetLongField(p, res->linked_chat_id_fieldID);
  res->slow_mode_delay_ = env->GetIntField(p, res->slow_mode_delay_fieldID);
  res->slow_mode_delay_expires_in_ = env->GetDoubleField(p, res->slow_mode_delay_expires_in_fieldID);
  res->can_enable_paid_messages_ = (env->GetBooleanField(p, res->can_enable_paid_messages_fieldID) != 0);
  res->can_enable_paid_reaction_ = (env->GetBooleanField(p, res->can_enable_paid_reaction_fieldID) != 0);
  res->can_get_members_ = (env->GetBooleanField(p, res->can_get_members_fieldID) != 0);
  res->has_hidden_members_ = (env->GetBooleanField(p, res->has_hidden_members_fieldID) != 0);
  res->can_hide_members_ = (env->GetBooleanField(p, res->can_hide_members_fieldID) != 0);
  res->can_set_sticker_set_ = (env->GetBooleanField(p, res->can_set_sticker_set_fieldID) != 0);
  res->can_set_location_ = (env->GetBooleanField(p, res->can_set_location_fieldID) != 0);
  res->can_get_statistics_ = (env->GetBooleanField(p, res->can_get_statistics_fieldID) != 0);
  res->can_get_revenue_statistics_ = (env->GetBooleanField(p, res->can_get_revenue_statistics_fieldID) != 0);
  res->can_get_star_revenue_statistics_ = (env->GetBooleanField(p, res->can_get_star_revenue_statistics_fieldID) != 0);
  res->can_send_gift_ = (env->GetBooleanField(p, res->can_send_gift_fieldID) != 0);
  res->can_toggle_aggressive_anti_spam_ = (env->GetBooleanField(p, res->can_toggle_aggressive_anti_spam_fieldID) != 0);
  res->is_all_history_available_ = (env->GetBooleanField(p, res->is_all_history_available_fieldID) != 0);
  res->can_have_sponsored_messages_ = (env->GetBooleanField(p, res->can_have_sponsored_messages_fieldID) != 0);
  res->has_aggressive_anti_spam_enabled_ = (env->GetBooleanField(p, res->has_aggressive_anti_spam_enabled_fieldID) != 0);
  res->has_paid_media_allowed_ = (env->GetBooleanField(p, res->has_paid_media_allowed_fieldID) != 0);
  res->has_pinned_stories_ = (env->GetBooleanField(p, res->has_pinned_stories_fieldID) != 0);
  res->gift_count_ = env->GetIntField(p, res->gift_count_fieldID);
  res->my_boost_count_ = env->GetIntField(p, res->my_boost_count_fieldID);
  res->unrestrict_boost_count_ = env->GetIntField(p, res->unrestrict_boost_count_fieldID);
  res->sticker_set_id_ = env->GetLongField(p, res->sticker_set_id_fieldID);
  res->custom_emoji_sticker_set_id_ = env->GetLongField(p, res->custom_emoji_sticker_set_id_fieldID);
  res->location_ = jni::fetch_tl_object<chatLocation>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  res->bot_commands_ = jni::FetchVector<botCommands>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->bot_commands_fieldID));
  res->bot_verification_ = jni::fetch_tl_object<botVerification>(env, jni::fetch_object(env, p, res->bot_verification_fieldID));
  res->upgraded_from_basic_group_id_ = env->GetLongField(p, res->upgraded_from_basic_group_id_fieldID);
  res->upgraded_from_max_message_id_ = env->GetLongField(p, res->upgraded_from_max_message_id_fieldID);
  return res;
}

void supergroupFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, member_count_fieldID, member_count_);
  env->SetIntField(s, administrator_count_fieldID, administrator_count_);
  env->SetIntField(s, restricted_count_fieldID, restricted_count_);
  env->SetIntField(s, banned_count_fieldID, banned_count_);
  env->SetLongField(s, linked_chat_id_fieldID, linked_chat_id_);
  env->SetIntField(s, slow_mode_delay_fieldID, slow_mode_delay_);
  env->SetDoubleField(s, slow_mode_delay_expires_in_fieldID, slow_mode_delay_expires_in_);
  env->SetBooleanField(s, can_enable_paid_messages_fieldID, can_enable_paid_messages_);
  env->SetBooleanField(s, can_enable_paid_reaction_fieldID, can_enable_paid_reaction_);
  env->SetBooleanField(s, can_get_members_fieldID, can_get_members_);
  env->SetBooleanField(s, has_hidden_members_fieldID, has_hidden_members_);
  env->SetBooleanField(s, can_hide_members_fieldID, can_hide_members_);
  env->SetBooleanField(s, can_set_sticker_set_fieldID, can_set_sticker_set_);
  env->SetBooleanField(s, can_set_location_fieldID, can_set_location_);
  env->SetBooleanField(s, can_get_statistics_fieldID, can_get_statistics_);
  env->SetBooleanField(s, can_get_revenue_statistics_fieldID, can_get_revenue_statistics_);
  env->SetBooleanField(s, can_get_star_revenue_statistics_fieldID, can_get_star_revenue_statistics_);
  env->SetBooleanField(s, can_send_gift_fieldID, can_send_gift_);
  env->SetBooleanField(s, can_toggle_aggressive_anti_spam_fieldID, can_toggle_aggressive_anti_spam_);
  env->SetBooleanField(s, is_all_history_available_fieldID, is_all_history_available_);
  env->SetBooleanField(s, can_have_sponsored_messages_fieldID, can_have_sponsored_messages_);
  env->SetBooleanField(s, has_aggressive_anti_spam_enabled_fieldID, has_aggressive_anti_spam_enabled_);
  env->SetBooleanField(s, has_paid_media_allowed_fieldID, has_paid_media_allowed_);
  env->SetBooleanField(s, has_pinned_stories_fieldID, has_pinned_stories_);
  env->SetIntField(s, gift_count_fieldID, gift_count_);
  env->SetIntField(s, my_boost_count_fieldID, my_boost_count_);
  env->SetIntField(s, unrestrict_boost_count_fieldID, unrestrict_boost_count_);
  env->SetLongField(s, sticker_set_id_fieldID, sticker_set_id_);
  env->SetLongField(s, custom_emoji_sticker_set_id_fieldID, custom_emoji_sticker_set_id_);
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, bot_commands_); if (arr_tmp_) { env->SetObjectField(s, bot_commands_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  if (bot_verification_ != nullptr) { jobject next; bot_verification_->store(env, next); if (next) { env->SetObjectField(s, bot_verification_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, upgraded_from_basic_group_id_fieldID, upgraded_from_basic_group_id_);
  env->SetLongField(s, upgraded_from_max_message_id_fieldID, upgraded_from_max_message_id_);
}

void supergroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupFullInfo");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("description", description_);
    s.store_field("memberCount", member_count_);
    s.store_field("administratorCount", administrator_count_);
    s.store_field("restrictedCount", restricted_count_);
    s.store_field("bannedCount", banned_count_);
    s.store_field("linkedChatId", linked_chat_id_);
    s.store_field("slowModeDelay", slow_mode_delay_);
    s.store_field("slowModeDelayExpiresIn", slow_mode_delay_expires_in_);
    s.store_field("canEnablePaidMessages", can_enable_paid_messages_);
    s.store_field("canEnablePaidReaction", can_enable_paid_reaction_);
    s.store_field("canGetMembers", can_get_members_);
    s.store_field("hasHiddenMembers", has_hidden_members_);
    s.store_field("canHideMembers", can_hide_members_);
    s.store_field("canSetStickerSet", can_set_sticker_set_);
    s.store_field("canSetLocation", can_set_location_);
    s.store_field("canGetStatistics", can_get_statistics_);
    s.store_field("canGetRevenueStatistics", can_get_revenue_statistics_);
    s.store_field("canGetStarRevenueStatistics", can_get_star_revenue_statistics_);
    s.store_field("canSendGift", can_send_gift_);
    s.store_field("canToggleAggressiveAntiSpam", can_toggle_aggressive_anti_spam_);
    s.store_field("isAllHistoryAvailable", is_all_history_available_);
    s.store_field("canHaveSponsoredMessages", can_have_sponsored_messages_);
    s.store_field("hasAggressiveAntiSpamEnabled", has_aggressive_anti_spam_enabled_);
    s.store_field("hasPaidMediaAllowed", has_paid_media_allowed_);
    s.store_field("hasPinnedStories", has_pinned_stories_);
    s.store_field("giftCount", gift_count_);
    s.store_field("myBoostCount", my_boost_count_);
    s.store_field("unrestrictBoostCount", unrestrict_boost_count_);
    s.store_field("stickerSetId", sticker_set_id_);
    s.store_field("customEmojiStickerSetId", custom_emoji_sticker_set_id_);
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    { s.store_vector_begin("botCommands", bot_commands_.size()); for (const auto &_value : bot_commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_object_field("botVerification", static_cast<const BaseObject *>(bot_verification_.get()));
    s.store_field("upgradedFromBasicGroupId", upgraded_from_basic_group_id_);
    s.store_field("upgradedFromMaxMessageId", upgraded_from_max_message_id_);
    s.store_class_end();
  }
}

void supergroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupFullInfo").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    member_count_fieldID = jni::get_field_id(env, Class, "memberCount", "I");
    administrator_count_fieldID = jni::get_field_id(env, Class, "administratorCount", "I");
    restricted_count_fieldID = jni::get_field_id(env, Class, "restrictedCount", "I");
    banned_count_fieldID = jni::get_field_id(env, Class, "bannedCount", "I");
    linked_chat_id_fieldID = jni::get_field_id(env, Class, "linkedChatId", "J");
    slow_mode_delay_fieldID = jni::get_field_id(env, Class, "slowModeDelay", "I");
    slow_mode_delay_expires_in_fieldID = jni::get_field_id(env, Class, "slowModeDelayExpiresIn", "D");
    can_enable_paid_messages_fieldID = jni::get_field_id(env, Class, "canEnablePaidMessages", "Z");
    can_enable_paid_reaction_fieldID = jni::get_field_id(env, Class, "canEnablePaidReaction", "Z");
    can_get_members_fieldID = jni::get_field_id(env, Class, "canGetMembers", "Z");
    has_hidden_members_fieldID = jni::get_field_id(env, Class, "hasHiddenMembers", "Z");
    can_hide_members_fieldID = jni::get_field_id(env, Class, "canHideMembers", "Z");
    can_set_sticker_set_fieldID = jni::get_field_id(env, Class, "canSetStickerSet", "Z");
    can_set_location_fieldID = jni::get_field_id(env, Class, "canSetLocation", "Z");
    can_get_statistics_fieldID = jni::get_field_id(env, Class, "canGetStatistics", "Z");
    can_get_revenue_statistics_fieldID = jni::get_field_id(env, Class, "canGetRevenueStatistics", "Z");
    can_get_star_revenue_statistics_fieldID = jni::get_field_id(env, Class, "canGetStarRevenueStatistics", "Z");
    can_send_gift_fieldID = jni::get_field_id(env, Class, "canSendGift", "Z");
    can_toggle_aggressive_anti_spam_fieldID = jni::get_field_id(env, Class, "canToggleAggressiveAntiSpam", "Z");
    is_all_history_available_fieldID = jni::get_field_id(env, Class, "isAllHistoryAvailable", "Z");
    can_have_sponsored_messages_fieldID = jni::get_field_id(env, Class, "canHaveSponsoredMessages", "Z");
    has_aggressive_anti_spam_enabled_fieldID = jni::get_field_id(env, Class, "hasAggressiveAntiSpamEnabled", "Z");
    has_paid_media_allowed_fieldID = jni::get_field_id(env, Class, "hasPaidMediaAllowed", "Z");
    has_pinned_stories_fieldID = jni::get_field_id(env, Class, "hasPinnedStories", "Z");
    gift_count_fieldID = jni::get_field_id(env, Class, "giftCount", "I");
    my_boost_count_fieldID = jni::get_field_id(env, Class, "myBoostCount", "I");
    unrestrict_boost_count_fieldID = jni::get_field_id(env, Class, "unrestrictBoostCount", "I");
    sticker_set_id_fieldID = jni::get_field_id(env, Class, "stickerSetId", "J");
    custom_emoji_sticker_set_id_fieldID = jni::get_field_id(env, Class, "customEmojiStickerSetId", "J");
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatLocation;").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    bot_commands_fieldID = jni::get_field_id(env, Class, "botCommands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotCommands;").c_str());
    bot_verification_fieldID = jni::get_field_id(env, Class, "botVerification", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotVerification;").c_str());
    upgraded_from_basic_group_id_fieldID = jni::get_field_id(env, Class, "upgradedFromBasicGroupId", "J");
    upgraded_from_max_message_id_fieldID = jni::get_field_id(env, Class, "upgradedFromMaxMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass SupergroupMembersFilter::Class;

object_ptr<SupergroupMembersFilter> SupergroupMembersFilter::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case supergroupMembersFilterRecent::ID:
      return supergroupMembersFilterRecent::fetch(env, p);
    case supergroupMembersFilterContacts::ID:
      return supergroupMembersFilterContacts::fetch(env, p);
    case supergroupMembersFilterAdministrators::ID:
      return supergroupMembersFilterAdministrators::fetch(env, p);
    case supergroupMembersFilterSearch::ID:
      return supergroupMembersFilterSearch::fetch(env, p);
    case supergroupMembersFilterRestricted::ID:
      return supergroupMembersFilterRestricted::fetch(env, p);
    case supergroupMembersFilterBanned::ID:
      return supergroupMembersFilterBanned::fetch(env, p);
    case supergroupMembersFilterMention::ID:
      return supergroupMembersFilterMention::fetch(env, p);
    case supergroupMembersFilterBots::ID:
      return supergroupMembersFilterBots::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void SupergroupMembersFilter::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilter").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterRecent::Class;

supergroupMembersFilterRecent::supergroupMembersFilterRecent() {
}

const std::int32_t supergroupMembersFilterRecent::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterRecent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<supergroupMembersFilterRecent>();
}

void supergroupMembersFilterRecent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterRecent");
    s.store_class_end();
  }
}

void supergroupMembersFilterRecent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterRecent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterContacts::Class;
jfieldID supergroupMembersFilterContacts::query_fieldID;

supergroupMembersFilterContacts::supergroupMembersFilterContacts()
  : query_()
{}

supergroupMembersFilterContacts::supergroupMembersFilterContacts(string const &query_)
  : query_(query_)
{}

const std::int32_t supergroupMembersFilterContacts::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupMembersFilterContacts> res = make_object<supergroupMembersFilterContacts>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return std::move(res);
}

void supergroupMembersFilterContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterContacts");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

void supergroupMembersFilterContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterContacts").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterAdministrators::Class;

supergroupMembersFilterAdministrators::supergroupMembersFilterAdministrators() {
}

const std::int32_t supergroupMembersFilterAdministrators::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<supergroupMembersFilterAdministrators>();
}

void supergroupMembersFilterAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterAdministrators");
    s.store_class_end();
  }
}

void supergroupMembersFilterAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterAdministrators").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterSearch::Class;
jfieldID supergroupMembersFilterSearch::query_fieldID;

supergroupMembersFilterSearch::supergroupMembersFilterSearch()
  : query_()
{}

supergroupMembersFilterSearch::supergroupMembersFilterSearch(string const &query_)
  : query_(query_)
{}

const std::int32_t supergroupMembersFilterSearch::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterSearch::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupMembersFilterSearch> res = make_object<supergroupMembersFilterSearch>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return std::move(res);
}

void supergroupMembersFilterSearch::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterSearch");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

void supergroupMembersFilterSearch::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterSearch").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterRestricted::Class;
jfieldID supergroupMembersFilterRestricted::query_fieldID;

supergroupMembersFilterRestricted::supergroupMembersFilterRestricted()
  : query_()
{}

supergroupMembersFilterRestricted::supergroupMembersFilterRestricted(string const &query_)
  : query_(query_)
{}

const std::int32_t supergroupMembersFilterRestricted::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterRestricted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupMembersFilterRestricted> res = make_object<supergroupMembersFilterRestricted>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return std::move(res);
}

void supergroupMembersFilterRestricted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterRestricted");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

void supergroupMembersFilterRestricted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterRestricted").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterBanned::Class;
jfieldID supergroupMembersFilterBanned::query_fieldID;

supergroupMembersFilterBanned::supergroupMembersFilterBanned()
  : query_()
{}

supergroupMembersFilterBanned::supergroupMembersFilterBanned(string const &query_)
  : query_(query_)
{}

const std::int32_t supergroupMembersFilterBanned::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterBanned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupMembersFilterBanned> res = make_object<supergroupMembersFilterBanned>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return std::move(res);
}

void supergroupMembersFilterBanned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterBanned");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

void supergroupMembersFilterBanned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterBanned").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterMention::Class;
jfieldID supergroupMembersFilterMention::query_fieldID;
jfieldID supergroupMembersFilterMention::message_thread_id_fieldID;

supergroupMembersFilterMention::supergroupMembersFilterMention()
  : query_()
  , message_thread_id_()
{}

supergroupMembersFilterMention::supergroupMembersFilterMention(string const &query_, int53 message_thread_id_)
  : query_(query_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t supergroupMembersFilterMention::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterMention::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<supergroupMembersFilterMention> res = make_object<supergroupMembersFilterMention>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return std::move(res);
}

void supergroupMembersFilterMention::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterMention");
    s.store_field("query", query_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

void supergroupMembersFilterMention::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterMention").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass supergroupMembersFilterBots::Class;

supergroupMembersFilterBots::supergroupMembersFilterBots() {
}

const std::int32_t supergroupMembersFilterBots::ID;

object_ptr<SupergroupMembersFilter> supergroupMembersFilterBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<supergroupMembersFilterBots>();
}

void supergroupMembersFilterBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SupergroupMembersFilterBots");
    s.store_class_end();
  }
}

void supergroupMembersFilterBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SupergroupMembersFilterBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrl::Class;
jfieldID tMeUrl::url_fieldID;
jfieldID tMeUrl::type_fieldID;

tMeUrl::tMeUrl()
  : url_()
  , type_()
{}

tMeUrl::tMeUrl(string const &url_, object_ptr<TMeUrlType> &&type_)
  : url_(url_)
  , type_(std::move(type_))
{}

const std::int32_t tMeUrl::ID;

object_ptr<tMeUrl> tMeUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrl> res = make_object<tMeUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->type_ = jni::fetch_tl_object<TMeUrlType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void tMeUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void tMeUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrl");
    s.store_field("url", url_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void tMeUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TMeUrlType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass TMeUrlType::Class;

object_ptr<TMeUrlType> TMeUrlType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case tMeUrlTypeUser::ID:
      return tMeUrlTypeUser::fetch(env, p);
    case tMeUrlTypeSupergroup::ID:
      return tMeUrlTypeSupergroup::fetch(env, p);
    case tMeUrlTypeChatInvite::ID:
      return tMeUrlTypeChatInvite::fetch(env, p);
    case tMeUrlTypeStickerSet::ID:
      return tMeUrlTypeStickerSet::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TMeUrlType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrlType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrlTypeUser::Class;
jfieldID tMeUrlTypeUser::user_id_fieldID;

tMeUrlTypeUser::tMeUrlTypeUser()
  : user_id_()
{}

tMeUrlTypeUser::tMeUrlTypeUser(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t tMeUrlTypeUser::ID;

object_ptr<TMeUrlType> tMeUrlTypeUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrlTypeUser> res = make_object<tMeUrlTypeUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void tMeUrlTypeUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void tMeUrlTypeUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrlTypeUser");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void tMeUrlTypeUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrlTypeUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrlTypeSupergroup::Class;
jfieldID tMeUrlTypeSupergroup::supergroup_id_fieldID;

tMeUrlTypeSupergroup::tMeUrlTypeSupergroup()
  : supergroup_id_()
{}

tMeUrlTypeSupergroup::tMeUrlTypeSupergroup(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t tMeUrlTypeSupergroup::ID;

object_ptr<TMeUrlType> tMeUrlTypeSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrlTypeSupergroup> res = make_object<tMeUrlTypeSupergroup>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return std::move(res);
}

void tMeUrlTypeSupergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, supergroup_id_fieldID, supergroup_id_);
}

void tMeUrlTypeSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrlTypeSupergroup");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

void tMeUrlTypeSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrlTypeSupergroup").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrlTypeChatInvite::Class;
jfieldID tMeUrlTypeChatInvite::info_fieldID;

tMeUrlTypeChatInvite::tMeUrlTypeChatInvite()
  : info_()
{}

tMeUrlTypeChatInvite::tMeUrlTypeChatInvite(object_ptr<chatInviteLinkInfo> &&info_)
  : info_(std::move(info_))
{}

const std::int32_t tMeUrlTypeChatInvite::ID;

object_ptr<TMeUrlType> tMeUrlTypeChatInvite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrlTypeChatInvite> res = make_object<tMeUrlTypeChatInvite>();
  res->info_ = jni::fetch_tl_object<chatInviteLinkInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  return std::move(res);
}

void tMeUrlTypeChatInvite::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (info_ != nullptr) { jobject next; info_->store(env, next); if (next) { env->SetObjectField(s, info_fieldID, next); env->DeleteLocalRef(next); } }
}

void tMeUrlTypeChatInvite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrlTypeChatInvite");
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    s.store_class_end();
  }
}

void tMeUrlTypeChatInvite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrlTypeChatInvite").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLinkInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrlTypeStickerSet::Class;
jfieldID tMeUrlTypeStickerSet::sticker_set_id_fieldID;

tMeUrlTypeStickerSet::tMeUrlTypeStickerSet()
  : sticker_set_id_()
{}

tMeUrlTypeStickerSet::tMeUrlTypeStickerSet(int64 sticker_set_id_)
  : sticker_set_id_(sticker_set_id_)
{}

const std::int32_t tMeUrlTypeStickerSet::ID;

object_ptr<TMeUrlType> tMeUrlTypeStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrlTypeStickerSet> res = make_object<tMeUrlTypeStickerSet>();
  res->sticker_set_id_ = env->GetLongField(p, res->sticker_set_id_fieldID);
  return std::move(res);
}

void tMeUrlTypeStickerSet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, sticker_set_id_fieldID, sticker_set_id_);
}

void tMeUrlTypeStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrlTypeStickerSet");
    s.store_field("stickerSetId", sticker_set_id_);
    s.store_class_end();
  }
}

void tMeUrlTypeStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrlTypeStickerSet").c_str());
    sticker_set_id_fieldID = jni::get_field_id(env, Class, "stickerSetId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass tMeUrls::Class;
jfieldID tMeUrls::urls_fieldID;

tMeUrls::tMeUrls()
  : urls_()
{}

tMeUrls::tMeUrls(array<object_ptr<tMeUrl>> &&urls_)
  : urls_(std::move(urls_))
{}

const std::int32_t tMeUrls::ID;

object_ptr<tMeUrls> tMeUrls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<tMeUrls> res = make_object<tMeUrls>();
  res->urls_ = jni::FetchVector<tMeUrl>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->urls_fieldID));
  return res;
}

void tMeUrls::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, urls_); if (arr_tmp_) { env->SetObjectField(s, urls_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void tMeUrls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TMeUrls");
    { s.store_vector_begin("urls", urls_.size()); for (const auto &_value : urls_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void tMeUrls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TMeUrls").c_str());
    urls_fieldID = jni::get_field_id(env, Class, "urls", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TMeUrl;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass TargetChat::Class;

object_ptr<TargetChat> TargetChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case targetChatCurrent::ID:
      return targetChatCurrent::fetch(env, p);
    case targetChatChosen::ID:
      return targetChatChosen::fetch(env, p);
    case targetChatInternalLink::ID:
      return targetChatInternalLink::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TargetChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TargetChat").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass targetChatCurrent::Class;

targetChatCurrent::targetChatCurrent() {
}

const std::int32_t targetChatCurrent::ID;

object_ptr<TargetChat> targetChatCurrent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<targetChatCurrent>();
}

void targetChatCurrent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void targetChatCurrent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TargetChatCurrent");
    s.store_class_end();
  }
}

void targetChatCurrent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TargetChatCurrent").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass targetChatChosen::Class;
jfieldID targetChatChosen::types_fieldID;

targetChatChosen::targetChatChosen()
  : types_()
{}

targetChatChosen::targetChatChosen(object_ptr<targetChatTypes> &&types_)
  : types_(std::move(types_))
{}

const std::int32_t targetChatChosen::ID;

object_ptr<TargetChat> targetChatChosen::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<targetChatChosen> res = make_object<targetChatChosen>();
  res->types_ = jni::fetch_tl_object<targetChatTypes>(env, jni::fetch_object(env, p, res->types_fieldID));
  return std::move(res);
}

void targetChatChosen::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (types_ != nullptr) { jobject next; types_->store(env, next); if (next) { env->SetObjectField(s, types_fieldID, next); env->DeleteLocalRef(next); } }
}

void targetChatChosen::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TargetChatChosen");
    s.store_object_field("types", static_cast<const BaseObject *>(types_.get()));
    s.store_class_end();
  }
}

void targetChatChosen::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TargetChatChosen").c_str());
    types_fieldID = jni::get_field_id(env, Class, "types", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TargetChatTypes;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass targetChatInternalLink::Class;
jfieldID targetChatInternalLink::link_fieldID;

targetChatInternalLink::targetChatInternalLink()
  : link_()
{}

targetChatInternalLink::targetChatInternalLink(object_ptr<InternalLinkType> &&link_)
  : link_(std::move(link_))
{}

const std::int32_t targetChatInternalLink::ID;

object_ptr<TargetChat> targetChatInternalLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<targetChatInternalLink> res = make_object<targetChatInternalLink>();
  res->link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->link_fieldID));
  return std::move(res);
}

void targetChatInternalLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (link_ != nullptr) { jobject next; link_->store(env, next); if (next) { env->SetObjectField(s, link_fieldID, next); env->DeleteLocalRef(next); } }
}

void targetChatInternalLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TargetChatInternalLink");
    s.store_object_field("link", static_cast<const BaseObject *>(link_.get()));
    s.store_class_end();
  }
}

void targetChatInternalLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TargetChatInternalLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass targetChatTypes::Class;
jfieldID targetChatTypes::allow_user_chats_fieldID;
jfieldID targetChatTypes::allow_bot_chats_fieldID;
jfieldID targetChatTypes::allow_group_chats_fieldID;
jfieldID targetChatTypes::allow_channel_chats_fieldID;

targetChatTypes::targetChatTypes()
  : allow_user_chats_()
  , allow_bot_chats_()
  , allow_group_chats_()
  , allow_channel_chats_()
{}

targetChatTypes::targetChatTypes(bool allow_user_chats_, bool allow_bot_chats_, bool allow_group_chats_, bool allow_channel_chats_)
  : allow_user_chats_(allow_user_chats_)
  , allow_bot_chats_(allow_bot_chats_)
  , allow_group_chats_(allow_group_chats_)
  , allow_channel_chats_(allow_channel_chats_)
{}

const std::int32_t targetChatTypes::ID;

object_ptr<targetChatTypes> targetChatTypes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<targetChatTypes> res = make_object<targetChatTypes>();
  res->allow_user_chats_ = (env->GetBooleanField(p, res->allow_user_chats_fieldID) != 0);
  res->allow_bot_chats_ = (env->GetBooleanField(p, res->allow_bot_chats_fieldID) != 0);
  res->allow_group_chats_ = (env->GetBooleanField(p, res->allow_group_chats_fieldID) != 0);
  res->allow_channel_chats_ = (env->GetBooleanField(p, res->allow_channel_chats_fieldID) != 0);
  return res;
}

void targetChatTypes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, allow_user_chats_fieldID, allow_user_chats_);
  env->SetBooleanField(s, allow_bot_chats_fieldID, allow_bot_chats_);
  env->SetBooleanField(s, allow_group_chats_fieldID, allow_group_chats_);
  env->SetBooleanField(s, allow_channel_chats_fieldID, allow_channel_chats_);
}

void targetChatTypes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TargetChatTypes");
    s.store_field("allowUserChats", allow_user_chats_);
    s.store_field("allowBotChats", allow_bot_chats_);
    s.store_field("allowGroupChats", allow_group_chats_);
    s.store_field("allowChannelChats", allow_channel_chats_);
    s.store_class_end();
  }
}

void targetChatTypes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TargetChatTypes").c_str());
    allow_user_chats_fieldID = jni::get_field_id(env, Class, "allowUserChats", "Z");
    allow_bot_chats_fieldID = jni::get_field_id(env, Class, "allowBotChats", "Z");
    allow_group_chats_fieldID = jni::get_field_id(env, Class, "allowGroupChats", "Z");
    allow_channel_chats_fieldID = jni::get_field_id(env, Class, "allowChannelChats", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass TelegramPaymentPurpose::Class;

object_ptr<TelegramPaymentPurpose> TelegramPaymentPurpose::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case telegramPaymentPurposePremiumGift::ID:
      return telegramPaymentPurposePremiumGift::fetch(env, p);
    case telegramPaymentPurposePremiumGiftCodes::ID:
      return telegramPaymentPurposePremiumGiftCodes::fetch(env, p);
    case telegramPaymentPurposePremiumGiveaway::ID:
      return telegramPaymentPurposePremiumGiveaway::fetch(env, p);
    case telegramPaymentPurposeStars::ID:
      return telegramPaymentPurposeStars::fetch(env, p);
    case telegramPaymentPurposeGiftedStars::ID:
      return telegramPaymentPurposeGiftedStars::fetch(env, p);
    case telegramPaymentPurposeStarGiveaway::ID:
      return telegramPaymentPurposeStarGiveaway::fetch(env, p);
    case telegramPaymentPurposeJoinChat::ID:
      return telegramPaymentPurposeJoinChat::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TelegramPaymentPurpose::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurpose").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposePremiumGift::Class;
jfieldID telegramPaymentPurposePremiumGift::currency_fieldID;
jfieldID telegramPaymentPurposePremiumGift::amount_fieldID;
jfieldID telegramPaymentPurposePremiumGift::user_id_fieldID;
jfieldID telegramPaymentPurposePremiumGift::month_count_fieldID;
jfieldID telegramPaymentPurposePremiumGift::text_fieldID;

telegramPaymentPurposePremiumGift::telegramPaymentPurposePremiumGift()
  : currency_()
  , amount_()
  , user_id_()
  , month_count_()
  , text_()
{}

telegramPaymentPurposePremiumGift::telegramPaymentPurposePremiumGift(string const &currency_, int53 amount_, int53 user_id_, int32 month_count_, object_ptr<formattedText> &&text_)
  : currency_(currency_)
  , amount_(amount_)
  , user_id_(user_id_)
  , month_count_(month_count_)
  , text_(std::move(text_))
{}

const std::int32_t telegramPaymentPurposePremiumGift::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposePremiumGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposePremiumGift> res = make_object<telegramPaymentPurposePremiumGift>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void telegramPaymentPurposePremiumGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposePremiumGift");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("userId", user_id_);
    s.store_field("monthCount", month_count_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void telegramPaymentPurposePremiumGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposePremiumGift").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposePremiumGiftCodes::Class;
jfieldID telegramPaymentPurposePremiumGiftCodes::boosted_chat_id_fieldID;
jfieldID telegramPaymentPurposePremiumGiftCodes::currency_fieldID;
jfieldID telegramPaymentPurposePremiumGiftCodes::amount_fieldID;
jfieldID telegramPaymentPurposePremiumGiftCodes::user_ids_fieldID;
jfieldID telegramPaymentPurposePremiumGiftCodes::month_count_fieldID;
jfieldID telegramPaymentPurposePremiumGiftCodes::text_fieldID;

telegramPaymentPurposePremiumGiftCodes::telegramPaymentPurposePremiumGiftCodes()
  : boosted_chat_id_()
  , currency_()
  , amount_()
  , user_ids_()
  , month_count_()
  , text_()
{}

telegramPaymentPurposePremiumGiftCodes::telegramPaymentPurposePremiumGiftCodes(int53 boosted_chat_id_, string const &currency_, int53 amount_, array<int53> &&user_ids_, int32 month_count_, object_ptr<formattedText> &&text_)
  : boosted_chat_id_(boosted_chat_id_)
  , currency_(currency_)
  , amount_(amount_)
  , user_ids_(std::move(user_ids_))
  , month_count_(month_count_)
  , text_(std::move(text_))
{}

const std::int32_t telegramPaymentPurposePremiumGiftCodes::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposePremiumGiftCodes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposePremiumGiftCodes> res = make_object<telegramPaymentPurposePremiumGiftCodes>();
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return std::move(res);
}

void telegramPaymentPurposePremiumGiftCodes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposePremiumGiftCodes");
    s.store_field("boostedChatId", boosted_chat_id_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("monthCount", month_count_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

void telegramPaymentPurposePremiumGiftCodes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposePremiumGiftCodes").c_str());
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposePremiumGiveaway::Class;
jfieldID telegramPaymentPurposePremiumGiveaway::parameters_fieldID;
jfieldID telegramPaymentPurposePremiumGiveaway::currency_fieldID;
jfieldID telegramPaymentPurposePremiumGiveaway::amount_fieldID;
jfieldID telegramPaymentPurposePremiumGiveaway::winner_count_fieldID;
jfieldID telegramPaymentPurposePremiumGiveaway::month_count_fieldID;

telegramPaymentPurposePremiumGiveaway::telegramPaymentPurposePremiumGiveaway()
  : parameters_()
  , currency_()
  , amount_()
  , winner_count_()
  , month_count_()
{}

telegramPaymentPurposePremiumGiveaway::telegramPaymentPurposePremiumGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int32 month_count_)
  : parameters_(std::move(parameters_))
  , currency_(currency_)
  , amount_(amount_)
  , winner_count_(winner_count_)
  , month_count_(month_count_)
{}

const std::int32_t telegramPaymentPurposePremiumGiveaway::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposePremiumGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposePremiumGiveaway> res = make_object<telegramPaymentPurposePremiumGiveaway>();
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  return std::move(res);
}

void telegramPaymentPurposePremiumGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposePremiumGiveaway");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("monthCount", month_count_);
    s.store_class_end();
  }
}

void telegramPaymentPurposePremiumGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposePremiumGiveaway").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposeStars::Class;
jfieldID telegramPaymentPurposeStars::currency_fieldID;
jfieldID telegramPaymentPurposeStars::amount_fieldID;
jfieldID telegramPaymentPurposeStars::star_count_fieldID;

telegramPaymentPurposeStars::telegramPaymentPurposeStars()
  : currency_()
  , amount_()
  , star_count_()
{}

telegramPaymentPurposeStars::telegramPaymentPurposeStars(string const &currency_, int53 amount_, int53 star_count_)
  : currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
{}

const std::int32_t telegramPaymentPurposeStars::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposeStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposeStars> res = make_object<telegramPaymentPurposeStars>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void telegramPaymentPurposeStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposeStars");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void telegramPaymentPurposeStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposeStars").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposeGiftedStars::Class;
jfieldID telegramPaymentPurposeGiftedStars::user_id_fieldID;
jfieldID telegramPaymentPurposeGiftedStars::currency_fieldID;
jfieldID telegramPaymentPurposeGiftedStars::amount_fieldID;
jfieldID telegramPaymentPurposeGiftedStars::star_count_fieldID;

telegramPaymentPurposeGiftedStars::telegramPaymentPurposeGiftedStars()
  : user_id_()
  , currency_()
  , amount_()
  , star_count_()
{}

telegramPaymentPurposeGiftedStars::telegramPaymentPurposeGiftedStars(int53 user_id_, string const &currency_, int53 amount_, int53 star_count_)
  : user_id_(user_id_)
  , currency_(currency_)
  , amount_(amount_)
  , star_count_(star_count_)
{}

const std::int32_t telegramPaymentPurposeGiftedStars::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposeGiftedStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposeGiftedStars> res = make_object<telegramPaymentPurposeGiftedStars>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void telegramPaymentPurposeGiftedStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposeGiftedStars");
    s.store_field("userId", user_id_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void telegramPaymentPurposeGiftedStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposeGiftedStars").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposeStarGiveaway::Class;
jfieldID telegramPaymentPurposeStarGiveaway::parameters_fieldID;
jfieldID telegramPaymentPurposeStarGiveaway::currency_fieldID;
jfieldID telegramPaymentPurposeStarGiveaway::amount_fieldID;
jfieldID telegramPaymentPurposeStarGiveaway::winner_count_fieldID;
jfieldID telegramPaymentPurposeStarGiveaway::star_count_fieldID;

telegramPaymentPurposeStarGiveaway::telegramPaymentPurposeStarGiveaway()
  : parameters_()
  , currency_()
  , amount_()
  , winner_count_()
  , star_count_()
{}

telegramPaymentPurposeStarGiveaway::telegramPaymentPurposeStarGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int53 star_count_)
  : parameters_(std::move(parameters_))
  , currency_(currency_)
  , amount_(amount_)
  , winner_count_(winner_count_)
  , star_count_(star_count_)
{}

const std::int32_t telegramPaymentPurposeStarGiveaway::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposeStarGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposeStarGiveaway> res = make_object<telegramPaymentPurposeStarGiveaway>();
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return std::move(res);
}

void telegramPaymentPurposeStarGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposeStarGiveaway");
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_field("winnerCount", winner_count_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

void telegramPaymentPurposeStarGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposeStarGiveaway").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass telegramPaymentPurposeJoinChat::Class;
jfieldID telegramPaymentPurposeJoinChat::invite_link_fieldID;

telegramPaymentPurposeJoinChat::telegramPaymentPurposeJoinChat()
  : invite_link_()
{}

telegramPaymentPurposeJoinChat::telegramPaymentPurposeJoinChat(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t telegramPaymentPurposeJoinChat::ID;

object_ptr<TelegramPaymentPurpose> telegramPaymentPurposeJoinChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<telegramPaymentPurposeJoinChat> res = make_object<telegramPaymentPurposeJoinChat>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return std::move(res);
}

void telegramPaymentPurposeJoinChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TelegramPaymentPurposeJoinChat");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

void telegramPaymentPurposeJoinChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TelegramPaymentPurposeJoinChat").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass temporaryPasswordState::Class;
jfieldID temporaryPasswordState::has_password_fieldID;
jfieldID temporaryPasswordState::valid_for_fieldID;

temporaryPasswordState::temporaryPasswordState()
  : has_password_()
  , valid_for_()
{}

temporaryPasswordState::temporaryPasswordState(bool has_password_, int32 valid_for_)
  : has_password_(has_password_)
  , valid_for_(valid_for_)
{}

const std::int32_t temporaryPasswordState::ID;

object_ptr<temporaryPasswordState> temporaryPasswordState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<temporaryPasswordState> res = make_object<temporaryPasswordState>();
  res->has_password_ = (env->GetBooleanField(p, res->has_password_fieldID) != 0);
  res->valid_for_ = env->GetIntField(p, res->valid_for_fieldID);
  return res;
}

void temporaryPasswordState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, has_password_fieldID, has_password_);
  env->SetIntField(s, valid_for_fieldID, valid_for_);
}

void temporaryPasswordState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TemporaryPasswordState");
    s.store_field("hasPassword", has_password_);
    s.store_field("validFor", valid_for_);
    s.store_class_end();
  }
}

void temporaryPasswordState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TemporaryPasswordState").c_str());
    has_password_fieldID = jni::get_field_id(env, Class, "hasPassword", "Z");
    valid_for_fieldID = jni::get_field_id(env, Class, "validFor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass termsOfService::Class;
jfieldID termsOfService::text_fieldID;
jfieldID termsOfService::min_user_age_fieldID;
jfieldID termsOfService::show_popup_fieldID;

termsOfService::termsOfService()
  : text_()
  , min_user_age_()
  , show_popup_()
{}

termsOfService::termsOfService(object_ptr<formattedText> &&text_, int32 min_user_age_, bool show_popup_)
  : text_(std::move(text_))
  , min_user_age_(min_user_age_)
  , show_popup_(show_popup_)
{}

const std::int32_t termsOfService::ID;

object_ptr<termsOfService> termsOfService::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<termsOfService> res = make_object<termsOfService>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->min_user_age_ = env->GetIntField(p, res->min_user_age_fieldID);
  res->show_popup_ = (env->GetBooleanField(p, res->show_popup_fieldID) != 0);
  return res;
}

void termsOfService::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, min_user_age_fieldID, min_user_age_);
  env->SetBooleanField(s, show_popup_fieldID, show_popup_);
}

void termsOfService::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TermsOfService");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("minUserAge", min_user_age_);
    s.store_field("showPopup", show_popup_);
    s.store_class_end();
  }
}

void termsOfService::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TermsOfService").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    min_user_age_fieldID = jni::get_field_id(env, Class, "minUserAge", "I");
    show_popup_fieldID = jni::get_field_id(env, Class, "showPopup", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass testBytes::Class;
jfieldID testBytes::value_fieldID;

testBytes::testBytes()
  : value_()
{}

testBytes::testBytes(bytes const &value_)
  : value_(std::move(value_))
{}

const std::int32_t testBytes::ID;

object_ptr<testBytes> testBytes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testBytes> res = make_object<testBytes>();
  res->value_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void testBytes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jbyteArray nextBytes = jni::to_bytes(env, value_); if (nextBytes) { env->SetObjectField(s, value_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void testBytes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestBytes");
    s.store_bytes_field("value", value_);
    s.store_class_end();
  }
}

void testBytes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestBytes").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass testInt::Class;
jfieldID testInt::value_fieldID;

testInt::testInt()
  : value_()
{}

testInt::testInt(int32 value_)
  : value_(value_)
{}

const std::int32_t testInt::ID;

object_ptr<testInt> testInt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testInt> res = make_object<testInt>();
  res->value_ = env->GetIntField(p, res->value_fieldID);
  return res;
}

void testInt::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, value_fieldID, value_);
}

void testInt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestInt");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void testInt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestInt").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "I");
    return true;
  }();
  (void)is_inited;
}

jclass testString::Class;
jfieldID testString::value_fieldID;

testString::testString()
  : value_()
{}

testString::testString(string const &value_)
  : value_(value_)
{}

const std::int32_t testString::ID;

object_ptr<testString> testString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testString> res = make_object<testString>();
  res->value_ = jni::fetch_string(env, p, res->value_fieldID);
  return res;
}

void testString::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, value_); if (nextString) { env->SetObjectField(s, value_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void testString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestString");
    s.store_field("value", value_);
    s.store_class_end();
  }
}

void testString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestString").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass testVectorInt::Class;
jfieldID testVectorInt::value_fieldID;

testVectorInt::testVectorInt()
  : value_()
{}

testVectorInt::testVectorInt(array<int32> &&value_)
  : value_(std::move(value_))
{}

const std::int32_t testVectorInt::ID;

object_ptr<testVectorInt> testVectorInt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testVectorInt> res = make_object<testVectorInt>();
  res->value_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void testVectorInt::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, value_); if (arr_tmp_) { env->SetObjectField(s, value_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void testVectorInt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestVectorInt");
    { s.store_vector_begin("value", value_.size()); for (const auto &_value : value_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void testVectorInt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestVectorInt").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass testVectorIntObject::Class;
jfieldID testVectorIntObject::value_fieldID;

testVectorIntObject::testVectorIntObject()
  : value_()
{}

testVectorIntObject::testVectorIntObject(array<object_ptr<testInt>> &&value_)
  : value_(std::move(value_))
{}

const std::int32_t testVectorIntObject::ID;

object_ptr<testVectorIntObject> testVectorIntObject::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testVectorIntObject> res = make_object<testVectorIntObject>();
  res->value_ = jni::FetchVector<testInt>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void testVectorIntObject::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, value_); if (arr_tmp_) { env->SetObjectField(s, value_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void testVectorIntObject::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestVectorIntObject");
    { s.store_vector_begin("value", value_.size()); for (const auto &_value : value_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void testVectorIntObject::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestVectorIntObject").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TestInt;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testVectorString::Class;
jfieldID testVectorString::value_fieldID;

testVectorString::testVectorString()
  : value_()
{}

testVectorString::testVectorString(array<string> &&value_)
  : value_(std::move(value_))
{}

const std::int32_t testVectorString::ID;

object_ptr<testVectorString> testVectorString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testVectorString> res = make_object<testVectorString>();
  res->value_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void testVectorString::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, value_); if (arr_tmp_) { env->SetObjectField(s, value_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void testVectorString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestVectorString");
    { s.store_vector_begin("value", value_.size()); for (const auto &_value : value_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void testVectorString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestVectorString").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass testVectorStringObject::Class;
jfieldID testVectorStringObject::value_fieldID;

testVectorStringObject::testVectorStringObject()
  : value_()
{}

testVectorStringObject::testVectorStringObject(array<object_ptr<testString>> &&value_)
  : value_(std::move(value_))
{}

const std::int32_t testVectorStringObject::ID;

object_ptr<testVectorStringObject> testVectorStringObject::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testVectorStringObject> res = make_object<testVectorStringObject>();
  res->value_ = jni::FetchVector<testString>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void testVectorStringObject::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, value_); if (arr_tmp_) { env->SetObjectField(s, value_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void testVectorStringObject::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestVectorStringObject");
    { s.store_vector_begin("value", value_.size()); for (const auto &_value : value_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void testVectorStringObject::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestVectorStringObject").c_str());
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TestString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass text::Class;
jfieldID text::text_fieldID;

text::text()
  : text_()
{}

text::text(string const &text_)
  : text_(text_)
{}

const std::int32_t text::ID;

object_ptr<text> text::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<text> res = make_object<text>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void text::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, text_); if (nextString) { env->SetObjectField(s, text_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void text::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Text");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

void text::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Text").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass textEntities::Class;
jfieldID textEntities::entities_fieldID;

textEntities::textEntities()
  : entities_()
{}

textEntities::textEntities(array<object_ptr<textEntity>> &&entities_)
  : entities_(std::move(entities_))
{}

const std::int32_t textEntities::ID;

object_ptr<textEntities> textEntities::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntities> res = make_object<textEntities>();
  res->entities_ = jni::FetchVector<textEntity>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->entities_fieldID));
  return res;
}

void textEntities::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, entities_); if (arr_tmp_) { env->SetObjectField(s, entities_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void textEntities::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntities");
    { s.store_vector_begin("entities", entities_.size()); for (const auto &_value : entities_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void textEntities::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntities").c_str());
    entities_fieldID = jni::get_field_id(env, Class, "entities", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TextEntity;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntity::Class;
jfieldID textEntity::offset_fieldID;
jfieldID textEntity::length_fieldID;
jfieldID textEntity::type_fieldID;

textEntity::textEntity()
  : offset_()
  , length_()
  , type_()
{}

textEntity::textEntity(int32 offset_, int32 length_, object_ptr<TextEntityType> &&type_)
  : offset_(offset_)
  , length_(length_)
  , type_(std::move(type_))
{}

const std::int32_t textEntity::ID;

object_ptr<textEntity> textEntity::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntity> res = make_object<textEntity>();
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->length_ = env->GetIntField(p, res->length_fieldID);
  res->type_ = jni::fetch_tl_object<TextEntityType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void textEntity::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, offset_fieldID, offset_);
  env->SetIntField(s, length_fieldID, length_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void textEntity::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntity");
    s.store_field("offset", offset_);
    s.store_field("length", length_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void textEntity::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntity").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TextEntityType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass TextEntityType::Class;

object_ptr<TextEntityType> TextEntityType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case textEntityTypeMention::ID:
      return textEntityTypeMention::fetch(env, p);
    case textEntityTypeHashtag::ID:
      return textEntityTypeHashtag::fetch(env, p);
    case textEntityTypeCashtag::ID:
      return textEntityTypeCashtag::fetch(env, p);
    case textEntityTypeBotCommand::ID:
      return textEntityTypeBotCommand::fetch(env, p);
    case textEntityTypeUrl::ID:
      return textEntityTypeUrl::fetch(env, p);
    case textEntityTypeEmailAddress::ID:
      return textEntityTypeEmailAddress::fetch(env, p);
    case textEntityTypePhoneNumber::ID:
      return textEntityTypePhoneNumber::fetch(env, p);
    case textEntityTypeBankCardNumber::ID:
      return textEntityTypeBankCardNumber::fetch(env, p);
    case textEntityTypeBold::ID:
      return textEntityTypeBold::fetch(env, p);
    case textEntityTypeItalic::ID:
      return textEntityTypeItalic::fetch(env, p);
    case textEntityTypeUnderline::ID:
      return textEntityTypeUnderline::fetch(env, p);
    case textEntityTypeStrikethrough::ID:
      return textEntityTypeStrikethrough::fetch(env, p);
    case textEntityTypeSpoiler::ID:
      return textEntityTypeSpoiler::fetch(env, p);
    case textEntityTypeCode::ID:
      return textEntityTypeCode::fetch(env, p);
    case textEntityTypePre::ID:
      return textEntityTypePre::fetch(env, p);
    case textEntityTypePreCode::ID:
      return textEntityTypePreCode::fetch(env, p);
    case textEntityTypeBlockQuote::ID:
      return textEntityTypeBlockQuote::fetch(env, p);
    case textEntityTypeExpandableBlockQuote::ID:
      return textEntityTypeExpandableBlockQuote::fetch(env, p);
    case textEntityTypeTextUrl::ID:
      return textEntityTypeTextUrl::fetch(env, p);
    case textEntityTypeMentionName::ID:
      return textEntityTypeMentionName::fetch(env, p);
    case textEntityTypeCustomEmoji::ID:
      return textEntityTypeCustomEmoji::fetch(env, p);
    case textEntityTypeMediaTimestamp::ID:
      return textEntityTypeMediaTimestamp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TextEntityType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeMention::Class;

textEntityTypeMention::textEntityTypeMention() {
}

const std::int32_t textEntityTypeMention::ID;

object_ptr<TextEntityType> textEntityTypeMention::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeMention>();
}

void textEntityTypeMention::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeMention::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeMention");
    s.store_class_end();
  }
}

void textEntityTypeMention::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeMention").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeHashtag::Class;

textEntityTypeHashtag::textEntityTypeHashtag() {
}

const std::int32_t textEntityTypeHashtag::ID;

object_ptr<TextEntityType> textEntityTypeHashtag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeHashtag>();
}

void textEntityTypeHashtag::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeHashtag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeHashtag");
    s.store_class_end();
  }
}

void textEntityTypeHashtag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeHashtag").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeCashtag::Class;

textEntityTypeCashtag::textEntityTypeCashtag() {
}

const std::int32_t textEntityTypeCashtag::ID;

object_ptr<TextEntityType> textEntityTypeCashtag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeCashtag>();
}

void textEntityTypeCashtag::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeCashtag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeCashtag");
    s.store_class_end();
  }
}

void textEntityTypeCashtag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeCashtag").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeBotCommand::Class;

textEntityTypeBotCommand::textEntityTypeBotCommand() {
}

const std::int32_t textEntityTypeBotCommand::ID;

object_ptr<TextEntityType> textEntityTypeBotCommand::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeBotCommand>();
}

void textEntityTypeBotCommand::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeBotCommand::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeBotCommand");
    s.store_class_end();
  }
}

void textEntityTypeBotCommand::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeBotCommand").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeUrl::Class;

textEntityTypeUrl::textEntityTypeUrl() {
}

const std::int32_t textEntityTypeUrl::ID;

object_ptr<TextEntityType> textEntityTypeUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeUrl>();
}

void textEntityTypeUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeUrl");
    s.store_class_end();
  }
}

void textEntityTypeUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeUrl").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeEmailAddress::Class;

textEntityTypeEmailAddress::textEntityTypeEmailAddress() {
}

const std::int32_t textEntityTypeEmailAddress::ID;

object_ptr<TextEntityType> textEntityTypeEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeEmailAddress>();
}

void textEntityTypeEmailAddress::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeEmailAddress");
    s.store_class_end();
  }
}

void textEntityTypeEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeEmailAddress").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypePhoneNumber::Class;

textEntityTypePhoneNumber::textEntityTypePhoneNumber() {
}

const std::int32_t textEntityTypePhoneNumber::ID;

object_ptr<TextEntityType> textEntityTypePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypePhoneNumber>();
}

void textEntityTypePhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypePhoneNumber");
    s.store_class_end();
  }
}

void textEntityTypePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypePhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeBankCardNumber::Class;

textEntityTypeBankCardNumber::textEntityTypeBankCardNumber() {
}

const std::int32_t textEntityTypeBankCardNumber::ID;

object_ptr<TextEntityType> textEntityTypeBankCardNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeBankCardNumber>();
}

void textEntityTypeBankCardNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeBankCardNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeBankCardNumber");
    s.store_class_end();
  }
}

void textEntityTypeBankCardNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeBankCardNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeBold::Class;

textEntityTypeBold::textEntityTypeBold() {
}

const std::int32_t textEntityTypeBold::ID;

object_ptr<TextEntityType> textEntityTypeBold::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeBold>();
}

void textEntityTypeBold::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeBold::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeBold");
    s.store_class_end();
  }
}

void textEntityTypeBold::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeBold").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeItalic::Class;

textEntityTypeItalic::textEntityTypeItalic() {
}

const std::int32_t textEntityTypeItalic::ID;

object_ptr<TextEntityType> textEntityTypeItalic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeItalic>();
}

void textEntityTypeItalic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeItalic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeItalic");
    s.store_class_end();
  }
}

void textEntityTypeItalic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeItalic").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeUnderline::Class;

textEntityTypeUnderline::textEntityTypeUnderline() {
}

const std::int32_t textEntityTypeUnderline::ID;

object_ptr<TextEntityType> textEntityTypeUnderline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeUnderline>();
}

void textEntityTypeUnderline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeUnderline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeUnderline");
    s.store_class_end();
  }
}

void textEntityTypeUnderline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeUnderline").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeStrikethrough::Class;

textEntityTypeStrikethrough::textEntityTypeStrikethrough() {
}

const std::int32_t textEntityTypeStrikethrough::ID;

object_ptr<TextEntityType> textEntityTypeStrikethrough::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeStrikethrough>();
}

void textEntityTypeStrikethrough::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeStrikethrough::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeStrikethrough");
    s.store_class_end();
  }
}

void textEntityTypeStrikethrough::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeStrikethrough").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeSpoiler::Class;

textEntityTypeSpoiler::textEntityTypeSpoiler() {
}

const std::int32_t textEntityTypeSpoiler::ID;

object_ptr<TextEntityType> textEntityTypeSpoiler::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeSpoiler>();
}

void textEntityTypeSpoiler::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeSpoiler::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeSpoiler");
    s.store_class_end();
  }
}

void textEntityTypeSpoiler::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeSpoiler").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeCode::Class;

textEntityTypeCode::textEntityTypeCode() {
}

const std::int32_t textEntityTypeCode::ID;

object_ptr<TextEntityType> textEntityTypeCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeCode>();
}

void textEntityTypeCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeCode");
    s.store_class_end();
  }
}

void textEntityTypeCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypePre::Class;

textEntityTypePre::textEntityTypePre() {
}

const std::int32_t textEntityTypePre::ID;

object_ptr<TextEntityType> textEntityTypePre::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypePre>();
}

void textEntityTypePre::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypePre::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypePre");
    s.store_class_end();
  }
}

void textEntityTypePre::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypePre").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypePreCode::Class;
jfieldID textEntityTypePreCode::language_fieldID;

textEntityTypePreCode::textEntityTypePreCode()
  : language_()
{}

textEntityTypePreCode::textEntityTypePreCode(string const &language_)
  : language_(language_)
{}

const std::int32_t textEntityTypePreCode::ID;

object_ptr<TextEntityType> textEntityTypePreCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntityTypePreCode> res = make_object<textEntityTypePreCode>();
  res->language_ = jni::fetch_string(env, p, res->language_fieldID);
  return std::move(res);
}

void textEntityTypePreCode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, language_); if (nextString) { env->SetObjectField(s, language_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void textEntityTypePreCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypePreCode");
    s.store_field("language", language_);
    s.store_class_end();
  }
}

void textEntityTypePreCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypePreCode").c_str());
    language_fieldID = jni::get_field_id(env, Class, "language", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeBlockQuote::Class;

textEntityTypeBlockQuote::textEntityTypeBlockQuote() {
}

const std::int32_t textEntityTypeBlockQuote::ID;

object_ptr<TextEntityType> textEntityTypeBlockQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeBlockQuote>();
}

void textEntityTypeBlockQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeBlockQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeBlockQuote");
    s.store_class_end();
  }
}

void textEntityTypeBlockQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeBlockQuote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeExpandableBlockQuote::Class;

textEntityTypeExpandableBlockQuote::textEntityTypeExpandableBlockQuote() {
}

const std::int32_t textEntityTypeExpandableBlockQuote::ID;

object_ptr<TextEntityType> textEntityTypeExpandableBlockQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textEntityTypeExpandableBlockQuote>();
}

void textEntityTypeExpandableBlockQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void textEntityTypeExpandableBlockQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeExpandableBlockQuote");
    s.store_class_end();
  }
}

void textEntityTypeExpandableBlockQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeExpandableBlockQuote").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeTextUrl::Class;
jfieldID textEntityTypeTextUrl::url_fieldID;

textEntityTypeTextUrl::textEntityTypeTextUrl()
  : url_()
{}

textEntityTypeTextUrl::textEntityTypeTextUrl(string const &url_)
  : url_(url_)
{}

const std::int32_t textEntityTypeTextUrl::ID;

object_ptr<TextEntityType> textEntityTypeTextUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntityTypeTextUrl> res = make_object<textEntityTypeTextUrl>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return std::move(res);
}

void textEntityTypeTextUrl::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void textEntityTypeTextUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeTextUrl");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void textEntityTypeTextUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeTextUrl").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeMentionName::Class;
jfieldID textEntityTypeMentionName::user_id_fieldID;

textEntityTypeMentionName::textEntityTypeMentionName()
  : user_id_()
{}

textEntityTypeMentionName::textEntityTypeMentionName(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t textEntityTypeMentionName::ID;

object_ptr<TextEntityType> textEntityTypeMentionName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntityTypeMentionName> res = make_object<textEntityTypeMentionName>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return std::move(res);
}

void textEntityTypeMentionName::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
}

void textEntityTypeMentionName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeMentionName");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

void textEntityTypeMentionName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeMentionName").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeCustomEmoji::Class;
jfieldID textEntityTypeCustomEmoji::custom_emoji_id_fieldID;

textEntityTypeCustomEmoji::textEntityTypeCustomEmoji()
  : custom_emoji_id_()
{}

textEntityTypeCustomEmoji::textEntityTypeCustomEmoji(int64 custom_emoji_id_)
  : custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t textEntityTypeCustomEmoji::ID;

object_ptr<TextEntityType> textEntityTypeCustomEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntityTypeCustomEmoji> res = make_object<textEntityTypeCustomEmoji>();
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return std::move(res);
}

void textEntityTypeCustomEmoji::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, custom_emoji_id_fieldID, custom_emoji_id_);
}

void textEntityTypeCustomEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeCustomEmoji");
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

void textEntityTypeCustomEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeCustomEmoji").c_str());
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass textEntityTypeMediaTimestamp::Class;
jfieldID textEntityTypeMediaTimestamp::media_timestamp_fieldID;

textEntityTypeMediaTimestamp::textEntityTypeMediaTimestamp()
  : media_timestamp_()
{}

textEntityTypeMediaTimestamp::textEntityTypeMediaTimestamp(int32 media_timestamp_)
  : media_timestamp_(media_timestamp_)
{}

const std::int32_t textEntityTypeMediaTimestamp::ID;

object_ptr<TextEntityType> textEntityTypeMediaTimestamp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textEntityTypeMediaTimestamp> res = make_object<textEntityTypeMediaTimestamp>();
  res->media_timestamp_ = env->GetIntField(p, res->media_timestamp_fieldID);
  return std::move(res);
}

void textEntityTypeMediaTimestamp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, media_timestamp_fieldID, media_timestamp_);
}

void textEntityTypeMediaTimestamp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextEntityTypeMediaTimestamp");
    s.store_field("mediaTimestamp", media_timestamp_);
    s.store_class_end();
  }
}

void textEntityTypeMediaTimestamp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextEntityTypeMediaTimestamp").c_str());
    media_timestamp_fieldID = jni::get_field_id(env, Class, "mediaTimestamp", "I");
    return true;
  }();
  (void)is_inited;
}

jclass TextParseMode::Class;

object_ptr<TextParseMode> TextParseMode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case textParseModeMarkdown::ID:
      return textParseModeMarkdown::fetch(env, p);
    case textParseModeHTML::ID:
      return textParseModeHTML::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TextParseMode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextParseMode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textParseModeMarkdown::Class;
jfieldID textParseModeMarkdown::version_fieldID;

textParseModeMarkdown::textParseModeMarkdown()
  : version_()
{}

textParseModeMarkdown::textParseModeMarkdown(int32 version_)
  : version_(version_)
{}

const std::int32_t textParseModeMarkdown::ID;

object_ptr<TextParseMode> textParseModeMarkdown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textParseModeMarkdown> res = make_object<textParseModeMarkdown>();
  res->version_ = env->GetIntField(p, res->version_fieldID);
  return std::move(res);
}

void textParseModeMarkdown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextParseModeMarkdown");
    s.store_field("version", version_);
    s.store_class_end();
  }
}

void textParseModeMarkdown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextParseModeMarkdown").c_str());
    version_fieldID = jni::get_field_id(env, Class, "version", "I");
    return true;
  }();
  (void)is_inited;
}

jclass textParseModeHTML::Class;

textParseModeHTML::textParseModeHTML() {
}

const std::int32_t textParseModeHTML::ID;

object_ptr<TextParseMode> textParseModeHTML::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<textParseModeHTML>();
}

void textParseModeHTML::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextParseModeHTML");
    s.store_class_end();
  }
}

void textParseModeHTML::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextParseModeHTML").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass textQuote::Class;
jfieldID textQuote::text_fieldID;
jfieldID textQuote::position_fieldID;
jfieldID textQuote::is_manual_fieldID;

textQuote::textQuote()
  : text_()
  , position_()
  , is_manual_()
{}

textQuote::textQuote(object_ptr<formattedText> &&text_, int32 position_, bool is_manual_)
  : text_(std::move(text_))
  , position_(position_)
  , is_manual_(is_manual_)
{}

const std::int32_t textQuote::ID;

object_ptr<textQuote> textQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<textQuote> res = make_object<textQuote>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->position_ = env->GetIntField(p, res->position_fieldID);
  res->is_manual_ = (env->GetBooleanField(p, res->is_manual_fieldID) != 0);
  return res;
}

void textQuote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, position_fieldID, position_);
  env->SetBooleanField(s, is_manual_fieldID, is_manual_);
}

void textQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TextQuote");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("position", position_);
    s.store_field("isManual", is_manual_);
    s.store_class_end();
  }
}

void textQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TextQuote").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    is_manual_fieldID = jni::get_field_id(env, Class, "isManual", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass themeParameters::Class;
jfieldID themeParameters::background_color_fieldID;
jfieldID themeParameters::secondary_background_color_fieldID;
jfieldID themeParameters::header_background_color_fieldID;
jfieldID themeParameters::bottom_bar_background_color_fieldID;
jfieldID themeParameters::section_background_color_fieldID;
jfieldID themeParameters::section_separator_color_fieldID;
jfieldID themeParameters::text_color_fieldID;
jfieldID themeParameters::accent_text_color_fieldID;
jfieldID themeParameters::section_header_text_color_fieldID;
jfieldID themeParameters::subtitle_text_color_fieldID;
jfieldID themeParameters::destructive_text_color_fieldID;
jfieldID themeParameters::hint_color_fieldID;
jfieldID themeParameters::link_color_fieldID;
jfieldID themeParameters::button_color_fieldID;
jfieldID themeParameters::button_text_color_fieldID;

themeParameters::themeParameters()
  : background_color_()
  , secondary_background_color_()
  , header_background_color_()
  , bottom_bar_background_color_()
  , section_background_color_()
  , section_separator_color_()
  , text_color_()
  , accent_text_color_()
  , section_header_text_color_()
  , subtitle_text_color_()
  , destructive_text_color_()
  , hint_color_()
  , link_color_()
  , button_color_()
  , button_text_color_()
{}

themeParameters::themeParameters(int32 background_color_, int32 secondary_background_color_, int32 header_background_color_, int32 bottom_bar_background_color_, int32 section_background_color_, int32 section_separator_color_, int32 text_color_, int32 accent_text_color_, int32 section_header_text_color_, int32 subtitle_text_color_, int32 destructive_text_color_, int32 hint_color_, int32 link_color_, int32 button_color_, int32 button_text_color_)
  : background_color_(background_color_)
  , secondary_background_color_(secondary_background_color_)
  , header_background_color_(header_background_color_)
  , bottom_bar_background_color_(bottom_bar_background_color_)
  , section_background_color_(section_background_color_)
  , section_separator_color_(section_separator_color_)
  , text_color_(text_color_)
  , accent_text_color_(accent_text_color_)
  , section_header_text_color_(section_header_text_color_)
  , subtitle_text_color_(subtitle_text_color_)
  , destructive_text_color_(destructive_text_color_)
  , hint_color_(hint_color_)
  , link_color_(link_color_)
  , button_color_(button_color_)
  , button_text_color_(button_text_color_)
{}

const std::int32_t themeParameters::ID;

object_ptr<themeParameters> themeParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<themeParameters> res = make_object<themeParameters>();
  res->background_color_ = env->GetIntField(p, res->background_color_fieldID);
  res->secondary_background_color_ = env->GetIntField(p, res->secondary_background_color_fieldID);
  res->header_background_color_ = env->GetIntField(p, res->header_background_color_fieldID);
  res->bottom_bar_background_color_ = env->GetIntField(p, res->bottom_bar_background_color_fieldID);
  res->section_background_color_ = env->GetIntField(p, res->section_background_color_fieldID);
  res->section_separator_color_ = env->GetIntField(p, res->section_separator_color_fieldID);
  res->text_color_ = env->GetIntField(p, res->text_color_fieldID);
  res->accent_text_color_ = env->GetIntField(p, res->accent_text_color_fieldID);
  res->section_header_text_color_ = env->GetIntField(p, res->section_header_text_color_fieldID);
  res->subtitle_text_color_ = env->GetIntField(p, res->subtitle_text_color_fieldID);
  res->destructive_text_color_ = env->GetIntField(p, res->destructive_text_color_fieldID);
  res->hint_color_ = env->GetIntField(p, res->hint_color_fieldID);
  res->link_color_ = env->GetIntField(p, res->link_color_fieldID);
  res->button_color_ = env->GetIntField(p, res->button_color_fieldID);
  res->button_text_color_ = env->GetIntField(p, res->button_text_color_fieldID);
  return res;
}

void themeParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThemeParameters");
    s.store_field("backgroundColor", background_color_);
    s.store_field("secondaryBackgroundColor", secondary_background_color_);
    s.store_field("headerBackgroundColor", header_background_color_);
    s.store_field("bottomBarBackgroundColor", bottom_bar_background_color_);
    s.store_field("sectionBackgroundColor", section_background_color_);
    s.store_field("sectionSeparatorColor", section_separator_color_);
    s.store_field("textColor", text_color_);
    s.store_field("accentTextColor", accent_text_color_);
    s.store_field("sectionHeaderTextColor", section_header_text_color_);
    s.store_field("subtitleTextColor", subtitle_text_color_);
    s.store_field("destructiveTextColor", destructive_text_color_);
    s.store_field("hintColor", hint_color_);
    s.store_field("linkColor", link_color_);
    s.store_field("buttonColor", button_color_);
    s.store_field("buttonTextColor", button_text_color_);
    s.store_class_end();
  }
}

void themeParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThemeParameters").c_str());
    background_color_fieldID = jni::get_field_id(env, Class, "backgroundColor", "I");
    secondary_background_color_fieldID = jni::get_field_id(env, Class, "secondaryBackgroundColor", "I");
    header_background_color_fieldID = jni::get_field_id(env, Class, "headerBackgroundColor", "I");
    bottom_bar_background_color_fieldID = jni::get_field_id(env, Class, "bottomBarBackgroundColor", "I");
    section_background_color_fieldID = jni::get_field_id(env, Class, "sectionBackgroundColor", "I");
    section_separator_color_fieldID = jni::get_field_id(env, Class, "sectionSeparatorColor", "I");
    text_color_fieldID = jni::get_field_id(env, Class, "textColor", "I");
    accent_text_color_fieldID = jni::get_field_id(env, Class, "accentTextColor", "I");
    section_header_text_color_fieldID = jni::get_field_id(env, Class, "sectionHeaderTextColor", "I");
    subtitle_text_color_fieldID = jni::get_field_id(env, Class, "subtitleTextColor", "I");
    destructive_text_color_fieldID = jni::get_field_id(env, Class, "destructiveTextColor", "I");
    hint_color_fieldID = jni::get_field_id(env, Class, "hintColor", "I");
    link_color_fieldID = jni::get_field_id(env, Class, "linkColor", "I");
    button_color_fieldID = jni::get_field_id(env, Class, "buttonColor", "I");
    button_text_color_fieldID = jni::get_field_id(env, Class, "buttonTextColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass themeSettings::Class;
jfieldID themeSettings::accent_color_fieldID;
jfieldID themeSettings::background_fieldID;
jfieldID themeSettings::outgoing_message_fill_fieldID;
jfieldID themeSettings::animate_outgoing_message_fill_fieldID;
jfieldID themeSettings::outgoing_message_accent_color_fieldID;

themeSettings::themeSettings()
  : accent_color_()
  , background_()
  , outgoing_message_fill_()
  , animate_outgoing_message_fill_()
  , outgoing_message_accent_color_()
{}

themeSettings::themeSettings(int32 accent_color_, object_ptr<background> &&background_, object_ptr<BackgroundFill> &&outgoing_message_fill_, bool animate_outgoing_message_fill_, int32 outgoing_message_accent_color_)
  : accent_color_(accent_color_)
  , background_(std::move(background_))
  , outgoing_message_fill_(std::move(outgoing_message_fill_))
  , animate_outgoing_message_fill_(animate_outgoing_message_fill_)
  , outgoing_message_accent_color_(outgoing_message_accent_color_)
{}

const std::int32_t themeSettings::ID;

object_ptr<themeSettings> themeSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<themeSettings> res = make_object<themeSettings>();
  res->accent_color_ = env->GetIntField(p, res->accent_color_fieldID);
  res->background_ = jni::fetch_tl_object<background>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->outgoing_message_fill_ = jni::fetch_tl_object<BackgroundFill>(env, jni::fetch_object(env, p, res->outgoing_message_fill_fieldID));
  res->animate_outgoing_message_fill_ = (env->GetBooleanField(p, res->animate_outgoing_message_fill_fieldID) != 0);
  res->outgoing_message_accent_color_ = env->GetIntField(p, res->outgoing_message_accent_color_fieldID);
  return res;
}

void themeSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, accent_color_fieldID, accent_color_);
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
  if (outgoing_message_fill_ != nullptr) { jobject next; outgoing_message_fill_->store(env, next); if (next) { env->SetObjectField(s, outgoing_message_fill_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, animate_outgoing_message_fill_fieldID, animate_outgoing_message_fill_);
  env->SetIntField(s, outgoing_message_accent_color_fieldID, outgoing_message_accent_color_);
}

void themeSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThemeSettings");
    s.store_field("accentColor", accent_color_);
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_object_field("outgoingMessageFill", static_cast<const BaseObject *>(outgoing_message_fill_.get()));
    s.store_field("animateOutgoingMessageFill", animate_outgoing_message_fill_);
    s.store_field("outgoingMessageAccentColor", outgoing_message_accent_color_);
    s.store_class_end();
  }
}

void themeSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThemeSettings").c_str());
    accent_color_fieldID = jni::get_field_id(env, Class, "accentColor", "I");
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Background;").c_str());
    outgoing_message_fill_fieldID = jni::get_field_id(env, Class, "outgoingMessageFill", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundFill;").c_str());
    animate_outgoing_message_fill_fieldID = jni::get_field_id(env, Class, "animateOutgoingMessageFill", "Z");
    outgoing_message_accent_color_fieldID = jni::get_field_id(env, Class, "outgoingMessageAccentColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass thumbnail::Class;
jfieldID thumbnail::format_fieldID;
jfieldID thumbnail::width_fieldID;
jfieldID thumbnail::height_fieldID;
jfieldID thumbnail::file_fieldID;

thumbnail::thumbnail()
  : format_()
  , width_()
  , height_()
  , file_()
{}

thumbnail::thumbnail(object_ptr<ThumbnailFormat> &&format_, int32 width_, int32 height_, object_ptr<file> &&file_)
  : format_(std::move(format_))
  , width_(width_)
  , height_(height_)
  , file_(std::move(file_))
{}

const std::int32_t thumbnail::ID;

object_ptr<thumbnail> thumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<thumbnail> res = make_object<thumbnail>();
  res->format_ = jni::fetch_tl_object<ThumbnailFormat>(env, jni::fetch_object(env, p, res->format_fieldID));
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->file_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->file_fieldID));
  return res;
}

void thumbnail::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (format_ != nullptr) { jobject next; format_->store(env, next); if (next) { env->SetObjectField(s, format_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  if (file_ != nullptr) { jobject next; file_->store(env, next); if (next) { env->SetObjectField(s, file_fieldID, next); env->DeleteLocalRef(next); } }
}

void thumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Thumbnail");
    s.store_object_field("format", static_cast<const BaseObject *>(format_.get()));
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_object_field("file", static_cast<const BaseObject *>(file_.get()));
    s.store_class_end();
  }
}

void thumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Thumbnail").c_str());
    format_fieldID = jni::get_field_id(env, Class, "format", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThumbnailFormat;").c_str());
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    file_fieldID = jni::get_field_id(env, Class, "file", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass ThumbnailFormat::Class;

object_ptr<ThumbnailFormat> ThumbnailFormat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case thumbnailFormatJpeg::ID:
      return thumbnailFormatJpeg::fetch(env, p);
    case thumbnailFormatGif::ID:
      return thumbnailFormatGif::fetch(env, p);
    case thumbnailFormatMpeg4::ID:
      return thumbnailFormatMpeg4::fetch(env, p);
    case thumbnailFormatPng::ID:
      return thumbnailFormatPng::fetch(env, p);
    case thumbnailFormatTgs::ID:
      return thumbnailFormatTgs::fetch(env, p);
    case thumbnailFormatWebm::ID:
      return thumbnailFormatWebm::fetch(env, p);
    case thumbnailFormatWebp::ID:
      return thumbnailFormatWebp::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void ThumbnailFormat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormat").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatJpeg::Class;

thumbnailFormatJpeg::thumbnailFormatJpeg() {
}

const std::int32_t thumbnailFormatJpeg::ID;

object_ptr<ThumbnailFormat> thumbnailFormatJpeg::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatJpeg>();
}

void thumbnailFormatJpeg::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatJpeg::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatJpeg");
    s.store_class_end();
  }
}

void thumbnailFormatJpeg::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatJpeg").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatGif::Class;

thumbnailFormatGif::thumbnailFormatGif() {
}

const std::int32_t thumbnailFormatGif::ID;

object_ptr<ThumbnailFormat> thumbnailFormatGif::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatGif>();
}

void thumbnailFormatGif::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatGif::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatGif");
    s.store_class_end();
  }
}

void thumbnailFormatGif::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatGif").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatMpeg4::Class;

thumbnailFormatMpeg4::thumbnailFormatMpeg4() {
}

const std::int32_t thumbnailFormatMpeg4::ID;

object_ptr<ThumbnailFormat> thumbnailFormatMpeg4::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatMpeg4>();
}

void thumbnailFormatMpeg4::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatMpeg4::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatMpeg4");
    s.store_class_end();
  }
}

void thumbnailFormatMpeg4::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatMpeg4").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatPng::Class;

thumbnailFormatPng::thumbnailFormatPng() {
}

const std::int32_t thumbnailFormatPng::ID;

object_ptr<ThumbnailFormat> thumbnailFormatPng::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatPng>();
}

void thumbnailFormatPng::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatPng::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatPng");
    s.store_class_end();
  }
}

void thumbnailFormatPng::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatPng").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatTgs::Class;

thumbnailFormatTgs::thumbnailFormatTgs() {
}

const std::int32_t thumbnailFormatTgs::ID;

object_ptr<ThumbnailFormat> thumbnailFormatTgs::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatTgs>();
}

void thumbnailFormatTgs::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatTgs::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatTgs");
    s.store_class_end();
  }
}

void thumbnailFormatTgs::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatTgs").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatWebm::Class;

thumbnailFormatWebm::thumbnailFormatWebm() {
}

const std::int32_t thumbnailFormatWebm::ID;

object_ptr<ThumbnailFormat> thumbnailFormatWebm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatWebm>();
}

void thumbnailFormatWebm::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatWebm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatWebm");
    s.store_class_end();
  }
}

void thumbnailFormatWebm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatWebm").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass thumbnailFormatWebp::Class;

thumbnailFormatWebp::thumbnailFormatWebp() {
}

const std::int32_t thumbnailFormatWebp::ID;

object_ptr<ThumbnailFormat> thumbnailFormatWebp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<thumbnailFormatWebp>();
}

void thumbnailFormatWebp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void thumbnailFormatWebp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ThumbnailFormatWebp");
    s.store_class_end();
  }
}

void thumbnailFormatWebp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ThumbnailFormatWebp").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass timeZone::Class;
jfieldID timeZone::id_fieldID;
jfieldID timeZone::name_fieldID;
jfieldID timeZone::utc_time_offset_fieldID;

timeZone::timeZone()
  : id_()
  , name_()
  , utc_time_offset_()
{}

timeZone::timeZone(string const &id_, string const &name_, int32 utc_time_offset_)
  : id_(id_)
  , name_(name_)
  , utc_time_offset_(utc_time_offset_)
{}

const std::int32_t timeZone::ID;

object_ptr<timeZone> timeZone::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<timeZone> res = make_object<timeZone>();
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->utc_time_offset_ = env->GetIntField(p, res->utc_time_offset_fieldID);
  return res;
}

void timeZone::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, utc_time_offset_fieldID, utc_time_offset_);
}

void timeZone::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TimeZone");
    s.store_field("id", id_);
    s.store_field("name", name_);
    s.store_field("utcTimeOffset", utc_time_offset_);
    s.store_class_end();
  }
}

void timeZone::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TimeZone").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    utc_time_offset_fieldID = jni::get_field_id(env, Class, "utcTimeOffset", "I");
    return true;
  }();
  (void)is_inited;
}

jclass timeZones::Class;
jfieldID timeZones::time_zones_fieldID;

timeZones::timeZones()
  : time_zones_()
{}

timeZones::timeZones(array<object_ptr<timeZone>> &&time_zones_)
  : time_zones_(std::move(time_zones_))
{}

const std::int32_t timeZones::ID;

object_ptr<timeZones> timeZones::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<timeZones> res = make_object<timeZones>();
  res->time_zones_ = jni::FetchVector<timeZone>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->time_zones_fieldID));
  return res;
}

void timeZones::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, time_zones_); if (arr_tmp_) { env->SetObjectField(s, time_zones_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void timeZones::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TimeZones");
    { s.store_vector_begin("timeZones", time_zones_.size()); for (const auto &_value : time_zones_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void timeZones::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TimeZones").c_str());
    time_zones_fieldID = jni::get_field_id(env, Class, "timeZones", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TimeZone;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass TopChatCategory::Class;

object_ptr<TopChatCategory> TopChatCategory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case topChatCategoryUsers::ID:
      return topChatCategoryUsers::fetch(env, p);
    case topChatCategoryBots::ID:
      return topChatCategoryBots::fetch(env, p);
    case topChatCategoryGroups::ID:
      return topChatCategoryGroups::fetch(env, p);
    case topChatCategoryChannels::ID:
      return topChatCategoryChannels::fetch(env, p);
    case topChatCategoryInlineBots::ID:
      return topChatCategoryInlineBots::fetch(env, p);
    case topChatCategoryWebAppBots::ID:
      return topChatCategoryWebAppBots::fetch(env, p);
    case topChatCategoryCalls::ID:
      return topChatCategoryCalls::fetch(env, p);
    case topChatCategoryForwardChats::ID:
      return topChatCategoryForwardChats::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void TopChatCategory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategory").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryUsers::Class;

topChatCategoryUsers::topChatCategoryUsers() {
}

const std::int32_t topChatCategoryUsers::ID;

object_ptr<TopChatCategory> topChatCategoryUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryUsers>();
}

void topChatCategoryUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryUsers");
    s.store_class_end();
  }
}

void topChatCategoryUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryUsers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryBots::Class;

topChatCategoryBots::topChatCategoryBots() {
}

const std::int32_t topChatCategoryBots::ID;

object_ptr<TopChatCategory> topChatCategoryBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryBots>();
}

void topChatCategoryBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryBots");
    s.store_class_end();
  }
}

void topChatCategoryBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryGroups::Class;

topChatCategoryGroups::topChatCategoryGroups() {
}

const std::int32_t topChatCategoryGroups::ID;

object_ptr<TopChatCategory> topChatCategoryGroups::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryGroups>();
}

void topChatCategoryGroups::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryGroups");
    s.store_class_end();
  }
}

void topChatCategoryGroups::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryGroups").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryChannels::Class;

topChatCategoryChannels::topChatCategoryChannels() {
}

const std::int32_t topChatCategoryChannels::ID;

object_ptr<TopChatCategory> topChatCategoryChannels::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryChannels>();
}

void topChatCategoryChannels::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryChannels");
    s.store_class_end();
  }
}

void topChatCategoryChannels::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryChannels").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryInlineBots::Class;

topChatCategoryInlineBots::topChatCategoryInlineBots() {
}

const std::int32_t topChatCategoryInlineBots::ID;

object_ptr<TopChatCategory> topChatCategoryInlineBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryInlineBots>();
}

void topChatCategoryInlineBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryInlineBots");
    s.store_class_end();
  }
}

void topChatCategoryInlineBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryInlineBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryWebAppBots::Class;

topChatCategoryWebAppBots::topChatCategoryWebAppBots() {
}

const std::int32_t topChatCategoryWebAppBots::ID;

object_ptr<TopChatCategory> topChatCategoryWebAppBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryWebAppBots>();
}

void topChatCategoryWebAppBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryWebAppBots");
    s.store_class_end();
  }
}

void topChatCategoryWebAppBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryWebAppBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryCalls::Class;

topChatCategoryCalls::topChatCategoryCalls() {
}

const std::int32_t topChatCategoryCalls::ID;

object_ptr<TopChatCategory> topChatCategoryCalls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryCalls>();
}

void topChatCategoryCalls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryCalls");
    s.store_class_end();
  }
}

void topChatCategoryCalls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryCalls").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass topChatCategoryForwardChats::Class;

topChatCategoryForwardChats::topChatCategoryForwardChats() {
}

const std::int32_t topChatCategoryForwardChats::ID;

object_ptr<TopChatCategory> topChatCategoryForwardChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<topChatCategoryForwardChats>();
}

void topChatCategoryForwardChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TopChatCategoryForwardChats");
    s.store_class_end();
  }
}

void topChatCategoryForwardChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TopChatCategoryForwardChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass trendingStickerSets::Class;
jfieldID trendingStickerSets::total_count_fieldID;
jfieldID trendingStickerSets::sets_fieldID;
jfieldID trendingStickerSets::is_premium_fieldID;

trendingStickerSets::trendingStickerSets()
  : total_count_()
  , sets_()
  , is_premium_()
{}

trendingStickerSets::trendingStickerSets(int32 total_count_, array<object_ptr<stickerSetInfo>> &&sets_, bool is_premium_)
  : total_count_(total_count_)
  , sets_(std::move(sets_))
  , is_premium_(is_premium_)
{}

const std::int32_t trendingStickerSets::ID;

object_ptr<trendingStickerSets> trendingStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<trendingStickerSets> res = make_object<trendingStickerSets>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->sets_ = jni::FetchVector<stickerSetInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->sets_fieldID));
  res->is_premium_ = (env->GetBooleanField(p, res->is_premium_fieldID) != 0);
  return res;
}

void trendingStickerSets::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, sets_); if (arr_tmp_) { env->SetObjectField(s, sets_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_premium_fieldID, is_premium_);
}

void trendingStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TrendingStickerSets");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("sets", sets_.size()); for (const auto &_value : sets_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isPremium", is_premium_);
    s.store_class_end();
  }
}

void trendingStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TrendingStickerSets").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    sets_fieldID = jni::get_field_id(env, Class, "sets", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$StickerSetInfo;").c_str());
    is_premium_fieldID = jni::get_field_id(env, Class, "isPremium", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass unconfirmedSession::Class;
jfieldID unconfirmedSession::id_fieldID;
jfieldID unconfirmedSession::log_in_date_fieldID;
jfieldID unconfirmedSession::device_model_fieldID;
jfieldID unconfirmedSession::location_fieldID;

unconfirmedSession::unconfirmedSession()
  : id_()
  , log_in_date_()
  , device_model_()
  , location_()
{}

unconfirmedSession::unconfirmedSession(int64 id_, int32 log_in_date_, string const &device_model_, string const &location_)
  : id_(id_)
  , log_in_date_(log_in_date_)
  , device_model_(device_model_)
  , location_(location_)
{}

const std::int32_t unconfirmedSession::ID;

object_ptr<unconfirmedSession> unconfirmedSession::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<unconfirmedSession> res = make_object<unconfirmedSession>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->log_in_date_ = env->GetIntField(p, res->log_in_date_fieldID);
  res->device_model_ = jni::fetch_string(env, p, res->device_model_fieldID);
  res->location_ = jni::fetch_string(env, p, res->location_fieldID);
  return res;
}

void unconfirmedSession::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetIntField(s, log_in_date_fieldID, log_in_date_);
  { jstring nextString = jni::to_jstring(env, device_model_); if (nextString) { env->SetObjectField(s, device_model_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, location_); if (nextString) { env->SetObjectField(s, location_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void unconfirmedSession::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UnconfirmedSession");
    s.store_field("id", id_);
    s.store_field("logInDate", log_in_date_);
    s.store_field("deviceModel", device_model_);
    s.store_field("location", location_);
    s.store_class_end();
  }
}

void unconfirmedSession::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UnconfirmedSession").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    log_in_date_fieldID = jni::get_field_id(env, Class, "logInDate", "I");
    device_model_fieldID = jni::get_field_id(env, Class, "deviceModel", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass unreadReaction::Class;
jfieldID unreadReaction::type_fieldID;
jfieldID unreadReaction::sender_id_fieldID;
jfieldID unreadReaction::is_big_fieldID;

unreadReaction::unreadReaction()
  : type_()
  , sender_id_()
  , is_big_()
{}

unreadReaction::unreadReaction(object_ptr<ReactionType> &&type_, object_ptr<MessageSender> &&sender_id_, bool is_big_)
  : type_(std::move(type_))
  , sender_id_(std::move(sender_id_))
  , is_big_(is_big_)
{}

const std::int32_t unreadReaction::ID;

object_ptr<unreadReaction> unreadReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<unreadReaction> res = make_object<unreadReaction>();
  res->type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->is_big_ = (env->GetBooleanField(p, res->is_big_fieldID) != 0);
  return res;
}

void unreadReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_big_fieldID, is_big_);
}

void unreadReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UnreadReaction");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("isBig", is_big_);
    s.store_class_end();
  }
}

void unreadReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UnreadReaction").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_big_fieldID = jni::get_field_id(env, Class, "isBig", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass Update::Class;

object_ptr<Update> Update::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case updateAuthorizationState::ID:
      return updateAuthorizationState::fetch(env, p);
    case updateNewMessage::ID:
      return updateNewMessage::fetch(env, p);
    case updateMessageSendAcknowledged::ID:
      return updateMessageSendAcknowledged::fetch(env, p);
    case updateMessageSendSucceeded::ID:
      return updateMessageSendSucceeded::fetch(env, p);
    case updateMessageSendFailed::ID:
      return updateMessageSendFailed::fetch(env, p);
    case updateMessageContent::ID:
      return updateMessageContent::fetch(env, p);
    case updateMessageEdited::ID:
      return updateMessageEdited::fetch(env, p);
    case updateMessageIsPinned::ID:
      return updateMessageIsPinned::fetch(env, p);
    case updateMessageInteractionInfo::ID:
      return updateMessageInteractionInfo::fetch(env, p);
    case updateMessageContentOpened::ID:
      return updateMessageContentOpened::fetch(env, p);
    case updateMessageMentionRead::ID:
      return updateMessageMentionRead::fetch(env, p);
    case updateMessageUnreadReactions::ID:
      return updateMessageUnreadReactions::fetch(env, p);
    case updateMessageFactCheck::ID:
      return updateMessageFactCheck::fetch(env, p);
    case updateMessageLiveLocationViewed::ID:
      return updateMessageLiveLocationViewed::fetch(env, p);
    case updateVideoPublished::ID:
      return updateVideoPublished::fetch(env, p);
    case updateNewChat::ID:
      return updateNewChat::fetch(env, p);
    case updateChatTitle::ID:
      return updateChatTitle::fetch(env, p);
    case updateChatPhoto::ID:
      return updateChatPhoto::fetch(env, p);
    case updateChatAccentColors::ID:
      return updateChatAccentColors::fetch(env, p);
    case updateChatPermissions::ID:
      return updateChatPermissions::fetch(env, p);
    case updateChatLastMessage::ID:
      return updateChatLastMessage::fetch(env, p);
    case updateChatPosition::ID:
      return updateChatPosition::fetch(env, p);
    case updateChatAddedToList::ID:
      return updateChatAddedToList::fetch(env, p);
    case updateChatRemovedFromList::ID:
      return updateChatRemovedFromList::fetch(env, p);
    case updateChatReadInbox::ID:
      return updateChatReadInbox::fetch(env, p);
    case updateChatReadOutbox::ID:
      return updateChatReadOutbox::fetch(env, p);
    case updateChatActionBar::ID:
      return updateChatActionBar::fetch(env, p);
    case updateChatBusinessBotManageBar::ID:
      return updateChatBusinessBotManageBar::fetch(env, p);
    case updateChatAvailableReactions::ID:
      return updateChatAvailableReactions::fetch(env, p);
    case updateChatDraftMessage::ID:
      return updateChatDraftMessage::fetch(env, p);
    case updateChatEmojiStatus::ID:
      return updateChatEmojiStatus::fetch(env, p);
    case updateChatMessageSender::ID:
      return updateChatMessageSender::fetch(env, p);
    case updateChatMessageAutoDeleteTime::ID:
      return updateChatMessageAutoDeleteTime::fetch(env, p);
    case updateChatNotificationSettings::ID:
      return updateChatNotificationSettings::fetch(env, p);
    case updateChatPendingJoinRequests::ID:
      return updateChatPendingJoinRequests::fetch(env, p);
    case updateChatReplyMarkup::ID:
      return updateChatReplyMarkup::fetch(env, p);
    case updateChatBackground::ID:
      return updateChatBackground::fetch(env, p);
    case updateChatTheme::ID:
      return updateChatTheme::fetch(env, p);
    case updateChatUnreadMentionCount::ID:
      return updateChatUnreadMentionCount::fetch(env, p);
    case updateChatUnreadReactionCount::ID:
      return updateChatUnreadReactionCount::fetch(env, p);
    case updateChatVideoChat::ID:
      return updateChatVideoChat::fetch(env, p);
    case updateChatDefaultDisableNotification::ID:
      return updateChatDefaultDisableNotification::fetch(env, p);
    case updateChatHasProtectedContent::ID:
      return updateChatHasProtectedContent::fetch(env, p);
    case updateChatIsTranslatable::ID:
      return updateChatIsTranslatable::fetch(env, p);
    case updateChatIsMarkedAsUnread::ID:
      return updateChatIsMarkedAsUnread::fetch(env, p);
    case updateChatViewAsTopics::ID:
      return updateChatViewAsTopics::fetch(env, p);
    case updateChatBlockList::ID:
      return updateChatBlockList::fetch(env, p);
    case updateChatHasScheduledMessages::ID:
      return updateChatHasScheduledMessages::fetch(env, p);
    case updateChatFolders::ID:
      return updateChatFolders::fetch(env, p);
    case updateChatOnlineMemberCount::ID:
      return updateChatOnlineMemberCount::fetch(env, p);
    case updateSavedMessagesTopic::ID:
      return updateSavedMessagesTopic::fetch(env, p);
    case updateSavedMessagesTopicCount::ID:
      return updateSavedMessagesTopicCount::fetch(env, p);
    case updateQuickReplyShortcut::ID:
      return updateQuickReplyShortcut::fetch(env, p);
    case updateQuickReplyShortcutDeleted::ID:
      return updateQuickReplyShortcutDeleted::fetch(env, p);
    case updateQuickReplyShortcuts::ID:
      return updateQuickReplyShortcuts::fetch(env, p);
    case updateQuickReplyShortcutMessages::ID:
      return updateQuickReplyShortcutMessages::fetch(env, p);
    case updateForumTopicInfo::ID:
      return updateForumTopicInfo::fetch(env, p);
    case updateForumTopic::ID:
      return updateForumTopic::fetch(env, p);
    case updateScopeNotificationSettings::ID:
      return updateScopeNotificationSettings::fetch(env, p);
    case updateReactionNotificationSettings::ID:
      return updateReactionNotificationSettings::fetch(env, p);
    case updateNotification::ID:
      return updateNotification::fetch(env, p);
    case updateNotificationGroup::ID:
      return updateNotificationGroup::fetch(env, p);
    case updateActiveNotifications::ID:
      return updateActiveNotifications::fetch(env, p);
    case updateHavePendingNotifications::ID:
      return updateHavePendingNotifications::fetch(env, p);
    case updateDeleteMessages::ID:
      return updateDeleteMessages::fetch(env, p);
    case updateChatAction::ID:
      return updateChatAction::fetch(env, p);
    case updateUserStatus::ID:
      return updateUserStatus::fetch(env, p);
    case updateUser::ID:
      return updateUser::fetch(env, p);
    case updateBasicGroup::ID:
      return updateBasicGroup::fetch(env, p);
    case updateSupergroup::ID:
      return updateSupergroup::fetch(env, p);
    case updateSecretChat::ID:
      return updateSecretChat::fetch(env, p);
    case updateUserFullInfo::ID:
      return updateUserFullInfo::fetch(env, p);
    case updateBasicGroupFullInfo::ID:
      return updateBasicGroupFullInfo::fetch(env, p);
    case updateSupergroupFullInfo::ID:
      return updateSupergroupFullInfo::fetch(env, p);
    case updateServiceNotification::ID:
      return updateServiceNotification::fetch(env, p);
    case updateFile::ID:
      return updateFile::fetch(env, p);
    case updateFileGenerationStart::ID:
      return updateFileGenerationStart::fetch(env, p);
    case updateFileGenerationStop::ID:
      return updateFileGenerationStop::fetch(env, p);
    case updateFileDownloads::ID:
      return updateFileDownloads::fetch(env, p);
    case updateFileAddedToDownloads::ID:
      return updateFileAddedToDownloads::fetch(env, p);
    case updateFileDownload::ID:
      return updateFileDownload::fetch(env, p);
    case updateFileRemovedFromDownloads::ID:
      return updateFileRemovedFromDownloads::fetch(env, p);
    case updateApplicationVerificationRequired::ID:
      return updateApplicationVerificationRequired::fetch(env, p);
    case updateApplicationRecaptchaVerificationRequired::ID:
      return updateApplicationRecaptchaVerificationRequired::fetch(env, p);
    case updateCall::ID:
      return updateCall::fetch(env, p);
    case updateGroupCall::ID:
      return updateGroupCall::fetch(env, p);
    case updateGroupCallParticipant::ID:
      return updateGroupCallParticipant::fetch(env, p);
    case updateGroupCallParticipants::ID:
      return updateGroupCallParticipants::fetch(env, p);
    case updateGroupCallVerificationState::ID:
      return updateGroupCallVerificationState::fetch(env, p);
    case updateNewCallSignalingData::ID:
      return updateNewCallSignalingData::fetch(env, p);
    case updateUserPrivacySettingRules::ID:
      return updateUserPrivacySettingRules::fetch(env, p);
    case updateUnreadMessageCount::ID:
      return updateUnreadMessageCount::fetch(env, p);
    case updateUnreadChatCount::ID:
      return updateUnreadChatCount::fetch(env, p);
    case updateStory::ID:
      return updateStory::fetch(env, p);
    case updateStoryDeleted::ID:
      return updateStoryDeleted::fetch(env, p);
    case updateStoryPostSucceeded::ID:
      return updateStoryPostSucceeded::fetch(env, p);
    case updateStoryPostFailed::ID:
      return updateStoryPostFailed::fetch(env, p);
    case updateChatActiveStories::ID:
      return updateChatActiveStories::fetch(env, p);
    case updateStoryListChatCount::ID:
      return updateStoryListChatCount::fetch(env, p);
    case updateStoryStealthMode::ID:
      return updateStoryStealthMode::fetch(env, p);
    case updateOption::ID:
      return updateOption::fetch(env, p);
    case updateStickerSet::ID:
      return updateStickerSet::fetch(env, p);
    case updateInstalledStickerSets::ID:
      return updateInstalledStickerSets::fetch(env, p);
    case updateTrendingStickerSets::ID:
      return updateTrendingStickerSets::fetch(env, p);
    case updateRecentStickers::ID:
      return updateRecentStickers::fetch(env, p);
    case updateFavoriteStickers::ID:
      return updateFavoriteStickers::fetch(env, p);
    case updateSavedAnimations::ID:
      return updateSavedAnimations::fetch(env, p);
    case updateSavedNotificationSounds::ID:
      return updateSavedNotificationSounds::fetch(env, p);
    case updateDefaultBackground::ID:
      return updateDefaultBackground::fetch(env, p);
    case updateChatThemes::ID:
      return updateChatThemes::fetch(env, p);
    case updateAccentColors::ID:
      return updateAccentColors::fetch(env, p);
    case updateProfileAccentColors::ID:
      return updateProfileAccentColors::fetch(env, p);
    case updateLanguagePackStrings::ID:
      return updateLanguagePackStrings::fetch(env, p);
    case updateConnectionState::ID:
      return updateConnectionState::fetch(env, p);
    case updateFreezeState::ID:
      return updateFreezeState::fetch(env, p);
    case updateTermsOfService::ID:
      return updateTermsOfService::fetch(env, p);
    case updateUnconfirmedSession::ID:
      return updateUnconfirmedSession::fetch(env, p);
    case updateAttachmentMenuBots::ID:
      return updateAttachmentMenuBots::fetch(env, p);
    case updateWebAppMessageSent::ID:
      return updateWebAppMessageSent::fetch(env, p);
    case updateActiveEmojiReactions::ID:
      return updateActiveEmojiReactions::fetch(env, p);
    case updateAvailableMessageEffects::ID:
      return updateAvailableMessageEffects::fetch(env, p);
    case updateDefaultReactionType::ID:
      return updateDefaultReactionType::fetch(env, p);
    case updateDefaultPaidReactionType::ID:
      return updateDefaultPaidReactionType::fetch(env, p);
    case updateSavedMessagesTags::ID:
      return updateSavedMessagesTags::fetch(env, p);
    case updateActiveLiveLocationMessages::ID:
      return updateActiveLiveLocationMessages::fetch(env, p);
    case updateOwnedStarCount::ID:
      return updateOwnedStarCount::fetch(env, p);
    case updateChatRevenueAmount::ID:
      return updateChatRevenueAmount::fetch(env, p);
    case updateStarRevenueStatus::ID:
      return updateStarRevenueStatus::fetch(env, p);
    case updateSpeechRecognitionTrial::ID:
      return updateSpeechRecognitionTrial::fetch(env, p);
    case updateDiceEmojis::ID:
      return updateDiceEmojis::fetch(env, p);
    case updateAnimatedEmojiMessageClicked::ID:
      return updateAnimatedEmojiMessageClicked::fetch(env, p);
    case updateAnimationSearchParameters::ID:
      return updateAnimationSearchParameters::fetch(env, p);
    case updateSuggestedActions::ID:
      return updateSuggestedActions::fetch(env, p);
    case updateSpeedLimitNotification::ID:
      return updateSpeedLimitNotification::fetch(env, p);
    case updateContactCloseBirthdays::ID:
      return updateContactCloseBirthdays::fetch(env, p);
    case updateAutosaveSettings::ID:
      return updateAutosaveSettings::fetch(env, p);
    case updateBusinessConnection::ID:
      return updateBusinessConnection::fetch(env, p);
    case updateNewBusinessMessage::ID:
      return updateNewBusinessMessage::fetch(env, p);
    case updateBusinessMessageEdited::ID:
      return updateBusinessMessageEdited::fetch(env, p);
    case updateBusinessMessagesDeleted::ID:
      return updateBusinessMessagesDeleted::fetch(env, p);
    case updateNewInlineQuery::ID:
      return updateNewInlineQuery::fetch(env, p);
    case updateNewChosenInlineResult::ID:
      return updateNewChosenInlineResult::fetch(env, p);
    case updateNewCallbackQuery::ID:
      return updateNewCallbackQuery::fetch(env, p);
    case updateNewInlineCallbackQuery::ID:
      return updateNewInlineCallbackQuery::fetch(env, p);
    case updateNewBusinessCallbackQuery::ID:
      return updateNewBusinessCallbackQuery::fetch(env, p);
    case updateNewShippingQuery::ID:
      return updateNewShippingQuery::fetch(env, p);
    case updateNewPreCheckoutQuery::ID:
      return updateNewPreCheckoutQuery::fetch(env, p);
    case updateNewCustomEvent::ID:
      return updateNewCustomEvent::fetch(env, p);
    case updateNewCustomQuery::ID:
      return updateNewCustomQuery::fetch(env, p);
    case updatePoll::ID:
      return updatePoll::fetch(env, p);
    case updatePollAnswer::ID:
      return updatePollAnswer::fetch(env, p);
    case updateChatMember::ID:
      return updateChatMember::fetch(env, p);
    case updateNewChatJoinRequest::ID:
      return updateNewChatJoinRequest::fetch(env, p);
    case updateChatBoost::ID:
      return updateChatBoost::fetch(env, p);
    case updateMessageReaction::ID:
      return updateMessageReaction::fetch(env, p);
    case updateMessageReactions::ID:
      return updateMessageReactions::fetch(env, p);
    case updatePaidMediaPurchased::ID:
      return updatePaidMediaPurchased::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void Update::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Update").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateAuthorizationState::Class;
jfieldID updateAuthorizationState::authorization_state_fieldID;

updateAuthorizationState::updateAuthorizationState()
  : authorization_state_()
{}

updateAuthorizationState::updateAuthorizationState(object_ptr<AuthorizationState> &&authorization_state_)
  : authorization_state_(std::move(authorization_state_))
{}

const std::int32_t updateAuthorizationState::ID;

object_ptr<Update> updateAuthorizationState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAuthorizationState> res = make_object<updateAuthorizationState>();
  res->authorization_state_ = jni::fetch_tl_object<AuthorizationState>(env, jni::fetch_object(env, p, res->authorization_state_fieldID));
  return std::move(res);
}

void updateAuthorizationState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (authorization_state_ != nullptr) { jobject next; authorization_state_->store(env, next); if (next) { env->SetObjectField(s, authorization_state_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateAuthorizationState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAuthorizationState");
    s.store_object_field("authorizationState", static_cast<const BaseObject *>(authorization_state_.get()));
    s.store_class_end();
  }
}

void updateAuthorizationState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAuthorizationState").c_str());
    authorization_state_fieldID = jni::get_field_id(env, Class, "authorizationState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AuthorizationState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewMessage::Class;
jfieldID updateNewMessage::message_fieldID;

updateNewMessage::updateNewMessage()
  : message_()
{}

updateNewMessage::updateNewMessage(object_ptr<message> &&message_)
  : message_(std::move(message_))
{}

const std::int32_t updateNewMessage::ID;

object_ptr<Update> updateNewMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewMessage> res = make_object<updateNewMessage>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void updateNewMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewMessage");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void updateNewMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewMessage").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageSendAcknowledged::Class;
jfieldID updateMessageSendAcknowledged::chat_id_fieldID;
jfieldID updateMessageSendAcknowledged::message_id_fieldID;

updateMessageSendAcknowledged::updateMessageSendAcknowledged()
  : chat_id_()
  , message_id_()
{}

updateMessageSendAcknowledged::updateMessageSendAcknowledged(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t updateMessageSendAcknowledged::ID;

object_ptr<Update> updateMessageSendAcknowledged::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageSendAcknowledged> res = make_object<updateMessageSendAcknowledged>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void updateMessageSendAcknowledged::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void updateMessageSendAcknowledged::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageSendAcknowledged");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void updateMessageSendAcknowledged::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageSendAcknowledged").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageSendSucceeded::Class;
jfieldID updateMessageSendSucceeded::message_fieldID;
jfieldID updateMessageSendSucceeded::old_message_id_fieldID;

updateMessageSendSucceeded::updateMessageSendSucceeded()
  : message_()
  , old_message_id_()
{}

updateMessageSendSucceeded::updateMessageSendSucceeded(object_ptr<message> &&message_, int53 old_message_id_)
  : message_(std::move(message_))
  , old_message_id_(old_message_id_)
{}

const std::int32_t updateMessageSendSucceeded::ID;

object_ptr<Update> updateMessageSendSucceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageSendSucceeded> res = make_object<updateMessageSendSucceeded>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->old_message_id_ = env->GetLongField(p, res->old_message_id_fieldID);
  return std::move(res);
}

void updateMessageSendSucceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, old_message_id_fieldID, old_message_id_);
}

void updateMessageSendSucceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageSendSucceeded");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("oldMessageId", old_message_id_);
    s.store_class_end();
  }
}

void updateMessageSendSucceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageSendSucceeded").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    old_message_id_fieldID = jni::get_field_id(env, Class, "oldMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageSendFailed::Class;
jfieldID updateMessageSendFailed::message_fieldID;
jfieldID updateMessageSendFailed::old_message_id_fieldID;
jfieldID updateMessageSendFailed::error_fieldID;

updateMessageSendFailed::updateMessageSendFailed()
  : message_()
  , old_message_id_()
  , error_()
{}

updateMessageSendFailed::updateMessageSendFailed(object_ptr<message> &&message_, int53 old_message_id_, object_ptr<error> &&error_)
  : message_(std::move(message_))
  , old_message_id_(old_message_id_)
  , error_(std::move(error_))
{}

const std::int32_t updateMessageSendFailed::ID;

object_ptr<Update> updateMessageSendFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageSendFailed> res = make_object<updateMessageSendFailed>();
  res->message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->old_message_id_ = env->GetLongField(p, res->old_message_id_fieldID);
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  return std::move(res);
}

void updateMessageSendFailed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, old_message_id_fieldID, old_message_id_);
  if (error_ != nullptr) { jobject next; error_->store(env, next); if (next) { env->SetObjectField(s, error_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateMessageSendFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageSendFailed");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("oldMessageId", old_message_id_);
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_class_end();
  }
}

void updateMessageSendFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageSendFailed").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    old_message_id_fieldID = jni::get_field_id(env, Class, "oldMessageId", "J");
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageContent::Class;
jfieldID updateMessageContent::chat_id_fieldID;
jfieldID updateMessageContent::message_id_fieldID;
jfieldID updateMessageContent::new_content_fieldID;

updateMessageContent::updateMessageContent()
  : chat_id_()
  , message_id_()
  , new_content_()
{}

updateMessageContent::updateMessageContent(int53 chat_id_, int53 message_id_, object_ptr<MessageContent> &&new_content_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , new_content_(std::move(new_content_))
{}

const std::int32_t updateMessageContent::ID;

object_ptr<Update> updateMessageContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageContent> res = make_object<updateMessageContent>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->new_content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->new_content_fieldID));
  return std::move(res);
}

void updateMessageContent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (new_content_ != nullptr) { jobject next; new_content_->store(env, next); if (next) { env->SetObjectField(s, new_content_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateMessageContent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageContent");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("newContent", static_cast<const BaseObject *>(new_content_.get()));
    s.store_class_end();
  }
}

void updateMessageContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageContent").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    new_content_fieldID = jni::get_field_id(env, Class, "newContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageEdited::Class;
jfieldID updateMessageEdited::chat_id_fieldID;
jfieldID updateMessageEdited::message_id_fieldID;
jfieldID updateMessageEdited::edit_date_fieldID;
jfieldID updateMessageEdited::reply_markup_fieldID;

updateMessageEdited::updateMessageEdited()
  : chat_id_()
  , message_id_()
  , edit_date_()
  , reply_markup_()
{}

updateMessageEdited::updateMessageEdited(int53 chat_id_, int53 message_id_, int32 edit_date_, object_ptr<ReplyMarkup> &&reply_markup_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , edit_date_(edit_date_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t updateMessageEdited::ID;

object_ptr<Update> updateMessageEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageEdited> res = make_object<updateMessageEdited>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->edit_date_ = env->GetIntField(p, res->edit_date_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return std::move(res);
}

void updateMessageEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetIntField(s, edit_date_fieldID, edit_date_);
  if (reply_markup_ != nullptr) { jobject next; reply_markup_->store(env, next); if (next) { env->SetObjectField(s, reply_markup_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateMessageEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageEdited");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("editDate", edit_date_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

void updateMessageEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageEdited").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    edit_date_fieldID = jni::get_field_id(env, Class, "editDate", "I");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageIsPinned::Class;
jfieldID updateMessageIsPinned::chat_id_fieldID;
jfieldID updateMessageIsPinned::message_id_fieldID;
jfieldID updateMessageIsPinned::is_pinned_fieldID;

updateMessageIsPinned::updateMessageIsPinned()
  : chat_id_()
  , message_id_()
  , is_pinned_()
{}

updateMessageIsPinned::updateMessageIsPinned(int53 chat_id_, int53 message_id_, bool is_pinned_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t updateMessageIsPinned::ID;

object_ptr<Update> updateMessageIsPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageIsPinned> res = make_object<updateMessageIsPinned>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return std::move(res);
}

void updateMessageIsPinned::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
}

void updateMessageIsPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageIsPinned");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

void updateMessageIsPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageIsPinned").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageInteractionInfo::Class;
jfieldID updateMessageInteractionInfo::chat_id_fieldID;
jfieldID updateMessageInteractionInfo::message_id_fieldID;
jfieldID updateMessageInteractionInfo::interaction_info_fieldID;

updateMessageInteractionInfo::updateMessageInteractionInfo()
  : chat_id_()
  , message_id_()
  , interaction_info_()
{}

updateMessageInteractionInfo::updateMessageInteractionInfo(int53 chat_id_, int53 message_id_, object_ptr<messageInteractionInfo> &&interaction_info_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , interaction_info_(std::move(interaction_info_))
{}

const std::int32_t updateMessageInteractionInfo::ID;

object_ptr<Update> updateMessageInteractionInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageInteractionInfo> res = make_object<updateMessageInteractionInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->interaction_info_ = jni::fetch_tl_object<messageInteractionInfo>(env, jni::fetch_object(env, p, res->interaction_info_fieldID));
  return std::move(res);
}

void updateMessageInteractionInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (interaction_info_ != nullptr) { jobject next; interaction_info_->store(env, next); if (next) { env->SetObjectField(s, interaction_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateMessageInteractionInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageInteractionInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("interactionInfo", static_cast<const BaseObject *>(interaction_info_.get()));
    s.store_class_end();
  }
}

void updateMessageInteractionInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageInteractionInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    interaction_info_fieldID = jni::get_field_id(env, Class, "interactionInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageInteractionInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageContentOpened::Class;
jfieldID updateMessageContentOpened::chat_id_fieldID;
jfieldID updateMessageContentOpened::message_id_fieldID;

updateMessageContentOpened::updateMessageContentOpened()
  : chat_id_()
  , message_id_()
{}

updateMessageContentOpened::updateMessageContentOpened(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t updateMessageContentOpened::ID;

object_ptr<Update> updateMessageContentOpened::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageContentOpened> res = make_object<updateMessageContentOpened>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void updateMessageContentOpened::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void updateMessageContentOpened::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageContentOpened");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void updateMessageContentOpened::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageContentOpened").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageMentionRead::Class;
jfieldID updateMessageMentionRead::chat_id_fieldID;
jfieldID updateMessageMentionRead::message_id_fieldID;
jfieldID updateMessageMentionRead::unread_mention_count_fieldID;

updateMessageMentionRead::updateMessageMentionRead()
  : chat_id_()
  , message_id_()
  , unread_mention_count_()
{}

updateMessageMentionRead::updateMessageMentionRead(int53 chat_id_, int53 message_id_, int32 unread_mention_count_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , unread_mention_count_(unread_mention_count_)
{}

const std::int32_t updateMessageMentionRead::ID;

object_ptr<Update> updateMessageMentionRead::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageMentionRead> res = make_object<updateMessageMentionRead>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->unread_mention_count_ = env->GetIntField(p, res->unread_mention_count_fieldID);
  return std::move(res);
}

void updateMessageMentionRead::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetIntField(s, unread_mention_count_fieldID, unread_mention_count_);
}

void updateMessageMentionRead::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageMentionRead");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("unreadMentionCount", unread_mention_count_);
    s.store_class_end();
  }
}

void updateMessageMentionRead::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageMentionRead").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    unread_mention_count_fieldID = jni::get_field_id(env, Class, "unreadMentionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageUnreadReactions::Class;
jfieldID updateMessageUnreadReactions::chat_id_fieldID;
jfieldID updateMessageUnreadReactions::message_id_fieldID;
jfieldID updateMessageUnreadReactions::unread_reactions_fieldID;
jfieldID updateMessageUnreadReactions::unread_reaction_count_fieldID;

updateMessageUnreadReactions::updateMessageUnreadReactions()
  : chat_id_()
  , message_id_()
  , unread_reactions_()
  , unread_reaction_count_()
{}

updateMessageUnreadReactions::updateMessageUnreadReactions(int53 chat_id_, int53 message_id_, array<object_ptr<unreadReaction>> &&unread_reactions_, int32 unread_reaction_count_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , unread_reactions_(std::move(unread_reactions_))
  , unread_reaction_count_(unread_reaction_count_)
{}

const std::int32_t updateMessageUnreadReactions::ID;

object_ptr<Update> updateMessageUnreadReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageUnreadReactions> res = make_object<updateMessageUnreadReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->unread_reactions_ = jni::FetchVector<unreadReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->unread_reactions_fieldID));
  res->unread_reaction_count_ = env->GetIntField(p, res->unread_reaction_count_fieldID);
  return std::move(res);
}

void updateMessageUnreadReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  { auto arr_tmp_ = jni::store_vector(env, unread_reactions_); if (arr_tmp_) { env->SetObjectField(s, unread_reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, unread_reaction_count_fieldID, unread_reaction_count_);
}

void updateMessageUnreadReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageUnreadReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    { s.store_vector_begin("unreadReactions", unread_reactions_.size()); for (const auto &_value : unread_reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("unreadReactionCount", unread_reaction_count_);
    s.store_class_end();
  }
}

void updateMessageUnreadReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageUnreadReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    unread_reactions_fieldID = jni::get_field_id(env, Class, "unreadReactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UnreadReaction;").c_str());
    unread_reaction_count_fieldID = jni::get_field_id(env, Class, "unreadReactionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageFactCheck::Class;
jfieldID updateMessageFactCheck::chat_id_fieldID;
jfieldID updateMessageFactCheck::message_id_fieldID;
jfieldID updateMessageFactCheck::fact_check_fieldID;

updateMessageFactCheck::updateMessageFactCheck()
  : chat_id_()
  , message_id_()
  , fact_check_()
{}

updateMessageFactCheck::updateMessageFactCheck(int53 chat_id_, int53 message_id_, object_ptr<factCheck> &&fact_check_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , fact_check_(std::move(fact_check_))
{}

const std::int32_t updateMessageFactCheck::ID;

object_ptr<Update> updateMessageFactCheck::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageFactCheck> res = make_object<updateMessageFactCheck>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->fact_check_ = jni::fetch_tl_object<factCheck>(env, jni::fetch_object(env, p, res->fact_check_fieldID));
  return std::move(res);
}

void updateMessageFactCheck::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (fact_check_ != nullptr) { jobject next; fact_check_->store(env, next); if (next) { env->SetObjectField(s, fact_check_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateMessageFactCheck::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageFactCheck");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("factCheck", static_cast<const BaseObject *>(fact_check_.get()));
    s.store_class_end();
  }
}

void updateMessageFactCheck::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageFactCheck").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    fact_check_fieldID = jni::get_field_id(env, Class, "factCheck", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FactCheck;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageLiveLocationViewed::Class;
jfieldID updateMessageLiveLocationViewed::chat_id_fieldID;
jfieldID updateMessageLiveLocationViewed::message_id_fieldID;

updateMessageLiveLocationViewed::updateMessageLiveLocationViewed()
  : chat_id_()
  , message_id_()
{}

updateMessageLiveLocationViewed::updateMessageLiveLocationViewed(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t updateMessageLiveLocationViewed::ID;

object_ptr<Update> updateMessageLiveLocationViewed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageLiveLocationViewed> res = make_object<updateMessageLiveLocationViewed>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void updateMessageLiveLocationViewed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void updateMessageLiveLocationViewed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageLiveLocationViewed");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void updateMessageLiveLocationViewed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageLiveLocationViewed").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateVideoPublished::Class;
jfieldID updateVideoPublished::chat_id_fieldID;
jfieldID updateVideoPublished::message_id_fieldID;

updateVideoPublished::updateVideoPublished()
  : chat_id_()
  , message_id_()
{}

updateVideoPublished::updateVideoPublished(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t updateVideoPublished::ID;

object_ptr<Update> updateVideoPublished::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateVideoPublished> res = make_object<updateVideoPublished>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return std::move(res);
}

void updateVideoPublished::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
}

void updateVideoPublished::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateVideoPublished");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

void updateVideoPublished::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateVideoPublished").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewChat::Class;
jfieldID updateNewChat::chat_fieldID;

updateNewChat::updateNewChat()
  : chat_()
{}

updateNewChat::updateNewChat(object_ptr<chat> &&chat_)
  : chat_(std::move(chat_))
{}

const std::int32_t updateNewChat::ID;

object_ptr<Update> updateNewChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewChat> res = make_object<updateNewChat>();
  res->chat_ = jni::fetch_tl_object<chat>(env, jni::fetch_object(env, p, res->chat_fieldID));
  return std::move(res);
}

void updateNewChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chat_ != nullptr) { jobject next; chat_->store(env, next); if (next) { env->SetObjectField(s, chat_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewChat");
    s.store_object_field("chat", static_cast<const BaseObject *>(chat_.get()));
    s.store_class_end();
  }
}

void updateNewChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewChat").c_str());
    chat_fieldID = jni::get_field_id(env, Class, "chat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Chat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatTitle::Class;
jfieldID updateChatTitle::chat_id_fieldID;
jfieldID updateChatTitle::title_fieldID;

updateChatTitle::updateChatTitle()
  : chat_id_()
  , title_()
{}

updateChatTitle::updateChatTitle(int53 chat_id_, string const &title_)
  : chat_id_(chat_id_)
  , title_(title_)
{}

const std::int32_t updateChatTitle::ID;

object_ptr<Update> updateChatTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatTitle> res = make_object<updateChatTitle>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return std::move(res);
}

void updateChatTitle::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateChatTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatTitle");
    s.store_field("chatId", chat_id_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

void updateChatTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatTitle").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatPhoto::Class;
jfieldID updateChatPhoto::chat_id_fieldID;
jfieldID updateChatPhoto::photo_fieldID;

updateChatPhoto::updateChatPhoto()
  : chat_id_()
  , photo_()
{}

updateChatPhoto::updateChatPhoto(int53 chat_id_, object_ptr<chatPhotoInfo> &&photo_)
  : chat_id_(chat_id_)
  , photo_(std::move(photo_))
{}

const std::int32_t updateChatPhoto::ID;

object_ptr<Update> updateChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatPhoto> res = make_object<updateChatPhoto>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->photo_ = jni::fetch_tl_object<chatPhotoInfo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return std::move(res);
}

void updateChatPhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatPhoto");
    s.store_field("chatId", chat_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

void updateChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatPhoto").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhotoInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatAccentColors::Class;
jfieldID updateChatAccentColors::chat_id_fieldID;
jfieldID updateChatAccentColors::accent_color_id_fieldID;
jfieldID updateChatAccentColors::background_custom_emoji_id_fieldID;
jfieldID updateChatAccentColors::profile_accent_color_id_fieldID;
jfieldID updateChatAccentColors::profile_background_custom_emoji_id_fieldID;

updateChatAccentColors::updateChatAccentColors()
  : chat_id_()
  , accent_color_id_()
  , background_custom_emoji_id_()
  , profile_accent_color_id_()
  , profile_background_custom_emoji_id_()
{}

updateChatAccentColors::updateChatAccentColors(int53 chat_id_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_)
  : chat_id_(chat_id_)
  , accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
  , profile_accent_color_id_(profile_accent_color_id_)
  , profile_background_custom_emoji_id_(profile_background_custom_emoji_id_)
{}

const std::int32_t updateChatAccentColors::ID;

object_ptr<Update> updateChatAccentColors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatAccentColors> res = make_object<updateChatAccentColors>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  res->profile_accent_color_id_ = env->GetIntField(p, res->profile_accent_color_id_fieldID);
  res->profile_background_custom_emoji_id_ = env->GetLongField(p, res->profile_background_custom_emoji_id_fieldID);
  return std::move(res);
}

void updateChatAccentColors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  env->SetLongField(s, background_custom_emoji_id_fieldID, background_custom_emoji_id_);
  env->SetIntField(s, profile_accent_color_id_fieldID, profile_accent_color_id_);
  env->SetLongField(s, profile_background_custom_emoji_id_fieldID, profile_background_custom_emoji_id_);
}

void updateChatAccentColors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatAccentColors");
    s.store_field("chatId", chat_id_);
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_field("profileAccentColorId", profile_accent_color_id_);
    s.store_field("profileBackgroundCustomEmojiId", profile_background_custom_emoji_id_);
    s.store_class_end();
  }
}

void updateChatAccentColors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatAccentColors").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "profileAccentColorId", "I");
    profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "profileBackgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatPermissions::Class;
jfieldID updateChatPermissions::chat_id_fieldID;
jfieldID updateChatPermissions::permissions_fieldID;

updateChatPermissions::updateChatPermissions()
  : chat_id_()
  , permissions_()
{}

updateChatPermissions::updateChatPermissions(int53 chat_id_, object_ptr<chatPermissions> &&permissions_)
  : chat_id_(chat_id_)
  , permissions_(std::move(permissions_))
{}

const std::int32_t updateChatPermissions::ID;

object_ptr<Update> updateChatPermissions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatPermissions> res = make_object<updateChatPermissions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->permissions_fieldID));
  return std::move(res);
}

void updateChatPermissions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (permissions_ != nullptr) { jobject next; permissions_->store(env, next); if (next) { env->SetObjectField(s, permissions_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatPermissions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatPermissions");
    s.store_field("chatId", chat_id_);
    s.store_object_field("permissions", static_cast<const BaseObject *>(permissions_.get()));
    s.store_class_end();
  }
}

void updateChatPermissions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatPermissions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    permissions_fieldID = jni::get_field_id(env, Class, "permissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatLastMessage::Class;
jfieldID updateChatLastMessage::chat_id_fieldID;
jfieldID updateChatLastMessage::last_message_fieldID;
jfieldID updateChatLastMessage::positions_fieldID;

updateChatLastMessage::updateChatLastMessage()
  : chat_id_()
  , last_message_()
  , positions_()
{}

updateChatLastMessage::updateChatLastMessage(int53 chat_id_, object_ptr<message> &&last_message_, array<object_ptr<chatPosition>> &&positions_)
  : chat_id_(chat_id_)
  , last_message_(std::move(last_message_))
  , positions_(std::move(positions_))
{}

const std::int32_t updateChatLastMessage::ID;

object_ptr<Update> updateChatLastMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatLastMessage> res = make_object<updateChatLastMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->last_message_ = jni::fetch_tl_object<message>(env, jni::fetch_object(env, p, res->last_message_fieldID));
  res->positions_ = jni::FetchVector<chatPosition>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->positions_fieldID));
  return std::move(res);
}

void updateChatLastMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (last_message_ != nullptr) { jobject next; last_message_->store(env, next); if (next) { env->SetObjectField(s, last_message_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, positions_); if (arr_tmp_) { env->SetObjectField(s, positions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateChatLastMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatLastMessage");
    s.store_field("chatId", chat_id_);
    s.store_object_field("lastMessage", static_cast<const BaseObject *>(last_message_.get()));
    { s.store_vector_begin("positions", positions_.size()); for (const auto &_value : positions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateChatLastMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatLastMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    last_message_fieldID = jni::get_field_id(env, Class, "lastMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    positions_fieldID = jni::get_field_id(env, Class, "positions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatPosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatPosition::Class;
jfieldID updateChatPosition::chat_id_fieldID;
jfieldID updateChatPosition::position_fieldID;

updateChatPosition::updateChatPosition()
  : chat_id_()
  , position_()
{}

updateChatPosition::updateChatPosition(int53 chat_id_, object_ptr<chatPosition> &&position_)
  : chat_id_(chat_id_)
  , position_(std::move(position_))
{}

const std::int32_t updateChatPosition::ID;

object_ptr<Update> updateChatPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatPosition> res = make_object<updateChatPosition>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->position_ = jni::fetch_tl_object<chatPosition>(env, jni::fetch_object(env, p, res->position_fieldID));
  return std::move(res);
}

void updateChatPosition::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (position_ != nullptr) { jobject next; position_->store(env, next); if (next) { env->SetObjectField(s, position_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatPosition");
    s.store_field("chatId", chat_id_);
    s.store_object_field("position", static_cast<const BaseObject *>(position_.get()));
    s.store_class_end();
  }
}

void updateChatPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatPosition").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    position_fieldID = jni::get_field_id(env, Class, "position", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatAddedToList::Class;
jfieldID updateChatAddedToList::chat_id_fieldID;
jfieldID updateChatAddedToList::chat_list_fieldID;

updateChatAddedToList::updateChatAddedToList()
  : chat_id_()
  , chat_list_()
{}

updateChatAddedToList::updateChatAddedToList(int53 chat_id_, object_ptr<ChatList> &&chat_list_)
  : chat_id_(chat_id_)
  , chat_list_(std::move(chat_list_))
{}

const std::int32_t updateChatAddedToList::ID;

object_ptr<Update> updateChatAddedToList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatAddedToList> res = make_object<updateChatAddedToList>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  return std::move(res);
}

void updateChatAddedToList::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (chat_list_ != nullptr) { jobject next; chat_list_->store(env, next); if (next) { env->SetObjectField(s, chat_list_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatAddedToList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatAddedToList");
    s.store_field("chatId", chat_id_);
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_class_end();
  }
}

void updateChatAddedToList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatAddedToList").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatRemovedFromList::Class;
jfieldID updateChatRemovedFromList::chat_id_fieldID;
jfieldID updateChatRemovedFromList::chat_list_fieldID;

updateChatRemovedFromList::updateChatRemovedFromList()
  : chat_id_()
  , chat_list_()
{}

updateChatRemovedFromList::updateChatRemovedFromList(int53 chat_id_, object_ptr<ChatList> &&chat_list_)
  : chat_id_(chat_id_)
  , chat_list_(std::move(chat_list_))
{}

const std::int32_t updateChatRemovedFromList::ID;

object_ptr<Update> updateChatRemovedFromList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatRemovedFromList> res = make_object<updateChatRemovedFromList>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  return std::move(res);
}

void updateChatRemovedFromList::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (chat_list_ != nullptr) { jobject next; chat_list_->store(env, next); if (next) { env->SetObjectField(s, chat_list_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatRemovedFromList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatRemovedFromList");
    s.store_field("chatId", chat_id_);
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_class_end();
  }
}

void updateChatRemovedFromList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatRemovedFromList").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatReadInbox::Class;
jfieldID updateChatReadInbox::chat_id_fieldID;
jfieldID updateChatReadInbox::last_read_inbox_message_id_fieldID;
jfieldID updateChatReadInbox::unread_count_fieldID;

updateChatReadInbox::updateChatReadInbox()
  : chat_id_()
  , last_read_inbox_message_id_()
  , unread_count_()
{}

updateChatReadInbox::updateChatReadInbox(int53 chat_id_, int53 last_read_inbox_message_id_, int32 unread_count_)
  : chat_id_(chat_id_)
  , last_read_inbox_message_id_(last_read_inbox_message_id_)
  , unread_count_(unread_count_)
{}

const std::int32_t updateChatReadInbox::ID;

object_ptr<Update> updateChatReadInbox::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatReadInbox> res = make_object<updateChatReadInbox>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->last_read_inbox_message_id_ = env->GetLongField(p, res->last_read_inbox_message_id_fieldID);
  res->unread_count_ = env->GetIntField(p, res->unread_count_fieldID);
  return std::move(res);
}

void updateChatReadInbox::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, last_read_inbox_message_id_fieldID, last_read_inbox_message_id_);
  env->SetIntField(s, unread_count_fieldID, unread_count_);
}

void updateChatReadInbox::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatReadInbox");
    s.store_field("chatId", chat_id_);
    s.store_field("lastReadInboxMessageId", last_read_inbox_message_id_);
    s.store_field("unreadCount", unread_count_);
    s.store_class_end();
  }
}

void updateChatReadInbox::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatReadInbox").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    last_read_inbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadInboxMessageId", "J");
    unread_count_fieldID = jni::get_field_id(env, Class, "unreadCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatReadOutbox::Class;
jfieldID updateChatReadOutbox::chat_id_fieldID;
jfieldID updateChatReadOutbox::last_read_outbox_message_id_fieldID;

updateChatReadOutbox::updateChatReadOutbox()
  : chat_id_()
  , last_read_outbox_message_id_()
{}

updateChatReadOutbox::updateChatReadOutbox(int53 chat_id_, int53 last_read_outbox_message_id_)
  : chat_id_(chat_id_)
  , last_read_outbox_message_id_(last_read_outbox_message_id_)
{}

const std::int32_t updateChatReadOutbox::ID;

object_ptr<Update> updateChatReadOutbox::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatReadOutbox> res = make_object<updateChatReadOutbox>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->last_read_outbox_message_id_ = env->GetLongField(p, res->last_read_outbox_message_id_fieldID);
  return std::move(res);
}

void updateChatReadOutbox::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, last_read_outbox_message_id_fieldID, last_read_outbox_message_id_);
}

void updateChatReadOutbox::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatReadOutbox");
    s.store_field("chatId", chat_id_);
    s.store_field("lastReadOutboxMessageId", last_read_outbox_message_id_);
    s.store_class_end();
  }
}

void updateChatReadOutbox::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatReadOutbox").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    last_read_outbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadOutboxMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatActionBar::Class;
jfieldID updateChatActionBar::chat_id_fieldID;
jfieldID updateChatActionBar::action_bar_fieldID;

updateChatActionBar::updateChatActionBar()
  : chat_id_()
  , action_bar_()
{}

updateChatActionBar::updateChatActionBar(int53 chat_id_, object_ptr<ChatActionBar> &&action_bar_)
  : chat_id_(chat_id_)
  , action_bar_(std::move(action_bar_))
{}

const std::int32_t updateChatActionBar::ID;

object_ptr<Update> updateChatActionBar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatActionBar> res = make_object<updateChatActionBar>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->action_bar_ = jni::fetch_tl_object<ChatActionBar>(env, jni::fetch_object(env, p, res->action_bar_fieldID));
  return std::move(res);
}

void updateChatActionBar::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (action_bar_ != nullptr) { jobject next; action_bar_->store(env, next); if (next) { env->SetObjectField(s, action_bar_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatActionBar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatActionBar");
    s.store_field("chatId", chat_id_);
    s.store_object_field("actionBar", static_cast<const BaseObject *>(action_bar_.get()));
    s.store_class_end();
  }
}

void updateChatActionBar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatActionBar").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    action_bar_fieldID = jni::get_field_id(env, Class, "actionBar", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatActionBar;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatBusinessBotManageBar::Class;
jfieldID updateChatBusinessBotManageBar::chat_id_fieldID;
jfieldID updateChatBusinessBotManageBar::business_bot_manage_bar_fieldID;

updateChatBusinessBotManageBar::updateChatBusinessBotManageBar()
  : chat_id_()
  , business_bot_manage_bar_()
{}

updateChatBusinessBotManageBar::updateChatBusinessBotManageBar(int53 chat_id_, object_ptr<businessBotManageBar> &&business_bot_manage_bar_)
  : chat_id_(chat_id_)
  , business_bot_manage_bar_(std::move(business_bot_manage_bar_))
{}

const std::int32_t updateChatBusinessBotManageBar::ID;

object_ptr<Update> updateChatBusinessBotManageBar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatBusinessBotManageBar> res = make_object<updateChatBusinessBotManageBar>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->business_bot_manage_bar_ = jni::fetch_tl_object<businessBotManageBar>(env, jni::fetch_object(env, p, res->business_bot_manage_bar_fieldID));
  return std::move(res);
}

void updateChatBusinessBotManageBar::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (business_bot_manage_bar_ != nullptr) { jobject next; business_bot_manage_bar_->store(env, next); if (next) { env->SetObjectField(s, business_bot_manage_bar_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatBusinessBotManageBar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatBusinessBotManageBar");
    s.store_field("chatId", chat_id_);
    s.store_object_field("businessBotManageBar", static_cast<const BaseObject *>(business_bot_manage_bar_.get()));
    s.store_class_end();
  }
}

void updateChatBusinessBotManageBar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatBusinessBotManageBar").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    business_bot_manage_bar_fieldID = jni::get_field_id(env, Class, "businessBotManageBar", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessBotManageBar;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatAvailableReactions::Class;
jfieldID updateChatAvailableReactions::chat_id_fieldID;
jfieldID updateChatAvailableReactions::available_reactions_fieldID;

updateChatAvailableReactions::updateChatAvailableReactions()
  : chat_id_()
  , available_reactions_()
{}

updateChatAvailableReactions::updateChatAvailableReactions(int53 chat_id_, object_ptr<ChatAvailableReactions> &&available_reactions_)
  : chat_id_(chat_id_)
  , available_reactions_(std::move(available_reactions_))
{}

const std::int32_t updateChatAvailableReactions::ID;

object_ptr<Update> updateChatAvailableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatAvailableReactions> res = make_object<updateChatAvailableReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->available_reactions_ = jni::fetch_tl_object<ChatAvailableReactions>(env, jni::fetch_object(env, p, res->available_reactions_fieldID));
  return std::move(res);
}

void updateChatAvailableReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (available_reactions_ != nullptr) { jobject next; available_reactions_->store(env, next); if (next) { env->SetObjectField(s, available_reactions_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatAvailableReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatAvailableReactions");
    s.store_field("chatId", chat_id_);
    s.store_object_field("availableReactions", static_cast<const BaseObject *>(available_reactions_.get()));
    s.store_class_end();
  }
}

void updateChatAvailableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatAvailableReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    available_reactions_fieldID = jni::get_field_id(env, Class, "availableReactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAvailableReactions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatDraftMessage::Class;
jfieldID updateChatDraftMessage::chat_id_fieldID;
jfieldID updateChatDraftMessage::draft_message_fieldID;
jfieldID updateChatDraftMessage::positions_fieldID;

updateChatDraftMessage::updateChatDraftMessage()
  : chat_id_()
  , draft_message_()
  , positions_()
{}

updateChatDraftMessage::updateChatDraftMessage(int53 chat_id_, object_ptr<draftMessage> &&draft_message_, array<object_ptr<chatPosition>> &&positions_)
  : chat_id_(chat_id_)
  , draft_message_(std::move(draft_message_))
  , positions_(std::move(positions_))
{}

const std::int32_t updateChatDraftMessage::ID;

object_ptr<Update> updateChatDraftMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatDraftMessage> res = make_object<updateChatDraftMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  res->positions_ = jni::FetchVector<chatPosition>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->positions_fieldID));
  return std::move(res);
}

void updateChatDraftMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (draft_message_ != nullptr) { jobject next; draft_message_->store(env, next); if (next) { env->SetObjectField(s, draft_message_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, positions_); if (arr_tmp_) { env->SetObjectField(s, positions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateChatDraftMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatDraftMessage");
    s.store_field("chatId", chat_id_);
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    { s.store_vector_begin("positions", positions_.size()); for (const auto &_value : positions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateChatDraftMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatDraftMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    positions_fieldID = jni::get_field_id(env, Class, "positions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatPosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatEmojiStatus::Class;
jfieldID updateChatEmojiStatus::chat_id_fieldID;
jfieldID updateChatEmojiStatus::emoji_status_fieldID;

updateChatEmojiStatus::updateChatEmojiStatus()
  : chat_id_()
  , emoji_status_()
{}

updateChatEmojiStatus::updateChatEmojiStatus(int53 chat_id_, object_ptr<emojiStatus> &&emoji_status_)
  : chat_id_(chat_id_)
  , emoji_status_(std::move(emoji_status_))
{}

const std::int32_t updateChatEmojiStatus::ID;

object_ptr<Update> updateChatEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatEmojiStatus> res = make_object<updateChatEmojiStatus>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  return std::move(res);
}

void updateChatEmojiStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (emoji_status_ != nullptr) { jobject next; emoji_status_->store(env, next); if (next) { env->SetObjectField(s, emoji_status_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatEmojiStatus");
    s.store_field("chatId", chat_id_);
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_class_end();
  }
}

void updateChatEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatEmojiStatus").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatMessageSender::Class;
jfieldID updateChatMessageSender::chat_id_fieldID;
jfieldID updateChatMessageSender::message_sender_id_fieldID;

updateChatMessageSender::updateChatMessageSender()
  : chat_id_()
  , message_sender_id_()
{}

updateChatMessageSender::updateChatMessageSender(int53 chat_id_, object_ptr<MessageSender> &&message_sender_id_)
  : chat_id_(chat_id_)
  , message_sender_id_(std::move(message_sender_id_))
{}

const std::int32_t updateChatMessageSender::ID;

object_ptr<Update> updateChatMessageSender::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatMessageSender> res = make_object<updateChatMessageSender>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->message_sender_id_fieldID));
  return std::move(res);
}

void updateChatMessageSender::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (message_sender_id_ != nullptr) { jobject next; message_sender_id_->store(env, next); if (next) { env->SetObjectField(s, message_sender_id_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatMessageSender::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatMessageSender");
    s.store_field("chatId", chat_id_);
    s.store_object_field("messageSenderId", static_cast<const BaseObject *>(message_sender_id_.get()));
    s.store_class_end();
  }
}

void updateChatMessageSender::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatMessageSender").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_sender_id_fieldID = jni::get_field_id(env, Class, "messageSenderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatMessageAutoDeleteTime::Class;
jfieldID updateChatMessageAutoDeleteTime::chat_id_fieldID;
jfieldID updateChatMessageAutoDeleteTime::message_auto_delete_time_fieldID;

updateChatMessageAutoDeleteTime::updateChatMessageAutoDeleteTime()
  : chat_id_()
  , message_auto_delete_time_()
{}

updateChatMessageAutoDeleteTime::updateChatMessageAutoDeleteTime(int53 chat_id_, int32 message_auto_delete_time_)
  : chat_id_(chat_id_)
  , message_auto_delete_time_(message_auto_delete_time_)
{}

const std::int32_t updateChatMessageAutoDeleteTime::ID;

object_ptr<Update> updateChatMessageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatMessageAutoDeleteTime> res = make_object<updateChatMessageAutoDeleteTime>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  return std::move(res);
}

void updateChatMessageAutoDeleteTime::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, message_auto_delete_time_fieldID, message_auto_delete_time_);
}

void updateChatMessageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatMessageAutoDeleteTime");
    s.store_field("chatId", chat_id_);
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_class_end();
  }
}

void updateChatMessageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatMessageAutoDeleteTime").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatNotificationSettings::Class;
jfieldID updateChatNotificationSettings::chat_id_fieldID;
jfieldID updateChatNotificationSettings::notification_settings_fieldID;

updateChatNotificationSettings::updateChatNotificationSettings()
  : chat_id_()
  , notification_settings_()
{}

updateChatNotificationSettings::updateChatNotificationSettings(int53 chat_id_, object_ptr<chatNotificationSettings> &&notification_settings_)
  : chat_id_(chat_id_)
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t updateChatNotificationSettings::ID;

object_ptr<Update> updateChatNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatNotificationSettings> res = make_object<updateChatNotificationSettings>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return std::move(res);
}

void updateChatNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatNotificationSettings");
    s.store_field("chatId", chat_id_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

void updateChatNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatNotificationSettings").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatPendingJoinRequests::Class;
jfieldID updateChatPendingJoinRequests::chat_id_fieldID;
jfieldID updateChatPendingJoinRequests::pending_join_requests_fieldID;

updateChatPendingJoinRequests::updateChatPendingJoinRequests()
  : chat_id_()
  , pending_join_requests_()
{}

updateChatPendingJoinRequests::updateChatPendingJoinRequests(int53 chat_id_, object_ptr<chatJoinRequestsInfo> &&pending_join_requests_)
  : chat_id_(chat_id_)
  , pending_join_requests_(std::move(pending_join_requests_))
{}

const std::int32_t updateChatPendingJoinRequests::ID;

object_ptr<Update> updateChatPendingJoinRequests::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatPendingJoinRequests> res = make_object<updateChatPendingJoinRequests>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->pending_join_requests_ = jni::fetch_tl_object<chatJoinRequestsInfo>(env, jni::fetch_object(env, p, res->pending_join_requests_fieldID));
  return std::move(res);
}

void updateChatPendingJoinRequests::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (pending_join_requests_ != nullptr) { jobject next; pending_join_requests_->store(env, next); if (next) { env->SetObjectField(s, pending_join_requests_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatPendingJoinRequests::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatPendingJoinRequests");
    s.store_field("chatId", chat_id_);
    s.store_object_field("pendingJoinRequests", static_cast<const BaseObject *>(pending_join_requests_.get()));
    s.store_class_end();
  }
}

void updateChatPendingJoinRequests::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatPendingJoinRequests").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    pending_join_requests_fieldID = jni::get_field_id(env, Class, "pendingJoinRequests", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatJoinRequestsInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatReplyMarkup::Class;
jfieldID updateChatReplyMarkup::chat_id_fieldID;
jfieldID updateChatReplyMarkup::reply_markup_message_id_fieldID;

updateChatReplyMarkup::updateChatReplyMarkup()
  : chat_id_()
  , reply_markup_message_id_()
{}

updateChatReplyMarkup::updateChatReplyMarkup(int53 chat_id_, int53 reply_markup_message_id_)
  : chat_id_(chat_id_)
  , reply_markup_message_id_(reply_markup_message_id_)
{}

const std::int32_t updateChatReplyMarkup::ID;

object_ptr<Update> updateChatReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatReplyMarkup> res = make_object<updateChatReplyMarkup>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->reply_markup_message_id_ = env->GetLongField(p, res->reply_markup_message_id_fieldID);
  return std::move(res);
}

void updateChatReplyMarkup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, reply_markup_message_id_fieldID, reply_markup_message_id_);
}

void updateChatReplyMarkup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatReplyMarkup");
    s.store_field("chatId", chat_id_);
    s.store_field("replyMarkupMessageId", reply_markup_message_id_);
    s.store_class_end();
  }
}

void updateChatReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatReplyMarkup").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    reply_markup_message_id_fieldID = jni::get_field_id(env, Class, "replyMarkupMessageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatBackground::Class;
jfieldID updateChatBackground::chat_id_fieldID;
jfieldID updateChatBackground::background_fieldID;

updateChatBackground::updateChatBackground()
  : chat_id_()
  , background_()
{}

updateChatBackground::updateChatBackground(int53 chat_id_, object_ptr<chatBackground> &&background_)
  : chat_id_(chat_id_)
  , background_(std::move(background_))
{}

const std::int32_t updateChatBackground::ID;

object_ptr<Update> updateChatBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatBackground> res = make_object<updateChatBackground>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->background_ = jni::fetch_tl_object<chatBackground>(env, jni::fetch_object(env, p, res->background_fieldID));
  return std::move(res);
}

void updateChatBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatBackground");
    s.store_field("chatId", chat_id_);
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_class_end();
  }
}

void updateChatBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatBackground").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBackground;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatTheme::Class;
jfieldID updateChatTheme::chat_id_fieldID;
jfieldID updateChatTheme::theme_name_fieldID;

updateChatTheme::updateChatTheme()
  : chat_id_()
  , theme_name_()
{}

updateChatTheme::updateChatTheme(int53 chat_id_, string const &theme_name_)
  : chat_id_(chat_id_)
  , theme_name_(theme_name_)
{}

const std::int32_t updateChatTheme::ID;

object_ptr<Update> updateChatTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatTheme> res = make_object<updateChatTheme>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return std::move(res);
}

void updateChatTheme::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { jstring nextString = jni::to_jstring(env, theme_name_); if (nextString) { env->SetObjectField(s, theme_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateChatTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatTheme");
    s.store_field("chatId", chat_id_);
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

void updateChatTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatTheme").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatUnreadMentionCount::Class;
jfieldID updateChatUnreadMentionCount::chat_id_fieldID;
jfieldID updateChatUnreadMentionCount::unread_mention_count_fieldID;

updateChatUnreadMentionCount::updateChatUnreadMentionCount()
  : chat_id_()
  , unread_mention_count_()
{}

updateChatUnreadMentionCount::updateChatUnreadMentionCount(int53 chat_id_, int32 unread_mention_count_)
  : chat_id_(chat_id_)
  , unread_mention_count_(unread_mention_count_)
{}

const std::int32_t updateChatUnreadMentionCount::ID;

object_ptr<Update> updateChatUnreadMentionCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatUnreadMentionCount> res = make_object<updateChatUnreadMentionCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->unread_mention_count_ = env->GetIntField(p, res->unread_mention_count_fieldID);
  return std::move(res);
}

void updateChatUnreadMentionCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, unread_mention_count_fieldID, unread_mention_count_);
}

void updateChatUnreadMentionCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatUnreadMentionCount");
    s.store_field("chatId", chat_id_);
    s.store_field("unreadMentionCount", unread_mention_count_);
    s.store_class_end();
  }
}

void updateChatUnreadMentionCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatUnreadMentionCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    unread_mention_count_fieldID = jni::get_field_id(env, Class, "unreadMentionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatUnreadReactionCount::Class;
jfieldID updateChatUnreadReactionCount::chat_id_fieldID;
jfieldID updateChatUnreadReactionCount::unread_reaction_count_fieldID;

updateChatUnreadReactionCount::updateChatUnreadReactionCount()
  : chat_id_()
  , unread_reaction_count_()
{}

updateChatUnreadReactionCount::updateChatUnreadReactionCount(int53 chat_id_, int32 unread_reaction_count_)
  : chat_id_(chat_id_)
  , unread_reaction_count_(unread_reaction_count_)
{}

const std::int32_t updateChatUnreadReactionCount::ID;

object_ptr<Update> updateChatUnreadReactionCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatUnreadReactionCount> res = make_object<updateChatUnreadReactionCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->unread_reaction_count_ = env->GetIntField(p, res->unread_reaction_count_fieldID);
  return std::move(res);
}

void updateChatUnreadReactionCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, unread_reaction_count_fieldID, unread_reaction_count_);
}

void updateChatUnreadReactionCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatUnreadReactionCount");
    s.store_field("chatId", chat_id_);
    s.store_field("unreadReactionCount", unread_reaction_count_);
    s.store_class_end();
  }
}

void updateChatUnreadReactionCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatUnreadReactionCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    unread_reaction_count_fieldID = jni::get_field_id(env, Class, "unreadReactionCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatVideoChat::Class;
jfieldID updateChatVideoChat::chat_id_fieldID;
jfieldID updateChatVideoChat::video_chat_fieldID;

updateChatVideoChat::updateChatVideoChat()
  : chat_id_()
  , video_chat_()
{}

updateChatVideoChat::updateChatVideoChat(int53 chat_id_, object_ptr<videoChat> &&video_chat_)
  : chat_id_(chat_id_)
  , video_chat_(std::move(video_chat_))
{}

const std::int32_t updateChatVideoChat::ID;

object_ptr<Update> updateChatVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatVideoChat> res = make_object<updateChatVideoChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->video_chat_ = jni::fetch_tl_object<videoChat>(env, jni::fetch_object(env, p, res->video_chat_fieldID));
  return std::move(res);
}

void updateChatVideoChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (video_chat_ != nullptr) { jobject next; video_chat_->store(env, next); if (next) { env->SetObjectField(s, video_chat_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatVideoChat");
    s.store_field("chatId", chat_id_);
    s.store_object_field("videoChat", static_cast<const BaseObject *>(video_chat_.get()));
    s.store_class_end();
  }
}

void updateChatVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatVideoChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    video_chat_fieldID = jni::get_field_id(env, Class, "videoChat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VideoChat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatDefaultDisableNotification::Class;
jfieldID updateChatDefaultDisableNotification::chat_id_fieldID;
jfieldID updateChatDefaultDisableNotification::default_disable_notification_fieldID;

updateChatDefaultDisableNotification::updateChatDefaultDisableNotification()
  : chat_id_()
  , default_disable_notification_()
{}

updateChatDefaultDisableNotification::updateChatDefaultDisableNotification(int53 chat_id_, bool default_disable_notification_)
  : chat_id_(chat_id_)
  , default_disable_notification_(default_disable_notification_)
{}

const std::int32_t updateChatDefaultDisableNotification::ID;

object_ptr<Update> updateChatDefaultDisableNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatDefaultDisableNotification> res = make_object<updateChatDefaultDisableNotification>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->default_disable_notification_ = (env->GetBooleanField(p, res->default_disable_notification_fieldID) != 0);
  return std::move(res);
}

void updateChatDefaultDisableNotification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, default_disable_notification_fieldID, default_disable_notification_);
}

void updateChatDefaultDisableNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatDefaultDisableNotification");
    s.store_field("chatId", chat_id_);
    s.store_field("defaultDisableNotification", default_disable_notification_);
    s.store_class_end();
  }
}

void updateChatDefaultDisableNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatDefaultDisableNotification").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    default_disable_notification_fieldID = jni::get_field_id(env, Class, "defaultDisableNotification", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatHasProtectedContent::Class;
jfieldID updateChatHasProtectedContent::chat_id_fieldID;
jfieldID updateChatHasProtectedContent::has_protected_content_fieldID;

updateChatHasProtectedContent::updateChatHasProtectedContent()
  : chat_id_()
  , has_protected_content_()
{}

updateChatHasProtectedContent::updateChatHasProtectedContent(int53 chat_id_, bool has_protected_content_)
  : chat_id_(chat_id_)
  , has_protected_content_(has_protected_content_)
{}

const std::int32_t updateChatHasProtectedContent::ID;

object_ptr<Update> updateChatHasProtectedContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatHasProtectedContent> res = make_object<updateChatHasProtectedContent>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->has_protected_content_ = (env->GetBooleanField(p, res->has_protected_content_fieldID) != 0);
  return std::move(res);
}

void updateChatHasProtectedContent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, has_protected_content_fieldID, has_protected_content_);
}

void updateChatHasProtectedContent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatHasProtectedContent");
    s.store_field("chatId", chat_id_);
    s.store_field("hasProtectedContent", has_protected_content_);
    s.store_class_end();
  }
}

void updateChatHasProtectedContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatHasProtectedContent").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    has_protected_content_fieldID = jni::get_field_id(env, Class, "hasProtectedContent", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatIsTranslatable::Class;
jfieldID updateChatIsTranslatable::chat_id_fieldID;
jfieldID updateChatIsTranslatable::is_translatable_fieldID;

updateChatIsTranslatable::updateChatIsTranslatable()
  : chat_id_()
  , is_translatable_()
{}

updateChatIsTranslatable::updateChatIsTranslatable(int53 chat_id_, bool is_translatable_)
  : chat_id_(chat_id_)
  , is_translatable_(is_translatable_)
{}

const std::int32_t updateChatIsTranslatable::ID;

object_ptr<Update> updateChatIsTranslatable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatIsTranslatable> res = make_object<updateChatIsTranslatable>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_translatable_ = (env->GetBooleanField(p, res->is_translatable_fieldID) != 0);
  return std::move(res);
}

void updateChatIsTranslatable::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, is_translatable_fieldID, is_translatable_);
}

void updateChatIsTranslatable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatIsTranslatable");
    s.store_field("chatId", chat_id_);
    s.store_field("isTranslatable", is_translatable_);
    s.store_class_end();
  }
}

void updateChatIsTranslatable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatIsTranslatable").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_translatable_fieldID = jni::get_field_id(env, Class, "isTranslatable", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatIsMarkedAsUnread::Class;
jfieldID updateChatIsMarkedAsUnread::chat_id_fieldID;
jfieldID updateChatIsMarkedAsUnread::is_marked_as_unread_fieldID;

updateChatIsMarkedAsUnread::updateChatIsMarkedAsUnread()
  : chat_id_()
  , is_marked_as_unread_()
{}

updateChatIsMarkedAsUnread::updateChatIsMarkedAsUnread(int53 chat_id_, bool is_marked_as_unread_)
  : chat_id_(chat_id_)
  , is_marked_as_unread_(is_marked_as_unread_)
{}

const std::int32_t updateChatIsMarkedAsUnread::ID;

object_ptr<Update> updateChatIsMarkedAsUnread::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatIsMarkedAsUnread> res = make_object<updateChatIsMarkedAsUnread>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_marked_as_unread_ = (env->GetBooleanField(p, res->is_marked_as_unread_fieldID) != 0);
  return std::move(res);
}

void updateChatIsMarkedAsUnread::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, is_marked_as_unread_fieldID, is_marked_as_unread_);
}

void updateChatIsMarkedAsUnread::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatIsMarkedAsUnread");
    s.store_field("chatId", chat_id_);
    s.store_field("isMarkedAsUnread", is_marked_as_unread_);
    s.store_class_end();
  }
}

void updateChatIsMarkedAsUnread::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatIsMarkedAsUnread").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_marked_as_unread_fieldID = jni::get_field_id(env, Class, "isMarkedAsUnread", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatViewAsTopics::Class;
jfieldID updateChatViewAsTopics::chat_id_fieldID;
jfieldID updateChatViewAsTopics::view_as_topics_fieldID;

updateChatViewAsTopics::updateChatViewAsTopics()
  : chat_id_()
  , view_as_topics_()
{}

updateChatViewAsTopics::updateChatViewAsTopics(int53 chat_id_, bool view_as_topics_)
  : chat_id_(chat_id_)
  , view_as_topics_(view_as_topics_)
{}

const std::int32_t updateChatViewAsTopics::ID;

object_ptr<Update> updateChatViewAsTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatViewAsTopics> res = make_object<updateChatViewAsTopics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->view_as_topics_ = (env->GetBooleanField(p, res->view_as_topics_fieldID) != 0);
  return std::move(res);
}

void updateChatViewAsTopics::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, view_as_topics_fieldID, view_as_topics_);
}

void updateChatViewAsTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatViewAsTopics");
    s.store_field("chatId", chat_id_);
    s.store_field("viewAsTopics", view_as_topics_);
    s.store_class_end();
  }
}

void updateChatViewAsTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatViewAsTopics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    view_as_topics_fieldID = jni::get_field_id(env, Class, "viewAsTopics", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatBlockList::Class;
jfieldID updateChatBlockList::chat_id_fieldID;
jfieldID updateChatBlockList::block_list_fieldID;

updateChatBlockList::updateChatBlockList()
  : chat_id_()
  , block_list_()
{}

updateChatBlockList::updateChatBlockList(int53 chat_id_, object_ptr<BlockList> &&block_list_)
  : chat_id_(chat_id_)
  , block_list_(std::move(block_list_))
{}

const std::int32_t updateChatBlockList::ID;

object_ptr<Update> updateChatBlockList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatBlockList> res = make_object<updateChatBlockList>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  return std::move(res);
}

void updateChatBlockList::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (block_list_ != nullptr) { jobject next; block_list_->store(env, next); if (next) { env->SetObjectField(s, block_list_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatBlockList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatBlockList");
    s.store_field("chatId", chat_id_);
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_class_end();
  }
}

void updateChatBlockList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatBlockList").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatHasScheduledMessages::Class;
jfieldID updateChatHasScheduledMessages::chat_id_fieldID;
jfieldID updateChatHasScheduledMessages::has_scheduled_messages_fieldID;

updateChatHasScheduledMessages::updateChatHasScheduledMessages()
  : chat_id_()
  , has_scheduled_messages_()
{}

updateChatHasScheduledMessages::updateChatHasScheduledMessages(int53 chat_id_, bool has_scheduled_messages_)
  : chat_id_(chat_id_)
  , has_scheduled_messages_(has_scheduled_messages_)
{}

const std::int32_t updateChatHasScheduledMessages::ID;

object_ptr<Update> updateChatHasScheduledMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatHasScheduledMessages> res = make_object<updateChatHasScheduledMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->has_scheduled_messages_ = (env->GetBooleanField(p, res->has_scheduled_messages_fieldID) != 0);
  return std::move(res);
}

void updateChatHasScheduledMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetBooleanField(s, has_scheduled_messages_fieldID, has_scheduled_messages_);
}

void updateChatHasScheduledMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatHasScheduledMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("hasScheduledMessages", has_scheduled_messages_);
    s.store_class_end();
  }
}

void updateChatHasScheduledMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatHasScheduledMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    has_scheduled_messages_fieldID = jni::get_field_id(env, Class, "hasScheduledMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatFolders::Class;
jfieldID updateChatFolders::chat_folders_fieldID;
jfieldID updateChatFolders::main_chat_list_position_fieldID;
jfieldID updateChatFolders::are_tags_enabled_fieldID;

updateChatFolders::updateChatFolders()
  : chat_folders_()
  , main_chat_list_position_()
  , are_tags_enabled_()
{}

updateChatFolders::updateChatFolders(array<object_ptr<chatFolderInfo>> &&chat_folders_, int32 main_chat_list_position_, bool are_tags_enabled_)
  : chat_folders_(std::move(chat_folders_))
  , main_chat_list_position_(main_chat_list_position_)
  , are_tags_enabled_(are_tags_enabled_)
{}

const std::int32_t updateChatFolders::ID;

object_ptr<Update> updateChatFolders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatFolders> res = make_object<updateChatFolders>();
  res->chat_folders_ = jni::FetchVector<chatFolderInfo>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chat_folders_fieldID));
  res->main_chat_list_position_ = env->GetIntField(p, res->main_chat_list_position_fieldID);
  res->are_tags_enabled_ = (env->GetBooleanField(p, res->are_tags_enabled_fieldID) != 0);
  return std::move(res);
}

void updateChatFolders::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_folders_); if (arr_tmp_) { env->SetObjectField(s, chat_folders_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, main_chat_list_position_fieldID, main_chat_list_position_);
  env->SetBooleanField(s, are_tags_enabled_fieldID, are_tags_enabled_);
}

void updateChatFolders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatFolders");
    { s.store_vector_begin("chatFolders", chat_folders_.size()); for (const auto &_value : chat_folders_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("mainChatListPosition", main_chat_list_position_);
    s.store_field("areTagsEnabled", are_tags_enabled_);
    s.store_class_end();
  }
}

void updateChatFolders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatFolders").c_str());
    chat_folders_fieldID = jni::get_field_id(env, Class, "chatFolders", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatFolderInfo;").c_str());
    main_chat_list_position_fieldID = jni::get_field_id(env, Class, "mainChatListPosition", "I");
    are_tags_enabled_fieldID = jni::get_field_id(env, Class, "areTagsEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatOnlineMemberCount::Class;
jfieldID updateChatOnlineMemberCount::chat_id_fieldID;
jfieldID updateChatOnlineMemberCount::online_member_count_fieldID;

updateChatOnlineMemberCount::updateChatOnlineMemberCount()
  : chat_id_()
  , online_member_count_()
{}

updateChatOnlineMemberCount::updateChatOnlineMemberCount(int53 chat_id_, int32 online_member_count_)
  : chat_id_(chat_id_)
  , online_member_count_(online_member_count_)
{}

const std::int32_t updateChatOnlineMemberCount::ID;

object_ptr<Update> updateChatOnlineMemberCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatOnlineMemberCount> res = make_object<updateChatOnlineMemberCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->online_member_count_ = env->GetIntField(p, res->online_member_count_fieldID);
  return std::move(res);
}

void updateChatOnlineMemberCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetIntField(s, online_member_count_fieldID, online_member_count_);
}

void updateChatOnlineMemberCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatOnlineMemberCount");
    s.store_field("chatId", chat_id_);
    s.store_field("onlineMemberCount", online_member_count_);
    s.store_class_end();
  }
}

void updateChatOnlineMemberCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatOnlineMemberCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    online_member_count_fieldID = jni::get_field_id(env, Class, "onlineMemberCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateSavedMessagesTopic::Class;
jfieldID updateSavedMessagesTopic::topic_fieldID;

updateSavedMessagesTopic::updateSavedMessagesTopic()
  : topic_()
{}

updateSavedMessagesTopic::updateSavedMessagesTopic(object_ptr<savedMessagesTopic> &&topic_)
  : topic_(std::move(topic_))
{}

const std::int32_t updateSavedMessagesTopic::ID;

object_ptr<Update> updateSavedMessagesTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSavedMessagesTopic> res = make_object<updateSavedMessagesTopic>();
  res->topic_ = jni::fetch_tl_object<savedMessagesTopic>(env, jni::fetch_object(env, p, res->topic_fieldID));
  return std::move(res);
}

void updateSavedMessagesTopic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (topic_ != nullptr) { jobject next; topic_->store(env, next); if (next) { env->SetObjectField(s, topic_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateSavedMessagesTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSavedMessagesTopic");
    s.store_object_field("topic", static_cast<const BaseObject *>(topic_.get()));
    s.store_class_end();
  }
}

void updateSavedMessagesTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSavedMessagesTopic").c_str());
    topic_fieldID = jni::get_field_id(env, Class, "topic", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SavedMessagesTopic;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSavedMessagesTopicCount::Class;
jfieldID updateSavedMessagesTopicCount::topic_count_fieldID;

updateSavedMessagesTopicCount::updateSavedMessagesTopicCount()
  : topic_count_()
{}

updateSavedMessagesTopicCount::updateSavedMessagesTopicCount(int32 topic_count_)
  : topic_count_(topic_count_)
{}

const std::int32_t updateSavedMessagesTopicCount::ID;

object_ptr<Update> updateSavedMessagesTopicCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSavedMessagesTopicCount> res = make_object<updateSavedMessagesTopicCount>();
  res->topic_count_ = env->GetIntField(p, res->topic_count_fieldID);
  return std::move(res);
}

void updateSavedMessagesTopicCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, topic_count_fieldID, topic_count_);
}

void updateSavedMessagesTopicCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSavedMessagesTopicCount");
    s.store_field("topicCount", topic_count_);
    s.store_class_end();
  }
}

void updateSavedMessagesTopicCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSavedMessagesTopicCount").c_str());
    topic_count_fieldID = jni::get_field_id(env, Class, "topicCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateQuickReplyShortcut::Class;
jfieldID updateQuickReplyShortcut::shortcut_fieldID;

updateQuickReplyShortcut::updateQuickReplyShortcut()
  : shortcut_()
{}

updateQuickReplyShortcut::updateQuickReplyShortcut(object_ptr<quickReplyShortcut> &&shortcut_)
  : shortcut_(std::move(shortcut_))
{}

const std::int32_t updateQuickReplyShortcut::ID;

object_ptr<Update> updateQuickReplyShortcut::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateQuickReplyShortcut> res = make_object<updateQuickReplyShortcut>();
  res->shortcut_ = jni::fetch_tl_object<quickReplyShortcut>(env, jni::fetch_object(env, p, res->shortcut_fieldID));
  return std::move(res);
}

void updateQuickReplyShortcut::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (shortcut_ != nullptr) { jobject next; shortcut_->store(env, next); if (next) { env->SetObjectField(s, shortcut_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateQuickReplyShortcut::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateQuickReplyShortcut");
    s.store_object_field("shortcut", static_cast<const BaseObject *>(shortcut_.get()));
    s.store_class_end();
  }
}

void updateQuickReplyShortcut::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateQuickReplyShortcut").c_str());
    shortcut_fieldID = jni::get_field_id(env, Class, "shortcut", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$QuickReplyShortcut;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateQuickReplyShortcutDeleted::Class;
jfieldID updateQuickReplyShortcutDeleted::shortcut_id_fieldID;

updateQuickReplyShortcutDeleted::updateQuickReplyShortcutDeleted()
  : shortcut_id_()
{}

updateQuickReplyShortcutDeleted::updateQuickReplyShortcutDeleted(int32 shortcut_id_)
  : shortcut_id_(shortcut_id_)
{}

const std::int32_t updateQuickReplyShortcutDeleted::ID;

object_ptr<Update> updateQuickReplyShortcutDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateQuickReplyShortcutDeleted> res = make_object<updateQuickReplyShortcutDeleted>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  return std::move(res);
}

void updateQuickReplyShortcutDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, shortcut_id_fieldID, shortcut_id_);
}

void updateQuickReplyShortcutDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateQuickReplyShortcutDeleted");
    s.store_field("shortcutId", shortcut_id_);
    s.store_class_end();
  }
}

void updateQuickReplyShortcutDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateQuickReplyShortcutDeleted").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateQuickReplyShortcuts::Class;
jfieldID updateQuickReplyShortcuts::shortcut_ids_fieldID;

updateQuickReplyShortcuts::updateQuickReplyShortcuts()
  : shortcut_ids_()
{}

updateQuickReplyShortcuts::updateQuickReplyShortcuts(array<int32> &&shortcut_ids_)
  : shortcut_ids_(std::move(shortcut_ids_))
{}

const std::int32_t updateQuickReplyShortcuts::ID;

object_ptr<Update> updateQuickReplyShortcuts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateQuickReplyShortcuts> res = make_object<updateQuickReplyShortcuts>();
  res->shortcut_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->shortcut_ids_fieldID));
  return std::move(res);
}

void updateQuickReplyShortcuts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, shortcut_ids_); if (arr_tmp_) { env->SetObjectField(s, shortcut_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateQuickReplyShortcuts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateQuickReplyShortcuts");
    { s.store_vector_begin("shortcutIds", shortcut_ids_.size()); for (const auto &_value : shortcut_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateQuickReplyShortcuts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateQuickReplyShortcuts").c_str());
    shortcut_ids_fieldID = jni::get_field_id(env, Class, "shortcutIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateQuickReplyShortcutMessages::Class;
jfieldID updateQuickReplyShortcutMessages::shortcut_id_fieldID;
jfieldID updateQuickReplyShortcutMessages::messages_fieldID;

updateQuickReplyShortcutMessages::updateQuickReplyShortcutMessages()
  : shortcut_id_()
  , messages_()
{}

updateQuickReplyShortcutMessages::updateQuickReplyShortcutMessages(int32 shortcut_id_, array<object_ptr<quickReplyMessage>> &&messages_)
  : shortcut_id_(shortcut_id_)
  , messages_(std::move(messages_))
{}

const std::int32_t updateQuickReplyShortcutMessages::ID;

object_ptr<Update> updateQuickReplyShortcutMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateQuickReplyShortcutMessages> res = make_object<updateQuickReplyShortcutMessages>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->messages_ = jni::FetchVector<quickReplyMessage>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  return std::move(res);
}

void updateQuickReplyShortcutMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, shortcut_id_fieldID, shortcut_id_);
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateQuickReplyShortcutMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateQuickReplyShortcutMessages");
    s.store_field("shortcutId", shortcut_id_);
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateQuickReplyShortcutMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateQuickReplyShortcutMessages").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$QuickReplyMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateForumTopicInfo::Class;
jfieldID updateForumTopicInfo::info_fieldID;

updateForumTopicInfo::updateForumTopicInfo()
  : info_()
{}

updateForumTopicInfo::updateForumTopicInfo(object_ptr<forumTopicInfo> &&info_)
  : info_(std::move(info_))
{}

const std::int32_t updateForumTopicInfo::ID;

object_ptr<Update> updateForumTopicInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateForumTopicInfo> res = make_object<updateForumTopicInfo>();
  res->info_ = jni::fetch_tl_object<forumTopicInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  return std::move(res);
}

void updateForumTopicInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (info_ != nullptr) { jobject next; info_->store(env, next); if (next) { env->SetObjectField(s, info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateForumTopicInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateForumTopicInfo");
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    s.store_class_end();
  }
}

void updateForumTopicInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateForumTopicInfo").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateForumTopic::Class;
jfieldID updateForumTopic::chat_id_fieldID;
jfieldID updateForumTopic::message_thread_id_fieldID;
jfieldID updateForumTopic::is_pinned_fieldID;
jfieldID updateForumTopic::last_read_inbox_message_id_fieldID;
jfieldID updateForumTopic::last_read_outbox_message_id_fieldID;
jfieldID updateForumTopic::notification_settings_fieldID;

updateForumTopic::updateForumTopic()
  : chat_id_()
  , message_thread_id_()
  , is_pinned_()
  , last_read_inbox_message_id_()
  , last_read_outbox_message_id_()
  , notification_settings_()
{}

updateForumTopic::updateForumTopic(int53 chat_id_, int53 message_thread_id_, bool is_pinned_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, object_ptr<chatNotificationSettings> &&notification_settings_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , is_pinned_(is_pinned_)
  , last_read_inbox_message_id_(last_read_inbox_message_id_)
  , last_read_outbox_message_id_(last_read_outbox_message_id_)
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t updateForumTopic::ID;

object_ptr<Update> updateForumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateForumTopic> res = make_object<updateForumTopic>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  res->last_read_inbox_message_id_ = env->GetLongField(p, res->last_read_inbox_message_id_fieldID);
  res->last_read_outbox_message_id_ = env->GetLongField(p, res->last_read_outbox_message_id_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return std::move(res);
}

void updateForumTopic::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  env->SetBooleanField(s, is_pinned_fieldID, is_pinned_);
  env->SetLongField(s, last_read_inbox_message_id_fieldID, last_read_inbox_message_id_);
  env->SetLongField(s, last_read_outbox_message_id_fieldID, last_read_outbox_message_id_);
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateForumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateForumTopic");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_field("lastReadInboxMessageId", last_read_inbox_message_id_);
    s.store_field("lastReadOutboxMessageId", last_read_outbox_message_id_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

void updateForumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateForumTopic").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    last_read_inbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadInboxMessageId", "J");
    last_read_outbox_message_id_fieldID = jni::get_field_id(env, Class, "lastReadOutboxMessageId", "J");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateScopeNotificationSettings::Class;
jfieldID updateScopeNotificationSettings::scope_fieldID;
jfieldID updateScopeNotificationSettings::notification_settings_fieldID;

updateScopeNotificationSettings::updateScopeNotificationSettings()
  : scope_()
  , notification_settings_()
{}

updateScopeNotificationSettings::updateScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_, object_ptr<scopeNotificationSettings> &&notification_settings_)
  : scope_(std::move(scope_))
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t updateScopeNotificationSettings::ID;

object_ptr<Update> updateScopeNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateScopeNotificationSettings> res = make_object<updateScopeNotificationSettings>();
  res->scope_ = jni::fetch_tl_object<NotificationSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->notification_settings_ = jni::fetch_tl_object<scopeNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return std::move(res);
}

void updateScopeNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (scope_ != nullptr) { jobject next; scope_->store(env, next); if (next) { env->SetObjectField(s, scope_fieldID, next); env->DeleteLocalRef(next); } }
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateScopeNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateScopeNotificationSettings");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

void updateScopeNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateScopeNotificationSettings").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationSettingsScope;").c_str());
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateReactionNotificationSettings::Class;
jfieldID updateReactionNotificationSettings::notification_settings_fieldID;

updateReactionNotificationSettings::updateReactionNotificationSettings()
  : notification_settings_()
{}

updateReactionNotificationSettings::updateReactionNotificationSettings(object_ptr<reactionNotificationSettings> &&notification_settings_)
  : notification_settings_(std::move(notification_settings_))
{}

const std::int32_t updateReactionNotificationSettings::ID;

object_ptr<Update> updateReactionNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateReactionNotificationSettings> res = make_object<updateReactionNotificationSettings>();
  res->notification_settings_ = jni::fetch_tl_object<reactionNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return std::move(res);
}

void updateReactionNotificationSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (notification_settings_ != nullptr) { jobject next; notification_settings_->store(env, next); if (next) { env->SetObjectField(s, notification_settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateReactionNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateReactionNotificationSettings");
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

void updateReactionNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateReactionNotificationSettings").c_str());
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNotification::Class;
jfieldID updateNotification::notification_group_id_fieldID;
jfieldID updateNotification::notification_fieldID;

updateNotification::updateNotification()
  : notification_group_id_()
  , notification_()
{}

updateNotification::updateNotification(int32 notification_group_id_, object_ptr<notification> &&notification_)
  : notification_group_id_(notification_group_id_)
  , notification_(std::move(notification_))
{}

const std::int32_t updateNotification::ID;

object_ptr<Update> updateNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNotification> res = make_object<updateNotification>();
  res->notification_group_id_ = env->GetIntField(p, res->notification_group_id_fieldID);
  res->notification_ = jni::fetch_tl_object<notification>(env, jni::fetch_object(env, p, res->notification_fieldID));
  return std::move(res);
}

void updateNotification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, notification_group_id_fieldID, notification_group_id_);
  if (notification_ != nullptr) { jobject next; notification_->store(env, next); if (next) { env->SetObjectField(s, notification_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNotification");
    s.store_field("notificationGroupId", notification_group_id_);
    s.store_object_field("notification", static_cast<const BaseObject *>(notification_.get()));
    s.store_class_end();
  }
}

void updateNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNotification").c_str());
    notification_group_id_fieldID = jni::get_field_id(env, Class, "notificationGroupId", "I");
    notification_fieldID = jni::get_field_id(env, Class, "notification", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Notification;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNotificationGroup::Class;
jfieldID updateNotificationGroup::notification_group_id_fieldID;
jfieldID updateNotificationGroup::type_fieldID;
jfieldID updateNotificationGroup::chat_id_fieldID;
jfieldID updateNotificationGroup::notification_settings_chat_id_fieldID;
jfieldID updateNotificationGroup::notification_sound_id_fieldID;
jfieldID updateNotificationGroup::total_count_fieldID;
jfieldID updateNotificationGroup::added_notifications_fieldID;
jfieldID updateNotificationGroup::removed_notification_ids_fieldID;

updateNotificationGroup::updateNotificationGroup()
  : notification_group_id_()
  , type_()
  , chat_id_()
  , notification_settings_chat_id_()
  , notification_sound_id_()
  , total_count_()
  , added_notifications_()
  , removed_notification_ids_()
{}

updateNotificationGroup::updateNotificationGroup(int32 notification_group_id_, object_ptr<NotificationGroupType> &&type_, int53 chat_id_, int53 notification_settings_chat_id_, int64 notification_sound_id_, int32 total_count_, array<object_ptr<notification>> &&added_notifications_, array<int32> &&removed_notification_ids_)
  : notification_group_id_(notification_group_id_)
  , type_(std::move(type_))
  , chat_id_(chat_id_)
  , notification_settings_chat_id_(notification_settings_chat_id_)
  , notification_sound_id_(notification_sound_id_)
  , total_count_(total_count_)
  , added_notifications_(std::move(added_notifications_))
  , removed_notification_ids_(std::move(removed_notification_ids_))
{}

const std::int32_t updateNotificationGroup::ID;

object_ptr<Update> updateNotificationGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNotificationGroup> res = make_object<updateNotificationGroup>();
  res->notification_group_id_ = env->GetIntField(p, res->notification_group_id_fieldID);
  res->type_ = jni::fetch_tl_object<NotificationGroupType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->notification_settings_chat_id_ = env->GetLongField(p, res->notification_settings_chat_id_fieldID);
  res->notification_sound_id_ = env->GetLongField(p, res->notification_sound_id_fieldID);
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->added_notifications_ = jni::FetchVector<notification>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->added_notifications_fieldID));
  res->removed_notification_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->removed_notification_ids_fieldID));
  return std::move(res);
}

void updateNotificationGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, notification_group_id_fieldID, notification_group_id_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, notification_settings_chat_id_fieldID, notification_settings_chat_id_);
  env->SetLongField(s, notification_sound_id_fieldID, notification_sound_id_);
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, added_notifications_); if (arr_tmp_) { env->SetObjectField(s, added_notifications_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, removed_notification_ids_); if (arr_tmp_) { env->SetObjectField(s, removed_notification_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateNotificationGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNotificationGroup");
    s.store_field("notificationGroupId", notification_group_id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("chatId", chat_id_);
    s.store_field("notificationSettingsChatId", notification_settings_chat_id_);
    s.store_field("notificationSoundId", notification_sound_id_);
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("addedNotifications", added_notifications_.size()); for (const auto &_value : added_notifications_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("removedNotificationIds", removed_notification_ids_.size()); for (const auto &_value : removed_notification_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateNotificationGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNotificationGroup").c_str());
    notification_group_id_fieldID = jni::get_field_id(env, Class, "notificationGroupId", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationGroupType;").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    notification_settings_chat_id_fieldID = jni::get_field_id(env, Class, "notificationSettingsChatId", "J");
    notification_sound_id_fieldID = jni::get_field_id(env, Class, "notificationSoundId", "J");
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    added_notifications_fieldID = jni::get_field_id(env, Class, "addedNotifications", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Notification;").c_str());
    removed_notification_ids_fieldID = jni::get_field_id(env, Class, "removedNotificationIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateActiveNotifications::Class;
jfieldID updateActiveNotifications::groups_fieldID;

updateActiveNotifications::updateActiveNotifications()
  : groups_()
{}

updateActiveNotifications::updateActiveNotifications(array<object_ptr<notificationGroup>> &&groups_)
  : groups_(std::move(groups_))
{}

const std::int32_t updateActiveNotifications::ID;

object_ptr<Update> updateActiveNotifications::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateActiveNotifications> res = make_object<updateActiveNotifications>();
  res->groups_ = jni::FetchVector<notificationGroup>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->groups_fieldID));
  return std::move(res);
}

void updateActiveNotifications::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, groups_); if (arr_tmp_) { env->SetObjectField(s, groups_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateActiveNotifications::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateActiveNotifications");
    { s.store_vector_begin("groups", groups_.size()); for (const auto &_value : groups_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateActiveNotifications::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateActiveNotifications").c_str());
    groups_fieldID = jni::get_field_id(env, Class, "groups", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$NotificationGroup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateHavePendingNotifications::Class;
jfieldID updateHavePendingNotifications::have_delayed_notifications_fieldID;
jfieldID updateHavePendingNotifications::have_unreceived_notifications_fieldID;

updateHavePendingNotifications::updateHavePendingNotifications()
  : have_delayed_notifications_()
  , have_unreceived_notifications_()
{}

updateHavePendingNotifications::updateHavePendingNotifications(bool have_delayed_notifications_, bool have_unreceived_notifications_)
  : have_delayed_notifications_(have_delayed_notifications_)
  , have_unreceived_notifications_(have_unreceived_notifications_)
{}

const std::int32_t updateHavePendingNotifications::ID;

object_ptr<Update> updateHavePendingNotifications::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateHavePendingNotifications> res = make_object<updateHavePendingNotifications>();
  res->have_delayed_notifications_ = (env->GetBooleanField(p, res->have_delayed_notifications_fieldID) != 0);
  res->have_unreceived_notifications_ = (env->GetBooleanField(p, res->have_unreceived_notifications_fieldID) != 0);
  return std::move(res);
}

void updateHavePendingNotifications::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, have_delayed_notifications_fieldID, have_delayed_notifications_);
  env->SetBooleanField(s, have_unreceived_notifications_fieldID, have_unreceived_notifications_);
}

void updateHavePendingNotifications::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateHavePendingNotifications");
    s.store_field("haveDelayedNotifications", have_delayed_notifications_);
    s.store_field("haveUnreceivedNotifications", have_unreceived_notifications_);
    s.store_class_end();
  }
}

void updateHavePendingNotifications::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateHavePendingNotifications").c_str());
    have_delayed_notifications_fieldID = jni::get_field_id(env, Class, "haveDelayedNotifications", "Z");
    have_unreceived_notifications_fieldID = jni::get_field_id(env, Class, "haveUnreceivedNotifications", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateDeleteMessages::Class;
jfieldID updateDeleteMessages::chat_id_fieldID;
jfieldID updateDeleteMessages::message_ids_fieldID;
jfieldID updateDeleteMessages::is_permanent_fieldID;
jfieldID updateDeleteMessages::from_cache_fieldID;

updateDeleteMessages::updateDeleteMessages()
  : chat_id_()
  , message_ids_()
  , is_permanent_()
  , from_cache_()
{}

updateDeleteMessages::updateDeleteMessages(int53 chat_id_, array<int53> &&message_ids_, bool is_permanent_, bool from_cache_)
  : chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
  , is_permanent_(is_permanent_)
  , from_cache_(from_cache_)
{}

const std::int32_t updateDeleteMessages::ID;

object_ptr<Update> updateDeleteMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateDeleteMessages> res = make_object<updateDeleteMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->is_permanent_ = (env->GetBooleanField(p, res->is_permanent_fieldID) != 0);
  res->from_cache_ = (env->GetBooleanField(p, res->from_cache_fieldID) != 0);
  return std::move(res);
}

void updateDeleteMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { auto arr_tmp_ = jni::store_vector(env, message_ids_); if (arr_tmp_) { env->SetObjectField(s, message_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetBooleanField(s, is_permanent_fieldID, is_permanent_);
  env->SetBooleanField(s, from_cache_fieldID, from_cache_);
}

void updateDeleteMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateDeleteMessages");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("isPermanent", is_permanent_);
    s.store_field("fromCache", from_cache_);
    s.store_class_end();
  }
}

void updateDeleteMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateDeleteMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    is_permanent_fieldID = jni::get_field_id(env, Class, "isPermanent", "Z");
    from_cache_fieldID = jni::get_field_id(env, Class, "fromCache", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatAction::Class;
jfieldID updateChatAction::chat_id_fieldID;
jfieldID updateChatAction::message_thread_id_fieldID;
jfieldID updateChatAction::sender_id_fieldID;
jfieldID updateChatAction::action_fieldID;

updateChatAction::updateChatAction()
  : chat_id_()
  , message_thread_id_()
  , sender_id_()
  , action_()
{}

updateChatAction::updateChatAction(int53 chat_id_, int53 message_thread_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<ChatAction> &&action_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , sender_id_(std::move(sender_id_))
  , action_(std::move(action_))
{}

const std::int32_t updateChatAction::ID;

object_ptr<Update> updateChatAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatAction> res = make_object<updateChatAction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->action_ = jni::fetch_tl_object<ChatAction>(env, jni::fetch_object(env, p, res->action_fieldID));
  return std::move(res);
}

void updateChatAction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_thread_id_fieldID, message_thread_id_);
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (action_ != nullptr) { jobject next; action_->store(env, next); if (next) { env->SetObjectField(s, action_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatAction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatAction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_object_field("action", static_cast<const BaseObject *>(action_.get()));
    s.store_class_end();
  }
}

void updateChatAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatAction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    action_fieldID = jni::get_field_id(env, Class, "action", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateUserStatus::Class;
jfieldID updateUserStatus::user_id_fieldID;
jfieldID updateUserStatus::status_fieldID;

updateUserStatus::updateUserStatus()
  : user_id_()
  , status_()
{}

updateUserStatus::updateUserStatus(int53 user_id_, object_ptr<UserStatus> &&status_)
  : user_id_(user_id_)
  , status_(std::move(status_))
{}

const std::int32_t updateUserStatus::ID;

object_ptr<Update> updateUserStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUserStatus> res = make_object<updateUserStatus>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->status_ = jni::fetch_tl_object<UserStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  return std::move(res);
}

void updateUserStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateUserStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUserStatus");
    s.store_field("userId", user_id_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_class_end();
  }
}

void updateUserStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUserStatus").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateUser::Class;
jfieldID updateUser::user_fieldID;

updateUser::updateUser()
  : user_()
{}

updateUser::updateUser(object_ptr<user> &&user_)
  : user_(std::move(user_))
{}

const std::int32_t updateUser::ID;

object_ptr<Update> updateUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUser> res = make_object<updateUser>();
  res->user_ = jni::fetch_tl_object<user>(env, jni::fetch_object(env, p, res->user_fieldID));
  return std::move(res);
}

void updateUser::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (user_ != nullptr) { jobject next; user_->store(env, next); if (next) { env->SetObjectField(s, user_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUser");
    s.store_object_field("user", static_cast<const BaseObject *>(user_.get()));
    s.store_class_end();
  }
}

void updateUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUser").c_str());
    user_fieldID = jni::get_field_id(env, Class, "user", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$User;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateBasicGroup::Class;
jfieldID updateBasicGroup::basic_group_fieldID;

updateBasicGroup::updateBasicGroup()
  : basic_group_()
{}

updateBasicGroup::updateBasicGroup(object_ptr<basicGroup> &&basic_group_)
  : basic_group_(std::move(basic_group_))
{}

const std::int32_t updateBasicGroup::ID;

object_ptr<Update> updateBasicGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateBasicGroup> res = make_object<updateBasicGroup>();
  res->basic_group_ = jni::fetch_tl_object<basicGroup>(env, jni::fetch_object(env, p, res->basic_group_fieldID));
  return std::move(res);
}

void updateBasicGroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (basic_group_ != nullptr) { jobject next; basic_group_->store(env, next); if (next) { env->SetObjectField(s, basic_group_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateBasicGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateBasicGroup");
    s.store_object_field("basicGroup", static_cast<const BaseObject *>(basic_group_.get()));
    s.store_class_end();
  }
}

void updateBasicGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateBasicGroup").c_str());
    basic_group_fieldID = jni::get_field_id(env, Class, "basicGroup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BasicGroup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSupergroup::Class;
jfieldID updateSupergroup::supergroup_fieldID;

updateSupergroup::updateSupergroup()
  : supergroup_()
{}

updateSupergroup::updateSupergroup(object_ptr<supergroup> &&supergroup_)
  : supergroup_(std::move(supergroup_))
{}

const std::int32_t updateSupergroup::ID;

object_ptr<Update> updateSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSupergroup> res = make_object<updateSupergroup>();
  res->supergroup_ = jni::fetch_tl_object<supergroup>(env, jni::fetch_object(env, p, res->supergroup_fieldID));
  return std::move(res);
}

void updateSupergroup::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (supergroup_ != nullptr) { jobject next; supergroup_->store(env, next); if (next) { env->SetObjectField(s, supergroup_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSupergroup");
    s.store_object_field("supergroup", static_cast<const BaseObject *>(supergroup_.get()));
    s.store_class_end();
  }
}

void updateSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSupergroup").c_str());
    supergroup_fieldID = jni::get_field_id(env, Class, "supergroup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Supergroup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSecretChat::Class;
jfieldID updateSecretChat::secret_chat_fieldID;

updateSecretChat::updateSecretChat()
  : secret_chat_()
{}

updateSecretChat::updateSecretChat(object_ptr<secretChat> &&secret_chat_)
  : secret_chat_(std::move(secret_chat_))
{}

const std::int32_t updateSecretChat::ID;

object_ptr<Update> updateSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSecretChat> res = make_object<updateSecretChat>();
  res->secret_chat_ = jni::fetch_tl_object<secretChat>(env, jni::fetch_object(env, p, res->secret_chat_fieldID));
  return std::move(res);
}

void updateSecretChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (secret_chat_ != nullptr) { jobject next; secret_chat_->store(env, next); if (next) { env->SetObjectField(s, secret_chat_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSecretChat");
    s.store_object_field("secretChat", static_cast<const BaseObject *>(secret_chat_.get()));
    s.store_class_end();
  }
}

void updateSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSecretChat").c_str());
    secret_chat_fieldID = jni::get_field_id(env, Class, "secretChat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SecretChat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateUserFullInfo::Class;
jfieldID updateUserFullInfo::user_id_fieldID;
jfieldID updateUserFullInfo::user_full_info_fieldID;

updateUserFullInfo::updateUserFullInfo()
  : user_id_()
  , user_full_info_()
{}

updateUserFullInfo::updateUserFullInfo(int53 user_id_, object_ptr<userFullInfo> &&user_full_info_)
  : user_id_(user_id_)
  , user_full_info_(std::move(user_full_info_))
{}

const std::int32_t updateUserFullInfo::ID;

object_ptr<Update> updateUserFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUserFullInfo> res = make_object<updateUserFullInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->user_full_info_ = jni::fetch_tl_object<userFullInfo>(env, jni::fetch_object(env, p, res->user_full_info_fieldID));
  return std::move(res);
}

void updateUserFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  if (user_full_info_ != nullptr) { jobject next; user_full_info_->store(env, next); if (next) { env->SetObjectField(s, user_full_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateUserFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUserFullInfo");
    s.store_field("userId", user_id_);
    s.store_object_field("userFullInfo", static_cast<const BaseObject *>(user_full_info_.get()));
    s.store_class_end();
  }
}

void updateUserFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUserFullInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    user_full_info_fieldID = jni::get_field_id(env, Class, "userFullInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserFullInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateBasicGroupFullInfo::Class;
jfieldID updateBasicGroupFullInfo::basic_group_id_fieldID;
jfieldID updateBasicGroupFullInfo::basic_group_full_info_fieldID;

updateBasicGroupFullInfo::updateBasicGroupFullInfo()
  : basic_group_id_()
  , basic_group_full_info_()
{}

updateBasicGroupFullInfo::updateBasicGroupFullInfo(int53 basic_group_id_, object_ptr<basicGroupFullInfo> &&basic_group_full_info_)
  : basic_group_id_(basic_group_id_)
  , basic_group_full_info_(std::move(basic_group_full_info_))
{}

const std::int32_t updateBasicGroupFullInfo::ID;

object_ptr<Update> updateBasicGroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateBasicGroupFullInfo> res = make_object<updateBasicGroupFullInfo>();
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  res->basic_group_full_info_ = jni::fetch_tl_object<basicGroupFullInfo>(env, jni::fetch_object(env, p, res->basic_group_full_info_fieldID));
  return std::move(res);
}

void updateBasicGroupFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, basic_group_id_fieldID, basic_group_id_);
  if (basic_group_full_info_ != nullptr) { jobject next; basic_group_full_info_->store(env, next); if (next) { env->SetObjectField(s, basic_group_full_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateBasicGroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateBasicGroupFullInfo");
    s.store_field("basicGroupId", basic_group_id_);
    s.store_object_field("basicGroupFullInfo", static_cast<const BaseObject *>(basic_group_full_info_.get()));
    s.store_class_end();
  }
}

void updateBasicGroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateBasicGroupFullInfo").c_str());
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    basic_group_full_info_fieldID = jni::get_field_id(env, Class, "basicGroupFullInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BasicGroupFullInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSupergroupFullInfo::Class;
jfieldID updateSupergroupFullInfo::supergroup_id_fieldID;
jfieldID updateSupergroupFullInfo::supergroup_full_info_fieldID;

updateSupergroupFullInfo::updateSupergroupFullInfo()
  : supergroup_id_()
  , supergroup_full_info_()
{}

updateSupergroupFullInfo::updateSupergroupFullInfo(int53 supergroup_id_, object_ptr<supergroupFullInfo> &&supergroup_full_info_)
  : supergroup_id_(supergroup_id_)
  , supergroup_full_info_(std::move(supergroup_full_info_))
{}

const std::int32_t updateSupergroupFullInfo::ID;

object_ptr<Update> updateSupergroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSupergroupFullInfo> res = make_object<updateSupergroupFullInfo>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->supergroup_full_info_ = jni::fetch_tl_object<supergroupFullInfo>(env, jni::fetch_object(env, p, res->supergroup_full_info_fieldID));
  return std::move(res);
}

void updateSupergroupFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, supergroup_id_fieldID, supergroup_id_);
  if (supergroup_full_info_ != nullptr) { jobject next; supergroup_full_info_->store(env, next); if (next) { env->SetObjectField(s, supergroup_full_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateSupergroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSupergroupFullInfo");
    s.store_field("supergroupId", supergroup_id_);
    s.store_object_field("supergroupFullInfo", static_cast<const BaseObject *>(supergroup_full_info_.get()));
    s.store_class_end();
  }
}

void updateSupergroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSupergroupFullInfo").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    supergroup_full_info_fieldID = jni::get_field_id(env, Class, "supergroupFullInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SupergroupFullInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateServiceNotification::Class;
jfieldID updateServiceNotification::type_fieldID;
jfieldID updateServiceNotification::content_fieldID;

updateServiceNotification::updateServiceNotification()
  : type_()
  , content_()
{}

updateServiceNotification::updateServiceNotification(string const &type_, object_ptr<MessageContent> &&content_)
  : type_(type_)
  , content_(std::move(content_))
{}

const std::int32_t updateServiceNotification::ID;

object_ptr<Update> updateServiceNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateServiceNotification> res = make_object<updateServiceNotification>();
  res->type_ = jni::fetch_string(env, p, res->type_fieldID);
  res->content_ = jni::fetch_tl_object<MessageContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return std::move(res);
}

void updateServiceNotification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, type_); if (nextString) { env->SetObjectField(s, type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (content_ != nullptr) { jobject next; content_->store(env, next); if (next) { env->SetObjectField(s, content_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateServiceNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateServiceNotification");
    s.store_field("type", type_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

void updateServiceNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateServiceNotification").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", "Ljava/lang/String;");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateFile::Class;
jfieldID updateFile::file_fieldID;

updateFile::updateFile()
  : file_()
{}

updateFile::updateFile(object_ptr<file> &&file_)
  : file_(std::move(file_))
{}

const std::int32_t updateFile::ID;

object_ptr<Update> updateFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFile> res = make_object<updateFile>();
  res->file_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->file_fieldID));
  return std::move(res);
}

void updateFile::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (file_ != nullptr) { jobject next; file_->store(env, next); if (next) { env->SetObjectField(s, file_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFile");
    s.store_object_field("file", static_cast<const BaseObject *>(file_.get()));
    s.store_class_end();
  }
}

void updateFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFile").c_str());
    file_fieldID = jni::get_field_id(env, Class, "file", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateFileGenerationStart::Class;
jfieldID updateFileGenerationStart::generation_id_fieldID;
jfieldID updateFileGenerationStart::original_path_fieldID;
jfieldID updateFileGenerationStart::destination_path_fieldID;
jfieldID updateFileGenerationStart::conversion_fieldID;

updateFileGenerationStart::updateFileGenerationStart()
  : generation_id_()
  , original_path_()
  , destination_path_()
  , conversion_()
{}

updateFileGenerationStart::updateFileGenerationStart(int64 generation_id_, string const &original_path_, string const &destination_path_, string const &conversion_)
  : generation_id_(generation_id_)
  , original_path_(original_path_)
  , destination_path_(destination_path_)
  , conversion_(conversion_)
{}

const std::int32_t updateFileGenerationStart::ID;

object_ptr<Update> updateFileGenerationStart::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileGenerationStart> res = make_object<updateFileGenerationStart>();
  res->generation_id_ = env->GetLongField(p, res->generation_id_fieldID);
  res->original_path_ = jni::fetch_string(env, p, res->original_path_fieldID);
  res->destination_path_ = jni::fetch_string(env, p, res->destination_path_fieldID);
  res->conversion_ = jni::fetch_string(env, p, res->conversion_fieldID);
  return std::move(res);
}

void updateFileGenerationStart::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, generation_id_fieldID, generation_id_);
  { jstring nextString = jni::to_jstring(env, original_path_); if (nextString) { env->SetObjectField(s, original_path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, destination_path_); if (nextString) { env->SetObjectField(s, destination_path_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, conversion_); if (nextString) { env->SetObjectField(s, conversion_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateFileGenerationStart::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileGenerationStart");
    s.store_field("generationId", generation_id_);
    s.store_field("originalPath", original_path_);
    s.store_field("destinationPath", destination_path_);
    s.store_field("conversion", conversion_);
    s.store_class_end();
  }
}

void updateFileGenerationStart::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileGenerationStart").c_str());
    generation_id_fieldID = jni::get_field_id(env, Class, "generationId", "J");
    original_path_fieldID = jni::get_field_id(env, Class, "originalPath", "Ljava/lang/String;");
    destination_path_fieldID = jni::get_field_id(env, Class, "destinationPath", "Ljava/lang/String;");
    conversion_fieldID = jni::get_field_id(env, Class, "conversion", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateFileGenerationStop::Class;
jfieldID updateFileGenerationStop::generation_id_fieldID;

updateFileGenerationStop::updateFileGenerationStop()
  : generation_id_()
{}

updateFileGenerationStop::updateFileGenerationStop(int64 generation_id_)
  : generation_id_(generation_id_)
{}

const std::int32_t updateFileGenerationStop::ID;

object_ptr<Update> updateFileGenerationStop::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileGenerationStop> res = make_object<updateFileGenerationStop>();
  res->generation_id_ = env->GetLongField(p, res->generation_id_fieldID);
  return std::move(res);
}

void updateFileGenerationStop::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, generation_id_fieldID, generation_id_);
}

void updateFileGenerationStop::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileGenerationStop");
    s.store_field("generationId", generation_id_);
    s.store_class_end();
  }
}

void updateFileGenerationStop::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileGenerationStop").c_str());
    generation_id_fieldID = jni::get_field_id(env, Class, "generationId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateFileDownloads::Class;
jfieldID updateFileDownloads::total_size_fieldID;
jfieldID updateFileDownloads::total_count_fieldID;
jfieldID updateFileDownloads::downloaded_size_fieldID;

updateFileDownloads::updateFileDownloads()
  : total_size_()
  , total_count_()
  , downloaded_size_()
{}

updateFileDownloads::updateFileDownloads(int53 total_size_, int32 total_count_, int53 downloaded_size_)
  : total_size_(total_size_)
  , total_count_(total_count_)
  , downloaded_size_(downloaded_size_)
{}

const std::int32_t updateFileDownloads::ID;

object_ptr<Update> updateFileDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileDownloads> res = make_object<updateFileDownloads>();
  res->total_size_ = env->GetLongField(p, res->total_size_fieldID);
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->downloaded_size_ = env->GetLongField(p, res->downloaded_size_fieldID);
  return std::move(res);
}

void updateFileDownloads::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, total_size_fieldID, total_size_);
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetLongField(s, downloaded_size_fieldID, downloaded_size_);
}

void updateFileDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileDownloads");
    s.store_field("totalSize", total_size_);
    s.store_field("totalCount", total_count_);
    s.store_field("downloadedSize", downloaded_size_);
    s.store_class_end();
  }
}

void updateFileDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileDownloads").c_str());
    total_size_fieldID = jni::get_field_id(env, Class, "totalSize", "J");
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    downloaded_size_fieldID = jni::get_field_id(env, Class, "downloadedSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateFileAddedToDownloads::Class;
jfieldID updateFileAddedToDownloads::file_download_fieldID;
jfieldID updateFileAddedToDownloads::counts_fieldID;

updateFileAddedToDownloads::updateFileAddedToDownloads()
  : file_download_()
  , counts_()
{}

updateFileAddedToDownloads::updateFileAddedToDownloads(object_ptr<fileDownload> &&file_download_, object_ptr<downloadedFileCounts> &&counts_)
  : file_download_(std::move(file_download_))
  , counts_(std::move(counts_))
{}

const std::int32_t updateFileAddedToDownloads::ID;

object_ptr<Update> updateFileAddedToDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileAddedToDownloads> res = make_object<updateFileAddedToDownloads>();
  res->file_download_ = jni::fetch_tl_object<fileDownload>(env, jni::fetch_object(env, p, res->file_download_fieldID));
  res->counts_ = jni::fetch_tl_object<downloadedFileCounts>(env, jni::fetch_object(env, p, res->counts_fieldID));
  return std::move(res);
}

void updateFileAddedToDownloads::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (file_download_ != nullptr) { jobject next; file_download_->store(env, next); if (next) { env->SetObjectField(s, file_download_fieldID, next); env->DeleteLocalRef(next); } }
  if (counts_ != nullptr) { jobject next; counts_->store(env, next); if (next) { env->SetObjectField(s, counts_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateFileAddedToDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileAddedToDownloads");
    s.store_object_field("fileDownload", static_cast<const BaseObject *>(file_download_.get()));
    s.store_object_field("counts", static_cast<const BaseObject *>(counts_.get()));
    s.store_class_end();
  }
}

void updateFileAddedToDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileAddedToDownloads").c_str());
    file_download_fieldID = jni::get_field_id(env, Class, "fileDownload", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FileDownload;").c_str());
    counts_fieldID = jni::get_field_id(env, Class, "counts", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DownloadedFileCounts;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateFileDownload::Class;
jfieldID updateFileDownload::file_id_fieldID;
jfieldID updateFileDownload::complete_date_fieldID;
jfieldID updateFileDownload::is_paused_fieldID;
jfieldID updateFileDownload::counts_fieldID;

updateFileDownload::updateFileDownload()
  : file_id_()
  , complete_date_()
  , is_paused_()
  , counts_()
{}

updateFileDownload::updateFileDownload(int32 file_id_, int32 complete_date_, bool is_paused_, object_ptr<downloadedFileCounts> &&counts_)
  : file_id_(file_id_)
  , complete_date_(complete_date_)
  , is_paused_(is_paused_)
  , counts_(std::move(counts_))
{}

const std::int32_t updateFileDownload::ID;

object_ptr<Update> updateFileDownload::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileDownload> res = make_object<updateFileDownload>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->complete_date_ = env->GetIntField(p, res->complete_date_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  res->counts_ = jni::fetch_tl_object<downloadedFileCounts>(env, jni::fetch_object(env, p, res->counts_fieldID));
  return std::move(res);
}

void updateFileDownload::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, file_id_fieldID, file_id_);
  env->SetIntField(s, complete_date_fieldID, complete_date_);
  env->SetBooleanField(s, is_paused_fieldID, is_paused_);
  if (counts_ != nullptr) { jobject next; counts_->store(env, next); if (next) { env->SetObjectField(s, counts_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateFileDownload::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileDownload");
    s.store_field("fileId", file_id_);
    s.store_field("completeDate", complete_date_);
    s.store_field("isPaused", is_paused_);
    s.store_object_field("counts", static_cast<const BaseObject *>(counts_.get()));
    s.store_class_end();
  }
}

void updateFileDownload::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileDownload").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    complete_date_fieldID = jni::get_field_id(env, Class, "completeDate", "I");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    counts_fieldID = jni::get_field_id(env, Class, "counts", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DownloadedFileCounts;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateFileRemovedFromDownloads::Class;
jfieldID updateFileRemovedFromDownloads::file_id_fieldID;
jfieldID updateFileRemovedFromDownloads::counts_fieldID;

updateFileRemovedFromDownloads::updateFileRemovedFromDownloads()
  : file_id_()
  , counts_()
{}

updateFileRemovedFromDownloads::updateFileRemovedFromDownloads(int32 file_id_, object_ptr<downloadedFileCounts> &&counts_)
  : file_id_(file_id_)
  , counts_(std::move(counts_))
{}

const std::int32_t updateFileRemovedFromDownloads::ID;

object_ptr<Update> updateFileRemovedFromDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFileRemovedFromDownloads> res = make_object<updateFileRemovedFromDownloads>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->counts_ = jni::fetch_tl_object<downloadedFileCounts>(env, jni::fetch_object(env, p, res->counts_fieldID));
  return std::move(res);
}

void updateFileRemovedFromDownloads::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, file_id_fieldID, file_id_);
  if (counts_ != nullptr) { jobject next; counts_->store(env, next); if (next) { env->SetObjectField(s, counts_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateFileRemovedFromDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFileRemovedFromDownloads");
    s.store_field("fileId", file_id_);
    s.store_object_field("counts", static_cast<const BaseObject *>(counts_.get()));
    s.store_class_end();
  }
}

void updateFileRemovedFromDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFileRemovedFromDownloads").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    counts_fieldID = jni::get_field_id(env, Class, "counts", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DownloadedFileCounts;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateApplicationVerificationRequired::Class;
jfieldID updateApplicationVerificationRequired::verification_id_fieldID;
jfieldID updateApplicationVerificationRequired::nonce_fieldID;
jfieldID updateApplicationVerificationRequired::cloud_project_number_fieldID;

updateApplicationVerificationRequired::updateApplicationVerificationRequired()
  : verification_id_()
  , nonce_()
  , cloud_project_number_()
{}

updateApplicationVerificationRequired::updateApplicationVerificationRequired(int53 verification_id_, string const &nonce_, int64 cloud_project_number_)
  : verification_id_(verification_id_)
  , nonce_(nonce_)
  , cloud_project_number_(cloud_project_number_)
{}

const std::int32_t updateApplicationVerificationRequired::ID;

object_ptr<Update> updateApplicationVerificationRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateApplicationVerificationRequired> res = make_object<updateApplicationVerificationRequired>();
  res->verification_id_ = env->GetLongField(p, res->verification_id_fieldID);
  res->nonce_ = jni::fetch_string(env, p, res->nonce_fieldID);
  res->cloud_project_number_ = env->GetLongField(p, res->cloud_project_number_fieldID);
  return std::move(res);
}

void updateApplicationVerificationRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, verification_id_fieldID, verification_id_);
  { jstring nextString = jni::to_jstring(env, nonce_); if (nextString) { env->SetObjectField(s, nonce_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, cloud_project_number_fieldID, cloud_project_number_);
}

void updateApplicationVerificationRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateApplicationVerificationRequired");
    s.store_field("verificationId", verification_id_);
    s.store_field("nonce", nonce_);
    s.store_field("cloudProjectNumber", cloud_project_number_);
    s.store_class_end();
  }
}

void updateApplicationVerificationRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateApplicationVerificationRequired").c_str());
    verification_id_fieldID = jni::get_field_id(env, Class, "verificationId", "J");
    nonce_fieldID = jni::get_field_id(env, Class, "nonce", "Ljava/lang/String;");
    cloud_project_number_fieldID = jni::get_field_id(env, Class, "cloudProjectNumber", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateApplicationRecaptchaVerificationRequired::Class;
jfieldID updateApplicationRecaptchaVerificationRequired::verification_id_fieldID;
jfieldID updateApplicationRecaptchaVerificationRequired::action_fieldID;
jfieldID updateApplicationRecaptchaVerificationRequired::recaptcha_key_id_fieldID;

updateApplicationRecaptchaVerificationRequired::updateApplicationRecaptchaVerificationRequired()
  : verification_id_()
  , action_()
  , recaptcha_key_id_()
{}

updateApplicationRecaptchaVerificationRequired::updateApplicationRecaptchaVerificationRequired(int53 verification_id_, string const &action_, string const &recaptcha_key_id_)
  : verification_id_(verification_id_)
  , action_(action_)
  , recaptcha_key_id_(recaptcha_key_id_)
{}

const std::int32_t updateApplicationRecaptchaVerificationRequired::ID;

object_ptr<Update> updateApplicationRecaptchaVerificationRequired::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateApplicationRecaptchaVerificationRequired> res = make_object<updateApplicationRecaptchaVerificationRequired>();
  res->verification_id_ = env->GetLongField(p, res->verification_id_fieldID);
  res->action_ = jni::fetch_string(env, p, res->action_fieldID);
  res->recaptcha_key_id_ = jni::fetch_string(env, p, res->recaptcha_key_id_fieldID);
  return std::move(res);
}

void updateApplicationRecaptchaVerificationRequired::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, verification_id_fieldID, verification_id_);
  { jstring nextString = jni::to_jstring(env, action_); if (nextString) { env->SetObjectField(s, action_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, recaptcha_key_id_); if (nextString) { env->SetObjectField(s, recaptcha_key_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateApplicationRecaptchaVerificationRequired::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateApplicationRecaptchaVerificationRequired");
    s.store_field("verificationId", verification_id_);
    s.store_field("action", action_);
    s.store_field("recaptchaKeyId", recaptcha_key_id_);
    s.store_class_end();
  }
}

void updateApplicationRecaptchaVerificationRequired::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateApplicationRecaptchaVerificationRequired").c_str());
    verification_id_fieldID = jni::get_field_id(env, Class, "verificationId", "J");
    action_fieldID = jni::get_field_id(env, Class, "action", "Ljava/lang/String;");
    recaptcha_key_id_fieldID = jni::get_field_id(env, Class, "recaptchaKeyId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateCall::Class;
jfieldID updateCall::call_fieldID;

updateCall::updateCall()
  : call_()
{}

updateCall::updateCall(object_ptr<call> &&call_)
  : call_(std::move(call_))
{}

const std::int32_t updateCall::ID;

object_ptr<Update> updateCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateCall> res = make_object<updateCall>();
  res->call_ = jni::fetch_tl_object<call>(env, jni::fetch_object(env, p, res->call_fieldID));
  return std::move(res);
}

void updateCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (call_ != nullptr) { jobject next; call_->store(env, next); if (next) { env->SetObjectField(s, call_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateCall");
    s.store_object_field("call", static_cast<const BaseObject *>(call_.get()));
    s.store_class_end();
  }
}

void updateCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateCall").c_str());
    call_fieldID = jni::get_field_id(env, Class, "call", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Call;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateGroupCall::Class;
jfieldID updateGroupCall::group_call_fieldID;

updateGroupCall::updateGroupCall()
  : group_call_()
{}

updateGroupCall::updateGroupCall(object_ptr<groupCall> &&group_call_)
  : group_call_(std::move(group_call_))
{}

const std::int32_t updateGroupCall::ID;

object_ptr<Update> updateGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateGroupCall> res = make_object<updateGroupCall>();
  res->group_call_ = jni::fetch_tl_object<groupCall>(env, jni::fetch_object(env, p, res->group_call_fieldID));
  return std::move(res);
}

void updateGroupCall::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (group_call_ != nullptr) { jobject next; group_call_->store(env, next); if (next) { env->SetObjectField(s, group_call_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateGroupCall");
    s.store_object_field("groupCall", static_cast<const BaseObject *>(group_call_.get()));
    s.store_class_end();
  }
}

void updateGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateGroupCall").c_str());
    group_call_fieldID = jni::get_field_id(env, Class, "groupCall", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCall;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateGroupCallParticipant::Class;
jfieldID updateGroupCallParticipant::group_call_id_fieldID;
jfieldID updateGroupCallParticipant::participant_fieldID;

updateGroupCallParticipant::updateGroupCallParticipant()
  : group_call_id_()
  , participant_()
{}

updateGroupCallParticipant::updateGroupCallParticipant(int32 group_call_id_, object_ptr<groupCallParticipant> &&participant_)
  : group_call_id_(group_call_id_)
  , participant_(std::move(participant_))
{}

const std::int32_t updateGroupCallParticipant::ID;

object_ptr<Update> updateGroupCallParticipant::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateGroupCallParticipant> res = make_object<updateGroupCallParticipant>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_ = jni::fetch_tl_object<groupCallParticipant>(env, jni::fetch_object(env, p, res->participant_fieldID));
  return std::move(res);
}

void updateGroupCallParticipant::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  if (participant_ != nullptr) { jobject next; participant_->store(env, next); if (next) { env->SetObjectField(s, participant_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateGroupCallParticipant::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateGroupCallParticipant");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participant", static_cast<const BaseObject *>(participant_.get()));
    s.store_class_end();
  }
}

void updateGroupCallParticipant::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateGroupCallParticipant").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_fieldID = jni::get_field_id(env, Class, "participant", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallParticipant;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateGroupCallParticipants::Class;
jfieldID updateGroupCallParticipants::group_call_id_fieldID;
jfieldID updateGroupCallParticipants::participant_user_ids_fieldID;

updateGroupCallParticipants::updateGroupCallParticipants()
  : group_call_id_()
  , participant_user_ids_()
{}

updateGroupCallParticipants::updateGroupCallParticipants(int32 group_call_id_, array<int64> &&participant_user_ids_)
  : group_call_id_(group_call_id_)
  , participant_user_ids_(std::move(participant_user_ids_))
{}

const std::int32_t updateGroupCallParticipants::ID;

object_ptr<Update> updateGroupCallParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateGroupCallParticipants> res = make_object<updateGroupCallParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->participant_user_ids_fieldID));
  return std::move(res);
}

void updateGroupCallParticipants::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  { auto arr_tmp_ = jni::store_vector(env, participant_user_ids_); if (arr_tmp_) { env->SetObjectField(s, participant_user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateGroupCallParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateGroupCallParticipants");
    s.store_field("groupCallId", group_call_id_);
    { s.store_vector_begin("participantUserIds", participant_user_ids_.size()); for (const auto &_value : participant_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateGroupCallParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateGroupCallParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_user_ids_fieldID = jni::get_field_id(env, Class, "participantUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass updateGroupCallVerificationState::Class;
jfieldID updateGroupCallVerificationState::group_call_id_fieldID;
jfieldID updateGroupCallVerificationState::generation_fieldID;
jfieldID updateGroupCallVerificationState::emojis_fieldID;

updateGroupCallVerificationState::updateGroupCallVerificationState()
  : group_call_id_()
  , generation_()
  , emojis_()
{}

updateGroupCallVerificationState::updateGroupCallVerificationState(int32 group_call_id_, int32 generation_, array<string> &&emojis_)
  : group_call_id_(group_call_id_)
  , generation_(generation_)
  , emojis_(std::move(emojis_))
{}

const std::int32_t updateGroupCallVerificationState::ID;

object_ptr<Update> updateGroupCallVerificationState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateGroupCallVerificationState> res = make_object<updateGroupCallVerificationState>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->generation_ = env->GetIntField(p, res->generation_fieldID);
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return std::move(res);
}

void updateGroupCallVerificationState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  env->SetIntField(s, generation_fieldID, generation_);
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateGroupCallVerificationState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateGroupCallVerificationState");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("generation", generation_);
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateGroupCallVerificationState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateGroupCallVerificationState").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    generation_fieldID = jni::get_field_id(env, Class, "generation", "I");
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewCallSignalingData::Class;
jfieldID updateNewCallSignalingData::call_id_fieldID;
jfieldID updateNewCallSignalingData::data_fieldID;

updateNewCallSignalingData::updateNewCallSignalingData()
  : call_id_()
  , data_()
{}

updateNewCallSignalingData::updateNewCallSignalingData(int32 call_id_, bytes const &data_)
  : call_id_(call_id_)
  , data_(std::move(data_))
{}

const std::int32_t updateNewCallSignalingData::ID;

object_ptr<Update> updateNewCallSignalingData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewCallSignalingData> res = make_object<updateNewCallSignalingData>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return std::move(res);
}

void updateNewCallSignalingData::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, call_id_fieldID, call_id_);
  { jbyteArray nextBytes = jni::to_bytes(env, data_); if (nextBytes) { env->SetObjectField(s, data_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
}

void updateNewCallSignalingData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewCallSignalingData");
    s.store_field("callId", call_id_);
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

void updateNewCallSignalingData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewCallSignalingData").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass updateUserPrivacySettingRules::Class;
jfieldID updateUserPrivacySettingRules::setting_fieldID;
jfieldID updateUserPrivacySettingRules::rules_fieldID;

updateUserPrivacySettingRules::updateUserPrivacySettingRules()
  : setting_()
  , rules_()
{}

updateUserPrivacySettingRules::updateUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_, object_ptr<userPrivacySettingRules> &&rules_)
  : setting_(std::move(setting_))
  , rules_(std::move(rules_))
{}

const std::int32_t updateUserPrivacySettingRules::ID;

object_ptr<Update> updateUserPrivacySettingRules::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUserPrivacySettingRules> res = make_object<updateUserPrivacySettingRules>();
  res->setting_ = jni::fetch_tl_object<UserPrivacySetting>(env, jni::fetch_object(env, p, res->setting_fieldID));
  res->rules_ = jni::fetch_tl_object<userPrivacySettingRules>(env, jni::fetch_object(env, p, res->rules_fieldID));
  return std::move(res);
}

void updateUserPrivacySettingRules::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (setting_ != nullptr) { jobject next; setting_->store(env, next); if (next) { env->SetObjectField(s, setting_fieldID, next); env->DeleteLocalRef(next); } }
  if (rules_ != nullptr) { jobject next; rules_->store(env, next); if (next) { env->SetObjectField(s, rules_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateUserPrivacySettingRules::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUserPrivacySettingRules");
    s.store_object_field("setting", static_cast<const BaseObject *>(setting_.get()));
    s.store_object_field("rules", static_cast<const BaseObject *>(rules_.get()));
    s.store_class_end();
  }
}

void updateUserPrivacySettingRules::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUserPrivacySettingRules").c_str());
    setting_fieldID = jni::get_field_id(env, Class, "setting", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserPrivacySetting;").c_str());
    rules_fieldID = jni::get_field_id(env, Class, "rules", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserPrivacySettingRules;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateUnreadMessageCount::Class;
jfieldID updateUnreadMessageCount::chat_list_fieldID;
jfieldID updateUnreadMessageCount::unread_count_fieldID;
jfieldID updateUnreadMessageCount::unread_unmuted_count_fieldID;

updateUnreadMessageCount::updateUnreadMessageCount()
  : chat_list_()
  , unread_count_()
  , unread_unmuted_count_()
{}

updateUnreadMessageCount::updateUnreadMessageCount(object_ptr<ChatList> &&chat_list_, int32 unread_count_, int32 unread_unmuted_count_)
  : chat_list_(std::move(chat_list_))
  , unread_count_(unread_count_)
  , unread_unmuted_count_(unread_unmuted_count_)
{}

const std::int32_t updateUnreadMessageCount::ID;

object_ptr<Update> updateUnreadMessageCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUnreadMessageCount> res = make_object<updateUnreadMessageCount>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->unread_count_ = env->GetIntField(p, res->unread_count_fieldID);
  res->unread_unmuted_count_ = env->GetIntField(p, res->unread_unmuted_count_fieldID);
  return std::move(res);
}

void updateUnreadMessageCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chat_list_ != nullptr) { jobject next; chat_list_->store(env, next); if (next) { env->SetObjectField(s, chat_list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, unread_count_fieldID, unread_count_);
  env->SetIntField(s, unread_unmuted_count_fieldID, unread_unmuted_count_);
}

void updateUnreadMessageCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUnreadMessageCount");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("unreadCount", unread_count_);
    s.store_field("unreadUnmutedCount", unread_unmuted_count_);
    s.store_class_end();
  }
}

void updateUnreadMessageCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUnreadMessageCount").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    unread_count_fieldID = jni::get_field_id(env, Class, "unreadCount", "I");
    unread_unmuted_count_fieldID = jni::get_field_id(env, Class, "unreadUnmutedCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateUnreadChatCount::Class;
jfieldID updateUnreadChatCount::chat_list_fieldID;
jfieldID updateUnreadChatCount::total_count_fieldID;
jfieldID updateUnreadChatCount::unread_count_fieldID;
jfieldID updateUnreadChatCount::unread_unmuted_count_fieldID;
jfieldID updateUnreadChatCount::marked_as_unread_count_fieldID;
jfieldID updateUnreadChatCount::marked_as_unread_unmuted_count_fieldID;

updateUnreadChatCount::updateUnreadChatCount()
  : chat_list_()
  , total_count_()
  , unread_count_()
  , unread_unmuted_count_()
  , marked_as_unread_count_()
  , marked_as_unread_unmuted_count_()
{}

updateUnreadChatCount::updateUnreadChatCount(object_ptr<ChatList> &&chat_list_, int32 total_count_, int32 unread_count_, int32 unread_unmuted_count_, int32 marked_as_unread_count_, int32 marked_as_unread_unmuted_count_)
  : chat_list_(std::move(chat_list_))
  , total_count_(total_count_)
  , unread_count_(unread_count_)
  , unread_unmuted_count_(unread_unmuted_count_)
  , marked_as_unread_count_(marked_as_unread_count_)
  , marked_as_unread_unmuted_count_(marked_as_unread_unmuted_count_)
{}

const std::int32_t updateUnreadChatCount::ID;

object_ptr<Update> updateUnreadChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUnreadChatCount> res = make_object<updateUnreadChatCount>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->unread_count_ = env->GetIntField(p, res->unread_count_fieldID);
  res->unread_unmuted_count_ = env->GetIntField(p, res->unread_unmuted_count_fieldID);
  res->marked_as_unread_count_ = env->GetIntField(p, res->marked_as_unread_count_fieldID);
  res->marked_as_unread_unmuted_count_ = env->GetIntField(p, res->marked_as_unread_unmuted_count_fieldID);
  return std::move(res);
}

void updateUnreadChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (chat_list_ != nullptr) { jobject next; chat_list_->store(env, next); if (next) { env->SetObjectField(s, chat_list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
  env->SetIntField(s, unread_count_fieldID, unread_count_);
  env->SetIntField(s, unread_unmuted_count_fieldID, unread_unmuted_count_);
  env->SetIntField(s, marked_as_unread_count_fieldID, marked_as_unread_count_);
  env->SetIntField(s, marked_as_unread_unmuted_count_fieldID, marked_as_unread_unmuted_count_);
}

void updateUnreadChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUnreadChatCount");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("totalCount", total_count_);
    s.store_field("unreadCount", unread_count_);
    s.store_field("unreadUnmutedCount", unread_unmuted_count_);
    s.store_field("markedAsUnreadCount", marked_as_unread_count_);
    s.store_field("markedAsUnreadUnmutedCount", marked_as_unread_unmuted_count_);
    s.store_class_end();
  }
}

void updateUnreadChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUnreadChatCount").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    unread_count_fieldID = jni::get_field_id(env, Class, "unreadCount", "I");
    unread_unmuted_count_fieldID = jni::get_field_id(env, Class, "unreadUnmutedCount", "I");
    marked_as_unread_count_fieldID = jni::get_field_id(env, Class, "markedAsUnreadCount", "I");
    marked_as_unread_unmuted_count_fieldID = jni::get_field_id(env, Class, "markedAsUnreadUnmutedCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateStory::Class;
jfieldID updateStory::story_fieldID;

updateStory::updateStory()
  : story_()
{}

updateStory::updateStory(object_ptr<story> &&story_)
  : story_(std::move(story_))
{}

const std::int32_t updateStory::ID;

object_ptr<Update> updateStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStory> res = make_object<updateStory>();
  res->story_ = jni::fetch_tl_object<story>(env, jni::fetch_object(env, p, res->story_fieldID));
  return std::move(res);
}

void updateStory::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_ != nullptr) { jobject next; story_->store(env, next); if (next) { env->SetObjectField(s, story_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStory");
    s.store_object_field("story", static_cast<const BaseObject *>(story_.get()));
    s.store_class_end();
  }
}

void updateStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStory").c_str());
    story_fieldID = jni::get_field_id(env, Class, "story", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateStoryDeleted::Class;
jfieldID updateStoryDeleted::story_poster_chat_id_fieldID;
jfieldID updateStoryDeleted::story_id_fieldID;

updateStoryDeleted::updateStoryDeleted()
  : story_poster_chat_id_()
  , story_id_()
{}

updateStoryDeleted::updateStoryDeleted(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t updateStoryDeleted::ID;

object_ptr<Update> updateStoryDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStoryDeleted> res = make_object<updateStoryDeleted>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return std::move(res);
}

void updateStoryDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, story_poster_chat_id_fieldID, story_poster_chat_id_);
  env->SetIntField(s, story_id_fieldID, story_id_);
}

void updateStoryDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStoryDeleted");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

void updateStoryDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStoryDeleted").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateStoryPostSucceeded::Class;
jfieldID updateStoryPostSucceeded::story_fieldID;
jfieldID updateStoryPostSucceeded::old_story_id_fieldID;

updateStoryPostSucceeded::updateStoryPostSucceeded()
  : story_()
  , old_story_id_()
{}

updateStoryPostSucceeded::updateStoryPostSucceeded(object_ptr<story> &&story_, int32 old_story_id_)
  : story_(std::move(story_))
  , old_story_id_(old_story_id_)
{}

const std::int32_t updateStoryPostSucceeded::ID;

object_ptr<Update> updateStoryPostSucceeded::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStoryPostSucceeded> res = make_object<updateStoryPostSucceeded>();
  res->story_ = jni::fetch_tl_object<story>(env, jni::fetch_object(env, p, res->story_fieldID));
  res->old_story_id_ = env->GetIntField(p, res->old_story_id_fieldID);
  return std::move(res);
}

void updateStoryPostSucceeded::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_ != nullptr) { jobject next; story_->store(env, next); if (next) { env->SetObjectField(s, story_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, old_story_id_fieldID, old_story_id_);
}

void updateStoryPostSucceeded::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStoryPostSucceeded");
    s.store_object_field("story", static_cast<const BaseObject *>(story_.get()));
    s.store_field("oldStoryId", old_story_id_);
    s.store_class_end();
  }
}

void updateStoryPostSucceeded::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStoryPostSucceeded").c_str());
    story_fieldID = jni::get_field_id(env, Class, "story", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    old_story_id_fieldID = jni::get_field_id(env, Class, "oldStoryId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateStoryPostFailed::Class;
jfieldID updateStoryPostFailed::story_fieldID;
jfieldID updateStoryPostFailed::error_fieldID;
jfieldID updateStoryPostFailed::error_type_fieldID;

updateStoryPostFailed::updateStoryPostFailed()
  : story_()
  , error_()
  , error_type_()
{}

updateStoryPostFailed::updateStoryPostFailed(object_ptr<story> &&story_, object_ptr<error> &&error_, object_ptr<CanPostStoryResult> &&error_type_)
  : story_(std::move(story_))
  , error_(std::move(error_))
  , error_type_(std::move(error_type_))
{}

const std::int32_t updateStoryPostFailed::ID;

object_ptr<Update> updateStoryPostFailed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStoryPostFailed> res = make_object<updateStoryPostFailed>();
  res->story_ = jni::fetch_tl_object<story>(env, jni::fetch_object(env, p, res->story_fieldID));
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  res->error_type_ = jni::fetch_tl_object<CanPostStoryResult>(env, jni::fetch_object(env, p, res->error_type_fieldID));
  return std::move(res);
}

void updateStoryPostFailed::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_ != nullptr) { jobject next; story_->store(env, next); if (next) { env->SetObjectField(s, story_fieldID, next); env->DeleteLocalRef(next); } }
  if (error_ != nullptr) { jobject next; error_->store(env, next); if (next) { env->SetObjectField(s, error_fieldID, next); env->DeleteLocalRef(next); } }
  if (error_type_ != nullptr) { jobject next; error_type_->store(env, next); if (next) { env->SetObjectField(s, error_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateStoryPostFailed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStoryPostFailed");
    s.store_object_field("story", static_cast<const BaseObject *>(story_.get()));
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_object_field("errorType", static_cast<const BaseObject *>(error_type_.get()));
    s.store_class_end();
  }
}

void updateStoryPostFailed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStoryPostFailed").c_str());
    story_fieldID = jni::get_field_id(env, Class, "story", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Story;").c_str());
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    error_type_fieldID = jni::get_field_id(env, Class, "errorType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CanPostStoryResult;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatActiveStories::Class;
jfieldID updateChatActiveStories::active_stories_fieldID;

updateChatActiveStories::updateChatActiveStories()
  : active_stories_()
{}

updateChatActiveStories::updateChatActiveStories(object_ptr<chatActiveStories> &&active_stories_)
  : active_stories_(std::move(active_stories_))
{}

const std::int32_t updateChatActiveStories::ID;

object_ptr<Update> updateChatActiveStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatActiveStories> res = make_object<updateChatActiveStories>();
  res->active_stories_ = jni::fetch_tl_object<chatActiveStories>(env, jni::fetch_object(env, p, res->active_stories_fieldID));
  return std::move(res);
}

void updateChatActiveStories::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (active_stories_ != nullptr) { jobject next; active_stories_->store(env, next); if (next) { env->SetObjectField(s, active_stories_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatActiveStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatActiveStories");
    s.store_object_field("activeStories", static_cast<const BaseObject *>(active_stories_.get()));
    s.store_class_end();
  }
}

void updateChatActiveStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatActiveStories").c_str());
    active_stories_fieldID = jni::get_field_id(env, Class, "activeStories", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatActiveStories;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateStoryListChatCount::Class;
jfieldID updateStoryListChatCount::story_list_fieldID;
jfieldID updateStoryListChatCount::chat_count_fieldID;

updateStoryListChatCount::updateStoryListChatCount()
  : story_list_()
  , chat_count_()
{}

updateStoryListChatCount::updateStoryListChatCount(object_ptr<StoryList> &&story_list_, int32 chat_count_)
  : story_list_(std::move(story_list_))
  , chat_count_(chat_count_)
{}

const std::int32_t updateStoryListChatCount::ID;

object_ptr<Update> updateStoryListChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStoryListChatCount> res = make_object<updateStoryListChatCount>();
  res->story_list_ = jni::fetch_tl_object<StoryList>(env, jni::fetch_object(env, p, res->story_list_fieldID));
  res->chat_count_ = env->GetIntField(p, res->chat_count_fieldID);
  return std::move(res);
}

void updateStoryListChatCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (story_list_ != nullptr) { jobject next; story_list_->store(env, next); if (next) { env->SetObjectField(s, story_list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, chat_count_fieldID, chat_count_);
}

void updateStoryListChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStoryListChatCount");
    s.store_object_field("storyList", static_cast<const BaseObject *>(story_list_.get()));
    s.store_field("chatCount", chat_count_);
    s.store_class_end();
  }
}

void updateStoryListChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStoryListChatCount").c_str());
    story_list_fieldID = jni::get_field_id(env, Class, "storyList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryList;").c_str());
    chat_count_fieldID = jni::get_field_id(env, Class, "chatCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateStoryStealthMode::Class;
jfieldID updateStoryStealthMode::active_until_date_fieldID;
jfieldID updateStoryStealthMode::cooldown_until_date_fieldID;

updateStoryStealthMode::updateStoryStealthMode()
  : active_until_date_()
  , cooldown_until_date_()
{}

updateStoryStealthMode::updateStoryStealthMode(int32 active_until_date_, int32 cooldown_until_date_)
  : active_until_date_(active_until_date_)
  , cooldown_until_date_(cooldown_until_date_)
{}

const std::int32_t updateStoryStealthMode::ID;

object_ptr<Update> updateStoryStealthMode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStoryStealthMode> res = make_object<updateStoryStealthMode>();
  res->active_until_date_ = env->GetIntField(p, res->active_until_date_fieldID);
  res->cooldown_until_date_ = env->GetIntField(p, res->cooldown_until_date_fieldID);
  return std::move(res);
}

void updateStoryStealthMode::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, active_until_date_fieldID, active_until_date_);
  env->SetIntField(s, cooldown_until_date_fieldID, cooldown_until_date_);
}

void updateStoryStealthMode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStoryStealthMode");
    s.store_field("activeUntilDate", active_until_date_);
    s.store_field("cooldownUntilDate", cooldown_until_date_);
    s.store_class_end();
  }
}

void updateStoryStealthMode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStoryStealthMode").c_str());
    active_until_date_fieldID = jni::get_field_id(env, Class, "activeUntilDate", "I");
    cooldown_until_date_fieldID = jni::get_field_id(env, Class, "cooldownUntilDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateOption::Class;
jfieldID updateOption::name_fieldID;
jfieldID updateOption::value_fieldID;

updateOption::updateOption()
  : name_()
  , value_()
{}

updateOption::updateOption(string const &name_, object_ptr<OptionValue> &&value_)
  : name_(name_)
  , value_(std::move(value_))
{}

const std::int32_t updateOption::ID;

object_ptr<Update> updateOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateOption> res = make_object<updateOption>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->value_ = jni::fetch_tl_object<OptionValue>(env, jni::fetch_object(env, p, res->value_fieldID));
  return std::move(res);
}

void updateOption::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (value_ != nullptr) { jobject next; value_->store(env, next); if (next) { env->SetObjectField(s, value_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateOption");
    s.store_field("name", name_);
    s.store_object_field("value", static_cast<const BaseObject *>(value_.get()));
    s.store_class_end();
  }
}

void updateOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateOption").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OptionValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateStickerSet::Class;
jfieldID updateStickerSet::sticker_set_fieldID;

updateStickerSet::updateStickerSet()
  : sticker_set_()
{}

updateStickerSet::updateStickerSet(object_ptr<stickerSet> &&sticker_set_)
  : sticker_set_(std::move(sticker_set_))
{}

const std::int32_t updateStickerSet::ID;

object_ptr<Update> updateStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStickerSet> res = make_object<updateStickerSet>();
  res->sticker_set_ = jni::fetch_tl_object<stickerSet>(env, jni::fetch_object(env, p, res->sticker_set_fieldID));
  return std::move(res);
}

void updateStickerSet::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_set_ != nullptr) { jobject next; sticker_set_->store(env, next); if (next) { env->SetObjectField(s, sticker_set_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStickerSet");
    s.store_object_field("stickerSet", static_cast<const BaseObject *>(sticker_set_.get()));
    s.store_class_end();
  }
}

void updateStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStickerSet").c_str());
    sticker_set_fieldID = jni::get_field_id(env, Class, "stickerSet", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerSet;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateInstalledStickerSets::Class;
jfieldID updateInstalledStickerSets::sticker_type_fieldID;
jfieldID updateInstalledStickerSets::sticker_set_ids_fieldID;

updateInstalledStickerSets::updateInstalledStickerSets()
  : sticker_type_()
  , sticker_set_ids_()
{}

updateInstalledStickerSets::updateInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, array<int64> &&sticker_set_ids_)
  : sticker_type_(std::move(sticker_type_))
  , sticker_set_ids_(std::move(sticker_set_ids_))
{}

const std::int32_t updateInstalledStickerSets::ID;

object_ptr<Update> updateInstalledStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateInstalledStickerSets> res = make_object<updateInstalledStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->sticker_set_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->sticker_set_ids_fieldID));
  return std::move(res);
}

void updateInstalledStickerSets::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_type_ != nullptr) { jobject next; sticker_type_->store(env, next); if (next) { env->SetObjectField(s, sticker_type_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, sticker_set_ids_); if (arr_tmp_) { env->SetObjectField(s, sticker_set_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateInstalledStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateInstalledStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    { s.store_vector_begin("stickerSetIds", sticker_set_ids_.size()); for (const auto &_value : sticker_set_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateInstalledStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateInstalledStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    sticker_set_ids_fieldID = jni::get_field_id(env, Class, "stickerSetIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass updateTrendingStickerSets::Class;
jfieldID updateTrendingStickerSets::sticker_type_fieldID;
jfieldID updateTrendingStickerSets::sticker_sets_fieldID;

updateTrendingStickerSets::updateTrendingStickerSets()
  : sticker_type_()
  , sticker_sets_()
{}

updateTrendingStickerSets::updateTrendingStickerSets(object_ptr<StickerType> &&sticker_type_, object_ptr<trendingStickerSets> &&sticker_sets_)
  : sticker_type_(std::move(sticker_type_))
  , sticker_sets_(std::move(sticker_sets_))
{}

const std::int32_t updateTrendingStickerSets::ID;

object_ptr<Update> updateTrendingStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateTrendingStickerSets> res = make_object<updateTrendingStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->sticker_sets_ = jni::fetch_tl_object<trendingStickerSets>(env, jni::fetch_object(env, p, res->sticker_sets_fieldID));
  return std::move(res);
}

void updateTrendingStickerSets::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sticker_type_ != nullptr) { jobject next; sticker_type_->store(env, next); if (next) { env->SetObjectField(s, sticker_type_fieldID, next); env->DeleteLocalRef(next); } }
  if (sticker_sets_ != nullptr) { jobject next; sticker_sets_->store(env, next); if (next) { env->SetObjectField(s, sticker_sets_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateTrendingStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateTrendingStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_object_field("stickerSets", static_cast<const BaseObject *>(sticker_sets_.get()));
    s.store_class_end();
  }
}

void updateTrendingStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateTrendingStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    sticker_sets_fieldID = jni::get_field_id(env, Class, "stickerSets", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TrendingStickerSets;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateRecentStickers::Class;
jfieldID updateRecentStickers::is_attached_fieldID;
jfieldID updateRecentStickers::sticker_ids_fieldID;

updateRecentStickers::updateRecentStickers()
  : is_attached_()
  , sticker_ids_()
{}

updateRecentStickers::updateRecentStickers(bool is_attached_, array<int32> &&sticker_ids_)
  : is_attached_(is_attached_)
  , sticker_ids_(std::move(sticker_ids_))
{}

const std::int32_t updateRecentStickers::ID;

object_ptr<Update> updateRecentStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateRecentStickers> res = make_object<updateRecentStickers>();
  res->is_attached_ = (env->GetBooleanField(p, res->is_attached_fieldID) != 0);
  res->sticker_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->sticker_ids_fieldID));
  return std::move(res);
}

void updateRecentStickers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_attached_fieldID, is_attached_);
  { auto arr_tmp_ = jni::store_vector(env, sticker_ids_); if (arr_tmp_) { env->SetObjectField(s, sticker_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateRecentStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateRecentStickers");
    s.store_field("isAttached", is_attached_);
    { s.store_vector_begin("stickerIds", sticker_ids_.size()); for (const auto &_value : sticker_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateRecentStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateRecentStickers").c_str());
    is_attached_fieldID = jni::get_field_id(env, Class, "isAttached", "Z");
    sticker_ids_fieldID = jni::get_field_id(env, Class, "stickerIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateFavoriteStickers::Class;
jfieldID updateFavoriteStickers::sticker_ids_fieldID;

updateFavoriteStickers::updateFavoriteStickers()
  : sticker_ids_()
{}

updateFavoriteStickers::updateFavoriteStickers(array<int32> &&sticker_ids_)
  : sticker_ids_(std::move(sticker_ids_))
{}

const std::int32_t updateFavoriteStickers::ID;

object_ptr<Update> updateFavoriteStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFavoriteStickers> res = make_object<updateFavoriteStickers>();
  res->sticker_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->sticker_ids_fieldID));
  return std::move(res);
}

void updateFavoriteStickers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, sticker_ids_); if (arr_tmp_) { env->SetObjectField(s, sticker_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateFavoriteStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFavoriteStickers");
    { s.store_vector_begin("stickerIds", sticker_ids_.size()); for (const auto &_value : sticker_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateFavoriteStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFavoriteStickers").c_str());
    sticker_ids_fieldID = jni::get_field_id(env, Class, "stickerIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateSavedAnimations::Class;
jfieldID updateSavedAnimations::animation_ids_fieldID;

updateSavedAnimations::updateSavedAnimations()
  : animation_ids_()
{}

updateSavedAnimations::updateSavedAnimations(array<int32> &&animation_ids_)
  : animation_ids_(std::move(animation_ids_))
{}

const std::int32_t updateSavedAnimations::ID;

object_ptr<Update> updateSavedAnimations::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSavedAnimations> res = make_object<updateSavedAnimations>();
  res->animation_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->animation_ids_fieldID));
  return std::move(res);
}

void updateSavedAnimations::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, animation_ids_); if (arr_tmp_) { env->SetObjectField(s, animation_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateSavedAnimations::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSavedAnimations");
    { s.store_vector_begin("animationIds", animation_ids_.size()); for (const auto &_value : animation_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateSavedAnimations::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSavedAnimations").c_str());
    animation_ids_fieldID = jni::get_field_id(env, Class, "animationIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateSavedNotificationSounds::Class;
jfieldID updateSavedNotificationSounds::notification_sound_ids_fieldID;

updateSavedNotificationSounds::updateSavedNotificationSounds()
  : notification_sound_ids_()
{}

updateSavedNotificationSounds::updateSavedNotificationSounds(array<int64> &&notification_sound_ids_)
  : notification_sound_ids_(std::move(notification_sound_ids_))
{}

const std::int32_t updateSavedNotificationSounds::ID;

object_ptr<Update> updateSavedNotificationSounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSavedNotificationSounds> res = make_object<updateSavedNotificationSounds>();
  res->notification_sound_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->notification_sound_ids_fieldID));
  return std::move(res);
}

void updateSavedNotificationSounds::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, notification_sound_ids_); if (arr_tmp_) { env->SetObjectField(s, notification_sound_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateSavedNotificationSounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSavedNotificationSounds");
    { s.store_vector_begin("notificationSoundIds", notification_sound_ids_.size()); for (const auto &_value : notification_sound_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateSavedNotificationSounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSavedNotificationSounds").c_str());
    notification_sound_ids_fieldID = jni::get_field_id(env, Class, "notificationSoundIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass updateDefaultBackground::Class;
jfieldID updateDefaultBackground::for_dark_theme_fieldID;
jfieldID updateDefaultBackground::background_fieldID;

updateDefaultBackground::updateDefaultBackground()
  : for_dark_theme_()
  , background_()
{}

updateDefaultBackground::updateDefaultBackground(bool for_dark_theme_, object_ptr<background> &&background_)
  : for_dark_theme_(for_dark_theme_)
  , background_(std::move(background_))
{}

const std::int32_t updateDefaultBackground::ID;

object_ptr<Update> updateDefaultBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateDefaultBackground> res = make_object<updateDefaultBackground>();
  res->for_dark_theme_ = (env->GetBooleanField(p, res->for_dark_theme_fieldID) != 0);
  res->background_ = jni::fetch_tl_object<background>(env, jni::fetch_object(env, p, res->background_fieldID));
  return std::move(res);
}

void updateDefaultBackground::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, for_dark_theme_fieldID, for_dark_theme_);
  if (background_ != nullptr) { jobject next; background_->store(env, next); if (next) { env->SetObjectField(s, background_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateDefaultBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateDefaultBackground");
    s.store_field("forDarkTheme", for_dark_theme_);
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_class_end();
  }
}

void updateDefaultBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateDefaultBackground").c_str());
    for_dark_theme_fieldID = jni::get_field_id(env, Class, "forDarkTheme", "Z");
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Background;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatThemes::Class;
jfieldID updateChatThemes::chat_themes_fieldID;

updateChatThemes::updateChatThemes()
  : chat_themes_()
{}

updateChatThemes::updateChatThemes(array<object_ptr<chatTheme>> &&chat_themes_)
  : chat_themes_(std::move(chat_themes_))
{}

const std::int32_t updateChatThemes::ID;

object_ptr<Update> updateChatThemes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatThemes> res = make_object<updateChatThemes>();
  res->chat_themes_ = jni::FetchVector<chatTheme>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->chat_themes_fieldID));
  return std::move(res);
}

void updateChatThemes::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_themes_); if (arr_tmp_) { env->SetObjectField(s, chat_themes_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateChatThemes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatThemes");
    { s.store_vector_begin("chatThemes", chat_themes_.size()); for (const auto &_value : chat_themes_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateChatThemes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatThemes").c_str());
    chat_themes_fieldID = jni::get_field_id(env, Class, "chatThemes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ChatTheme;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateAccentColors::Class;
jfieldID updateAccentColors::colors_fieldID;
jfieldID updateAccentColors::available_accent_color_ids_fieldID;

updateAccentColors::updateAccentColors()
  : colors_()
  , available_accent_color_ids_()
{}

updateAccentColors::updateAccentColors(array<object_ptr<accentColor>> &&colors_, array<int32> &&available_accent_color_ids_)
  : colors_(std::move(colors_))
  , available_accent_color_ids_(std::move(available_accent_color_ids_))
{}

const std::int32_t updateAccentColors::ID;

object_ptr<Update> updateAccentColors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAccentColors> res = make_object<updateAccentColors>();
  res->colors_ = jni::FetchVector<accentColor>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->colors_fieldID));
  res->available_accent_color_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->available_accent_color_ids_fieldID));
  return std::move(res);
}

void updateAccentColors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, colors_); if (arr_tmp_) { env->SetObjectField(s, colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, available_accent_color_ids_); if (arr_tmp_) { env->SetObjectField(s, available_accent_color_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateAccentColors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAccentColors");
    { s.store_vector_begin("colors", colors_.size()); for (const auto &_value : colors_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("availableAccentColorIds", available_accent_color_ids_.size()); for (const auto &_value : available_accent_color_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateAccentColors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAccentColors").c_str());
    colors_fieldID = jni::get_field_id(env, Class, "colors", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AccentColor;").c_str());
    available_accent_color_ids_fieldID = jni::get_field_id(env, Class, "availableAccentColorIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateProfileAccentColors::Class;
jfieldID updateProfileAccentColors::colors_fieldID;
jfieldID updateProfileAccentColors::available_accent_color_ids_fieldID;

updateProfileAccentColors::updateProfileAccentColors()
  : colors_()
  , available_accent_color_ids_()
{}

updateProfileAccentColors::updateProfileAccentColors(array<object_ptr<profileAccentColor>> &&colors_, array<int32> &&available_accent_color_ids_)
  : colors_(std::move(colors_))
  , available_accent_color_ids_(std::move(available_accent_color_ids_))
{}

const std::int32_t updateProfileAccentColors::ID;

object_ptr<Update> updateProfileAccentColors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateProfileAccentColors> res = make_object<updateProfileAccentColors>();
  res->colors_ = jni::FetchVector<profileAccentColor>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->colors_fieldID));
  res->available_accent_color_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->available_accent_color_ids_fieldID));
  return std::move(res);
}

void updateProfileAccentColors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, colors_); if (arr_tmp_) { env->SetObjectField(s, colors_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, available_accent_color_ids_); if (arr_tmp_) { env->SetObjectField(s, available_accent_color_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateProfileAccentColors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateProfileAccentColors");
    { s.store_vector_begin("colors", colors_.size()); for (const auto &_value : colors_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("availableAccentColorIds", available_accent_color_ids_.size()); for (const auto &_value : available_accent_color_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateProfileAccentColors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateProfileAccentColors").c_str());
    colors_fieldID = jni::get_field_id(env, Class, "colors", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ProfileAccentColor;").c_str());
    available_accent_color_ids_fieldID = jni::get_field_id(env, Class, "availableAccentColorIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateLanguagePackStrings::Class;
jfieldID updateLanguagePackStrings::localization_target_fieldID;
jfieldID updateLanguagePackStrings::language_pack_id_fieldID;
jfieldID updateLanguagePackStrings::strings_fieldID;

updateLanguagePackStrings::updateLanguagePackStrings()
  : localization_target_()
  , language_pack_id_()
  , strings_()
{}

updateLanguagePackStrings::updateLanguagePackStrings(string const &localization_target_, string const &language_pack_id_, array<object_ptr<languagePackString>> &&strings_)
  : localization_target_(localization_target_)
  , language_pack_id_(language_pack_id_)
  , strings_(std::move(strings_))
{}

const std::int32_t updateLanguagePackStrings::ID;

object_ptr<Update> updateLanguagePackStrings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateLanguagePackStrings> res = make_object<updateLanguagePackStrings>();
  res->localization_target_ = jni::fetch_string(env, p, res->localization_target_fieldID);
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  res->strings_ = jni::FetchVector<languagePackString>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->strings_fieldID));
  return std::move(res);
}

void updateLanguagePackStrings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, localization_target_); if (nextString) { env->SetObjectField(s, localization_target_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, language_pack_id_); if (nextString) { env->SetObjectField(s, language_pack_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, strings_); if (arr_tmp_) { env->SetObjectField(s, strings_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateLanguagePackStrings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateLanguagePackStrings");
    s.store_field("localizationTarget", localization_target_);
    s.store_field("languagePackId", language_pack_id_);
    { s.store_vector_begin("strings", strings_.size()); for (const auto &_value : strings_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateLanguagePackStrings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateLanguagePackStrings").c_str());
    localization_target_fieldID = jni::get_field_id(env, Class, "localizationTarget", "Ljava/lang/String;");
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    strings_fieldID = jni::get_field_id(env, Class, "strings", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LanguagePackString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateConnectionState::Class;
jfieldID updateConnectionState::state_fieldID;

updateConnectionState::updateConnectionState()
  : state_()
{}

updateConnectionState::updateConnectionState(object_ptr<ConnectionState> &&state_)
  : state_(std::move(state_))
{}

const std::int32_t updateConnectionState::ID;

object_ptr<Update> updateConnectionState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateConnectionState> res = make_object<updateConnectionState>();
  res->state_ = jni::fetch_tl_object<ConnectionState>(env, jni::fetch_object(env, p, res->state_fieldID));
  return std::move(res);
}

void updateConnectionState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (state_ != nullptr) { jobject next; state_->store(env, next); if (next) { env->SetObjectField(s, state_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateConnectionState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateConnectionState");
    s.store_object_field("state", static_cast<const BaseObject *>(state_.get()));
    s.store_class_end();
  }
}

void updateConnectionState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateConnectionState").c_str());
    state_fieldID = jni::get_field_id(env, Class, "state", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ConnectionState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateFreezeState::Class;
jfieldID updateFreezeState::is_frozen_fieldID;
jfieldID updateFreezeState::freezing_date_fieldID;
jfieldID updateFreezeState::deletion_date_fieldID;
jfieldID updateFreezeState::appeal_link_fieldID;

updateFreezeState::updateFreezeState()
  : is_frozen_()
  , freezing_date_()
  , deletion_date_()
  , appeal_link_()
{}

updateFreezeState::updateFreezeState(bool is_frozen_, int32 freezing_date_, int32 deletion_date_, string const &appeal_link_)
  : is_frozen_(is_frozen_)
  , freezing_date_(freezing_date_)
  , deletion_date_(deletion_date_)
  , appeal_link_(appeal_link_)
{}

const std::int32_t updateFreezeState::ID;

object_ptr<Update> updateFreezeState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateFreezeState> res = make_object<updateFreezeState>();
  res->is_frozen_ = (env->GetBooleanField(p, res->is_frozen_fieldID) != 0);
  res->freezing_date_ = env->GetIntField(p, res->freezing_date_fieldID);
  res->deletion_date_ = env->GetIntField(p, res->deletion_date_fieldID);
  res->appeal_link_ = jni::fetch_string(env, p, res->appeal_link_fieldID);
  return std::move(res);
}

void updateFreezeState::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_frozen_fieldID, is_frozen_);
  env->SetIntField(s, freezing_date_fieldID, freezing_date_);
  env->SetIntField(s, deletion_date_fieldID, deletion_date_);
  { jstring nextString = jni::to_jstring(env, appeal_link_); if (nextString) { env->SetObjectField(s, appeal_link_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateFreezeState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateFreezeState");
    s.store_field("isFrozen", is_frozen_);
    s.store_field("freezingDate", freezing_date_);
    s.store_field("deletionDate", deletion_date_);
    s.store_field("appealLink", appeal_link_);
    s.store_class_end();
  }
}

void updateFreezeState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateFreezeState").c_str());
    is_frozen_fieldID = jni::get_field_id(env, Class, "isFrozen", "Z");
    freezing_date_fieldID = jni::get_field_id(env, Class, "freezingDate", "I");
    deletion_date_fieldID = jni::get_field_id(env, Class, "deletionDate", "I");
    appeal_link_fieldID = jni::get_field_id(env, Class, "appealLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateTermsOfService::Class;
jfieldID updateTermsOfService::terms_of_service_id_fieldID;
jfieldID updateTermsOfService::terms_of_service_fieldID;

updateTermsOfService::updateTermsOfService()
  : terms_of_service_id_()
  , terms_of_service_()
{}

updateTermsOfService::updateTermsOfService(string const &terms_of_service_id_, object_ptr<termsOfService> &&terms_of_service_)
  : terms_of_service_id_(terms_of_service_id_)
  , terms_of_service_(std::move(terms_of_service_))
{}

const std::int32_t updateTermsOfService::ID;

object_ptr<Update> updateTermsOfService::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateTermsOfService> res = make_object<updateTermsOfService>();
  res->terms_of_service_id_ = jni::fetch_string(env, p, res->terms_of_service_id_fieldID);
  res->terms_of_service_ = jni::fetch_tl_object<termsOfService>(env, jni::fetch_object(env, p, res->terms_of_service_fieldID));
  return std::move(res);
}

void updateTermsOfService::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, terms_of_service_id_); if (nextString) { env->SetObjectField(s, terms_of_service_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (terms_of_service_ != nullptr) { jobject next; terms_of_service_->store(env, next); if (next) { env->SetObjectField(s, terms_of_service_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateTermsOfService::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateTermsOfService");
    s.store_field("termsOfServiceId", terms_of_service_id_);
    s.store_object_field("termsOfService", static_cast<const BaseObject *>(terms_of_service_.get()));
    s.store_class_end();
  }
}

void updateTermsOfService::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateTermsOfService").c_str());
    terms_of_service_id_fieldID = jni::get_field_id(env, Class, "termsOfServiceId", "Ljava/lang/String;");
    terms_of_service_fieldID = jni::get_field_id(env, Class, "termsOfService", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TermsOfService;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateUnconfirmedSession::Class;
jfieldID updateUnconfirmedSession::session_fieldID;

updateUnconfirmedSession::updateUnconfirmedSession()
  : session_()
{}

updateUnconfirmedSession::updateUnconfirmedSession(object_ptr<unconfirmedSession> &&session_)
  : session_(std::move(session_))
{}

const std::int32_t updateUnconfirmedSession::ID;

object_ptr<Update> updateUnconfirmedSession::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateUnconfirmedSession> res = make_object<updateUnconfirmedSession>();
  res->session_ = jni::fetch_tl_object<unconfirmedSession>(env, jni::fetch_object(env, p, res->session_fieldID));
  return std::move(res);
}

void updateUnconfirmedSession::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (session_ != nullptr) { jobject next; session_->store(env, next); if (next) { env->SetObjectField(s, session_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateUnconfirmedSession::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateUnconfirmedSession");
    s.store_object_field("session", static_cast<const BaseObject *>(session_.get()));
    s.store_class_end();
  }
}

void updateUnconfirmedSession::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateUnconfirmedSession").c_str());
    session_fieldID = jni::get_field_id(env, Class, "session", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UnconfirmedSession;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateAttachmentMenuBots::Class;
jfieldID updateAttachmentMenuBots::bots_fieldID;

updateAttachmentMenuBots::updateAttachmentMenuBots()
  : bots_()
{}

updateAttachmentMenuBots::updateAttachmentMenuBots(array<object_ptr<attachmentMenuBot>> &&bots_)
  : bots_(std::move(bots_))
{}

const std::int32_t updateAttachmentMenuBots::ID;

object_ptr<Update> updateAttachmentMenuBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAttachmentMenuBots> res = make_object<updateAttachmentMenuBots>();
  res->bots_ = jni::FetchVector<attachmentMenuBot>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->bots_fieldID));
  return std::move(res);
}

void updateAttachmentMenuBots::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, bots_); if (arr_tmp_) { env->SetObjectField(s, bots_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateAttachmentMenuBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAttachmentMenuBots");
    { s.store_vector_begin("bots", bots_.size()); for (const auto &_value : bots_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateAttachmentMenuBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAttachmentMenuBots").c_str());
    bots_fieldID = jni::get_field_id(env, Class, "bots", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$AttachmentMenuBot;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateWebAppMessageSent::Class;
jfieldID updateWebAppMessageSent::web_app_launch_id_fieldID;

updateWebAppMessageSent::updateWebAppMessageSent()
  : web_app_launch_id_()
{}

updateWebAppMessageSent::updateWebAppMessageSent(int64 web_app_launch_id_)
  : web_app_launch_id_(web_app_launch_id_)
{}

const std::int32_t updateWebAppMessageSent::ID;

object_ptr<Update> updateWebAppMessageSent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateWebAppMessageSent> res = make_object<updateWebAppMessageSent>();
  res->web_app_launch_id_ = env->GetLongField(p, res->web_app_launch_id_fieldID);
  return std::move(res);
}

void updateWebAppMessageSent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, web_app_launch_id_fieldID, web_app_launch_id_);
}

void updateWebAppMessageSent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateWebAppMessageSent");
    s.store_field("webAppLaunchId", web_app_launch_id_);
    s.store_class_end();
  }
}

void updateWebAppMessageSent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateWebAppMessageSent").c_str());
    web_app_launch_id_fieldID = jni::get_field_id(env, Class, "webAppLaunchId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass updateActiveEmojiReactions::Class;
jfieldID updateActiveEmojiReactions::emojis_fieldID;

updateActiveEmojiReactions::updateActiveEmojiReactions()
  : emojis_()
{}

updateActiveEmojiReactions::updateActiveEmojiReactions(array<string> &&emojis_)
  : emojis_(std::move(emojis_))
{}

const std::int32_t updateActiveEmojiReactions::ID;

object_ptr<Update> updateActiveEmojiReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateActiveEmojiReactions> res = make_object<updateActiveEmojiReactions>();
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return std::move(res);
}

void updateActiveEmojiReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateActiveEmojiReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateActiveEmojiReactions");
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateActiveEmojiReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateActiveEmojiReactions").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateAvailableMessageEffects::Class;
jfieldID updateAvailableMessageEffects::reaction_effect_ids_fieldID;
jfieldID updateAvailableMessageEffects::sticker_effect_ids_fieldID;

updateAvailableMessageEffects::updateAvailableMessageEffects()
  : reaction_effect_ids_()
  , sticker_effect_ids_()
{}

updateAvailableMessageEffects::updateAvailableMessageEffects(array<int64> &&reaction_effect_ids_, array<int64> &&sticker_effect_ids_)
  : reaction_effect_ids_(std::move(reaction_effect_ids_))
  , sticker_effect_ids_(std::move(sticker_effect_ids_))
{}

const std::int32_t updateAvailableMessageEffects::ID;

object_ptr<Update> updateAvailableMessageEffects::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAvailableMessageEffects> res = make_object<updateAvailableMessageEffects>();
  res->reaction_effect_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->reaction_effect_ids_fieldID));
  res->sticker_effect_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->sticker_effect_ids_fieldID));
  return std::move(res);
}

void updateAvailableMessageEffects::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, reaction_effect_ids_); if (arr_tmp_) { env->SetObjectField(s, reaction_effect_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, sticker_effect_ids_); if (arr_tmp_) { env->SetObjectField(s, sticker_effect_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateAvailableMessageEffects::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAvailableMessageEffects");
    { s.store_vector_begin("reactionEffectIds", reaction_effect_ids_.size()); for (const auto &_value : reaction_effect_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("stickerEffectIds", sticker_effect_ids_.size()); for (const auto &_value : sticker_effect_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateAvailableMessageEffects::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAvailableMessageEffects").c_str());
    reaction_effect_ids_fieldID = jni::get_field_id(env, Class, "reactionEffectIds", "[J");
    sticker_effect_ids_fieldID = jni::get_field_id(env, Class, "stickerEffectIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass updateDefaultReactionType::Class;
jfieldID updateDefaultReactionType::reaction_type_fieldID;

updateDefaultReactionType::updateDefaultReactionType()
  : reaction_type_()
{}

updateDefaultReactionType::updateDefaultReactionType(object_ptr<ReactionType> &&reaction_type_)
  : reaction_type_(std::move(reaction_type_))
{}

const std::int32_t updateDefaultReactionType::ID;

object_ptr<Update> updateDefaultReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateDefaultReactionType> res = make_object<updateDefaultReactionType>();
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  return std::move(res);
}

void updateDefaultReactionType::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (reaction_type_ != nullptr) { jobject next; reaction_type_->store(env, next); if (next) { env->SetObjectField(s, reaction_type_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateDefaultReactionType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateDefaultReactionType");
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_class_end();
  }
}

void updateDefaultReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateDefaultReactionType").c_str());
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateDefaultPaidReactionType::Class;
jfieldID updateDefaultPaidReactionType::type_fieldID;

updateDefaultPaidReactionType::updateDefaultPaidReactionType()
  : type_()
{}

updateDefaultPaidReactionType::updateDefaultPaidReactionType(object_ptr<PaidReactionType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t updateDefaultPaidReactionType::ID;

object_ptr<Update> updateDefaultPaidReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateDefaultPaidReactionType> res = make_object<updateDefaultPaidReactionType>();
  res->type_ = jni::fetch_tl_object<PaidReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return std::move(res);
}

void updateDefaultPaidReactionType::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateDefaultPaidReactionType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateDefaultPaidReactionType");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

void updateDefaultPaidReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateDefaultPaidReactionType").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaidReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSavedMessagesTags::Class;
jfieldID updateSavedMessagesTags::saved_messages_topic_id_fieldID;
jfieldID updateSavedMessagesTags::tags_fieldID;

updateSavedMessagesTags::updateSavedMessagesTags()
  : saved_messages_topic_id_()
  , tags_()
{}

updateSavedMessagesTags::updateSavedMessagesTags(int53 saved_messages_topic_id_, object_ptr<savedMessagesTags> &&tags_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , tags_(std::move(tags_))
{}

const std::int32_t updateSavedMessagesTags::ID;

object_ptr<Update> updateSavedMessagesTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSavedMessagesTags> res = make_object<updateSavedMessagesTags>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->tags_ = jni::fetch_tl_object<savedMessagesTags>(env, jni::fetch_object(env, p, res->tags_fieldID));
  return std::move(res);
}

void updateSavedMessagesTags::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, saved_messages_topic_id_fieldID, saved_messages_topic_id_);
  if (tags_ != nullptr) { jobject next; tags_->store(env, next); if (next) { env->SetObjectField(s, tags_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateSavedMessagesTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSavedMessagesTags");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_object_field("tags", static_cast<const BaseObject *>(tags_.get()));
    s.store_class_end();
  }
}

void updateSavedMessagesTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSavedMessagesTags").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    tags_fieldID = jni::get_field_id(env, Class, "tags", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SavedMessagesTags;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateActiveLiveLocationMessages::Class;
jfieldID updateActiveLiveLocationMessages::messages_fieldID;

updateActiveLiveLocationMessages::updateActiveLiveLocationMessages()
  : messages_()
{}

updateActiveLiveLocationMessages::updateActiveLiveLocationMessages(array<object_ptr<message>> &&messages_)
  : messages_(std::move(messages_))
{}

const std::int32_t updateActiveLiveLocationMessages::ID;

object_ptr<Update> updateActiveLiveLocationMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateActiveLiveLocationMessages> res = make_object<updateActiveLiveLocationMessages>();
  res->messages_ = jni::FetchVector<message>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->messages_fieldID));
  return std::move(res);
}

void updateActiveLiveLocationMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, messages_); if (arr_tmp_) { env->SetObjectField(s, messages_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateActiveLiveLocationMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateActiveLiveLocationMessages");
    { s.store_vector_begin("messages", messages_.size()); for (const auto &_value : messages_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateActiveLiveLocationMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateActiveLiveLocationMessages").c_str());
    messages_fieldID = jni::get_field_id(env, Class, "messages", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Message;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateOwnedStarCount::Class;
jfieldID updateOwnedStarCount::star_amount_fieldID;

updateOwnedStarCount::updateOwnedStarCount()
  : star_amount_()
{}

updateOwnedStarCount::updateOwnedStarCount(object_ptr<starAmount> &&star_amount_)
  : star_amount_(std::move(star_amount_))
{}

const std::int32_t updateOwnedStarCount::ID;

object_ptr<Update> updateOwnedStarCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateOwnedStarCount> res = make_object<updateOwnedStarCount>();
  res->star_amount_ = jni::fetch_tl_object<starAmount>(env, jni::fetch_object(env, p, res->star_amount_fieldID));
  return std::move(res);
}

void updateOwnedStarCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (star_amount_ != nullptr) { jobject next; star_amount_->store(env, next); if (next) { env->SetObjectField(s, star_amount_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateOwnedStarCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateOwnedStarCount");
    s.store_object_field("starAmount", static_cast<const BaseObject *>(star_amount_.get()));
    s.store_class_end();
  }
}

void updateOwnedStarCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateOwnedStarCount").c_str());
    star_amount_fieldID = jni::get_field_id(env, Class, "starAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarAmount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatRevenueAmount::Class;
jfieldID updateChatRevenueAmount::chat_id_fieldID;
jfieldID updateChatRevenueAmount::revenue_amount_fieldID;

updateChatRevenueAmount::updateChatRevenueAmount()
  : chat_id_()
  , revenue_amount_()
{}

updateChatRevenueAmount::updateChatRevenueAmount(int53 chat_id_, object_ptr<chatRevenueAmount> &&revenue_amount_)
  : chat_id_(chat_id_)
  , revenue_amount_(std::move(revenue_amount_))
{}

const std::int32_t updateChatRevenueAmount::ID;

object_ptr<Update> updateChatRevenueAmount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatRevenueAmount> res = make_object<updateChatRevenueAmount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->revenue_amount_ = jni::fetch_tl_object<chatRevenueAmount>(env, jni::fetch_object(env, p, res->revenue_amount_fieldID));
  return std::move(res);
}

void updateChatRevenueAmount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (revenue_amount_ != nullptr) { jobject next; revenue_amount_->store(env, next); if (next) { env->SetObjectField(s, revenue_amount_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatRevenueAmount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatRevenueAmount");
    s.store_field("chatId", chat_id_);
    s.store_object_field("revenueAmount", static_cast<const BaseObject *>(revenue_amount_.get()));
    s.store_class_end();
  }
}

void updateChatRevenueAmount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatRevenueAmount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    revenue_amount_fieldID = jni::get_field_id(env, Class, "revenueAmount", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatRevenueAmount;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateStarRevenueStatus::Class;
jfieldID updateStarRevenueStatus::owner_id_fieldID;
jfieldID updateStarRevenueStatus::status_fieldID;

updateStarRevenueStatus::updateStarRevenueStatus()
  : owner_id_()
  , status_()
{}

updateStarRevenueStatus::updateStarRevenueStatus(object_ptr<MessageSender> &&owner_id_, object_ptr<starRevenueStatus> &&status_)
  : owner_id_(std::move(owner_id_))
  , status_(std::move(status_))
{}

const std::int32_t updateStarRevenueStatus::ID;

object_ptr<Update> updateStarRevenueStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateStarRevenueStatus> res = make_object<updateStarRevenueStatus>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->status_ = jni::fetch_tl_object<starRevenueStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  return std::move(res);
}

void updateStarRevenueStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (owner_id_ != nullptr) { jobject next; owner_id_->store(env, next); if (next) { env->SetObjectField(s, owner_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateStarRevenueStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateStarRevenueStatus");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_class_end();
  }
}

void updateStarRevenueStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateStarRevenueStatus").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarRevenueStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSpeechRecognitionTrial::Class;
jfieldID updateSpeechRecognitionTrial::max_media_duration_fieldID;
jfieldID updateSpeechRecognitionTrial::weekly_count_fieldID;
jfieldID updateSpeechRecognitionTrial::left_count_fieldID;
jfieldID updateSpeechRecognitionTrial::next_reset_date_fieldID;

updateSpeechRecognitionTrial::updateSpeechRecognitionTrial()
  : max_media_duration_()
  , weekly_count_()
  , left_count_()
  , next_reset_date_()
{}

updateSpeechRecognitionTrial::updateSpeechRecognitionTrial(int32 max_media_duration_, int32 weekly_count_, int32 left_count_, int32 next_reset_date_)
  : max_media_duration_(max_media_duration_)
  , weekly_count_(weekly_count_)
  , left_count_(left_count_)
  , next_reset_date_(next_reset_date_)
{}

const std::int32_t updateSpeechRecognitionTrial::ID;

object_ptr<Update> updateSpeechRecognitionTrial::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSpeechRecognitionTrial> res = make_object<updateSpeechRecognitionTrial>();
  res->max_media_duration_ = env->GetIntField(p, res->max_media_duration_fieldID);
  res->weekly_count_ = env->GetIntField(p, res->weekly_count_fieldID);
  res->left_count_ = env->GetIntField(p, res->left_count_fieldID);
  res->next_reset_date_ = env->GetIntField(p, res->next_reset_date_fieldID);
  return std::move(res);
}

void updateSpeechRecognitionTrial::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, max_media_duration_fieldID, max_media_duration_);
  env->SetIntField(s, weekly_count_fieldID, weekly_count_);
  env->SetIntField(s, left_count_fieldID, left_count_);
  env->SetIntField(s, next_reset_date_fieldID, next_reset_date_);
}

void updateSpeechRecognitionTrial::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSpeechRecognitionTrial");
    s.store_field("maxMediaDuration", max_media_duration_);
    s.store_field("weeklyCount", weekly_count_);
    s.store_field("leftCount", left_count_);
    s.store_field("nextResetDate", next_reset_date_);
    s.store_class_end();
  }
}

void updateSpeechRecognitionTrial::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSpeechRecognitionTrial").c_str());
    max_media_duration_fieldID = jni::get_field_id(env, Class, "maxMediaDuration", "I");
    weekly_count_fieldID = jni::get_field_id(env, Class, "weeklyCount", "I");
    left_count_fieldID = jni::get_field_id(env, Class, "leftCount", "I");
    next_reset_date_fieldID = jni::get_field_id(env, Class, "nextResetDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updateDiceEmojis::Class;
jfieldID updateDiceEmojis::emojis_fieldID;

updateDiceEmojis::updateDiceEmojis()
  : emojis_()
{}

updateDiceEmojis::updateDiceEmojis(array<string> &&emojis_)
  : emojis_(std::move(emojis_))
{}

const std::int32_t updateDiceEmojis::ID;

object_ptr<Update> updateDiceEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateDiceEmojis> res = make_object<updateDiceEmojis>();
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return std::move(res);
}

void updateDiceEmojis::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateDiceEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateDiceEmojis");
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateDiceEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateDiceEmojis").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateAnimatedEmojiMessageClicked::Class;
jfieldID updateAnimatedEmojiMessageClicked::chat_id_fieldID;
jfieldID updateAnimatedEmojiMessageClicked::message_id_fieldID;
jfieldID updateAnimatedEmojiMessageClicked::sticker_fieldID;

updateAnimatedEmojiMessageClicked::updateAnimatedEmojiMessageClicked()
  : chat_id_()
  , message_id_()
  , sticker_()
{}

updateAnimatedEmojiMessageClicked::updateAnimatedEmojiMessageClicked(int53 chat_id_, int53 message_id_, object_ptr<sticker> &&sticker_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t updateAnimatedEmojiMessageClicked::ID;

object_ptr<Update> updateAnimatedEmojiMessageClicked::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAnimatedEmojiMessageClicked> res = make_object<updateAnimatedEmojiMessageClicked>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return std::move(res);
}

void updateAnimatedEmojiMessageClicked::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateAnimatedEmojiMessageClicked::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAnimatedEmojiMessageClicked");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

void updateAnimatedEmojiMessageClicked::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAnimatedEmojiMessageClicked").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateAnimationSearchParameters::Class;
jfieldID updateAnimationSearchParameters::provider_fieldID;
jfieldID updateAnimationSearchParameters::emojis_fieldID;

updateAnimationSearchParameters::updateAnimationSearchParameters()
  : provider_()
  , emojis_()
{}

updateAnimationSearchParameters::updateAnimationSearchParameters(string const &provider_, array<string> &&emojis_)
  : provider_(provider_)
  , emojis_(std::move(emojis_))
{}

const std::int32_t updateAnimationSearchParameters::ID;

object_ptr<Update> updateAnimationSearchParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAnimationSearchParameters> res = make_object<updateAnimationSearchParameters>();
  res->provider_ = jni::fetch_string(env, p, res->provider_fieldID);
  res->emojis_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->emojis_fieldID));
  return std::move(res);
}

void updateAnimationSearchParameters::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, provider_); if (nextString) { env->SetObjectField(s, provider_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, emojis_); if (arr_tmp_) { env->SetObjectField(s, emojis_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateAnimationSearchParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAnimationSearchParameters");
    s.store_field("provider", provider_);
    { s.store_vector_begin("emojis", emojis_.size()); for (const auto &_value : emojis_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateAnimationSearchParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAnimationSearchParameters").c_str());
    provider_fieldID = jni::get_field_id(env, Class, "provider", "Ljava/lang/String;");
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateSuggestedActions::Class;
jfieldID updateSuggestedActions::added_actions_fieldID;
jfieldID updateSuggestedActions::removed_actions_fieldID;

updateSuggestedActions::updateSuggestedActions()
  : added_actions_()
  , removed_actions_()
{}

updateSuggestedActions::updateSuggestedActions(array<object_ptr<SuggestedAction>> &&added_actions_, array<object_ptr<SuggestedAction>> &&removed_actions_)
  : added_actions_(std::move(added_actions_))
  , removed_actions_(std::move(removed_actions_))
{}

const std::int32_t updateSuggestedActions::ID;

object_ptr<Update> updateSuggestedActions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSuggestedActions> res = make_object<updateSuggestedActions>();
  res->added_actions_ = jni::FetchVector<SuggestedAction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->added_actions_fieldID));
  res->removed_actions_ = jni::FetchVector<SuggestedAction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->removed_actions_fieldID));
  return std::move(res);
}

void updateSuggestedActions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, added_actions_); if (arr_tmp_) { env->SetObjectField(s, added_actions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, removed_actions_); if (arr_tmp_) { env->SetObjectField(s, removed_actions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateSuggestedActions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSuggestedActions");
    { s.store_vector_begin("addedActions", added_actions_.size()); for (const auto &_value : added_actions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("removedActions", removed_actions_.size()); for (const auto &_value : removed_actions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateSuggestedActions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSuggestedActions").c_str());
    added_actions_fieldID = jni::get_field_id(env, Class, "addedActions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SuggestedAction;").c_str());
    removed_actions_fieldID = jni::get_field_id(env, Class, "removedActions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$SuggestedAction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateSpeedLimitNotification::Class;
jfieldID updateSpeedLimitNotification::is_upload_fieldID;

updateSpeedLimitNotification::updateSpeedLimitNotification()
  : is_upload_()
{}

updateSpeedLimitNotification::updateSpeedLimitNotification(bool is_upload_)
  : is_upload_(is_upload_)
{}

const std::int32_t updateSpeedLimitNotification::ID;

object_ptr<Update> updateSpeedLimitNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateSpeedLimitNotification> res = make_object<updateSpeedLimitNotification>();
  res->is_upload_ = (env->GetBooleanField(p, res->is_upload_fieldID) != 0);
  return std::move(res);
}

void updateSpeedLimitNotification::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_upload_fieldID, is_upload_);
}

void updateSpeedLimitNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateSpeedLimitNotification");
    s.store_field("isUpload", is_upload_);
    s.store_class_end();
  }
}

void updateSpeedLimitNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateSpeedLimitNotification").c_str());
    is_upload_fieldID = jni::get_field_id(env, Class, "isUpload", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass updateContactCloseBirthdays::Class;
jfieldID updateContactCloseBirthdays::close_birthday_users_fieldID;

updateContactCloseBirthdays::updateContactCloseBirthdays()
  : close_birthday_users_()
{}

updateContactCloseBirthdays::updateContactCloseBirthdays(array<object_ptr<closeBirthdayUser>> &&close_birthday_users_)
  : close_birthday_users_(std::move(close_birthday_users_))
{}

const std::int32_t updateContactCloseBirthdays::ID;

object_ptr<Update> updateContactCloseBirthdays::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateContactCloseBirthdays> res = make_object<updateContactCloseBirthdays>();
  res->close_birthday_users_ = jni::FetchVector<closeBirthdayUser>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->close_birthday_users_fieldID));
  return std::move(res);
}

void updateContactCloseBirthdays::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, close_birthday_users_); if (arr_tmp_) { env->SetObjectField(s, close_birthday_users_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateContactCloseBirthdays::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateContactCloseBirthdays");
    { s.store_vector_begin("closeBirthdayUsers", close_birthday_users_.size()); for (const auto &_value : close_birthday_users_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateContactCloseBirthdays::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateContactCloseBirthdays").c_str());
    close_birthday_users_fieldID = jni::get_field_id(env, Class, "closeBirthdayUsers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$CloseBirthdayUser;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateAutosaveSettings::Class;
jfieldID updateAutosaveSettings::scope_fieldID;
jfieldID updateAutosaveSettings::settings_fieldID;

updateAutosaveSettings::updateAutosaveSettings()
  : scope_()
  , settings_()
{}

updateAutosaveSettings::updateAutosaveSettings(object_ptr<AutosaveSettingsScope> &&scope_, object_ptr<scopeAutosaveSettings> &&settings_)
  : scope_(std::move(scope_))
  , settings_(std::move(settings_))
{}

const std::int32_t updateAutosaveSettings::ID;

object_ptr<Update> updateAutosaveSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateAutosaveSettings> res = make_object<updateAutosaveSettings>();
  res->scope_ = jni::fetch_tl_object<AutosaveSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return std::move(res);
}

void updateAutosaveSettings::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (scope_ != nullptr) { jobject next; scope_->store(env, next); if (next) { env->SetObjectField(s, scope_fieldID, next); env->DeleteLocalRef(next); } }
  if (settings_ != nullptr) { jobject next; settings_->store(env, next); if (next) { env->SetObjectField(s, settings_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateAutosaveSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateAutosaveSettings");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

void updateAutosaveSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateAutosaveSettings").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutosaveSettingsScope;").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateBusinessConnection::Class;
jfieldID updateBusinessConnection::connection_fieldID;

updateBusinessConnection::updateBusinessConnection()
  : connection_()
{}

updateBusinessConnection::updateBusinessConnection(object_ptr<businessConnection> &&connection_)
  : connection_(std::move(connection_))
{}

const std::int32_t updateBusinessConnection::ID;

object_ptr<Update> updateBusinessConnection::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateBusinessConnection> res = make_object<updateBusinessConnection>();
  res->connection_ = jni::fetch_tl_object<businessConnection>(env, jni::fetch_object(env, p, res->connection_fieldID));
  return std::move(res);
}

void updateBusinessConnection::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (connection_ != nullptr) { jobject next; connection_->store(env, next); if (next) { env->SetObjectField(s, connection_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateBusinessConnection::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateBusinessConnection");
    s.store_object_field("connection", static_cast<const BaseObject *>(connection_.get()));
    s.store_class_end();
  }
}

void updateBusinessConnection::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateBusinessConnection").c_str());
    connection_fieldID = jni::get_field_id(env, Class, "connection", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessConnection;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewBusinessMessage::Class;
jfieldID updateNewBusinessMessage::connection_id_fieldID;
jfieldID updateNewBusinessMessage::message_fieldID;

updateNewBusinessMessage::updateNewBusinessMessage()
  : connection_id_()
  , message_()
{}

updateNewBusinessMessage::updateNewBusinessMessage(string const &connection_id_, object_ptr<businessMessage> &&message_)
  : connection_id_(connection_id_)
  , message_(std::move(message_))
{}

const std::int32_t updateNewBusinessMessage::ID;

object_ptr<Update> updateNewBusinessMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewBusinessMessage> res = make_object<updateNewBusinessMessage>();
  res->connection_id_ = jni::fetch_string(env, p, res->connection_id_fieldID);
  res->message_ = jni::fetch_tl_object<businessMessage>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void updateNewBusinessMessage::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, connection_id_); if (nextString) { env->SetObjectField(s, connection_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewBusinessMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewBusinessMessage");
    s.store_field("connectionId", connection_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void updateNewBusinessMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewBusinessMessage").c_str());
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "Ljava/lang/String;");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateBusinessMessageEdited::Class;
jfieldID updateBusinessMessageEdited::connection_id_fieldID;
jfieldID updateBusinessMessageEdited::message_fieldID;

updateBusinessMessageEdited::updateBusinessMessageEdited()
  : connection_id_()
  , message_()
{}

updateBusinessMessageEdited::updateBusinessMessageEdited(string const &connection_id_, object_ptr<businessMessage> &&message_)
  : connection_id_(connection_id_)
  , message_(std::move(message_))
{}

const std::int32_t updateBusinessMessageEdited::ID;

object_ptr<Update> updateBusinessMessageEdited::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateBusinessMessageEdited> res = make_object<updateBusinessMessageEdited>();
  res->connection_id_ = jni::fetch_string(env, p, res->connection_id_fieldID);
  res->message_ = jni::fetch_tl_object<businessMessage>(env, jni::fetch_object(env, p, res->message_fieldID));
  return std::move(res);
}

void updateBusinessMessageEdited::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, connection_id_); if (nextString) { env->SetObjectField(s, connection_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateBusinessMessageEdited::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateBusinessMessageEdited");
    s.store_field("connectionId", connection_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

void updateBusinessMessageEdited::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateBusinessMessageEdited").c_str());
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "Ljava/lang/String;");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateBusinessMessagesDeleted::Class;
jfieldID updateBusinessMessagesDeleted::connection_id_fieldID;
jfieldID updateBusinessMessagesDeleted::chat_id_fieldID;
jfieldID updateBusinessMessagesDeleted::message_ids_fieldID;

updateBusinessMessagesDeleted::updateBusinessMessagesDeleted()
  : connection_id_()
  , chat_id_()
  , message_ids_()
{}

updateBusinessMessagesDeleted::updateBusinessMessagesDeleted(string const &connection_id_, int53 chat_id_, array<int53> &&message_ids_)
  : connection_id_(connection_id_)
  , chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t updateBusinessMessagesDeleted::ID;

object_ptr<Update> updateBusinessMessagesDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateBusinessMessagesDeleted> res = make_object<updateBusinessMessagesDeleted>();
  res->connection_id_ = jni::fetch_string(env, p, res->connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return std::move(res);
}

void updateBusinessMessagesDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, connection_id_); if (nextString) { env->SetObjectField(s, connection_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  { auto arr_tmp_ = jni::store_vector(env, message_ids_); if (arr_tmp_) { env->SetObjectField(s, message_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateBusinessMessagesDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateBusinessMessagesDeleted");
    s.store_field("connectionId", connection_id_);
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateBusinessMessagesDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateBusinessMessagesDeleted").c_str());
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewInlineQuery::Class;
jfieldID updateNewInlineQuery::id_fieldID;
jfieldID updateNewInlineQuery::sender_user_id_fieldID;
jfieldID updateNewInlineQuery::user_location_fieldID;
jfieldID updateNewInlineQuery::chat_type_fieldID;
jfieldID updateNewInlineQuery::query_fieldID;
jfieldID updateNewInlineQuery::offset_fieldID;

updateNewInlineQuery::updateNewInlineQuery()
  : id_()
  , sender_user_id_()
  , user_location_()
  , chat_type_()
  , query_()
  , offset_()
{}

updateNewInlineQuery::updateNewInlineQuery(int64 id_, int53 sender_user_id_, object_ptr<location> &&user_location_, object_ptr<ChatType> &&chat_type_, string const &query_, string const &offset_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , user_location_(std::move(user_location_))
  , chat_type_(std::move(chat_type_))
  , query_(query_)
  , offset_(offset_)
{}

const std::int32_t updateNewInlineQuery::ID;

object_ptr<Update> updateNewInlineQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewInlineQuery> res = make_object<updateNewInlineQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->user_location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->user_location_fieldID));
  res->chat_type_ = jni::fetch_tl_object<ChatType>(env, jni::fetch_object(env, p, res->chat_type_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  return std::move(res);
}

void updateNewInlineQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  if (user_location_ != nullptr) { jobject next; user_location_->store(env, next); if (next) { env->SetObjectField(s, user_location_fieldID, next); env->DeleteLocalRef(next); } }
  if (chat_type_ != nullptr) { jobject next; chat_type_->store(env, next); if (next) { env->SetObjectField(s, chat_type_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, query_); if (nextString) { env->SetObjectField(s, query_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, offset_); if (nextString) { env->SetObjectField(s, offset_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateNewInlineQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewInlineQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_object_field("userLocation", static_cast<const BaseObject *>(user_location_.get()));
    s.store_object_field("chatType", static_cast<const BaseObject *>(chat_type_.get()));
    s.store_field("query", query_);
    s.store_field("offset", offset_);
    s.store_class_end();
  }
}

void updateNewInlineQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewInlineQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    user_location_fieldID = jni::get_field_id(env, Class, "userLocation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    chat_type_fieldID = jni::get_field_id(env, Class, "chatType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewChosenInlineResult::Class;
jfieldID updateNewChosenInlineResult::sender_user_id_fieldID;
jfieldID updateNewChosenInlineResult::user_location_fieldID;
jfieldID updateNewChosenInlineResult::query_fieldID;
jfieldID updateNewChosenInlineResult::result_id_fieldID;
jfieldID updateNewChosenInlineResult::inline_message_id_fieldID;

updateNewChosenInlineResult::updateNewChosenInlineResult()
  : sender_user_id_()
  , user_location_()
  , query_()
  , result_id_()
  , inline_message_id_()
{}

updateNewChosenInlineResult::updateNewChosenInlineResult(int53 sender_user_id_, object_ptr<location> &&user_location_, string const &query_, string const &result_id_, string const &inline_message_id_)
  : sender_user_id_(sender_user_id_)
  , user_location_(std::move(user_location_))
  , query_(query_)
  , result_id_(result_id_)
  , inline_message_id_(inline_message_id_)
{}

const std::int32_t updateNewChosenInlineResult::ID;

object_ptr<Update> updateNewChosenInlineResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewChosenInlineResult> res = make_object<updateNewChosenInlineResult>();
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->user_location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->user_location_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->result_id_ = jni::fetch_string(env, p, res->result_id_fieldID);
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  return std::move(res);
}

void updateNewChosenInlineResult::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  if (user_location_ != nullptr) { jobject next; user_location_->store(env, next); if (next) { env->SetObjectField(s, user_location_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, query_); if (nextString) { env->SetObjectField(s, query_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, result_id_); if (nextString) { env->SetObjectField(s, result_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, inline_message_id_); if (nextString) { env->SetObjectField(s, inline_message_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateNewChosenInlineResult::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewChosenInlineResult");
    s.store_field("senderUserId", sender_user_id_);
    s.store_object_field("userLocation", static_cast<const BaseObject *>(user_location_.get()));
    s.store_field("query", query_);
    s.store_field("resultId", result_id_);
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_class_end();
  }
}

void updateNewChosenInlineResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewChosenInlineResult").c_str());
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    user_location_fieldID = jni::get_field_id(env, Class, "userLocation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    result_id_fieldID = jni::get_field_id(env, Class, "resultId", "Ljava/lang/String;");
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewCallbackQuery::Class;
jfieldID updateNewCallbackQuery::id_fieldID;
jfieldID updateNewCallbackQuery::sender_user_id_fieldID;
jfieldID updateNewCallbackQuery::chat_id_fieldID;
jfieldID updateNewCallbackQuery::message_id_fieldID;
jfieldID updateNewCallbackQuery::chat_instance_fieldID;
jfieldID updateNewCallbackQuery::payload_fieldID;

updateNewCallbackQuery::updateNewCallbackQuery()
  : id_()
  , sender_user_id_()
  , chat_id_()
  , message_id_()
  , chat_instance_()
  , payload_()
{}

updateNewCallbackQuery::updateNewCallbackQuery(int64 id_, int53 sender_user_id_, int53 chat_id_, int53 message_id_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , chat_instance_(chat_instance_)
  , payload_(std::move(payload_))
{}

const std::int32_t updateNewCallbackQuery::ID;

object_ptr<Update> updateNewCallbackQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewCallbackQuery> res = make_object<updateNewCallbackQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->chat_instance_ = env->GetLongField(p, res->chat_instance_fieldID);
  res->payload_ = jni::fetch_tl_object<CallbackQueryPayload>(env, jni::fetch_object(env, p, res->payload_fieldID));
  return std::move(res);
}

void updateNewCallbackQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetLongField(s, chat_instance_fieldID, chat_instance_);
  if (payload_ != nullptr) { jobject next; payload_->store(env, next); if (next) { env->SetObjectField(s, payload_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewCallbackQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewCallbackQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("chatInstance", chat_instance_);
    s.store_object_field("payload", static_cast<const BaseObject *>(payload_.get()));
    s.store_class_end();
  }
}

void updateNewCallbackQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewCallbackQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    chat_instance_fieldID = jni::get_field_id(env, Class, "chatInstance", "J");
    payload_fieldID = jni::get_field_id(env, Class, "payload", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallbackQueryPayload;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewInlineCallbackQuery::Class;
jfieldID updateNewInlineCallbackQuery::id_fieldID;
jfieldID updateNewInlineCallbackQuery::sender_user_id_fieldID;
jfieldID updateNewInlineCallbackQuery::inline_message_id_fieldID;
jfieldID updateNewInlineCallbackQuery::chat_instance_fieldID;
jfieldID updateNewInlineCallbackQuery::payload_fieldID;

updateNewInlineCallbackQuery::updateNewInlineCallbackQuery()
  : id_()
  , sender_user_id_()
  , inline_message_id_()
  , chat_instance_()
  , payload_()
{}

updateNewInlineCallbackQuery::updateNewInlineCallbackQuery(int64 id_, int53 sender_user_id_, string const &inline_message_id_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , inline_message_id_(inline_message_id_)
  , chat_instance_(chat_instance_)
  , payload_(std::move(payload_))
{}

const std::int32_t updateNewInlineCallbackQuery::ID;

object_ptr<Update> updateNewInlineCallbackQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewInlineCallbackQuery> res = make_object<updateNewInlineCallbackQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->chat_instance_ = env->GetLongField(p, res->chat_instance_fieldID);
  res->payload_ = jni::fetch_tl_object<CallbackQueryPayload>(env, jni::fetch_object(env, p, res->payload_fieldID));
  return std::move(res);
}

void updateNewInlineCallbackQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  { jstring nextString = jni::to_jstring(env, inline_message_id_); if (nextString) { env->SetObjectField(s, inline_message_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, chat_instance_fieldID, chat_instance_);
  if (payload_ != nullptr) { jobject next; payload_->store(env, next); if (next) { env->SetObjectField(s, payload_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewInlineCallbackQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewInlineCallbackQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_field("chatInstance", chat_instance_);
    s.store_object_field("payload", static_cast<const BaseObject *>(payload_.get()));
    s.store_class_end();
  }
}

void updateNewInlineCallbackQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewInlineCallbackQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    chat_instance_fieldID = jni::get_field_id(env, Class, "chatInstance", "J");
    payload_fieldID = jni::get_field_id(env, Class, "payload", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallbackQueryPayload;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewBusinessCallbackQuery::Class;
jfieldID updateNewBusinessCallbackQuery::id_fieldID;
jfieldID updateNewBusinessCallbackQuery::sender_user_id_fieldID;
jfieldID updateNewBusinessCallbackQuery::connection_id_fieldID;
jfieldID updateNewBusinessCallbackQuery::message_fieldID;
jfieldID updateNewBusinessCallbackQuery::chat_instance_fieldID;
jfieldID updateNewBusinessCallbackQuery::payload_fieldID;

updateNewBusinessCallbackQuery::updateNewBusinessCallbackQuery()
  : id_()
  , sender_user_id_()
  , connection_id_()
  , message_()
  , chat_instance_()
  , payload_()
{}

updateNewBusinessCallbackQuery::updateNewBusinessCallbackQuery(int64 id_, int53 sender_user_id_, string const &connection_id_, object_ptr<businessMessage> &&message_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , connection_id_(connection_id_)
  , message_(std::move(message_))
  , chat_instance_(chat_instance_)
  , payload_(std::move(payload_))
{}

const std::int32_t updateNewBusinessCallbackQuery::ID;

object_ptr<Update> updateNewBusinessCallbackQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewBusinessCallbackQuery> res = make_object<updateNewBusinessCallbackQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->connection_id_ = jni::fetch_string(env, p, res->connection_id_fieldID);
  res->message_ = jni::fetch_tl_object<businessMessage>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->chat_instance_ = env->GetLongField(p, res->chat_instance_fieldID);
  res->payload_ = jni::fetch_tl_object<CallbackQueryPayload>(env, jni::fetch_object(env, p, res->payload_fieldID));
  return std::move(res);
}

void updateNewBusinessCallbackQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  { jstring nextString = jni::to_jstring(env, connection_id_); if (nextString) { env->SetObjectField(s, connection_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, chat_instance_fieldID, chat_instance_);
  if (payload_ != nullptr) { jobject next; payload_->store(env, next); if (next) { env->SetObjectField(s, payload_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewBusinessCallbackQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewBusinessCallbackQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_field("connectionId", connection_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("chatInstance", chat_instance_);
    s.store_object_field("payload", static_cast<const BaseObject *>(payload_.get()));
    s.store_class_end();
  }
}

void updateNewBusinessCallbackQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewBusinessCallbackQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "Ljava/lang/String;");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessMessage;").c_str());
    chat_instance_fieldID = jni::get_field_id(env, Class, "chatInstance", "J");
    payload_fieldID = jni::get_field_id(env, Class, "payload", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallbackQueryPayload;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewShippingQuery::Class;
jfieldID updateNewShippingQuery::id_fieldID;
jfieldID updateNewShippingQuery::sender_user_id_fieldID;
jfieldID updateNewShippingQuery::invoice_payload_fieldID;
jfieldID updateNewShippingQuery::shipping_address_fieldID;

updateNewShippingQuery::updateNewShippingQuery()
  : id_()
  , sender_user_id_()
  , invoice_payload_()
  , shipping_address_()
{}

updateNewShippingQuery::updateNewShippingQuery(int64 id_, int53 sender_user_id_, string const &invoice_payload_, object_ptr<address> &&shipping_address_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , invoice_payload_(invoice_payload_)
  , shipping_address_(std::move(shipping_address_))
{}

const std::int32_t updateNewShippingQuery::ID;

object_ptr<Update> updateNewShippingQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewShippingQuery> res = make_object<updateNewShippingQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->invoice_payload_ = jni::fetch_string(env, p, res->invoice_payload_fieldID);
  res->shipping_address_ = jni::fetch_tl_object<address>(env, jni::fetch_object(env, p, res->shipping_address_fieldID));
  return std::move(res);
}

void updateNewShippingQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  { jstring nextString = jni::to_jstring(env, invoice_payload_); if (nextString) { env->SetObjectField(s, invoice_payload_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (shipping_address_ != nullptr) { jobject next; shipping_address_->store(env, next); if (next) { env->SetObjectField(s, shipping_address_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewShippingQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewShippingQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_field("invoicePayload", invoice_payload_);
    s.store_object_field("shippingAddress", static_cast<const BaseObject *>(shipping_address_.get()));
    s.store_class_end();
  }
}

void updateNewShippingQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewShippingQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "Ljava/lang/String;");
    shipping_address_fieldID = jni::get_field_id(env, Class, "shippingAddress", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Address;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewPreCheckoutQuery::Class;
jfieldID updateNewPreCheckoutQuery::id_fieldID;
jfieldID updateNewPreCheckoutQuery::sender_user_id_fieldID;
jfieldID updateNewPreCheckoutQuery::currency_fieldID;
jfieldID updateNewPreCheckoutQuery::total_amount_fieldID;
jfieldID updateNewPreCheckoutQuery::invoice_payload_fieldID;
jfieldID updateNewPreCheckoutQuery::shipping_option_id_fieldID;
jfieldID updateNewPreCheckoutQuery::order_info_fieldID;

updateNewPreCheckoutQuery::updateNewPreCheckoutQuery()
  : id_()
  , sender_user_id_()
  , currency_()
  , total_amount_()
  , invoice_payload_()
  , shipping_option_id_()
  , order_info_()
{}

updateNewPreCheckoutQuery::updateNewPreCheckoutQuery(int64 id_, int53 sender_user_id_, string const &currency_, int53 total_amount_, bytes const &invoice_payload_, string const &shipping_option_id_, object_ptr<orderInfo> &&order_info_)
  : id_(id_)
  , sender_user_id_(sender_user_id_)
  , currency_(currency_)
  , total_amount_(total_amount_)
  , invoice_payload_(std::move(invoice_payload_))
  , shipping_option_id_(shipping_option_id_)
  , order_info_(std::move(order_info_))
{}

const std::int32_t updateNewPreCheckoutQuery::ID;

object_ptr<Update> updateNewPreCheckoutQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewPreCheckoutQuery> res = make_object<updateNewPreCheckoutQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->sender_user_id_ = env->GetLongField(p, res->sender_user_id_fieldID);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->total_amount_ = env->GetLongField(p, res->total_amount_fieldID);
  res->invoice_payload_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->invoice_payload_fieldID));
  res->shipping_option_id_ = jni::fetch_string(env, p, res->shipping_option_id_fieldID);
  res->order_info_ = jni::fetch_tl_object<orderInfo>(env, jni::fetch_object(env, p, res->order_info_fieldID));
  return std::move(res);
}

void updateNewPreCheckoutQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  env->SetLongField(s, sender_user_id_fieldID, sender_user_id_);
  { jstring nextString = jni::to_jstring(env, currency_); if (nextString) { env->SetObjectField(s, currency_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetLongField(s, total_amount_fieldID, total_amount_);
  { jbyteArray nextBytes = jni::to_bytes(env, invoice_payload_); if (nextBytes) { env->SetObjectField(s, invoice_payload_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, shipping_option_id_); if (nextString) { env->SetObjectField(s, shipping_option_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (order_info_ != nullptr) { jobject next; order_info_->store(env, next); if (next) { env->SetObjectField(s, order_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewPreCheckoutQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewPreCheckoutQuery");
    s.store_field("id", id_);
    s.store_field("senderUserId", sender_user_id_);
    s.store_field("currency", currency_);
    s.store_field("totalAmount", total_amount_);
    s.store_bytes_field("invoicePayload", invoice_payload_);
    s.store_field("shippingOptionId", shipping_option_id_);
    s.store_object_field("orderInfo", static_cast<const BaseObject *>(order_info_.get()));
    s.store_class_end();
  }
}

void updateNewPreCheckoutQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewPreCheckoutQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    sender_user_id_fieldID = jni::get_field_id(env, Class, "senderUserId", "J");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    total_amount_fieldID = jni::get_field_id(env, Class, "totalAmount", "J");
    invoice_payload_fieldID = jni::get_field_id(env, Class, "invoicePayload", "[B");
    shipping_option_id_fieldID = jni::get_field_id(env, Class, "shippingOptionId", "Ljava/lang/String;");
    order_info_fieldID = jni::get_field_id(env, Class, "orderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OrderInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewCustomEvent::Class;
jfieldID updateNewCustomEvent::event_fieldID;

updateNewCustomEvent::updateNewCustomEvent()
  : event_()
{}

updateNewCustomEvent::updateNewCustomEvent(string const &event_)
  : event_(event_)
{}

const std::int32_t updateNewCustomEvent::ID;

object_ptr<Update> updateNewCustomEvent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewCustomEvent> res = make_object<updateNewCustomEvent>();
  res->event_ = jni::fetch_string(env, p, res->event_fieldID);
  return std::move(res);
}

void updateNewCustomEvent::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, event_); if (nextString) { env->SetObjectField(s, event_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updateNewCustomEvent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewCustomEvent");
    s.store_field("event", event_);
    s.store_class_end();
  }
}

void updateNewCustomEvent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewCustomEvent").c_str());
    event_fieldID = jni::get_field_id(env, Class, "event", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updateNewCustomQuery::Class;
jfieldID updateNewCustomQuery::id_fieldID;
jfieldID updateNewCustomQuery::data_fieldID;
jfieldID updateNewCustomQuery::timeout_fieldID;

updateNewCustomQuery::updateNewCustomQuery()
  : id_()
  , data_()
  , timeout_()
{}

updateNewCustomQuery::updateNewCustomQuery(int64 id_, string const &data_, int32 timeout_)
  : id_(id_)
  , data_(data_)
  , timeout_(timeout_)
{}

const std::int32_t updateNewCustomQuery::ID;

object_ptr<Update> updateNewCustomQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewCustomQuery> res = make_object<updateNewCustomQuery>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  res->timeout_ = env->GetIntField(p, res->timeout_fieldID);
  return std::move(res);
}

void updateNewCustomQuery::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, data_); if (nextString) { env->SetObjectField(s, data_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, timeout_fieldID, timeout_);
}

void updateNewCustomQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewCustomQuery");
    s.store_field("id", id_);
    s.store_field("data", data_);
    s.store_field("timeout", timeout_);
    s.store_class_end();
  }
}

void updateNewCustomQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewCustomQuery").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    timeout_fieldID = jni::get_field_id(env, Class, "timeout", "I");
    return true;
  }();
  (void)is_inited;
}

jclass updatePoll::Class;
jfieldID updatePoll::poll_fieldID;

updatePoll::updatePoll()
  : poll_()
{}

updatePoll::updatePoll(object_ptr<poll> &&poll_)
  : poll_(std::move(poll_))
{}

const std::int32_t updatePoll::ID;

object_ptr<Update> updatePoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updatePoll> res = make_object<updatePoll>();
  res->poll_ = jni::fetch_tl_object<poll>(env, jni::fetch_object(env, p, res->poll_fieldID));
  return std::move(res);
}

void updatePoll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (poll_ != nullptr) { jobject next; poll_->store(env, next); if (next) { env->SetObjectField(s, poll_fieldID, next); env->DeleteLocalRef(next); } }
}

void updatePoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdatePoll");
    s.store_object_field("poll", static_cast<const BaseObject *>(poll_.get()));
    s.store_class_end();
  }
}

void updatePoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdatePoll").c_str());
    poll_fieldID = jni::get_field_id(env, Class, "poll", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Poll;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updatePollAnswer::Class;
jfieldID updatePollAnswer::poll_id_fieldID;
jfieldID updatePollAnswer::voter_id_fieldID;
jfieldID updatePollAnswer::option_ids_fieldID;

updatePollAnswer::updatePollAnswer()
  : poll_id_()
  , voter_id_()
  , option_ids_()
{}

updatePollAnswer::updatePollAnswer(int64 poll_id_, object_ptr<MessageSender> &&voter_id_, array<int32> &&option_ids_)
  : poll_id_(poll_id_)
  , voter_id_(std::move(voter_id_))
  , option_ids_(std::move(option_ids_))
{}

const std::int32_t updatePollAnswer::ID;

object_ptr<Update> updatePollAnswer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updatePollAnswer> res = make_object<updatePollAnswer>();
  res->poll_id_ = env->GetLongField(p, res->poll_id_fieldID);
  res->voter_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->voter_id_fieldID));
  res->option_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->option_ids_fieldID));
  return std::move(res);
}

void updatePollAnswer::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, poll_id_fieldID, poll_id_);
  if (voter_id_ != nullptr) { jobject next; voter_id_->store(env, next); if (next) { env->SetObjectField(s, voter_id_fieldID, next); env->DeleteLocalRef(next); } }
  { auto arr_tmp_ = jni::store_vector(env, option_ids_); if (arr_tmp_) { env->SetObjectField(s, option_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updatePollAnswer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdatePollAnswer");
    s.store_field("pollId", poll_id_);
    s.store_object_field("voterId", static_cast<const BaseObject *>(voter_id_.get()));
    { s.store_vector_begin("optionIds", option_ids_.size()); for (const auto &_value : option_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updatePollAnswer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdatePollAnswer").c_str());
    poll_id_fieldID = jni::get_field_id(env, Class, "pollId", "J");
    voter_id_fieldID = jni::get_field_id(env, Class, "voterId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    option_ids_fieldID = jni::get_field_id(env, Class, "optionIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass updateChatMember::Class;
jfieldID updateChatMember::chat_id_fieldID;
jfieldID updateChatMember::actor_user_id_fieldID;
jfieldID updateChatMember::date_fieldID;
jfieldID updateChatMember::invite_link_fieldID;
jfieldID updateChatMember::via_join_request_fieldID;
jfieldID updateChatMember::via_chat_folder_invite_link_fieldID;
jfieldID updateChatMember::old_chat_member_fieldID;
jfieldID updateChatMember::new_chat_member_fieldID;

updateChatMember::updateChatMember()
  : chat_id_()
  , actor_user_id_()
  , date_()
  , invite_link_()
  , via_join_request_()
  , via_chat_folder_invite_link_()
  , old_chat_member_()
  , new_chat_member_()
{}

updateChatMember::updateChatMember(int53 chat_id_, int53 actor_user_id_, int32 date_, object_ptr<chatInviteLink> &&invite_link_, bool via_join_request_, bool via_chat_folder_invite_link_, object_ptr<chatMember> &&old_chat_member_, object_ptr<chatMember> &&new_chat_member_)
  : chat_id_(chat_id_)
  , actor_user_id_(actor_user_id_)
  , date_(date_)
  , invite_link_(std::move(invite_link_))
  , via_join_request_(via_join_request_)
  , via_chat_folder_invite_link_(via_chat_folder_invite_link_)
  , old_chat_member_(std::move(old_chat_member_))
  , new_chat_member_(std::move(new_chat_member_))
{}

const std::int32_t updateChatMember::ID;

object_ptr<Update> updateChatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatMember> res = make_object<updateChatMember>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->actor_user_id_ = env->GetLongField(p, res->actor_user_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  res->via_join_request_ = (env->GetBooleanField(p, res->via_join_request_fieldID) != 0);
  res->via_chat_folder_invite_link_ = (env->GetBooleanField(p, res->via_chat_folder_invite_link_fieldID) != 0);
  res->old_chat_member_ = jni::fetch_tl_object<chatMember>(env, jni::fetch_object(env, p, res->old_chat_member_fieldID));
  res->new_chat_member_ = jni::fetch_tl_object<chatMember>(env, jni::fetch_object(env, p, res->new_chat_member_fieldID));
  return std::move(res);
}

void updateChatMember::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, actor_user_id_fieldID, actor_user_id_);
  env->SetIntField(s, date_fieldID, date_);
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, via_join_request_fieldID, via_join_request_);
  env->SetBooleanField(s, via_chat_folder_invite_link_fieldID, via_chat_folder_invite_link_);
  if (old_chat_member_ != nullptr) { jobject next; old_chat_member_->store(env, next); if (next) { env->SetObjectField(s, old_chat_member_fieldID, next); env->DeleteLocalRef(next); } }
  if (new_chat_member_ != nullptr) { jobject next; new_chat_member_->store(env, next); if (next) { env->SetObjectField(s, new_chat_member_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatMember");
    s.store_field("chatId", chat_id_);
    s.store_field("actorUserId", actor_user_id_);
    s.store_field("date", date_);
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_field("viaJoinRequest", via_join_request_);
    s.store_field("viaChatFolderInviteLink", via_chat_folder_invite_link_);
    s.store_object_field("oldChatMember", static_cast<const BaseObject *>(old_chat_member_.get()));
    s.store_object_field("newChatMember", static_cast<const BaseObject *>(new_chat_member_.get()));
    s.store_class_end();
  }
}

void updateChatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatMember").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    actor_user_id_fieldID = jni::get_field_id(env, Class, "actorUserId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    via_join_request_fieldID = jni::get_field_id(env, Class, "viaJoinRequest", "Z");
    via_chat_folder_invite_link_fieldID = jni::get_field_id(env, Class, "viaChatFolderInviteLink", "Z");
    old_chat_member_fieldID = jni::get_field_id(env, Class, "oldChatMember", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMember;").c_str());
    new_chat_member_fieldID = jni::get_field_id(env, Class, "newChatMember", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMember;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateNewChatJoinRequest::Class;
jfieldID updateNewChatJoinRequest::chat_id_fieldID;
jfieldID updateNewChatJoinRequest::request_fieldID;
jfieldID updateNewChatJoinRequest::user_chat_id_fieldID;
jfieldID updateNewChatJoinRequest::invite_link_fieldID;

updateNewChatJoinRequest::updateNewChatJoinRequest()
  : chat_id_()
  , request_()
  , user_chat_id_()
  , invite_link_()
{}

updateNewChatJoinRequest::updateNewChatJoinRequest(int53 chat_id_, object_ptr<chatJoinRequest> &&request_, int53 user_chat_id_, object_ptr<chatInviteLink> &&invite_link_)
  : chat_id_(chat_id_)
  , request_(std::move(request_))
  , user_chat_id_(user_chat_id_)
  , invite_link_(std::move(invite_link_))
{}

const std::int32_t updateNewChatJoinRequest::ID;

object_ptr<Update> updateNewChatJoinRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateNewChatJoinRequest> res = make_object<updateNewChatJoinRequest>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->request_ = jni::fetch_tl_object<chatJoinRequest>(env, jni::fetch_object(env, p, res->request_fieldID));
  res->user_chat_id_ = env->GetLongField(p, res->user_chat_id_fieldID);
  res->invite_link_ = jni::fetch_tl_object<chatInviteLink>(env, jni::fetch_object(env, p, res->invite_link_fieldID));
  return std::move(res);
}

void updateNewChatJoinRequest::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (request_ != nullptr) { jobject next; request_->store(env, next); if (next) { env->SetObjectField(s, request_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, user_chat_id_fieldID, user_chat_id_);
  if (invite_link_ != nullptr) { jobject next; invite_link_->store(env, next); if (next) { env->SetObjectField(s, invite_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateNewChatJoinRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateNewChatJoinRequest");
    s.store_field("chatId", chat_id_);
    s.store_object_field("request", static_cast<const BaseObject *>(request_.get()));
    s.store_field("userChatId", user_chat_id_);
    s.store_object_field("inviteLink", static_cast<const BaseObject *>(invite_link_.get()));
    s.store_class_end();
  }
}

void updateNewChatJoinRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateNewChatJoinRequest").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    request_fieldID = jni::get_field_id(env, Class, "request", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatJoinRequest;").c_str());
    user_chat_id_fieldID = jni::get_field_id(env, Class, "userChatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateChatBoost::Class;
jfieldID updateChatBoost::chat_id_fieldID;
jfieldID updateChatBoost::boost_fieldID;

updateChatBoost::updateChatBoost()
  : chat_id_()
  , boost_()
{}

updateChatBoost::updateChatBoost(int53 chat_id_, object_ptr<chatBoost> &&boost_)
  : chat_id_(chat_id_)
  , boost_(std::move(boost_))
{}

const std::int32_t updateChatBoost::ID;

object_ptr<Update> updateChatBoost::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateChatBoost> res = make_object<updateChatBoost>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->boost_ = jni::fetch_tl_object<chatBoost>(env, jni::fetch_object(env, p, res->boost_fieldID));
  return std::move(res);
}

void updateChatBoost::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  if (boost_ != nullptr) { jobject next; boost_->store(env, next); if (next) { env->SetObjectField(s, boost_fieldID, next); env->DeleteLocalRef(next); } }
}

void updateChatBoost::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateChatBoost");
    s.store_field("chatId", chat_id_);
    s.store_object_field("boost", static_cast<const BaseObject *>(boost_.get()));
    s.store_class_end();
  }
}

void updateChatBoost::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateChatBoost").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    boost_fieldID = jni::get_field_id(env, Class, "boost", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatBoost;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageReaction::Class;
jfieldID updateMessageReaction::chat_id_fieldID;
jfieldID updateMessageReaction::message_id_fieldID;
jfieldID updateMessageReaction::actor_id_fieldID;
jfieldID updateMessageReaction::date_fieldID;
jfieldID updateMessageReaction::old_reaction_types_fieldID;
jfieldID updateMessageReaction::new_reaction_types_fieldID;

updateMessageReaction::updateMessageReaction()
  : chat_id_()
  , message_id_()
  , actor_id_()
  , date_()
  , old_reaction_types_()
  , new_reaction_types_()
{}

updateMessageReaction::updateMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&actor_id_, int32 date_, array<object_ptr<ReactionType>> &&old_reaction_types_, array<object_ptr<ReactionType>> &&new_reaction_types_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , actor_id_(std::move(actor_id_))
  , date_(date_)
  , old_reaction_types_(std::move(old_reaction_types_))
  , new_reaction_types_(std::move(new_reaction_types_))
{}

const std::int32_t updateMessageReaction::ID;

object_ptr<Update> updateMessageReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageReaction> res = make_object<updateMessageReaction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->actor_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->actor_id_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->old_reaction_types_ = jni::FetchVector<ReactionType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->old_reaction_types_fieldID));
  res->new_reaction_types_ = jni::FetchVector<ReactionType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->new_reaction_types_fieldID));
  return std::move(res);
}

void updateMessageReaction::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  if (actor_id_ != nullptr) { jobject next; actor_id_->store(env, next); if (next) { env->SetObjectField(s, actor_id_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
  { auto arr_tmp_ = jni::store_vector(env, old_reaction_types_); if (arr_tmp_) { env->SetObjectField(s, old_reaction_types_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, new_reaction_types_); if (arr_tmp_) { env->SetObjectField(s, new_reaction_types_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateMessageReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageReaction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("actorId", static_cast<const BaseObject *>(actor_id_.get()));
    s.store_field("date", date_);
    { s.store_vector_begin("oldReactionTypes", old_reaction_types_.size()); for (const auto &_value : old_reaction_types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("newReactionTypes", new_reaction_types_.size()); for (const auto &_value : new_reaction_types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateMessageReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageReaction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    actor_id_fieldID = jni::get_field_id(env, Class, "actorId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    old_reaction_types_fieldID = jni::get_field_id(env, Class, "oldReactionTypes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    new_reaction_types_fieldID = jni::get_field_id(env, Class, "newReactionTypes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updateMessageReactions::Class;
jfieldID updateMessageReactions::chat_id_fieldID;
jfieldID updateMessageReactions::message_id_fieldID;
jfieldID updateMessageReactions::date_fieldID;
jfieldID updateMessageReactions::reactions_fieldID;

updateMessageReactions::updateMessageReactions()
  : chat_id_()
  , message_id_()
  , date_()
  , reactions_()
{}

updateMessageReactions::updateMessageReactions(int53 chat_id_, int53 message_id_, int32 date_, array<object_ptr<messageReaction>> &&reactions_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , date_(date_)
  , reactions_(std::move(reactions_))
{}

const std::int32_t updateMessageReactions::ID;

object_ptr<Update> updateMessageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updateMessageReactions> res = make_object<updateMessageReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  res->reactions_ = jni::FetchVector<messageReaction>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->reactions_fieldID));
  return std::move(res);
}

void updateMessageReactions::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, chat_id_fieldID, chat_id_);
  env->SetLongField(s, message_id_fieldID, message_id_);
  env->SetIntField(s, date_fieldID, date_);
  { auto arr_tmp_ = jni::store_vector(env, reactions_); if (arr_tmp_) { env->SetObjectField(s, reactions_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updateMessageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdateMessageReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("date", date_);
    { s.store_vector_begin("reactions", reactions_.size()); for (const auto &_value : reactions_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updateMessageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdateMessageReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    reactions_fieldID = jni::get_field_id(env, Class, "reactions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$MessageReaction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass updatePaidMediaPurchased::Class;
jfieldID updatePaidMediaPurchased::user_id_fieldID;
jfieldID updatePaidMediaPurchased::payload_fieldID;

updatePaidMediaPurchased::updatePaidMediaPurchased()
  : user_id_()
  , payload_()
{}

updatePaidMediaPurchased::updatePaidMediaPurchased(int53 user_id_, string const &payload_)
  : user_id_(user_id_)
  , payload_(payload_)
{}

const std::int32_t updatePaidMediaPurchased::ID;

object_ptr<Update> updatePaidMediaPurchased::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updatePaidMediaPurchased> res = make_object<updatePaidMediaPurchased>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  return std::move(res);
}

void updatePaidMediaPurchased::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, user_id_fieldID, user_id_);
  { jstring nextString = jni::to_jstring(env, payload_); if (nextString) { env->SetObjectField(s, payload_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void updatePaidMediaPurchased::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpdatePaidMediaPurchased");
    s.store_field("userId", user_id_);
    s.store_field("payload", payload_);
    s.store_class_end();
  }
}

void updatePaidMediaPurchased::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpdatePaidMediaPurchased").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass updates::Class;
jfieldID updates::updates_fieldID;

updates::updates()
  : updates_()
{}

updates::updates(array<object_ptr<Update>> &&updates_)
  : updates_(std::move(updates_))
{}

const std::int32_t updates::ID;

object_ptr<updates> updates::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<updates> res = make_object<updates>();
  res->updates_ = jni::FetchVector<Update>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->updates_fieldID));
  return res;
}

void updates::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, updates_); if (arr_tmp_) { env->SetObjectField(s, updates_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void updates::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Updates");
    { s.store_vector_begin("updates", updates_.size()); for (const auto &_value : updates_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void updates::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Updates").c_str());
    updates_fieldID = jni::get_field_id(env, Class, "updates", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Update;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass upgradeGiftResult::Class;
jfieldID upgradeGiftResult::gift_fieldID;
jfieldID upgradeGiftResult::received_gift_id_fieldID;
jfieldID upgradeGiftResult::is_saved_fieldID;
jfieldID upgradeGiftResult::can_be_transferred_fieldID;
jfieldID upgradeGiftResult::transfer_star_count_fieldID;
jfieldID upgradeGiftResult::next_transfer_date_fieldID;
jfieldID upgradeGiftResult::next_resale_date_fieldID;
jfieldID upgradeGiftResult::export_date_fieldID;

upgradeGiftResult::upgradeGiftResult()
  : gift_()
  , received_gift_id_()
  , is_saved_()
  , can_be_transferred_()
  , transfer_star_count_()
  , next_transfer_date_()
  , next_resale_date_()
  , export_date_()
{}

upgradeGiftResult::upgradeGiftResult(object_ptr<upgradedGift> &&gift_, string const &received_gift_id_, bool is_saved_, bool can_be_transferred_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_)
  : gift_(std::move(gift_))
  , received_gift_id_(received_gift_id_)
  , is_saved_(is_saved_)
  , can_be_transferred_(can_be_transferred_)
  , transfer_star_count_(transfer_star_count_)
  , next_transfer_date_(next_transfer_date_)
  , next_resale_date_(next_resale_date_)
  , export_date_(export_date_)
{}

const std::int32_t upgradeGiftResult::ID;

object_ptr<upgradeGiftResult> upgradeGiftResult::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradeGiftResult> res = make_object<upgradeGiftResult>();
  res->gift_ = jni::fetch_tl_object<upgradedGift>(env, jni::fetch_object(env, p, res->gift_fieldID));
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->is_saved_ = (env->GetBooleanField(p, res->is_saved_fieldID) != 0);
  res->can_be_transferred_ = (env->GetBooleanField(p, res->can_be_transferred_fieldID) != 0);
  res->transfer_star_count_ = env->GetLongField(p, res->transfer_star_count_fieldID);
  res->next_transfer_date_ = env->GetIntField(p, res->next_transfer_date_fieldID);
  res->next_resale_date_ = env->GetIntField(p, res->next_resale_date_fieldID);
  res->export_date_ = env->GetIntField(p, res->export_date_fieldID);
  return res;
}

void upgradeGiftResult::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (gift_ != nullptr) { jobject next; gift_->store(env, next); if (next) { env->SetObjectField(s, gift_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, received_gift_id_); if (nextString) { env->SetObjectField(s, received_gift_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, is_saved_fieldID, is_saved_);
  env->SetBooleanField(s, can_be_transferred_fieldID, can_be_transferred_);
  env->SetLongField(s, transfer_star_count_fieldID, transfer_star_count_);
  env->SetIntField(s, next_transfer_date_fieldID, next_transfer_date_);
  env->SetIntField(s, next_resale_date_fieldID, next_resale_date_);
  env->SetIntField(s, export_date_fieldID, export_date_);
}

void upgradeGiftResult::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradeGiftResult");
    s.store_object_field("gift", static_cast<const BaseObject *>(gift_.get()));
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("isSaved", is_saved_);
    s.store_field("canBeTransferred", can_be_transferred_);
    s.store_field("transferStarCount", transfer_star_count_);
    s.store_field("nextTransferDate", next_transfer_date_);
    s.store_field("nextResaleDate", next_resale_date_);
    s.store_field("exportDate", export_date_);
    s.store_class_end();
  }
}

void upgradeGiftResult::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradeGiftResult").c_str());
    gift_fieldID = jni::get_field_id(env, Class, "gift", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGift;").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    is_saved_fieldID = jni::get_field_id(env, Class, "isSaved", "Z");
    can_be_transferred_fieldID = jni::get_field_id(env, Class, "canBeTransferred", "Z");
    transfer_star_count_fieldID = jni::get_field_id(env, Class, "transferStarCount", "J");
    next_transfer_date_fieldID = jni::get_field_id(env, Class, "nextTransferDate", "I");
    next_resale_date_fieldID = jni::get_field_id(env, Class, "nextResaleDate", "I");
    export_date_fieldID = jni::get_field_id(env, Class, "exportDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGift::Class;
jfieldID upgradedGift::id_fieldID;
jfieldID upgradedGift::title_fieldID;
jfieldID upgradedGift::name_fieldID;
jfieldID upgradedGift::number_fieldID;
jfieldID upgradedGift::total_upgraded_count_fieldID;
jfieldID upgradedGift::max_upgraded_count_fieldID;
jfieldID upgradedGift::owner_id_fieldID;
jfieldID upgradedGift::owner_address_fieldID;
jfieldID upgradedGift::owner_name_fieldID;
jfieldID upgradedGift::gift_address_fieldID;
jfieldID upgradedGift::model_fieldID;
jfieldID upgradedGift::symbol_fieldID;
jfieldID upgradedGift::backdrop_fieldID;
jfieldID upgradedGift::original_details_fieldID;
jfieldID upgradedGift::resale_star_count_fieldID;

upgradedGift::upgradedGift()
  : id_()
  , title_()
  , name_()
  , number_()
  , total_upgraded_count_()
  , max_upgraded_count_()
  , owner_id_()
  , owner_address_()
  , owner_name_()
  , gift_address_()
  , model_()
  , symbol_()
  , backdrop_()
  , original_details_()
  , resale_star_count_()
{}

upgradedGift::upgradedGift(int64 id_, string const &title_, string const &name_, int32 number_, int32 total_upgraded_count_, int32 max_upgraded_count_, object_ptr<MessageSender> &&owner_id_, string const &owner_address_, string const &owner_name_, string const &gift_address_, object_ptr<upgradedGiftModel> &&model_, object_ptr<upgradedGiftSymbol> &&symbol_, object_ptr<upgradedGiftBackdrop> &&backdrop_, object_ptr<upgradedGiftOriginalDetails> &&original_details_, int53 resale_star_count_)
  : id_(id_)
  , title_(title_)
  , name_(name_)
  , number_(number_)
  , total_upgraded_count_(total_upgraded_count_)
  , max_upgraded_count_(max_upgraded_count_)
  , owner_id_(std::move(owner_id_))
  , owner_address_(owner_address_)
  , owner_name_(owner_name_)
  , gift_address_(gift_address_)
  , model_(std::move(model_))
  , symbol_(std::move(symbol_))
  , backdrop_(std::move(backdrop_))
  , original_details_(std::move(original_details_))
  , resale_star_count_(resale_star_count_)
{}

const std::int32_t upgradedGift::ID;

object_ptr<upgradedGift> upgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGift> res = make_object<upgradedGift>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->number_ = env->GetIntField(p, res->number_fieldID);
  res->total_upgraded_count_ = env->GetIntField(p, res->total_upgraded_count_fieldID);
  res->max_upgraded_count_ = env->GetIntField(p, res->max_upgraded_count_fieldID);
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->owner_address_ = jni::fetch_string(env, p, res->owner_address_fieldID);
  res->owner_name_ = jni::fetch_string(env, p, res->owner_name_fieldID);
  res->gift_address_ = jni::fetch_string(env, p, res->gift_address_fieldID);
  res->model_ = jni::fetch_tl_object<upgradedGiftModel>(env, jni::fetch_object(env, p, res->model_fieldID));
  res->symbol_ = jni::fetch_tl_object<upgradedGiftSymbol>(env, jni::fetch_object(env, p, res->symbol_fieldID));
  res->backdrop_ = jni::fetch_tl_object<upgradedGiftBackdrop>(env, jni::fetch_object(env, p, res->backdrop_fieldID));
  res->original_details_ = jni::fetch_tl_object<upgradedGiftOriginalDetails>(env, jni::fetch_object(env, p, res->original_details_fieldID));
  res->resale_star_count_ = env->GetLongField(p, res->resale_star_count_fieldID);
  return res;
}

void upgradedGift::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, number_fieldID, number_);
  env->SetIntField(s, total_upgraded_count_fieldID, total_upgraded_count_);
  env->SetIntField(s, max_upgraded_count_fieldID, max_upgraded_count_);
  if (owner_id_ != nullptr) { jobject next; owner_id_->store(env, next); if (next) { env->SetObjectField(s, owner_id_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, owner_address_); if (nextString) { env->SetObjectField(s, owner_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, owner_name_); if (nextString) { env->SetObjectField(s, owner_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, gift_address_); if (nextString) { env->SetObjectField(s, gift_address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (model_ != nullptr) { jobject next; model_->store(env, next); if (next) { env->SetObjectField(s, model_fieldID, next); env->DeleteLocalRef(next); } }
  if (symbol_ != nullptr) { jobject next; symbol_->store(env, next); if (next) { env->SetObjectField(s, symbol_fieldID, next); env->DeleteLocalRef(next); } }
  if (backdrop_ != nullptr) { jobject next; backdrop_->store(env, next); if (next) { env->SetObjectField(s, backdrop_fieldID, next); env->DeleteLocalRef(next); } }
  if (original_details_ != nullptr) { jobject next; original_details_->store(env, next); if (next) { env->SetObjectField(s, original_details_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, resale_star_count_fieldID, resale_star_count_);
}

void upgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGift");
    s.store_field("id", id_);
    s.store_field("title", title_);
    s.store_field("name", name_);
    s.store_field("number", number_);
    s.store_field("totalUpgradedCount", total_upgraded_count_);
    s.store_field("maxUpgradedCount", max_upgraded_count_);
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("ownerAddress", owner_address_);
    s.store_field("ownerName", owner_name_);
    s.store_field("giftAddress", gift_address_);
    s.store_object_field("model", static_cast<const BaseObject *>(model_.get()));
    s.store_object_field("symbol", static_cast<const BaseObject *>(symbol_.get()));
    s.store_object_field("backdrop", static_cast<const BaseObject *>(backdrop_.get()));
    s.store_object_field("originalDetails", static_cast<const BaseObject *>(original_details_.get()));
    s.store_field("resaleStarCount", resale_star_count_);
    s.store_class_end();
  }
}

void upgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGift").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    number_fieldID = jni::get_field_id(env, Class, "number", "I");
    total_upgraded_count_fieldID = jni::get_field_id(env, Class, "totalUpgradedCount", "I");
    max_upgraded_count_fieldID = jni::get_field_id(env, Class, "maxUpgradedCount", "I");
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    owner_address_fieldID = jni::get_field_id(env, Class, "ownerAddress", "Ljava/lang/String;");
    owner_name_fieldID = jni::get_field_id(env, Class, "ownerName", "Ljava/lang/String;");
    gift_address_fieldID = jni::get_field_id(env, Class, "giftAddress", "Ljava/lang/String;");
    model_fieldID = jni::get_field_id(env, Class, "model", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftModel;").c_str());
    symbol_fieldID = jni::get_field_id(env, Class, "symbol", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftSymbol;").c_str());
    backdrop_fieldID = jni::get_field_id(env, Class, "backdrop", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdrop;").c_str());
    original_details_fieldID = jni::get_field_id(env, Class, "originalDetails", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftOriginalDetails;").c_str());
    resale_star_count_fieldID = jni::get_field_id(env, Class, "resaleStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass UpgradedGiftAttributeId::Class;

object_ptr<UpgradedGiftAttributeId> UpgradedGiftAttributeId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case upgradedGiftAttributeIdModel::ID:
      return upgradedGiftAttributeIdModel::fetch(env, p);
    case upgradedGiftAttributeIdSymbol::ID:
      return upgradedGiftAttributeIdSymbol::fetch(env, p);
    case upgradedGiftAttributeIdBackdrop::ID:
      return upgradedGiftAttributeIdBackdrop::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void UpgradedGiftAttributeId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftAttributeId").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftAttributeIdModel::Class;
jfieldID upgradedGiftAttributeIdModel::sticker_id_fieldID;

upgradedGiftAttributeIdModel::upgradedGiftAttributeIdModel()
  : sticker_id_()
{}

upgradedGiftAttributeIdModel::upgradedGiftAttributeIdModel(int64 sticker_id_)
  : sticker_id_(sticker_id_)
{}

const std::int32_t upgradedGiftAttributeIdModel::ID;

object_ptr<UpgradedGiftAttributeId> upgradedGiftAttributeIdModel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftAttributeIdModel> res = make_object<upgradedGiftAttributeIdModel>();
  res->sticker_id_ = env->GetLongField(p, res->sticker_id_fieldID);
  return std::move(res);
}

void upgradedGiftAttributeIdModel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftAttributeIdModel");
    s.store_field("stickerId", sticker_id_);
    s.store_class_end();
  }
}

void upgradedGiftAttributeIdModel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftAttributeIdModel").c_str());
    sticker_id_fieldID = jni::get_field_id(env, Class, "stickerId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftAttributeIdSymbol::Class;
jfieldID upgradedGiftAttributeIdSymbol::sticker_id_fieldID;

upgradedGiftAttributeIdSymbol::upgradedGiftAttributeIdSymbol()
  : sticker_id_()
{}

upgradedGiftAttributeIdSymbol::upgradedGiftAttributeIdSymbol(int64 sticker_id_)
  : sticker_id_(sticker_id_)
{}

const std::int32_t upgradedGiftAttributeIdSymbol::ID;

object_ptr<UpgradedGiftAttributeId> upgradedGiftAttributeIdSymbol::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftAttributeIdSymbol> res = make_object<upgradedGiftAttributeIdSymbol>();
  res->sticker_id_ = env->GetLongField(p, res->sticker_id_fieldID);
  return std::move(res);
}

void upgradedGiftAttributeIdSymbol::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftAttributeIdSymbol");
    s.store_field("stickerId", sticker_id_);
    s.store_class_end();
  }
}

void upgradedGiftAttributeIdSymbol::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftAttributeIdSymbol").c_str());
    sticker_id_fieldID = jni::get_field_id(env, Class, "stickerId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftAttributeIdBackdrop::Class;
jfieldID upgradedGiftAttributeIdBackdrop::backdrop_id_fieldID;

upgradedGiftAttributeIdBackdrop::upgradedGiftAttributeIdBackdrop()
  : backdrop_id_()
{}

upgradedGiftAttributeIdBackdrop::upgradedGiftAttributeIdBackdrop(int32 backdrop_id_)
  : backdrop_id_(backdrop_id_)
{}

const std::int32_t upgradedGiftAttributeIdBackdrop::ID;

object_ptr<UpgradedGiftAttributeId> upgradedGiftAttributeIdBackdrop::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftAttributeIdBackdrop> res = make_object<upgradedGiftAttributeIdBackdrop>();
  res->backdrop_id_ = env->GetIntField(p, res->backdrop_id_fieldID);
  return std::move(res);
}

void upgradedGiftAttributeIdBackdrop::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftAttributeIdBackdrop");
    s.store_field("backdropId", backdrop_id_);
    s.store_class_end();
  }
}

void upgradedGiftAttributeIdBackdrop::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftAttributeIdBackdrop").c_str());
    backdrop_id_fieldID = jni::get_field_id(env, Class, "backdropId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftBackdrop::Class;
jfieldID upgradedGiftBackdrop::id_fieldID;
jfieldID upgradedGiftBackdrop::name_fieldID;
jfieldID upgradedGiftBackdrop::colors_fieldID;
jfieldID upgradedGiftBackdrop::rarity_per_mille_fieldID;

upgradedGiftBackdrop::upgradedGiftBackdrop()
  : id_()
  , name_()
  , colors_()
  , rarity_per_mille_()
{}

upgradedGiftBackdrop::upgradedGiftBackdrop(int32 id_, string const &name_, object_ptr<upgradedGiftBackdropColors> &&colors_, int32 rarity_per_mille_)
  : id_(id_)
  , name_(name_)
  , colors_(std::move(colors_))
  , rarity_per_mille_(rarity_per_mille_)
{}

const std::int32_t upgradedGiftBackdrop::ID;

object_ptr<upgradedGiftBackdrop> upgradedGiftBackdrop::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftBackdrop> res = make_object<upgradedGiftBackdrop>();
  res->id_ = env->GetIntField(p, res->id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->colors_ = jni::fetch_tl_object<upgradedGiftBackdropColors>(env, jni::fetch_object(env, p, res->colors_fieldID));
  res->rarity_per_mille_ = env->GetIntField(p, res->rarity_per_mille_fieldID);
  return res;
}

void upgradedGiftBackdrop::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (colors_ != nullptr) { jobject next; colors_->store(env, next); if (next) { env->SetObjectField(s, colors_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, rarity_per_mille_fieldID, rarity_per_mille_);
}

void upgradedGiftBackdrop::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftBackdrop");
    s.store_field("id", id_);
    s.store_field("name", name_);
    s.store_object_field("colors", static_cast<const BaseObject *>(colors_.get()));
    s.store_field("rarityPerMille", rarity_per_mille_);
    s.store_class_end();
  }
}

void upgradedGiftBackdrop::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftBackdrop").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "I");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    colors_fieldID = jni::get_field_id(env, Class, "colors", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdropColors;").c_str());
    rarity_per_mille_fieldID = jni::get_field_id(env, Class, "rarityPerMille", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftBackdropColors::Class;
jfieldID upgradedGiftBackdropColors::center_color_fieldID;
jfieldID upgradedGiftBackdropColors::edge_color_fieldID;
jfieldID upgradedGiftBackdropColors::symbol_color_fieldID;
jfieldID upgradedGiftBackdropColors::text_color_fieldID;

upgradedGiftBackdropColors::upgradedGiftBackdropColors()
  : center_color_()
  , edge_color_()
  , symbol_color_()
  , text_color_()
{}

upgradedGiftBackdropColors::upgradedGiftBackdropColors(int32 center_color_, int32 edge_color_, int32 symbol_color_, int32 text_color_)
  : center_color_(center_color_)
  , edge_color_(edge_color_)
  , symbol_color_(symbol_color_)
  , text_color_(text_color_)
{}

const std::int32_t upgradedGiftBackdropColors::ID;

object_ptr<upgradedGiftBackdropColors> upgradedGiftBackdropColors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftBackdropColors> res = make_object<upgradedGiftBackdropColors>();
  res->center_color_ = env->GetIntField(p, res->center_color_fieldID);
  res->edge_color_ = env->GetIntField(p, res->edge_color_fieldID);
  res->symbol_color_ = env->GetIntField(p, res->symbol_color_fieldID);
  res->text_color_ = env->GetIntField(p, res->text_color_fieldID);
  return res;
}

void upgradedGiftBackdropColors::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, center_color_fieldID, center_color_);
  env->SetIntField(s, edge_color_fieldID, edge_color_);
  env->SetIntField(s, symbol_color_fieldID, symbol_color_);
  env->SetIntField(s, text_color_fieldID, text_color_);
}

void upgradedGiftBackdropColors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftBackdropColors");
    s.store_field("centerColor", center_color_);
    s.store_field("edgeColor", edge_color_);
    s.store_field("symbolColor", symbol_color_);
    s.store_field("textColor", text_color_);
    s.store_class_end();
  }
}

void upgradedGiftBackdropColors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftBackdropColors").c_str());
    center_color_fieldID = jni::get_field_id(env, Class, "centerColor", "I");
    edge_color_fieldID = jni::get_field_id(env, Class, "edgeColor", "I");
    symbol_color_fieldID = jni::get_field_id(env, Class, "symbolColor", "I");
    text_color_fieldID = jni::get_field_id(env, Class, "textColor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftBackdropCount::Class;
jfieldID upgradedGiftBackdropCount::backdrop_fieldID;
jfieldID upgradedGiftBackdropCount::total_count_fieldID;

upgradedGiftBackdropCount::upgradedGiftBackdropCount()
  : backdrop_()
  , total_count_()
{}

upgradedGiftBackdropCount::upgradedGiftBackdropCount(object_ptr<upgradedGiftBackdrop> &&backdrop_, int32 total_count_)
  : backdrop_(std::move(backdrop_))
  , total_count_(total_count_)
{}

const std::int32_t upgradedGiftBackdropCount::ID;

object_ptr<upgradedGiftBackdropCount> upgradedGiftBackdropCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftBackdropCount> res = make_object<upgradedGiftBackdropCount>();
  res->backdrop_ = jni::fetch_tl_object<upgradedGiftBackdrop>(env, jni::fetch_object(env, p, res->backdrop_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  return res;
}

void upgradedGiftBackdropCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (backdrop_ != nullptr) { jobject next; backdrop_->store(env, next); if (next) { env->SetObjectField(s, backdrop_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
}

void upgradedGiftBackdropCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftBackdropCount");
    s.store_object_field("backdrop", static_cast<const BaseObject *>(backdrop_.get()));
    s.store_field("totalCount", total_count_);
    s.store_class_end();
  }
}

void upgradedGiftBackdropCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftBackdropCount").c_str());
    backdrop_fieldID = jni::get_field_id(env, Class, "backdrop", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftBackdrop;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftModel::Class;
jfieldID upgradedGiftModel::name_fieldID;
jfieldID upgradedGiftModel::sticker_fieldID;
jfieldID upgradedGiftModel::rarity_per_mille_fieldID;

upgradedGiftModel::upgradedGiftModel()
  : name_()
  , sticker_()
  , rarity_per_mille_()
{}

upgradedGiftModel::upgradedGiftModel(string const &name_, object_ptr<sticker> &&sticker_, int32 rarity_per_mille_)
  : name_(name_)
  , sticker_(std::move(sticker_))
  , rarity_per_mille_(rarity_per_mille_)
{}

const std::int32_t upgradedGiftModel::ID;

object_ptr<upgradedGiftModel> upgradedGiftModel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftModel> res = make_object<upgradedGiftModel>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->rarity_per_mille_ = env->GetIntField(p, res->rarity_per_mille_fieldID);
  return res;
}

void upgradedGiftModel::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, rarity_per_mille_fieldID, rarity_per_mille_);
}

void upgradedGiftModel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftModel");
    s.store_field("name", name_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("rarityPerMille", rarity_per_mille_);
    s.store_class_end();
  }
}

void upgradedGiftModel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftModel").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    rarity_per_mille_fieldID = jni::get_field_id(env, Class, "rarityPerMille", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftModelCount::Class;
jfieldID upgradedGiftModelCount::model_fieldID;
jfieldID upgradedGiftModelCount::total_count_fieldID;

upgradedGiftModelCount::upgradedGiftModelCount()
  : model_()
  , total_count_()
{}

upgradedGiftModelCount::upgradedGiftModelCount(object_ptr<upgradedGiftModel> &&model_, int32 total_count_)
  : model_(std::move(model_))
  , total_count_(total_count_)
{}

const std::int32_t upgradedGiftModelCount::ID;

object_ptr<upgradedGiftModelCount> upgradedGiftModelCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftModelCount> res = make_object<upgradedGiftModelCount>();
  res->model_ = jni::fetch_tl_object<upgradedGiftModel>(env, jni::fetch_object(env, p, res->model_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  return res;
}

void upgradedGiftModelCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (model_ != nullptr) { jobject next; model_->store(env, next); if (next) { env->SetObjectField(s, model_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
}

void upgradedGiftModelCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftModelCount");
    s.store_object_field("model", static_cast<const BaseObject *>(model_.get()));
    s.store_field("totalCount", total_count_);
    s.store_class_end();
  }
}

void upgradedGiftModelCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftModelCount").c_str());
    model_fieldID = jni::get_field_id(env, Class, "model", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftModel;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftOriginalDetails::Class;
jfieldID upgradedGiftOriginalDetails::sender_id_fieldID;
jfieldID upgradedGiftOriginalDetails::receiver_id_fieldID;
jfieldID upgradedGiftOriginalDetails::text_fieldID;
jfieldID upgradedGiftOriginalDetails::date_fieldID;

upgradedGiftOriginalDetails::upgradedGiftOriginalDetails()
  : sender_id_()
  , receiver_id_()
  , text_()
  , date_()
{}

upgradedGiftOriginalDetails::upgradedGiftOriginalDetails(object_ptr<MessageSender> &&sender_id_, object_ptr<MessageSender> &&receiver_id_, object_ptr<formattedText> &&text_, int32 date_)
  : sender_id_(std::move(sender_id_))
  , receiver_id_(std::move(receiver_id_))
  , text_(std::move(text_))
  , date_(date_)
{}

const std::int32_t upgradedGiftOriginalDetails::ID;

object_ptr<upgradedGiftOriginalDetails> upgradedGiftOriginalDetails::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftOriginalDetails> res = make_object<upgradedGiftOriginalDetails>();
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->receiver_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->receiver_id_fieldID));
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void upgradedGiftOriginalDetails::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (sender_id_ != nullptr) { jobject next; sender_id_->store(env, next); if (next) { env->SetObjectField(s, sender_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (receiver_id_ != nullptr) { jobject next; receiver_id_->store(env, next); if (next) { env->SetObjectField(s, receiver_id_fieldID, next); env->DeleteLocalRef(next); } }
  if (text_ != nullptr) { jobject next; text_->store(env, next); if (next) { env->SetObjectField(s, text_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, date_fieldID, date_);
}

void upgradedGiftOriginalDetails::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftOriginalDetails");
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_object_field("receiverId", static_cast<const BaseObject *>(receiver_id_.get()));
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void upgradedGiftOriginalDetails::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftOriginalDetails").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    receiver_id_fieldID = jni::get_field_id(env, Class, "receiverId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftSymbol::Class;
jfieldID upgradedGiftSymbol::name_fieldID;
jfieldID upgradedGiftSymbol::sticker_fieldID;
jfieldID upgradedGiftSymbol::rarity_per_mille_fieldID;

upgradedGiftSymbol::upgradedGiftSymbol()
  : name_()
  , sticker_()
  , rarity_per_mille_()
{}

upgradedGiftSymbol::upgradedGiftSymbol(string const &name_, object_ptr<sticker> &&sticker_, int32 rarity_per_mille_)
  : name_(name_)
  , sticker_(std::move(sticker_))
  , rarity_per_mille_(rarity_per_mille_)
{}

const std::int32_t upgradedGiftSymbol::ID;

object_ptr<upgradedGiftSymbol> upgradedGiftSymbol::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftSymbol> res = make_object<upgradedGiftSymbol>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->sticker_ = jni::fetch_tl_object<sticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->rarity_per_mille_ = env->GetIntField(p, res->rarity_per_mille_fieldID);
  return res;
}

void upgradedGiftSymbol::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, name_); if (nextString) { env->SetObjectField(s, name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (sticker_ != nullptr) { jobject next; sticker_->store(env, next); if (next) { env->SetObjectField(s, sticker_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, rarity_per_mille_fieldID, rarity_per_mille_);
}

void upgradedGiftSymbol::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftSymbol");
    s.store_field("name", name_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("rarityPerMille", rarity_per_mille_);
    s.store_class_end();
  }
}

void upgradedGiftSymbol::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftSymbol").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Sticker;").c_str());
    rarity_per_mille_fieldID = jni::get_field_id(env, Class, "rarityPerMille", "I");
    return true;
  }();
  (void)is_inited;
}

jclass upgradedGiftSymbolCount::Class;
jfieldID upgradedGiftSymbolCount::symbol_fieldID;
jfieldID upgradedGiftSymbolCount::total_count_fieldID;

upgradedGiftSymbolCount::upgradedGiftSymbolCount()
  : symbol_()
  , total_count_()
{}

upgradedGiftSymbolCount::upgradedGiftSymbolCount(object_ptr<upgradedGiftSymbol> &&symbol_, int32 total_count_)
  : symbol_(std::move(symbol_))
  , total_count_(total_count_)
{}

const std::int32_t upgradedGiftSymbolCount::ID;

object_ptr<upgradedGiftSymbolCount> upgradedGiftSymbolCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradedGiftSymbolCount> res = make_object<upgradedGiftSymbolCount>();
  res->symbol_ = jni::fetch_tl_object<upgradedGiftSymbol>(env, jni::fetch_object(env, p, res->symbol_fieldID));
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  return res;
}

void upgradedGiftSymbolCount::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (symbol_ != nullptr) { jobject next; symbol_->store(env, next); if (next) { env->SetObjectField(s, symbol_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, total_count_fieldID, total_count_);
}

void upgradedGiftSymbolCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradedGiftSymbolCount");
    s.store_object_field("symbol", static_cast<const BaseObject *>(symbol_.get()));
    s.store_field("totalCount", total_count_);
    s.store_class_end();
  }
}

void upgradedGiftSymbolCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradedGiftSymbolCount").c_str());
    symbol_fieldID = jni::get_field_id(env, Class, "symbol", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UpgradedGiftSymbol;").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass user::Class;
jfieldID user::id_fieldID;
jfieldID user::first_name_fieldID;
jfieldID user::last_name_fieldID;
jfieldID user::usernames_fieldID;
jfieldID user::phone_number_fieldID;
jfieldID user::status_fieldID;
jfieldID user::profile_photo_fieldID;
jfieldID user::accent_color_id_fieldID;
jfieldID user::background_custom_emoji_id_fieldID;
jfieldID user::profile_accent_color_id_fieldID;
jfieldID user::profile_background_custom_emoji_id_fieldID;
jfieldID user::emoji_status_fieldID;
jfieldID user::is_contact_fieldID;
jfieldID user::is_mutual_contact_fieldID;
jfieldID user::is_close_friend_fieldID;
jfieldID user::verification_status_fieldID;
jfieldID user::is_premium_fieldID;
jfieldID user::is_support_fieldID;
jfieldID user::restriction_reason_fieldID;
jfieldID user::has_active_stories_fieldID;
jfieldID user::has_unread_active_stories_fieldID;
jfieldID user::restricts_new_chats_fieldID;
jfieldID user::paid_message_star_count_fieldID;
jfieldID user::have_access_fieldID;
jfieldID user::type_fieldID;
jfieldID user::language_code_fieldID;
jfieldID user::added_to_attachment_menu_fieldID;

user::user()
  : id_()
  , first_name_()
  , last_name_()
  , usernames_()
  , phone_number_()
  , status_()
  , profile_photo_()
  , accent_color_id_()
  , background_custom_emoji_id_()
  , profile_accent_color_id_()
  , profile_background_custom_emoji_id_()
  , emoji_status_()
  , is_contact_()
  , is_mutual_contact_()
  , is_close_friend_()
  , verification_status_()
  , is_premium_()
  , is_support_()
  , restriction_reason_()
  , has_active_stories_()
  , has_unread_active_stories_()
  , restricts_new_chats_()
  , paid_message_star_count_()
  , have_access_()
  , type_()
  , language_code_()
  , added_to_attachment_menu_()
{}

user::user(int53 id_, string const &first_name_, string const &last_name_, object_ptr<usernames> &&usernames_, string const &phone_number_, object_ptr<UserStatus> &&status_, object_ptr<profilePhoto> &&profile_photo_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_, object_ptr<emojiStatus> &&emoji_status_, bool is_contact_, bool is_mutual_contact_, bool is_close_friend_, object_ptr<verificationStatus> &&verification_status_, bool is_premium_, bool is_support_, string const &restriction_reason_, bool has_active_stories_, bool has_unread_active_stories_, bool restricts_new_chats_, int53 paid_message_star_count_, bool have_access_, object_ptr<UserType> &&type_, string const &language_code_, bool added_to_attachment_menu_)
  : id_(id_)
  , first_name_(first_name_)
  , last_name_(last_name_)
  , usernames_(std::move(usernames_))
  , phone_number_(phone_number_)
  , status_(std::move(status_))
  , profile_photo_(std::move(profile_photo_))
  , accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
  , profile_accent_color_id_(profile_accent_color_id_)
  , profile_background_custom_emoji_id_(profile_background_custom_emoji_id_)
  , emoji_status_(std::move(emoji_status_))
  , is_contact_(is_contact_)
  , is_mutual_contact_(is_mutual_contact_)
  , is_close_friend_(is_close_friend_)
  , verification_status_(std::move(verification_status_))
  , is_premium_(is_premium_)
  , is_support_(is_support_)
  , restriction_reason_(restriction_reason_)
  , has_active_stories_(has_active_stories_)
  , has_unread_active_stories_(has_unread_active_stories_)
  , restricts_new_chats_(restricts_new_chats_)
  , paid_message_star_count_(paid_message_star_count_)
  , have_access_(have_access_)
  , type_(std::move(type_))
  , language_code_(language_code_)
  , added_to_attachment_menu_(added_to_attachment_menu_)
{}

const std::int32_t user::ID;

object_ptr<user> user::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<user> res = make_object<user>();
  res->id_ = env->GetLongField(p, res->id_fieldID);
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  res->usernames_ = jni::fetch_tl_object<usernames>(env, jni::fetch_object(env, p, res->usernames_fieldID));
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->status_ = jni::fetch_tl_object<UserStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  res->profile_photo_ = jni::fetch_tl_object<profilePhoto>(env, jni::fetch_object(env, p, res->profile_photo_fieldID));
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  res->profile_accent_color_id_ = env->GetIntField(p, res->profile_accent_color_id_fieldID);
  res->profile_background_custom_emoji_id_ = env->GetLongField(p, res->profile_background_custom_emoji_id_fieldID);
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  res->is_contact_ = (env->GetBooleanField(p, res->is_contact_fieldID) != 0);
  res->is_mutual_contact_ = (env->GetBooleanField(p, res->is_mutual_contact_fieldID) != 0);
  res->is_close_friend_ = (env->GetBooleanField(p, res->is_close_friend_fieldID) != 0);
  res->verification_status_ = jni::fetch_tl_object<verificationStatus>(env, jni::fetch_object(env, p, res->verification_status_fieldID));
  res->is_premium_ = (env->GetBooleanField(p, res->is_premium_fieldID) != 0);
  res->is_support_ = (env->GetBooleanField(p, res->is_support_fieldID) != 0);
  res->restriction_reason_ = jni::fetch_string(env, p, res->restriction_reason_fieldID);
  res->has_active_stories_ = (env->GetBooleanField(p, res->has_active_stories_fieldID) != 0);
  res->has_unread_active_stories_ = (env->GetBooleanField(p, res->has_unread_active_stories_fieldID) != 0);
  res->restricts_new_chats_ = (env->GetBooleanField(p, res->restricts_new_chats_fieldID) != 0);
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  res->have_access_ = (env->GetBooleanField(p, res->have_access_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<UserType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->added_to_attachment_menu_ = (env->GetBooleanField(p, res->added_to_attachment_menu_fieldID) != 0);
  return res;
}

void user::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, id_fieldID, id_);
  { jstring nextString = jni::to_jstring(env, first_name_); if (nextString) { env->SetObjectField(s, first_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, last_name_); if (nextString) { env->SetObjectField(s, last_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (usernames_ != nullptr) { jobject next; usernames_->store(env, next); if (next) { env->SetObjectField(s, usernames_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, phone_number_); if (nextString) { env->SetObjectField(s, phone_number_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (status_ != nullptr) { jobject next; status_->store(env, next); if (next) { env->SetObjectField(s, status_fieldID, next); env->DeleteLocalRef(next); } }
  if (profile_photo_ != nullptr) { jobject next; profile_photo_->store(env, next); if (next) { env->SetObjectField(s, profile_photo_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetIntField(s, accent_color_id_fieldID, accent_color_id_);
  env->SetLongField(s, background_custom_emoji_id_fieldID, background_custom_emoji_id_);
  env->SetIntField(s, profile_accent_color_id_fieldID, profile_accent_color_id_);
  env->SetLongField(s, profile_background_custom_emoji_id_fieldID, profile_background_custom_emoji_id_);
  if (emoji_status_ != nullptr) { jobject next; emoji_status_->store(env, next); if (next) { env->SetObjectField(s, emoji_status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_contact_fieldID, is_contact_);
  env->SetBooleanField(s, is_mutual_contact_fieldID, is_mutual_contact_);
  env->SetBooleanField(s, is_close_friend_fieldID, is_close_friend_);
  if (verification_status_ != nullptr) { jobject next; verification_status_->store(env, next); if (next) { env->SetObjectField(s, verification_status_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, is_premium_fieldID, is_premium_);
  env->SetBooleanField(s, is_support_fieldID, is_support_);
  { jstring nextString = jni::to_jstring(env, restriction_reason_); if (nextString) { env->SetObjectField(s, restriction_reason_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, has_active_stories_fieldID, has_active_stories_);
  env->SetBooleanField(s, has_unread_active_stories_fieldID, has_unread_active_stories_);
  env->SetBooleanField(s, restricts_new_chats_fieldID, restricts_new_chats_);
  env->SetLongField(s, paid_message_star_count_fieldID, paid_message_star_count_);
  env->SetBooleanField(s, have_access_fieldID, have_access_);
  if (type_ != nullptr) { jobject next; type_->store(env, next); if (next) { env->SetObjectField(s, type_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, language_code_); if (nextString) { env->SetObjectField(s, language_code_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, added_to_attachment_menu_fieldID, added_to_attachment_menu_);
}

void user::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "User");
    s.store_field("id", id_);
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_object_field("usernames", static_cast<const BaseObject *>(usernames_.get()));
    s.store_field("phoneNumber", phone_number_);
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_object_field("profilePhoto", static_cast<const BaseObject *>(profile_photo_.get()));
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_field("profileAccentColorId", profile_accent_color_id_);
    s.store_field("profileBackgroundCustomEmojiId", profile_background_custom_emoji_id_);
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_field("isContact", is_contact_);
    s.store_field("isMutualContact", is_mutual_contact_);
    s.store_field("isCloseFriend", is_close_friend_);
    s.store_object_field("verificationStatus", static_cast<const BaseObject *>(verification_status_.get()));
    s.store_field("isPremium", is_premium_);
    s.store_field("isSupport", is_support_);
    s.store_field("restrictionReason", restriction_reason_);
    s.store_field("hasActiveStories", has_active_stories_);
    s.store_field("hasUnreadActiveStories", has_unread_active_stories_);
    s.store_field("restrictsNewChats", restricts_new_chats_);
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_field("haveAccess", have_access_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("languageCode", language_code_);
    s.store_field("addedToAttachmentMenu", added_to_attachment_menu_);
    s.store_class_end();
  }
}

void user::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$User").c_str());
    id_fieldID = jni::get_field_id(env, Class, "id", "J");
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    usernames_fieldID = jni::get_field_id(env, Class, "usernames", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Usernames;").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserStatus;").c_str());
    profile_photo_fieldID = jni::get_field_id(env, Class, "profilePhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProfilePhoto;").c_str());
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "profileAccentColorId", "I");
    profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "profileBackgroundCustomEmojiId", "J");
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    is_contact_fieldID = jni::get_field_id(env, Class, "isContact", "Z");
    is_mutual_contact_fieldID = jni::get_field_id(env, Class, "isMutualContact", "Z");
    is_close_friend_fieldID = jni::get_field_id(env, Class, "isCloseFriend", "Z");
    verification_status_fieldID = jni::get_field_id(env, Class, "verificationStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$VerificationStatus;").c_str());
    is_premium_fieldID = jni::get_field_id(env, Class, "isPremium", "Z");
    is_support_fieldID = jni::get_field_id(env, Class, "isSupport", "Z");
    restriction_reason_fieldID = jni::get_field_id(env, Class, "restrictionReason", "Ljava/lang/String;");
    has_active_stories_fieldID = jni::get_field_id(env, Class, "hasActiveStories", "Z");
    has_unread_active_stories_fieldID = jni::get_field_id(env, Class, "hasUnreadActiveStories", "Z");
    restricts_new_chats_fieldID = jni::get_field_id(env, Class, "restrictsNewChats", "Z");
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    have_access_fieldID = jni::get_field_id(env, Class, "haveAccess", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserType;").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    added_to_attachment_menu_fieldID = jni::get_field_id(env, Class, "addedToAttachmentMenu", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass userFullInfo::Class;
jfieldID userFullInfo::personal_photo_fieldID;
jfieldID userFullInfo::photo_fieldID;
jfieldID userFullInfo::public_photo_fieldID;
jfieldID userFullInfo::block_list_fieldID;
jfieldID userFullInfo::can_be_called_fieldID;
jfieldID userFullInfo::supports_video_calls_fieldID;
jfieldID userFullInfo::has_private_calls_fieldID;
jfieldID userFullInfo::has_private_forwards_fieldID;
jfieldID userFullInfo::has_restricted_voice_and_video_note_messages_fieldID;
jfieldID userFullInfo::has_posted_to_profile_stories_fieldID;
jfieldID userFullInfo::has_sponsored_messages_enabled_fieldID;
jfieldID userFullInfo::need_phone_number_privacy_exception_fieldID;
jfieldID userFullInfo::set_chat_background_fieldID;
jfieldID userFullInfo::bio_fieldID;
jfieldID userFullInfo::birthdate_fieldID;
jfieldID userFullInfo::personal_chat_id_fieldID;
jfieldID userFullInfo::gift_count_fieldID;
jfieldID userFullInfo::group_in_common_count_fieldID;
jfieldID userFullInfo::incoming_paid_message_star_count_fieldID;
jfieldID userFullInfo::outgoing_paid_message_star_count_fieldID;
jfieldID userFullInfo::gift_settings_fieldID;
jfieldID userFullInfo::bot_verification_fieldID;
jfieldID userFullInfo::business_info_fieldID;
jfieldID userFullInfo::bot_info_fieldID;

userFullInfo::userFullInfo()
  : personal_photo_()
  , photo_()
  , public_photo_()
  , block_list_()
  , can_be_called_()
  , supports_video_calls_()
  , has_private_calls_()
  , has_private_forwards_()
  , has_restricted_voice_and_video_note_messages_()
  , has_posted_to_profile_stories_()
  , has_sponsored_messages_enabled_()
  , need_phone_number_privacy_exception_()
  , set_chat_background_()
  , bio_()
  , birthdate_()
  , personal_chat_id_()
  , gift_count_()
  , group_in_common_count_()
  , incoming_paid_message_star_count_()
  , outgoing_paid_message_star_count_()
  , gift_settings_()
  , bot_verification_()
  , business_info_()
  , bot_info_()
{}

userFullInfo::userFullInfo(object_ptr<chatPhoto> &&personal_photo_, object_ptr<chatPhoto> &&photo_, object_ptr<chatPhoto> &&public_photo_, object_ptr<BlockList> &&block_list_, bool can_be_called_, bool supports_video_calls_, bool has_private_calls_, bool has_private_forwards_, bool has_restricted_voice_and_video_note_messages_, bool has_posted_to_profile_stories_, bool has_sponsored_messages_enabled_, bool need_phone_number_privacy_exception_, bool set_chat_background_, object_ptr<formattedText> &&bio_, object_ptr<birthdate> &&birthdate_, int53 personal_chat_id_, int32 gift_count_, int32 group_in_common_count_, int53 incoming_paid_message_star_count_, int53 outgoing_paid_message_star_count_, object_ptr<giftSettings> &&gift_settings_, object_ptr<botVerification> &&bot_verification_, object_ptr<businessInfo> &&business_info_, object_ptr<botInfo> &&bot_info_)
  : personal_photo_(std::move(personal_photo_))
  , photo_(std::move(photo_))
  , public_photo_(std::move(public_photo_))
  , block_list_(std::move(block_list_))
  , can_be_called_(can_be_called_)
  , supports_video_calls_(supports_video_calls_)
  , has_private_calls_(has_private_calls_)
  , has_private_forwards_(has_private_forwards_)
  , has_restricted_voice_and_video_note_messages_(has_restricted_voice_and_video_note_messages_)
  , has_posted_to_profile_stories_(has_posted_to_profile_stories_)
  , has_sponsored_messages_enabled_(has_sponsored_messages_enabled_)
  , need_phone_number_privacy_exception_(need_phone_number_privacy_exception_)
  , set_chat_background_(set_chat_background_)
  , bio_(std::move(bio_))
  , birthdate_(std::move(birthdate_))
  , personal_chat_id_(personal_chat_id_)
  , gift_count_(gift_count_)
  , group_in_common_count_(group_in_common_count_)
  , incoming_paid_message_star_count_(incoming_paid_message_star_count_)
  , outgoing_paid_message_star_count_(outgoing_paid_message_star_count_)
  , gift_settings_(std::move(gift_settings_))
  , bot_verification_(std::move(bot_verification_))
  , business_info_(std::move(business_info_))
  , bot_info_(std::move(bot_info_))
{}

const std::int32_t userFullInfo::ID;

object_ptr<userFullInfo> userFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userFullInfo> res = make_object<userFullInfo>();
  res->personal_photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->personal_photo_fieldID));
  res->photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->public_photo_ = jni::fetch_tl_object<chatPhoto>(env, jni::fetch_object(env, p, res->public_photo_fieldID));
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  res->can_be_called_ = (env->GetBooleanField(p, res->can_be_called_fieldID) != 0);
  res->supports_video_calls_ = (env->GetBooleanField(p, res->supports_video_calls_fieldID) != 0);
  res->has_private_calls_ = (env->GetBooleanField(p, res->has_private_calls_fieldID) != 0);
  res->has_private_forwards_ = (env->GetBooleanField(p, res->has_private_forwards_fieldID) != 0);
  res->has_restricted_voice_and_video_note_messages_ = (env->GetBooleanField(p, res->has_restricted_voice_and_video_note_messages_fieldID) != 0);
  res->has_posted_to_profile_stories_ = (env->GetBooleanField(p, res->has_posted_to_profile_stories_fieldID) != 0);
  res->has_sponsored_messages_enabled_ = (env->GetBooleanField(p, res->has_sponsored_messages_enabled_fieldID) != 0);
  res->need_phone_number_privacy_exception_ = (env->GetBooleanField(p, res->need_phone_number_privacy_exception_fieldID) != 0);
  res->set_chat_background_ = (env->GetBooleanField(p, res->set_chat_background_fieldID) != 0);
  res->bio_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->bio_fieldID));
  res->birthdate_ = jni::fetch_tl_object<birthdate>(env, jni::fetch_object(env, p, res->birthdate_fieldID));
  res->personal_chat_id_ = env->GetLongField(p, res->personal_chat_id_fieldID);
  res->gift_count_ = env->GetIntField(p, res->gift_count_fieldID);
  res->group_in_common_count_ = env->GetIntField(p, res->group_in_common_count_fieldID);
  res->incoming_paid_message_star_count_ = env->GetLongField(p, res->incoming_paid_message_star_count_fieldID);
  res->outgoing_paid_message_star_count_ = env->GetLongField(p, res->outgoing_paid_message_star_count_fieldID);
  res->gift_settings_ = jni::fetch_tl_object<giftSettings>(env, jni::fetch_object(env, p, res->gift_settings_fieldID));
  res->bot_verification_ = jni::fetch_tl_object<botVerification>(env, jni::fetch_object(env, p, res->bot_verification_fieldID));
  res->business_info_ = jni::fetch_tl_object<businessInfo>(env, jni::fetch_object(env, p, res->business_info_fieldID));
  res->bot_info_ = jni::fetch_tl_object<botInfo>(env, jni::fetch_object(env, p, res->bot_info_fieldID));
  return res;
}

void userFullInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (personal_photo_ != nullptr) { jobject next; personal_photo_->store(env, next); if (next) { env->SetObjectField(s, personal_photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (public_photo_ != nullptr) { jobject next; public_photo_->store(env, next); if (next) { env->SetObjectField(s, public_photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (block_list_ != nullptr) { jobject next; block_list_->store(env, next); if (next) { env->SetObjectField(s, block_list_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetBooleanField(s, can_be_called_fieldID, can_be_called_);
  env->SetBooleanField(s, supports_video_calls_fieldID, supports_video_calls_);
  env->SetBooleanField(s, has_private_calls_fieldID, has_private_calls_);
  env->SetBooleanField(s, has_private_forwards_fieldID, has_private_forwards_);
  env->SetBooleanField(s, has_restricted_voice_and_video_note_messages_fieldID, has_restricted_voice_and_video_note_messages_);
  env->SetBooleanField(s, has_posted_to_profile_stories_fieldID, has_posted_to_profile_stories_);
  env->SetBooleanField(s, has_sponsored_messages_enabled_fieldID, has_sponsored_messages_enabled_);
  env->SetBooleanField(s, need_phone_number_privacy_exception_fieldID, need_phone_number_privacy_exception_);
  env->SetBooleanField(s, set_chat_background_fieldID, set_chat_background_);
  if (bio_ != nullptr) { jobject next; bio_->store(env, next); if (next) { env->SetObjectField(s, bio_fieldID, next); env->DeleteLocalRef(next); } }
  if (birthdate_ != nullptr) { jobject next; birthdate_->store(env, next); if (next) { env->SetObjectField(s, birthdate_fieldID, next); env->DeleteLocalRef(next); } }
  env->SetLongField(s, personal_chat_id_fieldID, personal_chat_id_);
  env->SetIntField(s, gift_count_fieldID, gift_count_);
  env->SetIntField(s, group_in_common_count_fieldID, group_in_common_count_);
  env->SetLongField(s, incoming_paid_message_star_count_fieldID, incoming_paid_message_star_count_);
  env->SetLongField(s, outgoing_paid_message_star_count_fieldID, outgoing_paid_message_star_count_);
  if (gift_settings_ != nullptr) { jobject next; gift_settings_->store(env, next); if (next) { env->SetObjectField(s, gift_settings_fieldID, next); env->DeleteLocalRef(next); } }
  if (bot_verification_ != nullptr) { jobject next; bot_verification_->store(env, next); if (next) { env->SetObjectField(s, bot_verification_fieldID, next); env->DeleteLocalRef(next); } }
  if (business_info_ != nullptr) { jobject next; business_info_->store(env, next); if (next) { env->SetObjectField(s, business_info_fieldID, next); env->DeleteLocalRef(next); } }
  if (bot_info_ != nullptr) { jobject next; bot_info_->store(env, next); if (next) { env->SetObjectField(s, bot_info_fieldID, next); env->DeleteLocalRef(next); } }
}

void userFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserFullInfo");
    s.store_object_field("personalPhoto", static_cast<const BaseObject *>(personal_photo_.get()));
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("publicPhoto", static_cast<const BaseObject *>(public_photo_.get()));
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_field("canBeCalled", can_be_called_);
    s.store_field("supportsVideoCalls", supports_video_calls_);
    s.store_field("hasPrivateCalls", has_private_calls_);
    s.store_field("hasPrivateForwards", has_private_forwards_);
    s.store_field("hasRestrictedVoiceAndVideoNoteMessages", has_restricted_voice_and_video_note_messages_);
    s.store_field("hasPostedToProfileStories", has_posted_to_profile_stories_);
    s.store_field("hasSponsoredMessagesEnabled", has_sponsored_messages_enabled_);
    s.store_field("needPhoneNumberPrivacyException", need_phone_number_privacy_exception_);
    s.store_field("setChatBackground", set_chat_background_);
    s.store_object_field("bio", static_cast<const BaseObject *>(bio_.get()));
    s.store_object_field("birthdate", static_cast<const BaseObject *>(birthdate_.get()));
    s.store_field("personalChatId", personal_chat_id_);
    s.store_field("giftCount", gift_count_);
    s.store_field("groupInCommonCount", group_in_common_count_);
    s.store_field("incomingPaidMessageStarCount", incoming_paid_message_star_count_);
    s.store_field("outgoingPaidMessageStarCount", outgoing_paid_message_star_count_);
    s.store_object_field("giftSettings", static_cast<const BaseObject *>(gift_settings_.get()));
    s.store_object_field("botVerification", static_cast<const BaseObject *>(bot_verification_.get()));
    s.store_object_field("businessInfo", static_cast<const BaseObject *>(business_info_.get()));
    s.store_object_field("botInfo", static_cast<const BaseObject *>(bot_info_.get()));
    s.store_class_end();
  }
}

void userFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserFullInfo").c_str());
    personal_photo_fieldID = jni::get_field_id(env, Class, "personalPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    public_photo_fieldID = jni::get_field_id(env, Class, "publicPhoto", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPhoto;").c_str());
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    can_be_called_fieldID = jni::get_field_id(env, Class, "canBeCalled", "Z");
    supports_video_calls_fieldID = jni::get_field_id(env, Class, "supportsVideoCalls", "Z");
    has_private_calls_fieldID = jni::get_field_id(env, Class, "hasPrivateCalls", "Z");
    has_private_forwards_fieldID = jni::get_field_id(env, Class, "hasPrivateForwards", "Z");
    has_restricted_voice_and_video_note_messages_fieldID = jni::get_field_id(env, Class, "hasRestrictedVoiceAndVideoNoteMessages", "Z");
    has_posted_to_profile_stories_fieldID = jni::get_field_id(env, Class, "hasPostedToProfileStories", "Z");
    has_sponsored_messages_enabled_fieldID = jni::get_field_id(env, Class, "hasSponsoredMessagesEnabled", "Z");
    need_phone_number_privacy_exception_fieldID = jni::get_field_id(env, Class, "needPhoneNumberPrivacyException", "Z");
    set_chat_background_fieldID = jni::get_field_id(env, Class, "setChatBackground", "Z");
    bio_fieldID = jni::get_field_id(env, Class, "bio", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    birthdate_fieldID = jni::get_field_id(env, Class, "birthdate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Birthdate;").c_str());
    personal_chat_id_fieldID = jni::get_field_id(env, Class, "personalChatId", "J");
    gift_count_fieldID = jni::get_field_id(env, Class, "giftCount", "I");
    group_in_common_count_fieldID = jni::get_field_id(env, Class, "groupInCommonCount", "I");
    incoming_paid_message_star_count_fieldID = jni::get_field_id(env, Class, "incomingPaidMessageStarCount", "J");
    outgoing_paid_message_star_count_fieldID = jni::get_field_id(env, Class, "outgoingPaidMessageStarCount", "J");
    gift_settings_fieldID = jni::get_field_id(env, Class, "giftSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiftSettings;").c_str());
    bot_verification_fieldID = jni::get_field_id(env, Class, "botVerification", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotVerification;").c_str());
    business_info_fieldID = jni::get_field_id(env, Class, "businessInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessInfo;").c_str());
    bot_info_fieldID = jni::get_field_id(env, Class, "botInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userLink::Class;
jfieldID userLink::url_fieldID;
jfieldID userLink::expires_in_fieldID;

userLink::userLink()
  : url_()
  , expires_in_()
{}

userLink::userLink(string const &url_, int32 expires_in_)
  : url_(url_)
  , expires_in_(expires_in_)
{}

const std::int32_t userLink::ID;

object_ptr<userLink> userLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userLink> res = make_object<userLink>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->expires_in_ = env->GetIntField(p, res->expires_in_fieldID);
  return res;
}

void userLink::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, expires_in_fieldID, expires_in_);
}

void userLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserLink");
    s.store_field("url", url_);
    s.store_field("expiresIn", expires_in_);
    s.store_class_end();
  }
}

void userLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserLink").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    expires_in_fieldID = jni::get_field_id(env, Class, "expiresIn", "I");
    return true;
  }();
  (void)is_inited;
}

jclass UserPrivacySetting::Class;

object_ptr<UserPrivacySetting> UserPrivacySetting::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case userPrivacySettingShowStatus::ID:
      return userPrivacySettingShowStatus::fetch(env, p);
    case userPrivacySettingShowProfilePhoto::ID:
      return userPrivacySettingShowProfilePhoto::fetch(env, p);
    case userPrivacySettingShowLinkInForwardedMessages::ID:
      return userPrivacySettingShowLinkInForwardedMessages::fetch(env, p);
    case userPrivacySettingShowPhoneNumber::ID:
      return userPrivacySettingShowPhoneNumber::fetch(env, p);
    case userPrivacySettingShowBio::ID:
      return userPrivacySettingShowBio::fetch(env, p);
    case userPrivacySettingShowBirthdate::ID:
      return userPrivacySettingShowBirthdate::fetch(env, p);
    case userPrivacySettingAllowChatInvites::ID:
      return userPrivacySettingAllowChatInvites::fetch(env, p);
    case userPrivacySettingAllowCalls::ID:
      return userPrivacySettingAllowCalls::fetch(env, p);
    case userPrivacySettingAllowPeerToPeerCalls::ID:
      return userPrivacySettingAllowPeerToPeerCalls::fetch(env, p);
    case userPrivacySettingAllowFindingByPhoneNumber::ID:
      return userPrivacySettingAllowFindingByPhoneNumber::fetch(env, p);
    case userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::ID:
      return userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::fetch(env, p);
    case userPrivacySettingAutosaveGifts::ID:
      return userPrivacySettingAutosaveGifts::fetch(env, p);
    case userPrivacySettingAllowUnpaidMessages::ID:
      return userPrivacySettingAllowUnpaidMessages::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void UserPrivacySetting::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySetting").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowStatus::Class;

userPrivacySettingShowStatus::userPrivacySettingShowStatus() {
}

const std::int32_t userPrivacySettingShowStatus::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowStatus>();
}

void userPrivacySettingShowStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowStatus");
    s.store_class_end();
  }
}

void userPrivacySettingShowStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowProfilePhoto::Class;

userPrivacySettingShowProfilePhoto::userPrivacySettingShowProfilePhoto() {
}

const std::int32_t userPrivacySettingShowProfilePhoto::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowProfilePhoto>();
}

void userPrivacySettingShowProfilePhoto::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowProfilePhoto");
    s.store_class_end();
  }
}

void userPrivacySettingShowProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowProfilePhoto").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowLinkInForwardedMessages::Class;

userPrivacySettingShowLinkInForwardedMessages::userPrivacySettingShowLinkInForwardedMessages() {
}

const std::int32_t userPrivacySettingShowLinkInForwardedMessages::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowLinkInForwardedMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowLinkInForwardedMessages>();
}

void userPrivacySettingShowLinkInForwardedMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowLinkInForwardedMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowLinkInForwardedMessages");
    s.store_class_end();
  }
}

void userPrivacySettingShowLinkInForwardedMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowLinkInForwardedMessages").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowPhoneNumber::Class;

userPrivacySettingShowPhoneNumber::userPrivacySettingShowPhoneNumber() {
}

const std::int32_t userPrivacySettingShowPhoneNumber::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowPhoneNumber>();
}

void userPrivacySettingShowPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowPhoneNumber");
    s.store_class_end();
  }
}

void userPrivacySettingShowPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowPhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowBio::Class;

userPrivacySettingShowBio::userPrivacySettingShowBio() {
}

const std::int32_t userPrivacySettingShowBio::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowBio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowBio>();
}

void userPrivacySettingShowBio::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowBio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowBio");
    s.store_class_end();
  }
}

void userPrivacySettingShowBio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowBio").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingShowBirthdate::Class;

userPrivacySettingShowBirthdate::userPrivacySettingShowBirthdate() {
}

const std::int32_t userPrivacySettingShowBirthdate::ID;

object_ptr<UserPrivacySetting> userPrivacySettingShowBirthdate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingShowBirthdate>();
}

void userPrivacySettingShowBirthdate::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingShowBirthdate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingShowBirthdate");
    s.store_class_end();
  }
}

void userPrivacySettingShowBirthdate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingShowBirthdate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowChatInvites::Class;

userPrivacySettingAllowChatInvites::userPrivacySettingAllowChatInvites() {
}

const std::int32_t userPrivacySettingAllowChatInvites::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowChatInvites::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowChatInvites>();
}

void userPrivacySettingAllowChatInvites::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowChatInvites::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowChatInvites");
    s.store_class_end();
  }
}

void userPrivacySettingAllowChatInvites::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowChatInvites").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowCalls::Class;

userPrivacySettingAllowCalls::userPrivacySettingAllowCalls() {
}

const std::int32_t userPrivacySettingAllowCalls::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowCalls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowCalls>();
}

void userPrivacySettingAllowCalls::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowCalls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowCalls");
    s.store_class_end();
  }
}

void userPrivacySettingAllowCalls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowCalls").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowPeerToPeerCalls::Class;

userPrivacySettingAllowPeerToPeerCalls::userPrivacySettingAllowPeerToPeerCalls() {
}

const std::int32_t userPrivacySettingAllowPeerToPeerCalls::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowPeerToPeerCalls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowPeerToPeerCalls>();
}

void userPrivacySettingAllowPeerToPeerCalls::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowPeerToPeerCalls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowPeerToPeerCalls");
    s.store_class_end();
  }
}

void userPrivacySettingAllowPeerToPeerCalls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowPeerToPeerCalls").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowFindingByPhoneNumber::Class;

userPrivacySettingAllowFindingByPhoneNumber::userPrivacySettingAllowFindingByPhoneNumber() {
}

const std::int32_t userPrivacySettingAllowFindingByPhoneNumber::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowFindingByPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowFindingByPhoneNumber>();
}

void userPrivacySettingAllowFindingByPhoneNumber::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowFindingByPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowFindingByPhoneNumber");
    s.store_class_end();
  }
}

void userPrivacySettingAllowFindingByPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowFindingByPhoneNumber").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::Class;

userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages() {
}

const std::int32_t userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages>();
}

void userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowPrivateVoiceAndVideoNoteMessages");
    s.store_class_end();
  }
}

void userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowPrivateVoiceAndVideoNoteMessages").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAutosaveGifts::Class;

userPrivacySettingAutosaveGifts::userPrivacySettingAutosaveGifts() {
}

const std::int32_t userPrivacySettingAutosaveGifts::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAutosaveGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAutosaveGifts>();
}

void userPrivacySettingAutosaveGifts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAutosaveGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAutosaveGifts");
    s.store_class_end();
  }
}

void userPrivacySettingAutosaveGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAutosaveGifts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingAllowUnpaidMessages::Class;

userPrivacySettingAllowUnpaidMessages::userPrivacySettingAllowUnpaidMessages() {
}

const std::int32_t userPrivacySettingAllowUnpaidMessages::ID;

object_ptr<UserPrivacySetting> userPrivacySettingAllowUnpaidMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingAllowUnpaidMessages>();
}

void userPrivacySettingAllowUnpaidMessages::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingAllowUnpaidMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingAllowUnpaidMessages");
    s.store_class_end();
  }
}

void userPrivacySettingAllowUnpaidMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingAllowUnpaidMessages").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass UserPrivacySettingRule::Class;

object_ptr<UserPrivacySettingRule> UserPrivacySettingRule::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case userPrivacySettingRuleAllowAll::ID:
      return userPrivacySettingRuleAllowAll::fetch(env, p);
    case userPrivacySettingRuleAllowContacts::ID:
      return userPrivacySettingRuleAllowContacts::fetch(env, p);
    case userPrivacySettingRuleAllowBots::ID:
      return userPrivacySettingRuleAllowBots::fetch(env, p);
    case userPrivacySettingRuleAllowPremiumUsers::ID:
      return userPrivacySettingRuleAllowPremiumUsers::fetch(env, p);
    case userPrivacySettingRuleAllowUsers::ID:
      return userPrivacySettingRuleAllowUsers::fetch(env, p);
    case userPrivacySettingRuleAllowChatMembers::ID:
      return userPrivacySettingRuleAllowChatMembers::fetch(env, p);
    case userPrivacySettingRuleRestrictAll::ID:
      return userPrivacySettingRuleRestrictAll::fetch(env, p);
    case userPrivacySettingRuleRestrictContacts::ID:
      return userPrivacySettingRuleRestrictContacts::fetch(env, p);
    case userPrivacySettingRuleRestrictBots::ID:
      return userPrivacySettingRuleRestrictBots::fetch(env, p);
    case userPrivacySettingRuleRestrictUsers::ID:
      return userPrivacySettingRuleRestrictUsers::fetch(env, p);
    case userPrivacySettingRuleRestrictChatMembers::ID:
      return userPrivacySettingRuleRestrictChatMembers::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void UserPrivacySettingRule::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRule").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowAll::Class;

userPrivacySettingRuleAllowAll::userPrivacySettingRuleAllowAll() {
}

const std::int32_t userPrivacySettingRuleAllowAll::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowAll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleAllowAll>();
}

void userPrivacySettingRuleAllowAll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleAllowAll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowAll");
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowAll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowAll").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowContacts::Class;

userPrivacySettingRuleAllowContacts::userPrivacySettingRuleAllowContacts() {
}

const std::int32_t userPrivacySettingRuleAllowContacts::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleAllowContacts>();
}

void userPrivacySettingRuleAllowContacts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleAllowContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowContacts");
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowBots::Class;

userPrivacySettingRuleAllowBots::userPrivacySettingRuleAllowBots() {
}

const std::int32_t userPrivacySettingRuleAllowBots::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleAllowBots>();
}

void userPrivacySettingRuleAllowBots::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleAllowBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowBots");
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowPremiumUsers::Class;

userPrivacySettingRuleAllowPremiumUsers::userPrivacySettingRuleAllowPremiumUsers() {
}

const std::int32_t userPrivacySettingRuleAllowPremiumUsers::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowPremiumUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleAllowPremiumUsers>();
}

void userPrivacySettingRuleAllowPremiumUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleAllowPremiumUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowPremiumUsers");
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowPremiumUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowPremiumUsers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowUsers::Class;
jfieldID userPrivacySettingRuleAllowUsers::user_ids_fieldID;

userPrivacySettingRuleAllowUsers::userPrivacySettingRuleAllowUsers()
  : user_ids_()
{}

userPrivacySettingRuleAllowUsers::userPrivacySettingRuleAllowUsers(array<int53> &&user_ids_)
  : user_ids_(std::move(user_ids_))
{}

const std::int32_t userPrivacySettingRuleAllowUsers::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userPrivacySettingRuleAllowUsers> res = make_object<userPrivacySettingRuleAllowUsers>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return std::move(res);
}

void userPrivacySettingRuleAllowUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void userPrivacySettingRuleAllowUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowUsers");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowUsers").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleAllowChatMembers::Class;
jfieldID userPrivacySettingRuleAllowChatMembers::chat_ids_fieldID;

userPrivacySettingRuleAllowChatMembers::userPrivacySettingRuleAllowChatMembers()
  : chat_ids_()
{}

userPrivacySettingRuleAllowChatMembers::userPrivacySettingRuleAllowChatMembers(array<int53> &&chat_ids_)
  : chat_ids_(std::move(chat_ids_))
{}

const std::int32_t userPrivacySettingRuleAllowChatMembers::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleAllowChatMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userPrivacySettingRuleAllowChatMembers> res = make_object<userPrivacySettingRuleAllowChatMembers>();
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return std::move(res);
}

void userPrivacySettingRuleAllowChatMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_ids_); if (arr_tmp_) { env->SetObjectField(s, chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void userPrivacySettingRuleAllowChatMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleAllowChatMembers");
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void userPrivacySettingRuleAllowChatMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleAllowChatMembers").c_str());
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleRestrictAll::Class;

userPrivacySettingRuleRestrictAll::userPrivacySettingRuleRestrictAll() {
}

const std::int32_t userPrivacySettingRuleRestrictAll::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleRestrictAll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleRestrictAll>();
}

void userPrivacySettingRuleRestrictAll::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleRestrictAll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleRestrictAll");
    s.store_class_end();
  }
}

void userPrivacySettingRuleRestrictAll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleRestrictAll").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleRestrictContacts::Class;

userPrivacySettingRuleRestrictContacts::userPrivacySettingRuleRestrictContacts() {
}

const std::int32_t userPrivacySettingRuleRestrictContacts::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleRestrictContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleRestrictContacts>();
}

void userPrivacySettingRuleRestrictContacts::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleRestrictContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleRestrictContacts");
    s.store_class_end();
  }
}

void userPrivacySettingRuleRestrictContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleRestrictContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleRestrictBots::Class;

userPrivacySettingRuleRestrictBots::userPrivacySettingRuleRestrictBots() {
}

const std::int32_t userPrivacySettingRuleRestrictBots::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleRestrictBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userPrivacySettingRuleRestrictBots>();
}

void userPrivacySettingRuleRestrictBots::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userPrivacySettingRuleRestrictBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleRestrictBots");
    s.store_class_end();
  }
}

void userPrivacySettingRuleRestrictBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleRestrictBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleRestrictUsers::Class;
jfieldID userPrivacySettingRuleRestrictUsers::user_ids_fieldID;

userPrivacySettingRuleRestrictUsers::userPrivacySettingRuleRestrictUsers()
  : user_ids_()
{}

userPrivacySettingRuleRestrictUsers::userPrivacySettingRuleRestrictUsers(array<int53> &&user_ids_)
  : user_ids_(std::move(user_ids_))
{}

const std::int32_t userPrivacySettingRuleRestrictUsers::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleRestrictUsers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userPrivacySettingRuleRestrictUsers> res = make_object<userPrivacySettingRuleRestrictUsers>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return std::move(res);
}

void userPrivacySettingRuleRestrictUsers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void userPrivacySettingRuleRestrictUsers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleRestrictUsers");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void userPrivacySettingRuleRestrictUsers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleRestrictUsers").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRuleRestrictChatMembers::Class;
jfieldID userPrivacySettingRuleRestrictChatMembers::chat_ids_fieldID;

userPrivacySettingRuleRestrictChatMembers::userPrivacySettingRuleRestrictChatMembers()
  : chat_ids_()
{}

userPrivacySettingRuleRestrictChatMembers::userPrivacySettingRuleRestrictChatMembers(array<int53> &&chat_ids_)
  : chat_ids_(std::move(chat_ids_))
{}

const std::int32_t userPrivacySettingRuleRestrictChatMembers::ID;

object_ptr<UserPrivacySettingRule> userPrivacySettingRuleRestrictChatMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userPrivacySettingRuleRestrictChatMembers> res = make_object<userPrivacySettingRuleRestrictChatMembers>();
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return std::move(res);
}

void userPrivacySettingRuleRestrictChatMembers::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, chat_ids_); if (arr_tmp_) { env->SetObjectField(s, chat_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void userPrivacySettingRuleRestrictChatMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRuleRestrictChatMembers");
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void userPrivacySettingRuleRestrictChatMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRuleRestrictChatMembers").c_str());
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass userPrivacySettingRules::Class;
jfieldID userPrivacySettingRules::rules_fieldID;

userPrivacySettingRules::userPrivacySettingRules()
  : rules_()
{}

userPrivacySettingRules::userPrivacySettingRules(array<object_ptr<UserPrivacySettingRule>> &&rules_)
  : rules_(std::move(rules_))
{}

const std::int32_t userPrivacySettingRules::ID;

object_ptr<userPrivacySettingRules> userPrivacySettingRules::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userPrivacySettingRules> res = make_object<userPrivacySettingRules>();
  res->rules_ = jni::FetchVector<UserPrivacySettingRule>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->rules_fieldID));
  return res;
}

void userPrivacySettingRules::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, rules_); if (arr_tmp_) { env->SetObjectField(s, rules_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void userPrivacySettingRules::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserPrivacySettingRules");
    { s.store_vector_begin("rules", rules_.size()); for (const auto &_value : rules_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void userPrivacySettingRules::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserPrivacySettingRules").c_str());
    rules_fieldID = jni::get_field_id(env, Class, "rules", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UserPrivacySettingRule;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass UserStatus::Class;

object_ptr<UserStatus> UserStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case userStatusEmpty::ID:
      return userStatusEmpty::fetch(env, p);
    case userStatusOnline::ID:
      return userStatusOnline::fetch(env, p);
    case userStatusOffline::ID:
      return userStatusOffline::fetch(env, p);
    case userStatusRecently::ID:
      return userStatusRecently::fetch(env, p);
    case userStatusLastWeek::ID:
      return userStatusLastWeek::fetch(env, p);
    case userStatusLastMonth::ID:
      return userStatusLastMonth::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void UserStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatus").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userStatusEmpty::Class;

userStatusEmpty::userStatusEmpty() {
}

const std::int32_t userStatusEmpty::ID;

object_ptr<UserStatus> userStatusEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userStatusEmpty>();
}

void userStatusEmpty::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userStatusEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusEmpty");
    s.store_class_end();
  }
}

void userStatusEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userStatusOnline::Class;
jfieldID userStatusOnline::expires_fieldID;

userStatusOnline::userStatusOnline()
  : expires_()
{}

userStatusOnline::userStatusOnline(int32 expires_)
  : expires_(expires_)
{}

const std::int32_t userStatusOnline::ID;

object_ptr<UserStatus> userStatusOnline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userStatusOnline> res = make_object<userStatusOnline>();
  res->expires_ = env->GetIntField(p, res->expires_fieldID);
  return std::move(res);
}

void userStatusOnline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, expires_fieldID, expires_);
}

void userStatusOnline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusOnline");
    s.store_field("expires", expires_);
    s.store_class_end();
  }
}

void userStatusOnline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusOnline").c_str());
    expires_fieldID = jni::get_field_id(env, Class, "expires", "I");
    return true;
  }();
  (void)is_inited;
}

jclass userStatusOffline::Class;
jfieldID userStatusOffline::was_online_fieldID;

userStatusOffline::userStatusOffline()
  : was_online_()
{}

userStatusOffline::userStatusOffline(int32 was_online_)
  : was_online_(was_online_)
{}

const std::int32_t userStatusOffline::ID;

object_ptr<UserStatus> userStatusOffline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userStatusOffline> res = make_object<userStatusOffline>();
  res->was_online_ = env->GetIntField(p, res->was_online_fieldID);
  return std::move(res);
}

void userStatusOffline::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, was_online_fieldID, was_online_);
}

void userStatusOffline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusOffline");
    s.store_field("wasOnline", was_online_);
    s.store_class_end();
  }
}

void userStatusOffline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusOffline").c_str());
    was_online_fieldID = jni::get_field_id(env, Class, "wasOnline", "I");
    return true;
  }();
  (void)is_inited;
}

jclass userStatusRecently::Class;
jfieldID userStatusRecently::by_my_privacy_settings_fieldID;

userStatusRecently::userStatusRecently()
  : by_my_privacy_settings_()
{}

userStatusRecently::userStatusRecently(bool by_my_privacy_settings_)
  : by_my_privacy_settings_(by_my_privacy_settings_)
{}

const std::int32_t userStatusRecently::ID;

object_ptr<UserStatus> userStatusRecently::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userStatusRecently> res = make_object<userStatusRecently>();
  res->by_my_privacy_settings_ = (env->GetBooleanField(p, res->by_my_privacy_settings_fieldID) != 0);
  return std::move(res);
}

void userStatusRecently::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, by_my_privacy_settings_fieldID, by_my_privacy_settings_);
}

void userStatusRecently::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusRecently");
    s.store_field("byMyPrivacySettings", by_my_privacy_settings_);
    s.store_class_end();
  }
}

void userStatusRecently::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusRecently").c_str());
    by_my_privacy_settings_fieldID = jni::get_field_id(env, Class, "byMyPrivacySettings", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass userStatusLastWeek::Class;
jfieldID userStatusLastWeek::by_my_privacy_settings_fieldID;

userStatusLastWeek::userStatusLastWeek()
  : by_my_privacy_settings_()
{}

userStatusLastWeek::userStatusLastWeek(bool by_my_privacy_settings_)
  : by_my_privacy_settings_(by_my_privacy_settings_)
{}

const std::int32_t userStatusLastWeek::ID;

object_ptr<UserStatus> userStatusLastWeek::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userStatusLastWeek> res = make_object<userStatusLastWeek>();
  res->by_my_privacy_settings_ = (env->GetBooleanField(p, res->by_my_privacy_settings_fieldID) != 0);
  return std::move(res);
}

void userStatusLastWeek::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, by_my_privacy_settings_fieldID, by_my_privacy_settings_);
}

void userStatusLastWeek::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusLastWeek");
    s.store_field("byMyPrivacySettings", by_my_privacy_settings_);
    s.store_class_end();
  }
}

void userStatusLastWeek::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusLastWeek").c_str());
    by_my_privacy_settings_fieldID = jni::get_field_id(env, Class, "byMyPrivacySettings", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass userStatusLastMonth::Class;
jfieldID userStatusLastMonth::by_my_privacy_settings_fieldID;

userStatusLastMonth::userStatusLastMonth()
  : by_my_privacy_settings_()
{}

userStatusLastMonth::userStatusLastMonth(bool by_my_privacy_settings_)
  : by_my_privacy_settings_(by_my_privacy_settings_)
{}

const std::int32_t userStatusLastMonth::ID;

object_ptr<UserStatus> userStatusLastMonth::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userStatusLastMonth> res = make_object<userStatusLastMonth>();
  res->by_my_privacy_settings_ = (env->GetBooleanField(p, res->by_my_privacy_settings_fieldID) != 0);
  return std::move(res);
}

void userStatusLastMonth::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, by_my_privacy_settings_fieldID, by_my_privacy_settings_);
}

void userStatusLastMonth::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserStatusLastMonth");
    s.store_field("byMyPrivacySettings", by_my_privacy_settings_);
    s.store_class_end();
  }
}

void userStatusLastMonth::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserStatusLastMonth").c_str());
    by_my_privacy_settings_fieldID = jni::get_field_id(env, Class, "byMyPrivacySettings", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass userSupportInfo::Class;
jfieldID userSupportInfo::message_fieldID;
jfieldID userSupportInfo::author_fieldID;
jfieldID userSupportInfo::date_fieldID;

userSupportInfo::userSupportInfo()
  : message_()
  , author_()
  , date_()
{}

userSupportInfo::userSupportInfo(object_ptr<formattedText> &&message_, string const &author_, int32 date_)
  : message_(std::move(message_))
  , author_(author_)
  , date_(date_)
{}

const std::int32_t userSupportInfo::ID;

object_ptr<userSupportInfo> userSupportInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userSupportInfo> res = make_object<userSupportInfo>();
  res->message_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->message_fieldID));
  res->author_ = jni::fetch_string(env, p, res->author_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void userSupportInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (message_ != nullptr) { jobject next; message_->store(env, next); if (next) { env->SetObjectField(s, message_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, author_); if (nextString) { env->SetObjectField(s, author_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetIntField(s, date_fieldID, date_);
}

void userSupportInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserSupportInfo");
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_field("author", author_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

void userSupportInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserSupportInfo").c_str());
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    author_fieldID = jni::get_field_id(env, Class, "author", "Ljava/lang/String;");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass UserType::Class;

object_ptr<UserType> UserType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case userTypeRegular::ID:
      return userTypeRegular::fetch(env, p);
    case userTypeDeleted::ID:
      return userTypeDeleted::fetch(env, p);
    case userTypeBot::ID:
      return userTypeBot::fetch(env, p);
    case userTypeUnknown::ID:
      return userTypeUnknown::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void UserType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserType").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userTypeRegular::Class;

userTypeRegular::userTypeRegular() {
}

const std::int32_t userTypeRegular::ID;

object_ptr<UserType> userTypeRegular::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userTypeRegular>();
}

void userTypeRegular::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userTypeRegular::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserTypeRegular");
    s.store_class_end();
  }
}

void userTypeRegular::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserTypeRegular").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userTypeDeleted::Class;

userTypeDeleted::userTypeDeleted() {
}

const std::int32_t userTypeDeleted::ID;

object_ptr<UserType> userTypeDeleted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userTypeDeleted>();
}

void userTypeDeleted::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userTypeDeleted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserTypeDeleted");
    s.store_class_end();
  }
}

void userTypeDeleted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserTypeDeleted").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass userTypeBot::Class;
jfieldID userTypeBot::can_be_edited_fieldID;
jfieldID userTypeBot::can_join_groups_fieldID;
jfieldID userTypeBot::can_read_all_group_messages_fieldID;
jfieldID userTypeBot::has_main_web_app_fieldID;
jfieldID userTypeBot::is_inline_fieldID;
jfieldID userTypeBot::inline_query_placeholder_fieldID;
jfieldID userTypeBot::need_location_fieldID;
jfieldID userTypeBot::can_connect_to_business_fieldID;
jfieldID userTypeBot::can_be_added_to_attachment_menu_fieldID;
jfieldID userTypeBot::active_user_count_fieldID;

userTypeBot::userTypeBot()
  : can_be_edited_()
  , can_join_groups_()
  , can_read_all_group_messages_()
  , has_main_web_app_()
  , is_inline_()
  , inline_query_placeholder_()
  , need_location_()
  , can_connect_to_business_()
  , can_be_added_to_attachment_menu_()
  , active_user_count_()
{}

userTypeBot::userTypeBot(bool can_be_edited_, bool can_join_groups_, bool can_read_all_group_messages_, bool has_main_web_app_, bool is_inline_, string const &inline_query_placeholder_, bool need_location_, bool can_connect_to_business_, bool can_be_added_to_attachment_menu_, int32 active_user_count_)
  : can_be_edited_(can_be_edited_)
  , can_join_groups_(can_join_groups_)
  , can_read_all_group_messages_(can_read_all_group_messages_)
  , has_main_web_app_(has_main_web_app_)
  , is_inline_(is_inline_)
  , inline_query_placeholder_(inline_query_placeholder_)
  , need_location_(need_location_)
  , can_connect_to_business_(can_connect_to_business_)
  , can_be_added_to_attachment_menu_(can_be_added_to_attachment_menu_)
  , active_user_count_(active_user_count_)
{}

const std::int32_t userTypeBot::ID;

object_ptr<UserType> userTypeBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<userTypeBot> res = make_object<userTypeBot>();
  res->can_be_edited_ = (env->GetBooleanField(p, res->can_be_edited_fieldID) != 0);
  res->can_join_groups_ = (env->GetBooleanField(p, res->can_join_groups_fieldID) != 0);
  res->can_read_all_group_messages_ = (env->GetBooleanField(p, res->can_read_all_group_messages_fieldID) != 0);
  res->has_main_web_app_ = (env->GetBooleanField(p, res->has_main_web_app_fieldID) != 0);
  res->is_inline_ = (env->GetBooleanField(p, res->is_inline_fieldID) != 0);
  res->inline_query_placeholder_ = jni::fetch_string(env, p, res->inline_query_placeholder_fieldID);
  res->need_location_ = (env->GetBooleanField(p, res->need_location_fieldID) != 0);
  res->can_connect_to_business_ = (env->GetBooleanField(p, res->can_connect_to_business_fieldID) != 0);
  res->can_be_added_to_attachment_menu_ = (env->GetBooleanField(p, res->can_be_added_to_attachment_menu_fieldID) != 0);
  res->active_user_count_ = env->GetIntField(p, res->active_user_count_fieldID);
  return std::move(res);
}

void userTypeBot::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, can_be_edited_fieldID, can_be_edited_);
  env->SetBooleanField(s, can_join_groups_fieldID, can_join_groups_);
  env->SetBooleanField(s, can_read_all_group_messages_fieldID, can_read_all_group_messages_);
  env->SetBooleanField(s, has_main_web_app_fieldID, has_main_web_app_);
  env->SetBooleanField(s, is_inline_fieldID, is_inline_);
  { jstring nextString = jni::to_jstring(env, inline_query_placeholder_); if (nextString) { env->SetObjectField(s, inline_query_placeholder_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, need_location_fieldID, need_location_);
  env->SetBooleanField(s, can_connect_to_business_fieldID, can_connect_to_business_);
  env->SetBooleanField(s, can_be_added_to_attachment_menu_fieldID, can_be_added_to_attachment_menu_);
  env->SetIntField(s, active_user_count_fieldID, active_user_count_);
}

void userTypeBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserTypeBot");
    s.store_field("canBeEdited", can_be_edited_);
    s.store_field("canJoinGroups", can_join_groups_);
    s.store_field("canReadAllGroupMessages", can_read_all_group_messages_);
    s.store_field("hasMainWebApp", has_main_web_app_);
    s.store_field("isInline", is_inline_);
    s.store_field("inlineQueryPlaceholder", inline_query_placeholder_);
    s.store_field("needLocation", need_location_);
    s.store_field("canConnectToBusiness", can_connect_to_business_);
    s.store_field("canBeAddedToAttachmentMenu", can_be_added_to_attachment_menu_);
    s.store_field("activeUserCount", active_user_count_);
    s.store_class_end();
  }
}

void userTypeBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserTypeBot").c_str());
    can_be_edited_fieldID = jni::get_field_id(env, Class, "canBeEdited", "Z");
    can_join_groups_fieldID = jni::get_field_id(env, Class, "canJoinGroups", "Z");
    can_read_all_group_messages_fieldID = jni::get_field_id(env, Class, "canReadAllGroupMessages", "Z");
    has_main_web_app_fieldID = jni::get_field_id(env, Class, "hasMainWebApp", "Z");
    is_inline_fieldID = jni::get_field_id(env, Class, "isInline", "Z");
    inline_query_placeholder_fieldID = jni::get_field_id(env, Class, "inlineQueryPlaceholder", "Ljava/lang/String;");
    need_location_fieldID = jni::get_field_id(env, Class, "needLocation", "Z");
    can_connect_to_business_fieldID = jni::get_field_id(env, Class, "canConnectToBusiness", "Z");
    can_be_added_to_attachment_menu_fieldID = jni::get_field_id(env, Class, "canBeAddedToAttachmentMenu", "Z");
    active_user_count_fieldID = jni::get_field_id(env, Class, "activeUserCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass userTypeUnknown::Class;

userTypeUnknown::userTypeUnknown() {
}

const std::int32_t userTypeUnknown::ID;

object_ptr<UserType> userTypeUnknown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<userTypeUnknown>();
}

void userTypeUnknown::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void userTypeUnknown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UserTypeUnknown");
    s.store_class_end();
  }
}

void userTypeUnknown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UserTypeUnknown").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass usernames::Class;
jfieldID usernames::active_usernames_fieldID;
jfieldID usernames::disabled_usernames_fieldID;
jfieldID usernames::editable_username_fieldID;

usernames::usernames()
  : active_usernames_()
  , disabled_usernames_()
  , editable_username_()
{}

usernames::usernames(array<string> &&active_usernames_, array<string> &&disabled_usernames_, string const &editable_username_)
  : active_usernames_(std::move(active_usernames_))
  , disabled_usernames_(std::move(disabled_usernames_))
  , editable_username_(editable_username_)
{}

const std::int32_t usernames::ID;

object_ptr<usernames> usernames::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<usernames> res = make_object<usernames>();
  res->active_usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->active_usernames_fieldID));
  res->disabled_usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->disabled_usernames_fieldID));
  res->editable_username_ = jni::fetch_string(env, p, res->editable_username_fieldID);
  return res;
}

void usernames::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, active_usernames_); if (arr_tmp_) { env->SetObjectField(s, active_usernames_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { auto arr_tmp_ = jni::store_vector(env, disabled_usernames_); if (arr_tmp_) { env->SetObjectField(s, disabled_usernames_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  { jstring nextString = jni::to_jstring(env, editable_username_); if (nextString) { env->SetObjectField(s, editable_username_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void usernames::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Usernames");
    { s.store_vector_begin("activeUsernames", active_usernames_.size()); for (const auto &_value : active_usernames_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("disabledUsernames", disabled_usernames_.size()); for (const auto &_value : disabled_usernames_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("editableUsername", editable_username_);
    s.store_class_end();
  }
}

void usernames::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Usernames").c_str());
    active_usernames_fieldID = jni::get_field_id(env, Class, "activeUsernames", "[Ljava/lang/String;");
    disabled_usernames_fieldID = jni::get_field_id(env, Class, "disabledUsernames", "[Ljava/lang/String;");
    editable_username_fieldID = jni::get_field_id(env, Class, "editableUsername", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass users::Class;
jfieldID users::total_count_fieldID;
jfieldID users::user_ids_fieldID;

users::users()
  : total_count_()
  , user_ids_()
{}

users::users(int32 total_count_, array<int53> &&user_ids_)
  : total_count_(total_count_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t users::ID;

object_ptr<users> users::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<users> res = make_object<users>();
  res->total_count_ = env->GetIntField(p, res->total_count_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void users::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, total_count_fieldID, total_count_);
  { auto arr_tmp_ = jni::store_vector(env, user_ids_); if (arr_tmp_) { env->SetObjectField(s, user_ids_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void users::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Users");
    s.store_field("totalCount", total_count_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

void users::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Users").c_str());
    total_count_fieldID = jni::get_field_id(env, Class, "totalCount", "I");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass validatedOrderInfo::Class;
jfieldID validatedOrderInfo::order_info_id_fieldID;
jfieldID validatedOrderInfo::shipping_options_fieldID;

validatedOrderInfo::validatedOrderInfo()
  : order_info_id_()
  , shipping_options_()
{}

validatedOrderInfo::validatedOrderInfo(string const &order_info_id_, array<object_ptr<shippingOption>> &&shipping_options_)
  : order_info_id_(order_info_id_)
  , shipping_options_(std::move(shipping_options_))
{}

const std::int32_t validatedOrderInfo::ID;

object_ptr<validatedOrderInfo> validatedOrderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<validatedOrderInfo> res = make_object<validatedOrderInfo>();
  res->order_info_id_ = jni::fetch_string(env, p, res->order_info_id_fieldID);
  res->shipping_options_ = jni::FetchVector<shippingOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->shipping_options_fieldID));
  return res;
}

void validatedOrderInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, order_info_id_); if (nextString) { env->SetObjectField(s, order_info_id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { auto arr_tmp_ = jni::store_vector(env, shipping_options_); if (arr_tmp_) { env->SetObjectField(s, shipping_options_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void validatedOrderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ValidatedOrderInfo");
    s.store_field("orderInfoId", order_info_id_);
    { s.store_vector_begin("shippingOptions", shipping_options_.size()); for (const auto &_value : shipping_options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void validatedOrderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ValidatedOrderInfo").c_str());
    order_info_id_fieldID = jni::get_field_id(env, Class, "orderInfoId", "Ljava/lang/String;");
    shipping_options_fieldID = jni::get_field_id(env, Class, "shippingOptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ShippingOption;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass VectorPathCommand::Class;

object_ptr<VectorPathCommand> VectorPathCommand::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case vectorPathCommandLine::ID:
      return vectorPathCommandLine::fetch(env, p);
    case vectorPathCommandCubicBezierCurve::ID:
      return vectorPathCommandCubicBezierCurve::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void VectorPathCommand::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VectorPathCommand").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass vectorPathCommandLine::Class;
jfieldID vectorPathCommandLine::end_point_fieldID;

vectorPathCommandLine::vectorPathCommandLine()
  : end_point_()
{}

vectorPathCommandLine::vectorPathCommandLine(object_ptr<point> &&end_point_)
  : end_point_(std::move(end_point_))
{}

const std::int32_t vectorPathCommandLine::ID;

object_ptr<VectorPathCommand> vectorPathCommandLine::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<vectorPathCommandLine> res = make_object<vectorPathCommandLine>();
  res->end_point_ = jni::fetch_tl_object<point>(env, jni::fetch_object(env, p, res->end_point_fieldID));
  return std::move(res);
}

void vectorPathCommandLine::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (end_point_ != nullptr) { jobject next; end_point_->store(env, next); if (next) { env->SetObjectField(s, end_point_fieldID, next); env->DeleteLocalRef(next); } }
}

void vectorPathCommandLine::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VectorPathCommandLine");
    s.store_object_field("endPoint", static_cast<const BaseObject *>(end_point_.get()));
    s.store_class_end();
  }
}

void vectorPathCommandLine::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VectorPathCommandLine").c_str());
    end_point_fieldID = jni::get_field_id(env, Class, "endPoint", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Point;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass vectorPathCommandCubicBezierCurve::Class;
jfieldID vectorPathCommandCubicBezierCurve::start_control_point_fieldID;
jfieldID vectorPathCommandCubicBezierCurve::end_control_point_fieldID;
jfieldID vectorPathCommandCubicBezierCurve::end_point_fieldID;

vectorPathCommandCubicBezierCurve::vectorPathCommandCubicBezierCurve()
  : start_control_point_()
  , end_control_point_()
  , end_point_()
{}

vectorPathCommandCubicBezierCurve::vectorPathCommandCubicBezierCurve(object_ptr<point> &&start_control_point_, object_ptr<point> &&end_control_point_, object_ptr<point> &&end_point_)
  : start_control_point_(std::move(start_control_point_))
  , end_control_point_(std::move(end_control_point_))
  , end_point_(std::move(end_point_))
{}

const std::int32_t vectorPathCommandCubicBezierCurve::ID;

object_ptr<VectorPathCommand> vectorPathCommandCubicBezierCurve::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<vectorPathCommandCubicBezierCurve> res = make_object<vectorPathCommandCubicBezierCurve>();
  res->start_control_point_ = jni::fetch_tl_object<point>(env, jni::fetch_object(env, p, res->start_control_point_fieldID));
  res->end_control_point_ = jni::fetch_tl_object<point>(env, jni::fetch_object(env, p, res->end_control_point_fieldID));
  res->end_point_ = jni::fetch_tl_object<point>(env, jni::fetch_object(env, p, res->end_point_fieldID));
  return std::move(res);
}

void vectorPathCommandCubicBezierCurve::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (start_control_point_ != nullptr) { jobject next; start_control_point_->store(env, next); if (next) { env->SetObjectField(s, start_control_point_fieldID, next); env->DeleteLocalRef(next); } }
  if (end_control_point_ != nullptr) { jobject next; end_control_point_->store(env, next); if (next) { env->SetObjectField(s, end_control_point_fieldID, next); env->DeleteLocalRef(next); } }
  if (end_point_ != nullptr) { jobject next; end_point_->store(env, next); if (next) { env->SetObjectField(s, end_point_fieldID, next); env->DeleteLocalRef(next); } }
}

void vectorPathCommandCubicBezierCurve::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VectorPathCommandCubicBezierCurve");
    s.store_object_field("startControlPoint", static_cast<const BaseObject *>(start_control_point_.get()));
    s.store_object_field("endControlPoint", static_cast<const BaseObject *>(end_control_point_.get()));
    s.store_object_field("endPoint", static_cast<const BaseObject *>(end_point_.get()));
    s.store_class_end();
  }
}

void vectorPathCommandCubicBezierCurve::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VectorPathCommandCubicBezierCurve").c_str());
    start_control_point_fieldID = jni::get_field_id(env, Class, "startControlPoint", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Point;").c_str());
    end_control_point_fieldID = jni::get_field_id(env, Class, "endControlPoint", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Point;").c_str());
    end_point_fieldID = jni::get_field_id(env, Class, "endPoint", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Point;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass venue::Class;
jfieldID venue::location_fieldID;
jfieldID venue::title_fieldID;
jfieldID venue::address_fieldID;
jfieldID venue::provider_fieldID;
jfieldID venue::id_fieldID;
jfieldID venue::type_fieldID;

venue::venue()
  : location_()
  , title_()
  , address_()
  , provider_()
  , id_()
  , type_()
{}

venue::venue(object_ptr<location> &&location_, string const &title_, string const &address_, string const &provider_, string const &id_, string const &type_)
  : location_(std::move(location_))
  , title_(title_)
  , address_(address_)
  , provider_(provider_)
  , id_(id_)
  , type_(type_)
{}

const std::int32_t venue::ID;

object_ptr<venue> venue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<venue> res = make_object<venue>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->address_ = jni::fetch_string(env, p, res->address_fieldID);
  res->provider_ = jni::fetch_string(env, p, res->provider_fieldID);
  res->id_ = jni::fetch_string(env, p, res->id_fieldID);
  res->type_ = jni::fetch_string(env, p, res->type_fieldID);
  return res;
}

void venue::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  if (location_ != nullptr) { jobject next; location_->store(env, next); if (next) { env->SetObjectField(s, location_fieldID, next); env->DeleteLocalRef(next); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, address_); if (nextString) { env->SetObjectField(s, address_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, provider_); if (nextString) { env->SetObjectField(s, provider_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, id_); if (nextString) { env->SetObjectField(s, id_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, type_); if (nextString) { env->SetObjectField(s, type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void venue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Venue");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("title", title_);
    s.store_field("address", address_);
    s.store_field("provider", provider_);
    s.store_field("id", id_);
    s.store_field("type", type_);
    s.store_class_end();
  }
}

void venue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Venue").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    address_fieldID = jni::get_field_id(env, Class, "address", "Ljava/lang/String;");
    provider_fieldID = jni::get_field_id(env, Class, "provider", "Ljava/lang/String;");
    id_fieldID = jni::get_field_id(env, Class, "id", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass verificationStatus::Class;
jfieldID verificationStatus::is_verified_fieldID;
jfieldID verificationStatus::is_scam_fieldID;
jfieldID verificationStatus::is_fake_fieldID;
jfieldID verificationStatus::bot_verification_icon_custom_emoji_id_fieldID;

verificationStatus::verificationStatus()
  : is_verified_()
  , is_scam_()
  , is_fake_()
  , bot_verification_icon_custom_emoji_id_()
{}

verificationStatus::verificationStatus(bool is_verified_, bool is_scam_, bool is_fake_, int64 bot_verification_icon_custom_emoji_id_)
  : is_verified_(is_verified_)
  , is_scam_(is_scam_)
  , is_fake_(is_fake_)
  , bot_verification_icon_custom_emoji_id_(bot_verification_icon_custom_emoji_id_)
{}

const std::int32_t verificationStatus::ID;

object_ptr<verificationStatus> verificationStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<verificationStatus> res = make_object<verificationStatus>();
  res->is_verified_ = (env->GetBooleanField(p, res->is_verified_fieldID) != 0);
  res->is_scam_ = (env->GetBooleanField(p, res->is_scam_fieldID) != 0);
  res->is_fake_ = (env->GetBooleanField(p, res->is_fake_fieldID) != 0);
  res->bot_verification_icon_custom_emoji_id_ = env->GetLongField(p, res->bot_verification_icon_custom_emoji_id_fieldID);
  return res;
}

void verificationStatus::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetBooleanField(s, is_verified_fieldID, is_verified_);
  env->SetBooleanField(s, is_scam_fieldID, is_scam_);
  env->SetBooleanField(s, is_fake_fieldID, is_fake_);
  env->SetLongField(s, bot_verification_icon_custom_emoji_id_fieldID, bot_verification_icon_custom_emoji_id_);
}

void verificationStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VerificationStatus");
    s.store_field("isVerified", is_verified_);
    s.store_field("isScam", is_scam_);
    s.store_field("isFake", is_fake_);
    s.store_field("botVerificationIconCustomEmojiId", bot_verification_icon_custom_emoji_id_);
    s.store_class_end();
  }
}

void verificationStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VerificationStatus").c_str());
    is_verified_fieldID = jni::get_field_id(env, Class, "isVerified", "Z");
    is_scam_fieldID = jni::get_field_id(env, Class, "isScam", "Z");
    is_fake_fieldID = jni::get_field_id(env, Class, "isFake", "Z");
    bot_verification_icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "botVerificationIconCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass video::Class;
jfieldID video::duration_fieldID;
jfieldID video::width_fieldID;
jfieldID video::height_fieldID;
jfieldID video::file_name_fieldID;
jfieldID video::mime_type_fieldID;
jfieldID video::has_stickers_fieldID;
jfieldID video::supports_streaming_fieldID;
jfieldID video::minithumbnail_fieldID;
jfieldID video::thumbnail_fieldID;
jfieldID video::video_fieldID;

video::video()
  : duration_()
  , width_()
  , height_()
  , file_name_()
  , mime_type_()
  , has_stickers_()
  , supports_streaming_()
  , minithumbnail_()
  , thumbnail_()
  , video_()
{}

video::video(int32 duration_, int32 width_, int32 height_, string const &file_name_, string const &mime_type_, bool has_stickers_, bool supports_streaming_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&video_)
  : duration_(duration_)
  , width_(width_)
  , height_(height_)
  , file_name_(file_name_)
  , mime_type_(mime_type_)
  , has_stickers_(has_stickers_)
  , supports_streaming_(supports_streaming_)
  , minithumbnail_(std::move(minithumbnail_))
  , thumbnail_(std::move(thumbnail_))
  , video_(std::move(video_))
{}

const std::int32_t video::ID;

object_ptr<video> video::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<video> res = make_object<video>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->has_stickers_ = (env->GetBooleanField(p, res->has_stickers_fieldID) != 0);
  res->supports_streaming_ = (env->GetBooleanField(p, res->supports_streaming_fieldID) != 0);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->video_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->video_fieldID));
  return res;
}

void video::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
  env->SetIntField(s, width_fieldID, width_);
  env->SetIntField(s, height_fieldID, height_);
  { jstring nextString = jni::to_jstring(env, file_name_); if (nextString) { env->SetObjectField(s, file_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  env->SetBooleanField(s, has_stickers_fieldID, has_stickers_);
  env->SetBooleanField(s, supports_streaming_fieldID, supports_streaming_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
}

void video::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Video");
    s.store_field("duration", duration_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("fileName", file_name_);
    s.store_field("mimeType", mime_type_);
    s.store_field("hasStickers", has_stickers_);
    s.store_field("supportsStreaming", supports_streaming_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_class_end();
  }
}

void video::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Video").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    has_stickers_fieldID = jni::get_field_id(env, Class, "hasStickers", "Z");
    supports_streaming_fieldID = jni::get_field_id(env, Class, "supportsStreaming", "Z");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass videoChat::Class;
jfieldID videoChat::group_call_id_fieldID;
jfieldID videoChat::has_participants_fieldID;
jfieldID videoChat::default_participant_id_fieldID;

videoChat::videoChat()
  : group_call_id_()
  , has_participants_()
  , default_participant_id_()
{}

videoChat::videoChat(int32 group_call_id_, bool has_participants_, object_ptr<MessageSender> &&default_participant_id_)
  : group_call_id_(group_call_id_)
  , has_participants_(has_participants_)
  , default_participant_id_(std::move(default_participant_id_))
{}

const std::int32_t videoChat::ID;

object_ptr<videoChat> videoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<videoChat> res = make_object<videoChat>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->has_participants_ = (env->GetBooleanField(p, res->has_participants_fieldID) != 0);
  res->default_participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->default_participant_id_fieldID));
  return res;
}

void videoChat::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, group_call_id_fieldID, group_call_id_);
  env->SetBooleanField(s, has_participants_fieldID, has_participants_);
  if (default_participant_id_ != nullptr) { jobject next; default_participant_id_->store(env, next); if (next) { env->SetObjectField(s, default_participant_id_fieldID, next); env->DeleteLocalRef(next); } }
}

void videoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VideoChat");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("hasParticipants", has_participants_);
    s.store_object_field("defaultParticipantId", static_cast<const BaseObject *>(default_participant_id_.get()));
    s.store_class_end();
  }
}

void videoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VideoChat").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    has_participants_fieldID = jni::get_field_id(env, Class, "hasParticipants", "Z");
    default_participant_id_fieldID = jni::get_field_id(env, Class, "defaultParticipantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass videoChatStream::Class;
jfieldID videoChatStream::channel_id_fieldID;
jfieldID videoChatStream::scale_fieldID;
jfieldID videoChatStream::time_offset_fieldID;

videoChatStream::videoChatStream()
  : channel_id_()
  , scale_()
  , time_offset_()
{}

videoChatStream::videoChatStream(int32 channel_id_, int32 scale_, int53 time_offset_)
  : channel_id_(channel_id_)
  , scale_(scale_)
  , time_offset_(time_offset_)
{}

const std::int32_t videoChatStream::ID;

object_ptr<videoChatStream> videoChatStream::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<videoChatStream> res = make_object<videoChatStream>();
  res->channel_id_ = env->GetIntField(p, res->channel_id_fieldID);
  res->scale_ = env->GetIntField(p, res->scale_fieldID);
  res->time_offset_ = env->GetLongField(p, res->time_offset_fieldID);
  return res;
}

void videoChatStream::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, channel_id_fieldID, channel_id_);
  env->SetIntField(s, scale_fieldID, scale_);
  env->SetLongField(s, time_offset_fieldID, time_offset_);
}

void videoChatStream::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VideoChatStream");
    s.store_field("channelId", channel_id_);
    s.store_field("scale", scale_);
    s.store_field("timeOffset", time_offset_);
    s.store_class_end();
  }
}

void videoChatStream::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VideoChatStream").c_str());
    channel_id_fieldID = jni::get_field_id(env, Class, "channelId", "I");
    scale_fieldID = jni::get_field_id(env, Class, "scale", "I");
    time_offset_fieldID = jni::get_field_id(env, Class, "timeOffset", "J");
    return true;
  }();
  (void)is_inited;
}

jclass videoChatStreams::Class;
jfieldID videoChatStreams::streams_fieldID;

videoChatStreams::videoChatStreams()
  : streams_()
{}

videoChatStreams::videoChatStreams(array<object_ptr<videoChatStream>> &&streams_)
  : streams_(std::move(streams_))
{}

const std::int32_t videoChatStreams::ID;

object_ptr<videoChatStreams> videoChatStreams::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<videoChatStreams> res = make_object<videoChatStreams>();
  res->streams_ = jni::FetchVector<videoChatStream>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->streams_fieldID));
  return res;
}

void videoChatStreams::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, streams_); if (arr_tmp_) { env->SetObjectField(s, streams_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
}

void videoChatStreams::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VideoChatStreams");
    { s.store_vector_begin("streams", streams_.size()); for (const auto &_value : streams_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

void videoChatStreams::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VideoChatStreams").c_str());
    streams_fieldID = jni::get_field_id(env, Class, "streams", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$VideoChatStream;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass videoNote::Class;
jfieldID videoNote::duration_fieldID;
jfieldID videoNote::waveform_fieldID;
jfieldID videoNote::length_fieldID;
jfieldID videoNote::minithumbnail_fieldID;
jfieldID videoNote::thumbnail_fieldID;
jfieldID videoNote::speech_recognition_result_fieldID;
jfieldID videoNote::video_fieldID;

videoNote::videoNote()
  : duration_()
  , waveform_()
  , length_()
  , minithumbnail_()
  , thumbnail_()
  , speech_recognition_result_()
  , video_()
{}

videoNote::videoNote(int32 duration_, bytes const &waveform_, int32 length_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<SpeechRecognitionResult> &&speech_recognition_result_, object_ptr<file> &&video_)
  : duration_(duration_)
  , waveform_(std::move(waveform_))
  , length_(length_)
  , minithumbnail_(std::move(minithumbnail_))
  , thumbnail_(std::move(thumbnail_))
  , speech_recognition_result_(std::move(speech_recognition_result_))
  , video_(std::move(video_))
{}

const std::int32_t videoNote::ID;

object_ptr<videoNote> videoNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<videoNote> res = make_object<videoNote>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->waveform_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->waveform_fieldID));
  res->length_ = env->GetIntField(p, res->length_fieldID);
  res->minithumbnail_ = jni::fetch_tl_object<minithumbnail>(env, jni::fetch_object(env, p, res->minithumbnail_fieldID));
  res->thumbnail_ = jni::fetch_tl_object<thumbnail>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->speech_recognition_result_ = jni::fetch_tl_object<SpeechRecognitionResult>(env, jni::fetch_object(env, p, res->speech_recognition_result_fieldID));
  res->video_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->video_fieldID));
  return res;
}

void videoNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
  { jbyteArray nextBytes = jni::to_bytes(env, waveform_); if (nextBytes) { env->SetObjectField(s, waveform_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  env->SetIntField(s, length_fieldID, length_);
  if (minithumbnail_ != nullptr) { jobject next; minithumbnail_->store(env, next); if (next) { env->SetObjectField(s, minithumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (thumbnail_ != nullptr) { jobject next; thumbnail_->store(env, next); if (next) { env->SetObjectField(s, thumbnail_fieldID, next); env->DeleteLocalRef(next); } }
  if (speech_recognition_result_ != nullptr) { jobject next; speech_recognition_result_->store(env, next); if (next) { env->SetObjectField(s, speech_recognition_result_fieldID, next); env->DeleteLocalRef(next); } }
  if (video_ != nullptr) { jobject next; video_->store(env, next); if (next) { env->SetObjectField(s, video_fieldID, next); env->DeleteLocalRef(next); } }
}

void videoNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VideoNote");
    s.store_field("duration", duration_);
    s.store_bytes_field("waveform", waveform_);
    s.store_field("length", length_);
    s.store_object_field("minithumbnail", static_cast<const BaseObject *>(minithumbnail_.get()));
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("speechRecognitionResult", static_cast<const BaseObject *>(speech_recognition_result_.get()));
    s.store_object_field("video", static_cast<const BaseObject *>(video_.get()));
    s.store_class_end();
  }
}

void videoNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VideoNote").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    waveform_fieldID = jni::get_field_id(env, Class, "waveform", "[B");
    length_fieldID = jni::get_field_id(env, Class, "length", "I");
    minithumbnail_fieldID = jni::get_field_id(env, Class, "minithumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Minithumbnail;").c_str());
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Thumbnail;").c_str());
    speech_recognition_result_fieldID = jni::get_field_id(env, Class, "speechRecognitionResult", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SpeechRecognitionResult;").c_str());
    video_fieldID = jni::get_field_id(env, Class, "video", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass voiceNote::Class;
jfieldID voiceNote::duration_fieldID;
jfieldID voiceNote::waveform_fieldID;
jfieldID voiceNote::mime_type_fieldID;
jfieldID voiceNote::speech_recognition_result_fieldID;
jfieldID voiceNote::voice_fieldID;

voiceNote::voiceNote()
  : duration_()
  , waveform_()
  , mime_type_()
  , speech_recognition_result_()
  , voice_()
{}

voiceNote::voiceNote(int32 duration_, bytes const &waveform_, string const &mime_type_, object_ptr<SpeechRecognitionResult> &&speech_recognition_result_, object_ptr<file> &&voice_)
  : duration_(duration_)
  , waveform_(std::move(waveform_))
  , mime_type_(mime_type_)
  , speech_recognition_result_(std::move(speech_recognition_result_))
  , voice_(std::move(voice_))
{}

const std::int32_t voiceNote::ID;

object_ptr<voiceNote> voiceNote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<voiceNote> res = make_object<voiceNote>();
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->waveform_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->waveform_fieldID));
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  res->speech_recognition_result_ = jni::fetch_tl_object<SpeechRecognitionResult>(env, jni::fetch_object(env, p, res->speech_recognition_result_fieldID));
  res->voice_ = jni::fetch_tl_object<file>(env, jni::fetch_object(env, p, res->voice_fieldID));
  return res;
}

void voiceNote::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetIntField(s, duration_fieldID, duration_);
  { jbyteArray nextBytes = jni::to_bytes(env, waveform_); if (nextBytes) { env->SetObjectField(s, waveform_fieldID, nextBytes); env->DeleteLocalRef(nextBytes); } }
  { jstring nextString = jni::to_jstring(env, mime_type_); if (nextString) { env->SetObjectField(s, mime_type_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (speech_recognition_result_ != nullptr) { jobject next; speech_recognition_result_->store(env, next); if (next) { env->SetObjectField(s, speech_recognition_result_fieldID, next); env->DeleteLocalRef(next); } }
  if (voice_ != nullptr) { jobject next; voice_->store(env, next); if (next) { env->SetObjectField(s, voice_fieldID, next); env->DeleteLocalRef(next); } }
}

void voiceNote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "VoiceNote");
    s.store_field("duration", duration_);
    s.store_bytes_field("waveform", waveform_);
    s.store_field("mimeType", mime_type_);
    s.store_object_field("speechRecognitionResult", static_cast<const BaseObject *>(speech_recognition_result_.get()));
    s.store_object_field("voice", static_cast<const BaseObject *>(voice_.get()));
    s.store_class_end();
  }
}

void voiceNote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$VoiceNote").c_str());
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    waveform_fieldID = jni::get_field_id(env, Class, "waveform", "[B");
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    speech_recognition_result_fieldID = jni::get_field_id(env, Class, "speechRecognitionResult", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SpeechRecognitionResult;").c_str());
    voice_fieldID = jni::get_field_id(env, Class, "voice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$File;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webApp::Class;
jfieldID webApp::short_name_fieldID;
jfieldID webApp::title_fieldID;
jfieldID webApp::description_fieldID;
jfieldID webApp::photo_fieldID;
jfieldID webApp::animation_fieldID;

webApp::webApp()
  : short_name_()
  , title_()
  , description_()
  , photo_()
  , animation_()
{}

webApp::webApp(string const &short_name_, string const &title_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_)
  : short_name_(short_name_)
  , title_(title_)
  , description_(description_)
  , photo_(std::move(photo_))
  , animation_(std::move(animation_))
{}

const std::int32_t webApp::ID;

object_ptr<webApp> webApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<webApp> res = make_object<webApp>();
  res->short_name_ = jni::fetch_string(env, p, res->short_name_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->photo_ = jni::fetch_tl_object<photo>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->animation_ = jni::fetch_tl_object<animation>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void webApp::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { jstring nextString = jni::to_jstring(env, short_name_); if (nextString) { env->SetObjectField(s, short_name_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, title_); if (nextString) { env->SetObjectField(s, title_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  { jstring nextString = jni::to_jstring(env, description_); if (nextString) { env->SetObjectField(s, description_fieldID, nextString); env->DeleteLocalRef(nextString); } }
  if (photo_ != nullptr) { jobject next; photo_->store(env, next); if (next) { env->SetObjectField(s, photo_fieldID, next); env->DeleteLocalRef(next); } }
  if (animation_ != nullptr) { jobject next; animation_->store(env, next); if (next) { env->SetObjectField(s, animation_fieldID, next); env->DeleteLocalRef(next); } }
}

void webApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebApp");
    s.store_field("shortName", short_name_);
    s.store_field("title", title_);
    s.store_field("description", description_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

void webApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebApp").c_str());
    short_name_fieldID = jni::get_field_id(env, Class, "shortName", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Photo;").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Animation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webAppInfo::Class;
jfieldID webAppInfo::launch_id_fieldID;
jfieldID webAppInfo::url_fieldID;

webAppInfo::webAppInfo()
  : launch_id_()
  , url_()
{}

webAppInfo::webAppInfo(int64 launch_id_, string const &url_)
  : launch_id_(launch_id_)
  , url_(url_)
{}

const std::int32_t webAppInfo::ID;

object_ptr<webAppInfo> webAppInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<webAppInfo> res = make_object<webAppInfo>();
  res->launch_id_ = env->GetLongField(p, res->launch_id_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void webAppInfo::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  env->SetLongField(s, launch_id_fieldID, launch_id_);
  { jstring nextString = jni::to_jstring(env, url_); if (nextString) { env->SetObjectField(s, url_fieldID, nextString); env->DeleteLocalRef(nextString); } }
}

void webAppInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebAppInfo");
    s.store_field("launchId", launch_id_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

void webAppInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppInfo").c_str());
    launch_id_fieldID = jni::get_field_id(env, Class, "launchId", "J");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass WebAppOpenMode::Class;

object_ptr<WebAppOpenMode> WebAppOpenMode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) { return nullptr; }
  jint constructor = env->CallIntMethod(p, jni::GetConstructorID);  switch (constructor) {
    case webAppOpenModeCompact::ID:
      return webAppOpenModeCompact::fetch(env, p);
    case webAppOpenModeFullSize::ID:
      return webAppOpenModeFullSize::fetch(env, p);
    case webAppOpenModeFullScreen::ID:
      return webAppOpenModeFullScreen::fetch(env, p);
    default:
      LOG(WARNING) << "Unknown Java API constructor found " << format::as_hex(constructor);
      return nullptr;
  }
}

void WebAppOpenMode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppOpenMode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webAppOpenModeCompact::Class;

webAppOpenModeCompact::webAppOpenModeCompact() {
}

const std::int32_t webAppOpenModeCompact::ID;

object_ptr<WebAppOpenMode> webAppOpenModeCompact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<webAppOpenModeCompact>();
}

void webAppOpenModeCompact::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void webAppOpenModeCompact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebAppOpenModeCompact");
    s.store_class_end();
  }
}

void webAppOpenModeCompact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppOpenModeCompact").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webAppOpenModeFullSize::Class;

webAppOpenModeFullSize::webAppOpenModeFullSize() {
}

const std::int32_t webAppOpenModeFullSize::ID;

object_ptr<WebAppOpenMode> webAppOpenModeFullSize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<webAppOpenModeFullSize>();
}

void webAppOpenModeFullSize::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void webAppOpenModeFullSize::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebAppOpenModeFullSize");
    s.store_class_end();
  }
}

void webAppOpenModeFullSize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppOpenModeFullSize").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webAppOpenModeFullScreen::Class;

webAppOpenModeFullScreen::webAppOpenModeFullScreen() {
}

const std::int32_t webAppOpenModeFullScreen::ID;

object_ptr<WebAppOpenMode> webAppOpenModeFullScreen::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<webAppOpenModeFullScreen>();
}

void webAppOpenModeFullScreen::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
}

void webAppOpenModeFullScreen::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebAppOpenModeFullScreen");
    s.store_class_end();
  }
}

void webAppOpenModeFullScreen::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppOpenModeFullScreen").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webAppOpenParameters::Class;
jfieldID webAppOpenParameters::theme_fieldID;
jfieldID webAppOpenParameters::application_name_fieldID;
jfieldID webAppOpenParameters::mode_fieldID;

webAppOpenParameters::webAppOpenParameters()
  : theme_()
  , application_name_()
  , mode_()
{}

webAppOpenParameters::webAppOpenParameters(object_ptr<themeParameters> &&theme_, string const &application_name_, object_ptr<WebAppOpenMode> &&mode_)
  : theme_(std::move(theme_))
  , application_name_(application_name_)
  , mode_(std::move(mode_))
{}

const std::int32_t webAppOpenParameters::ID;

object_ptr<webAppOpenParameters> webAppOpenParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<webAppOpenParameters> res = make_object<webAppOpenParameters>();
  res->theme_ = jni::fetch_tl_object<themeParameters>(env, jni::fetch_object(env, p, res->theme_fieldID));
  res->application_name_ = jni::fetch_string(env, p, res->application_name_fieldID);
  res->mode_ = jni::fetch_tl_object<WebAppOpenMode>(env, jni::fetch_object(env, p, res->mode_fieldID));
  return res;
}

void webAppOpenParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebAppOpenParameters");
    s.store_object_field("theme", static_cast<const BaseObject *>(theme_.get()));
    s.store_field("applicationName", application_name_);
    s.store_object_field("mode", static_cast<const BaseObject *>(mode_.get()));
    s.store_class_end();
  }
}

void webAppOpenParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebAppOpenParameters").c_str());
    theme_fieldID = jni::get_field_id(env, Class, "theme", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeParameters;").c_str());
    application_name_fieldID = jni::get_field_id(env, Class, "applicationName", "Ljava/lang/String;");
    mode_fieldID = jni::get_field_id(env, Class, "mode", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenMode;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass webPageInstantView::Class;
jfieldID webPageInstantView::page_blocks_fieldID;
jfieldID webPageInstantView::view_count_fieldID;
jfieldID webPageInstantView::version_fieldID;
jfieldID webPageInstantView::is_rtl_fieldID;
jfieldID webPageInstantView::is_full_fieldID;
jfieldID webPageInstantView::feedback_link_fieldID;

webPageInstantView::webPageInstantView()
  : page_blocks_()
  , view_count_()
  , version_()
  , is_rtl_()
  , is_full_()
  , feedback_link_()
{}

webPageInstantView::webPageInstantView(array<object_ptr<PageBlock>> &&page_blocks_, int32 view_count_, int32 version_, bool is_rtl_, bool is_full_, object_ptr<InternalLinkType> &&feedback_link_)
  : page_blocks_(std::move(page_blocks_))
  , view_count_(view_count_)
  , version_(version_)
  , is_rtl_(is_rtl_)
  , is_full_(is_full_)
  , feedback_link_(std::move(feedback_link_))
{}

const std::int32_t webPageInstantView::ID;

object_ptr<webPageInstantView> webPageInstantView::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<webPageInstantView> res = make_object<webPageInstantView>();
  res->page_blocks_ = jni::FetchVector<PageBlock>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->page_blocks_fieldID));
  res->view_count_ = env->GetIntField(p, res->view_count_fieldID);
  res->version_ = env->GetIntField(p, res->version_fieldID);
  res->is_rtl_ = (env->GetBooleanField(p, res->is_rtl_fieldID) != 0);
  res->is_full_ = (env->GetBooleanField(p, res->is_full_fieldID) != 0);
  res->feedback_link_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->feedback_link_fieldID));
  return res;
}

void webPageInstantView::store(JNIEnv *env, jobject &s) const {
  init_jni_vars(env);
  s = env->AllocObject(Class);
  if (!s) { return; }
  { auto arr_tmp_ = jni::store_vector(env, page_blocks_); if (arr_tmp_) { env->SetObjectField(s, page_blocks_fieldID, arr_tmp_); env->DeleteLocalRef(arr_tmp_); } }
  env->SetIntField(s, view_count_fieldID, view_count_);
  env->SetIntField(s, version_fieldID, version_);
  env->SetBooleanField(s, is_rtl_fieldID, is_rtl_);
  env->SetBooleanField(s, is_full_fieldID, is_full_);
  if (feedback_link_ != nullptr) { jobject next; feedback_link_->store(env, next); if (next) { env->SetObjectField(s, feedback_link_fieldID, next); env->DeleteLocalRef(next); } }
}

void webPageInstantView::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WebPageInstantView");
    { s.store_vector_begin("pageBlocks", page_blocks_.size()); for (const auto &_value : page_blocks_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("viewCount", view_count_);
    s.store_field("version", version_);
    s.store_field("isRtl", is_rtl_);
    s.store_field("isFull", is_full_);
    s.store_object_field("feedbackLink", static_cast<const BaseObject *>(feedback_link_.get()));
    s.store_class_end();
  }
}

void webPageInstantView::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WebPageInstantView").c_str());
    page_blocks_fieldID = jni::get_field_id(env, Class, "pageBlocks", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PageBlock;").c_str());
    view_count_fieldID = jni::get_field_id(env, Class, "viewCount", "I");
    version_fieldID = jni::get_field_id(env, Class, "version", "I");
    is_rtl_fieldID = jni::get_field_id(env, Class, "isRtl", "Z");
    is_full_fieldID = jni::get_field_id(env, Class, "isFull", "Z");
    feedback_link_fieldID = jni::get_field_id(env, Class, "feedbackLink", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass acceptCall::Class;
jfieldID acceptCall::call_id_fieldID;
jfieldID acceptCall::protocol_fieldID;

acceptCall::acceptCall()
  : call_id_()
  , protocol_()
{}

acceptCall::acceptCall(int32 call_id_, object_ptr<callProtocol> &&protocol_)
  : call_id_(call_id_)
  , protocol_(std::move(protocol_))
{}

const std::int32_t acceptCall::ID;

object_ptr<acceptCall> acceptCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<acceptCall> res = make_object<acceptCall>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->protocol_ = jni::fetch_tl_object<callProtocol>(env, jni::fetch_object(env, p, res->protocol_fieldID));
  return res;
}

void acceptCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AcceptCall");
    s.store_field("callId", call_id_);
    s.store_object_field("protocol", static_cast<const BaseObject *>(protocol_.get()));
    s.store_class_end();
  }
}

acceptCall::ReturnType acceptCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void acceptCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AcceptCall").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    protocol_fieldID = jni::get_field_id(env, Class, "protocol", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallProtocol;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass acceptTermsOfService::Class;
jfieldID acceptTermsOfService::terms_of_service_id_fieldID;

acceptTermsOfService::acceptTermsOfService()
  : terms_of_service_id_()
{}

acceptTermsOfService::acceptTermsOfService(string const &terms_of_service_id_)
  : terms_of_service_id_(terms_of_service_id_)
{}

const std::int32_t acceptTermsOfService::ID;

object_ptr<acceptTermsOfService> acceptTermsOfService::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<acceptTermsOfService> res = make_object<acceptTermsOfService>();
  res->terms_of_service_id_ = jni::fetch_string(env, p, res->terms_of_service_id_fieldID);
  return res;
}

void acceptTermsOfService::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AcceptTermsOfService");
    s.store_field("termsOfServiceId", terms_of_service_id_);
    s.store_class_end();
  }
}

acceptTermsOfService::ReturnType acceptTermsOfService::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void acceptTermsOfService::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AcceptTermsOfService").c_str());
    terms_of_service_id_fieldID = jni::get_field_id(env, Class, "termsOfServiceId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass activateStoryStealthMode::Class;

activateStoryStealthMode::activateStoryStealthMode() {
}

const std::int32_t activateStoryStealthMode::ID;

object_ptr<activateStoryStealthMode> activateStoryStealthMode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<activateStoryStealthMode>();
}

void activateStoryStealthMode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ActivateStoryStealthMode");
    s.store_class_end();
  }
}

activateStoryStealthMode::ReturnType activateStoryStealthMode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void activateStoryStealthMode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ActivateStoryStealthMode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addBotMediaPreview::Class;
jfieldID addBotMediaPreview::bot_user_id_fieldID;
jfieldID addBotMediaPreview::language_code_fieldID;
jfieldID addBotMediaPreview::content_fieldID;

addBotMediaPreview::addBotMediaPreview()
  : bot_user_id_()
  , language_code_()
  , content_()
{}

addBotMediaPreview::addBotMediaPreview(int53 bot_user_id_, string const &language_code_, object_ptr<InputStoryContent> &&content_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , content_(std::move(content_))
{}

const std::int32_t addBotMediaPreview::ID;

object_ptr<addBotMediaPreview> addBotMediaPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addBotMediaPreview> res = make_object<addBotMediaPreview>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->content_ = jni::fetch_tl_object<InputStoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return res;
}

void addBotMediaPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddBotMediaPreview");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

addBotMediaPreview::ReturnType addBotMediaPreview::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botMediaPreview::fetch(env, p);
}

void addBotMediaPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddBotMediaPreview").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addChatFolderByInviteLink::Class;
jfieldID addChatFolderByInviteLink::invite_link_fieldID;
jfieldID addChatFolderByInviteLink::chat_ids_fieldID;

addChatFolderByInviteLink::addChatFolderByInviteLink()
  : invite_link_()
  , chat_ids_()
{}

addChatFolderByInviteLink::addChatFolderByInviteLink(string const &invite_link_, array<int53> &&chat_ids_)
  : invite_link_(invite_link_)
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t addChatFolderByInviteLink::ID;

object_ptr<addChatFolderByInviteLink> addChatFolderByInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addChatFolderByInviteLink> res = make_object<addChatFolderByInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void addChatFolderByInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddChatFolderByInviteLink");
    s.store_field("inviteLink", invite_link_);
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

addChatFolderByInviteLink::ReturnType addChatFolderByInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addChatFolderByInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddChatFolderByInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass addChatMember::Class;
jfieldID addChatMember::chat_id_fieldID;
jfieldID addChatMember::user_id_fieldID;
jfieldID addChatMember::forward_limit_fieldID;

addChatMember::addChatMember()
  : chat_id_()
  , user_id_()
  , forward_limit_()
{}

addChatMember::addChatMember(int53 chat_id_, int53 user_id_, int32 forward_limit_)
  : chat_id_(chat_id_)
  , user_id_(user_id_)
  , forward_limit_(forward_limit_)
{}

const std::int32_t addChatMember::ID;

object_ptr<addChatMember> addChatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addChatMember> res = make_object<addChatMember>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->forward_limit_ = env->GetIntField(p, res->forward_limit_fieldID);
  return res;
}

void addChatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddChatMember");
    s.store_field("chatId", chat_id_);
    s.store_field("userId", user_id_);
    s.store_field("forwardLimit", forward_limit_);
    s.store_class_end();
  }
}

addChatMember::ReturnType addChatMember::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return failedToAddMembers::fetch(env, p);
}

void addChatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddChatMember").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    forward_limit_fieldID = jni::get_field_id(env, Class, "forwardLimit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass addChatMembers::Class;
jfieldID addChatMembers::chat_id_fieldID;
jfieldID addChatMembers::user_ids_fieldID;

addChatMembers::addChatMembers()
  : chat_id_()
  , user_ids_()
{}

addChatMembers::addChatMembers(int53 chat_id_, array<int53> &&user_ids_)
  : chat_id_(chat_id_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t addChatMembers::ID;

object_ptr<addChatMembers> addChatMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addChatMembers> res = make_object<addChatMembers>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void addChatMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddChatMembers");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

addChatMembers::ReturnType addChatMembers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return failedToAddMembers::fetch(env, p);
}

void addChatMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddChatMembers").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass addChatToList::Class;
jfieldID addChatToList::chat_id_fieldID;
jfieldID addChatToList::chat_list_fieldID;

addChatToList::addChatToList()
  : chat_id_()
  , chat_list_()
{}

addChatToList::addChatToList(int53 chat_id_, object_ptr<ChatList> &&chat_list_)
  : chat_id_(chat_id_)
  , chat_list_(std::move(chat_list_))
{}

const std::int32_t addChatToList::ID;

object_ptr<addChatToList> addChatToList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addChatToList> res = make_object<addChatToList>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  return res;
}

void addChatToList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddChatToList");
    s.store_field("chatId", chat_id_);
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_class_end();
  }
}

addChatToList::ReturnType addChatToList::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addChatToList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddChatToList").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addContact::Class;
jfieldID addContact::contact_fieldID;
jfieldID addContact::share_phone_number_fieldID;

addContact::addContact()
  : contact_()
  , share_phone_number_()
{}

addContact::addContact(object_ptr<contact> &&contact_, bool share_phone_number_)
  : contact_(std::move(contact_))
  , share_phone_number_(share_phone_number_)
{}

const std::int32_t addContact::ID;

object_ptr<addContact> addContact::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addContact> res = make_object<addContact>();
  res->contact_ = jni::fetch_tl_object<contact>(env, jni::fetch_object(env, p, res->contact_fieldID));
  res->share_phone_number_ = (env->GetBooleanField(p, res->share_phone_number_fieldID) != 0);
  return res;
}

void addContact::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddContact");
    s.store_object_field("contact", static_cast<const BaseObject *>(contact_.get()));
    s.store_field("sharePhoneNumber", share_phone_number_);
    s.store_class_end();
  }
}

addContact::ReturnType addContact::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addContact::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddContact").c_str());
    contact_fieldID = jni::get_field_id(env, Class, "contact", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    share_phone_number_fieldID = jni::get_field_id(env, Class, "sharePhoneNumber", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass addCustomServerLanguagePack::Class;
jfieldID addCustomServerLanguagePack::language_pack_id_fieldID;

addCustomServerLanguagePack::addCustomServerLanguagePack()
  : language_pack_id_()
{}

addCustomServerLanguagePack::addCustomServerLanguagePack(string const &language_pack_id_)
  : language_pack_id_(language_pack_id_)
{}

const std::int32_t addCustomServerLanguagePack::ID;

object_ptr<addCustomServerLanguagePack> addCustomServerLanguagePack::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addCustomServerLanguagePack> res = make_object<addCustomServerLanguagePack>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  return res;
}

void addCustomServerLanguagePack::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddCustomServerLanguagePack");
    s.store_field("languagePackId", language_pack_id_);
    s.store_class_end();
  }
}

addCustomServerLanguagePack::ReturnType addCustomServerLanguagePack::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addCustomServerLanguagePack::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddCustomServerLanguagePack").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass addFavoriteSticker::Class;
jfieldID addFavoriteSticker::sticker_fieldID;

addFavoriteSticker::addFavoriteSticker()
  : sticker_()
{}

addFavoriteSticker::addFavoriteSticker(object_ptr<InputFile> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t addFavoriteSticker::ID;

object_ptr<addFavoriteSticker> addFavoriteSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addFavoriteSticker> res = make_object<addFavoriteSticker>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void addFavoriteSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddFavoriteSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

addFavoriteSticker::ReturnType addFavoriteSticker::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addFavoriteSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddFavoriteSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addFileToDownloads::Class;
jfieldID addFileToDownloads::file_id_fieldID;
jfieldID addFileToDownloads::chat_id_fieldID;
jfieldID addFileToDownloads::message_id_fieldID;
jfieldID addFileToDownloads::priority_fieldID;

addFileToDownloads::addFileToDownloads()
  : file_id_()
  , chat_id_()
  , message_id_()
  , priority_()
{}

addFileToDownloads::addFileToDownloads(int32 file_id_, int53 chat_id_, int53 message_id_, int32 priority_)
  : file_id_(file_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , priority_(priority_)
{}

const std::int32_t addFileToDownloads::ID;

object_ptr<addFileToDownloads> addFileToDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addFileToDownloads> res = make_object<addFileToDownloads>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->priority_ = env->GetIntField(p, res->priority_fieldID);
  return res;
}

void addFileToDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddFileToDownloads");
    s.store_field("fileId", file_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("priority", priority_);
    s.store_class_end();
  }
}

addFileToDownloads::ReturnType addFileToDownloads::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void addFileToDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddFileToDownloads").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    priority_fieldID = jni::get_field_id(env, Class, "priority", "I");
    return true;
  }();
  (void)is_inited;
}

jclass addLocalMessage::Class;
jfieldID addLocalMessage::chat_id_fieldID;
jfieldID addLocalMessage::sender_id_fieldID;
jfieldID addLocalMessage::reply_to_fieldID;
jfieldID addLocalMessage::disable_notification_fieldID;
jfieldID addLocalMessage::input_message_content_fieldID;

addLocalMessage::addLocalMessage()
  : chat_id_()
  , sender_id_()
  , reply_to_()
  , disable_notification_()
  , input_message_content_()
{}

addLocalMessage::addLocalMessage(int53 chat_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, object_ptr<InputMessageContent> &&input_message_content_)
  : chat_id_(chat_id_)
  , sender_id_(std::move(sender_id_))
  , reply_to_(std::move(reply_to_))
  , disable_notification_(disable_notification_)
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t addLocalMessage::ID;

object_ptr<addLocalMessage> addLocalMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addLocalMessage> res = make_object<addLocalMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void addLocalMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddLocalMessage");
    s.store_field("chatId", chat_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_field("disableNotification", disable_notification_);
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

addLocalMessage::ReturnType addLocalMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void addLocalMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddLocalMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addLogMessage::Class;
jfieldID addLogMessage::verbosity_level_fieldID;
jfieldID addLogMessage::text_fieldID;

addLogMessage::addLogMessage()
  : verbosity_level_()
  , text_()
{}

addLogMessage::addLogMessage(int32 verbosity_level_, string const &text_)
  : verbosity_level_(verbosity_level_)
  , text_(text_)
{}

const std::int32_t addLogMessage::ID;

object_ptr<addLogMessage> addLogMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addLogMessage> res = make_object<addLogMessage>();
  res->verbosity_level_ = env->GetIntField(p, res->verbosity_level_fieldID);
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void addLogMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddLogMessage");
    s.store_field("verbosityLevel", verbosity_level_);
    s.store_field("text", text_);
    s.store_class_end();
  }
}

addLogMessage::ReturnType addLogMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addLogMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddLogMessage").c_str());
    verbosity_level_fieldID = jni::get_field_id(env, Class, "verbosityLevel", "I");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass addMessageReaction::Class;
jfieldID addMessageReaction::chat_id_fieldID;
jfieldID addMessageReaction::message_id_fieldID;
jfieldID addMessageReaction::reaction_type_fieldID;
jfieldID addMessageReaction::is_big_fieldID;
jfieldID addMessageReaction::update_recent_reactions_fieldID;

addMessageReaction::addMessageReaction()
  : chat_id_()
  , message_id_()
  , reaction_type_()
  , is_big_()
  , update_recent_reactions_()
{}

addMessageReaction::addMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_, bool is_big_, bool update_recent_reactions_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reaction_type_(std::move(reaction_type_))
  , is_big_(is_big_)
  , update_recent_reactions_(update_recent_reactions_)
{}

const std::int32_t addMessageReaction::ID;

object_ptr<addMessageReaction> addMessageReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addMessageReaction> res = make_object<addMessageReaction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->is_big_ = (env->GetBooleanField(p, res->is_big_fieldID) != 0);
  res->update_recent_reactions_ = (env->GetBooleanField(p, res->update_recent_reactions_fieldID) != 0);
  return res;
}

void addMessageReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddMessageReaction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("isBig", is_big_);
    s.store_field("updateRecentReactions", update_recent_reactions_);
    s.store_class_end();
  }
}

addMessageReaction::ReturnType addMessageReaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addMessageReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddMessageReaction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    is_big_fieldID = jni::get_field_id(env, Class, "isBig", "Z");
    update_recent_reactions_fieldID = jni::get_field_id(env, Class, "updateRecentReactions", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass addNetworkStatistics::Class;
jfieldID addNetworkStatistics::entry_fieldID;

addNetworkStatistics::addNetworkStatistics()
  : entry_()
{}

addNetworkStatistics::addNetworkStatistics(object_ptr<NetworkStatisticsEntry> &&entry_)
  : entry_(std::move(entry_))
{}

const std::int32_t addNetworkStatistics::ID;

object_ptr<addNetworkStatistics> addNetworkStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addNetworkStatistics> res = make_object<addNetworkStatistics>();
  res->entry_ = jni::fetch_tl_object<NetworkStatisticsEntry>(env, jni::fetch_object(env, p, res->entry_fieldID));
  return res;
}

void addNetworkStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddNetworkStatistics");
    s.store_object_field("entry", static_cast<const BaseObject *>(entry_.get()));
    s.store_class_end();
  }
}

addNetworkStatistics::ReturnType addNetworkStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addNetworkStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddNetworkStatistics").c_str());
    entry_fieldID = jni::get_field_id(env, Class, "entry", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NetworkStatisticsEntry;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addPendingPaidMessageReaction::Class;
jfieldID addPendingPaidMessageReaction::chat_id_fieldID;
jfieldID addPendingPaidMessageReaction::message_id_fieldID;
jfieldID addPendingPaidMessageReaction::star_count_fieldID;
jfieldID addPendingPaidMessageReaction::type_fieldID;

addPendingPaidMessageReaction::addPendingPaidMessageReaction()
  : chat_id_()
  , message_id_()
  , star_count_()
  , type_()
{}

addPendingPaidMessageReaction::addPendingPaidMessageReaction(int53 chat_id_, int53 message_id_, int53 star_count_, object_ptr<PaidReactionType> &&type_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , star_count_(star_count_)
  , type_(std::move(type_))
{}

const std::int32_t addPendingPaidMessageReaction::ID;

object_ptr<addPendingPaidMessageReaction> addPendingPaidMessageReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addPendingPaidMessageReaction> res = make_object<addPendingPaidMessageReaction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->type_ = jni::fetch_tl_object<PaidReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void addPendingPaidMessageReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddPendingPaidMessageReaction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("starCount", star_count_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

addPendingPaidMessageReaction::ReturnType addPendingPaidMessageReaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addPendingPaidMessageReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddPendingPaidMessageReaction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaidReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addProxy::Class;
jfieldID addProxy::server_fieldID;
jfieldID addProxy::port_fieldID;
jfieldID addProxy::enable_fieldID;
jfieldID addProxy::type_fieldID;

addProxy::addProxy()
  : server_()
  , port_()
  , enable_()
  , type_()
{}

addProxy::addProxy(string const &server_, int32 port_, bool enable_, object_ptr<ProxyType> &&type_)
  : server_(server_)
  , port_(port_)
  , enable_(enable_)
  , type_(std::move(type_))
{}

const std::int32_t addProxy::ID;

object_ptr<addProxy> addProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addProxy> res = make_object<addProxy>();
  res->server_ = jni::fetch_string(env, p, res->server_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->enable_ = (env->GetBooleanField(p, res->enable_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<ProxyType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void addProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddProxy");
    s.store_field("server", server_);
    s.store_field("port", port_);
    s.store_field("enable", enable_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

addProxy::ReturnType addProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return proxy::fetch(env, p);
}

void addProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddProxy").c_str());
    server_fieldID = jni::get_field_id(env, Class, "server", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    enable_fieldID = jni::get_field_id(env, Class, "enable", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProxyType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addQuickReplyShortcutInlineQueryResultMessage::Class;
jfieldID addQuickReplyShortcutInlineQueryResultMessage::shortcut_name_fieldID;
jfieldID addQuickReplyShortcutInlineQueryResultMessage::reply_to_message_id_fieldID;
jfieldID addQuickReplyShortcutInlineQueryResultMessage::query_id_fieldID;
jfieldID addQuickReplyShortcutInlineQueryResultMessage::result_id_fieldID;
jfieldID addQuickReplyShortcutInlineQueryResultMessage::hide_via_bot_fieldID;

addQuickReplyShortcutInlineQueryResultMessage::addQuickReplyShortcutInlineQueryResultMessage()
  : shortcut_name_()
  , reply_to_message_id_()
  , query_id_()
  , result_id_()
  , hide_via_bot_()
{}

addQuickReplyShortcutInlineQueryResultMessage::addQuickReplyShortcutInlineQueryResultMessage(string const &shortcut_name_, int53 reply_to_message_id_, int64 query_id_, string const &result_id_, bool hide_via_bot_)
  : shortcut_name_(shortcut_name_)
  , reply_to_message_id_(reply_to_message_id_)
  , query_id_(query_id_)
  , result_id_(result_id_)
  , hide_via_bot_(hide_via_bot_)
{}

const std::int32_t addQuickReplyShortcutInlineQueryResultMessage::ID;

object_ptr<addQuickReplyShortcutInlineQueryResultMessage> addQuickReplyShortcutInlineQueryResultMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addQuickReplyShortcutInlineQueryResultMessage> res = make_object<addQuickReplyShortcutInlineQueryResultMessage>();
  res->shortcut_name_ = jni::fetch_string(env, p, res->shortcut_name_fieldID);
  res->reply_to_message_id_ = env->GetLongField(p, res->reply_to_message_id_fieldID);
  res->query_id_ = env->GetLongField(p, res->query_id_fieldID);
  res->result_id_ = jni::fetch_string(env, p, res->result_id_fieldID);
  res->hide_via_bot_ = (env->GetBooleanField(p, res->hide_via_bot_fieldID) != 0);
  return res;
}

void addQuickReplyShortcutInlineQueryResultMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddQuickReplyShortcutInlineQueryResultMessage");
    s.store_field("shortcutName", shortcut_name_);
    s.store_field("replyToMessageId", reply_to_message_id_);
    s.store_field("queryId", query_id_);
    s.store_field("resultId", result_id_);
    s.store_field("hideViaBot", hide_via_bot_);
    s.store_class_end();
  }
}

addQuickReplyShortcutInlineQueryResultMessage::ReturnType addQuickReplyShortcutInlineQueryResultMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return quickReplyMessage::fetch(env, p);
}

void addQuickReplyShortcutInlineQueryResultMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddQuickReplyShortcutInlineQueryResultMessage").c_str());
    shortcut_name_fieldID = jni::get_field_id(env, Class, "shortcutName", "Ljava/lang/String;");
    reply_to_message_id_fieldID = jni::get_field_id(env, Class, "replyToMessageId", "J");
    query_id_fieldID = jni::get_field_id(env, Class, "queryId", "J");
    result_id_fieldID = jni::get_field_id(env, Class, "resultId", "Ljava/lang/String;");
    hide_via_bot_fieldID = jni::get_field_id(env, Class, "hideViaBot", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass addQuickReplyShortcutMessage::Class;
jfieldID addQuickReplyShortcutMessage::shortcut_name_fieldID;
jfieldID addQuickReplyShortcutMessage::reply_to_message_id_fieldID;
jfieldID addQuickReplyShortcutMessage::input_message_content_fieldID;

addQuickReplyShortcutMessage::addQuickReplyShortcutMessage()
  : shortcut_name_()
  , reply_to_message_id_()
  , input_message_content_()
{}

addQuickReplyShortcutMessage::addQuickReplyShortcutMessage(string const &shortcut_name_, int53 reply_to_message_id_, object_ptr<InputMessageContent> &&input_message_content_)
  : shortcut_name_(shortcut_name_)
  , reply_to_message_id_(reply_to_message_id_)
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t addQuickReplyShortcutMessage::ID;

object_ptr<addQuickReplyShortcutMessage> addQuickReplyShortcutMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addQuickReplyShortcutMessage> res = make_object<addQuickReplyShortcutMessage>();
  res->shortcut_name_ = jni::fetch_string(env, p, res->shortcut_name_fieldID);
  res->reply_to_message_id_ = env->GetLongField(p, res->reply_to_message_id_fieldID);
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void addQuickReplyShortcutMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddQuickReplyShortcutMessage");
    s.store_field("shortcutName", shortcut_name_);
    s.store_field("replyToMessageId", reply_to_message_id_);
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

addQuickReplyShortcutMessage::ReturnType addQuickReplyShortcutMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return quickReplyMessage::fetch(env, p);
}

void addQuickReplyShortcutMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddQuickReplyShortcutMessage").c_str());
    shortcut_name_fieldID = jni::get_field_id(env, Class, "shortcutName", "Ljava/lang/String;");
    reply_to_message_id_fieldID = jni::get_field_id(env, Class, "replyToMessageId", "J");
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addQuickReplyShortcutMessageAlbum::Class;
jfieldID addQuickReplyShortcutMessageAlbum::shortcut_name_fieldID;
jfieldID addQuickReplyShortcutMessageAlbum::reply_to_message_id_fieldID;
jfieldID addQuickReplyShortcutMessageAlbum::input_message_contents_fieldID;

addQuickReplyShortcutMessageAlbum::addQuickReplyShortcutMessageAlbum()
  : shortcut_name_()
  , reply_to_message_id_()
  , input_message_contents_()
{}

addQuickReplyShortcutMessageAlbum::addQuickReplyShortcutMessageAlbum(string const &shortcut_name_, int53 reply_to_message_id_, array<object_ptr<InputMessageContent>> &&input_message_contents_)
  : shortcut_name_(shortcut_name_)
  , reply_to_message_id_(reply_to_message_id_)
  , input_message_contents_(std::move(input_message_contents_))
{}

const std::int32_t addQuickReplyShortcutMessageAlbum::ID;

object_ptr<addQuickReplyShortcutMessageAlbum> addQuickReplyShortcutMessageAlbum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addQuickReplyShortcutMessageAlbum> res = make_object<addQuickReplyShortcutMessageAlbum>();
  res->shortcut_name_ = jni::fetch_string(env, p, res->shortcut_name_fieldID);
  res->reply_to_message_id_ = env->GetLongField(p, res->reply_to_message_id_fieldID);
  res->input_message_contents_ = jni::FetchVector<InputMessageContent>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_message_contents_fieldID));
  return res;
}

void addQuickReplyShortcutMessageAlbum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddQuickReplyShortcutMessageAlbum");
    s.store_field("shortcutName", shortcut_name_);
    s.store_field("replyToMessageId", reply_to_message_id_);
    { s.store_vector_begin("inputMessageContents", input_message_contents_.size()); for (const auto &_value : input_message_contents_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

addQuickReplyShortcutMessageAlbum::ReturnType addQuickReplyShortcutMessageAlbum::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return quickReplyMessages::fetch(env, p);
}

void addQuickReplyShortcutMessageAlbum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddQuickReplyShortcutMessageAlbum").c_str());
    shortcut_name_fieldID = jni::get_field_id(env, Class, "shortcutName", "Ljava/lang/String;");
    reply_to_message_id_fieldID = jni::get_field_id(env, Class, "replyToMessageId", "J");
    input_message_contents_fieldID = jni::get_field_id(env, Class, "inputMessageContents", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addRecentSticker::Class;
jfieldID addRecentSticker::is_attached_fieldID;
jfieldID addRecentSticker::sticker_fieldID;

addRecentSticker::addRecentSticker()
  : is_attached_()
  , sticker_()
{}

addRecentSticker::addRecentSticker(bool is_attached_, object_ptr<InputFile> &&sticker_)
  : is_attached_(is_attached_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t addRecentSticker::ID;

object_ptr<addRecentSticker> addRecentSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addRecentSticker> res = make_object<addRecentSticker>();
  res->is_attached_ = (env->GetBooleanField(p, res->is_attached_fieldID) != 0);
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void addRecentSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddRecentSticker");
    s.store_field("isAttached", is_attached_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

addRecentSticker::ReturnType addRecentSticker::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void addRecentSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddRecentSticker").c_str());
    is_attached_fieldID = jni::get_field_id(env, Class, "isAttached", "Z");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addRecentlyFoundChat::Class;
jfieldID addRecentlyFoundChat::chat_id_fieldID;

addRecentlyFoundChat::addRecentlyFoundChat()
  : chat_id_()
{}

addRecentlyFoundChat::addRecentlyFoundChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t addRecentlyFoundChat::ID;

object_ptr<addRecentlyFoundChat> addRecentlyFoundChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addRecentlyFoundChat> res = make_object<addRecentlyFoundChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void addRecentlyFoundChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddRecentlyFoundChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

addRecentlyFoundChat::ReturnType addRecentlyFoundChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addRecentlyFoundChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddRecentlyFoundChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass addSavedAnimation::Class;
jfieldID addSavedAnimation::animation_fieldID;

addSavedAnimation::addSavedAnimation()
  : animation_()
{}

addSavedAnimation::addSavedAnimation(object_ptr<InputFile> &&animation_)
  : animation_(std::move(animation_))
{}

const std::int32_t addSavedAnimation::ID;

object_ptr<addSavedAnimation> addSavedAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addSavedAnimation> res = make_object<addSavedAnimation>();
  res->animation_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void addSavedAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddSavedAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

addSavedAnimation::ReturnType addSavedAnimation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addSavedAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddSavedAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addSavedNotificationSound::Class;
jfieldID addSavedNotificationSound::sound_fieldID;

addSavedNotificationSound::addSavedNotificationSound()
  : sound_()
{}

addSavedNotificationSound::addSavedNotificationSound(object_ptr<InputFile> &&sound_)
  : sound_(std::move(sound_))
{}

const std::int32_t addSavedNotificationSound::ID;

object_ptr<addSavedNotificationSound> addSavedNotificationSound::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addSavedNotificationSound> res = make_object<addSavedNotificationSound>();
  res->sound_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sound_fieldID));
  return res;
}

void addSavedNotificationSound::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddSavedNotificationSound");
    s.store_object_field("sound", static_cast<const BaseObject *>(sound_.get()));
    s.store_class_end();
  }
}

addSavedNotificationSound::ReturnType addSavedNotificationSound::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return notificationSound::fetch(env, p);
}

void addSavedNotificationSound::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddSavedNotificationSound").c_str());
    sound_fieldID = jni::get_field_id(env, Class, "sound", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass addStickerToSet::Class;
jfieldID addStickerToSet::user_id_fieldID;
jfieldID addStickerToSet::name_fieldID;
jfieldID addStickerToSet::sticker_fieldID;

addStickerToSet::addStickerToSet()
  : user_id_()
  , name_()
  , sticker_()
{}

addStickerToSet::addStickerToSet(int53 user_id_, string const &name_, object_ptr<inputSticker> &&sticker_)
  : user_id_(user_id_)
  , name_(name_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t addStickerToSet::ID;

object_ptr<addStickerToSet> addStickerToSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<addStickerToSet> res = make_object<addStickerToSet>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->sticker_ = jni::fetch_tl_object<inputSticker>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void addStickerToSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AddStickerToSet");
    s.store_field("userId", user_id_);
    s.store_field("name", name_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

addStickerToSet::ReturnType addStickerToSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void addStickerToSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AddStickerToSet").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputSticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass allowBotToSendMessages::Class;
jfieldID allowBotToSendMessages::bot_user_id_fieldID;

allowBotToSendMessages::allowBotToSendMessages()
  : bot_user_id_()
{}

allowBotToSendMessages::allowBotToSendMessages(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t allowBotToSendMessages::ID;

object_ptr<allowBotToSendMessages> allowBotToSendMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<allowBotToSendMessages> res = make_object<allowBotToSendMessages>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void allowBotToSendMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AllowBotToSendMessages");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

allowBotToSendMessages::ReturnType allowBotToSendMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void allowBotToSendMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AllowBotToSendMessages").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass allowUnpaidMessagesFromUser::Class;
jfieldID allowUnpaidMessagesFromUser::user_id_fieldID;
jfieldID allowUnpaidMessagesFromUser::refund_payments_fieldID;

allowUnpaidMessagesFromUser::allowUnpaidMessagesFromUser()
  : user_id_()
  , refund_payments_()
{}

allowUnpaidMessagesFromUser::allowUnpaidMessagesFromUser(int53 user_id_, bool refund_payments_)
  : user_id_(user_id_)
  , refund_payments_(refund_payments_)
{}

const std::int32_t allowUnpaidMessagesFromUser::ID;

object_ptr<allowUnpaidMessagesFromUser> allowUnpaidMessagesFromUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<allowUnpaidMessagesFromUser> res = make_object<allowUnpaidMessagesFromUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->refund_payments_ = (env->GetBooleanField(p, res->refund_payments_fieldID) != 0);
  return res;
}

void allowUnpaidMessagesFromUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AllowUnpaidMessagesFromUser");
    s.store_field("userId", user_id_);
    s.store_field("refundPayments", refund_payments_);
    s.store_class_end();
  }
}

allowUnpaidMessagesFromUser::ReturnType allowUnpaidMessagesFromUser::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void allowUnpaidMessagesFromUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AllowUnpaidMessagesFromUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    refund_payments_fieldID = jni::get_field_id(env, Class, "refundPayments", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass answerCallbackQuery::Class;
jfieldID answerCallbackQuery::callback_query_id_fieldID;
jfieldID answerCallbackQuery::text_fieldID;
jfieldID answerCallbackQuery::show_alert_fieldID;
jfieldID answerCallbackQuery::url_fieldID;
jfieldID answerCallbackQuery::cache_time_fieldID;

answerCallbackQuery::answerCallbackQuery()
  : callback_query_id_()
  , text_()
  , show_alert_()
  , url_()
  , cache_time_()
{}

answerCallbackQuery::answerCallbackQuery(int64 callback_query_id_, string const &text_, bool show_alert_, string const &url_, int32 cache_time_)
  : callback_query_id_(callback_query_id_)
  , text_(text_)
  , show_alert_(show_alert_)
  , url_(url_)
  , cache_time_(cache_time_)
{}

const std::int32_t answerCallbackQuery::ID;

object_ptr<answerCallbackQuery> answerCallbackQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerCallbackQuery> res = make_object<answerCallbackQuery>();
  res->callback_query_id_ = env->GetLongField(p, res->callback_query_id_fieldID);
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->show_alert_ = (env->GetBooleanField(p, res->show_alert_fieldID) != 0);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->cache_time_ = env->GetIntField(p, res->cache_time_fieldID);
  return res;
}

void answerCallbackQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerCallbackQuery");
    s.store_field("callbackQueryId", callback_query_id_);
    s.store_field("text", text_);
    s.store_field("showAlert", show_alert_);
    s.store_field("url", url_);
    s.store_field("cacheTime", cache_time_);
    s.store_class_end();
  }
}

answerCallbackQuery::ReturnType answerCallbackQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void answerCallbackQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerCallbackQuery").c_str());
    callback_query_id_fieldID = jni::get_field_id(env, Class, "callbackQueryId", "J");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    show_alert_fieldID = jni::get_field_id(env, Class, "showAlert", "Z");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    cache_time_fieldID = jni::get_field_id(env, Class, "cacheTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass answerCustomQuery::Class;
jfieldID answerCustomQuery::custom_query_id_fieldID;
jfieldID answerCustomQuery::data_fieldID;

answerCustomQuery::answerCustomQuery()
  : custom_query_id_()
  , data_()
{}

answerCustomQuery::answerCustomQuery(int64 custom_query_id_, string const &data_)
  : custom_query_id_(custom_query_id_)
  , data_(data_)
{}

const std::int32_t answerCustomQuery::ID;

object_ptr<answerCustomQuery> answerCustomQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerCustomQuery> res = make_object<answerCustomQuery>();
  res->custom_query_id_ = env->GetLongField(p, res->custom_query_id_fieldID);
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  return res;
}

void answerCustomQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerCustomQuery");
    s.store_field("customQueryId", custom_query_id_);
    s.store_field("data", data_);
    s.store_class_end();
  }
}

answerCustomQuery::ReturnType answerCustomQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void answerCustomQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerCustomQuery").c_str());
    custom_query_id_fieldID = jni::get_field_id(env, Class, "customQueryId", "J");
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass answerInlineQuery::Class;
jfieldID answerInlineQuery::inline_query_id_fieldID;
jfieldID answerInlineQuery::is_personal_fieldID;
jfieldID answerInlineQuery::button_fieldID;
jfieldID answerInlineQuery::results_fieldID;
jfieldID answerInlineQuery::cache_time_fieldID;
jfieldID answerInlineQuery::next_offset_fieldID;

answerInlineQuery::answerInlineQuery()
  : inline_query_id_()
  , is_personal_()
  , button_()
  , results_()
  , cache_time_()
  , next_offset_()
{}

answerInlineQuery::answerInlineQuery(int64 inline_query_id_, bool is_personal_, object_ptr<inlineQueryResultsButton> &&button_, array<object_ptr<InputInlineQueryResult>> &&results_, int32 cache_time_, string const &next_offset_)
  : inline_query_id_(inline_query_id_)
  , is_personal_(is_personal_)
  , button_(std::move(button_))
  , results_(std::move(results_))
  , cache_time_(cache_time_)
  , next_offset_(next_offset_)
{}

const std::int32_t answerInlineQuery::ID;

object_ptr<answerInlineQuery> answerInlineQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerInlineQuery> res = make_object<answerInlineQuery>();
  res->inline_query_id_ = env->GetLongField(p, res->inline_query_id_fieldID);
  res->is_personal_ = (env->GetBooleanField(p, res->is_personal_fieldID) != 0);
  res->button_ = jni::fetch_tl_object<inlineQueryResultsButton>(env, jni::fetch_object(env, p, res->button_fieldID));
  res->results_ = jni::FetchVector<InputInlineQueryResult>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->results_fieldID));
  res->cache_time_ = env->GetIntField(p, res->cache_time_fieldID);
  res->next_offset_ = jni::fetch_string(env, p, res->next_offset_fieldID);
  return res;
}

void answerInlineQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerInlineQuery");
    s.store_field("inlineQueryId", inline_query_id_);
    s.store_field("isPersonal", is_personal_);
    s.store_object_field("button", static_cast<const BaseObject *>(button_.get()));
    { s.store_vector_begin("results", results_.size()); for (const auto &_value : results_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("cacheTime", cache_time_);
    s.store_field("nextOffset", next_offset_);
    s.store_class_end();
  }
}

answerInlineQuery::ReturnType answerInlineQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void answerInlineQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerInlineQuery").c_str());
    inline_query_id_fieldID = jni::get_field_id(env, Class, "inlineQueryId", "J");
    is_personal_fieldID = jni::get_field_id(env, Class, "isPersonal", "Z");
    button_fieldID = jni::get_field_id(env, Class, "button", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InlineQueryResultsButton;").c_str());
    results_fieldID = jni::get_field_id(env, Class, "results", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputInlineQueryResult;").c_str());
    cache_time_fieldID = jni::get_field_id(env, Class, "cacheTime", "I");
    next_offset_fieldID = jni::get_field_id(env, Class, "nextOffset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass answerPreCheckoutQuery::Class;
jfieldID answerPreCheckoutQuery::pre_checkout_query_id_fieldID;
jfieldID answerPreCheckoutQuery::error_message_fieldID;

answerPreCheckoutQuery::answerPreCheckoutQuery()
  : pre_checkout_query_id_()
  , error_message_()
{}

answerPreCheckoutQuery::answerPreCheckoutQuery(int64 pre_checkout_query_id_, string const &error_message_)
  : pre_checkout_query_id_(pre_checkout_query_id_)
  , error_message_(error_message_)
{}

const std::int32_t answerPreCheckoutQuery::ID;

object_ptr<answerPreCheckoutQuery> answerPreCheckoutQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerPreCheckoutQuery> res = make_object<answerPreCheckoutQuery>();
  res->pre_checkout_query_id_ = env->GetLongField(p, res->pre_checkout_query_id_fieldID);
  res->error_message_ = jni::fetch_string(env, p, res->error_message_fieldID);
  return res;
}

void answerPreCheckoutQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerPreCheckoutQuery");
    s.store_field("preCheckoutQueryId", pre_checkout_query_id_);
    s.store_field("errorMessage", error_message_);
    s.store_class_end();
  }
}

answerPreCheckoutQuery::ReturnType answerPreCheckoutQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void answerPreCheckoutQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerPreCheckoutQuery").c_str());
    pre_checkout_query_id_fieldID = jni::get_field_id(env, Class, "preCheckoutQueryId", "J");
    error_message_fieldID = jni::get_field_id(env, Class, "errorMessage", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass answerShippingQuery::Class;
jfieldID answerShippingQuery::shipping_query_id_fieldID;
jfieldID answerShippingQuery::shipping_options_fieldID;
jfieldID answerShippingQuery::error_message_fieldID;

answerShippingQuery::answerShippingQuery()
  : shipping_query_id_()
  , shipping_options_()
  , error_message_()
{}

answerShippingQuery::answerShippingQuery(int64 shipping_query_id_, array<object_ptr<shippingOption>> &&shipping_options_, string const &error_message_)
  : shipping_query_id_(shipping_query_id_)
  , shipping_options_(std::move(shipping_options_))
  , error_message_(error_message_)
{}

const std::int32_t answerShippingQuery::ID;

object_ptr<answerShippingQuery> answerShippingQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerShippingQuery> res = make_object<answerShippingQuery>();
  res->shipping_query_id_ = env->GetLongField(p, res->shipping_query_id_fieldID);
  res->shipping_options_ = jni::FetchVector<shippingOption>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->shipping_options_fieldID));
  res->error_message_ = jni::fetch_string(env, p, res->error_message_fieldID);
  return res;
}

void answerShippingQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerShippingQuery");
    s.store_field("shippingQueryId", shipping_query_id_);
    { s.store_vector_begin("shippingOptions", shipping_options_.size()); for (const auto &_value : shipping_options_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("errorMessage", error_message_);
    s.store_class_end();
  }
}

answerShippingQuery::ReturnType answerShippingQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void answerShippingQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerShippingQuery").c_str());
    shipping_query_id_fieldID = jni::get_field_id(env, Class, "shippingQueryId", "J");
    shipping_options_fieldID = jni::get_field_id(env, Class, "shippingOptions", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ShippingOption;").c_str());
    error_message_fieldID = jni::get_field_id(env, Class, "errorMessage", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass answerWebAppQuery::Class;
jfieldID answerWebAppQuery::web_app_query_id_fieldID;
jfieldID answerWebAppQuery::result_fieldID;

answerWebAppQuery::answerWebAppQuery()
  : web_app_query_id_()
  , result_()
{}

answerWebAppQuery::answerWebAppQuery(string const &web_app_query_id_, object_ptr<InputInlineQueryResult> &&result_)
  : web_app_query_id_(web_app_query_id_)
  , result_(std::move(result_))
{}

const std::int32_t answerWebAppQuery::ID;

object_ptr<answerWebAppQuery> answerWebAppQuery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<answerWebAppQuery> res = make_object<answerWebAppQuery>();
  res->web_app_query_id_ = jni::fetch_string(env, p, res->web_app_query_id_fieldID);
  res->result_ = jni::fetch_tl_object<InputInlineQueryResult>(env, jni::fetch_object(env, p, res->result_fieldID));
  return res;
}

void answerWebAppQuery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AnswerWebAppQuery");
    s.store_field("webAppQueryId", web_app_query_id_);
    s.store_object_field("result", static_cast<const BaseObject *>(result_.get()));
    s.store_class_end();
  }
}

answerWebAppQuery::ReturnType answerWebAppQuery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sentWebAppMessage::fetch(env, p);
}

void answerWebAppQuery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AnswerWebAppQuery").c_str());
    web_app_query_id_fieldID = jni::get_field_id(env, Class, "webAppQueryId", "Ljava/lang/String;");
    result_fieldID = jni::get_field_id(env, Class, "result", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputInlineQueryResult;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass applyPremiumGiftCode::Class;
jfieldID applyPremiumGiftCode::code_fieldID;

applyPremiumGiftCode::applyPremiumGiftCode()
  : code_()
{}

applyPremiumGiftCode::applyPremiumGiftCode(string const &code_)
  : code_(code_)
{}

const std::int32_t applyPremiumGiftCode::ID;

object_ptr<applyPremiumGiftCode> applyPremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<applyPremiumGiftCode> res = make_object<applyPremiumGiftCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void applyPremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ApplyPremiumGiftCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

applyPremiumGiftCode::ReturnType applyPremiumGiftCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void applyPremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ApplyPremiumGiftCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass assignStoreTransaction::Class;
jfieldID assignStoreTransaction::transaction_fieldID;
jfieldID assignStoreTransaction::purpose_fieldID;

assignStoreTransaction::assignStoreTransaction()
  : transaction_()
  , purpose_()
{}

assignStoreTransaction::assignStoreTransaction(object_ptr<StoreTransaction> &&transaction_, object_ptr<StorePaymentPurpose> &&purpose_)
  : transaction_(std::move(transaction_))
  , purpose_(std::move(purpose_))
{}

const std::int32_t assignStoreTransaction::ID;

object_ptr<assignStoreTransaction> assignStoreTransaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<assignStoreTransaction> res = make_object<assignStoreTransaction>();
  res->transaction_ = jni::fetch_tl_object<StoreTransaction>(env, jni::fetch_object(env, p, res->transaction_fieldID));
  res->purpose_ = jni::fetch_tl_object<StorePaymentPurpose>(env, jni::fetch_object(env, p, res->purpose_fieldID));
  return res;
}

void assignStoreTransaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "AssignStoreTransaction");
    s.store_object_field("transaction", static_cast<const BaseObject *>(transaction_.get()));
    s.store_object_field("purpose", static_cast<const BaseObject *>(purpose_.get()));
    s.store_class_end();
  }
}

assignStoreTransaction::ReturnType assignStoreTransaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void assignStoreTransaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$AssignStoreTransaction").c_str());
    transaction_fieldID = jni::get_field_id(env, Class, "transaction", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoreTransaction;").c_str());
    purpose_fieldID = jni::get_field_id(env, Class, "purpose", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StorePaymentPurpose;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass banChatMember::Class;
jfieldID banChatMember::chat_id_fieldID;
jfieldID banChatMember::member_id_fieldID;
jfieldID banChatMember::banned_until_date_fieldID;
jfieldID banChatMember::revoke_messages_fieldID;

banChatMember::banChatMember()
  : chat_id_()
  , member_id_()
  , banned_until_date_()
  , revoke_messages_()
{}

banChatMember::banChatMember(int53 chat_id_, object_ptr<MessageSender> &&member_id_, int32 banned_until_date_, bool revoke_messages_)
  : chat_id_(chat_id_)
  , member_id_(std::move(member_id_))
  , banned_until_date_(banned_until_date_)
  , revoke_messages_(revoke_messages_)
{}

const std::int32_t banChatMember::ID;

object_ptr<banChatMember> banChatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<banChatMember> res = make_object<banChatMember>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  res->banned_until_date_ = env->GetIntField(p, res->banned_until_date_fieldID);
  res->revoke_messages_ = (env->GetBooleanField(p, res->revoke_messages_fieldID) != 0);
  return res;
}

void banChatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BanChatMember");
    s.store_field("chatId", chat_id_);
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_field("bannedUntilDate", banned_until_date_);
    s.store_field("revokeMessages", revoke_messages_);
    s.store_class_end();
  }
}

banChatMember::ReturnType banChatMember::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void banChatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BanChatMember").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    banned_until_date_fieldID = jni::get_field_id(env, Class, "bannedUntilDate", "I");
    revoke_messages_fieldID = jni::get_field_id(env, Class, "revokeMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass banGroupCallParticipants::Class;
jfieldID banGroupCallParticipants::group_call_id_fieldID;
jfieldID banGroupCallParticipants::user_ids_fieldID;

banGroupCallParticipants::banGroupCallParticipants()
  : group_call_id_()
  , user_ids_()
{}

banGroupCallParticipants::banGroupCallParticipants(int32 group_call_id_, array<int64> &&user_ids_)
  : group_call_id_(group_call_id_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t banGroupCallParticipants::ID;

object_ptr<banGroupCallParticipants> banGroupCallParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<banGroupCallParticipants> res = make_object<banGroupCallParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void banGroupCallParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BanGroupCallParticipants");
    s.store_field("groupCallId", group_call_id_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

banGroupCallParticipants::ReturnType banGroupCallParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void banGroupCallParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BanGroupCallParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass blockMessageSenderFromReplies::Class;
jfieldID blockMessageSenderFromReplies::message_id_fieldID;
jfieldID blockMessageSenderFromReplies::delete_message_fieldID;
jfieldID blockMessageSenderFromReplies::delete_all_messages_fieldID;
jfieldID blockMessageSenderFromReplies::report_spam_fieldID;

blockMessageSenderFromReplies::blockMessageSenderFromReplies()
  : message_id_()
  , delete_message_()
  , delete_all_messages_()
  , report_spam_()
{}

blockMessageSenderFromReplies::blockMessageSenderFromReplies(int53 message_id_, bool delete_message_, bool delete_all_messages_, bool report_spam_)
  : message_id_(message_id_)
  , delete_message_(delete_message_)
  , delete_all_messages_(delete_all_messages_)
  , report_spam_(report_spam_)
{}

const std::int32_t blockMessageSenderFromReplies::ID;

object_ptr<blockMessageSenderFromReplies> blockMessageSenderFromReplies::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<blockMessageSenderFromReplies> res = make_object<blockMessageSenderFromReplies>();
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->delete_message_ = (env->GetBooleanField(p, res->delete_message_fieldID) != 0);
  res->delete_all_messages_ = (env->GetBooleanField(p, res->delete_all_messages_fieldID) != 0);
  res->report_spam_ = (env->GetBooleanField(p, res->report_spam_fieldID) != 0);
  return res;
}

void blockMessageSenderFromReplies::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BlockMessageSenderFromReplies");
    s.store_field("messageId", message_id_);
    s.store_field("deleteMessage", delete_message_);
    s.store_field("deleteAllMessages", delete_all_messages_);
    s.store_field("reportSpam", report_spam_);
    s.store_class_end();
  }
}

blockMessageSenderFromReplies::ReturnType blockMessageSenderFromReplies::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void blockMessageSenderFromReplies::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BlockMessageSenderFromReplies").c_str());
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    delete_message_fieldID = jni::get_field_id(env, Class, "deleteMessage", "Z");
    delete_all_messages_fieldID = jni::get_field_id(env, Class, "deleteAllMessages", "Z");
    report_spam_fieldID = jni::get_field_id(env, Class, "reportSpam", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass boostChat::Class;
jfieldID boostChat::chat_id_fieldID;
jfieldID boostChat::slot_ids_fieldID;

boostChat::boostChat()
  : chat_id_()
  , slot_ids_()
{}

boostChat::boostChat(int53 chat_id_, array<int32> &&slot_ids_)
  : chat_id_(chat_id_)
  , slot_ids_(std::move(slot_ids_))
{}

const std::int32_t boostChat::ID;

object_ptr<boostChat> boostChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<boostChat> res = make_object<boostChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->slot_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->slot_ids_fieldID));
  return res;
}

void boostChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "BoostChat");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("slotIds", slot_ids_.size()); for (const auto &_value : slot_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

boostChat::ReturnType boostChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostSlots::fetch(env, p);
}

void boostChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$BoostChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    slot_ids_fieldID = jni::get_field_id(env, Class, "slotIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass canBotSendMessages::Class;
jfieldID canBotSendMessages::bot_user_id_fieldID;

canBotSendMessages::canBotSendMessages()
  : bot_user_id_()
{}

canBotSendMessages::canBotSendMessages(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t canBotSendMessages::ID;

object_ptr<canBotSendMessages> canBotSendMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canBotSendMessages> res = make_object<canBotSendMessages>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void canBotSendMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanBotSendMessages");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

canBotSendMessages::ReturnType canBotSendMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void canBotSendMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanBotSendMessages").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass canPostStory::Class;
jfieldID canPostStory::chat_id_fieldID;

canPostStory::canPostStory()
  : chat_id_()
{}

canPostStory::canPostStory(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t canPostStory::ID;

object_ptr<canPostStory> canPostStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canPostStory> res = make_object<canPostStory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void canPostStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPostStory");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

canPostStory::ReturnType canPostStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return CanPostStoryResult::fetch(env, p);
}

void canPostStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPostStory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass canPurchaseFromStore::Class;
jfieldID canPurchaseFromStore::purpose_fieldID;

canPurchaseFromStore::canPurchaseFromStore()
  : purpose_()
{}

canPurchaseFromStore::canPurchaseFromStore(object_ptr<StorePaymentPurpose> &&purpose_)
  : purpose_(std::move(purpose_))
{}

const std::int32_t canPurchaseFromStore::ID;

object_ptr<canPurchaseFromStore> canPurchaseFromStore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canPurchaseFromStore> res = make_object<canPurchaseFromStore>();
  res->purpose_ = jni::fetch_tl_object<StorePaymentPurpose>(env, jni::fetch_object(env, p, res->purpose_fieldID));
  return res;
}

void canPurchaseFromStore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanPurchaseFromStore");
    s.store_object_field("purpose", static_cast<const BaseObject *>(purpose_.get()));
    s.store_class_end();
  }
}

canPurchaseFromStore::ReturnType canPurchaseFromStore::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void canPurchaseFromStore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanPurchaseFromStore").c_str());
    purpose_fieldID = jni::get_field_id(env, Class, "purpose", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StorePaymentPurpose;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass canSendMessageToUser::Class;
jfieldID canSendMessageToUser::user_id_fieldID;
jfieldID canSendMessageToUser::only_local_fieldID;

canSendMessageToUser::canSendMessageToUser()
  : user_id_()
  , only_local_()
{}

canSendMessageToUser::canSendMessageToUser(int53 user_id_, bool only_local_)
  : user_id_(user_id_)
  , only_local_(only_local_)
{}

const std::int32_t canSendMessageToUser::ID;

object_ptr<canSendMessageToUser> canSendMessageToUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<canSendMessageToUser> res = make_object<canSendMessageToUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void canSendMessageToUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanSendMessageToUser");
    s.store_field("userId", user_id_);
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

canSendMessageToUser::ReturnType canSendMessageToUser::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return CanSendMessageToUserResult::fetch(env, p);
}

void canSendMessageToUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanSendMessageToUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass canTransferOwnership::Class;

canTransferOwnership::canTransferOwnership() {
}

const std::int32_t canTransferOwnership::ID;

object_ptr<canTransferOwnership> canTransferOwnership::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<canTransferOwnership>();
}

void canTransferOwnership::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CanTransferOwnership");
    s.store_class_end();
  }
}

canTransferOwnership::ReturnType canTransferOwnership::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return CanTransferOwnershipResult::fetch(env, p);
}

void canTransferOwnership::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CanTransferOwnership").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass cancelDownloadFile::Class;
jfieldID cancelDownloadFile::file_id_fieldID;
jfieldID cancelDownloadFile::only_if_pending_fieldID;

cancelDownloadFile::cancelDownloadFile()
  : file_id_()
  , only_if_pending_()
{}

cancelDownloadFile::cancelDownloadFile(int32 file_id_, bool only_if_pending_)
  : file_id_(file_id_)
  , only_if_pending_(only_if_pending_)
{}

const std::int32_t cancelDownloadFile::ID;

object_ptr<cancelDownloadFile> cancelDownloadFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<cancelDownloadFile> res = make_object<cancelDownloadFile>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->only_if_pending_ = (env->GetBooleanField(p, res->only_if_pending_fieldID) != 0);
  return res;
}

void cancelDownloadFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CancelDownloadFile");
    s.store_field("fileId", file_id_);
    s.store_field("onlyIfPending", only_if_pending_);
    s.store_class_end();
  }
}

cancelDownloadFile::ReturnType cancelDownloadFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void cancelDownloadFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CancelDownloadFile").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    only_if_pending_fieldID = jni::get_field_id(env, Class, "onlyIfPending", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass cancelPasswordReset::Class;

cancelPasswordReset::cancelPasswordReset() {
}

const std::int32_t cancelPasswordReset::ID;

object_ptr<cancelPasswordReset> cancelPasswordReset::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<cancelPasswordReset>();
}

void cancelPasswordReset::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CancelPasswordReset");
    s.store_class_end();
  }
}

cancelPasswordReset::ReturnType cancelPasswordReset::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void cancelPasswordReset::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CancelPasswordReset").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass cancelPreliminaryUploadFile::Class;
jfieldID cancelPreliminaryUploadFile::file_id_fieldID;

cancelPreliminaryUploadFile::cancelPreliminaryUploadFile()
  : file_id_()
{}

cancelPreliminaryUploadFile::cancelPreliminaryUploadFile(int32 file_id_)
  : file_id_(file_id_)
{}

const std::int32_t cancelPreliminaryUploadFile::ID;

object_ptr<cancelPreliminaryUploadFile> cancelPreliminaryUploadFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<cancelPreliminaryUploadFile> res = make_object<cancelPreliminaryUploadFile>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  return res;
}

void cancelPreliminaryUploadFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CancelPreliminaryUploadFile");
    s.store_field("fileId", file_id_);
    s.store_class_end();
  }
}

cancelPreliminaryUploadFile::ReturnType cancelPreliminaryUploadFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void cancelPreliminaryUploadFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CancelPreliminaryUploadFile").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass cancelRecoveryEmailAddressVerification::Class;

cancelRecoveryEmailAddressVerification::cancelRecoveryEmailAddressVerification() {
}

const std::int32_t cancelRecoveryEmailAddressVerification::ID;

object_ptr<cancelRecoveryEmailAddressVerification> cancelRecoveryEmailAddressVerification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<cancelRecoveryEmailAddressVerification>();
}

void cancelRecoveryEmailAddressVerification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CancelRecoveryEmailAddressVerification");
    s.store_class_end();
  }
}

cancelRecoveryEmailAddressVerification::ReturnType cancelRecoveryEmailAddressVerification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void cancelRecoveryEmailAddressVerification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CancelRecoveryEmailAddressVerification").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass changeImportedContacts::Class;
jfieldID changeImportedContacts::contacts_fieldID;

changeImportedContacts::changeImportedContacts()
  : contacts_()
{}

changeImportedContacts::changeImportedContacts(array<object_ptr<contact>> &&contacts_)
  : contacts_(std::move(contacts_))
{}

const std::int32_t changeImportedContacts::ID;

object_ptr<changeImportedContacts> changeImportedContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<changeImportedContacts> res = make_object<changeImportedContacts>();
  res->contacts_ = jni::FetchVector<contact>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->contacts_fieldID));
  return res;
}

void changeImportedContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChangeImportedContacts");
    { s.store_vector_begin("contacts", contacts_.size()); for (const auto &_value : contacts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

changeImportedContacts::ReturnType changeImportedContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return importedContacts::fetch(env, p);
}

void changeImportedContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChangeImportedContacts").c_str());
    contacts_fieldID = jni::get_field_id(env, Class, "contacts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass changeStickerSet::Class;
jfieldID changeStickerSet::set_id_fieldID;
jfieldID changeStickerSet::is_installed_fieldID;
jfieldID changeStickerSet::is_archived_fieldID;

changeStickerSet::changeStickerSet()
  : set_id_()
  , is_installed_()
  , is_archived_()
{}

changeStickerSet::changeStickerSet(int64 set_id_, bool is_installed_, bool is_archived_)
  : set_id_(set_id_)
  , is_installed_(is_installed_)
  , is_archived_(is_archived_)
{}

const std::int32_t changeStickerSet::ID;

object_ptr<changeStickerSet> changeStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<changeStickerSet> res = make_object<changeStickerSet>();
  res->set_id_ = env->GetLongField(p, res->set_id_fieldID);
  res->is_installed_ = (env->GetBooleanField(p, res->is_installed_fieldID) != 0);
  res->is_archived_ = (env->GetBooleanField(p, res->is_archived_fieldID) != 0);
  return res;
}

void changeStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ChangeStickerSet");
    s.store_field("setId", set_id_);
    s.store_field("isInstalled", is_installed_);
    s.store_field("isArchived", is_archived_);
    s.store_class_end();
  }
}

changeStickerSet::ReturnType changeStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void changeStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ChangeStickerSet").c_str());
    set_id_fieldID = jni::get_field_id(env, Class, "setId", "J");
    is_installed_fieldID = jni::get_field_id(env, Class, "isInstalled", "Z");
    is_archived_fieldID = jni::get_field_id(env, Class, "isArchived", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationBotToken::Class;
jfieldID checkAuthenticationBotToken::token_fieldID;

checkAuthenticationBotToken::checkAuthenticationBotToken()
  : token_()
{}

checkAuthenticationBotToken::checkAuthenticationBotToken(string const &token_)
  : token_(token_)
{}

const std::int32_t checkAuthenticationBotToken::ID;

object_ptr<checkAuthenticationBotToken> checkAuthenticationBotToken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationBotToken> res = make_object<checkAuthenticationBotToken>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return res;
}

void checkAuthenticationBotToken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationBotToken");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

checkAuthenticationBotToken::ReturnType checkAuthenticationBotToken::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationBotToken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationBotToken").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationCode::Class;
jfieldID checkAuthenticationCode::code_fieldID;

checkAuthenticationCode::checkAuthenticationCode()
  : code_()
{}

checkAuthenticationCode::checkAuthenticationCode(string const &code_)
  : code_(code_)
{}

const std::int32_t checkAuthenticationCode::ID;

object_ptr<checkAuthenticationCode> checkAuthenticationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationCode> res = make_object<checkAuthenticationCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void checkAuthenticationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

checkAuthenticationCode::ReturnType checkAuthenticationCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationEmailCode::Class;
jfieldID checkAuthenticationEmailCode::code_fieldID;

checkAuthenticationEmailCode::checkAuthenticationEmailCode()
  : code_()
{}

checkAuthenticationEmailCode::checkAuthenticationEmailCode(object_ptr<EmailAddressAuthentication> &&code_)
  : code_(std::move(code_))
{}

const std::int32_t checkAuthenticationEmailCode::ID;

object_ptr<checkAuthenticationEmailCode> checkAuthenticationEmailCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationEmailCode> res = make_object<checkAuthenticationEmailCode>();
  res->code_ = jni::fetch_tl_object<EmailAddressAuthentication>(env, jni::fetch_object(env, p, res->code_fieldID));
  return res;
}

void checkAuthenticationEmailCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationEmailCode");
    s.store_object_field("code", static_cast<const BaseObject *>(code_.get()));
    s.store_class_end();
  }
}

checkAuthenticationEmailCode::ReturnType checkAuthenticationEmailCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationEmailCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationEmailCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmailAddressAuthentication;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationPassword::Class;
jfieldID checkAuthenticationPassword::password_fieldID;

checkAuthenticationPassword::checkAuthenticationPassword()
  : password_()
{}

checkAuthenticationPassword::checkAuthenticationPassword(string const &password_)
  : password_(password_)
{}

const std::int32_t checkAuthenticationPassword::ID;

object_ptr<checkAuthenticationPassword> checkAuthenticationPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationPassword> res = make_object<checkAuthenticationPassword>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void checkAuthenticationPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationPassword");
    s.store_field("password", password_);
    s.store_class_end();
  }
}

checkAuthenticationPassword::ReturnType checkAuthenticationPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationPassword").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationPasswordRecoveryCode::Class;
jfieldID checkAuthenticationPasswordRecoveryCode::recovery_code_fieldID;

checkAuthenticationPasswordRecoveryCode::checkAuthenticationPasswordRecoveryCode()
  : recovery_code_()
{}

checkAuthenticationPasswordRecoveryCode::checkAuthenticationPasswordRecoveryCode(string const &recovery_code_)
  : recovery_code_(recovery_code_)
{}

const std::int32_t checkAuthenticationPasswordRecoveryCode::ID;

object_ptr<checkAuthenticationPasswordRecoveryCode> checkAuthenticationPasswordRecoveryCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationPasswordRecoveryCode> res = make_object<checkAuthenticationPasswordRecoveryCode>();
  res->recovery_code_ = jni::fetch_string(env, p, res->recovery_code_fieldID);
  return res;
}

void checkAuthenticationPasswordRecoveryCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationPasswordRecoveryCode");
    s.store_field("recoveryCode", recovery_code_);
    s.store_class_end();
  }
}

checkAuthenticationPasswordRecoveryCode::ReturnType checkAuthenticationPasswordRecoveryCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationPasswordRecoveryCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationPasswordRecoveryCode").c_str());
    recovery_code_fieldID = jni::get_field_id(env, Class, "recoveryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkAuthenticationPremiumPurchase::Class;
jfieldID checkAuthenticationPremiumPurchase::currency_fieldID;
jfieldID checkAuthenticationPremiumPurchase::amount_fieldID;

checkAuthenticationPremiumPurchase::checkAuthenticationPremiumPurchase()
  : currency_()
  , amount_()
{}

checkAuthenticationPremiumPurchase::checkAuthenticationPremiumPurchase(string const &currency_, int53 amount_)
  : currency_(currency_)
  , amount_(amount_)
{}

const std::int32_t checkAuthenticationPremiumPurchase::ID;

object_ptr<checkAuthenticationPremiumPurchase> checkAuthenticationPremiumPurchase::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkAuthenticationPremiumPurchase> res = make_object<checkAuthenticationPremiumPurchase>();
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  return res;
}

void checkAuthenticationPremiumPurchase::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckAuthenticationPremiumPurchase");
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_class_end();
  }
}

checkAuthenticationPremiumPurchase::ReturnType checkAuthenticationPremiumPurchase::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkAuthenticationPremiumPurchase::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckAuthenticationPremiumPurchase").c_str());
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass checkChatFolderInviteLink::Class;
jfieldID checkChatFolderInviteLink::invite_link_fieldID;

checkChatFolderInviteLink::checkChatFolderInviteLink()
  : invite_link_()
{}

checkChatFolderInviteLink::checkChatFolderInviteLink(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t checkChatFolderInviteLink::ID;

object_ptr<checkChatFolderInviteLink> checkChatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkChatFolderInviteLink> res = make_object<checkChatFolderInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void checkChatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatFolderInviteLink");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

checkChatFolderInviteLink::ReturnType checkChatFolderInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInviteLinkInfo::fetch(env, p);
}

void checkChatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatFolderInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkChatInviteLink::Class;
jfieldID checkChatInviteLink::invite_link_fieldID;

checkChatInviteLink::checkChatInviteLink()
  : invite_link_()
{}

checkChatInviteLink::checkChatInviteLink(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t checkChatInviteLink::ID;

object_ptr<checkChatInviteLink> checkChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkChatInviteLink> res = make_object<checkChatInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void checkChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatInviteLink");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

checkChatInviteLink::ReturnType checkChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLinkInfo::fetch(env, p);
}

void checkChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkChatUsername::Class;
jfieldID checkChatUsername::chat_id_fieldID;
jfieldID checkChatUsername::username_fieldID;

checkChatUsername::checkChatUsername()
  : chat_id_()
  , username_()
{}

checkChatUsername::checkChatUsername(int53 chat_id_, string const &username_)
  : chat_id_(chat_id_)
  , username_(username_)
{}

const std::int32_t checkChatUsername::ID;

object_ptr<checkChatUsername> checkChatUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkChatUsername> res = make_object<checkChatUsername>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return res;
}

void checkChatUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckChatUsername");
    s.store_field("chatId", chat_id_);
    s.store_field("username", username_);
    s.store_class_end();
  }
}

checkChatUsername::ReturnType checkChatUsername::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return CheckChatUsernameResult::fetch(env, p);
}

void checkChatUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckChatUsername").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkCreatedPublicChatsLimit::Class;
jfieldID checkCreatedPublicChatsLimit::type_fieldID;

checkCreatedPublicChatsLimit::checkCreatedPublicChatsLimit()
  : type_()
{}

checkCreatedPublicChatsLimit::checkCreatedPublicChatsLimit(object_ptr<PublicChatType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t checkCreatedPublicChatsLimit::ID;

object_ptr<checkCreatedPublicChatsLimit> checkCreatedPublicChatsLimit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkCreatedPublicChatsLimit> res = make_object<checkCreatedPublicChatsLimit>();
  res->type_ = jni::fetch_tl_object<PublicChatType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void checkCreatedPublicChatsLimit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckCreatedPublicChatsLimit");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

checkCreatedPublicChatsLimit::ReturnType checkCreatedPublicChatsLimit::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkCreatedPublicChatsLimit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckCreatedPublicChatsLimit").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PublicChatType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkEmailAddressVerificationCode::Class;
jfieldID checkEmailAddressVerificationCode::code_fieldID;

checkEmailAddressVerificationCode::checkEmailAddressVerificationCode()
  : code_()
{}

checkEmailAddressVerificationCode::checkEmailAddressVerificationCode(string const &code_)
  : code_(code_)
{}

const std::int32_t checkEmailAddressVerificationCode::ID;

object_ptr<checkEmailAddressVerificationCode> checkEmailAddressVerificationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkEmailAddressVerificationCode> res = make_object<checkEmailAddressVerificationCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void checkEmailAddressVerificationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckEmailAddressVerificationCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

checkEmailAddressVerificationCode::ReturnType checkEmailAddressVerificationCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkEmailAddressVerificationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckEmailAddressVerificationCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkLoginEmailAddressCode::Class;
jfieldID checkLoginEmailAddressCode::code_fieldID;

checkLoginEmailAddressCode::checkLoginEmailAddressCode()
  : code_()
{}

checkLoginEmailAddressCode::checkLoginEmailAddressCode(object_ptr<EmailAddressAuthentication> &&code_)
  : code_(std::move(code_))
{}

const std::int32_t checkLoginEmailAddressCode::ID;

object_ptr<checkLoginEmailAddressCode> checkLoginEmailAddressCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkLoginEmailAddressCode> res = make_object<checkLoginEmailAddressCode>();
  res->code_ = jni::fetch_tl_object<EmailAddressAuthentication>(env, jni::fetch_object(env, p, res->code_fieldID));
  return res;
}

void checkLoginEmailAddressCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckLoginEmailAddressCode");
    s.store_object_field("code", static_cast<const BaseObject *>(code_.get()));
    s.store_class_end();
  }
}

checkLoginEmailAddressCode::ReturnType checkLoginEmailAddressCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkLoginEmailAddressCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckLoginEmailAddressCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmailAddressAuthentication;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass checkPasswordRecoveryCode::Class;
jfieldID checkPasswordRecoveryCode::recovery_code_fieldID;

checkPasswordRecoveryCode::checkPasswordRecoveryCode()
  : recovery_code_()
{}

checkPasswordRecoveryCode::checkPasswordRecoveryCode(string const &recovery_code_)
  : recovery_code_(recovery_code_)
{}

const std::int32_t checkPasswordRecoveryCode::ID;

object_ptr<checkPasswordRecoveryCode> checkPasswordRecoveryCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkPasswordRecoveryCode> res = make_object<checkPasswordRecoveryCode>();
  res->recovery_code_ = jni::fetch_string(env, p, res->recovery_code_fieldID);
  return res;
}

void checkPasswordRecoveryCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckPasswordRecoveryCode");
    s.store_field("recoveryCode", recovery_code_);
    s.store_class_end();
  }
}

checkPasswordRecoveryCode::ReturnType checkPasswordRecoveryCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkPasswordRecoveryCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckPasswordRecoveryCode").c_str());
    recovery_code_fieldID = jni::get_field_id(env, Class, "recoveryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkPhoneNumberCode::Class;
jfieldID checkPhoneNumberCode::code_fieldID;

checkPhoneNumberCode::checkPhoneNumberCode()
  : code_()
{}

checkPhoneNumberCode::checkPhoneNumberCode(string const &code_)
  : code_(code_)
{}

const std::int32_t checkPhoneNumberCode::ID;

object_ptr<checkPhoneNumberCode> checkPhoneNumberCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkPhoneNumberCode> res = make_object<checkPhoneNumberCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void checkPhoneNumberCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckPhoneNumberCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

checkPhoneNumberCode::ReturnType checkPhoneNumberCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkPhoneNumberCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckPhoneNumberCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkPremiumGiftCode::Class;
jfieldID checkPremiumGiftCode::code_fieldID;

checkPremiumGiftCode::checkPremiumGiftCode()
  : code_()
{}

checkPremiumGiftCode::checkPremiumGiftCode(string const &code_)
  : code_(code_)
{}

const std::int32_t checkPremiumGiftCode::ID;

object_ptr<checkPremiumGiftCode> checkPremiumGiftCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkPremiumGiftCode> res = make_object<checkPremiumGiftCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void checkPremiumGiftCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckPremiumGiftCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

checkPremiumGiftCode::ReturnType checkPremiumGiftCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumGiftCodeInfo::fetch(env, p);
}

void checkPremiumGiftCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckPremiumGiftCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkQuickReplyShortcutName::Class;
jfieldID checkQuickReplyShortcutName::name_fieldID;

checkQuickReplyShortcutName::checkQuickReplyShortcutName()
  : name_()
{}

checkQuickReplyShortcutName::checkQuickReplyShortcutName(string const &name_)
  : name_(name_)
{}

const std::int32_t checkQuickReplyShortcutName::ID;

object_ptr<checkQuickReplyShortcutName> checkQuickReplyShortcutName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkQuickReplyShortcutName> res = make_object<checkQuickReplyShortcutName>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void checkQuickReplyShortcutName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckQuickReplyShortcutName");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

checkQuickReplyShortcutName::ReturnType checkQuickReplyShortcutName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkQuickReplyShortcutName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckQuickReplyShortcutName").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkRecoveryEmailAddressCode::Class;
jfieldID checkRecoveryEmailAddressCode::code_fieldID;

checkRecoveryEmailAddressCode::checkRecoveryEmailAddressCode()
  : code_()
{}

checkRecoveryEmailAddressCode::checkRecoveryEmailAddressCode(string const &code_)
  : code_(code_)
{}

const std::int32_t checkRecoveryEmailAddressCode::ID;

object_ptr<checkRecoveryEmailAddressCode> checkRecoveryEmailAddressCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkRecoveryEmailAddressCode> res = make_object<checkRecoveryEmailAddressCode>();
  res->code_ = jni::fetch_string(env, p, res->code_fieldID);
  return res;
}

void checkRecoveryEmailAddressCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckRecoveryEmailAddressCode");
    s.store_field("code", code_);
    s.store_class_end();
  }
}

checkRecoveryEmailAddressCode::ReturnType checkRecoveryEmailAddressCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void checkRecoveryEmailAddressCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckRecoveryEmailAddressCode").c_str());
    code_fieldID = jni::get_field_id(env, Class, "code", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkStickerSetName::Class;
jfieldID checkStickerSetName::name_fieldID;

checkStickerSetName::checkStickerSetName()
  : name_()
{}

checkStickerSetName::checkStickerSetName(string const &name_)
  : name_(name_)
{}

const std::int32_t checkStickerSetName::ID;

object_ptr<checkStickerSetName> checkStickerSetName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkStickerSetName> res = make_object<checkStickerSetName>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void checkStickerSetName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckStickerSetName");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

checkStickerSetName::ReturnType checkStickerSetName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return CheckStickerSetNameResult::fetch(env, p);
}

void checkStickerSetName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckStickerSetName").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass checkWebAppFileDownload::Class;
jfieldID checkWebAppFileDownload::bot_user_id_fieldID;
jfieldID checkWebAppFileDownload::file_name_fieldID;
jfieldID checkWebAppFileDownload::url_fieldID;

checkWebAppFileDownload::checkWebAppFileDownload()
  : bot_user_id_()
  , file_name_()
  , url_()
{}

checkWebAppFileDownload::checkWebAppFileDownload(int53 bot_user_id_, string const &file_name_, string const &url_)
  : bot_user_id_(bot_user_id_)
  , file_name_(file_name_)
  , url_(url_)
{}

const std::int32_t checkWebAppFileDownload::ID;

object_ptr<checkWebAppFileDownload> checkWebAppFileDownload::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<checkWebAppFileDownload> res = make_object<checkWebAppFileDownload>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void checkWebAppFileDownload::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CheckWebAppFileDownload");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("fileName", file_name_);
    s.store_field("url", url_);
    s.store_class_end();
  }
}

checkWebAppFileDownload::ReturnType checkWebAppFileDownload::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void checkWebAppFileDownload::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CheckWebAppFileDownload").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass cleanFileName::Class;
jfieldID cleanFileName::file_name_fieldID;

cleanFileName::cleanFileName()
  : file_name_()
{}

cleanFileName::cleanFileName(string const &file_name_)
  : file_name_(file_name_)
{}

const std::int32_t cleanFileName::ID;

object_ptr<cleanFileName> cleanFileName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<cleanFileName> res = make_object<cleanFileName>();
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  return res;
}

void cleanFileName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CleanFileName");
    s.store_field("fileName", file_name_);
    s.store_class_end();
  }
}

cleanFileName::ReturnType cleanFileName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void cleanFileName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CleanFileName").c_str());
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass clearAllDraftMessages::Class;
jfieldID clearAllDraftMessages::exclude_secret_chats_fieldID;

clearAllDraftMessages::clearAllDraftMessages()
  : exclude_secret_chats_()
{}

clearAllDraftMessages::clearAllDraftMessages(bool exclude_secret_chats_)
  : exclude_secret_chats_(exclude_secret_chats_)
{}

const std::int32_t clearAllDraftMessages::ID;

object_ptr<clearAllDraftMessages> clearAllDraftMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<clearAllDraftMessages> res = make_object<clearAllDraftMessages>();
  res->exclude_secret_chats_ = (env->GetBooleanField(p, res->exclude_secret_chats_fieldID) != 0);
  return res;
}

void clearAllDraftMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearAllDraftMessages");
    s.store_field("excludeSecretChats", exclude_secret_chats_);
    s.store_class_end();
  }
}

clearAllDraftMessages::ReturnType clearAllDraftMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearAllDraftMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearAllDraftMessages").c_str());
    exclude_secret_chats_fieldID = jni::get_field_id(env, Class, "excludeSecretChats", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass clearAutosaveSettingsExceptions::Class;

clearAutosaveSettingsExceptions::clearAutosaveSettingsExceptions() {
}

const std::int32_t clearAutosaveSettingsExceptions::ID;

object_ptr<clearAutosaveSettingsExceptions> clearAutosaveSettingsExceptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clearAutosaveSettingsExceptions>();
}

void clearAutosaveSettingsExceptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearAutosaveSettingsExceptions");
    s.store_class_end();
  }
}

clearAutosaveSettingsExceptions::ReturnType clearAutosaveSettingsExceptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearAutosaveSettingsExceptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearAutosaveSettingsExceptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass clearImportedContacts::Class;

clearImportedContacts::clearImportedContacts() {
}

const std::int32_t clearImportedContacts::ID;

object_ptr<clearImportedContacts> clearImportedContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clearImportedContacts>();
}

void clearImportedContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearImportedContacts");
    s.store_class_end();
  }
}

clearImportedContacts::ReturnType clearImportedContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearImportedContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearImportedContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass clearRecentEmojiStatuses::Class;

clearRecentEmojiStatuses::clearRecentEmojiStatuses() {
}

const std::int32_t clearRecentEmojiStatuses::ID;

object_ptr<clearRecentEmojiStatuses> clearRecentEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clearRecentEmojiStatuses>();
}

void clearRecentEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearRecentEmojiStatuses");
    s.store_class_end();
  }
}

clearRecentEmojiStatuses::ReturnType clearRecentEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearRecentEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearRecentEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass clearRecentReactions::Class;

clearRecentReactions::clearRecentReactions() {
}

const std::int32_t clearRecentReactions::ID;

object_ptr<clearRecentReactions> clearRecentReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clearRecentReactions>();
}

void clearRecentReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearRecentReactions");
    s.store_class_end();
  }
}

clearRecentReactions::ReturnType clearRecentReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearRecentReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearRecentReactions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass clearRecentStickers::Class;
jfieldID clearRecentStickers::is_attached_fieldID;

clearRecentStickers::clearRecentStickers()
  : is_attached_()
{}

clearRecentStickers::clearRecentStickers(bool is_attached_)
  : is_attached_(is_attached_)
{}

const std::int32_t clearRecentStickers::ID;

object_ptr<clearRecentStickers> clearRecentStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<clearRecentStickers> res = make_object<clearRecentStickers>();
  res->is_attached_ = (env->GetBooleanField(p, res->is_attached_fieldID) != 0);
  return res;
}

void clearRecentStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearRecentStickers");
    s.store_field("isAttached", is_attached_);
    s.store_class_end();
  }
}

clearRecentStickers::ReturnType clearRecentStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearRecentStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearRecentStickers").c_str());
    is_attached_fieldID = jni::get_field_id(env, Class, "isAttached", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass clearRecentlyFoundChats::Class;

clearRecentlyFoundChats::clearRecentlyFoundChats() {
}

const std::int32_t clearRecentlyFoundChats::ID;

object_ptr<clearRecentlyFoundChats> clearRecentlyFoundChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clearRecentlyFoundChats>();
}

void clearRecentlyFoundChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearRecentlyFoundChats");
    s.store_class_end();
  }
}

clearRecentlyFoundChats::ReturnType clearRecentlyFoundChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearRecentlyFoundChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearRecentlyFoundChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass clearSearchedForTags::Class;
jfieldID clearSearchedForTags::clear_cashtags_fieldID;

clearSearchedForTags::clearSearchedForTags()
  : clear_cashtags_()
{}

clearSearchedForTags::clearSearchedForTags(bool clear_cashtags_)
  : clear_cashtags_(clear_cashtags_)
{}

const std::int32_t clearSearchedForTags::ID;

object_ptr<clearSearchedForTags> clearSearchedForTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<clearSearchedForTags> res = make_object<clearSearchedForTags>();
  res->clear_cashtags_ = (env->GetBooleanField(p, res->clear_cashtags_fieldID) != 0);
  return res;
}

void clearSearchedForTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClearSearchedForTags");
    s.store_field("clearCashtags", clear_cashtags_);
    s.store_class_end();
  }
}

clearSearchedForTags::ReturnType clearSearchedForTags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clearSearchedForTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClearSearchedForTags").c_str());
    clear_cashtags_fieldID = jni::get_field_id(env, Class, "clearCashtags", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass clickAnimatedEmojiMessage::Class;
jfieldID clickAnimatedEmojiMessage::chat_id_fieldID;
jfieldID clickAnimatedEmojiMessage::message_id_fieldID;

clickAnimatedEmojiMessage::clickAnimatedEmojiMessage()
  : chat_id_()
  , message_id_()
{}

clickAnimatedEmojiMessage::clickAnimatedEmojiMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t clickAnimatedEmojiMessage::ID;

object_ptr<clickAnimatedEmojiMessage> clickAnimatedEmojiMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<clickAnimatedEmojiMessage> res = make_object<clickAnimatedEmojiMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void clickAnimatedEmojiMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClickAnimatedEmojiMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

clickAnimatedEmojiMessage::ReturnType clickAnimatedEmojiMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sticker::fetch(env, p);
}

void clickAnimatedEmojiMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClickAnimatedEmojiMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass clickChatSponsoredMessage::Class;
jfieldID clickChatSponsoredMessage::chat_id_fieldID;
jfieldID clickChatSponsoredMessage::message_id_fieldID;
jfieldID clickChatSponsoredMessage::is_media_click_fieldID;
jfieldID clickChatSponsoredMessage::from_fullscreen_fieldID;

clickChatSponsoredMessage::clickChatSponsoredMessage()
  : chat_id_()
  , message_id_()
  , is_media_click_()
  , from_fullscreen_()
{}

clickChatSponsoredMessage::clickChatSponsoredMessage(int53 chat_id_, int53 message_id_, bool is_media_click_, bool from_fullscreen_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , is_media_click_(is_media_click_)
  , from_fullscreen_(from_fullscreen_)
{}

const std::int32_t clickChatSponsoredMessage::ID;

object_ptr<clickChatSponsoredMessage> clickChatSponsoredMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<clickChatSponsoredMessage> res = make_object<clickChatSponsoredMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_media_click_ = (env->GetBooleanField(p, res->is_media_click_fieldID) != 0);
  res->from_fullscreen_ = (env->GetBooleanField(p, res->from_fullscreen_fieldID) != 0);
  return res;
}

void clickChatSponsoredMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClickChatSponsoredMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("isMediaClick", is_media_click_);
    s.store_field("fromFullscreen", from_fullscreen_);
    s.store_class_end();
  }
}

clickChatSponsoredMessage::ReturnType clickChatSponsoredMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clickChatSponsoredMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClickChatSponsoredMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_media_click_fieldID = jni::get_field_id(env, Class, "isMediaClick", "Z");
    from_fullscreen_fieldID = jni::get_field_id(env, Class, "fromFullscreen", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass clickPremiumSubscriptionButton::Class;

clickPremiumSubscriptionButton::clickPremiumSubscriptionButton() {
}

const std::int32_t clickPremiumSubscriptionButton::ID;

object_ptr<clickPremiumSubscriptionButton> clickPremiumSubscriptionButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<clickPremiumSubscriptionButton>();
}

void clickPremiumSubscriptionButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ClickPremiumSubscriptionButton");
    s.store_class_end();
  }
}

clickPremiumSubscriptionButton::ReturnType clickPremiumSubscriptionButton::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void clickPremiumSubscriptionButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ClickPremiumSubscriptionButton").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass close::Class;

close::close() {
}

const std::int32_t close::ID;

object_ptr<close> close::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<close>();
}

void close::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Close");
    s.store_class_end();
  }
}

close::ReturnType close::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void close::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Close").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass closeChat::Class;
jfieldID closeChat::chat_id_fieldID;

closeChat::closeChat()
  : chat_id_()
{}

closeChat::closeChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t closeChat::ID;

object_ptr<closeChat> closeChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closeChat> res = make_object<closeChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void closeChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CloseChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

closeChat::ReturnType closeChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void closeChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CloseChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass closeSecretChat::Class;
jfieldID closeSecretChat::secret_chat_id_fieldID;

closeSecretChat::closeSecretChat()
  : secret_chat_id_()
{}

closeSecretChat::closeSecretChat(int32 secret_chat_id_)
  : secret_chat_id_(secret_chat_id_)
{}

const std::int32_t closeSecretChat::ID;

object_ptr<closeSecretChat> closeSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closeSecretChat> res = make_object<closeSecretChat>();
  res->secret_chat_id_ = env->GetIntField(p, res->secret_chat_id_fieldID);
  return res;
}

void closeSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CloseSecretChat");
    s.store_field("secretChatId", secret_chat_id_);
    s.store_class_end();
  }
}

closeSecretChat::ReturnType closeSecretChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void closeSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CloseSecretChat").c_str());
    secret_chat_id_fieldID = jni::get_field_id(env, Class, "secretChatId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass closeStory::Class;
jfieldID closeStory::story_poster_chat_id_fieldID;
jfieldID closeStory::story_id_fieldID;

closeStory::closeStory()
  : story_poster_chat_id_()
  , story_id_()
{}

closeStory::closeStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t closeStory::ID;

object_ptr<closeStory> closeStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closeStory> res = make_object<closeStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return res;
}

void closeStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CloseStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

closeStory::ReturnType closeStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void closeStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CloseStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass closeWebApp::Class;
jfieldID closeWebApp::web_app_launch_id_fieldID;

closeWebApp::closeWebApp()
  : web_app_launch_id_()
{}

closeWebApp::closeWebApp(int64 web_app_launch_id_)
  : web_app_launch_id_(web_app_launch_id_)
{}

const std::int32_t closeWebApp::ID;

object_ptr<closeWebApp> closeWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<closeWebApp> res = make_object<closeWebApp>();
  res->web_app_launch_id_ = env->GetLongField(p, res->web_app_launch_id_fieldID);
  return res;
}

void closeWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CloseWebApp");
    s.store_field("webAppLaunchId", web_app_launch_id_);
    s.store_class_end();
  }
}

closeWebApp::ReturnType closeWebApp::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void closeWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CloseWebApp").c_str());
    web_app_launch_id_fieldID = jni::get_field_id(env, Class, "webAppLaunchId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass commitPendingPaidMessageReactions::Class;
jfieldID commitPendingPaidMessageReactions::chat_id_fieldID;
jfieldID commitPendingPaidMessageReactions::message_id_fieldID;

commitPendingPaidMessageReactions::commitPendingPaidMessageReactions()
  : chat_id_()
  , message_id_()
{}

commitPendingPaidMessageReactions::commitPendingPaidMessageReactions(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t commitPendingPaidMessageReactions::ID;

object_ptr<commitPendingPaidMessageReactions> commitPendingPaidMessageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<commitPendingPaidMessageReactions> res = make_object<commitPendingPaidMessageReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void commitPendingPaidMessageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CommitPendingPaidMessageReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

commitPendingPaidMessageReactions::ReturnType commitPendingPaidMessageReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void commitPendingPaidMessageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CommitPendingPaidMessageReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass confirmQrCodeAuthentication::Class;
jfieldID confirmQrCodeAuthentication::link_fieldID;

confirmQrCodeAuthentication::confirmQrCodeAuthentication()
  : link_()
{}

confirmQrCodeAuthentication::confirmQrCodeAuthentication(string const &link_)
  : link_(link_)
{}

const std::int32_t confirmQrCodeAuthentication::ID;

object_ptr<confirmQrCodeAuthentication> confirmQrCodeAuthentication::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<confirmQrCodeAuthentication> res = make_object<confirmQrCodeAuthentication>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return res;
}

void confirmQrCodeAuthentication::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConfirmQrCodeAuthentication");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

confirmQrCodeAuthentication::ReturnType confirmQrCodeAuthentication::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return session::fetch(env, p);
}

void confirmQrCodeAuthentication::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConfirmQrCodeAuthentication").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass confirmSession::Class;
jfieldID confirmSession::session_id_fieldID;

confirmSession::confirmSession()
  : session_id_()
{}

confirmSession::confirmSession(int64 session_id_)
  : session_id_(session_id_)
{}

const std::int32_t confirmSession::ID;

object_ptr<confirmSession> confirmSession::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<confirmSession> res = make_object<confirmSession>();
  res->session_id_ = env->GetLongField(p, res->session_id_fieldID);
  return res;
}

void confirmSession::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConfirmSession");
    s.store_field("sessionId", session_id_);
    s.store_class_end();
  }
}

confirmSession::ReturnType confirmSession::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void confirmSession::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConfirmSession").c_str());
    session_id_fieldID = jni::get_field_id(env, Class, "sessionId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass connectAffiliateProgram::Class;
jfieldID connectAffiliateProgram::affiliate_fieldID;
jfieldID connectAffiliateProgram::bot_user_id_fieldID;

connectAffiliateProgram::connectAffiliateProgram()
  : affiliate_()
  , bot_user_id_()
{}

connectAffiliateProgram::connectAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, int53 bot_user_id_)
  : affiliate_(std::move(affiliate_))
  , bot_user_id_(bot_user_id_)
{}

const std::int32_t connectAffiliateProgram::ID;

object_ptr<connectAffiliateProgram> connectAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<connectAffiliateProgram> res = make_object<connectAffiliateProgram>();
  res->affiliate_ = jni::fetch_tl_object<AffiliateType>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void connectAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ConnectAffiliateProgram");
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

connectAffiliateProgram::ReturnType connectAffiliateProgram::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return connectedAffiliateProgram::fetch(env, p);
}

void connectAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ConnectAffiliateProgram").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateType;").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass createBasicGroupChat::Class;
jfieldID createBasicGroupChat::basic_group_id_fieldID;
jfieldID createBasicGroupChat::force_fieldID;

createBasicGroupChat::createBasicGroupChat()
  : basic_group_id_()
  , force_()
{}

createBasicGroupChat::createBasicGroupChat(int53 basic_group_id_, bool force_)
  : basic_group_id_(basic_group_id_)
  , force_(force_)
{}

const std::int32_t createBasicGroupChat::ID;

object_ptr<createBasicGroupChat> createBasicGroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createBasicGroupChat> res = make_object<createBasicGroupChat>();
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  res->force_ = (env->GetBooleanField(p, res->force_fieldID) != 0);
  return res;
}

void createBasicGroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateBasicGroupChat");
    s.store_field("basicGroupId", basic_group_id_);
    s.store_field("force", force_);
    s.store_class_end();
  }
}

createBasicGroupChat::ReturnType createBasicGroupChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createBasicGroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateBasicGroupChat").c_str());
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    force_fieldID = jni::get_field_id(env, Class, "force", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createBusinessChatLink::Class;
jfieldID createBusinessChatLink::link_info_fieldID;

createBusinessChatLink::createBusinessChatLink()
  : link_info_()
{}

createBusinessChatLink::createBusinessChatLink(object_ptr<inputBusinessChatLink> &&link_info_)
  : link_info_(std::move(link_info_))
{}

const std::int32_t createBusinessChatLink::ID;

object_ptr<createBusinessChatLink> createBusinessChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createBusinessChatLink> res = make_object<createBusinessChatLink>();
  res->link_info_ = jni::fetch_tl_object<inputBusinessChatLink>(env, jni::fetch_object(env, p, res->link_info_fieldID));
  return res;
}

void createBusinessChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateBusinessChatLink");
    s.store_object_field("linkInfo", static_cast<const BaseObject *>(link_info_.get()));
    s.store_class_end();
  }
}

createBusinessChatLink::ReturnType createBusinessChatLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessChatLink::fetch(env, p);
}

void createBusinessChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateBusinessChatLink").c_str());
    link_info_fieldID = jni::get_field_id(env, Class, "linkInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputBusinessChatLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createCall::Class;
jfieldID createCall::user_id_fieldID;
jfieldID createCall::protocol_fieldID;
jfieldID createCall::is_video_fieldID;

createCall::createCall()
  : user_id_()
  , protocol_()
  , is_video_()
{}

createCall::createCall(int53 user_id_, object_ptr<callProtocol> &&protocol_, bool is_video_)
  : user_id_(user_id_)
  , protocol_(std::move(protocol_))
  , is_video_(is_video_)
{}

const std::int32_t createCall::ID;

object_ptr<createCall> createCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createCall> res = make_object<createCall>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->protocol_ = jni::fetch_tl_object<callProtocol>(env, jni::fetch_object(env, p, res->protocol_fieldID));
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  return res;
}

void createCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateCall");
    s.store_field("userId", user_id_);
    s.store_object_field("protocol", static_cast<const BaseObject *>(protocol_.get()));
    s.store_field("isVideo", is_video_);
    s.store_class_end();
  }
}

createCall::ReturnType createCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return callId::fetch(env, p);
}

void createCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateCall").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    protocol_fieldID = jni::get_field_id(env, Class, "protocol", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallProtocol;").c_str());
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createChatFolder::Class;
jfieldID createChatFolder::folder_fieldID;

createChatFolder::createChatFolder()
  : folder_()
{}

createChatFolder::createChatFolder(object_ptr<chatFolder> &&folder_)
  : folder_(std::move(folder_))
{}

const std::int32_t createChatFolder::ID;

object_ptr<createChatFolder> createChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createChatFolder> res = make_object<createChatFolder>();
  res->folder_ = jni::fetch_tl_object<chatFolder>(env, jni::fetch_object(env, p, res->folder_fieldID));
  return res;
}

void createChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateChatFolder");
    s.store_object_field("folder", static_cast<const BaseObject *>(folder_.get()));
    s.store_class_end();
  }
}

createChatFolder::ReturnType createChatFolder::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInfo::fetch(env, p);
}

void createChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateChatFolder").c_str());
    folder_fieldID = jni::get_field_id(env, Class, "folder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolder;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createChatFolderInviteLink::Class;
jfieldID createChatFolderInviteLink::chat_folder_id_fieldID;
jfieldID createChatFolderInviteLink::name_fieldID;
jfieldID createChatFolderInviteLink::chat_ids_fieldID;

createChatFolderInviteLink::createChatFolderInviteLink()
  : chat_folder_id_()
  , name_()
  , chat_ids_()
{}

createChatFolderInviteLink::createChatFolderInviteLink(int32 chat_folder_id_, string const &name_, array<int53> &&chat_ids_)
  : chat_folder_id_(chat_folder_id_)
  , name_(name_)
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t createChatFolderInviteLink::ID;

object_ptr<createChatFolderInviteLink> createChatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createChatFolderInviteLink> res = make_object<createChatFolderInviteLink>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void createChatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateChatFolderInviteLink");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_field("name", name_);
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

createChatFolderInviteLink::ReturnType createChatFolderInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInviteLink::fetch(env, p);
}

void createChatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateChatFolderInviteLink").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass createChatInviteLink::Class;
jfieldID createChatInviteLink::chat_id_fieldID;
jfieldID createChatInviteLink::name_fieldID;
jfieldID createChatInviteLink::expiration_date_fieldID;
jfieldID createChatInviteLink::member_limit_fieldID;
jfieldID createChatInviteLink::creates_join_request_fieldID;

createChatInviteLink::createChatInviteLink()
  : chat_id_()
  , name_()
  , expiration_date_()
  , member_limit_()
  , creates_join_request_()
{}

createChatInviteLink::createChatInviteLink(int53 chat_id_, string const &name_, int32 expiration_date_, int32 member_limit_, bool creates_join_request_)
  : chat_id_(chat_id_)
  , name_(name_)
  , expiration_date_(expiration_date_)
  , member_limit_(member_limit_)
  , creates_join_request_(creates_join_request_)
{}

const std::int32_t createChatInviteLink::ID;

object_ptr<createChatInviteLink> createChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createChatInviteLink> res = make_object<createChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  res->member_limit_ = env->GetIntField(p, res->member_limit_fieldID);
  res->creates_join_request_ = (env->GetBooleanField(p, res->creates_join_request_fieldID) != 0);
  return res;
}

void createChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("name", name_);
    s.store_field("expirationDate", expiration_date_);
    s.store_field("memberLimit", member_limit_);
    s.store_field("createsJoinRequest", creates_join_request_);
    s.store_class_end();
  }
}

createChatInviteLink::ReturnType createChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void createChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    member_limit_fieldID = jni::get_field_id(env, Class, "memberLimit", "I");
    creates_join_request_fieldID = jni::get_field_id(env, Class, "createsJoinRequest", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createChatSubscriptionInviteLink::Class;
jfieldID createChatSubscriptionInviteLink::chat_id_fieldID;
jfieldID createChatSubscriptionInviteLink::name_fieldID;
jfieldID createChatSubscriptionInviteLink::subscription_pricing_fieldID;

createChatSubscriptionInviteLink::createChatSubscriptionInviteLink()
  : chat_id_()
  , name_()
  , subscription_pricing_()
{}

createChatSubscriptionInviteLink::createChatSubscriptionInviteLink(int53 chat_id_, string const &name_, object_ptr<starSubscriptionPricing> &&subscription_pricing_)
  : chat_id_(chat_id_)
  , name_(name_)
  , subscription_pricing_(std::move(subscription_pricing_))
{}

const std::int32_t createChatSubscriptionInviteLink::ID;

object_ptr<createChatSubscriptionInviteLink> createChatSubscriptionInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createChatSubscriptionInviteLink> res = make_object<createChatSubscriptionInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->subscription_pricing_ = jni::fetch_tl_object<starSubscriptionPricing>(env, jni::fetch_object(env, p, res->subscription_pricing_fieldID));
  return res;
}

void createChatSubscriptionInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateChatSubscriptionInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("name", name_);
    s.store_object_field("subscriptionPricing", static_cast<const BaseObject *>(subscription_pricing_.get()));
    s.store_class_end();
  }
}

createChatSubscriptionInviteLink::ReturnType createChatSubscriptionInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void createChatSubscriptionInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateChatSubscriptionInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    subscription_pricing_fieldID = jni::get_field_id(env, Class, "subscriptionPricing", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarSubscriptionPricing;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createForumTopic::Class;
jfieldID createForumTopic::chat_id_fieldID;
jfieldID createForumTopic::name_fieldID;
jfieldID createForumTopic::icon_fieldID;

createForumTopic::createForumTopic()
  : chat_id_()
  , name_()
  , icon_()
{}

createForumTopic::createForumTopic(int53 chat_id_, string const &name_, object_ptr<forumTopicIcon> &&icon_)
  : chat_id_(chat_id_)
  , name_(name_)
  , icon_(std::move(icon_))
{}

const std::int32_t createForumTopic::ID;

object_ptr<createForumTopic> createForumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createForumTopic> res = make_object<createForumTopic>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->icon_ = jni::fetch_tl_object<forumTopicIcon>(env, jni::fetch_object(env, p, res->icon_fieldID));
  return res;
}

void createForumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateForumTopic");
    s.store_field("chatId", chat_id_);
    s.store_field("name", name_);
    s.store_object_field("icon", static_cast<const BaseObject *>(icon_.get()));
    s.store_class_end();
  }
}

createForumTopic::ReturnType createForumTopic::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return forumTopicInfo::fetch(env, p);
}

void createForumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateForumTopic").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    icon_fieldID = jni::get_field_id(env, Class, "icon", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ForumTopicIcon;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createGroupCall::Class;
jfieldID createGroupCall::join_parameters_fieldID;

createGroupCall::createGroupCall()
  : join_parameters_()
{}

createGroupCall::createGroupCall(object_ptr<groupCallJoinParameters> &&join_parameters_)
  : join_parameters_(std::move(join_parameters_))
{}

const std::int32_t createGroupCall::ID;

object_ptr<createGroupCall> createGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createGroupCall> res = make_object<createGroupCall>();
  res->join_parameters_ = jni::fetch_tl_object<groupCallJoinParameters>(env, jni::fetch_object(env, p, res->join_parameters_fieldID));
  return res;
}

void createGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateGroupCall");
    s.store_object_field("joinParameters", static_cast<const BaseObject *>(join_parameters_.get()));
    s.store_class_end();
  }
}

createGroupCall::ReturnType createGroupCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return groupCallInfo::fetch(env, p);
}

void createGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateGroupCall").c_str());
    join_parameters_fieldID = jni::get_field_id(env, Class, "joinParameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallJoinParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createInvoiceLink::Class;
jfieldID createInvoiceLink::business_connection_id_fieldID;
jfieldID createInvoiceLink::invoice_fieldID;

createInvoiceLink::createInvoiceLink()
  : business_connection_id_()
  , invoice_()
{}

createInvoiceLink::createInvoiceLink(string const &business_connection_id_, object_ptr<InputMessageContent> &&invoice_)
  : business_connection_id_(business_connection_id_)
  , invoice_(std::move(invoice_))
{}

const std::int32_t createInvoiceLink::ID;

object_ptr<createInvoiceLink> createInvoiceLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createInvoiceLink> res = make_object<createInvoiceLink>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->invoice_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->invoice_fieldID));
  return res;
}

void createInvoiceLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateInvoiceLink");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_object_field("invoice", static_cast<const BaseObject *>(invoice_.get()));
    s.store_class_end();
  }
}

createInvoiceLink::ReturnType createInvoiceLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void createInvoiceLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateInvoiceLink").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    invoice_fieldID = jni::get_field_id(env, Class, "invoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass createNewBasicGroupChat::Class;
jfieldID createNewBasicGroupChat::user_ids_fieldID;
jfieldID createNewBasicGroupChat::title_fieldID;
jfieldID createNewBasicGroupChat::message_auto_delete_time_fieldID;

createNewBasicGroupChat::createNewBasicGroupChat()
  : user_ids_()
  , title_()
  , message_auto_delete_time_()
{}

createNewBasicGroupChat::createNewBasicGroupChat(array<int53> &&user_ids_, string const &title_, int32 message_auto_delete_time_)
  : user_ids_(std::move(user_ids_))
  , title_(title_)
  , message_auto_delete_time_(message_auto_delete_time_)
{}

const std::int32_t createNewBasicGroupChat::ID;

object_ptr<createNewBasicGroupChat> createNewBasicGroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createNewBasicGroupChat> res = make_object<createNewBasicGroupChat>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  return res;
}

void createNewBasicGroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateNewBasicGroupChat");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("title", title_);
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_class_end();
  }
}

createNewBasicGroupChat::ReturnType createNewBasicGroupChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return createdBasicGroupChat::fetch(env, p);
}

void createNewBasicGroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateNewBasicGroupChat").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass createNewSecretChat::Class;
jfieldID createNewSecretChat::user_id_fieldID;

createNewSecretChat::createNewSecretChat()
  : user_id_()
{}

createNewSecretChat::createNewSecretChat(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t createNewSecretChat::ID;

object_ptr<createNewSecretChat> createNewSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createNewSecretChat> res = make_object<createNewSecretChat>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void createNewSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateNewSecretChat");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

createNewSecretChat::ReturnType createNewSecretChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createNewSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateNewSecretChat").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass createNewStickerSet::Class;
jfieldID createNewStickerSet::user_id_fieldID;
jfieldID createNewStickerSet::title_fieldID;
jfieldID createNewStickerSet::name_fieldID;
jfieldID createNewStickerSet::sticker_type_fieldID;
jfieldID createNewStickerSet::needs_repainting_fieldID;
jfieldID createNewStickerSet::stickers_fieldID;
jfieldID createNewStickerSet::source_fieldID;

createNewStickerSet::createNewStickerSet()
  : user_id_()
  , title_()
  , name_()
  , sticker_type_()
  , needs_repainting_()
  , stickers_()
  , source_()
{}

createNewStickerSet::createNewStickerSet(int53 user_id_, string const &title_, string const &name_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, array<object_ptr<inputSticker>> &&stickers_, string const &source_)
  : user_id_(user_id_)
  , title_(title_)
  , name_(name_)
  , sticker_type_(std::move(sticker_type_))
  , needs_repainting_(needs_repainting_)
  , stickers_(std::move(stickers_))
  , source_(source_)
{}

const std::int32_t createNewStickerSet::ID;

object_ptr<createNewStickerSet> createNewStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createNewStickerSet> res = make_object<createNewStickerSet>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->needs_repainting_ = (env->GetBooleanField(p, res->needs_repainting_fieldID) != 0);
  res->stickers_ = jni::FetchVector<inputSticker>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->stickers_fieldID));
  res->source_ = jni::fetch_string(env, p, res->source_fieldID);
  return res;
}

void createNewStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateNewStickerSet");
    s.store_field("userId", user_id_);
    s.store_field("title", title_);
    s.store_field("name", name_);
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("needsRepainting", needs_repainting_);
    { s.store_vector_begin("stickers", stickers_.size()); for (const auto &_value : stickers_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("source", source_);
    s.store_class_end();
  }
}

createNewStickerSet::ReturnType createNewStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSet::fetch(env, p);
}

void createNewStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateNewStickerSet").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    needs_repainting_fieldID = jni::get_field_id(env, Class, "needsRepainting", "Z");
    stickers_fieldID = jni::get_field_id(env, Class, "stickers", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputSticker;").c_str());
    source_fieldID = jni::get_field_id(env, Class, "source", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass createNewSupergroupChat::Class;
jfieldID createNewSupergroupChat::title_fieldID;
jfieldID createNewSupergroupChat::is_forum_fieldID;
jfieldID createNewSupergroupChat::is_channel_fieldID;
jfieldID createNewSupergroupChat::description_fieldID;
jfieldID createNewSupergroupChat::location_fieldID;
jfieldID createNewSupergroupChat::message_auto_delete_time_fieldID;
jfieldID createNewSupergroupChat::for_import_fieldID;

createNewSupergroupChat::createNewSupergroupChat()
  : title_()
  , is_forum_()
  , is_channel_()
  , description_()
  , location_()
  , message_auto_delete_time_()
  , for_import_()
{}

createNewSupergroupChat::createNewSupergroupChat(string const &title_, bool is_forum_, bool is_channel_, string const &description_, object_ptr<chatLocation> &&location_, int32 message_auto_delete_time_, bool for_import_)
  : title_(title_)
  , is_forum_(is_forum_)
  , is_channel_(is_channel_)
  , description_(description_)
  , location_(std::move(location_))
  , message_auto_delete_time_(message_auto_delete_time_)
  , for_import_(for_import_)
{}

const std::int32_t createNewSupergroupChat::ID;

object_ptr<createNewSupergroupChat> createNewSupergroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createNewSupergroupChat> res = make_object<createNewSupergroupChat>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->is_forum_ = (env->GetBooleanField(p, res->is_forum_fieldID) != 0);
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  res->location_ = jni::fetch_tl_object<chatLocation>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  res->for_import_ = (env->GetBooleanField(p, res->for_import_fieldID) != 0);
  return res;
}

void createNewSupergroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateNewSupergroupChat");
    s.store_field("title", title_);
    s.store_field("isForum", is_forum_);
    s.store_field("isChannel", is_channel_);
    s.store_field("description", description_);
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_field("forImport", for_import_);
    s.store_class_end();
  }
}

createNewSupergroupChat::ReturnType createNewSupergroupChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createNewSupergroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateNewSupergroupChat").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    is_forum_fieldID = jni::get_field_id(env, Class, "isForum", "Z");
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatLocation;").c_str());
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    for_import_fieldID = jni::get_field_id(env, Class, "forImport", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createPrivateChat::Class;
jfieldID createPrivateChat::user_id_fieldID;
jfieldID createPrivateChat::force_fieldID;

createPrivateChat::createPrivateChat()
  : user_id_()
  , force_()
{}

createPrivateChat::createPrivateChat(int53 user_id_, bool force_)
  : user_id_(user_id_)
  , force_(force_)
{}

const std::int32_t createPrivateChat::ID;

object_ptr<createPrivateChat> createPrivateChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createPrivateChat> res = make_object<createPrivateChat>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->force_ = (env->GetBooleanField(p, res->force_fieldID) != 0);
  return res;
}

void createPrivateChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreatePrivateChat");
    s.store_field("userId", user_id_);
    s.store_field("force", force_);
    s.store_class_end();
  }
}

createPrivateChat::ReturnType createPrivateChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createPrivateChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreatePrivateChat").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    force_fieldID = jni::get_field_id(env, Class, "force", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createSecretChat::Class;
jfieldID createSecretChat::secret_chat_id_fieldID;

createSecretChat::createSecretChat()
  : secret_chat_id_()
{}

createSecretChat::createSecretChat(int32 secret_chat_id_)
  : secret_chat_id_(secret_chat_id_)
{}

const std::int32_t createSecretChat::ID;

object_ptr<createSecretChat> createSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createSecretChat> res = make_object<createSecretChat>();
  res->secret_chat_id_ = env->GetIntField(p, res->secret_chat_id_fieldID);
  return res;
}

void createSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateSecretChat");
    s.store_field("secretChatId", secret_chat_id_);
    s.store_class_end();
  }
}

createSecretChat::ReturnType createSecretChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateSecretChat").c_str());
    secret_chat_id_fieldID = jni::get_field_id(env, Class, "secretChatId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass createSupergroupChat::Class;
jfieldID createSupergroupChat::supergroup_id_fieldID;
jfieldID createSupergroupChat::force_fieldID;

createSupergroupChat::createSupergroupChat()
  : supergroup_id_()
  , force_()
{}

createSupergroupChat::createSupergroupChat(int53 supergroup_id_, bool force_)
  : supergroup_id_(supergroup_id_)
  , force_(force_)
{}

const std::int32_t createSupergroupChat::ID;

object_ptr<createSupergroupChat> createSupergroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createSupergroupChat> res = make_object<createSupergroupChat>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->force_ = (env->GetBooleanField(p, res->force_fieldID) != 0);
  return res;
}

void createSupergroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateSupergroupChat");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("force", force_);
    s.store_class_end();
  }
}

createSupergroupChat::ReturnType createSupergroupChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void createSupergroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateSupergroupChat").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    force_fieldID = jni::get_field_id(env, Class, "force", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass createTemporaryPassword::Class;
jfieldID createTemporaryPassword::password_fieldID;
jfieldID createTemporaryPassword::valid_for_fieldID;

createTemporaryPassword::createTemporaryPassword()
  : password_()
  , valid_for_()
{}

createTemporaryPassword::createTemporaryPassword(string const &password_, int32 valid_for_)
  : password_(password_)
  , valid_for_(valid_for_)
{}

const std::int32_t createTemporaryPassword::ID;

object_ptr<createTemporaryPassword> createTemporaryPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createTemporaryPassword> res = make_object<createTemporaryPassword>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  res->valid_for_ = env->GetIntField(p, res->valid_for_fieldID);
  return res;
}

void createTemporaryPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateTemporaryPassword");
    s.store_field("password", password_);
    s.store_field("validFor", valid_for_);
    s.store_class_end();
  }
}

createTemporaryPassword::ReturnType createTemporaryPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return temporaryPasswordState::fetch(env, p);
}

void createTemporaryPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateTemporaryPassword").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    valid_for_fieldID = jni::get_field_id(env, Class, "validFor", "I");
    return true;
  }();
  (void)is_inited;
}

jclass createVideoChat::Class;
jfieldID createVideoChat::chat_id_fieldID;
jfieldID createVideoChat::title_fieldID;
jfieldID createVideoChat::start_date_fieldID;
jfieldID createVideoChat::is_rtmp_stream_fieldID;

createVideoChat::createVideoChat()
  : chat_id_()
  , title_()
  , start_date_()
  , is_rtmp_stream_()
{}

createVideoChat::createVideoChat(int53 chat_id_, string const &title_, int32 start_date_, bool is_rtmp_stream_)
  : chat_id_(chat_id_)
  , title_(title_)
  , start_date_(start_date_)
  , is_rtmp_stream_(is_rtmp_stream_)
{}

const std::int32_t createVideoChat::ID;

object_ptr<createVideoChat> createVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<createVideoChat> res = make_object<createVideoChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->start_date_ = env->GetIntField(p, res->start_date_fieldID);
  res->is_rtmp_stream_ = (env->GetBooleanField(p, res->is_rtmp_stream_fieldID) != 0);
  return res;
}

void createVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "CreateVideoChat");
    s.store_field("chatId", chat_id_);
    s.store_field("title", title_);
    s.store_field("startDate", start_date_);
    s.store_field("isRtmpStream", is_rtmp_stream_);
    s.store_class_end();
  }
}

createVideoChat::ReturnType createVideoChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return groupCallId::fetch(env, p);
}

void createVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$CreateVideoChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    start_date_fieldID = jni::get_field_id(env, Class, "startDate", "I");
    is_rtmp_stream_fieldID = jni::get_field_id(env, Class, "isRtmpStream", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass declineGroupCallInvitation::Class;
jfieldID declineGroupCallInvitation::chat_id_fieldID;
jfieldID declineGroupCallInvitation::message_id_fieldID;

declineGroupCallInvitation::declineGroupCallInvitation()
  : chat_id_()
  , message_id_()
{}

declineGroupCallInvitation::declineGroupCallInvitation(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t declineGroupCallInvitation::ID;

object_ptr<declineGroupCallInvitation> declineGroupCallInvitation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<declineGroupCallInvitation> res = make_object<declineGroupCallInvitation>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void declineGroupCallInvitation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeclineGroupCallInvitation");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

declineGroupCallInvitation::ReturnType declineGroupCallInvitation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void declineGroupCallInvitation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeclineGroupCallInvitation").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass decryptGroupCallData::Class;
jfieldID decryptGroupCallData::group_call_id_fieldID;
jfieldID decryptGroupCallData::participant_id_fieldID;
jfieldID decryptGroupCallData::data_channel_fieldID;
jfieldID decryptGroupCallData::data_fieldID;

decryptGroupCallData::decryptGroupCallData()
  : group_call_id_()
  , participant_id_()
  , data_channel_()
  , data_()
{}

decryptGroupCallData::decryptGroupCallData(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, object_ptr<GroupCallDataChannel> &&data_channel_, bytes const &data_)
  : group_call_id_(group_call_id_)
  , participant_id_(std::move(participant_id_))
  , data_channel_(std::move(data_channel_))
  , data_(std::move(data_))
{}

const std::int32_t decryptGroupCallData::ID;

object_ptr<decryptGroupCallData> decryptGroupCallData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<decryptGroupCallData> res = make_object<decryptGroupCallData>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->data_channel_ = jni::fetch_tl_object<GroupCallDataChannel>(env, jni::fetch_object(env, p, res->data_channel_fieldID));
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void decryptGroupCallData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DecryptGroupCallData");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_object_field("dataChannel", static_cast<const BaseObject *>(data_channel_.get()));
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

decryptGroupCallData::ReturnType decryptGroupCallData::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return data::fetch(env, p);
}

void decryptGroupCallData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DecryptGroupCallData").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    data_channel_fieldID = jni::get_field_id(env, Class, "dataChannel", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallDataChannel;").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass deleteAccount::Class;
jfieldID deleteAccount::reason_fieldID;
jfieldID deleteAccount::password_fieldID;

deleteAccount::deleteAccount()
  : reason_()
  , password_()
{}

deleteAccount::deleteAccount(string const &reason_, string const &password_)
  : reason_(reason_)
  , password_(password_)
{}

const std::int32_t deleteAccount::ID;

object_ptr<deleteAccount> deleteAccount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteAccount> res = make_object<deleteAccount>();
  res->reason_ = jni::fetch_string(env, p, res->reason_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void deleteAccount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteAccount");
    s.store_field("reason", reason_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

deleteAccount::ReturnType deleteAccount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteAccount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteAccount").c_str());
    reason_fieldID = jni::get_field_id(env, Class, "reason", "Ljava/lang/String;");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteAllCallMessages::Class;
jfieldID deleteAllCallMessages::revoke_fieldID;

deleteAllCallMessages::deleteAllCallMessages()
  : revoke_()
{}

deleteAllCallMessages::deleteAllCallMessages(bool revoke_)
  : revoke_(revoke_)
{}

const std::int32_t deleteAllCallMessages::ID;

object_ptr<deleteAllCallMessages> deleteAllCallMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteAllCallMessages> res = make_object<deleteAllCallMessages>();
  res->revoke_ = (env->GetBooleanField(p, res->revoke_fieldID) != 0);
  return res;
}

void deleteAllCallMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteAllCallMessages");
    s.store_field("revoke", revoke_);
    s.store_class_end();
  }
}

deleteAllCallMessages::ReturnType deleteAllCallMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteAllCallMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteAllCallMessages").c_str());
    revoke_fieldID = jni::get_field_id(env, Class, "revoke", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deleteAllRevokedChatInviteLinks::Class;
jfieldID deleteAllRevokedChatInviteLinks::chat_id_fieldID;
jfieldID deleteAllRevokedChatInviteLinks::creator_user_id_fieldID;

deleteAllRevokedChatInviteLinks::deleteAllRevokedChatInviteLinks()
  : chat_id_()
  , creator_user_id_()
{}

deleteAllRevokedChatInviteLinks::deleteAllRevokedChatInviteLinks(int53 chat_id_, int53 creator_user_id_)
  : chat_id_(chat_id_)
  , creator_user_id_(creator_user_id_)
{}

const std::int32_t deleteAllRevokedChatInviteLinks::ID;

object_ptr<deleteAllRevokedChatInviteLinks> deleteAllRevokedChatInviteLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteAllRevokedChatInviteLinks> res = make_object<deleteAllRevokedChatInviteLinks>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->creator_user_id_ = env->GetLongField(p, res->creator_user_id_fieldID);
  return res;
}

void deleteAllRevokedChatInviteLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteAllRevokedChatInviteLinks");
    s.store_field("chatId", chat_id_);
    s.store_field("creatorUserId", creator_user_id_);
    s.store_class_end();
  }
}

deleteAllRevokedChatInviteLinks::ReturnType deleteAllRevokedChatInviteLinks::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteAllRevokedChatInviteLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteAllRevokedChatInviteLinks").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    creator_user_id_fieldID = jni::get_field_id(env, Class, "creatorUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteBotMediaPreviews::Class;
jfieldID deleteBotMediaPreviews::bot_user_id_fieldID;
jfieldID deleteBotMediaPreviews::language_code_fieldID;
jfieldID deleteBotMediaPreviews::file_ids_fieldID;

deleteBotMediaPreviews::deleteBotMediaPreviews()
  : bot_user_id_()
  , language_code_()
  , file_ids_()
{}

deleteBotMediaPreviews::deleteBotMediaPreviews(int53 bot_user_id_, string const &language_code_, array<int32> &&file_ids_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , file_ids_(std::move(file_ids_))
{}

const std::int32_t deleteBotMediaPreviews::ID;

object_ptr<deleteBotMediaPreviews> deleteBotMediaPreviews::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteBotMediaPreviews> res = make_object<deleteBotMediaPreviews>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->file_ids_fieldID));
  return res;
}

void deleteBotMediaPreviews::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteBotMediaPreviews");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    { s.store_vector_begin("fileIds", file_ids_.size()); for (const auto &_value : file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

deleteBotMediaPreviews::ReturnType deleteBotMediaPreviews::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteBotMediaPreviews::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteBotMediaPreviews").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    file_ids_fieldID = jni::get_field_id(env, Class, "fileIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass deleteBusinessChatLink::Class;
jfieldID deleteBusinessChatLink::link_fieldID;

deleteBusinessChatLink::deleteBusinessChatLink()
  : link_()
{}

deleteBusinessChatLink::deleteBusinessChatLink(string const &link_)
  : link_(link_)
{}

const std::int32_t deleteBusinessChatLink::ID;

object_ptr<deleteBusinessChatLink> deleteBusinessChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteBusinessChatLink> res = make_object<deleteBusinessChatLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return res;
}

void deleteBusinessChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteBusinessChatLink");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

deleteBusinessChatLink::ReturnType deleteBusinessChatLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteBusinessChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteBusinessChatLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteBusinessConnectedBot::Class;
jfieldID deleteBusinessConnectedBot::bot_user_id_fieldID;

deleteBusinessConnectedBot::deleteBusinessConnectedBot()
  : bot_user_id_()
{}

deleteBusinessConnectedBot::deleteBusinessConnectedBot(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t deleteBusinessConnectedBot::ID;

object_ptr<deleteBusinessConnectedBot> deleteBusinessConnectedBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteBusinessConnectedBot> res = make_object<deleteBusinessConnectedBot>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void deleteBusinessConnectedBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteBusinessConnectedBot");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

deleteBusinessConnectedBot::ReturnType deleteBusinessConnectedBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteBusinessConnectedBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteBusinessConnectedBot").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteBusinessMessages::Class;
jfieldID deleteBusinessMessages::business_connection_id_fieldID;
jfieldID deleteBusinessMessages::message_ids_fieldID;

deleteBusinessMessages::deleteBusinessMessages()
  : business_connection_id_()
  , message_ids_()
{}

deleteBusinessMessages::deleteBusinessMessages(string const &business_connection_id_, array<int53> &&message_ids_)
  : business_connection_id_(business_connection_id_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t deleteBusinessMessages::ID;

object_ptr<deleteBusinessMessages> deleteBusinessMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteBusinessMessages> res = make_object<deleteBusinessMessages>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return res;
}

void deleteBusinessMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteBusinessMessages");
    s.store_field("businessConnectionId", business_connection_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

deleteBusinessMessages::ReturnType deleteBusinessMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteBusinessMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteBusinessMessages").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteBusinessStory::Class;
jfieldID deleteBusinessStory::business_connection_id_fieldID;
jfieldID deleteBusinessStory::story_id_fieldID;

deleteBusinessStory::deleteBusinessStory()
  : business_connection_id_()
  , story_id_()
{}

deleteBusinessStory::deleteBusinessStory(string const &business_connection_id_, int32 story_id_)
  : business_connection_id_(business_connection_id_)
  , story_id_(story_id_)
{}

const std::int32_t deleteBusinessStory::ID;

object_ptr<deleteBusinessStory> deleteBusinessStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteBusinessStory> res = make_object<deleteBusinessStory>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return res;
}

void deleteBusinessStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteBusinessStory");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

deleteBusinessStory::ReturnType deleteBusinessStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteBusinessStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteBusinessStory").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChat::Class;
jfieldID deleteChat::chat_id_fieldID;

deleteChat::deleteChat()
  : chat_id_()
{}

deleteChat::deleteChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t deleteChat::ID;

object_ptr<deleteChat> deleteChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChat> res = make_object<deleteChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void deleteChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

deleteChat::ReturnType deleteChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatBackground::Class;
jfieldID deleteChatBackground::chat_id_fieldID;
jfieldID deleteChatBackground::restore_previous_fieldID;

deleteChatBackground::deleteChatBackground()
  : chat_id_()
  , restore_previous_()
{}

deleteChatBackground::deleteChatBackground(int53 chat_id_, bool restore_previous_)
  : chat_id_(chat_id_)
  , restore_previous_(restore_previous_)
{}

const std::int32_t deleteChatBackground::ID;

object_ptr<deleteChatBackground> deleteChatBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatBackground> res = make_object<deleteChatBackground>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->restore_previous_ = (env->GetBooleanField(p, res->restore_previous_fieldID) != 0);
  return res;
}

void deleteChatBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatBackground");
    s.store_field("chatId", chat_id_);
    s.store_field("restorePrevious", restore_previous_);
    s.store_class_end();
  }
}

deleteChatBackground::ReturnType deleteChatBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatBackground").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    restore_previous_fieldID = jni::get_field_id(env, Class, "restorePrevious", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatFolder::Class;
jfieldID deleteChatFolder::chat_folder_id_fieldID;
jfieldID deleteChatFolder::leave_chat_ids_fieldID;

deleteChatFolder::deleteChatFolder()
  : chat_folder_id_()
  , leave_chat_ids_()
{}

deleteChatFolder::deleteChatFolder(int32 chat_folder_id_, array<int53> &&leave_chat_ids_)
  : chat_folder_id_(chat_folder_id_)
  , leave_chat_ids_(std::move(leave_chat_ids_))
{}

const std::int32_t deleteChatFolder::ID;

object_ptr<deleteChatFolder> deleteChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatFolder> res = make_object<deleteChatFolder>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->leave_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->leave_chat_ids_fieldID));
  return res;
}

void deleteChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatFolder");
    s.store_field("chatFolderId", chat_folder_id_);
    { s.store_vector_begin("leaveChatIds", leave_chat_ids_.size()); for (const auto &_value : leave_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

deleteChatFolder::ReturnType deleteChatFolder::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatFolder").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    leave_chat_ids_fieldID = jni::get_field_id(env, Class, "leaveChatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatFolderInviteLink::Class;
jfieldID deleteChatFolderInviteLink::chat_folder_id_fieldID;
jfieldID deleteChatFolderInviteLink::invite_link_fieldID;

deleteChatFolderInviteLink::deleteChatFolderInviteLink()
  : chat_folder_id_()
  , invite_link_()
{}

deleteChatFolderInviteLink::deleteChatFolderInviteLink(int32 chat_folder_id_, string const &invite_link_)
  : chat_folder_id_(chat_folder_id_)
  , invite_link_(invite_link_)
{}

const std::int32_t deleteChatFolderInviteLink::ID;

object_ptr<deleteChatFolderInviteLink> deleteChatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatFolderInviteLink> res = make_object<deleteChatFolderInviteLink>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void deleteChatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatFolderInviteLink");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

deleteChatFolderInviteLink::ReturnType deleteChatFolderInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatFolderInviteLink").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatHistory::Class;
jfieldID deleteChatHistory::chat_id_fieldID;
jfieldID deleteChatHistory::remove_from_chat_list_fieldID;
jfieldID deleteChatHistory::revoke_fieldID;

deleteChatHistory::deleteChatHistory()
  : chat_id_()
  , remove_from_chat_list_()
  , revoke_()
{}

deleteChatHistory::deleteChatHistory(int53 chat_id_, bool remove_from_chat_list_, bool revoke_)
  : chat_id_(chat_id_)
  , remove_from_chat_list_(remove_from_chat_list_)
  , revoke_(revoke_)
{}

const std::int32_t deleteChatHistory::ID;

object_ptr<deleteChatHistory> deleteChatHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatHistory> res = make_object<deleteChatHistory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->remove_from_chat_list_ = (env->GetBooleanField(p, res->remove_from_chat_list_fieldID) != 0);
  res->revoke_ = (env->GetBooleanField(p, res->revoke_fieldID) != 0);
  return res;
}

void deleteChatHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatHistory");
    s.store_field("chatId", chat_id_);
    s.store_field("removeFromChatList", remove_from_chat_list_);
    s.store_field("revoke", revoke_);
    s.store_class_end();
  }
}

deleteChatHistory::ReturnType deleteChatHistory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatHistory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    remove_from_chat_list_fieldID = jni::get_field_id(env, Class, "removeFromChatList", "Z");
    revoke_fieldID = jni::get_field_id(env, Class, "revoke", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatMessagesByDate::Class;
jfieldID deleteChatMessagesByDate::chat_id_fieldID;
jfieldID deleteChatMessagesByDate::min_date_fieldID;
jfieldID deleteChatMessagesByDate::max_date_fieldID;
jfieldID deleteChatMessagesByDate::revoke_fieldID;

deleteChatMessagesByDate::deleteChatMessagesByDate()
  : chat_id_()
  , min_date_()
  , max_date_()
  , revoke_()
{}

deleteChatMessagesByDate::deleteChatMessagesByDate(int53 chat_id_, int32 min_date_, int32 max_date_, bool revoke_)
  : chat_id_(chat_id_)
  , min_date_(min_date_)
  , max_date_(max_date_)
  , revoke_(revoke_)
{}

const std::int32_t deleteChatMessagesByDate::ID;

object_ptr<deleteChatMessagesByDate> deleteChatMessagesByDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatMessagesByDate> res = make_object<deleteChatMessagesByDate>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->min_date_ = env->GetIntField(p, res->min_date_fieldID);
  res->max_date_ = env->GetIntField(p, res->max_date_fieldID);
  res->revoke_ = (env->GetBooleanField(p, res->revoke_fieldID) != 0);
  return res;
}

void deleteChatMessagesByDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatMessagesByDate");
    s.store_field("chatId", chat_id_);
    s.store_field("minDate", min_date_);
    s.store_field("maxDate", max_date_);
    s.store_field("revoke", revoke_);
    s.store_class_end();
  }
}

deleteChatMessagesByDate::ReturnType deleteChatMessagesByDate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatMessagesByDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatMessagesByDate").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    min_date_fieldID = jni::get_field_id(env, Class, "minDate", "I");
    max_date_fieldID = jni::get_field_id(env, Class, "maxDate", "I");
    revoke_fieldID = jni::get_field_id(env, Class, "revoke", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatMessagesBySender::Class;
jfieldID deleteChatMessagesBySender::chat_id_fieldID;
jfieldID deleteChatMessagesBySender::sender_id_fieldID;

deleteChatMessagesBySender::deleteChatMessagesBySender()
  : chat_id_()
  , sender_id_()
{}

deleteChatMessagesBySender::deleteChatMessagesBySender(int53 chat_id_, object_ptr<MessageSender> &&sender_id_)
  : chat_id_(chat_id_)
  , sender_id_(std::move(sender_id_))
{}

const std::int32_t deleteChatMessagesBySender::ID;

object_ptr<deleteChatMessagesBySender> deleteChatMessagesBySender::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatMessagesBySender> res = make_object<deleteChatMessagesBySender>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  return res;
}

void deleteChatMessagesBySender::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatMessagesBySender");
    s.store_field("chatId", chat_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_class_end();
  }
}

deleteChatMessagesBySender::ReturnType deleteChatMessagesBySender::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatMessagesBySender::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatMessagesBySender").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass deleteChatReplyMarkup::Class;
jfieldID deleteChatReplyMarkup::chat_id_fieldID;
jfieldID deleteChatReplyMarkup::message_id_fieldID;

deleteChatReplyMarkup::deleteChatReplyMarkup()
  : chat_id_()
  , message_id_()
{}

deleteChatReplyMarkup::deleteChatReplyMarkup(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t deleteChatReplyMarkup::ID;

object_ptr<deleteChatReplyMarkup> deleteChatReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteChatReplyMarkup> res = make_object<deleteChatReplyMarkup>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void deleteChatReplyMarkup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteChatReplyMarkup");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

deleteChatReplyMarkup::ReturnType deleteChatReplyMarkup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteChatReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteChatReplyMarkup").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteCommands::Class;
jfieldID deleteCommands::scope_fieldID;
jfieldID deleteCommands::language_code_fieldID;

deleteCommands::deleteCommands()
  : scope_()
  , language_code_()
{}

deleteCommands::deleteCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_)
  : scope_(std::move(scope_))
  , language_code_(language_code_)
{}

const std::int32_t deleteCommands::ID;

object_ptr<deleteCommands> deleteCommands::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteCommands> res = make_object<deleteCommands>();
  res->scope_ = jni::fetch_tl_object<BotCommandScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void deleteCommands::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteCommands");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

deleteCommands::ReturnType deleteCommands::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteCommands::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteCommands").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotCommandScope;").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteDefaultBackground::Class;
jfieldID deleteDefaultBackground::for_dark_theme_fieldID;

deleteDefaultBackground::deleteDefaultBackground()
  : for_dark_theme_()
{}

deleteDefaultBackground::deleteDefaultBackground(bool for_dark_theme_)
  : for_dark_theme_(for_dark_theme_)
{}

const std::int32_t deleteDefaultBackground::ID;

object_ptr<deleteDefaultBackground> deleteDefaultBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteDefaultBackground> res = make_object<deleteDefaultBackground>();
  res->for_dark_theme_ = (env->GetBooleanField(p, res->for_dark_theme_fieldID) != 0);
  return res;
}

void deleteDefaultBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteDefaultBackground");
    s.store_field("forDarkTheme", for_dark_theme_);
    s.store_class_end();
  }
}

deleteDefaultBackground::ReturnType deleteDefaultBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteDefaultBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteDefaultBackground").c_str());
    for_dark_theme_fieldID = jni::get_field_id(env, Class, "forDarkTheme", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deleteFile::Class;
jfieldID deleteFile::file_id_fieldID;

deleteFile::deleteFile()
  : file_id_()
{}

deleteFile::deleteFile(int32 file_id_)
  : file_id_(file_id_)
{}

const std::int32_t deleteFile::ID;

object_ptr<deleteFile> deleteFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteFile> res = make_object<deleteFile>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  return res;
}

void deleteFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteFile");
    s.store_field("fileId", file_id_);
    s.store_class_end();
  }
}

deleteFile::ReturnType deleteFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteFile").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass deleteForumTopic::Class;
jfieldID deleteForumTopic::chat_id_fieldID;
jfieldID deleteForumTopic::message_thread_id_fieldID;

deleteForumTopic::deleteForumTopic()
  : chat_id_()
  , message_thread_id_()
{}

deleteForumTopic::deleteForumTopic(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t deleteForumTopic::ID;

object_ptr<deleteForumTopic> deleteForumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteForumTopic> res = make_object<deleteForumTopic>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void deleteForumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteForumTopic");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

deleteForumTopic::ReturnType deleteForumTopic::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteForumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteForumTopic").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteLanguagePack::Class;
jfieldID deleteLanguagePack::language_pack_id_fieldID;

deleteLanguagePack::deleteLanguagePack()
  : language_pack_id_()
{}

deleteLanguagePack::deleteLanguagePack(string const &language_pack_id_)
  : language_pack_id_(language_pack_id_)
{}

const std::int32_t deleteLanguagePack::ID;

object_ptr<deleteLanguagePack> deleteLanguagePack::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteLanguagePack> res = make_object<deleteLanguagePack>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  return res;
}

void deleteLanguagePack::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteLanguagePack");
    s.store_field("languagePackId", language_pack_id_);
    s.store_class_end();
  }
}

deleteLanguagePack::ReturnType deleteLanguagePack::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteLanguagePack::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteLanguagePack").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteMessages::Class;
jfieldID deleteMessages::chat_id_fieldID;
jfieldID deleteMessages::message_ids_fieldID;
jfieldID deleteMessages::revoke_fieldID;

deleteMessages::deleteMessages()
  : chat_id_()
  , message_ids_()
  , revoke_()
{}

deleteMessages::deleteMessages(int53 chat_id_, array<int53> &&message_ids_, bool revoke_)
  : chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
  , revoke_(revoke_)
{}

const std::int32_t deleteMessages::ID;

object_ptr<deleteMessages> deleteMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteMessages> res = make_object<deleteMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->revoke_ = (env->GetBooleanField(p, res->revoke_fieldID) != 0);
  return res;
}

void deleteMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteMessages");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("revoke", revoke_);
    s.store_class_end();
  }
}

deleteMessages::ReturnType deleteMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    revoke_fieldID = jni::get_field_id(env, Class, "revoke", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass deletePassportElement::Class;
jfieldID deletePassportElement::type_fieldID;

deletePassportElement::deletePassportElement()
  : type_()
{}

deletePassportElement::deletePassportElement(object_ptr<PassportElementType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t deletePassportElement::ID;

object_ptr<deletePassportElement> deletePassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deletePassportElement> res = make_object<deletePassportElement>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void deletePassportElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeletePassportElement");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

deletePassportElement::ReturnType deletePassportElement::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deletePassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeletePassportElement").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass deleteProfilePhoto::Class;
jfieldID deleteProfilePhoto::profile_photo_id_fieldID;

deleteProfilePhoto::deleteProfilePhoto()
  : profile_photo_id_()
{}

deleteProfilePhoto::deleteProfilePhoto(int64 profile_photo_id_)
  : profile_photo_id_(profile_photo_id_)
{}

const std::int32_t deleteProfilePhoto::ID;

object_ptr<deleteProfilePhoto> deleteProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteProfilePhoto> res = make_object<deleteProfilePhoto>();
  res->profile_photo_id_ = env->GetLongField(p, res->profile_photo_id_fieldID);
  return res;
}

void deleteProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteProfilePhoto");
    s.store_field("profilePhotoId", profile_photo_id_);
    s.store_class_end();
  }
}

deleteProfilePhoto::ReturnType deleteProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteProfilePhoto").c_str());
    profile_photo_id_fieldID = jni::get_field_id(env, Class, "profilePhotoId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteQuickReplyShortcut::Class;
jfieldID deleteQuickReplyShortcut::shortcut_id_fieldID;

deleteQuickReplyShortcut::deleteQuickReplyShortcut()
  : shortcut_id_()
{}

deleteQuickReplyShortcut::deleteQuickReplyShortcut(int32 shortcut_id_)
  : shortcut_id_(shortcut_id_)
{}

const std::int32_t deleteQuickReplyShortcut::ID;

object_ptr<deleteQuickReplyShortcut> deleteQuickReplyShortcut::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteQuickReplyShortcut> res = make_object<deleteQuickReplyShortcut>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  return res;
}

void deleteQuickReplyShortcut::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteQuickReplyShortcut");
    s.store_field("shortcutId", shortcut_id_);
    s.store_class_end();
  }
}

deleteQuickReplyShortcut::ReturnType deleteQuickReplyShortcut::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteQuickReplyShortcut::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteQuickReplyShortcut").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass deleteQuickReplyShortcutMessages::Class;
jfieldID deleteQuickReplyShortcutMessages::shortcut_id_fieldID;
jfieldID deleteQuickReplyShortcutMessages::message_ids_fieldID;

deleteQuickReplyShortcutMessages::deleteQuickReplyShortcutMessages()
  : shortcut_id_()
  , message_ids_()
{}

deleteQuickReplyShortcutMessages::deleteQuickReplyShortcutMessages(int32 shortcut_id_, array<int53> &&message_ids_)
  : shortcut_id_(shortcut_id_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t deleteQuickReplyShortcutMessages::ID;

object_ptr<deleteQuickReplyShortcutMessages> deleteQuickReplyShortcutMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteQuickReplyShortcutMessages> res = make_object<deleteQuickReplyShortcutMessages>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return res;
}

void deleteQuickReplyShortcutMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteQuickReplyShortcutMessages");
    s.store_field("shortcutId", shortcut_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

deleteQuickReplyShortcutMessages::ReturnType deleteQuickReplyShortcutMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteQuickReplyShortcutMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteQuickReplyShortcutMessages").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteRevokedChatInviteLink::Class;
jfieldID deleteRevokedChatInviteLink::chat_id_fieldID;
jfieldID deleteRevokedChatInviteLink::invite_link_fieldID;

deleteRevokedChatInviteLink::deleteRevokedChatInviteLink()
  : chat_id_()
  , invite_link_()
{}

deleteRevokedChatInviteLink::deleteRevokedChatInviteLink(int53 chat_id_, string const &invite_link_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
{}

const std::int32_t deleteRevokedChatInviteLink::ID;

object_ptr<deleteRevokedChatInviteLink> deleteRevokedChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteRevokedChatInviteLink> res = make_object<deleteRevokedChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void deleteRevokedChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteRevokedChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

deleteRevokedChatInviteLink::ReturnType deleteRevokedChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteRevokedChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteRevokedChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteSavedCredentials::Class;

deleteSavedCredentials::deleteSavedCredentials() {
}

const std::int32_t deleteSavedCredentials::ID;

object_ptr<deleteSavedCredentials> deleteSavedCredentials::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<deleteSavedCredentials>();
}

void deleteSavedCredentials::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteSavedCredentials");
    s.store_class_end();
  }
}

deleteSavedCredentials::ReturnType deleteSavedCredentials::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteSavedCredentials::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteSavedCredentials").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass deleteSavedMessagesTopicHistory::Class;
jfieldID deleteSavedMessagesTopicHistory::saved_messages_topic_id_fieldID;

deleteSavedMessagesTopicHistory::deleteSavedMessagesTopicHistory()
  : saved_messages_topic_id_()
{}

deleteSavedMessagesTopicHistory::deleteSavedMessagesTopicHistory(int53 saved_messages_topic_id_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t deleteSavedMessagesTopicHistory::ID;

object_ptr<deleteSavedMessagesTopicHistory> deleteSavedMessagesTopicHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteSavedMessagesTopicHistory> res = make_object<deleteSavedMessagesTopicHistory>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void deleteSavedMessagesTopicHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteSavedMessagesTopicHistory");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

deleteSavedMessagesTopicHistory::ReturnType deleteSavedMessagesTopicHistory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteSavedMessagesTopicHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteSavedMessagesTopicHistory").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass deleteSavedMessagesTopicMessagesByDate::Class;
jfieldID deleteSavedMessagesTopicMessagesByDate::saved_messages_topic_id_fieldID;
jfieldID deleteSavedMessagesTopicMessagesByDate::min_date_fieldID;
jfieldID deleteSavedMessagesTopicMessagesByDate::max_date_fieldID;

deleteSavedMessagesTopicMessagesByDate::deleteSavedMessagesTopicMessagesByDate()
  : saved_messages_topic_id_()
  , min_date_()
  , max_date_()
{}

deleteSavedMessagesTopicMessagesByDate::deleteSavedMessagesTopicMessagesByDate(int53 saved_messages_topic_id_, int32 min_date_, int32 max_date_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , min_date_(min_date_)
  , max_date_(max_date_)
{}

const std::int32_t deleteSavedMessagesTopicMessagesByDate::ID;

object_ptr<deleteSavedMessagesTopicMessagesByDate> deleteSavedMessagesTopicMessagesByDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteSavedMessagesTopicMessagesByDate> res = make_object<deleteSavedMessagesTopicMessagesByDate>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->min_date_ = env->GetIntField(p, res->min_date_fieldID);
  res->max_date_ = env->GetIntField(p, res->max_date_fieldID);
  return res;
}

void deleteSavedMessagesTopicMessagesByDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteSavedMessagesTopicMessagesByDate");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_field("minDate", min_date_);
    s.store_field("maxDate", max_date_);
    s.store_class_end();
  }
}

deleteSavedMessagesTopicMessagesByDate::ReturnType deleteSavedMessagesTopicMessagesByDate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteSavedMessagesTopicMessagesByDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteSavedMessagesTopicMessagesByDate").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    min_date_fieldID = jni::get_field_id(env, Class, "minDate", "I");
    max_date_fieldID = jni::get_field_id(env, Class, "maxDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass deleteSavedOrderInfo::Class;

deleteSavedOrderInfo::deleteSavedOrderInfo() {
}

const std::int32_t deleteSavedOrderInfo::ID;

object_ptr<deleteSavedOrderInfo> deleteSavedOrderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<deleteSavedOrderInfo>();
}

void deleteSavedOrderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteSavedOrderInfo");
    s.store_class_end();
  }
}

deleteSavedOrderInfo::ReturnType deleteSavedOrderInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteSavedOrderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteSavedOrderInfo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass deleteStickerSet::Class;
jfieldID deleteStickerSet::name_fieldID;

deleteStickerSet::deleteStickerSet()
  : name_()
{}

deleteStickerSet::deleteStickerSet(string const &name_)
  : name_(name_)
{}

const std::int32_t deleteStickerSet::ID;

object_ptr<deleteStickerSet> deleteStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteStickerSet> res = make_object<deleteStickerSet>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void deleteStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteStickerSet");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

deleteStickerSet::ReturnType deleteStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteStickerSet").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass deleteStory::Class;
jfieldID deleteStory::story_poster_chat_id_fieldID;
jfieldID deleteStory::story_id_fieldID;

deleteStory::deleteStory()
  : story_poster_chat_id_()
  , story_id_()
{}

deleteStory::deleteStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t deleteStory::ID;

object_ptr<deleteStory> deleteStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<deleteStory> res = make_object<deleteStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return res;
}

void deleteStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DeleteStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

deleteStory::ReturnType deleteStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void deleteStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DeleteStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass destroy::Class;

destroy::destroy() {
}

const std::int32_t destroy::ID;

object_ptr<destroy> destroy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<destroy>();
}

void destroy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "Destroy");
    s.store_class_end();
  }
}

destroy::ReturnType destroy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void destroy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$Destroy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass disableAllSupergroupUsernames::Class;
jfieldID disableAllSupergroupUsernames::supergroup_id_fieldID;

disableAllSupergroupUsernames::disableAllSupergroupUsernames()
  : supergroup_id_()
{}

disableAllSupergroupUsernames::disableAllSupergroupUsernames(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t disableAllSupergroupUsernames::ID;

object_ptr<disableAllSupergroupUsernames> disableAllSupergroupUsernames::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<disableAllSupergroupUsernames> res = make_object<disableAllSupergroupUsernames>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return res;
}

void disableAllSupergroupUsernames::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DisableAllSupergroupUsernames");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

disableAllSupergroupUsernames::ReturnType disableAllSupergroupUsernames::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void disableAllSupergroupUsernames::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DisableAllSupergroupUsernames").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass disableProxy::Class;

disableProxy::disableProxy() {
}

const std::int32_t disableProxy::ID;

object_ptr<disableProxy> disableProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<disableProxy>();
}

void disableProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DisableProxy");
    s.store_class_end();
  }
}

disableProxy::ReturnType disableProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void disableProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DisableProxy").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass discardCall::Class;
jfieldID discardCall::call_id_fieldID;
jfieldID discardCall::is_disconnected_fieldID;
jfieldID discardCall::invite_link_fieldID;
jfieldID discardCall::duration_fieldID;
jfieldID discardCall::is_video_fieldID;
jfieldID discardCall::connection_id_fieldID;

discardCall::discardCall()
  : call_id_()
  , is_disconnected_()
  , invite_link_()
  , duration_()
  , is_video_()
  , connection_id_()
{}

discardCall::discardCall(int32 call_id_, bool is_disconnected_, string const &invite_link_, int32 duration_, bool is_video_, int64 connection_id_)
  : call_id_(call_id_)
  , is_disconnected_(is_disconnected_)
  , invite_link_(invite_link_)
  , duration_(duration_)
  , is_video_(is_video_)
  , connection_id_(connection_id_)
{}

const std::int32_t discardCall::ID;

object_ptr<discardCall> discardCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<discardCall> res = make_object<discardCall>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->is_disconnected_ = (env->GetBooleanField(p, res->is_disconnected_fieldID) != 0);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->duration_ = env->GetIntField(p, res->duration_fieldID);
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  res->connection_id_ = env->GetLongField(p, res->connection_id_fieldID);
  return res;
}

void discardCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DiscardCall");
    s.store_field("callId", call_id_);
    s.store_field("isDisconnected", is_disconnected_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("duration", duration_);
    s.store_field("isVideo", is_video_);
    s.store_field("connectionId", connection_id_);
    s.store_class_end();
  }
}

discardCall::ReturnType discardCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void discardCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DiscardCall").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    is_disconnected_fieldID = jni::get_field_id(env, Class, "isDisconnected", "Z");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    duration_fieldID = jni::get_field_id(env, Class, "duration", "I");
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass disconnectAffiliateProgram::Class;
jfieldID disconnectAffiliateProgram::affiliate_fieldID;
jfieldID disconnectAffiliateProgram::url_fieldID;

disconnectAffiliateProgram::disconnectAffiliateProgram()
  : affiliate_()
  , url_()
{}

disconnectAffiliateProgram::disconnectAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, string const &url_)
  : affiliate_(std::move(affiliate_))
  , url_(url_)
{}

const std::int32_t disconnectAffiliateProgram::ID;

object_ptr<disconnectAffiliateProgram> disconnectAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<disconnectAffiliateProgram> res = make_object<disconnectAffiliateProgram>();
  res->affiliate_ = jni::fetch_tl_object<AffiliateType>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void disconnectAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DisconnectAffiliateProgram");
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_field("url", url_);
    s.store_class_end();
  }
}

disconnectAffiliateProgram::ReturnType disconnectAffiliateProgram::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return connectedAffiliateProgram::fetch(env, p);
}

void disconnectAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DisconnectAffiliateProgram").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateType;").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass disconnectAllWebsites::Class;

disconnectAllWebsites::disconnectAllWebsites() {
}

const std::int32_t disconnectAllWebsites::ID;

object_ptr<disconnectAllWebsites> disconnectAllWebsites::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<disconnectAllWebsites>();
}

void disconnectAllWebsites::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DisconnectAllWebsites");
    s.store_class_end();
  }
}

disconnectAllWebsites::ReturnType disconnectAllWebsites::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void disconnectAllWebsites::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DisconnectAllWebsites").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass disconnectWebsite::Class;
jfieldID disconnectWebsite::website_id_fieldID;

disconnectWebsite::disconnectWebsite()
  : website_id_()
{}

disconnectWebsite::disconnectWebsite(int64 website_id_)
  : website_id_(website_id_)
{}

const std::int32_t disconnectWebsite::ID;

object_ptr<disconnectWebsite> disconnectWebsite::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<disconnectWebsite> res = make_object<disconnectWebsite>();
  res->website_id_ = env->GetLongField(p, res->website_id_fieldID);
  return res;
}

void disconnectWebsite::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DisconnectWebsite");
    s.store_field("websiteId", website_id_);
    s.store_class_end();
  }
}

disconnectWebsite::ReturnType disconnectWebsite::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void disconnectWebsite::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DisconnectWebsite").c_str());
    website_id_fieldID = jni::get_field_id(env, Class, "websiteId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass downloadFile::Class;
jfieldID downloadFile::file_id_fieldID;
jfieldID downloadFile::priority_fieldID;
jfieldID downloadFile::offset_fieldID;
jfieldID downloadFile::limit_fieldID;
jfieldID downloadFile::synchronous_fieldID;

downloadFile::downloadFile()
  : file_id_()
  , priority_()
  , offset_()
  , limit_()
  , synchronous_()
{}

downloadFile::downloadFile(int32 file_id_, int32 priority_, int53 offset_, int53 limit_, bool synchronous_)
  : file_id_(file_id_)
  , priority_(priority_)
  , offset_(offset_)
  , limit_(limit_)
  , synchronous_(synchronous_)
{}

const std::int32_t downloadFile::ID;

object_ptr<downloadFile> downloadFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<downloadFile> res = make_object<downloadFile>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->priority_ = env->GetIntField(p, res->priority_fieldID);
  res->offset_ = env->GetLongField(p, res->offset_fieldID);
  res->limit_ = env->GetLongField(p, res->limit_fieldID);
  res->synchronous_ = (env->GetBooleanField(p, res->synchronous_fieldID) != 0);
  return res;
}

void downloadFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "DownloadFile");
    s.store_field("fileId", file_id_);
    s.store_field("priority", priority_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_field("synchronous", synchronous_);
    s.store_class_end();
  }
}

downloadFile::ReturnType downloadFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void downloadFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$DownloadFile").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    priority_fieldID = jni::get_field_id(env, Class, "priority", "I");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "J");
    synchronous_fieldID = jni::get_field_id(env, Class, "synchronous", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editBotMediaPreview::Class;
jfieldID editBotMediaPreview::bot_user_id_fieldID;
jfieldID editBotMediaPreview::language_code_fieldID;
jfieldID editBotMediaPreview::file_id_fieldID;
jfieldID editBotMediaPreview::content_fieldID;

editBotMediaPreview::editBotMediaPreview()
  : bot_user_id_()
  , language_code_()
  , file_id_()
  , content_()
{}

editBotMediaPreview::editBotMediaPreview(int53 bot_user_id_, string const &language_code_, int32 file_id_, object_ptr<InputStoryContent> &&content_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , file_id_(file_id_)
  , content_(std::move(content_))
{}

const std::int32_t editBotMediaPreview::ID;

object_ptr<editBotMediaPreview> editBotMediaPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBotMediaPreview> res = make_object<editBotMediaPreview>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->content_ = jni::fetch_tl_object<InputStoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  return res;
}

void editBotMediaPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBotMediaPreview");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_field("fileId", file_id_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_class_end();
  }
}

editBotMediaPreview::ReturnType editBotMediaPreview::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botMediaPreview::fetch(env, p);
}

void editBotMediaPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBotMediaPreview").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessChatLink::Class;
jfieldID editBusinessChatLink::link_fieldID;
jfieldID editBusinessChatLink::link_info_fieldID;

editBusinessChatLink::editBusinessChatLink()
  : link_()
  , link_info_()
{}

editBusinessChatLink::editBusinessChatLink(string const &link_, object_ptr<inputBusinessChatLink> &&link_info_)
  : link_(link_)
  , link_info_(std::move(link_info_))
{}

const std::int32_t editBusinessChatLink::ID;

object_ptr<editBusinessChatLink> editBusinessChatLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessChatLink> res = make_object<editBusinessChatLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  res->link_info_ = jni::fetch_tl_object<inputBusinessChatLink>(env, jni::fetch_object(env, p, res->link_info_fieldID));
  return res;
}

void editBusinessChatLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessChatLink");
    s.store_field("link", link_);
    s.store_object_field("linkInfo", static_cast<const BaseObject *>(link_info_.get()));
    s.store_class_end();
  }
}

editBusinessChatLink::ReturnType editBusinessChatLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessChatLink::fetch(env, p);
}

void editBusinessChatLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessChatLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    link_info_fieldID = jni::get_field_id(env, Class, "linkInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputBusinessChatLink;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessMessageCaption::Class;
jfieldID editBusinessMessageCaption::business_connection_id_fieldID;
jfieldID editBusinessMessageCaption::chat_id_fieldID;
jfieldID editBusinessMessageCaption::message_id_fieldID;
jfieldID editBusinessMessageCaption::reply_markup_fieldID;
jfieldID editBusinessMessageCaption::caption_fieldID;
jfieldID editBusinessMessageCaption::show_caption_above_media_fieldID;

editBusinessMessageCaption::editBusinessMessageCaption()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
  , caption_()
  , show_caption_above_media_()
{}

editBusinessMessageCaption::editBusinessMessageCaption(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
{}

const std::int32_t editBusinessMessageCaption::ID;

object_ptr<editBusinessMessageCaption> editBusinessMessageCaption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessMessageCaption> res = make_object<editBusinessMessageCaption>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  return res;
}

void editBusinessMessageCaption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessMessageCaption");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_class_end();
  }
}

editBusinessMessageCaption::ReturnType editBusinessMessageCaption::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void editBusinessMessageCaption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessMessageCaption").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessMessageLiveLocation::Class;
jfieldID editBusinessMessageLiveLocation::business_connection_id_fieldID;
jfieldID editBusinessMessageLiveLocation::chat_id_fieldID;
jfieldID editBusinessMessageLiveLocation::message_id_fieldID;
jfieldID editBusinessMessageLiveLocation::reply_markup_fieldID;
jfieldID editBusinessMessageLiveLocation::location_fieldID;
jfieldID editBusinessMessageLiveLocation::live_period_fieldID;
jfieldID editBusinessMessageLiveLocation::heading_fieldID;
jfieldID editBusinessMessageLiveLocation::proximity_alert_radius_fieldID;

editBusinessMessageLiveLocation::editBusinessMessageLiveLocation()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
  , location_()
  , live_period_()
  , heading_()
  , proximity_alert_radius_()
{}

editBusinessMessageLiveLocation::editBusinessMessageLiveLocation(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , location_(std::move(location_))
  , live_period_(live_period_)
  , heading_(heading_)
  , proximity_alert_radius_(proximity_alert_radius_)
{}

const std::int32_t editBusinessMessageLiveLocation::ID;

object_ptr<editBusinessMessageLiveLocation> editBusinessMessageLiveLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessMessageLiveLocation> res = make_object<editBusinessMessageLiveLocation>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->heading_ = env->GetIntField(p, res->heading_fieldID);
  res->proximity_alert_radius_ = env->GetIntField(p, res->proximity_alert_radius_fieldID);
  return res;
}

void editBusinessMessageLiveLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessMessageLiveLocation");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("heading", heading_);
    s.store_field("proximityAlertRadius", proximity_alert_radius_);
    s.store_class_end();
  }
}

editBusinessMessageLiveLocation::ReturnType editBusinessMessageLiveLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void editBusinessMessageLiveLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessMessageLiveLocation").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    heading_fieldID = jni::get_field_id(env, Class, "heading", "I");
    proximity_alert_radius_fieldID = jni::get_field_id(env, Class, "proximityAlertRadius", "I");
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessMessageMedia::Class;
jfieldID editBusinessMessageMedia::business_connection_id_fieldID;
jfieldID editBusinessMessageMedia::chat_id_fieldID;
jfieldID editBusinessMessageMedia::message_id_fieldID;
jfieldID editBusinessMessageMedia::reply_markup_fieldID;
jfieldID editBusinessMessageMedia::input_message_content_fieldID;

editBusinessMessageMedia::editBusinessMessageMedia()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editBusinessMessageMedia::editBusinessMessageMedia(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editBusinessMessageMedia::ID;

object_ptr<editBusinessMessageMedia> editBusinessMessageMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessMessageMedia> res = make_object<editBusinessMessageMedia>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editBusinessMessageMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessMessageMedia");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editBusinessMessageMedia::ReturnType editBusinessMessageMedia::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void editBusinessMessageMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessMessageMedia").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessMessageReplyMarkup::Class;
jfieldID editBusinessMessageReplyMarkup::business_connection_id_fieldID;
jfieldID editBusinessMessageReplyMarkup::chat_id_fieldID;
jfieldID editBusinessMessageReplyMarkup::message_id_fieldID;
jfieldID editBusinessMessageReplyMarkup::reply_markup_fieldID;

editBusinessMessageReplyMarkup::editBusinessMessageReplyMarkup()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
{}

editBusinessMessageReplyMarkup::editBusinessMessageReplyMarkup(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t editBusinessMessageReplyMarkup::ID;

object_ptr<editBusinessMessageReplyMarkup> editBusinessMessageReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessMessageReplyMarkup> res = make_object<editBusinessMessageReplyMarkup>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void editBusinessMessageReplyMarkup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessMessageReplyMarkup");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

editBusinessMessageReplyMarkup::ReturnType editBusinessMessageReplyMarkup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void editBusinessMessageReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessMessageReplyMarkup").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessMessageText::Class;
jfieldID editBusinessMessageText::business_connection_id_fieldID;
jfieldID editBusinessMessageText::chat_id_fieldID;
jfieldID editBusinessMessageText::message_id_fieldID;
jfieldID editBusinessMessageText::reply_markup_fieldID;
jfieldID editBusinessMessageText::input_message_content_fieldID;

editBusinessMessageText::editBusinessMessageText()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editBusinessMessageText::editBusinessMessageText(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editBusinessMessageText::ID;

object_ptr<editBusinessMessageText> editBusinessMessageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessMessageText> res = make_object<editBusinessMessageText>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editBusinessMessageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessMessageText");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editBusinessMessageText::ReturnType editBusinessMessageText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void editBusinessMessageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessMessageText").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editBusinessStory::Class;
jfieldID editBusinessStory::story_poster_chat_id_fieldID;
jfieldID editBusinessStory::story_id_fieldID;
jfieldID editBusinessStory::content_fieldID;
jfieldID editBusinessStory::areas_fieldID;
jfieldID editBusinessStory::caption_fieldID;
jfieldID editBusinessStory::privacy_settings_fieldID;

editBusinessStory::editBusinessStory()
  : story_poster_chat_id_()
  , story_id_()
  , content_()
  , areas_()
  , caption_()
  , privacy_settings_()
{}

editBusinessStory::editBusinessStory(int53 story_poster_chat_id_, int32 story_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_, object_ptr<StoryPrivacySettings> &&privacy_settings_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , content_(std::move(content_))
  , areas_(std::move(areas_))
  , caption_(std::move(caption_))
  , privacy_settings_(std::move(privacy_settings_))
{}

const std::int32_t editBusinessStory::ID;

object_ptr<editBusinessStory> editBusinessStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editBusinessStory> res = make_object<editBusinessStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->content_ = jni::fetch_tl_object<InputStoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->areas_ = jni::fetch_tl_object<inputStoryAreas>(env, jni::fetch_object(env, p, res->areas_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->privacy_settings_ = jni::fetch_tl_object<StoryPrivacySettings>(env, jni::fetch_object(env, p, res->privacy_settings_fieldID));
  return res;
}

void editBusinessStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditBusinessStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("areas", static_cast<const BaseObject *>(areas_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_object_field("privacySettings", static_cast<const BaseObject *>(privacy_settings_.get()));
    s.store_class_end();
  }
}

editBusinessStory::ReturnType editBusinessStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return story::fetch(env, p);
}

void editBusinessStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditBusinessStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryContent;").c_str());
    areas_fieldID = jni::get_field_id(env, Class, "areas", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryAreas;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    privacy_settings_fieldID = jni::get_field_id(env, Class, "privacySettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryPrivacySettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editChatFolder::Class;
jfieldID editChatFolder::chat_folder_id_fieldID;
jfieldID editChatFolder::folder_fieldID;

editChatFolder::editChatFolder()
  : chat_folder_id_()
  , folder_()
{}

editChatFolder::editChatFolder(int32 chat_folder_id_, object_ptr<chatFolder> &&folder_)
  : chat_folder_id_(chat_folder_id_)
  , folder_(std::move(folder_))
{}

const std::int32_t editChatFolder::ID;

object_ptr<editChatFolder> editChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editChatFolder> res = make_object<editChatFolder>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->folder_ = jni::fetch_tl_object<chatFolder>(env, jni::fetch_object(env, p, res->folder_fieldID));
  return res;
}

void editChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditChatFolder");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_object_field("folder", static_cast<const BaseObject *>(folder_.get()));
    s.store_class_end();
  }
}

editChatFolder::ReturnType editChatFolder::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInfo::fetch(env, p);
}

void editChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditChatFolder").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    folder_fieldID = jni::get_field_id(env, Class, "folder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolder;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editChatFolderInviteLink::Class;
jfieldID editChatFolderInviteLink::chat_folder_id_fieldID;
jfieldID editChatFolderInviteLink::invite_link_fieldID;
jfieldID editChatFolderInviteLink::name_fieldID;
jfieldID editChatFolderInviteLink::chat_ids_fieldID;

editChatFolderInviteLink::editChatFolderInviteLink()
  : chat_folder_id_()
  , invite_link_()
  , name_()
  , chat_ids_()
{}

editChatFolderInviteLink::editChatFolderInviteLink(int32 chat_folder_id_, string const &invite_link_, string const &name_, array<int53> &&chat_ids_)
  : chat_folder_id_(chat_folder_id_)
  , invite_link_(invite_link_)
  , name_(name_)
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t editChatFolderInviteLink::ID;

object_ptr<editChatFolderInviteLink> editChatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editChatFolderInviteLink> res = make_object<editChatFolderInviteLink>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void editChatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditChatFolderInviteLink");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("name", name_);
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

editChatFolderInviteLink::ReturnType editChatFolderInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInviteLink::fetch(env, p);
}

void editChatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditChatFolderInviteLink").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass editChatInviteLink::Class;
jfieldID editChatInviteLink::chat_id_fieldID;
jfieldID editChatInviteLink::invite_link_fieldID;
jfieldID editChatInviteLink::name_fieldID;
jfieldID editChatInviteLink::expiration_date_fieldID;
jfieldID editChatInviteLink::member_limit_fieldID;
jfieldID editChatInviteLink::creates_join_request_fieldID;

editChatInviteLink::editChatInviteLink()
  : chat_id_()
  , invite_link_()
  , name_()
  , expiration_date_()
  , member_limit_()
  , creates_join_request_()
{}

editChatInviteLink::editChatInviteLink(int53 chat_id_, string const &invite_link_, string const &name_, int32 expiration_date_, int32 member_limit_, bool creates_join_request_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
  , name_(name_)
  , expiration_date_(expiration_date_)
  , member_limit_(member_limit_)
  , creates_join_request_(creates_join_request_)
{}

const std::int32_t editChatInviteLink::ID;

object_ptr<editChatInviteLink> editChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editChatInviteLink> res = make_object<editChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->expiration_date_ = env->GetIntField(p, res->expiration_date_fieldID);
  res->member_limit_ = env->GetIntField(p, res->member_limit_fieldID);
  res->creates_join_request_ = (env->GetBooleanField(p, res->creates_join_request_fieldID) != 0);
  return res;
}

void editChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("name", name_);
    s.store_field("expirationDate", expiration_date_);
    s.store_field("memberLimit", member_limit_);
    s.store_field("createsJoinRequest", creates_join_request_);
    s.store_class_end();
  }
}

editChatInviteLink::ReturnType editChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void editChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    expiration_date_fieldID = jni::get_field_id(env, Class, "expirationDate", "I");
    member_limit_fieldID = jni::get_field_id(env, Class, "memberLimit", "I");
    creates_join_request_fieldID = jni::get_field_id(env, Class, "createsJoinRequest", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editChatSubscriptionInviteLink::Class;
jfieldID editChatSubscriptionInviteLink::chat_id_fieldID;
jfieldID editChatSubscriptionInviteLink::invite_link_fieldID;
jfieldID editChatSubscriptionInviteLink::name_fieldID;

editChatSubscriptionInviteLink::editChatSubscriptionInviteLink()
  : chat_id_()
  , invite_link_()
  , name_()
{}

editChatSubscriptionInviteLink::editChatSubscriptionInviteLink(int53 chat_id_, string const &invite_link_, string const &name_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
  , name_(name_)
{}

const std::int32_t editChatSubscriptionInviteLink::ID;

object_ptr<editChatSubscriptionInviteLink> editChatSubscriptionInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editChatSubscriptionInviteLink> res = make_object<editChatSubscriptionInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void editChatSubscriptionInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditChatSubscriptionInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("name", name_);
    s.store_class_end();
  }
}

editChatSubscriptionInviteLink::ReturnType editChatSubscriptionInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void editChatSubscriptionInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditChatSubscriptionInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass editCustomLanguagePackInfo::Class;
jfieldID editCustomLanguagePackInfo::info_fieldID;

editCustomLanguagePackInfo::editCustomLanguagePackInfo()
  : info_()
{}

editCustomLanguagePackInfo::editCustomLanguagePackInfo(object_ptr<languagePackInfo> &&info_)
  : info_(std::move(info_))
{}

const std::int32_t editCustomLanguagePackInfo::ID;

object_ptr<editCustomLanguagePackInfo> editCustomLanguagePackInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editCustomLanguagePackInfo> res = make_object<editCustomLanguagePackInfo>();
  res->info_ = jni::fetch_tl_object<languagePackInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  return res;
}

void editCustomLanguagePackInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditCustomLanguagePackInfo");
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    s.store_class_end();
  }
}

editCustomLanguagePackInfo::ReturnType editCustomLanguagePackInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editCustomLanguagePackInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditCustomLanguagePackInfo").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LanguagePackInfo;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editForumTopic::Class;
jfieldID editForumTopic::chat_id_fieldID;
jfieldID editForumTopic::message_thread_id_fieldID;
jfieldID editForumTopic::name_fieldID;
jfieldID editForumTopic::edit_icon_custom_emoji_fieldID;
jfieldID editForumTopic::icon_custom_emoji_id_fieldID;

editForumTopic::editForumTopic()
  : chat_id_()
  , message_thread_id_()
  , name_()
  , edit_icon_custom_emoji_()
  , icon_custom_emoji_id_()
{}

editForumTopic::editForumTopic(int53 chat_id_, int53 message_thread_id_, string const &name_, bool edit_icon_custom_emoji_, int64 icon_custom_emoji_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , name_(name_)
  , edit_icon_custom_emoji_(edit_icon_custom_emoji_)
  , icon_custom_emoji_id_(icon_custom_emoji_id_)
{}

const std::int32_t editForumTopic::ID;

object_ptr<editForumTopic> editForumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editForumTopic> res = make_object<editForumTopic>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->edit_icon_custom_emoji_ = (env->GetBooleanField(p, res->edit_icon_custom_emoji_fieldID) != 0);
  res->icon_custom_emoji_id_ = env->GetLongField(p, res->icon_custom_emoji_id_fieldID);
  return res;
}

void editForumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditForumTopic");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("name", name_);
    s.store_field("editIconCustomEmoji", edit_icon_custom_emoji_);
    s.store_field("iconCustomEmojiId", icon_custom_emoji_id_);
    s.store_class_end();
  }
}

editForumTopic::ReturnType editForumTopic::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editForumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditForumTopic").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    edit_icon_custom_emoji_fieldID = jni::get_field_id(env, Class, "editIconCustomEmoji", "Z");
    icon_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "iconCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass editInlineMessageCaption::Class;
jfieldID editInlineMessageCaption::inline_message_id_fieldID;
jfieldID editInlineMessageCaption::reply_markup_fieldID;
jfieldID editInlineMessageCaption::caption_fieldID;
jfieldID editInlineMessageCaption::show_caption_above_media_fieldID;

editInlineMessageCaption::editInlineMessageCaption()
  : inline_message_id_()
  , reply_markup_()
  , caption_()
  , show_caption_above_media_()
{}

editInlineMessageCaption::editInlineMessageCaption(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_)
  : inline_message_id_(inline_message_id_)
  , reply_markup_(std::move(reply_markup_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
{}

const std::int32_t editInlineMessageCaption::ID;

object_ptr<editInlineMessageCaption> editInlineMessageCaption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editInlineMessageCaption> res = make_object<editInlineMessageCaption>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  return res;
}

void editInlineMessageCaption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditInlineMessageCaption");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_class_end();
  }
}

editInlineMessageCaption::ReturnType editInlineMessageCaption::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editInlineMessageCaption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditInlineMessageCaption").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editInlineMessageLiveLocation::Class;
jfieldID editInlineMessageLiveLocation::inline_message_id_fieldID;
jfieldID editInlineMessageLiveLocation::reply_markup_fieldID;
jfieldID editInlineMessageLiveLocation::location_fieldID;
jfieldID editInlineMessageLiveLocation::live_period_fieldID;
jfieldID editInlineMessageLiveLocation::heading_fieldID;
jfieldID editInlineMessageLiveLocation::proximity_alert_radius_fieldID;

editInlineMessageLiveLocation::editInlineMessageLiveLocation()
  : inline_message_id_()
  , reply_markup_()
  , location_()
  , live_period_()
  , heading_()
  , proximity_alert_radius_()
{}

editInlineMessageLiveLocation::editInlineMessageLiveLocation(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_)
  : inline_message_id_(inline_message_id_)
  , reply_markup_(std::move(reply_markup_))
  , location_(std::move(location_))
  , live_period_(live_period_)
  , heading_(heading_)
  , proximity_alert_radius_(proximity_alert_radius_)
{}

const std::int32_t editInlineMessageLiveLocation::ID;

object_ptr<editInlineMessageLiveLocation> editInlineMessageLiveLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editInlineMessageLiveLocation> res = make_object<editInlineMessageLiveLocation>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->heading_ = env->GetIntField(p, res->heading_fieldID);
  res->proximity_alert_radius_ = env->GetIntField(p, res->proximity_alert_radius_fieldID);
  return res;
}

void editInlineMessageLiveLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditInlineMessageLiveLocation");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("heading", heading_);
    s.store_field("proximityAlertRadius", proximity_alert_radius_);
    s.store_class_end();
  }
}

editInlineMessageLiveLocation::ReturnType editInlineMessageLiveLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editInlineMessageLiveLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditInlineMessageLiveLocation").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    heading_fieldID = jni::get_field_id(env, Class, "heading", "I");
    proximity_alert_radius_fieldID = jni::get_field_id(env, Class, "proximityAlertRadius", "I");
    return true;
  }();
  (void)is_inited;
}

jclass editInlineMessageMedia::Class;
jfieldID editInlineMessageMedia::inline_message_id_fieldID;
jfieldID editInlineMessageMedia::reply_markup_fieldID;
jfieldID editInlineMessageMedia::input_message_content_fieldID;

editInlineMessageMedia::editInlineMessageMedia()
  : inline_message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editInlineMessageMedia::editInlineMessageMedia(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : inline_message_id_(inline_message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editInlineMessageMedia::ID;

object_ptr<editInlineMessageMedia> editInlineMessageMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editInlineMessageMedia> res = make_object<editInlineMessageMedia>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editInlineMessageMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditInlineMessageMedia");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editInlineMessageMedia::ReturnType editInlineMessageMedia::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editInlineMessageMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditInlineMessageMedia").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editInlineMessageReplyMarkup::Class;
jfieldID editInlineMessageReplyMarkup::inline_message_id_fieldID;
jfieldID editInlineMessageReplyMarkup::reply_markup_fieldID;

editInlineMessageReplyMarkup::editInlineMessageReplyMarkup()
  : inline_message_id_()
  , reply_markup_()
{}

editInlineMessageReplyMarkup::editInlineMessageReplyMarkup(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_)
  : inline_message_id_(inline_message_id_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t editInlineMessageReplyMarkup::ID;

object_ptr<editInlineMessageReplyMarkup> editInlineMessageReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editInlineMessageReplyMarkup> res = make_object<editInlineMessageReplyMarkup>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void editInlineMessageReplyMarkup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditInlineMessageReplyMarkup");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

editInlineMessageReplyMarkup::ReturnType editInlineMessageReplyMarkup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editInlineMessageReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditInlineMessageReplyMarkup").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editInlineMessageText::Class;
jfieldID editInlineMessageText::inline_message_id_fieldID;
jfieldID editInlineMessageText::reply_markup_fieldID;
jfieldID editInlineMessageText::input_message_content_fieldID;

editInlineMessageText::editInlineMessageText()
  : inline_message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editInlineMessageText::editInlineMessageText(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : inline_message_id_(inline_message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editInlineMessageText::ID;

object_ptr<editInlineMessageText> editInlineMessageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editInlineMessageText> res = make_object<editInlineMessageText>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editInlineMessageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditInlineMessageText");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editInlineMessageText::ReturnType editInlineMessageText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editInlineMessageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditInlineMessageText").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editMessageCaption::Class;
jfieldID editMessageCaption::chat_id_fieldID;
jfieldID editMessageCaption::message_id_fieldID;
jfieldID editMessageCaption::reply_markup_fieldID;
jfieldID editMessageCaption::caption_fieldID;
jfieldID editMessageCaption::show_caption_above_media_fieldID;

editMessageCaption::editMessageCaption()
  : chat_id_()
  , message_id_()
  , reply_markup_()
  , caption_()
  , show_caption_above_media_()
{}

editMessageCaption::editMessageCaption(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , caption_(std::move(caption_))
  , show_caption_above_media_(show_caption_above_media_)
{}

const std::int32_t editMessageCaption::ID;

object_ptr<editMessageCaption> editMessageCaption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageCaption> res = make_object<editMessageCaption>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->show_caption_above_media_ = (env->GetBooleanField(p, res->show_caption_above_media_fieldID) != 0);
  return res;
}

void editMessageCaption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageCaption");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_field("showCaptionAboveMedia", show_caption_above_media_);
    s.store_class_end();
  }
}

editMessageCaption::ReturnType editMessageCaption::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void editMessageCaption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageCaption").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    show_caption_above_media_fieldID = jni::get_field_id(env, Class, "showCaptionAboveMedia", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editMessageLiveLocation::Class;
jfieldID editMessageLiveLocation::chat_id_fieldID;
jfieldID editMessageLiveLocation::message_id_fieldID;
jfieldID editMessageLiveLocation::reply_markup_fieldID;
jfieldID editMessageLiveLocation::location_fieldID;
jfieldID editMessageLiveLocation::live_period_fieldID;
jfieldID editMessageLiveLocation::heading_fieldID;
jfieldID editMessageLiveLocation::proximity_alert_radius_fieldID;

editMessageLiveLocation::editMessageLiveLocation()
  : chat_id_()
  , message_id_()
  , reply_markup_()
  , location_()
  , live_period_()
  , heading_()
  , proximity_alert_radius_()
{}

editMessageLiveLocation::editMessageLiveLocation(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , location_(std::move(location_))
  , live_period_(live_period_)
  , heading_(heading_)
  , proximity_alert_radius_(proximity_alert_radius_)
{}

const std::int32_t editMessageLiveLocation::ID;

object_ptr<editMessageLiveLocation> editMessageLiveLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageLiveLocation> res = make_object<editMessageLiveLocation>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->live_period_ = env->GetIntField(p, res->live_period_fieldID);
  res->heading_ = env->GetIntField(p, res->heading_fieldID);
  res->proximity_alert_radius_ = env->GetIntField(p, res->proximity_alert_radius_fieldID);
  return res;
}

void editMessageLiveLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageLiveLocation");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("livePeriod", live_period_);
    s.store_field("heading", heading_);
    s.store_field("proximityAlertRadius", proximity_alert_radius_);
    s.store_class_end();
  }
}

editMessageLiveLocation::ReturnType editMessageLiveLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void editMessageLiveLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageLiveLocation").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    live_period_fieldID = jni::get_field_id(env, Class, "livePeriod", "I");
    heading_fieldID = jni::get_field_id(env, Class, "heading", "I");
    proximity_alert_radius_fieldID = jni::get_field_id(env, Class, "proximityAlertRadius", "I");
    return true;
  }();
  (void)is_inited;
}

jclass editMessageMedia::Class;
jfieldID editMessageMedia::chat_id_fieldID;
jfieldID editMessageMedia::message_id_fieldID;
jfieldID editMessageMedia::reply_markup_fieldID;
jfieldID editMessageMedia::input_message_content_fieldID;

editMessageMedia::editMessageMedia()
  : chat_id_()
  , message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editMessageMedia::editMessageMedia(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editMessageMedia::ID;

object_ptr<editMessageMedia> editMessageMedia::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageMedia> res = make_object<editMessageMedia>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editMessageMedia::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageMedia");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editMessageMedia::ReturnType editMessageMedia::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void editMessageMedia::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageMedia").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editMessageReplyMarkup::Class;
jfieldID editMessageReplyMarkup::chat_id_fieldID;
jfieldID editMessageReplyMarkup::message_id_fieldID;
jfieldID editMessageReplyMarkup::reply_markup_fieldID;

editMessageReplyMarkup::editMessageReplyMarkup()
  : chat_id_()
  , message_id_()
  , reply_markup_()
{}

editMessageReplyMarkup::editMessageReplyMarkup(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t editMessageReplyMarkup::ID;

object_ptr<editMessageReplyMarkup> editMessageReplyMarkup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageReplyMarkup> res = make_object<editMessageReplyMarkup>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void editMessageReplyMarkup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageReplyMarkup");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

editMessageReplyMarkup::ReturnType editMessageReplyMarkup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void editMessageReplyMarkup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageReplyMarkup").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editMessageSchedulingState::Class;
jfieldID editMessageSchedulingState::chat_id_fieldID;
jfieldID editMessageSchedulingState::message_id_fieldID;
jfieldID editMessageSchedulingState::scheduling_state_fieldID;

editMessageSchedulingState::editMessageSchedulingState()
  : chat_id_()
  , message_id_()
  , scheduling_state_()
{}

editMessageSchedulingState::editMessageSchedulingState(int53 chat_id_, int53 message_id_, object_ptr<MessageSchedulingState> &&scheduling_state_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , scheduling_state_(std::move(scheduling_state_))
{}

const std::int32_t editMessageSchedulingState::ID;

object_ptr<editMessageSchedulingState> editMessageSchedulingState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageSchedulingState> res = make_object<editMessageSchedulingState>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->scheduling_state_ = jni::fetch_tl_object<MessageSchedulingState>(env, jni::fetch_object(env, p, res->scheduling_state_fieldID));
  return res;
}

void editMessageSchedulingState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageSchedulingState");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("schedulingState", static_cast<const BaseObject *>(scheduling_state_.get()));
    s.store_class_end();
  }
}

editMessageSchedulingState::ReturnType editMessageSchedulingState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editMessageSchedulingState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageSchedulingState").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    scheduling_state_fieldID = jni::get_field_id(env, Class, "schedulingState", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSchedulingState;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editMessageText::Class;
jfieldID editMessageText::chat_id_fieldID;
jfieldID editMessageText::message_id_fieldID;
jfieldID editMessageText::reply_markup_fieldID;
jfieldID editMessageText::input_message_content_fieldID;

editMessageText::editMessageText()
  : chat_id_()
  , message_id_()
  , reply_markup_()
  , input_message_content_()
{}

editMessageText::editMessageText(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editMessageText::ID;

object_ptr<editMessageText> editMessageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editMessageText> res = make_object<editMessageText>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editMessageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditMessageText");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editMessageText::ReturnType editMessageText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void editMessageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditMessageText").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editProxy::Class;
jfieldID editProxy::proxy_id_fieldID;
jfieldID editProxy::server_fieldID;
jfieldID editProxy::port_fieldID;
jfieldID editProxy::enable_fieldID;
jfieldID editProxy::type_fieldID;

editProxy::editProxy()
  : proxy_id_()
  , server_()
  , port_()
  , enable_()
  , type_()
{}

editProxy::editProxy(int32 proxy_id_, string const &server_, int32 port_, bool enable_, object_ptr<ProxyType> &&type_)
  : proxy_id_(proxy_id_)
  , server_(server_)
  , port_(port_)
  , enable_(enable_)
  , type_(std::move(type_))
{}

const std::int32_t editProxy::ID;

object_ptr<editProxy> editProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editProxy> res = make_object<editProxy>();
  res->proxy_id_ = env->GetIntField(p, res->proxy_id_fieldID);
  res->server_ = jni::fetch_string(env, p, res->server_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->enable_ = (env->GetBooleanField(p, res->enable_fieldID) != 0);
  res->type_ = jni::fetch_tl_object<ProxyType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void editProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditProxy");
    s.store_field("proxyId", proxy_id_);
    s.store_field("server", server_);
    s.store_field("port", port_);
    s.store_field("enable", enable_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

editProxy::ReturnType editProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return proxy::fetch(env, p);
}

void editProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditProxy").c_str());
    proxy_id_fieldID = jni::get_field_id(env, Class, "proxyId", "I");
    server_fieldID = jni::get_field_id(env, Class, "server", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    enable_fieldID = jni::get_field_id(env, Class, "enable", "Z");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProxyType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editQuickReplyMessage::Class;
jfieldID editQuickReplyMessage::shortcut_id_fieldID;
jfieldID editQuickReplyMessage::message_id_fieldID;
jfieldID editQuickReplyMessage::input_message_content_fieldID;

editQuickReplyMessage::editQuickReplyMessage()
  : shortcut_id_()
  , message_id_()
  , input_message_content_()
{}

editQuickReplyMessage::editQuickReplyMessage(int32 shortcut_id_, int53 message_id_, object_ptr<InputMessageContent> &&input_message_content_)
  : shortcut_id_(shortcut_id_)
  , message_id_(message_id_)
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t editQuickReplyMessage::ID;

object_ptr<editQuickReplyMessage> editQuickReplyMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editQuickReplyMessage> res = make_object<editQuickReplyMessage>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void editQuickReplyMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditQuickReplyMessage");
    s.store_field("shortcutId", shortcut_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

editQuickReplyMessage::ReturnType editQuickReplyMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editQuickReplyMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditQuickReplyMessage").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editStarSubscription::Class;
jfieldID editStarSubscription::subscription_id_fieldID;
jfieldID editStarSubscription::is_canceled_fieldID;

editStarSubscription::editStarSubscription()
  : subscription_id_()
  , is_canceled_()
{}

editStarSubscription::editStarSubscription(string const &subscription_id_, bool is_canceled_)
  : subscription_id_(subscription_id_)
  , is_canceled_(is_canceled_)
{}

const std::int32_t editStarSubscription::ID;

object_ptr<editStarSubscription> editStarSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editStarSubscription> res = make_object<editStarSubscription>();
  res->subscription_id_ = jni::fetch_string(env, p, res->subscription_id_fieldID);
  res->is_canceled_ = (env->GetBooleanField(p, res->is_canceled_fieldID) != 0);
  return res;
}

void editStarSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditStarSubscription");
    s.store_field("subscriptionId", subscription_id_);
    s.store_field("isCanceled", is_canceled_);
    s.store_class_end();
  }
}

editStarSubscription::ReturnType editStarSubscription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editStarSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditStarSubscription").c_str());
    subscription_id_fieldID = jni::get_field_id(env, Class, "subscriptionId", "Ljava/lang/String;");
    is_canceled_fieldID = jni::get_field_id(env, Class, "isCanceled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass editStory::Class;
jfieldID editStory::story_poster_chat_id_fieldID;
jfieldID editStory::story_id_fieldID;
jfieldID editStory::content_fieldID;
jfieldID editStory::areas_fieldID;
jfieldID editStory::caption_fieldID;

editStory::editStory()
  : story_poster_chat_id_()
  , story_id_()
  , content_()
  , areas_()
  , caption_()
{}

editStory::editStory(int53 story_poster_chat_id_, int32 story_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , content_(std::move(content_))
  , areas_(std::move(areas_))
  , caption_(std::move(caption_))
{}

const std::int32_t editStory::ID;

object_ptr<editStory> editStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editStory> res = make_object<editStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->content_ = jni::fetch_tl_object<InputStoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->areas_ = jni::fetch_tl_object<inputStoryAreas>(env, jni::fetch_object(env, p, res->areas_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  return res;
}

void editStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("areas", static_cast<const BaseObject *>(areas_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_class_end();
  }
}

editStory::ReturnType editStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryContent;").c_str());
    areas_fieldID = jni::get_field_id(env, Class, "areas", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryAreas;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass editStoryCover::Class;
jfieldID editStoryCover::story_poster_chat_id_fieldID;
jfieldID editStoryCover::story_id_fieldID;
jfieldID editStoryCover::cover_frame_timestamp_fieldID;

editStoryCover::editStoryCover()
  : story_poster_chat_id_()
  , story_id_()
  , cover_frame_timestamp_()
{}

editStoryCover::editStoryCover(int53 story_poster_chat_id_, int32 story_id_, double cover_frame_timestamp_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , cover_frame_timestamp_(cover_frame_timestamp_)
{}

const std::int32_t editStoryCover::ID;

object_ptr<editStoryCover> editStoryCover::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editStoryCover> res = make_object<editStoryCover>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->cover_frame_timestamp_ = env->GetDoubleField(p, res->cover_frame_timestamp_fieldID);
  return res;
}

void editStoryCover::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditStoryCover");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("coverFrameTimestamp", cover_frame_timestamp_);
    s.store_class_end();
  }
}

editStoryCover::ReturnType editStoryCover::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editStoryCover::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditStoryCover").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    cover_frame_timestamp_fieldID = jni::get_field_id(env, Class, "coverFrameTimestamp", "D");
    return true;
  }();
  (void)is_inited;
}

jclass editUserStarSubscription::Class;
jfieldID editUserStarSubscription::user_id_fieldID;
jfieldID editUserStarSubscription::telegram_payment_charge_id_fieldID;
jfieldID editUserStarSubscription::is_canceled_fieldID;

editUserStarSubscription::editUserStarSubscription()
  : user_id_()
  , telegram_payment_charge_id_()
  , is_canceled_()
{}

editUserStarSubscription::editUserStarSubscription(int53 user_id_, string const &telegram_payment_charge_id_, bool is_canceled_)
  : user_id_(user_id_)
  , telegram_payment_charge_id_(telegram_payment_charge_id_)
  , is_canceled_(is_canceled_)
{}

const std::int32_t editUserStarSubscription::ID;

object_ptr<editUserStarSubscription> editUserStarSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<editUserStarSubscription> res = make_object<editUserStarSubscription>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->telegram_payment_charge_id_ = jni::fetch_string(env, p, res->telegram_payment_charge_id_fieldID);
  res->is_canceled_ = (env->GetBooleanField(p, res->is_canceled_fieldID) != 0);
  return res;
}

void editUserStarSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EditUserStarSubscription");
    s.store_field("userId", user_id_);
    s.store_field("telegramPaymentChargeId", telegram_payment_charge_id_);
    s.store_field("isCanceled", is_canceled_);
    s.store_class_end();
  }
}

editUserStarSubscription::ReturnType editUserStarSubscription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void editUserStarSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EditUserStarSubscription").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    telegram_payment_charge_id_fieldID = jni::get_field_id(env, Class, "telegramPaymentChargeId", "Ljava/lang/String;");
    is_canceled_fieldID = jni::get_field_id(env, Class, "isCanceled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass enableProxy::Class;
jfieldID enableProxy::proxy_id_fieldID;

enableProxy::enableProxy()
  : proxy_id_()
{}

enableProxy::enableProxy(int32 proxy_id_)
  : proxy_id_(proxy_id_)
{}

const std::int32_t enableProxy::ID;

object_ptr<enableProxy> enableProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<enableProxy> res = make_object<enableProxy>();
  res->proxy_id_ = env->GetIntField(p, res->proxy_id_fieldID);
  return res;
}

void enableProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EnableProxy");
    s.store_field("proxyId", proxy_id_);
    s.store_class_end();
  }
}

enableProxy::ReturnType enableProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void enableProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EnableProxy").c_str());
    proxy_id_fieldID = jni::get_field_id(env, Class, "proxyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass encryptGroupCallData::Class;
jfieldID encryptGroupCallData::group_call_id_fieldID;
jfieldID encryptGroupCallData::data_channel_fieldID;
jfieldID encryptGroupCallData::data_fieldID;
jfieldID encryptGroupCallData::unencrypted_prefix_size_fieldID;

encryptGroupCallData::encryptGroupCallData()
  : group_call_id_()
  , data_channel_()
  , data_()
  , unencrypted_prefix_size_()
{}

encryptGroupCallData::encryptGroupCallData(int32 group_call_id_, object_ptr<GroupCallDataChannel> &&data_channel_, bytes const &data_, int32 unencrypted_prefix_size_)
  : group_call_id_(group_call_id_)
  , data_channel_(std::move(data_channel_))
  , data_(std::move(data_))
  , unencrypted_prefix_size_(unencrypted_prefix_size_)
{}

const std::int32_t encryptGroupCallData::ID;

object_ptr<encryptGroupCallData> encryptGroupCallData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<encryptGroupCallData> res = make_object<encryptGroupCallData>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->data_channel_ = jni::fetch_tl_object<GroupCallDataChannel>(env, jni::fetch_object(env, p, res->data_channel_fieldID));
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  res->unencrypted_prefix_size_ = env->GetIntField(p, res->unencrypted_prefix_size_fieldID);
  return res;
}

void encryptGroupCallData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EncryptGroupCallData");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("dataChannel", static_cast<const BaseObject *>(data_channel_.get()));
    s.store_bytes_field("data", data_);
    s.store_field("unencryptedPrefixSize", unencrypted_prefix_size_);
    s.store_class_end();
  }
}

encryptGroupCallData::ReturnType encryptGroupCallData::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return data::fetch(env, p);
}

void encryptGroupCallData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EncryptGroupCallData").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    data_channel_fieldID = jni::get_field_id(env, Class, "dataChannel", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallDataChannel;").c_str());
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    unencrypted_prefix_size_fieldID = jni::get_field_id(env, Class, "unencryptedPrefixSize", "I");
    return true;
  }();
  (void)is_inited;
}

jclass endGroupCall::Class;
jfieldID endGroupCall::group_call_id_fieldID;

endGroupCall::endGroupCall()
  : group_call_id_()
{}

endGroupCall::endGroupCall(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t endGroupCall::ID;

object_ptr<endGroupCall> endGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<endGroupCall> res = make_object<endGroupCall>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void endGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EndGroupCall");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

endGroupCall::ReturnType endGroupCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void endGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EndGroupCall").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass endGroupCallRecording::Class;
jfieldID endGroupCallRecording::group_call_id_fieldID;

endGroupCallRecording::endGroupCallRecording()
  : group_call_id_()
{}

endGroupCallRecording::endGroupCallRecording(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t endGroupCallRecording::ID;

object_ptr<endGroupCallRecording> endGroupCallRecording::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<endGroupCallRecording> res = make_object<endGroupCallRecording>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void endGroupCallRecording::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EndGroupCallRecording");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

endGroupCallRecording::ReturnType endGroupCallRecording::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void endGroupCallRecording::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EndGroupCallRecording").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass endGroupCallScreenSharing::Class;
jfieldID endGroupCallScreenSharing::group_call_id_fieldID;

endGroupCallScreenSharing::endGroupCallScreenSharing()
  : group_call_id_()
{}

endGroupCallScreenSharing::endGroupCallScreenSharing(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t endGroupCallScreenSharing::ID;

object_ptr<endGroupCallScreenSharing> endGroupCallScreenSharing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<endGroupCallScreenSharing> res = make_object<endGroupCallScreenSharing>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void endGroupCallScreenSharing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "EndGroupCallScreenSharing");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

endGroupCallScreenSharing::ReturnType endGroupCallScreenSharing::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void endGroupCallScreenSharing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$EndGroupCallScreenSharing").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass finishFileGeneration::Class;
jfieldID finishFileGeneration::generation_id_fieldID;
jfieldID finishFileGeneration::error_fieldID;

finishFileGeneration::finishFileGeneration()
  : generation_id_()
  , error_()
{}

finishFileGeneration::finishFileGeneration(int64 generation_id_, object_ptr<error> &&error_)
  : generation_id_(generation_id_)
  , error_(std::move(error_))
{}

const std::int32_t finishFileGeneration::ID;

object_ptr<finishFileGeneration> finishFileGeneration::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<finishFileGeneration> res = make_object<finishFileGeneration>();
  res->generation_id_ = env->GetLongField(p, res->generation_id_fieldID);
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  return res;
}

void finishFileGeneration::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "FinishFileGeneration");
    s.store_field("generationId", generation_id_);
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_class_end();
  }
}

finishFileGeneration::ReturnType finishFileGeneration::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void finishFileGeneration::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$FinishFileGeneration").c_str());
    generation_id_fieldID = jni::get_field_id(env, Class, "generationId", "J");
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass forwardMessages::Class;
jfieldID forwardMessages::chat_id_fieldID;
jfieldID forwardMessages::message_thread_id_fieldID;
jfieldID forwardMessages::from_chat_id_fieldID;
jfieldID forwardMessages::message_ids_fieldID;
jfieldID forwardMessages::options_fieldID;
jfieldID forwardMessages::send_copy_fieldID;
jfieldID forwardMessages::remove_caption_fieldID;

forwardMessages::forwardMessages()
  : chat_id_()
  , message_thread_id_()
  , from_chat_id_()
  , message_ids_()
  , options_()
  , send_copy_()
  , remove_caption_()
{}

forwardMessages::forwardMessages(int53 chat_id_, int53 message_thread_id_, int53 from_chat_id_, array<int53> &&message_ids_, object_ptr<messageSendOptions> &&options_, bool send_copy_, bool remove_caption_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , from_chat_id_(from_chat_id_)
  , message_ids_(std::move(message_ids_))
  , options_(std::move(options_))
  , send_copy_(send_copy_)
  , remove_caption_(remove_caption_)
{}

const std::int32_t forwardMessages::ID;

object_ptr<forwardMessages> forwardMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<forwardMessages> res = make_object<forwardMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->from_chat_id_ = env->GetLongField(p, res->from_chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->options_ = jni::fetch_tl_object<messageSendOptions>(env, jni::fetch_object(env, p, res->options_fieldID));
  res->send_copy_ = (env->GetBooleanField(p, res->send_copy_fieldID) != 0);
  res->remove_caption_ = (env->GetBooleanField(p, res->remove_caption_fieldID) != 0);
  return res;
}

void forwardMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ForwardMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("fromChatId", from_chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_object_field("options", static_cast<const BaseObject *>(options_.get()));
    s.store_field("sendCopy", send_copy_);
    s.store_field("removeCaption", remove_caption_);
    s.store_class_end();
  }
}

forwardMessages::ReturnType forwardMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void forwardMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ForwardMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    from_chat_id_fieldID = jni::get_field_id(env, Class, "fromChatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendOptions;").c_str());
    send_copy_fieldID = jni::get_field_id(env, Class, "sendCopy", "Z");
    remove_caption_fieldID = jni::get_field_id(env, Class, "removeCaption", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getAccountTtl::Class;

getAccountTtl::getAccountTtl() {
}

const std::int32_t getAccountTtl::ID;

object_ptr<getAccountTtl> getAccountTtl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAccountTtl>();
}

void getAccountTtl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAccountTtl");
    s.store_class_end();
  }
}

getAccountTtl::ReturnType getAccountTtl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return accountTtl::fetch(env, p);
}

void getAccountTtl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAccountTtl").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getActiveSessions::Class;

getActiveSessions::getActiveSessions() {
}

const std::int32_t getActiveSessions::ID;

object_ptr<getActiveSessions> getActiveSessions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getActiveSessions>();
}

void getActiveSessions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetActiveSessions");
    s.store_class_end();
  }
}

getActiveSessions::ReturnType getActiveSessions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sessions::fetch(env, p);
}

void getActiveSessions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetActiveSessions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getAllPassportElements::Class;
jfieldID getAllPassportElements::password_fieldID;

getAllPassportElements::getAllPassportElements()
  : password_()
{}

getAllPassportElements::getAllPassportElements(string const &password_)
  : password_(password_)
{}

const std::int32_t getAllPassportElements::ID;

object_ptr<getAllPassportElements> getAllPassportElements::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getAllPassportElements> res = make_object<getAllPassportElements>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getAllPassportElements::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAllPassportElements");
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getAllPassportElements::ReturnType getAllPassportElements::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passportElements::fetch(env, p);
}

void getAllPassportElements::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAllPassportElements").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getAllStickerEmojis::Class;
jfieldID getAllStickerEmojis::sticker_type_fieldID;
jfieldID getAllStickerEmojis::query_fieldID;
jfieldID getAllStickerEmojis::chat_id_fieldID;
jfieldID getAllStickerEmojis::return_only_main_emoji_fieldID;

getAllStickerEmojis::getAllStickerEmojis()
  : sticker_type_()
  , query_()
  , chat_id_()
  , return_only_main_emoji_()
{}

getAllStickerEmojis::getAllStickerEmojis(object_ptr<StickerType> &&sticker_type_, string const &query_, int53 chat_id_, bool return_only_main_emoji_)
  : sticker_type_(std::move(sticker_type_))
  , query_(query_)
  , chat_id_(chat_id_)
  , return_only_main_emoji_(return_only_main_emoji_)
{}

const std::int32_t getAllStickerEmojis::ID;

object_ptr<getAllStickerEmojis> getAllStickerEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getAllStickerEmojis> res = make_object<getAllStickerEmojis>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->return_only_main_emoji_ = (env->GetBooleanField(p, res->return_only_main_emoji_fieldID) != 0);
  return res;
}

void getAllStickerEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAllStickerEmojis");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("query", query_);
    s.store_field("chatId", chat_id_);
    s.store_field("returnOnlyMainEmoji", return_only_main_emoji_);
    s.store_class_end();
  }
}

getAllStickerEmojis::ReturnType getAllStickerEmojis::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojis::fetch(env, p);
}

void getAllStickerEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAllStickerEmojis").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return_only_main_emoji_fieldID = jni::get_field_id(env, Class, "returnOnlyMainEmoji", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getAnimatedEmoji::Class;
jfieldID getAnimatedEmoji::emoji_fieldID;

getAnimatedEmoji::getAnimatedEmoji()
  : emoji_()
{}

getAnimatedEmoji::getAnimatedEmoji(string const &emoji_)
  : emoji_(emoji_)
{}

const std::int32_t getAnimatedEmoji::ID;

object_ptr<getAnimatedEmoji> getAnimatedEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getAnimatedEmoji> res = make_object<getAnimatedEmoji>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return res;
}

void getAnimatedEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAnimatedEmoji");
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

getAnimatedEmoji::ReturnType getAnimatedEmoji::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return animatedEmoji::fetch(env, p);
}

void getAnimatedEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAnimatedEmoji").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getApplicationConfig::Class;

getApplicationConfig::getApplicationConfig() {
}

const std::int32_t getApplicationConfig::ID;

object_ptr<getApplicationConfig> getApplicationConfig::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getApplicationConfig>();
}

void getApplicationConfig::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetApplicationConfig");
    s.store_class_end();
  }
}

getApplicationConfig::ReturnType getApplicationConfig::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return JsonValue::fetch(env, p);
}

void getApplicationConfig::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetApplicationConfig").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getApplicationDownloadLink::Class;

getApplicationDownloadLink::getApplicationDownloadLink() {
}

const std::int32_t getApplicationDownloadLink::ID;

object_ptr<getApplicationDownloadLink> getApplicationDownloadLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getApplicationDownloadLink>();
}

void getApplicationDownloadLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetApplicationDownloadLink");
    s.store_class_end();
  }
}

getApplicationDownloadLink::ReturnType getApplicationDownloadLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getApplicationDownloadLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetApplicationDownloadLink").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getArchiveChatListSettings::Class;

getArchiveChatListSettings::getArchiveChatListSettings() {
}

const std::int32_t getArchiveChatListSettings::ID;

object_ptr<getArchiveChatListSettings> getArchiveChatListSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getArchiveChatListSettings>();
}

void getArchiveChatListSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetArchiveChatListSettings");
    s.store_class_end();
  }
}

getArchiveChatListSettings::ReturnType getArchiveChatListSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return archiveChatListSettings::fetch(env, p);
}

void getArchiveChatListSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetArchiveChatListSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getArchivedStickerSets::Class;
jfieldID getArchivedStickerSets::sticker_type_fieldID;
jfieldID getArchivedStickerSets::offset_sticker_set_id_fieldID;
jfieldID getArchivedStickerSets::limit_fieldID;

getArchivedStickerSets::getArchivedStickerSets()
  : sticker_type_()
  , offset_sticker_set_id_()
  , limit_()
{}

getArchivedStickerSets::getArchivedStickerSets(object_ptr<StickerType> &&sticker_type_, int64 offset_sticker_set_id_, int32 limit_)
  : sticker_type_(std::move(sticker_type_))
  , offset_sticker_set_id_(offset_sticker_set_id_)
  , limit_(limit_)
{}

const std::int32_t getArchivedStickerSets::ID;

object_ptr<getArchivedStickerSets> getArchivedStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getArchivedStickerSets> res = make_object<getArchivedStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->offset_sticker_set_id_ = env->GetLongField(p, res->offset_sticker_set_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getArchivedStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetArchivedStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("offsetStickerSetId", offset_sticker_set_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getArchivedStickerSets::ReturnType getArchivedStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void getArchivedStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetArchivedStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    offset_sticker_set_id_fieldID = jni::get_field_id(env, Class, "offsetStickerSetId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getAttachedStickerSets::Class;
jfieldID getAttachedStickerSets::file_id_fieldID;

getAttachedStickerSets::getAttachedStickerSets()
  : file_id_()
{}

getAttachedStickerSets::getAttachedStickerSets(int32 file_id_)
  : file_id_(file_id_)
{}

const std::int32_t getAttachedStickerSets::ID;

object_ptr<getAttachedStickerSets> getAttachedStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getAttachedStickerSets> res = make_object<getAttachedStickerSets>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  return res;
}

void getAttachedStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAttachedStickerSets");
    s.store_field("fileId", file_id_);
    s.store_class_end();
  }
}

getAttachedStickerSets::ReturnType getAttachedStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void getAttachedStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAttachedStickerSets").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getAttachmentMenuBot::Class;
jfieldID getAttachmentMenuBot::bot_user_id_fieldID;

getAttachmentMenuBot::getAttachmentMenuBot()
  : bot_user_id_()
{}

getAttachmentMenuBot::getAttachmentMenuBot(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t getAttachmentMenuBot::ID;

object_ptr<getAttachmentMenuBot> getAttachmentMenuBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getAttachmentMenuBot> res = make_object<getAttachmentMenuBot>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void getAttachmentMenuBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAttachmentMenuBot");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

getAttachmentMenuBot::ReturnType getAttachmentMenuBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return attachmentMenuBot::fetch(env, p);
}

void getAttachmentMenuBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAttachmentMenuBot").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getAuthorizationState::Class;

getAuthorizationState::getAuthorizationState() {
}

const std::int32_t getAuthorizationState::ID;

object_ptr<getAuthorizationState> getAuthorizationState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAuthorizationState>();
}

void getAuthorizationState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAuthorizationState");
    s.store_class_end();
  }
}

getAuthorizationState::ReturnType getAuthorizationState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return AuthorizationState::fetch(env, p);
}

void getAuthorizationState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAuthorizationState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getAutoDownloadSettingsPresets::Class;

getAutoDownloadSettingsPresets::getAutoDownloadSettingsPresets() {
}

const std::int32_t getAutoDownloadSettingsPresets::ID;

object_ptr<getAutoDownloadSettingsPresets> getAutoDownloadSettingsPresets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAutoDownloadSettingsPresets>();
}

void getAutoDownloadSettingsPresets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAutoDownloadSettingsPresets");
    s.store_class_end();
  }
}

getAutoDownloadSettingsPresets::ReturnType getAutoDownloadSettingsPresets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return autoDownloadSettingsPresets::fetch(env, p);
}

void getAutoDownloadSettingsPresets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAutoDownloadSettingsPresets").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getAutosaveSettings::Class;

getAutosaveSettings::getAutosaveSettings() {
}

const std::int32_t getAutosaveSettings::ID;

object_ptr<getAutosaveSettings> getAutosaveSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAutosaveSettings>();
}

void getAutosaveSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAutosaveSettings");
    s.store_class_end();
  }
}

getAutosaveSettings::ReturnType getAutosaveSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return autosaveSettings::fetch(env, p);
}

void getAutosaveSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAutosaveSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getAvailableChatBoostSlots::Class;

getAvailableChatBoostSlots::getAvailableChatBoostSlots() {
}

const std::int32_t getAvailableChatBoostSlots::ID;

object_ptr<getAvailableChatBoostSlots> getAvailableChatBoostSlots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAvailableChatBoostSlots>();
}

void getAvailableChatBoostSlots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAvailableChatBoostSlots");
    s.store_class_end();
  }
}

getAvailableChatBoostSlots::ReturnType getAvailableChatBoostSlots::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostSlots::fetch(env, p);
}

void getAvailableChatBoostSlots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAvailableChatBoostSlots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getAvailableGifts::Class;

getAvailableGifts::getAvailableGifts() {
}

const std::int32_t getAvailableGifts::ID;

object_ptr<getAvailableGifts> getAvailableGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getAvailableGifts>();
}

void getAvailableGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetAvailableGifts");
    s.store_class_end();
  }
}

getAvailableGifts::ReturnType getAvailableGifts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return availableGifts::fetch(env, p);
}

void getAvailableGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetAvailableGifts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getBackgroundUrl::Class;
jfieldID getBackgroundUrl::name_fieldID;
jfieldID getBackgroundUrl::type_fieldID;

getBackgroundUrl::getBackgroundUrl()
  : name_()
  , type_()
{}

getBackgroundUrl::getBackgroundUrl(string const &name_, object_ptr<BackgroundType> &&type_)
  : name_(name_)
  , type_(std::move(type_))
{}

const std::int32_t getBackgroundUrl::ID;

object_ptr<getBackgroundUrl> getBackgroundUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBackgroundUrl> res = make_object<getBackgroundUrl>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->type_ = jni::fetch_tl_object<BackgroundType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void getBackgroundUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBackgroundUrl");
    s.store_field("name", name_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

getBackgroundUrl::ReturnType getBackgroundUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getBackgroundUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBackgroundUrl").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getBankCardInfo::Class;
jfieldID getBankCardInfo::bank_card_number_fieldID;

getBankCardInfo::getBankCardInfo()
  : bank_card_number_()
{}

getBankCardInfo::getBankCardInfo(string const &bank_card_number_)
  : bank_card_number_(bank_card_number_)
{}

const std::int32_t getBankCardInfo::ID;

object_ptr<getBankCardInfo> getBankCardInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBankCardInfo> res = make_object<getBankCardInfo>();
  res->bank_card_number_ = jni::fetch_string(env, p, res->bank_card_number_fieldID);
  return res;
}

void getBankCardInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBankCardInfo");
    s.store_field("bankCardNumber", bank_card_number_);
    s.store_class_end();
  }
}

getBankCardInfo::ReturnType getBankCardInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return bankCardInfo::fetch(env, p);
}

void getBankCardInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBankCardInfo").c_str());
    bank_card_number_fieldID = jni::get_field_id(env, Class, "bankCardNumber", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBasicGroup::Class;
jfieldID getBasicGroup::basic_group_id_fieldID;

getBasicGroup::getBasicGroup()
  : basic_group_id_()
{}

getBasicGroup::getBasicGroup(int53 basic_group_id_)
  : basic_group_id_(basic_group_id_)
{}

const std::int32_t getBasicGroup::ID;

object_ptr<getBasicGroup> getBasicGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBasicGroup> res = make_object<getBasicGroup>();
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  return res;
}

void getBasicGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBasicGroup");
    s.store_field("basicGroupId", basic_group_id_);
    s.store_class_end();
  }
}

getBasicGroup::ReturnType getBasicGroup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return basicGroup::fetch(env, p);
}

void getBasicGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBasicGroup").c_str());
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getBasicGroupFullInfo::Class;
jfieldID getBasicGroupFullInfo::basic_group_id_fieldID;

getBasicGroupFullInfo::getBasicGroupFullInfo()
  : basic_group_id_()
{}

getBasicGroupFullInfo::getBasicGroupFullInfo(int53 basic_group_id_)
  : basic_group_id_(basic_group_id_)
{}

const std::int32_t getBasicGroupFullInfo::ID;

object_ptr<getBasicGroupFullInfo> getBasicGroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBasicGroupFullInfo> res = make_object<getBasicGroupFullInfo>();
  res->basic_group_id_ = env->GetLongField(p, res->basic_group_id_fieldID);
  return res;
}

void getBasicGroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBasicGroupFullInfo");
    s.store_field("basicGroupId", basic_group_id_);
    s.store_class_end();
  }
}

getBasicGroupFullInfo::ReturnType getBasicGroupFullInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return basicGroupFullInfo::fetch(env, p);
}

void getBasicGroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBasicGroupFullInfo").c_str());
    basic_group_id_fieldID = jni::get_field_id(env, Class, "basicGroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getBlockedMessageSenders::Class;
jfieldID getBlockedMessageSenders::block_list_fieldID;
jfieldID getBlockedMessageSenders::offset_fieldID;
jfieldID getBlockedMessageSenders::limit_fieldID;

getBlockedMessageSenders::getBlockedMessageSenders()
  : block_list_()
  , offset_()
  , limit_()
{}

getBlockedMessageSenders::getBlockedMessageSenders(object_ptr<BlockList> &&block_list_, int32 offset_, int32 limit_)
  : block_list_(std::move(block_list_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getBlockedMessageSenders::ID;

object_ptr<getBlockedMessageSenders> getBlockedMessageSenders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBlockedMessageSenders> res = make_object<getBlockedMessageSenders>();
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getBlockedMessageSenders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBlockedMessageSenders");
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getBlockedMessageSenders::ReturnType getBlockedMessageSenders::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageSenders::fetch(env, p);
}

void getBlockedMessageSenders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBlockedMessageSenders").c_str());
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getBotInfoDescription::Class;
jfieldID getBotInfoDescription::bot_user_id_fieldID;
jfieldID getBotInfoDescription::language_code_fieldID;

getBotInfoDescription::getBotInfoDescription()
  : bot_user_id_()
  , language_code_()
{}

getBotInfoDescription::getBotInfoDescription(int53 bot_user_id_, string const &language_code_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
{}

const std::int32_t getBotInfoDescription::ID;

object_ptr<getBotInfoDescription> getBotInfoDescription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotInfoDescription> res = make_object<getBotInfoDescription>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getBotInfoDescription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotInfoDescription");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getBotInfoDescription::ReturnType getBotInfoDescription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getBotInfoDescription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotInfoDescription").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBotInfoShortDescription::Class;
jfieldID getBotInfoShortDescription::bot_user_id_fieldID;
jfieldID getBotInfoShortDescription::language_code_fieldID;

getBotInfoShortDescription::getBotInfoShortDescription()
  : bot_user_id_()
  , language_code_()
{}

getBotInfoShortDescription::getBotInfoShortDescription(int53 bot_user_id_, string const &language_code_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
{}

const std::int32_t getBotInfoShortDescription::ID;

object_ptr<getBotInfoShortDescription> getBotInfoShortDescription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotInfoShortDescription> res = make_object<getBotInfoShortDescription>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getBotInfoShortDescription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotInfoShortDescription");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getBotInfoShortDescription::ReturnType getBotInfoShortDescription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getBotInfoShortDescription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotInfoShortDescription").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBotMediaPreviewInfo::Class;
jfieldID getBotMediaPreviewInfo::bot_user_id_fieldID;
jfieldID getBotMediaPreviewInfo::language_code_fieldID;

getBotMediaPreviewInfo::getBotMediaPreviewInfo()
  : bot_user_id_()
  , language_code_()
{}

getBotMediaPreviewInfo::getBotMediaPreviewInfo(int53 bot_user_id_, string const &language_code_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
{}

const std::int32_t getBotMediaPreviewInfo::ID;

object_ptr<getBotMediaPreviewInfo> getBotMediaPreviewInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotMediaPreviewInfo> res = make_object<getBotMediaPreviewInfo>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getBotMediaPreviewInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotMediaPreviewInfo");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getBotMediaPreviewInfo::ReturnType getBotMediaPreviewInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botMediaPreviewInfo::fetch(env, p);
}

void getBotMediaPreviewInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotMediaPreviewInfo").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBotMediaPreviews::Class;
jfieldID getBotMediaPreviews::bot_user_id_fieldID;

getBotMediaPreviews::getBotMediaPreviews()
  : bot_user_id_()
{}

getBotMediaPreviews::getBotMediaPreviews(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t getBotMediaPreviews::ID;

object_ptr<getBotMediaPreviews> getBotMediaPreviews::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotMediaPreviews> res = make_object<getBotMediaPreviews>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void getBotMediaPreviews::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotMediaPreviews");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

getBotMediaPreviews::ReturnType getBotMediaPreviews::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botMediaPreviews::fetch(env, p);
}

void getBotMediaPreviews::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotMediaPreviews").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getBotName::Class;
jfieldID getBotName::bot_user_id_fieldID;
jfieldID getBotName::language_code_fieldID;

getBotName::getBotName()
  : bot_user_id_()
  , language_code_()
{}

getBotName::getBotName(int53 bot_user_id_, string const &language_code_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
{}

const std::int32_t getBotName::ID;

object_ptr<getBotName> getBotName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotName> res = make_object<getBotName>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getBotName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotName");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getBotName::ReturnType getBotName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getBotName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotName").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBotSimilarBotCount::Class;
jfieldID getBotSimilarBotCount::bot_user_id_fieldID;
jfieldID getBotSimilarBotCount::return_local_fieldID;

getBotSimilarBotCount::getBotSimilarBotCount()
  : bot_user_id_()
  , return_local_()
{}

getBotSimilarBotCount::getBotSimilarBotCount(int53 bot_user_id_, bool return_local_)
  : bot_user_id_(bot_user_id_)
  , return_local_(return_local_)
{}

const std::int32_t getBotSimilarBotCount::ID;

object_ptr<getBotSimilarBotCount> getBotSimilarBotCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotSimilarBotCount> res = make_object<getBotSimilarBotCount>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->return_local_ = (env->GetBooleanField(p, res->return_local_fieldID) != 0);
  return res;
}

void getBotSimilarBotCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotSimilarBotCount");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("returnLocal", return_local_);
    s.store_class_end();
  }
}

getBotSimilarBotCount::ReturnType getBotSimilarBotCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getBotSimilarBotCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotSimilarBotCount").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return_local_fieldID = jni::get_field_id(env, Class, "returnLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getBotSimilarBots::Class;
jfieldID getBotSimilarBots::bot_user_id_fieldID;

getBotSimilarBots::getBotSimilarBots()
  : bot_user_id_()
{}

getBotSimilarBots::getBotSimilarBots(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t getBotSimilarBots::ID;

object_ptr<getBotSimilarBots> getBotSimilarBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBotSimilarBots> res = make_object<getBotSimilarBots>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void getBotSimilarBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBotSimilarBots");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

getBotSimilarBots::ReturnType getBotSimilarBots::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void getBotSimilarBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBotSimilarBots").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessAccountStarAmount::Class;
jfieldID getBusinessAccountStarAmount::business_connection_id_fieldID;

getBusinessAccountStarAmount::getBusinessAccountStarAmount()
  : business_connection_id_()
{}

getBusinessAccountStarAmount::getBusinessAccountStarAmount(string const &business_connection_id_)
  : business_connection_id_(business_connection_id_)
{}

const std::int32_t getBusinessAccountStarAmount::ID;

object_ptr<getBusinessAccountStarAmount> getBusinessAccountStarAmount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBusinessAccountStarAmount> res = make_object<getBusinessAccountStarAmount>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  return res;
}

void getBusinessAccountStarAmount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessAccountStarAmount");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_class_end();
  }
}

getBusinessAccountStarAmount::ReturnType getBusinessAccountStarAmount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starAmount::fetch(env, p);
}

void getBusinessAccountStarAmount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessAccountStarAmount").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessChatLinkInfo::Class;
jfieldID getBusinessChatLinkInfo::link_name_fieldID;

getBusinessChatLinkInfo::getBusinessChatLinkInfo()
  : link_name_()
{}

getBusinessChatLinkInfo::getBusinessChatLinkInfo(string const &link_name_)
  : link_name_(link_name_)
{}

const std::int32_t getBusinessChatLinkInfo::ID;

object_ptr<getBusinessChatLinkInfo> getBusinessChatLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBusinessChatLinkInfo> res = make_object<getBusinessChatLinkInfo>();
  res->link_name_ = jni::fetch_string(env, p, res->link_name_fieldID);
  return res;
}

void getBusinessChatLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessChatLinkInfo");
    s.store_field("linkName", link_name_);
    s.store_class_end();
  }
}

getBusinessChatLinkInfo::ReturnType getBusinessChatLinkInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessChatLinkInfo::fetch(env, p);
}

void getBusinessChatLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessChatLinkInfo").c_str());
    link_name_fieldID = jni::get_field_id(env, Class, "linkName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessChatLinks::Class;

getBusinessChatLinks::getBusinessChatLinks() {
}

const std::int32_t getBusinessChatLinks::ID;

object_ptr<getBusinessChatLinks> getBusinessChatLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getBusinessChatLinks>();
}

void getBusinessChatLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessChatLinks");
    s.store_class_end();
  }
}

getBusinessChatLinks::ReturnType getBusinessChatLinks::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessChatLinks::fetch(env, p);
}

void getBusinessChatLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessChatLinks").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessConnectedBot::Class;

getBusinessConnectedBot::getBusinessConnectedBot() {
}

const std::int32_t getBusinessConnectedBot::ID;

object_ptr<getBusinessConnectedBot> getBusinessConnectedBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getBusinessConnectedBot>();
}

void getBusinessConnectedBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessConnectedBot");
    s.store_class_end();
  }
}

getBusinessConnectedBot::ReturnType getBusinessConnectedBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessConnectedBot::fetch(env, p);
}

void getBusinessConnectedBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessConnectedBot").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessConnection::Class;
jfieldID getBusinessConnection::connection_id_fieldID;

getBusinessConnection::getBusinessConnection()
  : connection_id_()
{}

getBusinessConnection::getBusinessConnection(string const &connection_id_)
  : connection_id_(connection_id_)
{}

const std::int32_t getBusinessConnection::ID;

object_ptr<getBusinessConnection> getBusinessConnection::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBusinessConnection> res = make_object<getBusinessConnection>();
  res->connection_id_ = jni::fetch_string(env, p, res->connection_id_fieldID);
  return res;
}

void getBusinessConnection::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessConnection");
    s.store_field("connectionId", connection_id_);
    s.store_class_end();
  }
}

getBusinessConnection::ReturnType getBusinessConnection::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessConnection::fetch(env, p);
}

void getBusinessConnection::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessConnection").c_str());
    connection_id_fieldID = jni::get_field_id(env, Class, "connectionId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getBusinessFeatures::Class;
jfieldID getBusinessFeatures::source_fieldID;

getBusinessFeatures::getBusinessFeatures()
  : source_()
{}

getBusinessFeatures::getBusinessFeatures(object_ptr<BusinessFeature> &&source_)
  : source_(std::move(source_))
{}

const std::int32_t getBusinessFeatures::ID;

object_ptr<getBusinessFeatures> getBusinessFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getBusinessFeatures> res = make_object<getBusinessFeatures>();
  res->source_ = jni::fetch_tl_object<BusinessFeature>(env, jni::fetch_object(env, p, res->source_fieldID));
  return res;
}

void getBusinessFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetBusinessFeatures");
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_class_end();
  }
}

getBusinessFeatures::ReturnType getBusinessFeatures::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessFeatures::fetch(env, p);
}

void getBusinessFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetBusinessFeatures").c_str());
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCallbackQueryAnswer::Class;
jfieldID getCallbackQueryAnswer::chat_id_fieldID;
jfieldID getCallbackQueryAnswer::message_id_fieldID;
jfieldID getCallbackQueryAnswer::payload_fieldID;

getCallbackQueryAnswer::getCallbackQueryAnswer()
  : chat_id_()
  , message_id_()
  , payload_()
{}

getCallbackQueryAnswer::getCallbackQueryAnswer(int53 chat_id_, int53 message_id_, object_ptr<CallbackQueryPayload> &&payload_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , payload_(std::move(payload_))
{}

const std::int32_t getCallbackQueryAnswer::ID;

object_ptr<getCallbackQueryAnswer> getCallbackQueryAnswer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCallbackQueryAnswer> res = make_object<getCallbackQueryAnswer>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->payload_ = jni::fetch_tl_object<CallbackQueryPayload>(env, jni::fetch_object(env, p, res->payload_fieldID));
  return res;
}

void getCallbackQueryAnswer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCallbackQueryAnswer");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("payload", static_cast<const BaseObject *>(payload_.get()));
    s.store_class_end();
  }
}

getCallbackQueryAnswer::ReturnType getCallbackQueryAnswer::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return callbackQueryAnswer::fetch(env, p);
}

void getCallbackQueryAnswer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCallbackQueryAnswer").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    payload_fieldID = jni::get_field_id(env, Class, "payload", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CallbackQueryPayload;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCallbackQueryMessage::Class;
jfieldID getCallbackQueryMessage::chat_id_fieldID;
jfieldID getCallbackQueryMessage::message_id_fieldID;
jfieldID getCallbackQueryMessage::callback_query_id_fieldID;

getCallbackQueryMessage::getCallbackQueryMessage()
  : chat_id_()
  , message_id_()
  , callback_query_id_()
{}

getCallbackQueryMessage::getCallbackQueryMessage(int53 chat_id_, int53 message_id_, int64 callback_query_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , callback_query_id_(callback_query_id_)
{}

const std::int32_t getCallbackQueryMessage::ID;

object_ptr<getCallbackQueryMessage> getCallbackQueryMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCallbackQueryMessage> res = make_object<getCallbackQueryMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->callback_query_id_ = env->GetLongField(p, res->callback_query_id_fieldID);
  return res;
}

void getCallbackQueryMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCallbackQueryMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("callbackQueryId", callback_query_id_);
    s.store_class_end();
  }
}

getCallbackQueryMessage::ReturnType getCallbackQueryMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getCallbackQueryMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCallbackQueryMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    callback_query_id_fieldID = jni::get_field_id(env, Class, "callbackQueryId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChat::Class;
jfieldID getChat::chat_id_fieldID;

getChat::getChat()
  : chat_id_()
{}

getChat::getChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChat::ID;

object_ptr<getChat> getChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChat> res = make_object<getChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChat::ReturnType getChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void getChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatActiveStories::Class;
jfieldID getChatActiveStories::chat_id_fieldID;

getChatActiveStories::getChatActiveStories()
  : chat_id_()
{}

getChatActiveStories::getChatActiveStories(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatActiveStories::ID;

object_ptr<getChatActiveStories> getChatActiveStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatActiveStories> res = make_object<getChatActiveStories>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatActiveStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatActiveStories");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatActiveStories::ReturnType getChatActiveStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatActiveStories::fetch(env, p);
}

void getChatActiveStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatActiveStories").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatAdministrators::Class;
jfieldID getChatAdministrators::chat_id_fieldID;

getChatAdministrators::getChatAdministrators()
  : chat_id_()
{}

getChatAdministrators::getChatAdministrators(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatAdministrators::ID;

object_ptr<getChatAdministrators> getChatAdministrators::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatAdministrators> res = make_object<getChatAdministrators>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatAdministrators::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatAdministrators");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatAdministrators::ReturnType getChatAdministrators::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatAdministrators::fetch(env, p);
}

void getChatAdministrators::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatAdministrators").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatArchivedStories::Class;
jfieldID getChatArchivedStories::chat_id_fieldID;
jfieldID getChatArchivedStories::from_story_id_fieldID;
jfieldID getChatArchivedStories::limit_fieldID;

getChatArchivedStories::getChatArchivedStories()
  : chat_id_()
  , from_story_id_()
  , limit_()
{}

getChatArchivedStories::getChatArchivedStories(int53 chat_id_, int32 from_story_id_, int32 limit_)
  : chat_id_(chat_id_)
  , from_story_id_(from_story_id_)
  , limit_(limit_)
{}

const std::int32_t getChatArchivedStories::ID;

object_ptr<getChatArchivedStories> getChatArchivedStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatArchivedStories> res = make_object<getChatArchivedStories>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->from_story_id_ = env->GetIntField(p, res->from_story_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatArchivedStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatArchivedStories");
    s.store_field("chatId", chat_id_);
    s.store_field("fromStoryId", from_story_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatArchivedStories::ReturnType getChatArchivedStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stories::fetch(env, p);
}

void getChatArchivedStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatArchivedStories").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    from_story_id_fieldID = jni::get_field_id(env, Class, "fromStoryId", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatAvailableMessageSenders::Class;
jfieldID getChatAvailableMessageSenders::chat_id_fieldID;

getChatAvailableMessageSenders::getChatAvailableMessageSenders()
  : chat_id_()
{}

getChatAvailableMessageSenders::getChatAvailableMessageSenders(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatAvailableMessageSenders::ID;

object_ptr<getChatAvailableMessageSenders> getChatAvailableMessageSenders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatAvailableMessageSenders> res = make_object<getChatAvailableMessageSenders>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatAvailableMessageSenders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatAvailableMessageSenders");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatAvailableMessageSenders::ReturnType getChatAvailableMessageSenders::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatMessageSenders::fetch(env, p);
}

void getChatAvailableMessageSenders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatAvailableMessageSenders").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatAvailablePaidMessageReactionSenders::Class;
jfieldID getChatAvailablePaidMessageReactionSenders::chat_id_fieldID;

getChatAvailablePaidMessageReactionSenders::getChatAvailablePaidMessageReactionSenders()
  : chat_id_()
{}

getChatAvailablePaidMessageReactionSenders::getChatAvailablePaidMessageReactionSenders(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatAvailablePaidMessageReactionSenders::ID;

object_ptr<getChatAvailablePaidMessageReactionSenders> getChatAvailablePaidMessageReactionSenders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatAvailablePaidMessageReactionSenders> res = make_object<getChatAvailablePaidMessageReactionSenders>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatAvailablePaidMessageReactionSenders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatAvailablePaidMessageReactionSenders");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatAvailablePaidMessageReactionSenders::ReturnType getChatAvailablePaidMessageReactionSenders::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageSenders::fetch(env, p);
}

void getChatAvailablePaidMessageReactionSenders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatAvailablePaidMessageReactionSenders").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoostFeatures::Class;
jfieldID getChatBoostFeatures::is_channel_fieldID;

getChatBoostFeatures::getChatBoostFeatures()
  : is_channel_()
{}

getChatBoostFeatures::getChatBoostFeatures(bool is_channel_)
  : is_channel_(is_channel_)
{}

const std::int32_t getChatBoostFeatures::ID;

object_ptr<getChatBoostFeatures> getChatBoostFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoostFeatures> res = make_object<getChatBoostFeatures>();
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  return res;
}

void getChatBoostFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoostFeatures");
    s.store_field("isChannel", is_channel_);
    s.store_class_end();
  }
}

getChatBoostFeatures::ReturnType getChatBoostFeatures::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostFeatures::fetch(env, p);
}

void getChatBoostFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoostFeatures").c_str());
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoostLevelFeatures::Class;
jfieldID getChatBoostLevelFeatures::is_channel_fieldID;
jfieldID getChatBoostLevelFeatures::level_fieldID;

getChatBoostLevelFeatures::getChatBoostLevelFeatures()
  : is_channel_()
  , level_()
{}

getChatBoostLevelFeatures::getChatBoostLevelFeatures(bool is_channel_, int32 level_)
  : is_channel_(is_channel_)
  , level_(level_)
{}

const std::int32_t getChatBoostLevelFeatures::ID;

object_ptr<getChatBoostLevelFeatures> getChatBoostLevelFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoostLevelFeatures> res = make_object<getChatBoostLevelFeatures>();
  res->is_channel_ = (env->GetBooleanField(p, res->is_channel_fieldID) != 0);
  res->level_ = env->GetIntField(p, res->level_fieldID);
  return res;
}

void getChatBoostLevelFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoostLevelFeatures");
    s.store_field("isChannel", is_channel_);
    s.store_field("level", level_);
    s.store_class_end();
  }
}

getChatBoostLevelFeatures::ReturnType getChatBoostLevelFeatures::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostLevelFeatures::fetch(env, p);
}

void getChatBoostLevelFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoostLevelFeatures").c_str());
    is_channel_fieldID = jni::get_field_id(env, Class, "isChannel", "Z");
    level_fieldID = jni::get_field_id(env, Class, "level", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoostLink::Class;
jfieldID getChatBoostLink::chat_id_fieldID;

getChatBoostLink::getChatBoostLink()
  : chat_id_()
{}

getChatBoostLink::getChatBoostLink(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatBoostLink::ID;

object_ptr<getChatBoostLink> getChatBoostLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoostLink> res = make_object<getChatBoostLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatBoostLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoostLink");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatBoostLink::ReturnType getChatBoostLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostLink::fetch(env, p);
}

void getChatBoostLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoostLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoostLinkInfo::Class;
jfieldID getChatBoostLinkInfo::url_fieldID;

getChatBoostLinkInfo::getChatBoostLinkInfo()
  : url_()
{}

getChatBoostLinkInfo::getChatBoostLinkInfo(string const &url_)
  : url_(url_)
{}

const std::int32_t getChatBoostLinkInfo::ID;

object_ptr<getChatBoostLinkInfo> getChatBoostLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoostLinkInfo> res = make_object<getChatBoostLinkInfo>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void getChatBoostLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoostLinkInfo");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

getChatBoostLinkInfo::ReturnType getChatBoostLinkInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostLinkInfo::fetch(env, p);
}

void getChatBoostLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoostLinkInfo").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoostStatus::Class;
jfieldID getChatBoostStatus::chat_id_fieldID;

getChatBoostStatus::getChatBoostStatus()
  : chat_id_()
{}

getChatBoostStatus::getChatBoostStatus(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatBoostStatus::ID;

object_ptr<getChatBoostStatus> getChatBoostStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoostStatus> res = make_object<getChatBoostStatus>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatBoostStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoostStatus");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatBoostStatus::ReturnType getChatBoostStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatBoostStatus::fetch(env, p);
}

void getChatBoostStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoostStatus").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatBoosts::Class;
jfieldID getChatBoosts::chat_id_fieldID;
jfieldID getChatBoosts::only_gift_codes_fieldID;
jfieldID getChatBoosts::offset_fieldID;
jfieldID getChatBoosts::limit_fieldID;

getChatBoosts::getChatBoosts()
  : chat_id_()
  , only_gift_codes_()
  , offset_()
  , limit_()
{}

getChatBoosts::getChatBoosts(int53 chat_id_, bool only_gift_codes_, string const &offset_, int32 limit_)
  : chat_id_(chat_id_)
  , only_gift_codes_(only_gift_codes_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getChatBoosts::ID;

object_ptr<getChatBoosts> getChatBoosts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatBoosts> res = make_object<getChatBoosts>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->only_gift_codes_ = (env->GetBooleanField(p, res->only_gift_codes_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatBoosts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatBoosts");
    s.store_field("chatId", chat_id_);
    s.store_field("onlyGiftCodes", only_gift_codes_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatBoosts::ReturnType getChatBoosts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundChatBoosts::fetch(env, p);
}

void getChatBoosts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatBoosts").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    only_gift_codes_fieldID = jni::get_field_id(env, Class, "onlyGiftCodes", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatEventLog::Class;
jfieldID getChatEventLog::chat_id_fieldID;
jfieldID getChatEventLog::query_fieldID;
jfieldID getChatEventLog::from_event_id_fieldID;
jfieldID getChatEventLog::limit_fieldID;
jfieldID getChatEventLog::filters_fieldID;
jfieldID getChatEventLog::user_ids_fieldID;

getChatEventLog::getChatEventLog()
  : chat_id_()
  , query_()
  , from_event_id_()
  , limit_()
  , filters_()
  , user_ids_()
{}

getChatEventLog::getChatEventLog(int53 chat_id_, string const &query_, int64 from_event_id_, int32 limit_, object_ptr<chatEventLogFilters> &&filters_, array<int53> &&user_ids_)
  : chat_id_(chat_id_)
  , query_(query_)
  , from_event_id_(from_event_id_)
  , limit_(limit_)
  , filters_(std::move(filters_))
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t getChatEventLog::ID;

object_ptr<getChatEventLog> getChatEventLog::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatEventLog> res = make_object<getChatEventLog>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->from_event_id_ = env->GetLongField(p, res->from_event_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->filters_ = jni::fetch_tl_object<chatEventLogFilters>(env, jni::fetch_object(env, p, res->filters_fieldID));
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void getChatEventLog::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatEventLog");
    s.store_field("chatId", chat_id_);
    s.store_field("query", query_);
    s.store_field("fromEventId", from_event_id_);
    s.store_field("limit", limit_);
    s.store_object_field("filters", static_cast<const BaseObject *>(filters_.get()));
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

getChatEventLog::ReturnType getChatEventLog::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatEvents::fetch(env, p);
}

void getChatEventLog::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatEventLog").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    from_event_id_fieldID = jni::get_field_id(env, Class, "fromEventId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    filters_fieldID = jni::get_field_id(env, Class, "filters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatEventLogFilters;").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolder::Class;
jfieldID getChatFolder::chat_folder_id_fieldID;

getChatFolder::getChatFolder()
  : chat_folder_id_()
{}

getChatFolder::getChatFolder(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t getChatFolder::ID;

object_ptr<getChatFolder> getChatFolder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolder> res = make_object<getChatFolder>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return res;
}

void getChatFolder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolder");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

getChatFolder::ReturnType getChatFolder::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolder::fetch(env, p);
}

void getChatFolder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolder").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolderChatCount::Class;
jfieldID getChatFolderChatCount::folder_fieldID;

getChatFolderChatCount::getChatFolderChatCount()
  : folder_()
{}

getChatFolderChatCount::getChatFolderChatCount(object_ptr<chatFolder> &&folder_)
  : folder_(std::move(folder_))
{}

const std::int32_t getChatFolderChatCount::ID;

object_ptr<getChatFolderChatCount> getChatFolderChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolderChatCount> res = make_object<getChatFolderChatCount>();
  res->folder_ = jni::fetch_tl_object<chatFolder>(env, jni::fetch_object(env, p, res->folder_fieldID));
  return res;
}

void getChatFolderChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolderChatCount");
    s.store_object_field("folder", static_cast<const BaseObject *>(folder_.get()));
    s.store_class_end();
  }
}

getChatFolderChatCount::ReturnType getChatFolderChatCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getChatFolderChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolderChatCount").c_str());
    folder_fieldID = jni::get_field_id(env, Class, "folder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolder;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolderChatsToLeave::Class;
jfieldID getChatFolderChatsToLeave::chat_folder_id_fieldID;

getChatFolderChatsToLeave::getChatFolderChatsToLeave()
  : chat_folder_id_()
{}

getChatFolderChatsToLeave::getChatFolderChatsToLeave(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t getChatFolderChatsToLeave::ID;

object_ptr<getChatFolderChatsToLeave> getChatFolderChatsToLeave::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolderChatsToLeave> res = make_object<getChatFolderChatsToLeave>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return res;
}

void getChatFolderChatsToLeave::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolderChatsToLeave");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

getChatFolderChatsToLeave::ReturnType getChatFolderChatsToLeave::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatFolderChatsToLeave::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolderChatsToLeave").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolderDefaultIconName::Class;
jfieldID getChatFolderDefaultIconName::folder_fieldID;

getChatFolderDefaultIconName::getChatFolderDefaultIconName()
  : folder_()
{}

getChatFolderDefaultIconName::getChatFolderDefaultIconName(object_ptr<chatFolder> &&folder_)
  : folder_(std::move(folder_))
{}

const std::int32_t getChatFolderDefaultIconName::ID;

object_ptr<getChatFolderDefaultIconName> getChatFolderDefaultIconName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolderDefaultIconName> res = make_object<getChatFolderDefaultIconName>();
  res->folder_ = jni::fetch_tl_object<chatFolder>(env, jni::fetch_object(env, p, res->folder_fieldID));
  return res;
}

void getChatFolderDefaultIconName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolderDefaultIconName");
    s.store_object_field("folder", static_cast<const BaseObject *>(folder_.get()));
    s.store_class_end();
  }
}

getChatFolderDefaultIconName::ReturnType getChatFolderDefaultIconName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderIcon::fetch(env, p);
}

void getChatFolderDefaultIconName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolderDefaultIconName").c_str());
    folder_fieldID = jni::get_field_id(env, Class, "folder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatFolder;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolderInviteLinks::Class;
jfieldID getChatFolderInviteLinks::chat_folder_id_fieldID;

getChatFolderInviteLinks::getChatFolderInviteLinks()
  : chat_folder_id_()
{}

getChatFolderInviteLinks::getChatFolderInviteLinks(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t getChatFolderInviteLinks::ID;

object_ptr<getChatFolderInviteLinks> getChatFolderInviteLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolderInviteLinks> res = make_object<getChatFolderInviteLinks>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return res;
}

void getChatFolderInviteLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolderInviteLinks");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

getChatFolderInviteLinks::ReturnType getChatFolderInviteLinks::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatFolderInviteLinks::fetch(env, p);
}

void getChatFolderInviteLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolderInviteLinks").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatFolderNewChats::Class;
jfieldID getChatFolderNewChats::chat_folder_id_fieldID;

getChatFolderNewChats::getChatFolderNewChats()
  : chat_folder_id_()
{}

getChatFolderNewChats::getChatFolderNewChats(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t getChatFolderNewChats::ID;

object_ptr<getChatFolderNewChats> getChatFolderNewChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatFolderNewChats> res = make_object<getChatFolderNewChats>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return res;
}

void getChatFolderNewChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatFolderNewChats");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

getChatFolderNewChats::ReturnType getChatFolderNewChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatFolderNewChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatFolderNewChats").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatHistory::Class;
jfieldID getChatHistory::chat_id_fieldID;
jfieldID getChatHistory::from_message_id_fieldID;
jfieldID getChatHistory::offset_fieldID;
jfieldID getChatHistory::limit_fieldID;
jfieldID getChatHistory::only_local_fieldID;

getChatHistory::getChatHistory()
  : chat_id_()
  , from_message_id_()
  , offset_()
  , limit_()
  , only_local_()
{}

getChatHistory::getChatHistory(int53 chat_id_, int53 from_message_id_, int32 offset_, int32 limit_, bool only_local_)
  : chat_id_(chat_id_)
  , from_message_id_(from_message_id_)
  , offset_(offset_)
  , limit_(limit_)
  , only_local_(only_local_)
{}

const std::int32_t getChatHistory::ID;

object_ptr<getChatHistory> getChatHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatHistory> res = make_object<getChatHistory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void getChatHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatHistory");
    s.store_field("chatId", chat_id_);
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

getChatHistory::ReturnType getChatHistory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void getChatHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatHistory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatInviteLink::Class;
jfieldID getChatInviteLink::chat_id_fieldID;
jfieldID getChatInviteLink::invite_link_fieldID;

getChatInviteLink::getChatInviteLink()
  : chat_id_()
  , invite_link_()
{}

getChatInviteLink::getChatInviteLink(int53 chat_id_, string const &invite_link_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
{}

const std::int32_t getChatInviteLink::ID;

object_ptr<getChatInviteLink> getChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatInviteLink> res = make_object<getChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void getChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

getChatInviteLink::ReturnType getChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void getChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getChatInviteLinkCounts::Class;
jfieldID getChatInviteLinkCounts::chat_id_fieldID;

getChatInviteLinkCounts::getChatInviteLinkCounts()
  : chat_id_()
{}

getChatInviteLinkCounts::getChatInviteLinkCounts(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatInviteLinkCounts::ID;

object_ptr<getChatInviteLinkCounts> getChatInviteLinkCounts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatInviteLinkCounts> res = make_object<getChatInviteLinkCounts>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatInviteLinkCounts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatInviteLinkCounts");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatInviteLinkCounts::ReturnType getChatInviteLinkCounts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLinkCounts::fetch(env, p);
}

void getChatInviteLinkCounts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatInviteLinkCounts").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatInviteLinkMembers::Class;
jfieldID getChatInviteLinkMembers::chat_id_fieldID;
jfieldID getChatInviteLinkMembers::invite_link_fieldID;
jfieldID getChatInviteLinkMembers::only_with_expired_subscription_fieldID;
jfieldID getChatInviteLinkMembers::offset_member_fieldID;
jfieldID getChatInviteLinkMembers::limit_fieldID;

getChatInviteLinkMembers::getChatInviteLinkMembers()
  : chat_id_()
  , invite_link_()
  , only_with_expired_subscription_()
  , offset_member_()
  , limit_()
{}

getChatInviteLinkMembers::getChatInviteLinkMembers(int53 chat_id_, string const &invite_link_, bool only_with_expired_subscription_, object_ptr<chatInviteLinkMember> &&offset_member_, int32 limit_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
  , only_with_expired_subscription_(only_with_expired_subscription_)
  , offset_member_(std::move(offset_member_))
  , limit_(limit_)
{}

const std::int32_t getChatInviteLinkMembers::ID;

object_ptr<getChatInviteLinkMembers> getChatInviteLinkMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatInviteLinkMembers> res = make_object<getChatInviteLinkMembers>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->only_with_expired_subscription_ = (env->GetBooleanField(p, res->only_with_expired_subscription_fieldID) != 0);
  res->offset_member_ = jni::fetch_tl_object<chatInviteLinkMember>(env, jni::fetch_object(env, p, res->offset_member_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatInviteLinkMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatInviteLinkMembers");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("onlyWithExpiredSubscription", only_with_expired_subscription_);
    s.store_object_field("offsetMember", static_cast<const BaseObject *>(offset_member_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatInviteLinkMembers::ReturnType getChatInviteLinkMembers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLinkMembers::fetch(env, p);
}

void getChatInviteLinkMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatInviteLinkMembers").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    only_with_expired_subscription_fieldID = jni::get_field_id(env, Class, "onlyWithExpiredSubscription", "Z");
    offset_member_fieldID = jni::get_field_id(env, Class, "offsetMember", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatInviteLinkMember;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatInviteLinks::Class;
jfieldID getChatInviteLinks::chat_id_fieldID;
jfieldID getChatInviteLinks::creator_user_id_fieldID;
jfieldID getChatInviteLinks::is_revoked_fieldID;
jfieldID getChatInviteLinks::offset_date_fieldID;
jfieldID getChatInviteLinks::offset_invite_link_fieldID;
jfieldID getChatInviteLinks::limit_fieldID;

getChatInviteLinks::getChatInviteLinks()
  : chat_id_()
  , creator_user_id_()
  , is_revoked_()
  , offset_date_()
  , offset_invite_link_()
  , limit_()
{}

getChatInviteLinks::getChatInviteLinks(int53 chat_id_, int53 creator_user_id_, bool is_revoked_, int32 offset_date_, string const &offset_invite_link_, int32 limit_)
  : chat_id_(chat_id_)
  , creator_user_id_(creator_user_id_)
  , is_revoked_(is_revoked_)
  , offset_date_(offset_date_)
  , offset_invite_link_(offset_invite_link_)
  , limit_(limit_)
{}

const std::int32_t getChatInviteLinks::ID;

object_ptr<getChatInviteLinks> getChatInviteLinks::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatInviteLinks> res = make_object<getChatInviteLinks>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->creator_user_id_ = env->GetLongField(p, res->creator_user_id_fieldID);
  res->is_revoked_ = (env->GetBooleanField(p, res->is_revoked_fieldID) != 0);
  res->offset_date_ = env->GetIntField(p, res->offset_date_fieldID);
  res->offset_invite_link_ = jni::fetch_string(env, p, res->offset_invite_link_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatInviteLinks::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatInviteLinks");
    s.store_field("chatId", chat_id_);
    s.store_field("creatorUserId", creator_user_id_);
    s.store_field("isRevoked", is_revoked_);
    s.store_field("offsetDate", offset_date_);
    s.store_field("offsetInviteLink", offset_invite_link_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatInviteLinks::ReturnType getChatInviteLinks::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLinks::fetch(env, p);
}

void getChatInviteLinks::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatInviteLinks").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    creator_user_id_fieldID = jni::get_field_id(env, Class, "creatorUserId", "J");
    is_revoked_fieldID = jni::get_field_id(env, Class, "isRevoked", "Z");
    offset_date_fieldID = jni::get_field_id(env, Class, "offsetDate", "I");
    offset_invite_link_fieldID = jni::get_field_id(env, Class, "offsetInviteLink", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatJoinRequests::Class;
jfieldID getChatJoinRequests::chat_id_fieldID;
jfieldID getChatJoinRequests::invite_link_fieldID;
jfieldID getChatJoinRequests::query_fieldID;
jfieldID getChatJoinRequests::offset_request_fieldID;
jfieldID getChatJoinRequests::limit_fieldID;

getChatJoinRequests::getChatJoinRequests()
  : chat_id_()
  , invite_link_()
  , query_()
  , offset_request_()
  , limit_()
{}

getChatJoinRequests::getChatJoinRequests(int53 chat_id_, string const &invite_link_, string const &query_, object_ptr<chatJoinRequest> &&offset_request_, int32 limit_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
  , query_(query_)
  , offset_request_(std::move(offset_request_))
  , limit_(limit_)
{}

const std::int32_t getChatJoinRequests::ID;

object_ptr<getChatJoinRequests> getChatJoinRequests::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatJoinRequests> res = make_object<getChatJoinRequests>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_request_ = jni::fetch_tl_object<chatJoinRequest>(env, jni::fetch_object(env, p, res->offset_request_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatJoinRequests::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatJoinRequests");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("query", query_);
    s.store_object_field("offsetRequest", static_cast<const BaseObject *>(offset_request_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatJoinRequests::ReturnType getChatJoinRequests::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatJoinRequests::fetch(env, p);
}

void getChatJoinRequests::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatJoinRequests").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_request_fieldID = jni::get_field_id(env, Class, "offsetRequest", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatJoinRequest;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatListsToAddChat::Class;
jfieldID getChatListsToAddChat::chat_id_fieldID;

getChatListsToAddChat::getChatListsToAddChat()
  : chat_id_()
{}

getChatListsToAddChat::getChatListsToAddChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatListsToAddChat::ID;

object_ptr<getChatListsToAddChat> getChatListsToAddChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatListsToAddChat> res = make_object<getChatListsToAddChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatListsToAddChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatListsToAddChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatListsToAddChat::ReturnType getChatListsToAddChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatLists::fetch(env, p);
}

void getChatListsToAddChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatListsToAddChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatMember::Class;
jfieldID getChatMember::chat_id_fieldID;
jfieldID getChatMember::member_id_fieldID;

getChatMember::getChatMember()
  : chat_id_()
  , member_id_()
{}

getChatMember::getChatMember(int53 chat_id_, object_ptr<MessageSender> &&member_id_)
  : chat_id_(chat_id_)
  , member_id_(std::move(member_id_))
{}

const std::int32_t getChatMember::ID;

object_ptr<getChatMember> getChatMember::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatMember> res = make_object<getChatMember>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  return res;
}

void getChatMember::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatMember");
    s.store_field("chatId", chat_id_);
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_class_end();
  }
}

getChatMember::ReturnType getChatMember::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatMember::fetch(env, p);
}

void getChatMember::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatMember").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getChatMessageByDate::Class;
jfieldID getChatMessageByDate::chat_id_fieldID;
jfieldID getChatMessageByDate::date_fieldID;

getChatMessageByDate::getChatMessageByDate()
  : chat_id_()
  , date_()
{}

getChatMessageByDate::getChatMessageByDate(int53 chat_id_, int32 date_)
  : chat_id_(chat_id_)
  , date_(date_)
{}

const std::int32_t getChatMessageByDate::ID;

object_ptr<getChatMessageByDate> getChatMessageByDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatMessageByDate> res = make_object<getChatMessageByDate>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void getChatMessageByDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatMessageByDate");
    s.store_field("chatId", chat_id_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

getChatMessageByDate::ReturnType getChatMessageByDate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getChatMessageByDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatMessageByDate").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatMessageCalendar::Class;
jfieldID getChatMessageCalendar::chat_id_fieldID;
jfieldID getChatMessageCalendar::filter_fieldID;
jfieldID getChatMessageCalendar::from_message_id_fieldID;
jfieldID getChatMessageCalendar::saved_messages_topic_id_fieldID;

getChatMessageCalendar::getChatMessageCalendar()
  : chat_id_()
  , filter_()
  , from_message_id_()
  , saved_messages_topic_id_()
{}

getChatMessageCalendar::getChatMessageCalendar(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 from_message_id_, int53 saved_messages_topic_id_)
  : chat_id_(chat_id_)
  , filter_(std::move(filter_))
  , from_message_id_(from_message_id_)
  , saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t getChatMessageCalendar::ID;

object_ptr<getChatMessageCalendar> getChatMessageCalendar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatMessageCalendar> res = make_object<getChatMessageCalendar>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void getChatMessageCalendar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatMessageCalendar");
    s.store_field("chatId", chat_id_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

getChatMessageCalendar::ReturnType getChatMessageCalendar::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageCalendar::fetch(env, p);
}

void getChatMessageCalendar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatMessageCalendar").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatMessageCount::Class;
jfieldID getChatMessageCount::chat_id_fieldID;
jfieldID getChatMessageCount::filter_fieldID;
jfieldID getChatMessageCount::saved_messages_topic_id_fieldID;
jfieldID getChatMessageCount::return_local_fieldID;

getChatMessageCount::getChatMessageCount()
  : chat_id_()
  , filter_()
  , saved_messages_topic_id_()
  , return_local_()
{}

getChatMessageCount::getChatMessageCount(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 saved_messages_topic_id_, bool return_local_)
  : chat_id_(chat_id_)
  , filter_(std::move(filter_))
  , saved_messages_topic_id_(saved_messages_topic_id_)
  , return_local_(return_local_)
{}

const std::int32_t getChatMessageCount::ID;

object_ptr<getChatMessageCount> getChatMessageCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatMessageCount> res = make_object<getChatMessageCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->return_local_ = (env->GetBooleanField(p, res->return_local_fieldID) != 0);
  return res;
}

void getChatMessageCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatMessageCount");
    s.store_field("chatId", chat_id_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_field("returnLocal", return_local_);
    s.store_class_end();
  }
}

getChatMessageCount::ReturnType getChatMessageCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getChatMessageCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatMessageCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return_local_fieldID = jni::get_field_id(env, Class, "returnLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatMessagePosition::Class;
jfieldID getChatMessagePosition::chat_id_fieldID;
jfieldID getChatMessagePosition::message_id_fieldID;
jfieldID getChatMessagePosition::filter_fieldID;
jfieldID getChatMessagePosition::message_thread_id_fieldID;
jfieldID getChatMessagePosition::saved_messages_topic_id_fieldID;

getChatMessagePosition::getChatMessagePosition()
  : chat_id_()
  , message_id_()
  , filter_()
  , message_thread_id_()
  , saved_messages_topic_id_()
{}

getChatMessagePosition::getChatMessagePosition(int53 chat_id_, int53 message_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 message_thread_id_, int53 saved_messages_topic_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , filter_(std::move(filter_))
  , message_thread_id_(message_thread_id_)
  , saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t getChatMessagePosition::ID;

object_ptr<getChatMessagePosition> getChatMessagePosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatMessagePosition> res = make_object<getChatMessagePosition>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void getChatMessagePosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatMessagePosition");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

getChatMessagePosition::ReturnType getChatMessagePosition::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getChatMessagePosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatMessagePosition").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatNotificationSettingsExceptions::Class;
jfieldID getChatNotificationSettingsExceptions::scope_fieldID;
jfieldID getChatNotificationSettingsExceptions::compare_sound_fieldID;

getChatNotificationSettingsExceptions::getChatNotificationSettingsExceptions()
  : scope_()
  , compare_sound_()
{}

getChatNotificationSettingsExceptions::getChatNotificationSettingsExceptions(object_ptr<NotificationSettingsScope> &&scope_, bool compare_sound_)
  : scope_(std::move(scope_))
  , compare_sound_(compare_sound_)
{}

const std::int32_t getChatNotificationSettingsExceptions::ID;

object_ptr<getChatNotificationSettingsExceptions> getChatNotificationSettingsExceptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatNotificationSettingsExceptions> res = make_object<getChatNotificationSettingsExceptions>();
  res->scope_ = jni::fetch_tl_object<NotificationSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->compare_sound_ = (env->GetBooleanField(p, res->compare_sound_fieldID) != 0);
  return res;
}

void getChatNotificationSettingsExceptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatNotificationSettingsExceptions");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_field("compareSound", compare_sound_);
    s.store_class_end();
  }
}

getChatNotificationSettingsExceptions::ReturnType getChatNotificationSettingsExceptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatNotificationSettingsExceptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatNotificationSettingsExceptions").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationSettingsScope;").c_str());
    compare_sound_fieldID = jni::get_field_id(env, Class, "compareSound", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatPinnedMessage::Class;
jfieldID getChatPinnedMessage::chat_id_fieldID;

getChatPinnedMessage::getChatPinnedMessage()
  : chat_id_()
{}

getChatPinnedMessage::getChatPinnedMessage(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatPinnedMessage::ID;

object_ptr<getChatPinnedMessage> getChatPinnedMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatPinnedMessage> res = make_object<getChatPinnedMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatPinnedMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatPinnedMessage");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatPinnedMessage::ReturnType getChatPinnedMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getChatPinnedMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatPinnedMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatPostedToChatPageStories::Class;
jfieldID getChatPostedToChatPageStories::chat_id_fieldID;
jfieldID getChatPostedToChatPageStories::from_story_id_fieldID;
jfieldID getChatPostedToChatPageStories::limit_fieldID;

getChatPostedToChatPageStories::getChatPostedToChatPageStories()
  : chat_id_()
  , from_story_id_()
  , limit_()
{}

getChatPostedToChatPageStories::getChatPostedToChatPageStories(int53 chat_id_, int32 from_story_id_, int32 limit_)
  : chat_id_(chat_id_)
  , from_story_id_(from_story_id_)
  , limit_(limit_)
{}

const std::int32_t getChatPostedToChatPageStories::ID;

object_ptr<getChatPostedToChatPageStories> getChatPostedToChatPageStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatPostedToChatPageStories> res = make_object<getChatPostedToChatPageStories>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->from_story_id_ = env->GetIntField(p, res->from_story_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatPostedToChatPageStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatPostedToChatPageStories");
    s.store_field("chatId", chat_id_);
    s.store_field("fromStoryId", from_story_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatPostedToChatPageStories::ReturnType getChatPostedToChatPageStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stories::fetch(env, p);
}

void getChatPostedToChatPageStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatPostedToChatPageStories").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    from_story_id_fieldID = jni::get_field_id(env, Class, "fromStoryId", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatRevenueStatistics::Class;
jfieldID getChatRevenueStatistics::chat_id_fieldID;
jfieldID getChatRevenueStatistics::is_dark_fieldID;

getChatRevenueStatistics::getChatRevenueStatistics()
  : chat_id_()
  , is_dark_()
{}

getChatRevenueStatistics::getChatRevenueStatistics(int53 chat_id_, bool is_dark_)
  : chat_id_(chat_id_)
  , is_dark_(is_dark_)
{}

const std::int32_t getChatRevenueStatistics::ID;

object_ptr<getChatRevenueStatistics> getChatRevenueStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatRevenueStatistics> res = make_object<getChatRevenueStatistics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  return res;
}

void getChatRevenueStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatRevenueStatistics");
    s.store_field("chatId", chat_id_);
    s.store_field("isDark", is_dark_);
    s.store_class_end();
  }
}

getChatRevenueStatistics::ReturnType getChatRevenueStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatRevenueStatistics::fetch(env, p);
}

void getChatRevenueStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatRevenueStatistics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatRevenueTransactions::Class;
jfieldID getChatRevenueTransactions::chat_id_fieldID;
jfieldID getChatRevenueTransactions::offset_fieldID;
jfieldID getChatRevenueTransactions::limit_fieldID;

getChatRevenueTransactions::getChatRevenueTransactions()
  : chat_id_()
  , offset_()
  , limit_()
{}

getChatRevenueTransactions::getChatRevenueTransactions(int53 chat_id_, int32 offset_, int32 limit_)
  : chat_id_(chat_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getChatRevenueTransactions::ID;

object_ptr<getChatRevenueTransactions> getChatRevenueTransactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatRevenueTransactions> res = make_object<getChatRevenueTransactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatRevenueTransactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatRevenueTransactions");
    s.store_field("chatId", chat_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatRevenueTransactions::ReturnType getChatRevenueTransactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatRevenueTransactions::fetch(env, p);
}

void getChatRevenueTransactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatRevenueTransactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatRevenueWithdrawalUrl::Class;
jfieldID getChatRevenueWithdrawalUrl::chat_id_fieldID;
jfieldID getChatRevenueWithdrawalUrl::password_fieldID;

getChatRevenueWithdrawalUrl::getChatRevenueWithdrawalUrl()
  : chat_id_()
  , password_()
{}

getChatRevenueWithdrawalUrl::getChatRevenueWithdrawalUrl(int53 chat_id_, string const &password_)
  : chat_id_(chat_id_)
  , password_(password_)
{}

const std::int32_t getChatRevenueWithdrawalUrl::ID;

object_ptr<getChatRevenueWithdrawalUrl> getChatRevenueWithdrawalUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatRevenueWithdrawalUrl> res = make_object<getChatRevenueWithdrawalUrl>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getChatRevenueWithdrawalUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatRevenueWithdrawalUrl");
    s.store_field("chatId", chat_id_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getChatRevenueWithdrawalUrl::ReturnType getChatRevenueWithdrawalUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getChatRevenueWithdrawalUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatRevenueWithdrawalUrl").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getChatScheduledMessages::Class;
jfieldID getChatScheduledMessages::chat_id_fieldID;

getChatScheduledMessages::getChatScheduledMessages()
  : chat_id_()
{}

getChatScheduledMessages::getChatScheduledMessages(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatScheduledMessages::ID;

object_ptr<getChatScheduledMessages> getChatScheduledMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatScheduledMessages> res = make_object<getChatScheduledMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatScheduledMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatScheduledMessages");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatScheduledMessages::ReturnType getChatScheduledMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void getChatScheduledMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatScheduledMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatSimilarChatCount::Class;
jfieldID getChatSimilarChatCount::chat_id_fieldID;
jfieldID getChatSimilarChatCount::return_local_fieldID;

getChatSimilarChatCount::getChatSimilarChatCount()
  : chat_id_()
  , return_local_()
{}

getChatSimilarChatCount::getChatSimilarChatCount(int53 chat_id_, bool return_local_)
  : chat_id_(chat_id_)
  , return_local_(return_local_)
{}

const std::int32_t getChatSimilarChatCount::ID;

object_ptr<getChatSimilarChatCount> getChatSimilarChatCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatSimilarChatCount> res = make_object<getChatSimilarChatCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->return_local_ = (env->GetBooleanField(p, res->return_local_fieldID) != 0);
  return res;
}

void getChatSimilarChatCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatSimilarChatCount");
    s.store_field("chatId", chat_id_);
    s.store_field("returnLocal", return_local_);
    s.store_class_end();
  }
}

getChatSimilarChatCount::ReturnType getChatSimilarChatCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getChatSimilarChatCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatSimilarChatCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return_local_fieldID = jni::get_field_id(env, Class, "returnLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatSimilarChats::Class;
jfieldID getChatSimilarChats::chat_id_fieldID;

getChatSimilarChats::getChatSimilarChats()
  : chat_id_()
{}

getChatSimilarChats::getChatSimilarChats(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatSimilarChats::ID;

object_ptr<getChatSimilarChats> getChatSimilarChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatSimilarChats> res = make_object<getChatSimilarChats>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatSimilarChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatSimilarChats");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatSimilarChats::ReturnType getChatSimilarChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatSimilarChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatSimilarChats").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatSparseMessagePositions::Class;
jfieldID getChatSparseMessagePositions::chat_id_fieldID;
jfieldID getChatSparseMessagePositions::filter_fieldID;
jfieldID getChatSparseMessagePositions::from_message_id_fieldID;
jfieldID getChatSparseMessagePositions::limit_fieldID;
jfieldID getChatSparseMessagePositions::saved_messages_topic_id_fieldID;

getChatSparseMessagePositions::getChatSparseMessagePositions()
  : chat_id_()
  , filter_()
  , from_message_id_()
  , limit_()
  , saved_messages_topic_id_()
{}

getChatSparseMessagePositions::getChatSparseMessagePositions(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 from_message_id_, int32 limit_, int53 saved_messages_topic_id_)
  : chat_id_(chat_id_)
  , filter_(std::move(filter_))
  , from_message_id_(from_message_id_)
  , limit_(limit_)
  , saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t getChatSparseMessagePositions::ID;

object_ptr<getChatSparseMessagePositions> getChatSparseMessagePositions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatSparseMessagePositions> res = make_object<getChatSparseMessagePositions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void getChatSparseMessagePositions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatSparseMessagePositions");
    s.store_field("chatId", chat_id_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("limit", limit_);
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

getChatSparseMessagePositions::ReturnType getChatSparseMessagePositions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messagePositions::fetch(env, p);
}

void getChatSparseMessagePositions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatSparseMessagePositions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatSponsoredMessages::Class;
jfieldID getChatSponsoredMessages::chat_id_fieldID;

getChatSponsoredMessages::getChatSponsoredMessages()
  : chat_id_()
{}

getChatSponsoredMessages::getChatSponsoredMessages(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getChatSponsoredMessages::ID;

object_ptr<getChatSponsoredMessages> getChatSponsoredMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatSponsoredMessages> res = make_object<getChatSponsoredMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getChatSponsoredMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatSponsoredMessages");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getChatSponsoredMessages::ReturnType getChatSponsoredMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sponsoredMessages::fetch(env, p);
}

void getChatSponsoredMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatSponsoredMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getChatStatistics::Class;
jfieldID getChatStatistics::chat_id_fieldID;
jfieldID getChatStatistics::is_dark_fieldID;

getChatStatistics::getChatStatistics()
  : chat_id_()
  , is_dark_()
{}

getChatStatistics::getChatStatistics(int53 chat_id_, bool is_dark_)
  : chat_id_(chat_id_)
  , is_dark_(is_dark_)
{}

const std::int32_t getChatStatistics::ID;

object_ptr<getChatStatistics> getChatStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatStatistics> res = make_object<getChatStatistics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  return res;
}

void getChatStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatStatistics");
    s.store_field("chatId", chat_id_);
    s.store_field("isDark", is_dark_);
    s.store_class_end();
  }
}

getChatStatistics::ReturnType getChatStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ChatStatistics::fetch(env, p);
}

void getChatStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatStatistics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getChatStoryInteractions::Class;
jfieldID getChatStoryInteractions::story_poster_chat_id_fieldID;
jfieldID getChatStoryInteractions::story_id_fieldID;
jfieldID getChatStoryInteractions::reaction_type_fieldID;
jfieldID getChatStoryInteractions::prefer_forwards_fieldID;
jfieldID getChatStoryInteractions::offset_fieldID;
jfieldID getChatStoryInteractions::limit_fieldID;

getChatStoryInteractions::getChatStoryInteractions()
  : story_poster_chat_id_()
  , story_id_()
  , reaction_type_()
  , prefer_forwards_()
  , offset_()
  , limit_()
{}

getChatStoryInteractions::getChatStoryInteractions(int53 story_poster_chat_id_, int32 story_id_, object_ptr<ReactionType> &&reaction_type_, bool prefer_forwards_, string const &offset_, int32 limit_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , reaction_type_(std::move(reaction_type_))
  , prefer_forwards_(prefer_forwards_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getChatStoryInteractions::ID;

object_ptr<getChatStoryInteractions> getChatStoryInteractions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatStoryInteractions> res = make_object<getChatStoryInteractions>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->prefer_forwards_ = (env->GetBooleanField(p, res->prefer_forwards_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChatStoryInteractions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatStoryInteractions");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("preferForwards", prefer_forwards_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChatStoryInteractions::ReturnType getChatStoryInteractions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storyInteractions::fetch(env, p);
}

void getChatStoryInteractions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatStoryInteractions").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    prefer_forwards_fieldID = jni::get_field_id(env, Class, "preferForwards", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChats::Class;
jfieldID getChats::chat_list_fieldID;
jfieldID getChats::limit_fieldID;

getChats::getChats()
  : chat_list_()
  , limit_()
{}

getChats::getChats(object_ptr<ChatList> &&chat_list_, int32 limit_)
  : chat_list_(std::move(chat_list_))
  , limit_(limit_)
{}

const std::int32_t getChats::ID;

object_ptr<getChats> getChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChats> res = make_object<getChats>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChats");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getChats::ReturnType getChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChats").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatsForChatFolderInviteLink::Class;
jfieldID getChatsForChatFolderInviteLink::chat_folder_id_fieldID;

getChatsForChatFolderInviteLink::getChatsForChatFolderInviteLink()
  : chat_folder_id_()
{}

getChatsForChatFolderInviteLink::getChatsForChatFolderInviteLink(int32 chat_folder_id_)
  : chat_folder_id_(chat_folder_id_)
{}

const std::int32_t getChatsForChatFolderInviteLink::ID;

object_ptr<getChatsForChatFolderInviteLink> getChatsForChatFolderInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getChatsForChatFolderInviteLink> res = make_object<getChatsForChatFolderInviteLink>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  return res;
}

void getChatsForChatFolderInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatsForChatFolderInviteLink");
    s.store_field("chatFolderId", chat_folder_id_);
    s.store_class_end();
  }
}

getChatsForChatFolderInviteLink::ReturnType getChatsForChatFolderInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatsForChatFolderInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatsForChatFolderInviteLink").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getChatsToPostStories::Class;

getChatsToPostStories::getChatsToPostStories() {
}

const std::int32_t getChatsToPostStories::ID;

object_ptr<getChatsToPostStories> getChatsToPostStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getChatsToPostStories>();
}

void getChatsToPostStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetChatsToPostStories");
    s.store_class_end();
  }
}

getChatsToPostStories::ReturnType getChatsToPostStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getChatsToPostStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetChatsToPostStories").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCloseFriends::Class;

getCloseFriends::getCloseFriends() {
}

const std::int32_t getCloseFriends::ID;

object_ptr<getCloseFriends> getCloseFriends::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getCloseFriends>();
}

void getCloseFriends::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCloseFriends");
    s.store_class_end();
  }
}

getCloseFriends::ReturnType getCloseFriends::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void getCloseFriends::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCloseFriends").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCollectibleItemInfo::Class;
jfieldID getCollectibleItemInfo::type_fieldID;

getCollectibleItemInfo::getCollectibleItemInfo()
  : type_()
{}

getCollectibleItemInfo::getCollectibleItemInfo(object_ptr<CollectibleItemType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t getCollectibleItemInfo::ID;

object_ptr<getCollectibleItemInfo> getCollectibleItemInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCollectibleItemInfo> res = make_object<getCollectibleItemInfo>();
  res->type_ = jni::fetch_tl_object<CollectibleItemType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void getCollectibleItemInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCollectibleItemInfo");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

getCollectibleItemInfo::ReturnType getCollectibleItemInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return collectibleItemInfo::fetch(env, p);
}

void getCollectibleItemInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCollectibleItemInfo").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$CollectibleItemType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCommands::Class;
jfieldID getCommands::scope_fieldID;
jfieldID getCommands::language_code_fieldID;

getCommands::getCommands()
  : scope_()
  , language_code_()
{}

getCommands::getCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_)
  : scope_(std::move(scope_))
  , language_code_(language_code_)
{}

const std::int32_t getCommands::ID;

object_ptr<getCommands> getCommands::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCommands> res = make_object<getCommands>();
  res->scope_ = jni::fetch_tl_object<BotCommandScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getCommands::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCommands");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getCommands::ReturnType getCommands::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botCommands::fetch(env, p);
}

void getCommands::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCommands").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotCommandScope;").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getConnectedAffiliateProgram::Class;
jfieldID getConnectedAffiliateProgram::affiliate_fieldID;
jfieldID getConnectedAffiliateProgram::bot_user_id_fieldID;

getConnectedAffiliateProgram::getConnectedAffiliateProgram()
  : affiliate_()
  , bot_user_id_()
{}

getConnectedAffiliateProgram::getConnectedAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, int53 bot_user_id_)
  : affiliate_(std::move(affiliate_))
  , bot_user_id_(bot_user_id_)
{}

const std::int32_t getConnectedAffiliateProgram::ID;

object_ptr<getConnectedAffiliateProgram> getConnectedAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getConnectedAffiliateProgram> res = make_object<getConnectedAffiliateProgram>();
  res->affiliate_ = jni::fetch_tl_object<AffiliateType>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void getConnectedAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetConnectedAffiliateProgram");
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

getConnectedAffiliateProgram::ReturnType getConnectedAffiliateProgram::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return connectedAffiliateProgram::fetch(env, p);
}

void getConnectedAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetConnectedAffiliateProgram").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateType;").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getConnectedAffiliatePrograms::Class;
jfieldID getConnectedAffiliatePrograms::affiliate_fieldID;
jfieldID getConnectedAffiliatePrograms::offset_fieldID;
jfieldID getConnectedAffiliatePrograms::limit_fieldID;

getConnectedAffiliatePrograms::getConnectedAffiliatePrograms()
  : affiliate_()
  , offset_()
  , limit_()
{}

getConnectedAffiliatePrograms::getConnectedAffiliatePrograms(object_ptr<AffiliateType> &&affiliate_, string const &offset_, int32 limit_)
  : affiliate_(std::move(affiliate_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getConnectedAffiliatePrograms::ID;

object_ptr<getConnectedAffiliatePrograms> getConnectedAffiliatePrograms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getConnectedAffiliatePrograms> res = make_object<getConnectedAffiliatePrograms>();
  res->affiliate_ = jni::fetch_tl_object<AffiliateType>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getConnectedAffiliatePrograms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetConnectedAffiliatePrograms");
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getConnectedAffiliatePrograms::ReturnType getConnectedAffiliatePrograms::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return connectedAffiliatePrograms::fetch(env, p);
}

void getConnectedAffiliatePrograms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetConnectedAffiliatePrograms").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateType;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getConnectedWebsites::Class;

getConnectedWebsites::getConnectedWebsites() {
}

const std::int32_t getConnectedWebsites::ID;

object_ptr<getConnectedWebsites> getConnectedWebsites::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getConnectedWebsites>();
}

void getConnectedWebsites::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetConnectedWebsites");
    s.store_class_end();
  }
}

getConnectedWebsites::ReturnType getConnectedWebsites::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return connectedWebsites::fetch(env, p);
}

void getConnectedWebsites::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetConnectedWebsites").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getContacts::Class;

getContacts::getContacts() {
}

const std::int32_t getContacts::ID;

object_ptr<getContacts> getContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getContacts>();
}

void getContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetContacts");
    s.store_class_end();
  }
}

getContacts::ReturnType getContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void getContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetContacts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCountries::Class;

getCountries::getCountries() {
}

const std::int32_t getCountries::ID;

object_ptr<getCountries> getCountries::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getCountries>();
}

void getCountries::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCountries");
    s.store_class_end();
  }
}

getCountries::ReturnType getCountries::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return countries::fetch(env, p);
}

void getCountries::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCountries").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCountryCode::Class;

getCountryCode::getCountryCode() {
}

const std::int32_t getCountryCode::ID;

object_ptr<getCountryCode> getCountryCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getCountryCode>();
}

void getCountryCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCountryCode");
    s.store_class_end();
  }
}

getCountryCode::ReturnType getCountryCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getCountryCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCountryCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCountryFlagEmoji::Class;
jfieldID getCountryFlagEmoji::country_code_fieldID;

getCountryFlagEmoji::getCountryFlagEmoji()
  : country_code_()
{}

getCountryFlagEmoji::getCountryFlagEmoji(string const &country_code_)
  : country_code_(country_code_)
{}

const std::int32_t getCountryFlagEmoji::ID;

object_ptr<getCountryFlagEmoji> getCountryFlagEmoji::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCountryFlagEmoji> res = make_object<getCountryFlagEmoji>();
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  return res;
}

void getCountryFlagEmoji::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCountryFlagEmoji");
    s.store_field("countryCode", country_code_);
    s.store_class_end();
  }
}

getCountryFlagEmoji::ReturnType getCountryFlagEmoji::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getCountryFlagEmoji::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCountryFlagEmoji").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getCreatedPublicChats::Class;
jfieldID getCreatedPublicChats::type_fieldID;

getCreatedPublicChats::getCreatedPublicChats()
  : type_()
{}

getCreatedPublicChats::getCreatedPublicChats(object_ptr<PublicChatType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t getCreatedPublicChats::ID;

object_ptr<getCreatedPublicChats> getCreatedPublicChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCreatedPublicChats> res = make_object<getCreatedPublicChats>();
  res->type_ = jni::fetch_tl_object<PublicChatType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void getCreatedPublicChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCreatedPublicChats");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

getCreatedPublicChats::ReturnType getCreatedPublicChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getCreatedPublicChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCreatedPublicChats").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PublicChatType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCurrentState::Class;

getCurrentState::getCurrentState() {
}

const std::int32_t getCurrentState::ID;

object_ptr<getCurrentState> getCurrentState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getCurrentState>();
}

void getCurrentState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCurrentState");
    s.store_class_end();
  }
}

getCurrentState::ReturnType getCurrentState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return updates::fetch(env, p);
}

void getCurrentState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCurrentState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCurrentWeather::Class;
jfieldID getCurrentWeather::location_fieldID;

getCurrentWeather::getCurrentWeather()
  : location_()
{}

getCurrentWeather::getCurrentWeather(object_ptr<location> &&location_)
  : location_(std::move(location_))
{}

const std::int32_t getCurrentWeather::ID;

object_ptr<getCurrentWeather> getCurrentWeather::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCurrentWeather> res = make_object<getCurrentWeather>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  return res;
}

void getCurrentWeather::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCurrentWeather");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_class_end();
  }
}

getCurrentWeather::ReturnType getCurrentWeather::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return currentWeather::fetch(env, p);
}

void getCurrentWeather::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCurrentWeather").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCustomEmojiReactionAnimations::Class;

getCustomEmojiReactionAnimations::getCustomEmojiReactionAnimations() {
}

const std::int32_t getCustomEmojiReactionAnimations::ID;

object_ptr<getCustomEmojiReactionAnimations> getCustomEmojiReactionAnimations::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getCustomEmojiReactionAnimations>();
}

void getCustomEmojiReactionAnimations::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCustomEmojiReactionAnimations");
    s.store_class_end();
  }
}

getCustomEmojiReactionAnimations::ReturnType getCustomEmojiReactionAnimations::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getCustomEmojiReactionAnimations::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCustomEmojiReactionAnimations").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getCustomEmojiStickers::Class;
jfieldID getCustomEmojiStickers::custom_emoji_ids_fieldID;

getCustomEmojiStickers::getCustomEmojiStickers()
  : custom_emoji_ids_()
{}

getCustomEmojiStickers::getCustomEmojiStickers(array<int64> &&custom_emoji_ids_)
  : custom_emoji_ids_(std::move(custom_emoji_ids_))
{}

const std::int32_t getCustomEmojiStickers::ID;

object_ptr<getCustomEmojiStickers> getCustomEmojiStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getCustomEmojiStickers> res = make_object<getCustomEmojiStickers>();
  res->custom_emoji_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->custom_emoji_ids_fieldID));
  return res;
}

void getCustomEmojiStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetCustomEmojiStickers");
    { s.store_vector_begin("customEmojiIds", custom_emoji_ids_.size()); for (const auto &_value : custom_emoji_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

getCustomEmojiStickers::ReturnType getCustomEmojiStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getCustomEmojiStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetCustomEmojiStickers").c_str());
    custom_emoji_ids_fieldID = jni::get_field_id(env, Class, "customEmojiIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass getDatabaseStatistics::Class;

getDatabaseStatistics::getDatabaseStatistics() {
}

const std::int32_t getDatabaseStatistics::ID;

object_ptr<getDatabaseStatistics> getDatabaseStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDatabaseStatistics>();
}

void getDatabaseStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDatabaseStatistics");
    s.store_class_end();
  }
}

getDatabaseStatistics::ReturnType getDatabaseStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return databaseStatistics::fetch(env, p);
}

void getDatabaseStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDatabaseStatistics").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDeepLinkInfo::Class;
jfieldID getDeepLinkInfo::link_fieldID;

getDeepLinkInfo::getDeepLinkInfo()
  : link_()
{}

getDeepLinkInfo::getDeepLinkInfo(string const &link_)
  : link_(link_)
{}

const std::int32_t getDeepLinkInfo::ID;

object_ptr<getDeepLinkInfo> getDeepLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getDeepLinkInfo> res = make_object<getDeepLinkInfo>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return res;
}

void getDeepLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDeepLinkInfo");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

getDeepLinkInfo::ReturnType getDeepLinkInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return deepLinkInfo::fetch(env, p);
}

void getDeepLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDeepLinkInfo").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultBackgroundCustomEmojiStickers::Class;

getDefaultBackgroundCustomEmojiStickers::getDefaultBackgroundCustomEmojiStickers() {
}

const std::int32_t getDefaultBackgroundCustomEmojiStickers::ID;

object_ptr<getDefaultBackgroundCustomEmojiStickers> getDefaultBackgroundCustomEmojiStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultBackgroundCustomEmojiStickers>();
}

void getDefaultBackgroundCustomEmojiStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultBackgroundCustomEmojiStickers");
    s.store_class_end();
  }
}

getDefaultBackgroundCustomEmojiStickers::ReturnType getDefaultBackgroundCustomEmojiStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getDefaultBackgroundCustomEmojiStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultBackgroundCustomEmojiStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultChatEmojiStatuses::Class;

getDefaultChatEmojiStatuses::getDefaultChatEmojiStatuses() {
}

const std::int32_t getDefaultChatEmojiStatuses::ID;

object_ptr<getDefaultChatEmojiStatuses> getDefaultChatEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultChatEmojiStatuses>();
}

void getDefaultChatEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultChatEmojiStatuses");
    s.store_class_end();
  }
}

getDefaultChatEmojiStatuses::ReturnType getDefaultChatEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatusCustomEmojis::fetch(env, p);
}

void getDefaultChatEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultChatEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultChatPhotoCustomEmojiStickers::Class;

getDefaultChatPhotoCustomEmojiStickers::getDefaultChatPhotoCustomEmojiStickers() {
}

const std::int32_t getDefaultChatPhotoCustomEmojiStickers::ID;

object_ptr<getDefaultChatPhotoCustomEmojiStickers> getDefaultChatPhotoCustomEmojiStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultChatPhotoCustomEmojiStickers>();
}

void getDefaultChatPhotoCustomEmojiStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultChatPhotoCustomEmojiStickers");
    s.store_class_end();
  }
}

getDefaultChatPhotoCustomEmojiStickers::ReturnType getDefaultChatPhotoCustomEmojiStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getDefaultChatPhotoCustomEmojiStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultChatPhotoCustomEmojiStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultEmojiStatuses::Class;

getDefaultEmojiStatuses::getDefaultEmojiStatuses() {
}

const std::int32_t getDefaultEmojiStatuses::ID;

object_ptr<getDefaultEmojiStatuses> getDefaultEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultEmojiStatuses>();
}

void getDefaultEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultEmojiStatuses");
    s.store_class_end();
  }
}

getDefaultEmojiStatuses::ReturnType getDefaultEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatusCustomEmojis::fetch(env, p);
}

void getDefaultEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultMessageAutoDeleteTime::Class;

getDefaultMessageAutoDeleteTime::getDefaultMessageAutoDeleteTime() {
}

const std::int32_t getDefaultMessageAutoDeleteTime::ID;

object_ptr<getDefaultMessageAutoDeleteTime> getDefaultMessageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultMessageAutoDeleteTime>();
}

void getDefaultMessageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultMessageAutoDeleteTime");
    s.store_class_end();
  }
}

getDefaultMessageAutoDeleteTime::ReturnType getDefaultMessageAutoDeleteTime::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageAutoDeleteTime::fetch(env, p);
}

void getDefaultMessageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultMessageAutoDeleteTime").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDefaultProfilePhotoCustomEmojiStickers::Class;

getDefaultProfilePhotoCustomEmojiStickers::getDefaultProfilePhotoCustomEmojiStickers() {
}

const std::int32_t getDefaultProfilePhotoCustomEmojiStickers::ID;

object_ptr<getDefaultProfilePhotoCustomEmojiStickers> getDefaultProfilePhotoCustomEmojiStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDefaultProfilePhotoCustomEmojiStickers>();
}

void getDefaultProfilePhotoCustomEmojiStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDefaultProfilePhotoCustomEmojiStickers");
    s.store_class_end();
  }
}

getDefaultProfilePhotoCustomEmojiStickers::ReturnType getDefaultProfilePhotoCustomEmojiStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getDefaultProfilePhotoCustomEmojiStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDefaultProfilePhotoCustomEmojiStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getDisallowedChatEmojiStatuses::Class;

getDisallowedChatEmojiStatuses::getDisallowedChatEmojiStatuses() {
}

const std::int32_t getDisallowedChatEmojiStatuses::ID;

object_ptr<getDisallowedChatEmojiStatuses> getDisallowedChatEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getDisallowedChatEmojiStatuses>();
}

void getDisallowedChatEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetDisallowedChatEmojiStatuses");
    s.store_class_end();
  }
}

getDisallowedChatEmojiStatuses::ReturnType getDisallowedChatEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatusCustomEmojis::fetch(env, p);
}

void getDisallowedChatEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetDisallowedChatEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getEmojiCategories::Class;
jfieldID getEmojiCategories::type_fieldID;

getEmojiCategories::getEmojiCategories()
  : type_()
{}

getEmojiCategories::getEmojiCategories(object_ptr<EmojiCategoryType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t getEmojiCategories::ID;

object_ptr<getEmojiCategories> getEmojiCategories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getEmojiCategories> res = make_object<getEmojiCategories>();
  res->type_ = jni::fetch_tl_object<EmojiCategoryType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void getEmojiCategories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetEmojiCategories");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

getEmojiCategories::ReturnType getEmojiCategories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiCategories::fetch(env, p);
}

void getEmojiCategories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetEmojiCategories").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiCategoryType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getEmojiReaction::Class;
jfieldID getEmojiReaction::emoji_fieldID;

getEmojiReaction::getEmojiReaction()
  : emoji_()
{}

getEmojiReaction::getEmojiReaction(string const &emoji_)
  : emoji_(emoji_)
{}

const std::int32_t getEmojiReaction::ID;

object_ptr<getEmojiReaction> getEmojiReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getEmojiReaction> res = make_object<getEmojiReaction>();
  res->emoji_ = jni::fetch_string(env, p, res->emoji_fieldID);
  return res;
}

void getEmojiReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetEmojiReaction");
    s.store_field("emoji", emoji_);
    s.store_class_end();
  }
}

getEmojiReaction::ReturnType getEmojiReaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiReaction::fetch(env, p);
}

void getEmojiReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetEmojiReaction").c_str());
    emoji_fieldID = jni::get_field_id(env, Class, "emoji", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getEmojiSuggestionsUrl::Class;
jfieldID getEmojiSuggestionsUrl::language_code_fieldID;

getEmojiSuggestionsUrl::getEmojiSuggestionsUrl()
  : language_code_()
{}

getEmojiSuggestionsUrl::getEmojiSuggestionsUrl(string const &language_code_)
  : language_code_(language_code_)
{}

const std::int32_t getEmojiSuggestionsUrl::ID;

object_ptr<getEmojiSuggestionsUrl> getEmojiSuggestionsUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getEmojiSuggestionsUrl> res = make_object<getEmojiSuggestionsUrl>();
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  return res;
}

void getEmojiSuggestionsUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetEmojiSuggestionsUrl");
    s.store_field("languageCode", language_code_);
    s.store_class_end();
  }
}

getEmojiSuggestionsUrl::ReturnType getEmojiSuggestionsUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getEmojiSuggestionsUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetEmojiSuggestionsUrl").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getExternalLink::Class;
jfieldID getExternalLink::link_fieldID;
jfieldID getExternalLink::allow_write_access_fieldID;

getExternalLink::getExternalLink()
  : link_()
  , allow_write_access_()
{}

getExternalLink::getExternalLink(string const &link_, bool allow_write_access_)
  : link_(link_)
  , allow_write_access_(allow_write_access_)
{}

const std::int32_t getExternalLink::ID;

object_ptr<getExternalLink> getExternalLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getExternalLink> res = make_object<getExternalLink>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  res->allow_write_access_ = (env->GetBooleanField(p, res->allow_write_access_fieldID) != 0);
  return res;
}

void getExternalLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetExternalLink");
    s.store_field("link", link_);
    s.store_field("allowWriteAccess", allow_write_access_);
    s.store_class_end();
  }
}

getExternalLink::ReturnType getExternalLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getExternalLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetExternalLink").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    allow_write_access_fieldID = jni::get_field_id(env, Class, "allowWriteAccess", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getExternalLinkInfo::Class;
jfieldID getExternalLinkInfo::link_fieldID;

getExternalLinkInfo::getExternalLinkInfo()
  : link_()
{}

getExternalLinkInfo::getExternalLinkInfo(string const &link_)
  : link_(link_)
{}

const std::int32_t getExternalLinkInfo::ID;

object_ptr<getExternalLinkInfo> getExternalLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getExternalLinkInfo> res = make_object<getExternalLinkInfo>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return res;
}

void getExternalLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetExternalLinkInfo");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

getExternalLinkInfo::ReturnType getExternalLinkInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return LoginUrlInfo::fetch(env, p);
}

void getExternalLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetExternalLinkInfo").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getFavoriteStickers::Class;

getFavoriteStickers::getFavoriteStickers() {
}

const std::int32_t getFavoriteStickers::ID;

object_ptr<getFavoriteStickers> getFavoriteStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getFavoriteStickers>();
}

void getFavoriteStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetFavoriteStickers");
    s.store_class_end();
  }
}

getFavoriteStickers::ReturnType getFavoriteStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getFavoriteStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetFavoriteStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getFile::Class;
jfieldID getFile::file_id_fieldID;

getFile::getFile()
  : file_id_()
{}

getFile::getFile(int32 file_id_)
  : file_id_(file_id_)
{}

const std::int32_t getFile::ID;

object_ptr<getFile> getFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getFile> res = make_object<getFile>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  return res;
}

void getFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetFile");
    s.store_field("fileId", file_id_);
    s.store_class_end();
  }
}

getFile::ReturnType getFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void getFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetFile").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getFileDownloadedPrefixSize::Class;
jfieldID getFileDownloadedPrefixSize::file_id_fieldID;
jfieldID getFileDownloadedPrefixSize::offset_fieldID;

getFileDownloadedPrefixSize::getFileDownloadedPrefixSize()
  : file_id_()
  , offset_()
{}

getFileDownloadedPrefixSize::getFileDownloadedPrefixSize(int32 file_id_, int53 offset_)
  : file_id_(file_id_)
  , offset_(offset_)
{}

const std::int32_t getFileDownloadedPrefixSize::ID;

object_ptr<getFileDownloadedPrefixSize> getFileDownloadedPrefixSize::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getFileDownloadedPrefixSize> res = make_object<getFileDownloadedPrefixSize>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->offset_ = env->GetLongField(p, res->offset_fieldID);
  return res;
}

void getFileDownloadedPrefixSize::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetFileDownloadedPrefixSize");
    s.store_field("fileId", file_id_);
    s.store_field("offset", offset_);
    s.store_class_end();
  }
}

getFileDownloadedPrefixSize::ReturnType getFileDownloadedPrefixSize::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return fileDownloadedPrefixSize::fetch(env, p);
}

void getFileDownloadedPrefixSize::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetFileDownloadedPrefixSize").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getFileExtension::Class;
jfieldID getFileExtension::mime_type_fieldID;

getFileExtension::getFileExtension()
  : mime_type_()
{}

getFileExtension::getFileExtension(string const &mime_type_)
  : mime_type_(mime_type_)
{}

const std::int32_t getFileExtension::ID;

object_ptr<getFileExtension> getFileExtension::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getFileExtension> res = make_object<getFileExtension>();
  res->mime_type_ = jni::fetch_string(env, p, res->mime_type_fieldID);
  return res;
}

void getFileExtension::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetFileExtension");
    s.store_field("mimeType", mime_type_);
    s.store_class_end();
  }
}

getFileExtension::ReturnType getFileExtension::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getFileExtension::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetFileExtension").c_str());
    mime_type_fieldID = jni::get_field_id(env, Class, "mimeType", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getFileMimeType::Class;
jfieldID getFileMimeType::file_name_fieldID;

getFileMimeType::getFileMimeType()
  : file_name_()
{}

getFileMimeType::getFileMimeType(string const &file_name_)
  : file_name_(file_name_)
{}

const std::int32_t getFileMimeType::ID;

object_ptr<getFileMimeType> getFileMimeType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getFileMimeType> res = make_object<getFileMimeType>();
  res->file_name_ = jni::fetch_string(env, p, res->file_name_fieldID);
  return res;
}

void getFileMimeType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetFileMimeType");
    s.store_field("fileName", file_name_);
    s.store_class_end();
  }
}

getFileMimeType::ReturnType getFileMimeType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getFileMimeType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetFileMimeType").c_str());
    file_name_fieldID = jni::get_field_id(env, Class, "fileName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getForumTopic::Class;
jfieldID getForumTopic::chat_id_fieldID;
jfieldID getForumTopic::message_thread_id_fieldID;

getForumTopic::getForumTopic()
  : chat_id_()
  , message_thread_id_()
{}

getForumTopic::getForumTopic(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t getForumTopic::ID;

object_ptr<getForumTopic> getForumTopic::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getForumTopic> res = make_object<getForumTopic>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void getForumTopic::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetForumTopic");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

getForumTopic::ReturnType getForumTopic::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return forumTopic::fetch(env, p);
}

void getForumTopic::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetForumTopic").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getForumTopicDefaultIcons::Class;

getForumTopicDefaultIcons::getForumTopicDefaultIcons() {
}

const std::int32_t getForumTopicDefaultIcons::ID;

object_ptr<getForumTopicDefaultIcons> getForumTopicDefaultIcons::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getForumTopicDefaultIcons>();
}

void getForumTopicDefaultIcons::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetForumTopicDefaultIcons");
    s.store_class_end();
  }
}

getForumTopicDefaultIcons::ReturnType getForumTopicDefaultIcons::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getForumTopicDefaultIcons::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetForumTopicDefaultIcons").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getForumTopicLink::Class;
jfieldID getForumTopicLink::chat_id_fieldID;
jfieldID getForumTopicLink::message_thread_id_fieldID;

getForumTopicLink::getForumTopicLink()
  : chat_id_()
  , message_thread_id_()
{}

getForumTopicLink::getForumTopicLink(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t getForumTopicLink::ID;

object_ptr<getForumTopicLink> getForumTopicLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getForumTopicLink> res = make_object<getForumTopicLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void getForumTopicLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetForumTopicLink");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

getForumTopicLink::ReturnType getForumTopicLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageLink::fetch(env, p);
}

void getForumTopicLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetForumTopicLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getForumTopics::Class;
jfieldID getForumTopics::chat_id_fieldID;
jfieldID getForumTopics::query_fieldID;
jfieldID getForumTopics::offset_date_fieldID;
jfieldID getForumTopics::offset_message_id_fieldID;
jfieldID getForumTopics::offset_message_thread_id_fieldID;
jfieldID getForumTopics::limit_fieldID;

getForumTopics::getForumTopics()
  : chat_id_()
  , query_()
  , offset_date_()
  , offset_message_id_()
  , offset_message_thread_id_()
  , limit_()
{}

getForumTopics::getForumTopics(int53 chat_id_, string const &query_, int32 offset_date_, int53 offset_message_id_, int53 offset_message_thread_id_, int32 limit_)
  : chat_id_(chat_id_)
  , query_(query_)
  , offset_date_(offset_date_)
  , offset_message_id_(offset_message_id_)
  , offset_message_thread_id_(offset_message_thread_id_)
  , limit_(limit_)
{}

const std::int32_t getForumTopics::ID;

object_ptr<getForumTopics> getForumTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getForumTopics> res = make_object<getForumTopics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_date_ = env->GetIntField(p, res->offset_date_fieldID);
  res->offset_message_id_ = env->GetLongField(p, res->offset_message_id_fieldID);
  res->offset_message_thread_id_ = env->GetLongField(p, res->offset_message_thread_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getForumTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetForumTopics");
    s.store_field("chatId", chat_id_);
    s.store_field("query", query_);
    s.store_field("offsetDate", offset_date_);
    s.store_field("offsetMessageId", offset_message_id_);
    s.store_field("offsetMessageThreadId", offset_message_thread_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getForumTopics::ReturnType getForumTopics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return forumTopics::fetch(env, p);
}

void getForumTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetForumTopics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_date_fieldID = jni::get_field_id(env, Class, "offsetDate", "I");
    offset_message_id_fieldID = jni::get_field_id(env, Class, "offsetMessageId", "J");
    offset_message_thread_id_fieldID = jni::get_field_id(env, Class, "offsetMessageThreadId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getGameHighScores::Class;
jfieldID getGameHighScores::chat_id_fieldID;
jfieldID getGameHighScores::message_id_fieldID;
jfieldID getGameHighScores::user_id_fieldID;

getGameHighScores::getGameHighScores()
  : chat_id_()
  , message_id_()
  , user_id_()
{}

getGameHighScores::getGameHighScores(int53 chat_id_, int53 message_id_, int53 user_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , user_id_(user_id_)
{}

const std::int32_t getGameHighScores::ID;

object_ptr<getGameHighScores> getGameHighScores::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGameHighScores> res = make_object<getGameHighScores>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getGameHighScores::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGameHighScores");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getGameHighScores::ReturnType getGameHighScores::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return gameHighScores::fetch(env, p);
}

void getGameHighScores::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGameHighScores").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getGiftUpgradePreview::Class;
jfieldID getGiftUpgradePreview::gift_id_fieldID;

getGiftUpgradePreview::getGiftUpgradePreview()
  : gift_id_()
{}

getGiftUpgradePreview::getGiftUpgradePreview(int64 gift_id_)
  : gift_id_(gift_id_)
{}

const std::int32_t getGiftUpgradePreview::ID;

object_ptr<getGiftUpgradePreview> getGiftUpgradePreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGiftUpgradePreview> res = make_object<getGiftUpgradePreview>();
  res->gift_id_ = env->GetLongField(p, res->gift_id_fieldID);
  return res;
}

void getGiftUpgradePreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGiftUpgradePreview");
    s.store_field("giftId", gift_id_);
    s.store_class_end();
  }
}

getGiftUpgradePreview::ReturnType getGiftUpgradePreview::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return giftUpgradePreview::fetch(env, p);
}

void getGiftUpgradePreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGiftUpgradePreview").c_str());
    gift_id_fieldID = jni::get_field_id(env, Class, "giftId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getGiveawayInfo::Class;
jfieldID getGiveawayInfo::chat_id_fieldID;
jfieldID getGiveawayInfo::message_id_fieldID;

getGiveawayInfo::getGiveawayInfo()
  : chat_id_()
  , message_id_()
{}

getGiveawayInfo::getGiveawayInfo(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getGiveawayInfo::ID;

object_ptr<getGiveawayInfo> getGiveawayInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGiveawayInfo> res = make_object<getGiveawayInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getGiveawayInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGiveawayInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getGiveawayInfo::ReturnType getGiveawayInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return GiveawayInfo::fetch(env, p);
}

void getGiveawayInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGiveawayInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getGreetingStickers::Class;

getGreetingStickers::getGreetingStickers() {
}

const std::int32_t getGreetingStickers::ID;

object_ptr<getGreetingStickers> getGreetingStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getGreetingStickers>();
}

void getGreetingStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGreetingStickers");
    s.store_class_end();
  }
}

getGreetingStickers::ReturnType getGreetingStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getGreetingStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGreetingStickers").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getGrossingWebAppBots::Class;
jfieldID getGrossingWebAppBots::offset_fieldID;
jfieldID getGrossingWebAppBots::limit_fieldID;

getGrossingWebAppBots::getGrossingWebAppBots()
  : offset_()
  , limit_()
{}

getGrossingWebAppBots::getGrossingWebAppBots(string const &offset_, int32 limit_)
  : offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getGrossingWebAppBots::ID;

object_ptr<getGrossingWebAppBots> getGrossingWebAppBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGrossingWebAppBots> res = make_object<getGrossingWebAppBots>();
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getGrossingWebAppBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGrossingWebAppBots");
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getGrossingWebAppBots::ReturnType getGrossingWebAppBots::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundUsers::fetch(env, p);
}

void getGrossingWebAppBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGrossingWebAppBots").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getGroupCall::Class;
jfieldID getGroupCall::group_call_id_fieldID;

getGroupCall::getGroupCall()
  : group_call_id_()
{}

getGroupCall::getGroupCall(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t getGroupCall::ID;

object_ptr<getGroupCall> getGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGroupCall> res = make_object<getGroupCall>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void getGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGroupCall");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

getGroupCall::ReturnType getGroupCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return groupCall::fetch(env, p);
}

void getGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGroupCall").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getGroupCallParticipants::Class;
jfieldID getGroupCallParticipants::input_group_call_fieldID;
jfieldID getGroupCallParticipants::limit_fieldID;

getGroupCallParticipants::getGroupCallParticipants()
  : input_group_call_()
  , limit_()
{}

getGroupCallParticipants::getGroupCallParticipants(object_ptr<InputGroupCall> &&input_group_call_, int32 limit_)
  : input_group_call_(std::move(input_group_call_))
  , limit_(limit_)
{}

const std::int32_t getGroupCallParticipants::ID;

object_ptr<getGroupCallParticipants> getGroupCallParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGroupCallParticipants> res = make_object<getGroupCallParticipants>();
  res->input_group_call_ = jni::fetch_tl_object<InputGroupCall>(env, jni::fetch_object(env, p, res->input_group_call_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getGroupCallParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGroupCallParticipants");
    s.store_object_field("inputGroupCall", static_cast<const BaseObject *>(input_group_call_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getGroupCallParticipants::ReturnType getGroupCallParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return groupCallParticipants::fetch(env, p);
}

void getGroupCallParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGroupCallParticipants").c_str());
    input_group_call_fieldID = jni::get_field_id(env, Class, "inputGroupCall", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputGroupCall;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getGroupsInCommon::Class;
jfieldID getGroupsInCommon::user_id_fieldID;
jfieldID getGroupsInCommon::offset_chat_id_fieldID;
jfieldID getGroupsInCommon::limit_fieldID;

getGroupsInCommon::getGroupsInCommon()
  : user_id_()
  , offset_chat_id_()
  , limit_()
{}

getGroupsInCommon::getGroupsInCommon(int53 user_id_, int53 offset_chat_id_, int32 limit_)
  : user_id_(user_id_)
  , offset_chat_id_(offset_chat_id_)
  , limit_(limit_)
{}

const std::int32_t getGroupsInCommon::ID;

object_ptr<getGroupsInCommon> getGroupsInCommon::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getGroupsInCommon> res = make_object<getGroupsInCommon>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->offset_chat_id_ = env->GetLongField(p, res->offset_chat_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getGroupsInCommon::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetGroupsInCommon");
    s.store_field("userId", user_id_);
    s.store_field("offsetChatId", offset_chat_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getGroupsInCommon::ReturnType getGroupsInCommon::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getGroupsInCommon::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetGroupsInCommon").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    offset_chat_id_fieldID = jni::get_field_id(env, Class, "offsetChatId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getImportedContactCount::Class;

getImportedContactCount::getImportedContactCount() {
}

const std::int32_t getImportedContactCount::ID;

object_ptr<getImportedContactCount> getImportedContactCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getImportedContactCount>();
}

void getImportedContactCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetImportedContactCount");
    s.store_class_end();
  }
}

getImportedContactCount::ReturnType getImportedContactCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return count::fetch(env, p);
}

void getImportedContactCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetImportedContactCount").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getInactiveSupergroupChats::Class;

getInactiveSupergroupChats::getInactiveSupergroupChats() {
}

const std::int32_t getInactiveSupergroupChats::ID;

object_ptr<getInactiveSupergroupChats> getInactiveSupergroupChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getInactiveSupergroupChats>();
}

void getInactiveSupergroupChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInactiveSupergroupChats");
    s.store_class_end();
  }
}

getInactiveSupergroupChats::ReturnType getInactiveSupergroupChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getInactiveSupergroupChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInactiveSupergroupChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getInlineGameHighScores::Class;
jfieldID getInlineGameHighScores::inline_message_id_fieldID;
jfieldID getInlineGameHighScores::user_id_fieldID;

getInlineGameHighScores::getInlineGameHighScores()
  : inline_message_id_()
  , user_id_()
{}

getInlineGameHighScores::getInlineGameHighScores(string const &inline_message_id_, int53 user_id_)
  : inline_message_id_(inline_message_id_)
  , user_id_(user_id_)
{}

const std::int32_t getInlineGameHighScores::ID;

object_ptr<getInlineGameHighScores> getInlineGameHighScores::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInlineGameHighScores> res = make_object<getInlineGameHighScores>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getInlineGameHighScores::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInlineGameHighScores");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getInlineGameHighScores::ReturnType getInlineGameHighScores::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return gameHighScores::fetch(env, p);
}

void getInlineGameHighScores::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInlineGameHighScores").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getInlineQueryResults::Class;
jfieldID getInlineQueryResults::bot_user_id_fieldID;
jfieldID getInlineQueryResults::chat_id_fieldID;
jfieldID getInlineQueryResults::user_location_fieldID;
jfieldID getInlineQueryResults::query_fieldID;
jfieldID getInlineQueryResults::offset_fieldID;

getInlineQueryResults::getInlineQueryResults()
  : bot_user_id_()
  , chat_id_()
  , user_location_()
  , query_()
  , offset_()
{}

getInlineQueryResults::getInlineQueryResults(int53 bot_user_id_, int53 chat_id_, object_ptr<location> &&user_location_, string const &query_, string const &offset_)
  : bot_user_id_(bot_user_id_)
  , chat_id_(chat_id_)
  , user_location_(std::move(user_location_))
  , query_(query_)
  , offset_(offset_)
{}

const std::int32_t getInlineQueryResults::ID;

object_ptr<getInlineQueryResults> getInlineQueryResults::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInlineQueryResults> res = make_object<getInlineQueryResults>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->user_location_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  return res;
}

void getInlineQueryResults::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInlineQueryResults");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("chatId", chat_id_);
    s.store_object_field("userLocation", static_cast<const BaseObject *>(user_location_.get()));
    s.store_field("query", query_);
    s.store_field("offset", offset_);
    s.store_class_end();
  }
}

getInlineQueryResults::ReturnType getInlineQueryResults::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return inlineQueryResults::fetch(env, p);
}

void getInlineQueryResults::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInlineQueryResults").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_location_fieldID = jni::get_field_id(env, Class, "userLocation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getInstalledBackgrounds::Class;
jfieldID getInstalledBackgrounds::for_dark_theme_fieldID;

getInstalledBackgrounds::getInstalledBackgrounds()
  : for_dark_theme_()
{}

getInstalledBackgrounds::getInstalledBackgrounds(bool for_dark_theme_)
  : for_dark_theme_(for_dark_theme_)
{}

const std::int32_t getInstalledBackgrounds::ID;

object_ptr<getInstalledBackgrounds> getInstalledBackgrounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInstalledBackgrounds> res = make_object<getInstalledBackgrounds>();
  res->for_dark_theme_ = (env->GetBooleanField(p, res->for_dark_theme_fieldID) != 0);
  return res;
}

void getInstalledBackgrounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInstalledBackgrounds");
    s.store_field("forDarkTheme", for_dark_theme_);
    s.store_class_end();
  }
}

getInstalledBackgrounds::ReturnType getInstalledBackgrounds::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return backgrounds::fetch(env, p);
}

void getInstalledBackgrounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInstalledBackgrounds").c_str());
    for_dark_theme_fieldID = jni::get_field_id(env, Class, "forDarkTheme", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getInstalledStickerSets::Class;
jfieldID getInstalledStickerSets::sticker_type_fieldID;

getInstalledStickerSets::getInstalledStickerSets()
  : sticker_type_()
{}

getInstalledStickerSets::getInstalledStickerSets(object_ptr<StickerType> &&sticker_type_)
  : sticker_type_(std::move(sticker_type_))
{}

const std::int32_t getInstalledStickerSets::ID;

object_ptr<getInstalledStickerSets> getInstalledStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInstalledStickerSets> res = make_object<getInstalledStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  return res;
}

void getInstalledStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInstalledStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_class_end();
  }
}

getInstalledStickerSets::ReturnType getInstalledStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void getInstalledStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInstalledStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getInternalLink::Class;
jfieldID getInternalLink::type_fieldID;
jfieldID getInternalLink::is_http_fieldID;

getInternalLink::getInternalLink()
  : type_()
  , is_http_()
{}

getInternalLink::getInternalLink(object_ptr<InternalLinkType> &&type_, bool is_http_)
  : type_(std::move(type_))
  , is_http_(is_http_)
{}

const std::int32_t getInternalLink::ID;

object_ptr<getInternalLink> getInternalLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInternalLink> res = make_object<getInternalLink>();
  res->type_ = jni::fetch_tl_object<InternalLinkType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->is_http_ = (env->GetBooleanField(p, res->is_http_fieldID) != 0);
  return res;
}

void getInternalLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInternalLink");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("isHttp", is_http_);
    s.store_class_end();
  }
}

getInternalLink::ReturnType getInternalLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getInternalLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInternalLink").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InternalLinkType;").c_str());
    is_http_fieldID = jni::get_field_id(env, Class, "isHttp", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getInternalLinkType::Class;
jfieldID getInternalLinkType::link_fieldID;

getInternalLinkType::getInternalLinkType()
  : link_()
{}

getInternalLinkType::getInternalLinkType(string const &link_)
  : link_(link_)
{}

const std::int32_t getInternalLinkType::ID;

object_ptr<getInternalLinkType> getInternalLinkType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getInternalLinkType> res = make_object<getInternalLinkType>();
  res->link_ = jni::fetch_string(env, p, res->link_fieldID);
  return res;
}

void getInternalLinkType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetInternalLinkType");
    s.store_field("link", link_);
    s.store_class_end();
  }
}

getInternalLinkType::ReturnType getInternalLinkType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return InternalLinkType::fetch(env, p);
}

void getInternalLinkType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetInternalLinkType").c_str());
    link_fieldID = jni::get_field_id(env, Class, "link", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getJsonString::Class;
jfieldID getJsonString::json_value_fieldID;

getJsonString::getJsonString()
  : json_value_()
{}

getJsonString::getJsonString(object_ptr<JsonValue> &&json_value_)
  : json_value_(std::move(json_value_))
{}

const std::int32_t getJsonString::ID;

object_ptr<getJsonString> getJsonString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getJsonString> res = make_object<getJsonString>();
  res->json_value_ = jni::fetch_tl_object<JsonValue>(env, jni::fetch_object(env, p, res->json_value_fieldID));
  return res;
}

void getJsonString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetJsonString");
    s.store_object_field("jsonValue", static_cast<const BaseObject *>(json_value_.get()));
    s.store_class_end();
  }
}

getJsonString::ReturnType getJsonString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getJsonString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetJsonString").c_str());
    json_value_fieldID = jni::get_field_id(env, Class, "jsonValue", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$JsonValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getJsonValue::Class;
jfieldID getJsonValue::json_fieldID;

getJsonValue::getJsonValue()
  : json_()
{}

getJsonValue::getJsonValue(string const &json_)
  : json_(json_)
{}

const std::int32_t getJsonValue::ID;

object_ptr<getJsonValue> getJsonValue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getJsonValue> res = make_object<getJsonValue>();
  res->json_ = jni::fetch_string(env, p, res->json_fieldID);
  return res;
}

void getJsonValue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetJsonValue");
    s.store_field("json", json_);
    s.store_class_end();
  }
}

getJsonValue::ReturnType getJsonValue::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return JsonValue::fetch(env, p);
}

void getJsonValue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetJsonValue").c_str());
    json_fieldID = jni::get_field_id(env, Class, "json", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getKeywordEmojis::Class;
jfieldID getKeywordEmojis::text_fieldID;
jfieldID getKeywordEmojis::input_language_codes_fieldID;

getKeywordEmojis::getKeywordEmojis()
  : text_()
  , input_language_codes_()
{}

getKeywordEmojis::getKeywordEmojis(string const &text_, array<string> &&input_language_codes_)
  : text_(text_)
  , input_language_codes_(std::move(input_language_codes_))
{}

const std::int32_t getKeywordEmojis::ID;

object_ptr<getKeywordEmojis> getKeywordEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getKeywordEmojis> res = make_object<getKeywordEmojis>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->input_language_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_language_codes_fieldID));
  return res;
}

void getKeywordEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetKeywordEmojis");
    s.store_field("text", text_);
    { s.store_vector_begin("inputLanguageCodes", input_language_codes_.size()); for (const auto &_value : input_language_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

getKeywordEmojis::ReturnType getKeywordEmojis::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojis::fetch(env, p);
}

void getKeywordEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetKeywordEmojis").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    input_language_codes_fieldID = jni::get_field_id(env, Class, "inputLanguageCodes", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getLanguagePackInfo::Class;
jfieldID getLanguagePackInfo::language_pack_id_fieldID;

getLanguagePackInfo::getLanguagePackInfo()
  : language_pack_id_()
{}

getLanguagePackInfo::getLanguagePackInfo(string const &language_pack_id_)
  : language_pack_id_(language_pack_id_)
{}

const std::int32_t getLanguagePackInfo::ID;

object_ptr<getLanguagePackInfo> getLanguagePackInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLanguagePackInfo> res = make_object<getLanguagePackInfo>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  return res;
}

void getLanguagePackInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLanguagePackInfo");
    s.store_field("languagePackId", language_pack_id_);
    s.store_class_end();
  }
}

getLanguagePackInfo::ReturnType getLanguagePackInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return languagePackInfo::fetch(env, p);
}

void getLanguagePackInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLanguagePackInfo").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getLanguagePackString::Class;
jfieldID getLanguagePackString::language_pack_database_path_fieldID;
jfieldID getLanguagePackString::localization_target_fieldID;
jfieldID getLanguagePackString::language_pack_id_fieldID;
jfieldID getLanguagePackString::key_fieldID;

getLanguagePackString::getLanguagePackString()
  : language_pack_database_path_()
  , localization_target_()
  , language_pack_id_()
  , key_()
{}

getLanguagePackString::getLanguagePackString(string const &language_pack_database_path_, string const &localization_target_, string const &language_pack_id_, string const &key_)
  : language_pack_database_path_(language_pack_database_path_)
  , localization_target_(localization_target_)
  , language_pack_id_(language_pack_id_)
  , key_(key_)
{}

const std::int32_t getLanguagePackString::ID;

object_ptr<getLanguagePackString> getLanguagePackString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLanguagePackString> res = make_object<getLanguagePackString>();
  res->language_pack_database_path_ = jni::fetch_string(env, p, res->language_pack_database_path_fieldID);
  res->localization_target_ = jni::fetch_string(env, p, res->localization_target_fieldID);
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  res->key_ = jni::fetch_string(env, p, res->key_fieldID);
  return res;
}

void getLanguagePackString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLanguagePackString");
    s.store_field("languagePackDatabasePath", language_pack_database_path_);
    s.store_field("localizationTarget", localization_target_);
    s.store_field("languagePackId", language_pack_id_);
    s.store_field("key", key_);
    s.store_class_end();
  }
}

getLanguagePackString::ReturnType getLanguagePackString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return LanguagePackStringValue::fetch(env, p);
}

void getLanguagePackString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLanguagePackString").c_str());
    language_pack_database_path_fieldID = jni::get_field_id(env, Class, "languagePackDatabasePath", "Ljava/lang/String;");
    localization_target_fieldID = jni::get_field_id(env, Class, "localizationTarget", "Ljava/lang/String;");
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    key_fieldID = jni::get_field_id(env, Class, "key", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getLanguagePackStrings::Class;
jfieldID getLanguagePackStrings::language_pack_id_fieldID;
jfieldID getLanguagePackStrings::keys_fieldID;

getLanguagePackStrings::getLanguagePackStrings()
  : language_pack_id_()
  , keys_()
{}

getLanguagePackStrings::getLanguagePackStrings(string const &language_pack_id_, array<string> &&keys_)
  : language_pack_id_(language_pack_id_)
  , keys_(std::move(keys_))
{}

const std::int32_t getLanguagePackStrings::ID;

object_ptr<getLanguagePackStrings> getLanguagePackStrings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLanguagePackStrings> res = make_object<getLanguagePackStrings>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  res->keys_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->keys_fieldID));
  return res;
}

void getLanguagePackStrings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLanguagePackStrings");
    s.store_field("languagePackId", language_pack_id_);
    { s.store_vector_begin("keys", keys_.size()); for (const auto &_value : keys_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

getLanguagePackStrings::ReturnType getLanguagePackStrings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return languagePackStrings::fetch(env, p);
}

void getLanguagePackStrings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLanguagePackStrings").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    keys_fieldID = jni::get_field_id(env, Class, "keys", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getLinkPreview::Class;
jfieldID getLinkPreview::text_fieldID;
jfieldID getLinkPreview::link_preview_options_fieldID;

getLinkPreview::getLinkPreview()
  : text_()
  , link_preview_options_()
{}

getLinkPreview::getLinkPreview(object_ptr<formattedText> &&text_, object_ptr<linkPreviewOptions> &&link_preview_options_)
  : text_(std::move(text_))
  , link_preview_options_(std::move(link_preview_options_))
{}

const std::int32_t getLinkPreview::ID;

object_ptr<getLinkPreview> getLinkPreview::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLinkPreview> res = make_object<getLinkPreview>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->link_preview_options_ = jni::fetch_tl_object<linkPreviewOptions>(env, jni::fetch_object(env, p, res->link_preview_options_fieldID));
  return res;
}

void getLinkPreview::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLinkPreview");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("linkPreviewOptions", static_cast<const BaseObject *>(link_preview_options_.get()));
    s.store_class_end();
  }
}

getLinkPreview::ReturnType getLinkPreview::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return linkPreview::fetch(env, p);
}

void getLinkPreview::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLinkPreview").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    link_preview_options_fieldID = jni::get_field_id(env, Class, "linkPreviewOptions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LinkPreviewOptions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getLocalizationTargetInfo::Class;
jfieldID getLocalizationTargetInfo::only_local_fieldID;

getLocalizationTargetInfo::getLocalizationTargetInfo()
  : only_local_()
{}

getLocalizationTargetInfo::getLocalizationTargetInfo(bool only_local_)
  : only_local_(only_local_)
{}

const std::int32_t getLocalizationTargetInfo::ID;

object_ptr<getLocalizationTargetInfo> getLocalizationTargetInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLocalizationTargetInfo> res = make_object<getLocalizationTargetInfo>();
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void getLocalizationTargetInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLocalizationTargetInfo");
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

getLocalizationTargetInfo::ReturnType getLocalizationTargetInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return localizationTargetInfo::fetch(env, p);
}

void getLocalizationTargetInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLocalizationTargetInfo").c_str());
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getLogStream::Class;

getLogStream::getLogStream() {
}

const std::int32_t getLogStream::ID;

object_ptr<getLogStream> getLogStream::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getLogStream>();
}

void getLogStream::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLogStream");
    s.store_class_end();
  }
}

getLogStream::ReturnType getLogStream::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return LogStream::fetch(env, p);
}

void getLogStream::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLogStream").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getLogTagVerbosityLevel::Class;
jfieldID getLogTagVerbosityLevel::tag_fieldID;

getLogTagVerbosityLevel::getLogTagVerbosityLevel()
  : tag_()
{}

getLogTagVerbosityLevel::getLogTagVerbosityLevel(string const &tag_)
  : tag_(tag_)
{}

const std::int32_t getLogTagVerbosityLevel::ID;

object_ptr<getLogTagVerbosityLevel> getLogTagVerbosityLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLogTagVerbosityLevel> res = make_object<getLogTagVerbosityLevel>();
  res->tag_ = jni::fetch_string(env, p, res->tag_fieldID);
  return res;
}

void getLogTagVerbosityLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLogTagVerbosityLevel");
    s.store_field("tag", tag_);
    s.store_class_end();
  }
}

getLogTagVerbosityLevel::ReturnType getLogTagVerbosityLevel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return logVerbosityLevel::fetch(env, p);
}

void getLogTagVerbosityLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLogTagVerbosityLevel").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getLogTags::Class;

getLogTags::getLogTags() {
}

const std::int32_t getLogTags::ID;

object_ptr<getLogTags> getLogTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getLogTags>();
}

void getLogTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLogTags");
    s.store_class_end();
  }
}

getLogTags::ReturnType getLogTags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return logTags::fetch(env, p);
}

void getLogTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLogTags").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getLogVerbosityLevel::Class;

getLogVerbosityLevel::getLogVerbosityLevel() {
}

const std::int32_t getLogVerbosityLevel::ID;

object_ptr<getLogVerbosityLevel> getLogVerbosityLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getLogVerbosityLevel>();
}

void getLogVerbosityLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLogVerbosityLevel");
    s.store_class_end();
  }
}

getLogVerbosityLevel::ReturnType getLogVerbosityLevel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return logVerbosityLevel::fetch(env, p);
}

void getLogVerbosityLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLogVerbosityLevel").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getLoginUrl::Class;
jfieldID getLoginUrl::chat_id_fieldID;
jfieldID getLoginUrl::message_id_fieldID;
jfieldID getLoginUrl::button_id_fieldID;
jfieldID getLoginUrl::allow_write_access_fieldID;

getLoginUrl::getLoginUrl()
  : chat_id_()
  , message_id_()
  , button_id_()
  , allow_write_access_()
{}

getLoginUrl::getLoginUrl(int53 chat_id_, int53 message_id_, int53 button_id_, bool allow_write_access_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , button_id_(button_id_)
  , allow_write_access_(allow_write_access_)
{}

const std::int32_t getLoginUrl::ID;

object_ptr<getLoginUrl> getLoginUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLoginUrl> res = make_object<getLoginUrl>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->button_id_ = env->GetLongField(p, res->button_id_fieldID);
  res->allow_write_access_ = (env->GetBooleanField(p, res->allow_write_access_fieldID) != 0);
  return res;
}

void getLoginUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLoginUrl");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("buttonId", button_id_);
    s.store_field("allowWriteAccess", allow_write_access_);
    s.store_class_end();
  }
}

getLoginUrl::ReturnType getLoginUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getLoginUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLoginUrl").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "J");
    allow_write_access_fieldID = jni::get_field_id(env, Class, "allowWriteAccess", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getLoginUrlInfo::Class;
jfieldID getLoginUrlInfo::chat_id_fieldID;
jfieldID getLoginUrlInfo::message_id_fieldID;
jfieldID getLoginUrlInfo::button_id_fieldID;

getLoginUrlInfo::getLoginUrlInfo()
  : chat_id_()
  , message_id_()
  , button_id_()
{}

getLoginUrlInfo::getLoginUrlInfo(int53 chat_id_, int53 message_id_, int53 button_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , button_id_(button_id_)
{}

const std::int32_t getLoginUrlInfo::ID;

object_ptr<getLoginUrlInfo> getLoginUrlInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getLoginUrlInfo> res = make_object<getLoginUrlInfo>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->button_id_ = env->GetLongField(p, res->button_id_fieldID);
  return res;
}

void getLoginUrlInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetLoginUrlInfo");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("buttonId", button_id_);
    s.store_class_end();
  }
}

getLoginUrlInfo::ReturnType getLoginUrlInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return LoginUrlInfo::fetch(env, p);
}

void getLoginUrlInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetLoginUrlInfo").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMainWebApp::Class;
jfieldID getMainWebApp::chat_id_fieldID;
jfieldID getMainWebApp::bot_user_id_fieldID;
jfieldID getMainWebApp::start_parameter_fieldID;
jfieldID getMainWebApp::parameters_fieldID;

getMainWebApp::getMainWebApp()
  : chat_id_()
  , bot_user_id_()
  , start_parameter_()
  , parameters_()
{}

getMainWebApp::getMainWebApp(int53 chat_id_, int53 bot_user_id_, string const &start_parameter_, object_ptr<webAppOpenParameters> &&parameters_)
  : chat_id_(chat_id_)
  , bot_user_id_(bot_user_id_)
  , start_parameter_(start_parameter_)
  , parameters_(std::move(parameters_))
{}

const std::int32_t getMainWebApp::ID;

object_ptr<getMainWebApp> getMainWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMainWebApp> res = make_object<getMainWebApp>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->parameters_ = jni::fetch_tl_object<webAppOpenParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  return res;
}

void getMainWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMainWebApp");
    s.store_field("chatId", chat_id_);
    s.store_field("botUserId", bot_user_id_);
    s.store_field("startParameter", start_parameter_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_class_end();
  }
}

getMainWebApp::ReturnType getMainWebApp::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return mainWebApp::fetch(env, p);
}

void getMainWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMainWebApp").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getMapThumbnailFile::Class;
jfieldID getMapThumbnailFile::location_fieldID;
jfieldID getMapThumbnailFile::zoom_fieldID;
jfieldID getMapThumbnailFile::width_fieldID;
jfieldID getMapThumbnailFile::height_fieldID;
jfieldID getMapThumbnailFile::scale_fieldID;
jfieldID getMapThumbnailFile::chat_id_fieldID;

getMapThumbnailFile::getMapThumbnailFile()
  : location_()
  , zoom_()
  , width_()
  , height_()
  , scale_()
  , chat_id_()
{}

getMapThumbnailFile::getMapThumbnailFile(object_ptr<location> &&location_, int32 zoom_, int32 width_, int32 height_, int32 scale_, int53 chat_id_)
  : location_(std::move(location_))
  , zoom_(zoom_)
  , width_(width_)
  , height_(height_)
  , scale_(scale_)
  , chat_id_(chat_id_)
{}

const std::int32_t getMapThumbnailFile::ID;

object_ptr<getMapThumbnailFile> getMapThumbnailFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMapThumbnailFile> res = make_object<getMapThumbnailFile>();
  res->location_ = jni::fetch_tl_object<location>(env, jni::fetch_object(env, p, res->location_fieldID));
  res->zoom_ = env->GetIntField(p, res->zoom_fieldID);
  res->width_ = env->GetIntField(p, res->width_fieldID);
  res->height_ = env->GetIntField(p, res->height_fieldID);
  res->scale_ = env->GetIntField(p, res->scale_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getMapThumbnailFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMapThumbnailFile");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_field("zoom", zoom_);
    s.store_field("width", width_);
    s.store_field("height", height_);
    s.store_field("scale", scale_);
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getMapThumbnailFile::ReturnType getMapThumbnailFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void getMapThumbnailFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMapThumbnailFile").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Location;").c_str());
    zoom_fieldID = jni::get_field_id(env, Class, "zoom", "I");
    width_fieldID = jni::get_field_id(env, Class, "width", "I");
    height_fieldID = jni::get_field_id(env, Class, "height", "I");
    scale_fieldID = jni::get_field_id(env, Class, "scale", "I");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMarkdownText::Class;
jfieldID getMarkdownText::text_fieldID;

getMarkdownText::getMarkdownText()
  : text_()
{}

getMarkdownText::getMarkdownText(object_ptr<formattedText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t getMarkdownText::ID;

object_ptr<getMarkdownText> getMarkdownText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMarkdownText> res = make_object<getMarkdownText>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return res;
}

void getMarkdownText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMarkdownText");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

getMarkdownText::ReturnType getMarkdownText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return formattedText::fetch(env, p);
}

void getMarkdownText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMarkdownText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getMe::Class;

getMe::getMe() {
}

const std::int32_t getMe::ID;

object_ptr<getMe> getMe::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getMe>();
}

void getMe::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMe");
    s.store_class_end();
  }
}

getMe::ReturnType getMe::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return user::fetch(env, p);
}

void getMe::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMe").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getMenuButton::Class;
jfieldID getMenuButton::user_id_fieldID;

getMenuButton::getMenuButton()
  : user_id_()
{}

getMenuButton::getMenuButton(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getMenuButton::ID;

object_ptr<getMenuButton> getMenuButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMenuButton> res = make_object<getMenuButton>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getMenuButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMenuButton");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getMenuButton::ReturnType getMenuButton::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return botMenuButton::fetch(env, p);
}

void getMenuButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMenuButton").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessage::Class;
jfieldID getMessage::chat_id_fieldID;
jfieldID getMessage::message_id_fieldID;

getMessage::getMessage()
  : chat_id_()
  , message_id_()
{}

getMessage::getMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessage::ID;

object_ptr<getMessage> getMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessage> res = make_object<getMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessage::ReturnType getMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageAddedReactions::Class;
jfieldID getMessageAddedReactions::chat_id_fieldID;
jfieldID getMessageAddedReactions::message_id_fieldID;
jfieldID getMessageAddedReactions::reaction_type_fieldID;
jfieldID getMessageAddedReactions::offset_fieldID;
jfieldID getMessageAddedReactions::limit_fieldID;

getMessageAddedReactions::getMessageAddedReactions()
  : chat_id_()
  , message_id_()
  , reaction_type_()
  , offset_()
  , limit_()
{}

getMessageAddedReactions::getMessageAddedReactions(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_, string const &offset_, int32 limit_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reaction_type_(std::move(reaction_type_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getMessageAddedReactions::ID;

object_ptr<getMessageAddedReactions> getMessageAddedReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageAddedReactions> res = make_object<getMessageAddedReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getMessageAddedReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageAddedReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getMessageAddedReactions::ReturnType getMessageAddedReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return addedReactions::fetch(env, p);
}

void getMessageAddedReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageAddedReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageAvailableReactions::Class;
jfieldID getMessageAvailableReactions::chat_id_fieldID;
jfieldID getMessageAvailableReactions::message_id_fieldID;
jfieldID getMessageAvailableReactions::row_size_fieldID;

getMessageAvailableReactions::getMessageAvailableReactions()
  : chat_id_()
  , message_id_()
  , row_size_()
{}

getMessageAvailableReactions::getMessageAvailableReactions(int53 chat_id_, int53 message_id_, int32 row_size_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , row_size_(row_size_)
{}

const std::int32_t getMessageAvailableReactions::ID;

object_ptr<getMessageAvailableReactions> getMessageAvailableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageAvailableReactions> res = make_object<getMessageAvailableReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->row_size_ = env->GetIntField(p, res->row_size_fieldID);
  return res;
}

void getMessageAvailableReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageAvailableReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("rowSize", row_size_);
    s.store_class_end();
  }
}

getMessageAvailableReactions::ReturnType getMessageAvailableReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return availableReactions::fetch(env, p);
}

void getMessageAvailableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageAvailableReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    row_size_fieldID = jni::get_field_id(env, Class, "rowSize", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageEffect::Class;
jfieldID getMessageEffect::effect_id_fieldID;

getMessageEffect::getMessageEffect()
  : effect_id_()
{}

getMessageEffect::getMessageEffect(int64 effect_id_)
  : effect_id_(effect_id_)
{}

const std::int32_t getMessageEffect::ID;

object_ptr<getMessageEffect> getMessageEffect::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageEffect> res = make_object<getMessageEffect>();
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  return res;
}

void getMessageEffect::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageEffect");
    s.store_field("effectId", effect_id_);
    s.store_class_end();
  }
}

getMessageEffect::ReturnType getMessageEffect::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageEffect::fetch(env, p);
}

void getMessageEffect::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageEffect").c_str());
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageEmbeddingCode::Class;
jfieldID getMessageEmbeddingCode::chat_id_fieldID;
jfieldID getMessageEmbeddingCode::message_id_fieldID;
jfieldID getMessageEmbeddingCode::for_album_fieldID;

getMessageEmbeddingCode::getMessageEmbeddingCode()
  : chat_id_()
  , message_id_()
  , for_album_()
{}

getMessageEmbeddingCode::getMessageEmbeddingCode(int53 chat_id_, int53 message_id_, bool for_album_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , for_album_(for_album_)
{}

const std::int32_t getMessageEmbeddingCode::ID;

object_ptr<getMessageEmbeddingCode> getMessageEmbeddingCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageEmbeddingCode> res = make_object<getMessageEmbeddingCode>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->for_album_ = (env->GetBooleanField(p, res->for_album_fieldID) != 0);
  return res;
}

void getMessageEmbeddingCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageEmbeddingCode");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("forAlbum", for_album_);
    s.store_class_end();
  }
}

getMessageEmbeddingCode::ReturnType getMessageEmbeddingCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getMessageEmbeddingCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageEmbeddingCode").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    for_album_fieldID = jni::get_field_id(env, Class, "forAlbum", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageFileType::Class;
jfieldID getMessageFileType::message_file_head_fieldID;

getMessageFileType::getMessageFileType()
  : message_file_head_()
{}

getMessageFileType::getMessageFileType(string const &message_file_head_)
  : message_file_head_(message_file_head_)
{}

const std::int32_t getMessageFileType::ID;

object_ptr<getMessageFileType> getMessageFileType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageFileType> res = make_object<getMessageFileType>();
  res->message_file_head_ = jni::fetch_string(env, p, res->message_file_head_fieldID);
  return res;
}

void getMessageFileType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageFileType");
    s.store_field("messageFileHead", message_file_head_);
    s.store_class_end();
  }
}

getMessageFileType::ReturnType getMessageFileType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return MessageFileType::fetch(env, p);
}

void getMessageFileType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageFileType").c_str());
    message_file_head_fieldID = jni::get_field_id(env, Class, "messageFileHead", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageImportConfirmationText::Class;
jfieldID getMessageImportConfirmationText::chat_id_fieldID;

getMessageImportConfirmationText::getMessageImportConfirmationText()
  : chat_id_()
{}

getMessageImportConfirmationText::getMessageImportConfirmationText(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getMessageImportConfirmationText::ID;

object_ptr<getMessageImportConfirmationText> getMessageImportConfirmationText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageImportConfirmationText> res = make_object<getMessageImportConfirmationText>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getMessageImportConfirmationText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageImportConfirmationText");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getMessageImportConfirmationText::ReturnType getMessageImportConfirmationText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getMessageImportConfirmationText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageImportConfirmationText").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageLink::Class;
jfieldID getMessageLink::chat_id_fieldID;
jfieldID getMessageLink::message_id_fieldID;
jfieldID getMessageLink::media_timestamp_fieldID;
jfieldID getMessageLink::for_album_fieldID;
jfieldID getMessageLink::in_message_thread_fieldID;

getMessageLink::getMessageLink()
  : chat_id_()
  , message_id_()
  , media_timestamp_()
  , for_album_()
  , in_message_thread_()
{}

getMessageLink::getMessageLink(int53 chat_id_, int53 message_id_, int32 media_timestamp_, bool for_album_, bool in_message_thread_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , media_timestamp_(media_timestamp_)
  , for_album_(for_album_)
  , in_message_thread_(in_message_thread_)
{}

const std::int32_t getMessageLink::ID;

object_ptr<getMessageLink> getMessageLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageLink> res = make_object<getMessageLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->media_timestamp_ = env->GetIntField(p, res->media_timestamp_fieldID);
  res->for_album_ = (env->GetBooleanField(p, res->for_album_fieldID) != 0);
  res->in_message_thread_ = (env->GetBooleanField(p, res->in_message_thread_fieldID) != 0);
  return res;
}

void getMessageLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageLink");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("mediaTimestamp", media_timestamp_);
    s.store_field("forAlbum", for_album_);
    s.store_field("inMessageThread", in_message_thread_);
    s.store_class_end();
  }
}

getMessageLink::ReturnType getMessageLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageLink::fetch(env, p);
}

void getMessageLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    media_timestamp_fieldID = jni::get_field_id(env, Class, "mediaTimestamp", "I");
    for_album_fieldID = jni::get_field_id(env, Class, "forAlbum", "Z");
    in_message_thread_fieldID = jni::get_field_id(env, Class, "inMessageThread", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageLinkInfo::Class;
jfieldID getMessageLinkInfo::url_fieldID;

getMessageLinkInfo::getMessageLinkInfo()
  : url_()
{}

getMessageLinkInfo::getMessageLinkInfo(string const &url_)
  : url_(url_)
{}

const std::int32_t getMessageLinkInfo::ID;

object_ptr<getMessageLinkInfo> getMessageLinkInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageLinkInfo> res = make_object<getMessageLinkInfo>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  return res;
}

void getMessageLinkInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageLinkInfo");
    s.store_field("url", url_);
    s.store_class_end();
  }
}

getMessageLinkInfo::ReturnType getMessageLinkInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageLinkInfo::fetch(env, p);
}

void getMessageLinkInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageLinkInfo").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageLocally::Class;
jfieldID getMessageLocally::chat_id_fieldID;
jfieldID getMessageLocally::message_id_fieldID;

getMessageLocally::getMessageLocally()
  : chat_id_()
  , message_id_()
{}

getMessageLocally::getMessageLocally(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessageLocally::ID;

object_ptr<getMessageLocally> getMessageLocally::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageLocally> res = make_object<getMessageLocally>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessageLocally::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageLocally");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessageLocally::ReturnType getMessageLocally::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getMessageLocally::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageLocally").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageProperties::Class;
jfieldID getMessageProperties::chat_id_fieldID;
jfieldID getMessageProperties::message_id_fieldID;

getMessageProperties::getMessageProperties()
  : chat_id_()
  , message_id_()
{}

getMessageProperties::getMessageProperties(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessageProperties::ID;

object_ptr<getMessageProperties> getMessageProperties::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageProperties> res = make_object<getMessageProperties>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessageProperties::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageProperties");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessageProperties::ReturnType getMessageProperties::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageProperties::fetch(env, p);
}

void getMessageProperties::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageProperties").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessagePublicForwards::Class;
jfieldID getMessagePublicForwards::chat_id_fieldID;
jfieldID getMessagePublicForwards::message_id_fieldID;
jfieldID getMessagePublicForwards::offset_fieldID;
jfieldID getMessagePublicForwards::limit_fieldID;

getMessagePublicForwards::getMessagePublicForwards()
  : chat_id_()
  , message_id_()
  , offset_()
  , limit_()
{}

getMessagePublicForwards::getMessagePublicForwards(int53 chat_id_, int53 message_id_, string const &offset_, int32 limit_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getMessagePublicForwards::ID;

object_ptr<getMessagePublicForwards> getMessagePublicForwards::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessagePublicForwards> res = make_object<getMessagePublicForwards>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getMessagePublicForwards::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessagePublicForwards");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getMessagePublicForwards::ReturnType getMessagePublicForwards::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return publicForwards::fetch(env, p);
}

void getMessagePublicForwards::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessagePublicForwards").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageReadDate::Class;
jfieldID getMessageReadDate::chat_id_fieldID;
jfieldID getMessageReadDate::message_id_fieldID;

getMessageReadDate::getMessageReadDate()
  : chat_id_()
  , message_id_()
{}

getMessageReadDate::getMessageReadDate(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessageReadDate::ID;

object_ptr<getMessageReadDate> getMessageReadDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageReadDate> res = make_object<getMessageReadDate>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessageReadDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageReadDate");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessageReadDate::ReturnType getMessageReadDate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return MessageReadDate::fetch(env, p);
}

void getMessageReadDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageReadDate").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageStatistics::Class;
jfieldID getMessageStatistics::chat_id_fieldID;
jfieldID getMessageStatistics::message_id_fieldID;
jfieldID getMessageStatistics::is_dark_fieldID;

getMessageStatistics::getMessageStatistics()
  : chat_id_()
  , message_id_()
  , is_dark_()
{}

getMessageStatistics::getMessageStatistics(int53 chat_id_, int53 message_id_, bool is_dark_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , is_dark_(is_dark_)
{}

const std::int32_t getMessageStatistics::ID;

object_ptr<getMessageStatistics> getMessageStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageStatistics> res = make_object<getMessageStatistics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  return res;
}

void getMessageStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageStatistics");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("isDark", is_dark_);
    s.store_class_end();
  }
}

getMessageStatistics::ReturnType getMessageStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageStatistics::fetch(env, p);
}

void getMessageStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageStatistics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageThread::Class;
jfieldID getMessageThread::chat_id_fieldID;
jfieldID getMessageThread::message_id_fieldID;

getMessageThread::getMessageThread()
  : chat_id_()
  , message_id_()
{}

getMessageThread::getMessageThread(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessageThread::ID;

object_ptr<getMessageThread> getMessageThread::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageThread> res = make_object<getMessageThread>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessageThread::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageThread");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessageThread::ReturnType getMessageThread::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageThreadInfo::fetch(env, p);
}

void getMessageThread::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageThread").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageThreadHistory::Class;
jfieldID getMessageThreadHistory::chat_id_fieldID;
jfieldID getMessageThreadHistory::message_id_fieldID;
jfieldID getMessageThreadHistory::from_message_id_fieldID;
jfieldID getMessageThreadHistory::offset_fieldID;
jfieldID getMessageThreadHistory::limit_fieldID;

getMessageThreadHistory::getMessageThreadHistory()
  : chat_id_()
  , message_id_()
  , from_message_id_()
  , offset_()
  , limit_()
{}

getMessageThreadHistory::getMessageThreadHistory(int53 chat_id_, int53 message_id_, int53 from_message_id_, int32 offset_, int32 limit_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , from_message_id_(from_message_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getMessageThreadHistory::ID;

object_ptr<getMessageThreadHistory> getMessageThreadHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageThreadHistory> res = make_object<getMessageThreadHistory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getMessageThreadHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageThreadHistory");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getMessageThreadHistory::ReturnType getMessageThreadHistory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void getMessageThreadHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageThreadHistory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getMessageViewers::Class;
jfieldID getMessageViewers::chat_id_fieldID;
jfieldID getMessageViewers::message_id_fieldID;

getMessageViewers::getMessageViewers()
  : chat_id_()
  , message_id_()
{}

getMessageViewers::getMessageViewers(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getMessageViewers::ID;

object_ptr<getMessageViewers> getMessageViewers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessageViewers> res = make_object<getMessageViewers>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getMessageViewers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessageViewers");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getMessageViewers::ReturnType getMessageViewers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageViewers::fetch(env, p);
}

void getMessageViewers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessageViewers").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getMessages::Class;
jfieldID getMessages::chat_id_fieldID;
jfieldID getMessages::message_ids_fieldID;

getMessages::getMessages()
  : chat_id_()
  , message_ids_()
{}

getMessages::getMessages(int53 chat_id_, array<int53> &&message_ids_)
  : chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t getMessages::ID;

object_ptr<getMessages> getMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getMessages> res = make_object<getMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return res;
}

void getMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetMessages");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

getMessages::ReturnType getMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void getMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass getNetworkStatistics::Class;
jfieldID getNetworkStatistics::only_current_fieldID;

getNetworkStatistics::getNetworkStatistics()
  : only_current_()
{}

getNetworkStatistics::getNetworkStatistics(bool only_current_)
  : only_current_(only_current_)
{}

const std::int32_t getNetworkStatistics::ID;

object_ptr<getNetworkStatistics> getNetworkStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getNetworkStatistics> res = make_object<getNetworkStatistics>();
  res->only_current_ = (env->GetBooleanField(p, res->only_current_fieldID) != 0);
  return res;
}

void getNetworkStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetNetworkStatistics");
    s.store_field("onlyCurrent", only_current_);
    s.store_class_end();
  }
}

getNetworkStatistics::ReturnType getNetworkStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return networkStatistics::fetch(env, p);
}

void getNetworkStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetNetworkStatistics").c_str());
    only_current_fieldID = jni::get_field_id(env, Class, "onlyCurrent", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getNewChatPrivacySettings::Class;

getNewChatPrivacySettings::getNewChatPrivacySettings() {
}

const std::int32_t getNewChatPrivacySettings::ID;

object_ptr<getNewChatPrivacySettings> getNewChatPrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getNewChatPrivacySettings>();
}

void getNewChatPrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetNewChatPrivacySettings");
    s.store_class_end();
  }
}

getNewChatPrivacySettings::ReturnType getNewChatPrivacySettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return newChatPrivacySettings::fetch(env, p);
}

void getNewChatPrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetNewChatPrivacySettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getOption::Class;
jfieldID getOption::name_fieldID;

getOption::getOption()
  : name_()
{}

getOption::getOption(string const &name_)
  : name_(name_)
{}

const std::int32_t getOption::ID;

object_ptr<getOption> getOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getOption> res = make_object<getOption>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void getOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetOption");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

getOption::ReturnType getOption::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return OptionValue::fetch(env, p);
}

void getOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetOption").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getOwnedBots::Class;

getOwnedBots::getOwnedBots() {
}

const std::int32_t getOwnedBots::ID;

object_ptr<getOwnedBots> getOwnedBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getOwnedBots>();
}

void getOwnedBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetOwnedBots");
    s.store_class_end();
  }
}

getOwnedBots::ReturnType getOwnedBots::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void getOwnedBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetOwnedBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getOwnedStickerSets::Class;
jfieldID getOwnedStickerSets::offset_sticker_set_id_fieldID;
jfieldID getOwnedStickerSets::limit_fieldID;

getOwnedStickerSets::getOwnedStickerSets()
  : offset_sticker_set_id_()
  , limit_()
{}

getOwnedStickerSets::getOwnedStickerSets(int64 offset_sticker_set_id_, int32 limit_)
  : offset_sticker_set_id_(offset_sticker_set_id_)
  , limit_(limit_)
{}

const std::int32_t getOwnedStickerSets::ID;

object_ptr<getOwnedStickerSets> getOwnedStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getOwnedStickerSets> res = make_object<getOwnedStickerSets>();
  res->offset_sticker_set_id_ = env->GetLongField(p, res->offset_sticker_set_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getOwnedStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetOwnedStickerSets");
    s.store_field("offsetStickerSetId", offset_sticker_set_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getOwnedStickerSets::ReturnType getOwnedStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void getOwnedStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetOwnedStickerSets").c_str());
    offset_sticker_set_id_fieldID = jni::get_field_id(env, Class, "offsetStickerSetId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getPaidMessageRevenue::Class;
jfieldID getPaidMessageRevenue::user_id_fieldID;

getPaidMessageRevenue::getPaidMessageRevenue()
  : user_id_()
{}

getPaidMessageRevenue::getPaidMessageRevenue(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getPaidMessageRevenue::ID;

object_ptr<getPaidMessageRevenue> getPaidMessageRevenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPaidMessageRevenue> res = make_object<getPaidMessageRevenue>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getPaidMessageRevenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPaidMessageRevenue");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getPaidMessageRevenue::ReturnType getPaidMessageRevenue::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starCount::fetch(env, p);
}

void getPaidMessageRevenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPaidMessageRevenue").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getPassportAuthorizationForm::Class;
jfieldID getPassportAuthorizationForm::bot_user_id_fieldID;
jfieldID getPassportAuthorizationForm::scope_fieldID;
jfieldID getPassportAuthorizationForm::public_key_fieldID;
jfieldID getPassportAuthorizationForm::nonce_fieldID;

getPassportAuthorizationForm::getPassportAuthorizationForm()
  : bot_user_id_()
  , scope_()
  , public_key_()
  , nonce_()
{}

getPassportAuthorizationForm::getPassportAuthorizationForm(int53 bot_user_id_, string const &scope_, string const &public_key_, string const &nonce_)
  : bot_user_id_(bot_user_id_)
  , scope_(scope_)
  , public_key_(public_key_)
  , nonce_(nonce_)
{}

const std::int32_t getPassportAuthorizationForm::ID;

object_ptr<getPassportAuthorizationForm> getPassportAuthorizationForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPassportAuthorizationForm> res = make_object<getPassportAuthorizationForm>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->scope_ = jni::fetch_string(env, p, res->scope_fieldID);
  res->public_key_ = jni::fetch_string(env, p, res->public_key_fieldID);
  res->nonce_ = jni::fetch_string(env, p, res->nonce_fieldID);
  return res;
}

void getPassportAuthorizationForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPassportAuthorizationForm");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("scope", scope_);
    s.store_field("publicKey", public_key_);
    s.store_field("nonce", nonce_);
    s.store_class_end();
  }
}

getPassportAuthorizationForm::ReturnType getPassportAuthorizationForm::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passportAuthorizationForm::fetch(env, p);
}

void getPassportAuthorizationForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPassportAuthorizationForm").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    scope_fieldID = jni::get_field_id(env, Class, "scope", "Ljava/lang/String;");
    public_key_fieldID = jni::get_field_id(env, Class, "publicKey", "Ljava/lang/String;");
    nonce_fieldID = jni::get_field_id(env, Class, "nonce", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPassportAuthorizationFormAvailableElements::Class;
jfieldID getPassportAuthorizationFormAvailableElements::authorization_form_id_fieldID;
jfieldID getPassportAuthorizationFormAvailableElements::password_fieldID;

getPassportAuthorizationFormAvailableElements::getPassportAuthorizationFormAvailableElements()
  : authorization_form_id_()
  , password_()
{}

getPassportAuthorizationFormAvailableElements::getPassportAuthorizationFormAvailableElements(int32 authorization_form_id_, string const &password_)
  : authorization_form_id_(authorization_form_id_)
  , password_(password_)
{}

const std::int32_t getPassportAuthorizationFormAvailableElements::ID;

object_ptr<getPassportAuthorizationFormAvailableElements> getPassportAuthorizationFormAvailableElements::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPassportAuthorizationFormAvailableElements> res = make_object<getPassportAuthorizationFormAvailableElements>();
  res->authorization_form_id_ = env->GetIntField(p, res->authorization_form_id_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getPassportAuthorizationFormAvailableElements::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPassportAuthorizationFormAvailableElements");
    s.store_field("authorizationFormId", authorization_form_id_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getPassportAuthorizationFormAvailableElements::ReturnType getPassportAuthorizationFormAvailableElements::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passportElementsWithErrors::fetch(env, p);
}

void getPassportAuthorizationFormAvailableElements::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPassportAuthorizationFormAvailableElements").c_str());
    authorization_form_id_fieldID = jni::get_field_id(env, Class, "authorizationFormId", "I");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPassportElement::Class;
jfieldID getPassportElement::type_fieldID;
jfieldID getPassportElement::password_fieldID;

getPassportElement::getPassportElement()
  : type_()
  , password_()
{}

getPassportElement::getPassportElement(object_ptr<PassportElementType> &&type_, string const &password_)
  : type_(std::move(type_))
  , password_(password_)
{}

const std::int32_t getPassportElement::ID;

object_ptr<getPassportElement> getPassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPassportElement> res = make_object<getPassportElement>();
  res->type_ = jni::fetch_tl_object<PassportElementType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getPassportElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPassportElement");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getPassportElement::ReturnType getPassportElement::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return PassportElement::fetch(env, p);
}

void getPassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPassportElement").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPasswordState::Class;

getPasswordState::getPasswordState() {
}

const std::int32_t getPasswordState::ID;

object_ptr<getPasswordState> getPasswordState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getPasswordState>();
}

void getPasswordState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPasswordState");
    s.store_class_end();
  }
}

getPasswordState::ReturnType getPasswordState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void getPasswordState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPasswordState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPaymentForm::Class;
jfieldID getPaymentForm::input_invoice_fieldID;
jfieldID getPaymentForm::theme_fieldID;

getPaymentForm::getPaymentForm()
  : input_invoice_()
  , theme_()
{}

getPaymentForm::getPaymentForm(object_ptr<InputInvoice> &&input_invoice_, object_ptr<themeParameters> &&theme_)
  : input_invoice_(std::move(input_invoice_))
  , theme_(std::move(theme_))
{}

const std::int32_t getPaymentForm::ID;

object_ptr<getPaymentForm> getPaymentForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPaymentForm> res = make_object<getPaymentForm>();
  res->input_invoice_ = jni::fetch_tl_object<InputInvoice>(env, jni::fetch_object(env, p, res->input_invoice_fieldID));
  res->theme_ = jni::fetch_tl_object<themeParameters>(env, jni::fetch_object(env, p, res->theme_fieldID));
  return res;
}

void getPaymentForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPaymentForm");
    s.store_object_field("inputInvoice", static_cast<const BaseObject *>(input_invoice_.get()));
    s.store_object_field("theme", static_cast<const BaseObject *>(theme_.get()));
    s.store_class_end();
  }
}

getPaymentForm::ReturnType getPaymentForm::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return paymentForm::fetch(env, p);
}

void getPaymentForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPaymentForm").c_str());
    input_invoice_fieldID = jni::get_field_id(env, Class, "inputInvoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputInvoice;").c_str());
    theme_fieldID = jni::get_field_id(env, Class, "theme", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPaymentReceipt::Class;
jfieldID getPaymentReceipt::chat_id_fieldID;
jfieldID getPaymentReceipt::message_id_fieldID;

getPaymentReceipt::getPaymentReceipt()
  : chat_id_()
  , message_id_()
{}

getPaymentReceipt::getPaymentReceipt(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getPaymentReceipt::ID;

object_ptr<getPaymentReceipt> getPaymentReceipt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPaymentReceipt> res = make_object<getPaymentReceipt>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getPaymentReceipt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPaymentReceipt");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getPaymentReceipt::ReturnType getPaymentReceipt::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return paymentReceipt::fetch(env, p);
}

void getPaymentReceipt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPaymentReceipt").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getPhoneNumberInfo::Class;
jfieldID getPhoneNumberInfo::phone_number_prefix_fieldID;

getPhoneNumberInfo::getPhoneNumberInfo()
  : phone_number_prefix_()
{}

getPhoneNumberInfo::getPhoneNumberInfo(string const &phone_number_prefix_)
  : phone_number_prefix_(phone_number_prefix_)
{}

const std::int32_t getPhoneNumberInfo::ID;

object_ptr<getPhoneNumberInfo> getPhoneNumberInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPhoneNumberInfo> res = make_object<getPhoneNumberInfo>();
  res->phone_number_prefix_ = jni::fetch_string(env, p, res->phone_number_prefix_fieldID);
  return res;
}

void getPhoneNumberInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPhoneNumberInfo");
    s.store_field("phoneNumberPrefix", phone_number_prefix_);
    s.store_class_end();
  }
}

getPhoneNumberInfo::ReturnType getPhoneNumberInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return phoneNumberInfo::fetch(env, p);
}

void getPhoneNumberInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPhoneNumberInfo").c_str());
    phone_number_prefix_fieldID = jni::get_field_id(env, Class, "phoneNumberPrefix", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPhoneNumberInfoSync::Class;
jfieldID getPhoneNumberInfoSync::language_code_fieldID;
jfieldID getPhoneNumberInfoSync::phone_number_prefix_fieldID;

getPhoneNumberInfoSync::getPhoneNumberInfoSync()
  : language_code_()
  , phone_number_prefix_()
{}

getPhoneNumberInfoSync::getPhoneNumberInfoSync(string const &language_code_, string const &phone_number_prefix_)
  : language_code_(language_code_)
  , phone_number_prefix_(phone_number_prefix_)
{}

const std::int32_t getPhoneNumberInfoSync::ID;

object_ptr<getPhoneNumberInfoSync> getPhoneNumberInfoSync::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPhoneNumberInfoSync> res = make_object<getPhoneNumberInfoSync>();
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->phone_number_prefix_ = jni::fetch_string(env, p, res->phone_number_prefix_fieldID);
  return res;
}

void getPhoneNumberInfoSync::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPhoneNumberInfoSync");
    s.store_field("languageCode", language_code_);
    s.store_field("phoneNumberPrefix", phone_number_prefix_);
    s.store_class_end();
  }
}

getPhoneNumberInfoSync::ReturnType getPhoneNumberInfoSync::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return phoneNumberInfo::fetch(env, p);
}

void getPhoneNumberInfoSync::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPhoneNumberInfoSync").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    phone_number_prefix_fieldID = jni::get_field_id(env, Class, "phoneNumberPrefix", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPollVoters::Class;
jfieldID getPollVoters::chat_id_fieldID;
jfieldID getPollVoters::message_id_fieldID;
jfieldID getPollVoters::option_id_fieldID;
jfieldID getPollVoters::offset_fieldID;
jfieldID getPollVoters::limit_fieldID;

getPollVoters::getPollVoters()
  : chat_id_()
  , message_id_()
  , option_id_()
  , offset_()
  , limit_()
{}

getPollVoters::getPollVoters(int53 chat_id_, int53 message_id_, int32 option_id_, int32 offset_, int32 limit_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , option_id_(option_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getPollVoters::ID;

object_ptr<getPollVoters> getPollVoters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPollVoters> res = make_object<getPollVoters>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->option_id_ = env->GetIntField(p, res->option_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getPollVoters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPollVoters");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("optionId", option_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getPollVoters::ReturnType getPollVoters::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageSenders::fetch(env, p);
}

void getPollVoters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPollVoters").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "I");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getPreferredCountryLanguage::Class;
jfieldID getPreferredCountryLanguage::country_code_fieldID;

getPreferredCountryLanguage::getPreferredCountryLanguage()
  : country_code_()
{}

getPreferredCountryLanguage::getPreferredCountryLanguage(string const &country_code_)
  : country_code_(country_code_)
{}

const std::int32_t getPreferredCountryLanguage::ID;

object_ptr<getPreferredCountryLanguage> getPreferredCountryLanguage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPreferredCountryLanguage> res = make_object<getPreferredCountryLanguage>();
  res->country_code_ = jni::fetch_string(env, p, res->country_code_fieldID);
  return res;
}

void getPreferredCountryLanguage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPreferredCountryLanguage");
    s.store_field("countryCode", country_code_);
    s.store_class_end();
  }
}

getPreferredCountryLanguage::ReturnType getPreferredCountryLanguage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getPreferredCountryLanguage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPreferredCountryLanguage").c_str());
    country_code_fieldID = jni::get_field_id(env, Class, "countryCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumFeatures::Class;
jfieldID getPremiumFeatures::source_fieldID;

getPremiumFeatures::getPremiumFeatures()
  : source_()
{}

getPremiumFeatures::getPremiumFeatures(object_ptr<PremiumSource> &&source_)
  : source_(std::move(source_))
{}

const std::int32_t getPremiumFeatures::ID;

object_ptr<getPremiumFeatures> getPremiumFeatures::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPremiumFeatures> res = make_object<getPremiumFeatures>();
  res->source_ = jni::fetch_tl_object<PremiumSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  return res;
}

void getPremiumFeatures::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumFeatures");
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_class_end();
  }
}

getPremiumFeatures::ReturnType getPremiumFeatures::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumFeatures::fetch(env, p);
}

void getPremiumFeatures::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumFeatures").c_str());
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumSource;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumGiftPaymentOptions::Class;

getPremiumGiftPaymentOptions::getPremiumGiftPaymentOptions() {
}

const std::int32_t getPremiumGiftPaymentOptions::ID;

object_ptr<getPremiumGiftPaymentOptions> getPremiumGiftPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getPremiumGiftPaymentOptions>();
}

void getPremiumGiftPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumGiftPaymentOptions");
    s.store_class_end();
  }
}

getPremiumGiftPaymentOptions::ReturnType getPremiumGiftPaymentOptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumGiftPaymentOptions::fetch(env, p);
}

void getPremiumGiftPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumGiftPaymentOptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumGiveawayPaymentOptions::Class;
jfieldID getPremiumGiveawayPaymentOptions::boosted_chat_id_fieldID;

getPremiumGiveawayPaymentOptions::getPremiumGiveawayPaymentOptions()
  : boosted_chat_id_()
{}

getPremiumGiveawayPaymentOptions::getPremiumGiveawayPaymentOptions(int53 boosted_chat_id_)
  : boosted_chat_id_(boosted_chat_id_)
{}

const std::int32_t getPremiumGiveawayPaymentOptions::ID;

object_ptr<getPremiumGiveawayPaymentOptions> getPremiumGiveawayPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPremiumGiveawayPaymentOptions> res = make_object<getPremiumGiveawayPaymentOptions>();
  res->boosted_chat_id_ = env->GetLongField(p, res->boosted_chat_id_fieldID);
  return res;
}

void getPremiumGiveawayPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumGiveawayPaymentOptions");
    s.store_field("boostedChatId", boosted_chat_id_);
    s.store_class_end();
  }
}

getPremiumGiveawayPaymentOptions::ReturnType getPremiumGiveawayPaymentOptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumGiveawayPaymentOptions::fetch(env, p);
}

void getPremiumGiveawayPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumGiveawayPaymentOptions").c_str());
    boosted_chat_id_fieldID = jni::get_field_id(env, Class, "boostedChatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumInfoSticker::Class;
jfieldID getPremiumInfoSticker::month_count_fieldID;

getPremiumInfoSticker::getPremiumInfoSticker()
  : month_count_()
{}

getPremiumInfoSticker::getPremiumInfoSticker(int32 month_count_)
  : month_count_(month_count_)
{}

const std::int32_t getPremiumInfoSticker::ID;

object_ptr<getPremiumInfoSticker> getPremiumInfoSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPremiumInfoSticker> res = make_object<getPremiumInfoSticker>();
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  return res;
}

void getPremiumInfoSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumInfoSticker");
    s.store_field("monthCount", month_count_);
    s.store_class_end();
  }
}

getPremiumInfoSticker::ReturnType getPremiumInfoSticker::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sticker::fetch(env, p);
}

void getPremiumInfoSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumInfoSticker").c_str());
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumLimit::Class;
jfieldID getPremiumLimit::limit_type_fieldID;

getPremiumLimit::getPremiumLimit()
  : limit_type_()
{}

getPremiumLimit::getPremiumLimit(object_ptr<PremiumLimitType> &&limit_type_)
  : limit_type_(std::move(limit_type_))
{}

const std::int32_t getPremiumLimit::ID;

object_ptr<getPremiumLimit> getPremiumLimit::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPremiumLimit> res = make_object<getPremiumLimit>();
  res->limit_type_ = jni::fetch_tl_object<PremiumLimitType>(env, jni::fetch_object(env, p, res->limit_type_fieldID));
  return res;
}

void getPremiumLimit::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumLimit");
    s.store_object_field("limitType", static_cast<const BaseObject *>(limit_type_.get()));
    s.store_class_end();
  }
}

getPremiumLimit::ReturnType getPremiumLimit::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumLimit::fetch(env, p);
}

void getPremiumLimit::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumLimit").c_str());
    limit_type_fieldID = jni::get_field_id(env, Class, "limitType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumLimitType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumState::Class;

getPremiumState::getPremiumState() {
}

const std::int32_t getPremiumState::ID;

object_ptr<getPremiumState> getPremiumState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getPremiumState>();
}

void getPremiumState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumState");
    s.store_class_end();
  }
}

getPremiumState::ReturnType getPremiumState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return premiumState::fetch(env, p);
}

void getPremiumState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumStickerExamples::Class;

getPremiumStickerExamples::getPremiumStickerExamples() {
}

const std::int32_t getPremiumStickerExamples::ID;

object_ptr<getPremiumStickerExamples> getPremiumStickerExamples::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getPremiumStickerExamples>();
}

void getPremiumStickerExamples::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumStickerExamples");
    s.store_class_end();
  }
}

getPremiumStickerExamples::ReturnType getPremiumStickerExamples::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getPremiumStickerExamples::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumStickerExamples").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getPremiumStickers::Class;
jfieldID getPremiumStickers::limit_fieldID;

getPremiumStickers::getPremiumStickers()
  : limit_()
{}

getPremiumStickers::getPremiumStickers(int32 limit_)
  : limit_(limit_)
{}

const std::int32_t getPremiumStickers::ID;

object_ptr<getPremiumStickers> getPremiumStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPremiumStickers> res = make_object<getPremiumStickers>();
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getPremiumStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPremiumStickers");
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getPremiumStickers::ReturnType getPremiumStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getPremiumStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPremiumStickers").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getPreparedInlineMessage::Class;
jfieldID getPreparedInlineMessage::bot_user_id_fieldID;
jfieldID getPreparedInlineMessage::prepared_message_id_fieldID;

getPreparedInlineMessage::getPreparedInlineMessage()
  : bot_user_id_()
  , prepared_message_id_()
{}

getPreparedInlineMessage::getPreparedInlineMessage(int53 bot_user_id_, string const &prepared_message_id_)
  : bot_user_id_(bot_user_id_)
  , prepared_message_id_(prepared_message_id_)
{}

const std::int32_t getPreparedInlineMessage::ID;

object_ptr<getPreparedInlineMessage> getPreparedInlineMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPreparedInlineMessage> res = make_object<getPreparedInlineMessage>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->prepared_message_id_ = jni::fetch_string(env, p, res->prepared_message_id_fieldID);
  return res;
}

void getPreparedInlineMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPreparedInlineMessage");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("preparedMessageId", prepared_message_id_);
    s.store_class_end();
  }
}

getPreparedInlineMessage::ReturnType getPreparedInlineMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return preparedInlineMessage::fetch(env, p);
}

void getPreparedInlineMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPreparedInlineMessage").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    prepared_message_id_fieldID = jni::get_field_id(env, Class, "preparedMessageId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getProxies::Class;

getProxies::getProxies() {
}

const std::int32_t getProxies::ID;

object_ptr<getProxies> getProxies::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getProxies>();
}

void getProxies::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetProxies");
    s.store_class_end();
  }
}

getProxies::ReturnType getProxies::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return proxies::fetch(env, p);
}

void getProxies::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetProxies").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getProxyLink::Class;
jfieldID getProxyLink::proxy_id_fieldID;

getProxyLink::getProxyLink()
  : proxy_id_()
{}

getProxyLink::getProxyLink(int32 proxy_id_)
  : proxy_id_(proxy_id_)
{}

const std::int32_t getProxyLink::ID;

object_ptr<getProxyLink> getProxyLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getProxyLink> res = make_object<getProxyLink>();
  res->proxy_id_ = env->GetIntField(p, res->proxy_id_fieldID);
  return res;
}

void getProxyLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetProxyLink");
    s.store_field("proxyId", proxy_id_);
    s.store_class_end();
  }
}

getProxyLink::ReturnType getProxyLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getProxyLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetProxyLink").c_str());
    proxy_id_fieldID = jni::get_field_id(env, Class, "proxyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getPushReceiverId::Class;
jfieldID getPushReceiverId::payload_fieldID;

getPushReceiverId::getPushReceiverId()
  : payload_()
{}

getPushReceiverId::getPushReceiverId(string const &payload_)
  : payload_(payload_)
{}

const std::int32_t getPushReceiverId::ID;

object_ptr<getPushReceiverId> getPushReceiverId::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getPushReceiverId> res = make_object<getPushReceiverId>();
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  return res;
}

void getPushReceiverId::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetPushReceiverId");
    s.store_field("payload", payload_);
    s.store_class_end();
  }
}

getPushReceiverId::ReturnType getPushReceiverId::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return pushReceiverId::fetch(env, p);
}

void getPushReceiverId::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetPushReceiverId").c_str());
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getReadDatePrivacySettings::Class;

getReadDatePrivacySettings::getReadDatePrivacySettings() {
}

const std::int32_t getReadDatePrivacySettings::ID;

object_ptr<getReadDatePrivacySettings> getReadDatePrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getReadDatePrivacySettings>();
}

void getReadDatePrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetReadDatePrivacySettings");
    s.store_class_end();
  }
}

getReadDatePrivacySettings::ReturnType getReadDatePrivacySettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return readDatePrivacySettings::fetch(env, p);
}

void getReadDatePrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetReadDatePrivacySettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getReceivedGift::Class;
jfieldID getReceivedGift::received_gift_id_fieldID;

getReceivedGift::getReceivedGift()
  : received_gift_id_()
{}

getReceivedGift::getReceivedGift(string const &received_gift_id_)
  : received_gift_id_(received_gift_id_)
{}

const std::int32_t getReceivedGift::ID;

object_ptr<getReceivedGift> getReceivedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getReceivedGift> res = make_object<getReceivedGift>();
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  return res;
}

void getReceivedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetReceivedGift");
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_class_end();
  }
}

getReceivedGift::ReturnType getReceivedGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return receivedGift::fetch(env, p);
}

void getReceivedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetReceivedGift").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getReceivedGifts::Class;
jfieldID getReceivedGifts::business_connection_id_fieldID;
jfieldID getReceivedGifts::owner_id_fieldID;
jfieldID getReceivedGifts::exclude_unsaved_fieldID;
jfieldID getReceivedGifts::exclude_saved_fieldID;
jfieldID getReceivedGifts::exclude_unlimited_fieldID;
jfieldID getReceivedGifts::exclude_limited_fieldID;
jfieldID getReceivedGifts::exclude_upgraded_fieldID;
jfieldID getReceivedGifts::sort_by_price_fieldID;
jfieldID getReceivedGifts::offset_fieldID;
jfieldID getReceivedGifts::limit_fieldID;

getReceivedGifts::getReceivedGifts()
  : business_connection_id_()
  , owner_id_()
  , exclude_unsaved_()
  , exclude_saved_()
  , exclude_unlimited_()
  , exclude_limited_()
  , exclude_upgraded_()
  , sort_by_price_()
  , offset_()
  , limit_()
{}

getReceivedGifts::getReceivedGifts(string const &business_connection_id_, object_ptr<MessageSender> &&owner_id_, bool exclude_unsaved_, bool exclude_saved_, bool exclude_unlimited_, bool exclude_limited_, bool exclude_upgraded_, bool sort_by_price_, string const &offset_, int32 limit_)
  : business_connection_id_(business_connection_id_)
  , owner_id_(std::move(owner_id_))
  , exclude_unsaved_(exclude_unsaved_)
  , exclude_saved_(exclude_saved_)
  , exclude_unlimited_(exclude_unlimited_)
  , exclude_limited_(exclude_limited_)
  , exclude_upgraded_(exclude_upgraded_)
  , sort_by_price_(sort_by_price_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getReceivedGifts::ID;

object_ptr<getReceivedGifts> getReceivedGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getReceivedGifts> res = make_object<getReceivedGifts>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->exclude_unsaved_ = (env->GetBooleanField(p, res->exclude_unsaved_fieldID) != 0);
  res->exclude_saved_ = (env->GetBooleanField(p, res->exclude_saved_fieldID) != 0);
  res->exclude_unlimited_ = (env->GetBooleanField(p, res->exclude_unlimited_fieldID) != 0);
  res->exclude_limited_ = (env->GetBooleanField(p, res->exclude_limited_fieldID) != 0);
  res->exclude_upgraded_ = (env->GetBooleanField(p, res->exclude_upgraded_fieldID) != 0);
  res->sort_by_price_ = (env->GetBooleanField(p, res->sort_by_price_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getReceivedGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetReceivedGifts");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("excludeUnsaved", exclude_unsaved_);
    s.store_field("excludeSaved", exclude_saved_);
    s.store_field("excludeUnlimited", exclude_unlimited_);
    s.store_field("excludeLimited", exclude_limited_);
    s.store_field("excludeUpgraded", exclude_upgraded_);
    s.store_field("sortByPrice", sort_by_price_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getReceivedGifts::ReturnType getReceivedGifts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return receivedGifts::fetch(env, p);
}

void getReceivedGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetReceivedGifts").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    exclude_unsaved_fieldID = jni::get_field_id(env, Class, "excludeUnsaved", "Z");
    exclude_saved_fieldID = jni::get_field_id(env, Class, "excludeSaved", "Z");
    exclude_unlimited_fieldID = jni::get_field_id(env, Class, "excludeUnlimited", "Z");
    exclude_limited_fieldID = jni::get_field_id(env, Class, "excludeLimited", "Z");
    exclude_upgraded_fieldID = jni::get_field_id(env, Class, "excludeUpgraded", "Z");
    sort_by_price_fieldID = jni::get_field_id(env, Class, "sortByPrice", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getRecentEmojiStatuses::Class;

getRecentEmojiStatuses::getRecentEmojiStatuses() {
}

const std::int32_t getRecentEmojiStatuses::ID;

object_ptr<getRecentEmojiStatuses> getRecentEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getRecentEmojiStatuses>();
}

void getRecentEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecentEmojiStatuses");
    s.store_class_end();
  }
}

getRecentEmojiStatuses::ReturnType getRecentEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatuses::fetch(env, p);
}

void getRecentEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecentEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getRecentInlineBots::Class;

getRecentInlineBots::getRecentInlineBots() {
}

const std::int32_t getRecentInlineBots::ID;

object_ptr<getRecentInlineBots> getRecentInlineBots::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getRecentInlineBots>();
}

void getRecentInlineBots::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecentInlineBots");
    s.store_class_end();
  }
}

getRecentInlineBots::ReturnType getRecentInlineBots::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void getRecentInlineBots::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecentInlineBots").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getRecentStickers::Class;
jfieldID getRecentStickers::is_attached_fieldID;

getRecentStickers::getRecentStickers()
  : is_attached_()
{}

getRecentStickers::getRecentStickers(bool is_attached_)
  : is_attached_(is_attached_)
{}

const std::int32_t getRecentStickers::ID;

object_ptr<getRecentStickers> getRecentStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRecentStickers> res = make_object<getRecentStickers>();
  res->is_attached_ = (env->GetBooleanField(p, res->is_attached_fieldID) != 0);
  return res;
}

void getRecentStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecentStickers");
    s.store_field("isAttached", is_attached_);
    s.store_class_end();
  }
}

getRecentStickers::ReturnType getRecentStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getRecentStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecentStickers").c_str());
    is_attached_fieldID = jni::get_field_id(env, Class, "isAttached", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getRecentlyOpenedChats::Class;
jfieldID getRecentlyOpenedChats::limit_fieldID;

getRecentlyOpenedChats::getRecentlyOpenedChats()
  : limit_()
{}

getRecentlyOpenedChats::getRecentlyOpenedChats(int32 limit_)
  : limit_(limit_)
{}

const std::int32_t getRecentlyOpenedChats::ID;

object_ptr<getRecentlyOpenedChats> getRecentlyOpenedChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRecentlyOpenedChats> res = make_object<getRecentlyOpenedChats>();
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getRecentlyOpenedChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecentlyOpenedChats");
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getRecentlyOpenedChats::ReturnType getRecentlyOpenedChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getRecentlyOpenedChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecentlyOpenedChats").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getRecentlyVisitedTMeUrls::Class;
jfieldID getRecentlyVisitedTMeUrls::referrer_fieldID;

getRecentlyVisitedTMeUrls::getRecentlyVisitedTMeUrls()
  : referrer_()
{}

getRecentlyVisitedTMeUrls::getRecentlyVisitedTMeUrls(string const &referrer_)
  : referrer_(referrer_)
{}

const std::int32_t getRecentlyVisitedTMeUrls::ID;

object_ptr<getRecentlyVisitedTMeUrls> getRecentlyVisitedTMeUrls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRecentlyVisitedTMeUrls> res = make_object<getRecentlyVisitedTMeUrls>();
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return res;
}

void getRecentlyVisitedTMeUrls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecentlyVisitedTMeUrls");
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

getRecentlyVisitedTMeUrls::ReturnType getRecentlyVisitedTMeUrls::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return tMeUrls::fetch(env, p);
}

void getRecentlyVisitedTMeUrls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecentlyVisitedTMeUrls").c_str());
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getRecommendedChatFolders::Class;

getRecommendedChatFolders::getRecommendedChatFolders() {
}

const std::int32_t getRecommendedChatFolders::ID;

object_ptr<getRecommendedChatFolders> getRecommendedChatFolders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getRecommendedChatFolders>();
}

void getRecommendedChatFolders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecommendedChatFolders");
    s.store_class_end();
  }
}

getRecommendedChatFolders::ReturnType getRecommendedChatFolders::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return recommendedChatFolders::fetch(env, p);
}

void getRecommendedChatFolders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecommendedChatFolders").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getRecommendedChats::Class;

getRecommendedChats::getRecommendedChats() {
}

const std::int32_t getRecommendedChats::ID;

object_ptr<getRecommendedChats> getRecommendedChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getRecommendedChats>();
}

void getRecommendedChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecommendedChats");
    s.store_class_end();
  }
}

getRecommendedChats::ReturnType getRecommendedChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getRecommendedChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecommendedChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getRecoveryEmailAddress::Class;
jfieldID getRecoveryEmailAddress::password_fieldID;

getRecoveryEmailAddress::getRecoveryEmailAddress()
  : password_()
{}

getRecoveryEmailAddress::getRecoveryEmailAddress(string const &password_)
  : password_(password_)
{}

const std::int32_t getRecoveryEmailAddress::ID;

object_ptr<getRecoveryEmailAddress> getRecoveryEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRecoveryEmailAddress> res = make_object<getRecoveryEmailAddress>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getRecoveryEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRecoveryEmailAddress");
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getRecoveryEmailAddress::ReturnType getRecoveryEmailAddress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return recoveryEmailAddress::fetch(env, p);
}

void getRecoveryEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRecoveryEmailAddress").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getRemoteFile::Class;
jfieldID getRemoteFile::remote_file_id_fieldID;
jfieldID getRemoteFile::file_type_fieldID;

getRemoteFile::getRemoteFile()
  : remote_file_id_()
  , file_type_()
{}

getRemoteFile::getRemoteFile(string const &remote_file_id_, object_ptr<FileType> &&file_type_)
  : remote_file_id_(remote_file_id_)
  , file_type_(std::move(file_type_))
{}

const std::int32_t getRemoteFile::ID;

object_ptr<getRemoteFile> getRemoteFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRemoteFile> res = make_object<getRemoteFile>();
  res->remote_file_id_ = jni::fetch_string(env, p, res->remote_file_id_fieldID);
  res->file_type_ = jni::fetch_tl_object<FileType>(env, jni::fetch_object(env, p, res->file_type_fieldID));
  return res;
}

void getRemoteFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRemoteFile");
    s.store_field("remoteFileId", remote_file_id_);
    s.store_object_field("fileType", static_cast<const BaseObject *>(file_type_.get()));
    s.store_class_end();
  }
}

getRemoteFile::ReturnType getRemoteFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void getRemoteFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRemoteFile").c_str());
    remote_file_id_fieldID = jni::get_field_id(env, Class, "remoteFileId", "Ljava/lang/String;");
    file_type_fieldID = jni::get_field_id(env, Class, "fileType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FileType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getRepliedMessage::Class;
jfieldID getRepliedMessage::chat_id_fieldID;
jfieldID getRepliedMessage::message_id_fieldID;

getRepliedMessage::getRepliedMessage()
  : chat_id_()
  , message_id_()
{}

getRepliedMessage::getRepliedMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t getRepliedMessage::ID;

object_ptr<getRepliedMessage> getRepliedMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getRepliedMessage> res = make_object<getRepliedMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void getRepliedMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetRepliedMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

getRepliedMessage::ReturnType getRepliedMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getRepliedMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetRepliedMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getSavedAnimations::Class;

getSavedAnimations::getSavedAnimations() {
}

const std::int32_t getSavedAnimations::ID;

object_ptr<getSavedAnimations> getSavedAnimations::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSavedAnimations>();
}

void getSavedAnimations::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedAnimations");
    s.store_class_end();
  }
}

getSavedAnimations::ReturnType getSavedAnimations::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return animations::fetch(env, p);
}

void getSavedAnimations::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedAnimations").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSavedMessagesTags::Class;
jfieldID getSavedMessagesTags::saved_messages_topic_id_fieldID;

getSavedMessagesTags::getSavedMessagesTags()
  : saved_messages_topic_id_()
{}

getSavedMessagesTags::getSavedMessagesTags(int53 saved_messages_topic_id_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t getSavedMessagesTags::ID;

object_ptr<getSavedMessagesTags> getSavedMessagesTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSavedMessagesTags> res = make_object<getSavedMessagesTags>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void getSavedMessagesTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedMessagesTags");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

getSavedMessagesTags::ReturnType getSavedMessagesTags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return savedMessagesTags::fetch(env, p);
}

void getSavedMessagesTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedMessagesTags").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getSavedMessagesTopicHistory::Class;
jfieldID getSavedMessagesTopicHistory::saved_messages_topic_id_fieldID;
jfieldID getSavedMessagesTopicHistory::from_message_id_fieldID;
jfieldID getSavedMessagesTopicHistory::offset_fieldID;
jfieldID getSavedMessagesTopicHistory::limit_fieldID;

getSavedMessagesTopicHistory::getSavedMessagesTopicHistory()
  : saved_messages_topic_id_()
  , from_message_id_()
  , offset_()
  , limit_()
{}

getSavedMessagesTopicHistory::getSavedMessagesTopicHistory(int53 saved_messages_topic_id_, int53 from_message_id_, int32 offset_, int32 limit_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , from_message_id_(from_message_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getSavedMessagesTopicHistory::ID;

object_ptr<getSavedMessagesTopicHistory> getSavedMessagesTopicHistory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSavedMessagesTopicHistory> res = make_object<getSavedMessagesTopicHistory>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getSavedMessagesTopicHistory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedMessagesTopicHistory");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getSavedMessagesTopicHistory::ReturnType getSavedMessagesTopicHistory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void getSavedMessagesTopicHistory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedMessagesTopicHistory").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getSavedMessagesTopicMessageByDate::Class;
jfieldID getSavedMessagesTopicMessageByDate::saved_messages_topic_id_fieldID;
jfieldID getSavedMessagesTopicMessageByDate::date_fieldID;

getSavedMessagesTopicMessageByDate::getSavedMessagesTopicMessageByDate()
  : saved_messages_topic_id_()
  , date_()
{}

getSavedMessagesTopicMessageByDate::getSavedMessagesTopicMessageByDate(int53 saved_messages_topic_id_, int32 date_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , date_(date_)
{}

const std::int32_t getSavedMessagesTopicMessageByDate::ID;

object_ptr<getSavedMessagesTopicMessageByDate> getSavedMessagesTopicMessageByDate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSavedMessagesTopicMessageByDate> res = make_object<getSavedMessagesTopicMessageByDate>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->date_ = env->GetIntField(p, res->date_fieldID);
  return res;
}

void getSavedMessagesTopicMessageByDate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedMessagesTopicMessageByDate");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_field("date", date_);
    s.store_class_end();
  }
}

getSavedMessagesTopicMessageByDate::ReturnType getSavedMessagesTopicMessageByDate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void getSavedMessagesTopicMessageByDate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedMessagesTopicMessageByDate").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    date_fieldID = jni::get_field_id(env, Class, "date", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getSavedNotificationSound::Class;
jfieldID getSavedNotificationSound::notification_sound_id_fieldID;

getSavedNotificationSound::getSavedNotificationSound()
  : notification_sound_id_()
{}

getSavedNotificationSound::getSavedNotificationSound(int64 notification_sound_id_)
  : notification_sound_id_(notification_sound_id_)
{}

const std::int32_t getSavedNotificationSound::ID;

object_ptr<getSavedNotificationSound> getSavedNotificationSound::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSavedNotificationSound> res = make_object<getSavedNotificationSound>();
  res->notification_sound_id_ = env->GetLongField(p, res->notification_sound_id_fieldID);
  return res;
}

void getSavedNotificationSound::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedNotificationSound");
    s.store_field("notificationSoundId", notification_sound_id_);
    s.store_class_end();
  }
}

getSavedNotificationSound::ReturnType getSavedNotificationSound::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return notificationSounds::fetch(env, p);
}

void getSavedNotificationSound::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedNotificationSound").c_str());
    notification_sound_id_fieldID = jni::get_field_id(env, Class, "notificationSoundId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getSavedNotificationSounds::Class;

getSavedNotificationSounds::getSavedNotificationSounds() {
}

const std::int32_t getSavedNotificationSounds::ID;

object_ptr<getSavedNotificationSounds> getSavedNotificationSounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSavedNotificationSounds>();
}

void getSavedNotificationSounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedNotificationSounds");
    s.store_class_end();
  }
}

getSavedNotificationSounds::ReturnType getSavedNotificationSounds::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return notificationSounds::fetch(env, p);
}

void getSavedNotificationSounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedNotificationSounds").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSavedOrderInfo::Class;

getSavedOrderInfo::getSavedOrderInfo() {
}

const std::int32_t getSavedOrderInfo::ID;

object_ptr<getSavedOrderInfo> getSavedOrderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSavedOrderInfo>();
}

void getSavedOrderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSavedOrderInfo");
    s.store_class_end();
  }
}

getSavedOrderInfo::ReturnType getSavedOrderInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return orderInfo::fetch(env, p);
}

void getSavedOrderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSavedOrderInfo").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getScopeNotificationSettings::Class;
jfieldID getScopeNotificationSettings::scope_fieldID;

getScopeNotificationSettings::getScopeNotificationSettings()
  : scope_()
{}

getScopeNotificationSettings::getScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_)
  : scope_(std::move(scope_))
{}

const std::int32_t getScopeNotificationSettings::ID;

object_ptr<getScopeNotificationSettings> getScopeNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getScopeNotificationSettings> res = make_object<getScopeNotificationSettings>();
  res->scope_ = jni::fetch_tl_object<NotificationSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  return res;
}

void getScopeNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetScopeNotificationSettings");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_class_end();
  }
}

getScopeNotificationSettings::ReturnType getScopeNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return scopeNotificationSettings::fetch(env, p);
}

void getScopeNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetScopeNotificationSettings").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationSettingsScope;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSearchSponsoredChats::Class;
jfieldID getSearchSponsoredChats::query_fieldID;

getSearchSponsoredChats::getSearchSponsoredChats()
  : query_()
{}

getSearchSponsoredChats::getSearchSponsoredChats(string const &query_)
  : query_(query_)
{}

const std::int32_t getSearchSponsoredChats::ID;

object_ptr<getSearchSponsoredChats> getSearchSponsoredChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSearchSponsoredChats> res = make_object<getSearchSponsoredChats>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return res;
}

void getSearchSponsoredChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSearchSponsoredChats");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

getSearchSponsoredChats::ReturnType getSearchSponsoredChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return sponsoredChats::fetch(env, p);
}

void getSearchSponsoredChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSearchSponsoredChats").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getSearchedForTags::Class;
jfieldID getSearchedForTags::tag_prefix_fieldID;
jfieldID getSearchedForTags::limit_fieldID;

getSearchedForTags::getSearchedForTags()
  : tag_prefix_()
  , limit_()
{}

getSearchedForTags::getSearchedForTags(string const &tag_prefix_, int32 limit_)
  : tag_prefix_(tag_prefix_)
  , limit_(limit_)
{}

const std::int32_t getSearchedForTags::ID;

object_ptr<getSearchedForTags> getSearchedForTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSearchedForTags> res = make_object<getSearchedForTags>();
  res->tag_prefix_ = jni::fetch_string(env, p, res->tag_prefix_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getSearchedForTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSearchedForTags");
    s.store_field("tagPrefix", tag_prefix_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getSearchedForTags::ReturnType getSearchedForTags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return hashtags::fetch(env, p);
}

void getSearchedForTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSearchedForTags").c_str());
    tag_prefix_fieldID = jni::get_field_id(env, Class, "tagPrefix", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getSecretChat::Class;
jfieldID getSecretChat::secret_chat_id_fieldID;

getSecretChat::getSecretChat()
  : secret_chat_id_()
{}

getSecretChat::getSecretChat(int32 secret_chat_id_)
  : secret_chat_id_(secret_chat_id_)
{}

const std::int32_t getSecretChat::ID;

object_ptr<getSecretChat> getSecretChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSecretChat> res = make_object<getSecretChat>();
  res->secret_chat_id_ = env->GetIntField(p, res->secret_chat_id_fieldID);
  return res;
}

void getSecretChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSecretChat");
    s.store_field("secretChatId", secret_chat_id_);
    s.store_class_end();
  }
}

getSecretChat::ReturnType getSecretChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return secretChat::fetch(env, p);
}

void getSecretChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSecretChat").c_str());
    secret_chat_id_fieldID = jni::get_field_id(env, Class, "secretChatId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStarAdAccountUrl::Class;
jfieldID getStarAdAccountUrl::owner_id_fieldID;

getStarAdAccountUrl::getStarAdAccountUrl()
  : owner_id_()
{}

getStarAdAccountUrl::getStarAdAccountUrl(object_ptr<MessageSender> &&owner_id_)
  : owner_id_(std::move(owner_id_))
{}

const std::int32_t getStarAdAccountUrl::ID;

object_ptr<getStarAdAccountUrl> getStarAdAccountUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarAdAccountUrl> res = make_object<getStarAdAccountUrl>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  return res;
}

void getStarAdAccountUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarAdAccountUrl");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_class_end();
  }
}

getStarAdAccountUrl::ReturnType getStarAdAccountUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getStarAdAccountUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarAdAccountUrl").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStarGiftPaymentOptions::Class;
jfieldID getStarGiftPaymentOptions::user_id_fieldID;

getStarGiftPaymentOptions::getStarGiftPaymentOptions()
  : user_id_()
{}

getStarGiftPaymentOptions::getStarGiftPaymentOptions(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getStarGiftPaymentOptions::ID;

object_ptr<getStarGiftPaymentOptions> getStarGiftPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarGiftPaymentOptions> res = make_object<getStarGiftPaymentOptions>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getStarGiftPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarGiftPaymentOptions");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getStarGiftPaymentOptions::ReturnType getStarGiftPaymentOptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starPaymentOptions::fetch(env, p);
}

void getStarGiftPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarGiftPaymentOptions").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getStarGiveawayPaymentOptions::Class;

getStarGiveawayPaymentOptions::getStarGiveawayPaymentOptions() {
}

const std::int32_t getStarGiveawayPaymentOptions::ID;

object_ptr<getStarGiveawayPaymentOptions> getStarGiveawayPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getStarGiveawayPaymentOptions>();
}

void getStarGiveawayPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarGiveawayPaymentOptions");
    s.store_class_end();
  }
}

getStarGiveawayPaymentOptions::ReturnType getStarGiveawayPaymentOptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starGiveawayPaymentOptions::fetch(env, p);
}

void getStarGiveawayPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarGiveawayPaymentOptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStarPaymentOptions::Class;

getStarPaymentOptions::getStarPaymentOptions() {
}

const std::int32_t getStarPaymentOptions::ID;

object_ptr<getStarPaymentOptions> getStarPaymentOptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getStarPaymentOptions>();
}

void getStarPaymentOptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarPaymentOptions");
    s.store_class_end();
  }
}

getStarPaymentOptions::ReturnType getStarPaymentOptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starPaymentOptions::fetch(env, p);
}

void getStarPaymentOptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarPaymentOptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStarRevenueStatistics::Class;
jfieldID getStarRevenueStatistics::owner_id_fieldID;
jfieldID getStarRevenueStatistics::is_dark_fieldID;

getStarRevenueStatistics::getStarRevenueStatistics()
  : owner_id_()
  , is_dark_()
{}

getStarRevenueStatistics::getStarRevenueStatistics(object_ptr<MessageSender> &&owner_id_, bool is_dark_)
  : owner_id_(std::move(owner_id_))
  , is_dark_(is_dark_)
{}

const std::int32_t getStarRevenueStatistics::ID;

object_ptr<getStarRevenueStatistics> getStarRevenueStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarRevenueStatistics> res = make_object<getStarRevenueStatistics>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  return res;
}

void getStarRevenueStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarRevenueStatistics");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("isDark", is_dark_);
    s.store_class_end();
  }
}

getStarRevenueStatistics::ReturnType getStarRevenueStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starRevenueStatistics::fetch(env, p);
}

void getStarRevenueStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarRevenueStatistics").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getStarSubscriptions::Class;
jfieldID getStarSubscriptions::only_expiring_fieldID;
jfieldID getStarSubscriptions::offset_fieldID;

getStarSubscriptions::getStarSubscriptions()
  : only_expiring_()
  , offset_()
{}

getStarSubscriptions::getStarSubscriptions(bool only_expiring_, string const &offset_)
  : only_expiring_(only_expiring_)
  , offset_(offset_)
{}

const std::int32_t getStarSubscriptions::ID;

object_ptr<getStarSubscriptions> getStarSubscriptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarSubscriptions> res = make_object<getStarSubscriptions>();
  res->only_expiring_ = (env->GetBooleanField(p, res->only_expiring_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  return res;
}

void getStarSubscriptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarSubscriptions");
    s.store_field("onlyExpiring", only_expiring_);
    s.store_field("offset", offset_);
    s.store_class_end();
  }
}

getStarSubscriptions::ReturnType getStarSubscriptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starSubscriptions::fetch(env, p);
}

void getStarSubscriptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarSubscriptions").c_str());
    only_expiring_fieldID = jni::get_field_id(env, Class, "onlyExpiring", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getStarTransactions::Class;
jfieldID getStarTransactions::owner_id_fieldID;
jfieldID getStarTransactions::subscription_id_fieldID;
jfieldID getStarTransactions::direction_fieldID;
jfieldID getStarTransactions::offset_fieldID;
jfieldID getStarTransactions::limit_fieldID;

getStarTransactions::getStarTransactions()
  : owner_id_()
  , subscription_id_()
  , direction_()
  , offset_()
  , limit_()
{}

getStarTransactions::getStarTransactions(object_ptr<MessageSender> &&owner_id_, string const &subscription_id_, object_ptr<StarTransactionDirection> &&direction_, string const &offset_, int32 limit_)
  : owner_id_(std::move(owner_id_))
  , subscription_id_(subscription_id_)
  , direction_(std::move(direction_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getStarTransactions::ID;

object_ptr<getStarTransactions> getStarTransactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarTransactions> res = make_object<getStarTransactions>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->subscription_id_ = jni::fetch_string(env, p, res->subscription_id_fieldID);
  res->direction_ = jni::fetch_tl_object<StarTransactionDirection>(env, jni::fetch_object(env, p, res->direction_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getStarTransactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarTransactions");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("subscriptionId", subscription_id_);
    s.store_object_field("direction", static_cast<const BaseObject *>(direction_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getStarTransactions::ReturnType getStarTransactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return starTransactions::fetch(env, p);
}

void getStarTransactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarTransactions").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    subscription_id_fieldID = jni::get_field_id(env, Class, "subscriptionId", "Ljava/lang/String;");
    direction_fieldID = jni::get_field_id(env, Class, "direction", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StarTransactionDirection;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStarWithdrawalUrl::Class;
jfieldID getStarWithdrawalUrl::owner_id_fieldID;
jfieldID getStarWithdrawalUrl::star_count_fieldID;
jfieldID getStarWithdrawalUrl::password_fieldID;

getStarWithdrawalUrl::getStarWithdrawalUrl()
  : owner_id_()
  , star_count_()
  , password_()
{}

getStarWithdrawalUrl::getStarWithdrawalUrl(object_ptr<MessageSender> &&owner_id_, int53 star_count_, string const &password_)
  : owner_id_(std::move(owner_id_))
  , star_count_(star_count_)
  , password_(password_)
{}

const std::int32_t getStarWithdrawalUrl::ID;

object_ptr<getStarWithdrawalUrl> getStarWithdrawalUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStarWithdrawalUrl> res = make_object<getStarWithdrawalUrl>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getStarWithdrawalUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStarWithdrawalUrl");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("starCount", star_count_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getStarWithdrawalUrl::ReturnType getStarWithdrawalUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getStarWithdrawalUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStarWithdrawalUrl").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getStatisticalGraph::Class;
jfieldID getStatisticalGraph::chat_id_fieldID;
jfieldID getStatisticalGraph::token_fieldID;
jfieldID getStatisticalGraph::x_fieldID;

getStatisticalGraph::getStatisticalGraph()
  : chat_id_()
  , token_()
  , x_()
{}

getStatisticalGraph::getStatisticalGraph(int53 chat_id_, string const &token_, int53 x_)
  : chat_id_(chat_id_)
  , token_(token_)
  , x_(x_)
{}

const std::int32_t getStatisticalGraph::ID;

object_ptr<getStatisticalGraph> getStatisticalGraph::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStatisticalGraph> res = make_object<getStatisticalGraph>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  res->x_ = env->GetLongField(p, res->x_fieldID);
  return res;
}

void getStatisticalGraph::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStatisticalGraph");
    s.store_field("chatId", chat_id_);
    s.store_field("token", token_);
    s.store_field("x", x_);
    s.store_class_end();
  }
}

getStatisticalGraph::ReturnType getStatisticalGraph::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return StatisticalGraph::fetch(env, p);
}

void getStatisticalGraph::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStatisticalGraph").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    x_fieldID = jni::get_field_id(env, Class, "x", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getStickerEmojis::Class;
jfieldID getStickerEmojis::sticker_fieldID;

getStickerEmojis::getStickerEmojis()
  : sticker_()
{}

getStickerEmojis::getStickerEmojis(object_ptr<InputFile> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t getStickerEmojis::ID;

object_ptr<getStickerEmojis> getStickerEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStickerEmojis> res = make_object<getStickerEmojis>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void getStickerEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStickerEmojis");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

getStickerEmojis::ReturnType getStickerEmojis::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojis::fetch(env, p);
}

void getStickerEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStickerEmojis").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStickerOutline::Class;
jfieldID getStickerOutline::sticker_file_id_fieldID;
jfieldID getStickerOutline::for_animated_emoji_fieldID;
jfieldID getStickerOutline::for_clicked_animated_emoji_message_fieldID;

getStickerOutline::getStickerOutline()
  : sticker_file_id_()
  , for_animated_emoji_()
  , for_clicked_animated_emoji_message_()
{}

getStickerOutline::getStickerOutline(int32 sticker_file_id_, bool for_animated_emoji_, bool for_clicked_animated_emoji_message_)
  : sticker_file_id_(sticker_file_id_)
  , for_animated_emoji_(for_animated_emoji_)
  , for_clicked_animated_emoji_message_(for_clicked_animated_emoji_message_)
{}

const std::int32_t getStickerOutline::ID;

object_ptr<getStickerOutline> getStickerOutline::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStickerOutline> res = make_object<getStickerOutline>();
  res->sticker_file_id_ = env->GetIntField(p, res->sticker_file_id_fieldID);
  res->for_animated_emoji_ = (env->GetBooleanField(p, res->for_animated_emoji_fieldID) != 0);
  res->for_clicked_animated_emoji_message_ = (env->GetBooleanField(p, res->for_clicked_animated_emoji_message_fieldID) != 0);
  return res;
}

void getStickerOutline::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStickerOutline");
    s.store_field("stickerFileId", sticker_file_id_);
    s.store_field("forAnimatedEmoji", for_animated_emoji_);
    s.store_field("forClickedAnimatedEmojiMessage", for_clicked_animated_emoji_message_);
    s.store_class_end();
  }
}

getStickerOutline::ReturnType getStickerOutline::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return outline::fetch(env, p);
}

void getStickerOutline::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStickerOutline").c_str());
    sticker_file_id_fieldID = jni::get_field_id(env, Class, "stickerFileId", "I");
    for_animated_emoji_fieldID = jni::get_field_id(env, Class, "forAnimatedEmoji", "Z");
    for_clicked_animated_emoji_message_fieldID = jni::get_field_id(env, Class, "forClickedAnimatedEmojiMessage", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getStickerSet::Class;
jfieldID getStickerSet::set_id_fieldID;

getStickerSet::getStickerSet()
  : set_id_()
{}

getStickerSet::getStickerSet(int64 set_id_)
  : set_id_(set_id_)
{}

const std::int32_t getStickerSet::ID;

object_ptr<getStickerSet> getStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStickerSet> res = make_object<getStickerSet>();
  res->set_id_ = env->GetLongField(p, res->set_id_fieldID);
  return res;
}

void getStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStickerSet");
    s.store_field("setId", set_id_);
    s.store_class_end();
  }
}

getStickerSet::ReturnType getStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSet::fetch(env, p);
}

void getStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStickerSet").c_str());
    set_id_fieldID = jni::get_field_id(env, Class, "setId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getStickerSetName::Class;
jfieldID getStickerSetName::set_id_fieldID;

getStickerSetName::getStickerSetName()
  : set_id_()
{}

getStickerSetName::getStickerSetName(int64 set_id_)
  : set_id_(set_id_)
{}

const std::int32_t getStickerSetName::ID;

object_ptr<getStickerSetName> getStickerSetName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStickerSetName> res = make_object<getStickerSetName>();
  res->set_id_ = env->GetLongField(p, res->set_id_fieldID);
  return res;
}

void getStickerSetName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStickerSetName");
    s.store_field("setId", set_id_);
    s.store_class_end();
  }
}

getStickerSetName::ReturnType getStickerSetName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getStickerSetName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStickerSetName").c_str());
    set_id_fieldID = jni::get_field_id(env, Class, "setId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getStickers::Class;
jfieldID getStickers::sticker_type_fieldID;
jfieldID getStickers::query_fieldID;
jfieldID getStickers::limit_fieldID;
jfieldID getStickers::chat_id_fieldID;

getStickers::getStickers()
  : sticker_type_()
  , query_()
  , limit_()
  , chat_id_()
{}

getStickers::getStickers(object_ptr<StickerType> &&sticker_type_, string const &query_, int32 limit_, int53 chat_id_)
  : sticker_type_(std::move(sticker_type_))
  , query_(query_)
  , limit_(limit_)
  , chat_id_(chat_id_)
{}

const std::int32_t getStickers::ID;

object_ptr<getStickers> getStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStickers> res = make_object<getStickers>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStickers");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getStickers::ReturnType getStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void getStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStickers").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getStorageStatistics::Class;
jfieldID getStorageStatistics::chat_limit_fieldID;

getStorageStatistics::getStorageStatistics()
  : chat_limit_()
{}

getStorageStatistics::getStorageStatistics(int32 chat_limit_)
  : chat_limit_(chat_limit_)
{}

const std::int32_t getStorageStatistics::ID;

object_ptr<getStorageStatistics> getStorageStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStorageStatistics> res = make_object<getStorageStatistics>();
  res->chat_limit_ = env->GetIntField(p, res->chat_limit_fieldID);
  return res;
}

void getStorageStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStorageStatistics");
    s.store_field("chatLimit", chat_limit_);
    s.store_class_end();
  }
}

getStorageStatistics::ReturnType getStorageStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storageStatistics::fetch(env, p);
}

void getStorageStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStorageStatistics").c_str());
    chat_limit_fieldID = jni::get_field_id(env, Class, "chatLimit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStorageStatisticsFast::Class;

getStorageStatisticsFast::getStorageStatisticsFast() {
}

const std::int32_t getStorageStatisticsFast::ID;

object_ptr<getStorageStatisticsFast> getStorageStatisticsFast::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getStorageStatisticsFast>();
}

void getStorageStatisticsFast::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStorageStatisticsFast");
    s.store_class_end();
  }
}

getStorageStatisticsFast::ReturnType getStorageStatisticsFast::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storageStatisticsFast::fetch(env, p);
}

void getStorageStatisticsFast::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStorageStatisticsFast").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStory::Class;
jfieldID getStory::story_poster_chat_id_fieldID;
jfieldID getStory::story_id_fieldID;
jfieldID getStory::only_local_fieldID;

getStory::getStory()
  : story_poster_chat_id_()
  , story_id_()
  , only_local_()
{}

getStory::getStory(int53 story_poster_chat_id_, int32 story_id_, bool only_local_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , only_local_(only_local_)
{}

const std::int32_t getStory::ID;

object_ptr<getStory> getStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStory> res = make_object<getStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void getStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

getStory::ReturnType getStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return story::fetch(env, p);
}

void getStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getStoryAvailableReactions::Class;
jfieldID getStoryAvailableReactions::row_size_fieldID;

getStoryAvailableReactions::getStoryAvailableReactions()
  : row_size_()
{}

getStoryAvailableReactions::getStoryAvailableReactions(int32 row_size_)
  : row_size_(row_size_)
{}

const std::int32_t getStoryAvailableReactions::ID;

object_ptr<getStoryAvailableReactions> getStoryAvailableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStoryAvailableReactions> res = make_object<getStoryAvailableReactions>();
  res->row_size_ = env->GetIntField(p, res->row_size_fieldID);
  return res;
}

void getStoryAvailableReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStoryAvailableReactions");
    s.store_field("rowSize", row_size_);
    s.store_class_end();
  }
}

getStoryAvailableReactions::ReturnType getStoryAvailableReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return availableReactions::fetch(env, p);
}

void getStoryAvailableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStoryAvailableReactions").c_str());
    row_size_fieldID = jni::get_field_id(env, Class, "rowSize", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStoryInteractions::Class;
jfieldID getStoryInteractions::story_id_fieldID;
jfieldID getStoryInteractions::query_fieldID;
jfieldID getStoryInteractions::only_contacts_fieldID;
jfieldID getStoryInteractions::prefer_forwards_fieldID;
jfieldID getStoryInteractions::prefer_with_reaction_fieldID;
jfieldID getStoryInteractions::offset_fieldID;
jfieldID getStoryInteractions::limit_fieldID;

getStoryInteractions::getStoryInteractions()
  : story_id_()
  , query_()
  , only_contacts_()
  , prefer_forwards_()
  , prefer_with_reaction_()
  , offset_()
  , limit_()
{}

getStoryInteractions::getStoryInteractions(int32 story_id_, string const &query_, bool only_contacts_, bool prefer_forwards_, bool prefer_with_reaction_, string const &offset_, int32 limit_)
  : story_id_(story_id_)
  , query_(query_)
  , only_contacts_(only_contacts_)
  , prefer_forwards_(prefer_forwards_)
  , prefer_with_reaction_(prefer_with_reaction_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getStoryInteractions::ID;

object_ptr<getStoryInteractions> getStoryInteractions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStoryInteractions> res = make_object<getStoryInteractions>();
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->only_contacts_ = (env->GetBooleanField(p, res->only_contacts_fieldID) != 0);
  res->prefer_forwards_ = (env->GetBooleanField(p, res->prefer_forwards_fieldID) != 0);
  res->prefer_with_reaction_ = (env->GetBooleanField(p, res->prefer_with_reaction_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getStoryInteractions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStoryInteractions");
    s.store_field("storyId", story_id_);
    s.store_field("query", query_);
    s.store_field("onlyContacts", only_contacts_);
    s.store_field("preferForwards", prefer_forwards_);
    s.store_field("preferWithReaction", prefer_with_reaction_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getStoryInteractions::ReturnType getStoryInteractions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storyInteractions::fetch(env, p);
}

void getStoryInteractions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStoryInteractions").c_str());
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    only_contacts_fieldID = jni::get_field_id(env, Class, "onlyContacts", "Z");
    prefer_forwards_fieldID = jni::get_field_id(env, Class, "preferForwards", "Z");
    prefer_with_reaction_fieldID = jni::get_field_id(env, Class, "preferWithReaction", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStoryNotificationSettingsExceptions::Class;

getStoryNotificationSettingsExceptions::getStoryNotificationSettingsExceptions() {
}

const std::int32_t getStoryNotificationSettingsExceptions::ID;

object_ptr<getStoryNotificationSettingsExceptions> getStoryNotificationSettingsExceptions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getStoryNotificationSettingsExceptions>();
}

void getStoryNotificationSettingsExceptions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStoryNotificationSettingsExceptions");
    s.store_class_end();
  }
}

getStoryNotificationSettingsExceptions::ReturnType getStoryNotificationSettingsExceptions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getStoryNotificationSettingsExceptions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStoryNotificationSettingsExceptions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getStoryPublicForwards::Class;
jfieldID getStoryPublicForwards::story_poster_chat_id_fieldID;
jfieldID getStoryPublicForwards::story_id_fieldID;
jfieldID getStoryPublicForwards::offset_fieldID;
jfieldID getStoryPublicForwards::limit_fieldID;

getStoryPublicForwards::getStoryPublicForwards()
  : story_poster_chat_id_()
  , story_id_()
  , offset_()
  , limit_()
{}

getStoryPublicForwards::getStoryPublicForwards(int53 story_poster_chat_id_, int32 story_id_, string const &offset_, int32 limit_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getStoryPublicForwards::ID;

object_ptr<getStoryPublicForwards> getStoryPublicForwards::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStoryPublicForwards> res = make_object<getStoryPublicForwards>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getStoryPublicForwards::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStoryPublicForwards");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getStoryPublicForwards::ReturnType getStoryPublicForwards::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return publicForwards::fetch(env, p);
}

void getStoryPublicForwards::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStoryPublicForwards").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getStoryStatistics::Class;
jfieldID getStoryStatistics::chat_id_fieldID;
jfieldID getStoryStatistics::story_id_fieldID;
jfieldID getStoryStatistics::is_dark_fieldID;

getStoryStatistics::getStoryStatistics()
  : chat_id_()
  , story_id_()
  , is_dark_()
{}

getStoryStatistics::getStoryStatistics(int53 chat_id_, int32 story_id_, bool is_dark_)
  : chat_id_(chat_id_)
  , story_id_(story_id_)
  , is_dark_(is_dark_)
{}

const std::int32_t getStoryStatistics::ID;

object_ptr<getStoryStatistics> getStoryStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getStoryStatistics> res = make_object<getStoryStatistics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->is_dark_ = (env->GetBooleanField(p, res->is_dark_fieldID) != 0);
  return res;
}

void getStoryStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetStoryStatistics");
    s.store_field("chatId", chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("isDark", is_dark_);
    s.store_class_end();
  }
}

getStoryStatistics::ReturnType getStoryStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storyStatistics::fetch(env, p);
}

void getStoryStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetStoryStatistics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    is_dark_fieldID = jni::get_field_id(env, Class, "isDark", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getSuggestedFileName::Class;
jfieldID getSuggestedFileName::file_id_fieldID;
jfieldID getSuggestedFileName::directory_fieldID;

getSuggestedFileName::getSuggestedFileName()
  : file_id_()
  , directory_()
{}

getSuggestedFileName::getSuggestedFileName(int32 file_id_, string const &directory_)
  : file_id_(file_id_)
  , directory_(directory_)
{}

const std::int32_t getSuggestedFileName::ID;

object_ptr<getSuggestedFileName> getSuggestedFileName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSuggestedFileName> res = make_object<getSuggestedFileName>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->directory_ = jni::fetch_string(env, p, res->directory_fieldID);
  return res;
}

void getSuggestedFileName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSuggestedFileName");
    s.store_field("fileId", file_id_);
    s.store_field("directory", directory_);
    s.store_class_end();
  }
}

getSuggestedFileName::ReturnType getSuggestedFileName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getSuggestedFileName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSuggestedFileName").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    directory_fieldID = jni::get_field_id(env, Class, "directory", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getSuggestedStickerSetName::Class;
jfieldID getSuggestedStickerSetName::title_fieldID;

getSuggestedStickerSetName::getSuggestedStickerSetName()
  : title_()
{}

getSuggestedStickerSetName::getSuggestedStickerSetName(string const &title_)
  : title_(title_)
{}

const std::int32_t getSuggestedStickerSetName::ID;

object_ptr<getSuggestedStickerSetName> getSuggestedStickerSetName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSuggestedStickerSetName> res = make_object<getSuggestedStickerSetName>();
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void getSuggestedStickerSetName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSuggestedStickerSetName");
    s.store_field("title", title_);
    s.store_class_end();
  }
}

getSuggestedStickerSetName::ReturnType getSuggestedStickerSetName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getSuggestedStickerSetName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSuggestedStickerSetName").c_str());
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getSuitableDiscussionChats::Class;

getSuitableDiscussionChats::getSuitableDiscussionChats() {
}

const std::int32_t getSuitableDiscussionChats::ID;

object_ptr<getSuitableDiscussionChats> getSuitableDiscussionChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSuitableDiscussionChats>();
}

void getSuitableDiscussionChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSuitableDiscussionChats");
    s.store_class_end();
  }
}

getSuitableDiscussionChats::ReturnType getSuitableDiscussionChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getSuitableDiscussionChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSuitableDiscussionChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSuitablePersonalChats::Class;

getSuitablePersonalChats::getSuitablePersonalChats() {
}

const std::int32_t getSuitablePersonalChats::ID;

object_ptr<getSuitablePersonalChats> getSuitablePersonalChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSuitablePersonalChats>();
}

void getSuitablePersonalChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSuitablePersonalChats");
    s.store_class_end();
  }
}

getSuitablePersonalChats::ReturnType getSuitablePersonalChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getSuitablePersonalChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSuitablePersonalChats").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSupergroup::Class;
jfieldID getSupergroup::supergroup_id_fieldID;

getSupergroup::getSupergroup()
  : supergroup_id_()
{}

getSupergroup::getSupergroup(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t getSupergroup::ID;

object_ptr<getSupergroup> getSupergroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSupergroup> res = make_object<getSupergroup>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return res;
}

void getSupergroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSupergroup");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

getSupergroup::ReturnType getSupergroup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return supergroup::fetch(env, p);
}

void getSupergroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSupergroup").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getSupergroupFullInfo::Class;
jfieldID getSupergroupFullInfo::supergroup_id_fieldID;

getSupergroupFullInfo::getSupergroupFullInfo()
  : supergroup_id_()
{}

getSupergroupFullInfo::getSupergroupFullInfo(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t getSupergroupFullInfo::ID;

object_ptr<getSupergroupFullInfo> getSupergroupFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSupergroupFullInfo> res = make_object<getSupergroupFullInfo>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return res;
}

void getSupergroupFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSupergroupFullInfo");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

getSupergroupFullInfo::ReturnType getSupergroupFullInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return supergroupFullInfo::fetch(env, p);
}

void getSupergroupFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSupergroupFullInfo").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getSupergroupMembers::Class;
jfieldID getSupergroupMembers::supergroup_id_fieldID;
jfieldID getSupergroupMembers::filter_fieldID;
jfieldID getSupergroupMembers::offset_fieldID;
jfieldID getSupergroupMembers::limit_fieldID;

getSupergroupMembers::getSupergroupMembers()
  : supergroup_id_()
  , filter_()
  , offset_()
  , limit_()
{}

getSupergroupMembers::getSupergroupMembers(int53 supergroup_id_, object_ptr<SupergroupMembersFilter> &&filter_, int32 offset_, int32 limit_)
  : supergroup_id_(supergroup_id_)
  , filter_(std::move(filter_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getSupergroupMembers::ID;

object_ptr<getSupergroupMembers> getSupergroupMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getSupergroupMembers> res = make_object<getSupergroupMembers>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->filter_ = jni::fetch_tl_object<SupergroupMembersFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getSupergroupMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSupergroupMembers");
    s.store_field("supergroupId", supergroup_id_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getSupergroupMembers::ReturnType getSupergroupMembers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatMembers::fetch(env, p);
}

void getSupergroupMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSupergroupMembers").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SupergroupMembersFilter;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getSupportName::Class;

getSupportName::getSupportName() {
}

const std::int32_t getSupportName::ID;

object_ptr<getSupportName> getSupportName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSupportName>();
}

void getSupportName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSupportName");
    s.store_class_end();
  }
}

getSupportName::ReturnType getSupportName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getSupportName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSupportName").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getSupportUser::Class;

getSupportUser::getSupportUser() {
}

const std::int32_t getSupportUser::ID;

object_ptr<getSupportUser> getSupportUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getSupportUser>();
}

void getSupportUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetSupportUser");
    s.store_class_end();
  }
}

getSupportUser::ReturnType getSupportUser::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return user::fetch(env, p);
}

void getSupportUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetSupportUser").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getTemporaryPasswordState::Class;

getTemporaryPasswordState::getTemporaryPasswordState() {
}

const std::int32_t getTemporaryPasswordState::ID;

object_ptr<getTemporaryPasswordState> getTemporaryPasswordState::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getTemporaryPasswordState>();
}

void getTemporaryPasswordState::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetTemporaryPasswordState");
    s.store_class_end();
  }
}

getTemporaryPasswordState::ReturnType getTemporaryPasswordState::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return temporaryPasswordState::fetch(env, p);
}

void getTemporaryPasswordState::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetTemporaryPasswordState").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getTextEntities::Class;
jfieldID getTextEntities::text_fieldID;

getTextEntities::getTextEntities()
  : text_()
{}

getTextEntities::getTextEntities(string const &text_)
  : text_(text_)
{}

const std::int32_t getTextEntities::ID;

object_ptr<getTextEntities> getTextEntities::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getTextEntities> res = make_object<getTextEntities>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void getTextEntities::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetTextEntities");
    s.store_field("text", text_);
    s.store_class_end();
  }
}

getTextEntities::ReturnType getTextEntities::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return textEntities::fetch(env, p);
}

void getTextEntities::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetTextEntities").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getThemeParametersJsonString::Class;
jfieldID getThemeParametersJsonString::theme_fieldID;

getThemeParametersJsonString::getThemeParametersJsonString()
  : theme_()
{}

getThemeParametersJsonString::getThemeParametersJsonString(object_ptr<themeParameters> &&theme_)
  : theme_(std::move(theme_))
{}

const std::int32_t getThemeParametersJsonString::ID;

object_ptr<getThemeParametersJsonString> getThemeParametersJsonString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getThemeParametersJsonString> res = make_object<getThemeParametersJsonString>();
  res->theme_ = jni::fetch_tl_object<themeParameters>(env, jni::fetch_object(env, p, res->theme_fieldID));
  return res;
}

void getThemeParametersJsonString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetThemeParametersJsonString");
    s.store_object_field("theme", static_cast<const BaseObject *>(theme_.get()));
    s.store_class_end();
  }
}

getThemeParametersJsonString::ReturnType getThemeParametersJsonString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void getThemeParametersJsonString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetThemeParametersJsonString").c_str());
    theme_fieldID = jni::get_field_id(env, Class, "theme", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ThemeParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getThemedChatEmojiStatuses::Class;

getThemedChatEmojiStatuses::getThemedChatEmojiStatuses() {
}

const std::int32_t getThemedChatEmojiStatuses::ID;

object_ptr<getThemedChatEmojiStatuses> getThemedChatEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getThemedChatEmojiStatuses>();
}

void getThemedChatEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetThemedChatEmojiStatuses");
    s.store_class_end();
  }
}

getThemedChatEmojiStatuses::ReturnType getThemedChatEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatusCustomEmojis::fetch(env, p);
}

void getThemedChatEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetThemedChatEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getThemedEmojiStatuses::Class;

getThemedEmojiStatuses::getThemedEmojiStatuses() {
}

const std::int32_t getThemedEmojiStatuses::ID;

object_ptr<getThemedEmojiStatuses> getThemedEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getThemedEmojiStatuses>();
}

void getThemedEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetThemedEmojiStatuses");
    s.store_class_end();
  }
}

getThemedEmojiStatuses::ReturnType getThemedEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatusCustomEmojis::fetch(env, p);
}

void getThemedEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetThemedEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getTimeZones::Class;

getTimeZones::getTimeZones() {
}

const std::int32_t getTimeZones::ID;

object_ptr<getTimeZones> getTimeZones::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getTimeZones>();
}

void getTimeZones::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetTimeZones");
    s.store_class_end();
  }
}

getTimeZones::ReturnType getTimeZones::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return timeZones::fetch(env, p);
}

void getTimeZones::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetTimeZones").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getTopChats::Class;
jfieldID getTopChats::category_fieldID;
jfieldID getTopChats::limit_fieldID;

getTopChats::getTopChats()
  : category_()
  , limit_()
{}

getTopChats::getTopChats(object_ptr<TopChatCategory> &&category_, int32 limit_)
  : category_(std::move(category_))
  , limit_(limit_)
{}

const std::int32_t getTopChats::ID;

object_ptr<getTopChats> getTopChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getTopChats> res = make_object<getTopChats>();
  res->category_ = jni::fetch_tl_object<TopChatCategory>(env, jni::fetch_object(env, p, res->category_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getTopChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetTopChats");
    s.store_object_field("category", static_cast<const BaseObject *>(category_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getTopChats::ReturnType getTopChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void getTopChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetTopChats").c_str());
    category_fieldID = jni::get_field_id(env, Class, "category", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TopChatCategory;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getTrendingStickerSets::Class;
jfieldID getTrendingStickerSets::sticker_type_fieldID;
jfieldID getTrendingStickerSets::offset_fieldID;
jfieldID getTrendingStickerSets::limit_fieldID;

getTrendingStickerSets::getTrendingStickerSets()
  : sticker_type_()
  , offset_()
  , limit_()
{}

getTrendingStickerSets::getTrendingStickerSets(object_ptr<StickerType> &&sticker_type_, int32 offset_, int32 limit_)
  : sticker_type_(std::move(sticker_type_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getTrendingStickerSets::ID;

object_ptr<getTrendingStickerSets> getTrendingStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getTrendingStickerSets> res = make_object<getTrendingStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getTrendingStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetTrendingStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getTrendingStickerSets::ReturnType getTrendingStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return trendingStickerSets::fetch(env, p);
}

void getTrendingStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetTrendingStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getUpgradedGift::Class;
jfieldID getUpgradedGift::name_fieldID;

getUpgradedGift::getUpgradedGift()
  : name_()
{}

getUpgradedGift::getUpgradedGift(string const &name_)
  : name_(name_)
{}

const std::int32_t getUpgradedGift::ID;

object_ptr<getUpgradedGift> getUpgradedGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUpgradedGift> res = make_object<getUpgradedGift>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void getUpgradedGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUpgradedGift");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

getUpgradedGift::ReturnType getUpgradedGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return upgradedGift::fetch(env, p);
}

void getUpgradedGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUpgradedGift").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getUpgradedGiftEmojiStatuses::Class;

getUpgradedGiftEmojiStatuses::getUpgradedGiftEmojiStatuses() {
}

const std::int32_t getUpgradedGiftEmojiStatuses::ID;

object_ptr<getUpgradedGiftEmojiStatuses> getUpgradedGiftEmojiStatuses::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getUpgradedGiftEmojiStatuses>();
}

void getUpgradedGiftEmojiStatuses::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUpgradedGiftEmojiStatuses");
    s.store_class_end();
  }
}

getUpgradedGiftEmojiStatuses::ReturnType getUpgradedGiftEmojiStatuses::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiStatuses::fetch(env, p);
}

void getUpgradedGiftEmojiStatuses::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUpgradedGiftEmojiStatuses").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getUpgradedGiftWithdrawalUrl::Class;
jfieldID getUpgradedGiftWithdrawalUrl::received_gift_id_fieldID;
jfieldID getUpgradedGiftWithdrawalUrl::password_fieldID;

getUpgradedGiftWithdrawalUrl::getUpgradedGiftWithdrawalUrl()
  : received_gift_id_()
  , password_()
{}

getUpgradedGiftWithdrawalUrl::getUpgradedGiftWithdrawalUrl(string const &received_gift_id_, string const &password_)
  : received_gift_id_(received_gift_id_)
  , password_(password_)
{}

const std::int32_t getUpgradedGiftWithdrawalUrl::ID;

object_ptr<getUpgradedGiftWithdrawalUrl> getUpgradedGiftWithdrawalUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUpgradedGiftWithdrawalUrl> res = make_object<getUpgradedGiftWithdrawalUrl>();
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void getUpgradedGiftWithdrawalUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUpgradedGiftWithdrawalUrl");
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

getUpgradedGiftWithdrawalUrl::ReturnType getUpgradedGiftWithdrawalUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getUpgradedGiftWithdrawalUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUpgradedGiftWithdrawalUrl").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass getUser::Class;
jfieldID getUser::user_id_fieldID;

getUser::getUser()
  : user_id_()
{}

getUser::getUser(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getUser::ID;

object_ptr<getUser> getUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUser> res = make_object<getUser>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUser");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getUser::ReturnType getUser::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return user::fetch(env, p);
}

void getUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUser").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getUserChatBoosts::Class;
jfieldID getUserChatBoosts::chat_id_fieldID;
jfieldID getUserChatBoosts::user_id_fieldID;

getUserChatBoosts::getUserChatBoosts()
  : chat_id_()
  , user_id_()
{}

getUserChatBoosts::getUserChatBoosts(int53 chat_id_, int53 user_id_)
  : chat_id_(chat_id_)
  , user_id_(user_id_)
{}

const std::int32_t getUserChatBoosts::ID;

object_ptr<getUserChatBoosts> getUserChatBoosts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUserChatBoosts> res = make_object<getUserChatBoosts>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getUserChatBoosts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserChatBoosts");
    s.store_field("chatId", chat_id_);
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getUserChatBoosts::ReturnType getUserChatBoosts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundChatBoosts::fetch(env, p);
}

void getUserChatBoosts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserChatBoosts").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getUserFullInfo::Class;
jfieldID getUserFullInfo::user_id_fieldID;

getUserFullInfo::getUserFullInfo()
  : user_id_()
{}

getUserFullInfo::getUserFullInfo(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getUserFullInfo::ID;

object_ptr<getUserFullInfo> getUserFullInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUserFullInfo> res = make_object<getUserFullInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getUserFullInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserFullInfo");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getUserFullInfo::ReturnType getUserFullInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return userFullInfo::fetch(env, p);
}

void getUserFullInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserFullInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getUserLink::Class;

getUserLink::getUserLink() {
}

const std::int32_t getUserLink::ID;

object_ptr<getUserLink> getUserLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<getUserLink>();
}

void getUserLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserLink");
    s.store_class_end();
  }
}

getUserLink::ReturnType getUserLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return userLink::fetch(env, p);
}

void getUserLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserLink").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getUserPrivacySettingRules::Class;
jfieldID getUserPrivacySettingRules::setting_fieldID;

getUserPrivacySettingRules::getUserPrivacySettingRules()
  : setting_()
{}

getUserPrivacySettingRules::getUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_)
  : setting_(std::move(setting_))
{}

const std::int32_t getUserPrivacySettingRules::ID;

object_ptr<getUserPrivacySettingRules> getUserPrivacySettingRules::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUserPrivacySettingRules> res = make_object<getUserPrivacySettingRules>();
  res->setting_ = jni::fetch_tl_object<UserPrivacySetting>(env, jni::fetch_object(env, p, res->setting_fieldID));
  return res;
}

void getUserPrivacySettingRules::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserPrivacySettingRules");
    s.store_object_field("setting", static_cast<const BaseObject *>(setting_.get()));
    s.store_class_end();
  }
}

getUserPrivacySettingRules::ReturnType getUserPrivacySettingRules::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return userPrivacySettingRules::fetch(env, p);
}

void getUserPrivacySettingRules::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserPrivacySettingRules").c_str());
    setting_fieldID = jni::get_field_id(env, Class, "setting", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserPrivacySetting;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getUserProfilePhotos::Class;
jfieldID getUserProfilePhotos::user_id_fieldID;
jfieldID getUserProfilePhotos::offset_fieldID;
jfieldID getUserProfilePhotos::limit_fieldID;

getUserProfilePhotos::getUserProfilePhotos()
  : user_id_()
  , offset_()
  , limit_()
{}

getUserProfilePhotos::getUserProfilePhotos(int53 user_id_, int32 offset_, int32 limit_)
  : user_id_(user_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t getUserProfilePhotos::ID;

object_ptr<getUserProfilePhotos> getUserProfilePhotos::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUserProfilePhotos> res = make_object<getUserProfilePhotos>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void getUserProfilePhotos::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserProfilePhotos");
    s.store_field("userId", user_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

getUserProfilePhotos::ReturnType getUserProfilePhotos::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatPhotos::fetch(env, p);
}

void getUserProfilePhotos::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserProfilePhotos").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getUserSupportInfo::Class;
jfieldID getUserSupportInfo::user_id_fieldID;

getUserSupportInfo::getUserSupportInfo()
  : user_id_()
{}

getUserSupportInfo::getUserSupportInfo(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t getUserSupportInfo::ID;

object_ptr<getUserSupportInfo> getUserSupportInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getUserSupportInfo> res = make_object<getUserSupportInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void getUserSupportInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetUserSupportInfo");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

getUserSupportInfo::ReturnType getUserSupportInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return userSupportInfo::fetch(env, p);
}

void getUserSupportInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetUserSupportInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getVideoChatAvailableParticipants::Class;
jfieldID getVideoChatAvailableParticipants::chat_id_fieldID;

getVideoChatAvailableParticipants::getVideoChatAvailableParticipants()
  : chat_id_()
{}

getVideoChatAvailableParticipants::getVideoChatAvailableParticipants(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getVideoChatAvailableParticipants::ID;

object_ptr<getVideoChatAvailableParticipants> getVideoChatAvailableParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getVideoChatAvailableParticipants> res = make_object<getVideoChatAvailableParticipants>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getVideoChatAvailableParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetVideoChatAvailableParticipants");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getVideoChatAvailableParticipants::ReturnType getVideoChatAvailableParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messageSenders::fetch(env, p);
}

void getVideoChatAvailableParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetVideoChatAvailableParticipants").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getVideoChatInviteLink::Class;
jfieldID getVideoChatInviteLink::group_call_id_fieldID;
jfieldID getVideoChatInviteLink::can_self_unmute_fieldID;

getVideoChatInviteLink::getVideoChatInviteLink()
  : group_call_id_()
  , can_self_unmute_()
{}

getVideoChatInviteLink::getVideoChatInviteLink(int32 group_call_id_, bool can_self_unmute_)
  : group_call_id_(group_call_id_)
  , can_self_unmute_(can_self_unmute_)
{}

const std::int32_t getVideoChatInviteLink::ID;

object_ptr<getVideoChatInviteLink> getVideoChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getVideoChatInviteLink> res = make_object<getVideoChatInviteLink>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->can_self_unmute_ = (env->GetBooleanField(p, res->can_self_unmute_fieldID) != 0);
  return res;
}

void getVideoChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetVideoChatInviteLink");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("canSelfUnmute", can_self_unmute_);
    s.store_class_end();
  }
}

getVideoChatInviteLink::ReturnType getVideoChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getVideoChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetVideoChatInviteLink").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    can_self_unmute_fieldID = jni::get_field_id(env, Class, "canSelfUnmute", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass getVideoChatRtmpUrl::Class;
jfieldID getVideoChatRtmpUrl::chat_id_fieldID;

getVideoChatRtmpUrl::getVideoChatRtmpUrl()
  : chat_id_()
{}

getVideoChatRtmpUrl::getVideoChatRtmpUrl(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t getVideoChatRtmpUrl::ID;

object_ptr<getVideoChatRtmpUrl> getVideoChatRtmpUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getVideoChatRtmpUrl> res = make_object<getVideoChatRtmpUrl>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void getVideoChatRtmpUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetVideoChatRtmpUrl");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

getVideoChatRtmpUrl::ReturnType getVideoChatRtmpUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return rtmpUrl::fetch(env, p);
}

void getVideoChatRtmpUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetVideoChatRtmpUrl").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getVideoChatStreamSegment::Class;
jfieldID getVideoChatStreamSegment::group_call_id_fieldID;
jfieldID getVideoChatStreamSegment::time_offset_fieldID;
jfieldID getVideoChatStreamSegment::scale_fieldID;
jfieldID getVideoChatStreamSegment::channel_id_fieldID;
jfieldID getVideoChatStreamSegment::video_quality_fieldID;

getVideoChatStreamSegment::getVideoChatStreamSegment()
  : group_call_id_()
  , time_offset_()
  , scale_()
  , channel_id_()
  , video_quality_()
{}

getVideoChatStreamSegment::getVideoChatStreamSegment(int32 group_call_id_, int53 time_offset_, int32 scale_, int32 channel_id_, object_ptr<GroupCallVideoQuality> &&video_quality_)
  : group_call_id_(group_call_id_)
  , time_offset_(time_offset_)
  , scale_(scale_)
  , channel_id_(channel_id_)
  , video_quality_(std::move(video_quality_))
{}

const std::int32_t getVideoChatStreamSegment::ID;

object_ptr<getVideoChatStreamSegment> getVideoChatStreamSegment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getVideoChatStreamSegment> res = make_object<getVideoChatStreamSegment>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->time_offset_ = env->GetLongField(p, res->time_offset_fieldID);
  res->scale_ = env->GetIntField(p, res->scale_fieldID);
  res->channel_id_ = env->GetIntField(p, res->channel_id_fieldID);
  res->video_quality_ = jni::fetch_tl_object<GroupCallVideoQuality>(env, jni::fetch_object(env, p, res->video_quality_fieldID));
  return res;
}

void getVideoChatStreamSegment::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetVideoChatStreamSegment");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("timeOffset", time_offset_);
    s.store_field("scale", scale_);
    s.store_field("channelId", channel_id_);
    s.store_object_field("videoQuality", static_cast<const BaseObject *>(video_quality_.get()));
    s.store_class_end();
  }
}

getVideoChatStreamSegment::ReturnType getVideoChatStreamSegment::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return data::fetch(env, p);
}

void getVideoChatStreamSegment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetVideoChatStreamSegment").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    time_offset_fieldID = jni::get_field_id(env, Class, "timeOffset", "J");
    scale_fieldID = jni::get_field_id(env, Class, "scale", "I");
    channel_id_fieldID = jni::get_field_id(env, Class, "channelId", "I");
    video_quality_fieldID = jni::get_field_id(env, Class, "videoQuality", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallVideoQuality;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getVideoChatStreams::Class;
jfieldID getVideoChatStreams::group_call_id_fieldID;

getVideoChatStreams::getVideoChatStreams()
  : group_call_id_()
{}

getVideoChatStreams::getVideoChatStreams(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t getVideoChatStreams::ID;

object_ptr<getVideoChatStreams> getVideoChatStreams::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getVideoChatStreams> res = make_object<getVideoChatStreams>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void getVideoChatStreams::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetVideoChatStreams");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

getVideoChatStreams::ReturnType getVideoChatStreams::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return videoChatStreams::fetch(env, p);
}

void getVideoChatStreams::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetVideoChatStreams").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass getWebAppLinkUrl::Class;
jfieldID getWebAppLinkUrl::chat_id_fieldID;
jfieldID getWebAppLinkUrl::bot_user_id_fieldID;
jfieldID getWebAppLinkUrl::web_app_short_name_fieldID;
jfieldID getWebAppLinkUrl::start_parameter_fieldID;
jfieldID getWebAppLinkUrl::allow_write_access_fieldID;
jfieldID getWebAppLinkUrl::parameters_fieldID;

getWebAppLinkUrl::getWebAppLinkUrl()
  : chat_id_()
  , bot_user_id_()
  , web_app_short_name_()
  , start_parameter_()
  , allow_write_access_()
  , parameters_()
{}

getWebAppLinkUrl::getWebAppLinkUrl(int53 chat_id_, int53 bot_user_id_, string const &web_app_short_name_, string const &start_parameter_, bool allow_write_access_, object_ptr<webAppOpenParameters> &&parameters_)
  : chat_id_(chat_id_)
  , bot_user_id_(bot_user_id_)
  , web_app_short_name_(web_app_short_name_)
  , start_parameter_(start_parameter_)
  , allow_write_access_(allow_write_access_)
  , parameters_(std::move(parameters_))
{}

const std::int32_t getWebAppLinkUrl::ID;

object_ptr<getWebAppLinkUrl> getWebAppLinkUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getWebAppLinkUrl> res = make_object<getWebAppLinkUrl>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->web_app_short_name_ = jni::fetch_string(env, p, res->web_app_short_name_fieldID);
  res->start_parameter_ = jni::fetch_string(env, p, res->start_parameter_fieldID);
  res->allow_write_access_ = (env->GetBooleanField(p, res->allow_write_access_fieldID) != 0);
  res->parameters_ = jni::fetch_tl_object<webAppOpenParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  return res;
}

void getWebAppLinkUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetWebAppLinkUrl");
    s.store_field("chatId", chat_id_);
    s.store_field("botUserId", bot_user_id_);
    s.store_field("webAppShortName", web_app_short_name_);
    s.store_field("startParameter", start_parameter_);
    s.store_field("allowWriteAccess", allow_write_access_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_class_end();
  }
}

getWebAppLinkUrl::ReturnType getWebAppLinkUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getWebAppLinkUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetWebAppLinkUrl").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    web_app_short_name_fieldID = jni::get_field_id(env, Class, "webAppShortName", "Ljava/lang/String;");
    start_parameter_fieldID = jni::get_field_id(env, Class, "startParameter", "Ljava/lang/String;");
    allow_write_access_fieldID = jni::get_field_id(env, Class, "allowWriteAccess", "Z");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getWebAppPlaceholder::Class;
jfieldID getWebAppPlaceholder::bot_user_id_fieldID;

getWebAppPlaceholder::getWebAppPlaceholder()
  : bot_user_id_()
{}

getWebAppPlaceholder::getWebAppPlaceholder(int53 bot_user_id_)
  : bot_user_id_(bot_user_id_)
{}

const std::int32_t getWebAppPlaceholder::ID;

object_ptr<getWebAppPlaceholder> getWebAppPlaceholder::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getWebAppPlaceholder> res = make_object<getWebAppPlaceholder>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  return res;
}

void getWebAppPlaceholder::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetWebAppPlaceholder");
    s.store_field("botUserId", bot_user_id_);
    s.store_class_end();
  }
}

getWebAppPlaceholder::ReturnType getWebAppPlaceholder::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return outline::fetch(env, p);
}

void getWebAppPlaceholder::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetWebAppPlaceholder").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass getWebAppUrl::Class;
jfieldID getWebAppUrl::bot_user_id_fieldID;
jfieldID getWebAppUrl::url_fieldID;
jfieldID getWebAppUrl::parameters_fieldID;

getWebAppUrl::getWebAppUrl()
  : bot_user_id_()
  , url_()
  , parameters_()
{}

getWebAppUrl::getWebAppUrl(int53 bot_user_id_, string const &url_, object_ptr<webAppOpenParameters> &&parameters_)
  : bot_user_id_(bot_user_id_)
  , url_(url_)
  , parameters_(std::move(parameters_))
{}

const std::int32_t getWebAppUrl::ID;

object_ptr<getWebAppUrl> getWebAppUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getWebAppUrl> res = make_object<getWebAppUrl>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->parameters_ = jni::fetch_tl_object<webAppOpenParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  return res;
}

void getWebAppUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetWebAppUrl");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("url", url_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_class_end();
  }
}

getWebAppUrl::ReturnType getWebAppUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return httpUrl::fetch(env, p);
}

void getWebAppUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetWebAppUrl").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass getWebPageInstantView::Class;
jfieldID getWebPageInstantView::url_fieldID;
jfieldID getWebPageInstantView::only_local_fieldID;

getWebPageInstantView::getWebPageInstantView()
  : url_()
  , only_local_()
{}

getWebPageInstantView::getWebPageInstantView(string const &url_, bool only_local_)
  : url_(url_)
  , only_local_(only_local_)
{}

const std::int32_t getWebPageInstantView::ID;

object_ptr<getWebPageInstantView> getWebPageInstantView::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<getWebPageInstantView> res = make_object<getWebPageInstantView>();
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void getWebPageInstantView::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GetWebPageInstantView");
    s.store_field("url", url_);
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

getWebPageInstantView::ReturnType getWebPageInstantView::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return webPageInstantView::fetch(env, p);
}

void getWebPageInstantView::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GetWebPageInstantView").c_str());
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass giftPremiumWithStars::Class;
jfieldID giftPremiumWithStars::user_id_fieldID;
jfieldID giftPremiumWithStars::star_count_fieldID;
jfieldID giftPremiumWithStars::month_count_fieldID;
jfieldID giftPremiumWithStars::text_fieldID;

giftPremiumWithStars::giftPremiumWithStars()
  : user_id_()
  , star_count_()
  , month_count_()
  , text_()
{}

giftPremiumWithStars::giftPremiumWithStars(int53 user_id_, int53 star_count_, int32 month_count_, object_ptr<formattedText> &&text_)
  : user_id_(user_id_)
  , star_count_(star_count_)
  , month_count_(month_count_)
  , text_(std::move(text_))
{}

const std::int32_t giftPremiumWithStars::ID;

object_ptr<giftPremiumWithStars> giftPremiumWithStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<giftPremiumWithStars> res = make_object<giftPremiumWithStars>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  res->month_count_ = env->GetIntField(p, res->month_count_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return res;
}

void giftPremiumWithStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "GiftPremiumWithStars");
    s.store_field("userId", user_id_);
    s.store_field("starCount", star_count_);
    s.store_field("monthCount", month_count_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

giftPremiumWithStars::ReturnType giftPremiumWithStars::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void giftPremiumWithStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$GiftPremiumWithStars").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    month_count_fieldID = jni::get_field_id(env, Class, "monthCount", "I");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass hideContactCloseBirthdays::Class;

hideContactCloseBirthdays::hideContactCloseBirthdays() {
}

const std::int32_t hideContactCloseBirthdays::ID;

object_ptr<hideContactCloseBirthdays> hideContactCloseBirthdays::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<hideContactCloseBirthdays>();
}

void hideContactCloseBirthdays::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "HideContactCloseBirthdays");
    s.store_class_end();
  }
}

hideContactCloseBirthdays::ReturnType hideContactCloseBirthdays::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void hideContactCloseBirthdays::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$HideContactCloseBirthdays").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass hideSuggestedAction::Class;
jfieldID hideSuggestedAction::action_fieldID;

hideSuggestedAction::hideSuggestedAction()
  : action_()
{}

hideSuggestedAction::hideSuggestedAction(object_ptr<SuggestedAction> &&action_)
  : action_(std::move(action_))
{}

const std::int32_t hideSuggestedAction::ID;

object_ptr<hideSuggestedAction> hideSuggestedAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<hideSuggestedAction> res = make_object<hideSuggestedAction>();
  res->action_ = jni::fetch_tl_object<SuggestedAction>(env, jni::fetch_object(env, p, res->action_fieldID));
  return res;
}

void hideSuggestedAction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "HideSuggestedAction");
    s.store_object_field("action", static_cast<const BaseObject *>(action_.get()));
    s.store_class_end();
  }
}

hideSuggestedAction::ReturnType hideSuggestedAction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void hideSuggestedAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$HideSuggestedAction").c_str());
    action_fieldID = jni::get_field_id(env, Class, "action", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SuggestedAction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass importContacts::Class;
jfieldID importContacts::contacts_fieldID;

importContacts::importContacts()
  : contacts_()
{}

importContacts::importContacts(array<object_ptr<contact>> &&contacts_)
  : contacts_(std::move(contacts_))
{}

const std::int32_t importContacts::ID;

object_ptr<importContacts> importContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<importContacts> res = make_object<importContacts>();
  res->contacts_ = jni::FetchVector<contact>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->contacts_fieldID));
  return res;
}

void importContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ImportContacts");
    { s.store_vector_begin("contacts", contacts_.size()); for (const auto &_value : contacts_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

importContacts::ReturnType importContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return importedContacts::fetch(env, p);
}

void importContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ImportContacts").c_str());
    contacts_fieldID = jni::get_field_id(env, Class, "contacts", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$Contact;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass importMessages::Class;
jfieldID importMessages::chat_id_fieldID;
jfieldID importMessages::message_file_fieldID;
jfieldID importMessages::attached_files_fieldID;

importMessages::importMessages()
  : chat_id_()
  , message_file_()
  , attached_files_()
{}

importMessages::importMessages(int53 chat_id_, object_ptr<InputFile> &&message_file_, array<object_ptr<InputFile>> &&attached_files_)
  : chat_id_(chat_id_)
  , message_file_(std::move(message_file_))
  , attached_files_(std::move(attached_files_))
{}

const std::int32_t importMessages::ID;

object_ptr<importMessages> importMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<importMessages> res = make_object<importMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_file_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->message_file_fieldID));
  res->attached_files_ = jni::FetchVector<InputFile>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->attached_files_fieldID));
  return res;
}

void importMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ImportMessages");
    s.store_field("chatId", chat_id_);
    s.store_object_field("messageFile", static_cast<const BaseObject *>(message_file_.get()));
    { s.store_vector_begin("attachedFiles", attached_files_.size()); for (const auto &_value : attached_files_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

importMessages::ReturnType importMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void importMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ImportMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_file_fieldID = jni::get_field_id(env, Class, "messageFile", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    attached_files_fieldID = jni::get_field_id(env, Class, "attachedFiles", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass inviteGroupCallParticipant::Class;
jfieldID inviteGroupCallParticipant::group_call_id_fieldID;
jfieldID inviteGroupCallParticipant::user_id_fieldID;
jfieldID inviteGroupCallParticipant::is_video_fieldID;

inviteGroupCallParticipant::inviteGroupCallParticipant()
  : group_call_id_()
  , user_id_()
  , is_video_()
{}

inviteGroupCallParticipant::inviteGroupCallParticipant(int32 group_call_id_, int53 user_id_, bool is_video_)
  : group_call_id_(group_call_id_)
  , user_id_(user_id_)
  , is_video_(is_video_)
{}

const std::int32_t inviteGroupCallParticipant::ID;

object_ptr<inviteGroupCallParticipant> inviteGroupCallParticipant::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inviteGroupCallParticipant> res = make_object<inviteGroupCallParticipant>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->is_video_ = (env->GetBooleanField(p, res->is_video_fieldID) != 0);
  return res;
}

void inviteGroupCallParticipant::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteGroupCallParticipant");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("userId", user_id_);
    s.store_field("isVideo", is_video_);
    s.store_class_end();
  }
}

inviteGroupCallParticipant::ReturnType inviteGroupCallParticipant::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return InviteGroupCallParticipantResult::fetch(env, p);
}

void inviteGroupCallParticipant::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteGroupCallParticipant").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    is_video_fieldID = jni::get_field_id(env, Class, "isVideo", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass inviteVideoChatParticipants::Class;
jfieldID inviteVideoChatParticipants::group_call_id_fieldID;
jfieldID inviteVideoChatParticipants::user_ids_fieldID;

inviteVideoChatParticipants::inviteVideoChatParticipants()
  : group_call_id_()
  , user_ids_()
{}

inviteVideoChatParticipants::inviteVideoChatParticipants(int32 group_call_id_, array<int53> &&user_ids_)
  : group_call_id_(group_call_id_)
  , user_ids_(std::move(user_ids_))
{}

const std::int32_t inviteVideoChatParticipants::ID;

object_ptr<inviteVideoChatParticipants> inviteVideoChatParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<inviteVideoChatParticipants> res = make_object<inviteVideoChatParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void inviteVideoChatParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "InviteVideoChatParticipants");
    s.store_field("groupCallId", group_call_id_);
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

inviteVideoChatParticipants::ReturnType inviteVideoChatParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void inviteVideoChatParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$InviteVideoChatParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass joinChat::Class;
jfieldID joinChat::chat_id_fieldID;

joinChat::joinChat()
  : chat_id_()
{}

joinChat::joinChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t joinChat::ID;

object_ptr<joinChat> joinChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<joinChat> res = make_object<joinChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void joinChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JoinChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

joinChat::ReturnType joinChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void joinChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JoinChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass joinChatByInviteLink::Class;
jfieldID joinChatByInviteLink::invite_link_fieldID;

joinChatByInviteLink::joinChatByInviteLink()
  : invite_link_()
{}

joinChatByInviteLink::joinChatByInviteLink(string const &invite_link_)
  : invite_link_(invite_link_)
{}

const std::int32_t joinChatByInviteLink::ID;

object_ptr<joinChatByInviteLink> joinChatByInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<joinChatByInviteLink> res = make_object<joinChatByInviteLink>();
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void joinChatByInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JoinChatByInviteLink");
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

joinChatByInviteLink::ReturnType joinChatByInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void joinChatByInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JoinChatByInviteLink").c_str());
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass joinGroupCall::Class;
jfieldID joinGroupCall::input_group_call_fieldID;
jfieldID joinGroupCall::join_parameters_fieldID;

joinGroupCall::joinGroupCall()
  : input_group_call_()
  , join_parameters_()
{}

joinGroupCall::joinGroupCall(object_ptr<InputGroupCall> &&input_group_call_, object_ptr<groupCallJoinParameters> &&join_parameters_)
  : input_group_call_(std::move(input_group_call_))
  , join_parameters_(std::move(join_parameters_))
{}

const std::int32_t joinGroupCall::ID;

object_ptr<joinGroupCall> joinGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<joinGroupCall> res = make_object<joinGroupCall>();
  res->input_group_call_ = jni::fetch_tl_object<InputGroupCall>(env, jni::fetch_object(env, p, res->input_group_call_fieldID));
  res->join_parameters_ = jni::fetch_tl_object<groupCallJoinParameters>(env, jni::fetch_object(env, p, res->join_parameters_fieldID));
  return res;
}

void joinGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JoinGroupCall");
    s.store_object_field("inputGroupCall", static_cast<const BaseObject *>(input_group_call_.get()));
    s.store_object_field("joinParameters", static_cast<const BaseObject *>(join_parameters_.get()));
    s.store_class_end();
  }
}

joinGroupCall::ReturnType joinGroupCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return groupCallInfo::fetch(env, p);
}

void joinGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JoinGroupCall").c_str());
    input_group_call_fieldID = jni::get_field_id(env, Class, "inputGroupCall", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputGroupCall;").c_str());
    join_parameters_fieldID = jni::get_field_id(env, Class, "joinParameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallJoinParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass joinVideoChat::Class;
jfieldID joinVideoChat::group_call_id_fieldID;
jfieldID joinVideoChat::participant_id_fieldID;
jfieldID joinVideoChat::join_parameters_fieldID;
jfieldID joinVideoChat::invite_hash_fieldID;

joinVideoChat::joinVideoChat()
  : group_call_id_()
  , participant_id_()
  , join_parameters_()
  , invite_hash_()
{}

joinVideoChat::joinVideoChat(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, object_ptr<groupCallJoinParameters> &&join_parameters_, string const &invite_hash_)
  : group_call_id_(group_call_id_)
  , participant_id_(std::move(participant_id_))
  , join_parameters_(std::move(join_parameters_))
  , invite_hash_(invite_hash_)
{}

const std::int32_t joinVideoChat::ID;

object_ptr<joinVideoChat> joinVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<joinVideoChat> res = make_object<joinVideoChat>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->join_parameters_ = jni::fetch_tl_object<groupCallJoinParameters>(env, jni::fetch_object(env, p, res->join_parameters_fieldID));
  res->invite_hash_ = jni::fetch_string(env, p, res->invite_hash_fieldID);
  return res;
}

void joinVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "JoinVideoChat");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_object_field("joinParameters", static_cast<const BaseObject *>(join_parameters_.get()));
    s.store_field("inviteHash", invite_hash_);
    s.store_class_end();
  }
}

joinVideoChat::ReturnType joinVideoChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void joinVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$JoinVideoChat").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    join_parameters_fieldID = jni::get_field_id(env, Class, "joinParameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GroupCallJoinParameters;").c_str());
    invite_hash_fieldID = jni::get_field_id(env, Class, "inviteHash", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass launchPrepaidGiveaway::Class;
jfieldID launchPrepaidGiveaway::giveaway_id_fieldID;
jfieldID launchPrepaidGiveaway::parameters_fieldID;
jfieldID launchPrepaidGiveaway::winner_count_fieldID;
jfieldID launchPrepaidGiveaway::star_count_fieldID;

launchPrepaidGiveaway::launchPrepaidGiveaway()
  : giveaway_id_()
  , parameters_()
  , winner_count_()
  , star_count_()
{}

launchPrepaidGiveaway::launchPrepaidGiveaway(int64 giveaway_id_, object_ptr<giveawayParameters> &&parameters_, int32 winner_count_, int53 star_count_)
  : giveaway_id_(giveaway_id_)
  , parameters_(std::move(parameters_))
  , winner_count_(winner_count_)
  , star_count_(star_count_)
{}

const std::int32_t launchPrepaidGiveaway::ID;

object_ptr<launchPrepaidGiveaway> launchPrepaidGiveaway::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<launchPrepaidGiveaway> res = make_object<launchPrepaidGiveaway>();
  res->giveaway_id_ = env->GetLongField(p, res->giveaway_id_fieldID);
  res->parameters_ = jni::fetch_tl_object<giveawayParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  res->winner_count_ = env->GetIntField(p, res->winner_count_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void launchPrepaidGiveaway::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LaunchPrepaidGiveaway");
    s.store_field("giveawayId", giveaway_id_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_field("winnerCount", winner_count_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

launchPrepaidGiveaway::ReturnType launchPrepaidGiveaway::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void launchPrepaidGiveaway::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LaunchPrepaidGiveaway").c_str());
    giveaway_id_fieldID = jni::get_field_id(env, Class, "giveawayId", "J");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiveawayParameters;").c_str());
    winner_count_fieldID = jni::get_field_id(env, Class, "winnerCount", "I");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass leaveChat::Class;
jfieldID leaveChat::chat_id_fieldID;

leaveChat::leaveChat()
  : chat_id_()
{}

leaveChat::leaveChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t leaveChat::ID;

object_ptr<leaveChat> leaveChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<leaveChat> res = make_object<leaveChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void leaveChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LeaveChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

leaveChat::ReturnType leaveChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void leaveChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LeaveChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass leaveGroupCall::Class;
jfieldID leaveGroupCall::group_call_id_fieldID;

leaveGroupCall::leaveGroupCall()
  : group_call_id_()
{}

leaveGroupCall::leaveGroupCall(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t leaveGroupCall::ID;

object_ptr<leaveGroupCall> leaveGroupCall::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<leaveGroupCall> res = make_object<leaveGroupCall>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void leaveGroupCall::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LeaveGroupCall");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

leaveGroupCall::ReturnType leaveGroupCall::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void leaveGroupCall::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LeaveGroupCall").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass loadActiveStories::Class;
jfieldID loadActiveStories::story_list_fieldID;

loadActiveStories::loadActiveStories()
  : story_list_()
{}

loadActiveStories::loadActiveStories(object_ptr<StoryList> &&story_list_)
  : story_list_(std::move(story_list_))
{}

const std::int32_t loadActiveStories::ID;

object_ptr<loadActiveStories> loadActiveStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loadActiveStories> res = make_object<loadActiveStories>();
  res->story_list_ = jni::fetch_tl_object<StoryList>(env, jni::fetch_object(env, p, res->story_list_fieldID));
  return res;
}

void loadActiveStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadActiveStories");
    s.store_object_field("storyList", static_cast<const BaseObject *>(story_list_.get()));
    s.store_class_end();
  }
}

loadActiveStories::ReturnType loadActiveStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadActiveStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadActiveStories").c_str());
    story_list_fieldID = jni::get_field_id(env, Class, "storyList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass loadChats::Class;
jfieldID loadChats::chat_list_fieldID;
jfieldID loadChats::limit_fieldID;

loadChats::loadChats()
  : chat_list_()
  , limit_()
{}

loadChats::loadChats(object_ptr<ChatList> &&chat_list_, int32 limit_)
  : chat_list_(std::move(chat_list_))
  , limit_(limit_)
{}

const std::int32_t loadChats::ID;

object_ptr<loadChats> loadChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loadChats> res = make_object<loadChats>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void loadChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadChats");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

loadChats::ReturnType loadChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadChats").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass loadGroupCallParticipants::Class;
jfieldID loadGroupCallParticipants::group_call_id_fieldID;
jfieldID loadGroupCallParticipants::limit_fieldID;

loadGroupCallParticipants::loadGroupCallParticipants()
  : group_call_id_()
  , limit_()
{}

loadGroupCallParticipants::loadGroupCallParticipants(int32 group_call_id_, int32 limit_)
  : group_call_id_(group_call_id_)
  , limit_(limit_)
{}

const std::int32_t loadGroupCallParticipants::ID;

object_ptr<loadGroupCallParticipants> loadGroupCallParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loadGroupCallParticipants> res = make_object<loadGroupCallParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void loadGroupCallParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadGroupCallParticipants");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

loadGroupCallParticipants::ReturnType loadGroupCallParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadGroupCallParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadGroupCallParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass loadQuickReplyShortcutMessages::Class;
jfieldID loadQuickReplyShortcutMessages::shortcut_id_fieldID;

loadQuickReplyShortcutMessages::loadQuickReplyShortcutMessages()
  : shortcut_id_()
{}

loadQuickReplyShortcutMessages::loadQuickReplyShortcutMessages(int32 shortcut_id_)
  : shortcut_id_(shortcut_id_)
{}

const std::int32_t loadQuickReplyShortcutMessages::ID;

object_ptr<loadQuickReplyShortcutMessages> loadQuickReplyShortcutMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loadQuickReplyShortcutMessages> res = make_object<loadQuickReplyShortcutMessages>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  return res;
}

void loadQuickReplyShortcutMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadQuickReplyShortcutMessages");
    s.store_field("shortcutId", shortcut_id_);
    s.store_class_end();
  }
}

loadQuickReplyShortcutMessages::ReturnType loadQuickReplyShortcutMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadQuickReplyShortcutMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadQuickReplyShortcutMessages").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass loadQuickReplyShortcuts::Class;

loadQuickReplyShortcuts::loadQuickReplyShortcuts() {
}

const std::int32_t loadQuickReplyShortcuts::ID;

object_ptr<loadQuickReplyShortcuts> loadQuickReplyShortcuts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<loadQuickReplyShortcuts>();
}

void loadQuickReplyShortcuts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadQuickReplyShortcuts");
    s.store_class_end();
  }
}

loadQuickReplyShortcuts::ReturnType loadQuickReplyShortcuts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadQuickReplyShortcuts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadQuickReplyShortcuts").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass loadSavedMessagesTopics::Class;
jfieldID loadSavedMessagesTopics::limit_fieldID;

loadSavedMessagesTopics::loadSavedMessagesTopics()
  : limit_()
{}

loadSavedMessagesTopics::loadSavedMessagesTopics(int32 limit_)
  : limit_(limit_)
{}

const std::int32_t loadSavedMessagesTopics::ID;

object_ptr<loadSavedMessagesTopics> loadSavedMessagesTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<loadSavedMessagesTopics> res = make_object<loadSavedMessagesTopics>();
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void loadSavedMessagesTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LoadSavedMessagesTopics");
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

loadSavedMessagesTopics::ReturnType loadSavedMessagesTopics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void loadSavedMessagesTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LoadSavedMessagesTopics").c_str());
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass logOut::Class;

logOut::logOut() {
}

const std::int32_t logOut::ID;

object_ptr<logOut> logOut::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<logOut>();
}

void logOut::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "LogOut");
    s.store_class_end();
  }
}

logOut::ReturnType logOut::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void logOut::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$LogOut").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass openBotSimilarBot::Class;
jfieldID openBotSimilarBot::bot_user_id_fieldID;
jfieldID openBotSimilarBot::opened_bot_user_id_fieldID;

openBotSimilarBot::openBotSimilarBot()
  : bot_user_id_()
  , opened_bot_user_id_()
{}

openBotSimilarBot::openBotSimilarBot(int53 bot_user_id_, int53 opened_bot_user_id_)
  : bot_user_id_(bot_user_id_)
  , opened_bot_user_id_(opened_bot_user_id_)
{}

const std::int32_t openBotSimilarBot::ID;

object_ptr<openBotSimilarBot> openBotSimilarBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openBotSimilarBot> res = make_object<openBotSimilarBot>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->opened_bot_user_id_ = env->GetLongField(p, res->opened_bot_user_id_fieldID);
  return res;
}

void openBotSimilarBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenBotSimilarBot");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("openedBotUserId", opened_bot_user_id_);
    s.store_class_end();
  }
}

openBotSimilarBot::ReturnType openBotSimilarBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openBotSimilarBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenBotSimilarBot").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    opened_bot_user_id_fieldID = jni::get_field_id(env, Class, "openedBotUserId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass openChat::Class;
jfieldID openChat::chat_id_fieldID;

openChat::openChat()
  : chat_id_()
{}

openChat::openChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t openChat::ID;

object_ptr<openChat> openChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openChat> res = make_object<openChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void openChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

openChat::ReturnType openChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass openChatSimilarChat::Class;
jfieldID openChatSimilarChat::chat_id_fieldID;
jfieldID openChatSimilarChat::opened_chat_id_fieldID;

openChatSimilarChat::openChatSimilarChat()
  : chat_id_()
  , opened_chat_id_()
{}

openChatSimilarChat::openChatSimilarChat(int53 chat_id_, int53 opened_chat_id_)
  : chat_id_(chat_id_)
  , opened_chat_id_(opened_chat_id_)
{}

const std::int32_t openChatSimilarChat::ID;

object_ptr<openChatSimilarChat> openChatSimilarChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openChatSimilarChat> res = make_object<openChatSimilarChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->opened_chat_id_ = env->GetLongField(p, res->opened_chat_id_fieldID);
  return res;
}

void openChatSimilarChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenChatSimilarChat");
    s.store_field("chatId", chat_id_);
    s.store_field("openedChatId", opened_chat_id_);
    s.store_class_end();
  }
}

openChatSimilarChat::ReturnType openChatSimilarChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openChatSimilarChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenChatSimilarChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    opened_chat_id_fieldID = jni::get_field_id(env, Class, "openedChatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass openMessageContent::Class;
jfieldID openMessageContent::chat_id_fieldID;
jfieldID openMessageContent::message_id_fieldID;

openMessageContent::openMessageContent()
  : chat_id_()
  , message_id_()
{}

openMessageContent::openMessageContent(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t openMessageContent::ID;

object_ptr<openMessageContent> openMessageContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openMessageContent> res = make_object<openMessageContent>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void openMessageContent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenMessageContent");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

openMessageContent::ReturnType openMessageContent::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openMessageContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenMessageContent").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass openSponsoredChat::Class;
jfieldID openSponsoredChat::sponsored_chat_unique_id_fieldID;

openSponsoredChat::openSponsoredChat()
  : sponsored_chat_unique_id_()
{}

openSponsoredChat::openSponsoredChat(int53 sponsored_chat_unique_id_)
  : sponsored_chat_unique_id_(sponsored_chat_unique_id_)
{}

const std::int32_t openSponsoredChat::ID;

object_ptr<openSponsoredChat> openSponsoredChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openSponsoredChat> res = make_object<openSponsoredChat>();
  res->sponsored_chat_unique_id_ = env->GetLongField(p, res->sponsored_chat_unique_id_fieldID);
  return res;
}

void openSponsoredChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenSponsoredChat");
    s.store_field("sponsoredChatUniqueId", sponsored_chat_unique_id_);
    s.store_class_end();
  }
}

openSponsoredChat::ReturnType openSponsoredChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openSponsoredChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenSponsoredChat").c_str());
    sponsored_chat_unique_id_fieldID = jni::get_field_id(env, Class, "sponsoredChatUniqueId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass openStory::Class;
jfieldID openStory::story_poster_chat_id_fieldID;
jfieldID openStory::story_id_fieldID;

openStory::openStory()
  : story_poster_chat_id_()
  , story_id_()
{}

openStory::openStory(int53 story_poster_chat_id_, int32 story_id_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
{}

const std::int32_t openStory::ID;

object_ptr<openStory> openStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openStory> res = make_object<openStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  return res;
}

void openStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_class_end();
  }
}

openStory::ReturnType openStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void openStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass openWebApp::Class;
jfieldID openWebApp::chat_id_fieldID;
jfieldID openWebApp::bot_user_id_fieldID;
jfieldID openWebApp::url_fieldID;
jfieldID openWebApp::message_thread_id_fieldID;
jfieldID openWebApp::reply_to_fieldID;
jfieldID openWebApp::parameters_fieldID;

openWebApp::openWebApp()
  : chat_id_()
  , bot_user_id_()
  , url_()
  , message_thread_id_()
  , reply_to_()
  , parameters_()
{}

openWebApp::openWebApp(int53 chat_id_, int53 bot_user_id_, string const &url_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<webAppOpenParameters> &&parameters_)
  : chat_id_(chat_id_)
  , bot_user_id_(bot_user_id_)
  , url_(url_)
  , message_thread_id_(message_thread_id_)
  , reply_to_(std::move(reply_to_))
  , parameters_(std::move(parameters_))
{}

const std::int32_t openWebApp::ID;

object_ptr<openWebApp> openWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<openWebApp> res = make_object<openWebApp>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->url_ = jni::fetch_string(env, p, res->url_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->parameters_ = jni::fetch_tl_object<webAppOpenParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  return res;
}

void openWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OpenWebApp");
    s.store_field("chatId", chat_id_);
    s.store_field("botUserId", bot_user_id_);
    s.store_field("url", url_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_class_end();
  }
}

openWebApp::ReturnType openWebApp::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return webAppInfo::fetch(env, p);
}

void openWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OpenWebApp").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    url_fieldID = jni::get_field_id(env, Class, "url", "Ljava/lang/String;");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$WebAppOpenParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass optimizeStorage::Class;
jfieldID optimizeStorage::size_fieldID;
jfieldID optimizeStorage::ttl_fieldID;
jfieldID optimizeStorage::count_fieldID;
jfieldID optimizeStorage::immunity_delay_fieldID;
jfieldID optimizeStorage::file_types_fieldID;
jfieldID optimizeStorage::chat_ids_fieldID;
jfieldID optimizeStorage::exclude_chat_ids_fieldID;
jfieldID optimizeStorage::return_deleted_file_statistics_fieldID;
jfieldID optimizeStorage::chat_limit_fieldID;

optimizeStorage::optimizeStorage()
  : size_()
  , ttl_()
  , count_()
  , immunity_delay_()
  , file_types_()
  , chat_ids_()
  , exclude_chat_ids_()
  , return_deleted_file_statistics_()
  , chat_limit_()
{}

optimizeStorage::optimizeStorage(int53 size_, int32 ttl_, int32 count_, int32 immunity_delay_, array<object_ptr<FileType>> &&file_types_, array<int53> &&chat_ids_, array<int53> &&exclude_chat_ids_, bool return_deleted_file_statistics_, int32 chat_limit_)
  : size_(size_)
  , ttl_(ttl_)
  , count_(count_)
  , immunity_delay_(immunity_delay_)
  , file_types_(std::move(file_types_))
  , chat_ids_(std::move(chat_ids_))
  , exclude_chat_ids_(std::move(exclude_chat_ids_))
  , return_deleted_file_statistics_(return_deleted_file_statistics_)
  , chat_limit_(chat_limit_)
{}

const std::int32_t optimizeStorage::ID;

object_ptr<optimizeStorage> optimizeStorage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<optimizeStorage> res = make_object<optimizeStorage>();
  res->size_ = env->GetLongField(p, res->size_fieldID);
  res->ttl_ = env->GetIntField(p, res->ttl_fieldID);
  res->count_ = env->GetIntField(p, res->count_fieldID);
  res->immunity_delay_ = env->GetIntField(p, res->immunity_delay_fieldID);
  res->file_types_ = jni::FetchVector<FileType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->file_types_fieldID));
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  res->exclude_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->exclude_chat_ids_fieldID));
  res->return_deleted_file_statistics_ = (env->GetBooleanField(p, res->return_deleted_file_statistics_fieldID) != 0);
  res->chat_limit_ = env->GetIntField(p, res->chat_limit_fieldID);
  return res;
}

void optimizeStorage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "OptimizeStorage");
    s.store_field("size", size_);
    s.store_field("ttl", ttl_);
    s.store_field("count", count_);
    s.store_field("immunityDelay", immunity_delay_);
    { s.store_vector_begin("fileTypes", file_types_.size()); for (const auto &_value : file_types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    { s.store_vector_begin("excludeChatIds", exclude_chat_ids_.size()); for (const auto &_value : exclude_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("returnDeletedFileStatistics", return_deleted_file_statistics_);
    s.store_field("chatLimit", chat_limit_);
    s.store_class_end();
  }
}

optimizeStorage::ReturnType optimizeStorage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return storageStatistics::fetch(env, p);
}

void optimizeStorage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$OptimizeStorage").c_str());
    size_fieldID = jni::get_field_id(env, Class, "size", "J");
    ttl_fieldID = jni::get_field_id(env, Class, "ttl", "I");
    count_fieldID = jni::get_field_id(env, Class, "count", "I");
    immunity_delay_fieldID = jni::get_field_id(env, Class, "immunityDelay", "I");
    file_types_fieldID = jni::get_field_id(env, Class, "fileTypes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$FileType;").c_str());
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    exclude_chat_ids_fieldID = jni::get_field_id(env, Class, "excludeChatIds", "[J");
    return_deleted_file_statistics_fieldID = jni::get_field_id(env, Class, "returnDeletedFileStatistics", "Z");
    chat_limit_fieldID = jni::get_field_id(env, Class, "chatLimit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass parseMarkdown::Class;
jfieldID parseMarkdown::text_fieldID;

parseMarkdown::parseMarkdown()
  : text_()
{}

parseMarkdown::parseMarkdown(object_ptr<formattedText> &&text_)
  : text_(std::move(text_))
{}

const std::int32_t parseMarkdown::ID;

object_ptr<parseMarkdown> parseMarkdown::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<parseMarkdown> res = make_object<parseMarkdown>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return res;
}

void parseMarkdown::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ParseMarkdown");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

parseMarkdown::ReturnType parseMarkdown::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return formattedText::fetch(env, p);
}

void parseMarkdown::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ParseMarkdown").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass parseTextEntities::Class;
jfieldID parseTextEntities::text_fieldID;
jfieldID parseTextEntities::parse_mode_fieldID;

parseTextEntities::parseTextEntities()
  : text_()
  , parse_mode_()
{}

parseTextEntities::parseTextEntities(string const &text_, object_ptr<TextParseMode> &&parse_mode_)
  : text_(text_)
  , parse_mode_(std::move(parse_mode_))
{}

const std::int32_t parseTextEntities::ID;

object_ptr<parseTextEntities> parseTextEntities::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<parseTextEntities> res = make_object<parseTextEntities>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->parse_mode_ = jni::fetch_tl_object<TextParseMode>(env, jni::fetch_object(env, p, res->parse_mode_fieldID));
  return res;
}

void parseTextEntities::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ParseTextEntities");
    s.store_field("text", text_);
    s.store_object_field("parseMode", static_cast<const BaseObject *>(parse_mode_.get()));
    s.store_class_end();
  }
}

parseTextEntities::ReturnType parseTextEntities::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return formattedText::fetch(env, p);
}

void parseTextEntities::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ParseTextEntities").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    parse_mode_fieldID = jni::get_field_id(env, Class, "parseMode", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TextParseMode;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass pinChatMessage::Class;
jfieldID pinChatMessage::chat_id_fieldID;
jfieldID pinChatMessage::message_id_fieldID;
jfieldID pinChatMessage::disable_notification_fieldID;
jfieldID pinChatMessage::only_for_self_fieldID;

pinChatMessage::pinChatMessage()
  : chat_id_()
  , message_id_()
  , disable_notification_()
  , only_for_self_()
{}

pinChatMessage::pinChatMessage(int53 chat_id_, int53 message_id_, bool disable_notification_, bool only_for_self_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , disable_notification_(disable_notification_)
  , only_for_self_(only_for_self_)
{}

const std::int32_t pinChatMessage::ID;

object_ptr<pinChatMessage> pinChatMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pinChatMessage> res = make_object<pinChatMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  res->only_for_self_ = (env->GetBooleanField(p, res->only_for_self_fieldID) != 0);
  return res;
}

void pinChatMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PinChatMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("disableNotification", disable_notification_);
    s.store_field("onlyForSelf", only_for_self_);
    s.store_class_end();
  }
}

pinChatMessage::ReturnType pinChatMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void pinChatMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PinChatMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    only_for_self_fieldID = jni::get_field_id(env, Class, "onlyForSelf", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass pingProxy::Class;
jfieldID pingProxy::proxy_id_fieldID;

pingProxy::pingProxy()
  : proxy_id_()
{}

pingProxy::pingProxy(int32 proxy_id_)
  : proxy_id_(proxy_id_)
{}

const std::int32_t pingProxy::ID;

object_ptr<pingProxy> pingProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<pingProxy> res = make_object<pingProxy>();
  res->proxy_id_ = env->GetIntField(p, res->proxy_id_fieldID);
  return res;
}

void pingProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PingProxy");
    s.store_field("proxyId", proxy_id_);
    s.store_class_end();
  }
}

pingProxy::ReturnType pingProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return seconds::fetch(env, p);
}

void pingProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PingProxy").c_str());
    proxy_id_fieldID = jni::get_field_id(env, Class, "proxyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass postStory::Class;
jfieldID postStory::chat_id_fieldID;
jfieldID postStory::content_fieldID;
jfieldID postStory::areas_fieldID;
jfieldID postStory::caption_fieldID;
jfieldID postStory::privacy_settings_fieldID;
jfieldID postStory::active_period_fieldID;
jfieldID postStory::from_story_full_id_fieldID;
jfieldID postStory::is_posted_to_chat_page_fieldID;
jfieldID postStory::protect_content_fieldID;

postStory::postStory()
  : chat_id_()
  , content_()
  , areas_()
  , caption_()
  , privacy_settings_()
  , active_period_()
  , from_story_full_id_()
  , is_posted_to_chat_page_()
  , protect_content_()
{}

postStory::postStory(int53 chat_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_, object_ptr<StoryPrivacySettings> &&privacy_settings_, int32 active_period_, object_ptr<storyFullId> &&from_story_full_id_, bool is_posted_to_chat_page_, bool protect_content_)
  : chat_id_(chat_id_)
  , content_(std::move(content_))
  , areas_(std::move(areas_))
  , caption_(std::move(caption_))
  , privacy_settings_(std::move(privacy_settings_))
  , active_period_(active_period_)
  , from_story_full_id_(std::move(from_story_full_id_))
  , is_posted_to_chat_page_(is_posted_to_chat_page_)
  , protect_content_(protect_content_)
{}

const std::int32_t postStory::ID;

object_ptr<postStory> postStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<postStory> res = make_object<postStory>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->content_ = jni::fetch_tl_object<InputStoryContent>(env, jni::fetch_object(env, p, res->content_fieldID));
  res->areas_ = jni::fetch_tl_object<inputStoryAreas>(env, jni::fetch_object(env, p, res->areas_fieldID));
  res->caption_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->caption_fieldID));
  res->privacy_settings_ = jni::fetch_tl_object<StoryPrivacySettings>(env, jni::fetch_object(env, p, res->privacy_settings_fieldID));
  res->active_period_ = env->GetIntField(p, res->active_period_fieldID);
  res->from_story_full_id_ = jni::fetch_tl_object<storyFullId>(env, jni::fetch_object(env, p, res->from_story_full_id_fieldID));
  res->is_posted_to_chat_page_ = (env->GetBooleanField(p, res->is_posted_to_chat_page_fieldID) != 0);
  res->protect_content_ = (env->GetBooleanField(p, res->protect_content_fieldID) != 0);
  return res;
}

void postStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PostStory");
    s.store_field("chatId", chat_id_);
    s.store_object_field("content", static_cast<const BaseObject *>(content_.get()));
    s.store_object_field("areas", static_cast<const BaseObject *>(areas_.get()));
    s.store_object_field("caption", static_cast<const BaseObject *>(caption_.get()));
    s.store_object_field("privacySettings", static_cast<const BaseObject *>(privacy_settings_.get()));
    s.store_field("activePeriod", active_period_);
    s.store_object_field("fromStoryFullId", static_cast<const BaseObject *>(from_story_full_id_.get()));
    s.store_field("isPostedToChatPage", is_posted_to_chat_page_);
    s.store_field("protectContent", protect_content_);
    s.store_class_end();
  }
}

postStory::ReturnType postStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return story::fetch(env, p);
}

void postStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PostStory").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    content_fieldID = jni::get_field_id(env, Class, "content", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryContent;").c_str());
    areas_fieldID = jni::get_field_id(env, Class, "areas", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputStoryAreas;").c_str());
    caption_fieldID = jni::get_field_id(env, Class, "caption", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    privacy_settings_fieldID = jni::get_field_id(env, Class, "privacySettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryPrivacySettings;").c_str());
    active_period_fieldID = jni::get_field_id(env, Class, "activePeriod", "I");
    from_story_full_id_fieldID = jni::get_field_id(env, Class, "fromStoryFullId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryFullId;").c_str());
    is_posted_to_chat_page_fieldID = jni::get_field_id(env, Class, "isPostedToChatPage", "Z");
    protect_content_fieldID = jni::get_field_id(env, Class, "protectContent", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass preliminaryUploadFile::Class;
jfieldID preliminaryUploadFile::file_fieldID;
jfieldID preliminaryUploadFile::file_type_fieldID;
jfieldID preliminaryUploadFile::priority_fieldID;

preliminaryUploadFile::preliminaryUploadFile()
  : file_()
  , file_type_()
  , priority_()
{}

preliminaryUploadFile::preliminaryUploadFile(object_ptr<InputFile> &&file_, object_ptr<FileType> &&file_type_, int32 priority_)
  : file_(std::move(file_))
  , file_type_(std::move(file_type_))
  , priority_(priority_)
{}

const std::int32_t preliminaryUploadFile::ID;

object_ptr<preliminaryUploadFile> preliminaryUploadFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<preliminaryUploadFile> res = make_object<preliminaryUploadFile>();
  res->file_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->file_fieldID));
  res->file_type_ = jni::fetch_tl_object<FileType>(env, jni::fetch_object(env, p, res->file_type_fieldID));
  res->priority_ = env->GetIntField(p, res->priority_fieldID);
  return res;
}

void preliminaryUploadFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "PreliminaryUploadFile");
    s.store_object_field("file", static_cast<const BaseObject *>(file_.get()));
    s.store_object_field("fileType", static_cast<const BaseObject *>(file_type_.get()));
    s.store_field("priority", priority_);
    s.store_class_end();
  }
}

preliminaryUploadFile::ReturnType preliminaryUploadFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void preliminaryUploadFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$PreliminaryUploadFile").c_str());
    file_fieldID = jni::get_field_id(env, Class, "file", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    file_type_fieldID = jni::get_field_id(env, Class, "fileType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FileType;").c_str());
    priority_fieldID = jni::get_field_id(env, Class, "priority", "I");
    return true;
  }();
  (void)is_inited;
}

jclass processChatFolderNewChats::Class;
jfieldID processChatFolderNewChats::chat_folder_id_fieldID;
jfieldID processChatFolderNewChats::added_chat_ids_fieldID;

processChatFolderNewChats::processChatFolderNewChats()
  : chat_folder_id_()
  , added_chat_ids_()
{}

processChatFolderNewChats::processChatFolderNewChats(int32 chat_folder_id_, array<int53> &&added_chat_ids_)
  : chat_folder_id_(chat_folder_id_)
  , added_chat_ids_(std::move(added_chat_ids_))
{}

const std::int32_t processChatFolderNewChats::ID;

object_ptr<processChatFolderNewChats> processChatFolderNewChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<processChatFolderNewChats> res = make_object<processChatFolderNewChats>();
  res->chat_folder_id_ = env->GetIntField(p, res->chat_folder_id_fieldID);
  res->added_chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->added_chat_ids_fieldID));
  return res;
}

void processChatFolderNewChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProcessChatFolderNewChats");
    s.store_field("chatFolderId", chat_folder_id_);
    { s.store_vector_begin("addedChatIds", added_chat_ids_.size()); for (const auto &_value : added_chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

processChatFolderNewChats::ReturnType processChatFolderNewChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void processChatFolderNewChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProcessChatFolderNewChats").c_str());
    chat_folder_id_fieldID = jni::get_field_id(env, Class, "chatFolderId", "I");
    added_chat_ids_fieldID = jni::get_field_id(env, Class, "addedChatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass processChatJoinRequest::Class;
jfieldID processChatJoinRequest::chat_id_fieldID;
jfieldID processChatJoinRequest::user_id_fieldID;
jfieldID processChatJoinRequest::approve_fieldID;

processChatJoinRequest::processChatJoinRequest()
  : chat_id_()
  , user_id_()
  , approve_()
{}

processChatJoinRequest::processChatJoinRequest(int53 chat_id_, int53 user_id_, bool approve_)
  : chat_id_(chat_id_)
  , user_id_(user_id_)
  , approve_(approve_)
{}

const std::int32_t processChatJoinRequest::ID;

object_ptr<processChatJoinRequest> processChatJoinRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<processChatJoinRequest> res = make_object<processChatJoinRequest>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->approve_ = (env->GetBooleanField(p, res->approve_fieldID) != 0);
  return res;
}

void processChatJoinRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProcessChatJoinRequest");
    s.store_field("chatId", chat_id_);
    s.store_field("userId", user_id_);
    s.store_field("approve", approve_);
    s.store_class_end();
  }
}

processChatJoinRequest::ReturnType processChatJoinRequest::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void processChatJoinRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProcessChatJoinRequest").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    approve_fieldID = jni::get_field_id(env, Class, "approve", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass processChatJoinRequests::Class;
jfieldID processChatJoinRequests::chat_id_fieldID;
jfieldID processChatJoinRequests::invite_link_fieldID;
jfieldID processChatJoinRequests::approve_fieldID;

processChatJoinRequests::processChatJoinRequests()
  : chat_id_()
  , invite_link_()
  , approve_()
{}

processChatJoinRequests::processChatJoinRequests(int53 chat_id_, string const &invite_link_, bool approve_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
  , approve_(approve_)
{}

const std::int32_t processChatJoinRequests::ID;

object_ptr<processChatJoinRequests> processChatJoinRequests::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<processChatJoinRequests> res = make_object<processChatJoinRequests>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  res->approve_ = (env->GetBooleanField(p, res->approve_fieldID) != 0);
  return res;
}

void processChatJoinRequests::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProcessChatJoinRequests");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_field("approve", approve_);
    s.store_class_end();
  }
}

processChatJoinRequests::ReturnType processChatJoinRequests::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void processChatJoinRequests::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProcessChatJoinRequests").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    approve_fieldID = jni::get_field_id(env, Class, "approve", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass processPushNotification::Class;
jfieldID processPushNotification::payload_fieldID;

processPushNotification::processPushNotification()
  : payload_()
{}

processPushNotification::processPushNotification(string const &payload_)
  : payload_(payload_)
{}

const std::int32_t processPushNotification::ID;

object_ptr<processPushNotification> processPushNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<processPushNotification> res = make_object<processPushNotification>();
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  return res;
}

void processPushNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ProcessPushNotification");
    s.store_field("payload", payload_);
    s.store_class_end();
  }
}

processPushNotification::ReturnType processPushNotification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void processPushNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ProcessPushNotification").c_str());
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass rateSpeechRecognition::Class;
jfieldID rateSpeechRecognition::chat_id_fieldID;
jfieldID rateSpeechRecognition::message_id_fieldID;
jfieldID rateSpeechRecognition::is_good_fieldID;

rateSpeechRecognition::rateSpeechRecognition()
  : chat_id_()
  , message_id_()
  , is_good_()
{}

rateSpeechRecognition::rateSpeechRecognition(int53 chat_id_, int53 message_id_, bool is_good_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , is_good_(is_good_)
{}

const std::int32_t rateSpeechRecognition::ID;

object_ptr<rateSpeechRecognition> rateSpeechRecognition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<rateSpeechRecognition> res = make_object<rateSpeechRecognition>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_good_ = (env->GetBooleanField(p, res->is_good_fieldID) != 0);
  return res;
}

void rateSpeechRecognition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RateSpeechRecognition");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("isGood", is_good_);
    s.store_class_end();
  }
}

rateSpeechRecognition::ReturnType rateSpeechRecognition::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void rateSpeechRecognition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RateSpeechRecognition").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_good_fieldID = jni::get_field_id(env, Class, "isGood", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass readAllChatMentions::Class;
jfieldID readAllChatMentions::chat_id_fieldID;

readAllChatMentions::readAllChatMentions()
  : chat_id_()
{}

readAllChatMentions::readAllChatMentions(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t readAllChatMentions::ID;

object_ptr<readAllChatMentions> readAllChatMentions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readAllChatMentions> res = make_object<readAllChatMentions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void readAllChatMentions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadAllChatMentions");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

readAllChatMentions::ReturnType readAllChatMentions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readAllChatMentions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadAllChatMentions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readAllChatReactions::Class;
jfieldID readAllChatReactions::chat_id_fieldID;

readAllChatReactions::readAllChatReactions()
  : chat_id_()
{}

readAllChatReactions::readAllChatReactions(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t readAllChatReactions::ID;

object_ptr<readAllChatReactions> readAllChatReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readAllChatReactions> res = make_object<readAllChatReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void readAllChatReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadAllChatReactions");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

readAllChatReactions::ReturnType readAllChatReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readAllChatReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadAllChatReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readAllMessageThreadMentions::Class;
jfieldID readAllMessageThreadMentions::chat_id_fieldID;
jfieldID readAllMessageThreadMentions::message_thread_id_fieldID;

readAllMessageThreadMentions::readAllMessageThreadMentions()
  : chat_id_()
  , message_thread_id_()
{}

readAllMessageThreadMentions::readAllMessageThreadMentions(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t readAllMessageThreadMentions::ID;

object_ptr<readAllMessageThreadMentions> readAllMessageThreadMentions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readAllMessageThreadMentions> res = make_object<readAllMessageThreadMentions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void readAllMessageThreadMentions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadAllMessageThreadMentions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

readAllMessageThreadMentions::ReturnType readAllMessageThreadMentions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readAllMessageThreadMentions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadAllMessageThreadMentions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readAllMessageThreadReactions::Class;
jfieldID readAllMessageThreadReactions::chat_id_fieldID;
jfieldID readAllMessageThreadReactions::message_thread_id_fieldID;

readAllMessageThreadReactions::readAllMessageThreadReactions()
  : chat_id_()
  , message_thread_id_()
{}

readAllMessageThreadReactions::readAllMessageThreadReactions(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t readAllMessageThreadReactions::ID;

object_ptr<readAllMessageThreadReactions> readAllMessageThreadReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readAllMessageThreadReactions> res = make_object<readAllMessageThreadReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void readAllMessageThreadReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadAllMessageThreadReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

readAllMessageThreadReactions::ReturnType readAllMessageThreadReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readAllMessageThreadReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadAllMessageThreadReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readBusinessMessage::Class;
jfieldID readBusinessMessage::business_connection_id_fieldID;
jfieldID readBusinessMessage::chat_id_fieldID;
jfieldID readBusinessMessage::message_id_fieldID;

readBusinessMessage::readBusinessMessage()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
{}

readBusinessMessage::readBusinessMessage(string const &business_connection_id_, int53 chat_id_, int53 message_id_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t readBusinessMessage::ID;

object_ptr<readBusinessMessage> readBusinessMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readBusinessMessage> res = make_object<readBusinessMessage>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void readBusinessMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadBusinessMessage");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

readBusinessMessage::ReturnType readBusinessMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readBusinessMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadBusinessMessage").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readChatList::Class;
jfieldID readChatList::chat_list_fieldID;

readChatList::readChatList()
  : chat_list_()
{}

readChatList::readChatList(object_ptr<ChatList> &&chat_list_)
  : chat_list_(std::move(chat_list_))
{}

const std::int32_t readChatList::ID;

object_ptr<readChatList> readChatList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readChatList> res = make_object<readChatList>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  return res;
}

void readChatList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadChatList");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_class_end();
  }
}

readChatList::ReturnType readChatList::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void readChatList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadChatList").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass readFilePart::Class;
jfieldID readFilePart::file_id_fieldID;
jfieldID readFilePart::offset_fieldID;
jfieldID readFilePart::count_fieldID;

readFilePart::readFilePart()
  : file_id_()
  , offset_()
  , count_()
{}

readFilePart::readFilePart(int32 file_id_, int53 offset_, int53 count_)
  : file_id_(file_id_)
  , offset_(offset_)
  , count_(count_)
{}

const std::int32_t readFilePart::ID;

object_ptr<readFilePart> readFilePart::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readFilePart> res = make_object<readFilePart>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->offset_ = env->GetLongField(p, res->offset_fieldID);
  res->count_ = env->GetLongField(p, res->count_fieldID);
  return res;
}

void readFilePart::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReadFilePart");
    s.store_field("fileId", file_id_);
    s.store_field("offset", offset_);
    s.store_field("count", count_);
    s.store_class_end();
  }
}

readFilePart::ReturnType readFilePart::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return data::fetch(env, p);
}

void readFilePart::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReadFilePart").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "J");
    count_fieldID = jni::get_field_id(env, Class, "count", "J");
    return true;
  }();
  (void)is_inited;
}

jclass readdQuickReplyShortcutMessages::Class;
jfieldID readdQuickReplyShortcutMessages::shortcut_name_fieldID;
jfieldID readdQuickReplyShortcutMessages::message_ids_fieldID;

readdQuickReplyShortcutMessages::readdQuickReplyShortcutMessages()
  : shortcut_name_()
  , message_ids_()
{}

readdQuickReplyShortcutMessages::readdQuickReplyShortcutMessages(string const &shortcut_name_, array<int53> &&message_ids_)
  : shortcut_name_(shortcut_name_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t readdQuickReplyShortcutMessages::ID;

object_ptr<readdQuickReplyShortcutMessages> readdQuickReplyShortcutMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<readdQuickReplyShortcutMessages> res = make_object<readdQuickReplyShortcutMessages>();
  res->shortcut_name_ = jni::fetch_string(env, p, res->shortcut_name_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return res;
}

void readdQuickReplyShortcutMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReaddQuickReplyShortcutMessages");
    s.store_field("shortcutName", shortcut_name_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

readdQuickReplyShortcutMessages::ReturnType readdQuickReplyShortcutMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return quickReplyMessages::fetch(env, p);
}

void readdQuickReplyShortcutMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReaddQuickReplyShortcutMessages").c_str());
    shortcut_name_fieldID = jni::get_field_id(env, Class, "shortcutName", "Ljava/lang/String;");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass recognizeSpeech::Class;
jfieldID recognizeSpeech::chat_id_fieldID;
jfieldID recognizeSpeech::message_id_fieldID;

recognizeSpeech::recognizeSpeech()
  : chat_id_()
  , message_id_()
{}

recognizeSpeech::recognizeSpeech(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t recognizeSpeech::ID;

object_ptr<recognizeSpeech> recognizeSpeech::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recognizeSpeech> res = make_object<recognizeSpeech>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void recognizeSpeech::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecognizeSpeech");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

recognizeSpeech::ReturnType recognizeSpeech::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void recognizeSpeech::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecognizeSpeech").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass recoverAuthenticationPassword::Class;
jfieldID recoverAuthenticationPassword::recovery_code_fieldID;
jfieldID recoverAuthenticationPassword::new_password_fieldID;
jfieldID recoverAuthenticationPassword::new_hint_fieldID;

recoverAuthenticationPassword::recoverAuthenticationPassword()
  : recovery_code_()
  , new_password_()
  , new_hint_()
{}

recoverAuthenticationPassword::recoverAuthenticationPassword(string const &recovery_code_, string const &new_password_, string const &new_hint_)
  : recovery_code_(recovery_code_)
  , new_password_(new_password_)
  , new_hint_(new_hint_)
{}

const std::int32_t recoverAuthenticationPassword::ID;

object_ptr<recoverAuthenticationPassword> recoverAuthenticationPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recoverAuthenticationPassword> res = make_object<recoverAuthenticationPassword>();
  res->recovery_code_ = jni::fetch_string(env, p, res->recovery_code_fieldID);
  res->new_password_ = jni::fetch_string(env, p, res->new_password_fieldID);
  res->new_hint_ = jni::fetch_string(env, p, res->new_hint_fieldID);
  return res;
}

void recoverAuthenticationPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecoverAuthenticationPassword");
    s.store_field("recoveryCode", recovery_code_);
    s.store_field("newPassword", new_password_);
    s.store_field("newHint", new_hint_);
    s.store_class_end();
  }
}

recoverAuthenticationPassword::ReturnType recoverAuthenticationPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void recoverAuthenticationPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecoverAuthenticationPassword").c_str());
    recovery_code_fieldID = jni::get_field_id(env, Class, "recoveryCode", "Ljava/lang/String;");
    new_password_fieldID = jni::get_field_id(env, Class, "newPassword", "Ljava/lang/String;");
    new_hint_fieldID = jni::get_field_id(env, Class, "newHint", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass recoverPassword::Class;
jfieldID recoverPassword::recovery_code_fieldID;
jfieldID recoverPassword::new_password_fieldID;
jfieldID recoverPassword::new_hint_fieldID;

recoverPassword::recoverPassword()
  : recovery_code_()
  , new_password_()
  , new_hint_()
{}

recoverPassword::recoverPassword(string const &recovery_code_, string const &new_password_, string const &new_hint_)
  : recovery_code_(recovery_code_)
  , new_password_(new_password_)
  , new_hint_(new_hint_)
{}

const std::int32_t recoverPassword::ID;

object_ptr<recoverPassword> recoverPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<recoverPassword> res = make_object<recoverPassword>();
  res->recovery_code_ = jni::fetch_string(env, p, res->recovery_code_fieldID);
  res->new_password_ = jni::fetch_string(env, p, res->new_password_fieldID);
  res->new_hint_ = jni::fetch_string(env, p, res->new_hint_fieldID);
  return res;
}

void recoverPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RecoverPassword");
    s.store_field("recoveryCode", recovery_code_);
    s.store_field("newPassword", new_password_);
    s.store_field("newHint", new_hint_);
    s.store_class_end();
  }
}

recoverPassword::ReturnType recoverPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void recoverPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RecoverPassword").c_str());
    recovery_code_fieldID = jni::get_field_id(env, Class, "recoveryCode", "Ljava/lang/String;");
    new_password_fieldID = jni::get_field_id(env, Class, "newPassword", "Ljava/lang/String;");
    new_hint_fieldID = jni::get_field_id(env, Class, "newHint", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass refundStarPayment::Class;
jfieldID refundStarPayment::user_id_fieldID;
jfieldID refundStarPayment::telegram_payment_charge_id_fieldID;

refundStarPayment::refundStarPayment()
  : user_id_()
  , telegram_payment_charge_id_()
{}

refundStarPayment::refundStarPayment(int53 user_id_, string const &telegram_payment_charge_id_)
  : user_id_(user_id_)
  , telegram_payment_charge_id_(telegram_payment_charge_id_)
{}

const std::int32_t refundStarPayment::ID;

object_ptr<refundStarPayment> refundStarPayment::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<refundStarPayment> res = make_object<refundStarPayment>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->telegram_payment_charge_id_ = jni::fetch_string(env, p, res->telegram_payment_charge_id_fieldID);
  return res;
}

void refundStarPayment::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RefundStarPayment");
    s.store_field("userId", user_id_);
    s.store_field("telegramPaymentChargeId", telegram_payment_charge_id_);
    s.store_class_end();
  }
}

refundStarPayment::ReturnType refundStarPayment::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void refundStarPayment::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RefundStarPayment").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    telegram_payment_charge_id_fieldID = jni::get_field_id(env, Class, "telegramPaymentChargeId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass registerDevice::Class;
jfieldID registerDevice::device_token_fieldID;
jfieldID registerDevice::other_user_ids_fieldID;

registerDevice::registerDevice()
  : device_token_()
  , other_user_ids_()
{}

registerDevice::registerDevice(object_ptr<DeviceToken> &&device_token_, array<int53> &&other_user_ids_)
  : device_token_(std::move(device_token_))
  , other_user_ids_(std::move(other_user_ids_))
{}

const std::int32_t registerDevice::ID;

object_ptr<registerDevice> registerDevice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<registerDevice> res = make_object<registerDevice>();
  res->device_token_ = jni::fetch_tl_object<DeviceToken>(env, jni::fetch_object(env, p, res->device_token_fieldID));
  res->other_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->other_user_ids_fieldID));
  return res;
}

void registerDevice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RegisterDevice");
    s.store_object_field("deviceToken", static_cast<const BaseObject *>(device_token_.get()));
    { s.store_vector_begin("otherUserIds", other_user_ids_.size()); for (const auto &_value : other_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

registerDevice::ReturnType registerDevice::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return pushReceiverId::fetch(env, p);
}

void registerDevice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RegisterDevice").c_str());
    device_token_fieldID = jni::get_field_id(env, Class, "deviceToken", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DeviceToken;").c_str());
    other_user_ids_fieldID = jni::get_field_id(env, Class, "otherUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass registerUser::Class;
jfieldID registerUser::first_name_fieldID;
jfieldID registerUser::last_name_fieldID;
jfieldID registerUser::disable_notification_fieldID;

registerUser::registerUser()
  : first_name_()
  , last_name_()
  , disable_notification_()
{}

registerUser::registerUser(string const &first_name_, string const &last_name_, bool disable_notification_)
  : first_name_(first_name_)
  , last_name_(last_name_)
  , disable_notification_(disable_notification_)
{}

const std::int32_t registerUser::ID;

object_ptr<registerUser> registerUser::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<registerUser> res = make_object<registerUser>();
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  return res;
}

void registerUser::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RegisterUser");
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_field("disableNotification", disable_notification_);
    s.store_class_end();
  }
}

registerUser::ReturnType registerUser::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void registerUser::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RegisterUser").c_str());
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass removeAllFilesFromDownloads::Class;
jfieldID removeAllFilesFromDownloads::only_active_fieldID;
jfieldID removeAllFilesFromDownloads::only_completed_fieldID;
jfieldID removeAllFilesFromDownloads::delete_from_cache_fieldID;

removeAllFilesFromDownloads::removeAllFilesFromDownloads()
  : only_active_()
  , only_completed_()
  , delete_from_cache_()
{}

removeAllFilesFromDownloads::removeAllFilesFromDownloads(bool only_active_, bool only_completed_, bool delete_from_cache_)
  : only_active_(only_active_)
  , only_completed_(only_completed_)
  , delete_from_cache_(delete_from_cache_)
{}

const std::int32_t removeAllFilesFromDownloads::ID;

object_ptr<removeAllFilesFromDownloads> removeAllFilesFromDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeAllFilesFromDownloads> res = make_object<removeAllFilesFromDownloads>();
  res->only_active_ = (env->GetBooleanField(p, res->only_active_fieldID) != 0);
  res->only_completed_ = (env->GetBooleanField(p, res->only_completed_fieldID) != 0);
  res->delete_from_cache_ = (env->GetBooleanField(p, res->delete_from_cache_fieldID) != 0);
  return res;
}

void removeAllFilesFromDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveAllFilesFromDownloads");
    s.store_field("onlyActive", only_active_);
    s.store_field("onlyCompleted", only_completed_);
    s.store_field("deleteFromCache", delete_from_cache_);
    s.store_class_end();
  }
}

removeAllFilesFromDownloads::ReturnType removeAllFilesFromDownloads::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeAllFilesFromDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveAllFilesFromDownloads").c_str());
    only_active_fieldID = jni::get_field_id(env, Class, "onlyActive", "Z");
    only_completed_fieldID = jni::get_field_id(env, Class, "onlyCompleted", "Z");
    delete_from_cache_fieldID = jni::get_field_id(env, Class, "deleteFromCache", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass removeBusinessConnectedBotFromChat::Class;
jfieldID removeBusinessConnectedBotFromChat::chat_id_fieldID;

removeBusinessConnectedBotFromChat::removeBusinessConnectedBotFromChat()
  : chat_id_()
{}

removeBusinessConnectedBotFromChat::removeBusinessConnectedBotFromChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t removeBusinessConnectedBotFromChat::ID;

object_ptr<removeBusinessConnectedBotFromChat> removeBusinessConnectedBotFromChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeBusinessConnectedBotFromChat> res = make_object<removeBusinessConnectedBotFromChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void removeBusinessConnectedBotFromChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveBusinessConnectedBotFromChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

removeBusinessConnectedBotFromChat::ReturnType removeBusinessConnectedBotFromChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeBusinessConnectedBotFromChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveBusinessConnectedBotFromChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeChatActionBar::Class;
jfieldID removeChatActionBar::chat_id_fieldID;

removeChatActionBar::removeChatActionBar()
  : chat_id_()
{}

removeChatActionBar::removeChatActionBar(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t removeChatActionBar::ID;

object_ptr<removeChatActionBar> removeChatActionBar::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeChatActionBar> res = make_object<removeChatActionBar>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void removeChatActionBar::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveChatActionBar");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

removeChatActionBar::ReturnType removeChatActionBar::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeChatActionBar::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveChatActionBar").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeContacts::Class;
jfieldID removeContacts::user_ids_fieldID;

removeContacts::removeContacts()
  : user_ids_()
{}

removeContacts::removeContacts(array<int53> &&user_ids_)
  : user_ids_(std::move(user_ids_))
{}

const std::int32_t removeContacts::ID;

object_ptr<removeContacts> removeContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeContacts> res = make_object<removeContacts>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void removeContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveContacts");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

removeContacts::ReturnType removeContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveContacts").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass removeFavoriteSticker::Class;
jfieldID removeFavoriteSticker::sticker_fieldID;

removeFavoriteSticker::removeFavoriteSticker()
  : sticker_()
{}

removeFavoriteSticker::removeFavoriteSticker(object_ptr<InputFile> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t removeFavoriteSticker::ID;

object_ptr<removeFavoriteSticker> removeFavoriteSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeFavoriteSticker> res = make_object<removeFavoriteSticker>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void removeFavoriteSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveFavoriteSticker");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

removeFavoriteSticker::ReturnType removeFavoriteSticker::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeFavoriteSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveFavoriteSticker").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeFileFromDownloads::Class;
jfieldID removeFileFromDownloads::file_id_fieldID;
jfieldID removeFileFromDownloads::delete_from_cache_fieldID;

removeFileFromDownloads::removeFileFromDownloads()
  : file_id_()
  , delete_from_cache_()
{}

removeFileFromDownloads::removeFileFromDownloads(int32 file_id_, bool delete_from_cache_)
  : file_id_(file_id_)
  , delete_from_cache_(delete_from_cache_)
{}

const std::int32_t removeFileFromDownloads::ID;

object_ptr<removeFileFromDownloads> removeFileFromDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeFileFromDownloads> res = make_object<removeFileFromDownloads>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->delete_from_cache_ = (env->GetBooleanField(p, res->delete_from_cache_fieldID) != 0);
  return res;
}

void removeFileFromDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveFileFromDownloads");
    s.store_field("fileId", file_id_);
    s.store_field("deleteFromCache", delete_from_cache_);
    s.store_class_end();
  }
}

removeFileFromDownloads::ReturnType removeFileFromDownloads::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeFileFromDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveFileFromDownloads").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    delete_from_cache_fieldID = jni::get_field_id(env, Class, "deleteFromCache", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass removeInstalledBackground::Class;
jfieldID removeInstalledBackground::background_id_fieldID;

removeInstalledBackground::removeInstalledBackground()
  : background_id_()
{}

removeInstalledBackground::removeInstalledBackground(int64 background_id_)
  : background_id_(background_id_)
{}

const std::int32_t removeInstalledBackground::ID;

object_ptr<removeInstalledBackground> removeInstalledBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeInstalledBackground> res = make_object<removeInstalledBackground>();
  res->background_id_ = env->GetLongField(p, res->background_id_fieldID);
  return res;
}

void removeInstalledBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveInstalledBackground");
    s.store_field("backgroundId", background_id_);
    s.store_class_end();
  }
}

removeInstalledBackground::ReturnType removeInstalledBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeInstalledBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveInstalledBackground").c_str());
    background_id_fieldID = jni::get_field_id(env, Class, "backgroundId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeMessageReaction::Class;
jfieldID removeMessageReaction::chat_id_fieldID;
jfieldID removeMessageReaction::message_id_fieldID;
jfieldID removeMessageReaction::reaction_type_fieldID;

removeMessageReaction::removeMessageReaction()
  : chat_id_()
  , message_id_()
  , reaction_type_()
{}

removeMessageReaction::removeMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reaction_type_(std::move(reaction_type_))
{}

const std::int32_t removeMessageReaction::ID;

object_ptr<removeMessageReaction> removeMessageReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeMessageReaction> res = make_object<removeMessageReaction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  return res;
}

void removeMessageReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveMessageReaction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_class_end();
  }
}

removeMessageReaction::ReturnType removeMessageReaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeMessageReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveMessageReaction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeMessageSenderBotVerification::Class;
jfieldID removeMessageSenderBotVerification::bot_user_id_fieldID;
jfieldID removeMessageSenderBotVerification::verified_id_fieldID;

removeMessageSenderBotVerification::removeMessageSenderBotVerification()
  : bot_user_id_()
  , verified_id_()
{}

removeMessageSenderBotVerification::removeMessageSenderBotVerification(int53 bot_user_id_, object_ptr<MessageSender> &&verified_id_)
  : bot_user_id_(bot_user_id_)
  , verified_id_(std::move(verified_id_))
{}

const std::int32_t removeMessageSenderBotVerification::ID;

object_ptr<removeMessageSenderBotVerification> removeMessageSenderBotVerification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeMessageSenderBotVerification> res = make_object<removeMessageSenderBotVerification>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->verified_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->verified_id_fieldID));
  return res;
}

void removeMessageSenderBotVerification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveMessageSenderBotVerification");
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("verifiedId", static_cast<const BaseObject *>(verified_id_.get()));
    s.store_class_end();
  }
}

removeMessageSenderBotVerification::ReturnType removeMessageSenderBotVerification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeMessageSenderBotVerification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveMessageSenderBotVerification").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    verified_id_fieldID = jni::get_field_id(env, Class, "verifiedId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeNotification::Class;
jfieldID removeNotification::notification_group_id_fieldID;
jfieldID removeNotification::notification_id_fieldID;

removeNotification::removeNotification()
  : notification_group_id_()
  , notification_id_()
{}

removeNotification::removeNotification(int32 notification_group_id_, int32 notification_id_)
  : notification_group_id_(notification_group_id_)
  , notification_id_(notification_id_)
{}

const std::int32_t removeNotification::ID;

object_ptr<removeNotification> removeNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeNotification> res = make_object<removeNotification>();
  res->notification_group_id_ = env->GetIntField(p, res->notification_group_id_fieldID);
  res->notification_id_ = env->GetIntField(p, res->notification_id_fieldID);
  return res;
}

void removeNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveNotification");
    s.store_field("notificationGroupId", notification_group_id_);
    s.store_field("notificationId", notification_id_);
    s.store_class_end();
  }
}

removeNotification::ReturnType removeNotification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveNotification").c_str());
    notification_group_id_fieldID = jni::get_field_id(env, Class, "notificationGroupId", "I");
    notification_id_fieldID = jni::get_field_id(env, Class, "notificationId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass removeNotificationGroup::Class;
jfieldID removeNotificationGroup::notification_group_id_fieldID;
jfieldID removeNotificationGroup::max_notification_id_fieldID;

removeNotificationGroup::removeNotificationGroup()
  : notification_group_id_()
  , max_notification_id_()
{}

removeNotificationGroup::removeNotificationGroup(int32 notification_group_id_, int32 max_notification_id_)
  : notification_group_id_(notification_group_id_)
  , max_notification_id_(max_notification_id_)
{}

const std::int32_t removeNotificationGroup::ID;

object_ptr<removeNotificationGroup> removeNotificationGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeNotificationGroup> res = make_object<removeNotificationGroup>();
  res->notification_group_id_ = env->GetIntField(p, res->notification_group_id_fieldID);
  res->max_notification_id_ = env->GetIntField(p, res->max_notification_id_fieldID);
  return res;
}

void removeNotificationGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveNotificationGroup");
    s.store_field("notificationGroupId", notification_group_id_);
    s.store_field("maxNotificationId", max_notification_id_);
    s.store_class_end();
  }
}

removeNotificationGroup::ReturnType removeNotificationGroup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeNotificationGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveNotificationGroup").c_str());
    notification_group_id_fieldID = jni::get_field_id(env, Class, "notificationGroupId", "I");
    max_notification_id_fieldID = jni::get_field_id(env, Class, "maxNotificationId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass removePendingPaidMessageReactions::Class;
jfieldID removePendingPaidMessageReactions::chat_id_fieldID;
jfieldID removePendingPaidMessageReactions::message_id_fieldID;

removePendingPaidMessageReactions::removePendingPaidMessageReactions()
  : chat_id_()
  , message_id_()
{}

removePendingPaidMessageReactions::removePendingPaidMessageReactions(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t removePendingPaidMessageReactions::ID;

object_ptr<removePendingPaidMessageReactions> removePendingPaidMessageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removePendingPaidMessageReactions> res = make_object<removePendingPaidMessageReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void removePendingPaidMessageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemovePendingPaidMessageReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

removePendingPaidMessageReactions::ReturnType removePendingPaidMessageReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removePendingPaidMessageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemovePendingPaidMessageReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeProxy::Class;
jfieldID removeProxy::proxy_id_fieldID;

removeProxy::removeProxy()
  : proxy_id_()
{}

removeProxy::removeProxy(int32 proxy_id_)
  : proxy_id_(proxy_id_)
{}

const std::int32_t removeProxy::ID;

object_ptr<removeProxy> removeProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeProxy> res = make_object<removeProxy>();
  res->proxy_id_ = env->GetIntField(p, res->proxy_id_fieldID);
  return res;
}

void removeProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveProxy");
    s.store_field("proxyId", proxy_id_);
    s.store_class_end();
  }
}

removeProxy::ReturnType removeProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveProxy").c_str());
    proxy_id_fieldID = jni::get_field_id(env, Class, "proxyId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass removeRecentHashtag::Class;
jfieldID removeRecentHashtag::hashtag_fieldID;

removeRecentHashtag::removeRecentHashtag()
  : hashtag_()
{}

removeRecentHashtag::removeRecentHashtag(string const &hashtag_)
  : hashtag_(hashtag_)
{}

const std::int32_t removeRecentHashtag::ID;

object_ptr<removeRecentHashtag> removeRecentHashtag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeRecentHashtag> res = make_object<removeRecentHashtag>();
  res->hashtag_ = jni::fetch_string(env, p, res->hashtag_fieldID);
  return res;
}

void removeRecentHashtag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveRecentHashtag");
    s.store_field("hashtag", hashtag_);
    s.store_class_end();
  }
}

removeRecentHashtag::ReturnType removeRecentHashtag::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeRecentHashtag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveRecentHashtag").c_str());
    hashtag_fieldID = jni::get_field_id(env, Class, "hashtag", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass removeRecentSticker::Class;
jfieldID removeRecentSticker::is_attached_fieldID;
jfieldID removeRecentSticker::sticker_fieldID;

removeRecentSticker::removeRecentSticker()
  : is_attached_()
  , sticker_()
{}

removeRecentSticker::removeRecentSticker(bool is_attached_, object_ptr<InputFile> &&sticker_)
  : is_attached_(is_attached_)
  , sticker_(std::move(sticker_))
{}

const std::int32_t removeRecentSticker::ID;

object_ptr<removeRecentSticker> removeRecentSticker::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeRecentSticker> res = make_object<removeRecentSticker>();
  res->is_attached_ = (env->GetBooleanField(p, res->is_attached_fieldID) != 0);
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void removeRecentSticker::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveRecentSticker");
    s.store_field("isAttached", is_attached_);
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

removeRecentSticker::ReturnType removeRecentSticker::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeRecentSticker::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveRecentSticker").c_str());
    is_attached_fieldID = jni::get_field_id(env, Class, "isAttached", "Z");
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeRecentlyFoundChat::Class;
jfieldID removeRecentlyFoundChat::chat_id_fieldID;

removeRecentlyFoundChat::removeRecentlyFoundChat()
  : chat_id_()
{}

removeRecentlyFoundChat::removeRecentlyFoundChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t removeRecentlyFoundChat::ID;

object_ptr<removeRecentlyFoundChat> removeRecentlyFoundChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeRecentlyFoundChat> res = make_object<removeRecentlyFoundChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void removeRecentlyFoundChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveRecentlyFoundChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

removeRecentlyFoundChat::ReturnType removeRecentlyFoundChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeRecentlyFoundChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveRecentlyFoundChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeSavedAnimation::Class;
jfieldID removeSavedAnimation::animation_fieldID;

removeSavedAnimation::removeSavedAnimation()
  : animation_()
{}

removeSavedAnimation::removeSavedAnimation(object_ptr<InputFile> &&animation_)
  : animation_(std::move(animation_))
{}

const std::int32_t removeSavedAnimation::ID;

object_ptr<removeSavedAnimation> removeSavedAnimation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeSavedAnimation> res = make_object<removeSavedAnimation>();
  res->animation_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->animation_fieldID));
  return res;
}

void removeSavedAnimation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveSavedAnimation");
    s.store_object_field("animation", static_cast<const BaseObject *>(animation_.get()));
    s.store_class_end();
  }
}

removeSavedAnimation::ReturnType removeSavedAnimation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeSavedAnimation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveSavedAnimation").c_str());
    animation_fieldID = jni::get_field_id(env, Class, "animation", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeSavedNotificationSound::Class;
jfieldID removeSavedNotificationSound::notification_sound_id_fieldID;

removeSavedNotificationSound::removeSavedNotificationSound()
  : notification_sound_id_()
{}

removeSavedNotificationSound::removeSavedNotificationSound(int64 notification_sound_id_)
  : notification_sound_id_(notification_sound_id_)
{}

const std::int32_t removeSavedNotificationSound::ID;

object_ptr<removeSavedNotificationSound> removeSavedNotificationSound::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeSavedNotificationSound> res = make_object<removeSavedNotificationSound>();
  res->notification_sound_id_ = env->GetLongField(p, res->notification_sound_id_fieldID);
  return res;
}

void removeSavedNotificationSound::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveSavedNotificationSound");
    s.store_field("notificationSoundId", notification_sound_id_);
    s.store_class_end();
  }
}

removeSavedNotificationSound::ReturnType removeSavedNotificationSound::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeSavedNotificationSound::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveSavedNotificationSound").c_str());
    notification_sound_id_fieldID = jni::get_field_id(env, Class, "notificationSoundId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass removeSearchedForTag::Class;
jfieldID removeSearchedForTag::tag_fieldID;

removeSearchedForTag::removeSearchedForTag()
  : tag_()
{}

removeSearchedForTag::removeSearchedForTag(string const &tag_)
  : tag_(tag_)
{}

const std::int32_t removeSearchedForTag::ID;

object_ptr<removeSearchedForTag> removeSearchedForTag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeSearchedForTag> res = make_object<removeSearchedForTag>();
  res->tag_ = jni::fetch_string(env, p, res->tag_fieldID);
  return res;
}

void removeSearchedForTag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveSearchedForTag");
    s.store_field("tag", tag_);
    s.store_class_end();
  }
}

removeSearchedForTag::ReturnType removeSearchedForTag::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeSearchedForTag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveSearchedForTag").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass removeStickerFromSet::Class;
jfieldID removeStickerFromSet::sticker_fieldID;

removeStickerFromSet::removeStickerFromSet()
  : sticker_()
{}

removeStickerFromSet::removeStickerFromSet(object_ptr<InputFile> &&sticker_)
  : sticker_(std::move(sticker_))
{}

const std::int32_t removeStickerFromSet::ID;

object_ptr<removeStickerFromSet> removeStickerFromSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeStickerFromSet> res = make_object<removeStickerFromSet>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void removeStickerFromSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveStickerFromSet");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

removeStickerFromSet::ReturnType removeStickerFromSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeStickerFromSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveStickerFromSet").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass removeTopChat::Class;
jfieldID removeTopChat::category_fieldID;
jfieldID removeTopChat::chat_id_fieldID;

removeTopChat::removeTopChat()
  : category_()
  , chat_id_()
{}

removeTopChat::removeTopChat(object_ptr<TopChatCategory> &&category_, int53 chat_id_)
  : category_(std::move(category_))
  , chat_id_(chat_id_)
{}

const std::int32_t removeTopChat::ID;

object_ptr<removeTopChat> removeTopChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<removeTopChat> res = make_object<removeTopChat>();
  res->category_ = jni::fetch_tl_object<TopChatCategory>(env, jni::fetch_object(env, p, res->category_fieldID));
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void removeTopChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RemoveTopChat");
    s.store_object_field("category", static_cast<const BaseObject *>(category_.get()));
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

removeTopChat::ReturnType removeTopChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void removeTopChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RemoveTopChat").c_str());
    category_fieldID = jni::get_field_id(env, Class, "category", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TopChatCategory;").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass reorderActiveUsernames::Class;
jfieldID reorderActiveUsernames::usernames_fieldID;

reorderActiveUsernames::reorderActiveUsernames()
  : usernames_()
{}

reorderActiveUsernames::reorderActiveUsernames(array<string> &&usernames_)
  : usernames_(std::move(usernames_))
{}

const std::int32_t reorderActiveUsernames::ID;

object_ptr<reorderActiveUsernames> reorderActiveUsernames::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderActiveUsernames> res = make_object<reorderActiveUsernames>();
  res->usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->usernames_fieldID));
  return res;
}

void reorderActiveUsernames::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderActiveUsernames");
    { s.store_vector_begin("usernames", usernames_.size()); for (const auto &_value : usernames_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderActiveUsernames::ReturnType reorderActiveUsernames::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderActiveUsernames::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderActiveUsernames").c_str());
    usernames_fieldID = jni::get_field_id(env, Class, "usernames", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reorderBotActiveUsernames::Class;
jfieldID reorderBotActiveUsernames::bot_user_id_fieldID;
jfieldID reorderBotActiveUsernames::usernames_fieldID;

reorderBotActiveUsernames::reorderBotActiveUsernames()
  : bot_user_id_()
  , usernames_()
{}

reorderBotActiveUsernames::reorderBotActiveUsernames(int53 bot_user_id_, array<string> &&usernames_)
  : bot_user_id_(bot_user_id_)
  , usernames_(std::move(usernames_))
{}

const std::int32_t reorderBotActiveUsernames::ID;

object_ptr<reorderBotActiveUsernames> reorderBotActiveUsernames::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderBotActiveUsernames> res = make_object<reorderBotActiveUsernames>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->usernames_fieldID));
  return res;
}

void reorderBotActiveUsernames::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderBotActiveUsernames");
    s.store_field("botUserId", bot_user_id_);
    { s.store_vector_begin("usernames", usernames_.size()); for (const auto &_value : usernames_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderBotActiveUsernames::ReturnType reorderBotActiveUsernames::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderBotActiveUsernames::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderBotActiveUsernames").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    usernames_fieldID = jni::get_field_id(env, Class, "usernames", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reorderBotMediaPreviews::Class;
jfieldID reorderBotMediaPreviews::bot_user_id_fieldID;
jfieldID reorderBotMediaPreviews::language_code_fieldID;
jfieldID reorderBotMediaPreviews::file_ids_fieldID;

reorderBotMediaPreviews::reorderBotMediaPreviews()
  : bot_user_id_()
  , language_code_()
  , file_ids_()
{}

reorderBotMediaPreviews::reorderBotMediaPreviews(int53 bot_user_id_, string const &language_code_, array<int32> &&file_ids_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , file_ids_(std::move(file_ids_))
{}

const std::int32_t reorderBotMediaPreviews::ID;

object_ptr<reorderBotMediaPreviews> reorderBotMediaPreviews::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderBotMediaPreviews> res = make_object<reorderBotMediaPreviews>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->file_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->file_ids_fieldID));
  return res;
}

void reorderBotMediaPreviews::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderBotMediaPreviews");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    { s.store_vector_begin("fileIds", file_ids_.size()); for (const auto &_value : file_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderBotMediaPreviews::ReturnType reorderBotMediaPreviews::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderBotMediaPreviews::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderBotMediaPreviews").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    file_ids_fieldID = jni::get_field_id(env, Class, "fileIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass reorderChatFolders::Class;
jfieldID reorderChatFolders::chat_folder_ids_fieldID;
jfieldID reorderChatFolders::main_chat_list_position_fieldID;

reorderChatFolders::reorderChatFolders()
  : chat_folder_ids_()
  , main_chat_list_position_()
{}

reorderChatFolders::reorderChatFolders(array<int32> &&chat_folder_ids_, int32 main_chat_list_position_)
  : chat_folder_ids_(std::move(chat_folder_ids_))
  , main_chat_list_position_(main_chat_list_position_)
{}

const std::int32_t reorderChatFolders::ID;

object_ptr<reorderChatFolders> reorderChatFolders::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderChatFolders> res = make_object<reorderChatFolders>();
  res->chat_folder_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->chat_folder_ids_fieldID));
  res->main_chat_list_position_ = env->GetIntField(p, res->main_chat_list_position_fieldID);
  return res;
}

void reorderChatFolders::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderChatFolders");
    { s.store_vector_begin("chatFolderIds", chat_folder_ids_.size()); for (const auto &_value : chat_folder_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("mainChatListPosition", main_chat_list_position_);
    s.store_class_end();
  }
}

reorderChatFolders::ReturnType reorderChatFolders::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderChatFolders::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderChatFolders").c_str());
    chat_folder_ids_fieldID = jni::get_field_id(env, Class, "chatFolderIds", "[I");
    main_chat_list_position_fieldID = jni::get_field_id(env, Class, "mainChatListPosition", "I");
    return true;
  }();
  (void)is_inited;
}

jclass reorderInstalledStickerSets::Class;
jfieldID reorderInstalledStickerSets::sticker_type_fieldID;
jfieldID reorderInstalledStickerSets::sticker_set_ids_fieldID;

reorderInstalledStickerSets::reorderInstalledStickerSets()
  : sticker_type_()
  , sticker_set_ids_()
{}

reorderInstalledStickerSets::reorderInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, array<int64> &&sticker_set_ids_)
  : sticker_type_(std::move(sticker_type_))
  , sticker_set_ids_(std::move(sticker_set_ids_))
{}

const std::int32_t reorderInstalledStickerSets::ID;

object_ptr<reorderInstalledStickerSets> reorderInstalledStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderInstalledStickerSets> res = make_object<reorderInstalledStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->sticker_set_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->sticker_set_ids_fieldID));
  return res;
}

void reorderInstalledStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderInstalledStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    { s.store_vector_begin("stickerSetIds", sticker_set_ids_.size()); for (const auto &_value : sticker_set_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderInstalledStickerSets::ReturnType reorderInstalledStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderInstalledStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderInstalledStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    sticker_set_ids_fieldID = jni::get_field_id(env, Class, "stickerSetIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass reorderQuickReplyShortcuts::Class;
jfieldID reorderQuickReplyShortcuts::shortcut_ids_fieldID;

reorderQuickReplyShortcuts::reorderQuickReplyShortcuts()
  : shortcut_ids_()
{}

reorderQuickReplyShortcuts::reorderQuickReplyShortcuts(array<int32> &&shortcut_ids_)
  : shortcut_ids_(std::move(shortcut_ids_))
{}

const std::int32_t reorderQuickReplyShortcuts::ID;

object_ptr<reorderQuickReplyShortcuts> reorderQuickReplyShortcuts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderQuickReplyShortcuts> res = make_object<reorderQuickReplyShortcuts>();
  res->shortcut_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->shortcut_ids_fieldID));
  return res;
}

void reorderQuickReplyShortcuts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderQuickReplyShortcuts");
    { s.store_vector_begin("shortcutIds", shortcut_ids_.size()); for (const auto &_value : shortcut_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderQuickReplyShortcuts::ReturnType reorderQuickReplyShortcuts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderQuickReplyShortcuts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderQuickReplyShortcuts").c_str());
    shortcut_ids_fieldID = jni::get_field_id(env, Class, "shortcutIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass reorderSupergroupActiveUsernames::Class;
jfieldID reorderSupergroupActiveUsernames::supergroup_id_fieldID;
jfieldID reorderSupergroupActiveUsernames::usernames_fieldID;

reorderSupergroupActiveUsernames::reorderSupergroupActiveUsernames()
  : supergroup_id_()
  , usernames_()
{}

reorderSupergroupActiveUsernames::reorderSupergroupActiveUsernames(int53 supergroup_id_, array<string> &&usernames_)
  : supergroup_id_(supergroup_id_)
  , usernames_(std::move(usernames_))
{}

const std::int32_t reorderSupergroupActiveUsernames::ID;

object_ptr<reorderSupergroupActiveUsernames> reorderSupergroupActiveUsernames::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reorderSupergroupActiveUsernames> res = make_object<reorderSupergroupActiveUsernames>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->usernames_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->usernames_fieldID));
  return res;
}

void reorderSupergroupActiveUsernames::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReorderSupergroupActiveUsernames");
    s.store_field("supergroupId", supergroup_id_);
    { s.store_vector_begin("usernames", usernames_.size()); for (const auto &_value : usernames_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reorderSupergroupActiveUsernames::ReturnType reorderSupergroupActiveUsernames::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reorderSupergroupActiveUsernames::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReorderSupergroupActiveUsernames").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    usernames_fieldID = jni::get_field_id(env, Class, "usernames", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass replacePrimaryChatInviteLink::Class;
jfieldID replacePrimaryChatInviteLink::chat_id_fieldID;

replacePrimaryChatInviteLink::replacePrimaryChatInviteLink()
  : chat_id_()
{}

replacePrimaryChatInviteLink::replacePrimaryChatInviteLink(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t replacePrimaryChatInviteLink::ID;

object_ptr<replacePrimaryChatInviteLink> replacePrimaryChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replacePrimaryChatInviteLink> res = make_object<replacePrimaryChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void replacePrimaryChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplacePrimaryChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

replacePrimaryChatInviteLink::ReturnType replacePrimaryChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLink::fetch(env, p);
}

void replacePrimaryChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplacePrimaryChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass replaceStickerInSet::Class;
jfieldID replaceStickerInSet::user_id_fieldID;
jfieldID replaceStickerInSet::name_fieldID;
jfieldID replaceStickerInSet::old_sticker_fieldID;
jfieldID replaceStickerInSet::new_sticker_fieldID;

replaceStickerInSet::replaceStickerInSet()
  : user_id_()
  , name_()
  , old_sticker_()
  , new_sticker_()
{}

replaceStickerInSet::replaceStickerInSet(int53 user_id_, string const &name_, object_ptr<InputFile> &&old_sticker_, object_ptr<inputSticker> &&new_sticker_)
  : user_id_(user_id_)
  , name_(name_)
  , old_sticker_(std::move(old_sticker_))
  , new_sticker_(std::move(new_sticker_))
{}

const std::int32_t replaceStickerInSet::ID;

object_ptr<replaceStickerInSet> replaceStickerInSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replaceStickerInSet> res = make_object<replaceStickerInSet>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->old_sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->old_sticker_fieldID));
  res->new_sticker_ = jni::fetch_tl_object<inputSticker>(env, jni::fetch_object(env, p, res->new_sticker_fieldID));
  return res;
}

void replaceStickerInSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplaceStickerInSet");
    s.store_field("userId", user_id_);
    s.store_field("name", name_);
    s.store_object_field("oldSticker", static_cast<const BaseObject *>(old_sticker_.get()));
    s.store_object_field("newSticker", static_cast<const BaseObject *>(new_sticker_.get()));
    s.store_class_end();
  }
}

replaceStickerInSet::ReturnType replaceStickerInSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void replaceStickerInSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplaceStickerInSet").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    old_sticker_fieldID = jni::get_field_id(env, Class, "oldSticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    new_sticker_fieldID = jni::get_field_id(env, Class, "newSticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputSticker;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass replaceVideoChatRtmpUrl::Class;
jfieldID replaceVideoChatRtmpUrl::chat_id_fieldID;

replaceVideoChatRtmpUrl::replaceVideoChatRtmpUrl()
  : chat_id_()
{}

replaceVideoChatRtmpUrl::replaceVideoChatRtmpUrl(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t replaceVideoChatRtmpUrl::ID;

object_ptr<replaceVideoChatRtmpUrl> replaceVideoChatRtmpUrl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<replaceVideoChatRtmpUrl> res = make_object<replaceVideoChatRtmpUrl>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void replaceVideoChatRtmpUrl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReplaceVideoChatRtmpUrl");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

replaceVideoChatRtmpUrl::ReturnType replaceVideoChatRtmpUrl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return rtmpUrl::fetch(env, p);
}

void replaceVideoChatRtmpUrl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReplaceVideoChatRtmpUrl").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass reportAuthenticationCodeMissing::Class;
jfieldID reportAuthenticationCodeMissing::mobile_network_code_fieldID;

reportAuthenticationCodeMissing::reportAuthenticationCodeMissing()
  : mobile_network_code_()
{}

reportAuthenticationCodeMissing::reportAuthenticationCodeMissing(string const &mobile_network_code_)
  : mobile_network_code_(mobile_network_code_)
{}

const std::int32_t reportAuthenticationCodeMissing::ID;

object_ptr<reportAuthenticationCodeMissing> reportAuthenticationCodeMissing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportAuthenticationCodeMissing> res = make_object<reportAuthenticationCodeMissing>();
  res->mobile_network_code_ = jni::fetch_string(env, p, res->mobile_network_code_fieldID);
  return res;
}

void reportAuthenticationCodeMissing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportAuthenticationCodeMissing");
    s.store_field("mobileNetworkCode", mobile_network_code_);
    s.store_class_end();
  }
}

reportAuthenticationCodeMissing::ReturnType reportAuthenticationCodeMissing::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportAuthenticationCodeMissing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportAuthenticationCodeMissing").c_str());
    mobile_network_code_fieldID = jni::get_field_id(env, Class, "mobileNetworkCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reportChat::Class;
jfieldID reportChat::chat_id_fieldID;
jfieldID reportChat::option_id_fieldID;
jfieldID reportChat::message_ids_fieldID;
jfieldID reportChat::text_fieldID;

reportChat::reportChat()
  : chat_id_()
  , option_id_()
  , message_ids_()
  , text_()
{}

reportChat::reportChat(int53 chat_id_, bytes const &option_id_, array<int53> &&message_ids_, string const &text_)
  : chat_id_(chat_id_)
  , option_id_(std::move(option_id_))
  , message_ids_(std::move(message_ids_))
  , text_(text_)
{}

const std::int32_t reportChat::ID;

object_ptr<reportChat> reportChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportChat> res = make_object<reportChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void reportChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChat");
    s.store_field("chatId", chat_id_);
    s.store_bytes_field("optionId", option_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("text", text_);
    s.store_class_end();
  }
}

reportChat::ReturnType reportChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ReportChatResult::fetch(env, p);
}

void reportChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reportChatPhoto::Class;
jfieldID reportChatPhoto::chat_id_fieldID;
jfieldID reportChatPhoto::file_id_fieldID;
jfieldID reportChatPhoto::reason_fieldID;
jfieldID reportChatPhoto::text_fieldID;

reportChatPhoto::reportChatPhoto()
  : chat_id_()
  , file_id_()
  , reason_()
  , text_()
{}

reportChatPhoto::reportChatPhoto(int53 chat_id_, int32 file_id_, object_ptr<ReportReason> &&reason_, string const &text_)
  : chat_id_(chat_id_)
  , file_id_(file_id_)
  , reason_(std::move(reason_))
  , text_(text_)
{}

const std::int32_t reportChatPhoto::ID;

object_ptr<reportChatPhoto> reportChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportChatPhoto> res = make_object<reportChatPhoto>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->reason_ = jni::fetch_tl_object<ReportReason>(env, jni::fetch_object(env, p, res->reason_fieldID));
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void reportChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatPhoto");
    s.store_field("chatId", chat_id_);
    s.store_field("fileId", file_id_);
    s.store_object_field("reason", static_cast<const BaseObject *>(reason_.get()));
    s.store_field("text", text_);
    s.store_class_end();
  }
}

reportChatPhoto::ReturnType reportChatPhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatPhoto").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    reason_fieldID = jni::get_field_id(env, Class, "reason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReportReason;").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reportChatSponsoredMessage::Class;
jfieldID reportChatSponsoredMessage::chat_id_fieldID;
jfieldID reportChatSponsoredMessage::message_id_fieldID;
jfieldID reportChatSponsoredMessage::option_id_fieldID;

reportChatSponsoredMessage::reportChatSponsoredMessage()
  : chat_id_()
  , message_id_()
  , option_id_()
{}

reportChatSponsoredMessage::reportChatSponsoredMessage(int53 chat_id_, int53 message_id_, bytes const &option_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , option_id_(std::move(option_id_))
{}

const std::int32_t reportChatSponsoredMessage::ID;

object_ptr<reportChatSponsoredMessage> reportChatSponsoredMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportChatSponsoredMessage> res = make_object<reportChatSponsoredMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  return res;
}

void reportChatSponsoredMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportChatSponsoredMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_bytes_field("optionId", option_id_);
    s.store_class_end();
  }
}

reportChatSponsoredMessage::ReturnType reportChatSponsoredMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ReportSponsoredResult::fetch(env, p);
}

void reportChatSponsoredMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportChatSponsoredMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass reportMessageReactions::Class;
jfieldID reportMessageReactions::chat_id_fieldID;
jfieldID reportMessageReactions::message_id_fieldID;
jfieldID reportMessageReactions::sender_id_fieldID;

reportMessageReactions::reportMessageReactions()
  : chat_id_()
  , message_id_()
  , sender_id_()
{}

reportMessageReactions::reportMessageReactions(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&sender_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , sender_id_(std::move(sender_id_))
{}

const std::int32_t reportMessageReactions::ID;

object_ptr<reportMessageReactions> reportMessageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportMessageReactions> res = make_object<reportMessageReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  return res;
}

void reportMessageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportMessageReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_class_end();
  }
}

reportMessageReactions::ReturnType reportMessageReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportMessageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportMessageReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reportPhoneNumberCodeMissing::Class;
jfieldID reportPhoneNumberCodeMissing::mobile_network_code_fieldID;

reportPhoneNumberCodeMissing::reportPhoneNumberCodeMissing()
  : mobile_network_code_()
{}

reportPhoneNumberCodeMissing::reportPhoneNumberCodeMissing(string const &mobile_network_code_)
  : mobile_network_code_(mobile_network_code_)
{}

const std::int32_t reportPhoneNumberCodeMissing::ID;

object_ptr<reportPhoneNumberCodeMissing> reportPhoneNumberCodeMissing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportPhoneNumberCodeMissing> res = make_object<reportPhoneNumberCodeMissing>();
  res->mobile_network_code_ = jni::fetch_string(env, p, res->mobile_network_code_fieldID);
  return res;
}

void reportPhoneNumberCodeMissing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportPhoneNumberCodeMissing");
    s.store_field("mobileNetworkCode", mobile_network_code_);
    s.store_class_end();
  }
}

reportPhoneNumberCodeMissing::ReturnType reportPhoneNumberCodeMissing::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportPhoneNumberCodeMissing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportPhoneNumberCodeMissing").c_str());
    mobile_network_code_fieldID = jni::get_field_id(env, Class, "mobileNetworkCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reportSponsoredChat::Class;
jfieldID reportSponsoredChat::sponsored_chat_unique_id_fieldID;
jfieldID reportSponsoredChat::option_id_fieldID;

reportSponsoredChat::reportSponsoredChat()
  : sponsored_chat_unique_id_()
  , option_id_()
{}

reportSponsoredChat::reportSponsoredChat(int53 sponsored_chat_unique_id_, bytes const &option_id_)
  : sponsored_chat_unique_id_(sponsored_chat_unique_id_)
  , option_id_(std::move(option_id_))
{}

const std::int32_t reportSponsoredChat::ID;

object_ptr<reportSponsoredChat> reportSponsoredChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportSponsoredChat> res = make_object<reportSponsoredChat>();
  res->sponsored_chat_unique_id_ = env->GetLongField(p, res->sponsored_chat_unique_id_fieldID);
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  return res;
}

void reportSponsoredChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSponsoredChat");
    s.store_field("sponsoredChatUniqueId", sponsored_chat_unique_id_);
    s.store_bytes_field("optionId", option_id_);
    s.store_class_end();
  }
}

reportSponsoredChat::ReturnType reportSponsoredChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ReportSponsoredResult::fetch(env, p);
}

void reportSponsoredChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSponsoredChat").c_str());
    sponsored_chat_unique_id_fieldID = jni::get_field_id(env, Class, "sponsoredChatUniqueId", "J");
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass reportStory::Class;
jfieldID reportStory::story_poster_chat_id_fieldID;
jfieldID reportStory::story_id_fieldID;
jfieldID reportStory::option_id_fieldID;
jfieldID reportStory::text_fieldID;

reportStory::reportStory()
  : story_poster_chat_id_()
  , story_id_()
  , option_id_()
  , text_()
{}

reportStory::reportStory(int53 story_poster_chat_id_, int32 story_id_, bytes const &option_id_, string const &text_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , option_id_(std::move(option_id_))
  , text_(text_)
{}

const std::int32_t reportStory::ID;

object_ptr<reportStory> reportStory::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportStory> res = make_object<reportStory>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->option_id_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->option_id_fieldID));
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  return res;
}

void reportStory::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportStory");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_bytes_field("optionId", option_id_);
    s.store_field("text", text_);
    s.store_class_end();
  }
}

reportStory::ReturnType reportStory::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ReportStoryResult::fetch(env, p);
}

void reportStory::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportStory").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    option_id_fieldID = jni::get_field_id(env, Class, "optionId", "[B");
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass reportSupergroupAntiSpamFalsePositive::Class;
jfieldID reportSupergroupAntiSpamFalsePositive::supergroup_id_fieldID;
jfieldID reportSupergroupAntiSpamFalsePositive::message_id_fieldID;

reportSupergroupAntiSpamFalsePositive::reportSupergroupAntiSpamFalsePositive()
  : supergroup_id_()
  , message_id_()
{}

reportSupergroupAntiSpamFalsePositive::reportSupergroupAntiSpamFalsePositive(int53 supergroup_id_, int53 message_id_)
  : supergroup_id_(supergroup_id_)
  , message_id_(message_id_)
{}

const std::int32_t reportSupergroupAntiSpamFalsePositive::ID;

object_ptr<reportSupergroupAntiSpamFalsePositive> reportSupergroupAntiSpamFalsePositive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportSupergroupAntiSpamFalsePositive> res = make_object<reportSupergroupAntiSpamFalsePositive>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void reportSupergroupAntiSpamFalsePositive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSupergroupAntiSpamFalsePositive");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

reportSupergroupAntiSpamFalsePositive::ReturnType reportSupergroupAntiSpamFalsePositive::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportSupergroupAntiSpamFalsePositive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSupergroupAntiSpamFalsePositive").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass reportSupergroupSpam::Class;
jfieldID reportSupergroupSpam::supergroup_id_fieldID;
jfieldID reportSupergroupSpam::message_ids_fieldID;

reportSupergroupSpam::reportSupergroupSpam()
  : supergroup_id_()
  , message_ids_()
{}

reportSupergroupSpam::reportSupergroupSpam(int53 supergroup_id_, array<int53> &&message_ids_)
  : supergroup_id_(supergroup_id_)
  , message_ids_(std::move(message_ids_))
{}

const std::int32_t reportSupergroupSpam::ID;

object_ptr<reportSupergroupSpam> reportSupergroupSpam::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reportSupergroupSpam> res = make_object<reportSupergroupSpam>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  return res;
}

void reportSupergroupSpam::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReportSupergroupSpam");
    s.store_field("supergroupId", supergroup_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

reportSupergroupSpam::ReturnType reportSupergroupSpam::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reportSupergroupSpam::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReportSupergroupSpam").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass requestAuthenticationPasswordRecovery::Class;

requestAuthenticationPasswordRecovery::requestAuthenticationPasswordRecovery() {
}

const std::int32_t requestAuthenticationPasswordRecovery::ID;

object_ptr<requestAuthenticationPasswordRecovery> requestAuthenticationPasswordRecovery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<requestAuthenticationPasswordRecovery>();
}

void requestAuthenticationPasswordRecovery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RequestAuthenticationPasswordRecovery");
    s.store_class_end();
  }
}

requestAuthenticationPasswordRecovery::ReturnType requestAuthenticationPasswordRecovery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void requestAuthenticationPasswordRecovery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RequestAuthenticationPasswordRecovery").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass requestPasswordRecovery::Class;

requestPasswordRecovery::requestPasswordRecovery() {
}

const std::int32_t requestPasswordRecovery::ID;

object_ptr<requestPasswordRecovery> requestPasswordRecovery::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<requestPasswordRecovery>();
}

void requestPasswordRecovery::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RequestPasswordRecovery");
    s.store_class_end();
  }
}

requestPasswordRecovery::ReturnType requestPasswordRecovery::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emailAddressAuthenticationCodeInfo::fetch(env, p);
}

void requestPasswordRecovery::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RequestPasswordRecovery").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass requestQrCodeAuthentication::Class;
jfieldID requestQrCodeAuthentication::other_user_ids_fieldID;

requestQrCodeAuthentication::requestQrCodeAuthentication()
  : other_user_ids_()
{}

requestQrCodeAuthentication::requestQrCodeAuthentication(array<int53> &&other_user_ids_)
  : other_user_ids_(std::move(other_user_ids_))
{}

const std::int32_t requestQrCodeAuthentication::ID;

object_ptr<requestQrCodeAuthentication> requestQrCodeAuthentication::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<requestQrCodeAuthentication> res = make_object<requestQrCodeAuthentication>();
  res->other_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->other_user_ids_fieldID));
  return res;
}

void requestQrCodeAuthentication::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RequestQrCodeAuthentication");
    { s.store_vector_begin("otherUserIds", other_user_ids_.size()); for (const auto &_value : other_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

requestQrCodeAuthentication::ReturnType requestQrCodeAuthentication::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void requestQrCodeAuthentication::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RequestQrCodeAuthentication").c_str());
    other_user_ids_fieldID = jni::get_field_id(env, Class, "otherUserIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass resendAuthenticationCode::Class;
jfieldID resendAuthenticationCode::reason_fieldID;

resendAuthenticationCode::resendAuthenticationCode()
  : reason_()
{}

resendAuthenticationCode::resendAuthenticationCode(object_ptr<ResendCodeReason> &&reason_)
  : reason_(std::move(reason_))
{}

const std::int32_t resendAuthenticationCode::ID;

object_ptr<resendAuthenticationCode> resendAuthenticationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resendAuthenticationCode> res = make_object<resendAuthenticationCode>();
  res->reason_ = jni::fetch_tl_object<ResendCodeReason>(env, jni::fetch_object(env, p, res->reason_fieldID));
  return res;
}

void resendAuthenticationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendAuthenticationCode");
    s.store_object_field("reason", static_cast<const BaseObject *>(reason_.get()));
    s.store_class_end();
  }
}

resendAuthenticationCode::ReturnType resendAuthenticationCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void resendAuthenticationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendAuthenticationCode").c_str());
    reason_fieldID = jni::get_field_id(env, Class, "reason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ResendCodeReason;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendEmailAddressVerificationCode::Class;

resendEmailAddressVerificationCode::resendEmailAddressVerificationCode() {
}

const std::int32_t resendEmailAddressVerificationCode::ID;

object_ptr<resendEmailAddressVerificationCode> resendEmailAddressVerificationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resendEmailAddressVerificationCode>();
}

void resendEmailAddressVerificationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendEmailAddressVerificationCode");
    s.store_class_end();
  }
}

resendEmailAddressVerificationCode::ReturnType resendEmailAddressVerificationCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emailAddressAuthenticationCodeInfo::fetch(env, p);
}

void resendEmailAddressVerificationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendEmailAddressVerificationCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendLoginEmailAddressCode::Class;

resendLoginEmailAddressCode::resendLoginEmailAddressCode() {
}

const std::int32_t resendLoginEmailAddressCode::ID;

object_ptr<resendLoginEmailAddressCode> resendLoginEmailAddressCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resendLoginEmailAddressCode>();
}

void resendLoginEmailAddressCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendLoginEmailAddressCode");
    s.store_class_end();
  }
}

resendLoginEmailAddressCode::ReturnType resendLoginEmailAddressCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emailAddressAuthenticationCodeInfo::fetch(env, p);
}

void resendLoginEmailAddressCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendLoginEmailAddressCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendMessages::Class;
jfieldID resendMessages::chat_id_fieldID;
jfieldID resendMessages::message_ids_fieldID;
jfieldID resendMessages::quote_fieldID;
jfieldID resendMessages::paid_message_star_count_fieldID;

resendMessages::resendMessages()
  : chat_id_()
  , message_ids_()
  , quote_()
  , paid_message_star_count_()
{}

resendMessages::resendMessages(int53 chat_id_, array<int53> &&message_ids_, object_ptr<inputTextQuote> &&quote_, int53 paid_message_star_count_)
  : chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
  , quote_(std::move(quote_))
  , paid_message_star_count_(paid_message_star_count_)
{}

const std::int32_t resendMessages::ID;

object_ptr<resendMessages> resendMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resendMessages> res = make_object<resendMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->quote_ = jni::fetch_tl_object<inputTextQuote>(env, jni::fetch_object(env, p, res->quote_fieldID));
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  return res;
}

void resendMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendMessages");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_object_field("quote", static_cast<const BaseObject *>(quote_.get()));
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_class_end();
  }
}

resendMessages::ReturnType resendMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void resendMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    quote_fieldID = jni::get_field_id(env, Class, "quote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputTextQuote;").c_str());
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass resendPhoneNumberCode::Class;
jfieldID resendPhoneNumberCode::reason_fieldID;

resendPhoneNumberCode::resendPhoneNumberCode()
  : reason_()
{}

resendPhoneNumberCode::resendPhoneNumberCode(object_ptr<ResendCodeReason> &&reason_)
  : reason_(std::move(reason_))
{}

const std::int32_t resendPhoneNumberCode::ID;

object_ptr<resendPhoneNumberCode> resendPhoneNumberCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<resendPhoneNumberCode> res = make_object<resendPhoneNumberCode>();
  res->reason_ = jni::fetch_tl_object<ResendCodeReason>(env, jni::fetch_object(env, p, res->reason_fieldID));
  return res;
}

void resendPhoneNumberCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendPhoneNumberCode");
    s.store_object_field("reason", static_cast<const BaseObject *>(reason_.get()));
    s.store_class_end();
  }
}

resendPhoneNumberCode::ReturnType resendPhoneNumberCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return authenticationCodeInfo::fetch(env, p);
}

void resendPhoneNumberCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendPhoneNumberCode").c_str());
    reason_fieldID = jni::get_field_id(env, Class, "reason", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ResendCodeReason;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resendRecoveryEmailAddressCode::Class;

resendRecoveryEmailAddressCode::resendRecoveryEmailAddressCode() {
}

const std::int32_t resendRecoveryEmailAddressCode::ID;

object_ptr<resendRecoveryEmailAddressCode> resendRecoveryEmailAddressCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resendRecoveryEmailAddressCode>();
}

void resendRecoveryEmailAddressCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResendRecoveryEmailAddressCode");
    s.store_class_end();
  }
}

resendRecoveryEmailAddressCode::ReturnType resendRecoveryEmailAddressCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void resendRecoveryEmailAddressCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResendRecoveryEmailAddressCode").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetAllNotificationSettings::Class;

resetAllNotificationSettings::resetAllNotificationSettings() {
}

const std::int32_t resetAllNotificationSettings::ID;

object_ptr<resetAllNotificationSettings> resetAllNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetAllNotificationSettings>();
}

void resetAllNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetAllNotificationSettings");
    s.store_class_end();
  }
}

resetAllNotificationSettings::ReturnType resetAllNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void resetAllNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetAllNotificationSettings").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetAuthenticationEmailAddress::Class;

resetAuthenticationEmailAddress::resetAuthenticationEmailAddress() {
}

const std::int32_t resetAuthenticationEmailAddress::ID;

object_ptr<resetAuthenticationEmailAddress> resetAuthenticationEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetAuthenticationEmailAddress>();
}

void resetAuthenticationEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetAuthenticationEmailAddress");
    s.store_class_end();
  }
}

resetAuthenticationEmailAddress::ReturnType resetAuthenticationEmailAddress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void resetAuthenticationEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetAuthenticationEmailAddress").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetInstalledBackgrounds::Class;

resetInstalledBackgrounds::resetInstalledBackgrounds() {
}

const std::int32_t resetInstalledBackgrounds::ID;

object_ptr<resetInstalledBackgrounds> resetInstalledBackgrounds::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetInstalledBackgrounds>();
}

void resetInstalledBackgrounds::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetInstalledBackgrounds");
    s.store_class_end();
  }
}

resetInstalledBackgrounds::ReturnType resetInstalledBackgrounds::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void resetInstalledBackgrounds::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetInstalledBackgrounds").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetNetworkStatistics::Class;

resetNetworkStatistics::resetNetworkStatistics() {
}

const std::int32_t resetNetworkStatistics::ID;

object_ptr<resetNetworkStatistics> resetNetworkStatistics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetNetworkStatistics>();
}

void resetNetworkStatistics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetNetworkStatistics");
    s.store_class_end();
  }
}

resetNetworkStatistics::ReturnType resetNetworkStatistics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void resetNetworkStatistics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetNetworkStatistics").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass resetPassword::Class;

resetPassword::resetPassword() {
}

const std::int32_t resetPassword::ID;

object_ptr<resetPassword> resetPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<resetPassword>();
}

void resetPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ResetPassword");
    s.store_class_end();
  }
}

resetPassword::ReturnType resetPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ResetPasswordResult::fetch(env, p);
}

void resetPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ResetPassword").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass reuseStarSubscription::Class;
jfieldID reuseStarSubscription::subscription_id_fieldID;

reuseStarSubscription::reuseStarSubscription()
  : subscription_id_()
{}

reuseStarSubscription::reuseStarSubscription(string const &subscription_id_)
  : subscription_id_(subscription_id_)
{}

const std::int32_t reuseStarSubscription::ID;

object_ptr<reuseStarSubscription> reuseStarSubscription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<reuseStarSubscription> res = make_object<reuseStarSubscription>();
  res->subscription_id_ = jni::fetch_string(env, p, res->subscription_id_fieldID);
  return res;
}

void reuseStarSubscription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ReuseStarSubscription");
    s.store_field("subscriptionId", subscription_id_);
    s.store_class_end();
  }
}

reuseStarSubscription::ReturnType reuseStarSubscription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void reuseStarSubscription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ReuseStarSubscription").c_str());
    subscription_id_fieldID = jni::get_field_id(env, Class, "subscriptionId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass revokeChatInviteLink::Class;
jfieldID revokeChatInviteLink::chat_id_fieldID;
jfieldID revokeChatInviteLink::invite_link_fieldID;

revokeChatInviteLink::revokeChatInviteLink()
  : chat_id_()
  , invite_link_()
{}

revokeChatInviteLink::revokeChatInviteLink(int53 chat_id_, string const &invite_link_)
  : chat_id_(chat_id_)
  , invite_link_(invite_link_)
{}

const std::int32_t revokeChatInviteLink::ID;

object_ptr<revokeChatInviteLink> revokeChatInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<revokeChatInviteLink> res = make_object<revokeChatInviteLink>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->invite_link_ = jni::fetch_string(env, p, res->invite_link_fieldID);
  return res;
}

void revokeChatInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RevokeChatInviteLink");
    s.store_field("chatId", chat_id_);
    s.store_field("inviteLink", invite_link_);
    s.store_class_end();
  }
}

revokeChatInviteLink::ReturnType revokeChatInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatInviteLinks::fetch(env, p);
}

void revokeChatInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevokeChatInviteLink").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    invite_link_fieldID = jni::get_field_id(env, Class, "inviteLink", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass revokeGroupCallInviteLink::Class;
jfieldID revokeGroupCallInviteLink::group_call_id_fieldID;

revokeGroupCallInviteLink::revokeGroupCallInviteLink()
  : group_call_id_()
{}

revokeGroupCallInviteLink::revokeGroupCallInviteLink(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t revokeGroupCallInviteLink::ID;

object_ptr<revokeGroupCallInviteLink> revokeGroupCallInviteLink::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<revokeGroupCallInviteLink> res = make_object<revokeGroupCallInviteLink>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void revokeGroupCallInviteLink::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "RevokeGroupCallInviteLink");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

revokeGroupCallInviteLink::ReturnType revokeGroupCallInviteLink::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void revokeGroupCallInviteLink::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$RevokeGroupCallInviteLink").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass saveApplicationLogEvent::Class;
jfieldID saveApplicationLogEvent::type_fieldID;
jfieldID saveApplicationLogEvent::chat_id_fieldID;
jfieldID saveApplicationLogEvent::data_fieldID;

saveApplicationLogEvent::saveApplicationLogEvent()
  : type_()
  , chat_id_()
  , data_()
{}

saveApplicationLogEvent::saveApplicationLogEvent(string const &type_, int53 chat_id_, object_ptr<JsonValue> &&data_)
  : type_(type_)
  , chat_id_(chat_id_)
  , data_(std::move(data_))
{}

const std::int32_t saveApplicationLogEvent::ID;

object_ptr<saveApplicationLogEvent> saveApplicationLogEvent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<saveApplicationLogEvent> res = make_object<saveApplicationLogEvent>();
  res->type_ = jni::fetch_string(env, p, res->type_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->data_ = jni::fetch_tl_object<JsonValue>(env, jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void saveApplicationLogEvent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SaveApplicationLogEvent");
    s.store_field("type", type_);
    s.store_field("chatId", chat_id_);
    s.store_object_field("data", static_cast<const BaseObject *>(data_.get()));
    s.store_class_end();
  }
}

saveApplicationLogEvent::ReturnType saveApplicationLogEvent::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void saveApplicationLogEvent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SaveApplicationLogEvent").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    data_fieldID = jni::get_field_id(env, Class, "data", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$JsonValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass savePreparedInlineMessage::Class;
jfieldID savePreparedInlineMessage::user_id_fieldID;
jfieldID savePreparedInlineMessage::result_fieldID;
jfieldID savePreparedInlineMessage::chat_types_fieldID;

savePreparedInlineMessage::savePreparedInlineMessage()
  : user_id_()
  , result_()
  , chat_types_()
{}

savePreparedInlineMessage::savePreparedInlineMessage(int53 user_id_, object_ptr<InputInlineQueryResult> &&result_, object_ptr<targetChatTypes> &&chat_types_)
  : user_id_(user_id_)
  , result_(std::move(result_))
  , chat_types_(std::move(chat_types_))
{}

const std::int32_t savePreparedInlineMessage::ID;

object_ptr<savePreparedInlineMessage> savePreparedInlineMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<savePreparedInlineMessage> res = make_object<savePreparedInlineMessage>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->result_ = jni::fetch_tl_object<InputInlineQueryResult>(env, jni::fetch_object(env, p, res->result_fieldID));
  res->chat_types_ = jni::fetch_tl_object<targetChatTypes>(env, jni::fetch_object(env, p, res->chat_types_fieldID));
  return res;
}

void savePreparedInlineMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SavePreparedInlineMessage");
    s.store_field("userId", user_id_);
    s.store_object_field("result", static_cast<const BaseObject *>(result_.get()));
    s.store_object_field("chatTypes", static_cast<const BaseObject *>(chat_types_.get()));
    s.store_class_end();
  }
}

savePreparedInlineMessage::ReturnType savePreparedInlineMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return preparedInlineMessageId::fetch(env, p);
}

void savePreparedInlineMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SavePreparedInlineMessage").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    result_fieldID = jni::get_field_id(env, Class, "result", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputInlineQueryResult;").c_str());
    chat_types_fieldID = jni::get_field_id(env, Class, "chatTypes", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$TargetChatTypes;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchAffiliatePrograms::Class;
jfieldID searchAffiliatePrograms::affiliate_fieldID;
jfieldID searchAffiliatePrograms::sort_order_fieldID;
jfieldID searchAffiliatePrograms::offset_fieldID;
jfieldID searchAffiliatePrograms::limit_fieldID;

searchAffiliatePrograms::searchAffiliatePrograms()
  : affiliate_()
  , sort_order_()
  , offset_()
  , limit_()
{}

searchAffiliatePrograms::searchAffiliatePrograms(object_ptr<AffiliateType> &&affiliate_, object_ptr<AffiliateProgramSortOrder> &&sort_order_, string const &offset_, int32 limit_)
  : affiliate_(std::move(affiliate_))
  , sort_order_(std::move(sort_order_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchAffiliatePrograms::ID;

object_ptr<searchAffiliatePrograms> searchAffiliatePrograms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchAffiliatePrograms> res = make_object<searchAffiliatePrograms>();
  res->affiliate_ = jni::fetch_tl_object<AffiliateType>(env, jni::fetch_object(env, p, res->affiliate_fieldID));
  res->sort_order_ = jni::fetch_tl_object<AffiliateProgramSortOrder>(env, jni::fetch_object(env, p, res->sort_order_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchAffiliatePrograms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchAffiliatePrograms");
    s.store_object_field("affiliate", static_cast<const BaseObject *>(affiliate_.get()));
    s.store_object_field("sortOrder", static_cast<const BaseObject *>(sort_order_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchAffiliatePrograms::ReturnType searchAffiliatePrograms::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundAffiliatePrograms::fetch(env, p);
}

void searchAffiliatePrograms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchAffiliatePrograms").c_str());
    affiliate_fieldID = jni::get_field_id(env, Class, "affiliate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateType;").c_str());
    sort_order_fieldID = jni::get_field_id(env, Class, "sortOrder", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramSortOrder;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchBackground::Class;
jfieldID searchBackground::name_fieldID;

searchBackground::searchBackground()
  : name_()
{}

searchBackground::searchBackground(string const &name_)
  : name_(name_)
{}

const std::int32_t searchBackground::ID;

object_ptr<searchBackground> searchBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchBackground> res = make_object<searchBackground>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void searchBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchBackground");
    s.store_field("name", name_);
    s.store_class_end();
  }
}

searchBackground::ReturnType searchBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return background::fetch(env, p);
}

void searchBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchBackground").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchCallMessages::Class;
jfieldID searchCallMessages::offset_fieldID;
jfieldID searchCallMessages::limit_fieldID;
jfieldID searchCallMessages::only_missed_fieldID;

searchCallMessages::searchCallMessages()
  : offset_()
  , limit_()
  , only_missed_()
{}

searchCallMessages::searchCallMessages(string const &offset_, int32 limit_, bool only_missed_)
  : offset_(offset_)
  , limit_(limit_)
  , only_missed_(only_missed_)
{}

const std::int32_t searchCallMessages::ID;

object_ptr<searchCallMessages> searchCallMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchCallMessages> res = make_object<searchCallMessages>();
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->only_missed_ = (env->GetBooleanField(p, res->only_missed_fieldID) != 0);
  return res;
}

void searchCallMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchCallMessages");
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_field("onlyMissed", only_missed_);
    s.store_class_end();
  }
}

searchCallMessages::ReturnType searchCallMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundMessages::fetch(env, p);
}

void searchCallMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchCallMessages").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    only_missed_fieldID = jni::get_field_id(env, Class, "onlyMissed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass searchChatAffiliateProgram::Class;
jfieldID searchChatAffiliateProgram::username_fieldID;
jfieldID searchChatAffiliateProgram::referrer_fieldID;

searchChatAffiliateProgram::searchChatAffiliateProgram()
  : username_()
  , referrer_()
{}

searchChatAffiliateProgram::searchChatAffiliateProgram(string const &username_, string const &referrer_)
  : username_(username_)
  , referrer_(referrer_)
{}

const std::int32_t searchChatAffiliateProgram::ID;

object_ptr<searchChatAffiliateProgram> searchChatAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChatAffiliateProgram> res = make_object<searchChatAffiliateProgram>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->referrer_ = jni::fetch_string(env, p, res->referrer_fieldID);
  return res;
}

void searchChatAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChatAffiliateProgram");
    s.store_field("username", username_);
    s.store_field("referrer", referrer_);
    s.store_class_end();
  }
}

searchChatAffiliateProgram::ReturnType searchChatAffiliateProgram::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void searchChatAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChatAffiliateProgram").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    referrer_fieldID = jni::get_field_id(env, Class, "referrer", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchChatMembers::Class;
jfieldID searchChatMembers::chat_id_fieldID;
jfieldID searchChatMembers::query_fieldID;
jfieldID searchChatMembers::limit_fieldID;
jfieldID searchChatMembers::filter_fieldID;

searchChatMembers::searchChatMembers()
  : chat_id_()
  , query_()
  , limit_()
  , filter_()
{}

searchChatMembers::searchChatMembers(int53 chat_id_, string const &query_, int32 limit_, object_ptr<ChatMembersFilter> &&filter_)
  : chat_id_(chat_id_)
  , query_(query_)
  , limit_(limit_)
  , filter_(std::move(filter_))
{}

const std::int32_t searchChatMembers::ID;

object_ptr<searchChatMembers> searchChatMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChatMembers> res = make_object<searchChatMembers>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->filter_ = jni::fetch_tl_object<ChatMembersFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  return res;
}

void searchChatMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChatMembers");
    s.store_field("chatId", chat_id_);
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_class_end();
  }
}

searchChatMembers::ReturnType searchChatMembers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chatMembers::fetch(env, p);
}

void searchChatMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChatMembers").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMembersFilter;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchChatMessages::Class;
jfieldID searchChatMessages::chat_id_fieldID;
jfieldID searchChatMessages::query_fieldID;
jfieldID searchChatMessages::sender_id_fieldID;
jfieldID searchChatMessages::from_message_id_fieldID;
jfieldID searchChatMessages::offset_fieldID;
jfieldID searchChatMessages::limit_fieldID;
jfieldID searchChatMessages::filter_fieldID;
jfieldID searchChatMessages::message_thread_id_fieldID;
jfieldID searchChatMessages::saved_messages_topic_id_fieldID;

searchChatMessages::searchChatMessages()
  : chat_id_()
  , query_()
  , sender_id_()
  , from_message_id_()
  , offset_()
  , limit_()
  , filter_()
  , message_thread_id_()
  , saved_messages_topic_id_()
{}

searchChatMessages::searchChatMessages(int53 chat_id_, string const &query_, object_ptr<MessageSender> &&sender_id_, int53 from_message_id_, int32 offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_, int53 message_thread_id_, int53 saved_messages_topic_id_)
  : chat_id_(chat_id_)
  , query_(query_)
  , sender_id_(std::move(sender_id_))
  , from_message_id_(from_message_id_)
  , offset_(offset_)
  , limit_(limit_)
  , filter_(std::move(filter_))
  , message_thread_id_(message_thread_id_)
  , saved_messages_topic_id_(saved_messages_topic_id_)
{}

const std::int32_t searchChatMessages::ID;

object_ptr<searchChatMessages> searchChatMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChatMessages> res = make_object<searchChatMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  return res;
}

void searchChatMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChatMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("query", query_);
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_class_end();
  }
}

searchChatMessages::ReturnType searchChatMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundChatMessages::fetch(env, p);
}

void searchChatMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChatMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass searchChatRecentLocationMessages::Class;
jfieldID searchChatRecentLocationMessages::chat_id_fieldID;
jfieldID searchChatRecentLocationMessages::limit_fieldID;

searchChatRecentLocationMessages::searchChatRecentLocationMessages()
  : chat_id_()
  , limit_()
{}

searchChatRecentLocationMessages::searchChatRecentLocationMessages(int53 chat_id_, int32 limit_)
  : chat_id_(chat_id_)
  , limit_(limit_)
{}

const std::int32_t searchChatRecentLocationMessages::ID;

object_ptr<searchChatRecentLocationMessages> searchChatRecentLocationMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChatRecentLocationMessages> res = make_object<searchChatRecentLocationMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchChatRecentLocationMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChatRecentLocationMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchChatRecentLocationMessages::ReturnType searchChatRecentLocationMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void searchChatRecentLocationMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChatRecentLocationMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchChats::Class;
jfieldID searchChats::query_fieldID;
jfieldID searchChats::limit_fieldID;

searchChats::searchChats()
  : query_()
  , limit_()
{}

searchChats::searchChats(string const &query_, int32 limit_)
  : query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchChats::ID;

object_ptr<searchChats> searchChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChats> res = make_object<searchChats>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChats");
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchChats::ReturnType searchChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void searchChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChats").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchChatsOnServer::Class;
jfieldID searchChatsOnServer::query_fieldID;
jfieldID searchChatsOnServer::limit_fieldID;

searchChatsOnServer::searchChatsOnServer()
  : query_()
  , limit_()
{}

searchChatsOnServer::searchChatsOnServer(string const &query_, int32 limit_)
  : query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchChatsOnServer::ID;

object_ptr<searchChatsOnServer> searchChatsOnServer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchChatsOnServer> res = make_object<searchChatsOnServer>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchChatsOnServer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchChatsOnServer");
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchChatsOnServer::ReturnType searchChatsOnServer::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void searchChatsOnServer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchChatsOnServer").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchContacts::Class;
jfieldID searchContacts::query_fieldID;
jfieldID searchContacts::limit_fieldID;

searchContacts::searchContacts()
  : query_()
  , limit_()
{}

searchContacts::searchContacts(string const &query_, int32 limit_)
  : query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchContacts::ID;

object_ptr<searchContacts> searchContacts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchContacts> res = make_object<searchContacts>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchContacts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchContacts");
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchContacts::ReturnType searchContacts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return users::fetch(env, p);
}

void searchContacts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchContacts").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchEmojis::Class;
jfieldID searchEmojis::text_fieldID;
jfieldID searchEmojis::input_language_codes_fieldID;

searchEmojis::searchEmojis()
  : text_()
  , input_language_codes_()
{}

searchEmojis::searchEmojis(string const &text_, array<string> &&input_language_codes_)
  : text_(text_)
  , input_language_codes_(std::move(input_language_codes_))
{}

const std::int32_t searchEmojis::ID;

object_ptr<searchEmojis> searchEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchEmojis> res = make_object<searchEmojis>();
  res->text_ = jni::fetch_string(env, p, res->text_fieldID);
  res->input_language_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_language_codes_fieldID));
  return res;
}

void searchEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchEmojis");
    s.store_field("text", text_);
    { s.store_vector_begin("inputLanguageCodes", input_language_codes_.size()); for (const auto &_value : input_language_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

searchEmojis::ReturnType searchEmojis::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emojiKeywords::fetch(env, p);
}

void searchEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchEmojis").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", "Ljava/lang/String;");
    input_language_codes_fieldID = jni::get_field_id(env, Class, "inputLanguageCodes", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchFileDownloads::Class;
jfieldID searchFileDownloads::query_fieldID;
jfieldID searchFileDownloads::only_active_fieldID;
jfieldID searchFileDownloads::only_completed_fieldID;
jfieldID searchFileDownloads::offset_fieldID;
jfieldID searchFileDownloads::limit_fieldID;

searchFileDownloads::searchFileDownloads()
  : query_()
  , only_active_()
  , only_completed_()
  , offset_()
  , limit_()
{}

searchFileDownloads::searchFileDownloads(string const &query_, bool only_active_, bool only_completed_, string const &offset_, int32 limit_)
  : query_(query_)
  , only_active_(only_active_)
  , only_completed_(only_completed_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchFileDownloads::ID;

object_ptr<searchFileDownloads> searchFileDownloads::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchFileDownloads> res = make_object<searchFileDownloads>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->only_active_ = (env->GetBooleanField(p, res->only_active_fieldID) != 0);
  res->only_completed_ = (env->GetBooleanField(p, res->only_completed_fieldID) != 0);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchFileDownloads::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchFileDownloads");
    s.store_field("query", query_);
    s.store_field("onlyActive", only_active_);
    s.store_field("onlyCompleted", only_completed_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchFileDownloads::ReturnType searchFileDownloads::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundFileDownloads::fetch(env, p);
}

void searchFileDownloads::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchFileDownloads").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    only_active_fieldID = jni::get_field_id(env, Class, "onlyActive", "Z");
    only_completed_fieldID = jni::get_field_id(env, Class, "onlyCompleted", "Z");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchGiftsForResale::Class;
jfieldID searchGiftsForResale::gift_id_fieldID;
jfieldID searchGiftsForResale::order_fieldID;
jfieldID searchGiftsForResale::attributes_fieldID;
jfieldID searchGiftsForResale::offset_fieldID;
jfieldID searchGiftsForResale::limit_fieldID;

searchGiftsForResale::searchGiftsForResale()
  : gift_id_()
  , order_()
  , attributes_()
  , offset_()
  , limit_()
{}

searchGiftsForResale::searchGiftsForResale(int64 gift_id_, object_ptr<GiftForResaleOrder> &&order_, array<object_ptr<UpgradedGiftAttributeId>> &&attributes_, string const &offset_, int32 limit_)
  : gift_id_(gift_id_)
  , order_(std::move(order_))
  , attributes_(std::move(attributes_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchGiftsForResale::ID;

object_ptr<searchGiftsForResale> searchGiftsForResale::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchGiftsForResale> res = make_object<searchGiftsForResale>();
  res->gift_id_ = env->GetLongField(p, res->gift_id_fieldID);
  res->order_ = jni::fetch_tl_object<GiftForResaleOrder>(env, jni::fetch_object(env, p, res->order_fieldID));
  res->attributes_ = jni::FetchVector<UpgradedGiftAttributeId>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->attributes_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchGiftsForResale::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchGiftsForResale");
    s.store_field("giftId", gift_id_);
    s.store_object_field("order", static_cast<const BaseObject *>(order_.get()));
    { s.store_vector_begin("attributes", attributes_.size()); for (const auto &_value : attributes_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchGiftsForResale::ReturnType searchGiftsForResale::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return giftsForResale::fetch(env, p);
}

void searchGiftsForResale::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchGiftsForResale").c_str());
    gift_id_fieldID = jni::get_field_id(env, Class, "giftId", "J");
    order_fieldID = jni::get_field_id(env, Class, "order", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiftForResaleOrder;").c_str());
    attributes_fieldID = jni::get_field_id(env, Class, "attributes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$UpgradedGiftAttributeId;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchHashtags::Class;
jfieldID searchHashtags::prefix_fieldID;
jfieldID searchHashtags::limit_fieldID;

searchHashtags::searchHashtags()
  : prefix_()
  , limit_()
{}

searchHashtags::searchHashtags(string const &prefix_, int32 limit_)
  : prefix_(prefix_)
  , limit_(limit_)
{}

const std::int32_t searchHashtags::ID;

object_ptr<searchHashtags> searchHashtags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchHashtags> res = make_object<searchHashtags>();
  res->prefix_ = jni::fetch_string(env, p, res->prefix_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchHashtags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchHashtags");
    s.store_field("prefix", prefix_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchHashtags::ReturnType searchHashtags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return hashtags::fetch(env, p);
}

void searchHashtags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchHashtags").c_str());
    prefix_fieldID = jni::get_field_id(env, Class, "prefix", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchInstalledStickerSets::Class;
jfieldID searchInstalledStickerSets::sticker_type_fieldID;
jfieldID searchInstalledStickerSets::query_fieldID;
jfieldID searchInstalledStickerSets::limit_fieldID;

searchInstalledStickerSets::searchInstalledStickerSets()
  : sticker_type_()
  , query_()
  , limit_()
{}

searchInstalledStickerSets::searchInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, string const &query_, int32 limit_)
  : sticker_type_(std::move(sticker_type_))
  , query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchInstalledStickerSets::ID;

object_ptr<searchInstalledStickerSets> searchInstalledStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchInstalledStickerSets> res = make_object<searchInstalledStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchInstalledStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchInstalledStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchInstalledStickerSets::ReturnType searchInstalledStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void searchInstalledStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchInstalledStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchMessages::Class;
jfieldID searchMessages::chat_list_fieldID;
jfieldID searchMessages::query_fieldID;
jfieldID searchMessages::offset_fieldID;
jfieldID searchMessages::limit_fieldID;
jfieldID searchMessages::filter_fieldID;
jfieldID searchMessages::chat_type_filter_fieldID;
jfieldID searchMessages::min_date_fieldID;
jfieldID searchMessages::max_date_fieldID;

searchMessages::searchMessages()
  : chat_list_()
  , query_()
  , offset_()
  , limit_()
  , filter_()
  , chat_type_filter_()
  , min_date_()
  , max_date_()
{}

searchMessages::searchMessages(object_ptr<ChatList> &&chat_list_, string const &query_, string const &offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_, object_ptr<SearchMessagesChatTypeFilter> &&chat_type_filter_, int32 min_date_, int32 max_date_)
  : chat_list_(std::move(chat_list_))
  , query_(query_)
  , offset_(offset_)
  , limit_(limit_)
  , filter_(std::move(filter_))
  , chat_type_filter_(std::move(chat_type_filter_))
  , min_date_(min_date_)
  , max_date_(max_date_)
{}

const std::int32_t searchMessages::ID;

object_ptr<searchMessages> searchMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchMessages> res = make_object<searchMessages>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  res->chat_type_filter_ = jni::fetch_tl_object<SearchMessagesChatTypeFilter>(env, jni::fetch_object(env, p, res->chat_type_filter_fieldID));
  res->min_date_ = env->GetIntField(p, res->min_date_fieldID);
  res->max_date_ = env->GetIntField(p, res->max_date_fieldID);
  return res;
}

void searchMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchMessages");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("query", query_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_object_field("chatTypeFilter", static_cast<const BaseObject *>(chat_type_filter_.get()));
    s.store_field("minDate", min_date_);
    s.store_field("maxDate", max_date_);
    s.store_class_end();
  }
}

searchMessages::ReturnType searchMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundMessages::fetch(env, p);
}

void searchMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchMessages").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    chat_type_filter_fieldID = jni::get_field_id(env, Class, "chatTypeFilter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesChatTypeFilter;").c_str());
    min_date_fieldID = jni::get_field_id(env, Class, "minDate", "I");
    max_date_fieldID = jni::get_field_id(env, Class, "maxDate", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchOutgoingDocumentMessages::Class;
jfieldID searchOutgoingDocumentMessages::query_fieldID;
jfieldID searchOutgoingDocumentMessages::limit_fieldID;

searchOutgoingDocumentMessages::searchOutgoingDocumentMessages()
  : query_()
  , limit_()
{}

searchOutgoingDocumentMessages::searchOutgoingDocumentMessages(string const &query_, int32 limit_)
  : query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchOutgoingDocumentMessages::ID;

object_ptr<searchOutgoingDocumentMessages> searchOutgoingDocumentMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchOutgoingDocumentMessages> res = make_object<searchOutgoingDocumentMessages>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchOutgoingDocumentMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchOutgoingDocumentMessages");
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchOutgoingDocumentMessages::ReturnType searchOutgoingDocumentMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundMessages::fetch(env, p);
}

void searchOutgoingDocumentMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchOutgoingDocumentMessages").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicChat::Class;
jfieldID searchPublicChat::username_fieldID;

searchPublicChat::searchPublicChat()
  : username_()
{}

searchPublicChat::searchPublicChat(string const &username_)
  : username_(username_)
{}

const std::int32_t searchPublicChat::ID;

object_ptr<searchPublicChat> searchPublicChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicChat> res = make_object<searchPublicChat>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return res;
}

void searchPublicChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicChat");
    s.store_field("username", username_);
    s.store_class_end();
  }
}

searchPublicChat::ReturnType searchPublicChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void searchPublicChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicChat").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicChats::Class;
jfieldID searchPublicChats::query_fieldID;

searchPublicChats::searchPublicChats()
  : query_()
{}

searchPublicChats::searchPublicChats(string const &query_)
  : query_(query_)
{}

const std::int32_t searchPublicChats::ID;

object_ptr<searchPublicChats> searchPublicChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicChats> res = make_object<searchPublicChats>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return res;
}

void searchPublicChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicChats");
    s.store_field("query", query_);
    s.store_class_end();
  }
}

searchPublicChats::ReturnType searchPublicChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void searchPublicChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicChats").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicMessagesByTag::Class;
jfieldID searchPublicMessagesByTag::tag_fieldID;
jfieldID searchPublicMessagesByTag::offset_fieldID;
jfieldID searchPublicMessagesByTag::limit_fieldID;

searchPublicMessagesByTag::searchPublicMessagesByTag()
  : tag_()
  , offset_()
  , limit_()
{}

searchPublicMessagesByTag::searchPublicMessagesByTag(string const &tag_, string const &offset_, int32 limit_)
  : tag_(tag_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchPublicMessagesByTag::ID;

object_ptr<searchPublicMessagesByTag> searchPublicMessagesByTag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicMessagesByTag> res = make_object<searchPublicMessagesByTag>();
  res->tag_ = jni::fetch_string(env, p, res->tag_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchPublicMessagesByTag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicMessagesByTag");
    s.store_field("tag", tag_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchPublicMessagesByTag::ReturnType searchPublicMessagesByTag::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundMessages::fetch(env, p);
}

void searchPublicMessagesByTag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicMessagesByTag").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicStoriesByLocation::Class;
jfieldID searchPublicStoriesByLocation::address_fieldID;
jfieldID searchPublicStoriesByLocation::offset_fieldID;
jfieldID searchPublicStoriesByLocation::limit_fieldID;

searchPublicStoriesByLocation::searchPublicStoriesByLocation()
  : address_()
  , offset_()
  , limit_()
{}

searchPublicStoriesByLocation::searchPublicStoriesByLocation(object_ptr<locationAddress> &&address_, string const &offset_, int32 limit_)
  : address_(std::move(address_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchPublicStoriesByLocation::ID;

object_ptr<searchPublicStoriesByLocation> searchPublicStoriesByLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicStoriesByLocation> res = make_object<searchPublicStoriesByLocation>();
  res->address_ = jni::fetch_tl_object<locationAddress>(env, jni::fetch_object(env, p, res->address_fieldID));
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchPublicStoriesByLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicStoriesByLocation");
    s.store_object_field("address", static_cast<const BaseObject *>(address_.get()));
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchPublicStoriesByLocation::ReturnType searchPublicStoriesByLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundStories::fetch(env, p);
}

void searchPublicStoriesByLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicStoriesByLocation").c_str());
    address_fieldID = jni::get_field_id(env, Class, "address", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LocationAddress;").c_str());
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicStoriesByTag::Class;
jfieldID searchPublicStoriesByTag::story_poster_chat_id_fieldID;
jfieldID searchPublicStoriesByTag::tag_fieldID;
jfieldID searchPublicStoriesByTag::offset_fieldID;
jfieldID searchPublicStoriesByTag::limit_fieldID;

searchPublicStoriesByTag::searchPublicStoriesByTag()
  : story_poster_chat_id_()
  , tag_()
  , offset_()
  , limit_()
{}

searchPublicStoriesByTag::searchPublicStoriesByTag(int53 story_poster_chat_id_, string const &tag_, string const &offset_, int32 limit_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , tag_(tag_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchPublicStoriesByTag::ID;

object_ptr<searchPublicStoriesByTag> searchPublicStoriesByTag::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicStoriesByTag> res = make_object<searchPublicStoriesByTag>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->tag_ = jni::fetch_string(env, p, res->tag_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchPublicStoriesByTag::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicStoriesByTag");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("tag", tag_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchPublicStoriesByTag::ReturnType searchPublicStoriesByTag::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundStories::fetch(env, p);
}

void searchPublicStoriesByTag::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicStoriesByTag").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    tag_fieldID = jni::get_field_id(env, Class, "tag", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchPublicStoriesByVenue::Class;
jfieldID searchPublicStoriesByVenue::venue_provider_fieldID;
jfieldID searchPublicStoriesByVenue::venue_id_fieldID;
jfieldID searchPublicStoriesByVenue::offset_fieldID;
jfieldID searchPublicStoriesByVenue::limit_fieldID;

searchPublicStoriesByVenue::searchPublicStoriesByVenue()
  : venue_provider_()
  , venue_id_()
  , offset_()
  , limit_()
{}

searchPublicStoriesByVenue::searchPublicStoriesByVenue(string const &venue_provider_, string const &venue_id_, string const &offset_, int32 limit_)
  : venue_provider_(venue_provider_)
  , venue_id_(venue_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchPublicStoriesByVenue::ID;

object_ptr<searchPublicStoriesByVenue> searchPublicStoriesByVenue::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchPublicStoriesByVenue> res = make_object<searchPublicStoriesByVenue>();
  res->venue_provider_ = jni::fetch_string(env, p, res->venue_provider_fieldID);
  res->venue_id_ = jni::fetch_string(env, p, res->venue_id_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchPublicStoriesByVenue::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchPublicStoriesByVenue");
    s.store_field("venueProvider", venue_provider_);
    s.store_field("venueId", venue_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchPublicStoriesByVenue::ReturnType searchPublicStoriesByVenue::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundStories::fetch(env, p);
}

void searchPublicStoriesByVenue::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchPublicStoriesByVenue").c_str());
    venue_provider_fieldID = jni::get_field_id(env, Class, "venueProvider", "Ljava/lang/String;");
    venue_id_fieldID = jni::get_field_id(env, Class, "venueId", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchQuote::Class;
jfieldID searchQuote::text_fieldID;
jfieldID searchQuote::quote_fieldID;
jfieldID searchQuote::quote_position_fieldID;

searchQuote::searchQuote()
  : text_()
  , quote_()
  , quote_position_()
{}

searchQuote::searchQuote(object_ptr<formattedText> &&text_, object_ptr<formattedText> &&quote_, int32 quote_position_)
  : text_(std::move(text_))
  , quote_(std::move(quote_))
  , quote_position_(quote_position_)
{}

const std::int32_t searchQuote::ID;

object_ptr<searchQuote> searchQuote::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchQuote> res = make_object<searchQuote>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->quote_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->quote_fieldID));
  res->quote_position_ = env->GetIntField(p, res->quote_position_fieldID);
  return res;
}

void searchQuote::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchQuote");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_object_field("quote", static_cast<const BaseObject *>(quote_.get()));
    s.store_field("quotePosition", quote_position_);
    s.store_class_end();
  }
}

searchQuote::ReturnType searchQuote::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundPosition::fetch(env, p);
}

void searchQuote::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchQuote").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    quote_fieldID = jni::get_field_id(env, Class, "quote", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    quote_position_fieldID = jni::get_field_id(env, Class, "quotePosition", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchRecentlyFoundChats::Class;
jfieldID searchRecentlyFoundChats::query_fieldID;
jfieldID searchRecentlyFoundChats::limit_fieldID;

searchRecentlyFoundChats::searchRecentlyFoundChats()
  : query_()
  , limit_()
{}

searchRecentlyFoundChats::searchRecentlyFoundChats(string const &query_, int32 limit_)
  : query_(query_)
  , limit_(limit_)
{}

const std::int32_t searchRecentlyFoundChats::ID;

object_ptr<searchRecentlyFoundChats> searchRecentlyFoundChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchRecentlyFoundChats> res = make_object<searchRecentlyFoundChats>();
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchRecentlyFoundChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchRecentlyFoundChats");
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchRecentlyFoundChats::ReturnType searchRecentlyFoundChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chats::fetch(env, p);
}

void searchRecentlyFoundChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchRecentlyFoundChats").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchSavedMessages::Class;
jfieldID searchSavedMessages::saved_messages_topic_id_fieldID;
jfieldID searchSavedMessages::tag_fieldID;
jfieldID searchSavedMessages::query_fieldID;
jfieldID searchSavedMessages::from_message_id_fieldID;
jfieldID searchSavedMessages::offset_fieldID;
jfieldID searchSavedMessages::limit_fieldID;

searchSavedMessages::searchSavedMessages()
  : saved_messages_topic_id_()
  , tag_()
  , query_()
  , from_message_id_()
  , offset_()
  , limit_()
{}

searchSavedMessages::searchSavedMessages(int53 saved_messages_topic_id_, object_ptr<ReactionType> &&tag_, string const &query_, int53 from_message_id_, int32 offset_, int32 limit_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , tag_(std::move(tag_))
  , query_(query_)
  , from_message_id_(from_message_id_)
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchSavedMessages::ID;

object_ptr<searchSavedMessages> searchSavedMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchSavedMessages> res = make_object<searchSavedMessages>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->tag_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->tag_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->from_message_id_ = env->GetLongField(p, res->from_message_id_fieldID);
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchSavedMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchSavedMessages");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_object_field("tag", static_cast<const BaseObject *>(tag_.get()));
    s.store_field("query", query_);
    s.store_field("fromMessageId", from_message_id_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchSavedMessages::ReturnType searchSavedMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundChatMessages::fetch(env, p);
}

void searchSavedMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchSavedMessages").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    tag_fieldID = jni::get_field_id(env, Class, "tag", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    from_message_id_fieldID = jni::get_field_id(env, Class, "fromMessageId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchSecretMessages::Class;
jfieldID searchSecretMessages::chat_id_fieldID;
jfieldID searchSecretMessages::query_fieldID;
jfieldID searchSecretMessages::offset_fieldID;
jfieldID searchSecretMessages::limit_fieldID;
jfieldID searchSecretMessages::filter_fieldID;

searchSecretMessages::searchSecretMessages()
  : chat_id_()
  , query_()
  , offset_()
  , limit_()
  , filter_()
{}

searchSecretMessages::searchSecretMessages(int53 chat_id_, string const &query_, string const &offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_)
  : chat_id_(chat_id_)
  , query_(query_)
  , offset_(offset_)
  , limit_(limit_)
  , filter_(std::move(filter_))
{}

const std::int32_t searchSecretMessages::ID;

object_ptr<searchSecretMessages> searchSecretMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchSecretMessages> res = make_object<searchSecretMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->offset_ = jni::fetch_string(env, p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->filter_ = jni::fetch_tl_object<SearchMessagesFilter>(env, jni::fetch_object(env, p, res->filter_fieldID));
  return res;
}

void searchSecretMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchSecretMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("query", query_);
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_object_field("filter", static_cast<const BaseObject *>(filter_.get()));
    s.store_class_end();
  }
}

searchSecretMessages::ReturnType searchSecretMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundMessages::fetch(env, p);
}

void searchSecretMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchSecretMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    filter_fieldID = jni::get_field_id(env, Class, "filter", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$SearchMessagesFilter;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass searchStickerSet::Class;
jfieldID searchStickerSet::name_fieldID;
jfieldID searchStickerSet::ignore_cache_fieldID;

searchStickerSet::searchStickerSet()
  : name_()
  , ignore_cache_()
{}

searchStickerSet::searchStickerSet(string const &name_, bool ignore_cache_)
  : name_(name_)
  , ignore_cache_(ignore_cache_)
{}

const std::int32_t searchStickerSet::ID;

object_ptr<searchStickerSet> searchStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchStickerSet> res = make_object<searchStickerSet>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->ignore_cache_ = (env->GetBooleanField(p, res->ignore_cache_fieldID) != 0);
  return res;
}

void searchStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchStickerSet");
    s.store_field("name", name_);
    s.store_field("ignoreCache", ignore_cache_);
    s.store_class_end();
  }
}

searchStickerSet::ReturnType searchStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSet::fetch(env, p);
}

void searchStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchStickerSet").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    ignore_cache_fieldID = jni::get_field_id(env, Class, "ignoreCache", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass searchStickerSets::Class;
jfieldID searchStickerSets::sticker_type_fieldID;
jfieldID searchStickerSets::query_fieldID;

searchStickerSets::searchStickerSets()
  : sticker_type_()
  , query_()
{}

searchStickerSets::searchStickerSets(object_ptr<StickerType> &&sticker_type_, string const &query_)
  : sticker_type_(std::move(sticker_type_))
  , query_(query_)
{}

const std::int32_t searchStickerSets::ID;

object_ptr<searchStickerSets> searchStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchStickerSets> res = make_object<searchStickerSets>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  return res;
}

void searchStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchStickerSets");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("query", query_);
    s.store_class_end();
  }
}

searchStickerSets::ReturnType searchStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickerSets::fetch(env, p);
}

void searchStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchStickerSets").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchStickers::Class;
jfieldID searchStickers::sticker_type_fieldID;
jfieldID searchStickers::emojis_fieldID;
jfieldID searchStickers::query_fieldID;
jfieldID searchStickers::input_language_codes_fieldID;
jfieldID searchStickers::offset_fieldID;
jfieldID searchStickers::limit_fieldID;

searchStickers::searchStickers()
  : sticker_type_()
  , emojis_()
  , query_()
  , input_language_codes_()
  , offset_()
  , limit_()
{}

searchStickers::searchStickers(object_ptr<StickerType> &&sticker_type_, string const &emojis_, string const &query_, array<string> &&input_language_codes_, int32 offset_, int32 limit_)
  : sticker_type_(std::move(sticker_type_))
  , emojis_(emojis_)
  , query_(query_)
  , input_language_codes_(std::move(input_language_codes_))
  , offset_(offset_)
  , limit_(limit_)
{}

const std::int32_t searchStickers::ID;

object_ptr<searchStickers> searchStickers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchStickers> res = make_object<searchStickers>();
  res->sticker_type_ = jni::fetch_tl_object<StickerType>(env, jni::fetch_object(env, p, res->sticker_type_fieldID));
  res->emojis_ = jni::fetch_string(env, p, res->emojis_fieldID);
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->input_language_codes_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_language_codes_fieldID));
  res->offset_ = env->GetIntField(p, res->offset_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  return res;
}

void searchStickers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchStickers");
    s.store_object_field("stickerType", static_cast<const BaseObject *>(sticker_type_.get()));
    s.store_field("emojis", emojis_);
    s.store_field("query", query_);
    { s.store_vector_begin("inputLanguageCodes", input_language_codes_.size()); for (const auto &_value : input_language_codes_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("offset", offset_);
    s.store_field("limit", limit_);
    s.store_class_end();
  }
}

searchStickers::ReturnType searchStickers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return stickers::fetch(env, p);
}

void searchStickers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchStickers").c_str());
    sticker_type_fieldID = jni::get_field_id(env, Class, "stickerType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerType;").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "Ljava/lang/String;");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    input_language_codes_fieldID = jni::get_field_id(env, Class, "inputLanguageCodes", "[Ljava/lang/String;");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "I");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return true;
  }();
  (void)is_inited;
}

jclass searchStringsByPrefix::Class;
jfieldID searchStringsByPrefix::strings_fieldID;
jfieldID searchStringsByPrefix::query_fieldID;
jfieldID searchStringsByPrefix::limit_fieldID;
jfieldID searchStringsByPrefix::return_none_for_empty_query_fieldID;

searchStringsByPrefix::searchStringsByPrefix()
  : strings_()
  , query_()
  , limit_()
  , return_none_for_empty_query_()
{}

searchStringsByPrefix::searchStringsByPrefix(array<string> &&strings_, string const &query_, int32 limit_, bool return_none_for_empty_query_)
  : strings_(std::move(strings_))
  , query_(query_)
  , limit_(limit_)
  , return_none_for_empty_query_(return_none_for_empty_query_)
{}

const std::int32_t searchStringsByPrefix::ID;

object_ptr<searchStringsByPrefix> searchStringsByPrefix::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchStringsByPrefix> res = make_object<searchStringsByPrefix>();
  res->strings_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->strings_fieldID));
  res->query_ = jni::fetch_string(env, p, res->query_fieldID);
  res->limit_ = env->GetIntField(p, res->limit_fieldID);
  res->return_none_for_empty_query_ = (env->GetBooleanField(p, res->return_none_for_empty_query_fieldID) != 0);
  return res;
}

void searchStringsByPrefix::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchStringsByPrefix");
    { s.store_vector_begin("strings", strings_.size()); for (const auto &_value : strings_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("query", query_);
    s.store_field("limit", limit_);
    s.store_field("returnNoneForEmptyQuery", return_none_for_empty_query_);
    s.store_class_end();
  }
}

searchStringsByPrefix::ReturnType searchStringsByPrefix::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundPositions::fetch(env, p);
}

void searchStringsByPrefix::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchStringsByPrefix").c_str());
    strings_fieldID = jni::get_field_id(env, Class, "strings", "[Ljava/lang/String;");
    query_fieldID = jni::get_field_id(env, Class, "query", "Ljava/lang/String;");
    limit_fieldID = jni::get_field_id(env, Class, "limit", "I");
    return_none_for_empty_query_fieldID = jni::get_field_id(env, Class, "returnNoneForEmptyQuery", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass searchUserByPhoneNumber::Class;
jfieldID searchUserByPhoneNumber::phone_number_fieldID;
jfieldID searchUserByPhoneNumber::only_local_fieldID;

searchUserByPhoneNumber::searchUserByPhoneNumber()
  : phone_number_()
  , only_local_()
{}

searchUserByPhoneNumber::searchUserByPhoneNumber(string const &phone_number_, bool only_local_)
  : phone_number_(phone_number_)
  , only_local_(only_local_)
{}

const std::int32_t searchUserByPhoneNumber::ID;

object_ptr<searchUserByPhoneNumber> searchUserByPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchUserByPhoneNumber> res = make_object<searchUserByPhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->only_local_ = (env->GetBooleanField(p, res->only_local_fieldID) != 0);
  return res;
}

void searchUserByPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchUserByPhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_field("onlyLocal", only_local_);
    s.store_class_end();
  }
}

searchUserByPhoneNumber::ReturnType searchUserByPhoneNumber::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return user::fetch(env, p);
}

void searchUserByPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchUserByPhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    only_local_fieldID = jni::get_field_id(env, Class, "onlyLocal", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass searchUserByToken::Class;
jfieldID searchUserByToken::token_fieldID;

searchUserByToken::searchUserByToken()
  : token_()
{}

searchUserByToken::searchUserByToken(string const &token_)
  : token_(token_)
{}

const std::int32_t searchUserByToken::ID;

object_ptr<searchUserByToken> searchUserByToken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchUserByToken> res = make_object<searchUserByToken>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return res;
}

void searchUserByToken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchUserByToken");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

searchUserByToken::ReturnType searchUserByToken::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return user::fetch(env, p);
}

void searchUserByToken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchUserByToken").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass searchWebApp::Class;
jfieldID searchWebApp::bot_user_id_fieldID;
jfieldID searchWebApp::web_app_short_name_fieldID;

searchWebApp::searchWebApp()
  : bot_user_id_()
  , web_app_short_name_()
{}

searchWebApp::searchWebApp(int53 bot_user_id_, string const &web_app_short_name_)
  : bot_user_id_(bot_user_id_)
  , web_app_short_name_(web_app_short_name_)
{}

const std::int32_t searchWebApp::ID;

object_ptr<searchWebApp> searchWebApp::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<searchWebApp> res = make_object<searchWebApp>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->web_app_short_name_ = jni::fetch_string(env, p, res->web_app_short_name_fieldID);
  return res;
}

void searchWebApp::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SearchWebApp");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("webAppShortName", web_app_short_name_);
    s.store_class_end();
  }
}

searchWebApp::ReturnType searchWebApp::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return foundWebApp::fetch(env, p);
}

void searchWebApp::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SearchWebApp").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    web_app_short_name_fieldID = jni::get_field_id(env, Class, "webAppShortName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sellGift::Class;
jfieldID sellGift::business_connection_id_fieldID;
jfieldID sellGift::received_gift_id_fieldID;

sellGift::sellGift()
  : business_connection_id_()
  , received_gift_id_()
{}

sellGift::sellGift(string const &business_connection_id_, string const &received_gift_id_)
  : business_connection_id_(business_connection_id_)
  , received_gift_id_(received_gift_id_)
{}

const std::int32_t sellGift::ID;

object_ptr<sellGift> sellGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sellGift> res = make_object<sellGift>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  return res;
}

void sellGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SellGift");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_class_end();
  }
}

sellGift::ReturnType sellGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sellGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SellGift").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendAuthenticationFirebaseSms::Class;
jfieldID sendAuthenticationFirebaseSms::token_fieldID;

sendAuthenticationFirebaseSms::sendAuthenticationFirebaseSms()
  : token_()
{}

sendAuthenticationFirebaseSms::sendAuthenticationFirebaseSms(string const &token_)
  : token_(token_)
{}

const std::int32_t sendAuthenticationFirebaseSms::ID;

object_ptr<sendAuthenticationFirebaseSms> sendAuthenticationFirebaseSms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendAuthenticationFirebaseSms> res = make_object<sendAuthenticationFirebaseSms>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return res;
}

void sendAuthenticationFirebaseSms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendAuthenticationFirebaseSms");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

sendAuthenticationFirebaseSms::ReturnType sendAuthenticationFirebaseSms::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendAuthenticationFirebaseSms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendAuthenticationFirebaseSms").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendBotStartMessage::Class;
jfieldID sendBotStartMessage::bot_user_id_fieldID;
jfieldID sendBotStartMessage::chat_id_fieldID;
jfieldID sendBotStartMessage::parameter_fieldID;

sendBotStartMessage::sendBotStartMessage()
  : bot_user_id_()
  , chat_id_()
  , parameter_()
{}

sendBotStartMessage::sendBotStartMessage(int53 bot_user_id_, int53 chat_id_, string const &parameter_)
  : bot_user_id_(bot_user_id_)
  , chat_id_(chat_id_)
  , parameter_(parameter_)
{}

const std::int32_t sendBotStartMessage::ID;

object_ptr<sendBotStartMessage> sendBotStartMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendBotStartMessage> res = make_object<sendBotStartMessage>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->parameter_ = jni::fetch_string(env, p, res->parameter_fieldID);
  return res;
}

void sendBotStartMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendBotStartMessage");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("parameter", parameter_);
    s.store_class_end();
  }
}

sendBotStartMessage::ReturnType sendBotStartMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void sendBotStartMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendBotStartMessage").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    parameter_fieldID = jni::get_field_id(env, Class, "parameter", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendBusinessMessage::Class;
jfieldID sendBusinessMessage::business_connection_id_fieldID;
jfieldID sendBusinessMessage::chat_id_fieldID;
jfieldID sendBusinessMessage::reply_to_fieldID;
jfieldID sendBusinessMessage::disable_notification_fieldID;
jfieldID sendBusinessMessage::protect_content_fieldID;
jfieldID sendBusinessMessage::effect_id_fieldID;
jfieldID sendBusinessMessage::reply_markup_fieldID;
jfieldID sendBusinessMessage::input_message_content_fieldID;

sendBusinessMessage::sendBusinessMessage()
  : business_connection_id_()
  , chat_id_()
  , reply_to_()
  , disable_notification_()
  , protect_content_()
  , effect_id_()
  , reply_markup_()
  , input_message_content_()
{}

sendBusinessMessage::sendBusinessMessage(string const &business_connection_id_, int53 chat_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, bool protect_content_, int64 effect_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , reply_to_(std::move(reply_to_))
  , disable_notification_(disable_notification_)
  , protect_content_(protect_content_)
  , effect_id_(effect_id_)
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t sendBusinessMessage::ID;

object_ptr<sendBusinessMessage> sendBusinessMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendBusinessMessage> res = make_object<sendBusinessMessage>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  res->protect_content_ = (env->GetBooleanField(p, res->protect_content_fieldID) != 0);
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void sendBusinessMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendBusinessMessage");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_field("disableNotification", disable_notification_);
    s.store_field("protectContent", protect_content_);
    s.store_field("effectId", effect_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

sendBusinessMessage::ReturnType sendBusinessMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void sendBusinessMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendBusinessMessage").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    protect_content_fieldID = jni::get_field_id(env, Class, "protectContent", "Z");
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendBusinessMessageAlbum::Class;
jfieldID sendBusinessMessageAlbum::business_connection_id_fieldID;
jfieldID sendBusinessMessageAlbum::chat_id_fieldID;
jfieldID sendBusinessMessageAlbum::reply_to_fieldID;
jfieldID sendBusinessMessageAlbum::disable_notification_fieldID;
jfieldID sendBusinessMessageAlbum::protect_content_fieldID;
jfieldID sendBusinessMessageAlbum::effect_id_fieldID;
jfieldID sendBusinessMessageAlbum::input_message_contents_fieldID;

sendBusinessMessageAlbum::sendBusinessMessageAlbum()
  : business_connection_id_()
  , chat_id_()
  , reply_to_()
  , disable_notification_()
  , protect_content_()
  , effect_id_()
  , input_message_contents_()
{}

sendBusinessMessageAlbum::sendBusinessMessageAlbum(string const &business_connection_id_, int53 chat_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, bool protect_content_, int64 effect_id_, array<object_ptr<InputMessageContent>> &&input_message_contents_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , reply_to_(std::move(reply_to_))
  , disable_notification_(disable_notification_)
  , protect_content_(protect_content_)
  , effect_id_(effect_id_)
  , input_message_contents_(std::move(input_message_contents_))
{}

const std::int32_t sendBusinessMessageAlbum::ID;

object_ptr<sendBusinessMessageAlbum> sendBusinessMessageAlbum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendBusinessMessageAlbum> res = make_object<sendBusinessMessageAlbum>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->disable_notification_ = (env->GetBooleanField(p, res->disable_notification_fieldID) != 0);
  res->protect_content_ = (env->GetBooleanField(p, res->protect_content_fieldID) != 0);
  res->effect_id_ = env->GetLongField(p, res->effect_id_fieldID);
  res->input_message_contents_ = jni::FetchVector<InputMessageContent>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_message_contents_fieldID));
  return res;
}

void sendBusinessMessageAlbum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendBusinessMessageAlbum");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_field("disableNotification", disable_notification_);
    s.store_field("protectContent", protect_content_);
    s.store_field("effectId", effect_id_);
    { s.store_vector_begin("inputMessageContents", input_message_contents_.size()); for (const auto &_value : input_message_contents_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

sendBusinessMessageAlbum::ReturnType sendBusinessMessageAlbum::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessages::fetch(env, p);
}

void sendBusinessMessageAlbum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendBusinessMessageAlbum").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    disable_notification_fieldID = jni::get_field_id(env, Class, "disableNotification", "Z");
    protect_content_fieldID = jni::get_field_id(env, Class, "protectContent", "Z");
    effect_id_fieldID = jni::get_field_id(env, Class, "effectId", "J");
    input_message_contents_fieldID = jni::get_field_id(env, Class, "inputMessageContents", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendCallDebugInformation::Class;
jfieldID sendCallDebugInformation::call_id_fieldID;
jfieldID sendCallDebugInformation::debug_information_fieldID;

sendCallDebugInformation::sendCallDebugInformation()
  : call_id_()
  , debug_information_()
{}

sendCallDebugInformation::sendCallDebugInformation(int32 call_id_, string const &debug_information_)
  : call_id_(call_id_)
  , debug_information_(debug_information_)
{}

const std::int32_t sendCallDebugInformation::ID;

object_ptr<sendCallDebugInformation> sendCallDebugInformation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendCallDebugInformation> res = make_object<sendCallDebugInformation>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->debug_information_ = jni::fetch_string(env, p, res->debug_information_fieldID);
  return res;
}

void sendCallDebugInformation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendCallDebugInformation");
    s.store_field("callId", call_id_);
    s.store_field("debugInformation", debug_information_);
    s.store_class_end();
  }
}

sendCallDebugInformation::ReturnType sendCallDebugInformation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendCallDebugInformation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendCallDebugInformation").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    debug_information_fieldID = jni::get_field_id(env, Class, "debugInformation", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendCallLog::Class;
jfieldID sendCallLog::call_id_fieldID;
jfieldID sendCallLog::log_file_fieldID;

sendCallLog::sendCallLog()
  : call_id_()
  , log_file_()
{}

sendCallLog::sendCallLog(int32 call_id_, object_ptr<InputFile> &&log_file_)
  : call_id_(call_id_)
  , log_file_(std::move(log_file_))
{}

const std::int32_t sendCallLog::ID;

object_ptr<sendCallLog> sendCallLog::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendCallLog> res = make_object<sendCallLog>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->log_file_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->log_file_fieldID));
  return res;
}

void sendCallLog::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendCallLog");
    s.store_field("callId", call_id_);
    s.store_object_field("logFile", static_cast<const BaseObject *>(log_file_.get()));
    s.store_class_end();
  }
}

sendCallLog::ReturnType sendCallLog::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendCallLog::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendCallLog").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    log_file_fieldID = jni::get_field_id(env, Class, "logFile", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendCallRating::Class;
jfieldID sendCallRating::call_id_fieldID;
jfieldID sendCallRating::rating_fieldID;
jfieldID sendCallRating::comment_fieldID;
jfieldID sendCallRating::problems_fieldID;

sendCallRating::sendCallRating()
  : call_id_()
  , rating_()
  , comment_()
  , problems_()
{}

sendCallRating::sendCallRating(int32 call_id_, int32 rating_, string const &comment_, array<object_ptr<CallProblem>> &&problems_)
  : call_id_(call_id_)
  , rating_(rating_)
  , comment_(comment_)
  , problems_(std::move(problems_))
{}

const std::int32_t sendCallRating::ID;

object_ptr<sendCallRating> sendCallRating::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendCallRating> res = make_object<sendCallRating>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->rating_ = env->GetIntField(p, res->rating_fieldID);
  res->comment_ = jni::fetch_string(env, p, res->comment_fieldID);
  res->problems_ = jni::FetchVector<CallProblem>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->problems_fieldID));
  return res;
}

void sendCallRating::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendCallRating");
    s.store_field("callId", call_id_);
    s.store_field("rating", rating_);
    s.store_field("comment", comment_);
    { s.store_vector_begin("problems", problems_.size()); for (const auto &_value : problems_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

sendCallRating::ReturnType sendCallRating::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendCallRating::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendCallRating").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    rating_fieldID = jni::get_field_id(env, Class, "rating", "I");
    comment_fieldID = jni::get_field_id(env, Class, "comment", "Ljava/lang/String;");
    problems_fieldID = jni::get_field_id(env, Class, "problems", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$CallProblem;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendCallSignalingData::Class;
jfieldID sendCallSignalingData::call_id_fieldID;
jfieldID sendCallSignalingData::data_fieldID;

sendCallSignalingData::sendCallSignalingData()
  : call_id_()
  , data_()
{}

sendCallSignalingData::sendCallSignalingData(int32 call_id_, bytes const &data_)
  : call_id_(call_id_)
  , data_(std::move(data_))
{}

const std::int32_t sendCallSignalingData::ID;

object_ptr<sendCallSignalingData> sendCallSignalingData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendCallSignalingData> res = make_object<sendCallSignalingData>();
  res->call_id_ = env->GetIntField(p, res->call_id_fieldID);
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void sendCallSignalingData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendCallSignalingData");
    s.store_field("callId", call_id_);
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

sendCallSignalingData::ReturnType sendCallSignalingData::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendCallSignalingData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendCallSignalingData").c_str());
    call_id_fieldID = jni::get_field_id(env, Class, "callId", "I");
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass sendChatAction::Class;
jfieldID sendChatAction::chat_id_fieldID;
jfieldID sendChatAction::message_thread_id_fieldID;
jfieldID sendChatAction::business_connection_id_fieldID;
jfieldID sendChatAction::action_fieldID;

sendChatAction::sendChatAction()
  : chat_id_()
  , message_thread_id_()
  , business_connection_id_()
  , action_()
{}

sendChatAction::sendChatAction(int53 chat_id_, int53 message_thread_id_, string const &business_connection_id_, object_ptr<ChatAction> &&action_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , business_connection_id_(business_connection_id_)
  , action_(std::move(action_))
{}

const std::int32_t sendChatAction::ID;

object_ptr<sendChatAction> sendChatAction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendChatAction> res = make_object<sendChatAction>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->action_ = jni::fetch_tl_object<ChatAction>(env, jni::fetch_object(env, p, res->action_fieldID));
  return res;
}

void sendChatAction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendChatAction");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_object_field("action", static_cast<const BaseObject *>(action_.get()));
    s.store_class_end();
  }
}

sendChatAction::ReturnType sendChatAction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendChatAction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendChatAction").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    action_fieldID = jni::get_field_id(env, Class, "action", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAction;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendCustomRequest::Class;
jfieldID sendCustomRequest::method_fieldID;
jfieldID sendCustomRequest::parameters_fieldID;

sendCustomRequest::sendCustomRequest()
  : method_()
  , parameters_()
{}

sendCustomRequest::sendCustomRequest(string const &method_, string const &parameters_)
  : method_(method_)
  , parameters_(parameters_)
{}

const std::int32_t sendCustomRequest::ID;

object_ptr<sendCustomRequest> sendCustomRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendCustomRequest> res = make_object<sendCustomRequest>();
  res->method_ = jni::fetch_string(env, p, res->method_fieldID);
  res->parameters_ = jni::fetch_string(env, p, res->parameters_fieldID);
  return res;
}

void sendCustomRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendCustomRequest");
    s.store_field("method", method_);
    s.store_field("parameters", parameters_);
    s.store_class_end();
  }
}

sendCustomRequest::ReturnType sendCustomRequest::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return customRequestResult::fetch(env, p);
}

void sendCustomRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendCustomRequest").c_str());
    method_fieldID = jni::get_field_id(env, Class, "method", "Ljava/lang/String;");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendEmailAddressVerificationCode::Class;
jfieldID sendEmailAddressVerificationCode::email_address_fieldID;

sendEmailAddressVerificationCode::sendEmailAddressVerificationCode()
  : email_address_()
{}

sendEmailAddressVerificationCode::sendEmailAddressVerificationCode(string const &email_address_)
  : email_address_(email_address_)
{}

const std::int32_t sendEmailAddressVerificationCode::ID;

object_ptr<sendEmailAddressVerificationCode> sendEmailAddressVerificationCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendEmailAddressVerificationCode> res = make_object<sendEmailAddressVerificationCode>();
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  return res;
}

void sendEmailAddressVerificationCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendEmailAddressVerificationCode");
    s.store_field("emailAddress", email_address_);
    s.store_class_end();
  }
}

sendEmailAddressVerificationCode::ReturnType sendEmailAddressVerificationCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emailAddressAuthenticationCodeInfo::fetch(env, p);
}

void sendEmailAddressVerificationCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendEmailAddressVerificationCode").c_str());
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendGift::Class;
jfieldID sendGift::gift_id_fieldID;
jfieldID sendGift::owner_id_fieldID;
jfieldID sendGift::text_fieldID;
jfieldID sendGift::is_private_fieldID;
jfieldID sendGift::pay_for_upgrade_fieldID;

sendGift::sendGift()
  : gift_id_()
  , owner_id_()
  , text_()
  , is_private_()
  , pay_for_upgrade_()
{}

sendGift::sendGift(int64 gift_id_, object_ptr<MessageSender> &&owner_id_, object_ptr<formattedText> &&text_, bool is_private_, bool pay_for_upgrade_)
  : gift_id_(gift_id_)
  , owner_id_(std::move(owner_id_))
  , text_(std::move(text_))
  , is_private_(is_private_)
  , pay_for_upgrade_(pay_for_upgrade_)
{}

const std::int32_t sendGift::ID;

object_ptr<sendGift> sendGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendGift> res = make_object<sendGift>();
  res->gift_id_ = env->GetLongField(p, res->gift_id_fieldID);
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->is_private_ = (env->GetBooleanField(p, res->is_private_fieldID) != 0);
  res->pay_for_upgrade_ = (env->GetBooleanField(p, res->pay_for_upgrade_fieldID) != 0);
  return res;
}

void sendGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendGift");
    s.store_field("giftId", gift_id_);
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("isPrivate", is_private_);
    s.store_field("payForUpgrade", pay_for_upgrade_);
    s.store_class_end();
  }
}

sendGift::ReturnType sendGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendGift").c_str());
    gift_id_fieldID = jni::get_field_id(env, Class, "giftId", "J");
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    is_private_fieldID = jni::get_field_id(env, Class, "isPrivate", "Z");
    pay_for_upgrade_fieldID = jni::get_field_id(env, Class, "payForUpgrade", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass sendInlineQueryResultMessage::Class;
jfieldID sendInlineQueryResultMessage::chat_id_fieldID;
jfieldID sendInlineQueryResultMessage::message_thread_id_fieldID;
jfieldID sendInlineQueryResultMessage::reply_to_fieldID;
jfieldID sendInlineQueryResultMessage::options_fieldID;
jfieldID sendInlineQueryResultMessage::query_id_fieldID;
jfieldID sendInlineQueryResultMessage::result_id_fieldID;
jfieldID sendInlineQueryResultMessage::hide_via_bot_fieldID;

sendInlineQueryResultMessage::sendInlineQueryResultMessage()
  : chat_id_()
  , message_thread_id_()
  , reply_to_()
  , options_()
  , query_id_()
  , result_id_()
  , hide_via_bot_()
{}

sendInlineQueryResultMessage::sendInlineQueryResultMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, int64 query_id_, string const &result_id_, bool hide_via_bot_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , reply_to_(std::move(reply_to_))
  , options_(std::move(options_))
  , query_id_(query_id_)
  , result_id_(result_id_)
  , hide_via_bot_(hide_via_bot_)
{}

const std::int32_t sendInlineQueryResultMessage::ID;

object_ptr<sendInlineQueryResultMessage> sendInlineQueryResultMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendInlineQueryResultMessage> res = make_object<sendInlineQueryResultMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->options_ = jni::fetch_tl_object<messageSendOptions>(env, jni::fetch_object(env, p, res->options_fieldID));
  res->query_id_ = env->GetLongField(p, res->query_id_fieldID);
  res->result_id_ = jni::fetch_string(env, p, res->result_id_fieldID);
  res->hide_via_bot_ = (env->GetBooleanField(p, res->hide_via_bot_fieldID) != 0);
  return res;
}

void sendInlineQueryResultMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendInlineQueryResultMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_object_field("options", static_cast<const BaseObject *>(options_.get()));
    s.store_field("queryId", query_id_);
    s.store_field("resultId", result_id_);
    s.store_field("hideViaBot", hide_via_bot_);
    s.store_class_end();
  }
}

sendInlineQueryResultMessage::ReturnType sendInlineQueryResultMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void sendInlineQueryResultMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendInlineQueryResultMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendOptions;").c_str());
    query_id_fieldID = jni::get_field_id(env, Class, "queryId", "J");
    result_id_fieldID = jni::get_field_id(env, Class, "resultId", "Ljava/lang/String;");
    hide_via_bot_fieldID = jni::get_field_id(env, Class, "hideViaBot", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass sendMessage::Class;
jfieldID sendMessage::chat_id_fieldID;
jfieldID sendMessage::message_thread_id_fieldID;
jfieldID sendMessage::reply_to_fieldID;
jfieldID sendMessage::options_fieldID;
jfieldID sendMessage::reply_markup_fieldID;
jfieldID sendMessage::input_message_content_fieldID;

sendMessage::sendMessage()
  : chat_id_()
  , message_thread_id_()
  , reply_to_()
  , options_()
  , reply_markup_()
  , input_message_content_()
{}

sendMessage::sendMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , reply_to_(std::move(reply_to_))
  , options_(std::move(options_))
  , reply_markup_(std::move(reply_markup_))
  , input_message_content_(std::move(input_message_content_))
{}

const std::int32_t sendMessage::ID;

object_ptr<sendMessage> sendMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendMessage> res = make_object<sendMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->options_ = jni::fetch_tl_object<messageSendOptions>(env, jni::fetch_object(env, p, res->options_fieldID));
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  res->input_message_content_ = jni::fetch_tl_object<InputMessageContent>(env, jni::fetch_object(env, p, res->input_message_content_fieldID));
  return res;
}

void sendMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_object_field("options", static_cast<const BaseObject *>(options_.get()));
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_object_field("inputMessageContent", static_cast<const BaseObject *>(input_message_content_.get()));
    s.store_class_end();
  }
}

sendMessage::ReturnType sendMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void sendMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendOptions;").c_str());
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    input_message_content_fieldID = jni::get_field_id(env, Class, "inputMessageContent", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendMessageAlbum::Class;
jfieldID sendMessageAlbum::chat_id_fieldID;
jfieldID sendMessageAlbum::message_thread_id_fieldID;
jfieldID sendMessageAlbum::reply_to_fieldID;
jfieldID sendMessageAlbum::options_fieldID;
jfieldID sendMessageAlbum::input_message_contents_fieldID;

sendMessageAlbum::sendMessageAlbum()
  : chat_id_()
  , message_thread_id_()
  , reply_to_()
  , options_()
  , input_message_contents_()
{}

sendMessageAlbum::sendMessageAlbum(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, array<object_ptr<InputMessageContent>> &&input_message_contents_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , reply_to_(std::move(reply_to_))
  , options_(std::move(options_))
  , input_message_contents_(std::move(input_message_contents_))
{}

const std::int32_t sendMessageAlbum::ID;

object_ptr<sendMessageAlbum> sendMessageAlbum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendMessageAlbum> res = make_object<sendMessageAlbum>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->reply_to_ = jni::fetch_tl_object<InputMessageReplyTo>(env, jni::fetch_object(env, p, res->reply_to_fieldID));
  res->options_ = jni::fetch_tl_object<messageSendOptions>(env, jni::fetch_object(env, p, res->options_fieldID));
  res->input_message_contents_ = jni::FetchVector<InputMessageContent>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->input_message_contents_fieldID));
  return res;
}

void sendMessageAlbum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendMessageAlbum");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("replyTo", static_cast<const BaseObject *>(reply_to_.get()));
    s.store_object_field("options", static_cast<const BaseObject *>(options_.get()));
    { s.store_vector_begin("inputMessageContents", input_message_contents_.size()); for (const auto &_value : input_message_contents_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

sendMessageAlbum::ReturnType sendMessageAlbum::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void sendMessageAlbum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendMessageAlbum").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    reply_to_fieldID = jni::get_field_id(env, Class, "replyTo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputMessageReplyTo;").c_str());
    options_fieldID = jni::get_field_id(env, Class, "options", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSendOptions;").c_str());
    input_message_contents_fieldID = jni::get_field_id(env, Class, "inputMessageContents", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputMessageContent;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendPassportAuthorizationForm::Class;
jfieldID sendPassportAuthorizationForm::authorization_form_id_fieldID;
jfieldID sendPassportAuthorizationForm::types_fieldID;

sendPassportAuthorizationForm::sendPassportAuthorizationForm()
  : authorization_form_id_()
  , types_()
{}

sendPassportAuthorizationForm::sendPassportAuthorizationForm(int32 authorization_form_id_, array<object_ptr<PassportElementType>> &&types_)
  : authorization_form_id_(authorization_form_id_)
  , types_(std::move(types_))
{}

const std::int32_t sendPassportAuthorizationForm::ID;

object_ptr<sendPassportAuthorizationForm> sendPassportAuthorizationForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendPassportAuthorizationForm> res = make_object<sendPassportAuthorizationForm>();
  res->authorization_form_id_ = env->GetIntField(p, res->authorization_form_id_fieldID);
  res->types_ = jni::FetchVector<PassportElementType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->types_fieldID));
  return res;
}

void sendPassportAuthorizationForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendPassportAuthorizationForm");
    s.store_field("authorizationFormId", authorization_form_id_);
    { s.store_vector_begin("types", types_.size()); for (const auto &_value : types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

sendPassportAuthorizationForm::ReturnType sendPassportAuthorizationForm::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendPassportAuthorizationForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendPassportAuthorizationForm").c_str());
    authorization_form_id_fieldID = jni::get_field_id(env, Class, "authorizationFormId", "I");
    types_fieldID = jni::get_field_id(env, Class, "types", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$PassportElementType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendPaymentForm::Class;
jfieldID sendPaymentForm::input_invoice_fieldID;
jfieldID sendPaymentForm::payment_form_id_fieldID;
jfieldID sendPaymentForm::order_info_id_fieldID;
jfieldID sendPaymentForm::shipping_option_id_fieldID;
jfieldID sendPaymentForm::credentials_fieldID;
jfieldID sendPaymentForm::tip_amount_fieldID;

sendPaymentForm::sendPaymentForm()
  : input_invoice_()
  , payment_form_id_()
  , order_info_id_()
  , shipping_option_id_()
  , credentials_()
  , tip_amount_()
{}

sendPaymentForm::sendPaymentForm(object_ptr<InputInvoice> &&input_invoice_, int64 payment_form_id_, string const &order_info_id_, string const &shipping_option_id_, object_ptr<InputCredentials> &&credentials_, int53 tip_amount_)
  : input_invoice_(std::move(input_invoice_))
  , payment_form_id_(payment_form_id_)
  , order_info_id_(order_info_id_)
  , shipping_option_id_(shipping_option_id_)
  , credentials_(std::move(credentials_))
  , tip_amount_(tip_amount_)
{}

const std::int32_t sendPaymentForm::ID;

object_ptr<sendPaymentForm> sendPaymentForm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendPaymentForm> res = make_object<sendPaymentForm>();
  res->input_invoice_ = jni::fetch_tl_object<InputInvoice>(env, jni::fetch_object(env, p, res->input_invoice_fieldID));
  res->payment_form_id_ = env->GetLongField(p, res->payment_form_id_fieldID);
  res->order_info_id_ = jni::fetch_string(env, p, res->order_info_id_fieldID);
  res->shipping_option_id_ = jni::fetch_string(env, p, res->shipping_option_id_fieldID);
  res->credentials_ = jni::fetch_tl_object<InputCredentials>(env, jni::fetch_object(env, p, res->credentials_fieldID));
  res->tip_amount_ = env->GetLongField(p, res->tip_amount_fieldID);
  return res;
}

void sendPaymentForm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendPaymentForm");
    s.store_object_field("inputInvoice", static_cast<const BaseObject *>(input_invoice_.get()));
    s.store_field("paymentFormId", payment_form_id_);
    s.store_field("orderInfoId", order_info_id_);
    s.store_field("shippingOptionId", shipping_option_id_);
    s.store_object_field("credentials", static_cast<const BaseObject *>(credentials_.get()));
    s.store_field("tipAmount", tip_amount_);
    s.store_class_end();
  }
}

sendPaymentForm::ReturnType sendPaymentForm::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return paymentResult::fetch(env, p);
}

void sendPaymentForm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendPaymentForm").c_str());
    input_invoice_fieldID = jni::get_field_id(env, Class, "inputInvoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputInvoice;").c_str());
    payment_form_id_fieldID = jni::get_field_id(env, Class, "paymentFormId", "J");
    order_info_id_fieldID = jni::get_field_id(env, Class, "orderInfoId", "Ljava/lang/String;");
    shipping_option_id_fieldID = jni::get_field_id(env, Class, "shippingOptionId", "Ljava/lang/String;");
    credentials_fieldID = jni::get_field_id(env, Class, "credentials", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputCredentials;").c_str());
    tip_amount_fieldID = jni::get_field_id(env, Class, "tipAmount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass sendPhoneNumberCode::Class;
jfieldID sendPhoneNumberCode::phone_number_fieldID;
jfieldID sendPhoneNumberCode::settings_fieldID;
jfieldID sendPhoneNumberCode::type_fieldID;

sendPhoneNumberCode::sendPhoneNumberCode()
  : phone_number_()
  , settings_()
  , type_()
{}

sendPhoneNumberCode::sendPhoneNumberCode(string const &phone_number_, object_ptr<phoneNumberAuthenticationSettings> &&settings_, object_ptr<PhoneNumberCodeType> &&type_)
  : phone_number_(phone_number_)
  , settings_(std::move(settings_))
  , type_(std::move(type_))
{}

const std::int32_t sendPhoneNumberCode::ID;

object_ptr<sendPhoneNumberCode> sendPhoneNumberCode::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendPhoneNumberCode> res = make_object<sendPhoneNumberCode>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->settings_ = jni::fetch_tl_object<phoneNumberAuthenticationSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  res->type_ = jni::fetch_tl_object<PhoneNumberCodeType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void sendPhoneNumberCode::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendPhoneNumberCode");
    s.store_field("phoneNumber", phone_number_);
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

sendPhoneNumberCode::ReturnType sendPhoneNumberCode::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return authenticationCodeInfo::fetch(env, p);
}

void sendPhoneNumberCode::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendPhoneNumberCode").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PhoneNumberAuthenticationSettings;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PhoneNumberCodeType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass sendPhoneNumberFirebaseSms::Class;
jfieldID sendPhoneNumberFirebaseSms::token_fieldID;

sendPhoneNumberFirebaseSms::sendPhoneNumberFirebaseSms()
  : token_()
{}

sendPhoneNumberFirebaseSms::sendPhoneNumberFirebaseSms(string const &token_)
  : token_(token_)
{}

const std::int32_t sendPhoneNumberFirebaseSms::ID;

object_ptr<sendPhoneNumberFirebaseSms> sendPhoneNumberFirebaseSms::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendPhoneNumberFirebaseSms> res = make_object<sendPhoneNumberFirebaseSms>();
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return res;
}

void sendPhoneNumberFirebaseSms::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendPhoneNumberFirebaseSms");
    s.store_field("token", token_);
    s.store_class_end();
  }
}

sendPhoneNumberFirebaseSms::ReturnType sendPhoneNumberFirebaseSms::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendPhoneNumberFirebaseSms::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendPhoneNumberFirebaseSms").c_str());
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendQuickReplyShortcutMessages::Class;
jfieldID sendQuickReplyShortcutMessages::chat_id_fieldID;
jfieldID sendQuickReplyShortcutMessages::shortcut_id_fieldID;
jfieldID sendQuickReplyShortcutMessages::sending_id_fieldID;

sendQuickReplyShortcutMessages::sendQuickReplyShortcutMessages()
  : chat_id_()
  , shortcut_id_()
  , sending_id_()
{}

sendQuickReplyShortcutMessages::sendQuickReplyShortcutMessages(int53 chat_id_, int32 shortcut_id_, int32 sending_id_)
  : chat_id_(chat_id_)
  , shortcut_id_(shortcut_id_)
  , sending_id_(sending_id_)
{}

const std::int32_t sendQuickReplyShortcutMessages::ID;

object_ptr<sendQuickReplyShortcutMessages> sendQuickReplyShortcutMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendQuickReplyShortcutMessages> res = make_object<sendQuickReplyShortcutMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->sending_id_ = env->GetIntField(p, res->sending_id_fieldID);
  return res;
}

void sendQuickReplyShortcutMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendQuickReplyShortcutMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("shortcutId", shortcut_id_);
    s.store_field("sendingId", sending_id_);
    s.store_class_end();
  }
}

sendQuickReplyShortcutMessages::ReturnType sendQuickReplyShortcutMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return messages::fetch(env, p);
}

void sendQuickReplyShortcutMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendQuickReplyShortcutMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    sending_id_fieldID = jni::get_field_id(env, Class, "sendingId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass sendResoldGift::Class;
jfieldID sendResoldGift::gift_name_fieldID;
jfieldID sendResoldGift::owner_id_fieldID;
jfieldID sendResoldGift::star_count_fieldID;

sendResoldGift::sendResoldGift()
  : gift_name_()
  , owner_id_()
  , star_count_()
{}

sendResoldGift::sendResoldGift(string const &gift_name_, object_ptr<MessageSender> &&owner_id_, int53 star_count_)
  : gift_name_(gift_name_)
  , owner_id_(std::move(owner_id_))
  , star_count_(star_count_)
{}

const std::int32_t sendResoldGift::ID;

object_ptr<sendResoldGift> sendResoldGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendResoldGift> res = make_object<sendResoldGift>();
  res->gift_name_ = jni::fetch_string(env, p, res->gift_name_fieldID);
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void sendResoldGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendResoldGift");
    s.store_field("giftName", gift_name_);
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

sendResoldGift::ReturnType sendResoldGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendResoldGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendResoldGift").c_str());
    gift_name_fieldID = jni::get_field_id(env, Class, "giftName", "Ljava/lang/String;");
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass sendWebAppCustomRequest::Class;
jfieldID sendWebAppCustomRequest::bot_user_id_fieldID;
jfieldID sendWebAppCustomRequest::method_fieldID;
jfieldID sendWebAppCustomRequest::parameters_fieldID;

sendWebAppCustomRequest::sendWebAppCustomRequest()
  : bot_user_id_()
  , method_()
  , parameters_()
{}

sendWebAppCustomRequest::sendWebAppCustomRequest(int53 bot_user_id_, string const &method_, string const &parameters_)
  : bot_user_id_(bot_user_id_)
  , method_(method_)
  , parameters_(parameters_)
{}

const std::int32_t sendWebAppCustomRequest::ID;

object_ptr<sendWebAppCustomRequest> sendWebAppCustomRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendWebAppCustomRequest> res = make_object<sendWebAppCustomRequest>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->method_ = jni::fetch_string(env, p, res->method_fieldID);
  res->parameters_ = jni::fetch_string(env, p, res->parameters_fieldID);
  return res;
}

void sendWebAppCustomRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendWebAppCustomRequest");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("method", method_);
    s.store_field("parameters", parameters_);
    s.store_class_end();
  }
}

sendWebAppCustomRequest::ReturnType sendWebAppCustomRequest::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return customRequestResult::fetch(env, p);
}

void sendWebAppCustomRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendWebAppCustomRequest").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    method_fieldID = jni::get_field_id(env, Class, "method", "Ljava/lang/String;");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass sendWebAppData::Class;
jfieldID sendWebAppData::bot_user_id_fieldID;
jfieldID sendWebAppData::button_text_fieldID;
jfieldID sendWebAppData::data_fieldID;

sendWebAppData::sendWebAppData()
  : bot_user_id_()
  , button_text_()
  , data_()
{}

sendWebAppData::sendWebAppData(int53 bot_user_id_, string const &button_text_, string const &data_)
  : bot_user_id_(bot_user_id_)
  , button_text_(button_text_)
  , data_(data_)
{}

const std::int32_t sendWebAppData::ID;

object_ptr<sendWebAppData> sendWebAppData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sendWebAppData> res = make_object<sendWebAppData>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->button_text_ = jni::fetch_string(env, p, res->button_text_fieldID);
  res->data_ = jni::fetch_string(env, p, res->data_fieldID);
  return res;
}

void sendWebAppData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SendWebAppData");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("buttonText", button_text_);
    s.store_field("data", data_);
    s.store_class_end();
  }
}

sendWebAppData::ReturnType sendWebAppData::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sendWebAppData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SendWebAppData").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    button_text_fieldID = jni::get_field_id(env, Class, "buttonText", "Ljava/lang/String;");
    data_fieldID = jni::get_field_id(env, Class, "data", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setAccentColor::Class;
jfieldID setAccentColor::accent_color_id_fieldID;
jfieldID setAccentColor::background_custom_emoji_id_fieldID;

setAccentColor::setAccentColor()
  : accent_color_id_()
  , background_custom_emoji_id_()
{}

setAccentColor::setAccentColor(int32 accent_color_id_, int64 background_custom_emoji_id_)
  : accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
{}

const std::int32_t setAccentColor::ID;

object_ptr<setAccentColor> setAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAccentColor> res = make_object<setAccentColor>();
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  return res;
}

void setAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAccentColor");
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_class_end();
  }
}

setAccentColor::ReturnType setAccentColor::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAccentColor").c_str());
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setAccountTtl::Class;
jfieldID setAccountTtl::ttl_fieldID;

setAccountTtl::setAccountTtl()
  : ttl_()
{}

setAccountTtl::setAccountTtl(object_ptr<accountTtl> &&ttl_)
  : ttl_(std::move(ttl_))
{}

const std::int32_t setAccountTtl::ID;

object_ptr<setAccountTtl> setAccountTtl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAccountTtl> res = make_object<setAccountTtl>();
  res->ttl_ = jni::fetch_tl_object<accountTtl>(env, jni::fetch_object(env, p, res->ttl_fieldID));
  return res;
}

void setAccountTtl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAccountTtl");
    s.store_object_field("ttl", static_cast<const BaseObject *>(ttl_.get()));
    s.store_class_end();
  }
}

setAccountTtl::ReturnType setAccountTtl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAccountTtl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAccountTtl").c_str());
    ttl_fieldID = jni::get_field_id(env, Class, "ttl", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AccountTtl;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setAlarm::Class;
jfieldID setAlarm::seconds_fieldID;

setAlarm::setAlarm()
  : seconds_()
{}

setAlarm::setAlarm(double seconds_)
  : seconds_(seconds_)
{}

const std::int32_t setAlarm::ID;

object_ptr<setAlarm> setAlarm::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAlarm> res = make_object<setAlarm>();
  res->seconds_ = env->GetDoubleField(p, res->seconds_fieldID);
  return res;
}

void setAlarm::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAlarm");
    s.store_field("seconds", seconds_);
    s.store_class_end();
  }
}

setAlarm::ReturnType setAlarm::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAlarm::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAlarm").c_str());
    seconds_fieldID = jni::get_field_id(env, Class, "seconds", "D");
    return true;
  }();
  (void)is_inited;
}

jclass setApplicationVerificationToken::Class;
jfieldID setApplicationVerificationToken::verification_id_fieldID;
jfieldID setApplicationVerificationToken::token_fieldID;

setApplicationVerificationToken::setApplicationVerificationToken()
  : verification_id_()
  , token_()
{}

setApplicationVerificationToken::setApplicationVerificationToken(int53 verification_id_, string const &token_)
  : verification_id_(verification_id_)
  , token_(token_)
{}

const std::int32_t setApplicationVerificationToken::ID;

object_ptr<setApplicationVerificationToken> setApplicationVerificationToken::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setApplicationVerificationToken> res = make_object<setApplicationVerificationToken>();
  res->verification_id_ = env->GetLongField(p, res->verification_id_fieldID);
  res->token_ = jni::fetch_string(env, p, res->token_fieldID);
  return res;
}

void setApplicationVerificationToken::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetApplicationVerificationToken");
    s.store_field("verificationId", verification_id_);
    s.store_field("token", token_);
    s.store_class_end();
  }
}

setApplicationVerificationToken::ReturnType setApplicationVerificationToken::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setApplicationVerificationToken::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetApplicationVerificationToken").c_str());
    verification_id_fieldID = jni::get_field_id(env, Class, "verificationId", "J");
    token_fieldID = jni::get_field_id(env, Class, "token", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setArchiveChatListSettings::Class;
jfieldID setArchiveChatListSettings::settings_fieldID;

setArchiveChatListSettings::setArchiveChatListSettings()
  : settings_()
{}

setArchiveChatListSettings::setArchiveChatListSettings(object_ptr<archiveChatListSettings> &&settings_)
  : settings_(std::move(settings_))
{}

const std::int32_t setArchiveChatListSettings::ID;

object_ptr<setArchiveChatListSettings> setArchiveChatListSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setArchiveChatListSettings> res = make_object<setArchiveChatListSettings>();
  res->settings_ = jni::fetch_tl_object<archiveChatListSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setArchiveChatListSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetArchiveChatListSettings");
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setArchiveChatListSettings::ReturnType setArchiveChatListSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setArchiveChatListSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetArchiveChatListSettings").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ArchiveChatListSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setAuthenticationEmailAddress::Class;
jfieldID setAuthenticationEmailAddress::email_address_fieldID;

setAuthenticationEmailAddress::setAuthenticationEmailAddress()
  : email_address_()
{}

setAuthenticationEmailAddress::setAuthenticationEmailAddress(string const &email_address_)
  : email_address_(email_address_)
{}

const std::int32_t setAuthenticationEmailAddress::ID;

object_ptr<setAuthenticationEmailAddress> setAuthenticationEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAuthenticationEmailAddress> res = make_object<setAuthenticationEmailAddress>();
  res->email_address_ = jni::fetch_string(env, p, res->email_address_fieldID);
  return res;
}

void setAuthenticationEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAuthenticationEmailAddress");
    s.store_field("emailAddress", email_address_);
    s.store_class_end();
  }
}

setAuthenticationEmailAddress::ReturnType setAuthenticationEmailAddress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAuthenticationEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAuthenticationEmailAddress").c_str());
    email_address_fieldID = jni::get_field_id(env, Class, "emailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setAuthenticationPhoneNumber::Class;
jfieldID setAuthenticationPhoneNumber::phone_number_fieldID;
jfieldID setAuthenticationPhoneNumber::settings_fieldID;

setAuthenticationPhoneNumber::setAuthenticationPhoneNumber()
  : phone_number_()
  , settings_()
{}

setAuthenticationPhoneNumber::setAuthenticationPhoneNumber(string const &phone_number_, object_ptr<phoneNumberAuthenticationSettings> &&settings_)
  : phone_number_(phone_number_)
  , settings_(std::move(settings_))
{}

const std::int32_t setAuthenticationPhoneNumber::ID;

object_ptr<setAuthenticationPhoneNumber> setAuthenticationPhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAuthenticationPhoneNumber> res = make_object<setAuthenticationPhoneNumber>();
  res->phone_number_ = jni::fetch_string(env, p, res->phone_number_fieldID);
  res->settings_ = jni::fetch_tl_object<phoneNumberAuthenticationSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setAuthenticationPhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAuthenticationPhoneNumber");
    s.store_field("phoneNumber", phone_number_);
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setAuthenticationPhoneNumber::ReturnType setAuthenticationPhoneNumber::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAuthenticationPhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAuthenticationPhoneNumber").c_str());
    phone_number_fieldID = jni::get_field_id(env, Class, "phoneNumber", "Ljava/lang/String;");
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PhoneNumberAuthenticationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setAuthenticationPremiumPurchaseTransaction::Class;
jfieldID setAuthenticationPremiumPurchaseTransaction::transaction_fieldID;
jfieldID setAuthenticationPremiumPurchaseTransaction::is_restore_fieldID;
jfieldID setAuthenticationPremiumPurchaseTransaction::currency_fieldID;
jfieldID setAuthenticationPremiumPurchaseTransaction::amount_fieldID;

setAuthenticationPremiumPurchaseTransaction::setAuthenticationPremiumPurchaseTransaction()
  : transaction_()
  , is_restore_()
  , currency_()
  , amount_()
{}

setAuthenticationPremiumPurchaseTransaction::setAuthenticationPremiumPurchaseTransaction(object_ptr<StoreTransaction> &&transaction_, bool is_restore_, string const &currency_, int53 amount_)
  : transaction_(std::move(transaction_))
  , is_restore_(is_restore_)
  , currency_(currency_)
  , amount_(amount_)
{}

const std::int32_t setAuthenticationPremiumPurchaseTransaction::ID;

object_ptr<setAuthenticationPremiumPurchaseTransaction> setAuthenticationPremiumPurchaseTransaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAuthenticationPremiumPurchaseTransaction> res = make_object<setAuthenticationPremiumPurchaseTransaction>();
  res->transaction_ = jni::fetch_tl_object<StoreTransaction>(env, jni::fetch_object(env, p, res->transaction_fieldID));
  res->is_restore_ = (env->GetBooleanField(p, res->is_restore_fieldID) != 0);
  res->currency_ = jni::fetch_string(env, p, res->currency_fieldID);
  res->amount_ = env->GetLongField(p, res->amount_fieldID);
  return res;
}

void setAuthenticationPremiumPurchaseTransaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAuthenticationPremiumPurchaseTransaction");
    s.store_object_field("transaction", static_cast<const BaseObject *>(transaction_.get()));
    s.store_field("isRestore", is_restore_);
    s.store_field("currency", currency_);
    s.store_field("amount", amount_);
    s.store_class_end();
  }
}

setAuthenticationPremiumPurchaseTransaction::ReturnType setAuthenticationPremiumPurchaseTransaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAuthenticationPremiumPurchaseTransaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAuthenticationPremiumPurchaseTransaction").c_str());
    transaction_fieldID = jni::get_field_id(env, Class, "transaction", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoreTransaction;").c_str());
    is_restore_fieldID = jni::get_field_id(env, Class, "isRestore", "Z");
    currency_fieldID = jni::get_field_id(env, Class, "currency", "Ljava/lang/String;");
    amount_fieldID = jni::get_field_id(env, Class, "amount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setAutoDownloadSettings::Class;
jfieldID setAutoDownloadSettings::settings_fieldID;
jfieldID setAutoDownloadSettings::type_fieldID;

setAutoDownloadSettings::setAutoDownloadSettings()
  : settings_()
  , type_()
{}

setAutoDownloadSettings::setAutoDownloadSettings(object_ptr<autoDownloadSettings> &&settings_, object_ptr<NetworkType> &&type_)
  : settings_(std::move(settings_))
  , type_(std::move(type_))
{}

const std::int32_t setAutoDownloadSettings::ID;

object_ptr<setAutoDownloadSettings> setAutoDownloadSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAutoDownloadSettings> res = make_object<setAutoDownloadSettings>();
  res->settings_ = jni::fetch_tl_object<autoDownloadSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  res->type_ = jni::fetch_tl_object<NetworkType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void setAutoDownloadSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAutoDownloadSettings");
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

setAutoDownloadSettings::ReturnType setAutoDownloadSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAutoDownloadSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAutoDownloadSettings").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutoDownloadSettings;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NetworkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setAutosaveSettings::Class;
jfieldID setAutosaveSettings::scope_fieldID;
jfieldID setAutosaveSettings::settings_fieldID;

setAutosaveSettings::setAutosaveSettings()
  : scope_()
  , settings_()
{}

setAutosaveSettings::setAutosaveSettings(object_ptr<AutosaveSettingsScope> &&scope_, object_ptr<scopeAutosaveSettings> &&settings_)
  : scope_(std::move(scope_))
  , settings_(std::move(settings_))
{}

const std::int32_t setAutosaveSettings::ID;

object_ptr<setAutosaveSettings> setAutosaveSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setAutosaveSettings> res = make_object<setAutosaveSettings>();
  res->scope_ = jni::fetch_tl_object<AutosaveSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->settings_ = jni::fetch_tl_object<scopeAutosaveSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setAutosaveSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetAutosaveSettings");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setAutosaveSettings::ReturnType setAutosaveSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setAutosaveSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetAutosaveSettings").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AutosaveSettingsScope;").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeAutosaveSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBio::Class;
jfieldID setBio::bio_fieldID;

setBio::setBio()
  : bio_()
{}

setBio::setBio(string const &bio_)
  : bio_(bio_)
{}

const std::int32_t setBio::ID;

object_ptr<setBio> setBio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBio> res = make_object<setBio>();
  res->bio_ = jni::fetch_string(env, p, res->bio_fieldID);
  return res;
}

void setBio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBio");
    s.store_field("bio", bio_);
    s.store_class_end();
  }
}

setBio::ReturnType setBio::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBio").c_str());
    bio_fieldID = jni::get_field_id(env, Class, "bio", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBirthdate::Class;
jfieldID setBirthdate::birthdate_fieldID;

setBirthdate::setBirthdate()
  : birthdate_()
{}

setBirthdate::setBirthdate(object_ptr<birthdate> &&birthdate_)
  : birthdate_(std::move(birthdate_))
{}

const std::int32_t setBirthdate::ID;

object_ptr<setBirthdate> setBirthdate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBirthdate> res = make_object<setBirthdate>();
  res->birthdate_ = jni::fetch_tl_object<birthdate>(env, jni::fetch_object(env, p, res->birthdate_fieldID));
  return res;
}

void setBirthdate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBirthdate");
    s.store_object_field("birthdate", static_cast<const BaseObject *>(birthdate_.get()));
    s.store_class_end();
  }
}

setBirthdate::ReturnType setBirthdate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBirthdate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBirthdate").c_str());
    birthdate_fieldID = jni::get_field_id(env, Class, "birthdate", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Birthdate;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBotInfoDescription::Class;
jfieldID setBotInfoDescription::bot_user_id_fieldID;
jfieldID setBotInfoDescription::language_code_fieldID;
jfieldID setBotInfoDescription::description_fieldID;

setBotInfoDescription::setBotInfoDescription()
  : bot_user_id_()
  , language_code_()
  , description_()
{}

setBotInfoDescription::setBotInfoDescription(int53 bot_user_id_, string const &language_code_, string const &description_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , description_(description_)
{}

const std::int32_t setBotInfoDescription::ID;

object_ptr<setBotInfoDescription> setBotInfoDescription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBotInfoDescription> res = make_object<setBotInfoDescription>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return res;
}

void setBotInfoDescription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBotInfoDescription");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

setBotInfoDescription::ReturnType setBotInfoDescription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBotInfoDescription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBotInfoDescription").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBotInfoShortDescription::Class;
jfieldID setBotInfoShortDescription::bot_user_id_fieldID;
jfieldID setBotInfoShortDescription::language_code_fieldID;
jfieldID setBotInfoShortDescription::short_description_fieldID;

setBotInfoShortDescription::setBotInfoShortDescription()
  : bot_user_id_()
  , language_code_()
  , short_description_()
{}

setBotInfoShortDescription::setBotInfoShortDescription(int53 bot_user_id_, string const &language_code_, string const &short_description_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , short_description_(short_description_)
{}

const std::int32_t setBotInfoShortDescription::ID;

object_ptr<setBotInfoShortDescription> setBotInfoShortDescription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBotInfoShortDescription> res = make_object<setBotInfoShortDescription>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->short_description_ = jni::fetch_string(env, p, res->short_description_fieldID);
  return res;
}

void setBotInfoShortDescription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBotInfoShortDescription");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_field("shortDescription", short_description_);
    s.store_class_end();
  }
}

setBotInfoShortDescription::ReturnType setBotInfoShortDescription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBotInfoShortDescription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBotInfoShortDescription").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    short_description_fieldID = jni::get_field_id(env, Class, "shortDescription", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBotName::Class;
jfieldID setBotName::bot_user_id_fieldID;
jfieldID setBotName::language_code_fieldID;
jfieldID setBotName::name_fieldID;

setBotName::setBotName()
  : bot_user_id_()
  , language_code_()
  , name_()
{}

setBotName::setBotName(int53 bot_user_id_, string const &language_code_, string const &name_)
  : bot_user_id_(bot_user_id_)
  , language_code_(language_code_)
  , name_(name_)
{}

const std::int32_t setBotName::ID;

object_ptr<setBotName> setBotName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBotName> res = make_object<setBotName>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void setBotName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBotName");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("languageCode", language_code_);
    s.store_field("name", name_);
    s.store_class_end();
  }
}

setBotName::ReturnType setBotName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBotName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBotName").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBotProfilePhoto::Class;
jfieldID setBotProfilePhoto::bot_user_id_fieldID;
jfieldID setBotProfilePhoto::photo_fieldID;

setBotProfilePhoto::setBotProfilePhoto()
  : bot_user_id_()
  , photo_()
{}

setBotProfilePhoto::setBotProfilePhoto(int53 bot_user_id_, object_ptr<InputChatPhoto> &&photo_)
  : bot_user_id_(bot_user_id_)
  , photo_(std::move(photo_))
{}

const std::int32_t setBotProfilePhoto::ID;

object_ptr<setBotProfilePhoto> setBotProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBotProfilePhoto> res = make_object<setBotProfilePhoto>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void setBotProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBotProfilePhoto");
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

setBotProfilePhoto::ReturnType setBotProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBotProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBotProfilePhoto").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBotUpdatesStatus::Class;
jfieldID setBotUpdatesStatus::pending_update_count_fieldID;
jfieldID setBotUpdatesStatus::error_message_fieldID;

setBotUpdatesStatus::setBotUpdatesStatus()
  : pending_update_count_()
  , error_message_()
{}

setBotUpdatesStatus::setBotUpdatesStatus(int32 pending_update_count_, string const &error_message_)
  : pending_update_count_(pending_update_count_)
  , error_message_(error_message_)
{}

const std::int32_t setBotUpdatesStatus::ID;

object_ptr<setBotUpdatesStatus> setBotUpdatesStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBotUpdatesStatus> res = make_object<setBotUpdatesStatus>();
  res->pending_update_count_ = env->GetIntField(p, res->pending_update_count_fieldID);
  res->error_message_ = jni::fetch_string(env, p, res->error_message_fieldID);
  return res;
}

void setBotUpdatesStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBotUpdatesStatus");
    s.store_field("pendingUpdateCount", pending_update_count_);
    s.store_field("errorMessage", error_message_);
    s.store_class_end();
  }
}

setBotUpdatesStatus::ReturnType setBotUpdatesStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBotUpdatesStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBotUpdatesStatus").c_str());
    pending_update_count_fieldID = jni::get_field_id(env, Class, "pendingUpdateCount", "I");
    error_message_fieldID = jni::get_field_id(env, Class, "errorMessage", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAccountBio::Class;
jfieldID setBusinessAccountBio::business_connection_id_fieldID;
jfieldID setBusinessAccountBio::bio_fieldID;

setBusinessAccountBio::setBusinessAccountBio()
  : business_connection_id_()
  , bio_()
{}

setBusinessAccountBio::setBusinessAccountBio(string const &business_connection_id_, string const &bio_)
  : business_connection_id_(business_connection_id_)
  , bio_(bio_)
{}

const std::int32_t setBusinessAccountBio::ID;

object_ptr<setBusinessAccountBio> setBusinessAccountBio::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAccountBio> res = make_object<setBusinessAccountBio>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->bio_ = jni::fetch_string(env, p, res->bio_fieldID);
  return res;
}

void setBusinessAccountBio::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAccountBio");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("bio", bio_);
    s.store_class_end();
  }
}

setBusinessAccountBio::ReturnType setBusinessAccountBio::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAccountBio::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAccountBio").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    bio_fieldID = jni::get_field_id(env, Class, "bio", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAccountGiftSettings::Class;
jfieldID setBusinessAccountGiftSettings::business_connection_id_fieldID;
jfieldID setBusinessAccountGiftSettings::settings_fieldID;

setBusinessAccountGiftSettings::setBusinessAccountGiftSettings()
  : business_connection_id_()
  , settings_()
{}

setBusinessAccountGiftSettings::setBusinessAccountGiftSettings(string const &business_connection_id_, object_ptr<giftSettings> &&settings_)
  : business_connection_id_(business_connection_id_)
  , settings_(std::move(settings_))
{}

const std::int32_t setBusinessAccountGiftSettings::ID;

object_ptr<setBusinessAccountGiftSettings> setBusinessAccountGiftSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAccountGiftSettings> res = make_object<setBusinessAccountGiftSettings>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->settings_ = jni::fetch_tl_object<giftSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setBusinessAccountGiftSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAccountGiftSettings");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setBusinessAccountGiftSettings::ReturnType setBusinessAccountGiftSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAccountGiftSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAccountGiftSettings").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiftSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAccountName::Class;
jfieldID setBusinessAccountName::business_connection_id_fieldID;
jfieldID setBusinessAccountName::first_name_fieldID;
jfieldID setBusinessAccountName::last_name_fieldID;

setBusinessAccountName::setBusinessAccountName()
  : business_connection_id_()
  , first_name_()
  , last_name_()
{}

setBusinessAccountName::setBusinessAccountName(string const &business_connection_id_, string const &first_name_, string const &last_name_)
  : business_connection_id_(business_connection_id_)
  , first_name_(first_name_)
  , last_name_(last_name_)
{}

const std::int32_t setBusinessAccountName::ID;

object_ptr<setBusinessAccountName> setBusinessAccountName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAccountName> res = make_object<setBusinessAccountName>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  return res;
}

void setBusinessAccountName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAccountName");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_class_end();
  }
}

setBusinessAccountName::ReturnType setBusinessAccountName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAccountName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAccountName").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAccountProfilePhoto::Class;
jfieldID setBusinessAccountProfilePhoto::business_connection_id_fieldID;
jfieldID setBusinessAccountProfilePhoto::photo_fieldID;
jfieldID setBusinessAccountProfilePhoto::is_public_fieldID;

setBusinessAccountProfilePhoto::setBusinessAccountProfilePhoto()
  : business_connection_id_()
  , photo_()
  , is_public_()
{}

setBusinessAccountProfilePhoto::setBusinessAccountProfilePhoto(string const &business_connection_id_, object_ptr<InputChatPhoto> &&photo_, bool is_public_)
  : business_connection_id_(business_connection_id_)
  , photo_(std::move(photo_))
  , is_public_(is_public_)
{}

const std::int32_t setBusinessAccountProfilePhoto::ID;

object_ptr<setBusinessAccountProfilePhoto> setBusinessAccountProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAccountProfilePhoto> res = make_object<setBusinessAccountProfilePhoto>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  return res;
}

void setBusinessAccountProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAccountProfilePhoto");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("isPublic", is_public_);
    s.store_class_end();
  }
}

setBusinessAccountProfilePhoto::ReturnType setBusinessAccountProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAccountProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAccountProfilePhoto").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAccountUsername::Class;
jfieldID setBusinessAccountUsername::business_connection_id_fieldID;
jfieldID setBusinessAccountUsername::username_fieldID;

setBusinessAccountUsername::setBusinessAccountUsername()
  : business_connection_id_()
  , username_()
{}

setBusinessAccountUsername::setBusinessAccountUsername(string const &business_connection_id_, string const &username_)
  : business_connection_id_(business_connection_id_)
  , username_(username_)
{}

const std::int32_t setBusinessAccountUsername::ID;

object_ptr<setBusinessAccountUsername> setBusinessAccountUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAccountUsername> res = make_object<setBusinessAccountUsername>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return res;
}

void setBusinessAccountUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAccountUsername");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("username", username_);
    s.store_class_end();
  }
}

setBusinessAccountUsername::ReturnType setBusinessAccountUsername::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAccountUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAccountUsername").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessAwayMessageSettings::Class;
jfieldID setBusinessAwayMessageSettings::away_message_settings_fieldID;

setBusinessAwayMessageSettings::setBusinessAwayMessageSettings()
  : away_message_settings_()
{}

setBusinessAwayMessageSettings::setBusinessAwayMessageSettings(object_ptr<businessAwayMessageSettings> &&away_message_settings_)
  : away_message_settings_(std::move(away_message_settings_))
{}

const std::int32_t setBusinessAwayMessageSettings::ID;

object_ptr<setBusinessAwayMessageSettings> setBusinessAwayMessageSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessAwayMessageSettings> res = make_object<setBusinessAwayMessageSettings>();
  res->away_message_settings_ = jni::fetch_tl_object<businessAwayMessageSettings>(env, jni::fetch_object(env, p, res->away_message_settings_fieldID));
  return res;
}

void setBusinessAwayMessageSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessAwayMessageSettings");
    s.store_object_field("awayMessageSettings", static_cast<const BaseObject *>(away_message_settings_.get()));
    s.store_class_end();
  }
}

setBusinessAwayMessageSettings::ReturnType setBusinessAwayMessageSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessAwayMessageSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessAwayMessageSettings").c_str());
    away_message_settings_fieldID = jni::get_field_id(env, Class, "awayMessageSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessAwayMessageSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessConnectedBot::Class;
jfieldID setBusinessConnectedBot::bot_fieldID;

setBusinessConnectedBot::setBusinessConnectedBot()
  : bot_()
{}

setBusinessConnectedBot::setBusinessConnectedBot(object_ptr<businessConnectedBot> &&bot_)
  : bot_(std::move(bot_))
{}

const std::int32_t setBusinessConnectedBot::ID;

object_ptr<setBusinessConnectedBot> setBusinessConnectedBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessConnectedBot> res = make_object<setBusinessConnectedBot>();
  res->bot_ = jni::fetch_tl_object<businessConnectedBot>(env, jni::fetch_object(env, p, res->bot_fieldID));
  return res;
}

void setBusinessConnectedBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessConnectedBot");
    s.store_object_field("bot", static_cast<const BaseObject *>(bot_.get()));
    s.store_class_end();
  }
}

setBusinessConnectedBot::ReturnType setBusinessConnectedBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessConnectedBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessConnectedBot").c_str());
    bot_fieldID = jni::get_field_id(env, Class, "bot", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessConnectedBot;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessGreetingMessageSettings::Class;
jfieldID setBusinessGreetingMessageSettings::greeting_message_settings_fieldID;

setBusinessGreetingMessageSettings::setBusinessGreetingMessageSettings()
  : greeting_message_settings_()
{}

setBusinessGreetingMessageSettings::setBusinessGreetingMessageSettings(object_ptr<businessGreetingMessageSettings> &&greeting_message_settings_)
  : greeting_message_settings_(std::move(greeting_message_settings_))
{}

const std::int32_t setBusinessGreetingMessageSettings::ID;

object_ptr<setBusinessGreetingMessageSettings> setBusinessGreetingMessageSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessGreetingMessageSettings> res = make_object<setBusinessGreetingMessageSettings>();
  res->greeting_message_settings_ = jni::fetch_tl_object<businessGreetingMessageSettings>(env, jni::fetch_object(env, p, res->greeting_message_settings_fieldID));
  return res;
}

void setBusinessGreetingMessageSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessGreetingMessageSettings");
    s.store_object_field("greetingMessageSettings", static_cast<const BaseObject *>(greeting_message_settings_.get()));
    s.store_class_end();
  }
}

setBusinessGreetingMessageSettings::ReturnType setBusinessGreetingMessageSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessGreetingMessageSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessGreetingMessageSettings").c_str());
    greeting_message_settings_fieldID = jni::get_field_id(env, Class, "greetingMessageSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessGreetingMessageSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessLocation::Class;
jfieldID setBusinessLocation::location_fieldID;

setBusinessLocation::setBusinessLocation()
  : location_()
{}

setBusinessLocation::setBusinessLocation(object_ptr<businessLocation> &&location_)
  : location_(std::move(location_))
{}

const std::int32_t setBusinessLocation::ID;

object_ptr<setBusinessLocation> setBusinessLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessLocation> res = make_object<setBusinessLocation>();
  res->location_ = jni::fetch_tl_object<businessLocation>(env, jni::fetch_object(env, p, res->location_fieldID));
  return res;
}

void setBusinessLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessLocation");
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_class_end();
  }
}

setBusinessLocation::ReturnType setBusinessLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessLocation").c_str());
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessLocation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessMessageIsPinned::Class;
jfieldID setBusinessMessageIsPinned::business_connection_id_fieldID;
jfieldID setBusinessMessageIsPinned::chat_id_fieldID;
jfieldID setBusinessMessageIsPinned::message_id_fieldID;
jfieldID setBusinessMessageIsPinned::is_pinned_fieldID;

setBusinessMessageIsPinned::setBusinessMessageIsPinned()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , is_pinned_()
{}

setBusinessMessageIsPinned::setBusinessMessageIsPinned(string const &business_connection_id_, int53 chat_id_, int53 message_id_, bool is_pinned_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t setBusinessMessageIsPinned::ID;

object_ptr<setBusinessMessageIsPinned> setBusinessMessageIsPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessMessageIsPinned> res = make_object<setBusinessMessageIsPinned>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return res;
}

void setBusinessMessageIsPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessMessageIsPinned");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

setBusinessMessageIsPinned::ReturnType setBusinessMessageIsPinned::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessMessageIsPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessMessageIsPinned").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessOpeningHours::Class;
jfieldID setBusinessOpeningHours::opening_hours_fieldID;

setBusinessOpeningHours::setBusinessOpeningHours()
  : opening_hours_()
{}

setBusinessOpeningHours::setBusinessOpeningHours(object_ptr<businessOpeningHours> &&opening_hours_)
  : opening_hours_(std::move(opening_hours_))
{}

const std::int32_t setBusinessOpeningHours::ID;

object_ptr<setBusinessOpeningHours> setBusinessOpeningHours::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessOpeningHours> res = make_object<setBusinessOpeningHours>();
  res->opening_hours_ = jni::fetch_tl_object<businessOpeningHours>(env, jni::fetch_object(env, p, res->opening_hours_fieldID));
  return res;
}

void setBusinessOpeningHours::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessOpeningHours");
    s.store_object_field("openingHours", static_cast<const BaseObject *>(opening_hours_.get()));
    s.store_class_end();
  }
}

setBusinessOpeningHours::ReturnType setBusinessOpeningHours::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessOpeningHours::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessOpeningHours").c_str());
    opening_hours_fieldID = jni::get_field_id(env, Class, "openingHours", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BusinessOpeningHours;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setBusinessStartPage::Class;
jfieldID setBusinessStartPage::start_page_fieldID;

setBusinessStartPage::setBusinessStartPage()
  : start_page_()
{}

setBusinessStartPage::setBusinessStartPage(object_ptr<inputBusinessStartPage> &&start_page_)
  : start_page_(std::move(start_page_))
{}

const std::int32_t setBusinessStartPage::ID;

object_ptr<setBusinessStartPage> setBusinessStartPage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setBusinessStartPage> res = make_object<setBusinessStartPage>();
  res->start_page_ = jni::fetch_tl_object<inputBusinessStartPage>(env, jni::fetch_object(env, p, res->start_page_fieldID));
  return res;
}

void setBusinessStartPage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetBusinessStartPage");
    s.store_object_field("startPage", static_cast<const BaseObject *>(start_page_.get()));
    s.store_class_end();
  }
}

setBusinessStartPage::ReturnType setBusinessStartPage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setBusinessStartPage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetBusinessStartPage").c_str());
    start_page_fieldID = jni::get_field_id(env, Class, "startPage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputBusinessStartPage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatAccentColor::Class;
jfieldID setChatAccentColor::chat_id_fieldID;
jfieldID setChatAccentColor::accent_color_id_fieldID;
jfieldID setChatAccentColor::background_custom_emoji_id_fieldID;

setChatAccentColor::setChatAccentColor()
  : chat_id_()
  , accent_color_id_()
  , background_custom_emoji_id_()
{}

setChatAccentColor::setChatAccentColor(int53 chat_id_, int32 accent_color_id_, int64 background_custom_emoji_id_)
  : chat_id_(chat_id_)
  , accent_color_id_(accent_color_id_)
  , background_custom_emoji_id_(background_custom_emoji_id_)
{}

const std::int32_t setChatAccentColor::ID;

object_ptr<setChatAccentColor> setChatAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatAccentColor> res = make_object<setChatAccentColor>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->accent_color_id_ = env->GetIntField(p, res->accent_color_id_fieldID);
  res->background_custom_emoji_id_ = env->GetLongField(p, res->background_custom_emoji_id_fieldID);
  return res;
}

void setChatAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatAccentColor");
    s.store_field("chatId", chat_id_);
    s.store_field("accentColorId", accent_color_id_);
    s.store_field("backgroundCustomEmojiId", background_custom_emoji_id_);
    s.store_class_end();
  }
}

setChatAccentColor::ReturnType setChatAccentColor::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatAccentColor").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    accent_color_id_fieldID = jni::get_field_id(env, Class, "accentColorId", "I");
    background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "backgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setChatActiveStoriesList::Class;
jfieldID setChatActiveStoriesList::chat_id_fieldID;
jfieldID setChatActiveStoriesList::story_list_fieldID;

setChatActiveStoriesList::setChatActiveStoriesList()
  : chat_id_()
  , story_list_()
{}

setChatActiveStoriesList::setChatActiveStoriesList(int53 chat_id_, object_ptr<StoryList> &&story_list_)
  : chat_id_(chat_id_)
  , story_list_(std::move(story_list_))
{}

const std::int32_t setChatActiveStoriesList::ID;

object_ptr<setChatActiveStoriesList> setChatActiveStoriesList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatActiveStoriesList> res = make_object<setChatActiveStoriesList>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->story_list_ = jni::fetch_tl_object<StoryList>(env, jni::fetch_object(env, p, res->story_list_fieldID));
  return res;
}

void setChatActiveStoriesList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatActiveStoriesList");
    s.store_field("chatId", chat_id_);
    s.store_object_field("storyList", static_cast<const BaseObject *>(story_list_.get()));
    s.store_class_end();
  }
}

setChatActiveStoriesList::ReturnType setChatActiveStoriesList::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatActiveStoriesList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatActiveStoriesList").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    story_list_fieldID = jni::get_field_id(env, Class, "storyList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatAffiliateProgram::Class;
jfieldID setChatAffiliateProgram::chat_id_fieldID;
jfieldID setChatAffiliateProgram::parameters_fieldID;

setChatAffiliateProgram::setChatAffiliateProgram()
  : chat_id_()
  , parameters_()
{}

setChatAffiliateProgram::setChatAffiliateProgram(int53 chat_id_, object_ptr<affiliateProgramParameters> &&parameters_)
  : chat_id_(chat_id_)
  , parameters_(std::move(parameters_))
{}

const std::int32_t setChatAffiliateProgram::ID;

object_ptr<setChatAffiliateProgram> setChatAffiliateProgram::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatAffiliateProgram> res = make_object<setChatAffiliateProgram>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->parameters_ = jni::fetch_tl_object<affiliateProgramParameters>(env, jni::fetch_object(env, p, res->parameters_fieldID));
  return res;
}

void setChatAffiliateProgram::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatAffiliateProgram");
    s.store_field("chatId", chat_id_);
    s.store_object_field("parameters", static_cast<const BaseObject *>(parameters_.get()));
    s.store_class_end();
  }
}

setChatAffiliateProgram::ReturnType setChatAffiliateProgram::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatAffiliateProgram::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatAffiliateProgram").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    parameters_fieldID = jni::get_field_id(env, Class, "parameters", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$AffiliateProgramParameters;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatAvailableReactions::Class;
jfieldID setChatAvailableReactions::chat_id_fieldID;
jfieldID setChatAvailableReactions::available_reactions_fieldID;

setChatAvailableReactions::setChatAvailableReactions()
  : chat_id_()
  , available_reactions_()
{}

setChatAvailableReactions::setChatAvailableReactions(int53 chat_id_, object_ptr<ChatAvailableReactions> &&available_reactions_)
  : chat_id_(chat_id_)
  , available_reactions_(std::move(available_reactions_))
{}

const std::int32_t setChatAvailableReactions::ID;

object_ptr<setChatAvailableReactions> setChatAvailableReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatAvailableReactions> res = make_object<setChatAvailableReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->available_reactions_ = jni::fetch_tl_object<ChatAvailableReactions>(env, jni::fetch_object(env, p, res->available_reactions_fieldID));
  return res;
}

void setChatAvailableReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatAvailableReactions");
    s.store_field("chatId", chat_id_);
    s.store_object_field("availableReactions", static_cast<const BaseObject *>(available_reactions_.get()));
    s.store_class_end();
  }
}

setChatAvailableReactions::ReturnType setChatAvailableReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatAvailableReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatAvailableReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    available_reactions_fieldID = jni::get_field_id(env, Class, "availableReactions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAvailableReactions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatBackground::Class;
jfieldID setChatBackground::chat_id_fieldID;
jfieldID setChatBackground::background_fieldID;
jfieldID setChatBackground::type_fieldID;
jfieldID setChatBackground::dark_theme_dimming_fieldID;
jfieldID setChatBackground::only_for_self_fieldID;

setChatBackground::setChatBackground()
  : chat_id_()
  , background_()
  , type_()
  , dark_theme_dimming_()
  , only_for_self_()
{}

setChatBackground::setChatBackground(int53 chat_id_, object_ptr<InputBackground> &&background_, object_ptr<BackgroundType> &&type_, int32 dark_theme_dimming_, bool only_for_self_)
  : chat_id_(chat_id_)
  , background_(std::move(background_))
  , type_(std::move(type_))
  , dark_theme_dimming_(dark_theme_dimming_)
  , only_for_self_(only_for_self_)
{}

const std::int32_t setChatBackground::ID;

object_ptr<setChatBackground> setChatBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatBackground> res = make_object<setChatBackground>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->background_ = jni::fetch_tl_object<InputBackground>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->type_ = jni::fetch_tl_object<BackgroundType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->dark_theme_dimming_ = env->GetIntField(p, res->dark_theme_dimming_fieldID);
  res->only_for_self_ = (env->GetBooleanField(p, res->only_for_self_fieldID) != 0);
  return res;
}

void setChatBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatBackground");
    s.store_field("chatId", chat_id_);
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("darkThemeDimming", dark_theme_dimming_);
    s.store_field("onlyForSelf", only_for_self_);
    s.store_class_end();
  }
}

setChatBackground::ReturnType setChatBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatBackground").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputBackground;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundType;").c_str());
    dark_theme_dimming_fieldID = jni::get_field_id(env, Class, "darkThemeDimming", "I");
    only_for_self_fieldID = jni::get_field_id(env, Class, "onlyForSelf", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setChatClientData::Class;
jfieldID setChatClientData::chat_id_fieldID;
jfieldID setChatClientData::client_data_fieldID;

setChatClientData::setChatClientData()
  : chat_id_()
  , client_data_()
{}

setChatClientData::setChatClientData(int53 chat_id_, string const &client_data_)
  : chat_id_(chat_id_)
  , client_data_(client_data_)
{}

const std::int32_t setChatClientData::ID;

object_ptr<setChatClientData> setChatClientData::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatClientData> res = make_object<setChatClientData>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->client_data_ = jni::fetch_string(env, p, res->client_data_fieldID);
  return res;
}

void setChatClientData::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatClientData");
    s.store_field("chatId", chat_id_);
    s.store_field("clientData", client_data_);
    s.store_class_end();
  }
}

setChatClientData::ReturnType setChatClientData::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatClientData::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatClientData").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    client_data_fieldID = jni::get_field_id(env, Class, "clientData", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setChatDescription::Class;
jfieldID setChatDescription::chat_id_fieldID;
jfieldID setChatDescription::description_fieldID;

setChatDescription::setChatDescription()
  : chat_id_()
  , description_()
{}

setChatDescription::setChatDescription(int53 chat_id_, string const &description_)
  : chat_id_(chat_id_)
  , description_(description_)
{}

const std::int32_t setChatDescription::ID;

object_ptr<setChatDescription> setChatDescription::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatDescription> res = make_object<setChatDescription>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->description_ = jni::fetch_string(env, p, res->description_fieldID);
  return res;
}

void setChatDescription::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatDescription");
    s.store_field("chatId", chat_id_);
    s.store_field("description", description_);
    s.store_class_end();
  }
}

setChatDescription::ReturnType setChatDescription::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatDescription::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatDescription").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    description_fieldID = jni::get_field_id(env, Class, "description", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setChatDiscussionGroup::Class;
jfieldID setChatDiscussionGroup::chat_id_fieldID;
jfieldID setChatDiscussionGroup::discussion_chat_id_fieldID;

setChatDiscussionGroup::setChatDiscussionGroup()
  : chat_id_()
  , discussion_chat_id_()
{}

setChatDiscussionGroup::setChatDiscussionGroup(int53 chat_id_, int53 discussion_chat_id_)
  : chat_id_(chat_id_)
  , discussion_chat_id_(discussion_chat_id_)
{}

const std::int32_t setChatDiscussionGroup::ID;

object_ptr<setChatDiscussionGroup> setChatDiscussionGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatDiscussionGroup> res = make_object<setChatDiscussionGroup>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->discussion_chat_id_ = env->GetLongField(p, res->discussion_chat_id_fieldID);
  return res;
}

void setChatDiscussionGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatDiscussionGroup");
    s.store_field("chatId", chat_id_);
    s.store_field("discussionChatId", discussion_chat_id_);
    s.store_class_end();
  }
}

setChatDiscussionGroup::ReturnType setChatDiscussionGroup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatDiscussionGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatDiscussionGroup").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    discussion_chat_id_fieldID = jni::get_field_id(env, Class, "discussionChatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setChatDraftMessage::Class;
jfieldID setChatDraftMessage::chat_id_fieldID;
jfieldID setChatDraftMessage::message_thread_id_fieldID;
jfieldID setChatDraftMessage::draft_message_fieldID;

setChatDraftMessage::setChatDraftMessage()
  : chat_id_()
  , message_thread_id_()
  , draft_message_()
{}

setChatDraftMessage::setChatDraftMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<draftMessage> &&draft_message_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , draft_message_(std::move(draft_message_))
{}

const std::int32_t setChatDraftMessage::ID;

object_ptr<setChatDraftMessage> setChatDraftMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatDraftMessage> res = make_object<setChatDraftMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->draft_message_ = jni::fetch_tl_object<draftMessage>(env, jni::fetch_object(env, p, res->draft_message_fieldID));
  return res;
}

void setChatDraftMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatDraftMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("draftMessage", static_cast<const BaseObject *>(draft_message_.get()));
    s.store_class_end();
  }
}

setChatDraftMessage::ReturnType setChatDraftMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatDraftMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatDraftMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    draft_message_fieldID = jni::get_field_id(env, Class, "draftMessage", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$DraftMessage;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatEmojiStatus::Class;
jfieldID setChatEmojiStatus::chat_id_fieldID;
jfieldID setChatEmojiStatus::emoji_status_fieldID;

setChatEmojiStatus::setChatEmojiStatus()
  : chat_id_()
  , emoji_status_()
{}

setChatEmojiStatus::setChatEmojiStatus(int53 chat_id_, object_ptr<emojiStatus> &&emoji_status_)
  : chat_id_(chat_id_)
  , emoji_status_(std::move(emoji_status_))
{}

const std::int32_t setChatEmojiStatus::ID;

object_ptr<setChatEmojiStatus> setChatEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatEmojiStatus> res = make_object<setChatEmojiStatus>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  return res;
}

void setChatEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatEmojiStatus");
    s.store_field("chatId", chat_id_);
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_class_end();
  }
}

setChatEmojiStatus::ReturnType setChatEmojiStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatEmojiStatus").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatLocation::Class;
jfieldID setChatLocation::chat_id_fieldID;
jfieldID setChatLocation::location_fieldID;

setChatLocation::setChatLocation()
  : chat_id_()
  , location_()
{}

setChatLocation::setChatLocation(int53 chat_id_, object_ptr<chatLocation> &&location_)
  : chat_id_(chat_id_)
  , location_(std::move(location_))
{}

const std::int32_t setChatLocation::ID;

object_ptr<setChatLocation> setChatLocation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatLocation> res = make_object<setChatLocation>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->location_ = jni::fetch_tl_object<chatLocation>(env, jni::fetch_object(env, p, res->location_fieldID));
  return res;
}

void setChatLocation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatLocation");
    s.store_field("chatId", chat_id_);
    s.store_object_field("location", static_cast<const BaseObject *>(location_.get()));
    s.store_class_end();
  }
}

setChatLocation::ReturnType setChatLocation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatLocation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatLocation").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    location_fieldID = jni::get_field_id(env, Class, "location", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatLocation;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatMemberStatus::Class;
jfieldID setChatMemberStatus::chat_id_fieldID;
jfieldID setChatMemberStatus::member_id_fieldID;
jfieldID setChatMemberStatus::status_fieldID;

setChatMemberStatus::setChatMemberStatus()
  : chat_id_()
  , member_id_()
  , status_()
{}

setChatMemberStatus::setChatMemberStatus(int53 chat_id_, object_ptr<MessageSender> &&member_id_, object_ptr<ChatMemberStatus> &&status_)
  : chat_id_(chat_id_)
  , member_id_(std::move(member_id_))
  , status_(std::move(status_))
{}

const std::int32_t setChatMemberStatus::ID;

object_ptr<setChatMemberStatus> setChatMemberStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatMemberStatus> res = make_object<setChatMemberStatus>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->member_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->member_id_fieldID));
  res->status_ = jni::fetch_tl_object<ChatMemberStatus>(env, jni::fetch_object(env, p, res->status_fieldID));
  return res;
}

void setChatMemberStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatMemberStatus");
    s.store_field("chatId", chat_id_);
    s.store_object_field("memberId", static_cast<const BaseObject *>(member_id_.get()));
    s.store_object_field("status", static_cast<const BaseObject *>(status_.get()));
    s.store_class_end();
  }
}

setChatMemberStatus::ReturnType setChatMemberStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatMemberStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatMemberStatus").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    member_id_fieldID = jni::get_field_id(env, Class, "memberId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    status_fieldID = jni::get_field_id(env, Class, "status", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatMemberStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatMessageAutoDeleteTime::Class;
jfieldID setChatMessageAutoDeleteTime::chat_id_fieldID;
jfieldID setChatMessageAutoDeleteTime::message_auto_delete_time_fieldID;

setChatMessageAutoDeleteTime::setChatMessageAutoDeleteTime()
  : chat_id_()
  , message_auto_delete_time_()
{}

setChatMessageAutoDeleteTime::setChatMessageAutoDeleteTime(int53 chat_id_, int32 message_auto_delete_time_)
  : chat_id_(chat_id_)
  , message_auto_delete_time_(message_auto_delete_time_)
{}

const std::int32_t setChatMessageAutoDeleteTime::ID;

object_ptr<setChatMessageAutoDeleteTime> setChatMessageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatMessageAutoDeleteTime> res = make_object<setChatMessageAutoDeleteTime>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_auto_delete_time_ = env->GetIntField(p, res->message_auto_delete_time_fieldID);
  return res;
}

void setChatMessageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatMessageAutoDeleteTime");
    s.store_field("chatId", chat_id_);
    s.store_field("messageAutoDeleteTime", message_auto_delete_time_);
    s.store_class_end();
  }
}

setChatMessageAutoDeleteTime::ReturnType setChatMessageAutoDeleteTime::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatMessageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatMessageAutoDeleteTime").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setChatMessageSender::Class;
jfieldID setChatMessageSender::chat_id_fieldID;
jfieldID setChatMessageSender::message_sender_id_fieldID;

setChatMessageSender::setChatMessageSender()
  : chat_id_()
  , message_sender_id_()
{}

setChatMessageSender::setChatMessageSender(int53 chat_id_, object_ptr<MessageSender> &&message_sender_id_)
  : chat_id_(chat_id_)
  , message_sender_id_(std::move(message_sender_id_))
{}

const std::int32_t setChatMessageSender::ID;

object_ptr<setChatMessageSender> setChatMessageSender::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatMessageSender> res = make_object<setChatMessageSender>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->message_sender_id_fieldID));
  return res;
}

void setChatMessageSender::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatMessageSender");
    s.store_field("chatId", chat_id_);
    s.store_object_field("messageSenderId", static_cast<const BaseObject *>(message_sender_id_.get()));
    s.store_class_end();
  }
}

setChatMessageSender::ReturnType setChatMessageSender::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatMessageSender::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatMessageSender").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_sender_id_fieldID = jni::get_field_id(env, Class, "messageSenderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatNotificationSettings::Class;
jfieldID setChatNotificationSettings::chat_id_fieldID;
jfieldID setChatNotificationSettings::notification_settings_fieldID;

setChatNotificationSettings::setChatNotificationSettings()
  : chat_id_()
  , notification_settings_()
{}

setChatNotificationSettings::setChatNotificationSettings(int53 chat_id_, object_ptr<chatNotificationSettings> &&notification_settings_)
  : chat_id_(chat_id_)
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t setChatNotificationSettings::ID;

object_ptr<setChatNotificationSettings> setChatNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatNotificationSettings> res = make_object<setChatNotificationSettings>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return res;
}

void setChatNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatNotificationSettings");
    s.store_field("chatId", chat_id_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

setChatNotificationSettings::ReturnType setChatNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatNotificationSettings").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatPaidMessageStarCount::Class;
jfieldID setChatPaidMessageStarCount::chat_id_fieldID;
jfieldID setChatPaidMessageStarCount::paid_message_star_count_fieldID;

setChatPaidMessageStarCount::setChatPaidMessageStarCount()
  : chat_id_()
  , paid_message_star_count_()
{}

setChatPaidMessageStarCount::setChatPaidMessageStarCount(int53 chat_id_, int53 paid_message_star_count_)
  : chat_id_(chat_id_)
  , paid_message_star_count_(paid_message_star_count_)
{}

const std::int32_t setChatPaidMessageStarCount::ID;

object_ptr<setChatPaidMessageStarCount> setChatPaidMessageStarCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatPaidMessageStarCount> res = make_object<setChatPaidMessageStarCount>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->paid_message_star_count_ = env->GetLongField(p, res->paid_message_star_count_fieldID);
  return res;
}

void setChatPaidMessageStarCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatPaidMessageStarCount");
    s.store_field("chatId", chat_id_);
    s.store_field("paidMessageStarCount", paid_message_star_count_);
    s.store_class_end();
  }
}

setChatPaidMessageStarCount::ReturnType setChatPaidMessageStarCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatPaidMessageStarCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatPaidMessageStarCount").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    paid_message_star_count_fieldID = jni::get_field_id(env, Class, "paidMessageStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setChatPermissions::Class;
jfieldID setChatPermissions::chat_id_fieldID;
jfieldID setChatPermissions::permissions_fieldID;

setChatPermissions::setChatPermissions()
  : chat_id_()
  , permissions_()
{}

setChatPermissions::setChatPermissions(int53 chat_id_, object_ptr<chatPermissions> &&permissions_)
  : chat_id_(chat_id_)
  , permissions_(std::move(permissions_))
{}

const std::int32_t setChatPermissions::ID;

object_ptr<setChatPermissions> setChatPermissions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatPermissions> res = make_object<setChatPermissions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->permissions_ = jni::fetch_tl_object<chatPermissions>(env, jni::fetch_object(env, p, res->permissions_fieldID));
  return res;
}

void setChatPermissions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatPermissions");
    s.store_field("chatId", chat_id_);
    s.store_object_field("permissions", static_cast<const BaseObject *>(permissions_.get()));
    s.store_class_end();
  }
}

setChatPermissions::ReturnType setChatPermissions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatPermissions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatPermissions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    permissions_fieldID = jni::get_field_id(env, Class, "permissions", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatPermissions;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatPhoto::Class;
jfieldID setChatPhoto::chat_id_fieldID;
jfieldID setChatPhoto::photo_fieldID;

setChatPhoto::setChatPhoto()
  : chat_id_()
  , photo_()
{}

setChatPhoto::setChatPhoto(int53 chat_id_, object_ptr<InputChatPhoto> &&photo_)
  : chat_id_(chat_id_)
  , photo_(std::move(photo_))
{}

const std::int32_t setChatPhoto::ID;

object_ptr<setChatPhoto> setChatPhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatPhoto> res = make_object<setChatPhoto>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void setChatPhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatPhoto");
    s.store_field("chatId", chat_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

setChatPhoto::ReturnType setChatPhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatPhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatPhoto").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setChatPinnedStories::Class;
jfieldID setChatPinnedStories::chat_id_fieldID;
jfieldID setChatPinnedStories::story_ids_fieldID;

setChatPinnedStories::setChatPinnedStories()
  : chat_id_()
  , story_ids_()
{}

setChatPinnedStories::setChatPinnedStories(int53 chat_id_, array<int32> &&story_ids_)
  : chat_id_(chat_id_)
  , story_ids_(std::move(story_ids_))
{}

const std::int32_t setChatPinnedStories::ID;

object_ptr<setChatPinnedStories> setChatPinnedStories::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatPinnedStories> res = make_object<setChatPinnedStories>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->story_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->story_ids_fieldID));
  return res;
}

void setChatPinnedStories::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatPinnedStories");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("storyIds", story_ids_.size()); for (const auto &_value : story_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setChatPinnedStories::ReturnType setChatPinnedStories::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatPinnedStories::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatPinnedStories").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    story_ids_fieldID = jni::get_field_id(env, Class, "storyIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass setChatProfileAccentColor::Class;
jfieldID setChatProfileAccentColor::chat_id_fieldID;
jfieldID setChatProfileAccentColor::profile_accent_color_id_fieldID;
jfieldID setChatProfileAccentColor::profile_background_custom_emoji_id_fieldID;

setChatProfileAccentColor::setChatProfileAccentColor()
  : chat_id_()
  , profile_accent_color_id_()
  , profile_background_custom_emoji_id_()
{}

setChatProfileAccentColor::setChatProfileAccentColor(int53 chat_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_)
  : chat_id_(chat_id_)
  , profile_accent_color_id_(profile_accent_color_id_)
  , profile_background_custom_emoji_id_(profile_background_custom_emoji_id_)
{}

const std::int32_t setChatProfileAccentColor::ID;

object_ptr<setChatProfileAccentColor> setChatProfileAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatProfileAccentColor> res = make_object<setChatProfileAccentColor>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->profile_accent_color_id_ = env->GetIntField(p, res->profile_accent_color_id_fieldID);
  res->profile_background_custom_emoji_id_ = env->GetLongField(p, res->profile_background_custom_emoji_id_fieldID);
  return res;
}

void setChatProfileAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatProfileAccentColor");
    s.store_field("chatId", chat_id_);
    s.store_field("profileAccentColorId", profile_accent_color_id_);
    s.store_field("profileBackgroundCustomEmojiId", profile_background_custom_emoji_id_);
    s.store_class_end();
  }
}

setChatProfileAccentColor::ReturnType setChatProfileAccentColor::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatProfileAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatProfileAccentColor").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "profileAccentColorId", "I");
    profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "profileBackgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setChatSlowModeDelay::Class;
jfieldID setChatSlowModeDelay::chat_id_fieldID;
jfieldID setChatSlowModeDelay::slow_mode_delay_fieldID;

setChatSlowModeDelay::setChatSlowModeDelay()
  : chat_id_()
  , slow_mode_delay_()
{}

setChatSlowModeDelay::setChatSlowModeDelay(int53 chat_id_, int32 slow_mode_delay_)
  : chat_id_(chat_id_)
  , slow_mode_delay_(slow_mode_delay_)
{}

const std::int32_t setChatSlowModeDelay::ID;

object_ptr<setChatSlowModeDelay> setChatSlowModeDelay::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatSlowModeDelay> res = make_object<setChatSlowModeDelay>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->slow_mode_delay_ = env->GetIntField(p, res->slow_mode_delay_fieldID);
  return res;
}

void setChatSlowModeDelay::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatSlowModeDelay");
    s.store_field("chatId", chat_id_);
    s.store_field("slowModeDelay", slow_mode_delay_);
    s.store_class_end();
  }
}

setChatSlowModeDelay::ReturnType setChatSlowModeDelay::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatSlowModeDelay::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatSlowModeDelay").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    slow_mode_delay_fieldID = jni::get_field_id(env, Class, "slowModeDelay", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setChatTheme::Class;
jfieldID setChatTheme::chat_id_fieldID;
jfieldID setChatTheme::theme_name_fieldID;

setChatTheme::setChatTheme()
  : chat_id_()
  , theme_name_()
{}

setChatTheme::setChatTheme(int53 chat_id_, string const &theme_name_)
  : chat_id_(chat_id_)
  , theme_name_(theme_name_)
{}

const std::int32_t setChatTheme::ID;

object_ptr<setChatTheme> setChatTheme::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatTheme> res = make_object<setChatTheme>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->theme_name_ = jni::fetch_string(env, p, res->theme_name_fieldID);
  return res;
}

void setChatTheme::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatTheme");
    s.store_field("chatId", chat_id_);
    s.store_field("themeName", theme_name_);
    s.store_class_end();
  }
}

setChatTheme::ReturnType setChatTheme::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatTheme::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatTheme").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    theme_name_fieldID = jni::get_field_id(env, Class, "themeName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setChatTitle::Class;
jfieldID setChatTitle::chat_id_fieldID;
jfieldID setChatTitle::title_fieldID;

setChatTitle::setChatTitle()
  : chat_id_()
  , title_()
{}

setChatTitle::setChatTitle(int53 chat_id_, string const &title_)
  : chat_id_(chat_id_)
  , title_(title_)
{}

const std::int32_t setChatTitle::ID;

object_ptr<setChatTitle> setChatTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setChatTitle> res = make_object<setChatTitle>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void setChatTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetChatTitle");
    s.store_field("chatId", chat_id_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

setChatTitle::ReturnType setChatTitle::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setChatTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetChatTitle").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setCloseFriends::Class;
jfieldID setCloseFriends::user_ids_fieldID;

setCloseFriends::setCloseFriends()
  : user_ids_()
{}

setCloseFriends::setCloseFriends(array<int53> &&user_ids_)
  : user_ids_(std::move(user_ids_))
{}

const std::int32_t setCloseFriends::ID;

object_ptr<setCloseFriends> setCloseFriends::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setCloseFriends> res = make_object<setCloseFriends>();
  res->user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->user_ids_fieldID));
  return res;
}

void setCloseFriends::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetCloseFriends");
    { s.store_vector_begin("userIds", user_ids_.size()); for (const auto &_value : user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setCloseFriends::ReturnType setCloseFriends::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setCloseFriends::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetCloseFriends").c_str());
    user_ids_fieldID = jni::get_field_id(env, Class, "userIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass setCommands::Class;
jfieldID setCommands::scope_fieldID;
jfieldID setCommands::language_code_fieldID;
jfieldID setCommands::commands_fieldID;

setCommands::setCommands()
  : scope_()
  , language_code_()
  , commands_()
{}

setCommands::setCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_, array<object_ptr<botCommand>> &&commands_)
  : scope_(std::move(scope_))
  , language_code_(language_code_)
  , commands_(std::move(commands_))
{}

const std::int32_t setCommands::ID;

object_ptr<setCommands> setCommands::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setCommands> res = make_object<setCommands>();
  res->scope_ = jni::fetch_tl_object<BotCommandScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->language_code_ = jni::fetch_string(env, p, res->language_code_fieldID);
  res->commands_ = jni::FetchVector<botCommand>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->commands_fieldID));
  return res;
}

void setCommands::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetCommands");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_field("languageCode", language_code_);
    { s.store_vector_begin("commands", commands_.size()); for (const auto &_value : commands_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

setCommands::ReturnType setCommands::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setCommands::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetCommands").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotCommandScope;").c_str());
    language_code_fieldID = jni::get_field_id(env, Class, "languageCode", "Ljava/lang/String;");
    commands_fieldID = jni::get_field_id(env, Class, "commands", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$BotCommand;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setCustomEmojiStickerSetThumbnail::Class;
jfieldID setCustomEmojiStickerSetThumbnail::name_fieldID;
jfieldID setCustomEmojiStickerSetThumbnail::custom_emoji_id_fieldID;

setCustomEmojiStickerSetThumbnail::setCustomEmojiStickerSetThumbnail()
  : name_()
  , custom_emoji_id_()
{}

setCustomEmojiStickerSetThumbnail::setCustomEmojiStickerSetThumbnail(string const &name_, int64 custom_emoji_id_)
  : name_(name_)
  , custom_emoji_id_(custom_emoji_id_)
{}

const std::int32_t setCustomEmojiStickerSetThumbnail::ID;

object_ptr<setCustomEmojiStickerSetThumbnail> setCustomEmojiStickerSetThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setCustomEmojiStickerSetThumbnail> res = make_object<setCustomEmojiStickerSetThumbnail>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->custom_emoji_id_ = env->GetLongField(p, res->custom_emoji_id_fieldID);
  return res;
}

void setCustomEmojiStickerSetThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetCustomEmojiStickerSetThumbnail");
    s.store_field("name", name_);
    s.store_field("customEmojiId", custom_emoji_id_);
    s.store_class_end();
  }
}

setCustomEmojiStickerSetThumbnail::ReturnType setCustomEmojiStickerSetThumbnail::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setCustomEmojiStickerSetThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetCustomEmojiStickerSetThumbnail").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    custom_emoji_id_fieldID = jni::get_field_id(env, Class, "customEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setCustomLanguagePack::Class;
jfieldID setCustomLanguagePack::info_fieldID;
jfieldID setCustomLanguagePack::strings_fieldID;

setCustomLanguagePack::setCustomLanguagePack()
  : info_()
  , strings_()
{}

setCustomLanguagePack::setCustomLanguagePack(object_ptr<languagePackInfo> &&info_, array<object_ptr<languagePackString>> &&strings_)
  : info_(std::move(info_))
  , strings_(std::move(strings_))
{}

const std::int32_t setCustomLanguagePack::ID;

object_ptr<setCustomLanguagePack> setCustomLanguagePack::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setCustomLanguagePack> res = make_object<setCustomLanguagePack>();
  res->info_ = jni::fetch_tl_object<languagePackInfo>(env, jni::fetch_object(env, p, res->info_fieldID));
  res->strings_ = jni::FetchVector<languagePackString>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->strings_fieldID));
  return res;
}

void setCustomLanguagePack::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetCustomLanguagePack");
    s.store_object_field("info", static_cast<const BaseObject *>(info_.get()));
    { s.store_vector_begin("strings", strings_.size()); for (const auto &_value : strings_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

setCustomLanguagePack::ReturnType setCustomLanguagePack::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setCustomLanguagePack::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetCustomLanguagePack").c_str());
    info_fieldID = jni::get_field_id(env, Class, "info", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LanguagePackInfo;").c_str());
    strings_fieldID = jni::get_field_id(env, Class, "strings", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$LanguagePackString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setCustomLanguagePackString::Class;
jfieldID setCustomLanguagePackString::language_pack_id_fieldID;
jfieldID setCustomLanguagePackString::new_string_fieldID;

setCustomLanguagePackString::setCustomLanguagePackString()
  : language_pack_id_()
  , new_string_()
{}

setCustomLanguagePackString::setCustomLanguagePackString(string const &language_pack_id_, object_ptr<languagePackString> &&new_string_)
  : language_pack_id_(language_pack_id_)
  , new_string_(std::move(new_string_))
{}

const std::int32_t setCustomLanguagePackString::ID;

object_ptr<setCustomLanguagePackString> setCustomLanguagePackString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setCustomLanguagePackString> res = make_object<setCustomLanguagePackString>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  res->new_string_ = jni::fetch_tl_object<languagePackString>(env, jni::fetch_object(env, p, res->new_string_fieldID));
  return res;
}

void setCustomLanguagePackString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetCustomLanguagePackString");
    s.store_field("languagePackId", language_pack_id_);
    s.store_object_field("newString", static_cast<const BaseObject *>(new_string_.get()));
    s.store_class_end();
  }
}

setCustomLanguagePackString::ReturnType setCustomLanguagePackString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setCustomLanguagePackString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetCustomLanguagePackString").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    new_string_fieldID = jni::get_field_id(env, Class, "newString", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LanguagePackString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setDatabaseEncryptionKey::Class;
jfieldID setDatabaseEncryptionKey::new_encryption_key_fieldID;

setDatabaseEncryptionKey::setDatabaseEncryptionKey()
  : new_encryption_key_()
{}

setDatabaseEncryptionKey::setDatabaseEncryptionKey(bytes const &new_encryption_key_)
  : new_encryption_key_(std::move(new_encryption_key_))
{}

const std::int32_t setDatabaseEncryptionKey::ID;

object_ptr<setDatabaseEncryptionKey> setDatabaseEncryptionKey::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDatabaseEncryptionKey> res = make_object<setDatabaseEncryptionKey>();
  res->new_encryption_key_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->new_encryption_key_fieldID));
  return res;
}

void setDatabaseEncryptionKey::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDatabaseEncryptionKey");
    s.store_bytes_field("newEncryptionKey", new_encryption_key_);
    s.store_class_end();
  }
}

setDatabaseEncryptionKey::ReturnType setDatabaseEncryptionKey::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setDatabaseEncryptionKey::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDatabaseEncryptionKey").c_str());
    new_encryption_key_fieldID = jni::get_field_id(env, Class, "newEncryptionKey", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass setDefaultBackground::Class;
jfieldID setDefaultBackground::background_fieldID;
jfieldID setDefaultBackground::type_fieldID;
jfieldID setDefaultBackground::for_dark_theme_fieldID;

setDefaultBackground::setDefaultBackground()
  : background_()
  , type_()
  , for_dark_theme_()
{}

setDefaultBackground::setDefaultBackground(object_ptr<InputBackground> &&background_, object_ptr<BackgroundType> &&type_, bool for_dark_theme_)
  : background_(std::move(background_))
  , type_(std::move(type_))
  , for_dark_theme_(for_dark_theme_)
{}

const std::int32_t setDefaultBackground::ID;

object_ptr<setDefaultBackground> setDefaultBackground::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDefaultBackground> res = make_object<setDefaultBackground>();
  res->background_ = jni::fetch_tl_object<InputBackground>(env, jni::fetch_object(env, p, res->background_fieldID));
  res->type_ = jni::fetch_tl_object<BackgroundType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->for_dark_theme_ = (env->GetBooleanField(p, res->for_dark_theme_fieldID) != 0);
  return res;
}

void setDefaultBackground::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDefaultBackground");
    s.store_object_field("background", static_cast<const BaseObject *>(background_.get()));
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("forDarkTheme", for_dark_theme_);
    s.store_class_end();
  }
}

setDefaultBackground::ReturnType setDefaultBackground::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return background::fetch(env, p);
}

void setDefaultBackground::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDefaultBackground").c_str());
    background_fieldID = jni::get_field_id(env, Class, "background", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputBackground;").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BackgroundType;").c_str());
    for_dark_theme_fieldID = jni::get_field_id(env, Class, "forDarkTheme", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setDefaultChannelAdministratorRights::Class;
jfieldID setDefaultChannelAdministratorRights::default_channel_administrator_rights_fieldID;

setDefaultChannelAdministratorRights::setDefaultChannelAdministratorRights()
  : default_channel_administrator_rights_()
{}

setDefaultChannelAdministratorRights::setDefaultChannelAdministratorRights(object_ptr<chatAdministratorRights> &&default_channel_administrator_rights_)
  : default_channel_administrator_rights_(std::move(default_channel_administrator_rights_))
{}

const std::int32_t setDefaultChannelAdministratorRights::ID;

object_ptr<setDefaultChannelAdministratorRights> setDefaultChannelAdministratorRights::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDefaultChannelAdministratorRights> res = make_object<setDefaultChannelAdministratorRights>();
  res->default_channel_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->default_channel_administrator_rights_fieldID));
  return res;
}

void setDefaultChannelAdministratorRights::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDefaultChannelAdministratorRights");
    s.store_object_field("defaultChannelAdministratorRights", static_cast<const BaseObject *>(default_channel_administrator_rights_.get()));
    s.store_class_end();
  }
}

setDefaultChannelAdministratorRights::ReturnType setDefaultChannelAdministratorRights::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setDefaultChannelAdministratorRights::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDefaultChannelAdministratorRights").c_str());
    default_channel_administrator_rights_fieldID = jni::get_field_id(env, Class, "defaultChannelAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setDefaultGroupAdministratorRights::Class;
jfieldID setDefaultGroupAdministratorRights::default_group_administrator_rights_fieldID;

setDefaultGroupAdministratorRights::setDefaultGroupAdministratorRights()
  : default_group_administrator_rights_()
{}

setDefaultGroupAdministratorRights::setDefaultGroupAdministratorRights(object_ptr<chatAdministratorRights> &&default_group_administrator_rights_)
  : default_group_administrator_rights_(std::move(default_group_administrator_rights_))
{}

const std::int32_t setDefaultGroupAdministratorRights::ID;

object_ptr<setDefaultGroupAdministratorRights> setDefaultGroupAdministratorRights::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDefaultGroupAdministratorRights> res = make_object<setDefaultGroupAdministratorRights>();
  res->default_group_administrator_rights_ = jni::fetch_tl_object<chatAdministratorRights>(env, jni::fetch_object(env, p, res->default_group_administrator_rights_fieldID));
  return res;
}

void setDefaultGroupAdministratorRights::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDefaultGroupAdministratorRights");
    s.store_object_field("defaultGroupAdministratorRights", static_cast<const BaseObject *>(default_group_administrator_rights_.get()));
    s.store_class_end();
  }
}

setDefaultGroupAdministratorRights::ReturnType setDefaultGroupAdministratorRights::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setDefaultGroupAdministratorRights::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDefaultGroupAdministratorRights").c_str());
    default_group_administrator_rights_fieldID = jni::get_field_id(env, Class, "defaultGroupAdministratorRights", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatAdministratorRights;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setDefaultMessageAutoDeleteTime::Class;
jfieldID setDefaultMessageAutoDeleteTime::message_auto_delete_time_fieldID;

setDefaultMessageAutoDeleteTime::setDefaultMessageAutoDeleteTime()
  : message_auto_delete_time_()
{}

setDefaultMessageAutoDeleteTime::setDefaultMessageAutoDeleteTime(object_ptr<messageAutoDeleteTime> &&message_auto_delete_time_)
  : message_auto_delete_time_(std::move(message_auto_delete_time_))
{}

const std::int32_t setDefaultMessageAutoDeleteTime::ID;

object_ptr<setDefaultMessageAutoDeleteTime> setDefaultMessageAutoDeleteTime::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDefaultMessageAutoDeleteTime> res = make_object<setDefaultMessageAutoDeleteTime>();
  res->message_auto_delete_time_ = jni::fetch_tl_object<messageAutoDeleteTime>(env, jni::fetch_object(env, p, res->message_auto_delete_time_fieldID));
  return res;
}

void setDefaultMessageAutoDeleteTime::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDefaultMessageAutoDeleteTime");
    s.store_object_field("messageAutoDeleteTime", static_cast<const BaseObject *>(message_auto_delete_time_.get()));
    s.store_class_end();
  }
}

setDefaultMessageAutoDeleteTime::ReturnType setDefaultMessageAutoDeleteTime::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setDefaultMessageAutoDeleteTime::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDefaultMessageAutoDeleteTime").c_str());
    message_auto_delete_time_fieldID = jni::get_field_id(env, Class, "messageAutoDeleteTime", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageAutoDeleteTime;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setDefaultReactionType::Class;
jfieldID setDefaultReactionType::reaction_type_fieldID;

setDefaultReactionType::setDefaultReactionType()
  : reaction_type_()
{}

setDefaultReactionType::setDefaultReactionType(object_ptr<ReactionType> &&reaction_type_)
  : reaction_type_(std::move(reaction_type_))
{}

const std::int32_t setDefaultReactionType::ID;

object_ptr<setDefaultReactionType> setDefaultReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setDefaultReactionType> res = make_object<setDefaultReactionType>();
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  return res;
}

void setDefaultReactionType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetDefaultReactionType");
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_class_end();
  }
}

setDefaultReactionType::ReturnType setDefaultReactionType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setDefaultReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetDefaultReactionType").c_str());
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setEmojiStatus::Class;
jfieldID setEmojiStatus::emoji_status_fieldID;

setEmojiStatus::setEmojiStatus()
  : emoji_status_()
{}

setEmojiStatus::setEmojiStatus(object_ptr<emojiStatus> &&emoji_status_)
  : emoji_status_(std::move(emoji_status_))
{}

const std::int32_t setEmojiStatus::ID;

object_ptr<setEmojiStatus> setEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setEmojiStatus> res = make_object<setEmojiStatus>();
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  return res;
}

void setEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetEmojiStatus");
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_class_end();
  }
}

setEmojiStatus::ReturnType setEmojiStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetEmojiStatus").c_str());
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setFileGenerationProgress::Class;
jfieldID setFileGenerationProgress::generation_id_fieldID;
jfieldID setFileGenerationProgress::expected_size_fieldID;
jfieldID setFileGenerationProgress::local_prefix_size_fieldID;

setFileGenerationProgress::setFileGenerationProgress()
  : generation_id_()
  , expected_size_()
  , local_prefix_size_()
{}

setFileGenerationProgress::setFileGenerationProgress(int64 generation_id_, int53 expected_size_, int53 local_prefix_size_)
  : generation_id_(generation_id_)
  , expected_size_(expected_size_)
  , local_prefix_size_(local_prefix_size_)
{}

const std::int32_t setFileGenerationProgress::ID;

object_ptr<setFileGenerationProgress> setFileGenerationProgress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setFileGenerationProgress> res = make_object<setFileGenerationProgress>();
  res->generation_id_ = env->GetLongField(p, res->generation_id_fieldID);
  res->expected_size_ = env->GetLongField(p, res->expected_size_fieldID);
  res->local_prefix_size_ = env->GetLongField(p, res->local_prefix_size_fieldID);
  return res;
}

void setFileGenerationProgress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetFileGenerationProgress");
    s.store_field("generationId", generation_id_);
    s.store_field("expectedSize", expected_size_);
    s.store_field("localPrefixSize", local_prefix_size_);
    s.store_class_end();
  }
}

setFileGenerationProgress::ReturnType setFileGenerationProgress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setFileGenerationProgress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetFileGenerationProgress").c_str());
    generation_id_fieldID = jni::get_field_id(env, Class, "generationId", "J");
    expected_size_fieldID = jni::get_field_id(env, Class, "expectedSize", "J");
    local_prefix_size_fieldID = jni::get_field_id(env, Class, "localPrefixSize", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setForumTopicNotificationSettings::Class;
jfieldID setForumTopicNotificationSettings::chat_id_fieldID;
jfieldID setForumTopicNotificationSettings::message_thread_id_fieldID;
jfieldID setForumTopicNotificationSettings::notification_settings_fieldID;

setForumTopicNotificationSettings::setForumTopicNotificationSettings()
  : chat_id_()
  , message_thread_id_()
  , notification_settings_()
{}

setForumTopicNotificationSettings::setForumTopicNotificationSettings(int53 chat_id_, int53 message_thread_id_, object_ptr<chatNotificationSettings> &&notification_settings_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t setForumTopicNotificationSettings::ID;

object_ptr<setForumTopicNotificationSettings> setForumTopicNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setForumTopicNotificationSettings> res = make_object<setForumTopicNotificationSettings>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->notification_settings_ = jni::fetch_tl_object<chatNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return res;
}

void setForumTopicNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetForumTopicNotificationSettings");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

setForumTopicNotificationSettings::ReturnType setForumTopicNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setForumTopicNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetForumTopicNotificationSettings").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setGameScore::Class;
jfieldID setGameScore::chat_id_fieldID;
jfieldID setGameScore::message_id_fieldID;
jfieldID setGameScore::edit_message_fieldID;
jfieldID setGameScore::user_id_fieldID;
jfieldID setGameScore::score_fieldID;
jfieldID setGameScore::force_fieldID;

setGameScore::setGameScore()
  : chat_id_()
  , message_id_()
  , edit_message_()
  , user_id_()
  , score_()
  , force_()
{}

setGameScore::setGameScore(int53 chat_id_, int53 message_id_, bool edit_message_, int53 user_id_, int32 score_, bool force_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , edit_message_(edit_message_)
  , user_id_(user_id_)
  , score_(score_)
  , force_(force_)
{}

const std::int32_t setGameScore::ID;

object_ptr<setGameScore> setGameScore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setGameScore> res = make_object<setGameScore>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->edit_message_ = (env->GetBooleanField(p, res->edit_message_fieldID) != 0);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->score_ = env->GetIntField(p, res->score_fieldID);
  res->force_ = (env->GetBooleanField(p, res->force_fieldID) != 0);
  return res;
}

void setGameScore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetGameScore");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("editMessage", edit_message_);
    s.store_field("userId", user_id_);
    s.store_field("score", score_);
    s.store_field("force", force_);
    s.store_class_end();
  }
}

setGameScore::ReturnType setGameScore::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return message::fetch(env, p);
}

void setGameScore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetGameScore").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    edit_message_fieldID = jni::get_field_id(env, Class, "editMessage", "Z");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    score_fieldID = jni::get_field_id(env, Class, "score", "I");
    force_fieldID = jni::get_field_id(env, Class, "force", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setGiftResalePrice::Class;
jfieldID setGiftResalePrice::received_gift_id_fieldID;
jfieldID setGiftResalePrice::resale_star_count_fieldID;

setGiftResalePrice::setGiftResalePrice()
  : received_gift_id_()
  , resale_star_count_()
{}

setGiftResalePrice::setGiftResalePrice(string const &received_gift_id_, int53 resale_star_count_)
  : received_gift_id_(received_gift_id_)
  , resale_star_count_(resale_star_count_)
{}

const std::int32_t setGiftResalePrice::ID;

object_ptr<setGiftResalePrice> setGiftResalePrice::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setGiftResalePrice> res = make_object<setGiftResalePrice>();
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->resale_star_count_ = env->GetLongField(p, res->resale_star_count_fieldID);
  return res;
}

void setGiftResalePrice::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetGiftResalePrice");
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("resaleStarCount", resale_star_count_);
    s.store_class_end();
  }
}

setGiftResalePrice::ReturnType setGiftResalePrice::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setGiftResalePrice::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetGiftResalePrice").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    resale_star_count_fieldID = jni::get_field_id(env, Class, "resaleStarCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setGiftSettings::Class;
jfieldID setGiftSettings::settings_fieldID;

setGiftSettings::setGiftSettings()
  : settings_()
{}

setGiftSettings::setGiftSettings(object_ptr<giftSettings> &&settings_)
  : settings_(std::move(settings_))
{}

const std::int32_t setGiftSettings::ID;

object_ptr<setGiftSettings> setGiftSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setGiftSettings> res = make_object<setGiftSettings>();
  res->settings_ = jni::fetch_tl_object<giftSettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setGiftSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetGiftSettings");
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setGiftSettings::ReturnType setGiftSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setGiftSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetGiftSettings").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$GiftSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setGroupCallParticipantIsSpeaking::Class;
jfieldID setGroupCallParticipantIsSpeaking::group_call_id_fieldID;
jfieldID setGroupCallParticipantIsSpeaking::audio_source_fieldID;
jfieldID setGroupCallParticipantIsSpeaking::is_speaking_fieldID;

setGroupCallParticipantIsSpeaking::setGroupCallParticipantIsSpeaking()
  : group_call_id_()
  , audio_source_()
  , is_speaking_()
{}

setGroupCallParticipantIsSpeaking::setGroupCallParticipantIsSpeaking(int32 group_call_id_, int32 audio_source_, bool is_speaking_)
  : group_call_id_(group_call_id_)
  , audio_source_(audio_source_)
  , is_speaking_(is_speaking_)
{}

const std::int32_t setGroupCallParticipantIsSpeaking::ID;

object_ptr<setGroupCallParticipantIsSpeaking> setGroupCallParticipantIsSpeaking::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setGroupCallParticipantIsSpeaking> res = make_object<setGroupCallParticipantIsSpeaking>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->audio_source_ = env->GetIntField(p, res->audio_source_fieldID);
  res->is_speaking_ = (env->GetBooleanField(p, res->is_speaking_fieldID) != 0);
  return res;
}

void setGroupCallParticipantIsSpeaking::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetGroupCallParticipantIsSpeaking");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("audioSource", audio_source_);
    s.store_field("isSpeaking", is_speaking_);
    s.store_class_end();
  }
}

setGroupCallParticipantIsSpeaking::ReturnType setGroupCallParticipantIsSpeaking::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setGroupCallParticipantIsSpeaking::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetGroupCallParticipantIsSpeaking").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    audio_source_fieldID = jni::get_field_id(env, Class, "audioSource", "I");
    is_speaking_fieldID = jni::get_field_id(env, Class, "isSpeaking", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setGroupCallParticipantVolumeLevel::Class;
jfieldID setGroupCallParticipantVolumeLevel::group_call_id_fieldID;
jfieldID setGroupCallParticipantVolumeLevel::participant_id_fieldID;
jfieldID setGroupCallParticipantVolumeLevel::volume_level_fieldID;

setGroupCallParticipantVolumeLevel::setGroupCallParticipantVolumeLevel()
  : group_call_id_()
  , participant_id_()
  , volume_level_()
{}

setGroupCallParticipantVolumeLevel::setGroupCallParticipantVolumeLevel(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, int32 volume_level_)
  : group_call_id_(group_call_id_)
  , participant_id_(std::move(participant_id_))
  , volume_level_(volume_level_)
{}

const std::int32_t setGroupCallParticipantVolumeLevel::ID;

object_ptr<setGroupCallParticipantVolumeLevel> setGroupCallParticipantVolumeLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setGroupCallParticipantVolumeLevel> res = make_object<setGroupCallParticipantVolumeLevel>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->volume_level_ = env->GetIntField(p, res->volume_level_fieldID);
  return res;
}

void setGroupCallParticipantVolumeLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetGroupCallParticipantVolumeLevel");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("volumeLevel", volume_level_);
    s.store_class_end();
  }
}

setGroupCallParticipantVolumeLevel::ReturnType setGroupCallParticipantVolumeLevel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setGroupCallParticipantVolumeLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetGroupCallParticipantVolumeLevel").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    volume_level_fieldID = jni::get_field_id(env, Class, "volumeLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setInactiveSessionTtl::Class;
jfieldID setInactiveSessionTtl::inactive_session_ttl_days_fieldID;

setInactiveSessionTtl::setInactiveSessionTtl()
  : inactive_session_ttl_days_()
{}

setInactiveSessionTtl::setInactiveSessionTtl(int32 inactive_session_ttl_days_)
  : inactive_session_ttl_days_(inactive_session_ttl_days_)
{}

const std::int32_t setInactiveSessionTtl::ID;

object_ptr<setInactiveSessionTtl> setInactiveSessionTtl::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setInactiveSessionTtl> res = make_object<setInactiveSessionTtl>();
  res->inactive_session_ttl_days_ = env->GetIntField(p, res->inactive_session_ttl_days_fieldID);
  return res;
}

void setInactiveSessionTtl::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetInactiveSessionTtl");
    s.store_field("inactiveSessionTtlDays", inactive_session_ttl_days_);
    s.store_class_end();
  }
}

setInactiveSessionTtl::ReturnType setInactiveSessionTtl::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setInactiveSessionTtl::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetInactiveSessionTtl").c_str());
    inactive_session_ttl_days_fieldID = jni::get_field_id(env, Class, "inactiveSessionTtlDays", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setInlineGameScore::Class;
jfieldID setInlineGameScore::inline_message_id_fieldID;
jfieldID setInlineGameScore::edit_message_fieldID;
jfieldID setInlineGameScore::user_id_fieldID;
jfieldID setInlineGameScore::score_fieldID;
jfieldID setInlineGameScore::force_fieldID;

setInlineGameScore::setInlineGameScore()
  : inline_message_id_()
  , edit_message_()
  , user_id_()
  , score_()
  , force_()
{}

setInlineGameScore::setInlineGameScore(string const &inline_message_id_, bool edit_message_, int53 user_id_, int32 score_, bool force_)
  : inline_message_id_(inline_message_id_)
  , edit_message_(edit_message_)
  , user_id_(user_id_)
  , score_(score_)
  , force_(force_)
{}

const std::int32_t setInlineGameScore::ID;

object_ptr<setInlineGameScore> setInlineGameScore::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setInlineGameScore> res = make_object<setInlineGameScore>();
  res->inline_message_id_ = jni::fetch_string(env, p, res->inline_message_id_fieldID);
  res->edit_message_ = (env->GetBooleanField(p, res->edit_message_fieldID) != 0);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->score_ = env->GetIntField(p, res->score_fieldID);
  res->force_ = (env->GetBooleanField(p, res->force_fieldID) != 0);
  return res;
}

void setInlineGameScore::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetInlineGameScore");
    s.store_field("inlineMessageId", inline_message_id_);
    s.store_field("editMessage", edit_message_);
    s.store_field("userId", user_id_);
    s.store_field("score", score_);
    s.store_field("force", force_);
    s.store_class_end();
  }
}

setInlineGameScore::ReturnType setInlineGameScore::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setInlineGameScore::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetInlineGameScore").c_str());
    inline_message_id_fieldID = jni::get_field_id(env, Class, "inlineMessageId", "Ljava/lang/String;");
    edit_message_fieldID = jni::get_field_id(env, Class, "editMessage", "Z");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    score_fieldID = jni::get_field_id(env, Class, "score", "I");
    force_fieldID = jni::get_field_id(env, Class, "force", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setLogStream::Class;
jfieldID setLogStream::log_stream_fieldID;

setLogStream::setLogStream()
  : log_stream_()
{}

setLogStream::setLogStream(object_ptr<LogStream> &&log_stream_)
  : log_stream_(std::move(log_stream_))
{}

const std::int32_t setLogStream::ID;

object_ptr<setLogStream> setLogStream::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setLogStream> res = make_object<setLogStream>();
  res->log_stream_ = jni::fetch_tl_object<LogStream>(env, jni::fetch_object(env, p, res->log_stream_fieldID));
  return res;
}

void setLogStream::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetLogStream");
    s.store_object_field("logStream", static_cast<const BaseObject *>(log_stream_.get()));
    s.store_class_end();
  }
}

setLogStream::ReturnType setLogStream::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setLogStream::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetLogStream").c_str());
    log_stream_fieldID = jni::get_field_id(env, Class, "logStream", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$LogStream;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setLogTagVerbosityLevel::Class;
jfieldID setLogTagVerbosityLevel::tag_fieldID;
jfieldID setLogTagVerbosityLevel::new_verbosity_level_fieldID;

setLogTagVerbosityLevel::setLogTagVerbosityLevel()
  : tag_()
  , new_verbosity_level_()
{}

setLogTagVerbosityLevel::setLogTagVerbosityLevel(string const &tag_, int32 new_verbosity_level_)
  : tag_(tag_)
  , new_verbosity_level_(new_verbosity_level_)
{}

const std::int32_t setLogTagVerbosityLevel::ID;

object_ptr<setLogTagVerbosityLevel> setLogTagVerbosityLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setLogTagVerbosityLevel> res = make_object<setLogTagVerbosityLevel>();
  res->tag_ = jni::fetch_string(env, p, res->tag_fieldID);
  res->new_verbosity_level_ = env->GetIntField(p, res->new_verbosity_level_fieldID);
  return res;
}

void setLogTagVerbosityLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetLogTagVerbosityLevel");
    s.store_field("tag", tag_);
    s.store_field("newVerbosityLevel", new_verbosity_level_);
    s.store_class_end();
  }
}

setLogTagVerbosityLevel::ReturnType setLogTagVerbosityLevel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setLogTagVerbosityLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetLogTagVerbosityLevel").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", "Ljava/lang/String;");
    new_verbosity_level_fieldID = jni::get_field_id(env, Class, "newVerbosityLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setLogVerbosityLevel::Class;
jfieldID setLogVerbosityLevel::new_verbosity_level_fieldID;

setLogVerbosityLevel::setLogVerbosityLevel()
  : new_verbosity_level_()
{}

setLogVerbosityLevel::setLogVerbosityLevel(int32 new_verbosity_level_)
  : new_verbosity_level_(new_verbosity_level_)
{}

const std::int32_t setLogVerbosityLevel::ID;

object_ptr<setLogVerbosityLevel> setLogVerbosityLevel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setLogVerbosityLevel> res = make_object<setLogVerbosityLevel>();
  res->new_verbosity_level_ = env->GetIntField(p, res->new_verbosity_level_fieldID);
  return res;
}

void setLogVerbosityLevel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetLogVerbosityLevel");
    s.store_field("newVerbosityLevel", new_verbosity_level_);
    s.store_class_end();
  }
}

setLogVerbosityLevel::ReturnType setLogVerbosityLevel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setLogVerbosityLevel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetLogVerbosityLevel").c_str());
    new_verbosity_level_fieldID = jni::get_field_id(env, Class, "newVerbosityLevel", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setLoginEmailAddress::Class;
jfieldID setLoginEmailAddress::new_login_email_address_fieldID;

setLoginEmailAddress::setLoginEmailAddress()
  : new_login_email_address_()
{}

setLoginEmailAddress::setLoginEmailAddress(string const &new_login_email_address_)
  : new_login_email_address_(new_login_email_address_)
{}

const std::int32_t setLoginEmailAddress::ID;

object_ptr<setLoginEmailAddress> setLoginEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setLoginEmailAddress> res = make_object<setLoginEmailAddress>();
  res->new_login_email_address_ = jni::fetch_string(env, p, res->new_login_email_address_fieldID);
  return res;
}

void setLoginEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetLoginEmailAddress");
    s.store_field("newLoginEmailAddress", new_login_email_address_);
    s.store_class_end();
  }
}

setLoginEmailAddress::ReturnType setLoginEmailAddress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return emailAddressAuthenticationCodeInfo::fetch(env, p);
}

void setLoginEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetLoginEmailAddress").c_str());
    new_login_email_address_fieldID = jni::get_field_id(env, Class, "newLoginEmailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setMenuButton::Class;
jfieldID setMenuButton::user_id_fieldID;
jfieldID setMenuButton::menu_button_fieldID;

setMenuButton::setMenuButton()
  : user_id_()
  , menu_button_()
{}

setMenuButton::setMenuButton(int53 user_id_, object_ptr<botMenuButton> &&menu_button_)
  : user_id_(user_id_)
  , menu_button_(std::move(menu_button_))
{}

const std::int32_t setMenuButton::ID;

object_ptr<setMenuButton> setMenuButton::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setMenuButton> res = make_object<setMenuButton>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->menu_button_ = jni::fetch_tl_object<botMenuButton>(env, jni::fetch_object(env, p, res->menu_button_fieldID));
  return res;
}

void setMenuButton::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetMenuButton");
    s.store_field("userId", user_id_);
    s.store_object_field("menuButton", static_cast<const BaseObject *>(menu_button_.get()));
    s.store_class_end();
  }
}

setMenuButton::ReturnType setMenuButton::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setMenuButton::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetMenuButton").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    menu_button_fieldID = jni::get_field_id(env, Class, "menuButton", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BotMenuButton;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setMessageFactCheck::Class;
jfieldID setMessageFactCheck::chat_id_fieldID;
jfieldID setMessageFactCheck::message_id_fieldID;
jfieldID setMessageFactCheck::text_fieldID;

setMessageFactCheck::setMessageFactCheck()
  : chat_id_()
  , message_id_()
  , text_()
{}

setMessageFactCheck::setMessageFactCheck(int53 chat_id_, int53 message_id_, object_ptr<formattedText> &&text_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , text_(std::move(text_))
{}

const std::int32_t setMessageFactCheck::ID;

object_ptr<setMessageFactCheck> setMessageFactCheck::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setMessageFactCheck> res = make_object<setMessageFactCheck>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  return res;
}

void setMessageFactCheck::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetMessageFactCheck");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_class_end();
  }
}

setMessageFactCheck::ReturnType setMessageFactCheck::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setMessageFactCheck::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetMessageFactCheck").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setMessageReactions::Class;
jfieldID setMessageReactions::chat_id_fieldID;
jfieldID setMessageReactions::message_id_fieldID;
jfieldID setMessageReactions::reaction_types_fieldID;
jfieldID setMessageReactions::is_big_fieldID;

setMessageReactions::setMessageReactions()
  : chat_id_()
  , message_id_()
  , reaction_types_()
  , is_big_()
{}

setMessageReactions::setMessageReactions(int53 chat_id_, int53 message_id_, array<object_ptr<ReactionType>> &&reaction_types_, bool is_big_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reaction_types_(std::move(reaction_types_))
  , is_big_(is_big_)
{}

const std::int32_t setMessageReactions::ID;

object_ptr<setMessageReactions> setMessageReactions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setMessageReactions> res = make_object<setMessageReactions>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reaction_types_ = jni::FetchVector<ReactionType>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->reaction_types_fieldID));
  res->is_big_ = (env->GetBooleanField(p, res->is_big_fieldID) != 0);
  return res;
}

void setMessageReactions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetMessageReactions");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    { s.store_vector_begin("reactionTypes", reaction_types_.size()); for (const auto &_value : reaction_types_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_field("isBig", is_big_);
    s.store_class_end();
  }
}

setMessageReactions::ReturnType setMessageReactions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setMessageReactions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetMessageReactions").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reaction_types_fieldID = jni::get_field_id(env, Class, "reactionTypes", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    is_big_fieldID = jni::get_field_id(env, Class, "isBig", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setMessageSenderBlockList::Class;
jfieldID setMessageSenderBlockList::sender_id_fieldID;
jfieldID setMessageSenderBlockList::block_list_fieldID;

setMessageSenderBlockList::setMessageSenderBlockList()
  : sender_id_()
  , block_list_()
{}

setMessageSenderBlockList::setMessageSenderBlockList(object_ptr<MessageSender> &&sender_id_, object_ptr<BlockList> &&block_list_)
  : sender_id_(std::move(sender_id_))
  , block_list_(std::move(block_list_))
{}

const std::int32_t setMessageSenderBlockList::ID;

object_ptr<setMessageSenderBlockList> setMessageSenderBlockList::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setMessageSenderBlockList> res = make_object<setMessageSenderBlockList>();
  res->sender_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->sender_id_fieldID));
  res->block_list_ = jni::fetch_tl_object<BlockList>(env, jni::fetch_object(env, p, res->block_list_fieldID));
  return res;
}

void setMessageSenderBlockList::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetMessageSenderBlockList");
    s.store_object_field("senderId", static_cast<const BaseObject *>(sender_id_.get()));
    s.store_object_field("blockList", static_cast<const BaseObject *>(block_list_.get()));
    s.store_class_end();
  }
}

setMessageSenderBlockList::ReturnType setMessageSenderBlockList::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setMessageSenderBlockList::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetMessageSenderBlockList").c_str());
    sender_id_fieldID = jni::get_field_id(env, Class, "senderId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    block_list_fieldID = jni::get_field_id(env, Class, "blockList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$BlockList;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setMessageSenderBotVerification::Class;
jfieldID setMessageSenderBotVerification::bot_user_id_fieldID;
jfieldID setMessageSenderBotVerification::verified_id_fieldID;
jfieldID setMessageSenderBotVerification::custom_description_fieldID;

setMessageSenderBotVerification::setMessageSenderBotVerification()
  : bot_user_id_()
  , verified_id_()
  , custom_description_()
{}

setMessageSenderBotVerification::setMessageSenderBotVerification(int53 bot_user_id_, object_ptr<MessageSender> &&verified_id_, string const &custom_description_)
  : bot_user_id_(bot_user_id_)
  , verified_id_(std::move(verified_id_))
  , custom_description_(custom_description_)
{}

const std::int32_t setMessageSenderBotVerification::ID;

object_ptr<setMessageSenderBotVerification> setMessageSenderBotVerification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setMessageSenderBotVerification> res = make_object<setMessageSenderBotVerification>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->verified_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->verified_id_fieldID));
  res->custom_description_ = jni::fetch_string(env, p, res->custom_description_fieldID);
  return res;
}

void setMessageSenderBotVerification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetMessageSenderBotVerification");
    s.store_field("botUserId", bot_user_id_);
    s.store_object_field("verifiedId", static_cast<const BaseObject *>(verified_id_.get()));
    s.store_field("customDescription", custom_description_);
    s.store_class_end();
  }
}

setMessageSenderBotVerification::ReturnType setMessageSenderBotVerification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setMessageSenderBotVerification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetMessageSenderBotVerification").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    verified_id_fieldID = jni::get_field_id(env, Class, "verifiedId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    custom_description_fieldID = jni::get_field_id(env, Class, "customDescription", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setName::Class;
jfieldID setName::first_name_fieldID;
jfieldID setName::last_name_fieldID;

setName::setName()
  : first_name_()
  , last_name_()
{}

setName::setName(string const &first_name_, string const &last_name_)
  : first_name_(first_name_)
  , last_name_(last_name_)
{}

const std::int32_t setName::ID;

object_ptr<setName> setName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setName> res = make_object<setName>();
  res->first_name_ = jni::fetch_string(env, p, res->first_name_fieldID);
  res->last_name_ = jni::fetch_string(env, p, res->last_name_fieldID);
  return res;
}

void setName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetName");
    s.store_field("firstName", first_name_);
    s.store_field("lastName", last_name_);
    s.store_class_end();
  }
}

setName::ReturnType setName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetName").c_str());
    first_name_fieldID = jni::get_field_id(env, Class, "firstName", "Ljava/lang/String;");
    last_name_fieldID = jni::get_field_id(env, Class, "lastName", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setNetworkType::Class;
jfieldID setNetworkType::type_fieldID;

setNetworkType::setNetworkType()
  : type_()
{}

setNetworkType::setNetworkType(object_ptr<NetworkType> &&type_)
  : type_(std::move(type_))
{}

const std::int32_t setNetworkType::ID;

object_ptr<setNetworkType> setNetworkType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setNetworkType> res = make_object<setNetworkType>();
  res->type_ = jni::fetch_tl_object<NetworkType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void setNetworkType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetNetworkType");
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

setNetworkType::ReturnType setNetworkType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setNetworkType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetNetworkType").c_str());
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NetworkType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setNewChatPrivacySettings::Class;
jfieldID setNewChatPrivacySettings::settings_fieldID;

setNewChatPrivacySettings::setNewChatPrivacySettings()
  : settings_()
{}

setNewChatPrivacySettings::setNewChatPrivacySettings(object_ptr<newChatPrivacySettings> &&settings_)
  : settings_(std::move(settings_))
{}

const std::int32_t setNewChatPrivacySettings::ID;

object_ptr<setNewChatPrivacySettings> setNewChatPrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setNewChatPrivacySettings> res = make_object<setNewChatPrivacySettings>();
  res->settings_ = jni::fetch_tl_object<newChatPrivacySettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setNewChatPrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetNewChatPrivacySettings");
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setNewChatPrivacySettings::ReturnType setNewChatPrivacySettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setNewChatPrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetNewChatPrivacySettings").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NewChatPrivacySettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setOption::Class;
jfieldID setOption::name_fieldID;
jfieldID setOption::value_fieldID;

setOption::setOption()
  : name_()
  , value_()
{}

setOption::setOption(string const &name_, object_ptr<OptionValue> &&value_)
  : name_(name_)
  , value_(std::move(value_))
{}

const std::int32_t setOption::ID;

object_ptr<setOption> setOption::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setOption> res = make_object<setOption>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->value_ = jni::fetch_tl_object<OptionValue>(env, jni::fetch_object(env, p, res->value_fieldID));
  return res;
}

void setOption::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetOption");
    s.store_field("name", name_);
    s.store_object_field("value", static_cast<const BaseObject *>(value_.get()));
    s.store_class_end();
  }
}

setOption::ReturnType setOption::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setOption::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetOption").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    value_fieldID = jni::get_field_id(env, Class, "value", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OptionValue;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setPaidMessageReactionType::Class;
jfieldID setPaidMessageReactionType::chat_id_fieldID;
jfieldID setPaidMessageReactionType::message_id_fieldID;
jfieldID setPaidMessageReactionType::type_fieldID;

setPaidMessageReactionType::setPaidMessageReactionType()
  : chat_id_()
  , message_id_()
  , type_()
{}

setPaidMessageReactionType::setPaidMessageReactionType(int53 chat_id_, int53 message_id_, object_ptr<PaidReactionType> &&type_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , type_(std::move(type_))
{}

const std::int32_t setPaidMessageReactionType::ID;

object_ptr<setPaidMessageReactionType> setPaidMessageReactionType::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPaidMessageReactionType> res = make_object<setPaidMessageReactionType>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->type_ = jni::fetch_tl_object<PaidReactionType>(env, jni::fetch_object(env, p, res->type_fieldID));
  return res;
}

void setPaidMessageReactionType::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPaidMessageReactionType");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_class_end();
  }
}

setPaidMessageReactionType::ReturnType setPaidMessageReactionType::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPaidMessageReactionType::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPaidMessageReactionType").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PaidReactionType;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setPassportElement::Class;
jfieldID setPassportElement::element_fieldID;
jfieldID setPassportElement::password_fieldID;

setPassportElement::setPassportElement()
  : element_()
  , password_()
{}

setPassportElement::setPassportElement(object_ptr<InputPassportElement> &&element_, string const &password_)
  : element_(std::move(element_))
  , password_(password_)
{}

const std::int32_t setPassportElement::ID;

object_ptr<setPassportElement> setPassportElement::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPassportElement> res = make_object<setPassportElement>();
  res->element_ = jni::fetch_tl_object<InputPassportElement>(env, jni::fetch_object(env, p, res->element_fieldID));
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void setPassportElement::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPassportElement");
    s.store_object_field("element", static_cast<const BaseObject *>(element_.get()));
    s.store_field("password", password_);
    s.store_class_end();
  }
}

setPassportElement::ReturnType setPassportElement::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return PassportElement::fetch(env, p);
}

void setPassportElement::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPassportElement").c_str());
    element_fieldID = jni::get_field_id(env, Class, "element", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputPassportElement;").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setPassportElementErrors::Class;
jfieldID setPassportElementErrors::user_id_fieldID;
jfieldID setPassportElementErrors::errors_fieldID;

setPassportElementErrors::setPassportElementErrors()
  : user_id_()
  , errors_()
{}

setPassportElementErrors::setPassportElementErrors(int53 user_id_, array<object_ptr<inputPassportElementError>> &&errors_)
  : user_id_(user_id_)
  , errors_(std::move(errors_))
{}

const std::int32_t setPassportElementErrors::ID;

object_ptr<setPassportElementErrors> setPassportElementErrors::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPassportElementErrors> res = make_object<setPassportElementErrors>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->errors_ = jni::FetchVector<inputPassportElementError>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->errors_fieldID));
  return res;
}

void setPassportElementErrors::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPassportElementErrors");
    s.store_field("userId", user_id_);
    { s.store_vector_begin("errors", errors_.size()); for (const auto &_value : errors_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPassportElementErrors::ReturnType setPassportElementErrors::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPassportElementErrors::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPassportElementErrors").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    errors_fieldID = jni::get_field_id(env, Class, "errors", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$InputPassportElementError;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setPassword::Class;
jfieldID setPassword::old_password_fieldID;
jfieldID setPassword::new_password_fieldID;
jfieldID setPassword::new_hint_fieldID;
jfieldID setPassword::set_recovery_email_address_fieldID;
jfieldID setPassword::new_recovery_email_address_fieldID;

setPassword::setPassword()
  : old_password_()
  , new_password_()
  , new_hint_()
  , set_recovery_email_address_()
  , new_recovery_email_address_()
{}

setPassword::setPassword(string const &old_password_, string const &new_password_, string const &new_hint_, bool set_recovery_email_address_, string const &new_recovery_email_address_)
  : old_password_(old_password_)
  , new_password_(new_password_)
  , new_hint_(new_hint_)
  , set_recovery_email_address_(set_recovery_email_address_)
  , new_recovery_email_address_(new_recovery_email_address_)
{}

const std::int32_t setPassword::ID;

object_ptr<setPassword> setPassword::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPassword> res = make_object<setPassword>();
  res->old_password_ = jni::fetch_string(env, p, res->old_password_fieldID);
  res->new_password_ = jni::fetch_string(env, p, res->new_password_fieldID);
  res->new_hint_ = jni::fetch_string(env, p, res->new_hint_fieldID);
  res->set_recovery_email_address_ = (env->GetBooleanField(p, res->set_recovery_email_address_fieldID) != 0);
  res->new_recovery_email_address_ = jni::fetch_string(env, p, res->new_recovery_email_address_fieldID);
  return res;
}

void setPassword::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPassword");
    s.store_field("oldPassword", old_password_);
    s.store_field("newPassword", new_password_);
    s.store_field("newHint", new_hint_);
    s.store_field("setRecoveryEmailAddress", set_recovery_email_address_);
    s.store_field("newRecoveryEmailAddress", new_recovery_email_address_);
    s.store_class_end();
  }
}

setPassword::ReturnType setPassword::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void setPassword::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPassword").c_str());
    old_password_fieldID = jni::get_field_id(env, Class, "oldPassword", "Ljava/lang/String;");
    new_password_fieldID = jni::get_field_id(env, Class, "newPassword", "Ljava/lang/String;");
    new_hint_fieldID = jni::get_field_id(env, Class, "newHint", "Ljava/lang/String;");
    set_recovery_email_address_fieldID = jni::get_field_id(env, Class, "setRecoveryEmailAddress", "Z");
    new_recovery_email_address_fieldID = jni::get_field_id(env, Class, "newRecoveryEmailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setPersonalChat::Class;
jfieldID setPersonalChat::chat_id_fieldID;

setPersonalChat::setPersonalChat()
  : chat_id_()
{}

setPersonalChat::setPersonalChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t setPersonalChat::ID;

object_ptr<setPersonalChat> setPersonalChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPersonalChat> res = make_object<setPersonalChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void setPersonalChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPersonalChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

setPersonalChat::ReturnType setPersonalChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPersonalChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPersonalChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setPinnedChats::Class;
jfieldID setPinnedChats::chat_list_fieldID;
jfieldID setPinnedChats::chat_ids_fieldID;

setPinnedChats::setPinnedChats()
  : chat_list_()
  , chat_ids_()
{}

setPinnedChats::setPinnedChats(object_ptr<ChatList> &&chat_list_, array<int53> &&chat_ids_)
  : chat_list_(std::move(chat_list_))
  , chat_ids_(std::move(chat_ids_))
{}

const std::int32_t setPinnedChats::ID;

object_ptr<setPinnedChats> setPinnedChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPinnedChats> res = make_object<setPinnedChats>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->chat_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->chat_ids_fieldID));
  return res;
}

void setPinnedChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPinnedChats");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    { s.store_vector_begin("chatIds", chat_ids_.size()); for (const auto &_value : chat_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPinnedChats::ReturnType setPinnedChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPinnedChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPinnedChats").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    chat_ids_fieldID = jni::get_field_id(env, Class, "chatIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass setPinnedForumTopics::Class;
jfieldID setPinnedForumTopics::chat_id_fieldID;
jfieldID setPinnedForumTopics::message_thread_ids_fieldID;

setPinnedForumTopics::setPinnedForumTopics()
  : chat_id_()
  , message_thread_ids_()
{}

setPinnedForumTopics::setPinnedForumTopics(int53 chat_id_, array<int53> &&message_thread_ids_)
  : chat_id_(chat_id_)
  , message_thread_ids_(std::move(message_thread_ids_))
{}

const std::int32_t setPinnedForumTopics::ID;

object_ptr<setPinnedForumTopics> setPinnedForumTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPinnedForumTopics> res = make_object<setPinnedForumTopics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_thread_ids_fieldID));
  return res;
}

void setPinnedForumTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPinnedForumTopics");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageThreadIds", message_thread_ids_.size()); for (const auto &_value : message_thread_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPinnedForumTopics::ReturnType setPinnedForumTopics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPinnedForumTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPinnedForumTopics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_ids_fieldID = jni::get_field_id(env, Class, "messageThreadIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass setPinnedGifts::Class;
jfieldID setPinnedGifts::owner_id_fieldID;
jfieldID setPinnedGifts::received_gift_ids_fieldID;

setPinnedGifts::setPinnedGifts()
  : owner_id_()
  , received_gift_ids_()
{}

setPinnedGifts::setPinnedGifts(object_ptr<MessageSender> &&owner_id_, array<string> &&received_gift_ids_)
  : owner_id_(std::move(owner_id_))
  , received_gift_ids_(std::move(received_gift_ids_))
{}

const std::int32_t setPinnedGifts::ID;

object_ptr<setPinnedGifts> setPinnedGifts::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPinnedGifts> res = make_object<setPinnedGifts>();
  res->owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->owner_id_fieldID));
  res->received_gift_ids_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->received_gift_ids_fieldID));
  return res;
}

void setPinnedGifts::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPinnedGifts");
    s.store_object_field("ownerId", static_cast<const BaseObject *>(owner_id_.get()));
    { s.store_vector_begin("receivedGiftIds", received_gift_ids_.size()); for (const auto &_value : received_gift_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPinnedGifts::ReturnType setPinnedGifts::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPinnedGifts::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPinnedGifts").c_str());
    owner_id_fieldID = jni::get_field_id(env, Class, "ownerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    received_gift_ids_fieldID = jni::get_field_id(env, Class, "receivedGiftIds", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setPinnedSavedMessagesTopics::Class;
jfieldID setPinnedSavedMessagesTopics::saved_messages_topic_ids_fieldID;

setPinnedSavedMessagesTopics::setPinnedSavedMessagesTopics()
  : saved_messages_topic_ids_()
{}

setPinnedSavedMessagesTopics::setPinnedSavedMessagesTopics(array<int53> &&saved_messages_topic_ids_)
  : saved_messages_topic_ids_(std::move(saved_messages_topic_ids_))
{}

const std::int32_t setPinnedSavedMessagesTopics::ID;

object_ptr<setPinnedSavedMessagesTopics> setPinnedSavedMessagesTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPinnedSavedMessagesTopics> res = make_object<setPinnedSavedMessagesTopics>();
  res->saved_messages_topic_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->saved_messages_topic_ids_fieldID));
  return res;
}

void setPinnedSavedMessagesTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPinnedSavedMessagesTopics");
    { s.store_vector_begin("savedMessagesTopicIds", saved_messages_topic_ids_.size()); for (const auto &_value : saved_messages_topic_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPinnedSavedMessagesTopics::ReturnType setPinnedSavedMessagesTopics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPinnedSavedMessagesTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPinnedSavedMessagesTopics").c_str());
    saved_messages_topic_ids_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass setPollAnswer::Class;
jfieldID setPollAnswer::chat_id_fieldID;
jfieldID setPollAnswer::message_id_fieldID;
jfieldID setPollAnswer::option_ids_fieldID;

setPollAnswer::setPollAnswer()
  : chat_id_()
  , message_id_()
  , option_ids_()
{}

setPollAnswer::setPollAnswer(int53 chat_id_, int53 message_id_, array<int32> &&option_ids_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , option_ids_(std::move(option_ids_))
{}

const std::int32_t setPollAnswer::ID;

object_ptr<setPollAnswer> setPollAnswer::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setPollAnswer> res = make_object<setPollAnswer>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->option_ids_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->option_ids_fieldID));
  return res;
}

void setPollAnswer::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetPollAnswer");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    { s.store_vector_begin("optionIds", option_ids_.size()); for (const auto &_value : option_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setPollAnswer::ReturnType setPollAnswer::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setPollAnswer::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetPollAnswer").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    option_ids_fieldID = jni::get_field_id(env, Class, "optionIds", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass setProfileAccentColor::Class;
jfieldID setProfileAccentColor::profile_accent_color_id_fieldID;
jfieldID setProfileAccentColor::profile_background_custom_emoji_id_fieldID;

setProfileAccentColor::setProfileAccentColor()
  : profile_accent_color_id_()
  , profile_background_custom_emoji_id_()
{}

setProfileAccentColor::setProfileAccentColor(int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_)
  : profile_accent_color_id_(profile_accent_color_id_)
  , profile_background_custom_emoji_id_(profile_background_custom_emoji_id_)
{}

const std::int32_t setProfileAccentColor::ID;

object_ptr<setProfileAccentColor> setProfileAccentColor::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setProfileAccentColor> res = make_object<setProfileAccentColor>();
  res->profile_accent_color_id_ = env->GetIntField(p, res->profile_accent_color_id_fieldID);
  res->profile_background_custom_emoji_id_ = env->GetLongField(p, res->profile_background_custom_emoji_id_fieldID);
  return res;
}

void setProfileAccentColor::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetProfileAccentColor");
    s.store_field("profileAccentColorId", profile_accent_color_id_);
    s.store_field("profileBackgroundCustomEmojiId", profile_background_custom_emoji_id_);
    s.store_class_end();
  }
}

setProfileAccentColor::ReturnType setProfileAccentColor::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setProfileAccentColor::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetProfileAccentColor").c_str());
    profile_accent_color_id_fieldID = jni::get_field_id(env, Class, "profileAccentColorId", "I");
    profile_background_custom_emoji_id_fieldID = jni::get_field_id(env, Class, "profileBackgroundCustomEmojiId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setProfilePhoto::Class;
jfieldID setProfilePhoto::photo_fieldID;
jfieldID setProfilePhoto::is_public_fieldID;

setProfilePhoto::setProfilePhoto()
  : photo_()
  , is_public_()
{}

setProfilePhoto::setProfilePhoto(object_ptr<InputChatPhoto> &&photo_, bool is_public_)
  : photo_(std::move(photo_))
  , is_public_(is_public_)
{}

const std::int32_t setProfilePhoto::ID;

object_ptr<setProfilePhoto> setProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setProfilePhoto> res = make_object<setProfilePhoto>();
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  res->is_public_ = (env->GetBooleanField(p, res->is_public_fieldID) != 0);
  return res;
}

void setProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetProfilePhoto");
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_field("isPublic", is_public_);
    s.store_class_end();
  }
}

setProfilePhoto::ReturnType setProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetProfilePhoto").c_str());
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    is_public_fieldID = jni::get_field_id(env, Class, "isPublic", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setQuickReplyShortcutName::Class;
jfieldID setQuickReplyShortcutName::shortcut_id_fieldID;
jfieldID setQuickReplyShortcutName::name_fieldID;

setQuickReplyShortcutName::setQuickReplyShortcutName()
  : shortcut_id_()
  , name_()
{}

setQuickReplyShortcutName::setQuickReplyShortcutName(int32 shortcut_id_, string const &name_)
  : shortcut_id_(shortcut_id_)
  , name_(name_)
{}

const std::int32_t setQuickReplyShortcutName::ID;

object_ptr<setQuickReplyShortcutName> setQuickReplyShortcutName::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setQuickReplyShortcutName> res = make_object<setQuickReplyShortcutName>();
  res->shortcut_id_ = env->GetIntField(p, res->shortcut_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  return res;
}

void setQuickReplyShortcutName::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetQuickReplyShortcutName");
    s.store_field("shortcutId", shortcut_id_);
    s.store_field("name", name_);
    s.store_class_end();
  }
}

setQuickReplyShortcutName::ReturnType setQuickReplyShortcutName::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setQuickReplyShortcutName::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetQuickReplyShortcutName").c_str());
    shortcut_id_fieldID = jni::get_field_id(env, Class, "shortcutId", "I");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setReactionNotificationSettings::Class;
jfieldID setReactionNotificationSettings::notification_settings_fieldID;

setReactionNotificationSettings::setReactionNotificationSettings()
  : notification_settings_()
{}

setReactionNotificationSettings::setReactionNotificationSettings(object_ptr<reactionNotificationSettings> &&notification_settings_)
  : notification_settings_(std::move(notification_settings_))
{}

const std::int32_t setReactionNotificationSettings::ID;

object_ptr<setReactionNotificationSettings> setReactionNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setReactionNotificationSettings> res = make_object<setReactionNotificationSettings>();
  res->notification_settings_ = jni::fetch_tl_object<reactionNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return res;
}

void setReactionNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetReactionNotificationSettings");
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

setReactionNotificationSettings::ReturnType setReactionNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setReactionNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetReactionNotificationSettings").c_str());
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setReadDatePrivacySettings::Class;
jfieldID setReadDatePrivacySettings::settings_fieldID;

setReadDatePrivacySettings::setReadDatePrivacySettings()
  : settings_()
{}

setReadDatePrivacySettings::setReadDatePrivacySettings(object_ptr<readDatePrivacySettings> &&settings_)
  : settings_(std::move(settings_))
{}

const std::int32_t setReadDatePrivacySettings::ID;

object_ptr<setReadDatePrivacySettings> setReadDatePrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setReadDatePrivacySettings> res = make_object<setReadDatePrivacySettings>();
  res->settings_ = jni::fetch_tl_object<readDatePrivacySettings>(env, jni::fetch_object(env, p, res->settings_fieldID));
  return res;
}

void setReadDatePrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetReadDatePrivacySettings");
    s.store_object_field("settings", static_cast<const BaseObject *>(settings_.get()));
    s.store_class_end();
  }
}

setReadDatePrivacySettings::ReturnType setReadDatePrivacySettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setReadDatePrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetReadDatePrivacySettings").c_str());
    settings_fieldID = jni::get_field_id(env, Class, "settings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReadDatePrivacySettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setRecoveryEmailAddress::Class;
jfieldID setRecoveryEmailAddress::password_fieldID;
jfieldID setRecoveryEmailAddress::new_recovery_email_address_fieldID;

setRecoveryEmailAddress::setRecoveryEmailAddress()
  : password_()
  , new_recovery_email_address_()
{}

setRecoveryEmailAddress::setRecoveryEmailAddress(string const &password_, string const &new_recovery_email_address_)
  : password_(password_)
  , new_recovery_email_address_(new_recovery_email_address_)
{}

const std::int32_t setRecoveryEmailAddress::ID;

object_ptr<setRecoveryEmailAddress> setRecoveryEmailAddress::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setRecoveryEmailAddress> res = make_object<setRecoveryEmailAddress>();
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  res->new_recovery_email_address_ = jni::fetch_string(env, p, res->new_recovery_email_address_fieldID);
  return res;
}

void setRecoveryEmailAddress::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetRecoveryEmailAddress");
    s.store_field("password", password_);
    s.store_field("newRecoveryEmailAddress", new_recovery_email_address_);
    s.store_class_end();
  }
}

setRecoveryEmailAddress::ReturnType setRecoveryEmailAddress::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return passwordState::fetch(env, p);
}

void setRecoveryEmailAddress::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetRecoveryEmailAddress").c_str());
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    new_recovery_email_address_fieldID = jni::get_field_id(env, Class, "newRecoveryEmailAddress", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setSavedMessagesTagLabel::Class;
jfieldID setSavedMessagesTagLabel::tag_fieldID;
jfieldID setSavedMessagesTagLabel::label_fieldID;

setSavedMessagesTagLabel::setSavedMessagesTagLabel()
  : tag_()
  , label_()
{}

setSavedMessagesTagLabel::setSavedMessagesTagLabel(object_ptr<ReactionType> &&tag_, string const &label_)
  : tag_(std::move(tag_))
  , label_(label_)
{}

const std::int32_t setSavedMessagesTagLabel::ID;

object_ptr<setSavedMessagesTagLabel> setSavedMessagesTagLabel::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setSavedMessagesTagLabel> res = make_object<setSavedMessagesTagLabel>();
  res->tag_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->tag_fieldID));
  res->label_ = jni::fetch_string(env, p, res->label_fieldID);
  return res;
}

void setSavedMessagesTagLabel::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetSavedMessagesTagLabel");
    s.store_object_field("tag", static_cast<const BaseObject *>(tag_.get()));
    s.store_field("label", label_);
    s.store_class_end();
  }
}

setSavedMessagesTagLabel::ReturnType setSavedMessagesTagLabel::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setSavedMessagesTagLabel::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetSavedMessagesTagLabel").c_str());
    tag_fieldID = jni::get_field_id(env, Class, "tag", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    label_fieldID = jni::get_field_id(env, Class, "label", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setScopeNotificationSettings::Class;
jfieldID setScopeNotificationSettings::scope_fieldID;
jfieldID setScopeNotificationSettings::notification_settings_fieldID;

setScopeNotificationSettings::setScopeNotificationSettings()
  : scope_()
  , notification_settings_()
{}

setScopeNotificationSettings::setScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_, object_ptr<scopeNotificationSettings> &&notification_settings_)
  : scope_(std::move(scope_))
  , notification_settings_(std::move(notification_settings_))
{}

const std::int32_t setScopeNotificationSettings::ID;

object_ptr<setScopeNotificationSettings> setScopeNotificationSettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setScopeNotificationSettings> res = make_object<setScopeNotificationSettings>();
  res->scope_ = jni::fetch_tl_object<NotificationSettingsScope>(env, jni::fetch_object(env, p, res->scope_fieldID));
  res->notification_settings_ = jni::fetch_tl_object<scopeNotificationSettings>(env, jni::fetch_object(env, p, res->notification_settings_fieldID));
  return res;
}

void setScopeNotificationSettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetScopeNotificationSettings");
    s.store_object_field("scope", static_cast<const BaseObject *>(scope_.get()));
    s.store_object_field("notificationSettings", static_cast<const BaseObject *>(notification_settings_.get()));
    s.store_class_end();
  }
}

setScopeNotificationSettings::ReturnType setScopeNotificationSettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setScopeNotificationSettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetScopeNotificationSettings").c_str());
    scope_fieldID = jni::get_field_id(env, Class, "scope", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$NotificationSettingsScope;").c_str());
    notification_settings_fieldID = jni::get_field_id(env, Class, "notificationSettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ScopeNotificationSettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setStickerEmojis::Class;
jfieldID setStickerEmojis::sticker_fieldID;
jfieldID setStickerEmojis::emojis_fieldID;

setStickerEmojis::setStickerEmojis()
  : sticker_()
  , emojis_()
{}

setStickerEmojis::setStickerEmojis(object_ptr<InputFile> &&sticker_, string const &emojis_)
  : sticker_(std::move(sticker_))
  , emojis_(emojis_)
{}

const std::int32_t setStickerEmojis::ID;

object_ptr<setStickerEmojis> setStickerEmojis::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerEmojis> res = make_object<setStickerEmojis>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->emojis_ = jni::fetch_string(env, p, res->emojis_fieldID);
  return res;
}

void setStickerEmojis::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerEmojis");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("emojis", emojis_);
    s.store_class_end();
  }
}

setStickerEmojis::ReturnType setStickerEmojis::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerEmojis::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerEmojis").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    emojis_fieldID = jni::get_field_id(env, Class, "emojis", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setStickerKeywords::Class;
jfieldID setStickerKeywords::sticker_fieldID;
jfieldID setStickerKeywords::keywords_fieldID;

setStickerKeywords::setStickerKeywords()
  : sticker_()
  , keywords_()
{}

setStickerKeywords::setStickerKeywords(object_ptr<InputFile> &&sticker_, array<string> &&keywords_)
  : sticker_(std::move(sticker_))
  , keywords_(std::move(keywords_))
{}

const std::int32_t setStickerKeywords::ID;

object_ptr<setStickerKeywords> setStickerKeywords::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerKeywords> res = make_object<setStickerKeywords>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->keywords_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->keywords_fieldID));
  return res;
}

void setStickerKeywords::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerKeywords");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    { s.store_vector_begin("keywords", keywords_.size()); for (const auto &_value : keywords_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

setStickerKeywords::ReturnType setStickerKeywords::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerKeywords::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerKeywords").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    keywords_fieldID = jni::get_field_id(env, Class, "keywords", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setStickerMaskPosition::Class;
jfieldID setStickerMaskPosition::sticker_fieldID;
jfieldID setStickerMaskPosition::mask_position_fieldID;

setStickerMaskPosition::setStickerMaskPosition()
  : sticker_()
  , mask_position_()
{}

setStickerMaskPosition::setStickerMaskPosition(object_ptr<InputFile> &&sticker_, object_ptr<maskPosition> &&mask_position_)
  : sticker_(std::move(sticker_))
  , mask_position_(std::move(mask_position_))
{}

const std::int32_t setStickerMaskPosition::ID;

object_ptr<setStickerMaskPosition> setStickerMaskPosition::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerMaskPosition> res = make_object<setStickerMaskPosition>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->mask_position_ = jni::fetch_tl_object<maskPosition>(env, jni::fetch_object(env, p, res->mask_position_fieldID));
  return res;
}

void setStickerMaskPosition::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerMaskPosition");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_object_field("maskPosition", static_cast<const BaseObject *>(mask_position_.get()));
    s.store_class_end();
  }
}

setStickerMaskPosition::ReturnType setStickerMaskPosition::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerMaskPosition::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerMaskPosition").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    mask_position_fieldID = jni::get_field_id(env, Class, "maskPosition", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MaskPosition;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setStickerPositionInSet::Class;
jfieldID setStickerPositionInSet::sticker_fieldID;
jfieldID setStickerPositionInSet::position_fieldID;

setStickerPositionInSet::setStickerPositionInSet()
  : sticker_()
  , position_()
{}

setStickerPositionInSet::setStickerPositionInSet(object_ptr<InputFile> &&sticker_, int32 position_)
  : sticker_(std::move(sticker_))
  , position_(position_)
{}

const std::int32_t setStickerPositionInSet::ID;

object_ptr<setStickerPositionInSet> setStickerPositionInSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerPositionInSet> res = make_object<setStickerPositionInSet>();
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  res->position_ = env->GetIntField(p, res->position_fieldID);
  return res;
}

void setStickerPositionInSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerPositionInSet");
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_field("position", position_);
    s.store_class_end();
  }
}

setStickerPositionInSet::ReturnType setStickerPositionInSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerPositionInSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerPositionInSet").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    position_fieldID = jni::get_field_id(env, Class, "position", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setStickerSetThumbnail::Class;
jfieldID setStickerSetThumbnail::user_id_fieldID;
jfieldID setStickerSetThumbnail::name_fieldID;
jfieldID setStickerSetThumbnail::thumbnail_fieldID;
jfieldID setStickerSetThumbnail::format_fieldID;

setStickerSetThumbnail::setStickerSetThumbnail()
  : user_id_()
  , name_()
  , thumbnail_()
  , format_()
{}

setStickerSetThumbnail::setStickerSetThumbnail(int53 user_id_, string const &name_, object_ptr<InputFile> &&thumbnail_, object_ptr<StickerFormat> &&format_)
  : user_id_(user_id_)
  , name_(name_)
  , thumbnail_(std::move(thumbnail_))
  , format_(std::move(format_))
{}

const std::int32_t setStickerSetThumbnail::ID;

object_ptr<setStickerSetThumbnail> setStickerSetThumbnail::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerSetThumbnail> res = make_object<setStickerSetThumbnail>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->thumbnail_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->thumbnail_fieldID));
  res->format_ = jni::fetch_tl_object<StickerFormat>(env, jni::fetch_object(env, p, res->format_fieldID));
  return res;
}

void setStickerSetThumbnail::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerSetThumbnail");
    s.store_field("userId", user_id_);
    s.store_field("name", name_);
    s.store_object_field("thumbnail", static_cast<const BaseObject *>(thumbnail_.get()));
    s.store_object_field("format", static_cast<const BaseObject *>(format_.get()));
    s.store_class_end();
  }
}

setStickerSetThumbnail::ReturnType setStickerSetThumbnail::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerSetThumbnail::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerSetThumbnail").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    thumbnail_fieldID = jni::get_field_id(env, Class, "thumbnail", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    format_fieldID = jni::get_field_id(env, Class, "format", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerFormat;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setStickerSetTitle::Class;
jfieldID setStickerSetTitle::name_fieldID;
jfieldID setStickerSetTitle::title_fieldID;

setStickerSetTitle::setStickerSetTitle()
  : name_()
  , title_()
{}

setStickerSetTitle::setStickerSetTitle(string const &name_, string const &title_)
  : name_(name_)
  , title_(title_)
{}

const std::int32_t setStickerSetTitle::ID;

object_ptr<setStickerSetTitle> setStickerSetTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStickerSetTitle> res = make_object<setStickerSetTitle>();
  res->name_ = jni::fetch_string(env, p, res->name_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void setStickerSetTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStickerSetTitle");
    s.store_field("name", name_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

setStickerSetTitle::ReturnType setStickerSetTitle::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStickerSetTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStickerSetTitle").c_str());
    name_fieldID = jni::get_field_id(env, Class, "name", "Ljava/lang/String;");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setStoryPrivacySettings::Class;
jfieldID setStoryPrivacySettings::story_id_fieldID;
jfieldID setStoryPrivacySettings::privacy_settings_fieldID;

setStoryPrivacySettings::setStoryPrivacySettings()
  : story_id_()
  , privacy_settings_()
{}

setStoryPrivacySettings::setStoryPrivacySettings(int32 story_id_, object_ptr<StoryPrivacySettings> &&privacy_settings_)
  : story_id_(story_id_)
  , privacy_settings_(std::move(privacy_settings_))
{}

const std::int32_t setStoryPrivacySettings::ID;

object_ptr<setStoryPrivacySettings> setStoryPrivacySettings::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStoryPrivacySettings> res = make_object<setStoryPrivacySettings>();
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->privacy_settings_ = jni::fetch_tl_object<StoryPrivacySettings>(env, jni::fetch_object(env, p, res->privacy_settings_fieldID));
  return res;
}

void setStoryPrivacySettings::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStoryPrivacySettings");
    s.store_field("storyId", story_id_);
    s.store_object_field("privacySettings", static_cast<const BaseObject *>(privacy_settings_.get()));
    s.store_class_end();
  }
}

setStoryPrivacySettings::ReturnType setStoryPrivacySettings::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStoryPrivacySettings::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStoryPrivacySettings").c_str());
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    privacy_settings_fieldID = jni::get_field_id(env, Class, "privacySettings", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StoryPrivacySettings;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setStoryReaction::Class;
jfieldID setStoryReaction::story_poster_chat_id_fieldID;
jfieldID setStoryReaction::story_id_fieldID;
jfieldID setStoryReaction::reaction_type_fieldID;
jfieldID setStoryReaction::update_recent_reactions_fieldID;

setStoryReaction::setStoryReaction()
  : story_poster_chat_id_()
  , story_id_()
  , reaction_type_()
  , update_recent_reactions_()
{}

setStoryReaction::setStoryReaction(int53 story_poster_chat_id_, int32 story_id_, object_ptr<ReactionType> &&reaction_type_, bool update_recent_reactions_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , reaction_type_(std::move(reaction_type_))
  , update_recent_reactions_(update_recent_reactions_)
{}

const std::int32_t setStoryReaction::ID;

object_ptr<setStoryReaction> setStoryReaction::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setStoryReaction> res = make_object<setStoryReaction>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->reaction_type_ = jni::fetch_tl_object<ReactionType>(env, jni::fetch_object(env, p, res->reaction_type_fieldID));
  res->update_recent_reactions_ = (env->GetBooleanField(p, res->update_recent_reactions_fieldID) != 0);
  return res;
}

void setStoryReaction::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetStoryReaction");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_object_field("reactionType", static_cast<const BaseObject *>(reaction_type_.get()));
    s.store_field("updateRecentReactions", update_recent_reactions_);
    s.store_class_end();
  }
}

setStoryReaction::ReturnType setStoryReaction::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setStoryReaction::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetStoryReaction").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    reaction_type_fieldID = jni::get_field_id(env, Class, "reactionType", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReactionType;").c_str());
    update_recent_reactions_fieldID = jni::get_field_id(env, Class, "updateRecentReactions", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass setSupergroupCustomEmojiStickerSet::Class;
jfieldID setSupergroupCustomEmojiStickerSet::supergroup_id_fieldID;
jfieldID setSupergroupCustomEmojiStickerSet::custom_emoji_sticker_set_id_fieldID;

setSupergroupCustomEmojiStickerSet::setSupergroupCustomEmojiStickerSet()
  : supergroup_id_()
  , custom_emoji_sticker_set_id_()
{}

setSupergroupCustomEmojiStickerSet::setSupergroupCustomEmojiStickerSet(int53 supergroup_id_, int64 custom_emoji_sticker_set_id_)
  : supergroup_id_(supergroup_id_)
  , custom_emoji_sticker_set_id_(custom_emoji_sticker_set_id_)
{}

const std::int32_t setSupergroupCustomEmojiStickerSet::ID;

object_ptr<setSupergroupCustomEmojiStickerSet> setSupergroupCustomEmojiStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setSupergroupCustomEmojiStickerSet> res = make_object<setSupergroupCustomEmojiStickerSet>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->custom_emoji_sticker_set_id_ = env->GetLongField(p, res->custom_emoji_sticker_set_id_fieldID);
  return res;
}

void setSupergroupCustomEmojiStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetSupergroupCustomEmojiStickerSet");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("customEmojiStickerSetId", custom_emoji_sticker_set_id_);
    s.store_class_end();
  }
}

setSupergroupCustomEmojiStickerSet::ReturnType setSupergroupCustomEmojiStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setSupergroupCustomEmojiStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetSupergroupCustomEmojiStickerSet").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    custom_emoji_sticker_set_id_fieldID = jni::get_field_id(env, Class, "customEmojiStickerSetId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setSupergroupStickerSet::Class;
jfieldID setSupergroupStickerSet::supergroup_id_fieldID;
jfieldID setSupergroupStickerSet::sticker_set_id_fieldID;

setSupergroupStickerSet::setSupergroupStickerSet()
  : supergroup_id_()
  , sticker_set_id_()
{}

setSupergroupStickerSet::setSupergroupStickerSet(int53 supergroup_id_, int64 sticker_set_id_)
  : supergroup_id_(supergroup_id_)
  , sticker_set_id_(sticker_set_id_)
{}

const std::int32_t setSupergroupStickerSet::ID;

object_ptr<setSupergroupStickerSet> setSupergroupStickerSet::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setSupergroupStickerSet> res = make_object<setSupergroupStickerSet>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->sticker_set_id_ = env->GetLongField(p, res->sticker_set_id_fieldID);
  return res;
}

void setSupergroupStickerSet::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetSupergroupStickerSet");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("stickerSetId", sticker_set_id_);
    s.store_class_end();
  }
}

setSupergroupStickerSet::ReturnType setSupergroupStickerSet::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setSupergroupStickerSet::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetSupergroupStickerSet").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    sticker_set_id_fieldID = jni::get_field_id(env, Class, "stickerSetId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass setSupergroupUnrestrictBoostCount::Class;
jfieldID setSupergroupUnrestrictBoostCount::supergroup_id_fieldID;
jfieldID setSupergroupUnrestrictBoostCount::unrestrict_boost_count_fieldID;

setSupergroupUnrestrictBoostCount::setSupergroupUnrestrictBoostCount()
  : supergroup_id_()
  , unrestrict_boost_count_()
{}

setSupergroupUnrestrictBoostCount::setSupergroupUnrestrictBoostCount(int53 supergroup_id_, int32 unrestrict_boost_count_)
  : supergroup_id_(supergroup_id_)
  , unrestrict_boost_count_(unrestrict_boost_count_)
{}

const std::int32_t setSupergroupUnrestrictBoostCount::ID;

object_ptr<setSupergroupUnrestrictBoostCount> setSupergroupUnrestrictBoostCount::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setSupergroupUnrestrictBoostCount> res = make_object<setSupergroupUnrestrictBoostCount>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->unrestrict_boost_count_ = env->GetIntField(p, res->unrestrict_boost_count_fieldID);
  return res;
}

void setSupergroupUnrestrictBoostCount::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetSupergroupUnrestrictBoostCount");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("unrestrictBoostCount", unrestrict_boost_count_);
    s.store_class_end();
  }
}

setSupergroupUnrestrictBoostCount::ReturnType setSupergroupUnrestrictBoostCount::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setSupergroupUnrestrictBoostCount::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetSupergroupUnrestrictBoostCount").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    unrestrict_boost_count_fieldID = jni::get_field_id(env, Class, "unrestrictBoostCount", "I");
    return true;
  }();
  (void)is_inited;
}

jclass setSupergroupUsername::Class;
jfieldID setSupergroupUsername::supergroup_id_fieldID;
jfieldID setSupergroupUsername::username_fieldID;

setSupergroupUsername::setSupergroupUsername()
  : supergroup_id_()
  , username_()
{}

setSupergroupUsername::setSupergroupUsername(int53 supergroup_id_, string const &username_)
  : supergroup_id_(supergroup_id_)
  , username_(username_)
{}

const std::int32_t setSupergroupUsername::ID;

object_ptr<setSupergroupUsername> setSupergroupUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setSupergroupUsername> res = make_object<setSupergroupUsername>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return res;
}

void setSupergroupUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetSupergroupUsername");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("username", username_);
    s.store_class_end();
  }
}

setSupergroupUsername::ReturnType setSupergroupUsername::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setSupergroupUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetSupergroupUsername").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setTdlibParameters::Class;
jfieldID setTdlibParameters::use_test_dc_fieldID;
jfieldID setTdlibParameters::database_directory_fieldID;
jfieldID setTdlibParameters::files_directory_fieldID;
jfieldID setTdlibParameters::database_encryption_key_fieldID;
jfieldID setTdlibParameters::use_file_database_fieldID;
jfieldID setTdlibParameters::use_chat_info_database_fieldID;
jfieldID setTdlibParameters::use_message_database_fieldID;
jfieldID setTdlibParameters::use_secret_chats_fieldID;
jfieldID setTdlibParameters::api_id_fieldID;
jfieldID setTdlibParameters::api_hash_fieldID;
jfieldID setTdlibParameters::system_language_code_fieldID;
jfieldID setTdlibParameters::device_model_fieldID;
jfieldID setTdlibParameters::system_version_fieldID;
jfieldID setTdlibParameters::application_version_fieldID;

setTdlibParameters::setTdlibParameters()
  : use_test_dc_()
  , database_directory_()
  , files_directory_()
  , database_encryption_key_()
  , use_file_database_()
  , use_chat_info_database_()
  , use_message_database_()
  , use_secret_chats_()
  , api_id_()
  , api_hash_()
  , system_language_code_()
  , device_model_()
  , system_version_()
  , application_version_()
{}

setTdlibParameters::setTdlibParameters(bool use_test_dc_, string const &database_directory_, string const &files_directory_, bytes const &database_encryption_key_, bool use_file_database_, bool use_chat_info_database_, bool use_message_database_, bool use_secret_chats_, int32 api_id_, string const &api_hash_, string const &system_language_code_, string const &device_model_, string const &system_version_, string const &application_version_)
  : use_test_dc_(use_test_dc_)
  , database_directory_(database_directory_)
  , files_directory_(files_directory_)
  , database_encryption_key_(std::move(database_encryption_key_))
  , use_file_database_(use_file_database_)
  , use_chat_info_database_(use_chat_info_database_)
  , use_message_database_(use_message_database_)
  , use_secret_chats_(use_secret_chats_)
  , api_id_(api_id_)
  , api_hash_(api_hash_)
  , system_language_code_(system_language_code_)
  , device_model_(device_model_)
  , system_version_(system_version_)
  , application_version_(application_version_)
{}

const std::int32_t setTdlibParameters::ID;

object_ptr<setTdlibParameters> setTdlibParameters::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setTdlibParameters> res = make_object<setTdlibParameters>();
  res->use_test_dc_ = (env->GetBooleanField(p, res->use_test_dc_fieldID) != 0);
  res->database_directory_ = jni::fetch_string(env, p, res->database_directory_fieldID);
  res->files_directory_ = jni::fetch_string(env, p, res->files_directory_fieldID);
  res->database_encryption_key_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->database_encryption_key_fieldID));
  res->use_file_database_ = (env->GetBooleanField(p, res->use_file_database_fieldID) != 0);
  res->use_chat_info_database_ = (env->GetBooleanField(p, res->use_chat_info_database_fieldID) != 0);
  res->use_message_database_ = (env->GetBooleanField(p, res->use_message_database_fieldID) != 0);
  res->use_secret_chats_ = (env->GetBooleanField(p, res->use_secret_chats_fieldID) != 0);
  res->api_id_ = env->GetIntField(p, res->api_id_fieldID);
  res->api_hash_ = jni::fetch_string(env, p, res->api_hash_fieldID);
  res->system_language_code_ = jni::fetch_string(env, p, res->system_language_code_fieldID);
  res->device_model_ = jni::fetch_string(env, p, res->device_model_fieldID);
  res->system_version_ = jni::fetch_string(env, p, res->system_version_fieldID);
  res->application_version_ = jni::fetch_string(env, p, res->application_version_fieldID);
  return res;
}

void setTdlibParameters::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetTdlibParameters");
    s.store_field("useTestDc", use_test_dc_);
    s.store_field("databaseDirectory", database_directory_);
    s.store_field("filesDirectory", files_directory_);
    s.store_bytes_field("databaseEncryptionKey", database_encryption_key_);
    s.store_field("useFileDatabase", use_file_database_);
    s.store_field("useChatInfoDatabase", use_chat_info_database_);
    s.store_field("useMessageDatabase", use_message_database_);
    s.store_field("useSecretChats", use_secret_chats_);
    s.store_field("apiId", api_id_);
    s.store_field("apiHash", api_hash_);
    s.store_field("systemLanguageCode", system_language_code_);
    s.store_field("deviceModel", device_model_);
    s.store_field("systemVersion", system_version_);
    s.store_field("applicationVersion", application_version_);
    s.store_class_end();
  }
}

setTdlibParameters::ReturnType setTdlibParameters::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setTdlibParameters::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetTdlibParameters").c_str());
    use_test_dc_fieldID = jni::get_field_id(env, Class, "useTestDc", "Z");
    database_directory_fieldID = jni::get_field_id(env, Class, "databaseDirectory", "Ljava/lang/String;");
    files_directory_fieldID = jni::get_field_id(env, Class, "filesDirectory", "Ljava/lang/String;");
    database_encryption_key_fieldID = jni::get_field_id(env, Class, "databaseEncryptionKey", "[B");
    use_file_database_fieldID = jni::get_field_id(env, Class, "useFileDatabase", "Z");
    use_chat_info_database_fieldID = jni::get_field_id(env, Class, "useChatInfoDatabase", "Z");
    use_message_database_fieldID = jni::get_field_id(env, Class, "useMessageDatabase", "Z");
    use_secret_chats_fieldID = jni::get_field_id(env, Class, "useSecretChats", "Z");
    api_id_fieldID = jni::get_field_id(env, Class, "apiId", "I");
    api_hash_fieldID = jni::get_field_id(env, Class, "apiHash", "Ljava/lang/String;");
    system_language_code_fieldID = jni::get_field_id(env, Class, "systemLanguageCode", "Ljava/lang/String;");
    device_model_fieldID = jni::get_field_id(env, Class, "deviceModel", "Ljava/lang/String;");
    system_version_fieldID = jni::get_field_id(env, Class, "systemVersion", "Ljava/lang/String;");
    application_version_fieldID = jni::get_field_id(env, Class, "applicationVersion", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setUserEmojiStatus::Class;
jfieldID setUserEmojiStatus::user_id_fieldID;
jfieldID setUserEmojiStatus::emoji_status_fieldID;

setUserEmojiStatus::setUserEmojiStatus()
  : user_id_()
  , emoji_status_()
{}

setUserEmojiStatus::setUserEmojiStatus(int53 user_id_, object_ptr<emojiStatus> &&emoji_status_)
  : user_id_(user_id_)
  , emoji_status_(std::move(emoji_status_))
{}

const std::int32_t setUserEmojiStatus::ID;

object_ptr<setUserEmojiStatus> setUserEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setUserEmojiStatus> res = make_object<setUserEmojiStatus>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->emoji_status_ = jni::fetch_tl_object<emojiStatus>(env, jni::fetch_object(env, p, res->emoji_status_fieldID));
  return res;
}

void setUserEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetUserEmojiStatus");
    s.store_field("userId", user_id_);
    s.store_object_field("emojiStatus", static_cast<const BaseObject *>(emoji_status_.get()));
    s.store_class_end();
  }
}

setUserEmojiStatus::ReturnType setUserEmojiStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setUserEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetUserEmojiStatus").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    emoji_status_fieldID = jni::get_field_id(env, Class, "emojiStatus", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$EmojiStatus;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setUserPersonalProfilePhoto::Class;
jfieldID setUserPersonalProfilePhoto::user_id_fieldID;
jfieldID setUserPersonalProfilePhoto::photo_fieldID;

setUserPersonalProfilePhoto::setUserPersonalProfilePhoto()
  : user_id_()
  , photo_()
{}

setUserPersonalProfilePhoto::setUserPersonalProfilePhoto(int53 user_id_, object_ptr<InputChatPhoto> &&photo_)
  : user_id_(user_id_)
  , photo_(std::move(photo_))
{}

const std::int32_t setUserPersonalProfilePhoto::ID;

object_ptr<setUserPersonalProfilePhoto> setUserPersonalProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setUserPersonalProfilePhoto> res = make_object<setUserPersonalProfilePhoto>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void setUserPersonalProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetUserPersonalProfilePhoto");
    s.store_field("userId", user_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

setUserPersonalProfilePhoto::ReturnType setUserPersonalProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setUserPersonalProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetUserPersonalProfilePhoto").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setUserPrivacySettingRules::Class;
jfieldID setUserPrivacySettingRules::setting_fieldID;
jfieldID setUserPrivacySettingRules::rules_fieldID;

setUserPrivacySettingRules::setUserPrivacySettingRules()
  : setting_()
  , rules_()
{}

setUserPrivacySettingRules::setUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_, object_ptr<userPrivacySettingRules> &&rules_)
  : setting_(std::move(setting_))
  , rules_(std::move(rules_))
{}

const std::int32_t setUserPrivacySettingRules::ID;

object_ptr<setUserPrivacySettingRules> setUserPrivacySettingRules::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setUserPrivacySettingRules> res = make_object<setUserPrivacySettingRules>();
  res->setting_ = jni::fetch_tl_object<UserPrivacySetting>(env, jni::fetch_object(env, p, res->setting_fieldID));
  res->rules_ = jni::fetch_tl_object<userPrivacySettingRules>(env, jni::fetch_object(env, p, res->rules_fieldID));
  return res;
}

void setUserPrivacySettingRules::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetUserPrivacySettingRules");
    s.store_object_field("setting", static_cast<const BaseObject *>(setting_.get()));
    s.store_object_field("rules", static_cast<const BaseObject *>(rules_.get()));
    s.store_class_end();
  }
}

setUserPrivacySettingRules::ReturnType setUserPrivacySettingRules::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setUserPrivacySettingRules::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetUserPrivacySettingRules").c_str());
    setting_fieldID = jni::get_field_id(env, Class, "setting", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserPrivacySetting;").c_str());
    rules_fieldID = jni::get_field_id(env, Class, "rules", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$UserPrivacySettingRules;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setUserSupportInfo::Class;
jfieldID setUserSupportInfo::user_id_fieldID;
jfieldID setUserSupportInfo::message_fieldID;

setUserSupportInfo::setUserSupportInfo()
  : user_id_()
  , message_()
{}

setUserSupportInfo::setUserSupportInfo(int53 user_id_, object_ptr<formattedText> &&message_)
  : user_id_(user_id_)
  , message_(std::move(message_))
{}

const std::int32_t setUserSupportInfo::ID;

object_ptr<setUserSupportInfo> setUserSupportInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setUserSupportInfo> res = make_object<setUserSupportInfo>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->message_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->message_fieldID));
  return res;
}

void setUserSupportInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetUserSupportInfo");
    s.store_field("userId", user_id_);
    s.store_object_field("message", static_cast<const BaseObject *>(message_.get()));
    s.store_class_end();
  }
}

setUserSupportInfo::ReturnType setUserSupportInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return userSupportInfo::fetch(env, p);
}

void setUserSupportInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetUserSupportInfo").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    message_fieldID = jni::get_field_id(env, Class, "message", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setUsername::Class;
jfieldID setUsername::username_fieldID;

setUsername::setUsername()
  : username_()
{}

setUsername::setUsername(string const &username_)
  : username_(username_)
{}

const std::int32_t setUsername::ID;

object_ptr<setUsername> setUsername::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setUsername> res = make_object<setUsername>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  return res;
}

void setUsername::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetUsername");
    s.store_field("username", username_);
    s.store_class_end();
  }
}

setUsername::ReturnType setUsername::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setUsername::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetUsername").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass setVideoChatDefaultParticipant::Class;
jfieldID setVideoChatDefaultParticipant::chat_id_fieldID;
jfieldID setVideoChatDefaultParticipant::default_participant_id_fieldID;

setVideoChatDefaultParticipant::setVideoChatDefaultParticipant()
  : chat_id_()
  , default_participant_id_()
{}

setVideoChatDefaultParticipant::setVideoChatDefaultParticipant(int53 chat_id_, object_ptr<MessageSender> &&default_participant_id_)
  : chat_id_(chat_id_)
  , default_participant_id_(std::move(default_participant_id_))
{}

const std::int32_t setVideoChatDefaultParticipant::ID;

object_ptr<setVideoChatDefaultParticipant> setVideoChatDefaultParticipant::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setVideoChatDefaultParticipant> res = make_object<setVideoChatDefaultParticipant>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->default_participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->default_participant_id_fieldID));
  return res;
}

void setVideoChatDefaultParticipant::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetVideoChatDefaultParticipant");
    s.store_field("chatId", chat_id_);
    s.store_object_field("defaultParticipantId", static_cast<const BaseObject *>(default_participant_id_.get()));
    s.store_class_end();
  }
}

setVideoChatDefaultParticipant::ReturnType setVideoChatDefaultParticipant::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setVideoChatDefaultParticipant::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetVideoChatDefaultParticipant").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    default_participant_id_fieldID = jni::get_field_id(env, Class, "defaultParticipantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass setVideoChatTitle::Class;
jfieldID setVideoChatTitle::group_call_id_fieldID;
jfieldID setVideoChatTitle::title_fieldID;

setVideoChatTitle::setVideoChatTitle()
  : group_call_id_()
  , title_()
{}

setVideoChatTitle::setVideoChatTitle(int32 group_call_id_, string const &title_)
  : group_call_id_(group_call_id_)
  , title_(title_)
{}

const std::int32_t setVideoChatTitle::ID;

object_ptr<setVideoChatTitle> setVideoChatTitle::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<setVideoChatTitle> res = make_object<setVideoChatTitle>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  return res;
}

void setVideoChatTitle::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SetVideoChatTitle");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("title", title_);
    s.store_class_end();
  }
}

setVideoChatTitle::ReturnType setVideoChatTitle::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void setVideoChatTitle::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SetVideoChatTitle").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass shareChatWithBot::Class;
jfieldID shareChatWithBot::chat_id_fieldID;
jfieldID shareChatWithBot::message_id_fieldID;
jfieldID shareChatWithBot::button_id_fieldID;
jfieldID shareChatWithBot::shared_chat_id_fieldID;
jfieldID shareChatWithBot::only_check_fieldID;

shareChatWithBot::shareChatWithBot()
  : chat_id_()
  , message_id_()
  , button_id_()
  , shared_chat_id_()
  , only_check_()
{}

shareChatWithBot::shareChatWithBot(int53 chat_id_, int53 message_id_, int32 button_id_, int53 shared_chat_id_, bool only_check_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , button_id_(button_id_)
  , shared_chat_id_(shared_chat_id_)
  , only_check_(only_check_)
{}

const std::int32_t shareChatWithBot::ID;

object_ptr<shareChatWithBot> shareChatWithBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<shareChatWithBot> res = make_object<shareChatWithBot>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->button_id_ = env->GetIntField(p, res->button_id_fieldID);
  res->shared_chat_id_ = env->GetLongField(p, res->shared_chat_id_fieldID);
  res->only_check_ = (env->GetBooleanField(p, res->only_check_fieldID) != 0);
  return res;
}

void shareChatWithBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ShareChatWithBot");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("buttonId", button_id_);
    s.store_field("sharedChatId", shared_chat_id_);
    s.store_field("onlyCheck", only_check_);
    s.store_class_end();
  }
}

shareChatWithBot::ReturnType shareChatWithBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void shareChatWithBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ShareChatWithBot").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "I");
    shared_chat_id_fieldID = jni::get_field_id(env, Class, "sharedChatId", "J");
    only_check_fieldID = jni::get_field_id(env, Class, "onlyCheck", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass sharePhoneNumber::Class;
jfieldID sharePhoneNumber::user_id_fieldID;

sharePhoneNumber::sharePhoneNumber()
  : user_id_()
{}

sharePhoneNumber::sharePhoneNumber(int53 user_id_)
  : user_id_(user_id_)
{}

const std::int32_t sharePhoneNumber::ID;

object_ptr<sharePhoneNumber> sharePhoneNumber::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<sharePhoneNumber> res = make_object<sharePhoneNumber>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  return res;
}

void sharePhoneNumber::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SharePhoneNumber");
    s.store_field("userId", user_id_);
    s.store_class_end();
  }
}

sharePhoneNumber::ReturnType sharePhoneNumber::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void sharePhoneNumber::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SharePhoneNumber").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass shareUsersWithBot::Class;
jfieldID shareUsersWithBot::chat_id_fieldID;
jfieldID shareUsersWithBot::message_id_fieldID;
jfieldID shareUsersWithBot::button_id_fieldID;
jfieldID shareUsersWithBot::shared_user_ids_fieldID;
jfieldID shareUsersWithBot::only_check_fieldID;

shareUsersWithBot::shareUsersWithBot()
  : chat_id_()
  , message_id_()
  , button_id_()
  , shared_user_ids_()
  , only_check_()
{}

shareUsersWithBot::shareUsersWithBot(int53 chat_id_, int53 message_id_, int32 button_id_, array<int53> &&shared_user_ids_, bool only_check_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , button_id_(button_id_)
  , shared_user_ids_(std::move(shared_user_ids_))
  , only_check_(only_check_)
{}

const std::int32_t shareUsersWithBot::ID;

object_ptr<shareUsersWithBot> shareUsersWithBot::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<shareUsersWithBot> res = make_object<shareUsersWithBot>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->button_id_ = env->GetIntField(p, res->button_id_fieldID);
  res->shared_user_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->shared_user_ids_fieldID));
  res->only_check_ = (env->GetBooleanField(p, res->only_check_fieldID) != 0);
  return res;
}

void shareUsersWithBot::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ShareUsersWithBot");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("buttonId", button_id_);
    { s.store_vector_begin("sharedUserIds", shared_user_ids_.size()); for (const auto &_value : shared_user_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_field("onlyCheck", only_check_);
    s.store_class_end();
  }
}

shareUsersWithBot::ReturnType shareUsersWithBot::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void shareUsersWithBot::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ShareUsersWithBot").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    button_id_fieldID = jni::get_field_id(env, Class, "buttonId", "I");
    shared_user_ids_fieldID = jni::get_field_id(env, Class, "sharedUserIds", "[J");
    only_check_fieldID = jni::get_field_id(env, Class, "onlyCheck", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass startGroupCallRecording::Class;
jfieldID startGroupCallRecording::group_call_id_fieldID;
jfieldID startGroupCallRecording::title_fieldID;
jfieldID startGroupCallRecording::record_video_fieldID;
jfieldID startGroupCallRecording::use_portrait_orientation_fieldID;

startGroupCallRecording::startGroupCallRecording()
  : group_call_id_()
  , title_()
  , record_video_()
  , use_portrait_orientation_()
{}

startGroupCallRecording::startGroupCallRecording(int32 group_call_id_, string const &title_, bool record_video_, bool use_portrait_orientation_)
  : group_call_id_(group_call_id_)
  , title_(title_)
  , record_video_(record_video_)
  , use_portrait_orientation_(use_portrait_orientation_)
{}

const std::int32_t startGroupCallRecording::ID;

object_ptr<startGroupCallRecording> startGroupCallRecording::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<startGroupCallRecording> res = make_object<startGroupCallRecording>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->title_ = jni::fetch_string(env, p, res->title_fieldID);
  res->record_video_ = (env->GetBooleanField(p, res->record_video_fieldID) != 0);
  res->use_portrait_orientation_ = (env->GetBooleanField(p, res->use_portrait_orientation_fieldID) != 0);
  return res;
}

void startGroupCallRecording::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StartGroupCallRecording");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("title", title_);
    s.store_field("recordVideo", record_video_);
    s.store_field("usePortraitOrientation", use_portrait_orientation_);
    s.store_class_end();
  }
}

startGroupCallRecording::ReturnType startGroupCallRecording::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void startGroupCallRecording::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StartGroupCallRecording").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    title_fieldID = jni::get_field_id(env, Class, "title", "Ljava/lang/String;");
    record_video_fieldID = jni::get_field_id(env, Class, "recordVideo", "Z");
    use_portrait_orientation_fieldID = jni::get_field_id(env, Class, "usePortraitOrientation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass startGroupCallScreenSharing::Class;
jfieldID startGroupCallScreenSharing::group_call_id_fieldID;
jfieldID startGroupCallScreenSharing::audio_source_id_fieldID;
jfieldID startGroupCallScreenSharing::payload_fieldID;

startGroupCallScreenSharing::startGroupCallScreenSharing()
  : group_call_id_()
  , audio_source_id_()
  , payload_()
{}

startGroupCallScreenSharing::startGroupCallScreenSharing(int32 group_call_id_, int32 audio_source_id_, string const &payload_)
  : group_call_id_(group_call_id_)
  , audio_source_id_(audio_source_id_)
  , payload_(payload_)
{}

const std::int32_t startGroupCallScreenSharing::ID;

object_ptr<startGroupCallScreenSharing> startGroupCallScreenSharing::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<startGroupCallScreenSharing> res = make_object<startGroupCallScreenSharing>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->audio_source_id_ = env->GetIntField(p, res->audio_source_id_fieldID);
  res->payload_ = jni::fetch_string(env, p, res->payload_fieldID);
  return res;
}

void startGroupCallScreenSharing::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StartGroupCallScreenSharing");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("audioSourceId", audio_source_id_);
    s.store_field("payload", payload_);
    s.store_class_end();
  }
}

startGroupCallScreenSharing::ReturnType startGroupCallScreenSharing::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return text::fetch(env, p);
}

void startGroupCallScreenSharing::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StartGroupCallScreenSharing").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    audio_source_id_fieldID = jni::get_field_id(env, Class, "audioSourceId", "I");
    payload_fieldID = jni::get_field_id(env, Class, "payload", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass startScheduledVideoChat::Class;
jfieldID startScheduledVideoChat::group_call_id_fieldID;

startScheduledVideoChat::startScheduledVideoChat()
  : group_call_id_()
{}

startScheduledVideoChat::startScheduledVideoChat(int32 group_call_id_)
  : group_call_id_(group_call_id_)
{}

const std::int32_t startScheduledVideoChat::ID;

object_ptr<startScheduledVideoChat> startScheduledVideoChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<startScheduledVideoChat> res = make_object<startScheduledVideoChat>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  return res;
}

void startScheduledVideoChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StartScheduledVideoChat");
    s.store_field("groupCallId", group_call_id_);
    s.store_class_end();
  }
}

startScheduledVideoChat::ReturnType startScheduledVideoChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void startScheduledVideoChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StartScheduledVideoChat").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    return true;
  }();
  (void)is_inited;
}

jclass stopBusinessPoll::Class;
jfieldID stopBusinessPoll::business_connection_id_fieldID;
jfieldID stopBusinessPoll::chat_id_fieldID;
jfieldID stopBusinessPoll::message_id_fieldID;
jfieldID stopBusinessPoll::reply_markup_fieldID;

stopBusinessPoll::stopBusinessPoll()
  : business_connection_id_()
  , chat_id_()
  , message_id_()
  , reply_markup_()
{}

stopBusinessPoll::stopBusinessPoll(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_)
  : business_connection_id_(business_connection_id_)
  , chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t stopBusinessPoll::ID;

object_ptr<stopBusinessPoll> stopBusinessPoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stopBusinessPoll> res = make_object<stopBusinessPoll>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void stopBusinessPoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StopBusinessPoll");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

stopBusinessPoll::ReturnType stopBusinessPoll::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return businessMessage::fetch(env, p);
}

void stopBusinessPoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StopBusinessPoll").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass stopPoll::Class;
jfieldID stopPoll::chat_id_fieldID;
jfieldID stopPoll::message_id_fieldID;
jfieldID stopPoll::reply_markup_fieldID;

stopPoll::stopPoll()
  : chat_id_()
  , message_id_()
  , reply_markup_()
{}

stopPoll::stopPoll(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , reply_markup_(std::move(reply_markup_))
{}

const std::int32_t stopPoll::ID;

object_ptr<stopPoll> stopPoll::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<stopPoll> res = make_object<stopPoll>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->reply_markup_ = jni::fetch_tl_object<ReplyMarkup>(env, jni::fetch_object(env, p, res->reply_markup_fieldID));
  return res;
}

void stopPoll::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "StopPoll");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_object_field("replyMarkup", static_cast<const BaseObject *>(reply_markup_.get()));
    s.store_class_end();
  }
}

stopPoll::ReturnType stopPoll::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void stopPoll::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$StopPoll").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    reply_markup_fieldID = jni::get_field_id(env, Class, "replyMarkup", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ReplyMarkup;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass suggestUserProfilePhoto::Class;
jfieldID suggestUserProfilePhoto::user_id_fieldID;
jfieldID suggestUserProfilePhoto::photo_fieldID;

suggestUserProfilePhoto::suggestUserProfilePhoto()
  : user_id_()
  , photo_()
{}

suggestUserProfilePhoto::suggestUserProfilePhoto(int53 user_id_, object_ptr<InputChatPhoto> &&photo_)
  : user_id_(user_id_)
  , photo_(std::move(photo_))
{}

const std::int32_t suggestUserProfilePhoto::ID;

object_ptr<suggestUserProfilePhoto> suggestUserProfilePhoto::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<suggestUserProfilePhoto> res = make_object<suggestUserProfilePhoto>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->photo_ = jni::fetch_tl_object<InputChatPhoto>(env, jni::fetch_object(env, p, res->photo_fieldID));
  return res;
}

void suggestUserProfilePhoto::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SuggestUserProfilePhoto");
    s.store_field("userId", user_id_);
    s.store_object_field("photo", static_cast<const BaseObject *>(photo_.get()));
    s.store_class_end();
  }
}

suggestUserProfilePhoto::ReturnType suggestUserProfilePhoto::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void suggestUserProfilePhoto::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SuggestUserProfilePhoto").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    photo_fieldID = jni::get_field_id(env, Class, "photo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputChatPhoto;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass synchronizeLanguagePack::Class;
jfieldID synchronizeLanguagePack::language_pack_id_fieldID;

synchronizeLanguagePack::synchronizeLanguagePack()
  : language_pack_id_()
{}

synchronizeLanguagePack::synchronizeLanguagePack(string const &language_pack_id_)
  : language_pack_id_(language_pack_id_)
{}

const std::int32_t synchronizeLanguagePack::ID;

object_ptr<synchronizeLanguagePack> synchronizeLanguagePack::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<synchronizeLanguagePack> res = make_object<synchronizeLanguagePack>();
  res->language_pack_id_ = jni::fetch_string(env, p, res->language_pack_id_fieldID);
  return res;
}

void synchronizeLanguagePack::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "SynchronizeLanguagePack");
    s.store_field("languagePackId", language_pack_id_);
    s.store_class_end();
  }
}

synchronizeLanguagePack::ReturnType synchronizeLanguagePack::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void synchronizeLanguagePack::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$SynchronizeLanguagePack").c_str());
    language_pack_id_fieldID = jni::get_field_id(env, Class, "languagePackId", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass terminateAllOtherSessions::Class;

terminateAllOtherSessions::terminateAllOtherSessions() {
}

const std::int32_t terminateAllOtherSessions::ID;

object_ptr<terminateAllOtherSessions> terminateAllOtherSessions::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<terminateAllOtherSessions>();
}

void terminateAllOtherSessions::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TerminateAllOtherSessions");
    s.store_class_end();
  }
}

terminateAllOtherSessions::ReturnType terminateAllOtherSessions::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void terminateAllOtherSessions::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TerminateAllOtherSessions").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass terminateSession::Class;
jfieldID terminateSession::session_id_fieldID;

terminateSession::terminateSession()
  : session_id_()
{}

terminateSession::terminateSession(int64 session_id_)
  : session_id_(session_id_)
{}

const std::int32_t terminateSession::ID;

object_ptr<terminateSession> terminateSession::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<terminateSession> res = make_object<terminateSession>();
  res->session_id_ = env->GetLongField(p, res->session_id_fieldID);
  return res;
}

void terminateSession::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TerminateSession");
    s.store_field("sessionId", session_id_);
    s.store_class_end();
  }
}

terminateSession::ReturnType terminateSession::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void terminateSession::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TerminateSession").c_str());
    session_id_fieldID = jni::get_field_id(env, Class, "sessionId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass testCallBytes::Class;
jfieldID testCallBytes::x_fieldID;

testCallBytes::testCallBytes()
  : x_()
{}

testCallBytes::testCallBytes(bytes const &x_)
  : x_(std::move(x_))
{}

const std::int32_t testCallBytes::ID;

object_ptr<testCallBytes> testCallBytes::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallBytes> res = make_object<testCallBytes>();
  res->x_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->x_fieldID));
  return res;
}

void testCallBytes::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallBytes");
    s.store_bytes_field("x", x_);
    s.store_class_end();
  }
}

testCallBytes::ReturnType testCallBytes::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testBytes::fetch(env, p);
}

void testCallBytes::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallBytes").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "[B");
    return true;
  }();
  (void)is_inited;
}

jclass testCallEmpty::Class;

testCallEmpty::testCallEmpty() {
}

const std::int32_t testCallEmpty::ID;

object_ptr<testCallEmpty> testCallEmpty::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<testCallEmpty>();
}

void testCallEmpty::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallEmpty");
    s.store_class_end();
  }
}

testCallEmpty::ReturnType testCallEmpty::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void testCallEmpty::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallEmpty").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testCallString::Class;
jfieldID testCallString::x_fieldID;

testCallString::testCallString()
  : x_()
{}

testCallString::testCallString(string const &x_)
  : x_(x_)
{}

const std::int32_t testCallString::ID;

object_ptr<testCallString> testCallString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallString> res = make_object<testCallString>();
  res->x_ = jni::fetch_string(env, p, res->x_fieldID);
  return res;
}

void testCallString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallString");
    s.store_field("x", x_);
    s.store_class_end();
  }
}

testCallString::ReturnType testCallString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testString::fetch(env, p);
}

void testCallString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallString").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass testCallVectorInt::Class;
jfieldID testCallVectorInt::x_fieldID;

testCallVectorInt::testCallVectorInt()
  : x_()
{}

testCallVectorInt::testCallVectorInt(array<int32> &&x_)
  : x_(std::move(x_))
{}

const std::int32_t testCallVectorInt::ID;

object_ptr<testCallVectorInt> testCallVectorInt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallVectorInt> res = make_object<testCallVectorInt>();
  res->x_ = jni::fetch_vector(env, (jintArray)jni::fetch_object(env, p, res->x_fieldID));
  return res;
}

void testCallVectorInt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallVectorInt");
    { s.store_vector_begin("x", x_.size()); for (const auto &_value : x_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

testCallVectorInt::ReturnType testCallVectorInt::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testVectorInt::fetch(env, p);
}

void testCallVectorInt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallVectorInt").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "[I");
    return true;
  }();
  (void)is_inited;
}

jclass testCallVectorIntObject::Class;
jfieldID testCallVectorIntObject::x_fieldID;

testCallVectorIntObject::testCallVectorIntObject()
  : x_()
{}

testCallVectorIntObject::testCallVectorIntObject(array<object_ptr<testInt>> &&x_)
  : x_(std::move(x_))
{}

const std::int32_t testCallVectorIntObject::ID;

object_ptr<testCallVectorIntObject> testCallVectorIntObject::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallVectorIntObject> res = make_object<testCallVectorIntObject>();
  res->x_ = jni::FetchVector<testInt>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->x_fieldID));
  return res;
}

void testCallVectorIntObject::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallVectorIntObject");
    { s.store_vector_begin("x", x_.size()); for (const auto &_value : x_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

testCallVectorIntObject::ReturnType testCallVectorIntObject::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testVectorIntObject::fetch(env, p);
}

void testCallVectorIntObject::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallVectorIntObject").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TestInt;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testCallVectorString::Class;
jfieldID testCallVectorString::x_fieldID;

testCallVectorString::testCallVectorString()
  : x_()
{}

testCallVectorString::testCallVectorString(array<string> &&x_)
  : x_(std::move(x_))
{}

const std::int32_t testCallVectorString::ID;

object_ptr<testCallVectorString> testCallVectorString::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallVectorString> res = make_object<testCallVectorString>();
  res->x_ = jni::FetchVector<string>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->x_fieldID));
  return res;
}

void testCallVectorString::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallVectorString");
    { s.store_vector_begin("x", x_.size()); for (const auto &_value : x_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

testCallVectorString::ReturnType testCallVectorString::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testVectorString::fetch(env, p);
}

void testCallVectorString::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallVectorString").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "[Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass testCallVectorStringObject::Class;
jfieldID testCallVectorStringObject::x_fieldID;

testCallVectorStringObject::testCallVectorStringObject()
  : x_()
{}

testCallVectorStringObject::testCallVectorStringObject(array<object_ptr<testString>> &&x_)
  : x_(std::move(x_))
{}

const std::int32_t testCallVectorStringObject::ID;

object_ptr<testCallVectorStringObject> testCallVectorStringObject::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testCallVectorStringObject> res = make_object<testCallVectorStringObject>();
  res->x_ = jni::FetchVector<testString>::fetch(env, (jobjectArray)jni::fetch_object(env, p, res->x_fieldID));
  return res;
}

void testCallVectorStringObject::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestCallVectorStringObject");
    { s.store_vector_begin("x", x_.size()); for (const auto &_value : x_) { s.store_object_field("", static_cast<const BaseObject *>(_value.get())); } s.store_class_end(); }
    s.store_class_end();
  }
}

testCallVectorStringObject::ReturnType testCallVectorStringObject::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testVectorStringObject::fetch(env, p);
}

void testCallVectorStringObject::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestCallVectorStringObject").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", (PSLICE() << "[L" << get_package_name_ref() << "/TdApi$TestString;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testGetDifference::Class;

testGetDifference::testGetDifference() {
}

const std::int32_t testGetDifference::ID;

object_ptr<testGetDifference> testGetDifference::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<testGetDifference>();
}

void testGetDifference::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestGetDifference");
    s.store_class_end();
  }
}

testGetDifference::ReturnType testGetDifference::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void testGetDifference::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestGetDifference").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testNetwork::Class;

testNetwork::testNetwork() {
}

const std::int32_t testNetwork::ID;

object_ptr<testNetwork> testNetwork::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<testNetwork>();
}

void testNetwork::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestNetwork");
    s.store_class_end();
  }
}

testNetwork::ReturnType testNetwork::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void testNetwork::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestNetwork").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testProxy::Class;
jfieldID testProxy::server_fieldID;
jfieldID testProxy::port_fieldID;
jfieldID testProxy::type_fieldID;
jfieldID testProxy::dc_id_fieldID;
jfieldID testProxy::timeout_fieldID;

testProxy::testProxy()
  : server_()
  , port_()
  , type_()
  , dc_id_()
  , timeout_()
{}

testProxy::testProxy(string const &server_, int32 port_, object_ptr<ProxyType> &&type_, int32 dc_id_, double timeout_)
  : server_(server_)
  , port_(port_)
  , type_(std::move(type_))
  , dc_id_(dc_id_)
  , timeout_(timeout_)
{}

const std::int32_t testProxy::ID;

object_ptr<testProxy> testProxy::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testProxy> res = make_object<testProxy>();
  res->server_ = jni::fetch_string(env, p, res->server_fieldID);
  res->port_ = env->GetIntField(p, res->port_fieldID);
  res->type_ = jni::fetch_tl_object<ProxyType>(env, jni::fetch_object(env, p, res->type_fieldID));
  res->dc_id_ = env->GetIntField(p, res->dc_id_fieldID);
  res->timeout_ = env->GetDoubleField(p, res->timeout_fieldID);
  return res;
}

void testProxy::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestProxy");
    s.store_field("server", server_);
    s.store_field("port", port_);
    s.store_object_field("type", static_cast<const BaseObject *>(type_.get()));
    s.store_field("dcId", dc_id_);
    s.store_field("timeout", timeout_);
    s.store_class_end();
  }
}

testProxy::ReturnType testProxy::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void testProxy::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestProxy").c_str());
    server_fieldID = jni::get_field_id(env, Class, "server", "Ljava/lang/String;");
    port_fieldID = jni::get_field_id(env, Class, "port", "I");
    type_fieldID = jni::get_field_id(env, Class, "type", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ProxyType;").c_str());
    dc_id_fieldID = jni::get_field_id(env, Class, "dcId", "I");
    timeout_fieldID = jni::get_field_id(env, Class, "timeout", "D");
    return true;
  }();
  (void)is_inited;
}

jclass testReturnError::Class;
jfieldID testReturnError::error_fieldID;

testReturnError::testReturnError()
  : error_()
{}

testReturnError::testReturnError(object_ptr<error> &&error_)
  : error_(std::move(error_))
{}

const std::int32_t testReturnError::ID;

object_ptr<testReturnError> testReturnError::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testReturnError> res = make_object<testReturnError>();
  res->error_ = jni::fetch_tl_object<error>(env, jni::fetch_object(env, p, res->error_fieldID));
  return res;
}

void testReturnError::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestReturnError");
    s.store_object_field("error", static_cast<const BaseObject *>(error_.get()));
    s.store_class_end();
  }
}

testReturnError::ReturnType testReturnError::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return error::fetch(env, p);
}

void testReturnError::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestReturnError").c_str());
    error_fieldID = jni::get_field_id(env, Class, "error", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$Error;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass testSquareInt::Class;
jfieldID testSquareInt::x_fieldID;

testSquareInt::testSquareInt()
  : x_()
{}

testSquareInt::testSquareInt(int32 x_)
  : x_(x_)
{}

const std::int32_t testSquareInt::ID;

object_ptr<testSquareInt> testSquareInt::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<testSquareInt> res = make_object<testSquareInt>();
  res->x_ = env->GetIntField(p, res->x_fieldID);
  return res;
}

void testSquareInt::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestSquareInt");
    s.store_field("x", x_);
    s.store_class_end();
  }
}

testSquareInt::ReturnType testSquareInt::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return testInt::fetch(env, p);
}

void testSquareInt::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestSquareInt").c_str());
    x_fieldID = jni::get_field_id(env, Class, "x", "I");
    return true;
  }();
  (void)is_inited;
}

jclass testUseUpdate::Class;

testUseUpdate::testUseUpdate() {
}

const std::int32_t testUseUpdate::ID;

object_ptr<testUseUpdate> testUseUpdate::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  return make_object<testUseUpdate>();
}

void testUseUpdate::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TestUseUpdate");
    s.store_class_end();
  }
}

testUseUpdate::ReturnType testUseUpdate::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return Update::fetch(env, p);
}

void testUseUpdate::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TestUseUpdate").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass toggleAllDownloadsArePaused::Class;
jfieldID toggleAllDownloadsArePaused::are_paused_fieldID;

toggleAllDownloadsArePaused::toggleAllDownloadsArePaused()
  : are_paused_()
{}

toggleAllDownloadsArePaused::toggleAllDownloadsArePaused(bool are_paused_)
  : are_paused_(are_paused_)
{}

const std::int32_t toggleAllDownloadsArePaused::ID;

object_ptr<toggleAllDownloadsArePaused> toggleAllDownloadsArePaused::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleAllDownloadsArePaused> res = make_object<toggleAllDownloadsArePaused>();
  res->are_paused_ = (env->GetBooleanField(p, res->are_paused_fieldID) != 0);
  return res;
}

void toggleAllDownloadsArePaused::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleAllDownloadsArePaused");
    s.store_field("arePaused", are_paused_);
    s.store_class_end();
  }
}

toggleAllDownloadsArePaused::ReturnType toggleAllDownloadsArePaused::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleAllDownloadsArePaused::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleAllDownloadsArePaused").c_str());
    are_paused_fieldID = jni::get_field_id(env, Class, "arePaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleBotCanManageEmojiStatus::Class;
jfieldID toggleBotCanManageEmojiStatus::bot_user_id_fieldID;
jfieldID toggleBotCanManageEmojiStatus::can_manage_emoji_status_fieldID;

toggleBotCanManageEmojiStatus::toggleBotCanManageEmojiStatus()
  : bot_user_id_()
  , can_manage_emoji_status_()
{}

toggleBotCanManageEmojiStatus::toggleBotCanManageEmojiStatus(int53 bot_user_id_, bool can_manage_emoji_status_)
  : bot_user_id_(bot_user_id_)
  , can_manage_emoji_status_(can_manage_emoji_status_)
{}

const std::int32_t toggleBotCanManageEmojiStatus::ID;

object_ptr<toggleBotCanManageEmojiStatus> toggleBotCanManageEmojiStatus::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleBotCanManageEmojiStatus> res = make_object<toggleBotCanManageEmojiStatus>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->can_manage_emoji_status_ = (env->GetBooleanField(p, res->can_manage_emoji_status_fieldID) != 0);
  return res;
}

void toggleBotCanManageEmojiStatus::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleBotCanManageEmojiStatus");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("canManageEmojiStatus", can_manage_emoji_status_);
    s.store_class_end();
  }
}

toggleBotCanManageEmojiStatus::ReturnType toggleBotCanManageEmojiStatus::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleBotCanManageEmojiStatus::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleBotCanManageEmojiStatus").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    can_manage_emoji_status_fieldID = jni::get_field_id(env, Class, "canManageEmojiStatus", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleBotIsAddedToAttachmentMenu::Class;
jfieldID toggleBotIsAddedToAttachmentMenu::bot_user_id_fieldID;
jfieldID toggleBotIsAddedToAttachmentMenu::is_added_fieldID;
jfieldID toggleBotIsAddedToAttachmentMenu::allow_write_access_fieldID;

toggleBotIsAddedToAttachmentMenu::toggleBotIsAddedToAttachmentMenu()
  : bot_user_id_()
  , is_added_()
  , allow_write_access_()
{}

toggleBotIsAddedToAttachmentMenu::toggleBotIsAddedToAttachmentMenu(int53 bot_user_id_, bool is_added_, bool allow_write_access_)
  : bot_user_id_(bot_user_id_)
  , is_added_(is_added_)
  , allow_write_access_(allow_write_access_)
{}

const std::int32_t toggleBotIsAddedToAttachmentMenu::ID;

object_ptr<toggleBotIsAddedToAttachmentMenu> toggleBotIsAddedToAttachmentMenu::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleBotIsAddedToAttachmentMenu> res = make_object<toggleBotIsAddedToAttachmentMenu>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->is_added_ = (env->GetBooleanField(p, res->is_added_fieldID) != 0);
  res->allow_write_access_ = (env->GetBooleanField(p, res->allow_write_access_fieldID) != 0);
  return res;
}

void toggleBotIsAddedToAttachmentMenu::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleBotIsAddedToAttachmentMenu");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("isAdded", is_added_);
    s.store_field("allowWriteAccess", allow_write_access_);
    s.store_class_end();
  }
}

toggleBotIsAddedToAttachmentMenu::ReturnType toggleBotIsAddedToAttachmentMenu::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleBotIsAddedToAttachmentMenu::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleBotIsAddedToAttachmentMenu").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    is_added_fieldID = jni::get_field_id(env, Class, "isAdded", "Z");
    allow_write_access_fieldID = jni::get_field_id(env, Class, "allowWriteAccess", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleBotUsernameIsActive::Class;
jfieldID toggleBotUsernameIsActive::bot_user_id_fieldID;
jfieldID toggleBotUsernameIsActive::username_fieldID;
jfieldID toggleBotUsernameIsActive::is_active_fieldID;

toggleBotUsernameIsActive::toggleBotUsernameIsActive()
  : bot_user_id_()
  , username_()
  , is_active_()
{}

toggleBotUsernameIsActive::toggleBotUsernameIsActive(int53 bot_user_id_, string const &username_, bool is_active_)
  : bot_user_id_(bot_user_id_)
  , username_(username_)
  , is_active_(is_active_)
{}

const std::int32_t toggleBotUsernameIsActive::ID;

object_ptr<toggleBotUsernameIsActive> toggleBotUsernameIsActive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleBotUsernameIsActive> res = make_object<toggleBotUsernameIsActive>();
  res->bot_user_id_ = env->GetLongField(p, res->bot_user_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  return res;
}

void toggleBotUsernameIsActive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleBotUsernameIsActive");
    s.store_field("botUserId", bot_user_id_);
    s.store_field("username", username_);
    s.store_field("isActive", is_active_);
    s.store_class_end();
  }
}

toggleBotUsernameIsActive::ReturnType toggleBotUsernameIsActive::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleBotUsernameIsActive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleBotUsernameIsActive").c_str());
    bot_user_id_fieldID = jni::get_field_id(env, Class, "botUserId", "J");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleBusinessConnectedBotChatIsPaused::Class;
jfieldID toggleBusinessConnectedBotChatIsPaused::chat_id_fieldID;
jfieldID toggleBusinessConnectedBotChatIsPaused::is_paused_fieldID;

toggleBusinessConnectedBotChatIsPaused::toggleBusinessConnectedBotChatIsPaused()
  : chat_id_()
  , is_paused_()
{}

toggleBusinessConnectedBotChatIsPaused::toggleBusinessConnectedBotChatIsPaused(int53 chat_id_, bool is_paused_)
  : chat_id_(chat_id_)
  , is_paused_(is_paused_)
{}

const std::int32_t toggleBusinessConnectedBotChatIsPaused::ID;

object_ptr<toggleBusinessConnectedBotChatIsPaused> toggleBusinessConnectedBotChatIsPaused::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleBusinessConnectedBotChatIsPaused> res = make_object<toggleBusinessConnectedBotChatIsPaused>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  return res;
}

void toggleBusinessConnectedBotChatIsPaused::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleBusinessConnectedBotChatIsPaused");
    s.store_field("chatId", chat_id_);
    s.store_field("isPaused", is_paused_);
    s.store_class_end();
  }
}

toggleBusinessConnectedBotChatIsPaused::ReturnType toggleBusinessConnectedBotChatIsPaused::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleBusinessConnectedBotChatIsPaused::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleBusinessConnectedBotChatIsPaused").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatDefaultDisableNotification::Class;
jfieldID toggleChatDefaultDisableNotification::chat_id_fieldID;
jfieldID toggleChatDefaultDisableNotification::default_disable_notification_fieldID;

toggleChatDefaultDisableNotification::toggleChatDefaultDisableNotification()
  : chat_id_()
  , default_disable_notification_()
{}

toggleChatDefaultDisableNotification::toggleChatDefaultDisableNotification(int53 chat_id_, bool default_disable_notification_)
  : chat_id_(chat_id_)
  , default_disable_notification_(default_disable_notification_)
{}

const std::int32_t toggleChatDefaultDisableNotification::ID;

object_ptr<toggleChatDefaultDisableNotification> toggleChatDefaultDisableNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatDefaultDisableNotification> res = make_object<toggleChatDefaultDisableNotification>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->default_disable_notification_ = (env->GetBooleanField(p, res->default_disable_notification_fieldID) != 0);
  return res;
}

void toggleChatDefaultDisableNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatDefaultDisableNotification");
    s.store_field("chatId", chat_id_);
    s.store_field("defaultDisableNotification", default_disable_notification_);
    s.store_class_end();
  }
}

toggleChatDefaultDisableNotification::ReturnType toggleChatDefaultDisableNotification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatDefaultDisableNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatDefaultDisableNotification").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    default_disable_notification_fieldID = jni::get_field_id(env, Class, "defaultDisableNotification", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatFolderTags::Class;
jfieldID toggleChatFolderTags::are_tags_enabled_fieldID;

toggleChatFolderTags::toggleChatFolderTags()
  : are_tags_enabled_()
{}

toggleChatFolderTags::toggleChatFolderTags(bool are_tags_enabled_)
  : are_tags_enabled_(are_tags_enabled_)
{}

const std::int32_t toggleChatFolderTags::ID;

object_ptr<toggleChatFolderTags> toggleChatFolderTags::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatFolderTags> res = make_object<toggleChatFolderTags>();
  res->are_tags_enabled_ = (env->GetBooleanField(p, res->are_tags_enabled_fieldID) != 0);
  return res;
}

void toggleChatFolderTags::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatFolderTags");
    s.store_field("areTagsEnabled", are_tags_enabled_);
    s.store_class_end();
  }
}

toggleChatFolderTags::ReturnType toggleChatFolderTags::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatFolderTags::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatFolderTags").c_str());
    are_tags_enabled_fieldID = jni::get_field_id(env, Class, "areTagsEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatGiftNotifications::Class;
jfieldID toggleChatGiftNotifications::chat_id_fieldID;
jfieldID toggleChatGiftNotifications::are_enabled_fieldID;

toggleChatGiftNotifications::toggleChatGiftNotifications()
  : chat_id_()
  , are_enabled_()
{}

toggleChatGiftNotifications::toggleChatGiftNotifications(int53 chat_id_, bool are_enabled_)
  : chat_id_(chat_id_)
  , are_enabled_(are_enabled_)
{}

const std::int32_t toggleChatGiftNotifications::ID;

object_ptr<toggleChatGiftNotifications> toggleChatGiftNotifications::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatGiftNotifications> res = make_object<toggleChatGiftNotifications>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->are_enabled_ = (env->GetBooleanField(p, res->are_enabled_fieldID) != 0);
  return res;
}

void toggleChatGiftNotifications::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatGiftNotifications");
    s.store_field("chatId", chat_id_);
    s.store_field("areEnabled", are_enabled_);
    s.store_class_end();
  }
}

toggleChatGiftNotifications::ReturnType toggleChatGiftNotifications::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatGiftNotifications::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatGiftNotifications").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    are_enabled_fieldID = jni::get_field_id(env, Class, "areEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatHasProtectedContent::Class;
jfieldID toggleChatHasProtectedContent::chat_id_fieldID;
jfieldID toggleChatHasProtectedContent::has_protected_content_fieldID;

toggleChatHasProtectedContent::toggleChatHasProtectedContent()
  : chat_id_()
  , has_protected_content_()
{}

toggleChatHasProtectedContent::toggleChatHasProtectedContent(int53 chat_id_, bool has_protected_content_)
  : chat_id_(chat_id_)
  , has_protected_content_(has_protected_content_)
{}

const std::int32_t toggleChatHasProtectedContent::ID;

object_ptr<toggleChatHasProtectedContent> toggleChatHasProtectedContent::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatHasProtectedContent> res = make_object<toggleChatHasProtectedContent>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->has_protected_content_ = (env->GetBooleanField(p, res->has_protected_content_fieldID) != 0);
  return res;
}

void toggleChatHasProtectedContent::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatHasProtectedContent");
    s.store_field("chatId", chat_id_);
    s.store_field("hasProtectedContent", has_protected_content_);
    s.store_class_end();
  }
}

toggleChatHasProtectedContent::ReturnType toggleChatHasProtectedContent::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatHasProtectedContent::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatHasProtectedContent").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    has_protected_content_fieldID = jni::get_field_id(env, Class, "hasProtectedContent", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatIsMarkedAsUnread::Class;
jfieldID toggleChatIsMarkedAsUnread::chat_id_fieldID;
jfieldID toggleChatIsMarkedAsUnread::is_marked_as_unread_fieldID;

toggleChatIsMarkedAsUnread::toggleChatIsMarkedAsUnread()
  : chat_id_()
  , is_marked_as_unread_()
{}

toggleChatIsMarkedAsUnread::toggleChatIsMarkedAsUnread(int53 chat_id_, bool is_marked_as_unread_)
  : chat_id_(chat_id_)
  , is_marked_as_unread_(is_marked_as_unread_)
{}

const std::int32_t toggleChatIsMarkedAsUnread::ID;

object_ptr<toggleChatIsMarkedAsUnread> toggleChatIsMarkedAsUnread::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatIsMarkedAsUnread> res = make_object<toggleChatIsMarkedAsUnread>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_marked_as_unread_ = (env->GetBooleanField(p, res->is_marked_as_unread_fieldID) != 0);
  return res;
}

void toggleChatIsMarkedAsUnread::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatIsMarkedAsUnread");
    s.store_field("chatId", chat_id_);
    s.store_field("isMarkedAsUnread", is_marked_as_unread_);
    s.store_class_end();
  }
}

toggleChatIsMarkedAsUnread::ReturnType toggleChatIsMarkedAsUnread::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatIsMarkedAsUnread::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatIsMarkedAsUnread").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_marked_as_unread_fieldID = jni::get_field_id(env, Class, "isMarkedAsUnread", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatIsPinned::Class;
jfieldID toggleChatIsPinned::chat_list_fieldID;
jfieldID toggleChatIsPinned::chat_id_fieldID;
jfieldID toggleChatIsPinned::is_pinned_fieldID;

toggleChatIsPinned::toggleChatIsPinned()
  : chat_list_()
  , chat_id_()
  , is_pinned_()
{}

toggleChatIsPinned::toggleChatIsPinned(object_ptr<ChatList> &&chat_list_, int53 chat_id_, bool is_pinned_)
  : chat_list_(std::move(chat_list_))
  , chat_id_(chat_id_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t toggleChatIsPinned::ID;

object_ptr<toggleChatIsPinned> toggleChatIsPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatIsPinned> res = make_object<toggleChatIsPinned>();
  res->chat_list_ = jni::fetch_tl_object<ChatList>(env, jni::fetch_object(env, p, res->chat_list_fieldID));
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return res;
}

void toggleChatIsPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatIsPinned");
    s.store_object_field("chatList", static_cast<const BaseObject *>(chat_list_.get()));
    s.store_field("chatId", chat_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

toggleChatIsPinned::ReturnType toggleChatIsPinned::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatIsPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatIsPinned").c_str());
    chat_list_fieldID = jni::get_field_id(env, Class, "chatList", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$ChatList;").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatIsTranslatable::Class;
jfieldID toggleChatIsTranslatable::chat_id_fieldID;
jfieldID toggleChatIsTranslatable::is_translatable_fieldID;

toggleChatIsTranslatable::toggleChatIsTranslatable()
  : chat_id_()
  , is_translatable_()
{}

toggleChatIsTranslatable::toggleChatIsTranslatable(int53 chat_id_, bool is_translatable_)
  : chat_id_(chat_id_)
  , is_translatable_(is_translatable_)
{}

const std::int32_t toggleChatIsTranslatable::ID;

object_ptr<toggleChatIsTranslatable> toggleChatIsTranslatable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatIsTranslatable> res = make_object<toggleChatIsTranslatable>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_translatable_ = (env->GetBooleanField(p, res->is_translatable_fieldID) != 0);
  return res;
}

void toggleChatIsTranslatable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatIsTranslatable");
    s.store_field("chatId", chat_id_);
    s.store_field("isTranslatable", is_translatable_);
    s.store_class_end();
  }
}

toggleChatIsTranslatable::ReturnType toggleChatIsTranslatable::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatIsTranslatable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatIsTranslatable").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_translatable_fieldID = jni::get_field_id(env, Class, "isTranslatable", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleChatViewAsTopics::Class;
jfieldID toggleChatViewAsTopics::chat_id_fieldID;
jfieldID toggleChatViewAsTopics::view_as_topics_fieldID;

toggleChatViewAsTopics::toggleChatViewAsTopics()
  : chat_id_()
  , view_as_topics_()
{}

toggleChatViewAsTopics::toggleChatViewAsTopics(int53 chat_id_, bool view_as_topics_)
  : chat_id_(chat_id_)
  , view_as_topics_(view_as_topics_)
{}

const std::int32_t toggleChatViewAsTopics::ID;

object_ptr<toggleChatViewAsTopics> toggleChatViewAsTopics::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleChatViewAsTopics> res = make_object<toggleChatViewAsTopics>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->view_as_topics_ = (env->GetBooleanField(p, res->view_as_topics_fieldID) != 0);
  return res;
}

void toggleChatViewAsTopics::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleChatViewAsTopics");
    s.store_field("chatId", chat_id_);
    s.store_field("viewAsTopics", view_as_topics_);
    s.store_class_end();
  }
}

toggleChatViewAsTopics::ReturnType toggleChatViewAsTopics::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleChatViewAsTopics::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleChatViewAsTopics").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    view_as_topics_fieldID = jni::get_field_id(env, Class, "viewAsTopics", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleDownloadIsPaused::Class;
jfieldID toggleDownloadIsPaused::file_id_fieldID;
jfieldID toggleDownloadIsPaused::is_paused_fieldID;

toggleDownloadIsPaused::toggleDownloadIsPaused()
  : file_id_()
  , is_paused_()
{}

toggleDownloadIsPaused::toggleDownloadIsPaused(int32 file_id_, bool is_paused_)
  : file_id_(file_id_)
  , is_paused_(is_paused_)
{}

const std::int32_t toggleDownloadIsPaused::ID;

object_ptr<toggleDownloadIsPaused> toggleDownloadIsPaused::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleDownloadIsPaused> res = make_object<toggleDownloadIsPaused>();
  res->file_id_ = env->GetIntField(p, res->file_id_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  return res;
}

void toggleDownloadIsPaused::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleDownloadIsPaused");
    s.store_field("fileId", file_id_);
    s.store_field("isPaused", is_paused_);
    s.store_class_end();
  }
}

toggleDownloadIsPaused::ReturnType toggleDownloadIsPaused::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleDownloadIsPaused::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleDownloadIsPaused").c_str());
    file_id_fieldID = jni::get_field_id(env, Class, "fileId", "I");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleForumTopicIsClosed::Class;
jfieldID toggleForumTopicIsClosed::chat_id_fieldID;
jfieldID toggleForumTopicIsClosed::message_thread_id_fieldID;
jfieldID toggleForumTopicIsClosed::is_closed_fieldID;

toggleForumTopicIsClosed::toggleForumTopicIsClosed()
  : chat_id_()
  , message_thread_id_()
  , is_closed_()
{}

toggleForumTopicIsClosed::toggleForumTopicIsClosed(int53 chat_id_, int53 message_thread_id_, bool is_closed_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , is_closed_(is_closed_)
{}

const std::int32_t toggleForumTopicIsClosed::ID;

object_ptr<toggleForumTopicIsClosed> toggleForumTopicIsClosed::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleForumTopicIsClosed> res = make_object<toggleForumTopicIsClosed>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->is_closed_ = (env->GetBooleanField(p, res->is_closed_fieldID) != 0);
  return res;
}

void toggleForumTopicIsClosed::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleForumTopicIsClosed");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("isClosed", is_closed_);
    s.store_class_end();
  }
}

toggleForumTopicIsClosed::ReturnType toggleForumTopicIsClosed::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleForumTopicIsClosed::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleForumTopicIsClosed").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    is_closed_fieldID = jni::get_field_id(env, Class, "isClosed", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleForumTopicIsPinned::Class;
jfieldID toggleForumTopicIsPinned::chat_id_fieldID;
jfieldID toggleForumTopicIsPinned::message_thread_id_fieldID;
jfieldID toggleForumTopicIsPinned::is_pinned_fieldID;

toggleForumTopicIsPinned::toggleForumTopicIsPinned()
  : chat_id_()
  , message_thread_id_()
  , is_pinned_()
{}

toggleForumTopicIsPinned::toggleForumTopicIsPinned(int53 chat_id_, int53 message_thread_id_, bool is_pinned_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t toggleForumTopicIsPinned::ID;

object_ptr<toggleForumTopicIsPinned> toggleForumTopicIsPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleForumTopicIsPinned> res = make_object<toggleForumTopicIsPinned>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return res;
}

void toggleForumTopicIsPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleForumTopicIsPinned");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

toggleForumTopicIsPinned::ReturnType toggleForumTopicIsPinned::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleForumTopicIsPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleForumTopicIsPinned").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGeneralForumTopicIsHidden::Class;
jfieldID toggleGeneralForumTopicIsHidden::chat_id_fieldID;
jfieldID toggleGeneralForumTopicIsHidden::is_hidden_fieldID;

toggleGeneralForumTopicIsHidden::toggleGeneralForumTopicIsHidden()
  : chat_id_()
  , is_hidden_()
{}

toggleGeneralForumTopicIsHidden::toggleGeneralForumTopicIsHidden(int53 chat_id_, bool is_hidden_)
  : chat_id_(chat_id_)
  , is_hidden_(is_hidden_)
{}

const std::int32_t toggleGeneralForumTopicIsHidden::ID;

object_ptr<toggleGeneralForumTopicIsHidden> toggleGeneralForumTopicIsHidden::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGeneralForumTopicIsHidden> res = make_object<toggleGeneralForumTopicIsHidden>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->is_hidden_ = (env->GetBooleanField(p, res->is_hidden_fieldID) != 0);
  return res;
}

void toggleGeneralForumTopicIsHidden::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGeneralForumTopicIsHidden");
    s.store_field("chatId", chat_id_);
    s.store_field("isHidden", is_hidden_);
    s.store_class_end();
  }
}

toggleGeneralForumTopicIsHidden::ReturnType toggleGeneralForumTopicIsHidden::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGeneralForumTopicIsHidden::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGeneralForumTopicIsHidden").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    is_hidden_fieldID = jni::get_field_id(env, Class, "isHidden", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGiftIsSaved::Class;
jfieldID toggleGiftIsSaved::received_gift_id_fieldID;
jfieldID toggleGiftIsSaved::is_saved_fieldID;

toggleGiftIsSaved::toggleGiftIsSaved()
  : received_gift_id_()
  , is_saved_()
{}

toggleGiftIsSaved::toggleGiftIsSaved(string const &received_gift_id_, bool is_saved_)
  : received_gift_id_(received_gift_id_)
  , is_saved_(is_saved_)
{}

const std::int32_t toggleGiftIsSaved::ID;

object_ptr<toggleGiftIsSaved> toggleGiftIsSaved::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGiftIsSaved> res = make_object<toggleGiftIsSaved>();
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->is_saved_ = (env->GetBooleanField(p, res->is_saved_fieldID) != 0);
  return res;
}

void toggleGiftIsSaved::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGiftIsSaved");
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("isSaved", is_saved_);
    s.store_class_end();
  }
}

toggleGiftIsSaved::ReturnType toggleGiftIsSaved::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGiftIsSaved::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGiftIsSaved").c_str());
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    is_saved_fieldID = jni::get_field_id(env, Class, "isSaved", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGroupCallIsMyVideoEnabled::Class;
jfieldID toggleGroupCallIsMyVideoEnabled::group_call_id_fieldID;
jfieldID toggleGroupCallIsMyVideoEnabled::is_my_video_enabled_fieldID;

toggleGroupCallIsMyVideoEnabled::toggleGroupCallIsMyVideoEnabled()
  : group_call_id_()
  , is_my_video_enabled_()
{}

toggleGroupCallIsMyVideoEnabled::toggleGroupCallIsMyVideoEnabled(int32 group_call_id_, bool is_my_video_enabled_)
  : group_call_id_(group_call_id_)
  , is_my_video_enabled_(is_my_video_enabled_)
{}

const std::int32_t toggleGroupCallIsMyVideoEnabled::ID;

object_ptr<toggleGroupCallIsMyVideoEnabled> toggleGroupCallIsMyVideoEnabled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGroupCallIsMyVideoEnabled> res = make_object<toggleGroupCallIsMyVideoEnabled>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->is_my_video_enabled_ = (env->GetBooleanField(p, res->is_my_video_enabled_fieldID) != 0);
  return res;
}

void toggleGroupCallIsMyVideoEnabled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGroupCallIsMyVideoEnabled");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("isMyVideoEnabled", is_my_video_enabled_);
    s.store_class_end();
  }
}

toggleGroupCallIsMyVideoEnabled::ReturnType toggleGroupCallIsMyVideoEnabled::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGroupCallIsMyVideoEnabled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGroupCallIsMyVideoEnabled").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    is_my_video_enabled_fieldID = jni::get_field_id(env, Class, "isMyVideoEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGroupCallIsMyVideoPaused::Class;
jfieldID toggleGroupCallIsMyVideoPaused::group_call_id_fieldID;
jfieldID toggleGroupCallIsMyVideoPaused::is_my_video_paused_fieldID;

toggleGroupCallIsMyVideoPaused::toggleGroupCallIsMyVideoPaused()
  : group_call_id_()
  , is_my_video_paused_()
{}

toggleGroupCallIsMyVideoPaused::toggleGroupCallIsMyVideoPaused(int32 group_call_id_, bool is_my_video_paused_)
  : group_call_id_(group_call_id_)
  , is_my_video_paused_(is_my_video_paused_)
{}

const std::int32_t toggleGroupCallIsMyVideoPaused::ID;

object_ptr<toggleGroupCallIsMyVideoPaused> toggleGroupCallIsMyVideoPaused::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGroupCallIsMyVideoPaused> res = make_object<toggleGroupCallIsMyVideoPaused>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->is_my_video_paused_ = (env->GetBooleanField(p, res->is_my_video_paused_fieldID) != 0);
  return res;
}

void toggleGroupCallIsMyVideoPaused::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGroupCallIsMyVideoPaused");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("isMyVideoPaused", is_my_video_paused_);
    s.store_class_end();
  }
}

toggleGroupCallIsMyVideoPaused::ReturnType toggleGroupCallIsMyVideoPaused::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGroupCallIsMyVideoPaused::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGroupCallIsMyVideoPaused").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    is_my_video_paused_fieldID = jni::get_field_id(env, Class, "isMyVideoPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGroupCallParticipantIsHandRaised::Class;
jfieldID toggleGroupCallParticipantIsHandRaised::group_call_id_fieldID;
jfieldID toggleGroupCallParticipantIsHandRaised::participant_id_fieldID;
jfieldID toggleGroupCallParticipantIsHandRaised::is_hand_raised_fieldID;

toggleGroupCallParticipantIsHandRaised::toggleGroupCallParticipantIsHandRaised()
  : group_call_id_()
  , participant_id_()
  , is_hand_raised_()
{}

toggleGroupCallParticipantIsHandRaised::toggleGroupCallParticipantIsHandRaised(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, bool is_hand_raised_)
  : group_call_id_(group_call_id_)
  , participant_id_(std::move(participant_id_))
  , is_hand_raised_(is_hand_raised_)
{}

const std::int32_t toggleGroupCallParticipantIsHandRaised::ID;

object_ptr<toggleGroupCallParticipantIsHandRaised> toggleGroupCallParticipantIsHandRaised::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGroupCallParticipantIsHandRaised> res = make_object<toggleGroupCallParticipantIsHandRaised>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->is_hand_raised_ = (env->GetBooleanField(p, res->is_hand_raised_fieldID) != 0);
  return res;
}

void toggleGroupCallParticipantIsHandRaised::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGroupCallParticipantIsHandRaised");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("isHandRaised", is_hand_raised_);
    s.store_class_end();
  }
}

toggleGroupCallParticipantIsHandRaised::ReturnType toggleGroupCallParticipantIsHandRaised::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGroupCallParticipantIsHandRaised::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGroupCallParticipantIsHandRaised").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_hand_raised_fieldID = jni::get_field_id(env, Class, "isHandRaised", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGroupCallParticipantIsMuted::Class;
jfieldID toggleGroupCallParticipantIsMuted::group_call_id_fieldID;
jfieldID toggleGroupCallParticipantIsMuted::participant_id_fieldID;
jfieldID toggleGroupCallParticipantIsMuted::is_muted_fieldID;

toggleGroupCallParticipantIsMuted::toggleGroupCallParticipantIsMuted()
  : group_call_id_()
  , participant_id_()
  , is_muted_()
{}

toggleGroupCallParticipantIsMuted::toggleGroupCallParticipantIsMuted(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, bool is_muted_)
  : group_call_id_(group_call_id_)
  , participant_id_(std::move(participant_id_))
  , is_muted_(is_muted_)
{}

const std::int32_t toggleGroupCallParticipantIsMuted::ID;

object_ptr<toggleGroupCallParticipantIsMuted> toggleGroupCallParticipantIsMuted::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGroupCallParticipantIsMuted> res = make_object<toggleGroupCallParticipantIsMuted>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->participant_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->participant_id_fieldID));
  res->is_muted_ = (env->GetBooleanField(p, res->is_muted_fieldID) != 0);
  return res;
}

void toggleGroupCallParticipantIsMuted::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGroupCallParticipantIsMuted");
    s.store_field("groupCallId", group_call_id_);
    s.store_object_field("participantId", static_cast<const BaseObject *>(participant_id_.get()));
    s.store_field("isMuted", is_muted_);
    s.store_class_end();
  }
}

toggleGroupCallParticipantIsMuted::ReturnType toggleGroupCallParticipantIsMuted::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGroupCallParticipantIsMuted::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGroupCallParticipantIsMuted").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    participant_id_fieldID = jni::get_field_id(env, Class, "participantId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    is_muted_fieldID = jni::get_field_id(env, Class, "isMuted", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleGroupCallScreenSharingIsPaused::Class;
jfieldID toggleGroupCallScreenSharingIsPaused::group_call_id_fieldID;
jfieldID toggleGroupCallScreenSharingIsPaused::is_paused_fieldID;

toggleGroupCallScreenSharingIsPaused::toggleGroupCallScreenSharingIsPaused()
  : group_call_id_()
  , is_paused_()
{}

toggleGroupCallScreenSharingIsPaused::toggleGroupCallScreenSharingIsPaused(int32 group_call_id_, bool is_paused_)
  : group_call_id_(group_call_id_)
  , is_paused_(is_paused_)
{}

const std::int32_t toggleGroupCallScreenSharingIsPaused::ID;

object_ptr<toggleGroupCallScreenSharingIsPaused> toggleGroupCallScreenSharingIsPaused::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleGroupCallScreenSharingIsPaused> res = make_object<toggleGroupCallScreenSharingIsPaused>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->is_paused_ = (env->GetBooleanField(p, res->is_paused_fieldID) != 0);
  return res;
}

void toggleGroupCallScreenSharingIsPaused::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleGroupCallScreenSharingIsPaused");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("isPaused", is_paused_);
    s.store_class_end();
  }
}

toggleGroupCallScreenSharingIsPaused::ReturnType toggleGroupCallScreenSharingIsPaused::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleGroupCallScreenSharingIsPaused::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleGroupCallScreenSharingIsPaused").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    is_paused_fieldID = jni::get_field_id(env, Class, "isPaused", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleHasSponsoredMessagesEnabled::Class;
jfieldID toggleHasSponsoredMessagesEnabled::has_sponsored_messages_enabled_fieldID;

toggleHasSponsoredMessagesEnabled::toggleHasSponsoredMessagesEnabled()
  : has_sponsored_messages_enabled_()
{}

toggleHasSponsoredMessagesEnabled::toggleHasSponsoredMessagesEnabled(bool has_sponsored_messages_enabled_)
  : has_sponsored_messages_enabled_(has_sponsored_messages_enabled_)
{}

const std::int32_t toggleHasSponsoredMessagesEnabled::ID;

object_ptr<toggleHasSponsoredMessagesEnabled> toggleHasSponsoredMessagesEnabled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleHasSponsoredMessagesEnabled> res = make_object<toggleHasSponsoredMessagesEnabled>();
  res->has_sponsored_messages_enabled_ = (env->GetBooleanField(p, res->has_sponsored_messages_enabled_fieldID) != 0);
  return res;
}

void toggleHasSponsoredMessagesEnabled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleHasSponsoredMessagesEnabled");
    s.store_field("hasSponsoredMessagesEnabled", has_sponsored_messages_enabled_);
    s.store_class_end();
  }
}

toggleHasSponsoredMessagesEnabled::ReturnType toggleHasSponsoredMessagesEnabled::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleHasSponsoredMessagesEnabled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleHasSponsoredMessagesEnabled").c_str());
    has_sponsored_messages_enabled_fieldID = jni::get_field_id(env, Class, "hasSponsoredMessagesEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSavedMessagesTopicIsPinned::Class;
jfieldID toggleSavedMessagesTopicIsPinned::saved_messages_topic_id_fieldID;
jfieldID toggleSavedMessagesTopicIsPinned::is_pinned_fieldID;

toggleSavedMessagesTopicIsPinned::toggleSavedMessagesTopicIsPinned()
  : saved_messages_topic_id_()
  , is_pinned_()
{}

toggleSavedMessagesTopicIsPinned::toggleSavedMessagesTopicIsPinned(int53 saved_messages_topic_id_, bool is_pinned_)
  : saved_messages_topic_id_(saved_messages_topic_id_)
  , is_pinned_(is_pinned_)
{}

const std::int32_t toggleSavedMessagesTopicIsPinned::ID;

object_ptr<toggleSavedMessagesTopicIsPinned> toggleSavedMessagesTopicIsPinned::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSavedMessagesTopicIsPinned> res = make_object<toggleSavedMessagesTopicIsPinned>();
  res->saved_messages_topic_id_ = env->GetLongField(p, res->saved_messages_topic_id_fieldID);
  res->is_pinned_ = (env->GetBooleanField(p, res->is_pinned_fieldID) != 0);
  return res;
}

void toggleSavedMessagesTopicIsPinned::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSavedMessagesTopicIsPinned");
    s.store_field("savedMessagesTopicId", saved_messages_topic_id_);
    s.store_field("isPinned", is_pinned_);
    s.store_class_end();
  }
}

toggleSavedMessagesTopicIsPinned::ReturnType toggleSavedMessagesTopicIsPinned::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSavedMessagesTopicIsPinned::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSavedMessagesTopicIsPinned").c_str());
    saved_messages_topic_id_fieldID = jni::get_field_id(env, Class, "savedMessagesTopicId", "J");
    is_pinned_fieldID = jni::get_field_id(env, Class, "isPinned", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSessionCanAcceptCalls::Class;
jfieldID toggleSessionCanAcceptCalls::session_id_fieldID;
jfieldID toggleSessionCanAcceptCalls::can_accept_calls_fieldID;

toggleSessionCanAcceptCalls::toggleSessionCanAcceptCalls()
  : session_id_()
  , can_accept_calls_()
{}

toggleSessionCanAcceptCalls::toggleSessionCanAcceptCalls(int64 session_id_, bool can_accept_calls_)
  : session_id_(session_id_)
  , can_accept_calls_(can_accept_calls_)
{}

const std::int32_t toggleSessionCanAcceptCalls::ID;

object_ptr<toggleSessionCanAcceptCalls> toggleSessionCanAcceptCalls::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSessionCanAcceptCalls> res = make_object<toggleSessionCanAcceptCalls>();
  res->session_id_ = env->GetLongField(p, res->session_id_fieldID);
  res->can_accept_calls_ = (env->GetBooleanField(p, res->can_accept_calls_fieldID) != 0);
  return res;
}

void toggleSessionCanAcceptCalls::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSessionCanAcceptCalls");
    s.store_field("sessionId", session_id_);
    s.store_field("canAcceptCalls", can_accept_calls_);
    s.store_class_end();
  }
}

toggleSessionCanAcceptCalls::ReturnType toggleSessionCanAcceptCalls::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSessionCanAcceptCalls::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSessionCanAcceptCalls").c_str());
    session_id_fieldID = jni::get_field_id(env, Class, "sessionId", "J");
    can_accept_calls_fieldID = jni::get_field_id(env, Class, "canAcceptCalls", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSessionCanAcceptSecretChats::Class;
jfieldID toggleSessionCanAcceptSecretChats::session_id_fieldID;
jfieldID toggleSessionCanAcceptSecretChats::can_accept_secret_chats_fieldID;

toggleSessionCanAcceptSecretChats::toggleSessionCanAcceptSecretChats()
  : session_id_()
  , can_accept_secret_chats_()
{}

toggleSessionCanAcceptSecretChats::toggleSessionCanAcceptSecretChats(int64 session_id_, bool can_accept_secret_chats_)
  : session_id_(session_id_)
  , can_accept_secret_chats_(can_accept_secret_chats_)
{}

const std::int32_t toggleSessionCanAcceptSecretChats::ID;

object_ptr<toggleSessionCanAcceptSecretChats> toggleSessionCanAcceptSecretChats::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSessionCanAcceptSecretChats> res = make_object<toggleSessionCanAcceptSecretChats>();
  res->session_id_ = env->GetLongField(p, res->session_id_fieldID);
  res->can_accept_secret_chats_ = (env->GetBooleanField(p, res->can_accept_secret_chats_fieldID) != 0);
  return res;
}

void toggleSessionCanAcceptSecretChats::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSessionCanAcceptSecretChats");
    s.store_field("sessionId", session_id_);
    s.store_field("canAcceptSecretChats", can_accept_secret_chats_);
    s.store_class_end();
  }
}

toggleSessionCanAcceptSecretChats::ReturnType toggleSessionCanAcceptSecretChats::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSessionCanAcceptSecretChats::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSessionCanAcceptSecretChats").c_str());
    session_id_fieldID = jni::get_field_id(env, Class, "sessionId", "J");
    can_accept_secret_chats_fieldID = jni::get_field_id(env, Class, "canAcceptSecretChats", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleStoryIsPostedToChatPage::Class;
jfieldID toggleStoryIsPostedToChatPage::story_poster_chat_id_fieldID;
jfieldID toggleStoryIsPostedToChatPage::story_id_fieldID;
jfieldID toggleStoryIsPostedToChatPage::is_posted_to_chat_page_fieldID;

toggleStoryIsPostedToChatPage::toggleStoryIsPostedToChatPage()
  : story_poster_chat_id_()
  , story_id_()
  , is_posted_to_chat_page_()
{}

toggleStoryIsPostedToChatPage::toggleStoryIsPostedToChatPage(int53 story_poster_chat_id_, int32 story_id_, bool is_posted_to_chat_page_)
  : story_poster_chat_id_(story_poster_chat_id_)
  , story_id_(story_id_)
  , is_posted_to_chat_page_(is_posted_to_chat_page_)
{}

const std::int32_t toggleStoryIsPostedToChatPage::ID;

object_ptr<toggleStoryIsPostedToChatPage> toggleStoryIsPostedToChatPage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleStoryIsPostedToChatPage> res = make_object<toggleStoryIsPostedToChatPage>();
  res->story_poster_chat_id_ = env->GetLongField(p, res->story_poster_chat_id_fieldID);
  res->story_id_ = env->GetIntField(p, res->story_id_fieldID);
  res->is_posted_to_chat_page_ = (env->GetBooleanField(p, res->is_posted_to_chat_page_fieldID) != 0);
  return res;
}

void toggleStoryIsPostedToChatPage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleStoryIsPostedToChatPage");
    s.store_field("storyPosterChatId", story_poster_chat_id_);
    s.store_field("storyId", story_id_);
    s.store_field("isPostedToChatPage", is_posted_to_chat_page_);
    s.store_class_end();
  }
}

toggleStoryIsPostedToChatPage::ReturnType toggleStoryIsPostedToChatPage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleStoryIsPostedToChatPage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleStoryIsPostedToChatPage").c_str());
    story_poster_chat_id_fieldID = jni::get_field_id(env, Class, "storyPosterChatId", "J");
    story_id_fieldID = jni::get_field_id(env, Class, "storyId", "I");
    is_posted_to_chat_page_fieldID = jni::get_field_id(env, Class, "isPostedToChatPage", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupCanHaveSponsoredMessages::Class;
jfieldID toggleSupergroupCanHaveSponsoredMessages::supergroup_id_fieldID;
jfieldID toggleSupergroupCanHaveSponsoredMessages::can_have_sponsored_messages_fieldID;

toggleSupergroupCanHaveSponsoredMessages::toggleSupergroupCanHaveSponsoredMessages()
  : supergroup_id_()
  , can_have_sponsored_messages_()
{}

toggleSupergroupCanHaveSponsoredMessages::toggleSupergroupCanHaveSponsoredMessages(int53 supergroup_id_, bool can_have_sponsored_messages_)
  : supergroup_id_(supergroup_id_)
  , can_have_sponsored_messages_(can_have_sponsored_messages_)
{}

const std::int32_t toggleSupergroupCanHaveSponsoredMessages::ID;

object_ptr<toggleSupergroupCanHaveSponsoredMessages> toggleSupergroupCanHaveSponsoredMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupCanHaveSponsoredMessages> res = make_object<toggleSupergroupCanHaveSponsoredMessages>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->can_have_sponsored_messages_ = (env->GetBooleanField(p, res->can_have_sponsored_messages_fieldID) != 0);
  return res;
}

void toggleSupergroupCanHaveSponsoredMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupCanHaveSponsoredMessages");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("canHaveSponsoredMessages", can_have_sponsored_messages_);
    s.store_class_end();
  }
}

toggleSupergroupCanHaveSponsoredMessages::ReturnType toggleSupergroupCanHaveSponsoredMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupCanHaveSponsoredMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupCanHaveSponsoredMessages").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    can_have_sponsored_messages_fieldID = jni::get_field_id(env, Class, "canHaveSponsoredMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupHasAggressiveAntiSpamEnabled::Class;
jfieldID toggleSupergroupHasAggressiveAntiSpamEnabled::supergroup_id_fieldID;
jfieldID toggleSupergroupHasAggressiveAntiSpamEnabled::has_aggressive_anti_spam_enabled_fieldID;

toggleSupergroupHasAggressiveAntiSpamEnabled::toggleSupergroupHasAggressiveAntiSpamEnabled()
  : supergroup_id_()
  , has_aggressive_anti_spam_enabled_()
{}

toggleSupergroupHasAggressiveAntiSpamEnabled::toggleSupergroupHasAggressiveAntiSpamEnabled(int53 supergroup_id_, bool has_aggressive_anti_spam_enabled_)
  : supergroup_id_(supergroup_id_)
  , has_aggressive_anti_spam_enabled_(has_aggressive_anti_spam_enabled_)
{}

const std::int32_t toggleSupergroupHasAggressiveAntiSpamEnabled::ID;

object_ptr<toggleSupergroupHasAggressiveAntiSpamEnabled> toggleSupergroupHasAggressiveAntiSpamEnabled::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupHasAggressiveAntiSpamEnabled> res = make_object<toggleSupergroupHasAggressiveAntiSpamEnabled>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->has_aggressive_anti_spam_enabled_ = (env->GetBooleanField(p, res->has_aggressive_anti_spam_enabled_fieldID) != 0);
  return res;
}

void toggleSupergroupHasAggressiveAntiSpamEnabled::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupHasAggressiveAntiSpamEnabled");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("hasAggressiveAntiSpamEnabled", has_aggressive_anti_spam_enabled_);
    s.store_class_end();
  }
}

toggleSupergroupHasAggressiveAntiSpamEnabled::ReturnType toggleSupergroupHasAggressiveAntiSpamEnabled::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupHasAggressiveAntiSpamEnabled::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupHasAggressiveAntiSpamEnabled").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    has_aggressive_anti_spam_enabled_fieldID = jni::get_field_id(env, Class, "hasAggressiveAntiSpamEnabled", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupHasAutomaticTranslation::Class;
jfieldID toggleSupergroupHasAutomaticTranslation::supergroup_id_fieldID;
jfieldID toggleSupergroupHasAutomaticTranslation::has_automatic_translation_fieldID;

toggleSupergroupHasAutomaticTranslation::toggleSupergroupHasAutomaticTranslation()
  : supergroup_id_()
  , has_automatic_translation_()
{}

toggleSupergroupHasAutomaticTranslation::toggleSupergroupHasAutomaticTranslation(int53 supergroup_id_, bool has_automatic_translation_)
  : supergroup_id_(supergroup_id_)
  , has_automatic_translation_(has_automatic_translation_)
{}

const std::int32_t toggleSupergroupHasAutomaticTranslation::ID;

object_ptr<toggleSupergroupHasAutomaticTranslation> toggleSupergroupHasAutomaticTranslation::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupHasAutomaticTranslation> res = make_object<toggleSupergroupHasAutomaticTranslation>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->has_automatic_translation_ = (env->GetBooleanField(p, res->has_automatic_translation_fieldID) != 0);
  return res;
}

void toggleSupergroupHasAutomaticTranslation::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupHasAutomaticTranslation");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("hasAutomaticTranslation", has_automatic_translation_);
    s.store_class_end();
  }
}

toggleSupergroupHasAutomaticTranslation::ReturnType toggleSupergroupHasAutomaticTranslation::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupHasAutomaticTranslation::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupHasAutomaticTranslation").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    has_automatic_translation_fieldID = jni::get_field_id(env, Class, "hasAutomaticTranslation", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupHasHiddenMembers::Class;
jfieldID toggleSupergroupHasHiddenMembers::supergroup_id_fieldID;
jfieldID toggleSupergroupHasHiddenMembers::has_hidden_members_fieldID;

toggleSupergroupHasHiddenMembers::toggleSupergroupHasHiddenMembers()
  : supergroup_id_()
  , has_hidden_members_()
{}

toggleSupergroupHasHiddenMembers::toggleSupergroupHasHiddenMembers(int53 supergroup_id_, bool has_hidden_members_)
  : supergroup_id_(supergroup_id_)
  , has_hidden_members_(has_hidden_members_)
{}

const std::int32_t toggleSupergroupHasHiddenMembers::ID;

object_ptr<toggleSupergroupHasHiddenMembers> toggleSupergroupHasHiddenMembers::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupHasHiddenMembers> res = make_object<toggleSupergroupHasHiddenMembers>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->has_hidden_members_ = (env->GetBooleanField(p, res->has_hidden_members_fieldID) != 0);
  return res;
}

void toggleSupergroupHasHiddenMembers::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupHasHiddenMembers");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("hasHiddenMembers", has_hidden_members_);
    s.store_class_end();
  }
}

toggleSupergroupHasHiddenMembers::ReturnType toggleSupergroupHasHiddenMembers::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupHasHiddenMembers::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupHasHiddenMembers").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    has_hidden_members_fieldID = jni::get_field_id(env, Class, "hasHiddenMembers", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupIsAllHistoryAvailable::Class;
jfieldID toggleSupergroupIsAllHistoryAvailable::supergroup_id_fieldID;
jfieldID toggleSupergroupIsAllHistoryAvailable::is_all_history_available_fieldID;

toggleSupergroupIsAllHistoryAvailable::toggleSupergroupIsAllHistoryAvailable()
  : supergroup_id_()
  , is_all_history_available_()
{}

toggleSupergroupIsAllHistoryAvailable::toggleSupergroupIsAllHistoryAvailable(int53 supergroup_id_, bool is_all_history_available_)
  : supergroup_id_(supergroup_id_)
  , is_all_history_available_(is_all_history_available_)
{}

const std::int32_t toggleSupergroupIsAllHistoryAvailable::ID;

object_ptr<toggleSupergroupIsAllHistoryAvailable> toggleSupergroupIsAllHistoryAvailable::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupIsAllHistoryAvailable> res = make_object<toggleSupergroupIsAllHistoryAvailable>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->is_all_history_available_ = (env->GetBooleanField(p, res->is_all_history_available_fieldID) != 0);
  return res;
}

void toggleSupergroupIsAllHistoryAvailable::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupIsAllHistoryAvailable");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("isAllHistoryAvailable", is_all_history_available_);
    s.store_class_end();
  }
}

toggleSupergroupIsAllHistoryAvailable::ReturnType toggleSupergroupIsAllHistoryAvailable::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupIsAllHistoryAvailable::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupIsAllHistoryAvailable").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    is_all_history_available_fieldID = jni::get_field_id(env, Class, "isAllHistoryAvailable", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupIsBroadcastGroup::Class;
jfieldID toggleSupergroupIsBroadcastGroup::supergroup_id_fieldID;

toggleSupergroupIsBroadcastGroup::toggleSupergroupIsBroadcastGroup()
  : supergroup_id_()
{}

toggleSupergroupIsBroadcastGroup::toggleSupergroupIsBroadcastGroup(int53 supergroup_id_)
  : supergroup_id_(supergroup_id_)
{}

const std::int32_t toggleSupergroupIsBroadcastGroup::ID;

object_ptr<toggleSupergroupIsBroadcastGroup> toggleSupergroupIsBroadcastGroup::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupIsBroadcastGroup> res = make_object<toggleSupergroupIsBroadcastGroup>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  return res;
}

void toggleSupergroupIsBroadcastGroup::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupIsBroadcastGroup");
    s.store_field("supergroupId", supergroup_id_);
    s.store_class_end();
  }
}

toggleSupergroupIsBroadcastGroup::ReturnType toggleSupergroupIsBroadcastGroup::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupIsBroadcastGroup::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupIsBroadcastGroup").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupIsForum::Class;
jfieldID toggleSupergroupIsForum::supergroup_id_fieldID;
jfieldID toggleSupergroupIsForum::is_forum_fieldID;

toggleSupergroupIsForum::toggleSupergroupIsForum()
  : supergroup_id_()
  , is_forum_()
{}

toggleSupergroupIsForum::toggleSupergroupIsForum(int53 supergroup_id_, bool is_forum_)
  : supergroup_id_(supergroup_id_)
  , is_forum_(is_forum_)
{}

const std::int32_t toggleSupergroupIsForum::ID;

object_ptr<toggleSupergroupIsForum> toggleSupergroupIsForum::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupIsForum> res = make_object<toggleSupergroupIsForum>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->is_forum_ = (env->GetBooleanField(p, res->is_forum_fieldID) != 0);
  return res;
}

void toggleSupergroupIsForum::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupIsForum");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("isForum", is_forum_);
    s.store_class_end();
  }
}

toggleSupergroupIsForum::ReturnType toggleSupergroupIsForum::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupIsForum::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupIsForum").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    is_forum_fieldID = jni::get_field_id(env, Class, "isForum", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupJoinByRequest::Class;
jfieldID toggleSupergroupJoinByRequest::supergroup_id_fieldID;
jfieldID toggleSupergroupJoinByRequest::join_by_request_fieldID;

toggleSupergroupJoinByRequest::toggleSupergroupJoinByRequest()
  : supergroup_id_()
  , join_by_request_()
{}

toggleSupergroupJoinByRequest::toggleSupergroupJoinByRequest(int53 supergroup_id_, bool join_by_request_)
  : supergroup_id_(supergroup_id_)
  , join_by_request_(join_by_request_)
{}

const std::int32_t toggleSupergroupJoinByRequest::ID;

object_ptr<toggleSupergroupJoinByRequest> toggleSupergroupJoinByRequest::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupJoinByRequest> res = make_object<toggleSupergroupJoinByRequest>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->join_by_request_ = (env->GetBooleanField(p, res->join_by_request_fieldID) != 0);
  return res;
}

void toggleSupergroupJoinByRequest::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupJoinByRequest");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("joinByRequest", join_by_request_);
    s.store_class_end();
  }
}

toggleSupergroupJoinByRequest::ReturnType toggleSupergroupJoinByRequest::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupJoinByRequest::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupJoinByRequest").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    join_by_request_fieldID = jni::get_field_id(env, Class, "joinByRequest", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupJoinToSendMessages::Class;
jfieldID toggleSupergroupJoinToSendMessages::supergroup_id_fieldID;
jfieldID toggleSupergroupJoinToSendMessages::join_to_send_messages_fieldID;

toggleSupergroupJoinToSendMessages::toggleSupergroupJoinToSendMessages()
  : supergroup_id_()
  , join_to_send_messages_()
{}

toggleSupergroupJoinToSendMessages::toggleSupergroupJoinToSendMessages(int53 supergroup_id_, bool join_to_send_messages_)
  : supergroup_id_(supergroup_id_)
  , join_to_send_messages_(join_to_send_messages_)
{}

const std::int32_t toggleSupergroupJoinToSendMessages::ID;

object_ptr<toggleSupergroupJoinToSendMessages> toggleSupergroupJoinToSendMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupJoinToSendMessages> res = make_object<toggleSupergroupJoinToSendMessages>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->join_to_send_messages_ = (env->GetBooleanField(p, res->join_to_send_messages_fieldID) != 0);
  return res;
}

void toggleSupergroupJoinToSendMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupJoinToSendMessages");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("joinToSendMessages", join_to_send_messages_);
    s.store_class_end();
  }
}

toggleSupergroupJoinToSendMessages::ReturnType toggleSupergroupJoinToSendMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupJoinToSendMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupJoinToSendMessages").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    join_to_send_messages_fieldID = jni::get_field_id(env, Class, "joinToSendMessages", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupSignMessages::Class;
jfieldID toggleSupergroupSignMessages::supergroup_id_fieldID;
jfieldID toggleSupergroupSignMessages::sign_messages_fieldID;
jfieldID toggleSupergroupSignMessages::show_message_sender_fieldID;

toggleSupergroupSignMessages::toggleSupergroupSignMessages()
  : supergroup_id_()
  , sign_messages_()
  , show_message_sender_()
{}

toggleSupergroupSignMessages::toggleSupergroupSignMessages(int53 supergroup_id_, bool sign_messages_, bool show_message_sender_)
  : supergroup_id_(supergroup_id_)
  , sign_messages_(sign_messages_)
  , show_message_sender_(show_message_sender_)
{}

const std::int32_t toggleSupergroupSignMessages::ID;

object_ptr<toggleSupergroupSignMessages> toggleSupergroupSignMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupSignMessages> res = make_object<toggleSupergroupSignMessages>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->sign_messages_ = (env->GetBooleanField(p, res->sign_messages_fieldID) != 0);
  res->show_message_sender_ = (env->GetBooleanField(p, res->show_message_sender_fieldID) != 0);
  return res;
}

void toggleSupergroupSignMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupSignMessages");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("signMessages", sign_messages_);
    s.store_field("showMessageSender", show_message_sender_);
    s.store_class_end();
  }
}

toggleSupergroupSignMessages::ReturnType toggleSupergroupSignMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupSignMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupSignMessages").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    sign_messages_fieldID = jni::get_field_id(env, Class, "signMessages", "Z");
    show_message_sender_fieldID = jni::get_field_id(env, Class, "showMessageSender", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleSupergroupUsernameIsActive::Class;
jfieldID toggleSupergroupUsernameIsActive::supergroup_id_fieldID;
jfieldID toggleSupergroupUsernameIsActive::username_fieldID;
jfieldID toggleSupergroupUsernameIsActive::is_active_fieldID;

toggleSupergroupUsernameIsActive::toggleSupergroupUsernameIsActive()
  : supergroup_id_()
  , username_()
  , is_active_()
{}

toggleSupergroupUsernameIsActive::toggleSupergroupUsernameIsActive(int53 supergroup_id_, string const &username_, bool is_active_)
  : supergroup_id_(supergroup_id_)
  , username_(username_)
  , is_active_(is_active_)
{}

const std::int32_t toggleSupergroupUsernameIsActive::ID;

object_ptr<toggleSupergroupUsernameIsActive> toggleSupergroupUsernameIsActive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleSupergroupUsernameIsActive> res = make_object<toggleSupergroupUsernameIsActive>();
  res->supergroup_id_ = env->GetLongField(p, res->supergroup_id_fieldID);
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  return res;
}

void toggleSupergroupUsernameIsActive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleSupergroupUsernameIsActive");
    s.store_field("supergroupId", supergroup_id_);
    s.store_field("username", username_);
    s.store_field("isActive", is_active_);
    s.store_class_end();
  }
}

toggleSupergroupUsernameIsActive::ReturnType toggleSupergroupUsernameIsActive::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleSupergroupUsernameIsActive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleSupergroupUsernameIsActive").c_str());
    supergroup_id_fieldID = jni::get_field_id(env, Class, "supergroupId", "J");
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleUsernameIsActive::Class;
jfieldID toggleUsernameIsActive::username_fieldID;
jfieldID toggleUsernameIsActive::is_active_fieldID;

toggleUsernameIsActive::toggleUsernameIsActive()
  : username_()
  , is_active_()
{}

toggleUsernameIsActive::toggleUsernameIsActive(string const &username_, bool is_active_)
  : username_(username_)
  , is_active_(is_active_)
{}

const std::int32_t toggleUsernameIsActive::ID;

object_ptr<toggleUsernameIsActive> toggleUsernameIsActive::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleUsernameIsActive> res = make_object<toggleUsernameIsActive>();
  res->username_ = jni::fetch_string(env, p, res->username_fieldID);
  res->is_active_ = (env->GetBooleanField(p, res->is_active_fieldID) != 0);
  return res;
}

void toggleUsernameIsActive::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleUsernameIsActive");
    s.store_field("username", username_);
    s.store_field("isActive", is_active_);
    s.store_class_end();
  }
}

toggleUsernameIsActive::ReturnType toggleUsernameIsActive::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleUsernameIsActive::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleUsernameIsActive").c_str());
    username_fieldID = jni::get_field_id(env, Class, "username", "Ljava/lang/String;");
    is_active_fieldID = jni::get_field_id(env, Class, "isActive", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleVideoChatEnabledStartNotification::Class;
jfieldID toggleVideoChatEnabledStartNotification::group_call_id_fieldID;
jfieldID toggleVideoChatEnabledStartNotification::enabled_start_notification_fieldID;

toggleVideoChatEnabledStartNotification::toggleVideoChatEnabledStartNotification()
  : group_call_id_()
  , enabled_start_notification_()
{}

toggleVideoChatEnabledStartNotification::toggleVideoChatEnabledStartNotification(int32 group_call_id_, bool enabled_start_notification_)
  : group_call_id_(group_call_id_)
  , enabled_start_notification_(enabled_start_notification_)
{}

const std::int32_t toggleVideoChatEnabledStartNotification::ID;

object_ptr<toggleVideoChatEnabledStartNotification> toggleVideoChatEnabledStartNotification::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleVideoChatEnabledStartNotification> res = make_object<toggleVideoChatEnabledStartNotification>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->enabled_start_notification_ = (env->GetBooleanField(p, res->enabled_start_notification_fieldID) != 0);
  return res;
}

void toggleVideoChatEnabledStartNotification::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleVideoChatEnabledStartNotification");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("enabledStartNotification", enabled_start_notification_);
    s.store_class_end();
  }
}

toggleVideoChatEnabledStartNotification::ReturnType toggleVideoChatEnabledStartNotification::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleVideoChatEnabledStartNotification::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleVideoChatEnabledStartNotification").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    enabled_start_notification_fieldID = jni::get_field_id(env, Class, "enabledStartNotification", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass toggleVideoChatMuteNewParticipants::Class;
jfieldID toggleVideoChatMuteNewParticipants::group_call_id_fieldID;
jfieldID toggleVideoChatMuteNewParticipants::mute_new_participants_fieldID;

toggleVideoChatMuteNewParticipants::toggleVideoChatMuteNewParticipants()
  : group_call_id_()
  , mute_new_participants_()
{}

toggleVideoChatMuteNewParticipants::toggleVideoChatMuteNewParticipants(int32 group_call_id_, bool mute_new_participants_)
  : group_call_id_(group_call_id_)
  , mute_new_participants_(mute_new_participants_)
{}

const std::int32_t toggleVideoChatMuteNewParticipants::ID;

object_ptr<toggleVideoChatMuteNewParticipants> toggleVideoChatMuteNewParticipants::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<toggleVideoChatMuteNewParticipants> res = make_object<toggleVideoChatMuteNewParticipants>();
  res->group_call_id_ = env->GetIntField(p, res->group_call_id_fieldID);
  res->mute_new_participants_ = (env->GetBooleanField(p, res->mute_new_participants_fieldID) != 0);
  return res;
}

void toggleVideoChatMuteNewParticipants::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ToggleVideoChatMuteNewParticipants");
    s.store_field("groupCallId", group_call_id_);
    s.store_field("muteNewParticipants", mute_new_participants_);
    s.store_class_end();
  }
}

toggleVideoChatMuteNewParticipants::ReturnType toggleVideoChatMuteNewParticipants::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void toggleVideoChatMuteNewParticipants::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ToggleVideoChatMuteNewParticipants").c_str());
    group_call_id_fieldID = jni::get_field_id(env, Class, "groupCallId", "I");
    mute_new_participants_fieldID = jni::get_field_id(env, Class, "muteNewParticipants", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass transferBusinessAccountStars::Class;
jfieldID transferBusinessAccountStars::business_connection_id_fieldID;
jfieldID transferBusinessAccountStars::star_count_fieldID;

transferBusinessAccountStars::transferBusinessAccountStars()
  : business_connection_id_()
  , star_count_()
{}

transferBusinessAccountStars::transferBusinessAccountStars(string const &business_connection_id_, int53 star_count_)
  : business_connection_id_(business_connection_id_)
  , star_count_(star_count_)
{}

const std::int32_t transferBusinessAccountStars::ID;

object_ptr<transferBusinessAccountStars> transferBusinessAccountStars::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<transferBusinessAccountStars> res = make_object<transferBusinessAccountStars>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void transferBusinessAccountStars::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TransferBusinessAccountStars");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

transferBusinessAccountStars::ReturnType transferBusinessAccountStars::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void transferBusinessAccountStars::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TransferBusinessAccountStars").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass transferChatOwnership::Class;
jfieldID transferChatOwnership::chat_id_fieldID;
jfieldID transferChatOwnership::user_id_fieldID;
jfieldID transferChatOwnership::password_fieldID;

transferChatOwnership::transferChatOwnership()
  : chat_id_()
  , user_id_()
  , password_()
{}

transferChatOwnership::transferChatOwnership(int53 chat_id_, int53 user_id_, string const &password_)
  : chat_id_(chat_id_)
  , user_id_(user_id_)
  , password_(password_)
{}

const std::int32_t transferChatOwnership::ID;

object_ptr<transferChatOwnership> transferChatOwnership::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<transferChatOwnership> res = make_object<transferChatOwnership>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->password_ = jni::fetch_string(env, p, res->password_fieldID);
  return res;
}

void transferChatOwnership::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TransferChatOwnership");
    s.store_field("chatId", chat_id_);
    s.store_field("userId", user_id_);
    s.store_field("password", password_);
    s.store_class_end();
  }
}

transferChatOwnership::ReturnType transferChatOwnership::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void transferChatOwnership::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TransferChatOwnership").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    password_fieldID = jni::get_field_id(env, Class, "password", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass transferGift::Class;
jfieldID transferGift::business_connection_id_fieldID;
jfieldID transferGift::received_gift_id_fieldID;
jfieldID transferGift::new_owner_id_fieldID;
jfieldID transferGift::star_count_fieldID;

transferGift::transferGift()
  : business_connection_id_()
  , received_gift_id_()
  , new_owner_id_()
  , star_count_()
{}

transferGift::transferGift(string const &business_connection_id_, string const &received_gift_id_, object_ptr<MessageSender> &&new_owner_id_, int53 star_count_)
  : business_connection_id_(business_connection_id_)
  , received_gift_id_(received_gift_id_)
  , new_owner_id_(std::move(new_owner_id_))
  , star_count_(star_count_)
{}

const std::int32_t transferGift::ID;

object_ptr<transferGift> transferGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<transferGift> res = make_object<transferGift>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->new_owner_id_ = jni::fetch_tl_object<MessageSender>(env, jni::fetch_object(env, p, res->new_owner_id_fieldID));
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void transferGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TransferGift");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_object_field("newOwnerId", static_cast<const BaseObject *>(new_owner_id_.get()));
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

transferGift::ReturnType transferGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void transferGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TransferGift").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    new_owner_id_fieldID = jni::get_field_id(env, Class, "newOwnerId", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSender;").c_str());
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass translateMessageText::Class;
jfieldID translateMessageText::chat_id_fieldID;
jfieldID translateMessageText::message_id_fieldID;
jfieldID translateMessageText::to_language_code_fieldID;

translateMessageText::translateMessageText()
  : chat_id_()
  , message_id_()
  , to_language_code_()
{}

translateMessageText::translateMessageText(int53 chat_id_, int53 message_id_, string const &to_language_code_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
  , to_language_code_(to_language_code_)
{}

const std::int32_t translateMessageText::ID;

object_ptr<translateMessageText> translateMessageText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<translateMessageText> res = make_object<translateMessageText>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  res->to_language_code_ = jni::fetch_string(env, p, res->to_language_code_fieldID);
  return res;
}

void translateMessageText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TranslateMessageText");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_field("toLanguageCode", to_language_code_);
    s.store_class_end();
  }
}

translateMessageText::ReturnType translateMessageText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return formattedText::fetch(env, p);
}

void translateMessageText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TranslateMessageText").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    to_language_code_fieldID = jni::get_field_id(env, Class, "toLanguageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass translateText::Class;
jfieldID translateText::text_fieldID;
jfieldID translateText::to_language_code_fieldID;

translateText::translateText()
  : text_()
  , to_language_code_()
{}

translateText::translateText(object_ptr<formattedText> &&text_, string const &to_language_code_)
  : text_(std::move(text_))
  , to_language_code_(to_language_code_)
{}

const std::int32_t translateText::ID;

object_ptr<translateText> translateText::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<translateText> res = make_object<translateText>();
  res->text_ = jni::fetch_tl_object<formattedText>(env, jni::fetch_object(env, p, res->text_fieldID));
  res->to_language_code_ = jni::fetch_string(env, p, res->to_language_code_fieldID);
  return res;
}

void translateText::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "TranslateText");
    s.store_object_field("text", static_cast<const BaseObject *>(text_.get()));
    s.store_field("toLanguageCode", to_language_code_);
    s.store_class_end();
  }
}

translateText::ReturnType translateText::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return formattedText::fetch(env, p);
}

void translateText::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$TranslateText").c_str());
    text_fieldID = jni::get_field_id(env, Class, "text", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$FormattedText;").c_str());
    to_language_code_fieldID = jni::get_field_id(env, Class, "toLanguageCode", "Ljava/lang/String;");
    return true;
  }();
  (void)is_inited;
}

jclass unpinAllChatMessages::Class;
jfieldID unpinAllChatMessages::chat_id_fieldID;

unpinAllChatMessages::unpinAllChatMessages()
  : chat_id_()
{}

unpinAllChatMessages::unpinAllChatMessages(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t unpinAllChatMessages::ID;

object_ptr<unpinAllChatMessages> unpinAllChatMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<unpinAllChatMessages> res = make_object<unpinAllChatMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void unpinAllChatMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UnpinAllChatMessages");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

unpinAllChatMessages::ReturnType unpinAllChatMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void unpinAllChatMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UnpinAllChatMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass unpinAllMessageThreadMessages::Class;
jfieldID unpinAllMessageThreadMessages::chat_id_fieldID;
jfieldID unpinAllMessageThreadMessages::message_thread_id_fieldID;

unpinAllMessageThreadMessages::unpinAllMessageThreadMessages()
  : chat_id_()
  , message_thread_id_()
{}

unpinAllMessageThreadMessages::unpinAllMessageThreadMessages(int53 chat_id_, int53 message_thread_id_)
  : chat_id_(chat_id_)
  , message_thread_id_(message_thread_id_)
{}

const std::int32_t unpinAllMessageThreadMessages::ID;

object_ptr<unpinAllMessageThreadMessages> unpinAllMessageThreadMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<unpinAllMessageThreadMessages> res = make_object<unpinAllMessageThreadMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_thread_id_ = env->GetLongField(p, res->message_thread_id_fieldID);
  return res;
}

void unpinAllMessageThreadMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UnpinAllMessageThreadMessages");
    s.store_field("chatId", chat_id_);
    s.store_field("messageThreadId", message_thread_id_);
    s.store_class_end();
  }
}

unpinAllMessageThreadMessages::ReturnType unpinAllMessageThreadMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void unpinAllMessageThreadMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UnpinAllMessageThreadMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_thread_id_fieldID = jni::get_field_id(env, Class, "messageThreadId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass unpinChatMessage::Class;
jfieldID unpinChatMessage::chat_id_fieldID;
jfieldID unpinChatMessage::message_id_fieldID;

unpinChatMessage::unpinChatMessage()
  : chat_id_()
  , message_id_()
{}

unpinChatMessage::unpinChatMessage(int53 chat_id_, int53 message_id_)
  : chat_id_(chat_id_)
  , message_id_(message_id_)
{}

const std::int32_t unpinChatMessage::ID;

object_ptr<unpinChatMessage> unpinChatMessage::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<unpinChatMessage> res = make_object<unpinChatMessage>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_id_ = env->GetLongField(p, res->message_id_fieldID);
  return res;
}

void unpinChatMessage::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UnpinChatMessage");
    s.store_field("chatId", chat_id_);
    s.store_field("messageId", message_id_);
    s.store_class_end();
  }
}

unpinChatMessage::ReturnType unpinChatMessage::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void unpinChatMessage::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UnpinChatMessage").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_id_fieldID = jni::get_field_id(env, Class, "messageId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass upgradeBasicGroupChatToSupergroupChat::Class;
jfieldID upgradeBasicGroupChatToSupergroupChat::chat_id_fieldID;

upgradeBasicGroupChatToSupergroupChat::upgradeBasicGroupChatToSupergroupChat()
  : chat_id_()
{}

upgradeBasicGroupChatToSupergroupChat::upgradeBasicGroupChatToSupergroupChat(int53 chat_id_)
  : chat_id_(chat_id_)
{}

const std::int32_t upgradeBasicGroupChatToSupergroupChat::ID;

object_ptr<upgradeBasicGroupChatToSupergroupChat> upgradeBasicGroupChatToSupergroupChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradeBasicGroupChatToSupergroupChat> res = make_object<upgradeBasicGroupChatToSupergroupChat>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  return res;
}

void upgradeBasicGroupChatToSupergroupChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradeBasicGroupChatToSupergroupChat");
    s.store_field("chatId", chat_id_);
    s.store_class_end();
  }
}

upgradeBasicGroupChatToSupergroupChat::ReturnType upgradeBasicGroupChatToSupergroupChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return chat::fetch(env, p);
}

void upgradeBasicGroupChatToSupergroupChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradeBasicGroupChatToSupergroupChat").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass upgradeGift::Class;
jfieldID upgradeGift::business_connection_id_fieldID;
jfieldID upgradeGift::received_gift_id_fieldID;
jfieldID upgradeGift::keep_original_details_fieldID;
jfieldID upgradeGift::star_count_fieldID;

upgradeGift::upgradeGift()
  : business_connection_id_()
  , received_gift_id_()
  , keep_original_details_()
  , star_count_()
{}

upgradeGift::upgradeGift(string const &business_connection_id_, string const &received_gift_id_, bool keep_original_details_, int53 star_count_)
  : business_connection_id_(business_connection_id_)
  , received_gift_id_(received_gift_id_)
  , keep_original_details_(keep_original_details_)
  , star_count_(star_count_)
{}

const std::int32_t upgradeGift::ID;

object_ptr<upgradeGift> upgradeGift::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<upgradeGift> res = make_object<upgradeGift>();
  res->business_connection_id_ = jni::fetch_string(env, p, res->business_connection_id_fieldID);
  res->received_gift_id_ = jni::fetch_string(env, p, res->received_gift_id_fieldID);
  res->keep_original_details_ = (env->GetBooleanField(p, res->keep_original_details_fieldID) != 0);
  res->star_count_ = env->GetLongField(p, res->star_count_fieldID);
  return res;
}

void upgradeGift::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UpgradeGift");
    s.store_field("businessConnectionId", business_connection_id_);
    s.store_field("receivedGiftId", received_gift_id_);
    s.store_field("keepOriginalDetails", keep_original_details_);
    s.store_field("starCount", star_count_);
    s.store_class_end();
  }
}

upgradeGift::ReturnType upgradeGift::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return upgradeGiftResult::fetch(env, p);
}

void upgradeGift::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UpgradeGift").c_str());
    business_connection_id_fieldID = jni::get_field_id(env, Class, "businessConnectionId", "Ljava/lang/String;");
    received_gift_id_fieldID = jni::get_field_id(env, Class, "receivedGiftId", "Ljava/lang/String;");
    keep_original_details_fieldID = jni::get_field_id(env, Class, "keepOriginalDetails", "Z");
    star_count_fieldID = jni::get_field_id(env, Class, "starCount", "J");
    return true;
  }();
  (void)is_inited;
}

jclass uploadStickerFile::Class;
jfieldID uploadStickerFile::user_id_fieldID;
jfieldID uploadStickerFile::sticker_format_fieldID;
jfieldID uploadStickerFile::sticker_fieldID;

uploadStickerFile::uploadStickerFile()
  : user_id_()
  , sticker_format_()
  , sticker_()
{}

uploadStickerFile::uploadStickerFile(int53 user_id_, object_ptr<StickerFormat> &&sticker_format_, object_ptr<InputFile> &&sticker_)
  : user_id_(user_id_)
  , sticker_format_(std::move(sticker_format_))
  , sticker_(std::move(sticker_))
{}

const std::int32_t uploadStickerFile::ID;

object_ptr<uploadStickerFile> uploadStickerFile::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<uploadStickerFile> res = make_object<uploadStickerFile>();
  res->user_id_ = env->GetLongField(p, res->user_id_fieldID);
  res->sticker_format_ = jni::fetch_tl_object<StickerFormat>(env, jni::fetch_object(env, p, res->sticker_format_fieldID));
  res->sticker_ = jni::fetch_tl_object<InputFile>(env, jni::fetch_object(env, p, res->sticker_fieldID));
  return res;
}

void uploadStickerFile::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "UploadStickerFile");
    s.store_field("userId", user_id_);
    s.store_object_field("stickerFormat", static_cast<const BaseObject *>(sticker_format_.get()));
    s.store_object_field("sticker", static_cast<const BaseObject *>(sticker_.get()));
    s.store_class_end();
  }
}

uploadStickerFile::ReturnType uploadStickerFile::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return file::fetch(env, p);
}

void uploadStickerFile::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$UploadStickerFile").c_str());
    user_id_fieldID = jni::get_field_id(env, Class, "userId", "J");
    sticker_format_fieldID = jni::get_field_id(env, Class, "stickerFormat", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$StickerFormat;").c_str());
    sticker_fieldID = jni::get_field_id(env, Class, "sticker", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputFile;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass validateOrderInfo::Class;
jfieldID validateOrderInfo::input_invoice_fieldID;
jfieldID validateOrderInfo::order_info_fieldID;
jfieldID validateOrderInfo::allow_save_fieldID;

validateOrderInfo::validateOrderInfo()
  : input_invoice_()
  , order_info_()
  , allow_save_()
{}

validateOrderInfo::validateOrderInfo(object_ptr<InputInvoice> &&input_invoice_, object_ptr<orderInfo> &&order_info_, bool allow_save_)
  : input_invoice_(std::move(input_invoice_))
  , order_info_(std::move(order_info_))
  , allow_save_(allow_save_)
{}

const std::int32_t validateOrderInfo::ID;

object_ptr<validateOrderInfo> validateOrderInfo::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<validateOrderInfo> res = make_object<validateOrderInfo>();
  res->input_invoice_ = jni::fetch_tl_object<InputInvoice>(env, jni::fetch_object(env, p, res->input_invoice_fieldID));
  res->order_info_ = jni::fetch_tl_object<orderInfo>(env, jni::fetch_object(env, p, res->order_info_fieldID));
  res->allow_save_ = (env->GetBooleanField(p, res->allow_save_fieldID) != 0);
  return res;
}

void validateOrderInfo::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ValidateOrderInfo");
    s.store_object_field("inputInvoice", static_cast<const BaseObject *>(input_invoice_.get()));
    s.store_object_field("orderInfo", static_cast<const BaseObject *>(order_info_.get()));
    s.store_field("allowSave", allow_save_);
    s.store_class_end();
  }
}

validateOrderInfo::ReturnType validateOrderInfo::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return validatedOrderInfo::fetch(env, p);
}

void validateOrderInfo::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ValidateOrderInfo").c_str());
    input_invoice_fieldID = jni::get_field_id(env, Class, "inputInvoice", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$InputInvoice;").c_str());
    order_info_fieldID = jni::get_field_id(env, Class, "orderInfo", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$OrderInfo;").c_str());
    allow_save_fieldID = jni::get_field_id(env, Class, "allowSave", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass viewMessages::Class;
jfieldID viewMessages::chat_id_fieldID;
jfieldID viewMessages::message_ids_fieldID;
jfieldID viewMessages::source_fieldID;
jfieldID viewMessages::force_read_fieldID;

viewMessages::viewMessages()
  : chat_id_()
  , message_ids_()
  , source_()
  , force_read_()
{}

viewMessages::viewMessages(int53 chat_id_, array<int53> &&message_ids_, object_ptr<MessageSource> &&source_, bool force_read_)
  : chat_id_(chat_id_)
  , message_ids_(std::move(message_ids_))
  , source_(std::move(source_))
  , force_read_(force_read_)
{}

const std::int32_t viewMessages::ID;

object_ptr<viewMessages> viewMessages::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<viewMessages> res = make_object<viewMessages>();
  res->chat_id_ = env->GetLongField(p, res->chat_id_fieldID);
  res->message_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->message_ids_fieldID));
  res->source_ = jni::fetch_tl_object<MessageSource>(env, jni::fetch_object(env, p, res->source_fieldID));
  res->force_read_ = (env->GetBooleanField(p, res->force_read_fieldID) != 0);
  return res;
}

void viewMessages::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ViewMessages");
    s.store_field("chatId", chat_id_);
    { s.store_vector_begin("messageIds", message_ids_.size()); for (const auto &_value : message_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_object_field("source", static_cast<const BaseObject *>(source_.get()));
    s.store_field("forceRead", force_read_);
    s.store_class_end();
  }
}

viewMessages::ReturnType viewMessages::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void viewMessages::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ViewMessages").c_str());
    chat_id_fieldID = jni::get_field_id(env, Class, "chatId", "J");
    message_ids_fieldID = jni::get_field_id(env, Class, "messageIds", "[J");
    source_fieldID = jni::get_field_id(env, Class, "source", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$MessageSource;").c_str());
    force_read_fieldID = jni::get_field_id(env, Class, "forceRead", "Z");
    return true;
  }();
  (void)is_inited;
}

jclass viewPremiumFeature::Class;
jfieldID viewPremiumFeature::feature_fieldID;

viewPremiumFeature::viewPremiumFeature()
  : feature_()
{}

viewPremiumFeature::viewPremiumFeature(object_ptr<PremiumFeature> &&feature_)
  : feature_(std::move(feature_))
{}

const std::int32_t viewPremiumFeature::ID;

object_ptr<viewPremiumFeature> viewPremiumFeature::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<viewPremiumFeature> res = make_object<viewPremiumFeature>();
  res->feature_ = jni::fetch_tl_object<PremiumFeature>(env, jni::fetch_object(env, p, res->feature_fieldID));
  return res;
}

void viewPremiumFeature::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ViewPremiumFeature");
    s.store_object_field("feature", static_cast<const BaseObject *>(feature_.get()));
    s.store_class_end();
  }
}

viewPremiumFeature::ReturnType viewPremiumFeature::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void viewPremiumFeature::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ViewPremiumFeature").c_str());
    feature_fieldID = jni::get_field_id(env, Class, "feature", (PSLICE() << "L" << get_package_name_ref() << "/TdApi$PremiumFeature;").c_str());
    return true;
  }();
  (void)is_inited;
}

jclass viewSponsoredChat::Class;
jfieldID viewSponsoredChat::sponsored_chat_unique_id_fieldID;

viewSponsoredChat::viewSponsoredChat()
  : sponsored_chat_unique_id_()
{}

viewSponsoredChat::viewSponsoredChat(int53 sponsored_chat_unique_id_)
  : sponsored_chat_unique_id_(sponsored_chat_unique_id_)
{}

const std::int32_t viewSponsoredChat::ID;

object_ptr<viewSponsoredChat> viewSponsoredChat::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<viewSponsoredChat> res = make_object<viewSponsoredChat>();
  res->sponsored_chat_unique_id_ = env->GetLongField(p, res->sponsored_chat_unique_id_fieldID);
  return res;
}

void viewSponsoredChat::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ViewSponsoredChat");
    s.store_field("sponsoredChatUniqueId", sponsored_chat_unique_id_);
    s.store_class_end();
  }
}

viewSponsoredChat::ReturnType viewSponsoredChat::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void viewSponsoredChat::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ViewSponsoredChat").c_str());
    sponsored_chat_unique_id_fieldID = jni::get_field_id(env, Class, "sponsoredChatUniqueId", "J");
    return true;
  }();
  (void)is_inited;
}

jclass viewTrendingStickerSets::Class;
jfieldID viewTrendingStickerSets::sticker_set_ids_fieldID;

viewTrendingStickerSets::viewTrendingStickerSets()
  : sticker_set_ids_()
{}

viewTrendingStickerSets::viewTrendingStickerSets(array<int64> &&sticker_set_ids_)
  : sticker_set_ids_(std::move(sticker_set_ids_))
{}

const std::int32_t viewTrendingStickerSets::ID;

object_ptr<viewTrendingStickerSets> viewTrendingStickerSets::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<viewTrendingStickerSets> res = make_object<viewTrendingStickerSets>();
  res->sticker_set_ids_ = jni::fetch_vector(env, (jlongArray)jni::fetch_object(env, p, res->sticker_set_ids_fieldID));
  return res;
}

void viewTrendingStickerSets::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "ViewTrendingStickerSets");
    { s.store_vector_begin("stickerSetIds", sticker_set_ids_.size()); for (const auto &_value : sticker_set_ids_) { s.store_field("", _value); } s.store_class_end(); }
    s.store_class_end();
  }
}

viewTrendingStickerSets::ReturnType viewTrendingStickerSets::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void viewTrendingStickerSets::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$ViewTrendingStickerSets").c_str());
    sticker_set_ids_fieldID = jni::get_field_id(env, Class, "stickerSetIds", "[J");
    return true;
  }();
  (void)is_inited;
}

jclass writeGeneratedFilePart::Class;
jfieldID writeGeneratedFilePart::generation_id_fieldID;
jfieldID writeGeneratedFilePart::offset_fieldID;
jfieldID writeGeneratedFilePart::data_fieldID;

writeGeneratedFilePart::writeGeneratedFilePart()
  : generation_id_()
  , offset_()
  , data_()
{}

writeGeneratedFilePart::writeGeneratedFilePart(int64 generation_id_, int53 offset_, bytes const &data_)
  : generation_id_(generation_id_)
  , offset_(offset_)
  , data_(std::move(data_))
{}

const std::int32_t writeGeneratedFilePart::ID;

object_ptr<writeGeneratedFilePart> writeGeneratedFilePart::fetch(JNIEnv *env, jobject &p) {
  if (p == nullptr) return nullptr;
  init_jni_vars(env);
  object_ptr<writeGeneratedFilePart> res = make_object<writeGeneratedFilePart>();
  res->generation_id_ = env->GetLongField(p, res->generation_id_fieldID);
  res->offset_ = env->GetLongField(p, res->offset_fieldID);
  res->data_ = jni::from_bytes(env, (jbyteArray)jni::fetch_object(env, p, res->data_fieldID));
  return res;
}

void writeGeneratedFilePart::store(TlStorerToString &s, const char *field_name) const {
  if (!LOG_IS_STRIPPED(ERROR)) {
    s.store_class_begin(field_name, "WriteGeneratedFilePart");
    s.store_field("generationId", generation_id_);
    s.store_field("offset", offset_);
    s.store_bytes_field("data", data_);
    s.store_class_end();
  }
}

writeGeneratedFilePart::ReturnType writeGeneratedFilePart::fetch_result(JNIEnv *env, jobject &p) {
  if (p == nullptr) return ReturnType();
  return ok::fetch(env, p);
}

void writeGeneratedFilePart::init_jni_vars(JNIEnv *env) {
  static bool is_inited = [&] {
    Class = jni::get_jclass(env, (PSLICE() << get_package_name_ref() << "/TdApi$WriteGeneratedFilePart").c_str());
    generation_id_fieldID = jni::get_field_id(env, Class, "generationId", "J");
    offset_fieldID = jni::get_field_id(env, Class, "offset", "J");
    data_fieldID = jni::get_field_id(env, Class, "data", "[B");
    return true;
  }();
  (void)is_inited;
}
}  // namespace td_api
}  // namespace td
